(function(T2,G0){typeof exports=="object"&&typeof module<"u"?module.exports=G0():typeof define=="function"&&define.amd?define(G0):(T2=typeof globalThis<"u"?globalThis:T2||self,T2.mermaid=G0())})(this,function(){"use strict";var VPn=Object.defineProperty;var UPn=(T2,G0,Ar)=>G0 in T2?VPn(T2,G0,{enumerable:!0,configurable:!0,writable:!0,value:Ar}):T2[G0]=Ar;var CL=(T2,G0,Ar)=>(UPn(T2,typeof G0!="symbol"?G0+"":G0,Ar),Ar);function T2(r){for(var i=[],o=1;o<arguments.length;o++)i[o-1]=arguments[o];var l=Array.from(typeof r=="string"?[r]:r);l[l.length-1]=l[l.length-1].replace(/\r?\n([\t ]*)$/,"");var f=l.reduce(function(w,y){var k=y.match(/\n([\t ]+|(?!\s).)/g);return k?w.concat(k.map(function(E){var T,C;return(C=(T=E.match(/[\t ]/g))===null||T===void 0?void 0:T.length)!==null&&C!==void 0?C:0})):w},[]);if(f.length){var b=new RegExp(`
[	 ]{`+Math.min.apply(Math,f)+"}","g");l=l.map(function(w){return w.replace(b,`
`)})}l[0]=l[0].replace(/^\r?\n/,"");var d=l[0];return i.forEach(function(w,y){var k=d.match(/(?:^|\n)( *)$/),E=k?k[1]:"",T=w;typeof w=="string"&&w.includes(`
`)&&(T=String(w).split(`
`).map(function(C,S){return S===0?C:""+E+C}).join(`
`)),d+=T+l[y+1]}),d}//! moment.js
//! version : 2.29.4
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com
var G0;function Ar(){return G0.apply(null,arguments)}function P_t(r){G0=r}function mv(r){return r instanceof Array||Object.prototype.toString.call(r)==="[object Array]"}function M8(r){return r!=null&&Object.prototype.toString.call(r)==="[object Object]"}function Mo(r,i){return Object.prototype.hasOwnProperty.call(r,i)}function vie(r){if(Object.getOwnPropertyNames)return Object.getOwnPropertyNames(r).length===0;var i;for(i in r)if(Mo(r,i))return!1;return!0}function V0(r){return r===void 0}function L3(r){return typeof r=="number"||Object.prototype.toString.call(r)==="[object Number]"}function SL(r){return r instanceof Date||Object.prototype.toString.call(r)==="[object Date]"}function QEe(r,i){var o=[],l,f=r.length;for(l=0;l<f;++l)o.push(i(r[l],l));return o}function n6(r,i){for(var o in i)Mo(i,o)&&(r[o]=i[o]);return Mo(i,"toString")&&(r.toString=i.toString),Mo(i,"valueOf")&&(r.valueOf=i.valueOf),r}function Tm(r,i,o,l){return _9e(r,i,o,l,!0).utc()}function B_t(){return{empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidEra:null,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],era:null,meridiem:null,rfc2822:!1,weekdayMismatch:!1}}function ba(r){return r._pf==null&&(r._pf=B_t()),r._pf}var wie;Array.prototype.some?wie=Array.prototype.some:wie=function(r){var i=Object(this),o=i.length>>>0,l;for(l=0;l<o;l++)if(l in i&&r.call(this,i[l],l,i))return!0;return!1};function mie(r){if(r._isValid==null){var i=ba(r),o=wie.call(i.parsedDateParts,function(f){return f!=null}),l=!isNaN(r._d.getTime())&&i.overflow<0&&!i.empty&&!i.invalidEra&&!i.invalidMonth&&!i.invalidWeekday&&!i.weekdayMismatch&&!i.nullInput&&!i.invalidFormat&&!i.userInvalidated&&(!i.meridiem||i.meridiem&&o);if(r._strict&&(l=l&&i.charsLeftOver===0&&i.unusedTokens.length===0&&i.bigHour===void 0),Object.isFrozen==null||!Object.isFrozen(r))r._isValid=l;else return l}return r._isValid}function sz(r){var i=Tm(NaN);return r!=null?n6(ba(i),r):ba(i).userInvalidated=!0,i}var ZEe=Ar.momentProperties=[],yie=!1;function kie(r,i){var o,l,f,b=ZEe.length;if(V0(i._isAMomentObject)||(r._isAMomentObject=i._isAMomentObject),V0(i._i)||(r._i=i._i),V0(i._f)||(r._f=i._f),V0(i._l)||(r._l=i._l),V0(i._strict)||(r._strict=i._strict),V0(i._tzm)||(r._tzm=i._tzm),V0(i._isUTC)||(r._isUTC=i._isUTC),V0(i._offset)||(r._offset=i._offset),V0(i._pf)||(r._pf=ba(i)),V0(i._locale)||(r._locale=i._locale),b>0)for(o=0;o<b;o++)l=ZEe[o],f=i[l],V0(f)||(r[l]=f);return r}function AL(r){kie(this,r),this._d=new Date(r._d!=null?r._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),yie===!1&&(yie=!0,Ar.updateOffset(this),yie=!1)}function yv(r){return r instanceof AL||r!=null&&r._isAMomentObject!=null}function JEe(r){Ar.suppressDeprecationWarnings===!1&&typeof console<"u"&&console.warn&&console.warn("Deprecation warning: "+r)}function C2(r,i){var o=!0;return n6(function(){if(Ar.deprecationHandler!=null&&Ar.deprecationHandler(null,r),o){var l=[],f,b,d,w=arguments.length;for(b=0;b<w;b++){if(f="",typeof arguments[b]=="object"){f+=`
[`+b+"] ";for(d in arguments[0])Mo(arguments[0],d)&&(f+=d+": "+arguments[0][d]+", ");f=f.slice(0,-2)}else f=arguments[b];l.push(f)}JEe(r+`
Arguments: `+Array.prototype.slice.call(l).join("")+`
`+new Error().stack),o=!1}return i.apply(this,arguments)},i)}var e9e={};function t9e(r,i){Ar.deprecationHandler!=null&&Ar.deprecationHandler(r,i),e9e[r]||(JEe(i),e9e[r]=!0)}Ar.suppressDeprecationWarnings=!1,Ar.deprecationHandler=null;function Cm(r){return typeof Function<"u"&&r instanceof Function||Object.prototype.toString.call(r)==="[object Function]"}function F_t(r){var i,o;for(o in r)Mo(r,o)&&(i=r[o],Cm(i)?this[o]=i:this["_"+o]=i);this._config=r,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)}function xie(r,i){var o=n6({},r),l;for(l in i)Mo(i,l)&&(M8(r[l])&&M8(i[l])?(o[l]={},n6(o[l],r[l]),n6(o[l],i[l])):i[l]!=null?o[l]=i[l]:delete o[l]);for(l in r)Mo(r,l)&&!Mo(i,l)&&M8(r[l])&&(o[l]=n6({},o[l]));return o}function Eie(r){r!=null&&this.set(r)}var _ie;Object.keys?_ie=Object.keys:_ie=function(r){var i,o=[];for(i in r)Mo(r,i)&&o.push(i);return o};var R_t={sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"};function j_t(r,i,o){var l=this._calendar[r]||this._calendar.sameElse;return Cm(l)?l.call(i,o):l}function Sm(r,i,o){var l=""+Math.abs(r),f=i-l.length,b=r>=0;return(b?o?"+":"":"-")+Math.pow(10,Math.max(0,f)).toString().substr(1)+l}var Tie=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,az=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,Cie={},_T={};function Ki(r,i,o,l){var f=l;typeof l=="string"&&(f=function(){return this[l]()}),r&&(_T[r]=f),i&&(_T[i[0]]=function(){return Sm(f.apply(this,arguments),i[1],i[2])}),o&&(_T[o]=function(){return this.localeData().ordinal(f.apply(this,arguments),r)})}function $_t(r){return r.match(/\[[\s\S]/)?r.replace(/^\[|\]$/g,""):r.replace(/\\/g,"")}function H_t(r){var i=r.match(Tie),o,l;for(o=0,l=i.length;o<l;o++)_T[i[o]]?i[o]=_T[i[o]]:i[o]=$_t(i[o]);return function(f){var b="",d;for(d=0;d<l;d++)b+=Cm(i[d])?i[d].call(f,r):i[d];return b}}function oz(r,i){return r.isValid()?(i=n9e(i,r.localeData()),Cie[i]=Cie[i]||H_t(i),Cie[i](r)):r.localeData().invalidDate()}function n9e(r,i){var o=5;function l(f){return i.longDateFormat(f)||f}for(az.lastIndex=0;o>=0&&az.test(r);)r=r.replace(az,l),az.lastIndex=0,o-=1;return r}var z_t={LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"};function G_t(r){var i=this._longDateFormat[r],o=this._longDateFormat[r.toUpperCase()];return i||!o?i:(this._longDateFormat[r]=o.match(Tie).map(function(l){return l==="MMMM"||l==="MM"||l==="DD"||l==="dddd"?l.slice(1):l}).join(""),this._longDateFormat[r])}var V_t="Invalid date";function U_t(){return this._invalidDate}var q_t="%d",Y_t=/\d{1,2}/;function W_t(r){return this._ordinal.replace("%d",r)}var K_t={future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",w:"a week",ww:"%d weeks",M:"a month",MM:"%d months",y:"a year",yy:"%d years"};function X_t(r,i,o,l){var f=this._relativeTime[o];return Cm(f)?f(r,i,o,l):f.replace(/%d/i,r)}function Q_t(r,i){var o=this._relativeTime[r>0?"future":"past"];return Cm(o)?o(i):o.replace(/%s/i,i)}var ML={};function nd(r,i){var o=r.toLowerCase();ML[o]=ML[o+"s"]=ML[i]=r}function S2(r){return typeof r=="string"?ML[r]||ML[r.toLowerCase()]:void 0}function Sie(r){var i={},o,l;for(l in r)Mo(r,l)&&(o=S2(l),o&&(i[o]=r[l]));return i}var r9e={};function rd(r,i){r9e[r]=i}function Z_t(r){var i=[],o;for(o in r)Mo(r,o)&&i.push({unit:o,priority:r9e[o]});return i.sort(function(l,f){return l.priority-f.priority}),i}function cz(r){return r%4===0&&r%100!==0||r%400===0}function A2(r){return r<0?Math.ceil(r)||0:Math.floor(r)}function Fa(r){var i=+r,o=0;return i!==0&&isFinite(i)&&(o=A2(i)),o}function TT(r,i){return function(o){return o!=null?(i9e(this,r,o),Ar.updateOffset(this,i),this):uz(this,r)}}function uz(r,i){return r.isValid()?r._d["get"+(r._isUTC?"UTC":"")+i]():NaN}function i9e(r,i,o){r.isValid()&&!isNaN(o)&&(i==="FullYear"&&cz(r.year())&&r.month()===1&&r.date()===29?(o=Fa(o),r._d["set"+(r._isUTC?"UTC":"")+i](o,r.month(),bz(o,r.month()))):r._d["set"+(r._isUTC?"UTC":"")+i](o))}function J_t(r){return r=S2(r),Cm(this[r])?this[r]():this}function eTt(r,i){if(typeof r=="object"){r=Sie(r);var o=Z_t(r),l,f=o.length;for(l=0;l<f;l++)this[o[l].unit](r[o[l].unit])}else if(r=S2(r),Cm(this[r]))return this[r](i);return this}var s9e=/\d/,op=/\d\d/,a9e=/\d{3}/,Aie=/\d{4}/,lz=/[+-]?\d{6}/,wu=/\d\d?/,o9e=/\d\d\d\d?/,c9e=/\d\d\d\d\d\d?/,hz=/\d{1,3}/,Mie=/\d{1,4}/,fz=/[+-]?\d{1,6}/,CT=/\d+/,dz=/[+-]?\d+/,tTt=/Z|[+-]\d\d:?\d\d/gi,gz=/Z|[+-]\d\d(?::?\d\d)?/gi,nTt=/[+-]?\d+(\.\d{1,3})?/,DL=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,pz;pz={};function ci(r,i,o){pz[r]=Cm(i)?i:function(l,f){return l&&o?o:i}}function rTt(r,i){return Mo(pz,r)?pz[r](i._strict,i._locale):new RegExp(iTt(r))}function iTt(r){return cp(r.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,function(i,o,l,f,b){return o||l||f||b}))}function cp(r){return r.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}var Die={};function Mc(r,i){var o,l=i,f;for(typeof r=="string"&&(r=[r]),L3(i)&&(l=function(b,d){d[i]=Fa(b)}),f=r.length,o=0;o<f;o++)Die[r[o]]=l}function LL(r,i){Mc(r,function(o,l,f,b){f._w=f._w||{},i(o,f._w,f,b)})}function sTt(r,i,o){i!=null&&Mo(Die,r)&&Die[r](i,o._a,o,r)}var id=0,I3=1,Am=2,of=3,kv=4,O3=5,D8=6,aTt=7,oTt=8;function cTt(r,i){return(r%i+i)%i}var ih;Array.prototype.indexOf?ih=Array.prototype.indexOf:ih=function(r){var i;for(i=0;i<this.length;++i)if(this[i]===r)return i;return-1};function bz(r,i){if(isNaN(r)||isNaN(i))return NaN;var o=cTt(i,12);return r+=(i-o)/12,o===1?cz(r)?29:28:31-o%7%2}Ki("M",["MM",2],"Mo",function(){return this.month()+1}),Ki("MMM",0,0,function(r){return this.localeData().monthsShort(this,r)}),Ki("MMMM",0,0,function(r){return this.localeData().months(this,r)}),nd("month","M"),rd("month",8),ci("M",wu),ci("MM",wu,op),ci("MMM",function(r,i){return i.monthsShortRegex(r)}),ci("MMMM",function(r,i){return i.monthsRegex(r)}),Mc(["M","MM"],function(r,i){i[I3]=Fa(r)-1}),Mc(["MMM","MMMM"],function(r,i,o,l){var f=o._locale.monthsParse(r,l,o._strict);f!=null?i[I3]=f:ba(o).invalidMonth=r});var uTt="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),u9e="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),l9e=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,lTt=DL,hTt=DL;function fTt(r,i){return r?mv(this._months)?this._months[r.month()]:this._months[(this._months.isFormat||l9e).test(i)?"format":"standalone"][r.month()]:mv(this._months)?this._months:this._months.standalone}function dTt(r,i){return r?mv(this._monthsShort)?this._monthsShort[r.month()]:this._monthsShort[l9e.test(i)?"format":"standalone"][r.month()]:mv(this._monthsShort)?this._monthsShort:this._monthsShort.standalone}function gTt(r,i,o){var l,f,b,d=r.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],l=0;l<12;++l)b=Tm([2e3,l]),this._shortMonthsParse[l]=this.monthsShort(b,"").toLocaleLowerCase(),this._longMonthsParse[l]=this.months(b,"").toLocaleLowerCase();return o?i==="MMM"?(f=ih.call(this._shortMonthsParse,d),f!==-1?f:null):(f=ih.call(this._longMonthsParse,d),f!==-1?f:null):i==="MMM"?(f=ih.call(this._shortMonthsParse,d),f!==-1?f:(f=ih.call(this._longMonthsParse,d),f!==-1?f:null)):(f=ih.call(this._longMonthsParse,d),f!==-1?f:(f=ih.call(this._shortMonthsParse,d),f!==-1?f:null))}function pTt(r,i,o){var l,f,b;if(this._monthsParseExact)return gTt.call(this,r,i,o);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),l=0;l<12;l++){if(f=Tm([2e3,l]),o&&!this._longMonthsParse[l]&&(this._longMonthsParse[l]=new RegExp("^"+this.months(f,"").replace(".","")+"$","i"),this._shortMonthsParse[l]=new RegExp("^"+this.monthsShort(f,"").replace(".","")+"$","i")),!o&&!this._monthsParse[l]&&(b="^"+this.months(f,"")+"|^"+this.monthsShort(f,""),this._monthsParse[l]=new RegExp(b.replace(".",""),"i")),o&&i==="MMMM"&&this._longMonthsParse[l].test(r))return l;if(o&&i==="MMM"&&this._shortMonthsParse[l].test(r))return l;if(!o&&this._monthsParse[l].test(r))return l}}function h9e(r,i){var o;if(!r.isValid())return r;if(typeof i=="string"){if(/^\d+$/.test(i))i=Fa(i);else if(i=r.localeData().monthsParse(i),!L3(i))return r}return o=Math.min(r.date(),bz(r.year(),i)),r._d["set"+(r._isUTC?"UTC":"")+"Month"](i,o),r}function f9e(r){return r!=null?(h9e(this,r),Ar.updateOffset(this,!0),this):uz(this,"Month")}function bTt(){return bz(this.year(),this.month())}function vTt(r){return this._monthsParseExact?(Mo(this,"_monthsRegex")||d9e.call(this),r?this._monthsShortStrictRegex:this._monthsShortRegex):(Mo(this,"_monthsShortRegex")||(this._monthsShortRegex=lTt),this._monthsShortStrictRegex&&r?this._monthsShortStrictRegex:this._monthsShortRegex)}function wTt(r){return this._monthsParseExact?(Mo(this,"_monthsRegex")||d9e.call(this),r?this._monthsStrictRegex:this._monthsRegex):(Mo(this,"_monthsRegex")||(this._monthsRegex=hTt),this._monthsStrictRegex&&r?this._monthsStrictRegex:this._monthsRegex)}function d9e(){function r(d,w){return w.length-d.length}var i=[],o=[],l=[],f,b;for(f=0;f<12;f++)b=Tm([2e3,f]),i.push(this.monthsShort(b,"")),o.push(this.months(b,"")),l.push(this.months(b,"")),l.push(this.monthsShort(b,""));for(i.sort(r),o.sort(r),l.sort(r),f=0;f<12;f++)i[f]=cp(i[f]),o[f]=cp(o[f]);for(f=0;f<24;f++)l[f]=cp(l[f]);this._monthsRegex=new RegExp("^("+l.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+o.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+i.join("|")+")","i")}Ki("Y",0,0,function(){var r=this.year();return r<=9999?Sm(r,4):"+"+r}),Ki(0,["YY",2],0,function(){return this.year()%100}),Ki(0,["YYYY",4],0,"year"),Ki(0,["YYYYY",5],0,"year"),Ki(0,["YYYYYY",6,!0],0,"year"),nd("year","y"),rd("year",1),ci("Y",dz),ci("YY",wu,op),ci("YYYY",Mie,Aie),ci("YYYYY",fz,lz),ci("YYYYYY",fz,lz),Mc(["YYYYY","YYYYYY"],id),Mc("YYYY",function(r,i){i[id]=r.length===2?Ar.parseTwoDigitYear(r):Fa(r)}),Mc("YY",function(r,i){i[id]=Ar.parseTwoDigitYear(r)}),Mc("Y",function(r,i){i[id]=parseInt(r,10)});function IL(r){return cz(r)?366:365}Ar.parseTwoDigitYear=function(r){return Fa(r)+(Fa(r)>68?1900:2e3)};var g9e=TT("FullYear",!0);function mTt(){return cz(this.year())}function yTt(r,i,o,l,f,b,d){var w;return r<100&&r>=0?(w=new Date(r+400,i,o,l,f,b,d),isFinite(w.getFullYear())&&w.setFullYear(r)):w=new Date(r,i,o,l,f,b,d),w}function OL(r){var i,o;return r<100&&r>=0?(o=Array.prototype.slice.call(arguments),o[0]=r+400,i=new Date(Date.UTC.apply(null,o)),isFinite(i.getUTCFullYear())&&i.setUTCFullYear(r)):i=new Date(Date.UTC.apply(null,arguments)),i}function vz(r,i,o){var l=7+i-o,f=(7+OL(r,0,l).getUTCDay()-i)%7;return-f+l-1}function p9e(r,i,o,l,f){var b=(7+o-l)%7,d=vz(r,l,f),w=1+7*(i-1)+b+d,y,k;return w<=0?(y=r-1,k=IL(y)+w):w>IL(r)?(y=r+1,k=w-IL(r)):(y=r,k=w),{year:y,dayOfYear:k}}function NL(r,i,o){var l=vz(r.year(),i,o),f=Math.floor((r.dayOfYear()-l-1)/7)+1,b,d;return f<1?(d=r.year()-1,b=f+N3(d,i,o)):f>N3(r.year(),i,o)?(b=f-N3(r.year(),i,o),d=r.year()+1):(d=r.year(),b=f),{week:b,year:d}}function N3(r,i,o){var l=vz(r,i,o),f=vz(r+1,i,o);return(IL(r)-l+f)/7}Ki("w",["ww",2],"wo","week"),Ki("W",["WW",2],"Wo","isoWeek"),nd("week","w"),nd("isoWeek","W"),rd("week",5),rd("isoWeek",5),ci("w",wu),ci("ww",wu,op),ci("W",wu),ci("WW",wu,op),LL(["w","ww","W","WW"],function(r,i,o,l){i[l.substr(0,1)]=Fa(r)});function kTt(r){return NL(r,this._week.dow,this._week.doy).week}var xTt={dow:0,doy:6};function ETt(){return this._week.dow}function _Tt(){return this._week.doy}function TTt(r){var i=this.localeData().week(this);return r==null?i:this.add((r-i)*7,"d")}function CTt(r){var i=NL(this,1,4).week;return r==null?i:this.add((r-i)*7,"d")}Ki("d",0,"do","day"),Ki("dd",0,0,function(r){return this.localeData().weekdaysMin(this,r)}),Ki("ddd",0,0,function(r){return this.localeData().weekdaysShort(this,r)}),Ki("dddd",0,0,function(r){return this.localeData().weekdays(this,r)}),Ki("e",0,0,"weekday"),Ki("E",0,0,"isoWeekday"),nd("day","d"),nd("weekday","e"),nd("isoWeekday","E"),rd("day",11),rd("weekday",11),rd("isoWeekday",11),ci("d",wu),ci("e",wu),ci("E",wu),ci("dd",function(r,i){return i.weekdaysMinRegex(r)}),ci("ddd",function(r,i){return i.weekdaysShortRegex(r)}),ci("dddd",function(r,i){return i.weekdaysRegex(r)}),LL(["dd","ddd","dddd"],function(r,i,o,l){var f=o._locale.weekdaysParse(r,l,o._strict);f!=null?i.d=f:ba(o).invalidWeekday=r}),LL(["d","e","E"],function(r,i,o,l){i[l]=Fa(r)});function STt(r,i){return typeof r!="string"?r:isNaN(r)?(r=i.weekdaysParse(r),typeof r=="number"?r:null):parseInt(r,10)}function ATt(r,i){return typeof r=="string"?i.weekdaysParse(r)%7||7:isNaN(r)?null:r}function Lie(r,i){return r.slice(i,7).concat(r.slice(0,i))}var MTt="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),b9e="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),DTt="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),LTt=DL,ITt=DL,OTt=DL;function NTt(r,i){var o=mv(this._weekdays)?this._weekdays:this._weekdays[r&&r!==!0&&this._weekdays.isFormat.test(i)?"format":"standalone"];return r===!0?Lie(o,this._week.dow):r?o[r.day()]:o}function PTt(r){return r===!0?Lie(this._weekdaysShort,this._week.dow):r?this._weekdaysShort[r.day()]:this._weekdaysShort}function BTt(r){return r===!0?Lie(this._weekdaysMin,this._week.dow):r?this._weekdaysMin[r.day()]:this._weekdaysMin}function FTt(r,i,o){var l,f,b,d=r.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],l=0;l<7;++l)b=Tm([2e3,1]).day(l),this._minWeekdaysParse[l]=this.weekdaysMin(b,"").toLocaleLowerCase(),this._shortWeekdaysParse[l]=this.weekdaysShort(b,"").toLocaleLowerCase(),this._weekdaysParse[l]=this.weekdays(b,"").toLocaleLowerCase();return o?i==="dddd"?(f=ih.call(this._weekdaysParse,d),f!==-1?f:null):i==="ddd"?(f=ih.call(this._shortWeekdaysParse,d),f!==-1?f:null):(f=ih.call(this._minWeekdaysParse,d),f!==-1?f:null):i==="dddd"?(f=ih.call(this._weekdaysParse,d),f!==-1||(f=ih.call(this._shortWeekdaysParse,d),f!==-1)?f:(f=ih.call(this._minWeekdaysParse,d),f!==-1?f:null)):i==="ddd"?(f=ih.call(this._shortWeekdaysParse,d),f!==-1||(f=ih.call(this._weekdaysParse,d),f!==-1)?f:(f=ih.call(this._minWeekdaysParse,d),f!==-1?f:null)):(f=ih.call(this._minWeekdaysParse,d),f!==-1||(f=ih.call(this._weekdaysParse,d),f!==-1)?f:(f=ih.call(this._shortWeekdaysParse,d),f!==-1?f:null))}function RTt(r,i,o){var l,f,b;if(this._weekdaysParseExact)return FTt.call(this,r,i,o);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),l=0;l<7;l++){if(f=Tm([2e3,1]).day(l),o&&!this._fullWeekdaysParse[l]&&(this._fullWeekdaysParse[l]=new RegExp("^"+this.weekdays(f,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[l]=new RegExp("^"+this.weekdaysShort(f,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[l]=new RegExp("^"+this.weekdaysMin(f,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[l]||(b="^"+this.weekdays(f,"")+"|^"+this.weekdaysShort(f,"")+"|^"+this.weekdaysMin(f,""),this._weekdaysParse[l]=new RegExp(b.replace(".",""),"i")),o&&i==="dddd"&&this._fullWeekdaysParse[l].test(r))return l;if(o&&i==="ddd"&&this._shortWeekdaysParse[l].test(r))return l;if(o&&i==="dd"&&this._minWeekdaysParse[l].test(r))return l;if(!o&&this._weekdaysParse[l].test(r))return l}}function jTt(r){if(!this.isValid())return r!=null?this:NaN;var i=this._isUTC?this._d.getUTCDay():this._d.getDay();return r!=null?(r=STt(r,this.localeData()),this.add(r-i,"d")):i}function $Tt(r){if(!this.isValid())return r!=null?this:NaN;var i=(this.day()+7-this.localeData()._week.dow)%7;return r==null?i:this.add(r-i,"d")}function HTt(r){if(!this.isValid())return r!=null?this:NaN;if(r!=null){var i=ATt(r,this.localeData());return this.day(this.day()%7?i:i-7)}else return this.day()||7}function zTt(r){return this._weekdaysParseExact?(Mo(this,"_weekdaysRegex")||Iie.call(this),r?this._weekdaysStrictRegex:this._weekdaysRegex):(Mo(this,"_weekdaysRegex")||(this._weekdaysRegex=LTt),this._weekdaysStrictRegex&&r?this._weekdaysStrictRegex:this._weekdaysRegex)}function GTt(r){return this._weekdaysParseExact?(Mo(this,"_weekdaysRegex")||Iie.call(this),r?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(Mo(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=ITt),this._weekdaysShortStrictRegex&&r?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)}function VTt(r){return this._weekdaysParseExact?(Mo(this,"_weekdaysRegex")||Iie.call(this),r?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(Mo(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=OTt),this._weekdaysMinStrictRegex&&r?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)}function Iie(){function r(E,T){return T.length-E.length}var i=[],o=[],l=[],f=[],b,d,w,y,k;for(b=0;b<7;b++)d=Tm([2e3,1]).day(b),w=cp(this.weekdaysMin(d,"")),y=cp(this.weekdaysShort(d,"")),k=cp(this.weekdays(d,"")),i.push(w),o.push(y),l.push(k),f.push(w),f.push(y),f.push(k);i.sort(r),o.sort(r),l.sort(r),f.sort(r),this._weekdaysRegex=new RegExp("^("+f.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+o.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+i.join("|")+")","i")}function Oie(){return this.hours()%12||12}function UTt(){return this.hours()||24}Ki("H",["HH",2],0,"hour"),Ki("h",["hh",2],0,Oie),Ki("k",["kk",2],0,UTt),Ki("hmm",0,0,function(){return""+Oie.apply(this)+Sm(this.minutes(),2)}),Ki("hmmss",0,0,function(){return""+Oie.apply(this)+Sm(this.minutes(),2)+Sm(this.seconds(),2)}),Ki("Hmm",0,0,function(){return""+this.hours()+Sm(this.minutes(),2)}),Ki("Hmmss",0,0,function(){return""+this.hours()+Sm(this.minutes(),2)+Sm(this.seconds(),2)});function v9e(r,i){Ki(r,0,0,function(){return this.localeData().meridiem(this.hours(),this.minutes(),i)})}v9e("a",!0),v9e("A",!1),nd("hour","h"),rd("hour",13);function w9e(r,i){return i._meridiemParse}ci("a",w9e),ci("A",w9e),ci("H",wu),ci("h",wu),ci("k",wu),ci("HH",wu,op),ci("hh",wu,op),ci("kk",wu,op),ci("hmm",o9e),ci("hmmss",c9e),ci("Hmm",o9e),ci("Hmmss",c9e),Mc(["H","HH"],of),Mc(["k","kk"],function(r,i,o){var l=Fa(r);i[of]=l===24?0:l}),Mc(["a","A"],function(r,i,o){o._isPm=o._locale.isPM(r),o._meridiem=r}),Mc(["h","hh"],function(r,i,o){i[of]=Fa(r),ba(o).bigHour=!0}),Mc("hmm",function(r,i,o){var l=r.length-2;i[of]=Fa(r.substr(0,l)),i[kv]=Fa(r.substr(l)),ba(o).bigHour=!0}),Mc("hmmss",function(r,i,o){var l=r.length-4,f=r.length-2;i[of]=Fa(r.substr(0,l)),i[kv]=Fa(r.substr(l,2)),i[O3]=Fa(r.substr(f)),ba(o).bigHour=!0}),Mc("Hmm",function(r,i,o){var l=r.length-2;i[of]=Fa(r.substr(0,l)),i[kv]=Fa(r.substr(l))}),Mc("Hmmss",function(r,i,o){var l=r.length-4,f=r.length-2;i[of]=Fa(r.substr(0,l)),i[kv]=Fa(r.substr(l,2)),i[O3]=Fa(r.substr(f))});function qTt(r){return(r+"").toLowerCase().charAt(0)==="p"}var YTt=/[ap]\.?m?\.?/i,WTt=TT("Hours",!0);function KTt(r,i,o){return r>11?o?"pm":"PM":o?"am":"AM"}var m9e={calendar:R_t,longDateFormat:z_t,invalidDate:V_t,ordinal:q_t,dayOfMonthOrdinalParse:Y_t,relativeTime:K_t,months:uTt,monthsShort:u9e,week:xTt,weekdays:MTt,weekdaysMin:DTt,weekdaysShort:b9e,meridiemParse:YTt},Nu={},PL={},BL;function XTt(r,i){var o,l=Math.min(r.length,i.length);for(o=0;o<l;o+=1)if(r[o]!==i[o])return o;return l}function y9e(r){return r&&r.toLowerCase().replace("_","-")}function QTt(r){for(var i=0,o,l,f,b;i<r.length;){for(b=y9e(r[i]).split("-"),o=b.length,l=y9e(r[i+1]),l=l?l.split("-"):null;o>0;){if(f=wz(b.slice(0,o).join("-")),f)return f;if(l&&l.length>=o&&XTt(b,l)>=o-1)break;o--}i++}return BL}function ZTt(r){return r.match("^[^/\\\\]*$")!=null}function wz(r){var i=null,o;if(Nu[r]===void 0&&typeof module<"u"&&module&&module.exports&&ZTt(r))try{i=BL._abbr,o=require,o("./locale/"+r),r6(i)}catch{Nu[r]=null}return Nu[r]}function r6(r,i){var o;return r&&(V0(i)?o=P3(r):o=Nie(r,i),o?BL=o:typeof console<"u"&&console.warn&&console.warn("Locale "+r+" not found. Did you forget to load it?")),BL._abbr}function Nie(r,i){if(i!==null){var o,l=m9e;if(i.abbr=r,Nu[r]!=null)t9e("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),l=Nu[r]._config;else if(i.parentLocale!=null)if(Nu[i.parentLocale]!=null)l=Nu[i.parentLocale]._config;else if(o=wz(i.parentLocale),o!=null)l=o._config;else return PL[i.parentLocale]||(PL[i.parentLocale]=[]),PL[i.parentLocale].push({name:r,config:i}),null;return Nu[r]=new Eie(xie(l,i)),PL[r]&&PL[r].forEach(function(f){Nie(f.name,f.config)}),r6(r),Nu[r]}else return delete Nu[r],null}function JTt(r,i){if(i!=null){var o,l,f=m9e;Nu[r]!=null&&Nu[r].parentLocale!=null?Nu[r].set(xie(Nu[r]._config,i)):(l=wz(r),l!=null&&(f=l._config),i=xie(f,i),l==null&&(i.abbr=r),o=new Eie(i),o.parentLocale=Nu[r],Nu[r]=o),r6(r)}else Nu[r]!=null&&(Nu[r].parentLocale!=null?(Nu[r]=Nu[r].parentLocale,r===r6()&&r6(r)):Nu[r]!=null&&delete Nu[r]);return Nu[r]}function P3(r){var i;if(r&&r._locale&&r._locale._abbr&&(r=r._locale._abbr),!r)return BL;if(!mv(r)){if(i=wz(r),i)return i;r=[r]}return QTt(r)}function eCt(){return _ie(Nu)}function Pie(r){var i,o=r._a;return o&&ba(r).overflow===-2&&(i=o[I3]<0||o[I3]>11?I3:o[Am]<1||o[Am]>bz(o[id],o[I3])?Am:o[of]<0||o[of]>24||o[of]===24&&(o[kv]!==0||o[O3]!==0||o[D8]!==0)?of:o[kv]<0||o[kv]>59?kv:o[O3]<0||o[O3]>59?O3:o[D8]<0||o[D8]>999?D8:-1,ba(r)._overflowDayOfYear&&(i<id||i>Am)&&(i=Am),ba(r)._overflowWeeks&&i===-1&&(i=aTt),ba(r)._overflowWeekday&&i===-1&&(i=oTt),ba(r).overflow=i),r}var tCt=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,nCt=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,rCt=/Z|[+-]\d\d(?::?\d\d)?/,mz=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/],["YYYYMM",/\d{6}/,!1],["YYYY",/\d{4}/,!1]],Bie=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],iCt=/^\/?Date\((-?\d+)/i,sCt=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,aCt={UT:0,GMT:0,EDT:-4*60,EST:-5*60,CDT:-5*60,CST:-6*60,MDT:-6*60,MST:-7*60,PDT:-7*60,PST:-8*60};function k9e(r){var i,o,l=r._i,f=tCt.exec(l)||nCt.exec(l),b,d,w,y,k=mz.length,E=Bie.length;if(f){for(ba(r).iso=!0,i=0,o=k;i<o;i++)if(mz[i][1].exec(f[1])){d=mz[i][0],b=mz[i][2]!==!1;break}if(d==null){r._isValid=!1;return}if(f[3]){for(i=0,o=E;i<o;i++)if(Bie[i][1].exec(f[3])){w=(f[2]||" ")+Bie[i][0];break}if(w==null){r._isValid=!1;return}}if(!b&&w!=null){r._isValid=!1;return}if(f[4])if(rCt.exec(f[4]))y="Z";else{r._isValid=!1;return}r._f=d+(w||"")+(y||""),Rie(r)}else r._isValid=!1}function oCt(r,i,o,l,f,b){var d=[cCt(r),u9e.indexOf(i),parseInt(o,10),parseInt(l,10),parseInt(f,10)];return b&&d.push(parseInt(b,10)),d}function cCt(r){var i=parseInt(r,10);return i<=49?2e3+i:i<=999?1900+i:i}function uCt(r){return r.replace(/\([^()]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,"")}function lCt(r,i,o){if(r){var l=b9e.indexOf(r),f=new Date(i[0],i[1],i[2]).getDay();if(l!==f)return ba(o).weekdayMismatch=!0,o._isValid=!1,!1}return!0}function hCt(r,i,o){if(r)return aCt[r];if(i)return 0;var l=parseInt(o,10),f=l%100,b=(l-f)/100;return b*60+f}function x9e(r){var i=sCt.exec(uCt(r._i)),o;if(i){if(o=oCt(i[4],i[3],i[2],i[5],i[6],i[7]),!lCt(i[1],o,r))return;r._a=o,r._tzm=hCt(i[8],i[9],i[10]),r._d=OL.apply(null,r._a),r._d.setUTCMinutes(r._d.getUTCMinutes()-r._tzm),ba(r).rfc2822=!0}else r._isValid=!1}function fCt(r){var i=iCt.exec(r._i);if(i!==null){r._d=new Date(+i[1]);return}if(k9e(r),r._isValid===!1)delete r._isValid;else return;if(x9e(r),r._isValid===!1)delete r._isValid;else return;r._strict?r._isValid=!1:Ar.createFromInputFallback(r)}Ar.createFromInputFallback=C2("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",function(r){r._d=new Date(r._i+(r._useUTC?" UTC":""))});function ST(r,i,o){return r!=null?r:i!=null?i:o}function dCt(r){var i=new Date(Ar.now());return r._useUTC?[i.getUTCFullYear(),i.getUTCMonth(),i.getUTCDate()]:[i.getFullYear(),i.getMonth(),i.getDate()]}function Fie(r){var i,o,l=[],f,b,d;if(!r._d){for(f=dCt(r),r._w&&r._a[Am]==null&&r._a[I3]==null&&gCt(r),r._dayOfYear!=null&&(d=ST(r._a[id],f[id]),(r._dayOfYear>IL(d)||r._dayOfYear===0)&&(ba(r)._overflowDayOfYear=!0),o=OL(d,0,r._dayOfYear),r._a[I3]=o.getUTCMonth(),r._a[Am]=o.getUTCDate()),i=0;i<3&&r._a[i]==null;++i)r._a[i]=l[i]=f[i];for(;i<7;i++)r._a[i]=l[i]=r._a[i]==null?i===2?1:0:r._a[i];r._a[of]===24&&r._a[kv]===0&&r._a[O3]===0&&r._a[D8]===0&&(r._nextDay=!0,r._a[of]=0),r._d=(r._useUTC?OL:yTt).apply(null,l),b=r._useUTC?r._d.getUTCDay():r._d.getDay(),r._tzm!=null&&r._d.setUTCMinutes(r._d.getUTCMinutes()-r._tzm),r._nextDay&&(r._a[of]=24),r._w&&typeof r._w.d<"u"&&r._w.d!==b&&(ba(r).weekdayMismatch=!0)}}function gCt(r){var i,o,l,f,b,d,w,y,k;i=r._w,i.GG!=null||i.W!=null||i.E!=null?(b=1,d=4,o=ST(i.GG,r._a[id],NL(mu(),1,4).year),l=ST(i.W,1),f=ST(i.E,1),(f<1||f>7)&&(y=!0)):(b=r._locale._week.dow,d=r._locale._week.doy,k=NL(mu(),b,d),o=ST(i.gg,r._a[id],k.year),l=ST(i.w,k.week),i.d!=null?(f=i.d,(f<0||f>6)&&(y=!0)):i.e!=null?(f=i.e+b,(i.e<0||i.e>6)&&(y=!0)):f=b),l<1||l>N3(o,b,d)?ba(r)._overflowWeeks=!0:y!=null?ba(r)._overflowWeekday=!0:(w=p9e(o,l,f,b,d),r._a[id]=w.year,r._dayOfYear=w.dayOfYear)}Ar.ISO_8601=function(){},Ar.RFC_2822=function(){};function Rie(r){if(r._f===Ar.ISO_8601){k9e(r);return}if(r._f===Ar.RFC_2822){x9e(r);return}r._a=[],ba(r).empty=!0;var i=""+r._i,o,l,f,b,d,w=i.length,y=0,k,E;for(f=n9e(r._f,r._locale).match(Tie)||[],E=f.length,o=0;o<E;o++)b=f[o],l=(i.match(rTt(b,r))||[])[0],l&&(d=i.substr(0,i.indexOf(l)),d.length>0&&ba(r).unusedInput.push(d),i=i.slice(i.indexOf(l)+l.length),y+=l.length),_T[b]?(l?ba(r).empty=!1:ba(r).unusedTokens.push(b),sTt(b,l,r)):r._strict&&!l&&ba(r).unusedTokens.push(b);ba(r).charsLeftOver=w-y,i.length>0&&ba(r).unusedInput.push(i),r._a[of]<=12&&ba(r).bigHour===!0&&r._a[of]>0&&(ba(r).bigHour=void 0),ba(r).parsedDateParts=r._a.slice(0),ba(r).meridiem=r._meridiem,r._a[of]=pCt(r._locale,r._a[of],r._meridiem),k=ba(r).era,k!==null&&(r._a[id]=r._locale.erasConvertYear(k,r._a[id])),Fie(r),Pie(r)}function pCt(r,i,o){var l;return o==null?i:r.meridiemHour!=null?r.meridiemHour(i,o):(r.isPM!=null&&(l=r.isPM(o),l&&i<12&&(i+=12),!l&&i===12&&(i=0)),i)}function bCt(r){var i,o,l,f,b,d,w=!1,y=r._f.length;if(y===0){ba(r).invalidFormat=!0,r._d=new Date(NaN);return}for(f=0;f<y;f++)b=0,d=!1,i=kie({},r),r._useUTC!=null&&(i._useUTC=r._useUTC),i._f=r._f[f],Rie(i),mie(i)&&(d=!0),b+=ba(i).charsLeftOver,b+=ba(i).unusedTokens.length*10,ba(i).score=b,w?b<l&&(l=b,o=i):(l==null||b<l||d)&&(l=b,o=i,d&&(w=!0));n6(r,o||i)}function vCt(r){if(!r._d){var i=Sie(r._i),o=i.day===void 0?i.date:i.day;r._a=QEe([i.year,i.month,o,i.hour,i.minute,i.second,i.millisecond],function(l){return l&&parseInt(l,10)}),Fie(r)}}function wCt(r){var i=new AL(Pie(E9e(r)));return i._nextDay&&(i.add(1,"d"),i._nextDay=void 0),i}function E9e(r){var i=r._i,o=r._f;return r._locale=r._locale||P3(r._l),i===null||o===void 0&&i===""?sz({nullInput:!0}):(typeof i=="string"&&(r._i=i=r._locale.preparse(i)),yv(i)?new AL(Pie(i)):(SL(i)?r._d=i:mv(o)?bCt(r):o?Rie(r):mCt(r),mie(r)||(r._d=null),r))}function mCt(r){var i=r._i;V0(i)?r._d=new Date(Ar.now()):SL(i)?r._d=new Date(i.valueOf()):typeof i=="string"?fCt(r):mv(i)?(r._a=QEe(i.slice(0),function(o){return parseInt(o,10)}),Fie(r)):M8(i)?vCt(r):L3(i)?r._d=new Date(i):Ar.createFromInputFallback(r)}function _9e(r,i,o,l,f){var b={};return(i===!0||i===!1)&&(l=i,i=void 0),(o===!0||o===!1)&&(l=o,o=void 0),(M8(r)&&vie(r)||mv(r)&&r.length===0)&&(r=void 0),b._isAMomentObject=!0,b._useUTC=b._isUTC=f,b._l=o,b._i=r,b._f=i,b._strict=l,wCt(b)}function mu(r,i,o,l){return _9e(r,i,o,l,!1)}var yCt=C2("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var r=mu.apply(null,arguments);return this.isValid()&&r.isValid()?r<this?this:r:sz()}),kCt=C2("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var r=mu.apply(null,arguments);return this.isValid()&&r.isValid()?r>this?this:r:sz()});function T9e(r,i){var o,l;if(i.length===1&&mv(i[0])&&(i=i[0]),!i.length)return mu();for(o=i[0],l=1;l<i.length;++l)(!i[l].isValid()||i[l][r](o))&&(o=i[l]);return o}function xCt(){var r=[].slice.call(arguments,0);return T9e("isBefore",r)}function ECt(){var r=[].slice.call(arguments,0);return T9e("isAfter",r)}var _Ct=function(){return Date.now?Date.now():+new Date},FL=["year","quarter","month","week","day","hour","minute","second","millisecond"];function TCt(r){var i,o=!1,l,f=FL.length;for(i in r)if(Mo(r,i)&&!(ih.call(FL,i)!==-1&&(r[i]==null||!isNaN(r[i]))))return!1;for(l=0;l<f;++l)if(r[FL[l]]){if(o)return!1;parseFloat(r[FL[l]])!==Fa(r[FL[l]])&&(o=!0)}return!0}function CCt(){return this._isValid}function SCt(){return xv(NaN)}function yz(r){var i=Sie(r),o=i.year||0,l=i.quarter||0,f=i.month||0,b=i.week||i.isoWeek||0,d=i.day||0,w=i.hour||0,y=i.minute||0,k=i.second||0,E=i.millisecond||0;this._isValid=TCt(i),this._milliseconds=+E+k*1e3+y*6e4+w*1e3*60*60,this._days=+d+b*7,this._months=+f+l*3+o*12,this._data={},this._locale=P3(),this._bubble()}function kz(r){return r instanceof yz}function jie(r){return r<0?Math.round(-1*r)*-1:Math.round(r)}function ACt(r,i,o){var l=Math.min(r.length,i.length),f=Math.abs(r.length-i.length),b=0,d;for(d=0;d<l;d++)(o&&r[d]!==i[d]||!o&&Fa(r[d])!==Fa(i[d]))&&b++;return b+f}function C9e(r,i){Ki(r,0,0,function(){var o=this.utcOffset(),l="+";return o<0&&(o=-o,l="-"),l+Sm(~~(o/60),2)+i+Sm(~~o%60,2)})}C9e("Z",":"),C9e("ZZ",""),ci("Z",gz),ci("ZZ",gz),Mc(["Z","ZZ"],function(r,i,o){o._useUTC=!0,o._tzm=$ie(gz,r)});var MCt=/([\+\-]|\d\d)/gi;function $ie(r,i){var o=(i||"").match(r),l,f,b;return o===null?null:(l=o[o.length-1]||[],f=(l+"").match(MCt)||["-",0,0],b=+(f[1]*60)+Fa(f[2]),b===0?0:f[0]==="+"?b:-b)}function Hie(r,i){var o,l;return i._isUTC?(o=i.clone(),l=(yv(r)||SL(r)?r.valueOf():mu(r).valueOf())-o.valueOf(),o._d.setTime(o._d.valueOf()+l),Ar.updateOffset(o,!1),o):mu(r).local()}function zie(r){return-Math.round(r._d.getTimezoneOffset())}Ar.updateOffset=function(){};function DCt(r,i,o){var l=this._offset||0,f;if(!this.isValid())return r!=null?this:NaN;if(r!=null){if(typeof r=="string"){if(r=$ie(gz,r),r===null)return this}else Math.abs(r)<16&&!o&&(r=r*60);return!this._isUTC&&i&&(f=zie(this)),this._offset=r,this._isUTC=!0,f!=null&&this.add(f,"m"),l!==r&&(!i||this._changeInProgress?D9e(this,xv(r-l,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,Ar.updateOffset(this,!0),this._changeInProgress=null)),this}else return this._isUTC?l:zie(this)}function LCt(r,i){return r!=null?(typeof r!="string"&&(r=-r),this.utcOffset(r,i),this):-this.utcOffset()}function ICt(r){return this.utcOffset(0,r)}function OCt(r){return this._isUTC&&(this.utcOffset(0,r),this._isUTC=!1,r&&this.subtract(zie(this),"m")),this}function NCt(){if(this._tzm!=null)this.utcOffset(this._tzm,!1,!0);else if(typeof this._i=="string"){var r=$ie(tTt,this._i);r!=null?this.utcOffset(r):this.utcOffset(0,!0)}return this}function PCt(r){return this.isValid()?(r=r?mu(r).utcOffset():0,(this.utcOffset()-r)%60===0):!1}function BCt(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()}function FCt(){if(!V0(this._isDSTShifted))return this._isDSTShifted;var r={},i;return kie(r,this),r=E9e(r),r._a?(i=r._isUTC?Tm(r._a):mu(r._a),this._isDSTShifted=this.isValid()&&ACt(r._a,i.toArray())>0):this._isDSTShifted=!1,this._isDSTShifted}function RCt(){return this.isValid()?!this._isUTC:!1}function jCt(){return this.isValid()?this._isUTC:!1}function S9e(){return this.isValid()?this._isUTC&&this._offset===0:!1}var $Ct=/^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,HCt=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function xv(r,i){var o=r,l=null,f,b,d;return kz(r)?o={ms:r._milliseconds,d:r._days,M:r._months}:L3(r)||!isNaN(+r)?(o={},i?o[i]=+r:o.milliseconds=+r):(l=$Ct.exec(r))?(f=l[1]==="-"?-1:1,o={y:0,d:Fa(l[Am])*f,h:Fa(l[of])*f,m:Fa(l[kv])*f,s:Fa(l[O3])*f,ms:Fa(jie(l[D8]*1e3))*f}):(l=HCt.exec(r))?(f=l[1]==="-"?-1:1,o={y:L8(l[2],f),M:L8(l[3],f),w:L8(l[4],f),d:L8(l[5],f),h:L8(l[6],f),m:L8(l[7],f),s:L8(l[8],f)}):o==null?o={}:typeof o=="object"&&("from"in o||"to"in o)&&(d=zCt(mu(o.from),mu(o.to)),o={},o.ms=d.milliseconds,o.M=d.months),b=new yz(o),kz(r)&&Mo(r,"_locale")&&(b._locale=r._locale),kz(r)&&Mo(r,"_isValid")&&(b._isValid=r._isValid),b}xv.fn=yz.prototype,xv.invalid=SCt;function L8(r,i){var o=r&&parseFloat(r.replace(",","."));return(isNaN(o)?0:o)*i}function A9e(r,i){var o={};return o.months=i.month()-r.month()+(i.year()-r.year())*12,r.clone().add(o.months,"M").isAfter(i)&&--o.months,o.milliseconds=+i-+r.clone().add(o.months,"M"),o}function zCt(r,i){var o;return r.isValid()&&i.isValid()?(i=Hie(i,r),r.isBefore(i)?o=A9e(r,i):(o=A9e(i,r),o.milliseconds=-o.milliseconds,o.months=-o.months),o):{milliseconds:0,months:0}}function M9e(r,i){return function(o,l){var f,b;return l!==null&&!isNaN(+l)&&(t9e(i,"moment()."+i+"(period, number) is deprecated. Please use moment()."+i+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),b=o,o=l,l=b),f=xv(o,l),D9e(this,f,r),this}}function D9e(r,i,o,l){var f=i._milliseconds,b=jie(i._days),d=jie(i._months);!r.isValid()||(l=l==null?!0:l,d&&h9e(r,uz(r,"Month")+d*o),b&&i9e(r,"Date",uz(r,"Date")+b*o),f&&r._d.setTime(r._d.valueOf()+f*o),l&&Ar.updateOffset(r,b||d))}var GCt=M9e(1,"add"),VCt=M9e(-1,"subtract");function L9e(r){return typeof r=="string"||r instanceof String}function UCt(r){return yv(r)||SL(r)||L9e(r)||L3(r)||YCt(r)||qCt(r)||r===null||r===void 0}function qCt(r){var i=M8(r)&&!vie(r),o=!1,l=["years","year","y","months","month","M","days","day","d","dates","date","D","hours","hour","h","minutes","minute","m","seconds","second","s","milliseconds","millisecond","ms"],f,b,d=l.length;for(f=0;f<d;f+=1)b=l[f],o=o||Mo(r,b);return i&&o}function YCt(r){var i=mv(r),o=!1;return i&&(o=r.filter(function(l){return!L3(l)&&L9e(r)}).length===0),i&&o}function WCt(r){var i=M8(r)&&!vie(r),o=!1,l=["sameDay","nextDay","lastDay","nextWeek","lastWeek","sameElse"],f,b;for(f=0;f<l.length;f+=1)b=l[f],o=o||Mo(r,b);return i&&o}function KCt(r,i){var o=r.diff(i,"days",!0);return o<-6?"sameElse":o<-1?"lastWeek":o<0?"lastDay":o<1?"sameDay":o<2?"nextDay":o<7?"nextWeek":"sameElse"}function XCt(r,i){arguments.length===1&&(arguments[0]?UCt(arguments[0])?(r=arguments[0],i=void 0):WCt(arguments[0])&&(i=arguments[0],r=void 0):(r=void 0,i=void 0));var o=r||mu(),l=Hie(o,this).startOf("day"),f=Ar.calendarFormat(this,l)||"sameElse",b=i&&(Cm(i[f])?i[f].call(this,o):i[f]);return this.format(b||this.localeData().calendar(f,this,mu(o)))}function QCt(){return new AL(this)}function ZCt(r,i){var o=yv(r)?r:mu(r);return this.isValid()&&o.isValid()?(i=S2(i)||"millisecond",i==="millisecond"?this.valueOf()>o.valueOf():o.valueOf()<this.clone().startOf(i).valueOf()):!1}function JCt(r,i){var o=yv(r)?r:mu(r);return this.isValid()&&o.isValid()?(i=S2(i)||"millisecond",i==="millisecond"?this.valueOf()<o.valueOf():this.clone().endOf(i).valueOf()<o.valueOf()):!1}function eSt(r,i,o,l){var f=yv(r)?r:mu(r),b=yv(i)?i:mu(i);return this.isValid()&&f.isValid()&&b.isValid()?(l=l||"()",(l[0]==="("?this.isAfter(f,o):!this.isBefore(f,o))&&(l[1]===")"?this.isBefore(b,o):!this.isAfter(b,o))):!1}function tSt(r,i){var o=yv(r)?r:mu(r),l;return this.isValid()&&o.isValid()?(i=S2(i)||"millisecond",i==="millisecond"?this.valueOf()===o.valueOf():(l=o.valueOf(),this.clone().startOf(i).valueOf()<=l&&l<=this.clone().endOf(i).valueOf())):!1}function nSt(r,i){return this.isSame(r,i)||this.isAfter(r,i)}function rSt(r,i){return this.isSame(r,i)||this.isBefore(r,i)}function iSt(r,i,o){var l,f,b;if(!this.isValid())return NaN;if(l=Hie(r,this),!l.isValid())return NaN;switch(f=(l.utcOffset()-this.utcOffset())*6e4,i=S2(i),i){case"year":b=xz(this,l)/12;break;case"month":b=xz(this,l);break;case"quarter":b=xz(this,l)/3;break;case"second":b=(this-l)/1e3;break;case"minute":b=(this-l)/6e4;break;case"hour":b=(this-l)/36e5;break;case"day":b=(this-l-f)/864e5;break;case"week":b=(this-l-f)/6048e5;break;default:b=this-l}return o?b:A2(b)}function xz(r,i){if(r.date()<i.date())return-xz(i,r);var o=(i.year()-r.year())*12+(i.month()-r.month()),l=r.clone().add(o,"months"),f,b;return i-l<0?(f=r.clone().add(o-1,"months"),b=(i-l)/(l-f)):(f=r.clone().add(o+1,"months"),b=(i-l)/(f-l)),-(o+b)||0}Ar.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",Ar.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";function sSt(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")}function aSt(r){if(!this.isValid())return null;var i=r!==!0,o=i?this.clone().utc():this;return o.year()<0||o.year()>9999?oz(o,i?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):Cm(Date.prototype.toISOString)?i?this.toDate().toISOString():new Date(this.valueOf()+this.utcOffset()*60*1e3).toISOString().replace("Z",oz(o,"Z")):oz(o,i?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")}function oSt(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var r="moment",i="",o,l,f,b;return this.isLocal()||(r=this.utcOffset()===0?"moment.utc":"moment.parseZone",i="Z"),o="["+r+'("]',l=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",f="-MM-DD[T]HH:mm:ss.SSS",b=i+'[")]',this.format(o+l+f+b)}function cSt(r){r||(r=this.isUtc()?Ar.defaultFormatUtc:Ar.defaultFormat);var i=oz(this,r);return this.localeData().postformat(i)}function uSt(r,i){return this.isValid()&&(yv(r)&&r.isValid()||mu(r).isValid())?xv({to:this,from:r}).locale(this.locale()).humanize(!i):this.localeData().invalidDate()}function lSt(r){return this.from(mu(),r)}function hSt(r,i){return this.isValid()&&(yv(r)&&r.isValid()||mu(r).isValid())?xv({from:this,to:r}).locale(this.locale()).humanize(!i):this.localeData().invalidDate()}function fSt(r){return this.to(mu(),r)}function I9e(r){var i;return r===void 0?this._locale._abbr:(i=P3(r),i!=null&&(this._locale=i),this)}var O9e=C2("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",function(r){return r===void 0?this.localeData():this.locale(r)});function N9e(){return this._locale}var Ez=1e3,AT=60*Ez,_z=60*AT,P9e=(365*400+97)*24*_z;function MT(r,i){return(r%i+i)%i}function B9e(r,i,o){return r<100&&r>=0?new Date(r+400,i,o)-P9e:new Date(r,i,o).valueOf()}function F9e(r,i,o){return r<100&&r>=0?Date.UTC(r+400,i,o)-P9e:Date.UTC(r,i,o)}function dSt(r){var i,o;if(r=S2(r),r===void 0||r==="millisecond"||!this.isValid())return this;switch(o=this._isUTC?F9e:B9e,r){case"year":i=o(this.year(),0,1);break;case"quarter":i=o(this.year(),this.month()-this.month()%3,1);break;case"month":i=o(this.year(),this.month(),1);break;case"week":i=o(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":i=o(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":i=o(this.year(),this.month(),this.date());break;case"hour":i=this._d.valueOf(),i-=MT(i+(this._isUTC?0:this.utcOffset()*AT),_z);break;case"minute":i=this._d.valueOf(),i-=MT(i,AT);break;case"second":i=this._d.valueOf(),i-=MT(i,Ez);break}return this._d.setTime(i),Ar.updateOffset(this,!0),this}function gSt(r){var i,o;if(r=S2(r),r===void 0||r==="millisecond"||!this.isValid())return this;switch(o=this._isUTC?F9e:B9e,r){case"year":i=o(this.year()+1,0,1)-1;break;case"quarter":i=o(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":i=o(this.year(),this.month()+1,1)-1;break;case"week":i=o(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":i=o(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":i=o(this.year(),this.month(),this.date()+1)-1;break;case"hour":i=this._d.valueOf(),i+=_z-MT(i+(this._isUTC?0:this.utcOffset()*AT),_z)-1;break;case"minute":i=this._d.valueOf(),i+=AT-MT(i,AT)-1;break;case"second":i=this._d.valueOf(),i+=Ez-MT(i,Ez)-1;break}return this._d.setTime(i),Ar.updateOffset(this,!0),this}function pSt(){return this._d.valueOf()-(this._offset||0)*6e4}function bSt(){return Math.floor(this.valueOf()/1e3)}function vSt(){return new Date(this.valueOf())}function wSt(){var r=this;return[r.year(),r.month(),r.date(),r.hour(),r.minute(),r.second(),r.millisecond()]}function mSt(){var r=this;return{years:r.year(),months:r.month(),date:r.date(),hours:r.hours(),minutes:r.minutes(),seconds:r.seconds(),milliseconds:r.milliseconds()}}function ySt(){return this.isValid()?this.toISOString():null}function kSt(){return mie(this)}function xSt(){return n6({},ba(this))}function ESt(){return ba(this).overflow}function _St(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}}Ki("N",0,0,"eraAbbr"),Ki("NN",0,0,"eraAbbr"),Ki("NNN",0,0,"eraAbbr"),Ki("NNNN",0,0,"eraName"),Ki("NNNNN",0,0,"eraNarrow"),Ki("y",["y",1],"yo","eraYear"),Ki("y",["yy",2],0,"eraYear"),Ki("y",["yyy",3],0,"eraYear"),Ki("y",["yyyy",4],0,"eraYear"),ci("N",Gie),ci("NN",Gie),ci("NNN",Gie),ci("NNNN",PSt),ci("NNNNN",BSt),Mc(["N","NN","NNN","NNNN","NNNNN"],function(r,i,o,l){var f=o._locale.erasParse(r,l,o._strict);f?ba(o).era=f:ba(o).invalidEra=r}),ci("y",CT),ci("yy",CT),ci("yyy",CT),ci("yyyy",CT),ci("yo",FSt),Mc(["y","yy","yyy","yyyy"],id),Mc(["yo"],function(r,i,o,l){var f;o._locale._eraYearOrdinalRegex&&(f=r.match(o._locale._eraYearOrdinalRegex)),o._locale.eraYearOrdinalParse?i[id]=o._locale.eraYearOrdinalParse(r,f):i[id]=parseInt(r,10)});function TSt(r,i){var o,l,f,b=this._eras||P3("en")._eras;for(o=0,l=b.length;o<l;++o){switch(typeof b[o].since){case"string":f=Ar(b[o].since).startOf("day"),b[o].since=f.valueOf();break}switch(typeof b[o].until){case"undefined":b[o].until=1/0;break;case"string":f=Ar(b[o].until).startOf("day").valueOf(),b[o].until=f.valueOf();break}}return b}function CSt(r,i,o){var l,f,b=this.eras(),d,w,y;for(r=r.toUpperCase(),l=0,f=b.length;l<f;++l)if(d=b[l].name.toUpperCase(),w=b[l].abbr.toUpperCase(),y=b[l].narrow.toUpperCase(),o)switch(i){case"N":case"NN":case"NNN":if(w===r)return b[l];break;case"NNNN":if(d===r)return b[l];break;case"NNNNN":if(y===r)return b[l];break}else if([d,w,y].indexOf(r)>=0)return b[l]}function SSt(r,i){var o=r.since<=r.until?1:-1;return i===void 0?Ar(r.since).year():Ar(r.since).year()+(i-r.offset)*o}function ASt(){var r,i,o,l=this.localeData().eras();for(r=0,i=l.length;r<i;++r)if(o=this.clone().startOf("day").valueOf(),l[r].since<=o&&o<=l[r].until||l[r].until<=o&&o<=l[r].since)return l[r].name;return""}function MSt(){var r,i,o,l=this.localeData().eras();for(r=0,i=l.length;r<i;++r)if(o=this.clone().startOf("day").valueOf(),l[r].since<=o&&o<=l[r].until||l[r].until<=o&&o<=l[r].since)return l[r].narrow;return""}function DSt(){var r,i,o,l=this.localeData().eras();for(r=0,i=l.length;r<i;++r)if(o=this.clone().startOf("day").valueOf(),l[r].since<=o&&o<=l[r].until||l[r].until<=o&&o<=l[r].since)return l[r].abbr;return""}function LSt(){var r,i,o,l,f=this.localeData().eras();for(r=0,i=f.length;r<i;++r)if(o=f[r].since<=f[r].until?1:-1,l=this.clone().startOf("day").valueOf(),f[r].since<=l&&l<=f[r].until||f[r].until<=l&&l<=f[r].since)return(this.year()-Ar(f[r].since).year())*o+f[r].offset;return this.year()}function ISt(r){return Mo(this,"_erasNameRegex")||Vie.call(this),r?this._erasNameRegex:this._erasRegex}function OSt(r){return Mo(this,"_erasAbbrRegex")||Vie.call(this),r?this._erasAbbrRegex:this._erasRegex}function NSt(r){return Mo(this,"_erasNarrowRegex")||Vie.call(this),r?this._erasNarrowRegex:this._erasRegex}function Gie(r,i){return i.erasAbbrRegex(r)}function PSt(r,i){return i.erasNameRegex(r)}function BSt(r,i){return i.erasNarrowRegex(r)}function FSt(r,i){return i._eraYearOrdinalRegex||CT}function Vie(){var r=[],i=[],o=[],l=[],f,b,d=this.eras();for(f=0,b=d.length;f<b;++f)i.push(cp(d[f].name)),r.push(cp(d[f].abbr)),o.push(cp(d[f].narrow)),l.push(cp(d[f].name)),l.push(cp(d[f].abbr)),l.push(cp(d[f].narrow));this._erasRegex=new RegExp("^("+l.join("|")+")","i"),this._erasNameRegex=new RegExp("^("+i.join("|")+")","i"),this._erasAbbrRegex=new RegExp("^("+r.join("|")+")","i"),this._erasNarrowRegex=new RegExp("^("+o.join("|")+")","i")}Ki(0,["gg",2],0,function(){return this.weekYear()%100}),Ki(0,["GG",2],0,function(){return this.isoWeekYear()%100});function Tz(r,i){Ki(0,[r,r.length],0,i)}Tz("gggg","weekYear"),Tz("ggggg","weekYear"),Tz("GGGG","isoWeekYear"),Tz("GGGGG","isoWeekYear"),nd("weekYear","gg"),nd("isoWeekYear","GG"),rd("weekYear",1),rd("isoWeekYear",1),ci("G",dz),ci("g",dz),ci("GG",wu,op),ci("gg",wu,op),ci("GGGG",Mie,Aie),ci("gggg",Mie,Aie),ci("GGGGG",fz,lz),ci("ggggg",fz,lz),LL(["gggg","ggggg","GGGG","GGGGG"],function(r,i,o,l){i[l.substr(0,2)]=Fa(r)}),LL(["gg","GG"],function(r,i,o,l){i[l]=Ar.parseTwoDigitYear(r)});function RSt(r){return R9e.call(this,r,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)}function jSt(r){return R9e.call(this,r,this.isoWeek(),this.isoWeekday(),1,4)}function $St(){return N3(this.year(),1,4)}function HSt(){return N3(this.isoWeekYear(),1,4)}function zSt(){var r=this.localeData()._week;return N3(this.year(),r.dow,r.doy)}function GSt(){var r=this.localeData()._week;return N3(this.weekYear(),r.dow,r.doy)}function R9e(r,i,o,l,f){var b;return r==null?NL(this,l,f).year:(b=N3(r,l,f),i>b&&(i=b),VSt.call(this,r,i,o,l,f))}function VSt(r,i,o,l,f){var b=p9e(r,i,o,l,f),d=OL(b.year,0,b.dayOfYear);return this.year(d.getUTCFullYear()),this.month(d.getUTCMonth()),this.date(d.getUTCDate()),this}Ki("Q",0,"Qo","quarter"),nd("quarter","Q"),rd("quarter",7),ci("Q",s9e),Mc("Q",function(r,i){i[I3]=(Fa(r)-1)*3});function USt(r){return r==null?Math.ceil((this.month()+1)/3):this.month((r-1)*3+this.month()%3)}Ki("D",["DD",2],"Do","date"),nd("date","D"),rd("date",9),ci("D",wu),ci("DD",wu,op),ci("Do",function(r,i){return r?i._dayOfMonthOrdinalParse||i._ordinalParse:i._dayOfMonthOrdinalParseLenient}),Mc(["D","DD"],Am),Mc("Do",function(r,i){i[Am]=Fa(r.match(wu)[0])});var j9e=TT("Date",!0);Ki("DDD",["DDDD",3],"DDDo","dayOfYear"),nd("dayOfYear","DDD"),rd("dayOfYear",4),ci("DDD",hz),ci("DDDD",a9e),Mc(["DDD","DDDD"],function(r,i,o){o._dayOfYear=Fa(r)});function qSt(r){var i=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return r==null?i:this.add(r-i,"d")}Ki("m",["mm",2],0,"minute"),nd("minute","m"),rd("minute",14),ci("m",wu),ci("mm",wu,op),Mc(["m","mm"],kv);var YSt=TT("Minutes",!1);Ki("s",["ss",2],0,"second"),nd("second","s"),rd("second",15),ci("s",wu),ci("ss",wu,op),Mc(["s","ss"],O3);var WSt=TT("Seconds",!1);Ki("S",0,0,function(){return~~(this.millisecond()/100)}),Ki(0,["SS",2],0,function(){return~~(this.millisecond()/10)}),Ki(0,["SSS",3],0,"millisecond"),Ki(0,["SSSS",4],0,function(){return this.millisecond()*10}),Ki(0,["SSSSS",5],0,function(){return this.millisecond()*100}),Ki(0,["SSSSSS",6],0,function(){return this.millisecond()*1e3}),Ki(0,["SSSSSSS",7],0,function(){return this.millisecond()*1e4}),Ki(0,["SSSSSSSS",8],0,function(){return this.millisecond()*1e5}),Ki(0,["SSSSSSSSS",9],0,function(){return this.millisecond()*1e6}),nd("millisecond","ms"),rd("millisecond",16),ci("S",hz,s9e),ci("SS",hz,op),ci("SSS",hz,a9e);var i6,$9e;for(i6="SSSS";i6.length<=9;i6+="S")ci(i6,CT);function KSt(r,i){i[D8]=Fa(("0."+r)*1e3)}for(i6="S";i6.length<=9;i6+="S")Mc(i6,KSt);$9e=TT("Milliseconds",!1),Ki("z",0,0,"zoneAbbr"),Ki("zz",0,0,"zoneName");function XSt(){return this._isUTC?"UTC":""}function QSt(){return this._isUTC?"Coordinated Universal Time":""}var xr=AL.prototype;xr.add=GCt,xr.calendar=XCt,xr.clone=QCt,xr.diff=iSt,xr.endOf=gSt,xr.format=cSt,xr.from=uSt,xr.fromNow=lSt,xr.to=hSt,xr.toNow=fSt,xr.get=J_t,xr.invalidAt=ESt,xr.isAfter=ZCt,xr.isBefore=JCt,xr.isBetween=eSt,xr.isSame=tSt,xr.isSameOrAfter=nSt,xr.isSameOrBefore=rSt,xr.isValid=kSt,xr.lang=O9e,xr.locale=I9e,xr.localeData=N9e,xr.max=kCt,xr.min=yCt,xr.parsingFlags=xSt,xr.set=eTt,xr.startOf=dSt,xr.subtract=VCt,xr.toArray=wSt,xr.toObject=mSt,xr.toDate=vSt,xr.toISOString=aSt,xr.inspect=oSt,typeof Symbol<"u"&&Symbol.for!=null&&(xr[Symbol.for("nodejs.util.inspect.custom")]=function(){return"Moment<"+this.format()+">"}),xr.toJSON=ySt,xr.toString=sSt,xr.unix=bSt,xr.valueOf=pSt,xr.creationData=_St,xr.eraName=ASt,xr.eraNarrow=MSt,xr.eraAbbr=DSt,xr.eraYear=LSt,xr.year=g9e,xr.isLeapYear=mTt,xr.weekYear=RSt,xr.isoWeekYear=jSt,xr.quarter=xr.quarters=USt,xr.month=f9e,xr.daysInMonth=bTt,xr.week=xr.weeks=TTt,xr.isoWeek=xr.isoWeeks=CTt,xr.weeksInYear=zSt,xr.weeksInWeekYear=GSt,xr.isoWeeksInYear=$St,xr.isoWeeksInISOWeekYear=HSt,xr.date=j9e,xr.day=xr.days=jTt,xr.weekday=$Tt,xr.isoWeekday=HTt,xr.dayOfYear=qSt,xr.hour=xr.hours=WTt,xr.minute=xr.minutes=YSt,xr.second=xr.seconds=WSt,xr.millisecond=xr.milliseconds=$9e,xr.utcOffset=DCt,xr.utc=ICt,xr.local=OCt,xr.parseZone=NCt,xr.hasAlignedHourOffset=PCt,xr.isDST=BCt,xr.isLocal=RCt,xr.isUtcOffset=jCt,xr.isUtc=S9e,xr.isUTC=S9e,xr.zoneAbbr=XSt,xr.zoneName=QSt,xr.dates=C2("dates accessor is deprecated. Use date instead.",j9e),xr.months=C2("months accessor is deprecated. Use month instead",f9e),xr.years=C2("years accessor is deprecated. Use year instead",g9e),xr.zone=C2("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",LCt),xr.isDSTShifted=C2("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",FCt);function ZSt(r){return mu(r*1e3)}function JSt(){return mu.apply(null,arguments).parseZone()}function H9e(r){return r}var Do=Eie.prototype;Do.calendar=j_t,Do.longDateFormat=G_t,Do.invalidDate=U_t,Do.ordinal=W_t,Do.preparse=H9e,Do.postformat=H9e,Do.relativeTime=X_t,Do.pastFuture=Q_t,Do.set=F_t,Do.eras=TSt,Do.erasParse=CSt,Do.erasConvertYear=SSt,Do.erasAbbrRegex=OSt,Do.erasNameRegex=ISt,Do.erasNarrowRegex=NSt,Do.months=fTt,Do.monthsShort=dTt,Do.monthsParse=pTt,Do.monthsRegex=wTt,Do.monthsShortRegex=vTt,Do.week=kTt,Do.firstDayOfYear=_Tt,Do.firstDayOfWeek=ETt,Do.weekdays=NTt,Do.weekdaysMin=BTt,Do.weekdaysShort=PTt,Do.weekdaysParse=RTt,Do.weekdaysRegex=zTt,Do.weekdaysShortRegex=GTt,Do.weekdaysMinRegex=VTt,Do.isPM=qTt,Do.meridiem=KTt;function Cz(r,i,o,l){var f=P3(),b=Tm().set(l,i);return f[o](b,r)}function z9e(r,i,o){if(L3(r)&&(i=r,r=void 0),r=r||"",i!=null)return Cz(r,i,o,"month");var l,f=[];for(l=0;l<12;l++)f[l]=Cz(r,l,o,"month");return f}function Uie(r,i,o,l){typeof r=="boolean"?(L3(i)&&(o=i,i=void 0),i=i||""):(i=r,o=i,r=!1,L3(i)&&(o=i,i=void 0),i=i||"");var f=P3(),b=r?f._week.dow:0,d,w=[];if(o!=null)return Cz(i,(o+b)%7,l,"day");for(d=0;d<7;d++)w[d]=Cz(i,(d+b)%7,l,"day");return w}function eAt(r,i){return z9e(r,i,"months")}function tAt(r,i){return z9e(r,i,"monthsShort")}function nAt(r,i,o){return Uie(r,i,o,"weekdays")}function rAt(r,i,o){return Uie(r,i,o,"weekdaysShort")}function iAt(r,i,o){return Uie(r,i,o,"weekdaysMin")}r6("en",{eras:[{since:"0001-01-01",until:1/0,offset:1,name:"Anno Domini",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"Before Christ",narrow:"BC",abbr:"BC"}],dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(r){var i=r%10,o=Fa(r%100/10)===1?"th":i===1?"st":i===2?"nd":i===3?"rd":"th";return r+o}}),Ar.lang=C2("moment.lang is deprecated. Use moment.locale instead.",r6),Ar.langData=C2("moment.langData is deprecated. Use moment.localeData instead.",P3);var B3=Math.abs;function sAt(){var r=this._data;return this._milliseconds=B3(this._milliseconds),this._days=B3(this._days),this._months=B3(this._months),r.milliseconds=B3(r.milliseconds),r.seconds=B3(r.seconds),r.minutes=B3(r.minutes),r.hours=B3(r.hours),r.months=B3(r.months),r.years=B3(r.years),this}function G9e(r,i,o,l){var f=xv(i,o);return r._milliseconds+=l*f._milliseconds,r._days+=l*f._days,r._months+=l*f._months,r._bubble()}function aAt(r,i){return G9e(this,r,i,1)}function oAt(r,i){return G9e(this,r,i,-1)}function V9e(r){return r<0?Math.floor(r):Math.ceil(r)}function cAt(){var r=this._milliseconds,i=this._days,o=this._months,l=this._data,f,b,d,w,y;return r>=0&&i>=0&&o>=0||r<=0&&i<=0&&o<=0||(r+=V9e(qie(o)+i)*864e5,i=0,o=0),l.milliseconds=r%1e3,f=A2(r/1e3),l.seconds=f%60,b=A2(f/60),l.minutes=b%60,d=A2(b/60),l.hours=d%24,i+=A2(d/24),y=A2(U9e(i)),o+=y,i-=V9e(qie(y)),w=A2(o/12),o%=12,l.days=i,l.months=o,l.years=w,this}function U9e(r){return r*4800/146097}function qie(r){return r*146097/4800}function uAt(r){if(!this.isValid())return NaN;var i,o,l=this._milliseconds;if(r=S2(r),r==="month"||r==="quarter"||r==="year")switch(i=this._days+l/864e5,o=this._months+U9e(i),r){case"month":return o;case"quarter":return o/3;case"year":return o/12}else switch(i=this._days+Math.round(qie(this._months)),r){case"week":return i/7+l/6048e5;case"day":return i+l/864e5;case"hour":return i*24+l/36e5;case"minute":return i*1440+l/6e4;case"second":return i*86400+l/1e3;case"millisecond":return Math.floor(i*864e5)+l;default:throw new Error("Unknown unit "+r)}}function lAt(){return this.isValid()?this._milliseconds+this._days*864e5+this._months%12*2592e6+Fa(this._months/12)*31536e6:NaN}function F3(r){return function(){return this.as(r)}}var hAt=F3("ms"),fAt=F3("s"),dAt=F3("m"),gAt=F3("h"),pAt=F3("d"),bAt=F3("w"),vAt=F3("M"),wAt=F3("Q"),mAt=F3("y");function yAt(){return xv(this)}function kAt(r){return r=S2(r),this.isValid()?this[r+"s"]():NaN}function I8(r){return function(){return this.isValid()?this._data[r]:NaN}}var xAt=I8("milliseconds"),EAt=I8("seconds"),_At=I8("minutes"),TAt=I8("hours"),CAt=I8("days"),SAt=I8("months"),AAt=I8("years");function MAt(){return A2(this.days()/7)}var R3=Math.round,DT={ss:44,s:45,m:45,h:22,d:26,w:null,M:11};function DAt(r,i,o,l,f){return f.relativeTime(i||1,!!o,r,l)}function LAt(r,i,o,l){var f=xv(r).abs(),b=R3(f.as("s")),d=R3(f.as("m")),w=R3(f.as("h")),y=R3(f.as("d")),k=R3(f.as("M")),E=R3(f.as("w")),T=R3(f.as("y")),C=b<=o.ss&&["s",b]||b<o.s&&["ss",b]||d<=1&&["m"]||d<o.m&&["mm",d]||w<=1&&["h"]||w<o.h&&["hh",w]||y<=1&&["d"]||y<o.d&&["dd",y];return o.w!=null&&(C=C||E<=1&&["w"]||E<o.w&&["ww",E]),C=C||k<=1&&["M"]||k<o.M&&["MM",k]||T<=1&&["y"]||["yy",T],C[2]=i,C[3]=+r>0,C[4]=l,DAt.apply(null,C)}function IAt(r){return r===void 0?R3:typeof r=="function"?(R3=r,!0):!1}function OAt(r,i){return DT[r]===void 0?!1:i===void 0?DT[r]:(DT[r]=i,r==="s"&&(DT.ss=i-1),!0)}function NAt(r,i){if(!this.isValid())return this.localeData().invalidDate();var o=!1,l=DT,f,b;return typeof r=="object"&&(i=r,r=!1),typeof r=="boolean"&&(o=r),typeof i=="object"&&(l=Object.assign({},DT,i),i.s!=null&&i.ss==null&&(l.ss=i.s-1)),f=this.localeData(),b=LAt(this,!o,l,f),o&&(b=f.pastFuture(+this,b)),f.postformat(b)}var Yie=Math.abs;function LT(r){return(r>0)-(r<0)||+r}function Sz(){if(!this.isValid())return this.localeData().invalidDate();var r=Yie(this._milliseconds)/1e3,i=Yie(this._days),o=Yie(this._months),l,f,b,d,w=this.asSeconds(),y,k,E,T;return w?(l=A2(r/60),f=A2(l/60),r%=60,l%=60,b=A2(o/12),o%=12,d=r?r.toFixed(3).replace(/\.?0+$/,""):"",y=w<0?"-":"",k=LT(this._months)!==LT(w)?"-":"",E=LT(this._days)!==LT(w)?"-":"",T=LT(this._milliseconds)!==LT(w)?"-":"",y+"P"+(b?k+b+"Y":"")+(o?k+o+"M":"")+(i?E+i+"D":"")+(f||l||r?"T":"")+(f?T+f+"H":"")+(l?T+l+"M":"")+(r?T+d+"S":"")):"P0D"}var vo=yz.prototype;vo.isValid=CCt,vo.abs=sAt,vo.add=aAt,vo.subtract=oAt,vo.as=uAt,vo.asMilliseconds=hAt,vo.asSeconds=fAt,vo.asMinutes=dAt,vo.asHours=gAt,vo.asDays=pAt,vo.asWeeks=bAt,vo.asMonths=vAt,vo.asQuarters=wAt,vo.asYears=mAt,vo.valueOf=lAt,vo._bubble=cAt,vo.clone=yAt,vo.get=kAt,vo.milliseconds=xAt,vo.seconds=EAt,vo.minutes=_At,vo.hours=TAt,vo.days=CAt,vo.weeks=MAt,vo.months=SAt,vo.years=AAt,vo.humanize=NAt,vo.toISOString=Sz,vo.toString=Sz,vo.toJSON=Sz,vo.locale=I9e,vo.localeData=N9e,vo.toIsoString=C2("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",Sz),vo.lang=O9e,Ki("X",0,0,"unix"),Ki("x",0,0,"valueOf"),ci("x",dz),ci("X",nTt),Mc("X",function(r,i,o){o._d=new Date(parseFloat(r)*1e3)}),Mc("x",function(r,i,o){o._d=new Date(Fa(r))});//! moment.js
Ar.version="2.29.4",P_t(mu),Ar.fn=xr,Ar.min=xCt,Ar.max=ECt,Ar.now=_Ct,Ar.utc=Tm,Ar.unix=ZSt,Ar.months=eAt,Ar.isDate=SL,Ar.locale=r6,Ar.invalid=sz,Ar.duration=xv,Ar.isMoment=yv,Ar.weekdays=nAt,Ar.parseZone=JSt,Ar.localeData=P3,Ar.isDuration=kz,Ar.monthsShort=tAt,Ar.weekdaysMin=iAt,Ar.defineLocale=Nie,Ar.updateLocale=JTt,Ar.locales=eCt,Ar.weekdaysShort=rAt,Ar.normalizeUnits=S2,Ar.relativeTimeRounding=IAt,Ar.relativeTimeThreshold=OAt,Ar.calendarFormat=KCt,Ar.prototype=xr,Ar.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"};const j3={trace:0,debug:1,info:2,warn:3,error:4,fatal:5},Se={trace:(...r)=>{},debug:(...r)=>{},info:(...r)=>{},warn:(...r)=>{},error:(...r)=>{},fatal:(...r)=>{}},Wie=function(r="fatal"){let i=j3.fatal;typeof r=="string"?(r=r.toLowerCase(),r in j3&&(i=j3[r])):typeof r=="number"&&(i=r),Se.trace=()=>{},Se.debug=()=>{},Se.info=()=>{},Se.warn=()=>{},Se.error=()=>{},Se.fatal=()=>{},i<=j3.fatal&&(Se.fatal=console.error?console.error.bind(console,M2("FATAL"),"color: orange"):console.log.bind(console,"\x1B[35m",M2("FATAL"))),i<=j3.error&&(Se.error=console.error?console.error.bind(console,M2("ERROR"),"color: orange"):console.log.bind(console,"\x1B[31m",M2("ERROR"))),i<=j3.warn&&(Se.warn=console.warn?console.warn.bind(console,M2("WARN"),"color: orange"):console.log.bind(console,"\x1B[33m",M2("WARN"))),i<=j3.info&&(Se.info=console.info?console.info.bind(console,M2("INFO"),"color: lightblue"):console.log.bind(console,"\x1B[34m",M2("INFO"))),i<=j3.debug&&(Se.debug=console.debug?console.debug.bind(console,M2("DEBUG"),"color: lightgreen"):console.log.bind(console,"\x1B[32m",M2("DEBUG"))),i<=j3.trace&&(Se.trace=console.debug?console.debug.bind(console,M2("TRACE"),"color: lightgreen"):console.log.bind(console,"\x1B[32m",M2("TRACE")))},M2=r=>`%c${Ar().format("ss.SSS")} : ${r} : `;var Mm=typeof globalThis<"u"?globalThis:typeof window<"u"?window:typeof global<"u"?global:typeof self<"u"?self:{};function q9e(r){return r&&r.__esModule&&Object.prototype.hasOwnProperty.call(r,"default")?r.default:r}var Kie={};Object.defineProperty(Kie,"__esModule",{value:!0});var $3=Kie.sanitizeUrl=void 0,PAt=/^([^\w]*)(javascript|data|vbscript)/im,BAt=/&#(\w+)(^\w|;)?/g,FAt=/[\u0000-\u001F\u007F-\u009F\u2000-\u200D\uFEFF]/gim,RAt=/^([^:]+):/gm,jAt=[".","/"];function $At(r){return jAt.indexOf(r[0])>-1}function HAt(r){return r.replace(BAt,function(i,o){return String.fromCharCode(o)})}function zAt(r){var i=HAt(r||"").replace(FAt,"").trim();if(!i)return"about:blank";if($At(i))return i;var o=i.match(RAt);if(!o)return i;var l=o[0];return PAt.test(l)?"about:blank":i}$3=Kie.sanitizeUrl=zAt;function Az(r,i){return r==null||i==null?NaN:r<i?-1:r>i?1:r>=i?0:NaN}function GAt(r,i){return r==null||i==null?NaN:i<r?-1:i>r?1:i>=r?0:NaN}function Xie(r){let i,o,l;r.length!==2?(i=Az,o=(w,y)=>Az(r(w),y),l=(w,y)=>r(w)-y):(i=r===Az||r===GAt?r:VAt,o=r,l=r);function f(w,y,k=0,E=w.length){if(k<E){if(i(y,y)!==0)return E;do{const T=k+E>>>1;o(w[T],y)<0?k=T+1:E=T}while(k<E)}return k}function b(w,y,k=0,E=w.length){if(k<E){if(i(y,y)!==0)return E;do{const T=k+E>>>1;o(w[T],y)<=0?k=T+1:E=T}while(k<E)}return k}function d(w,y,k=0,E=w.length){const T=f(w,y,k,E-1);return T>k&&l(w[T-1],y)>-l(w[T],y)?T-1:T}return{left:f,center:d,right:b}}function VAt(){return 0}function UAt(r){return r===null?NaN:+r}const qAt=Xie(Az).right;Xie(UAt).center;const YAt=qAt;class Y9e extends Map{constructor(i,o=XAt){if(super(),Object.defineProperties(this,{_intern:{value:new Map},_key:{value:o}}),i!=null)for(const[l,f]of i)this.set(l,f)}get(i){return super.get(W9e(this,i))}has(i){return super.has(W9e(this,i))}set(i,o){return super.set(WAt(this,i),o)}delete(i){return super.delete(KAt(this,i))}}function W9e({_intern:r,_key:i},o){const l=i(o);return r.has(l)?r.get(l):o}function WAt({_intern:r,_key:i},o){const l=i(o);return r.has(l)?r.get(l):(r.set(l,o),o)}function KAt({_intern:r,_key:i},o){const l=i(o);return r.has(l)&&(o=r.get(l),r.delete(l)),o}function XAt(r){return r!==null&&typeof r=="object"?r.valueOf():r}var Qie=Math.sqrt(50),Zie=Math.sqrt(10),Jie=Math.sqrt(2);function QAt(r,i,o){var l,f=-1,b,d,w;if(i=+i,r=+r,o=+o,r===i&&o>0)return[r];if((l=i<r)&&(b=r,r=i,i=b),(w=K9e(r,i,o))===0||!isFinite(w))return[];if(w>0){let y=Math.round(r/w),k=Math.round(i/w);for(y*w<r&&++y,k*w>i&&--k,d=new Array(b=k-y+1);++f<b;)d[f]=(y+f)*w}else{w=-w;let y=Math.round(r*w),k=Math.round(i*w);for(y/w<r&&++y,k/w>i&&--k,d=new Array(b=k-y+1);++f<b;)d[f]=(y+f)/w}return l&&d.reverse(),d}function K9e(r,i,o){var l=(i-r)/Math.max(0,o),f=Math.floor(Math.log(l)/Math.LN10),b=l/Math.pow(10,f);return f>=0?(b>=Qie?10:b>=Zie?5:b>=Jie?2:1)*Math.pow(10,f):-Math.pow(10,-f)/(b>=Qie?10:b>=Zie?5:b>=Jie?2:1)}function ese(r,i,o){var l=Math.abs(i-r)/Math.max(0,o),f=Math.pow(10,Math.floor(Math.log(l)/Math.LN10)),b=l/f;return b>=Qie?f*=10:b>=Zie?f*=5:b>=Jie&&(f*=2),i<r?-f:f}function ZAt(r,i){let o;if(i===void 0)for(const l of r)l!=null&&(o<l||o===void 0&&l>=l)&&(o=l);else{let l=-1;for(let f of r)(f=i(f,++l,r))!=null&&(o<f||o===void 0&&f>=f)&&(o=f)}return o}function JAt(r,i){let o;if(i===void 0)for(const l of r)l!=null&&(o>l||o===void 0&&l>=l)&&(o=l);else{let l=-1;for(let f of r)(f=i(f,++l,r))!=null&&(o>f||o===void 0&&f>=f)&&(o=f)}return o}function eMt(r){return r}var Mz=1,tse=2,nse=3,Dz=4,X9e=1e-6;function tMt(r){return"translate("+r+",0)"}function nMt(r){return"translate(0,"+r+")"}function rMt(r){return i=>+r(i)}function iMt(r,i){return i=Math.max(0,r.bandwidth()-i*2)/2,r.round()&&(i=Math.round(i)),o=>+r(o)+i}function sMt(){return!this.__axis}function Q9e(r,i){var o=[],l=null,f=null,b=6,d=6,w=3,y=typeof window<"u"&&window.devicePixelRatio>1?0:.5,k=r===Mz||r===Dz?-1:1,E=r===Dz||r===tse?"x":"y",T=r===Mz||r===nse?tMt:nMt;function C(S){var L=l==null?i.ticks?i.ticks.apply(i,o):i.domain():l,O=f==null?i.tickFormat?i.tickFormat.apply(i,o):eMt:f,B=Math.max(b,0)+w,N=i.range(),F=+N[0]+y,R=+N[N.length-1]+y,q=(i.bandwidth?iMt:rMt)(i.copy(),y),X=S.selection?S.selection():S,te=X.selectAll(".domain").data([null]),H=X.selectAll(".tick").data(L,i).order(),Y=H.exit(),z=H.enter().append("g").attr("class","tick"),W=H.select("line"),Z=H.select("text");te=te.merge(te.enter().insert("path",".tick").attr("class","domain").attr("stroke","currentColor")),H=H.merge(z),W=W.merge(z.append("line").attr("stroke","currentColor").attr(E+"2",k*b)),Z=Z.merge(z.append("text").attr("fill","currentColor").attr(E,k*B).attr("dy",r===Mz?"0em":r===nse?"0.71em":"0.32em")),S!==X&&(te=te.transition(S),H=H.transition(S),W=W.transition(S),Z=Z.transition(S),Y=Y.transition(S).attr("opacity",X9e).attr("transform",function(G){return isFinite(G=q(G))?T(G+y):this.getAttribute("transform")}),z.attr("opacity",X9e).attr("transform",function(G){var ae=this.parentNode.__axis;return T((ae&&isFinite(ae=ae(G))?ae:q(G))+y)})),Y.remove(),te.attr("d",r===Dz||r===tse?d?"M"+k*d+","+F+"H"+y+"V"+R+"H"+k*d:"M"+y+","+F+"V"+R:d?"M"+F+","+k*d+"V"+y+"H"+R+"V"+k*d:"M"+F+","+y+"H"+R),H.attr("opacity",1).attr("transform",function(G){return T(q(G)+y)}),W.attr(E+"2",k*b),Z.attr(E,k*B).text(O),X.filter(sMt).attr("fill","none").attr("font-size",10).attr("font-family","sans-serif").attr("text-anchor",r===tse?"start":r===Dz?"end":"middle"),X.each(function(){this.__axis=q})}return C.scale=function(S){return arguments.length?(i=S,C):i},C.ticks=function(){return o=Array.from(arguments),C},C.tickArguments=function(S){return arguments.length?(o=S==null?[]:Array.from(S),C):o.slice()},C.tickValues=function(S){return arguments.length?(l=S==null?null:Array.from(S),C):l&&l.slice()},C.tickFormat=function(S){return arguments.length?(f=S,C):f},C.tickSize=function(S){return arguments.length?(b=d=+S,C):b},C.tickSizeInner=function(S){return arguments.length?(b=+S,C):b},C.tickSizeOuter=function(S){return arguments.length?(d=+S,C):d},C.tickPadding=function(S){return arguments.length?(w=+S,C):w},C.offset=function(S){return arguments.length?(y=+S,C):y},C}function aMt(r){return Q9e(Mz,r)}function oMt(r){return Q9e(nse,r)}var cMt={value:()=>{}};function Z9e(){for(var r=0,i=arguments.length,o={},l;r<i;++r){if(!(l=arguments[r]+"")||l in o||/[\s.]/.test(l))throw new Error("illegal type: "+l);o[l]=[]}return new Lz(o)}function Lz(r){this._=r}function uMt(r,i){return r.trim().split(/^|\s+/).map(function(o){var l="",f=o.indexOf(".");if(f>=0&&(l=o.slice(f+1),o=o.slice(0,f)),o&&!i.hasOwnProperty(o))throw new Error("unknown type: "+o);return{type:o,name:l}})}Lz.prototype=Z9e.prototype={constructor:Lz,on:function(r,i){var o=this._,l=uMt(r+"",o),f,b=-1,d=l.length;if(arguments.length<2){for(;++b<d;)if((f=(r=l[b]).type)&&(f=lMt(o[f],r.name)))return f;return}if(i!=null&&typeof i!="function")throw new Error("invalid callback: "+i);for(;++b<d;)if(f=(r=l[b]).type)o[f]=J9e(o[f],r.name,i);else if(i==null)for(f in o)o[f]=J9e(o[f],r.name,null);return this},copy:function(){var r={},i=this._;for(var o in i)r[o]=i[o].slice();return new Lz(r)},call:function(r,i){if((f=arguments.length-2)>0)for(var o=new Array(f),l=0,f,b;l<f;++l)o[l]=arguments[l+2];if(!this._.hasOwnProperty(r))throw new Error("unknown type: "+r);for(b=this._[r],l=0,f=b.length;l<f;++l)b[l].value.apply(i,o)},apply:function(r,i,o){if(!this._.hasOwnProperty(r))throw new Error("unknown type: "+r);for(var l=this._[r],f=0,b=l.length;f<b;++f)l[f].value.apply(i,o)}};function lMt(r,i){for(var o=0,l=r.length,f;o<l;++o)if((f=r[o]).name===i)return f.value}function J9e(r,i,o){for(var l=0,f=r.length;l<f;++l)if(r[l].name===i){r[l]=cMt,r=r.slice(0,l).concat(r.slice(l+1));break}return o!=null&&r.push({name:i,value:o}),r}var rse="http://www.w3.org/1999/xhtml";const e_e={svg:"http://www.w3.org/2000/svg",xhtml:rse,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function Iz(r){var i=r+="",o=i.indexOf(":");return o>=0&&(i=r.slice(0,o))!=="xmlns"&&(r=r.slice(o+1)),e_e.hasOwnProperty(i)?{space:e_e[i],local:r}:r}function hMt(r){return function(){var i=this.ownerDocument,o=this.namespaceURI;return o===rse&&i.documentElement.namespaceURI===rse?i.createElement(r):i.createElementNS(o,r)}}function fMt(r){return function(){return this.ownerDocument.createElementNS(r.space,r.local)}}function t_e(r){var i=Iz(r);return(i.local?fMt:hMt)(i)}function dMt(){}function ise(r){return r==null?dMt:function(){return this.querySelector(r)}}function gMt(r){typeof r!="function"&&(r=ise(r));for(var i=this._groups,o=i.length,l=new Array(o),f=0;f<o;++f)for(var b=i[f],d=b.length,w=l[f]=new Array(d),y,k,E=0;E<d;++E)(y=b[E])&&(k=r.call(y,y.__data__,E,b))&&("__data__"in y&&(k.__data__=y.__data__),w[E]=k);return new Yd(l,this._parents)}function n_e(r){return r==null?[]:Array.isArray(r)?r:Array.from(r)}function pMt(){return[]}function r_e(r){return r==null?pMt:function(){return this.querySelectorAll(r)}}function bMt(r){return function(){return n_e(r.apply(this,arguments))}}function vMt(r){typeof r=="function"?r=bMt(r):r=r_e(r);for(var i=this._groups,o=i.length,l=[],f=[],b=0;b<o;++b)for(var d=i[b],w=d.length,y,k=0;k<w;++k)(y=d[k])&&(l.push(r.call(y,y.__data__,k,d)),f.push(y));return new Yd(l,f)}function i_e(r){return function(){return this.matches(r)}}function s_e(r){return function(i){return i.matches(r)}}var wMt=Array.prototype.find;function mMt(r){return function(){return wMt.call(this.children,r)}}function yMt(){return this.firstElementChild}function kMt(r){return this.select(r==null?yMt:mMt(typeof r=="function"?r:s_e(r)))}var xMt=Array.prototype.filter;function EMt(){return Array.from(this.children)}function _Mt(r){return function(){return xMt.call(this.children,r)}}function TMt(r){return this.selectAll(r==null?EMt:_Mt(typeof r=="function"?r:s_e(r)))}function CMt(r){typeof r!="function"&&(r=i_e(r));for(var i=this._groups,o=i.length,l=new Array(o),f=0;f<o;++f)for(var b=i[f],d=b.length,w=l[f]=[],y,k=0;k<d;++k)(y=b[k])&&r.call(y,y.__data__,k,b)&&w.push(y);return new Yd(l,this._parents)}function a_e(r){return new Array(r.length)}function SMt(){return new Yd(this._enter||this._groups.map(a_e),this._parents)}function Oz(r,i){this.ownerDocument=r.ownerDocument,this.namespaceURI=r.namespaceURI,this._next=null,this._parent=r,this.__data__=i}Oz.prototype={constructor:Oz,appendChild:function(r){return this._parent.insertBefore(r,this._next)},insertBefore:function(r,i){return this._parent.insertBefore(r,i)},querySelector:function(r){return this._parent.querySelector(r)},querySelectorAll:function(r){return this._parent.querySelectorAll(r)}};function AMt(r){return function(){return r}}function MMt(r,i,o,l,f,b){for(var d=0,w,y=i.length,k=b.length;d<k;++d)(w=i[d])?(w.__data__=b[d],l[d]=w):o[d]=new Oz(r,b[d]);for(;d<y;++d)(w=i[d])&&(f[d]=w)}function DMt(r,i,o,l,f,b,d){var w,y,k=new Map,E=i.length,T=b.length,C=new Array(E),S;for(w=0;w<E;++w)(y=i[w])&&(C[w]=S=d.call(y,y.__data__,w,i)+"",k.has(S)?f[w]=y:k.set(S,y));for(w=0;w<T;++w)S=d.call(r,b[w],w,b)+"",(y=k.get(S))?(l[w]=y,y.__data__=b[w],k.delete(S)):o[w]=new Oz(r,b[w]);for(w=0;w<E;++w)(y=i[w])&&k.get(C[w])===y&&(f[w]=y)}function LMt(r){return r.__data__}function IMt(r,i){if(!arguments.length)return Array.from(this,LMt);var o=i?DMt:MMt,l=this._parents,f=this._groups;typeof r!="function"&&(r=AMt(r));for(var b=f.length,d=new Array(b),w=new Array(b),y=new Array(b),k=0;k<b;++k){var E=l[k],T=f[k],C=T.length,S=OMt(r.call(E,E&&E.__data__,k,l)),L=S.length,O=w[k]=new Array(L),B=d[k]=new Array(L),N=y[k]=new Array(C);o(E,T,O,B,N,S,i);for(var F=0,R=0,q,X;F<L;++F)if(q=O[F]){for(F>=R&&(R=F+1);!(X=B[R])&&++R<L;);q._next=X||null}}return d=new Yd(d,l),d._enter=w,d._exit=y,d}function OMt(r){return typeof r=="object"&&"length"in r?r:Array.from(r)}function NMt(){return new Yd(this._exit||this._groups.map(a_e),this._parents)}function PMt(r,i,o){var l=this.enter(),f=this,b=this.exit();return typeof r=="function"?(l=r(l),l&&(l=l.selection())):l=l.append(r+""),i!=null&&(f=i(f),f&&(f=f.selection())),o==null?b.remove():o(b),l&&f?l.merge(f).order():f}function BMt(r){for(var i=r.selection?r.selection():r,o=this._groups,l=i._groups,f=o.length,b=l.length,d=Math.min(f,b),w=new Array(f),y=0;y<d;++y)for(var k=o[y],E=l[y],T=k.length,C=w[y]=new Array(T),S,L=0;L<T;++L)(S=k[L]||E[L])&&(C[L]=S);for(;y<f;++y)w[y]=o[y];return new Yd(w,this._parents)}function FMt(){for(var r=this._groups,i=-1,o=r.length;++i<o;)for(var l=r[i],f=l.length-1,b=l[f],d;--f>=0;)(d=l[f])&&(b&&d.compareDocumentPosition(b)^4&&b.parentNode.insertBefore(d,b),b=d);return this}function RMt(r){r||(r=jMt);function i(T,C){return T&&C?r(T.__data__,C.__data__):!T-!C}for(var o=this._groups,l=o.length,f=new Array(l),b=0;b<l;++b){for(var d=o[b],w=d.length,y=f[b]=new Array(w),k,E=0;E<w;++E)(k=d[E])&&(y[E]=k);y.sort(i)}return new Yd(f,this._parents).order()}function jMt(r,i){return r<i?-1:r>i?1:r>=i?0:NaN}function $Mt(){var r=arguments[0];return arguments[0]=this,r.apply(null,arguments),this}function HMt(){return Array.from(this)}function zMt(){for(var r=this._groups,i=0,o=r.length;i<o;++i)for(var l=r[i],f=0,b=l.length;f<b;++f){var d=l[f];if(d)return d}return null}function GMt(){let r=0;for(const i of this)++r;return r}function VMt(){return!this.node()}function UMt(r){for(var i=this._groups,o=0,l=i.length;o<l;++o)for(var f=i[o],b=0,d=f.length,w;b<d;++b)(w=f[b])&&r.call(w,w.__data__,b,f);return this}function qMt(r){return function(){this.removeAttribute(r)}}function YMt(r){return function(){this.removeAttributeNS(r.space,r.local)}}function WMt(r,i){return function(){this.setAttribute(r,i)}}function KMt(r,i){return function(){this.setAttributeNS(r.space,r.local,i)}}function XMt(r,i){return function(){var o=i.apply(this,arguments);o==null?this.removeAttribute(r):this.setAttribute(r,o)}}function QMt(r,i){return function(){var o=i.apply(this,arguments);o==null?this.removeAttributeNS(r.space,r.local):this.setAttributeNS(r.space,r.local,o)}}function ZMt(r,i){var o=Iz(r);if(arguments.length<2){var l=this.node();return o.local?l.getAttributeNS(o.space,o.local):l.getAttribute(o)}return this.each((i==null?o.local?YMt:qMt:typeof i=="function"?o.local?QMt:XMt:o.local?KMt:WMt)(o,i))}function o_e(r){return r.ownerDocument&&r.ownerDocument.defaultView||r.document&&r||r.defaultView}function JMt(r){return function(){this.style.removeProperty(r)}}function eDt(r,i,o){return function(){this.style.setProperty(r,i,o)}}function tDt(r,i,o){return function(){var l=i.apply(this,arguments);l==null?this.style.removeProperty(r):this.style.setProperty(r,l,o)}}function nDt(r,i,o){return arguments.length>1?this.each((i==null?JMt:typeof i=="function"?tDt:eDt)(r,i,o==null?"":o)):IT(this.node(),r)}function IT(r,i){return r.style.getPropertyValue(i)||o_e(r).getComputedStyle(r,null).getPropertyValue(i)}function rDt(r){return function(){delete this[r]}}function iDt(r,i){return function(){this[r]=i}}function sDt(r,i){return function(){var o=i.apply(this,arguments);o==null?delete this[r]:this[r]=o}}function aDt(r,i){return arguments.length>1?this.each((i==null?rDt:typeof i=="function"?sDt:iDt)(r,i)):this.node()[r]}function c_e(r){return r.trim().split(/^|\s+/)}function sse(r){return r.classList||new u_e(r)}function u_e(r){this._node=r,this._names=c_e(r.getAttribute("class")||"")}u_e.prototype={add:function(r){var i=this._names.indexOf(r);i<0&&(this._names.push(r),this._node.setAttribute("class",this._names.join(" ")))},remove:function(r){var i=this._names.indexOf(r);i>=0&&(this._names.splice(i,1),this._node.setAttribute("class",this._names.join(" ")))},contains:function(r){return this._names.indexOf(r)>=0}};function l_e(r,i){for(var o=sse(r),l=-1,f=i.length;++l<f;)o.add(i[l])}function h_e(r,i){for(var o=sse(r),l=-1,f=i.length;++l<f;)o.remove(i[l])}function oDt(r){return function(){l_e(this,r)}}function cDt(r){return function(){h_e(this,r)}}function uDt(r,i){return function(){(i.apply(this,arguments)?l_e:h_e)(this,r)}}function lDt(r,i){var o=c_e(r+"");if(arguments.length<2){for(var l=sse(this.node()),f=-1,b=o.length;++f<b;)if(!l.contains(o[f]))return!1;return!0}return this.each((typeof i=="function"?uDt:i?oDt:cDt)(o,i))}function hDt(){this.textContent=""}function fDt(r){return function(){this.textContent=r}}function dDt(r){return function(){var i=r.apply(this,arguments);this.textContent=i==null?"":i}}function gDt(r){return arguments.length?this.each(r==null?hDt:(typeof r=="function"?dDt:fDt)(r)):this.node().textContent}function pDt(){this.innerHTML=""}function bDt(r){return function(){this.innerHTML=r}}function vDt(r){return function(){var i=r.apply(this,arguments);this.innerHTML=i==null?"":i}}function wDt(r){return arguments.length?this.each(r==null?pDt:(typeof r=="function"?vDt:bDt)(r)):this.node().innerHTML}function mDt(){this.nextSibling&&this.parentNode.appendChild(this)}function yDt(){return this.each(mDt)}function kDt(){this.previousSibling&&this.parentNode.insertBefore(this,this.parentNode.firstChild)}function xDt(){return this.each(kDt)}function EDt(r){var i=typeof r=="function"?r:t_e(r);return this.select(function(){return this.appendChild(i.apply(this,arguments))})}function _Dt(){return null}function TDt(r,i){var o=typeof r=="function"?r:t_e(r),l=i==null?_Dt:typeof i=="function"?i:ise(i);return this.select(function(){return this.insertBefore(o.apply(this,arguments),l.apply(this,arguments)||null)})}function CDt(){var r=this.parentNode;r&&r.removeChild(this)}function SDt(){return this.each(CDt)}function ADt(){var r=this.cloneNode(!1),i=this.parentNode;return i?i.insertBefore(r,this.nextSibling):r}function MDt(){var r=this.cloneNode(!0),i=this.parentNode;return i?i.insertBefore(r,this.nextSibling):r}function DDt(r){return this.select(r?MDt:ADt)}function LDt(r){return arguments.length?this.property("__data__",r):this.node().__data__}function IDt(r){return function(i){r.call(this,i,this.__data__)}}function ODt(r){return r.trim().split(/^|\s+/).map(function(i){var o="",l=i.indexOf(".");return l>=0&&(o=i.slice(l+1),i=i.slice(0,l)),{type:i,name:o}})}function NDt(r){return function(){var i=this.__on;if(!!i){for(var o=0,l=-1,f=i.length,b;o<f;++o)b=i[o],(!r.type||b.type===r.type)&&b.name===r.name?this.removeEventListener(b.type,b.listener,b.options):i[++l]=b;++l?i.length=l:delete this.__on}}}function PDt(r,i,o){return function(){var l=this.__on,f,b=IDt(i);if(l){for(var d=0,w=l.length;d<w;++d)if((f=l[d]).type===r.type&&f.name===r.name){this.removeEventListener(f.type,f.listener,f.options),this.addEventListener(f.type,f.listener=b,f.options=o),f.value=i;return}}this.addEventListener(r.type,b,o),f={type:r.type,name:r.name,value:i,listener:b,options:o},l?l.push(f):this.__on=[f]}}function BDt(r,i,o){var l=ODt(r+""),f,b=l.length,d;if(arguments.length<2){var w=this.node().__on;if(w){for(var y=0,k=w.length,E;y<k;++y)for(f=0,E=w[y];f<b;++f)if((d=l[f]).type===E.type&&d.name===E.name)return E.value}return}for(w=i?PDt:NDt,f=0;f<b;++f)this.each(w(l[f],i,o));return this}function f_e(r,i,o){var l=o_e(r),f=l.CustomEvent;typeof f=="function"?f=new f(i,o):(f=l.document.createEvent("Event"),o?(f.initEvent(i,o.bubbles,o.cancelable),f.detail=o.detail):f.initEvent(i,!1,!1)),r.dispatchEvent(f)}function FDt(r,i){return function(){return f_e(this,r,i)}}function RDt(r,i){return function(){return f_e(this,r,i.apply(this,arguments))}}function jDt(r,i){return this.each((typeof i=="function"?RDt:FDt)(r,i))}function*$Dt(){for(var r=this._groups,i=0,o=r.length;i<o;++i)for(var l=r[i],f=0,b=l.length,d;f<b;++f)(d=l[f])&&(yield d)}var ase=[null];function Yd(r,i){this._groups=r,this._parents=i}function RL(){return new Yd([[document.documentElement]],ase)}function HDt(){return this}Yd.prototype=RL.prototype={constructor:Yd,select:gMt,selectAll:vMt,selectChild:kMt,selectChildren:TMt,filter:CMt,data:IMt,enter:SMt,exit:NMt,join:PMt,merge:BMt,selection:HDt,order:FMt,sort:RMt,call:$Mt,nodes:HMt,node:zMt,size:GMt,empty:VMt,each:UMt,attr:ZMt,style:nDt,property:aDt,classed:lDt,text:gDt,html:wDt,raise:yDt,lower:xDt,append:EDt,insert:TDt,remove:SDt,clone:DDt,datum:LDt,on:BDt,dispatch:jDt,[Symbol.iterator]:$Dt};function Cn(r){return typeof r=="string"?new Yd([[document.querySelector(r)]],[document.documentElement]):new Yd([[r]],ase)}function ose(r){return typeof r=="string"?new Yd([document.querySelectorAll(r)],[document.documentElement]):new Yd([n_e(r)],ase)}function jL(r,i,o){r.prototype=i.prototype=o,o.constructor=r}function Nz(r,i){var o=Object.create(r.prototype);for(var l in i)o[l]=i[l];return o}function O8(){}var $L=.7,Pz=1/$L,OT="\\s*([+-]?\\d+)\\s*",HL="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)\\s*",Dm="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)%\\s*",zDt=/^#([0-9a-f]{3,8})$/,GDt=new RegExp(`^rgb\\(${OT},${OT},${OT}\\)$`),VDt=new RegExp(`^rgb\\(${Dm},${Dm},${Dm}\\)$`),UDt=new RegExp(`^rgba\\(${OT},${OT},${OT},${HL}\\)$`),qDt=new RegExp(`^rgba\\(${Dm},${Dm},${Dm},${HL}\\)$`),YDt=new RegExp(`^hsl\\(${HL},${Dm},${Dm}\\)$`),WDt=new RegExp(`^hsla\\(${HL},${Dm},${Dm},${HL}\\)$`),d_e={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};jL(O8,N8,{copy(r){return Object.assign(new this.constructor,this,r)},displayable(){return this.rgb().displayable()},hex:g_e,formatHex:g_e,formatHex8:KDt,formatHsl:XDt,formatRgb:p_e,toString:p_e});function g_e(){return this.rgb().formatHex()}function KDt(){return this.rgb().formatHex8()}function XDt(){return k_e(this).formatHsl()}function p_e(){return this.rgb().formatRgb()}function N8(r){var i,o;return r=(r+"").trim().toLowerCase(),(i=zDt.exec(r))?(o=i[1].length,i=parseInt(i[1],16),o===6?b_e(i):o===3?new sd(i>>8&15|i>>4&240,i>>4&15|i&240,(i&15)<<4|i&15,1):o===8?Bz(i>>24&255,i>>16&255,i>>8&255,(i&255)/255):o===4?Bz(i>>12&15|i>>8&240,i>>8&15|i>>4&240,i>>4&15|i&240,((i&15)<<4|i&15)/255):null):(i=GDt.exec(r))?new sd(i[1],i[2],i[3],1):(i=VDt.exec(r))?new sd(i[1]*255/100,i[2]*255/100,i[3]*255/100,1):(i=UDt.exec(r))?Bz(i[1],i[2],i[3],i[4]):(i=qDt.exec(r))?Bz(i[1]*255/100,i[2]*255/100,i[3]*255/100,i[4]):(i=YDt.exec(r))?y_e(i[1],i[2]/100,i[3]/100,1):(i=WDt.exec(r))?y_e(i[1],i[2]/100,i[3]/100,i[4]):d_e.hasOwnProperty(r)?b_e(d_e[r]):r==="transparent"?new sd(NaN,NaN,NaN,0):null}function b_e(r){return new sd(r>>16&255,r>>8&255,r&255,1)}function Bz(r,i,o,l){return l<=0&&(r=i=o=NaN),new sd(r,i,o,l)}function v_e(r){return r instanceof O8||(r=N8(r)),r?(r=r.rgb(),new sd(r.r,r.g,r.b,r.opacity)):new sd}function cse(r,i,o,l){return arguments.length===1?v_e(r):new sd(r,i,o,l==null?1:l)}function sd(r,i,o,l){this.r=+r,this.g=+i,this.b=+o,this.opacity=+l}jL(sd,cse,Nz(O8,{brighter(r){return r=r==null?Pz:Math.pow(Pz,r),new sd(this.r*r,this.g*r,this.b*r,this.opacity)},darker(r){return r=r==null?$L:Math.pow($L,r),new sd(this.r*r,this.g*r,this.b*r,this.opacity)},rgb(){return this},clamp(){return new sd(P8(this.r),P8(this.g),P8(this.b),Fz(this.opacity))},displayable(){return-.5<=this.r&&this.r<255.5&&-.5<=this.g&&this.g<255.5&&-.5<=this.b&&this.b<255.5&&0<=this.opacity&&this.opacity<=1},hex:w_e,formatHex:w_e,formatHex8:QDt,formatRgb:m_e,toString:m_e}));function w_e(){return`#${B8(this.r)}${B8(this.g)}${B8(this.b)}`}function QDt(){return`#${B8(this.r)}${B8(this.g)}${B8(this.b)}${B8((isNaN(this.opacity)?1:this.opacity)*255)}`}function m_e(){const r=Fz(this.opacity);return`${r===1?"rgb(":"rgba("}${P8(this.r)}, ${P8(this.g)}, ${P8(this.b)}${r===1?")":`, ${r})`}`}function Fz(r){return isNaN(r)?1:Math.max(0,Math.min(1,r))}function P8(r){return Math.max(0,Math.min(255,Math.round(r)||0))}function B8(r){return r=P8(r),(r<16?"0":"")+r.toString(16)}function y_e(r,i,o,l){return l<=0?r=i=o=NaN:o<=0||o>=1?r=i=NaN:i<=0&&(r=NaN),new Ev(r,i,o,l)}function k_e(r){if(r instanceof Ev)return new Ev(r.h,r.s,r.l,r.opacity);if(r instanceof O8||(r=N8(r)),!r)return new Ev;if(r instanceof Ev)return r;r=r.rgb();var i=r.r/255,o=r.g/255,l=r.b/255,f=Math.min(i,o,l),b=Math.max(i,o,l),d=NaN,w=b-f,y=(b+f)/2;return w?(i===b?d=(o-l)/w+(o<l)*6:o===b?d=(l-i)/w+2:d=(i-o)/w+4,w/=y<.5?b+f:2-b-f,d*=60):w=y>0&&y<1?0:d,new Ev(d,w,y,r.opacity)}function ZDt(r,i,o,l){return arguments.length===1?k_e(r):new Ev(r,i,o,l==null?1:l)}function Ev(r,i,o,l){this.h=+r,this.s=+i,this.l=+o,this.opacity=+l}jL(Ev,ZDt,Nz(O8,{brighter(r){return r=r==null?Pz:Math.pow(Pz,r),new Ev(this.h,this.s,this.l*r,this.opacity)},darker(r){return r=r==null?$L:Math.pow($L,r),new Ev(this.h,this.s,this.l*r,this.opacity)},rgb(){var r=this.h%360+(this.h<0)*360,i=isNaN(r)||isNaN(this.s)?0:this.s,o=this.l,l=o+(o<.5?o:1-o)*i,f=2*o-l;return new sd(use(r>=240?r-240:r+120,f,l),use(r,f,l),use(r<120?r+240:r-120,f,l),this.opacity)},clamp(){return new Ev(x_e(this.h),Rz(this.s),Rz(this.l),Fz(this.opacity))},displayable(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1},formatHsl(){const r=Fz(this.opacity);return`${r===1?"hsl(":"hsla("}${x_e(this.h)}, ${Rz(this.s)*100}%, ${Rz(this.l)*100}%${r===1?")":`, ${r})`}`}}));function x_e(r){return r=(r||0)%360,r<0?r+360:r}function Rz(r){return Math.max(0,Math.min(1,r||0))}function use(r,i,o){return(r<60?i+(o-i)*r/60:r<180?o:r<240?i+(o-i)*(240-r)/60:i)*255}const JDt=Math.PI/180,eLt=180/Math.PI,jz=18,E_e=.96422,__e=1,T_e=.82521,C_e=4/29,NT=6/29,S_e=3*NT*NT,tLt=NT*NT*NT;function A_e(r){if(r instanceof Lm)return new Lm(r.l,r.a,r.b,r.opacity);if(r instanceof H3)return M_e(r);r instanceof sd||(r=v_e(r));var i=dse(r.r),o=dse(r.g),l=dse(r.b),f=lse((.2225045*i+.7168786*o+.0606169*l)/__e),b,d;return i===o&&o===l?b=d=f:(b=lse((.4360747*i+.3850649*o+.1430804*l)/E_e),d=lse((.0139322*i+.0971045*o+.7141733*l)/T_e)),new Lm(116*f-16,500*(b-f),200*(f-d),r.opacity)}function nLt(r,i,o,l){return arguments.length===1?A_e(r):new Lm(r,i,o,l==null?1:l)}function Lm(r,i,o,l){this.l=+r,this.a=+i,this.b=+o,this.opacity=+l}jL(Lm,nLt,Nz(O8,{brighter(r){return new Lm(this.l+jz*(r==null?1:r),this.a,this.b,this.opacity)},darker(r){return new Lm(this.l-jz*(r==null?1:r),this.a,this.b,this.opacity)},rgb(){var r=(this.l+16)/116,i=isNaN(this.a)?r:r+this.a/500,o=isNaN(this.b)?r:r-this.b/200;return i=E_e*hse(i),r=__e*hse(r),o=T_e*hse(o),new sd(fse(3.1338561*i-1.6168667*r-.4906146*o),fse(-.9787684*i+1.9161415*r+.033454*o),fse(.0719453*i-.2289914*r+1.4052427*o),this.opacity)}}));function lse(r){return r>tLt?Math.pow(r,1/3):r/S_e+C_e}function hse(r){return r>NT?r*r*r:S_e*(r-C_e)}function fse(r){return 255*(r<=.0031308?12.92*r:1.055*Math.pow(r,1/2.4)-.055)}function dse(r){return(r/=255)<=.04045?r/12.92:Math.pow((r+.055)/1.055,2.4)}function rLt(r){if(r instanceof H3)return new H3(r.h,r.c,r.l,r.opacity);if(r instanceof Lm||(r=A_e(r)),r.a===0&&r.b===0)return new H3(NaN,0<r.l&&r.l<100?0:NaN,r.l,r.opacity);var i=Math.atan2(r.b,r.a)*eLt;return new H3(i<0?i+360:i,Math.sqrt(r.a*r.a+r.b*r.b),r.l,r.opacity)}function gse(r,i,o,l){return arguments.length===1?rLt(r):new H3(r,i,o,l==null?1:l)}function H3(r,i,o,l){this.h=+r,this.c=+i,this.l=+o,this.opacity=+l}function M_e(r){if(isNaN(r.h))return new Lm(r.l,0,0,r.opacity);var i=r.h*JDt;return new Lm(r.l,Math.cos(i)*r.c,Math.sin(i)*r.c,r.opacity)}jL(H3,gse,Nz(O8,{brighter(r){return new H3(this.h,this.c,this.l+jz*(r==null?1:r),this.opacity)},darker(r){return new H3(this.h,this.c,this.l-jz*(r==null?1:r),this.opacity)},rgb(){return M_e(this).rgb()}}));const $z=r=>()=>r;function D_e(r,i){return function(o){return r+o*i}}function iLt(r,i,o){return r=Math.pow(r,o),i=Math.pow(i,o)-r,o=1/o,function(l){return Math.pow(r+l*i,o)}}function sLt(r,i){var o=i-r;return o?D_e(r,o>180||o<-180?o-360*Math.round(o/360):o):$z(isNaN(r)?i:r)}function aLt(r){return(r=+r)==1?zL:function(i,o){return o-i?iLt(i,o,r):$z(isNaN(i)?o:i)}}function zL(r,i){var o=i-r;return o?D_e(r,o):$z(isNaN(r)?i:r)}const Hz=function r(i){var o=aLt(i);function l(f,b){var d=o((f=cse(f)).r,(b=cse(b)).r),w=o(f.g,b.g),y=o(f.b,b.b),k=zL(f.opacity,b.opacity);return function(E){return f.r=d(E),f.g=w(E),f.b=y(E),f.opacity=k(E),f+""}}return l.gamma=r,l}(1);function oLt(r,i){i||(i=[]);var o=r?Math.min(i.length,r.length):0,l=i.slice(),f;return function(b){for(f=0;f<o;++f)l[f]=r[f]*(1-b)+i[f]*b;return l}}function cLt(r){return ArrayBuffer.isView(r)&&!(r instanceof DataView)}function uLt(r,i){var o=i?i.length:0,l=r?Math.min(o,r.length):0,f=new Array(l),b=new Array(o),d;for(d=0;d<l;++d)f[d]=vse(r[d],i[d]);for(;d<o;++d)b[d]=i[d];return function(w){for(d=0;d<l;++d)b[d]=f[d](w);return b}}function lLt(r,i){var o=new Date;return r=+r,i=+i,function(l){return o.setTime(r*(1-l)+i*l),o}}function _v(r,i){return r=+r,i=+i,function(o){return r*(1-o)+i*o}}function hLt(r,i){var o={},l={},f;(r===null||typeof r!="object")&&(r={}),(i===null||typeof i!="object")&&(i={});for(f in i)f in r?o[f]=vse(r[f],i[f]):l[f]=i[f];return function(b){for(f in o)l[f]=o[f](b);return l}}var pse=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,bse=new RegExp(pse.source,"g");function fLt(r){return function(){return r}}function dLt(r){return function(i){return r(i)+""}}function L_e(r,i){var o=pse.lastIndex=bse.lastIndex=0,l,f,b,d=-1,w=[],y=[];for(r=r+"",i=i+"";(l=pse.exec(r))&&(f=bse.exec(i));)(b=f.index)>o&&(b=i.slice(o,b),w[d]?w[d]+=b:w[++d]=b),(l=l[0])===(f=f[0])?w[d]?w[d]+=f:w[++d]=f:(w[++d]=null,y.push({i:d,x:_v(l,f)})),o=bse.lastIndex;return o<i.length&&(b=i.slice(o),w[d]?w[d]+=b:w[++d]=b),w.length<2?y[0]?dLt(y[0].x):fLt(i):(i=y.length,function(k){for(var E=0,T;E<i;++E)w[(T=y[E]).i]=T.x(k);return w.join("")})}function vse(r,i){var o=typeof i,l;return i==null||o==="boolean"?$z(i):(o==="number"?_v:o==="string"?(l=N8(i))?(i=l,Hz):L_e:i instanceof N8?Hz:i instanceof Date?lLt:cLt(i)?oLt:Array.isArray(i)?uLt:typeof i.valueOf!="function"&&typeof i.toString!="function"||isNaN(i)?hLt:_v)(r,i)}function gLt(r,i){return r=+r,i=+i,function(o){return Math.round(r*(1-o)+i*o)}}var I_e=180/Math.PI,wse={translateX:0,translateY:0,rotate:0,skewX:0,scaleX:1,scaleY:1};function O_e(r,i,o,l,f,b){var d,w,y;return(d=Math.sqrt(r*r+i*i))&&(r/=d,i/=d),(y=r*o+i*l)&&(o-=r*y,l-=i*y),(w=Math.sqrt(o*o+l*l))&&(o/=w,l/=w,y/=w),r*l<i*o&&(r=-r,i=-i,y=-y,d=-d),{translateX:f,translateY:b,rotate:Math.atan2(i,r)*I_e,skewX:Math.atan(y)*I_e,scaleX:d,scaleY:w}}var zz;function pLt(r){const i=new(typeof DOMMatrix=="function"?DOMMatrix:WebKitCSSMatrix)(r+"");return i.isIdentity?wse:O_e(i.a,i.b,i.c,i.d,i.e,i.f)}function bLt(r){return r==null||(zz||(zz=document.createElementNS("http://www.w3.org/2000/svg","g")),zz.setAttribute("transform",r),!(r=zz.transform.baseVal.consolidate()))?wse:(r=r.matrix,O_e(r.a,r.b,r.c,r.d,r.e,r.f))}function N_e(r,i,o,l){function f(k){return k.length?k.pop()+" ":""}function b(k,E,T,C,S,L){if(k!==T||E!==C){var O=S.push("translate(",null,i,null,o);L.push({i:O-4,x:_v(k,T)},{i:O-2,x:_v(E,C)})}else(T||C)&&S.push("translate("+T+i+C+o)}function d(k,E,T,C){k!==E?(k-E>180?E+=360:E-k>180&&(k+=360),C.push({i:T.push(f(T)+"rotate(",null,l)-2,x:_v(k,E)})):E&&T.push(f(T)+"rotate("+E+l)}function w(k,E,T,C){k!==E?C.push({i:T.push(f(T)+"skewX(",null,l)-2,x:_v(k,E)}):E&&T.push(f(T)+"skewX("+E+l)}function y(k,E,T,C,S,L){if(k!==T||E!==C){var O=S.push(f(S)+"scale(",null,",",null,")");L.push({i:O-4,x:_v(k,T)},{i:O-2,x:_v(E,C)})}else(T!==1||C!==1)&&S.push(f(S)+"scale("+T+","+C+")")}return function(k,E){var T=[],C=[];return k=r(k),E=r(E),b(k.translateX,k.translateY,E.translateX,E.translateY,T,C),d(k.rotate,E.rotate,T,C),w(k.skewX,E.skewX,T,C),y(k.scaleX,k.scaleY,E.scaleX,E.scaleY,T,C),k=E=null,function(S){for(var L=-1,O=C.length,B;++L<O;)T[(B=C[L]).i]=B.x(S);return T.join("")}}}var vLt=N_e(pLt,"px, ","px)","deg)"),wLt=N_e(bLt,", ",")",")");function mLt(r){return function(i,o){var l=r((i=gse(i)).h,(o=gse(o)).h),f=zL(i.c,o.c),b=zL(i.l,o.l),d=zL(i.opacity,o.opacity);return function(w){return i.h=l(w),i.c=f(w),i.l=b(w),i.opacity=d(w),i+""}}}const yLt=mLt(sLt);var PT=0,GL=0,VL=0,P_e=1e3,Gz,UL,Vz=0,F8=0,Uz=0,qL=typeof performance=="object"&&performance.now?performance:Date,B_e=typeof window=="object"&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(r){setTimeout(r,17)};function mse(){return F8||(B_e(kLt),F8=qL.now()+Uz)}function kLt(){F8=0}function qz(){this._call=this._time=this._next=null}qz.prototype=F_e.prototype={constructor:qz,restart:function(r,i,o){if(typeof r!="function")throw new TypeError("callback is not a function");o=(o==null?mse():+o)+(i==null?0:+i),!this._next&&UL!==this&&(UL?UL._next=this:Gz=this,UL=this),this._call=r,this._time=o,yse()},stop:function(){this._call&&(this._call=null,this._time=1/0,yse())}};function F_e(r,i,o){var l=new qz;return l.restart(r,i,o),l}function xLt(){mse(),++PT;for(var r=Gz,i;r;)(i=F8-r._time)>=0&&r._call.call(void 0,i),r=r._next;--PT}function R_e(){F8=(Vz=qL.now())+Uz,PT=GL=0;try{xLt()}finally{PT=0,_Lt(),F8=0}}function ELt(){var r=qL.now(),i=r-Vz;i>P_e&&(Uz-=i,Vz=r)}function _Lt(){for(var r,i=Gz,o,l=1/0;i;)i._call?(l>i._time&&(l=i._time),r=i,i=i._next):(o=i._next,i._next=null,i=r?r._next=o:Gz=o);UL=r,yse(l)}function yse(r){if(!PT){GL&&(GL=clearTimeout(GL));var i=r-F8;i>24?(r<1/0&&(GL=setTimeout(R_e,r-qL.now()-Uz)),VL&&(VL=clearInterval(VL))):(VL||(Vz=qL.now(),VL=setInterval(ELt,P_e)),PT=1,B_e(R_e))}}function j_e(r,i,o){var l=new qz;return i=i==null?0:+i,l.restart(f=>{l.stop(),r(f+i)},i,o),l}var TLt=Z9e("start","end","cancel","interrupt"),CLt=[],$_e=0,H_e=1,kse=2,Yz=3,z_e=4,xse=5,Wz=6;function Kz(r,i,o,l,f,b){var d=r.__transition;if(!d)r.__transition={};else if(o in d)return;SLt(r,o,{name:i,index:l,group:f,on:TLt,tween:CLt,time:b.time,delay:b.delay,duration:b.duration,ease:b.ease,timer:null,state:$_e})}function Ese(r,i){var o=Tv(r,i);if(o.state>$_e)throw new Error("too late; already scheduled");return o}function Im(r,i){var o=Tv(r,i);if(o.state>Yz)throw new Error("too late; already running");return o}function Tv(r,i){var o=r.__transition;if(!o||!(o=o[i]))throw new Error("transition not found");return o}function SLt(r,i,o){var l=r.__transition,f;l[i]=o,o.timer=F_e(b,0,o.time);function b(k){o.state=H_e,o.timer.restart(d,o.delay,o.time),o.delay<=k&&d(k-o.delay)}function d(k){var E,T,C,S;if(o.state!==H_e)return y();for(E in l)if(S=l[E],S.name===o.name){if(S.state===Yz)return j_e(d);S.state===z_e?(S.state=Wz,S.timer.stop(),S.on.call("interrupt",r,r.__data__,S.index,S.group),delete l[E]):+E<i&&(S.state=Wz,S.timer.stop(),S.on.call("cancel",r,r.__data__,S.index,S.group),delete l[E])}if(j_e(function(){o.state===Yz&&(o.state=z_e,o.timer.restart(w,o.delay,o.time),w(k))}),o.state=kse,o.on.call("start",r,r.__data__,o.index,o.group),o.state===kse){for(o.state=Yz,f=new Array(C=o.tween.length),E=0,T=-1;E<C;++E)(S=o.tween[E].value.call(r,r.__data__,o.index,o.group))&&(f[++T]=S);f.length=T+1}}function w(k){for(var E=k<o.duration?o.ease.call(null,k/o.duration):(o.timer.restart(y),o.state=xse,1),T=-1,C=f.length;++T<C;)f[T].call(r,E);o.state===xse&&(o.on.call("end",r,r.__data__,o.index,o.group),y())}function y(){o.state=Wz,o.timer.stop(),delete l[i];for(var k in l)return;delete r.__transition}}function ALt(r,i){var o=r.__transition,l,f,b=!0,d;if(!!o){i=i==null?null:i+"";for(d in o){if((l=o[d]).name!==i){b=!1;continue}f=l.state>kse&&l.state<xse,l.state=Wz,l.timer.stop(),l.on.call(f?"interrupt":"cancel",r,r.__data__,l.index,l.group),delete o[d]}b&&delete r.__transition}}function MLt(r){return this.each(function(){ALt(this,r)})}function DLt(r,i){var o,l;return function(){var f=Im(this,r),b=f.tween;if(b!==o){l=o=b;for(var d=0,w=l.length;d<w;++d)if(l[d].name===i){l=l.slice(),l.splice(d,1);break}}f.tween=l}}function LLt(r,i,o){var l,f;if(typeof o!="function")throw new Error;return function(){var b=Im(this,r),d=b.tween;if(d!==l){f=(l=d).slice();for(var w={name:i,value:o},y=0,k=f.length;y<k;++y)if(f[y].name===i){f[y]=w;break}y===k&&f.push(w)}b.tween=f}}function ILt(r,i){var o=this._id;if(r+="",arguments.length<2){for(var l=Tv(this.node(),o).tween,f=0,b=l.length,d;f<b;++f)if((d=l[f]).name===r)return d.value;return null}return this.each((i==null?DLt:LLt)(o,r,i))}function _se(r,i,o){var l=r._id;return r.each(function(){var f=Im(this,l);(f.value||(f.value={}))[i]=o.apply(this,arguments)}),function(f){return Tv(f,l).value[i]}}function G_e(r,i){var o;return(typeof i=="number"?_v:i instanceof N8?Hz:(o=N8(i))?(i=o,Hz):L_e)(r,i)}function OLt(r){return function(){this.removeAttribute(r)}}function NLt(r){return function(){this.removeAttributeNS(r.space,r.local)}}function PLt(r,i,o){var l,f=o+"",b;return function(){var d=this.getAttribute(r);return d===f?null:d===l?b:b=i(l=d,o)}}function BLt(r,i,o){var l,f=o+"",b;return function(){var d=this.getAttributeNS(r.space,r.local);return d===f?null:d===l?b:b=i(l=d,o)}}function FLt(r,i,o){var l,f,b;return function(){var d,w=o(this),y;return w==null?void this.removeAttribute(r):(d=this.getAttribute(r),y=w+"",d===y?null:d===l&&y===f?b:(f=y,b=i(l=d,w)))}}function RLt(r,i,o){var l,f,b;return function(){var d,w=o(this),y;return w==null?void this.removeAttributeNS(r.space,r.local):(d=this.getAttributeNS(r.space,r.local),y=w+"",d===y?null:d===l&&y===f?b:(f=y,b=i(l=d,w)))}}function jLt(r,i){var o=Iz(r),l=o==="transform"?wLt:G_e;return this.attrTween(r,typeof i=="function"?(o.local?RLt:FLt)(o,l,_se(this,"attr."+r,i)):i==null?(o.local?NLt:OLt)(o):(o.local?BLt:PLt)(o,l,i))}function $Lt(r,i){return function(o){this.setAttribute(r,i.call(this,o))}}function HLt(r,i){return function(o){this.setAttributeNS(r.space,r.local,i.call(this,o))}}function zLt(r,i){var o,l;function f(){var b=i.apply(this,arguments);return b!==l&&(o=(l=b)&&HLt(r,b)),o}return f._value=i,f}function GLt(r,i){var o,l;function f(){var b=i.apply(this,arguments);return b!==l&&(o=(l=b)&&$Lt(r,b)),o}return f._value=i,f}function VLt(r,i){var o="attr."+r;if(arguments.length<2)return(o=this.tween(o))&&o._value;if(i==null)return this.tween(o,null);if(typeof i!="function")throw new Error;var l=Iz(r);return this.tween(o,(l.local?zLt:GLt)(l,i))}function ULt(r,i){return function(){Ese(this,r).delay=+i.apply(this,arguments)}}function qLt(r,i){return i=+i,function(){Ese(this,r).delay=i}}function YLt(r){var i=this._id;return arguments.length?this.each((typeof r=="function"?ULt:qLt)(i,r)):Tv(this.node(),i).delay}function WLt(r,i){return function(){Im(this,r).duration=+i.apply(this,arguments)}}function KLt(r,i){return i=+i,function(){Im(this,r).duration=i}}function XLt(r){var i=this._id;return arguments.length?this.each((typeof r=="function"?WLt:KLt)(i,r)):Tv(this.node(),i).duration}function QLt(r,i){if(typeof i!="function")throw new Error;return function(){Im(this,r).ease=i}}function ZLt(r){var i=this._id;return arguments.length?this.each(QLt(i,r)):Tv(this.node(),i).ease}function JLt(r,i){return function(){var o=i.apply(this,arguments);if(typeof o!="function")throw new Error;Im(this,r).ease=o}}function eIt(r){if(typeof r!="function")throw new Error;return this.each(JLt(this._id,r))}function tIt(r){typeof r!="function"&&(r=i_e(r));for(var i=this._groups,o=i.length,l=new Array(o),f=0;f<o;++f)for(var b=i[f],d=b.length,w=l[f]=[],y,k=0;k<d;++k)(y=b[k])&&r.call(y,y.__data__,k,b)&&w.push(y);return new z3(l,this._parents,this._name,this._id)}function nIt(r){if(r._id!==this._id)throw new Error;for(var i=this._groups,o=r._groups,l=i.length,f=o.length,b=Math.min(l,f),d=new Array(l),w=0;w<b;++w)for(var y=i[w],k=o[w],E=y.length,T=d[w]=new Array(E),C,S=0;S<E;++S)(C=y[S]||k[S])&&(T[S]=C);for(;w<l;++w)d[w]=i[w];return new z3(d,this._parents,this._name,this._id)}function rIt(r){return(r+"").trim().split(/^|\s+/).every(function(i){var o=i.indexOf(".");return o>=0&&(i=i.slice(0,o)),!i||i==="start"})}function iIt(r,i,o){var l,f,b=rIt(i)?Ese:Im;return function(){var d=b(this,r),w=d.on;w!==l&&(f=(l=w).copy()).on(i,o),d.on=f}}function sIt(r,i){var o=this._id;return arguments.length<2?Tv(this.node(),o).on.on(r):this.each(iIt(o,r,i))}function aIt(r){return function(){var i=this.parentNode;for(var o in this.__transition)if(+o!==r)return;i&&i.removeChild(this)}}function oIt(){return this.on("end.remove",aIt(this._id))}function cIt(r){var i=this._name,o=this._id;typeof r!="function"&&(r=ise(r));for(var l=this._groups,f=l.length,b=new Array(f),d=0;d<f;++d)for(var w=l[d],y=w.length,k=b[d]=new Array(y),E,T,C=0;C<y;++C)(E=w[C])&&(T=r.call(E,E.__data__,C,w))&&("__data__"in E&&(T.__data__=E.__data__),k[C]=T,Kz(k[C],i,o,C,k,Tv(E,o)));return new z3(b,this._parents,i,o)}function uIt(r){var i=this._name,o=this._id;typeof r!="function"&&(r=r_e(r));for(var l=this._groups,f=l.length,b=[],d=[],w=0;w<f;++w)for(var y=l[w],k=y.length,E,T=0;T<k;++T)if(E=y[T]){for(var C=r.call(E,E.__data__,T,y),S,L=Tv(E,o),O=0,B=C.length;O<B;++O)(S=C[O])&&Kz(S,i,o,O,C,L);b.push(C),d.push(E)}return new z3(b,d,i,o)}var lIt=RL.prototype.constructor;function hIt(){return new lIt(this._groups,this._parents)}function fIt(r,i){var o,l,f;return function(){var b=IT(this,r),d=(this.style.removeProperty(r),IT(this,r));return b===d?null:b===o&&d===l?f:f=i(o=b,l=d)}}function V_e(r){return function(){this.style.removeProperty(r)}}function dIt(r,i,o){var l,f=o+"",b;return function(){var d=IT(this,r);return d===f?null:d===l?b:b=i(l=d,o)}}function gIt(r,i,o){var l,f,b;return function(){var d=IT(this,r),w=o(this),y=w+"";return w==null&&(y=w=(this.style.removeProperty(r),IT(this,r))),d===y?null:d===l&&y===f?b:(f=y,b=i(l=d,w))}}function pIt(r,i){var o,l,f,b="style."+i,d="end."+b,w;return function(){var y=Im(this,r),k=y.on,E=y.value[b]==null?w||(w=V_e(i)):void 0;(k!==o||f!==E)&&(l=(o=k).copy()).on(d,f=E),y.on=l}}function bIt(r,i,o){var l=(r+="")=="transform"?vLt:G_e;return i==null?this.styleTween(r,fIt(r,l)).on("end.style."+r,V_e(r)):typeof i=="function"?this.styleTween(r,gIt(r,l,_se(this,"style."+r,i))).each(pIt(this._id,r)):this.styleTween(r,dIt(r,l,i),o).on("end.style."+r,null)}function vIt(r,i,o){return function(l){this.style.setProperty(r,i.call(this,l),o)}}function wIt(r,i,o){var l,f;function b(){var d=i.apply(this,arguments);return d!==f&&(l=(f=d)&&vIt(r,d,o)),l}return b._value=i,b}function mIt(r,i,o){var l="style."+(r+="");if(arguments.length<2)return(l=this.tween(l))&&l._value;if(i==null)return this.tween(l,null);if(typeof i!="function")throw new Error;return this.tween(l,wIt(r,i,o==null?"":o))}function yIt(r){return function(){this.textContent=r}}function kIt(r){return function(){var i=r(this);this.textContent=i==null?"":i}}function xIt(r){return this.tween("text",typeof r=="function"?kIt(_se(this,"text",r)):yIt(r==null?"":r+""))}function EIt(r){return function(i){this.textContent=r.call(this,i)}}function _It(r){var i,o;function l(){var f=r.apply(this,arguments);return f!==o&&(i=(o=f)&&EIt(f)),i}return l._value=r,l}function TIt(r){var i="text";if(arguments.length<1)return(i=this.tween(i))&&i._value;if(r==null)return this.tween(i,null);if(typeof r!="function")throw new Error;return this.tween(i,_It(r))}function CIt(){for(var r=this._name,i=this._id,o=U_e(),l=this._groups,f=l.length,b=0;b<f;++b)for(var d=l[b],w=d.length,y,k=0;k<w;++k)if(y=d[k]){var E=Tv(y,i);Kz(y,r,o,k,d,{time:E.time+E.delay+E.duration,delay:0,duration:E.duration,ease:E.ease})}return new z3(l,this._parents,r,o)}function SIt(){var r,i,o=this,l=o._id,f=o.size();return new Promise(function(b,d){var w={value:d},y={value:function(){--f===0&&b()}};o.each(function(){var k=Im(this,l),E=k.on;E!==r&&(i=(r=E).copy(),i._.cancel.push(w),i._.interrupt.push(w),i._.end.push(y)),k.on=i}),f===0&&b()})}var AIt=0;function z3(r,i,o,l){this._groups=r,this._parents=i,this._name=o,this._id=l}function U_e(){return++AIt}var G3=RL.prototype;z3.prototype={constructor:z3,select:cIt,selectAll:uIt,selectChild:G3.selectChild,selectChildren:G3.selectChildren,filter:tIt,merge:nIt,selection:hIt,transition:CIt,call:G3.call,nodes:G3.nodes,node:G3.node,size:G3.size,empty:G3.empty,each:G3.each,on:sIt,attr:jLt,attrTween:VLt,style:bIt,styleTween:mIt,text:xIt,textTween:TIt,remove:oIt,tween:ILt,delay:YLt,duration:XLt,ease:ZLt,easeVarying:eIt,end:SIt,[Symbol.iterator]:G3[Symbol.iterator]};function MIt(r){return((r*=2)<=1?r*r*r:(r-=2)*r*r+2)/2}var DIt={time:null,delay:0,duration:250,ease:MIt};function LIt(r,i){for(var o;!(o=r.__transition)||!(o=o[i]);)if(!(r=r.parentNode))throw new Error(`transition ${i} not found`);return o}function IIt(r){var i,o;r instanceof z3?(i=r._id,r=r._name):(i=U_e(),(o=DIt).time=mse(),r=r==null?null:r+"");for(var l=this._groups,f=l.length,b=0;b<f;++b)for(var d=l[b],w=d.length,y,k=0;k<w;++k)(y=d[k])&&Kz(y,r,i,k,d,o||LIt(y,i));return new z3(l,this._parents,r,i)}RL.prototype.interrupt=MLt,RL.prototype.transition=IIt;const Tse=Math.PI,Cse=2*Tse,R8=1e-6,OIt=Cse-R8;function Sse(){this._x0=this._y0=this._x1=this._y1=null,this._=""}function Ase(){return new Sse}Sse.prototype=Ase.prototype={constructor:Sse,moveTo:function(r,i){this._+="M"+(this._x0=this._x1=+r)+","+(this._y0=this._y1=+i)},closePath:function(){this._x1!==null&&(this._x1=this._x0,this._y1=this._y0,this._+="Z")},lineTo:function(r,i){this._+="L"+(this._x1=+r)+","+(this._y1=+i)},quadraticCurveTo:function(r,i,o,l){this._+="Q"+ +r+","+ +i+","+(this._x1=+o)+","+(this._y1=+l)},bezierCurveTo:function(r,i,o,l,f,b){this._+="C"+ +r+","+ +i+","+ +o+","+ +l+","+(this._x1=+f)+","+(this._y1=+b)},arcTo:function(r,i,o,l,f){r=+r,i=+i,o=+o,l=+l,f=+f;var b=this._x1,d=this._y1,w=o-r,y=l-i,k=b-r,E=d-i,T=k*k+E*E;if(f<0)throw new Error("negative radius: "+f);if(this._x1===null)this._+="M"+(this._x1=r)+","+(this._y1=i);else if(T>R8)if(!(Math.abs(E*w-y*k)>R8)||!f)this._+="L"+(this._x1=r)+","+(this._y1=i);else{var C=o-b,S=l-d,L=w*w+y*y,O=C*C+S*S,B=Math.sqrt(L),N=Math.sqrt(T),F=f*Math.tan((Tse-Math.acos((L+T-O)/(2*B*N)))/2),R=F/N,q=F/B;Math.abs(R-1)>R8&&(this._+="L"+(r+R*k)+","+(i+R*E)),this._+="A"+f+","+f+",0,0,"+ +(E*C>k*S)+","+(this._x1=r+q*w)+","+(this._y1=i+q*y)}},arc:function(r,i,o,l,f,b){r=+r,i=+i,o=+o,b=!!b;var d=o*Math.cos(l),w=o*Math.sin(l),y=r+d,k=i+w,E=1^b,T=b?l-f:f-l;if(o<0)throw new Error("negative radius: "+o);this._x1===null?this._+="M"+y+","+k:(Math.abs(this._x1-y)>R8||Math.abs(this._y1-k)>R8)&&(this._+="L"+y+","+k),o&&(T<0&&(T=T%Cse+Cse),T>OIt?this._+="A"+o+","+o+",0,1,"+E+","+(r-d)+","+(i-w)+"A"+o+","+o+",0,1,"+E+","+(this._x1=y)+","+(this._y1=k):T>R8&&(this._+="A"+o+","+o+",0,"+ +(T>=Tse)+","+E+","+(this._x1=r+o*Math.cos(f))+","+(this._y1=i+o*Math.sin(f))))},rect:function(r,i,o,l){this._+="M"+(this._x0=this._x1=+r)+","+(this._y0=this._y1=+i)+"h"+ +o+"v"+ +l+"h"+-o+"Z"},toString:function(){return this._}};function NIt(r){if(!r.ok)throw new Error(r.status+" "+r.statusText);return r.text()}function PIt(r,i){return fetch(r,i).then(NIt)}function BIt(r){return(i,o)=>PIt(i,o).then(l=>new DOMParser().parseFromString(l,r))}var FIt=BIt("image/svg+xml");function RIt(r){return Math.abs(r=Math.round(r))>=1e21?r.toLocaleString("en").replace(/,/g,""):r.toString(10)}function Xz(r,i){if((o=(r=i?r.toExponential(i-1):r.toExponential()).indexOf("e"))<0)return null;var o,l=r.slice(0,o);return[l.length>1?l[0]+l.slice(2):l,+r.slice(o+1)]}function BT(r){return r=Xz(Math.abs(r)),r?r[1]:NaN}function jIt(r,i){return function(o,l){for(var f=o.length,b=[],d=0,w=r[0],y=0;f>0&&w>0&&(y+w+1>l&&(w=Math.max(1,l-y)),b.push(o.substring(f-=w,f+w)),!((y+=w+1)>l));)w=r[d=(d+1)%r.length];return b.reverse().join(i)}}function $It(r){return function(i){return i.replace(/[0-9]/g,function(o){return r[+o]})}}var HIt=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function Qz(r){if(!(i=HIt.exec(r)))throw new Error("invalid format: "+r);var i;return new Mse({fill:i[1],align:i[2],sign:i[3],symbol:i[4],zero:i[5],width:i[6],comma:i[7],precision:i[8]&&i[8].slice(1),trim:i[9],type:i[10]})}Qz.prototype=Mse.prototype;function Mse(r){this.fill=r.fill===void 0?" ":r.fill+"",this.align=r.align===void 0?">":r.align+"",this.sign=r.sign===void 0?"-":r.sign+"",this.symbol=r.symbol===void 0?"":r.symbol+"",this.zero=!!r.zero,this.width=r.width===void 0?void 0:+r.width,this.comma=!!r.comma,this.precision=r.precision===void 0?void 0:+r.precision,this.trim=!!r.trim,this.type=r.type===void 0?"":r.type+""}Mse.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(this.width===void 0?"":Math.max(1,this.width|0))+(this.comma?",":"")+(this.precision===void 0?"":"."+Math.max(0,this.precision|0))+(this.trim?"~":"")+this.type};function zIt(r){e:for(var i=r.length,o=1,l=-1,f;o<i;++o)switch(r[o]){case".":l=f=o;break;case"0":l===0&&(l=o),f=o;break;default:if(!+r[o])break e;l>0&&(l=0);break}return l>0?r.slice(0,l)+r.slice(f+1):r}var q_e;function GIt(r,i){var o=Xz(r,i);if(!o)return r+"";var l=o[0],f=o[1],b=f-(q_e=Math.max(-8,Math.min(8,Math.floor(f/3)))*3)+1,d=l.length;return b===d?l:b>d?l+new Array(b-d+1).join("0"):b>0?l.slice(0,b)+"."+l.slice(b):"0."+new Array(1-b).join("0")+Xz(r,Math.max(0,i+b-1))[0]}function Y_e(r,i){var o=Xz(r,i);if(!o)return r+"";var l=o[0],f=o[1];return f<0?"0."+new Array(-f).join("0")+l:l.length>f+1?l.slice(0,f+1)+"."+l.slice(f+1):l+new Array(f-l.length+2).join("0")}const W_e={"%":(r,i)=>(r*100).toFixed(i),b:r=>Math.round(r).toString(2),c:r=>r+"",d:RIt,e:(r,i)=>r.toExponential(i),f:(r,i)=>r.toFixed(i),g:(r,i)=>r.toPrecision(i),o:r=>Math.round(r).toString(8),p:(r,i)=>Y_e(r*100,i),r:Y_e,s:GIt,X:r=>Math.round(r).toString(16).toUpperCase(),x:r=>Math.round(r).toString(16)};function K_e(r){return r}var X_e=Array.prototype.map,Q_e=["y","z","a","f","p","n","\xB5","m","","k","M","G","T","P","E","Z","Y"];function VIt(r){var i=r.grouping===void 0||r.thousands===void 0?K_e:jIt(X_e.call(r.grouping,Number),r.thousands+""),o=r.currency===void 0?"":r.currency[0]+"",l=r.currency===void 0?"":r.currency[1]+"",f=r.decimal===void 0?".":r.decimal+"",b=r.numerals===void 0?K_e:$It(X_e.call(r.numerals,String)),d=r.percent===void 0?"%":r.percent+"",w=r.minus===void 0?"\u2212":r.minus+"",y=r.nan===void 0?"NaN":r.nan+"";function k(T){T=Qz(T);var C=T.fill,S=T.align,L=T.sign,O=T.symbol,B=T.zero,N=T.width,F=T.comma,R=T.precision,q=T.trim,X=T.type;X==="n"?(F=!0,X="g"):W_e[X]||(R===void 0&&(R=12),q=!0,X="g"),(B||C==="0"&&S==="=")&&(B=!0,C="0",S="=");var te=O==="$"?o:O==="#"&&/[boxX]/.test(X)?"0"+X.toLowerCase():"",H=O==="$"?l:/[%p]/.test(X)?d:"",Y=W_e[X],z=/[defgprs%]/.test(X);R=R===void 0?6:/[gprs]/.test(X)?Math.max(1,Math.min(21,R)):Math.max(0,Math.min(20,R));function W(Z){var G=te,ae=H,$,ge,ee;if(X==="c")ae=Y(Z)+ae,Z="";else{Z=+Z;var de=Z<0||1/Z<0;if(Z=isNaN(Z)?y:Y(Math.abs(Z),R),q&&(Z=zIt(Z)),de&&+Z==0&&L!=="+"&&(de=!1),G=(de?L==="("?L:w:L==="-"||L==="("?"":L)+G,ae=(X==="s"?Q_e[8+q_e/3]:"")+ae+(de&&L==="("?")":""),z){for($=-1,ge=Z.length;++$<ge;)if(ee=Z.charCodeAt($),48>ee||ee>57){ae=(ee===46?f+Z.slice($+1):Z.slice($))+ae,Z=Z.slice(0,$);break}}}F&&!B&&(Z=i(Z,1/0));var re=G.length+Z.length+ae.length,ke=re<N?new Array(N-re+1).join(C):"";switch(F&&B&&(Z=i(ke+Z,ke.length?N-ae.length:1/0),ke=""),S){case"<":Z=G+Z+ae+ke;break;case"=":Z=G+ke+Z+ae;break;case"^":Z=ke.slice(0,re=ke.length>>1)+G+Z+ae+ke.slice(re);break;default:Z=ke+G+Z+ae;break}return b(Z)}return W.toString=function(){return T+""},W}function E(T,C){var S=k((T=Qz(T),T.type="f",T)),L=Math.max(-8,Math.min(8,Math.floor(BT(C)/3)))*3,O=Math.pow(10,-L),B=Q_e[8+L/3];return function(N){return S(O*N)+B}}return{format:k,formatPrefix:E}}var Zz,Z_e,J_e;UIt({thousands:",",grouping:[3],currency:["$",""]});function UIt(r){return Zz=VIt(r),Z_e=Zz.format,J_e=Zz.formatPrefix,Zz}function qIt(r){return Math.max(0,-BT(Math.abs(r)))}function YIt(r,i){return Math.max(0,Math.max(-8,Math.min(8,Math.floor(BT(i)/3)))*3-BT(Math.abs(r)))}function WIt(r,i){return r=Math.abs(r),i=Math.abs(i)-r,Math.max(0,BT(i)-BT(r))+1}function Dse(r,i){switch(arguments.length){case 0:break;case 1:this.range(r);break;default:this.range(i).domain(r);break}return this}const eTe=Symbol("implicit");function tTe(){var r=new Y9e,i=[],o=[],l=eTe;function f(b){let d=r.get(b);if(d===void 0){if(l!==eTe)return l;r.set(b,d=i.push(b)-1)}return o[d%o.length]}return f.domain=function(b){if(!arguments.length)return i.slice();i=[],r=new Y9e;for(const d of b)r.has(d)||r.set(d,i.push(d)-1);return f},f.range=function(b){return arguments.length?(o=Array.from(b),f):o.slice()},f.unknown=function(b){return arguments.length?(l=b,f):l},f.copy=function(){return tTe(i,o).unknown(l)},Dse.apply(f,arguments),f}function KIt(r){return function(){return r}}function XIt(r){return+r}var nTe=[0,1];function FT(r){return r}function Lse(r,i){return(i-=r=+r)?function(o){return(o-r)/i}:KIt(isNaN(i)?NaN:.5)}function QIt(r,i){var o;return r>i&&(o=r,r=i,i=o),function(l){return Math.max(r,Math.min(i,l))}}function ZIt(r,i,o){var l=r[0],f=r[1],b=i[0],d=i[1];return f<l?(l=Lse(f,l),b=o(d,b)):(l=Lse(l,f),b=o(b,d)),function(w){return b(l(w))}}function JIt(r,i,o){var l=Math.min(r.length,i.length)-1,f=new Array(l),b=new Array(l),d=-1;for(r[l]<r[0]&&(r=r.slice().reverse(),i=i.slice().reverse());++d<l;)f[d]=Lse(r[d],r[d+1]),b[d]=o(i[d],i[d+1]);return function(w){var y=YAt(r,w,1,l)-1;return b[y](f[y](w))}}function rTe(r,i){return i.domain(r.domain()).range(r.range()).interpolate(r.interpolate()).clamp(r.clamp()).unknown(r.unknown())}function eOt(){var r=nTe,i=nTe,o=vse,l,f,b,d=FT,w,y,k;function E(){var C=Math.min(r.length,i.length);return d!==FT&&(d=QIt(r[0],r[C-1])),w=C>2?JIt:ZIt,y=k=null,T}function T(C){return C==null||isNaN(C=+C)?b:(y||(y=w(r.map(l),i,o)))(l(d(C)))}return T.invert=function(C){return d(f((k||(k=w(i,r.map(l),_v)))(C)))},T.domain=function(C){return arguments.length?(r=Array.from(C,XIt),E()):r.slice()},T.range=function(C){return arguments.length?(i=Array.from(C),E()):i.slice()},T.rangeRound=function(C){return i=Array.from(C),o=gLt,E()},T.clamp=function(C){return arguments.length?(d=C?!0:FT,E()):d!==FT},T.interpolate=function(C){return arguments.length?(o=C,E()):o},T.unknown=function(C){return arguments.length?(b=C,T):b},function(C,S){return l=C,f=S,E()}}function iTe(){return eOt()(FT,FT)}function tOt(r,i,o,l){var f=ese(r,i,o),b;switch(l=Qz(l==null?",f":l),l.type){case"s":{var d=Math.max(Math.abs(r),Math.abs(i));return l.precision==null&&!isNaN(b=YIt(f,d))&&(l.precision=b),J_e(l,d)}case"":case"e":case"g":case"p":case"r":{l.precision==null&&!isNaN(b=WIt(f,Math.max(Math.abs(r),Math.abs(i))))&&(l.precision=b-(l.type==="e"));break}case"f":case"%":{l.precision==null&&!isNaN(b=qIt(f))&&(l.precision=b-(l.type==="%")*2);break}}return Z_e(l)}function nOt(r){var i=r.domain;return r.ticks=function(o){var l=i();return QAt(l[0],l[l.length-1],o==null?10:o)},r.tickFormat=function(o,l){var f=i();return tOt(f[0],f[f.length-1],o==null?10:o,l)},r.nice=function(o){o==null&&(o=10);var l=i(),f=0,b=l.length-1,d=l[f],w=l[b],y,k,E=10;for(w<d&&(k=d,d=w,w=k,k=f,f=b,b=k);E-- >0;){if(k=K9e(d,w,o),k===y)return l[f]=d,l[b]=w,i(l);if(k>0)d=Math.floor(d/k)*k,w=Math.ceil(w/k)*k;else if(k<0)d=Math.ceil(d*k)/k,w=Math.floor(w*k)/k;else break;y=k}return r},r}function sTe(){var r=iTe();return r.copy=function(){return rTe(r,sTe())},Dse.apply(r,arguments),nOt(r)}function rOt(r,i){r=r.slice();var o=0,l=r.length-1,f=r[o],b=r[l],d;return b<f&&(d=o,o=l,l=d,d=f,f=b,b=d),r[o]=i.floor(f),r[l]=i.ceil(b),r}var Ise=new Date,Ose=new Date;function Uf(r,i,o,l){function f(b){return r(b=arguments.length===0?new Date:new Date(+b)),b}return f.floor=function(b){return r(b=new Date(+b)),b},f.ceil=function(b){return r(b=new Date(b-1)),i(b,1),r(b),b},f.round=function(b){var d=f(b),w=f.ceil(b);return b-d<w-b?d:w},f.offset=function(b,d){return i(b=new Date(+b),d==null?1:Math.floor(d)),b},f.range=function(b,d,w){var y=[],k;if(b=f.ceil(b),w=w==null?1:Math.floor(w),!(b<d)||!(w>0))return y;do y.push(k=new Date(+b)),i(b,w),r(b);while(k<b&&b<d);return y},f.filter=function(b){return Uf(function(d){if(d>=d)for(;r(d),!b(d);)d.setTime(d-1)},function(d,w){if(d>=d)if(w<0)for(;++w<=0;)for(;i(d,-1),!b(d););else for(;--w>=0;)for(;i(d,1),!b(d););})},o&&(f.count=function(b,d){return Ise.setTime(+b),Ose.setTime(+d),r(Ise),r(Ose),Math.floor(o(Ise,Ose))},f.every=function(b){return b=Math.floor(b),!isFinite(b)||!(b>0)?null:b>1?f.filter(l?function(d){return l(d)%b===0}:function(d){return f.count(0,d)%b===0}):f}),f}var Jz=Uf(function(){},function(r,i){r.setTime(+r+i)},function(r,i){return i-r});Jz.every=function(r){return r=Math.floor(r),!isFinite(r)||!(r>0)?null:r>1?Uf(function(i){i.setTime(Math.floor(i/r)*r)},function(i,o){i.setTime(+i+o*r)},function(i,o){return(o-i)/r}):Jz};const iOt=Jz;Jz.range;const V3=1e3,D2=V3*60,U3=D2*60,j8=U3*24,Nse=j8*7,aTe=j8*30,Pse=j8*365;var oTe=Uf(function(r){r.setTime(r-r.getMilliseconds())},function(r,i){r.setTime(+r+i*V3)},function(r,i){return(i-r)/V3},function(r){return r.getUTCSeconds()});const YL=oTe;oTe.range;var cTe=Uf(function(r){r.setTime(r-r.getMilliseconds()-r.getSeconds()*V3)},function(r,i){r.setTime(+r+i*D2)},function(r,i){return(i-r)/D2},function(r){return r.getMinutes()});const eG=cTe;cTe.range;var uTe=Uf(function(r){r.setTime(r-r.getMilliseconds()-r.getSeconds()*V3-r.getMinutes()*D2)},function(r,i){r.setTime(+r+i*U3)},function(r,i){return(i-r)/U3},function(r){return r.getHours()});const tG=uTe;uTe.range;var lTe=Uf(r=>r.setHours(0,0,0,0),(r,i)=>r.setDate(r.getDate()+i),(r,i)=>(i-r-(i.getTimezoneOffset()-r.getTimezoneOffset())*D2)/j8,r=>r.getDate()-1);const RT=lTe;lTe.range;function $8(r){return Uf(function(i){i.setDate(i.getDate()-(i.getDay()+7-r)%7),i.setHours(0,0,0,0)},function(i,o){i.setDate(i.getDate()+o*7)},function(i,o){return(o-i-(o.getTimezoneOffset()-i.getTimezoneOffset())*D2)/Nse})}var jT=$8(0),nG=$8(1),sOt=$8(2),aOt=$8(3),$T=$8(4),oOt=$8(5),cOt=$8(6);jT.range,nG.range,sOt.range,aOt.range,$T.range,oOt.range,cOt.range;var hTe=Uf(function(r){r.setDate(1),r.setHours(0,0,0,0)},function(r,i){r.setMonth(r.getMonth()+i)},function(r,i){return i.getMonth()-r.getMonth()+(i.getFullYear()-r.getFullYear())*12},function(r){return r.getMonth()});const rG=hTe;hTe.range;var Bse=Uf(function(r){r.setMonth(0,1),r.setHours(0,0,0,0)},function(r,i){r.setFullYear(r.getFullYear()+i)},function(r,i){return i.getFullYear()-r.getFullYear()},function(r){return r.getFullYear()});Bse.every=function(r){return!isFinite(r=Math.floor(r))||!(r>0)?null:Uf(function(i){i.setFullYear(Math.floor(i.getFullYear()/r)*r),i.setMonth(0,1),i.setHours(0,0,0,0)},function(i,o){i.setFullYear(i.getFullYear()+o*r)})};const H8=Bse;Bse.range;var fTe=Uf(function(r){r.setUTCSeconds(0,0)},function(r,i){r.setTime(+r+i*D2)},function(r,i){return(i-r)/D2},function(r){return r.getUTCMinutes()});const uOt=fTe;fTe.range;var dTe=Uf(function(r){r.setUTCMinutes(0,0,0)},function(r,i){r.setTime(+r+i*U3)},function(r,i){return(i-r)/U3},function(r){return r.getUTCHours()});const lOt=dTe;dTe.range;var gTe=Uf(function(r){r.setUTCHours(0,0,0,0)},function(r,i){r.setUTCDate(r.getUTCDate()+i)},function(r,i){return(i-r)/j8},function(r){return r.getUTCDate()-1});const Fse=gTe;gTe.range;function z8(r){return Uf(function(i){i.setUTCDate(i.getUTCDate()-(i.getUTCDay()+7-r)%7),i.setUTCHours(0,0,0,0)},function(i,o){i.setUTCDate(i.getUTCDate()+o*7)},function(i,o){return(o-i)/Nse})}var Rse=z8(0),iG=z8(1),hOt=z8(2),fOt=z8(3),HT=z8(4),dOt=z8(5),gOt=z8(6);Rse.range,iG.range,hOt.range,fOt.range,HT.range,dOt.range,gOt.range;var pTe=Uf(function(r){r.setUTCDate(1),r.setUTCHours(0,0,0,0)},function(r,i){r.setUTCMonth(r.getUTCMonth()+i)},function(r,i){return i.getUTCMonth()-r.getUTCMonth()+(i.getUTCFullYear()-r.getUTCFullYear())*12},function(r){return r.getUTCMonth()});const pOt=pTe;pTe.range;var jse=Uf(function(r){r.setUTCMonth(0,1),r.setUTCHours(0,0,0,0)},function(r,i){r.setUTCFullYear(r.getUTCFullYear()+i)},function(r,i){return i.getUTCFullYear()-r.getUTCFullYear()},function(r){return r.getUTCFullYear()});jse.every=function(r){return!isFinite(r=Math.floor(r))||!(r>0)?null:Uf(function(i){i.setUTCFullYear(Math.floor(i.getUTCFullYear()/r)*r),i.setUTCMonth(0,1),i.setUTCHours(0,0,0,0)},function(i,o){i.setUTCFullYear(i.getUTCFullYear()+o*r)})};const zT=jse;jse.range;function bTe(r,i,o,l,f,b){const d=[[YL,1,V3],[YL,5,5*V3],[YL,15,15*V3],[YL,30,30*V3],[b,1,D2],[b,5,5*D2],[b,15,15*D2],[b,30,30*D2],[f,1,U3],[f,3,3*U3],[f,6,6*U3],[f,12,12*U3],[l,1,j8],[l,2,2*j8],[o,1,Nse],[i,1,aTe],[i,3,3*aTe],[r,1,Pse]];function w(k,E,T){const C=E<k;C&&([k,E]=[E,k]);const S=T&&typeof T.range=="function"?T:y(k,E,T),L=S?S.range(k,+E+1):[];return C?L.reverse():L}function y(k,E,T){const C=Math.abs(E-k)/T,S=Xie(([,,B])=>B).right(d,C);if(S===d.length)return r.every(ese(k/Pse,E/Pse,T));if(S===0)return iOt.every(Math.max(ese(k,E,T),1));const[L,O]=d[C/d[S-1][2]<d[S][2]/C?S-1:S];return L.every(O)}return[w,y]}bTe(zT,pOt,Rse,Fse,lOt,uOt);const[bOt,vOt]=bTe(H8,rG,jT,RT,tG,eG);function $se(r){if(0<=r.y&&r.y<100){var i=new Date(-1,r.m,r.d,r.H,r.M,r.S,r.L);return i.setFullYear(r.y),i}return new Date(r.y,r.m,r.d,r.H,r.M,r.S,r.L)}function Hse(r){if(0<=r.y&&r.y<100){var i=new Date(Date.UTC(-1,r.m,r.d,r.H,r.M,r.S,r.L));return i.setUTCFullYear(r.y),i}return new Date(Date.UTC(r.y,r.m,r.d,r.H,r.M,r.S,r.L))}function WL(r,i,o){return{y:r,m:i,d:o,H:0,M:0,S:0,L:0}}function wOt(r){var i=r.dateTime,o=r.date,l=r.time,f=r.periods,b=r.days,d=r.shortDays,w=r.months,y=r.shortMonths,k=KL(f),E=XL(f),T=KL(b),C=XL(b),S=KL(d),L=XL(d),O=KL(w),B=XL(w),N=KL(y),F=XL(y),R={a:de,A:re,b:ke,B:Ce,c:null,d:xTe,e:xTe,f:HOt,g:QOt,G:JOt,H:ROt,I:jOt,j:$Ot,L:ETe,m:zOt,M:GOt,p:_e,q:Te,Q:MTe,s:DTe,S:VOt,u:UOt,U:qOt,V:YOt,w:WOt,W:KOt,x:null,X:null,y:XOt,Y:ZOt,Z:eNt,"%":ATe},q={a:Be,A:Ge,b:Xe,B:Ee,c:null,d:TTe,e:TTe,f:iNt,g:gNt,G:bNt,H:tNt,I:nNt,j:rNt,L:CTe,m:sNt,M:aNt,p:Ze,q:Ie,Q:MTe,s:DTe,S:oNt,u:cNt,U:uNt,V:lNt,w:hNt,W:fNt,x:null,X:null,y:dNt,Y:pNt,Z:vNt,"%":ATe},X={a:W,A:Z,b:G,B:ae,c:$,d:yTe,e:yTe,f:NOt,g:mTe,G:wTe,H:kTe,I:kTe,j:DOt,L:OOt,m:MOt,M:LOt,p:z,q:AOt,Q:BOt,s:FOt,S:IOt,u:EOt,U:_Ot,V:TOt,w:xOt,W:COt,x:ge,X:ee,y:mTe,Y:wTe,Z:SOt,"%":POt};R.x=te(o,R),R.X=te(l,R),R.c=te(i,R),q.x=te(o,q),q.X=te(l,q),q.c=te(i,q);function te(Oe,Le){return function($e){var xe=[],Ae=-1,je=0,me=Oe.length,vt,ve,Zt;for($e instanceof Date||($e=new Date(+$e));++Ae<me;)Oe.charCodeAt(Ae)===37&&(xe.push(Oe.slice(je,Ae)),(ve=vTe[vt=Oe.charAt(++Ae)])!=null?vt=Oe.charAt(++Ae):ve=vt==="e"?" ":"0",(Zt=Le[vt])&&(vt=Zt($e,ve)),xe.push(vt),je=Ae+1);return xe.push(Oe.slice(je,Ae)),xe.join("")}}function H(Oe,Le){return function($e){var xe=WL(1900,void 0,1),Ae=Y(xe,Oe,$e+="",0),je,me;if(Ae!=$e.length)return null;if("Q"in xe)return new Date(xe.Q);if("s"in xe)return new Date(xe.s*1e3+("L"in xe?xe.L:0));if(Le&&!("Z"in xe)&&(xe.Z=0),"p"in xe&&(xe.H=xe.H%12+xe.p*12),xe.m===void 0&&(xe.m="q"in xe?xe.q:0),"V"in xe){if(xe.V<1||xe.V>53)return null;"w"in xe||(xe.w=1),"Z"in xe?(je=Hse(WL(xe.y,0,1)),me=je.getUTCDay(),je=me>4||me===0?iG.ceil(je):iG(je),je=Fse.offset(je,(xe.V-1)*7),xe.y=je.getUTCFullYear(),xe.m=je.getUTCMonth(),xe.d=je.getUTCDate()+(xe.w+6)%7):(je=$se(WL(xe.y,0,1)),me=je.getDay(),je=me>4||me===0?nG.ceil(je):nG(je),je=RT.offset(je,(xe.V-1)*7),xe.y=je.getFullYear(),xe.m=je.getMonth(),xe.d=je.getDate()+(xe.w+6)%7)}else("W"in xe||"U"in xe)&&("w"in xe||(xe.w="u"in xe?xe.u%7:"W"in xe?1:0),me="Z"in xe?Hse(WL(xe.y,0,1)).getUTCDay():$se(WL(xe.y,0,1)).getDay(),xe.m=0,xe.d="W"in xe?(xe.w+6)%7+xe.W*7-(me+5)%7:xe.w+xe.U*7-(me+6)%7);return"Z"in xe?(xe.H+=xe.Z/100|0,xe.M+=xe.Z%100,Hse(xe)):$se(xe)}}function Y(Oe,Le,$e,xe){for(var Ae=0,je=Le.length,me=$e.length,vt,ve;Ae<je;){if(xe>=me)return-1;if(vt=Le.charCodeAt(Ae++),vt===37){if(vt=Le.charAt(Ae++),ve=X[vt in vTe?Le.charAt(Ae++):vt],!ve||(xe=ve(Oe,$e,xe))<0)return-1}else if(vt!=$e.charCodeAt(xe++))return-1}return xe}function z(Oe,Le,$e){var xe=k.exec(Le.slice($e));return xe?(Oe.p=E.get(xe[0].toLowerCase()),$e+xe[0].length):-1}function W(Oe,Le,$e){var xe=S.exec(Le.slice($e));return xe?(Oe.w=L.get(xe[0].toLowerCase()),$e+xe[0].length):-1}function Z(Oe,Le,$e){var xe=T.exec(Le.slice($e));return xe?(Oe.w=C.get(xe[0].toLowerCase()),$e+xe[0].length):-1}function G(Oe,Le,$e){var xe=N.exec(Le.slice($e));return xe?(Oe.m=F.get(xe[0].toLowerCase()),$e+xe[0].length):-1}function ae(Oe,Le,$e){var xe=O.exec(Le.slice($e));return xe?(Oe.m=B.get(xe[0].toLowerCase()),$e+xe[0].length):-1}function $(Oe,Le,$e){return Y(Oe,i,Le,$e)}function ge(Oe,Le,$e){return Y(Oe,o,Le,$e)}function ee(Oe,Le,$e){return Y(Oe,l,Le,$e)}function de(Oe){return d[Oe.getDay()]}function re(Oe){return b[Oe.getDay()]}function ke(Oe){return y[Oe.getMonth()]}function Ce(Oe){return w[Oe.getMonth()]}function _e(Oe){return f[+(Oe.getHours()>=12)]}function Te(Oe){return 1+~~(Oe.getMonth()/3)}function Be(Oe){return d[Oe.getUTCDay()]}function Ge(Oe){return b[Oe.getUTCDay()]}function Xe(Oe){return y[Oe.getUTCMonth()]}function Ee(Oe){return w[Oe.getUTCMonth()]}function Ze(Oe){return f[+(Oe.getUTCHours()>=12)]}function Ie(Oe){return 1+~~(Oe.getUTCMonth()/3)}return{format:function(Oe){var Le=te(Oe+="",R);return Le.toString=function(){return Oe},Le},parse:function(Oe){var Le=H(Oe+="",!1);return Le.toString=function(){return Oe},Le},utcFormat:function(Oe){var Le=te(Oe+="",q);return Le.toString=function(){return Oe},Le},utcParse:function(Oe){var Le=H(Oe+="",!0);return Le.toString=function(){return Oe},Le}}}var vTe={"-":"",_:" ",0:"0"},qf=/^\s*\d+/,mOt=/^%/,yOt=/[\\^$*+?|[\]().{}]/g;function Lo(r,i,o){var l=r<0?"-":"",f=(l?-r:r)+"",b=f.length;return l+(b<o?new Array(o-b+1).join(i)+f:f)}function kOt(r){return r.replace(yOt,"\\$&")}function KL(r){return new RegExp("^(?:"+r.map(kOt).join("|")+")","i")}function XL(r){return new Map(r.map((i,o)=>[i.toLowerCase(),o]))}function xOt(r,i,o){var l=qf.exec(i.slice(o,o+1));return l?(r.w=+l[0],o+l[0].length):-1}function EOt(r,i,o){var l=qf.exec(i.slice(o,o+1));return l?(r.u=+l[0],o+l[0].length):-1}function _Ot(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.U=+l[0],o+l[0].length):-1}function TOt(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.V=+l[0],o+l[0].length):-1}function COt(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.W=+l[0],o+l[0].length):-1}function wTe(r,i,o){var l=qf.exec(i.slice(o,o+4));return l?(r.y=+l[0],o+l[0].length):-1}function mTe(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.y=+l[0]+(+l[0]>68?1900:2e3),o+l[0].length):-1}function SOt(r,i,o){var l=/^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(i.slice(o,o+6));return l?(r.Z=l[1]?0:-(l[2]+(l[3]||"00")),o+l[0].length):-1}function AOt(r,i,o){var l=qf.exec(i.slice(o,o+1));return l?(r.q=l[0]*3-3,o+l[0].length):-1}function MOt(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.m=l[0]-1,o+l[0].length):-1}function yTe(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.d=+l[0],o+l[0].length):-1}function DOt(r,i,o){var l=qf.exec(i.slice(o,o+3));return l?(r.m=0,r.d=+l[0],o+l[0].length):-1}function kTe(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.H=+l[0],o+l[0].length):-1}function LOt(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.M=+l[0],o+l[0].length):-1}function IOt(r,i,o){var l=qf.exec(i.slice(o,o+2));return l?(r.S=+l[0],o+l[0].length):-1}function OOt(r,i,o){var l=qf.exec(i.slice(o,o+3));return l?(r.L=+l[0],o+l[0].length):-1}function NOt(r,i,o){var l=qf.exec(i.slice(o,o+6));return l?(r.L=Math.floor(l[0]/1e3),o+l[0].length):-1}function POt(r,i,o){var l=mOt.exec(i.slice(o,o+1));return l?o+l[0].length:-1}function BOt(r,i,o){var l=qf.exec(i.slice(o));return l?(r.Q=+l[0],o+l[0].length):-1}function FOt(r,i,o){var l=qf.exec(i.slice(o));return l?(r.s=+l[0],o+l[0].length):-1}function xTe(r,i){return Lo(r.getDate(),i,2)}function ROt(r,i){return Lo(r.getHours(),i,2)}function jOt(r,i){return Lo(r.getHours()%12||12,i,2)}function $Ot(r,i){return Lo(1+RT.count(H8(r),r),i,3)}function ETe(r,i){return Lo(r.getMilliseconds(),i,3)}function HOt(r,i){return ETe(r,i)+"000"}function zOt(r,i){return Lo(r.getMonth()+1,i,2)}function GOt(r,i){return Lo(r.getMinutes(),i,2)}function VOt(r,i){return Lo(r.getSeconds(),i,2)}function UOt(r){var i=r.getDay();return i===0?7:i}function qOt(r,i){return Lo(jT.count(H8(r)-1,r),i,2)}function _Te(r){var i=r.getDay();return i>=4||i===0?$T(r):$T.ceil(r)}function YOt(r,i){return r=_Te(r),Lo($T.count(H8(r),r)+(H8(r).getDay()===4),i,2)}function WOt(r){return r.getDay()}function KOt(r,i){return Lo(nG.count(H8(r)-1,r),i,2)}function XOt(r,i){return Lo(r.getFullYear()%100,i,2)}function QOt(r,i){return r=_Te(r),Lo(r.getFullYear()%100,i,2)}function ZOt(r,i){return Lo(r.getFullYear()%1e4,i,4)}function JOt(r,i){var o=r.getDay();return r=o>=4||o===0?$T(r):$T.ceil(r),Lo(r.getFullYear()%1e4,i,4)}function eNt(r){var i=r.getTimezoneOffset();return(i>0?"-":(i*=-1,"+"))+Lo(i/60|0,"0",2)+Lo(i%60,"0",2)}function TTe(r,i){return Lo(r.getUTCDate(),i,2)}function tNt(r,i){return Lo(r.getUTCHours(),i,2)}function nNt(r,i){return Lo(r.getUTCHours()%12||12,i,2)}function rNt(r,i){return Lo(1+Fse.count(zT(r),r),i,3)}function CTe(r,i){return Lo(r.getUTCMilliseconds(),i,3)}function iNt(r,i){return CTe(r,i)+"000"}function sNt(r,i){return Lo(r.getUTCMonth()+1,i,2)}function aNt(r,i){return Lo(r.getUTCMinutes(),i,2)}function oNt(r,i){return Lo(r.getUTCSeconds(),i,2)}function cNt(r){var i=r.getUTCDay();return i===0?7:i}function uNt(r,i){return Lo(Rse.count(zT(r)-1,r),i,2)}function STe(r){var i=r.getUTCDay();return i>=4||i===0?HT(r):HT.ceil(r)}function lNt(r,i){return r=STe(r),Lo(HT.count(zT(r),r)+(zT(r).getUTCDay()===4),i,2)}function hNt(r){return r.getUTCDay()}function fNt(r,i){return Lo(iG.count(zT(r)-1,r),i,2)}function dNt(r,i){return Lo(r.getUTCFullYear()%100,i,2)}function gNt(r,i){return r=STe(r),Lo(r.getUTCFullYear()%100,i,2)}function pNt(r,i){return Lo(r.getUTCFullYear()%1e4,i,4)}function bNt(r,i){var o=r.getUTCDay();return r=o>=4||o===0?HT(r):HT.ceil(r),Lo(r.getUTCFullYear()%1e4,i,4)}function vNt(){return"+0000"}function ATe(){return"%"}function MTe(r){return+r}function DTe(r){return Math.floor(+r/1e3)}var GT,sG;wNt({dateTime:"%x, %X",date:"%-m/%-d/%Y",time:"%-I:%M:%S %p",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function wNt(r){return GT=wOt(r),sG=GT.format,GT.parse,GT.utcFormat,GT.utcParse,GT}function mNt(r){return new Date(r)}function yNt(r){return r instanceof Date?+r:+new Date(+r)}function LTe(r,i,o,l,f,b,d,w,y,k){var E=iTe(),T=E.invert,C=E.domain,S=k(".%L"),L=k(":%S"),O=k("%I:%M"),B=k("%I %p"),N=k("%a %d"),F=k("%b %d"),R=k("%B"),q=k("%Y");function X(te){return(y(te)<te?S:w(te)<te?L:d(te)<te?O:b(te)<te?B:l(te)<te?f(te)<te?N:F:o(te)<te?R:q)(te)}return E.invert=function(te){return new Date(T(te))},E.domain=function(te){return arguments.length?C(Array.from(te,yNt)):C().map(mNt)},E.ticks=function(te){var H=C();return r(H[0],H[H.length-1],te==null?10:te)},E.tickFormat=function(te,H){return H==null?X:k(H)},E.nice=function(te){var H=C();return(!te||typeof te.range!="function")&&(te=i(H[0],H[H.length-1],te==null?10:te)),te?C(rOt(H,te)):E},E.copy=function(){return rTe(E,LTe(r,i,o,l,f,b,d,w,y,k))},E}function kNt(){return Dse.apply(LTe(bOt,vOt,H8,rG,jT,RT,tG,eG,YL,sG).domain([new Date(2e3,0,1),new Date(2e3,0,2)]),arguments)}function sh(r){return function(){return r}}const ITe=Math.abs,ad=Math.atan2,G8=Math.cos,xNt=Math.max,zse=Math.min,Om=Math.sin,VT=Math.sqrt,od=1e-12,QL=Math.PI,aG=QL/2,oG=2*QL;function ENt(r){return r>1?0:r<-1?QL:Math.acos(r)}function OTe(r){return r>=1?aG:r<=-1?-aG:Math.asin(r)}function _Nt(r){return r.innerRadius}function TNt(r){return r.outerRadius}function CNt(r){return r.startAngle}function SNt(r){return r.endAngle}function ANt(r){return r&&r.padAngle}function MNt(r,i,o,l,f,b,d,w){var y=o-r,k=l-i,E=d-f,T=w-b,C=T*y-E*k;if(!(C*C<od))return C=(E*(i-b)-T*(r-f))/C,[r+C*y,i+C*k]}function cG(r,i,o,l,f,b,d){var w=r-o,y=i-l,k=(d?b:-b)/VT(w*w+y*y),E=k*y,T=-k*w,C=r+E,S=i+T,L=o+E,O=l+T,B=(C+L)/2,N=(S+O)/2,F=L-C,R=O-S,q=F*F+R*R,X=f-b,te=C*O-L*S,H=(R<0?-1:1)*VT(xNt(0,X*X*q-te*te)),Y=(te*R-F*H)/q,z=(-te*F-R*H)/q,W=(te*R+F*H)/q,Z=(-te*F+R*H)/q,G=Y-B,ae=z-N,$=W-B,ge=Z-N;return G*G+ae*ae>$*$+ge*ge&&(Y=W,z=Z),{cx:Y,cy:z,x01:-E,y01:-T,x11:Y*(f/X-1),y11:z*(f/X-1)}}function ZL(){var r=_Nt,i=TNt,o=sh(0),l=null,f=CNt,b=SNt,d=ANt,w=null;function y(){var k,E,T=+r.apply(this,arguments),C=+i.apply(this,arguments),S=f.apply(this,arguments)-aG,L=b.apply(this,arguments)-aG,O=ITe(L-S),B=L>S;if(w||(w=k=Ase()),C<T&&(E=C,C=T,T=E),!(C>od))w.moveTo(0,0);else if(O>oG-od)w.moveTo(C*G8(S),C*Om(S)),w.arc(0,0,C,S,L,!B),T>od&&(w.moveTo(T*G8(L),T*Om(L)),w.arc(0,0,T,L,S,B));else{var N=S,F=L,R=S,q=L,X=O,te=O,H=d.apply(this,arguments)/2,Y=H>od&&(l?+l.apply(this,arguments):VT(T*T+C*C)),z=zse(ITe(C-T)/2,+o.apply(this,arguments)),W=z,Z=z,G,ae;if(Y>od){var $=OTe(Y/T*Om(H)),ge=OTe(Y/C*Om(H));(X-=$*2)>od?($*=B?1:-1,R+=$,q-=$):(X=0,R=q=(S+L)/2),(te-=ge*2)>od?(ge*=B?1:-1,N+=ge,F-=ge):(te=0,N=F=(S+L)/2)}var ee=C*G8(N),de=C*Om(N),re=T*G8(q),ke=T*Om(q);if(z>od){var Ce=C*G8(F),_e=C*Om(F),Te=T*G8(R),Be=T*Om(R),Ge;if(O<QL&&(Ge=MNt(ee,de,Te,Be,Ce,_e,re,ke))){var Xe=ee-Ge[0],Ee=de-Ge[1],Ze=Ce-Ge[0],Ie=_e-Ge[1],Oe=1/Om(ENt((Xe*Ze+Ee*Ie)/(VT(Xe*Xe+Ee*Ee)*VT(Ze*Ze+Ie*Ie)))/2),Le=VT(Ge[0]*Ge[0]+Ge[1]*Ge[1]);W=zse(z,(T-Le)/(Oe-1)),Z=zse(z,(C-Le)/(Oe+1))}}te>od?Z>od?(G=cG(Te,Be,ee,de,C,Z,B),ae=cG(Ce,_e,re,ke,C,Z,B),w.moveTo(G.cx+G.x01,G.cy+G.y01),Z<z?w.arc(G.cx,G.cy,Z,ad(G.y01,G.x01),ad(ae.y01,ae.x01),!B):(w.arc(G.cx,G.cy,Z,ad(G.y01,G.x01),ad(G.y11,G.x11),!B),w.arc(0,0,C,ad(G.cy+G.y11,G.cx+G.x11),ad(ae.cy+ae.y11,ae.cx+ae.x11),!B),w.arc(ae.cx,ae.cy,Z,ad(ae.y11,ae.x11),ad(ae.y01,ae.x01),!B))):(w.moveTo(ee,de),w.arc(0,0,C,N,F,!B)):w.moveTo(ee,de),!(T>od)||!(X>od)?w.lineTo(re,ke):W>od?(G=cG(re,ke,Ce,_e,T,-W,B),ae=cG(ee,de,Te,Be,T,-W,B),w.lineTo(G.cx+G.x01,G.cy+G.y01),W<z?w.arc(G.cx,G.cy,W,ad(G.y01,G.x01),ad(ae.y01,ae.x01),!B):(w.arc(G.cx,G.cy,W,ad(G.y01,G.x01),ad(G.y11,G.x11),!B),w.arc(0,0,T,ad(G.cy+G.y11,G.cx+G.x11),ad(ae.cy+ae.y11,ae.cx+ae.x11),B),w.arc(ae.cx,ae.cy,W,ad(ae.y11,ae.x11),ad(ae.y01,ae.x01),!B))):w.arc(0,0,T,q,R,B)}if(w.closePath(),k)return w=null,k+""||null}return y.centroid=function(){var k=(+r.apply(this,arguments)+ +i.apply(this,arguments))/2,E=(+f.apply(this,arguments)+ +b.apply(this,arguments))/2-QL/2;return[G8(E)*k,Om(E)*k]},y.innerRadius=function(k){return arguments.length?(r=typeof k=="function"?k:sh(+k),y):r},y.outerRadius=function(k){return arguments.length?(i=typeof k=="function"?k:sh(+k),y):i},y.cornerRadius=function(k){return arguments.length?(o=typeof k=="function"?k:sh(+k),y):o},y.padRadius=function(k){return arguments.length?(l=k==null?null:typeof k=="function"?k:sh(+k),y):l},y.startAngle=function(k){return arguments.length?(f=typeof k=="function"?k:sh(+k),y):f},y.endAngle=function(k){return arguments.length?(b=typeof k=="function"?k:sh(+k),y):b},y.padAngle=function(k){return arguments.length?(d=typeof k=="function"?k:sh(+k),y):d},y.context=function(k){return arguments.length?(w=k==null?null:k,y):w},y}function NTe(r){return typeof r=="object"&&"length"in r?r:Array.from(r)}function PTe(r){this._context=r}PTe.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1,this._line?this._context.lineTo(r,i):this._context.moveTo(r,i);break;case 1:this._point=2;default:this._context.lineTo(r,i);break}}};function x1(r){return new PTe(r)}function DNt(r){return r[0]}function LNt(r){return r[1]}function V8(r,i){var o=sh(!0),l=null,f=x1,b=null;r=typeof r=="function"?r:r===void 0?DNt:sh(r),i=typeof i=="function"?i:i===void 0?LNt:sh(i);function d(w){var y,k=(w=NTe(w)).length,E,T=!1,C;for(l==null&&(b=f(C=Ase())),y=0;y<=k;++y)!(y<k&&o(E=w[y],y,w))===T&&((T=!T)?b.lineStart():b.lineEnd()),T&&b.point(+r(E,y,w),+i(E,y,w));if(C)return b=null,C+""||null}return d.x=function(w){return arguments.length?(r=typeof w=="function"?w:sh(+w),d):r},d.y=function(w){return arguments.length?(i=typeof w=="function"?w:sh(+w),d):i},d.defined=function(w){return arguments.length?(o=typeof w=="function"?w:sh(!!w),d):o},d.curve=function(w){return arguments.length?(f=w,l!=null&&(b=f(l)),d):f},d.context=function(w){return arguments.length?(w==null?l=b=null:b=f(l=w),d):l},d}function INt(r,i){return i<r?-1:i>r?1:i>=r?0:NaN}function ONt(r){return r}function NNt(){var r=ONt,i=INt,o=null,l=sh(0),f=sh(oG),b=sh(0);function d(w){var y,k=(w=NTe(w)).length,E,T,C=0,S=new Array(k),L=new Array(k),O=+l.apply(this,arguments),B=Math.min(oG,Math.max(-oG,f.apply(this,arguments)-O)),N,F=Math.min(Math.abs(B)/k,b.apply(this,arguments)),R=F*(B<0?-1:1),q;for(y=0;y<k;++y)(q=L[S[y]=y]=+r(w[y],y,w))>0&&(C+=q);for(i!=null?S.sort(function(X,te){return i(L[X],L[te])}):o!=null&&S.sort(function(X,te){return o(w[X],w[te])}),y=0,T=C?(B-k*R)/C:0;y<k;++y,O=N)E=S[y],q=L[E],N=O+(q>0?q*T:0)+R,L[E]={data:w[E],index:y,value:q,startAngle:O,endAngle:N,padAngle:F};return L}return d.value=function(w){return arguments.length?(r=typeof w=="function"?w:sh(+w),d):r},d.sortValues=function(w){return arguments.length?(i=w,o=null,d):i},d.sort=function(w){return arguments.length?(o=w,i=null,d):o},d.startAngle=function(w){return arguments.length?(l=typeof w=="function"?w:sh(+w),d):l},d.endAngle=function(w){return arguments.length?(f=typeof w=="function"?w:sh(+w),d):f},d.padAngle=function(w){return arguments.length?(b=typeof w=="function"?w:sh(+w),d):b},d}class BTe{constructor(i,o){this._context=i,this._x=o}areaStart(){this._line=0}areaEnd(){this._line=NaN}lineStart(){this._point=0}lineEnd(){(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line}point(i,o){switch(i=+i,o=+o,this._point){case 0:{this._point=1,this._line?this._context.lineTo(i,o):this._context.moveTo(i,o);break}case 1:this._point=2;default:{this._x?this._context.bezierCurveTo(this._x0=(this._x0+i)/2,this._y0,this._x0,o,i,o):this._context.bezierCurveTo(this._x0,this._y0=(this._y0+o)/2,i,this._y0,i,o);break}}this._x0=i,this._y0=o}}function PNt(r){return new BTe(r,!0)}function BNt(r){return new BTe(r,!1)}function s6(){}function uG(r,i,o){r._context.bezierCurveTo((2*r._x0+r._x1)/3,(2*r._y0+r._y1)/3,(r._x0+2*r._x1)/3,(r._y0+2*r._y1)/3,(r._x0+4*r._x1+i)/6,(r._y0+4*r._y1+o)/6)}function lG(r){this._context=r}lG.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){switch(this._point){case 3:uG(this,this._x1,this._y1);case 2:this._context.lineTo(this._x1,this._y1);break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1,this._line?this._context.lineTo(r,i):this._context.moveTo(r,i);break;case 1:this._point=2;break;case 2:this._point=3,this._context.lineTo((5*this._x0+this._x1)/6,(5*this._y0+this._y1)/6);default:uG(this,r,i);break}this._x0=this._x1,this._x1=r,this._y0=this._y1,this._y1=i}};function UT(r){return new lG(r)}function FTe(r){this._context=r}FTe.prototype={areaStart:s6,areaEnd:s6,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._y0=this._y1=this._y2=this._y3=this._y4=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:{this._context.moveTo(this._x2,this._y2),this._context.closePath();break}case 2:{this._context.moveTo((this._x2+2*this._x3)/3,(this._y2+2*this._y3)/3),this._context.lineTo((this._x3+2*this._x2)/3,(this._y3+2*this._y2)/3),this._context.closePath();break}case 3:{this.point(this._x2,this._y2),this.point(this._x3,this._y3),this.point(this._x4,this._y4);break}}},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1,this._x2=r,this._y2=i;break;case 1:this._point=2,this._x3=r,this._y3=i;break;case 2:this._point=3,this._x4=r,this._y4=i,this._context.moveTo((this._x0+4*this._x1+r)/6,(this._y0+4*this._y1+i)/6);break;default:uG(this,r,i);break}this._x0=this._x1,this._x1=r,this._y0=this._y1,this._y1=i}};function FNt(r){return new FTe(r)}function RTe(r){this._context=r}RTe.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===3)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3;var o=(this._x0+4*this._x1+r)/6,l=(this._y0+4*this._y1+i)/6;this._line?this._context.lineTo(o,l):this._context.moveTo(o,l);break;case 3:this._point=4;default:uG(this,r,i);break}this._x0=this._x1,this._x1=r,this._y0=this._y1,this._y1=i}};function RNt(r){return new RTe(r)}function jTe(r,i){this._basis=new lG(r),this._beta=i}jTe.prototype={lineStart:function(){this._x=[],this._y=[],this._basis.lineStart()},lineEnd:function(){var r=this._x,i=this._y,o=r.length-1;if(o>0)for(var l=r[0],f=i[0],b=r[o]-l,d=i[o]-f,w=-1,y;++w<=o;)y=w/o,this._basis.point(this._beta*r[w]+(1-this._beta)*(l+y*b),this._beta*i[w]+(1-this._beta)*(f+y*d));this._x=this._y=null,this._basis.lineEnd()},point:function(r,i){this._x.push(+r),this._y.push(+i)}};const jNt=function r(i){function o(l){return i===1?new lG(l):new jTe(l,i)}return o.beta=function(l){return r(+l)},o}(.85);function hG(r,i,o){r._context.bezierCurveTo(r._x1+r._k*(r._x2-r._x0),r._y1+r._k*(r._y2-r._y0),r._x2+r._k*(r._x1-i),r._y2+r._k*(r._y1-o),r._x2,r._y2)}function Gse(r,i){this._context=r,this._k=(1-i)/6}Gse.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:hG(this,this._x1,this._y1);break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1,this._line?this._context.lineTo(r,i):this._context.moveTo(r,i);break;case 1:this._point=2,this._x1=r,this._y1=i;break;case 2:this._point=3;default:hG(this,r,i);break}this._x0=this._x1,this._x1=this._x2,this._x2=r,this._y0=this._y1,this._y1=this._y2,this._y2=i}};const $Nt=function r(i){function o(l){return new Gse(l,i)}return o.tension=function(l){return r(+l)},o}(0);function Vse(r,i){this._context=r,this._k=(1-i)/6}Vse.prototype={areaStart:s6,areaEnd:s6,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:{this._context.moveTo(this._x3,this._y3),this._context.closePath();break}case 2:{this._context.lineTo(this._x3,this._y3),this._context.closePath();break}case 3:{this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5);break}}},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1,this._x3=r,this._y3=i;break;case 1:this._point=2,this._context.moveTo(this._x4=r,this._y4=i);break;case 2:this._point=3,this._x5=r,this._y5=i;break;default:hG(this,r,i);break}this._x0=this._x1,this._x1=this._x2,this._x2=r,this._y0=this._y1,this._y1=this._y2,this._y2=i}};const HNt=function r(i){function o(l){return new Vse(l,i)}return o.tension=function(l){return r(+l)},o}(0);function Use(r,i){this._context=r,this._k=(1-i)/6}Use.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===3)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:hG(this,r,i);break}this._x0=this._x1,this._x1=this._x2,this._x2=r,this._y0=this._y1,this._y1=this._y2,this._y2=i}};const zNt=function r(i){function o(l){return new Use(l,i)}return o.tension=function(l){return r(+l)},o}(0);function qse(r,i,o){var l=r._x1,f=r._y1,b=r._x2,d=r._y2;if(r._l01_a>od){var w=2*r._l01_2a+3*r._l01_a*r._l12_a+r._l12_2a,y=3*r._l01_a*(r._l01_a+r._l12_a);l=(l*w-r._x0*r._l12_2a+r._x2*r._l01_2a)/y,f=(f*w-r._y0*r._l12_2a+r._y2*r._l01_2a)/y}if(r._l23_a>od){var k=2*r._l23_2a+3*r._l23_a*r._l12_a+r._l12_2a,E=3*r._l23_a*(r._l23_a+r._l12_a);b=(b*k+r._x1*r._l23_2a-i*r._l12_2a)/E,d=(d*k+r._y1*r._l23_2a-o*r._l12_2a)/E}r._context.bezierCurveTo(l,f,b,d,r._x2,r._y2)}function $Te(r,i){this._context=r,this._alpha=i}$Te.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:this.point(this._x2,this._y2);break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){if(r=+r,i=+i,this._point){var o=this._x2-r,l=this._y2-i;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(o*o+l*l,this._alpha))}switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(r,i):this._context.moveTo(r,i);break;case 1:this._point=2;break;case 2:this._point=3;default:qse(this,r,i);break}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=r,this._y0=this._y1,this._y1=this._y2,this._y2=i}};const GNt=function r(i){function o(l){return i?new $Te(l,i):new Gse(l,0)}return o.alpha=function(l){return r(+l)},o}(.5);function HTe(r,i){this._context=r,this._alpha=i}HTe.prototype={areaStart:s6,areaEnd:s6,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 1:{this._context.moveTo(this._x3,this._y3),this._context.closePath();break}case 2:{this._context.lineTo(this._x3,this._y3),this._context.closePath();break}case 3:{this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5);break}}},point:function(r,i){if(r=+r,i=+i,this._point){var o=this._x2-r,l=this._y2-i;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(o*o+l*l,this._alpha))}switch(this._point){case 0:this._point=1,this._x3=r,this._y3=i;break;case 1:this._point=2,this._context.moveTo(this._x4=r,this._y4=i);break;case 2:this._point=3,this._x5=r,this._y5=i;break;default:qse(this,r,i);break}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=r,this._y0=this._y1,this._y1=this._y2,this._y2=i}};const VNt=function r(i){function o(l){return i?new HTe(l,i):new Vse(l,0)}return o.alpha=function(l){return r(+l)},o}(.5);function zTe(r,i){this._context=r,this._alpha=i}zTe.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===3)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){if(r=+r,i=+i,this._point){var o=this._x2-r,l=this._y2-i;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(o*o+l*l,this._alpha))}switch(this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:qse(this,r,i);break}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=r,this._y0=this._y1,this._y1=this._y2,this._y2=i}};const UNt=function r(i){function o(l){return i?new zTe(l,i):new Use(l,0)}return o.alpha=function(l){return r(+l)},o}(.5);function GTe(r){this._context=r}GTe.prototype={areaStart:s6,areaEnd:s6,lineStart:function(){this._point=0},lineEnd:function(){this._point&&this._context.closePath()},point:function(r,i){r=+r,i=+i,this._point?this._context.lineTo(r,i):(this._point=1,this._context.moveTo(r,i))}};function qNt(r){return new GTe(r)}function VTe(r){return r<0?-1:1}function UTe(r,i,o){var l=r._x1-r._x0,f=i-r._x1,b=(r._y1-r._y0)/(l||f<0&&-0),d=(o-r._y1)/(f||l<0&&-0),w=(b*f+d*l)/(l+f);return(VTe(b)+VTe(d))*Math.min(Math.abs(b),Math.abs(d),.5*Math.abs(w))||0}function qTe(r,i){var o=r._x1-r._x0;return o?(3*(r._y1-r._y0)/o-i)/2:i}function Yse(r,i,o){var l=r._x0,f=r._y0,b=r._x1,d=r._y1,w=(b-l)/3;r._context.bezierCurveTo(l+w,f+w*i,b-w,d-w*o,b,d)}function fG(r){this._context=r}fG.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=this._t0=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x1,this._y1);break;case 3:Yse(this,this._t0,qTe(this,this._t0));break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(r,i){var o=NaN;if(r=+r,i=+i,!(r===this._x1&&i===this._y1)){switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(r,i):this._context.moveTo(r,i);break;case 1:this._point=2;break;case 2:this._point=3,Yse(this,qTe(this,o=UTe(this,r,i)),o);break;default:Yse(this,this._t0,o=UTe(this,r,i));break}this._x0=this._x1,this._x1=r,this._y0=this._y1,this._y1=i,this._t0=o}}};function YTe(r){this._context=new WTe(r)}(YTe.prototype=Object.create(fG.prototype)).point=function(r,i){fG.prototype.point.call(this,i,r)};function WTe(r){this._context=r}WTe.prototype={moveTo:function(r,i){this._context.moveTo(i,r)},closePath:function(){this._context.closePath()},lineTo:function(r,i){this._context.lineTo(i,r)},bezierCurveTo:function(r,i,o,l,f,b){this._context.bezierCurveTo(i,r,l,o,b,f)}};function YNt(r){return new fG(r)}function WNt(r){return new YTe(r)}function KTe(r){this._context=r}KTe.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=[],this._y=[]},lineEnd:function(){var r=this._x,i=this._y,o=r.length;if(o)if(this._line?this._context.lineTo(r[0],i[0]):this._context.moveTo(r[0],i[0]),o===2)this._context.lineTo(r[1],i[1]);else for(var l=XTe(r),f=XTe(i),b=0,d=1;d<o;++b,++d)this._context.bezierCurveTo(l[0][b],f[0][b],l[1][b],f[1][b],r[d],i[d]);(this._line||this._line!==0&&o===1)&&this._context.closePath(),this._line=1-this._line,this._x=this._y=null},point:function(r,i){this._x.push(+r),this._y.push(+i)}};function XTe(r){var i,o=r.length-1,l,f=new Array(o),b=new Array(o),d=new Array(o);for(f[0]=0,b[0]=2,d[0]=r[0]+2*r[1],i=1;i<o-1;++i)f[i]=1,b[i]=4,d[i]=4*r[i]+2*r[i+1];for(f[o-1]=2,b[o-1]=7,d[o-1]=8*r[o-1]+r[o],i=1;i<o;++i)l=f[i]/b[i-1],b[i]-=l,d[i]-=l*d[i-1];for(f[o-1]=d[o-1]/b[o-1],i=o-2;i>=0;--i)f[i]=(d[i]-f[i+1])/b[i];for(b[o-1]=(r[o]+f[o-1])/2,i=0;i<o-1;++i)b[i]=2*r[i+1]-f[i+1];return[f,b]}function KNt(r){return new KTe(r)}function dG(r,i){this._context=r,this._t=i}dG.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=this._y=NaN,this._point=0},lineEnd:function(){0<this._t&&this._t<1&&this._point===2&&this._context.lineTo(this._x,this._y),(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line>=0&&(this._t=1-this._t,this._line=1-this._line)},point:function(r,i){switch(r=+r,i=+i,this._point){case 0:this._point=1,this._line?this._context.lineTo(r,i):this._context.moveTo(r,i);break;case 1:this._point=2;default:{if(this._t<=0)this._context.lineTo(this._x,i),this._context.lineTo(r,i);else{var o=this._x*(1-this._t)+r*this._t;this._context.lineTo(o,this._y),this._context.lineTo(o,i)}break}}this._x=r,this._y=i}};function XNt(r){return new dG(r,.5)}function QNt(r){return new dG(r,0)}function ZNt(r){return new dG(r,1)}function qT(r,i,o){this.k=r,this.x=i,this.y=o}qT.prototype={constructor:qT,scale:function(r){return r===1?this:new qT(this.k*r,this.x,this.y)},translate:function(r,i){return r===0&i===0?this:new qT(this.k,this.x+this.k*r,this.y+this.k*i)},apply:function(r){return[r[0]*this.k+this.x,r[1]*this.k+this.y]},applyX:function(r){return r*this.k+this.x},applyY:function(r){return r*this.k+this.y},invert:function(r){return[(r[0]-this.x)/this.k,(r[1]-this.y)/this.k]},invertX:function(r){return(r-this.x)/this.k},invertY:function(r){return(r-this.y)/this.k},rescaleX:function(r){return r.copy().domain(r.range().map(this.invertX,this).map(r.invert,r))},rescaleY:function(r){return r.copy().domain(r.range().map(this.invertY,this).map(r.invert,r))},toString:function(){return"translate("+this.x+","+this.y+") scale("+this.k+")"}},new qT(1,0,0),qT.prototype;/*! @license DOMPurify 2.4.3 | (c) Cure53 and other contributors | Released under the Apache license 2.0 and Mozilla Public License 2.0 | github.com/cure53/DOMPurify/blob/2.4.3/LICENSE */function a6(r){return a6=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(i){return typeof i}:function(i){return i&&typeof Symbol=="function"&&i.constructor===Symbol&&i!==Symbol.prototype?"symbol":typeof i},a6(r)}function Wse(r,i){return Wse=Object.setPrototypeOf||function(l,f){return l.__proto__=f,l},Wse(r,i)}function JNt(){if(typeof Reflect>"u"||!Reflect.construct||Reflect.construct.sham)return!1;if(typeof Proxy=="function")return!0;try{return Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){})),!0}catch{return!1}}function gG(r,i,o){return JNt()?gG=Reflect.construct:gG=function(f,b,d){var w=[null];w.push.apply(w,b);var y=Function.bind.apply(f,w),k=new y;return d&&Wse(k,d.prototype),k},gG.apply(null,arguments)}function Cv(r){return ePt(r)||tPt(r)||nPt(r)||rPt()}function ePt(r){if(Array.isArray(r))return Kse(r)}function tPt(r){if(typeof Symbol<"u"&&r[Symbol.iterator]!=null||r["@@iterator"]!=null)return Array.from(r)}function nPt(r,i){if(!!r){if(typeof r=="string")return Kse(r,i);var o=Object.prototype.toString.call(r).slice(8,-1);if(o==="Object"&&r.constructor&&(o=r.constructor.name),o==="Map"||o==="Set")return Array.from(r);if(o==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(o))return Kse(r,i)}}function Kse(r,i){(i==null||i>r.length)&&(i=r.length);for(var o=0,l=new Array(i);o<i;o++)l[o]=r[o];return l}function rPt(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}var iPt=Object.hasOwnProperty,QTe=Object.setPrototypeOf,sPt=Object.isFrozen,aPt=Object.getPrototypeOf,oPt=Object.getOwnPropertyDescriptor,Wd=Object.freeze,Sv=Object.seal,cPt=Object.create,ZTe=typeof Reflect<"u"&&Reflect,pG=ZTe.apply,Xse=ZTe.construct;pG||(pG=function(i,o,l){return i.apply(o,l)}),Wd||(Wd=function(i){return i}),Sv||(Sv=function(i){return i}),Xse||(Xse=function(i,o){return gG(i,Cv(o))});var uPt=L2(Array.prototype.forEach),JTe=L2(Array.prototype.pop),JL=L2(Array.prototype.push),bG=L2(String.prototype.toLowerCase),Qse=L2(String.prototype.toString),lPt=L2(String.prototype.match),Av=L2(String.prototype.replace),hPt=L2(String.prototype.indexOf),fPt=L2(String.prototype.trim),Kd=L2(RegExp.prototype.test),Zse=dPt(TypeError);function L2(r){return function(i){for(var o=arguments.length,l=new Array(o>1?o-1:0),f=1;f<o;f++)l[f-1]=arguments[f];return pG(r,i,l)}}function dPt(r){return function(){for(var i=arguments.length,o=new Array(i),l=0;l<i;l++)o[l]=arguments[l];return Xse(r,o)}}function xa(r,i,o){o=o||bG,QTe&&QTe(r,null);for(var l=i.length;l--;){var f=i[l];if(typeof f=="string"){var b=o(f);b!==f&&(sPt(i)||(i[l]=b),f=b)}r[f]=!0}return r}function U8(r){var i=cPt(null),o;for(o in r)pG(iPt,r,[o])===!0&&(i[o]=r[o]);return i}function vG(r,i){for(;r!==null;){var o=oPt(r,i);if(o){if(o.get)return L2(o.get);if(typeof o.value=="function")return L2(o.value)}r=aPt(r)}function l(f){return console.warn("fallback value for",f),null}return l}var eCe=Wd(["a","abbr","acronym","address","area","article","aside","audio","b","bdi","bdo","big","blink","blockquote","body","br","button","canvas","caption","center","cite","code","col","colgroup","content","data","datalist","dd","decorator","del","details","dfn","dialog","dir","div","dl","dt","element","em","fieldset","figcaption","figure","font","footer","form","h1","h2","h3","h4","h5","h6","head","header","hgroup","hr","html","i","img","input","ins","kbd","label","legend","li","main","map","mark","marquee","menu","menuitem","meter","nav","nobr","ol","optgroup","option","output","p","picture","pre","progress","q","rp","rt","ruby","s","samp","section","select","shadow","small","source","spacer","span","strike","strong","style","sub","summary","sup","table","tbody","td","template","textarea","tfoot","th","thead","time","tr","track","tt","u","ul","var","video","wbr"]),Jse=Wd(["svg","a","altglyph","altglyphdef","altglyphitem","animatecolor","animatemotion","animatetransform","circle","clippath","defs","desc","ellipse","filter","font","g","glyph","glyphref","hkern","image","line","lineargradient","marker","mask","metadata","mpath","path","pattern","polygon","polyline","radialgradient","rect","stop","style","switch","symbol","text","textpath","title","tref","tspan","view","vkern"]),eae=Wd(["feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence"]),gPt=Wd(["animate","color-profile","cursor","discard","fedropshadow","font-face","font-face-format","font-face-name","font-face-src","font-face-uri","foreignobject","hatch","hatchpath","mesh","meshgradient","meshpatch","meshrow","missing-glyph","script","set","solidcolor","unknown","use"]),tae=Wd(["math","menclose","merror","mfenced","mfrac","mglyph","mi","mlabeledtr","mmultiscripts","mn","mo","mover","mpadded","mphantom","mroot","mrow","ms","mspace","msqrt","mstyle","msub","msup","msubsup","mtable","mtd","mtext","mtr","munder","munderover"]),pPt=Wd(["maction","maligngroup","malignmark","mlongdiv","mscarries","mscarry","msgroup","mstack","msline","msrow","semantics","annotation","annotation-xml","mprescripts","none"]),tCe=Wd(["#text"]),nCe=Wd(["accept","action","align","alt","autocapitalize","autocomplete","autopictureinpicture","autoplay","background","bgcolor","border","capture","cellpadding","cellspacing","checked","cite","class","clear","color","cols","colspan","controls","controlslist","coords","crossorigin","datetime","decoding","default","dir","disabled","disablepictureinpicture","disableremoteplayback","download","draggable","enctype","enterkeyhint","face","for","headers","height","hidden","high","href","hreflang","id","inputmode","integrity","ismap","kind","label","lang","list","loading","loop","low","max","maxlength","media","method","min","minlength","multiple","muted","name","nonce","noshade","novalidate","nowrap","open","optimum","pattern","placeholder","playsinline","poster","preload","pubdate","radiogroup","readonly","rel","required","rev","reversed","role","rows","rowspan","spellcheck","scope","selected","shape","size","sizes","span","srclang","start","src","srcset","step","style","summary","tabindex","title","translate","type","usemap","valign","value","width","xmlns","slot"]),nae=Wd(["accent-height","accumulate","additive","alignment-baseline","ascent","attributename","attributetype","azimuth","basefrequency","baseline-shift","begin","bias","by","class","clip","clippathunits","clip-path","clip-rule","color","color-interpolation","color-interpolation-filters","color-profile","color-rendering","cx","cy","d","dx","dy","diffuseconstant","direction","display","divisor","dur","edgemode","elevation","end","fill","fill-opacity","fill-rule","filter","filterunits","flood-color","flood-opacity","font-family","font-size","font-size-adjust","font-stretch","font-style","font-variant","font-weight","fx","fy","g1","g2","glyph-name","glyphref","gradientunits","gradienttransform","height","href","id","image-rendering","in","in2","k","k1","k2","k3","k4","kerning","keypoints","keysplines","keytimes","lang","lengthadjust","letter-spacing","kernelmatrix","kernelunitlength","lighting-color","local","marker-end","marker-mid","marker-start","markerheight","markerunits","markerwidth","maskcontentunits","maskunits","max","mask","media","method","mode","min","name","numoctaves","offset","operator","opacity","order","orient","orientation","origin","overflow","paint-order","path","pathlength","patterncontentunits","patterntransform","patternunits","points","preservealpha","preserveaspectratio","primitiveunits","r","rx","ry","radius","refx","refy","repeatcount","repeatdur","restart","result","rotate","scale","seed","shape-rendering","specularconstant","specularexponent","spreadmethod","startoffset","stddeviation","stitchtiles","stop-color","stop-opacity","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke","stroke-width","style","surfacescale","systemlanguage","tabindex","targetx","targety","transform","transform-origin","text-anchor","text-decoration","text-rendering","textlength","type","u1","u2","unicode","values","viewbox","visibility","version","vert-adv-y","vert-origin-x","vert-origin-y","width","word-spacing","wrap","writing-mode","xchannelselector","ychannelselector","x","x1","x2","xmlns","y","y1","y2","z","zoomandpan"]),rCe=Wd(["accent","accentunder","align","bevelled","close","columnsalign","columnlines","columnspan","denomalign","depth","dir","display","displaystyle","encoding","fence","frame","height","href","id","largeop","length","linethickness","lspace","lquote","mathbackground","mathcolor","mathsize","mathvariant","maxsize","minsize","movablelimits","notation","numalign","open","rowalign","rowlines","rowspacing","rowspan","rspace","rquote","scriptlevel","scriptminsize","scriptsizemultiplier","selection","separator","separators","stretchy","subscriptshift","supscriptshift","symmetric","voffset","width","xmlns"]),wG=Wd(["xlink:href","xml:id","xlink:title","xml:space","xmlns:xlink"]),bPt=Sv(/\{\{[\w\W]*|[\w\W]*\}\}/gm),vPt=Sv(/<%[\w\W]*|[\w\W]*%>/gm),wPt=Sv(/\${[\w\W]*}/gm),mPt=Sv(/^data-[\-\w.\u00B7-\uFFFF]/),yPt=Sv(/^aria-[\-\w]+$/),kPt=Sv(/^(?:(?:(?:f|ht)tps?|mailto|tel|callto|cid|xmpp):|[^a-z]|[a-z+.\-]+(?:[^a-z+.\-:]|$))/i),xPt=Sv(/^(?:\w+script|data):/i),EPt=Sv(/[\u0000-\u0020\u00A0\u1680\u180E\u2000-\u2029\u205F\u3000]/g),_Pt=Sv(/^html$/i),TPt=function(){return typeof window>"u"?null:window},CPt=function(i,o){if(a6(i)!=="object"||typeof i.createPolicy!="function")return null;var l=null,f="data-tt-policy-suffix";o.currentScript&&o.currentScript.hasAttribute(f)&&(l=o.currentScript.getAttribute(f));var b="dompurify"+(l?"#"+l:"");try{return i.createPolicy(b,{createHTML:function(w){return w},createScriptURL:function(w){return w}})}catch{return console.warn("TrustedTypes policy "+b+" could not be created."),null}};function iCe(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:TPt(),i=function(bt){return iCe(bt)};if(i.version="2.4.3",i.removed=[],!r||!r.document||r.document.nodeType!==9)return i.isSupported=!1,i;var o=r.document,l=r.document,f=r.DocumentFragment,b=r.HTMLTemplateElement,d=r.Node,w=r.Element,y=r.NodeFilter,k=r.NamedNodeMap,E=k===void 0?r.NamedNodeMap||r.MozNamedAttrMap:k,T=r.HTMLFormElement,C=r.DOMParser,S=r.trustedTypes,L=w.prototype,O=vG(L,"cloneNode"),B=vG(L,"nextSibling"),N=vG(L,"childNodes"),F=vG(L,"parentNode");if(typeof b=="function"){var R=l.createElement("template");R.content&&R.content.ownerDocument&&(l=R.content.ownerDocument)}var q=CPt(S,o),X=q?q.createHTML(""):"",te=l,H=te.implementation,Y=te.createNodeIterator,z=te.createDocumentFragment,W=te.getElementsByTagName,Z=o.importNode,G={};try{G=U8(l).documentMode?l.documentMode:{}}catch{}var ae={};i.isSupported=typeof F=="function"&&H&&typeof H.createHTMLDocument<"u"&&G!==9;var $=bPt,ge=vPt,ee=wPt,de=mPt,re=yPt,ke=xPt,Ce=EPt,_e=kPt,Te=null,Be=xa({},[].concat(Cv(eCe),Cv(Jse),Cv(eae),Cv(tae),Cv(tCe))),Ge=null,Xe=xa({},[].concat(Cv(nCe),Cv(nae),Cv(rCe),Cv(wG))),Ee=Object.seal(Object.create(null,{tagNameCheck:{writable:!0,configurable:!1,enumerable:!0,value:null},attributeNameCheck:{writable:!0,configurable:!1,enumerable:!0,value:null},allowCustomizedBuiltInElements:{writable:!0,configurable:!1,enumerable:!0,value:!1}})),Ze=null,Ie=null,Oe=!0,Le=!0,$e=!1,xe=!1,Ae=!1,je=!1,me=!1,vt=!1,ve=!1,Zt=!1,nt=!0,xn=!1,cn="user-content-",jt=!0,ot=!1,be={},We=null,ct=xa({},["annotation-xml","audio","colgroup","desc","foreignobject","head","iframe","math","mi","mn","mo","ms","mtext","noembed","noframes","noscript","plaintext","script","style","svg","template","thead","title","video","xmp"]),Yt=null,Ut=xa({},["audio","video","img","source","image","track"]),Wn=null,Gt=xa({},["alt","class","for","id","label","name","pattern","placeholder","role","summary","title","value","style","xmlns"]),Rn="http://www.w3.org/1998/Math/MathML",si="http://www.w3.org/2000/svg",$r="http://www.w3.org/1999/xhtml",nr=$r,Kn=!1,Jt=null,en=xa({},[Rn,si,$r],Qse),In,hn=["application/xhtml+xml","text/html"],Fr="text/html",Pt,ei=null,nn=l.createElement("form"),hi=function(bt){return bt instanceof RegExp||bt instanceof Function},Hi=function(bt){ei&&ei===bt||((!bt||a6(bt)!=="object")&&(bt={}),bt=U8(bt),In=hn.indexOf(bt.PARSER_MEDIA_TYPE)===-1?In=Fr:In=bt.PARSER_MEDIA_TYPE,Pt=In==="application/xhtml+xml"?Qse:bG,Te="ALLOWED_TAGS"in bt?xa({},bt.ALLOWED_TAGS,Pt):Be,Ge="ALLOWED_ATTR"in bt?xa({},bt.ALLOWED_ATTR,Pt):Xe,Jt="ALLOWED_NAMESPACES"in bt?xa({},bt.ALLOWED_NAMESPACES,Qse):en,Wn="ADD_URI_SAFE_ATTR"in bt?xa(U8(Gt),bt.ADD_URI_SAFE_ATTR,Pt):Gt,Yt="ADD_DATA_URI_TAGS"in bt?xa(U8(Ut),bt.ADD_DATA_URI_TAGS,Pt):Ut,We="FORBID_CONTENTS"in bt?xa({},bt.FORBID_CONTENTS,Pt):ct,Ze="FORBID_TAGS"in bt?xa({},bt.FORBID_TAGS,Pt):{},Ie="FORBID_ATTR"in bt?xa({},bt.FORBID_ATTR,Pt):{},be="USE_PROFILES"in bt?bt.USE_PROFILES:!1,Oe=bt.ALLOW_ARIA_ATTR!==!1,Le=bt.ALLOW_DATA_ATTR!==!1,$e=bt.ALLOW_UNKNOWN_PROTOCOLS||!1,xe=bt.SAFE_FOR_TEMPLATES||!1,Ae=bt.WHOLE_DOCUMENT||!1,vt=bt.RETURN_DOM||!1,ve=bt.RETURN_DOM_FRAGMENT||!1,Zt=bt.RETURN_TRUSTED_TYPE||!1,me=bt.FORCE_BODY||!1,nt=bt.SANITIZE_DOM!==!1,xn=bt.SANITIZE_NAMED_PROPS||!1,jt=bt.KEEP_CONTENT!==!1,ot=bt.IN_PLACE||!1,_e=bt.ALLOWED_URI_REGEXP||_e,nr=bt.NAMESPACE||$r,bt.CUSTOM_ELEMENT_HANDLING&&hi(bt.CUSTOM_ELEMENT_HANDLING.tagNameCheck)&&(Ee.tagNameCheck=bt.CUSTOM_ELEMENT_HANDLING.tagNameCheck),bt.CUSTOM_ELEMENT_HANDLING&&hi(bt.CUSTOM_ELEMENT_HANDLING.attributeNameCheck)&&(Ee.attributeNameCheck=bt.CUSTOM_ELEMENT_HANDLING.attributeNameCheck),bt.CUSTOM_ELEMENT_HANDLING&&typeof bt.CUSTOM_ELEMENT_HANDLING.allowCustomizedBuiltInElements=="boolean"&&(Ee.allowCustomizedBuiltInElements=bt.CUSTOM_ELEMENT_HANDLING.allowCustomizedBuiltInElements),xe&&(Le=!1),ve&&(vt=!0),be&&(Te=xa({},Cv(tCe)),Ge=[],be.html===!0&&(xa(Te,eCe),xa(Ge,nCe)),be.svg===!0&&(xa(Te,Jse),xa(Ge,nae),xa(Ge,wG)),be.svgFilters===!0&&(xa(Te,eae),xa(Ge,nae),xa(Ge,wG)),be.mathMl===!0&&(xa(Te,tae),xa(Ge,rCe),xa(Ge,wG))),bt.ADD_TAGS&&(Te===Be&&(Te=U8(Te)),xa(Te,bt.ADD_TAGS,Pt)),bt.ADD_ATTR&&(Ge===Xe&&(Ge=U8(Ge)),xa(Ge,bt.ADD_ATTR,Pt)),bt.ADD_URI_SAFE_ATTR&&xa(Wn,bt.ADD_URI_SAFE_ATTR,Pt),bt.FORBID_CONTENTS&&(We===ct&&(We=U8(We)),xa(We,bt.FORBID_CONTENTS,Pt)),jt&&(Te["#text"]=!0),Ae&&xa(Te,["html","head","body"]),Te.table&&(xa(Te,["tbody"]),delete Ze.tbody),Wd&&Wd(bt),ei=bt)},ss=xa({},["mi","mo","mn","ms","mtext"]),ls=xa({},["foreignobject","desc","title","annotation-xml"]),vs=xa({},["title","style","font","a","script"]),ti=xa({},Jse);xa(ti,eae),xa(ti,gPt);var zi=xa({},tae);xa(zi,pPt);var as=function(bt){var $n=F(bt);(!$n||!$n.tagName)&&($n={namespaceURI:nr,tagName:"template"});var Er=bG(bt.tagName),Ss=bG($n.tagName);return Jt[bt.namespaceURI]?bt.namespaceURI===si?$n.namespaceURI===$r?Er==="svg":$n.namespaceURI===Rn?Er==="svg"&&(Ss==="annotation-xml"||ss[Ss]):Boolean(ti[Er]):bt.namespaceURI===Rn?$n.namespaceURI===$r?Er==="math":$n.namespaceURI===si?Er==="math"&&ls[Ss]:Boolean(zi[Er]):bt.namespaceURI===$r?$n.namespaceURI===si&&!ls[Ss]||$n.namespaceURI===Rn&&!ss[Ss]?!1:!zi[Er]&&(vs[Er]||!ti[Er]):!!(In==="application/xhtml+xml"&&Jt[bt.namespaceURI]):!1},ai=function(bt){JL(i.removed,{element:bt});try{bt.parentNode.removeChild(bt)}catch{try{bt.outerHTML=X}catch{bt.remove()}}},hc=function(bt,$n){try{JL(i.removed,{attribute:$n.getAttributeNode(bt),from:$n})}catch{JL(i.removed,{attribute:null,from:$n})}if($n.removeAttribute(bt),bt==="is"&&!Ge[bt])if(vt||ve)try{ai($n)}catch{}else try{$n.setAttribute(bt,"")}catch{}},xu=function(bt){var $n,Er;if(me)bt="<remove></remove>"+bt;else{var Ss=lPt(bt,/^[\r\n\t ]+/);Er=Ss&&Ss[0]}In==="application/xhtml+xml"&&nr===$r&&(bt='<html xmlns="http://www.w3.org/1999/xhtml"><head></head><body>'+bt+"</body></html>");var wo=q?q.createHTML(bt):bt;if(nr===$r)try{$n=new C().parseFromString(wo,In)}catch{}if(!$n||!$n.documentElement){$n=H.createDocument(nr,"template",null);try{$n.documentElement.innerHTML=Kn?X:wo}catch{}}var Po=$n.body||$n.documentElement;return bt&&Er&&Po.insertBefore(l.createTextNode(Er),Po.childNodes[0]||null),nr===$r?W.call($n,Ae?"html":"body")[0]:Ae?$n.documentElement:Po},No=function(bt){return Y.call(bt.ownerDocument||bt,bt,y.SHOW_ELEMENT|y.SHOW_COMMENT|y.SHOW_TEXT,null,!1)},Si=function(bt){return bt instanceof T&&(typeof bt.nodeName!="string"||typeof bt.textContent!="string"||typeof bt.removeChild!="function"||!(bt.attributes instanceof E)||typeof bt.removeAttribute!="function"||typeof bt.setAttribute!="function"||typeof bt.namespaceURI!="string"||typeof bt.insertBefore!="function"||typeof bt.hasChildNodes!="function")},Yc=function(bt){return a6(d)==="object"?bt instanceof d:bt&&a6(bt)==="object"&&typeof bt.nodeType=="number"&&typeof bt.nodeName=="string"},lh=function(bt,$n,Er){!ae[bt]||uPt(ae[bt],function(Ss){Ss.call(i,$n,Er,ei)})},su=function(bt){var $n;if(lh("beforeSanitizeElements",bt,null),Si(bt)||Kd(/[\u0080-\uFFFF]/,bt.nodeName))return ai(bt),!0;var Er=Pt(bt.nodeName);if(lh("uponSanitizeElement",bt,{tagName:Er,allowedTags:Te}),bt.hasChildNodes()&&!Yc(bt.firstElementChild)&&(!Yc(bt.content)||!Yc(bt.content.firstElementChild))&&Kd(/<[/\w]/g,bt.innerHTML)&&Kd(/<[/\w]/g,bt.textContent)||Er==="select"&&Kd(/<template/i,bt.innerHTML))return ai(bt),!0;if(!Te[Er]||Ze[Er]){if(!Ze[Er]&&fc(Er)&&(Ee.tagNameCheck instanceof RegExp&&Kd(Ee.tagNameCheck,Er)||Ee.tagNameCheck instanceof Function&&Ee.tagNameCheck(Er)))return!1;if(jt&&!We[Er]){var Ss=F(bt)||bt.parentNode,wo=N(bt)||bt.childNodes;if(wo&&Ss)for(var Po=wo.length,Ra=Po-1;Ra>=0;--Ra)Ss.insertBefore(O(wo[Ra],!0),B(bt))}return ai(bt),!0}return bt instanceof w&&!as(bt)||(Er==="noscript"||Er==="noembed")&&Kd(/<\/no(script|embed)/i,bt.innerHTML)?(ai(bt),!0):(xe&&bt.nodeType===3&&($n=bt.textContent,$n=Av($n,$," "),$n=Av($n,ge," "),$n=Av($n,ee," "),bt.textContent!==$n&&(JL(i.removed,{element:bt.cloneNode()}),bt.textContent=$n)),lh("afterSanitizeElements",bt,null),!1)},Ru=function(bt,$n,Er){if(nt&&($n==="id"||$n==="name")&&(Er in l||Er in nn))return!1;if(!(Le&&!Ie[$n]&&Kd(de,$n))){if(!(Oe&&Kd(re,$n))){if(!Ge[$n]||Ie[$n]){if(!(fc(bt)&&(Ee.tagNameCheck instanceof RegExp&&Kd(Ee.tagNameCheck,bt)||Ee.tagNameCheck instanceof Function&&Ee.tagNameCheck(bt))&&(Ee.attributeNameCheck instanceof RegExp&&Kd(Ee.attributeNameCheck,$n)||Ee.attributeNameCheck instanceof Function&&Ee.attributeNameCheck($n))||$n==="is"&&Ee.allowCustomizedBuiltInElements&&(Ee.tagNameCheck instanceof RegExp&&Kd(Ee.tagNameCheck,Er)||Ee.tagNameCheck instanceof Function&&Ee.tagNameCheck(Er))))return!1}else if(!Wn[$n]){if(!Kd(_e,Av(Er,Ce,""))){if(!(($n==="src"||$n==="xlink:href"||$n==="href")&&bt!=="script"&&hPt(Er,"data:")===0&&Yt[bt])){if(!($e&&!Kd(ke,Av(Er,Ce,"")))){if(Er)return!1}}}}}}return!0},fc=function(bt){return bt.indexOf("-")>0},Ll=function(bt){var $n,Er,Ss,wo;lh("beforeSanitizeAttributes",bt,null);var Po=bt.attributes;if(!!Po){var Ra={attrName:"",attrValue:"",keepAttr:!0,allowedAttributes:Ge};for(wo=Po.length;wo--;){$n=Po[wo];var Eu=$n,Lc=Eu.name,bd=Eu.namespaceURI;if(Er=Lc==="value"?$n.value:fPt($n.value),Ss=Pt(Lc),Ra.attrName=Ss,Ra.attrValue=Er,Ra.keepAttr=!0,Ra.forceKeepAttr=void 0,lh("uponSanitizeAttribute",bt,Ra),Er=Ra.attrValue,!Ra.forceKeepAttr&&(hc(Lc,bt),!!Ra.keepAttr)){if(Kd(/\/>/i,Er)){hc(Lc,bt);continue}xe&&(Er=Av(Er,$," "),Er=Av(Er,ge," "),Er=Av(Er,ee," "));var hh=Pt(bt.nodeName);if(!!Ru(hh,Ss,Er)){if(xn&&(Ss==="id"||Ss==="name")&&(hc(Lc,bt),Er=cn+Er),q&&a6(S)==="object"&&typeof S.getAttributeType=="function"&&!bd)switch(S.getAttributeType(hh,Ss)){case"TrustedHTML":Er=q.createHTML(Er);break;case"TrustedScriptURL":Er=q.createScriptURL(Er);break}try{bd?bt.setAttributeNS(bd,Lc,Er):bt.setAttribute(Lc,Er),JTe(i.removed)}catch{}}}}lh("afterSanitizeAttributes",bt,null)}},ol=function Ur(bt){var $n,Er=No(bt);for(lh("beforeSanitizeShadowDOM",bt,null);$n=Er.nextNode();)lh("uponSanitizeShadowNode",$n,null),!su($n)&&($n.content instanceof f&&Ur($n.content),Ll($n));lh("afterSanitizeShadowDOM",bt,null)};return i.sanitize=function(Ur){var bt=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},$n,Er,Ss,wo,Po;if(Kn=!Ur,Kn&&(Ur="<!-->"),typeof Ur!="string"&&!Yc(Ur)){if(typeof Ur.toString!="function")throw Zse("toString is not a function");if(Ur=Ur.toString(),typeof Ur!="string")throw Zse("dirty is not a string, aborting")}if(!i.isSupported){if(a6(r.toStaticHTML)==="object"||typeof r.toStaticHTML=="function"){if(typeof Ur=="string")return r.toStaticHTML(Ur);if(Yc(Ur))return r.toStaticHTML(Ur.outerHTML)}return Ur}if(je||Hi(bt),i.removed=[],typeof Ur=="string"&&(ot=!1),ot){if(Ur.nodeName){var Ra=Pt(Ur.nodeName);if(!Te[Ra]||Ze[Ra])throw Zse("root node is forbidden and cannot be sanitized in-place")}}else if(Ur instanceof d)$n=xu("<!---->"),Er=$n.ownerDocument.importNode(Ur,!0),Er.nodeType===1&&Er.nodeName==="BODY"||Er.nodeName==="HTML"?$n=Er:$n.appendChild(Er);else{if(!vt&&!xe&&!Ae&&Ur.indexOf("<")===-1)return q&&Zt?q.createHTML(Ur):Ur;if($n=xu(Ur),!$n)return vt?null:Zt?X:""}$n&&me&&ai($n.firstChild);for(var Eu=No(ot?Ur:$n);Ss=Eu.nextNode();)Ss.nodeType===3&&Ss===wo||su(Ss)||(Ss.content instanceof f&&ol(Ss.content),Ll(Ss),wo=Ss);if(wo=null,ot)return Ur;if(vt){if(ve)for(Po=z.call($n.ownerDocument);$n.firstChild;)Po.appendChild($n.firstChild);else Po=$n;return Ge.shadowroot&&(Po=Z.call(o,Po,!0)),Po}var Lc=Ae?$n.outerHTML:$n.innerHTML;return Ae&&Te["!doctype"]&&$n.ownerDocument&&$n.ownerDocument.doctype&&$n.ownerDocument.doctype.name&&Kd(_Pt,$n.ownerDocument.doctype.name)&&(Lc="<!DOCTYPE "+$n.ownerDocument.doctype.name+`>
`+Lc),xe&&(Lc=Av(Lc,$," "),Lc=Av(Lc,ge," "),Lc=Av(Lc,ee," ")),q&&Zt?q.createHTML(Lc):Lc},i.setConfig=function(Ur){Hi(Ur),je=!0},i.clearConfig=function(){ei=null,je=!1},i.isValidAttribute=function(Ur,bt,$n){ei||Hi({});var Er=Pt(Ur),Ss=Pt(bt);return Ru(Er,Ss,$n)},i.addHook=function(Ur,bt){typeof bt=="function"&&(ae[Ur]=ae[Ur]||[],JL(ae[Ur],bt))},i.removeHook=function(Ur){if(ae[Ur])return JTe(ae[Ur])},i.removeHooks=function(Ur){ae[Ur]&&(ae[Ur]=[])},i.removeAllHooks=function(){ae={}},i}var eI=iCe();const SPt=r=>r?oCe(r).replace(/\\n/g,"#br#").split("#br#"):[""],sCe=r=>eI.sanitize(r),aCe=(r,i)=>{var o;if(((o=i.flowchart)==null?void 0:o.htmlLabels)!==!1){const l=i.securityLevel;l==="antiscript"||l==="strict"?r=sCe(r):l!=="loose"&&(r=oCe(r),r=r.replace(/</g,"&lt;").replace(/>/g,"&gt;"),r=r.replace(/=/g,"&equals;"),r=LPt(r))}return r},up=(r,i)=>r&&(i.dompurifyConfig?r=eI.sanitize(aCe(r,i),i.dompurifyConfig).toString():r=eI.sanitize(aCe(r,i),{FORBID_TAGS:["style"]}).toString(),r),APt=(r,i)=>typeof r=="string"?up(r,i):r.flat().map(o=>up(o,i)),mG=/<br\s*\/?>/gi,MPt=r=>mG.test(r),DPt=r=>r.split(mG),LPt=r=>r.replace(/#br#/g,"<br/>"),oCe=r=>r.replace(mG,"#br#"),IPt=r=>{let i="";return r&&(i=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,i=i.replaceAll(/\(/g,"\\("),i=i.replaceAll(/\)/g,"\\)")),i},cf=r=>!(r===!1||["false","null","0"].includes(String(r).trim().toLowerCase())),o6=function(r){let i=r;if(r.split("~").length-1>=2){let o=i;do i=o,o=i.replace(/~([^\s,:;]+)~/,"<$1>");while(o!=i);return o6(o)}else return i},Cs={getRows:SPt,sanitizeText:up,sanitizeTextOrArray:APt,hasBreaks:MPt,splitBreaks:DPt,lineBreakRegex:mG,removeScript:sCe,getUrl:IPt,evaluate:cf},yG={min:{r:0,g:0,b:0,s:0,l:0,a:0},max:{r:255,g:255,b:255,h:360,s:100,l:100,a:1},clamp:{r:r=>r>=255?255:r<0?0:r,g:r=>r>=255?255:r<0?0:r,b:r=>r>=255?255:r<0?0:r,h:r=>r%360,s:r=>r>=100?100:r<0?0:r,l:r=>r>=100?100:r<0?0:r,a:r=>r>=1?1:r<0?0:r},toLinear:r=>{const i=r/255;return r>.03928?Math.pow((i+.055)/1.055,2.4):i/12.92},hue2rgb:(r,i,o)=>(o<0&&(o+=1),o>1&&(o-=1),o<1/6?r+(i-r)*6*o:o<1/2?i:o<2/3?r+(i-r)*(2/3-o)*6:r),hsl2rgb:({h:r,s:i,l:o},l)=>{if(!i)return o*2.55;r/=360,i/=100,o/=100;const f=o<.5?o*(1+i):o+i-o*i,b=2*o-f;switch(l){case"r":return yG.hue2rgb(b,f,r+1/3)*255;case"g":return yG.hue2rgb(b,f,r)*255;case"b":return yG.hue2rgb(b,f,r-1/3)*255}},rgb2hsl:({r,g:i,b:o},l)=>{r/=255,i/=255,o/=255;const f=Math.max(r,i,o),b=Math.min(r,i,o),d=(f+b)/2;if(l==="l")return d*100;if(f===b)return 0;const w=f-b,y=d>.5?w/(2-f-b):w/(f+b);if(l==="s")return y*100;switch(f){case r:return((i-o)/w+(i<o?6:0))*60;case i:return((o-r)/w+2)*60;case o:return((r-i)/w+4)*60;default:return-1}}},Bs={channel:yG,lang:{clamp:(r,i,o)=>i>o?Math.min(i,Math.max(o,r)):Math.min(o,Math.max(i,r)),round:r=>Math.round(r*1e10)/1e10},unit:{dec2hex:r=>{const i=Math.round(r).toString(16);return i.length>1?i:`0${i}`}}},c6={};for(let r=0;r<=255;r++)c6[r]=Bs.unit.dec2hex(r);const cd={ALL:0,RGB:1,HSL:2};class OPt{constructor(){this.type=cd.ALL}get(){return this.type}set(i){if(this.type&&this.type!==i)throw new Error("Cannot change both RGB and HSL channels at the same time");this.type=i}reset(){this.type=cd.ALL}is(i){return this.type===i}}const NPt=OPt;class PPt{constructor(i,o){this.color=o,this.changed=!1,this.data=i,this.type=new NPt}set(i,o){return this.color=o,this.changed=!1,this.data=i,this.type.type=cd.ALL,this}_ensureHSL(){const i=this.data,{h:o,s:l,l:f}=i;o===void 0&&(i.h=Bs.channel.rgb2hsl(i,"h")),l===void 0&&(i.s=Bs.channel.rgb2hsl(i,"s")),f===void 0&&(i.l=Bs.channel.rgb2hsl(i,"l"))}_ensureRGB(){const i=this.data,{r:o,g:l,b:f}=i;o===void 0&&(i.r=Bs.channel.hsl2rgb(i,"r")),l===void 0&&(i.g=Bs.channel.hsl2rgb(i,"g")),f===void 0&&(i.b=Bs.channel.hsl2rgb(i,"b"))}get r(){const i=this.data,o=i.r;return!this.type.is(cd.HSL)&&o!==void 0?o:(this._ensureHSL(),Bs.channel.hsl2rgb(i,"r"))}get g(){const i=this.data,o=i.g;return!this.type.is(cd.HSL)&&o!==void 0?o:(this._ensureHSL(),Bs.channel.hsl2rgb(i,"g"))}get b(){const i=this.data,o=i.b;return!this.type.is(cd.HSL)&&o!==void 0?o:(this._ensureHSL(),Bs.channel.hsl2rgb(i,"b"))}get h(){const i=this.data,o=i.h;return!this.type.is(cd.RGB)&&o!==void 0?o:(this._ensureRGB(),Bs.channel.rgb2hsl(i,"h"))}get s(){const i=this.data,o=i.s;return!this.type.is(cd.RGB)&&o!==void 0?o:(this._ensureRGB(),Bs.channel.rgb2hsl(i,"s"))}get l(){const i=this.data,o=i.l;return!this.type.is(cd.RGB)&&o!==void 0?o:(this._ensureRGB(),Bs.channel.rgb2hsl(i,"l"))}get a(){return this.data.a}set r(i){this.type.set(cd.RGB),this.changed=!0,this.data.r=i}set g(i){this.type.set(cd.RGB),this.changed=!0,this.data.g=i}set b(i){this.type.set(cd.RGB),this.changed=!0,this.data.b=i}set h(i){this.type.set(cd.HSL),this.changed=!0,this.data.h=i}set s(i){this.type.set(cd.HSL),this.changed=!0,this.data.s=i}set l(i){this.type.set(cd.HSL),this.changed=!0,this.data.l=i}set a(i){this.changed=!0,this.data.a=i}}const BPt=PPt,kG=new BPt({r:0,g:0,b:0,a:0},"transparent"),cCe={re:/^#((?:[a-f0-9]{2}){2,4}|[a-f0-9]{3})$/i,parse:r=>{if(r.charCodeAt(0)!==35)return;const i=r.match(cCe.re);if(!i)return;const o=i[1],l=parseInt(o,16),f=o.length,b=f%4===0,d=f>4,w=d?1:17,y=d?8:4,k=b?0:-1,E=d?255:15;return kG.set({r:(l>>y*(k+3)&E)*w,g:(l>>y*(k+2)&E)*w,b:(l>>y*(k+1)&E)*w,a:b?(l&E)*w/255:1},r)},stringify:r=>{const{r:i,g:o,b:l,a:f}=r;return f<1?`#${c6[Math.round(i)]}${c6[Math.round(o)]}${c6[Math.round(l)]}${c6[Math.round(f*255)]}`:`#${c6[Math.round(i)]}${c6[Math.round(o)]}${c6[Math.round(l)]}`}},tI=cCe,xG={re:/^hsla?\(\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?(?:deg|grad|rad|turn)?)\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?%)\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?%)(?:\s*?(?:,|\/)\s*?\+?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?(%)?))?\s*?\)$/i,hueRe:/^(.+?)(deg|grad|rad|turn)$/i,_hue2deg:r=>{const i=r.match(xG.hueRe);if(i){const[,o,l]=i;switch(l){case"grad":return Bs.channel.clamp.h(parseFloat(o)*.9);case"rad":return Bs.channel.clamp.h(parseFloat(o)*180/Math.PI);case"turn":return Bs.channel.clamp.h(parseFloat(o)*360)}}return Bs.channel.clamp.h(parseFloat(r))},parse:r=>{const i=r.charCodeAt(0);if(i!==104&&i!==72)return;const o=r.match(xG.re);if(!o)return;const[,l,f,b,d,w]=o;return kG.set({h:xG._hue2deg(l),s:Bs.channel.clamp.s(parseFloat(f)),l:Bs.channel.clamp.l(parseFloat(b)),a:d?Bs.channel.clamp.a(w?parseFloat(d)/100:parseFloat(d)):1},r)},stringify:r=>{const{h:i,s:o,l,a:f}=r;return f<1?`hsla(${Bs.lang.round(i)}, ${Bs.lang.round(o)}%, ${Bs.lang.round(l)}%, ${f})`:`hsl(${Bs.lang.round(i)}, ${Bs.lang.round(o)}%, ${Bs.lang.round(l)}%)`}},EG=xG,_G={colors:{aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyanaqua:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgreen:"#006400",darkgrey:"#a9a9a9",darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkslategrey:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",gray:"#808080",green:"#008000",greenyellow:"#adff2f",grey:"#808080",honeydew:"#f0fff0",hotpink:"#ff69b4",indianred:"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrodyellow:"#fafad2",lightgray:"#d3d3d3",lightgreen:"#90ee90",lightgrey:"#d3d3d3",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#b0c4de",lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370db",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#db7093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",rebeccapurple:"#663399",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",slategray:"#708090",slategrey:"#708090",snow:"#fffafa",springgreen:"#00ff7f",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",transparent:"#00000000",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"},parse:r=>{r=r.toLowerCase();const i=_G.colors[r];if(!!i)return tI.parse(i)},stringify:r=>{const i=tI.stringify(r);for(const o in _G.colors)if(_G.colors[o]===i)return o}},uCe=_G,lCe={re:/^rgba?\(\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?))\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?))\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?))(?:\s*?(?:,|\/)\s*?\+?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?)))?\s*?\)$/i,parse:r=>{const i=r.charCodeAt(0);if(i!==114&&i!==82)return;const o=r.match(lCe.re);if(!o)return;const[,l,f,b,d,w,y,k,E]=o;return kG.set({r:Bs.channel.clamp.r(f?parseFloat(l)*2.55:parseFloat(l)),g:Bs.channel.clamp.g(d?parseFloat(b)*2.55:parseFloat(b)),b:Bs.channel.clamp.b(y?parseFloat(w)*2.55:parseFloat(w)),a:k?Bs.channel.clamp.a(E?parseFloat(k)/100:parseFloat(k)):1},r)},stringify:r=>{const{r:i,g:o,b:l,a:f}=r;return f<1?`rgba(${Bs.lang.round(i)}, ${Bs.lang.round(o)}, ${Bs.lang.round(l)}, ${Bs.lang.round(f)})`:`rgb(${Bs.lang.round(i)}, ${Bs.lang.round(o)}, ${Bs.lang.round(l)})`}},TG=lCe,Nm={format:{keyword:uCe,hex:tI,rgb:TG,rgba:TG,hsl:EG,hsla:EG},parse:r=>{if(typeof r!="string")return r;const i=tI.parse(r)||TG.parse(r)||EG.parse(r)||uCe.parse(r);if(i)return i;throw new Error(`Unsupported color format: "${r}"`)},stringify:r=>!r.changed&&r.color?r.color:r.type.is(cd.HSL)||r.data.r===void 0?EG.stringify(r):r.a<1||!Number.isInteger(r.r)||!Number.isInteger(r.g)||!Number.isInteger(r.b)?TG.stringify(r):tI.stringify(r)},hCe=(r,i)=>{const o=Nm.parse(r);for(const l in i)o[l]=Bs.channel.clamp[l](i[l]);return Nm.stringify(o)},nI=(r,i,o=0,l=1)=>{if(typeof r!="number")return hCe(r,{a:i});const f=kG.set({r:Bs.channel.clamp.r(r),g:Bs.channel.clamp.g(i),b:Bs.channel.clamp.b(o),a:Bs.channel.clamp.a(l)});return Nm.stringify(f)},FPt=r=>{const{r:i,g:o,b:l}=Nm.parse(r),f=.2126*Bs.channel.toLinear(i)+.7152*Bs.channel.toLinear(o)+.0722*Bs.channel.toLinear(l);return Bs.lang.round(f)},RPt=r=>FPt(r)>=.5,fCe=r=>!RPt(r),dCe=(r,i,o)=>{const l=Nm.parse(r),f=l[i],b=Bs.channel.clamp[i](f+o);return f!==b&&(l[i]=b),Nm.stringify(l)},is=(r,i)=>dCe(r,"l",i),Ds=(r,i)=>dCe(r,"l",-i),vn=(r,i)=>{const o=Nm.parse(r),l={};for(const f in i)!i[f]||(l[f]=o[f]+i[f]);return hCe(r,l)},jPt=(r,i,o=50)=>{const{r:l,g:f,b,a:d}=Nm.parse(r),{r:w,g:y,b:k,a:E}=Nm.parse(i),T=o/100,C=T*2-1,S=d-E,O=((C*S===-1?C:(C+S)/(1+C*S))+1)/2,B=1-O,N=l*O+w*B,F=f*O+y*B,R=b*O+k*B,q=d*T+E*(1-T);return nI(N,F,R,q)},Kr=(r,i=100)=>{const o=Nm.parse(r);return o.r=255-o.r,o.g=255-o.g,o.b=255-o.b,jPt(o,r,i)},Xd=(r,i)=>i?vn(r,{s:-40,l:10}):vn(r,{s:-40,l:-10}),CG="#ffffff",SG="#f2f2f2";class $Pt{constructor(){this.background="#f4f4f4",this.primaryColor="#fff4dd",this.noteBkgColor="#fff5ad",this.noteTextColor="#333",this.THEME_COLOR_LIMIT=12,this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px"}updateColors(){if(this.primaryTextColor=this.primaryTextColor||(this.darkMode?"#eee":"#333"),this.secondaryColor=this.secondaryColor||vn(this.primaryColor,{h:-120}),this.tertiaryColor=this.tertiaryColor||vn(this.primaryColor,{h:180,l:5}),this.primaryBorderColor=this.primaryBorderColor||Xd(this.primaryColor,this.darkMode),this.secondaryBorderColor=this.secondaryBorderColor||Xd(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=this.tertiaryBorderColor||Xd(this.tertiaryColor,this.darkMode),this.noteBorderColor=this.noteBorderColor||Xd(this.noteBkgColor,this.darkMode),this.noteBkgColor=this.noteBkgColor||"#fff5ad",this.noteTextColor=this.noteTextColor||"#333",this.secondaryTextColor=this.secondaryTextColor||Kr(this.secondaryColor),this.tertiaryTextColor=this.tertiaryTextColor||Kr(this.tertiaryColor),this.lineColor=this.lineColor||Kr(this.background),this.textColor=this.textColor||this.primaryTextColor,this.nodeBkg=this.nodeBkg||this.primaryColor,this.mainBkg=this.mainBkg||this.primaryColor,this.nodeBorder=this.nodeBorder||this.primaryBorderColor,this.clusterBkg=this.clusterBkg||this.tertiaryColor,this.clusterBorder=this.clusterBorder||this.tertiaryBorderColor,this.defaultLinkColor=this.defaultLinkColor||this.lineColor,this.titleColor=this.titleColor||this.tertiaryTextColor,this.edgeLabelBackground=this.edgeLabelBackground||(this.darkMode?Ds(this.secondaryColor,30):this.secondaryColor),this.nodeTextColor=this.nodeTextColor||this.primaryTextColor,this.actorBorder=this.actorBorder||this.primaryBorderColor,this.actorBkg=this.actorBkg||this.mainBkg,this.actorTextColor=this.actorTextColor||this.primaryTextColor,this.actorLineColor=this.actorLineColor||"grey",this.labelBoxBkgColor=this.labelBoxBkgColor||this.actorBkg,this.signalColor=this.signalColor||this.textColor,this.signalTextColor=this.signalTextColor||this.textColor,this.labelBoxBorderColor=this.labelBoxBorderColor||this.actorBorder,this.labelTextColor=this.labelTextColor||this.actorTextColor,this.loopTextColor=this.loopTextColor||this.actorTextColor,this.activationBorderColor=this.activationBorderColor||Ds(this.secondaryColor,10),this.activationBkgColor=this.activationBkgColor||this.secondaryColor,this.sequenceNumberColor=this.sequenceNumberColor||Kr(this.lineColor),this.sectionBkgColor=this.sectionBkgColor||this.tertiaryColor,this.altSectionBkgColor=this.altSectionBkgColor||"white",this.sectionBkgColor=this.sectionBkgColor||this.secondaryColor,this.sectionBkgColor2=this.sectionBkgColor2||this.primaryColor,this.excludeBkgColor=this.excludeBkgColor||"#eeeeee",this.taskBorderColor=this.taskBorderColor||this.primaryBorderColor,this.taskBkgColor=this.taskBkgColor||this.primaryColor,this.activeTaskBorderColor=this.activeTaskBorderColor||this.primaryColor,this.activeTaskBkgColor=this.activeTaskBkgColor||is(this.primaryColor,23),this.gridColor=this.gridColor||"lightgrey",this.doneTaskBkgColor=this.doneTaskBkgColor||"lightgrey",this.doneTaskBorderColor=this.doneTaskBorderColor||"grey",this.critBorderColor=this.critBorderColor||"#ff8888",this.critBkgColor=this.critBkgColor||"red",this.todayLineColor=this.todayLineColor||"red",this.taskTextColor=this.taskTextColor||this.textColor,this.taskTextOutsideColor=this.taskTextOutsideColor||this.textColor,this.taskTextLightColor=this.taskTextLightColor||this.textColor,this.taskTextColor=this.taskTextColor||this.primaryTextColor,this.taskTextDarkColor=this.taskTextDarkColor||this.textColor,this.taskTextClickableColor=this.taskTextClickableColor||"#003163",this.personBorder=this.personBorder||this.primaryBorderColor,this.personBkg=this.personBkg||this.mainBkg,this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||this.tertiaryColor,this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.nodeBorder,this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.transitionColor=this.transitionColor||this.lineColor,this.specialStateColor=this.lineColor,this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||vn(this.primaryColor,{h:30}),this.cScale4=this.cScale4||vn(this.primaryColor,{h:60}),this.cScale5=this.cScale5||vn(this.primaryColor,{h:90}),this.cScale6=this.cScale6||vn(this.primaryColor,{h:120}),this.cScale7=this.cScale7||vn(this.primaryColor,{h:150}),this.cScale8=this.cScale8||vn(this.primaryColor,{h:210,l:150}),this.cScale9=this.cScale9||vn(this.primaryColor,{h:270}),this.cScale10=this.cScale10||vn(this.primaryColor,{h:300}),this.cScale11=this.cScale11||vn(this.primaryColor,{h:330}),this.darkMode)for(let o=0;o<this.THEME_COLOR_LIMIT;o++)this["cScale"+o]=Ds(this["cScale"+o],75);else for(let o=0;o<this.THEME_COLOR_LIMIT;o++)this["cScale"+o]=Ds(this["cScale"+o],25);for(let o=0;o<this.THEME_COLOR_LIMIT;o++)this["cScaleInv"+o]=this["cScaleInv"+o]||Kr(this["cScale"+o]);for(let o=0;o<this.THEME_COLOR_LIMIT;o++)this.darkMode?this["cScalePeer"+o]=this["cScalePeer"+o]||is(this["cScale"+o],10):this["cScalePeer"+o]=this["cScalePeer"+o]||Ds(this["cScale"+o],10);this.scaleLabelColor=this.scaleLabelColor||this.labelTextColor;for(let o=0;o<this.THEME_COLOR_LIMIT;o++)this["cScaleLabel"+o]=this["cScaleLabel"+o]||this.scaleLabelColor;const i=this.darkMode?-4:-1;for(let o=0;o<5;o++)this["surface"+o]=this["surface"+o]||vn(this.mainBkg,{h:180,s:-15,l:i*(5+o*3)}),this["surfacePeer"+o]=this["surfacePeer"+o]||vn(this.mainBkg,{h:180,s:-15,l:i*(8+o*3)});this.classText=this.classText||this.textColor,this.fillType0=this.fillType0||this.primaryColor,this.fillType1=this.fillType1||this.secondaryColor,this.fillType2=this.fillType2||vn(this.primaryColor,{h:64}),this.fillType3=this.fillType3||vn(this.secondaryColor,{h:64}),this.fillType4=this.fillType4||vn(this.primaryColor,{h:-64}),this.fillType5=this.fillType5||vn(this.secondaryColor,{h:-64}),this.fillType6=this.fillType6||vn(this.primaryColor,{h:128}),this.fillType7=this.fillType7||vn(this.secondaryColor,{h:128}),this.pie1=this.pie1||this.primaryColor,this.pie2=this.pie2||this.secondaryColor,this.pie3=this.pie3||this.tertiaryColor,this.pie4=this.pie4||vn(this.primaryColor,{l:-10}),this.pie5=this.pie5||vn(this.secondaryColor,{l:-10}),this.pie6=this.pie6||vn(this.tertiaryColor,{l:-10}),this.pie7=this.pie7||vn(this.primaryColor,{h:60,l:-10}),this.pie8=this.pie8||vn(this.primaryColor,{h:-60,l:-10}),this.pie9=this.pie9||vn(this.primaryColor,{h:120,l:0}),this.pie10=this.pie10||vn(this.primaryColor,{h:60,l:-20}),this.pie11=this.pie11||vn(this.primaryColor,{h:-60,l:-20}),this.pie12=this.pie12||vn(this.primaryColor,{h:120,l:-10}),this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOpacity=this.pieOpacity||"0.7",this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||this.primaryBorderColor,this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||(this.darkMode?Ds(this.secondaryColor,30):this.secondaryColor),this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=this.git0||this.primaryColor,this.git1=this.git1||this.secondaryColor,this.git2=this.git2||this.tertiaryColor,this.git3=this.git3||vn(this.primaryColor,{h:-30}),this.git4=this.git4||vn(this.primaryColor,{h:-60}),this.git5=this.git5||vn(this.primaryColor,{h:-90}),this.git6=this.git6||vn(this.primaryColor,{h:60}),this.git7=this.git7||vn(this.primaryColor,{h:120}),this.darkMode?(this.git0=is(this.git0,25),this.git1=is(this.git1,25),this.git2=is(this.git2,25),this.git3=is(this.git3,25),this.git4=is(this.git4,25),this.git5=is(this.git5,25),this.git6=is(this.git6,25),this.git7=is(this.git7,25)):(this.git0=Ds(this.git0,25),this.git1=Ds(this.git1,25),this.git2=Ds(this.git2,25),this.git3=Ds(this.git3,25),this.git4=Ds(this.git4,25),this.git5=Ds(this.git5,25),this.git6=Ds(this.git6,25),this.git7=Ds(this.git7,25)),this.gitInv0=this.gitInv0||Kr(this.git0),this.gitInv1=this.gitInv1||Kr(this.git1),this.gitInv2=this.gitInv2||Kr(this.git2),this.gitInv3=this.gitInv3||Kr(this.git3),this.gitInv4=this.gitInv4||Kr(this.git4),this.gitInv5=this.gitInv5||Kr(this.git5),this.gitInv6=this.gitInv6||Kr(this.git6),this.gitInv7=this.gitInv7||Kr(this.git7),this.branchLabelColor=this.branchLabelColor||(this.darkMode?"black":this.labelTextColor),this.gitBranchLabel0=this.gitBranchLabel0||this.branchLabelColor,this.gitBranchLabel1=this.gitBranchLabel1||this.branchLabelColor,this.gitBranchLabel2=this.gitBranchLabel2||this.branchLabelColor,this.gitBranchLabel3=this.gitBranchLabel3||this.branchLabelColor,this.gitBranchLabel4=this.gitBranchLabel4||this.branchLabelColor,this.gitBranchLabel5=this.gitBranchLabel5||this.branchLabelColor,this.gitBranchLabel6=this.gitBranchLabel6||this.branchLabelColor,this.gitBranchLabel7=this.gitBranchLabel7||this.branchLabelColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||CG,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||SG}calculate(i){if(typeof i!="object"){this.updateColors();return}const o=Object.keys(i);o.forEach(l=>{this[l]=i[l]}),this.updateColors(),o.forEach(l=>{this[l]=i[l]})}}const HPt=r=>{const i=new $Pt;return i.calculate(r),i};class zPt{constructor(){this.background="#333",this.primaryColor="#1f2020",this.secondaryColor=is(this.primaryColor,16),this.tertiaryColor=vn(this.primaryColor,{h:-160}),this.primaryBorderColor=Kr(this.background),this.secondaryBorderColor=Xd(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Xd(this.tertiaryColor,this.darkMode),this.primaryTextColor=Kr(this.primaryColor),this.secondaryTextColor=Kr(this.secondaryColor),this.tertiaryTextColor=Kr(this.tertiaryColor),this.lineColor=Kr(this.background),this.textColor=Kr(this.background),this.mainBkg="#1f2020",this.secondBkg="calculated",this.mainContrastColor="lightgrey",this.darkTextColor=is(Kr("#323D47"),10),this.lineColor="calculated",this.border1="#81B1DB",this.border2=nI(255,255,255,.25),this.arrowheadColor="calculated",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.labelBackground="#181818",this.textColor="#ccc",this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="#F9FFFE",this.edgeLabelBackground="calculated",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="calculated",this.actorLineColor="calculated",this.signalColor="calculated",this.signalTextColor="calculated",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="calculated",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="#fff5ad",this.noteTextColor="calculated",this.activationBorderColor="calculated",this.activationBkgColor="calculated",this.sequenceNumberColor="black",this.sectionBkgColor=Ds("#EAE8D9",30),this.altSectionBkgColor="calculated",this.sectionBkgColor2="#EAE8D9",this.taskBorderColor=nI(255,255,255,70),this.taskBkgColor="calculated",this.taskTextColor="calculated",this.taskTextLightColor="calculated",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor=nI(255,255,255,50),this.activeTaskBkgColor="#81B1DB",this.gridColor="calculated",this.doneTaskBkgColor="calculated",this.doneTaskBorderColor="grey",this.critBorderColor="#E83737",this.critBkgColor="#E83737",this.taskTextDarkColor="calculated",this.todayLineColor="#DB5757",this.personBorder="calculated",this.personBkg="calculated",this.labelColor="calculated",this.errorBkgColor="#a44141",this.errorTextColor="#ddd"}updateColors(){this.secondBkg=is(this.mainBkg,16),this.lineColor=this.mainContrastColor,this.arrowheadColor=this.mainContrastColor,this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.edgeLabelBackground=is(this.labelBackground,25),this.actorBorder=this.border1,this.actorBkg=this.mainBkg,this.actorTextColor=this.mainContrastColor,this.actorLineColor=this.mainContrastColor,this.signalColor=this.mainContrastColor,this.signalTextColor=this.mainContrastColor,this.labelBoxBkgColor=this.actorBkg,this.labelBoxBorderColor=this.actorBorder,this.labelTextColor=this.mainContrastColor,this.loopTextColor=this.mainContrastColor,this.noteBorderColor=this.secondaryBorderColor,this.noteBkgColor=this.secondBkg,this.noteTextColor=this.secondaryTextColor,this.activationBorderColor=this.border1,this.activationBkgColor=this.secondBkg,this.altSectionBkgColor=this.background,this.taskBkgColor=is(this.mainBkg,23),this.taskTextColor=this.darkTextColor,this.taskTextLightColor=this.mainContrastColor,this.taskTextOutsideColor=this.taskTextLightColor,this.gridColor=this.mainContrastColor,this.doneTaskBkgColor=this.mainContrastColor,this.taskTextDarkColor=this.darkTextColor,this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#555",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.primaryBorderColor,this.specialStateColor="#f4f4f4",this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=vn(this.primaryColor,{h:64}),this.fillType3=vn(this.secondaryColor,{h:64}),this.fillType4=vn(this.primaryColor,{h:-64}),this.fillType5=vn(this.secondaryColor,{h:-64}),this.fillType6=vn(this.primaryColor,{h:128}),this.fillType7=vn(this.secondaryColor,{h:128}),this.cScale1=this.cScale1||"#0b0000",this.cScale2=this.cScale2||"#4d1037",this.cScale3=this.cScale3||"#3f5258",this.cScale4=this.cScale4||"#4f2f1b",this.cScale5=this.cScale5||"#6e0a0a",this.cScale6=this.cScale6||"#3b0048",this.cScale7=this.cScale7||"#995a01",this.cScale8=this.cScale8||"#154706",this.cScale9=this.cScale9||"#161722",this.cScale10=this.cScale10||"#00296f",this.cScale11=this.cScale11||"#01629c",this.cScale12=this.cScale12||"#010029",this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||vn(this.primaryColor,{h:30}),this.cScale4=this.cScale4||vn(this.primaryColor,{h:60}),this.cScale5=this.cScale5||vn(this.primaryColor,{h:90}),this.cScale6=this.cScale6||vn(this.primaryColor,{h:120}),this.cScale7=this.cScale7||vn(this.primaryColor,{h:150}),this.cScale8=this.cScale8||vn(this.primaryColor,{h:210}),this.cScale9=this.cScale9||vn(this.primaryColor,{h:270}),this.cScale10=this.cScale10||vn(this.primaryColor,{h:300}),this.cScale11=this.cScale11||vn(this.primaryColor,{h:330});for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleInv"+i]=this["cScaleInv"+i]||Kr(this["cScale"+i]);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScalePeer"+i]=this["cScalePeer"+i]||is(this["cScale"+i],10);for(let i=0;i<5;i++)this["surface"+i]=this["surface"+i]||vn(this.mainBkg,{h:30,s:-30,l:-(-10+i*4)}),this["surfacePeer"+i]=this["surfacePeer"+i]||vn(this.mainBkg,{h:30,s:-30,l:-(-7+i*4)});this.scaleLabelColor=this.scaleLabelColor||(this.darkMode?"black":this.labelTextColor);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleLabel"+i]=this["cScaleLabel"+i]||this.scaleLabelColor;for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["pie"+i]=this["cScale"+i];this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOpacity=this.pieOpacity||"0.7",this.classText=this.primaryTextColor,this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||this.primaryBorderColor,this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||(this.darkMode?Ds(this.secondaryColor,30):this.secondaryColor),this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=is(this.secondaryColor,20),this.git1=is(this.pie2||this.secondaryColor,20),this.git2=is(this.pie3||this.tertiaryColor,20),this.git3=is(this.pie4||vn(this.primaryColor,{h:-30}),20),this.git4=is(this.pie5||vn(this.primaryColor,{h:-60}),20),this.git5=is(this.pie6||vn(this.primaryColor,{h:-90}),10),this.git6=is(this.pie7||vn(this.primaryColor,{h:60}),10),this.git7=is(this.pie8||vn(this.primaryColor,{h:120}),20),this.gitInv0=this.gitInv0||Kr(this.git0),this.gitInv1=this.gitInv1||Kr(this.git1),this.gitInv2=this.gitInv2||Kr(this.git2),this.gitInv3=this.gitInv3||Kr(this.git3),this.gitInv4=this.gitInv4||Kr(this.git4),this.gitInv5=this.gitInv5||Kr(this.git5),this.gitInv6=this.gitInv6||Kr(this.git6),this.gitInv7=this.gitInv7||Kr(this.git7),this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||is(this.background,12),this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||is(this.background,2)}calculate(i){if(typeof i!="object"){this.updateColors();return}const o=Object.keys(i);o.forEach(l=>{this[l]=i[l]}),this.updateColors(),o.forEach(l=>{this[l]=i[l]})}}const GPt=r=>{const i=new zPt;return i.calculate(r),i};class VPt{constructor(){this.background="#f4f4f4",this.primaryColor="#ECECFF",this.secondaryColor=vn(this.primaryColor,{h:120}),this.secondaryColor="#ffffde",this.tertiaryColor=vn(this.primaryColor,{h:-160}),this.primaryBorderColor=Xd(this.primaryColor,this.darkMode),this.secondaryBorderColor=Xd(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Xd(this.tertiaryColor,this.darkMode),this.primaryTextColor=Kr(this.primaryColor),this.secondaryTextColor=Kr(this.secondaryColor),this.tertiaryTextColor=Kr(this.tertiaryColor),this.lineColor=Kr(this.background),this.textColor=Kr(this.background),this.background="white",this.mainBkg="#ECECFF",this.secondBkg="#ffffde",this.lineColor="#333333",this.border1="#9370DB",this.border2="#aaaa33",this.arrowheadColor="#333333",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.labelBackground="#e8e8e8",this.textColor="#333",this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="calculated",this.edgeLabelBackground="calculated",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="black",this.actorLineColor="grey",this.signalColor="calculated",this.signalTextColor="calculated",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="calculated",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="#fff5ad",this.noteTextColor="calculated",this.activationBorderColor="#666",this.activationBkgColor="#f4f4f4",this.sequenceNumberColor="white",this.sectionBkgColor="calculated",this.altSectionBkgColor="calculated",this.sectionBkgColor2="calculated",this.excludeBkgColor="#eeeeee",this.taskBorderColor="calculated",this.taskBkgColor="calculated",this.taskTextLightColor="calculated",this.taskTextColor=this.taskTextLightColor,this.taskTextDarkColor="calculated",this.taskTextOutsideColor=this.taskTextDarkColor,this.taskTextClickableColor="calculated",this.activeTaskBorderColor="calculated",this.activeTaskBkgColor="calculated",this.gridColor="calculated",this.doneTaskBkgColor="calculated",this.doneTaskBorderColor="calculated",this.critBorderColor="calculated",this.critBkgColor="calculated",this.todayLineColor="calculated",this.sectionBkgColor=nI(102,102,255,.49),this.altSectionBkgColor="white",this.sectionBkgColor2="#fff400",this.taskBorderColor="#534fbc",this.taskBkgColor="#8a90dd",this.taskTextLightColor="white",this.taskTextColor="calculated",this.taskTextDarkColor="black",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor="#534fbc",this.activeTaskBkgColor="#bfc7ff",this.gridColor="lightgrey",this.doneTaskBkgColor="lightgrey",this.doneTaskBorderColor="grey",this.critBorderColor="#ff8888",this.critBkgColor="red",this.todayLineColor="red",this.personBorder="calculated",this.personBkg="calculated",this.labelColor="black",this.errorBkgColor="#552222",this.errorTextColor="#552222",this.updateColors()}updateColors(){this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||vn(this.primaryColor,{h:30}),this.cScale4=this.cScale4||vn(this.primaryColor,{h:60}),this.cScale5=this.cScale5||vn(this.primaryColor,{h:90}),this.cScale6=this.cScale6||vn(this.primaryColor,{h:120}),this.cScale7=this.cScale7||vn(this.primaryColor,{h:150}),this.cScale8=this.cScale8||vn(this.primaryColor,{h:210}),this.cScale9=this.cScale9||vn(this.primaryColor,{h:270}),this.cScale10=this.cScale10||vn(this.primaryColor,{h:300}),this.cScale11=this.cScale11||vn(this.primaryColor,{h:330}),this["cScalePeer"+1]=this["cScalePeer"+1]||Ds(this.secondaryColor,45),this["cScalePeer"+2]=this["cScalePeer"+2]||Ds(this.tertiaryColor,40);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScale"+i]=Ds(this["cScale"+i],10),this["cScalePeer"+i]=this["cScalePeer"+i]||Ds(this["cScale"+i],25);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleInv"+i]=this["cScaleInv"+i]||vn(this["cScale"+i],{h:180});for(let i=0;i<5;i++)this["surface"+i]=this["surface"+i]||vn(this.mainBkg,{h:30,l:-(5+i*5)}),this["surfacePeer"+i]=this["surfacePeer"+i]||vn(this.mainBkg,{h:30,l:-(7+i*5)});if(this.scaleLabelColor=this.scaleLabelColor!=="calculated"&&this.scaleLabelColor?this.scaleLabelColor:this.labelTextColor,this.labelTextColor!=="calculated"){this.cScaleLabel0=this.cScaleLabel0||Kr(this.labelTextColor),this.cScaleLabel3=this.cScaleLabel3||Kr(this.labelTextColor);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleLabel"+i]=this["cScaleLabel"+i]||this.labelTextColor}this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.titleColor=this.textColor,this.edgeLabelBackground=this.labelBackground,this.actorBorder=is(this.border1,23),this.actorBkg=this.mainBkg,this.labelBoxBkgColor=this.actorBkg,this.signalColor=this.textColor,this.signalTextColor=this.textColor,this.labelBoxBorderColor=this.actorBorder,this.labelTextColor=this.actorTextColor,this.loopTextColor=this.actorTextColor,this.noteBorderColor=this.border2,this.noteTextColor=this.actorTextColor,this.taskTextColor=this.taskTextLightColor,this.taskTextOutsideColor=this.taskTextDarkColor,this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#f0f0f0",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.nodeBorder,this.specialStateColor=this.lineColor,this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.transitionColor=this.transitionColor||this.lineColor,this.classText=this.primaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=vn(this.primaryColor,{h:64}),this.fillType3=vn(this.secondaryColor,{h:64}),this.fillType4=vn(this.primaryColor,{h:-64}),this.fillType5=vn(this.secondaryColor,{h:-64}),this.fillType6=vn(this.primaryColor,{h:128}),this.fillType7=vn(this.secondaryColor,{h:128}),this.pie1=this.pie1||this.primaryColor,this.pie2=this.pie2||this.secondaryColor,this.pie3=this.pie3||vn(this.tertiaryColor,{l:-40}),this.pie4=this.pie4||vn(this.primaryColor,{l:-10}),this.pie5=this.pie5||vn(this.secondaryColor,{l:-30}),this.pie6=this.pie6||vn(this.tertiaryColor,{l:-20}),this.pie7=this.pie7||vn(this.primaryColor,{h:60,l:-20}),this.pie8=this.pie8||vn(this.primaryColor,{h:-60,l:-40}),this.pie9=this.pie9||vn(this.primaryColor,{h:120,l:-40}),this.pie10=this.pie10||vn(this.primaryColor,{h:60,l:-40}),this.pie11=this.pie11||vn(this.primaryColor,{h:-90,l:-40}),this.pie12=this.pie12||vn(this.primaryColor,{h:120,l:-30}),this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOpacity=this.pieOpacity||"0.7",this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||this.primaryBorderColor,this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||this.labelBackground,this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=this.git0||this.primaryColor,this.git1=this.git1||this.secondaryColor,this.git2=this.git2||this.tertiaryColor,this.git3=this.git3||vn(this.primaryColor,{h:-30}),this.git4=this.git4||vn(this.primaryColor,{h:-60}),this.git5=this.git5||vn(this.primaryColor,{h:-90}),this.git6=this.git6||vn(this.primaryColor,{h:60}),this.git7=this.git7||vn(this.primaryColor,{h:120}),this.darkMode?(this.git0=is(this.git0,25),this.git1=is(this.git1,25),this.git2=is(this.git2,25),this.git3=is(this.git3,25),this.git4=is(this.git4,25),this.git5=is(this.git5,25),this.git6=is(this.git6,25),this.git7=is(this.git7,25)):(this.git0=Ds(this.git0,25),this.git1=Ds(this.git1,25),this.git2=Ds(this.git2,25),this.git3=Ds(this.git3,25),this.git4=Ds(this.git4,25),this.git5=Ds(this.git5,25),this.git6=Ds(this.git6,25),this.git7=Ds(this.git7,25)),this.gitInv0=this.gitInv0||Ds(Kr(this.git0),25),this.gitInv1=this.gitInv1||Kr(this.git1),this.gitInv2=this.gitInv2||Kr(this.git2),this.gitInv3=this.gitInv3||Kr(this.git3),this.gitInv4=this.gitInv4||Kr(this.git4),this.gitInv5=this.gitInv5||Kr(this.git5),this.gitInv6=this.gitInv6||Kr(this.git6),this.gitInv7=this.gitInv7||Kr(this.git7),this.gitBranchLabel0=this.gitBranchLabel0||Kr(this.labelTextColor),this.gitBranchLabel1=this.gitBranchLabel1||this.labelTextColor,this.gitBranchLabel2=this.gitBranchLabel2||this.labelTextColor,this.gitBranchLabel3=this.gitBranchLabel3||Kr(this.labelTextColor),this.gitBranchLabel4=this.gitBranchLabel4||this.labelTextColor,this.gitBranchLabel5=this.gitBranchLabel5||this.labelTextColor,this.gitBranchLabel6=this.gitBranchLabel6||this.labelTextColor,this.gitBranchLabel7=this.gitBranchLabel7||this.labelTextColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||CG,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||SG}calculate(i){if(typeof i!="object"){this.updateColors();return}const o=Object.keys(i);o.forEach(l=>{this[l]=i[l]}),this.updateColors(),o.forEach(l=>{this[l]=i[l]})}}const UPt=r=>{const i=new VPt;return i.calculate(r),i};class qPt{constructor(){this.background="#f4f4f4",this.primaryColor="#cde498",this.secondaryColor="#cdffb2",this.background="white",this.mainBkg="#cde498",this.secondBkg="#cdffb2",this.lineColor="green",this.border1="#13540c",this.border2="#6eaa49",this.arrowheadColor="green",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.tertiaryColor=is("#cde498",10),this.primaryBorderColor=Xd(this.primaryColor,this.darkMode),this.secondaryBorderColor=Xd(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Xd(this.tertiaryColor,this.darkMode),this.primaryTextColor=Kr(this.primaryColor),this.secondaryTextColor=Kr(this.secondaryColor),this.tertiaryTextColor=Kr(this.primaryColor),this.lineColor=Kr(this.background),this.textColor=Kr(this.background),this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="#333",this.edgeLabelBackground="#e8e8e8",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="black",this.actorLineColor="grey",this.signalColor="#333",this.signalTextColor="#333",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="#326932",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="#fff5ad",this.noteTextColor="calculated",this.activationBorderColor="#666",this.activationBkgColor="#f4f4f4",this.sequenceNumberColor="white",this.sectionBkgColor="#6eaa49",this.altSectionBkgColor="white",this.sectionBkgColor2="#6eaa49",this.excludeBkgColor="#eeeeee",this.taskBorderColor="calculated",this.taskBkgColor="#487e3a",this.taskTextLightColor="white",this.taskTextColor="calculated",this.taskTextDarkColor="black",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor="calculated",this.activeTaskBkgColor="calculated",this.gridColor="lightgrey",this.doneTaskBkgColor="lightgrey",this.doneTaskBorderColor="grey",this.critBorderColor="#ff8888",this.critBkgColor="red",this.todayLineColor="red",this.personBorder="calculated",this.personBkg="calculated",this.labelColor="black",this.errorBkgColor="#552222",this.errorTextColor="#552222"}updateColors(){this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||vn(this.primaryColor,{h:30}),this.cScale4=this.cScale4||vn(this.primaryColor,{h:60}),this.cScale5=this.cScale5||vn(this.primaryColor,{h:90}),this.cScale6=this.cScale6||vn(this.primaryColor,{h:120}),this.cScale7=this.cScale7||vn(this.primaryColor,{h:150}),this.cScale8=this.cScale8||vn(this.primaryColor,{h:210}),this.cScale9=this.cScale9||vn(this.primaryColor,{h:270}),this.cScale10=this.cScale10||vn(this.primaryColor,{h:300}),this.cScale11=this.cScale11||vn(this.primaryColor,{h:330}),this["cScalePeer"+1]=this["cScalePeer"+1]||Ds(this.secondaryColor,45),this["cScalePeer"+2]=this["cScalePeer"+2]||Ds(this.tertiaryColor,40);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScale"+i]=Ds(this["cScale"+i],10),this["cScalePeer"+i]=this["cScalePeer"+i]||Ds(this["cScale"+i],25);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleInv"+i]=this["cScaleInv"+i]||vn(this["cScale"+i],{h:180});this.scaleLabelColor=this.scaleLabelColor!=="calculated"&&this.scaleLabelColor?this.scaleLabelColor:this.labelTextColor;for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleLabel"+i]=this["cScaleLabel"+i]||this.scaleLabelColor;for(let i=0;i<5;i++)this["surface"+i]=this["surface"+i]||vn(this.mainBkg,{h:30,s:-30,l:-(5+i*5)}),this["surfacePeer"+i]=this["surfacePeer"+i]||vn(this.mainBkg,{h:30,s:-30,l:-(8+i*5)});this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.actorBorder=Ds(this.mainBkg,20),this.actorBkg=this.mainBkg,this.labelBoxBkgColor=this.actorBkg,this.labelTextColor=this.actorTextColor,this.loopTextColor=this.actorTextColor,this.noteBorderColor=this.border2,this.noteTextColor=this.actorTextColor,this.taskBorderColor=this.border1,this.taskTextColor=this.taskTextLightColor,this.taskTextOutsideColor=this.taskTextDarkColor,this.activeTaskBorderColor=this.taskBorderColor,this.activeTaskBkgColor=this.mainBkg,this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#f0f0f0",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.primaryBorderColor,this.specialStateColor=this.lineColor,this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.transitionColor=this.transitionColor||this.lineColor,this.classText=this.primaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=vn(this.primaryColor,{h:64}),this.fillType3=vn(this.secondaryColor,{h:64}),this.fillType4=vn(this.primaryColor,{h:-64}),this.fillType5=vn(this.secondaryColor,{h:-64}),this.fillType6=vn(this.primaryColor,{h:128}),this.fillType7=vn(this.secondaryColor,{h:128}),this.pie1=this.pie1||this.primaryColor,this.pie2=this.pie2||this.secondaryColor,this.pie3=this.pie3||this.tertiaryColor,this.pie4=this.pie4||vn(this.primaryColor,{l:-30}),this.pie5=this.pie5||vn(this.secondaryColor,{l:-30}),this.pie6=this.pie6||vn(this.tertiaryColor,{h:40,l:-40}),this.pie7=this.pie7||vn(this.primaryColor,{h:60,l:-10}),this.pie8=this.pie8||vn(this.primaryColor,{h:-60,l:-10}),this.pie9=this.pie9||vn(this.primaryColor,{h:120,l:0}),this.pie10=this.pie10||vn(this.primaryColor,{h:60,l:-50}),this.pie11=this.pie11||vn(this.primaryColor,{h:-60,l:-50}),this.pie12=this.pie12||vn(this.primaryColor,{h:120,l:-50}),this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOpacity=this.pieOpacity||"0.7",this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||this.primaryBorderColor,this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||this.edgeLabelBackground,this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=this.git0||this.primaryColor,this.git1=this.git1||this.secondaryColor,this.git2=this.git2||this.tertiaryColor,this.git3=this.git3||vn(this.primaryColor,{h:-30}),this.git4=this.git4||vn(this.primaryColor,{h:-60}),this.git5=this.git5||vn(this.primaryColor,{h:-90}),this.git6=this.git6||vn(this.primaryColor,{h:60}),this.git7=this.git7||vn(this.primaryColor,{h:120}),this.darkMode?(this.git0=is(this.git0,25),this.git1=is(this.git1,25),this.git2=is(this.git2,25),this.git3=is(this.git3,25),this.git4=is(this.git4,25),this.git5=is(this.git5,25),this.git6=is(this.git6,25),this.git7=is(this.git7,25)):(this.git0=Ds(this.git0,25),this.git1=Ds(this.git1,25),this.git2=Ds(this.git2,25),this.git3=Ds(this.git3,25),this.git4=Ds(this.git4,25),this.git5=Ds(this.git5,25),this.git6=Ds(this.git6,25),this.git7=Ds(this.git7,25)),this.gitInv0=this.gitInv0||Kr(this.git0),this.gitInv1=this.gitInv1||Kr(this.git1),this.gitInv2=this.gitInv2||Kr(this.git2),this.gitInv3=this.gitInv3||Kr(this.git3),this.gitInv4=this.gitInv4||Kr(this.git4),this.gitInv5=this.gitInv5||Kr(this.git5),this.gitInv6=this.gitInv6||Kr(this.git6),this.gitInv7=this.gitInv7||Kr(this.git7),this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||CG,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||SG}calculate(i){if(typeof i!="object"){this.updateColors();return}const o=Object.keys(i);o.forEach(l=>{this[l]=i[l]}),this.updateColors(),o.forEach(l=>{this[l]=i[l]})}}const YPt=r=>{const i=new qPt;return i.calculate(r),i};class WPt{constructor(){this.primaryColor="#eee",this.contrast="#707070",this.secondaryColor=is(this.contrast,55),this.background="#ffffff",this.tertiaryColor=vn(this.primaryColor,{h:-160}),this.primaryBorderColor=Xd(this.primaryColor,this.darkMode),this.secondaryBorderColor=Xd(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Xd(this.tertiaryColor,this.darkMode),this.primaryTextColor=Kr(this.primaryColor),this.secondaryTextColor=Kr(this.secondaryColor),this.tertiaryTextColor=Kr(this.tertiaryColor),this.lineColor=Kr(this.background),this.textColor=Kr(this.background),this.mainBkg="#eee",this.secondBkg="calculated",this.lineColor="#666",this.border1="#999",this.border2="calculated",this.note="#ffa",this.text="#333",this.critical="#d42",this.done="#bbb",this.arrowheadColor="#333333",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="calculated",this.edgeLabelBackground="white",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="calculated",this.actorLineColor="calculated",this.signalColor="calculated",this.signalTextColor="calculated",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="calculated",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="calculated",this.noteTextColor="calculated",this.activationBorderColor="#666",this.activationBkgColor="#f4f4f4",this.sequenceNumberColor="white",this.sectionBkgColor="calculated",this.altSectionBkgColor="white",this.sectionBkgColor2="calculated",this.excludeBkgColor="#eeeeee",this.taskBorderColor="calculated",this.taskBkgColor="calculated",this.taskTextLightColor="white",this.taskTextColor="calculated",this.taskTextDarkColor="calculated",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor="calculated",this.activeTaskBkgColor="calculated",this.gridColor="calculated",this.doneTaskBkgColor="calculated",this.doneTaskBorderColor="calculated",this.critBkgColor="calculated",this.critBorderColor="calculated",this.todayLineColor="calculated",this.personBorder="calculated",this.personBkg="calculated",this.labelColor="black",this.errorBkgColor="#552222",this.errorTextColor="#552222"}updateColors(){this.secondBkg=is(this.contrast,55),this.border2=this.contrast,this.cScale0=this.cScale0||"#555",this.cScale1=this.cScale1||"#F4F4F4",this.cScale2=this.cScale2||"#555",this.cScale3=this.cScale3||"#BBB",this.cScale4=this.cScale4||"#777",this.cScale5=this.cScale5||"#999",this.cScale6=this.cScale6||"#DDD",this.cScale7=this.cScale7||"#FFF",this.cScale8=this.cScale8||"#DDD",this.cScale9=this.cScale9||"#BBB",this.cScale10=this.cScale10||"#999",this.cScale11=this.cScale11||"#777";for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleInv"+i]=this["cScaleInv"+i]||Kr(this["cScale"+i]);for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this.darkMode?this["cScalePeer"+i]=this["cScalePeer"+i]||is(this["cScale"+i],10):this["cScalePeer"+i]=this["cScalePeer"+i]||Ds(this["cScale"+i],10);this.scaleLabelColor=this.scaleLabelColor||(this.darkMode?"black":this.labelTextColor),this.cScaleLabel0=this.cScaleLabel0||this.cScale1,this.cScaleLabel2=this.cScaleLabel2||this.cScale1;for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["cScaleLabel"+i]=this["cScaleLabel"+i]||this.scaleLabelColor;for(let i=0;i<5;i++)this["surface"+i]=this["surface"+i]||vn(this.mainBkg,{l:-(5+i*5)}),this["surfacePeer"+i]=this["surfacePeer"+i]||vn(this.mainBkg,{l:-(8+i*5)});this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.titleColor=this.text,this.actorBorder=is(this.border1,23),this.actorBkg=this.mainBkg,this.actorTextColor=this.text,this.actorLineColor=this.lineColor,this.signalColor=this.text,this.signalTextColor=this.text,this.labelBoxBkgColor=this.actorBkg,this.labelBoxBorderColor=this.actorBorder,this.labelTextColor=this.text,this.loopTextColor=this.text,this.noteBorderColor="#999",this.noteBkgColor="#666",this.noteTextColor="#fff",this.sectionBkgColor=is(this.contrast,30),this.sectionBkgColor2=is(this.contrast,30),this.taskBorderColor=Ds(this.contrast,10),this.taskBkgColor=this.contrast,this.taskTextColor=this.taskTextLightColor,this.taskTextDarkColor=this.text,this.taskTextOutsideColor=this.taskTextDarkColor,this.activeTaskBorderColor=this.taskBorderColor,this.activeTaskBkgColor=this.mainBkg,this.gridColor=is(this.border1,30),this.doneTaskBkgColor=this.done,this.doneTaskBorderColor=this.lineColor,this.critBkgColor=this.critical,this.critBorderColor=Ds(this.critBkgColor,10),this.todayLineColor=this.critBkgColor,this.transitionColor=this.transitionColor||"#000",this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#f4f4f4",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.stateBorder=this.stateBorder||"#000",this.innerEndBackground=this.primaryBorderColor,this.specialStateColor="#222",this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.classText=this.primaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=vn(this.primaryColor,{h:64}),this.fillType3=vn(this.secondaryColor,{h:64}),this.fillType4=vn(this.primaryColor,{h:-64}),this.fillType5=vn(this.secondaryColor,{h:-64}),this.fillType6=vn(this.primaryColor,{h:128}),this.fillType7=vn(this.secondaryColor,{h:128});for(let i=0;i<this.THEME_COLOR_LIMIT;i++)this["pie"+i]=this["cScale"+i];this.pie12=this.pie0,this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOpacity=this.pieOpacity||"0.7",this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||this.primaryBorderColor,this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||this.edgeLabelBackground,this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=Ds(this.pie1,25)||this.primaryColor,this.git1=this.pie2||this.secondaryColor,this.git2=this.pie3||this.tertiaryColor,this.git3=this.pie4||vn(this.primaryColor,{h:-30}),this.git4=this.pie5||vn(this.primaryColor,{h:-60}),this.git5=this.pie6||vn(this.primaryColor,{h:-90}),this.git6=this.pie7||vn(this.primaryColor,{h:60}),this.git7=this.pie8||vn(this.primaryColor,{h:120}),this.gitInv0=this.gitInv0||Kr(this.git0),this.gitInv1=this.gitInv1||Kr(this.git1),this.gitInv2=this.gitInv2||Kr(this.git2),this.gitInv3=this.gitInv3||Kr(this.git3),this.gitInv4=this.gitInv4||Kr(this.git4),this.gitInv5=this.gitInv5||Kr(this.git5),this.gitInv6=this.gitInv6||Kr(this.git6),this.gitInv7=this.gitInv7||Kr(this.git7),this.branchLabelColor=this.branchLabelColor||this.labelTextColor,this.gitBranchLabel0=this.branchLabelColor,this.gitBranchLabel1="white",this.gitBranchLabel2=this.branchLabelColor,this.gitBranchLabel3="white",this.gitBranchLabel4=this.branchLabelColor,this.gitBranchLabel5=this.branchLabelColor,this.gitBranchLabel6=this.branchLabelColor,this.gitBranchLabel7=this.branchLabelColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||CG,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||SG}calculate(i){if(typeof i!="object"){this.updateColors();return}const o=Object.keys(i);o.forEach(l=>{this[l]=i[l]}),this.updateColors(),o.forEach(l=>{this[l]=i[l]})}}const q3={base:{getThemeVariables:HPt},dark:{getThemeVariables:GPt},default:{getThemeVariables:UPt},forest:{getThemeVariables:YPt},neutral:{getThemeVariables:r=>{const i=new WPt;return i.calculate(r),i}}},u6={theme:"default",themeVariables:q3.default.getThemeVariables(),themeCSS:void 0,maxTextSize:5e4,darkMode:!1,fontFamily:'"trebuchet ms", verdana, arial, sans-serif;',logLevel:5,securityLevel:"strict",startOnLoad:!0,arrowMarkerAbsolute:!1,secure:["secure","securityLevel","startOnLoad","maxTextSize"],deterministicIds:!1,deterministicIDSeed:void 0,flowchart:{titleTopMargin:25,diagramPadding:8,htmlLabels:!0,nodeSpacing:50,rankSpacing:50,curve:"basis",padding:15,useMaxWidth:!0,defaultRenderer:"dagre-wrapper"},sequence:{hideUnusedParticipants:!1,activationWidth:10,diagramMarginX:50,diagramMarginY:10,actorMargin:50,width:150,height:65,boxMargin:10,boxTextMargin:5,noteMargin:10,messageMargin:35,messageAlign:"center",mirrorActors:!0,forceMenus:!1,bottomMarginAdj:1,useMaxWidth:!0,rightAngles:!1,showSequenceNumbers:!1,actorFontSize:14,actorFontFamily:'"Open Sans", sans-serif',actorFontWeight:400,noteFontSize:14,noteFontFamily:'"trebuchet ms", verdana, arial, sans-serif',noteFontWeight:400,noteAlign:"center",messageFontSize:16,messageFontFamily:'"trebuchet ms", verdana, arial, sans-serif',messageFontWeight:400,wrap:!1,wrapPadding:10,labelBoxWidth:50,labelBoxHeight:20,messageFont:function(){return{fontFamily:this.messageFontFamily,fontSize:this.messageFontSize,fontWeight:this.messageFontWeight}},noteFont:function(){return{fontFamily:this.noteFontFamily,fontSize:this.noteFontSize,fontWeight:this.noteFontWeight}},actorFont:function(){return{fontFamily:this.actorFontFamily,fontSize:this.actorFontSize,fontWeight:this.actorFontWeight}}},gantt:{titleTopMargin:25,barHeight:20,barGap:4,topPadding:50,rightPadding:75,leftPadding:75,gridLineStartPadding:35,fontSize:11,sectionFontSize:11,numberSectionStyles:4,axisFormat:"%Y-%m-%d",tickInterval:void 0,useMaxWidth:!0,topAxis:!1,useWidth:void 0},journey:{diagramMarginX:50,diagramMarginY:10,leftMargin:150,width:150,height:50,boxMargin:10,boxTextMargin:5,noteMargin:10,messageMargin:35,messageAlign:"center",bottomMarginAdj:1,useMaxWidth:!0,rightAngles:!1,taskFontSize:14,taskFontFamily:'"Open Sans", sans-serif',taskMargin:50,activationWidth:10,textPlacement:"fo",actorColours:["#8FBC8F","#7CFC00","#00FFFF","#20B2AA","#B0E0E6","#FFFFE0"],sectionFills:["#191970","#8B008B","#4B0082","#2F4F4F","#800000","#8B4513","#00008B"],sectionColours:["#fff"]},timeline:{diagramMarginX:50,diagramMarginY:10,leftMargin:150,width:150,height:50,boxMargin:10,boxTextMargin:5,noteMargin:10,messageMargin:35,messageAlign:"center",bottomMarginAdj:1,useMaxWidth:!0,rightAngles:!1,taskFontSize:14,taskFontFamily:'"Open Sans", sans-serif',taskMargin:50,activationWidth:10,textPlacement:"fo",actorColours:["#8FBC8F","#7CFC00","#00FFFF","#20B2AA","#B0E0E6","#FFFFE0"],sectionFills:["#191970","#8B008B","#4B0082","#2F4F4F","#800000","#8B4513","#00008B"],sectionColours:["#fff"],disableMulticolor:!1},class:{titleTopMargin:25,arrowMarkerAbsolute:!1,dividerMargin:10,padding:5,textHeight:10,useMaxWidth:!0,defaultRenderer:"dagre-wrapper"},state:{titleTopMargin:25,dividerMargin:10,sizeUnit:5,padding:8,textHeight:10,titleShift:-15,noteMargin:10,forkWidth:70,forkHeight:7,miniPadding:2,fontSizeFactor:5.02,fontSize:24,labelHeight:16,edgeLengthFactor:"20",compositTitleSize:35,radius:5,useMaxWidth:!0,defaultRenderer:"dagre-wrapper"},er:{titleTopMargin:25,diagramPadding:20,layoutDirection:"TB",minEntityWidth:100,minEntityHeight:75,entityPadding:15,stroke:"gray",fill:"honeydew",fontSize:12,useMaxWidth:!0},pie:{useWidth:void 0,useMaxWidth:!0},requirement:{useWidth:void 0,useMaxWidth:!0,rect_fill:"#f9f9f9",text_color:"#333",rect_border_size:"0.5px",rect_border_color:"#bbb",rect_min_width:200,rect_min_height:200,fontSize:14,rect_padding:10,line_height:20},gitGraph:{titleTopMargin:25,diagramPadding:8,nodeLabel:{width:75,height:100,x:-25,y:0},mainBranchName:"main",mainBranchOrder:0,showCommitLabel:!0,showBranches:!0,rotateCommitLabel:!0},c4:{useWidth:void 0,diagramMarginX:50,diagramMarginY:10,c4ShapeMargin:50,c4ShapePadding:20,width:216,height:60,boxMargin:10,useMaxWidth:!0,c4ShapeInRow:4,nextLinePaddingX:0,c4BoundaryInRow:2,personFontSize:14,personFontFamily:'"Open Sans", sans-serif',personFontWeight:"normal",external_personFontSize:14,external_personFontFamily:'"Open Sans", sans-serif',external_personFontWeight:"normal",systemFontSize:14,systemFontFamily:'"Open Sans", sans-serif',systemFontWeight:"normal",external_systemFontSize:14,external_systemFontFamily:'"Open Sans", sans-serif',external_systemFontWeight:"normal",system_dbFontSize:14,system_dbFontFamily:'"Open Sans", sans-serif',system_dbFontWeight:"normal",external_system_dbFontSize:14,external_system_dbFontFamily:'"Open Sans", sans-serif',external_system_dbFontWeight:"normal",system_queueFontSize:14,system_queueFontFamily:'"Open Sans", sans-serif',system_queueFontWeight:"normal",external_system_queueFontSize:14,external_system_queueFontFamily:'"Open Sans", sans-serif',external_system_queueFontWeight:"normal",boundaryFontSize:14,boundaryFontFamily:'"Open Sans", sans-serif',boundaryFontWeight:"normal",messageFontSize:12,messageFontFamily:'"Open Sans", sans-serif',messageFontWeight:"normal",containerFontSize:14,containerFontFamily:'"Open Sans", sans-serif',containerFontWeight:"normal",external_containerFontSize:14,external_containerFontFamily:'"Open Sans", sans-serif',external_containerFontWeight:"normal",container_dbFontSize:14,container_dbFontFamily:'"Open Sans", sans-serif',container_dbFontWeight:"normal",external_container_dbFontSize:14,external_container_dbFontFamily:'"Open Sans", sans-serif',external_container_dbFontWeight:"normal",container_queueFontSize:14,container_queueFontFamily:'"Open Sans", sans-serif',container_queueFontWeight:"normal",external_container_queueFontSize:14,external_container_queueFontFamily:'"Open Sans", sans-serif',external_container_queueFontWeight:"normal",componentFontSize:14,componentFontFamily:'"Open Sans", sans-serif',componentFontWeight:"normal",external_componentFontSize:14,external_componentFontFamily:'"Open Sans", sans-serif',external_componentFontWeight:"normal",component_dbFontSize:14,component_dbFontFamily:'"Open Sans", sans-serif',component_dbFontWeight:"normal",external_component_dbFontSize:14,external_component_dbFontFamily:'"Open Sans", sans-serif',external_component_dbFontWeight:"normal",component_queueFontSize:14,component_queueFontFamily:'"Open Sans", sans-serif',component_queueFontWeight:"normal",external_component_queueFontSize:14,external_component_queueFontFamily:'"Open Sans", sans-serif',external_component_queueFontWeight:"normal",wrap:!0,wrapPadding:10,personFont:function(){return{fontFamily:this.personFontFamily,fontSize:this.personFontSize,fontWeight:this.personFontWeight}},external_personFont:function(){return{fontFamily:this.external_personFontFamily,fontSize:this.external_personFontSize,fontWeight:this.external_personFontWeight}},systemFont:function(){return{fontFamily:this.systemFontFamily,fontSize:this.systemFontSize,fontWeight:this.systemFontWeight}},external_systemFont:function(){return{fontFamily:this.external_systemFontFamily,fontSize:this.external_systemFontSize,fontWeight:this.external_systemFontWeight}},system_dbFont:function(){return{fontFamily:this.system_dbFontFamily,fontSize:this.system_dbFontSize,fontWeight:this.system_dbFontWeight}},external_system_dbFont:function(){return{fontFamily:this.external_system_dbFontFamily,fontSize:this.external_system_dbFontSize,fontWeight:this.external_system_dbFontWeight}},system_queueFont:function(){return{fontFamily:this.system_queueFontFamily,fontSize:this.system_queueFontSize,fontWeight:this.system_queueFontWeight}},external_system_queueFont:function(){return{fontFamily:this.external_system_queueFontFamily,fontSize:this.external_system_queueFontSize,fontWeight:this.external_system_queueFontWeight}},containerFont:function(){return{fontFamily:this.containerFontFamily,fontSize:this.containerFontSize,fontWeight:this.containerFontWeight}},external_containerFont:function(){return{fontFamily:this.external_containerFontFamily,fontSize:this.external_containerFontSize,fontWeight:this.external_containerFontWeight}},container_dbFont:function(){return{fontFamily:this.container_dbFontFamily,fontSize:this.container_dbFontSize,fontWeight:this.container_dbFontWeight}},external_container_dbFont:function(){return{fontFamily:this.external_container_dbFontFamily,fontSize:this.external_container_dbFontSize,fontWeight:this.external_container_dbFontWeight}},container_queueFont:function(){return{fontFamily:this.container_queueFontFamily,fontSize:this.container_queueFontSize,fontWeight:this.container_queueFontWeight}},external_container_queueFont:function(){return{fontFamily:this.external_container_queueFontFamily,fontSize:this.external_container_queueFontSize,fontWeight:this.external_container_queueFontWeight}},componentFont:function(){return{fontFamily:this.componentFontFamily,fontSize:this.componentFontSize,fontWeight:this.componentFontWeight}},external_componentFont:function(){return{fontFamily:this.external_componentFontFamily,fontSize:this.external_componentFontSize,fontWeight:this.external_componentFontWeight}},component_dbFont:function(){return{fontFamily:this.component_dbFontFamily,fontSize:this.component_dbFontSize,fontWeight:this.component_dbFontWeight}},external_component_dbFont:function(){return{fontFamily:this.external_component_dbFontFamily,fontSize:this.external_component_dbFontSize,fontWeight:this.external_component_dbFontWeight}},component_queueFont:function(){return{fontFamily:this.component_queueFontFamily,fontSize:this.component_queueFontSize,fontWeight:this.component_queueFontWeight}},external_component_queueFont:function(){return{fontFamily:this.external_component_queueFontFamily,fontSize:this.external_component_queueFontSize,fontWeight:this.external_component_queueFontWeight}},boundaryFont:function(){return{fontFamily:this.boundaryFontFamily,fontSize:this.boundaryFontSize,fontWeight:this.boundaryFontWeight}},messageFont:function(){return{fontFamily:this.messageFontFamily,fontSize:this.messageFontSize,fontWeight:this.messageFontWeight}},person_bg_color:"#08427B",person_border_color:"#073B6F",external_person_bg_color:"#686868",external_person_border_color:"#8A8A8A",system_bg_color:"#1168BD",system_border_color:"#3C7FC0",system_db_bg_color:"#1168BD",system_db_border_color:"#3C7FC0",system_queue_bg_color:"#1168BD",system_queue_border_color:"#3C7FC0",external_system_bg_color:"#999999",external_system_border_color:"#8A8A8A",external_system_db_bg_color:"#999999",external_system_db_border_color:"#8A8A8A",external_system_queue_bg_color:"#999999",external_system_queue_border_color:"#8A8A8A",container_bg_color:"#438DD5",container_border_color:"#3C7FC0",container_db_bg_color:"#438DD5",container_db_border_color:"#3C7FC0",container_queue_bg_color:"#438DD5",container_queue_border_color:"#3C7FC0",external_container_bg_color:"#B3B3B3",external_container_border_color:"#A6A6A6",external_container_db_bg_color:"#B3B3B3",external_container_db_border_color:"#A6A6A6",external_container_queue_bg_color:"#B3B3B3",external_container_queue_border_color:"#A6A6A6",component_bg_color:"#85BBF0",component_border_color:"#78A8D8",component_db_bg_color:"#85BBF0",component_db_border_color:"#78A8D8",component_queue_bg_color:"#85BBF0",component_queue_border_color:"#78A8D8",external_component_bg_color:"#CCCCCC",external_component_border_color:"#BFBFBF",external_component_db_bg_color:"#CCCCCC",external_component_db_border_color:"#BFBFBF",external_component_queue_bg_color:"#CCCCCC",external_component_queue_border_color:"#BFBFBF"},mindmap:{useMaxWidth:!0,padding:10,maxNodeWidth:200},fontSize:16};u6.class&&(u6.class.arrowMarkerAbsolute=u6.arrowMarkerAbsolute),u6.gitGraph&&(u6.gitGraph.arrowMarkerAbsolute=u6.arrowMarkerAbsolute);const gCe=(r,i="")=>Object.keys(r).reduce((o,l)=>Array.isArray(r[l])?o:typeof r[l]=="object"&&r[l]!==null?[...o,i+l,...gCe(r[l],"")]:[...o,i+l],[]),KPt=gCe(u6,""),XPt=u6;/*! js-yaml 4.1.0 https://github.com/nodeca/js-yaml @license MIT */function pCe(r){return typeof r>"u"||r===null}function QPt(r){return typeof r=="object"&&r!==null}function ZPt(r){return Array.isArray(r)?r:pCe(r)?[]:[r]}function JPt(r,i){var o,l,f,b;if(i)for(b=Object.keys(i),o=0,l=b.length;o<l;o+=1)f=b[o],r[f]=i[f];return r}function eBt(r,i){var o="",l;for(l=0;l<i;l+=1)o+=r;return o}function tBt(r){return r===0&&Number.NEGATIVE_INFINITY===1/r}var nBt=pCe,rBt=QPt,iBt=ZPt,sBt=eBt,aBt=tBt,oBt=JPt,Qd={isNothing:nBt,isObject:rBt,toArray:iBt,repeat:sBt,isNegativeZero:aBt,extend:oBt};function bCe(r,i){var o="",l=r.reason||"(unknown reason)";return r.mark?(r.mark.name&&(o+='in "'+r.mark.name+'" '),o+="("+(r.mark.line+1)+":"+(r.mark.column+1)+")",!i&&r.mark.snippet&&(o+=`

`+r.mark.snippet),l+" "+o):l}function rI(r,i){Error.call(this),this.name="YAMLException",this.reason=r,this.mark=i,this.message=bCe(this,!1),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=new Error().stack||""}rI.prototype=Object.create(Error.prototype),rI.prototype.constructor=rI,rI.prototype.toString=function(i){return this.name+": "+bCe(this,i)};var Y3=rI;function rae(r,i,o,l,f){var b="",d="",w=Math.floor(f/2)-1;return l-i>w&&(b=" ... ",i=l-w+b.length),o-l>w&&(d=" ...",o=l+w-d.length),{str:b+r.slice(i,o).replace(/\t/g,"\u2192")+d,pos:l-i+b.length}}function iae(r,i){return Qd.repeat(" ",i-r.length)+r}function cBt(r,i){if(i=Object.create(i||null),!r.buffer)return null;i.maxLength||(i.maxLength=79),typeof i.indent!="number"&&(i.indent=1),typeof i.linesBefore!="number"&&(i.linesBefore=3),typeof i.linesAfter!="number"&&(i.linesAfter=2);for(var o=/\r?\n|\r|\0/g,l=[0],f=[],b,d=-1;b=o.exec(r.buffer);)f.push(b.index),l.push(b.index+b[0].length),r.position<=b.index&&d<0&&(d=l.length-2);d<0&&(d=l.length-1);var w="",y,k,E=Math.min(r.line+i.linesAfter,f.length).toString().length,T=i.maxLength-(i.indent+E+3);for(y=1;y<=i.linesBefore&&!(d-y<0);y++)k=rae(r.buffer,l[d-y],f[d-y],r.position-(l[d]-l[d-y]),T),w=Qd.repeat(" ",i.indent)+iae((r.line-y+1).toString(),E)+" | "+k.str+`
`+w;for(k=rae(r.buffer,l[d],f[d],r.position,T),w+=Qd.repeat(" ",i.indent)+iae((r.line+1).toString(),E)+" | "+k.str+`
`,w+=Qd.repeat("-",i.indent+E+3+k.pos)+`^
`,y=1;y<=i.linesAfter&&!(d+y>=f.length);y++)k=rae(r.buffer,l[d+y],f[d+y],r.position-(l[d]-l[d+y]),T),w+=Qd.repeat(" ",i.indent)+iae((r.line+y+1).toString(),E)+" | "+k.str+`
`;return w.replace(/\n$/,"")}var uBt=cBt,lBt=["kind","multi","resolve","construct","instanceOf","predicate","represent","representName","defaultStyle","styleAliases"],hBt=["scalar","sequence","mapping"];function fBt(r){var i={};return r!==null&&Object.keys(r).forEach(function(o){r[o].forEach(function(l){i[String(l)]=o})}),i}function dBt(r,i){if(i=i||{},Object.keys(i).forEach(function(o){if(lBt.indexOf(o)===-1)throw new Y3('Unknown option "'+o+'" is met in definition of "'+r+'" YAML type.')}),this.options=i,this.tag=r,this.kind=i.kind||null,this.resolve=i.resolve||function(){return!0},this.construct=i.construct||function(o){return o},this.instanceOf=i.instanceOf||null,this.predicate=i.predicate||null,this.represent=i.represent||null,this.representName=i.representName||null,this.defaultStyle=i.defaultStyle||null,this.multi=i.multi||!1,this.styleAliases=fBt(i.styleAliases||null),hBt.indexOf(this.kind)===-1)throw new Y3('Unknown kind "'+this.kind+'" is specified for "'+r+'" YAML type.')}var ud=dBt;function vCe(r,i){var o=[];return r[i].forEach(function(l){var f=o.length;o.forEach(function(b,d){b.tag===l.tag&&b.kind===l.kind&&b.multi===l.multi&&(f=d)}),o[f]=l}),o}function gBt(){var r={scalar:{},sequence:{},mapping:{},fallback:{},multi:{scalar:[],sequence:[],mapping:[],fallback:[]}},i,o;function l(f){f.multi?(r.multi[f.kind].push(f),r.multi.fallback.push(f)):r[f.kind][f.tag]=r.fallback[f.tag]=f}for(i=0,o=arguments.length;i<o;i+=1)arguments[i].forEach(l);return r}function sae(r){return this.extend(r)}sae.prototype.extend=function(i){var o=[],l=[];if(i instanceof ud)l.push(i);else if(Array.isArray(i))l=l.concat(i);else if(i&&(Array.isArray(i.implicit)||Array.isArray(i.explicit)))i.implicit&&(o=o.concat(i.implicit)),i.explicit&&(l=l.concat(i.explicit));else throw new Y3("Schema.extend argument should be a Type, [ Type ], or a schema definition ({ implicit: [...], explicit: [...] })");o.forEach(function(b){if(!(b instanceof ud))throw new Y3("Specified list of YAML types (or a single Type object) contains a non-Type object.");if(b.loadKind&&b.loadKind!=="scalar")throw new Y3("There is a non-scalar type in the implicit list of a schema. Implicit resolving of such types is not supported.");if(b.multi)throw new Y3("There is a multi type in the implicit list of a schema. Multi tags can only be listed as explicit.")}),l.forEach(function(b){if(!(b instanceof ud))throw new Y3("Specified list of YAML types (or a single Type object) contains a non-Type object.")});var f=Object.create(sae.prototype);return f.implicit=(this.implicit||[]).concat(o),f.explicit=(this.explicit||[]).concat(l),f.compiledImplicit=vCe(f,"implicit"),f.compiledExplicit=vCe(f,"explicit"),f.compiledTypeMap=gBt(f.compiledImplicit,f.compiledExplicit),f};var pBt=sae,bBt=new ud("tag:yaml.org,2002:str",{kind:"scalar",construct:function(r){return r!==null?r:""}}),vBt=new ud("tag:yaml.org,2002:seq",{kind:"sequence",construct:function(r){return r!==null?r:[]}}),wBt=new ud("tag:yaml.org,2002:map",{kind:"mapping",construct:function(r){return r!==null?r:{}}}),wCe=new pBt({explicit:[bBt,vBt,wBt]});function mBt(r){if(r===null)return!0;var i=r.length;return i===1&&r==="~"||i===4&&(r==="null"||r==="Null"||r==="NULL")}function yBt(){return null}function kBt(r){return r===null}var xBt=new ud("tag:yaml.org,2002:null",{kind:"scalar",resolve:mBt,construct:yBt,predicate:kBt,represent:{canonical:function(){return"~"},lowercase:function(){return"null"},uppercase:function(){return"NULL"},camelcase:function(){return"Null"},empty:function(){return""}},defaultStyle:"lowercase"});function EBt(r){if(r===null)return!1;var i=r.length;return i===4&&(r==="true"||r==="True"||r==="TRUE")||i===5&&(r==="false"||r==="False"||r==="FALSE")}function _Bt(r){return r==="true"||r==="True"||r==="TRUE"}function TBt(r){return Object.prototype.toString.call(r)==="[object Boolean]"}var CBt=new ud("tag:yaml.org,2002:bool",{kind:"scalar",resolve:EBt,construct:_Bt,predicate:TBt,represent:{lowercase:function(r){return r?"true":"false"},uppercase:function(r){return r?"TRUE":"FALSE"},camelcase:function(r){return r?"True":"False"}},defaultStyle:"lowercase"});function SBt(r){return 48<=r&&r<=57||65<=r&&r<=70||97<=r&&r<=102}function ABt(r){return 48<=r&&r<=55}function MBt(r){return 48<=r&&r<=57}function DBt(r){if(r===null)return!1;var i=r.length,o=0,l=!1,f;if(!i)return!1;if(f=r[o],(f==="-"||f==="+")&&(f=r[++o]),f==="0"){if(o+1===i)return!0;if(f=r[++o],f==="b"){for(o++;o<i;o++)if(f=r[o],f!=="_"){if(f!=="0"&&f!=="1")return!1;l=!0}return l&&f!=="_"}if(f==="x"){for(o++;o<i;o++)if(f=r[o],f!=="_"){if(!SBt(r.charCodeAt(o)))return!1;l=!0}return l&&f!=="_"}if(f==="o"){for(o++;o<i;o++)if(f=r[o],f!=="_"){if(!ABt(r.charCodeAt(o)))return!1;l=!0}return l&&f!=="_"}}if(f==="_")return!1;for(;o<i;o++)if(f=r[o],f!=="_"){if(!MBt(r.charCodeAt(o)))return!1;l=!0}return!(!l||f==="_")}function LBt(r){var i=r,o=1,l;if(i.indexOf("_")!==-1&&(i=i.replace(/_/g,"")),l=i[0],(l==="-"||l==="+")&&(l==="-"&&(o=-1),i=i.slice(1),l=i[0]),i==="0")return 0;if(l==="0"){if(i[1]==="b")return o*parseInt(i.slice(2),2);if(i[1]==="x")return o*parseInt(i.slice(2),16);if(i[1]==="o")return o*parseInt(i.slice(2),8)}return o*parseInt(i,10)}function IBt(r){return Object.prototype.toString.call(r)==="[object Number]"&&r%1===0&&!Qd.isNegativeZero(r)}var OBt=new ud("tag:yaml.org,2002:int",{kind:"scalar",resolve:DBt,construct:LBt,predicate:IBt,represent:{binary:function(r){return r>=0?"0b"+r.toString(2):"-0b"+r.toString(2).slice(1)},octal:function(r){return r>=0?"0o"+r.toString(8):"-0o"+r.toString(8).slice(1)},decimal:function(r){return r.toString(10)},hexadecimal:function(r){return r>=0?"0x"+r.toString(16).toUpperCase():"-0x"+r.toString(16).toUpperCase().slice(1)}},defaultStyle:"decimal",styleAliases:{binary:[2,"bin"],octal:[8,"oct"],decimal:[10,"dec"],hexadecimal:[16,"hex"]}}),NBt=new RegExp("^(?:[-+]?(?:[0-9][0-9_]*)(?:\\.[0-9_]*)?(?:[eE][-+]?[0-9]+)?|\\.[0-9_]+(?:[eE][-+]?[0-9]+)?|[-+]?\\.(?:inf|Inf|INF)|\\.(?:nan|NaN|NAN))$");function PBt(r){return!(r===null||!NBt.test(r)||r[r.length-1]==="_")}function BBt(r){var i,o;return i=r.replace(/_/g,"").toLowerCase(),o=i[0]==="-"?-1:1,"+-".indexOf(i[0])>=0&&(i=i.slice(1)),i===".inf"?o===1?Number.POSITIVE_INFINITY:Number.NEGATIVE_INFINITY:i===".nan"?NaN:o*parseFloat(i,10)}var FBt=/^[-+]?[0-9]+e/;function RBt(r,i){var o;if(isNaN(r))switch(i){case"lowercase":return".nan";case"uppercase":return".NAN";case"camelcase":return".NaN"}else if(Number.POSITIVE_INFINITY===r)switch(i){case"lowercase":return".inf";case"uppercase":return".INF";case"camelcase":return".Inf"}else if(Number.NEGATIVE_INFINITY===r)switch(i){case"lowercase":return"-.inf";case"uppercase":return"-.INF";case"camelcase":return"-.Inf"}else if(Qd.isNegativeZero(r))return"-0.0";return o=r.toString(10),FBt.test(o)?o.replace("e",".e"):o}function jBt(r){return Object.prototype.toString.call(r)==="[object Number]"&&(r%1!==0||Qd.isNegativeZero(r))}var $Bt=new ud("tag:yaml.org,2002:float",{kind:"scalar",resolve:PBt,construct:BBt,predicate:jBt,represent:RBt,defaultStyle:"lowercase"}),HBt=wCe.extend({implicit:[xBt,CBt,OBt,$Bt]}),zBt=HBt,mCe=new RegExp("^([0-9][0-9][0-9][0-9])-([0-9][0-9])-([0-9][0-9])$"),yCe=new RegExp("^([0-9][0-9][0-9][0-9])-([0-9][0-9]?)-([0-9][0-9]?)(?:[Tt]|[ \\t]+)([0-9][0-9]?):([0-9][0-9]):([0-9][0-9])(?:\\.([0-9]*))?(?:[ \\t]*(Z|([-+])([0-9][0-9]?)(?::([0-9][0-9]))?))?$");function GBt(r){return r===null?!1:mCe.exec(r)!==null||yCe.exec(r)!==null}function VBt(r){var i,o,l,f,b,d,w,y=0,k=null,E,T,C;if(i=mCe.exec(r),i===null&&(i=yCe.exec(r)),i===null)throw new Error("Date resolve error");if(o=+i[1],l=+i[2]-1,f=+i[3],!i[4])return new Date(Date.UTC(o,l,f));if(b=+i[4],d=+i[5],w=+i[6],i[7]){for(y=i[7].slice(0,3);y.length<3;)y+="0";y=+y}return i[9]&&(E=+i[10],T=+(i[11]||0),k=(E*60+T)*6e4,i[9]==="-"&&(k=-k)),C=new Date(Date.UTC(o,l,f,b,d,w,y)),k&&C.setTime(C.getTime()-k),C}function UBt(r){return r.toISOString()}var qBt=new ud("tag:yaml.org,2002:timestamp",{kind:"scalar",resolve:GBt,construct:VBt,instanceOf:Date,represent:UBt});function YBt(r){return r==="<<"||r===null}var WBt=new ud("tag:yaml.org,2002:merge",{kind:"scalar",resolve:YBt}),aae=`ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=
\r`;function KBt(r){if(r===null)return!1;var i,o,l=0,f=r.length,b=aae;for(o=0;o<f;o++)if(i=b.indexOf(r.charAt(o)),!(i>64)){if(i<0)return!1;l+=6}return l%8===0}function XBt(r){var i,o,l=r.replace(/[\r\n=]/g,""),f=l.length,b=aae,d=0,w=[];for(i=0;i<f;i++)i%4===0&&i&&(w.push(d>>16&255),w.push(d>>8&255),w.push(d&255)),d=d<<6|b.indexOf(l.charAt(i));return o=f%4*6,o===0?(w.push(d>>16&255),w.push(d>>8&255),w.push(d&255)):o===18?(w.push(d>>10&255),w.push(d>>2&255)):o===12&&w.push(d>>4&255),new Uint8Array(w)}function QBt(r){var i="",o=0,l,f,b=r.length,d=aae;for(l=0;l<b;l++)l%3===0&&l&&(i+=d[o>>18&63],i+=d[o>>12&63],i+=d[o>>6&63],i+=d[o&63]),o=(o<<8)+r[l];return f=b%3,f===0?(i+=d[o>>18&63],i+=d[o>>12&63],i+=d[o>>6&63],i+=d[o&63]):f===2?(i+=d[o>>10&63],i+=d[o>>4&63],i+=d[o<<2&63],i+=d[64]):f===1&&(i+=d[o>>2&63],i+=d[o<<4&63],i+=d[64],i+=d[64]),i}function ZBt(r){return Object.prototype.toString.call(r)==="[object Uint8Array]"}var JBt=new ud("tag:yaml.org,2002:binary",{kind:"scalar",resolve:KBt,construct:XBt,predicate:ZBt,represent:QBt}),eFt=Object.prototype.hasOwnProperty,tFt=Object.prototype.toString;function nFt(r){if(r===null)return!0;var i=[],o,l,f,b,d,w=r;for(o=0,l=w.length;o<l;o+=1){if(f=w[o],d=!1,tFt.call(f)!=="[object Object]")return!1;for(b in f)if(eFt.call(f,b))if(!d)d=!0;else return!1;if(!d)return!1;if(i.indexOf(b)===-1)i.push(b);else return!1}return!0}function rFt(r){return r!==null?r:[]}var iFt=new ud("tag:yaml.org,2002:omap",{kind:"sequence",resolve:nFt,construct:rFt}),sFt=Object.prototype.toString;function aFt(r){if(r===null)return!0;var i,o,l,f,b,d=r;for(b=new Array(d.length),i=0,o=d.length;i<o;i+=1){if(l=d[i],sFt.call(l)!=="[object Object]"||(f=Object.keys(l),f.length!==1))return!1;b[i]=[f[0],l[f[0]]]}return!0}function oFt(r){if(r===null)return[];var i,o,l,f,b,d=r;for(b=new Array(d.length),i=0,o=d.length;i<o;i+=1)l=d[i],f=Object.keys(l),b[i]=[f[0],l[f[0]]];return b}var cFt=new ud("tag:yaml.org,2002:pairs",{kind:"sequence",resolve:aFt,construct:oFt}),uFt=Object.prototype.hasOwnProperty;function lFt(r){if(r===null)return!0;var i,o=r;for(i in o)if(uFt.call(o,i)&&o[i]!==null)return!1;return!0}function hFt(r){return r!==null?r:{}}var fFt=new ud("tag:yaml.org,2002:set",{kind:"mapping",resolve:lFt,construct:hFt}),dFt=zBt.extend({implicit:[qBt,WBt],explicit:[JBt,iFt,cFt,fFt]}),l6=Object.prototype.hasOwnProperty,AG=1,kCe=2,xCe=3,MG=4,oae=1,gFt=2,ECe=3,pFt=/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/,bFt=/[\x85\u2028\u2029]/,vFt=/[,\[\]\{\}]/,_Ce=/^(?:!|!!|![a-z\-]+!)$/i,TCe=/^(?:!|[^,\[\]\{\}])(?:%[0-9a-f]{2}|[0-9a-z\-#;\/\?:@&=\+\$,_\.!~\*'\(\)\[\]])*$/i;function CCe(r){return Object.prototype.toString.call(r)}function Pm(r){return r===10||r===13}function q8(r){return r===9||r===32}function U0(r){return r===9||r===32||r===10||r===13}function YT(r){return r===44||r===91||r===93||r===123||r===125}function wFt(r){var i;return 48<=r&&r<=57?r-48:(i=r|32,97<=i&&i<=102?i-97+10:-1)}function mFt(r){return r===120?2:r===117?4:r===85?8:0}function yFt(r){return 48<=r&&r<=57?r-48:-1}function SCe(r){return r===48?"\0":r===97?"\x07":r===98?"\b":r===116||r===9?"	":r===110?`
`:r===118?"\v":r===102?"\f":r===114?"\r":r===101?"\x1B":r===32?" ":r===34?'"':r===47?"/":r===92?"\\":r===78?"\x85":r===95?"\xA0":r===76?"\u2028":r===80?"\u2029":""}function kFt(r){return r<=65535?String.fromCharCode(r):String.fromCharCode((r-65536>>10)+55296,(r-65536&1023)+56320)}for(var ACe=new Array(256),MCe=new Array(256),WT=0;WT<256;WT++)ACe[WT]=SCe(WT)?1:0,MCe[WT]=SCe(WT);function xFt(r,i){this.input=r,this.filename=i.filename||null,this.schema=i.schema||dFt,this.onWarning=i.onWarning||null,this.legacy=i.legacy||!1,this.json=i.json||!1,this.listener=i.listener||null,this.implicitTypes=this.schema.compiledImplicit,this.typeMap=this.schema.compiledTypeMap,this.length=r.length,this.position=0,this.line=0,this.lineStart=0,this.lineIndent=0,this.firstTabInLine=-1,this.documents=[]}function DCe(r,i){var o={name:r.filename,buffer:r.input.slice(0,-1),position:r.position,line:r.line,column:r.position-r.lineStart};return o.snippet=uBt(o),new Y3(i,o)}function ji(r,i){throw DCe(r,i)}function DG(r,i){r.onWarning&&r.onWarning.call(null,DCe(r,i))}var LCe={YAML:function(i,o,l){var f,b,d;i.version!==null&&ji(i,"duplication of %YAML directive"),l.length!==1&&ji(i,"YAML directive accepts exactly one argument"),f=/^([0-9]+)\.([0-9]+)$/.exec(l[0]),f===null&&ji(i,"ill-formed argument of the YAML directive"),b=parseInt(f[1],10),d=parseInt(f[2],10),b!==1&&ji(i,"unacceptable YAML version of the document"),i.version=l[0],i.checkLineBreaks=d<2,d!==1&&d!==2&&DG(i,"unsupported YAML version of the document")},TAG:function(i,o,l){var f,b;l.length!==2&&ji(i,"TAG directive accepts exactly two arguments"),f=l[0],b=l[1],_Ce.test(f)||ji(i,"ill-formed tag handle (first argument) of the TAG directive"),l6.call(i.tagMap,f)&&ji(i,'there is a previously declared suffix for "'+f+'" tag handle'),TCe.test(b)||ji(i,"ill-formed tag prefix (second argument) of the TAG directive");try{b=decodeURIComponent(b)}catch{ji(i,"tag prefix is malformed: "+b)}i.tagMap[f]=b}};function h6(r,i,o,l){var f,b,d,w;if(i<o){if(w=r.input.slice(i,o),l)for(f=0,b=w.length;f<b;f+=1)d=w.charCodeAt(f),d===9||32<=d&&d<=1114111||ji(r,"expected valid JSON character");else pFt.test(w)&&ji(r,"the stream contains non-printable characters");r.result+=w}}function ICe(r,i,o,l){var f,b,d,w;for(Qd.isObject(o)||ji(r,"cannot merge mappings; the provided source object is unacceptable"),f=Object.keys(o),d=0,w=f.length;d<w;d+=1)b=f[d],l6.call(i,b)||(i[b]=o[b],l[b]=!0)}function KT(r,i,o,l,f,b,d,w,y){var k,E;if(Array.isArray(f))for(f=Array.prototype.slice.call(f),k=0,E=f.length;k<E;k+=1)Array.isArray(f[k])&&ji(r,"nested arrays are not supported inside keys"),typeof f=="object"&&CCe(f[k])==="[object Object]"&&(f[k]="[object Object]");if(typeof f=="object"&&CCe(f)==="[object Object]"&&(f="[object Object]"),f=String(f),i===null&&(i={}),l==="tag:yaml.org,2002:merge")if(Array.isArray(b))for(k=0,E=b.length;k<E;k+=1)ICe(r,i,b[k],o);else ICe(r,i,b,o);else!r.json&&!l6.call(o,f)&&l6.call(i,f)&&(r.line=d||r.line,r.lineStart=w||r.lineStart,r.position=y||r.position,ji(r,"duplicated mapping key")),f==="__proto__"?Object.defineProperty(i,f,{configurable:!0,enumerable:!0,writable:!0,value:b}):i[f]=b,delete o[f];return i}function cae(r){var i;i=r.input.charCodeAt(r.position),i===10?r.position++:i===13?(r.position++,r.input.charCodeAt(r.position)===10&&r.position++):ji(r,"a line break is expected"),r.line+=1,r.lineStart=r.position,r.firstTabInLine=-1}function Fh(r,i,o){for(var l=0,f=r.input.charCodeAt(r.position);f!==0;){for(;q8(f);)f===9&&r.firstTabInLine===-1&&(r.firstTabInLine=r.position),f=r.input.charCodeAt(++r.position);if(i&&f===35)do f=r.input.charCodeAt(++r.position);while(f!==10&&f!==13&&f!==0);if(Pm(f))for(cae(r),f=r.input.charCodeAt(r.position),l++,r.lineIndent=0;f===32;)r.lineIndent++,f=r.input.charCodeAt(++r.position);else break}return o!==-1&&l!==0&&r.lineIndent<o&&DG(r,"deficient indentation"),l}function LG(r){var i=r.position,o;return o=r.input.charCodeAt(i),!!((o===45||o===46)&&o===r.input.charCodeAt(i+1)&&o===r.input.charCodeAt(i+2)&&(i+=3,o=r.input.charCodeAt(i),o===0||U0(o)))}function uae(r,i){i===1?r.result+=" ":i>1&&(r.result+=Qd.repeat(`
`,i-1))}function EFt(r,i,o){var l,f,b,d,w,y,k,E,T=r.kind,C=r.result,S;if(S=r.input.charCodeAt(r.position),U0(S)||YT(S)||S===35||S===38||S===42||S===33||S===124||S===62||S===39||S===34||S===37||S===64||S===96||(S===63||S===45)&&(f=r.input.charCodeAt(r.position+1),U0(f)||o&&YT(f)))return!1;for(r.kind="scalar",r.result="",b=d=r.position,w=!1;S!==0;){if(S===58){if(f=r.input.charCodeAt(r.position+1),U0(f)||o&&YT(f))break}else if(S===35){if(l=r.input.charCodeAt(r.position-1),U0(l))break}else{if(r.position===r.lineStart&&LG(r)||o&&YT(S))break;if(Pm(S))if(y=r.line,k=r.lineStart,E=r.lineIndent,Fh(r,!1,-1),r.lineIndent>=i){w=!0,S=r.input.charCodeAt(r.position);continue}else{r.position=d,r.line=y,r.lineStart=k,r.lineIndent=E;break}}w&&(h6(r,b,d,!1),uae(r,r.line-y),b=d=r.position,w=!1),q8(S)||(d=r.position+1),S=r.input.charCodeAt(++r.position)}return h6(r,b,d,!1),r.result?!0:(r.kind=T,r.result=C,!1)}function _Ft(r,i){var o,l,f;if(o=r.input.charCodeAt(r.position),o!==39)return!1;for(r.kind="scalar",r.result="",r.position++,l=f=r.position;(o=r.input.charCodeAt(r.position))!==0;)if(o===39)if(h6(r,l,r.position,!0),o=r.input.charCodeAt(++r.position),o===39)l=r.position,r.position++,f=r.position;else return!0;else Pm(o)?(h6(r,l,f,!0),uae(r,Fh(r,!1,i)),l=f=r.position):r.position===r.lineStart&&LG(r)?ji(r,"unexpected end of the document within a single quoted scalar"):(r.position++,f=r.position);ji(r,"unexpected end of the stream within a single quoted scalar")}function TFt(r,i){var o,l,f,b,d,w;if(w=r.input.charCodeAt(r.position),w!==34)return!1;for(r.kind="scalar",r.result="",r.position++,o=l=r.position;(w=r.input.charCodeAt(r.position))!==0;){if(w===34)return h6(r,o,r.position,!0),r.position++,!0;if(w===92){if(h6(r,o,r.position,!0),w=r.input.charCodeAt(++r.position),Pm(w))Fh(r,!1,i);else if(w<256&&ACe[w])r.result+=MCe[w],r.position++;else if((d=mFt(w))>0){for(f=d,b=0;f>0;f--)w=r.input.charCodeAt(++r.position),(d=wFt(w))>=0?b=(b<<4)+d:ji(r,"expected hexadecimal character");r.result+=kFt(b),r.position++}else ji(r,"unknown escape sequence");o=l=r.position}else Pm(w)?(h6(r,o,l,!0),uae(r,Fh(r,!1,i)),o=l=r.position):r.position===r.lineStart&&LG(r)?ji(r,"unexpected end of the document within a double quoted scalar"):(r.position++,l=r.position)}ji(r,"unexpected end of the stream within a double quoted scalar")}function CFt(r,i){var o=!0,l,f,b,d=r.tag,w,y=r.anchor,k,E,T,C,S,L=Object.create(null),O,B,N,F;if(F=r.input.charCodeAt(r.position),F===91)E=93,S=!1,w=[];else if(F===123)E=125,S=!0,w={};else return!1;for(r.anchor!==null&&(r.anchorMap[r.anchor]=w),F=r.input.charCodeAt(++r.position);F!==0;){if(Fh(r,!0,i),F=r.input.charCodeAt(r.position),F===E)return r.position++,r.tag=d,r.anchor=y,r.kind=S?"mapping":"sequence",r.result=w,!0;o?F===44&&ji(r,"expected the node content, but found ','"):ji(r,"missed comma between flow collection entries"),B=O=N=null,T=C=!1,F===63&&(k=r.input.charCodeAt(r.position+1),U0(k)&&(T=C=!0,r.position++,Fh(r,!0,i))),l=r.line,f=r.lineStart,b=r.position,XT(r,i,AG,!1,!0),B=r.tag,O=r.result,Fh(r,!0,i),F=r.input.charCodeAt(r.position),(C||r.line===l)&&F===58&&(T=!0,F=r.input.charCodeAt(++r.position),Fh(r,!0,i),XT(r,i,AG,!1,!0),N=r.result),S?KT(r,w,L,B,O,N,l,f,b):T?w.push(KT(r,null,L,B,O,N,l,f,b)):w.push(O),Fh(r,!0,i),F=r.input.charCodeAt(r.position),F===44?(o=!0,F=r.input.charCodeAt(++r.position)):o=!1}ji(r,"unexpected end of the stream within a flow collection")}function SFt(r,i){var o,l,f=oae,b=!1,d=!1,w=i,y=0,k=!1,E,T;if(T=r.input.charCodeAt(r.position),T===124)l=!1;else if(T===62)l=!0;else return!1;for(r.kind="scalar",r.result="";T!==0;)if(T=r.input.charCodeAt(++r.position),T===43||T===45)oae===f?f=T===43?ECe:gFt:ji(r,"repeat of a chomping mode identifier");else if((E=yFt(T))>=0)E===0?ji(r,"bad explicit indentation width of a block scalar; it cannot be less than one"):d?ji(r,"repeat of an indentation width identifier"):(w=i+E-1,d=!0);else break;if(q8(T)){do T=r.input.charCodeAt(++r.position);while(q8(T));if(T===35)do T=r.input.charCodeAt(++r.position);while(!Pm(T)&&T!==0)}for(;T!==0;){for(cae(r),r.lineIndent=0,T=r.input.charCodeAt(r.position);(!d||r.lineIndent<w)&&T===32;)r.lineIndent++,T=r.input.charCodeAt(++r.position);if(!d&&r.lineIndent>w&&(w=r.lineIndent),Pm(T)){y++;continue}if(r.lineIndent<w){f===ECe?r.result+=Qd.repeat(`
`,b?1+y:y):f===oae&&b&&(r.result+=`
`);break}for(l?q8(T)?(k=!0,r.result+=Qd.repeat(`
`,b?1+y:y)):k?(k=!1,r.result+=Qd.repeat(`
`,y+1)):y===0?b&&(r.result+=" "):r.result+=Qd.repeat(`
`,y):r.result+=Qd.repeat(`
`,b?1+y:y),b=!0,d=!0,y=0,o=r.position;!Pm(T)&&T!==0;)T=r.input.charCodeAt(++r.position);h6(r,o,r.position,!1)}return!0}function OCe(r,i){var o,l=r.tag,f=r.anchor,b=[],d,w=!1,y;if(r.firstTabInLine!==-1)return!1;for(r.anchor!==null&&(r.anchorMap[r.anchor]=b),y=r.input.charCodeAt(r.position);y!==0&&(r.firstTabInLine!==-1&&(r.position=r.firstTabInLine,ji(r,"tab characters must not be used in indentation")),!(y!==45||(d=r.input.charCodeAt(r.position+1),!U0(d))));){if(w=!0,r.position++,Fh(r,!0,-1)&&r.lineIndent<=i){b.push(null),y=r.input.charCodeAt(r.position);continue}if(o=r.line,XT(r,i,xCe,!1,!0),b.push(r.result),Fh(r,!0,-1),y=r.input.charCodeAt(r.position),(r.line===o||r.lineIndent>i)&&y!==0)ji(r,"bad indentation of a sequence entry");else if(r.lineIndent<i)break}return w?(r.tag=l,r.anchor=f,r.kind="sequence",r.result=b,!0):!1}function AFt(r,i,o){var l,f,b,d,w,y,k=r.tag,E=r.anchor,T={},C=Object.create(null),S=null,L=null,O=null,B=!1,N=!1,F;if(r.firstTabInLine!==-1)return!1;for(r.anchor!==null&&(r.anchorMap[r.anchor]=T),F=r.input.charCodeAt(r.position);F!==0;){if(!B&&r.firstTabInLine!==-1&&(r.position=r.firstTabInLine,ji(r,"tab characters must not be used in indentation")),l=r.input.charCodeAt(r.position+1),b=r.line,(F===63||F===58)&&U0(l))F===63?(B&&(KT(r,T,C,S,L,null,d,w,y),S=L=O=null),N=!0,B=!0,f=!0):B?(B=!1,f=!0):ji(r,"incomplete explicit mapping pair; a key node is missed; or followed by a non-tabulated empty line"),r.position+=1,F=l;else{if(d=r.line,w=r.lineStart,y=r.position,!XT(r,o,kCe,!1,!0))break;if(r.line===b){for(F=r.input.charCodeAt(r.position);q8(F);)F=r.input.charCodeAt(++r.position);if(F===58)F=r.input.charCodeAt(++r.position),U0(F)||ji(r,"a whitespace character is expected after the key-value separator within a block mapping"),B&&(KT(r,T,C,S,L,null,d,w,y),S=L=O=null),N=!0,B=!1,f=!1,S=r.tag,L=r.result;else if(N)ji(r,"can not read an implicit mapping pair; a colon is missed");else return r.tag=k,r.anchor=E,!0}else if(N)ji(r,"can not read a block mapping entry; a multiline key may not be an implicit key");else return r.tag=k,r.anchor=E,!0}if((r.line===b||r.lineIndent>i)&&(B&&(d=r.line,w=r.lineStart,y=r.position),XT(r,i,MG,!0,f)&&(B?L=r.result:O=r.result),B||(KT(r,T,C,S,L,O,d,w,y),S=L=O=null),Fh(r,!0,-1),F=r.input.charCodeAt(r.position)),(r.line===b||r.lineIndent>i)&&F!==0)ji(r,"bad indentation of a mapping entry");else if(r.lineIndent<i)break}return B&&KT(r,T,C,S,L,null,d,w,y),N&&(r.tag=k,r.anchor=E,r.kind="mapping",r.result=T),N}function MFt(r){var i,o=!1,l=!1,f,b,d;if(d=r.input.charCodeAt(r.position),d!==33)return!1;if(r.tag!==null&&ji(r,"duplication of a tag property"),d=r.input.charCodeAt(++r.position),d===60?(o=!0,d=r.input.charCodeAt(++r.position)):d===33?(l=!0,f="!!",d=r.input.charCodeAt(++r.position)):f="!",i=r.position,o){do d=r.input.charCodeAt(++r.position);while(d!==0&&d!==62);r.position<r.length?(b=r.input.slice(i,r.position),d=r.input.charCodeAt(++r.position)):ji(r,"unexpected end of the stream within a verbatim tag")}else{for(;d!==0&&!U0(d);)d===33&&(l?ji(r,"tag suffix cannot contain exclamation marks"):(f=r.input.slice(i-1,r.position+1),_Ce.test(f)||ji(r,"named tag handle cannot contain such characters"),l=!0,i=r.position+1)),d=r.input.charCodeAt(++r.position);b=r.input.slice(i,r.position),vFt.test(b)&&ji(r,"tag suffix cannot contain flow indicator characters")}b&&!TCe.test(b)&&ji(r,"tag name cannot contain such characters: "+b);try{b=decodeURIComponent(b)}catch{ji(r,"tag name is malformed: "+b)}return o?r.tag=b:l6.call(r.tagMap,f)?r.tag=r.tagMap[f]+b:f==="!"?r.tag="!"+b:f==="!!"?r.tag="tag:yaml.org,2002:"+b:ji(r,'undeclared tag handle "'+f+'"'),!0}function DFt(r){var i,o;if(o=r.input.charCodeAt(r.position),o!==38)return!1;for(r.anchor!==null&&ji(r,"duplication of an anchor property"),o=r.input.charCodeAt(++r.position),i=r.position;o!==0&&!U0(o)&&!YT(o);)o=r.input.charCodeAt(++r.position);return r.position===i&&ji(r,"name of an anchor node must contain at least one character"),r.anchor=r.input.slice(i,r.position),!0}function LFt(r){var i,o,l;if(l=r.input.charCodeAt(r.position),l!==42)return!1;for(l=r.input.charCodeAt(++r.position),i=r.position;l!==0&&!U0(l)&&!YT(l);)l=r.input.charCodeAt(++r.position);return r.position===i&&ji(r,"name of an alias node must contain at least one character"),o=r.input.slice(i,r.position),l6.call(r.anchorMap,o)||ji(r,'unidentified alias "'+o+'"'),r.result=r.anchorMap[o],Fh(r,!0,-1),!0}function XT(r,i,o,l,f){var b,d,w,y=1,k=!1,E=!1,T,C,S,L,O,B;if(r.listener!==null&&r.listener("open",r),r.tag=null,r.anchor=null,r.kind=null,r.result=null,b=d=w=MG===o||xCe===o,l&&Fh(r,!0,-1)&&(k=!0,r.lineIndent>i?y=1:r.lineIndent===i?y=0:r.lineIndent<i&&(y=-1)),y===1)for(;MFt(r)||DFt(r);)Fh(r,!0,-1)?(k=!0,w=b,r.lineIndent>i?y=1:r.lineIndent===i?y=0:r.lineIndent<i&&(y=-1)):w=!1;if(w&&(w=k||f),(y===1||MG===o)&&(AG===o||kCe===o?O=i:O=i+1,B=r.position-r.lineStart,y===1?w&&(OCe(r,B)||AFt(r,B,O))||CFt(r,O)?E=!0:(d&&SFt(r,O)||_Ft(r,O)||TFt(r,O)?E=!0:LFt(r)?(E=!0,(r.tag!==null||r.anchor!==null)&&ji(r,"alias node should not have any properties")):EFt(r,O,AG===o)&&(E=!0,r.tag===null&&(r.tag="?")),r.anchor!==null&&(r.anchorMap[r.anchor]=r.result)):y===0&&(E=w&&OCe(r,B))),r.tag===null)r.anchor!==null&&(r.anchorMap[r.anchor]=r.result);else if(r.tag==="?"){for(r.result!==null&&r.kind!=="scalar"&&ji(r,'unacceptable node kind for !<?> tag; it should be "scalar", not "'+r.kind+'"'),T=0,C=r.implicitTypes.length;T<C;T+=1)if(L=r.implicitTypes[T],L.resolve(r.result)){r.result=L.construct(r.result),r.tag=L.tag,r.anchor!==null&&(r.anchorMap[r.anchor]=r.result);break}}else if(r.tag!=="!"){if(l6.call(r.typeMap[r.kind||"fallback"],r.tag))L=r.typeMap[r.kind||"fallback"][r.tag];else for(L=null,S=r.typeMap.multi[r.kind||"fallback"],T=0,C=S.length;T<C;T+=1)if(r.tag.slice(0,S[T].tag.length)===S[T].tag){L=S[T];break}L||ji(r,"unknown tag !<"+r.tag+">"),r.result!==null&&L.kind!==r.kind&&ji(r,"unacceptable node kind for !<"+r.tag+'> tag; it should be "'+L.kind+'", not "'+r.kind+'"'),L.resolve(r.result,r.tag)?(r.result=L.construct(r.result,r.tag),r.anchor!==null&&(r.anchorMap[r.anchor]=r.result)):ji(r,"cannot resolve a node with !<"+r.tag+"> explicit tag")}return r.listener!==null&&r.listener("close",r),r.tag!==null||r.anchor!==null||E}function IFt(r){var i=r.position,o,l,f,b=!1,d;for(r.version=null,r.checkLineBreaks=r.legacy,r.tagMap=Object.create(null),r.anchorMap=Object.create(null);(d=r.input.charCodeAt(r.position))!==0&&(Fh(r,!0,-1),d=r.input.charCodeAt(r.position),!(r.lineIndent>0||d!==37));){for(b=!0,d=r.input.charCodeAt(++r.position),o=r.position;d!==0&&!U0(d);)d=r.input.charCodeAt(++r.position);for(l=r.input.slice(o,r.position),f=[],l.length<1&&ji(r,"directive name must not be less than one character in length");d!==0;){for(;q8(d);)d=r.input.charCodeAt(++r.position);if(d===35){do d=r.input.charCodeAt(++r.position);while(d!==0&&!Pm(d));break}if(Pm(d))break;for(o=r.position;d!==0&&!U0(d);)d=r.input.charCodeAt(++r.position);f.push(r.input.slice(o,r.position))}d!==0&&cae(r),l6.call(LCe,l)?LCe[l](r,l,f):DG(r,'unknown document directive "'+l+'"')}if(Fh(r,!0,-1),r.lineIndent===0&&r.input.charCodeAt(r.position)===45&&r.input.charCodeAt(r.position+1)===45&&r.input.charCodeAt(r.position+2)===45?(r.position+=3,Fh(r,!0,-1)):b&&ji(r,"directives end mark is expected"),XT(r,r.lineIndent-1,MG,!1,!0),Fh(r,!0,-1),r.checkLineBreaks&&bFt.test(r.input.slice(i,r.position))&&DG(r,"non-ASCII line breaks are interpreted as content"),r.documents.push(r.result),r.position===r.lineStart&&LG(r)){r.input.charCodeAt(r.position)===46&&(r.position+=3,Fh(r,!0,-1));return}if(r.position<r.length-1)ji(r,"end of the stream or a document separator is expected");else return}function NCe(r,i){r=String(r),i=i||{},r.length!==0&&(r.charCodeAt(r.length-1)!==10&&r.charCodeAt(r.length-1)!==13&&(r+=`
`),r.charCodeAt(0)===65279&&(r=r.slice(1)));var o=new xFt(r,i),l=r.indexOf("\0");for(l!==-1&&(o.position=l,ji(o,"null byte is not allowed in input")),o.input+="\0";o.input.charCodeAt(o.position)===32;)o.lineIndent+=1,o.position+=1;for(;o.position<o.length-1;)IFt(o);return o.documents}function OFt(r,i,o){i!==null&&typeof i=="object"&&typeof o>"u"&&(o=i,i=null);var l=NCe(r,o);if(typeof i!="function")return l;for(var f=0,b=l.length;f<b;f+=1)i(l[f])}function NFt(r,i){var o=NCe(r,i);if(o.length!==0){if(o.length===1)return o[0];throw new Y3("expected a single document in the stream, but found more")}}var PFt=OFt,BFt=NFt,FFt={loadAll:PFt,load:BFt},RFt=wCe,jFt=FFt.load;const PCe=/^-{3}\s*[\n\r](.*?)[\n\r]-{3}\s*[\n\r]+/s;function $Ft(r,i){var l;const o=r.match(PCe);if(o){const f=jFt(o[1],{schema:RFt});return f!=null&&f.title&&((l=i.setDiagramTitle)==null||l.call(i,f.title)),r.slice(o[0].length)}else return r}const HFt=/%{2}{\s*(?:(\w+)\s*:|(\w+))\s*(?:(\w+)|((?:(?!}%{2}).|\r?\n)*))?\s*(?:}%{2})?/gi,zFt=/\s*%%.*\n/gm,IG={},lae=function(r,i){r=r.replace(PCe,"").replace(HFt,"").replace(zFt,`
`);for(const[o,{detector:l}]of Object.entries(IG))if(l(r,i))return o;throw new Error(`No diagram type detected for text: ${r}`)},BCe=(...r)=>{for(const{id:i,detector:o,loader:l}of r)FCe(i,o,l)},FCe=(r,i,o)=>{IG[r]?Se.error(`Detector with key ${r} already exists`):IG[r]={detector:i,loader:o},Se.debug(`Detector with key ${r} added${o?" with loader":""}`)},GFt=r=>IG[r].loader,hae=function(r,i,o){const{depth:l,clobber:f}=Object.assign({depth:2,clobber:!1},o);return Array.isArray(i)&&!Array.isArray(r)?(i.forEach(b=>hae(r,b,o)),r):Array.isArray(i)&&Array.isArray(r)?(i.forEach(b=>{r.includes(b)||r.push(b)}),r):r===void 0||l<=0?r!=null&&typeof r=="object"&&typeof i=="object"?Object.assign(r,i):i:(i!==void 0&&typeof r=="object"&&typeof i=="object"&&Object.keys(i).forEach(b=>{typeof i[b]=="object"&&(r[b]===void 0||typeof r[b]=="object")?(r[b]===void 0&&(r[b]=Array.isArray(i[b])?[]:{}),r[b]=hae(r[b],i[b],{depth:l-1,clobber:f})):(f||typeof r[b]!="object"&&typeof i[b]!="object")&&(r[b]=i[b])}),r)},Yf=hae;var VFt=typeof global=="object"&&global&&global.Object===Object&&global;const RCe=VFt;var UFt=typeof self=="object"&&self&&self.Object===Object&&self,qFt=RCe||UFt||Function("return this")();const Mv=qFt;var YFt=Mv.Symbol;const I2=YFt;var jCe=Object.prototype,WFt=jCe.hasOwnProperty,KFt=jCe.toString,iI=I2?I2.toStringTag:void 0;function XFt(r){var i=WFt.call(r,iI),o=r[iI];try{r[iI]=void 0;var l=!0}catch{}var f=KFt.call(r);return l&&(i?r[iI]=o:delete r[iI]),f}var QFt=Object.prototype,ZFt=QFt.toString;function JFt(r){return ZFt.call(r)}var eRt="[object Null]",tRt="[object Undefined]",$Ce=I2?I2.toStringTag:void 0;function Y8(r){return r==null?r===void 0?tRt:eRt:$Ce&&$Ce in Object(r)?XFt(r):JFt(r)}function lp(r){var i=typeof r;return r!=null&&(i=="object"||i=="function")}var nRt="[object AsyncFunction]",rRt="[object Function]",iRt="[object GeneratorFunction]",sRt="[object Proxy]";function QT(r){if(!lp(r))return!1;var i=Y8(r);return i==rRt||i==iRt||i==nRt||i==sRt}var aRt=Mv["__core-js_shared__"];const fae=aRt;var HCe=function(){var r=/[^.]+$/.exec(fae&&fae.keys&&fae.keys.IE_PROTO||"");return r?"Symbol(src)_1."+r:""}();function oRt(r){return!!HCe&&HCe in r}var cRt=Function.prototype,uRt=cRt.toString;function W8(r){if(r!=null){try{return uRt.call(r)}catch{}try{return r+""}catch{}}return""}var lRt=/[\\^$.*+?()[\]{}|]/g,hRt=/^\[object .+?Constructor\]$/,fRt=Function.prototype,dRt=Object.prototype,gRt=fRt.toString,pRt=dRt.hasOwnProperty,bRt=RegExp("^"+gRt.call(pRt).replace(lRt,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function vRt(r){if(!lp(r)||oRt(r))return!1;var i=QT(r)?bRt:hRt;return i.test(W8(r))}function wRt(r,i){return r==null?void 0:r[i]}function K8(r,i){var o=wRt(r,i);return vRt(o)?o:void 0}var mRt=K8(Object,"create");const sI=mRt;function yRt(){this.__data__=sI?sI(null):{},this.size=0}function kRt(r){var i=this.has(r)&&delete this.__data__[r];return this.size-=i?1:0,i}var xRt="__lodash_hash_undefined__",ERt=Object.prototype,_Rt=ERt.hasOwnProperty;function TRt(r){var i=this.__data__;if(sI){var o=i[r];return o===xRt?void 0:o}return _Rt.call(i,r)?i[r]:void 0}var CRt=Object.prototype,SRt=CRt.hasOwnProperty;function ARt(r){var i=this.__data__;return sI?i[r]!==void 0:SRt.call(i,r)}var MRt="__lodash_hash_undefined__";function DRt(r,i){var o=this.__data__;return this.size+=this.has(r)?0:1,o[r]=sI&&i===void 0?MRt:i,this}function X8(r){var i=-1,o=r==null?0:r.length;for(this.clear();++i<o;){var l=r[i];this.set(l[0],l[1])}}X8.prototype.clear=yRt,X8.prototype.delete=kRt,X8.prototype.get=TRt,X8.prototype.has=ARt,X8.prototype.set=DRt;function LRt(){this.__data__=[],this.size=0}function ZT(r,i){return r===i||r!==r&&i!==i}function OG(r,i){for(var o=r.length;o--;)if(ZT(r[o][0],i))return o;return-1}var IRt=Array.prototype,ORt=IRt.splice;function NRt(r){var i=this.__data__,o=OG(i,r);if(o<0)return!1;var l=i.length-1;return o==l?i.pop():ORt.call(i,o,1),--this.size,!0}function PRt(r){var i=this.__data__,o=OG(i,r);return o<0?void 0:i[o][1]}function BRt(r){return OG(this.__data__,r)>-1}function FRt(r,i){var o=this.__data__,l=OG(o,r);return l<0?(++this.size,o.push([r,i])):o[l][1]=i,this}function W3(r){var i=-1,o=r==null?0:r.length;for(this.clear();++i<o;){var l=r[i];this.set(l[0],l[1])}}W3.prototype.clear=LRt,W3.prototype.delete=NRt,W3.prototype.get=PRt,W3.prototype.has=BRt,W3.prototype.set=FRt;var RRt=K8(Mv,"Map");const aI=RRt;function jRt(){this.size=0,this.__data__={hash:new X8,map:new(aI||W3),string:new X8}}function $Rt(r){var i=typeof r;return i=="string"||i=="number"||i=="symbol"||i=="boolean"?r!=="__proto__":r===null}function NG(r,i){var o=r.__data__;return $Rt(i)?o[typeof i=="string"?"string":"hash"]:o.map}function HRt(r){var i=NG(this,r).delete(r);return this.size-=i?1:0,i}function zRt(r){return NG(this,r).get(r)}function GRt(r){return NG(this,r).has(r)}function VRt(r,i){var o=NG(this,r),l=o.size;return o.set(r,i),this.size+=o.size==l?0:1,this}function K3(r){var i=-1,o=r==null?0:r.length;for(this.clear();++i<o;){var l=r[i];this.set(l[0],l[1])}}K3.prototype.clear=jRt,K3.prototype.delete=HRt,K3.prototype.get=zRt,K3.prototype.has=GRt,K3.prototype.set=VRt;var URt="Expected a function";function JT(r,i){if(typeof r!="function"||i!=null&&typeof i!="function")throw new TypeError(URt);var o=function(){var l=arguments,f=i?i.apply(this,l):l[0],b=o.cache;if(b.has(f))return b.get(f);var d=r.apply(this,l);return o.cache=b.set(f,d)||b,d};return o.cache=new(JT.Cache||K3),o}JT.Cache=K3;const qRt={curveBasis:UT,curveBasisClosed:FNt,curveBasisOpen:RNt,curveBumpX:PNt,curveBumpY:BNt,curveBundle:jNt,curveCardinalClosed:HNt,curveCardinalOpen:zNt,curveCardinal:$Nt,curveCatmullRomClosed:VNt,curveCatmullRomOpen:UNt,curveCatmullRom:GNt,curveLinear:x1,curveLinearClosed:qNt,curveMonotoneX:YNt,curveMonotoneY:WNt,curveNatural:KNt,curveStep:XNt,curveStepAfter:ZNt,curveStepBefore:QNt},dae=/%{2}{\s*(?:(\w+)\s*:|(\w+))\s*(?:(\w+)|((?:(?!}%{2}).|\r?\n)*))?\s*(?:}%{2})?/gi,YRt=/\s*(?:(\w+)(?=:):|(\w+))\s*(?:(\w+)|((?:(?!}%{2}).|\r?\n)*))?\s*(?:}%{2})?/gi,WRt=function(r,i){const o=zCe(r,/(?:init\b)|(?:initialize\b)/);let l={};if(Array.isArray(o)){const f=o.map(b=>b.args);Q8(f),l=Yf(l,[...f])}else l=o.args;if(l){let f=lae(r,i);["config"].forEach(b=>{l[b]!==void 0&&(f==="flowchart-v2"&&(f="flowchart"),l[f]=l[b],delete l[b])})}return l},zCe=function(r,i=null){try{const o=new RegExp(`[%]{2}(?![{]${YRt.source})(?=[}][%]{2}).*
`,"ig");r=r.trim().replace(o,"").replace(/'/gm,'"'),Se.debug(`Detecting diagram directive${i!==null?" type:"+i:""} based on the text:${r}`);let l;const f=[];for(;(l=dae.exec(r))!==null;)if(l.index===dae.lastIndex&&dae.lastIndex++,l&&!i||i&&l[1]&&l[1].match(i)||i&&l[2]&&l[2].match(i)){const b=l[1]?l[1]:l[2],d=l[3]?l[3].trim():l[4]?JSON.parse(l[4].trim()):null;f.push({type:b,args:d})}return f.length===0&&f.push({type:r,args:null}),f.length===1?f[0]:f}catch(o){return Se.error(`ERROR: ${o.message} - Unable to parse directive
      ${i!==null?" type:"+i:""} based on the text:${r}`),{type:null,args:null}}},KRt=function(r,i){for(const[o,l]of i.entries())if(l.match(r))return o;return-1};function q0(r,i){if(!r)return i;const o=`curve${r.charAt(0).toUpperCase()+r.slice(1)}`;return qRt[o]||i}function XRt(r,i){const o=r.trim();if(o)return i.securityLevel!=="loose"?$3(o):o}const QRt=(r,...i)=>{const o=r.split("."),l=o.length-1,f=o[l];let b=window;for(let d=0;d<l;d++)if(b=b[o[d]],!b)return;b[f](...i)};function PG(r,i){return r&&i?Math.sqrt(Math.pow(i.x-r.x,2)+Math.pow(i.y-r.y,2)):0}function ZRt(r){let i,o=0;r.forEach(b=>{o+=PG(b,i),i=b});let l=o/2,f;return i=void 0,r.forEach(b=>{if(i&&!f){const d=PG(b,i);if(d<l)l-=d;else{const w=l/d;w<=0&&(f=i),w>=1&&(f={x:b.x,y:b.y}),w>0&&w<1&&(f={x:(1-w)*i.x+w*b.x,y:(1-w)*i.y+w*b.y})}}i=b}),f}function JRt(r){return r.length===1?r[0]:ZRt(r)}const ejt=(r,i,o)=>{let l;Se.info(`our points ${JSON.stringify(i)}`),i[0]!==o&&(i=i.reverse());let b=25,d;l=void 0,i.forEach(E=>{if(l&&!d){const T=PG(E,l);if(T<b)b-=T;else{const C=b/T;C<=0&&(d=l),C>=1&&(d={x:E.x,y:E.y}),C>0&&C<1&&(d={x:(1-C)*l.x+C*E.x,y:(1-C)*l.y+C*E.y})}}l=E});const w=r?10:5,y=Math.atan2(i[0].y-d.y,i[0].x-d.x),k={x:0,y:0};return k.x=Math.sin(y)*w+(i[0].x+d.x)/2,k.y=-Math.cos(y)*w+(i[0].y+d.y)/2,k};function tjt(r,i,o){let l=JSON.parse(JSON.stringify(o)),f;Se.info("our points",l),i!=="start_left"&&i!=="start_right"&&(l=l.reverse()),l.forEach(T=>{f=T});let d=25+r,w;f=void 0,l.forEach(T=>{if(f&&!w){const C=PG(T,f);if(C<d)d-=C;else{const S=d/C;S<=0&&(w=f),S>=1&&(w={x:T.x,y:T.y}),S>0&&S<1&&(w={x:(1-S)*f.x+S*T.x,y:(1-S)*f.y+S*T.y})}}f=T});const y=10+r*.5,k=Math.atan2(l[0].y-w.y,l[0].x-w.x),E={x:0,y:0};return E.x=Math.sin(k)*y+(l[0].x+w.x)/2,E.y=-Math.cos(k)*y+(l[0].y+w.y)/2,i==="start_left"&&(E.x=Math.sin(k+Math.PI)*y+(l[0].x+w.x)/2,E.y=-Math.cos(k+Math.PI)*y+(l[0].y+w.y)/2),i==="end_right"&&(E.x=Math.sin(k-Math.PI)*y+(l[0].x+w.x)/2-5,E.y=-Math.cos(k-Math.PI)*y+(l[0].y+w.y)/2-5),i==="end_left"&&(E.x=Math.sin(k)*y+(l[0].x+w.x)/2-5,E.y=-Math.cos(k)*y+(l[0].y+w.y)/2-5),E}function Dv(r){let i="",o="";for(const l of r)l!==void 0&&(l.startsWith("color:")||l.startsWith("text-align:")?o=o+l+";":i=i+l+";");return{style:i,labelStyle:o}}let GCe=0;const VCe=()=>(GCe++,"id-"+Math.random().toString(36).substr(2,12)+"-"+GCe);function njt(r){let i="";const o="0123456789abcdef",l=o.length;for(let f=0;f<r;f++)i+=o.charAt(Math.floor(Math.random()*l));return i}const UCe=r=>njt(r.length),rjt=function(){return{x:0,y:0,fill:void 0,anchor:"start",style:"#666",width:100,height:100,textMargin:0,rx:0,ry:0,valign:void 0}},ijt=function(r,i){const o=i.text.replace(Cs.lineBreakRegex," "),[,l]=oI(i.fontSize),f=r.append("text");f.attr("x",i.x),f.attr("y",i.y),f.style("text-anchor",i.anchor),f.style("font-family",i.fontFamily),f.style("font-size",l),f.style("font-weight",i.fontWeight),f.attr("fill",i.fill),i.class!==void 0&&f.attr("class",i.class);const b=f.append("tspan");return b.attr("x",i.x+i.textMargin*2),b.attr("fill",i.fill),b.text(o),f},qCe=JT((r,i,o)=>{if(!r||(o=Object.assign({fontSize:12,fontWeight:400,fontFamily:"Arial",joinWith:"<br/>"},o),Cs.lineBreakRegex.test(r)))return r;const l=r.split(" "),f=[];let b="";return l.forEach((d,w)=>{const y=Bm(`${d} `,o),k=Bm(b,o);if(y>i){const{hyphenatedStrings:C,remainingWord:S}=sjt(d,i,"-",o);f.push(b,...C),b=S}else k+y>=i?(f.push(b),b=d):b=[b,d].filter(Boolean).join(" ");w+1===l.length&&f.push(b)}),f.filter(d=>d!=="").join(o.joinWith)},(r,i,o)=>`${r}${i}${o.fontSize}${o.fontWeight}${o.fontFamily}${o.joinWith}`),sjt=JT((r,i,o="-",l)=>{l=Object.assign({fontSize:12,fontWeight:400,fontFamily:"Arial",margin:0},l);const f=[...r],b=[];let d="";return f.forEach((w,y)=>{const k=`${d}${w}`;if(Bm(k,l)>=i){const T=y+1,C=f.length===T,S=`${k}${o}`;b.push(C?k:S),d=""}else d=k}),{hyphenatedStrings:b,remainingWord:d}},(r,i,o="-",l)=>`${r}${i}${o}${l.fontSize}${l.fontWeight}${l.fontFamily}`);function gae(r,i){return i=Object.assign({fontSize:12,fontWeight:400,fontFamily:"Arial",margin:15},i),pae(r,i).height}function Bm(r,i){return i=Object.assign({fontSize:12,fontWeight:400,fontFamily:"Arial"},i),pae(r,i).width}const pae=JT((r,i)=>{i=Object.assign({fontSize:12,fontWeight:400,fontFamily:"Arial"},i);const{fontSize:o,fontFamily:l,fontWeight:f}=i;if(!r)return{width:0,height:0};const[,b]=oI(o),d=["sans-serif",l],w=r.split(Cs.lineBreakRegex),y=[],k=Cn("body");if(!k.remove)return{width:0,height:0,lineHeight:0};const E=k.append("svg");for(const C of d){let S=0;const L={width:0,height:0,lineHeight:0};for(const O of w){const B=rjt();B.text=O;const N=ijt(E,B).style("font-size",b).style("font-weight",f).style("font-family",C),F=(N._groups||N)[0][0].getBBox();L.width=Math.round(Math.max(L.width,F.width)),S=Math.round(F.height),L.height+=S,L.lineHeight=Math.round(Math.max(L.lineHeight,S))}y.push(L)}E.remove();const T=isNaN(y[1].height)||isNaN(y[1].width)||isNaN(y[1].lineHeight)||y[0].height>y[1].height&&y[0].width>y[1].width&&y[0].lineHeight>y[1].lineHeight?0:1;return y[T]},(r,i)=>`${r}${i.fontSize}${i.fontWeight}${i.fontFamily}`),ajt=class{constructor(i,o){this.deterministic=i,this.seed=o,this.count=o?o.length:0}next(){return this.deterministic?this.count++:Date.now()}};let BG;const ojt=function(r){return BG=BG||document.createElement("div"),r=escape(r).replace(/%26/g,"&").replace(/%23/g,"#").replace(/%3B/g,";"),BG.innerHTML=r,unescape(BG.textContent)},Q8=r=>{if(Se.debug("directiveSanitizer called with",r),typeof r=="object"&&(r.length?r.forEach(i=>Q8(i)):Object.keys(r).forEach(i=>{Se.debug("Checking key",i),i.startsWith("__")&&(Se.debug("sanitize deleting __ option",i),delete r[i]),i.includes("proto")&&(Se.debug("sanitize deleting proto option",i),delete r[i]),i.includes("constr")&&(Se.debug("sanitize deleting constr option",i),delete r[i]),i.includes("themeCSS")&&(Se.debug("sanitizing themeCss option"),r[i]=FG(r[i])),i.includes("fontFamily")&&(Se.debug("sanitizing fontFamily option"),r[i]=FG(r[i])),i.includes("altFontFamily")&&(Se.debug("sanitizing altFontFamily option"),r[i]=FG(r[i])),KPt.includes(i)?typeof r[i]=="object"&&(Se.debug("sanitize deleting object",i),Q8(r[i])):(Se.debug("sanitize deleting option",i),delete r[i])})),r.themeVariables){const i=Object.keys(r.themeVariables);for(const o of i){const l=r.themeVariables[o];l&&l.match&&!l.match(/^[\d "#%(),.;A-Za-z]+$/)&&(r.themeVariables[o]="")}}Se.debug("After sanitization",r)},FG=r=>{let i=0,o=0;for(const l of r){if(i<o)return"{ /* ERROR: Unbalanced CSS */ }";l==="{"?i++:l==="}"&&o++}return i!==o?"{ /* ERROR: Unbalanced CSS */ }":r};function bae(r){return"str"in r}function cjt(r){return r instanceof Error?r.message:String(r)}const ujt=(r,i,o,l)=>{if(!l)return;const f=r.node().getBBox();r.append("text").text(l).attr("x",f.x+f.width/2).attr("y",-o).attr("class",i)},oI=r=>{if(typeof r=="number")return[r,r+"px"];const i=parseInt(r,10);return Number.isNaN(i)?[void 0,void 0]:r===String(i)?[i,r+"px"]:[i,r]},Fs={assignWithDepth:Yf,wrapLabel:qCe,calculateTextHeight:gae,calculateTextWidth:Bm,calculateTextDimensions:pae,detectInit:WRt,detectDirective:zCe,isSubstringInArray:KRt,interpolateToCurve:q0,calcLabelPosition:JRt,calcCardinalityPosition:ejt,calcTerminalLabelPosition:tjt,formatUrl:XRt,getStylesFromArray:Dv,generateId:VCe,random:UCe,runFunc:QRt,entityDecode:ojt,initIdGenerator:ajt,directiveSanitizer:Q8,sanitizeCss:FG,insertTitle:ujt,parseFontSize:oI};var YCe="comm",WCe="rule",KCe="decl",ljt="@import",hjt="@keyframes",fjt=Math.abs,vae=String.fromCharCode;function XCe(r){return r.trim()}function wae(r,i,o){return r.replace(i,o)}function djt(r,i){return r.indexOf(i)}function RG(r,i){return r.charCodeAt(i)|0}function cI(r,i,o){return r.slice(i,o)}function f6(r){return r.length}function QCe(r){return r.length}function jG(r,i){return i.push(r),r}var $G=1,eC=1,ZCe=0,O2=0,Rh=0,tC="";function mae(r,i,o,l,f,b,d){return{value:r,root:i,parent:o,type:l,props:f,children:b,line:$G,column:eC,length:d,return:""}}function gjt(){return Rh}function pjt(){return Rh=O2>0?RG(tC,--O2):0,eC--,Rh===10&&(eC=1,$G--),Rh}function Lv(){return Rh=O2<ZCe?RG(tC,O2++):0,eC++,Rh===10&&(eC=1,$G++),Rh}function Z8(){return RG(tC,O2)}function HG(){return O2}function zG(r,i){return cI(tC,r,i)}function yae(r){switch(r){case 0:case 9:case 10:case 13:case 32:return 5;case 33:case 43:case 44:case 47:case 62:case 64:case 126:case 59:case 123:case 125:return 4;case 58:return 3;case 34:case 39:case 40:case 91:return 2;case 41:case 93:return 1}return 0}function bjt(r){return $G=eC=1,ZCe=f6(tC=r),O2=0,[]}function vjt(r){return tC="",r}function kae(r){return XCe(zG(O2-1,xae(r===91?r+2:r===40?r+1:r)))}function wjt(r){for(;(Rh=Z8())&&Rh<33;)Lv();return yae(r)>2||yae(Rh)>3?"":" "}function mjt(r,i){for(;--i&&Lv()&&!(Rh<48||Rh>102||Rh>57&&Rh<65||Rh>70&&Rh<97););return zG(r,HG()+(i<6&&Z8()==32&&Lv()==32))}function xae(r){for(;Lv();)switch(Rh){case r:return O2;case 34:case 39:r!==34&&r!==39&&xae(Rh);break;case 40:r===41&&xae(r);break;case 92:Lv();break}return O2}function yjt(r,i){for(;Lv()&&r+Rh!==47+10;)if(r+Rh===42+42&&Z8()===47)break;return"/*"+zG(i,O2-1)+"*"+vae(r===47?r:Lv())}function kjt(r){for(;!yae(Z8());)Lv();return zG(r,O2)}function xjt(r){return vjt(GG("",null,null,null,[""],r=bjt(r),0,[0],r))}function GG(r,i,o,l,f,b,d,w,y){for(var k=0,E=0,T=d,C=0,S=0,L=0,O=1,B=1,N=1,F=0,R="",q=f,X=b,te=l,H=R;B;)switch(L=F,F=Lv()){case 40:if(L!=108&&RG(H,T-1)==58){djt(H+=wae(kae(F),"&","&\f"),"&\f")!=-1&&(N=-1);break}case 34:case 39:case 91:H+=kae(F);break;case 9:case 10:case 13:case 32:H+=wjt(L);break;case 92:H+=mjt(HG()-1,7);continue;case 47:switch(Z8()){case 42:case 47:jG(Ejt(yjt(Lv(),HG()),i,o),y);break;default:H+="/"}break;case 123*O:w[k++]=f6(H)*N;case 125*O:case 59:case 0:switch(F){case 0:case 125:B=0;case 59+E:S>0&&f6(H)-T&&jG(S>32?eSe(H+";",l,o,T-1):eSe(wae(H," ","")+";",l,o,T-2),y);break;case 59:H+=";";default:if(jG(te=JCe(H,i,o,k,E,f,w,R,q=[],X=[],T),b),F===123)if(E===0)GG(H,i,te,te,q,b,T,w,X);else switch(C){case 100:case 109:case 115:GG(r,te,te,l&&jG(JCe(r,te,te,0,0,f,w,R,f,q=[],T),X),f,X,T,w,l?q:X);break;default:GG(H,te,te,te,[""],X,0,w,X)}}k=E=S=0,O=N=1,R=H="",T=d;break;case 58:T=1+f6(H),S=L;default:if(O<1){if(F==123)--O;else if(F==125&&O++==0&&pjt()==125)continue}switch(H+=vae(F),F*O){case 38:N=E>0?1:(H+="\f",-1);break;case 44:w[k++]=(f6(H)-1)*N,N=1;break;case 64:Z8()===45&&(H+=kae(Lv())),C=Z8(),E=T=f6(R=H+=kjt(HG())),F++;break;case 45:L===45&&f6(H)==2&&(O=0)}}return b}function JCe(r,i,o,l,f,b,d,w,y,k,E){for(var T=f-1,C=f===0?b:[""],S=QCe(C),L=0,O=0,B=0;L<l;++L)for(var N=0,F=cI(r,T+1,T=fjt(O=d[L])),R=r;N<S;++N)(R=XCe(O>0?C[N]+" "+F:wae(F,/&\f/g,C[N])))&&(y[B++]=R);return mae(r,i,o,f===0?WCe:w,y,k,E)}function Ejt(r,i,o){return mae(r,i,o,YCe,vae(gjt()),cI(r,2,-2),0)}function eSe(r,i,o,l){return mae(r,i,o,KCe,cI(r,0,l),cI(r,l+1,-1),l)}function Eae(r,i){for(var o="",l=QCe(r),f=0;f<l;f++)o+=i(r[f],f,r,i)||"";return o}function _jt(r,i,o,l){switch(r.type){case ljt:case KCe:return r.return=r.return||r.value;case YCe:return"";case hjt:return r.return=r.value+"{"+Eae(r.children,l)+"}";case WCe:r.value=r.props.join(",")}return f6(o=Eae(r.children,l))?r.return=r.value+"{"+o+"}":""}const VG={name:"mermaid",version:"9.4.0",description:"Markdown-ish syntax for generating flowcharts, sequence diagrams, class diagrams, gantt charts and git graphs.",main:"./dist/mermaid.min.js",module:"./dist/mermaid.core.mjs",types:"./dist/mermaid.d.ts",exports:{".":{require:"./dist/mermaid.min.js",import:"./dist/mermaid.core.mjs",types:"./dist/mermaid.d.ts"},"./*":"./*"},keywords:["diagram","markdown","flowchart","sequence diagram","gantt","class diagram","git graph"],scripts:{clean:"rimraf dist","docs:code":"typedoc src/defaultConfig.ts src/config.ts src/mermaidAPI.ts && prettier --write ./src/docs/config/setup","docs:build":"rimraf ../../docs && pnpm docs:spellcheck && pnpm docs:code && ts-node-esm src/docs.mts","docs:verify":"pnpm docs:spellcheck && pnpm docs:code && ts-node-esm src/docs.mts --verify","docs:pre:vitepress":"rimraf src/vitepress && pnpm docs:code && ts-node-esm src/docs.mts --vitepress","docs:build:vitepress":"pnpm docs:pre:vitepress && vitepress build src/vitepress && cpy --flat src/docs/landing/ ./src/vitepress/.vitepress/dist/landing","docs:dev":'pnpm docs:pre:vitepress && concurrently "vitepress dev src/vitepress" "ts-node-esm src/docs.mts --watch --vitepress"',"docs:serve":"pnpm docs:build:vitepress && vitepress serve src/vitepress","docs:spellcheck":'cspell --config ../../cSpell.json "src/docs/**/*.md"',release:"pnpm build",prepublishOnly:"pnpm -w run build"},repository:{type:"git",url:"https://github.com/mermaid-js/mermaid"},author:"Knut Sveidqvist",license:"MIT",standard:{ignore:["**/parser/*.js","dist/**/*.js","cypress/**/*.js"],globals:["page"]},dependencies:{"@braintree/sanitize-url":"^6.0.0",cytoscape:"^3.23.0","cytoscape-cose-bilkent":"^4.1.0","cytoscape-fcose":"^2.1.0",d3:"^7.0.0","dagre-d3-es":"7.0.8",dompurify:"2.4.3",elkjs:"^0.8.2",khroma:"^2.0.0","lodash-es":"^4.17.21",moment:"^2.29.4","non-layered-tidy-tree-layout":"^2.0.2",stylis:"^4.1.2","ts-dedent":"^2.2.0",uuid:"^9.0.0"},devDependencies:{"@types/cytoscape":"^3.19.9","@types/d3":"^7.4.0","@types/dompurify":"^2.4.0","@types/jsdom":"^20.0.1","@types/lodash-es":"^4.17.6","@types/micromatch":"^4.0.2","@types/prettier":"^2.7.1","@types/stylis":"^4.0.2","@types/uuid":"^8.3.4","@typescript-eslint/eslint-plugin":"^5.42.1","@typescript-eslint/parser":"^5.42.1",chokidar:"^3.5.3",concurrently:"^7.5.0",coveralls:"^3.1.1","cpy-cli":"^4.2.0",cspell:"^6.14.3",globby:"^13.1.2",jison:"^0.4.18","js-base64":"^3.7.2",jsdom:"^20.0.2",micromatch:"^4.0.5","path-browserify":"^1.0.1",prettier:"^2.7.1",remark:"^14.0.2","remark-frontmatter":"^4.0.1","remark-gfm":"^3.0.1",rimraf:"^3.0.2","start-server-and-test":"^1.14.0",typedoc:"^0.23.18","typedoc-plugin-markdown":"^3.13.6",typescript:"^4.8.4","unist-util-flatmap":"^1.0.0",vitepress:"^1.0.0-alpha.31","vitepress-plugin-search":"^1.0.4-alpha.16"},files:["dist","README.md"],sideEffects:["**/*.css","**/*.scss"]},nC=Object.freeze(XPt);let Y0=Yf({},nC),tSe,rC=[],uI=Yf({},nC);const UG=(r,i)=>{let o=Yf({},r),l={};for(const f of i)iSe(f),l=Yf(l,f);if(o=Yf(o,l),l.theme&&l.theme in q3){const f=Yf({},tSe),b=Yf(f.themeVariables||{},l.themeVariables);o.theme&&o.theme in q3&&(o.themeVariables=q3[o.theme].getThemeVariables(b))}return uI=o,oSe(uI),uI},Tjt=r=>(Y0=Yf({},nC),Y0=Yf(Y0,r),r.theme&&q3[r.theme]&&(Y0.themeVariables=q3[r.theme].getThemeVariables(r.themeVariables)),UG(Y0,rC),Y0),Cjt=r=>{tSe=Yf({},r)},Sjt=r=>(Y0=Yf(Y0,r),UG(Y0,rC),Y0),nSe=()=>Yf({},Y0),rSe=r=>(oSe(r),Yf(uI,r),ht()),ht=()=>Yf({},uI),iSe=r=>{var i;["secure",...(i=Y0.secure)!=null?i:[]].forEach(o=>{r[o]!==void 0&&(Se.debug(`Denied attempt to modify a secure key ${o}`,r[o]),delete r[o])}),Object.keys(r).forEach(o=>{o.indexOf("__")===0&&delete r[o]}),Object.keys(r).forEach(o=>{typeof r[o]=="string"&&(r[o].includes("<")||r[o].includes(">")||r[o].includes("url(data:"))&&delete r[o],typeof r[o]=="object"&&iSe(r[o])})},_ae=r=>{r.fontFamily&&(r.themeVariables?r.themeVariables.fontFamily||(r.themeVariables={fontFamily:r.fontFamily}):r.themeVariables={fontFamily:r.fontFamily}),rC.push(r),UG(Y0,rC)},lI=(r=Y0)=>{rC=[],UG(r,rC)};var sSe=(r=>(r.LAZY_LOAD_DEPRECATED="The configuration options lazyLoadedDiagrams and loadExternalDiagramsAtStartup are deprecated. Please use registerExternalDiagrams instead.",r))(sSe||{});const aSe={},Ajt=r=>{aSe[r]||(Se.warn(sSe[r]),aSe[r]=!0)},oSe=r=>{!r||(r.lazyLoadedDiagrams||r.loadExternalDiagramsAtStartup)&&Ajt("LAZY_LOAD_DEPRECATED")},Mjt=function(r,i){for(let o of i)r.attr(o[0],o[1])},Djt=function(r,i,o){let l=new Map;return o?(l.set("width","100%"),l.set("style",`max-width: ${i}px;`)):(l.set("height",r),l.set("width",i)),l},Iv=function(r,i,o,l){const f=Djt(i,o,l);Mjt(r,f)},J8=function(r,i,o,l){const f=i.node().getBBox(),b=f.width,d=f.height;Se.info(`SVG bounds: ${b}x${d}`,f);let w=0,y=0;Se.info(`Graph bounds: ${w}x${y}`,r),w=b+o*2,y=d+o*2,Se.info(`Calculated bounds: ${w}x${y}`),Iv(i,y,w,l);const k=`${f.x-o} ${f.y-o} ${f.width+2*o} ${f.height+2*o}`;i.attr("viewBox",k)},hI=r=>`g.classGroup text {
  fill: ${r.nodeBorder};
  fill: ${r.classText};
  stroke: none;
  font-family: ${r.fontFamily};
  font-size: 10px;

  .title {
    font-weight: bolder;
  }

}

.nodeLabel, .edgeLabel {
  color: ${r.classText};
}
.edgeLabel .label rect {
  fill: ${r.mainBkg};
}
.label text {
  fill: ${r.classText};
}
.edgeLabel .label span {
  background: ${r.mainBkg};
}

.classTitle {
  font-weight: bolder;
}
.node rect,
  .node circle,
  .node ellipse,
  .node polygon,
  .node path {
    fill: ${r.mainBkg};
    stroke: ${r.nodeBorder};
    stroke-width: 1px;
  }


.divider {
  stroke: ${r.nodeBorder};
  stroke: 1;
}

g.clickable {
  cursor: pointer;
}

g.classGroup rect {
  fill: ${r.mainBkg};
  stroke: ${r.nodeBorder};
}

g.classGroup line {
  stroke: ${r.nodeBorder};
  stroke-width: 1;
}

.classLabel .box {
  stroke: none;
  stroke-width: 0;
  fill: ${r.mainBkg};
  opacity: 0.5;
}

.classLabel .label {
  fill: ${r.nodeBorder};
  font-size: 10px;
}

.relation {
  stroke: ${r.lineColor};
  stroke-width: 1;
  fill: none;
}

.dashed-line{
  stroke-dasharray: 3;
}

.dotted-line{
  stroke-dasharray: 1 2;
}

#compositionStart, .composition {
  fill: ${r.lineColor} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#compositionEnd, .composition {
  fill: ${r.lineColor} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#dependencyStart, .dependency {
  fill: ${r.lineColor} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#dependencyStart, .dependency {
  fill: ${r.lineColor} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#extensionStart, .extension {
  fill: ${r.mainBkg} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#extensionEnd, .extension {
  fill: ${r.mainBkg} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#aggregationStart, .aggregation {
  fill: ${r.mainBkg} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#aggregationEnd, .aggregation {
  fill: ${r.mainBkg} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#lollipopStart, .lollipop {
  fill: ${r.mainBkg} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

#lollipopEnd, .lollipop {
  fill: ${r.mainBkg} !important;
  stroke: ${r.lineColor} !important;
  stroke-width: 1;
}

.edgeTerminals {
  font-size: 11px;
}

.classTitleText {
  text-anchor: middle;
  font-size: 18px;
  fill: ${r.textColor};
}
`,cSe=r=>`
  .entityBox {
    fill: ${r.mainBkg};
    stroke: ${r.nodeBorder};
  }

  .attributeBoxOdd {
    fill: ${r.attributeBackgroundColorOdd};
    stroke: ${r.nodeBorder};
  }

  .attributeBoxEven {
    fill:  ${r.attributeBackgroundColorEven};
    stroke: ${r.nodeBorder};
  }

  .relationshipLabelBox {
    fill: ${r.tertiaryColor};
    opacity: 0.7;
    background-color: ${r.tertiaryColor};
      rect {
        opacity: 0.5;
      }
  }

    .relationshipLine {
      stroke: ${r.lineColor};
    }

  .entityTitleText {
    text-anchor: middle;
    font-size: 18px;
    fill: ${r.textColor};
  }    
`,Tae=()=>"",qG=r=>`.label {
    font-family: ${r.fontFamily};
    color: ${r.nodeTextColor||r.textColor};
  }
  .cluster-label text {
    fill: ${r.titleColor};
  }
  .cluster-label span {
    color: ${r.titleColor};
  }

  .label text,span {
    fill: ${r.nodeTextColor||r.textColor};
    color: ${r.nodeTextColor||r.textColor};
  }

  .node rect,
  .node circle,
  .node ellipse,
  .node polygon,
  .node path {
    fill: ${r.mainBkg};
    stroke: ${r.nodeBorder};
    stroke-width: 1px;
  }

  .node .label {
    text-align: center;
  }
  .node.clickable {
    cursor: pointer;
  }

  .arrowheadPath {
    fill: ${r.arrowheadColor};
  }

  .edgePath .path {
    stroke: ${r.lineColor};
    stroke-width: 2.0px;
  }

  .flowchart-link {
    stroke: ${r.lineColor};
    fill: none;
  }

  .edgeLabel {
    background-color: ${r.edgeLabelBackground};
    rect {
      opacity: 0.5;
      background-color: ${r.edgeLabelBackground};
      fill: ${r.edgeLabelBackground};
    }
    text-align: center;
  }

  .cluster rect {
    fill: ${r.clusterBkg};
    stroke: ${r.clusterBorder};
    stroke-width: 1px;
  }

  .cluster text {
    fill: ${r.titleColor};
  }

  .cluster span {
    color: ${r.titleColor};
  }
  /* .cluster div {
    color: ${r.titleColor};
  } */

  div.mermaidTooltip {
    position: absolute;
    text-align: center;
    max-width: 200px;
    padding: 2px;
    font-family: ${r.fontFamily};
    font-size: 12px;
    background: ${r.tertiaryColor};
    border: 1px solid ${r.border2};
    border-radius: 2px;
    pointer-events: none;
    z-index: 100;
  }

  .flowchartTitleText {
    text-anchor: middle;
    font-size: 18px;
    fill: ${r.textColor};
  }
`,uSe=r=>`
  .mermaid-main-font {
    font-family: "trebuchet ms", verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);
  }
  .exclude-range {
    fill: ${r.excludeBkgColor};
  }

  .section {
    stroke: none;
    opacity: 0.2;
  }

  .section0 {
    fill: ${r.sectionBkgColor};
  }

  .section2 {
    fill: ${r.sectionBkgColor2};
  }

  .section1,
  .section3 {
    fill: ${r.altSectionBkgColor};
    opacity: 0.2;
  }

  .sectionTitle0 {
    fill: ${r.titleColor};
  }

  .sectionTitle1 {
    fill: ${r.titleColor};
  }

  .sectionTitle2 {
    fill: ${r.titleColor};
  }

  .sectionTitle3 {
    fill: ${r.titleColor};
  }

  .sectionTitle {
    text-anchor: start;
    // font-size: ${r.ganttFontSize};
    // text-height: 14px;
    font-family: 'trebuchet ms', verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);

  }


  /* Grid and axis */

  .grid .tick {
    stroke: ${r.gridColor};
    opacity: 0.8;
    shape-rendering: crispEdges;
    text {
      font-family: ${r.fontFamily};
      fill: ${r.textColor};
    }
  }

  .grid path {
    stroke-width: 0;
  }


  /* Today line */

  .today {
    fill: none;
    stroke: ${r.todayLineColor};
    stroke-width: 2px;
  }


  /* Task styling */

  /* Default task */

  .task {
    stroke-width: 2;
  }

  .taskText {
    text-anchor: middle;
    font-family: 'trebuchet ms', verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);
  }

  // .taskText:not([font-size]) {
  //   font-size: ${r.ganttFontSize};
  // }

  .taskTextOutsideRight {
    fill: ${r.taskTextDarkColor};
    text-anchor: start;
    // font-size: ${r.ganttFontSize};
    font-family: 'trebuchet ms', verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);

  }

  .taskTextOutsideLeft {
    fill: ${r.taskTextDarkColor};
    text-anchor: end;
    // font-size: ${r.ganttFontSize};
  }

  /* Special case clickable */
  .task.clickable {
    cursor: pointer;
  }
  .taskText.clickable {
    cursor: pointer;
    fill: ${r.taskTextClickableColor} !important;
    font-weight: bold;
  }

  .taskTextOutsideLeft.clickable {
    cursor: pointer;
    fill: ${r.taskTextClickableColor} !important;
    font-weight: bold;
  }

  .taskTextOutsideRight.clickable {
    cursor: pointer;
    fill: ${r.taskTextClickableColor} !important;
    font-weight: bold;
  }

  /* Specific task settings for the sections*/

  .taskText0,
  .taskText1,
  .taskText2,
  .taskText3 {
    fill: ${r.taskTextColor};
  }

  .task0,
  .task1,
  .task2,
  .task3 {
    fill: ${r.taskBkgColor};
    stroke: ${r.taskBorderColor};
  }

  .taskTextOutside0,
  .taskTextOutside2
  {
    fill: ${r.taskTextOutsideColor};
  }

  .taskTextOutside1,
  .taskTextOutside3 {
    fill: ${r.taskTextOutsideColor};
  }


  /* Active task */

  .active0,
  .active1,
  .active2,
  .active3 {
    fill: ${r.activeTaskBkgColor};
    stroke: ${r.activeTaskBorderColor};
  }

  .activeText0,
  .activeText1,
  .activeText2,
  .activeText3 {
    fill: ${r.taskTextDarkColor} !important;
  }


  /* Completed task */

  .done0,
  .done1,
  .done2,
  .done3 {
    stroke: ${r.doneTaskBorderColor};
    fill: ${r.doneTaskBkgColor};
    stroke-width: 2;
  }

  .doneText0,
  .doneText1,
  .doneText2,
  .doneText3 {
    fill: ${r.taskTextDarkColor} !important;
  }


  /* Tasks on the critical line */

  .crit0,
  .crit1,
  .crit2,
  .crit3 {
    stroke: ${r.critBorderColor};
    fill: ${r.critBkgColor};
    stroke-width: 2;
  }

  .activeCrit0,
  .activeCrit1,
  .activeCrit2,
  .activeCrit3 {
    stroke: ${r.critBorderColor};
    fill: ${r.activeTaskBkgColor};
    stroke-width: 2;
  }

  .doneCrit0,
  .doneCrit1,
  .doneCrit2,
  .doneCrit3 {
    stroke: ${r.critBorderColor};
    fill: ${r.doneTaskBkgColor};
    stroke-width: 2;
    cursor: pointer;
    shape-rendering: crispEdges;
  }

  .milestone {
    transform: rotate(45deg) scale(0.8,0.8);
  }

  .milestoneText {
    font-style: italic;
  }
  .doneCritText0,
  .doneCritText1,
  .doneCritText2,
  .doneCritText3 {
    fill: ${r.taskTextDarkColor} !important;
  }

  .activeCritText0,
  .activeCritText1,
  .activeCritText2,
  .activeCritText3 {
    fill: ${r.taskTextDarkColor} !important;
  }

  .titleText {
    text-anchor: middle;
    font-size: 18px;
    fill: ${r.textColor}    ;
    font-family: 'trebuchet ms', verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);
  }
`,lSe=()=>"",hSe=r=>`
  .pieCircle{
    stroke: ${r.pieStrokeColor};
    stroke-width : ${r.pieStrokeWidth};
    opacity : ${r.pieOpacity};
  }
  .pieTitleText {
    text-anchor: middle;
    font-size: ${r.pieTitleTextSize};
    fill: ${r.pieTitleTextColor};
    font-family: ${r.fontFamily};
  }
  .slice {
    font-family: ${r.fontFamily};
    fill: ${r.pieSectionTextColor};
    font-size:${r.pieSectionTextSize};
    // fill: white;
  }
  .legend text {
    fill: ${r.pieLegendTextColor};
    font-family: ${r.fontFamily};
    font-size: ${r.pieLegendTextSize};
  }
`,fSe=r=>`

  marker {
    fill: ${r.relationColor};
    stroke: ${r.relationColor};
  }

  marker.cross {
    stroke: ${r.lineColor};
  }

  svg {
    font-family: ${r.fontFamily};
    font-size: ${r.fontSize};
  }

  .reqBox {
    fill: ${r.requirementBackground};
    fill-opacity: 100%;
    stroke: ${r.requirementBorderColor};
    stroke-width: ${r.requirementBorderSize};
  }
  
  .reqTitle, .reqLabel{
    fill:  ${r.requirementTextColor};
  }
  .reqLabelBox {
    fill: ${r.relationLabelBackground};
    fill-opacity: 100%;
  }

  .req-title-line {
    stroke: ${r.requirementBorderColor};
    stroke-width: ${r.requirementBorderSize};
  }
  .relationshipLine {
    stroke: ${r.relationColor};
    stroke-width: 1;
  }
  .relationshipLabel {
    fill: ${r.relationLabelColor};
  }

`,dSe=r=>`.actor {
    stroke: ${r.actorBorder};
    fill: ${r.actorBkg};
  }

  text.actor > tspan {
    fill: ${r.actorTextColor};
    stroke: none;
  }

  .actor-line {
    stroke: ${r.actorLineColor};
  }

  .messageLine0 {
    stroke-width: 1.5;
    stroke-dasharray: none;
    stroke: ${r.signalColor};
  }

  .messageLine1 {
    stroke-width: 1.5;
    stroke-dasharray: 2, 2;
    stroke: ${r.signalColor};
  }

  #arrowhead path {
    fill: ${r.signalColor};
    stroke: ${r.signalColor};
  }

  .sequenceNumber {
    fill: ${r.sequenceNumberColor};
  }

  #sequencenumber {
    fill: ${r.signalColor};
  }

  #crosshead path {
    fill: ${r.signalColor};
    stroke: ${r.signalColor};
  }

  .messageText {
    fill: ${r.signalTextColor};
    stroke: none;
  }

  .labelBox {
    stroke: ${r.labelBoxBorderColor};
    fill: ${r.labelBoxBkgColor};
  }

  .labelText, .labelText > tspan {
    fill: ${r.labelTextColor};
    stroke: none;
  }

  .loopText, .loopText > tspan {
    fill: ${r.loopTextColor};
    stroke: none;
  }

  .loopLine {
    stroke-width: 2px;
    stroke-dasharray: 2, 2;
    stroke: ${r.labelBoxBorderColor};
    fill: ${r.labelBoxBorderColor};
  }

  .note {
    //stroke: #decc93;
    stroke: ${r.noteBorderColor};
    fill: ${r.noteBkgColor};
  }

  .noteText, .noteText > tspan {
    fill: ${r.noteTextColor};
    stroke: none;
  }

  .activation0 {
    fill: ${r.activationBkgColor};
    stroke: ${r.activationBorderColor};
  }

  .activation1 {
    fill: ${r.activationBkgColor};
    stroke: ${r.activationBorderColor};
  }

  .activation2 {
    fill: ${r.activationBkgColor};
    stroke: ${r.activationBorderColor};
  }

  .actorPopupMenu {
    position: absolute;
  }

  .actorPopupMenuPanel {
    position: absolute;
    fill: ${r.actorBkg};
    box-shadow: 0px 8px 16px 0px rgba(0,0,0,0.2);
    filter: drop-shadow(3px 5px 2px rgb(0 0 0 / 0.4));
}
  .actor-man line {
    stroke: ${r.actorBorder};
    fill: ${r.actorBkg};
  }
  .actor-man circle, line {
    stroke: ${r.actorBorder};
    fill: ${r.actorBkg};
    stroke-width: 2px;
  }
`,YG=r=>`
defs #statediagram-barbEnd {
    fill: ${r.transitionColor};
    stroke: ${r.transitionColor};
  }
g.stateGroup text {
  fill: ${r.nodeBorder};
  stroke: none;
  font-size: 10px;
}
g.stateGroup text {
  fill: ${r.textColor};
  stroke: none;
  font-size: 10px;

}
g.stateGroup .state-title {
  font-weight: bolder;
  fill: ${r.stateLabelColor};
}

g.stateGroup rect {
  fill: ${r.mainBkg};
  stroke: ${r.nodeBorder};
}

g.stateGroup line {
  stroke: ${r.lineColor};
  stroke-width: 1;
}

.transition {
  stroke: ${r.transitionColor};
  stroke-width: 1;
  fill: none;
}

.stateGroup .composit {
  fill: ${r.background};
  border-bottom: 1px
}

.stateGroup .alt-composit {
  fill: #e0e0e0;
  border-bottom: 1px
}

.state-note {
  stroke: ${r.noteBorderColor};
  fill: ${r.noteBkgColor};

  text {
    fill: ${r.noteTextColor};
    stroke: none;
    font-size: 10px;
  }
}

.stateLabel .box {
  stroke: none;
  stroke-width: 0;
  fill: ${r.mainBkg};
  opacity: 0.5;
}

.edgeLabel .label rect {
  fill: ${r.labelBackgroundColor};
  opacity: 0.5;
}
.edgeLabel .label text {
  fill: ${r.transitionLabelColor||r.tertiaryTextColor};
}
.label div .edgeLabel {
  color: ${r.transitionLabelColor||r.tertiaryTextColor};
}

.stateLabel text {
  fill: ${r.stateLabelColor};
  font-size: 10px;
  font-weight: bold;
}

.node circle.state-start {
  fill: ${r.specialStateColor};
  stroke: ${r.specialStateColor};
}

.node .fork-join {
  fill: ${r.specialStateColor};
  stroke: ${r.specialStateColor};
}

.node circle.state-end {
  fill: ${r.innerEndBackground};
  stroke: ${r.background};
  stroke-width: 1.5
}
.end-state-inner {
  fill: ${r.compositeBackground||r.background};
  // stroke: ${r.background};
  stroke-width: 1.5
}

.node rect {
  fill: ${r.stateBkg||r.mainBkg};
  stroke: ${r.stateBorder||r.nodeBorder};
  stroke-width: 1px;
}
.node polygon {
  fill: ${r.mainBkg};
  stroke: ${r.stateBorder||r.nodeBorder};;
  stroke-width: 1px;
}
#statediagram-barbEnd {
  fill: ${r.lineColor};
}

.statediagram-cluster rect {
  fill: ${r.compositeTitleBackground};
  stroke: ${r.stateBorder||r.nodeBorder};
  stroke-width: 1px;
}

.cluster-label, .nodeLabel {
  color: ${r.stateLabelColor};
}

.statediagram-cluster rect.outer {
  rx: 5px;
  ry: 5px;
}
.statediagram-state .divider {
  stroke: ${r.stateBorder||r.nodeBorder};
}

.statediagram-state .title-state {
  rx: 5px;
  ry: 5px;
}
.statediagram-cluster.statediagram-cluster .inner {
  fill: ${r.compositeBackground||r.background};
}
.statediagram-cluster.statediagram-cluster-alt .inner {
  fill: ${r.altBackground?r.altBackground:"#efefef"};
}

.statediagram-cluster .inner {
  rx:0;
  ry:0;
}

.statediagram-state rect.basic {
  rx: 5px;
  ry: 5px;
}
.statediagram-state rect.divider {
  stroke-dasharray: 10,10;
  fill: ${r.altBackground?r.altBackground:"#efefef"};
}

.note-edge {
  stroke-dasharray: 5;
}

.statediagram-note rect {
  fill: ${r.noteBkgColor};
  stroke: ${r.noteBorderColor};
  stroke-width: 1px;
  rx: 0;
  ry: 0;
}
.statediagram-note rect {
  fill: ${r.noteBkgColor};
  stroke: ${r.noteBorderColor};
  stroke-width: 1px;
  rx: 0;
  ry: 0;
}

.statediagram-note text {
  fill: ${r.noteTextColor};
}

.statediagram-note .nodeLabel {
  color: ${r.noteTextColor};
}
.statediagram .edgeLabel {
  color: red; // ${r.noteTextColor};
}

#dependencyStart, #dependencyEnd {
  fill: ${r.lineColor};
  stroke: ${r.lineColor};
  stroke-width: 1;
}

.statediagramTitleText {
  text-anchor: middle;
  font-size: 18px;
  fill: ${r.textColor};
}
`,gSe=r=>`.label {
    font-family: 'trebuchet ms', verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);
    color: ${r.textColor};
  }
  .mouth {
    stroke: #666;
  }

  line {
    stroke: ${r.textColor}
  }

  .legend {
    fill: ${r.textColor};
  }

  .label text {
    fill: #333;
  }
  .label {
    color: ${r.textColor}
  }

  .face {
    ${r.faceColor?`fill: ${r.faceColor}`:"fill: #FFF8DC"};
    stroke: #999;
  }

  .node rect,
  .node circle,
  .node ellipse,
  .node polygon,
  .node path {
    fill: ${r.mainBkg};
    stroke: ${r.nodeBorder};
    stroke-width: 1px;
  }

  .node .label {
    text-align: center;
  }
  .node.clickable {
    cursor: pointer;
  }

  .arrowheadPath {
    fill: ${r.arrowheadColor};
  }

  .edgePath .path {
    stroke: ${r.lineColor};
    stroke-width: 1.5px;
  }

  .flowchart-link {
    stroke: ${r.lineColor};
    fill: none;
  }

  .edgeLabel {
    background-color: ${r.edgeLabelBackground};
    rect {
      opacity: 0.5;
    }
    text-align: center;
  }

  .cluster rect {
  }

  .cluster text {
    fill: ${r.titleColor};
  }

  div.mermaidTooltip {
    position: absolute;
    text-align: center;
    max-width: 200px;
    padding: 2px;
    font-family: 'trebuchet ms', verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);
    font-size: 12px;
    background: ${r.tertiaryColor};
    border: 1px solid ${r.border2};
    border-radius: 2px;
    pointer-events: none;
    z-index: 100;
  }

  .task-type-0, .section-type-0  {
    ${r.fillType0?`fill: ${r.fillType0}`:""};
  }
  .task-type-1, .section-type-1  {
    ${r.fillType0?`fill: ${r.fillType1}`:""};
  }
  .task-type-2, .section-type-2  {
    ${r.fillType0?`fill: ${r.fillType2}`:""};
  }
  .task-type-3, .section-type-3  {
    ${r.fillType0?`fill: ${r.fillType3}`:""};
  }
  .task-type-4, .section-type-4  {
    ${r.fillType0?`fill: ${r.fillType4}`:""};
  }
  .task-type-5, .section-type-5  {
    ${r.fillType0?`fill: ${r.fillType5}`:""};
  }
  .task-type-6, .section-type-6  {
    ${r.fillType0?`fill: ${r.fillType6}`:""};
  }
  .task-type-7, .section-type-7  {
    ${r.fillType0?`fill: ${r.fillType7}`:""};
  }

  .actor-0 {
    ${r.actor0?`fill: ${r.actor0}`:""};
  }
  .actor-1 {
    ${r.actor1?`fill: ${r.actor1}`:""};
  }
  .actor-2 {
    ${r.actor2?`fill: ${r.actor2}`:""};
  }
  .actor-3 {
    ${r.actor3?`fill: ${r.actor3}`:""};
  }
  .actor-4 {
    ${r.actor4?`fill: ${r.actor4}`:""};
  }
  .actor-5 {
    ${r.actor5?`fill: ${r.actor5}`:""};
  }
`,pSe=r=>`.person {
    stroke: ${r.personBorder};
    fill: ${r.personBkg};
  }
`,WG={flowchart:qG,"flowchart-v2":qG,sequence:dSe,gantt:uSe,classDiagram:hI,"classDiagram-v2":hI,class:hI,stateDiagram:YG,state:YG,info:lSe,pie:hSe,er:cSe,error:Tae,journey:gSe,requirement:fSe,c4:pSe},Ljt=(r,i,o)=>{let l="";return r in WG&&WG[r]?l=WG[r](o):Se.warn(`No theme found for ${r}`),` & {
    font-family: ${o.fontFamily};
    font-size: ${o.fontSize};
    fill: ${o.textColor}
  }

  /* Classes common for multiple diagrams */

  & .error-icon {
    fill: ${o.errorBkgColor};
  }
  & .error-text {
    fill: ${o.errorTextColor};
    stroke: ${o.errorTextColor};
  }

  & .edge-thickness-normal {
    stroke-width: 2px;
  }
  & .edge-thickness-thick {
    stroke-width: 3.5px
  }
  & .edge-pattern-solid {
    stroke-dasharray: 0;
  }

  & .edge-pattern-dashed{
    stroke-dasharray: 3;
  }
  .edge-pattern-dotted {
    stroke-dasharray: 2;
  }

  & .marker {
    fill: ${o.lineColor};
    stroke: ${o.lineColor};
  }
  & .marker.cross {
    stroke: ${o.lineColor};
  }

  & svg {
    font-family: ${o.fontFamily};
    font-size: ${o.fontSize};
  }

  ${l}

  ${i}
`},Ijt=(r,i)=>{WG[r]=i},Ojt=Ljt;let Cae="",KG="",Sae="";const Aae=r=>up(r,ht()),W0=function(){Cae="",Sae="",KG=""},K0=function(r){Cae=Aae(r).replace(/^\s+/g,"")},hp=function(){return Cae||KG},fp=function(r){Sae=Aae(r).replace(/\n\s+/g,`
`)},dp=function(){return Sae},Ov=function(r){KG=Aae(r)},Nv=function(){return KG},bSe=Object.freeze(Object.defineProperty({__proto__:null,clear:W0,setAccTitle:K0,getAccTitle:hp,setAccDescription:fp,getAccDescription:dp,setDiagramTitle:Ov,getDiagramTitle:Nv,default:{setAccTitle:K0,getAccTitle:hp,setDiagramTitle:Ov,getDiagramTitle:Nv,getAccDescription:dp,setAccDescription:fp,clear:W0}},Symbol.toStringTag,{value:"Module"}));let e7={};const Mae=function(r,i,o,l){Se.debug("parseDirective is being called",i,o,l);try{if(i!==void 0)switch(i=i.trim(),o){case"open_directive":e7={};break;case"type_directive":if(!e7)throw new Error("currentDirective is undefined");e7.type=i.toLowerCase();break;case"arg_directive":if(!e7)throw new Error("currentDirective is undefined");e7.args=JSON.parse(i);break;case"close_directive":Njt(r,e7,l),e7=void 0;break}}catch(f){Se.error(`Error while rendering sequenceDiagram directive: ${i} jison context: ${o}`),Se.error(f.message)}},Njt=function(r,i,o){switch(Se.info(`Directive type=${i.type} with args:`,i.args),i.type){case"init":case"initialize":{["config"].forEach(l=>{i.args[l]!==void 0&&(o==="flowchart-v2"&&(o="flowchart"),i.args[o]=i.args[l],delete i.args[l])}),Se.info("sanitize in handleDirective",i.args),Q8(i.args),Se.info("sanitize in handleDirective (done)",i.args),_ae(i.args);break}case"wrap":case"nowrap":r&&r.setWrap&&r.setWrap(i.type==="wrap");break;case"themeCss":Se.warn("themeCss encountered");break;default:Se.warn(`Unhandled directive: source: '%%{${i.type}: ${JSON.stringify(i.args?i.args:{})}}%%`,i);break}},Pjt=Se,Bjt=Wie,fI=ht,Fjt=r=>up(r,fI()),vSe=J8,Rjt=()=>bSe,jjt=(r,i,o,l)=>Mae(r,i,o,l),XG={},uf=(r,i,o)=>{if(XG[r])throw new Error(`Diagram ${r} already registered.`);XG[r]=i,o&&FCe(r,o),Ijt(r,i.styles),i.injectUtils&&i.injectUtils(Pjt,Bjt,fI,Fjt,vSe,Rjt(),jjt)},wSe=r=>{if(r in XG)return XG[r];throw new Error(`Diagram ${r} not found.`)};var Dae=function(){var r=function(Y,z,W,Z){for(W=W||{},Z=Y.length;Z--;W[Y[Z]]=z);return W},i=[1,4],o=[1,7],l=[1,5],f=[1,9],b=[1,6],d=[2,6],w=[1,16],y=[6,8,14,20,22,24,25,27,29,32,37,40,50,55],k=[8,14,20,22,24,25,27,29,32,37,40],E=[8,13,14,20,22,24,25,27,29,32,37,40],T=[1,26],C=[6,8,14,50,55],S=[8,14,55],L=[1,53],O=[1,52],B=[8,14,30,33,35,38,55],N=[1,67],F=[1,68],R=[1,69],q=[8,14,33,35,42,55],X={trace:function(){},yy:{},symbols_:{error:2,start:3,eol:4,directive:5,GG:6,document:7,EOF:8,":":9,DIR:10,options:11,body:12,OPT:13,NL:14,line:15,statement:16,commitStatement:17,mergeStatement:18,cherryPickStatement:19,acc_title:20,acc_title_value:21,acc_descr:22,acc_descr_value:23,acc_descr_multiline_value:24,section:25,branchStatement:26,CHECKOUT:27,ref:28,BRANCH:29,ORDER:30,NUM:31,CHERRY_PICK:32,COMMIT_ID:33,STR:34,COMMIT_TAG:35,EMPTYSTR:36,MERGE:37,COMMIT_TYPE:38,commitType:39,COMMIT:40,commit_arg:41,COMMIT_MSG:42,NORMAL:43,REVERSE:44,HIGHLIGHT:45,openDirective:46,typeDirective:47,closeDirective:48,argDirective:49,open_directive:50,type_directive:51,arg_directive:52,close_directive:53,ID:54,";":55,$accept:0,$end:1},terminals_:{2:"error",6:"GG",8:"EOF",9:":",10:"DIR",13:"OPT",14:"NL",20:"acc_title",21:"acc_title_value",22:"acc_descr",23:"acc_descr_value",24:"acc_descr_multiline_value",25:"section",27:"CHECKOUT",29:"BRANCH",30:"ORDER",31:"NUM",32:"CHERRY_PICK",33:"COMMIT_ID",34:"STR",35:"COMMIT_TAG",36:"EMPTYSTR",37:"MERGE",38:"COMMIT_TYPE",40:"COMMIT",42:"COMMIT_MSG",43:"NORMAL",44:"REVERSE",45:"HIGHLIGHT",50:"open_directive",51:"type_directive",52:"arg_directive",53:"close_directive",54:"ID",55:";"},productions_:[0,[3,2],[3,2],[3,3],[3,4],[3,5],[7,0],[7,2],[11,2],[11,1],[12,0],[12,2],[15,2],[15,1],[16,1],[16,1],[16,1],[16,2],[16,2],[16,1],[16,1],[16,1],[16,2],[26,2],[26,4],[19,3],[19,5],[19,5],[19,5],[19,5],[18,2],[18,4],[18,4],[18,4],[18,6],[18,6],[18,6],[18,6],[18,6],[18,6],[18,8],[18,8],[18,8],[18,8],[18,8],[18,8],[17,2],[17,3],[17,3],[17,5],[17,5],[17,3],[17,5],[17,5],[17,5],[17,5],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,3],[17,5],[17,5],[17,5],[17,5],[17,5],[17,5],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,7],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[17,9],[41,0],[41,1],[39,1],[39,1],[39,1],[5,3],[5,5],[46,1],[47,1],[49,1],[48,1],[28,1],[28,1],[4,1],[4,1],[4,1]],performAction:function(z,W,Z,G,ae,$,ge){var ee=$.length-1;switch(ae){case 3:return $[ee];case 4:return $[ee-1];case 5:return G.setDirection($[ee-3]),$[ee-1];case 7:G.setOptions($[ee-1]),this.$=$[ee];break;case 8:$[ee-1]+=$[ee],this.$=$[ee-1];break;case 10:this.$=[];break;case 11:$[ee-1].push($[ee]),this.$=$[ee-1];break;case 12:this.$=$[ee-1];break;case 17:this.$=$[ee].trim(),G.setAccTitle(this.$);break;case 18:case 19:this.$=$[ee].trim(),G.setAccDescription(this.$);break;case 20:G.addSection($[ee].substr(8)),this.$=$[ee].substr(8);break;case 22:G.checkout($[ee]);break;case 23:G.branch($[ee]);break;case 24:G.branch($[ee-2],$[ee]);break;case 25:G.cherryPick($[ee],"",void 0);break;case 26:G.cherryPick($[ee-2],"",$[ee]);break;case 27:case 29:G.cherryPick($[ee-2],"","");break;case 28:G.cherryPick($[ee],"",$[ee-2]);break;case 30:G.merge($[ee],"","","");break;case 31:G.merge($[ee-2],$[ee],"","");break;case 32:G.merge($[ee-2],"",$[ee],"");break;case 33:G.merge($[ee-2],"","",$[ee]);break;case 34:G.merge($[ee-4],$[ee],"",$[ee-2]);break;case 35:G.merge($[ee-4],"",$[ee],$[ee-2]);break;case 36:G.merge($[ee-4],"",$[ee-2],$[ee]);break;case 37:G.merge($[ee-4],$[ee-2],$[ee],"");break;case 38:G.merge($[ee-4],$[ee-2],"",$[ee]);break;case 39:G.merge($[ee-4],$[ee],$[ee-2],"");break;case 40:G.merge($[ee-6],$[ee-4],$[ee-2],$[ee]);break;case 41:G.merge($[ee-6],$[ee],$[ee-4],$[ee-2]);break;case 42:G.merge($[ee-6],$[ee-4],$[ee],$[ee-2]);break;case 43:G.merge($[ee-6],$[ee-2],$[ee-4],$[ee]);break;case 44:G.merge($[ee-6],$[ee],$[ee-2],$[ee-4]);break;case 45:G.merge($[ee-6],$[ee-2],$[ee],$[ee-4]);break;case 46:G.commit($[ee]);break;case 47:G.commit("","",G.commitType.NORMAL,$[ee]);break;case 48:G.commit("","",$[ee],"");break;case 49:G.commit("","",$[ee],$[ee-2]);break;case 50:G.commit("","",$[ee-2],$[ee]);break;case 51:G.commit("",$[ee],G.commitType.NORMAL,"");break;case 52:G.commit("",$[ee-2],G.commitType.NORMAL,$[ee]);break;case 53:G.commit("",$[ee],G.commitType.NORMAL,$[ee-2]);break;case 54:G.commit("",$[ee-2],$[ee],"");break;case 55:G.commit("",$[ee],$[ee-2],"");break;case 56:G.commit("",$[ee-4],$[ee-2],$[ee]);break;case 57:G.commit("",$[ee-4],$[ee],$[ee-2]);break;case 58:G.commit("",$[ee-2],$[ee-4],$[ee]);break;case 59:G.commit("",$[ee],$[ee-4],$[ee-2]);break;case 60:G.commit("",$[ee],$[ee-2],$[ee-4]);break;case 61:G.commit("",$[ee-2],$[ee],$[ee-4]);break;case 62:G.commit($[ee],"",G.commitType.NORMAL,"");break;case 63:G.commit($[ee],"",G.commitType.NORMAL,$[ee-2]);break;case 64:G.commit($[ee-2],"",G.commitType.NORMAL,$[ee]);break;case 65:G.commit($[ee-2],"",$[ee],"");break;case 66:G.commit($[ee],"",$[ee-2],"");break;case 67:G.commit($[ee],$[ee-2],G.commitType.NORMAL,"");break;case 68:G.commit($[ee-2],$[ee],G.commitType.NORMAL,"");break;case 69:G.commit($[ee-4],"",$[ee-2],$[ee]);break;case 70:G.commit($[ee-4],"",$[ee],$[ee-2]);break;case 71:G.commit($[ee-2],"",$[ee-4],$[ee]);break;case 72:G.commit($[ee],"",$[ee-4],$[ee-2]);break;case 73:G.commit($[ee],"",$[ee-2],$[ee-4]);break;case 74:G.commit($[ee-2],"",$[ee],$[ee-4]);break;case 75:G.commit($[ee-4],$[ee],$[ee-2],"");break;case 76:G.commit($[ee-4],$[ee-2],$[ee],"");break;case 77:G.commit($[ee-2],$[ee],$[ee-4],"");break;case 78:G.commit($[ee],$[ee-2],$[ee-4],"");break;case 79:G.commit($[ee],$[ee-4],$[ee-2],"");break;case 80:G.commit($[ee-2],$[ee-4],$[ee],"");break;case 81:G.commit($[ee-4],$[ee],G.commitType.NORMAL,$[ee-2]);break;case 82:G.commit($[ee-4],$[ee-2],G.commitType.NORMAL,$[ee]);break;case 83:G.commit($[ee-2],$[ee],G.commitType.NORMAL,$[ee-4]);break;case 84:G.commit($[ee],$[ee-2],G.commitType.NORMAL,$[ee-4]);break;case 85:G.commit($[ee],$[ee-4],G.commitType.NORMAL,$[ee-2]);break;case 86:G.commit($[ee-2],$[ee-4],G.commitType.NORMAL,$[ee]);break;case 87:G.commit($[ee-6],$[ee-4],$[ee-2],$[ee]);break;case 88:G.commit($[ee-6],$[ee-4],$[ee],$[ee-2]);break;case 89:G.commit($[ee-6],$[ee-2],$[ee-4],$[ee]);break;case 90:G.commit($[ee-6],$[ee],$[ee-4],$[ee-2]);break;case 91:G.commit($[ee-6],$[ee-2],$[ee],$[ee-4]);break;case 92:G.commit($[ee-6],$[ee],$[ee-2],$[ee-4]);break;case 93:G.commit($[ee-4],$[ee-6],$[ee-2],$[ee]);break;case 94:G.commit($[ee-4],$[ee-6],$[ee],$[ee-2]);break;case 95:G.commit($[ee-2],$[ee-6],$[ee-4],$[ee]);break;case 96:G.commit($[ee],$[ee-6],$[ee-4],$[ee-2]);break;case 97:G.commit($[ee-2],$[ee-6],$[ee],$[ee-4]);break;case 98:G.commit($[ee],$[ee-6],$[ee-2],$[ee-4]);break;case 99:G.commit($[ee],$[ee-4],$[ee-2],$[ee-6]);break;case 100:G.commit($[ee-2],$[ee-4],$[ee],$[ee-6]);break;case 101:G.commit($[ee],$[ee-2],$[ee-4],$[ee-6]);break;case 102:G.commit($[ee-2],$[ee],$[ee-4],$[ee-6]);break;case 103:G.commit($[ee-4],$[ee-2],$[ee],$[ee-6]);break;case 104:G.commit($[ee-4],$[ee],$[ee-2],$[ee-6]);break;case 105:G.commit($[ee-2],$[ee-4],$[ee-6],$[ee]);break;case 106:G.commit($[ee],$[ee-4],$[ee-6],$[ee-2]);break;case 107:G.commit($[ee-2],$[ee],$[ee-6],$[ee-4]);break;case 108:G.commit($[ee],$[ee-2],$[ee-6],$[ee-4]);break;case 109:G.commit($[ee-4],$[ee-2],$[ee-6],$[ee]);break;case 110:G.commit($[ee-4],$[ee],$[ee-6],$[ee-2]);break;case 111:this.$="";break;case 112:this.$=$[ee];break;case 113:this.$=G.commitType.NORMAL;break;case 114:this.$=G.commitType.REVERSE;break;case 115:this.$=G.commitType.HIGHLIGHT;break;case 118:G.parseDirective("%%{","open_directive");break;case 119:G.parseDirective($[ee],"type_directive");break;case 120:$[ee]=$[ee].trim().replace(/'/g,'"'),G.parseDirective($[ee],"arg_directive");break;case 121:G.parseDirective("}%%","close_directive","gitGraph");break}},table:[{3:1,4:2,5:3,6:i,8:o,14:l,46:8,50:f,55:b},{1:[3]},{3:10,4:2,5:3,6:i,8:o,14:l,46:8,50:f,55:b},{3:11,4:2,5:3,6:i,8:o,14:l,46:8,50:f,55:b},{7:12,8:d,9:[1,13],10:[1,14],11:15,14:w},r(y,[2,124]),r(y,[2,125]),r(y,[2,126]),{47:17,51:[1,18]},{51:[2,118]},{1:[2,1]},{1:[2,2]},{8:[1,19]},{7:20,8:d,11:15,14:w},{9:[1,21]},r(k,[2,10],{12:22,13:[1,23]}),r(E,[2,9]),{9:[1,25],48:24,53:T},r([9,53],[2,119]),{1:[2,3]},{8:[1,27]},{7:28,8:d,11:15,14:w},{8:[2,7],14:[1,31],15:29,16:30,17:32,18:33,19:34,20:[1,35],22:[1,36],24:[1,37],25:[1,38],26:39,27:[1,40],29:[1,44],32:[1,43],37:[1,42],40:[1,41]},r(E,[2,8]),r(C,[2,116]),{49:45,52:[1,46]},r(C,[2,121]),{1:[2,4]},{8:[1,47]},r(k,[2,11]),{4:48,8:o,14:l,55:b},r(k,[2,13]),r(S,[2,14]),r(S,[2,15]),r(S,[2,16]),{21:[1,49]},{23:[1,50]},r(S,[2,19]),r(S,[2,20]),r(S,[2,21]),{28:51,34:L,54:O},r(S,[2,111],{41:54,33:[1,57],34:[1,59],35:[1,55],38:[1,56],42:[1,58]}),{28:60,34:L,54:O},{33:[1,61],35:[1,62]},{28:63,34:L,54:O},{48:64,53:T},{53:[2,120]},{1:[2,5]},r(k,[2,12]),r(S,[2,17]),r(S,[2,18]),r(S,[2,22]),r(B,[2,122]),r(B,[2,123]),r(S,[2,46]),{34:[1,65]},{39:66,43:N,44:F,45:R},{34:[1,70]},{34:[1,71]},r(S,[2,112]),r(S,[2,30],{33:[1,72],35:[1,74],38:[1,73]}),{34:[1,75]},{34:[1,76],36:[1,77]},r(S,[2,23],{30:[1,78]}),r(C,[2,117]),r(S,[2,47],{33:[1,80],38:[1,79],42:[1,81]}),r(S,[2,48],{33:[1,83],35:[1,82],42:[1,84]}),r(q,[2,113]),r(q,[2,114]),r(q,[2,115]),r(S,[2,51],{35:[1,85],38:[1,86],42:[1,87]}),r(S,[2,62],{33:[1,90],35:[1,88],38:[1,89]}),{34:[1,91]},{39:92,43:N,44:F,45:R},{34:[1,93]},r(S,[2,25],{35:[1,94]}),{33:[1,95]},{33:[1,96]},{31:[1,97]},{39:98,43:N,44:F,45:R},{34:[1,99]},{34:[1,100]},{34:[1,101]},{34:[1,102]},{34:[1,103]},{34:[1,104]},{39:105,43:N,44:F,45:R},{34:[1,106]},{34:[1,107]},{39:108,43:N,44:F,45:R},{34:[1,109]},r(S,[2,31],{35:[1,111],38:[1,110]}),r(S,[2,32],{33:[1,113],35:[1,112]}),r(S,[2,33],{33:[1,114],38:[1,115]}),{34:[1,116],36:[1,117]},{34:[1,118]},{34:[1,119]},r(S,[2,24]),r(S,[2,49],{33:[1,120],42:[1,121]}),r(S,[2,53],{38:[1,122],42:[1,123]}),r(S,[2,63],{33:[1,125],38:[1,124]}),r(S,[2,50],{33:[1,126],42:[1,127]}),r(S,[2,55],{35:[1,128],42:[1,129]}),r(S,[2,66],{33:[1,131],35:[1,130]}),r(S,[2,52],{38:[1,132],42:[1,133]}),r(S,[2,54],{35:[1,134],42:[1,135]}),r(S,[2,67],{35:[1,137],38:[1,136]}),r(S,[2,64],{33:[1,139],38:[1,138]}),r(S,[2,65],{33:[1,141],35:[1,140]}),r(S,[2,68],{35:[1,143],38:[1,142]}),{39:144,43:N,44:F,45:R},{34:[1,145]},{34:[1,146]},{34:[1,147]},{34:[1,148]},{39:149,43:N,44:F,45:R},r(S,[2,26]),r(S,[2,27]),r(S,[2,28]),r(S,[2,29]),{34:[1,150]},{34:[1,151]},{39:152,43:N,44:F,45:R},{34:[1,153]},{39:154,43:N,44:F,45:R},{34:[1,155]},{34:[1,156]},{34:[1,157]},{34:[1,158]},{34:[1,159]},{34:[1,160]},{34:[1,161]},{39:162,43:N,44:F,45:R},{34:[1,163]},{34:[1,164]},{34:[1,165]},{39:166,43:N,44:F,45:R},{34:[1,167]},{39:168,43:N,44:F,45:R},{34:[1,169]},{34:[1,170]},{34:[1,171]},{39:172,43:N,44:F,45:R},{34:[1,173]},r(S,[2,37],{35:[1,174]}),r(S,[2,38],{38:[1,175]}),r(S,[2,36],{33:[1,176]}),r(S,[2,39],{35:[1,177]}),r(S,[2,34],{38:[1,178]}),r(S,[2,35],{33:[1,179]}),r(S,[2,60],{42:[1,180]}),r(S,[2,73],{33:[1,181]}),r(S,[2,61],{42:[1,182]}),r(S,[2,84],{38:[1,183]}),r(S,[2,74],{33:[1,184]}),r(S,[2,83],{38:[1,185]}),r(S,[2,59],{42:[1,186]}),r(S,[2,72],{33:[1,187]}),r(S,[2,58],{42:[1,188]}),r(S,[2,78],{35:[1,189]}),r(S,[2,71],{33:[1,190]}),r(S,[2,77],{35:[1,191]}),r(S,[2,57],{42:[1,192]}),r(S,[2,85],{38:[1,193]}),r(S,[2,56],{42:[1,194]}),r(S,[2,79],{35:[1,195]}),r(S,[2,80],{35:[1,196]}),r(S,[2,86],{38:[1,197]}),r(S,[2,70],{33:[1,198]}),r(S,[2,81],{38:[1,199]}),r(S,[2,69],{33:[1,200]}),r(S,[2,75],{35:[1,201]}),r(S,[2,76],{35:[1,202]}),r(S,[2,82],{38:[1,203]}),{34:[1,204]},{39:205,43:N,44:F,45:R},{34:[1,206]},{34:[1,207]},{39:208,43:N,44:F,45:R},{34:[1,209]},{34:[1,210]},{34:[1,211]},{34:[1,212]},{39:213,43:N,44:F,45:R},{34:[1,214]},{39:215,43:N,44:F,45:R},{34:[1,216]},{34:[1,217]},{34:[1,218]},{34:[1,219]},{34:[1,220]},{34:[1,221]},{34:[1,222]},{39:223,43:N,44:F,45:R},{34:[1,224]},{34:[1,225]},{34:[1,226]},{39:227,43:N,44:F,45:R},{34:[1,228]},{39:229,43:N,44:F,45:R},{34:[1,230]},{34:[1,231]},{34:[1,232]},{39:233,43:N,44:F,45:R},r(S,[2,40]),r(S,[2,42]),r(S,[2,41]),r(S,[2,43]),r(S,[2,45]),r(S,[2,44]),r(S,[2,101]),r(S,[2,102]),r(S,[2,99]),r(S,[2,100]),r(S,[2,104]),r(S,[2,103]),r(S,[2,108]),r(S,[2,107]),r(S,[2,106]),r(S,[2,105]),r(S,[2,110]),r(S,[2,109]),r(S,[2,98]),r(S,[2,97]),r(S,[2,96]),r(S,[2,95]),r(S,[2,93]),r(S,[2,94]),r(S,[2,92]),r(S,[2,91]),r(S,[2,90]),r(S,[2,89]),r(S,[2,87]),r(S,[2,88])],defaultActions:{9:[2,118],10:[2,1],11:[2,2],19:[2,3],27:[2,4],46:[2,120],47:[2,5]},parseError:function(z,W){if(W.recoverable)this.trace(z);else{var Z=new Error(z);throw Z.hash=W,Z}},parse:function(z){var W=this,Z=[0],G=[],ae=[null],$=[],ge=this.table,ee="",de=0,re=0,ke=2,Ce=1,_e=$.slice.call(arguments,1),Te=Object.create(this.lexer),Be={yy:{}};for(var Ge in this.yy)Object.prototype.hasOwnProperty.call(this.yy,Ge)&&(Be.yy[Ge]=this.yy[Ge]);Te.setInput(z,Be.yy),Be.yy.lexer=Te,Be.yy.parser=this,typeof Te.yylloc>"u"&&(Te.yylloc={});var Xe=Te.yylloc;$.push(Xe);var Ee=Te.options&&Te.options.ranges;typeof Be.yy.parseError=="function"?this.parseError=Be.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Ze(){var Zt;return Zt=G.pop()||Te.lex()||Ce,typeof Zt!="number"&&(Zt instanceof Array&&(G=Zt,Zt=G.pop()),Zt=W.symbols_[Zt]||Zt),Zt}for(var Ie,Oe,Le,$e,xe={},Ae,je,me,vt;;){if(Oe=Z[Z.length-1],this.defaultActions[Oe]?Le=this.defaultActions[Oe]:((Ie===null||typeof Ie>"u")&&(Ie=Ze()),Le=ge[Oe]&&ge[Oe][Ie]),typeof Le>"u"||!Le.length||!Le[0]){var ve="";vt=[];for(Ae in ge[Oe])this.terminals_[Ae]&&Ae>ke&&vt.push("'"+this.terminals_[Ae]+"'");Te.showPosition?ve="Parse error on line "+(de+1)+`:
`+Te.showPosition()+`
Expecting `+vt.join(", ")+", got '"+(this.terminals_[Ie]||Ie)+"'":ve="Parse error on line "+(de+1)+": Unexpected "+(Ie==Ce?"end of input":"'"+(this.terminals_[Ie]||Ie)+"'"),this.parseError(ve,{text:Te.match,token:this.terminals_[Ie]||Ie,line:Te.yylineno,loc:Xe,expected:vt})}if(Le[0]instanceof Array&&Le.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Oe+", token: "+Ie);switch(Le[0]){case 1:Z.push(Ie),ae.push(Te.yytext),$.push(Te.yylloc),Z.push(Le[1]),Ie=null,re=Te.yyleng,ee=Te.yytext,de=Te.yylineno,Xe=Te.yylloc;break;case 2:if(je=this.productions_[Le[1]][1],xe.$=ae[ae.length-je],xe._$={first_line:$[$.length-(je||1)].first_line,last_line:$[$.length-1].last_line,first_column:$[$.length-(je||1)].first_column,last_column:$[$.length-1].last_column},Ee&&(xe._$.range=[$[$.length-(je||1)].range[0],$[$.length-1].range[1]]),$e=this.performAction.apply(xe,[ee,re,de,Be.yy,Le[1],ae,$].concat(_e)),typeof $e<"u")return $e;je&&(Z=Z.slice(0,-1*je*2),ae=ae.slice(0,-1*je),$=$.slice(0,-1*je)),Z.push(this.productions_[Le[1]][0]),ae.push(xe.$),$.push(xe._$),me=ge[Z[Z.length-2]][Z[Z.length-1]],Z.push(me);break;case 3:return!0}}return!0}},te=function(){var Y={EOF:1,parseError:function(W,Z){if(this.yy.parser)this.yy.parser.parseError(W,Z);else throw new Error(W)},setInput:function(z,W){return this.yy=W||this.yy||{},this._input=z,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var z=this._input[0];this.yytext+=z,this.yyleng++,this.offset++,this.match+=z,this.matched+=z;var W=z.match(/(?:\r\n?|\n).*/g);return W?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),z},unput:function(z){var W=z.length,Z=z.split(/(?:\r\n?|\n)/g);this._input=z+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-W),this.offset-=W;var G=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),Z.length-1&&(this.yylineno-=Z.length-1);var ae=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:Z?(Z.length===G.length?this.yylloc.first_column:0)+G[G.length-Z.length].length-Z[0].length:this.yylloc.first_column-W},this.options.ranges&&(this.yylloc.range=[ae[0],ae[0]+this.yyleng-W]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(z){this.unput(this.match.slice(z))},pastInput:function(){var z=this.matched.substr(0,this.matched.length-this.match.length);return(z.length>20?"...":"")+z.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var z=this.match;return z.length<20&&(z+=this._input.substr(0,20-z.length)),(z.substr(0,20)+(z.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var z=this.pastInput(),W=new Array(z.length+1).join("-");return z+this.upcomingInput()+`
`+W+"^"},test_match:function(z,W){var Z,G,ae;if(this.options.backtrack_lexer&&(ae={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(ae.yylloc.range=this.yylloc.range.slice(0))),G=z[0].match(/(?:\r\n?|\n).*/g),G&&(this.yylineno+=G.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:G?G[G.length-1].length-G[G.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+z[0].length},this.yytext+=z[0],this.match+=z[0],this.matches=z,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(z[0].length),this.matched+=z[0],Z=this.performAction.call(this,this.yy,this,W,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),Z)return Z;if(this._backtrack){for(var $ in ae)this[$]=ae[$];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var z,W,Z,G;this._more||(this.yytext="",this.match="");for(var ae=this._currentRules(),$=0;$<ae.length;$++)if(Z=this._input.match(this.rules[ae[$]]),Z&&(!W||Z[0].length>W[0].length)){if(W=Z,G=$,this.options.backtrack_lexer){if(z=this.test_match(Z,ae[$]),z!==!1)return z;if(this._backtrack){W=!1;continue}else return!1}else if(!this.options.flex)break}return W?(z=this.test_match(W,ae[G]),z!==!1?z:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var W=this.next();return W||this.lex()},begin:function(W){this.conditionStack.push(W)},popState:function(){var W=this.conditionStack.length-1;return W>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(W){return W=this.conditionStack.length-1-Math.abs(W||0),W>=0?this.conditionStack[W]:"INITIAL"},pushState:function(W){this.begin(W)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(W,Z,G,ae){switch(G){case 0:return this.begin("open_directive"),50;case 1:return this.begin("type_directive"),51;case 2:return this.popState(),this.begin("arg_directive"),9;case 3:return this.popState(),this.popState(),53;case 4:return 52;case 5:return this.begin("acc_title"),20;case 6:return this.popState(),"acc_title_value";case 7:return this.begin("acc_descr"),22;case 8:return this.popState(),"acc_descr_value";case 9:this.begin("acc_descr_multiline");break;case 10:this.popState();break;case 11:return"acc_descr_multiline_value";case 12:return 14;case 13:break;case 14:break;case 15:return 6;case 16:return 40;case 17:return 33;case 18:return 38;case 19:return 42;case 20:return 43;case 21:return 44;case 22:return 45;case 23:return 35;case 24:return 29;case 25:return 30;case 26:return 37;case 27:return 32;case 28:return 27;case 29:return 10;case 30:return 10;case 31:return 9;case 32:return"CARET";case 33:this.begin("options");break;case 34:this.popState();break;case 35:return 13;case 36:return 36;case 37:this.begin("string");break;case 38:this.popState();break;case 39:return 34;case 40:return 31;case 41:return 54;case 42:return 8}},rules:[/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:(\r?\n)+)/i,/^(?:#[^\n]*)/i,/^(?:%[^\n]*)/i,/^(?:gitGraph\b)/i,/^(?:commit(?=\s|$))/i,/^(?:id:)/i,/^(?:type:)/i,/^(?:msg:)/i,/^(?:NORMAL\b)/i,/^(?:REVERSE\b)/i,/^(?:HIGHLIGHT\b)/i,/^(?:tag:)/i,/^(?:branch(?=\s|$))/i,/^(?:order:)/i,/^(?:merge(?=\s|$))/i,/^(?:cherry-pick(?=\s|$))/i,/^(?:checkout(?=\s|$))/i,/^(?:LR\b)/i,/^(?:BT\b)/i,/^(?::)/i,/^(?:\^)/i,/^(?:options\r?\n)/i,/^(?:[ \r\n\t]+end\b)/i,/^(?:[\s\S]+(?=[ \r\n\t]+end))/i,/^(?:["]["])/i,/^(?:["])/i,/^(?:["])/i,/^(?:[^"]*)/i,/^(?:[0-9]+(?=\s|$))/i,/^(?:\w([-\./\w]*[-\w])?)/i,/^(?:$)/i,/^(?:\s+)/i],conditions:{acc_descr_multiline:{rules:[10,11],inclusive:!1},acc_descr:{rules:[8],inclusive:!1},acc_title:{rules:[6],inclusive:!1},close_directive:{rules:[],inclusive:!1},arg_directive:{rules:[3,4],inclusive:!1},type_directive:{rules:[2,3],inclusive:!1},open_directive:{rules:[1],inclusive:!1},options:{rules:[34,35],inclusive:!1},string:{rules:[38,39],inclusive:!1},INITIAL:{rules:[0,5,7,9,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,36,37,40,41,42,43],inclusive:!0}}};return Y}();X.lexer=te;function H(){this.yy={}}return H.prototype=X,X.Parser=H,new H}();Dae.parser=Dae;const $jt=Dae,Hjt=r=>r.match(/^\s*gitGraph/)!==null;let QG=ht().gitGraph.mainBranchName,zjt=ht().gitGraph.mainBranchOrder,lf={},Zd=null,dI={};dI[QG]={name:QG,order:zjt};let jh={};jh[QG]=Zd;let Wf=QG,mSe="LR",t7=0;function Lae(){return UCe({length:7})}const Gjt=function(r,i,o){Bu.parseDirective(this,r,i,o)};function Vjt(r,i){const o=Object.create(null);return r.reduce((l,f)=>{const b=i(f);return o[b]||(o[b]=!0,l.push(f)),l},[])}const Ujt=function(r){mSe=r};let ySe={};const qjt=function(r){Se.debug("options str",r),r=r&&r.trim(),r=r||"{}";try{ySe=JSON.parse(r)}catch(i){Se.error("error while parsing gitGraph options",i.message)}},Yjt=function(){return ySe},Wjt=function(r,i,o,l){Se.debug("Entering commit:",r,i,o,l),i=Cs.sanitizeText(i,ht()),r=Cs.sanitizeText(r,ht()),l=Cs.sanitizeText(l,ht());const f={id:i||t7+"-"+Lae(),message:r,seq:t7++,type:o||gI.NORMAL,tag:l||"",parents:Zd==null?[]:[Zd.id],branch:Wf};Zd=f,lf[f.id]=f,jh[Wf]=f.id,Se.debug("in pushCommit "+f.id)},Kjt=function(r,i){if(r=Cs.sanitizeText(r,ht()),jh[r]===void 0)jh[r]=Zd!=null?Zd.id:null,dI[r]={name:r,order:i?parseInt(i,10):null},kSe(r),Se.debug("in createBranch");else{let o=new Error('Trying to create an existing branch. (Help: Either use a new name if you want create a new branch or try using "checkout '+r+'")');throw o.hash={text:"branch "+r,token:"branch "+r,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:['"checkout '+r+'"']},o}},Xjt=function(r,i,o,l){r=Cs.sanitizeText(r,ht()),i=Cs.sanitizeText(i,ht());const f=lf[jh[Wf]],b=lf[jh[r]];if(Wf===r){let w=new Error('Incorrect usage of "merge". Cannot merge a branch to itself');throw w.hash={text:"merge "+r,token:"merge "+r,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["branch abc"]},w}else if(f===void 0||!f){let w=new Error('Incorrect usage of "merge". Current branch ('+Wf+")has no commits");throw w.hash={text:"merge "+r,token:"merge "+r,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["commit"]},w}else if(jh[r]===void 0){let w=new Error('Incorrect usage of "merge". Branch to be merged ('+r+") does not exist");throw w.hash={text:"merge "+r,token:"merge "+r,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["branch "+r]},w}else if(b===void 0||!b){let w=new Error('Incorrect usage of "merge". Branch to be merged ('+r+") has no commits");throw w.hash={text:"merge "+r,token:"merge "+r,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:['"commit"']},w}else if(f===b){let w=new Error('Incorrect usage of "merge". Both branches have same head');throw w.hash={text:"merge "+r,token:"merge "+r,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["branch abc"]},w}else if(i&&lf[i]!==void 0){let w=new Error('Incorrect usage of "merge". Commit with id:'+i+" already exists, use different custom Id");throw w.hash={text:"merge "+r+i+o+l,token:"merge "+r+i+o+l,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["merge "+r+" "+i+"_UNIQUE "+o+" "+l]},w}const d={id:i||t7+"-"+Lae(),message:"merged branch "+r+" into "+Wf,seq:t7++,parents:[Zd==null?null:Zd.id,jh[r]],branch:Wf,type:gI.MERGE,customType:o,customId:!!i,tag:l||""};Zd=d,lf[d.id]=d,jh[Wf]=d.id,Se.debug(jh),Se.debug("in mergeBranch")},Qjt=function(r,i,o){if(Se.debug("Entering cherryPick:",r,i,o),r=Cs.sanitizeText(r,ht()),i=Cs.sanitizeText(i,ht()),o=Cs.sanitizeText(o,ht()),!r||lf[r]===void 0){let b=new Error('Incorrect usage of "cherryPick". Source commit id should exist and provided');throw b.hash={text:"cherryPick "+r+" "+i,token:"cherryPick "+r+" "+i,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["cherry-pick abc"]},b}let l=lf[r],f=l.branch;if(l.type===gI.MERGE){let b=new Error('Incorrect usage of "cherryPick". Source commit should not be a merge commit');throw b.hash={text:"cherryPick "+r+" "+i,token:"cherryPick "+r+" "+i,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["cherry-pick abc"]},b}if(!i||lf[i]===void 0){if(f===Wf){let w=new Error('Incorrect usage of "cherryPick". Source commit is already on current branch');throw w.hash={text:"cherryPick "+r+" "+i,token:"cherryPick "+r+" "+i,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["cherry-pick abc"]},w}const b=lf[jh[Wf]];if(b===void 0||!b){let w=new Error('Incorrect usage of "cherry-pick". Current branch ('+Wf+")has no commits");throw w.hash={text:"cherryPick "+r+" "+i,token:"cherryPick "+r+" "+i,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["cherry-pick abc"]},w}const d={id:t7+"-"+Lae(),message:"cherry-picked "+l+" into "+Wf,seq:t7++,parents:[Zd==null?null:Zd.id,l.id],branch:Wf,type:gI.CHERRY_PICK,tag:o!=null?o:"cherry-pick:"+l.id};Zd=d,lf[d.id]=d,jh[Wf]=d.id,Se.debug(jh),Se.debug("in cherryPick")}},kSe=function(r){if(r=Cs.sanitizeText(r,ht()),jh[r]===void 0){let i=new Error('Trying to checkout branch which is not yet created. (Help try using "branch '+r+'")');throw i.hash={text:"checkout "+r,token:"checkout "+r,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:['"branch '+r+'"']},i}else{Wf=r;const i=jh[Wf];Zd=lf[i]}};function xSe(r,i,o){const l=r.indexOf(i);l===-1?r.push(o):r.splice(l,1,o)}function ESe(r){const i=r.reduce((f,b)=>f.seq>b.seq?f:b,r[0]);let o="";r.forEach(function(f){f===i?o+="	*":o+="	|"});const l=[o,i.id,i.seq];for(let f in jh)jh[f]===i.id&&l.push(f);if(Se.debug(l.join(" ")),i.parents&&i.parents.length==2){const f=lf[i.parents[0]];xSe(r,i,f),r.push(lf[i.parents[1]])}else{if(i.parents.length==0)return;{const f=lf[i.parents];xSe(r,i,f)}}r=Vjt(r,f=>f.id),ESe(r)}const Zjt=function(){Se.debug(lf);const r=_Se()[0];ESe([r])},Jjt=function(){lf={},Zd=null;let r=ht().gitGraph.mainBranchName,i=ht().gitGraph.mainBranchOrder;jh={},jh[r]=null,dI={},dI[r]={name:r,order:i},Wf=r,t7=0,W0()},e$t=function(){return Object.values(dI).map((i,o)=>i.order!==null?i:{...i,order:parseFloat(`0.${o}`,10)}).sort((i,o)=>i.order-o.order).map(({name:i})=>({name:i}))},t$t=function(){return jh},n$t=function(){return lf},_Se=function(){const r=Object.keys(lf).map(function(i){return lf[i]});return r.forEach(function(i){Se.debug(i.id)}),r.sort((i,o)=>i.seq-o.seq),r},r$t=function(){return Wf},i$t=function(){return mSe},s$t=function(){return Zd},gI={NORMAL:0,REVERSE:1,HIGHLIGHT:2,MERGE:3,CHERRY_PICK:4},a$t={parseDirective:Gjt,getConfig:()=>ht().gitGraph,setDirection:Ujt,setOptions:qjt,getOptions:Yjt,commit:Wjt,branch:Kjt,merge:Xjt,cherryPick:Qjt,checkout:kSe,prettyPrint:Zjt,clear:Jjt,getBranchesAsObjArray:e$t,getBranches:t$t,getCommits:n$t,getCommitsArray:_Se,getCurrentBranch:r$t,getDirection:i$t,getHead:s$t,setAccTitle:K0,getAccTitle:hp,getAccDescription:dp,setAccDescription:fp,setDiagramTitle:Ov,getDiagramTitle:Nv,commitType:gI};let pI={};const gp={NORMAL:0,REVERSE:1,HIGHLIGHT:2,MERGE:3,CHERRY_PICK:4},n7=8;let X0={},ZG={},JG=[],eV=0;const o$t=()=>{X0={},ZG={},pI={},eV=0,JG=[]},c$t=r=>{const i=document.createElementNS("http://www.w3.org/2000/svg","text");let o=[];typeof r=="string"?o=r.split(/\\n|\n|<br\s*\/?>/gi):Array.isArray(r)?o=r:o=[];for(const l of o){const f=document.createElementNS("http://www.w3.org/2000/svg","tspan");f.setAttributeNS("http://www.w3.org/XML/1998/namespace","xml:space","preserve"),f.setAttribute("dy","1em"),f.setAttribute("x","0"),f.setAttribute("class","row"),f.textContent=l.trim(),i.appendChild(f)}return i},TSe=(r,i,o)=>{const l=fI().gitGraph,f=r.append("g").attr("class","commit-bullets"),b=r.append("g").attr("class","commit-labels");let d=0;Object.keys(i).sort((k,E)=>i[k].seq-i[E].seq).forEach(k=>{const E=i[k],T=X0[E.branch].pos,C=d+10;if(o){let S,L=E.customType!==void 0&&E.customType!==""?E.customType:E.type;switch(L){case gp.NORMAL:S="commit-normal";break;case gp.REVERSE:S="commit-reverse";break;case gp.HIGHLIGHT:S="commit-highlight";break;case gp.MERGE:S="commit-merge";break;case gp.CHERRY_PICK:S="commit-cherry-pick";break;default:S="commit-normal"}if(L===gp.HIGHLIGHT){const O=f.append("rect");O.attr("x",C-10),O.attr("y",T-10),O.attr("height",20),O.attr("width",20),O.attr("class",`commit ${E.id} commit-highlight${X0[E.branch].index%n7} ${S}-outer`),f.append("rect").attr("x",C-6).attr("y",T-6).attr("height",12).attr("width",12).attr("class",`commit ${E.id} commit${X0[E.branch].index%n7} ${S}-inner`)}else if(L===gp.CHERRY_PICK)f.append("circle").attr("cx",C).attr("cy",T).attr("r",10).attr("class",`commit ${E.id} ${S}`),f.append("circle").attr("cx",C-3).attr("cy",T+2).attr("r",2.75).attr("fill","#fff").attr("class",`commit ${E.id} ${S}`),f.append("circle").attr("cx",C+3).attr("cy",T+2).attr("r",2.75).attr("fill","#fff").attr("class",`commit ${E.id} ${S}`),f.append("line").attr("x1",C+3).attr("y1",T+1).attr("x2",C).attr("y2",T-5).attr("stroke","#fff").attr("class",`commit ${E.id} ${S}`),f.append("line").attr("x1",C-3).attr("y1",T+1).attr("x2",C).attr("y2",T-5).attr("stroke","#fff").attr("class",`commit ${E.id} ${S}`);else{const O=f.append("circle");if(O.attr("cx",C),O.attr("cy",T),O.attr("r",E.type===gp.MERGE?9:10),O.attr("class",`commit ${E.id} commit${X0[E.branch].index%n7}`),L===gp.MERGE){const B=f.append("circle");B.attr("cx",C),B.attr("cy",T),B.attr("r",6),B.attr("class",`commit ${S} ${E.id} commit${X0[E.branch].index%n7}`)}L===gp.REVERSE&&f.append("path").attr("d",`M ${C-5},${T-5}L${C+5},${T+5}M${C-5},${T+5}L${C+5},${T-5}`).attr("class",`commit ${S} ${E.id} commit${X0[E.branch].index%n7}`)}}if(ZG[E.id]={x:d+10,y:T},o){if(E.type!==gp.CHERRY_PICK&&(E.customId&&E.type===gp.MERGE||E.type!==gp.MERGE)&&l.showCommitLabel){const O=b.append("g"),B=O.insert("rect").attr("class","commit-label-bkg"),N=O.append("text").attr("x",d).attr("y",T+25).attr("class","commit-label").text(E.id);let F=N.node().getBBox();if(B.attr("x",d+10-F.width/2-2).attr("y",T+13.5).attr("width",F.width+2*2).attr("height",F.height+2*2),N.attr("x",d+10-F.width/2),l.rotateCommitLabel){let R=-7.5-(F.width+10)/25*9.5,q=10+F.width/25*8.5;O.attr("transform","translate("+R+", "+q+") rotate("+-45+", "+d+", "+T+")")}}if(E.tag){const O=b.insert("polygon"),B=b.append("circle"),N=b.append("text").attr("y",T-16).attr("class","tag-label").text(E.tag);let F=N.node().getBBox();N.attr("x",d+10-F.width/2);const R=F.height/2,q=T-19.2;O.attr("class","tag-label-bkg").attr("points",`
          ${d-F.width/2-4/2},${q+2}
          ${d-F.width/2-4/2},${q-2}
          ${d+10-F.width/2-4},${q-R-2}
          ${d+10+F.width/2+4},${q-R-2}
          ${d+10+F.width/2+4},${q+R+2}
          ${d+10-F.width/2-4},${q+R+2}`),B.attr("cx",d-F.width/2+4/2).attr("cy",q).attr("r",1.5).attr("class","tag-hole")}}d+=50,d>eV&&(eV=d)})},u$t=(r,i,o)=>Object.keys(o).filter(b=>o[b].branch===i.branch&&o[b].seq>r.seq&&o[b].seq<i.seq).length>0,Iae=(r,i,o=0)=>{const l=r+Math.abs(r-i)/2;if(o>5)return l;if(JG.every(d=>Math.abs(d-l)>=10))return JG.push(l),l;const b=Math.abs(r-i);return Iae(r,i-b/5,o+1)},l$t=(r,i,o,l)=>{const f=ZG[i.id],b=ZG[o.id],d=u$t(i,o,l);let w="",y="",k=0,E=0,T=X0[o.branch].index,C;if(d){w="A 10 10, 0, 0, 0,",y="A 10 10, 0, 0, 1,",k=10,E=10,T=X0[o.branch].index;const S=f.y<b.y?Iae(f.y,b.y):Iae(b.y,f.y);f.y<b.y?C=`M ${f.x} ${f.y} L ${f.x} ${S-k} ${w} ${f.x+E} ${S} L ${b.x-k} ${S} ${y} ${b.x} ${S+E} L ${b.x} ${b.y}`:C=`M ${f.x} ${f.y} L ${f.x} ${S+k} ${y} ${f.x+E} ${S} L ${b.x-k} ${S} ${w} ${b.x} ${S-E} L ${b.x} ${b.y}`}else f.y<b.y&&(w="A 20 20, 0, 0, 0,",k=20,E=20,T=X0[o.branch].index,C=`M ${f.x} ${f.y} L ${f.x} ${b.y-k} ${w} ${f.x+E} ${b.y} L ${b.x} ${b.y}`),f.y>b.y&&(w="A 20 20, 0, 0, 0,",k=20,E=20,T=X0[i.branch].index,C=`M ${f.x} ${f.y} L ${b.x-k} ${f.y} ${w} ${b.x} ${f.y-E} L ${b.x} ${b.y}`),f.y===b.y&&(T=X0[i.branch].index,C=`M ${f.x} ${f.y} L ${f.x} ${b.y-k} ${w} ${f.x+E} ${b.y} L ${b.x} ${b.y}`);r.append("path").attr("d",C).attr("class","arrow arrow"+T%n7)},h$t=(r,i)=>{const o=r.append("g").attr("class","commit-arrows");Object.keys(i).forEach(l=>{const f=i[l];f.parents&&f.parents.length>0&&f.parents.forEach(b=>{l$t(o,i[b],f,i)})})},f$t=(r,i)=>{const o=fI().gitGraph,l=r.append("g");i.forEach((f,b)=>{const d=b%n7,w=X0[f.name].pos,y=l.append("line");y.attr("x1",0),y.attr("y1",w),y.attr("x2",eV),y.attr("y2",w),y.attr("class","branch branch"+d),JG.push(w);let k=f.name;const E=c$t(k),T=l.insert("rect"),S=l.insert("g").attr("class","branchLabel").insert("g").attr("class","label branch-label"+d);S.node().appendChild(E);let L=E.getBBox();T.attr("class","branchLabelBkg label"+d).attr("rx",4).attr("ry",4).attr("x",-L.width-4-(o.rotateCommitLabel===!0?30:0)).attr("y",-L.height/2+8).attr("width",L.width+18).attr("height",L.height+4),S.attr("transform","translate("+(-L.width-14-(o.rotateCommitLabel===!0?30:0))+", "+(w-L.height/2-1)+")"),T.attr("transform","translate("+-19+", "+(w-L.height/2)+")")})},d$t={draw:function(r,i,o,l){var k;o$t();const f=fI(),b=f.gitGraph;Se.debug("in gitgraph renderer",r+`
`,"id:",i,o),pI=l.db.getCommits();const d=l.db.getBranchesAsObjArray();let w=0;d.forEach((E,T)=>{X0[E.name]={pos:w,index:T},w+=50+(b.rotateCommitLabel?40:0)});const y=Cn(`[id="${i}"]`);TSe(y,pI,!1),b.showBranches&&f$t(y,d),h$t(y,pI),TSe(y,pI,!0),Fs.insertTitle(y,"gitTitleText",b.titleTopMargin,l.db.getDiagramTitle()),vSe(void 0,y,b.diagramPadding,(k=b.useMaxWidth)!=null?k:f.useMaxWidth)}},g$t=r=>`
  .commit-id,
  .commit-msg,
  .branch-label {
    fill: lightgrey;
    color: lightgrey;
    font-family: 'trebuchet ms', verdana, arial, sans-serif;
    font-family: var(--mermaid-font-family);
  }
  ${[0,1,2,3,4,5,6,7].map(i=>`
        .branch-label${i} { fill: ${r["gitBranchLabel"+i]}; }
        .commit${i} { stroke: ${r["git"+i]}; fill: ${r["git"+i]}; }
        .commit-highlight${i} { stroke: ${r["gitInv"+i]}; fill: ${r["gitInv"+i]}; }
        .label${i}  { fill: ${r["git"+i]}; }
        .arrow${i} { stroke: ${r["git"+i]}; }
        `).join(`
`)}

  .branch {
    stroke-width: 1;
    stroke: ${r.lineColor};
    stroke-dasharray: 2;
  }
  .commit-label { font-size: ${r.commitLabelFontSize}; fill: ${r.commitLabelColor};}
  .commit-label-bkg { font-size: ${r.commitLabelFontSize}; fill: ${r.commitLabelBackground}; opacity: 0.5; }
  .tag-label { font-size: ${r.tagLabelFontSize}; fill: ${r.tagLabelColor};}
  .tag-label-bkg { fill: ${r.tagLabelBackground}; stroke: ${r.tagLabelBorder}; }
  .tag-hole { fill: ${r.textColor}; }

  .commit-merge {
    stroke: ${r.primaryColor};
    fill: ${r.primaryColor};
  }
  .commit-reverse {
    stroke: ${r.primaryColor};
    fill: ${r.primaryColor};
    stroke-width: 3;
  }
  .commit-highlight-outer {
  }
  .commit-highlight-inner {
    stroke: ${r.primaryColor};
    fill: ${r.primaryColor};
  }

  .arrow { stroke-width: 8; stroke-linecap: round; fill: none}
  .gitTitleText {
    text-anchor: middle;
    font-size: 18px;
    fill: ${r.textColor};
  }
  }
`;var tV=function(){var r=function(Kn,Jt,en,In){for(en=en||{},In=Kn.length;In--;en[Kn[In]]=Jt);return en},i=[1,6],o=[1,7],l=[1,8],f=[1,9],b=[1,16],d=[1,11],w=[1,12],y=[1,13],k=[1,14],E=[1,15],T=[1,27],C=[1,33],S=[1,34],L=[1,35],O=[1,36],B=[1,37],N=[1,72],F=[1,73],R=[1,74],q=[1,75],X=[1,76],te=[1,77],H=[1,78],Y=[1,38],z=[1,39],W=[1,40],Z=[1,41],G=[1,42],ae=[1,43],$=[1,44],ge=[1,45],ee=[1,46],de=[1,47],re=[1,48],ke=[1,49],Ce=[1,50],_e=[1,51],Te=[1,52],Be=[1,53],Ge=[1,54],Xe=[1,55],Ee=[1,56],Ze=[1,57],Ie=[1,59],Oe=[1,60],Le=[1,61],$e=[1,62],xe=[1,63],Ae=[1,64],je=[1,65],me=[1,66],vt=[1,67],ve=[1,68],Zt=[1,69],nt=[24,52],xn=[24,44,46,47,48,49,50,51,52,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84],cn=[15,24,44,46,47,48,49,50,51,52,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84],jt=[1,94],ot=[1,95],be=[1,96],We=[1,97],ct=[15,24,52],Yt=[7,8,9,10,18,22,25,26,27,28],Ut=[15,24,43,52],Wn=[15,24,43,52,86,87,89,90],Gt=[15,43],Rn=[44,46,47,48,49,50,51,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84],si={trace:function(){},yy:{},symbols_:{error:2,start:3,mermaidDoc:4,direction:5,directive:6,direction_tb:7,direction_bt:8,direction_rl:9,direction_lr:10,graphConfig:11,openDirective:12,typeDirective:13,closeDirective:14,NEWLINE:15,":":16,argDirective:17,open_directive:18,type_directive:19,arg_directive:20,close_directive:21,C4_CONTEXT:22,statements:23,EOF:24,C4_CONTAINER:25,C4_COMPONENT:26,C4_DYNAMIC:27,C4_DEPLOYMENT:28,otherStatements:29,diagramStatements:30,otherStatement:31,title:32,accDescription:33,acc_title:34,acc_title_value:35,acc_descr:36,acc_descr_value:37,acc_descr_multiline_value:38,boundaryStatement:39,boundaryStartStatement:40,boundaryStopStatement:41,boundaryStart:42,LBRACE:43,ENTERPRISE_BOUNDARY:44,attributes:45,SYSTEM_BOUNDARY:46,BOUNDARY:47,CONTAINER_BOUNDARY:48,NODE:49,NODE_L:50,NODE_R:51,RBRACE:52,diagramStatement:53,PERSON:54,PERSON_EXT:55,SYSTEM:56,SYSTEM_DB:57,SYSTEM_QUEUE:58,SYSTEM_EXT:59,SYSTEM_EXT_DB:60,SYSTEM_EXT_QUEUE:61,CONTAINER:62,CONTAINER_DB:63,CONTAINER_QUEUE:64,CONTAINER_EXT:65,CONTAINER_EXT_DB:66,CONTAINER_EXT_QUEUE:67,COMPONENT:68,COMPONENT_DB:69,COMPONENT_QUEUE:70,COMPONENT_EXT:71,COMPONENT_EXT_DB:72,COMPONENT_EXT_QUEUE:73,REL:74,BIREL:75,REL_U:76,REL_D:77,REL_L:78,REL_R:79,REL_B:80,REL_INDEX:81,UPDATE_EL_STYLE:82,UPDATE_REL_STYLE:83,UPDATE_LAYOUT_CONFIG:84,attribute:85,STR:86,STR_KEY:87,STR_VALUE:88,ATTRIBUTE:89,ATTRIBUTE_EMPTY:90,$accept:0,$end:1},terminals_:{2:"error",7:"direction_tb",8:"direction_bt",9:"direction_rl",10:"direction_lr",15:"NEWLINE",16:":",18:"open_directive",19:"type_directive",20:"arg_directive",21:"close_directive",22:"C4_CONTEXT",24:"EOF",25:"C4_CONTAINER",26:"C4_COMPONENT",27:"C4_DYNAMIC",28:"C4_DEPLOYMENT",32:"title",33:"accDescription",34:"acc_title",35:"acc_title_value",36:"acc_descr",37:"acc_descr_value",38:"acc_descr_multiline_value",43:"LBRACE",44:"ENTERPRISE_BOUNDARY",46:"SYSTEM_BOUNDARY",47:"BOUNDARY",48:"CONTAINER_BOUNDARY",49:"NODE",50:"NODE_L",51:"NODE_R",52:"RBRACE",54:"PERSON",55:"PERSON_EXT",56:"SYSTEM",57:"SYSTEM_DB",58:"SYSTEM_QUEUE",59:"SYSTEM_EXT",60:"SYSTEM_EXT_DB",61:"SYSTEM_EXT_QUEUE",62:"CONTAINER",63:"CONTAINER_DB",64:"CONTAINER_QUEUE",65:"CONTAINER_EXT",66:"CONTAINER_EXT_DB",67:"CONTAINER_EXT_QUEUE",68:"COMPONENT",69:"COMPONENT_DB",70:"COMPONENT_QUEUE",71:"COMPONENT_EXT",72:"COMPONENT_EXT_DB",73:"COMPONENT_EXT_QUEUE",74:"REL",75:"BIREL",76:"REL_U",77:"REL_D",78:"REL_L",79:"REL_R",80:"REL_B",81:"REL_INDEX",82:"UPDATE_EL_STYLE",83:"UPDATE_REL_STYLE",84:"UPDATE_LAYOUT_CONFIG",86:"STR",87:"STR_KEY",88:"STR_VALUE",89:"ATTRIBUTE",90:"ATTRIBUTE_EMPTY"},productions_:[0,[3,1],[3,1],[3,2],[5,1],[5,1],[5,1],[5,1],[4,1],[6,4],[6,6],[12,1],[13,1],[17,1],[14,1],[11,4],[11,4],[11,4],[11,4],[11,4],[23,1],[23,1],[23,2],[29,1],[29,2],[29,3],[31,1],[31,1],[31,2],[31,2],[31,1],[39,3],[40,3],[40,3],[40,4],[42,2],[42,2],[42,2],[42,2],[42,2],[42,2],[42,2],[41,1],[30,1],[30,2],[30,3],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,1],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[53,2],[45,1],[45,2],[85,1],[85,2],[85,1],[85,1]],performAction:function(Jt,en,In,hn,Fr,Pt,ei){var nn=Pt.length-1;switch(Fr){case 4:hn.setDirection("TB");break;case 5:hn.setDirection("BT");break;case 6:hn.setDirection("RL");break;case 7:hn.setDirection("LR");break;case 11:hn.parseDirective("%%{","open_directive");break;case 12:break;case 13:Pt[nn]=Pt[nn].trim().replace(/'/g,'"'),hn.parseDirective(Pt[nn],"arg_directive");break;case 14:hn.parseDirective("}%%","close_directive","c4Context");break;case 15:case 16:case 17:case 18:case 19:hn.setC4Type(Pt[nn-3]);break;case 26:hn.setTitle(Pt[nn].substring(6)),this.$=Pt[nn].substring(6);break;case 27:hn.setAccDescription(Pt[nn].substring(15)),this.$=Pt[nn].substring(15);break;case 28:this.$=Pt[nn].trim(),hn.setTitle(this.$);break;case 29:case 30:this.$=Pt[nn].trim(),hn.setAccDescription(this.$);break;case 35:case 36:Pt[nn].splice(2,0,"ENTERPRISE"),hn.addPersonOrSystemBoundary(...Pt[nn]),this.$=Pt[nn];break;case 37:hn.addPersonOrSystemBoundary(...Pt[nn]),this.$=Pt[nn];break;case 38:Pt[nn].splice(2,0,"CONTAINER"),hn.addContainerBoundary(...Pt[nn]),this.$=Pt[nn];break;case 39:hn.addDeploymentNode("node",...Pt[nn]),this.$=Pt[nn];break;case 40:hn.addDeploymentNode("nodeL",...Pt[nn]),this.$=Pt[nn];break;case 41:hn.addDeploymentNode("nodeR",...Pt[nn]),this.$=Pt[nn];break;case 42:hn.popBoundaryParseStack();break;case 46:hn.addPersonOrSystem("person",...Pt[nn]),this.$=Pt[nn];break;case 47:hn.addPersonOrSystem("external_person",...Pt[nn]),this.$=Pt[nn];break;case 48:hn.addPersonOrSystem("system",...Pt[nn]),this.$=Pt[nn];break;case 49:hn.addPersonOrSystem("system_db",...Pt[nn]),this.$=Pt[nn];break;case 50:hn.addPersonOrSystem("system_queue",...Pt[nn]),this.$=Pt[nn];break;case 51:hn.addPersonOrSystem("external_system",...Pt[nn]),this.$=Pt[nn];break;case 52:hn.addPersonOrSystem("external_system_db",...Pt[nn]),this.$=Pt[nn];break;case 53:hn.addPersonOrSystem("external_system_queue",...Pt[nn]),this.$=Pt[nn];break;case 54:hn.addContainer("container",...Pt[nn]),this.$=Pt[nn];break;case 55:hn.addContainer("container_db",...Pt[nn]),this.$=Pt[nn];break;case 56:hn.addContainer("container_queue",...Pt[nn]),this.$=Pt[nn];break;case 57:hn.addContainer("external_container",...Pt[nn]),this.$=Pt[nn];break;case 58:hn.addContainer("external_container_db",...Pt[nn]),this.$=Pt[nn];break;case 59:hn.addContainer("external_container_queue",...Pt[nn]),this.$=Pt[nn];break;case 60:hn.addComponent("component",...Pt[nn]),this.$=Pt[nn];break;case 61:hn.addComponent("component_db",...Pt[nn]),this.$=Pt[nn];break;case 62:hn.addComponent("component_queue",...Pt[nn]),this.$=Pt[nn];break;case 63:hn.addComponent("external_component",...Pt[nn]),this.$=Pt[nn];break;case 64:hn.addComponent("external_component_db",...Pt[nn]),this.$=Pt[nn];break;case 65:hn.addComponent("external_component_queue",...Pt[nn]),this.$=Pt[nn];break;case 67:hn.addRel("rel",...Pt[nn]),this.$=Pt[nn];break;case 68:hn.addRel("birel",...Pt[nn]),this.$=Pt[nn];break;case 69:hn.addRel("rel_u",...Pt[nn]),this.$=Pt[nn];break;case 70:hn.addRel("rel_d",...Pt[nn]),this.$=Pt[nn];break;case 71:hn.addRel("rel_l",...Pt[nn]),this.$=Pt[nn];break;case 72:hn.addRel("rel_r",...Pt[nn]),this.$=Pt[nn];break;case 73:hn.addRel("rel_b",...Pt[nn]),this.$=Pt[nn];break;case 74:Pt[nn].splice(0,1),hn.addRel("rel",...Pt[nn]),this.$=Pt[nn];break;case 75:hn.updateElStyle("update_el_style",...Pt[nn]),this.$=Pt[nn];break;case 76:hn.updateRelStyle("update_rel_style",...Pt[nn]),this.$=Pt[nn];break;case 77:hn.updateLayoutConfig("update_layout_config",...Pt[nn]),this.$=Pt[nn];break;case 78:this.$=[Pt[nn]];break;case 79:Pt[nn].unshift(Pt[nn-1]),this.$=Pt[nn];break;case 80:case 82:this.$=Pt[nn].trim();break;case 81:let hi={};hi[Pt[nn-1].trim()]=Pt[nn].trim(),this.$=hi;break;case 83:this.$="";break}},table:[{3:1,4:2,5:3,6:4,7:i,8:o,9:l,10:f,11:5,12:10,18:b,22:d,25:w,26:y,27:k,28:E},{1:[3]},{1:[2,1]},{1:[2,2]},{3:17,4:2,5:3,6:4,7:i,8:o,9:l,10:f,11:5,12:10,18:b,22:d,25:w,26:y,27:k,28:E},{1:[2,8]},{1:[2,4]},{1:[2,5]},{1:[2,6]},{1:[2,7]},{13:18,19:[1,19]},{15:[1,20]},{15:[1,21]},{15:[1,22]},{15:[1,23]},{15:[1,24]},{19:[2,11]},{1:[2,3]},{14:25,16:[1,26],21:T},r([16,21],[2,12]),{23:28,29:29,30:30,31:31,32:C,33:S,34:L,36:O,38:B,39:58,40:70,42:71,44:N,46:F,47:R,48:q,49:X,50:te,51:H,53:32,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt},{23:79,29:29,30:30,31:31,32:C,33:S,34:L,36:O,38:B,39:58,40:70,42:71,44:N,46:F,47:R,48:q,49:X,50:te,51:H,53:32,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt},{23:80,29:29,30:30,31:31,32:C,33:S,34:L,36:O,38:B,39:58,40:70,42:71,44:N,46:F,47:R,48:q,49:X,50:te,51:H,53:32,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt},{23:81,29:29,30:30,31:31,32:C,33:S,34:L,36:O,38:B,39:58,40:70,42:71,44:N,46:F,47:R,48:q,49:X,50:te,51:H,53:32,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt},{23:82,29:29,30:30,31:31,32:C,33:S,34:L,36:O,38:B,39:58,40:70,42:71,44:N,46:F,47:R,48:q,49:X,50:te,51:H,53:32,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt},{15:[1,83]},{17:84,20:[1,85]},{15:[2,14]},{24:[1,86]},r(nt,[2,20],{53:32,39:58,40:70,42:71,30:87,44:N,46:F,47:R,48:q,49:X,50:te,51:H,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt}),r(nt,[2,21]),r(xn,[2,23],{15:[1,88]}),r(nt,[2,43],{15:[1,89]}),r(cn,[2,26]),r(cn,[2,27]),{35:[1,90]},{37:[1,91]},r(cn,[2,30]),{45:92,85:93,86:jt,87:ot,89:be,90:We},{45:98,85:93,86:jt,87:ot,89:be,90:We},{45:99,85:93,86:jt,87:ot,89:be,90:We},{45:100,85:93,86:jt,87:ot,89:be,90:We},{45:101,85:93,86:jt,87:ot,89:be,90:We},{45:102,85:93,86:jt,87:ot,89:be,90:We},{45:103,85:93,86:jt,87:ot,89:be,90:We},{45:104,85:93,86:jt,87:ot,89:be,90:We},{45:105,85:93,86:jt,87:ot,89:be,90:We},{45:106,85:93,86:jt,87:ot,89:be,90:We},{45:107,85:93,86:jt,87:ot,89:be,90:We},{45:108,85:93,86:jt,87:ot,89:be,90:We},{45:109,85:93,86:jt,87:ot,89:be,90:We},{45:110,85:93,86:jt,87:ot,89:be,90:We},{45:111,85:93,86:jt,87:ot,89:be,90:We},{45:112,85:93,86:jt,87:ot,89:be,90:We},{45:113,85:93,86:jt,87:ot,89:be,90:We},{45:114,85:93,86:jt,87:ot,89:be,90:We},{45:115,85:93,86:jt,87:ot,89:be,90:We},{45:116,85:93,86:jt,87:ot,89:be,90:We},r(ct,[2,66]),{45:117,85:93,86:jt,87:ot,89:be,90:We},{45:118,85:93,86:jt,87:ot,89:be,90:We},{45:119,85:93,86:jt,87:ot,89:be,90:We},{45:120,85:93,86:jt,87:ot,89:be,90:We},{45:121,85:93,86:jt,87:ot,89:be,90:We},{45:122,85:93,86:jt,87:ot,89:be,90:We},{45:123,85:93,86:jt,87:ot,89:be,90:We},{45:124,85:93,86:jt,87:ot,89:be,90:We},{45:125,85:93,86:jt,87:ot,89:be,90:We},{45:126,85:93,86:jt,87:ot,89:be,90:We},{45:127,85:93,86:jt,87:ot,89:be,90:We},{30:128,39:58,40:70,42:71,44:N,46:F,47:R,48:q,49:X,50:te,51:H,53:32,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt},{15:[1,130],43:[1,129]},{45:131,85:93,86:jt,87:ot,89:be,90:We},{45:132,85:93,86:jt,87:ot,89:be,90:We},{45:133,85:93,86:jt,87:ot,89:be,90:We},{45:134,85:93,86:jt,87:ot,89:be,90:We},{45:135,85:93,86:jt,87:ot,89:be,90:We},{45:136,85:93,86:jt,87:ot,89:be,90:We},{45:137,85:93,86:jt,87:ot,89:be,90:We},{24:[1,138]},{24:[1,139]},{24:[1,140]},{24:[1,141]},r(Yt,[2,9]),{14:142,21:T},{21:[2,13]},{1:[2,15]},r(nt,[2,22]),r(xn,[2,24],{31:31,29:143,32:C,33:S,34:L,36:O,38:B}),r(nt,[2,44],{29:29,30:30,31:31,53:32,39:58,40:70,42:71,23:144,32:C,33:S,34:L,36:O,38:B,44:N,46:F,47:R,48:q,49:X,50:te,51:H,54:Y,55:z,56:W,57:Z,58:G,59:ae,60:$,61:ge,62:ee,63:de,64:re,65:ke,66:Ce,67:_e,68:Te,69:Be,70:Ge,71:Xe,72:Ee,73:Ze,74:Ie,75:Oe,76:Le,77:$e,78:xe,79:Ae,80:je,81:me,82:vt,83:ve,84:Zt}),r(cn,[2,28]),r(cn,[2,29]),r(ct,[2,46]),r(Ut,[2,78],{85:93,45:145,86:jt,87:ot,89:be,90:We}),r(Wn,[2,80]),{88:[1,146]},r(Wn,[2,82]),r(Wn,[2,83]),r(ct,[2,47]),r(ct,[2,48]),r(ct,[2,49]),r(ct,[2,50]),r(ct,[2,51]),r(ct,[2,52]),r(ct,[2,53]),r(ct,[2,54]),r(ct,[2,55]),r(ct,[2,56]),r(ct,[2,57]),r(ct,[2,58]),r(ct,[2,59]),r(ct,[2,60]),r(ct,[2,61]),r(ct,[2,62]),r(ct,[2,63]),r(ct,[2,64]),r(ct,[2,65]),r(ct,[2,67]),r(ct,[2,68]),r(ct,[2,69]),r(ct,[2,70]),r(ct,[2,71]),r(ct,[2,72]),r(ct,[2,73]),r(ct,[2,74]),r(ct,[2,75]),r(ct,[2,76]),r(ct,[2,77]),{41:147,52:[1,148]},{15:[1,149]},{43:[1,150]},r(Gt,[2,35]),r(Gt,[2,36]),r(Gt,[2,37]),r(Gt,[2,38]),r(Gt,[2,39]),r(Gt,[2,40]),r(Gt,[2,41]),{1:[2,16]},{1:[2,17]},{1:[2,18]},{1:[2,19]},{15:[1,151]},r(xn,[2,25]),r(nt,[2,45]),r(Ut,[2,79]),r(Wn,[2,81]),r(ct,[2,31]),r(ct,[2,42]),r(Rn,[2,32]),r(Rn,[2,33],{15:[1,152]}),r(Yt,[2,10]),r(Rn,[2,34])],defaultActions:{2:[2,1],3:[2,2],5:[2,8],6:[2,4],7:[2,5],8:[2,6],9:[2,7],16:[2,11],17:[2,3],27:[2,14],85:[2,13],86:[2,15],138:[2,16],139:[2,17],140:[2,18],141:[2,19]},parseError:function(Jt,en){if(en.recoverable)this.trace(Jt);else{var In=new Error(Jt);throw In.hash=en,In}},parse:function(Jt){var en=this,In=[0],hn=[],Fr=[null],Pt=[],ei=this.table,nn="",hi=0,Hi=0,ss=2,ls=1,vs=Pt.slice.call(arguments,1),ti=Object.create(this.lexer),zi={yy:{}};for(var as in this.yy)Object.prototype.hasOwnProperty.call(this.yy,as)&&(zi.yy[as]=this.yy[as]);ti.setInput(Jt,zi.yy),zi.yy.lexer=ti,zi.yy.parser=this,typeof ti.yylloc>"u"&&(ti.yylloc={});var ai=ti.yylloc;Pt.push(ai);var hc=ti.options&&ti.options.ranges;typeof zi.yy.parseError=="function"?this.parseError=zi.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function xu(){var bt;return bt=hn.pop()||ti.lex()||ls,typeof bt!="number"&&(bt instanceof Array&&(hn=bt,bt=hn.pop()),bt=en.symbols_[bt]||bt),bt}for(var No,Si,Yc,lh,su={},Ru,fc,Ll,ol;;){if(Si=In[In.length-1],this.defaultActions[Si]?Yc=this.defaultActions[Si]:((No===null||typeof No>"u")&&(No=xu()),Yc=ei[Si]&&ei[Si][No]),typeof Yc>"u"||!Yc.length||!Yc[0]){var Ur="";ol=[];for(Ru in ei[Si])this.terminals_[Ru]&&Ru>ss&&ol.push("'"+this.terminals_[Ru]+"'");ti.showPosition?Ur="Parse error on line "+(hi+1)+`:
`+ti.showPosition()+`
Expecting `+ol.join(", ")+", got '"+(this.terminals_[No]||No)+"'":Ur="Parse error on line "+(hi+1)+": Unexpected "+(No==ls?"end of input":"'"+(this.terminals_[No]||No)+"'"),this.parseError(Ur,{text:ti.match,token:this.terminals_[No]||No,line:ti.yylineno,loc:ai,expected:ol})}if(Yc[0]instanceof Array&&Yc.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Si+", token: "+No);switch(Yc[0]){case 1:In.push(No),Fr.push(ti.yytext),Pt.push(ti.yylloc),In.push(Yc[1]),No=null,Hi=ti.yyleng,nn=ti.yytext,hi=ti.yylineno,ai=ti.yylloc;break;case 2:if(fc=this.productions_[Yc[1]][1],su.$=Fr[Fr.length-fc],su._$={first_line:Pt[Pt.length-(fc||1)].first_line,last_line:Pt[Pt.length-1].last_line,first_column:Pt[Pt.length-(fc||1)].first_column,last_column:Pt[Pt.length-1].last_column},hc&&(su._$.range=[Pt[Pt.length-(fc||1)].range[0],Pt[Pt.length-1].range[1]]),lh=this.performAction.apply(su,[nn,Hi,hi,zi.yy,Yc[1],Fr,Pt].concat(vs)),typeof lh<"u")return lh;fc&&(In=In.slice(0,-1*fc*2),Fr=Fr.slice(0,-1*fc),Pt=Pt.slice(0,-1*fc)),In.push(this.productions_[Yc[1]][0]),Fr.push(su.$),Pt.push(su._$),Ll=ei[In[In.length-2]][In[In.length-1]],In.push(Ll);break;case 3:return!0}}return!0}},$r=function(){var Kn={EOF:1,parseError:function(en,In){if(this.yy.parser)this.yy.parser.parseError(en,In);else throw new Error(en)},setInput:function(Jt,en){return this.yy=en||this.yy||{},this._input=Jt,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var Jt=this._input[0];this.yytext+=Jt,this.yyleng++,this.offset++,this.match+=Jt,this.matched+=Jt;var en=Jt.match(/(?:\r\n?|\n).*/g);return en?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),Jt},unput:function(Jt){var en=Jt.length,In=Jt.split(/(?:\r\n?|\n)/g);this._input=Jt+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-en),this.offset-=en;var hn=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),In.length-1&&(this.yylineno-=In.length-1);var Fr=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:In?(In.length===hn.length?this.yylloc.first_column:0)+hn[hn.length-In.length].length-In[0].length:this.yylloc.first_column-en},this.options.ranges&&(this.yylloc.range=[Fr[0],Fr[0]+this.yyleng-en]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(Jt){this.unput(this.match.slice(Jt))},pastInput:function(){var Jt=this.matched.substr(0,this.matched.length-this.match.length);return(Jt.length>20?"...":"")+Jt.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var Jt=this.match;return Jt.length<20&&(Jt+=this._input.substr(0,20-Jt.length)),(Jt.substr(0,20)+(Jt.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var Jt=this.pastInput(),en=new Array(Jt.length+1).join("-");return Jt+this.upcomingInput()+`
`+en+"^"},test_match:function(Jt,en){var In,hn,Fr;if(this.options.backtrack_lexer&&(Fr={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(Fr.yylloc.range=this.yylloc.range.slice(0))),hn=Jt[0].match(/(?:\r\n?|\n).*/g),hn&&(this.yylineno+=hn.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:hn?hn[hn.length-1].length-hn[hn.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+Jt[0].length},this.yytext+=Jt[0],this.match+=Jt[0],this.matches=Jt,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(Jt[0].length),this.matched+=Jt[0],In=this.performAction.call(this,this.yy,this,en,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),In)return In;if(this._backtrack){for(var Pt in Fr)this[Pt]=Fr[Pt];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var Jt,en,In,hn;this._more||(this.yytext="",this.match="");for(var Fr=this._currentRules(),Pt=0;Pt<Fr.length;Pt++)if(In=this._input.match(this.rules[Fr[Pt]]),In&&(!en||In[0].length>en[0].length)){if(en=In,hn=Pt,this.options.backtrack_lexer){if(Jt=this.test_match(In,Fr[Pt]),Jt!==!1)return Jt;if(this._backtrack){en=!1;continue}else return!1}else if(!this.options.flex)break}return en?(Jt=this.test_match(en,Fr[hn]),Jt!==!1?Jt:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var en=this.next();return en||this.lex()},begin:function(en){this.conditionStack.push(en)},popState:function(){var en=this.conditionStack.length-1;return en>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(en){return en=this.conditionStack.length-1-Math.abs(en||0),en>=0?this.conditionStack[en]:"INITIAL"},pushState:function(en){this.begin(en)},stateStackSize:function(){return this.conditionStack.length},options:{},performAction:function(en,In,hn,Fr){switch(hn){case 0:return this.begin("open_directive"),18;case 1:return 7;case 2:return 8;case 3:return 9;case 4:return 10;case 5:return this.begin("type_directive"),19;case 6:return this.popState(),this.begin("arg_directive"),16;case 7:return this.popState(),this.popState(),21;case 8:return 20;case 9:return 32;case 10:return 33;case 11:return this.begin("acc_title"),34;case 12:return this.popState(),"acc_title_value";case 13:return this.begin("acc_descr"),36;case 14:return this.popState(),"acc_descr_value";case 15:this.begin("acc_descr_multiline");break;case 16:this.popState();break;case 17:return"acc_descr_multiline_value";case 18:break;case 19:c;break;case 20:return 15;case 21:break;case 22:return 22;case 23:return 25;case 24:return 26;case 25:return 27;case 26:return 28;case 27:return this.begin("person_ext"),55;case 28:return this.begin("person"),54;case 29:return this.begin("system_ext_queue"),61;case 30:return this.begin("system_ext_db"),60;case 31:return this.begin("system_ext"),59;case 32:return this.begin("system_queue"),58;case 33:return this.begin("system_db"),57;case 34:return this.begin("system"),56;case 35:return this.begin("boundary"),47;case 36:return this.begin("enterprise_boundary"),44;case 37:return this.begin("system_boundary"),46;case 38:return this.begin("container_ext_queue"),67;case 39:return this.begin("container_ext_db"),66;case 40:return this.begin("container_ext"),65;case 41:return this.begin("container_queue"),64;case 42:return this.begin("container_db"),63;case 43:return this.begin("container"),62;case 44:return this.begin("container_boundary"),48;case 45:return this.begin("component_ext_queue"),73;case 46:return this.begin("component_ext_db"),72;case 47:return this.begin("component_ext"),71;case 48:return this.begin("component_queue"),70;case 49:return this.begin("component_db"),69;case 50:return this.begin("component"),68;case 51:return this.begin("node"),49;case 52:return this.begin("node"),49;case 53:return this.begin("node_l"),50;case 54:return this.begin("node_r"),51;case 55:return this.begin("rel"),74;case 56:return this.begin("birel"),75;case 57:return this.begin("rel_u"),76;case 58:return this.begin("rel_u"),76;case 59:return this.begin("rel_d"),77;case 60:return this.begin("rel_d"),77;case 61:return this.begin("rel_l"),78;case 62:return this.begin("rel_l"),78;case 63:return this.begin("rel_r"),79;case 64:return this.begin("rel_r"),79;case 65:return this.begin("rel_b"),80;case 66:return this.begin("rel_index"),81;case 67:return this.begin("update_el_style"),82;case 68:return this.begin("update_rel_style"),83;case 69:return this.begin("update_layout_config"),84;case 70:return"EOF_IN_STRUCT";case 71:return this.begin("attribute"),"ATTRIBUTE_EMPTY";case 72:this.begin("attribute");break;case 73:this.popState(),this.popState();break;case 74:return 90;case 75:break;case 76:return 90;case 77:this.begin("string");break;case 78:this.popState();break;case 79:return"STR";case 80:this.begin("string_kv");break;case 81:return this.begin("string_kv_key"),"STR_KEY";case 82:this.popState(),this.begin("string_kv_value");break;case 83:return"STR_VALUE";case 84:this.popState(),this.popState();break;case 85:return"STR";case 86:return"LBRACE";case 87:return"RBRACE";case 88:return"SPACE";case 89:return"EOL";case 90:return 24}},rules:[/^(?:%%\{)/,/^(?:.*direction\s+TB[^\n]*)/,/^(?:.*direction\s+BT[^\n]*)/,/^(?:.*direction\s+RL[^\n]*)/,/^(?:.*direction\s+LR[^\n]*)/,/^(?:((?:(?!\}%%)[^:.])*))/,/^(?::)/,/^(?:\}%%)/,/^(?:((?:(?!\}%%).|\n)*))/,/^(?:title\s[^#\n;]+)/,/^(?:accDescription\s[^#\n;]+)/,/^(?:accTitle\s*:\s*)/,/^(?:(?!\n||)*[^\n]*)/,/^(?:accDescr\s*:\s*)/,/^(?:(?!\n||)*[^\n]*)/,/^(?:accDescr\s*\{\s*)/,/^(?:[\}])/,/^(?:[^\}]*)/,/^(?:%%(?!\{)*[^\n]*(\r?\n?)+)/,/^(?:%%[^\n]*(\r?\n)*)/,/^(?:\s*(\r?\n)+)/,/^(?:\s+)/,/^(?:C4Context\b)/,/^(?:C4Container\b)/,/^(?:C4Component\b)/,/^(?:C4Dynamic\b)/,/^(?:C4Deployment\b)/,/^(?:Person_Ext\b)/,/^(?:Person\b)/,/^(?:SystemQueue_Ext\b)/,/^(?:SystemDb_Ext\b)/,/^(?:System_Ext\b)/,/^(?:SystemQueue\b)/,/^(?:SystemDb\b)/,/^(?:System\b)/,/^(?:Boundary\b)/,/^(?:Enterprise_Boundary\b)/,/^(?:System_Boundary\b)/,/^(?:ContainerQueue_Ext\b)/,/^(?:ContainerDb_Ext\b)/,/^(?:Container_Ext\b)/,/^(?:ContainerQueue\b)/,/^(?:ContainerDb\b)/,/^(?:Container\b)/,/^(?:Container_Boundary\b)/,/^(?:ComponentQueue_Ext\b)/,/^(?:ComponentDb_Ext\b)/,/^(?:Component_Ext\b)/,/^(?:ComponentQueue\b)/,/^(?:ComponentDb\b)/,/^(?:Component\b)/,/^(?:Deployment_Node\b)/,/^(?:Node\b)/,/^(?:Node_L\b)/,/^(?:Node_R\b)/,/^(?:Rel\b)/,/^(?:BiRel\b)/,/^(?:Rel_Up\b)/,/^(?:Rel_U\b)/,/^(?:Rel_Down\b)/,/^(?:Rel_D\b)/,/^(?:Rel_Left\b)/,/^(?:Rel_L\b)/,/^(?:Rel_Right\b)/,/^(?:Rel_R\b)/,/^(?:Rel_Back\b)/,/^(?:RelIndex\b)/,/^(?:UpdateElementStyle\b)/,/^(?:UpdateRelStyle\b)/,/^(?:UpdateLayoutConfig\b)/,/^(?:$)/,/^(?:[(][ ]*[,])/,/^(?:[(])/,/^(?:[)])/,/^(?:,,)/,/^(?:,)/,/^(?:[ ]*["]["])/,/^(?:[ ]*["])/,/^(?:["])/,/^(?:[^"]*)/,/^(?:[ ]*[\$])/,/^(?:[^=]*)/,/^(?:[=][ ]*["])/,/^(?:[^"]+)/,/^(?:["])/,/^(?:[^,]+)/,/^(?:\{)/,/^(?:\})/,/^(?:[\s]+)/,/^(?:[\n\r]+)/,/^(?:$)/],conditions:{acc_descr_multiline:{rules:[16,17],inclusive:!1},acc_descr:{rules:[14],inclusive:!1},acc_title:{rules:[12],inclusive:!1},close_directive:{rules:[],inclusive:!1},arg_directive:{rules:[7,8],inclusive:!1},type_directive:{rules:[6,7],inclusive:!1},open_directive:{rules:[5],inclusive:!1},string_kv_value:{rules:[83,84],inclusive:!1},string_kv_key:{rules:[82],inclusive:!1},string_kv:{rules:[81],inclusive:!1},string:{rules:[78,79],inclusive:!1},attribute:{rules:[73,74,75,76,77,80,85],inclusive:!1},update_layout_config:{rules:[70,71,72,73],inclusive:!1},update_rel_style:{rules:[70,71,72,73],inclusive:!1},update_el_style:{rules:[70,71,72,73],inclusive:!1},rel_b:{rules:[70,71,72,73],inclusive:!1},rel_r:{rules:[70,71,72,73],inclusive:!1},rel_l:{rules:[70,71,72,73],inclusive:!1},rel_d:{rules:[70,71,72,73],inclusive:!1},rel_u:{rules:[70,71,72,73],inclusive:!1},rel_bi:{rules:[],inclusive:!1},rel:{rules:[70,71,72,73],inclusive:!1},node_r:{rules:[70,71,72,73],inclusive:!1},node_l:{rules:[70,71,72,73],inclusive:!1},node:{rules:[70,71,72,73],inclusive:!1},index:{rules:[],inclusive:!1},rel_index:{rules:[70,71,72,73],inclusive:!1},component_ext_queue:{rules:[],inclusive:!1},component_ext_db:{rules:[70,71,72,73],inclusive:!1},component_ext:{rules:[70,71,72,73],inclusive:!1},component_queue:{rules:[70,71,72,73],inclusive:!1},component_db:{rules:[70,71,72,73],inclusive:!1},component:{rules:[70,71,72,73],inclusive:!1},container_boundary:{rules:[70,71,72,73],inclusive:!1},container_ext_queue:{rules:[],inclusive:!1},container_ext_db:{rules:[70,71,72,73],inclusive:!1},container_ext:{rules:[70,71,72,73],inclusive:!1},container_queue:{rules:[70,71,72,73],inclusive:!1},container_db:{rules:[70,71,72,73],inclusive:!1},container:{rules:[70,71,72,73],inclusive:!1},birel:{rules:[70,71,72,73],inclusive:!1},system_boundary:{rules:[70,71,72,73],inclusive:!1},enterprise_boundary:{rules:[70,71,72,73],inclusive:!1},boundary:{rules:[70,71,72,73],inclusive:!1},system_ext_queue:{rules:[70,71,72,73],inclusive:!1},system_ext_db:{rules:[70,71,72,73],inclusive:!1},system_ext:{rules:[70,71,72,73],inclusive:!1},system_queue:{rules:[70,71,72,73],inclusive:!1},system_db:{rules:[70,71,72,73],inclusive:!1},system:{rules:[70,71,72,73],inclusive:!1},person_ext:{rules:[70,71,72,73],inclusive:!1},person:{rules:[70,71,72,73],inclusive:!1},INITIAL:{rules:[0,1,2,3,4,9,10,11,13,15,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,86,87,88,89,90],inclusive:!0}}};return Kn}();si.lexer=$r;function nr(){this.yy={}}return nr.prototype=si,si.Parser=nr,new nr}();tV.parser=tV;const p$t=tV,b$t=r=>r.match(/^\s*C4Context|C4Container|C4Component|C4Dynamic|C4Deployment/)!==null;let Pv=[],d6=[""],Jd="global",Bv="",Fm=[{alias:"global",label:{text:"global"},type:{text:"global"},tags:null,link:null,parentBoundary:""}],bI=[],Oae="",Nae=!1,nV=4,rV=2;var CSe;const v$t=function(){return CSe},w$t=function(r){CSe=up(r,ht())},m$t=function(r,i,o){Bu.parseDirective(this,r,i,o)},y$t=function(r,i,o,l,f,b,d,w,y){if(r==null||i===void 0||i===null||o===void 0||o===null||l===void 0||l===null)return;let k={};const E=bI.find(T=>T.from===i&&T.to===o);if(E?k=E:bI.push(k),k.type=r,k.from=i,k.to=o,k.label={text:l},f==null)k.techn={text:""};else if(typeof f=="object"){let[T,C]=Object.entries(f)[0];k[T]={text:C}}else k.techn={text:f};if(b==null)k.descr={text:""};else if(typeof b=="object"){let[T,C]=Object.entries(b)[0];k[T]={text:C}}else k.descr={text:b};if(typeof d=="object"){let[T,C]=Object.entries(d)[0];k[T]=C}else k.sprite=d;if(typeof w=="object"){let[T,C]=Object.entries(w)[0];k[T]=C}else k.tags=w;if(typeof y=="object"){let[T,C]=Object.entries(y)[0];k[T]=C}else k.link=y;k.wrap=g6()},k$t=function(r,i,o,l,f,b,d){if(i===null||o===null)return;let w={};const y=Pv.find(k=>k.alias===i);if(y&&i===y.alias?w=y:(w.alias=i,Pv.push(w)),o==null?w.label={text:""}:w.label={text:o},l==null)w.descr={text:""};else if(typeof l=="object"){let[k,E]=Object.entries(l)[0];w[k]={text:E}}else w.descr={text:l};if(typeof f=="object"){let[k,E]=Object.entries(f)[0];w[k]=E}else w.sprite=f;if(typeof b=="object"){let[k,E]=Object.entries(b)[0];w[k]=E}else w.tags=b;if(typeof d=="object"){let[k,E]=Object.entries(d)[0];w[k]=E}else w.link=d;w.typeC4Shape={text:r},w.parentBoundary=Jd,w.wrap=g6()},x$t=function(r,i,o,l,f,b,d,w){if(i===null||o===null)return;let y={};const k=Pv.find(E=>E.alias===i);if(k&&i===k.alias?y=k:(y.alias=i,Pv.push(y)),o==null?y.label={text:""}:y.label={text:o},l==null)y.techn={text:""};else if(typeof l=="object"){let[E,T]=Object.entries(l)[0];y[E]={text:T}}else y.techn={text:l};if(f==null)y.descr={text:""};else if(typeof f=="object"){let[E,T]=Object.entries(f)[0];y[E]={text:T}}else y.descr={text:f};if(typeof b=="object"){let[E,T]=Object.entries(b)[0];y[E]=T}else y.sprite=b;if(typeof d=="object"){let[E,T]=Object.entries(d)[0];y[E]=T}else y.tags=d;if(typeof w=="object"){let[E,T]=Object.entries(w)[0];y[E]=T}else y.link=w;y.wrap=g6(),y.typeC4Shape={text:r},y.parentBoundary=Jd},E$t=function(r,i,o,l,f,b,d,w){if(i===null||o===null)return;let y={};const k=Pv.find(E=>E.alias===i);if(k&&i===k.alias?y=k:(y.alias=i,Pv.push(y)),o==null?y.label={text:""}:y.label={text:o},l==null)y.techn={text:""};else if(typeof l=="object"){let[E,T]=Object.entries(l)[0];y[E]={text:T}}else y.techn={text:l};if(f==null)y.descr={text:""};else if(typeof f=="object"){let[E,T]=Object.entries(f)[0];y[E]={text:T}}else y.descr={text:f};if(typeof b=="object"){let[E,T]=Object.entries(b)[0];y[E]=T}else y.sprite=b;if(typeof d=="object"){let[E,T]=Object.entries(d)[0];y[E]=T}else y.tags=d;if(typeof w=="object"){let[E,T]=Object.entries(w)[0];y[E]=T}else y.link=w;y.wrap=g6(),y.typeC4Shape={text:r},y.parentBoundary=Jd},_$t=function(r,i,o,l,f){if(r===null||i===null)return;let b={};const d=Fm.find(w=>w.alias===r);if(d&&r===d.alias?b=d:(b.alias=r,Fm.push(b)),i==null?b.label={text:""}:b.label={text:i},o==null)b.type={text:"system"};else if(typeof o=="object"){let[w,y]=Object.entries(o)[0];b[w]={text:y}}else b.type={text:o};if(typeof l=="object"){let[w,y]=Object.entries(l)[0];b[w]=y}else b.tags=l;if(typeof f=="object"){let[w,y]=Object.entries(f)[0];b[w]=y}else b.link=f;b.parentBoundary=Jd,b.wrap=g6(),Bv=Jd,Jd=r,d6.push(Bv)},T$t=function(r,i,o,l,f){if(r===null||i===null)return;let b={};const d=Fm.find(w=>w.alias===r);if(d&&r===d.alias?b=d:(b.alias=r,Fm.push(b)),i==null?b.label={text:""}:b.label={text:i},o==null)b.type={text:"container"};else if(typeof o=="object"){let[w,y]=Object.entries(o)[0];b[w]={text:y}}else b.type={text:o};if(typeof l=="object"){let[w,y]=Object.entries(l)[0];b[w]=y}else b.tags=l;if(typeof f=="object"){let[w,y]=Object.entries(f)[0];b[w]=y}else b.link=f;b.parentBoundary=Jd,b.wrap=g6(),Bv=Jd,Jd=r,d6.push(Bv)},C$t=function(r,i,o,l,f,b,d,w){if(i===null||o===null)return;let y={};const k=Fm.find(E=>E.alias===i);if(k&&i===k.alias?y=k:(y.alias=i,Fm.push(y)),o==null?y.label={text:""}:y.label={text:o},l==null)y.type={text:"node"};else if(typeof l=="object"){let[E,T]=Object.entries(l)[0];y[E]={text:T}}else y.type={text:l};if(f==null)y.descr={text:""};else if(typeof f=="object"){let[E,T]=Object.entries(f)[0];y[E]={text:T}}else y.descr={text:f};if(typeof d=="object"){let[E,T]=Object.entries(d)[0];y[E]=T}else y.tags=d;if(typeof w=="object"){let[E,T]=Object.entries(w)[0];y[E]=T}else y.link=w;y.nodeType=r,y.parentBoundary=Jd,y.wrap=g6(),Bv=Jd,Jd=i,d6.push(Bv)},S$t=function(){Jd=Bv,d6.pop(),Bv=d6.pop(),d6.push(Bv)},A$t=function(r,i,o,l,f,b,d,w,y,k,E){let T=Pv.find(C=>C.alias===i);if(!(T===void 0&&(T=Fm.find(C=>C.alias===i),T===void 0))){if(o!=null)if(typeof o=="object"){let[C,S]=Object.entries(o)[0];T[C]=S}else T.bgColor=o;if(l!=null)if(typeof l=="object"){let[C,S]=Object.entries(l)[0];T[C]=S}else T.fontColor=l;if(f!=null)if(typeof f=="object"){let[C,S]=Object.entries(f)[0];T[C]=S}else T.borderColor=f;if(b!=null)if(typeof b=="object"){let[C,S]=Object.entries(b)[0];T[C]=S}else T.shadowing=b;if(d!=null)if(typeof d=="object"){let[C,S]=Object.entries(d)[0];T[C]=S}else T.shape=d;if(w!=null)if(typeof w=="object"){let[C,S]=Object.entries(w)[0];T[C]=S}else T.sprite=w;if(y!=null)if(typeof y=="object"){let[C,S]=Object.entries(y)[0];T[C]=S}else T.techn=y;if(k!=null)if(typeof k=="object"){let[C,S]=Object.entries(k)[0];T[C]=S}else T.legendText=k;if(E!=null)if(typeof E=="object"){let[C,S]=Object.entries(E)[0];T[C]=S}else T.legendSprite=E}},M$t=function(r,i,o,l,f,b,d){const w=bI.find(y=>y.from===i&&y.to===o);if(w!==void 0){if(l!=null)if(typeof l=="object"){let[y,k]=Object.entries(l)[0];w[y]=k}else w.textColor=l;if(f!=null)if(typeof f=="object"){let[y,k]=Object.entries(f)[0];w[y]=k}else w.lineColor=f;if(b!=null)if(typeof b=="object"){let[y,k]=Object.entries(b)[0];w[y]=parseInt(k)}else w.offsetX=parseInt(b);if(d!=null)if(typeof d=="object"){let[y,k]=Object.entries(d)[0];w[y]=parseInt(k)}else w.offsetY=parseInt(d)}},D$t=function(r,i,o){let l=nV,f=rV;if(typeof i=="object"){const b=Object.values(i)[0];l=parseInt(b)}else l=parseInt(i);if(typeof o=="object"){const b=Object.values(o)[0];f=parseInt(b)}else f=parseInt(o);l>=1&&(nV=l),f>=1&&(rV=f)},L$t=function(){return nV},I$t=function(){return rV},O$t=function(){return Jd},N$t=function(){return Bv},SSe=function(r){return r==null?Pv:Pv.filter(i=>i.parentBoundary===r)},P$t=function(r){return Pv.find(i=>i.alias===r)},B$t=function(r){return Object.keys(SSe(r))},F$t=function(r){return r==null?Fm:Fm.filter(i=>i.parentBoundary===r)},R$t=function(){return bI},j$t=function(){return Oae},$$t=function(r){Nae=r},g6=function(){return Nae},ASe={addPersonOrSystem:k$t,addPersonOrSystemBoundary:_$t,addContainer:x$t,addContainerBoundary:T$t,addComponent:E$t,addDeploymentNode:C$t,popBoundaryParseStack:S$t,addRel:y$t,updateElStyle:A$t,updateRelStyle:M$t,updateLayoutConfig:D$t,autoWrap:g6,setWrap:$$t,getC4ShapeArray:SSe,getC4Shape:P$t,getC4ShapeKeys:B$t,getBoundarys:F$t,getCurrentBoundaryParse:O$t,getParentBoundaryParse:N$t,getRels:R$t,getTitle:j$t,getC4Type:v$t,getC4ShapeInRow:L$t,getC4BoundaryInRow:I$t,setAccTitle:K0,getAccTitle:hp,getAccDescription:dp,setAccDescription:fp,parseDirective:m$t,getConfig:()=>ht().c4,clear:function(){Pv=[],Fm=[{alias:"global",label:{text:"global"},type:{text:"global"},tags:null,link:null,parentBoundary:""}],Bv="",Jd="global",d6=[""],bI=[],d6=[""],Oae="",Nae=!1,nV=4,rV=2},LINETYPE:{SOLID:0,DOTTED:1,NOTE:2,SOLID_CROSS:3,DOTTED_CROSS:4,SOLID_OPEN:5,DOTTED_OPEN:6,LOOP_START:10,LOOP_END:11,ALT_START:12,ALT_ELSE:13,ALT_END:14,OPT_START:15,OPT_END:16,ACTIVE_START:17,ACTIVE_END:18,PAR_START:19,PAR_AND:20,PAR_END:21,RECT_START:22,RECT_END:23,SOLID_POINT:24,DOTTED_POINT:25},ARROWTYPE:{FILLED:0,OPEN:1},PLACEMENT:{LEFTOF:0,RIGHTOF:1,OVER:2},setTitle:function(r){Oae=up(r,ht())},setC4Type:w$t},Pae=function(r,i){const o=r.append("rect");if(o.attr("x",i.x),o.attr("y",i.y),o.attr("fill",i.fill),o.attr("stroke",i.stroke),o.attr("width",i.width),o.attr("height",i.height),o.attr("rx",i.rx),o.attr("ry",i.ry),i.attrs!=="undefined"&&i.attrs!==null)for(let l in i.attrs)o.attr(l,i.attrs[l]);return i.class!=="undefined"&&o.attr("class",i.class),o},MSe=function(r,i,o,l,f,b){const d=r.append("image");d.attr("width",i),d.attr("height",o),d.attr("x",l),d.attr("y",f);let w=b.startsWith("data:image/png;base64")?b:$3(b);d.attr("xlink:href",w)},H$t=(r,i,o)=>{const l=r.append("g");let f=0;for(let b of i){let d=b.textColor?b.textColor:"#444444",w=b.lineColor?b.lineColor:"#444444",y=b.offsetX?parseInt(b.offsetX):0,k=b.offsetY?parseInt(b.offsetY):0,E="";if(f===0){let C=l.append("line");C.attr("x1",b.startPoint.x),C.attr("y1",b.startPoint.y),C.attr("x2",b.endPoint.x),C.attr("y2",b.endPoint.y),C.attr("stroke-width","1"),C.attr("stroke",w),C.style("fill","none"),b.type!=="rel_b"&&C.attr("marker-end","url("+E+"#arrowhead)"),(b.type==="birel"||b.type==="rel_b")&&C.attr("marker-start","url("+E+"#arrowend)"),f=-1}else{let C=l.append("path");C.attr("fill","none").attr("stroke-width","1").attr("stroke",w).attr("d","Mstartx,starty Qcontrolx,controly stopx,stopy ".replaceAll("startx",b.startPoint.x).replaceAll("starty",b.startPoint.y).replaceAll("controlx",b.startPoint.x+(b.endPoint.x-b.startPoint.x)/2-(b.endPoint.x-b.startPoint.x)/4).replaceAll("controly",b.startPoint.y+(b.endPoint.y-b.startPoint.y)/2).replaceAll("stopx",b.endPoint.x).replaceAll("stopy",b.endPoint.y)),b.type!=="rel_b"&&C.attr("marker-end","url("+E+"#arrowhead)"),(b.type==="birel"||b.type==="rel_b")&&C.attr("marker-start","url("+E+"#arrowend)")}let T=o.messageFont();X3(o)(b.label.text,l,Math.min(b.startPoint.x,b.endPoint.x)+Math.abs(b.endPoint.x-b.startPoint.x)/2+y,Math.min(b.startPoint.y,b.endPoint.y)+Math.abs(b.endPoint.y-b.startPoint.y)/2+k,b.label.width,b.label.height,{fill:d},T),b.techn&&b.techn.text!==""&&(T=o.messageFont(),X3(o)("["+b.techn.text+"]",l,Math.min(b.startPoint.x,b.endPoint.x)+Math.abs(b.endPoint.x-b.startPoint.x)/2+y,Math.min(b.startPoint.y,b.endPoint.y)+Math.abs(b.endPoint.y-b.startPoint.y)/2+o.messageFontSize+5+k,Math.max(b.label.width,b.techn.width),b.techn.height,{fill:d,"font-style":"italic"},T))}},z$t=function(r,i,o){const l=r.append("g");let f=i.bgColor?i.bgColor:"none",b=i.borderColor?i.borderColor:"#444444",d=i.fontColor?i.fontColor:"black",w={"stroke-width":1,"stroke-dasharray":"7.0,7.0"};i.nodeType&&(w={"stroke-width":1});let y={x:i.x,y:i.y,fill:f,stroke:b,width:i.width,height:i.height,rx:2.5,ry:2.5,attrs:w};Pae(l,y);let k=o.boundaryFont();k.fontWeight="bold",k.fontSize=k.fontSize+2,k.fontColor=d,X3(o)(i.label.text,l,i.x,i.y+i.label.Y,i.width,i.height,{fill:"#444444"},k),i.type&&i.type.text!==""&&(k=o.boundaryFont(),k.fontColor=d,X3(o)(i.type.text,l,i.x,i.y+i.type.Y,i.width,i.height,{fill:"#444444"},k)),i.descr&&i.descr.text!==""&&(k=o.boundaryFont(),k.fontSize=k.fontSize-2,k.fontColor=d,X3(o)(i.descr.text,l,i.x,i.y+i.descr.Y,i.width,i.height,{fill:"#444444"},k))},G$t=function(r,i,o){var T;let l=i.bgColor?i.bgColor:o[i.typeC4Shape.text+"_bg_color"],f=i.borderColor?i.borderColor:o[i.typeC4Shape.text+"_border_color"],b=i.fontColor?i.fontColor:"#FFFFFF",d="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAIAAADYYG7QAAACD0lEQVR4Xu2YoU4EMRCGT+4j8Ai8AhaH4QHgAUjQuFMECUgMIUgwJAgMhgQsAYUiJCiQIBBY+EITsjfTdme6V24v4c8vyGbb+ZjOtN0bNcvjQXmkH83WvYBWto6PLm6v7p7uH1/w2fXD+PBycX1Pv2l3IdDm/vn7x+dXQiAubRzoURa7gRZWd0iGRIiJbOnhnfYBQZNJjNbuyY2eJG8fkDE3bbG4ep6MHUAsgYxmE3nVs6VsBWJSGccsOlFPmLIViMzLOB7pCVO2AtHJMohH7Fh6zqitQK7m0rJvAVYgGcEpe//PLdDz65sM4pF9N7ICcXDKIB5Nv6j7tD0NoSdM2QrU9Gg0ewE1LqBhHR3BBdvj2vapnidjHxD/q6vd7Pvhr31AwcY8eXMTXAKECZZJFXuEq27aLgQK5uLMohCenGGuGewOxSjBvYBqeG6B+Nqiblggdjnc+ZXDy+FNFpFzw76O3UBAROuXh6FoiAcf5g9eTvUgzy0nWg6I8cXHRUpg5bOVBCo+KDpFajOf23GgPme7RSQ+lacIENUgJ6gg1k6HjgOlqnLqip4tEuhv0hNEMXUD0clyXE3p6pZA0S2nnvTlXwLJEZWlb7cTQH1+USgTN4VhAenm/wea1OCAOmqo6fE1WCb9WSKBah+rbUWPWAmE2Rvk0ApiB45eOyNAzU8xcTvj8KvkKEoOaIYeHNA3ZuygAvFMUO0AAAAASUVORK5CYII=";switch(i.typeC4Shape.text){case"person":d="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAIAAADYYG7QAAACD0lEQVR4Xu2YoU4EMRCGT+4j8Ai8AhaH4QHgAUjQuFMECUgMIUgwJAgMhgQsAYUiJCiQIBBY+EITsjfTdme6V24v4c8vyGbb+ZjOtN0bNcvjQXmkH83WvYBWto6PLm6v7p7uH1/w2fXD+PBycX1Pv2l3IdDm/vn7x+dXQiAubRzoURa7gRZWd0iGRIiJbOnhnfYBQZNJjNbuyY2eJG8fkDE3bbG4ep6MHUAsgYxmE3nVs6VsBWJSGccsOlFPmLIViMzLOB7pCVO2AtHJMohH7Fh6zqitQK7m0rJvAVYgGcEpe//PLdDz65sM4pF9N7ICcXDKIB5Nv6j7tD0NoSdM2QrU9Gg0ewE1LqBhHR3BBdvj2vapnidjHxD/q6vd7Pvhr31AwcY8eXMTXAKECZZJFXuEq27aLgQK5uLMohCenGGuGewOxSjBvYBqeG6B+Nqiblggdjnc+ZXDy+FNFpFzw76O3UBAROuXh6FoiAcf5g9eTvUgzy0nWg6I8cXHRUpg5bOVBCo+KDpFajOf23GgPme7RSQ+lacIENUgJ6gg1k6HjgOlqnLqip4tEuhv0hNEMXUD0clyXE3p6pZA0S2nnvTlXwLJEZWlb7cTQH1+USgTN4VhAenm/wea1OCAOmqo6fE1WCb9WSKBah+rbUWPWAmE2Rvk0ApiB45eOyNAzU8xcTvj8KvkKEoOaIYeHNA3ZuygAvFMUO0AAAAASUVORK5CYII=";break;case"external_person":d="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAIAAADYYG7QAAAB6ElEQVR4Xu2YLY+EMBCG9+dWr0aj0Wg0Go1Go0+j8Xdv2uTCvv1gpt0ebHKPuhDaeW4605Z9mJvx4AdXUyTUdd08z+u6flmWZRnHsWkafk9DptAwDPu+f0eAYtu2PEaGWuj5fCIZrBAC2eLBAnRCsEkkxmeaJp7iDJ2QMDdHsLg8SxKFEJaAo8lAXnmuOFIhTMpxxKATebo4UiFknuNo4OniSIXQyRxEA3YsnjGCVEjVXD7yLUAqxBGUyPv/Y4W2beMgGuS7kVQIBycH0fD+oi5pezQETxdHKmQKGk1eQEYldK+jw5GxPfZ9z7Mk0Qnhf1W1m3w//EUn5BDmSZsbR44QQLBEqrBHqOrmSKaQAxdnLArCrxZcM7A7ZKs4ioRq8LFC+NpC3WCBJsvpVw5edm9iEXFuyNfxXAgSwfrFQ1c0iNda8AdejvUgnktOtJQQxmcfFzGglc5WVCj7oDgFqU18boeFSs52CUh8LE8BIVQDT1ABrB0HtgSEYlX5doJnCwv9TXocKCaKbnwhdDKPq4lf3SwU3HLq4V/+WYhHVMa/3b4IlfyikAduCkcBc7mQ3/z/Qq/cTuikhkzB12Ae/mcJC9U+Vo8Ej1gWAtgbeGgFsAMHr50BIWOLCbezvhpBFUdY6EJuJ/QDW0XoMX60zZ0AAAAASUVORK5CYII=";break}const w=r.append("g");w.attr("class","person-man");const y=DSe();switch(i.typeC4Shape.text){case"person":case"external_person":case"system":case"external_system":case"container":case"external_container":case"component":case"external_component":y.x=i.x,y.y=i.y,y.fill=l,y.width=i.width,y.height=i.height,y.stroke=f,y.rx=2.5,y.ry=2.5,y.attrs={"stroke-width":.5},Pae(w,y);break;case"system_db":case"external_system_db":case"container_db":case"external_container_db":case"component_db":case"external_component_db":w.append("path").attr("fill",l).attr("stroke-width","0.5").attr("stroke",f).attr("d","Mstartx,startyc0,-10 half,-10 half,-10c0,0 half,0 half,10l0,heightc0,10 -half,10 -half,10c0,0 -half,0 -half,-10l0,-height".replaceAll("startx",i.x).replaceAll("starty",i.y).replaceAll("half",i.width/2).replaceAll("height",i.height)),w.append("path").attr("fill","none").attr("stroke-width","0.5").attr("stroke",f).attr("d","Mstartx,startyc0,10 half,10 half,10c0,0 half,0 half,-10".replaceAll("startx",i.x).replaceAll("starty",i.y).replaceAll("half",i.width/2));break;case"system_queue":case"external_system_queue":case"container_queue":case"external_container_queue":case"component_queue":case"external_component_queue":w.append("path").attr("fill",l).attr("stroke-width","0.5").attr("stroke",f).attr("d","Mstartx,startylwidth,0c5,0 5,half 5,halfc0,0 0,half -5,halfl-width,0c-5,0 -5,-half -5,-halfc0,0 0,-half 5,-half".replaceAll("startx",i.x).replaceAll("starty",i.y).replaceAll("width",i.width).replaceAll("half",i.height/2)),w.append("path").attr("fill","none").attr("stroke-width","0.5").attr("stroke",f).attr("d","Mstartx,startyc-5,0 -5,half -5,halfc0,half 5,half 5,half".replaceAll("startx",i.x+i.width).replaceAll("starty",i.y).replaceAll("half",i.height/2));break}let k=Z$t(o,i.typeC4Shape.text);switch(w.append("text").attr("fill",b).attr("font-family",k.fontFamily).attr("font-size",k.fontSize-2).attr("font-style","italic").attr("lengthAdjust","spacing").attr("textLength",i.typeC4Shape.width).attr("x",i.x+i.width/2-i.typeC4Shape.width/2).attr("y",i.y+i.typeC4Shape.Y).text("<<"+i.typeC4Shape.text+">>"),i.typeC4Shape.text){case"person":case"external_person":MSe(w,48,48,i.x+i.width/2-24,i.y+i.image.Y,d);break}let E=o[i.typeC4Shape.text+"Font"]();return E.fontWeight="bold",E.fontSize=E.fontSize+2,E.fontColor=b,X3(o)(i.label.text,w,i.x,i.y+i.label.Y,i.width,i.height,{fill:b},E),E=o[i.typeC4Shape.text+"Font"](),E.fontColor=b,i.techn&&((T=i.techn)==null?void 0:T.text)!==""?X3(o)(i.techn.text,w,i.x,i.y+i.techn.Y,i.width,i.height,{fill:b,"font-style":"italic"},E):i.type&&i.type.text!==""&&X3(o)(i.type.text,w,i.x,i.y+i.type.Y,i.width,i.height,{fill:b,"font-style":"italic"},E),i.descr&&i.descr.text!==""&&(E=o.personFont(),E.fontColor=b,X3(o)(i.descr.text,w,i.x,i.y+i.descr.Y,i.width,i.height,{fill:b},E)),i.height},V$t=function(r){r.append("defs").append("symbol").attr("id","database").attr("fill-rule","evenodd").attr("clip-rule","evenodd").append("path").attr("transform","scale(.5)").attr("d","M12.258.001l.256.004.255.005.253.008.251.01.249.012.247.015.246.016.242.019.241.02.239.023.236.024.233.027.231.028.229.031.225.032.223.034.22.036.217.038.214.04.211.041.208.043.205.045.201.046.198.048.194.05.191.051.187.053.183.054.18.056.175.057.172.059.168.06.163.061.16.063.155.064.15.066.074.033.073.033.071.034.07.034.069.035.068.035.067.035.066.035.064.036.064.036.062.036.06.036.06.037.058.037.058.037.055.038.055.038.053.038.052.038.051.039.05.039.048.039.047.039.045.04.044.04.043.04.041.04.04.041.039.041.037.041.036.041.034.041.033.042.032.042.03.042.029.042.027.042.026.043.024.043.023.043.021.043.02.043.018.044.017.043.015.044.013.044.012.044.011.045.009.044.007.045.006.045.004.045.002.045.001.045v17l-.001.045-.002.045-.004.045-.006.045-.007.045-.009.044-.011.045-.012.044-.013.044-.015.044-.017.043-.018.044-.02.043-.021.043-.023.043-.024.043-.026.043-.027.042-.029.042-.03.042-.032.042-.033.042-.034.041-.036.041-.037.041-.039.041-.04.041-.041.04-.043.04-.044.04-.045.04-.047.039-.048.039-.05.039-.051.039-.052.038-.053.038-.055.038-.055.038-.058.037-.058.037-.06.037-.06.036-.062.036-.064.036-.064.036-.066.035-.067.035-.068.035-.069.035-.07.034-.071.034-.073.033-.074.033-.15.066-.155.064-.16.063-.163.061-.168.06-.172.059-.175.057-.18.056-.183.054-.187.053-.191.051-.194.05-.198.048-.201.046-.205.045-.208.043-.211.041-.214.04-.217.038-.22.036-.223.034-.225.032-.229.031-.231.028-.233.027-.236.024-.239.023-.241.02-.242.019-.246.016-.247.015-.249.012-.251.01-.253.008-.255.005-.256.004-.258.001-.258-.001-.256-.004-.255-.005-.253-.008-.251-.01-.249-.012-.247-.015-.245-.016-.243-.019-.241-.02-.238-.023-.236-.024-.234-.027-.231-.028-.228-.031-.226-.032-.223-.034-.22-.036-.217-.038-.214-.04-.211-.041-.208-.043-.204-.045-.201-.046-.198-.048-.195-.05-.19-.051-.187-.053-.184-.054-.179-.056-.176-.057-.172-.059-.167-.06-.164-.061-.159-.063-.155-.064-.151-.066-.074-.033-.072-.033-.072-.034-.07-.034-.069-.035-.068-.035-.067-.035-.066-.035-.064-.036-.063-.036-.062-.036-.061-.036-.06-.037-.058-.037-.057-.037-.056-.038-.055-.038-.053-.038-.052-.038-.051-.039-.049-.039-.049-.039-.046-.039-.046-.04-.044-.04-.043-.04-.041-.04-.04-.041-.039-.041-.037-.041-.036-.041-.034-.041-.033-.042-.032-.042-.03-.042-.029-.042-.027-.042-.026-.043-.024-.043-.023-.043-.021-.043-.02-.043-.018-.044-.017-.043-.015-.044-.013-.044-.012-.044-.011-.045-.009-.044-.007-.045-.006-.045-.004-.045-.002-.045-.001-.045v-17l.001-.045.002-.045.004-.045.006-.045.007-.045.009-.044.011-.045.012-.044.013-.044.015-.044.017-.043.018-.044.02-.043.021-.043.023-.043.024-.043.026-.043.027-.042.029-.042.03-.042.032-.042.033-.042.034-.041.036-.041.037-.041.039-.041.04-.041.041-.04.043-.04.044-.04.046-.04.046-.039.049-.039.049-.039.051-.039.052-.038.053-.038.055-.038.056-.038.057-.037.058-.037.06-.037.061-.036.062-.036.063-.036.064-.036.066-.035.067-.035.068-.035.069-.035.07-.034.072-.034.072-.033.074-.033.151-.066.155-.064.159-.063.164-.061.167-.06.172-.059.176-.057.179-.056.184-.054.187-.053.19-.051.195-.05.198-.048.201-.046.204-.045.208-.043.211-.041.214-.04.217-.038.22-.036.223-.034.226-.032.228-.031.231-.028.234-.027.236-.024.238-.023.241-.02.243-.019.245-.016.247-.015.249-.012.251-.01.253-.008.255-.005.256-.004.258-.001.258.001zm-9.258 20.499v.01l.001.021.003.021.004.022.005.021.006.022.007.022.009.023.01.022.011.023.012.023.013.023.015.023.016.024.017.023.018.024.019.024.021.024.022.025.023.024.024.025.052.049.056.05.061.051.066.051.07.051.075.051.079.052.084.052.088.052.092.052.097.052.102.051.105.052.11.052.114.051.119.051.123.051.127.05.131.05.135.05.139.048.144.049.147.047.152.047.155.047.16.045.163.045.167.043.171.043.176.041.178.041.183.039.187.039.19.037.194.035.197.035.202.033.204.031.209.03.212.029.216.027.219.025.222.024.226.021.23.02.233.018.236.016.24.015.243.012.246.01.249.008.253.005.256.004.259.001.26-.001.257-.004.254-.005.25-.008.247-.011.244-.012.241-.014.237-.016.233-.018.231-.021.226-.021.224-.024.22-.026.216-.027.212-.028.21-.031.205-.031.202-.034.198-.034.194-.036.191-.037.187-.039.183-.04.179-.04.175-.042.172-.043.168-.044.163-.045.16-.046.155-.046.152-.047.148-.048.143-.049.139-.049.136-.05.131-.05.126-.05.123-.051.118-.052.114-.051.11-.052.106-.052.101-.052.096-.052.092-.052.088-.053.083-.051.079-.052.074-.052.07-.051.065-.051.06-.051.056-.05.051-.05.023-.024.023-.025.021-.024.02-.024.019-.024.018-.024.017-.024.015-.023.014-.024.013-.023.012-.023.01-.023.01-.022.008-.022.006-.022.006-.022.004-.022.004-.021.001-.021.001-.021v-4.127l-.077.055-.08.053-.083.054-.085.053-.087.052-.09.052-.093.051-.095.05-.097.05-.1.049-.102.049-.105.048-.106.047-.109.047-.111.046-.114.045-.115.045-.118.044-.12.043-.122.042-.124.042-.126.041-.128.04-.13.04-.132.038-.134.038-.135.037-.138.037-.139.035-.142.035-.143.034-.144.033-.147.032-.148.031-.15.03-.151.03-.153.029-.154.027-.156.027-.158.026-.159.025-.161.024-.162.023-.163.022-.165.021-.166.02-.167.019-.169.018-.169.017-.171.016-.173.015-.173.014-.175.013-.175.012-.177.011-.178.01-.179.008-.179.008-.181.006-.182.005-.182.004-.184.003-.184.002h-.37l-.184-.002-.184-.003-.182-.004-.182-.005-.181-.006-.179-.008-.179-.008-.178-.01-.176-.011-.176-.012-.175-.013-.173-.014-.172-.015-.171-.016-.17-.017-.169-.018-.167-.019-.166-.02-.165-.021-.163-.022-.162-.023-.161-.024-.159-.025-.157-.026-.156-.027-.155-.027-.153-.029-.151-.03-.15-.03-.148-.031-.146-.032-.145-.033-.143-.034-.141-.035-.14-.035-.137-.037-.136-.037-.134-.038-.132-.038-.13-.04-.128-.04-.126-.041-.124-.042-.122-.042-.12-.044-.117-.043-.116-.045-.113-.045-.112-.046-.109-.047-.106-.047-.105-.048-.102-.049-.1-.049-.097-.05-.095-.05-.093-.052-.09-.051-.087-.052-.085-.053-.083-.054-.08-.054-.077-.054v4.127zm0-5.654v.011l.001.021.003.021.004.021.005.022.006.022.007.022.009.022.01.022.011.023.012.023.013.023.015.024.016.023.017.024.018.024.019.024.021.024.022.024.023.025.024.024.052.05.056.05.061.05.066.051.07.051.075.052.079.051.084.052.088.052.092.052.097.052.102.052.105.052.11.051.114.051.119.052.123.05.127.051.131.05.135.049.139.049.144.048.147.048.152.047.155.046.16.045.163.045.167.044.171.042.176.042.178.04.183.04.187.038.19.037.194.036.197.034.202.033.204.032.209.03.212.028.216.027.219.025.222.024.226.022.23.02.233.018.236.016.24.014.243.012.246.01.249.008.253.006.256.003.259.001.26-.001.257-.003.254-.006.25-.008.247-.01.244-.012.241-.015.237-.016.233-.018.231-.02.226-.022.224-.024.22-.025.216-.027.212-.029.21-.03.205-.032.202-.033.198-.035.194-.036.191-.037.187-.039.183-.039.179-.041.175-.042.172-.043.168-.044.163-.045.16-.045.155-.047.152-.047.148-.048.143-.048.139-.05.136-.049.131-.05.126-.051.123-.051.118-.051.114-.052.11-.052.106-.052.101-.052.096-.052.092-.052.088-.052.083-.052.079-.052.074-.051.07-.052.065-.051.06-.05.056-.051.051-.049.023-.025.023-.024.021-.025.02-.024.019-.024.018-.024.017-.024.015-.023.014-.023.013-.024.012-.022.01-.023.01-.023.008-.022.006-.022.006-.022.004-.021.004-.022.001-.021.001-.021v-4.139l-.077.054-.08.054-.083.054-.085.052-.087.053-.09.051-.093.051-.095.051-.097.05-.1.049-.102.049-.105.048-.106.047-.109.047-.111.046-.114.045-.115.044-.118.044-.12.044-.122.042-.124.042-.126.041-.128.04-.13.039-.132.039-.134.038-.135.037-.138.036-.139.036-.142.035-.143.033-.144.033-.147.033-.148.031-.15.03-.151.03-.153.028-.154.028-.156.027-.158.026-.159.025-.161.024-.162.023-.163.022-.165.021-.166.02-.167.019-.169.018-.169.017-.171.016-.173.015-.173.014-.175.013-.175.012-.177.011-.178.009-.179.009-.179.007-.181.007-.182.005-.182.004-.184.003-.184.002h-.37l-.184-.002-.184-.003-.182-.004-.182-.005-.181-.007-.179-.007-.179-.009-.178-.009-.176-.011-.176-.012-.175-.013-.173-.014-.172-.015-.171-.016-.17-.017-.169-.018-.167-.019-.166-.02-.165-.021-.163-.022-.162-.023-.161-.024-.159-.025-.157-.026-.156-.027-.155-.028-.153-.028-.151-.03-.15-.03-.148-.031-.146-.033-.145-.033-.143-.033-.141-.035-.14-.036-.137-.036-.136-.037-.134-.038-.132-.039-.13-.039-.128-.04-.126-.041-.124-.042-.122-.043-.12-.043-.117-.044-.116-.044-.113-.046-.112-.046-.109-.046-.106-.047-.105-.048-.102-.049-.1-.049-.097-.05-.095-.051-.093-.051-.09-.051-.087-.053-.085-.052-.083-.054-.08-.054-.077-.054v4.139zm0-5.666v.011l.001.02.003.022.004.021.005.022.006.021.007.022.009.023.01.022.011.023.012.023.013.023.015.023.016.024.017.024.018.023.019.024.021.025.022.024.023.024.024.025.052.05.056.05.061.05.066.051.07.051.075.052.079.051.084.052.088.052.092.052.097.052.102.052.105.051.11.052.114.051.119.051.123.051.127.05.131.05.135.05.139.049.144.048.147.048.152.047.155.046.16.045.163.045.167.043.171.043.176.042.178.04.183.04.187.038.19.037.194.036.197.034.202.033.204.032.209.03.212.028.216.027.219.025.222.024.226.021.23.02.233.018.236.017.24.014.243.012.246.01.249.008.253.006.256.003.259.001.26-.001.257-.003.254-.006.25-.008.247-.01.244-.013.241-.014.237-.016.233-.018.231-.02.226-.022.224-.024.22-.025.216-.027.212-.029.21-.03.205-.032.202-.033.198-.035.194-.036.191-.037.187-.039.183-.039.179-.041.175-.042.172-.043.168-.044.163-.045.16-.045.155-.047.152-.047.148-.048.143-.049.139-.049.136-.049.131-.051.126-.05.123-.051.118-.052.114-.051.11-.052.106-.052.101-.052.096-.052.092-.052.088-.052.083-.052.079-.052.074-.052.07-.051.065-.051.06-.051.056-.05.051-.049.023-.025.023-.025.021-.024.02-.024.019-.024.018-.024.017-.024.015-.023.014-.024.013-.023.012-.023.01-.022.01-.023.008-.022.006-.022.006-.022.004-.022.004-.021.001-.021.001-.021v-4.153l-.077.054-.08.054-.083.053-.085.053-.087.053-.09.051-.093.051-.095.051-.097.05-.1.049-.102.048-.105.048-.106.048-.109.046-.111.046-.114.046-.115.044-.118.044-.12.043-.122.043-.124.042-.126.041-.128.04-.13.039-.132.039-.134.038-.135.037-.138.036-.139.036-.142.034-.143.034-.144.033-.147.032-.148.032-.15.03-.151.03-.153.028-.154.028-.156.027-.158.026-.159.024-.161.024-.162.023-.163.023-.165.021-.166.02-.167.019-.169.018-.169.017-.171.016-.173.015-.173.014-.175.013-.175.012-.177.01-.178.01-.179.009-.179.007-.181.006-.182.006-.182.004-.184.003-.184.001-.185.001-.185-.001-.184-.001-.184-.003-.182-.004-.182-.006-.181-.006-.179-.007-.179-.009-.178-.01-.176-.01-.176-.012-.175-.013-.173-.014-.172-.015-.171-.016-.17-.017-.169-.018-.167-.019-.166-.02-.165-.021-.163-.023-.162-.023-.161-.024-.159-.024-.157-.026-.156-.027-.155-.028-.153-.028-.151-.03-.15-.03-.148-.032-.146-.032-.145-.033-.143-.034-.141-.034-.14-.036-.137-.036-.136-.037-.134-.038-.132-.039-.13-.039-.128-.041-.126-.041-.124-.041-.122-.043-.12-.043-.117-.044-.116-.044-.113-.046-.112-.046-.109-.046-.106-.048-.105-.048-.102-.048-.1-.05-.097-.049-.095-.051-.093-.051-.09-.052-.087-.052-.085-.053-.083-.053-.08-.054-.077-.054v4.153zm8.74-8.179l-.257.004-.254.005-.25.008-.247.011-.244.012-.241.014-.237.016-.233.018-.231.021-.226.022-.224.023-.22.026-.216.027-.212.028-.21.031-.205.032-.202.033-.198.034-.194.036-.191.038-.187.038-.183.04-.179.041-.175.042-.172.043-.168.043-.163.045-.16.046-.155.046-.152.048-.148.048-.143.048-.139.049-.136.05-.131.05-.126.051-.123.051-.118.051-.114.052-.11.052-.106.052-.101.052-.096.052-.092.052-.088.052-.083.052-.079.052-.074.051-.07.052-.065.051-.06.05-.056.05-.051.05-.023.025-.023.024-.021.024-.02.025-.019.024-.018.024-.017.023-.015.024-.014.023-.013.023-.012.023-.01.023-.01.022-.008.022-.006.023-.006.021-.004.022-.004.021-.001.021-.001.021.001.021.001.021.004.021.004.022.006.021.006.023.008.022.01.022.01.023.012.023.013.023.014.023.015.024.017.023.018.024.019.024.02.025.021.024.023.024.023.025.051.05.056.05.06.05.065.051.07.052.074.051.079.052.083.052.088.052.092.052.096.052.101.052.106.052.11.052.114.052.118.051.123.051.126.051.131.05.136.05.139.049.143.048.148.048.152.048.155.046.16.046.163.045.168.043.172.043.175.042.179.041.183.04.187.038.191.038.194.036.198.034.202.033.205.032.21.031.212.028.216.027.22.026.224.023.226.022.231.021.233.018.237.016.241.014.244.012.247.011.25.008.254.005.257.004.26.001.26-.001.257-.004.254-.005.25-.008.247-.011.244-.012.241-.014.237-.016.233-.018.231-.021.226-.022.224-.023.22-.026.216-.027.212-.028.21-.031.205-.032.202-.033.198-.034.194-.036.191-.038.187-.038.183-.04.179-.041.175-.042.172-.043.168-.043.163-.045.16-.046.155-.046.152-.048.148-.048.143-.048.139-.049.136-.05.131-.05.126-.051.123-.051.118-.051.114-.052.11-.052.106-.052.101-.052.096-.052.092-.052.088-.052.083-.052.079-.052.074-.051.07-.052.065-.051.06-.05.056-.05.051-.05.023-.025.023-.024.021-.024.02-.025.019-.024.018-.024.017-.023.015-.024.014-.023.013-.023.012-.023.01-.023.01-.022.008-.022.006-.023.006-.021.004-.022.004-.021.001-.021.001-.021-.001-.021-.001-.021-.004-.021-.004-.022-.006-.021-.006-.023-.008-.022-.01-.022-.01-.023-.012-.023-.013-.023-.014-.023-.015-.024-.017-.023-.018-.024-.019-.024-.02-.025-.021-.024-.023-.024-.023-.025-.051-.05-.056-.05-.06-.05-.065-.051-.07-.052-.074-.051-.079-.052-.083-.052-.088-.052-.092-.052-.096-.052-.101-.052-.106-.052-.11-.052-.114-.052-.118-.051-.123-.051-.126-.051-.131-.05-.136-.05-.139-.049-.143-.048-.148-.048-.152-.048-.155-.046-.16-.046-.163-.045-.168-.043-.172-.043-.175-.042-.179-.041-.183-.04-.187-.038-.191-.038-.194-.036-.198-.034-.202-.033-.205-.032-.21-.031-.212-.028-.216-.027-.22-.026-.224-.023-.226-.022-.231-.021-.233-.018-.237-.016-.241-.014-.244-.012-.247-.011-.25-.008-.254-.005-.257-.004-.26-.001-.26.001z")},U$t=function(r){r.append("defs").append("symbol").attr("id","computer").attr("width","24").attr("height","24").append("path").attr("transform","scale(.5)").attr("d","M2 2v13h20v-13h-20zm18 11h-16v-9h16v9zm-10.228 6l.466-1h3.524l.467 1h-4.457zm14.228 3h-24l2-6h2.104l-1.33 4h18.45l-1.297-4h2.073l2 6zm-5-10h-14v-7h14v7z")},q$t=function(r){r.append("defs").append("symbol").attr("id","clock").attr("width","24").attr("height","24").append("path").attr("transform","scale(.5)").attr("d","M12 2c5.514 0 10 4.486 10 10s-4.486 10-10 10-10-4.486-10-10 4.486-10 10-10zm0-2c-6.627 0-12 5.373-12 12s5.373 12 12 12 12-5.373 12-12-5.373-12-12-12zm5.848 12.459c.202.038.202.333.001.372-1.907.361-6.045 1.111-6.547 1.111-.719 0-1.301-.582-1.301-1.301 0-.512.77-5.447 1.125-7.445.034-.192.312-.181.343.014l.985 6.238 5.394 1.011z")},Y$t=function(r){r.append("defs").append("marker").attr("id","arrowhead").attr("refX",9).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",12).attr("markerHeight",12).attr("orient","auto").append("path").attr("d","M 0 0 L 10 5 L 0 10 z")},W$t=function(r){r.append("defs").append("marker").attr("id","arrowend").attr("refX",1).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",12).attr("markerHeight",12).attr("orient","auto").append("path").attr("d","M 10 0 L 0 5 L 10 10 z")},K$t=function(r){r.append("defs").append("marker").attr("id","filled-head").attr("refX",18).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L14,7 L9,1 Z")},X$t=function(r){r.append("defs").append("marker").attr("id","sequencenumber").attr("refX",15).attr("refY",15).attr("markerWidth",60).attr("markerHeight",40).attr("orient","auto").append("circle").attr("cx",15).attr("cy",15).attr("r",6)},Q$t=function(r){const o=r.append("defs").append("marker").attr("id","crosshead").attr("markerWidth",15).attr("markerHeight",8).attr("orient","auto").attr("refX",16).attr("refY",4);o.append("path").attr("fill","black").attr("stroke","#000000").style("stroke-dasharray","0, 0").attr("stroke-width","1px").attr("d","M 9,2 V 6 L16,4 Z"),o.append("path").attr("fill","none").attr("stroke","#000000").style("stroke-dasharray","0, 0").attr("stroke-width","1px").attr("d","M 0,1 L 6,7 M 6,1 L 0,7")},DSe=function(){return{x:0,y:0,fill:"#EDF2AE",stroke:"#666",width:100,anchor:"start",height:100,rx:0,ry:0}},Z$t=(r,i)=>({fontFamily:r[i+"FontFamily"],fontSize:r[i+"FontSize"],fontWeight:r[i+"FontWeight"]}),X3=function(){function r(f,b,d,w,y,k,E){const T=b.append("text").attr("x",d+y/2).attr("y",w+k/2+5).style("text-anchor","middle").text(f);l(T,E)}function i(f,b,d,w,y,k,E,T){const{fontSize:C,fontFamily:S,fontWeight:L}=T,O=f.split(Cs.lineBreakRegex);for(let B=0;B<O.length;B++){const N=B*C-C*(O.length-1)/2,F=b.append("text").attr("x",d+y/2).attr("y",w).style("text-anchor","middle").attr("dominant-baseline","middle").style("font-size",C).style("font-weight",L).style("font-family",S);F.append("tspan").attr("dy",N).text(O[B]).attr("alignment-baseline","mathematical"),l(F,E)}}function o(f,b,d,w,y,k,E,T){const C=b.append("switch"),L=C.append("foreignObject").attr("x",d).attr("y",w).attr("width",y).attr("height",k).append("xhtml:div").style("display","table").style("height","100%").style("width","100%");L.append("div").style("display","table-cell").style("text-align","center").style("vertical-align","middle").text(f),i(f,C,d,w,y,k,E,T),l(L,E)}function l(f,b){for(const d in b)b.hasOwnProperty(d)&&f.attr(d,b[d])}return function(f){return f.textPlacement==="fo"?o:f.textPlacement==="old"?r:i}}(),Rm={drawRect:Pae,drawBoundary:z$t,drawC4Shape:G$t,drawRels:H$t,drawImage:MSe,insertArrowHead:Y$t,insertArrowEnd:W$t,insertArrowFilledHead:K$t,insertDynamicNumber:X$t,insertArrowCrossHead:Q$t,insertDatabaseIcon:V$t,insertComputerIcon:U$t,insertClockIcon:q$t,getNoteRect:DSe,sanitizeUrl:$3};let iV=0,sV=0,LSe=4,Bae=2;tV.yy=ASe;let Mi={};class ISe{constructor(i){this.name="",this.data={},this.data.startx=void 0,this.data.stopx=void 0,this.data.starty=void 0,this.data.stopy=void 0,this.data.widthLimit=void 0,this.nextData={},this.nextData.startx=void 0,this.nextData.stopx=void 0,this.nextData.starty=void 0,this.nextData.stopy=void 0,this.nextData.cnt=0,Fae(i.db.getConfig())}setData(i,o,l,f){this.nextData.startx=this.data.startx=i,this.nextData.stopx=this.data.stopx=o,this.nextData.starty=this.data.starty=l,this.nextData.stopy=this.data.stopy=f}updateVal(i,o,l,f){i[o]===void 0?i[o]=l:i[o]=f(l,i[o])}insert(i){this.nextData.cnt=this.nextData.cnt+1;let o=this.nextData.startx===this.nextData.stopx?this.nextData.stopx+i.margin:this.nextData.stopx+i.margin*2,l=o+i.width,f=this.nextData.starty+i.margin*2,b=f+i.height;(o>=this.data.widthLimit||l>=this.data.widthLimit||this.nextData.cnt>LSe)&&(o=this.nextData.startx+i.margin+Mi.nextLinePaddingX,f=this.nextData.stopy+i.margin*2,this.nextData.stopx=l=o+i.width,this.nextData.starty=this.nextData.stopy,this.nextData.stopy=b=f+i.height,this.nextData.cnt=1),i.x=o,i.y=f,this.updateVal(this.data,"startx",o,Math.min),this.updateVal(this.data,"starty",f,Math.min),this.updateVal(this.data,"stopx",l,Math.max),this.updateVal(this.data,"stopy",b,Math.max),this.updateVal(this.nextData,"startx",o,Math.min),this.updateVal(this.nextData,"starty",f,Math.min),this.updateVal(this.nextData,"stopx",l,Math.max),this.updateVal(this.nextData,"stopy",b,Math.max)}init(i){this.name="",this.data={startx:void 0,stopx:void 0,starty:void 0,stopy:void 0,widthLimit:void 0},this.nextData={startx:void 0,stopx:void 0,starty:void 0,stopy:void 0,cnt:0},Fae(i.db.getConfig())}bumpLastMargin(i){this.data.stopx+=i,this.data.stopy+=i}}const Fae=function(r){Yf(Mi,r),r.fontFamily&&(Mi.personFontFamily=Mi.systemFontFamily=Mi.messageFontFamily=r.fontFamily),r.fontSize&&(Mi.personFontSize=Mi.systemFontSize=Mi.messageFontSize=r.fontSize),r.fontWeight&&(Mi.personFontWeight=Mi.systemFontWeight=Mi.messageFontWeight=r.fontWeight)},vI=(r,i)=>({fontFamily:r[i+"FontFamily"],fontSize:r[i+"FontSize"],fontWeight:r[i+"FontWeight"]}),aV=r=>({fontFamily:r.boundaryFontFamily,fontSize:r.boundaryFontSize,fontWeight:r.boundaryFontWeight}),J$t=r=>({fontFamily:r.messageFontFamily,fontSize:r.messageFontSize,fontWeight:r.messageFontWeight});function Fv(r,i,o,l,f){if(!i[r].width)if(o)i[r].text=qCe(i[r].text,f,l),i[r].textLines=i[r].text.split(Cs.lineBreakRegex).length,i[r].width=f,i[r].height=gae(i[r].text,l);else{let b=i[r].text.split(Cs.lineBreakRegex);i[r].textLines=b.length;let d=0;i[r].height=0,i[r].width=0;for(const w of b)i[r].width=Math.max(Bm(w,l),i[r].width),d=gae(w,l),i[r].height=i[r].height+d}}const OSe=function(r,i,o){i.x=o.data.startx,i.y=o.data.starty,i.width=o.data.stopx-o.data.startx,i.height=o.data.stopy-o.data.starty,i.label.y=Mi.c4ShapeMargin-35;let l=i.wrap&&Mi.wrap,f=aV(Mi);f.fontSize=f.fontSize+2,f.fontWeight="bold";let b=Bm(i.label.text,f);Fv("label",i,l,f,b),Rm.drawBoundary(r,i,Mi)},NSe=function(r,i,o,l){let f=0;for(const b of l){f=0;const d=o[b];let w=vI(Mi,d.typeC4Shape.text);switch(w.fontSize=w.fontSize-2,d.typeC4Shape.width=Bm("<<"+d.typeC4Shape.text+">>",w),d.typeC4Shape.height=w.fontSize+2,d.typeC4Shape.Y=Mi.c4ShapePadding,f=d.typeC4Shape.Y+d.typeC4Shape.height-4,d.image={width:0,height:0,Y:0},d.typeC4Shape.text){case"person":case"external_person":d.image.width=48,d.image.height=48,d.image.Y=f,f=d.image.Y+d.image.height;break}d.sprite&&(d.image.width=48,d.image.height=48,d.image.Y=f,f=d.image.Y+d.image.height);let y=d.wrap&&Mi.wrap,k=Mi.width-Mi.c4ShapePadding*2,E=vI(Mi,d.typeC4Shape.text);if(E.fontSize=E.fontSize+2,E.fontWeight="bold",Fv("label",d,y,E,k),d.label.Y=f+8,f=d.label.Y+d.label.height,d.type&&d.type.text!==""){d.type.text="["+d.type.text+"]";let S=vI(Mi,d.typeC4Shape.text);Fv("type",d,y,S,k),d.type.Y=f+5,f=d.type.Y+d.type.height}else if(d.techn&&d.techn.text!==""){d.techn.text="["+d.techn.text+"]";let S=vI(Mi,d.techn.text);Fv("techn",d,y,S,k),d.techn.Y=f+5,f=d.techn.Y+d.techn.height}let T=f,C=d.label.width;if(d.descr&&d.descr.text!==""){let S=vI(Mi,d.typeC4Shape.text);Fv("descr",d,y,S,k),d.descr.Y=f+20,f=d.descr.Y+d.descr.height,C=Math.max(d.label.width,d.descr.width),T=f-d.descr.textLines*5}C=C+Mi.c4ShapePadding,d.width=Math.max(d.width||Mi.width,C,Mi.width),d.height=Math.max(d.height||Mi.height,T,Mi.height),d.margin=d.margin||Mi.c4ShapeMargin,r.insert(d),Rm.drawC4Shape(i,d,Mi)}r.bumpLastMargin(Mi.c4ShapeMargin)};class N2{constructor(i,o){this.x=i,this.y=o}}let PSe=function(r,i){let o=r.x,l=r.y,f=i.x,b=i.y,d=o+r.width/2,w=l+r.height/2,y=Math.abs(o-f),k=Math.abs(l-b),E=k/y,T=r.height/r.width,C=null;return l==b&&o<f?C=new N2(o+r.width,w):l==b&&o>f?C=new N2(o,w):o==f&&l<b?C=new N2(d,l+r.height):o==f&&l>b&&(C=new N2(d,l)),o>f&&l<b?T>=E?C=new N2(o,w+E*r.width/2):C=new N2(d-y/k*r.height/2,l+r.height):o<f&&l<b?T>=E?C=new N2(o+r.width,w+E*r.width/2):C=new N2(d+y/k*r.height/2,l+r.height):o<f&&l>b?T>=E?C=new N2(o+r.width,w-E*r.width/2):C=new N2(d+r.height/2*y/k,l):o>f&&l>b&&(T>=E?C=new N2(o,w-r.width/2*E):C=new N2(d-r.height/2*y/k,l)),C},eHt=function(r,i){let o={x:0,y:0};o.x=i.x+i.width/2,o.y=i.y+i.height/2;let l=PSe(r,o);o.x=r.x+r.width/2,o.y=r.y+r.height/2;let f=PSe(i,o);return{startPoint:l,endPoint:f}};const tHt=function(r,i,o,l){let f=0;for(let b of i){f=f+1;let d=b.wrap&&Mi.wrap,w=J$t(Mi);l.db.getC4Type()==="C4Dynamic"&&(b.label.text=f+": "+b.label.text);let k=Bm(b.label.text,w);Fv("label",b,d,w,k),b.techn&&b.techn.text!==""&&(k=Bm(b.techn.text,w),Fv("techn",b,d,w,k)),b.descr&&b.descr.text!==""&&(k=Bm(b.descr.text,w),Fv("descr",b,d,w,k));let E=o(b.from),T=o(b.to),C=eHt(E,T);b.startPoint=C.startPoint,b.endPoint=C.endPoint}Rm.drawRels(r,i,Mi)};function BSe(r,i,o,l,f){let b=new ISe(f);b.data.widthLimit=o.data.widthLimit/Math.min(Bae,l.length);for(let[d,w]of l.entries()){let y=0;w.image={width:0,height:0,Y:0},w.sprite&&(w.image.width=48,w.image.height=48,w.image.Y=y,y=w.image.Y+w.image.height);let k=w.wrap&&Mi.wrap,E=aV(Mi);if(E.fontSize=E.fontSize+2,E.fontWeight="bold",Fv("label",w,k,E,b.data.widthLimit),w.label.Y=y+8,y=w.label.Y+w.label.height,w.type&&w.type.text!==""){w.type.text="["+w.type.text+"]";let L=aV(Mi);Fv("type",w,k,L,b.data.widthLimit),w.type.Y=y+5,y=w.type.Y+w.type.height}if(w.descr&&w.descr.text!==""){let L=aV(Mi);L.fontSize=L.fontSize-2,Fv("descr",w,k,L,b.data.widthLimit),w.descr.Y=y+20,y=w.descr.Y+w.descr.height}if(d==0||d%Bae===0){let L=o.data.startx+Mi.diagramMarginX,O=o.data.stopy+Mi.diagramMarginY+y;b.setData(L,L,O,O)}else{let L=b.data.stopx!==b.data.startx?b.data.stopx+Mi.diagramMarginX:b.data.startx,O=b.data.starty;b.setData(L,L,O,O)}b.name=w.alias;let T=f.db.getC4ShapeArray(w.alias),C=f.db.getC4ShapeKeys(w.alias);C.length>0&&NSe(b,r,T,C),i=w.alias;let S=f.db.getBoundarys(i);S.length>0&&BSe(r,i,b,S,f),w.alias!=="global"&&OSe(r,w,b),o.data.stopy=Math.max(b.data.stopy+Mi.c4ShapeMargin,o.data.stopy),o.data.stopx=Math.max(b.data.stopx+Mi.c4ShapeMargin,o.data.stopx),iV=Math.max(iV,o.data.stopx),sV=Math.max(sV,o.data.stopy)}}const FSe={drawPersonOrSystemArray:NSe,drawBoundary:OSe,setConf:Fae,draw:function(r,i,o,l){Mi=ht().c4;const f=ht().securityLevel;let b;f==="sandbox"&&(b=Cn("#i"+i));const d=Cn(f==="sandbox"?b.nodes()[0].contentDocument.body:"body");let w=l.db;l.db.setWrap(Mi.wrap),LSe=w.getC4ShapeInRow(),Bae=w.getC4BoundaryInRow(),Se.debug(`C:${JSON.stringify(Mi,null,2)}`);const y=f==="sandbox"?d.select(`[id="${i}"]`):Cn(`[id="${i}"]`);Rm.insertComputerIcon(y),Rm.insertDatabaseIcon(y),Rm.insertClockIcon(y);let k=new ISe(l);k.setData(Mi.diagramMarginX,Mi.diagramMarginX,Mi.diagramMarginY,Mi.diagramMarginY),k.data.widthLimit=screen.availWidth,iV=Mi.diagramMarginX,sV=Mi.diagramMarginY;const E=l.db.getTitle();let T=l.db.getBoundarys("");BSe(y,"",k,T,l),Rm.insertArrowHead(y),Rm.insertArrowEnd(y),Rm.insertArrowCrossHead(y),Rm.insertArrowFilledHead(y),tHt(y,l.db.getRels(),l.db.getC4Shape,l),k.data.stopx=iV,k.data.stopy=sV;const C=k.data;let L=C.stopy-C.starty+2*Mi.diagramMarginY;const B=C.stopx-C.startx+2*Mi.diagramMarginX;E&&y.append("text").text(E).attr("x",(C.stopx-C.startx)/2-4*Mi.diagramMarginX).attr("y",C.starty+Mi.diagramMarginY),Iv(y,L,B,Mi.useMaxWidth);const N=E?60:0;y.attr("viewBox",C.startx-Mi.diagramMarginX+" -"+(Mi.diagramMarginY+N)+" "+B+" "+(L+N)),Se.debug("models:",C)}};var Rae=function(){var r=function($e,xe,Ae,je){for(Ae=Ae||{},je=$e.length;je--;Ae[$e[je]]=xe);return Ae},i=[1,3],o=[1,7],l=[1,8],f=[1,9],b=[1,10],d=[1,13],w=[1,12],y=[1,16,25],k=[1,20],E=[1,32],T=[1,33],C=[1,34],S=[1,36],L=[1,39],O=[1,37],B=[1,38],N=[1,44],F=[1,45],R=[1,40],q=[1,41],X=[1,42],te=[1,43],H=[1,48],Y=[1,49],z=[1,50],W=[1,51],Z=[16,25],G=[1,65],ae=[1,66],$=[1,67],ge=[1,68],ee=[1,69],de=[1,70],re=[1,71],ke=[1,80],Ce=[16,25,32,45,46,54,60,61,62,63,64,65,66,71,73],_e=[16,25,30,32,45,46,50,54,60,61,62,63,64,65,66,71,73,88,89,90,91],Te=[5,8,9,10,11,16,19,23,25],Be=[54,88,89,90,91],Ge=[54,65,66,88,89,90,91],Xe=[54,60,61,62,63,64,88,89,90,91],Ee=[16,25,32],Ze=[1,107],Ie={trace:function(){},yy:{},symbols_:{error:2,start:3,mermaidDoc:4,statments:5,direction:6,directive:7,direction_tb:8,direction_bt:9,direction_rl:10,direction_lr:11,graphConfig:12,openDirective:13,typeDirective:14,closeDirective:15,NEWLINE:16,":":17,argDirective:18,open_directive:19,type_directive:20,arg_directive:21,close_directive:22,CLASS_DIAGRAM:23,statements:24,EOF:25,statement:26,className:27,alphaNumToken:28,classLiteralName:29,GENERICTYPE:30,relationStatement:31,LABEL:32,classStatement:33,methodStatement:34,annotationStatement:35,clickStatement:36,cssClassStatement:37,noteStatement:38,acc_title:39,acc_title_value:40,acc_descr:41,acc_descr_value:42,acc_descr_multiline_value:43,CLASS:44,STYLE_SEPARATOR:45,STRUCT_START:46,members:47,STRUCT_STOP:48,ANNOTATION_START:49,ANNOTATION_END:50,MEMBER:51,SEPARATOR:52,relation:53,STR:54,NOTE_FOR:55,noteText:56,NOTE:57,relationType:58,lineType:59,AGGREGATION:60,EXTENSION:61,COMPOSITION:62,DEPENDENCY:63,LOLLIPOP:64,LINE:65,DOTTED_LINE:66,CALLBACK:67,LINK:68,LINK_TARGET:69,CLICK:70,CALLBACK_NAME:71,CALLBACK_ARGS:72,HREF:73,CSSCLASS:74,commentToken:75,textToken:76,graphCodeTokens:77,textNoTagsToken:78,TAGSTART:79,TAGEND:80,"==":81,"--":82,PCT:83,DEFAULT:84,SPACE:85,MINUS:86,keywords:87,UNICODE_TEXT:88,NUM:89,ALPHA:90,BQUOTE_STR:91,$accept:0,$end:1},terminals_:{2:"error",5:"statments",8:"direction_tb",9:"direction_bt",10:"direction_rl",11:"direction_lr",16:"NEWLINE",17:":",19:"open_directive",20:"type_directive",21:"arg_directive",22:"close_directive",23:"CLASS_DIAGRAM",25:"EOF",30:"GENERICTYPE",32:"LABEL",39:"acc_title",40:"acc_title_value",41:"acc_descr",42:"acc_descr_value",43:"acc_descr_multiline_value",44:"CLASS",45:"STYLE_SEPARATOR",46:"STRUCT_START",48:"STRUCT_STOP",49:"ANNOTATION_START",50:"ANNOTATION_END",51:"MEMBER",52:"SEPARATOR",54:"STR",55:"NOTE_FOR",57:"NOTE",60:"AGGREGATION",61:"EXTENSION",62:"COMPOSITION",63:"DEPENDENCY",64:"LOLLIPOP",65:"LINE",66:"DOTTED_LINE",67:"CALLBACK",68:"LINK",69:"LINK_TARGET",70:"CLICK",71:"CALLBACK_NAME",72:"CALLBACK_ARGS",73:"HREF",74:"CSSCLASS",77:"graphCodeTokens",79:"TAGSTART",80:"TAGEND",81:"==",82:"--",83:"PCT",84:"DEFAULT",85:"SPACE",86:"MINUS",87:"keywords",88:"UNICODE_TEXT",89:"NUM",90:"ALPHA",91:"BQUOTE_STR"},productions_:[0,[3,1],[3,1],[3,1],[3,2],[6,1],[6,1],[6,1],[6,1],[4,1],[7,4],[7,6],[13,1],[14,1],[18,1],[15,1],[12,4],[24,1],[24,2],[24,3],[27,1],[27,1],[27,2],[27,2],[27,2],[26,1],[26,2],[26,1],[26,1],[26,1],[26,1],[26,1],[26,1],[26,1],[26,1],[26,2],[26,2],[26,1],[33,2],[33,4],[33,5],[33,7],[35,4],[47,1],[47,2],[34,1],[34,2],[34,1],[34,1],[31,3],[31,4],[31,4],[31,5],[38,3],[38,2],[53,3],[53,2],[53,2],[53,1],[58,1],[58,1],[58,1],[58,1],[58,1],[59,1],[59,1],[36,3],[36,4],[36,3],[36,4],[36,4],[36,5],[36,3],[36,4],[36,4],[36,5],[36,3],[36,4],[36,4],[36,5],[37,3],[75,1],[75,1],[76,1],[76,1],[76,1],[76,1],[76,1],[76,1],[76,1],[78,1],[78,1],[78,1],[78,1],[28,1],[28,1],[28,1],[29,1],[56,1]],performAction:function(xe,Ae,je,me,vt,ve,Zt){var nt=ve.length-1;switch(vt){case 5:me.setDirection("TB");break;case 6:me.setDirection("BT");break;case 7:me.setDirection("RL");break;case 8:me.setDirection("LR");break;case 12:me.parseDirective("%%{","open_directive");break;case 13:me.parseDirective(ve[nt],"type_directive");break;case 14:ve[nt]=ve[nt].trim().replace(/'/g,'"'),me.parseDirective(ve[nt],"arg_directive");break;case 15:me.parseDirective("}%%","close_directive","class");break;case 20:case 21:this.$=ve[nt];break;case 22:this.$=ve[nt-1]+ve[nt];break;case 23:case 24:this.$=ve[nt-1]+"~"+ve[nt];break;case 25:me.addRelation(ve[nt]);break;case 26:ve[nt-1].title=me.cleanupLabel(ve[nt]),me.addRelation(ve[nt-1]);break;case 35:this.$=ve[nt].trim(),me.setAccTitle(this.$);break;case 36:case 37:this.$=ve[nt].trim(),me.setAccDescription(this.$);break;case 38:me.addClass(ve[nt]);break;case 39:me.addClass(ve[nt-2]),me.setCssClass(ve[nt-2],ve[nt]);break;case 40:me.addClass(ve[nt-3]),me.addMembers(ve[nt-3],ve[nt-1]);break;case 41:me.addClass(ve[nt-5]),me.setCssClass(ve[nt-5],ve[nt-3]),me.addMembers(ve[nt-5],ve[nt-1]);break;case 42:me.addAnnotation(ve[nt],ve[nt-2]);break;case 43:this.$=[ve[nt]];break;case 44:ve[nt].push(ve[nt-1]),this.$=ve[nt];break;case 45:break;case 46:me.addMember(ve[nt-1],me.cleanupLabel(ve[nt]));break;case 47:break;case 48:break;case 49:this.$={id1:ve[nt-2],id2:ve[nt],relation:ve[nt-1],relationTitle1:"none",relationTitle2:"none"};break;case 50:this.$={id1:ve[nt-3],id2:ve[nt],relation:ve[nt-1],relationTitle1:ve[nt-2],relationTitle2:"none"};break;case 51:this.$={id1:ve[nt-3],id2:ve[nt],relation:ve[nt-2],relationTitle1:"none",relationTitle2:ve[nt-1]};break;case 52:this.$={id1:ve[nt-4],id2:ve[nt],relation:ve[nt-2],relationTitle1:ve[nt-3],relationTitle2:ve[nt-1]};break;case 53:me.addNote(ve[nt],ve[nt-1]);break;case 54:me.addNote(ve[nt]);break;case 55:this.$={type1:ve[nt-2],type2:ve[nt],lineType:ve[nt-1]};break;case 56:this.$={type1:"none",type2:ve[nt],lineType:ve[nt-1]};break;case 57:this.$={type1:ve[nt-1],type2:"none",lineType:ve[nt]};break;case 58:this.$={type1:"none",type2:"none",lineType:ve[nt]};break;case 59:this.$=me.relationType.AGGREGATION;break;case 60:this.$=me.relationType.EXTENSION;break;case 61:this.$=me.relationType.COMPOSITION;break;case 62:this.$=me.relationType.DEPENDENCY;break;case 63:this.$=me.relationType.LOLLIPOP;break;case 64:this.$=me.lineType.LINE;break;case 65:this.$=me.lineType.DOTTED_LINE;break;case 66:case 72:this.$=ve[nt-2],me.setClickEvent(ve[nt-1],ve[nt]);break;case 67:case 73:this.$=ve[nt-3],me.setClickEvent(ve[nt-2],ve[nt-1]),me.setTooltip(ve[nt-2],ve[nt]);break;case 68:case 76:this.$=ve[nt-2],me.setLink(ve[nt-1],ve[nt]);break;case 69:this.$=ve[nt-3],me.setLink(ve[nt-2],ve[nt-1],ve[nt]);break;case 70:case 78:this.$=ve[nt-3],me.setLink(ve[nt-2],ve[nt-1]),me.setTooltip(ve[nt-2],ve[nt]);break;case 71:case 79:this.$=ve[nt-4],me.setLink(ve[nt-3],ve[nt-2],ve[nt]),me.setTooltip(ve[nt-3],ve[nt-1]);break;case 74:this.$=ve[nt-3],me.setClickEvent(ve[nt-2],ve[nt-1],ve[nt]);break;case 75:this.$=ve[nt-4],me.setClickEvent(ve[nt-3],ve[nt-2],ve[nt-1]),me.setTooltip(ve[nt-3],ve[nt]);break;case 77:this.$=ve[nt-3],me.setLink(ve[nt-2],ve[nt-1],ve[nt]);break;case 80:me.setCssClass(ve[nt-1],ve[nt]);break}},table:[{3:1,4:2,5:i,6:4,7:5,8:o,9:l,10:f,11:b,12:6,13:11,19:d,23:w},{1:[3]},{1:[2,1]},{1:[2,2]},{1:[2,3]},{3:14,4:2,5:i,6:4,7:5,8:o,9:l,10:f,11:b,12:6,13:11,19:d,23:w},{1:[2,9]},r(y,[2,5]),r(y,[2,6]),r(y,[2,7]),r(y,[2,8]),{14:15,20:[1,16]},{16:[1,17]},{20:[2,12]},{1:[2,4]},{15:18,17:[1,19],22:k},r([17,22],[2,13]),{6:31,7:30,8:o,9:l,10:f,11:b,13:11,19:d,24:21,26:22,27:35,28:46,29:47,31:23,33:24,34:25,35:26,36:27,37:28,38:29,39:E,41:T,43:C,44:S,49:L,51:O,52:B,55:N,57:F,67:R,68:q,70:X,74:te,88:H,89:Y,90:z,91:W},{16:[1,52]},{18:53,21:[1,54]},{16:[2,15]},{25:[1,55]},{16:[1,56],25:[2,17]},r(Z,[2,25],{32:[1,57]}),r(Z,[2,27]),r(Z,[2,28]),r(Z,[2,29]),r(Z,[2,30]),r(Z,[2,31]),r(Z,[2,32]),r(Z,[2,33]),r(Z,[2,34]),{40:[1,58]},{42:[1,59]},r(Z,[2,37]),r(Z,[2,45],{53:60,58:63,59:64,32:[1,62],54:[1,61],60:G,61:ae,62:$,63:ge,64:ee,65:de,66:re}),{27:72,28:46,29:47,88:H,89:Y,90:z,91:W},r(Z,[2,47]),r(Z,[2,48]),{28:73,88:H,89:Y,90:z},{27:74,28:46,29:47,88:H,89:Y,90:z,91:W},{27:75,28:46,29:47,88:H,89:Y,90:z,91:W},{27:76,28:46,29:47,88:H,89:Y,90:z,91:W},{54:[1,77]},{27:78,28:46,29:47,88:H,89:Y,90:z,91:W},{54:ke,56:79},r(Ce,[2,20],{28:46,29:47,27:81,30:[1,82],88:H,89:Y,90:z,91:W}),r(Ce,[2,21],{30:[1,83]}),r(_e,[2,94]),r(_e,[2,95]),r(_e,[2,96]),r([16,25,30,32,45,46,54,60,61,62,63,64,65,66,71,73],[2,97]),r(Te,[2,10]),{15:84,22:k},{22:[2,14]},{1:[2,16]},{6:31,7:30,8:o,9:l,10:f,11:b,13:11,19:d,24:85,25:[2,18],26:22,27:35,28:46,29:47,31:23,33:24,34:25,35:26,36:27,37:28,38:29,39:E,41:T,43:C,44:S,49:L,51:O,52:B,55:N,57:F,67:R,68:q,70:X,74:te,88:H,89:Y,90:z,91:W},r(Z,[2,26]),r(Z,[2,35]),r(Z,[2,36]),{27:86,28:46,29:47,54:[1,87],88:H,89:Y,90:z,91:W},{53:88,58:63,59:64,60:G,61:ae,62:$,63:ge,64:ee,65:de,66:re},r(Z,[2,46]),{59:89,65:de,66:re},r(Be,[2,58],{58:90,60:G,61:ae,62:$,63:ge,64:ee}),r(Ge,[2,59]),r(Ge,[2,60]),r(Ge,[2,61]),r(Ge,[2,62]),r(Ge,[2,63]),r(Xe,[2,64]),r(Xe,[2,65]),r(Z,[2,38],{45:[1,91],46:[1,92]}),{50:[1,93]},{54:[1,94]},{54:[1,95]},{71:[1,96],73:[1,97]},{28:98,88:H,89:Y,90:z},{54:ke,56:99},r(Z,[2,54]),r(Z,[2,98]),r(Ce,[2,22]),r(Ce,[2,23]),r(Ce,[2,24]),{16:[1,100]},{25:[2,19]},r(Ee,[2,49]),{27:101,28:46,29:47,88:H,89:Y,90:z,91:W},{27:102,28:46,29:47,54:[1,103],88:H,89:Y,90:z,91:W},r(Be,[2,57],{58:104,60:G,61:ae,62:$,63:ge,64:ee}),r(Be,[2,56]),{28:105,88:H,89:Y,90:z},{47:106,51:Ze},{27:108,28:46,29:47,88:H,89:Y,90:z,91:W},r(Z,[2,66],{54:[1,109]}),r(Z,[2,68],{54:[1,111],69:[1,110]}),r(Z,[2,72],{54:[1,112],72:[1,113]}),r(Z,[2,76],{54:[1,115],69:[1,114]}),r(Z,[2,80]),r(Z,[2,53]),r(Te,[2,11]),r(Ee,[2,51]),r(Ee,[2,50]),{27:116,28:46,29:47,88:H,89:Y,90:z,91:W},r(Be,[2,55]),r(Z,[2,39],{46:[1,117]}),{48:[1,118]},{47:119,48:[2,43],51:Ze},r(Z,[2,42]),r(Z,[2,67]),r(Z,[2,69]),r(Z,[2,70],{69:[1,120]}),r(Z,[2,73]),r(Z,[2,74],{54:[1,121]}),r(Z,[2,77]),r(Z,[2,78],{69:[1,122]}),r(Ee,[2,52]),{47:123,51:Ze},r(Z,[2,40]),{48:[2,44]},r(Z,[2,71]),r(Z,[2,75]),r(Z,[2,79]),{48:[1,124]},r(Z,[2,41])],defaultActions:{2:[2,1],3:[2,2],4:[2,3],6:[2,9],13:[2,12],14:[2,4],20:[2,15],54:[2,14],55:[2,16],85:[2,19],119:[2,44]},parseError:function(xe,Ae){if(Ae.recoverable)this.trace(xe);else{var je=new Error(xe);throw je.hash=Ae,je}},parse:function(xe){var Ae=this,je=[0],me=[],vt=[null],ve=[],Zt=this.table,nt="",xn=0,cn=0,jt=2,ot=1,be=ve.slice.call(arguments,1),We=Object.create(this.lexer),ct={yy:{}};for(var Yt in this.yy)Object.prototype.hasOwnProperty.call(this.yy,Yt)&&(ct.yy[Yt]=this.yy[Yt]);We.setInput(xe,ct.yy),ct.yy.lexer=We,ct.yy.parser=this,typeof We.yylloc>"u"&&(We.yylloc={});var Ut=We.yylloc;ve.push(Ut);var Wn=We.options&&We.options.ranges;typeof ct.yy.parseError=="function"?this.parseError=ct.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Gt(){var Pt;return Pt=me.pop()||We.lex()||ot,typeof Pt!="number"&&(Pt instanceof Array&&(me=Pt,Pt=me.pop()),Pt=Ae.symbols_[Pt]||Pt),Pt}for(var Rn,si,$r,nr,Kn={},Jt,en,In,hn;;){if(si=je[je.length-1],this.defaultActions[si]?$r=this.defaultActions[si]:((Rn===null||typeof Rn>"u")&&(Rn=Gt()),$r=Zt[si]&&Zt[si][Rn]),typeof $r>"u"||!$r.length||!$r[0]){var Fr="";hn=[];for(Jt in Zt[si])this.terminals_[Jt]&&Jt>jt&&hn.push("'"+this.terminals_[Jt]+"'");We.showPosition?Fr="Parse error on line "+(xn+1)+`:
`+We.showPosition()+`
Expecting `+hn.join(", ")+", got '"+(this.terminals_[Rn]||Rn)+"'":Fr="Parse error on line "+(xn+1)+": Unexpected "+(Rn==ot?"end of input":"'"+(this.terminals_[Rn]||Rn)+"'"),this.parseError(Fr,{text:We.match,token:this.terminals_[Rn]||Rn,line:We.yylineno,loc:Ut,expected:hn})}if($r[0]instanceof Array&&$r.length>1)throw new Error("Parse Error: multiple actions possible at state: "+si+", token: "+Rn);switch($r[0]){case 1:je.push(Rn),vt.push(We.yytext),ve.push(We.yylloc),je.push($r[1]),Rn=null,cn=We.yyleng,nt=We.yytext,xn=We.yylineno,Ut=We.yylloc;break;case 2:if(en=this.productions_[$r[1]][1],Kn.$=vt[vt.length-en],Kn._$={first_line:ve[ve.length-(en||1)].first_line,last_line:ve[ve.length-1].last_line,first_column:ve[ve.length-(en||1)].first_column,last_column:ve[ve.length-1].last_column},Wn&&(Kn._$.range=[ve[ve.length-(en||1)].range[0],ve[ve.length-1].range[1]]),nr=this.performAction.apply(Kn,[nt,cn,xn,ct.yy,$r[1],vt,ve].concat(be)),typeof nr<"u")return nr;en&&(je=je.slice(0,-1*en*2),vt=vt.slice(0,-1*en),ve=ve.slice(0,-1*en)),je.push(this.productions_[$r[1]][0]),vt.push(Kn.$),ve.push(Kn._$),In=Zt[je[je.length-2]][je[je.length-1]],je.push(In);break;case 3:return!0}}return!0}},Oe=function(){var $e={EOF:1,parseError:function(Ae,je){if(this.yy.parser)this.yy.parser.parseError(Ae,je);else throw new Error(Ae)},setInput:function(xe,Ae){return this.yy=Ae||this.yy||{},this._input=xe,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var xe=this._input[0];this.yytext+=xe,this.yyleng++,this.offset++,this.match+=xe,this.matched+=xe;var Ae=xe.match(/(?:\r\n?|\n).*/g);return Ae?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),xe},unput:function(xe){var Ae=xe.length,je=xe.split(/(?:\r\n?|\n)/g);this._input=xe+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-Ae),this.offset-=Ae;var me=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),je.length-1&&(this.yylineno-=je.length-1);var vt=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:je?(je.length===me.length?this.yylloc.first_column:0)+me[me.length-je.length].length-je[0].length:this.yylloc.first_column-Ae},this.options.ranges&&(this.yylloc.range=[vt[0],vt[0]+this.yyleng-Ae]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(xe){this.unput(this.match.slice(xe))},pastInput:function(){var xe=this.matched.substr(0,this.matched.length-this.match.length);return(xe.length>20?"...":"")+xe.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var xe=this.match;return xe.length<20&&(xe+=this._input.substr(0,20-xe.length)),(xe.substr(0,20)+(xe.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var xe=this.pastInput(),Ae=new Array(xe.length+1).join("-");return xe+this.upcomingInput()+`
`+Ae+"^"},test_match:function(xe,Ae){var je,me,vt;if(this.options.backtrack_lexer&&(vt={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(vt.yylloc.range=this.yylloc.range.slice(0))),me=xe[0].match(/(?:\r\n?|\n).*/g),me&&(this.yylineno+=me.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:me?me[me.length-1].length-me[me.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+xe[0].length},this.yytext+=xe[0],this.match+=xe[0],this.matches=xe,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(xe[0].length),this.matched+=xe[0],je=this.performAction.call(this,this.yy,this,Ae,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),je)return je;if(this._backtrack){for(var ve in vt)this[ve]=vt[ve];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var xe,Ae,je,me;this._more||(this.yytext="",this.match="");for(var vt=this._currentRules(),ve=0;ve<vt.length;ve++)if(je=this._input.match(this.rules[vt[ve]]),je&&(!Ae||je[0].length>Ae[0].length)){if(Ae=je,me=ve,this.options.backtrack_lexer){if(xe=this.test_match(je,vt[ve]),xe!==!1)return xe;if(this._backtrack){Ae=!1;continue}else return!1}else if(!this.options.flex)break}return Ae?(xe=this.test_match(Ae,vt[me]),xe!==!1?xe:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var Ae=this.next();return Ae||this.lex()},begin:function(Ae){this.conditionStack.push(Ae)},popState:function(){var Ae=this.conditionStack.length-1;return Ae>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(Ae){return Ae=this.conditionStack.length-1-Math.abs(Ae||0),Ae>=0?this.conditionStack[Ae]:"INITIAL"},pushState:function(Ae){this.begin(Ae)},stateStackSize:function(){return this.conditionStack.length},options:{},performAction:function(Ae,je,me,vt){switch(me){case 0:return this.begin("open_directive"),19;case 1:return 8;case 2:return 9;case 3:return 10;case 4:return 11;case 5:return this.begin("type_directive"),20;case 6:return this.popState(),this.begin("arg_directive"),17;case 7:return this.popState(),this.popState(),22;case 8:return 21;case 9:break;case 10:break;case 11:return this.begin("acc_title"),39;case 12:return this.popState(),"acc_title_value";case 13:return this.begin("acc_descr"),41;case 14:return this.popState(),"acc_descr_value";case 15:this.begin("acc_descr_multiline");break;case 16:this.popState();break;case 17:return"acc_descr_multiline_value";case 18:return 16;case 19:break;case 20:return 23;case 21:return 23;case 22:return this.begin("struct"),46;case 23:return"EDGE_STATE";case 24:return"EOF_IN_STRUCT";case 25:return"OPEN_IN_STRUCT";case 26:return this.popState(),48;case 27:break;case 28:return"MEMBER";case 29:return 44;case 30:return 74;case 31:return 67;case 32:return 68;case 33:return 70;case 34:return 55;case 35:return 57;case 36:return 49;case 37:return 50;case 38:this.begin("generic");break;case 39:this.popState();break;case 40:return"GENERICTYPE";case 41:this.begin("string");break;case 42:this.popState();break;case 43:return"STR";case 44:this.begin("bqstring");break;case 45:this.popState();break;case 46:return"BQUOTE_STR";case 47:this.begin("href");break;case 48:this.popState();break;case 49:return 73;case 50:this.begin("callback_name");break;case 51:this.popState();break;case 52:this.popState(),this.begin("callback_args");break;case 53:return 71;case 54:this.popState();break;case 55:return 72;case 56:return 69;case 57:return 69;case 58:return 69;case 59:return 69;case 60:return 61;case 61:return 61;case 62:return 63;case 63:return 63;case 64:return 62;case 65:return 60;case 66:return 64;case 67:return 65;case 68:return 66;case 69:return 32;case 70:return 45;case 71:return 86;case 72:return"DOT";case 73:return"PLUS";case 74:return 83;case 75:return"EQUALS";case 76:return"EQUALS";case 77:return 90;case 78:return"PUNCTUATION";case 79:return 89;case 80:return 88;case 81:return 85;case 82:return 25}},rules:[/^(?:%%\{)/,/^(?:.*direction\s+TB[^\n]*)/,/^(?:.*direction\s+BT[^\n]*)/,/^(?:.*direction\s+RL[^\n]*)/,/^(?:.*direction\s+LR[^\n]*)/,/^(?:((?:(?!\}%%)[^:.])*))/,/^(?::)/,/^(?:\}%%)/,/^(?:((?:(?!\}%%).|\n)*))/,/^(?:%%(?!\{)*[^\n]*(\r?\n?)+)/,/^(?:%%[^\n]*(\r?\n)*)/,/^(?:accTitle\s*:\s*)/,/^(?:(?!\n||)*[^\n]*)/,/^(?:accDescr\s*:\s*)/,/^(?:(?!\n||)*[^\n]*)/,/^(?:accDescr\s*\{\s*)/,/^(?:[\}])/,/^(?:[^\}]*)/,/^(?:\s*(\r?\n)+)/,/^(?:\s+)/,/^(?:classDiagram-v2\b)/,/^(?:classDiagram\b)/,/^(?:[{])/,/^(?:\[\*\])/,/^(?:$)/,/^(?:[{])/,/^(?:[}])/,/^(?:[\n])/,/^(?:[^{}\n]*)/,/^(?:class\b)/,/^(?:cssClass\b)/,/^(?:callback\b)/,/^(?:link\b)/,/^(?:click\b)/,/^(?:note for\b)/,/^(?:note\b)/,/^(?:<<)/,/^(?:>>)/,/^(?:[~])/,/^(?:[~])/,/^(?:[^~]*)/,/^(?:["])/,/^(?:["])/,/^(?:[^"]*)/,/^(?:[`])/,/^(?:[`])/,/^(?:[^`]+)/,/^(?:href[\s]+["])/,/^(?:["])/,/^(?:[^"]*)/,/^(?:call[\s]+)/,/^(?:\([\s]*\))/,/^(?:\()/,/^(?:[^(]*)/,/^(?:\))/,/^(?:[^)]*)/,/^(?:_self\b)/,/^(?:_blank\b)/,/^(?:_parent\b)/,/^(?:_top\b)/,/^(?:\s*<\|)/,/^(?:\s*\|>)/,/^(?:\s*>)/,/^(?:\s*<)/,/^(?:\s*\*)/,/^(?:\s*o\b)/,/^(?:\s*\(\))/,/^(?:--)/,/^(?:\.\.)/,/^(?::{1}[^:\n;]+)/,/^(?::{3})/,/^(?:-)/,/^(?:\.)/,/^(?:\+)/,/^(?:%)/,/^(?:=)/,/^(?:=)/,/^(?:\w+)/,/^(?:[!"#$%&'*+,-.`?\\/])/,/^(?:[0-9]+)/,/^(?:[\u00AA\u00B5\u00BA\u00C0-\u00D6\u00D8-\u00F6]|[\u00F8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377]|[\u037A-\u037D\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5]|[\u03F7-\u0481\u048A-\u0527\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA]|[\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE]|[\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA]|[\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0]|[\u08A2-\u08AC\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0977]|[\u0979-\u097F\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2]|[\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A]|[\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39]|[\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8]|[\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0B05-\u0B0C]|[\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C]|[\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99]|[\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0]|[\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C33\u0C35-\u0C39\u0C3D]|[\u0C58\u0C59\u0C60\u0C61\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3]|[\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10]|[\u0D12-\u0D3A\u0D3D\u0D4E\u0D60\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1]|[\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81]|[\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3]|[\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6]|[\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A]|[\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081]|[\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D]|[\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0]|[\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310]|[\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F4\u1401-\u166C]|[\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u1700-\u170C\u170E-\u1711]|[\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7]|[\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191C]|[\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19C1-\u19C7\u1A00-\u1A16]|[\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF]|[\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1CE9-\u1CEC]|[\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D]|[\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D]|[\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3]|[\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F]|[\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128]|[\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2183\u2184]|[\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3]|[\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6]|[\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE]|[\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005\u3006\u3031-\u3035\u303B\u303C]|[\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D]|[\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FCC]|[\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B]|[\uA640-\uA66E\uA67F-\uA697\uA6A0-\uA6E5\uA717-\uA71F\uA722-\uA788]|[\uA78B-\uA78E\uA790-\uA793\uA7A0-\uA7AA\uA7F8-\uA801\uA803-\uA805]|[\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB]|[\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uAA00-\uAA28]|[\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA80-\uAAAF\uAAB1\uAAB5]|[\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4]|[\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E]|[\uABC0-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D]|[\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36]|[\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D]|[\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC]|[\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF]|[\uFFD2-\uFFD7\uFFDA-\uFFDC])/,/^(?:\s)/,/^(?:$)/],conditions:{acc_descr_multiline:{rules:[16,17],inclusive:!1},acc_descr:{rules:[14],inclusive:!1},acc_title:{rules:[12],inclusive:!1},arg_directive:{rules:[7,8],inclusive:!1},type_directive:{rules:[6,7],inclusive:!1},open_directive:{rules:[5],inclusive:!1},callback_args:{rules:[54,55],inclusive:!1},callback_name:{rules:[51,52,53],inclusive:!1},href:{rules:[48,49],inclusive:!1},struct:{rules:[23,24,25,26,27,28],inclusive:!1},generic:{rules:[39,40],inclusive:!1},bqstring:{rules:[45,46],inclusive:!1},string:{rules:[42,43],inclusive:!1},INITIAL:{rules:[0,1,2,3,4,9,10,11,13,15,18,19,20,21,22,23,29,30,31,32,33,34,35,36,37,38,41,44,47,50,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82],inclusive:!0}}};return $e}();Ie.lexer=Oe;function Le(){this.yy={}}return Le.prototype=Ie,Ie.Parser=Le,new Le}();Rae.parser=Rae;const RSe=Rae,nHt=(r,i)=>{var o;return((o=i==null?void 0:i.class)==null?void 0:o.defaultRenderer)==="dagre-wrapper"?!1:r.match(/^\s*classDiagram/)!==null},rHt=(r,i)=>{var o;return r.match(/^\s*classDiagram/)!==null&&((o=i==null?void 0:i.class)==null?void 0:o.defaultRenderer)==="dagre-wrapper"?!0:r.match(/^\s*classDiagram-v2/)!==null},jae="classid-";let $ae=[],ah={},oV=[],jSe=0,wI=[];const mI=r=>Cs.sanitizeText(r,ht()),iHt=function(r,i,o){Bu.parseDirective(this,r,i,o)},yI=function(r){let i="",o=r;if(r.indexOf("~")>0){let l=r.split("~");o=l[0],i=Cs.sanitizeText(l[1],ht())}return{className:o,type:i}},Hae=function(r){let i=yI(r);ah[i.className]===void 0&&(ah[i.className]={id:i.className,type:i.type,cssClasses:[],methods:[],members:[],annotations:[],domId:jae+i.className+"-"+jSe},jSe++)},$Se=function(r){const i=Object.keys(ah);for(const o of i)if(ah[o].id===r)return ah[o].domId},sHt=function(){$ae=[],ah={},oV=[],wI=[],wI.push(zSe),W0()},aHt=function(r){return ah[r]},oHt=function(){return ah},cHt=function(){return $ae},uHt=function(){return oV},lHt=function(r){Se.debug("Adding relation: "+JSON.stringify(r)),Hae(r.id1),Hae(r.id2),r.id1=yI(r.id1).className,r.id2=yI(r.id2).className,r.relationTitle1=Cs.sanitizeText(r.relationTitle1.trim(),ht()),r.relationTitle2=Cs.sanitizeText(r.relationTitle2.trim(),ht()),$ae.push(r)},hHt=function(r,i){const o=yI(r).className;ah[o].annotations.push(i)},HSe=function(r,i){const o=yI(r).className,l=ah[o];if(typeof i=="string"){const f=i.trim();f.startsWith("<<")&&f.endsWith(">>")?l.annotations.push(mI(f.substring(2,f.length-2))):f.indexOf(")")>0?l.methods.push(mI(f)):f&&l.members.push(mI(f))}},fHt=function(r,i){Array.isArray(i)&&(i.reverse(),i.forEach(o=>HSe(r,o)))},dHt=function(r,i){const o={id:`note${oV.length}`,class:i,text:r};oV.push(o)},gHt=function(r){return r.substring(0,1)===":"?Cs.sanitizeText(r.substr(1).trim(),ht()):mI(r.trim())},zae=function(r,i){r.split(",").forEach(function(o){let l=o;o[0].match(/\d/)&&(l=jae+l),ah[l]!==void 0&&ah[l].cssClasses.push(i)})},pHt=function(r,i){const o=ht();r.split(",").forEach(function(l){i!==void 0&&(ah[l].tooltip=Cs.sanitizeText(i,o))})},bHt=function(r){return ah[r].tooltip},vHt=function(r,i,o){const l=ht();r.split(",").forEach(function(f){let b=f;f[0].match(/\d/)&&(b=jae+b),ah[b]!==void 0&&(ah[b].link=Fs.formatUrl(i,l),l.securityLevel==="sandbox"?ah[b].linkTarget="_top":typeof o=="string"?ah[b].linkTarget=mI(o):ah[b].linkTarget="_blank")}),zae(r,"clickable")},wHt=function(r,i,o){r.split(",").forEach(function(l){mHt(l,i,o),ah[l].haveCallback=!0}),zae(r,"clickable")},mHt=function(r,i,o){const l=ht();let f=r,b=$Se(f);if(l.securityLevel==="loose"&&i!==void 0&&ah[f]!==void 0){let d=[];if(typeof o=="string"){d=o.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);for(let w=0;w<d.length;w++){let y=d[w].trim();y.charAt(0)==='"'&&y.charAt(y.length-1)==='"'&&(y=y.substr(1,y.length-2)),d[w]=y}}d.length===0&&d.push(b),wI.push(function(){const w=document.querySelector(`[id="${b}"]`);w!==null&&w.addEventListener("click",function(){Fs.runFunc(i,...d)},!1)})}},yHt=function(r){wI.forEach(function(i){i(r)})},kHt={LINE:0,DOTTED_LINE:1},xHt={AGGREGATION:0,EXTENSION:1,COMPOSITION:2,DEPENDENCY:3,LOLLIPOP:4},zSe=function(r){let i=Cn(".mermaidTooltip");(i._groups||i)[0][0]===null&&(i=Cn("body").append("div").attr("class","mermaidTooltip").style("opacity",0)),Cn(r).select("svg").selectAll("g.node").on("mouseover",function(){const f=Cn(this);if(f.attr("title")===null)return;const d=this.getBoundingClientRect();i.transition().duration(200).style("opacity",".9"),i.text(f.attr("title")).style("left",window.scrollX+d.left+(d.right-d.left)/2+"px").style("top",window.scrollY+d.top-14+document.body.scrollTop+"px"),i.html(i.html().replace(/&lt;br\/&gt;/g,"<br/>")),f.classed("hover",!0)}).on("mouseout",function(){i.transition().duration(500).style("opacity",0),Cn(this).classed("hover",!1)})};wI.push(zSe);let GSe="TB";const iC={parseDirective:iHt,setAccTitle:K0,getAccTitle:hp,getAccDescription:dp,setAccDescription:fp,getConfig:()=>ht().class,addClass:Hae,bindFunctions:yHt,clear:sHt,getClass:aHt,getClasses:oHt,getNotes:uHt,addAnnotation:hHt,addNote:dHt,getRelations:cHt,addRelation:lHt,getDirection:()=>GSe,setDirection:r=>{GSe=r},addMember:HSe,addMembers:fHt,cleanupLabel:gHt,lineType:kHt,relationType:xHt,setClickEvent:wHt,setCssClass:zae,setLink:vHt,getTooltip:bHt,setTooltip:pHt,lookUpDomId:$Se,setDiagramTitle:Ov,getDiagramTitle:Nv};function jm(r){return r!=null&&typeof r=="object"}var EHt="[object Symbol]";function r7(r){return typeof r=="symbol"||jm(r)&&Y8(r)==EHt}function sC(r,i){for(var o=-1,l=r==null?0:r.length,f=Array(l);++o<l;)f[o]=i(r[o],o,r);return f}var _Ht=Array.isArray;const $h=_Ht;var THt=1/0,VSe=I2?I2.prototype:void 0,USe=VSe?VSe.toString:void 0;function qSe(r){if(typeof r=="string")return r;if($h(r))return sC(r,qSe)+"";if(r7(r))return USe?USe.call(r):"";var i=r+"";return i=="0"&&1/r==-THt?"-0":i}var CHt=/\s/;function SHt(r){for(var i=r.length;i--&&CHt.test(r.charAt(i)););return i}var AHt=/^\s+/;function MHt(r){return r&&r.slice(0,SHt(r)+1).replace(AHt,"")}var YSe=0/0,DHt=/^[-+]0x[0-9a-f]+$/i,LHt=/^0b[01]+$/i,IHt=/^0o[0-7]+$/i,OHt=parseInt;function NHt(r){if(typeof r=="number")return r;if(r7(r))return YSe;if(lp(r)){var i=typeof r.valueOf=="function"?r.valueOf():r;r=lp(i)?i+"":i}if(typeof r!="string")return r===0?r:+r;r=MHt(r);var o=LHt.test(r);return o||IHt.test(r)?OHt(r.slice(2),o?2:8):DHt.test(r)?YSe:+r}var WSe=1/0,PHt=17976931348623157e292;function cV(r){if(!r)return r===0?r:0;if(r=NHt(r),r===WSe||r===-WSe){var i=r<0?-1:1;return i*PHt}return r===r?r:0}function BHt(r){var i=cV(r),o=i%1;return i===i?o?i-o:i:0}function i7(r){return r}var FHt=K8(Mv,"WeakMap");const Gae=FHt;var KSe=Object.create,RHt=function(){function r(){}return function(i){if(!lp(i))return{};if(KSe)return KSe(i);r.prototype=i;var o=new r;return r.prototype=void 0,o}}();const jHt=RHt;function $Ht(r,i,o){switch(o.length){case 0:return r.call(i);case 1:return r.call(i,o[0]);case 2:return r.call(i,o[0],o[1]);case 3:return r.call(i,o[0],o[1],o[2])}return r.apply(i,o)}function HHt(){}function XSe(r,i){var o=-1,l=r.length;for(i||(i=Array(l));++o<l;)i[o]=r[o];return i}var zHt=800,GHt=16,VHt=Date.now;function UHt(r){var i=0,o=0;return function(){var l=VHt(),f=GHt-(l-o);if(o=l,f>0){if(++i>=zHt)return arguments[0]}else i=0;return r.apply(void 0,arguments)}}function aC(r){return function(){return r}}var qHt=function(){try{var r=K8(Object,"defineProperty");return r({},"",{}),r}catch{}}();const uV=qHt;var YHt=uV?function(r,i){return uV(r,"toString",{configurable:!0,enumerable:!1,value:aC(i),writable:!0})}:i7,WHt=UHt(YHt);const QSe=WHt;function ZSe(r,i){for(var o=-1,l=r==null?0:r.length;++o<l&&i(r[o],o,r)!==!1;);return r}function JSe(r,i,o,l){for(var f=r.length,b=o+(l?1:-1);l?b--:++b<f;)if(i(r[b],b,r))return b;return-1}function KHt(r){return r!==r}function XHt(r,i,o){for(var l=o-1,f=r.length;++l<f;)if(r[l]===i)return l;return-1}function QHt(r,i,o){return i===i?XHt(r,i,o):JSe(r,KHt,o)}function ZHt(r,i){var o=r==null?0:r.length;return!!o&&QHt(r,i,0)>-1}var JHt=9007199254740991,ezt=/^(?:0|[1-9]\d*)$/;function lV(r,i){var o=typeof r;return i=i==null?JHt:i,!!i&&(o=="number"||o!="symbol"&&ezt.test(r))&&r>-1&&r%1==0&&r<i}function hV(r,i,o){i=="__proto__"&&uV?uV(r,i,{configurable:!0,enumerable:!0,value:o,writable:!0}):r[i]=o}var tzt=Object.prototype,nzt=tzt.hasOwnProperty;function fV(r,i,o){var l=r[i];(!(nzt.call(r,i)&&ZT(l,o))||o===void 0&&!(i in r))&&hV(r,i,o)}function kI(r,i,o,l){var f=!o;o||(o={});for(var b=-1,d=i.length;++b<d;){var w=i[b],y=l?l(o[w],r[w],w,o,r):void 0;y===void 0&&(y=r[w]),f?hV(o,w,y):fV(o,w,y)}return o}var eAe=Math.max;function tAe(r,i,o){return i=eAe(i===void 0?r.length-1:i,0),function(){for(var l=arguments,f=-1,b=eAe(l.length-i,0),d=Array(b);++f<b;)d[f]=l[i+f];f=-1;for(var w=Array(i+1);++f<i;)w[f]=l[f];return w[i]=o(d),$Ht(r,this,w)}}function dV(r,i){return QSe(tAe(r,i,i7),r+"")}var rzt=9007199254740991;function Vae(r){return typeof r=="number"&&r>-1&&r%1==0&&r<=rzt}function p6(r){return r!=null&&Vae(r.length)&&!QT(r)}function xI(r,i,o){if(!lp(o))return!1;var l=typeof i;return(l=="number"?p6(o)&&lV(i,o.length):l=="string"&&i in o)?ZT(o[i],r):!1}function izt(r){return dV(function(i,o){var l=-1,f=o.length,b=f>1?o[f-1]:void 0,d=f>2?o[2]:void 0;for(b=r.length>3&&typeof b=="function"?(f--,b):void 0,d&&xI(o[0],o[1],d)&&(b=f<3?void 0:b,f=1),i=Object(i);++l<f;){var w=o[l];w&&r(i,w,l,b)}return i})}var szt=Object.prototype;function gV(r){var i=r&&r.constructor,o=typeof i=="function"&&i.prototype||szt;return r===o}function azt(r,i){for(var o=-1,l=Array(r);++o<r;)l[o]=i(o);return l}var ozt="[object Arguments]";function nAe(r){return jm(r)&&Y8(r)==ozt}var rAe=Object.prototype,czt=rAe.hasOwnProperty,uzt=rAe.propertyIsEnumerable,lzt=nAe(function(){return arguments}())?nAe:function(r){return jm(r)&&czt.call(r,"callee")&&!uzt.call(r,"callee")};const oC=lzt;function hzt(){return!1}var iAe=typeof exports=="object"&&exports&&!exports.nodeType&&exports,sAe=iAe&&typeof module=="object"&&module&&!module.nodeType&&module,fzt=sAe&&sAe.exports===iAe,aAe=fzt?Mv.Buffer:void 0,dzt=aAe?aAe.isBuffer:void 0,gzt=dzt||hzt;const cC=gzt;var pzt="[object Arguments]",bzt="[object Array]",vzt="[object Boolean]",wzt="[object Date]",mzt="[object Error]",yzt="[object Function]",kzt="[object Map]",xzt="[object Number]",Ezt="[object Object]",_zt="[object RegExp]",Tzt="[object Set]",Czt="[object String]",Szt="[object WeakMap]",Azt="[object ArrayBuffer]",Mzt="[object DataView]",Dzt="[object Float32Array]",Lzt="[object Float64Array]",Izt="[object Int8Array]",Ozt="[object Int16Array]",Nzt="[object Int32Array]",Pzt="[object Uint8Array]",Bzt="[object Uint8ClampedArray]",Fzt="[object Uint16Array]",Rzt="[object Uint32Array]",yu={};yu[Dzt]=yu[Lzt]=yu[Izt]=yu[Ozt]=yu[Nzt]=yu[Pzt]=yu[Bzt]=yu[Fzt]=yu[Rzt]=!0,yu[pzt]=yu[bzt]=yu[Azt]=yu[vzt]=yu[Mzt]=yu[wzt]=yu[mzt]=yu[yzt]=yu[kzt]=yu[xzt]=yu[Ezt]=yu[_zt]=yu[Tzt]=yu[Czt]=yu[Szt]=!1;function jzt(r){return jm(r)&&Vae(r.length)&&!!yu[Y8(r)]}function pV(r){return function(i){return r(i)}}var oAe=typeof exports=="object"&&exports&&!exports.nodeType&&exports,EI=oAe&&typeof module=="object"&&module&&!module.nodeType&&module,$zt=EI&&EI.exports===oAe,Uae=$zt&&RCe.process,Hzt=function(){try{var r=EI&&EI.require&&EI.require("util").types;return r||Uae&&Uae.binding&&Uae.binding("util")}catch{}}();const uC=Hzt;var cAe=uC&&uC.isTypedArray,zzt=cAe?pV(cAe):jzt;const bV=zzt;var Gzt=Object.prototype,Vzt=Gzt.hasOwnProperty;function uAe(r,i){var o=$h(r),l=!o&&oC(r),f=!o&&!l&&cC(r),b=!o&&!l&&!f&&bV(r),d=o||l||f||b,w=d?azt(r.length,String):[],y=w.length;for(var k in r)(i||Vzt.call(r,k))&&!(d&&(k=="length"||f&&(k=="offset"||k=="parent")||b&&(k=="buffer"||k=="byteLength"||k=="byteOffset")||lV(k,y)))&&w.push(k);return w}function lAe(r,i){return function(o){return r(i(o))}}var Uzt=lAe(Object.keys,Object);const qzt=Uzt;var Yzt=Object.prototype,Wzt=Yzt.hasOwnProperty;function hAe(r){if(!gV(r))return qzt(r);var i=[];for(var o in Object(r))Wzt.call(r,o)&&o!="constructor"&&i.push(o);return i}function pp(r){return p6(r)?uAe(r):hAe(r)}function Kzt(r){var i=[];if(r!=null)for(var o in Object(r))i.push(o);return i}var Xzt=Object.prototype,Qzt=Xzt.hasOwnProperty;function Zzt(r){if(!lp(r))return Kzt(r);var i=gV(r),o=[];for(var l in r)l=="constructor"&&(i||!Qzt.call(r,l))||o.push(l);return o}function s7(r){return p6(r)?uAe(r,!0):Zzt(r)}var Jzt=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,eGt=/^\w*$/;function qae(r,i){if($h(r))return!1;var o=typeof r;return o=="number"||o=="symbol"||o=="boolean"||r==null||r7(r)?!0:eGt.test(r)||!Jzt.test(r)||i!=null&&r in Object(i)}var tGt=500;function nGt(r){var i=JT(r,function(l){return o.size===tGt&&o.clear(),l}),o=i.cache;return i}var rGt=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,iGt=/\\(\\)?/g,sGt=nGt(function(r){var i=[];return r.charCodeAt(0)===46&&i.push(""),r.replace(rGt,function(o,l,f,b){i.push(f?b.replace(iGt,"$1"):l||o)}),i});const aGt=sGt;function fAe(r){return r==null?"":qSe(r)}function vV(r,i){return $h(r)?r:qae(r,i)?[r]:aGt(fAe(r))}var oGt=1/0;function _I(r){if(typeof r=="string"||r7(r))return r;var i=r+"";return i=="0"&&1/r==-oGt?"-0":i}function wV(r,i){i=vV(i,r);for(var o=0,l=i.length;r!=null&&o<l;)r=r[_I(i[o++])];return o&&o==l?r:void 0}function cGt(r,i,o){var l=r==null?void 0:wV(r,i);return l===void 0?o:l}function Yae(r,i){for(var o=-1,l=i.length,f=r.length;++o<l;)r[f+o]=i[o];return r}var dAe=I2?I2.isConcatSpreadable:void 0;function uGt(r){return $h(r)||oC(r)||!!(dAe&&r&&r[dAe])}function mV(r,i,o,l,f){var b=-1,d=r.length;for(o||(o=uGt),f||(f=[]);++b<d;){var w=r[b];i>0&&o(w)?i>1?mV(w,i-1,o,l,f):Yae(f,w):l||(f[f.length]=w)}return f}function lC(r){var i=r==null?0:r.length;return i?mV(r,1):[]}function lGt(r){return QSe(tAe(r,void 0,lC),r+"")}var hGt=lAe(Object.getPrototypeOf,Object);const Wae=hGt;var fGt="[object Object]",dGt=Function.prototype,gGt=Object.prototype,gAe=dGt.toString,pGt=gGt.hasOwnProperty,bGt=gAe.call(Object);function pAe(r){if(!jm(r)||Y8(r)!=fGt)return!1;var i=Wae(r);if(i===null)return!0;var o=pGt.call(i,"constructor")&&i.constructor;return typeof o=="function"&&o instanceof o&&gAe.call(o)==bGt}function vGt(r,i,o,l){var f=-1,b=r==null?0:r.length;for(l&&b&&(o=r[++f]);++f<b;)o=i(o,r[f],f,r);return o}function wGt(){this.__data__=new W3,this.size=0}function mGt(r){var i=this.__data__,o=i.delete(r);return this.size=i.size,o}function yGt(r){return this.__data__.get(r)}function kGt(r){return this.__data__.has(r)}var xGt=200;function EGt(r,i){var o=this.__data__;if(o instanceof W3){var l=o.__data__;if(!aI||l.length<xGt-1)return l.push([r,i]),this.size=++o.size,this;o=this.__data__=new K3(l)}return o.set(r,i),this.size=o.size,this}function Rv(r){var i=this.__data__=new W3(r);this.size=i.size}Rv.prototype.clear=wGt,Rv.prototype.delete=mGt,Rv.prototype.get=yGt,Rv.prototype.has=kGt,Rv.prototype.set=EGt;function _Gt(r,i){return r&&kI(i,pp(i),r)}function TGt(r,i){return r&&kI(i,s7(i),r)}var bAe=typeof exports=="object"&&exports&&!exports.nodeType&&exports,vAe=bAe&&typeof module=="object"&&module&&!module.nodeType&&module,CGt=vAe&&vAe.exports===bAe,wAe=CGt?Mv.Buffer:void 0,mAe=wAe?wAe.allocUnsafe:void 0;function yAe(r,i){if(i)return r.slice();var o=r.length,l=mAe?mAe(o):new r.constructor(o);return r.copy(l),l}function kAe(r,i){for(var o=-1,l=r==null?0:r.length,f=0,b=[];++o<l;){var d=r[o];i(d,o,r)&&(b[f++]=d)}return b}function xAe(){return[]}var SGt=Object.prototype,AGt=SGt.propertyIsEnumerable,EAe=Object.getOwnPropertySymbols,MGt=EAe?function(r){return r==null?[]:(r=Object(r),kAe(EAe(r),function(i){return AGt.call(r,i)}))}:xAe;const Kae=MGt;function DGt(r,i){return kI(r,Kae(r),i)}var LGt=Object.getOwnPropertySymbols,IGt=LGt?function(r){for(var i=[];r;)Yae(i,Kae(r)),r=Wae(r);return i}:xAe;const _Ae=IGt;function OGt(r,i){return kI(r,_Ae(r),i)}function TAe(r,i,o){var l=i(r);return $h(r)?l:Yae(l,o(r))}function Xae(r){return TAe(r,pp,Kae)}function NGt(r){return TAe(r,s7,_Ae)}var PGt=K8(Mv,"DataView");const Qae=PGt;var BGt=K8(Mv,"Promise");const Zae=BGt;var FGt=K8(Mv,"Set");const hC=FGt;var CAe="[object Map]",RGt="[object Object]",SAe="[object Promise]",AAe="[object Set]",MAe="[object WeakMap]",DAe="[object DataView]",jGt=W8(Qae),$Gt=W8(aI),HGt=W8(Zae),zGt=W8(hC),GGt=W8(Gae),a7=Y8;(Qae&&a7(new Qae(new ArrayBuffer(1)))!=DAe||aI&&a7(new aI)!=CAe||Zae&&a7(Zae.resolve())!=SAe||hC&&a7(new hC)!=AAe||Gae&&a7(new Gae)!=MAe)&&(a7=function(r){var i=Y8(r),o=i==RGt?r.constructor:void 0,l=o?W8(o):"";if(l)switch(l){case jGt:return DAe;case $Gt:return CAe;case HGt:return SAe;case zGt:return AAe;case GGt:return MAe}return i});const fC=a7;var VGt=Object.prototype,UGt=VGt.hasOwnProperty;function qGt(r){var i=r.length,o=new r.constructor(i);return i&&typeof r[0]=="string"&&UGt.call(r,"index")&&(o.index=r.index,o.input=r.input),o}var YGt=Mv.Uint8Array;const yV=YGt;function Jae(r){var i=new r.constructor(r.byteLength);return new yV(i).set(new yV(r)),i}function WGt(r,i){var o=i?Jae(r.buffer):r.buffer;return new r.constructor(o,r.byteOffset,r.byteLength)}var KGt=/\w*$/;function XGt(r){var i=new r.constructor(r.source,KGt.exec(r));return i.lastIndex=r.lastIndex,i}var LAe=I2?I2.prototype:void 0,IAe=LAe?LAe.valueOf:void 0;function QGt(r){return IAe?Object(IAe.call(r)):{}}function OAe(r,i){var o=i?Jae(r.buffer):r.buffer;return new r.constructor(o,r.byteOffset,r.length)}var ZGt="[object Boolean]",JGt="[object Date]",eVt="[object Map]",tVt="[object Number]",nVt="[object RegExp]",rVt="[object Set]",iVt="[object String]",sVt="[object Symbol]",aVt="[object ArrayBuffer]",oVt="[object DataView]",cVt="[object Float32Array]",uVt="[object Float64Array]",lVt="[object Int8Array]",hVt="[object Int16Array]",fVt="[object Int32Array]",dVt="[object Uint8Array]",gVt="[object Uint8ClampedArray]",pVt="[object Uint16Array]",bVt="[object Uint32Array]";function vVt(r,i,o){var l=r.constructor;switch(i){case aVt:return Jae(r);case ZGt:case JGt:return new l(+r);case oVt:return WGt(r,o);case cVt:case uVt:case lVt:case hVt:case fVt:case dVt:case gVt:case pVt:case bVt:return OAe(r,o);case eVt:return new l;case tVt:case iVt:return new l(r);case nVt:return XGt(r);case rVt:return new l;case sVt:return QGt(r)}}function NAe(r){return typeof r.constructor=="function"&&!gV(r)?jHt(Wae(r)):{}}var wVt="[object Map]";function mVt(r){return jm(r)&&fC(r)==wVt}var PAe=uC&&uC.isMap,yVt=PAe?pV(PAe):mVt;const kVt=yVt;var xVt="[object Set]";function EVt(r){return jm(r)&&fC(r)==xVt}var BAe=uC&&uC.isSet,_Vt=BAe?pV(BAe):EVt;const TVt=_Vt;var CVt=1,SVt=2,AVt=4,FAe="[object Arguments]",MVt="[object Array]",DVt="[object Boolean]",LVt="[object Date]",IVt="[object Error]",RAe="[object Function]",OVt="[object GeneratorFunction]",NVt="[object Map]",PVt="[object Number]",jAe="[object Object]",BVt="[object RegExp]",FVt="[object Set]",RVt="[object String]",jVt="[object Symbol]",$Vt="[object WeakMap]",HVt="[object ArrayBuffer]",zVt="[object DataView]",GVt="[object Float32Array]",VVt="[object Float64Array]",UVt="[object Int8Array]",qVt="[object Int16Array]",YVt="[object Int32Array]",WVt="[object Uint8Array]",KVt="[object Uint8ClampedArray]",XVt="[object Uint16Array]",QVt="[object Uint32Array]",nu={};nu[FAe]=nu[MVt]=nu[HVt]=nu[zVt]=nu[DVt]=nu[LVt]=nu[GVt]=nu[VVt]=nu[UVt]=nu[qVt]=nu[YVt]=nu[NVt]=nu[PVt]=nu[jAe]=nu[BVt]=nu[FVt]=nu[RVt]=nu[jVt]=nu[WVt]=nu[KVt]=nu[XVt]=nu[QVt]=!0,nu[IVt]=nu[RAe]=nu[$Vt]=!1;function TI(r,i,o,l,f,b){var d,w=i&CVt,y=i&SVt,k=i&AVt;if(o&&(d=f?o(r,l,f,b):o(r)),d!==void 0)return d;if(!lp(r))return r;var E=$h(r);if(E){if(d=qGt(r),!w)return XSe(r,d)}else{var T=fC(r),C=T==RAe||T==OVt;if(cC(r))return yAe(r,w);if(T==jAe||T==FAe||C&&!f){if(d=y||C?{}:NAe(r),!w)return y?OGt(r,TGt(d,r)):DGt(r,_Gt(d,r))}else{if(!nu[T])return f?r:{};d=vVt(r,T,w)}}b||(b=new Rv);var S=b.get(r);if(S)return S;b.set(r,d),TVt(r)?r.forEach(function(B){d.add(TI(B,i,o,B,r,b))}):kVt(r)&&r.forEach(function(B,N){d.set(N,TI(B,i,o,N,r,b))});var L=k?y?NGt:Xae:y?s7:pp,O=E?void 0:L(r);return ZSe(O||r,function(B,N){O&&(N=B,B=r[N]),fV(d,N,TI(B,i,o,N,r,b))}),d}var ZVt=4;function JVt(r){return TI(r,ZVt)}var eUt=1,tUt=4;function nUt(r){return TI(r,eUt|tUt)}var rUt="__lodash_hash_undefined__";function iUt(r){return this.__data__.set(r,rUt),this}function sUt(r){return this.__data__.has(r)}function CI(r){var i=-1,o=r==null?0:r.length;for(this.__data__=new K3;++i<o;)this.add(r[i])}CI.prototype.add=CI.prototype.push=iUt,CI.prototype.has=sUt;function aUt(r,i){for(var o=-1,l=r==null?0:r.length;++o<l;)if(i(r[o],o,r))return!0;return!1}function $Ae(r,i){return r.has(i)}var oUt=1,cUt=2;function HAe(r,i,o,l,f,b){var d=o&oUt,w=r.length,y=i.length;if(w!=y&&!(d&&y>w))return!1;var k=b.get(r),E=b.get(i);if(k&&E)return k==i&&E==r;var T=-1,C=!0,S=o&cUt?new CI:void 0;for(b.set(r,i),b.set(i,r);++T<w;){var L=r[T],O=i[T];if(l)var B=d?l(O,L,T,i,r,b):l(L,O,T,r,i,b);if(B!==void 0){if(B)continue;C=!1;break}if(S){if(!aUt(i,function(N,F){if(!$Ae(S,F)&&(L===N||f(L,N,o,l,b)))return S.push(F)})){C=!1;break}}else if(!(L===O||f(L,O,o,l,b))){C=!1;break}}return b.delete(r),b.delete(i),C}function uUt(r){var i=-1,o=Array(r.size);return r.forEach(function(l,f){o[++i]=[f,l]}),o}function eoe(r){var i=-1,o=Array(r.size);return r.forEach(function(l){o[++i]=l}),o}var lUt=1,hUt=2,fUt="[object Boolean]",dUt="[object Date]",gUt="[object Error]",pUt="[object Map]",bUt="[object Number]",vUt="[object RegExp]",wUt="[object Set]",mUt="[object String]",yUt="[object Symbol]",kUt="[object ArrayBuffer]",xUt="[object DataView]",zAe=I2?I2.prototype:void 0,toe=zAe?zAe.valueOf:void 0;function EUt(r,i,o,l,f,b,d){switch(o){case xUt:if(r.byteLength!=i.byteLength||r.byteOffset!=i.byteOffset)return!1;r=r.buffer,i=i.buffer;case kUt:return!(r.byteLength!=i.byteLength||!b(new yV(r),new yV(i)));case fUt:case dUt:case bUt:return ZT(+r,+i);case gUt:return r.name==i.name&&r.message==i.message;case vUt:case mUt:return r==i+"";case pUt:var w=uUt;case wUt:var y=l&lUt;if(w||(w=eoe),r.size!=i.size&&!y)return!1;var k=d.get(r);if(k)return k==i;l|=hUt,d.set(r,i);var E=HAe(w(r),w(i),l,f,b,d);return d.delete(r),E;case yUt:if(toe)return toe.call(r)==toe.call(i)}return!1}var _Ut=1,TUt=Object.prototype,CUt=TUt.hasOwnProperty;function SUt(r,i,o,l,f,b){var d=o&_Ut,w=Xae(r),y=w.length,k=Xae(i),E=k.length;if(y!=E&&!d)return!1;for(var T=y;T--;){var C=w[T];if(!(d?C in i:CUt.call(i,C)))return!1}var S=b.get(r),L=b.get(i);if(S&&L)return S==i&&L==r;var O=!0;b.set(r,i),b.set(i,r);for(var B=d;++T<y;){C=w[T];var N=r[C],F=i[C];if(l)var R=d?l(F,N,C,i,r,b):l(N,F,C,r,i,b);if(!(R===void 0?N===F||f(N,F,o,l,b):R)){O=!1;break}B||(B=C=="constructor")}if(O&&!B){var q=r.constructor,X=i.constructor;q!=X&&"constructor"in r&&"constructor"in i&&!(typeof q=="function"&&q instanceof q&&typeof X=="function"&&X instanceof X)&&(O=!1)}return b.delete(r),b.delete(i),O}var AUt=1,GAe="[object Arguments]",VAe="[object Array]",kV="[object Object]",MUt=Object.prototype,UAe=MUt.hasOwnProperty;function DUt(r,i,o,l,f,b){var d=$h(r),w=$h(i),y=d?VAe:fC(r),k=w?VAe:fC(i);y=y==GAe?kV:y,k=k==GAe?kV:k;var E=y==kV,T=k==kV,C=y==k;if(C&&cC(r)){if(!cC(i))return!1;d=!0,E=!1}if(C&&!E)return b||(b=new Rv),d||bV(r)?HAe(r,i,o,l,f,b):EUt(r,i,y,o,l,f,b);if(!(o&AUt)){var S=E&&UAe.call(r,"__wrapped__"),L=T&&UAe.call(i,"__wrapped__");if(S||L){var O=S?r.value():r,B=L?i.value():i;return b||(b=new Rv),f(O,B,o,l,b)}}return C?(b||(b=new Rv),SUt(r,i,o,l,f,b)):!1}function noe(r,i,o,l,f){return r===i?!0:r==null||i==null||!jm(r)&&!jm(i)?r!==r&&i!==i:DUt(r,i,o,l,noe,f)}var LUt=1,IUt=2;function OUt(r,i,o,l){var f=o.length,b=f,d=!l;if(r==null)return!b;for(r=Object(r);f--;){var w=o[f];if(d&&w[2]?w[1]!==r[w[0]]:!(w[0]in r))return!1}for(;++f<b;){w=o[f];var y=w[0],k=r[y],E=w[1];if(d&&w[2]){if(k===void 0&&!(y in r))return!1}else{var T=new Rv;if(l)var C=l(k,E,y,r,i,T);if(!(C===void 0?noe(E,k,LUt|IUt,l,T):C))return!1}}return!0}function qAe(r){return r===r&&!lp(r)}function NUt(r){for(var i=pp(r),o=i.length;o--;){var l=i[o],f=r[l];i[o]=[l,f,qAe(f)]}return i}function YAe(r,i){return function(o){return o==null?!1:o[r]===i&&(i!==void 0||r in Object(o))}}function PUt(r){var i=NUt(r);return i.length==1&&i[0][2]?YAe(i[0][0],i[0][1]):function(o){return o===r||OUt(o,r,i)}}function BUt(r,i){return r!=null&&i in Object(r)}function WAe(r,i,o){i=vV(i,r);for(var l=-1,f=i.length,b=!1;++l<f;){var d=_I(i[l]);if(!(b=r!=null&&o(r,d)))break;r=r[d]}return b||++l!=f?b:(f=r==null?0:r.length,!!f&&Vae(f)&&lV(d,f)&&($h(r)||oC(r)))}function KAe(r,i){return r!=null&&WAe(r,i,BUt)}var FUt=1,RUt=2;function jUt(r,i){return qae(r)&&qAe(i)?YAe(_I(r),i):function(o){var l=cGt(o,r);return l===void 0&&l===i?KAe(o,r):noe(i,l,FUt|RUt)}}function $Ut(r){return function(i){return i==null?void 0:i[r]}}function HUt(r){return function(i){return wV(i,r)}}function zUt(r){return qae(r)?$Ut(_I(r)):HUt(r)}function b6(r){return typeof r=="function"?r:r==null?i7:typeof r=="object"?$h(r)?jUt(r[0],r[1]):PUt(r):zUt(r)}function GUt(r){return function(i,o,l){for(var f=-1,b=Object(i),d=l(i),w=d.length;w--;){var y=d[r?w:++f];if(o(b[y],y,b)===!1)break}return i}}var VUt=GUt();const roe=VUt;function XAe(r,i){return r&&roe(r,i,pp)}function UUt(r,i){return function(o,l){if(o==null)return o;if(!p6(o))return r(o,l);for(var f=o.length,b=i?f:-1,d=Object(o);(i?b--:++b<f)&&l(d[b],b,d)!==!1;);return o}}var qUt=UUt(XAe);const xV=qUt;var YUt=function(){return Mv.Date.now()};const QAe=YUt;var ZAe=Object.prototype,WUt=ZAe.hasOwnProperty,KUt=dV(function(r,i){r=Object(r);var o=-1,l=i.length,f=l>2?i[2]:void 0;for(f&&xI(i[0],i[1],f)&&(l=1);++o<l;)for(var b=i[o],d=s7(b),w=-1,y=d.length;++w<y;){var k=d[w],E=r[k];(E===void 0||ZT(E,ZAe[k])&&!WUt.call(r,k))&&(r[k]=b[k])}return r});const dC=KUt;function ioe(r,i,o){(o!==void 0&&!ZT(r[i],o)||o===void 0&&!(i in r))&&hV(r,i,o)}function JAe(r){return jm(r)&&p6(r)}function soe(r,i){if(!(i==="constructor"&&typeof r[i]=="function")&&i!="__proto__")return r[i]}function XUt(r){return kI(r,s7(r))}function QUt(r,i,o,l,f,b,d){var w=soe(r,o),y=soe(i,o),k=d.get(y);if(k){ioe(r,o,k);return}var E=b?b(w,y,o+"",r,i,d):void 0,T=E===void 0;if(T){var C=$h(y),S=!C&&cC(y),L=!C&&!S&&bV(y);E=y,C||S||L?$h(w)?E=w:JAe(w)?E=XSe(w):S?(T=!1,E=yAe(y,!0)):L?(T=!1,E=OAe(y,!0)):E=[]:pAe(y)||oC(y)?(E=w,oC(w)?E=XUt(w):(!lp(w)||QT(w))&&(E=NAe(y))):T=!1}T&&(d.set(y,E),f(E,y,l,b,d),d.delete(y)),ioe(r,o,E)}function eMe(r,i,o,l,f){r!==i&&roe(i,function(b,d){if(f||(f=new Rv),lp(b))QUt(r,i,d,o,eMe,l,f);else{var w=l?l(soe(r,d),b,d+"",r,i,f):void 0;w===void 0&&(w=b),ioe(r,d,w)}},s7)}function ZUt(r,i,o){for(var l=-1,f=r==null?0:r.length;++l<f;)if(o(i,r[l]))return!0;return!1}function EV(r){var i=r==null?0:r.length;return i?r[i-1]:void 0}function tMe(r){return typeof r=="function"?r:i7}function Mn(r,i){var o=$h(r)?ZSe:xV;return o(r,tMe(i))}function JUt(r,i){var o=[];return xV(r,function(l,f,b){i(l,f,b)&&o.push(l)}),o}function Q3(r,i){var o=$h(r)?kAe:JUt;return o(r,b6(i))}function eqt(r){return function(i,o,l){var f=Object(i);if(!p6(i)){var b=b6(o);i=pp(i),o=function(w){return b(f[w],w,f)}}var d=r(i,o,l);return d>-1?f[b?i[d]:d]:void 0}}var tqt=Math.max;function nqt(r,i,o){var l=r==null?0:r.length;if(!l)return-1;var f=o==null?0:BHt(o);return f<0&&(f=tqt(l+f,0)),JSe(r,b6(i),f)}var rqt=eqt(nqt);const aoe=rqt;function nMe(r,i){var o=-1,l=p6(r)?Array(r.length):[];return xV(r,function(f,b,d){l[++o]=i(f,b,d)}),l}function Hh(r,i){var o=$h(r)?sC:nMe;return o(r,b6(i))}function iqt(r,i){return r==null?r:roe(r,tMe(i),s7)}function sqt(r,i){return r>i}var aqt=Object.prototype,oqt=aqt.hasOwnProperty;function cqt(r,i){return r!=null&&oqt.call(r,i)}function Ks(r,i){return r!=null&&WAe(r,i,cqt)}function uqt(r,i){return sC(i,function(o){return r[o]})}function Z3(r){return r==null?[]:uqt(r,pp(r))}var lqt="[object Map]",hqt="[object Set]",fqt=Object.prototype,dqt=fqt.hasOwnProperty;function gC(r){if(r==null)return!0;if(p6(r)&&($h(r)||typeof r=="string"||typeof r.splice=="function"||cC(r)||bV(r)||oC(r)))return!r.length;var i=fC(r);if(i==lqt||i==hqt)return!r.size;if(gV(r))return!hAe(r).length;for(var o in r)if(dqt.call(r,o))return!1;return!0}function oh(r){return r===void 0}function rMe(r,i){return r<i}function _V(r,i){var o={};return i=b6(i),XAe(r,function(l,f,b){hV(o,f,i(l,f,b))}),o}function ooe(r,i,o){for(var l=-1,f=r.length;++l<f;){var b=r[l],d=i(b);if(d!=null&&(w===void 0?d===d&&!r7(d):o(d,w)))var w=d,y=b}return y}function o7(r){return r&&r.length?ooe(r,i7,sqt):void 0}var gqt=izt(function(r,i,o){eMe(r,i,o)});const coe=gqt;function SI(r){return r&&r.length?ooe(r,i7,rMe):void 0}function uoe(r,i){return r&&r.length?ooe(r,b6(i),rMe):void 0}function pqt(r,i,o,l){if(!lp(r))return r;i=vV(i,r);for(var f=-1,b=i.length,d=b-1,w=r;w!=null&&++f<b;){var y=_I(i[f]),k=o;if(y==="__proto__"||y==="constructor"||y==="prototype")return r;if(f!=d){var E=w[y];k=l?l(E,y,w):void 0,k===void 0&&(k=lp(E)?E:lV(i[f+1])?[]:{})}fV(w,y,k),w=w[y]}return r}function bqt(r,i,o){for(var l=-1,f=i.length,b={};++l<f;){var d=i[l],w=wV(r,d);o(w,d)&&pqt(b,vV(d,r),w)}return b}function vqt(r,i){var o=r.length;for(r.sort(i);o--;)r[o]=r[o].value;return r}function wqt(r,i){if(r!==i){var o=r!==void 0,l=r===null,f=r===r,b=r7(r),d=i!==void 0,w=i===null,y=i===i,k=r7(i);if(!w&&!k&&!b&&r>i||b&&d&&y&&!w&&!k||l&&d&&y||!o&&y||!f)return 1;if(!l&&!b&&!k&&r<i||k&&o&&f&&!l&&!b||w&&o&&f||!d&&f||!y)return-1}return 0}function mqt(r,i,o){for(var l=-1,f=r.criteria,b=i.criteria,d=f.length,w=o.length;++l<d;){var y=wqt(f[l],b[l]);if(y){if(l>=w)return y;var k=o[l];return y*(k=="desc"?-1:1)}}return r.index-i.index}function yqt(r,i,o){i.length?i=sC(i,function(b){return $h(b)?function(d){return wV(d,b.length===1?b[0]:b)}:b}):i=[i7];var l=-1;i=sC(i,pV(b6));var f=nMe(r,function(b,d,w){var y=sC(i,function(k){return k(b)});return{criteria:y,index:++l,value:b}});return vqt(f,function(b,d){return mqt(b,d,o)})}function kqt(r,i){return bqt(r,i,function(o,l){return KAe(r,l)})}var xqt=lGt(function(r,i){return r==null?{}:kqt(r,i)});const AI=xqt;var Eqt=Math.ceil,_qt=Math.max;function Tqt(r,i,o,l){for(var f=-1,b=_qt(Eqt((i-r)/(o||1)),0),d=Array(b);b--;)d[l?b:++f]=r,r+=o;return d}function Cqt(r){return function(i,o,l){return l&&typeof l!="number"&&xI(i,o,l)&&(o=l=void 0),i=cV(i),o===void 0?(o=i,i=0):o=cV(o),l=l===void 0?i<o?1:-1:cV(l),Tqt(i,o,l,r)}}var Sqt=Cqt();const c7=Sqt;function Aqt(r,i,o,l,f){return f(r,function(b,d,w){o=l?(l=!1,b):i(o,b,d,w)}),o}function MI(r,i,o){var l=$h(r)?vGt:Aqt,f=arguments.length<3;return l(r,b6(i),o,f,xV)}var Mqt=dV(function(r,i){if(r==null)return[];var o=i.length;return o>1&&xI(r,i[0],i[1])?i=[]:o>2&&xI(i[0],i[1],i[2])&&(i=[i[0]]),yqt(r,mV(i,1),[])});const DI=Mqt;var Dqt=1/0,Lqt=hC&&1/eoe(new hC([,-0]))[1]==Dqt?function(r){return new hC(r)}:HHt;const Iqt=Lqt;var Oqt=200;function Nqt(r,i,o){var l=-1,f=ZHt,b=r.length,d=!0,w=[],y=w;if(o)d=!1,f=ZUt;else if(b>=Oqt){var k=i?null:Iqt(r);if(k)return eoe(k);d=!1,f=$Ae,y=new CI}else y=i?[]:w;e:for(;++l<b;){var E=r[l],T=i?i(E):E;if(E=o||E!==0?E:0,d&&T===T){for(var C=y.length;C--;)if(y[C]===T)continue e;i&&y.push(T),w.push(E)}else f(y,T,o)||(y!==w&&y.push(T),w.push(E))}return w}var Pqt=dV(function(r){return Nqt(mV(r,1,JAe,!0))});const Bqt=Pqt;var Fqt=0;function TV(r){var i=++Fqt;return fAe(r)+i}function Rqt(r,i,o){for(var l=-1,f=r.length,b=i.length,d={};++l<f;){var w=l<b?i[l]:void 0;o(d,r[l],w)}return d}function jqt(r,i){return Rqt(r||[],i||[],fV)}var $qt="\0",u7="\0",iMe="";class zh{constructor(i={}){this._isDirected=Ks(i,"directed")?i.directed:!0,this._isMultigraph=Ks(i,"multigraph")?i.multigraph:!1,this._isCompound=Ks(i,"compound")?i.compound:!1,this._label=void 0,this._defaultNodeLabelFn=aC(void 0),this._defaultEdgeLabelFn=aC(void 0),this._nodes={},this._isCompound&&(this._parent={},this._children={},this._children[u7]={}),this._in={},this._preds={},this._out={},this._sucs={},this._edgeObjs={},this._edgeLabels={}}isDirected(){return this._isDirected}isMultigraph(){return this._isMultigraph}isCompound(){return this._isCompound}setGraph(i){return this._label=i,this}graph(){return this._label}setDefaultNodeLabel(i){return QT(i)||(i=aC(i)),this._defaultNodeLabelFn=i,this}nodeCount(){return this._nodeCount}nodes(){return pp(this._nodes)}sources(){var i=this;return Q3(this.nodes(),function(o){return gC(i._in[o])})}sinks(){var i=this;return Q3(this.nodes(),function(o){return gC(i._out[o])})}setNodes(i,o){var l=arguments,f=this;return Mn(i,function(b){l.length>1?f.setNode(b,o):f.setNode(b)}),this}setNode(i,o){return Ks(this._nodes,i)?(arguments.length>1&&(this._nodes[i]=o),this):(this._nodes[i]=arguments.length>1?o:this._defaultNodeLabelFn(i),this._isCompound&&(this._parent[i]=u7,this._children[i]={},this._children[u7][i]=!0),this._in[i]={},this._preds[i]={},this._out[i]={},this._sucs[i]={},++this._nodeCount,this)}node(i){return this._nodes[i]}hasNode(i){return Ks(this._nodes,i)}removeNode(i){var o=this;if(Ks(this._nodes,i)){var l=function(f){o.removeEdge(o._edgeObjs[f])};delete this._nodes[i],this._isCompound&&(this._removeFromParentsChildList(i),delete this._parent[i],Mn(this.children(i),function(f){o.setParent(f)}),delete this._children[i]),Mn(pp(this._in[i]),l),delete this._in[i],delete this._preds[i],Mn(pp(this._out[i]),l),delete this._out[i],delete this._sucs[i],--this._nodeCount}return this}setParent(i,o){if(!this._isCompound)throw new Error("Cannot set parent in a non-compound graph");if(oh(o))o=u7;else{o+="";for(var l=o;!oh(l);l=this.parent(l))if(l===i)throw new Error("Setting "+o+" as parent of "+i+" would create a cycle");this.setNode(o)}return this.setNode(i),this._removeFromParentsChildList(i),this._parent[i]=o,this._children[o][i]=!0,this}_removeFromParentsChildList(i){delete this._children[this._parent[i]][i]}parent(i){if(this._isCompound){var o=this._parent[i];if(o!==u7)return o}}children(i){if(oh(i)&&(i=u7),this._isCompound){var o=this._children[i];if(o)return pp(o)}else{if(i===u7)return this.nodes();if(this.hasNode(i))return[]}}predecessors(i){var o=this._preds[i];if(o)return pp(o)}successors(i){var o=this._sucs[i];if(o)return pp(o)}neighbors(i){var o=this.predecessors(i);if(o)return Bqt(o,this.successors(i))}isLeaf(i){var o;return this.isDirected()?o=this.successors(i):o=this.neighbors(i),o.length===0}filterNodes(i){var o=new this.constructor({directed:this._isDirected,multigraph:this._isMultigraph,compound:this._isCompound});o.setGraph(this.graph());var l=this;Mn(this._nodes,function(d,w){i(w)&&o.setNode(w,d)}),Mn(this._edgeObjs,function(d){o.hasNode(d.v)&&o.hasNode(d.w)&&o.setEdge(d,l.edge(d))});var f={};function b(d){var w=l.parent(d);return w===void 0||o.hasNode(w)?(f[d]=w,w):w in f?f[w]:b(w)}return this._isCompound&&Mn(o.nodes(),function(d){o.setParent(d,b(d))}),o}setDefaultEdgeLabel(i){return QT(i)||(i=aC(i)),this._defaultEdgeLabelFn=i,this}edgeCount(){return this._edgeCount}edges(){return Z3(this._edgeObjs)}setPath(i,o){var l=this,f=arguments;return MI(i,function(b,d){return f.length>1?l.setEdge(b,d,o):l.setEdge(b,d),d}),this}setEdge(){var i,o,l,f,b=!1,d=arguments[0];typeof d=="object"&&d!==null&&"v"in d?(i=d.v,o=d.w,l=d.name,arguments.length===2&&(f=arguments[1],b=!0)):(i=d,o=arguments[1],l=arguments[3],arguments.length>2&&(f=arguments[2],b=!0)),i=""+i,o=""+o,oh(l)||(l=""+l);var w=LI(this._isDirected,i,o,l);if(Ks(this._edgeLabels,w))return b&&(this._edgeLabels[w]=f),this;if(!oh(l)&&!this._isMultigraph)throw new Error("Cannot set a named edge when isMultigraph = false");this.setNode(i),this.setNode(o),this._edgeLabels[w]=b?f:this._defaultEdgeLabelFn(i,o,l);var y=Hqt(this._isDirected,i,o,l);return i=y.v,o=y.w,Object.freeze(y),this._edgeObjs[w]=y,sMe(this._preds[o],i),sMe(this._sucs[i],o),this._in[o][w]=y,this._out[i][w]=y,this._edgeCount++,this}edge(i,o,l){var f=arguments.length===1?loe(this._isDirected,arguments[0]):LI(this._isDirected,i,o,l);return this._edgeLabels[f]}hasEdge(i,o,l){var f=arguments.length===1?loe(this._isDirected,arguments[0]):LI(this._isDirected,i,o,l);return Ks(this._edgeLabels,f)}removeEdge(i,o,l){var f=arguments.length===1?loe(this._isDirected,arguments[0]):LI(this._isDirected,i,o,l),b=this._edgeObjs[f];return b&&(i=b.v,o=b.w,delete this._edgeLabels[f],delete this._edgeObjs[f],aMe(this._preds[o],i),aMe(this._sucs[i],o),delete this._in[o][f],delete this._out[i][f],this._edgeCount--),this}inEdges(i,o){var l=this._in[i];if(l){var f=Z3(l);return o?Q3(f,function(b){return b.v===o}):f}}outEdges(i,o){var l=this._out[i];if(l){var f=Z3(l);return o?Q3(f,function(b){return b.w===o}):f}}nodeEdges(i,o){var l=this.inEdges(i,o);if(l)return l.concat(this.outEdges(i,o))}}zh.prototype._nodeCount=0,zh.prototype._edgeCount=0;function sMe(r,i){r[i]?r[i]++:r[i]=1}function aMe(r,i){--r[i]||delete r[i]}function LI(r,i,o,l){var f=""+i,b=""+o;if(!r&&f>b){var d=f;f=b,b=d}return f+iMe+b+iMe+(oh(l)?$qt:l)}function Hqt(r,i,o,l){var f=""+i,b=""+o;if(!r&&f>b){var d=f;f=b,b=d}var w={v:f,w:b};return l&&(w.name=l),w}function loe(r,i){return LI(r,i.v,i.w,i.name)}class zqt{constructor(){var i={};i._next=i._prev=i,this._sentinel=i}dequeue(){var i=this._sentinel,o=i._prev;if(o!==i)return oMe(o),o}enqueue(i){var o=this._sentinel;i._prev&&i._next&&oMe(i),i._next=o._next,o._next._prev=i,o._next=i,i._prev=o}toString(){for(var i=[],o=this._sentinel,l=o._prev;l!==o;)i.push(JSON.stringify(l,Gqt)),l=l._prev;return"["+i.join(", ")+"]"}}function oMe(r){r._prev._next=r._next,r._next._prev=r._prev,delete r._next,delete r._prev}function Gqt(r,i){if(r!=="_next"&&r!=="_prev")return i}var Vqt=aC(1);function Uqt(r,i){if(r.nodeCount()<=1)return[];var o=Yqt(r,i||Vqt),l=qqt(o.graph,o.buckets,o.zeroIdx);return lC(Hh(l,function(f){return r.outEdges(f.v,f.w)}))}function qqt(r,i,o){for(var l=[],f=i[i.length-1],b=i[0],d;r.nodeCount();){for(;d=b.dequeue();)hoe(r,i,o,d);for(;d=f.dequeue();)hoe(r,i,o,d);if(r.nodeCount()){for(var w=i.length-2;w>0;--w)if(d=i[w].dequeue(),d){l=l.concat(hoe(r,i,o,d,!0));break}}}return l}function hoe(r,i,o,l,f){var b=f?[]:void 0;return Mn(r.inEdges(l.v),function(d){var w=r.edge(d),y=r.node(d.v);f&&b.push({v:d.v,w:d.w}),y.out-=w,foe(i,o,y)}),Mn(r.outEdges(l.v),function(d){var w=r.edge(d),y=d.w,k=r.node(y);k.in-=w,foe(i,o,k)}),r.removeNode(l.v),b}function Yqt(r,i){var o=new zh,l=0,f=0;Mn(r.nodes(),function(w){o.setNode(w,{v:w,in:0,out:0})}),Mn(r.edges(),function(w){var y=o.edge(w.v,w.w)||0,k=i(w),E=y+k;o.setEdge(w.v,w.w,E),f=Math.max(f,o.node(w.v).out+=k),l=Math.max(l,o.node(w.w).in+=k)});var b=c7(f+l+3).map(function(){return new zqt}),d=l+1;return Mn(o.nodes(),function(w){foe(b,d,o.node(w))}),{graph:o,buckets:b,zeroIdx:d}}function foe(r,i,o){o.out?o.in?r[o.out-o.in+i].enqueue(o):r[r.length-1].enqueue(o):r[0].enqueue(o)}function Wqt(r){var i=r.graph().acyclicer==="greedy"?Uqt(r,o(r)):Kqt(r);Mn(i,function(l){var f=r.edge(l);r.removeEdge(l),f.forwardName=l.name,f.reversed=!0,r.setEdge(l.w,l.v,f,TV("rev"))});function o(l){return function(f){return l.edge(f).weight}}}function Kqt(r){var i=[],o={},l={};function f(b){Ks(l,b)||(l[b]=!0,o[b]=!0,Mn(r.outEdges(b),function(d){Ks(o,d.w)?i.push(d):f(d.w)}),delete o[b])}return Mn(r.nodes(),f),i}function Xqt(r){Mn(r.edges(),function(i){var o=r.edge(i);if(o.reversed){r.removeEdge(i);var l=o.forwardName;delete o.reversed,delete o.forwardName,r.setEdge(i.w,i.v,o,l)}})}function pC(r,i,o,l){var f;do f=TV(l);while(r.hasNode(f));return o.dummy=i,r.setNode(f,o),f}function Qqt(r){var i=new zh().setGraph(r.graph());return Mn(r.nodes(),function(o){i.setNode(o,r.node(o))}),Mn(r.edges(),function(o){var l=i.edge(o.v,o.w)||{weight:0,minlen:1},f=r.edge(o);i.setEdge(o.v,o.w,{weight:l.weight+f.weight,minlen:Math.max(l.minlen,f.minlen)})}),i}function cMe(r){var i=new zh({multigraph:r.isMultigraph()}).setGraph(r.graph());return Mn(r.nodes(),function(o){r.children(o).length||i.setNode(o,r.node(o))}),Mn(r.edges(),function(o){i.setEdge(o,r.edge(o))}),i}function uMe(r,i){var o=r.x,l=r.y,f=i.x-o,b=i.y-l,d=r.width/2,w=r.height/2;if(!f&&!b)throw new Error("Not possible to find intersection inside of the rectangle");var y,k;return Math.abs(b)*d>Math.abs(f)*w?(b<0&&(w=-w),y=w*f/b,k=w):(f<0&&(d=-d),y=d,k=d*b/f),{x:o+y,y:l+k}}function CV(r){var i=Hh(c7(hMe(r)+1),function(){return[]});return Mn(r.nodes(),function(o){var l=r.node(o),f=l.rank;oh(f)||(i[f][l.order]=o)}),i}function Zqt(r){var i=SI(Hh(r.nodes(),function(o){return r.node(o).rank}));Mn(r.nodes(),function(o){var l=r.node(o);Ks(l,"rank")&&(l.rank-=i)})}function Jqt(r){var i=SI(Hh(r.nodes(),function(b){return r.node(b).rank})),o=[];Mn(r.nodes(),function(b){var d=r.node(b).rank-i;o[d]||(o[d]=[]),o[d].push(b)});var l=0,f=r.graph().nodeRankFactor;Mn(o,function(b,d){oh(b)&&d%f!==0?--l:l&&Mn(b,function(w){r.node(w).rank+=l})})}function lMe(r,i,o,l){var f={width:0,height:0};return arguments.length>=4&&(f.rank=o,f.order=l),pC(r,"border",f,i)}function hMe(r){return o7(Hh(r.nodes(),function(i){var o=r.node(i).rank;if(!oh(o))return o}))}function eYt(r,i){var o={lhs:[],rhs:[]};return Mn(r,function(l){i(l)?o.lhs.push(l):o.rhs.push(l)}),o}function tYt(r,i){var o=QAe();try{return i()}finally{console.log(r+" time: "+(QAe()-o)+"ms")}}function nYt(r,i){return i()}function rYt(r){function i(o){var l=r.children(o),f=r.node(o);if(l.length&&Mn(l,i),Ks(f,"minRank")){f.borderLeft=[],f.borderRight=[];for(var b=f.minRank,d=f.maxRank+1;b<d;++b)fMe(r,"borderLeft","_bl",o,f,b),fMe(r,"borderRight","_br",o,f,b)}}Mn(r.children(),i)}function fMe(r,i,o,l,f,b){var d={width:0,height:0,rank:b,borderType:i},w=f[i][b-1],y=pC(r,"border",d,o);f[i][b]=y,r.setParent(y,l),w&&r.setEdge(w,y,{weight:1})}function iYt(r){var i=r.graph().rankdir.toLowerCase();(i==="lr"||i==="rl")&&dMe(r)}function sYt(r){var i=r.graph().rankdir.toLowerCase();(i==="bt"||i==="rl")&&aYt(r),(i==="lr"||i==="rl")&&(oYt(r),dMe(r))}function dMe(r){Mn(r.nodes(),function(i){gMe(r.node(i))}),Mn(r.edges(),function(i){gMe(r.edge(i))})}function gMe(r){var i=r.width;r.width=r.height,r.height=i}function aYt(r){Mn(r.nodes(),function(i){doe(r.node(i))}),Mn(r.edges(),function(i){var o=r.edge(i);Mn(o.points,doe),Ks(o,"y")&&doe(o)})}function doe(r){r.y=-r.y}function oYt(r){Mn(r.nodes(),function(i){goe(r.node(i))}),Mn(r.edges(),function(i){var o=r.edge(i);Mn(o.points,goe),Ks(o,"x")&&goe(o)})}function goe(r){var i=r.x;r.x=r.y,r.y=i}function cYt(r){var i=pC(r,"root",{},"_root"),o=uYt(r),l=o7(Z3(o))-1,f=2*l+1;r.graph().nestingRoot=i,Mn(r.edges(),function(d){r.edge(d).minlen*=f});var b=lYt(r)+1;Mn(r.children(),function(d){pMe(r,i,f,b,l,o,d)}),r.graph().nodeRankFactor=f}function pMe(r,i,o,l,f,b,d){var w=r.children(d);if(!w.length){d!==i&&r.setEdge(i,d,{weight:0,minlen:o});return}var y=lMe(r,"_bt"),k=lMe(r,"_bb"),E=r.node(d);r.setParent(y,d),E.borderTop=y,r.setParent(k,d),E.borderBottom=k,Mn(w,function(T){pMe(r,i,o,l,f,b,T);var C=r.node(T),S=C.borderTop?C.borderTop:T,L=C.borderBottom?C.borderBottom:T,O=C.borderTop?l:2*l,B=S!==L?1:f-b[d]+1;r.setEdge(y,S,{weight:O,minlen:B,nestingEdge:!0}),r.setEdge(L,k,{weight:O,minlen:B,nestingEdge:!0})}),r.parent(d)||r.setEdge(i,y,{weight:0,minlen:f+b[d]})}function uYt(r){var i={};function o(l,f){var b=r.children(l);b&&b.length&&Mn(b,function(d){o(d,f+1)}),i[l]=f}return Mn(r.children(),function(l){o(l,1)}),i}function lYt(r){return MI(r.edges(),function(i,o){return i+r.edge(o).weight},0)}function hYt(r){var i=r.graph();r.removeNode(i.nestingRoot),delete i.nestingRoot,Mn(r.edges(),function(o){var l=r.edge(o);l.nestingEdge&&r.removeEdge(o)})}function fYt(r,i,o){var l={},f;Mn(o,function(b){for(var d=r.parent(b),w,y;d;){if(w=r.parent(d),w?(y=l[w],l[w]=d):(y=f,f=d),y&&y!==d){i.setEdge(y,d);return}d=w}})}function dYt(r,i,o){var l=gYt(r),f=new zh({compound:!0}).setGraph({root:l}).setDefaultNodeLabel(function(b){return r.node(b)});return Mn(r.nodes(),function(b){var d=r.node(b),w=r.parent(b);(d.rank===i||d.minRank<=i&&i<=d.maxRank)&&(f.setNode(b),f.setParent(b,w||l),Mn(r[o](b),function(y){var k=y.v===b?y.w:y.v,E=f.edge(k,b),T=oh(E)?0:E.weight;f.setEdge(k,b,{weight:r.edge(y).weight+T})}),Ks(d,"minRank")&&f.setNode(b,{borderLeft:d.borderLeft[i],borderRight:d.borderRight[i]}))}),f}function gYt(r){for(var i;r.hasNode(i=TV("_root")););return i}function pYt(r,i){for(var o=0,l=1;l<i.length;++l)o+=bYt(r,i[l-1],i[l]);return o}function bYt(r,i,o){for(var l=jqt(o,Hh(o,function(k,E){return E})),f=lC(Hh(i,function(k){return DI(Hh(r.outEdges(k),function(E){return{pos:l[E.w],weight:r.edge(E).weight}}),"pos")})),b=1;b<o.length;)b<<=1;var d=2*b-1;b-=1;var w=Hh(new Array(d),function(){return 0}),y=0;return Mn(f.forEach(function(k){var E=k.pos+b;w[E]+=k.weight;for(var T=0;E>0;)E%2&&(T+=w[E+1]),E=E-1>>1,w[E]+=k.weight;y+=k.weight*T})),y}function vYt(r){var i={},o=Q3(r.nodes(),function(w){return!r.children(w).length}),l=o7(Hh(o,function(w){return r.node(w).rank})),f=Hh(c7(l+1),function(){return[]});function b(w){if(!Ks(i,w)){i[w]=!0;var y=r.node(w);f[y.rank].push(w),Mn(r.successors(w),b)}}var d=DI(o,function(w){return r.node(w).rank});return Mn(d,b),f}function wYt(r,i){return Hh(i,function(o){var l=r.inEdges(o);if(l.length){var f=MI(l,function(b,d){var w=r.edge(d),y=r.node(d.v);return{sum:b.sum+w.weight*y.order,weight:b.weight+w.weight}},{sum:0,weight:0});return{v:o,barycenter:f.sum/f.weight,weight:f.weight}}else return{v:o}})}function mYt(r,i){var o={};Mn(r,function(f,b){var d=o[f.v]={indegree:0,in:[],out:[],vs:[f.v],i:b};oh(f.barycenter)||(d.barycenter=f.barycenter,d.weight=f.weight)}),Mn(i.edges(),function(f){var b=o[f.v],d=o[f.w];!oh(b)&&!oh(d)&&(d.indegree++,b.out.push(o[f.w]))});var l=Q3(o,function(f){return!f.indegree});return yYt(l)}function yYt(r){var i=[];function o(b){return function(d){d.merged||(oh(d.barycenter)||oh(b.barycenter)||d.barycenter>=b.barycenter)&&kYt(b,d)}}function l(b){return function(d){d.in.push(b),--d.indegree===0&&r.push(d)}}for(;r.length;){var f=r.pop();i.push(f),Mn(f.in.reverse(),o(f)),Mn(f.out,l(f))}return Hh(Q3(i,function(b){return!b.merged}),function(b){return AI(b,["vs","i","barycenter","weight"])})}function kYt(r,i){var o=0,l=0;r.weight&&(o+=r.barycenter*r.weight,l+=r.weight),i.weight&&(o+=i.barycenter*i.weight,l+=i.weight),r.vs=i.vs.concat(r.vs),r.barycenter=o/l,r.weight=l,r.i=Math.min(i.i,r.i),i.merged=!0}function xYt(r,i){var o=eYt(r,function(E){return Ks(E,"barycenter")}),l=o.lhs,f=DI(o.rhs,function(E){return-E.i}),b=[],d=0,w=0,y=0;l.sort(EYt(!!i)),y=bMe(b,f,y),Mn(l,function(E){y+=E.vs.length,b.push(E.vs),d+=E.barycenter*E.weight,w+=E.weight,y=bMe(b,f,y)});var k={vs:lC(b)};return w&&(k.barycenter=d/w,k.weight=w),k}function bMe(r,i,o){for(var l;i.length&&(l=EV(i)).i<=o;)i.pop(),r.push(l.vs),o++;return o}function EYt(r){return function(i,o){return i.barycenter<o.barycenter?-1:i.barycenter>o.barycenter?1:r?o.i-i.i:i.i-o.i}}function vMe(r,i,o,l){var f=r.children(i),b=r.node(i),d=b?b.borderLeft:void 0,w=b?b.borderRight:void 0,y={};d&&(f=Q3(f,function(L){return L!==d&&L!==w}));var k=wYt(r,f);Mn(k,function(L){if(r.children(L.v).length){var O=vMe(r,L.v,o,l);y[L.v]=O,Ks(O,"barycenter")&&TYt(L,O)}});var E=mYt(k,o);_Yt(E,y);var T=xYt(E,l);if(d&&(T.vs=lC([d,T.vs,w]),r.predecessors(d).length)){var C=r.node(r.predecessors(d)[0]),S=r.node(r.predecessors(w)[0]);Ks(T,"barycenter")||(T.barycenter=0,T.weight=0),T.barycenter=(T.barycenter*T.weight+C.order+S.order)/(T.weight+2),T.weight+=2}return T}function _Yt(r,i){Mn(r,function(o){o.vs=lC(o.vs.map(function(l){return i[l]?i[l].vs:l}))})}function TYt(r,i){oh(r.barycenter)?(r.barycenter=i.barycenter,r.weight=i.weight):(r.barycenter=(r.barycenter*r.weight+i.barycenter*i.weight)/(r.weight+i.weight),r.weight+=i.weight)}function CYt(r){var i=hMe(r),o=wMe(r,c7(1,i+1),"inEdges"),l=wMe(r,c7(i-1,-1,-1),"outEdges"),f=vYt(r);mMe(r,f);for(var b=Number.POSITIVE_INFINITY,d,w=0,y=0;y<4;++w,++y){SYt(w%2?o:l,w%4>=2),f=CV(r);var k=pYt(r,f);k<b&&(y=0,d=nUt(f),b=k)}mMe(r,d)}function wMe(r,i,o){return Hh(i,function(l){return dYt(r,l,o)})}function SYt(r,i){var o=new zh;Mn(r,function(l){var f=l.graph().root,b=vMe(l,f,o,i);Mn(b.vs,function(d,w){l.node(d).order=w}),fYt(l,o,b.vs)})}function mMe(r,i){Mn(i,function(o){Mn(o,function(l,f){r.node(l).order=f})})}function AYt(r){var i=DYt(r);Mn(r.graph().dummyChains,function(o){for(var l=r.node(o),f=l.edgeObj,b=MYt(r,i,f.v,f.w),d=b.path,w=b.lca,y=0,k=d[y],E=!0;o!==f.w;){if(l=r.node(o),E){for(;(k=d[y])!==w&&r.node(k).maxRank<l.rank;)y++;k===w&&(E=!1)}if(!E){for(;y<d.length-1&&r.node(k=d[y+1]).minRank<=l.rank;)y++;k=d[y]}r.setParent(o,k),o=r.successors(o)[0]}})}function MYt(r,i,o,l){var f=[],b=[],d=Math.min(i[o].low,i[l].low),w=Math.max(i[o].lim,i[l].lim),y,k;y=o;do y=r.parent(y),f.push(y);while(y&&(i[y].low>d||w>i[y].lim));for(k=y,y=l;(y=r.parent(y))!==k;)b.push(y);return{path:f.concat(b.reverse()),lca:k}}function DYt(r){var i={},o=0;function l(f){var b=o;Mn(r.children(f),l),i[f]={low:b,lim:o++}}return Mn(r.children(),l),i}function LYt(r,i){var o={};function l(f,b){var d=0,w=0,y=f.length,k=EV(b);return Mn(b,function(E,T){var C=OYt(r,E),S=C?r.node(C).order:y;(C||E===k)&&(Mn(b.slice(w,T+1),function(L){Mn(r.predecessors(L),function(O){var B=r.node(O),N=B.order;(N<d||S<N)&&!(B.dummy&&r.node(L).dummy)&&yMe(o,O,L)})}),w=T+1,d=S)}),b}return MI(i,l),o}function IYt(r,i){var o={};function l(b,d,w,y,k){var E;Mn(c7(d,w),function(T){E=b[T],r.node(E).dummy&&Mn(r.predecessors(E),function(C){var S=r.node(C);S.dummy&&(S.order<y||S.order>k)&&yMe(o,C,E)})})}function f(b,d){var w=-1,y,k=0;return Mn(d,function(E,T){if(r.node(E).dummy==="border"){var C=r.predecessors(E);C.length&&(y=r.node(C[0]).order,l(d,k,T,w,y),k=T,w=y)}l(d,k,d.length,y,b.length)}),d}return MI(i,f),o}function OYt(r,i){if(r.node(i).dummy)return aoe(r.predecessors(i),function(o){return r.node(o).dummy})}function yMe(r,i,o){if(i>o){var l=i;i=o,o=l}var f=r[i];f||(r[i]=f={}),f[o]=!0}function NYt(r,i,o){if(i>o){var l=i;i=o,o=l}return Ks(r[i],o)}function PYt(r,i,o,l){var f={},b={},d={};return Mn(i,function(w){Mn(w,function(y,k){f[y]=y,b[y]=y,d[y]=k})}),Mn(i,function(w){var y=-1;Mn(w,function(k){var E=l(k);if(E.length){E=DI(E,function(O){return d[O]});for(var T=(E.length-1)/2,C=Math.floor(T),S=Math.ceil(T);C<=S;++C){var L=E[C];b[k]===k&&y<d[L]&&!NYt(o,k,L)&&(b[L]=k,b[k]=f[k]=f[L],y=d[L])}}})}),{root:f,align:b}}function BYt(r,i,o,l,f){var b={},d=FYt(r,i,o,f),w=f?"borderLeft":"borderRight";function y(T,C){for(var S=d.nodes(),L=S.pop(),O={};L;)O[L]?T(L):(O[L]=!0,S.push(L),S=S.concat(C(L))),L=S.pop()}function k(T){b[T]=d.inEdges(T).reduce(function(C,S){return Math.max(C,b[S.v]+d.edge(S))},0)}function E(T){var C=d.outEdges(T).reduce(function(L,O){return Math.min(L,b[O.w]-d.edge(O))},Number.POSITIVE_INFINITY),S=r.node(T);C!==Number.POSITIVE_INFINITY&&S.borderType!==w&&(b[T]=Math.max(b[T],C))}return y(k,d.predecessors.bind(d)),y(E,d.successors.bind(d)),Mn(l,function(T){b[T]=b[o[T]]}),b}function FYt(r,i,o,l){var f=new zh,b=r.graph(),d=zYt(b.nodesep,b.edgesep,l);return Mn(i,function(w){var y;Mn(w,function(k){var E=o[k];if(f.setNode(E),y){var T=o[y],C=f.edge(T,E);f.setEdge(T,E,Math.max(d(r,k,y),C||0))}y=k})}),f}function RYt(r,i){return uoe(Z3(i),function(o){var l=Number.NEGATIVE_INFINITY,f=Number.POSITIVE_INFINITY;return iqt(o,function(b,d){var w=GYt(r,d)/2;l=Math.max(b+w,l),f=Math.min(b-w,f)}),l-f})}function jYt(r,i){var o=Z3(i),l=SI(o),f=o7(o);Mn(["u","d"],function(b){Mn(["l","r"],function(d){var w=b+d,y=r[w],k;if(y!==i){var E=Z3(y);k=d==="l"?l-SI(E):f-o7(E),k&&(r[w]=_V(y,function(T){return T+k}))}})})}function $Yt(r,i){return _V(r.ul,function(o,l){if(i)return r[i.toLowerCase()][l];var f=DI(Hh(r,l));return(f[1]+f[2])/2})}function HYt(r){var i=CV(r),o=coe(LYt(r,i),IYt(r,i)),l={},f;Mn(["u","d"],function(d){f=d==="u"?i:Z3(i).reverse(),Mn(["l","r"],function(w){w==="r"&&(f=Hh(f,function(T){return Z3(T).reverse()}));var y=(d==="u"?r.predecessors:r.successors).bind(r),k=PYt(r,f,o,y),E=BYt(r,f,k.root,k.align,w==="r");w==="r"&&(E=_V(E,function(T){return-T})),l[d+w]=E})});var b=RYt(r,l);return jYt(l,b),$Yt(l,r.graph().align)}function zYt(r,i,o){return function(l,f,b){var d=l.node(f),w=l.node(b),y=0,k;if(y+=d.width/2,Ks(d,"labelpos"))switch(d.labelpos.toLowerCase()){case"l":k=-d.width/2;break;case"r":k=d.width/2;break}if(k&&(y+=o?k:-k),k=0,y+=(d.dummy?i:r)/2,y+=(w.dummy?i:r)/2,y+=w.width/2,Ks(w,"labelpos"))switch(w.labelpos.toLowerCase()){case"l":k=w.width/2;break;case"r":k=-w.width/2;break}return k&&(y+=o?k:-k),k=0,y}}function GYt(r,i){return r.node(i).width}function VYt(r){r=cMe(r),UYt(r),Mn(HYt(r),function(i,o){r.node(o).x=i})}function UYt(r){var i=CV(r),o=r.graph().ranksep,l=0;Mn(i,function(f){var b=o7(Hh(f,function(d){return r.node(d).height}));Mn(f,function(d){r.node(d).y=l+b/2}),l+=b+o})}function bC(r,i){var o=i&&i.debugTiming?tYt:nYt;o("layout",function(){var l=o("  buildLayoutGraph",function(){return nWt(r)});o("  runLayout",function(){qYt(l,o)}),o("  updateInputGraph",function(){YYt(r,l)})})}function qYt(r,i){i("    makeSpaceForEdgeLabels",function(){rWt(r)}),i("    removeSelfEdges",function(){fWt(r)}),i("    acyclic",function(){Wqt(r)}),i("    nestingGraph.run",function(){cYt(r)}),i("    rank",function(){SWt(cMe(r))}),i("    injectEdgeLabelProxies",function(){iWt(r)}),i("    removeEmptyRanks",function(){Jqt(r)}),i("    nestingGraph.cleanup",function(){hYt(r)}),i("    normalizeRanks",function(){Zqt(r)}),i("    assignRankMinMax",function(){sWt(r)}),i("    removeEdgeLabelProxies",function(){aWt(r)}),i("    normalize.run",function(){pWt(r)}),i("    parentDummyChains",function(){AYt(r)}),i("    addBorderSegments",function(){rYt(r)}),i("    order",function(){CYt(r)}),i("    insertSelfEdges",function(){dWt(r)}),i("    adjustCoordinateSystem",function(){iYt(r)}),i("    position",function(){VYt(r)}),i("    positionSelfEdges",function(){gWt(r)}),i("    removeBorderNodes",function(){hWt(r)}),i("    normalize.undo",function(){vWt(r)}),i("    fixupEdgeLabelCoords",function(){uWt(r)}),i("    undoCoordinateSystem",function(){sYt(r)}),i("    translateGraph",function(){oWt(r)}),i("    assignNodeIntersects",function(){cWt(r)}),i("    reversePoints",function(){lWt(r)}),i("    acyclic.undo",function(){Xqt(r)})}function YYt(r,i){Mn(r.nodes(),function(o){var l=r.node(o),f=i.node(o);l&&(l.x=f.x,l.y=f.y,i.children(o).length&&(l.width=f.width,l.height=f.height))}),Mn(r.edges(),function(o){var l=r.edge(o),f=i.edge(o);l.points=f.points,Ks(f,"x")&&(l.x=f.x,l.y=f.y)}),r.graph().width=i.graph().width,r.graph().height=i.graph().height}var WYt=["nodesep","edgesep","ranksep","marginx","marginy"],KYt={ranksep:50,edgesep:20,nodesep:50,rankdir:"tb"},XYt=["acyclicer","ranker","rankdir","align"],QYt=["width","height"],ZYt={width:0,height:0},JYt=["minlen","weight","width","height","labeloffset"],eWt={minlen:1,weight:1,width:0,height:0,labeloffset:10,labelpos:"r"},tWt=["labelpos"];function nWt(r){var i=new zh({multigraph:!0,compound:!0}),o=boe(r.graph());return i.setGraph(coe({},KYt,poe(o,WYt),AI(o,XYt))),Mn(r.nodes(),function(l){var f=boe(r.node(l));i.setNode(l,dC(poe(f,QYt),ZYt)),i.setParent(l,r.parent(l))}),Mn(r.edges(),function(l){var f=boe(r.edge(l));i.setEdge(l,coe({},eWt,poe(f,JYt),AI(f,tWt)))}),i}function rWt(r){var i=r.graph();i.ranksep/=2,Mn(r.edges(),function(o){var l=r.edge(o);l.minlen*=2,l.labelpos.toLowerCase()!=="c"&&(i.rankdir==="TB"||i.rankdir==="BT"?l.width+=l.labeloffset:l.height+=l.labeloffset)})}function iWt(r){Mn(r.edges(),function(i){var o=r.edge(i);if(o.width&&o.height){var l=r.node(i.v),f=r.node(i.w),b={rank:(f.rank-l.rank)/2+l.rank,e:i};pC(r,"edge-proxy",b,"_ep")}})}function sWt(r){var i=0;Mn(r.nodes(),function(o){var l=r.node(o);l.borderTop&&(l.minRank=r.node(l.borderTop).rank,l.maxRank=r.node(l.borderBottom).rank,i=o7(i,l.maxRank))}),r.graph().maxRank=i}function aWt(r){Mn(r.nodes(),function(i){var o=r.node(i);o.dummy==="edge-proxy"&&(r.edge(o.e).labelRank=o.rank,r.removeNode(i))})}function oWt(r){var i=Number.POSITIVE_INFINITY,o=0,l=Number.POSITIVE_INFINITY,f=0,b=r.graph(),d=b.marginx||0,w=b.marginy||0;function y(k){var E=k.x,T=k.y,C=k.width,S=k.height;i=Math.min(i,E-C/2),o=Math.max(o,E+C/2),l=Math.min(l,T-S/2),f=Math.max(f,T+S/2)}Mn(r.nodes(),function(k){y(r.node(k))}),Mn(r.edges(),function(k){var E=r.edge(k);Ks(E,"x")&&y(E)}),i-=d,l-=w,Mn(r.nodes(),function(k){var E=r.node(k);E.x-=i,E.y-=l}),Mn(r.edges(),function(k){var E=r.edge(k);Mn(E.points,function(T){T.x-=i,T.y-=l}),Ks(E,"x")&&(E.x-=i),Ks(E,"y")&&(E.y-=l)}),b.width=o-i+d,b.height=f-l+w}function cWt(r){Mn(r.edges(),function(i){var o=r.edge(i),l=r.node(i.v),f=r.node(i.w),b,d;o.points?(b=o.points[0],d=o.points[o.points.length-1]):(o.points=[],b=f,d=l),o.points.unshift(uMe(l,b)),o.points.push(uMe(f,d))})}function uWt(r){Mn(r.edges(),function(i){var o=r.edge(i);if(Ks(o,"x"))switch((o.labelpos==="l"||o.labelpos==="r")&&(o.width-=o.labeloffset),o.labelpos){case"l":o.x-=o.width/2+o.labeloffset;break;case"r":o.x+=o.width/2+o.labeloffset;break}})}function lWt(r){Mn(r.edges(),function(i){var o=r.edge(i);o.reversed&&o.points.reverse()})}function hWt(r){Mn(r.nodes(),function(i){if(r.children(i).length){var o=r.node(i),l=r.node(o.borderTop),f=r.node(o.borderBottom),b=r.node(EV(o.borderLeft)),d=r.node(EV(o.borderRight));o.width=Math.abs(d.x-b.x),o.height=Math.abs(f.y-l.y),o.x=b.x+o.width/2,o.y=l.y+o.height/2}}),Mn(r.nodes(),function(i){r.node(i).dummy==="border"&&r.removeNode(i)})}function fWt(r){Mn(r.edges(),function(i){if(i.v===i.w){var o=r.node(i.v);o.selfEdges||(o.selfEdges=[]),o.selfEdges.push({e:i,label:r.edge(i)}),r.removeEdge(i)}})}function dWt(r){var i=CV(r);Mn(i,function(o){var l=0;Mn(o,function(f,b){var d=r.node(f);d.order=b+l,Mn(d.selfEdges,function(w){pC(r,"selfedge",{width:w.label.width,height:w.label.height,rank:d.rank,order:b+ ++l,e:w.e,label:w.label},"_se")}),delete d.selfEdges})})}function gWt(r){Mn(r.nodes(),function(i){var o=r.node(i);if(o.dummy==="selfedge"){var l=r.node(o.e.v),f=l.x+l.width/2,b=l.y,d=o.x-f,w=l.height/2;r.setEdge(o.e,o.label),r.removeNode(i),o.label.points=[{x:f+2*d/3,y:b-w},{x:f+5*d/6,y:b-w},{x:f+d,y:b},{x:f+5*d/6,y:b+w},{x:f+2*d/3,y:b+w}],o.label.x=o.x,o.label.y=o.y}})}function poe(r,i){return _V(AI(r,i),Number)}function boe(r){var i={};return Mn(r,function(o,l){i[l.toLowerCase()]=o}),i}function pWt(r){r.graph().dummyChains=[],Mn(r.edges(),function(i){bWt(r,i)})}function bWt(r,i){var o=i.v,l=r.node(o).rank,f=i.w,b=r.node(f).rank,d=i.name,w=r.edge(i),y=w.labelRank;if(b!==l+1){r.removeEdge(i);var k,E,T;for(T=0,++l;l<b;++T,++l)w.points=[],E={width:0,height:0,edgeLabel:w,edgeObj:i,rank:l},k=pC(r,"edge",E,"_d"),l===y&&(E.width=w.width,E.height=w.height,E.dummy="edge-label",E.labelpos=w.labelpos),r.setEdge(o,k,{weight:w.weight},d),T===0&&r.graph().dummyChains.push(k),o=k;r.setEdge(o,f,{weight:w.weight},d)}}function vWt(r){Mn(r.graph().dummyChains,function(i){var o=r.node(i),l=o.edgeLabel,f;for(r.setEdge(o.edgeObj,l);o.dummy;)f=r.successors(i)[0],r.removeNode(i),l.points.push({x:o.x,y:o.y}),o.dummy==="edge-label"&&(l.x=o.x,l.y=o.y,l.width=o.width,l.height=o.height),i=f,o=r.node(i)})}function voe(r){var i={};function o(l){var f=r.node(l);if(Ks(i,l))return f.rank;i[l]=!0;var b=SI(Hh(r.outEdges(l),function(d){return o(d.w)-r.edge(d).minlen}));return(b===Number.POSITIVE_INFINITY||b===void 0||b===null)&&(b=0),f.rank=b}Mn(r.sources(),o)}function II(r,i){return r.node(i.w).rank-r.node(i.v).rank-r.edge(i).minlen}function kMe(r){var i=new zh({directed:!1}),o=r.nodes()[0],l=r.nodeCount();i.setNode(o,{});for(var f,b;wWt(i,r)<l;)f=mWt(i,r),b=i.hasNode(f.v)?II(r,f):-II(r,f),yWt(i,r,b);return i}function wWt(r,i){function o(l){Mn(i.nodeEdges(l),function(f){var b=f.v,d=l===b?f.w:b;!r.hasNode(d)&&!II(i,f)&&(r.setNode(d,{}),r.setEdge(l,d,{}),o(d))})}return Mn(r.nodes(),o),r.nodeCount()}function mWt(r,i){return uoe(i.edges(),function(o){if(r.hasNode(o.v)!==r.hasNode(o.w))return II(i,o)})}function yWt(r,i,o){Mn(r.nodes(),function(l){i.node(l).rank+=o})}function kWt(){}kWt.prototype=new Error;function xMe(r,i,o){$h(i)||(i=[i]);var l=(r.isDirected()?r.successors:r.neighbors).bind(r),f=[],b={};return Mn(i,function(d){if(!r.hasNode(d))throw new Error("Graph does not have node: "+d);EMe(r,d,o==="post",b,l,f)}),f}function EMe(r,i,o,l,f,b){Ks(l,i)||(l[i]=!0,o||b.push(i),Mn(f(i),function(d){EMe(r,d,o,l,f,b)}),o&&b.push(i))}function xWt(r,i){return xMe(r,i,"post")}function EWt(r,i){return xMe(r,i,"pre")}l7.initLowLimValues=moe,l7.initCutValues=woe,l7.calcCutValue=_Me,l7.leaveEdge=CMe,l7.enterEdge=SMe,l7.exchangeEdges=AMe;function l7(r){r=Qqt(r),voe(r);var i=kMe(r);moe(i),woe(i,r);for(var o,l;o=CMe(i);)l=SMe(i,r,o),AMe(i,r,o,l)}function woe(r,i){var o=xWt(r,r.nodes());o=o.slice(0,o.length-1),Mn(o,function(l){_Wt(r,i,l)})}function _Wt(r,i,o){var l=r.node(o),f=l.parent;r.edge(o,f).cutvalue=_Me(r,i,o)}function _Me(r,i,o){var l=r.node(o),f=l.parent,b=!0,d=i.edge(o,f),w=0;return d||(b=!1,d=i.edge(f,o)),w=d.weight,Mn(i.nodeEdges(o),function(y){var k=y.v===o,E=k?y.w:y.v;if(E!==f){var T=k===b,C=i.edge(y).weight;if(w+=T?C:-C,CWt(r,o,E)){var S=r.edge(o,E).cutvalue;w+=T?-S:S}}}),w}function moe(r,i){arguments.length<2&&(i=r.nodes()[0]),TMe(r,{},1,i)}function TMe(r,i,o,l,f){var b=o,d=r.node(l);return i[l]=!0,Mn(r.neighbors(l),function(w){Ks(i,w)||(o=TMe(r,i,o,w,l))}),d.low=b,d.lim=o++,f?d.parent=f:delete d.parent,o}function CMe(r){return aoe(r.edges(),function(i){return r.edge(i).cutvalue<0})}function SMe(r,i,o){var l=o.v,f=o.w;i.hasEdge(l,f)||(l=o.w,f=o.v);var b=r.node(l),d=r.node(f),w=b,y=!1;b.lim>d.lim&&(w=d,y=!0);var k=Q3(i.edges(),function(E){return y===MMe(r,r.node(E.v),w)&&y!==MMe(r,r.node(E.w),w)});return uoe(k,function(E){return II(i,E)})}function AMe(r,i,o,l){var f=o.v,b=o.w;r.removeEdge(f,b),r.setEdge(l.v,l.w,{}),moe(r),woe(r,i),TWt(r,i)}function TWt(r,i){var o=aoe(r.nodes(),function(f){return!i.node(f).parent}),l=EWt(r,o);l=l.slice(1),Mn(l,function(f){var b=r.node(f).parent,d=i.edge(f,b),w=!1;d||(d=i.edge(b,f),w=!0),i.node(f).rank=i.node(b).rank+(w?d.minlen:-d.minlen)})}function CWt(r,i,o){return r.hasEdge(i,o)}function MMe(r,i,o){return o.low<=i.lim&&i.lim<=o.lim}function SWt(r){switch(r.graph().ranker){case"network-simplex":DMe(r);break;case"tight-tree":MWt(r);break;case"longest-path":AWt(r);break;default:DMe(r)}}var AWt=voe;function MWt(r){voe(r),kMe(r)}function DMe(r){l7(r)}let LMe=0;const DWt=function(r,i,o,l,f){const b=function(F){switch(F){case f.db.relationType.AGGREGATION:return"aggregation";case f.db.relationType.EXTENSION:return"extension";case f.db.relationType.COMPOSITION:return"composition";case f.db.relationType.DEPENDENCY:return"dependency";case f.db.relationType.LOLLIPOP:return"lollipop"}};i.points=i.points.filter(F=>!Number.isNaN(F.y));const d=i.points,w=V8().x(function(F){return F.x}).y(function(F){return F.y}).curve(UT),y=r.append("path").attr("d",w(d)).attr("id","edge"+LMe).attr("class","relation");let k="";l.arrowMarkerAbsolute&&(k=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,k=k.replace(/\(/g,"\\("),k=k.replace(/\)/g,"\\)")),o.relation.lineType==1&&y.attr("class","relation dashed-line"),o.relation.lineType==10&&y.attr("class","relation dotted-line"),o.relation.type1!=="none"&&y.attr("marker-start","url("+k+"#"+b(o.relation.type1)+"Start)"),o.relation.type2!=="none"&&y.attr("marker-end","url("+k+"#"+b(o.relation.type2)+"End)");let E,T;const C=i.points.length;let S=Fs.calcLabelPosition(i.points);E=S.x,T=S.y;let L,O,B,N;if(C%2!==0&&C>1){let F=Fs.calcCardinalityPosition(o.relation.type1!=="none",i.points,i.points[0]),R=Fs.calcCardinalityPosition(o.relation.type2!=="none",i.points,i.points[C-1]);Se.debug("cardinality_1_point "+JSON.stringify(F)),Se.debug("cardinality_2_point "+JSON.stringify(R)),L=F.x,O=F.y,B=R.x,N=R.y}if(o.title!==void 0){const F=r.append("g").attr("class","classLabel"),R=F.append("text").attr("class","label").attr("x",E).attr("y",T).attr("fill","red").attr("text-anchor","middle").text(o.title);window.label=R;const q=R.node().getBBox();F.insert("rect",":first-child").attr("class","box").attr("x",q.x-l.padding/2).attr("y",q.y-l.padding/2).attr("width",q.width+l.padding).attr("height",q.height+l.padding)}Se.info("Rendering relation "+JSON.stringify(o)),o.relationTitle1!==void 0&&o.relationTitle1!=="none"&&r.append("g").attr("class","cardinality").append("text").attr("class","type1").attr("x",L).attr("y",O).attr("fill","black").attr("font-size","6").text(o.relationTitle1),o.relationTitle2!==void 0&&o.relationTitle2!=="none"&&r.append("g").attr("class","cardinality").append("text").attr("class","type2").attr("x",B).attr("y",N).attr("fill","black").attr("font-size","6").text(o.relationTitle2),LMe++},LWt=function(r,i,o,l){Se.debug("Rendering class ",i,o);const f=i.id,b={id:f,label:i.id,width:0,height:0},d=r.append("g").attr("id",l.db.lookUpDomId(f)).attr("class","classGroup");let w;i.link?w=d.append("svg:a").attr("xlink:href",i.link).attr("target",i.linkTarget).append("text").attr("y",o.textHeight+o.padding).attr("x",0):w=d.append("text").attr("y",o.textHeight+o.padding).attr("x",0);let y=!0;i.annotations.forEach(function(X){const te=w.append("tspan").text("\xAB"+X+"\xBB");y||te.attr("dy",o.textHeight),y=!1});let k=i.id;i.type!==void 0&&i.type!==""&&(k+="<"+i.type+">");const E=w.append("tspan").text(k).attr("class","title");y||E.attr("dy",o.textHeight);const T=w.node().getBBox().height,C=d.append("line").attr("x1",0).attr("y1",o.padding+T+o.dividerMargin/2).attr("y2",o.padding+T+o.dividerMargin/2),S=d.append("text").attr("x",o.padding).attr("y",T+o.dividerMargin+o.textHeight).attr("fill","white").attr("class","classText");y=!0,i.members.forEach(function(X){IMe(S,X,y,o),y=!1});const L=S.node().getBBox(),O=d.append("line").attr("x1",0).attr("y1",o.padding+T+o.dividerMargin+L.height).attr("y2",o.padding+T+o.dividerMargin+L.height),B=d.append("text").attr("x",o.padding).attr("y",T+2*o.dividerMargin+L.height+o.textHeight).attr("fill","white").attr("class","classText");y=!0,i.methods.forEach(function(X){IMe(B,X,y,o),y=!1});const N=d.node().getBBox();var F=" ";i.cssClasses.length>0&&(F=F+i.cssClasses.join(" "));const q=d.insert("rect",":first-child").attr("x",0).attr("y",0).attr("width",N.width+2*o.padding).attr("height",N.height+o.padding+.5*o.dividerMargin).attr("class",F).node().getBBox().width;return w.node().childNodes.forEach(function(X){X.setAttribute("x",(q-X.getBBox().width)/2)}),i.tooltip&&w.insert("title").text(i.tooltip),C.attr("x2",q),O.attr("x2",q),b.width=q,b.height=N.height+o.padding+.5*o.dividerMargin,b},IWt=function(r,i,o,l){Se.debug("Rendering note ",i,o);const f=i.id,b={id:f,text:i.text,width:0,height:0},d=r.append("g").attr("id",f).attr("class","classGroup");let w=d.append("text").attr("y",o.textHeight+o.padding).attr("x",0);const y=JSON.parse(`"${i.text}"`).split(`
`);y.forEach(function(C){Se.debug(`Adding line: ${C}`),w.append("tspan").text(C).attr("class","title").attr("dy",o.textHeight)});const k=d.node().getBBox(),T=d.insert("rect",":first-child").attr("x",0).attr("y",0).attr("width",k.width+2*o.padding).attr("height",k.height+y.length*o.textHeight+o.padding+.5*o.dividerMargin).node().getBBox().width;return w.node().childNodes.forEach(function(C){C.setAttribute("x",(T-C.getBBox().width)/2)}),b.width=T,b.height=k.height+y.length*o.textHeight+o.padding+.5*o.dividerMargin,b},SV=function(r){const i=/^([#+~-])?(\w+)(~\w+~|\[])?\s+(\w+) *([$*])?$/,o=/^([#+|~-])?(\w+) *\( *(.*)\) *([$*])? *(\w*[[\]|~]*\s*\w*~?)$/;let l=r.match(i),f=r.match(o);return l&&!f?OWt(l):f?NWt(f):PWt(r)},OWt=function(r){let i="",o="";try{let l=r[1]?r[1].trim():"",f=r[2]?r[2].trim():"",b=r[3]?o6(r[3].trim()):"",d=r[4]?r[4].trim():"",w=r[5]?r[5].trim():"";o=l+f+b+" "+d,i=yoe(w)}catch{o=r}return{displayText:o,cssStyle:i}},NWt=function(r){let i="",o="";try{let l=r[1]?r[1].trim():"",f=r[2]?r[2].trim():"",b=r[3]?o6(r[3].trim()):"",d=r[4]?r[4].trim():"",w=r[5]?" : "+o6(r[5]).trim():"";o=l+f+"("+b+")"+w,i=yoe(d)}catch{o=r}return{displayText:o,cssStyle:i}},PWt=function(r){let i="",o="",l="",f=r.indexOf("("),b=r.indexOf(")");if(f>1&&b>f&&b<=r.length){let d="",w="",y=r.substring(0,1);y.match(/\w/)?w=r.substring(0,f).trim():(y.match(/[#+~-]/)&&(d=y),w=r.substring(1,f).trim());const k=r.substring(f+1,b);r.substring(b+1,1),o=yoe(r.substring(b+1,b+2)),i=d+w+"("+o6(k.trim())+")",b<r.length&&(l=r.substring(b+2).trim(),l!==""&&(l=" : "+o6(l),i+=l))}else i=o6(r);return{displayText:i,cssStyle:o}},IMe=function(r,i,o,l){let f=SV(i);const b=r.append("tspan").attr("x",l.padding).text(f.displayText);f.cssStyle!==""&&b.attr("style",f.cssStyle),o||b.attr("dy",l.textHeight)},yoe=function(r){switch(r){case"*":return"font-style:italic;";case"$":return"text-decoration:underline;";default:return""}},koe={drawClass:LWt,drawEdge:DWt,drawNote:IWt,parseMember:SV};let AV={};const MV=20,OI=function(r){const i=Object.entries(AV).find(o=>o[1].label===r);if(i)return i[0]},BWt=function(r){r.append("defs").append("marker").attr("id","extensionStart").attr("class","extension").attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 1,7 L18,13 V 1 Z"),r.append("defs").append("marker").attr("id","extensionEnd").attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 1,1 V 13 L18,7 Z"),r.append("defs").append("marker").attr("id","compositionStart").attr("class","extension").attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id","compositionEnd").attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id","aggregationStart").attr("class","extension").attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id","aggregationEnd").attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id","dependencyStart").attr("class","extension").attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 5,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id","dependencyEnd").attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L14,7 L9,1 Z")},FWt={draw:function(r,i,o,l){const f=ht().class;AV={},Se.info("Rendering diagram "+r);const b=ht().securityLevel;let d;b==="sandbox"&&(d=Cn("#i"+i));const w=Cn(b==="sandbox"?d.nodes()[0].contentDocument.body:"body"),y=w.select(`[id='${i}']`);BWt(y);const k=new zh({multigraph:!0});k.setGraph({isMultiGraph:!0}),k.setDefaultEdgeLabel(function(){return{}});const E=l.db.getClasses(),T=Object.keys(E);for(const F of T){const R=E[F],q=koe.drawClass(y,R,f,l);AV[q.id]=q,k.setNode(q.id,q),Se.info("Org height: "+q.height)}l.db.getRelations().forEach(function(F){Se.info("tjoho"+OI(F.id1)+OI(F.id2)+JSON.stringify(F)),k.setEdge(OI(F.id1),OI(F.id2),{relation:F},F.title||"DEFAULT")}),l.db.getNotes().forEach(function(F){Se.debug(`Adding note: ${JSON.stringify(F)}`);const R=koe.drawNote(y,F,f,l);AV[R.id]=R,k.setNode(R.id,R),F.class&&F.class in E&&k.setEdge(F.id,OI(F.class),{relation:{id1:F.id,id2:F.class,relation:{type1:"none",type2:"none",lineType:10}}},"DEFAULT")}),bC(k),k.nodes().forEach(function(F){F!==void 0&&k.node(F)!==void 0&&(Se.debug("Node "+F+": "+JSON.stringify(k.node(F))),w.select("#"+(l.db.lookUpDomId(F)||F)).attr("transform","translate("+(k.node(F).x-k.node(F).width/2)+","+(k.node(F).y-k.node(F).height/2)+" )"))}),k.edges().forEach(function(F){F!==void 0&&k.edge(F)!==void 0&&(Se.debug("Edge "+F.v+" -> "+F.w+": "+JSON.stringify(k.edge(F))),koe.drawEdge(y,k.edge(F),k.edge(F).relation,f,l))});const L=y.node().getBBox(),O=L.width+MV*2,B=L.height+MV*2;Iv(y,B,O,f.useMaxWidth);const N=`${L.x-MV} ${L.y-MV} ${O} ${B}`;Se.debug(`viewBox ${N}`),y.attr("viewBox",N)}};function J3(r){var i={options:{directed:r.isDirected(),multigraph:r.isMultigraph(),compound:r.isCompound()},nodes:RWt(r),edges:jWt(r)};return oh(r.graph())||(i.value=JVt(r.graph())),i}function RWt(r){return Hh(r.nodes(),function(i){var o=r.node(i),l=r.parent(i),f={v:i};return oh(o)||(f.value=o),oh(l)||(f.parent=l),f})}function jWt(r){return Hh(r.edges(),function(i){var o=r.edge(i),l={v:i.v,w:i.w};return oh(i.name)||(l.name=i.name),oh(o)||(l.value=o),l})}const $Wt=(r,i,o,l)=>{i.forEach(f=>{HWt[f](r,o,l)})},HWt={extension:(r,i,o)=>{Se.trace("Making markers for ",o),r.append("defs").append("marker").attr("id",i+"-extensionStart").attr("class","marker extension "+i).attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 1,7 L18,13 V 1 Z"),r.append("defs").append("marker").attr("id",i+"-extensionEnd").attr("class","marker extension "+i).attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 1,1 V 13 L18,7 Z")},composition:(r,i)=>{r.append("defs").append("marker").attr("id",i+"-compositionStart").attr("class","marker composition "+i).attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id",i+"-compositionEnd").attr("class","marker composition "+i).attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z")},aggregation:(r,i)=>{r.append("defs").append("marker").attr("id",i+"-aggregationStart").attr("class","marker aggregation "+i).attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id",i+"-aggregationEnd").attr("class","marker aggregation "+i).attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z")},dependency:(r,i)=>{r.append("defs").append("marker").attr("id",i+"-dependencyStart").attr("class","marker dependency "+i).attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 5,7 L9,13 L1,7 L9,1 Z"),r.append("defs").append("marker").attr("id",i+"-dependencyEnd").attr("class","marker dependency "+i).attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L14,7 L9,1 Z")},lollipop:(r,i)=>{r.append("defs").append("marker").attr("id",i+"-lollipopStart").attr("class","marker lollipop "+i).attr("refX",0).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("circle").attr("stroke","black").attr("fill","white").attr("cx",6).attr("cy",7).attr("r",6)},point:(r,i)=>{r.append("marker").attr("id",i+"-pointEnd").attr("class","marker "+i).attr("viewBox","0 0 12 20").attr("refX",10).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",12).attr("markerHeight",12).attr("orient","auto").append("path").attr("d","M 0 0 L 10 5 L 0 10 z").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0"),r.append("marker").attr("id",i+"-pointStart").attr("class","marker "+i).attr("viewBox","0 0 10 10").attr("refX",0).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",12).attr("markerHeight",12).attr("orient","auto").append("path").attr("d","M 0 5 L 10 10 L 10 0 z").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0")},circle:(r,i)=>{r.append("marker").attr("id",i+"-circleEnd").attr("class","marker "+i).attr("viewBox","0 0 10 10").attr("refX",11).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("circle").attr("cx","5").attr("cy","5").attr("r","5").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0"),r.append("marker").attr("id",i+"-circleStart").attr("class","marker "+i).attr("viewBox","0 0 10 10").attr("refX",-1).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("circle").attr("cx","5").attr("cy","5").attr("r","5").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0")},cross:(r,i)=>{r.append("marker").attr("id",i+"-crossEnd").attr("class","marker cross "+i).attr("viewBox","0 0 11 11").attr("refX",12).attr("refY",5.2).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("path").attr("d","M 1,1 l 9,9 M 10,1 l -9,9").attr("class","arrowMarkerPath").style("stroke-width",2).style("stroke-dasharray","1,0"),r.append("marker").attr("id",i+"-crossStart").attr("class","marker cross "+i).attr("viewBox","0 0 11 11").attr("refX",-1).attr("refY",5.2).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("path").attr("d","M 1,1 l 9,9 M 10,1 l -9,9").attr("class","arrowMarkerPath").style("stroke-width",2).style("stroke-dasharray","1,0")},barb:(r,i)=>{r.append("defs").append("marker").attr("id",i+"-barbEnd").attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",14).attr("markerUnits","strokeWidth").attr("orient","auto").append("path").attr("d","M 19,7 L9,13 L14,7 L9,1 Z")}},OMe=$Wt;function zWt(r,i){i&&r.attr("style",i)}function GWt(r){const i=Cn(document.createElementNS("http://www.w3.org/2000/svg","foreignObject")),o=i.append("xhtml:div"),l=r.label,f=r.isNode?"nodeLabel":"edgeLabel";return o.html('<span class="'+f+'" '+(r.labelStyle?'style="'+r.labelStyle+'"':"")+">"+l+"</span>"),zWt(o,r.labelStyle),o.style("display","inline-block"),o.style("white-space","nowrap"),o.attr("xmlns","http://www.w3.org/1999/xhtml"),i.node()}const Q0=(r,i,o,l)=>{let f=r||"";if(typeof f=="object"&&(f=f[0]),cf(ht().flowchart.htmlLabels)){f=f.replace(/\\n|\n/g,"<br />"),Se.info("vertexText"+f);const b={isNode:l,label:Pce(f).replace(/fa[blrs]?:fa-[\w-]+/g,w=>`<i class='${w.replace(":"," ")}'></i>`),labelStyle:i.replace("fill:","color:")};return GWt(b)}else{const b=document.createElementNS("http://www.w3.org/2000/svg","text");b.setAttribute("style",i.replace("color:","fill:"));let d=[];typeof f=="string"?d=f.split(/\\n|\n|<br\s*\/?>/gi):Array.isArray(f)?d=f:d=[];for(const w of d){const y=document.createElementNS("http://www.w3.org/2000/svg","tspan");y.setAttributeNS("http://www.w3.org/XML/1998/namespace","xml:space","preserve"),y.setAttribute("dy","1em"),y.setAttribute("x","0"),o?y.setAttribute("class","title-row"):y.setAttribute("class","row"),y.textContent=w.trim(),b.appendChild(y)}return b}},ld=(r,i,o,l)=>{let f;o?f=o:f="node default";const b=r.insert("g").attr("class",f).attr("id",i.domId||i.id),d=b.insert("g").attr("class","label").attr("style",i.labelStyle);let w;i.labelText===void 0?w="":w=typeof i.labelText=="string"?i.labelText:i.labelText[0];const y=d.node().appendChild(Q0(up(Pce(w),ht()),i.labelStyle,!1,l));let k=y.getBBox();if(cf(ht().flowchart.htmlLabels)){const T=y.children[0],C=Cn(y);k=T.getBoundingClientRect(),C.attr("width",k.width),C.attr("height",k.height)}const E=i.padding/2;return d.attr("transform","translate("+-k.width/2+", "+-k.height/2+")"),{shapeSvg:b,bbox:k,halfPadding:E,label:d}},ch=(r,i)=>{const o=i.node().getBBox();r.width=o.width,r.height=o.height};function e4(r,i,o,l){return r.insert("polygon",":first-child").attr("points",l.map(function(f){return f.x+","+f.y}).join(" ")).attr("class","label-container").attr("transform","translate("+-i/2+","+o/2+")")}let qa={},jv={},NMe={};const VWt=()=>{jv={},NMe={},qa={}},DV=(r,i)=>(Se.trace("In isDecendant",i," ",r," = ",jv[i].includes(r)),!!jv[i].includes(r)),UWt=(r,i)=>(Se.info("Decendants of ",i," is ",jv[i]),Se.info("Edge is ",r),r.v===i||r.w===i?!1:jv[i]?jv[i].includes(r.v)||DV(r.v,i)||DV(r.w,i)||jv[i].includes(r.w):(Se.debug("Tilt, ",i,",not in decendants"),!1)),PMe=(r,i,o,l)=>{Se.warn("Copying children of ",r,"root",l,"data",i.node(r),l);const f=i.children(r)||[];r!==l&&f.push(r),Se.warn("Copying (nodes) clusterId",r,"nodes",f),f.forEach(b=>{if(i.children(b).length>0)PMe(b,i,o,l);else{const d=i.node(b);Se.info("cp ",b," to ",l," with parent ",r),o.setNode(b,d),l!==i.parent(b)&&(Se.warn("Setting parent",b,i.parent(b)),o.setParent(b,i.parent(b))),r!==l&&b!==r?(Se.debug("Setting parent",b,r),o.setParent(b,r)):(Se.info("In copy ",r,"root",l,"data",i.node(r),l),Se.debug("Not Setting parent for node=",b,"cluster!==rootId",r!==l,"node!==clusterId",b!==r));const w=i.edges(b);Se.debug("Copying Edges",w),w.forEach(y=>{Se.info("Edge",y);const k=i.edge(y.v,y.w,y.name);Se.info("Edge data",k,l);try{UWt(y,l)?(Se.info("Copying as ",y.v,y.w,k,y.name),o.setEdge(y.v,y.w,k,y.name),Se.info("newGraph edges ",o.edges(),o.edge(o.edges()[0]))):Se.info("Skipping copy of edge ",y.v,"-->",y.w," rootId: ",l," clusterId:",r)}catch(E){Se.error(E)}})}Se.debug("Removing node",b),i.removeNode(b)})},BMe=(r,i)=>{const o=i.children(r);let l=[...o];for(const f of o)NMe[f]=r,l=[...l,...BMe(f,i)];return l},NI=(r,i)=>{Se.trace("Searching",r);const o=i.children(r);if(Se.trace("Searching children of id ",r,o),o.length<1)return Se.trace("This is a valid node",r),r;for(const l of o){const f=NI(l,i);if(f)return Se.trace("Found replacement for",r," => ",f),f}},LV=r=>!qa[r]||!qa[r].externalConnections?r:qa[r]?qa[r].id:r,qWt=(r,i)=>{if(!r||i>10){Se.debug("Opting out, no graph ");return}else Se.debug("Opting in, graph ");r.nodes().forEach(function(o){r.children(o).length>0&&(Se.warn("Cluster identified",o," Replacement id in edges: ",NI(o,r)),jv[o]=BMe(o,r),qa[o]={id:NI(o,r),clusterData:r.node(o)})}),r.nodes().forEach(function(o){const l=r.children(o),f=r.edges();l.length>0?(Se.debug("Cluster identified",o,jv),f.forEach(b=>{if(b.v!==o&&b.w!==o){const d=DV(b.v,o),w=DV(b.w,o);d^w&&(Se.warn("Edge: ",b," leaves cluster ",o),Se.warn("Decendants of XXX ",o,": ",jv[o]),qa[o].externalConnections=!0)}})):Se.debug("Not a cluster ",o,jv)}),r.edges().forEach(function(o){const l=r.edge(o);Se.warn("Edge "+o.v+" -> "+o.w+": "+JSON.stringify(o)),Se.warn("Edge "+o.v+" -> "+o.w+": "+JSON.stringify(r.edge(o)));let f=o.v,b=o.w;if(Se.warn("Fix XXX",qa,"ids:",o.v,o.w,"Translating: ",qa[o.v]," --- ",qa[o.w]),qa[o.v]&&qa[o.w]&&qa[o.v]===qa[o.w]){Se.warn("Fixing and trixing link to self - removing XXX",o.v,o.w,o.name),Se.warn("Fixing and trixing - removing XXX",o.v,o.w,o.name),f=LV(o.v),b=LV(o.w),r.removeEdge(o.v,o.w,o.name);const d=o.w+"---"+o.v;r.setNode(d,{domId:d,id:d,labelStyle:"",labelText:l.label,padding:0,shape:"labelRect",style:""});const w=JSON.parse(JSON.stringify(l)),y=JSON.parse(JSON.stringify(l));w.label="",w.arrowTypeEnd="none",y.label="",w.fromCluster=o.v,y.toCluster=o.v,r.setEdge(f,d,w,o.name+"-cyclic-special"),r.setEdge(d,b,y,o.name+"-cyclic-special")}else(qa[o.v]||qa[o.w])&&(Se.warn("Fixing and trixing - removing XXX",o.v,o.w,o.name),f=LV(o.v),b=LV(o.w),r.removeEdge(o.v,o.w,o.name),f!==o.v&&(l.fromCluster=o.v),b!==o.w&&(l.toCluster=o.w),Se.warn("Fix Replacing with XXX",f,b,o.name),r.setEdge(f,b,l,o.name))}),Se.warn("Adjusted Graph",J3(r)),FMe(r,0),Se.trace(qa)},FMe=(r,i)=>{if(Se.warn("extractor - ",i,J3(r),r.children("D")),i>10){Se.error("Bailing out");return}let o=r.nodes(),l=!1;for(const f of o){const b=r.children(f);l=l||b.length>0}if(!l){Se.debug("Done, no node has children",r.nodes());return}Se.debug("Nodes = ",o,i);for(const f of o)if(Se.debug("Extracting node",f,qa,qa[f]&&!qa[f].externalConnections,!r.parent(f),r.node(f),r.children("D")," Depth ",i),!qa[f])Se.debug("Not a cluster",f,i);else if(!qa[f].externalConnections&&r.children(f)&&r.children(f).length>0){Se.warn("Cluster without external connections, without a parent and with children",f,i);let d=r.graph().rankdir==="TB"?"LR":"TB";qa[f]&&qa[f].clusterData&&qa[f].clusterData.dir&&(d=qa[f].clusterData.dir,Se.warn("Fixing dir",qa[f].clusterData.dir,d));const w=new zh({multigraph:!0,compound:!0}).setGraph({rankdir:d,nodesep:50,ranksep:50,marginx:8,marginy:8}).setDefaultEdgeLabel(function(){return{}});Se.warn("Old graph before copy",J3(r)),PMe(f,r,w,f),r.setNode(f,{clusterNode:!0,id:f,clusterData:qa[f].clusterData,labelText:qa[f].labelText,graph:w}),Se.warn("New graph after copy node: (",f,")",J3(w)),Se.debug("Old graph after copy",J3(r))}else Se.warn("Cluster ** ",f," **not meeting the criteria !externalConnections:",!qa[f].externalConnections," no parent: ",!r.parent(f)," children ",r.children(f)&&r.children(f).length>0,r.children("D"),i),Se.debug(qa);o=r.nodes(),Se.warn("New list of nodes",o);for(const f of o){const b=r.node(f);Se.warn(" Now next level",f,b),b.clusterNode&&FMe(b.graph,i+1)}},RMe=(r,i)=>{if(i.length===0)return[];let o=Object.assign(i);return i.forEach(l=>{const f=r.children(l),b=RMe(r,f);o=[...o,...b]}),o},YWt=r=>RMe(r,r.children());function WWt(r,i){return r.intersect(i)}function jMe(r,i,o,l){var f=r.x,b=r.y,d=f-l.x,w=b-l.y,y=Math.sqrt(i*i*w*w+o*o*d*d),k=Math.abs(i*o*d/y);l.x<f&&(k=-k);var E=Math.abs(i*o*w/y);return l.y<b&&(E=-E),{x:f+k,y:b+E}}function KWt(r,i,o){return jMe(r,i,i,o)}function XWt(r,i,o,l){var f,b,d,w,y,k,E,T,C,S,L,O,B,N,F;if(f=i.y-r.y,d=r.x-i.x,y=i.x*r.y-r.x*i.y,C=f*o.x+d*o.y+y,S=f*l.x+d*l.y+y,!(C!==0&&S!==0&&$Me(C,S))&&(b=l.y-o.y,w=o.x-l.x,k=l.x*o.y-o.x*l.y,E=b*r.x+w*r.y+k,T=b*i.x+w*i.y+k,!(E!==0&&T!==0&&$Me(E,T))&&(L=f*w-b*d,L!==0)))return O=Math.abs(L/2),B=d*k-w*y,N=B<0?(B-O)/L:(B+O)/L,B=b*y-f*k,F=B<0?(B-O)/L:(B+O)/L,{x:N,y:F}}function $Me(r,i){return r*i>0}function QWt(r,i,o){var l=r.x,f=r.y,b=[],d=Number.POSITIVE_INFINITY,w=Number.POSITIVE_INFINITY;typeof i.forEach=="function"?i.forEach(function(L){d=Math.min(d,L.x),w=Math.min(w,L.y)}):(d=Math.min(d,i.x),w=Math.min(w,i.y));for(var y=l-r.width/2-d,k=f-r.height/2-w,E=0;E<i.length;E++){var T=i[E],C=i[E<i.length-1?E+1:0],S=XWt(r,o,{x:y+T.x,y:k+T.y},{x:y+C.x,y:k+C.y});S&&b.push(S)}return b.length?(b.length>1&&b.sort(function(L,O){var B=L.x-o.x,N=L.y-o.y,F=Math.sqrt(B*B+N*N),R=O.x-o.x,q=O.y-o.y,X=Math.sqrt(R*R+q*q);return F<X?-1:F===X?0:1}),b[0]):r}const PI=(r,i)=>{var o=r.x,l=r.y,f=i.x-o,b=i.y-l,d=r.width/2,w=r.height/2,y,k;return Math.abs(b)*d>Math.abs(f)*w?(b<0&&(w=-w),y=b===0?0:w*f/b,k=w):(f<0&&(d=-d),y=d,k=f===0?0:d*b/f),{x:o+y,y:l+k}},Sl={node:WWt,circle:KWt,ellipse:jMe,polygon:QWt,rect:PI},ZWt=(r,i)=>{const{shapeSvg:o,bbox:l,halfPadding:f}=ld(r,i,"node "+i.classes,!0);Se.info("Classes = ",i.classes);const b=o.insert("rect",":first-child");return b.attr("rx",i.rx).attr("ry",i.ry).attr("x",-l.width/2-f).attr("y",-l.height/2-f).attr("width",l.width+i.padding).attr("height",l.height+i.padding),ch(i,b),i.intersect=function(d){return Sl.rect(i,d)},o},HMe=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=f+b,w=[{x:d/2,y:0},{x:d,y:-d/2},{x:d/2,y:-d},{x:0,y:-d/2}];Se.info("Question main (Circle)");const y=e4(o,d,d,w);return y.attr("style",i.style),ch(i,y),i.intersect=function(k){return Se.warn("Intersect called"),Sl.polygon(i,w,k)},o},JWt=(r,i)=>{const o=r.insert("g").attr("class","node default").attr("id",i.domId||i.id),l=28,f=[{x:0,y:l/2},{x:l/2,y:0},{x:0,y:-l/2},{x:-l/2,y:0}];return o.insert("polygon",":first-child").attr("points",f.map(function(d){return d.x+","+d.y}).join(" ")).attr("class","state-start").attr("r",7).attr("width",28).attr("height",28),i.width=28,i.height=28,i.intersect=function(d){return Sl.circle(i,14,d)},o},eKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=4,b=l.height+i.padding,d=b/f,w=l.width+2*d+i.padding,y=[{x:d,y:0},{x:w-d,y:0},{x:w,y:-b/2},{x:w-d,y:-b},{x:d,y:-b},{x:0,y:-b/2}],k=e4(o,w,b,y);return k.attr("style",i.style),ch(i,k),i.intersect=function(E){return Sl.polygon(i,y,E)},o},tKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=[{x:-b/2,y:0},{x:f,y:0},{x:f,y:-b},{x:-b/2,y:-b},{x:0,y:-b/2}];return e4(o,f,b,d).attr("style",i.style),i.width=f+b,i.height=b,i.intersect=function(y){return Sl.polygon(i,d,y)},o},nKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=[{x:-2*b/6,y:0},{x:f-b/6,y:0},{x:f+2*b/6,y:-b},{x:b/6,y:-b}],w=e4(o,f,b,d);return w.attr("style",i.style),ch(i,w),i.intersect=function(y){return Sl.polygon(i,d,y)},o},rKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=[{x:2*b/6,y:0},{x:f+b/6,y:0},{x:f-2*b/6,y:-b},{x:-b/6,y:-b}],w=e4(o,f,b,d);return w.attr("style",i.style),ch(i,w),i.intersect=function(y){return Sl.polygon(i,d,y)},o},iKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=[{x:-2*b/6,y:0},{x:f+2*b/6,y:0},{x:f-b/6,y:-b},{x:b/6,y:-b}],w=e4(o,f,b,d);return w.attr("style",i.style),ch(i,w),i.intersect=function(y){return Sl.polygon(i,d,y)},o},sKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=[{x:b/6,y:0},{x:f-b/6,y:0},{x:f+2*b/6,y:-b},{x:-2*b/6,y:-b}],w=e4(o,f,b,d);return w.attr("style",i.style),ch(i,w),i.intersect=function(y){return Sl.polygon(i,d,y)},o},aKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=[{x:0,y:0},{x:f+b/2,y:0},{x:f,y:-b/2},{x:f+b/2,y:-b},{x:0,y:-b}],w=e4(o,f,b,d);return w.attr("style",i.style),ch(i,w),i.intersect=function(y){return Sl.polygon(i,d,y)},o},oKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=f/2,d=b/(2.5+f/50),w=l.height+d+i.padding,y="M 0,"+d+" a "+b+","+d+" 0,0,0 "+f+" 0 a "+b+","+d+" 0,0,0 "+-f+" 0 l 0,"+w+" a "+b+","+d+" 0,0,0 "+f+" 0 l 0,"+-w,k=o.attr("label-offset-y",d).insert("path",":first-child").attr("style",i.style).attr("d",y).attr("transform","translate("+-f/2+","+-(w/2+d)+")");return ch(i,k),i.intersect=function(E){const T=Sl.rect(i,E),C=T.x-i.x;if(b!=0&&(Math.abs(C)<i.width/2||Math.abs(C)==i.width/2&&Math.abs(T.y-i.y)>i.height/2-d)){let S=d*d*(1-C*C/(b*b));S!=0&&(S=Math.sqrt(S)),S=d-S,E.y-i.y>0&&(S=-S),T.y+=S}return T},o},cKt=(r,i)=>{const{shapeSvg:o,bbox:l,halfPadding:f}=ld(r,i,"node "+i.classes,!0);Se.trace("Classes = ",i.classes);const b=o.insert("rect",":first-child"),d=l.width+i.padding,w=l.height+i.padding;if(b.attr("class","basic label-container").attr("style",i.style).attr("rx",i.rx).attr("ry",i.ry).attr("x",-l.width/2-f).attr("y",-l.height/2-f).attr("width",d).attr("height",w),i.props){const y=new Set(Object.keys(i.props));i.props.borders&&(zMe(b,i.props.borders,d,w),y.delete("borders")),y.forEach(k=>{Se.warn(`Unknown node property ${k}`)})}return ch(i,b),i.intersect=function(y){return Sl.rect(i,y)},o},uKt=(r,i)=>{const{shapeSvg:o}=ld(r,i,"label",!0);Se.trace("Classes = ",i.classes);const l=o.insert("rect",":first-child"),f=0,b=0;if(l.attr("width",f).attr("height",b),o.attr("class","label edgeLabel"),i.props){const d=new Set(Object.keys(i.props));i.props.borders&&(zMe(l,i.props.borders,f,b),d.delete("borders")),d.forEach(w=>{Se.warn(`Unknown node property ${w}`)})}return ch(i,l),i.intersect=function(d){return Sl.rect(i,d)},o};function zMe(r,i,o,l){const f=[],b=w=>{f.push(w,0)},d=w=>{f.push(0,w)};i.includes("t")?(Se.debug("add top border"),b(o)):d(o),i.includes("r")?(Se.debug("add right border"),b(l)):d(l),i.includes("b")?(Se.debug("add bottom border"),b(o)):d(o),i.includes("l")?(Se.debug("add left border"),b(l)):d(l),r.attr("stroke-dasharray",f.join(" "))}const lKt=(r,i)=>{let o;i.classes?o="node "+i.classes:o="node default";const l=r.insert("g").attr("class",o).attr("id",i.domId||i.id),f=l.insert("rect",":first-child"),b=l.insert("line"),d=l.insert("g").attr("class","label"),w=i.labelText.flat?i.labelText.flat():i.labelText;let y="";typeof w=="object"?y=w[0]:y=w,Se.info("Label text abc79",y,w,typeof w=="object");const k=d.node().appendChild(Q0(y,i.labelStyle,!0,!0));let E={width:0,height:0};if(cf(ht().flowchart.htmlLabels)){const O=k.children[0],B=Cn(k);E=O.getBoundingClientRect(),B.attr("width",E.width),B.attr("height",E.height)}Se.info("Text 2",w);const T=w.slice(1,w.length);let C=k.getBBox();const S=d.node().appendChild(Q0(T.join?T.join("<br/>"):T,i.labelStyle,!0,!0));if(cf(ht().flowchart.htmlLabels)){const O=S.children[0],B=Cn(S);E=O.getBoundingClientRect(),B.attr("width",E.width),B.attr("height",E.height)}const L=i.padding/2;return Cn(S).attr("transform","translate( "+(E.width>C.width?0:(C.width-E.width)/2)+", "+(C.height+L+5)+")"),Cn(k).attr("transform","translate( "+(E.width<C.width?0:-(C.width-E.width)/2)+", "+0+")"),E=d.node().getBBox(),d.attr("transform","translate("+-E.width/2+", "+(-E.height/2-L+3)+")"),f.attr("class","outer title-state").attr("x",-E.width/2-L).attr("y",-E.height/2-L).attr("width",E.width+i.padding).attr("height",E.height+i.padding),b.attr("class","divider").attr("x1",-E.width/2-L).attr("x2",E.width/2+L).attr("y1",-E.height/2-L+C.height+L).attr("y2",-E.height/2-L+C.height+L),ch(i,f),i.intersect=function(O){return Sl.rect(i,O)},l},hKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.height+i.padding,b=l.width+f/4+i.padding,d=o.insert("rect",":first-child").attr("style",i.style).attr("rx",f/2).attr("ry",f/2).attr("x",-b/2).attr("y",-f/2).attr("width",b).attr("height",f);return ch(i,d),i.intersect=function(w){return Sl.rect(i,w)},o},fKt=(r,i)=>{const{shapeSvg:o,bbox:l,halfPadding:f}=ld(r,i,void 0,!0),b=o.insert("circle",":first-child");return b.attr("style",i.style).attr("rx",i.rx).attr("ry",i.ry).attr("r",l.width/2+f).attr("width",l.width+i.padding).attr("height",l.height+i.padding),Se.info("Circle main"),ch(i,b),i.intersect=function(d){return Se.info("Circle intersect",i,l.width/2+f,d),Sl.circle(i,l.width/2+f,d)},o},dKt=(r,i)=>{const{shapeSvg:o,bbox:l,halfPadding:f}=ld(r,i,void 0,!0),b=5,d=o.insert("g",":first-child"),w=d.insert("circle"),y=d.insert("circle");return w.attr("style",i.style).attr("rx",i.rx).attr("ry",i.ry).attr("r",l.width/2+f+b).attr("width",l.width+i.padding+b*2).attr("height",l.height+i.padding+b*2),y.attr("style",i.style).attr("rx",i.rx).attr("ry",i.ry).attr("r",l.width/2+f).attr("width",l.width+i.padding).attr("height",l.height+i.padding),Se.info("DoubleCircle main"),ch(i,w),i.intersect=function(k){return Se.info("DoubleCircle intersect",i,l.width/2+f+b,k),Sl.circle(i,l.width/2+f+b,k)},o},gKt=(r,i)=>{const{shapeSvg:o,bbox:l}=ld(r,i,void 0,!0),f=l.width+i.padding,b=l.height+i.padding,d=[{x:0,y:0},{x:f,y:0},{x:f,y:-b},{x:0,y:-b},{x:0,y:0},{x:-8,y:0},{x:f+8,y:0},{x:f+8,y:-b},{x:-8,y:-b},{x:-8,y:0}],w=e4(o,f,b,d);return w.attr("style",i.style),ch(i,w),i.intersect=function(y){return Sl.polygon(i,d,y)},o},pKt=(r,i)=>{const o=r.insert("g").attr("class","node default").attr("id",i.domId||i.id),l=o.insert("circle",":first-child");return l.attr("class","state-start").attr("r",7).attr("width",14).attr("height",14),ch(i,l),i.intersect=function(f){return Sl.circle(i,7,f)},o},GMe=(r,i,o)=>{const l=r.insert("g").attr("class","node default").attr("id",i.domId||i.id);let f=70,b=10;o==="LR"&&(f=10,b=70);const d=l.append("rect").attr("x",-1*f/2).attr("y",-1*b/2).attr("width",f).attr("height",b).attr("class","fork-join");return ch(i,d),i.height=i.height+i.padding/2,i.width=i.width+i.padding/2,i.intersect=function(w){return Sl.rect(i,w)},l},VMe={rhombus:HMe,question:HMe,rect:cKt,labelRect:uKt,rectWithTitle:lKt,choice:JWt,circle:fKt,doublecircle:dKt,stadium:hKt,hexagon:eKt,rect_left_inv_arrow:tKt,lean_right:nKt,lean_left:rKt,trapezoid:iKt,inv_trapezoid:sKt,rect_right_inv_arrow:aKt,cylinder:oKt,start:pKt,end:(r,i)=>{const o=r.insert("g").attr("class","node default").attr("id",i.domId||i.id),l=o.insert("circle",":first-child"),f=o.insert("circle",":first-child");return f.attr("class","state-start").attr("r",7).attr("width",14).attr("height",14),l.attr("class","state-end").attr("r",5).attr("width",10).attr("height",10),ch(i,f),i.intersect=function(b){return Sl.circle(i,7,b)},o},note:ZWt,subroutine:gKt,fork:GMe,join:GMe,class_box:(r,i)=>{const o=i.padding/2,l=4,f=8;let b;i.classes?b="node "+i.classes:b="node default";const d=r.insert("g").attr("class",b).attr("id",i.domId||i.id),w=d.insert("rect",":first-child"),y=d.insert("line"),k=d.insert("line");let E=0,T=l;const C=d.insert("g").attr("class","label");let S=0;const L=i.classData.annotations&&i.classData.annotations[0],O=i.classData.annotations[0]?"\xAB"+i.classData.annotations[0]+"\xBB":"",B=C.node().appendChild(Q0(O,i.labelStyle,!0,!0));let N=B.getBBox();if(cf(ht().flowchart.htmlLabels)){const Y=B.children[0],z=Cn(B);N=Y.getBoundingClientRect(),z.attr("width",N.width),z.attr("height",N.height)}i.classData.annotations[0]&&(T+=N.height+l,E+=N.width);let F=i.classData.id;i.classData.type!==void 0&&i.classData.type!==""&&(ht().flowchart.htmlLabels?F+="&lt;"+i.classData.type+"&gt;":F+="<"+i.classData.type+">");const R=C.node().appendChild(Q0(F,i.labelStyle,!0,!0));Cn(R).attr("class","classTitle");let q=R.getBBox();if(cf(ht().flowchart.htmlLabels)){const Y=R.children[0],z=Cn(R);q=Y.getBoundingClientRect(),z.attr("width",q.width),z.attr("height",q.height)}T+=q.height+l,q.width>E&&(E=q.width);const X=[];i.classData.members.forEach(Y=>{const z=SV(Y);let W=z.displayText;ht().flowchart.htmlLabels&&(W=W.replace(/</g,"&lt;").replace(/>/g,"&gt;"));const Z=C.node().appendChild(Q0(W,z.cssStyle?z.cssStyle:i.labelStyle,!0,!0));let G=Z.getBBox();if(cf(ht().flowchart.htmlLabels)){const ae=Z.children[0],$=Cn(Z);G=ae.getBoundingClientRect(),$.attr("width",G.width),$.attr("height",G.height)}G.width>E&&(E=G.width),T+=G.height+l,X.push(Z)}),T+=f;const te=[];if(i.classData.methods.forEach(Y=>{const z=SV(Y);let W=z.displayText;ht().flowchart.htmlLabels&&(W=W.replace(/</g,"&lt;").replace(/>/g,"&gt;"));const Z=C.node().appendChild(Q0(W,z.cssStyle?z.cssStyle:i.labelStyle,!0,!0));let G=Z.getBBox();if(cf(ht().flowchart.htmlLabels)){const ae=Z.children[0],$=Cn(Z);G=ae.getBoundingClientRect(),$.attr("width",G.width),$.attr("height",G.height)}G.width>E&&(E=G.width),T+=G.height+l,te.push(Z)}),T+=f,L){let Y=(E-N.width)/2;Cn(B).attr("transform","translate( "+(-1*E/2+Y)+", "+-1*T/2+")"),S=N.height+l}let H=(E-q.width)/2;return Cn(R).attr("transform","translate( "+(-1*E/2+H)+", "+(-1*T/2+S)+")"),S+=q.height+l,y.attr("class","divider").attr("x1",-E/2-o).attr("x2",E/2+o).attr("y1",-T/2-o+f+S).attr("y2",-T/2-o+f+S),S+=f,X.forEach(Y=>{Cn(Y).attr("transform","translate( "+-E/2+", "+(-1*T/2+S+f/2)+")"),S+=q.height+l}),S+=f,k.attr("class","divider").attr("x1",-E/2-o).attr("x2",E/2+o).attr("y1",-T/2-o+f+S).attr("y2",-T/2-o+f+S),S+=f,te.forEach(Y=>{Cn(Y).attr("transform","translate( "+-E/2+", "+(-1*T/2+S)+")"),S+=q.height+l}),w.attr("class","outer title-state").attr("x",-E/2-o).attr("y",-(T/2)-o).attr("width",E+i.padding).attr("height",T+i.padding),ch(i,w),i.intersect=function(Y){return Sl.rect(i,Y)},d}};let vC={};const UMe=(r,i,o)=>{let l,f;if(i.link){let b;ht().securityLevel==="sandbox"?b="_top":i.linkTarget&&(b=i.linkTarget||"_blank"),l=r.insert("svg:a").attr("xlink:href",i.link).attr("target",b),f=VMe[i.shape](l,i,o)}else f=VMe[i.shape](r,i,o),l=f;return i.tooltip&&f.attr("title",i.tooltip),i.class&&f.attr("class","node default "+i.class),vC[i.id]=l,i.haveCallback&&vC[i.id].attr("class",vC[i.id].attr("class")+" clickable"),l},bKt=(r,i)=>{vC[i.id]=r},vKt=()=>{vC={}},qMe=r=>{const i=vC[r.id];Se.trace("Transforming node",r.diff,r,"translate("+(r.x-r.width/2-5)+", "+r.width/2+")");const o=8,l=r.diff||0;return r.clusterNode?i.attr("transform","translate("+(r.x+l-r.width/2)+", "+(r.y-r.height/2-o)+")"):i.attr("transform","translate("+r.x+", "+r.y+")"),l},wKt={rect:(r,i)=>{Se.trace("Creating subgraph rect for ",i.id,i);const o=r.insert("g").attr("class","cluster"+(i.class?" "+i.class:"")).attr("id",i.id),l=o.insert("rect",":first-child"),f=o.insert("g").attr("class","cluster-label"),b=f.node().appendChild(Q0(i.labelText,i.labelStyle,void 0,!0));let d=b.getBBox();if(cf(ht().flowchart.htmlLabels)){const T=b.children[0],C=Cn(b);d=T.getBoundingClientRect(),C.attr("width",d.width),C.attr("height",d.height)}const w=0*i.padding,y=w/2,k=i.width<=d.width+w?d.width+w:i.width;i.width<=d.width+w?i.diff=(d.width-i.width)/2-i.padding/2:i.diff=-i.padding/2,Se.trace("Data ",i,JSON.stringify(i)),l.attr("style",i.style).attr("rx",i.rx).attr("ry",i.ry).attr("x",i.x-k/2).attr("y",i.y-i.height/2-y).attr("width",k).attr("height",i.height+w),f.attr("transform","translate("+(i.x-d.width/2)+", "+(i.y-i.height/2)+")");const E=l.node().getBBox();return i.width=E.width,i.height=E.height,i.intersect=function(T){return PI(i,T)},o},roundedWithTitle:(r,i)=>{const o=r.insert("g").attr("class",i.classes).attr("id",i.id),l=o.insert("rect",":first-child"),f=o.insert("g").attr("class","cluster-label"),b=o.append("rect"),d=f.node().appendChild(Q0(i.labelText,i.labelStyle,void 0,!0));let w=d.getBBox();if(cf(ht().flowchart.htmlLabels)){const C=d.children[0],S=Cn(d);w=C.getBoundingClientRect(),S.attr("width",w.width),S.attr("height",w.height)}w=d.getBBox();const y=0*i.padding,k=y/2,E=i.width<=w.width+i.padding?w.width+i.padding:i.width;i.width<=w.width+i.padding?i.diff=(w.width+i.padding*0-i.width)/2:i.diff=-i.padding/2,l.attr("class","outer").attr("x",i.x-E/2-k).attr("y",i.y-i.height/2-k).attr("width",E+y).attr("height",i.height+y),b.attr("class","inner").attr("x",i.x-E/2-k).attr("y",i.y-i.height/2-k+w.height-1).attr("width",E+y).attr("height",i.height+y-w.height-3),f.attr("transform","translate("+(i.x-w.width/2)+", "+(i.y-i.height/2-i.padding/3+(cf(ht().flowchart.htmlLabels)?5:3))+")");const T=l.node().getBBox();return i.height=T.height,i.intersect=function(C){return PI(i,C)},o},noteGroup:(r,i)=>{const o=r.insert("g").attr("class","note-cluster").attr("id",i.id),l=o.insert("rect",":first-child"),f=0*i.padding,b=f/2;l.attr("rx",i.rx).attr("ry",i.ry).attr("x",i.x-i.width/2-b).attr("y",i.y-i.height/2-b).attr("width",i.width+f).attr("height",i.height+f).attr("fill","none");const d=l.node().getBBox();return i.width=d.width,i.height=d.height,i.intersect=function(w){return PI(i,w)},o},divider:(r,i)=>{const o=r.insert("g").attr("class",i.classes).attr("id",i.id),l=o.insert("rect",":first-child"),f=0*i.padding,b=f/2;l.attr("class","divider").attr("x",i.x-i.width/2-b).attr("y",i.y-i.height/2).attr("width",i.width+f).attr("height",i.height+f);const d=l.node().getBBox();return i.width=d.width,i.height=d.height,i.diff=-i.padding/2,i.intersect=function(w){return PI(i,w)},o}};let YMe={};const mKt=(r,i)=>{Se.trace("Inserting cluster");const o=i.shape||"rect";YMe[i.id]=wKt[o](r,i)},yKt=()=>{YMe={}};let IV={},E1={};const kKt=()=>{IV={},E1={}},WMe=(r,i)=>{const o=Q0(i.label,i.labelStyle),l=r.insert("g").attr("class","edgeLabel"),f=l.insert("g").attr("class","label");f.node().appendChild(o);let b=o.getBBox();if(cf(ht().flowchart.htmlLabels)){const w=o.children[0],y=Cn(o);b=w.getBoundingClientRect(),y.attr("width",b.width),y.attr("height",b.height)}f.attr("transform","translate("+-b.width/2+", "+-b.height/2+")"),IV[i.id]=l,i.width=b.width,i.height=b.height;let d;if(i.startLabelLeft){const w=Q0(i.startLabelLeft,i.labelStyle),y=r.insert("g").attr("class","edgeTerminals"),k=y.insert("g").attr("class","inner");d=k.node().appendChild(w);const E=w.getBBox();k.attr("transform","translate("+-E.width/2+", "+-E.height/2+")"),E1[i.id]||(E1[i.id]={}),E1[i.id].startLeft=y,OV(d,i.startLabelLeft)}if(i.startLabelRight){const w=Q0(i.startLabelRight,i.labelStyle),y=r.insert("g").attr("class","edgeTerminals"),k=y.insert("g").attr("class","inner");d=y.node().appendChild(w),k.node().appendChild(w);const E=w.getBBox();k.attr("transform","translate("+-E.width/2+", "+-E.height/2+")"),E1[i.id]||(E1[i.id]={}),E1[i.id].startRight=y,OV(d,i.startLabelRight)}if(i.endLabelLeft){const w=Q0(i.endLabelLeft,i.labelStyle),y=r.insert("g").attr("class","edgeTerminals"),k=y.insert("g").attr("class","inner");d=k.node().appendChild(w);const E=w.getBBox();k.attr("transform","translate("+-E.width/2+", "+-E.height/2+")"),y.node().appendChild(w),E1[i.id]||(E1[i.id]={}),E1[i.id].endLeft=y,OV(d,i.endLabelLeft)}if(i.endLabelRight){const w=Q0(i.endLabelRight,i.labelStyle),y=r.insert("g").attr("class","edgeTerminals"),k=y.insert("g").attr("class","inner");d=k.node().appendChild(w);const E=w.getBBox();k.attr("transform","translate("+-E.width/2+", "+-E.height/2+")"),y.node().appendChild(w),E1[i.id]||(E1[i.id]={}),E1[i.id].endRight=y,OV(d,i.endLabelRight)}return o};function OV(r,i){ht().flowchart.htmlLabels&&r&&(r.style.width=i.length*9+"px",r.style.height="12px")}const xKt=(r,i)=>{Se.info("Moving label abc78 ",r.id,r.label,IV[r.id]);let o=i.updatedPath?i.updatedPath:i.originalPath;if(r.label){const l=IV[r.id];let f=r.x,b=r.y;if(o){const d=Fs.calcLabelPosition(o);Se.info("Moving label "+r.label+" from (",f,",",b,") to (",d.x,",",d.y,") abc78"),i.updatedPath&&(f=d.x,b=d.y)}l.attr("transform","translate("+f+", "+b+")")}if(r.startLabelLeft){const l=E1[r.id].startLeft;let f=r.x,b=r.y;if(o){const d=Fs.calcTerminalLabelPosition(r.arrowTypeStart?10:0,"start_left",o);f=d.x,b=d.y}l.attr("transform","translate("+f+", "+b+")")}if(r.startLabelRight){const l=E1[r.id].startRight;let f=r.x,b=r.y;if(o){const d=Fs.calcTerminalLabelPosition(r.arrowTypeStart?10:0,"start_right",o);f=d.x,b=d.y}l.attr("transform","translate("+f+", "+b+")")}if(r.endLabelLeft){const l=E1[r.id].endLeft;let f=r.x,b=r.y;if(o){const d=Fs.calcTerminalLabelPosition(r.arrowTypeEnd?10:0,"end_left",o);f=d.x,b=d.y}l.attr("transform","translate("+f+", "+b+")")}if(r.endLabelRight){const l=E1[r.id].endRight;let f=r.x,b=r.y;if(o){const d=Fs.calcTerminalLabelPosition(r.arrowTypeEnd?10:0,"end_right",o);f=d.x,b=d.y}l.attr("transform","translate("+f+", "+b+")")}},EKt=(r,i)=>{const o=r.x,l=r.y,f=Math.abs(i.x-o),b=Math.abs(i.y-l),d=r.width/2,w=r.height/2;return f>=d||b>=w},_Kt=(r,i,o)=>{Se.warn(`intersection calc abc89:
  outsidePoint: ${JSON.stringify(i)}
  insidePoint : ${JSON.stringify(o)}
  node        : x:${r.x} y:${r.y} w:${r.width} h:${r.height}`);const l=r.x,f=r.y,b=Math.abs(l-o.x),d=r.width/2;let w=o.x<i.x?d-b:d+b;const y=r.height/2,k=Math.abs(i.y-o.y),E=Math.abs(i.x-o.x);if(Math.abs(f-i.y)*d>Math.abs(l-i.x)*y){let T=o.y<i.y?i.y-y-f:f-y-i.y;w=E*T/k;const C={x:o.x<i.x?o.x+w:o.x-E+w,y:o.y<i.y?o.y+k-T:o.y-k+T};return w===0&&(C.x=i.x,C.y=i.y),E===0&&(C.x=i.x),k===0&&(C.y=i.y),Se.warn(`abc89 topp/bott calc, Q ${k}, q ${T}, R ${E}, r ${w}`,C),C}else{o.x<i.x?w=i.x-d-l:w=l-d-i.x;let T=k*w/E,C=o.x<i.x?o.x+E-w:o.x-E+w,S=o.y<i.y?o.y+T:o.y-T;return Se.warn(`sides calc abc89, Q ${k}, q ${T}, R ${E}, r ${w}`,{_x:C,_y:S}),w===0&&(C=i.x,S=i.y),E===0&&(C=i.x),k===0&&(S=i.y),{x:C,y:S}}},KMe=(r,i)=>{Se.warn("abc88 cutPathAtIntersect",r,i);let o=[],l=r[0],f=!1;return r.forEach(b=>{if(Se.info("abc88 checking point",b,i),!EKt(i,b)&&!f){const d=_Kt(i,l,b);Se.warn("abc88 inside",b,l,d),Se.warn("abc88 intersection",d);let w=!1;o.forEach(y=>{w=w||y.x===d.x&&y.y===d.y}),o.some(y=>y.x===d.x&&y.y===d.y)?Se.warn("abc88 no intersect",d,o):o.push(d),f=!0}else Se.warn("abc88 outside",b,l),l=b,f||o.push(b)}),Se.warn("abc88 returning points",o),o},TKt=function(r,i,o,l,f,b){let d=o.points,w=!1;const y=b.node(i.v);var k=b.node(i.w);Se.info("abc88 InsertEdge: ",o),k.intersect&&y.intersect&&(d=d.slice(1,o.points.length-1),d.unshift(y.intersect(d[0])),Se.info("Last point",d[d.length-1],k,k.intersect(d[d.length-1])),d.push(k.intersect(d[d.length-1]))),o.toCluster&&(Se.info("to cluster abc88",l[o.toCluster]),d=KMe(o.points,l[o.toCluster].node),w=!0),o.fromCluster&&(Se.info("from cluster abc88",l[o.fromCluster]),d=KMe(d.reverse(),l[o.fromCluster].node).reverse(),w=!0);const E=d.filter(N=>!Number.isNaN(N.y));let T;f==="graph"||f==="flowchart"?T=o.curve||UT:T=UT;const C=V8().x(function(N){return N.x}).y(function(N){return N.y}).curve(T);let S;switch(o.thickness){case"normal":S="edge-thickness-normal";break;case"thick":S="edge-thickness-thick";break;default:S=""}switch(o.pattern){case"solid":S+=" edge-pattern-solid";break;case"dotted":S+=" edge-pattern-dotted";break;case"dashed":S+=" edge-pattern-dashed";break}const L=r.append("path").attr("d",C(E)).attr("id",o.id).attr("class"," "+S+(o.classes?" "+o.classes:"")).attr("style",o.style);let O="";switch((ht().flowchart.arrowMarkerAbsolute||ht().state.arrowMarkerAbsolute)&&(O=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,O=O.replace(/\(/g,"\\("),O=O.replace(/\)/g,"\\)")),Se.info("arrowTypeStart",o.arrowTypeStart),Se.info("arrowTypeEnd",o.arrowTypeEnd),o.arrowTypeStart){case"arrow_cross":L.attr("marker-start","url("+O+"#"+f+"-crossStart)");break;case"arrow_point":L.attr("marker-start","url("+O+"#"+f+"-pointStart)");break;case"arrow_barb":L.attr("marker-start","url("+O+"#"+f+"-barbStart)");break;case"arrow_circle":L.attr("marker-start","url("+O+"#"+f+"-circleStart)");break;case"aggregation":L.attr("marker-start","url("+O+"#"+f+"-aggregationStart)");break;case"extension":L.attr("marker-start","url("+O+"#"+f+"-extensionStart)");break;case"composition":L.attr("marker-start","url("+O+"#"+f+"-compositionStart)");break;case"dependency":L.attr("marker-start","url("+O+"#"+f+"-dependencyStart)");break;case"lollipop":L.attr("marker-start","url("+O+"#"+f+"-lollipopStart)");break}switch(o.arrowTypeEnd){case"arrow_cross":L.attr("marker-end","url("+O+"#"+f+"-crossEnd)");break;case"arrow_point":L.attr("marker-end","url("+O+"#"+f+"-pointEnd)");break;case"arrow_barb":L.attr("marker-end","url("+O+"#"+f+"-barbEnd)");break;case"arrow_circle":L.attr("marker-end","url("+O+"#"+f+"-circleEnd)");break;case"aggregation":L.attr("marker-end","url("+O+"#"+f+"-aggregationEnd)");break;case"extension":L.attr("marker-end","url("+O+"#"+f+"-extensionEnd)");break;case"composition":L.attr("marker-end","url("+O+"#"+f+"-compositionEnd)");break;case"dependency":L.attr("marker-end","url("+O+"#"+f+"-dependencyEnd)");break;case"lollipop":L.attr("marker-end","url("+O+"#"+f+"-lollipopEnd)");break}let B={};return w&&(B.updatedPath=d),B.originalPath=o.points,B},XMe=(r,i,o,l)=>{Se.info("Graph in recursive render: XXX",J3(i),l);const f=i.graph().rankdir;Se.trace("Dir in recursive render - dir:",f);const b=r.insert("g").attr("class","root");i.nodes()?Se.info("Recursive render XXX",i.nodes()):Se.info("No nodes found for",i),i.edges().length>0&&Se.trace("Recursive edges",i.edge(i.edges()[0]));const d=b.insert("g").attr("class","clusters"),w=b.insert("g").attr("class","edgePaths"),y=b.insert("g").attr("class","edgeLabels"),k=b.insert("g").attr("class","nodes");i.nodes().forEach(function(T){const C=i.node(T);if(l!==void 0){const S=JSON.parse(JSON.stringify(l.clusterData));Se.info("Setting data for cluster XXX (",T,") ",S,l),i.setNode(l.id,S),i.parent(T)||(Se.trace("Setting parent",T,l.id),i.setParent(T,l.id,S))}if(Se.info("(Insert) Node XXX"+T+": "+JSON.stringify(i.node(T))),C&&C.clusterNode){Se.info("Cluster identified",T,C.width,i.node(T));const S=XMe(k,C.graph,o,i.node(T)),L=S.elem;ch(C,L),C.diff=S.diff||0,Se.info("Node bounds (abc123)",T,C,C.width,C.x,C.y),bKt(L,C),Se.warn("Recursive render complete ",L,C)}else i.children(T).length>0?(Se.info("Cluster - the non recursive path XXX",T,C.id,C,i),Se.info(NI(C.id,i)),qa[C.id]={id:NI(C.id,i),node:C}):(Se.info("Node - the non recursive path",T,C.id,C),UMe(k,i.node(T),f))}),i.edges().forEach(function(T){const C=i.edge(T.v,T.w,T.name);Se.info("Edge "+T.v+" -> "+T.w+": "+JSON.stringify(T)),Se.info("Edge "+T.v+" -> "+T.w+": ",T," ",JSON.stringify(i.edge(T))),Se.info("Fix",qa,"ids:",T.v,T.w,"Translateing: ",qa[T.v],qa[T.w]),WMe(y,C)}),i.edges().forEach(function(T){Se.info("Edge "+T.v+" -> "+T.w+": "+JSON.stringify(T))}),Se.info("#############################################"),Se.info("###                Layout                 ###"),Se.info("#############################################"),Se.info(i),bC(i),Se.info("Graph after layout:",J3(i));let E=0;return YWt(i).forEach(function(T){const C=i.node(T);Se.info("Position "+T+": "+JSON.stringify(i.node(T))),Se.info("Position "+T+": ("+C.x,","+C.y,") width: ",C.width," height: ",C.height),C&&C.clusterNode?qMe(C):i.children(T).length>0?(mKt(d,C),qa[C.id].node=C):qMe(C)}),i.edges().forEach(function(T){const C=i.edge(T);Se.info("Edge "+T.v+" -> "+T.w+": "+JSON.stringify(C),C);const S=TKt(w,T,C,qa,o,i);xKt(C,S)}),i.nodes().forEach(function(T){const C=i.node(T);Se.info(T,C.type,C.diff),C.type==="group"&&(E=C.diff)}),{elem:b,diff:E}},xoe=(r,i,o,l,f)=>{OMe(r,o,l,f),vKt(),kKt(),yKt(),VWt(),Se.warn("Graph at first:",J3(i)),qWt(i),Se.warn("Graph after:",J3(i)),XMe(r,i,l)},QMe=r=>Cs.sanitizeText(r,ht());let ZMe={dividerMargin:10,padding:5,textHeight:10};const CKt=function(r,i,o,l){const f=Object.keys(r);Se.info("keys:",f),Se.info(r),f.forEach(function(b){const d=r[b];let w="";d.cssClasses.length>0&&(w=w+" "+d.cssClasses.join(" "));const y={labelStyle:""};let k=d.text!==void 0?d.text:d.id,E=0,T="";switch(d.type){case"class":T="class_box";break;default:T="class_box"}i.setNode(d.id,{labelStyle:y.labelStyle,shape:T,labelText:QMe(k),classData:d,rx:E,ry:E,class:w,style:y.style,id:d.id,domId:d.domId,tooltip:l.db.getTooltip(d.id)||"",haveCallback:d.haveCallback,link:d.link,width:d.type==="group"?500:void 0,type:d.type,padding:ht().flowchart.padding}),Se.info("setNode",{labelStyle:y.labelStyle,shape:T,labelText:k,rx:E,ry:E,class:w,style:y.style,id:d.id,width:d.type==="group"?500:void 0,type:d.type,padding:ht().flowchart.padding})})},SKt=function(r,i,o,l){Se.info(r),r.forEach(function(f,b){const d=f;let w="";const y={labelStyle:"",style:""};let k=d.text,E=0,T="note";if(i.setNode(d.id,{labelStyle:y.labelStyle,shape:T,labelText:QMe(k),noteData:d,rx:E,ry:E,class:w,style:y.style,id:d.id,domId:d.id,tooltip:"",type:"note",padding:ht().flowchart.padding}),Se.info("setNode",{labelStyle:y.labelStyle,shape:T,labelText:k,rx:E,ry:E,style:y.style,id:d.id,type:"note",padding:ht().flowchart.padding}),!d.class||!(d.class in l))return;const C=o+b,S={};S.classes="relation",S.pattern="dotted",S.id=`edgeNote${C}`,S.arrowhead="none",Se.info(`Note edge: ${JSON.stringify(S)}, ${JSON.stringify(d)}`),S.startLabelRight="",S.endLabelLeft="",S.arrowTypeStart="none",S.arrowTypeEnd="none";let L="fill:none",O="";S.style=L,S.labelStyle=O,S.curve=q0(ZMe.curve,x1),i.setEdge(d.id,d.class,S,C)})},AKt=function(r,i){const o=ht().flowchart;let l=0;r.forEach(function(f){l++;const b={};b.classes="relation",b.pattern=f.relation.lineType==1?"dashed":"solid",b.id="id"+l,f.type==="arrow_open"?b.arrowhead="none":b.arrowhead="normal",Se.info(b,f),b.startLabelRight=f.relationTitle1==="none"?"":f.relationTitle1,b.endLabelLeft=f.relationTitle2==="none"?"":f.relationTitle2,b.arrowTypeStart=JMe(f.relation.type1),b.arrowTypeEnd=JMe(f.relation.type2);let d="",w="";if(f.style!==void 0){const y=Dv(f.style);d=y.style,w=y.labelStyle}else d="fill:none";b.style=d,b.labelStyle=w,f.interpolate!==void 0?b.curve=q0(f.interpolate,x1):r.defaultInterpolate!==void 0?b.curve=q0(r.defaultInterpolate,x1):b.curve=q0(o.curve,x1),f.text=f.title,f.text===void 0?f.style!==void 0&&(b.arrowheadStyle="fill: #333"):(b.arrowheadStyle="fill: #333",b.labelpos="c",ht().flowchart.htmlLabels?(b.labelType="html",b.label='<span class="edgeLabel">'+f.text+"</span>"):(b.labelType="text",b.label=f.text.replace(Cs.lineBreakRegex,`
`),f.style===void 0&&(b.style=b.style||"stroke: #333; stroke-width: 1.5px;fill:none"),b.labelStyle=b.labelStyle.replace("color:","fill:"))),i.setEdge(f.id1,f.id2,b,l)})},MKt=function(r){Object.keys(r).forEach(function(o){ZMe[o]=r[o]})},DKt=function(r,i,o,l){Se.info("Drawing class - ",i);const f=ht().flowchart,b=ht().securityLevel;Se.info("config:",f);const d=f.nodeSpacing||50,w=f.rankSpacing||50,y=new zh({multigraph:!0,compound:!0}).setGraph({rankdir:l.db.getDirection(),nodesep:d,ranksep:w,marginx:8,marginy:8}).setDefaultEdgeLabel(function(){return{}}),k=l.db.getClasses(),E=l.db.getRelations(),T=l.db.getNotes();Se.info(E),CKt(k,y,i,l),AKt(E,y),SKt(T,y,E.length+1,k);let C;b==="sandbox"&&(C=Cn("#i"+i));const S=Cn(b==="sandbox"?C.nodes()[0].contentDocument.body:"body"),L=S.select(`[id="${i}"]`),O=S.select("#"+i+" g");if(xoe(O,y,["aggregation","extension","composition","dependency","lollipop"],"classDiagram",i),Fs.insertTitle(L,"classTitleText",f.titleTopMargin,l.db.getDiagramTitle()),J8(y,L,f.diagramPadding,f.useMaxWidth),!f.htmlLabels){const B=b==="sandbox"?C.nodes()[0].contentDocument:document,N=B.querySelectorAll('[id="'+i+'"] .edgeLabel .label');for(const F of N){const R=F.getBBox(),q=B.createElementNS("http://www.w3.org/2000/svg","rect");q.setAttribute("rx",0),q.setAttribute("ry",0),q.setAttribute("width",R.width),q.setAttribute("height",R.height),F.insertBefore(q,F.firstChild)}}};function JMe(r){let i;switch(r){case 0:i="aggregation";break;case 1:i="extension";break;case 2:i="composition";break;case 3:i="dependency";break;case 4:i="lollipop";break;default:i="none"}return i}const LKt={setConf:MKt,draw:DKt};var Eoe=function(){var r=function(Z,G,ae,$){for(ae=ae||{},$=Z.length;$--;ae[Z[$]]=G);return ae},i=[1,2],o=[1,5],l=[6,9,11,23,25,27,29,30,31,51],f=[1,17],b=[1,18],d=[1,19],w=[1,20],y=[1,21],k=[1,22],E=[1,25],T=[1,30],C=[1,31],S=[1,32],L=[1,33],O=[6,9,11,15,20,23,25,27,29,30,31,44,45,46,47,51],B=[1,45],N=[30,31,48,49],F=[4,6,9,11,23,25,27,29,30,31,51],R=[44,45,46,47],q=[22,37],X=[1,65],te=[1,64],H=[22,37,39,41],Y={trace:function(){},yy:{},symbols_:{error:2,start:3,ER_DIAGRAM:4,document:5,EOF:6,directive:7,line:8,SPACE:9,statement:10,NEWLINE:11,openDirective:12,typeDirective:13,closeDirective:14,":":15,argDirective:16,entityName:17,relSpec:18,role:19,BLOCK_START:20,attributes:21,BLOCK_STOP:22,title:23,title_value:24,acc_title:25,acc_title_value:26,acc_descr:27,acc_descr_value:28,acc_descr_multiline_value:29,ALPHANUM:30,ENTITY_NAME:31,attribute:32,attributeType:33,attributeName:34,attributeKeyTypeList:35,attributeComment:36,ATTRIBUTE_WORD:37,attributeKeyType:38,COMMA:39,ATTRIBUTE_KEY:40,COMMENT:41,cardinality:42,relType:43,ZERO_OR_ONE:44,ZERO_OR_MORE:45,ONE_OR_MORE:46,ONLY_ONE:47,NON_IDENTIFYING:48,IDENTIFYING:49,WORD:50,open_directive:51,type_directive:52,arg_directive:53,close_directive:54,$accept:0,$end:1},terminals_:{2:"error",4:"ER_DIAGRAM",6:"EOF",9:"SPACE",11:"NEWLINE",15:":",20:"BLOCK_START",22:"BLOCK_STOP",23:"title",24:"title_value",25:"acc_title",26:"acc_title_value",27:"acc_descr",28:"acc_descr_value",29:"acc_descr_multiline_value",30:"ALPHANUM",31:"ENTITY_NAME",37:"ATTRIBUTE_WORD",39:"COMMA",40:"ATTRIBUTE_KEY",41:"COMMENT",44:"ZERO_OR_ONE",45:"ZERO_OR_MORE",46:"ONE_OR_MORE",47:"ONLY_ONE",48:"NON_IDENTIFYING",49:"IDENTIFYING",50:"WORD",51:"open_directive",52:"type_directive",53:"arg_directive",54:"close_directive"},productions_:[0,[3,3],[3,2],[5,0],[5,2],[8,2],[8,1],[8,1],[8,1],[7,4],[7,6],[10,1],[10,5],[10,4],[10,3],[10,1],[10,2],[10,2],[10,2],[10,1],[17,1],[17,1],[21,1],[21,2],[32,2],[32,3],[32,3],[32,4],[33,1],[34,1],[35,1],[35,3],[38,1],[36,1],[18,3],[42,1],[42,1],[42,1],[42,1],[43,1],[43,1],[19,1],[19,1],[19,1],[12,1],[13,1],[16,1],[14,1]],performAction:function(G,ae,$,ge,ee,de,re){var ke=de.length-1;switch(ee){case 1:break;case 3:this.$=[];break;case 4:de[ke-1].push(de[ke]),this.$=de[ke-1];break;case 5:case 6:this.$=de[ke];break;case 7:case 8:this.$=[];break;case 12:ge.addEntity(de[ke-4]),ge.addEntity(de[ke-2]),ge.addRelationship(de[ke-4],de[ke],de[ke-2],de[ke-3]);break;case 13:ge.addEntity(de[ke-3]),ge.addAttributes(de[ke-3],de[ke-1]);break;case 14:ge.addEntity(de[ke-2]);break;case 15:ge.addEntity(de[ke]);break;case 16:case 17:this.$=de[ke].trim(),ge.setAccTitle(this.$);break;case 18:case 19:this.$=de[ke].trim(),ge.setAccDescription(this.$);break;case 20:case 43:this.$=de[ke];break;case 21:case 41:case 42:this.$=de[ke].replace(/"/g,"");break;case 22:case 30:this.$=[de[ke]];break;case 23:de[ke].push(de[ke-1]),this.$=de[ke];break;case 24:this.$={attributeType:de[ke-1],attributeName:de[ke]};break;case 25:this.$={attributeType:de[ke-2],attributeName:de[ke-1],attributeKeyTypeList:de[ke]};break;case 26:this.$={attributeType:de[ke-2],attributeName:de[ke-1],attributeComment:de[ke]};break;case 27:this.$={attributeType:de[ke-3],attributeName:de[ke-2],attributeKeyTypeList:de[ke-1],attributeComment:de[ke]};break;case 28:case 29:case 32:this.$=de[ke];break;case 31:de[ke-2].push(de[ke]),this.$=de[ke-2];break;case 33:this.$=de[ke].replace(/"/g,"");break;case 34:this.$={cardA:de[ke],relType:de[ke-1],cardB:de[ke-2]};break;case 35:this.$=ge.Cardinality.ZERO_OR_ONE;break;case 36:this.$=ge.Cardinality.ZERO_OR_MORE;break;case 37:this.$=ge.Cardinality.ONE_OR_MORE;break;case 38:this.$=ge.Cardinality.ONLY_ONE;break;case 39:this.$=ge.Identification.NON_IDENTIFYING;break;case 40:this.$=ge.Identification.IDENTIFYING;break;case 44:ge.parseDirective("%%{","open_directive");break;case 45:ge.parseDirective(de[ke],"type_directive");break;case 46:de[ke]=de[ke].trim().replace(/'/g,'"'),ge.parseDirective(de[ke],"arg_directive");break;case 47:ge.parseDirective("}%%","close_directive","er");break}},table:[{3:1,4:i,7:3,12:4,51:o},{1:[3]},r(l,[2,3],{5:6}),{3:7,4:i,7:3,12:4,51:o},{13:8,52:[1,9]},{52:[2,44]},{6:[1,10],7:15,8:11,9:[1,12],10:13,11:[1,14],12:4,17:16,23:f,25:b,27:d,29:w,30:y,31:k,51:o},{1:[2,2]},{14:23,15:[1,24],54:E},r([15,54],[2,45]),r(l,[2,8],{1:[2,1]}),r(l,[2,4]),{7:15,10:26,12:4,17:16,23:f,25:b,27:d,29:w,30:y,31:k,51:o},r(l,[2,6]),r(l,[2,7]),r(l,[2,11]),r(l,[2,15],{18:27,42:29,20:[1,28],44:T,45:C,46:S,47:L}),{24:[1,34]},{26:[1,35]},{28:[1,36]},r(l,[2,19]),r(O,[2,20]),r(O,[2,21]),{11:[1,37]},{16:38,53:[1,39]},{11:[2,47]},r(l,[2,5]),{17:40,30:y,31:k},{21:41,22:[1,42],32:43,33:44,37:B},{43:46,48:[1,47],49:[1,48]},r(N,[2,35]),r(N,[2,36]),r(N,[2,37]),r(N,[2,38]),r(l,[2,16]),r(l,[2,17]),r(l,[2,18]),r(F,[2,9]),{14:49,54:E},{54:[2,46]},{15:[1,50]},{22:[1,51]},r(l,[2,14]),{21:52,22:[2,22],32:43,33:44,37:B},{34:53,37:[1,54]},{37:[2,28]},{42:55,44:T,45:C,46:S,47:L},r(R,[2,39]),r(R,[2,40]),{11:[1,56]},{19:57,30:[1,60],31:[1,59],50:[1,58]},r(l,[2,13]),{22:[2,23]},r(q,[2,24],{35:61,36:62,38:63,40:X,41:te}),r([22,37,40,41],[2,29]),r([30,31],[2,34]),r(F,[2,10]),r(l,[2,12]),r(l,[2,41]),r(l,[2,42]),r(l,[2,43]),r(q,[2,25],{36:66,39:[1,67],41:te}),r(q,[2,26]),r(H,[2,30]),r(q,[2,33]),r(H,[2,32]),r(q,[2,27]),{38:68,40:X},r(H,[2,31])],defaultActions:{5:[2,44],7:[2,2],25:[2,47],39:[2,46],45:[2,28],52:[2,23]},parseError:function(G,ae){if(ae.recoverable)this.trace(G);else{var $=new Error(G);throw $.hash=ae,$}},parse:function(G){var ae=this,$=[0],ge=[],ee=[null],de=[],re=this.table,ke="",Ce=0,_e=0,Te=2,Be=1,Ge=de.slice.call(arguments,1),Xe=Object.create(this.lexer),Ee={yy:{}};for(var Ze in this.yy)Object.prototype.hasOwnProperty.call(this.yy,Ze)&&(Ee.yy[Ze]=this.yy[Ze]);Xe.setInput(G,Ee.yy),Ee.yy.lexer=Xe,Ee.yy.parser=this,typeof Xe.yylloc>"u"&&(Xe.yylloc={});var Ie=Xe.yylloc;de.push(Ie);var Oe=Xe.options&&Xe.options.ranges;typeof Ee.yy.parseError=="function"?this.parseError=Ee.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Le(){var cn;return cn=ge.pop()||Xe.lex()||Be,typeof cn!="number"&&(cn instanceof Array&&(ge=cn,cn=ge.pop()),cn=ae.symbols_[cn]||cn),cn}for(var $e,xe,Ae,je,me={},vt,ve,Zt,nt;;){if(xe=$[$.length-1],this.defaultActions[xe]?Ae=this.defaultActions[xe]:(($e===null||typeof $e>"u")&&($e=Le()),Ae=re[xe]&&re[xe][$e]),typeof Ae>"u"||!Ae.length||!Ae[0]){var xn="";nt=[];for(vt in re[xe])this.terminals_[vt]&&vt>Te&&nt.push("'"+this.terminals_[vt]+"'");Xe.showPosition?xn="Parse error on line "+(Ce+1)+`:
`+Xe.showPosition()+`
Expecting `+nt.join(", ")+", got '"+(this.terminals_[$e]||$e)+"'":xn="Parse error on line "+(Ce+1)+": Unexpected "+($e==Be?"end of input":"'"+(this.terminals_[$e]||$e)+"'"),this.parseError(xn,{text:Xe.match,token:this.terminals_[$e]||$e,line:Xe.yylineno,loc:Ie,expected:nt})}if(Ae[0]instanceof Array&&Ae.length>1)throw new Error("Parse Error: multiple actions possible at state: "+xe+", token: "+$e);switch(Ae[0]){case 1:$.push($e),ee.push(Xe.yytext),de.push(Xe.yylloc),$.push(Ae[1]),$e=null,_e=Xe.yyleng,ke=Xe.yytext,Ce=Xe.yylineno,Ie=Xe.yylloc;break;case 2:if(ve=this.productions_[Ae[1]][1],me.$=ee[ee.length-ve],me._$={first_line:de[de.length-(ve||1)].first_line,last_line:de[de.length-1].last_line,first_column:de[de.length-(ve||1)].first_column,last_column:de[de.length-1].last_column},Oe&&(me._$.range=[de[de.length-(ve||1)].range[0],de[de.length-1].range[1]]),je=this.performAction.apply(me,[ke,_e,Ce,Ee.yy,Ae[1],ee,de].concat(Ge)),typeof je<"u")return je;ve&&($=$.slice(0,-1*ve*2),ee=ee.slice(0,-1*ve),de=de.slice(0,-1*ve)),$.push(this.productions_[Ae[1]][0]),ee.push(me.$),de.push(me._$),Zt=re[$[$.length-2]][$[$.length-1]],$.push(Zt);break;case 3:return!0}}return!0}},z=function(){var Z={EOF:1,parseError:function(ae,$){if(this.yy.parser)this.yy.parser.parseError(ae,$);else throw new Error(ae)},setInput:function(G,ae){return this.yy=ae||this.yy||{},this._input=G,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var G=this._input[0];this.yytext+=G,this.yyleng++,this.offset++,this.match+=G,this.matched+=G;var ae=G.match(/(?:\r\n?|\n).*/g);return ae?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),G},unput:function(G){var ae=G.length,$=G.split(/(?:\r\n?|\n)/g);this._input=G+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-ae),this.offset-=ae;var ge=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),$.length-1&&(this.yylineno-=$.length-1);var ee=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:$?($.length===ge.length?this.yylloc.first_column:0)+ge[ge.length-$.length].length-$[0].length:this.yylloc.first_column-ae},this.options.ranges&&(this.yylloc.range=[ee[0],ee[0]+this.yyleng-ae]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(G){this.unput(this.match.slice(G))},pastInput:function(){var G=this.matched.substr(0,this.matched.length-this.match.length);return(G.length>20?"...":"")+G.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var G=this.match;return G.length<20&&(G+=this._input.substr(0,20-G.length)),(G.substr(0,20)+(G.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var G=this.pastInput(),ae=new Array(G.length+1).join("-");return G+this.upcomingInput()+`
`+ae+"^"},test_match:function(G,ae){var $,ge,ee;if(this.options.backtrack_lexer&&(ee={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(ee.yylloc.range=this.yylloc.range.slice(0))),ge=G[0].match(/(?:\r\n?|\n).*/g),ge&&(this.yylineno+=ge.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:ge?ge[ge.length-1].length-ge[ge.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+G[0].length},this.yytext+=G[0],this.match+=G[0],this.matches=G,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(G[0].length),this.matched+=G[0],$=this.performAction.call(this,this.yy,this,ae,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),$)return $;if(this._backtrack){for(var de in ee)this[de]=ee[de];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var G,ae,$,ge;this._more||(this.yytext="",this.match="");for(var ee=this._currentRules(),de=0;de<ee.length;de++)if($=this._input.match(this.rules[ee[de]]),$&&(!ae||$[0].length>ae[0].length)){if(ae=$,ge=de,this.options.backtrack_lexer){if(G=this.test_match($,ee[de]),G!==!1)return G;if(this._backtrack){ae=!1;continue}else return!1}else if(!this.options.flex)break}return ae?(G=this.test_match(ae,ee[ge]),G!==!1?G:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var ae=this.next();return ae||this.lex()},begin:function(ae){this.conditionStack.push(ae)},popState:function(){var ae=this.conditionStack.length-1;return ae>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(ae){return ae=this.conditionStack.length-1-Math.abs(ae||0),ae>=0?this.conditionStack[ae]:"INITIAL"},pushState:function(ae){this.begin(ae)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(ae,$,ge,ee){switch(ge){case 0:return this.begin("acc_title"),25;case 1:return this.popState(),"acc_title_value";case 2:return this.begin("acc_descr"),27;case 3:return this.popState(),"acc_descr_value";case 4:this.begin("acc_descr_multiline");break;case 5:this.popState();break;case 6:return"acc_descr_multiline_value";case 7:return this.begin("open_directive"),51;case 8:return this.begin("type_directive"),52;case 9:return this.popState(),this.begin("arg_directive"),15;case 10:return this.popState(),this.popState(),54;case 11:return 53;case 12:break;case 13:break;case 14:return 11;case 15:break;case 16:return 9;case 17:return 31;case 18:return 50;case 19:return 4;case 20:return this.begin("block"),20;case 21:return 39;case 22:break;case 23:return 40;case 24:return 37;case 25:return 37;case 26:return 41;case 27:break;case 28:return this.popState(),22;case 29:return $.yytext[0];case 30:return 44;case 31:return 46;case 32:return 46;case 33:return 46;case 34:return 44;case 35:return 44;case 36:return 45;case 37:return 45;case 38:return 45;case 39:return 45;case 40:return 45;case 41:return 46;case 42:return 45;case 43:return 46;case 44:return 47;case 45:return 47;case 46:return 47;case 47:return 47;case 48:return 44;case 49:return 45;case 50:return 46;case 51:return 48;case 52:return 49;case 53:return 49;case 54:return 48;case 55:return 48;case 56:return 48;case 57:return 30;case 58:return $.yytext[0];case 59:return 6}},rules:[/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:%(?!\{)[^\n]*)/i,/^(?:[^\}]%%[^\n]*)/i,/^(?:[\n]+)/i,/^(?:\s+)/i,/^(?:[\s]+)/i,/^(?:"[^"%\r\n\v\b\\]+")/i,/^(?:"[^"]*")/i,/^(?:erDiagram\b)/i,/^(?:\{)/i,/^(?:,)/i,/^(?:\s+)/i,/^(?:\b((?:PK)|(?:FK)|(?:UK))\b)/i,/^(?:(.*?)[~](.*?)*[~])/i,/^(?:[A-Za-z_][A-Za-z0-9\-_\[\]\(\)]*)/i,/^(?:"[^"]*")/i,/^(?:[\n]+)/i,/^(?:\})/i,/^(?:.)/i,/^(?:one or zero\b)/i,/^(?:one or more\b)/i,/^(?:one or many\b)/i,/^(?:1\+)/i,/^(?:\|o\b)/i,/^(?:zero or one\b)/i,/^(?:zero or more\b)/i,/^(?:zero or many\b)/i,/^(?:0\+)/i,/^(?:\}o\b)/i,/^(?:many\(0\))/i,/^(?:many\(1\))/i,/^(?:many\b)/i,/^(?:\}\|)/i,/^(?:one\b)/i,/^(?:only one\b)/i,/^(?:1\b)/i,/^(?:\|\|)/i,/^(?:o\|)/i,/^(?:o\{)/i,/^(?:\|\{)/i,/^(?:\.\.)/i,/^(?:--)/i,/^(?:to\b)/i,/^(?:optionally to\b)/i,/^(?:\.-)/i,/^(?:-\.)/i,/^(?:[A-Za-z][A-Za-z0-9\-_]*)/i,/^(?:.)/i,/^(?:$)/i],conditions:{acc_descr_multiline:{rules:[5,6],inclusive:!1},acc_descr:{rules:[3],inclusive:!1},acc_title:{rules:[1],inclusive:!1},open_directive:{rules:[8],inclusive:!1},type_directive:{rules:[9,10],inclusive:!1},arg_directive:{rules:[10,11],inclusive:!1},block:{rules:[21,22,23,24,25,26,27,28,29],inclusive:!1},INITIAL:{rules:[0,2,4,7,12,13,14,15,16,17,18,19,20,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59],inclusive:!0}}};return Z}();Y.lexer=z;function W(){this.yy={}}return W.prototype=Y,Y.Parser=W,new W}();Eoe.parser=Eoe;const IKt=Eoe,OKt=r=>r.match(/^\s*erDiagram/)!==null;let BI={},_oe=[];const NKt={ZERO_OR_ONE:"ZERO_OR_ONE",ZERO_OR_MORE:"ZERO_OR_MORE",ONE_OR_MORE:"ONE_OR_MORE",ONLY_ONE:"ONLY_ONE"},PKt={NON_IDENTIFYING:"NON_IDENTIFYING",IDENTIFYING:"IDENTIFYING"},BKt=function(r,i,o){Bu.parseDirective(this,r,i,o)},eDe=function(r){return BI[r]===void 0&&(BI[r]={attributes:[]},Se.info("Added new entity :",r)),BI[r]},FKt={Cardinality:NKt,Identification:PKt,parseDirective:BKt,getConfig:()=>ht().er,addEntity:eDe,addAttributes:function(r,i){let o=eDe(r),l;for(l=i.length-1;l>=0;l--)o.attributes.push(i[l]),Se.debug("Added attribute ",i[l].attributeName)},getEntities:()=>BI,addRelationship:function(r,i,o,l){let f={entityA:r,roleA:i,entityB:o,relSpec:l};_oe.push(f),Se.debug("Added new relationship :",f)},getRelationships:()=>_oe,clear:function(){BI={},_oe=[],W0()},setAccTitle:K0,getAccTitle:hp,setAccDescription:fp,getAccDescription:dp,setDiagramTitle:Ov,getDiagramTitle:Nv},t4={ONLY_ONE_START:"ONLY_ONE_START",ONLY_ONE_END:"ONLY_ONE_END",ZERO_OR_ONE_START:"ZERO_OR_ONE_START",ZERO_OR_ONE_END:"ZERO_OR_ONE_END",ONE_OR_MORE_START:"ONE_OR_MORE_START",ONE_OR_MORE_END:"ONE_OR_MORE_END",ZERO_OR_MORE_START:"ZERO_OR_MORE_START",ZERO_OR_MORE_END:"ZERO_OR_MORE_END"},n4={ERMarkers:t4,insertMarkers:function(r,i){let o;r.append("defs").append("marker").attr("id",t4.ONLY_ONE_START).attr("refX",0).attr("refY",9).attr("markerWidth",18).attr("markerHeight",18).attr("orient","auto").append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M9,0 L9,18 M15,0 L15,18"),r.append("defs").append("marker").attr("id",t4.ONLY_ONE_END).attr("refX",18).attr("refY",9).attr("markerWidth",18).attr("markerHeight",18).attr("orient","auto").append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M3,0 L3,18 M9,0 L9,18"),o=r.append("defs").append("marker").attr("id",t4.ZERO_OR_ONE_START).attr("refX",0).attr("refY",9).attr("markerWidth",30).attr("markerHeight",18).attr("orient","auto"),o.append("circle").attr("stroke",i.stroke).attr("fill","white").attr("cx",21).attr("cy",9).attr("r",6),o.append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M9,0 L9,18"),o=r.append("defs").append("marker").attr("id",t4.ZERO_OR_ONE_END).attr("refX",30).attr("refY",9).attr("markerWidth",30).attr("markerHeight",18).attr("orient","auto"),o.append("circle").attr("stroke",i.stroke).attr("fill","white").attr("cx",9).attr("cy",9).attr("r",6),o.append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M21,0 L21,18"),r.append("defs").append("marker").attr("id",t4.ONE_OR_MORE_START).attr("refX",18).attr("refY",18).attr("markerWidth",45).attr("markerHeight",36).attr("orient","auto").append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M0,18 Q 18,0 36,18 Q 18,36 0,18 M42,9 L42,27"),r.append("defs").append("marker").attr("id",t4.ONE_OR_MORE_END).attr("refX",27).attr("refY",18).attr("markerWidth",45).attr("markerHeight",36).attr("orient","auto").append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M3,9 L3,27 M9,18 Q27,0 45,18 Q27,36 9,18"),o=r.append("defs").append("marker").attr("id",t4.ZERO_OR_MORE_START).attr("refX",18).attr("refY",18).attr("markerWidth",57).attr("markerHeight",36).attr("orient","auto"),o.append("circle").attr("stroke",i.stroke).attr("fill","white").attr("cx",48).attr("cy",18).attr("r",6),o.append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M0,18 Q18,0 36,18 Q18,36 0,18"),o=r.append("defs").append("marker").attr("id",t4.ZERO_OR_MORE_END).attr("refX",39).attr("refY",18).attr("markerWidth",57).attr("markerHeight",36).attr("orient","auto"),o.append("circle").attr("stroke",i.stroke).attr("fill","white").attr("cx",9).attr("cy",18).attr("r",6),o.append("path").attr("stroke",i.stroke).attr("fill","none").attr("d","M21,18 Q39,0 57,18 Q39,36 21,18")}},RKt=/^(?:[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}|00000000-0000-0000-0000-000000000000)$/i;function jKt(r){return typeof r=="string"&&RKt.test(r)}const _1=[];for(let r=0;r<256;++r)_1.push((r+256).toString(16).slice(1));function $Kt(r,i=0){return(_1[r[i+0]]+_1[r[i+1]]+_1[r[i+2]]+_1[r[i+3]]+"-"+_1[r[i+4]]+_1[r[i+5]]+"-"+_1[r[i+6]]+_1[r[i+7]]+"-"+_1[r[i+8]]+_1[r[i+9]]+"-"+_1[r[i+10]]+_1[r[i+11]]+_1[r[i+12]]+_1[r[i+13]]+_1[r[i+14]]+_1[r[i+15]]).toLowerCase()}function HKt(r){if(!jKt(r))throw TypeError("Invalid UUID");let i;const o=new Uint8Array(16);return o[0]=(i=parseInt(r.slice(0,8),16))>>>24,o[1]=i>>>16&255,o[2]=i>>>8&255,o[3]=i&255,o[4]=(i=parseInt(r.slice(9,13),16))>>>8,o[5]=i&255,o[6]=(i=parseInt(r.slice(14,18),16))>>>8,o[7]=i&255,o[8]=(i=parseInt(r.slice(19,23),16))>>>8,o[9]=i&255,o[10]=(i=parseInt(r.slice(24,36),16))/1099511627776&255,o[11]=i/4294967296&255,o[12]=i>>>24&255,o[13]=i>>>16&255,o[14]=i>>>8&255,o[15]=i&255,o}function zKt(r){r=unescape(encodeURIComponent(r));const i=[];for(let o=0;o<r.length;++o)i.push(r.charCodeAt(o));return i}const GKt="6ba7b810-9dad-11d1-80b4-00c04fd430c8",VKt="6ba7b811-9dad-11d1-80b4-00c04fd430c8";function UKt(r,i,o){function l(f,b,d,w){var y;if(typeof f=="string"&&(f=zKt(f)),typeof b=="string"&&(b=HKt(b)),((y=b)===null||y===void 0?void 0:y.length)!==16)throw TypeError("Namespace must be array-like (16 iterable integer values, 0-255)");let k=new Uint8Array(16+f.length);if(k.set(b),k.set(f,b.length),k=o(k),k[6]=k[6]&15|i,k[8]=k[8]&63|128,d){w=w||0;for(let E=0;E<16;++E)d[w+E]=k[E];return d}return $Kt(k)}try{l.name=r}catch{}return l.DNS=GKt,l.URL=VKt,l}function qKt(r,i,o,l){switch(r){case 0:return i&o^~i&l;case 1:return i^o^l;case 2:return i&o^i&l^o&l;case 3:return i^o^l}}function Toe(r,i){return r<<i|r>>>32-i}function YKt(r){const i=[1518500249,1859775393,2400959708,3395469782],o=[1732584193,4023233417,2562383102,271733878,3285377520];if(typeof r=="string"){const d=unescape(encodeURIComponent(r));r=[];for(let w=0;w<d.length;++w)r.push(d.charCodeAt(w))}else Array.isArray(r)||(r=Array.prototype.slice.call(r));r.push(128);const l=r.length/4+2,f=Math.ceil(l/16),b=new Array(f);for(let d=0;d<f;++d){const w=new Uint32Array(16);for(let y=0;y<16;++y)w[y]=r[d*64+y*4]<<24|r[d*64+y*4+1]<<16|r[d*64+y*4+2]<<8|r[d*64+y*4+3];b[d]=w}b[f-1][14]=(r.length-1)*8/Math.pow(2,32),b[f-1][14]=Math.floor(b[f-1][14]),b[f-1][15]=(r.length-1)*8&4294967295;for(let d=0;d<f;++d){const w=new Uint32Array(80);for(let S=0;S<16;++S)w[S]=b[d][S];for(let S=16;S<80;++S)w[S]=Toe(w[S-3]^w[S-8]^w[S-14]^w[S-16],1);let y=o[0],k=o[1],E=o[2],T=o[3],C=o[4];for(let S=0;S<80;++S){const L=Math.floor(S/20),O=Toe(y,5)+qKt(L,k,E,T)+C+i[L]+w[S]>>>0;C=T,T=E,E=Toe(k,30)>>>0,k=y,y=O}o[0]=o[0]+y>>>0,o[1]=o[1]+k>>>0,o[2]=o[2]+E>>>0,o[3]=o[3]+T>>>0,o[4]=o[4]+C>>>0}return[o[0]>>24&255,o[0]>>16&255,o[0]>>8&255,o[0]&255,o[1]>>24&255,o[1]>>16&255,o[1]>>8&255,o[1]&255,o[2]>>24&255,o[2]>>16&255,o[2]>>8&255,o[2]&255,o[3]>>24&255,o[3]>>16&255,o[3]>>8&255,o[3]&255,o[4]>>24&255,o[4]>>16&255,o[4]>>8&255,o[4]&255]}const WKt=UKt("v5",80,YKt),KKt=/[^\dA-Za-z](\W)*/g;let hf={},FI=new Map;const XKt=function(r){const i=Object.keys(r);for(const o of i)hf[o]=r[o]},QKt=(r,i,o)=>{const l=hf.entityPadding/3,f=hf.entityPadding/3,b=hf.fontSize*.85,d=i.node().getBBox(),w=[];let y=!1,k=!1,E=0,T=0,C=0,S=0,L=d.height+l*2,O=1;o.forEach(R=>{R.attributeKeyTypeList!==void 0&&R.attributeKeyTypeList.length>0&&(y=!0),R.attributeComment!==void 0&&(k=!0)}),o.forEach(R=>{const q=`${i.node().id}-attr-${O}`;let X=0;const te=o6(R.attributeType),H=r.append("text").classed("er entityLabel",!0).attr("id",`${q}-type`).attr("x",0).attr("y",0).style("dominant-baseline","middle").style("text-anchor","left").style("font-family",ht().fontFamily).style("font-size",b+"px").text(te),Y=r.append("text").classed("er entityLabel",!0).attr("id",`${q}-name`).attr("x",0).attr("y",0).style("dominant-baseline","middle").style("text-anchor","left").style("font-family",ht().fontFamily).style("font-size",b+"px").text(R.attributeName),z={};z.tn=H,z.nn=Y;const W=H.node().getBBox(),Z=Y.node().getBBox();if(E=Math.max(E,W.width),T=Math.max(T,Z.width),X=Math.max(W.height,Z.height),y){const G=R.attributeKeyTypeList!==void 0?R.attributeKeyTypeList.join(","):"",ae=r.append("text").classed("er entityLabel",!0).attr("id",`${q}-key`).attr("x",0).attr("y",0).style("dominant-baseline","middle").style("text-anchor","left").style("font-family",ht().fontFamily).style("font-size",b+"px").text(G);z.kn=ae;const $=ae.node().getBBox();C=Math.max(C,$.width),X=Math.max(X,$.height)}if(k){const G=r.append("text").classed("er entityLabel",!0).attr("id",`${q}-comment`).attr("x",0).attr("y",0).style("dominant-baseline","middle").style("text-anchor","left").style("font-family",ht().fontFamily).style("font-size",b+"px").text(R.attributeComment||"");z.cn=G;const ae=G.node().getBBox();S=Math.max(S,ae.width),X=Math.max(X,ae.height)}z.height=X,w.push(z),L+=X+l*2,O+=1});let B=4;y&&(B+=2),k&&(B+=2);const N=E+T+C+S,F={width:Math.max(hf.minEntityWidth,Math.max(d.width+hf.entityPadding*2,N+f*B)),height:o.length>0?L:Math.max(hf.minEntityHeight,d.height+hf.entityPadding*2)};if(o.length>0){const R=Math.max(0,(F.width-N-f*B)/(B/2));i.attr("transform","translate("+F.width/2+","+(l+d.height/2)+")");let q=d.height+l*2,X="attributeBoxOdd";w.forEach(te=>{const H=q+l+te.height/2;te.tn.attr("transform","translate("+f+","+H+")");const Y=r.insert("rect","#"+te.tn.node().id).classed(`er ${X}`,!0).attr("x",0).attr("y",q).attr("width",E+f*2+R).attr("height",te.height+l*2),z=parseFloat(Y.attr("x"))+parseFloat(Y.attr("width"));te.nn.attr("transform","translate("+(z+f)+","+H+")");const W=r.insert("rect","#"+te.nn.node().id).classed(`er ${X}`,!0).attr("x",z).attr("y",q).attr("width",T+f*2+R).attr("height",te.height+l*2);let Z=parseFloat(W.attr("x"))+parseFloat(W.attr("width"));if(y){te.kn.attr("transform","translate("+(Z+f)+","+H+")");const G=r.insert("rect","#"+te.kn.node().id).classed(`er ${X}`,!0).attr("x",Z).attr("y",q).attr("width",C+f*2+R).attr("height",te.height+l*2);Z=parseFloat(G.attr("x"))+parseFloat(G.attr("width"))}k&&(te.cn.attr("transform","translate("+(Z+f)+","+H+")"),r.insert("rect","#"+te.cn.node().id).classed(`er ${X}`,"true").attr("x",Z).attr("y",q).attr("width",S+f*2+R).attr("height",te.height+l*2)),q+=te.height+l*2,X=X==="attributeBoxOdd"?"attributeBoxEven":"attributeBoxOdd"})}else F.height=Math.max(hf.minEntityHeight,L),i.attr("transform","translate("+F.width/2+","+F.height/2+")");return F},ZKt=function(r,i,o){const l=Object.keys(i);let f;return l.forEach(function(b){const d=iXt(b,"entity");FI.set(b,d);const w=r.append("g").attr("id",d);f=f===void 0?d:f;const y="text-"+d,k=w.append("text").classed("er entityLabel",!0).attr("id",y).attr("x",0).attr("y",0).style("dominant-baseline","middle").style("text-anchor","middle").style("font-family",ht().fontFamily).style("font-size",hf.fontSize+"px").text(b),{width:E,height:T}=QKt(w,k,i[b].attributes),S=w.insert("rect","#"+y).classed("er entityBox",!0).attr("x",0).attr("y",0).attr("width",E).attr("height",T).node().getBBox();o.setNode(d,{width:S.width,height:S.height,shape:"rect",id:d})}),f},JKt=function(r,i){i.nodes().forEach(function(o){o!==void 0&&i.node(o)!==void 0&&r.select("#"+o).attr("transform","translate("+(i.node(o).x-i.node(o).width/2)+","+(i.node(o).y-i.node(o).height/2)+" )")})},tDe=function(r){return(r.entityA+r.roleA+r.entityB).replace(/\s/g,"")},eXt=function(r,i){return r.forEach(function(o){i.setEdge(FI.get(o.entityA),FI.get(o.entityB),{relationship:o},tDe(o))}),r};let nDe=0;const tXt=function(r,i,o,l,f){nDe++;const b=o.edge(FI.get(i.entityA),FI.get(i.entityB),tDe(i)),d=V8().x(function(L){return L.x}).y(function(L){return L.y}).curve(UT),w=r.insert("path","#"+l).classed("er relationshipLine",!0).attr("d",d(b.points)).style("stroke",hf.stroke).style("fill","none");i.relSpec.relType===f.db.Identification.NON_IDENTIFYING&&w.attr("stroke-dasharray","8,8");let y="";switch(hf.arrowMarkerAbsolute&&(y=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,y=y.replace(/\(/g,"\\("),y=y.replace(/\)/g,"\\)")),i.relSpec.cardA){case f.db.Cardinality.ZERO_OR_ONE:w.attr("marker-end","url("+y+"#"+n4.ERMarkers.ZERO_OR_ONE_END+")");break;case f.db.Cardinality.ZERO_OR_MORE:w.attr("marker-end","url("+y+"#"+n4.ERMarkers.ZERO_OR_MORE_END+")");break;case f.db.Cardinality.ONE_OR_MORE:w.attr("marker-end","url("+y+"#"+n4.ERMarkers.ONE_OR_MORE_END+")");break;case f.db.Cardinality.ONLY_ONE:w.attr("marker-end","url("+y+"#"+n4.ERMarkers.ONLY_ONE_END+")");break}switch(i.relSpec.cardB){case f.db.Cardinality.ZERO_OR_ONE:w.attr("marker-start","url("+y+"#"+n4.ERMarkers.ZERO_OR_ONE_START+")");break;case f.db.Cardinality.ZERO_OR_MORE:w.attr("marker-start","url("+y+"#"+n4.ERMarkers.ZERO_OR_MORE_START+")");break;case f.db.Cardinality.ONE_OR_MORE:w.attr("marker-start","url("+y+"#"+n4.ERMarkers.ONE_OR_MORE_START+")");break;case f.db.Cardinality.ONLY_ONE:w.attr("marker-start","url("+y+"#"+n4.ERMarkers.ONLY_ONE_START+")");break}const k=w.node().getTotalLength(),E=w.node().getPointAtLength(k*.5),T="rel"+nDe,S=r.append("text").classed("er relationshipLabel",!0).attr("id",T).attr("x",E.x).attr("y",E.y).style("text-anchor","middle").style("dominant-baseline","middle").style("font-family",ht().fontFamily).style("font-size",hf.fontSize+"px").text(i.roleA).node().getBBox();r.insert("rect","#"+T).classed("er relationshipLabelBox",!0).attr("x",E.x-S.width/2).attr("y",E.y-S.height/2).attr("width",S.width).attr("height",S.height)},nXt=function(r,i,o,l){hf=ht().er,Se.info("Drawing ER diagram");const f=ht().securityLevel;let b;f==="sandbox"&&(b=Cn("#i"+i));const w=Cn(f==="sandbox"?b.nodes()[0].contentDocument.body:"body").select(`[id='${i}']`);n4.insertMarkers(w,hf);let y;y=new zh({multigraph:!0,directed:!0,compound:!1}).setGraph({rankdir:hf.layoutDirection,marginx:20,marginy:20,nodesep:100,edgesep:100,ranksep:100}).setDefaultEdgeLabel(function(){return{}});const k=ZKt(w,l.db.getEntities(),y),E=eXt(l.db.getRelationships(),y);bC(y),JKt(w,y),E.forEach(function(O){tXt(w,O,y,k,l)});const T=hf.diagramPadding;Fs.insertTitle(w,"entityTitleText",hf.titleTopMargin,l.db.getDiagramTitle());const C=w.node().getBBox(),S=C.width+T*2,L=C.height+T*2;Iv(w,L,S,hf.useMaxWidth),w.attr("viewBox",`${C.x-T} ${C.y-T} ${S} ${L}`)},rXt="28e9f9db-3c8d-5aa5-9faf-44286ae5937c";function iXt(r="",i=""){const o=r.replace(KKt,"");return`${rDe(i)}${rDe(o)}${WKt(r,rXt)}`}function rDe(r=""){return r.length>0?`${r}-`:""}const sXt={setConf:XKt,draw:nXt};var Coe=function(){var r=function(tg,Di,Bi,ra){for(Bi=Bi||{},ra=tg.length;ra--;Bi[tg[ra]]=Di);return Bi},i=[1,9],o=[1,7],l=[1,6],f=[1,8],b=[1,20,21,22,23,38,44,46,48,52,66,67,86,87,88,89,90,91,95,105,106,109,111,112,118,119,120,121,122,123,124,125,126,127],d=[2,10],w=[1,20],y=[1,21],k=[1,22],E=[1,23],T=[1,30],C=[1,32],S=[1,33],L=[1,34],O=[1,62],B=[1,48],N=[1,52],F=[1,36],R=[1,37],q=[1,38],X=[1,39],te=[1,40],H=[1,56],Y=[1,63],z=[1,51],W=[1,53],Z=[1,55],G=[1,59],ae=[1,60],$=[1,41],ge=[1,42],ee=[1,43],de=[1,44],re=[1,61],ke=[1,50],Ce=[1,54],_e=[1,57],Te=[1,58],Be=[1,49],Ge=[1,66],Xe=[1,71],Ee=[1,20,21,22,23,38,42,44,46,48,52,66,67,86,87,88,89,90,91,95,105,106,109,111,112,118,119,120,121,122,123,124,125,126,127],Ze=[1,75],Ie=[1,74],Oe=[1,76],Le=[20,21,23,81,82],$e=[1,99],xe=[1,104],Ae=[1,107],je=[1,108],me=[1,101],vt=[1,106],ve=[1,109],Zt=[1,102],nt=[1,114],xn=[1,113],cn=[1,103],jt=[1,105],ot=[1,110],be=[1,111],We=[1,112],ct=[1,115],Yt=[20,21,22,23,81,82],Ut=[20,21,22,23,53,81,82],Wn=[20,21,22,23,40,52,53,55,57,59,61,63,65,66,67,69,71,73,74,76,81,82,91,95,105,106,109,111,112,122,123,124,125,126,127],Gt=[20,21,23],Rn=[20,21,23,52,66,67,81,82,91,95,105,106,109,111,112,122,123,124,125,126,127],si=[1,12,20,21,22,23,24,38,42,44,46,48,52,66,67,86,87,88,89,90,91,95,105,106,109,111,112,118,119,120,121,122,123,124,125,126,127],$r=[52,66,67,91,95,105,106,109,111,112,122,123,124,125,126,127],nr=[1,149],Kn=[1,157],Jt=[1,158],en=[1,159],In=[1,160],hn=[1,144],Fr=[1,145],Pt=[1,141],ei=[1,152],nn=[1,153],hi=[1,154],Hi=[1,155],ss=[1,156],ls=[1,161],vs=[1,162],ti=[1,147],zi=[1,150],as=[1,146],ai=[1,143],hc=[20,21,22,23,38,42,44,46,48,52,66,67,86,87,88,89,90,91,95,105,106,109,111,112,118,119,120,121,122,123,124,125,126,127],xu=[1,165],No=[20,21,22,23,26,52,66,67,91,105,106,109,111,112,122,123,124,125,126,127],Si=[20,21,22,23,24,26,38,40,41,42,52,56,58,60,62,64,66,67,68,70,72,73,75,77,81,82,86,87,88,89,90,91,92,95,105,106,109,111,112,113,114,122,123,124,125,126,127],Yc=[12,21,22,24],lh=[22,106],su=[1,250],Ru=[1,245],fc=[1,246],Ll=[1,254],ol=[1,251],Ur=[1,248],bt=[1,247],$n=[1,249],Er=[1,252],Ss=[1,253],wo=[1,255],Po=[1,273],Ra=[20,21,23,106],Eu=[20,21,22,23,66,67,86,102,105,106,109,110,111,112,113],Lc={trace:function(){},yy:{},symbols_:{error:2,start:3,mermaidDoc:4,directive:5,openDirective:6,typeDirective:7,closeDirective:8,separator:9,":":10,argDirective:11,open_directive:12,type_directive:13,arg_directive:14,close_directive:15,graphConfig:16,document:17,line:18,statement:19,SEMI:20,NEWLINE:21,SPACE:22,EOF:23,GRAPH:24,NODIR:25,DIR:26,FirstStmtSeperator:27,ending:28,endToken:29,spaceList:30,spaceListNewline:31,verticeStatement:32,styleStatement:33,linkStyleStatement:34,classDefStatement:35,classStatement:36,clickStatement:37,subgraph:38,text:39,SQS:40,SQE:41,end:42,direction:43,acc_title:44,acc_title_value:45,acc_descr:46,acc_descr_value:47,acc_descr_multiline_value:48,link:49,node:50,vertex:51,AMP:52,STYLE_SEPARATOR:53,idString:54,DOUBLECIRCLESTART:55,DOUBLECIRCLEEND:56,PS:57,PE:58,"(-":59,"-)":60,STADIUMSTART:61,STADIUMEND:62,SUBROUTINESTART:63,SUBROUTINEEND:64,VERTEX_WITH_PROPS_START:65,ALPHA:66,COLON:67,PIPE:68,CYLINDERSTART:69,CYLINDEREND:70,DIAMOND_START:71,DIAMOND_STOP:72,TAGEND:73,TRAPSTART:74,TRAPEND:75,INVTRAPSTART:76,INVTRAPEND:77,linkStatement:78,arrowText:79,TESTSTR:80,START_LINK:81,LINK:82,textToken:83,STR:84,keywords:85,STYLE:86,LINKSTYLE:87,CLASSDEF:88,CLASS:89,CLICK:90,DOWN:91,UP:92,textNoTags:93,textNoTagsToken:94,DEFAULT:95,stylesOpt:96,alphaNum:97,CALLBACKNAME:98,CALLBACKARGS:99,HREF:100,LINK_TARGET:101,HEX:102,numList:103,INTERPOLATE:104,NUM:105,COMMA:106,style:107,styleComponent:108,MINUS:109,UNIT:110,BRKT:111,DOT:112,PCT:113,TAGSTART:114,alphaNumToken:115,idStringToken:116,alphaNumStatement:117,direction_tb:118,direction_bt:119,direction_rl:120,direction_lr:121,PUNCTUATION:122,UNICODE_TEXT:123,PLUS:124,EQUALS:125,MULT:126,UNDERSCORE:127,graphCodeTokens:128,ARROW_CROSS:129,ARROW_POINT:130,ARROW_CIRCLE:131,ARROW_OPEN:132,QUOTE:133,$accept:0,$end:1},terminals_:{2:"error",10:":",12:"open_directive",13:"type_directive",14:"arg_directive",15:"close_directive",20:"SEMI",21:"NEWLINE",22:"SPACE",23:"EOF",24:"GRAPH",25:"NODIR",26:"DIR",38:"subgraph",40:"SQS",41:"SQE",42:"end",44:"acc_title",45:"acc_title_value",46:"acc_descr",47:"acc_descr_value",48:"acc_descr_multiline_value",52:"AMP",53:"STYLE_SEPARATOR",55:"DOUBLECIRCLESTART",56:"DOUBLECIRCLEEND",57:"PS",58:"PE",59:"(-",60:"-)",61:"STADIUMSTART",62:"STADIUMEND",63:"SUBROUTINESTART",64:"SUBROUTINEEND",65:"VERTEX_WITH_PROPS_START",66:"ALPHA",67:"COLON",68:"PIPE",69:"CYLINDERSTART",70:"CYLINDEREND",71:"DIAMOND_START",72:"DIAMOND_STOP",73:"TAGEND",74:"TRAPSTART",75:"TRAPEND",76:"INVTRAPSTART",77:"INVTRAPEND",80:"TESTSTR",81:"START_LINK",82:"LINK",84:"STR",86:"STYLE",87:"LINKSTYLE",88:"CLASSDEF",89:"CLASS",90:"CLICK",91:"DOWN",92:"UP",95:"DEFAULT",98:"CALLBACKNAME",99:"CALLBACKARGS",100:"HREF",101:"LINK_TARGET",102:"HEX",104:"INTERPOLATE",105:"NUM",106:"COMMA",109:"MINUS",110:"UNIT",111:"BRKT",112:"DOT",113:"PCT",114:"TAGSTART",118:"direction_tb",119:"direction_bt",120:"direction_rl",121:"direction_lr",122:"PUNCTUATION",123:"UNICODE_TEXT",124:"PLUS",125:"EQUALS",126:"MULT",127:"UNDERSCORE",129:"ARROW_CROSS",130:"ARROW_POINT",131:"ARROW_CIRCLE",132:"ARROW_OPEN",133:"QUOTE"},productions_:[0,[3,1],[3,2],[5,4],[5,6],[6,1],[7,1],[11,1],[8,1],[4,2],[17,0],[17,2],[18,1],[18,1],[18,1],[18,1],[18,1],[16,2],[16,2],[16,2],[16,3],[28,2],[28,1],[29,1],[29,1],[29,1],[27,1],[27,1],[27,2],[31,2],[31,2],[31,1],[31,1],[30,2],[30,1],[19,2],[19,2],[19,2],[19,2],[19,2],[19,2],[19,9],[19,6],[19,4],[19,1],[19,2],[19,2],[19,1],[9,1],[9,1],[9,1],[32,3],[32,4],[32,2],[32,1],[50,1],[50,5],[50,3],[51,4],[51,4],[51,6],[51,4],[51,4],[51,4],[51,8],[51,4],[51,4],[51,4],[51,6],[51,4],[51,4],[51,4],[51,4],[51,4],[51,1],[49,2],[49,3],[49,3],[49,1],[49,3],[78,1],[79,3],[39,1],[39,2],[39,1],[85,1],[85,1],[85,1],[85,1],[85,1],[85,1],[85,1],[85,1],[85,1],[85,1],[85,1],[93,1],[93,2],[35,5],[35,5],[36,5],[37,2],[37,4],[37,3],[37,5],[37,2],[37,4],[37,4],[37,6],[37,2],[37,4],[37,2],[37,4],[37,4],[37,6],[33,5],[33,5],[34,5],[34,5],[34,9],[34,9],[34,7],[34,7],[103,1],[103,3],[96,1],[96,3],[107,1],[107,2],[108,1],[108,1],[108,1],[108,1],[108,1],[108,1],[108,1],[108,1],[108,1],[108,1],[108,1],[83,1],[83,1],[83,1],[83,1],[83,1],[83,1],[94,1],[94,1],[94,1],[94,1],[54,1],[54,2],[97,1],[97,2],[117,1],[117,1],[117,1],[117,1],[43,1],[43,1],[43,1],[43,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[115,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[116,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1],[128,1]],performAction:function(Di,Bi,ra,Rr,au,gt,O6){var It=gt.length-1;switch(au){case 5:Rr.parseDirective("%%{","open_directive");break;case 6:Rr.parseDirective(gt[It],"type_directive");break;case 7:gt[It]=gt[It].trim().replace(/'/g,'"'),Rr.parseDirective(gt[It],"arg_directive");break;case 8:Rr.parseDirective("}%%","close_directive","flowchart");break;case 10:this.$=[];break;case 11:(!Array.isArray(gt[It])||gt[It].length>0)&&gt[It-1].push(gt[It]),this.$=gt[It-1];break;case 12:case 82:case 84:case 96:case 152:case 154:case 155:this.$=gt[It];break;case 19:Rr.setDirection("TB"),this.$="TB";break;case 20:Rr.setDirection(gt[It-1]),this.$=gt[It-1];break;case 35:this.$=gt[It-1].nodes;break;case 36:case 37:case 38:case 39:case 40:this.$=[];break;case 41:this.$=Rr.addSubGraph(gt[It-6],gt[It-1],gt[It-4]);break;case 42:this.$=Rr.addSubGraph(gt[It-3],gt[It-1],gt[It-3]);break;case 43:this.$=Rr.addSubGraph(void 0,gt[It-1],void 0);break;case 45:this.$=gt[It].trim(),Rr.setAccTitle(this.$);break;case 46:case 47:this.$=gt[It].trim(),Rr.setAccDescription(this.$);break;case 51:Rr.addLink(gt[It-2].stmt,gt[It],gt[It-1]),this.$={stmt:gt[It],nodes:gt[It].concat(gt[It-2].nodes)};break;case 52:Rr.addLink(gt[It-3].stmt,gt[It-1],gt[It-2]),this.$={stmt:gt[It-1],nodes:gt[It-1].concat(gt[It-3].nodes)};break;case 53:this.$={stmt:gt[It-1],nodes:gt[It-1]};break;case 54:this.$={stmt:gt[It],nodes:gt[It]};break;case 55:this.$=[gt[It]];break;case 56:this.$=gt[It-4].concat(gt[It]);break;case 57:this.$=[gt[It-2]],Rr.setClass(gt[It-2],gt[It]);break;case 58:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"square");break;case 59:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"doublecircle");break;case 60:this.$=gt[It-5],Rr.addVertex(gt[It-5],gt[It-2],"circle");break;case 61:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"ellipse");break;case 62:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"stadium");break;case 63:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"subroutine");break;case 64:this.$=gt[It-7],Rr.addVertex(gt[It-7],gt[It-1],"rect",void 0,void 0,void 0,Object.fromEntries([[gt[It-5],gt[It-3]]]));break;case 65:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"cylinder");break;case 66:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"round");break;case 67:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"diamond");break;case 68:this.$=gt[It-5],Rr.addVertex(gt[It-5],gt[It-2],"hexagon");break;case 69:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"odd");break;case 70:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"trapezoid");break;case 71:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"inv_trapezoid");break;case 72:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"lean_right");break;case 73:this.$=gt[It-3],Rr.addVertex(gt[It-3],gt[It-1],"lean_left");break;case 74:this.$=gt[It],Rr.addVertex(gt[It]);break;case 75:gt[It-1].text=gt[It],this.$=gt[It-1];break;case 76:case 77:gt[It-2].text=gt[It-1],this.$=gt[It-2];break;case 78:this.$=gt[It];break;case 79:var n0=Rr.destructLink(gt[It],gt[It-2]);this.$={type:n0.type,stroke:n0.stroke,length:n0.length,text:gt[It-1]};break;case 80:var n0=Rr.destructLink(gt[It]);this.$={type:n0.type,stroke:n0.stroke,length:n0.length};break;case 81:this.$=gt[It-1];break;case 83:case 97:case 153:this.$=gt[It-1]+""+gt[It];break;case 98:case 99:this.$=gt[It-4],Rr.addClass(gt[It-2],gt[It]);break;case 100:this.$=gt[It-4],Rr.setClass(gt[It-2],gt[It]);break;case 101:case 109:this.$=gt[It-1],Rr.setClickEvent(gt[It-1],gt[It]);break;case 102:case 110:this.$=gt[It-3],Rr.setClickEvent(gt[It-3],gt[It-2]),Rr.setTooltip(gt[It-3],gt[It]);break;case 103:this.$=gt[It-2],Rr.setClickEvent(gt[It-2],gt[It-1],gt[It]);break;case 104:this.$=gt[It-4],Rr.setClickEvent(gt[It-4],gt[It-3],gt[It-2]),Rr.setTooltip(gt[It-4],gt[It]);break;case 105:case 111:this.$=gt[It-1],Rr.setLink(gt[It-1],gt[It]);break;case 106:case 112:this.$=gt[It-3],Rr.setLink(gt[It-3],gt[It-2]),Rr.setTooltip(gt[It-3],gt[It]);break;case 107:case 113:this.$=gt[It-3],Rr.setLink(gt[It-3],gt[It-2],gt[It]);break;case 108:case 114:this.$=gt[It-5],Rr.setLink(gt[It-5],gt[It-4],gt[It]),Rr.setTooltip(gt[It-5],gt[It-2]);break;case 115:this.$=gt[It-4],Rr.addVertex(gt[It-2],void 0,void 0,gt[It]);break;case 116:case 118:this.$=gt[It-4],Rr.updateLink(gt[It-2],gt[It]);break;case 117:this.$=gt[It-4],Rr.updateLink([gt[It-2]],gt[It]);break;case 119:this.$=gt[It-8],Rr.updateLinkInterpolate([gt[It-6]],gt[It-2]),Rr.updateLink([gt[It-6]],gt[It]);break;case 120:this.$=gt[It-8],Rr.updateLinkInterpolate(gt[It-6],gt[It-2]),Rr.updateLink(gt[It-6],gt[It]);break;case 121:this.$=gt[It-6],Rr.updateLinkInterpolate([gt[It-4]],gt[It]);break;case 122:this.$=gt[It-6],Rr.updateLinkInterpolate(gt[It-4],gt[It]);break;case 123:case 125:this.$=[gt[It]];break;case 124:case 126:gt[It-2].push(gt[It]),this.$=gt[It-2];break;case 128:this.$=gt[It-1]+gt[It];break;case 150:this.$=gt[It];break;case 151:this.$=gt[It-1]+""+gt[It];break;case 156:this.$="v";break;case 157:this.$="-";break;case 158:this.$={stmt:"dir",value:"TB"};break;case 159:this.$={stmt:"dir",value:"BT"};break;case 160:this.$={stmt:"dir",value:"RL"};break;case 161:this.$={stmt:"dir",value:"LR"};break}},table:[{3:1,4:2,5:3,6:5,12:i,16:4,21:o,22:l,24:f},{1:[3]},{1:[2,1]},{3:10,4:2,5:3,6:5,12:i,16:4,21:o,22:l,24:f},r(b,d,{17:11}),{7:12,13:[1,13]},{16:14,21:o,22:l,24:f},{16:15,21:o,22:l,24:f},{25:[1,16],26:[1,17]},{13:[2,5]},{1:[2,2]},{1:[2,9],18:18,19:19,20:w,21:y,22:k,23:E,32:24,33:25,34:26,35:27,36:28,37:29,38:T,43:31,44:C,46:S,48:L,50:35,51:45,52:O,54:46,66:B,67:N,86:F,87:R,88:q,89:X,90:te,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,116:47,118:$,119:ge,120:ee,121:de,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be},{8:64,10:[1,65],15:Ge},r([10,15],[2,6]),r(b,[2,17]),r(b,[2,18]),r(b,[2,19]),{20:[1,68],21:[1,69],22:Xe,27:67,30:70},r(Ee,[2,11]),r(Ee,[2,12]),r(Ee,[2,13]),r(Ee,[2,14]),r(Ee,[2,15]),r(Ee,[2,16]),{9:72,20:Ze,21:Ie,23:Oe,49:73,78:77,81:[1,78],82:[1,79]},{9:80,20:Ze,21:Ie,23:Oe},{9:81,20:Ze,21:Ie,23:Oe},{9:82,20:Ze,21:Ie,23:Oe},{9:83,20:Ze,21:Ie,23:Oe},{9:84,20:Ze,21:Ie,23:Oe},{9:86,20:Ze,21:Ie,22:[1,85],23:Oe},r(Ee,[2,44]),{45:[1,87]},{47:[1,88]},r(Ee,[2,47]),r(Le,[2,54],{30:89,22:Xe}),{22:[1,90]},{22:[1,91]},{22:[1,92]},{22:[1,93]},{26:$e,52:xe,66:Ae,67:je,84:[1,97],91:me,97:96,98:[1,94],100:[1,95],105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:98,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Ee,[2,158]),r(Ee,[2,159]),r(Ee,[2,160]),r(Ee,[2,161]),r(Yt,[2,55],{53:[1,116]}),r(Ut,[2,74],{116:129,40:[1,117],52:O,55:[1,118],57:[1,119],59:[1,120],61:[1,121],63:[1,122],65:[1,123],66:B,67:N,69:[1,124],71:[1,125],73:[1,126],74:[1,127],76:[1,128],91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be}),r(Wn,[2,150]),r(Wn,[2,175]),r(Wn,[2,176]),r(Wn,[2,177]),r(Wn,[2,178]),r(Wn,[2,179]),r(Wn,[2,180]),r(Wn,[2,181]),r(Wn,[2,182]),r(Wn,[2,183]),r(Wn,[2,184]),r(Wn,[2,185]),r(Wn,[2,186]),r(Wn,[2,187]),r(Wn,[2,188]),r(Wn,[2,189]),r(Wn,[2,190]),{9:130,20:Ze,21:Ie,23:Oe},{11:131,14:[1,132]},r(Gt,[2,8]),r(b,[2,20]),r(b,[2,26]),r(b,[2,27]),{21:[1,133]},r(Rn,[2,34],{30:134,22:Xe}),r(Ee,[2,35]),{50:135,51:45,52:O,54:46,66:B,67:N,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,116:47,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be},r(si,[2,48]),r(si,[2,49]),r(si,[2,50]),r($r,[2,78],{79:136,68:[1,138],80:[1,137]}),{22:nr,24:Kn,26:Jt,38:en,39:139,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r([52,66,67,68,80,91,95,105,106,109,111,112,122,123,124,125,126,127],[2,80]),r(Ee,[2,36]),r(Ee,[2,37]),r(Ee,[2,38]),r(Ee,[2,39]),r(Ee,[2,40]),{22:nr,24:Kn,26:Jt,38:en,39:163,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(hc,d,{17:164}),r(Ee,[2,45]),r(Ee,[2,46]),r(Le,[2,53],{52:xu}),{26:$e,52:xe,66:Ae,67:je,91:me,97:166,102:[1,167],105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:98,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{95:[1,168],103:169,105:[1,170]},{26:$e,52:xe,66:Ae,67:je,91:me,95:[1,171],97:172,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:98,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{26:$e,52:xe,66:Ae,67:je,91:me,97:173,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:98,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Gt,[2,101],{22:[1,174],99:[1,175]}),r(Gt,[2,105],{22:[1,176]}),r(Gt,[2,109],{115:100,117:178,22:[1,177],26:$e,52:xe,66:Ae,67:je,91:me,105:vt,106:ve,109:Zt,111:nt,112:xn,122:cn,123:jt,124:ot,125:be,126:We,127:ct}),r(Gt,[2,111],{22:[1,179]}),r(No,[2,152]),r(No,[2,154]),r(No,[2,155]),r(No,[2,156]),r(No,[2,157]),r(Si,[2,162]),r(Si,[2,163]),r(Si,[2,164]),r(Si,[2,165]),r(Si,[2,166]),r(Si,[2,167]),r(Si,[2,168]),r(Si,[2,169]),r(Si,[2,170]),r(Si,[2,171]),r(Si,[2,172]),r(Si,[2,173]),r(Si,[2,174]),{52:O,54:180,66:B,67:N,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,116:47,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be},{22:nr,24:Kn,26:Jt,38:en,39:181,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:182,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:184,42:In,52:xe,57:[1,183],66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:185,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:186,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:187,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{66:[1,188]},{22:nr,24:Kn,26:Jt,38:en,39:189,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:190,42:In,52:xe,66:Ae,67:je,71:[1,191],73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:192,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:193,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:194,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Wn,[2,151]),r(Yc,[2,3]),{8:195,15:Ge},{15:[2,7]},r(b,[2,28]),r(Rn,[2,33]),r(Le,[2,51],{30:196,22:Xe}),r($r,[2,75],{22:[1,197]}),{22:[1,198]},{22:nr,24:Kn,26:Jt,38:en,39:199,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,82:[1,200],83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Si,[2,82]),r(Si,[2,84]),r(Si,[2,140]),r(Si,[2,141]),r(Si,[2,142]),r(Si,[2,143]),r(Si,[2,144]),r(Si,[2,145]),r(Si,[2,146]),r(Si,[2,147]),r(Si,[2,148]),r(Si,[2,149]),r(Si,[2,85]),r(Si,[2,86]),r(Si,[2,87]),r(Si,[2,88]),r(Si,[2,89]),r(Si,[2,90]),r(Si,[2,91]),r(Si,[2,92]),r(Si,[2,93]),r(Si,[2,94]),r(Si,[2,95]),{9:203,20:Ze,21:Ie,22:nr,23:Oe,24:Kn,26:Jt,38:en,40:[1,202],42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{18:18,19:19,20:w,21:y,22:k,23:E,32:24,33:25,34:26,35:27,36:28,37:29,38:T,42:[1,204],43:31,44:C,46:S,48:L,50:35,51:45,52:O,54:46,66:B,67:N,86:F,87:R,88:q,89:X,90:te,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,116:47,118:$,119:ge,120:ee,121:de,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be},{22:Xe,30:205},{22:[1,206],26:$e,52:xe,66:Ae,67:je,91:me,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:178,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:[1,207]},{22:[1,208]},{22:[1,209],106:[1,210]},r(lh,[2,123]),{22:[1,211]},{22:[1,212],26:$e,52:xe,66:Ae,67:je,91:me,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:178,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:[1,213],26:$e,52:xe,66:Ae,67:je,91:me,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:178,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{84:[1,214]},r(Gt,[2,103],{22:[1,215]}),{84:[1,216],101:[1,217]},{84:[1,218]},r(No,[2,153]),{84:[1,219],101:[1,220]},r(Yt,[2,57],{116:129,52:O,66:B,67:N,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be}),{22:nr,24:Kn,26:Jt,38:en,41:[1,221],42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,56:[1,222],66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:223,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,58:[1,224],66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,60:[1,225],66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,62:[1,226],66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,64:[1,227],66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{67:[1,228]},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,66:Ae,67:je,70:[1,229],73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,66:Ae,67:je,72:[1,230],73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,39:231,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,41:[1,232],42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,66:Ae,67:je,73:hn,75:[1,233],77:[1,234],81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,66:Ae,67:je,73:hn,75:[1,236],77:[1,235],81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{9:237,20:Ze,21:Ie,23:Oe},r(Le,[2,52],{52:xu}),r($r,[2,77]),r($r,[2,76]),{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,66:Ae,67:je,68:[1,238],73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r($r,[2,79]),r(Si,[2,83]),{22:nr,24:Kn,26:Jt,38:en,39:239,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(hc,d,{17:240}),r(Ee,[2,43]),{51:241,52:O,54:46,66:B,67:N,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,116:47,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be},{22:su,66:Ru,67:fc,86:Ll,96:242,102:ol,105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},{22:su,66:Ru,67:fc,86:Ll,96:256,102:ol,105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},{22:su,66:Ru,67:fc,86:Ll,96:257,102:ol,104:[1,258],105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},{22:su,66:Ru,67:fc,86:Ll,96:259,102:ol,104:[1,260],105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},{105:[1,261]},{22:su,66:Ru,67:fc,86:Ll,96:262,102:ol,105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},{22:su,66:Ru,67:fc,86:Ll,96:263,102:ol,105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},{26:$e,52:xe,66:Ae,67:je,91:me,97:264,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:98,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Gt,[2,102]),{84:[1,265]},r(Gt,[2,106],{22:[1,266]}),r(Gt,[2,107]),r(Gt,[2,110]),r(Gt,[2,112],{22:[1,267]}),r(Gt,[2,113]),r(Ut,[2,58]),r(Ut,[2,59]),{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,58:[1,268],66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Ut,[2,66]),r(Ut,[2,61]),r(Ut,[2,62]),r(Ut,[2,63]),{66:[1,269]},r(Ut,[2,65]),r(Ut,[2,67]),{22:nr,24:Kn,26:Jt,38:en,42:In,52:xe,66:Ae,67:je,72:[1,270],73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Ut,[2,69]),r(Ut,[2,70]),r(Ut,[2,72]),r(Ut,[2,71]),r(Ut,[2,73]),r(Yc,[2,4]),r([22,52,66,67,91,95,105,106,109,111,112,122,123,124,125,126,127],[2,81]),{22:nr,24:Kn,26:Jt,38:en,41:[1,271],42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{18:18,19:19,20:w,21:y,22:k,23:E,32:24,33:25,34:26,35:27,36:28,37:29,38:T,42:[1,272],43:31,44:C,46:S,48:L,50:35,51:45,52:O,54:46,66:B,67:N,86:F,87:R,88:q,89:X,90:te,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,116:47,118:$,119:ge,120:ee,121:de,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be},r(Yt,[2,56]),r(Gt,[2,115],{106:Po}),r(Ra,[2,125],{108:274,22:su,66:Ru,67:fc,86:Ll,102:ol,105:Ur,109:bt,110:$n,111:Er,112:Ss,113:wo}),r(Eu,[2,127]),r(Eu,[2,129]),r(Eu,[2,130]),r(Eu,[2,131]),r(Eu,[2,132]),r(Eu,[2,133]),r(Eu,[2,134]),r(Eu,[2,135]),r(Eu,[2,136]),r(Eu,[2,137]),r(Eu,[2,138]),r(Eu,[2,139]),r(Gt,[2,116],{106:Po}),r(Gt,[2,117],{106:Po}),{22:[1,275]},r(Gt,[2,118],{106:Po}),{22:[1,276]},r(lh,[2,124]),r(Gt,[2,98],{106:Po}),r(Gt,[2,99],{106:Po}),r(Gt,[2,100],{115:100,117:178,26:$e,52:xe,66:Ae,67:je,91:me,105:vt,106:ve,109:Zt,111:nt,112:xn,122:cn,123:jt,124:ot,125:be,126:We,127:ct}),r(Gt,[2,104]),{101:[1,277]},{101:[1,278]},{58:[1,279]},{68:[1,280]},{72:[1,281]},{9:282,20:Ze,21:Ie,23:Oe},r(Ee,[2,42]),{22:su,66:Ru,67:fc,86:Ll,102:ol,105:Ur,107:283,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},r(Eu,[2,128]),{26:$e,52:xe,66:Ae,67:je,91:me,97:284,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:98,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{26:$e,52:xe,66:Ae,67:je,91:me,97:285,105:vt,106:ve,109:Zt,111:nt,112:xn,115:100,117:98,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Gt,[2,108]),r(Gt,[2,114]),r(Ut,[2,60]),{22:nr,24:Kn,26:Jt,38:en,39:286,42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:140,84:Pt,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},r(Ut,[2,68]),r(hc,d,{17:287}),r(Ra,[2,126],{108:274,22:su,66:Ru,67:fc,86:Ll,102:ol,105:Ur,109:bt,110:$n,111:Er,112:Ss,113:wo}),r(Gt,[2,121],{115:100,117:178,22:[1,288],26:$e,52:xe,66:Ae,67:je,91:me,105:vt,106:ve,109:Zt,111:nt,112:xn,122:cn,123:jt,124:ot,125:be,126:We,127:ct}),r(Gt,[2,122],{115:100,117:178,22:[1,289],26:$e,52:xe,66:Ae,67:je,91:me,105:vt,106:ve,109:Zt,111:nt,112:xn,122:cn,123:jt,124:ot,125:be,126:We,127:ct}),{22:nr,24:Kn,26:Jt,38:en,41:[1,290],42:In,52:xe,66:Ae,67:je,73:hn,81:Fr,83:201,85:151,86:ei,87:nn,88:hi,89:Hi,90:ss,91:ls,92:vs,94:142,95:ti,105:vt,106:ve,109:zi,111:nt,112:xn,113:as,114:ai,115:148,122:cn,123:jt,124:ot,125:be,126:We,127:ct},{18:18,19:19,20:w,21:y,22:k,23:E,32:24,33:25,34:26,35:27,36:28,37:29,38:T,42:[1,291],43:31,44:C,46:S,48:L,50:35,51:45,52:O,54:46,66:B,67:N,86:F,87:R,88:q,89:X,90:te,91:H,95:Y,105:z,106:W,109:Z,111:G,112:ae,116:47,118:$,119:ge,120:ee,121:de,122:re,123:ke,124:Ce,125:_e,126:Te,127:Be},{22:su,66:Ru,67:fc,86:Ll,96:292,102:ol,105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},{22:su,66:Ru,67:fc,86:Ll,96:293,102:ol,105:Ur,107:243,108:244,109:bt,110:$n,111:Er,112:Ss,113:wo},r(Ut,[2,64]),r(Ee,[2,41]),r(Gt,[2,119],{106:Po}),r(Gt,[2,120],{106:Po})],defaultActions:{2:[2,1],9:[2,5],10:[2,2],132:[2,7]},parseError:function(Di,Bi){if(Bi.recoverable)this.trace(Di);else{var ra=new Error(Di);throw ra.hash=Bi,ra}},parse:function(Di){var Bi=this,ra=[0],Rr=[],au=[null],gt=[],O6=this.table,It="",n0=0,jO=0,rq=2,tS=1,iq=gt.slice.call(arguments,1),fh=Object.create(this.lexer),Qm={yy:{}};for(var nS in this.yy)Object.prototype.hasOwnProperty.call(this.yy,nS)&&(Qm.yy[nS]=this.yy[nS]);fh.setInput(Di,Qm.yy),Qm.yy.lexer=fh,Qm.yy.parser=this,typeof fh.yylloc>"u"&&(fh.yylloc={});var rS=fh.yylloc;gt.push(rS);var $O=fh.options&&fh.options.ranges;typeof Qm.yy.parseError=="function"?this.parseError=Qm.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function sq(){var z2;return z2=Rr.pop()||fh.lex()||tS,typeof z2!="number"&&(z2 instanceof Array&&(Rr=z2,z2=Rr.pop()),z2=Bi.symbols_[z2]||z2),z2}for(var D1,Zm,r0,iS,w4={},M7,H2,HO,D7;;){if(Zm=ra[ra.length-1],this.defaultActions[Zm]?r0=this.defaultActions[Zm]:((D1===null||typeof D1>"u")&&(D1=sq()),r0=O6[Zm]&&O6[Zm][D1]),typeof r0>"u"||!r0.length||!r0[0]){var sS="";D7=[];for(M7 in O6[Zm])this.terminals_[M7]&&M7>rq&&D7.push("'"+this.terminals_[M7]+"'");fh.showPosition?sS="Parse error on line "+(n0+1)+`:
`+fh.showPosition()+`
Expecting `+D7.join(", ")+", got '"+(this.terminals_[D1]||D1)+"'":sS="Parse error on line "+(n0+1)+": Unexpected "+(D1==tS?"end of input":"'"+(this.terminals_[D1]||D1)+"'"),this.parseError(sS,{text:fh.match,token:this.terminals_[D1]||D1,line:fh.yylineno,loc:rS,expected:D7})}if(r0[0]instanceof Array&&r0.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Zm+", token: "+D1);switch(r0[0]){case 1:ra.push(D1),au.push(fh.yytext),gt.push(fh.yylloc),ra.push(r0[1]),D1=null,jO=fh.yyleng,It=fh.yytext,n0=fh.yylineno,rS=fh.yylloc;break;case 2:if(H2=this.productions_[r0[1]][1],w4.$=au[au.length-H2],w4._$={first_line:gt[gt.length-(H2||1)].first_line,last_line:gt[gt.length-1].last_line,first_column:gt[gt.length-(H2||1)].first_column,last_column:gt[gt.length-1].last_column},$O&&(w4._$.range=[gt[gt.length-(H2||1)].range[0],gt[gt.length-1].range[1]]),iS=this.performAction.apply(w4,[It,jO,n0,Qm.yy,r0[1],au,gt].concat(iq)),typeof iS<"u")return iS;H2&&(ra=ra.slice(0,-1*H2*2),au=au.slice(0,-1*H2),gt=gt.slice(0,-1*H2)),ra.push(this.productions_[r0[1]][0]),au.push(w4.$),gt.push(w4._$),HO=O6[ra[ra.length-2]][ra[ra.length-1]],ra.push(HO);break;case 3:return!0}}return!0}},bd=function(){var tg={EOF:1,parseError:function(Bi,ra){if(this.yy.parser)this.yy.parser.parseError(Bi,ra);else throw new Error(Bi)},setInput:function(Di,Bi){return this.yy=Bi||this.yy||{},this._input=Di,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var Di=this._input[0];this.yytext+=Di,this.yyleng++,this.offset++,this.match+=Di,this.matched+=Di;var Bi=Di.match(/(?:\r\n?|\n).*/g);return Bi?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),Di},unput:function(Di){var Bi=Di.length,ra=Di.split(/(?:\r\n?|\n)/g);this._input=Di+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-Bi),this.offset-=Bi;var Rr=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),ra.length-1&&(this.yylineno-=ra.length-1);var au=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:ra?(ra.length===Rr.length?this.yylloc.first_column:0)+Rr[Rr.length-ra.length].length-ra[0].length:this.yylloc.first_column-Bi},this.options.ranges&&(this.yylloc.range=[au[0],au[0]+this.yyleng-Bi]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(Di){this.unput(this.match.slice(Di))},pastInput:function(){var Di=this.matched.substr(0,this.matched.length-this.match.length);return(Di.length>20?"...":"")+Di.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var Di=this.match;return Di.length<20&&(Di+=this._input.substr(0,20-Di.length)),(Di.substr(0,20)+(Di.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var Di=this.pastInput(),Bi=new Array(Di.length+1).join("-");return Di+this.upcomingInput()+`
`+Bi+"^"},test_match:function(Di,Bi){var ra,Rr,au;if(this.options.backtrack_lexer&&(au={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(au.yylloc.range=this.yylloc.range.slice(0))),Rr=Di[0].match(/(?:\r\n?|\n).*/g),Rr&&(this.yylineno+=Rr.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:Rr?Rr[Rr.length-1].length-Rr[Rr.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+Di[0].length},this.yytext+=Di[0],this.match+=Di[0],this.matches=Di,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(Di[0].length),this.matched+=Di[0],ra=this.performAction.call(this,this.yy,this,Bi,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),ra)return ra;if(this._backtrack){for(var gt in au)this[gt]=au[gt];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var Di,Bi,ra,Rr;this._more||(this.yytext="",this.match="");for(var au=this._currentRules(),gt=0;gt<au.length;gt++)if(ra=this._input.match(this.rules[au[gt]]),ra&&(!Bi||ra[0].length>Bi[0].length)){if(Bi=ra,Rr=gt,this.options.backtrack_lexer){if(Di=this.test_match(ra,au[gt]),Di!==!1)return Di;if(this._backtrack){Bi=!1;continue}else return!1}else if(!this.options.flex)break}return Bi?(Di=this.test_match(Bi,au[Rr]),Di!==!1?Di:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var Bi=this.next();return Bi||this.lex()},begin:function(Bi){this.conditionStack.push(Bi)},popState:function(){var Bi=this.conditionStack.length-1;return Bi>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(Bi){return Bi=this.conditionStack.length-1-Math.abs(Bi||0),Bi>=0?this.conditionStack[Bi]:"INITIAL"},pushState:function(Bi){this.begin(Bi)},stateStackSize:function(){return this.conditionStack.length},options:{},performAction:function(Bi,ra,Rr,au){switch(Rr){case 0:return this.begin("open_directive"),12;case 1:return this.begin("type_directive"),13;case 2:return this.popState(),this.begin("arg_directive"),10;case 3:return this.popState(),this.popState(),15;case 4:return 14;case 5:break;case 6:break;case 7:return this.begin("acc_title"),44;case 8:return this.popState(),"acc_title_value";case 9:return this.begin("acc_descr"),46;case 10:return this.popState(),"acc_descr_value";case 11:this.begin("acc_descr_multiline");break;case 12:this.popState();break;case 13:return"acc_descr_multiline_value";case 14:this.begin("string");break;case 15:this.popState();break;case 16:return"STR";case 17:return 86;case 18:return 95;case 19:return 87;case 20:return 104;case 21:return 88;case 22:return 89;case 23:this.begin("href");break;case 24:this.popState();break;case 25:return 100;case 26:this.begin("callbackname");break;case 27:this.popState();break;case 28:this.popState(),this.begin("callbackargs");break;case 29:return 98;case 30:this.popState();break;case 31:return 99;case 32:this.begin("click");break;case 33:this.popState();break;case 34:return 90;case 35:return Bi.lex.firstGraph()&&this.begin("dir"),24;case 36:return Bi.lex.firstGraph()&&this.begin("dir"),24;case 37:return Bi.lex.firstGraph()&&this.begin("dir"),24;case 38:return 38;case 39:return 42;case 40:return 101;case 41:return 101;case 42:return 101;case 43:return 101;case 44:return this.popState(),25;case 45:return this.popState(),26;case 46:return this.popState(),26;case 47:return this.popState(),26;case 48:return this.popState(),26;case 49:return this.popState(),26;case 50:return this.popState(),26;case 51:return this.popState(),26;case 52:return this.popState(),26;case 53:return this.popState(),26;case 54:return this.popState(),26;case 55:return 118;case 56:return 119;case 57:return 120;case 58:return 121;case 59:return 105;case 60:return 111;case 61:return 53;case 62:return 67;case 63:return 52;case 64:return 20;case 65:return 106;case 66:return 126;case 67:return 82;case 68:return 82;case 69:return 82;case 70:return 81;case 71:return 81;case 72:return 81;case 73:return 59;case 74:return 60;case 75:return 61;case 76:return 62;case 77:return 63;case 78:return 64;case 79:return 65;case 80:return 69;case 81:return 70;case 82:return 55;case 83:return 56;case 84:return 109;case 85:return 112;case 86:return 127;case 87:return 124;case 88:return 113;case 89:return 125;case 90:return 125;case 91:return 114;case 92:return 73;case 93:return 92;case 94:return"SEP";case 95:return 91;case 96:return 66;case 97:return 75;case 98:return 74;case 99:return 77;case 100:return 76;case 101:return 122;case 102:return 123;case 103:return 68;case 104:return 57;case 105:return 58;case 106:return 40;case 107:return 41;case 108:return 71;case 109:return 72;case 110:return 133;case 111:return 21;case 112:return 22;case 113:return 23}},rules:[/^(?:%%\{)/,/^(?:((?:(?!\}%%)[^:.])*))/,/^(?::)/,/^(?:\}%%)/,/^(?:((?:(?!\}%%).|\n)*))/,/^(?:%%(?!\{)[^\n]*)/,/^(?:[^\}]%%[^\n]*)/,/^(?:accTitle\s*:\s*)/,/^(?:(?!\n||)*[^\n]*)/,/^(?:accDescr\s*:\s*)/,/^(?:(?!\n||)*[^\n]*)/,/^(?:accDescr\s*\{\s*)/,/^(?:[\}])/,/^(?:[^\}]*)/,/^(?:["])/,/^(?:["])/,/^(?:[^"]*)/,/^(?:style\b)/,/^(?:default\b)/,/^(?:linkStyle\b)/,/^(?:interpolate\b)/,/^(?:classDef\b)/,/^(?:class\b)/,/^(?:href[\s]+["])/,/^(?:["])/,/^(?:[^"]*)/,/^(?:call[\s]+)/,/^(?:\([\s]*\))/,/^(?:\()/,/^(?:[^(]*)/,/^(?:\))/,/^(?:[^)]*)/,/^(?:click[\s]+)/,/^(?:[\s\n])/,/^(?:[^\s\n]*)/,/^(?:flowchart-elk\b)/,/^(?:graph\b)/,/^(?:flowchart\b)/,/^(?:subgraph\b)/,/^(?:end\b\s*)/,/^(?:_self\b)/,/^(?:_blank\b)/,/^(?:_parent\b)/,/^(?:_top\b)/,/^(?:(\r?\n)*\s*\n)/,/^(?:\s*LR\b)/,/^(?:\s*RL\b)/,/^(?:\s*TB\b)/,/^(?:\s*BT\b)/,/^(?:\s*TD\b)/,/^(?:\s*BR\b)/,/^(?:\s*<)/,/^(?:\s*>)/,/^(?:\s*\^)/,/^(?:\s*v\b)/,/^(?:.*direction\s+TB[^\n]*)/,/^(?:.*direction\s+BT[^\n]*)/,/^(?:.*direction\s+RL[^\n]*)/,/^(?:.*direction\s+LR[^\n]*)/,/^(?:[0-9]+)/,/^(?:#)/,/^(?::::)/,/^(?::)/,/^(?:&)/,/^(?:;)/,/^(?:,)/,/^(?:\*)/,/^(?:\s*[xo<]?--+[-xo>]\s*)/,/^(?:\s*[xo<]?==+[=xo>]\s*)/,/^(?:\s*[xo<]?-?\.+-[xo>]?\s*)/,/^(?:\s*[xo<]?--\s*)/,/^(?:\s*[xo<]?==\s*)/,/^(?:\s*[xo<]?-\.\s*)/,/^(?:\(-)/,/^(?:-\))/,/^(?:\(\[)/,/^(?:\]\))/,/^(?:\[\[)/,/^(?:\]\])/,/^(?:\[\|)/,/^(?:\[\()/,/^(?:\)\])/,/^(?:\(\(\()/,/^(?:\)\)\))/,/^(?:-)/,/^(?:\.)/,/^(?:[\_])/,/^(?:\+)/,/^(?:%)/,/^(?:=)/,/^(?:=)/,/^(?:<)/,/^(?:>)/,/^(?:\^)/,/^(?:\\\|)/,/^(?:v\b)/,/^(?:[A-Za-z]+)/,/^(?:\\\])/,/^(?:\[\/)/,/^(?:\/\])/,/^(?:\[\\)/,/^(?:[!"#$%&'*+,-.`?\\_/])/,/^(?:[\u00AA\u00B5\u00BA\u00C0-\u00D6\u00D8-\u00F6]|[\u00F8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377]|[\u037A-\u037D\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5]|[\u03F7-\u0481\u048A-\u0527\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA]|[\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE]|[\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA]|[\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0]|[\u08A2-\u08AC\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0977]|[\u0979-\u097F\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2]|[\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A]|[\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39]|[\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8]|[\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0B05-\u0B0C]|[\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C]|[\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99]|[\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0]|[\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C33\u0C35-\u0C39\u0C3D]|[\u0C58\u0C59\u0C60\u0C61\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3]|[\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10]|[\u0D12-\u0D3A\u0D3D\u0D4E\u0D60\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1]|[\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81]|[\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3]|[\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6]|[\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A]|[\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081]|[\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D]|[\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0]|[\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310]|[\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F4\u1401-\u166C]|[\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u1700-\u170C\u170E-\u1711]|[\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7]|[\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191C]|[\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19C1-\u19C7\u1A00-\u1A16]|[\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF]|[\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1CE9-\u1CEC]|[\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D]|[\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D]|[\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3]|[\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F]|[\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128]|[\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2183\u2184]|[\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3]|[\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6]|[\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE]|[\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005\u3006\u3031-\u3035\u303B\u303C]|[\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D]|[\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FCC]|[\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B]|[\uA640-\uA66E\uA67F-\uA697\uA6A0-\uA6E5\uA717-\uA71F\uA722-\uA788]|[\uA78B-\uA78E\uA790-\uA793\uA7A0-\uA7AA\uA7F8-\uA801\uA803-\uA805]|[\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB]|[\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uAA00-\uAA28]|[\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA80-\uAAAF\uAAB1\uAAB5]|[\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4]|[\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E]|[\uABC0-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D]|[\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36]|[\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D]|[\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC]|[\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF]|[\uFFD2-\uFFD7\uFFDA-\uFFDC])/,/^(?:\|)/,/^(?:\()/,/^(?:\))/,/^(?:\[)/,/^(?:\])/,/^(?:\{)/,/^(?:\})/,/^(?:")/,/^(?:(\r?\n)+)/,/^(?:\s)/,/^(?:$)/],conditions:{close_directive:{rules:[],inclusive:!1},arg_directive:{rules:[3,4],inclusive:!1},type_directive:{rules:[2,3],inclusive:!1},open_directive:{rules:[1],inclusive:!1},callbackargs:{rules:[30,31],inclusive:!1},callbackname:{rules:[27,28,29],inclusive:!1},href:{rules:[24,25],inclusive:!1},click:{rules:[33,34],inclusive:!1},vertex:{rules:[],inclusive:!1},dir:{rules:[44,45,46,47,48,49,50,51,52,53,54],inclusive:!1},acc_descr_multiline:{rules:[12,13],inclusive:!1},acc_descr:{rules:[10],inclusive:!1},acc_title:{rules:[8],inclusive:!1},string:{rules:[15,16],inclusive:!1},INITIAL:{rules:[0,5,6,7,9,11,14,17,18,19,20,21,22,23,26,32,35,36,37,38,39,40,41,42,43,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113],inclusive:!0}}};return tg}();Lc.lexer=bd;function hh(){this.yy={}}return hh.prototype=Lc,Lc.Parser=hh,new hh}();Coe.parser=Coe;const Soe=Coe,aXt=(r,i)=>{var o,l;return((o=i==null?void 0:i.flowchart)==null?void 0:o.defaultRenderer)==="dagre-wrapper"||((l=i==null?void 0:i.flowchart)==null?void 0:l.defaultRenderer)==="elk"?!1:r.match(/^\s*graph/)!==null},oXt=(r,i)=>{var o,l;return((o=i==null?void 0:i.flowchart)==null?void 0:o.defaultRenderer)==="dagre-d3"||((l=i==null?void 0:i.flowchart)==null?void 0:l.defaultRenderer)==="elk"?!1:r.match(/^\s*graph/)!==null?!0:r.match(/^\s*flowchart/)!==null},cXt="flowchart-";let iDe=0,Aoe=ht(),Pu={},h7=[],wC={},r4=[],NV={},Moe={},PV=0,Doe=!0,P2,BV,FV=[];const RV=r=>Cs.sanitizeText(r,Aoe),sDe=function(r,i,o){Bu.parseDirective(this,r,i,o)},RI=function(r){const i=Object.keys(Pu);for(const o of i)if(Pu[o].id===r)return Pu[o].domId;return r},aDe=function(r,i,o,l,f,b,d={}){let w,y=r;y!==void 0&&y.trim().length!==0&&(Pu[y]===void 0&&(Pu[y]={id:y,domId:cXt+y+"-"+iDe,styles:[],classes:[]}),iDe++,i!==void 0?(Aoe=ht(),w=RV(i.trim()),w[0]==='"'&&w[w.length-1]==='"'&&(w=w.substring(1,w.length-1)),Pu[y].text=w):Pu[y].text===void 0&&(Pu[y].text=r),o!==void 0&&(Pu[y].type=o),l!=null&&l.forEach(function(k){Pu[y].styles.push(k)}),f!=null&&f.forEach(function(k){Pu[y].classes.push(k)}),b!==void 0&&(Pu[y].dir=b),Pu[y].props===void 0?Pu[y].props=d:d!==void 0&&Object.assign(Pu[y].props,d))},oDe=function(r,i,o,l){const d={start:r,end:i,type:void 0,text:""};l=o.text,l!==void 0&&(d.text=RV(l.trim()),d.text[0]==='"'&&d.text[d.text.length-1]==='"'&&(d.text=d.text.substring(1,d.text.length-1))),o!==void 0&&(d.type=o.type,d.stroke=o.stroke,d.length=o.length),h7.push(d)},cDe=function(r,i,o,l){let f,b;for(f=0;f<r.length;f++)for(b=0;b<i.length;b++)oDe(r[f],i[b],o,l)},uDe=function(r,i){r.forEach(function(o){o==="default"?h7.defaultInterpolate=i:h7[o].interpolate=i})},lDe=function(r,i){r.forEach(function(o){o==="default"?h7.defaultStyle=i:(Fs.isSubstringInArray("fill",i)===-1&&i.push("fill:none"),h7[o].style=i)})},hDe=function(r,i){wC[r]===void 0&&(wC[r]={id:r,styles:[],textStyles:[]}),i!=null&&i.forEach(function(o){if(o.match("color")){const f=o.replace("fill","bgFill").replace("color","fill");wC[r].textStyles.push(f)}wC[r].styles.push(o)})},fDe=function(r){P2=r,P2.match(/.*</)&&(P2="RL"),P2.match(/.*\^/)&&(P2="BT"),P2.match(/.*>/)&&(P2="LR"),P2.match(/.*v/)&&(P2="TB"),P2==="TD"&&(P2="TB")},jV=function(r,i){r.split(",").forEach(function(o){let l=o;Pu[l]!==void 0&&Pu[l].classes.push(i),NV[l]!==void 0&&NV[l].classes.push(i)})},uXt=function(r,i){r.split(",").forEach(function(o){i!==void 0&&(Moe[BV==="gen-1"?RI(o):o]=RV(i))})},lXt=function(r,i,o){let l=RI(r);if(ht().securityLevel!=="loose"||i===void 0)return;let f=[];if(typeof o=="string"){f=o.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);for(let b=0;b<f.length;b++){let d=f[b].trim();d.charAt(0)==='"'&&d.charAt(d.length-1)==='"'&&(d=d.substr(1,d.length-2)),f[b]=d}}f.length===0&&f.push(r),Pu[r]!==void 0&&(Pu[r].haveCallback=!0,FV.push(function(){const b=document.querySelector(`[id="${l}"]`);b!==null&&b.addEventListener("click",function(){Fs.runFunc(i,...f)},!1)}))},dDe=function(r,i,o){r.split(",").forEach(function(l){Pu[l]!==void 0&&(Pu[l].link=Fs.formatUrl(i,Aoe),Pu[l].linkTarget=o)}),jV(r,"clickable")},gDe=function(r){return Moe[r]},pDe=function(r,i,o){r.split(",").forEach(function(l){lXt(l,i,o)}),jV(r,"clickable")},bDe=function(r){FV.forEach(function(i){i(r)})},vDe=function(){return P2.trim()},wDe=function(){return Pu},mDe=function(){return h7},yDe=function(){return wC},kDe=function(r){let i=Cn(".mermaidTooltip");(i._groups||i)[0][0]===null&&(i=Cn("body").append("div").attr("class","mermaidTooltip").style("opacity",0)),Cn(r).select("svg").selectAll("g.node").on("mouseover",function(){const f=Cn(this);if(f.attr("title")===null)return;const d=this.getBoundingClientRect();i.transition().duration(200).style("opacity",".9"),i.text(f.attr("title")).style("left",window.scrollX+d.left+(d.right-d.left)/2+"px").style("top",window.scrollY+d.top-14+document.body.scrollTop+"px"),i.html(i.html().replace(/&lt;br\/&gt;/g,"<br/>")),f.classed("hover",!0)}).on("mouseout",function(){i.transition().duration(500).style("opacity",0),Cn(this).classed("hover",!1)})};FV.push(kDe);const xDe=function(r="gen-1"){Pu={},wC={},h7=[],FV=[kDe],r4=[],NV={},PV=0,Moe=[],Doe=!0,BV=r,W0()},EDe=r=>{BV=r||"gen-2"},_De=function(){return"fill:#ffa;stroke: #f66; stroke-width: 3px; stroke-dasharray: 5, 5;fill:#ffa;stroke: #666;"},TDe=function(r,i,o){let l=r.trim(),f=o;r===o&&o.match(/\s/)&&(l=void 0);function b(E){const T={boolean:{},number:{},string:{}},C=[];let S;return{nodeList:E.filter(function(O){const B=typeof O;return O.stmt&&O.stmt==="dir"?(S=O.value,!1):O.trim()===""?!1:B in T?T[B].hasOwnProperty(O)?!1:T[B][O]=!0:C.includes(O)?!1:C.push(O)}),dir:S}}let d=[];const{nodeList:w,dir:y}=b(d.concat.apply(d,i));if(d=w,BV==="gen-1")for(let E=0;E<d.length;E++)d[E]=RI(d[E]);l=l||"subGraph"+PV,f=f||"",f=RV(f),PV=PV+1;const k={id:l,nodes:d,title:f.trim(),classes:[],dir:y};return Se.info("Adding",k.id,k.nodes,k.dir),k.nodes=NDe(k,r4).nodes,r4.push(k),NV[l]=k,l},hXt=function(r){for(const[i,o]of r4.entries())if(o.id===r)return i;return-1};let jI=-1;const CDe=[],SDe=function(r,i){const o=r4[i].nodes;if(jI=jI+1,jI>2e3)return;if(CDe[jI]=i,r4[i].id===r)return{result:!0,count:0};let l=0,f=1;for(;l<o.length;){const b=hXt(o[l]);if(b>=0){const d=SDe(r,b);if(d.result)return{result:!0,count:f+d.count};f=f+d.count}l=l+1}return{result:!1,count:f}},ADe=function(r){return CDe[r]},MDe=function(){jI=-1,r4.length>0&&SDe("none",r4.length-1)},DDe=function(){return r4},LDe=()=>Doe?(Doe=!1,!0):!1,fXt=r=>{let i=r.trim(),o="arrow_open";switch(i[0]){case"<":o="arrow_point",i=i.slice(1);break;case"x":o="arrow_cross",i=i.slice(1);break;case"o":o="arrow_circle",i=i.slice(1);break}let l="normal";return i.includes("=")&&(l="thick"),i.includes(".")&&(l="dotted"),{type:o,stroke:l}},dXt=(r,i)=>{const o=i.length;let l=0;for(let f=0;f<o;++f)i[f]===r&&++l;return l},gXt=r=>{const i=r.trim();let o=i.slice(0,-1),l="arrow_open";switch(i.slice(-1)){case"x":l="arrow_cross",i[0]==="x"&&(l="double_"+l,o=o.slice(1));break;case">":l="arrow_point",i[0]==="<"&&(l="double_"+l,o=o.slice(1));break;case"o":l="arrow_circle",i[0]==="o"&&(l="double_"+l,o=o.slice(1));break}let f="normal",b=o.length-1;o[0]==="="&&(f="thick");let d=dXt(".",o);return d&&(f="dotted",b=d),{type:l,stroke:f,length:b}},IDe=(r,i)=>{const o=gXt(r);let l;if(i){if(l=fXt(i),l.stroke!==o.stroke)return{type:"INVALID",stroke:"INVALID"};if(l.type==="arrow_open")l.type=o.type;else{if(l.type!==o.type)return{type:"INVALID",stroke:"INVALID"};l.type="double_"+l.type}return l.type==="double_arrow"&&(l.type="double_arrow_point"),l.length=o.length,l}return o},ODe=(r,i)=>{let o=!1;return r.forEach(l=>{l.nodes.indexOf(i)>=0&&(o=!0)}),o},NDe=(r,i)=>{const o=[];return r.nodes.forEach((l,f)=>{ODe(i,l)||o.push(r.nodes[f])}),{nodes:o}},PDe={firstGraph:LDe},$m={parseDirective:sDe,defaultConfig:()=>nC.flowchart,setAccTitle:K0,getAccTitle:hp,getAccDescription:dp,setAccDescription:fp,addVertex:aDe,lookUpDomId:RI,addLink:cDe,updateLinkInterpolate:uDe,updateLink:lDe,addClass:hDe,setDirection:fDe,setClass:jV,setTooltip:uXt,getTooltip:gDe,setClickEvent:pDe,setLink:dDe,bindFunctions:bDe,getDirection:vDe,getVertices:wDe,getEdges:mDe,getClasses:yDe,clear:xDe,setGen:EDe,defaultStyle:_De,addSubGraph:TDe,getDepthFirstPos:ADe,indexNodes:MDe,getSubGraphs:DDe,destructLink:IDe,lex:PDe,exists:ODe,makeUniq:NDe,setDiagramTitle:Ov,getDiagramTitle:Nv},pXt=Object.freeze(Object.defineProperty({__proto__:null,parseDirective:sDe,lookUpDomId:RI,addVertex:aDe,addSingleLink:oDe,addLink:cDe,updateLinkInterpolate:uDe,updateLink:lDe,addClass:hDe,setDirection:fDe,setClass:jV,setLink:dDe,getTooltip:gDe,setClickEvent:pDe,bindFunctions:bDe,getDirection:vDe,getVertices:wDe,getEdges:mDe,getClasses:yDe,clear:xDe,setGen:EDe,defaultStyle:_De,addSubGraph:TDe,getDepthFirstPos:ADe,indexNodes:MDe,getSubGraphs:DDe,firstGraph:LDe,destructLink:IDe,lex:PDe,default:$m},Symbol.toStringTag,{value:"Module"}));function BDe(r,i){return!!r.children(i).length}function FDe(r){return Loe(r.v)+":"+Loe(r.w)+":"+Loe(r.name)}var bXt=/:/g;function Loe(r){return r?String(r).replace(bXt,"\\:"):""}function Hm(r,i){i&&r.attr("style",i)}function RDe(r,i,o){i&&r.attr("class",i).attr("class",o+" "+r.attr("class"))}function $v(r,i){var o=i.graph();if(pAe(o)){var l=o.transition;if(QT(l))return l(r)}return r}var Ioe={normal:wXt,vee:mXt,undirected:yXt};function vXt(r){Ioe=r}function wXt(r,i,o,l){var f=r.append("marker").attr("id",i).attr("viewBox","0 0 10 10").attr("refX",9).attr("refY",5).attr("markerUnits","strokeWidth").attr("markerWidth",8).attr("markerHeight",6).attr("orient","auto"),b=f.append("path").attr("d","M 0 0 L 10 5 L 0 10 z").style("stroke-width",1).style("stroke-dasharray","1,0");Hm(b,o[l+"Style"]),o[l+"Class"]&&b.attr("class",o[l+"Class"])}function mXt(r,i,o,l){var f=r.append("marker").attr("id",i).attr("viewBox","0 0 10 10").attr("refX",9).attr("refY",5).attr("markerUnits","strokeWidth").attr("markerWidth",8).attr("markerHeight",6).attr("orient","auto"),b=f.append("path").attr("d","M 0 0 L 10 5 L 0 10 L 4 5 z").style("stroke-width",1).style("stroke-dasharray","1,0");Hm(b,o[l+"Style"]),o[l+"Class"]&&b.attr("class",o[l+"Class"])}function yXt(r,i,o,l){var f=r.append("marker").attr("id",i).attr("viewBox","0 0 10 10").attr("refX",9).attr("refY",5).attr("markerUnits","strokeWidth").attr("markerWidth",8).attr("markerHeight",6).attr("orient","auto"),b=f.append("path").attr("d","M 0 5 L 10 5").style("stroke-width",1).style("stroke-dasharray","1,0");Hm(b,o[l+"Style"]),o[l+"Class"]&&b.attr("class",o[l+"Class"])}function $V(r,i){var o=r.append("foreignObject").attr("width","100000"),l=o.append("xhtml:div");l.attr("xmlns","http://www.w3.org/1999/xhtml");var f=i.label;switch(typeof f){case"function":l.insert(f);break;case"object":l.insert(function(){return f});break;default:l.html(f)}Hm(l,i.labelStyle),l.style("display","inline-block"),l.style("white-space","nowrap");var b=l.node().getBoundingClientRect();return o.attr("width",b.width).attr("height",b.height),o}function kXt(r,i){var o=r;return o.node().appendChild(i.label),Hm(o,i.labelStyle),o}function xXt(r,i){for(var o=r.append("text"),l=EXt(i.label).split(`
`),f=0;f<l.length;f++)o.append("tspan").attr("xml:space","preserve").attr("dy","1em").attr("x","1").text(l[f]);return Hm(o,i.labelStyle),o}function EXt(r){for(var i="",o=!1,l,f=0;f<r.length;++f)if(l=r[f],o){switch(l){case"n":i+=`
`;break;default:i+=l}o=!1}else l==="\\"?o=!0:i+=l;return i}function Ooe(r,i,o){var l=i.label,f=r.append("g");i.labelType==="svg"?kXt(f,i):typeof l!="string"||i.labelType==="html"?$V(f,i):xXt(f,i);var b=f.node().getBBox(),d;switch(o){case"top":d=-i.height/2;break;case"bottom":d=i.height/2-b.height;break;default:d=-b.height/2}return f.attr("transform","translate("+-b.width/2+","+d+")"),f}var Noe=function(r,i){var o=i.nodes().filter(function(b){return BDe(i,b)}),l=r.selectAll("g.cluster").data(o,function(b){return b});$v(l.exit(),i).style("opacity",0).remove();var f=l.enter().append("g").attr("class","cluster").attr("id",function(b){var d=i.node(b);return d.id}).style("opacity",0).each(function(b){var d=i.node(b),w=Cn(this);Cn(this).append("rect");var y=w.append("g").attr("class","label");Ooe(y,d,d.clusterLabelPos)});return l=l.merge(f),l=$v(l,i).style("opacity",1),l.selectAll("rect").each(function(b){var d=i.node(b),w=Cn(this);Hm(w,d.style)}),l};function _Xt(r){Noe=r}let Poe=function(r,i){var o=r.selectAll("g.edgeLabel").data(i.edges(),function(f){return FDe(f)}).classed("update",!0);o.exit().remove(),o.enter().append("g").classed("edgeLabel",!0).style("opacity",0),o=r.selectAll("g.edgeLabel"),o.each(function(f){var b=Cn(this);b.select(".label").remove();var d=i.edge(f),w=Ooe(b,i.edge(f),0).classed("label",!0),y=w.node().getBBox();d.labelId&&w.attr("id",d.labelId),Ks(d,"width")||(d.width=y.width),Ks(d,"height")||(d.height=y.height)});var l;return o.exit?l=o.exit():l=o.selectAll(null),$v(l,i).style("opacity",0).remove(),o};function TXt(r){Poe=r}function jDe(r,i){return r.intersect(i)}var Boe=function(r,i,o){var l=r.selectAll("g.edgePath").data(i.edges(),function(d){return FDe(d)}).classed("update",!0),f=DXt(l,i);LXt(l,i);var b=l.merge!==void 0?l.merge(f):l;return $v(b,i).style("opacity",1),b.each(function(d){var w=Cn(this),y=i.edge(d);y.elem=this,y.id&&w.attr("id",y.id),RDe(w,y.class,(w.classed("update")?"update ":"")+"edgePath")}),b.selectAll("path.path").each(function(d){var w=i.edge(d);w.arrowheadId=TV("arrowhead");var y=Cn(this).attr("marker-end",function(){return"url("+SXt(location.href,w.arrowheadId)+")"}).style("fill","none");$v(y,i).attr("d",function(k){return AXt(i,k)}),Hm(y,w.style)}),b.selectAll("defs *").remove(),b.selectAll("defs").each(function(d){var w=i.edge(d),y=o[w.arrowhead];y(Cn(this),w.arrowheadId,w,"arrowhead")}),b};function CXt(r){Boe=r}function SXt(r,i){var o=r.split("#")[0];return o+"#"+i}function AXt(r,i){var o=r.edge(i),l=r.node(i.v),f=r.node(i.w),b=o.points.slice(1,o.points.length-1);return b.unshift(jDe(l,b[0])),b.push(jDe(f,b[b.length-1])),$De(o,b)}function $De(r,i){var o=(V8||FIt.line)().x(function(l){return l.x}).y(function(l){return l.y});return(o.curve||o.interpolate)(r.curve),o(i)}function MXt(r){var i=r.getBBox(),o=r.ownerSVGElement.getScreenCTM().inverse().multiply(r.getScreenCTM()).translate(i.width/2,i.height/2);return{x:o.e,y:o.f}}function DXt(r,i){var o=r.enter().append("g").attr("class","edgePath").style("opacity",0);return o.append("path").attr("class","path").attr("d",function(l){var f=i.edge(l),b=i.node(l.v).elem,d=c7(f.points.length).map(function(){return MXt(b)});return $De(f,d)}),o.append("defs"),o}function LXt(r,i){var o=r.exit();$v(o,i).style("opacity",0).remove()}var Foe=function(r,i,o){var l=i.nodes().filter(function(d){return!BDe(i,d)}),f=r.selectAll("g.node").data(l,function(d){return d}).classed("update",!0);f.exit().remove(),f.enter().append("g").attr("class","node").style("opacity",0),f=r.selectAll("g.node"),f.each(function(d){var w=i.node(d),y=Cn(this);RDe(y,w.class,(y.classed("update")?"update ":"")+"node"),y.select("g.label").remove();var k=y.append("g").attr("class","label"),E=Ooe(k,w),T=o[w.shape],C=AI(E.node().getBBox(),"width","height");w.elem=this,w.id&&y.attr("id",w.id),w.labelId&&k.attr("id",w.labelId),Ks(w,"width")&&(C.width=w.width),Ks(w,"height")&&(C.height=w.height),C.width+=w.paddingLeft+w.paddingRight,C.height+=w.paddingTop+w.paddingBottom,k.attr("transform","translate("+(w.paddingLeft-w.paddingRight)/2+","+(w.paddingTop-w.paddingBottom)/2+")");var S=Cn(this);S.select(".label-container").remove();var L=T(S,C,w).classed("label-container",!0);Hm(L,w.style);var O=L.node().getBBox();w.width=O.width,w.height=O.height});var b;return f.exit?b=f.exit():b=f.selectAll(null),$v(b,i).style("opacity",0).remove(),f};function IXt(r){Foe=r}function OXt(r,i){var o=r.filter(function(){return!Cn(this).classed("update")});function l(f){var b=i.node(f);return"translate("+b.x+","+b.y+")"}o.attr("transform",l),$v(r,i).style("opacity",1).attr("transform",l),$v(o.selectAll("rect"),i).attr("width",function(f){return i.node(f).width}).attr("height",function(f){return i.node(f).height}).attr("x",function(f){var b=i.node(f);return-b.width/2}).attr("y",function(f){var b=i.node(f);return-b.height/2})}function NXt(r,i){var o=r.filter(function(){return!Cn(this).classed("update")});function l(f){var b=i.edge(f);return Ks(b,"x")?"translate("+b.x+","+b.y+")":""}o.attr("transform",l),$v(r,i).style("opacity",1).attr("transform",l)}function PXt(r,i){var o=r.filter(function(){return!Cn(this).classed("update")});function l(f){var b=i.node(f);return"translate("+b.x+","+b.y+")"}o.attr("transform",l),$v(r,i).style("opacity",1).attr("transform",l)}function HDe(r,i,o,l){var f=r.x,b=r.y,d=f-l.x,w=b-l.y,y=Math.sqrt(i*i*w*w+o*o*d*d),k=Math.abs(i*o*d/y);l.x<f&&(k=-k);var E=Math.abs(i*o*w/y);return l.y<b&&(E=-E),{x:f+k,y:b+E}}function BXt(r,i,o){return HDe(r,i,i,o)}function FXt(r,i,o,l){var f,b,d,w,y,k,E,T,C,S,L,O,B,N,F;if(f=i.y-r.y,d=r.x-i.x,y=i.x*r.y-r.x*i.y,C=f*o.x+d*o.y+y,S=f*l.x+d*l.y+y,!(C!==0&&S!==0&&zDe(C,S))&&(b=l.y-o.y,w=o.x-l.x,k=l.x*o.y-o.x*l.y,E=b*r.x+w*r.y+k,T=b*i.x+w*i.y+k,!(E!==0&&T!==0&&zDe(E,T))&&(L=f*w-b*d,L!==0)))return O=Math.abs(L/2),B=d*k-w*y,N=B<0?(B-O)/L:(B+O)/L,B=b*y-f*k,F=B<0?(B-O)/L:(B+O)/L,{x:N,y:F}}function zDe(r,i){return r*i>0}function zm(r,i,o){var l=r.x,f=r.y,b=[],d=Number.POSITIVE_INFINITY,w=Number.POSITIVE_INFINITY;i.forEach(function(L){d=Math.min(d,L.x),w=Math.min(w,L.y)});for(var y=l-r.width/2-d,k=f-r.height/2-w,E=0;E<i.length;E++){var T=i[E],C=i[E<i.length-1?E+1:0],S=FXt(r,o,{x:y+T.x,y:k+T.y},{x:y+C.x,y:k+C.y});S&&b.push(S)}return b.length?(b.length>1&&b.sort(function(L,O){var B=L.x-o.x,N=L.y-o.y,F=Math.sqrt(B*B+N*N),R=O.x-o.x,q=O.y-o.y,X=Math.sqrt(R*R+q*q);return F<X?-1:F===X?0:1}),b[0]):(console.log("NO INTERSECTION FOUND, RETURN NODE CENTER",r),r)}function Roe(r,i){var o=r.x,l=r.y,f=i.x-o,b=i.y-l,d=r.width/2,w=r.height/2,y,k;return Math.abs(b)*d>Math.abs(f)*w?(b<0&&(w=-w),y=b===0?0:w*f/b,k=w):(f<0&&(d=-d),y=d,k=f===0?0:d*b/f),{x:o+y,y:l+k}}var joe={rect:jXt,ellipse:$Xt,circle:HXt,diamond:zXt};function RXt(r){joe=r}function jXt(r,i,o){var l=r.insert("rect",":first-child").attr("rx",o.rx).attr("ry",o.ry).attr("x",-i.width/2).attr("y",-i.height/2).attr("width",i.width).attr("height",i.height);return o.intersect=function(f){return Roe(o,f)},l}function $Xt(r,i,o){var l=i.width/2,f=i.height/2,b=r.insert("ellipse",":first-child").attr("x",-i.width/2).attr("y",-i.height/2).attr("rx",l).attr("ry",f);return o.intersect=function(d){return HDe(o,l,f,d)},b}function HXt(r,i,o){var l=Math.max(i.width,i.height)/2,f=r.insert("circle",":first-child").attr("x",-i.width/2).attr("y",-i.height/2).attr("r",l);return o.intersect=function(b){return BXt(o,l,b)},f}function zXt(r,i,o){var l=i.width*Math.SQRT2/2,f=i.height*Math.SQRT2/2,b=[{x:0,y:-f},{x:-l,y:0},{x:0,y:f},{x:l,y:0}],d=r.insert("polygon",":first-child").attr("points",b.map(function(w){return w.x+","+w.y}).join(" "));return o.intersect=function(w){return zm(o,b,w)},d}function GXt(){var r=function(i,o){qXt(o);var l=$I(i,"output"),f=$I(l,"clusters"),b=$I(l,"edgePaths"),d=Poe($I(l,"edgeLabels"),o),w=Foe($I(l,"nodes"),o,joe);bC(o),PXt(w,o),NXt(d,o),Boe(b,o,Ioe);var y=Noe(f,o);OXt(y,o),YXt(o)};return r.createNodes=function(i){return arguments.length?(IXt(i),r):Foe},r.createClusters=function(i){return arguments.length?(_Xt(i),r):Noe},r.createEdgeLabels=function(i){return arguments.length?(TXt(i),r):Poe},r.createEdgePaths=function(i){return arguments.length?(CXt(i),r):Boe},r.shapes=function(i){return arguments.length?(RXt(i),r):joe},r.arrows=function(i){return arguments.length?(vXt(i),r):Ioe},r}var VXt={paddingLeft:10,paddingRight:10,paddingTop:10,paddingBottom:10,rx:0,ry:0,shape:"rect"},UXt={arrowhead:"normal",curve:x1};function qXt(r){r.nodes().forEach(function(i){var o=r.node(i);!Ks(o,"label")&&!r.children(i).length&&(o.label=i),Ks(o,"paddingX")&&dC(o,{paddingLeft:o.paddingX,paddingRight:o.paddingX}),Ks(o,"paddingY")&&dC(o,{paddingTop:o.paddingY,paddingBottom:o.paddingY}),Ks(o,"padding")&&dC(o,{paddingLeft:o.padding,paddingRight:o.padding,paddingTop:o.padding,paddingBottom:o.padding}),dC(o,VXt),Mn(["paddingLeft","paddingRight","paddingTop","paddingBottom"],function(l){o[l]=Number(o[l])}),Ks(o,"width")&&(o._prevWidth=o.width),Ks(o,"height")&&(o._prevHeight=o.height)}),r.edges().forEach(function(i){var o=r.edge(i);Ks(o,"label")||(o.label=""),dC(o,UXt)})}function YXt(r){Mn(r.nodes(),function(i){var o=r.node(i);Ks(o,"_prevWidth")?o.width=o._prevWidth:delete o.width,Ks(o,"_prevHeight")?o.height=o._prevHeight:delete o.height,delete o._prevWidth,delete o._prevHeight})}function $I(r,i){var o=r.select("g."+i);return o.empty()&&(o=r.append("g").attr("class",i)),o}function GDe(r,i,o){const l=i.width,f=i.height,b=(l+f)*.9,d=[{x:b/2,y:0},{x:b,y:-b/2},{x:b/2,y:-b},{x:0,y:-b/2}],w=i4(r,b,b,d);return o.intersect=function(y){return zm(o,d,y)},w}function VDe(r,i,o){const f=i.height,b=f/4,d=i.width+2*b,w=[{x:b,y:0},{x:d-b,y:0},{x:d,y:-f/2},{x:d-b,y:-f},{x:b,y:-f},{x:0,y:-f/2}],y=i4(r,d,f,w);return o.intersect=function(k){return zm(o,w,k)},y}function UDe(r,i,o){const l=i.width,f=i.height,b=[{x:-f/2,y:0},{x:l,y:0},{x:l,y:-f},{x:-f/2,y:-f},{x:0,y:-f/2}],d=i4(r,l,f,b);return o.intersect=function(w){return zm(o,b,w)},d}function qDe(r,i,o){const l=i.width,f=i.height,b=[{x:-2*f/6,y:0},{x:l-f/6,y:0},{x:l+2*f/6,y:-f},{x:f/6,y:-f}],d=i4(r,l,f,b);return o.intersect=function(w){return zm(o,b,w)},d}function YDe(r,i,o){const l=i.width,f=i.height,b=[{x:2*f/6,y:0},{x:l+f/6,y:0},{x:l-2*f/6,y:-f},{x:-f/6,y:-f}],d=i4(r,l,f,b);return o.intersect=function(w){return zm(o,b,w)},d}function WDe(r,i,o){const l=i.width,f=i.height,b=[{x:-2*f/6,y:0},{x:l+2*f/6,y:0},{x:l-f/6,y:-f},{x:f/6,y:-f}],d=i4(r,l,f,b);return o.intersect=function(w){return zm(o,b,w)},d}function KDe(r,i,o){const l=i.width,f=i.height,b=[{x:f/6,y:0},{x:l-f/6,y:0},{x:l+2*f/6,y:-f},{x:-2*f/6,y:-f}],d=i4(r,l,f,b);return o.intersect=function(w){return zm(o,b,w)},d}function XDe(r,i,o){const l=i.width,f=i.height,b=[{x:0,y:0},{x:l+f/2,y:0},{x:l,y:-f/2},{x:l+f/2,y:-f},{x:0,y:-f}],d=i4(r,l,f,b);return o.intersect=function(w){return zm(o,b,w)},d}function QDe(r,i,o){const l=i.height,f=i.width+l/4,b=r.insert("rect",":first-child").attr("rx",l/2).attr("ry",l/2).attr("x",-f/2).attr("y",-l/2).attr("width",f).attr("height",l);return o.intersect=function(d){return Roe(o,d)},b}function ZDe(r,i,o){const l=i.width,f=i.height,b=[{x:0,y:0},{x:l,y:0},{x:l,y:-f},{x:0,y:-f},{x:0,y:0},{x:-8,y:0},{x:l+8,y:0},{x:l+8,y:-f},{x:-8,y:-f},{x:-8,y:0}],d=i4(r,l,f,b);return o.intersect=function(w){return zm(o,b,w)},d}function JDe(r,i,o){const l=i.width,f=l/2,b=f/(2.5+l/50),d=i.height+b,w="M 0,"+b+" a "+f+","+b+" 0,0,0 "+l+" 0 a "+f+","+b+" 0,0,0 "+-l+" 0 l 0,"+d+" a "+f+","+b+" 0,0,0 "+l+" 0 l 0,"+-d,y=r.attr("label-offset-y",b).insert("path",":first-child").attr("d",w).attr("transform","translate("+-l/2+","+-(d/2+b)+")");return o.intersect=function(k){const E=Roe(o,k),T=E.x-o.x;if(f!=0&&(Math.abs(T)<o.width/2||Math.abs(T)==o.width/2&&Math.abs(E.y-o.y)>o.height/2-b)){let C=b*b*(1-T*T/(f*f));C!=0&&(C=Math.sqrt(C)),C=b-C,k.y-o.y>0&&(C=-C),E.y+=C}return E},y}function WXt(r){r.shapes().question=GDe,r.shapes().hexagon=VDe,r.shapes().stadium=QDe,r.shapes().subroutine=ZDe,r.shapes().cylinder=JDe,r.shapes().rect_left_inv_arrow=UDe,r.shapes().lean_right=qDe,r.shapes().lean_left=YDe,r.shapes().trapezoid=WDe,r.shapes().inv_trapezoid=KDe,r.shapes().rect_right_inv_arrow=XDe}function KXt(r){r({question:GDe}),r({hexagon:VDe}),r({stadium:QDe}),r({subroutine:ZDe}),r({cylinder:JDe}),r({rect_left_inv_arrow:UDe}),r({lean_right:qDe}),r({lean_left:YDe}),r({trapezoid:WDe}),r({inv_trapezoid:KDe}),r({rect_right_inv_arrow:XDe})}function i4(r,i,o,l){return r.insert("polygon",":first-child").attr("points",l.map(function(f){return f.x+","+f.y}).join(" ")).attr("transform","translate("+-i/2+","+o/2+")")}const XXt={addToRender:WXt,addToRenderV2:KXt},eLe={},QXt=function(r){const i=Object.keys(r);for(const o of i)eLe[o]=r[o]},tLe=function(r,i,o,l,f,b){const d=l?l.select(`[id="${o}"]`):Cn(`[id="${o}"]`),w=f||document;Object.keys(r).forEach(function(k){const E=r[k];let T="default";E.classes.length>0&&(T=E.classes.join(" "));const C=Dv(E.styles);let S=E.text!==void 0?E.text:E.id,L;if(cf(ht().flowchart.htmlLabels)){const N={label:S.replace(/fa[blrs]?:fa-[\w-]+/g,F=>`<i class='${F.replace(":"," ")}'></i>`)};L=$V(d,N).node(),L.parentNode.removeChild(L)}else{const N=w.createElementNS("http://www.w3.org/2000/svg","text");N.setAttribute("style",C.labelStyle.replace("color:","fill:"));const F=S.split(Cs.lineBreakRegex);for(const R of F){const q=w.createElementNS("http://www.w3.org/2000/svg","tspan");q.setAttributeNS("http://www.w3.org/XML/1998/namespace","xml:space","preserve"),q.setAttribute("dy","1em"),q.setAttribute("x","1"),q.textContent=R,N.appendChild(q)}L=N}let O=0,B="";switch(E.type){case"round":O=5,B="rect";break;case"square":B="rect";break;case"diamond":B="question";break;case"hexagon":B="hexagon";break;case"odd":B="rect_left_inv_arrow";break;case"lean_right":B="lean_right";break;case"lean_left":B="lean_left";break;case"trapezoid":B="trapezoid";break;case"inv_trapezoid":B="inv_trapezoid";break;case"odd_right":B="rect_left_inv_arrow";break;case"circle":B="circle";break;case"ellipse":B="ellipse";break;case"stadium":B="stadium";break;case"subroutine":B="subroutine";break;case"cylinder":B="cylinder";break;case"group":B="rect";break;default:B="rect"}Se.warn("Adding node",E.id,E.domId),i.setNode(b.db.lookUpDomId(E.id),{labelType:"svg",labelStyle:C.labelStyle,shape:B,label:L,rx:O,ry:O,class:T,style:C.style,id:b.db.lookUpDomId(E.id)})})},nLe=function(r,i,o){let l=0,f,b;if(r.defaultStyle!==void 0){const d=Dv(r.defaultStyle);f=d.style,b=d.labelStyle}r.forEach(function(d){l++;var w="L-"+d.start+"-"+d.end,y="LS-"+d.start,k="LE-"+d.end;const E={};d.type==="arrow_open"?E.arrowhead="none":E.arrowhead="normal";let T="",C="";if(d.style!==void 0){const S=Dv(d.style);T=S.style,C=S.labelStyle}else switch(d.stroke){case"normal":T="fill:none",f!==void 0&&(T=f),b!==void 0&&(C=b);break;case"dotted":T="fill:none;stroke-width:2px;stroke-dasharray:3;";break;case"thick":T=" stroke-width: 3.5px;fill:none";break}E.style=T,E.labelStyle=C,d.interpolate!==void 0?E.curve=q0(d.interpolate,x1):r.defaultInterpolate!==void 0?E.curve=q0(r.defaultInterpolate,x1):E.curve=q0(eLe.curve,x1),d.text===void 0?d.style!==void 0&&(E.arrowheadStyle="fill: #333"):(E.arrowheadStyle="fill: #333",E.labelpos="c",cf(ht().flowchart.htmlLabels)?(E.labelType="html",E.label=`<span id="L-${w}" class="edgeLabel L-${y}' L-${k}" style="${E.labelStyle}">${d.text.replace(/fa[blrs]?:fa-[\w-]+/g,S=>`<i class='${S.replace(":"," ")}'></i>`)}</span>`):(E.labelType="text",E.label=d.text.replace(Cs.lineBreakRegex,`
`),d.style===void 0&&(E.style=E.style||"stroke: #333; stroke-width: 1.5px;fill:none"),E.labelStyle=E.labelStyle.replace("color:","fill:"))),E.id=w,E.class=y+" "+k,E.minlen=d.length||1,i.setEdge(o.db.lookUpDomId(d.start),o.db.lookUpDomId(d.end),E,l)})},ZXt={setConf:QXt,addVertices:tLe,addEdges:nLe,getClasses:function(r,i){Se.info("Extracting classes"),i.db.clear();try{return i.parse(r),i.db.getClasses()}catch(o){return Se.error(o),{}}},draw:function(r,i,o,l){Se.info("Drawing flowchart"),l.db.clear();const{securityLevel:f,flowchart:b}=ht();let d;f==="sandbox"&&(d=Cn("#i"+i));const w=Cn(f==="sandbox"?d.nodes()[0].contentDocument.body:"body"),y=f==="sandbox"?d.nodes()[0].contentDocument:document;try{l.parser.parse(r)}catch{Se.debug("Parsing failed")}let k=l.db.getDirection();k===void 0&&(k="TD");const E=b.nodeSpacing||50,T=b.rankSpacing||50,C=new zh({multigraph:!0,compound:!0}).setGraph({rankdir:k,nodesep:E,ranksep:T,marginx:8,marginy:8}).setDefaultEdgeLabel(function(){return{}});let S;const L=l.db.getSubGraphs();for(let te=L.length-1;te>=0;te--)S=L[te],l.db.addVertex(S.id,S.title,"group",void 0,S.classes);const O=l.db.getVertices();Se.warn("Get vertices",O);const B=l.db.getEdges();let N=0;for(N=L.length-1;N>=0;N--){S=L[N],ose("cluster").append("text");for(let te=0;te<S.nodes.length;te++)Se.warn("Setting subgraph",S.nodes[te],l.db.lookUpDomId(S.nodes[te]),l.db.lookUpDomId(S.id)),C.setParent(l.db.lookUpDomId(S.nodes[te]),l.db.lookUpDomId(S.id))}tLe(O,C,i,w,y,l),nLe(B,C,l);const F=new GXt;XXt.addToRender(F),F.arrows().none=function(H,Y,z,W){const G=H.append("marker").attr("id",Y).attr("viewBox","0 0 10 10").attr("refX",9).attr("refY",5).attr("markerUnits","strokeWidth").attr("markerWidth",8).attr("markerHeight",6).attr("orient","auto").append("path").attr("d","M 0 0 L 0 0 L 0 0 z");Hm(G,z[W+"Style"])},F.arrows().normal=function(H,Y){H.append("marker").attr("id",Y).attr("viewBox","0 0 10 10").attr("refX",9).attr("refY",5).attr("markerUnits","strokeWidth").attr("markerWidth",8).attr("markerHeight",6).attr("orient","auto").append("path").attr("d","M 0 0 L 10 5 L 0 10 z").attr("class","arrowheadPath").style("stroke-width",1).style("stroke-dasharray","1,0")};const R=w.select(`[id="${i}"]`),q=w.select("#"+i+" g");for(F(q,C),q.selectAll("g.node").attr("title",function(){return l.db.getTooltip(this.id)}),l.db.indexNodes("subGraph"+N),N=0;N<L.length;N++)if(S=L[N],S.title!=="undefined"){const te=y.querySelectorAll("#"+i+' [id="'+l.db.lookUpDomId(S.id)+'"] rect'),H=y.querySelectorAll("#"+i+' [id="'+l.db.lookUpDomId(S.id)+'"]'),Y=te[0].x.baseVal.value,z=te[0].y.baseVal.value,W=te[0].width.baseVal.value,G=Cn(H[0]).select(".label");G.attr("transform",`translate(${Y+W/2}, ${z+14})`),G.attr("id",i+"Text");for(let ae=0;ae<S.classes.length;ae++)H[0].classList.add(S.classes[ae])}if(!b.htmlLabels){const te=y.querySelectorAll('[id="'+i+'"] .edgeLabel .label');for(const H of te){const Y=H.getBBox(),z=y.createElementNS("http://www.w3.org/2000/svg","rect");z.setAttribute("rx",0),z.setAttribute("ry",0),z.setAttribute("width",Y.width),z.setAttribute("height",Y.height),H.insertBefore(z,H.firstChild)}}J8(C,R,b.diagramPadding,b.useMaxWidth),Object.keys(O).forEach(function(te){const H=O[te];if(H.link){const Y=w.select("#"+i+' [id="'+l.db.lookUpDomId(te)+'"]');if(Y){const z=y.createElementNS("http://www.w3.org/2000/svg","a");z.setAttributeNS("http://www.w3.org/2000/svg","class",H.classes.join(" ")),z.setAttributeNS("http://www.w3.org/2000/svg","href",H.link),z.setAttributeNS("http://www.w3.org/2000/svg","rel","noopener"),f==="sandbox"?z.setAttributeNS("http://www.w3.org/2000/svg","target","_top"):H.linkTarget&&z.setAttributeNS("http://www.w3.org/2000/svg","target",H.linkTarget);const W=Y.insert(function(){return z},":first-child"),Z=Y.select(".label-container");Z&&W.append(function(){return Z.node()});const G=Y.select(".label");G&&W.append(function(){return G.node()})}}})}},rLe={},JXt=function(r){const i=Object.keys(r);for(const o of i)rLe[o]=r[o]},iLe=function(r,i,o,l,f,b){const d=l.select(`[id="${o}"]`);Object.keys(r).forEach(function(y){const k=r[y];let E="default";k.classes.length>0&&(E=k.classes.join(" "));const T=Dv(k.styles);let C=k.text!==void 0?k.text:k.id,S;if(cf(ht().flowchart.htmlLabels)){const B={label:C.replace(/fa[blrs]?:fa-[\w-]+/g,N=>`<i class='${N.replace(":"," ")}'></i>`)};S=$V(d,B).node(),S.parentNode.removeChild(S)}else{const B=f.createElementNS("http://www.w3.org/2000/svg","text");B.setAttribute("style",T.labelStyle.replace("color:","fill:"));const N=C.split(Cs.lineBreakRegex);for(const F of N){const R=f.createElementNS("http://www.w3.org/2000/svg","tspan");R.setAttributeNS("http://www.w3.org/XML/1998/namespace","xml:space","preserve"),R.setAttribute("dy","1em"),R.setAttribute("x","1"),R.textContent=F,B.appendChild(R)}S=B}let L=0,O="";switch(k.type){case"round":L=5,O="rect";break;case"square":O="rect";break;case"diamond":O="question";break;case"hexagon":O="hexagon";break;case"odd":O="rect_left_inv_arrow";break;case"lean_right":O="lean_right";break;case"lean_left":O="lean_left";break;case"trapezoid":O="trapezoid";break;case"inv_trapezoid":O="inv_trapezoid";break;case"odd_right":O="rect_left_inv_arrow";break;case"circle":O="circle";break;case"ellipse":O="ellipse";break;case"stadium":O="stadium";break;case"subroutine":O="subroutine";break;case"cylinder":O="cylinder";break;case"group":O="rect";break;case"doublecircle":O="doublecircle";break;default:O="rect"}i.setNode(k.id,{labelStyle:T.labelStyle,shape:O,labelText:C,rx:L,ry:L,class:E,style:T.style,id:k.id,link:k.link,linkTarget:k.linkTarget,tooltip:b.db.getTooltip(k.id)||"",domId:b.db.lookUpDomId(k.id),haveCallback:k.haveCallback,width:k.type==="group"?500:void 0,dir:k.dir,type:k.type,props:k.props,padding:ht().flowchart.padding}),Se.info("setNode",{labelStyle:T.labelStyle,shape:O,labelText:C,rx:L,ry:L,class:E,style:T.style,id:k.id,domId:b.db.lookUpDomId(k.id),width:k.type==="group"?500:void 0,type:k.type,dir:k.dir,props:k.props,padding:ht().flowchart.padding})})},sLe=function(r,i,o){Se.info("abc78 edges = ",r);let l=0,f={},b,d;if(r.defaultStyle!==void 0){const w=Dv(r.defaultStyle);b=w.style,d=w.labelStyle}r.forEach(function(w){l++;var y="L-"+w.start+"-"+w.end;f[y]===void 0?(f[y]=0,Se.info("abc78 new entry",y,f[y])):(f[y]++,Se.info("abc78 new entry",y,f[y]));let k=y+"-"+f[y];Se.info("abc78 new link id to be used is",y,k,f[y]);var E="LS-"+w.start,T="LE-"+w.end;const C={style:"",labelStyle:""};switch(C.minlen=w.length||1,w.type==="arrow_open"?C.arrowhead="none":C.arrowhead="normal",C.arrowTypeStart="arrow_open",C.arrowTypeEnd="arrow_open",w.type){case"double_arrow_cross":C.arrowTypeStart="arrow_cross";case"arrow_cross":C.arrowTypeEnd="arrow_cross";break;case"double_arrow_point":C.arrowTypeStart="arrow_point";case"arrow_point":C.arrowTypeEnd="arrow_point";break;case"double_arrow_circle":C.arrowTypeStart="arrow_circle";case"arrow_circle":C.arrowTypeEnd="arrow_circle";break}let S="",L="";switch(w.stroke){case"normal":S="fill:none;",b!==void 0&&(S=b),d!==void 0&&(L=d),C.thickness="normal",C.pattern="solid";break;case"dotted":C.thickness="normal",C.pattern="dotted",C.style="fill:none;stroke-width:2px;stroke-dasharray:3;";break;case"thick":C.thickness="thick",C.pattern="solid",C.style="stroke-width: 3.5px;fill:none;";break}if(w.style!==void 0){const O=Dv(w.style);S=O.style,L=O.labelStyle}C.style=C.style+=S,C.labelStyle=C.labelStyle+=L,w.interpolate!==void 0?C.curve=q0(w.interpolate,x1):r.defaultInterpolate!==void 0?C.curve=q0(r.defaultInterpolate,x1):C.curve=q0(rLe.curve,x1),w.text===void 0?w.style!==void 0&&(C.arrowheadStyle="fill: #333"):(C.arrowheadStyle="fill: #333",C.labelpos="c"),C.labelType="text",C.label=w.text.replace(Cs.lineBreakRegex,`
`),w.style===void 0&&(C.style=C.style||"stroke: #333; stroke-width: 1.5px;fill:none;"),C.labelStyle=C.labelStyle.replace("color:","fill:"),C.id=k,C.classes="flowchart-link "+E+" "+T,i.setEdge(w.start,w.end,C,l)})},$oe={setConf:JXt,addVertices:iLe,addEdges:sLe,getClasses:function(r,i){Se.info("Extracting classes"),i.db.clear();try{return i.parse(r),i.db.getClasses()}catch{return}},draw:function(r,i,o,l){Se.info("Drawing flowchart"),l.db.clear(),$m.setGen("gen-2"),l.parser.parse(r);let f=l.db.getDirection();f===void 0&&(f="TD");const{securityLevel:b,flowchart:d}=ht(),w=d.nodeSpacing||50,y=d.rankSpacing||50;let k;b==="sandbox"&&(k=Cn("#i"+i));const E=Cn(b==="sandbox"?k.nodes()[0].contentDocument.body:"body"),T=b==="sandbox"?k.nodes()[0].contentDocument:document,C=new zh({multigraph:!0,compound:!0}).setGraph({rankdir:f,nodesep:w,ranksep:y,marginx:0,marginy:0}).setDefaultEdgeLabel(function(){return{}});let S;const L=l.db.getSubGraphs();Se.info("Subgraphs - ",L);for(let X=L.length-1;X>=0;X--)S=L[X],Se.info("Subgraph - ",S),l.db.addVertex(S.id,S.title,"group",void 0,S.classes,S.dir);const O=l.db.getVertices(),B=l.db.getEdges();Se.info("Edges",B);let N=0;for(N=L.length-1;N>=0;N--){S=L[N],ose("cluster").append("text");for(let X=0;X<S.nodes.length;X++)Se.info("Setting up subgraphs",S.nodes[X],S.id),C.setParent(S.nodes[X],S.id)}iLe(O,C,i,E,T,l),sLe(B,C);const F=E.select(`[id="${i}"]`),R=E.select("#"+i+" g");if(xoe(R,C,["point","circle","cross"],"flowchart",i),Fs.insertTitle(F,"flowchartTitleText",d.titleTopMargin,l.db.getDiagramTitle()),J8(C,F,d.diagramPadding,d.useMaxWidth),l.db.indexNodes("subGraph"+N),!d.htmlLabels){const X=T.querySelectorAll('[id="'+i+'"] .edgeLabel .label');for(const te of X){const H=te.getBBox(),Y=T.createElementNS("http://www.w3.org/2000/svg","rect");Y.setAttribute("rx",0),Y.setAttribute("ry",0),Y.setAttribute("width",H.width),Y.setAttribute("height",H.height),te.insertBefore(Y,te.firstChild)}}Object.keys(O).forEach(function(X){const te=O[X];if(te.link){const H=Cn("#"+i+' [id="'+X+'"]');if(H){const Y=T.createElementNS("http://www.w3.org/2000/svg","a");Y.setAttributeNS("http://www.w3.org/2000/svg","class",te.classes.join(" ")),Y.setAttributeNS("http://www.w3.org/2000/svg","href",te.link),Y.setAttributeNS("http://www.w3.org/2000/svg","rel","noopener"),b==="sandbox"?Y.setAttributeNS("http://www.w3.org/2000/svg","target","_top"):te.linkTarget&&Y.setAttributeNS("http://www.w3.org/2000/svg","target",te.linkTarget);const z=H.insert(function(){return Y},":first-child"),W=H.select(".label-container");W&&z.append(function(){return W.node()});const Z=H.select(".label");Z&&z.append(function(){return Z.node()})}}})}};var Hoe=function(){var r=function(Y,z,W,Z){for(W=W||{},Z=Y.length;Z--;W[Y[Z]]=z);return W},i=[1,3],o=[1,5],l=[7,9,11,12,13,14,15,16,17,18,19,20,21,23,25,26,28,35,40],f=[1,15],b=[1,16],d=[1,17],w=[1,18],y=[1,19],k=[1,20],E=[1,21],T=[1,22],C=[1,23],S=[1,24],L=[1,25],O=[1,26],B=[1,27],N=[1,29],F=[1,31],R=[1,34],q=[5,7,9,11,12,13,14,15,16,17,18,19,20,21,23,25,26,28,35,40],X={trace:function(){},yy:{},symbols_:{error:2,start:3,directive:4,gantt:5,document:6,EOF:7,line:8,SPACE:9,statement:10,NL:11,dateFormat:12,inclusiveEndDates:13,topAxis:14,axisFormat:15,tickInterval:16,excludes:17,includes:18,todayMarker:19,title:20,acc_title:21,acc_title_value:22,acc_descr:23,acc_descr_value:24,acc_descr_multiline_value:25,section:26,clickStatement:27,taskTxt:28,taskData:29,openDirective:30,typeDirective:31,closeDirective:32,":":33,argDirective:34,click:35,callbackname:36,callbackargs:37,href:38,clickStatementDebug:39,open_directive:40,type_directive:41,arg_directive:42,close_directive:43,$accept:0,$end:1},terminals_:{2:"error",5:"gantt",7:"EOF",9:"SPACE",11:"NL",12:"dateFormat",13:"inclusiveEndDates",14:"topAxis",15:"axisFormat",16:"tickInterval",17:"excludes",18:"includes",19:"todayMarker",20:"title",21:"acc_title",22:"acc_title_value",23:"acc_descr",24:"acc_descr_value",25:"acc_descr_multiline_value",26:"section",28:"taskTxt",29:"taskData",33:":",35:"click",36:"callbackname",37:"callbackargs",38:"href",40:"open_directive",41:"type_directive",42:"arg_directive",43:"close_directive"},productions_:[0,[3,2],[3,3],[6,0],[6,2],[8,2],[8,1],[8,1],[8,1],[10,1],[10,1],[10,1],[10,1],[10,1],[10,1],[10,1],[10,1],[10,1],[10,2],[10,2],[10,1],[10,1],[10,1],[10,2],[10,1],[4,4],[4,6],[27,2],[27,3],[27,3],[27,4],[27,3],[27,4],[27,2],[39,2],[39,3],[39,3],[39,4],[39,3],[39,4],[39,2],[30,1],[31,1],[34,1],[32,1]],performAction:function(z,W,Z,G,ae,$,ge){var ee=$.length-1;switch(ae){case 2:return $[ee-1];case 3:this.$=[];break;case 4:$[ee-1].push($[ee]),this.$=$[ee-1];break;case 5:case 6:this.$=$[ee];break;case 7:case 8:this.$=[];break;case 9:G.setDateFormat($[ee].substr(11)),this.$=$[ee].substr(11);break;case 10:G.enableInclusiveEndDates(),this.$=$[ee].substr(18);break;case 11:G.TopAxis(),this.$=$[ee].substr(8);break;case 12:G.setAxisFormat($[ee].substr(11)),this.$=$[ee].substr(11);break;case 13:G.setTickInterval($[ee].substr(13)),this.$=$[ee].substr(13);break;case 14:G.setExcludes($[ee].substr(9)),this.$=$[ee].substr(9);break;case 15:G.setIncludes($[ee].substr(9)),this.$=$[ee].substr(9);break;case 16:G.setTodayMarker($[ee].substr(12)),this.$=$[ee].substr(12);break;case 17:G.setDiagramTitle($[ee].substr(6)),this.$=$[ee].substr(6);break;case 18:this.$=$[ee].trim(),G.setAccTitle(this.$);break;case 19:case 20:this.$=$[ee].trim(),G.setAccDescription(this.$);break;case 21:G.addSection($[ee].substr(8)),this.$=$[ee].substr(8);break;case 23:G.addTask($[ee-1],$[ee]),this.$="task";break;case 27:this.$=$[ee-1],G.setClickEvent($[ee-1],$[ee],null);break;case 28:this.$=$[ee-2],G.setClickEvent($[ee-2],$[ee-1],$[ee]);break;case 29:this.$=$[ee-2],G.setClickEvent($[ee-2],$[ee-1],null),G.setLink($[ee-2],$[ee]);break;case 30:this.$=$[ee-3],G.setClickEvent($[ee-3],$[ee-2],$[ee-1]),G.setLink($[ee-3],$[ee]);break;case 31:this.$=$[ee-2],G.setClickEvent($[ee-2],$[ee],null),G.setLink($[ee-2],$[ee-1]);break;case 32:this.$=$[ee-3],G.setClickEvent($[ee-3],$[ee-1],$[ee]),G.setLink($[ee-3],$[ee-2]);break;case 33:this.$=$[ee-1],G.setLink($[ee-1],$[ee]);break;case 34:case 40:this.$=$[ee-1]+" "+$[ee];break;case 35:case 36:case 38:this.$=$[ee-2]+" "+$[ee-1]+" "+$[ee];break;case 37:case 39:this.$=$[ee-3]+" "+$[ee-2]+" "+$[ee-1]+" "+$[ee];break;case 41:G.parseDirective("%%{","open_directive");break;case 42:G.parseDirective($[ee],"type_directive");break;case 43:$[ee]=$[ee].trim().replace(/'/g,'"'),G.parseDirective($[ee],"arg_directive");break;case 44:G.parseDirective("}%%","close_directive","gantt");break}},table:[{3:1,4:2,5:i,30:4,40:o},{1:[3]},{3:6,4:2,5:i,30:4,40:o},r(l,[2,3],{6:7}),{31:8,41:[1,9]},{41:[2,41]},{1:[2,1]},{4:30,7:[1,10],8:11,9:[1,12],10:13,11:[1,14],12:f,13:b,14:d,15:w,16:y,17:k,18:E,19:T,20:C,21:S,23:L,25:O,26:B,27:28,28:N,30:4,35:F,40:o},{32:32,33:[1,33],43:R},r([33,43],[2,42]),r(l,[2,8],{1:[2,2]}),r(l,[2,4]),{4:30,10:35,12:f,13:b,14:d,15:w,16:y,17:k,18:E,19:T,20:C,21:S,23:L,25:O,26:B,27:28,28:N,30:4,35:F,40:o},r(l,[2,6]),r(l,[2,7]),r(l,[2,9]),r(l,[2,10]),r(l,[2,11]),r(l,[2,12]),r(l,[2,13]),r(l,[2,14]),r(l,[2,15]),r(l,[2,16]),r(l,[2,17]),{22:[1,36]},{24:[1,37]},r(l,[2,20]),r(l,[2,21]),r(l,[2,22]),{29:[1,38]},r(l,[2,24]),{36:[1,39],38:[1,40]},{11:[1,41]},{34:42,42:[1,43]},{11:[2,44]},r(l,[2,5]),r(l,[2,18]),r(l,[2,19]),r(l,[2,23]),r(l,[2,27],{37:[1,44],38:[1,45]}),r(l,[2,33],{36:[1,46]}),r(q,[2,25]),{32:47,43:R},{43:[2,43]},r(l,[2,28],{38:[1,48]}),r(l,[2,29]),r(l,[2,31],{37:[1,49]}),{11:[1,50]},r(l,[2,30]),r(l,[2,32]),r(q,[2,26])],defaultActions:{5:[2,41],6:[2,1],34:[2,44],43:[2,43]},parseError:function(z,W){if(W.recoverable)this.trace(z);else{var Z=new Error(z);throw Z.hash=W,Z}},parse:function(z){var W=this,Z=[0],G=[],ae=[null],$=[],ge=this.table,ee="",de=0,re=0,ke=2,Ce=1,_e=$.slice.call(arguments,1),Te=Object.create(this.lexer),Be={yy:{}};for(var Ge in this.yy)Object.prototype.hasOwnProperty.call(this.yy,Ge)&&(Be.yy[Ge]=this.yy[Ge]);Te.setInput(z,Be.yy),Be.yy.lexer=Te,Be.yy.parser=this,typeof Te.yylloc>"u"&&(Te.yylloc={});var Xe=Te.yylloc;$.push(Xe);var Ee=Te.options&&Te.options.ranges;typeof Be.yy.parseError=="function"?this.parseError=Be.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Ze(){var Zt;return Zt=G.pop()||Te.lex()||Ce,typeof Zt!="number"&&(Zt instanceof Array&&(G=Zt,Zt=G.pop()),Zt=W.symbols_[Zt]||Zt),Zt}for(var Ie,Oe,Le,$e,xe={},Ae,je,me,vt;;){if(Oe=Z[Z.length-1],this.defaultActions[Oe]?Le=this.defaultActions[Oe]:((Ie===null||typeof Ie>"u")&&(Ie=Ze()),Le=ge[Oe]&&ge[Oe][Ie]),typeof Le>"u"||!Le.length||!Le[0]){var ve="";vt=[];for(Ae in ge[Oe])this.terminals_[Ae]&&Ae>ke&&vt.push("'"+this.terminals_[Ae]+"'");Te.showPosition?ve="Parse error on line "+(de+1)+`:
`+Te.showPosition()+`
Expecting `+vt.join(", ")+", got '"+(this.terminals_[Ie]||Ie)+"'":ve="Parse error on line "+(de+1)+": Unexpected "+(Ie==Ce?"end of input":"'"+(this.terminals_[Ie]||Ie)+"'"),this.parseError(ve,{text:Te.match,token:this.terminals_[Ie]||Ie,line:Te.yylineno,loc:Xe,expected:vt})}if(Le[0]instanceof Array&&Le.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Oe+", token: "+Ie);switch(Le[0]){case 1:Z.push(Ie),ae.push(Te.yytext),$.push(Te.yylloc),Z.push(Le[1]),Ie=null,re=Te.yyleng,ee=Te.yytext,de=Te.yylineno,Xe=Te.yylloc;break;case 2:if(je=this.productions_[Le[1]][1],xe.$=ae[ae.length-je],xe._$={first_line:$[$.length-(je||1)].first_line,last_line:$[$.length-1].last_line,first_column:$[$.length-(je||1)].first_column,last_column:$[$.length-1].last_column},Ee&&(xe._$.range=[$[$.length-(je||1)].range[0],$[$.length-1].range[1]]),$e=this.performAction.apply(xe,[ee,re,de,Be.yy,Le[1],ae,$].concat(_e)),typeof $e<"u")return $e;je&&(Z=Z.slice(0,-1*je*2),ae=ae.slice(0,-1*je),$=$.slice(0,-1*je)),Z.push(this.productions_[Le[1]][0]),ae.push(xe.$),$.push(xe._$),me=ge[Z[Z.length-2]][Z[Z.length-1]],Z.push(me);break;case 3:return!0}}return!0}},te=function(){var Y={EOF:1,parseError:function(W,Z){if(this.yy.parser)this.yy.parser.parseError(W,Z);else throw new Error(W)},setInput:function(z,W){return this.yy=W||this.yy||{},this._input=z,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var z=this._input[0];this.yytext+=z,this.yyleng++,this.offset++,this.match+=z,this.matched+=z;var W=z.match(/(?:\r\n?|\n).*/g);return W?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),z},unput:function(z){var W=z.length,Z=z.split(/(?:\r\n?|\n)/g);this._input=z+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-W),this.offset-=W;var G=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),Z.length-1&&(this.yylineno-=Z.length-1);var ae=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:Z?(Z.length===G.length?this.yylloc.first_column:0)+G[G.length-Z.length].length-Z[0].length:this.yylloc.first_column-W},this.options.ranges&&(this.yylloc.range=[ae[0],ae[0]+this.yyleng-W]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(z){this.unput(this.match.slice(z))},pastInput:function(){var z=this.matched.substr(0,this.matched.length-this.match.length);return(z.length>20?"...":"")+z.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var z=this.match;return z.length<20&&(z+=this._input.substr(0,20-z.length)),(z.substr(0,20)+(z.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var z=this.pastInput(),W=new Array(z.length+1).join("-");return z+this.upcomingInput()+`
`+W+"^"},test_match:function(z,W){var Z,G,ae;if(this.options.backtrack_lexer&&(ae={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(ae.yylloc.range=this.yylloc.range.slice(0))),G=z[0].match(/(?:\r\n?|\n).*/g),G&&(this.yylineno+=G.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:G?G[G.length-1].length-G[G.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+z[0].length},this.yytext+=z[0],this.match+=z[0],this.matches=z,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(z[0].length),this.matched+=z[0],Z=this.performAction.call(this,this.yy,this,W,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),Z)return Z;if(this._backtrack){for(var $ in ae)this[$]=ae[$];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var z,W,Z,G;this._more||(this.yytext="",this.match="");for(var ae=this._currentRules(),$=0;$<ae.length;$++)if(Z=this._input.match(this.rules[ae[$]]),Z&&(!W||Z[0].length>W[0].length)){if(W=Z,G=$,this.options.backtrack_lexer){if(z=this.test_match(Z,ae[$]),z!==!1)return z;if(this._backtrack){W=!1;continue}else return!1}else if(!this.options.flex)break}return W?(z=this.test_match(W,ae[G]),z!==!1?z:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var W=this.next();return W||this.lex()},begin:function(W){this.conditionStack.push(W)},popState:function(){var W=this.conditionStack.length-1;return W>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(W){return W=this.conditionStack.length-1-Math.abs(W||0),W>=0?this.conditionStack[W]:"INITIAL"},pushState:function(W){this.begin(W)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(W,Z,G,ae){switch(G){case 0:return this.begin("open_directive"),40;case 1:return this.begin("type_directive"),41;case 2:return this.popState(),this.begin("arg_directive"),33;case 3:return this.popState(),this.popState(),43;case 4:return 42;case 5:return this.begin("acc_title"),21;case 6:return this.popState(),"acc_title_value";case 7:return this.begin("acc_descr"),23;case 8:return this.popState(),"acc_descr_value";case 9:this.begin("acc_descr_multiline");break;case 10:this.popState();break;case 11:return"acc_descr_multiline_value";case 12:break;case 13:break;case 14:break;case 15:return 11;case 16:break;case 17:break;case 18:break;case 19:this.begin("href");break;case 20:this.popState();break;case 21:return 38;case 22:this.begin("callbackname");break;case 23:this.popState();break;case 24:this.popState(),this.begin("callbackargs");break;case 25:return 36;case 26:this.popState();break;case 27:return 37;case 28:this.begin("click");break;case 29:this.popState();break;case 30:return 35;case 31:return 5;case 32:return 12;case 33:return 13;case 34:return 14;case 35:return 15;case 36:return 16;case 37:return 18;case 38:return 17;case 39:return 19;case 40:return"date";case 41:return 20;case 42:return"accDescription";case 43:return 26;case 44:return 28;case 45:return 29;case 46:return 33;case 47:return 7;case 48:return"INVALID"}},rules:[/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:%%(?!\{)*[^\n]*)/i,/^(?:[^\}]%%*[^\n]*)/i,/^(?:%%*[^\n]*[\n]*)/i,/^(?:[\n]+)/i,/^(?:\s+)/i,/^(?:#[^\n]*)/i,/^(?:%[^\n]*)/i,/^(?:href[\s]+["])/i,/^(?:["])/i,/^(?:[^"]*)/i,/^(?:call[\s]+)/i,/^(?:\([\s]*\))/i,/^(?:\()/i,/^(?:[^(]*)/i,/^(?:\))/i,/^(?:[^)]*)/i,/^(?:click[\s]+)/i,/^(?:[\s\n])/i,/^(?:[^\s\n]*)/i,/^(?:gantt\b)/i,/^(?:dateFormat\s[^#\n;]+)/i,/^(?:inclusiveEndDates\b)/i,/^(?:topAxis\b)/i,/^(?:axisFormat\s[^#\n;]+)/i,/^(?:tickInterval\s[^#\n;]+)/i,/^(?:includes\s[^#\n;]+)/i,/^(?:excludes\s[^#\n;]+)/i,/^(?:todayMarker\s[^\n;]+)/i,/^(?:\d\d\d\d-\d\d-\d\d\b)/i,/^(?:title\s[^#\n;]+)/i,/^(?:accDescription\s[^#\n;]+)/i,/^(?:section\s[^#:\n;]+)/i,/^(?:[^#:\n;]+)/i,/^(?::[^#\n;]+)/i,/^(?::)/i,/^(?:$)/i,/^(?:.)/i],conditions:{acc_descr_multiline:{rules:[10,11],inclusive:!1},acc_descr:{rules:[8],inclusive:!1},acc_title:{rules:[6],inclusive:!1},close_directive:{rules:[],inclusive:!1},arg_directive:{rules:[3,4],inclusive:!1},type_directive:{rules:[2,3],inclusive:!1},open_directive:{rules:[1],inclusive:!1},callbackargs:{rules:[26,27],inclusive:!1},callbackname:{rules:[23,24,25],inclusive:!1},href:{rules:[20,21],inclusive:!1},click:{rules:[29,30],inclusive:!1},INITIAL:{rules:[0,5,7,9,12,13,14,15,16,17,18,19,22,28,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48],inclusive:!0}}};return Y}();X.lexer=te;function H(){this.yy={}}return H.prototype=X,X.Parser=H,new H}();Hoe.parser=Hoe;const eQt=Hoe,tQt=r=>r.match(/^\s*gantt/)!==null;let Gm="",zoe="",Goe,Voe="",HI=[],zI=[],Uoe={},qoe=[],HV=[],mC="";const aLe=["active","done","crit","milestone"];let Yoe=[],GI=!1,Woe=!1,Koe=0;const nQt=function(r,i,o){Bu.parseDirective(this,r,i,o)},rQt=function(){qoe=[],HV=[],mC="",Yoe=[],zV=0,Qoe=void 0,GV=void 0,ff=[],Gm="",zoe="",Goe=void 0,Voe="",HI=[],zI=[],GI=!1,Woe=!1,Koe=0,Uoe={},W0()},iQt=function(r){zoe=r},sQt=function(){return zoe},aQt=function(r){Goe=r},oQt=function(){return Goe},cQt=function(r){Voe=r},uQt=function(){return Voe},lQt=function(r){Gm=r},hQt=function(){GI=!0},fQt=function(){return GI},dQt=function(){Woe=!0},gQt=function(){return Woe},pQt=function(){return Gm},bQt=function(r){HI=r.toLowerCase().split(/[\s,]+/)},vQt=function(){return HI},wQt=function(r){zI=r.toLowerCase().split(/[\s,]+/)},mQt=function(){return zI},yQt=function(){return Uoe},kQt=function(r){mC=r,qoe.push(r)},xQt=function(){return qoe},EQt=function(){let r=fLe();const i=10;let o=0;for(;!r&&o<i;)r=fLe(),o++;return HV=ff,HV},oLe=function(r,i,o,l){return l.includes(r.format(i.trim()))?!1:r.isoWeekday()>=6&&o.includes("weekends")||o.includes(r.format("dddd").toLowerCase())?!0:o.includes(r.format(i.trim()))},cLe=function(r,i,o,l){if(!o.length||r.manualEndTime)return;let f=Ar(r.startTime,i,!0);f.add(1,"d");let b=Ar(r.endTime,i,!0),d=_Qt(f,b,i,o,l);r.endTime=b.toDate(),r.renderEndTime=d},_Qt=function(r,i,o,l,f){let b=!1,d=null;for(;r<=i;)b||(d=i.toDate()),b=oLe(r,o,l,f),b&&i.add(1,"d"),r.add(1,"d");return d},Xoe=function(r,i,o){o=o.trim();const f=/^after\s+([\d\w- ]+)/.exec(o.trim());if(f!==null){let d=null;if(f[1].split(" ").forEach(function(w){let y=kC(w);y!==void 0&&(d?y.endTime>d.endTime&&(d=y):d=y)}),d)return d.endTime;{const w=new Date;return w.setHours(0,0,0,0),w}}let b=Ar(o,i.trim(),!0);if(b.isValid())return b.toDate();{Se.debug("Invalid date:"+o),Se.debug("With date format:"+i.trim());const d=new Date(o);if(d===void 0||isNaN(d.getTime()))throw new Error("Invalid date:"+o);return d}},uLe=function(r){const i=/^(\d+(?:\.\d+)?)([Mdhmswy]|ms)$/.exec(r.trim());return i!==null?Ar.duration(Number.parseFloat(i[1]),i[2]):Ar.duration.invalid()},lLe=function(r,i,o,l=!1){o=o.trim();let f=Ar(o,i.trim(),!0);if(f.isValid())return l&&f.add(1,"d"),f.toDate();const b=Ar(r),d=uLe(o);return d.isValid()&&b.add(d),b.toDate()};let zV=0;const yC=function(r){return r===void 0?(zV=zV+1,"task"+zV):r},TQt=function(r,i){let o;i.substr(0,1)===":"?o=i.substr(1,i.length):o=i;const l=o.split(","),f={};pLe(l,f,aLe);for(let d=0;d<l.length;d++)l[d]=l[d].trim();let b="";switch(l.length){case 1:f.id=yC(),f.startTime=r.endTime,b=l[0];break;case 2:f.id=yC(),f.startTime=Xoe(void 0,Gm,l[0]),b=l[1];break;case 3:f.id=yC(l[0]),f.startTime=Xoe(void 0,Gm,l[1]),b=l[2];break}return b&&(f.endTime=lLe(f.startTime,Gm,b,GI),f.manualEndTime=Ar(b,"YYYY-MM-DD",!0).isValid(),cLe(f,Gm,zI,HI)),f},CQt=function(r,i){let o;i.substr(0,1)===":"?o=i.substr(1,i.length):o=i;const l=o.split(","),f={};pLe(l,f,aLe);for(let b=0;b<l.length;b++)l[b]=l[b].trim();switch(l.length){case 1:f.id=yC(),f.startTime={type:"prevTaskEnd",id:r},f.endTime={data:l[0]};break;case 2:f.id=yC(),f.startTime={type:"getStartDate",startData:l[0]},f.endTime={data:l[1]};break;case 3:f.id=yC(l[0]),f.startTime={type:"getStartDate",startData:l[1]},f.endTime={data:l[2]};break}return f};let Qoe,GV,ff=[];const hLe={},SQt=function(r,i){const o={section:mC,type:mC,processed:!1,manualEndTime:!1,renderEndTime:null,raw:{data:i},task:r,classes:[]},l=CQt(GV,i);o.raw.startTime=l.startTime,o.raw.endTime=l.endTime,o.id=l.id,o.prevTaskId=GV,o.active=l.active,o.done=l.done,o.crit=l.crit,o.milestone=l.milestone,o.order=Koe,Koe++;const f=ff.push(o);GV=o.id,hLe[o.id]=f-1},kC=function(r){const i=hLe[r];return ff[i]},AQt=function(r,i){const o={section:mC,type:mC,description:r,task:r,classes:[]},l=TQt(Qoe,i);o.startTime=l.startTime,o.endTime=l.endTime,o.id=l.id,o.active=l.active,o.done=l.done,o.crit=l.crit,o.milestone=l.milestone,Qoe=o,HV.push(o)},fLe=function(){const r=function(o){const l=ff[o];let f="";switch(ff[o].raw.startTime.type){case"prevTaskEnd":{const b=kC(l.prevTaskId);l.startTime=b.endTime;break}case"getStartDate":f=Xoe(void 0,Gm,ff[o].raw.startTime.startData),f&&(ff[o].startTime=f);break}return ff[o].startTime&&(ff[o].endTime=lLe(ff[o].startTime,Gm,ff[o].raw.endTime.data,GI),ff[o].endTime&&(ff[o].processed=!0,ff[o].manualEndTime=Ar(ff[o].raw.endTime.data,"YYYY-MM-DD",!0).isValid(),cLe(ff[o],Gm,zI,HI))),ff[o].processed};let i=!0;for(const[o,l]of ff.entries())r(o),i=i&&l.processed;return i},MQt=function(r,i){let o=i;ht().securityLevel!=="loose"&&(o=$3(i)),r.split(",").forEach(function(l){kC(l)!==void 0&&(gLe(l,()=>{window.open(o,"_self")}),Uoe[l]=o)}),dLe(r,"clickable")},dLe=function(r,i){r.split(",").forEach(function(o){let l=kC(o);l!==void 0&&l.classes.push(i)})},DQt=function(r,i,o){if(ht().securityLevel!=="loose"||i===void 0)return;let l=[];if(typeof o=="string"){l=o.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);for(let b=0;b<l.length;b++){let d=l[b].trim();d.charAt(0)==='"'&&d.charAt(d.length-1)==='"'&&(d=d.substr(1,d.length-2)),l[b]=d}}l.length===0&&l.push(r),kC(r)!==void 0&&gLe(r,()=>{Fs.runFunc(i,...l)})},gLe=function(r,i){Yoe.push(function(){const o=document.querySelector(`[id="${r}"]`);o!==null&&o.addEventListener("click",function(){i()})},function(){const o=document.querySelector(`[id="${r}-text"]`);o!==null&&o.addEventListener("click",function(){i()})})},Zoe={parseDirective:nQt,getConfig:()=>ht().gantt,clear:rQt,setDateFormat:lQt,getDateFormat:pQt,enableInclusiveEndDates:hQt,endDatesAreInclusive:fQt,enableTopAxis:dQt,topAxisEnabled:gQt,setAxisFormat:iQt,getAxisFormat:sQt,setTickInterval:aQt,getTickInterval:oQt,setTodayMarker:cQt,getTodayMarker:uQt,setAccTitle:K0,getAccTitle:hp,setDiagramTitle:Ov,getDiagramTitle:Nv,setAccDescription:fp,getAccDescription:dp,addSection:kQt,getSections:xQt,getTasks:EQt,addTask:SQt,findTaskById:kC,addTaskOrg:AQt,setIncludes:bQt,getIncludes:vQt,setExcludes:wQt,getExcludes:mQt,setClickEvent:function(r,i,o){r.split(",").forEach(function(l){DQt(l,i,o)}),dLe(r,"clickable")},setLink:MQt,getLinks:yQt,bindFunctions:function(r){Yoe.forEach(function(i){i(r)})},parseDuration:uLe,isInvalidDate:oLe};function pLe(r,i,o){let l=!0;for(;l;)l=!1,o.forEach(function(f){const b="^\\s*"+f+"\\s*$",d=new RegExp(b);r[0].match(d)&&(i[f]=!0,r.shift(1),l=!0)})}const LQt=function(){Se.debug("Something is calling, setConf, remove the call")};let s4;const IQt={setConf:LQt,draw:function(r,i,o,l){const f=ht().gantt,b=ht().securityLevel;let d;b==="sandbox"&&(d=Cn("#i"+i));const w=Cn(b==="sandbox"?d.nodes()[0].contentDocument.body:"body"),y=b==="sandbox"?d.nodes()[0].contentDocument:document,k=y.getElementById(i);s4=k.parentElement.offsetWidth,s4===void 0&&(s4=1200),f.useWidth!==void 0&&(s4=f.useWidth);const E=l.db.getTasks(),T=E.length*(f.barHeight+f.barGap)+2*f.topPadding;k.setAttribute("viewBox","0 0 "+s4+" "+T);const C=w.select(`[id="${i}"]`),S=kNt().domain([JAt(E,function(W){return W.startTime}),ZAt(E,function(W){return W.endTime})]).rangeRound([0,s4-f.leftPadding-f.rightPadding]);let L=[];for(const W of E)L.push(W.type);const O=L;L=H(L);function B(W,Z){const G=W.startTime,ae=Z.startTime;let $=0;return G>ae?$=1:G<ae&&($=-1),$}E.sort(B),N(E,s4,T),Iv(C,T,s4,f.useMaxWidth),C.append("text").text(l.db.getDiagramTitle()).attr("x",s4/2).attr("y",f.titleTopMargin).attr("class","titleText");function N(W,Z,G){const ae=f.barHeight,$=ae+f.barGap,ge=f.topPadding,ee=f.leftPadding,de=sTe().domain([0,L.length]).range(["#00B9FA","#F95002"]).interpolate(yLt);R($,ge,ee,Z,G,W,l.db.getExcludes(),l.db.getIncludes()),q(ee,ge,Z,G),F(W,$,ge,ee,ae,de,Z),X($,ge),te(ee,ge,Z,G)}function F(W,Z,G,ae,$,ge,ee){C.append("g").selectAll("rect").data(W).enter().append("rect").attr("x",0).attr("y",function(Ce,_e){return _e=Ce.order,_e*Z+G-2}).attr("width",function(){return ee-f.rightPadding/2}).attr("height",Z).attr("class",function(Ce){for(const[_e,Te]of L.entries())if(Ce.type===Te)return"section section"+_e%f.numberSectionStyles;return"section section0"});const de=C.append("g").selectAll("rect").data(W).enter(),re=l.db.getLinks();if(de.append("rect").attr("id",function(Ce){return Ce.id}).attr("rx",3).attr("ry",3).attr("x",function(Ce){return Ce.milestone?S(Ce.startTime)+ae+.5*(S(Ce.endTime)-S(Ce.startTime))-.5*$:S(Ce.startTime)+ae}).attr("y",function(Ce,_e){return _e=Ce.order,_e*Z+G}).attr("width",function(Ce){return Ce.milestone?$:S(Ce.renderEndTime||Ce.endTime)-S(Ce.startTime)}).attr("height",$).attr("transform-origin",function(Ce,_e){return _e=Ce.order,(S(Ce.startTime)+ae+.5*(S(Ce.endTime)-S(Ce.startTime))).toString()+"px "+(_e*Z+G+.5*$).toString()+"px"}).attr("class",function(Ce){const _e="task";let Te="";Ce.classes.length>0&&(Te=Ce.classes.join(" "));let Be=0;for(const[Xe,Ee]of L.entries())Ce.type===Ee&&(Be=Xe%f.numberSectionStyles);let Ge="";return Ce.active?Ce.crit?Ge+=" activeCrit":Ge=" active":Ce.done?Ce.crit?Ge=" doneCrit":Ge=" done":Ce.crit&&(Ge+=" crit"),Ge.length===0&&(Ge=" task"),Ce.milestone&&(Ge=" milestone "+Ge),Ge+=Be,Ge+=" "+Te,_e+Ge}),de.append("text").attr("id",function(Ce){return Ce.id+"-text"}).text(function(Ce){return Ce.task}).attr("font-size",f.fontSize).attr("x",function(Ce){let _e=S(Ce.startTime),Te=S(Ce.renderEndTime||Ce.endTime);Ce.milestone&&(_e+=.5*(S(Ce.endTime)-S(Ce.startTime))-.5*$),Ce.milestone&&(Te=_e+$);const Be=this.getBBox().width;return Be>Te-_e?Te+Be+1.5*f.leftPadding>ee?_e+ae-5:Te+ae+5:(Te-_e)/2+_e+ae}).attr("y",function(Ce,_e){return _e=Ce.order,_e*Z+f.barHeight/2+(f.fontSize/2-2)+G}).attr("text-height",$).attr("class",function(Ce){const _e=S(Ce.startTime);let Te=S(Ce.endTime);Ce.milestone&&(Te=_e+$);const Be=this.getBBox().width;let Ge="";Ce.classes.length>0&&(Ge=Ce.classes.join(" "));let Xe=0;for(const[Ze,Ie]of L.entries())Ce.type===Ie&&(Xe=Ze%f.numberSectionStyles);let Ee="";return Ce.active&&(Ce.crit?Ee="activeCritText"+Xe:Ee="activeText"+Xe),Ce.done?Ce.crit?Ee=Ee+" doneCritText"+Xe:Ee=Ee+" doneText"+Xe:Ce.crit&&(Ee=Ee+" critText"+Xe),Ce.milestone&&(Ee+=" milestoneText"),Be>Te-_e?Te+Be+1.5*f.leftPadding>ee?Ge+" taskTextOutsideLeft taskTextOutside"+Xe+" "+Ee:Ge+" taskTextOutsideRight taskTextOutside"+Xe+" "+Ee+" width-"+Be:Ge+" taskText taskText"+Xe+" "+Ee+" width-"+Be}),ht().securityLevel==="sandbox"){let Ce;Ce=Cn("#i"+i);const _e=Ce.nodes()[0].contentDocument;de.filter(function(Te){return re[Te.id]!==void 0}).each(function(Te){var Be=_e.querySelector("#"+Te.id),Ge=_e.querySelector("#"+Te.id+"-text");const Xe=Be.parentNode;var Ee=_e.createElement("a");Ee.setAttribute("xlink:href",re[Te.id]),Ee.setAttribute("target","_top"),Xe.appendChild(Ee),Ee.appendChild(Be),Ee.appendChild(Ge)})}}function R(W,Z,G,ae,$,ge,ee,de){const re=ge.reduce((Xe,{startTime:Ee})=>Xe?Math.min(Xe,Ee):Ee,0),ke=ge.reduce((Xe,{endTime:Ee})=>Xe?Math.max(Xe,Ee):Ee,0),Ce=l.db.getDateFormat();if(!re||!ke)return;const _e=[];let Te=null,Be=Ar(re);for(;Be.valueOf()<=ke;)l.db.isInvalidDate(Be,Ce,ee,de)?Te?Te.end=Be.clone():Te={start:Be.clone(),end:Be.clone()}:Te&&(_e.push(Te),Te=null),Be.add(1,"d");C.append("g").selectAll("rect").data(_e).enter().append("rect").attr("id",function(Xe){return"exclude-"+Xe.start.format("YYYY-MM-DD")}).attr("x",function(Xe){return S(Xe.start)+G}).attr("y",f.gridLineStartPadding).attr("width",function(Xe){const Ee=Xe.end.clone().add(1,"day");return S(Ee)-S(Xe.start)}).attr("height",$-Z-f.gridLineStartPadding).attr("transform-origin",function(Xe,Ee){return(S(Xe.start)+G+.5*(S(Xe.end)-S(Xe.start))).toString()+"px "+(Ee*W+.5*$).toString()+"px"}).attr("class","exclude-range")}function q(W,Z,G,ae){let $=oMt(S).tickSize(-ae+Z+f.gridLineStartPadding).tickFormat(sG(l.db.getAxisFormat()||f.axisFormat||"%Y-%m-%d"));const ee=/^([1-9]\d*)(minute|hour|day|week|month)$/.exec(l.db.getTickInterval()||f.tickInterval);if(ee!==null){const de=ee[1];switch(ee[2]){case"minute":$.ticks(eG.every(de));break;case"hour":$.ticks(tG.every(de));break;case"day":$.ticks(RT.every(de));break;case"week":$.ticks(jT.every(de));break;case"month":$.ticks(rG.every(de));break}}if(C.append("g").attr("class","grid").attr("transform","translate("+W+", "+(ae-50)+")").call($).selectAll("text").style("text-anchor","middle").attr("fill","#000").attr("stroke","none").attr("font-size",10).attr("dy","1em"),l.db.topAxisEnabled()||f.topAxis){let de=aMt(S).tickSize(-ae+Z+f.gridLineStartPadding).tickFormat(sG(l.db.getAxisFormat()||f.axisFormat||"%Y-%m-%d"));if(ee!==null){const re=ee[1];switch(ee[2]){case"minute":de.ticks(eG.every(re));break;case"hour":de.ticks(tG.every(re));break;case"day":de.ticks(RT.every(re));break;case"week":de.ticks(jT.every(re));break;case"month":de.ticks(rG.every(re));break}}C.append("g").attr("class","grid").attr("transform","translate("+W+", "+Z+")").call(de).selectAll("text").style("text-anchor","middle").attr("fill","#000").attr("stroke","none").attr("font-size",10)}}function X(W,Z){const G=[];let ae=0;for(const[$,ge]of L.entries())G[$]=[ge,z(ge,O)];C.append("g").selectAll("text").data(G).enter().append(function($){const ge=$[0].split(Cs.lineBreakRegex),ee=-(ge.length-1)/2,de=y.createElementNS("http://www.w3.org/2000/svg","text");de.setAttribute("dy",ee+"em");for(const[re,ke]of ge.entries()){const Ce=y.createElementNS("http://www.w3.org/2000/svg","tspan");Ce.setAttribute("alignment-baseline","central"),Ce.setAttribute("x","10"),re>0&&Ce.setAttribute("dy","1em"),Ce.textContent=ke,de.appendChild(Ce)}return de}).attr("x",10).attr("y",function($,ge){if(ge>0)for(let ee=0;ee<ge;ee++)return ae+=G[ge-1][1],$[1]*W/2+ae*W+Z;else return $[1]*W/2+Z}).attr("font-size",f.sectionFontSize).attr("font-size",f.sectionFontSize).attr("class",function($){for(const[ge,ee]of L.entries())if($[0]===ee)return"sectionTitle sectionTitle"+ge%f.numberSectionStyles;return"sectionTitle"})}function te(W,Z,G,ae){const $=l.db.getTodayMarker();if($==="off")return;const ge=C.append("g").attr("class","today"),ee=new Date,de=ge.append("line");de.attr("x1",S(ee)+W).attr("x2",S(ee)+W).attr("y1",f.titleTopMargin).attr("y2",ae-f.titleTopMargin).attr("class","today"),$!==""&&de.attr("style",$.replace(/,/g,";"))}function H(W){const Z={},G=[];for(let ae=0,$=W.length;ae<$;++ae)Object.prototype.hasOwnProperty.call(Z,W[ae])||(Z[W[ae]]=!0,G.push(W[ae]));return G}function Y(W){let Z=W.length;const G={};for(;Z;)G[W[--Z]]=(G[W[Z]]||0)+1;return G}function z(W,Z){return Y(Z)[W]||0}}};var Joe=function(){var r=function(b,d,w,y){for(w=w||{},y=b.length;y--;w[b[y]]=d);return w},i=[6,9,10],o={trace:function(){},yy:{},symbols_:{error:2,start:3,info:4,document:5,EOF:6,line:7,statement:8,NL:9,showInfo:10,$accept:0,$end:1},terminals_:{2:"error",4:"info",6:"EOF",9:"NL",10:"showInfo"},productions_:[0,[3,3],[5,0],[5,2],[7,1],[7,1],[8,1]],performAction:function(d,w,y,k,E,T,C){switch(T.length-1,E){case 1:return k;case 4:break;case 6:k.setInfo(!0);break}},table:[{3:1,4:[1,2]},{1:[3]},r(i,[2,2],{5:3}),{6:[1,4],7:5,8:6,9:[1,7],10:[1,8]},{1:[2,1]},r(i,[2,3]),r(i,[2,4]),r(i,[2,5]),r(i,[2,6])],defaultActions:{4:[2,1]},parseError:function(d,w){if(w.recoverable)this.trace(d);else{var y=new Error(d);throw y.hash=w,y}},parse:function(d){var w=this,y=[0],k=[],E=[null],T=[],C=this.table,S="",L=0,O=0,B=2,N=1,F=T.slice.call(arguments,1),R=Object.create(this.lexer),q={yy:{}};for(var X in this.yy)Object.prototype.hasOwnProperty.call(this.yy,X)&&(q.yy[X]=this.yy[X]);R.setInput(d,q.yy),q.yy.lexer=R,q.yy.parser=this,typeof R.yylloc>"u"&&(R.yylloc={});var te=R.yylloc;T.push(te);var H=R.options&&R.options.ranges;typeof q.yy.parseError=="function"?this.parseError=q.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Y(){var ke;return ke=k.pop()||R.lex()||N,typeof ke!="number"&&(ke instanceof Array&&(k=ke,ke=k.pop()),ke=w.symbols_[ke]||ke),ke}for(var z,W,Z,G,ae={},$,ge,ee,de;;){if(W=y[y.length-1],this.defaultActions[W]?Z=this.defaultActions[W]:((z===null||typeof z>"u")&&(z=Y()),Z=C[W]&&C[W][z]),typeof Z>"u"||!Z.length||!Z[0]){var re="";de=[];for($ in C[W])this.terminals_[$]&&$>B&&de.push("'"+this.terminals_[$]+"'");R.showPosition?re="Parse error on line "+(L+1)+`:
`+R.showPosition()+`
Expecting `+de.join(", ")+", got '"+(this.terminals_[z]||z)+"'":re="Parse error on line "+(L+1)+": Unexpected "+(z==N?"end of input":"'"+(this.terminals_[z]||z)+"'"),this.parseError(re,{text:R.match,token:this.terminals_[z]||z,line:R.yylineno,loc:te,expected:de})}if(Z[0]instanceof Array&&Z.length>1)throw new Error("Parse Error: multiple actions possible at state: "+W+", token: "+z);switch(Z[0]){case 1:y.push(z),E.push(R.yytext),T.push(R.yylloc),y.push(Z[1]),z=null,O=R.yyleng,S=R.yytext,L=R.yylineno,te=R.yylloc;break;case 2:if(ge=this.productions_[Z[1]][1],ae.$=E[E.length-ge],ae._$={first_line:T[T.length-(ge||1)].first_line,last_line:T[T.length-1].last_line,first_column:T[T.length-(ge||1)].first_column,last_column:T[T.length-1].last_column},H&&(ae._$.range=[T[T.length-(ge||1)].range[0],T[T.length-1].range[1]]),G=this.performAction.apply(ae,[S,O,L,q.yy,Z[1],E,T].concat(F)),typeof G<"u")return G;ge&&(y=y.slice(0,-1*ge*2),E=E.slice(0,-1*ge),T=T.slice(0,-1*ge)),y.push(this.productions_[Z[1]][0]),E.push(ae.$),T.push(ae._$),ee=C[y[y.length-2]][y[y.length-1]],y.push(ee);break;case 3:return!0}}return!0}},l=function(){var b={EOF:1,parseError:function(w,y){if(this.yy.parser)this.yy.parser.parseError(w,y);else throw new Error(w)},setInput:function(d,w){return this.yy=w||this.yy||{},this._input=d,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var d=this._input[0];this.yytext+=d,this.yyleng++,this.offset++,this.match+=d,this.matched+=d;var w=d.match(/(?:\r\n?|\n).*/g);return w?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),d},unput:function(d){var w=d.length,y=d.split(/(?:\r\n?|\n)/g);this._input=d+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-w),this.offset-=w;var k=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),y.length-1&&(this.yylineno-=y.length-1);var E=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:y?(y.length===k.length?this.yylloc.first_column:0)+k[k.length-y.length].length-y[0].length:this.yylloc.first_column-w},this.options.ranges&&(this.yylloc.range=[E[0],E[0]+this.yyleng-w]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(d){this.unput(this.match.slice(d))},pastInput:function(){var d=this.matched.substr(0,this.matched.length-this.match.length);return(d.length>20?"...":"")+d.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var d=this.match;return d.length<20&&(d+=this._input.substr(0,20-d.length)),(d.substr(0,20)+(d.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var d=this.pastInput(),w=new Array(d.length+1).join("-");return d+this.upcomingInput()+`
`+w+"^"},test_match:function(d,w){var y,k,E;if(this.options.backtrack_lexer&&(E={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(E.yylloc.range=this.yylloc.range.slice(0))),k=d[0].match(/(?:\r\n?|\n).*/g),k&&(this.yylineno+=k.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:k?k[k.length-1].length-k[k.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+d[0].length},this.yytext+=d[0],this.match+=d[0],this.matches=d,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(d[0].length),this.matched+=d[0],y=this.performAction.call(this,this.yy,this,w,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),y)return y;if(this._backtrack){for(var T in E)this[T]=E[T];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var d,w,y,k;this._more||(this.yytext="",this.match="");for(var E=this._currentRules(),T=0;T<E.length;T++)if(y=this._input.match(this.rules[E[T]]),y&&(!w||y[0].length>w[0].length)){if(w=y,k=T,this.options.backtrack_lexer){if(d=this.test_match(y,E[T]),d!==!1)return d;if(this._backtrack){w=!1;continue}else return!1}else if(!this.options.flex)break}return w?(d=this.test_match(w,E[k]),d!==!1?d:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var w=this.next();return w||this.lex()},begin:function(w){this.conditionStack.push(w)},popState:function(){var w=this.conditionStack.length-1;return w>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(w){return w=this.conditionStack.length-1-Math.abs(w||0),w>=0?this.conditionStack[w]:"INITIAL"},pushState:function(w){this.begin(w)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(w,y,k,E){switch(k){case 0:return 4;case 1:return 9;case 2:return"space";case 3:return 10;case 4:return 6;case 5:return"TXT"}},rules:[/^(?:info\b)/i,/^(?:[\s\n\r]+)/i,/^(?:[\s]+)/i,/^(?:showInfo\b)/i,/^(?:$)/i,/^(?:.)/i],conditions:{INITIAL:{rules:[0,1,2,3,4,5],inclusive:!0}}};return b}();o.lexer=l;function f(){this.yy={}}return f.prototype=o,o.Parser=f,new f}();Joe.parser=Joe;const OQt=Joe;var bLe="",vLe=!1;const NQt={setMessage:r=>{Se.debug("Setting message to: "+r),bLe=r},getMessage:()=>bLe,setInfo:r=>{vLe=r},getInfo:()=>vLe,clear:W0},PQt={draw:(r,i,o)=>{try{Se.debug(`Rendering info diagram
`+r);const l=ht().securityLevel;let f;l==="sandbox"&&(f=Cn("#i"+i));const d=Cn(l==="sandbox"?f.nodes()[0].contentDocument.body:"body").select("#"+i);d.append("g").append("text").attr("x",100).attr("y",40).attr("class","version").attr("font-size","32px").style("text-anchor","middle").text("v "+o),d.attr("height",100),d.attr("width",400)}catch(l){Se.error("Error while rendering info diagram"),Se.error(l.message)}}},BQt=r=>r.match(/^\s*info/)!==null;var ece=function(){var r=function(te,H,Y,z){for(Y=Y||{},z=te.length;z--;Y[te[z]]=H);return Y},i=[1,4],o=[1,5],l=[1,6],f=[1,7],b=[1,9],d=[1,11,13,15,17,19,20,26,27,28,29],w=[2,5],y=[1,6,11,13,15,17,19,20,26,27,28,29],k=[26,27,28],E=[2,8],T=[1,18],C=[1,19],S=[1,20],L=[1,21],O=[1,22],B=[1,23],N=[1,28],F=[6,26,27,28,29],R={trace:function(){},yy:{},symbols_:{error:2,start:3,eol:4,directive:5,PIE:6,document:7,showData:8,line:9,statement:10,txt:11,value:12,title:13,title_value:14,acc_title:15,acc_title_value:16,acc_descr:17,acc_descr_value:18,acc_descr_multiline_value:19,section:20,openDirective:21,typeDirective:22,closeDirective:23,":":24,argDirective:25,NEWLINE:26,";":27,EOF:28,open_directive:29,type_directive:30,arg_directive:31,close_directive:32,$accept:0,$end:1},terminals_:{2:"error",6:"PIE",8:"showData",11:"txt",12:"value",13:"title",14:"title_value",15:"acc_title",16:"acc_title_value",17:"acc_descr",18:"acc_descr_value",19:"acc_descr_multiline_value",20:"section",24:":",26:"NEWLINE",27:";",28:"EOF",29:"open_directive",30:"type_directive",31:"arg_directive",32:"close_directive"},productions_:[0,[3,2],[3,2],[3,2],[3,3],[7,0],[7,2],[9,2],[10,0],[10,2],[10,2],[10,2],[10,2],[10,1],[10,1],[10,1],[5,3],[5,5],[4,1],[4,1],[4,1],[21,1],[22,1],[25,1],[23,1]],performAction:function(H,Y,z,W,Z,G,ae){var $=G.length-1;switch(Z){case 4:W.setShowData(!0);break;case 7:this.$=G[$-1];break;case 9:W.addSection(G[$-1],W.cleanupValue(G[$]));break;case 10:this.$=G[$].trim(),W.setDiagramTitle(this.$);break;case 11:this.$=G[$].trim(),W.setAccTitle(this.$);break;case 12:case 13:this.$=G[$].trim(),W.setAccDescription(this.$);break;case 14:W.addSection(G[$].substr(8)),this.$=G[$].substr(8);break;case 21:W.parseDirective("%%{","open_directive");break;case 22:W.parseDirective(G[$],"type_directive");break;case 23:G[$]=G[$].trim().replace(/'/g,'"'),W.parseDirective(G[$],"arg_directive");break;case 24:W.parseDirective("}%%","close_directive","pie");break}},table:[{3:1,4:2,5:3,6:i,21:8,26:o,27:l,28:f,29:b},{1:[3]},{3:10,4:2,5:3,6:i,21:8,26:o,27:l,28:f,29:b},{3:11,4:2,5:3,6:i,21:8,26:o,27:l,28:f,29:b},r(d,w,{7:12,8:[1,13]}),r(y,[2,18]),r(y,[2,19]),r(y,[2,20]),{22:14,30:[1,15]},{30:[2,21]},{1:[2,1]},{1:[2,2]},r(k,E,{21:8,9:16,10:17,5:24,1:[2,3],11:T,13:C,15:S,17:L,19:O,20:B,29:b}),r(d,w,{7:25}),{23:26,24:[1,27],32:N},r([24,32],[2,22]),r(d,[2,6]),{4:29,26:o,27:l,28:f},{12:[1,30]},{14:[1,31]},{16:[1,32]},{18:[1,33]},r(k,[2,13]),r(k,[2,14]),r(k,[2,15]),r(k,E,{21:8,9:16,10:17,5:24,1:[2,4],11:T,13:C,15:S,17:L,19:O,20:B,29:b}),r(F,[2,16]),{25:34,31:[1,35]},r(F,[2,24]),r(d,[2,7]),r(k,[2,9]),r(k,[2,10]),r(k,[2,11]),r(k,[2,12]),{23:36,32:N},{32:[2,23]},r(F,[2,17])],defaultActions:{9:[2,21],10:[2,1],11:[2,2],35:[2,23]},parseError:function(H,Y){if(Y.recoverable)this.trace(H);else{var z=new Error(H);throw z.hash=Y,z}},parse:function(H){var Y=this,z=[0],W=[],Z=[null],G=[],ae=this.table,$="",ge=0,ee=0,de=2,re=1,ke=G.slice.call(arguments,1),Ce=Object.create(this.lexer),_e={yy:{}};for(var Te in this.yy)Object.prototype.hasOwnProperty.call(this.yy,Te)&&(_e.yy[Te]=this.yy[Te]);Ce.setInput(H,_e.yy),_e.yy.lexer=Ce,_e.yy.parser=this,typeof Ce.yylloc>"u"&&(Ce.yylloc={});var Be=Ce.yylloc;G.push(Be);var Ge=Ce.options&&Ce.options.ranges;typeof _e.yy.parseError=="function"?this.parseError=_e.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Xe(){var vt;return vt=W.pop()||Ce.lex()||re,typeof vt!="number"&&(vt instanceof Array&&(W=vt,vt=W.pop()),vt=Y.symbols_[vt]||vt),vt}for(var Ee,Ze,Ie,Oe,Le={},$e,xe,Ae,je;;){if(Ze=z[z.length-1],this.defaultActions[Ze]?Ie=this.defaultActions[Ze]:((Ee===null||typeof Ee>"u")&&(Ee=Xe()),Ie=ae[Ze]&&ae[Ze][Ee]),typeof Ie>"u"||!Ie.length||!Ie[0]){var me="";je=[];for($e in ae[Ze])this.terminals_[$e]&&$e>de&&je.push("'"+this.terminals_[$e]+"'");Ce.showPosition?me="Parse error on line "+(ge+1)+`:
`+Ce.showPosition()+`
Expecting `+je.join(", ")+", got '"+(this.terminals_[Ee]||Ee)+"'":me="Parse error on line "+(ge+1)+": Unexpected "+(Ee==re?"end of input":"'"+(this.terminals_[Ee]||Ee)+"'"),this.parseError(me,{text:Ce.match,token:this.terminals_[Ee]||Ee,line:Ce.yylineno,loc:Be,expected:je})}if(Ie[0]instanceof Array&&Ie.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Ze+", token: "+Ee);switch(Ie[0]){case 1:z.push(Ee),Z.push(Ce.yytext),G.push(Ce.yylloc),z.push(Ie[1]),Ee=null,ee=Ce.yyleng,$=Ce.yytext,ge=Ce.yylineno,Be=Ce.yylloc;break;case 2:if(xe=this.productions_[Ie[1]][1],Le.$=Z[Z.length-xe],Le._$={first_line:G[G.length-(xe||1)].first_line,last_line:G[G.length-1].last_line,first_column:G[G.length-(xe||1)].first_column,last_column:G[G.length-1].last_column},Ge&&(Le._$.range=[G[G.length-(xe||1)].range[0],G[G.length-1].range[1]]),Oe=this.performAction.apply(Le,[$,ee,ge,_e.yy,Ie[1],Z,G].concat(ke)),typeof Oe<"u")return Oe;xe&&(z=z.slice(0,-1*xe*2),Z=Z.slice(0,-1*xe),G=G.slice(0,-1*xe)),z.push(this.productions_[Ie[1]][0]),Z.push(Le.$),G.push(Le._$),Ae=ae[z[z.length-2]][z[z.length-1]],z.push(Ae);break;case 3:return!0}}return!0}},q=function(){var te={EOF:1,parseError:function(Y,z){if(this.yy.parser)this.yy.parser.parseError(Y,z);else throw new Error(Y)},setInput:function(H,Y){return this.yy=Y||this.yy||{},this._input=H,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var H=this._input[0];this.yytext+=H,this.yyleng++,this.offset++,this.match+=H,this.matched+=H;var Y=H.match(/(?:\r\n?|\n).*/g);return Y?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),H},unput:function(H){var Y=H.length,z=H.split(/(?:\r\n?|\n)/g);this._input=H+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-Y),this.offset-=Y;var W=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),z.length-1&&(this.yylineno-=z.length-1);var Z=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:z?(z.length===W.length?this.yylloc.first_column:0)+W[W.length-z.length].length-z[0].length:this.yylloc.first_column-Y},this.options.ranges&&(this.yylloc.range=[Z[0],Z[0]+this.yyleng-Y]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(H){this.unput(this.match.slice(H))},pastInput:function(){var H=this.matched.substr(0,this.matched.length-this.match.length);return(H.length>20?"...":"")+H.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var H=this.match;return H.length<20&&(H+=this._input.substr(0,20-H.length)),(H.substr(0,20)+(H.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var H=this.pastInput(),Y=new Array(H.length+1).join("-");return H+this.upcomingInput()+`
`+Y+"^"},test_match:function(H,Y){var z,W,Z;if(this.options.backtrack_lexer&&(Z={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(Z.yylloc.range=this.yylloc.range.slice(0))),W=H[0].match(/(?:\r\n?|\n).*/g),W&&(this.yylineno+=W.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:W?W[W.length-1].length-W[W.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+H[0].length},this.yytext+=H[0],this.match+=H[0],this.matches=H,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(H[0].length),this.matched+=H[0],z=this.performAction.call(this,this.yy,this,Y,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),z)return z;if(this._backtrack){for(var G in Z)this[G]=Z[G];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var H,Y,z,W;this._more||(this.yytext="",this.match="");for(var Z=this._currentRules(),G=0;G<Z.length;G++)if(z=this._input.match(this.rules[Z[G]]),z&&(!Y||z[0].length>Y[0].length)){if(Y=z,W=G,this.options.backtrack_lexer){if(H=this.test_match(z,Z[G]),H!==!1)return H;if(this._backtrack){Y=!1;continue}else return!1}else if(!this.options.flex)break}return Y?(H=this.test_match(Y,Z[W]),H!==!1?H:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var Y=this.next();return Y||this.lex()},begin:function(Y){this.conditionStack.push(Y)},popState:function(){var Y=this.conditionStack.length-1;return Y>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(Y){return Y=this.conditionStack.length-1-Math.abs(Y||0),Y>=0?this.conditionStack[Y]:"INITIAL"},pushState:function(Y){this.begin(Y)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(Y,z,W,Z){switch(W){case 0:return this.begin("open_directive"),29;case 1:return this.begin("type_directive"),30;case 2:return this.popState(),this.begin("arg_directive"),24;case 3:return this.popState(),this.popState(),32;case 4:return 31;case 5:break;case 6:break;case 7:return 26;case 8:break;case 9:break;case 10:return this.begin("title"),13;case 11:return this.popState(),"title_value";case 12:return this.begin("acc_title"),15;case 13:return this.popState(),"acc_title_value";case 14:return this.begin("acc_descr"),17;case 15:return this.popState(),"acc_descr_value";case 16:this.begin("acc_descr_multiline");break;case 17:this.popState();break;case 18:return"acc_descr_multiline_value";case 19:this.begin("string");break;case 20:this.popState();break;case 21:return"txt";case 22:return 6;case 23:return 8;case 24:return"value";case 25:return 28}},rules:[/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:%%(?!\{)[^\n]*)/i,/^(?:[^\}]%%[^\n]*)/i,/^(?:[\n\r]+)/i,/^(?:%%[^\n]*)/i,/^(?:[\s]+)/i,/^(?:title\b)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:["])/i,/^(?:["])/i,/^(?:[^"]*)/i,/^(?:pie\b)/i,/^(?:showData\b)/i,/^(?::[\s]*[\d]+(?:\.[\d]+)?)/i,/^(?:$)/i],conditions:{acc_descr_multiline:{rules:[17,18],inclusive:!1},acc_descr:{rules:[15],inclusive:!1},acc_title:{rules:[13],inclusive:!1},close_directive:{rules:[],inclusive:!1},arg_directive:{rules:[3,4],inclusive:!1},type_directive:{rules:[2,3],inclusive:!1},open_directive:{rules:[1],inclusive:!1},title:{rules:[11],inclusive:!1},string:{rules:[20,21],inclusive:!1},INITIAL:{rules:[0,5,6,7,8,9,10,12,14,16,19,22,23,24,25],inclusive:!0}}};return te}();R.lexer=q;function X(){this.yy={}}return X.prototype=R,R.Parser=X,new X}();ece.parser=ece;const FQt=ece,RQt=r=>r.match(/^\s*pie/)!==null||r.match(/^\s*bar/)!==null;let VV={},tce=!1;const jQt={parseDirective:function(r,i,o){Bu.parseDirective(this,r,i,o)},getConfig:()=>ht().pie,addSection:function(r,i){r=Cs.sanitizeText(r,ht()),VV[r]===void 0&&(VV[r]=i,Se.debug("Added new section :",r))},getSections:()=>VV,cleanupValue:function(r){return r.substring(0,1)===":"&&(r=r.substring(1).trim()),Number(r.trim())},clear:function(){VV={},tce=!1,W0()},setAccTitle:K0,getAccTitle:hp,setDiagramTitle:Ov,getDiagramTitle:Nv,setShowData:function(r){tce=r},getShowData:function(){return tce},getAccDescription:dp,setAccDescription:fp};let a4=ht(),o4;const VI=450,$Qt={draw:(r,i,o,l)=>{try{a4=ht(),Se.debug(`Rendering info diagram
`+r);const F=ht().securityLevel;let R;F==="sandbox"&&(R=Cn("#i"+i));const q=Cn(F==="sandbox"?R.nodes()[0].contentDocument.body:"body"),X=F==="sandbox"?R.nodes()[0].contentDocument:document;l.db.clear(),l.parser.parse(r),Se.debug("Parsed info diagram");const te=X.getElementById(i);o4=te.parentElement.offsetWidth,o4===void 0&&(o4=1200),a4.useWidth!==void 0&&(o4=a4.useWidth),a4.pie.useWidth!==void 0&&(o4=a4.pie.useWidth);const H=q.select("#"+i);Iv(H,VI,o4,a4.pie.useMaxWidth),te.setAttribute("viewBox","0 0 "+o4+" "+VI);var f=40,b=18,d=4,w=Math.min(o4,VI)/2-f,y=H.append("g").attr("transform","translate("+o4/2+","+VI/2+")"),k=l.db.getSections(),E=0;Object.keys(k).forEach(function(z){E+=k[z]});const Y=a4.themeVariables;var T=[Y.pie1,Y.pie2,Y.pie3,Y.pie4,Y.pie5,Y.pie6,Y.pie7,Y.pie8,Y.pie9,Y.pie10,Y.pie11,Y.pie12],C=tTe().range(T),S=Object.entries(k).map(function(z,W){return{order:W,name:z[0],value:z[1]}}),L=NNt().value(function(z){return z.value}).sort(function(z,W){return z.order-W.order}),O=L(S),B=ZL().innerRadius(0).outerRadius(w);y.selectAll("mySlices").data(O).enter().append("path").attr("d",B).attr("fill",function(z){return C(z.data.name)}).attr("class","pieCircle"),y.selectAll("mySlices").data(O).enter().append("text").text(function(z){return(z.data.value/E*100).toFixed(0)+"%"}).attr("transform",function(z){return"translate("+B.centroid(z)+")"}).style("text-anchor","middle").attr("class","slice"),y.append("text").text(l.db.getDiagramTitle()).attr("x",0).attr("y",-(VI-50)/2).attr("class","pieTitleText");var N=y.selectAll(".legend").data(C.domain()).enter().append("g").attr("class","legend").attr("transform",function(z,W){const Z=b+d,G=Z*C.domain().length/2,ae=12*b,$=W*Z-G;return"translate("+ae+","+$+")"});N.append("rect").attr("width",b).attr("height",b).style("fill",C).style("stroke",C),N.data(O).append("text").attr("x",b+d).attr("y",b-d).text(function(z){return l.db.getShowData()||a4.showData||a4.pie.showData?z.data.name+" ["+z.data.value+"]":z.data.name})}catch(F){Se.error("Error while rendering info diagram"),Se.error(F)}}};var nce=function(){var r=function(Ze,Ie,Oe,Le){for(Oe=Oe||{},Le=Ze.length;Le--;Oe[Ze[Le]]=Ie);return Oe},i=[1,3],o=[1,5],l=[1,6],f=[1,7],b=[1,8],d=[5,6,8,14,16,18,19,40,41,42,43,44,45,53,71,72],w=[1,22],y=[2,13],k=[1,26],E=[1,27],T=[1,28],C=[1,29],S=[1,30],L=[1,31],O=[1,24],B=[1,32],N=[1,33],F=[1,36],R=[71,72],q=[5,8,14,16,18,19,40,41,42,43,44,45,53,60,62,71,72],X=[1,56],te=[1,57],H=[1,58],Y=[1,59],z=[1,60],W=[1,61],Z=[1,62],G=[62,63],ae=[1,74],$=[1,70],ge=[1,71],ee=[1,72],de=[1,73],re=[1,75],ke=[1,79],Ce=[1,80],_e=[1,77],Te=[1,78],Be=[5,8,14,16,18,19,40,41,42,43,44,45,53,71,72],Ge={trace:function(){},yy:{},symbols_:{error:2,start:3,directive:4,NEWLINE:5,RD:6,diagram:7,EOF:8,openDirective:9,typeDirective:10,closeDirective:11,":":12,argDirective:13,acc_title:14,acc_title_value:15,acc_descr:16,acc_descr_value:17,acc_descr_multiline_value:18,open_directive:19,type_directive:20,arg_directive:21,close_directive:22,requirementDef:23,elementDef:24,relationshipDef:25,requirementType:26,requirementName:27,STRUCT_START:28,requirementBody:29,ID:30,COLONSEP:31,id:32,TEXT:33,text:34,RISK:35,riskLevel:36,VERIFYMTHD:37,verifyType:38,STRUCT_STOP:39,REQUIREMENT:40,FUNCTIONAL_REQUIREMENT:41,INTERFACE_REQUIREMENT:42,PERFORMANCE_REQUIREMENT:43,PHYSICAL_REQUIREMENT:44,DESIGN_CONSTRAINT:45,LOW_RISK:46,MED_RISK:47,HIGH_RISK:48,VERIFY_ANALYSIS:49,VERIFY_DEMONSTRATION:50,VERIFY_INSPECTION:51,VERIFY_TEST:52,ELEMENT:53,elementName:54,elementBody:55,TYPE:56,type:57,DOCREF:58,ref:59,END_ARROW_L:60,relationship:61,LINE:62,END_ARROW_R:63,CONTAINS:64,COPIES:65,DERIVES:66,SATISFIES:67,VERIFIES:68,REFINES:69,TRACES:70,unqString:71,qString:72,$accept:0,$end:1},terminals_:{2:"error",5:"NEWLINE",6:"RD",8:"EOF",12:":",14:"acc_title",15:"acc_title_value",16:"acc_descr",17:"acc_descr_value",18:"acc_descr_multiline_value",19:"open_directive",20:"type_directive",21:"arg_directive",22:"close_directive",28:"STRUCT_START",30:"ID",31:"COLONSEP",33:"TEXT",35:"RISK",37:"VERIFYMTHD",39:"STRUCT_STOP",40:"REQUIREMENT",41:"FUNCTIONAL_REQUIREMENT",42:"INTERFACE_REQUIREMENT",43:"PERFORMANCE_REQUIREMENT",44:"PHYSICAL_REQUIREMENT",45:"DESIGN_CONSTRAINT",46:"LOW_RISK",47:"MED_RISK",48:"HIGH_RISK",49:"VERIFY_ANALYSIS",50:"VERIFY_DEMONSTRATION",51:"VERIFY_INSPECTION",52:"VERIFY_TEST",53:"ELEMENT",56:"TYPE",58:"DOCREF",60:"END_ARROW_L",62:"LINE",63:"END_ARROW_R",64:"CONTAINS",65:"COPIES",66:"DERIVES",67:"SATISFIES",68:"VERIFIES",69:"REFINES",70:"TRACES",71:"unqString",72:"qString"},productions_:[0,[3,3],[3,2],[3,4],[4,3],[4,5],[4,2],[4,2],[4,1],[9,1],[10,1],[13,1],[11,1],[7,0],[7,2],[7,2],[7,2],[7,2],[7,2],[23,5],[29,5],[29,5],[29,5],[29,5],[29,2],[29,1],[26,1],[26,1],[26,1],[26,1],[26,1],[26,1],[36,1],[36,1],[36,1],[38,1],[38,1],[38,1],[38,1],[24,5],[55,5],[55,5],[55,2],[55,1],[25,5],[25,5],[61,1],[61,1],[61,1],[61,1],[61,1],[61,1],[61,1],[27,1],[27,1],[32,1],[32,1],[34,1],[34,1],[54,1],[54,1],[57,1],[57,1],[59,1],[59,1]],performAction:function(Ie,Oe,Le,$e,xe,Ae,je){var me=Ae.length-1;switch(xe){case 6:this.$=Ae[me].trim(),$e.setAccTitle(this.$);break;case 7:case 8:this.$=Ae[me].trim(),$e.setAccDescription(this.$);break;case 9:$e.parseDirective("%%{","open_directive");break;case 10:$e.parseDirective(Ae[me],"type_directive");break;case 11:Ae[me]=Ae[me].trim().replace(/'/g,'"'),$e.parseDirective(Ae[me],"arg_directive");break;case 12:$e.parseDirective("}%%","close_directive","pie");break;case 13:this.$=[];break;case 19:$e.addRequirement(Ae[me-3],Ae[me-4]);break;case 20:$e.setNewReqId(Ae[me-2]);break;case 21:$e.setNewReqText(Ae[me-2]);break;case 22:$e.setNewReqRisk(Ae[me-2]);break;case 23:$e.setNewReqVerifyMethod(Ae[me-2]);break;case 26:this.$=$e.RequirementType.REQUIREMENT;break;case 27:this.$=$e.RequirementType.FUNCTIONAL_REQUIREMENT;break;case 28:this.$=$e.RequirementType.INTERFACE_REQUIREMENT;break;case 29:this.$=$e.RequirementType.PERFORMANCE_REQUIREMENT;break;case 30:this.$=$e.RequirementType.PHYSICAL_REQUIREMENT;break;case 31:this.$=$e.RequirementType.DESIGN_CONSTRAINT;break;case 32:this.$=$e.RiskLevel.LOW_RISK;break;case 33:this.$=$e.RiskLevel.MED_RISK;break;case 34:this.$=$e.RiskLevel.HIGH_RISK;break;case 35:this.$=$e.VerifyType.VERIFY_ANALYSIS;break;case 36:this.$=$e.VerifyType.VERIFY_DEMONSTRATION;break;case 37:this.$=$e.VerifyType.VERIFY_INSPECTION;break;case 38:this.$=$e.VerifyType.VERIFY_TEST;break;case 39:$e.addElement(Ae[me-3]);break;case 40:$e.setNewElementType(Ae[me-2]);break;case 41:$e.setNewElementDocRef(Ae[me-2]);break;case 44:$e.addRelationship(Ae[me-2],Ae[me],Ae[me-4]);break;case 45:$e.addRelationship(Ae[me-2],Ae[me-4],Ae[me]);break;case 46:this.$=$e.Relationships.CONTAINS;break;case 47:this.$=$e.Relationships.COPIES;break;case 48:this.$=$e.Relationships.DERIVES;break;case 49:this.$=$e.Relationships.SATISFIES;break;case 50:this.$=$e.Relationships.VERIFIES;break;case 51:this.$=$e.Relationships.REFINES;break;case 52:this.$=$e.Relationships.TRACES;break}},table:[{3:1,4:2,6:i,9:4,14:o,16:l,18:f,19:b},{1:[3]},{3:10,4:2,5:[1,9],6:i,9:4,14:o,16:l,18:f,19:b},{5:[1,11]},{10:12,20:[1,13]},{15:[1,14]},{17:[1,15]},r(d,[2,8]),{20:[2,9]},{3:16,4:2,6:i,9:4,14:o,16:l,18:f,19:b},{1:[2,2]},{4:21,5:w,7:17,8:y,9:4,14:o,16:l,18:f,19:b,23:18,24:19,25:20,26:23,32:25,40:k,41:E,42:T,43:C,44:S,45:L,53:O,71:B,72:N},{11:34,12:[1,35],22:F},r([12,22],[2,10]),r(d,[2,6]),r(d,[2,7]),{1:[2,1]},{8:[1,37]},{4:21,5:w,7:38,8:y,9:4,14:o,16:l,18:f,19:b,23:18,24:19,25:20,26:23,32:25,40:k,41:E,42:T,43:C,44:S,45:L,53:O,71:B,72:N},{4:21,5:w,7:39,8:y,9:4,14:o,16:l,18:f,19:b,23:18,24:19,25:20,26:23,32:25,40:k,41:E,42:T,43:C,44:S,45:L,53:O,71:B,72:N},{4:21,5:w,7:40,8:y,9:4,14:o,16:l,18:f,19:b,23:18,24:19,25:20,26:23,32:25,40:k,41:E,42:T,43:C,44:S,45:L,53:O,71:B,72:N},{4:21,5:w,7:41,8:y,9:4,14:o,16:l,18:f,19:b,23:18,24:19,25:20,26:23,32:25,40:k,41:E,42:T,43:C,44:S,45:L,53:O,71:B,72:N},{4:21,5:w,7:42,8:y,9:4,14:o,16:l,18:f,19:b,23:18,24:19,25:20,26:23,32:25,40:k,41:E,42:T,43:C,44:S,45:L,53:O,71:B,72:N},{27:43,71:[1,44],72:[1,45]},{54:46,71:[1,47],72:[1,48]},{60:[1,49],62:[1,50]},r(R,[2,26]),r(R,[2,27]),r(R,[2,28]),r(R,[2,29]),r(R,[2,30]),r(R,[2,31]),r(q,[2,55]),r(q,[2,56]),r(d,[2,4]),{13:51,21:[1,52]},r(d,[2,12]),{1:[2,3]},{8:[2,14]},{8:[2,15]},{8:[2,16]},{8:[2,17]},{8:[2,18]},{28:[1,53]},{28:[2,53]},{28:[2,54]},{28:[1,54]},{28:[2,59]},{28:[2,60]},{61:55,64:X,65:te,66:H,67:Y,68:z,69:W,70:Z},{61:63,64:X,65:te,66:H,67:Y,68:z,69:W,70:Z},{11:64,22:F},{22:[2,11]},{5:[1,65]},{5:[1,66]},{62:[1,67]},r(G,[2,46]),r(G,[2,47]),r(G,[2,48]),r(G,[2,49]),r(G,[2,50]),r(G,[2,51]),r(G,[2,52]),{63:[1,68]},r(d,[2,5]),{5:ae,29:69,30:$,33:ge,35:ee,37:de,39:re},{5:ke,39:Ce,55:76,56:_e,58:Te},{32:81,71:B,72:N},{32:82,71:B,72:N},r(Be,[2,19]),{31:[1,83]},{31:[1,84]},{31:[1,85]},{31:[1,86]},{5:ae,29:87,30:$,33:ge,35:ee,37:de,39:re},r(Be,[2,25]),r(Be,[2,39]),{31:[1,88]},{31:[1,89]},{5:ke,39:Ce,55:90,56:_e,58:Te},r(Be,[2,43]),r(Be,[2,44]),r(Be,[2,45]),{32:91,71:B,72:N},{34:92,71:[1,93],72:[1,94]},{36:95,46:[1,96],47:[1,97],48:[1,98]},{38:99,49:[1,100],50:[1,101],51:[1,102],52:[1,103]},r(Be,[2,24]),{57:104,71:[1,105],72:[1,106]},{59:107,71:[1,108],72:[1,109]},r(Be,[2,42]),{5:[1,110]},{5:[1,111]},{5:[2,57]},{5:[2,58]},{5:[1,112]},{5:[2,32]},{5:[2,33]},{5:[2,34]},{5:[1,113]},{5:[2,35]},{5:[2,36]},{5:[2,37]},{5:[2,38]},{5:[1,114]},{5:[2,61]},{5:[2,62]},{5:[1,115]},{5:[2,63]},{5:[2,64]},{5:ae,29:116,30:$,33:ge,35:ee,37:de,39:re},{5:ae,29:117,30:$,33:ge,35:ee,37:de,39:re},{5:ae,29:118,30:$,33:ge,35:ee,37:de,39:re},{5:ae,29:119,30:$,33:ge,35:ee,37:de,39:re},{5:ke,39:Ce,55:120,56:_e,58:Te},{5:ke,39:Ce,55:121,56:_e,58:Te},r(Be,[2,20]),r(Be,[2,21]),r(Be,[2,22]),r(Be,[2,23]),r(Be,[2,40]),r(Be,[2,41])],defaultActions:{8:[2,9],10:[2,2],16:[2,1],37:[2,3],38:[2,14],39:[2,15],40:[2,16],41:[2,17],42:[2,18],44:[2,53],45:[2,54],47:[2,59],48:[2,60],52:[2,11],93:[2,57],94:[2,58],96:[2,32],97:[2,33],98:[2,34],100:[2,35],101:[2,36],102:[2,37],103:[2,38],105:[2,61],106:[2,62],108:[2,63],109:[2,64]},parseError:function(Ie,Oe){if(Oe.recoverable)this.trace(Ie);else{var Le=new Error(Ie);throw Le.hash=Oe,Le}},parse:function(Ie){var Oe=this,Le=[0],$e=[],xe=[null],Ae=[],je=this.table,me="",vt=0,ve=0,Zt=2,nt=1,xn=Ae.slice.call(arguments,1),cn=Object.create(this.lexer),jt={yy:{}};for(var ot in this.yy)Object.prototype.hasOwnProperty.call(this.yy,ot)&&(jt.yy[ot]=this.yy[ot]);cn.setInput(Ie,jt.yy),jt.yy.lexer=cn,jt.yy.parser=this,typeof cn.yylloc>"u"&&(cn.yylloc={});var be=cn.yylloc;Ae.push(be);var We=cn.options&&cn.options.ranges;typeof jt.yy.parseError=="function"?this.parseError=jt.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function ct(){var en;return en=$e.pop()||cn.lex()||nt,typeof en!="number"&&(en instanceof Array&&($e=en,en=$e.pop()),en=Oe.symbols_[en]||en),en}for(var Yt,Ut,Wn,Gt,Rn={},si,$r,nr,Kn;;){if(Ut=Le[Le.length-1],this.defaultActions[Ut]?Wn=this.defaultActions[Ut]:((Yt===null||typeof Yt>"u")&&(Yt=ct()),Wn=je[Ut]&&je[Ut][Yt]),typeof Wn>"u"||!Wn.length||!Wn[0]){var Jt="";Kn=[];for(si in je[Ut])this.terminals_[si]&&si>Zt&&Kn.push("'"+this.terminals_[si]+"'");cn.showPosition?Jt="Parse error on line "+(vt+1)+`:
`+cn.showPosition()+`
Expecting `+Kn.join(", ")+", got '"+(this.terminals_[Yt]||Yt)+"'":Jt="Parse error on line "+(vt+1)+": Unexpected "+(Yt==nt?"end of input":"'"+(this.terminals_[Yt]||Yt)+"'"),this.parseError(Jt,{text:cn.match,token:this.terminals_[Yt]||Yt,line:cn.yylineno,loc:be,expected:Kn})}if(Wn[0]instanceof Array&&Wn.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Ut+", token: "+Yt);switch(Wn[0]){case 1:Le.push(Yt),xe.push(cn.yytext),Ae.push(cn.yylloc),Le.push(Wn[1]),Yt=null,ve=cn.yyleng,me=cn.yytext,vt=cn.yylineno,be=cn.yylloc;break;case 2:if($r=this.productions_[Wn[1]][1],Rn.$=xe[xe.length-$r],Rn._$={first_line:Ae[Ae.length-($r||1)].first_line,last_line:Ae[Ae.length-1].last_line,first_column:Ae[Ae.length-($r||1)].first_column,last_column:Ae[Ae.length-1].last_column},We&&(Rn._$.range=[Ae[Ae.length-($r||1)].range[0],Ae[Ae.length-1].range[1]]),Gt=this.performAction.apply(Rn,[me,ve,vt,jt.yy,Wn[1],xe,Ae].concat(xn)),typeof Gt<"u")return Gt;$r&&(Le=Le.slice(0,-1*$r*2),xe=xe.slice(0,-1*$r),Ae=Ae.slice(0,-1*$r)),Le.push(this.productions_[Wn[1]][0]),xe.push(Rn.$),Ae.push(Rn._$),nr=je[Le[Le.length-2]][Le[Le.length-1]],Le.push(nr);break;case 3:return!0}}return!0}},Xe=function(){var Ze={EOF:1,parseError:function(Oe,Le){if(this.yy.parser)this.yy.parser.parseError(Oe,Le);else throw new Error(Oe)},setInput:function(Ie,Oe){return this.yy=Oe||this.yy||{},this._input=Ie,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var Ie=this._input[0];this.yytext+=Ie,this.yyleng++,this.offset++,this.match+=Ie,this.matched+=Ie;var Oe=Ie.match(/(?:\r\n?|\n).*/g);return Oe?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),Ie},unput:function(Ie){var Oe=Ie.length,Le=Ie.split(/(?:\r\n?|\n)/g);this._input=Ie+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-Oe),this.offset-=Oe;var $e=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),Le.length-1&&(this.yylineno-=Le.length-1);var xe=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:Le?(Le.length===$e.length?this.yylloc.first_column:0)+$e[$e.length-Le.length].length-Le[0].length:this.yylloc.first_column-Oe},this.options.ranges&&(this.yylloc.range=[xe[0],xe[0]+this.yyleng-Oe]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(Ie){this.unput(this.match.slice(Ie))},pastInput:function(){var Ie=this.matched.substr(0,this.matched.length-this.match.length);return(Ie.length>20?"...":"")+Ie.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var Ie=this.match;return Ie.length<20&&(Ie+=this._input.substr(0,20-Ie.length)),(Ie.substr(0,20)+(Ie.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var Ie=this.pastInput(),Oe=new Array(Ie.length+1).join("-");return Ie+this.upcomingInput()+`
`+Oe+"^"},test_match:function(Ie,Oe){var Le,$e,xe;if(this.options.backtrack_lexer&&(xe={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(xe.yylloc.range=this.yylloc.range.slice(0))),$e=Ie[0].match(/(?:\r\n?|\n).*/g),$e&&(this.yylineno+=$e.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:$e?$e[$e.length-1].length-$e[$e.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+Ie[0].length},this.yytext+=Ie[0],this.match+=Ie[0],this.matches=Ie,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(Ie[0].length),this.matched+=Ie[0],Le=this.performAction.call(this,this.yy,this,Oe,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),Le)return Le;if(this._backtrack){for(var Ae in xe)this[Ae]=xe[Ae];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var Ie,Oe,Le,$e;this._more||(this.yytext="",this.match="");for(var xe=this._currentRules(),Ae=0;Ae<xe.length;Ae++)if(Le=this._input.match(this.rules[xe[Ae]]),Le&&(!Oe||Le[0].length>Oe[0].length)){if(Oe=Le,$e=Ae,this.options.backtrack_lexer){if(Ie=this.test_match(Le,xe[Ae]),Ie!==!1)return Ie;if(this._backtrack){Oe=!1;continue}else return!1}else if(!this.options.flex)break}return Oe?(Ie=this.test_match(Oe,xe[$e]),Ie!==!1?Ie:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var Oe=this.next();return Oe||this.lex()},begin:function(Oe){this.conditionStack.push(Oe)},popState:function(){var Oe=this.conditionStack.length-1;return Oe>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(Oe){return Oe=this.conditionStack.length-1-Math.abs(Oe||0),Oe>=0?this.conditionStack[Oe]:"INITIAL"},pushState:function(Oe){this.begin(Oe)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(Oe,Le,$e,xe){switch($e){case 0:return this.begin("open_directive"),19;case 1:return this.begin("type_directive"),20;case 2:return this.popState(),this.begin("arg_directive"),12;case 3:return this.popState(),this.popState(),22;case 4:return 21;case 5:return"title";case 6:return this.begin("acc_title"),14;case 7:return this.popState(),"acc_title_value";case 8:return this.begin("acc_descr"),16;case 9:return this.popState(),"acc_descr_value";case 10:this.begin("acc_descr_multiline");break;case 11:this.popState();break;case 12:return"acc_descr_multiline_value";case 13:return 5;case 14:break;case 15:break;case 16:break;case 17:return 8;case 18:return 6;case 19:return 28;case 20:return 39;case 21:return 31;case 22:return 30;case 23:return 33;case 24:return 35;case 25:return 37;case 26:return 40;case 27:return 41;case 28:return 42;case 29:return 43;case 30:return 44;case 31:return 45;case 32:return 46;case 33:return 47;case 34:return 48;case 35:return 49;case 36:return 50;case 37:return 51;case 38:return 52;case 39:return 53;case 40:return 64;case 41:return 65;case 42:return 66;case 43:return 67;case 44:return 68;case 45:return 69;case 46:return 70;case 47:return 56;case 48:return 58;case 49:return 60;case 50:return 63;case 51:return 62;case 52:this.begin("string");break;case 53:this.popState();break;case 54:return"qString";case 55:return Le.yytext=Le.yytext.trim(),71}},rules:[/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:title\s[^#\n;]+)/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:(\r?\n)+)/i,/^(?:\s+)/i,/^(?:#[^\n]*)/i,/^(?:%[^\n]*)/i,/^(?:$)/i,/^(?:requirementDiagram\b)/i,/^(?:\{)/i,/^(?:\})/i,/^(?::)/i,/^(?:id\b)/i,/^(?:text\b)/i,/^(?:risk\b)/i,/^(?:verifyMethod\b)/i,/^(?:requirement\b)/i,/^(?:functionalRequirement\b)/i,/^(?:interfaceRequirement\b)/i,/^(?:performanceRequirement\b)/i,/^(?:physicalRequirement\b)/i,/^(?:designConstraint\b)/i,/^(?:low\b)/i,/^(?:medium\b)/i,/^(?:high\b)/i,/^(?:analysis\b)/i,/^(?:demonstration\b)/i,/^(?:inspection\b)/i,/^(?:test\b)/i,/^(?:element\b)/i,/^(?:contains\b)/i,/^(?:copies\b)/i,/^(?:derives\b)/i,/^(?:satisfies\b)/i,/^(?:verifies\b)/i,/^(?:refines\b)/i,/^(?:traces\b)/i,/^(?:type\b)/i,/^(?:docref\b)/i,/^(?:<-)/i,/^(?:->)/i,/^(?:-)/i,/^(?:["])/i,/^(?:["])/i,/^(?:[^"]*)/i,/^(?:[\w][^\r\n\{\<\>\-\=]*)/i],conditions:{acc_descr_multiline:{rules:[11,12],inclusive:!1},acc_descr:{rules:[9],inclusive:!1},acc_title:{rules:[7],inclusive:!1},close_directive:{rules:[],inclusive:!1},arg_directive:{rules:[3,4],inclusive:!1},type_directive:{rules:[2,3],inclusive:!1},open_directive:{rules:[1],inclusive:!1},unqString:{rules:[],inclusive:!1},token:{rules:[],inclusive:!1},string:{rules:[53,54],inclusive:!1},INITIAL:{rules:[0,5,6,8,10,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,55],inclusive:!0}}};return Ze}();Ge.lexer=Xe;function Ee(){this.yy={}}return Ee.prototype=Ge,Ge.Parser=Ee,new Ee}();nce.parser=nce;const HQt=nce,zQt=r=>r.match(/^\s*requirement(Diagram)?/)!==null;let rce=[],Z0={},UI={},v6={},qI={};const GQt={RequirementType:{REQUIREMENT:"Requirement",FUNCTIONAL_REQUIREMENT:"Functional Requirement",INTERFACE_REQUIREMENT:"Interface Requirement",PERFORMANCE_REQUIREMENT:"Performance Requirement",PHYSICAL_REQUIREMENT:"Physical Requirement",DESIGN_CONSTRAINT:"Design Constraint"},RiskLevel:{LOW_RISK:"Low",MED_RISK:"Medium",HIGH_RISK:"High"},VerifyType:{VERIFY_ANALYSIS:"Analysis",VERIFY_DEMONSTRATION:"Demonstration",VERIFY_INSPECTION:"Inspection",VERIFY_TEST:"Test"},Relationships:{CONTAINS:"contains",COPIES:"copies",DERIVES:"derives",SATISFIES:"satisfies",VERIFIES:"verifies",REFINES:"refines",TRACES:"traces"},parseDirective:function(r,i,o){Bu.parseDirective(this,r,i,o)},getConfig:()=>ht().req,addRequirement:(r,i)=>(UI[r]===void 0&&(UI[r]={name:r,type:i,id:Z0.id,text:Z0.text,risk:Z0.risk,verifyMethod:Z0.verifyMethod}),Z0={},UI[r]),getRequirements:()=>UI,setNewReqId:r=>{Z0!==void 0&&(Z0.id=r)},setNewReqText:r=>{Z0!==void 0&&(Z0.text=r)},setNewReqRisk:r=>{Z0!==void 0&&(Z0.risk=r)},setNewReqVerifyMethod:r=>{Z0!==void 0&&(Z0.verifyMethod=r)},setAccTitle:K0,getAccTitle:hp,setAccDescription:fp,getAccDescription:dp,addElement:r=>(qI[r]===void 0&&(qI[r]={name:r,type:v6.type,docRef:v6.docRef},Se.info("Added new requirement: ",r)),v6={},qI[r]),getElements:()=>qI,setNewElementType:r=>{v6!==void 0&&(v6.type=r)},setNewElementDocRef:r=>{v6!==void 0&&(v6.docRef=r)},addRelationship:(r,i,o)=>{rce.push({type:r,src:i,dst:o})},getRelationships:()=>rce,clear:()=>{rce=[],Z0={},UI={},v6={},qI={},W0()}},ice={CONTAINS:"contains",ARROW:"arrow"},wLe={ReqMarkers:ice,insertLineEndings:(r,i)=>{let o=r.append("defs").append("marker").attr("id",ice.CONTAINS+"_line_ending").attr("refX",0).attr("refY",i.line_height/2).attr("markerWidth",i.line_height).attr("markerHeight",i.line_height).attr("orient","auto").append("g");o.append("circle").attr("cx",i.line_height/2).attr("cy",i.line_height/2).attr("r",i.line_height/2).attr("fill","none"),o.append("line").attr("x1",0).attr("x2",i.line_height).attr("y1",i.line_height/2).attr("y2",i.line_height/2).attr("stroke-width",1),o.append("line").attr("y1",0).attr("y2",i.line_height).attr("x1",i.line_height/2).attr("x2",i.line_height/2).attr("stroke-width",1),r.append("defs").append("marker").attr("id",ice.ARROW+"_line_ending").attr("refX",i.line_height).attr("refY",.5*i.line_height).attr("markerWidth",i.line_height).attr("markerHeight",i.line_height).attr("orient","auto").append("path").attr("d",`M0,0
      L${i.line_height},${i.line_height/2}
      M${i.line_height},${i.line_height/2}
      L0,${i.line_height}`).attr("stroke-width",1)}};let Al={},mLe=0;const yLe=(r,i)=>r.insert("rect","#"+i).attr("class","req reqBox").attr("x",0).attr("y",0).attr("width",Al.rect_min_width+"px").attr("height",Al.rect_min_height+"px"),kLe=(r,i,o)=>{let l=Al.rect_min_width/2,f=r.append("text").attr("class","req reqLabel reqTitle").attr("id",i).attr("x",l).attr("y",Al.rect_padding).attr("dominant-baseline","hanging"),b=0;o.forEach(k=>{b==0?f.append("tspan").attr("text-anchor","middle").attr("x",Al.rect_min_width/2).attr("dy",0).text(k):f.append("tspan").attr("text-anchor","middle").attr("x",Al.rect_min_width/2).attr("dy",Al.line_height*.75).text(k),b++});let d=1.5*Al.rect_padding,w=b*Al.line_height*.75,y=d+w;return r.append("line").attr("class","req-title-line").attr("x1","0").attr("x2",Al.rect_min_width).attr("y1",y).attr("y2",y),{titleNode:f,y}},xLe=(r,i,o,l)=>{let f=r.append("text").attr("class","req reqLabel").attr("id",i).attr("x",Al.rect_padding).attr("y",l).attr("dominant-baseline","hanging"),b=0;const d=30;let w=[];return o.forEach(y=>{let k=y.length;for(;k>d&&b<3;){let E=y.substring(0,d);y=y.substring(d,y.length),k=y.length,w[w.length]=E,b++}if(b==3){let E=w[w.length-1];w[w.length-1]=E.substring(0,E.length-4)+"..."}else w[w.length]=y;b=0}),w.forEach(y=>{f.append("tspan").attr("x",Al.rect_padding).attr("dy",Al.line_height).text(y)}),f},VQt=(r,i,o,l)=>{const f=i.node().getTotalLength(),b=i.node().getPointAtLength(f*.5),d="rel"+mLe;mLe++;const y=r.append("text").attr("class","req relationshipLabel").attr("id",d).attr("x",b.x).attr("y",b.y).attr("text-anchor","middle").attr("dominant-baseline","middle").text(l).node().getBBox();r.insert("rect","#"+d).attr("class","req reqLabelBox").attr("x",b.x-y.width/2).attr("y",b.y-y.height/2).attr("width",y.width).attr("height",y.height).attr("fill","white").attr("fill-opacity","85%")},UQt=function(r,i,o,l,f){const b=o.edge(xC(i.src),xC(i.dst)),d=V8().x(function(y){return y.x}).y(function(y){return y.y}),w=r.insert("path","#"+l).attr("class","er relationshipLine").attr("d",d(b.points)).attr("fill","none");i.type==f.db.Relationships.CONTAINS?w.attr("marker-start","url("+Cs.getUrl(Al.arrowMarkerAbsolute)+"#"+i.type+"_line_ending)"):(w.attr("stroke-dasharray","10,7"),w.attr("marker-end","url("+Cs.getUrl(Al.arrowMarkerAbsolute)+"#"+wLe.ReqMarkers.ARROW+"_line_ending)")),VQt(r,w,Al,`<<${i.type}>>`)},qQt=(r,i,o)=>{Object.keys(r).forEach(l=>{let f=r[l];l=xC(l),Se.info("Added new requirement: ",l);const b=o.append("g").attr("id",l),d="req-"+l,w=yLe(b,d);let y=kLe(b,l+"_title",[`<<${f.type}>>`,`${f.name}`]);xLe(b,l+"_body",[`Id: ${f.id}`,`Text: ${f.text}`,`Risk: ${f.risk}`,`Verification: ${f.verifyMethod}`],y.y);const k=w.node().getBBox();i.setNode(l,{width:k.width,height:k.height,shape:"rect",id:l})})},YQt=(r,i,o)=>{Object.keys(r).forEach(l=>{let f=r[l];const b=xC(l),d=o.append("g").attr("id",b),w="element-"+b,y=yLe(d,w);let k=kLe(d,w+"_title",["<<Element>>",`${l}`]);xLe(d,w+"_body",[`Type: ${f.type||"Not Specified"}`,`Doc Ref: ${f.docRef||"None"}`],k.y);const E=y.node().getBBox();i.setNode(b,{width:E.width,height:E.height,shape:"rect",id:b})})},WQt=(r,i)=>(r.forEach(function(o){let l=xC(o.src),f=xC(o.dst);i.setEdge(l,f,{relationship:o})}),r),KQt=function(r,i){i.nodes().forEach(function(o){o!==void 0&&i.node(o)!==void 0&&(r.select("#"+o),r.select("#"+o).attr("transform","translate("+(i.node(o).x-i.node(o).width/2)+","+(i.node(o).y-i.node(o).height/2)+" )"))})},xC=r=>r.replace(/\s/g,"").replace(/\./g,"_"),XQt={draw:(r,i,o,l)=>{Al=ht().requirement,l.db.clear(),l.parser.parse(r);const f=Al.securityLevel;let b;f==="sandbox"&&(b=Cn("#i"+i));const w=Cn(f==="sandbox"?b.nodes()[0].contentDocument.body:"body").select(`[id='${i}']`);wLe.insertLineEndings(w,Al);const y=new zh({multigraph:!1,compound:!1,directed:!0}).setGraph({rankdir:Al.layoutDirection,marginx:20,marginy:20,nodesep:100,edgesep:100,ranksep:100}).setDefaultEdgeLabel(function(){return{}});let k=l.db.getRequirements(),E=l.db.getElements(),T=l.db.getRelationships();qQt(k,y,w),YQt(E,y,w),WQt(T,y),bC(y),KQt(w,y),T.forEach(function(B){UQt(w,B,y,i,l)});const C=Al.rect_padding,S=w.node().getBBox(),L=S.width+C*2,O=S.height+C*2;Iv(w,O,L,Al.useMaxWidth),w.attr("viewBox",`${S.x-C} ${S.y-C} ${L} ${O}`)}};var sce=function(){var r=function(Oe,Le,$e,xe){for($e=$e||{},xe=Oe.length;xe--;$e[Oe[xe]]=Le);return $e},i=[1,2],o=[1,3],l=[1,5],f=[1,7],b=[2,5],d=[1,15],w=[1,17],y=[1,19],k=[1,21],E=[1,22],T=[1,23],C=[1,29],S=[1,30],L=[1,31],O=[1,32],B=[1,33],N=[1,34],F=[1,35],R=[1,36],q=[1,37],X=[1,38],te=[1,39],H=[1,40],Y=[1,42],z=[1,43],W=[1,45],Z=[1,46],G=[1,47],ae=[1,48],$=[1,49],ge=[1,50],ee=[1,53],de=[1,4,5,19,21,23,26,28,34,35,36,38,40,41,42,43,44,46,48,50,51,52,53,54,56,57,62,63,64,65,73,83],re=[4,5,21,54,56],ke=[4,5,19,21,23,26,28,34,35,36,38,40,41,42,43,44,46,48,50,54,56,57,62,63,64,65,73,83],Ce=[4,5,19,21,23,26,28,34,35,36,38,40,41,42,43,44,46,48,50,53,54,56,57,62,63,64,65,73,83],_e=[4,5,19,21,23,26,28,34,35,36,38,40,41,42,43,44,46,48,50,52,54,56,57,62,63,64,65,73,83],Te=[4,5,19,21,23,26,28,34,35,36,38,40,41,42,43,44,46,48,50,51,54,56,57,62,63,64,65,73,83],Be=[71,72,73],Ge=[1,125],Xe=[1,4,5,7,19,21,23,26,28,34,35,36,38,40,41,42,43,44,46,48,50,51,52,53,54,56,57,62,63,64,65,73,83],Ee={trace:function(){},yy:{},symbols_:{error:2,start:3,SPACE:4,NEWLINE:5,directive:6,SD:7,document:8,line:9,statement:10,box_section:11,box_line:12,participant_statement:13,openDirective:14,typeDirective:15,closeDirective:16,":":17,argDirective:18,box:19,restOfLine:20,end:21,signal:22,autonumber:23,NUM:24,off:25,activate:26,actor:27,deactivate:28,note_statement:29,links_statement:30,link_statement:31,properties_statement:32,details_statement:33,title:34,legacy_title:35,acc_title:36,acc_title_value:37,acc_descr:38,acc_descr_value:39,acc_descr_multiline_value:40,loop:41,rect:42,opt:43,alt:44,else_sections:45,par:46,par_sections:47,critical:48,option_sections:49,break:50,option:51,and:52,else:53,participant:54,AS:55,participant_actor:56,note:57,placement:58,text2:59,over:60,actor_pair:61,links:62,link:63,properties:64,details:65,spaceList:66,",":67,left_of:68,right_of:69,signaltype:70,"+":71,"-":72,ACTOR:73,SOLID_OPEN_ARROW:74,DOTTED_OPEN_ARROW:75,SOLID_ARROW:76,DOTTED_ARROW:77,SOLID_CROSS:78,DOTTED_CROSS:79,SOLID_POINT:80,DOTTED_POINT:81,TXT:82,open_directive:83,type_directive:84,arg_directive:85,close_directive:86,$accept:0,$end:1},terminals_:{2:"error",4:"SPACE",5:"NEWLINE",7:"SD",17:":",19:"box",20:"restOfLine",21:"end",23:"autonumber",24:"NUM",25:"off",26:"activate",28:"deactivate",34:"title",35:"legacy_title",36:"acc_title",37:"acc_title_value",38:"acc_descr",39:"acc_descr_value",40:"acc_descr_multiline_value",41:"loop",42:"rect",43:"opt",44:"alt",46:"par",48:"critical",50:"break",51:"option",52:"and",53:"else",54:"participant",55:"AS",56:"participant_actor",57:"note",60:"over",62:"links",63:"link",64:"properties",65:"details",67:",",68:"left_of",69:"right_of",71:"+",72:"-",73:"ACTOR",74:"SOLID_OPEN_ARROW",75:"DOTTED_OPEN_ARROW",76:"SOLID_ARROW",77:"DOTTED_ARROW",78:"SOLID_CROSS",79:"DOTTED_CROSS",80:"SOLID_POINT",81:"DOTTED_POINT",82:"TXT",83:"open_directive",84:"type_directive",85:"arg_directive",86:"close_directive"},productions_:[0,[3,2],[3,2],[3,2],[3,2],[8,0],[8,2],[9,2],[9,1],[9,1],[11,0],[11,2],[12,2],[12,1],[12,1],[6,4],[6,6],[10,1],[10,4],[10,2],[10,4],[10,3],[10,3],[10,2],[10,3],[10,3],[10,2],[10,2],[10,2],[10,2],[10,2],[10,1],[10,1],[10,2],[10,2],[10,1],[10,4],[10,4],[10,4],[10,4],[10,4],[10,4],[10,4],[10,1],[49,1],[49,4],[47,1],[47,4],[45,1],[45,4],[13,5],[13,3],[13,5],[13,3],[29,4],[29,4],[30,3],[31,3],[32,3],[33,3],[66,2],[66,1],[61,3],[61,1],[58,1],[58,1],[22,5],[22,5],[22,4],[27,1],[70,1],[70,1],[70,1],[70,1],[70,1],[70,1],[70,1],[70,1],[59,1],[14,1],[15,1],[18,1],[16,1]],performAction:function(Le,$e,xe,Ae,je,me,vt){var ve=me.length-1;switch(je){case 4:return Ae.apply(me[ve]),me[ve];case 5:case 10:this.$=[];break;case 6:case 11:me[ve-1].push(me[ve]),this.$=me[ve-1];break;case 7:case 8:case 12:case 13:this.$=me[ve];break;case 9:case 14:this.$=[];break;case 18:me[ve-1].unshift({type:"boxStart",boxData:Ae.parseBoxData(me[ve-2])}),me[ve-1].push({type:"boxEnd",boxText:me[ve-2]}),this.$=me[ve-1];break;case 20:this.$={type:"sequenceIndex",sequenceIndex:Number(me[ve-2]),sequenceIndexStep:Number(me[ve-1]),sequenceVisible:!0,signalType:Ae.LINETYPE.AUTONUMBER};break;case 21:this.$={type:"sequenceIndex",sequenceIndex:Number(me[ve-1]),sequenceIndexStep:1,sequenceVisible:!0,signalType:Ae.LINETYPE.AUTONUMBER};break;case 22:this.$={type:"sequenceIndex",sequenceVisible:!1,signalType:Ae.LINETYPE.AUTONUMBER};break;case 23:this.$={type:"sequenceIndex",sequenceVisible:!0,signalType:Ae.LINETYPE.AUTONUMBER};break;case 24:this.$={type:"activeStart",signalType:Ae.LINETYPE.ACTIVE_START,actor:me[ve-1]};break;case 25:this.$={type:"activeEnd",signalType:Ae.LINETYPE.ACTIVE_END,actor:me[ve-1]};break;case 31:Ae.setDiagramTitle(me[ve].substring(6)),this.$=me[ve].substring(6);break;case 32:Ae.setDiagramTitle(me[ve].substring(7)),this.$=me[ve].substring(7);break;case 33:this.$=me[ve].trim(),Ae.setAccTitle(this.$);break;case 34:case 35:this.$=me[ve].trim(),Ae.setAccDescription(this.$);break;case 36:me[ve-1].unshift({type:"loopStart",loopText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.LOOP_START}),me[ve-1].push({type:"loopEnd",loopText:me[ve-2],signalType:Ae.LINETYPE.LOOP_END}),this.$=me[ve-1];break;case 37:me[ve-1].unshift({type:"rectStart",color:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.RECT_START}),me[ve-1].push({type:"rectEnd",color:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.RECT_END}),this.$=me[ve-1];break;case 38:me[ve-1].unshift({type:"optStart",optText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.OPT_START}),me[ve-1].push({type:"optEnd",optText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.OPT_END}),this.$=me[ve-1];break;case 39:me[ve-1].unshift({type:"altStart",altText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.ALT_START}),me[ve-1].push({type:"altEnd",signalType:Ae.LINETYPE.ALT_END}),this.$=me[ve-1];break;case 40:me[ve-1].unshift({type:"parStart",parText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.PAR_START}),me[ve-1].push({type:"parEnd",signalType:Ae.LINETYPE.PAR_END}),this.$=me[ve-1];break;case 41:me[ve-1].unshift({type:"criticalStart",criticalText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.CRITICAL_START}),me[ve-1].push({type:"criticalEnd",signalType:Ae.LINETYPE.CRITICAL_END}),this.$=me[ve-1];break;case 42:me[ve-1].unshift({type:"breakStart",breakText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.BREAK_START}),me[ve-1].push({type:"breakEnd",optText:Ae.parseMessage(me[ve-2]),signalType:Ae.LINETYPE.BREAK_END}),this.$=me[ve-1];break;case 45:this.$=me[ve-3].concat([{type:"option",optionText:Ae.parseMessage(me[ve-1]),signalType:Ae.LINETYPE.CRITICAL_OPTION},me[ve]]);break;case 47:this.$=me[ve-3].concat([{type:"and",parText:Ae.parseMessage(me[ve-1]),signalType:Ae.LINETYPE.PAR_AND},me[ve]]);break;case 49:this.$=me[ve-3].concat([{type:"else",altText:Ae.parseMessage(me[ve-1]),signalType:Ae.LINETYPE.ALT_ELSE},me[ve]]);break;case 50:me[ve-3].type="addParticipant",me[ve-3].description=Ae.parseMessage(me[ve-1]),this.$=me[ve-3];break;case 51:me[ve-1].type="addParticipant",this.$=me[ve-1];break;case 52:me[ve-3].type="addActor",me[ve-3].description=Ae.parseMessage(me[ve-1]),this.$=me[ve-3];break;case 53:me[ve-1].type="addActor",this.$=me[ve-1];break;case 54:this.$=[me[ve-1],{type:"addNote",placement:me[ve-2],actor:me[ve-1].actor,text:me[ve]}];break;case 55:me[ve-2]=[].concat(me[ve-1],me[ve-1]).slice(0,2),me[ve-2][0]=me[ve-2][0].actor,me[ve-2][1]=me[ve-2][1].actor,this.$=[me[ve-1],{type:"addNote",placement:Ae.PLACEMENT.OVER,actor:me[ve-2].slice(0,2),text:me[ve]}];break;case 56:this.$=[me[ve-1],{type:"addLinks",actor:me[ve-1].actor,text:me[ve]}];break;case 57:this.$=[me[ve-1],{type:"addALink",actor:me[ve-1].actor,text:me[ve]}];break;case 58:this.$=[me[ve-1],{type:"addProperties",actor:me[ve-1].actor,text:me[ve]}];break;case 59:this.$=[me[ve-1],{type:"addDetails",actor:me[ve-1].actor,text:me[ve]}];break;case 62:this.$=[me[ve-2],me[ve]];break;case 63:this.$=me[ve];break;case 64:this.$=Ae.PLACEMENT.LEFTOF;break;case 65:this.$=Ae.PLACEMENT.RIGHTOF;break;case 66:this.$=[me[ve-4],me[ve-1],{type:"addMessage",from:me[ve-4].actor,to:me[ve-1].actor,signalType:me[ve-3],msg:me[ve]},{type:"activeStart",signalType:Ae.LINETYPE.ACTIVE_START,actor:me[ve-1]}];break;case 67:this.$=[me[ve-4],me[ve-1],{type:"addMessage",from:me[ve-4].actor,to:me[ve-1].actor,signalType:me[ve-3],msg:me[ve]},{type:"activeEnd",signalType:Ae.LINETYPE.ACTIVE_END,actor:me[ve-4]}];break;case 68:this.$=[me[ve-3],me[ve-1],{type:"addMessage",from:me[ve-3].actor,to:me[ve-1].actor,signalType:me[ve-2],msg:me[ve]}];break;case 69:this.$={type:"addParticipant",actor:me[ve]};break;case 70:this.$=Ae.LINETYPE.SOLID_OPEN;break;case 71:this.$=Ae.LINETYPE.DOTTED_OPEN;break;case 72:this.$=Ae.LINETYPE.SOLID;break;case 73:this.$=Ae.LINETYPE.DOTTED;break;case 74:this.$=Ae.LINETYPE.SOLID_CROSS;break;case 75:this.$=Ae.LINETYPE.DOTTED_CROSS;break;case 76:this.$=Ae.LINETYPE.SOLID_POINT;break;case 77:this.$=Ae.LINETYPE.DOTTED_POINT;break;case 78:this.$=Ae.parseMessage(me[ve].trim().substring(1));break;case 79:Ae.parseDirective("%%{","open_directive");break;case 80:Ae.parseDirective(me[ve],"type_directive");break;case 81:me[ve]=me[ve].trim().replace(/'/g,'"'),Ae.parseDirective(me[ve],"arg_directive");break;case 82:Ae.parseDirective("}%%","close_directive","sequence");break}},table:[{3:1,4:i,5:o,6:4,7:l,14:6,83:f},{1:[3]},{3:8,4:i,5:o,6:4,7:l,14:6,83:f},{3:9,4:i,5:o,6:4,7:l,14:6,83:f},{3:10,4:i,5:o,6:4,7:l,14:6,83:f},r([1,4,5,19,23,26,28,34,35,36,38,40,41,42,43,44,46,48,50,54,56,57,62,63,64,65,73,83],b,{8:11}),{15:12,84:[1,13]},{84:[2,79]},{1:[2,1]},{1:[2,2]},{1:[2,3]},{1:[2,4],4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{16:51,17:[1,52],86:ee},r([17,86],[2,80]),r(de,[2,6]),{6:41,10:54,13:18,14:6,19:y,22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},r(de,[2,8]),r(de,[2,9]),r(de,[2,17]),{20:[1,55]},{5:[1,56]},{5:[1,59],24:[1,57],25:[1,58]},{27:60,73:ge},{27:61,73:ge},{5:[1,62]},{5:[1,63]},{5:[1,64]},{5:[1,65]},{5:[1,66]},r(de,[2,31]),r(de,[2,32]),{37:[1,67]},{39:[1,68]},r(de,[2,35]),{20:[1,69]},{20:[1,70]},{20:[1,71]},{20:[1,72]},{20:[1,73]},{20:[1,74]},{20:[1,75]},r(de,[2,43]),{27:76,73:ge},{27:77,73:ge},{70:78,74:[1,79],75:[1,80],76:[1,81],77:[1,82],78:[1,83],79:[1,84],80:[1,85],81:[1,86]},{58:87,60:[1,88],68:[1,89],69:[1,90]},{27:91,73:ge},{27:92,73:ge},{27:93,73:ge},{27:94,73:ge},r([5,55,67,74,75,76,77,78,79,80,81,82],[2,69]),{5:[1,95]},{18:96,85:[1,97]},{5:[2,82]},r(de,[2,7]),r(re,[2,10],{11:98}),r(de,[2,19]),{5:[1,100],24:[1,99]},{5:[1,101]},r(de,[2,23]),{5:[1,102]},{5:[1,103]},r(de,[2,26]),r(de,[2,27]),r(de,[2,28]),r(de,[2,29]),r(de,[2,30]),r(de,[2,33]),r(de,[2,34]),r(ke,b,{8:104}),r(ke,b,{8:105}),r(ke,b,{8:106}),r(Ce,b,{45:107,8:108}),r(_e,b,{47:109,8:110}),r(Te,b,{49:111,8:112}),r(ke,b,{8:113}),{5:[1,115],55:[1,114]},{5:[1,117],55:[1,116]},{27:120,71:[1,118],72:[1,119],73:ge},r(Be,[2,70]),r(Be,[2,71]),r(Be,[2,72]),r(Be,[2,73]),r(Be,[2,74]),r(Be,[2,75]),r(Be,[2,76]),r(Be,[2,77]),{27:121,73:ge},{27:123,61:122,73:ge},{73:[2,64]},{73:[2,65]},{59:124,82:Ge},{59:126,82:Ge},{59:127,82:Ge},{59:128,82:Ge},r(Xe,[2,15]),{16:129,86:ee},{86:[2,81]},{4:[1,132],5:[1,134],12:131,13:133,21:[1,130],54:Y,56:z},{5:[1,135]},r(de,[2,21]),r(de,[2,22]),r(de,[2,24]),r(de,[2,25]),{4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,21:[1,136],22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,21:[1,137],22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,21:[1,138],22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{21:[1,139]},{4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,21:[2,48],22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,53:[1,140],54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{21:[1,141]},{4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,21:[2,46],22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,52:[1,142],54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{21:[1,143]},{4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,21:[2,44],22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,51:[1,144],54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{4:d,5:w,6:41,9:14,10:16,13:18,14:6,19:y,21:[1,145],22:20,23:k,26:E,27:44,28:T,29:24,30:25,31:26,32:27,33:28,34:C,35:S,36:L,38:O,40:B,41:N,42:F,43:R,44:q,46:X,48:te,50:H,54:Y,56:z,57:W,62:Z,63:G,64:ae,65:$,73:ge,83:f},{20:[1,146]},r(de,[2,51]),{20:[1,147]},r(de,[2,53]),{27:148,73:ge},{27:149,73:ge},{59:150,82:Ge},{59:151,82:Ge},{59:152,82:Ge},{67:[1,153],82:[2,63]},{5:[2,56]},{5:[2,78]},{5:[2,57]},{5:[2,58]},{5:[2,59]},{5:[1,154]},r(de,[2,18]),r(re,[2,11]),{13:155,54:Y,56:z},r(re,[2,13]),r(re,[2,14]),r(de,[2,20]),r(de,[2,36]),r(de,[2,37]),r(de,[2,38]),r(de,[2,39]),{20:[1,156]},r(de,[2,40]),{20:[1,157]},r(de,[2,41]),{20:[1,158]},r(de,[2,42]),{5:[1,159]},{5:[1,160]},{59:161,82:Ge},{59:162,82:Ge},{5:[2,68]},{5:[2,54]},{5:[2,55]},{27:163,73:ge},r(Xe,[2,16]),r(re,[2,12]),r(Ce,b,{8:108,45:164}),r(_e,b,{8:110,47:165}),r(Te,b,{8:112,49:166}),r(de,[2,50]),r(de,[2,52]),{5:[2,66]},{5:[2,67]},{82:[2,62]},{21:[2,49]},{21:[2,47]},{21:[2,45]}],defaultActions:{7:[2,79],8:[2,1],9:[2,2],10:[2,3],53:[2,82],89:[2,64],90:[2,65],97:[2,81],124:[2,56],125:[2,78],126:[2,57],127:[2,58],128:[2,59],150:[2,68],151:[2,54],152:[2,55],161:[2,66],162:[2,67],163:[2,62],164:[2,49],165:[2,47],166:[2,45]},parseError:function(Le,$e){if($e.recoverable)this.trace(Le);else{var xe=new Error(Le);throw xe.hash=$e,xe}},parse:function(Le){var $e=this,xe=[0],Ae=[],je=[null],me=[],vt=this.table,ve="",Zt=0,nt=0,xn=2,cn=1,jt=me.slice.call(arguments,1),ot=Object.create(this.lexer),be={yy:{}};for(var We in this.yy)Object.prototype.hasOwnProperty.call(this.yy,We)&&(be.yy[We]=this.yy[We]);ot.setInput(Le,be.yy),be.yy.lexer=ot,be.yy.parser=this,typeof ot.yylloc>"u"&&(ot.yylloc={});var ct=ot.yylloc;me.push(ct);var Yt=ot.options&&ot.options.ranges;typeof be.yy.parseError=="function"?this.parseError=be.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Ut(){var hn;return hn=Ae.pop()||ot.lex()||cn,typeof hn!="number"&&(hn instanceof Array&&(Ae=hn,hn=Ae.pop()),hn=$e.symbols_[hn]||hn),hn}for(var Wn,Gt,Rn,si,$r={},nr,Kn,Jt,en;;){if(Gt=xe[xe.length-1],this.defaultActions[Gt]?Rn=this.defaultActions[Gt]:((Wn===null||typeof Wn>"u")&&(Wn=Ut()),Rn=vt[Gt]&&vt[Gt][Wn]),typeof Rn>"u"||!Rn.length||!Rn[0]){var In="";en=[];for(nr in vt[Gt])this.terminals_[nr]&&nr>xn&&en.push("'"+this.terminals_[nr]+"'");ot.showPosition?In="Parse error on line "+(Zt+1)+`:
`+ot.showPosition()+`
Expecting `+en.join(", ")+", got '"+(this.terminals_[Wn]||Wn)+"'":In="Parse error on line "+(Zt+1)+": Unexpected "+(Wn==cn?"end of input":"'"+(this.terminals_[Wn]||Wn)+"'"),this.parseError(In,{text:ot.match,token:this.terminals_[Wn]||Wn,line:ot.yylineno,loc:ct,expected:en})}if(Rn[0]instanceof Array&&Rn.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Gt+", token: "+Wn);switch(Rn[0]){case 1:xe.push(Wn),je.push(ot.yytext),me.push(ot.yylloc),xe.push(Rn[1]),Wn=null,nt=ot.yyleng,ve=ot.yytext,Zt=ot.yylineno,ct=ot.yylloc;break;case 2:if(Kn=this.productions_[Rn[1]][1],$r.$=je[je.length-Kn],$r._$={first_line:me[me.length-(Kn||1)].first_line,last_line:me[me.length-1].last_line,first_column:me[me.length-(Kn||1)].first_column,last_column:me[me.length-1].last_column},Yt&&($r._$.range=[me[me.length-(Kn||1)].range[0],me[me.length-1].range[1]]),si=this.performAction.apply($r,[ve,nt,Zt,be.yy,Rn[1],je,me].concat(jt)),typeof si<"u")return si;Kn&&(xe=xe.slice(0,-1*Kn*2),je=je.slice(0,-1*Kn),me=me.slice(0,-1*Kn)),xe.push(this.productions_[Rn[1]][0]),je.push($r.$),me.push($r._$),Jt=vt[xe[xe.length-2]][xe[xe.length-1]],xe.push(Jt);break;case 3:return!0}}return!0}},Ze=function(){var Oe={EOF:1,parseError:function($e,xe){if(this.yy.parser)this.yy.parser.parseError($e,xe);else throw new Error($e)},setInput:function(Le,$e){return this.yy=$e||this.yy||{},this._input=Le,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var Le=this._input[0];this.yytext+=Le,this.yyleng++,this.offset++,this.match+=Le,this.matched+=Le;var $e=Le.match(/(?:\r\n?|\n).*/g);return $e?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),Le},unput:function(Le){var $e=Le.length,xe=Le.split(/(?:\r\n?|\n)/g);this._input=Le+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-$e),this.offset-=$e;var Ae=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),xe.length-1&&(this.yylineno-=xe.length-1);var je=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:xe?(xe.length===Ae.length?this.yylloc.first_column:0)+Ae[Ae.length-xe.length].length-xe[0].length:this.yylloc.first_column-$e},this.options.ranges&&(this.yylloc.range=[je[0],je[0]+this.yyleng-$e]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(Le){this.unput(this.match.slice(Le))},pastInput:function(){var Le=this.matched.substr(0,this.matched.length-this.match.length);return(Le.length>20?"...":"")+Le.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var Le=this.match;return Le.length<20&&(Le+=this._input.substr(0,20-Le.length)),(Le.substr(0,20)+(Le.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var Le=this.pastInput(),$e=new Array(Le.length+1).join("-");return Le+this.upcomingInput()+`
`+$e+"^"},test_match:function(Le,$e){var xe,Ae,je;if(this.options.backtrack_lexer&&(je={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(je.yylloc.range=this.yylloc.range.slice(0))),Ae=Le[0].match(/(?:\r\n?|\n).*/g),Ae&&(this.yylineno+=Ae.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:Ae?Ae[Ae.length-1].length-Ae[Ae.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+Le[0].length},this.yytext+=Le[0],this.match+=Le[0],this.matches=Le,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(Le[0].length),this.matched+=Le[0],xe=this.performAction.call(this,this.yy,this,$e,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),xe)return xe;if(this._backtrack){for(var me in je)this[me]=je[me];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var Le,$e,xe,Ae;this._more||(this.yytext="",this.match="");for(var je=this._currentRules(),me=0;me<je.length;me++)if(xe=this._input.match(this.rules[je[me]]),xe&&(!$e||xe[0].length>$e[0].length)){if($e=xe,Ae=me,this.options.backtrack_lexer){if(Le=this.test_match(xe,je[me]),Le!==!1)return Le;if(this._backtrack){$e=!1;continue}else return!1}else if(!this.options.flex)break}return $e?(Le=this.test_match($e,je[Ae]),Le!==!1?Le:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var $e=this.next();return $e||this.lex()},begin:function($e){this.conditionStack.push($e)},popState:function(){var $e=this.conditionStack.length-1;return $e>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function($e){return $e=this.conditionStack.length-1-Math.abs($e||0),$e>=0?this.conditionStack[$e]:"INITIAL"},pushState:function($e){this.begin($e)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function($e,xe,Ae,je){switch(Ae){case 0:return this.begin("open_directive"),83;case 1:return this.begin("type_directive"),84;case 2:return this.popState(),this.begin("arg_directive"),17;case 3:return this.popState(),this.popState(),86;case 4:return 85;case 5:return 5;case 6:break;case 7:break;case 8:break;case 9:break;case 10:break;case 11:return 24;case 12:return this.begin("LINE"),19;case 13:return this.begin("ID"),54;case 14:return this.begin("ID"),56;case 15:return xe.yytext=xe.yytext.trim(),this.begin("ALIAS"),73;case 16:return this.popState(),this.popState(),this.begin("LINE"),55;case 17:return this.popState(),this.popState(),5;case 18:return this.begin("LINE"),41;case 19:return this.begin("LINE"),42;case 20:return this.begin("LINE"),43;case 21:return this.begin("LINE"),44;case 22:return this.begin("LINE"),53;case 23:return this.begin("LINE"),46;case 24:return this.begin("LINE"),52;case 25:return this.begin("LINE"),48;case 26:return this.begin("LINE"),51;case 27:return this.begin("LINE"),50;case 28:return this.popState(),20;case 29:return 21;case 30:return 68;case 31:return 69;case 32:return 62;case 33:return 63;case 34:return 64;case 35:return 65;case 36:return 60;case 37:return 57;case 38:return this.begin("ID"),26;case 39:return this.begin("ID"),28;case 40:return 34;case 41:return 35;case 42:return this.begin("acc_title"),36;case 43:return this.popState(),"acc_title_value";case 44:return this.begin("acc_descr"),38;case 45:return this.popState(),"acc_descr_value";case 46:this.begin("acc_descr_multiline");break;case 47:this.popState();break;case 48:return"acc_descr_multiline_value";case 49:return 7;case 50:return 23;case 51:return 25;case 52:return 67;case 53:return 5;case 54:return xe.yytext=xe.yytext.trim(),73;case 55:return 76;case 56:return 77;case 57:return 74;case 58:return 75;case 59:return 78;case 60:return 79;case 61:return 80;case 62:return 81;case 63:return 82;case 64:return 71;case 65:return 72;case 66:return 5;case 67:return"INVALID"}},rules:[/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:[\n]+)/i,/^(?:\s+)/i,/^(?:((?!\n)\s)+)/i,/^(?:#[^\n]*)/i,/^(?:%(?!\{)[^\n]*)/i,/^(?:[^\}]%%[^\n]*)/i,/^(?:[0-9]+(?=[ \n]+))/i,/^(?:box\b)/i,/^(?:participant\b)/i,/^(?:actor\b)/i,/^(?:[^\->:\n,;]+?([\-]*[^\->:\n,;]+?)*?(?=((?!\n)\s)+as(?!\n)\s|[#\n;]|$))/i,/^(?:as\b)/i,/^(?:(?:))/i,/^(?:loop\b)/i,/^(?:rect\b)/i,/^(?:opt\b)/i,/^(?:alt\b)/i,/^(?:else\b)/i,/^(?:par\b)/i,/^(?:and\b)/i,/^(?:critical\b)/i,/^(?:option\b)/i,/^(?:break\b)/i,/^(?:(?:[:]?(?:no)?wrap)?[^#\n;]*)/i,/^(?:end\b)/i,/^(?:left of\b)/i,/^(?:right of\b)/i,/^(?:links\b)/i,/^(?:link\b)/i,/^(?:properties\b)/i,/^(?:details\b)/i,/^(?:over\b)/i,/^(?:note\b)/i,/^(?:activate\b)/i,/^(?:deactivate\b)/i,/^(?:title\s[^#\n;]+)/i,/^(?:title:\s[^#\n;]+)/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:sequenceDiagram\b)/i,/^(?:autonumber\b)/i,/^(?:off\b)/i,/^(?:,)/i,/^(?:;)/i,/^(?:[^\+\->:\n,;]+((?!(-x|--x|-\)|--\)))[\-]*[^\+\->:\n,;]+)*)/i,/^(?:->>)/i,/^(?:-->>)/i,/^(?:->)/i,/^(?:-->)/i,/^(?:-[x])/i,/^(?:--[x])/i,/^(?:-[\)])/i,/^(?:--[\)])/i,/^(?::(?:(?:no)?wrap)?[^#\n;]+)/i,/^(?:\+)/i,/^(?:-)/i,/^(?:$)/i,/^(?:.)/i],conditions:{acc_descr_multiline:{rules:[47,48],inclusive:!1},acc_descr:{rules:[45],inclusive:!1},acc_title:{rules:[43],inclusive:!1},open_directive:{rules:[1,8],inclusive:!1},type_directive:{rules:[2,3,8],inclusive:!1},arg_directive:{rules:[3,4,8],inclusive:!1},ID:{rules:[7,8,15],inclusive:!1},ALIAS:{rules:[7,8,16,17],inclusive:!1},LINE:{rules:[7,8,28],inclusive:!1},INITIAL:{rules:[0,5,6,8,9,10,11,12,13,14,18,19,20,21,22,23,24,25,26,27,29,30,31,32,33,34,35,36,37,38,39,40,41,42,44,46,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67],inclusive:!0}}};return Oe}();Ee.lexer=Ze;function Ie(){this.yy={}}return Ie.prototype=Ee,Ee.Parser=Ie,new Ie}();sce.parser=sce;const QQt=sce,ZQt=r=>r.match(/^\s*sequenceDiagram/)!==null;let YI,w6={},EC=[],Hv=[],UV=!1,ace,c4;const JQt=function(r,i,o){Bu.parseDirective(this,r,i,o)},eZt=function(r){EC.push({name:r.text,wrap:r.wrap===void 0&&f7()||!!r.wrap,fill:r.color,actorKeys:[]}),c4=EC.slice(-1)[0]},oce=function(r,i,o,l){let f=c4;const b=w6[r];if(b){if(c4&&b.box&&c4!==b.box)throw new Error("A same participant should only be defined in one Box: "+b.name+" can't be in '"+b.box.name+"' and in '"+c4.name+"' at the same time.");if(f=b.box?b.box:c4,b.box=f,b&&i===b.name&&o==null)return}(o==null||o.text==null)&&(o={text:i,wrap:null,type:l}),(l==null||o.text==null)&&(o={text:i,wrap:null,type:l}),w6[r]={box:f,name:i,description:o.text,wrap:o.wrap===void 0&&f7()||!!o.wrap,prevActor:YI,links:{},properties:{},actorCnt:null,rectData:null,type:l||"participant"},YI&&w6[YI]&&(w6[YI].nextActor=r),c4&&c4.actorKeys.push(r),YI=r},tZt=r=>{let i,o=0;for(i=0;i<Hv.length;i++)Hv[i].type===KI.ACTIVE_START&&Hv[i].from.actor===r&&o++,Hv[i].type===KI.ACTIVE_END&&Hv[i].from.actor===r&&o--;return o},nZt=function(r,i,o,l){Hv.push({from:r,to:i,message:o.text,wrap:o.wrap===void 0&&f7()||!!o.wrap,answer:l})},uh=function(r,i,o={text:void 0,wrap:void 0},l){if(l===KI.ACTIVE_END&&tZt(r.actor)<1){let b=new Error("Trying to inactivate an inactive participant ("+r.actor+")");throw b.hash={text:"->>-",token:"->>-",line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:["'ACTIVE_PARTICIPANT'"]},b}return Hv.push({from:r,to:i,message:o.text,wrap:o.wrap===void 0&&f7()||!!o.wrap,type:l}),!0},rZt=function(){return EC.length>0},iZt=function(){return EC.some(r=>r.name)},sZt=function(){return Hv},aZt=function(){return EC},oZt=function(){return w6},WI=function(r){return w6[r]},cZt=function(){return Object.keys(w6)},uZt=function(){UV=!0},lZt=function(){UV=!1},hZt=()=>UV,fZt=function(r){ace=r},f7=()=>ace!==void 0?ace:ht().sequence.wrap,dZt=function(){w6={},EC=[],Hv=[],UV=!1,W0()},gZt=function(r){const i=r.trim(),o={text:i.replace(/^:?(?:no)?wrap:/,"").trim(),wrap:i.match(/^:?wrap:/)!==null?!0:i.match(/^:?nowrap:/)!==null?!1:void 0};return Se.debug("parseMessage:",o),o},pZt=function(r){const i=r.match(/^((?:rgba?|hsla?)\s*\(.*\)|\w*)(.*)$/);let o=i!=null&&i[1]?i[1].trim():"transparent",l=i!=null&&i[2]?i[2].trim():void 0;if(window&&window.CSS)window.CSS.supports("color",o)||(o="transparent",l=r.trim());else{const b=new Option().style;b.color=o,b.color!==o&&(o="transparent",l=r.trim())}return{color:o,text:l!==void 0?up(l.replace(/^:?(?:no)?wrap:/,""),ht()):void 0,wrap:l!==void 0?l.match(/^:?wrap:/)!==null?!0:l.match(/^:?nowrap:/)!==null?!1:void 0:void 0}},KI={SOLID:0,DOTTED:1,NOTE:2,SOLID_CROSS:3,DOTTED_CROSS:4,SOLID_OPEN:5,DOTTED_OPEN:6,LOOP_START:10,LOOP_END:11,ALT_START:12,ALT_ELSE:13,ALT_END:14,OPT_START:15,OPT_END:16,ACTIVE_START:17,ACTIVE_END:18,PAR_START:19,PAR_AND:20,PAR_END:21,RECT_START:22,RECT_END:23,SOLID_POINT:24,DOTTED_POINT:25,AUTONUMBER:26,CRITICAL_START:27,CRITICAL_OPTION:28,CRITICAL_END:29,BREAK_START:30,BREAK_END:31},bZt={FILLED:0,OPEN:1},vZt={LEFTOF:0,RIGHTOF:1,OVER:2},ELe=function(r,i,o){o.text,o.wrap===void 0&&f7()||o.wrap;const l=[].concat(r,r);Hv.push({from:l[0],to:l[1],message:o.text,wrap:o.wrap===void 0&&f7()||!!o.wrap,type:KI.NOTE,placement:i})},_Le=function(r,i){const o=WI(r);try{let l=up(i.text,ht());l=l.replace(/&amp;/g,"&"),l=l.replace(/&equals;/g,"=");const f=JSON.parse(l);cce(o,f)}catch(l){Se.error("error while parsing actor link text",l)}},wZt=function(r,i){const o=WI(r);try{const d={};let w=up(i.text,ht());var l=w.indexOf("@");w=w.replace(/&amp;/g,"&"),w=w.replace(/&equals;/g,"=");var f=w.slice(0,l-1).trim(),b=w.slice(l+1).trim();d[f]=b,cce(o,d)}catch(d){Se.error("error while parsing actor link text",d)}};function cce(r,i){if(r.links==null)r.links=i;else for(let o in i)r.links[o]=i[o]}const TLe=function(r,i){const o=WI(r);try{let l=up(i.text,ht());const f=JSON.parse(l);CLe(o,f)}catch(l){Se.error("error while parsing actor properties text",l)}};function CLe(r,i){if(r.properties==null)r.properties=i;else for(let o in i)r.properties[o]=i[o]}function mZt(){c4=void 0}const SLe=function(r,i){const o=WI(r),l=document.getElementById(i.text);try{const f=l.innerHTML,b=JSON.parse(f);b.properties&&CLe(o,b.properties),b.links&&cce(o,b.links)}catch(f){Se.error("error while parsing actor details text",f)}},yZt=function(r,i){if(r!==void 0&&r.properties!==void 0)return r.properties[i]},ALe=function(r){if(Array.isArray(r))r.forEach(function(i){ALe(i)});else switch(r.type){case"sequenceIndex":Hv.push({from:void 0,to:void 0,message:{start:r.sequenceIndex,step:r.sequenceIndexStep,visible:r.sequenceVisible},wrap:!1,type:r.signalType});break;case"addParticipant":oce(r.actor,r.actor,r.description,"participant");break;case"addActor":oce(r.actor,r.actor,r.description,"actor");break;case"activeStart":uh(r.actor,void 0,void 0,r.signalType);break;case"activeEnd":uh(r.actor,void 0,void 0,r.signalType);break;case"addNote":ELe(r.actor,r.placement,r.text);break;case"addLinks":_Le(r.actor,r.text);break;case"addALink":wZt(r.actor,r.text);break;case"addProperties":TLe(r.actor,r.text);break;case"addDetails":SLe(r.actor,r.text);break;case"addMessage":uh(r.from,r.to,r.msg,r.signalType);break;case"boxStart":eZt(r.boxData);break;case"boxEnd":mZt();break;case"loopStart":uh(void 0,void 0,r.loopText,r.signalType);break;case"loopEnd":uh(void 0,void 0,void 0,r.signalType);break;case"rectStart":uh(void 0,void 0,r.color,r.signalType);break;case"rectEnd":uh(void 0,void 0,void 0,r.signalType);break;case"optStart":uh(void 0,void 0,r.optText,r.signalType);break;case"optEnd":uh(void 0,void 0,void 0,r.signalType);break;case"altStart":uh(void 0,void 0,r.altText,r.signalType);break;case"else":uh(void 0,void 0,r.altText,r.signalType);break;case"altEnd":uh(void 0,void 0,void 0,r.signalType);break;case"setAccTitle":K0(r.text);break;case"parStart":uh(void 0,void 0,r.parText,r.signalType);break;case"and":uh(void 0,void 0,r.parText,r.signalType);break;case"parEnd":uh(void 0,void 0,void 0,r.signalType);break;case"criticalStart":uh(void 0,void 0,r.criticalText,r.signalType);break;case"option":uh(void 0,void 0,r.optionText,r.signalType);break;case"criticalEnd":uh(void 0,void 0,void 0,r.signalType);break;case"breakStart":uh(void 0,void 0,r.breakText,r.signalType);break;case"breakEnd":uh(void 0,void 0,void 0,r.signalType);break}},MLe={addActor:oce,addMessage:nZt,addSignal:uh,addLinks:_Le,addDetails:SLe,addProperties:TLe,autoWrap:f7,setWrap:fZt,enableSequenceNumbers:uZt,disableSequenceNumbers:lZt,showSequenceNumbers:hZt,getMessages:sZt,getActors:oZt,getActor:WI,getActorKeys:cZt,getActorProperty:yZt,getAccTitle:hp,getBoxes:aZt,getDiagramTitle:Nv,setDiagramTitle:Ov,parseDirective:JQt,getConfig:()=>ht().sequence,clear:dZt,parseMessage:gZt,parseBoxData:pZt,LINETYPE:KI,ARROWTYPE:bZt,PLACEMENT:vZt,addNote:ELe,setAccTitle:K0,apply:ALe,setAccDescription:fp,getAccDescription:dp,hasAtLeastOneBox:rZt,hasAtLeastOneBoxWithTitle:iZt};let uce=[];const kZt=r=>{uce.push(r)},DLe=()=>{uce.forEach(r=>{r()}),uce=[]},qV=function(r,i){const o=r.append("rect");return o.attr("x",i.x),o.attr("y",i.y),o.attr("fill",i.fill),o.attr("stroke",i.stroke),o.attr("width",i.width),o.attr("height",i.height),o.attr("rx",i.rx),o.attr("ry",i.ry),i.class!==void 0&&o.attr("class",i.class),o},LLe=(r,i)=>{kZt(()=>{const o=document.querySelectorAll(r);o.length!==0&&(o[0].addEventListener("mouseover",function(){TZt("actor"+i+"_popup")}),o[0].addEventListener("mouseout",function(){CZt("actor"+i+"_popup")}))})},xZt=function(r,i,o,l,f){if(i.links===void 0||i.links===null||Object.keys(i.links).length===0)return{height:0,width:0};const b=i.links,d=i.actorCnt,w=i.rectData;var y="none";f&&(y="block !important");const k=r.append("g");k.attr("id","actor"+d+"_popup"),k.attr("class","actorPopupMenu"),k.attr("display",y),LLe("#actor"+d+"_popup",d);var E="";w.class!==void 0&&(E=" "+w.class);let T=w.width>o?w.width:o;const C=k.append("rect");if(C.attr("class","actorPopupMenuPanel"+E),C.attr("x",w.x),C.attr("y",w.height),C.attr("fill",w.fill),C.attr("stroke",w.stroke),C.attr("width",T),C.attr("height",w.height),C.attr("rx",w.rx),C.attr("ry",w.ry),b!=null){var S=20;for(let B in b){var L=k.append("a"),O=$3(b[B]);L.attr("xlink:href",O),L.attr("target","_blank"),HZt(l)(B,L,w.x+10,w.height+S,T,20,{class:"actor"},l),S+=30}}return C.attr("height",S),{height:w.height+S,width:T}},ILe=function(r,i,o,l){const f=r.append("image");f.attr("x",i),f.attr("y",o);var b=$3(l);f.attr("xlink:href",b)},OLe=function(r,i,o,l){const f=r.append("use");f.attr("x",i),f.attr("y",o);var b=$3(l);f.attr("xlink:href","#"+b)},EZt=function(r){return"var pu = document.getElementById('"+r+"'); if (pu != null) { pu.style.display = 'block'; }"},_Zt=function(r){return"var pu = document.getElementById('"+r+"'); if (pu != null) { pu.style.display = 'none'; }"},TZt=function(r){var i=document.getElementById(r);i!=null&&(i.style.display="block")},CZt=function(r){var i=document.getElementById(r);i!=null&&(i.style.display="none")},_C=function(r,i){let o=0,l=0;const f=i.text.split(Cs.lineBreakRegex),[b,d]=oI(i.fontSize);let w=[],y=0,k=()=>i.y;if(i.valign!==void 0&&i.textMargin!==void 0&&i.textMargin>0)switch(i.valign){case"top":case"start":k=()=>Math.round(i.y+i.textMargin);break;case"middle":case"center":k=()=>Math.round(i.y+(o+l+i.textMargin)/2);break;case"bottom":case"end":k=()=>Math.round(i.y+(o+l+2*i.textMargin)-i.textMargin);break}if(i.anchor!==void 0&&i.textMargin!==void 0&&i.width!==void 0)switch(i.anchor){case"left":case"start":i.x=Math.round(i.x+i.textMargin),i.anchor="start",i.dominantBaseline="middle",i.alignmentBaseline="middle";break;case"middle":case"center":i.x=Math.round(i.x+i.width/2),i.anchor="middle",i.dominantBaseline="middle",i.alignmentBaseline="middle";break;case"right":case"end":i.x=Math.round(i.x+i.width-i.textMargin),i.anchor="end",i.dominantBaseline="middle",i.alignmentBaseline="middle";break}for(let[E,T]of f.entries()){i.textMargin!==void 0&&i.textMargin===0&&b!==void 0&&(y=E*b);const C=r.append("text");if(C.attr("x",i.x),C.attr("y",k()),i.anchor!==void 0&&C.attr("text-anchor",i.anchor).attr("dominant-baseline",i.dominantBaseline).attr("alignment-baseline",i.alignmentBaseline),i.fontFamily!==void 0&&C.style("font-family",i.fontFamily),d!==void 0&&C.style("font-size",d),i.fontWeight!==void 0&&C.style("font-weight",i.fontWeight),i.fill!==void 0&&C.attr("fill",i.fill),i.class!==void 0&&C.attr("class",i.class),i.dy!==void 0?C.attr("dy",i.dy):y!==0&&C.attr("dy",y),i.tspan){const S=C.append("tspan");S.attr("x",i.x),i.fill!==void 0&&S.attr("fill",i.fill),S.text(T)}else C.text(T);i.valign!==void 0&&i.textMargin!==void 0&&i.textMargin>0&&(l+=(C._groups||C)[0][0].getBBox().height,o=l),w.push(C)}return w},NLe=function(r,i){function o(f,b,d,w,y){return f+","+b+" "+(f+d)+","+b+" "+(f+d)+","+(b+w-y)+" "+(f+d-y*1.2)+","+(b+w)+" "+f+","+(b+w)}const l=r.append("polygon");return l.attr("points",o(i.x,i.y,i.width,i.height,7)),l.attr("class","labelBox"),i.y=i.y+i.height/2,_C(r,i),l};let Vm=-1;const PLe=(r,i)=>{!r.selectAll||r.selectAll(".actor-line").attr("class","200").attr("y2",i-55)},SZt=function(r,i,o,l){const f=i.x+i.width/2,b=i.y+5,d=r.append("g");var w=d;l||(Vm++,w.append("line").attr("id","actor"+Vm).attr("x1",f).attr("y1",b).attr("x2",f).attr("y2",2e3).attr("class","actor-line").attr("stroke-width","0.5px").attr("stroke","#999"),w=d.append("g"),i.actorCnt=Vm,i.links!=null&&(w.attr("id","root-"+Vm),LLe("#root-"+Vm,Vm)));const y=YV();var k="actor";i.properties!=null&&i.properties.class?k=i.properties.class:y.fill="#eaeaea",y.x=i.x,y.y=i.y,y.width=i.width,y.height=i.height,y.class=k,y.rx=3,y.ry=3;const E=qV(w,y);if(i.rectData=y,i.properties!=null&&i.properties.icon){const C=i.properties.icon.trim();C.charAt(0)==="@"?OLe(w,y.x+y.width-20,y.y+10,C.substr(1)):ILe(w,y.x+y.width-20,y.y+10,C)}hce(o)(i.description,w,y.x,y.y,y.width,y.height,{class:"actor"},o);let T=i.height;if(E.node){const C=E.node().getBBox();i.height=C.height,T=C.height}return T},AZt=function(r,i,o,l){const f=i.x+i.width/2,b=i.y+80;l||(Vm++,r.append("line").attr("id","actor"+Vm).attr("x1",f).attr("y1",b).attr("x2",f).attr("y2",2e3).attr("class","actor-line").attr("stroke-width","0.5px").attr("stroke","#999"));const d=r.append("g");d.attr("class","actor-man");const w=YV();w.x=i.x,w.y=i.y,w.fill="#eaeaea",w.width=i.width,w.height=i.height,w.class="actor",w.rx=3,w.ry=3,d.append("line").attr("id","actor-man-torso"+Vm).attr("x1",f).attr("y1",i.y+25).attr("x2",f).attr("y2",i.y+45),d.append("line").attr("id","actor-man-arms"+Vm).attr("x1",f-18).attr("y1",i.y+33).attr("x2",f+18).attr("y2",i.y+33),d.append("line").attr("x1",f-18).attr("y1",i.y+60).attr("x2",f).attr("y2",i.y+45),d.append("line").attr("x1",f).attr("y1",i.y+45).attr("x2",f+16).attr("y2",i.y+60);const y=d.append("circle");y.attr("cx",i.x+i.width/2),y.attr("cy",i.y+10),y.attr("r",15),y.attr("width",i.width),y.attr("height",i.height);const k=d.node().getBBox();return i.height=k.height,hce(o)(i.description,d,w.x,w.y+35,w.width,w.height,{class:"actor"},o),i.height},MZt=function(r,i,o,l){switch(i.type){case"actor":return AZt(r,i,o,l);case"participant":return SZt(r,i,o,l)}},DZt=function(r,i,o){const f=r.append("g");BLe(f,i),i.name&&hce(o)(i.name,f,i.x,i.y+(i.textMaxHeight||0)/2,i.width,0,{class:"text"},o),f.lower()},LZt=function(r){return r.append("g")},IZt=function(r,i,o,l,f){const b=YV(),d=i.anchored;b.x=i.startx,b.y=i.starty,b.class="activation"+f%3,b.width=i.stopx-i.startx,b.height=o-i.starty,qV(d,b)},OZt=function(r,i,o,l){const{boxMargin:f,boxTextMargin:b,labelBoxHeight:d,labelBoxWidth:w,messageFontFamily:y,messageFontSize:k,messageFontWeight:E}=l,T=r.append("g"),C=function(O,B,N,F){return T.append("line").attr("x1",O).attr("y1",B).attr("x2",N).attr("y2",F).attr("class","loopLine")};C(i.startx,i.starty,i.stopx,i.starty),C(i.stopx,i.starty,i.stopx,i.stopy),C(i.startx,i.stopy,i.stopx,i.stopy),C(i.startx,i.starty,i.startx,i.stopy),i.sections!==void 0&&i.sections.forEach(function(O){C(i.startx,O.y,i.stopx,O.y).style("stroke-dasharray","3, 3")});let S=lce();S.text=o,S.x=i.startx,S.y=i.starty,S.fontFamily=y,S.fontSize=k,S.fontWeight=E,S.anchor="middle",S.valign="middle",S.tspan=!1,S.width=w||50,S.height=d||20,S.textMargin=b,S.class="labelText",NLe(T,S),S=lce(),S.text=i.title,S.x=i.startx+w/2+(i.stopx-i.startx)/2,S.y=i.starty+f+b,S.anchor="middle",S.valign="middle",S.textMargin=b,S.class="loopText",S.fontFamily=y,S.fontSize=k,S.fontWeight=E,S.wrap=!0;let L=_C(T,S);return i.sectionTitles!==void 0&&i.sectionTitles.forEach(function(O,B){if(O.message){S.text=O.message,S.x=i.startx+(i.stopx-i.startx)/2,S.y=i.sections[B].y+f+b,S.class="loopText",S.anchor="middle",S.valign="middle",S.tspan=!1,S.fontFamily=y,S.fontSize=k,S.fontWeight=E,S.wrap=i.wrap,L=_C(T,S);let N=Math.round(L.map(F=>(F._groups||F)[0][0].getBBox().height).reduce((F,R)=>F+R));i.sections[B].height+=N-(f+b)}}),i.height=Math.round(i.stopy-i.starty),T},BLe=function(r,i){qV(r,{x:i.startx,y:i.starty,width:i.stopx-i.startx,height:i.stopy-i.starty,fill:i.fill,stroke:i.stroke,class:"rect"}).lower()},NZt=function(r){r.append("defs").append("symbol").attr("id","database").attr("fill-rule","evenodd").attr("clip-rule","evenodd").append("path").attr("transform","scale(.5)").attr("d","M12.258.001l.256.004.255.005.253.008.251.01.249.012.247.015.246.016.242.019.241.02.239.023.236.024.233.027.231.028.229.031.225.032.223.034.22.036.217.038.214.04.211.041.208.043.205.045.201.046.198.048.194.05.191.051.187.053.183.054.18.056.175.057.172.059.168.06.163.061.16.063.155.064.15.066.074.033.073.033.071.034.07.034.069.035.068.035.067.035.066.035.064.036.064.036.062.036.06.036.06.037.058.037.058.037.055.038.055.038.053.038.052.038.051.039.05.039.048.039.047.039.045.04.044.04.043.04.041.04.04.041.039.041.037.041.036.041.034.041.033.042.032.042.03.042.029.042.027.042.026.043.024.043.023.043.021.043.02.043.018.044.017.043.015.044.013.044.012.044.011.045.009.044.007.045.006.045.004.045.002.045.001.045v17l-.001.045-.002.045-.004.045-.006.045-.007.045-.009.044-.011.045-.012.044-.013.044-.015.044-.017.043-.018.044-.02.043-.021.043-.023.043-.024.043-.026.043-.027.042-.029.042-.03.042-.032.042-.033.042-.034.041-.036.041-.037.041-.039.041-.04.041-.041.04-.043.04-.044.04-.045.04-.047.039-.048.039-.05.039-.051.039-.052.038-.053.038-.055.038-.055.038-.058.037-.058.037-.06.037-.06.036-.062.036-.064.036-.064.036-.066.035-.067.035-.068.035-.069.035-.07.034-.071.034-.073.033-.074.033-.15.066-.155.064-.16.063-.163.061-.168.06-.172.059-.175.057-.18.056-.183.054-.187.053-.191.051-.194.05-.198.048-.201.046-.205.045-.208.043-.211.041-.214.04-.217.038-.22.036-.223.034-.225.032-.229.031-.231.028-.233.027-.236.024-.239.023-.241.02-.242.019-.246.016-.247.015-.249.012-.251.01-.253.008-.255.005-.256.004-.258.001-.258-.001-.256-.004-.255-.005-.253-.008-.251-.01-.249-.012-.247-.015-.245-.016-.243-.019-.241-.02-.238-.023-.236-.024-.234-.027-.231-.028-.228-.031-.226-.032-.223-.034-.22-.036-.217-.038-.214-.04-.211-.041-.208-.043-.204-.045-.201-.046-.198-.048-.195-.05-.19-.051-.187-.053-.184-.054-.179-.056-.176-.057-.172-.059-.167-.06-.164-.061-.159-.063-.155-.064-.151-.066-.074-.033-.072-.033-.072-.034-.07-.034-.069-.035-.068-.035-.067-.035-.066-.035-.064-.036-.063-.036-.062-.036-.061-.036-.06-.037-.058-.037-.057-.037-.056-.038-.055-.038-.053-.038-.052-.038-.051-.039-.049-.039-.049-.039-.046-.039-.046-.04-.044-.04-.043-.04-.041-.04-.04-.041-.039-.041-.037-.041-.036-.041-.034-.041-.033-.042-.032-.042-.03-.042-.029-.042-.027-.042-.026-.043-.024-.043-.023-.043-.021-.043-.02-.043-.018-.044-.017-.043-.015-.044-.013-.044-.012-.044-.011-.045-.009-.044-.007-.045-.006-.045-.004-.045-.002-.045-.001-.045v-17l.001-.045.002-.045.004-.045.006-.045.007-.045.009-.044.011-.045.012-.044.013-.044.015-.044.017-.043.018-.044.02-.043.021-.043.023-.043.024-.043.026-.043.027-.042.029-.042.03-.042.032-.042.033-.042.034-.041.036-.041.037-.041.039-.041.04-.041.041-.04.043-.04.044-.04.046-.04.046-.039.049-.039.049-.039.051-.039.052-.038.053-.038.055-.038.056-.038.057-.037.058-.037.06-.037.061-.036.062-.036.063-.036.064-.036.066-.035.067-.035.068-.035.069-.035.07-.034.072-.034.072-.033.074-.033.151-.066.155-.064.159-.063.164-.061.167-.06.172-.059.176-.057.179-.056.184-.054.187-.053.19-.051.195-.05.198-.048.201-.046.204-.045.208-.043.211-.041.214-.04.217-.038.22-.036.223-.034.226-.032.228-.031.231-.028.234-.027.236-.024.238-.023.241-.02.243-.019.245-.016.247-.015.249-.012.251-.01.253-.008.255-.005.256-.004.258-.001.258.001zm-9.258 20.499v.01l.001.021.003.021.004.022.005.021.006.022.007.022.009.023.01.022.011.023.012.023.013.023.015.023.016.024.017.023.018.024.019.024.021.024.022.025.023.024.024.025.052.049.056.05.061.051.066.051.07.051.075.051.079.052.084.052.088.052.092.052.097.052.102.051.105.052.11.052.114.051.119.051.123.051.127.05.131.05.135.05.139.048.144.049.147.047.152.047.155.047.16.045.163.045.167.043.171.043.176.041.178.041.183.039.187.039.19.037.194.035.197.035.202.033.204.031.209.03.212.029.216.027.219.025.222.024.226.021.23.02.233.018.236.016.24.015.243.012.246.01.249.008.253.005.256.004.259.001.26-.001.257-.004.254-.005.25-.008.247-.011.244-.012.241-.014.237-.016.233-.018.231-.021.226-.021.224-.024.22-.026.216-.027.212-.028.21-.031.205-.031.202-.034.198-.034.194-.036.191-.037.187-.039.183-.04.179-.04.175-.042.172-.043.168-.044.163-.045.16-.046.155-.046.152-.047.148-.048.143-.049.139-.049.136-.05.131-.05.126-.05.123-.051.118-.052.114-.051.11-.052.106-.052.101-.052.096-.052.092-.052.088-.053.083-.051.079-.052.074-.052.07-.051.065-.051.06-.051.056-.05.051-.05.023-.024.023-.025.021-.024.02-.024.019-.024.018-.024.017-.024.015-.023.014-.024.013-.023.012-.023.01-.023.01-.022.008-.022.006-.022.006-.022.004-.022.004-.021.001-.021.001-.021v-4.127l-.077.055-.08.053-.083.054-.085.053-.087.052-.09.052-.093.051-.095.05-.097.05-.1.049-.102.049-.105.048-.106.047-.109.047-.111.046-.114.045-.115.045-.118.044-.12.043-.122.042-.124.042-.126.041-.128.04-.13.04-.132.038-.134.038-.135.037-.138.037-.139.035-.142.035-.143.034-.144.033-.147.032-.148.031-.15.03-.151.03-.153.029-.154.027-.156.027-.158.026-.159.025-.161.024-.162.023-.163.022-.165.021-.166.02-.167.019-.169.018-.169.017-.171.016-.173.015-.173.014-.175.013-.175.012-.177.011-.178.01-.179.008-.179.008-.181.006-.182.005-.182.004-.184.003-.184.002h-.37l-.184-.002-.184-.003-.182-.004-.182-.005-.181-.006-.179-.008-.179-.008-.178-.01-.176-.011-.176-.012-.175-.013-.173-.014-.172-.015-.171-.016-.17-.017-.169-.018-.167-.019-.166-.02-.165-.021-.163-.022-.162-.023-.161-.024-.159-.025-.157-.026-.156-.027-.155-.027-.153-.029-.151-.03-.15-.03-.148-.031-.146-.032-.145-.033-.143-.034-.141-.035-.14-.035-.137-.037-.136-.037-.134-.038-.132-.038-.13-.04-.128-.04-.126-.041-.124-.042-.122-.042-.12-.044-.117-.043-.116-.045-.113-.045-.112-.046-.109-.047-.106-.047-.105-.048-.102-.049-.1-.049-.097-.05-.095-.05-.093-.052-.09-.051-.087-.052-.085-.053-.083-.054-.08-.054-.077-.054v4.127zm0-5.654v.011l.001.021.003.021.004.021.005.022.006.022.007.022.009.022.01.022.011.023.012.023.013.023.015.024.016.023.017.024.018.024.019.024.021.024.022.024.023.025.024.024.052.05.056.05.061.05.066.051.07.051.075.052.079.051.084.052.088.052.092.052.097.052.102.052.105.052.11.051.114.051.119.052.123.05.127.051.131.05.135.049.139.049.144.048.147.048.152.047.155.046.16.045.163.045.167.044.171.042.176.042.178.04.183.04.187.038.19.037.194.036.197.034.202.033.204.032.209.03.212.028.216.027.219.025.222.024.226.022.23.02.233.018.236.016.24.014.243.012.246.01.249.008.253.006.256.003.259.001.26-.001.257-.003.254-.006.25-.008.247-.01.244-.012.241-.015.237-.016.233-.018.231-.02.226-.022.224-.024.22-.025.216-.027.212-.029.21-.03.205-.032.202-.033.198-.035.194-.036.191-.037.187-.039.183-.039.179-.041.175-.042.172-.043.168-.044.163-.045.16-.045.155-.047.152-.047.148-.048.143-.048.139-.05.136-.049.131-.05.126-.051.123-.051.118-.051.114-.052.11-.052.106-.052.101-.052.096-.052.092-.052.088-.052.083-.052.079-.052.074-.051.07-.052.065-.051.06-.05.056-.051.051-.049.023-.025.023-.024.021-.025.02-.024.019-.024.018-.024.017-.024.015-.023.014-.023.013-.024.012-.022.01-.023.01-.023.008-.022.006-.022.006-.022.004-.021.004-.022.001-.021.001-.021v-4.139l-.077.054-.08.054-.083.054-.085.052-.087.053-.09.051-.093.051-.095.051-.097.05-.1.049-.102.049-.105.048-.106.047-.109.047-.111.046-.114.045-.115.044-.118.044-.12.044-.122.042-.124.042-.126.041-.128.04-.13.039-.132.039-.134.038-.135.037-.138.036-.139.036-.142.035-.143.033-.144.033-.147.033-.148.031-.15.03-.151.03-.153.028-.154.028-.156.027-.158.026-.159.025-.161.024-.162.023-.163.022-.165.021-.166.02-.167.019-.169.018-.169.017-.171.016-.173.015-.173.014-.175.013-.175.012-.177.011-.178.009-.179.009-.179.007-.181.007-.182.005-.182.004-.184.003-.184.002h-.37l-.184-.002-.184-.003-.182-.004-.182-.005-.181-.007-.179-.007-.179-.009-.178-.009-.176-.011-.176-.012-.175-.013-.173-.014-.172-.015-.171-.016-.17-.017-.169-.018-.167-.019-.166-.02-.165-.021-.163-.022-.162-.023-.161-.024-.159-.025-.157-.026-.156-.027-.155-.028-.153-.028-.151-.03-.15-.03-.148-.031-.146-.033-.145-.033-.143-.033-.141-.035-.14-.036-.137-.036-.136-.037-.134-.038-.132-.039-.13-.039-.128-.04-.126-.041-.124-.042-.122-.043-.12-.043-.117-.044-.116-.044-.113-.046-.112-.046-.109-.046-.106-.047-.105-.048-.102-.049-.1-.049-.097-.05-.095-.051-.093-.051-.09-.051-.087-.053-.085-.052-.083-.054-.08-.054-.077-.054v4.139zm0-5.666v.011l.001.02.003.022.004.021.005.022.006.021.007.022.009.023.01.022.011.023.012.023.013.023.015.023.016.024.017.024.018.023.019.024.021.025.022.024.023.024.024.025.052.05.056.05.061.05.066.051.07.051.075.052.079.051.084.052.088.052.092.052.097.052.102.052.105.051.11.052.114.051.119.051.123.051.127.05.131.05.135.05.139.049.144.048.147.048.152.047.155.046.16.045.163.045.167.043.171.043.176.042.178.04.183.04.187.038.19.037.194.036.197.034.202.033.204.032.209.03.212.028.216.027.219.025.222.024.226.021.23.02.233.018.236.017.24.014.243.012.246.01.249.008.253.006.256.003.259.001.26-.001.257-.003.254-.006.25-.008.247-.01.244-.013.241-.014.237-.016.233-.018.231-.02.226-.022.224-.024.22-.025.216-.027.212-.029.21-.03.205-.032.202-.033.198-.035.194-.036.191-.037.187-.039.183-.039.179-.041.175-.042.172-.043.168-.044.163-.045.16-.045.155-.047.152-.047.148-.048.143-.049.139-.049.136-.049.131-.051.126-.05.123-.051.118-.052.114-.051.11-.052.106-.052.101-.052.096-.052.092-.052.088-.052.083-.052.079-.052.074-.052.07-.051.065-.051.06-.051.056-.05.051-.049.023-.025.023-.025.021-.024.02-.024.019-.024.018-.024.017-.024.015-.023.014-.024.013-.023.012-.023.01-.022.01-.023.008-.022.006-.022.006-.022.004-.022.004-.021.001-.021.001-.021v-4.153l-.077.054-.08.054-.083.053-.085.053-.087.053-.09.051-.093.051-.095.051-.097.05-.1.049-.102.048-.105.048-.106.048-.109.046-.111.046-.114.046-.115.044-.118.044-.12.043-.122.043-.124.042-.126.041-.128.04-.13.039-.132.039-.134.038-.135.037-.138.036-.139.036-.142.034-.143.034-.144.033-.147.032-.148.032-.15.03-.151.03-.153.028-.154.028-.156.027-.158.026-.159.024-.161.024-.162.023-.163.023-.165.021-.166.02-.167.019-.169.018-.169.017-.171.016-.173.015-.173.014-.175.013-.175.012-.177.01-.178.01-.179.009-.179.007-.181.006-.182.006-.182.004-.184.003-.184.001-.185.001-.185-.001-.184-.001-.184-.003-.182-.004-.182-.006-.181-.006-.179-.007-.179-.009-.178-.01-.176-.01-.176-.012-.175-.013-.173-.014-.172-.015-.171-.016-.17-.017-.169-.018-.167-.019-.166-.02-.165-.021-.163-.023-.162-.023-.161-.024-.159-.024-.157-.026-.156-.027-.155-.028-.153-.028-.151-.03-.15-.03-.148-.032-.146-.032-.145-.033-.143-.034-.141-.034-.14-.036-.137-.036-.136-.037-.134-.038-.132-.039-.13-.039-.128-.041-.126-.041-.124-.041-.122-.043-.12-.043-.117-.044-.116-.044-.113-.046-.112-.046-.109-.046-.106-.048-.105-.048-.102-.048-.1-.05-.097-.049-.095-.051-.093-.051-.09-.052-.087-.052-.085-.053-.083-.053-.08-.054-.077-.054v4.153zm8.74-8.179l-.257.004-.254.005-.25.008-.247.011-.244.012-.241.014-.237.016-.233.018-.231.021-.226.022-.224.023-.22.026-.216.027-.212.028-.21.031-.205.032-.202.033-.198.034-.194.036-.191.038-.187.038-.183.04-.179.041-.175.042-.172.043-.168.043-.163.045-.16.046-.155.046-.152.048-.148.048-.143.048-.139.049-.136.05-.131.05-.126.051-.123.051-.118.051-.114.052-.11.052-.106.052-.101.052-.096.052-.092.052-.088.052-.083.052-.079.052-.074.051-.07.052-.065.051-.06.05-.056.05-.051.05-.023.025-.023.024-.021.024-.02.025-.019.024-.018.024-.017.023-.015.024-.014.023-.013.023-.012.023-.01.023-.01.022-.008.022-.006.023-.006.021-.004.022-.004.021-.001.021-.001.021.001.021.001.021.004.021.004.022.006.021.006.023.008.022.01.022.01.023.012.023.013.023.014.023.015.024.017.023.018.024.019.024.02.025.021.024.023.024.023.025.051.05.056.05.06.05.065.051.07.052.074.051.079.052.083.052.088.052.092.052.096.052.101.052.106.052.11.052.114.052.118.051.123.051.126.051.131.05.136.05.139.049.143.048.148.048.152.048.155.046.16.046.163.045.168.043.172.043.175.042.179.041.183.04.187.038.191.038.194.036.198.034.202.033.205.032.21.031.212.028.216.027.22.026.224.023.226.022.231.021.233.018.237.016.241.014.244.012.247.011.25.008.254.005.257.004.26.001.26-.001.257-.004.254-.005.25-.008.247-.011.244-.012.241-.014.237-.016.233-.018.231-.021.226-.022.224-.023.22-.026.216-.027.212-.028.21-.031.205-.032.202-.033.198-.034.194-.036.191-.038.187-.038.183-.04.179-.041.175-.042.172-.043.168-.043.163-.045.16-.046.155-.046.152-.048.148-.048.143-.048.139-.049.136-.05.131-.05.126-.051.123-.051.118-.051.114-.052.11-.052.106-.052.101-.052.096-.052.092-.052.088-.052.083-.052.079-.052.074-.051.07-.052.065-.051.06-.05.056-.05.051-.05.023-.025.023-.024.021-.024.02-.025.019-.024.018-.024.017-.023.015-.024.014-.023.013-.023.012-.023.01-.023.01-.022.008-.022.006-.023.006-.021.004-.022.004-.021.001-.021.001-.021-.001-.021-.001-.021-.004-.021-.004-.022-.006-.021-.006-.023-.008-.022-.01-.022-.01-.023-.012-.023-.013-.023-.014-.023-.015-.024-.017-.023-.018-.024-.019-.024-.02-.025-.021-.024-.023-.024-.023-.025-.051-.05-.056-.05-.06-.05-.065-.051-.07-.052-.074-.051-.079-.052-.083-.052-.088-.052-.092-.052-.096-.052-.101-.052-.106-.052-.11-.052-.114-.052-.118-.051-.123-.051-.126-.051-.131-.05-.136-.05-.139-.049-.143-.048-.148-.048-.152-.048-.155-.046-.16-.046-.163-.045-.168-.043-.172-.043-.175-.042-.179-.041-.183-.04-.187-.038-.191-.038-.194-.036-.198-.034-.202-.033-.205-.032-.21-.031-.212-.028-.216-.027-.22-.026-.224-.023-.226-.022-.231-.021-.233-.018-.237-.016-.241-.014-.244-.012-.247-.011-.25-.008-.254-.005-.257-.004-.26-.001-.26.001z")},PZt=function(r){r.append("defs").append("symbol").attr("id","computer").attr("width","24").attr("height","24").append("path").attr("transform","scale(.5)").attr("d","M2 2v13h20v-13h-20zm18 11h-16v-9h16v9zm-10.228 6l.466-1h3.524l.467 1h-4.457zm14.228 3h-24l2-6h2.104l-1.33 4h18.45l-1.297-4h2.073l2 6zm-5-10h-14v-7h14v7z")},BZt=function(r){r.append("defs").append("symbol").attr("id","clock").attr("width","24").attr("height","24").append("path").attr("transform","scale(.5)").attr("d","M12 2c5.514 0 10 4.486 10 10s-4.486 10-10 10-10-4.486-10-10 4.486-10 10-10zm0-2c-6.627 0-12 5.373-12 12s5.373 12 12 12 12-5.373 12-12-5.373-12-12-12zm5.848 12.459c.202.038.202.333.001.372-1.907.361-6.045 1.111-6.547 1.111-.719 0-1.301-.582-1.301-1.301 0-.512.77-5.447 1.125-7.445.034-.192.312-.181.343.014l.985 6.238 5.394 1.011z")},FZt=function(r){r.append("defs").append("marker").attr("id","arrowhead").attr("refX",9).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",12).attr("markerHeight",12).attr("orient","auto").append("path").attr("d","M 0 0 L 10 5 L 0 10 z")},RZt=function(r){r.append("defs").append("marker").attr("id","filled-head").attr("refX",18).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L14,7 L9,1 Z")},jZt=function(r){r.append("defs").append("marker").attr("id","sequencenumber").attr("refX",15).attr("refY",15).attr("markerWidth",60).attr("markerHeight",40).attr("orient","auto").append("circle").attr("cx",15).attr("cy",15).attr("r",6)},$Zt=function(r){r.append("defs").append("marker").attr("id","crosshead").attr("markerWidth",15).attr("markerHeight",8).attr("orient","auto").attr("refX",4).attr("refY",5).append("path").attr("fill","none").attr("stroke","#000000").style("stroke-dasharray","0, 0").attr("stroke-width","1pt").attr("d","M 1,2 L 6,7 M 6,2 L 1,7")},lce=function(){return{x:0,y:0,fill:void 0,anchor:void 0,style:"#666",width:void 0,height:void 0,textMargin:0,rx:0,ry:0,tspan:!0,valign:void 0}},YV=function(){return{x:0,y:0,fill:"#EDF2AE",stroke:"#666",width:100,anchor:"start",height:100,rx:0,ry:0}},hce=function(){function r(f,b,d,w,y,k,E){const T=b.append("text").attr("x",d+y/2).attr("y",w+k/2+5).style("text-anchor","middle").text(f);l(T,E)}function i(f,b,d,w,y,k,E,T){const{actorFontSize:C,actorFontFamily:S,actorFontWeight:L}=T,[O,B]=oI(C),N=f.split(Cs.lineBreakRegex);for(let F=0;F<N.length;F++){const R=F*O-O*(N.length-1)/2,q=b.append("text").attr("x",d+y/2).attr("y",w).style("text-anchor","middle").style("font-size",B).style("font-weight",L).style("font-family",S);q.append("tspan").attr("x",d+y/2).attr("dy",R).text(N[F]),q.attr("y",w+k/2).attr("dominant-baseline","central").attr("alignment-baseline","central"),l(q,E)}}function o(f,b,d,w,y,k,E,T){const C=b.append("switch"),L=C.append("foreignObject").attr("x",d).attr("y",w).attr("width",y).attr("height",k).append("xhtml:div").style("display","table").style("height","100%").style("width","100%");L.append("div").style("display","table-cell").style("text-align","center").style("vertical-align","middle").text(f),i(f,C,d,w,y,k,E,T),l(L,E)}function l(f,b){for(const d in b)b.hasOwnProperty(d)&&f.attr(d,b[d])}return function(f){return f.textPlacement==="fo"?o:f.textPlacement==="old"?r:i}}(),HZt=function(){function r(f,b,d,w,y,k,E){const T=b.append("text").attr("x",d).attr("y",w).style("text-anchor","start").text(f);l(T,E)}function i(f,b,d,w,y,k,E,T){const{actorFontSize:C,actorFontFamily:S,actorFontWeight:L}=T,O=f.split(Cs.lineBreakRegex);for(let B=0;B<O.length;B++){const N=B*C-C*(O.length-1)/2,F=b.append("text").attr("x",d).attr("y",w).style("text-anchor","start").style("font-size",C).style("font-weight",L).style("font-family",S);F.append("tspan").attr("x",d).attr("dy",N).text(O[B]),F.attr("y",w+k/2).attr("dominant-baseline","central").attr("alignment-baseline","central"),l(F,E)}}function o(f,b,d,w,y,k,E,T){const C=b.append("switch"),L=C.append("foreignObject").attr("x",d).attr("y",w).attr("width",y).attr("height",k).append("xhtml:div").style("display","table").style("height","100%").style("width","100%");L.append("div").style("display","table-cell").style("text-align","center").style("vertical-align","middle").text(f),i(f,C,d,w,y,k,E,T),l(L,E)}function l(f,b){for(const d in b)b.hasOwnProperty(d)&&f.attr(d,b[d])}return function(f){return f.textPlacement==="fo"?o:f.textPlacement==="old"?r:i}}(),sl={drawRect:qV,drawText:_C,drawLabel:NLe,drawActor:MZt,drawBox:DZt,drawPopup:xZt,drawImage:ILe,drawEmbeddedImage:OLe,anchorElement:LZt,drawActivation:IZt,drawLoop:OZt,drawBackgroundRect:BLe,insertArrowHead:FZt,insertArrowFilledHead:RZt,insertSequenceNumber:jZt,insertArrowCrossHead:$Zt,insertDatabaseIcon:NZt,insertComputerIcon:PZt,insertClockIcon:BZt,getTextObj:lce,getNoteRect:YV,popupMenu:EZt,popdownMenu:_Zt,fixLifeLineHeights:PLe,sanitizeUrl:$3};let Ft={};const br={data:{startx:void 0,stopx:void 0,starty:void 0,stopy:void 0},verticalPos:0,sequenceItems:[],activations:[],models:{getHeight:function(){return Math.max.apply(null,this.actors.length===0?[0]:this.actors.map(r=>r.height||0))+(this.loops.length===0?0:this.loops.map(r=>r.height||0).reduce((r,i)=>r+i))+(this.messages.length===0?0:this.messages.map(r=>r.height||0).reduce((r,i)=>r+i))+(this.notes.length===0?0:this.notes.map(r=>r.height||0).reduce((r,i)=>r+i))},clear:function(){this.actors=[],this.boxes=[],this.loops=[],this.messages=[],this.notes=[]},addBox:function(r){this.boxes.push(r)},addActor:function(r){this.actors.push(r)},addLoop:function(r){this.loops.push(r)},addMessage:function(r){this.messages.push(r)},addNote:function(r){this.notes.push(r)},lastActor:function(){return this.actors[this.actors.length-1]},lastLoop:function(){return this.loops[this.loops.length-1]},lastMessage:function(){return this.messages[this.messages.length-1]},lastNote:function(){return this.notes[this.notes.length-1]},actors:[],boxes:[],loops:[],messages:[],notes:[]},init:function(){this.sequenceItems=[],this.activations=[],this.models.clear(),this.data={startx:void 0,stopx:void 0,starty:void 0,stopy:void 0},this.verticalPos=0,RLe(ht())},updateVal:function(r,i,o,l){r[i]===void 0?r[i]=o:r[i]=l(o,r[i])},updateBounds:function(r,i,o,l){const f=this;let b=0;function d(w){return function(k){b++;const E=f.sequenceItems.length-b+1;f.updateVal(k,"starty",i-E*Ft.boxMargin,Math.min),f.updateVal(k,"stopy",l+E*Ft.boxMargin,Math.max),f.updateVal(br.data,"startx",r-E*Ft.boxMargin,Math.min),f.updateVal(br.data,"stopx",o+E*Ft.boxMargin,Math.max),w!=="activation"&&(f.updateVal(k,"startx",r-E*Ft.boxMargin,Math.min),f.updateVal(k,"stopx",o+E*Ft.boxMargin,Math.max),f.updateVal(br.data,"starty",i-E*Ft.boxMargin,Math.min),f.updateVal(br.data,"stopy",l+E*Ft.boxMargin,Math.max))}}this.sequenceItems.forEach(d()),this.activations.forEach(d("activation"))},insert:function(r,i,o,l){const f=Math.min(r,o),b=Math.max(r,o),d=Math.min(i,l),w=Math.max(i,l);this.updateVal(br.data,"startx",f,Math.min),this.updateVal(br.data,"starty",d,Math.min),this.updateVal(br.data,"stopx",b,Math.max),this.updateVal(br.data,"stopy",w,Math.max),this.updateBounds(f,d,b,w)},newActivation:function(r,i,o){const l=o[r.from.actor],f=WV(r.from.actor).length||0,b=l.x+l.width/2+(f-1)*Ft.activationWidth/2;this.activations.push({startx:b,starty:this.verticalPos+2,stopx:b+Ft.activationWidth,stopy:void 0,actor:r.from.actor,anchored:sl.anchorElement(i)})},endActivation:function(r){const i=this.activations.map(function(o){return o.actor}).lastIndexOf(r.from.actor);return this.activations.splice(i,1)[0]},createLoop:function(r={message:void 0,wrap:!1,width:void 0},i){return{startx:void 0,starty:this.verticalPos,stopx:void 0,stopy:void 0,title:r.message,wrap:r.wrap,width:r.width,height:0,fill:i}},newLoop:function(r={message:void 0,wrap:!1,width:void 0},i){this.sequenceItems.push(this.createLoop(r,i))},endLoop:function(){return this.sequenceItems.pop()},addSectionToLoop:function(r){const i=this.sequenceItems.pop();i.sections=i.sections||[],i.sectionTitles=i.sectionTitles||[],i.sections.push({y:br.getVerticalPos(),height:0}),i.sectionTitles.push(r),this.sequenceItems.push(i)},bumpVerticalPos:function(r){this.verticalPos=this.verticalPos+r,this.data.stopy=this.verticalPos},getVerticalPos:function(){return this.verticalPos},getBounds:function(){return{bounds:this.data,models:this.models}}},zZt=function(r,i){br.bumpVerticalPos(Ft.boxMargin),i.height=Ft.boxMargin,i.starty=br.getVerticalPos();const o=sl.getNoteRect();o.x=i.startx,o.y=i.starty,o.width=i.width||Ft.width,o.class="note";const l=r.append("g"),f=sl.drawRect(l,o),b=sl.getTextObj();b.x=i.startx,b.y=i.starty,b.width=o.width,b.dy="1em",b.text=i.message,b.class="noteText",b.fontFamily=Ft.noteFontFamily,b.fontSize=Ft.noteFontSize,b.fontWeight=Ft.noteFontWeight,b.anchor=Ft.noteAlign,b.textMargin=Ft.noteMargin,b.valign="center";const d=_C(l,b),w=Math.round(d.map(y=>(y._groups||y)[0][0].getBBox().height).reduce((y,k)=>y+k));f.attr("height",w+2*Ft.noteMargin),i.height+=w+2*Ft.noteMargin,br.bumpVerticalPos(w+2*Ft.noteMargin),i.stopy=i.starty+w+2*Ft.noteMargin,i.stopx=i.startx+o.width,br.insert(i.startx,i.starty,i.stopx,i.stopy),br.models.addNote(i)},d7=r=>({fontFamily:r.messageFontFamily,fontSize:r.messageFontSize,fontWeight:r.messageFontWeight}),TC=r=>({fontFamily:r.noteFontFamily,fontSize:r.noteFontSize,fontWeight:r.noteFontWeight}),fce=r=>({fontFamily:r.actorFontFamily,fontSize:r.actorFontSize,fontWeight:r.actorFontWeight});function GZt(r,i){br.bumpVerticalPos(10);const{startx:o,stopx:l,message:f}=i,b=Cs.splitBreaks(f).length,d=Fs.calculateTextDimensions(f,d7(Ft)),w=d.height/b;i.height+=w,br.bumpVerticalPos(w);let y,k=d.height-10;const E=d.width;if(o===l){y=br.getVerticalPos()+k,Ft.rightAngles||(k+=Ft.boxMargin,y=br.getVerticalPos()+k),k+=30;const T=Math.max(E/2,Ft.width/2);br.insert(o-T,br.getVerticalPos()-10+k,l+T,br.getVerticalPos()+30+k)}else k+=Ft.boxMargin,y=br.getVerticalPos()+k,br.insert(o,y-10,l,y);return br.bumpVerticalPos(k),i.height+=k,i.stopy=i.starty+i.height,br.insert(i.fromBounds,i.starty,i.toBounds,i.stopy),y}const VZt=function(r,i,o,l){const{startx:f,stopx:b,starty:d,message:w,type:y,sequenceIndex:k,sequenceVisible:E}=i,T=Fs.calculateTextDimensions(w,d7(Ft)),C=sl.getTextObj();C.x=f,C.y=d+10,C.width=b-f,C.class="messageText",C.dy="1em",C.text=w,C.fontFamily=Ft.messageFontFamily,C.fontSize=Ft.messageFontSize,C.fontWeight=Ft.messageFontWeight,C.anchor=Ft.messageAlign,C.valign="center",C.textMargin=Ft.wrapPadding,C.tspan=!1,_C(r,C);const S=T.width;let L;f===b?Ft.rightAngles?L=r.append("path").attr("d",`M  ${f},${o} H ${f+Math.max(Ft.width/2,S/2)} V ${o+25} H ${f}`):L=r.append("path").attr("d","M "+f+","+o+" C "+(f+60)+","+(o-10)+" "+(f+60)+","+(o+30)+" "+f+","+(o+20)):(L=r.append("line"),L.attr("x1",f),L.attr("y1",o),L.attr("x2",b),L.attr("y2",o)),y===l.db.LINETYPE.DOTTED||y===l.db.LINETYPE.DOTTED_CROSS||y===l.db.LINETYPE.DOTTED_POINT||y===l.db.LINETYPE.DOTTED_OPEN?(L.style("stroke-dasharray","3, 3"),L.attr("class","messageLine1")):L.attr("class","messageLine0");let O="";Ft.arrowMarkerAbsolute&&(O=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,O=O.replace(/\(/g,"\\("),O=O.replace(/\)/g,"\\)")),L.attr("stroke-width",2),L.attr("stroke","none"),L.style("fill","none"),(y===l.db.LINETYPE.SOLID||y===l.db.LINETYPE.DOTTED)&&L.attr("marker-end","url("+O+"#arrowhead)"),(y===l.db.LINETYPE.SOLID_POINT||y===l.db.LINETYPE.DOTTED_POINT)&&L.attr("marker-end","url("+O+"#filled-head)"),(y===l.db.LINETYPE.SOLID_CROSS||y===l.db.LINETYPE.DOTTED_CROSS)&&L.attr("marker-end","url("+O+"#crosshead)"),(E||Ft.showSequenceNumbers)&&(L.attr("marker-start","url("+O+"#sequencenumber)"),r.append("text").attr("x",f).attr("y",o+4).attr("font-family","sans-serif").attr("font-size","12px").attr("text-anchor","middle").attr("class","sequenceNumber").text(k))},dce=function(r,i,o,l,f,b,d){if(f.hideUnusedParticipants===!0){const T=new Set;b.forEach(C=>{T.add(C.from),T.add(C.to)}),o=o.filter(C=>T.has(C))}let w=0,y=0,k=0,E;for(const T of o){const C=i[T],S=C.box;E&&E!=S&&(d||br.models.addBox(E),y+=Ft.boxMargin+E.margin),S&&S!=E&&(d||(S.x=w+y,S.y=l),y+=S.margin),C.width=C.width||Ft.width,C.height=Math.max(C.height||Ft.height,Ft.height),C.margin=C.margin||Ft.actorMargin,C.x=w+y,C.y=br.getVerticalPos();const L=sl.drawActor(r,C,Ft,d);k=Math.max(k,L),br.insert(C.x,l,C.x+C.width,C.height),w+=C.width+y,C.box&&(C.box.width=w+S.margin-C.box.x),y=C.margin,E=C.box,br.models.addActor(C)}E&&!d&&br.models.addBox(E),br.bumpVerticalPos(k)},FLe=function(r,i,o,l){let f=0,b=0;for(const d of o){const w=i[d],y=YZt(w),k=sl.drawPopup(r,w,y,Ft,Ft.forceMenus,l);k.height>f&&(f=k.height),k.width+w.x>b&&(b=k.width+w.x)}return{maxHeight:f,maxWidth:b}},RLe=function(r){Yf(Ft,r),r.fontFamily&&(Ft.actorFontFamily=Ft.noteFontFamily=Ft.messageFontFamily=r.fontFamily),r.fontSize&&(Ft.actorFontSize=Ft.noteFontSize=Ft.messageFontSize=r.fontSize),r.fontWeight&&(Ft.actorFontWeight=Ft.noteFontWeight=Ft.messageFontWeight=r.fontWeight)},WV=function(r){return br.activations.filter(function(i){return i.actor===r})},jLe=function(r,i){const o=i[r],l=WV(r),f=l.reduce(function(d,w){return Math.min(d,w.startx)},o.x+o.width/2),b=l.reduce(function(d,w){return Math.max(d,w.stopx)},o.x+o.width/2);return[f,b]};function Um(r,i,o,l,f){br.bumpVerticalPos(o);let b=l;if(i.id&&i.message&&r[i.id]){const d=r[i.id].width,w=d7(Ft);i.message=Fs.wrapLabel(`[${i.message}]`,d-2*Ft.wrapPadding,w),i.width=d,i.wrap=!0;const y=Fs.calculateTextDimensions(i.message,w),k=Math.max(y.height,Ft.labelBoxHeight);b=l+k,Se.debug(`${k} - ${i.message}`)}f(i),br.bumpVerticalPos(b)}const UZt=function(r,i,o,l){const{securityLevel:f,sequence:b}=ht();Ft=b,l.db.clear(),l.parser.parse(r);let d;f==="sandbox"&&(d=Cn("#i"+i));const w=Cn(f==="sandbox"?d.nodes()[0].contentDocument.body:"body"),y=f==="sandbox"?d.nodes()[0].contentDocument:document;br.init(),Se.debug(l.db);const k=f==="sandbox"?w.select(`[id="${i}"]`):Cn(`[id="${i}"]`),E=l.db.getActors(),T=l.db.getBoxes(),C=l.db.getActorKeys(),S=l.db.getMessages(),L=l.db.getDiagramTitle(),O=l.db.hasAtLeastOneBox(),B=l.db.hasAtLeastOneBoxWithTitle(),N=qZt(E,S,l);Ft.height=WZt(E,N,T),sl.insertComputerIcon(k),sl.insertDatabaseIcon(k),sl.insertClockIcon(k),O&&(br.bumpVerticalPos(Ft.boxMargin),B&&br.bumpVerticalPos(T[0].textMaxHeight)),dce(k,E,C,0,Ft,S,!1);const F=QZt(S,E,N,l);sl.insertArrowHead(k),sl.insertArrowCrossHead(k),sl.insertArrowFilledHead(k),sl.insertSequenceNumber(k);function R(ge,ee){const de=br.endActivation(ge);de.starty+18>ee&&(de.starty=ee-6,ee+=12),sl.drawActivation(k,de,ee,Ft,WV(ge.from.actor).length),br.insert(de.startx,ee-10,de.stopx,ee)}let q=1,X=1;const te=[];S.forEach(function(ge){let ee,de,re;switch(ge.type){case l.db.LINETYPE.NOTE:de=ge.noteModel,zZt(k,de);break;case l.db.LINETYPE.ACTIVE_START:br.newActivation(ge,k,E);break;case l.db.LINETYPE.ACTIVE_END:R(ge,br.getVerticalPos());break;case l.db.LINETYPE.LOOP_START:Um(F,ge,Ft.boxMargin,Ft.boxMargin+Ft.boxTextMargin,ke=>br.newLoop(ke));break;case l.db.LINETYPE.LOOP_END:ee=br.endLoop(),sl.drawLoop(k,ee,"loop",Ft),br.bumpVerticalPos(ee.stopy-br.getVerticalPos()),br.models.addLoop(ee);break;case l.db.LINETYPE.RECT_START:Um(F,ge,Ft.boxMargin,Ft.boxMargin,ke=>br.newLoop(void 0,ke.message));break;case l.db.LINETYPE.RECT_END:ee=br.endLoop(),sl.drawBackgroundRect(k,ee),br.models.addLoop(ee),br.bumpVerticalPos(ee.stopy-br.getVerticalPos());break;case l.db.LINETYPE.OPT_START:Um(F,ge,Ft.boxMargin,Ft.boxMargin+Ft.boxTextMargin,ke=>br.newLoop(ke));break;case l.db.LINETYPE.OPT_END:ee=br.endLoop(),sl.drawLoop(k,ee,"opt",Ft),br.bumpVerticalPos(ee.stopy-br.getVerticalPos()),br.models.addLoop(ee);break;case l.db.LINETYPE.ALT_START:Um(F,ge,Ft.boxMargin,Ft.boxMargin+Ft.boxTextMargin,ke=>br.newLoop(ke));break;case l.db.LINETYPE.ALT_ELSE:Um(F,ge,Ft.boxMargin+Ft.boxTextMargin,Ft.boxMargin,ke=>br.addSectionToLoop(ke));break;case l.db.LINETYPE.ALT_END:ee=br.endLoop(),sl.drawLoop(k,ee,"alt",Ft),br.bumpVerticalPos(ee.stopy-br.getVerticalPos()),br.models.addLoop(ee);break;case l.db.LINETYPE.PAR_START:Um(F,ge,Ft.boxMargin,Ft.boxMargin+Ft.boxTextMargin,ke=>br.newLoop(ke));break;case l.db.LINETYPE.PAR_AND:Um(F,ge,Ft.boxMargin+Ft.boxTextMargin,Ft.boxMargin,ke=>br.addSectionToLoop(ke));break;case l.db.LINETYPE.PAR_END:ee=br.endLoop(),sl.drawLoop(k,ee,"par",Ft),br.bumpVerticalPos(ee.stopy-br.getVerticalPos()),br.models.addLoop(ee);break;case l.db.LINETYPE.AUTONUMBER:q=ge.message.start||q,X=ge.message.step||X,ge.message.visible?l.db.enableSequenceNumbers():l.db.disableSequenceNumbers();break;case l.db.LINETYPE.CRITICAL_START:Um(F,ge,Ft.boxMargin,Ft.boxMargin+Ft.boxTextMargin,ke=>br.newLoop(ke));break;case l.db.LINETYPE.CRITICAL_OPTION:Um(F,ge,Ft.boxMargin+Ft.boxTextMargin,Ft.boxMargin,ke=>br.addSectionToLoop(ke));break;case l.db.LINETYPE.CRITICAL_END:ee=br.endLoop(),sl.drawLoop(k,ee,"critical",Ft),br.bumpVerticalPos(ee.stopy-br.getVerticalPos()),br.models.addLoop(ee);break;case l.db.LINETYPE.BREAK_START:Um(F,ge,Ft.boxMargin,Ft.boxMargin+Ft.boxTextMargin,ke=>br.newLoop(ke));break;case l.db.LINETYPE.BREAK_END:ee=br.endLoop(),sl.drawLoop(k,ee,"break",Ft),br.bumpVerticalPos(ee.stopy-br.getVerticalPos()),br.models.addLoop(ee);break;default:try{re=ge.msgModel,re.starty=br.getVerticalPos(),re.sequenceIndex=q,re.sequenceVisible=l.db.showSequenceNumbers();const ke=GZt(k,re);te.push({messageModel:re,lineStartY:ke}),br.models.addMessage(re)}catch(ke){Se.error("error while drawing message",ke)}}[l.db.LINETYPE.SOLID_OPEN,l.db.LINETYPE.DOTTED_OPEN,l.db.LINETYPE.SOLID,l.db.LINETYPE.DOTTED,l.db.LINETYPE.SOLID_CROSS,l.db.LINETYPE.DOTTED_CROSS,l.db.LINETYPE.SOLID_POINT,l.db.LINETYPE.DOTTED_POINT].includes(ge.type)&&(q=q+X)}),te.forEach(ge=>VZt(k,ge.messageModel,ge.lineStartY,l)),Ft.mirrorActors&&(br.bumpVerticalPos(Ft.boxMargin*2),dce(k,E,C,br.getVerticalPos(),Ft,S,!0),br.bumpVerticalPos(Ft.boxMargin),PLe(k,br.getVerticalPos())),br.models.boxes.forEach(function(ge){ge.height=br.getVerticalPos()-ge.y,br.insert(ge.x,ge.y,ge.x+ge.width,ge.height),ge.startx=ge.x,ge.starty=ge.y,ge.stopx=ge.startx+ge.width,ge.stopy=ge.starty+ge.height,ge.stroke="rgb(0,0,0, 0.5)",sl.drawBox(k,ge,Ft)}),O&&br.bumpVerticalPos(Ft.boxMargin);const H=FLe(k,E,C,y),{bounds:Y}=br.getBounds();Se.debug("For line height fix Querying: #"+i+" .actor-line"),ose("#"+i+" .actor-line").attr("y2",Y.stopy);let W=Y.stopy-Y.starty;W<H.maxHeight&&(W=H.maxHeight);let Z=W+2*Ft.diagramMarginY;Ft.mirrorActors&&(Z=Z-Ft.boxMargin+Ft.bottomMarginAdj);let G=Y.stopx-Y.startx;G<H.maxWidth&&(G=H.maxWidth);const ae=G+2*Ft.diagramMarginX;L&&k.append("text").text(L).attr("x",(Y.stopx-Y.startx)/2-2*Ft.diagramMarginX).attr("y",-25),Iv(k,Z,ae,Ft.useMaxWidth);const $=L?40:0;k.attr("viewBox",Y.startx-Ft.diagramMarginX+" -"+(Ft.diagramMarginY+$)+" "+ae+" "+(Z+$)),Se.debug("models:",br.models)};function qZt(r,i,o){const l={};return i.forEach(function(f){if(r[f.to]&&r[f.from]){const b=r[f.to];if(f.placement===o.db.PLACEMENT.LEFTOF&&!b.prevActor||f.placement===o.db.PLACEMENT.RIGHTOF&&!b.nextActor)return;const d=f.placement!==void 0,w=!d,y=d?TC(Ft):d7(Ft),k=f.wrap?Fs.wrapLabel(f.message,Ft.width-2*Ft.wrapPadding,y):f.message,T=Fs.calculateTextDimensions(k,y).width+2*Ft.wrapPadding;w&&f.from===b.nextActor?l[f.to]=Math.max(l[f.to]||0,T):w&&f.from===b.prevActor?l[f.from]=Math.max(l[f.from]||0,T):w&&f.from===f.to?(l[f.from]=Math.max(l[f.from]||0,T/2),l[f.to]=Math.max(l[f.to]||0,T/2)):f.placement===o.db.PLACEMENT.RIGHTOF?l[f.from]=Math.max(l[f.from]||0,T):f.placement===o.db.PLACEMENT.LEFTOF?l[b.prevActor]=Math.max(l[b.prevActor]||0,T):f.placement===o.db.PLACEMENT.OVER&&(b.prevActor&&(l[b.prevActor]=Math.max(l[b.prevActor]||0,T/2)),b.nextActor&&(l[f.from]=Math.max(l[f.from]||0,T/2)))}}),Se.debug("maxMessageWidthPerActor:",l),l}const YZt=function(r){let i=0;const o=fce(Ft);for(const l in r.links){const b=Fs.calculateTextDimensions(l,o).width+2*Ft.wrapPadding+2*Ft.boxMargin;i<b&&(i=b)}return i};function WZt(r,i,o){let l=0;Object.keys(r).forEach(b=>{const d=r[b];d.wrap&&(d.description=Fs.wrapLabel(d.description,Ft.width-2*Ft.wrapPadding,fce(Ft)));const w=Fs.calculateTextDimensions(d.description,fce(Ft));d.width=d.wrap?Ft.width:Math.max(Ft.width,w.width+2*Ft.wrapPadding),d.height=d.wrap?Math.max(w.height,Ft.height):Ft.height,l=Math.max(l,d.height)});for(const b in i){const d=r[b];if(!d)continue;const w=r[d.nextActor];if(!w){const T=i[b]+Ft.actorMargin-d.width/2;d.margin=Math.max(T,Ft.actorMargin);continue}const k=i[b]+Ft.actorMargin-d.width/2-w.width/2;d.margin=Math.max(k,Ft.actorMargin)}let f=0;return o.forEach(b=>{const d=d7(Ft);let w=b.actorKeys.reduce((E,T)=>E+=r[T].width+(r[T].margin||0),0);w-=2*Ft.boxTextMargin,b.wrap&&(b.name=Fs.wrapLabel(b.name,w-2*Ft.wrapPadding,d));const y=Fs.calculateTextDimensions(b.name,d);f=Math.max(y.height,f);const k=Math.max(w,y.width+2*Ft.wrapPadding);if(b.margin=Ft.boxTextMargin,w<k){const E=(k-w)/2;b.margin+=E}}),o.forEach(b=>b.textMaxHeight=f),Math.max(l,Ft.height)}const KZt=function(r,i,o){const l=i[r.from].x,f=i[r.to].x,b=r.wrap&&r.message;let d=Fs.calculateTextDimensions(b?Fs.wrapLabel(r.message,Ft.width,TC(Ft)):r.message,TC(Ft));const w={width:b?Ft.width:Math.max(Ft.width,d.width+2*Ft.noteMargin),height:0,startx:i[r.from].x,stopx:0,starty:0,stopy:0,message:r.message};return r.placement===o.db.PLACEMENT.RIGHTOF?(w.width=b?Math.max(Ft.width,d.width):Math.max(i[r.from].width/2+i[r.to].width/2,d.width+2*Ft.noteMargin),w.startx=l+(i[r.from].width+Ft.actorMargin)/2):r.placement===o.db.PLACEMENT.LEFTOF?(w.width=Math.max(b?Ft.width:i[r.from].width/2+i[r.to].width/2,d.width+2*Ft.noteMargin),w.startx=l-w.width+(i[r.from].width-Ft.actorMargin)/2):r.to===r.from?(d=Fs.calculateTextDimensions(b?Fs.wrapLabel(r.message,Math.max(Ft.width,i[r.from].width),TC(Ft)):r.message,TC(Ft)),w.width=b?Math.max(Ft.width,i[r.from].width):Math.max(i[r.from].width,Ft.width,d.width+2*Ft.noteMargin),w.startx=l+(i[r.from].width-w.width)/2):(w.width=Math.abs(l+i[r.from].width/2-(f+i[r.to].width/2))+Ft.actorMargin,w.startx=l<f?l+i[r.from].width/2-Ft.actorMargin/2:f+i[r.to].width/2-Ft.actorMargin/2),b&&(w.message=Fs.wrapLabel(r.message,w.width-2*Ft.wrapPadding,TC(Ft))),Se.debug(`NM:[${w.startx},${w.stopx},${w.starty},${w.stopy}:${w.width},${w.height}=${r.message}]`),w},XZt=function(r,i,o){let l=!1;if([o.db.LINETYPE.SOLID_OPEN,o.db.LINETYPE.DOTTED_OPEN,o.db.LINETYPE.SOLID,o.db.LINETYPE.DOTTED,o.db.LINETYPE.SOLID_CROSS,o.db.LINETYPE.DOTTED_CROSS,o.db.LINETYPE.SOLID_POINT,o.db.LINETYPE.DOTTED_POINT].includes(r.type)&&(l=!0),!l)return{};const f=jLe(r.from,i),b=jLe(r.to,i),d=f[0]<=b[0]?1:0,w=f[0]<b[0]?0:1,y=[...f,...b],k=Math.abs(b[w]-f[d]);r.wrap&&r.message&&(r.message=Fs.wrapLabel(r.message,Math.max(k+2*Ft.wrapPadding,Ft.width),d7(Ft)));const E=Fs.calculateTextDimensions(r.message,d7(Ft));return{width:Math.max(r.wrap?0:E.width+2*Ft.wrapPadding,k+2*Ft.wrapPadding,Ft.width),height:0,startx:f[d],stopx:b[w],starty:0,stopy:0,message:r.message,type:r.type,wrap:r.wrap,fromBounds:Math.min.apply(null,y),toBounds:Math.max.apply(null,y)}},QZt=function(r,i,o,l){const f={},b=[];let d,w,y;return r.forEach(function(k){switch(k.id=Fs.random({length:10}),k.type){case l.db.LINETYPE.LOOP_START:case l.db.LINETYPE.ALT_START:case l.db.LINETYPE.OPT_START:case l.db.LINETYPE.PAR_START:case l.db.LINETYPE.CRITICAL_START:case l.db.LINETYPE.BREAK_START:b.push({id:k.id,msg:k.message,from:Number.MAX_SAFE_INTEGER,to:Number.MIN_SAFE_INTEGER,width:0});break;case l.db.LINETYPE.ALT_ELSE:case l.db.LINETYPE.PAR_AND:case l.db.LINETYPE.CRITICAL_OPTION:k.message&&(d=b.pop(),f[d.id]=d,f[k.id]=d,b.push(d));break;case l.db.LINETYPE.LOOP_END:case l.db.LINETYPE.ALT_END:case l.db.LINETYPE.OPT_END:case l.db.LINETYPE.PAR_END:case l.db.LINETYPE.CRITICAL_END:case l.db.LINETYPE.BREAK_END:d=b.pop(),f[d.id]=d;break;case l.db.LINETYPE.ACTIVE_START:{const T=i[k.from?k.from.actor:k.to.actor],C=WV(k.from?k.from.actor:k.to.actor).length,S=T.x+T.width/2+(C-1)*Ft.activationWidth/2,L={startx:S,stopx:S+Ft.activationWidth,actor:k.from.actor,enabled:!0};br.activations.push(L)}break;case l.db.LINETYPE.ACTIVE_END:{const T=br.activations.map(C=>C.actor).lastIndexOf(k.from.actor);delete br.activations.splice(T,1)[0]}break}k.placement!==void 0?(w=KZt(k,i,l),k.noteModel=w,b.forEach(T=>{d=T,d.from=Math.min(d.from,w.startx),d.to=Math.max(d.to,w.startx+w.width),d.width=Math.max(d.width,Math.abs(d.from-d.to))-Ft.labelBoxWidth})):(y=XZt(k,i,l),k.msgModel=y,y.startx&&y.stopx&&b.length>0&&b.forEach(T=>{if(d=T,y.startx===y.stopx){const C=i[k.from],S=i[k.to];d.from=Math.min(C.x-y.width/2,C.x-C.width/2,d.from),d.to=Math.max(S.x+y.width/2,S.x+C.width/2,d.to),d.width=Math.max(d.width,Math.abs(d.to-d.from))-Ft.labelBoxWidth}else d.from=Math.min(y.startx,d.from),d.to=Math.max(y.stopx,d.to),d.width=Math.max(d.width,y.width)-Ft.labelBoxWidth}))}),br.activations=[],Se.debug("Loop type widths:",f),f},$Le={bounds:br,drawActors:dce,drawActorsPopup:FLe,setConf:RLe,draw:UZt};var gce=function(){var r=function(Ce,_e,Te,Be){for(Te=Te||{},Be=Ce.length;Be--;Te[Ce[Be]]=_e);return Te},i=[1,2],o=[1,3],l=[1,5],f=[1,7],b=[2,5],d=[1,15],w=[1,17],y=[1,21],k=[1,22],E=[1,23],T=[1,24],C=[1,37],S=[1,25],L=[1,26],O=[1,27],B=[1,28],N=[1,29],F=[1,32],R=[1,33],q=[1,34],X=[1,35],te=[1,36],H=[1,39],Y=[1,40],z=[1,41],W=[1,42],Z=[1,38],G=[1,45],ae=[1,4,5,16,17,19,21,22,24,25,26,27,28,29,33,35,37,38,42,50,51,52,53,56,60],$=[1,4,5,14,15,16,17,19,21,22,24,25,26,27,28,29,33,35,37,38,42,50,51,52,53,56,60],ge=[1,4,5,7,16,17,19,21,22,24,25,26,27,28,29,33,35,37,38,42,50,51,52,53,56,60],ee=[4,5,16,17,19,21,22,24,25,26,27,28,29,33,35,37,38,42,50,51,52,53,56,60],de={trace:function(){},yy:{},symbols_:{error:2,start:3,SPACE:4,NL:5,directive:6,SD:7,document:8,line:9,statement:10,classDefStatement:11,cssClassStatement:12,idStatement:13,DESCR:14,"-->":15,HIDE_EMPTY:16,scale:17,WIDTH:18,COMPOSIT_STATE:19,STRUCT_START:20,STRUCT_STOP:21,STATE_DESCR:22,AS:23,ID:24,FORK:25,JOIN:26,CHOICE:27,CONCURRENT:28,note:29,notePosition:30,NOTE_TEXT:31,direction:32,acc_title:33,acc_title_value:34,acc_descr:35,acc_descr_value:36,acc_descr_multiline_value:37,classDef:38,CLASSDEF_ID:39,CLASSDEF_STYLEOPTS:40,DEFAULT:41,class:42,CLASSENTITY_IDS:43,STYLECLASS:44,openDirective:45,typeDirective:46,closeDirective:47,":":48,argDirective:49,direction_tb:50,direction_bt:51,direction_rl:52,direction_lr:53,eol:54,";":55,EDGE_STATE:56,STYLE_SEPARATOR:57,left_of:58,right_of:59,open_directive:60,type_directive:61,arg_directive:62,close_directive:63,$accept:0,$end:1},terminals_:{2:"error",4:"SPACE",5:"NL",7:"SD",14:"DESCR",15:"-->",16:"HIDE_EMPTY",17:"scale",18:"WIDTH",19:"COMPOSIT_STATE",20:"STRUCT_START",21:"STRUCT_STOP",22:"STATE_DESCR",23:"AS",24:"ID",25:"FORK",26:"JOIN",27:"CHOICE",28:"CONCURRENT",29:"note",31:"NOTE_TEXT",33:"acc_title",34:"acc_title_value",35:"acc_descr",36:"acc_descr_value",37:"acc_descr_multiline_value",38:"classDef",39:"CLASSDEF_ID",40:"CLASSDEF_STYLEOPTS",41:"DEFAULT",42:"class",43:"CLASSENTITY_IDS",44:"STYLECLASS",48:":",50:"direction_tb",51:"direction_bt",52:"direction_rl",53:"direction_lr",55:";",56:"EDGE_STATE",57:"STYLE_SEPARATOR",58:"left_of",59:"right_of",60:"open_directive",61:"type_directive",62:"arg_directive",63:"close_directive"},productions_:[0,[3,2],[3,2],[3,2],[3,2],[8,0],[8,2],[9,2],[9,1],[9,1],[10,1],[10,1],[10,1],[10,2],[10,3],[10,4],[10,1],[10,2],[10,1],[10,4],[10,3],[10,6],[10,1],[10,1],[10,1],[10,1],[10,4],[10,4],[10,1],[10,1],[10,2],[10,2],[10,1],[11,3],[11,3],[12,3],[6,3],[6,5],[32,1],[32,1],[32,1],[32,1],[54,1],[54,1],[13,1],[13,1],[13,3],[13,3],[30,1],[30,1],[45,1],[46,1],[49,1],[47,1]],performAction:function(_e,Te,Be,Ge,Xe,Ee,Ze){var Ie=Ee.length-1;switch(Xe){case 4:return Ge.setRootDoc(Ee[Ie]),Ee[Ie];case 5:this.$=[];break;case 6:Ee[Ie]!="nl"&&(Ee[Ie-1].push(Ee[Ie]),this.$=Ee[Ie-1]);break;case 7:case 8:this.$=Ee[Ie];break;case 9:this.$="nl";break;case 12:this.$=Ee[Ie];break;case 13:const xe=Ee[Ie-1];xe.description=Ge.trimColon(Ee[Ie]),this.$=xe;break;case 14:this.$={stmt:"relation",state1:Ee[Ie-2],state2:Ee[Ie]};break;case 15:const Ae=Ge.trimColon(Ee[Ie]);this.$={stmt:"relation",state1:Ee[Ie-3],state2:Ee[Ie-1],description:Ae};break;case 19:this.$={stmt:"state",id:Ee[Ie-3],type:"default",description:"",doc:Ee[Ie-1]};break;case 20:var Oe=Ee[Ie],Le=Ee[Ie-2].trim();if(Ee[Ie].match(":")){var $e=Ee[Ie].split(":");Oe=$e[0],Le=[Le,$e[1]]}this.$={stmt:"state",id:Oe,type:"default",description:Le};break;case 21:this.$={stmt:"state",id:Ee[Ie-3],type:"default",description:Ee[Ie-5],doc:Ee[Ie-1]};break;case 22:this.$={stmt:"state",id:Ee[Ie],type:"fork"};break;case 23:this.$={stmt:"state",id:Ee[Ie],type:"join"};break;case 24:this.$={stmt:"state",id:Ee[Ie],type:"choice"};break;case 25:this.$={stmt:"state",id:Ge.getDividerId(),type:"divider"};break;case 26:this.$={stmt:"state",id:Ee[Ie-1].trim(),note:{position:Ee[Ie-2].trim(),text:Ee[Ie].trim()}};break;case 30:this.$=Ee[Ie].trim(),Ge.setAccTitle(this.$);break;case 31:case 32:this.$=Ee[Ie].trim(),Ge.setAccDescription(this.$);break;case 33:case 34:this.$={stmt:"classDef",id:Ee[Ie-1].trim(),classes:Ee[Ie].trim()};break;case 35:this.$={stmt:"applyClass",id:Ee[Ie-1].trim(),styleClass:Ee[Ie].trim()};break;case 38:Ge.setDirection("TB"),this.$={stmt:"dir",value:"TB"};break;case 39:Ge.setDirection("BT"),this.$={stmt:"dir",value:"BT"};break;case 40:Ge.setDirection("RL"),this.$={stmt:"dir",value:"RL"};break;case 41:Ge.setDirection("LR"),this.$={stmt:"dir",value:"LR"};break;case 44:case 45:this.$={stmt:"state",id:Ee[Ie].trim(),type:"default",description:""};break;case 46:this.$={stmt:"state",id:Ee[Ie-2].trim(),classes:[Ee[Ie].trim()],type:"default",description:""};break;case 47:this.$={stmt:"state",id:Ee[Ie-2].trim(),classes:[Ee[Ie].trim()],type:"default",description:""};break;case 50:Ge.parseDirective("%%{","open_directive");break;case 51:Ge.parseDirective(Ee[Ie],"type_directive");break;case 52:Ee[Ie]=Ee[Ie].trim().replace(/'/g,'"'),Ge.parseDirective(Ee[Ie],"arg_directive");break;case 53:Ge.parseDirective("}%%","close_directive","state");break}},table:[{3:1,4:i,5:o,6:4,7:l,45:6,60:f},{1:[3]},{3:8,4:i,5:o,6:4,7:l,45:6,60:f},{3:9,4:i,5:o,6:4,7:l,45:6,60:f},{3:10,4:i,5:o,6:4,7:l,45:6,60:f},r([1,4,5,16,17,19,22,24,25,26,27,28,29,33,35,37,38,42,50,51,52,53,56,60],b,{8:11}),{46:12,61:[1,13]},{61:[2,50]},{1:[2,1]},{1:[2,2]},{1:[2,3]},{1:[2,4],4:d,5:w,6:30,9:14,10:16,11:18,12:19,13:20,16:y,17:k,19:E,22:T,24:C,25:S,26:L,27:O,28:B,29:N,32:31,33:F,35:R,37:q,38:X,42:te,45:6,50:H,51:Y,52:z,53:W,56:Z,60:f},{47:43,48:[1,44],63:G},r([48,63],[2,51]),r(ae,[2,6]),{6:30,10:46,11:18,12:19,13:20,16:y,17:k,19:E,22:T,24:C,25:S,26:L,27:O,28:B,29:N,32:31,33:F,35:R,37:q,38:X,42:te,45:6,50:H,51:Y,52:z,53:W,56:Z,60:f},r(ae,[2,8]),r(ae,[2,9]),r(ae,[2,10]),r(ae,[2,11]),r(ae,[2,12],{14:[1,47],15:[1,48]}),r(ae,[2,16]),{18:[1,49]},r(ae,[2,18],{20:[1,50]}),{23:[1,51]},r(ae,[2,22]),r(ae,[2,23]),r(ae,[2,24]),r(ae,[2,25]),{30:52,31:[1,53],58:[1,54],59:[1,55]},r(ae,[2,28]),r(ae,[2,29]),{34:[1,56]},{36:[1,57]},r(ae,[2,32]),{39:[1,58],41:[1,59]},{43:[1,60]},r($,[2,44],{57:[1,61]}),r($,[2,45],{57:[1,62]}),r(ae,[2,38]),r(ae,[2,39]),r(ae,[2,40]),r(ae,[2,41]),r(ge,[2,36]),{49:63,62:[1,64]},r(ge,[2,53]),r(ae,[2,7]),r(ae,[2,13]),{13:65,24:C,56:Z},r(ae,[2,17]),r(ee,b,{8:66}),{24:[1,67]},{24:[1,68]},{23:[1,69]},{24:[2,48]},{24:[2,49]},r(ae,[2,30]),r(ae,[2,31]),{40:[1,70]},{40:[1,71]},{44:[1,72]},{24:[1,73]},{24:[1,74]},{47:75,63:G},{63:[2,52]},r(ae,[2,14],{14:[1,76]}),{4:d,5:w,6:30,9:14,10:16,11:18,12:19,13:20,16:y,17:k,19:E,21:[1,77],22:T,24:C,25:S,26:L,27:O,28:B,29:N,32:31,33:F,35:R,37:q,38:X,42:te,45:6,50:H,51:Y,52:z,53:W,56:Z,60:f},r(ae,[2,20],{20:[1,78]}),{31:[1,79]},{24:[1,80]},r(ae,[2,33]),r(ae,[2,34]),r(ae,[2,35]),r($,[2,46]),r($,[2,47]),r(ge,[2,37]),r(ae,[2,15]),r(ae,[2,19]),r(ee,b,{8:81}),r(ae,[2,26]),r(ae,[2,27]),{4:d,5:w,6:30,9:14,10:16,11:18,12:19,13:20,16:y,17:k,19:E,21:[1,82],22:T,24:C,25:S,26:L,27:O,28:B,29:N,32:31,33:F,35:R,37:q,38:X,42:te,45:6,50:H,51:Y,52:z,53:W,56:Z,60:f},r(ae,[2,21])],defaultActions:{7:[2,50],8:[2,1],9:[2,2],10:[2,3],54:[2,48],55:[2,49],64:[2,52]},parseError:function(_e,Te){if(Te.recoverable)this.trace(_e);else{var Be=new Error(_e);throw Be.hash=Te,Be}},parse:function(_e){var Te=this,Be=[0],Ge=[],Xe=[null],Ee=[],Ze=this.table,Ie="",Oe=0,Le=0,$e=2,xe=1,Ae=Ee.slice.call(arguments,1),je=Object.create(this.lexer),me={yy:{}};for(var vt in this.yy)Object.prototype.hasOwnProperty.call(this.yy,vt)&&(me.yy[vt]=this.yy[vt]);je.setInput(_e,me.yy),me.yy.lexer=je,me.yy.parser=this,typeof je.yylloc>"u"&&(je.yylloc={});var ve=je.yylloc;Ee.push(ve);var Zt=je.options&&je.options.ranges;typeof me.yy.parseError=="function"?this.parseError=me.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function nt(){var Gt;return Gt=Ge.pop()||je.lex()||xe,typeof Gt!="number"&&(Gt instanceof Array&&(Ge=Gt,Gt=Ge.pop()),Gt=Te.symbols_[Gt]||Gt),Gt}for(var xn,cn,jt,ot,be={},We,ct,Yt,Ut;;){if(cn=Be[Be.length-1],this.defaultActions[cn]?jt=this.defaultActions[cn]:((xn===null||typeof xn>"u")&&(xn=nt()),jt=Ze[cn]&&Ze[cn][xn]),typeof jt>"u"||!jt.length||!jt[0]){var Wn="";Ut=[];for(We in Ze[cn])this.terminals_[We]&&We>$e&&Ut.push("'"+this.terminals_[We]+"'");je.showPosition?Wn="Parse error on line "+(Oe+1)+`:
`+je.showPosition()+`
Expecting `+Ut.join(", ")+", got '"+(this.terminals_[xn]||xn)+"'":Wn="Parse error on line "+(Oe+1)+": Unexpected "+(xn==xe?"end of input":"'"+(this.terminals_[xn]||xn)+"'"),this.parseError(Wn,{text:je.match,token:this.terminals_[xn]||xn,line:je.yylineno,loc:ve,expected:Ut})}if(jt[0]instanceof Array&&jt.length>1)throw new Error("Parse Error: multiple actions possible at state: "+cn+", token: "+xn);switch(jt[0]){case 1:Be.push(xn),Xe.push(je.yytext),Ee.push(je.yylloc),Be.push(jt[1]),xn=null,Le=je.yyleng,Ie=je.yytext,Oe=je.yylineno,ve=je.yylloc;break;case 2:if(ct=this.productions_[jt[1]][1],be.$=Xe[Xe.length-ct],be._$={first_line:Ee[Ee.length-(ct||1)].first_line,last_line:Ee[Ee.length-1].last_line,first_column:Ee[Ee.length-(ct||1)].first_column,last_column:Ee[Ee.length-1].last_column},Zt&&(be._$.range=[Ee[Ee.length-(ct||1)].range[0],Ee[Ee.length-1].range[1]]),ot=this.performAction.apply(be,[Ie,Le,Oe,me.yy,jt[1],Xe,Ee].concat(Ae)),typeof ot<"u")return ot;ct&&(Be=Be.slice(0,-1*ct*2),Xe=Xe.slice(0,-1*ct),Ee=Ee.slice(0,-1*ct)),Be.push(this.productions_[jt[1]][0]),Xe.push(be.$),Ee.push(be._$),Yt=Ze[Be[Be.length-2]][Be[Be.length-1]],Be.push(Yt);break;case 3:return!0}}return!0}},re=function(){var Ce={EOF:1,parseError:function(Te,Be){if(this.yy.parser)this.yy.parser.parseError(Te,Be);else throw new Error(Te)},setInput:function(_e,Te){return this.yy=Te||this.yy||{},this._input=_e,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var _e=this._input[0];this.yytext+=_e,this.yyleng++,this.offset++,this.match+=_e,this.matched+=_e;var Te=_e.match(/(?:\r\n?|\n).*/g);return Te?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),_e},unput:function(_e){var Te=_e.length,Be=_e.split(/(?:\r\n?|\n)/g);this._input=_e+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-Te),this.offset-=Te;var Ge=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),Be.length-1&&(this.yylineno-=Be.length-1);var Xe=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:Be?(Be.length===Ge.length?this.yylloc.first_column:0)+Ge[Ge.length-Be.length].length-Be[0].length:this.yylloc.first_column-Te},this.options.ranges&&(this.yylloc.range=[Xe[0],Xe[0]+this.yyleng-Te]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(_e){this.unput(this.match.slice(_e))},pastInput:function(){var _e=this.matched.substr(0,this.matched.length-this.match.length);return(_e.length>20?"...":"")+_e.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var _e=this.match;return _e.length<20&&(_e+=this._input.substr(0,20-_e.length)),(_e.substr(0,20)+(_e.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var _e=this.pastInput(),Te=new Array(_e.length+1).join("-");return _e+this.upcomingInput()+`
`+Te+"^"},test_match:function(_e,Te){var Be,Ge,Xe;if(this.options.backtrack_lexer&&(Xe={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(Xe.yylloc.range=this.yylloc.range.slice(0))),Ge=_e[0].match(/(?:\r\n?|\n).*/g),Ge&&(this.yylineno+=Ge.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:Ge?Ge[Ge.length-1].length-Ge[Ge.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+_e[0].length},this.yytext+=_e[0],this.match+=_e[0],this.matches=_e,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(_e[0].length),this.matched+=_e[0],Be=this.performAction.call(this,this.yy,this,Te,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),Be)return Be;if(this._backtrack){for(var Ee in Xe)this[Ee]=Xe[Ee];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var _e,Te,Be,Ge;this._more||(this.yytext="",this.match="");for(var Xe=this._currentRules(),Ee=0;Ee<Xe.length;Ee++)if(Be=this._input.match(this.rules[Xe[Ee]]),Be&&(!Te||Be[0].length>Te[0].length)){if(Te=Be,Ge=Ee,this.options.backtrack_lexer){if(_e=this.test_match(Be,Xe[Ee]),_e!==!1)return _e;if(this._backtrack){Te=!1;continue}else return!1}else if(!this.options.flex)break}return Te?(_e=this.test_match(Te,Xe[Ge]),_e!==!1?_e:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var Te=this.next();return Te||this.lex()},begin:function(Te){this.conditionStack.push(Te)},popState:function(){var Te=this.conditionStack.length-1;return Te>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(Te){return Te=this.conditionStack.length-1-Math.abs(Te||0),Te>=0?this.conditionStack[Te]:"INITIAL"},pushState:function(Te){this.begin(Te)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(Te,Be,Ge,Xe){switch(Ge){case 0:return 41;case 1:return 50;case 2:return 51;case 3:return 52;case 4:return 53;case 5:return this.begin("open_directive"),60;case 6:return this.begin("type_directive"),61;case 7:return this.popState(),this.begin("arg_directive"),48;case 8:return this.popState(),this.popState(),63;case 9:return 62;case 10:break;case 11:break;case 12:return 5;case 13:break;case 14:break;case 15:break;case 16:break;case 17:return this.pushState("SCALE"),17;case 18:return 18;case 19:this.popState();break;case 20:return this.begin("acc_title"),33;case 21:return this.popState(),"acc_title_value";case 22:return this.begin("acc_descr"),35;case 23:return this.popState(),"acc_descr_value";case 24:this.begin("acc_descr_multiline");break;case 25:this.popState();break;case 26:return"acc_descr_multiline_value";case 27:return this.pushState("CLASSDEF"),38;case 28:return this.popState(),this.pushState("CLASSDEFID"),"DEFAULT_CLASSDEF_ID";case 29:return this.popState(),this.pushState("CLASSDEFID"),39;case 30:return this.popState(),40;case 31:return this.pushState("CLASS"),42;case 32:return this.popState(),this.pushState("CLASS_STYLE"),43;case 33:return this.popState(),44;case 34:return this.pushState("SCALE"),17;case 35:return 18;case 36:this.popState();break;case 37:this.pushState("STATE");break;case 38:return this.popState(),Be.yytext=Be.yytext.slice(0,-8).trim(),25;case 39:return this.popState(),Be.yytext=Be.yytext.slice(0,-8).trim(),26;case 40:return this.popState(),Be.yytext=Be.yytext.slice(0,-10).trim(),27;case 41:return this.popState(),Be.yytext=Be.yytext.slice(0,-8).trim(),25;case 42:return this.popState(),Be.yytext=Be.yytext.slice(0,-8).trim(),26;case 43:return this.popState(),Be.yytext=Be.yytext.slice(0,-10).trim(),27;case 44:return 50;case 45:return 51;case 46:return 52;case 47:return 53;case 48:this.pushState("STATE_STRING");break;case 49:return this.pushState("STATE_ID"),"AS";case 50:return this.popState(),"ID";case 51:this.popState();break;case 52:return"STATE_DESCR";case 53:return 19;case 54:this.popState();break;case 55:return this.popState(),this.pushState("struct"),20;case 56:break;case 57:return this.popState(),21;case 58:break;case 59:return this.begin("NOTE"),29;case 60:return this.popState(),this.pushState("NOTE_ID"),58;case 61:return this.popState(),this.pushState("NOTE_ID"),59;case 62:this.popState(),this.pushState("FLOATING_NOTE");break;case 63:return this.popState(),this.pushState("FLOATING_NOTE_ID"),"AS";case 64:break;case 65:return"NOTE_TEXT";case 66:return this.popState(),"ID";case 67:return this.popState(),this.pushState("NOTE_TEXT"),24;case 68:return this.popState(),Be.yytext=Be.yytext.substr(2).trim(),31;case 69:return this.popState(),Be.yytext=Be.yytext.slice(0,-8).trim(),31;case 70:return 7;case 71:return 7;case 72:return 16;case 73:return 56;case 74:return 24;case 75:return Be.yytext=Be.yytext.trim(),14;case 76:return 15;case 77:return 28;case 78:return 57;case 79:return 5;case 80:return"INVALID"}},rules:[/^(?:default\b)/i,/^(?:.*direction\s+TB[^\n]*)/i,/^(?:.*direction\s+BT[^\n]*)/i,/^(?:.*direction\s+RL[^\n]*)/i,/^(?:.*direction\s+LR[^\n]*)/i,/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:%%(?!\{)[^\n]*)/i,/^(?:[^\}]%%[^\n]*)/i,/^(?:[\n]+)/i,/^(?:[\s]+)/i,/^(?:((?!\n)\s)+)/i,/^(?:#[^\n]*)/i,/^(?:%[^\n]*)/i,/^(?:scale\s+)/i,/^(?:\d+)/i,/^(?:\s+width\b)/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:classDef\s+)/i,/^(?:DEFAULT\s+)/i,/^(?:\w+\s+)/i,/^(?:[^\n]*)/i,/^(?:class\s+)/i,/^(?:(\w+)+((,\s*\w+)*))/i,/^(?:[^\n]*)/i,/^(?:scale\s+)/i,/^(?:\d+)/i,/^(?:\s+width\b)/i,/^(?:state\s+)/i,/^(?:.*<<fork>>)/i,/^(?:.*<<join>>)/i,/^(?:.*<<choice>>)/i,/^(?:.*\[\[fork\]\])/i,/^(?:.*\[\[join\]\])/i,/^(?:.*\[\[choice\]\])/i,/^(?:.*direction\s+TB[^\n]*)/i,/^(?:.*direction\s+BT[^\n]*)/i,/^(?:.*direction\s+RL[^\n]*)/i,/^(?:.*direction\s+LR[^\n]*)/i,/^(?:["])/i,/^(?:\s*as\s+)/i,/^(?:[^\n\{]*)/i,/^(?:["])/i,/^(?:[^"]*)/i,/^(?:[^\n\s\{]+)/i,/^(?:\n)/i,/^(?:\{)/i,/^(?:%%(?!\{)[^\n]*)/i,/^(?:\})/i,/^(?:[\n])/i,/^(?:note\s+)/i,/^(?:left of\b)/i,/^(?:right of\b)/i,/^(?:")/i,/^(?:\s*as\s*)/i,/^(?:["])/i,/^(?:[^"]*)/i,/^(?:[^\n]*)/i,/^(?:\s*[^:\n\s\-]+)/i,/^(?:\s*:[^:\n;]+)/i,/^(?:[\s\S]*?end note\b)/i,/^(?:stateDiagram\s+)/i,/^(?:stateDiagram-v2\s+)/i,/^(?:hide empty description\b)/i,/^(?:\[\*\])/i,/^(?:[^:\n\s\-\{]+)/i,/^(?:\s*:[^:\n;]+)/i,/^(?:-->)/i,/^(?:--)/i,/^(?::::)/i,/^(?:$)/i,/^(?:.)/i],conditions:{LINE:{rules:[14,15],inclusive:!1},close_directive:{rules:[14,15],inclusive:!1},arg_directive:{rules:[8,9,14,15],inclusive:!1},type_directive:{rules:[7,8,14,15],inclusive:!1},open_directive:{rules:[6,14,15],inclusive:!1},struct:{rules:[14,15,27,31,37,44,45,46,47,56,57,58,59,73,74,75,76,77],inclusive:!1},FLOATING_NOTE_ID:{rules:[66],inclusive:!1},FLOATING_NOTE:{rules:[63,64,65],inclusive:!1},NOTE_TEXT:{rules:[68,69],inclusive:!1},NOTE_ID:{rules:[67],inclusive:!1},NOTE:{rules:[60,61,62],inclusive:!1},CLASS_STYLE:{rules:[33],inclusive:!1},CLASS:{rules:[32],inclusive:!1},CLASSDEFID:{rules:[30],inclusive:!1},CLASSDEF:{rules:[28,29],inclusive:!1},acc_descr_multiline:{rules:[25,26],inclusive:!1},acc_descr:{rules:[23],inclusive:!1},acc_title:{rules:[21],inclusive:!1},SCALE:{rules:[18,19,35,36],inclusive:!1},ALIAS:{rules:[],inclusive:!1},STATE_ID:{rules:[50],inclusive:!1},STATE_STRING:{rules:[51,52],inclusive:!1},FORK_STATE:{rules:[],inclusive:!1},STATE:{rules:[14,15,38,39,40,41,42,43,48,49,53,54,55],inclusive:!1},ID:{rules:[14,15],inclusive:!1},INITIAL:{rules:[0,1,2,3,4,5,10,11,12,13,15,16,17,20,22,24,27,31,34,37,55,59,70,71,72,73,74,75,76,78,79,80],inclusive:!0}}};return Ce}();de.lexer=re;function ke(){this.yy={}}return ke.prototype=de,de.Parser=ke,new ke}();gce.parser=gce;const HLe=gce,ZZt=(r,i)=>{var o;return((o=i==null?void 0:i.state)==null?void 0:o.defaultRenderer)==="dagre-wrapper"?!1:r.match(/^\s*stateDiagram/)!==null},JZt=(r,i)=>{var o;return!!(r.match(/^\s*stateDiagram-v2/)!==null||r.match(/^\s*stateDiagram/)&&((o=i==null?void 0:i.state)==null?void 0:o.defaultRenderer)==="dagre-wrapper")},zLe="LR",eJt="TB",KV="state",pce="relation",tJt="classDef",nJt="applyClass",XI="default",GLe="divider",bce="[*]",VLe="start",ULe=bce,qLe="end",YLe="color",WLe="fill",rJt="bgFill",iJt=",";function KLe(){return{}}let XLe=zLe,XV=[],QI=KLe();const QLe=()=>({relations:[],states:{},documents:{}});let QV={root:QLe()},hd=QV.root,ZI=0,ZLe=0;const sJt={LINE:0,DOTTED_LINE:1},aJt={AGGREGATION:0,EXTENSION:1,COMPOSITION:2,DEPENDENCY:3},ZV=r=>JSON.parse(JSON.stringify(r)),oJt=function(r,i,o){Bu.parseDirective(this,r,i,o)},cJt=r=>{Se.info("Setting root doc",r),XV=r},uJt=()=>XV,JV=(r,i,o)=>{if(i.stmt===pce)JV(r,i.state1,!0),JV(r,i.state2,!1);else if(i.stmt===KV&&(i.id==="[*]"?(i.id=o?r.id+"_start":r.id+"_end",i.start=o):i.id=i.id.trim()),i.doc){const l=[];let f=[],b;for(b=0;b<i.doc.length;b++)if(i.doc[b].type===GLe){const d=ZV(i.doc[b]);d.doc=ZV(f),l.push(d),f=[]}else f.push(i.doc[b]);if(l.length>0&&f.length>0){const d={stmt:KV,id:VCe(),type:"divider",doc:ZV(f)};l.push(ZV(d)),i.doc=l}i.doc.forEach(d=>JV(i,d,!0))}},lJt=()=>(JV({id:"root"},{id:"root",doc:XV},!0),{id:"root",doc:XV}),hJt=r=>{let i;r.doc?i=r.doc:i=r,Se.info(i),JLe(!0),Se.info("Extract",i),i.forEach(o=>{switch(o.stmt){case KV:g7(o.id.trim(),o.type,o.doc,o.description,o.note,o.classes,o.styles,o.textStyles);break;case pce:eIe(o.state1,o.state2,o.description);break;case tJt:tIe(o.id.trim(),o.classes);break;case nJt:yce(o.id.trim(),o.styleClass);break}})},g7=function(r,i=XI,o=null,l=null,f=null,b=null,d=null,w=null){const y=r==null?void 0:r.trim();hd.states[y]===void 0?(Se.info("Adding state ",y,l),hd.states[y]={id:y,descriptions:[],type:i,doc:o,note:f,classes:[],styles:[],textStyles:[]}):(hd.states[y].doc||(hd.states[y].doc=o),hd.states[y].type||(hd.states[y].type=i)),l&&(Se.info("Setting state description",y,l),typeof l=="string"&&mce(y,l.trim()),typeof l=="object"&&l.forEach(k=>mce(y,k.trim()))),f&&(hd.states[y].note=f,hd.states[y].note.text=Cs.sanitizeText(hd.states[y].note.text,ht())),b&&(Se.info("Setting state classes",y,b),(typeof b=="string"?[b]:b).forEach(E=>yce(y,E.trim()))),d&&(Se.info("Setting state styles",y,d),(typeof d=="string"?[d]:d).forEach(E=>kJt(y,E.trim()))),w&&(Se.info("Setting state styles",y,d),(typeof w=="string"?[w]:w).forEach(E=>xJt(y,E.trim())))},JLe=function(r){QV={root:QLe()},hd=QV.root,ZI=0,QI=KLe(),r||W0()},JI=function(r){return hd.states[r]},fJt=function(){return hd.states},dJt=function(){Se.info("Documents = ",QV)},gJt=function(){return hd.relations};function vce(r=""){let i=r;return r===bce&&(ZI++,i=`${VLe}${ZI}`),i}function wce(r="",i=XI){return r===bce?VLe:i}function pJt(r=""){let i=r;return r===ULe&&(ZI++,i=`${qLe}${ZI}`),i}function bJt(r="",i=XI){return r===ULe?qLe:i}function vJt(r,i,o){let l=vce(r.id.trim()),f=wce(r.id.trim(),r.type),b=vce(i.id.trim()),d=wce(i.id.trim(),i.type);g7(l,f,r.doc,r.description,r.note,r.classes,r.styles,r.textStyles),g7(b,d,i.doc,i.description,i.note,i.classes,i.styles,i.textStyles),hd.relations.push({id1:l,id2:b,relationTitle:Cs.sanitizeText(o,ht())})}const eIe=function(r,i,o){if(typeof r=="object")vJt(r,i,o);else{const l=vce(r.trim()),f=wce(r),b=pJt(i.trim()),d=bJt(i);g7(l,f),g7(b,d),hd.relations.push({id1:l,id2:b,title:Cs.sanitizeText(o,ht())})}},mce=function(r,i){const o=hd.states[r],l=i.startsWith(":")?i.replace(":","").trim():i;o.descriptions.push(Cs.sanitizeText(l,ht()))},wJt=function(r){return r.substring(0,1)===":"?r.substr(2).trim():r.trim()},mJt=()=>(ZLe++,"divider-id-"+ZLe),tIe=function(r,i=""){QI[r]===void 0&&(QI[r]={id:r,styles:[],textStyles:[]});const o=QI[r];i!=null&&i.split(iJt).forEach(l=>{const f=l.replace(/([^;]*);/,"$1").trim();if(l.match(YLe)){const d=f.replace(WLe,rJt).replace(YLe,WLe);o.textStyles.push(d)}o.styles.push(f)})},yJt=function(){return QI},yce=function(r,i){r.split(",").forEach(function(o){let l=JI(o);if(l===void 0){const f=o.trim();g7(f),l=JI(f)}l.classes.push(i)})},kJt=function(r,i){const o=JI(r);o!==void 0&&o.textStyles.push(i)},xJt=function(r,i){const o=JI(r);o!==void 0&&o.textStyles.push(i)},u4={parseDirective:oJt,getConfig:()=>ht().state,addState:g7,clear:JLe,getState:JI,getStates:fJt,getRelations:gJt,getClasses:yJt,getDirection:()=>XLe,addRelation:eIe,getDividerId:mJt,setDirection:r=>{XLe=r},cleanupLabel:wJt,lineType:sJt,relationType:aJt,logDocuments:dJt,getRootDoc:uJt,setRootDoc:cJt,getRootDocV2:lJt,extract:hJt,trimColon:r=>r&&r[0]===":"?r.substr(1).trim():r.trim(),getAccTitle:hp,setAccTitle:K0,getAccDescription:dp,setAccDescription:fp,addStyleClass:tIe,setCssClass:yce,addDescription:mce,setDiagramTitle:Ov,getDiagramTitle:Nv},kce={},EJt=(r,i)=>{kce[r]=i},_Jt=r=>kce[r],nIe=()=>Object.keys(kce),TJt={get:_Jt,set:EJt,keys:nIe,size:()=>nIe().length},CJt=r=>r.append("circle").attr("class","start-state").attr("r",ht().state.sizeUnit).attr("cx",ht().state.padding+ht().state.sizeUnit).attr("cy",ht().state.padding+ht().state.sizeUnit),SJt=r=>r.append("line").style("stroke","grey").style("stroke-dasharray","3").attr("x1",ht().state.textHeight).attr("class","divider").attr("x2",ht().state.textHeight*2).attr("y1",0).attr("y2",0),AJt=(r,i)=>{const o=r.append("text").attr("x",2*ht().state.padding).attr("y",ht().state.textHeight+2*ht().state.padding).attr("font-size",ht().state.fontSize).attr("class","state-title").text(i.id),l=o.node().getBBox();return r.insert("rect",":first-child").attr("x",ht().state.padding).attr("y",ht().state.padding).attr("width",l.width+2*ht().state.padding).attr("height",l.height+2*ht().state.padding).attr("rx",ht().state.radius),o},MJt=(r,i)=>{const o=function(C,S,L){const O=C.append("tspan").attr("x",2*ht().state.padding).text(S);L||O.attr("dy",ht().state.textHeight)},f=r.append("text").attr("x",2*ht().state.padding).attr("y",ht().state.textHeight+1.3*ht().state.padding).attr("font-size",ht().state.fontSize).attr("class","state-title").text(i.descriptions[0]).node().getBBox(),b=f.height,d=r.append("text").attr("x",ht().state.padding).attr("y",b+ht().state.padding*.4+ht().state.dividerMargin+ht().state.textHeight).attr("class","state-description");let w=!0,y=!0;i.descriptions.forEach(function(C){w||(o(d,C,y),y=!1),w=!1});const k=r.append("line").attr("x1",ht().state.padding).attr("y1",ht().state.padding+b+ht().state.dividerMargin/2).attr("y2",ht().state.padding+b+ht().state.dividerMargin/2).attr("class","descr-divider"),E=d.node().getBBox(),T=Math.max(E.width,f.width);return k.attr("x2",T+3*ht().state.padding),r.insert("rect",":first-child").attr("x",ht().state.padding).attr("y",ht().state.padding).attr("width",T+2*ht().state.padding).attr("height",E.height+b+2*ht().state.padding).attr("rx",ht().state.radius),r},DJt=(r,i,o)=>{const l=ht().state.padding,f=2*ht().state.padding,b=r.node().getBBox(),d=b.width,w=b.x,y=r.append("text").attr("x",0).attr("y",ht().state.titleShift).attr("font-size",ht().state.fontSize).attr("class","state-title").text(i.id),E=y.node().getBBox().width+f;let T=Math.max(E,d);T===d&&(T=T+f);let C;const S=r.node().getBBox();i.doc,C=w-l,E>d&&(C=(d-T)/2+l),Math.abs(w-S.x)<l&&E>d&&(C=w-(E-d)/2);const L=1-ht().state.textHeight;return r.insert("rect",":first-child").attr("x",C).attr("y",L).attr("class",o?"alt-composit":"composit").attr("width",T).attr("height",S.height+ht().state.textHeight+ht().state.titleShift+1).attr("rx","0"),y.attr("x",C+l),E<=d&&y.attr("x",w+(T-f)/2-E/2+l),r.insert("rect",":first-child").attr("x",C).attr("y",ht().state.titleShift-ht().state.textHeight-ht().state.padding).attr("width",T).attr("height",ht().state.textHeight*3).attr("rx",ht().state.radius),r.insert("rect",":first-child").attr("x",C).attr("y",ht().state.titleShift-ht().state.textHeight-ht().state.padding).attr("width",T).attr("height",S.height+3+2*ht().state.textHeight).attr("rx",ht().state.radius),r},LJt=r=>(r.append("circle").attr("class","end-state-outer").attr("r",ht().state.sizeUnit+ht().state.miniPadding).attr("cx",ht().state.padding+ht().state.sizeUnit+ht().state.miniPadding).attr("cy",ht().state.padding+ht().state.sizeUnit+ht().state.miniPadding),r.append("circle").attr("class","end-state-inner").attr("r",ht().state.sizeUnit).attr("cx",ht().state.padding+ht().state.sizeUnit+2).attr("cy",ht().state.padding+ht().state.sizeUnit+2)),IJt=(r,i)=>{let o=ht().state.forkWidth,l=ht().state.forkHeight;if(i.parentId){let f=o;o=l,l=f}return r.append("rect").style("stroke","black").style("fill","black").attr("width",o).attr("height",l).attr("x",ht().state.padding).attr("y",ht().state.padding)},OJt=(r,i,o,l)=>{let f=0;const b=l.append("text");b.style("text-anchor","start"),b.attr("class","noteText");let d=r.replace(/\r\n/g,"<br/>");d=d.replace(/\n/g,"<br/>");const w=d.split(Cs.lineBreakRegex);let y=1.25*ht().state.noteMargin;for(const k of w){const E=k.trim();if(E.length>0){const T=b.append("tspan");if(T.text(E),y===0){const C=T.node().getBBox();y+=C.height}f+=y,T.attr("x",i+ht().state.noteMargin),T.attr("y",o+f+1.25*ht().state.noteMargin)}}return{textWidth:b.node().getBBox().width,textHeight:f}},NJt=(r,i)=>{i.attr("class","state-note");const o=i.append("rect").attr("x",0).attr("y",ht().state.padding),l=i.append("g"),{textWidth:f,textHeight:b}=OJt(r,0,0,l);return o.attr("height",b+2*ht().state.noteMargin),o.attr("width",f+ht().state.noteMargin*2),o},rIe=function(r,i){const o=i.id,l={id:o,label:i.id,width:0,height:0},f=r.append("g").attr("id",o).attr("class","stateGroup");i.type==="start"&&CJt(f),i.type==="end"&&LJt(f),(i.type==="fork"||i.type==="join")&&IJt(f,i),i.type==="note"&&NJt(i.note.text,f),i.type==="divider"&&SJt(f),i.type==="default"&&i.descriptions.length===0&&AJt(f,i),i.type==="default"&&i.descriptions.length>0&&MJt(f,i);const b=f.node().getBBox();return l.width=b.width+2*ht().state.padding,l.height=b.height+2*ht().state.padding,TJt.set(o,l),l};let iIe=0;const PJt=function(r,i,o){const l=function(y){switch(y){case u4.relationType.AGGREGATION:return"aggregation";case u4.relationType.EXTENSION:return"extension";case u4.relationType.COMPOSITION:return"composition";case u4.relationType.DEPENDENCY:return"dependency"}};i.points=i.points.filter(y=>!Number.isNaN(y.y));const f=i.points,b=V8().x(function(y){return y.x}).y(function(y){return y.y}).curve(UT),d=r.append("path").attr("d",b(f)).attr("id","edge"+iIe).attr("class","transition");let w="";if(ht().state.arrowMarkerAbsolute&&(w=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,w=w.replace(/\(/g,"\\("),w=w.replace(/\)/g,"\\)")),d.attr("marker-end","url("+w+"#"+l(u4.relationType.DEPENDENCY)+"End)"),o.title!==void 0){const y=r.append("g").attr("class","stateLabel"),{x:k,y:E}=Fs.calcLabelPosition(i.points),T=Cs.getRows(o.title);let C=0;const S=[];let L=0,O=0;for(let F=0;F<=T.length;F++){const R=y.append("text").attr("text-anchor","middle").text(T[F]).attr("x",k).attr("y",E+C),q=R.node().getBBox();L=Math.max(L,q.width),O=Math.min(O,q.x),Se.info(q.x,k,E+C),C===0&&(C=R.node().getBBox().height,Se.info("Title height",C,E)),S.push(R)}let B=C*T.length;if(T.length>1){const F=(T.length-1)*C*.5;S.forEach((R,q)=>R.attr("y",E+q*C-F)),B=C*T.length}const N=y.node().getBBox();y.insert("rect",":first-child").attr("class","box").attr("x",k-L/2-ht().state.padding/2).attr("y",E-B/2-ht().state.padding/2-3.5).attr("width",L+ht().state.padding).attr("height",B+ht().state.padding),Se.info(N)}iIe++};let bp;const xce={},BJt=function(){},FJt=function(r){r.append("defs").append("marker").attr("id","dependencyEnd").attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 19,7 L9,13 L14,7 L9,1 Z")},RJt=function(r,i,o,l){bp=ht().state;const f=ht().securityLevel;let b;f==="sandbox"&&(b=Cn("#i"+i));const d=Cn(f==="sandbox"?b.nodes()[0].contentDocument.body:"body"),w=f==="sandbox"?b.nodes()[0].contentDocument:document;Se.debug("Rendering diagram "+r);const y=d.select(`[id='${i}']`);FJt(y),new zh({multigraph:!0,compound:!0,rankdir:"RL"}).setDefaultEdgeLabel(function(){return{}});const E=l.db.getRootDoc();sIe(E,y,void 0,!1,d,w,l);const T=bp.padding,C=y.node().getBBox(),S=C.width+T*2,L=C.height+T*2,O=S*1.75;Iv(y,L,O,bp.useMaxWidth),y.attr("viewBox",`${C.x-bp.padding}  ${C.y-bp.padding} `+S+" "+L)},jJt=r=>r?r.length*bp.fontSizeFactor:1,sIe=(r,i,o,l,f,b,d)=>{const w=new zh({compound:!0,multigraph:!0});let y,k=!0;for(y=0;y<r.length;y++)if(r[y].stmt==="relation"){k=!1;break}o?w.setGraph({rankdir:"LR",multigraph:!0,compound:!0,ranker:"tight-tree",ranksep:k?1:bp.edgeLengthFactor,nodeSep:k?1:50,isMultiGraph:!0}):w.setGraph({rankdir:"TB",multigraph:!0,compound:!0,ranksep:k?1:bp.edgeLengthFactor,nodeSep:k?1:50,ranker:"tight-tree",isMultiGraph:!0}),w.setDefaultEdgeLabel(function(){return{}}),d.db.extract(r);const E=d.db.getStates(),T=d.db.getRelations(),C=Object.keys(E);for(const N of C){const F=E[N];o&&(F.parentId=o);let R;if(F.doc){let q=i.append("g").attr("id",F.id).attr("class","stateGroup");R=sIe(F.doc,q,F.id,!l,f,b,d);{q=DJt(q,F,l);let X=q.node().getBBox();R.width=X.width,R.height=X.height+bp.padding/2,xce[F.id]={y:bp.compositTitleSize}}}else R=rIe(i,F);if(F.note){const q={descriptions:[],id:F.id+"-note",note:F.note,type:"note"},X=rIe(i,q);F.note.position==="left of"?(w.setNode(R.id+"-note",X),w.setNode(R.id,R)):(w.setNode(R.id,R),w.setNode(R.id+"-note",X)),w.setParent(R.id,R.id+"-group"),w.setParent(R.id+"-note",R.id+"-group")}else w.setNode(R.id,R)}Se.debug("Count=",w.nodeCount(),w);let S=0;T.forEach(function(N){S++,Se.debug("Setting edge",N),w.setEdge(N.id1,N.id2,{relation:N,width:jJt(N.title),height:bp.labelHeight*Cs.getRows(N.title).length,labelpos:"c"},"id"+S)}),bC(w),Se.debug("Graph after layout",w.nodes());const L=i.node();w.nodes().forEach(function(N){N!==void 0&&w.node(N)!==void 0?(Se.warn("Node "+N+": "+JSON.stringify(w.node(N))),f.select("#"+L.id+" #"+N).attr("transform","translate("+(w.node(N).x-w.node(N).width/2)+","+(w.node(N).y+(xce[N]?xce[N].y:0)-w.node(N).height/2)+" )"),f.select("#"+L.id+" #"+N).attr("data-x-shift",w.node(N).x-w.node(N).width/2),b.querySelectorAll("#"+L.id+" #"+N+" .divider").forEach(R=>{const q=R.parentElement;let X=0,te=0;q&&(q.parentElement&&(X=q.parentElement.getBBox().width),te=parseInt(q.getAttribute("data-x-shift"),10),Number.isNaN(te)&&(te=0)),R.setAttribute("x1",0-te+8),R.setAttribute("x2",X-te-8)})):Se.debug("No Node "+N+": "+JSON.stringify(w.node(N)))});let O=L.getBBox();w.edges().forEach(function(N){N!==void 0&&w.edge(N)!==void 0&&(Se.debug("Edge "+N.v+" -> "+N.w+": "+JSON.stringify(w.edge(N))),PJt(i,w.edge(N),w.edge(N).relation))}),O=L.getBBox();const B={id:o||"root",label:o||"root",width:0,height:0};return B.width=O.width+2*bp.padding,B.height=O.height+2*bp.padding,Se.debug("Doc rendered",B,w),B},$Jt={setConf:BJt,draw:RJt},eU="rect",Ece="rectWithTitle",HJt="start",zJt="end",GJt="divider",VJt="roundedWithTitle",UJt="note",qJt="noteGroup",CC="statediagram",YJt=`${CC}-state`,aIe="transition",WJt="note",KJt=`${aIe} note-edge`,XJt=`${CC}-${WJt}`,QJt=`${CC}-cluster`,ZJt=`${CC}-cluster-alt`,oIe="parent",cIe="note",JJt="state",_ce="----",een=`${_ce}${cIe}`,uIe=`${_ce}${oIe}`,lIe="fill:none",hIe="fill: #333",fIe="c",dIe="text",gIe="normal";let tU={},m6=0;const ten=function(r){const i=Object.keys(r);for(const o of i)r[o]},nen=function(r,i){Se.trace("Extracting classes"),i.db.clear();try{return i.parser.parse(r),i.db.extract(i.db.getRootDocV2()),i.db.getClasses()}catch(o){return o}};function ren(r){return r==null?"":r.classes?r.classes.join(" "):""}function Tce(r="",i=0,o="",l=_ce){const f=o!==null&&o.length>0?`${l}${o}`:"";return`${JJt}-${r}${f}-${i}`}const eO=(r,i,o,l,f,b)=>{const d=o.id,w=ren(l[d]);if(d!=="root"){let y=eU;o.start===!0&&(y=HJt),o.start===!1&&(y=zJt),o.type!==XI&&(y=o.type),tU[d]||(tU[d]={id:d,shape:y,description:Cs.sanitizeText(d,ht()),classes:`${w} ${YJt}`});const k=tU[d];o.description&&(Array.isArray(k.description)?(k.shape=Ece,k.description.push(o.description)):k.description.length>0?(k.shape=Ece,k.description===d?k.description=[o.description]:k.description=[k.description,o.description]):(k.shape=eU,k.description=o.description),k.description=Cs.sanitizeTextOrArray(k.description,ht())),k.description.length===1&&k.shape===Ece&&(k.shape=eU),!k.type&&o.doc&&(Se.info("Setting cluster for ",d,Cce(o)),k.type="group",k.dir=Cce(o),k.shape=o.type===GLe?GJt:VJt,k.classes=k.classes+" "+QJt+" "+(b?ZJt:""));const E={labelStyle:"",shape:k.shape,labelText:k.description,classes:k.classes,style:"",id:d,dir:k.dir,domId:Tce(d,m6),type:k.type,padding:15};if(o.note){const T={labelStyle:"",shape:UJt,labelText:o.note.text,classes:XJt,style:"",id:d+een+"-"+m6,domId:Tce(d,m6,cIe),type:k.type,padding:15},C={labelStyle:"",shape:qJt,labelText:o.note.text,classes:k.classes,style:"",id:d+uIe,domId:Tce(d,m6,oIe),type:"group",padding:0};m6++;const S=d+uIe;r.setNode(S,C),r.setNode(T.id,T),r.setNode(d,E),r.setParent(d,S),r.setParent(T.id,S);let L=d,O=T.id;o.note.position==="left of"&&(L=T.id,O=d),r.setEdge(L,O,{arrowhead:"none",arrowType:"",style:lIe,labelStyle:"",classes:KJt,arrowheadStyle:hIe,labelpos:fIe,labelType:dIe,thickness:gIe})}else r.setNode(d,E)}i&&i.id!=="root"&&(Se.trace("Setting node ",d," to be child of its parent ",i.id),r.setParent(d,i.id)),o.doc&&(Se.trace("Adding nodes children "),ien(r,o,o.doc,l,f,!b))},ien=(r,i,o,l,f,b)=>{Se.trace("items",o),o.forEach(d=>{switch(d.stmt){case KV:eO(r,i,d,l,f,b);break;case XI:eO(r,i,d,l,f,b);break;case pce:{eO(r,i,d.state1,l,f,b),eO(r,i,d.state2,l,f,b);const w={id:"edge"+m6,arrowhead:"normal",arrowTypeEnd:"arrow_barb",style:lIe,labelStyle:"",label:Cs.sanitizeText(d.description,ht()),arrowheadStyle:hIe,labelpos:fIe,labelType:dIe,thickness:gIe,classes:aIe};r.setEdge(d.state1.id,d.state2.id,w,m6),m6++}break}})},Cce=(r,i=eJt)=>{let o=i;if(r.doc)for(let l=0;l<r.doc.length;l++){const f=r.doc[l];f.stmt==="dir"&&(o=f.value)}return o},sen={setConf:ten,getClasses:nen,draw:function(r,i,o,l){Se.info("Drawing state diagram (v2)",i),tU={};let f=l.db.getDirection();f===void 0&&(f=zLe);const{securityLevel:b,state:d}=ht(),w=d.nodeSpacing||50,y=d.rankSpacing||50;Se.info(l.db.getRootDocV2()),l.db.extract(l.db.getRootDocV2()),Se.info(l.db.getRootDocV2());const k=l.db.getStates(),E=new zh({multigraph:!0,compound:!0}).setGraph({rankdir:Cce(l.db.getRootDocV2()),nodesep:w,ranksep:y,marginx:8,marginy:8}).setDefaultEdgeLabel(function(){return{}});eO(E,void 0,l.db.getRootDocV2(),k,l.db,!0);let T;b==="sandbox"&&(T=Cn("#i"+i));const C=Cn(b==="sandbox"?T.nodes()[0].contentDocument.body:"body"),S=C.select(`[id="${i}"]`),L=C.select("#"+i+" g");xoe(L,E,["barb"],CC,i);const O=8;Fs.insertTitle(S,"statediagramTitleText",d.titleTopMargin,l.db.getDiagramTitle());const B=S.node().getBBox(),N=B.width+O*2,F=B.height+O*2;S.attr("class",CC);const R=S.node().getBBox();Iv(S,F,N,d.useMaxWidth);const q=`${R.x-O} ${R.y-O} ${N} ${F}`;Se.debug(`viewBox ${q}`),S.attr("viewBox",q);const X=document.querySelectorAll('[id="'+i+'"] .edgeLabel .label');for(const te of X){const H=te.getBBox(),Y=document.createElementNS("http://www.w3.org/2000/svg",eU);Y.setAttribute("rx",0),Y.setAttribute("ry",0),Y.setAttribute("width",H.width),Y.setAttribute("height",H.height),te.insertBefore(Y,te.firstChild)}}};var Sce=function(){var r=function(O,B,N,F){for(N=N||{},F=O.length;F--;N[O[F]]=B);return N},i=[1,2],o=[1,5],l=[6,9,11,17,18,20,22,23,24,26],f=[1,15],b=[1,16],d=[1,17],w=[1,18],y=[1,19],k=[1,20],E=[1,24],T=[4,6,9,11,17,18,20,22,23,24,26],C={trace:function(){},yy:{},symbols_:{error:2,start:3,journey:4,document:5,EOF:6,directive:7,line:8,SPACE:9,statement:10,NEWLINE:11,openDirective:12,typeDirective:13,closeDirective:14,":":15,argDirective:16,title:17,acc_title:18,acc_title_value:19,acc_descr:20,acc_descr_value:21,acc_descr_multiline_value:22,section:23,taskName:24,taskData:25,open_directive:26,type_directive:27,arg_directive:28,close_directive:29,$accept:0,$end:1},terminals_:{2:"error",4:"journey",6:"EOF",9:"SPACE",11:"NEWLINE",15:":",17:"title",18:"acc_title",19:"acc_title_value",20:"acc_descr",21:"acc_descr_value",22:"acc_descr_multiline_value",23:"section",24:"taskName",25:"taskData",26:"open_directive",27:"type_directive",28:"arg_directive",29:"close_directive"},productions_:[0,[3,3],[3,2],[5,0],[5,2],[8,2],[8,1],[8,1],[8,1],[7,4],[7,6],[10,1],[10,2],[10,2],[10,1],[10,1],[10,2],[10,1],[12,1],[13,1],[16,1],[14,1]],performAction:function(B,N,F,R,q,X,te){var H=X.length-1;switch(q){case 1:return X[H-1];case 3:this.$=[];break;case 4:X[H-1].push(X[H]),this.$=X[H-1];break;case 5:case 6:this.$=X[H];break;case 7:case 8:this.$=[];break;case 11:R.setDiagramTitle(X[H].substr(6)),this.$=X[H].substr(6);break;case 12:this.$=X[H].trim(),R.setAccTitle(this.$);break;case 13:case 14:this.$=X[H].trim(),R.setAccDescription(this.$);break;case 15:R.addSection(X[H].substr(8)),this.$=X[H].substr(8);break;case 16:R.addTask(X[H-1],X[H]),this.$="task";break;case 18:R.parseDirective("%%{","open_directive");break;case 19:R.parseDirective(X[H],"type_directive");break;case 20:X[H]=X[H].trim().replace(/'/g,'"'),R.parseDirective(X[H],"arg_directive");break;case 21:R.parseDirective("}%%","close_directive","journey");break}},table:[{3:1,4:i,7:3,12:4,26:o},{1:[3]},r(l,[2,3],{5:6}),{3:7,4:i,7:3,12:4,26:o},{13:8,27:[1,9]},{27:[2,18]},{6:[1,10],7:21,8:11,9:[1,12],10:13,11:[1,14],12:4,17:f,18:b,20:d,22:w,23:y,24:k,26:o},{1:[2,2]},{14:22,15:[1,23],29:E},r([15,29],[2,19]),r(l,[2,8],{1:[2,1]}),r(l,[2,4]),{7:21,10:25,12:4,17:f,18:b,20:d,22:w,23:y,24:k,26:o},r(l,[2,6]),r(l,[2,7]),r(l,[2,11]),{19:[1,26]},{21:[1,27]},r(l,[2,14]),r(l,[2,15]),{25:[1,28]},r(l,[2,17]),{11:[1,29]},{16:30,28:[1,31]},{11:[2,21]},r(l,[2,5]),r(l,[2,12]),r(l,[2,13]),r(l,[2,16]),r(T,[2,9]),{14:32,29:E},{29:[2,20]},{11:[1,33]},r(T,[2,10])],defaultActions:{5:[2,18],7:[2,2],24:[2,21],31:[2,20]},parseError:function(B,N){if(N.recoverable)this.trace(B);else{var F=new Error(B);throw F.hash=N,F}},parse:function(B){var N=this,F=[0],R=[],q=[null],X=[],te=this.table,H="",Y=0,z=0,W=2,Z=1,G=X.slice.call(arguments,1),ae=Object.create(this.lexer),$={yy:{}};for(var ge in this.yy)Object.prototype.hasOwnProperty.call(this.yy,ge)&&($.yy[ge]=this.yy[ge]);ae.setInput(B,$.yy),$.yy.lexer=ae,$.yy.parser=this,typeof ae.yylloc>"u"&&(ae.yylloc={});var ee=ae.yylloc;X.push(ee);var de=ae.options&&ae.options.ranges;typeof $.yy.parseError=="function"?this.parseError=$.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function re(){var Oe;return Oe=R.pop()||ae.lex()||Z,typeof Oe!="number"&&(Oe instanceof Array&&(R=Oe,Oe=R.pop()),Oe=N.symbols_[Oe]||Oe),Oe}for(var ke,Ce,_e,Te,Be={},Ge,Xe,Ee,Ze;;){if(Ce=F[F.length-1],this.defaultActions[Ce]?_e=this.defaultActions[Ce]:((ke===null||typeof ke>"u")&&(ke=re()),_e=te[Ce]&&te[Ce][ke]),typeof _e>"u"||!_e.length||!_e[0]){var Ie="";Ze=[];for(Ge in te[Ce])this.terminals_[Ge]&&Ge>W&&Ze.push("'"+this.terminals_[Ge]+"'");ae.showPosition?Ie="Parse error on line "+(Y+1)+`:
`+ae.showPosition()+`
Expecting `+Ze.join(", ")+", got '"+(this.terminals_[ke]||ke)+"'":Ie="Parse error on line "+(Y+1)+": Unexpected "+(ke==Z?"end of input":"'"+(this.terminals_[ke]||ke)+"'"),this.parseError(Ie,{text:ae.match,token:this.terminals_[ke]||ke,line:ae.yylineno,loc:ee,expected:Ze})}if(_e[0]instanceof Array&&_e.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Ce+", token: "+ke);switch(_e[0]){case 1:F.push(ke),q.push(ae.yytext),X.push(ae.yylloc),F.push(_e[1]),ke=null,z=ae.yyleng,H=ae.yytext,Y=ae.yylineno,ee=ae.yylloc;break;case 2:if(Xe=this.productions_[_e[1]][1],Be.$=q[q.length-Xe],Be._$={first_line:X[X.length-(Xe||1)].first_line,last_line:X[X.length-1].last_line,first_column:X[X.length-(Xe||1)].first_column,last_column:X[X.length-1].last_column},de&&(Be._$.range=[X[X.length-(Xe||1)].range[0],X[X.length-1].range[1]]),Te=this.performAction.apply(Be,[H,z,Y,$.yy,_e[1],q,X].concat(G)),typeof Te<"u")return Te;Xe&&(F=F.slice(0,-1*Xe*2),q=q.slice(0,-1*Xe),X=X.slice(0,-1*Xe)),F.push(this.productions_[_e[1]][0]),q.push(Be.$),X.push(Be._$),Ee=te[F[F.length-2]][F[F.length-1]],F.push(Ee);break;case 3:return!0}}return!0}},S=function(){var O={EOF:1,parseError:function(N,F){if(this.yy.parser)this.yy.parser.parseError(N,F);else throw new Error(N)},setInput:function(B,N){return this.yy=N||this.yy||{},this._input=B,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var B=this._input[0];this.yytext+=B,this.yyleng++,this.offset++,this.match+=B,this.matched+=B;var N=B.match(/(?:\r\n?|\n).*/g);return N?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),B},unput:function(B){var N=B.length,F=B.split(/(?:\r\n?|\n)/g);this._input=B+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-N),this.offset-=N;var R=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),F.length-1&&(this.yylineno-=F.length-1);var q=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:F?(F.length===R.length?this.yylloc.first_column:0)+R[R.length-F.length].length-F[0].length:this.yylloc.first_column-N},this.options.ranges&&(this.yylloc.range=[q[0],q[0]+this.yyleng-N]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(B){this.unput(this.match.slice(B))},pastInput:function(){var B=this.matched.substr(0,this.matched.length-this.match.length);return(B.length>20?"...":"")+B.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var B=this.match;return B.length<20&&(B+=this._input.substr(0,20-B.length)),(B.substr(0,20)+(B.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var B=this.pastInput(),N=new Array(B.length+1).join("-");return B+this.upcomingInput()+`
`+N+"^"},test_match:function(B,N){var F,R,q;if(this.options.backtrack_lexer&&(q={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(q.yylloc.range=this.yylloc.range.slice(0))),R=B[0].match(/(?:\r\n?|\n).*/g),R&&(this.yylineno+=R.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:R?R[R.length-1].length-R[R.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+B[0].length},this.yytext+=B[0],this.match+=B[0],this.matches=B,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(B[0].length),this.matched+=B[0],F=this.performAction.call(this,this.yy,this,N,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),F)return F;if(this._backtrack){for(var X in q)this[X]=q[X];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var B,N,F,R;this._more||(this.yytext="",this.match="");for(var q=this._currentRules(),X=0;X<q.length;X++)if(F=this._input.match(this.rules[q[X]]),F&&(!N||F[0].length>N[0].length)){if(N=F,R=X,this.options.backtrack_lexer){if(B=this.test_match(F,q[X]),B!==!1)return B;if(this._backtrack){N=!1;continue}else return!1}else if(!this.options.flex)break}return N?(B=this.test_match(N,q[R]),B!==!1?B:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var N=this.next();return N||this.lex()},begin:function(N){this.conditionStack.push(N)},popState:function(){var N=this.conditionStack.length-1;return N>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(N){return N=this.conditionStack.length-1-Math.abs(N||0),N>=0?this.conditionStack[N]:"INITIAL"},pushState:function(N){this.begin(N)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(N,F,R,q){switch(R){case 0:return this.begin("open_directive"),26;case 1:return this.begin("type_directive"),27;case 2:return this.popState(),this.begin("arg_directive"),15;case 3:return this.popState(),this.popState(),29;case 4:return 28;case 5:break;case 6:break;case 7:return 11;case 8:break;case 9:break;case 10:return 4;case 11:return 17;case 12:return this.begin("acc_title"),18;case 13:return this.popState(),"acc_title_value";case 14:return this.begin("acc_descr"),20;case 15:return this.popState(),"acc_descr_value";case 16:this.begin("acc_descr_multiline");break;case 17:this.popState();break;case 18:return"acc_descr_multiline_value";case 19:return 23;case 20:return 24;case 21:return 25;case 22:return 15;case 23:return 6;case 24:return"INVALID"}},rules:[/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:%(?!\{)[^\n]*)/i,/^(?:[^\}]%%[^\n]*)/i,/^(?:[\n]+)/i,/^(?:\s+)/i,/^(?:#[^\n]*)/i,/^(?:journey\b)/i,/^(?:title\s[^#\n;]+)/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:section\s[^#:\n;]+)/i,/^(?:[^#:\n;]+)/i,/^(?::[^#\n;]+)/i,/^(?::)/i,/^(?:$)/i,/^(?:.)/i],conditions:{open_directive:{rules:[1],inclusive:!1},type_directive:{rules:[2,3],inclusive:!1},arg_directive:{rules:[3,4],inclusive:!1},acc_descr_multiline:{rules:[17,18],inclusive:!1},acc_descr:{rules:[15],inclusive:!1},acc_title:{rules:[13],inclusive:!1},INITIAL:{rules:[0,5,6,7,8,9,10,11,12,14,16,19,20,21,22,23,24],inclusive:!0}}};return O}();C.lexer=S;function L(){this.yy={}}return L.prototype=C,C.Parser=L,new L}();Sce.parser=Sce;const aen=Sce,oen=r=>r.match(/^\s*journey/)!==null;let SC="";const Ace=[],tO=[],nO=[],cen=function(r,i,o){Bu.parseDirective(this,r,i,o)},uen=function(){Ace.length=0,tO.length=0,SC="",nO.length=0,W0()},len=function(r){SC=r,Ace.push(r)},hen=function(){return Ace},fen=function(){let r=pIe();const i=100;let o=0;for(;!r&&o<i;)r=pIe(),o++;return tO.push(...nO),tO},den=function(){const r=[];return tO.forEach(o=>{o.people&&r.push(...o.people)}),[...new Set(r)].sort()},gen=function(r,i){const o=i.substr(1).split(":");let l=0,f=[];o.length===1?(l=Number(o[0]),f=[]):(l=Number(o[0]),f=o[1].split(","));const b=f.map(w=>w.trim()),d={section:SC,type:SC,people:b,task:r,score:l};nO.push(d)},pen=function(r){const i={section:SC,type:SC,description:r,task:r,classes:[]};tO.push(i)},pIe=function(){const r=function(o){return nO[o].processed};let i=!0;for(const[o,l]of nO.entries())r(o),i=i&&l.processed;return i},bIe={parseDirective:cen,getConfig:()=>ht().journey,clear:uen,setDiagramTitle:Ov,getDiagramTitle:Nv,setAccTitle:K0,getAccTitle:hp,setAccDescription:fp,getAccDescription:dp,addSection:len,getSections:hen,getTasks:fen,addTask:gen,addTaskOrg:pen,getActors:function(){return den()}},nU=function(r,i){const o=r.append("rect");return o.attr("x",i.x),o.attr("y",i.y),o.attr("fill",i.fill),o.attr("stroke",i.stroke),o.attr("width",i.width),o.attr("height",i.height),o.attr("rx",i.rx),o.attr("ry",i.ry),i.class!==void 0&&o.attr("class",i.class),o},ben=function(r,i){const l=r.append("circle").attr("cx",i.cx).attr("cy",i.cy).attr("class","face").attr("r",15).attr("stroke-width",2).attr("overflow","visible"),f=r.append("g");f.append("circle").attr("cx",i.cx-15/3).attr("cy",i.cy-15/3).attr("r",1.5).attr("stroke-width",2).attr("fill","#666").attr("stroke","#666"),f.append("circle").attr("cx",i.cx+15/3).attr("cy",i.cy-15/3).attr("r",1.5).attr("stroke-width",2).attr("fill","#666").attr("stroke","#666");function b(y){const k=ZL().startAngle(Math.PI/2).endAngle(3*(Math.PI/2)).innerRadius(7.5).outerRadius(6.8181818181818175);y.append("path").attr("class","mouth").attr("d",k).attr("transform","translate("+i.cx+","+(i.cy+2)+")")}function d(y){const k=ZL().startAngle(3*Math.PI/2).endAngle(5*(Math.PI/2)).innerRadius(7.5).outerRadius(6.8181818181818175);y.append("path").attr("class","mouth").attr("d",k).attr("transform","translate("+i.cx+","+(i.cy+7)+")")}function w(y){y.append("line").attr("class","mouth").attr("stroke",2).attr("x1",i.cx-5).attr("y1",i.cy+7).attr("x2",i.cx+5).attr("y2",i.cy+7).attr("class","mouth").attr("stroke-width","1px").attr("stroke","#666")}return i.score>3?b(f):i.score<3?d(f):w(f),l},vIe=function(r,i){const o=r.append("circle");return o.attr("cx",i.cx),o.attr("cy",i.cy),o.attr("class","actor-"+i.pos),o.attr("fill",i.fill),o.attr("stroke",i.stroke),o.attr("r",i.r),o.class!==void 0&&o.attr("class",o.class),i.title!==void 0&&o.append("title").text(i.title),o},wIe=function(r,i){const o=i.text.replace(/<br\s*\/?>/gi," "),l=r.append("text");l.attr("x",i.x),l.attr("y",i.y),l.attr("class","legend"),l.style("text-anchor",i.anchor),i.class!==void 0&&l.attr("class",i.class);const f=l.append("tspan");return f.attr("x",i.x+i.textMargin*2),f.text(o),l},ven=function(r,i){function o(f,b,d,w,y){return f+","+b+" "+(f+d)+","+b+" "+(f+d)+","+(b+w-y)+" "+(f+d-y*1.2)+","+(b+w)+" "+f+","+(b+w)}const l=r.append("polygon");l.attr("points",o(i.x,i.y,50,20,7)),l.attr("class","labelBox"),i.y=i.y+i.labelMargin,i.x=i.x+.5*i.labelMargin,wIe(r,i)},wen=function(r,i,o){const l=r.append("g"),f=Mce();f.x=i.x,f.y=i.y,f.fill=i.fill,f.width=o.width,f.height=o.height,f.class="journey-section section-type-"+i.num,f.rx=3,f.ry=3,nU(l,f),yIe(o)(i.text,l,f.x,f.y,f.width,f.height,{class:"journey-section section-type-"+i.num},o,i.colour)};let mIe=-1;const men=function(r,i,o){const l=i.x+o.width/2,f=r.append("g");mIe++;const b=300+5*30;f.append("line").attr("id","task"+mIe).attr("x1",l).attr("y1",i.y).attr("x2",l).attr("y2",b).attr("class","task-line").attr("stroke-width","1px").attr("stroke-dasharray","4 2").attr("stroke","#666"),ben(f,{cx:l,cy:300+(5-i.score)*30,score:i.score});const d=Mce();d.x=i.x,d.y=i.y,d.fill=i.fill,d.width=o.width,d.height=o.height,d.class="task task-type-"+i.num,d.rx=3,d.ry=3,nU(f,d);let w=i.x+14;i.people.forEach(y=>{const k=i.actors[y].color,E={cx:w,cy:i.y,r:7,fill:k,stroke:"#000",title:y,pos:i.actors[y].position};vIe(f,E),w+=10}),yIe(o)(i.task,f,d.x,d.y,d.width,d.height,{class:"task"},o,i.colour)},yen=function(r,i){nU(r,{x:i.startx,y:i.starty,width:i.stopx-i.startx,height:i.stopy-i.starty,fill:i.fill,class:"rect"}).lower()},ken=function(){return{x:0,y:0,fill:void 0,"text-anchor":"start",width:100,height:100,textMargin:0,rx:0,ry:0}},Mce=function(){return{x:0,y:0,width:100,anchor:"start",height:100,rx:0,ry:0}},yIe=function(){function r(f,b,d,w,y,k,E,T){const C=b.append("text").attr("x",d+y/2).attr("y",w+k/2+5).style("font-color",T).style("text-anchor","middle").text(f);l(C,E)}function i(f,b,d,w,y,k,E,T,C){const{taskFontSize:S,taskFontFamily:L}=T,O=f.split(/<br\s*\/?>/gi);for(let B=0;B<O.length;B++){const N=B*S-S*(O.length-1)/2,F=b.append("text").attr("x",d+y/2).attr("y",w).attr("fill",C).style("text-anchor","middle").style("font-size",S).style("font-family",L);F.append("tspan").attr("x",d+y/2).attr("dy",N).text(O[B]),F.attr("y",w+k/2).attr("dominant-baseline","central").attr("alignment-baseline","central"),l(F,E)}}function o(f,b,d,w,y,k,E,T){const C=b.append("switch"),L=C.append("foreignObject").attr("x",d).attr("y",w).attr("width",y).attr("height",k).attr("position","fixed").append("xhtml:div").style("display","table").style("height","100%").style("width","100%");L.append("div").attr("class","label").style("display","table-cell").style("text-align","center").style("vertical-align","middle").text(f),i(f,C,d,w,y,k,E,T),l(L,E)}function l(f,b){for(const d in b)d in b&&f.attr(d,b[d])}return function(f){return f.textPlacement==="fo"?o:f.textPlacement==="old"?r:i}}(),rO={drawRect:nU,drawCircle:vIe,drawSection:wen,drawText:wIe,drawLabel:ven,drawTask:men,drawBackgroundRect:yen,getTextObj:ken,getNoteRect:Mce,initGraphics:function(r){r.append("defs").append("marker").attr("id","arrowhead").attr("refX",5).attr("refY",2).attr("markerWidth",6).attr("markerHeight",4).attr("orient","auto").append("path").attr("d","M 0,0 V 4 L6,2 Z")}},xen=function(r){Object.keys(r).forEach(function(o){rU[o]=r[o]})},l4={};function Een(r){const i=ht().journey;let o=60;Object.keys(l4).forEach(l=>{const f=l4[l].color,b={cx:20,cy:o,r:7,fill:f,stroke:"#000",pos:l4[l].position};rO.drawCircle(r,b);const d={x:40,y:o+7,fill:"#666",text:l,textMargin:i.boxTextMargin|5};rO.drawText(r,d),o+=20})}const rU=ht().journey,p7=rU.leftMargin,_en=function(r,i,o,l){const f=ht().journey;l.db.clear(),l.parser.parse(r+`
`);const b=ht().securityLevel;let d;b==="sandbox"&&(d=Cn("#i"+i));const w=Cn(b==="sandbox"?d.nodes()[0].contentDocument.body:"body");B2.init();const y=w.select("#"+i);rO.initGraphics(y);const k=l.db.getTasks(),E=l.db.getDiagramTitle(),T=l.db.getActors();for(const N in l4)delete l4[N];let C=0;T.forEach(N=>{l4[N]={color:f.actorColours[C%f.actorColours.length],position:C},C++}),Een(y),B2.insert(0,0,p7,Object.keys(l4).length*50),Ten(y,k,0);const S=B2.getBounds();E&&y.append("text").text(E).attr("x",p7).attr("font-size","4ex").attr("font-weight","bold").attr("y",25);const L=S.stopy-S.starty+2*f.diagramMarginY,O=p7+S.stopx+2*f.diagramMarginX;Iv(y,L,O,f.useMaxWidth),y.append("line").attr("x1",p7).attr("y1",f.height*4).attr("x2",O-p7-4).attr("y2",f.height*4).attr("stroke-width",4).attr("stroke","black").attr("marker-end","url(#arrowhead)");const B=E?70:0;y.attr("viewBox",`${S.startx} -25 ${O} ${L+B}`),y.attr("preserveAspectRatio","xMinYMin meet"),y.attr("height",L+B+25)},B2={data:{startx:void 0,stopx:void 0,starty:void 0,stopy:void 0},verticalPos:0,sequenceItems:[],init:function(){this.sequenceItems=[],this.data={startx:void 0,stopx:void 0,starty:void 0,stopy:void 0},this.verticalPos=0},updateVal:function(r,i,o,l){r[i]===void 0?r[i]=o:r[i]=l(o,r[i])},updateBounds:function(r,i,o,l){const f=ht().journey,b=this;let d=0;function w(y){return function(E){d++;const T=b.sequenceItems.length-d+1;b.updateVal(E,"starty",i-T*f.boxMargin,Math.min),b.updateVal(E,"stopy",l+T*f.boxMargin,Math.max),b.updateVal(B2.data,"startx",r-T*f.boxMargin,Math.min),b.updateVal(B2.data,"stopx",o+T*f.boxMargin,Math.max),y!=="activation"&&(b.updateVal(E,"startx",r-T*f.boxMargin,Math.min),b.updateVal(E,"stopx",o+T*f.boxMargin,Math.max),b.updateVal(B2.data,"starty",i-T*f.boxMargin,Math.min),b.updateVal(B2.data,"stopy",l+T*f.boxMargin,Math.max))}}this.sequenceItems.forEach(w())},insert:function(r,i,o,l){const f=Math.min(r,o),b=Math.max(r,o),d=Math.min(i,l),w=Math.max(i,l);this.updateVal(B2.data,"startx",f,Math.min),this.updateVal(B2.data,"starty",d,Math.min),this.updateVal(B2.data,"stopx",b,Math.max),this.updateVal(B2.data,"stopy",w,Math.max),this.updateBounds(f,d,b,w)},bumpVerticalPos:function(r){this.verticalPos=this.verticalPos+r,this.data.stopy=this.verticalPos},getVerticalPos:function(){return this.verticalPos},getBounds:function(){return this.data}},Dce=rU.sectionFills,kIe=rU.sectionColours,Ten=function(r,i,o){const l=ht().journey;let f="";const b=l.height*2+l.diagramMarginY,d=o+b;let w=0,y="#CCC",k="black",E=0;for(const[T,C]of i.entries()){if(f!==C.section){y=Dce[w%Dce.length],E=w%Dce.length,k=kIe[w%kIe.length];const L={x:T*l.taskMargin+T*l.width+p7,y:50,text:C.section,fill:y,num:E,colour:k};rO.drawSection(r,L,l),f=C.section,w++}const S=C.people.reduce((L,O)=>(l4[O]&&(L[O]=l4[O]),L),{});C.x=T*l.taskMargin+T*l.width+p7,C.y=d,C.width=l.diagramMarginX,C.height=l.diagramMarginY,C.colour=k,C.fill=y,C.num=E,C.actors=S,rO.drawTask(r,C,l),B2.insert(C.x,C.y,C.x+C.width+l.taskMargin,300+5*30)}},xIe={setConf:xen,draw:_en};let EIe={};const iU={setConf:function(r){EIe={...EIe,...r}},draw:(r,i,o)=>{try{Se.debug(`Renering svg for syntax error
`);const l=Cn("#"+i),f=l.append("g");f.append("path").attr("class","error-icon").attr("d","m411.313,123.313c6.25-6.25 6.25-16.375 0-22.625s-16.375-6.25-22.625,0l-32,32-9.375,9.375-20.688-20.688c-12.484-12.5-32.766-12.5-45.25,0l-16,16c-1.261,1.261-2.304,2.648-3.31,4.051-21.739-8.561-45.324-13.426-70.065-13.426-105.867,0-192,86.133-192,192s86.133,192 192,192 192-86.133 192-192c0-24.741-4.864-48.327-13.426-70.065 1.402-1.007 2.79-2.049 4.051-3.31l16-16c12.5-12.492 12.5-32.758 0-45.25l-20.688-20.688 9.375-9.375 32.001-31.999zm-219.313,100.687c-52.938,0-96,43.063-96,96 0,8.836-7.164,16-16,16s-16-7.164-16-16c0-70.578 57.422-128 128-128 8.836,0 16,7.164 16,16s-7.164,16-16,16z"),f.append("path").attr("class","error-icon").attr("d","m459.02,148.98c-6.25-6.25-16.375-6.25-22.625,0s-6.25,16.375 0,22.625l16,16c3.125,3.125 7.219,4.688 11.313,4.688 4.094,0 8.188-1.563 11.313-4.688 6.25-6.25 6.25-16.375 0-22.625l-16.001-16z"),f.append("path").attr("class","error-icon").attr("d","m340.395,75.605c3.125,3.125 7.219,4.688 11.313,4.688 4.094,0 8.188-1.563 11.313-4.688 6.25-6.25 6.25-16.375 0-22.625l-16-16c-6.25-6.25-16.375-6.25-22.625,0s-6.25,16.375 0,22.625l15.999,16z"),f.append("path").attr("class","error-icon").attr("d","m400,64c8.844,0 16-7.164 16-16v-32c0-8.836-7.156-16-16-16-8.844,0-16,7.164-16,16v32c0,8.836 7.156,16 16,16z"),f.append("path").attr("class","error-icon").attr("d","m496,96.586h-32c-8.844,0-16,7.164-16,16 0,8.836 7.156,16 16,16h32c8.844,0 16-7.164 16-16 0-8.836-7.156-16-16-16z"),f.append("path").attr("class","error-icon").attr("d","m436.98,75.605c3.125,3.125 7.219,4.688 11.313,4.688 4.094,0 8.188-1.563 11.313-4.688l32-32c6.25-6.25 6.25-16.375 0-22.625s-16.375-6.25-22.625,0l-32,32c-6.251,6.25-6.251,16.375-0.001,22.625z"),f.append("text").attr("class","error-text").attr("x",1440).attr("y",250).attr("font-size","150px").style("text-anchor","middle").text("Syntax error in graph"),f.append("text").attr("class","error-text").attr("x",1250).attr("y",400).attr("font-size","100px").style("text-anchor","middle").text("mermaid version "+o),l.attr("height",100),l.attr("width",500),l.attr("viewBox","768 0 912 512")}catch(l){Se.error("Error while rendering info diagram"),Se.error(cjt(l))}}},_Ie="flowchart-elk",Cen={id:_Ie,detector:(r,i)=>{var o;return!!(r.match(/^\s*flowchart-elk/)||r.match(/^\s*flowchart|graph/)&&((o=i==null?void 0:i.flowchart)==null?void 0:o.defaultRenderer)==="elk")},loader:async()=>{const{diagram:r}=await Promise.resolve().then(()=>gtn);return{id:_Ie,diagram:r}}},TIe="timeline",Sen={id:TIe,detector:r=>r.match(/^\s*timeline/)!==null,loader:async()=>{const{diagram:r}=await Promise.resolve().then(()=>Ntn);return{id:TIe,diagram:r}}},CIe="mindmap",Aen={id:CIe,detector:r=>r.match(/^\s*mindmap/)!==null,loader:async()=>{const{diagram:r}=await Promise.resolve().then(()=>Ahn);return{id:CIe,diagram:r}}};let SIe=!1;const iO=()=>{SIe||(SIe=!0,BCe(Cen,Sen,Aen),uf("error",{db:{clear:()=>{}},styles:Tae,renderer:iU,parser:{parser:{yy:{}},parse:()=>{}},init:()=>{}},r=>r.toLowerCase().trim()==="error"),uf("---",{db:{clear:()=>{}},styles:Tae,renderer:iU,parser:{parser:{yy:{}},parse:()=>{throw new Error("Diagrams beginning with --- are not valid. If you were trying to use a YAML front-matter, please ensure that you've correctly opened and closed the YAML front-matter with unindented `---` blocks")}},init:()=>null},r=>r.toLowerCase().trimStart().startsWith("---")),uf("c4",{parser:p$t,db:ASe,renderer:FSe,styles:pSe,init:r=>{FSe.setConf(r.c4)}},b$t),uf("class",{parser:RSe,db:iC,renderer:FWt,styles:hI,init:r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute,iC.clear()}},nHt),uf("classDiagram",{parser:RSe,db:iC,renderer:LKt,styles:hI,init:r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute,iC.clear()}},rHt),uf("er",{parser:IKt,db:FKt,renderer:sXt,styles:cSe},OKt),uf("gantt",{parser:eQt,db:Zoe,renderer:IQt,styles:uSe},tQt),uf("info",{parser:OQt,db:NQt,renderer:PQt,styles:lSe},BQt),uf("pie",{parser:FQt,db:jQt,renderer:$Qt,styles:hSe},RQt),uf("requirement",{parser:HQt,db:GQt,renderer:XQt,styles:fSe},zQt),uf("sequence",{parser:QQt,db:MLe,renderer:$Le,styles:dSe,init:r=>{if(r.sequence||(r.sequence={}),r.sequence.arrowMarkerAbsolute=r.arrowMarkerAbsolute,"sequenceDiagram"in r)throw new Error("`mermaid config.sequenceDiagram` has been renamed to `config.sequence`. Please update your mermaid config.");MLe.setWrap(r.wrap),$Le.setConf(r.sequence)}},ZQt),uf("state",{parser:HLe,db:u4,renderer:$Jt,styles:YG,init:r=>{r.state||(r.state={}),r.state.arrowMarkerAbsolute=r.arrowMarkerAbsolute,u4.clear()}},ZZt),uf("stateDiagram",{parser:HLe,db:u4,renderer:sen,styles:YG,init:r=>{r.state||(r.state={}),r.state.arrowMarkerAbsolute=r.arrowMarkerAbsolute,u4.clear()}},JZt),uf("journey",{parser:aen,db:bIe,renderer:xIe,styles:gSe,init:r=>{xIe.setConf(r.journey),bIe.clear()}},oen),uf("flowchart",{parser:Soe,db:$m,renderer:$oe,styles:qG,init:r=>{r.flowchart||(r.flowchart={}),r.flowchart.arrowMarkerAbsolute=r.arrowMarkerAbsolute,ZXt.setConf(r.flowchart),$m.clear(),$m.setGen("gen-1")}},aXt),uf("flowchart-v2",{parser:Soe,db:$m,renderer:$oe,styles:qG,init:r=>{r.flowchart||(r.flowchart={}),r.flowchart.arrowMarkerAbsolute=r.arrowMarkerAbsolute,rSe({flowchart:{arrowMarkerAbsolute:r.arrowMarkerAbsolute}}),$oe.setConf(r.flowchart),$m.clear(),$m.setGen("gen-2")}},oXt),uf("gitGraph",{parser:$jt,db:a$t,renderer:d$t,styles:g$t},Hjt))};class Lce{constructor(i,o){CL(this,"type","graph");CL(this,"parser");CL(this,"renderer");CL(this,"db");CL(this,"detectTypeFailed",!1);var d,w;this.txt=i;const l=ht();this.txt=i;try{this.type=lae(i,l)}catch(y){this.handleError(y,o),this.type="error",this.detectTypeFailed=!0}const f=wSe(this.type);Se.debug("Type "+this.type),this.db=f.db,(w=(d=this.db).clear)==null||w.call(d),this.renderer=f.renderer,this.parser=f.parser;const b=this.parser.parse.bind(this.parser);this.parser.parse=y=>b($Ft(y,this.db)),this.parser.parser.yy=this.db,f.init&&(f.init(l),Se.info("Initialized diagram "+this.type,l)),this.txt+=`
`,this.parse(this.txt,o)}parse(i,o){var l,f;if(this.detectTypeFailed)return!1;try{return i=i+`
`,(f=(l=this.db).clear)==null||f.call(l),this.parser.parse(i),!0}catch(b){this.handleError(b,o)}return!1}handleError(i,o){if(o===void 0)throw i;if(bae(i)){o(i.str,i.hash);return}o(i)}getParser(){return this.parser}getType(){return this.type}}const Ice=(r,i)=>{const o=lae(r,ht());try{wSe(o)}catch{const f=GFt(o);if(!f)throw new Error(`Diagram ${o} not found.`);return f().then(({diagram:b})=>(uf(o,b,void 0),new Lce(r,i)))}return new Lce(r,i)},Oce=Lce,Men="graphics-document document";function Den(r,i){r.attr("role",Men),gC(i)||r.attr("aria-roledescription",i)}function Len(r,i,o,l){if(r.insert!==void 0)if(i||o){if(o){const f="chart-desc-"+l;r.attr("aria-describedby",f),r.insert("desc",":first-child").attr("id",f).text(o)}if(i){const f="chart-title-"+l;r.attr("aria-labelledby",f),r.insert("title",":first-child").attr("id",f).text(i)}}else return}const Nce=["graph","flowchart","flowchart-v2","stateDiagram","stateDiagram-v2"],AIe="graph TB;a[Maximum text size in diagram exceeded];style a fill:#faa",MIe="sandbox",DIe="loose",Ien="http://www.w3.org/2000/svg",LIe="http://www.w3.org/1999/xlink",IIe="http://www.w3.org/1999/xhtml",Oen="100%",Nen="100%",Pen="border:0;margin:0;",Ben="margin:0",Fen="allow-top-navigation-by-user-activation allow-popups",Ren='The "iframe" tag is not supported by your browser.',OIe=["foreignobject"],NIe=["dominant-baseline"];function jen(r,i){return iO(),new Oce(r,i).parse(r,i)}async function $en(r,i){return iO(),(await Ice(r,i)).parse(r,i)}const PIe=function(r){let i=r;return i=i.replace(/style.*:\S*#.*;/g,function(o){return o.substring(0,o.length-1)}),i=i.replace(/classDef.*:\S*#.*;/g,function(o){return o.substring(0,o.length-1)}),i=i.replace(/#\w+;/g,function(o){const l=o.substring(1,o.length-1);return/^\+?\d+$/.test(l)?"\uFB02\xB0\xB0"+l+"\xB6\xDF":"\uFB02\xB0"+l+"\xB6\xDF"}),i},Pce=function(r){let i=r;return i=i.replace(/ﬂ°°/g,"&#"),i=i.replace(/ﬂ°/g,"&"),i=i.replace(/¶ß/g,";"),i},BIe=(r,i,o=[])=>`
.${r} ${i} { ${o.join(" !important; ")} !important; }`,Hen=(r,i,o={})=>{var f;let l="";if(r.themeCSS!==void 0&&(l+=`
${r.themeCSS}`),r.fontFamily!==void 0&&(l+=`
:root { --mermaid-font-family: ${r.fontFamily}}`),r.altFontFamily!==void 0&&(l+=`
:root { --mermaid-alt-font-family: ${r.altFontFamily}}`),!gC(o)&&Nce.includes(i)){const y=r.htmlLabels||((f=r.flowchart)==null?void 0:f.htmlLabels)?["> *","span"]:["rect","polygon","ellipse","circle","path"];for(const k in o){const E=o[k];gC(E.styles)||y.forEach(T=>{l+=BIe(E.id,T,E.styles)}),gC(E.textStyles)||(l+=BIe(E.id,"tspan",E.textStyles))}}return l},FIe=(r,i,o,l)=>{const f=Hen(r,i,o),b=Ojt(i,f,r.themeVariables);return Eae(xjt(`${l}{${b}}`),_jt)},RIe=(r="",i,o)=>{let l=r;return!o&&!i&&(l=l.replace(/marker-end="url\(.*?#/g,'marker-end="url(#')),l=Pce(l),l=l.replace(/<br>/g,"<br/>"),l},jIe=(r="",i)=>{const o=i?i.viewBox.baseVal.height+"px":Nen,l=btoa('<body style="'+Ben+'">'+r+"</body>");return`<iframe style="width:${Oen};height:${o};${Pen}" src="data:text/html;base64,${l}" sandbox="${Fen}">
  ${Ren}
</iframe>`},sU=(r,i,o,l,f)=>{const b=r.append("div");b.attr("id",o),l&&b.attr("style",l);const d=b.append("svg").attr("id",i).attr("width","100%").attr("xmlns",Ien);return f&&d.attr("xmlns:xlink",f),d.append("g"),r};function aU(r,i){return r.append("iframe").attr("id",i).attr("style","width: 100%; height: 100%;").attr("sandbox","")}const $Ie=(r,i,o,l)=>{var f,b,d;(f=r.getElementById(i))==null||f.remove(),(b=r.getElementById(o))==null||b.remove(),(d=r.getElementById(l))==null||d.remove()},zen=function(r,i,o,l){var $,ge,ee,de,re;iO(),lI();const f=Fs.detectInit(i);f&&(Q8(f),_ae(f));const b=ht();Se.debug(b),i.length>(($=b==null?void 0:b.maxTextSize)!=null?$:5e4)&&(i=AIe),i=i.replace(/\r\n?/g,`
`);const d="#"+r,w="i"+r,y="#"+w,k="d"+r,E="#"+k;let T=Cn("body");const C=b.securityLevel===MIe,S=b.securityLevel===DIe,L=b.fontFamily;if(l!==void 0){if(l&&(l.innerHTML=""),C){const ke=aU(Cn(l),w);T=Cn(ke.nodes()[0].contentDocument.body),T.node().style.margin=0}else T=Cn(l);sU(T,r,k,`font-family: ${L}`,LIe)}else{if($Ie(document,r,k,w),C){const ke=aU(Cn("body"),w);T=Cn(ke.nodes()[0].contentDocument.body),T.node().style.margin=0}else T=Cn("body");sU(T,r,k)}i=PIe(i);let O,B;try{if(O=Ice(i),"then"in O)throw new Error("Diagram is a promise. Use renderAsync.")}catch(ke){O=new Oce("error"),B=ke}const N=T.select(E).node(),F=O.type,R=N.firstChild,q=R.firstChild,X=Nce.includes(F)?O.renderer.getClasses(i,O):{},te=FIe(b,F,X,d),H=document.createElement("style");H.innerHTML=te,R.insertBefore(H,q);try{O.renderer.draw(i,r,VG.version,O)}catch(ke){throw iU.draw(i,r,VG.version),ke}const Y=T.select(`${E} svg`),z=(ee=(ge=O.db).getAccTitle)==null?void 0:ee.call(ge),W=(re=(de=O.db).getAccDescription)==null?void 0:re.call(de);HIe(F,Y,z,W),T.select(`[id="${r}"]`).selectAll("foreignobject > *").attr("xmlns",IIe);let Z=T.select(E).node().innerHTML;if(Se.debug("config.arrowMarkerAbsolute",b.arrowMarkerAbsolute),Z=RIe(Z,C,cf(b.arrowMarkerAbsolute)),C){const ke=T.select(E+" svg").node();Z=jIe(Z,ke)}else S||(Z=eI.sanitize(Z,{ADD_TAGS:OIe,ADD_ATTR:NIe}));if(o!==void 0)switch(F){case"flowchart":case"flowchart-v2":o(Z,$m.bindFunctions);break;case"gantt":o(Z,Zoe.bindFunctions);break;case"class":case"classDiagram":o(Z,iC.bindFunctions);break;default:o(Z)}else Se.debug("CB = undefined!");DLe();const ae=Cn(C?y:E).node();if(ae&&"remove"in ae&&ae.remove(),B)throw B;return Z},Gen=async function(r,i,o,l){var $,ge,ee,de,re;iO(),lI();const f=Fs.detectInit(i);f&&(Q8(f),_ae(f));const b=ht();Se.debug(b),i.length>(($=b==null?void 0:b.maxTextSize)!=null?$:5e4)&&(i=AIe),i=i.replace(/\r\n?/g,`
`);const d="#"+r,w="i"+r,y="#"+w,k="d"+r,E="#"+k;let T=Cn("body");const C=b.securityLevel===MIe,S=b.securityLevel===DIe,L=b.fontFamily;if(l!==void 0){if(l&&(l.innerHTML=""),C){const ke=aU(Cn(l),w);T=Cn(ke.nodes()[0].contentDocument.body),T.node().style.margin=0}else T=Cn(l);sU(T,r,k,`font-family: ${L}`,LIe)}else{if($Ie(document,r,k,w),C){const ke=aU(Cn("body"),w);T=Cn(ke.nodes()[0].contentDocument.body),T.node().style.margin=0}else T=Cn("body");sU(T,r,k)}i=PIe(i);let O,B;try{O=await Ice(i)}catch(ke){O=new Oce("error"),B=ke}const N=T.select(E).node(),F=O.type,R=N.firstChild,q=R.firstChild,X=Nce.includes(F)?O.renderer.getClasses(i,O):{},te=FIe(b,F,X,d),H=document.createElement("style");H.innerHTML=te,R.insertBefore(H,q);try{await O.renderer.draw(i,r,VG.version,O)}catch(ke){throw iU.draw(i,r,VG.version),ke}const Y=T.select(`${E} svg`),z=(ee=(ge=O.db).getAccTitle)==null?void 0:ee.call(ge),W=(re=(de=O.db).getAccDescription)==null?void 0:re.call(de);HIe(F,Y,z,W),T.select(`[id="${r}"]`).selectAll("foreignobject > *").attr("xmlns",IIe);let Z=T.select(E).node().innerHTML;if(Se.debug("config.arrowMarkerAbsolute",b.arrowMarkerAbsolute),Z=RIe(Z,C,cf(b.arrowMarkerAbsolute)),C){const ke=T.select(E+" svg").node();Z=jIe(Z,ke)}else S||(Z=eI.sanitize(Z,{ADD_TAGS:OIe,ADD_ATTR:NIe}));if(o!==void 0)switch(F){case"flowchart":case"flowchart-v2":o(Z,$m.bindFunctions);break;case"gantt":o(Z,Zoe.bindFunctions);break;case"class":case"classDiagram":o(Z,iC.bindFunctions);break;default:o(Z)}else Se.debug("CB = undefined!");DLe();const ae=Cn(C?y:E).node();if(ae&&"remove"in ae&&ae.remove(),B)throw B;return Z};function Ven(r={}){var o;(r==null?void 0:r.fontFamily)&&!((o=r.themeVariables)!=null&&o.fontFamily)&&(r.themeVariables={fontFamily:r.fontFamily}),Cjt(r),(r==null?void 0:r.theme)&&r.theme in q3?r.themeVariables=q3[r.theme].getThemeVariables(r.themeVariables):r&&(r.themeVariables=q3.default.getThemeVariables(r.themeVariables));const i=typeof r=="object"?Tjt(r):nSe();Wie(i.logLevel),iO()}function HIe(r,i,o,l){Den(i,r),Len(i,o,l,i.attr("id"))}const Bu=Object.freeze({render:zen,renderAsync:Gen,parse:jen,parseAsync:$en,parseDirective:Mae,initialize:Ven,getConfig:ht,setConfig:rSe,getSiteConfig:nSe,updateSiteConfig:Sjt,reset:()=>{lI()},globalReset:()=>{lI(nC)},defaultConfig:nC});Wie(ht().logLevel),lI(ht());const Uen=async function(r,i,o){try{await GIe(r,i,o)}catch(l){Se.warn("Syntax Error rendering"),bae(l)&&Se.warn(l.str),F2.parseError&&F2.parseError(l)}},zIe=(r,i,o)=>{Se.warn(r),bae(r)?(o&&o(r.str,r.hash),i.push({...r,message:r.str,error:r})):(o&&o(r),r instanceof Error&&i.push({str:r.message,message:r.message,hash:r.name,error:r}))},qen=function(r,i,o){const l=Bu.getConfig();r&&(F2.sequenceConfig=r),Se.debug(`${o?"":"No "}Callback function found`);let f;if(i===void 0)f=document.querySelectorAll(".mermaid");else if(typeof i=="string")f=document.querySelectorAll(i);else if(i instanceof HTMLElement)f=[i];else if(i instanceof NodeList)f=i;else throw new Error("Invalid argument nodes for mermaid.init");Se.debug(`Found ${f.length} diagrams`),(r==null?void 0:r.startOnLoad)!==void 0&&(Se.debug("Start On Load: "+(r==null?void 0:r.startOnLoad)),Bu.updateSiteConfig({startOnLoad:r==null?void 0:r.startOnLoad}));const b=new Fs.initIdGenerator(l.deterministicIds,l.deterministicIDSeed);let d;const w=[];for(const y of Array.from(f)){Se.info("Rendering diagram: "+y.id);/*! Check if previously processed */if(y.getAttribute("data-processed"))continue;y.setAttribute("data-processed","true");const k=`mermaid-${b.next()}`;d=y.innerHTML,d=T2(Fs.entityDecode(d)).trim().replace(/<br\s*\/?>/gi,"<br/>");const E=Fs.detectInit(d);E&&Se.debug("Detected early reinit: ",E);try{Bu.render(k,d,(T,C)=>{y.innerHTML=T,o!==void 0&&o(k),C&&C(y)},y)}catch(T){zIe(T,w,F2.parseError)}}if(w.length>0)throw w[0]},Yen=async(...r)=>{Se.debug(`Loading ${r.length} external diagrams`);const o=(await Promise.allSettled(r.map(async({id:l,detector:f,loader:b})=>{const{diagram:d}=await b();uf(l,d,f)}))).filter(l=>l.status==="rejected");if(o.length>0){Se.error(`Failed to load ${o.length} external diagrams`);for(const l of o)Se.error(l);throw new Error(`Failed to load ${o.length} external diagrams`)}},GIe=async function(r,i,o){const l=Bu.getConfig();r&&(F2.sequenceConfig=r),Se.debug(`${o?"":"No "}Callback function found`);let f;if(i===void 0)f=document.querySelectorAll(".mermaid");else if(typeof i=="string")f=document.querySelectorAll(i);else if(i instanceof HTMLElement)f=[i];else if(i instanceof NodeList)f=i;else throw new Error("Invalid argument nodes for mermaid.init");Se.debug(`Found ${f.length} diagrams`),(r==null?void 0:r.startOnLoad)!==void 0&&(Se.debug("Start On Load: "+(r==null?void 0:r.startOnLoad)),Bu.updateSiteConfig({startOnLoad:r==null?void 0:r.startOnLoad}));const b=new Fs.initIdGenerator(l.deterministicIds,l.deterministicIDSeed);let d;const w=[];for(const y of Array.from(f)){Se.info("Rendering diagram: "+y.id);/*! Check if previously processed */if(y.getAttribute("data-processed"))continue;y.setAttribute("data-processed","true");const k=`mermaid-${b.next()}`;d=y.innerHTML,d=T2(Fs.entityDecode(d)).trim().replace(/<br\s*\/?>/gi,"<br/>");const E=Fs.detectInit(d);E&&Se.debug("Detected early reinit: ",E);try{await Bu.renderAsync(k,d,(T,C)=>{y.innerHTML=T,o!==void 0&&o(k),C&&C(y)},y)}catch(T){zIe(T,w,F2.parseError)}}if(w.length>0)throw w[0]},Wen=function(r){Bu.initialize(r)},Ken=async(r,{lazyLoad:i=!0}={})=>{i?BCe(...r):await Yen(...r)},VIe=function(){if(F2.startOnLoad){const{startOnLoad:r}=Bu.getConfig();r&&F2.init().catch(i=>Se.error("Mermaid failed to initialize",i))}};if(typeof document<"u"){/*!
 * Wait for document loaded before starting the execution
 */window.addEventListener("load",VIe,!1)}const Xen=function(r){F2.parseError=r},Qen=r=>Bu.parse(r,F2.parseError),oU=[];let Bce=!1;const UIe=async()=>{if(!Bce){for(Bce=!0;oU.length>0;){const r=oU.shift();if(r)try{await r()}catch(i){Se.error("Error executing queue",i)}}Bce=!1}},Zen=r=>new Promise((i,o)=>{const l=()=>new Promise((f,b)=>{Bu.parseAsync(r,F2.parseError).then(d=>{f(d),i(d)},d=>{Se.error("Error parsing",d),b(d),o(d)})});oU.push(l),UIe().catch(o)}),Jen=(r,i,o,l)=>new Promise((f,b)=>{const d=()=>new Promise((w,y)=>{Bu.renderAsync(r,i,o,l).then(k=>{w(k),f(k)},k=>{Se.error("Error parsing",k),y(k),b(k)})});oU.push(d),UIe().catch(b)}),F2={startOnLoad:!0,diagrams:{},mermaidAPI:Bu,parse:Qen,parseAsync:Zen,render:Bu.render,renderAsync:Jen,init:Uen,initThrowsErrors:qen,initThrowsErrorsAsync:GIe,registerExternalDiagrams:Ken,initialize:Wen,parseError:void 0,contentLoaded:VIe,setParseErrorHandler:Xen},etn=(r,i,o)=>{const{parentById:l}=o,f=new Set;let b=r;for(;b;){if(f.add(b),b===i)return b;b=l[b]}for(b=i;b;){if(f.has(b))return b;b=l[b]}return"root"};function cU(r){throw new Error('Could not dynamically require "'+r+'". Please configure the dynamicRequireTargets or/and ignoreDynamicRequires option of @rollup/plugin-commonjs appropriately for this require call to work.')}var qIe={exports:{}};(function(r,i){(function(o){r.exports=o()})(function(){return function(){function o(l,f,b){function d(k,E){if(!f[k]){if(!l[k]){var T=typeof cU=="function"&&cU;if(!E&&T)return T(k,!0);if(w)return w(k,!0);var C=new Error("Cannot find module '"+k+"'");throw C.code="MODULE_NOT_FOUND",C}var S=f[k]={exports:{}};l[k][0].call(S.exports,function(L){var O=l[k][1][L];return d(O||L)},S,S.exports,o,l,f,b)}return f[k].exports}for(var w=typeof cU=="function"&&cU,y=0;y<b.length;y++)d(b[y]);return d}return o}()({1:[function(o,l,f){Object.defineProperty(f,"__esModule",{value:!0});var b=function(){function k(E,T){for(var C=0;C<T.length;C++){var S=T[C];S.enumerable=S.enumerable||!1,S.configurable=!0,"value"in S&&(S.writable=!0),Object.defineProperty(E,S.key,S)}}return function(E,T,C){return T&&k(E.prototype,T),C&&k(E,C),E}}();function d(k,E){if(!(k instanceof E))throw new TypeError("Cannot call a class as a function")}var w=function(){function k(){var E=this,T=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},C=T.defaultLayoutOptions,S=C===void 0?{}:C,L=T.algorithms,O=L===void 0?["layered","stress","mrtree","radial","force","disco","sporeOverlap","sporeCompaction","rectpacking"]:L,B=T.workerFactory,N=T.workerUrl;if(d(this,k),this.defaultLayoutOptions=S,this.initialized=!1,typeof N>"u"&&typeof B>"u")throw new Error("Cannot construct an ELK without both 'workerUrl' and 'workerFactory'.");var F=B;typeof N<"u"&&typeof B>"u"&&(F=function(X){return new Worker(X)});var R=F(N);if(typeof R.postMessage!="function")throw new TypeError("Created worker does not provide the required 'postMessage' function.");this.worker=new y(R),this.worker.postMessage({cmd:"register",algorithms:O}).then(function(q){return E.initialized=!0}).catch(console.err)}return b(k,[{key:"layout",value:function(T){var C=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},S=C.layoutOptions,L=S===void 0?this.defaultLayoutOptions:S,O=C.logging,B=O===void 0?!1:O,N=C.measureExecutionTime,F=N===void 0?!1:N;return T?this.worker.postMessage({cmd:"layout",graph:T,layoutOptions:L,options:{logging:B,measureExecutionTime:F}}):Promise.reject(new Error("Missing mandatory parameter 'graph'."))}},{key:"knownLayoutAlgorithms",value:function(){return this.worker.postMessage({cmd:"algorithms"})}},{key:"knownLayoutOptions",value:function(){return this.worker.postMessage({cmd:"options"})}},{key:"knownLayoutCategories",value:function(){return this.worker.postMessage({cmd:"categories"})}},{key:"terminateWorker",value:function(){this.worker.terminate()}}]),k}();f.default=w;var y=function(){function k(E){var T=this;if(d(this,k),E===void 0)throw new Error("Missing mandatory parameter 'worker'.");this.resolvers={},this.worker=E,this.worker.onmessage=function(C){setTimeout(function(){T.receive(T,C)},0)}}return b(k,[{key:"postMessage",value:function(T){var C=this.id||0;this.id=C+1,T.id=C;var S=this;return new Promise(function(L,O){S.resolvers[C]=function(B,N){B?(S.convertGwtStyleError(B),O(B)):L(N)},S.worker.postMessage(T)})}},{key:"receive",value:function(T,C){var S=C.data,L=T.resolvers[S.id];L&&(delete T.resolvers[S.id],S.error?L(S.error):L(null,S.data))}},{key:"terminate",value:function(){this.worker.terminate&&this.worker.terminate()}},{key:"convertGwtStyleError",value:function(T){if(!!T){var C=T.__java$exception;C&&(C.cause&&C.cause.backingJsObject&&(T.cause=C.cause.backingJsObject,this.convertGwtStyleError(T.cause)),delete T.__java$exception)}}}]),k}()},{}],2:[function(o,l,f){(function(b){(function(){var d;typeof window<"u"?d=window:typeof b<"u"?d=b:typeof self<"u"&&(d=self);var w;function y(){}function k(){}function E(){}function T(){}function C(){}function S(){}function L(){}function O(){}function B(){}function N(){}function F(){}function R(){}function q(){}function X(){}function te(){}function H(){}function Y(){}function z(){}function W(){}function Z(){}function G(){}function ae(){}function $(){}function ge(){}function ee(){}function de(){}function re(){}function ke(){}function Ce(){}function _e(){}function Te(){}function Be(){}function Ge(){}function Xe(){}function Ee(){}function Ze(){}function Ie(){}function Oe(){}function Le(){}function $e(){}function xe(){}function Ae(){}function je(){}function me(){}function vt(){}function ve(){}function Zt(){}function nt(){}function xn(){}function cn(){}function jt(){}function ot(){}function be(){}function We(){}function ct(){}function Yt(){}function Ut(){}function Wn(){}function Gt(){}function Rn(){}function si(){}function $r(){}function nr(){}function Kn(){}function Jt(){}function en(){}function In(){}function hn(){}function Fr(){}function Pt(){}function ei(){}function nn(){}function hi(){}function Hi(){}function ss(){}function ls(){}function vs(){}function ti(){}function zi(){}function as(){}function ai(){}function hc(){}function xu(){}function No(){}function Si(){}function Yc(){}function lh(){}function su(){}function Ru(){}function fc(){}function Ll(){}function ol(){}function Ur(){}function bt(){}function $n(){}function Er(){}function Ss(){}function wo(){}function Po(){}function Ra(){}function Eu(){}function Lc(){}function bd(){}function hh(){}function tg(){}function Di(){}function Bi(){}function ra(){}function Rr(){}function au(){}function gt(){}function O6(){}function It(){}function n0(){}function jO(){}function rq(){}function tS(){}function iq(){}function fh(){}function Qm(){}function nS(){}function rS(){}function $O(){}function sq(){}function D1(){}function Zm(){}function r0(){}function iS(){}function w4(){}function M7(){}function H2(){}function HO(){}function D7(){}function sS(){}function z2(){}function _Be(){}function TBe(){}function aq(){}function CBe(){}function SBe(){}function ABe(){}function MBe(){}function DBe(){}function $ue(){}function LBe(){}function IBe(){}function OBe(){}function NBe(){}function PBe(){}function BBe(){}function FBe(){}function RBe(){}function jBe(){}function $Be(){}function HBe(){}function zBe(){}function GBe(){}function VBe(){}function UBe(){}function qBe(){}function YBe(){}function WBe(){}function KBe(){}function XBe(){}function QBe(){}function ZBe(){}function JBe(){}function eFe(){}function tFe(){}function nFe(){}function rFe(){}function iFe(){}function sFe(){}function aFe(){}function oFe(){}function cFe(){}function uFe(){}function lFe(){}function hFe(){}function fFe(){}function dFe(){}function gFe(){}function pFe(){}function bFe(){}function vFe(){}function wFe(){}function mFe(){}function yFe(){}function kFe(){}function xFe(){}function EFe(){}function _Fe(){}function TFe(){}function CFe(){}function SFe(){}function AFe(){}function MFe(){}function DFe(){}function LFe(){}function IFe(){}function OFe(){}function NFe(){}function PFe(){}function BFe(){}function FFe(){}function RFe(){}function jFe(){}function $Fe(){}function HFe(){}function zFe(){}function GFe(){}function VFe(){}function UFe(){}function qFe(){}function YFe(){}function WFe(){}function KFe(){}function XFe(){}function QFe(){}function ZFe(){}function JFe(){}function eRe(){}function tRe(){}function nRe(){}function rRe(){}function iRe(){}function sRe(){}function aRe(){}function oRe(){}function cRe(){}function uRe(){}function lRe(){}function hRe(){}function fRe(){}function dRe(){}function gRe(){}function pRe(){}function bRe(){}function vRe(){}function wRe(){}function mRe(){}function yRe(){}function kRe(){}function xRe(){}function Hue(){}function ERe(){}function _Re(){}function TRe(){}function CRe(){}function SRe(){}function ARe(){}function MRe(){}function DRe(){}function LRe(){}function IRe(){}function ORe(){}function NRe(){}function PRe(){}function BRe(){}function FRe(){}function RRe(){}function jRe(){}function $Re(){}function HRe(){}function zRe(){}function GRe(){}function VRe(){}function URe(){}function qRe(){}function YRe(){}function WRe(){}function KRe(){}function XRe(){}function QRe(){}function ZRe(){}function JRe(){}function eje(){}function tje(){}function nje(){}function rje(){}function ije(){}function sje(){}function aje(){}function oje(){}function cje(){}function uje(){}function lje(){}function hje(){}function fje(){}function dje(){}function gje(){}function pje(){}function bje(){}function vje(){}function wje(){}function mje(){}function yje(){}function kje(){}function xje(){}function Eje(){}function _je(){}function Tje(){}function Cje(){}function Sje(){}function Aje(){}function Mje(){}function Dje(){}function Lje(){}function Ije(){}function Oje(){}function Nje(){}function Pje(){}function zue(){}function Bje(){}function Fje(){}function Rje(){}function jje(){}function $je(){}function Hje(){}function zje(){}function Gje(){}function Vje(){}function Uje(){}function qje(){}function Yje(){}function Wje(){}function Kje(){}function Xje(){}function Qje(){}function Zje(){}function Jje(){}function e$e(){}function t$e(){}function Gue(){}function n$e(){}function r$e(){}function i$e(){}function s$e(){}function a$e(){}function o$e(){}function Vue(){}function Uue(){}function c$e(){}function que(){}function Yue(){}function u$e(){}function l$e(){}function h$e(){}function f$e(){}function d$e(){}function g$e(){}function p$e(){}function b$e(){}function v$e(){}function Wue(){}function w$e(){}function m$e(){}function y$e(){}function k$e(){}function x$e(){}function E$e(){}function _$e(){}function T$e(){}function C$e(){}function S$e(){}function A$e(){}function M$e(){}function D$e(){}function L$e(){}function I$e(){}function O$e(){}function N$e(){}function P$e(){}function B$e(){}function F$e(){}function R$e(){}function j$e(){}function $$e(){}function H$e(){}function z$e(){}function G$e(){}function V$e(){}function U$e(){}function q$e(){}function Y$e(){}function W$e(){}function K$e(){}function X$e(){}function Q$e(){}function Z$e(){}function J$e(){}function eHe(){}function tHe(){}function nHe(){}function rHe(){}function iHe(){}function sHe(){}function aHe(){}function oHe(){}function cHe(){}function uHe(){}function lHe(){}function hHe(){}function fHe(){}function dHe(){}function gHe(){}function pHe(){}function bHe(){}function vHe(){}function wHe(){}function mHe(){}function yHe(){}function kHe(){}function xHe(){}function EHe(){}function _He(){}function THe(){}function CHe(){}function SHe(){}function AHe(){}function MHe(){}function DHe(){}function LHe(){}function oq(){}function cq(){}function uq(){}function IHe(){}function OHe(){}function NHe(){}function PHe(){}function BHe(){}function Kue(){}function FHe(){}function RHe(){}function Mhn(){}function jHe(){}function $He(){}function HHe(){}function zHe(){}function GHe(){}function VHe(){}function UHe(){}function mp(){}function qHe(){}function m4(){}function Xue(){}function YHe(){}function WHe(){}function KHe(){}function XHe(){}function QHe(){}function ZHe(){}function JHe(){}function eze(){}function tze(){}function nze(){}function rze(){}function ize(){}function sze(){}function aze(){}function oze(){}function cze(){}function uze(){}function lze(){}function hze(){}function fze(){}function dze(){}function yt(){}function gze(){}function pze(){}function bze(){}function vze(){}function wze(){}function mze(){}function yze(){}function kze(){}function xze(){}function Eze(){}function lq(){}function _ze(){}function Tze(){}function Cze(){}function Sze(){}function Aze(){}function Que(){}function zO(){}function GO(){}function Mze(){}function Zue(){}function VO(){}function Dze(){}function Lze(){}function Ize(){}function Oze(){}function Nze(){}function Pze(){}function UO(){}function Bze(){}function Fze(){}function Rze(){}function qO(){}function jze(){}function Jue(){}function $ze(){}function hq(){}function ele(){}function Hze(){}function zze(){}function Gze(){}function Vze(){}function Dhn(){}function Uze(){}function qze(){}function Yze(){}function Wze(){}function Kze(){}function Xze(){}function Qze(){}function Zze(){}function Jze(){}function eGe(){}function N6(){}function fq(){}function tGe(){}function nGe(){}function rGe(){}function iGe(){}function sGe(){}function aGe(){}function oGe(){}function cGe(){}function uGe(){}function lGe(){}function hGe(){}function fGe(){}function dGe(){}function gGe(){}function pGe(){}function bGe(){}function vGe(){}function wGe(){}function mGe(){}function yGe(){}function kGe(){}function xGe(){}function EGe(){}function _Ge(){}function TGe(){}function CGe(){}function SGe(){}function AGe(){}function MGe(){}function DGe(){}function LGe(){}function IGe(){}function OGe(){}function NGe(){}function PGe(){}function BGe(){}function FGe(){}function RGe(){}function jGe(){}function $Ge(){}function HGe(){}function zGe(){}function GGe(){}function VGe(){}function UGe(){}function qGe(){}function YGe(){}function WGe(){}function KGe(){}function XGe(){}function QGe(){}function ZGe(){}function JGe(){}function eVe(){}function tVe(){}function nVe(){}function rVe(){}function iVe(){}function sVe(){}function aVe(){}function oVe(){}function cVe(){}function uVe(){}function lVe(){}function hVe(){}function fVe(){}function dVe(){}function gVe(){}function pVe(){}function bVe(){}function vVe(){}function wVe(){}function mVe(){}function yVe(){}function kVe(){}function xVe(){}function EVe(){}function _Ve(){}function TVe(){}function CVe(){}function SVe(){}function AVe(){}function MVe(){}function DVe(){}function LVe(){}function IVe(){}function OVe(){}function NVe(){}function PVe(){}function BVe(){}function FVe(){}function RVe(){}function jVe(){}function $Ve(){}function HVe(){}function zVe(){}function GVe(){}function VVe(){}function UVe(){}function qVe(){}function YVe(){}function tle(){}function WVe(){}function KVe(){}function dq(){F7()}function XVe(){sX()}function QVe(){l9()}function ZVe(){HF()}function JVe(){ebe()}function eUe(){Jh()}function tUe(){h2e()}function nUe(){sM()}function rUe(){_S()}function iUe(){ES()}function sUe(){XS()}function aUe(){cZe()}function oUe(){t5()}function cUe(){xN()}function uUe(){kot()}function lUe(){Nut()}function hUe(){Jot()}function fUe(){bst()}function dUe(){Uk()}function gUe(){Mg()}function pUe(){Put()}function bUe(){uat()}function vUe(){Yve()}function wUe(){x1t()}function mUe(){vst()}function yUe(){tt()}function kUe(){pst()}function xUe(){But()}function EUe(){jlt()}function _Ue(){mst()}function TUe(){ict()}function CUe(){uZe()}function SUe(){Pbe()}function AUe(){Cw()}function MUe(){ult()}function DUe(){uM()}function LUe(){NQ()}function IUe(){HX()}function OUe(){Fb()}function NUe(){Lge()}function PUe(){wst()}function BUe(){Dpt()}function FUe(){Ibe()}function RUe(){CQ()}function jUe(){CB()}function $Ue(){WF()}function nle(){Tr()}function HUe(){uF()}function zUe(){jpe()}function rle(){dR()}function Gh(){snt()}function ile(){GK()}function GUe(){mve()}function sle(e){rn(e)}function VUe(e){this.a=e}function YO(e){this.a=e}function UUe(e){this.a=e}function qUe(e){this.a=e}function YUe(e){this.a=e}function WUe(e){this.a=e}function KUe(e){this.a=e}function XUe(e){this.a=e}function ale(e){this.a=e}function ole(e){this.a=e}function QUe(e){this.a=e}function gq(e){this.a=e}function ZUe(e){this.a=e}function pq(e){this.a=e}function JUe(e){this.a=e}function bq(e){this.a=e}function eqe(e){this.a=e}function vq(e){this.a=e}function tqe(e){this.a=e}function nqe(e){this.a=e}function rqe(e){this.a=e}function cle(e){this.b=e}function iqe(e){this.c=e}function sqe(e){this.a=e}function aqe(e){this.a=e}function oqe(e){this.a=e}function cqe(e){this.a=e}function uqe(e){this.a=e}function lqe(e){this.a=e}function hqe(e){this.a=e}function fqe(e){this.a=e}function dqe(e){this.a=e}function gqe(e){this.a=e}function pqe(e){this.a=e}function bqe(e){this.a=e}function vqe(e){this.a=e}function ule(e){this.a=e}function lle(e){this.a=e}function WO(e){this.a=e}function aS(e){this.a=e}function yp(){this.a=[]}function wqe(e,t){e.a=t}function Lhn(e,t){e.a=t}function Ihn(e,t){e.b=t}function Ohn(e,t){e.b=t}function Nhn(e,t){e.b=t}function hle(e,t){e.j=t}function Phn(e,t){e.g=t}function Bhn(e,t){e.i=t}function Fhn(e,t){e.c=t}function Rhn(e,t){e.d=t}function jhn(e,t){e.d=t}function $hn(e,t){e.c=t}function kp(e,t){e.k=t}function Hhn(e,t){e.c=t}function fle(e,t){e.c=t}function dle(e,t){e.a=t}function zhn(e,t){e.a=t}function Ghn(e,t){e.f=t}function Vhn(e,t){e.a=t}function Uhn(e,t){e.b=t}function wq(e,t){e.d=t}function KO(e,t){e.i=t}function gle(e,t){e.o=t}function qhn(e,t){e.r=t}function Yhn(e,t){e.a=t}function Whn(e,t){e.b=t}function mqe(e,t){e.e=t}function Khn(e,t){e.f=t}function ple(e,t){e.g=t}function Xhn(e,t){e.e=t}function Qhn(e,t){e.f=t}function Zhn(e,t){e.f=t}function Jhn(e,t){e.n=t}function efn(e,t){e.a=t}function tfn(e,t){e.a=t}function nfn(e,t){e.c=t}function rfn(e,t){e.c=t}function ifn(e,t){e.d=t}function sfn(e,t){e.e=t}function afn(e,t){e.g=t}function ofn(e,t){e.a=t}function cfn(e,t){e.c=t}function ufn(e,t){e.d=t}function lfn(e,t){e.e=t}function hfn(e,t){e.f=t}function ffn(e,t){e.j=t}function dfn(e,t){e.a=t}function gfn(e,t){e.b=t}function pfn(e,t){e.a=t}function yqe(e){e.b=e.a}function kqe(e){e.c=e.d.d}function L7(e){this.d=e}function xp(e){this.a=e}function P6(e){this.a=e}function ble(e){this.a=e}function vd(e){this.a=e}function oS(e){this.a=e}function xqe(e){this.a=e}function vle(e){this.a=e}function cS(e){this.a=e}function wle(e){this.a=e}function mle(e){this.a=e}function yle(e){this.a=e}function qv(e){this.a=e}function uS(e){this.a=e}function lS(e){this.a=e}function kle(e){this.b=e}function B6(e){this.b=e}function F6(e){this.b=e}function mq(e){this.a=e}function Eqe(e){this.a=e}function xle(e){this.a=e}function yq(e){this.c=e}function Q(e){this.c=e}function _qe(e){this.c=e}function Ele(e){this.a=e}function _le(e){this.a=e}function Tle(e){this.a=e}function Cle(e){this.a=e}function Hr(e){this.a=e}function Tqe(e){this.a=e}function Sle(e){this.a=e}function Ale(e){this.a=e}function Cqe(e){this.a=e}function Sqe(e){this.a=e}function I7(e){this.a=e}function Aqe(e){this.a=e}function Mqe(e){this.a=e}function Dqe(e){this.a=e}function Lqe(e){this.a=e}function Iqe(e){this.a=e}function Oqe(e){this.a=e}function Nqe(e){this.a=e}function Pqe(e){this.a=e}function Bqe(e){this.a=e}function Fqe(e){this.a=e}function Rqe(e){this.a=e}function jqe(e){this.a=e}function $qe(e){this.a=e}function Hqe(e){this.a=e}function zqe(e){this.a=e}function Gqe(e){this.a=e}function Vqe(e){this.a=e}function hS(e){this.a=e}function Uqe(e){this.a=e}function qqe(e){this.a=e}function XO(e){this.a=e}function Yqe(e){this.a=e}function Wqe(e){this.a=e}function R6(e){this.a=e}function Mle(e){this.a=e}function Kqe(e){this.a=e}function Xqe(e){this.a=e}function Qqe(e){this.a=e}function Zqe(e){this.a=e}function Jqe(e){this.a=e}function Dle(e){this.a=e}function Lle(e){this.a=e}function Ile(e){this.a=e}function QO(e){this.a=e}function ZO(e){this.e=e}function j6(e){this.a=e}function eYe(e){this.a=e}function y4(e){this.a=e}function Ole(e){this.a=e}function tYe(e){this.a=e}function nYe(e){this.a=e}function rYe(e){this.a=e}function iYe(e){this.a=e}function sYe(e){this.a=e}function aYe(e){this.a=e}function oYe(e){this.a=e}function cYe(e){this.a=e}function uYe(e){this.a=e}function lYe(e){this.a=e}function hYe(e){this.a=e}function Nle(e){this.a=e}function fYe(e){this.a=e}function dYe(e){this.a=e}function gYe(e){this.a=e}function pYe(e){this.a=e}function bYe(e){this.a=e}function vYe(e){this.a=e}function wYe(e){this.a=e}function mYe(e){this.a=e}function yYe(e){this.a=e}function kYe(e){this.a=e}function xYe(e){this.a=e}function EYe(e){this.a=e}function _Ye(e){this.a=e}function TYe(e){this.a=e}function CYe(e){this.a=e}function SYe(e){this.a=e}function AYe(e){this.a=e}function MYe(e){this.a=e}function DYe(e){this.a=e}function LYe(e){this.a=e}function IYe(e){this.a=e}function OYe(e){this.a=e}function NYe(e){this.a=e}function PYe(e){this.a=e}function BYe(e){this.a=e}function FYe(e){this.a=e}function RYe(e){this.a=e}function jYe(e){this.a=e}function $Ye(e){this.a=e}function HYe(e){this.a=e}function zYe(e){this.a=e}function GYe(e){this.a=e}function VYe(e){this.a=e}function UYe(e){this.a=e}function qYe(e){this.a=e}function YYe(e){this.a=e}function WYe(e){this.a=e}function KYe(e){this.c=e}function XYe(e){this.b=e}function QYe(e){this.a=e}function ZYe(e){this.a=e}function JYe(e){this.a=e}function eWe(e){this.a=e}function tWe(e){this.a=e}function nWe(e){this.a=e}function rWe(e){this.a=e}function iWe(e){this.a=e}function sWe(e){this.a=e}function aWe(e){this.a=e}function oWe(e){this.a=e}function cWe(e){this.a=e}function uWe(e){this.a=e}function lWe(e){this.a=e}function hWe(e){this.a=e}function fWe(e){this.a=e}function dWe(e){this.a=e}function gWe(e){this.a=e}function pWe(e){this.a=e}function bWe(e){this.a=e}function vWe(e){this.a=e}function wWe(e){this.a=e}function mWe(e){this.a=e}function yWe(e){this.a=e}function ng(e){this.a=e}function k4(e){this.a=e}function kWe(e){this.a=e}function xWe(e){this.a=e}function EWe(e){this.a=e}function _We(e){this.a=e}function TWe(e){this.a=e}function CWe(e){this.a=e}function SWe(e){this.a=e}function AWe(e){this.a=e}function MWe(e){this.a=e}function DWe(e){this.a=e}function LWe(e){this.a=e}function IWe(e){this.a=e}function OWe(e){this.a=e}function NWe(e){this.a=e}function PWe(e){this.a=e}function BWe(e){this.a=e}function JO(e){this.a=e}function FWe(e){this.a=e}function RWe(e){this.a=e}function jWe(e){this.a=e}function $We(e){this.a=e}function HWe(e){this.a=e}function zWe(e){this.a=e}function GWe(e){this.a=e}function VWe(e){this.a=e}function UWe(e){this.a=e}function qWe(e){this.a=e}function YWe(e){this.a=e}function WWe(e){this.a=e}function KWe(e){this.a=e}function XWe(e){this.a=e}function QWe(e){this.a=e}function ZWe(e){this.a=e}function JWe(e){this.a=e}function eKe(e){this.a=e}function tKe(e){this.a=e}function nKe(e){this.a=e}function rKe(e){this.a=e}function iKe(e){this.a=e}function sKe(e){this.a=e}function aKe(e){this.a=e}function oKe(e){this.a=e}function cKe(e){this.a=e}function uKe(e){this.a=e}function lKe(e){this.a=e}function Ple(e){this.a=e}function Xr(e){this.b=e}function hKe(e){this.f=e}function Ble(e){this.a=e}function fKe(e){this.a=e}function dKe(e){this.a=e}function gKe(e){this.a=e}function pKe(e){this.a=e}function bKe(e){this.a=e}function vKe(e){this.a=e}function wKe(e){this.a=e}function mKe(e){this.a=e}function fS(e){this.a=e}function yKe(e){this.a=e}function kKe(e){this.b=e}function Fle(e){this.c=e}function eN(e){this.e=e}function xKe(e){this.a=e}function tN(e){this.a=e}function nN(e){this.a=e}function kq(e){this.a=e}function EKe(e){this.a=e}function _Ke(e){this.d=e}function Rle(e){this.a=e}function jle(e){this.a=e}function G2(e){this.e=e}function dS(){this.a=0}function Jm(){ret(this)}function qe(){SY(this)}function zn(){Kc(this)}function xq(){uit(this)}function TKe(){}function V2(){this.c=yEe}function bfn(e,t){t.Wb(e)}function CKe(e,t){e.b+=t}function SKe(e){e.b=new jq}function ne(e){return e.e}function vfn(e){return e.a}function wfn(e){return e.a}function mfn(e){return e.a}function yfn(e){return e.a}function kfn(e){return e.a}function xfn(){return null}function Efn(){return null}function _fn(){The(),PNn()}function Tfn(e){e.b.tf(e.e)}function O7(e,t){e.b=t-e.b}function N7(e,t){e.a=t-e.a}function AKe(e,t){t.ad(e.a)}function Cfn(e,t){Ji(t,e)}function Sfn(e,t,n){e.Od(n,t)}function gS(e,t){e.e=t,t.b=e}function $le(e){t1(),this.a=e}function MKe(e){t1(),this.a=e}function DKe(e){t1(),this.a=e}function Hle(e){lw(),this.a=e}function LKe(e){bk(),ete.be(e)}function Ep(){zet.call(this)}function zle(){zet.call(this)}function Gle(){Ep.call(this)}function Eq(){Ep.call(this)}function IKe(){Ep.call(this)}function pS(){Ep.call(this)}function Ic(){Ep.call(this)}function P7(){Ep.call(this)}function Xn(){Ep.call(this)}function cl(){Ep.call(this)}function OKe(){Ep.call(this)}function lo(){Ep.call(this)}function NKe(){Ep.call(this)}function PKe(){this.a=this}function rN(){this.Bb|=256}function BKe(){this.b=new RJe}function Vle(){Vle=re,new zn}function Ule(){Gle.call(this)}function FKe(e,t){e.length=t}function iN(e,t){Ue(e.a,t)}function Afn(e,t){nbe(e.c,t)}function Mfn(e,t){Qi(e.b,t)}function Dfn(e,t){PF(e.a,t)}function Lfn(e,t){gX(e.a,t)}function $6(e,t){Or(e.e,t)}function x4(e){QF(e.c,e.b)}function Ifn(e,t){e.kc().Nb(t)}function qle(e){this.a=Dkn(e)}function ns(){this.a=new zn}function RKe(){this.a=new zn}function sN(){this.a=new qe}function _q(){this.a=new qe}function Yle(){this.a=new qe}function Il(){this.a=new ei}function _p(){this.a=new vot}function Wle(){this.a=new tS}function Kle(){this.a=new eZe}function jKe(){this.a=new Vst}function Xle(){this.a=new ast}function Qle(){this.a=new Dtt}function $Ke(){this.a=new qe}function Zle(){this.a=new qe}function HKe(){this.a=new qe}function zKe(){this.a=new qe}function GKe(){this.d=new qe}function VKe(){this.a=new ns}function UKe(){this.a=new zn}function qKe(){this.b=new zn}function YKe(){this.b=new qe}function Jle(){this.e=new qe}function WKe(){this.d=new qe}function KKe(){this.a=new gUe}function XKe(){qe.call(this)}function ehe(){sN.call(this)}function QKe(){gP.call(this)}function ZKe(){Zle.call(this)}function Tq(){B7.call(this)}function B7(){TKe.call(this)}function E4(){TKe.call(this)}function the(){E4.call(this)}function JKe(){Fit.call(this)}function eXe(){Fit.call(this)}function tXe(){che.call(this)}function nXe(){che.call(this)}function rXe(){che.call(this)}function iXe(){uhe.call(this)}function Oc(){di.call(this)}function nhe(){Aze.call(this)}function rhe(){Aze.call(this)}function sXe(){kXe.call(this)}function aXe(){kXe.call(this)}function oXe(){zn.call(this)}function cXe(){zn.call(this)}function uXe(){zn.call(this)}function lXe(){ns.call(this)}function Cq(){Lut.call(this)}function hXe(){rN.call(this)}function Sq(){Ofe.call(this)}function Aq(){Ofe.call(this)}function ihe(){zn.call(this)}function Mq(){zn.call(this)}function fXe(){zn.call(this)}function she(){qO.call(this)}function dXe(){qO.call(this)}function gXe(){she.call(this)}function pXe(){tle.call(this)}function bXe(e){nut.call(this,e)}function vXe(e){nut.call(this,e)}function ahe(e){ale.call(this,e)}function ohe(e){VQe.call(this,e)}function Ofn(e){ohe.call(this,e)}function Nfn(e){VQe.call(this,e)}function H6(){this.a=new di}function che(){this.a=new ns}function uhe(){this.a=new zn}function wXe(){this.a=new qe}function mXe(){this.j=new qe}function lhe(){this.a=new MHe}function yXe(){this.a=new pQe}function kXe(){this.a=new Rze}function Dq(){Dq=re,Wee=new jXe}function Lq(){Lq=re,Yee=new RXe}function F7(){F7=re,qee=new k}function aN(){aN=re,Qee=new $et}function Pfn(e){ohe.call(this,e)}function Bfn(e){ohe.call(this,e)}function xXe(e){oK.call(this,e)}function EXe(e){oK.call(this,e)}function _Xe(e){Ztt.call(this,e)}function Iq(e){n_n.call(this,e)}function U2(e){Xv.call(this,e)}function R7(e){wN.call(this,e)}function hhe(e){wN.call(this,e)}function TXe(e){wN.call(this,e)}function Ya(e){frt.call(this,e)}function CXe(e){Ya.call(this,e)}function _4(){aS.call(this,{})}function oN(e){tk(),this.a=e}function j7(e){e.b=null,e.c=0}function Ffn(e,t){e.e=t,Mgt(e,t)}function Rfn(e,t){e.a=t,z_n(e)}function Oq(e,t,n){e.a[t.g]=n}function jfn(e,t,n){iEn(n,e,t)}function $fn(e,t){hgn(t.i,e.n)}function SXe(e,t){f6n(e).td(t)}function Hfn(e,t){return e*e/t}function AXe(e,t){return e.g-t.g}function zfn(e){return new WO(e)}function Gfn(e){return new uw(e)}function cN(e){Ya.call(this,e)}function Ea(e){Ya.call(this,e)}function MXe(e){Ya.call(this,e)}function Nq(e){frt.call(this,e)}function Pq(e){Mge(),this.a=e}function DXe(e){int(),this.a=e}function Yv(e){hW(),this.f=e}function Bq(e){hW(),this.f=e}function z6(e){Ya.call(this,e)}function an(e){Ya.call(this,e)}function ja(e){Ya.call(this,e)}function LXe(e){Ya.call(this,e)}function T4(e){Ya.call(this,e)}function ft(e){return rn(e),e}function Pe(e){return rn(e),e}function bS(e){return rn(e),e}function fhe(e){return rn(e),e}function Vfn(e){return rn(e),e}function $7(e){return e.b==e.c}function Wv(e){return!!e&&e.b}function Ufn(e){return!!e&&e.k}function qfn(e){return!!e&&e.j}function _u(e){rn(e),this.a=e}function dhe(e){return zp(e),e}function H7(e){Tde(e,e.length)}function i0(e){Ya.call(this,e)}function Xf(e){Ya.call(this,e)}function Fq(e){Ya.call(this,e)}function ey(e){Ya.call(this,e)}function z7(e){Ya.call(this,e)}function Zn(e){Ya.call(this,e)}function Rq(e){Qfe.call(this,e,0)}function jq(){s0e.call(this,12,3)}function ghe(){ghe=re,b3e=new W}function IXe(){IXe=re,p3e=new y}function uN(){uN=re,h_=new q}function OXe(){OXe=re,h3t=new te}function NXe(){throw ne(new Xn)}function phe(){throw ne(new Xn)}function PXe(){throw ne(new Xn)}function Yfn(){throw ne(new Xn)}function Wfn(){throw ne(new Xn)}function Kfn(){throw ne(new Xn)}function $q(){this.a=Jn(Vn(Js))}function C4(e){t1(),this.a=Vn(e)}function BXe(e,t){e.Td(t),t.Sd(e)}function Xfn(e,t){e.a.ec().Mc(t)}function Qfn(e,t,n){e.c.lf(t,n)}function bhe(e){Ea.call(this,e)}function Qf(e){an.call(this,e)}function s0(){oS.call(this,"")}function G7(){oS.call(this,"")}function rg(){oS.call(this,"")}function Kv(){oS.call(this,"")}function vhe(e){Ea.call(this,e)}function G6(e){B6.call(this,e)}function Hq(e){sP.call(this,e)}function FXe(e){G6.call(this,e)}function RXe(){pq.call(this,null)}function jXe(){pq.call(this,null)}function lN(){lN=re,bk()}function $Xe(){$Xe=re,k3t=LEn()}function HXe(e){return e.a?e.b:0}function Zfn(e){return e.a?e.b:0}function Jfn(e,t){return e.a-t.a}function e1n(e,t){return e.a-t.a}function t1n(e,t){return e.a-t.a}function hN(e,t){return K0e(e,t)}function ie(e,t){return ust(e,t)}function n1n(e,t){return t in e.a}function zXe(e,t){return e.f=t,e}function r1n(e,t){return e.b=t,e}function GXe(e,t){return e.c=t,e}function i1n(e,t){return e.g=t,e}function whe(e,t){return e.a=t,e}function mhe(e,t){return e.f=t,e}function s1n(e,t){return e.k=t,e}function yhe(e,t){return e.a=t,e}function a1n(e,t){return e.e=t,e}function khe(e,t){return e.e=t,e}function o1n(e,t){return e.f=t,e}function c1n(e,t){e.b=!0,e.d=t}function u1n(e,t){e.b=new _a(t)}function l1n(e,t,n){t.td(e.a[n])}function h1n(e,t,n){t.we(e.a[n])}function f1n(e,t){return e.b-t.b}function d1n(e,t){return e.g-t.g}function g1n(e,t){return e.s-t.s}function p1n(e,t){return e?0:t-1}function VXe(e,t){return e?0:t-1}function b1n(e,t){return e?t-1:0}function v1n(e,t){return t.Yf(e)}function q2(e,t){return e.b=t,e}function fN(e,t){return e.a=t,e}function Y2(e,t){return e.c=t,e}function W2(e,t){return e.d=t,e}function K2(e,t){return e.e=t,e}function xhe(e,t){return e.f=t,e}function V7(e,t){return e.a=t,e}function V6(e,t){return e.b=t,e}function U6(e,t){return e.c=t,e}function Et(e,t){return e.c=t,e}function zt(e,t){return e.b=t,e}function _t(e,t){return e.d=t,e}function Tt(e,t){return e.e=t,e}function w1n(e,t){return e.f=t,e}function Ct(e,t){return e.g=t,e}function St(e,t){return e.a=t,e}function At(e,t){return e.i=t,e}function Mt(e,t){return e.j=t,e}function UXe(e,t){return e.k=t,e}function m1n(e,t){return e.j=t,e}function y1n(e,t){Mg(),Ka(t,e)}function k1n(e,t,n){pbn(e.a,t,n)}function qXe(e){hit.call(this,e)}function Ehe(e){hit.call(this,e)}function dN(e){WY.call(this,e)}function YXe(e){Rkn.call(this,e)}function ig(e){yb.call(this,e)}function WXe(e){BW.call(this,e)}function KXe(e){BW.call(this,e)}function XXe(){Afe.call(this,"")}function Rs(){this.a=0,this.b=0}function QXe(){this.b=0,this.a=0}function ZXe(e,t){e.b=0,yw(e,t)}function x1n(e,t){e.c=t,e.b=!0}function JXe(e,t){return e.c._b(t)}function Vh(e){return e.e&&e.e()}function zq(e){return e?e.d:null}function eQe(e,t){return Aht(e.b,t)}function E1n(e){return e?e.g:null}function _1n(e){return e?e.i:null}function sg(e){return yd(e),e.o}function X2(){X2=re,m9t=z7n()}function tQe(){tQe=re,ps=e9n()}function q6(){q6=re,mEe=V7n()}function nQe(){nQe=re,r_t=G7n()}function _he(){_he=re,bo=j_n()}function The(){The=re,tp=Ok()}function rQe(){throw ne(new Xn)}function iQe(){throw ne(new Xn)}function sQe(){throw ne(new Xn)}function aQe(){throw ne(new Xn)}function oQe(){throw ne(new Xn)}function cQe(){throw ne(new Xn)}function gN(e){this.a=new S4(e)}function Che(e){sbt(),WNn(this,e)}function ag(e){this.a=new pW(e)}function ty(e,t){for(;e.ye(t););}function She(e,t){for(;e.sd(t););}function ny(e,t){return e.a+=t,e}function Gq(e,t){return e.a+=t,e}function a0(e,t){return e.a+=t,e}function Q2(e,t){return e.a+=t,e}function U7(e){return vg(e),e.a}function pN(e){return e.b!=e.d.c}function uQe(e){return e.l|e.m<<22}function Ahe(e,t){return e.d[t.p]}function lQe(e,t){return AAn(e,t)}function Mhe(e,t,n){e.splice(t,n)}function hQe(e){e.c?Xgt(e):Qgt(e)}function bN(e){this.a=0,this.b=e}function fQe(){this.a=new mM(Ixe)}function dQe(){this.b=new mM(wxe)}function gQe(){this.b=new mM(mre)}function pQe(){this.b=new mM(mre)}function bQe(){throw ne(new Xn)}function vQe(){throw ne(new Xn)}function wQe(){throw ne(new Xn)}function mQe(){throw ne(new Xn)}function yQe(){throw ne(new Xn)}function kQe(){throw ne(new Xn)}function xQe(){throw ne(new Xn)}function EQe(){throw ne(new Xn)}function _Qe(){throw ne(new Xn)}function TQe(){throw ne(new Xn)}function T1n(){throw ne(new lo)}function C1n(){throw ne(new lo)}function vS(e){this.a=new CQe(e)}function CQe(e){F4n(this,e,FEn())}function wS(e){return!e||Krt(e)}function mS(e){return Gf[e]!=-1}function S1n(){Nj!=0&&(Nj=0),Pj=-1}function SQe(){Uee==null&&(Uee=[])}function A1n(e,t){kQ(De(e.a),t)}function M1n(e,t){kQ(De(e.a),t)}function yS(e,t){hy.call(this,e,t)}function Y6(e,t){yS.call(this,e,t)}function Dhe(e,t){this.b=e,this.c=t}function AQe(e,t){this.b=e,this.a=t}function MQe(e,t){this.a=e,this.b=t}function DQe(e,t){this.a=e,this.b=t}function LQe(e,t){this.a=e,this.b=t}function IQe(e,t){this.a=e,this.b=t}function OQe(e,t){this.a=e,this.b=t}function NQe(e,t){this.a=e,this.b=t}function PQe(e,t){this.a=e,this.b=t}function BQe(e,t){this.a=e,this.b=t}function FQe(e,t){this.b=e,this.a=t}function RQe(e,t){this.b=e,this.a=t}function jQe(e,t){this.b=e,this.a=t}function $Qe(e,t){this.b=e,this.a=t}function ar(e,t){this.f=e,this.g=t}function W6(e,t){this.e=e,this.d=t}function Z2(e,t){this.g=e,this.i=t}function Vq(e,t){this.a=e,this.b=t}function HQe(e,t){this.a=e,this.f=t}function zQe(e,t){this.b=e,this.c=t}function D1n(e,t){this.a=e,this.b=t}function GQe(e,t){this.a=e,this.b=t}function Uq(e,t){this.a=e,this.b=t}function VQe(e){$fe(e.dc()),this.c=e}function vN(e){this.b=h(Vn(e),83)}function UQe(e){this.a=h(Vn(e),83)}function Xv(e){this.a=h(Vn(e),15)}function qQe(e){this.a=h(Vn(e),15)}function wN(e){this.b=h(Vn(e),47)}function mN(){this.q=new d.Date}function L1(){L1=re,L3e=new ke}function K6(){K6=re,Vx=new ge}function q7(e){return e.f.c+e.g.c}function kS(e,t){return e.b.Hc(t)}function YQe(e,t){return e.b.Ic(t)}function WQe(e,t){return e.b.Qc(t)}function KQe(e,t){return e.b.Hc(t)}function XQe(e,t){return e.c.uc(t)}function wd(e,t){return e.a._b(t)}function QQe(e,t){return Nr(e.c,t)}function ZQe(e,t){return Au(e.b,t)}function JQe(e,t){return e>t&&t<N9}function L1n(e,t){return e.Gc(t),e}function I1n(e,t){return Qs(e,t),e}function O1n(e){return Vrt(),e?l3t:u3t}function S4(e){$lt.call(this,e,0)}function eZe(){pW.call(this,null)}function Lhe(){vK.call(this,null)}function A4(e){this.c=e,Pct(this)}function di(){fet(this),Pl(this)}function Ti(e,t){vg(e),e.a.Nb(t)}function tZe(e,t){return e.Gc(t),e}function N1n(e,t){return e.a.f=t,e}function nZe(e,t){return e.a.d=t,e}function rZe(e,t){return e.a.g=t,e}function iZe(e,t){return e.a.j=t,e}function dh(e,t){return e.a.a=t,e}function gh(e,t){return e.a.d=t,e}function ph(e,t){return e.a.e=t,e}function bh(e,t){return e.a.g=t,e}function xS(e,t){return e.a.f=t,e}function P1n(e){return e.b=!1,e}function X6(){X6=re,G3e=new jJe}function yN(){yN=re,E3t=new $Je}function Q6(){Q6=re,K3e=new je}function sZe(){sZe=re,E5t=new MBe}function kN(){kN=re,bte=new Jet}function Tp(){Tp=re,Ux=new Rn}function Y7(){Y7=re,T5t=new DBe}function aZe(){aZe=re,O3t=new Jt}function Ihe(){Ihe=re,f5t=new sq}function ES(){ES=re,A5t=new Rs}function oZe(){oZe=re,d5t=new H2}function Ohe(){Ohe=re,g5t=new irt}function Nhe(){Nhe=re,q4e=new w4}function _S(){_S=re,M5t=new rFe}function cZe(){cZe=re,O5t=new HFe}function W7(){W7=re,j5t=new xje}function xN(){xN=re,s3=new qje}function uZe(){uZe=re,o7t=new rs}function EN(){EN=re,yre=new hZe}function _N(){_N=re,kre=new dtt}function K7(){K7=re,UD=new rit}function TN(){TN=re,Mxt=new kze}function lZe(){Out(),this.c=new jq}function hZe(){ar.call(this,ewt,0)}function J2(e,t){Yp(e.c.b,t.c,t)}function X7(e,t){Yp(e.c.c,t.b,t)}function B1n(e,t,n){Ta(e.d,t.f,n)}function F1n(e,t,n,s){d8n(e,s,t,n)}function R1n(e,t,n,s){PSn(s,e,t,n)}function j1n(e,t,n,s){ZOn(s,e,t,n)}function CN(e,t){return e.a=t.g,e}function fZe(e,t){return x7n(e.a,t)}function $1n(e){return e.b?e.b:e.a}function H1n(e){return(e.c+e.a)/2}function Qv(){Qv=re,v9t=new Sze}function dZe(){dZe=re,T9t=new Lze}function gZe(){gZe=re,R9t=new cXe}function pZe(){pZe=re,j9t=new uXe}function og(){og=re,th=new ihe}function Phe(){Phe=re,wEe=new fXe}function Q7(){Q7=re,sie=new wet}function o0(){o0=re,vL=new met}function qq(){qq=re,J9t=new qGe}function ia(){ia=re,t_t=new YGe}function SN(){SN=re,KH=new zn}function Bhe(){Bhe=re,MEe=new qe}function cg(){cg=re,sp=new KVe}function z1n(e){d.clearTimeout(e)}function Fhe(e){this.a=h(Vn(e),224)}function AN(e){return h(e,42).cd()}function TS(e){return e.b<e.d.gc()}function bZe(e,t){return cW(e.a,t)}function vZe(e,t){return mo(e,t)>0}function Yq(e,t){return mo(e,t)<0}function Z7(e,t){return e.a.get(t)}function G1n(e,t){return t.split(e)}function wZe(e,t){return Au(e.e,t)}function Rhe(e){return rn(e),!1}function MN(e){Wt.call(this,e,21)}function V1n(e,t){Qit.call(this,e,t)}function DN(e,t){ar.call(this,e,t)}function Wq(e,t){ar.call(this,e,t)}function jhe(e){AW(),Ztt.call(this,e)}function $he(e,t){trt(e,e.length,t)}function CS(e,t){Drt(e,e.length,t)}function U1n(e,t,n){t.ud(e.a.Ge(n))}function q1n(e,t,n){t.we(e.a.Fe(n))}function Y1n(e,t,n){t.td(e.a.Kb(n))}function W1n(e,t,n){e.Mb(n)&&t.td(n)}function J7(e,t,n){e.splice(t,0,n)}function K1n(e,t){return Pc(e.e,t)}function LN(e,t){this.d=e,this.e=t}function mZe(e,t){this.b=e,this.a=t}function yZe(e,t){this.b=e,this.a=t}function Hhe(e,t){this.b=e,this.a=t}function kZe(e,t){this.a=e,this.b=t}function xZe(e,t){this.a=e,this.b=t}function EZe(e,t){this.a=e,this.b=t}function _Ze(e,t){this.a=e,this.b=t}function M4(e,t){this.a=e,this.b=t}function zhe(e,t){this.b=e,this.a=t}function Ghe(e,t){this.b=e,this.a=t}function IN(e,t){ar.call(this,e,t)}function ON(e,t){ar.call(this,e,t)}function Vhe(e,t){ar.call(this,e,t)}function Uhe(e,t){ar.call(this,e,t)}function ry(e,t){ar.call(this,e,t)}function Kq(e,t){ar.call(this,e,t)}function Xq(e,t){ar.call(this,e,t)}function Qq(e,t){ar.call(this,e,t)}function NN(e,t){ar.call(this,e,t)}function qhe(e,t){ar.call(this,e,t)}function Zq(e,t){ar.call(this,e,t)}function SS(e,t){ar.call(this,e,t)}function PN(e,t){ar.call(this,e,t)}function Jq(e,t){ar.call(this,e,t)}function eE(e,t){ar.call(this,e,t)}function Yhe(e,t){ar.call(this,e,t)}function Ii(e,t){ar.call(this,e,t)}function BN(e,t){ar.call(this,e,t)}function TZe(e,t){this.a=e,this.b=t}function CZe(e,t){this.a=e,this.b=t}function SZe(e,t){this.a=e,this.b=t}function AZe(e,t){this.a=e,this.b=t}function MZe(e,t){this.a=e,this.b=t}function DZe(e,t){this.a=e,this.b=t}function LZe(e,t){this.a=e,this.b=t}function IZe(e,t){this.a=e,this.b=t}function OZe(e,t){this.a=e,this.b=t}function Whe(e,t){this.b=e,this.a=t}function NZe(e,t){this.b=e,this.a=t}function PZe(e,t){this.b=e,this.a=t}function BZe(e,t){this.b=e,this.a=t}function Z6(e,t){this.c=e,this.d=t}function FZe(e,t){this.e=e,this.d=t}function RZe(e,t){this.a=e,this.b=t}function jZe(e,t){this.b=t,this.c=e}function FN(e,t){ar.call(this,e,t)}function AS(e,t){ar.call(this,e,t)}function eY(e,t){ar.call(this,e,t)}function tE(e,t){ar.call(this,e,t)}function Khe(e,t){ar.call(this,e,t)}function tY(e,t){ar.call(this,e,t)}function nY(e,t){ar.call(this,e,t)}function MS(e,t){ar.call(this,e,t)}function Xhe(e,t){ar.call(this,e,t)}function rY(e,t){ar.call(this,e,t)}function nE(e,t){ar.call(this,e,t)}function Qhe(e,t){ar.call(this,e,t)}function rE(e,t){ar.call(this,e,t)}function iE(e,t){ar.call(this,e,t)}function Zv(e,t){ar.call(this,e,t)}function iY(e,t){ar.call(this,e,t)}function sY(e,t){ar.call(this,e,t)}function Zhe(e,t){ar.call(this,e,t)}function sE(e,t){ar.call(this,e,t)}function aY(e,t){ar.call(this,e,t)}function RN(e,t){ar.call(this,e,t)}function DS(e,t){ar.call(this,e,t)}function LS(e,t){ar.call(this,e,t)}function D4(e,t){ar.call(this,e,t)}function oY(e,t){ar.call(this,e,t)}function Jhe(e,t){ar.call(this,e,t)}function cY(e,t){ar.call(this,e,t)}function uY(e,t){ar.call(this,e,t)}function efe(e,t){ar.call(this,e,t)}function lY(e,t){ar.call(this,e,t)}function hY(e,t){ar.call(this,e,t)}function fY(e,t){ar.call(this,e,t)}function dY(e,t){ar.call(this,e,t)}function tfe(e,t){ar.call(this,e,t)}function $Ze(e,t){this.b=e,this.a=t}function HZe(e,t){this.a=e,this.b=t}function zZe(e,t){this.a=e,this.b=t}function GZe(e,t){this.a=e,this.b=t}function VZe(e,t){this.a=e,this.b=t}function nfe(e,t){ar.call(this,e,t)}function rfe(e,t){ar.call(this,e,t)}function UZe(e,t){this.b=e,this.d=t}function ife(e,t){ar.call(this,e,t)}function sfe(e,t){ar.call(this,e,t)}function qZe(e,t){this.a=e,this.b=t}function YZe(e,t){this.a=e,this.b=t}function jN(e,t){ar.call(this,e,t)}function aE(e,t){ar.call(this,e,t)}function afe(e,t){ar.call(this,e,t)}function ofe(e,t){ar.call(this,e,t)}function cfe(e,t){ar.call(this,e,t)}function gY(e,t){ar.call(this,e,t)}function ufe(e,t){ar.call(this,e,t)}function pY(e,t){ar.call(this,e,t)}function $N(e,t){ar.call(this,e,t)}function bY(e,t){ar.call(this,e,t)}function vY(e,t){ar.call(this,e,t)}function IS(e,t){ar.call(this,e,t)}function wY(e,t){ar.call(this,e,t)}function lfe(e,t){ar.call(this,e,t)}function OS(e,t){ar.call(this,e,t)}function hfe(e,t){ar.call(this,e,t)}function X1n(e,t){return Pc(e.c,t)}function Q1n(e,t){return Pc(t.b,e)}function Z1n(e,t){return-e.b.Je(t)}function ffe(e,t){return Pc(e.g,t)}function NS(e,t){ar.call(this,e,t)}function L4(e,t){ar.call(this,e,t)}function WZe(e,t){this.a=e,this.b=t}function KZe(e,t){this.a=e,this.b=t}function dt(e,t){this.a=e,this.b=t}function oE(e,t){ar.call(this,e,t)}function cE(e,t){ar.call(this,e,t)}function PS(e,t){ar.call(this,e,t)}function mY(e,t){ar.call(this,e,t)}function HN(e,t){ar.call(this,e,t)}function uE(e,t){ar.call(this,e,t)}function yY(e,t){ar.call(this,e,t)}function zN(e,t){ar.call(this,e,t)}function iy(e,t){ar.call(this,e,t)}function BS(e,t){ar.call(this,e,t)}function lE(e,t){ar.call(this,e,t)}function hE(e,t){ar.call(this,e,t)}function FS(e,t){ar.call(this,e,t)}function GN(e,t){ar.call(this,e,t)}function sy(e,t){ar.call(this,e,t)}function VN(e,t){ar.call(this,e,t)}function XZe(e,t){this.a=e,this.b=t}function QZe(e,t){this.a=e,this.b=t}function ZZe(e,t){this.a=e,this.b=t}function JZe(e,t){this.a=e,this.b=t}function eJe(e,t){this.a=e,this.b=t}function tJe(e,t){this.a=e,this.b=t}function As(e,t){this.a=e,this.b=t}function UN(e,t){ar.call(this,e,t)}function nJe(e,t){this.a=e,this.b=t}function rJe(e,t){this.a=e,this.b=t}function iJe(e,t){this.a=e,this.b=t}function sJe(e,t){this.a=e,this.b=t}function aJe(e,t){this.a=e,this.b=t}function oJe(e,t){this.a=e,this.b=t}function cJe(e,t){this.b=e,this.a=t}function uJe(e,t){this.b=e,this.a=t}function lJe(e,t){this.b=e,this.a=t}function hJe(e,t){this.b=e,this.a=t}function fJe(e,t){this.a=e,this.b=t}function dJe(e,t){this.a=e,this.b=t}function J1n(e,t){MSn(e.a,h(t,56))}function gJe(e,t){H3n(e.a,h(t,11))}function edn(e,t){return ak(),t!=e}function pJe(){return $Xe(),new k3t}function bJe(){YW(),this.b=new ns}function vJe(){iR(),this.a=new ns}function wJe(){i0e(),dde.call(this)}function I4(e,t){ar.call(this,e,t)}function mJe(e,t){this.a=e,this.b=t}function yJe(e,t){this.a=e,this.b=t}function qN(e,t){this.a=e,this.b=t}function kJe(e,t){this.a=e,this.b=t}function xJe(e,t){this.a=e,this.b=t}function EJe(e,t){this.a=e,this.b=t}function _Je(e,t){this.d=e,this.b=t}function dfe(e,t){this.d=e,this.e=t}function TJe(e,t){this.f=e,this.c=t}function RS(e,t){this.b=e,this.c=t}function gfe(e,t){this.i=e,this.g=t}function CJe(e,t){this.e=e,this.a=t}function SJe(e,t){this.a=e,this.b=t}function pfe(e,t){e.i=null,WB(e,t)}function tdn(e,t){e&&Pr(gL,e,t)}function AJe(e,t){return TX(e.a,t)}function YN(e){return QA(e.c,e.b)}function no(e){return e?e.dd():null}function Me(e){return e==null?null:e}function Jv(e){return typeof e===g5}function ew(e){return typeof e===Wve}function ws(e){return typeof e===hZ}function ug(e,t){return e.Hd().Xb(t)}function WN(e,t){return v5n(e.Kc(),t)}function eb(e,t){return mo(e,t)==0}function ndn(e,t){return mo(e,t)>=0}function fE(e,t){return mo(e,t)!=0}function rdn(e){return""+(rn(e),e)}function jS(e,t){return e.substr(t)}function MJe(e){return lu(e),e.d.gc()}function kY(e){return JTn(e,e.c),e}function KN(e){return TE(e==null),e}function dE(e,t){return e.a+=""+t,e}function va(e,t){return e.a+=""+t,e}function gE(e,t){return e.a+=""+t,e}function ho(e,t){return e.a+=""+t,e}function or(e,t){return e.a+=""+t,e}function bfe(e,t){return e.a+=""+t,e}function DJe(e,t){Ai(e,t,e.a,e.a.a)}function Cp(e,t){Ai(e,t,e.c.b,e.c)}function idn(e,t,n){H1t(t,gQ(e,n))}function sdn(e,t,n){H1t(t,gQ(e,n))}function adn(e,t){Z3n(new Tn(e),t)}function LJe(e,t){e.q.setTime(vb(t))}function IJe(e,t){xde.call(this,e,t)}function OJe(e,t){xde.call(this,e,t)}function xY(e,t){xde.call(this,e,t)}function NJe(e){Kc(this),QE(this,e)}function vfe(e){return Kt(e,0),null}function Uh(e){return e.a=0,e.b=0,e}function PJe(e,t){return e.a=t.g+1,e}function odn(e,t){return e.j[t.p]==2}function wfe(e){return Gbn(h(e,79))}function BJe(){BJe=re,p4t=ur(LX())}function FJe(){FJe=re,I5t=ur(kgt())}function RJe(){this.b=new S4(vw(12))}function jJe(){this.b=0,this.a=!1}function $Je(){this.b=0,this.a=!1}function pE(e){this.a=e,dq.call(this)}function HJe(e){this.a=e,dq.call(this)}function $t(e,t){Xi.call(this,e,t)}function EY(e,t){sw.call(this,e,t)}function ay(e,t){gfe.call(this,e,t)}function _Y(e,t){Rk.call(this,e,t)}function zJe(e,t){$S.call(this,e,t)}function wr(e,t){SN(),Pr(KH,e,t)}function TY(e,t){return $u(e.a,0,t)}function GJe(e,t){return e.a.a.a.cc(t)}function VJe(e,t){return Me(e)===Me(t)}function cdn(e,t){return Ui(e.a,t.a)}function udn(e,t){return dc(e.a,t.a)}function ldn(e,t){return Srt(e.a,t.a)}function Zf(e,t){return e.indexOf(t)}function tb(e,t){return e==t?0:e?1:-1}function XN(e){return e<10?"0"+e:""+e}function hdn(e){return Vn(e),new pE(e)}function UJe(e){return rc(e.l,e.m,e.h)}function J6(e){return Li((rn(e),e))}function fdn(e){return Li((rn(e),e))}function qJe(e,t){return dc(e.g,t.g)}function $a(e){return typeof e===Wve}function ddn(e){return e==Zb||e==Zw}function gdn(e){return e==Zb||e==Qw}function mfe(e){return Ha(e.b.b,e,0)}function YJe(e){this.a=pJe(),this.b=e}function WJe(e){this.a=pJe(),this.b=e}function pdn(e,t){return Ue(e.a,t),t}function bdn(e,t){return Ue(e.c,t),e}function KJe(e,t){return qu(e.a,t),e}function vdn(e,t){return wh(),t.a+=e}function wdn(e,t){return wh(),t.a+=e}function mdn(e,t){return wh(),t.c+=e}function yfe(e,t){_k(e,0,e.length,t)}function md(){Sle.call(this,new Np)}function XJe(){MP.call(this,0,0,0,0)}function O4(){hl.call(this,0,0,0,0)}function _a(e){this.a=e.a,this.b=e.b}function lg(e){return e==Wl||e==Dh}function ek(e){return e==Gd||e==zd}function QJe(e){return e==w3||e==v3}function oy(e){return e!=$f&&e!=Qg}function Tu(e){return e.Lg()&&e.Mg()}function ZJe(e){return UP(h(e,118))}function QN(e){return qu(new rs,e)}function JJe(e,t){return new Rk(t,e)}function ydn(e,t){return new Rk(t,e)}function kfe(e,t,n){$B(e,t),HB(e,n)}function ZN(e,t,n){Eb(e,t),xb(e,n)}function bf(e,t,n){yc(e,t),kc(e,n)}function JN(e,t,n){Ak(e,t),Dk(e,n)}function eP(e,t,n){Mk(e,t),Lk(e,n)}function CY(e,t){Vk(e,t),Ik(e,e.D)}function xfe(e){TJe.call(this,e,!0)}function eet(e,t,n){h1e.call(this,e,t,n)}function hg(e){Dg(),k5n.call(this,e)}function tet(){DN.call(this,"Head",1)}function net(){DN.call(this,"Tail",3)}function SY(e){e.c=pe(yn,st,1,0,5,1)}function ret(e){e.a=pe(yn,st,1,8,5,1)}function iet(e){mc(e.xf(),new qqe(e))}function cy(e){return e!=null?ii(e):0}function kdn(e,t){return ww(t,mf(e))}function xdn(e,t){return ww(t,mf(e))}function Edn(e,t){return e[e.length]=t}function _dn(e,t){return e[e.length]=t}function Efe(e){return E2n(e.b.Kc(),e.a)}function Tdn(e,t){return YB(CW(e.d),t)}function Cdn(e,t){return YB(CW(e.g),t)}function Sdn(e,t){return YB(CW(e.j),t)}function sa(e,t){Xi.call(this,e.b,t)}function nb(e){MP.call(this,e,e,e,e)}function _fe(e){return e.b&&WQ(e),e.a}function Tfe(e){return e.b&&WQ(e),e.c}function Adn(e,t){Pf||(e.b=t)}function AY(e,t,n){return pi(e,t,n),n}function set(e,t,n){pi(e.c[t.g],t.g,n)}function Mdn(e,t,n){h(e.c,69).Xh(t,n)}function Ddn(e,t,n){bf(n,n.i+e,n.j+t)}function Ldn(e,t){qn(Co(e.a),kst(t))}function Idn(e,t){qn(cu(e.a),xst(t))}function bE(e){Mr(),G2.call(this,e)}function Odn(e){return e==null?0:ii(e)}function aet(){aet=re,Zne=new o9(Yre)}function Qn(){Qn=re,new oet,new qe}function oet(){new zn,new zn,new zn}function Cfe(){Cfe=re,Vle(),v3e=new zn}function vf(){vf=re,d.Math.log(2)}function ul(){ul=re,td=(dZe(),T9t)}function Ndn(){throw ne(new i0(Xyt))}function Pdn(){throw ne(new i0(Xyt))}function Bdn(){throw ne(new i0(Qyt))}function Fdn(){throw ne(new i0(Qyt))}function cet(e){this.a=e,V1e.call(this,e)}function MY(e){this.a=e,vN.call(this,e)}function DY(e){this.a=e,vN.call(this,e)}function hs(e,t){oW(e.c,e.c.length,t)}function Wa(e){return e.a<e.c.c.length}function Sfe(e){return e.a<e.c.a.length}function uet(e,t){return e.a?e.b:t.De()}function dc(e,t){return e<t?-1:e>t?1:0}function het(e,t){return mo(e,t)>0?e:t}function rc(e,t,n){return{l:e,m:t,h:n}}function Rdn(e,t){e.a!=null&&gJe(t,e.a)}function fet(e){e.a=new Ce,e.c=new Ce}function tP(e){this.b=e,this.a=new qe}function det(e){this.b=new Di,this.a=e}function Afe(e){y1e.call(this),this.a=e}function get(){DN.call(this,"Range",2)}function pet(){x2e(),this.a=new mM(U4e)}function jdn(e,t){Vn(t),dy(e).Jc(new N)}function $dn(e,t){return Hu(),t.n.b+=e}function Hdn(e,t,n){return Pr(e.g,n,t)}function zdn(e,t,n){return Pr(e.k,n,t)}function Gdn(e,t){return Pr(e.a,t.a,t)}function uy(e,t,n){return zpe(t,n,e.c)}function Mfe(e){return new dt(e.c,e.d)}function Vdn(e){return new dt(e.c,e.d)}function ro(e){return new dt(e.a,e.b)}function bet(e,t){return dOn(e.a,t,null)}function Udn(e){Gs(e,null),xs(e,null)}function vet(e){KW(e,null),XW(e,null)}function wet(){$S.call(this,null,null)}function met(){uP.call(this,null,null)}function Dfe(e){this.a=e,zn.call(this)}function qdn(e){this.b=(Rt(),new yq(e))}function nP(e){e.j=pe(D3e,Re,310,0,0,1)}function Ydn(e,t,n){e.c.Vc(t,h(n,133))}function Wdn(e,t,n){e.c.ji(t,h(n,133))}function yet(e,t){jn(e),e.Gc(h(t,15))}function vE(e,t){return MIn(e.c,e.b,t)}function Kdn(e,t){return new Get(e.Kc(),t)}function LY(e,t){return W5n(e.Kc(),t)!=-1}function Lfe(e,t){return e.a.Bc(t)!=null}function rP(e){return e.Ob()?e.Pb():null}function ket(e){return Fl(e,0,e.length)}function ue(e,t){return e!=null&&PX(e,t)}function Xdn(e,t){e.q.setHours(t),C9(e,t)}function xet(e,t){e.c&&(tde(t),Kit(t))}function Qdn(e,t,n){h(e.Kb(n),164).Nb(t)}function Zdn(e,t,n){return aOn(e,t,n),n}function Eet(e,t,n){e.a=t^1502,e.b=n^VZ}function IY(e,t,n){return e.a[t.g][n.g]}function wf(e,t){return e.a[t.c.p][t.p]}function Jdn(e,t){return e.e[t.c.p][t.p]}function e0n(e,t){return e.c[t.c.p][t.p]}function t0n(e,t){return e.j[t.p]=lSn(t)}function n0n(e,t){return N0e(e.f,t.tg())}function r0n(e,t){return N0e(e.b,t.tg())}function i0n(e,t){return e.a<L1e(t)?-1:1}function s0n(e,t,n){return n?t!=0:t!=e-1}function a0n(e,t,n){return e.a=t,e.b=n,e}function Jf(e,t){return e.a*=t,e.b*=t,e}function wE(e,t,n){return pi(e.g,t,n),n}function o0n(e,t,n,s){pi(e.a[t.g],n.g,s)}function c0n(e,t){nw(t,e.a.a.a,e.a.a.b)}function _et(e){e.a=h(tn(e.b.a,4),126)}function Tet(e){e.a=h(tn(e.b.a,4),126)}function u0n(e){aA(e,_mt),AQ(e,lNn(e))}function tk(){tk=re,$j=new oN(null)}function Ife(){Ife=re,Ife(),_3t=new Ae}function Ofe(){this.Bb|=256,this.Bb|=512}function Tn(e){this.i=e,this.f=this.i.j}function Gi(e,t,n){JS.call(this,e,t,n)}function iP(e,t,n){Gi.call(this,e,t,n)}function Nc(e,t,n){Gi.call(this,e,t,n)}function Cet(e,t,n){iP.call(this,e,t,n)}function Nfe(e,t,n){JS.call(this,e,t,n)}function ly(e,t,n){JS.call(this,e,t,n)}function Pfe(e,t,n){yP.call(this,e,t,n)}function Aet(e,t,n){yP.call(this,e,t,n)}function Met(e,t,n){Pfe.call(this,e,t,n)}function Det(e,t,n){Nfe.call(this,e,t,n)}function hy(e,t){this.a=e,vN.call(this,t)}function Let(e,t){this.a=e,Rq.call(this,t)}function Iet(e,t){this.a=e,Rq.call(this,t)}function Oet(e,t){this.a=e,Rq.call(this,t)}function Bfe(e){this.a=e,iqe.call(this,e.d)}function fg(e){this.c=e,this.a=this.c.a}function Ffe(e,t){this.a=t,Rq.call(this,e)}function Net(e,t){this.a=t,oK.call(this,e)}function Pet(e,t){this.a=e,oK.call(this,t)}function l0n(e,t){return yde(fW(e.c)).Xb(t)}function Rfe(e,t){return Tkn(e,new rg,t).a}function Dn(e,t){return Vn(t),new Bet(e,t)}function Bet(e,t){this.a=t,wN.call(this,e)}function jfe(e){this.b=e,this.a=this.b.a.e}function Fet(e){e.b.Qb(),--e.d.f.d,LP(e.d)}function Ret(e){pq.call(this,h(Vn(e),35))}function jet(e){pq.call(this,h(Vn(e),35))}function $et(){ar.call(this,"INSTANCE",0)}function $fe(e){if(!e)throw ne(new pS)}function Hfe(e){if(!e)throw ne(new Ic)}function zfe(e){if(!e)throw ne(new lo)}function Het(){Het=re,qq(),e_t=new GUe}function on(){on=re,u2=!1,Hx=!0}function Ol(e){oS.call(this,(rn(e),e))}function ju(e){oS.call(this,(rn(e),e))}function sP(e){B6.call(this,e),this.a=e}function Gfe(e){F6.call(this,e),this.a=e}function Vfe(e){G6.call(this,e),this.a=e}function zet(){nP(this),KP(this),this._d()}function Get(e,t){this.a=t,wN.call(this,e)}function Vet(e,t){return new Rdt(e.a,e.b,t)}function aP(e,t){return e.lastIndexOf(t)}function Ufe(e,t,n){return e.indexOf(t,n)}function mE(e){return e==null?_c:Ga(e)}function h0n(e){return e==null?null:e.name}function qfe(e){return e.a!=null?e.a:null}function f0n(e){return pN(e.a)?Cst(e):null}function OY(e,t){return FE(e.a,t)!=null}function Pc(e,t){return!!t&&e.b[t.g]==t}function rb(e){return e.$H||(e.$H=++EPn)}function d0n(e){return e.l+e.m*v5+e.h*Jp}function Uet(e,t){return Ue(t.a,e.a),e.a}function qet(e,t){return Ue(t.b,e.a),e.a}function ib(e,t){return Ue(t.a,e.a),e.a}function sb(e){return kn(e.a!=null),e.a}function NY(e){Sle.call(this,new z0e(e))}function Yfe(e,t){Wpe.call(this,e,t,null)}function yE(e){this.a=e,kle.call(this,e)}function oP(){oP=re,Uj=new Xi(Evt,0)}function cP(e,t){return++e.b,Ue(e.a,t)}function Wfe(e,t){return++e.b,vc(e.a,t)}function g0n(e,t){return Ui(e.n.a,t.n.a)}function p0n(e,t){return Ui(e.c.d,t.c.d)}function b0n(e,t){return Ui(e.c.c,t.c.c)}function gc(e,t){return h(jr(e.b,t),15)}function v0n(e,t){return e.n.b=(rn(t),t)}function w0n(e,t){return e.n.b=(rn(t),t)}function kE(e){return Wa(e.a)||Wa(e.b)}function m0n(e,t,n){return mat(e,t,n,e.b)}function Kfe(e,t,n){return mat(e,t,n,e.c)}function Xfe(e,t,n){h(_A(e,t),21).Fc(n)}function y0n(e,t,n){gX(e.a,n),PF(e.a,t)}function $S(e,t){Q7(),this.a=e,this.b=t}function uP(e,t){o0(),this.b=e,this.c=t}function PY(e,t){hW(),this.f=t,this.d=e}function Qfe(e,t){P0e(t,e),this.d=e,this.c=t}function c0(e){var t;t=e.a,e.a=e.b,e.b=t}function k0n(e){return wh(),!!e&&!e.dc()}function x0n(e){return new dw(3,e)}function Zfe(e,t){return new Gtt(e,e.gc(),t)}function E0n(e){return aN(),cr((gst(),s3t),e)}function N4(e){this.d=e,Tn.call(this,e)}function P4(e){this.c=e,Tn.call(this,e)}function HS(e){this.c=e,N4.call(this,e)}function Yet(){W7(),this.b=new AYe(this)}function I1(e){return Uu(e,jw),new Wo(e)}function Wet(e){return bk(),parseInt(e)||-1}function $u(e,t,n){return e.substr(t,n-t)}function nk(e,t,n){return Ufe(e,Ec(t),n)}function BY(e){return Ide(e.c,e.c.length)}function _0n(e){return e.f!=null?e.f:""+e.g}function FY(e){return e.f!=null?e.f:""+e.g}function RY(e){return kn(e.b!=0),e.a.a.c}function lP(e){return kn(e.b!=0),e.c.b.c}function zS(e){ue(e,150)&&h(e,150).Gh()}function hP(e){return e.b=h(Tit(e.a),42)}function Jfe(e){X6(),this.b=e,this.a=!0}function Ket(e){yN(),this.b=e,this.a=!0}function Xet(e){e.d=new Zet(e),e.e=new zn}function Qet(e){if(!e)throw ne(new cl)}function e1e(e){if(!e)throw ne(new pS)}function tw(e){if(!e)throw ne(new Ic)}function T0n(e){if(!e)throw ne(new Eq)}function kn(e){if(!e)throw ne(new lo)}function Zet(e){U1e.call(this,e,null,null)}function Jet(){ar.call(this,"POLYOMINO",0)}function ett(e,t,n,s){mde.call(this,e,t,n,s)}function C0n(e,t){return Mg(),Ot(e,t.e,t)}function S0n(e,t,n){return TN(),n.qg(e,t)}function os(e,t){return!!e.q&&Au(e.q,t)}function A0n(e,t){return e>0?t*t/e:t*t*100}function M0n(e,t){return e>0?t/(e*e):t*100}function D0n(e,t,n){return Ue(t,Yht(e,n))}function L0n(e,t,n){CB(),e.Xe(t)&&n.td(e)}function rk(e,t,n){var s;s=e.Zc(t),s.Rb(n)}function nw(e,t,n){return e.a+=t,e.b+=n,e}function I0n(e,t,n){return e.a*=t,e.b*=n,e}function GS(e,t,n){return e.a-=t,e.b-=n,e}function t1e(e,t){return e.a=t.a,e.b=t.b,e}function fP(e){return e.a=-e.a,e.b=-e.b,e}function ttt(e){this.c=e,this.a=1,this.b=1}function ntt(e){this.c=e,yc(e,0),kc(e,0)}function rtt(e){di.call(this),YE(this,e)}function itt(e){uZ(),SKe(this),this.mf(e)}function stt(e,t){Q7(),$S.call(this,e,t)}function n1e(e,t){o0(),uP.call(this,e,t)}function att(e,t){o0(),uP.call(this,e,t)}function ott(e,t){o0(),n1e.call(this,e,t)}function Cu(e,t,n){Du.call(this,e,t,n,2)}function jY(e,t){ul(),AP.call(this,e,t)}function ctt(e,t){ul(),jY.call(this,e,t)}function r1e(e,t){ul(),jY.call(this,e,t)}function utt(e,t){ul(),r1e.call(this,e,t)}function i1e(e,t){ul(),AP.call(this,e,t)}function ltt(e,t){ul(),i1e.call(this,e,t)}function htt(e,t){ul(),AP.call(this,e,t)}function O0n(e,t){return e.c.Fc(h(t,133))}function s1e(e,t,n){return bR(TA(e,t),n)}function N0n(e,t,n){return t.Qk(e.e,e.c,n)}function P0n(e,t,n){return t.Rk(e.e,e.c,n)}function $Y(e,t){return Tg(e.e,h(t,49))}function B0n(e,t,n){s9(cu(e.a),t,xst(n))}function F0n(e,t,n){s9(Co(e.a),t,kst(n))}function a1e(e,t){t.$modCount=e.$modCount}function xE(){xE=re,q_=new Xr("root")}function ik(){ik=re,bL=new sXe,new aXe}function ftt(){this.a=new pb,this.b=new pb}function o1e(){Lut.call(this),this.Bb|=ea}function dtt(){ar.call(this,"GROW_TREE",0)}function R0n(e){return e==null?null:nNn(e)}function j0n(e){return e==null?null:l_n(e)}function $0n(e){return e==null?null:Ga(e)}function H0n(e){return e==null?null:Ga(e)}function yd(e){e.o==null&&RCn(e)}function lt(e){return TE(e==null||Jv(e)),e}function Je(e){return TE(e==null||ew(e)),e}function Jn(e){return TE(e==null||ws(e)),e}function c1e(e){this.q=new d.Date(vb(e))}function VS(e,t){this.c=e,W6.call(this,e,t)}function dP(e,t){this.a=e,VS.call(this,e,t)}function z0n(e,t){this.d=e,kqe(this),this.b=t}function u1e(e,t){vK.call(this,e),this.a=t}function l1e(e,t){vK.call(this,e),this.a=t}function G0n(e){Fpe.call(this,0,0),this.f=e}function h1e(e,t,n){_B.call(this,e,t,n,null)}function gtt(e,t,n){_B.call(this,e,t,n,null)}function V0n(e,t,n){return e.ue(t,n)<=0?n:t}function U0n(e,t,n){return e.ue(t,n)<=0?t:n}function q0n(e,t){return h(mb(e.b,t),149)}function Y0n(e,t){return h(mb(e.c,t),229)}function HY(e){return h(ut(e.a,e.b),287)}function ptt(e){return new dt(e.c,e.d+e.a)}function btt(e){return Hu(),QJe(h(e,197))}function rw(){rw=re,y4e=Lt((Iu(),E2))}function W0n(e,t){t.a?OAn(e,t):OY(e.a,t.b)}function vtt(e,t){Pf||Ue(e.a,t)}function K0n(e,t){return ES(),Fk(t.d.i,e)}function X0n(e,t){return t5(),new mpt(t,e)}function e1(e,t){return aA(t,gwe),e.f=t,e}function f1e(e,t,n){return n=Ku(e,t,3,n),n}function d1e(e,t,n){return n=Ku(e,t,6,n),n}function g1e(e,t,n){return n=Ku(e,t,9,n),n}function US(e,t,n){++e.j,e.Ki(),pK(e,t,n)}function wtt(e,t,n){++e.j,e.Hi(t,e.oi(t,n))}function mtt(e,t,n){var s;s=e.Zc(t),s.Rb(n)}function ytt(e,t,n){return Ave(e.c,e.b,t,n)}function p1e(e,t){return(t&Ir)%e.d.length}function Xi(e,t){Xr.call(this,e),this.a=t}function b1e(e,t){Fle.call(this,e),this.a=t}function zY(e,t){Fle.call(this,e),this.a=t}function ktt(e,t){this.c=e,yb.call(this,t)}function xtt(e,t){this.a=e,kKe.call(this,t)}function qS(e,t){this.a=e,kKe.call(this,t)}function Ett(e){this.a=(Uu(e,jw),new Wo(e))}function _tt(e){this.a=(Uu(e,jw),new Wo(e))}function YS(e){return!e.a&&(e.a=new F),e.a}function Ttt(e){return e>8?0:e+1}function Q0n(e,t){return on(),e==t?0:e?1:-1}function v1e(e,t,n){return j4(e,h(t,22),n)}function Z0n(e,t,n){return e.apply(t,n)}function Ctt(e,t,n){return e.a+=Fl(t,0,n),e}function w1e(e,t){var n;return n=e.e,e.e=t,n}function J0n(e,t){var n;n=e[GZ],n.call(e,t)}function egn(e,t){var n;n=e[GZ],n.call(e,t)}function iw(e,t){e.a.Vc(e.b,t),++e.b,e.c=-1}function Stt(e){Kc(e.e),e.d.b=e.d,e.d.a=e.d}function WS(e){e.b?WS(e.b):e.f.c.zc(e.e,e.d)}function tgn(e,t,n){Tp(),wqe(e,t.Ce(e.a,n))}function ngn(e,t){return zq(uft(e.a,t,!0))}function rgn(e,t){return zq(lft(e.a,t,!0))}function vh(e,t){return hN(new Array(t),e)}function GY(e){return String.fromCharCode(e)}function ign(e){return e==null?null:e.message}function Att(){this.a=new qe,this.b=new qe}function Mtt(){this.a=new tS,this.b=new BKe}function Dtt(){this.b=new Rs,this.c=new qe}function m1e(){this.d=new Rs,this.e=new Rs}function y1e(){this.n=new Rs,this.o=new Rs}function gP(){this.n=new E4,this.i=new O4}function Ltt(){this.a=new cUe,this.b=new Kje}function Itt(){this.a=new qe,this.d=new qe}function Ott(){this.b=new ns,this.a=new ns}function Ntt(){this.b=new zn,this.a=new zn}function Ptt(){this.b=new dQe,this.a=new Q$e}function Btt(){gP.call(this),this.a=new Rs}function EE(e){M5n.call(this,e,(AB(),ate))}function k1e(e,t,n,s){MP.call(this,e,t,n,s)}function sgn(e,t,n){n!=null&&UB(t,qX(e,n))}function agn(e,t,n){n!=null&&qB(t,qX(e,n))}function x1e(e,t,n){return n=Ku(e,t,11,n),n}function zr(e,t){return e.a+=t.a,e.b+=t.b,e}function ms(e,t){return e.a-=t.a,e.b-=t.b,e}function ogn(e,t){return e.n.a=(rn(t),t+10)}function cgn(e,t){return e.n.a=(rn(t),t+10)}function ugn(e,t){return t==e||ix(tR(t),e)}function Ftt(e,t){return Pr(e.a,t,"")==null}function lgn(e,t){return ES(),!Fk(t.d.i,e)}function hgn(e,t){lg(e.f)?ACn(e,t):f9n(e,t)}function fgn(e,t){var n;return n=t.Hh(e.a),n}function sw(e,t){Ea.call(this,n_+e+s2+t)}function B4(e,t,n,s){Ye.call(this,e,t,n,s)}function E1e(e,t,n,s){Ye.call(this,e,t,n,s)}function Rtt(e,t,n,s){E1e.call(this,e,t,n,s)}function jtt(e,t,n,s){jP.call(this,e,t,n,s)}function VY(e,t,n,s){jP.call(this,e,t,n,s)}function _1e(e,t,n,s){jP.call(this,e,t,n,s)}function $tt(e,t,n,s){VY.call(this,e,t,n,s)}function T1e(e,t,n,s){VY.call(this,e,t,n,s)}function qt(e,t,n,s){_1e.call(this,e,t,n,s)}function Htt(e,t,n,s){T1e.call(this,e,t,n,s)}function ztt(e,t,n,s){Ede.call(this,e,t,n,s)}function Gtt(e,t,n){this.a=e,Qfe.call(this,t,n)}function Vtt(e,t,n){this.c=t,this.b=n,this.a=e}function dgn(e,t,n){return e.d=h(t.Kb(n),164)}function C1e(e,t){return e.Aj().Nh().Kh(e,t)}function S1e(e,t){return e.Aj().Nh().Ih(e,t)}function Utt(e,t){return rn(e),Me(e)===Me(t)}function Nt(e,t){return rn(e),Me(e)===Me(t)}function UY(e,t){return zq(uft(e.a,t,!1))}function qY(e,t){return zq(lft(e.a,t,!1))}function ggn(e,t){return e.b.sd(new xZe(e,t))}function pgn(e,t){return e.b.sd(new EZe(e,t))}function qtt(e,t){return e.b.sd(new _Ze(e,t))}function A1e(e,t,n){return e.lastIndexOf(t,n)}function bgn(e,t,n){return Ui(e[t.b],e[n.b])}function vgn(e,t){return Fe(t,(tt(),ID),e)}function wgn(e,t){return dc(t.a.d.p,e.a.d.p)}function mgn(e,t){return dc(e.a.d.p,t.a.d.p)}function ygn(e,t){return Ui(e.c-e.s,t.c-t.s)}function Ytt(e){return e.c?Ha(e.c.a,e,0):-1}function kgn(e){return e<100?null:new ig(e)}function F4(e){return e==x2||e==ed||e==qo}function Wtt(e,t){return ue(t,15)&&ept(e.c,t)}function xgn(e,t){Pf||t&&(e.d=t)}function YY(e,t){var n;return n=t,!!Qge(e,n)}function M1e(e,t){this.c=e,yW.call(this,e,t)}function Ktt(e){this.c=e,xY.call(this,_R,0)}function Xtt(e,t){S2n.call(this,e,e.length,t)}function Egn(e,t,n){return h(e.c,69).lk(t,n)}function pP(e,t,n){return h(e.c,69).mk(t,n)}function _gn(e,t,n){return N0n(e,h(t,332),n)}function D1e(e,t,n){return P0n(e,h(t,332),n)}function Tgn(e,t,n){return W1t(e,h(t,332),n)}function Qtt(e,t,n){return E9n(e,h(t,332),n)}function _E(e,t){return t==null?null:xw(e.b,t)}function L1e(e){return ew(e)?(rn(e),e):e.ke()}function bP(e){return!isNaN(e)&&!isFinite(e)}function Ztt(e){t1(),this.a=(Rt(),new G6(e))}function KS(e){ak(),this.d=e,this.a=new Jm}function ll(e,t,n){this.a=e,this.b=t,this.c=n}function Jtt(e,t,n){this.a=e,this.b=t,this.c=n}function ent(e,t,n){this.d=e,this.b=n,this.a=t}function WY(e){fet(this),Pl(this),Qs(this,e)}function Bc(e){SY(this),q1e(this.c,0,e.Pc())}function tnt(e){Mu(e.a),Zot(e.c,e.b),e.b=null}function nnt(e){this.a=e,L1(),xc(Date.now())}function rnt(){rnt=re,Q3e=new y,zj=new y}function KY(){KY=re,z3e=new _e,x3t=new Te}function int(){int=re,D9t=pe(yn,st,1,0,5,1)}function snt(){snt=re,W9t=pe(yn,st,1,0,5,1)}function I1e(){I1e=re,K9t=pe(yn,st,1,0,5,1)}function t1(){t1=re,new $le((Rt(),Rt(),ca))}function Cgn(e){return AB(),cr((uct(),T3t),e)}function Sgn(e){return Af(),cr((Aot(),L3t),e)}function Agn(e){return SF(),cr((Pat(),F3t),e)}function Mgn(e){return OB(),cr((Bat(),R3t),e)}function Dgn(e){return oR(),cr((xlt(),j3t),e)}function Lgn(e){return Kh(),cr((Tot(),z3t),e)}function Ign(e){return Xc(),cr((Cot(),V3t),e)}function Ogn(e){return wc(),cr((Sot(),q3t),e)}function Ngn(e){return xR(),cr((BJe(),p4t),e)}function Pgn(e){return Cb(),cr((hct(),v4t),e)}function Bgn(e){return o5(),cr((fct(),m4t),e)}function Fgn(e){return h9(),cr((dct(),x4t),e)}function Rgn(e){return kN(),cr((dat(),E4t),e)}function jgn(e){return NB(),cr((Fat(),$4t),e)}function $gn(e){return UE(),cr((Mot(),o5t),e)}function Hgn(e){return Zs(),cr((Gct(),h5t),e)}function zgn(e){return $k(),cr((lct(),b5t),e)}function Ggn(e){return Sb(),cr((Dot(),x5t),e)}function O1e(e,t){if(!e)throw ne(new an(t))}function Vgn(e){return bn(),cr((lut(),C5t),e)}function N1e(e){MP.call(this,e.d,e.c,e.a,e.b)}function XY(e){MP.call(this,e.d,e.c,e.a,e.b)}function P1e(e,t,n){this.b=e,this.c=t,this.a=n}function vP(e,t,n){this.b=e,this.a=t,this.c=n}function ant(e,t,n){this.a=e,this.b=t,this.c=n}function B1e(e,t,n){this.a=e,this.b=t,this.c=n}function ont(e,t,n){this.a=e,this.b=t,this.c=n}function F1e(e,t,n){this.a=e,this.b=t,this.c=n}function cnt(e,t,n){this.b=e,this.a=t,this.c=n}function wP(e,t,n){this.e=t,this.b=e,this.d=n}function Ugn(e,t,n){return Tp(),e.a.Od(t,n),t}function QY(e){var t;return t=new nn,t.e=e,t}function R1e(e){var t;return t=new GKe,t.b=e,t}function XS(){XS=re,t$=new _Fe,n$=new TFe}function wh(){wh=re,$5t=new oje,H5t=new cje}function qgn(e){return aF(),cr((pct(),P5t),e)}function Ygn(e){return Sf(),cr((vct(),V5t),e)}function Wgn(e){return sR(),cr((hlt(),Z5t),e)}function Kgn(e){return u5(),cr((dut(),J5t),e)}function Xgn(e){return TB(),cr((Gat(),e6t),e)}function Qgn(e){return e5(),cr((Lot(),t6t),e)}function Zgn(e){return Iy(),cr((Rct(),q5t),e)}function Jgn(e){return Tb(),cr((Not(),Q5t),e)}function epn(e){return GB(),cr((Iot(),n6t),e)}function tpn(e){return Kp(),cr((Bct(),r6t),e)}function npn(e){return DA(),cr((jat(),i6t),e)}function rpn(e){return Hp(),cr((Oot(),a6t),e)}function ipn(e){return KF(),cr((vut(),o6t),e)}function spn(e){return xA(),cr(($at(),c6t),e)}function apn(e){return cM(),cr((put(),u6t),e)}function opn(e){return ax(),cr((gut(),l6t),e)}function cpn(e){return ha(),cr((Flt(),h6t),e)}function upn(e){return jk(),cr((Bot(),f6t),e)}function lpn(e){return Ad(),cr((Pot(),g6t),e)}function hpn(e){return gB(),cr((Vat(),p6t),e)}function fpn(e){return wl(),cr((jct(),b6t),e)}function dpn(e){return UF(),cr((but(),Lxt),e)}function gpn(e){return t9(),cr((Fot(),Ixt),e)}function ppn(e){return Tw(),cr((wct(),Oxt),e)}function bpn(e){return ua(),cr(($ot(),jxt),e)}function vpn(e){return By(),cr((llt(),Pxt),e)}function wpn(e){return Dd(),cr((jot(),Bxt),e)}function mpn(e){return CA(),cr((zat(),Fxt),e)}function ypn(e){return nF(),cr((Rot(),$xt),e)}function kpn(e){return f9(),cr((Fct(),Nxt),e)}function xpn(e){return bA(),cr((Hat(),Hxt),e)}function Epn(e){return qk(),cr((zot(),zxt),e)}function _pn(e){return eF(),cr((Got(),Gxt),e)}function Tpn(e){return oF(),cr((Hot(),Vxt),e)}function Cpn(e){return _b(),cr((Vot(),r8t),e)}function Spn(e){return GE(),cr((qat(),c8t),e)}function Apn(e){return r1(),cr((Yat(),p8t),e)}function Mpn(e){return yf(),cr((Wat(),v8t),e)}function Dpn(e){return qh(),cr((Uat(),I8t),e)}function Lpn(e){return gb(),cr((Kat(),j8t),e)}function Ipn(e){return tx(),cr((gct(),$8t),e)}function Opn(e){return w9(),cr((wut(),z8t),e)}function Npn(e){return aB(),cr((Zat(),n7t),e)}function Ppn(e){return QB(),cr((Qat(),c7t),e)}function Bpn(e){return lB(),cr((Xat(),r7t),e)}function Fpn(e){return wF(),cr((Uot(),l7t),e)}function Rpn(e){return SB(),cr((Jat(),h7t),e)}function jpn(e){return VA(),cr((qot(),f7t),e)}function $pn(e){return FF(),cr((bct(),S7t),e)}function Hpn(e){return tF(),cr((Wot(),A7t),e)}function zpn(e){return vF(),cr((Yot(),M7t),e)}function Gpn(e){return fx(),cr((zct(),K7t),e)}function Vpn(e){return XA(),cr((Kot(),X7t),e)}function Upn(e){return EN(),cr((hat(),Q7t),e)}function qpn(e){return _N(),cr((lat(),J7t),e)}function Ypn(e){return vA(),cr((tot(),eEt),e)}function Wpn(e){return lM(),cr(($ct(),tEt),e)}function Kpn(e){return K7(),cr((fat(),vEt),e)}function Xpn(e){return $A(),cr((eot(),wEt),e)}function Qpn(e){return Qh(),cr((Hct(),_Et),e)}function Zpn(e){return E0(),cr((flt(),CEt),e)}function Jpn(e){return $1(),cr((fut(),SEt),e)}function e2n(e){return Aw(),cr((hut(),OEt),e)}function t2n(e){return oa(),cr((FJe(),I5t),e)}function n2n(e){return Nk(),cr((Rat(),L5t),e)}function r2n(e){return la(),cr((Vct(),WEt),e)}function i2n(e){return Tf(),cr((Qot(),KEt),e)}function s2n(e){return Od(),cr((kct(),XEt),e)}function a2n(e){return YF(),cr((yut(),QEt),e)}function o2n(e){return Ld(),cr((Xot(),JEt),e)}function c2n(e){return Yu(),cr((yct(),t9t),e)}function u2n(e){return Iw(),cr((klt(),n9t),e)}function l2n(e){return Cy(),cr((Uct(),r9t),e)}function h2n(e){return _s(),cr((aut(),i9t),e)}function f2n(e){return Qc(),cr((mut(),s9t),e)}function d2n(e){return Iu(),cr((Ect(),h9t),e)}function g2n(e){return fu(),cr((Rlt(),f9t),e)}function p2n(e){return Qe(),cr((qct(),a9t),e)}function b2n(e){return kF(),cr((xct(),d9t),e)}function v2n(e){return Lu(),cr((mct(),b9t),e)}function w2n(e){return ux(),cr((dlt(),M9t),e)}function m2n(e,t){return rn(e),e+(rn(t),t)}function y2n(e,t){return L1(),qn(De(e.a),t)}function k2n(e,t){return L1(),qn(De(e.a),t)}function ZY(e,t){this.c=e,this.a=t,this.b=t-e}function unt(e,t,n){this.a=e,this.b=t,this.c=n}function j1e(e,t,n){this.a=e,this.b=t,this.c=n}function $1e(e,t,n){this.a=e,this.b=t,this.c=n}function lnt(e,t,n){this.a=e,this.b=t,this.c=n}function hnt(e,t,n){this.a=e,this.b=t,this.c=n}function u0(e,t,n){this.e=e,this.a=t,this.c=n}function fnt(e,t,n){ul(),Zde.call(this,e,t,n)}function JY(e,t,n){ul(),Bde.call(this,e,t,n)}function H1e(e,t,n){ul(),Bde.call(this,e,t,n)}function z1e(e,t,n){ul(),Bde.call(this,e,t,n)}function dnt(e,t,n){ul(),JY.call(this,e,t,n)}function G1e(e,t,n){ul(),JY.call(this,e,t,n)}function gnt(e,t,n){ul(),G1e.call(this,e,t,n)}function pnt(e,t,n){ul(),H1e.call(this,e,t,n)}function bnt(e,t,n){ul(),z1e.call(this,e,t,n)}function QS(e,t){return Vn(e),Vn(t),new PQe(e,t)}function R4(e,t){return Vn(e),Vn(t),new Mnt(e,t)}function x2n(e,t){return Vn(e),Vn(t),new Dnt(e,t)}function E2n(e,t){return Vn(e),Vn(t),new FQe(e,t)}function h(e,t){return TE(e==null||PX(e,t)),e}function sk(e){var t;return t=new qe,AK(t,e),t}function _2n(e){var t;return t=new ns,AK(t,e),t}function vnt(e){var t;return t=new Kle,zK(t,e),t}function ZS(e){var t;return t=new di,zK(t,e),t}function T2n(e){return!e.e&&(e.e=new qe),e.e}function C2n(e){return!e.c&&(e.c=new N6),e.c}function Ue(e,t){return e.c[e.c.length]=t,!0}function wnt(e,t){this.c=e,this.b=t,this.a=!1}function V1e(e){this.d=e,kqe(this),this.b=wbn(e.d)}function mnt(){this.a=";,;",this.b="",this.c=""}function S2n(e,t,n){yrt.call(this,t,n),this.a=e}function ynt(e,t,n){this.b=e,IJe.call(this,t,n)}function U1e(e,t,n){this.c=e,LN.call(this,t,n)}function q1e(e,t,n){gbe(n,0,e,t,n.length,!1)}function O1(e,t,n,s,a){e.b=t,e.c=n,e.d=s,e.a=a}function A2n(e,t){t&&(e.b=t,e.a=(vg(t),t.a))}function Y1e(e,t,n,s,a){e.d=t,e.c=n,e.a=s,e.b=a}function W1e(e){var t,n;t=e.b,n=e.c,e.b=n,e.c=t}function K1e(e){var t,n;n=e.d,t=e.a,e.d=t,e.a=n}function X1e(e){return xg(Nbn($a(e)?Bl(e):e))}function M2n(e,t){return dc(Bnt(e.d),Bnt(t.d))}function D2n(e,t){return t==(Qe(),un)?e.c:e.d}function ak(){ak=re,bxe=(Qe(),un),oH=pn}function knt(){this.b=Pe(Je(at((Jh(),Ete))))}function xnt(e){return Tp(),pe(yn,st,1,e,5,1)}function L2n(e){return new dt(e.c+e.b,e.d+e.a)}function I2n(e,t){return xN(),dc(e.d.p,t.d.p)}function eW(e){return kn(e.b!=0),pl(e,e.a.a)}function O2n(e){return kn(e.b!=0),pl(e,e.c.b)}function Q1e(e,t){if(!e)throw ne(new MXe(t))}function mP(e,t){if(!e)throw ne(new an(t))}function Z1e(e,t,n){Z6.call(this,e,t),this.b=n}function JS(e,t,n){dfe.call(this,e,t),this.c=n}function Ent(e,t,n){eut.call(this,t,n),this.d=e}function J1e(e){I1e(),qO.call(this),this.th(e)}function _nt(e,t,n){this.a=e,ay.call(this,t,n)}function Tnt(e,t,n){this.a=e,ay.call(this,t,n)}function yP(e,t,n){dfe.call(this,e,t),this.c=n}function Cnt(){kk(),Zbn.call(this,(og(),th))}function Snt(e){return e!=null&&!kX(e,lT,hT)}function N2n(e,t){return(Bht(e)<<4|Bht(t))&Oi}function P2n(e,t){return VP(),GX(e,t),new eit(e,t)}function Sp(e,t){var n;e.n&&(n=t,Ue(e.f,n))}function ok(e,t,n){var s;s=new uw(n),Wh(e,t,s)}function B2n(e,t){var n;return n=e.c,vge(e,t),n}function ede(e,t){return t<0?e.g=-1:e.g=t,e}function kP(e,t){return y4n(e),e.a*=t,e.b*=t,e}function Ant(e,t,n,s,a){e.c=t,e.d=n,e.b=s,e.a=a}function vr(e,t){return Ai(e,t,e.c.b,e.c),!0}function tde(e){e.a.b=e.b,e.b.a=e.a,e.a=e.b=null}function tW(e){this.b=e,this.a=ob(this.b.a).Ed()}function Mnt(e,t){this.b=e,this.a=t,dq.call(this)}function Dnt(e,t){this.a=e,this.b=t,dq.call(this)}function Lnt(e,t){yrt.call(this,t,1040),this.a=e}function eA(e){return e==0||isNaN(e)?e:e<0?-1:1}function F2n(e){return V4(),H1(e)==bi(Sg(e))}function R2n(e){return V4(),Sg(e)==bi(H1(e))}function ab(e,t){return b9(e,new Z6(t.a,t.b))}function j2n(e){return!Xs(e)&&e.c.i.c==e.d.i.c}function xP(e){var t;return t=e.n,e.a.b+t.d+t.a}function Int(e){var t;return t=e.n,e.e.b+t.d+t.a}function nde(e){var t;return t=e.n,e.e.a+t.b+t.c}function Ont(e){return Mr(),new N1(0,e)}function $2n(e){return e.a?e.a:PW(e)}function TE(e){if(!e)throw ne(new z6(null))}function Nnt(){Nnt=re,oie=(Rt(),new mq(Bee))}function EP(){EP=re,new Jpe((Dq(),Wee),(Lq(),Yee))}function Pnt(){Pnt=re,T3e=pe(Ws,Re,19,256,0,1)}function nW(e,t,n,s){Ape.call(this,e,t,n,s,0,0)}function H2n(e,t,n){return Pr(e.b,h(n.b,17),t)}function z2n(e,t,n){return Pr(e.b,h(n.b,17),t)}function G2n(e,t){return Ue(e,new dt(t.a,t.b))}function V2n(e,t){return e.c<t.c?-1:e.c==t.c?0:1}function rW(e){return e.e.c.length+e.g.c.length}function Bnt(e){return e.e.c.length-e.g.c.length}function Fnt(e){return e.b.c.length-e.e.c.length}function U2n(e){return Hu(),(Qe(),uc).Hc(e.j)}function Rnt(e){I1e(),J1e.call(this,e),this.a=-1}function _P(e,t){RS.call(this,e,t),this.a=this}function fi(e,t){var n;return n=_W(e,t),n.i=2,n}function TP(e,t){var n;return++e.j,n=e.Ti(t),n}function Lr(e,t,n){return e.a=-1,Xfe(e,t.g,n),e}function q2n(e,t,n){FOn(e.a,e.b,e.c,h(t,202),n)}function Y2n(e,t){mge(e,t==null?null:(rn(t),t))}function W2n(e,t){bge(e,t==null?null:(rn(t),t))}function K2n(e,t){bge(e,t==null?null:(rn(t),t))}function iW(e,t,n){return new Vtt(Jbn(e).Ie(),n,t)}function Ap(e,t,n,s,a,u){return z1t(e,t,n,s,a,0,u)}function jnt(){jnt=re,x3e=pe(A5,Re,217,256,0,1)}function $nt(){$nt=re,C3e=pe(Xb,Re,162,256,0,1)}function Hnt(){Hnt=re,M3e=pe(Qb,Re,184,256,0,1)}function znt(){znt=re,_3e=pe(f_,Re,172,128,0,1)}function rde(){O1(this,!1,!1,!1,!1)}function sW(e){lw(),this.a=(Rt(),new mq(Vn(e)))}function CP(e){for(Vn(e);e.Ob();)e.Pb(),e.Qb()}function X2n(e){e.a.cd(),h(e.a.dd(),14).gc(),phe()}function ide(e){this.c=e,this.b=this.c.d.vc().Kc()}function Gnt(e){this.c=e,this.a=new A4(this.c.a)}function CE(e){this.a=new S4(e.gc()),Qs(this,e)}function sde(e){Sle.call(this,new Np),Qs(this,e)}function Vnt(e,t){return e.a+=Fl(t,0,t.length),e}function ut(e,t){return Kt(t,e.c.length),e.c[t]}function Unt(e,t){return Kt(t,e.a.length),e.a[t]}function Vt(e,t){Tp(),vK.call(this,e),this.a=t}function Q2n(e,t){return qp(Vs(qp(e.a).a,t.a))}function Z2n(e,t){return rn(e),OA(e,(rn(t),t))}function J2n(e,t){return rn(t),OA(t,(rn(e),e))}function ebn(e,t){return pi(t,0,ade(t[0],qp(1)))}function ade(e,t){return Q2n(h(e,162),h(t,162))}function qnt(e){return e.c-h(ut(e.a,e.b),287).b}function Ynt(e){return e.q?e.q:(Rt(),Rt(),K1)}function Wnt(e){return e.e.Hd().gc()*e.c.Hd().gc()}function tbn(e,t,n){return dc(t.d[e.g],n.d[e.g])}function nbn(e,t,n){return dc(e.d[t.p],e.d[n.p])}function rbn(e,t,n){return dc(e.d[t.p],e.d[n.p])}function ibn(e,t,n){return dc(e.d[t.p],e.d[n.p])}function sbn(e,t,n){return dc(e.d[t.p],e.d[n.p])}function ode(e,t,n){return d.Math.min(n/e,1/t)}function Knt(e,t){return e?0:d.Math.max(0,t-1)}function abn(e,t){var n;for(n=0;n<t;++n)e[n]=-1}function Xnt(e){var t;return t=V1t(e),t?Xnt(t):e}function obn(e,t){return e.a==null&&spt(e),e.a[t]}function Su(e){return e.c?e.c.f:e.e.b}function Fc(e){return e.c?e.c.g:e.e.a}function SP(e){yb.call(this,e.gc()),mi(this,e)}function AP(e,t){ul(),eN.call(this,t),this.a=e}function SE(e,t,n){this.a=e,Gi.call(this,t,n,2)}function MP(e,t,n,s){Y1e(this,e,t,n,s)}function N1(e,t){Mr(),G2.call(this,e),this.a=t}function Qnt(e){this.b=new di,this.a=e,this.c=-1}function Znt(){this.d=new dt(0,0),this.e=new ns}function Jnt(e){Qfe.call(this,0,0),this.a=e,this.b=0}function ert(e){this.a=e,this.c=new zn,Z5n(this)}function cde(e){if(e.e.c!=e.b)throw ne(new cl)}function ude(e){if(e.c.e!=e.a)throw ne(new cl)}function Gn(e){return $a(e)?e|0:uQe(e)}function DP(e,t){return Mr(),new Lde(e,t)}function aW(e,t){return e==null?t==null:Nt(e,t)}function cbn(e,t){return e==null?t==null:CF(e,t)}function j4(e,t,n){return Eh(e.a,t),Sde(e,t.g,n)}function oW(e,t,n){lht(0,t,e.length),_k(e,0,t,n)}function aw(e,t,n){fw(t,e.c.length),J7(e.c,t,n)}function trt(e,t,n){var s;for(s=0;s<t;++s)e[s]=n}function ni(e,t){var n;return n=Lt(e),Uge(n,t),n}function ubn(e,t){return!e&&(e=[]),e[e.length]=t,e}function lbn(e,t){return e.a.get(t)!==void 0}function nrt(e,t){return R4n(new jt,new Aqe(e),t)}function hbn(e){return e==null?$j:new oN(rn(e))}function cW(e,t){return ue(t,22)&&Pc(e,h(t,22))}function rrt(e,t){return ue(t,22)&&g4n(e,h(t,22))}function lde(e){return hu(e,26)*P9+hu(e,27)*B9}function hde(e){return Array.isArray(e)&&e.im===de}function LP(e){e.b?LP(e.b):e.d.dc()&&e.f.c.Bc(e.e)}function uW(e,t){zr(e.c,t),e.b.c+=t.a,e.b.d+=t.b}function fbn(e,t){uW(e,ms(new dt(t.a,t.b),e.c))}function lW(e,t){this.b=new di,this.a=e,this.c=t}function irt(){this.b=new sS,this.c=new Pit(this)}function fde(){this.d=new Fr,this.e=new Nit(this)}function dde(){i0e(),this.f=new di,this.e=new di}function srt(){Hu(),this.k=new zn,this.d=new ns}function hW(){hW=re,w9t=new sa((Tr(),pu),0)}function art(){art=re,r3t=new Jnt(pe(yn,st,1,0,5,1))}function dbn(e,t,n){Z0t(n,e,1),Ue(t,new AZe(n,e))}function gbn(e,t,n){g9(n,e,1),Ue(t,new PZe(n,e))}function pbn(e,t,n){return Qi(e,new M4(t.a,n.a))}function bbn(e,t,n){return-dc(e.f[t.p],e.f[n.p])}function IP(e,t,n){var s;e&&(s=e.i,s.c=t,s.b=n)}function OP(e,t,n){var s;e&&(s=e.i,s.d=t,s.a=n)}function Wc(e,t,n){return e.a=-1,Xfe(e,t.g+1,n),e}function gde(e,t,n){return n=Ku(e,h(t,49),7,n),n}function pde(e,t,n){return n=Ku(e,h(t,49),3,n),n}function ort(e,t,n){this.a=e,iP.call(this,t,n,22)}function crt(e,t,n){this.a=e,iP.call(this,t,n,14)}function urt(e,t,n,s){ul(),Gst.call(this,e,t,n,s)}function lrt(e,t,n,s){ul(),Gst.call(this,e,t,n,s)}function vbn(e,t){(t.Bb&go)!=0&&!e.a.o&&(e.a.o=t)}function hrt(e){return e!=null&&mW(e)&&e.im!==de}function bde(e){return!Array.isArray(e)&&e.im===de}function wbn(e){return ue(e,15)?h(e,15).Yc():e.Kc()}function vde(e){return e.Qc(pe(yn,st,1,e.gc(),5,1))}function AE(e,t){return Ixn(TA(e,t))?t.Qh():null}function wde(e){e?c2e(e,(L1(),L3e)):L1()}function Sn(e){this.a=(art(),r3t),this.d=h(Vn(e),47)}function mde(e,t,n,s){this.a=e,_B.call(this,e,t,n,s)}function Mp(e){cg(),this.a=0,this.b=e-1,this.c=1}function frt(e){nP(this),this.g=e,KP(this),this._d()}function ob(e){return e.c?e.c:e.c=e.Id()}function fW(e){return e.d?e.d:e.d=e.Jd()}function yde(e){var t;return t=e.c,t||(e.c=e.Dd())}function drt(e){var t;return t=e.f,t||(e.f=e.Dc())}function $4(e){var t;return t=e.i,t||(e.i=e.bc())}function grt(e){return Mr(),new GW(10,e,0)}function ME(e){return $a(e)?""+e:Jgt(e)}function NP(e){if(e.e.j!=e.d)throw ne(new cl)}function kd(e,t){return xg(ydt($a(e)?Bl(e):e,t))}function dg(e,t){return xg(Dbe($a(e)?Bl(e):e,t))}function ow(e,t){return xg(y_n($a(e)?Bl(e):e,t))}function mbn(e,t){return Q0n((rn(e),e),(rn(t),t))}function dW(e,t){return Ui((rn(e),e),(rn(t),t))}function prt(e,t){return Vn(t),e.a.Ad(t)&&!e.b.Ad(t)}function ybn(e,t){return rc(e.l&t.l,e.m&t.m,e.h&t.h)}function kbn(e,t){return rc(e.l|t.l,e.m|t.m,e.h|t.h)}function xbn(e,t){return rc(e.l^t.l,e.m^t.m,e.h^t.h)}function PP(e,t){return WA(e,(rn(t),new Cqe(t)))}function BP(e,t){return WA(e,(rn(t),new Sqe(t)))}function brt(e){return Rc(),h(e,11).e.c.length!=0}function vrt(e){return Rc(),h(e,11).g.c.length!=0}function Ebn(e,t){return t5(),Ui(t.a.o.a,e.a.o.a)}function wrt(e,t,n){return aNn(e,h(t,11),h(n,11))}function _bn(e){return e.e?F0e(e.e):null}function kde(e){e.d||(e.d=e.b.Kc(),e.c=e.b.gc())}function Tbn(e,t,n){e.a.Mb(n)&&(e.b=!0,t.td(n))}function ck(e,t){if(e<0||e>=t)throw ne(new Ule)}function Cbn(e,t,n){return pi(t,0,ade(t[0],n[0])),t}function Sbn(e,t,n){t.Ye(n,Pe(Je(_n(e.b,n)))*e.a)}function mrt(e,t,n){return Ry(),Pk(e,t)&&Pk(e,n)}function DE(e){return Qc(),!e.Hc(Ud)&&!e.Hc(Zg)}function FP(e){return new dt(e.c+e.b/2,e.d+e.a/2)}function gW(e,t){return t.kh()?Tg(e.b,h(t,49)):t}function xde(e,t){this.e=e,this.d=(t&64)!=0?t|a1:t}function yrt(e,t){this.c=0,this.d=e,this.b=t|64|a1}function RP(e){this.b=new Wo(11),this.a=(gy(),e)}function pW(e){this.b=null,this.a=(gy(),e||j3e)}function krt(e){this.a=Vft(e.a),this.b=new Bc(e.b)}function xrt(e){this.b=e,N4.call(this,e),_et(this)}function Ert(e){this.b=e,HS.call(this,e),Tet(this)}function cw(e,t,n){this.a=e,B4.call(this,t,n,5,6)}function Ede(e,t,n,s){this.b=e,Gi.call(this,t,n,s)}function fs(e,t,n,s,a){yK.call(this,e,t,n,s,a,-1)}function LE(e,t,n,s,a){yA.call(this,e,t,n,s,a,-1)}function Ye(e,t,n,s){Gi.call(this,e,t,n),this.b=s}function jP(e,t,n,s){JS.call(this,e,t,n),this.b=s}function _rt(e){TJe.call(this,e,!1),this.a=!1}function Trt(e,t){this.b=e,iqe.call(this,e.b),this.a=t}function Crt(e,t){lw(),D1n.call(this,e,dF(new _u(t)))}function $P(e,t){return Mr(),new Fde(e,t,0)}function bW(e,t){return Mr(),new Fde(6,e,t)}function Abn(e,t){return Nt(e.substr(0,t.length),t)}function Au(e,t){return ws(t)?RW(e,t):!!Ia(e.f,t)}function Ls(e,t){for(rn(t);e.Ob();)t.td(e.Pb())}function fy(e,t,n){Dg(),this.e=e,this.d=t,this.a=n}function l0(e,t,n,s){var a;a=e.i,a.i=t,a.a=n,a.b=s}function _de(e){var t;for(t=e;t.f;)t=t.f;return t}function H4(e){var t;return t=e9(e),kn(t!=null),t}function Mbn(e){var t;return t=gkn(e),kn(t!=null),t}function uk(e,t){var n;return n=e.a.gc(),P0e(t,n),n-t}function Tde(e,t){var n;for(n=0;n<t;++n)e[n]=!1}function Dbn(e,t,n,s){var a;for(a=t;a<n;++a)e[a]=s}function ye(e,t,n,s){lht(t,n,e.length),Dbn(e,t,n,s)}function Lbn(e,t,n){ck(n,e.a.c.length),dl(e.a,n,t)}function Cde(e,t,n){this.c=e,this.a=t,Rt(),this.b=n}function Sde(e,t,n){var s;return s=e.b[t],e.b[t]=n,s}function Qi(e,t){var n;return n=e.a.zc(t,e),n==null}function Ibn(e){if(!e)throw ne(new lo);return e.d}function Ade(e,t){if(e==null)throw ne(new T4(t))}function Mde(e,t){return t?Qs(e,t):!1}function P1(e,t,n){return C6n(e,t.g,n),Eh(e.c,t),e}function Obn(e){return zy(e,(la(),Wl)),e.d=!0,e}function vW(e){return!e.j&&ffn(e,ZMn(e.g,e.b)),e.j}function IE(e){tw(e.b!=-1),f0(e.c,e.a=e.b),e.b=-1}function Kc(e){e.f=new YJe(e),e.g=new WJe(e),JP(e)}function wW(e){return new Vt(null,Rbn(e,e.length))}function gg(e){return new Sn(new Ffe(e.a.length,e.a))}function Nbn(e){return rc(~e.l&du,~e.m&du,~e.h&Rd)}function mW(e){return typeof e===LM||typeof e===fZ}function Pbn(e){return e==ki?cD:e==Fi?"-INF":""+e}function Bbn(e){return e==ki?cD:e==Fi?"-INF":""+e}function Fbn(e,t){return e>0?d.Math.log(e/t):-100}function Srt(e,t){return mo(e,t)<0?-1:mo(e,t)>0?1:0}function Dde(e,t,n){return w2t(e,h(t,46),h(n,167))}function Art(e,t){return h(yde(ob(e.a)).Xb(t),42).cd()}function Rbn(e,t){return o4n(t,e.length),new Lnt(e,t)}function yW(e,t){this.d=e,Tn.call(this,e),this.e=t}function cb(e){this.d=(rn(e),e),this.a=0,this.c=_R}function Lde(e,t){G2.call(this,1),this.a=e,this.b=t}function Mrt(e,t){return e.c?Mrt(e.c,t):Ue(e.b,t),e}function jbn(e,t,n){var s;return s=bw(e,t),iK(e,t,n),s}function Ide(e,t){var n;return n=e.slice(0,t),K0e(n,e)}function Drt(e,t,n){var s;for(s=0;s<t;++s)pi(e,s,n)}function Lrt(e,t,n,s,a){for(;t<n;)s[a++]=Is(e,t++)}function $bn(e,t){return Ui(e.c.c+e.c.b,t.c.c+t.c.b)}function tA(e,t){return cF(e.a,t,(on(),u2))==null}function nA(e,t){Ai(e.d,t,e.b.b,e.b),++e.a,e.c=null}function rA(e,t){yet(e,ue(t,153)?t:h(t,1937).gl())}function ub(e,t){Ti(pc(e.Oc(),new jje),new FYe(t))}function lk(e,t,n,s,a){rQ(e,h(jr(t.k,n),15),n,s,a)}function HP(e){e.s=NaN,e.c=NaN,R0t(e,e.e),R0t(e,e.j)}function Irt(e){e.a=null,e.e=null,Kc(e.b),e.d=0,++e.c}function kW(e){return d.Math.abs(e.d.e-e.e.e)-e.a}function Hbn(e,t,n){return h(e.c._c(t,h(n,133)),42)}function zbn(){return aN(),se(ie(i3t,1),ze,538,0,[Qee])}function Gbn(e){return V4(),bi(H1(e))==bi(Sg(e))}function Ort(e){m1e.call(this),this.a=e,Ue(e.a,this)}function xW(e,t){this.d=pkn(e),this.c=t,this.a=.5*t}function Nrt(){Np.call(this),this.a=!0,this.b=!0}function En(e){return(e.i==null&&s1(e),e.i).length}function Prt(e){return ue(e,99)&&(h(e,18).Bb&go)!=0}function Vbn(e,t){++e.j,sQ(e,e.i,t),ITn(e,h(t,332))}function EW(e,t){return t=e.nk(null,t),ndt(e,null,t)}function mi(e,t){return e.hi()&&(t=ist(e,t)),e.Wh(t)}function D(e,t,n){var s;return s=_W(e,t),_ut(n,s),s}function _W(e,t){var n;return n=new Hge,n.j=e,n.d=t,n}function Vn(e){if(e==null)throw ne(new P7);return e}function TW(e){var t;return t=e.j,t||(e.j=new cqe(e))}function Brt(e){var t;return t=e.f,t||(e.f=new Bfe(e))}function Ode(e){var t;return t=e.k,t||(e.k=new ole(e))}function zP(e){var t;return t=e.k,t||(e.k=new ole(e))}function OE(e){var t;return t=e.g,t||(e.g=new ale(e))}function Ubn(e){var t;return t=e.i,t||(e.i=new ZUe(e))}function CW(e){var t;return t=e.d,t||(e.d=new bq(e))}function Frt(e){return Vn(e),ue(e,475)?h(e,475):Ga(e)}function Nde(e){return ue(e,607)?e:new qit(e)}function Rrt(e,t){return dA(t,e.c.b.c.gc()),new NQe(e,t)}function jrt(e,t,n){return Mr(),new Nst(e,t,n)}function pi(e,t,n){return T0n(n==null||NDn(e,n)),e[t]=n}function Pde(e,t){var n;return n=e.a.gc(),dA(t,n),n-1-t}function hk(e,t){return e.a+=String.fromCharCode(t),e}function pg(e,t){return e.a+=String.fromCharCode(t),e}function SW(e,t){for(rn(t);e.c<e.d;)e.ze(t,e.c++)}function _n(e,t){return ws(t)?Fo(e,t):no(Ia(e.f,t))}function qbn(e,t){return V4(),e==H1(t)?Sg(t):H1(t)}function Ybn(e,t){z4(e,new uw(t.f!=null?t.f:""+t.g))}function Wbn(e,t){z4(e,new uw(t.f!=null?t.f:""+t.g))}function $rt(e){this.b=new qe,this.a=new qe,this.c=e}function Nl(e){this.c=new Rs,this.a=new qe,this.b=e}function Hrt(e){m1e.call(this),this.a=new Rs,this.c=e}function uw(e){if(e==null)throw ne(new P7);this.a=e}function zrt(e){Vle(),this.b=new qe,this.a=e,UIn(this,e)}function Grt(e){this.c=e,this.a=new di,this.b=new di}function Vrt(){Vrt=re,u3t=new lle(!1),l3t=new lle(!0)}function lw(){lw=re,t1(),Kee=new jW((Rt(),Rt(),ca))}function AW(){AW=re,t1(),g3e=new jhe((Rt(),Rt(),jj))}function bg(){bg=re,Xt=LTn(),Bt(),ym&&R7n()}function Kbn(e,t){return t5(),h(wa(e,t.d),15).Fc(t)}function Xbn(e,t,n,s){return n==0||(n-s)/n<e.e||t>=e.g}function MW(e,t,n){var s;return s=$K(e,t,n),ave(e,s)}function z4(e,t){var n;n=e.a.length,bw(e,n),iK(e,n,t)}function Urt(e,t){var n;n=console[e],n.call(console,t)}function qrt(e,t){var n;++e.j,n=e.Vi(),e.Ii(e.oi(n,t))}function Qbn(e,t,n){h(t.b,65),mc(t.a,new j1e(e,n,t))}function Bde(e,t,n){eN.call(this,t),this.a=e,this.b=n}function Fde(e,t,n){G2.call(this,e),this.a=t,this.b=n}function Rde(e,t,n){this.a=e,Fle.call(this,t),this.b=n}function Yrt(e,t,n){this.a=e,g0e.call(this,8,t,null,n)}function Zbn(e){this.a=(rn(lr),lr),this.b=e,new ihe}function Wrt(e){this.c=e,this.b=this.c.a,this.a=this.c.e}function jde(e){this.c=e,this.b=e.a.d.a,a1e(e.a.e,this)}function Mu(e){tw(e.c!=-1),e.d.$c(e.c),e.b=e.c,e.c=-1}function NE(e){return d.Math.sqrt(e.a*e.a+e.b*e.b)}function lb(e,t){return ck(t,e.a.c.length),ut(e.a,t)}function n1(e,t){return Me(e)===Me(t)||e!=null&&Nr(e,t)}function Jbn(e){return 0>=e?new Lhe:j4n(e-1)}function evn(e){return xm?RW(xm,e):!1}function Krt(e){return e?e.dc():!e.Kc().Ob()}function Us(e){return!e.a&&!!e.c?e.c.b:e.a}function tvn(e){return!e.a&&(e.a=new Gi(Jg,e,4)),e.a}function hb(e){return!e.d&&(e.d=new Gi(pa,e,1)),e.d}function rn(e){if(e==null)throw ne(new P7);return e}function PE(e){e.c?e.c.He():(e.d=!0,aAn(e))}function vg(e){e.c?vg(e.c):(Up(e),e.d=!0)}function Xrt(e){Gde(e.a),e.b=pe(yn,st,1,e.b.length,5,1)}function nvn(e,t){return dc(t.j.c.length,e.j.c.length)}function rvn(e,t){e.c<0||e.b.b<e.c?Cp(e.b,t):e.a._e(t)}function ivn(e,t){var n;n=e.Yg(t),n>=0?e.Bh(n):bbe(e,t)}function Qrt(e){var t,n;return t=e.c.i.c,n=e.d.i.c,t==n}function svn(e){if(e.p!=4)throw ne(new Ic);return e.e}function avn(e){if(e.p!=3)throw ne(new Ic);return e.e}function ovn(e){if(e.p!=6)throw ne(new Ic);return e.f}function cvn(e){if(e.p!=6)throw ne(new Ic);return e.k}function uvn(e){if(e.p!=3)throw ne(new Ic);return e.j}function lvn(e){if(e.p!=4)throw ne(new Ic);return e.j}function $de(e){return!e.b&&(e.b=new tN(new Mq)),e.b}function fb(e){return e.c==-2&&cfn(e,A9n(e.g,e.b)),e.c}function fk(e,t){var n;return n=_W("",e),n.n=t,n.i=1,n}function hvn(e,t){uW(h(t.b,65),e),mc(t.a,new Mle(e))}function fvn(e,t){qn((!e.a&&(e.a=new qS(e,e)),e.a),t)}function Zrt(e,t){this.b=e,yW.call(this,e,t),_et(this)}function Jrt(e,t){this.b=e,M1e.call(this,e,t),Tet(this)}function Hde(e,t,n,s){Z2.call(this,e,t),this.d=n,this.a=s}function GP(e,t,n,s){Z2.call(this,e,n),this.a=t,this.f=s}function eit(e,t){qdn.call(this,$4n(Vn(e),Vn(t))),this.a=t}function tit(){W2e.call(this,o2,(nQe(),r_t)),NIn(this)}function nit(){W2e.call(this,Vl,(q6(),mEe)),GLn(this)}function rit(){ar.call(this,"DELAUNAY_TRIANGULATION",0)}function dvn(e){return String.fromCharCode.apply(null,e)}function Pr(e,t,n){return ws(t)?Ta(e,t,n):sc(e.f,t,n)}function zde(e){return Rt(),e?e.ve():(gy(),gy(),H3e)}function gvn(e,t,n){return n5(),n.pg(e,h(t.cd(),146))}function iit(e,t){return EP(),new Jpe(new jet(e),new Ret(t))}function pvn(e){return Uu(e,pZ),PB(Vs(Vs(5,e),e/10|0))}function VP(){VP=re,n3t=new Iq(se(ie(c2,1),TR,42,0,[]))}function sit(e){return!e.d&&(e.d=new B6(e.c.Cc())),e.d}function dk(e){return!e.a&&(e.a=new FXe(e.c.vc())),e.a}function ait(e){return!e.b&&(e.b=new G6(e.c.ec())),e.b}function B1(e,t){for(;t-- >0;)e=e<<1|(e<0?1:0);return e}function Bo(e,t){return Me(e)===Me(t)||e!=null&&Nr(e,t)}function bvn(e,t){return on(),h(t.b,19).a<e}function vvn(e,t){return on(),h(t.a,19).a<e}function wa(e,t){return cW(e.a,t)?e.b[h(t,22).g]:null}function wvn(e,t,n,s){e.a=$u(e.a,0,t)+(""+s)+jS(e.a,n)}function oit(e,t){e.u.Hc((Qc(),Ud))&&xTn(e,t),i3n(e,t)}function Is(e,t){return er(t,e.length),e.charCodeAt(t)}function cit(){Ya.call(this,"There is no more element.")}function BE(e){this.d=e,this.a=this.d.b,this.b=this.d.c}function uit(e){e.b=!1,e.c=!1,e.d=!1,e.a=!1}function yr(e,t,n,s){return clt(e,t,n,!1),gF(e,s),e}function mvn(e){return e.j.c=pe(yn,st,1,0,5,1),e.a=-1,e}function yvn(e){return!e.c&&(e.c=new qt(Pn,e,5,8)),e.c}function kvn(e){return!e.b&&(e.b=new qt(Pn,e,4,7)),e.b}function UP(e){return!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n}function DW(e){return!e.c&&(e.c=new Ye(bu,e,9,9)),e.c}function gk(e){return e.e==Rx&&lfn(e,jxn(e.g,e.b)),e.e}function iA(e){return e.f==Rx&&hfn(e,C7n(e.g,e.b)),e.f}function dy(e){var t;return t=e.b,!t&&(e.b=t=new QUe(e)),t}function Gde(e){var t;for(t=e.Kc();t.Ob();)t.Pb(),t.Qb()}function pk(e){if(lu(e.d),e.d.d!=e.c)throw ne(new cl)}function Vde(e,t){this.b=e,this.c=t,this.a=new A4(this.b)}function LW(e,t,n){this.a=Jve,this.d=e,this.b=t,this.c=n}function lit(e,t){this.d=(rn(e),e),this.a=16449,this.c=t}function xvn(e,t){zht(e,Pe(Md(t,"x")),Pe(Md(t,"y")))}function Evn(e,t){zht(e,Pe(Md(t,"x")),Pe(Md(t,"y")))}function Jr(e,t){return Up(e),new Vt(e,new W0e(t,e.a))}function pc(e,t){return Up(e),new Vt(e,new B0e(t,e.a))}function qP(e,t){return Up(e),new u1e(e,new mot(t,e.a))}function YP(e,t){return Up(e),new l1e(e,new yot(t,e.a))}function _vn(e,t){return new Rit(h(Vn(e),62),h(Vn(t),62))}function Tvn(e,t){return Nhe(),Ui((rn(e),e),(rn(t),t))}function Cvn(){return kN(),se(ie(E4e,1),ze,481,0,[bte])}function Svn(){return EN(),se(ie(g8e,1),ze,482,0,[yre])}function Avn(){return _N(),se(ie(Z7t,1),ze,551,0,[kre])}function Mvn(){return K7(),se(ie(P8e,1),ze,530,0,[UD])}function hit(e){this.a=new qe,this.e=pe(Hn,Re,48,e,0,2)}function IW(e,t,n,s){this.a=e,this.e=t,this.d=n,this.c=s}function WP(e,t,n,s){this.a=e,this.c=t,this.b=n,this.d=s}function Ude(e,t,n,s){this.c=e,this.b=t,this.a=n,this.d=s}function fit(e,t,n,s){this.c=e,this.b=t,this.d=n,this.a=s}function hl(e,t,n,s){this.c=e,this.d=t,this.b=n,this.a=s}function dit(e,t,n,s){this.a=e,this.d=t,this.c=n,this.b=s}function G4(e,t,n,s){ar.call(this,e,t),this.a=n,this.b=s}function git(e,t,n,s){this.a=e,this.c=t,this.d=n,this.b=s}function Dvn(e,t,n){$Ln(e.a,n),F5n(n),dCn(e.b,n),iIn(t,n)}function OW(e,t,n){var s,a;return s=zve(e),a=t.Kh(n,s),a}function pit(e,t){var n,s;return n=e/t,s=Li(n),n>s&&++s,s}function xd(e){var t,n;return n=(t=new V2,t),Sk(n,e),n}function NW(e){var t,n;return n=(t=new V2,t),Q2e(n,e),n}function Lvn(e,t){var n;return n=_n(e.f,t),Age(t,n),null}function PW(e){var t;return t=z4n(e),t||null}function bit(e){return!e.b&&(e.b=new Ye(us,e,12,3)),e.b}function Ivn(e){return e!=null&&kS(XH,e.toLowerCase())}function Ovn(e,t){return Ui(Fc(e)*Su(e),Fc(t)*Su(t))}function Nvn(e,t){return Ui(Fc(e)*Su(e),Fc(t)*Su(t))}function Pvn(e,t){return Ui(e.d.c+e.d.b/2,t.d.c+t.d.b/2)}function Bvn(e,t){return Ui(e.g.c+e.g.b/2,t.g.c+t.g.b/2)}function vit(e,t,n){n.a?kc(e,t.b-e.f/2):yc(e,t.a-e.g/2)}function wit(e,t,n,s){this.a=e,this.b=t,this.c=n,this.d=s}function mit(e,t,n,s){this.a=e,this.b=t,this.c=n,this.d=s}function Dp(e,t,n,s){this.e=e,this.a=t,this.c=n,this.d=s}function yit(e,t,n,s){this.a=e,this.c=t,this.d=n,this.b=s}function kit(e,t,n,s){ul(),dot.call(this,t,n,s),this.a=e}function xit(e,t,n,s){ul(),dot.call(this,t,n,s),this.a=e}function Eit(e,t){this.a=e,z0n.call(this,e,h(e.d,15).Zc(t))}function BW(e){this.f=e,this.c=this.f.e,e.f>0&&P1t(this)}function _it(e,t,n,s){this.b=e,this.c=s,xY.call(this,t,n)}function Tit(e){return kn(e.b<e.d.gc()),e.d.Xb(e.c=e.b++)}function Pl(e){e.a.a=e.c,e.c.b=e.a,e.a.b=e.c.a=null,e.b=0}function qde(e,t){return e.b=t.b,e.c=t.c,e.d=t.d,e.a=t.a,e}function KP(e){return e.n&&(e.e!==Ubt&&e._d(),e.j=null),e}function Cit(e){return TE(e==null||mW(e)&&e.im!==de),e}function Sit(e){this.b=new qe,Vi(this.b,this.b),this.a=e}function V4(){V4=re,Kj=new qe,yte=new zn,mte=new qe}function Rt(){Rt=re,ca=new $,K1=new ee,jj=new ae}function gy(){gy=re,j3e=new Xe,$3e=new Xe,H3e=new Ee}function Yde(){Yde=re,N3t=new en,B3t=new fde,P3t=new In}function Fvn(){Z3e==256&&(Q3e=zj,zj=new y,Z3e=0),++Z3e}function U4(e){var t;return t=e.f,t||(e.f=new W6(e,e.c))}function Rvn(e){return Pb(e)&&ft(lt(pt(e,(tt(),d2))))}function jvn(e,t){return Ot(e,h(U(t,(tt(),c8)),19),t)}function Ait(e,t){return _y(e.j,t.s,t.c)+_y(t.e,e.s,e.c)}function Mit(e,t){!!e.e&&!e.e.a&&(CKe(e.e,t),Mit(e.e,t))}function Dit(e,t){!!e.d&&!e.d.a&&(CKe(e.d,t),Dit(e.d,t))}function $vn(e,t){return-Ui(Fc(e)*Su(e),Fc(t)*Su(t))}function Hvn(e){return h(e.cd(),146).tg()+":"+Ga(e.dd())}function zvn(e){wh();var t;t=h(e.g,10),t.n.a=e.d.c+t.d.b}function Gvn(e,t,n){return W7(),Nkn(h(_n(e.e,t),522),n)}function Vvn(e,t){return uu(e),uu(t),AXe(h(e,22),h(t,22))}function Uvn(e,t,n){e.i=0,e.e=0,t!=n&&Qlt(e,t,n)}function qvn(e,t,n){e.i=0,e.e=0,t!=n&&Zlt(e,t,n)}function Lp(e,t,n){var s,a;s=L1e(n),a=new WO(s),Wh(e,t,a)}function FW(e,t,n,s,a,u){yA.call(this,e,t,n,s,a,u?-2:-1)}function Lit(e,t,n,s){dfe.call(this,t,n),this.b=e,this.a=s}function Wde(e,t){new di,this.a=new Oc,this.b=e,this.c=t}function Yvn(e,t){return h(U(e,(He(),B5)),15).Fc(t),t}function XP(e,t){if(e==null)throw ne(new T4(t));return e}function Gr(e){return!e.q&&(e.q=new Ye(eh,e,11,10)),e.q}function De(e){return!e.s&&(e.s=new Ye(Ac,e,21,17)),e.s}function QP(e){return!e.a&&(e.a=new Ye(wi,e,10,11)),e.a}function ZP(e){return ue(e,14)?new CE(h(e,14)):_2n(e.Kc())}function Wvn(e){return new Let(e,e.e.Hd().gc()*e.c.Hd().gc())}function Kvn(e){return new Iet(e,e.e.Hd().gc()*e.c.Hd().gc())}function Kde(e){return!!e&&!!e.hashCode?e.hashCode():rb(e)}function RW(e,t){return t==null?!!Ia(e.f,null):lbn(e.g,t)}function Xvn(e){return Vn(e),Jft(new Sn(Dn(e.a.Kc(),new L)))}function sA(e){return Rt(),ue(e,54)?new Hq(e):new sP(e)}function Iit(e,t,n){return e.f?e.f.Ne(t,n):!1}function Qvn(e,t){return e.a=$u(e.a,0,t)+""+jS(e.a,t+1),e}function Zvn(e,t){var n;return n=Lfe(e.a,t),n&&(t.d=null),n}function JP(e){var t,n;n=e,t=n.$modCount|0,n.$modCount=t+1}function Xde(e){this.b=e,this.c=e,e.e=null,e.c=null,this.a=1}function Oit(e){this.b=e,this.a=new ag(h(Vn(new tg),62))}function Nit(e){this.c=e,this.b=new ag(h(Vn(new hn),62))}function Pit(e){this.c=e,this.b=new ag(h(Vn(new D7),62))}function Bit(){this.a=new Zle,this.b=new ZKe,this.d=new SBe}function db(){this.a=new Oc,this.b=(Uu(3,jw),new Wo(3))}function Fit(){this.b=new ns,this.d=new di,this.e=new ehe}function eB(e){this.c=e.c,this.d=e.d,this.b=e.b,this.a=e.a}function Rit(e,t){Ofn.call(this,new pW(e)),this.a=e,this.b=t}function jit(){lQ(this,new rle),this.wb=(bg(),Xt),q6()}function Jvn(e){Bn(e,"No crossing minimization",1),An(e)}function ewn(e){lN(),d.setTimeout(function(){throw e},0)}function La(e){return e.u||(ou(e),e.u=new xtt(e,e)),e.u}function bc(e){var t;return t=h(tn(e,16),26),t||e.zh()}function $it(e,t){return ue(t,146)&&Nt(e.b,h(t,146).tg())}function twn(e,t){return e.a?t.Wg().Kc():h(t.Wg(),69).Zh()}function nwn(e){return e.k==(bn(),Wi)&&os(e,(He(),T_))}function jW(e){this.a=(Rt(),ue(e,54)?new Hq(e):new sP(e))}function bk(){bk=re;var e,t;t=!Exn(),e=new R,ete=t?new Z:e}function $W(e,t){var n;return n=sg(e.gm),t==null?n:n+": "+t}function Hit(e,t){var n;return n=e.b.Qc(t),Iat(n,e.b.gc()),n}function aA(e,t){if(e==null)throw ne(new T4(t));return e}function Ia(e,t){return fht(e,t,Rwn(e,t==null?0:e.b.se(t)))}function rwn(e,t,n){return n>=0&&Nt(e.substr(n,t.length),t)}function wg(e,t,n,s,a,u,p){return new aK(e.e,t,n,s,a,u,p)}function zit(e,t,n,s,a,u){this.a=e,OK.call(this,t,n,s,a,u)}function Git(e,t,n,s,a,u){this.a=e,OK.call(this,t,n,s,a,u)}function Vit(e,t){this.g=e,this.d=se(ie(X1,1),T0,10,0,[t])}function h0(e,t){this.e=e,this.a=yn,this.b=dpt(t),this.c=t}function Uit(e,t){gP.call(this),rge(this),this.a=e,this.c=t}function oA(e,t,n,s){pi(e.c[t.g],n.g,s),pi(e.c[n.g],t.g,s)}function HW(e,t,n,s){pi(e.c[t.g],t.g,n),pi(e.b[t.g],t.g,s)}function iwn(){return bA(),se(ie(oxe,1),ze,376,0,[Wne,RD])}function swn(){return xA(),se(ie(J5e,1),ze,479,0,[Z5e,M$])}function awn(){return DA(),se(ie(X5e,1),ze,419,0,[S$,K5e])}function own(){return TB(),se(ie(z5e,1),ze,422,0,[H5e,Kte])}function cwn(){return gB(),se(ie(f6e,1),ze,420,0,[lne,h6e])}function uwn(){return CA(),se(ie(rxe,1),ze,421,0,[Une,qne])}function lwn(){return GE(),se(ie(o8t,1),ze,523,0,[$_,j_])}function hwn(){return qh(),se(ie(L8t,1),ze,520,0,[fm,qg])}function fwn(){return r1(),se(ie(g8t,1),ze,516,0,[lv,I0])}function dwn(){return yf(),se(ie(b8t,1),ze,515,0,[v2,Rf])}function gwn(){return gb(),se(ie(R8t,1),ze,455,0,[Yg,m3])}function pwn(){return lB(),se(ie(Lxe,1),ze,425,0,[ure,Dxe])}function bwn(){return aB(),se(ie(Mxe,1),ze,480,0,[cre,Axe])}function vwn(){return QB(),se(ie(Ixe,1),ze,495,0,[vH,p8])}function wwn(){return SB(),se(ie(Nxe,1),ze,426,0,[Oxe,dre])}function mwn(){return $A(),se(ie(F8e,1),ze,429,0,[CH,B8e])}function ywn(){return vA(),se(ie(p8e,1),ze,430,0,[xre,_H])}function kwn(){return SF(),se(ie(e4e,1),ze,428,0,[ute,J3e])}function xwn(){return OB(),se(ie(n4e,1),ze,427,0,[t4e,lte])}function Ewn(){return NB(),se(ie(D4e,1),ze,424,0,[kte,Xj])}function _wn(){return Nk(),se(ie(D5t,1),ze,511,0,[kD,Nte])}function tB(e,t,n,s){return n>=0?e.jh(t,n,s):e.Sg(null,n,s)}function zW(e){return e.b.b==0?e.a.$e():eW(e.b)}function Twn(e){if(e.p!=5)throw ne(new Ic);return Gn(e.f)}function Cwn(e){if(e.p!=5)throw ne(new Ic);return Gn(e.k)}function Qde(e){return Me(e.a)===Me((GK(),iie))&&SIn(e),e.a}function qit(e){this.a=h(Vn(e),271),this.b=(Rt(),new Vfe(e))}function Yit(e,t){Yhn(this,new dt(e.a,e.b)),Whn(this,ZS(t))}function gb(){gb=re,Yg=new sfe(w5,0),m3=new sfe(m5,1)}function r1(){r1=re,lv=new rfe(m5,0),I0=new rfe(w5,1)}function pb(){Bfn.call(this,new S4(vw(12))),$fe(!0),this.a=2}function GW(e,t,n){Mr(),G2.call(this,e),this.b=t,this.a=n}function Zde(e,t,n){ul(),eN.call(this,t),this.a=e,this.b=n}function Wit(e){gP.call(this),rge(this),this.a=e,this.c=!0}function Kit(e){var t;t=e.c.d.b,e.b=t,e.a=e.c.d,t.a=e.c.d.b=e}function nB(e){var t;O4n(e.a),iet(e.a),t=new XO(e.a),Spe(t)}function Swn(e,t){ipt(e,!0),mc(e.e.wf(),new P1e(e,!0,t))}function rB(e,t){return Aat(t),D4n(e,pe(Hn,hr,25,t,15,1),t)}function Awn(e,t){return V4(),e==bi(H1(t))||e==bi(Sg(t))}function Fo(e,t){return t==null?no(Ia(e.f,null)):Z7(e.g,t)}function Mwn(e){return e.b==0?null:(kn(e.b!=0),pl(e,e.a.a))}function Li(e){return Math.max(Math.min(e,Ir),-2147483648)|0}function Dwn(e,t){var n=Jee[e.charCodeAt(0)];return n==null?e:n}function iB(e,t){return XP(e,"set1"),XP(t,"set2"),new GQe(e,t)}function Lwn(e,t){var n;return n=T4n(e.f,t),zr(fP(n),e.f.d)}function FE(e,t){var n,s;return n=t,s=new $e,q2t(e,n,s),s.d}function VW(e,t,n,s){var a;a=new Btt,t.a[n.g]=a,j4(e.b,s,a)}function Jde(e,t,n){var s;s=e.Yg(t),s>=0?e.sh(s,n):zbe(e,t,n)}function py(e,t,n){oB(),e&&Pr(tie,e,t),e&&Pr(gL,e,n)}function Xit(e,t,n){this.i=new qe,this.b=e,this.g=t,this.a=n}function sB(e,t,n){this.c=new qe,this.e=e,this.f=t,this.b=n}function e0e(e,t,n){this.a=new qe,this.e=e,this.f=t,this.c=n}function Qit(e,t){nP(this),this.f=t,this.g=e,KP(this),this._d()}function cA(e,t){var n;n=e.q.getHours(),e.q.setDate(t),C9(e,n)}function Zit(e,t){var n;for(Vn(t),n=e.a;n;n=n.c)t.Od(n.g,n.i)}function Jit(e){var t;return t=new gN(vw(e.length)),Uge(t,e),t}function Iwn(e){function t(){}return t.prototype=e||{},new t}function Own(e,t){return Alt(e,t)?(Cut(e),!0):!1}function Ed(e,t){if(t==null)throw ne(new P7);return Dxn(e,t)}function Nwn(e){if(e.qe())return null;var t=e.n;return Oj[t]}function uA(e){return e.Db>>16!=3?null:h(e.Cb,33)}function mf(e){return e.Db>>16!=9?null:h(e.Cb,33)}function est(e){return e.Db>>16!=6?null:h(e.Cb,79)}function tst(e){return e.Db>>16!=7?null:h(e.Cb,235)}function nst(e){return e.Db>>16!=7?null:h(e.Cb,160)}function bi(e){return e.Db>>16!=11?null:h(e.Cb,33)}function rst(e,t){var n;return n=e.Yg(t),n>=0?e.lh(n):mQ(e,t)}function ist(e,t){var n;return n=new sde(t),sdt(n,e),new Bc(n)}function t0e(e){var t;return t=e.d,t=e.si(e.f),qn(e,t),t.Ob()}function sst(e,t){return e.b+=t.b,e.c+=t.c,e.d+=t.d,e.a+=t.a,e}function UW(e,t){return d.Math.abs(e)<d.Math.abs(t)?e:t}function Pwn(e){return!e.a&&(e.a=new Ye(wi,e,10,11)),e.a.i>0}function ast(){this.a=new md,this.e=new ns,this.g=0,this.i=0}function ost(e){this.a=e,this.b=pe(i8t,Re,1944,e.e.length,0,2)}function qW(e,t,n){var s;s=Wlt(e,t,n),e.b=new XB(s.c.length)}function yf(){yf=re,v2=new nfe(ZZ,0),Rf=new nfe("UP",1)}function aB(){aB=re,cre=new afe(ywt,0),Axe=new afe("FAN",1)}function oB(){oB=re,tie=new zn,gL=new zn,tdn(y3t,new Pze)}function Bwn(e){if(e.p!=0)throw ne(new Ic);return fE(e.f,0)}function Fwn(e){if(e.p!=0)throw ne(new Ic);return fE(e.k,0)}function cst(e){return e.Db>>16!=3?null:h(e.Cb,147)}function vk(e){return e.Db>>16!=6?null:h(e.Cb,235)}function hw(e){return e.Db>>16!=17?null:h(e.Cb,26)}function ust(e,t){var n=e.a=e.a||[];return n[t]||(n[t]=e.le(t))}function Rwn(e,t){var n;return n=e.a.get(t),n==null?new Array:n}function jwn(e,t){var n;n=e.q.getHours(),e.q.setMonth(t),C9(e,n)}function Ta(e,t,n){return t==null?sc(e.f,null,n):Ab(e.g,t,n)}function RE(e,t,n,s,a,u){return new Sd(e.e,t,e.aj(),n,s,a,u)}function lA(e,t,n){return e.a=$u(e.a,0,t)+(""+n)+jS(e.a,t),e}function $wn(e,t,n){return Ue(e.a,(VP(),GX(t,n),new Z2(t,n))),e}function n0e(e){return zfe(e.c),e.e=e.a=e.c,e.c=e.c.c,++e.d,e.a.f}function lst(e){return zfe(e.e),e.c=e.a=e.e,e.e=e.e.e,--e.d,e.a.f}function xs(e,t){e.d&&vc(e.d.e,e),e.d=t,e.d&&Ue(e.d.e,e)}function Gs(e,t){e.c&&vc(e.c.g,e),e.c=t,e.c&&Ue(e.c.g,e)}function Ca(e,t){e.c&&vc(e.c.a,e),e.c=t,e.c&&Ue(e.c.a,e)}function Ka(e,t){e.i&&vc(e.i.j,e),e.i=t,e.i&&Ue(e.i.j,e)}function hst(e,t,n){this.a=t,this.c=e,this.b=(Vn(n),new Bc(n))}function fst(e,t,n){this.a=t,this.c=e,this.b=(Vn(n),new Bc(n))}function dst(e,t){this.a=e,this.c=ro(this.a),this.b=new eB(t)}function Hwn(e){var t;return Up(e),t=new ns,Jr(e,new jqe(t))}function fw(e,t){if(e<0||e>t)throw ne(new Ea(owe+e+cwe+t))}function r0e(e,t){return rrt(e.a,t)?Sde(e,h(t,22).g,null):null}function zwn(e){return uX(),on(),h(e.a,81).d.e!=0}function gst(){gst=re,s3t=ur((aN(),se(ie(i3t,1),ze,538,0,[Qee])))}function pst(){pst=re,Uxt=Wc(new rs,(Zs(),Pa),(oa(),xD))}function i0e(){i0e=re,qxt=Wc(new rs,(Zs(),Pa),(oa(),xD))}function bst(){bst=re,Wxt=Wc(new rs,(Zs(),Pa),(oa(),xD))}function vst(){vst=re,u8t=Lr(new rs,(Zs(),Pa),(oa(),w_))}function Hu(){Hu=re,f8t=Lr(new rs,(Zs(),Pa),(oa(),w_))}function wst(){wst=re,d8t=Lr(new rs,(Zs(),Pa),(oa(),w_))}function YW(){YW=re,w8t=Lr(new rs,(Zs(),Pa),(oa(),w_))}function mst(){mst=re,i7t=Wc(new rs,(tx(),z_),(w9(),Jne))}function Ip(e,t,n,s){this.c=e,this.d=s,KW(this,t),XW(this,n)}function q4(e){this.c=new di,this.b=e.b,this.d=e.c,this.a=e.a}function WW(e){this.a=d.Math.cos(e),this.b=d.Math.sin(e)}function KW(e,t){e.a&&vc(e.a.k,e),e.a=t,e.a&&Ue(e.a.k,e)}function XW(e,t){e.b&&vc(e.b.f,e),e.b=t,e.b&&Ue(e.b.f,e)}function yst(e,t){Qbn(e,e.b,e.c),h(e.b.b,65),t&&h(t.b,65).b}function Gwn(e,t){ype(e,t),ue(e.Cb,88)&&Lw(ou(h(e.Cb,88)),2)}function QW(e,t){ue(e.Cb,88)&&Lw(ou(h(e.Cb,88)),4),Ko(e,t)}function cB(e,t){ue(e.Cb,179)&&(h(e.Cb,179).tb=null),Ko(e,t)}function Ro(e,t){return ia(),SK(t)?new _P(t,e):new RS(t,e)}function Vwn(e,t){var n,s;n=t.c,s=n!=null,s&&z4(e,new uw(t.c))}function kst(e){var t,n;return n=(q6(),t=new V2,t),Sk(n,e),n}function xst(e){var t,n;return n=(q6(),t=new V2,t),Sk(n,e),n}function Est(e,t){var n;return n=new Nl(e),t.c[t.c.length]=n,n}function _st(e,t){var n;return n=h(xw(U4(e.a),t),14),n?n.gc():0}function Tst(e){var t;return Up(e),t=(gy(),gy(),$3e),FB(e,t)}function Cst(e){for(var t;;)if(t=e.Pb(),!e.Ob())return t}function s0e(e,t){Nfn.call(this,new S4(vw(e))),Uu(t,jbt),this.a=t}function F1(e,t,n){Oht(t,n,e.gc()),this.c=e,this.a=t,this.b=n-t}function Sst(e,t,n){var s;Oht(t,n,e.c.length),s=n-t,Mhe(e.c,t,s)}function Uwn(e,t){Eet(e,Gn(Zi(dg(t,24),AR)),Gn(Zi(t,AR)))}function Kt(e,t){if(e<0||e>=t)throw ne(new Ea(owe+e+cwe+t))}function er(e,t){if(e<0||e>=t)throw ne(new vhe(owe+e+cwe+t))}function Wt(e,t){this.b=(rn(e),e),this.a=(t&$w)==0?t|64|a1:t}function Ast(e){ret(this),FKe(this.a,Gge(d.Math.max(8,e))<<1)}function kf(e){return Qa(se(ie(cs,1),Re,8,0,[e.i.n,e.n,e.a]))}function qwn(){return Af(),se(ie(gu,1),ze,132,0,[X3e,Zu,Kw])}function Ywn(){return Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])}function Wwn(){return Xc(),se(ie(G3t,1),ze,461,0,[v1,jg,Sh])}function Kwn(){return wc(),se(ie(U3t,1),ze,462,0,[nf,$g,Ah])}function Xwn(){return Sb(),se(ie(W4e,1),ze,423,0,[i3,Y4e,Lte])}function Qwn(){return UE(),se(ie(V4e,1),ze,379,0,[Tte,_te,Cte])}function Zwn(){return t9(),se(ie(Yke,1),ze,378,0,[jne,qke,nH])}function Jwn(){return e5(),se(ie(V5e,1),ze,314,0,[I5,_D,G5e])}function emn(){return GB(),se(ie(q5e,1),ze,337,0,[U5e,C$,Xte])}function tmn(){return Hp(),se(ie(s6t,1),ze,450,0,[Jte,t8,c3])}function nmn(){return Tb(),se(ie($te,1),ze,361,0,[ev,zg,Jb])}function rmn(){return Ad(),se(ie(d6t,1),ze,303,0,[CD,l3,O5])}function imn(){return jk(),se(ie(une,1),ze,292,0,[one,cne,TD])}function smn(){return ua(),se(ie(Rxt,1),ze,452,0,[B_,Jc,ec])}function amn(){return Dd(),se(ie(nxe,1),ze,339,0,[Ug,txe,Vne])}function omn(){return nF(),se(ie(axe,1),ze,375,0,[ixe,Yne,sxe])}function cmn(){return oF(),se(ie(dxe,1),ze,377,0,[Qne,d8,hm])}function umn(){return qk(),se(ie(uxe,1),ze,336,0,[Kne,cxe,F_])}function lmn(){return eF(),se(ie(fxe,1),ze,338,0,[hxe,Xne,lxe])}function hmn(){return _b(),se(ie(n8t,1),ze,454,0,[jD,R_,aH])}function fmn(){return wF(),se(ie(u7t,1),ze,442,0,[fre,lre,hre])}function dmn(){return VA(),se(ie(Fxe,1),ze,380,0,[wH,Pxe,Bxe])}function gmn(){return vF(),se(ie(t8e,1),ze,381,0,[e8e,wre,Jxe])}function pmn(){return tF(),se(ie(Qxe,1),ze,293,0,[vre,Xxe,Kxe])}function bmn(){return XA(),se(ie(mre,1),ze,437,0,[kH,xH,EH])}function vmn(){return Ld(),se(ie(V7e,1),ze,334,0,[BH,B0,tT])}function wmn(){return Tf(),se(ie(I7e,1),ze,272,0,[w8,pm,m8])}function mmn(e,t){return $Cn(e,t,ue(t,99)&&(h(t,18).Bb&ea)!=0)}function ymn(e,t,n){var s;return s=M9(e,t,!1),s.b<=t&&s.a<=n}function Mst(e,t,n){var s;s=new H$e,s.b=t,s.a=n,++t.b,Ue(e.d,s)}function kmn(e,t){var n;return n=(rn(e),e).g,e1e(!!n),rn(t),n(t)}function a0e(e,t){var n,s;return s=uk(e,t),n=e.a.Zc(s),new zQe(e,n)}function xmn(e){return e.Db>>16!=6?null:h(xQ(e),235)}function Emn(e){if(e.p!=2)throw ne(new Ic);return Gn(e.f)&Oi}function _mn(e){if(e.p!=2)throw ne(new Ic);return Gn(e.k)&Oi}function Tmn(e){return e.a==(kk(),ez)&&ofn(e,nSn(e.g,e.b)),e.a}function Y4(e){return e.d==(kk(),ez)&&ufn(e,KMn(e.g,e.b)),e.d}function K(e){return kn(e.a<e.c.c.length),e.b=e.a++,e.c.c[e.b]}function Cmn(e,t){e.b=e.b|t.b,e.c=e.c|t.c,e.d=e.d|t.d,e.a=e.a|t.a}function Zi(e,t){return xg(ybn($a(e)?Bl(e):e,$a(t)?Bl(t):t))}function xf(e,t){return xg(kbn($a(e)?Bl(e):e,$a(t)?Bl(t):t))}function ZW(e,t){return xg(xbn($a(e)?Bl(e):e,$a(t)?Bl(t):t))}function Smn(e){return Vs(kd(xc(hu(e,32)),32),xc(hu(e,32)))}function bb(e){return Vn(e),ue(e,14)?new Bc(h(e,14)):sk(e.Kc())}function Amn(e,t){return qE(),e.c==t.c?Ui(t.d,e.d):Ui(e.c,t.c)}function Mmn(e,t){return qE(),e.c==t.c?Ui(e.d,t.d):Ui(e.c,t.c)}function Dmn(e,t){return qE(),e.c==t.c?Ui(e.d,t.d):Ui(t.c,e.c)}function Lmn(e,t){return qE(),e.c==t.c?Ui(t.d,e.d):Ui(t.c,e.c)}function Imn(e,t){var n;n=Pe(Je(e.a.We((Tr(),PH)))),vbt(e,t,n)}function Omn(e,t){var n;n=h(_n(e.g,t),57),mc(t.d,new LZe(e,n))}function Nmn(e,t){var n,s;return n=p1t(e),s=p1t(t),n<s?-1:n>s?1:0}function Dst(e,t){var n,s;return n=xK(t),s=n,h(_n(e.c,s),19).a}function Lst(e,t){var n;for(n=e+"";n.length<t;)n="0"+n;return n}function uB(e){return e.c==null||e.c.length==0?"n_"+e.g:"n_"+e.c}function o0e(e){return e.c==null||e.c.length==0?"n_"+e.b:"n_"+e.c}function c0e(e,t){return!!e&&!!e.equals?e.equals(t):Me(e)===Me(t)}function u0e(e,t){return t==0?!!e.o&&e.o.f!=0:OX(e,t)}function mh(e,t,n){var s;e.n&&!!t&&!!n&&(s=new Tze,Ue(e.e,s))}function JW(e,t,n){var s;s=e.d[t.p],e.d[t.p]=e.d[n.p],e.d[n.p]=s}function Ist(e,t,n){this.d=e,this.j=t,this.e=n,this.o=-1,this.p=3}function Ost(e,t,n){this.d=e,this.k=t,this.f=n,this.o=-1,this.p=5}function Nst(e,t,n){G2.call(this,25),this.b=e,this.a=t,this.c=n}function zu(e){Mr(),G2.call(this,e),this.c=!1,this.a=!1}function Pst(e,t,n,s,a,u){age.call(this,e,t,n,s,a),u&&(this.o=-2)}function Bst(e,t,n,s,a,u){oge.call(this,e,t,n,s,a),u&&(this.o=-2)}function Fst(e,t,n,s,a,u){L0e.call(this,e,t,n,s,a),u&&(this.o=-2)}function Rst(e,t,n,s,a,u){lge.call(this,e,t,n,s,a),u&&(this.o=-2)}function jst(e,t,n,s,a,u){I0e.call(this,e,t,n,s,a),u&&(this.o=-2)}function $st(e,t,n,s,a,u){cge.call(this,e,t,n,s,a),u&&(this.o=-2)}function Hst(e,t,n,s,a,u){uge.call(this,e,t,n,s,a),u&&(this.o=-2)}function zst(e,t,n,s,a,u){O0e.call(this,e,t,n,s,a),u&&(this.o=-2)}function Gst(e,t,n,s){eN.call(this,n),this.b=e,this.c=t,this.d=s}function l0e(e,t){this.a=new qe,this.d=new qe,this.f=e,this.c=t}function Vst(){this.c=new pet,this.a=new Bit,this.b=new qKe,sZe()}function Ust(){n5(),this.b=new zn,this.a=new zn,this.c=new qe}function qst(e,t){this.g=e,this.d=(kk(),ez),this.a=ez,this.b=t}function Yst(e,t){this.f=e,this.a=(kk(),JH),this.c=JH,this.b=t}function h0e(e,t){!e.c&&(e.c=new yi(e,0)),pR(e.c,(qr(),dT),t)}function lB(){lB=re,ure=new ofe("DFS",0),Dxe=new ofe("BFS",1)}function Pmn(e,t,n){var s;return s=h(e.Zb().xc(t),14),!!s&&s.Hc(n)}function Wst(e,t,n){var s;return s=h(e.Zb().xc(t),14),!!s&&s.Mc(n)}function Kst(e,t,n,s){return e.a+=""+$u(t==null?_c:Ga(t),n,s),e}function io(e,t,n,s,a,u){return clt(e,t,n,u),ppe(e,s),bpe(e,a),e}function eK(e){return kn(e.b.b!=e.d.a),e.c=e.b=e.b.b,--e.a,e.c.c}function jE(e){for(;e.d>0&&e.a[--e.d]==0;);e.a[e.d++]==0&&(e.e=0)}function Xst(e){return e.a?e.e.length==0?e.a.a:e.a.a+(""+e.e):e.c}function Bmn(e){return!!e.a&&cu(e.a.a).i!=0&&!(!!e.b&&$X(e.b))}function Fmn(e){return!!e.u&&Co(e.u.a).i!=0&&!(!!e.n&&jX(e.n))}function Qst(e){return iW(e.e.Hd().gc()*e.c.Hd().gc(),16,new qUe(e))}function Rmn(e,t){return Srt(xc(e.q.getTime()),xc(t.q.getTime()))}function i1(e){return h(Mf(e,pe(Ite,gJ,17,e.c.length,0,1)),474)}function hA(e){return h(Mf(e,pe(X1,T0,10,e.c.length,0,1)),193)}function jmn(e){return Hu(),!Xs(e)&&!(!Xs(e)&&e.c.i.c==e.d.i.c)}function Zst(e,t,n){var s;s=(Vn(e),new Bc(e)),p7n(new hst(s,t,n))}function fA(e,t,n){var s;s=(Vn(e),new Bc(e)),b7n(new fst(s,t,n))}function Jst(e,t){var n;return n=1-t,e.a[n]=KB(e.a[n],n),KB(e,t)}function eat(e,t){var n;e.e=new lhe,n=Nw(t),hs(n,e.c),Ygt(e,n,0)}function Es(e,t,n,s){var a;a=new Xue,a.a=t,a.b=n,a.c=s,vr(e.a,a)}function et(e,t,n,s){var a;a=new Xue,a.a=t,a.b=n,a.c=s,vr(e.b,a)}function yh(e){var t,n,s;return t=new Nrt,n=HQ(t,e),_On(t),s=n,s}function f0e(){var e,t,n;return t=(n=(e=new V2,e),n),Ue(MEe,t),t}function hB(e){return e.j.c=pe(yn,st,1,0,5,1),Gde(e.c),mvn(e.a),e}function by(e){return W7(),ue(e.g,10)?h(e.g,10):null}function $mn(e){return dy(e).dc()?!1:(jdn(e,new H),!0)}function Hmn(e){if(!("stack"in e))try{throw e}catch{}return e}function dA(e,t){if(e<0||e>=t)throw ne(new Ea(eTn(e,t)));return e}function tat(e,t,n){if(e<0||t<e||t>n)throw ne(new Ea(T_n(e,t,n)))}function tK(e,t){if(Qi(e.a,t),t.d)throw ne(new Ya(ovt));t.d=e}function nK(e,t){if(t.$modCount!=e.$modCount)throw ne(new cl)}function nat(e,t){return ue(t,42)?UX(e.a,h(t,42)):!1}function rat(e,t){return ue(t,42)?UX(e.a,h(t,42)):!1}function iat(e,t){return ue(t,42)?UX(e.a,h(t,42)):!1}function zmn(e,t){return e.a<=e.b?(t.ud(e.a++),!0):!1}function vb(e){var t;return $a(e)?(t=e,t==-0?0:t):Q3n(e)}function fB(e){var t;return vg(e),t=new Ie,ty(e.a,new Fqe(t)),t}function sat(e){var t;return vg(e),t=new Ze,ty(e.a,new Bqe(t)),t}function Ms(e,t){this.a=e,L7.call(this,e),fw(t,e.gc()),this.b=t}function d0e(e){this.e=e,this.b=this.e.a.entries(),this.a=new Array}function Gmn(e){return iW(e.e.Hd().gc()*e.c.Hd().gc(),273,new UUe(e))}function dB(e){return new Wo((Uu(e,pZ),PB(Vs(Vs(5,e),e/10|0))))}function aat(e){return h(Mf(e,pe(S5t,Fvt,11,e.c.length,0,1)),1943)}function Vmn(e,t,n){return n.f.c.length>0?Dde(e.a,t,n):Dde(e.b,t,n)}function Umn(e,t,n){e.d&&vc(e.d.e,e),e.d=t,e.d&&aw(e.d.e,n,e)}function rK(e,t){RNn(t,e),K1e(e.d),K1e(h(U(e,(tt(),K$)),207))}function $E(e,t){FNn(t,e),W1e(e.d),W1e(h(U(e,(tt(),K$)),207))}function wb(e,t){var n,s;return n=Ed(e,t),s=null,n&&(s=n.fe()),s}function wk(e,t){var n,s;return n=bw(e,t),s=null,n&&(s=n.ie()),s}function HE(e,t){var n,s;return n=Ed(e,t),s=null,n&&(s=n.ie()),s}function _d(e,t){var n,s;return n=Ed(e,t),s=null,n&&(s=ibe(n)),s}function qmn(e,t,n){var s;return s=Jk(n),lR(e.g,s,t),lR(e.i,t,n),t}function Ymn(e,t,n){var s;s=kxn();try{return Z0n(e,t,n)}finally{iyn(s)}}function oat(e){var t;t=e.Wg(),this.a=ue(t,69)?h(t,69).Zh():t.Kc()}function rs(){mXe.call(this),this.j.c=pe(yn,st,1,0,5,1),this.a=-1}function g0e(e,t,n,s){this.d=e,this.n=t,this.g=n,this.o=s,this.p=-1}function cat(e,t,n,s){this.e=s,this.d=null,this.c=e,this.a=t,this.b=n}function p0e(e,t,n){this.d=new WYe(this),this.e=e,this.i=t,this.f=n}function gB(){gB=re,lne=new Zhe(Cx,0),h6e=new Zhe("TOP_LEFT",1)}function uat(){uat=re,pxe=iit(Ke(1),Ke(4)),gxe=iit(Ke(1),Ke(2))}function lat(){lat=re,J7t=ur((_N(),se(ie(Z7t,1),ze,551,0,[kre])))}function hat(){hat=re,Q7t=ur((EN(),se(ie(g8e,1),ze,482,0,[yre])))}function fat(){fat=re,vEt=ur((K7(),se(ie(P8e,1),ze,530,0,[UD])))}function dat(){dat=re,E4t=ur((kN(),se(ie(E4e,1),ze,481,0,[bte])))}function Wmn(){return Cb(),se(ie(b4t,1),ze,406,0,[fD,hD,gte,pte])}function Kmn(){return AB(),se(ie(Hj,1),ze,297,0,[ate,q3e,Y3e,W3e])}function Xmn(){return h9(),se(ie(k4t,1),ze,394,0,[vD,qj,Yj,wD])}function Qmn(){return o5(),se(ie(w4t,1),ze,323,0,[gD,dD,pD,bD])}function Zmn(){return $k(),se(ie(p5t,1),ze,405,0,[Zb,Zw,Qw,r3])}function Jmn(){return aF(),se(ie(N5t,1),ze,360,0,[Rte,m$,y$,ED])}function gat(e,t,n,s){return ue(n,54)?new ett(e,t,n,s):new mde(e,t,n,s)}function eyn(){return Sf(),se(ie(G5t,1),ze,411,0,[L5,Xx,Qx,jte])}function tyn(e){var t;return e.j==(Qe(),Fn)&&(t=B0t(e),Pc(t,pn))}function nyn(e,t){var n;n=t.a,Gs(n,t.c.d),xs(n,t.d.d),mw(n.a,e.n)}function pat(e,t){return h(sb(PP(h(jr(e.k,t),15).Oc(),s3)),113)}function bat(e,t){return h(sb(BP(h(jr(e.k,t),15).Oc(),s3)),113)}function ryn(e){return new Wt(e5n(h(e.a.dd(),14).gc(),e.a.cd()),16)}function mk(e){return ue(e,14)?h(e,14).dc():!e.Kc().Ob()}function W4(e){return W7(),ue(e.g,145)?h(e.g,145):null}function vat(e){if(e.e.g!=e.b)throw ne(new cl);return!!e.c&&e.d>0}function gr(e){return kn(e.b!=e.d.c),e.c=e.b,e.b=e.b.a,++e.a,e.c.c}function b0e(e,t){rn(t),pi(e.a,e.c,t),e.c=e.c+1&e.a.length-1,w1t(e)}function mg(e,t){rn(t),e.b=e.b-1&e.a.length-1,pi(e.a,e.b,t),w1t(e)}function wat(e,t){var n;for(n=e.j.c.length;n<t;n++)Ue(e.j,e.rg())}function mat(e,t,n,s){var a;return a=s[t.g][n.g],Pe(Je(U(e.a,a)))}function v0e(e,t,n,s,a){this.i=e,this.a=t,this.e=n,this.j=s,this.f=a}function yat(e,t,n,s,a){this.a=e,this.e=t,this.f=n,this.b=s,this.g=a}function iyn(e){e&&e4n((ghe(),b3e)),--Nj,e&&Pj!=-1&&(z1n(Pj),Pj=-1)}function syn(){return Tw(),se(ie(Hne,1),ze,197,0,[rH,$ne,w3,v3])}function ayn(){return tx(),se(ie(wxe,1),ze,393,0,[lH,H_,HD,z_])}function oyn(){return FF(),se(ie(Wxe,1),ze,340,0,[bre,qxe,Yxe,Uxe])}function cyn(){return Iu(),se(ie(aT,1),ze,374,0,[cL,E2,oL,bm])}function uyn(){return Yu(),se(ie(e9t,1),ze,285,0,[iL,Z1,Kg,rL])}function lyn(){return Od(),se(ie(Yre,1),ze,218,0,[qre,nL,y8,W5])}function hyn(){return kF(),se(ie(J7e,1),ze,311,0,[Xre,X7e,Z7e,Q7e])}function fyn(){return Lu(),se(ie(p9t,1),ze,396,0,[rl,tEe,eEe,nEe])}function dyn(e){return oB(),Au(tie,e)?h(_n(tie,e),331).ug():null}function fl(e,t,n){return t<0?mQ(e,n):h(n,66).Nj().Sj(e,e.yh(),t)}function gyn(e,t,n){var s;return s=Jk(n),lR(e.d,s,t),Pr(e.e,t,n),t}function pyn(e,t,n){var s;return s=Jk(n),lR(e.j,s,t),Pr(e.k,t,n),t}function kat(e){var t,n;return t=(X2(),n=new Que,n),e&&AQ(t,e),t}function w0e(e){var t;return t=e.ri(e.i),e.i>0&&Ao(e.g,0,t,0,e.i),t}function xat(e,t){SN();var n;return n=h(_n(KH,e),55),!n||n.wj(t)}function byn(e){if(e.p!=1)throw ne(new Ic);return Gn(e.f)<<24>>24}function vyn(e){if(e.p!=1)throw ne(new Ic);return Gn(e.k)<<24>>24}function wyn(e){if(e.p!=7)throw ne(new Ic);return Gn(e.k)<<16>>16}function myn(e){if(e.p!=7)throw ne(new Ic);return Gn(e.f)<<16>>16}function Td(e){var t;for(t=0;e.Ob();)e.Pb(),t=Vs(t,1);return PB(t)}function Eat(e,t){var n;return n=new Kv,e.xd(n),n.a+="..",t.yd(n),n.a}function yyn(e,t,n){var s;s=h(_n(e.g,n),57),Ue(e.a.c,new As(t,s))}function kyn(e,t,n){return dW(Je(no(Ia(e.f,t))),Je(no(Ia(e.f,n))))}function pB(e,t,n){return fR(e,t,n,ue(t,99)&&(h(t,18).Bb&ea)!=0)}function xyn(e,t,n){return px(e,t,n,ue(t,99)&&(h(t,18).Bb&ea)!=0)}function Eyn(e,t,n){return qCn(e,t,n,ue(t,99)&&(h(t,18).Bb&ea)!=0)}function m0e(e,t){return e==(bn(),Wi)&&t==Wi?4:e==Wi||t==Wi?8:32}function _at(e,t){return Me(t)===Me(e)?"(this Map)":t==null?_c:Ga(t)}function _yn(e,t){return h(t==null?no(Ia(e.f,null)):Z7(e.g,t),281)}function Tat(e,t,n){var s;return s=Jk(n),Pr(e.b,s,t),Pr(e.c,t,n),t}function Cat(e,t){var n;for(n=t;n;)nw(e,n.i,n.j),n=bi(n);return e}function y0e(e,t){var n;return n=sA(sk(new EK(e,t))),CP(new EK(e,t)),n}function R1(e,t){ia();var n;return n=h(e,66).Mj(),i_n(n,t),n.Ok(t)}function Tyn(e,t,n,s,a){var u;u=QCn(a,n,s),Ue(t,K_n(a,u)),$9n(e,a,t)}function Sat(e,t,n){e.i=0,e.e=0,t!=n&&(Zlt(e,t,n),Qlt(e,t,n))}function k0e(e,t){var n;n=e.q.getHours(),e.q.setFullYear(t+Og),C9(e,n)}function Cyn(e,t,n){if(n){var s=n.ee();e.a[t]=s(n)}else delete e.a[t]}function iK(e,t,n){if(n){var s=n.ee();n=s(n)}else n=void 0;e.a[t]=n}function Aat(e){if(e<0)throw ne(new LXe("Negative array size: "+e))}function Co(e){return e.n||(ou(e),e.n=new ort(e,pa,e),La(e)),e.n}function zE(e){return kn(e.a<e.c.a.length),e.b=e.a,Pct(e),e.c.b[e.b]}function x0e(e){e.b!=e.c&&(e.a=pe(yn,st,1,8,5,1),e.b=0,e.c=0)}function Mat(e){this.b=new zn,this.c=new zn,this.d=new zn,this.a=e}function dw(e,t){Mr(),G2.call(this,e),this.a=t,this.c=-1,this.b=-1}function gw(e,t,n,s){Ist.call(this,1,n,s),this.c=e,this.b=t}function sK(e,t,n,s){Ost.call(this,1,n,s),this.c=e,this.b=t}function aK(e,t,n,s,a,u,p){OK.call(this,t,s,a,u,p),this.c=e,this.a=n}function Op(e,t,n){this.e=e,this.a=yn,this.b=dpt(t),this.c=t,this.d=n}function oK(e){this.e=e,this.c=this.e.a,this.b=this.e.g,this.d=this.e.i}function Dat(e){this.c=e,this.a=h(Rl(e),148),this.b=this.a.Aj().Nh()}function E0e(e){this.d=e,this.b=this.d.a.entries(),this.a=this.b.next()}function Np(){zn.call(this),Xet(this),this.d.b=this.d,this.d.a=this.d}function Lat(e,t){m1e.call(this),this.a=e,this.b=t,Ue(this.a.b,this)}function Syn(e,t){var n;return n=t!=null?Fo(e,t):no(Ia(e.f,t)),KN(n)}function Ayn(e,t){var n;return n=t!=null?Fo(e,t):no(Ia(e.f,t)),KN(n)}function Iat(e,t){var n;for(n=0;n<t;++n)pi(e,n,new xle(h(e[n],42)))}function Myn(e,t){var n;for(n=e.d-1;n>=0&&e.a[n]===t[n];n--);return n<0}function Oat(e,t){Uk();var n;return n=e.j.g-t.j.g,n!=0?n:0}function Nat(e,t){return rn(t),e.a!=null?hbn(t.Kb(e.a)):$j}function bB(e){var t;return e?new sde(e):(t=new md,zK(t,e),t)}function Gu(e,t){var n;return t.b.Kb(Cct(e,t.c.Ee(),(n=new Hqe(t),n)))}function vB(e){z2e(),Eet(this,Gn(Zi(dg(e,24),AR)),Gn(Zi(e,AR)))}function Pat(){Pat=re,F3t=ur((SF(),se(ie(e4e,1),ze,428,0,[ute,J3e])))}function Bat(){Bat=re,R3t=ur((OB(),se(ie(n4e,1),ze,427,0,[t4e,lte])))}function Fat(){Fat=re,$4t=ur((NB(),se(ie(D4e,1),ze,424,0,[kte,Xj])))}function Rat(){Rat=re,L5t=ur((Nk(),se(ie(D5t,1),ze,511,0,[kD,Nte])))}function jat(){jat=re,i6t=ur((DA(),se(ie(X5e,1),ze,419,0,[S$,K5e])))}function $at(){$at=re,c6t=ur((xA(),se(ie(J5e,1),ze,479,0,[Z5e,M$])))}function Hat(){Hat=re,Hxt=ur((bA(),se(ie(oxe,1),ze,376,0,[Wne,RD])))}function zat(){zat=re,Fxt=ur((CA(),se(ie(rxe,1),ze,421,0,[Une,qne])))}function Gat(){Gat=re,e6t=ur((TB(),se(ie(z5e,1),ze,422,0,[H5e,Kte])))}function Vat(){Vat=re,p6t=ur((gB(),se(ie(f6e,1),ze,420,0,[lne,h6e])))}function Uat(){Uat=re,I8t=ur((qh(),se(ie(L8t,1),ze,520,0,[fm,qg])))}function qat(){qat=re,c8t=ur((GE(),se(ie(o8t,1),ze,523,0,[$_,j_])))}function Yat(){Yat=re,p8t=ur((r1(),se(ie(g8t,1),ze,516,0,[lv,I0])))}function Wat(){Wat=re,v8t=ur((yf(),se(ie(b8t,1),ze,515,0,[v2,Rf])))}function Kat(){Kat=re,j8t=ur((gb(),se(ie(R8t,1),ze,455,0,[Yg,m3])))}function Xat(){Xat=re,r7t=ur((lB(),se(ie(Lxe,1),ze,425,0,[ure,Dxe])))}function Qat(){Qat=re,c7t=ur((QB(),se(ie(Ixe,1),ze,495,0,[vH,p8])))}function Zat(){Zat=re,n7t=ur((aB(),se(ie(Mxe,1),ze,480,0,[cre,Axe])))}function Jat(){Jat=re,h7t=ur((SB(),se(ie(Nxe,1),ze,426,0,[Oxe,dre])))}function eot(){eot=re,wEt=ur(($A(),se(ie(F8e,1),ze,429,0,[CH,B8e])))}function tot(){tot=re,eEt=ur((vA(),se(ie(p8e,1),ze,430,0,[xre,_H])))}function GE(){GE=re,$_=new tfe("UPPER",0),j_=new tfe("LOWER",1)}function Dyn(e,t){var n;n=new _4,Lp(n,"x",t.a),Lp(n,"y",t.b),z4(e,n)}function Lyn(e,t){var n;n=new _4,Lp(n,"x",t.a),Lp(n,"y",t.b),z4(e,n)}function Iyn(e,t){var n,s;s=!1;do n=Glt(e,t),s=s|n;while(n);return s}function _0e(e,t){var n,s;for(n=t,s=0;n>0;)s+=e.a[n],n-=n&-n;return s}function not(e,t){var n;for(n=t;n;)nw(e,-n.i,-n.j),n=bi(n);return e}function Os(e,t){var n,s;for(rn(t),s=e.Kc();s.Ob();)n=s.Pb(),t.td(n)}function rot(e,t){var n;return n=t.cd(),new Z2(n,e.e.pc(n,h(t.dd(),14)))}function Ai(e,t,n,s){var a;a=new Ce,a.c=t,a.b=n,a.a=s,s.b=n.a=a,++e.b}function dl(e,t,n){var s;return s=(Kt(t,e.c.length),e.c[t]),e.c[t]=n,s}function Oyn(e,t,n){return h(t==null?sc(e.f,null,n):Ab(e.g,t,n),281)}function cK(e){return!!e.c&&!!e.d?o0e(e.c)+"->"+o0e(e.d):"e_"+rb(e)}function yk(e,t){return(Up(e),U7(new Vt(e,new W0e(t,e.a)))).sd(Ux)}function Nyn(){return Zs(),se(ie(U4e,1),ze,356,0,[w1,Hg,oc,Go,Pa])}function Pyn(){return Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])}function Byn(e){return lN(),function(){return Ymn(e,this,arguments)}}function Fyn(){return Date.now?Date.now():new Date().getTime()}function Xs(e){return!e.c||!e.d?!1:!!e.c.i&&e.c.i==e.d.i}function iot(e){if(!e.c.Sb())throw ne(new lo);return e.a=!0,e.c.Ub()}function gA(e){e.i=0,CS(e.b,null),CS(e.c,null),e.a=null,e.e=null,++e.g}function T0e(e){V1n.call(this,e==null?_c:Ga(e),ue(e,78)?h(e,78):null)}function sot(e){Dbt(),SKe(this),this.a=new di,Zge(this,e),vr(this.a,e)}function aot(){SY(this),this.b=new dt(ki,ki),this.a=new dt(Fi,Fi)}function oot(e,t){this.c=0,this.b=t,OJe.call(this,e,17493),this.a=this.c}function uK(e){wB(),!Pf&&(this.c=e,this.e=!0,this.a=new qe)}function wB(){wB=re,Pf=!0,S3t=!1,A3t=!1,D3t=!1,M3t=!1}function C0e(e,t){return ue(t,149)?Nt(e.c,h(t,149).c):!1}function S0e(e,t){var n;return n=0,e&&(n+=e.f.a/2),t&&(n+=t.f.a/2),n}function lK(e,t){var n;return n=h(mb(e.d,t),23),n||h(mb(e.e,t),23)}function cot(e){this.b=e,Tn.call(this,e),this.a=h(tn(this.b.a,4),126)}function uot(e){this.b=e,P4.call(this,e),this.a=h(tn(this.b.a,4),126)}function ou(e){return e.t||(e.t=new vKe(e),s9(new DXe(e),0,e.t)),e.t}function Ryn(){return la(),se(ie(J_,1),ze,103,0,[Q1,Dh,Wl,zd,Gd])}function jyn(){return Cy(),se(ie(rT,1),ze,249,0,[Xg,sL,U7e,nT,q7e])}function $yn(){return Qh(),se(ie(P0,1),ze,175,0,[gn,gs,k1,w2,N0])}function Hyn(){return lM(),se(ie(m8e,1),ze,316,0,[b8e,Ere,w8e,_re,v8e])}function zyn(){return f9(),se(ie(Kke,1),ze,315,0,[Wke,zne,Gne,N_,P_])}function Gyn(){return Kp(),se(ie(W5e,1),ze,335,0,[Qte,Y5e,Zte,k_,y_])}function Vyn(){return fx(),se(ie(W7t,1),ze,355,0,[y3,G5,W_,Y_,K_])}function Uyn(){return Iy(),se(ie(U5t,1),ze,363,0,[x$,_$,T$,E$,k$])}function qyn(){return wl(),se(ie(T6e,1),ze,163,0,[LD,C_,Gg,S_,nm])}function kk(){kk=re;var e,t;JH=(q6(),t=new rN,t),ez=(e=new Cq,e)}function lot(e){var t;return e.c||(t=e.r,ue(t,88)&&(e.c=h(t,26))),e.c}function Yyn(e){return e.e=3,e.d=e.Yb(),e.e!=2?(e.e=0,!0):!1}function hK(e){var t,n,s;return t=e&du,n=e>>22&du,s=e<0?Rd:0,rc(t,n,s)}function Wyn(e){var t,n,s,a;for(n=e,s=0,a=n.length;s<a;++s)t=n[s],PE(t)}function Kyn(e,t){var n,s;n=h(U6n(e.c,t),14),n&&(s=n.gc(),n.$b(),e.d-=s)}function hot(e,t){var n,s;return n=t.cd(),s=Qge(e,n),!!s&&Bo(s.e,t.dd())}function K4(e,t){return t==0||e.e==0?e:t>0?cft(e,t):Dgt(e,-t)}function A0e(e,t){return t==0||e.e==0?e:t>0?Dgt(e,t):cft(e,-t)}function Un(e){if(rr(e))return e.c=e.a,e.a.Pb();throw ne(new lo)}function fot(e){var t,n;return t=e.c.i,n=e.d.i,t.k==(bn(),Pi)&&n.k==Pi}function fK(e){var t;return t=new db,Oa(t,e),Fe(t,(tt(),Da),null),t}function dK(e,t,n){var s;return s=e.Yg(t),s>=0?e._g(s,n,!0):Bb(e,t,n)}function M0e(e,t,n,s){var a;for(a=0;a<fte;a++)OP(e.a[t.g][a],n,s[t.g])}function D0e(e,t,n,s){var a;for(a=0;a<Vj;a++)IP(e.a[a][t.g],n,s[t.g])}function L0e(e,t,n,s,a){Ist.call(this,t,s,a),this.c=e,this.a=n}function I0e(e,t,n,s,a){Ost.call(this,t,s,a),this.c=e,this.a=n}function O0e(e,t,n,s,a){got.call(this,t,s,a),this.c=e,this.a=n}function kh(e,t,n,s,a){got.call(this,t,s,a),this.c=e,this.b=n}function dot(e,t,n){eN.call(this,n),this.b=e,this.c=t,this.d=(_X(),aie)}function got(e,t,n){this.d=e,this.k=t?1:0,this.f=n?1:0,this.o=-1,this.p=0}function pot(e,t,n){var s;s=new Dfe(e.a),QE(s,e.a.a),sc(s.f,t,n),e.a.a=s}function pA(e,t){e.qi(e.i+1),wE(e,e.i,e.oi(e.i,t)),e.bi(e.i++,t),e.ci()}function VE(e){var t,n;++e.j,t=e.g,n=e.i,e.g=null,e.i=0,e.di(n,t),e.ci()}function Ef(e){var t,n;return Vn(e),t=pvn(e.length),n=new Wo(t),Uge(n,e),n}function X4(e){var t;return t=(Vn(e),e?new Bc(e):sk(e.Kc())),pQ(t),dF(t)}function f0(e,t){var n;return n=(Kt(t,e.c.length),e.c[t]),Mhe(e.c,t,1),n}function jr(e,t){var n;return n=h(e.c.xc(t),14),!n&&(n=e.ic(t)),e.pc(t,n)}function N0e(e,t){var n,s;return n=(rn(e),e),s=(rn(t),t),n==s?0:n<s?-1:1}function bot(e){var t;return t=e.e+e.f,isNaN(t)&&bP(e.d)?e.d:t}function Cd(e,t){return e.a?or(e.a,e.b):e.a=new ju(e.d),gE(e.a,t),e}function P0e(e,t){if(e<0||e>t)throw ne(new Ea(hbe(e,t,"index")));return e}function gK(e,t,n,s){var a;return a=pe(Hn,hr,25,t,15,1),o9n(a,e,t,n,s),a}function Xyn(e,t){var n;n=e.q.getHours()+(t/60|0),e.q.setMinutes(t),C9(e,n)}function Qyn(e,t){return d.Math.min(yg(t.a,e.d.d.c),yg(t.b,e.d.d.c))}function Q4(e,t){return ws(t)?t==null?Abe(e.f,null):Tlt(e.g,t):Abe(e.f,t)}function _f(e){this.c=e,this.a=new Q(this.c.a),this.b=new Q(this.c.b)}function mB(){this.e=new qe,this.c=new qe,this.d=new qe,this.b=new qe}function vot(){this.g=new Yle,this.b=new Yle,this.a=new qe,this.k=new qe}function wot(e,t,n){this.a=e,this.c=t,this.d=n,Ue(t.e,this),Ue(n.b,this)}function mot(e,t){IJe.call(this,t.rd(),t.qd()&-6),rn(e),this.a=e,this.b=t}function yot(e,t){OJe.call(this,t.rd(),t.qd()&-6),rn(e),this.a=e,this.b=t}function B0e(e,t){xY.call(this,t.rd(),t.qd()&-6),rn(e),this.a=e,this.b=t}function yB(e,t,n){this.a=e,this.b=t,this.c=n,Ue(e.t,this),Ue(t.i,this)}function kB(){this.b=new di,this.a=new di,this.b=new di,this.a=new di}function xB(){xB=re,X_=new Xr("org.eclipse.elk.labels.labelManager")}function kot(){kot=re,B5e=new Xi("separateLayerConnections",(aF(),Rte))}function qh(){qh=re,fm=new ife("REGULAR",0),qg=new ife("CRITICAL",1)}function bA(){bA=re,Wne=new efe("STACKED",0),RD=new efe("SEQUENCED",1)}function vA(){vA=re,xre=new lfe("FIXED",0),_H=new lfe("CENTER_NODE",1)}function Zyn(e,t){var n;return n=nOn(e,t),e.b=new XB(n.c.length),gIn(e,n)}function Jyn(e,t,n){var s;return++e.e,--e.f,s=h(e.d[t].$c(n),133),s.dd()}function xot(e){var t;return e.a||(t=e.r,ue(t,148)&&(e.a=h(t,148))),e.a}function F0e(e){if(e.a){if(e.e)return F0e(e.e)}else return e;return null}function e3n(e,t){return e.p<t.p?1:e.p>t.p?-1:0}function EB(e,t){return rn(t),e.c<e.d?(e.ze(t,e.c++),!0):!1}function Eot(e,t){return Au(e.a,t)?(Q4(e.a,t),!0):!1}function t3n(e){var t,n;return t=e.cd(),n=h(e.dd(),14),QS(n.Nc(),new XUe(t))}function n3n(e){var t;return t=h(Ide(e.b,e.b.length),9),new ll(e.a,t,e.c)}function r3n(e){var t;return Up(e),t=new ynt(e,e.a.e,e.a.d|4),new u1e(e,t)}function _ot(e){var t;for(vg(e),t=0;e.a.sd(new si);)t=Vs(t,1);return t}function R0e(e,t,n){var s,a;for(s=0,a=0;a<t.length;a++)s+=e.$f(t[a],s,n)}function i3n(e,t){var n;e.C&&(n=h(wa(e.b,t),124).n,n.d=e.C.d,n.a=e.C.a)}function Z4(e,t,n){return dA(t,e.e.Hd().gc()),dA(n,e.c.Hd().gc()),e.a[t][n]}function d0(e,t){Dg(),this.e=e,this.d=1,this.a=se(ie(Hn,1),hr,25,15,[t])}function _B(e,t,n,s){this.f=e,this.e=t,this.d=n,this.b=s,this.c=s?s.d:null}function j0e(e){var t,n,s,a;a=e.d,t=e.a,n=e.b,s=e.c,e.d=n,e.a=s,e.b=a,e.c=t}function s3n(e,t,n,s){Xpt(e,t,n,px(e,t,s,ue(t,99)&&(h(t,18).Bb&ea)!=0))}function a3n(e,t){Bn(t,"Label management",1),KN(U(e,(xB(),X_))),An(t)}function Wo(e){SY(this),mP(e>=0,"Initial capacity must not be negative")}function Tot(){Tot=re,z3t=ur((Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])))}function Cot(){Cot=re,V3t=ur((Xc(),se(ie(G3t,1),ze,461,0,[v1,jg,Sh])))}function Sot(){Sot=re,q3t=ur((wc(),se(ie(U3t,1),ze,462,0,[nf,$g,Ah])))}function Aot(){Aot=re,L3t=ur((Af(),se(ie(gu,1),ze,132,0,[X3e,Zu,Kw])))}function Mot(){Mot=re,o5t=ur((UE(),se(ie(V4e,1),ze,379,0,[Tte,_te,Cte])))}function Dot(){Dot=re,x5t=ur((Sb(),se(ie(W4e,1),ze,423,0,[i3,Y4e,Lte])))}function Lot(){Lot=re,t6t=ur((e5(),se(ie(V5e,1),ze,314,0,[I5,_D,G5e])))}function Iot(){Iot=re,n6t=ur((GB(),se(ie(q5e,1),ze,337,0,[U5e,C$,Xte])))}function Oot(){Oot=re,a6t=ur((Hp(),se(ie(s6t,1),ze,450,0,[Jte,t8,c3])))}function Not(){Not=re,Q5t=ur((Tb(),se(ie($te,1),ze,361,0,[ev,zg,Jb])))}function Pot(){Pot=re,g6t=ur((Ad(),se(ie(d6t,1),ze,303,0,[CD,l3,O5])))}function Bot(){Bot=re,f6t=ur((jk(),se(ie(une,1),ze,292,0,[one,cne,TD])))}function Fot(){Fot=re,Ixt=ur((t9(),se(ie(Yke,1),ze,378,0,[jne,qke,nH])))}function Rot(){Rot=re,$xt=ur((nF(),se(ie(axe,1),ze,375,0,[ixe,Yne,sxe])))}function jot(){jot=re,Bxt=ur((Dd(),se(ie(nxe,1),ze,339,0,[Ug,txe,Vne])))}function $ot(){$ot=re,jxt=ur((ua(),se(ie(Rxt,1),ze,452,0,[B_,Jc,ec])))}function Hot(){Hot=re,Vxt=ur((oF(),se(ie(dxe,1),ze,377,0,[Qne,d8,hm])))}function zot(){zot=re,zxt=ur((qk(),se(ie(uxe,1),ze,336,0,[Kne,cxe,F_])))}function Got(){Got=re,Gxt=ur((eF(),se(ie(fxe,1),ze,338,0,[hxe,Xne,lxe])))}function Vot(){Vot=re,r8t=ur((_b(),se(ie(n8t,1),ze,454,0,[jD,R_,aH])))}function Uot(){Uot=re,l7t=ur((wF(),se(ie(u7t,1),ze,442,0,[fre,lre,hre])))}function qot(){qot=re,f7t=ur((VA(),se(ie(Fxe,1),ze,380,0,[wH,Pxe,Bxe])))}function Yot(){Yot=re,M7t=ur((vF(),se(ie(t8e,1),ze,381,0,[e8e,wre,Jxe])))}function Wot(){Wot=re,A7t=ur((tF(),se(ie(Qxe,1),ze,293,0,[vre,Xxe,Kxe])))}function Kot(){Kot=re,X7t=ur((XA(),se(ie(mre,1),ze,437,0,[kH,xH,EH])))}function Xot(){Xot=re,JEt=ur((Ld(),se(ie(V7e,1),ze,334,0,[BH,B0,tT])))}function Qot(){Qot=re,KEt=ur((Tf(),se(ie(I7e,1),ze,272,0,[w8,pm,m8])))}function o3n(){return _s(),se(ie(Y7e,1),ze,98,0,[Qg,$f,k8,x2,ed,qo])}function Pp(e,t){return!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),TX(e.o,t)}function c3n(e){return!e.g&&(e.g=new VO),!e.g.d&&(e.g.d=new gKe(e)),e.g.d}function u3n(e){return!e.g&&(e.g=new VO),!e.g.a&&(e.g.a=new pKe(e)),e.g.a}function l3n(e){return!e.g&&(e.g=new VO),!e.g.b&&(e.g.b=new dKe(e)),e.g.b}function wA(e){return!e.g&&(e.g=new VO),!e.g.c&&(e.g.c=new bKe(e)),e.g.c}function h3n(e,t,n){var s,a;for(a=new Rk(t,e),s=0;s<n;++s)OF(a);return a}function pK(e,t,n){var s,a;if(n!=null)for(s=0;s<t;++s)a=n[s],e.fi(s,a)}function bK(e,t,n,s){var a;return a=pe(Hn,hr,25,t+1,15,1),YDn(a,e,t,n,s),a}function pe(e,t,n,s,a,u){var p;return p=u1t(a,s),a!=10&&se(ie(e,u),t,n,a,p),p}function f3n(e,t,n,s){return n&&(s=n.gh(t,oi(n.Tg(),e.c.Lj()),null,s)),s}function d3n(e,t,n,s){return n&&(s=n.ih(t,oi(n.Tg(),e.c.Lj()),null,s)),s}function $0e(e,t,n){h(e.b,65),h(e.b,65),h(e.b,65),mc(e.a,new ant(n,t,e))}function H0e(e,t,n){if(e<0||t>n||t<e)throw ne(new vhe(MR+e+awe+t+iwe+n))}function xk(e){if(!e)throw ne(new ja("Unable to add element to queue"))}function vK(e){e?(this.c=e,this.b=null):(this.c=null,this.b=new qe)}function wK(e,t){LN.call(this,e,t),this.a=pe(yPn,TR,436,2,0,1),this.b=!0}function z0e(e){$lt.call(this,e,0),Xet(this),this.d.b=this.d,this.d.a=this.d}function mK(e){var t;return t=e.b,t.b==0?null:h(Zh(t,0),188).b}function Zot(e,t){var n;return n=new $e,n.c=!0,n.d=t.dd(),q2t(e,t.cd(),n)}function g3n(e,t){var n;n=e.q.getHours()+(t/3600|0),e.q.setSeconds(t),C9(e,n)}function G0e(e,t,n){var s;s=e.b[n.c.p][n.p],s.b+=t.b,s.c+=t.c,s.a+=t.a,++s.a}function yg(e,t){var n,s;return n=e.a-t.a,s=e.b-t.b,d.Math.sqrt(n*n+s*s)}function TB(){TB=re,H5e=new Khe("QUADRATIC",0),Kte=new Khe("SCANLINE",1)}function Jot(){Jot=re,Yxt=Wc(Lr(new rs,(Zs(),w1),(oa(),Pte)),Pa,xD)}function p3n(){return Aw(),se(ie(Nre,1),ze,291,0,[Ore,QD,XD,Ire,WD,KD])}function b3n(){return $1(),se(ie(z8e,1),ze,248,0,[Dre,qD,YD,MH,SH,AH])}function v3n(){return u5(),se(ie(e8,1),ze,227,0,[Jx,m_,Zx,Jw,o3,a3])}function w3n(){return ax(),se(ie(l6e,1),ze,275,0,[x_,a6e,u6e,c6e,o6e,s6e])}function m3n(){return cM(),se(ie(i6e,1),ze,274,0,[D$,t6e,r6e,e6e,n6e,sne])}function y3n(){return UF(),se(ie(Uke,1),ze,313,0,[Rne,Gke,Fne,zke,Vke,tH])}function k3n(){return KF(),se(ie(Q5e,1),ze,276,0,[tne,ene,rne,nne,ine,A$])}function x3n(){return w9(),se(ie(H8t,1),ze,327,0,[hH,ere,nre,tre,rre,Jne])}function E3n(){return Qc(),se(ie(FH,1),ze,273,0,[Zg,Ud,aL,sT,iT,K5])}function _3n(){return YF(),se(ie(R7e,1),ze,312,0,[Wre,P7e,F7e,O7e,B7e,N7e])}function T3n(){return bn(),se(ie(Ote,1),ze,267,0,[Wi,ds,Pi,Vo,Ou,Bf])}function ect(e){tw(!!e.c),nK(e.e,e),e.c.Qb(),e.c=null,e.b=Oge(e),a1e(e.e,e)}function tct(e){return nK(e.c.a.e,e),kn(e.b!=e.c.a.d),e.a=e.b,e.b=e.b.a,e.a}function V0e(e){var t;return!e.a&&e.b!=-1&&(t=e.c.Tg(),e.a=Ht(t,e.b)),e.a}function qn(e,t){return e.hi()&&e.Hc(t)?!1:(e.Yh(t),!0)}function Yh(e,t){return aA(t,"Horizontal alignment cannot be null"),e.b=t,e}function nct(e,t,n){Mr();var s;return s=Lg(e,t),n&&!!s&&evn(e)&&(s=null),s}function g0(e,t,n){var s,a;for(a=e.Kc();a.Ob();)s=h(a.Pb(),37),_9(s,t,n)}function U0e(e,t){var n,s;for(s=t.Kc();s.Ob();)n=h(s.Pb(),37),_pt(e,n,0,0)}function q0e(e,t,n){var s;e.d[t.g]=n,s=e.g.c,s[t.g]=d.Math.max(s[t.g],n+1)}function mA(e,t){var n,s,a;return a=e.r,s=e.d,n=M9(e,t,!0),n.b!=a||n.a!=s}function rct(e,t){return wZe(e.e,t)||Yp(e.e,t,new Nft(t)),h(mb(e.e,t),113)}function Bp(e,t,n,s){return rn(e),rn(t),rn(n),rn(s),new Cde(e,t,new xe)}function Du(e,t,n,s){this.rj(),this.a=t,this.b=e,this.c=new Ede(this,t,n,s)}function yK(e,t,n,s,a,u){g0e.call(this,t,s,a,u),this.c=e,this.b=n}function yA(e,t,n,s,a,u){g0e.call(this,t,s,a,u),this.c=e,this.a=n}function C3n(e,t,n){var s,a,u;s=Ed(e,n),a=null,s&&(a=ibe(s)),u=a,Lft(t,n,u)}function S3n(e,t,n){var s,a,u;s=Ed(e,n),a=null,s&&(a=ibe(s)),u=a,Lft(t,n,u)}function kA(e,t,n){var s,a;return a=(s=lx(e.b,t),s),a?bR(TA(e,a),n):null}function Ek(e,t){var n;return n=e.Yg(t),n>=0?e._g(n,!0,!0):Bb(e,t,!0)}function A3n(e,t){return Ui(Pe(Je(U(e,(He(),iv)))),Pe(Je(U(t,iv))))}function ict(){ict=re,s7t=Ib(Ib(CN(new rs,(tx(),H_)),(w9(),hH)),ere)}function M3n(e,t,n){var s;return s=Wlt(e,t,n),e.b=new XB(s.c.length),Jbe(e,s)}function D3n(e){if(e.b<=0)throw ne(new lo);return--e.b,e.a-=e.c.c,Ke(e.a)}function L3n(e){var t;if(!e.a)throw ne(new cit);return t=e.a,e.a=bi(e.a),t}function I3n(e){for(;!e.a;)if(!qtt(e.c,new Rqe(e)))return!1;return!0}function J4(e){var t;return Vn(e),ue(e,198)?(t=h(e,198),t):new rqe(e)}function O3n(e){CB(),h(e.We((Tr(),gm)),174).Fc((Qc(),aL)),e.Ye(Gre,null)}function CB(){CB=re,kEt=new VHe,EEt=new UHe,xEt=v6n((Tr(),Gre),kEt,Wg,EEt)}function SB(){SB=re,Oxe=new ufe("LEAF_NUMBER",0),dre=new ufe("NODE_SIZE",1)}function N3n(e,t,n){e.a=t,e.c=n,e.b.a.$b(),Pl(e.d),e.e.a.c=pe(yn,st,1,0,5,1)}function kK(e){e.a=pe(Hn,hr,25,e.b+1,15,1),e.c=pe(Hn,hr,25,e.b,15,1),e.d=0}function P3n(e,t){e.a.ue(t.d,e.b)>0&&(Ue(e.c,new Z1e(t.c,t.d,e.d)),e.b=t.d)}function Y0e(e,t){if(e.g==null||t>=e.i)throw ne(new EY(t,e.i));return e.g[t]}function sct(e,t,n){if(Gk(e,n),n!=null&&!e.wj(n))throw ne(new Eq);return n}function act(e){var t;if(e.Ek())for(t=e.i-1;t>=0;--t)he(e,t);return w0e(e)}function B3n(e){var t,n;if(!e.b)return null;for(n=e.b;t=n.a[0];)n=t;return n}function F3n(e,t){var n,s;return Aat(t),n=(s=e.slice(0,t),K0e(s,e)),n.length=t,n}function _k(e,t,n,s){var a;s=(gy(),s||j3e),a=e.slice(t,n),fbe(a,e,t,n,-t,s)}function gl(e,t,n,s,a){return t<0?Bb(e,n,s):h(n,66).Nj().Pj(e,e.yh(),t,s,a)}function R3n(e){return ue(e,172)?""+h(e,172).a:e==null?null:Ga(e)}function j3n(e){return ue(e,172)?""+h(e,172).a:e==null?null:Ga(e)}function oct(e,t){if(t.a)throw ne(new Ya(ovt));Qi(e.a,t),t.a=e,!e.j&&(e.j=t)}function W0e(e,t){xY.call(this,t.rd(),t.qd()&-16449),rn(e),this.a=e,this.c=t}function cct(e,t){var n,s;return s=t/e.c.Hd().gc()|0,n=t%e.c.Hd().gc(),Z4(e,s,n)}function Xc(){Xc=re,v1=new Xq(w5,0),jg=new Xq(Cx,1),Sh=new Xq(m5,2)}function AB(){AB=re,ate=new DN("All",0),q3e=new tet,Y3e=new get,W3e=new net}function uct(){uct=re,T3t=ur((AB(),se(ie(Hj,1),ze,297,0,[ate,q3e,Y3e,W3e])))}function lct(){lct=re,b5t=ur(($k(),se(ie(p5t,1),ze,405,0,[Zb,Zw,Qw,r3])))}function hct(){hct=re,v4t=ur((Cb(),se(ie(b4t,1),ze,406,0,[fD,hD,gte,pte])))}function fct(){fct=re,m4t=ur((o5(),se(ie(w4t,1),ze,323,0,[gD,dD,pD,bD])))}function dct(){dct=re,x4t=ur((h9(),se(ie(k4t,1),ze,394,0,[vD,qj,Yj,wD])))}function gct(){gct=re,$8t=ur((tx(),se(ie(wxe,1),ze,393,0,[lH,H_,HD,z_])))}function pct(){pct=re,P5t=ur((aF(),se(ie(N5t,1),ze,360,0,[Rte,m$,y$,ED])))}function bct(){bct=re,S7t=ur((FF(),se(ie(Wxe,1),ze,340,0,[bre,qxe,Yxe,Uxe])))}function vct(){vct=re,V5t=ur((Sf(),se(ie(G5t,1),ze,411,0,[L5,Xx,Qx,jte])))}function wct(){wct=re,Oxt=ur((Tw(),se(ie(Hne,1),ze,197,0,[rH,$ne,w3,v3])))}function mct(){mct=re,b9t=ur((Lu(),se(ie(p9t,1),ze,396,0,[rl,tEe,eEe,nEe])))}function yct(){yct=re,t9t=ur((Yu(),se(ie(e9t,1),ze,285,0,[iL,Z1,Kg,rL])))}function kct(){kct=re,XEt=ur((Od(),se(ie(Yre,1),ze,218,0,[qre,nL,y8,W5])))}function xct(){xct=re,d9t=ur((kF(),se(ie(J7e,1),ze,311,0,[Xre,X7e,Z7e,Q7e])))}function Ect(){Ect=re,h9t=ur((Iu(),se(ie(aT,1),ze,374,0,[cL,E2,oL,bm])))}function _ct(){_ct=re,dR(),FEe=ki,w_t=Fi,REe=new cS(ki),m_t=new cS(Fi)}function xA(){xA=re,Z5e=new Qhe(jd,0),M$=new Qhe("IMPROVE_STRAIGHTNESS",1)}function $3n(e,t){return ak(),Ue(e,new As(t,Ke(t.e.c.length+t.g.c.length)))}function H3n(e,t){return ak(),Ue(e,new As(t,Ke(t.e.c.length+t.g.c.length)))}function K0e(e,t){return SA(t)!=10&&se(uu(t),t.hm,t.__elementTypeId$,SA(t),e),e}function vc(e,t){var n;return n=Ha(e,t,0),n==-1?!1:(f0(e,n),!0)}function Tct(e,t){var n;return n=h(Q4(e.e,t),387),n?(tde(n),n.e):null}function Tk(e){var t;return $a(e)&&(t=0-e,!isNaN(t))?t:xg(Hk(e))}function Ha(e,t,n){for(;n<e.c.length;++n)if(Bo(t,e.c[n]))return n;return-1}function Cct(e,t,n){var s;return vg(e),s=new Gt,s.a=t,e.a.Nb(new yZe(s,n)),s.a}function z3n(e){var t;return vg(e),t=pe(ks,ka,25,0,15,1),ty(e.a,new Pqe(t)),t}function xK(e){var t,n;return n=h(ut(e.j,0),11),t=h(U(n,(He(),Br)),11),t}function X0e(e){var t;if(!FK(e))throw ne(new lo);return e.e=1,t=e.d,e.d=null,t}function EK(e,t){var n;this.f=e,this.b=t,n=h(_n(e.b,t),283),this.c=n?n.b:null}function Sct(){wh(),this.b=new zn,this.f=new zn,this.g=new zn,this.e=new zn}function Act(e,t){this.a=pe(X1,T0,10,e.a.c.length,0,1),Mf(e.a,this.a),this.b=t}function EA(e){var t;for(t=e.p+1;t<e.c.a.c.length;++t)--h(ut(e.c.a,t),10).p}function _K(e){var t;t=e.Ai(),t!=null&&e.d!=-1&&h(t,92).Ng(e),e.i&&e.i.Fi()}function MB(e){nP(this),this.g=e?$W(e,e.$d()):null,this.f=e,KP(this),this._d()}function Sd(e,t,n,s,a,u,p){OK.call(this,t,s,a,u,p),this.c=e,this.b=n}function pw(e,t,n,s,a){return rn(e),rn(t),rn(n),rn(s),rn(a),new Cde(e,t,s)}function _A(e,t){if(t<0)throw ne(new Ea(Lwt+t));return wat(e,t+1),ut(e.j,t)}function Mct(e,t,n,s){if(!e)throw ne(new an(E9(t,se(ie(yn,1),st,1,5,[n,s]))))}function DB(e,t){return Bo(t,ut(e.f,0))||Bo(t,ut(e.f,1))||Bo(t,ut(e.f,2))}function G3n(e,t){F4(h(h(e.f,33).We((Tr(),Z_)),98))&&Gkn(DW(h(e.f,33)),t)}function TA(e,t){var n,s;return n=h(t,675),s=n.Oh(),!s&&n.Rh(s=new CJe(e,t)),s}function Sa(e,t){var n,s;return n=h(t,677),s=n.pk(),!s&&n.tk(s=new qst(e,t)),s}function cu(e){return e.b||(e.b=new crt(e,pa,e),!e.a&&(e.a=new qS(e,e))),e.b}function UE(){UE=re,Tte=new Zq("XY",0),_te=new Zq("X",1),Cte=new Zq("Y",2)}function wc(){wc=re,nf=new Qq("TOP",0),$g=new Qq(Cx,1),Ah=new Qq(pwe,2)}function Ad(){Ad=re,CD=new sY(jd,0),l3=new sY("TOP",1),O5=new sY(pwe,2)}function CA(){CA=re,Une=new Jhe("INPUT_ORDER",0),qne=new Jhe("PORT_DEGREE",1)}function Ck(){Ck=re,w3e=rc(du,du,524287),f3t=rc(0,0,FM),m3e=hK(1),hK(2),y3e=hK(0)}function Q0e(e,t,n){e.a.c=pe(yn,st,1,0,5,1),OIn(e,t,n),e.a.c.length==0||hDn(e,t)}function LB(e){var t,n;return n=e.length,t=pe(Cl,o1,25,n,15,1),Lrt(e,0,n,t,0),t}function Z0e(e){var t;return e.dh()||(t=En(e.Tg())-e.Ah(),e.ph().bk(t)),e.Pg()}function J0e(e){var t;return t=Fp(tn(e,32)),t==null&&(ic(e),t=Fp(tn(e,32))),t}function TK(e,t){var n;return n=oi(e.d,t),n>=0?IF(e,n,!0,!0):Bb(e,t,!0)}function ege(e,t){W7();var n,s;return n=W4(e),s=W4(t),!!n&&!!s&&!Hft(n.k,s.k)}function V3n(e,t){yc(e,t==null||bP((rn(t),t))||isNaN((rn(t),t))?0:(rn(t),t))}function U3n(e,t){kc(e,t==null||bP((rn(t),t))||isNaN((rn(t),t))?0:(rn(t),t))}function q3n(e,t){Eb(e,t==null||bP((rn(t),t))||isNaN((rn(t),t))?0:(rn(t),t))}function Y3n(e,t){xb(e,t==null||bP((rn(t),t))||isNaN((rn(t),t))?0:(rn(t),t))}function Dct(e){(this.q?this.q:(Rt(),Rt(),K1)).Ac(e.q?e.q:(Rt(),Rt(),K1))}function W3n(e,t){return ue(t,99)&&(h(t,18).Bb&ea)!=0?new _Y(t,e):new Rk(t,e)}function K3n(e,t){return ue(t,99)&&(h(t,18).Bb&ea)!=0?new _Y(t,e):new Rk(t,e)}function Lct(e,t){k4e=new Bi,y4t=t,g_=e,h(g_.b,65),$0e(g_,k4e,null),_2t(g_)}function CK(e,t,n){var s;return s=e.g[t],wE(e,t,e.oi(t,n)),e.gi(t,n,s),e.ci(),s}function IB(e,t){var n;return n=e.Xc(t),n>=0?(e.$c(n),!0):!1}function SK(e){var t;return e.d!=e.r&&(t=Rl(e),e.e=!!t&&t.Cj()==dyt,e.d=t),e.e}function AK(e,t){var n;for(Vn(e),Vn(t),n=!1;t.Ob();)n=n|e.Fc(t.Pb());return n}function mb(e,t){var n;return n=h(_n(e.e,t),387),n?(xet(e,n),n.e):null}function Ict(e){var t,n;return t=e/60|0,n=e%60,n==0?""+t:""+t+":"+(""+n)}function Xa(e,t){var n,s;return Up(e),s=new B0e(t,e.a),n=new Ktt(s),new Vt(e,n)}function bw(e,t){var n=e.a[t],s=(YK(),tte)[typeof n];return s?s(n):ipe(typeof n)}function X3n(e){switch(e.g){case 0:return Ir;case 1:return-1;default:return 0}}function Q3n(e){return k2e(e,(Ck(),y3e))<0?-d0n(Hk(e)):e.l+e.m*v5+e.h*Jp}function SA(e){return e.__elementTypeCategory$==null?10:e.__elementTypeCategory$}function MK(e){var t;return t=e.b.c.length==0?null:ut(e.b,0),t!=null&&jK(e,0),t}function Oct(e,t){for(;t[0]<e.length&&Zf(` 	\r
`,Ec(Is(e,t[0])))>=0;)++t[0]}function AA(e,t){this.e=t,this.a=Clt(e),this.a<54?this.f=vb(e):this.c=eM(e)}function Nct(e,t,n,s){Mr(),G2.call(this,26),this.c=e,this.a=t,this.d=n,this.b=s}function j1(e,t,n){var s,a;for(s=10,a=0;a<n-1;a++)t<s&&(e.a+="0"),s*=10;e.a+=t}function Z3n(e,t){var n;for(n=0;e.e!=e.i.gc();)q2n(t,On(e),Ke(n)),n!=Ir&&++n}function tge(e,t){var n;for(++e.d,++e.c[t],n=t+1;n<e.a.length;)++e.a[n],n+=n&-n}function J3n(e,t){var n,s,a;a=t.c.i,n=h(_n(e.f,a),57),s=n.d.c-n.e.c,Tge(t.a,s,0)}function MA(e){var t,n;return t=e+128,n=(jnt(),x3e)[t],!n&&(n=x3e[t]=new xqe(e)),n}function cr(e,t){var n;return rn(t),n=e[":"+t],i4n(!!n,se(ie(yn,1),st,1,5,[t])),n}function e4n(e){var t,n;if(e.b){n=null;do t=e.b,e.b=null,n=Ldt(t,n);while(e.b);e.b=n}}function t4n(e){var t,n;if(e.a){n=null;do t=e.a,e.a=null,n=Ldt(t,n);while(e.a);e.a=n}}function Pct(e){var t;for(++e.a,t=e.c.a.length;e.a<t;++e.a)if(e.c.b[e.a])return}function n4n(e,t){var n,s;for(s=t.c,n=s+1;n<=t.f;n++)e.a[n]>e.a[s]&&(s=n);return s}function r4n(e,t){var n;return n=Mb(e.e.c,t.e.c),n==0?Ui(e.e.d,t.e.d):n}function vy(e,t){return t.e==0||e.e==0?Gx:(cx(),DQ(e,t))}function i4n(e,t){if(!e)throw ne(new an(oAn("Enum constant undefined: %s",t)))}function qE(){qE=re,m5t=new M7,y5t=new iS,v5t=new z2,w5t=new _Be,k5t=new TBe}function OB(){OB=re,t4e=new Uhe("BY_SIZE",0),lte=new Uhe("BY_SIZE_AND_SHAPE",1)}function NB(){NB=re,kte=new qhe("EADES",0),Xj=new qhe("FRUCHTERMAN_REINGOLD",1)}function DA(){DA=re,S$=new Xhe("READING_DIRECTION",0),K5e=new Xhe("ROTATION",1)}function Bct(){Bct=re,r6t=ur((Kp(),se(ie(W5e,1),ze,335,0,[Qte,Y5e,Zte,k_,y_])))}function Fct(){Fct=re,Nxt=ur((f9(),se(ie(Kke,1),ze,315,0,[Wke,zne,Gne,N_,P_])))}function Rct(){Rct=re,q5t=ur((Iy(),se(ie(U5t,1),ze,363,0,[x$,_$,T$,E$,k$])))}function jct(){jct=re,b6t=ur((wl(),se(ie(T6e,1),ze,163,0,[LD,C_,Gg,S_,nm])))}function $ct(){$ct=re,tEt=ur((lM(),se(ie(m8e,1),ze,316,0,[b8e,Ere,w8e,_re,v8e])))}function Hct(){Hct=re,_Et=ur((Qh(),se(ie(P0,1),ze,175,0,[gn,gs,k1,w2,N0])))}function zct(){zct=re,K7t=ur((fx(),se(ie(W7t,1),ze,355,0,[y3,G5,W_,Y_,K_])))}function Gct(){Gct=re,h5t=ur((Zs(),se(ie(U4e,1),ze,356,0,[w1,Hg,oc,Go,Pa])))}function Vct(){Vct=re,WEt=ur((la(),se(ie(J_,1),ze,103,0,[Q1,Dh,Wl,zd,Gd])))}function Uct(){Uct=re,r9t=ur((Cy(),se(ie(rT,1),ze,249,0,[Xg,sL,U7e,nT,q7e])))}function qct(){qct=re,a9t=ur((Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])))}function DK(e,t){var n;return n=h(_n(e.a,t),134),n||(n=new bd,Pr(e.a,t,n)),n}function Yct(e){var t;return t=h(U(e,(He(),tv)),305),t?t.a==e:!1}function Wct(e){var t;return t=h(U(e,(He(),tv)),305),t?t.i==e:!1}function Kct(e,t){return rn(t),kde(e),e.d.Ob()?(t.td(e.d.Pb()),!0):!1}function PB(e){return mo(e,Ir)>0?Ir:mo(e,$s)<0?$s:Gn(e)}function vw(e){return e<3?(Uu(e,Gbt),e+1):e<N9?Li(e/.75+1):Ir}function Ht(e,t){var n;return n=(e.i==null&&s1(e),e.i),t>=0&&t<n.length?n[t]:null}function Wh(e,t,n){var s;if(t==null)throw ne(new P7);return s=Ed(e,t),Cyn(e,t,n),s}function Xct(e){return e.a>=-.01&&e.a<=If&&(e.a=0),e.b>=-.01&&e.b<=If&&(e.b=0),e}function Qct(e,t){return t==(KY(),KY(),x3t)?e.toLocaleLowerCase():e.toLowerCase()}function nge(e){return((e.i&2)!=0?"interface ":(e.i&1)!=0?"":"class ")+(yd(e),e.o)}function Aa(e){var t,n;n=(t=new Sq,t),qn((!e.q&&(e.q=new Ye(eh,e,11,10)),e.q),n)}function s4n(e,t){var n;return n=t>0?t-1:t,UXe(m1n(xut(ede(new H6,n),e.n),e.j),e.k)}function a4n(e,t,n,s){var a;e.j=-1,Tbe(e,sbe(e,t,n),(ia(),a=h(t,66).Mj(),a.Ok(s)))}function Zct(e){this.g=e,this.f=new qe,this.a=d.Math.min(this.g.c.c,this.g.d.c)}function Jct(e){this.b=new qe,this.a=new qe,this.c=new qe,this.d=new qe,this.e=e}function eut(e,t){this.a=new zn,this.e=new zn,this.b=(t9(),nH),this.c=e,this.b=t}function tut(e,t,n){gP.call(this),rge(this),this.a=e,this.c=n,this.b=t.d,this.f=t.e}function nut(e){this.d=e,this.c=e.c.vc().Kc(),this.b=null,this.a=null,this.e=(aN(),Qee)}function yb(e){if(e<0)throw ne(new an("Illegal Capacity: "+e));this.g=this.ri(e)}function o4n(e,t){if(0>e||e>t)throw ne(new bhe("fromIndex: 0, toIndex: "+e+iwe+t))}function c4n(e){var t;if(e.a==e.b.a)throw ne(new lo);return t=e.a,e.c=t,e.a=e.a.e,t}function BB(e){var t;tw(!!e.c),t=e.c.a,pl(e.d,e.c),e.b==e.c?e.b=t:--e.a,e.c=null}function FB(e,t){var n;return Up(e),n=new _it(e,e.a.rd(),e.a.qd()|4,t),new Vt(e,n)}function u4n(e,t){var n,s;return n=h(xw(e.d,t),14),n?(s=t,e.e.pc(s,n)):null}function RB(e,t){var n,s;for(s=e.Kc();s.Ob();)n=h(s.Pb(),70),Fe(n,(He(),F5),t)}function l4n(e){var t;return t=Pe(Je(U(e,(tt(),D0)))),t<0&&(t=0,Fe(e,D0,t)),t}function h4n(e,t,n){var s;s=d.Math.max(0,e.b/2-.5),g9(n,s,1),Ue(t,new CZe(n,s))}function f4n(e,t,n){var s;return s=e.a.e[h(t.a,10).p]-e.a.e[h(n.a,10).p],Li(eA(s))}function rut(e,t,n,s,a,u){var p;p=fK(s),Gs(p,a),xs(p,u),Ot(e.a,s,new vP(p,t,n.f))}function iut(e,t){var n;if(n=yM(e.Tg(),t),!n)throw ne(new an(Fg+t+gee));return n}function ww(e,t){var n;for(n=e;bi(n);)if(n=bi(n),n==t)return!0;return!1}function d4n(e,t){var n,s,a;for(s=t.a.cd(),n=h(t.a.dd(),14).gc(),a=0;a<n;a++)e.td(s)}function mc(e,t){var n,s,a,u;for(rn(t),s=e.c,a=0,u=s.length;a<u;++a)n=s[a],t.td(n)}function pl(e,t){var n;return n=t.c,t.a.b=t.b,t.b.a=t.a,t.a=t.b=null,t.c=null,--e.b,n}function g4n(e,t){return!!t&&e.b[t.g]==t?(pi(e.b,t.g,null),--e.c,!0):!1}function sut(e,t){return!!XE(e,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))))}function p4n(e,t){F4(h(U(h(e.e,10),(tt(),Ei)),98))&&(Rt(),hs(h(e.e,10).j,t))}function rge(e){e.b=(Xc(),jg),e.f=(wc(),$g),e.d=(Uu(2,jw),new Wo(2)),e.e=new Rs}function Kh(){Kh=re,ao=new Kq("BEGIN",0),Jo=new Kq(Cx,1),oo=new Kq("END",2)}function Tf(){Tf=re,w8=new mY(Cx,0),pm=new mY("HEAD",1),m8=new mY("TAIL",2)}function b4n(){return ux(),se(ie(WH,1),ze,237,0,[eie,qH,YH,UH,Jre,VH,GH,Zre])}function v4n(){return E0(),se(ie(TEt,1),ze,277,0,[j8e,zs,po,b8,Ba,_i,V5,jf])}function w4n(){return sR(),se(ie($5e,1),ze,270,0,[zte,Ute,Hte,Wte,Vte,Gte,Yte,qte])}function m4n(){return By(),se(ie(exe,1),ze,260,0,[iH,BD,FD,Qke,Zke,Xke,Jke,sH])}function aut(){aut=re,i9t=ur((_s(),se(ie(Y7e,1),ze,98,0,[Qg,$f,k8,x2,ed,qo])))}function out(){out=re,Vj=(Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])).length,fte=Vj}function jB(e){this.b=(Vn(e),new Bc(e)),this.a=new qe,this.d=new qe,this.e=new Rs}function y4n(e){var t;return t=d.Math.sqrt(e.a*e.a+e.b*e.b),t>0&&(e.a/=t,e.b/=t),e}function Vu(e){var t;return e.w?e.w:(t=xmn(e),!!t&&!t.kh()&&(e.w=t),t)}function k4n(e){var t;return e==null?null:(t=h(e,190),x9n(t,t.length))}function he(e,t){if(e.g==null||t>=e.i)throw ne(new EY(t,e.i));return e.li(t,e.g[t])}function x4n(e){var t,n;for(t=e.a.d.j,n=e.c.d.j;t!=n;)Eh(e.b,t),t=pF(t);Eh(e.b,t)}function E4n(e){var t;for(t=0;t<e.c.length;t++)(Kt(t,e.c.length),h(e.c[t],11)).p=t}function _4n(e,t,n){var s,a,u;for(a=t[n],s=0;s<a.length;s++)u=a[s],e.e[u.c.p][u.p]=s}function LK(e,t){var n,s,a,u;for(s=e.d,a=0,u=s.length;a<u;++a)n=s[a],wf(e.g,n).a=t}function mw(e,t){var n,s;for(s=pr(e,0);s.b!=s.d.c;)n=h(gr(s),8),zr(n,t);return e}function T4n(e,t){var n;return n=ms(ro(h(_n(e.g,t),8)),Mfe(h(_n(e.f,t),460).b)),n}function kb(e){var t;return nK(e.e,e),kn(e.b),e.c=e.a,t=h(e.a.Pb(),42),e.b=Oge(e),t}function Fp(e){var t;return TE(e==null||Array.isArray(e)&&(t=SA(e),!(t>=14&&t<=16))),e}function cut(e,t,n){var s=function(){return e.apply(s,arguments)};return t.apply(s,n),s}function uut(e,t,n){var s,a;s=t;do a=Pe(e.p[s.p])+n,e.p[s.p]=a,s=e.a[s.p];while(s!=t)}function Sk(e,t){var n,s;s=e.a,n=rkn(e,t,null),s!=t&&!e.e&&(n=wx(e,t,n)),n&&n.Fi()}function ige(e,t){return vf(),xh(Ig),d.Math.abs(e-t)<=Ig||e==t||isNaN(e)&&isNaN(t)}function sge(e,t){return vf(),xh(Ig),d.Math.abs(e-t)<=Ig||e==t||isNaN(e)&&isNaN(t)}function C4n(e,t){return Mg(),dc(e.b.c.length-e.e.c.length,t.b.c.length-t.e.c.length)}function wy(e,t){return _1n(KE(e,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15)))))}function lut(){lut=re,C5t=ur((bn(),se(ie(Ote,1),ze,267,0,[Wi,ds,Pi,Vo,Ou,Bf])))}function hut(){hut=re,OEt=ur((Aw(),se(ie(Nre,1),ze,291,0,[Ore,QD,XD,Ire,WD,KD])))}function fut(){fut=re,SEt=ur(($1(),se(ie(z8e,1),ze,248,0,[Dre,qD,YD,MH,SH,AH])))}function dut(){dut=re,J5t=ur((u5(),se(ie(e8,1),ze,227,0,[Jx,m_,Zx,Jw,o3,a3])))}function gut(){gut=re,l6t=ur((ax(),se(ie(l6e,1),ze,275,0,[x_,a6e,u6e,c6e,o6e,s6e])))}function put(){put=re,u6t=ur((cM(),se(ie(i6e,1),ze,274,0,[D$,t6e,r6e,e6e,n6e,sne])))}function but(){but=re,Lxt=ur((UF(),se(ie(Uke,1),ze,313,0,[Rne,Gke,Fne,zke,Vke,tH])))}function vut(){vut=re,o6t=ur((KF(),se(ie(Q5e,1),ze,276,0,[tne,ene,rne,nne,ine,A$])))}function wut(){wut=re,z8t=ur((w9(),se(ie(H8t,1),ze,327,0,[hH,ere,nre,tre,rre,Jne])))}function mut(){mut=re,s9t=ur((Qc(),se(ie(FH,1),ze,273,0,[Zg,Ud,aL,sT,iT,K5])))}function yut(){yut=re,QEt=ur((YF(),se(ie(R7e,1),ze,312,0,[Wre,P7e,F7e,O7e,B7e,N7e])))}function S4n(){return Iw(),se(ie(ga,1),ze,93,0,[Lh,Vd,Ih,Nh,J1,Xl,el,Oh,Kl])}function $B(e,t){var n;n=e.a,e.a=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,0,n,e.a))}function HB(e,t){var n;n=e.b,e.b=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,1,n,e.b))}function Ak(e,t){var n;n=e.b,e.b=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,3,n,e.b))}function xb(e,t){var n;n=e.f,e.f=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,3,n,e.f))}function Eb(e,t){var n;n=e.g,e.g=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,4,n,e.g))}function yc(e,t){var n;n=e.i,e.i=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,5,n,e.i))}function kc(e,t){var n;n=e.j,e.j=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,6,n,e.j))}function Mk(e,t){var n;n=e.j,e.j=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,1,n,e.j))}function Dk(e,t){var n;n=e.c,e.c=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,4,n,e.c))}function Lk(e,t){var n;n=e.k,e.k=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new gw(e,2,n,e.k))}function IK(e,t){var n;n=e.d,e.d=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new sK(e,2,n,e.d))}function p0(e,t){var n;n=e.s,e.s=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new sK(e,4,n,e.s))}function yw(e,t){var n;n=e.t,e.t=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new sK(e,5,n,e.t))}function Ik(e,t){var n;n=e.F,e.F=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,5,n,t))}function LA(e,t){var n;return n=h(_n((SN(),KH),e),55),n?n.xj(t):pe(yn,st,1,t,5,1)}function Md(e,t){var n,s;return n=t in e.a,n&&(s=Ed(e,t).he(),s)?s.a:null}function A4n(e,t){var n,s,a;return n=(s=(X2(),a=new Zue,a),t&&Ybe(s,t),s),gge(n,e),n}function kut(e,t,n){if(Gk(e,n),!e.Bk()&&n!=null&&!e.wj(n))throw ne(new Eq);return n}function xut(e,t){return e.n=t,e.n?(e.f=new qe,e.e=new qe):(e.f=null,e.e=null),e}function tr(e,t,n,s,a,u){var p;return p=_W(e,t),_ut(n,p),p.i=a?8:0,p.f=s,p.e=a,p.g=u,p}function age(e,t,n,s,a){this.d=t,this.k=s,this.f=a,this.o=-1,this.p=1,this.c=e,this.a=n}function oge(e,t,n,s,a){this.d=t,this.k=s,this.f=a,this.o=-1,this.p=2,this.c=e,this.a=n}function cge(e,t,n,s,a){this.d=t,this.k=s,this.f=a,this.o=-1,this.p=6,this.c=e,this.a=n}function uge(e,t,n,s,a){this.d=t,this.k=s,this.f=a,this.o=-1,this.p=7,this.c=e,this.a=n}function lge(e,t,n,s,a){this.d=t,this.j=s,this.e=a,this.o=-1,this.p=4,this.c=e,this.a=n}function Eut(e,t){var n,s,a,u;for(s=t,a=0,u=s.length;a<u;++a)n=s[a],oct(e.a,n);return e}function Cf(e){var t,n,s,a;for(n=e,s=0,a=n.length;s<a;++s)t=n[s],Vn(t);return new HJe(e)}function M4n(e){var t=/function(?:\s+([\w$]+))?\s*\(/,n=t.exec(e);return n&&n[1]||bZ}function _ut(e,t){if(!!e){t.n=e;var n=Nwn(t);if(!n){Oj[e]=[t];return}n.gm=t}}function D4n(e,t,n){var s,a;return a=e.length,s=d.Math.min(n,a),gbe(e,0,t,0,s,!0),t}function Tut(e,t,n){var s,a;for(a=t.Kc();a.Ob();)s=h(a.Pb(),79),Qi(e,h(n.Kb(s),33))}function L4n(){SQe();for(var e=Uee,t=0;t<arguments.length;t++)e.push(arguments[t])}function YE(e,t){var n,s,a,u;for(s=t,a=0,u=s.length;a<u;++a)n=s[a],Ai(e,n,e.c.b,e.c)}function zB(e,t){e.b=d.Math.max(e.b,t.d),e.e+=t.r+(e.a.c.length==0?0:e.c),Ue(e.a,t)}function Cut(e){tw(e.c>=0),Axn(e.d,e.c)<0&&(e.a=e.a-1&e.d.a.length-1,e.b=e.d.c),e.c=-1}function hge(e){return e.a<54?e.f<0?-1:e.f>0?1:0:(!e.c&&(e.c=UA(e.f)),e.c).e}function xh(e){if(!(e>=0))throw ne(new an("tolerance ("+e+") must be >= 0"));return e}function Ok(){return Are||(Are=new fpt,Ey(Are,se(ie(n3,1),st,130,0,[new nle]))),Are}function ua(){ua=re,B_=new cY(j9,0),Jc=new cY("INPUT",1),ec=new cY("OUTPUT",2)}function GB(){GB=re,U5e=new nY("ARD",0),C$=new nY("MSD",1),Xte=new nY("MANUAL",2)}function _b(){_b=re,jD=new dY("BARYCENTER",0),R_=new dY(Kvt,1),aH=new dY(Xvt,2)}function IA(e,t){var n;if(n=e.gc(),t<0||t>n)throw ne(new sw(t,n));return new M1e(e,t)}function Sut(e,t){var n;return ue(t,42)?e.c.Mc(t):(n=TX(e,t),_F(e,t),n)}function ma(e,t,n){return Vp(e,t),Ko(e,n),p0(e,0),yw(e,1),m0(e,!0),w0(e,!0),e}function Uu(e,t){if(e<0)throw ne(new an(t+" cannot be negative but was: "+e));return e}function Aut(e,t){var n,s;for(n=0,s=e.gc();n<s;++n)if(Bo(t,e.Xb(n)))return n;return-1}function VB(e){var t,n;for(n=e.c.Cc().Kc();n.Ob();)t=h(n.Pb(),14),t.$b();e.c.$b(),e.d=0}function I4n(e){var t,n,s,a;for(n=e.a,s=0,a=n.length;s<a;++s)t=n[s],Drt(t,t.length,null)}function fge(e){var t,n;if(e==0)return 32;for(n=0,t=1;(t&e)==0;t<<=1)++n;return n}function O4n(e){var t,n;for(n=new Q(g1t(e));n.a<n.c.c.length;)t=h(K(n),680),t.Gf()}function Mut(e){Ihe(),this.g=new zn,this.f=new zn,this.b=new zn,this.c=new pb,this.i=e}function dge(){this.f=new Rs,this.d=new the,this.c=new Rs,this.a=new qe,this.b=new qe}function Dut(e,t,n,s){this.rj(),this.a=t,this.b=e,this.c=null,this.c=new ztt(this,t,n,s)}function OK(e,t,n,s,a){this.d=e,this.n=t,this.g=n,this.o=s,this.p=-1,a||(this.o=-2-s-1)}function Lut(){Ofe.call(this),this.n=-1,this.g=null,this.i=null,this.j=null,this.Bb|=Ch}function N4n(){return fu(),se(ie(K7e,1),ze,259,0,[x8,lL,RH,oT,jH,HH,$H,Kre,uL])}function P4n(){return oR(),se(ie(w4e,1),ze,250,0,[v4e,d4e,g4e,f4e,hte,b4e,p4e,h4e,l4e])}function Iut(){Iut=re,d3t=se(ie(Hn,1),hr,25,15,[0,8,4,12,2,10,6,14,1,9,5,13,3,11,7,15])}function Out(){Out=re,Kxt=Lr(Lr(Lr(new rs,(Zs(),w1),(oa(),v_)),Hg,Kx),oc,Wx)}function Nut(){Nut=re,Xxt=Lr(Lr(Lr(new rs,(Zs(),w1),(oa(),v_)),Hg,Kx),oc,Wx)}function Put(){Put=re,Qxt=Lr(Lr(Lr(new rs,(Zs(),w1),(oa(),v_)),Hg,Kx),oc,Wx)}function But(){But=re,e8t=Wc(Lr(Lr(new rs,(Zs(),oc),(oa(),h$)),Go,a$),Pa,l$)}function e5(){e5=re,I5=new tY("LAYER_SWEEP",0),_D=new tY(vJ,1),G5e=new tY(jd,2)}function B4n(e,t){var n,s;return n=e.c,s=t.e[e.p],s>0?h(ut(n.a,s-1),10):null}function WE(e,t){var n;n=e.k,e.k=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,2,n,e.k))}function UB(e,t){var n;n=e.f,e.f=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,8,n,e.f))}function qB(e,t){var n;n=e.i,e.i=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,7,n,e.i))}function gge(e,t){var n;n=e.a,e.a=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,8,n,e.a))}function pge(e,t){var n;n=e.b,e.b=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,0,n,e.b))}function bge(e,t){var n;n=e.b,e.b=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,0,n,e.b))}function vge(e,t){var n;n=e.c,e.c=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,1,n,e.c))}function wge(e,t){var n;n=e.c,e.c=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,1,n,e.c))}function NK(e,t){var n;n=e.c,e.c=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,4,n,e.c))}function mge(e,t){var n;n=e.d,e.d=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,1,n,e.d))}function PK(e,t){var n;n=e.D,e.D=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,2,n,e.D))}function BK(e,t){e.r>0&&e.c<e.r&&(e.c+=t,!!e.i&&e.i.d>0&&e.g!=0&&BK(e.i,t/e.r*e.i.d))}function F4n(e,t,n){var s;e.b=t,e.a=n,s=(e.a&512)==512?new pXe:new tle,e.c=JAn(s,e.b,e.a)}function Fut(e,t){return Bd(e.e,t)?(ia(),SK(t)?new _P(t,e):new RS(t,e)):new SJe(t,e)}function YB(e,t){return E1n(XE(e.a,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15)))))}function R4n(e,t,n){return pw(e,new Iqe(t),new Ut,new Oqe(n),se(ie(gu,1),ze,132,0,[]))}function j4n(e){var t,n;return 0>e?new Lhe:(t=e+1,n=new oot(t,e),new l1e(null,n))}function $4n(e,t){Rt();var n;return n=new S4(1),ws(e)?Ta(n,e,t):sc(n.f,e,t),new yq(n)}function H4n(e,t){var n,s;return n=e.o+e.p,s=t.o+t.p,n<s?-1:n==s?0:1}function z4n(e){var t;return t=U(e,(He(),Br)),ue(t,160)?Xht(h(t,160)):null}function Rut(e){var t;return e=d.Math.max(e,2),t=Gge(e),e>t?(t<<=1,t>0?t:N9):t}function FK(e){switch(Hfe(e.e!=3),e.e){case 2:return!1;case 0:return!0}return Yyn(e)}function jut(e,t){var n;return ue(t,8)?(n=h(t,8),e.a==n.a&&e.b==n.b):!1}function RK(e,t,n){var s,a,u;return u=t>>5,a=t&31,s=Zi(ow(e.n[n][u],Gn(kd(a,1))),3),s}function G4n(e,t){var n,s;for(s=t.vc().Kc();s.Ob();)n=h(s.Pb(),42),zF(e,n.cd(),n.dd())}function V4n(e,t){var n;n=new Bi,h(t.b,65),h(t.b,65),h(t.b,65),mc(t.a,new $1e(e,n,t))}function yge(e,t){var n;n=e.b,e.b=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,21,n,e.b))}function kge(e,t){var n;n=e.d,e.d=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,11,n,e.d))}function WB(e,t){var n;n=e.j,e.j=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,13,n,e.j))}function $ut(e,t,n){var s,a,u;for(u=e.a.length-1,a=e.b,s=0;s<n;a=a+1&u,++s)pi(t,s,e.a[a])}function Eh(e,t){var n;return rn(t),n=t.g,e.b[n]?!1:(pi(e.b,n,t),++e.c,!0)}function Hut(e,t){var n;return n=t==null?-1:Ha(e.b,t,0),n<0?!1:(jK(e,n),!0)}function jK(e,t){var n;n=f0(e.b,e.b.c.length-1),t<e.b.c.length&&(dl(e.b,t,n),t0t(e,t))}function U4n(e,t){(wB(),Pf?null:t.c).length==0&&vtt(t,new me),Ta(e.a,Pf?null:t.c,t)}function q4n(e,t){Bn(t,"Hierarchical port constraint processing",1),e8n(e),tPn(e),An(t)}function Y4n(e,t){var n,s;for(s=t.Kc();s.Ob();)n=h(s.Pb(),266),e.b=!0,Qi(e.e,n),n.b=e}function KB(e,t){var n,s;return n=1-t,s=e.a[n],e.a[n]=s.a[t],s.a[t]=e,e.b=!0,s.b=!1,s}function W4n(e,t){var n,s;return n=h(U(e,(tt(),um)),8),s=h(U(t,um),8),Ui(n.b,s.b)}function zut(e){fde.call(this),this.b=Pe(Je(U(e,(tt(),Mh)))),this.a=h(U(e,Hd),218)}function Gut(e,t,n){p0e.call(this,e,t,n),this.a=new zn,this.b=new zn,this.d=new nWe(this)}function Vut(e){this.e=e,this.d=new gN(vw($4(this.e).gc())),this.c=this.e.a,this.b=this.e.c}function XB(e){this.b=e,this.a=pe(Hn,hr,25,e+1,15,1),this.c=pe(Hn,hr,25,e,15,1),this.d=0}function $K(e,t,n){var s;return s=new qe,Xbe(e,t,s,n,!0,!0),e.b=new XB(s.c.length),s}function Uut(e,t){var n;return n=h(_n(e.c,t),458),n||(n=new WKe,n.c=t,Pr(e.c,n.c,n)),n}function HK(e,t){var n=e.a,s=0;for(var a in n)n.hasOwnProperty(a)&&(t[s++]=a);return t}function xge(e){var t;return e.b==null?(o0(),o0(),vL):(t=e.Lk()?e.Kk():e.Jk(),t)}function qut(e){var t,n;for(n=new Tn(e);n.e!=n.i.gc();)t=h(On(n),33),yc(t,0),kc(t,0)}function kg(){kg=re,D5=new Xr(Mwe),Jj=new Xr(Dwe),b_=new Xr(Lvt),mD=new Xr(Ivt)}function Nk(){Nk=re,kD=new Yhe("TO_INTERNAL_LTR",0),Nte=new Yhe("TO_INPUT_DIRECTION",1)}function QB(){QB=re,vH=new cfe("P1_NODE_PLACEMENT",0),p8=new cfe("P2_EDGE_ROUTING",1)}function Tb(){Tb=re,ev=new eY("START",0),zg=new eY("MIDDLE",1),Jb=new eY("END",2)}function t5(){t5=re,Fte=new Xi("edgelabelcenterednessanalysis.includelabel",(on(),u2))}function K4n(e,t){Ti(Jr(new Vt(null,new Wt(new P6(e.b),1)),new XZe(e,t)),new ZZe(e,t))}function Yut(){this.c=new bN(0),this.b=new bN(wwt),this.d=new bN(vwt),this.a=new bN(cJ)}function Ege(e){var t,n;for(n=e.c.a.ec().Kc();n.Ob();)t=h(n.Pb(),214),Ghn(t,new pdt(t.e))}function Wut(e){var t,n;for(n=e.c.a.ec().Kc();n.Ob();)t=h(n.Pb(),214),zhn(t,new krt(t.f))}function Ko(e,t){var n;n=e.zb,e.zb=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,1,n,e.zb))}function ZB(e,t){var n;n=e.xb,e.xb=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,3,n,e.xb))}function JB(e,t){var n;n=e.yb,e.yb=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,2,n,e.yb))}function vi(e,t){var n,s;n=(s=new Cq,s),n.n=t,qn((!e.s&&(e.s=new Ye(Ac,e,21,17)),e.s),n)}function Qr(e,t){var n,s;s=(n=new o1e,n),s.n=t,qn((!e.s&&(e.s=new Ye(Ac,e,21,17)),e.s),s)}function my(e,t){var n,s;for(n=e.Pc(),_k(n,0,n.length,t),s=0;s<n.length;s++)e._c(s,n[s])}function Qs(e,t){var n,s,a;for(rn(t),n=!1,a=t.Kc();a.Ob();)s=a.Pb(),n=n|e.Fc(s);return n}function Kut(e){var t,n,s;for(t=0,s=e.Kc();s.Ob();)n=s.Pb(),t+=n!=null?ii(n):0,t=~~t;return t}function Xut(e){var t;return e==0?"UTC":(e<0?(e=-e,t="UTC+"):t="UTC-",t+Ict(e))}function zK(e,t){var n;return ue(t,14)?(n=h(t,14),e.Gc(n)):AK(e,h(Vn(t),20).Kc())}function Qut(e,t,n){eut.call(this,t,n),this.d=pe(X1,T0,10,e.a.c.length,0,1),Mf(e.a,this.d)}function X4n(e){e.a=null,e.e=null,e.b.c=pe(yn,st,1,0,5,1),e.f.c=pe(yn,st,1,0,5,1),e.c=null}function Zut(e,t){t?e.B==null&&(e.B=e.D,e.D=null):e.B!=null&&(e.D=e.B,e.B=null)}function Jut(e,t){return Pe(Je(sb(WA(pc(new Vt(null,new Wt(e.c.b,16)),new GYe(e)),t))))}function _ge(e,t){return Pe(Je(sb(WA(pc(new Vt(null,new Wt(e.c.b,16)),new zYe(e)),t))))}function Q4n(e,t){Bn(t,Wvt,1),Ti(Xa(new Vt(null,new Wt(e.b,16)),new GBe),new VBe),An(t)}function Z4n(e,t){var n,s;return n=h(pt(e,(Cw(),mH)),19),s=h(pt(t,mH),19),dc(n.a,s.a)}function Tge(e,t,n){var s,a;for(a=pr(e,0);a.b!=a.d.c;)s=h(gr(a),8),s.a+=t,s.b+=n;return e}function KE(e,t,n){var s;for(s=e.b[n&e.f];s;s=s.b)if(n==s.a&&n1(t,s.g))return s;return null}function XE(e,t,n){var s;for(s=e.c[n&e.f];s;s=s.d)if(n==s.f&&n1(t,s.i))return s;return null}function J4n(e,t,n){var s,a,u;for(s=0,a=0;a<n;a++)u=t[a],e[a]=u<<1|s,s=u>>>31;s!=0&&(e[n]=s)}function e5n(e,t){Rt();var n,s;for(s=new qe,n=0;n<e;++n)s.c[s.c.length]=t;return new Hq(s)}function elt(e){var t;return t=sat(e),eb(t.a,0)?(X6(),X6(),G3e):(X6(),new Jfe(t.b))}function tlt(e){var t;return t=sat(e),eb(t.a,0)?(X6(),X6(),G3e):(X6(),new Jfe(t.c))}function t5n(e){var t;return t=fB(e),eb(t.a,0)?(yN(),yN(),E3t):(yN(),new Ket(t.b))}function n5n(e){return e.b.c.i.k==(bn(),Pi)?h(U(e.b.c.i,(He(),Br)),11):e.b.c}function nlt(e){return e.b.d.i.k==(bn(),Pi)?h(U(e.b.d.i,(He(),Br)),11):e.b.d}function Ci(e,t,n,s,a,u,p,v,m,x,_,A,I){return Wdt(e,t,n,s,a,u,p,v,m,x,_,A,I),bX(e,!1),e}function bl(e,t,n,s,a,u,p){ar.call(this,e,t),this.d=n,this.e=s,this.c=a,this.b=u,this.a=Ef(p)}function r5n(e,t){typeof window===LM&&typeof window.$gwt===LM&&(window.$gwt[e]=t)}function i5n(e,t){return $k(),e==Zb&&t==Zw||e==Zw&&t==Zb||e==r3&&t==Qw||e==Qw&&t==r3}function s5n(e,t){return $k(),e==Zb&&t==Qw||e==Zb&&t==r3||e==Zw&&t==r3||e==Zw&&t==Qw}function rlt(e,t){return vf(),xh(If),d.Math.abs(0-t)<=If||t==0||isNaN(0)&&isNaN(t)?0:e/t}function a5n(){return ha(),se(ie(ane,1),ze,256,0,[L$,El,E_,I$,i8,u3,__,n8,r8,O$])}function GK(){GK=re,fT=new lXe,iie=se(ie(Ac,1),Jy,170,0,[]),X9t=se(ie(eh,1),Jye,59,0,[])}function eF(){eF=re,hxe=new hY("NO",0),Xne=new hY("GREEDY",1),lxe=new hY("LOOK_BACK",2)}function Rc(){Rc=re,e5e=new IBe,Z4e=new LBe,J4e=new OBe,Q4e=new NBe,t5e=new PBe,n5e=new BBe}function o5n(e){var t,n,s;for(s=0,n=new Q(e.b);n.a<n.c.c.length;)t=h(K(n),29),t.p=s,++s}function c5n(e,t){var n;return n=V2e(e),mbe(new dt(n.c,n.d),new dt(n.b,n.a),e.rf(),t,e.Hf())}function jo(e,t){var n;return e.b?null:(n=s4n(e,e.g),vr(e.a,n),n.i=e,e.d=t,n)}function u5n(e,t,n){Bn(n,"DFS Treeifying phase",1),wxn(e,t),gAn(e,t),e.a=null,e.b=null,An(n)}function ilt(e,t,n){this.g=e,this.d=t,this.e=n,this.a=new qe,oTn(this),Rt(),hs(this.a,null)}function Cge(e){this.i=e.gc(),this.i>0&&(this.g=this.ri(this.i+(this.i/8|0)+1),e.Qc(this.g))}function yi(e,t){yP.call(this,Q9t,e,t),this.b=this,this.a=ac(e.Tg(),Ht(this.e.Tg(),this.c))}function QE(e,t){var n,s;for(rn(t),s=t.vc().Kc();s.Ob();)n=h(s.Pb(),42),e.zc(n.cd(),n.dd())}function l5n(e,t,n){var s;for(s=n.Kc();s.Ob();)if(!pB(e,t,s.Pb()))return!1;return!0}function h5n(e,t,n,s,a){var u;return n&&(u=oi(t.Tg(),e.c),a=n.gh(t,-1-(u==-1?s:u),null,a)),a}function f5n(e,t,n,s,a){var u;return n&&(u=oi(t.Tg(),e.c),a=n.ih(t,-1-(u==-1?s:u),null,a)),a}function slt(e){var t;if(e.b==-2){if(e.e==0)t=-1;else for(t=0;e.a[t]==0;t++);e.b=t}return e.b}function alt(e){switch(e.g){case 2:return Qe(),un;case 4:return Qe(),pn;default:return e}}function olt(e){switch(e.g){case 1:return Qe(),Fn;case 3:return Qe(),sn;default:return e}}function d5n(e){var t,n,s;return e.j==(Qe(),sn)&&(t=B0t(e),n=Pc(t,pn),s=Pc(t,un),s||s&&n)}function g5n(e){var t,n;return t=h(e.e&&e.e(),9),n=h(Ide(t,t.length),9),new ll(t,n,t.length)}function p5n(e,t){Bn(t,Wvt,1),Spe(P1n(new XO((Y7(),new IW(e,!1,!1,new $ue))))),An(t)}function OA(e,t){return on(),ws(e)?N0e(e,Jn(t)):ew(e)?dW(e,Je(t)):Jv(e)?mbn(e,lt(t)):e.wd(t)}function Sge(e,t){t.q=e,e.d=d.Math.max(e.d,t.r),e.b+=t.d+(e.a.c.length==0?0:e.c),Ue(e.a,t)}function Pk(e,t){var n,s,a,u;return a=e.c,n=e.c+e.b,u=e.d,s=e.d+e.a,t.a>a&&t.a<n&&t.b>u&&t.b<s}function clt(e,t,n,s){ue(e.Cb,179)&&(h(e.Cb,179).tb=null),Ko(e,n),t&&MTn(e,t),s&&e.xk(!0)}function Age(e,t){var n;n=h(t,183),Lp(n,"x",e.i),Lp(n,"y",e.j),Lp(n,mee,e.g),Lp(n,wee,e.f)}function Mge(){Mge=re,t8t=Ib(PJe(Lr(Lr(new rs,(Zs(),oc),(oa(),h$)),Go,a$),Pa),l$)}function ult(){ult=re,s8t=Ib(PJe(Lr(Lr(new rs,(Zs(),oc),(oa(),h$)),Go,a$),Pa),l$)}function tF(){tF=re,vre=new bY(jd,0),Xxe=new bY("POLAR_COORDINATE",1),Kxe=new bY("ID",2)}function nF(){nF=re,ixe=new uY("EQUALLY",0),Yne=new uY(LR,1),sxe=new uY("NORTH_SOUTH",2)}function llt(){llt=re,Pxt=ur((By(),se(ie(exe,1),ze,260,0,[iH,BD,FD,Qke,Zke,Xke,Jke,sH])))}function hlt(){hlt=re,Z5t=ur((sR(),se(ie($5e,1),ze,270,0,[zte,Ute,Hte,Wte,Vte,Gte,Yte,qte])))}function flt(){flt=re,CEt=ur((E0(),se(ie(TEt,1),ze,277,0,[j8e,zs,po,b8,Ba,_i,V5,jf])))}function dlt(){dlt=re,M9t=ur((ux(),se(ie(WH,1),ze,237,0,[eie,qH,YH,UH,Jre,VH,GH,Zre])))}function Bk(){Bk=re,p_=new Xi("debugSVG",(on(),!1)),x4e=new Xi("overlapsExisted",!0)}function glt(e,t){return pw(new Mqe(e),new Dqe(t),new Lqe(t),new ot,se(ie(gu,1),ze,132,0,[]))}function b5n(){var e;return ote||(ote=new RKe,e=new uK(""),Adn(e,(Q6(),K3e)),U4n(ote,e)),ote}function v5n(e,t){var n;for(Vn(t);e.Ob();)if(n=e.Pb(),!Ige(h(n,10)))return!1;return!0}function plt(e,t){var n;return n=FQ(Ok(),e),n?(ya(t,(Tr(),v8),n),!0):!1}function qu(e,t){var n;for(n=0;n<t.j.c.length;n++)h(_A(e,n),21).Gc(h(_A(t,n),14));return e}function w5n(e,t){var n,s;for(s=new Q(t.b);s.a<s.c.c.length;)n=h(K(s),29),e.a[n.p]=z9n(n)}function ZE(e,t){var n,s;for(rn(t),s=e.vc().Kc();s.Ob();)n=h(s.Pb(),42),t.Od(n.cd(),n.dd())}function rF(e,t){var n;ue(t,83)?(h(e.c,76).Xj(),n=h(t,83),G4n(e,n)):h(e.c,76).Wb(t)}function Rp(e){return ue(e,152)?X4(h(e,152)):ue(e,131)?h(e,131).a:ue(e,54)?new U2(e):new Xv(e)}function m5n(e,t){return t<e.b.gc()?h(e.b.Xb(t),10):t==e.b.gc()?e.a:h(ut(e.e,t-e.b.gc()-1),10)}function blt(e,t){e.a=Vs(e.a,1),e.c=d.Math.min(e.c,t),e.b=d.Math.max(e.b,t),e.d=Vs(e.d,t)}function y5n(e,t){var n;Bn(t,"Edge and layer constraint edge reversal",1),n=EMn(e),jOn(n),An(t)}function vlt(e){var t;e.d==null?(++e.e,e.f=0,Cht(null)):(++e.e,t=e.d,e.d=null,e.f=0,Cht(t))}function xg(e){var t;return t=e.h,t==0?e.l+e.m*v5:t==Rd?e.l+e.m*v5-Jp:e}function wlt(e){return rw(),e.A.Hc((Iu(),bm))&&!e.B.Hc((fu(),lL))?eft(e):null}function k5n(e){if(rn(e),e.length==0)throw ne(new Qf("Zero length BigInteger"));NMn(this,e)}function yy(e){if(!e)throw ne(new ja("no calls to next() since the last call to remove()"))}function xc(e){return RM<e&&e<Jp?e<0?d.Math.ceil(e):d.Math.floor(e):xg(HSn(e))}function x5n(e,t){var n,s,a;for(n=e.c.Ee(),a=t.Kc();a.Ob();)s=a.Pb(),e.a.Od(n,s);return e.b.Kb(n)}function Or(e,t){var n,s,a;if(n=e.Jg(),n!=null&&e.Mg())for(s=0,a=n.length;s<a;++s)n[s].ui(t)}function Fk(e,t){var n,s;for(n=e,s=Us(n).e;s;){if(n=s,n==t)return!0;s=Us(n).e}return!1}function E5n(e,t,n){var s,a;return s=e.a.f[t.p],a=e.a.f[n.p],s<a?-1:s==a?0:1}function jp(e,t,n){var s,a;return a=h(_E(e.d,t),19),s=h(_E(e.b,n),19),!a||!s?null:Z4(e,a.a,s.a)}function _5n(e,t){var n,s;for(s=new Tn(e);s.e!=s.i.gc();)n=h(On(s),33),bf(n,n.i+t.b,n.j+t.d)}function T5n(e,t){var n,s;for(s=new Q(t);s.a<s.c.c.length;)n=h(K(s),70),Ue(e.d,n),P9n(e,n)}function C5n(e,t){var n,s;s=new qe,n=t;do s.c[s.c.length]=n,n=h(_n(e.k,n),17);while(n);return s}function tn(e,t){var n;return(e.Db&t)!=0?(n=WX(e,t),n==-1?e.Eb:Fp(e.Eb)[n]):null}function so(e,t){var n,s;return n=(s=new ile,s),n.G=t,!e.rb&&(e.rb=new cw(e,af,e)),qn(e.rb,n),n}function kr(e,t){var n,s;return n=(s=new rN,s),n.G=t,!e.rb&&(e.rb=new cw(e,af,e)),qn(e.rb,n),n}function Dge(e,t){switch(t){case 1:return!!e.n&&e.n.i!=0;case 2:return e.k!=null}return u0e(e,t)}function mlt(e){switch(e.a.g){case 1:return new vJe;case 3:return new v1t;default:return new BUe}}function iF(e){var t;if(e.g>1||e.Ob())return++e.a,e.g=0,t=e.i,e.Ob(),t;throw ne(new lo)}function S5n(e){aet();var t;return bZe(Zne,e)||(t=new G$e,t.a=e,v1e(Zne,e,t)),h(wa(Zne,e),635)}function Bl(e){var t,n,s,a;return a=e,s=0,a<0&&(a+=Jp,s=Rd),n=Li(a/v5),t=Li(a-n*v5),rc(t,n,s)}function NA(e){var t,n,s;for(s=0,n=new A4(e.a);n.a<n.c.a.length;)t=zE(n),e.b.Hc(t)&&++s;return s}function A5n(e){var t,n,s;for(t=1,s=e.Kc();s.Ob();)n=s.Pb(),t=31*t+(n==null?0:ii(n)),t=~~t;return t}function M5n(e,t){var n;this.c=e,n=new qe,a2e(e,n,t,e.b,null,!1,null,!1),this.a=new Ms(n,0)}function Rk(e,t){this.b=e,this.e=t,this.d=t.j,this.f=(ia(),h(e,66).Oj()),this.k=ac(t.e.Tg(),e)}function $p(e,t,n){this.b=(rn(e),e),this.d=(rn(t),t),this.e=(rn(n),n),this.c=this.d+(""+this.e)}function ylt(){this.a=h(at((Jh(),Qj)),19).a,this.c=Pe(Je(at(Zj))),this.b=Pe(Je(at(xte)))}function klt(){klt=re,n9t=ur((Iw(),se(ie(ga,1),ze,93,0,[Lh,Vd,Ih,Nh,J1,Xl,el,Oh,Kl])))}function xlt(){xlt=re,j3t=ur((oR(),se(ie(w4e,1),ze,250,0,[v4e,d4e,g4e,f4e,hte,b4e,p4e,h4e,l4e])))}function Cb(){Cb=re,fD=new NN("UP",0),hD=new NN(ZZ,1),gte=new NN(w5,2),pte=new NN(m5,3)}function Lge(){Lge=re,xxe=(aB(),cre),q8t=new $t(nye,xxe),kxe=(lB(),ure),U8t=new $t(rye,kxe)}function jk(){jk=re,one=new iY("ONE_SIDED",0),cne=new iY("TWO_SIDED",1),TD=new iY("OFF",2)}function Elt(e){e.r=new ns,e.w=new ns,e.t=new qe,e.i=new qe,e.d=new ns,e.a=new O4,e.c=new zn}function PA(e){this.n=new qe,this.e=new di,this.j=new di,this.k=new qe,this.f=new qe,this.p=e}function _lt(e,t){e.c&&(Spt(e,t,!0),Ti(new Vt(null,new Wt(t,16)),new QYe(e))),Spt(e,t,!1)}function D5n(e,t,n){return e==(_b(),aH)?new l$e:hu(t,1)!=0?new Ehe(n.length):new qXe(n.length)}function Oa(e,t){var n;return t&&(n=t.Ve(),n.dc()||(e.q?QE(e.q,n):e.q=new NJe(n))),e}function Tlt(e,t){var n;return n=e.a.get(t),n===void 0?++e.d:(egn(e.a,t),--e.c,JP(e.b)),n}function L5n(e,t){var n,s,a;return n=t.p-e.p,n==0?(s=e.f.a*e.f.b,a=t.f.a*t.f.b,Ui(s,a)):n}function I5n(e,t){var n,s;return n=e.f.c.length,s=t.f.c.length,n<s?-1:n==s?0:1}function O5n(e){return e.b.c.length!=0&&!!h(ut(e.b,0),70).a?h(ut(e.b,0),70).a:PW(e)}function N5n(e){var t;if(e){if(t=e,t.dc())throw ne(new lo);return t.Xb(t.gc()-1)}return Cst(e.Kc())}function Clt(e){var t;return mo(e,0)<0&&(e=X1e(e)),t=Gn(dg(e,32)),64-(t!=0?bM(t):bM(Gn(e))+32)}function Ige(e){var t;return t=h(U(e,(He(),co)),61),e.k==(bn(),Pi)&&(t==(Qe(),un)||t==pn)}function P5n(e,t,n){var s,a;a=h(U(e,(tt(),Da)),74),a&&(s=new Oc,aX(s,0,a),mw(s,n),Qs(t,s))}function sF(e,t,n){var s,a,u,p;p=Us(e),s=p.d,a=p.c,u=e.n,t&&(u.a=u.a-s.b-a.a),n&&(u.b=u.b-s.d-a.b)}function B5n(e,t){var n,s;return n=e.j,s=t.j,n!=s?n.g-s.g:e.p==t.p?0:n==(Qe(),sn)?e.p-t.p:t.p-e.p}function F5n(e){var t,n;for(cNn(e),n=new Q(e.d);n.a<n.c.c.length;)t=h(K(n),101),t.i&&v_n(t)}function JE(e,t,n,s,a){pi(e.c[t.g],n.g,s),pi(e.c[n.g],t.g,s),pi(e.b[t.g],n.g,a),pi(e.b[n.g],t.g,a)}function R5n(e,t,n,s){h(n.b,65),h(n.b,65),h(s.b,65),h(s.b,65),h(s.b,65),mc(s.a,new j1e(e,t,s))}function j5n(e,t){e.d==(la(),Wl)||e.d==Gd?h(t.a,57).c.Fc(h(t.b,57)):h(t.b,57).c.Fc(h(t.a,57))}function VK(e,t,n,s){return n==1?(!e.n&&(e.n=new Ye(Ua,e,1,7)),qs(e.n,t,s)):Z2e(e,t,n,s)}function BA(e,t){var n,s;return s=(n=new fq,n),Ko(s,t),qn((!e.A&&(e.A=new Nc(lc,e,7)),e.A),s),s}function $5n(e,t,n){var s,a,u,p;return u=null,p=t,a=wb(p,xee),s=new nJe(e,n),u=(sEn(s.a,s.b,a),a),u}function UK(e){var t;return(!e.a||(e.Bb&1)==0&&e.a.kh())&&(t=Rl(e),ue(t,148)&&(e.a=h(t,148))),e.a}function FA(e,t){var n,s;for(rn(t),s=t.Kc();s.Ob();)if(n=s.Pb(),!e.Hc(n))return!1;return!0}function H5n(e,t){var n,s,a;return n=e.l+t.l,s=e.m+t.m+(n>>22),a=e.h+t.h+(s>>22),rc(n&du,s&du,a&Rd)}function Slt(e,t){var n,s,a;return n=e.l-t.l,s=e.m-t.m+(n>>22),a=e.h-t.h+(s>>22),rc(n&du,s&du,a&Rd)}function RA(e){var t;return e<128?(t=(znt(),_3e)[e],!t&&(t=_3e[e]=new vle(e)),t):new vle(e)}function ui(e){var t;return ue(e,78)?e:(t=e&&e.__java$exception,t||(t=new bht(e),LKe(t)),t)}function jA(e){if(ue(e,186))return h(e,118);if(e)return null;throw ne(new T4(Emt))}function Alt(e,t){if(t==null)return!1;for(;e.a!=e.b;)if(Nr(t,fF(e)))return!0;return!1}function Oge(e){return e.a.Ob()?!0:e.a!=e.d?!1:(e.a=new d0e(e.e.f),e.a.Ob())}function Vi(e,t){var n,s;return n=t.Pc(),s=n.length,s==0?!1:(q1e(e.c,e.c.length,n),!0)}function z5n(e,t,n){var s,a;for(a=t.vc().Kc();a.Ob();)s=h(a.Pb(),42),e.yc(s.cd(),s.dd(),n);return e}function Mlt(e,t){var n,s;for(s=new Q(e.b);s.a<s.c.c.length;)n=h(K(s),70),Fe(n,(He(),F5),t)}function G5n(e,t,n){var s,a;for(a=new Q(e.b);a.a<a.c.c.length;)s=h(K(a),33),bf(s,s.i+t,s.j+n)}function Dlt(e,t){if(!e)throw ne(new an(E9("value already present: %s",se(ie(yn,1),st,1,5,[t]))))}function Llt(e,t){return!e||!t||e==t?!1:dft(e.d.c,t.d.c+t.d.b)&&dft(t.d.c,e.d.c+e.d.b)}function V5n(){return wB(),Pf?new uK(null):N0t(b5n(),"com.google.common.base.Strings")}function Ilt(e,t){var n;return n=I1(t.a.gc()),Ti(FB(new Vt(null,new Wt(t,1)),e.i),new KZe(e,n)),n}function Olt(e){var t,n;return n=(t=new fq,t),Ko(n,"T"),qn((!e.d&&(e.d=new Nc(lc,e,11)),e.d),n),n}function Nge(e){var t,n,s,a;for(t=1,n=0,a=e.gc();n<a;++n)s=e.ki(n),t=31*t+(s==null?0:ii(s));return t}function Nlt(e,t,n,s){var a;return dA(t,e.e.Hd().gc()),dA(n,e.c.Hd().gc()),a=e.a[t][n],pi(e.a[t],n,s),a}function se(e,t,n,s,a){return a.gm=e,a.hm=t,a.im=de,a.__elementTypeId$=n,a.__elementTypeCategory$=s,a}function U5n(e,t,n,s,a){return Ry(),d.Math.min(V2t(e,t,n,s,a),V2t(n,s,e,t,fP(new dt(a.a,a.b))))}function aF(){aF=re,Rte=new BN(jd,0),m$=new BN(Qvt,1),y$=new BN(Zvt,2),ED=new BN("BOTH",3)}function Sf(){Sf=re,L5=new FN(Cx,0),Xx=new FN(w5,1),Qx=new FN(m5,2),jte=new FN("TOP",3)}function $k(){$k=re,Zb=new PN("Q1",0),Zw=new PN("Q4",1),Qw=new PN("Q2",2),r3=new PN("Q3",3)}function oF(){oF=re,Qne=new fY("OFF",0),d8=new fY("SINGLE_EDGE",1),hm=new fY("MULTI_EDGE",2)}function $A(){$A=re,CH=new hfe("MINIMUM_SPANNING_TREE",0),B8e=new hfe("MAXIMUM_SPANNING_TREE",1)}function n5(){n5=re,new Xr("org.eclipse.elk.addLayoutConfig"),yEt=new jHe,mEt=new RHe}function Pge(e){var t,n,s;for(t=new di,s=pr(e.d,0);s.b!=s.d.c;)n=h(gr(s),188),vr(t,n.c);return t}function qK(e){var t,n,s,a;for(a=new qe,s=e.Kc();s.Ob();)n=h(s.Pb(),33),t=Nw(n),Vi(a,t);return a}function q5n(e){var t;jb(e,!0),t=_0,os(e,(tt(),l8))&&(t+=h(U(e,l8),19).a),Fe(e,l8,Ke(t))}function Plt(e,t,n){var s;Kc(e.a),mc(n.i,new AWe(e)),s=new tP(h(_n(e.a,t.b),65)),xft(e,s,t),n.f=s}function Y5n(e,t){var n,s;return n=e.c,s=t.e[e.p],s<n.a.c.length-1?h(ut(n.a,s+1),10):null}function W5n(e,t){var n,s;for(XP(t,"predicate"),s=0;e.Ob();s++)if(n=e.Pb(),t.Lb(n))return s;return-1}function r5(e,t){var n,s;if(s=0,e<64&&e<=t)for(t=t<64?t:63,n=e;n<=t;n++)s=xf(s,kd(1,n));return s}function Bge(e){Rt();var t,n,s;for(s=0,n=e.Kc();n.Ob();)t=n.Pb(),s=s+(t!=null?ii(t):0),s=s|0;return s}function Fge(e){var t,n;return n=(X2(),t=new GO,t),e&&qn((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a),n),n}function K5n(e){var t;return t=new X,t.a=e,t.b=n6n(e),t.c=pe(it,Re,2,2,6,1),t.c[0]=Xut(e),t.c[1]=Xut(e),t}function Rge(e,t){switch(t){case 0:!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),e.o.c.$b();return}fQ(e,t)}function HA(e,t,n){switch(n.g){case 2:e.b=t;break;case 1:e.c=t;break;case 4:e.d=t;break;case 3:e.a=t}}function Blt(e){switch(e.g){case 1:return Kg;case 2:return Z1;case 3:return rL;default:return iL}}function X5n(e){switch(h(U(e,(tt(),cc)),163).g){case 2:case 4:return!0;default:return!1}}function Flt(){Flt=re,h6t=ur((ha(),se(ie(ane,1),ze,256,0,[L$,El,E_,I$,i8,u3,__,n8,r8,O$])))}function Rlt(){Rlt=re,f9t=ur((fu(),se(ie(K7e,1),ze,259,0,[x8,lL,RH,oT,jH,HH,$H,Kre,uL])))}function jlt(){jlt=re,a7t=Lr(Ib(Ib(CN(Lr(new rs,(tx(),H_),(w9(),hH)),HD),tre),nre),z_,rre)}function Hp(){Hp=re,Jte=new rY(jd,0),t8=new rY("INCOMING_ONLY",1),c3=new rY("OUTGOING_ONLY",2)}function YK(){YK=re,tte={boolean:O1n,number:zfn,string:Gfn,object:Zdt,function:Zdt,undefined:xfn}}function $lt(e,t){mP(e>=0,"Negative initial capacity"),mP(t>=0,"Non-positive load factor"),Kc(this)}function WK(e,t,n){return e>=128?!1:e<64?fE(Zi(kd(1,e),n),0):fE(Zi(kd(1,e-64),t),0)}function Q5n(e,t){return!e||!t||e==t?!1:Mb(e.b.c,t.b.c+t.b.b)<0&&Mb(t.b.c,e.b.c+e.b.b)<0}function Hlt(e){var t,n,s;return n=e.n,s=e.o,t=e.d,new hl(n.a-t.b,n.b-t.d,s.a+(t.b+t.c),s.b+(t.d+t.a))}function Z5n(e){var t,n,s,a;for(n=e.a,s=0,a=n.length;s<a;++s)t=n[s],Ylt(e,t,(Qe(),Fn)),Ylt(e,t,sn)}function J5n(e){var t,n,s,a;for(t=(e.j==null&&(e.j=(bk(),a=ete.ce(e),hEn(a))),e.j),n=0,s=t.length;n<s;++n);}function Hk(e){var t,n,s;return t=~e.l+1&du,n=~e.m+(t==0?1:0)&du,s=~e.h+(t==0&&n==0?1:0)&Rd,rc(t,n,s)}function e6n(e,t){var n,s;return n=h(h(_n(e.g,t.a),46).a,65),s=h(h(_n(e.g,t.b),46).a,65),Jpt(n,s)}function jge(e,t,n){var s;if(s=e.gc(),t>s)throw ne(new sw(t,s));return e.hi()&&(n=ist(e,n)),e.Vh(t,n)}function zA(e,t,n){return n==null?(!e.q&&(e.q=new zn),Q4(e.q,t)):(!e.q&&(e.q=new zn),Pr(e.q,t,n)),e}function Fe(e,t,n){return n==null?(!e.q&&(e.q=new zn),Q4(e.q,t)):(!e.q&&(e.q=new zn),Pr(e.q,t,n)),e}function zlt(e){var t,n;return n=new mB,Oa(n,e),Fe(n,(kg(),D5),e),t=new zn,nLn(e,n,t),AOn(e,n,t),n}function t6n(e){Ry();var t,n,s;for(n=pe(cs,Re,8,2,0,1),s=0,t=0;t<2;t++)s+=.5,n[t]=B8n(s,e);return n}function Glt(e,t){var n,s,a,u;for(n=!1,s=e.a[t].length,u=0;u<s-1;u++)a=u+1,n=n|mxn(e,t,u,a);return n}function zk(e,t,n,s,a){var u,p;for(p=n;p<=a;p++)for(u=t;u<=s;u++)Ay(e,u,p)||uR(e,u,p,!0,!1)}function Vlt(e,t){this.b=e,ay.call(this,(h(he(De((bg(),Xt).o),10),18),t.i),t.g),this.a=(GK(),iie)}function $ge(e,t){this.c=e,this.d=t,this.b=this.d/this.c.c.Hd().gc()|0,this.a=this.d%this.c.c.Hd().gc()}function Hge(){this.o=null,this.k=null,this.j=null,this.d=null,this.b=null,this.n=null,this.a=null}function zge(e,t,n){this.q=new d.Date,this.q.setFullYear(e+Og,t,n),this.q.setHours(0,0,0,0),C9(this,0)}function Dd(){Dd=re,Ug=new oY(jd,0),txe=new oY("NODES_AND_EDGES",1),Vne=new oY("PREFER_EDGES",2)}function n6n(e){var t;return e==0?"Etc/GMT":(e<0?(e=-e,t="Etc/GMT-"):t="Etc/GMT+",t+Ict(e))}function Gge(e){var t;if(e<0)return $s;if(e==0)return 0;for(t=N9;(t&e)==0;t>>=1);return t}function Ult(e){var t,n;return n=bM(e.h),n==32?(t=bM(e.m),t==32?bM(e.l)+32:t+20-10):n-12}function e9(e){var t;return t=e.a[e.b],t==null?null:(pi(e.a,e.b,null),e.b=e.b+1&e.a.length-1,t)}function qlt(e){var t,n;return t=e.t-e.k[e.o.p]*e.d+e.j[e.o.p]>e.f,n=e.u+e.e[e.o.p]*e.d>e.f*e.s*e.d,t||n}function cF(e,t,n){var s,a;return s=new wK(t,n),a=new $e,e.b=Fgt(e,e.b,s,a),a.b||++e.c,e.b.b=!1,a.d}function Ylt(e,t,n){var s,a,u,p;for(p=r9(t,n),u=0,a=p.Kc();a.Ob();)s=h(a.Pb(),11),Pr(e.c,s,Ke(u++))}function Eg(e){var t,n;for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),81),t.g.c=-t.g.c-t.g.b;nR(e)}function _g(e){var t,n;for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),57),t.d.c=-t.d.c-t.d.b;qbe(e)}function Vge(e){var t;return(!e.c||(e.Bb&1)==0&&(e.c.Db&64)!=0)&&(t=Rl(e),ue(t,88)&&(e.c=h(t,26))),e.c}function KK(e){var t,n,s;t=~e.l+1&du,n=~e.m+(t==0?1:0)&du,s=~e.h+(t==0&&n==0?1:0)&Rd,e.l=t,e.m=n,e.h=s}function Qa(e){var t,n,s,a,u;for(t=new Rs,s=e,a=0,u=s.length;a<u;++a)n=s[a],t.a+=n.a,t.b+=n.b;return t}function Uge(e,t){Rt();var n,s,a,u,p;for(p=!1,s=t,a=0,u=s.length;a<u;++a)n=s[a],p=p|e.Fc(n);return p}function ky(e){Ry();var t,n;for(n=-17976931348623157e292,t=0;t<e.length;t++)e[t]>n&&(n=e[t]);return n}function Wlt(e,t,n){var s;return s=new qe,Xbe(e,t,s,(Qe(),pn),!0,!1),Xbe(e,n,s,un,!1,!1),s}function XK(e,t,n){var s,a,u,p;return u=null,p=t,a=wb(p,"labels"),s=new fJe(e,n),u=(yCn(s.a,s.b,a),a),u}function r6n(e,t,n,s){var a;return a=Fbe(e,t,n,s),!a&&(a=ikn(e,n,s),!!a&&!Hy(e,t,a))?null:a}function i6n(e,t,n,s){var a;return a=Rbe(e,t,n,s),!a&&(a=dX(e,n,s),!!a&&!Hy(e,t,a))?null:a}function Klt(e,t){var n;for(n=0;n<e.a.a.length;n++)if(!h(Unt(e.a,n),169).Lb(t))return!1;return!0}function s6n(e,t,n){if(Vn(t),n.Ob())for(bfe(t,Frt(n.Pb()));n.Ob();)bfe(t,e.a),bfe(t,Frt(n.Pb()));return t}function qge(e){Rt();var t,n,s;for(s=1,n=e.Kc();n.Ob();)t=n.Pb(),s=31*s+(t!=null?ii(t):0),s=s|0;return s}function a6n(e,t,n,s,a){var u;return u=Dbe(e,t),n&&KK(u),a&&(e=z8n(e,t),s?Rg=Hk(e):Rg=rc(e.l,e.m,e.h)),u}function o6n(e,t){var n;try{t.Vd()}catch(s){if(s=ui(s),ue(s,78))n=s,e.c[e.c.length]=n;else throw ne(s)}}function Xlt(e,t,n){var s,a;return ue(t,144)&&!!n?(s=h(t,144),a=n,e.a[s.b][a.b]+e.a[a.b][s.b]):0}function Yge(e,t){switch(t){case 7:return!!e.e&&e.e.i!=0;case 8:return!!e.d&&e.d.i!=0}return Lpe(e,t)}function c6n(e,t){switch(t.g){case 0:ue(e.b,631)||(e.b=new ylt);break;case 1:ue(e.b,632)||(e.b=new knt)}}function u6n(e,t){for(;e.g==null&&!e.c?t0e(e):e.g==null||e.i!=0&&h(e.g[e.i-1],47).Ob();)J1n(t,JF(e))}function Qlt(e,t,n){e.g=dQ(e,t,(Qe(),pn),e.b),e.d=dQ(e,n,pn,e.b),!(e.g.c==0||e.d.c==0)&&bdt(e)}function Zlt(e,t,n){e.g=dQ(e,t,(Qe(),un),e.j),e.d=dQ(e,n,un,e.j),!(e.g.c==0||e.d.c==0)&&bdt(e)}function l6n(e,t,n){return!U7(Jr(new Vt(null,new Wt(e.c,16)),new I7(new QZe(t,n)))).sd((Tp(),Ux))}function QK(e){var t;return vg(e),t=new Gt,e.a.sd(t)?(tk(),new oN(rn(t.a))):(tk(),tk(),$j)}function Wge(e){var t;return e.b<=0?!1:(t=Zf("MLydhHmsSDkK",Ec(Is(e.c,0))),t>1||t>=0&&e.b<3)}function GA(e){var t,n,s;for(t=new Oc,s=pr(e,0);s.b!=s.d.c;)n=h(gr(s),8),rk(t,0,new _a(n));return t}function zp(e){var t,n;for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),81),t.f.$b();Rfn(e.b,e),_gt(e)}function ii(e){return ws(e)?k0(e):ew(e)?J6(e):Jv(e)?(rn(e),e?1231:1237):bde(e)?e.Hb():hde(e)?rb(e):Kde(e)}function uu(e){return ws(e)?it:ew(e)?Ts:Jv(e)?es:bde(e)||hde(e)?e.gm:e.gm||Array.isArray(e)&&ie(o3t,1)||o3t}function Jlt(e){switch(e.g){case 0:return new FHe;default:throw ne(new an(cj+(e.f!=null?e.f:""+e.g)))}}function eht(e){switch(e.g){case 0:return new BHe;default:throw ne(new an(cj+(e.f!=null?e.f:""+e.g)))}}function Kge(e,t,n){switch(t){case 0:!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),rF(e.o,n);return}vQ(e,t,n)}function ZK(e,t,n){this.g=e,this.e=new Rs,this.f=new Rs,this.d=new di,this.b=new di,this.a=t,this.c=n}function JK(e,t,n,s){this.b=new qe,this.n=new qe,this.i=s,this.j=n,this.s=e,this.t=t,this.r=0,this.d=0}function Gp(e){this.e=e,this.d=new E0e(this.e.g),this.a=this.d,this.b=Oge(this),this.$modCount=e.$modCount}function h6n(e){for(;!e.d||!e.d.Ob();)if(!!e.b&&!$7(e.b))e.d=h(H4(e.b),47);else return null;return e.d}function f6n(e){return Ue(e.c,(n5(),yEt)),sge(e.a,Pe(Je(at((EX(),eH)))))?new bze:new NWe(e)}function Xge(e){switch(e.g){case 1:return vwt;default:case 2:return 0;case 3:return cJ;case 4:return wwt}}function d6n(){Mr();var e;return lie||(e=x0n(Lg("M",!0)),e=DP(Lg("M",!1),e),lie=e,lie)}function Qge(e,t){var n,s,a;for(a=e.b;a;){if(n=e.a.ue(t,a.d),n==0)return a;s=n<0?0:1,a=a.a[s]}return null}function g6n(e,t,n){var s,a;s=(on(),!!wfe(n)),a=h(t.xc(s),15),a||(a=new qe,t.zc(s,a)),a.Fc(n)}function p6n(e,t){var n,s;return n=h(pt(e,(CQ(),VD)),19).a,s=h(pt(t,VD),19).a,n==s||n<s?-1:n>s?1:0}function Zge(e,t){return mgt(e,t)?(Ot(e.b,h(U(t,(He(),em)),21),t),vr(e.a,t),!0):!1}function b6n(e){var t,n;t=h(U(e,(He(),Zc)),10),t&&(n=t.c,vc(n.a,t),n.a.c.length==0&&vc(Us(t).b,n))}function tht(e){return Pf?pe(C3t,rvt,572,0,0,1):h(Mf(e.a,pe(C3t,rvt,572,e.a.c.length,0,1)),842)}function v6n(e,t,n,s){return VP(),new Iq(se(ie(c2,1),TR,42,0,[(GX(e,t),new Z2(e,t)),(GX(n,s),new Z2(n,s))]))}function xy(e,t,n){var s,a;return a=(s=new Sq,s),ma(a,t,n),qn((!e.q&&(e.q=new Ye(eh,e,11,10)),e.q),a),a}function eX(e){var t,n,s,a;for(a=G1n(C9t,e),n=a.length,s=pe(it,Re,2,n,6,1),t=0;t<n;++t)s[t]=a[t];return s}function Ey(e,t){var n,s,a,u,p;for(s=t,a=0,u=s.length;a<u;++a)n=s[a],p=new Grt(e),n.Qe(p),kLn(p);Kc(e.f)}function tX(e,t){var n;return t===e?!0:ue(t,224)?(n=h(t,224),Nr(e.Zb(),n.Zb())):!1}function Jge(e,t){var n;t*2+1>=e.b.c.length||(Jge(e,2*t+1),n=2*t+2,n<e.b.c.length&&Jge(e,n),t0t(e,t))}function nht(e,t,n){var s,a;this.g=e,this.c=t,this.a=this,this.d=this,a=Rut(n),s=pe(a3t,PM,330,a,0,1),this.b=s}function epe(e,t,n){var s;for(s=n-1;s>=0&&e[s]===t[s];s--);return s<0?0:Yq(Zi(e[s],fa),Zi(t[s],fa))?-1:1}function w6n(e,t){var n,s;for(s=pr(e,0);s.b!=s.d.c;)n=h(gr(s),214),n.e.length>0&&(t.td(n),n.i&&fkn(n))}function nX(e,t){var n,s;return s=h(tn(e.a,4),126),n=pe(nie,Lee,415,t,0,1),s!=null&&Ao(s,0,n,0,s.length),n}function rht(e,t){var n;return n=new OQ((e.f&256)!=0,e.i,e.a,e.d,(e.f&16)!=0,e.j,e.g,t),e.e!=null||(n.c=e),n}function m6n(e,t){var n,s;for(s=e.Zb().Cc().Kc();s.Ob();)if(n=h(s.Pb(),14),n.Hc(t))return!0;return!1}function rX(e,t,n,s,a){var u,p;for(p=n;p<=a;p++)for(u=t;u<=s;u++)if(Ay(e,u,p))return!0;return!1}function iht(e,t,n){var s,a,u,p;for(rn(n),p=!1,u=e.Zc(t),a=n.Kc();a.Ob();)s=a.Pb(),u.Rb(s),p=!0;return p}function y6n(e,t){var n;return e===t?!0:ue(t,83)?(n=h(t,83),tbe(ob(e),n.vc())):!1}function sht(e,t,n){var s,a;for(a=n.Kc();a.Ob();)if(s=h(a.Pb(),42),e.re(t,s.dd()))return!0;return!1}function aht(e,t,n){return e.d[t.p][n.p]||(b8n(e,t,n),e.d[t.p][n.p]=!0,e.d[n.p][t.p]=!0),e.a[t.p][n.p]}function Gk(e,t){if(!e.ai()&&t==null)throw ne(new an("The 'no null' constraint is violated"));return t}function Vk(e,t){e.D==null&&e.B!=null&&(e.D=e.B,e.B=null),PK(e,t==null?null:(rn(t),t)),e.C&&e.yk(null)}function k6n(e,t){var n;return!e||e==t||!os(t,(He(),rv))?!1:(n=h(U(t,(He(),rv)),10),n!=e)}function iX(e){switch(e.i){case 2:return!0;case 1:return!1;case-1:++e.c;default:return e.pl()}}function oht(e){switch(e.i){case-2:return!0;case-1:return!1;case 1:--e.c;default:return e.ql()}}function cht(e){Qit.call(this,"The given string does not match the expected format for individual spacings.",e)}function Lu(){Lu=re,rl=new UN("ELK",0),tEe=new UN("JSON",1),eEe=new UN("DOT",2),nEe=new UN("SVG",3)}function VA(){VA=re,wH=new pY(jd,0),Pxe=new pY("RADIAL_COMPACTION",1),Bxe=new pY("WEDGE_COMPACTION",2)}function Af(){Af=re,X3e=new Wq("CONCURRENT",0),Zu=new Wq("IDENTITY_FINISH",1),Kw=new Wq("UNORDERED",2)}function sX(){sX=re,T4e=(kN(),bte),_4e=new $t(mwe,T4e),_4t=new Xr(ywe),T4t=new Xr(kwe),C4t=new Xr(xwe)}function Uk(){Uk=re,R5e=new fRe,j5e=new dRe,R5t=new gRe,F5t=new pRe,B5t=new bRe,F5e=(rn(B5t),new Ge)}function qk(){qk=re,Kne=new lY("CONSERVATIVE",0),cxe=new lY("CONSERVATIVE_SOFT",1),F_=new lY("SLOPPY",2)}function uF(){uF=re,G7e=new nb(15),ZEt=new sa((Tr(),y2),G7e),eT=q5,j7e=PEt,$7e=m2,z7e=_3,H7e=IH}function aX(e,t,n){var s,a,u;for(s=new di,u=pr(n,0);u.b!=u.d.c;)a=h(gr(u),8),vr(s,new _a(a));iht(e,t,s)}function x6n(e){var t,n,s;for(t=0,s=pe(cs,Re,8,e.b,0,1),n=pr(e,0);n.b!=n.d.c;)s[t++]=h(gr(n),8);return s}function tpe(e){var t;return t=(!e.a&&(e.a=new Ye(qd,e,9,5)),e.a),t.i!=0?$1n(h(he(t,0),678)):null}function E6n(e,t){var n;return n=Vs(e,t),Yq(ZW(e,t),0)|ndn(ZW(e,n),0)?n:Vs(_R,ZW(ow(n,63),1))}function _6n(e,t){var n;n=at((EX(),eH))!=null&&t.wg()!=null?Pe(Je(t.wg()))/Pe(Je(at(eH))):1,Pr(e.b,t,n)}function T6n(e,t){var n,s;return n=h(e.d.Bc(t),14),n?(s=e.e.hc(),s.Gc(n),e.e.d-=n.gc(),n.$b(),s):null}function npe(e,t){var n,s;if(s=e.c[t],s!=0)for(e.c[t]=0,e.d-=s,n=t+1;n<e.a.length;)e.a[n]-=s,n+=n&-n}function uht(e){var t;if(t=e.a.c.length,t>0)return ck(t-1,e.a.c.length),f0(e.a,t-1);throw ne(new OKe)}function C6n(e,t,n){if(t<0)throw ne(new Ea(Lwt+t));t<e.j.c.length?dl(e.j,t,n):(wat(e,t),Ue(e.j,n))}function lht(e,t,n){if(e>t)throw ne(new an(MR+e+ivt+t));if(e<0||t>n)throw ne(new bhe(MR+e+awe+t+iwe+n))}function hht(e){if(!e.a||(e.a.i&8)==0)throw ne(new ja("Enumeration class expected for layout option "+e.f))}function kw(e){var t;++e.j,e.i==0?e.g=null:e.i<e.g.length&&(t=e.g,e.g=e.ri(e.i),Ao(t,0,e.g,0,e.i))}function S6n(e,t){var n,s;for(n=e.a.length-1,e.c=e.c-1&n;t!=e.c;)s=t+1&n,pi(e.a,t,e.a[s]),t=s;pi(e.a,e.c,null)}function A6n(e,t){var n,s;for(n=e.a.length-1;t!=e.b;)s=t-1&n,pi(e.a,t,e.a[s]),t=s;pi(e.a,e.b,null),e.b=e.b+1&n}function rpe(e,t,n){var s,a;return fw(t,e.c.length),s=n.Pc(),a=s.length,a==0?!1:(q1e(e.c,t,s),!0)}function M6n(e){var t,n;if(e==null)return null;for(t=0,n=e.length;t<n;t++)if(!Snt(e[t]))return e[t];return null}function fht(e,t,n){var s,a,u,p;for(a=n,u=0,p=a.length;u<p;++u)if(s=a[u],e.b.re(t,s.cd()))return s;return null}function lF(e){var t,n,s,a,u;for(u=1,n=e,s=0,a=n.length;s<a;++s)t=n[s],u=31*u+(t!=null?ii(t):0),u=u|0;return u}function ur(e){var t,n,s,a,u;for(t={},s=e,a=0,u=s.length;a<u;++a)n=s[a],t[":"+(n.f!=null?n.f:""+n.g)]=n;return t}function D6n(e){var t;for(Vn(e),O1e(!0,"numberToAdvance must be nonnegative"),t=0;t<0&&rr(e);t++)Un(e);return t}function dht(e){var t,n,s;for(s=0,n=new Sn(Dn(e.a.Kc(),new L));rr(n);)t=h(Un(n),17),t.c.i==t.d.i||++s;return s}function ght(e,t){var n,s,a;n=e,a=0;do{if(n==t)return a;if(s=n.e,!s)throw ne(new pS);n=Us(s),++a}while(!0)}function pht(e,t){var n,s,a;for(a=t-e.f,s=new Q(e.d);s.a<s.c.c.length;)n=h(K(s),443),oft(n,n.e,n.f+a);e.f=t}function oX(e,t,n){return d.Math.abs(t-e)<ij||d.Math.abs(n-e)<ij?!0:t-e>ij?e-n>ij:n-e>ij}function cX(e,t){return!e||t&&!e.j||ue(e,124)&&h(e,124).a.b==0?0:e.Re()}function hF(e,t){return!e||t&&!e.k||ue(e,124)&&h(e,124).a.a==0?0:e.Se()}function UA(e){return Dg(),e<0?e!=-1?new Qpe(-1,-e):ite:e<=10?R3e[Li(e)]:new Qpe(1,e)}function ipe(e){throw YK(),ne(new CXe("Unexpected typeof result '"+e+"'; please report this bug to the GWT team"))}function bht(e){IXe(),nP(this),KP(this),this.e=e,Mgt(this,e),this.g=e==null?_c:Ga(e),this.a="",this.b=e,this.a=""}function spe(){this.a=new OHe,this.f=new kWe(this),this.b=new xWe(this),this.i=new EWe(this),this.e=new _We(this)}function vht(){Pfn.call(this,new z0e(vw(16))),Uu(2,jbt),this.b=2,this.a=new Hde(null,null,0,null),gS(this.a,this.a)}function t9(){t9=re,jne=new aY("DUMMY_NODE_OVER",0),qke=new aY("DUMMY_NODE_UNDER",1),nH=new aY("EQUAL",2)}function uX(){uX=re,Ate=Jit(se(ie(J_,1),ze,103,0,[(la(),Wl),Dh])),Mte=Jit(se(ie(J_,1),ze,103,0,[Gd,zd]))}function lX(e){return(Qe(),Cc).Hc(e.j)?Pe(Je(U(e,(He(),a8)))):Qa(se(ie(cs,1),Re,8,0,[e.i.n,e.n,e.a])).b}function L6n(e){var t,n,s,a;for(s=e.b.a,n=s.a.ec().Kc();n.Ob();)t=h(n.Pb(),561),a=new ugt(t,e.e,e.f),Ue(e.g,a)}function Vp(e,t){var n,s,a;s=e.nk(t,null),a=null,t&&(a=(q6(),n=new V2,n),Sk(a,e.r)),s=Df(e,a,s),s&&s.Fi()}function I6n(e,t){var n,s;for(s=hu(e.d,1)!=0,n=!0;n;)n=!1,n=t.c.Tf(t.e,s),n=n|kM(e,t,s,!1),s=!s;Ege(e)}function ape(e,t){var n,s,a;return s=!1,n=t.q.d,t.d<e.b&&(a=ygt(t.q,e.b),t.q.d>a&&(G1t(t.q,a),s=n!=t.q.d)),s}function wht(e,t){var n,s,a,u,p,v,m,x;return m=t.i,x=t.j,s=e.f,a=s.i,u=s.j,p=m-a,v=x-u,n=d.Math.sqrt(p*p+v*v),n}function ope(e,t){var n,s;return s=TF(e),s||(n=(nZ(),Vdt(t)),s=new _Ke(n),qn(s.Vk(),e)),s}function qA(e,t){var n,s;return n=h(e.c.Bc(t),14),n?(s=e.hc(),s.Gc(n),e.d-=n.gc(),n.$b(),e.mc(s)):e.jc()}function mht(e,t){var n;for(n=0;n<t.length;n++)if(e==(er(n,t.length),t.charCodeAt(n)))return!0;return!1}function yht(e,t){var n;for(n=0;n<t.length;n++)if(e==(er(n,t.length),t.charCodeAt(n)))return!0;return!1}function O6n(e){var t,n;if(e==null)return!1;for(t=0,n=e.length;t<n;t++)if(!Snt(e[t]))return!1;return!0}function kht(e){var t;if(e.c!=0)return e.c;for(t=0;t<e.a.length;t++)e.c=e.c*33+(e.a[t]&-1);return e.c=e.c*e.e,e.c}function fF(e){var t;return kn(e.a!=e.b),t=e.d.a[e.a],Qet(e.b==e.d.c&&t!=null),e.c=e.a,e.a=e.a+1&e.d.a.length-1,t}function N6n(e){var t;if(!(e.c.c<0?e.a>=e.c.b:e.a<=e.c.b))throw ne(new lo);return t=e.a,e.a+=e.c.c,++e.b,Ke(t)}function P6n(e){var t;return t=new Zct(e),fA(e.a,k5t,new _u(se(ie(yD,1),st,369,0,[t]))),t.d&&Ue(t.f,t.d),t.f}function hX(e){var t;return t=new Afe(e.a),Oa(t,e),Fe(t,(He(),Br),e),t.o.a=e.g,t.o.b=e.f,t.n.a=e.i,t.n.b=e.j,t}function B6n(e,t,n,s){var a,u;for(u=e.Kc();u.Ob();)a=h(u.Pb(),70),a.n.a=t.a+(s.a-a.o.a)/2,a.n.b=t.b,t.b+=a.o.b+n}function F6n(e,t,n){var s,a;for(a=t.a.a.ec().Kc();a.Ob();)if(s=h(a.Pb(),57),Iit(e,s,n))return!0;return!1}function R6n(e){var t,n;for(n=new Q(e.r);n.a<n.c.c.length;)if(t=h(K(n),10),e.n[t.p]<=0)return t;return null}function xht(e){var t,n,s,a;for(a=new ns,s=new Q(e);s.a<s.c.c.length;)n=h(K(s),33),t=YAn(n),Qs(a,t);return a}function j6n(e){var t;return t=QN(e8t),h(U(e,(He(),Uo)),21).Hc((ha(),i8))&&Lr(t,(Zs(),oc),(oa(),g$)),t}function $6n(e,t,n){var s;s=new S0t(e,t),Ot(e.r,t.Hf(),s),n&&!DE(e.u)&&(s.c=new Wit(e.d),mc(t.wf(),new Yqe(s)))}function mo(e,t){var n;return $a(e)&&$a(t)&&(n=e-t,!isNaN(n))?n:k2e($a(e)?Bl(e):e,$a(t)?Bl(t):t)}function H6n(e,t){return t<e.length&&(er(t,e.length),e.charCodeAt(t)!=63)&&(er(t,e.length),e.charCodeAt(t)!=35)}function Eht(e,t,n,s){var a,u;e.a=t,u=s?0:1,e.f=(a=new Cdt(e.c,e.a,n,u),new rpt(n,e.a,a,e.e,e.b,e.c==(_b(),R_)))}function cpe(e,t,n){var s,a;return a=e.a,e.a=t,(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,1,a,t),n?n.Ei(s):n=s),n}function _ht(e,t,n){var s,a;return a=e.b,e.b=t,(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,3,a,t),n?n.Ei(s):n=s),n}function Tht(e,t,n){var s,a;return a=e.f,e.f=t,(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,0,a,t),n?n.Ei(s):n=s),n}function Tg(e,t){var n,s,a,u;return u=(a=e?TF(e):null,Xdt((s=t,a&&a.Xk(),s))),u==t&&(n=TF(e),n&&n.Xk()),u}function upe(e,t){var n,s,a;for(a=1,n=e,s=t>=0?t:-t;s>0;)s%2==0?(n*=n,s=s/2|0):(a*=n,s-=1);return t<0?1/a:a}function z6n(e,t){var n,s,a;for(a=1,n=e,s=t>=0?t:-t;s>0;)s%2==0?(n*=n,s=s/2|0):(a*=n,s-=1);return t<0?1/a:a}function Cht(e){var t,n;if(e!=null)for(n=0;n<e.length;++n)t=e[n],t&&(h(t.g,367),t.i)}function G6n(e){var t,n,s;for(s=0,n=new Q(e.a);n.a<n.c.c.length;)t=h(K(n),187),s=d.Math.max(s,t.g);return s}function V6n(e){var t,n,s;for(s=new Q(e.b);s.a<s.c.c.length;)n=h(K(s),214),t=n.c.Rf()?n.f:n.a,t&&uIn(t,n.j)}function Ld(){Ld=re,BH=new yY("INHERIT",0),B0=new yY("INCLUDE_CHILDREN",1),tT=new yY("SEPARATE_CHILDREN",2)}function lpe(e,t){switch(t){case 1:!e.n&&(e.n=new Ye(Ua,e,1,7)),jn(e.n);return;case 2:WE(e,null);return}Rge(e,t)}function dF(e){var t;switch(e.gc()){case 0:return Kee;case 1:return new sW(Vn(e.Xb(0)));default:return t=e,new jW(t)}}function Sht(e){switch(t1(),e.gc()){case 0:return AW(),g3e;case 1:return new C4(e.Kc().Pb());default:return new jhe(e)}}function b0(e){switch(t1(),e.c){case 0:return AW(),g3e;case 1:return new C4(s0t(new A4(e)));default:return new _Xe(e)}}function xw(e,t){Vn(e);try{return e.xc(t)}catch(n){if(n=ui(n),ue(n,205)||ue(n,173))return null;throw ne(n)}}function U6n(e,t){Vn(e);try{return e.Bc(t)}catch(n){if(n=ui(n),ue(n,205)||ue(n,173))return null;throw ne(n)}}function hpe(e,t){Vn(e);try{return e.Hc(t)}catch(n){if(n=ui(n),ue(n,205)||ue(n,173))return!1;throw ne(n)}}function q6n(e,t){Vn(e);try{return e.Mc(t)}catch(n){if(n=ui(n),ue(n,205)||ue(n,173))return!1;throw ne(n)}}function Aht(e,t){Vn(e);try{return e._b(t)}catch(n){if(n=ui(n),ue(n,205)||ue(n,173))return!1;throw ne(n)}}function Mht(e,t){var n;e.a.c.length>0&&(n=h(ut(e.a,e.a.c.length-1),570),Zge(n,t))||Ue(e.a,new sot(t))}function Y6n(e){wh();var t,n;t=e.d.c-e.e.c,n=h(e.g,145),mc(n.b,new DYe(t)),mc(n.c,new LYe(t)),Os(n.i,new IYe(t))}function Dht(e){var t;return t=new rg,t.a+="VerticalSegment ",ho(t,e.e),t.a+=" ",or(t,Rfe(new $q,new Q(e.k))),t.a}function W6n(e){var t;return t=h(mb(e.c.c,""),229),t||(t=new q4(U6(V6(new m4,""),"Other")),Yp(e.c.c,"",t)),t}function n9(e){var t;return(e.Db&64)!=0?_h(e):(t=new Ol(_h(e)),t.a+=" (name: ",va(t,e.zb),t.a+=")",t.a)}function fpe(e,t,n){var s,a;return a=e.sb,e.sb=t,(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,4,a,t),n?n.Ei(s):n=s),n}function fX(e,t){var n,s,a;for(n=0,a=Za(e,t).Kc();a.Ob();)s=h(a.Pb(),11),n+=U(s,(He(),Zc))!=null?1:0;return n}function _y(e,t,n){var s,a,u;for(s=0,u=pr(e,0);u.b!=u.d.c&&(a=Pe(Je(gr(u))),!(a>n));)a>=t&&++s;return s}function K6n(e,t,n){var s,a;return s=new Sd(e.e,3,13,null,(a=t.c,a||(Bt(),zf)),y0(e,t),!1),n?n.Ei(s):n=s,n}function X6n(e,t,n){var s,a;return s=new Sd(e.e,4,13,(a=t.c,a||(Bt(),zf)),null,y0(e,t),!1),n?n.Ei(s):n=s,n}function dpe(e,t,n){var s,a;return a=e.r,e.r=t,(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,8,a,e.r),n?n.Ei(s):n=s),n}function v0(e,t){var n,s;return n=h(t,676),s=n.vk(),!s&&n.wk(s=ue(t,88)?new _Je(e,h(t,26)):new Yst(e,h(t,148))),s}function YA(e,t,n){var s;e.qi(e.i+1),s=e.oi(t,n),t!=e.i&&Ao(e.g,t,e.g,t+1,e.i-t),pi(e.g,t,s),++e.i,e.bi(t,n),e.ci()}function Q6n(e,t){var n;return t.a&&(n=t.a.a.length,e.a?or(e.a,e.b):e.a=new ju(e.d),Kst(e.a,t.a,t.d.length,n)),e}function Z6n(e,t){var n,s,a,u;if(t.vi(e.a),u=h(tn(e.a,8),1936),u!=null)for(n=u,s=0,a=n.length;s<a;++s)null.jm()}function WA(e,t){var n;return n=new Gt,e.a.sd(n)?(tk(),new oN(rn(Cct(e,n.a,t)))):(vg(e),tk(),tk(),$j)}function r9(e,t){switch(t.g){case 2:case 1:return Za(e,t);case 3:case 4:return Rp(Za(e,t))}return Rt(),Rt(),ca}function Nr(e,t){return ws(e)?Nt(e,t):ew(e)?Utt(e,t):Jv(e)?(rn(e),Me(e)===Me(t)):bde(e)?e.Fb(t):hde(e)?VJe(e,t):c0e(e,t)}function J6n(e){return e?(e.i&1)!=0?e==vu?es:e==Hn?Ws:e==Em?zx:e==ks?Ts:e==ap?Xb:e==D3?Qb:e==Vc?A5:f_:e:null}function ekn(e,t,n,s,a){t==0||s==0||(t==1?a[s]=e2e(a,n,s,e[0]):s==1?a[t]=e2e(a,e,t,n[0]):nCn(e,n,a,t,s))}function Lht(e,t){var n;e.c.length!=0&&(n=h(Mf(e,pe(X1,T0,10,e.c.length,0,1)),193),yfe(n,new yFe),w0t(n,t))}function Iht(e,t){var n;e.c.length!=0&&(n=h(Mf(e,pe(X1,T0,10,e.c.length,0,1)),193),yfe(n,new kFe),w0t(n,t))}function gpe(e,t,n,s){switch(t){case 1:return!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n;case 2:return e.k}return l2e(e,t,n,s)}function la(){la=re,Q1=new PS(j9,0),Dh=new PS(m5,1),Wl=new PS(w5,2),zd=new PS(ZZ,3),Gd=new PS("UP",4)}function Sb(){Sb=re,i3=new Jq(jd,0),Y4e=new Jq("INSIDE_PORT_SIDE_GROUPS",1),Lte=new Jq("FORCE_MODEL_ORDER",2)}function Oht(e,t,n){if(e<0||t>n)throw ne(new Ea(MR+e+awe+t+", size: "+n));if(e>t)throw ne(new an(MR+e+ivt+t))}function vl(e,t,n){if(t<0)bbe(e,n);else{if(!n.Ij())throw ne(new an(Fg+n.ne()+J9));h(n,66).Nj().Vj(e,e.yh(),t)}}function tkn(e,t,n,s,a,u,p,v){var m;for(m=n;u<p;)m>=s||t<n&&v.ue(e[t],e[m])<=0?pi(a,u++,e[t++]):pi(a,u++,e[m++])}function Nht(e,t,n,s,a,u){this.e=new qe,this.f=(ua(),B_),Ue(this.e,e),this.d=t,this.a=n,this.b=s,this.f=a,this.c=u}function Pht(e,t){var n,s;for(s=new Tn(e);s.e!=s.i.gc();)if(n=h(On(s),26),Me(t)===Me(n))return!0;return!1}function nkn(e){xR();var t,n,s,a;for(n=LX(),s=0,a=n.length;s<a;++s)if(t=n[s],Ha(t.a,e,0)!=-1)return t;return dte}function Bht(e){return e>=65&&e<=70?e-65+10:e>=97&&e<=102?e-97+10:e>=48&&e<=57?e-48:0}function Fht(e){var t;return(e.Db&64)!=0?_h(e):(t=new Ol(_h(e)),t.a+=" (source: ",va(t,e.d),t.a+=")",t.a)}function rkn(e,t,n){var s,a;return a=e.a,e.a=t,(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,5,a,e.a),n?B2e(n,s):n=s),n}function w0(e,t){var n;n=(e.Bb&256)!=0,t?e.Bb|=256:e.Bb&=-257,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,2,n,t))}function ppe(e,t){var n;n=(e.Bb&256)!=0,t?e.Bb|=256:e.Bb&=-257,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,8,n,t))}function gF(e,t){var n;n=(e.Bb&256)!=0,t?e.Bb|=256:e.Bb&=-257,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,8,n,t))}function m0(e,t){var n;n=(e.Bb&512)!=0,t?e.Bb|=512:e.Bb&=-513,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,3,n,t))}function bpe(e,t){var n;n=(e.Bb&512)!=0,t?e.Bb|=512:e.Bb&=-513,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,9,n,t))}function i9(e,t){var n;return e.b==-1&&!!e.a&&(n=e.a.Gj(),e.b=n?e.c.Xg(e.a.aj(),n):oi(e.c.Tg(),e.a)),e.c.Og(e.b,t)}function Ke(e){var t,n;return e>-129&&e<128?(t=e+128,n=(Pnt(),T3e)[t],!n&&(n=T3e[t]=new wle(e)),n):new wle(e)}function Yk(e){var t,n;return e>-129&&e<128?(t=e+128,n=(Hnt(),M3e)[t],!n&&(n=M3e[t]=new yle(e)),n):new yle(e)}function vpe(e){var t,n;return t=e.k,t==(bn(),Pi)?(n=h(U(e,(He(),co)),61),n==(Qe(),sn)||n==Fn):!1}function ikn(e,t,n){var s,a,u;return u=(a=lx(e.b,t),a),u&&(s=h(bR(TA(e,u),""),26),s)?Fbe(e,s,t,n):null}function dX(e,t,n){var s,a,u;return u=(a=lx(e.b,t),a),u&&(s=h(bR(TA(e,u),""),26),s)?Rbe(e,s,t,n):null}function Rht(e,t){var n,s;for(s=new Tn(e);s.e!=s.i.gc();)if(n=h(On(s),138),Me(t)===Me(n))return!0;return!1}function s9(e,t,n){var s;if(s=e.gc(),t>s)throw ne(new sw(t,s));if(e.hi()&&e.Hc(n))throw ne(new an(nD));e.Xh(t,n)}function skn(e,t){var n;if(n=wy(e.i,t),n==null)throw ne(new Xf("Node did not exist in input."));return Age(t,n),null}function akn(e,t){var n;if(n=yM(e,t),ue(n,322))return h(n,34);throw ne(new an(Fg+t+"' is not a valid attribute"))}function okn(e,t,n){var s,a;for(a=ue(t,99)&&(h(t,18).Bb&ea)!=0?new _Y(t,e):new Rk(t,e),s=0;s<n;++s)OF(a);return a}function ckn(e){var t,n,s;for(s=0,n=e.length,t=0;t<n;t++)e[t]==32||e[t]==13||e[t]==10||e[t]==9||(e[s++]=e[t]);return s}function ukn(e){var t,n,s;for(t=new qe,s=new Q(e.b);s.a<s.c.c.length;)n=h(K(s),594),Vi(t,h(n.jf(),14));return t}function lkn(e){var t,n,s;for(t=h(U(e,(fo(),mxe)),15),s=t.Kc();s.Ob();)n=h(s.Pb(),188),vr(n.b.d,n),vr(n.c.b,n)}function hkn(e){switch(h(U(e,(He(),h2)),303).g){case 1:Fe(e,h2,(Ad(),O5));break;case 2:Fe(e,h2,(Ad(),l3))}}function fkn(e){var t;e.g&&(t=e.c.Rf()?e.f:e.a,Qbe(t.a,e.o,!0),Qbe(t.a,e.o,!1),Fe(e.o,(tt(),Ei),(_s(),x2)))}function dkn(e){var t;if(!e.a)throw ne(new ja("Cannot offset an unassigned cut."));t=e.c-e.b,e.b+=t,Dit(e,t),Mit(e,t)}function gkn(e){var t;return t=e.a[e.c-1&e.a.length-1],t==null?null:(e.c=e.c-1&e.a.length-1,pi(e.a,e.c,null),t)}function jht(e){var t,n;for(n=e.p.a.ec().Kc();n.Ob();)if(t=h(n.Pb(),213),t.f&&e.b[t.c]<-1e-10)return t;return null}function wpe(e,t){switch(e.b.g){case 0:case 1:return t;case 2:case 3:return new hl(t.d,0,t.a,t.b);default:return null}}function $ht(e){switch(e.g){case 2:return Dh;case 1:return Wl;case 4:return zd;case 3:return Gd;default:return Q1}}function mpe(e){switch(e.g){case 1:return un;case 2:return sn;case 3:return pn;case 4:return Fn;default:return to}}function KA(e){switch(e.g){case 1:return Fn;case 2:return un;case 3:return sn;case 4:return pn;default:return to}}function pF(e){switch(e.g){case 1:return pn;case 2:return Fn;case 3:return un;case 4:return sn;default:return to}}function pkn(e){switch(e){case 0:return new rXe;case 1:return new tXe;case 2:return new nXe;default:throw ne(new pS)}}function Ui(e,t){return e<t?-1:e>t?1:e==t?e==0?Ui(1/e,1/t):0:isNaN(e)?isNaN(t)?0:1:-1}function bkn(e,t){Bn(t,"Sort end labels",1),Ti(Jr(Xa(new Vt(null,new Wt(e.b,16)),new iFe),new sFe),new aFe),An(t)}function a9(e,t,n){var s,a;return e.ej()?(a=e.fj(),s=yQ(e,t,n),e.$i(e.Zi(7,Ke(n),s,t,a)),s):yQ(e,t,n)}function gX(e,t){var n,s,a;e.d==null?(++e.e,--e.f):(a=t.cd(),n=t.Sh(),s=(n&Ir)%e.d.length,Jyn(e,s,ngt(e,s,n,a)))}function Wk(e,t){var n;n=(e.Bb&Ch)!=0,t?e.Bb|=Ch:e.Bb&=-1025,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,10,n,t))}function Kk(e,t){var n;n=(e.Bb&$w)!=0,t?e.Bb|=$w:e.Bb&=-4097,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,12,n,t))}function Xk(e,t){var n;n=(e.Bb&zc)!=0,t?e.Bb|=zc:e.Bb&=-8193,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,15,n,t))}function Qk(e,t){var n;n=(e.Bb&Ww)!=0,t?e.Bb|=Ww:e.Bb&=-2049,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,11,n,t))}function vkn(e,t){var n;return n=Ui(e.b.c,t.b.c),n!=0||(n=Ui(e.a.a,t.a.a),n!=0)?n:Ui(e.a.b,t.a.b)}function wkn(e,t){var n;if(n=_n(e.k,t),n==null)throw ne(new Xf("Port did not exist in input."));return Age(t,n),null}function mkn(e){var t,n;for(n=ogt(Vu(e)).Kc();n.Ob();)if(t=Jn(n.Pb()),T9(e,t))return Syn((gZe(),R9t),t);return null}function ykn(e,t){var n,s,a,u,p;for(p=ac(e.e.Tg(),t),u=0,n=h(e.g,119),a=0;a<e.i;++a)s=n[a],p.rl(s.ak())&&++u;return u}function kkn(e,t,n){var s,a;return s=h(t.We(e.a),35),a=h(n.We(e.a),35),s!=null&&a!=null?OA(s,a):s!=null?-1:a!=null?1:0}function Hht(e,t,n){var s,a;if(e.c)Y2e(e.c,t,n);else for(a=new Q(e.b);a.a<a.c.c.length;)s=h(K(a),157),Hht(s,t,n)}function pX(e,t){var n,s;for(s=new Q(t);s.a<s.c.c.length;)n=h(K(s),46),vc(e.b.b,n.b),Zvn(h(n.a,189),h(n.b,81))}function xkn(e){var t,n;for(n=pg(new rg,91),t=!0;e.Ob();)t||(n.a+=Js),t=!1,ho(n,e.Pb());return(n.a+="]",n).a}function Zk(e,t){var n;n=(e.Bb&a1)!=0,t?e.Bb|=a1:e.Bb&=-16385,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,16,n,t))}function bX(e,t){var n;n=(e.Bb&go)!=0,t?e.Bb|=go:e.Bb&=-32769,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,18,n,t))}function ype(e,t){var n;n=(e.Bb&go)!=0,t?e.Bb|=go:e.Bb&=-32769,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,18,n,t))}function kpe(e,t){var n;n=(e.Bb&ea)!=0,t?e.Bb|=ea:e.Bb&=-65537,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new kh(e,1,20,n,t))}function xpe(e){var t;return t=pe(Cl,o1,25,2,15,1),e-=ea,t[0]=(e>>10)+jM&Oi,t[1]=(e&1023)+56320&Oi,Fl(t,0,t.length)}function bF(e){var t,n;return n=h(U(e,(tt(),Ju)),103),n==(la(),Q1)?(t=Pe(Je(U(e,j$))),t>=1?Dh:zd):n}function Ekn(e){switch(h(U(e,(tt(),Hd)),218).g){case 1:return new Uje;case 3:return new Xje;default:return new Vje}}function Up(e){if(e.c)Up(e.c);else if(e.d)throw ne(new ja("Stream already terminated, can't be modified or used"))}function vX(e){var t;return(e.Db&64)!=0?_h(e):(t=new Ol(_h(e)),t.a+=" (identifier: ",va(t,e.k),t.a+=")",t.a)}function zht(e,t,n){var s,a;return s=(X2(),a=new zO,a),$B(s,t),HB(s,n),e&&qn((!e.a&&(e.a=new Gi(Zl,e,5)),e.a),s),s}function wX(e,t,n,s){var a,u;return rn(s),rn(n),a=e.xc(t),u=a==null?n:tZe(h(a,15),h(n,14)),u==null?e.Bc(t):e.zc(t,u),u}function Lt(e){var t,n,s,a;return n=(t=h(Vh((s=e.gm,a=s.f,a==sr?s:a)),9),new ll(t,h(vh(t,t.length),9),0)),Eh(n,e),n}function _kn(e,t,n){var s,a;for(a=e.a.ec().Kc();a.Ob();)if(s=h(a.Pb(),10),FA(n,h(ut(t,s.p),14)))return s;return null}function Tkn(e,t,n){var s;try{s6n(e,t,n)}catch(a){throw a=ui(a),ue(a,597)?(s=a,ne(new T0e(s))):ne(a)}return t}function Cg(e,t){var n;return $a(e)&&$a(t)&&(n=e-t,RM<n&&n<Jp)?n:xg(Slt($a(e)?Bl(e):e,$a(t)?Bl(t):t))}function Vs(e,t){var n;return $a(e)&&$a(t)&&(n=e+t,RM<n&&n<Jp)?n:xg(H5n($a(e)?Bl(e):e,$a(t)?Bl(t):t))}function js(e,t){var n;return $a(e)&&$a(t)&&(n=e*t,RM<n&&n<Jp)?n:xg(cIn($a(e)?Bl(e):e,$a(t)?Bl(t):t))}function Za(e,t){var n;return e.i||pbe(e),n=h(wa(e.g,t),46),n?new F1(e.j,h(n.a,19).a,h(n.b,19).a):(Rt(),Rt(),ca)}function Ab(e,t,n){var s;return s=e.a.get(t),e.a.set(t,n===void 0?null:n),s===void 0?(++e.c,JP(e.b)):++e.d,s}function Ckn(e,t,n){e.n=Ap(ap,[Re,RZ],[364,25],14,[n,Li(d.Math.ceil(t/32))],2),e.o=t,e.p=n,e.j=t-1>>1,e.k=n-1>>1}function mX(){z2e();var e,t,n;n=mPn+++Date.now(),e=Li(d.Math.floor(n*HM))&AR,t=Li(n-e*rwe),this.a=e^1502,this.b=t^VZ}function Id(e){var t,n,s;for(t=new qe,s=new Q(e.j);s.a<s.c.c.length;)n=h(K(s),11),Ue(t,n.b);return Vn(t),new pE(t)}function za(e){var t,n,s;for(t=new qe,s=new Q(e.j);s.a<s.c.c.length;)n=h(K(s),11),Ue(t,n.e);return Vn(t),new pE(t)}function qi(e){var t,n,s;for(t=new qe,s=new Q(e.j);s.a<s.c.c.length;)n=h(K(s),11),Ue(t,n.g);return Vn(t),new pE(t)}function Skn(e){var t,n;for(n=_Cn(Vu(hw(e))).Kc();n.Ob();)if(t=Jn(n.Pb()),T9(e,t))return Ayn((pZe(),j9t),t);return null}function Akn(e){var t,n,s;for(n=0,s=e.length;n<s;n++)if(e[n]==null)throw ne(new T4("at index "+n));return t=e,new _u(t)}function Mkn(e,t){var n;if(n=yM(e.Tg(),t),ue(n,99))return h(n,18);throw ne(new an(Fg+t+"' is not a valid reference"))}function Dkn(e){var t;return t=Dw(e),t>34028234663852886e22?ki:t<-34028234663852886e22?Fi:t}function Ght(e){return e-=e>>1&1431655765,e=(e>>2&858993459)+(e&858993459),e=(e>>4)+e&252645135,e+=e>>8,e+=e>>16,e&63}function Vht(e){var t,n,s,a;for(t=new _tt(e.Hd().gc()),a=0,s=J4(e.Hd().Kc());s.Ob();)n=s.Pb(),$wn(t,n,Ke(a++));return h_n(t.a)}function Lkn(e,t){var n,s,a;for(a=new zn,s=t.vc().Kc();s.Ob();)n=h(s.Pb(),42),Pr(a,n.cd(),x5n(e,h(n.dd(),15)));return a}function Epe(e,t){e.n.c.length==0&&Ue(e.n,new sB(e.s,e.t,e.i)),Ue(e.b,t),s2e(h(ut(e.n,e.n.c.length-1),211),t),e2t(e,t)}function Ty(e){return(e.c!=e.b.b||e.i!=e.g.b)&&(e.a.c=pe(yn,st,1,0,5,1),Vi(e.a,e.b),Vi(e.a,e.g),e.c=e.b.b,e.i=e.g.b),e.a}function yX(e,t){var n,s,a;for(a=0,s=h(t.Kb(e),20).Kc();s.Ob();)n=h(s.Pb(),17),ft(lt(U(n,(He(),Ff))))||++a;return a}function Ikn(e,t){var n,s,a;s=by(t),a=Pe(Je(_w(s,(tt(),Mh)))),n=d.Math.max(0,a/2-.5),g9(t,n,1),Ue(e,new NZe(t,n))}function wl(){wl=re,LD=new LS(jd,0),C_=new LS("FIRST",1),Gg=new LS(Qvt,2),S_=new LS("LAST",3),nm=new LS(Zvt,4)}function Od(){Od=re,qre=new HN(j9,0),nL=new HN("POLYLINE",1),y8=new HN("ORTHOGONAL",2),W5=new HN("SPLINES",3)}function vF(){vF=re,e8e=new vY("ASPECT_RATIO_DRIVEN",0),wre=new vY("MAX_SCALE_DRIVEN",1),Jxe=new vY("AREA_DRIVEN",2)}function XA(){XA=re,kH=new wY("P1_STRUCTURE",0),xH=new wY("P2_PROCESSING_ORDER",1),EH=new wY("P3_EXECUTION",2)}function wF(){wF=re,fre=new gY("OVERLAP_REMOVAL",0),lre=new gY("COMPACTION",1),hre=new gY("GRAPH_SIZE_CALCULATION",2)}function Mb(e,t){return vf(),xh(Ig),d.Math.abs(e-t)<=Ig||e==t||isNaN(e)&&isNaN(t)?0:e<t?-1:e>t?1:tb(isNaN(e),isNaN(t))}function Uht(e,t){var n,s;for(n=pr(e,0);n.b!=n.d.c;){if(s=bS(Je(gr(n))),s==t)return;if(s>t){eK(n);break}}nA(n,t)}function Dt(e,t){var n,s,a,u,p;if(n=t.f,Yp(e.c.d,n,t),t.g!=null)for(a=t.g,u=0,p=a.length;u<p;++u)s=a[u],Yp(e.c.e,s,t)}function Okn(e,t,n,s){var a,u,p;for(a=t+1;a<n;++a)for(u=a;u>t&&s.ue(e[u-1],e[u])>0;--u)p=e[u],pi(e,u,e[u-1]),pi(e,u-1,p)}function ml(e,t,n,s){if(t<0)zbe(e,n,s);else{if(!n.Ij())throw ne(new an(Fg+n.ne()+J9));h(n,66).Nj().Tj(e,e.yh(),t,s)}}function mF(e,t){if(t==e.d)return e.e;if(t==e.e)return e.d;throw ne(new an("Node "+t+" not part of edge "+e))}function Nkn(e,t){switch(t.g){case 2:return e.b;case 1:return e.c;case 4:return e.d;case 3:return e.a;default:return!1}}function qht(e,t){switch(t.g){case 2:return e.b;case 1:return e.c;case 4:return e.d;case 3:return e.a;default:return!1}}function _pe(e,t,n,s){switch(t){case 3:return e.f;case 4:return e.g;case 5:return e.i;case 6:return e.j}return gpe(e,t,n,s)}function Pkn(e){return e.k!=(bn(),Wi)?!1:yk(new Vt(null,new cb(new Sn(Dn(qi(e).a.Kc(),new L)))),new Ije)}function Bkn(e){return e.e==null?e:(!e.c&&(e.c=new OQ((e.f&256)!=0,e.i,e.a,e.d,(e.f&16)!=0,e.j,e.g,null)),e.c)}function Fkn(e,t){return e.h==FM&&e.m==0&&e.l==0?(t&&(Rg=rc(0,0,0)),UJe((Ck(),m3e))):(t&&(Rg=rc(e.l,e.m,e.h)),rc(0,0,0))}function Ga(e){var t;return Array.isArray(e)&&e.im===de?sg(uu(e))+"@"+(t=ii(e)>>>0,t.toString(16)):e.toString()}function o9(e){var t;this.a=(t=h(e.e&&e.e(),9),new ll(t,h(vh(t,t.length),9),0)),this.b=pe(yn,st,1,this.a.a.length,5,1)}function Rkn(e){var t,n,s;for(this.a=new md,s=new Q(e);s.a<s.c.c.length;)n=h(K(s),14),t=new Znt,Y4n(t,n),Qi(this.a,t)}function jkn(e){rw();var t,n,s,a;for(t=e.o.b,s=h(h(jr(e.r,(Qe(),Fn)),21),84).Kc();s.Ob();)n=h(s.Pb(),111),a=n.e,a.b+=t}function lu(e){var t;if(e.b){if(lu(e.b),e.b.d!=e.c)throw ne(new cl)}else e.d.dc()&&(t=h(e.f.c.xc(e.e),14),t&&(e.d=t))}function $kn(e){var t;return e==null?!0:(t=e.length,t>0&&(er(t-1,e.length),e.charCodeAt(t-1)==58)&&!kX(e,lT,hT))}function kX(e,t,n){var s,a;for(s=0,a=e.length;s<a;s++)if(WK((er(s,e.length),e.charCodeAt(s)),t,n))return!0;return!1}function Hkn(e,t){var n,s;for(s=e.e.a.ec().Kc();s.Ob();)if(n=h(s.Pb(),266),U9n(t,n.d)||Z_n(t,n.d))return!0;return!1}function zkn(e,t){var n,s,a;for(s=SMn(e,t),a=s[s.length-1]/2,n=0;n<s.length;n++)if(s[n]>=a)return t.c+n;return t.c+t.b.gc()}function Gkn(e,t){ik();var n,s,a,u;for(s=act(e),a=t,_k(s,0,s.length,a),n=0;n<s.length;n++)u=T7n(e,s[n],n),n!=u&&a9(e,n,u)}function Tpe(e,t){var n,s,a,u,p,v;for(s=0,n=0,u=t,p=0,v=u.length;p<v;++p)a=u[p],a>0&&(s+=a,++n);return n>1&&(s+=e.d*(n-1)),s}function Cpe(e){var t,n,s;for(s=new s0,s.a+="[",t=0,n=e.gc();t<n;)va(s,mE(e.ki(t))),++t<n&&(s.a+=Js);return s.a+="]",s.a}function Vkn(e){var t,n,s,a,u;return u=G2e(e),n=wS(e.c),s=!n,s&&(a=new yp,Wh(u,"knownLayouters",a),t=new uKe(a),Os(e.c,t)),u}function Ukn(e,t){var n,s,a;for(rn(t),n=!1,s=new Q(e);s.a<s.c.c.length;)a=K(s),Ew(t,a,!1)&&(IE(s),n=!0);return n}function Spe(e){var t,n,s;for(s=Pe(Je(e.a.We((Tr(),PH)))),n=new Q(e.a.xf());n.a<n.c.c.length;)t=h(K(n),680),vbt(e,t,s)}function xX(e,t){var n,s;for(s=new Q(t);s.a<s.c.c.length;)n=h(K(s),46),Ue(e.b.b,h(n.b,81)),tK(h(n.a,189),h(n.b,81))}function qkn(e,t,n){var s,a;for(a=e.a.b,s=a.c.length;s<n;s++)aw(a,0,new Nl(e.a));Ca(t,h(ut(a,a.c.length-n),29)),e.b[t.p]=n}function Ykn(e,t,n){var s;s=n,!s&&(s=ede(new H6,0)),Bn(s,Nvt,2),Yft(e.b,t,jo(s,1)),AIn(e,t,jo(s,1)),yNn(t,jo(s,1)),An(s)}function Wkn(e,t,n,s,a){Hu(),Th(ph(gh(dh(bh(new Il,0),a.d.e-e),t),a.d)),Th(ph(gh(dh(bh(new Il,0),n-a.a.e),a.a),s))}function Ape(e,t,n,s,a,u){this.a=e,this.c=t,this.b=n,this.f=s,this.d=a,this.e=u,this.c>0&&this.b>0&&ode(this.c,this.b,this.a)}function Mpe(e){EX(),this.c=Ef(se(ie(FPn,1),st,831,0,[Mxt])),this.b=new zn,this.a=e,Pr(this.b,eH,1),mc(Dxt,new OWe(this))}function Yht(e,t){var n;return e.d?Au(e.b,t)?h(_n(e.b,t),51):(n=t.Kf(),Pr(e.b,t,n),n):t.Kf()}function Dpe(e,t){var n;return Me(e)===Me(t)?!0:ue(t,91)?(n=h(t,91),e.e==n.e&&e.d==n.d&&Myn(e,n.a)):!1}function i5(e){switch(Qe(),e.g){case 4:return sn;case 1:return pn;case 3:return Fn;case 2:return un;default:return to}}function Lpe(e,t){switch(t){case 3:return e.f!=0;case 4:return e.g!=0;case 5:return e.i!=0;case 6:return e.j!=0}return Dge(e,t)}function Kkn(e){switch(e.g){case 0:return new CHe;case 1:return new SHe;default:throw ne(new an(JJ+(e.f!=null?e.f:""+e.g)))}}function Wht(e){switch(e.g){case 0:return new THe;case 1:return new AHe;default:throw ne(new an(pJ+(e.f!=null?e.f:""+e.g)))}}function Kht(e){switch(e.g){case 0:return new uhe;case 1:return new iXe;default:throw ne(new an(cj+(e.f!=null?e.f:""+e.g)))}}function Xkn(e){switch(e.g){case 1:return new mHe;case 2:return new ftt;default:throw ne(new an(JJ+(e.f!=null?e.f:""+e.g)))}}function Qkn(e){var t,n;if(e.b)return e.b;for(n=Pf?null:e.d;n;){if(t=Pf?null:n.b,t)return t;n=Pf?null:n.d}return Q6(),K3e}function Zkn(e){var t,n,s;return e.e==0?0:(t=e.d<<5,n=e.a[e.d-1],e.e<0&&(s=slt(e),s==e.d-1&&(--n,n=n|0)),t-=bM(n),t)}function Jkn(e){var t,n,s;return e<Rj.length?Rj[e]:(n=e>>5,t=e&31,s=pe(Hn,hr,25,n+1,15,1),s[n]=1<<t,new fy(1,n+1,s))}function Xht(e){var t,n,s;return n=e.zg(),n?(t=e.Ug(),ue(t,160)&&(s=Xht(h(t,160)),s!=null)?s+"."+n:n):null}function Ew(e,t,n){var s,a;for(a=e.Kc();a.Ob();)if(s=a.Pb(),Me(t)===Me(s)||t!=null&&Nr(t,s))return n&&a.Qb(),!0;return!1}function Ipe(e,t,n){var s,a;if(++e.j,n.dc())return!1;for(a=n.Kc();a.Ob();)s=a.Pb(),e.Hi(t,e.oi(t,s)),++t;return!0}function exn(e,t,n,s){var a,u;if(u=n-t,u<3)for(;u<3;)e*=10,++u;else{for(a=1;u>3;)a*=10,--u;e=(e+(a>>1))/a|0}return s.i=e,!0}function txn(e){return uX(),on(),!!(qht(h(e.a,81).j,h(e.b,103))||h(e.a,81).d.e!=0&&qht(h(e.a,81).j,h(e.b,103)))}function nxn(e){CB(),h(e.We((Tr(),Wg)),174).Hc((fu(),$H))&&(h(e.We(gm),174).Fc((Qc(),K5)),h(e.We(Wg),174).Mc($H))}function Qht(e,t){var n,s;if(t){for(n=0;n<e.i;++n)if(s=h(e.g[n],366),s.Di(t))return!1;return qn(e,t)}else return!1}function Ope(e){var t,n,s,a;for(t=new yp,a=new F6(e.b.Kc());a.b.Ob();)s=h(a.b.Pb(),686),n=w_n(s),jbn(t,t.a.length,n);return t.a}function Npe(e){var t;return!e.c&&(e.c=new No),hs(e.d,new Yc),iDn(e),t=qAn(e),Ti(new Vt(null,new Wt(e.d,16)),new Wqe(e)),t}function yF(e){var t;return(e.Db&64)!=0?n9(e):(t=new Ol(n9(e)),t.a+=" (instanceClassName: ",va(t,e.D),t.a+=")",t.a)}function rxn(e,t){var n,s,a,u;t&&(a=Md(t,"x"),n=new iKe(e),Ak(n.a,(rn(a),a)),u=Md(t,"y"),s=new sKe(e),Dk(s.a,(rn(u),u)))}function ixn(e,t){var n,s,a,u;t&&(a=Md(t,"x"),n=new eKe(e),Mk(n.a,(rn(a),a)),u=Md(t,"y"),s=new nKe(e),Lk(s.a,(rn(u),u)))}function oi(e,t){var n,s,a;if(n=(e.i==null&&s1(e),e.i),s=t.aj(),s!=-1){for(a=n.length;s<a;++s)if(n[s]==t)return s}return-1}function sxn(e){var t,n,s,a,u;for(n=h(e.g,674),s=e.i-1;s>=0;--s)for(t=n[s],a=0;a<s;++a)if(u=n[a],Kpt(e,t,u)){c5(e,s);break}}function axn(e){var t=e.e;function n(s){return!s||s.length==0?"":"	"+s.join(`
	`)}return t&&(t.stack||n(e[vZ]))}function Ppe(e){lw();var t;switch(t=e.Pc(),t.length){case 0:return Kee;case 1:return new sW(Vn(t[0]));default:return new jW(Akn(t))}}function Db(e,t){switch(t.g){case 1:return R4(e.j,(Rc(),Z4e));case 2:return R4(e.j,(Rc(),e5e));default:return Rt(),Rt(),ca}}function Bpe(e,t){switch(t){case 3:xb(e,0);return;case 4:Eb(e,0);return;case 5:yc(e,0);return;case 6:kc(e,0);return}lpe(e,t)}function EX(){EX=re,TN(),eH=(tt(),uv),Dxt=Ef(se(ie(Mre,1),eye,146,0,[ND,Mh,z5,cv,lm,Ine,h8,f8,One,I_,p3,b2,b3]))}function Zht(e){var t,n;t=e.d==(u5(),Zx),n=P2e(e),t&&!n||!t&&n?Fe(e.a,(tt(),m1),($1(),YD)):Fe(e.a,(tt(),m1),($1(),qD))}function oxn(e,t){var n;return n=h(Gu(e,Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),n.Qc(xnt(n.gc()))}function kF(){kF=re,Xre=new VN("SIMPLE",0),X7e=new VN("GROUP_DEC",1),Z7e=new VN("GROUP_MIXED",2),Q7e=new VN("GROUP_INC",3)}function _X(){_X=re,aie=new qze,kEe=new Yze,xEe=new Wze,EEe=new Kze,_Ee=new Xze,TEe=new Qze,CEe=new Zze,SEe=new Jze,AEe=new eGe}function Jht(e,t,n){out(),QKe.call(this),this.a=Ap(H3t,[Re,dwe],[595,212],0,[Vj,fte],2),this.c=new O4,this.g=e,this.f=t,this.d=n}function Fpe(e,t){this.n=Ap(ap,[Re,RZ],[364,25],14,[t,Li(d.Math.ceil(e/32))],2),this.o=e,this.p=t,this.j=e-1>>1,this.k=t-1>>1}function cxn(e,t){Bn(t,"End label post-processing",1),Ti(Jr(Xa(new Vt(null,new Wt(e.b,16)),new XBe),new QBe),new ZBe),An(t)}function uxn(e,t,n){var s,a;return s=Pe(e.p[t.i.p])+Pe(e.d[t.i.p])+t.n.b+t.a.b,a=Pe(e.p[n.i.p])+Pe(e.d[n.i.p])+n.n.b+n.a.b,a-s}function lxn(e,t,n){var s,a;for(s=Zi(n,fa),a=0;mo(s,0)!=0&&a<t;a++)s=Vs(s,Zi(e[a],fa)),e[a]=Gn(s),s=dg(s,32);return Gn(s)}function xF(e){var t,n,s,a;for(a=0,n=0,s=e.length;n<s;n++)t=(er(n,e.length),e.charCodeAt(n)),t<64&&(a=xf(a,kd(1,t)));return a}function hxn(e){var t;return e==null?null:new hg((t=Ho(e,!0),t.length>0&&(er(0,t.length),t.charCodeAt(0)==43)?t.substr(1):t))}function fxn(e){var t;return e==null?null:new hg((t=Ho(e,!0),t.length>0&&(er(0,t.length),t.charCodeAt(0)==43)?t.substr(1):t))}function Rpe(e,t){var n;return e.i>0&&(t.length<e.i&&(n=LA(uu(t).c,e.i),t=n),Ao(e.g,0,t,0,e.i)),t.length>e.i&&pi(t,e.i,null),t}function Xo(e,t,n){var s,a,u;return e.ej()?(s=e.i,u=e.fj(),YA(e,s,t),a=e.Zi(3,null,t,s,u),n?n.Ei(a):n=a):YA(e,e.i,t),n}function dxn(e,t,n){var s,a;return s=new Sd(e.e,4,10,(a=t.c,ue(a,88)?h(a,26):(Bt(),nh)),null,y0(e,t),!1),n?n.Ei(s):n=s,n}function gxn(e,t,n){var s,a;return s=new Sd(e.e,3,10,null,(a=t.c,ue(a,88)?h(a,26):(Bt(),nh)),y0(e,t),!1),n?n.Ei(s):n=s,n}function eft(e){rw();var t;return t=new _a(h(e.e.We((Tr(),_3)),8)),e.B.Hc((fu(),x8))&&(t.a<=0&&(t.a=20),t.b<=0&&(t.b=20)),t}function tft(e){Tw();var t;return(e.q?e.q:(Rt(),Rt(),K1))._b((tt(),av))?t=h(U(e,av),197):t=h(U(Us(e),L_),197),t}function _w(e,t){var n,s;return s=null,os(e,(tt(),Z$))&&(n=h(U(e,Z$),94),n.Xe(t)&&(s=n.We(t))),s==null&&(s=U(Us(e),t)),s}function nft(e,t){var n,s,a;return ue(t,42)?(n=h(t,42),s=n.cd(),a=xw(e.Rc(),s),n1(a,n.dd())&&(a!=null||e.Rc()._b(s))):!1}function TX(e,t){var n,s,a;return e.f>0?(e.qj(),s=t==null?0:ii(t),a=(s&Ir)%e.d.length,n=ngt(e,a,s,t),n!=-1):!1}function Xh(e,t){var n,s,a;return e.f>0&&(e.qj(),s=t==null?0:ii(t),a=(s&Ir)%e.d.length,n=xbe(e,a,s,t),n)?n.dd():null}function QA(e,t){var n,s,a,u;for(u=ac(e.e.Tg(),t),n=h(e.g,119),a=0;a<e.i;++a)if(s=n[a],u.rl(s.ak()))return!1;return!0}function rft(e){if(e.b==null){for(;e.a.Ob();)if(e.b=e.a.Pb(),!h(e.b,49).Zg())return!0;return e.b=null,!1}else return!0}function ift(e,t){e.mj();try{e.d.Vc(e.e++,t),e.f=e.d.j,e.g=-1}catch(n){throw n=ui(n),ue(n,73)?ne(new cl):ne(n)}}function EF(e,t){Cfe();var n,s;return n=YS((uN(),uN(),h_)),s=null,t==n&&(s=h(Fo(v3e,e),615)),s||(s=new zrt(e),t==n&&Ta(v3e,e,s)),s}function sft(e,t){var n,s;e.a=Vs(e.a,1),e.c=d.Math.min(e.c,t),e.b=d.Math.max(e.b,t),e.d+=t,n=t-e.f,s=e.e+n,e.f=s-e.e-n,e.e=s}function pxn(e,t){var n;e.c=t,e.a=Zkn(t),e.a<54&&(e.f=(n=t.d>1?xf(kd(t.a[1],32),Zi(t.a[0],fa)):Zi(t.a[0],fa),vb(js(t.e,n))))}function ZA(e,t){var n;return $a(e)&&$a(t)&&(n=e%t,RM<n&&n<Jp)?n:xg((Ive($a(e)?Bl(e):e,$a(t)?Bl(t):t,!0),Rg))}function c9(e,t){var n;POn(t),n=h(U(e,(tt(),U$)),276),n&&Fe(e,U$,g7n(n)),c0(e.c),c0(e.f),j0e(e.d),j0e(h(U(e,K$),207))}function aft(e){this.e=pe(Hn,hr,25,e.length,15,1),this.c=pe(vu,Y1,25,e.length,16,1),this.b=pe(vu,Y1,25,e.length,16,1),this.f=0}function bxn(e){var t,n;for(e.j=pe(ks,ka,25,e.p.c.length,15,1),n=new Q(e.p);n.a<n.c.c.length;)t=h(K(n),10),e.j[t.p]=t.o.b/e.i}function CX(e){var t;e.c!=0&&(t=h(ut(e.a,e.b),287),t.b==1?(++e.b,e.b<e.a.c.length&&yqe(h(ut(e.a,e.b),287))):--t.b,--e.c)}function vxn(e){var t;t=e.a;do t=h(Un(new Sn(Dn(qi(t).a.Kc(),new L))),17).d.i,t.k==(bn(),ds)&&Ue(e.e,t);while(t.k==(bn(),ds))}function jpe(){jpe=re,W7e=new nb(15),c9t=new sa((Tr(),y2),W7e),l9t=new sa(k2,15),u9t=new sa(Vre,Ke(0)),o9t=new sa(U5,Mx)}function Iu(){Iu=re,cL=new GN("PORTS",0),E2=new GN("PORT_LABELS",1),oL=new GN("NODE_LABELS",2),bm=new GN("MINIMUM_SIZE",3)}function JA(e,t){var n,s;for(s=t.length,n=0;n<s;n+=2)$o(e,(er(n,t.length),t.charCodeAt(n)),(er(n+1,t.length),t.charCodeAt(n+1)))}function oft(e,t,n){var s,a,u,p;for(u=t-e.e,p=n-e.f,a=new Q(e.a);a.a<a.c.c.length;)s=h(K(a),187),tM(s,s.s+u,s.t+p);e.e=t,e.f=n}function wxn(e,t){var n,s,a,u;for(u=t.b.b,e.a=new di,e.b=pe(Hn,hr,25,u,15,1),n=0,a=pr(t.b,0);a.b!=a.d.c;)s=h(gr(a),86),s.g=n++}function cft(e,t){var n,s,a,u;return n=t>>5,t&=31,a=e.d+n+(t==0?0:1),s=pe(Hn,hr,25,a,15,1),p9n(s,e.a,n,t),u=new fy(e.e,a,s),jE(u),u}function $pe(e,t,n){var s,a;s=h(Fo(T8,t),117),a=h(Fo(vT,t),117),n?(Ta(T8,e,s),Ta(vT,e,a)):(Ta(vT,e,s),Ta(T8,e,a))}function uft(e,t,n){var s,a,u;for(a=null,u=e.b;u;){if(s=e.a.ue(t,u.d),n&&s==0)return u;s>=0?u=u.a[1]:(a=u,u=u.a[0])}return a}function lft(e,t,n){var s,a,u;for(a=null,u=e.b;u;){if(s=e.a.ue(t,u.d),n&&s==0)return u;s<=0?u=u.a[0]:(a=u,u=u.a[1])}return a}function mxn(e,t,n,s){var a,u,p;return a=!1,eOn(e.f,n,s)&&(Vxn(e.f,e.a[t][n],e.a[t][s]),u=e.a[t],p=u[s],u[s]=u[n],u[n]=p,a=!0),a}function Hpe(e,t,n,s,a){var u,p,v;for(p=a;t.b!=t.c;)u=h(H4(t),10),v=h(Za(u,s).Xb(0),11),e.d[v.p]=p++,n.c[n.c.length]=v;return p}function zpe(e,t,n){var s,a,u,p,v;return p=e.k,v=t.k,s=n[p.g][v.g],a=Je(_w(e,s)),u=Je(_w(t,s)),d.Math.max((rn(a),a),(rn(u),u))}function yxn(e,t,n){var s,a,u,p;for(s=n/e.c.length,a=0,p=new Q(e);p.a<p.c.c.length;)u=h(K(p),200),pht(u,u.f+s*a),J7n(u,t,s),++a}function hft(e,t,n){var s,a,u,p;for(a=h(_n(e.b,n),177),s=0,p=new Q(t.j);p.a<p.c.c.length;)u=h(K(p),113),a[u.d.p]&&++s;return s}function fft(e){var t,n;return t=h(tn(e.a,4),126),t!=null?(n=pe(nie,Lee,415,t.length,0,1),Ao(t,0,n,0,t.length),n):D9t}function kxn(){var e;return Nj!=0&&(e=Fyn(),e-c3t>2e3&&(c3t=e,Pj=d.setTimeout(S1n,10))),Nj++==0?(t4n((ghe(),b3e)),!0):!1}function xxn(e,t){var n,s,a;for(s=new Sn(Dn(qi(e).a.Kc(),new L));rr(s);)if(n=h(Un(s),17),a=n.d.i,a.c==t)return!1;return!0}function Gpe(e,t){var n,s;if(ue(t,245)){s=h(t,245);try{return n=e.vd(s),n==0}catch(a){if(a=ui(a),!ue(a,205))throw ne(a)}}return!1}function Exn(){return Error.stackTraceLimit>0?(d.Error.stackTraceLimit=Error.stackTraceLimit=64,!0):"stack"in new Error}function _xn(e,t){return vf(),vf(),xh(Ig),(d.Math.abs(e-t)<=Ig||e==t||isNaN(e)&&isNaN(t)?0:e<t?-1:e>t?1:tb(isNaN(e),isNaN(t)))>0}function Vpe(e,t){return vf(),vf(),xh(Ig),(d.Math.abs(e-t)<=Ig||e==t||isNaN(e)&&isNaN(t)?0:e<t?-1:e>t?1:tb(isNaN(e),isNaN(t)))<0}function dft(e,t){return vf(),vf(),xh(Ig),(d.Math.abs(e-t)<=Ig||e==t||isNaN(e)&&isNaN(t)?0:e<t?-1:e>t?1:tb(isNaN(e),isNaN(t)))<=0}function SX(e,t){for(var n=0;!t[n]||t[n]=="";)n++;for(var s=t[n++];n<t.length;n++)!t[n]||t[n]==""||(s+=e+t[n]);return s}function Fl(e,t,n){var s,a,u,p;for(u=t+n,H0e(t,u,e.length),p="",a=t;a<u;)s=d.Math.min(a+1e4,u),p+=dvn(e.slice(a,s)),a=s;return p}function gft(e){var t,n,s,a,u;if(e==null)return null;for(u=new qe,n=eX(e),s=0,a=n.length;s<a;++s)t=n[s],Ue(u,Ho(t,!0));return u}function pft(e){var t,n,s,a,u;if(e==null)return null;for(u=new qe,n=eX(e),s=0,a=n.length;s<a;++s)t=n[s],Ue(u,Ho(t,!0));return u}function bft(e){var t,n,s,a,u;if(e==null)return null;for(u=new qe,n=eX(e),s=0,a=n.length;s<a;++s)t=n[s],Ue(u,Ho(t,!0));return u}function vft(e,t){var n,s,a;if(e.c)Eb(e.c,t);else for(n=t-Fc(e),a=new Q(e.d);a.a<a.c.c.length;)s=h(K(a),157),vft(s,Fc(s)+n)}function wft(e,t){var n,s,a;if(e.c)xb(e.c,t);else for(n=t-Su(e),a=new Q(e.a);a.a<a.c.c.length;)s=h(K(a),157),wft(s,Su(s)+n)}function Txn(e,t){var n,s,a,u;for(a=new Wo(t.gc()),s=t.Kc();s.Ob();)n=s.Pb(),u=HQ(e,h(n,56)),u&&(a.c[a.c.length]=u);return a}function _F(e,t){var n,s,a;return e.qj(),s=t==null?0:ii(t),a=(s&Ir)%e.d.length,n=xbe(e,a,s,t),n?(Sut(e,n),n.dd()):null}function Lb(e){var t,n;for(n=Ddt(e),t=null;e.c==2;)Sr(e),t||(t=(Mr(),Mr(),new bE(2)),Qp(t,n),n=t),n.$l(Ddt(e));return n}function Jk(e){var t,n,s;if(s=null,t=p1 in e.a,n=!t,n)throw ne(new Xf("Every element must have an id."));return s=l5(Ed(e,p1)),s}function TF(e){var t,n,s;if(s=e.Zg(),!s)for(t=0,n=e.eh();n;n=n.eh()){if(++t>jZ)return n.fh();if(s=n.Zg(),!!s||n==e)break}return s}function Upe(e){return oB(),ue(e,156)?h(_n(gL,y3t),288).vg(e):Au(gL,uu(e))?h(_n(gL,uu(e)),288).vg(e):null}function Cxn(e){if(CF(Nx,e))return on(),Hx;if(CF(hee,e))return on(),u2;throw ne(new an("Expecting true or false"))}function Sxn(e,t){if(t.c==e)return t.d;if(t.d==e)return t.c;throw ne(new an("Input edge is not connected to the input port."))}function mft(e,t){return e.e>t.e?1:e.e<t.e?-1:e.d>t.d?e.e:e.d<t.d?-t.e:e.e*epe(e.a,t.a,e.d)}function yft(e){return e>=48&&e<48+d.Math.min(10,10)?e-48:e>=97&&e<97?e-97+10:e>=65&&e<65?e-65+10:-1}function kft(e,t){var n;return Me(t)===Me(e)?!0:!ue(t,21)||(n=h(t,21),n.gc()!=e.gc())?!1:e.Ic(n)}function Axn(e,t){var n,s,a,u;return s=e.a.length-1,n=t-e.b&s,u=e.c-t&s,a=e.c-e.b&s,Qet(n<a),n>=u?(S6n(e,t),-1):(A6n(e,t),1)}function Mxn(e,t){var n,s;for(n=(er(t,e.length),e.charCodeAt(t)),s=t+1;s<e.length&&(er(s,e.length),e.charCodeAt(s)==n);)++s;return s-t}function qpe(e){switch(e.g){case 12:case 13:case 14:case 15:case 16:case 17:case 18:case 19:case 20:return!0;default:return!1}}function Dxn(e,t){var n=e.a,s;t=String(t),n.hasOwnProperty(t)&&(s=n[t]);var a=(YK(),tte)[typeof s],u=a?a(s):ipe(typeof s);return u}function Ib(e,t){if(e.a<0)throw ne(new ja("Did not call before(...) or after(...) before calling add(...)."));return Xfe(e,e.a,t),e}function Lxn(e,t,n,s){var a,u;t.c.length!=0&&(a=pSn(n,s),u=gTn(t),Ti(FB(new Vt(null,new Wt(u,1)),new iHe),new dit(e,n,a,s)))}function s5(e,t,n){var s;(e.Db&t)!=0?n==null?eCn(e,t):(s=WX(e,t),s==-1?e.Eb=n:pi(Fp(e.Eb),s,n)):n!=null&&TAn(e,t,n)}function ic(e){var t,n;return(e.Db&32)==0&&(n=(t=h(tn(e,16),26),En(t||e.zh())-En(e.zh())),n!=0&&s5(e,32,pe(yn,st,1,n,5,1))),e}function Ixn(e){var t;return e.b||x1n(e,(t=fgn(e.e,e.a),!t||!Nt(hee,Xh((!t.b&&(t.b=new Cu((Bt(),na),uo,t)),t.b),"qualified")))),e.c}function Oxn(e,t,n){var s,a,u;return s=h(he(cu(e.a),t),87),u=(a=s.c,a||(Bt(),zf)),(u.kh()?Tg(e.b,h(u,49)):u)==n?hR(s):Sk(s,n),u}function Nxn(e,t){(!t&&console.groupCollapsed!=null?console.groupCollapsed:console.group!=null?console.group:console.log).call(console,e)}function Pxn(e,t,n,s){s==e,h(n.b,65),h(n.b,65),h(s.b,65),h(s.b,65).c.b,$0e(s,t,e)}function Bxn(e){var t,n;for(t=new Q(e.g);t.a<t.c.c.length;)h(K(t),562);n=new Cgt(e.g,Pe(e.a),e.c),GOn(n),e.g=n.b,e.d=n.a}function Ype(e,t,n){t.b=d.Math.max(t.b,-n.a),t.c=d.Math.max(t.c,n.a-e.a),t.d=d.Math.max(t.d,-n.b),t.a=d.Math.max(t.a,n.b-e.b)}function Fxn(e,t){return e.e<t.e?-1:e.e>t.e?1:e.f<t.f?-1:e.f>t.f?1:ii(e)-ii(t)}function CF(e,t){return rn(e),t==null?!1:Nt(e,t)?!0:e.length==t.length&&Nt(e.toLowerCase(),t.toLowerCase())}function Rxn(e,t){var n,s,a,u;for(s=0,a=t.gc();s<a;++s)n=t.il(s),ue(n,99)&&(h(n,18).Bb&go)!=0&&(u=t.jl(s),u!=null&&HQ(e,h(u,56)))}function xft(e,t,n){var s,a,u;for(u=new Q(n.a);u.a<u.c.c.length;)a=h(K(u),221),s=new tP(h(_n(e.a,a.b),65)),Ue(t.a,s),xft(e,s,a)}function qp(e){var t,n;return mo(e,-129)>0&&mo(e,128)<0?(t=Gn(e)+128,n=($nt(),C3e)[t],!n&&(n=C3e[t]=new mle(e)),n):new mle(e)}function Eft(e,t){var n,s;return n=t.Hh(e.a),n&&(s=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),_r)),s!=null)?s:t.ne()}function jxn(e,t){var n,s;return n=t.Hh(e.a),n&&(s=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),_r)),s!=null)?s:t.ne()}function $xn(e,t){YW();var n,s;for(s=new Sn(Dn(Id(e).a.Kc(),new L));rr(s);)if(n=h(Un(s),17),n.d.i==t||n.c.i==t)return n;return null}function Wpe(e,t,n){this.c=e,this.f=new qe,this.e=new Rs,this.j=new rde,this.n=new rde,this.b=t,this.g=new hl(t.c,t.d,t.b,t.a),this.a=n}function AX(e){var t,n,s,a;for(this.a=new md,this.d=new ns,this.e=0,n=e,s=0,a=n.length;s<a;++s)t=n[s],!this.f&&(this.f=t),tK(this,t)}function _ft(e){Dg(),e.length==0?(this.e=0,this.d=1,this.a=se(ie(Hn,1),hr,25,15,[0])):(this.e=1,this.d=e.length,this.a=e,jE(this))}function u9(e,t,n){QKe.call(this),this.a=pe(H3t,dwe,212,(Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])).length,0,1),this.b=e,this.d=t,this.c=n}function Tft(e){this.d=new qe,this.e=new Np,this.c=pe(Hn,hr,25,(Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])).length,15,1),this.b=e}function Hxn(e){var t,n,s,a,u,p;for(p=h(U(e,(He(),Br)),11),Fe(p,a8,e.i.n.b),t=i1(e.e),s=t,a=0,u=s.length;a<u;++a)n=s[a],xs(n,p)}function zxn(e){var t,n,s,a,u,p;for(n=h(U(e,(He(),Br)),11),Fe(n,a8,e.i.n.b),t=i1(e.g),a=t,u=0,p=a.length;u<p;++u)s=a[u],Gs(s,n)}function Gxn(e){var t,n;return os(e.d.i,(tt(),c8))?(t=h(U(e.c.i,c8),19),n=h(U(e.d.i,c8),19),dc(t.a,n.a)>0):!1}function Cft(e){var t;Me(pt(e,(Tr(),k3)))===Me((Ld(),BH))&&(bi(e)?(t=h(pt(bi(e),k3),334),ya(e,k3,t)):ya(e,k3,tT))}function Vxn(e,t,n){var s,a;uQ(e.e,t,n,(Qe(),un)),uQ(e.i,t,n,pn),e.a&&(a=h(U(t,(He(),Br)),11),s=h(U(n,Br),11),JW(e.g,a,s))}function Sft(e,t,n){var s,a,u;s=t.c.p,u=t.p,e.b[s][u]=new Vit(e,t),n&&(e.a[s][u]=new KYe(t),a=h(U(t,(He(),rv)),10),a&&Ot(e.d,a,t))}function Aft(e,t){var n,s,a;if(Ue(Kj,e),t.Fc(e),n=h(_n(yte,e),21),n)for(a=n.Kc();a.Ob();)s=h(a.Pb(),33),Ha(Kj,s,0)!=-1||Aft(s,t)}function Uxn(e,t,n){var s;(S3t?(Qkn(e),!0):A3t||D3t?(Q6(),!0):M3t&&(Q6(),!1))&&(s=new nnt(t),s.b=n,W9n(e,s))}function MX(e,t){var n;n=!e.A.Hc((Iu(),E2))||e.q==(_s(),qo),e.u.Hc((Qc(),Ud))?n?dNn(e,t):ibt(e,t):e.u.Hc(Zg)&&(n?IOn(e,t):wbt(e,t))}function ex(e,t){var n,s;if(++e.j,t!=null&&(n=(s=e.a.Cb,ue(s,97)?h(s,97).Jg():null),$Tn(t,n))){s5(e.a,4,n);return}s5(e.a,4,h(t,126))}function Mft(e,t,n){return new hl(d.Math.min(e.a,t.a)-n/2,d.Math.min(e.b,t.b)-n/2,d.Math.abs(e.a-t.a)+n,d.Math.abs(e.b-t.b)+n)}function qxn(e,t){var n,s;return n=dc(e.a.c.p,t.a.c.p),n!=0?n:(s=dc(e.a.d.i.p,t.a.d.i.p),s!=0?s:dc(t.a.d.p,e.a.d.p))}function Yxn(e,t,n){var s,a,u,p;return u=t.j,p=n.j,u!=p?u.g-p.g:(s=e.f[t.p],a=e.f[n.p],s==0&&a==0?0:s==0?-1:a==0?1:Ui(s,a))}function Dft(e,t,n){var s,a,u;if(!n[t.d])for(n[t.d]=!0,a=new Q(Ty(t));a.a<a.c.c.length;)s=h(K(a),213),u=mF(s,t),Dft(e,u,n)}function Kpe(e,t,n){var s;switch(s=n[e.g][t],e.g){case 1:case 3:return new dt(0,s);case 2:case 4:return new dt(s,0);default:return null}}function Wxn(e,t,n){var s,a;a=h(zW(t.f),209);try{a.Ze(e,n),rvn(t.f,a)}catch(u){throw u=ui(u),ue(u,102)?(s=u,ne(s)):ne(u)}}function Lft(e,t,n){var s,a,u,p,v,m;return s=null,v=Pve(Ok(),t),u=null,v&&(a=null,m=Ove(v,n),p=null,m!=null&&(p=e.Ye(v,m)),a=p,u=a),s=u,s}function Kxn(e,t,n,s){var a,u,p;return a=new Sd(e.e,1,13,(p=t.c,p||(Bt(),zf)),(u=n.c,u||(Bt(),zf)),y0(e,t),!1),s?s.Ei(a):s=a,s}function DX(e,t,n,s){var a;if(a=e.length,t>=a)return a;for(t=t>0?t:0;t<a&&!WK((er(t,e.length),e.charCodeAt(t)),n,s);t++);return t}function Mf(e,t){var n,s;for(s=e.c.length,t.length<s&&(t=hN(new Array(s),t)),n=0;n<s;++n)pi(t,n,e.c[n]);return t.length>s&&pi(t,s,null),t}function Ift(e,t){var n,s;for(s=e.a.length,t.length<s&&(t=hN(new Array(s),t)),n=0;n<s;++n)pi(t,n,e.a[n]);return t.length>s&&pi(t,s,null),t}function Yp(e,t,n){var s,a,u;return a=h(_n(e.e,t),387),a?(u=w1e(a,n),xet(e,a),u):(s=new U1e(e,t,n),Pr(e.e,t,s),Kit(s),null)}function Xxn(e){var t;if(e==null)return null;if(t=VCn(Ho(e,!0)),t==null)throw ne(new Fq("Invalid hexBinary value: '"+e+"'"));return t}function eM(e){return Dg(),mo(e,0)<0?mo(e,-1)!=0?new O2e(-1,Tk(e)):ite:mo(e,10)<=0?R3e[Gn(e)]:new O2e(1,e)}function LX(){return xR(),se(ie(g4t,1),ze,159,0,[f4t,h4t,d4t,r4t,n4t,i4t,o4t,a4t,s4t,l4t,u4t,c4t,e4t,J3t,t4t,Q3t,X3t,Z3t,W3t,Y3t,K3t,dte])}function Oft(e){var t;this.d=new qe,this.j=new Rs,this.g=new Rs,t=e.g.b,this.f=h(U(Us(t),(tt(),Ju)),103),this.e=Pe(Je(MF(t,lm)))}function Nft(e){this.b=new qe,this.e=new qe,this.d=e,this.a=!U7(Jr(new Vt(null,new cb(new _f(e.b))),new I7(new Oje))).sd((Tp(),Ux))}function Qh(){Qh=re,gn=new NS("PARENTS",0),gs=new NS("NODES",1),k1=new NS("EDGES",2),w2=new NS("PORTS",3),N0=new NS("LABELS",4)}function Cy(){Cy=re,Xg=new BS("DISTRIBUTED",0),sL=new BS("JUSTIFIED",1),U7e=new BS("BEGIN",2),nT=new BS(Cx,3),q7e=new BS("END",4)}function Qxn(e){var t;switch(t=e.yi(null),t){case 10:return 0;case 15:return 1;case 14:return 2;case 11:return 3;case 21:return 4}return-1}function IX(e){switch(e.g){case 1:return la(),Gd;case 4:return la(),Wl;case 2:return la(),Dh;case 3:return la(),zd}return la(),Q1}function Zxn(e,t,n){var s;switch(s=n.q.getFullYear()-Og+Og,s<0&&(s=-s),t){case 1:e.a+=s;break;case 2:j1(e,s%100,2);break;default:j1(e,s,t)}}function pr(e,t){var n,s;if(fw(t,e.b),t>=e.b>>1)for(s=e.c,n=e.b;n>t;--n)s=s.b;else for(s=e.a.a,n=0;n<t;++n)s=s.a;return new ent(e,t,s)}function SF(){SF=re,ute=new Vhe("NUM_OF_EXTERNAL_SIDES_THAN_NUM_OF_EXTENSIONS_LAST",0),J3e=new Vhe("CORNER_CASES_THAN_SINGLE_SIDE_LAST",1)}function Jxn(e){var t,n,s,a;for(s=G_n(e),hs(s,M5t),a=e.d,a.c=pe(yn,st,1,0,5,1),n=new Q(s);n.a<n.c.c.length;)t=h(K(n),456),Vi(a,t.b)}function Pft(e){var t,n,s;for(s=(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),e.o),n=s.c.Kc();n.e!=n.i.gc();)t=h(n.nj(),42),t.dd();return wA(s)}function e8n(e){var t;!F4(h(U(e,(tt(),Ei)),98))||(t=e.b,y0t((Kt(0,t.c.length),h(t.c[0],29))),y0t(h(ut(t,t.c.length-1),29)))}function Bft(e,t){var n,s,a,u;for(n=0,a=new Q(t.a);a.a<a.c.c.length;)s=h(K(a),10),u=s.o.a+s.d.c+s.d.b+e.j,n=d.Math.max(n,u);return n}function AF(e){var t,n,s,a;for(a=0,n=0,s=e.length;n<s;n++)t=(er(n,e.length),e.charCodeAt(n)),t>=64&&t<128&&(a=xf(a,kd(1,t-64)));return a}function MF(e,t){var n,s;return s=null,os(e,(Tr(),Y5))&&(n=h(U(e,Y5),94),n.Xe(t)&&(s=n.We(t))),s==null&&!!Us(e)&&(s=U(Us(e),t)),s}function Fft(e,t){var n,s,a;a=t.d.i,s=a.k,!(s==(bn(),Wi)||s==Bf)&&(n=new Sn(Dn(qi(a).a.Kc(),new L)),rr(n)&&Pr(e.k,t,h(Un(n),17)))}function OX(e,t){var n,s,a;return s=Ht(e.Tg(),t),n=t-e.Ah(),n<0?(a=e.Yg(s),a>=0?e.lh(a):mQ(e,s)):n<0?mQ(e,s):h(s,66).Nj().Sj(e,e.yh(),n)}function at(e){var t;if(ue(e.a,4)){if(t=Upe(e.a),t==null)throw ne(new ja(Owt+e.b+"'. "+Iwt+(yd(pL),pL.k)+Cye));return t}else return e.a}function t8n(e){var t;if(e==null)return null;if(t=kNn(Ho(e,!0)),t==null)throw ne(new Fq("Invalid base64Binary value: '"+e+"'"));return t}function On(e){var t;try{return t=e.i.Xb(e.e),e.mj(),e.g=e.e++,t}catch(n){throw n=ui(n),ue(n,73)?(e.mj(),ne(new lo)):ne(n)}}function NX(e){var t;try{return t=e.c.ki(e.e),e.mj(),e.g=e.e++,t}catch(n){throw n=ui(n),ue(n,73)?(e.mj(),ne(new lo)):ne(n)}}function l9(){l9=re,A4e=(Tr(),_7e),wte=n7e,S4t=U5,S4e=y2,L4t=(HF(),a4e),D4t=i4e,I4t=c4e,M4t=r4e,A4t=(sX(),_4e),vte=_4t,C4e=T4t,Wj=C4t}function DF(e){switch(Nhe(),this.c=new qe,this.d=e,e.g){case 0:case 2:this.a=zde(q4e),this.b=ki;break;case 3:case 1:this.a=q4e,this.b=Fi}}function Rft(e,t,n){var s,a;if(e.c)yc(e.c,e.c.i+t),kc(e.c,e.c.j+n);else for(a=new Q(e.b);a.a<a.c.c.length;)s=h(K(a),157),Rft(s,t,n)}function n8n(e,t){var n,s;if(e.j.length!=t.j.length)return!1;for(n=0,s=e.j.length;n<s;n++)if(!Nt(e.j[n],t.j[n]))return!1;return!0}function LF(e,t,n){var s;t.a.length>0&&(Ue(e.b,new wnt(t.a,n)),s=t.a.length,0<s?t.a=t.a.substr(0,0):0>s&&(t.a+=ket(pe(Cl,o1,25,-s,15,1))))}function jft(e,t){var n,s,a;for(n=e.o,a=h(h(jr(e.r,t),21),84).Kc();a.Ob();)s=h(a.Pb(),111),s.e.a=i7n(s,n.a),s.e.b=n.b*Pe(Je(s.b.We(Uj)))}function r8n(e,t){var n,s,a,u;return a=e.k,n=Pe(Je(U(e,(He(),iv)))),u=t.k,s=Pe(Je(U(t,iv))),u!=(bn(),Pi)?-1:a!=Pi?1:n==s?0:n<s?-1:1}function i8n(e,t){var n,s;return n=h(h(_n(e.g,t.a),46).a,65),s=h(h(_n(e.g,t.b),46).a,65),yg(t.a,t.b)-yg(t.a,Mfe(n.b))-yg(t.b,Mfe(s.b))}function s8n(e,t){var n;return n=h(U(e,(tt(),Da)),74),LY(t,E5t)?n?Pl(n):(n=new Oc,Fe(e,Da,n)):n&&Fe(e,Da,null),n}function $ft(e){var t;return t=new rg,t.a+="n",e.k!=(bn(),Wi)&&or(or((t.a+="(",t),FY(e.k).toLowerCase()),")"),or((t.a+="_",t),iM(e)),t.a}function a8n(e,t){Bn(t,"Self-Loop post-processing",1),Ti(Jr(Jr(Xa(new Vt(null,new Wt(e.b,16)),new _Re),new TRe),new CRe),new SRe),An(t)}function a5(e,t,n,s){var a;return n>=0?e.hh(t,n,s):(e.eh()&&(s=(a=e.Vg(),a>=0?e.Qg(s):e.eh().ih(e,-1-a,null,s))),e.Sg(t,n,s))}function Xpe(e,t){switch(t){case 7:!e.e&&(e.e=new qt(us,e,7,4)),jn(e.e);return;case 8:!e.d&&(e.d=new qt(us,e,8,5)),jn(e.d);return}Bpe(e,t)}function Zh(e,t){var n;n=e.Zc(t);try{return n.Pb()}catch(s){throw s=ui(s),ue(s,109)?ne(new Ea("Can't get element "+t)):ne(s)}}function Qpe(e,t){this.e=e,t<$Z?(this.d=1,this.a=se(ie(Hn,1),hr,25,15,[t|0])):(this.d=2,this.a=se(ie(Hn,1),hr,25,15,[t%$Z|0,t/$Z|0]))}function Hft(e,t){Rt();var n,s,a,u;for(n=e,u=t,ue(e,21)&&!ue(t,21)&&(n=t,u=e),a=n.Kc();a.Ob();)if(s=a.Pb(),u.Hc(s))return!1;return!0}function qs(e,t,n){var s,a,u,p;return s=e.Xc(t),s!=-1&&(e.ej()?(u=e.fj(),p=c5(e,s),a=e.Zi(4,p,null,s,u),n?n.Ei(a):n=a):c5(e,s)),n}function o8n(e,t,n){var s,a,u,p;return s=e.Xc(t),s!=-1&&(e.ej()?(u=e.fj(),p=TP(e,s),a=e.Zi(4,p,null,s,u),n?n.Ei(a):n=a):TP(e,s)),n}function zft(e,t){var n;switch(n=h(wa(e.b,t),124).n,t.g){case 1:e.t>=0&&(n.d=e.t);break;case 3:e.t>=0&&(n.a=e.t)}e.C&&(n.b=e.C.b,n.c=e.C.c)}function o5(){o5=re,gD=new ON(LR,0),dD=new ON(JZ,1),pD=new ON(eJ,2),bD=new ON(tJ,3),gD.a=!1,dD.a=!0,pD.a=!1,bD.a=!0}function h9(){h9=re,vD=new IN(LR,0),qj=new IN(JZ,1),Yj=new IN(eJ,2),wD=new IN(tJ,3),vD.a=!1,qj.a=!0,Yj.a=!1,wD.a=!0}function c8n(e){var t;t=e.a;do t=h(Un(new Sn(Dn(za(t).a.Kc(),new L))),17).c.i,t.k==(bn(),ds)&&e.b.Fc(t);while(t.k==(bn(),ds));e.b=Rp(e.b)}function u8n(e){var t,n,s;for(s=e.c.a,e.p=(Vn(s),new Bc(s)),n=new Q(s);n.a<n.c.c.length;)t=h(K(n),10),t.p=q_n(t).a;Rt(),hs(e.p,new d$e)}function Gft(e){var t,n,s,a;if(s=0,a=Nw(e),a.c.length==0)return 1;for(n=new Q(a);n.a<n.c.c.length;)t=h(K(n),33),s+=Gft(t);return s}function l8n(e,t){var n,s,a;for(a=0,s=h(h(jr(e.r,t),21),84).Kc();s.Ob();)n=h(s.Pb(),111),a+=n.d.b+n.b.rf().a+n.d.c,s.Ob()&&(a+=e.w);return a}function h8n(e,t){var n,s,a;for(a=0,s=h(h(jr(e.r,t),21),84).Kc();s.Ob();)n=h(s.Pb(),111),a+=n.d.d+n.b.rf().b+n.d.a,s.Ob()&&(a+=e.w);return a}function f8n(e,t,n,s){if(t.a<s.a)return!0;if(t.a==s.a){if(t.b<s.b)return!0;if(t.b==s.b&&e.b>n.b)return!0}return!1}function PX(e,t){return ws(e)?!!t3t[t]:e.hm?!!e.hm[t]:ew(e)?!!e3t[t]:Jv(e)?!!Jyt[t]:!1}function ya(e,t,n){return n==null?(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),_F(e.o,t)):(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),zF(e.o,t,n)),e}function d8n(e,t,n,s){var a,u;u=t.Xe((Tr(),E3))?h(t.We(E3),21):e.j,a=nkn(u),a!=(xR(),dte)&&(n&&!qpe(a)||nbe(UCn(e,a,s),t))}function IF(e,t,n,s){var a,u,p;return u=Ht(e.Tg(),t),a=t-e.Ah(),a<0?(p=e.Yg(u),p>=0?e._g(p,n,!0):Bb(e,u,n)):h(u,66).Nj().Pj(e,e.yh(),a,n,s)}function g8n(e,t,n,s){var a,u,p;n.mh(t)&&(ia(),SK(t)?(a=h(n.ah(t),153),Rxn(e,a)):(u=(p=t,p?h(s,49).xh(p):null),u&&bfn(n.ah(t),u)))}function p8n(e){switch(e.g){case 1:return Cb(),fD;case 3:return Cb(),hD;case 2:return Cb(),pte;case 4:return Cb(),gte;default:return null}}function Zpe(e){switch(typeof e){case hZ:return k0(e);case Wve:return Li(e);case g5:return on(),e?1231:1237;default:return e==null?0:rb(e)}}function b8n(e,t,n){if(e.e)switch(e.b){case 1:Uvn(e.c,t,n);break;case 0:qvn(e.c,t,n)}else Sat(e.c,t,n);e.a[t.p][n.p]=e.c.i,e.a[n.p][t.p]=e.c.e}function Vft(e){var t,n;if(e==null)return null;for(n=pe(X1,Re,193,e.length,0,2),t=0;t<n.length;t++)n[t]=h(F3n(e[t],e[t].length),193);return n}function OF(e){var t;if(iX(e))return NP(e),e.Lk()&&(t=m9(e.e,e.b,e.c,e.a,e.j),e.j=t),e.g=e.a,++e.a,++e.c,e.i=0,e.j;throw ne(new lo)}function v8n(e,t){var n,s,a,u;return u=e.o,n=e.p,u<n?u*=u:n*=n,s=u+n,u=t.o,n=t.p,u<n?u*=u:n*=n,a=u+n,s<a?-1:s==a?0:1}function y0(e,t){var n,s,a;if(a=E1t(e,t),a>=0)return a;if(e.Fk()){for(s=0;s<e.i;++s)if(n=e.Gk(h(e.g[s],56)),Me(n)===Me(t))return s}return-1}function Sy(e,t,n){var s,a;if(a=e.gc(),t>=a)throw ne(new sw(t,a));if(e.hi()&&(s=e.Xc(n),s>=0&&s!=t))throw ne(new an(nD));return e.mi(t,n)}function Jpe(e,t){if(this.a=h(Vn(e),245),this.b=h(Vn(t),245),e.vd(t)>0||e==(Lq(),Yee)||t==(Dq(),Wee))throw ne(new an("Invalid range: "+Eat(e,t)))}function Uft(e){var t,n;for(this.b=new qe,this.c=e,this.a=!1,n=new Q(e.a);n.a<n.c.c.length;)t=h(K(n),10),this.a=this.a|t.k==(bn(),Wi)}function w8n(e,t){var n,s,a;for(n=ib(new _p,e),a=new Q(t);a.a<a.c.c.length;)s=h(K(a),121),Th(ph(gh(bh(dh(new Il,0),0),n),s));return n}function qft(e,t,n){var s,a,u;for(a=new Sn(Dn((t?za(e):qi(e)).a.Kc(),new L));rr(a);)s=h(Un(a),17),u=t?s.c.i:s.d.i,u.k==(bn(),Ou)&&Ca(u,n)}function Tw(){Tw=re,rH=new RN(jd,0),$ne=new RN("PORT_POSITION",1),w3=new RN("NODE_SIZE_WHERE_SPACE_PERMITS",2),v3=new RN("NODE_SIZE",3)}function $1(){$1=re,Dre=new oE("AUTOMATIC",0),qD=new oE(w5,1),YD=new oE(m5,2),MH=new oE("TOP",3),SH=new oE(pwe,4),AH=new oE(Cx,5)}function e2e(e,t,n,s){cx();var a,u;for(a=0,u=0;u<n;u++)a=Vs(js(Zi(t[u],fa),Zi(s,fa)),Zi(Gn(a),fa)),e[u]=Gn(a),a=ow(a,32);return Gn(a)}function t2e(e,t,n){var s,a;for(a=0,s=0;s<fte;s++)a=d.Math.max(a,cX(e.a[t.g][s],n));return t==(Kh(),Jo)&&!!e.b&&(a=d.Math.max(a,e.b.b)),a}function NF(e,t){var n,s;if(e1e(t>0),(t&-t)==t)return Li(t*hu(e,31)*4656612873077393e-25);do n=hu(e,31),s=n%t;while(n-s+(t-1)<0);return Li(s)}function k0(e){rnt();var t,n,s;return n=":"+e,s=zj[n],s!=null?Li((rn(s),s)):(s=Q3e[n],t=s==null?cAn(e):Li((rn(s),s)),Fvn(),zj[n]=t,t)}function Yft(e,t,n){Bn(n,"Compound graph preprocessor",1),e.a=new pb,J2t(e,t,null),KLn(e,t),LSn(e),Fe(t,(He(),b6e),e.a),e.a=null,Kc(e.b),An(n)}function m8n(e,t,n){switch(n.g){case 1:e.a=t.a/2,e.b=0;break;case 2:e.a=t.a,e.b=t.b/2;break;case 3:e.a=t.a/2,e.b=t.b;break;case 4:e.a=0,e.b=t.b/2}}function y8n(e){var t,n,s;for(s=h(jr(e.a,(Iy(),_$)),15).Kc();s.Ob();)n=h(s.Pb(),101),t=f2e(n),lk(e,n,t[0],(Tb(),Jb),0),lk(e,n,t[1],ev,1)}function k8n(e){var t,n,s;for(s=h(jr(e.a,(Iy(),T$)),15).Kc();s.Ob();)n=h(s.Pb(),101),t=f2e(n),lk(e,n,t[0],(Tb(),Jb),0),lk(e,n,t[1],ev,1)}function BX(e){switch(e.g){case 0:return null;case 1:return new Yut;case 2:return new lhe;default:throw ne(new an(JJ+(e.f!=null?e.f:""+e.g)))}}function tM(e,t,n){var s,a;for(G5n(e,t-e.s,n-e.t),a=new Q(e.n);a.a<a.c.c.length;)s=h(K(a),211),Xhn(s,s.e+t-e.s),Qhn(s,s.f+n-e.t);e.s=t,e.t=n}function x8n(e){var t,n,s,a,u;for(n=0,a=new Q(e.a);a.a<a.c.c.length;)s=h(K(a),121),s.d=n++;return t=J9n(e),u=null,t.c.length>1&&(u=w8n(e,t)),u}function FX(e){var t;return!!e.f&&e.f.kh()&&(t=h(e.f,49),e.f=h(Tg(e,t),82),e.f!=t&&(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,9,8,t,e.f))),e.f}function RX(e){var t;return!!e.i&&e.i.kh()&&(t=h(e.i,49),e.i=h(Tg(e,t),82),e.i!=t&&(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,9,7,t,e.i))),e.i}function aa(e){var t;return!!e.b&&(e.b.Db&64)!=0&&(t=e.b,e.b=h(Tg(e,t),18),e.b!=t&&(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,9,21,t,e.b))),e.b}function PF(e,t){var n,s,a;e.d==null?(++e.e,++e.f):(s=t.Sh(),RSn(e,e.f+1),a=(s&Ir)%e.d.length,n=e.d[a],!n&&(n=e.d[a]=e.uj()),n.Fc(t),++e.f)}function n2e(e,t,n){var s;return t.Kj()?!1:t.Zj()!=-2?(s=t.zj(),s==null?n==null:Nr(s,n)):t.Hj()==e.e.Tg()&&n==null}function BF(){var e;Uu(16,Gbt),e=Rut(16),this.b=pe(Xee,PM,317,e,0,1),this.c=pe(Xee,PM,317,e,0,1),this.a=null,this.e=null,this.i=0,this.f=e-1,this.g=0}function Nd(e){y1e.call(this),this.k=(bn(),Wi),this.j=(Uu(6,jw),new Wo(6)),this.b=(Uu(2,jw),new Wo(2)),this.d=new Tq,this.f=new the,this.a=e}function E8n(e){var t,n;e.c.length<=1||(t=Ugt(e,(Qe(),Fn)),Odt(e,h(t.a,19).a,h(t.b,19).a),n=Ugt(e,un),Odt(e,h(n.a,19).a,h(n.b,19).a))}function f9(){f9=re,Wke=new DS("SIMPLE",0),zne=new DS(vJ,1),Gne=new DS("LINEAR_SEGMENTS",2),N_=new DS("BRANDES_KOEPF",3),P_=new DS(gwt,4)}function r2e(e,t,n){F4(h(U(t,(tt(),Ei)),98))||(Q0e(e,t,x0(t,n)),Q0e(e,t,x0(t,(Qe(),Fn))),Q0e(e,t,x0(t,sn)),Rt(),hs(t.j,new YYe(e)))}function Wft(e,t,n,s){var a,u,p;for(a=h(jr(s?e.a:e.b,t),21),p=a.Kc();p.Ob();)if(u=h(p.Pb(),33),aR(e,n,u))return!0;return!1}function jX(e){var t,n;for(n=new Tn(e);n.e!=n.i.gc();)if(t=h(On(n),87),!!t.e||(!t.d&&(t.d=new Gi(pa,t,1)),t.d).i!=0)return!0;return!1}function $X(e){var t,n;for(n=new Tn(e);n.e!=n.i.gc();)if(t=h(On(n),87),!!t.e||(!t.d&&(t.d=new Gi(pa,t,1)),t.d).i!=0)return!0;return!1}function _8n(e){var t,n,s;for(t=0,s=new Q(e.c.a);s.a<s.c.c.length;)n=h(K(s),10),t+=Td(new Sn(Dn(qi(n).a.Kc(),new L)));return t/e.c.a.c.length}function Kft(e){var t,n;for(e.c||IIn(e),n=new Oc,t=new Q(e.a),K(t);t.a<t.c.c.length;)vr(n,h(K(t),407).a);return kn(n.b!=0),pl(n,n.c.b),n}function HX(){HX=re,N8e=(NQ(),D8e),O8e=new nb(8),new sa((Tr(),y2),O8e),new sa(k2,8),bEt=A8e,L8e=cEt,I8e=uEt,pEt=new sa(ZD,(on(),!1))}function i2e(e,t,n,s){switch(t){case 7:return!e.e&&(e.e=new qt(us,e,7,4)),e.e;case 8:return!e.d&&(e.d=new qt(us,e,8,5)),e.d}return _pe(e,t,n,s)}function zX(e){var t;return!!e.a&&e.a.kh()&&(t=h(e.a,49),e.a=h(Tg(e,t),138),e.a!=t&&(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,9,5,t,e.a))),e.a}function Wp(e){return e<48||e>102?-1:e<=57?e-48:e<65?-1:e<=70?e-65+10:e<97?-1:e-97+10}function GX(e,t){if(e==null)throw ne(new T4("null key in entry: null="+t));if(t==null)throw ne(new T4("null value in entry: "+e+"=null"))}function T8n(e,t){for(var n,s;e.Ob();)if(!t.Ob()||(n=e.Pb(),s=t.Pb(),!(Me(n)===Me(s)||n!=null&&Nr(n,s))))return!1;return!t.Ob()}function Xft(e,t){var n;return n=se(ie(ks,1),ka,25,15,[cX(e.a[0],t),cX(e.a[1],t),cX(e.a[2],t)]),e.d&&(n[0]=d.Math.max(n[0],n[2]),n[2]=n[0]),n}function Qft(e,t){var n;return n=se(ie(ks,1),ka,25,15,[hF(e.a[0],t),hF(e.a[1],t),hF(e.a[2],t)]),e.d&&(n[0]=d.Math.max(n[0],n[2]),n[2]=n[0]),n}function Kp(){Kp=re,Qte=new MS("GREEDY",0),Y5e=new MS(ewt,1),Zte=new MS(vJ,2),k_=new MS("MODEL_ORDER",3),y_=new MS("GREEDY_MODEL_ORDER",4)}function Zft(e,t){var n,s,a;for(e.b[t.g]=1,s=pr(t.d,0);s.b!=s.d.c;)n=h(gr(s),188),a=n.c,e.b[a.g]==1?vr(e.a,n):e.b[a.g]==2?e.b[a.g]=1:Zft(e,a)}function C8n(e,t){var n,s,a;for(a=new Wo(t.gc()),s=t.Kc();s.Ob();)n=h(s.Pb(),286),n.c==n.f?ox(e,n,n.c):__n(e,n)||(a.c[a.c.length]=n);return a}function S8n(e,t,n){var s,a,u,p,v;for(v=e.r+t,e.r+=t,e.d+=n,s=n/e.n.c.length,a=0,p=new Q(e.n);p.a<p.c.c.length;)u=h(K(p),211),BCn(u,v,s,a),++a}function A8n(e){var t,n,s;for(j7(e.b.a),e.a=pe(Gj,st,57,e.c.c.a.b.c.length,0,1),t=0,s=new Q(e.c.c.a.b);s.a<s.c.c.length;)n=h(K(s),57),n.f=t++}function M8n(e){var t,n,s;for(j7(e.b.a),e.a=pe(Ste,st,81,e.c.a.a.b.c.length,0,1),t=0,s=new Q(e.c.a.a.b);s.a<s.c.c.length;)n=h(K(s),81),n.i=t++}function D8n(e,t,n){var s;Bn(n,"Shrinking tree compaction",1),ft(lt(U(t,(Bk(),p_))))?(V4n(e,t.f),Lct(t.f,(s=t.c,s))):Lct(t.f,t.c),An(n)}function Jft(e){var t;if(t=D6n(e),!rr(e))throw ne(new Ea("position (0) must be less than the number of elements that remained ("+t+")"));return Un(e)}function e1t(e,t,n){var s;try{return Ay(e,t+e.j,n+e.k)}catch(a){throw a=ui(a),ue(a,73)?(s=a,ne(new Ea(s.g+OR+t+Js+n+")."))):ne(a)}}function L8n(e,t,n){var s;try{return r1t(e,t+e.j,n+e.k)}catch(a){throw a=ui(a),ue(a,73)?(s=a,ne(new Ea(s.g+OR+t+Js+n+")."))):ne(a)}}function I8n(e,t,n){var s;try{return i1t(e,t+e.j,n+e.k)}catch(a){throw a=ui(a),ue(a,73)?(s=a,ne(new Ea(s.g+OR+t+Js+n+")."))):ne(a)}}function t1t(e){switch(e.g){case 1:return Qe(),un;case 4:return Qe(),sn;case 3:return Qe(),pn;case 2:return Qe(),Fn;default:return Qe(),to}}function O8n(e,t,n){t.k==(bn(),Wi)&&n.k==ds&&(e.d=fX(t,(Qe(),Fn)),e.b=fX(t,sn)),n.k==Wi&&t.k==ds&&(e.d=fX(n,(Qe(),sn)),e.b=fX(n,Fn))}function VX(e,t){var n,s,a;for(a=Za(e,t),s=a.Kc();s.Ob();)if(n=h(s.Pb(),11),U(n,(He(),Zc))!=null||kE(new _f(n.b)))return!0;return!1}function s2e(e,t){return yc(t,e.e+e.d+(e.c.c.length==0?0:e.b)),kc(t,e.f),e.a=d.Math.max(e.a,t.f),e.d+=t.g+(e.c.c.length==0?0:e.b),Ue(e.c,t),!0}function N8n(e,t,n){var s,a,u,p;for(p=0,s=n/e.a.c.length,u=new Q(e.a);u.a<u.c.c.length;)a=h(K(u),187),tM(a,a.s,a.t+p*s),S8n(a,e.d-a.r+t,s),++p}function P8n(e){var t,n,s,a,u;for(s=new Q(e.b);s.a<s.c.c.length;)for(n=h(K(s),29),t=0,u=new Q(n.a);u.a<u.c.c.length;)a=h(K(u),10),a.p=t++}function B8n(e,t){var n,s,a,u,p,v;for(a=t.length-1,p=0,v=0,s=0;s<=a;s++)u=t[s],n=t_n(a,s)*upe(1-e,a-s)*upe(e,s),p+=u.a*n,v+=u.b*n;return new dt(p,v)}function n1t(e,t){var n,s,a,u,p;for(n=t.gc(),e.qi(e.i+n),u=t.Kc(),p=e.i,e.i+=n,s=p;s<e.i;++s)a=u.Pb(),wE(e,s,e.oi(s,a)),e.bi(s,a),e.ci();return n!=0}function F8n(e,t,n){var s,a,u;return e.ej()?(s=e.Vi(),u=e.fj(),++e.j,e.Hi(s,e.oi(s,t)),a=e.Zi(3,null,t,s,u),n?n.Ei(a):n=a):wtt(e,e.Vi(),t),n}function R8n(e,t,n){var s,a,u;return s=h(he(Co(e.a),t),87),u=(a=s.c,ue(a,88)?h(a,26):(Bt(),nh)),((u.Db&64)!=0?Tg(e.b,u):u)==n?hR(s):Sk(s,n),u}function a2e(e,t,n,s,a,u,p,v){var m,x;!s||(m=s.a[0],m&&a2e(e,t,n,m,a,u,p,v),f7n(e,n,s.d,a,u,p,v)&&t.Fc(s),x=s.a[1],x&&a2e(e,t,n,x,a,u,p,v))}function j8n(e,t){var n;return e.a||(n=pe(ks,ka,25,0,15,1),ty(e.b.a,new Nqe(n)),n.sort(cut(G.prototype.te,G,[])),e.a=new Xtt(n,e.d)),EB(e.a,t)}function Ay(e,t,n){try{return eb(RK(e,t,n),1)}catch(s){throw s=ui(s),ue(s,320)?ne(new Ea(nJ+e.o+"*"+e.p+rJ+t+Js+n+iJ)):ne(s)}}function r1t(e,t,n){try{return eb(RK(e,t,n),0)}catch(s){throw s=ui(s),ue(s,320)?ne(new Ea(nJ+e.o+"*"+e.p+rJ+t+Js+n+iJ)):ne(s)}}function i1t(e,t,n){try{return eb(RK(e,t,n),2)}catch(s){throw s=ui(s),ue(s,320)?ne(new Ea(nJ+e.o+"*"+e.p+rJ+t+Js+n+iJ)):ne(s)}}function s1t(e,t){if(e.g==-1)throw ne(new Ic);e.mj();try{e.d._c(e.g,t),e.f=e.d.j}catch(n){throw n=ui(n),ue(n,73)?ne(new cl):ne(n)}}function $8n(e,t,n){Bn(n,"Linear segments node placement",1),e.b=h(U(t,(He(),d3)),304),ePn(e,t),FLn(e,t),JLn(e,t),ONn(e),e.a=null,e.b=null,An(n)}function nM(e,t){var n,s,a,u;for(u=e.gc(),t.length<u&&(t=hN(new Array(u),t)),a=t,s=e.Kc(),n=0;n<u;++n)pi(a,n,s.Pb());return t.length>u&&pi(t,u,null),t}function H8n(e,t){var n,s;if(s=e.gc(),t==null){for(n=0;n<s;n++)if(e.Xb(n)==null)return n}else for(n=0;n<s;n++)if(Nr(t,e.Xb(n)))return n;return-1}function UX(e,t){var n,s,a;return n=t.cd(),a=t.dd(),s=e.xc(n),!(!(Me(a)===Me(s)||a!=null&&Nr(a,s))||s==null&&!e._b(n))}function z8n(e,t){var n,s,a;return t<=22?(n=e.l&(1<<t)-1,s=a=0):t<=44?(n=e.l,s=e.m&(1<<t-22)-1,a=0):(n=e.l,s=e.m,a=e.h&(1<<t-44)-1),rc(n,s,a)}function G8n(e,t){switch(t.g){case 1:return e.f.n.d+e.t;case 3:return e.f.n.a+e.t;case 2:return e.f.n.c+e.s;case 4:return e.f.n.b+e.s;default:return 0}}function V8n(e,t){var n,s;switch(s=t.c,n=t.a,e.b.g){case 0:n.d=e.e-s.a-s.d;break;case 1:n.d+=e.e;break;case 2:n.c=e.e-s.a-s.d;break;case 3:n.c=e.e+s.d}}function o2e(e,t,n,s){var a,u;this.a=t,this.c=s,a=e.a,Ohn(this,new dt(-a.c,-a.d)),zr(this.b,n),u=s/2,t.a?GS(this.b,0,u):GS(this.b,u,0),Ue(e.c,this)}function FF(){FF=re,bre=new $N(jd,0),qxe=new $N(twt,1),Yxe=new $N("EDGE_LENGTH_BY_POSITION",2),Uxe=new $N("CROSSING_MINIMIZATION_BY_POSITION",3)}function qX(e,t){var n,s;if(n=h(wy(e.g,t),33),n)return n;if(s=h(wy(e.j,t),118),s)return s;throw ne(new Xf("Referenced shape does not exist: "+t))}function U8n(e,t){if(e.c==t)return e.d;if(e.d==t)return e.c;throw ne(new an("Node 'one' must be either source or target of edge 'edge'."))}function q8n(e,t){if(e.c.i==t)return e.d.i;if(e.d.i==t)return e.c.i;throw ne(new an("Node "+t+" is neither source nor target of edge "+e))}function Y8n(e,t){var n;switch(t.g){case 2:case 4:n=e.a,e.c.d.n.b<n.d.n.b&&(n=e.c),l0(e,t,(Sf(),jte),n);break;case 1:case 3:l0(e,t,(Sf(),L5),null)}}function YX(e,t,n,s,a,u){var p,v,m,x,_;for(p=X7n(t,n,u),v=n==(Qe(),sn)||n==un?-1:1,x=e[n.g],_=0;_<x.length;_++)m=x[_],m>0&&(m+=a),x[_]=p,p+=v*(m+s)}function a1t(e){var t,n,s;for(s=e.f,e.n=pe(ks,ka,25,s,15,1),e.d=pe(ks,ka,25,s,15,1),t=0;t<s;t++)n=h(ut(e.c.b,t),29),e.n[t]=Bft(e,n),e.d[t]=vgt(e,n)}function WX(e,t){var n,s,a;for(a=0,s=2;s<t;s<<=1)(e.Db&s)!=0&&++a;if(a==0){for(n=t<<=1;n<=128;n<<=1)if((e.Db&n)!=0)return 0;return-1}else return a}function o1t(e,t){var n,s,a,u,p;for(p=ac(e.e.Tg(),t),u=null,n=h(e.g,119),a=0;a<e.i;++a)s=n[a],p.rl(s.ak())&&(!u&&(u=new UO),qn(u,s));u&&mbt(e,u)}function c1t(e){var t,n,s;if(!e)return null;if(e.dc())return"";for(s=new s0,n=e.Kc();n.Ob();)t=n.Pb(),va(s,Jn(t)),s.a+=" ";return TY(s,s.a.length-1)}function c2e(e,t,n){var s,a,u,p,v;for(J5n(e),a=(e.k==null&&(e.k=pe(Zee,Re,78,0,0,1)),e.k),u=0,p=a.length;u<p;++u)s=a[u],c2e(s);v=e.f,v&&c2e(v)}function u1t(e,t){var n=new Array(t),s;switch(e){case 14:case 15:s=0;break;case 16:s=!1;break;default:return n}for(var a=0;a<t;++a)n[a]=s;return n}function Ob(e){var t,n,s;for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),57),t.c.$b();lg(e.d)?s=e.a.c:s=e.a.d,mc(s,new Uqe(e)),e.c.Me(e),Ngt(e)}function l1t(e){var t,n,s,a;for(n=new Q(e.e.c);n.a<n.c.c.length;){for(t=h(K(n),282),a=new Q(t.b);a.a<a.c.c.length;)s=h(K(a),447),Tve(s);n0t(t)}}function RF(e){var t,n,s,a,u;for(s=0,u=0,a=0,n=new Q(e.a);n.a<n.c.c.length;)t=h(K(n),187),u=d.Math.max(u,t.r),s+=t.d+(a>0?e.c:0),++a;e.b=s,e.d=u}function W8n(e,t){var n,s,a,u,p;for(s=0,a=0,n=0,p=new Q(t);p.a<p.c.c.length;)u=h(K(p),200),s=d.Math.max(s,u.e),a+=u.b+(n>0?e.g:0),++n;e.c=a,e.d=s}function h1t(e,t){var n;return n=se(ie(ks,1),ka,25,15,[t2e(e,(Kh(),ao),t),t2e(e,Jo,t),t2e(e,oo,t)]),e.f&&(n[0]=d.Math.max(n[0],n[2]),n[2]=n[0]),n}function K8n(e,t,n){var s;try{uR(e,t+e.j,n+e.k,!1,!0)}catch(a){throw a=ui(a),ue(a,73)?(s=a,ne(new Ea(s.g+OR+t+Js+n+")."))):ne(a)}}function X8n(e,t,n){var s;try{uR(e,t+e.j,n+e.k,!0,!1)}catch(a){throw a=ui(a),ue(a,73)?(s=a,ne(new Ea(s.g+OR+t+Js+n+")."))):ne(a)}}function f1t(e){var t;!os(e,(tt(),sv))||(t=h(U(e,sv),21),t.Hc((Iw(),Lh))?(t.Mc(Lh),t.Fc(Ih)):t.Hc(Ih)&&(t.Mc(Ih),t.Fc(Lh)))}function d1t(e){var t;!os(e,(tt(),sv))||(t=h(U(e,sv),21),t.Hc((Iw(),Nh))?(t.Mc(Nh),t.Fc(Xl)):t.Hc(Xl)&&(t.Mc(Xl),t.Fc(Nh)))}function Q8n(e,t,n){Bn(n,"Self-Loop ordering",1),Ti(pc(Jr(Jr(Xa(new Vt(null,new Wt(t.b,16)),new mRe),new yRe),new kRe),new xRe),new kYe(e)),An(n)}function rM(e,t,n,s){var a,u;for(a=t;a<e.c.length;a++)if(u=(Kt(a,e.c.length),h(e.c[a],11)),n.Mb(u))s.c[s.c.length]=u;else return a;return e.c.length}function KX(e,t,n,s){var a,u,p,v;return e.a==null&&e_n(e,t),p=t.b.j.c.length,u=n.d.p,v=s.d.p,a=v-1,a<0&&(a=p-1),u<=a?e.a[a]-e.a[u]:e.a[p-1]-e.a[u]+e.a[a]}function Z8n(e){var t,n;if(!e.b)for(e.b=dB(h(e.f,33).Ag().i),n=new Tn(h(e.f,33).Ag());n.e!=n.i.gc();)t=h(On(n),137),Ue(e.b,new Bq(t));return e.b}function J8n(e){var t,n;if(!e.e)for(e.e=dB(DW(h(e.f,33)).i),n=new Tn(DW(h(e.f,33)));n.e!=n.i.gc();)t=h(On(n),118),Ue(e.e,new hKe(t));return e.e}function g1t(e){var t,n;if(!e.a)for(e.a=dB(QP(h(e.f,33)).i),n=new Tn(QP(h(e.f,33)));n.e!=n.i.gc();)t=h(On(n),33),Ue(e.a,new PY(e,t));return e.a}function Nb(e){var t;if(!e.C&&(e.D!=null||e.B!=null))if(t=yOn(e),t)e.yk(t);else try{e.yk(null)}catch(n){if(n=ui(n),!ue(n,60))throw ne(n)}return e.C}function e7n(e){switch(e.q.g){case 5:R1t(e,(Qe(),sn)),R1t(e,Fn);break;case 4:D2t(e,(Qe(),sn)),D2t(e,Fn);break;default:jdt(e,(Qe(),sn)),jdt(e,Fn)}}function t7n(e){switch(e.q.g){case 5:j1t(e,(Qe(),pn)),j1t(e,un);break;case 4:L2t(e,(Qe(),pn)),L2t(e,un);break;default:$dt(e,(Qe(),pn)),$dt(e,un)}}function My(e,t){var n,s,a;for(a=new Rs,s=e.Kc();s.Ob();)n=h(s.Pb(),37),_9(n,a.a,0),a.a+=n.f.a+t,a.b=d.Math.max(a.b,n.f.b);return a.b>0&&(a.b+=t),a}function jF(e,t){var n,s,a;for(a=new Rs,s=e.Kc();s.Ob();)n=h(s.Pb(),37),_9(n,0,a.b),a.b+=n.f.b+t,a.a=d.Math.max(a.a,n.f.a);return a.a>0&&(a.a+=t),a}function p1t(e){var t,n,s;for(s=Ir,n=new Q(e.a);n.a<n.c.c.length;)t=h(K(n),10),os(t,(He(),Eo))&&(s=d.Math.min(s,h(U(t,Eo),19).a));return s}function b1t(e,t){var n,s;if(t.length==0)return 0;for(n=MW(e.a,t[0],(Qe(),un)),n+=MW(e.a,t[t.length-1],pn),s=0;s<t.length;s++)n+=R9n(e,s,t);return n}function v1t(){x9(),this.c=new qe,this.i=new qe,this.e=new md,this.f=new md,this.g=new md,this.j=new qe,this.a=new qe,this.b=new zn,this.k=new zn}function XX(e,t){var n,s;return e.Db>>16==6?e.Cb.ih(e,5,sf,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||e.zh()),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function n7n(e){bk();var t=e.e;if(t&&t.stack){var n=t.stack,s=t+`
`;return n.substring(0,s.length)==s&&(n=n.substring(s.length)),n.split(`
`)}return[]}function r7n(e){var t;return t=(Iut(),d3t),t[e>>>28]|t[e>>24&15]<<4|t[e>>20&15]<<8|t[e>>16&15]<<12|t[e>>12&15]<<16|t[e>>8&15]<<20|t[e>>4&15]<<24|t[e&15]<<28}function w1t(e){var t,n,s;e.b==e.c&&(s=e.a.length,n=Gge(d.Math.max(8,s))<<1,e.b!=0?(t=vh(e.a,n),$ut(e,t,s),e.a=t,e.b=0):FKe(e.a,n),e.c=s)}function i7n(e,t){var n;return n=e.b,n.Xe((Tr(),pu))?n.Hf()==(Qe(),un)?-n.rf().a-Pe(Je(n.We(pu))):t+Pe(Je(n.We(pu))):n.Hf()==(Qe(),un)?-n.rf().a:t}function iM(e){var t;return e.b.c.length!=0&&!!h(ut(e.b,0),70).a?h(ut(e.b,0),70).a:(t=PW(e),t!=null?t:""+(e.c?Ha(e.c.a,e,0):-1))}function $F(e){var t;return e.f.c.length!=0&&!!h(ut(e.f,0),70).a?h(ut(e.f,0),70).a:(t=PW(e),t!=null?t:""+(e.i?Ha(e.i.j,e,0):-1))}function s7n(e,t){var n,s;if(t<0||t>=e.gc())return null;for(n=t;n<e.gc();++n)if(s=h(e.Xb(n),128),n==e.gc()-1||!s.o)return new As(Ke(n),s);return null}function a7n(e,t,n){var s,a,u,p,v;for(u=e.c,v=n?t:e,s=n?e:t,a=v.p+1;a<s.p;++a)if(p=h(ut(u.a,a),10),!(p.k==(bn(),Bf)||N7n(p)))return!1;return!0}function u2e(e){var t,n,s,a,u;for(u=0,a=Fi,s=0,n=new Q(e.a);n.a<n.c.c.length;)t=h(K(n),187),u+=t.r+(s>0?e.c:0),a=d.Math.max(a,t.d),++s;e.e=u,e.b=a}function o7n(e){var t,n;if(!e.b)for(e.b=dB(h(e.f,118).Ag().i),n=new Tn(h(e.f,118).Ag());n.e!=n.i.gc();)t=h(On(n),137),Ue(e.b,new Bq(t));return e.b}function c7n(e,t){var n,s,a;if(t.dc())return ik(),ik(),bL;for(n=new ktt(e,t.gc()),a=new Tn(e);a.e!=a.i.gc();)s=On(a),t.Hc(s)&&qn(n,s);return n}function l2e(e,t,n,s){return t==0?s?(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),e.o):(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),wA(e.o)):IF(e,t,n,s)}function QX(e){var t,n;if(e.rb)for(t=0,n=e.rb.i;t<n;++t)zS(he(e.rb,t));if(e.vb)for(t=0,n=e.vb.i;t<n;++t)zS(he(e.vb,t));AE((jc(),Ps),e),e.Bb|=1}function Zr(e,t,n,s,a,u,p,v,m,x,_,A,I,P){return Wdt(e,t,s,null,a,u,p,v,m,x,I,!0,P),ype(e,_),ue(e.Cb,88)&&Lw(ou(h(e.Cb,88)),2),n&&yge(e,n),kpe(e,A),e}function u7n(e){var t,n;if(e==null)return null;n=0;try{n=Wu(e,$s,Ir)&Oi}catch(s){if(s=ui(s),ue(s,127))t=LB(e),n=t[0];else throw ne(s)}return RA(n)}function l7n(e){var t,n;if(e==null)return null;n=0;try{n=Wu(e,$s,Ir)&Oi}catch(s){if(s=ui(s),ue(s,127))t=LB(e),n=t[0];else throw ne(s)}return RA(n)}function h7n(e,t){var n,s,a;return a=e.h-t.h,a<0||(n=e.l-t.l,s=e.m-t.m+(n>>22),a+=s>>22,a<0)?!1:(e.l=n&du,e.m=s&du,e.h=a&Rd,!0)}function f7n(e,t,n,s,a,u,p){var v,m;return!(t.Ae()&&(m=e.a.ue(n,s),m<0||!a&&m==0)||t.Be()&&(v=e.a.ue(n,u),v>0||!p&&v==0))}function d7n(e,t){Uk();var n;if(n=e.j.g-t.j.g,n!=0)return 0;switch(e.j.g){case 2:return yX(t,j5e)-yX(e,j5e);case 4:return yX(e,R5e)-yX(t,R5e)}return 0}function g7n(e){switch(e.g){case 0:return ene;case 1:return tne;case 2:return nne;case 3:return rne;case 4:return A$;case 5:return ine;default:return null}}function Ma(e,t,n){var s,a;return s=(a=new Aq,Vp(a,t),Ko(a,n),qn((!e.c&&(e.c=new Ye(dv,e,12,10)),e.c),a),a),p0(s,0),yw(s,1),m0(s,!0),w0(s,!0),s}function c5(e,t){var n,s;if(t>=e.i)throw ne(new EY(t,e.i));return++e.j,n=e.g[t],s=e.i-t-1,s>0&&Ao(e.g,t+1,e.g,t,s),pi(e.g,--e.i,null),e.fi(t,n),e.ci(),n}function m1t(e,t){var n,s;return e.Db>>16==17?e.Cb.ih(e,21,Jl,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||e.zh()),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function p7n(e){var t,n,s,a;for(Rt(),hs(e.c,e.a),a=new Q(e.c);a.a<a.c.c.length;)for(s=K(a),n=new Q(e.b);n.a<n.c.c.length;)t=h(K(n),679),t.Ke(s)}function b7n(e){var t,n,s,a;for(Rt(),hs(e.c,e.a),a=new Q(e.c);a.a<a.c.c.length;)for(s=K(a),n=new Q(e.b);n.a<n.c.c.length;)t=h(K(n),369),t.Ke(s)}function v7n(e){var t,n,s,a,u;for(a=Ir,u=null,s=new Q(e.d);s.a<s.c.c.length;)n=h(K(s),213),n.d.j^n.e.j&&(t=n.e.e-n.d.e-n.a,t<a&&(a=t,u=n));return u}function h2e(){h2e=re,e5t=new $t(Lwe,(on(),!1)),Q4t=new $t(Iwe,100),F4e=(UE(),Tte),Z4t=new $t(Owe,F4e),J4t=new $t(Nwe,l1),t5t=new $t(Pwe,Ke(Ir))}function y1t(e,t,n){var s,a,u,p,v,m,x,_;for(x=0,a=e.a[t],u=0,p=a.length;u<p;++u)for(s=a[u],_=r9(s,n),m=_.Kc();m.Ob();)v=h(m.Pb(),11),Pr(e.f,v,Ke(x++))}function w7n(e,t,n){var s,a,u,p;if(n)for(a=n.a.length,s=new Mp(a),p=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);p.Ob();)u=h(p.Pb(),19),Ot(e,t,l5(bw(n,u.a)))}function m7n(e,t,n){var s,a,u,p;if(n)for(a=n.a.length,s=new Mp(a),p=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);p.Ob();)u=h(p.Pb(),19),Ot(e,t,l5(bw(n,u.a)))}function f2e(e){Mg();var t;return t=h(nM($4(e.k),pe(ta,yo,61,2,0,1)),122),_k(t,0,t.length,null),t[0]==(Qe(),sn)&&t[1]==un&&(pi(t,0,un),pi(t,1,sn)),t}function k1t(e,t,n){var s,a,u;return a=iAn(e,t,n),u=Jbe(e,a),kK(e.b),JW(e,t,n),Rt(),hs(a,new rWe(e)),s=Jbe(e,a),kK(e.b),JW(e,n,t),new As(Ke(u),Ke(s))}function x1t(){x1t=re,l8t=Lr(new rs,(Zs(),Pa),(oa(),w_)),cH=new Xi("linearSegments.inputPrio",Ke(0)),uH=new Xi("linearSegments.outputPrio",Ke(0))}function tx(){tx=re,lH=new jN("P1_TREEIFICATION",0),H_=new jN("P2_NODE_ORDERING",1),HD=new jN("P3_NODE_PLACEMENT",2),z_=new jN("P4_EDGE_ROUTING",3)}function Cw(){Cw=re,T7t=(Tr(),q5),C7t=k2,k7t=m2,x7t=_3,E7t=Wg,y7t=E3,zxe=tL,_7t=gm,gre=(Pbe(),d7t),pre=g7t,Gxe=p7t,mH=b7t,yH=v7t,GD=w7t,Vxe=m7t}function Yu(){Yu=re,iL=new zN("UNKNOWN",0),Z1=new zN("ABOVE",1),Kg=new zN("BELOW",2),rL=new zN("INLINE",3),new Xi("org.eclipse.elk.labelSide",iL)}function E1t(e,t){var n;if(e.ni()&&t!=null){for(n=0;n<e.i;++n)if(Nr(t,e.g[n]))return n}else for(n=0;n<e.i;++n)if(Me(e.g[n])===Me(t))return n;return-1}function y7n(e,t,n){var s,a;return t.c==(ua(),ec)&&n.c==Jc?-1:t.c==Jc&&n.c==ec?1:(s=ght(t.a,e.a),a=ght(n.a,e.a),t.c==ec?a-s:s-a)}function Sw(e,t,n){if(!!n&&(t<0||t>n.a.c.length))throw ne(new an("index must be >= 0 and <= layer node count"));e.c&&vc(e.c.a,e),e.c=n,n&&aw(n.a,t,e)}function _1t(e,t){var n,s,a;for(s=new Sn(Dn(Id(e).a.Kc(),new L));rr(s);)return n=h(Un(s),17),a=h(t.Kb(n),10),new YO(Vn(a.n.b+a.o.b/2));return F7(),F7(),qee}function T1t(e,t){this.c=new zn,this.a=e,this.b=t,this.d=h(U(e,(He(),d3)),304),Me(U(e,(tt(),_ke)))===Me((xA(),M$))?this.e=new eXe:this.e=new JKe}function k7n(e,t){var n,s,a,u;for(u=0,s=new Q(e);s.a<s.c.c.length;)n=h(K(s),33),u+=d.Math.pow(n.g*n.f-t,2);return a=d.Math.sqrt(u/(e.c.length-1)),a}function d9(e,t){var n,s;return s=null,e.Xe((Tr(),Y5))&&(n=h(e.We(Y5),94),n.Xe(t)&&(s=n.We(t))),s==null&&!!e.yf()&&(s=e.yf().We(t)),s==null&&(s=at(t)),s}function ZX(e,t){var n,s;n=e.Zc(t);try{return s=n.Pb(),n.Qb(),s}catch(a){throw a=ui(a),ue(a,109)?ne(new Ea("Can't remove element "+t)):ne(a)}}function x7n(e,t){var n,s,a;if(s=new mN,a=new zge(s.q.getFullYear()-Og,s.q.getMonth(),s.q.getDate()),n=ELn(e,t,a),n==0||n<t.length)throw ne(new an(t));return a}function d2e(e,t){var n,s,a;for(rn(t),e1e(t!=e),a=e.b.c.length,s=t.Kc();s.Ob();)n=s.Pb(),Ue(e.b,rn(n));return a!=e.b.c.length?(Jge(e,0),!0):!1}function sM(){sM=re,H4e=(Tr(),LH),new sa(Pre,(on(),!0)),r5t=m2,i5t=_3,s5t=Wg,n5t=E3,G4e=tL,a5t=gm,$4e=(h2e(),e5t),R4e=Z4t,j4e=J4t,z4e=t5t,e$=Q4t}function E7n(e,t){if(t==e.c)return e.d;if(t==e.d)return e.c;throw ne(new an("'port' must be either the source port or target port of the edge."))}function _7n(e,t,n){var s,a;switch(a=e.o,s=e.d,t.g){case 1:return-s.d-n;case 3:return a.b+s.a+n;case 2:return a.a+s.c+n;case 4:return-s.b-n;default:return 0}}function g2e(e,t,n,s){var a,u,p,v;for(Ca(t,h(s.Xb(0),29)),v=s.bd(1,s.gc()),u=h(n.Kb(t),20).Kc();u.Ob();)a=h(u.Pb(),17),p=a.c.i==t?a.d.i:a.c.i,g2e(e,p,n,v)}function C1t(e){var t;return t=new zn,os(e,(He(),vne))?h(U(e,vne),83):(Ti(Jr(new Vt(null,new Wt(e.j,16)),new VRe),new _Ye(t)),Fe(e,vne,t),t)}function p2e(e,t){var n,s;return e.Db>>16==6?e.Cb.ih(e,6,us,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Qo(),zH)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function b2e(e,t){var n,s;return e.Db>>16==7?e.Cb.ih(e,1,hL,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Qo(),iEe)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function v2e(e,t){var n,s;return e.Db>>16==9?e.Cb.ih(e,9,wi,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Qo(),aEe)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function S1t(e,t){var n,s;return e.Db>>16==5?e.Cb.ih(e,9,QH,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Bt(),R0)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function w2e(e,t){var n,s;return e.Db>>16==3?e.Cb.ih(e,0,dL,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Bt(),F0)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function A1t(e,t){var n,s;return e.Db>>16==7?e.Cb.ih(e,6,sf,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Bt(),$0)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function M1t(){this.a=new Oze,this.g=new BF,this.j=new BF,this.b=new zn,this.d=new BF,this.i=new BF,this.k=new zn,this.c=new zn,this.e=new zn,this.f=new zn}function T7n(e,t,n){var s,a,u;for(n<0&&(n=0),u=e.i,a=n;a<u;a++)if(s=he(e,a),t==null){if(s==null)return a}else if(Me(t)===Me(s)||Nr(t,s))return a;return-1}function C7n(e,t){var n,s;return n=t.Hh(e.a),n?(s=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),Cj)),Nt(aD,s)?AE(e,Vu(t.Hj())):s):null}function nx(e,t){var n,s;if(t){if(t==e)return!0;for(n=0,s=h(t,49).eh();!!s&&s!=t;s=s.eh()){if(++n>jZ)return nx(e,s);if(s==e)return!0}}return!1}function S7n(e){switch(oP(),e.q.g){case 5:d0t(e,(Qe(),sn)),d0t(e,Fn);break;case 4:cgt(e,(Qe(),sn)),cgt(e,Fn);break;default:cbt(e,(Qe(),sn)),cbt(e,Fn)}}function A7n(e){switch(oP(),e.q.g){case 5:A0t(e,(Qe(),pn)),A0t(e,un);break;case 4:jft(e,(Qe(),pn)),jft(e,un);break;default:ubt(e,(Qe(),pn)),ubt(e,un)}}function M7n(e){var t,n;t=h(U(e,(Jh(),W4t)),19),t?(n=t.a,n==0?Fe(e,(kg(),Jj),new mX):Fe(e,(kg(),Jj),new vB(n))):Fe(e,(kg(),Jj),new vB(1))}function D7n(e,t){var n;switch(n=e.i,t.g){case 1:return-(e.n.b+e.o.b);case 2:return e.n.a-n.o.a;case 3:return e.n.b-n.o.b;case 4:return-(e.n.a+e.o.a)}return 0}function L7n(e,t){switch(e.g){case 0:return t==(wl(),Gg)?m$:y$;case 1:return t==(wl(),Gg)?m$:ED;case 2:return t==(wl(),Gg)?ED:y$;default:return ED}}function aM(e,t){var n,s,a;for(vc(e.a,t),e.e-=t.r+(e.a.c.length==0?0:e.c),a=sye,s=new Q(e.a);s.a<s.c.c.length;)n=h(K(s),187),a=d.Math.max(a,n.d);e.b=a}function m2e(e,t){var n,s;return e.Db>>16==3?e.Cb.ih(e,12,wi,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Qo(),rEe)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function y2e(e,t){var n,s;return e.Db>>16==11?e.Cb.ih(e,10,wi,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Qo(),sEe)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function D1t(e,t){var n,s;return e.Db>>16==10?e.Cb.ih(e,11,Jl,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Bt(),j0)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function L1t(e,t){var n,s;return e.Db>>16==10?e.Cb.ih(e,12,eh,t):(s=aa(h(Ht((n=h(tn(e,16),26),n||(Bt(),km)),e.Db>>16),18)),e.Cb.ih(e,s.n,s.f,t))}function Rl(e){var t;return(e.Bb&1)==0&&!!e.r&&e.r.kh()&&(t=h(e.r,49),e.r=h(Tg(e,t),138),e.r!=t&&(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,9,8,t,e.r))),e.r}function JX(e,t,n){var s;return s=se(ie(ks,1),ka,25,15,[U2e(e,(Kh(),ao),t,n),U2e(e,Jo,t,n),U2e(e,oo,t,n)]),e.f&&(s[0]=d.Math.max(s[0],s[2]),s[2]=s[0]),s}function I7n(e,t){var n,s,a;if(a=C8n(e,t),a.c.length!=0)for(hs(a,new UFe),n=a.c.length,s=0;s<n;s++)ox(e,(Kt(s,a.c.length),h(a.c[s],286)),ZAn(e,a,s))}function O7n(e){var t,n,s,a;for(a=h(jr(e.a,(Iy(),k$)),15).Kc();a.Ob();)for(s=h(a.Pb(),101),n=$4(s.k).Kc();n.Ob();)t=h(n.Pb(),61),lk(e,s,t,(Tb(),zg),1)}function N7n(e){var t,n;if(e.k==(bn(),ds)){for(n=new Sn(Dn(Id(e).a.Kc(),new L));rr(n);)if(t=h(Un(n),17),!Xs(t)&&e.c==S2e(t,e).c)return!0}return!1}function P7n(e){var t,n;if(e.k==(bn(),ds)){for(n=new Sn(Dn(Id(e).a.Kc(),new L));rr(n);)if(t=h(Un(n),17),!Xs(t)&&t.c.i.c==t.d.i.c)return!0}return!1}function B7n(e,t){var n,s,a,u;for(Bn(t,"Dull edge routing",1),u=pr(e.b,0);u.b!=u.d.c;)for(a=h(gr(u),86),s=pr(a.d,0);s.b!=s.d.c;)n=h(gr(s),188),Pl(n.a)}function F7n(e,t){var n,s,a,u,p;if(t)for(a=t.a.length,n=new Mp(a),p=(n.b-n.a)*n.c<0?(cg(),sp):new fg(n);p.Ob();)u=h(p.Pb(),19),s=wk(t,u.a),s&&tgt(e,s)}function R7n(){Bhe();var e,t;for(rPn((bg(),Xt)),KNn(Xt),QX(Xt),yEe=(Bt(),zf),t=new Q(MEe);t.a<t.c.c.length;)e=h(K(t),241),wx(e,zf,null);return!0}function k2e(e,t){var n,s,a,u,p,v,m,x;return m=e.h>>19,x=t.h>>19,m!=x?x-m:(a=e.h,v=t.h,a!=v?a-v:(s=e.m,p=t.m,s!=p?s-p:(n=e.l,u=t.l,n-u)))}function HF(){HF=re,u4e=(oR(),hte),c4e=new $t(uwe,u4e),o4e=(OB(),lte),a4e=new $t(lwe,o4e),s4e=(SF(),ute),i4e=new $t(hwe,s4e),r4e=new $t(fwe,(on(),!0))}function g9(e,t,n){var s,a;s=t*n,ue(e.g,145)?(a=W4(e),a.f.d?a.f.a||(e.d.a+=s+If):(e.d.d-=s+If,e.d.a+=s+If)):ue(e.g,10)&&(e.d.d-=s,e.d.a+=2*s)}function I1t(e,t,n){var s,a,u,p,v;for(a=e[n.g],v=new Q(t.d);v.a<v.c.c.length;)p=h(K(v),101),u=p.i,!!u&&u.i==n&&(s=p.d[n.g],a[s]=d.Math.max(a[s],u.j.b))}function j7n(e,t){var n,s,a,u,p;for(s=0,a=0,n=0,p=new Q(t.d);p.a<p.c.c.length;)u=h(K(p),443),RF(u),s=d.Math.max(s,u.b),a+=u.d+(n>0?e.g:0),++n;t.b=s,t.e=a}function O1t(e){var t,n,s;if(s=e.b,JQe(e.i,s.length)){for(n=s.length*2,e.b=pe(Xee,PM,317,n,0,1),e.c=pe(Xee,PM,317,n,0,1),e.f=n-1,e.i=0,t=e.a;t;t=t.c)dM(e,t,t);++e.g}}function $7n(e,t,n,s){var a,u,p,v;for(a=0;a<t.o;a++)for(u=a-t.j+n,p=0;p<t.p;p++)v=p-t.k+s,Ay(t,a,p)?I8n(e,u,v)||K8n(e,u,v):i1t(t,a,p)&&(e1t(e,u,v)||X8n(e,u,v))}function H7n(e,t,n){var s;s=t.c.i,s.k==(bn(),ds)?(Fe(e,(He(),rf),h(U(s,rf),11)),Fe(e,Yl,h(U(s,Yl),11))):(Fe(e,(He(),rf),t.c),Fe(e,Yl,n.d))}function rx(e,t,n){Ry();var s,a,u,p,v,m;return p=t/2,u=n/2,s=d.Math.abs(e.a),a=d.Math.abs(e.b),v=1,m=1,s>p&&(v=p/s),a>u&&(m=u/a),Jf(e,d.Math.min(v,m)),e}function z7n(){dR();var e,t;try{if(t=h(L2e((og(),th),Bx),2014),t)return t}catch(n){if(n=ui(n),ue(n,102))e=n,wde((Qn(),e));else throw ne(n)}return new Mze}function G7n(){_ct();var e,t;try{if(t=h(L2e((og(),th),o2),2024),t)return t}catch(n){if(n=ui(n),ue(n,102))e=n,wde((Qn(),e));else throw ne(n)}return new WGe}function V7n(){dR();var e,t;try{if(t=h(L2e((og(),th),Vl),1941),t)return t}catch(n){if(n=ui(n),ue(n,102))e=n,wde((Qn(),e));else throw ne(n)}return new tGe}function U7n(e,t,n){var s,a;return a=e.e,e.e=t,(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,4,a,t),n?n.Ei(s):n=s),a!=t&&(t?n=wx(e,eR(e,t),n):n=wx(e,e.a,n)),n}function N1t(){mN.call(this),this.e=-1,this.a=!1,this.p=$s,this.k=-1,this.c=-1,this.b=-1,this.g=!1,this.f=-1,this.j=-1,this.n=-1,this.i=-1,this.d=-1,this.o=$s}function q7n(e,t){var n,s,a;if(s=e.b.d.d,e.a||(s+=e.b.d.a),a=t.b.d.d,t.a||(a+=t.b.d.a),n=Ui(s,a),n==0){if(!e.a&&t.a)return-1;if(!t.a&&e.a)return 1}return n}function Y7n(e,t){var n,s,a;if(s=e.b.b.d,e.a||(s+=e.b.b.a),a=t.b.b.d,t.a||(a+=t.b.b.a),n=Ui(s,a),n==0){if(!e.a&&t.a)return-1;if(!t.a&&e.a)return 1}return n}function W7n(e,t){var n,s,a;if(s=e.b.g.d,e.a||(s+=e.b.g.a),a=t.b.g.d,t.a||(a+=t.b.g.a),n=Ui(s,a),n==0){if(!e.a&&t.a)return-1;if(!t.a&&e.a)return 1}return n}function x2e(){x2e=re,c5t=Wc(Lr(Lr(Lr(new rs,(Zs(),Go),(oa(),_5e)),Go,T5e),Pa,C5e),Pa,d5e),l5t=Lr(Lr(new rs,Go,a5e),Go,g5e),u5t=Wc(new rs,Pa,b5e)}function K7n(e){var t,n,s,a,u;for(t=h(U(e,(He(),T_)),83),u=e.n,s=t.Cc().Kc();s.Ob();)n=h(s.Pb(),306),a=n.i,a.c+=u.a,a.d+=u.b,n.c?Xgt(n):Qgt(n);Fe(e,T_,null)}function X7n(e,t,n){var s,a;switch(a=e.b,s=a.d,t.g){case 1:return-s.d-n;case 2:return a.o.a+s.c+n;case 3:return a.o.b+s.a+n;case 4:return-s.b-n;default:return-1}}function Q7n(e){var t,n,s,a,u;if(s=0,a=Ax,e.b)for(t=0;t<360;t++)n=t*.017453292519943295,fve(e,e.d,0,0,Ky,n),u=e.b.ig(e.d),u<a&&(s=n,a=u);fve(e,e.d,0,0,Ky,s)}function Z7n(e,t){var n,s,a,u;for(u=new zn,t.e=null,t.f=null,s=new Q(t.i);s.a<s.c.c.length;)n=h(K(s),65),a=h(_n(e.g,n.a),46),n.a=FP(n.b),Pr(u,n.a,a);e.g=u}function J7n(e,t,n){var s,a,u,p,v,m;for(a=t-e.e,u=a/e.d.c.length,p=0,m=new Q(e.d);m.a<m.c.c.length;)v=h(K(m),443),s=e.b-v.b+n,oft(v,v.e+p*u,v.f),N8n(v,u,s),++p}function P1t(e){var t;if(e.f.qj(),e.b!=-1){if(++e.b,t=e.f.d[e.a],e.b<t.i)return;++e.a}for(;e.a<e.f.d.length;++e.a)if(t=e.f.d[e.a],!!t&&t.i!=0){e.b=0;return}e.b=-1}function eEn(e,t){var n,s,a;for(a=t.c.length,n=J_n(e,a==0?"":(Kt(0,t.c.length),Jn(t.c[0]))),s=1;s<a&&!!n;++s)n=h(n,49).oh((Kt(s,t.c.length),Jn(t.c[s])));return n}function B1t(e,t){var n,s;for(s=new Q(t);s.a<s.c.c.length;)n=h(K(s),10),e.c[n.c.p][n.p].a=lde(e.i),e.c[n.c.p][n.p].d=Pe(e.c[n.c.p][n.p].a),e.c[n.c.p][n.p].b=1}function tEn(e,t){var n,s,a,u;for(u=0,s=new Q(e);s.a<s.c.c.length;)n=h(K(s),157),u+=d.Math.pow(Fc(n)*Su(n)-t,2);return a=d.Math.sqrt(u/(e.c.length-1)),a}function F1t(e,t,n,s){var a,u,p;return u=AMn(e,t,n,s),p=ave(e,u),uQ(e,t,n,s),kK(e.b),Rt(),hs(u,new iWe(e)),a=ave(e,u),uQ(e,n,t,s),kK(e.b),new As(Ke(p),Ke(a))}function nEn(e,t,n){var s,a;for(Bn(n,"Interactive node placement",1),e.a=h(U(t,(He(),d3)),304),a=new Q(t.b);a.a<a.c.c.length;)s=h(K(a),29),$An(e,s);An(n)}function rEn(e,t){var n;Bn(t,"General Compactor",1),t.n&&!!e&&mh(t,yh(e),(Lu(),rl)),n=Xkn(h(pt(e,(Cw(),pre)),380)),n.hg(e),t.n&&!!e&&mh(t,yh(e),(Lu(),rl))}function iEn(e,t,n){var s,a;for(eP(e,e.j+t,e.k+n),a=new Tn((!e.a&&(e.a=new Gi(Zl,e,5)),e.a));a.e!=a.i.gc();)s=h(On(a),469),kfe(s,s.a+t,s.b+n);JN(e,e.b+t,e.c+n)}function E2e(e,t,n,s){switch(n){case 7:return!e.e&&(e.e=new qt(us,e,7,4)),Xo(e.e,t,s);case 8:return!e.d&&(e.d=new qt(us,e,8,5)),Xo(e.d,t,s)}return aQ(e,t,n,s)}function _2e(e,t,n,s){switch(n){case 7:return!e.e&&(e.e=new qt(us,e,7,4)),qs(e.e,t,s);case 8:return!e.d&&(e.d=new qt(us,e,8,5)),qs(e.d,t,s)}return VK(e,t,n,s)}function sEn(e,t,n){var s,a,u,p,v;if(n)for(u=n.a.length,s=new Mp(u),v=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);v.Ob();)p=h(v.Pb(),19),a=wk(n,p.a),a&&f0t(e,a,t)}function zF(e,t,n){var s,a,u,p,v;return e.qj(),u=t==null?0:ii(t),e.f>0&&(p=(u&Ir)%e.d.length,a=xbe(e,p,u,t),a)?(v=a.ed(n),v):(s=e.tj(u,t,n),e.c.Fc(s),null)}function T2e(e,t){var n,s,a,u;switch(v0(e,t)._k()){case 3:case 2:{for(n=$y(t),a=0,u=n.i;a<u;++a)if(s=h(he(n,a),34),fb(Sa(e,s))==5)return s;break}}return null}function aEn(e){var t,n,s,a,u;if(JQe(e.f,e.b.length))for(s=pe(a3t,PM,330,e.b.length*2,0,1),e.b=s,a=s.length-1,n=e.a;n!=e;n=n.Rd())u=h(n,330),t=u.d&a,u.a=s[t],s[t]=u}function R1t(e,t){var n,s,a,u;for(u=0,a=h(h(jr(e.r,t),21),84).Kc();a.Ob();)s=h(a.Pb(),111),u=d.Math.max(u,s.e.a+s.b.rf().a);n=h(wa(e.b,t),124),n.n.b=0,n.a.a=u}function j1t(e,t){var n,s,a,u;for(n=0,u=h(h(jr(e.r,t),21),84).Kc();u.Ob();)a=h(u.Pb(),111),n=d.Math.max(n,a.e.b+a.b.rf().b);s=h(wa(e.b,t),124),s.n.d=0,s.a.b=n}function oEn(e){var t,n;return n=h(U(e,(He(),Uo)),21),t=QN(C8t),n.Hc((ha(),u3))&&qu(t,M8t),n.Hc(__)&&qu(t,D8t),n.Hc(n8)&&qu(t,S8t),n.Hc(r8)&&qu(t,A8t),t}function cEn(e,t){var n;Bn(t,"Delaunay triangulation",1),n=new qe,mc(e.i,new CWe(n)),ft(lt(U(e,(Bk(),p_)))),e.e?Qs(e.e,_bt(n)):e.e=_bt(n),An(t)}function C2e(e){if(e<0)throw ne(new an("The input must be positive"));return e<$8e.length?vb($8e[e]):d.Math.sqrt(Ky*e)*(z6n(e,e)/upe(2.718281828459045,e))}function ix(e,t){var n;if(e.ni()&&t!=null){for(n=0;n<e.i;++n)if(Nr(t,e.g[n]))return!0}else for(n=0;n<e.i;++n)if(Me(e.g[n])===Me(t))return!0;return!1}function uEn(e,t){if(t==null){for(;e.a.Ob();)if(h(e.a.Pb(),42).dd()==null)return!0}else for(;e.a.Ob();)if(Nr(t,h(e.a.Pb(),42).dd()))return!0;return!1}function lEn(e,t){var n,s,a;return t===e?!0:ue(t,664)?(a=h(t,1947),kft((s=e.g,s||(e.g=new gq(e))),(n=a.g,n||(a.g=new gq(a))))):!1}function hEn(e){var t,n,s,a;for(t="Sz",n="ez",a=d.Math.min(e.length,5),s=a-1;s>=0;s--)if(Nt(e[s].d,t)||Nt(e[s].d,n)){e.length>=s+1&&e.splice(0,s+1);break}return e}function oM(e,t){var n;return $a(e)&&$a(t)&&(n=e/t,RM<n&&n<Jp)?n<0?d.Math.ceil(n):d.Math.floor(n):xg(Ive($a(e)?Bl(e):e,$a(t)?Bl(t):t,!1))}function S2e(e,t){if(t==e.c.i)return e.d.i;if(t==e.d.i)return e.c.i;throw ne(new an("'node' must either be the source node or target node of the edge."))}function fEn(e){var t,n,s,a;if(a=h(U(e,(He(),p6e)),37),a){for(s=new Rs,t=Us(e.c.i);t!=a;)n=t.e,t=Us(n),nw(zr(zr(s,n.n),t.c),t.d.b,t.d.d);return s}return A5t}function dEn(e){var t;t=h(U(e,(He(),s8)),403),Ti(Xa(new Vt(null,new Wt(t.d,16)),new ARe),new xYe(e)),Ti(Jr(new Vt(null,new Wt(t.d,16)),new MRe),new EYe(e))}function eQ(e,t){var n,s,a,u;for(a=t?qi(e):za(e),s=new Sn(Dn(a.a.Kc(),new L));rr(s);)if(n=h(Un(s),17),u=S2e(n,e),u.k==(bn(),ds)&&u.c!=e.c)return u;return null}function gEn(e){var t,n,s;for(n=new Q(e.p);n.a<n.c.c.length;)t=h(K(n),10),t.k==(bn(),Wi)&&(s=t.o.b,e.i=d.Math.min(e.i,s),e.g=d.Math.max(e.g,s))}function $1t(e,t,n){var s,a,u;for(u=new Q(t);u.a<u.c.c.length;)s=h(K(u),10),e.c[s.c.p][s.p].e=!1;for(a=new Q(t);a.a<a.c.c.length;)s=h(K(a),10),Rve(e,s,n)}function tQ(e,t,n){var s,a;s=_y(t.j,n.s,n.c)+_y(n.e,t.s,t.c),a=_y(n.j,t.s,t.c)+_y(t.e,n.s,n.c),s==a?s>0&&(e.b+=2,e.a+=s):(e.b+=1,e.a+=d.Math.min(s,a))}function H1t(e,t){var n,s;if(s=!1,ws(t)&&(s=!0,z4(e,new uw(Jn(t)))),s||ue(t,236)&&(s=!0,z4(e,(n=L1e(h(t,236)),new WO(n)))),!s)throw ne(new Nq(Hye))}function pEn(e,t,n,s){var a,u,p;return a=new Sd(e.e,1,10,(p=t.c,ue(p,88)?h(p,26):(Bt(),nh)),(u=n.c,ue(u,88)?h(u,26):(Bt(),nh)),y0(e,t),!1),s?s.Ei(a):s=a,s}function A2e(e){var t,n;switch(h(U(Us(e),(tt(),bke)),420).g){case 0:return t=e.n,n=e.o,new dt(t.a+n.a/2,t.b+n.b/2);case 1:return new _a(e.n);default:return null}}function cM(){cM=re,D$=new rE(jd,0),t6e=new rE("LEFTUP",1),r6e=new rE("RIGHTUP",2),e6e=new rE("LEFTDOWN",3),n6e=new rE("RIGHTDOWN",4),sne=new rE("BALANCED",5)}function bEn(e,t,n){var s,a,u;if(s=Ui(e.a[t.p],e.a[n.p]),s==0){if(a=h(U(t,(He(),B5)),15),u=h(U(n,B5),15),a.Hc(n))return-1;if(u.Hc(t))return 1}return s}function vEn(e){switch(e.g){case 1:return new kHe;case 2:return new xHe;case 3:return new yHe;case 0:return null;default:throw ne(new an(JJ+(e.f!=null?e.f:""+e.g)))}}function M2e(e,t,n){switch(t){case 1:!e.n&&(e.n=new Ye(Ua,e,1,7)),jn(e.n),!e.n&&(e.n=new Ye(Ua,e,1,7)),mi(e.n,h(n,14));return;case 2:WE(e,Jn(n));return}Kge(e,t,n)}function D2e(e,t,n){switch(t){case 3:xb(e,Pe(Je(n)));return;case 4:Eb(e,Pe(Je(n)));return;case 5:yc(e,Pe(Je(n)));return;case 6:kc(e,Pe(Je(n)));return}M2e(e,t,n)}function GF(e,t,n){var s,a,u;u=(s=new Aq,s),a=Df(u,t,null),a&&a.Fi(),Ko(u,n),qn((!e.c&&(e.c=new Ye(dv,e,12,10)),e.c),u),p0(u,0),yw(u,1),m0(u,!0),w0(u,!0)}function L2e(e,t){var n,s,a;return n=Z7(e.g,t),ue(n,235)?(a=h(n,235),a.Qh()==null,a.Nh()):ue(n,498)?(s=h(n,1938),a=s.b,a):null}function wEn(e,t,n,s){var a,u;return Vn(t),Vn(n),u=h(_E(e.d,t),19),Mct(!!u,"Row %s not in %s",t,e.e),a=h(_E(e.b,n),19),Mct(!!a,"Column %s not in %s",n,e.c),Nlt(e,u.a,a.a,s)}function z1t(e,t,n,s,a,u,p){var v,m,x,_,A;if(_=a[u],x=u==p-1,v=x?s:0,A=u1t(v,_),s!=10&&se(ie(e,p-u),t[u],n[u],v,A),!x)for(++u,m=0;m<_;++m)A[m]=z1t(e,t,n,s,a,u,p);return A}function p9(e){if(e.g==-1)throw ne(new Ic);e.mj();try{e.i.$c(e.g),e.f=e.i.j,e.g<e.e&&--e.e,e.g=-1}catch(t){throw t=ui(t),ue(t,73)?ne(new cl):ne(t)}}function b9(e,t){return e.b.a=d.Math.min(e.b.a,t.c),e.b.b=d.Math.min(e.b.b,t.d),e.a.a=d.Math.max(e.a.a,t.c),e.a.b=d.Math.max(e.a.b,t.d),e.c[e.c.length]=t,!0}function mEn(e){var t,n,s,a;for(a=-1,s=0,n=new Q(e);n.a<n.c.c.length;){if(t=h(K(n),243),t.c==(ua(),Jc)){a=s==0?0:s-1;break}else s==e.c.length-1&&(a=s);s+=1}return a}function yEn(e){var t,n,s,a;for(a=0,t=0,s=new Q(e.c);s.a<s.c.c.length;)n=h(K(s),33),yc(n,e.e+a),kc(n,e.f),a+=n.g+e.b,t=d.Math.max(t,n.f+e.b);e.d=a-e.b,e.a=t-e.b}function Dy(e){var t,n,s;for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),57),s=t.d.c,t.d.c=t.d.d,t.d.d=s,s=t.d.b,t.d.b=t.d.a,t.d.a=s,s=t.b.a,t.b.a=t.b.b,t.b.b=s;qbe(e)}function Ly(e){var t,n,s;for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),81),s=t.g.c,t.g.c=t.g.d,t.g.d=s,s=t.g.b,t.g.b=t.g.a,t.g.a=s,s=t.e.a,t.e.a=t.e.b,t.e.b=s;nR(e)}function kEn(e){var t,n,s,a,u;for(u=$4(e.k),n=(Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])),s=0,a=n.length;s<a;++s)if(t=n[s],t!=to&&!u.Hc(t))return t;return null}function nQ(e,t){var n,s;return s=h(qfe(QK(Jr(new Vt(null,new Wt(t.j,16)),new t$e))),11),s&&(n=h(ut(s.e,0),17),n)?h(U(n,(He(),Eo)),19).a:X3n(e.b)}function xEn(e,t){var n,s,a,u;for(u=new Q(t.a);u.a<u.c.c.length;)for(a=h(K(u),10),H7(e.d),s=new Sn(Dn(qi(a).a.Kc(),new L));rr(s);)n=h(Un(s),17),F0t(e,a,n.d.i)}function EEn(e,t){var n,s;for(vc(e.b,t),s=new Q(e.n);s.a<s.c.c.length;)if(n=h(K(s),211),Ha(n.c,t,0)!=-1){vc(n.c,t),yEn(n),n.c.c.length==0&&vc(e.n,n);break}PLn(e)}function G1t(e,t){var n,s,a,u,p;for(p=e.f,a=0,u=0,s=new Q(e.a);s.a<s.c.c.length;)n=h(K(s),187),tM(n,e.e,p),mA(n,t),u=d.Math.max(u,n.r),p+=n.d+e.c,a=p;e.d=u,e.b=a}function V1t(e){var t,n;return n=wM(e),mk(n)?null:(t=(Vn(n),h(Jft(new Sn(Dn(n.a.Kc(),new L))),79)),Na(h(he((!t.b&&(t.b=new qt(Pn,t,4,7)),t.b),0),82)))}function VF(e){var t;return e.o||(t=e.Lj(),t?e.o=new Rde(e,e,null):e.rk()?e.o=new b1e(e,null):fb(Sa((jc(),Ps),e))==1?e.o=new Dat(e):e.o=new zY(e,null)),e.o}function _En(e,t,n,s){var a,u,p,v,m;n.mh(t)&&(a=(p=t,p?h(s,49).xh(p):null),a&&(m=n.ah(t),v=t.t,v>1||v==-1?(u=h(m,15),a.Wb(Txn(e,u))):a.Wb(HQ(e,h(m,56)))))}function TEn(e,t,n,s){SQe();var a=Uee;function u(){for(var p=0;p<a.length;p++)a[p]()}if(e)try{D_t(u)()}catch(p){e(t,p)}else D_t(u)()}function CEn(e){var t,n,s,a,u;for(s=new Gp(new xp(e.b).a);s.b;)n=kb(s),t=h(n.cd(),10),u=h(h(n.dd(),46).a,10),a=h(h(n.dd(),46).b,8),zr(Uh(t.n),zr(ro(u.n),a))}function SEn(e){switch(h(U(e.b,(tt(),uke)),375).g){case 1:Ti(pc(Xa(new Vt(null,new Wt(e.d,16)),new Aje),new Mje),new Dje);break;case 2:bMn(e);break;case 0:Q_n(e)}}function AEn(e,t,n){var s;Bn(n,"Straight Line Edge Routing",1),n.n&&!!t&&mh(n,yh(t),(Lu(),rl)),s=h(pt(t,(xE(),q_)),33),b2t(e,s),n.n&&!!t&&mh(n,yh(t),(Lu(),rl))}function Aw(){Aw=re,Ore=new cE("V_TOP",0),QD=new cE("V_CENTER",1),XD=new cE("V_BOTTOM",2),Ire=new cE("H_LEFT",3),WD=new cE("H_CENTER",4),KD=new cE("H_RIGHT",5)}function I2e(e){var t;return(e.Db&64)!=0?yF(e):(t=new Ol(yF(e)),t.a+=" (abstract: ",a0(t,(e.Bb&256)!=0),t.a+=", interface: ",a0(t,(e.Bb&512)!=0),t.a+=")",t.a)}function MEn(e,t,n,s){var a,u,p,v;return Tu(e.e)&&(a=t.ak(),v=t.dd(),u=n.dd(),p=wg(e,1,a,v,u,a.$j()?px(e,a,u,ue(a,99)&&(h(a,18).Bb&ea)!=0):-1,!0),s?s.Ei(p):s=p),s}function DEn(e){var t;e.c==null&&(t=Me(e.b)===Me(p3e)?null:e.b,e.d=t==null?_c:hrt(t)?h0n(Cit(t)):ws(t)?Qve:sg(uu(t)),e.a=e.a+": "+(hrt(t)?ign(Cit(t)):t+""),e.c="("+e.d+") "+e.a)}function O2e(e,t){this.e=e,eb(Zi(t,-4294967296),0)?(this.d=1,this.a=se(ie(Hn,1),hr,25,15,[Gn(t)])):(this.d=2,this.a=se(ie(Hn,1),hr,25,15,[Gn(t),Gn(dg(t,32))]))}function LEn(){function e(){try{return new Map().entries().next().done}catch{return!1}}return typeof Map===fZ&&Map.prototype.entries&&e()?Map:tNn()}function IEn(e,t){var n,s,a,u;for(u=new Ms(e.e,0),n=0;u.b<u.d.gc();){if(s=Pe((kn(u.b<u.d.gc()),Je(u.d.Xb(u.c=u.b++)))),a=s-t,a>XJ)return n;a>-1e-6&&++n}return n}function N2e(e,t){var n;t!=e.b?(n=null,e.b&&(n=tB(e.b,e,-4,n)),t&&(n=a5(t,e,-4,n)),n=_ht(e,t,n),n&&n.Fi()):(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,3,t,t))}function U1t(e,t){var n;t!=e.f?(n=null,e.f&&(n=tB(e.f,e,-1,n)),t&&(n=a5(t,e,-1,n)),n=Tht(e,t,n),n&&n.Fi()):(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,0,t,t))}function q1t(e){var t,n,s;if(e==null)return null;if(n=h(e,15),n.dc())return"";for(s=new s0,t=n.Kc();t.Ob();)va(s,(qr(),Jn(t.Pb()))),s.a+=" ";return TY(s,s.a.length-1)}function Y1t(e){var t,n,s;if(e==null)return null;if(n=h(e,15),n.dc())return"";for(s=new s0,t=n.Kc();t.Ob();)va(s,(qr(),Jn(t.Pb()))),s.a+=" ";return TY(s,s.a.length-1)}function OEn(e,t,n){var s,a;return s=e.c[t.c.p][t.p],a=e.c[n.c.p][n.p],s.a!=null&&a.a!=null?dW(s.a,a.a):s.a!=null?-1:a.a!=null?1:0}function NEn(e,t){var n,s,a,u,p,v;if(t)for(u=t.a.length,n=new Mp(u),v=(n.b-n.a)*n.c<0?(cg(),sp):new fg(n);v.Ob();)p=h(v.Pb(),19),a=wk(t,p.a),s=new qWe(e),Evn(s.a,a)}function PEn(e,t){var n,s,a,u,p,v;if(t)for(u=t.a.length,n=new Mp(u),v=(n.b-n.a)*n.c<0?(cg(),sp):new fg(n);v.Ob();)p=h(v.Pb(),19),a=wk(t,p.a),s=new FWe(e),xvn(s.a,a)}function BEn(e){var t;if(e!=null&&e.length>0&&Is(e,e.length-1)==33)try{return t=Vdt($u(e,0,e.length-1)),t.e==null}catch(n){if(n=ui(n),!ue(n,32))throw ne(n)}return!1}function W1t(e,t,n){var s,a,u;return s=t.ak(),u=t.dd(),a=s.$j()?wg(e,3,s,null,u,px(e,s,u,ue(s,99)&&(h(s,18).Bb&ea)!=0),!0):wg(e,1,s,s.zj(),u,-1,!0),n?n.Ei(a):n=a,n}function FEn(){var e,t,n;for(t=0,e=0;e<1;e++){if(n=Cbe((er(e,1),"X".charCodeAt(e))),n==0)throw ne(new Zn("Unknown Option: "+"X".substr(e)));t|=n}return t}function REn(e,t,n){var s,a,u;switch(s=Us(t),a=bF(s),u=new So,Ka(u,t),n.g){case 1:Ji(u,KA(i5(a)));break;case 2:Ji(u,i5(a))}return Fe(u,(tt(),om),Je(U(e,om))),u}function P2e(e){var t,n;return t=h(Un(new Sn(Dn(za(e.a).a.Kc(),new L))),17),n=h(Un(new Sn(Dn(qi(e.a).a.Kc(),new L))),17),ft(lt(U(t,(He(),Ff))))||ft(lt(U(n,Ff)))}function Iy(){Iy=re,x$=new AS("ONE_SIDE",0),_$=new AS("TWO_SIDES_CORNER",1),T$=new AS("TWO_SIDES_OPPOSING",2),E$=new AS("THREE_SIDES",3),k$=new AS("FOUR_SIDES",4)}function rQ(e,t,n,s,a){var u,p;u=h(Gu(Jr(t.Oc(),new Hje),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),p=h(jp(e.b,n,s),15),a==0?p.Wc(0,u):p.Gc(u)}function jEn(e,t){var n,s,a,u,p;for(u=new Q(t.a);u.a<u.c.c.length;)for(a=h(K(u),10),s=new Sn(Dn(za(a).a.Kc(),new L));rr(s);)n=h(Un(s),17),p=n.c.i.p,e.n[p]=e.n[p]-1}function $En(e,t){var n,s,a,u,p;for(u=new Q(t.d);u.a<u.c.c.length;)for(a=h(K(u),101),p=h(_n(e.c,a),112).o,s=new A4(a.b);s.a<s.c.a.length;)n=h(zE(s),61),q0e(a,n,p)}function HEn(e){var t,n;for(n=new Q(e.e.b);n.a<n.c.c.length;)t=h(K(n),29),vOn(e,t);Ti(Jr(Xa(Xa(new Vt(null,new Wt(e.e.b,16)),new M$e),new P$e),new B$e),new wWe(e))}function B2e(e,t){return t?e.Di(t)?!1:e.i?e.i.Ei(t):ue(t,143)?(e.i=h(t,143),!0):(e.i=new Bze,e.i.Ei(t)):!1}function zEn(e){if(e=Ho(e,!0),Nt(Nx,e)||Nt("1",e))return on(),Hx;if(Nt(hee,e)||Nt("0",e))return on(),u2;throw ne(new Fq("Invalid boolean value: '"+e+"'"))}function F2e(e,t,n){var s,a,u;for(a=e.vc().Kc();a.Ob();)if(s=h(a.Pb(),42),u=s.cd(),Me(t)===Me(u)||t!=null&&Nr(t,u))return n&&(s=new LN(s.cd(),s.dd()),a.Qb()),s;return null}function GEn(e){rw();var t,n,s;!e.B.Hc((fu(),RH))||(s=e.f.i,t=new eB(e.a.c),n=new E4,n.b=t.c-s.c,n.d=t.d-s.d,n.c=s.c+s.b-(t.c+t.b),n.a=s.d+s.a-(t.d+t.a),e.e.Ff(n))}function K1t(e,t,n,s){var a,u,p;for(p=d.Math.min(n,r2t(h(e.b,65),t,n,s)),u=new Q(e.a);u.a<u.c.c.length;)a=h(K(u),221),a!=t&&(p=d.Math.min(p,K1t(a,t,p,s)));return p}function R2e(e){var t,n,s,a;for(a=pe(X1,Re,193,e.b.c.length,0,2),s=new Ms(e.b,0);s.b<s.d.gc();)t=(kn(s.b<s.d.gc()),h(s.d.Xb(s.c=s.b++),29)),n=s.b-1,a[n]=hA(t.a);return a}function iQ(e,t,n,s,a){var u,p,v,m;for(p=a1n(yhe(R1e(p8n(n)),s),_7n(e,n,a)),m=x0(e,n).Kc();m.Ob();)v=h(m.Pb(),11),t[v.p]&&(u=t[v.p].i,Ue(p.d,new lW(u,wpe(p,u))));Npe(p)}function j2e(e,t){this.f=new zn,this.b=new zn,this.j=new zn,this.a=e,this.c=t,this.c>0&&y1t(this,this.c-1,(Qe(),pn)),this.c<this.a.length-1&&y1t(this,this.c+1,(Qe(),un))}function $2e(e){e.length>0&&e[0].length>0&&(this.c=ft(lt(U(Us(e[0][0]),(He(),w6e))))),this.a=pe(Zxt,Re,2018,e.length,0,2),this.b=pe(Jxt,Re,2019,e.length,0,2),this.d=new vht}function VEn(e){return e.c.length==0?!1:(Kt(0,e.c.length),h(e.c[0],17)).c.i.k==(bn(),ds)?!0:yk(pc(new Vt(null,new Wt(e,16)),new y$e),new k$e)}function UEn(e,t,n){return Bn(n,"Tree layout",1),hB(e.b),P1(e.b,(tx(),lH),lH),P1(e.b,H_,H_),P1(e.b,HD,HD),P1(e.b,z_,z_),e.a=vR(e.b,t),yAn(e,t,jo(n,1)),An(n),t}function X1t(e,t){var n,s,a,u,p,v,m;for(v=Nw(t),u=t.f,m=t.g,p=d.Math.sqrt(u*u+m*m),a=0,s=new Q(v);s.a<s.c.c.length;)n=h(K(s),33),a+=X1t(e,n);return d.Math.max(a,p)}function _s(){_s=re,Qg=new lE(j9,0),$f=new lE("FREE",1),k8=new lE("FIXED_SIDE",2),x2=new lE("FIXED_ORDER",3),ed=new lE("FIXED_RATIO",4),qo=new lE("FIXED_POS",5)}function qEn(e,t){var n,s,a;if(n=t.Hh(e.a),n){for(a=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),Ul)),s=1;s<(jc(),OEe).length;++s)if(Nt(OEe[s],a))return s}return 0}function YEn(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],Cd(u,""+t);return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function WEn(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],Cd(u,""+t);return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function Q1t(e){var t,n,s;for(s=new $p(Js,"{","}"),n=e.vc().Kc();n.Ob();)t=h(n.Pb(),42),Cd(s,_at(e,t.cd())+"="+_at(e,t.dd()));return s.a?s.e.length==0?s.a.a:s.a.a+(""+s.e):s.c}function KEn(e){for(var t,n,s,a;!$7(e.o);)n=h(H4(e.o),46),s=h(n.a,121),t=h(n.b,213),a=mF(t,s),t.e==s?(cP(a.g,t),s.e=a.e+t.a):(cP(a.b,t),s.e=a.e-t.a),Ue(e.e.a,s)}function H2e(e,t){var n,s,a;for(n=null,a=h(t.Kb(e),20).Kc();a.Ob();)if(s=h(a.Pb(),17),!n)n=s.c.i==e?s.d.i:s.c.i;else if((s.c.i==e?s.d.i:s.c.i)!=n)return!1;return!0}function Z1t(e,t){var n,s,a,u,p;for(n=wgt(e,!1,t),a=new Q(n);a.a<a.c.c.length;)s=h(K(a),129),s.d==0?(KW(s,null),XW(s,null)):(u=s.a,p=s.b,KW(s,p),XW(s,u))}function XEn(e){var t,n;return t=new rs,qu(t,O8t),n=h(U(e,(He(),Uo)),21),n.Hc((ha(),__))&&qu(t,F8t),n.Hc(n8)&&qu(t,N8t),n.Hc(u3)&&qu(t,B8t),n.Hc(r8)&&qu(t,P8t),t}function QEn(e){var t,n,s,a;for(mIn(e),n=new Sn(Dn(Id(e).a.Kc(),new L));rr(n);)t=h(Un(n),17),s=t.c.i==e,a=s?t.d:t.c,s?xs(t,null):Gs(t,null),Fe(t,(He(),x6e),a),aCn(e,a.i)}function ZEn(e,t,n,s){var a,u;switch(u=t.i,a=n[u.g][e.d[u.g]],u.g){case 1:a-=s+t.j.b,t.g.b=a;break;case 3:a+=s,t.g.b=a;break;case 4:a-=s+t.j.a,t.g.a=a;break;case 2:a+=s,t.g.a=a}}function JEn(e){var t,n,s;for(n=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));n.e!=n.i.gc();)if(t=h(On(n),33),s=wM(t),!rr(new Sn(Dn(s.a.Kc(),new L))))return t;return null}function e9n(){var e;return S9t?h(lx((og(),th),Bx),2016):(e=h(ue(Fo((og(),th),Bx),555)?Fo(th,Bx):new H0t,555),S9t=!0,iOn(e),fPn(e),QX(e),Ta(th,Bx,e),e)}function sQ(e,t,n){var s,a;if(e.j==0)return n;if(a=h(kut(e,t,n),72),s=n.ak(),!s.Ij()||!e.a.rl(s))throw ne(new Ya("Invalid entry feature '"+s.Hj().zb+"."+s.ne()+"'"));return a}function t9n(e,t){var n,s,a,u,p,v,m,x;for(v=e.a,m=0,x=v.length;m<x;++m)for(p=v[m],s=p,a=0,u=s.length;a<u;++a)if(n=s[a],Me(t)===Me(n)||t!=null&&Nr(t,n))return!0;return!1}function n9n(e){var t,n,s;return mo(e,0)>=0?(n=oM(e,SR),s=ZA(e,SR)):(t=ow(e,1),n=oM(t,5e8),s=ZA(t,5e8),s=Vs(kd(s,1),Zi(e,1))),xf(kd(s,32),Zi(n,fa))}function J1t(e,t,n){var s,a;switch(s=(kn(t.b!=0),h(pl(t,t.a.a),8)),n.g){case 0:s.b=0;break;case 2:s.b=e.f;break;case 3:s.a=0;break;default:s.a=e.g}return a=pr(t,0),nA(a,s),t}function edt(e,t,n,s){var a,u,p,v,m;switch(m=e.b,u=t.d,p=u.j,v=Kpe(p,m.d[p.g],n),a=zr(ro(u.n),u.a),u.j.g){case 1:case 3:v.a+=a.a;break;case 2:case 4:v.b+=a.b}Ai(s,v,s.c.b,s.c)}function r9n(e,t,n){var s,a,u,p;for(p=Ha(e.e,t,0),u=new Jle,u.b=n,s=new Ms(e.e,p);s.b<s.d.gc();)a=(kn(s.b<s.d.gc()),h(s.d.Xb(s.c=s.b++),10)),a.p=n,Ue(u.e,a),Mu(s);return u}function i9n(e,t,n,s){var a,u,p,v,m;for(a=null,u=0,v=new Q(t);v.a<v.c.c.length;)p=h(K(v),33),m=p.i+p.g,e<p.j+p.f+s&&(a?n.i-m<n.i-u&&(a=p):a=p,u=a.i+a.g);return a?u+s:0}function s9n(e,t,n,s){var a,u,p,v,m;for(u=null,a=0,v=new Q(t);v.a<v.c.c.length;)p=h(K(v),33),m=p.j+p.f,e<p.i+p.g+s&&(u?n.j-m<n.j-a&&(u=p):u=p,a=u.j+u.f);return u?a+s:0}function a9n(e){var t,n,s;for(t=!1,s=e.b.c.length,n=0;n<s;n++)Wge(h(ut(e.b,n),434))?!t&&n+1<s&&Wge(h(ut(e.b,n+1),434))&&(t=!0,h(ut(e.b,n),434).a=!0):t=!1}function o9n(e,t,n,s,a){var u,p;for(u=0,p=0;p<a;p++)u=Vs(u,Cg(Zi(t[p],fa),Zi(s[p],fa))),e[p]=Gn(u),u=dg(u,32);for(;p<n;p++)u=Vs(u,Zi(t[p],fa)),e[p]=Gn(u),u=dg(u,32)}function c9n(e,t){cx();var n,s;for(s=(Dg(),Fj),n=e;t>1;t>>=1)(t&1)!=0&&(s=vy(s,n)),n.d==1?n=vy(n,n):n=new _ft(Opt(n.a,n.d,pe(Hn,hr,25,n.d<<1,15,1)));return s=vy(s,n),s}function z2e(){z2e=re;var e,t,n,s;for(V3e=pe(ks,ka,25,25,15,1),U3e=pe(ks,ka,25,33,15,1),s=152587890625e-16,t=32;t>=0;t--)U3e[t]=s,s*=.5;for(n=1,e=24;e>=0;e--)V3e[e]=n,n*=.5}function u9n(e){var t,n;if(ft(lt(pt(e,(tt(),am))))){for(n=new Sn(Dn(Pd(e).a.Kc(),new L));rr(n);)if(t=h(Un(n),79),Pb(t)&&ft(lt(pt(t,d2))))return!0}return!1}function tdt(e,t){var n,s,a;Qi(e.f,t)&&(t.b=e,s=t.c,Ha(e.j,s,0)!=-1||Ue(e.j,s),a=t.d,Ha(e.j,a,0)!=-1||Ue(e.j,a),n=t.a.b,n.c.length!=0&&(!e.i&&(e.i=new Oft(e)),T5n(e.i,n)))}function l9n(e){var t,n,s,a,u;return n=e.c.d,s=n.j,a=e.d.d,u=a.j,s==u?n.p<a.p?0:1:pF(s)==u?0:mpe(s)==u?1:(t=e.b,Pc(t.b,pF(s))?0:1)}function UF(){UF=re,Rne=new sE(gwt,0),Gke=new sE("LONGEST_PATH",1),Fne=new sE("COFFMAN_GRAHAM",2),zke=new sE(vJ,3),Vke=new sE("STRETCH_WIDTH",4),tH=new sE("MIN_WIDTH",5)}function Xp(e){var t;this.d=new zn,this.c=e.c,this.e=e.d,this.b=e.b,this.f=new Qnt(e.e),this.a=e.a,e.f?this.g=e.f:this.g=(t=h(Vh(WH),9),new ll(t,h(vh(t,t.length),9),0))}function qF(e,t){var n,s,a,u,p,v;a=e,p=HE(a,"layoutOptions"),!p&&(p=HE(a,gmt)),p&&(v=p,s=null,v&&(s=(u=HK(v,pe(it,Re,2,0,6,1)),new Uq(v,u))),s&&(n=new lJe(v,t),Os(s,n)))}function Na(e){if(ue(e,239))return h(e,33);if(ue(e,186))return mf(h(e,118));throw ne(e?new i0("Only support nodes and ports."):new T4(Emt))}function h9n(e,t,n,s){return(t>=0&&Nt(e.substr(t,3),"GMT")||t>=0&&Nt(e.substr(t,3),"UTC"))&&(n[0]=t+3),pve(e,n,s)}function f9n(e,t){var n,s,a,u,p;for(u=e.g.a,p=e.g.b,s=new Q(e.d);s.a<s.c.c.length;)n=h(K(s),70),a=n.n,a.a=u,e.i==(Qe(),sn)?a.b=p+e.j.b-n.o.b:a.b=p,zr(a,t),u+=n.o.a+e.e}function Bn(e,t,n){if(e.b)throw ne(new ja("The task is already done."));return e.p!=null?!1:(e.p=t,e.r=n,e.k&&(e.o=(L1(),js(xc(Date.now()),_0))),!0)}function G2e(e){var t,n,s,a,u,p,v;return v=new _4,n=e.tg(),a=n!=null,a&&ok(v,p1,e.tg()),s=e.ne(),u=s!=null,u&&ok(v,_r,e.ne()),t=e.sg(),p=t!=null,p&&ok(v,"description",e.sg()),v}function ndt(e,t,n){var s,a,u;return u=e.q,e.q=t,(e.Db&4)!=0&&(e.Db&1)==0&&(a=new fs(e,1,9,u,t),n?n.Ei(a):n=a),t?(s=t.c,s!=e.r&&(n=e.nk(s,n))):e.r&&(n=e.nk(null,n)),n}function d9n(e,t,n){var s,a,u,p,v;for(n=(v=t,a5(v,e.e,-1-e.c,n)),p=$de(e.a),u=(s=new Gp(new xp(p.a).a),new nN(s));u.a.b;)a=h(kb(u.a).cd(),87),n=wx(a,eR(a,e.a),n);return n}function g9n(e,t,n){var s,a,u,p,v;for(n=(v=t,tB(v,e.e,-1-e.c,n)),p=$de(e.a),u=(s=new Gp(new xp(p.a).a),new nN(s));u.a.b;)a=h(kb(u.a).cd(),87),n=wx(a,eR(a,e.a),n);return n}function p9n(e,t,n,s){var a,u,p;if(s==0)Ao(t,0,e,n,e.length-n);else for(p=32-s,e[e.length-1]=0,u=e.length-1;u>n;u--)e[u]|=t[u-n-1]>>>p,e[u-1]=t[u-n-1]<<s;for(a=0;a<n;a++)e[a]=0}function b9n(e){var t,n,s,a,u;for(t=0,n=0,u=e.Kc();u.Ob();)s=h(u.Pb(),111),t=d.Math.max(t,s.d.b),n=d.Math.max(n,s.d.c);for(a=e.Kc();a.Ob();)s=h(a.Pb(),111),s.d.b=t,s.d.c=n}function v9n(e){var t,n,s,a,u;for(n=0,t=0,u=e.Kc();u.Ob();)s=h(u.Pb(),111),n=d.Math.max(n,s.d.d),t=d.Math.max(t,s.d.a);for(a=e.Kc();a.Ob();)s=h(a.Pb(),111),s.d.d=n,s.d.a=t}function rdt(e,t){var n,s,a,u;for(u=new qe,a=0,s=t.Kc();s.Ob();){for(n=Ke(h(s.Pb(),19).a+a);n.a<e.f&&!obn(e,n.a);)n=Ke(n.a+1),++a;if(n.a>=e.f)break;u.c[u.c.length]=n}return u}function V2e(e){var t,n,s,a;for(t=null,a=new Q(e.wf());a.a<a.c.c.length;)s=h(K(a),181),n=new hl(s.qf().a,s.qf().b,s.rf().a,s.rf().b),t?v9(t,n):t=n;return!t&&(t=new O4),t}function aQ(e,t,n,s){var a,u;return n==1?(!e.n&&(e.n=new Ye(Ua,e,1,7)),Xo(e.n,t,s)):(u=h(Ht((a=h(tn(e,16),26),a||e.zh()),n),66),u.Nj().Qj(e,ic(e),n-En(e.zh()),t,s))}function oQ(e,t,n){var s,a,u,p,v;for(s=n.gc(),e.qi(e.i+s),v=e.i-t,v>0&&Ao(e.g,t,e.g,t+s,v),p=n.Kc(),e.i+=s,a=0;a<s;++a)u=p.Pb(),wE(e,t,e.oi(t,u)),e.bi(t,u),e.ci(),++t;return s!=0}function Df(e,t,n){var s;return t!=e.q?(e.q&&(n=tB(e.q,e,-10,n)),t&&(n=a5(t,e,-10,n)),n=ndt(e,t,n)):(e.Db&4)!=0&&(e.Db&1)==0&&(s=new fs(e,1,9,t,t),n?n.Ei(s):n=s),n}function cQ(e,t,n,s){return O1e((n&a1)==0,"flatMap does not support SUBSIZED characteristic"),O1e((n&4)==0,"flatMap does not support SORTED characteristic"),Vn(e),Vn(t),new cat(e,n,s,t)}function w9n(e,t){Ade(t,"Cannot suppress a null exception."),mP(t!=e,"Exception can not suppress itself."),!e.i&&(e.k==null?e.k=se(ie(Zee,1),Re,78,0,[t]):e.k[e.k.length]=t)}function Oy(e,t,n,s){var a,u,p,v,m,x;for(p=n.length,u=0,a=-1,x=Qct(e.substr(t),(KY(),z3e)),v=0;v<p;++v)m=n[v].length,m>u&&Abn(x,Qct(n[v],z3e))&&(a=v,u=m);return a>=0&&(s[0]=t+u),a}function m9n(e,t){var n;if(n=qJe(e.b.Hf(),t.b.Hf()),n!=0)return n;switch(e.b.Hf().g){case 1:case 2:return dc(e.b.sf(),t.b.sf());case 3:case 4:return dc(t.b.sf(),e.b.sf())}return 0}function y9n(e){var t,n,s;for(s=e.e.c.length,e.a=Ap(Hn,[Re,hr],[48,25],15,[s,s],2),n=new Q(e.c);n.a<n.c.c.length;)t=h(K(n),282),e.a[t.c.b][t.d.b]+=h(U(t,(Jh(),qx)),19).a}function k9n(e,t,n){Bn(n,"Grow Tree",1),e.b=t.f,ft(lt(U(t,(Bk(),p_))))?(e.c=new Bi,yst(e,null)):e.c=new Bi,e.a=!1,Egt(e,t.f),Fe(t,x4e,(on(),!!e.a)),An(n)}function x9n(e,t){var n,s,a,u,p;if(e==null)return null;for(p=pe(Cl,o1,25,2*t,15,1),s=0,a=0;s<t;++s)n=e[s]>>4&15,u=e[s]&15,p[a++]=oEe[n],p[a++]=oEe[u];return Fl(p,0,p.length)}function E9n(e,t,n){var s,a,u;return s=t.ak(),u=t.dd(),a=s.$j()?wg(e,4,s,u,null,px(e,s,u,ue(s,99)&&(h(s,18).Bb&ea)!=0),!0):wg(e,s.Kj()?2:1,s,u,s.zj(),-1,!0),n?n.Ei(a):n=a,n}function Ec(e){var t,n;return e>=ea?(t=jM+(e-ea>>10&1023)&Oi,n=56320+(e-ea&1023)&Oi,String.fromCharCode(t)+(""+String.fromCharCode(n))):String.fromCharCode(e&Oi)}function _9n(e,t){rw();var n,s,a,u;return a=h(h(jr(e.r,t),21),84),a.gc()>=2?(s=h(a.Kc().Pb(),111),n=e.u.Hc((Qc(),sT)),u=e.u.Hc(K5),!s.a&&!n&&(a.gc()==2||u)):!1}function idt(e,t,n,s,a){var u,p,v;for(u=Hgt(e,t,n,s,a),v=!1;!u;)ZF(e,a,!0),v=!0,u=Hgt(e,t,n,s,a);v&&ZF(e,a,!1),p=qK(a),p.c.length!=0&&(e.d&&e.d.lg(p),idt(e,a,n,s,p))}function YF(){YF=re,Wre=new uE(jd,0),P7e=new uE("DIRECTED",1),F7e=new uE("UNDIRECTED",2),O7e=new uE("ASSOCIATION",3),B7e=new uE("GENERALIZATION",4),N7e=new uE("DEPENDENCY",5)}function T9n(e,t){var n;if(!mf(e))throw ne(new ja(emt));switch(n=mf(e),t.g){case 1:return-(e.j+e.f);case 2:return e.i-n.g;case 3:return e.j-n.f;case 4:return-(e.i+e.g)}return 0}function sx(e,t){var n,s;for(rn(t),s=e.b.c.length,Ue(e.b,t);s>0;){if(n=s,s=(s-1)/2|0,e.a.ue(ut(e.b,s),t)<=0)return dl(e.b,n,t),!0;dl(e.b,n,ut(e.b,s))}return dl(e.b,s,t),!0}function U2e(e,t,n,s){var a,u;if(a=0,n)a=hF(e.a[n.g][t.g],s);else for(u=0;u<Vj;u++)a=d.Math.max(a,hF(e.a[u][t.g],s));return t==(Kh(),Jo)&&!!e.b&&(a=d.Math.max(a,e.b.a)),a}function C9n(e,t){var n,s,a,u,p,v;return a=e.i,u=t.i,!a||!u||a.i!=u.i||a.i==(Qe(),pn)||a.i==(Qe(),un)?!1:(p=a.g.a,n=p+a.j.a,v=u.g.a,s=v+u.j.a,p<=s&&n>=v)}function q2e(e,t,n,s){var a;if(a=!1,ws(s)&&(a=!0,ok(t,n,Jn(s))),a||Jv(s)&&(a=!0,q2e(e,t,n,s)),a||ue(s,236)&&(a=!0,Lp(t,n,h(s,236))),!a)throw ne(new Nq(Hye))}function S9n(e,t){var n,s,a;if(n=t.Hh(e.a),n&&(a=Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),Gl),a!=null)){for(s=1;s<(jc(),LEe).length;++s)if(Nt(LEe[s],a))return s}return 0}function A9n(e,t){var n,s,a;if(n=t.Hh(e.a),n&&(a=Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),Gl),a!=null)){for(s=1;s<(jc(),IEe).length;++s)if(Nt(IEe[s],a))return s}return 0}function sdt(e,t){var n,s,a,u;if(rn(t),u=e.a.gc(),u<t.gc())for(n=e.a.ec().Kc();n.Ob();)s=n.Pb(),t.Hc(s)&&n.Qb();else for(a=t.Kc();a.Ob();)s=a.Pb(),e.a.Bc(s)!=null;return u!=e.a.gc()}function adt(e){var t,n;switch(n=ro(Qa(se(ie(cs,1),Re,8,0,[e.i.n,e.n,e.a]))),t=e.i.d,e.j.g){case 1:n.b-=t.d;break;case 2:n.a+=t.c;break;case 3:n.b+=t.a;break;case 4:n.a-=t.b}return n}function M9n(e){var t;for(t=(t5(),h(Un(new Sn(Dn(za(e).a.Kc(),new L))),17).c.i);t.k==(bn(),ds);)Fe(t,(He(),MD),(on(),!0)),t=h(Un(new Sn(Dn(za(t).a.Kc(),new L))),17).c.i}function uQ(e,t,n,s){var a,u,p,v;for(v=r9(t,s),p=v.Kc();p.Ob();)a=h(p.Pb(),11),e.d[a.p]=e.d[a.p]+e.c[n.p];for(v=r9(n,s),u=v.Kc();u.Ob();)a=h(u.Pb(),11),e.d[a.p]=e.d[a.p]-e.c[t.p]}function Y2e(e,t,n){var s,a;for(a=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));a.e!=a.i.gc();)s=h(On(a),33),bf(s,s.i+t,s.j+n);Os((!e.b&&(e.b=new Ye(us,e,12,3)),e.b),new JZe(t,n))}function D9n(e,t,n,s){var a,u;for(u=t,a=u.d==null||e.a.ue(n.d,u.d)>0?1:0;u.a[a]!=n;)u=u.a[a],a=e.a.ue(n.d,u.d)>0?1:0;u.a[a]=s,s.b=n.b,s.a[0]=n.a[0],s.a[1]=n.a[1],n.a[0]=null,n.a[1]=null}function L9n(e){Qc();var t,n;return t=ni(Ud,se(ie(FH,1),ze,273,0,[Zg])),!(NA(iB(t,e))>1||(n=ni(sT,se(ie(FH,1),ze,273,0,[iT,K5])),NA(iB(n,e))>1))}function W2e(e,t){var n;n=Fo((og(),th),e),ue(n,498)?Ta(th,e,new xJe(this,t)):Ta(th,e,this),lQ(this,t),t==(q6(),mEe)?(this.wb=h(this,1939),h(t,1941)):this.wb=(bg(),Xt)}function I9n(e){var t,n,s;if(e==null)return null;for(t=null,n=0;n<uT.length;++n)try{return fZe(uT[n],e)}catch(a){if(a=ui(a),ue(a,32))s=a,t=s;else throw ne(a)}throw ne(new MB(t))}function odt(){odt=re,w3t=se(ie(it,1),Re,2,6,["Sun","Mon","Tue","Wed","Thu","Fri","Sat"]),m3t=se(ie(it,1),Re,2,6,["Jan","Feb","Mar","Apr",p5,"Jun","Jul","Aug","Sep","Oct","Nov","Dec"])}function cdt(e){var t,n,s;t=Nt(typeof t,YZ)?null:new $r,t&&(Q6(),n=(s=900,s>=_0?"error":s>=900?"warn":s>=800?"info":"log"),Urt(n,e.a),e.b&&Wbe(t,n,e.b,"Exception: ",!0))}function U(e,t){var n,s;return s=(!e.q&&(e.q=new zn),_n(e.q,t)),s!=null?s:(n=t.wg(),ue(n,4)&&(n==null?(!e.q&&(e.q=new zn),Q4(e.q,t)):(!e.q&&(e.q=new zn),Pr(e.q,t,n))),n)}function Zs(){Zs=re,w1=new SS("P1_CYCLE_BREAKING",0),Hg=new SS("P2_LAYERING",1),oc=new SS("P3_NODE_ORDERING",2),Go=new SS("P4_NODE_PLACEMENT",3),Pa=new SS("P5_EDGE_ROUTING",4)}function udt(e,t){var n,s,a,u,p;for(a=t==1?Mte:Ate,s=a.a.ec().Kc();s.Ob();)for(n=h(s.Pb(),103),p=h(jr(e.f.c,n),21).Kc();p.Ob();)u=h(p.Pb(),46),vc(e.b.b,u.b),vc(e.b.a,h(u.b,81).d)}function O9n(e,t){qE();var n;if(e.c==t.c){if(e.b==t.b||i5n(e.b,t.b)){if(n=ddn(e.b)?1:-1,e.a&&!t.a)return n;if(!e.a&&t.a)return-n}return dc(e.b.g,t.b.g)}else return Ui(e.c,t.c)}function N9n(e,t){var n;Bn(t,"Hierarchical port position processing",1),n=e.b,n.c.length>0&&Apt((Kt(0,n.c.length),h(n.c[0],29)),e),n.c.length>1&&Apt(h(ut(n,n.c.length-1),29),e),An(t)}function ldt(e,t){var n,s,a;if(X2e(e,t))return!0;for(s=new Q(t);s.a<s.c.c.length;)if(n=h(K(s),33),a=V1t(n),aR(e,n,a)||wht(e,n)-e.g<=e.a)return!0;return!1}function uM(){uM=re,TH=(NQ(),D8e),Sre=gEt,Cre=dEt,E8e=lEt,Tre=fEt,x8e=new nb(8),rEt=new sa((Tr(),y2),x8e),iEt=new sa(k2,8),sEt=A8e,y8e=aEt,k8e=oEt,nEt=new sa(ZD,(on(),!1))}function WF(){WF=re,Y8e=new nb(15),LEt=new sa((Tr(),y2),Y8e),IEt=new sa(k2,15),W8e=new sa(NH,Ke(0)),V8e=i7e,MEt=m2,DEt=Wg,G8e=new sa(U5,Nwt),U8e=LH,q8e=_3,Lre=BEt,AEt=DH}function H1(e){if((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i!=1||(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c).i!=1)throw ne(new an(_ee));return Na(h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82))}function hdt(e){if((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i!=1||(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c).i!=1)throw ne(new an(_ee));return jA(h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82))}function fdt(e){if((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i!=1||(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c).i!=1)throw ne(new an(_ee));return jA(h(he((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c),0),82))}function Sg(e){if((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i!=1||(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c).i!=1)throw ne(new an(_ee));return Na(h(he((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c),0),82))}function K2e(e,t,n){var s,a,u;if(++e.j,a=e.Vi(),t>=a||t<0)throw ne(new Ea(Tee+t+s2+a));if(n>=a||n<0)throw ne(new Ea(Cee+n+s2+a));return t!=n?s=(u=e.Ti(n),e.Hi(t,u),u):s=e.Oi(n),s}function ddt(e){var t,n,s;if(s=e,e)for(t=0,n=e.Ug();n;n=n.Ug()){if(++t>jZ)return ddt(n);if(s=n,n==e)throw ne(new ja("There is a cycle in the containment hierarchy of "+e))}return s}function Ag(e){var t,n,s;for(s=new $p(Js,"[","]"),n=e.Kc();n.Ob();)t=n.Pb(),Cd(s,Me(t)===Me(e)?"(this Collection)":t==null?_c:Ga(t));return s.a?s.e.length==0?s.a.a:s.a.a+(""+s.e):s.c}function X2e(e,t){var n,s;if(s=!1,t.gc()<2)return!1;for(n=0;n<t.gc();n++)n<t.gc()-1?s=s|aR(e,h(t.Xb(n),33),h(t.Xb(n+1),33)):s=s|aR(e,h(t.Xb(n),33),h(t.Xb(0),33));return s}function gdt(e,t){var n;t!=e.a?(n=null,e.a&&(n=h(e.a,49).ih(e,4,sf,n)),t&&(n=h(t,49).gh(e,4,sf,n)),n=cpe(e,t,n),n&&n.Fi()):(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,1,t,t))}function Q2e(e,t){var n;t!=e.e?(e.e&&Eot($de(e.e),e),t&&(!t.b&&(t.b=new tN(new Mq)),Ftt(t.b,e)),n=U7n(e,t,null),n&&n.Fi()):(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,4,t,t))}function Mw(e){var t,n,s;for(n=e.length,s=0;s<n&&(er(s,e.length),e.charCodeAt(s)<=32);)++s;for(t=n;t>s&&(er(t-1,e.length),e.charCodeAt(t-1)<=32);)--t;return s>0||t<n?e.substr(s,t-s):e}function P9n(e,t){var n;n=t.o,lg(e.f)?(e.j.a=d.Math.max(e.j.a,n.a),e.j.b+=n.b,e.d.c.length>1&&(e.j.b+=e.e)):(e.j.a+=n.a,e.j.b=d.Math.max(e.j.b,n.b),e.d.c.length>1&&(e.j.a+=e.e))}function Mg(){Mg=re,W5t=se(ie(ta,1),yo,61,0,[(Qe(),sn),pn,Fn]),Y5t=se(ie(ta,1),yo,61,0,[pn,Fn,un]),K5t=se(ie(ta,1),yo,61,0,[Fn,un,sn]),X5t=se(ie(ta,1),yo,61,0,[un,sn,pn])}function B9n(e,t,n,s){var a,u,p,v,m,x,_;if(p=e.c.d,v=e.d.d,p.j!=v.j)for(_=e.b,a=p.j,m=null;a!=v.j;)m=t==0?pF(a):mpe(a),u=Kpe(a,_.d[a.g],n),x=Kpe(m,_.d[m.g],n),vr(s,zr(u,x)),a=m}function F9n(e,t,n,s){var a,u,p,v,m;return p=k1t(e.a,t,n),v=h(p.a,19).a,u=h(p.b,19).a,s&&(m=h(U(t,(He(),Zc)),10),a=h(U(n,Zc),10),!!m&&!!a&&(Sat(e.b,m,a),v+=e.b.i,u+=e.b.e)),v>u}function pdt(e){var t,n,s,a,u,p,v,m,x;for(this.a=Vft(e),this.b=new qe,n=e,s=0,a=n.length;s<a;++s)for(t=n[s],u=new qe,Ue(this.b,u),v=t,m=0,x=v.length;m<x;++m)p=v[m],Ue(u,new Bc(p.j))}function R9n(e,t,n){var s,a,u;return u=0,s=n[t],t<n.length-1&&(a=n[t+1],e.b[t]?(u=uPn(e.d,s,a),u+=MW(e.a,s,(Qe(),pn)),u+=MW(e.a,a,un)):u=M3n(e.a,s,a)),e.c[t]&&(u+=Zyn(e.a,s)),u}function j9n(e,t,n,s,a){var u,p,v,m;for(m=null,v=new Q(s);v.a<v.c.c.length;)if(p=h(K(v),441),p!=n&&Ha(p.e,a,0)!=-1){m=p;break}u=fK(a),Gs(u,n.b),xs(u,m.b),Ot(e.a,a,new vP(u,t,n.f))}function bdt(e){for(;e.g.c!=0&&e.d.c!=0;)HY(e.g).c>HY(e.d).c?(e.i+=e.g.c,CX(e.d)):HY(e.d).c>HY(e.g).c?(e.e+=e.d.c,CX(e.g)):(e.i+=qnt(e.g),e.e+=qnt(e.d),CX(e.g),CX(e.d))}function $9n(e,t,n){var s,a,u,p;for(u=t.q,p=t.r,new Ip((qh(),qg),t,u,1),new Ip(qg,u,p,1),a=new Q(n);a.a<a.c.c.length;)s=h(K(a),112),s!=u&&s!=t&&s!=p&&(Mve(e.a,s,t),Mve(e.a,s,p))}function vdt(e,t,n,s){e.a.d=d.Math.min(t,n),e.a.a=d.Math.max(t,s)-e.a.d,t<n?(e.b=.5*(t+n),e.g=QJ*e.b+.9*t,e.f=QJ*e.b+.9*n):(e.b=.5*(t+s),e.g=QJ*e.b+.9*s,e.f=QJ*e.b+.9*t)}function H9n(){Oj={},!Array.isArray&&(Array.isArray=function(t){return Object.prototype.toString.call(t)==="[object Array]"});function e(){return new Date().getTime()}!Date.now&&(Date.now=e)}function wdt(e,t){var n,s;s=h(U(t,(tt(),Ei)),98),Fe(t,(He(),E6e),s),n=t.e,n&&(Ti(new Vt(null,new Wt(n.a,16)),new Dle(e)),Ti(Xa(new Vt(null,new Wt(n.b,16)),new $O),new Lle(e)))}function z9n(e){var t,n,s,a;if(ek(h(U(e.b,(tt(),Ju)),103)))return 0;for(t=0,s=new Q(e.a);s.a<s.c.c.length;)n=h(K(s),10),n.k==(bn(),Wi)&&(a=n.o.a,t=d.Math.max(t,a));return t}function G9n(e){switch(h(U(e,(tt(),cc)),163).g){case 1:Fe(e,cc,(wl(),S_));break;case 2:Fe(e,cc,(wl(),nm));break;case 3:Fe(e,cc,(wl(),C_));break;case 4:Fe(e,cc,(wl(),Gg))}}function ax(){ax=re,x_=new iE(jd,0),a6e=new iE(w5,1),u6e=new iE(m5,2),c6e=new iE("LEFT_RIGHT_CONSTRAINT_LOCKING",3),o6e=new iE("LEFT_RIGHT_CONNECTION_LOCKING",4),s6e=new iE(twt,5)}function mdt(e,t,n){var s,a,u,p,v,m,x;v=n.a/2,u=n.b/2,s=d.Math.abs(t.a-e.a),a=d.Math.abs(t.b-e.b),m=1,x=1,s>v&&(m=v/s),a>u&&(x=u/a),p=d.Math.min(m,x),e.a+=p*(t.a-e.a),e.b+=p*(t.b-e.b)}function V9n(e,t,n,s,a){var u,p;for(p=!1,u=h(ut(n.b,0),33);sLn(e,t,u,s,a)&&(p=!0,EEn(n,u),n.b.c.length!=0);)u=h(ut(n.b,0),33);return n.b.c.length==0&&aM(n.j,n),p&&RF(t.q),p}function U9n(e,t){Ry();var n,s,a,u;if(t.b<2)return!1;for(u=pr(t,0),n=h(gr(u),8),s=n;u.b!=u.d.c;){if(a=h(gr(u),8),MQ(e,s,a))return!0;s=a}return!!MQ(e,s,n)}function Z2e(e,t,n,s){var a,u;return n==0?(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),pP(e.o,t,s)):(u=h(Ht((a=h(tn(e,16),26),a||e.zh()),n),66),u.Nj().Rj(e,ic(e),n-En(e.zh()),t,s))}function lQ(e,t){var n;t!=e.sb?(n=null,e.sb&&(n=h(e.sb,49).ih(e,1,cT,n)),t&&(n=h(t,49).gh(e,1,cT,n)),n=fpe(e,t,n),n&&n.Fi()):(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,4,t,t))}function q9n(e,t){var n,s,a,u;if(t)a=Md(t,"x"),n=new GWe(e),Ak(n.a,(rn(a),a)),u=Md(t,"y"),s=new VWe(e),Dk(s.a,(rn(u),u));else throw ne(new Xf("All edge sections need an end point."))}function Y9n(e,t){var n,s,a,u;if(t)a=Md(t,"x"),n=new $We(e),Mk(n.a,(rn(a),a)),u=Md(t,"y"),s=new HWe(e),Lk(s.a,(rn(u),u));else throw ne(new Xf("All edge sections need a start point."))}function W9n(e,t){var n,s,a,u,p,v,m;for(s=tht(e),u=0,v=s.length;u<v;++u)cdt(t);for(m=!Pf&&e.e?Pf?null:e.d:null;m;){for(n=tht(m),a=0,p=n.length;a<p;++a)cdt(t);m=!Pf&&m.e?Pf?null:m.d:null}}function bn(){bn=re,Wi=new eE("NORMAL",0),ds=new eE("LONG_EDGE",1),Pi=new eE("EXTERNAL_PORT",2),Vo=new eE("NORTH_SOUTH_PORT",3),Ou=new eE("LABEL",4),Bf=new eE("BREAKING_POINT",5)}function K9n(e){var t,n,s,a;if(t=!1,os(e,(He(),T_)))for(n=h(U(e,T_),83),a=new Q(e.j);a.a<a.c.c.length;)s=h(K(a),11),ZCn(s)&&(t||(dTn(Us(e)),t=!0),Jxn(h(n.xc(s),306)))}function X9n(e,t,n){var s;Bn(n,"Self-Loop routing",1),s=Ekn(t),KN(U(t,(xB(),X_))),Ti(pc(Jr(Jr(Xa(new Vt(null,new Wt(t.b,16)),new NRe),new PRe),new BRe),new FRe),new TZe(e,s)),An(n)}function Q9n(e){var t,n,s,a,u,p,v,m,x;return x=G2e(e),n=e.e,u=n!=null,u&&ok(x,fj,e.e),v=e.k,p=!!v,p&&ok(x,"type",FY(e.k)),s=wS(e.j),a=!s,a&&(m=new yp,Wh(x,Eee,m),t=new lKe(m),Os(e.j,t)),x}function Z9n(e){var t,n,s,a;for(a=pg((Uu(e.gc(),"size"),new Kv),123),s=!0,n=ob(e).Kc();n.Ob();)t=h(n.Pb(),42),s||(a.a+=Js),s=!1,ho(pg(ho(a,t.cd()),61),t.dd());return(a.a+="}",a).a}function ydt(e,t){var n,s,a;return t&=63,t<22?(n=e.l<<t,s=e.m<<t|e.l>>22-t,a=e.h<<t|e.m>>22-t):t<44?(n=0,s=e.l<<t-22,a=e.m<<t-22|e.l>>44-t):(n=0,s=0,a=e.l<<t-44),rc(n&du,s&du,a&Rd)}function Dw(e){if(k3e==null&&(k3e=new RegExp("^\\s*[+-]?(NaN|Infinity|((\\d+\\.?\\d*)|(\\.\\d+))([eE][+-]?\\d+)?[dDfF]?)\\s*$")),!k3e.test(e))throw ne(new Qf(Gb+e+'"'));return parseFloat(e)}function J9n(e){var t,n,s,a;for(t=new qe,n=pe(vu,Y1,25,e.a.c.length,16,1),Tde(n,n.length),a=new Q(e.a);a.a<a.c.c.length;)s=h(K(a),121),n[s.d]||(t.c[t.c.length]=s,Dft(e,s,n));return t}function e_n(e,t){var n,s,a,u;for(u=t.b.j,e.a=pe(Hn,hr,25,u.c.length,15,1),a=0,s=0;s<u.c.length;s++)n=(Kt(s,u.c.length),h(u.c[s],11)),n.e.c.length==0&&n.g.c.length==0?a+=1:a+=3,e.a[s]=a}function KF(){KF=re,tne=new nE("ALWAYS_UP",0),ene=new nE("ALWAYS_DOWN",1),rne=new nE("DIRECTION_UP",2),nne=new nE("DIRECTION_DOWN",3),ine=new nE("SMART_UP",4),A$=new nE("SMART_DOWN",5)}function t_n(e,t){if(e<0||t<0)throw ne(new an("k and n must be positive"));if(t>e)throw ne(new an("k must be smaller than n"));return t==0||t==e?1:e==0?0:C2e(e)/(C2e(t)*C2e(e-t))}function J2e(e,t){var n,s,a,u;for(n=new xfe(e);n.g==null&&!n.c?t0e(n):n.g==null||n.i!=0&&h(n.g[n.i-1],47).Ob();)if(u=h(JF(n),56),ue(u,160))for(s=h(u,160),a=0;a<t.length;a++)t[a].og(s)}function hQ(e){var t;return(e.Db&64)!=0?vX(e):(t=new Ol(vX(e)),t.a+=" (height: ",ny(t,e.f),t.a+=", width: ",ny(t,e.g),t.a+=", x: ",ny(t,e.i),t.a+=", y: ",ny(t,e.j),t.a+=")",t.a)}function n_n(e){var t,n,s,a,u,p,v;for(t=new Np,s=e,a=0,u=s.length;a<u;++a)if(n=s[a],p=Vn(n.cd()),v=Yp(t,p,Vn(n.dd())),v!=null)throw ne(new an("duplicate key: "+p));this.b=(Rt(),new yq(t))}function r_n(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],Cd(u,String.fromCharCode(t));return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function ebe(){ebe=re,M4e=(NB(),Xj),N4t=new $t(PR,M4e),Ke(1),O4t=new $t(Swe,Ke(300)),Ke(0),F4t=new $t(Awe,Ke(0)),new dS,R4t=new $t(uJ,l1),new dS,P4t=new $t(lJ,5),j4t=Xj,B4t=kte}function kdt(e,t){var n,s,a,u,p;for(a=t==1?Mte:Ate,s=a.a.ec().Kc();s.Ob();)for(n=h(s.Pb(),103),p=h(jr(e.f.c,n),21).Kc();p.Ob();)u=h(p.Pb(),46),Ue(e.b.b,h(u.b,81)),Ue(e.b.a,h(u.b,81).d)}function i_n(e,t){var n;if(t!=null&&!e.c.Yj().wj(t))throw n=ue(t,56)?h(t,56).Tg().zb:sg(uu(t)),ne(new z6(Fg+e.c.ne()+"'s type '"+e.c.Yj().ne()+"' does not permit a value of type '"+n+"'"))}function s_n(e,t,n){var s,a;for(a=new Ms(e.b,0);a.b<a.d.gc();)s=(kn(a.b<a.d.gc()),h(a.d.Xb(a.c=a.b++),70)),Me(U(s,(He(),k6e)))===Me(t)&&(abe(s.n,Us(e.c.i),n),Mu(a),Ue(t.b,s))}function a_n(e,t){if(t.a)switch(h(U(t.b,(He(),E6e)),98).g){case 0:case 1:SEn(t);case 2:Ti(new Vt(null,new Wt(t.d,16)),new Hue),oCn(e.a,t)}else Ti(new Vt(null,new Wt(t.d,16)),new Hue)}function xdt(e){var t,n;return n=d.Math.sqrt((e.k==null&&(e.k=_ge(e,new c$e)),Pe(e.k)/(e.b*(e.g==null&&(e.g=Jut(e,new Uue)),Pe(e.g))))),t=Gn(xc(d.Math.round(n))),t=d.Math.min(t,e.f),t}function So(){Rc(),y1e.call(this),this.j=(Qe(),to),this.a=new Rs,new Tq,this.f=(Uu(2,jw),new Wo(2)),this.e=(Uu(4,jw),new Wo(4)),this.g=(Uu(4,jw),new Wo(4)),this.b=new OZe(this.e,this.g)}function o_n(e,t){var n,s;return!(ft(lt(U(t,(He(),Ff))))||(s=t.c.i,e==(wl(),C_)&&s.k==(bn(),Ou))||(n=h(U(s,(tt(),cc)),163),n==Gg))}function c_n(e,t){var n,s;return!(ft(lt(U(t,(He(),Ff))))||(s=t.d.i,e==(wl(),S_)&&s.k==(bn(),Ou))||(n=h(U(s,(tt(),cc)),163),n==nm))}function u_n(e,t){var n,s,a,u,p,v,m;for(p=e.d,m=e.o,v=new hl(-p.b,-p.d,p.b+m.a+p.c,p.d+m.b+p.a),s=t,a=0,u=s.length;a<u;++a)n=s[a],n&&v9(v,n.i);p.b=-v.c,p.d=-v.d,p.c=v.b-p.b-m.a,p.a=v.a-p.d-m.b}function lM(){lM=re,b8e=new OS("CENTER_DISTANCE",0),Ere=new OS("CIRCLE_UNDERLAP",1),w8e=new OS("RECTANGLE_UNDERLAP",2),_re=new OS("INVERTED_OVERLAP",3),v8e=new OS("MINIMUM_ROOT_DISTANCE",4)}function l_n(e){eve();var t,n,s,a,u;if(e==null)return null;for(s=e.length,a=s*2,t=pe(Cl,o1,25,a,15,1),n=0;n<s;n++)u=e[n],u<0&&(u+=256),t[n*2]=nz[u>>4],t[n*2+1]=nz[u&15];return Fl(t,0,t.length)}function h_n(e){VP();var t,n,s;switch(s=e.c.length,s){case 0:return n3t;case 1:return t=h(s0t(new Q(e)),42),P2n(t.cd(),t.dd());default:return n=h(Mf(e,pe(c2,TR,42,e.c.length,0,1)),165),new Iq(n)}}function f_n(e){var t,n,s,a,u,p;for(t=new Jm,n=new Jm,mg(t,e),mg(n,e);n.b!=n.c;)for(a=h(H4(n),37),p=new Q(a.a);p.a<p.c.c.length;)u=h(K(p),10),u.e&&(s=u.e,mg(t,s),mg(n,s));return t}function x0(e,t){switch(t.g){case 1:return R4(e.j,(Rc(),J4e));case 2:return R4(e.j,(Rc(),Q4e));case 3:return R4(e.j,(Rc(),t5e));case 4:return R4(e.j,(Rc(),n5e));default:return Rt(),Rt(),ca}}function d_n(e,t){var n,s,a;n=D2n(t,e.e),s=h(_n(e.g.f,n),19).a,a=e.a.c.length-1,e.a.c.length!=0&&h(ut(e.a,a),287).c==s?(++h(ut(e.a,a),287).a,++h(ut(e.a,a),287).b):Ue(e.a,new ttt(s))}function g_n(e,t,n){var s,a;return s=yMn(e,t,n),s!=0?s:os(t,(He(),Eo))&&os(n,Eo)?(a=dc(h(U(t,Eo),19).a,h(U(n,Eo),19).a),a<0?EM(e,t,n):a>0&&EM(e,n,t),a):WTn(e,t,n)}function Edt(e,t,n){var s,a,u,p;if(t.b!=0){for(s=new di,p=pr(t,0);p.b!=p.d.c;)u=h(gr(p),86),Qs(s,Pge(u)),a=u.e,a.a=h(U(u,(fo(),ore)),19).a,a.b=h(U(u,yxe),19).a;Edt(e,s,jo(n,s.b/e.a|0))}}function _dt(e,t){var n,s,a,u,p;if(e.e<=t||ymn(e,e.g,t))return e.g;for(u=e.r,s=e.g,p=e.r,a=(u-s)/2+s;s+1<u;)n=M9(e,a,!1),n.b<=a&&n.a<=t?(p=a,u=a):s=a,a=(u-s)/2+s;return p}function p_n(e,t,n){var s;s=Ggt(e,t,!0),Bn(n,"Recursive Graph Layout",s),J2e(t,se(ie(R8e,1),st,527,0,[new jUe])),Pp(t,(Tr(),v8))||J2e(t,se(ie(R8e,1),st,527,0,[new qHe])),Gve(e,t,null,n),An(n)}function An(e){var t;if(e.p==null)throw ne(new ja("The task has not begun yet."));e.b||(e.k&&(t=(L1(),js(xc(Date.now()),_0)),e.q=vb(Cg(t,e.o))*1e-9),e.c<e.r&&BK(e,e.r-e.c),e.b=!0)}function hM(e){var t,n,s;for(s=new Oc,vr(s,new dt(e.j,e.k)),n=new Tn((!e.a&&(e.a=new Gi(Zl,e,5)),e.a));n.e!=n.i.gc();)t=h(On(n),469),vr(s,new dt(t.a,t.b));return vr(s,new dt(e.b,e.c)),s}function b_n(e,t,n,s,a){var u,p,v,m,x,_;if(a)for(m=a.a.length,u=new Mp(m),_=(u.b-u.a)*u.c<0?(cg(),sp):new fg(u);_.Ob();)x=h(_.Pb(),19),v=wk(a,x.a),p=new wit(e,t,n,s),CDn(p.a,p.b,p.c,p.d,v)}function tbe(e,t){var n;if(Me(e)===Me(t))return!0;if(ue(t,21)){n=h(t,21);try{return e.gc()==n.gc()&&e.Ic(n)}catch(s){if(s=ui(s),ue(s,173)||ue(s,205))return!1;throw ne(s)}}return!1}function nbe(e,t){var n;Ue(e.d,t),n=t.rf(),e.c?(e.e.a=d.Math.max(e.e.a,n.a),e.e.b+=n.b,e.d.c.length>1&&(e.e.b+=e.a)):(e.e.a+=n.a,e.e.b=d.Math.max(e.e.b,n.b),e.d.c.length>1&&(e.e.a+=e.a))}function v_n(e){var t,n,s,a;switch(a=e.i,t=a.b,s=a.j,n=a.g,a.a.g){case 0:n.a=(e.g.b.o.a-s.a)/2;break;case 1:n.a=t.d.n.a+t.d.a.a;break;case 2:n.a=t.d.n.a+t.d.a.a-s.a;break;case 3:n.b=t.d.n.b+t.d.a.b}}function Tdt(e,t,n,s,a){if(s<t||a<n)throw ne(new an("The highx must be bigger then lowx and the highy must be bigger then lowy"));return e.a<t?e.a=t:e.a>s&&(e.a=s),e.b<n?e.b=n:e.b>a&&(e.b=a),e}function w_n(e){if(ue(e,149))return YSn(h(e,149));if(ue(e,229))return Vkn(h(e,229));if(ue(e,23))return Q9n(h(e,23));throw ne(new an(zye+Ag(new _u(se(ie(yn,1),st,1,5,[e])))))}function m_n(e,t,n,s,a){var u,p,v;for(u=!0,p=0;p<s;p++)u=u&n[p]==0;if(a==0)Ao(n,s,e,0,t),p=t;else{for(v=32-a,u=u&n[p]<<v==0,p=0;p<t-1;p++)e[p]=n[p+s]>>>a|n[p+s+1]<<v;e[p]=n[p+s]>>>a,++p}return u}function rbe(e,t,n,s){var a,u,p;if(t.k==(bn(),ds)){for(u=new Sn(Dn(za(t).a.Kc(),new L));rr(u);)if(a=h(Un(u),17),p=a.c.i.k,p==ds&&e.c.a[a.c.i.c.p]==s&&e.c.a[t.c.p]==n)return!0}return!1}function y_n(e,t){var n,s,a,u;return t&=63,n=e.h&Rd,t<22?(u=n>>>t,a=e.m>>t|n<<22-t,s=e.l>>t|e.m<<22-t):t<44?(u=0,a=n>>>t-22,s=e.m>>t-22|e.h<<44-t):(u=0,a=0,s=n>>>t-44),rc(s&du,a&du,u&Rd)}function Cdt(e,t,n,s){var a;this.b=s,this.e=e==(_b(),R_),a=t[n],this.d=Ap(vu,[Re,Y1],[177,25],16,[a.length,a.length],2),this.a=Ap(Hn,[Re,hr],[48,25],15,[a.length,a.length],2),this.c=new j2e(t,n)}function k_n(e){var t,n,s;for(e.k=new s0e((Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])).length,e.j.c.length),s=new Q(e.j);s.a<s.c.c.length;)n=h(K(s),113),t=n.d.j,Ot(e.k,t,n);e.e=OSn($4(e.k))}function Sdt(e,t){var n,s,a;Qi(e.d,t),n=new rHe,Pr(e.c,t,n),n.f=lX(t.c),n.a=lX(t.d),n.d=(x9(),a=t.c.i.k,a==(bn(),Wi)||a==Bf),n.e=(s=t.d.i.k,s==Wi||s==Bf),n.b=t.c.j==(Qe(),un),n.c=t.d.j==pn}function x_n(e){var t,n,s,a,u;for(u=Ir,a=Ir,s=new Q(Ty(e));s.a<s.c.c.length;)n=h(K(s),213),t=n.e.e-n.d.e,n.e==e&&t<a?a=t:t<u&&(u=t);return a==Ir&&(a=-1),u==Ir&&(u=-1),new As(Ke(a),Ke(u))}function E_n(e,t){var n,s,a;return a=Ax,s=(h9(),vD),a=d.Math.abs(e.b),n=d.Math.abs(t.f-e.b),n<a&&(a=n,s=Yj),n=d.Math.abs(e.a),n<a&&(a=n,s=wD),n=d.Math.abs(t.g-e.a),n<a&&(a=n,s=qj),s}function __n(e,t){var n,s,a,u;for(n=t.a.o.a,u=new F1(Us(t.a).b,t.c,t.f+1),a=new L7(u);a.b<a.d.gc();)if(s=(kn(a.b<a.d.gc()),h(a.d.Xb(a.c=a.b++),29)),s.c.a>=n)return ox(e,t,s.p),!0;return!1}function Adt(e){var t;return(e.Db&64)!=0?hQ(e):(t=new ju(Lye),!e.a||or(or((t.a+=' "',t),e.a),'"'),or(Q2(or(Q2(or(Q2(or(Q2((t.a+=" (",t),e.i),","),e.j)," | "),e.g),","),e.f),")"),t.a)}function Mdt(e,t,n){var s,a,u,p,v;for(v=ac(e.e.Tg(),t),a=h(e.g,119),s=0,p=0;p<e.i;++p)if(u=a[p],v.rl(u.ak())){if(s==n)return Ow(e,p),ia(),h(t,66).Oj()?u:u.dd();++s}throw ne(new Ea(n_+n+s2+s))}function Ddt(e){var t,n,s;if(t=e.c,t==2||t==7||t==1)return Mr(),Mr(),wT;for(s=Vve(e),n=null;(t=e.c)!=2&&t!=7&&t!=1;)n||(n=(Mr(),Mr(),new bE(1)),Qp(n,s),s=n),Qp(n,Vve(e));return s}function T_n(e,t,n){return e<0||e>n?hbe(e,n,"start index"):t<0||t>n?hbe(t,n,"end index"):E9("end index (%s) must not be less than start index (%s)",se(ie(yn,1),st,1,5,[Ke(t),Ke(e)]))}function Ldt(e,t){var n,s,a,u;for(s=0,a=e.length;s<a;s++){u=e[s];try{u[1]?u[0].jm()&&(t=ubn(t,u)):u[0].jm()}catch(p){if(p=ui(p),ue(p,78))n=p,lN(),ewn(ue(n,477)?h(n,477).ae():n);else throw ne(p)}}return t}function ox(e,t,n){var s,a,u;for(n!=t.c+t.b.gc()&&BLn(t.a,m5n(t,n-t.c)),u=t.a.c.p,e.a[u]=d.Math.max(e.a[u],t.a.o.a),a=h(U(t.a,(He(),DD)),15).Kc();a.Ob();)s=h(a.Pb(),70),Fe(s,Fte,(on(),!0))}function C_n(e,t){var n,s,a;a=tAn(t),Fe(t,(He(),gne),a),a&&(s=Ir,Ia(e.f,a)&&(s=h(no(Ia(e.f,a)),19).a),n=h(ut(t.g,0),17),ft(lt(U(n,Ff)))||Pr(e,a,Ke(d.Math.min(h(U(n,Eo),19).a,s))))}function Idt(e,t,n){var s,a,u,p,v;for(t.p=-1,v=Db(t,(ua(),ec)).Kc();v.Ob();)for(p=h(v.Pb(),11),a=new Q(p.g);a.a<a.c.c.length;)s=h(K(a),17),u=s.d.i,t!=u&&(u.p<0?n.Fc(s):u.p>0&&Idt(e,u,n));t.p=0}function kt(e){var t;this.c=new di,this.f=e.e,this.e=e.d,this.i=e.g,this.d=e.c,this.b=e.b,this.k=e.j,this.a=e.a,e.i?this.j=e.i:this.j=(t=h(Vh(P0),9),new ll(t,h(vh(t,t.length),9),0)),this.g=e.f}function S_n(e){var t,n,s,a;for(t=pg(or(new ju("Predicates."),"and"),40),n=!0,a=new L7(e);a.b<a.d.gc();)s=(kn(a.b<a.d.gc()),a.d.Xb(a.c=a.b++)),n||(t.a+=","),t.a+=""+s,n=!1;return(t.a+=")",t).a}function Odt(e,t,n){var s,a,u;if(!(n<=t+2))for(a=(n-t)/2|0,s=0;s<a;++s)u=(Kt(t+s,e.c.length),h(e.c[t+s],11)),dl(e,t+s,(Kt(n-s-1,e.c.length),h(e.c[n-s-1],11))),Kt(n-s-1,e.c.length),e.c[n-s-1]=u}function A_n(e,t,n){var s,a,u,p,v,m,x,_;u=e.d.p,v=u.e,m=u.r,e.g=new KS(m),p=e.d.o.c.p,s=p>0?v[p-1]:pe(X1,T0,10,0,0,1),a=v[p],x=p<v.length-1?v[p+1]:pe(X1,T0,10,0,0,1),_=t==n-1,_?qW(e.g,a,x):qW(e.g,s,a)}function Ndt(e){var t;this.j=new qe,this.f=new ns,this.b=(t=h(Vh(ta),9),new ll(t,h(vh(t,t.length),9),0)),this.d=pe(Hn,hr,25,(Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])).length,15,1),this.g=e}function Pdt(e,t){var n,s,a;if(t.c.length!=0){for(n=ldt(e,t),a=!1;!n;)ZF(e,t,!0),a=!0,n=ldt(e,t);a&&ZF(e,t,!1),s=qK(t),e.b&&e.b.lg(s),e.a=wht(e,(Kt(0,t.c.length),h(t.c[0],33))),Pdt(e,s)}}function fQ(e,t){var n,s,a;if(s=Ht(e.Tg(),t),n=t-e.Ah(),n<0)if(s)if(s.Ij())a=e.Yg(s),a>=0?e.Bh(a):bbe(e,s);else throw ne(new an(Fg+s.ne()+J9));else throw ne(new an(hmt+t+fmt));else vl(e,n,s)}function ibe(e){var t,n;if(n=null,t=!1,ue(e,204)&&(t=!0,n=h(e,204).a),t||ue(e,258)&&(t=!0,n=""+h(e,258).a),t||ue(e,483)&&(t=!0,n=""+h(e,483).a),!t)throw ne(new Nq(Hye));return n}function Bdt(e,t){var n,s;if(e.f){for(;t.Ob();)if(n=h(t.Pb(),72),s=n.ak(),ue(s,99)&&(h(s,18).Bb&go)!=0&&(!e.e||s.Gj()!=E8||s.aj()!=0)&&n.dd()!=null)return t.Ub(),!0;return!1}else return t.Ob()}function Fdt(e,t){var n,s;if(e.f){for(;t.Sb();)if(n=h(t.Ub(),72),s=n.ak(),ue(s,99)&&(h(s,18).Bb&go)!=0&&(!e.e||s.Gj()!=E8||s.aj()!=0)&&n.dd()!=null)return t.Pb(),!0;return!1}else return t.Sb()}function sbe(e,t,n){var s,a,u,p,v,m;for(m=ac(e.e.Tg(),t),s=0,v=e.i,a=h(e.g,119),p=0;p<e.i;++p)if(u=a[p],m.rl(u.ak())){if(n==s)return p;++s,v=p+1}if(n==s)return v;throw ne(new Ea(n_+n+s2+s))}function M_n(e,t){var n,s,a,u;if(e.f.c.length==0)return null;for(u=new O4,s=new Q(e.f);s.a<s.c.c.length;)n=h(K(s),70),a=n.o,u.b=d.Math.max(u.b,a.a),u.a+=a.b;return u.a+=(e.f.c.length-1)*t,u}function D_n(e,t,n){var s,a,u;for(a=new Sn(Dn(Id(n).a.Kc(),new L));rr(a);)s=h(Un(a),17),!Xs(s)&&!(!Xs(s)&&s.c.i.c==s.d.i.c)&&(u=bgt(e,s,n,new XKe),u.c.length>1&&(t.c[t.c.length]=u))}function L_n(e){var t,n,s,a;for(n=new di,Qs(n,e.o),s=new ehe;n.b!=0;)t=h(n.b==0?null:(kn(n.b!=0),pl(n,n.a.a)),508),a=bbt(e,t,!0),a&&Ue(s.a,t);for(;s.a.c.length!=0;)t=h(uht(s),508),bbt(e,t,!1)}function E0(){E0=re,j8e=new L4(j9,0),zs=new L4("BOOLEAN",1),po=new L4("INT",2),b8=new L4("STRING",3),Ba=new L4("DOUBLE",4),_i=new L4("ENUM",5),V5=new L4("ENUMSET",6),jf=new L4("OBJECT",7)}function v9(e,t){var n,s,a,u,p;s=d.Math.min(e.c,t.c),u=d.Math.min(e.d,t.d),a=d.Math.max(e.c+e.b,t.c+t.b),p=d.Math.max(e.d+e.a,t.d+t.a),a<s&&(n=s,s=a,a=n),p<u&&(n=u,u=p,p=n),Ant(e,s,u,a-s,p-u)}function jc(){jc=re,IEe=se(ie(it,1),Re,2,6,[n3e,sD,_j,Ayt,Tj,Nee,fj]),LEe=se(ie(it,1),Re,2,6,[n3e,"empty",sD,iD,"elementOnly"]),OEe=se(ie(it,1),Re,2,6,[n3e,"preserve","replace",tf]),Ps=new Cnt}function abe(e,t,n){var s,a,u;if(t!=n){s=t;do zr(e,s.c),a=s.e,a&&(u=s.d,nw(e,u.b,u.d),zr(e,a.n),s=Us(a));while(a);s=n;do ms(e,s.c),a=s.e,a&&(u=s.d,GS(e,u.b,u.d),ms(e,a.n),s=Us(a));while(a)}}function dQ(e,t,n,s){var a,u,p,v,m;if(s.f.c+s.g.c==0)for(p=e.a[e.c],v=0,m=p.length;v<m;++v)u=p[v],Pr(s,u,new ilt(e,u,n));return a=h(no(Ia(s.f,t)),663),a.b=0,a.c=a.f,a.c==0||yqe(h(ut(a.a,a.b),287)),a}function u5(){u5=re,Jx=new tE("MEDIAN_LAYER",0),m_=new tE("TAIL_LAYER",1),Zx=new tE("HEAD_LAYER",2),Jw=new tE("SPACE_EFFICIENT_LAYER",3),o3=new tE("WIDEST_LAYER",4),a3=new tE("CENTER_LAYER",5)}function I_n(e){switch(e.g){case 0:case 1:case 2:return Qe(),sn;case 3:case 4:case 5:return Qe(),Fn;case 6:case 7:case 8:return Qe(),un;case 9:case 10:case 11:return Qe(),pn;default:return Qe(),to}}function O_n(e,t){var n;return e.c.length==0?!1:(n=tft((Kt(0,e.c.length),h(e.c[0],17)).c.i),Hu(),n==(Tw(),w3)||n==v3?!0:yk(pc(new Vt(null,new Wt(e,16)),new x$e),new hWe(t)))}function obe(e,t,n){var s,a,u;if(!e.b[t.g]){for(e.b[t.g]=!0,s=n,!s&&(s=new kB),vr(s.b,t),u=e.a[t.g].Kc();u.Ob();)a=h(u.Pb(),188),a.b!=t&&obe(e,a.b,s),a.c!=t&&obe(e,a.c,s),vr(s.a,a);return s}return null}function w9(){w9=re,hH=new aE("ROOT_PROC",0),ere=new aE("FAN_PROC",1),nre=new aE("NEIGHBORS_PROC",2),tre=new aE("LEVEL_HEIGHT",3),rre=new aE("NODE_POSITION_PROC",4),Jne=new aE("DETREEIFYING_PROC",5)}function gQ(e,t){if(ue(t,239))return Cdn(e,h(t,33));if(ue(t,186))return Sdn(e,h(t,118));if(ue(t,439))return Tdn(e,h(t,202));throw ne(new an(zye+Ag(new _u(se(ie(yn,1),st,1,5,[t])))))}function Rdt(e,t,n){var s,a;if(this.f=e,s=h(_n(e.b,t),283),a=s?s.a:0,P0e(n,a),n>=(a/2|0))for(this.e=s?s.c:null,this.d=a;n++<a;)lst(this);else for(this.c=s?s.b:null;n-- >0;)n0e(this);this.b=t,this.a=null}function N_n(e,t){var n,s;t.a?rAn(e,t):(n=h(qY(e.b,t.b),57),!!n&&n==e.a[t.b.f]&&!!n.a&&n.a!=t.b.a&&n.c.Fc(t.b),s=h(UY(e.b,t.b),57),!!s&&e.a[s.f]==t.b&&!!s.a&&s.a!=t.b.a&&t.b.c.Fc(s),OY(e.b,t.b))}function jdt(e,t){var n,s;if(n=h(wa(e.b,t),124),h(h(jr(e.r,t),21),84).dc()){n.n.b=0,n.n.c=0;return}n.n.b=e.C.b,n.n.c=e.C.c,e.A.Hc((Iu(),E2))&&upt(e,t),s=l8n(e,t),LQ(e,t)==(Cy(),Xg)&&(s+=2*e.w),n.a.a=s}function $dt(e,t){var n,s;if(n=h(wa(e.b,t),124),h(h(jr(e.r,t),21),84).dc()){n.n.d=0,n.n.a=0;return}n.n.d=e.C.d,n.n.a=e.C.a,e.A.Hc((Iu(),E2))&&lpt(e,t),s=h8n(e,t),LQ(e,t)==(Cy(),Xg)&&(s+=2*e.w),n.a.b=s}function P_n(e,t){var n,s,a,u;for(u=new qe,s=new Q(t);s.a<s.c.c.length;)n=h(K(s),65),Ue(u,new Ghe(n,!0)),Ue(u,new Ghe(n,!1));a=new Oit(e),j7(a.a.a),Zst(u,e.b,new _u(se(ie(I3t,1),st,679,0,[a])))}function Hdt(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J;return m=e.a,P=e.b,x=t.a,j=t.b,_=n.a,V=n.b,A=s.a,J=s.b,u=m*j-P*x,p=_*J-V*A,a=(m-x)*(V-J)-(P-j)*(_-A),v=(u*(_-A)-p*(m-x))/a,I=(u*(V-J)-p*(P-j))/a,new dt(v,I)}function cbe(e,t){var n,s,a;if(!e.d[t.p]){for(e.d[t.p]=!0,e.a[t.p]=!0,s=new Sn(Dn(qi(t).a.Kc(),new L));rr(s);)n=h(Un(s),17),!Xs(n)&&(a=n.d.i,e.a[a.p]?Ue(e.b,n):cbe(e,a));e.a[t.p]=!1}}function zdt(e,t,n){var s;switch(s=0,h(U(t,(tt(),cc)),163).g){case 2:s=2*-n+e.a,++e.a;break;case 1:s=-n;break;case 3:s=n;break;case 4:s=2*n+e.b,++e.b}return os(t,(He(),Eo))&&(s+=h(U(t,Eo),19).a),s}function Gdt(e,t,n){var s,a,u;for(n.zc(t,e),Ue(e.n,t),u=e.p.eg(t),t.j==e.p.fg()?Uht(e.e,u):Uht(e.j,u),HP(e),a=gg(Cf(se(ie(Nf,1),st,20,0,[new j6(t),new y4(t)])));rr(a);)s=h(Un(a),11),n._b(s)||Gdt(e,s,n)}function ube(e){var t,n,s;return n=h(pt(e,(Tr(),m2)),21),n.Hc((Iu(),bm))?(s=h(pt(e,Wg),21),t=new _a(h(pt(e,_3),8)),s.Hc((fu(),x8))&&(t.a<=0&&(t.a=20),t.b<=0&&(t.b=20)),t):new Rs}function lbe(e){var t,n,s;if(!e.b){for(s=new Gze,n=new P4(A9(e));n.e!=n.i.gc();)t=h(NX(n),18),(t.Bb&go)!=0&&qn(s,t);kw(s),e.b=new ay((h(he(De((bg(),Xt).o),8),18),s.i),s.g),ou(e).b&=-9}return e.b}function B_n(e,t){var n,s,a,u,p,v,m,x;m=h(nM($4(t.k),pe(ta,yo,61,2,0,1)),122),x=t.g,n=bat(t,m[0]),a=pat(t,m[1]),s=KX(e,x,n,a),u=bat(t,m[1]),v=pat(t,m[0]),p=KX(e,x,u,v),s<=p?(t.a=n,t.c=a):(t.a=u,t.c=v)}function F_n(e,t,n){var s,a,u;for(Bn(n,"Processor set neighbors",1),e.a=t.b.b==0?1:t.b.b,a=null,s=pr(t.b,0);!a&&s.b!=s.d.c;)u=h(gr(s),86),ft(lt(U(u,(fo(),dm))))&&(a=u);a&&Kgt(e,new ng(a),n),An(n)}function Vdt(e){nZ();var t,n,s,a;return s=Zf(e,Ec(35)),t=s==-1?e:e.substr(0,s),n=s==-1?null:e.substr(s+1),a=_yn(vEe,t),a?n!=null&&(a=rht(a,(rn(n),n))):(a=JNn(t),Oyn(vEe,t,a),n!=null&&(a=rht(a,n))),a}function pQ(e){var t;Rt();var n,s,a,u,p,v;if(ue(e,54))for(u=0,a=e.gc()-1;u<a;++u,--a)t=e.Xb(u),e._c(u,e.Xb(a)),e._c(a,t);else for(n=e.Yc(),p=e.Zc(e.gc());n.Tb()<p.Vb();)s=n.Pb(),v=p.Ub(),n.Wb(v),p.Wb(s)}function R_n(e,t){var n,s,a;Bn(t,"End label pre-processing",1),n=Pe(Je(U(e,(tt(),z5)))),s=Pe(Je(U(e,lm))),a=ek(h(U(e,Ju),103)),Ti(Xa(new Vt(null,new Wt(e.b,16)),new eFe),new ont(n,s,a)),An(t)}function bQ(e,t){var n,s,a,u,p,v;for(v=0,u=new Jm,mg(u,t);u.b!=u.c;)for(p=h(H4(u),214),v+=b1t(p.d,p.e),a=new Q(p.b);a.a<a.c.c.length;)s=h(K(a),37),n=h(ut(e.b,s.p),214),n.s||(v+=bQ(e,n));return v}function Udt(e,t,n){var s,a;Elt(this),t==(gb(),Yg)?Qi(this.r,e.c):Qi(this.w,e.c),n==Yg?Qi(this.r,e.d):Qi(this.w,e.d),Sdt(this,e),s=lX(e.c),a=lX(e.d),vdt(this,s,a,a),this.o=(x9(),d.Math.abs(s-a)<.2)}function qdt(e,t,n){var s,a,u,p,v,m;if(v=h(tn(e.a,8),1936),v!=null)for(a=v,u=0,p=a.length;u<p;++u)null.jm();s=n,(e.a.Db&1)==0&&(m=new Yrt(e,n,t),s.ui(m)),ue(s,672)?h(s,672).wi(e.a):s.ti()==e.a&&s.vi(null)}function j_n(){var e;return y_t?h(lx((og(),th),o2),1945):(JOn(),e=h(ue(Fo((og(),th),o2),586)?Fo(th,o2):new tit,586),y_t=!0,BNn(e),lPn(e),Pr((Phe(),wEe),e,new XGe),QX(e),Ta(th,o2,e),e)}function $_n(e,t,n,s){var a;return a=Oy(e,n,se(ie(it,1),Re,2,6,[MZ,DZ,LZ,IZ,OZ,NZ,PZ]),t),a<0&&(a=Oy(e,n,se(ie(it,1),Re,2,6,["Sun","Mon","Tue","Wed","Thu","Fri","Sat"]),t)),a<0?!1:(s.d=a,!0)}function H_n(e,t,n,s){var a;return a=Oy(e,n,se(ie(it,1),Re,2,6,[MZ,DZ,LZ,IZ,OZ,NZ,PZ]),t),a<0&&(a=Oy(e,n,se(ie(it,1),Re,2,6,["Sun","Mon","Tue","Wed","Thu","Fri","Sat"]),t)),a<0?!1:(s.d=a,!0)}function z_n(e){var t,n,s;for(WCn(e),s=new qe,n=new Q(e.a.a.b);n.a<n.c.c.length;)t=h(K(n),81),Ue(s,new Whe(t,!0)),Ue(s,new Whe(t,!1));M8n(e.c),fA(s,e.b,new _u(se(ie(yD,1),st,369,0,[e.c]))),uCn(e)}function G_n(e){var t,n,s,a;for(n=new zn,a=new Q(e.d);a.a<a.c.c.length;)s=h(K(a),181),t=h(s.We((He(),P5)),17),Ia(n.f,t)||Pr(n,t,new Sit(t)),Ue(h(no(Ia(n.f,t)),456).b,s);return new Bc(new vd(n))}function V_n(e,t){var n,s,a,u,p;for(s=new Ast(e.j.c.length),n=null,u=new Q(e.j);u.a<u.c.c.length;)a=h(K(u),11),a.j!=n&&(s.b==s.c||sgt(s,n,t),x0e(s),n=a.j),p=G0t(a),p&&b0e(s,p);s.b==s.c||sgt(s,n,t)}function U_n(e,t){var n,s,a;for(s=new Ms(e.b,0);s.b<s.d.gc();)n=(kn(s.b<s.d.gc()),h(s.d.Xb(s.c=s.b++),70)),a=h(U(n,(tt(),y1)),272),a==(Tf(),pm)&&(Mu(s),Ue(t.b,n),os(n,(He(),P5))||Fe(n,P5,e))}function q_n(e){var t,n,s,a,u;for(t=Td(new Sn(Dn(qi(e).a.Kc(),new L))),a=new Sn(Dn(za(e).a.Kc(),new L));rr(a);)s=h(Un(a),17),n=s.c.i,u=Td(new Sn(Dn(qi(n).a.Kc(),new L))),t=d.Math.max(t,u);return Ke(t)}function Y_n(e,t,n){var s,a,u,p;for(Bn(n,"Processor arrange node",1),a=null,u=new di,s=pr(t.b,0);!a&&s.b!=s.d.c;)p=h(gr(s),86),ft(lt(U(p,(fo(),dm))))&&(a=p);Ai(u,a,u.c.b,u.c),nbt(e,u,jo(n,1)),An(n)}function Ydt(e,t,n){var s,a,u;s=h(pt(e,(Tr(),DH)),21),a=0,u=0,t.a>n.a&&(s.Hc((Aw(),WD))?a=(t.a-n.a)/2:s.Hc(KD)&&(a=t.a-n.a)),t.b>n.b&&(s.Hc((Aw(),QD))?u=(t.b-n.b)/2:s.Hc(XD)&&(u=t.b-n.b)),Y2e(e,a,u)}function Wdt(e,t,n,s,a,u,p,v,m,x,_,A,I){ue(e.Cb,88)&&Lw(ou(h(e.Cb,88)),4),Ko(e,n),e.f=p,Kk(e,v),Qk(e,m),Wk(e,x),Xk(e,_),m0(e,A),Zk(e,I),w0(e,!0),p0(e,a),e.ok(u),Vp(e,t),s!=null&&(e.i=null,WB(e,s))}function Kdt(e){var t,n;if(e.f){for(;e.n>0;){if(t=h(e.k.Xb(e.n-1),72),n=t.ak(),ue(n,99)&&(h(n,18).Bb&go)!=0&&(!e.e||n.Gj()!=E8||n.aj()!=0)&&t.dd()!=null)return!0;--e.n}return!1}else return e.n>0}function hbe(e,t,n){if(e<0)return E9(Obt,se(ie(yn,1),st,1,5,[n,Ke(e)]));if(t<0)throw ne(new an(Nbt+t));return E9("%s (%s) must not be greater than size (%s)",se(ie(yn,1),st,1,5,[n,Ke(e),Ke(t)]))}function fbe(e,t,n,s,a,u){var p,v,m,x;if(p=s-n,p<7){Okn(t,n,s,u);return}if(m=n+a,v=s+a,x=m+(v-m>>1),fbe(t,e,m,x,-a,u),fbe(t,e,x,v,-a,u),u.ue(e[x-1],e[x])<=0){for(;n<s;)pi(t,n++,e[m++]);return}tkn(e,m,x,v,t,n,s,u)}function fM(e,t){var n,s,a;for(a=new qe,s=new Q(e.c.a.b);s.a<s.c.c.length;)n=h(K(s),57),t.Lb(n)&&(Ue(a,new Hhe(n,!0)),Ue(a,new Hhe(n,!1)));A8n(e.e),Zst(a,e.d,new _u(se(ie(I3t,1),st,679,0,[e.e])))}function W_n(e,t){var n,s,a,u,p,v,m;for(m=t.d,a=t.b.j,v=new Q(m);v.a<v.c.c.length;)for(p=h(K(v),101),u=pe(vu,Y1,25,a.c.length,16,1),Pr(e.b,p,u),n=p.a.d.p-1,s=p.c.d.p;n!=s;)n=(n+1)%a.c.length,u[n]=!0}function K_n(e,t){for(e.r=new PA(e.p),qhn(e.r,e),Qs(e.r.j,e.j),Pl(e.j),vr(e.j,t),vr(e.r.e,t),HP(e),HP(e.r);e.f.c.length!=0;)vet(h(ut(e.f,0),129));for(;e.k.c.length!=0;)vet(h(ut(e.k,0),129));return e.r}function vQ(e,t,n){var s,a,u;if(a=Ht(e.Tg(),t),s=t-e.Ah(),s<0)if(a)if(a.Ij())u=e.Yg(a),u>=0?e.sh(u,n):zbe(e,a,n);else throw ne(new an(Fg+a.ne()+J9));else throw ne(new an(hmt+t+fmt));else ml(e,s,a,n)}function Xdt(e){var t,n,s,a;if(n=h(e,49).qh(),n)try{if(s=null,t=lx((og(),th),Ipt(Bkn(n))),t&&(a=t.rh(),a&&(s=a.Wk(Vfn(n.e)))),!!s&&s!=e)return Xdt(s)}catch(u){if(u=ui(u),!ue(u,60))throw ne(u)}return e}function sc(e,t,n){var s,a,u,p;if(p=t==null?0:e.b.se(t),a=(s=e.a.get(p),s==null?new Array:s),a.length==0)e.a.set(p,a);else if(u=fht(e,t,a),u)return u.ed(n);return pi(a,a.length,new LN(t,n)),++e.c,JP(e.b),null}function Qdt(e,t){var n,s;return hB(e.a),P1(e.a,(QB(),vH),vH),P1(e.a,p8,p8),s=new rs,Lr(s,p8,(wF(),fre)),Me(pt(t,(Cw(),pre)))!==Me((VA(),wH))&&Lr(s,p8,lre),Lr(s,p8,hre),KJe(e.a,s),n=vR(e.a,t),n}function Zdt(e){if(!e)return OXe(),h3t;var t=e.valueOf?e.valueOf():e;if(t!==e){var n=tte[typeof t];return n?n(t):ipe(typeof t)}else return e instanceof Array||e instanceof d.Array?new ule(e):new aS(e)}function Jdt(e,t,n){var s,a,u;switch(u=e.o,s=h(wa(e.p,n),244),a=s.i,a.b=pM(s),a.a=gM(s),a.b=d.Math.max(a.b,u.a),a.b>u.a&&!t&&(a.b=u.a),a.c=-(a.b-u.a)/2,n.g){case 1:a.d=-a.a;break;case 3:a.d=u.b}VQ(s),UQ(s)}function e0t(e,t,n){var s,a,u;switch(u=e.o,s=h(wa(e.p,n),244),a=s.i,a.b=pM(s),a.a=gM(s),a.a=d.Math.max(a.a,u.b),a.a>u.b&&!t&&(a.a=u.b),a.d=-(a.a-u.b)/2,n.g){case 4:a.c=-a.b;break;case 2:a.c=u.a}VQ(s),UQ(s)}function X_n(e,t){var n,s,a,u,p;if(!t.dc()){if(a=h(t.Xb(0),128),t.gc()==1){Sgt(e,a,a,1,0,t);return}for(n=1;n<t.gc();)(a.j||!a.o)&&(u=s7n(t,n),u&&(s=h(u.a,19).a,p=h(u.b,128),Sgt(e,a,p,n,s,t),n=s+1,a=p))}}function Q_n(e){var t,n,s,a,u,p;for(p=new Bc(e.d),hs(p,new Nje),t=(sR(),se(ie($5e,1),ze,270,0,[zte,Ute,Hte,Wte,Vte,Gte,Yte,qte])),n=0,u=new Q(p);u.a<u.c.c.length;)a=h(K(u),101),s=t[n%t.length],YTn(a,s),++n}function Z_n(e,t){Ry();var n,s,a,u;if(t.b<2)return!1;for(u=pr(t,0),n=h(gr(u),8),s=n;u.b!=u.d.c;){if(a=h(gr(u),8),!(Pk(e,s)&&Pk(e,a)))return!1;s=a}return!!(Pk(e,s)&&Pk(e,n))}function dbe(e,t){var n,s,a,u,p,v,m,x,_,A;return _=null,A=e,p=Md(A,"x"),n=new WWe(t),V3n(n.a,p),v=Md(A,"y"),s=new KWe(t),U3n(s.a,v),m=Md(A,mee),a=new XWe(t),q3n(a.a,m),x=Md(A,wee),u=new QWe(t),_=(Y3n(u.a,x),x),_}function Lw(e,t){apt(e,t),(e.b&1)!=0&&(e.a.a=null),(e.b&2)!=0&&(e.a.f=null),(e.b&4)!=0&&(e.a.g=null,e.a.i=null),(e.b&16)!=0&&(e.a.d=null,e.a.e=null),(e.b&8)!=0&&(e.a.b=null),(e.b&32)!=0&&(e.a.j=null,e.a.c=null)}function J_n(e,t){var n,s,a;if(a=0,t.length>0)try{a=Wu(t,$s,Ir)}catch(u){throw u=ui(u),ue(u,127)?(s=u,ne(new MB(s))):ne(u)}return n=(!e.a&&(e.a=new kq(e)),e.a),a<n.i&&a>=0?h(he(n,a),56):null}function eTn(e,t){if(e<0)return E9(Obt,se(ie(yn,1),st,1,5,["index",Ke(e)]));if(t<0)throw ne(new an(Nbt+t));return E9("%s (%s) must be less than size (%s)",se(ie(yn,1),st,1,5,["index",Ke(e),Ke(t)]))}function tTn(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],u.a?or(u.a,u.b):u.a=new ju(u.d),gE(u.a,""+t);return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function nTn(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],u.a?or(u.a,u.b):u.a=new ju(u.d),gE(u.a,""+t);return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function rTn(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],u.a?or(u.a,u.b):u.a=new ju(u.d),gE(u.a,""+t);return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function iTn(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],u.a?or(u.a,u.b):u.a=new ju(u.d),gE(u.a,""+t);return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function t0t(e,t){var n,s,a,u,p,v;for(n=e.b.c.length,a=ut(e.b,t);t*2+1<n&&(s=(u=2*t+1,p=u+1,v=u,p<n&&e.a.ue(ut(e.b,p),ut(e.b,u))<0&&(v=p),v),!(e.a.ue(a,ut(e.b,s))<0));)dl(e.b,t,ut(e.b,s)),t=s;dl(e.b,t,a)}function gbe(e,t,n,s,a,u){var p,v,m,x,_;for(Me(e)===Me(n)&&(e=e.slice(t,t+a),t=0),m=n,v=t,x=t+a;v<x;)p=d.Math.min(v+1e4,x),a=p-v,_=e.slice(v,p),_.splice(0,0,s,u?a:0),Array.prototype.splice.apply(m,_),v=p,s+=a}function wQ(e,t,n){var s,a;return s=n.d,a=n.e,e.g[s.d]<=e.i[t.d]&&e.i[t.d]<=e.i[s.d]&&e.g[a.d]<=e.i[t.d]&&e.i[t.d]<=e.i[a.d]?!(e.i[s.d]<e.i[a.d]):e.i[s.d]<e.i[a.d]}function n0t(e){var t,n,s,a,u,p,v;if(s=e.a.c.length,s>0)for(p=e.c.d,v=e.d.d,a=Jf(ms(new dt(v.a,v.b),p),1/(s+1)),u=new dt(p.a,p.b),n=new Q(e.a);n.a<n.c.c.length;)t=h(K(n),559),t.d.a=u.a,t.d.b=u.b,zr(u,a)}function r0t(e,t,n){var s,a,u,p,v,m;for(m=ki,u=new Q(pgt(e.b));u.a<u.c.c.length;)for(a=h(K(u),168),v=new Q(pgt(t.b));v.a<v.c.c.length;)p=h(K(v),168),s=U5n(a.a,a.b,p.a,p.b,n),m=d.Math.min(m,s);return m}function Ji(e,t){if(!t)throw ne(new P7);if(e.j=t,!e.d)switch(e.j.g){case 1:e.a.a=e.o.a/2,e.a.b=0;break;case 2:e.a.a=e.o.a,e.a.b=e.o.b/2;break;case 3:e.a.a=e.o.a/2,e.a.b=e.o.b;break;case 4:e.a.a=0,e.a.b=e.o.b/2}}function sTn(e,t){var n,s,a;return ue(t.g,10)&&h(t.g,10).k==(bn(),Pi)?ki:(a=W4(t),a?d.Math.max(0,e.b/2-.5):(n=by(t),n?(s=Pe(Je(_w(n,(tt(),uv)))),d.Math.max(0,s/2-.5)):ki))}function aTn(e,t){var n,s,a;return ue(t.g,10)&&h(t.g,10).k==(bn(),Pi)?ki:(a=W4(t),a?d.Math.max(0,e.b/2-.5):(n=by(t),n?(s=Pe(Je(_w(n,(tt(),uv)))),d.Math.max(0,s/2-.5)):ki))}function oTn(e){var t,n,s,a,u,p;for(p=r9(e.d,e.e),u=p.Kc();u.Ob();)for(a=h(u.Pb(),11),s=e.e==(Qe(),un)?a.e:a.g,n=new Q(s);n.a<n.c.c.length;)t=h(K(n),17),!Xs(t)&&t.c.i.c!=t.d.i.c&&(d_n(e,t),++e.f,++e.c)}function i0t(e,t){var n,s;if(t.dc())return Rt(),Rt(),ca;for(s=new qe,Ue(s,Ke($s)),n=1;n<e.f;++n)e.a==null&&spt(e),e.a[n]&&Ue(s,Ke(n));return s.c.length==1?(Rt(),Rt(),ca):(Ue(s,Ke(Ir)),cLn(t,s))}function cTn(e,t){var n,s,a,u,p,v,m;p=t.c.i.k!=(bn(),Wi),m=p?t.d:t.c,n=E7n(t,m).i,a=h(_n(e.k,m),121),s=e.i[n.p].a,Ytt(m.i)<(n.c?Ha(n.c.a,n,0):-1)?(u=a,v=s):(u=s,v=a),Th(ph(gh(bh(dh(new Il,0),4),u),v))}function uTn(e,t,n){var s,a,u,p,v,m;if(n)for(a=n.a.length,s=new Mp(a),v=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);v.Ob();)p=h(v.Pb(),19),m=qX(e,l5(bw(n,p.a))),m&&(u=(!t.b&&(t.b=new qt(Pn,t,4,7)),t.b),qn(u,m))}function lTn(e,t,n){var s,a,u,p,v,m;if(n)for(a=n.a.length,s=new Mp(a),v=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);v.Ob();)p=h(v.Pb(),19),m=qX(e,l5(bw(n,p.a))),m&&(u=(!t.c&&(t.c=new qt(Pn,t,5,8)),t.c),qn(u,m))}function dM(e,t,n){var s,a;s=t.a&e.f,t.b=e.b[s],e.b[s]=t,a=t.f&e.f,t.d=e.c[a],e.c[a]=t,n?(t.e=n.e,t.e?t.e.c=t:e.a=t,t.c=n.c,t.c?t.c.e=t:e.e=t):(t.e=e.e,t.c=null,e.e?e.e.c=t:e.a=t,e.e=t),++e.i,++e.g}function s0t(e){var t,n,s;if(t=e.Pb(),!e.Ob())return t;for(s=ho(or(new rg,"expected one element but was: <"),t),n=0;n<4&&e.Ob();n++)ho((s.a+=Js,s),e.Pb());throw e.Ob()&&(s.a+=", ..."),s.a+=">",ne(new an(s.a))}function hTn(e,t){var n;t.d?t.d.b=t.b:e.a=t.b,t.b?t.b.d=t.d:e.e=t.d,!t.e&&!t.c?(n=h(Q4(e.b,t.a),283),n.a=0,++e.c):(n=h(_n(e.b,t.a),283),--n.a,t.e?t.e.c=t.c:n.b=t.c,t.c?t.c.e=t.e:n.c=t.e),--e.d}function fTn(e){var t,n;return n=-e.a,t=se(ie(Cl,1),o1,25,15,[43,48,48,48,48]),n<0&&(t[0]=45,n=-n),t[1]=t[1]+((n/60|0)/10|0)&Oi,t[2]=t[2]+(n/60|0)%10&Oi,t[3]=t[3]+(n%60/10|0)&Oi,t[4]=t[4]+n%10&Oi,Fl(t,0,t.length)}function a0t(e,t,n){var s,a;for(s=t.d,a=n.d;s.a-a.a==0&&s.b-a.b==0;)s.a+=hu(e,26)*P9+hu(e,27)*B9-.5,s.b+=hu(e,26)*P9+hu(e,27)*B9-.5,a.a+=hu(e,26)*P9+hu(e,27)*B9-.5,a.b+=hu(e,26)*P9+hu(e,27)*B9-.5}function pbe(e){var t,n,s,a;for(e.g=new o9(h(Vn(ta),290)),s=0,n=(Qe(),sn),t=0;t<e.j.c.length;t++)a=h(ut(e.j,t),11),a.j!=n&&(s!=t&&j4(e.g,n,new As(Ke(s),Ke(t))),n=a.j,s=t);j4(e.g,n,new As(Ke(s),Ke(t)))}function dTn(e){var t,n,s,a,u,p,v;for(s=0,n=new Q(e.b);n.a<n.c.c.length;)for(t=h(K(n),29),u=new Q(t.a);u.a<u.c.c.length;)for(a=h(K(u),10),a.p=s++,v=new Q(a.j);v.a<v.c.c.length;)p=h(K(v),11),p.p=s++}function o0t(e,t,n,s,a){var u,p,v,m,x;if(t)for(v=t.Kc();v.Ob();)for(p=h(v.Pb(),10),x=ive(p,(ua(),ec),n).Kc();x.Ob();)m=h(x.Pb(),11),u=h(no(Ia(a.f,m)),112),u||(u=new PA(e.d),s.c[s.c.length]=u,Gdt(u,m,a))}function bbe(e,t){var n,s,a;if(a=Hy((jc(),Ps),e.Tg(),t),a)ia(),h(a,66).Oj()||(a=Y4(Sa(Ps,a))),s=(n=e.Yg(a),h(n>=0?e._g(n,!0,!0):Bb(e,a,!0),153)),h(s,215).ol(t);else throw ne(new an(Fg+t.ne()+J9))}function vbe(e){var t,n;return e>-0x800000000000&&e<0x800000000000?e==0?0:(t=e<0,t&&(e=-e),n=Li(d.Math.floor(d.Math.log(e)/.6931471805599453)),(!t||e!=d.Math.pow(2,n))&&++n,n):Clt(xc(e))}function gTn(e){var t,n,s,a,u,p,v;for(u=new md,n=new Q(e);n.a<n.c.c.length;)t=h(K(n),129),p=t.a,v=t.b,!(u.a._b(p)||u.a._b(v))&&(a=p,s=v,p.e.b+p.j.b>2&&v.e.b+v.j.b<=2&&(a=v,s=p),u.a.zc(a,u),a.q=s);return u}function c0t(e,t){var n,s,a;return s=new Nd(e),Oa(s,t),Fe(s,(He(),F$),t),Fe(s,(tt(),Ei),(_s(),qo)),Fe(s,m1,($1(),AH)),kp(s,(bn(),Pi)),n=new So,Ka(n,s),Ji(n,(Qe(),un)),a=new So,Ka(a,s),Ji(a,pn),s}function u0t(e){switch(e.g){case 0:return new Pq((_b(),jD));case 1:return new xUe;case 2:return new MUe;default:throw ne(new an("No implementation is available for the crossing minimizer "+(e.f!=null?e.f:""+e.g)))}}function l0t(e,t){var n,s,a,u,p;for(e.c[t.p]=!0,Ue(e.a,t),p=new Q(t.j);p.a<p.c.c.length;)for(u=h(K(p),11),s=new _f(u.b);Wa(s.a)||Wa(s.b);)n=h(Wa(s.a)?K(s.a):K(s.b),17),a=Sxn(u,n).i,e.c[a.p]||l0t(e,a)}function h0t(e){var t,n,s,a,u,p,v;for(p=0,n=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));n.e!=n.i.gc();)t=h(On(n),33),v=t.g,a=t.f,s=d.Math.sqrt(v*v+a*a),p=d.Math.max(s,p),u=h0t(t),p=d.Math.max(u,p);return p}function Qc(){Qc=re,Zg=new hE("OUTSIDE",0),Ud=new hE("INSIDE",1),aL=new hE("NEXT_TO_PORT_IF_POSSIBLE",2),sT=new hE("ALWAYS_SAME_SIDE",3),iT=new hE("ALWAYS_OTHER_SAME_SIDE",4),K5=new hE("SPACE_EFFICIENT",5)}function f0t(e,t,n){var s,a,u,p,v,m;return s=qmn(e,(a=(X2(),u=new nhe,u),n&&Vbe(a,n),a),t),WE(s,_d(t,p1)),qF(t,s),LCn(t,s),dbe(t,s),p=t,v=wb(p,"ports"),m=new dJe(e,s),vSn(m.a,m.b,v),XK(e,t,s),$5n(e,t,s),s}function pTn(e){var t,n;return n=-e.a,t=se(ie(Cl,1),o1,25,15,[43,48,48,58,48,48]),n<0&&(t[0]=45,n=-n),t[1]=t[1]+((n/60|0)/10|0)&Oi,t[2]=t[2]+(n/60|0)%10&Oi,t[4]=t[4]+(n%60/10|0)&Oi,t[5]=t[5]+n%10&Oi,Fl(t,0,t.length)}function bTn(e){var t;return t=se(ie(Cl,1),o1,25,15,[71,77,84,45,48,48,58,48,48]),e<=0&&(t[3]=43,e=-e),t[4]=t[4]+((e/60|0)/10|0)&Oi,t[5]=t[5]+(e/60|0)%10&Oi,t[7]=t[7]+(e%60/10|0)&Oi,t[8]=t[8]+e%10&Oi,Fl(t,0,t.length)}function vTn(e){var t,n,s,a,u;if(e==null)return _c;for(u=new $p(Js,"[","]"),n=e,s=0,a=n.length;s<a;++s)t=n[s],u.a?or(u.a,u.b):u.a=new ju(u.d),gE(u.a,""+ME(t));return u.a?u.e.length==0?u.a.a:u.a.a+(""+u.e):u.c}function wbe(e,t){var n,s,a;for(a=Ir,s=new Q(Ty(t));s.a<s.c.c.length;)n=h(K(s),213),n.f&&!e.c[n.c]&&(e.c[n.c]=!0,a=d.Math.min(a,wbe(e,mF(n,t))));return e.i[t.d]=e.j,e.g[t.d]=d.Math.min(a,e.j++),e.g[t.d]}function d0t(e,t){var n,s,a;for(a=h(h(jr(e.r,t),21),84).Kc();a.Ob();)s=h(a.Pb(),111),s.e.b=(n=s.b,n.Xe((Tr(),pu))?n.Hf()==(Qe(),sn)?-n.rf().b-Pe(Je(n.We(pu))):Pe(Je(n.We(pu))):n.Hf()==(Qe(),sn)?-n.rf().b:0)}function wTn(e){var t,n,s,a,u,p,v;for(n=Tfe(e.e),u=Jf(GS(ro(_fe(e.e)),e.d*e.a,e.c*e.b),-.5),t=n.a-u.a,a=n.b-u.b,v=0;v<e.c;v++){for(s=t,p=0;p<e.d;p++)Hkn(e.e,new hl(s,a,e.a,e.b))&&uR(e,p,v,!1,!0),s+=e.a;a+=e.b}}function mTn(e){var t,n,s;if(ft(lt(pt(e,(Tr(),JD))))){for(s=new qe,n=new Sn(Dn(Pd(e).a.Kc(),new L));rr(n);)t=h(Un(n),79),Pb(t)&&ft(lt(pt(t,Fre)))&&(s.c[s.c.length]=t);return s}else return Rt(),Rt(),ca}function l5(e){var t,n;if(n=!1,ue(e,204))return n=!0,h(e,204).a;if(!n&&ue(e,258)&&(t=h(e,258).a%1==0,t))return n=!0,Ke(fdn(h(e,258).a));throw ne(new Xf("Id must be a string or an integer: '"+e+"'."))}function yTn(e,t){var n,s,a,u,p,v;for(u=null,a=new _rt((!e.a&&(e.a=new kq(e)),e.a));Ebe(a);)if(n=h(JF(a),56),s=(p=n.Tg(),v=($y(p),p.o),!v||!n.mh(v)?null:S1e(UK(v),n.ah(v))),s!=null&&Nt(s,t)){u=n;break}return u}function g0t(e,t,n){var s,a,u,p,v;if(Uu(n,"occurrences"),n==0)return v=h(xw(U4(e.a),t),14),v?v.gc():0;if(p=h(xw(U4(e.a),t),14),!p)return 0;if(u=p.gc(),n>=u)p.$b();else for(a=p.Kc(),s=0;s<n;s++)a.Pb(),a.Qb();return u}function kTn(e,t,n){var s,a,u,p;return Uu(n,"oldCount"),Uu(0,"newCount"),s=h(xw(U4(e.a),t),14),(s?s.gc():0)==n?(Uu(0,"count"),a=(u=h(xw(U4(e.a),t),14),u?u.gc():0),p=-a,p>0?phe():p<0&&g0t(e,t,-p),!0):!1}function gM(e){var t,n,s,a,u,p,v;if(v=0,e.b==0){for(p=Xft(e,!0),t=0,s=p,a=0,u=s.length;a<u;++a)n=s[a],n>0&&(v+=n,++t);t>1&&(v+=e.c*(t-1))}else v=HXe(elt(qP(Jr(wW(e.a),new ss),new ls)));return v>0?v+e.n.d+e.n.a:0}function pM(e){var t,n,s,a,u,p,v;if(v=0,e.b==0)v=HXe(elt(qP(Jr(wW(e.a),new hi),new Hi)));else{for(p=Qft(e,!0),t=0,s=p,a=0,u=s.length;a<u;++a)n=s[a],n>0&&(v+=n,++t);t>1&&(v+=e.c*(t-1))}return v>0?v+e.n.b+e.n.c:0}function xTn(e,t){var n,s,a,u;for(u=h(wa(e.b,t),124),n=u.a,a=h(h(jr(e.r,t),21),84).Kc();a.Ob();)s=h(a.Pb(),111),s.c&&(n.a=d.Math.max(n.a,nde(s.c)));if(n.a>0)switch(t.g){case 2:u.n.c=e.s;break;case 4:u.n.b=e.s}}function ETn(e,t){var n,s,a;return n=h(U(t,(Jh(),qx)),19).a-h(U(e,qx),19).a,n==0?(s=ms(ro(h(U(e,(kg(),mD)),8)),h(U(e,b_),8)),a=ms(ro(h(U(t,mD),8)),h(U(t,b_),8)),Ui(s.a*s.b,a.a*a.b)):n}function _Tn(e,t){var n,s,a;return n=h(U(t,(Fb(),bH)),19).a-h(U(e,bH),19).a,n==0?(s=ms(ro(h(U(e,(fo(),zD)),8)),h(U(e,G_),8)),a=ms(ro(h(U(t,zD),8)),h(U(t,G_),8)),Ui(s.a*s.b,a.a*a.b)):n}function p0t(e){var t,n;return n=new rg,n.a+="e_",t=O5n(e),t!=null&&(n.a+=""+t),!!e.c&&!!e.d&&(or((n.a+=" ",n),$F(e.c)),or(ho((n.a+="[",n),e.c.i),"]"),or((n.a+=dJ,n),$F(e.d)),or(ho((n.a+="[",n),e.d.i),"]")),n.a}function b0t(e){switch(e.g){case 0:return new _Ue;case 1:return new TUe;case 2:return new EUe;case 3:return new CUe;default:throw ne(new an("No implementation is available for the layout phase "+(e.f!=null?e.f:""+e.g)))}}function mbe(e,t,n,s,a){var u;switch(u=0,a.g){case 1:u=d.Math.max(0,t.b+e.b-(n.b+s));break;case 3:u=d.Math.max(0,-e.b-s);break;case 2:u=d.Math.max(0,-e.a-s);break;case 4:u=d.Math.max(0,t.a+e.a-(n.a+s))}return u}function TTn(e,t,n){var s,a,u,p,v;if(n)for(a=n.a.length,s=new Mp(a),v=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);v.Ob();)p=h(v.Pb(),19),u=wk(n,p.a),Pye in u.a||Eee in u.a?BMn(e,u,t):zNn(e,u,t),u0n(h(_n(e.b,Jk(u)),79))}function ybe(e){var t,n;switch(e.b){case-1:return!0;case 0:return n=e.t,n>1||n==-1?(e.b=-1,!0):(t=Rl(e),!!t&&(ia(),t.Cj()==dyt)?(e.b=-1,!0):(e.b=1,!1));default:case 1:return!1}}function CTn(e,t){var n,s,a,u,p;for(s=(!t.s&&(t.s=new Ye(Ac,t,21,17)),t.s),u=null,a=0,p=s.i;a<p;++a)switch(n=h(he(s,a),170),fb(Sa(e,n))){case 2:case 3:!u&&(u=new qe),u.c[u.c.length]=n}return u||(Rt(),Rt(),ca)}function kbe(e,t){var n,s,a,u;if(Sr(e),e.c!=0||e.a!=123)throw ne(new Zn(ir((Qn(),Pmt))));if(u=t==112,s=e.d,n=nk(e.i,125,s),n<0)throw ne(new Zn(ir((Qn(),Bmt))));return a=$u(e.i,s,n),e.d=n+1,nct(a,u,(e.e&512)==512)}function STn(e){var t;if(t=h(U(e,(tt(),M_)),314),t==(e5(),I5))throw ne(new z7("The hierarchy aware processor "+t+" in child node "+e+" is only allowed if the root node specifies the same hierarchical processor."))}function ATn(e,t){wh();var n,s,a,u,p,v;for(n=null,p=t.Kc();p.Ob();)u=h(p.Pb(),128),!u.o&&(s=Vdn(u.a),a=L2n(u.a),v=new S9(s,a,null,h(u.d.a.ec().Kc().Pb(),17)),Ue(v.c,u.a),e.c[e.c.length]=v,n&&Ue(n.d,v),n=v)}function MTn(e,t){var n,s,a;if(!t)PK(e,null),Ik(e,null);else if((t.i&4)!=0)for(s="[]",n=t.c;;n=n.c){if((n.i&4)==0){a=fhe((yd(n),n.o+s)),PK(e,a),Ik(e,a);break}s+="[]"}else a=fhe((yd(t),t.o)),PK(e,a),Ik(e,a);e.yk(t)}function m9(e,t,n,s,a){var u,p,v,m;return m=$Y(e,h(a,56)),Me(m)!==Me(a)?(v=h(e.g[n],72),u=R1(t,m),wE(e,n,sQ(e,n,u)),Tu(e.e)&&(p=wg(e,9,u.ak(),a,m,s,!1),B2e(p,new Sd(e.e,9,e.c,v,u,s,!1)),_K(p)),m):a}function DTn(e,t,n){var s,a,u,p,v,m;for(s=h(jr(e.c,t),15),a=h(jr(e.c,n),15),u=s.Zc(s.gc()),p=a.Zc(a.gc());u.Sb()&&p.Sb();)if(v=h(u.Ub(),19),m=h(p.Ub(),19),v!=m)return dc(v.a,m.a);return!u.Ob()&&!p.Ob()?0:u.Ob()?1:-1}function v0t(e,t){var n,s,a;try{return a=kmn(e.a,t),a}catch(u){if(u=ui(u),ue(u,32)){try{if(s=Wu(t,$s,Ir),n=Vh(e.a),s>=0&&s<n.length)return n[s]}catch(p){if(p=ui(p),!ue(p,127))throw ne(p)}return null}else throw ne(u)}}function mQ(e,t){var n,s,a;if(a=Hy((jc(),Ps),e.Tg(),t),a)return ia(),h(a,66).Oj()||(a=Y4(Sa(Ps,a))),s=(n=e.Yg(a),h(n>=0?e._g(n,!0,!0):Bb(e,a,!0),153)),h(s,215).ll(t);throw ne(new an(Fg+t.ne()+gee))}function LTn(){Bhe();var e;return Z9t?h(lx((og(),th),Vl),1939):(wr(c2,new $Ge),fOn(),e=h(ue(Fo((og(),th),Vl),547)?Fo(th,Vl):new nit,547),Z9t=!0,cPn(e),dPn(e),Pr((Phe(),wEe),e,new nGe),Ta(th,Vl,e),e)}function ITn(e,t){var n,s,a,u;e.j=-1,Tu(e.e)?(n=e.i,u=e.i!=0,pA(e,t),s=new Sd(e.e,3,e.c,null,t,n,u),a=t.Qk(e.e,e.c,null),a=W1t(e,t,a),a?(a.Ei(s),a.Fi()):Or(e.e,s)):(pA(e,t),a=t.Qk(e.e,e.c,null),a&&a.Fi())}function XF(e,t){var n,s,a;if(a=0,s=t[0],s>=e.length)return-1;for(n=(er(s,e.length),e.charCodeAt(s));n>=48&&n<=57&&(a=a*10+(n-48),++s,!(s>=e.length));)n=(er(s,e.length),e.charCodeAt(s));return s>t[0]?t[0]=s:a=-1,a}function OTn(e){var t,n,s,a,u;return a=h(e.a,19).a,u=h(e.b,19).a,n=a,s=u,t=d.Math.max(d.Math.abs(a),d.Math.abs(u)),a<=0&&a==u?(n=0,s=u-1):a==-t&&u!=t?(n=u,s=a,u>=0&&++n):(n=-u,s=a),new As(Ke(n),Ke(s))}function NTn(e,t,n,s){var a,u,p,v,m,x;for(a=0;a<t.o;a++)for(u=a-t.j+n,p=0;p<t.p;p++)if(v=p-t.k+s,m=u,x=v,m+=e.j,x+=e.k,m>=0&&x>=0&&m<e.o&&x<e.p&&(!r1t(t,a,p)&&e1t(e,u,v)||Ay(t,a,p)&&!L8n(e,u,v)))return!0;return!1}function PTn(e,t,n){var s,a,u,p,v;p=e.c,v=e.d,u=Qa(se(ie(cs,1),Re,8,0,[p.i.n,p.n,p.a])).b,a=(u+Qa(se(ie(cs,1),Re,8,0,[v.i.n,v.n,v.a])).b)/2,s=null,p.j==(Qe(),pn)?s=new dt(t+p.i.c.c.a+n,a):s=new dt(t-n,a),rk(e.a,0,s)}function Pb(e){var t,n,s,a;for(t=null,s=gg(Cf(se(ie(Nf,1),st,20,0,[(!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c)])));rr(s);)if(n=h(Un(s),82),a=Na(n),!t)t=a;else if(t!=a)return!1;return!0}function yQ(e,t,n){var s;if(++e.j,t>=e.i)throw ne(new Ea(Tee+t+s2+e.i));if(n>=e.i)throw ne(new Ea(Cee+n+s2+e.i));return s=e.g[n],t!=n&&(t<n?Ao(e.g,t,e.g,t+1,n-t):Ao(e.g,n+1,e.g,n,t-n),pi(e.g,t,s),e.ei(t,s,n),e.ci()),s}function Ot(e,t,n){var s;if(s=h(e.c.xc(t),14),s)return s.Fc(n)?(++e.d,!0):!1;if(s=e.ic(t),s.Fc(n))return++e.d,e.c.zc(t,s),!0;throw ne(new T0e("New Collection violated the Collection spec"))}function bM(e){var t,n,s;return e<0?0:e==0?32:(s=-(e>>16),t=s>>16&16,n=16-t,e=e>>t,s=e-256,t=s>>16&8,n+=t,e<<=t,s=e-$w,t=s>>16&4,n+=t,e<<=t,s=e-a1,t=s>>16&2,n+=t,e<<=t,s=e>>14,t=s&~(s>>1),n+2-t)}function BTn(e){V4();var t,n,s,a;for(Kj=new qe,yte=new zn,mte=new qe,t=(!e.a&&(e.a=new Ye(wi,e,10,11)),e.a),gNn(t),a=new Tn(t);a.e!=a.i.gc();)s=h(On(a),33),Ha(Kj,s,0)==-1&&(n=new qe,Ue(mte,n),Aft(s,n));return mte}function FTn(e,t,n){var s,a,u,p;e.a=n.b.d,ue(t,352)?(a=Fy(h(t,79),!1,!1),u=hM(a),s=new Qqe(e),Os(u,s),CM(u,a),t.We((Tr(),x3))!=null&&Os(h(t.We(x3),74),s)):(p=h(t,470),p.Hg(p.Dg()+e.a.a),p.Ig(p.Eg()+e.a.b))}function w0t(e,t){var n,s,a,u,p,v,m,x;for(x=Pe(Je(U(t,(tt(),I_)))),m=e[0].n.a+e[0].o.a+e[0].d.c+x,v=1;v<e.length;v++)s=e[v].n,a=e[v].o,n=e[v].d,u=s.a-n.b-m,u<0&&(s.a-=u),p=t.f,p.a=d.Math.max(p.a,s.a+a.a),m=s.a+a.a+n.c+x}function RTn(e,t){var n,s,a,u,p,v;return s=h(h(_n(e.g,t.a),46).a,65),a=h(h(_n(e.g,t.b),46).a,65),u=s.b,p=a.b,n=nIn(u,p),n>=0?n:(v=NE(ms(new dt(p.c+p.b/2,p.d+p.a/2),new dt(u.c+u.b/2,u.d+u.a/2))),-($pt(u,p)-1)*v)}function jTn(e,t,n){var s;Ti(new Vt(null,(!n.a&&(n.a=new Ye(gi,n,6,6)),new Wt(n.a,16))),new eJe(e,t)),Ti(new Vt(null,(!n.n&&(n.n=new Ye(Ua,n,1,7)),new Wt(n.n,16))),new tJe(e,t)),s=h(pt(n,(Tr(),x3)),74),s&&Tge(s,e,t)}function Bb(e,t,n){var s,a,u;if(u=Hy((jc(),Ps),e.Tg(),t),u)return ia(),h(u,66).Oj()||(u=Y4(Sa(Ps,u))),a=(s=e.Yg(u),h(s>=0?e._g(s,!0,!0):Bb(e,u,!0),153)),h(a,215).hl(t,n);throw ne(new an(Fg+t.ne()+gee))}function xbe(e,t,n,s){var a,u,p,v,m;if(a=e.d[t],a){if(u=a.g,m=a.i,s!=null){for(v=0;v<m;++v)if(p=h(u[v],133),p.Sh()==n&&Nr(s,p.cd()))return p}else for(v=0;v<m;++v)if(p=h(u[v],133),Me(p.cd())===Me(s))return p}return null}function vM(e,t){var n;if(t<0)throw ne(new cN("Negative exponent"));if(t==0)return Fj;if(t==1||Dpe(e,Fj)||Dpe(e,Gx))return e;if(!z0t(e,0)){for(n=1;!z0t(e,n);)++n;return vy(Jkn(n*t),vM(A0e(e,n),t))}return c9n(e,t)}function $Tn(e,t){var n,s,a;if(Me(e)===Me(t))return!0;if(e==null||t==null||e.length!=t.length)return!1;for(n=0;n<e.length;++n)if(s=e[n],a=t[n],!(Me(s)===Me(a)||s!=null&&Nr(s,a)))return!1;return!0}function m0t(e){Ohe();var t,n,s;for(this.b=g5t,this.c=(la(),Q1),this.f=(oZe(),d5t),this.a=e,khe(this,new HO),nR(this),s=new Q(e.b);s.a<s.c.c.length;)n=h(K(s),81),n.d||(t=new AX(se(ie(Ste,1),st,81,0,[n])),Ue(e.a,t))}function HTn(e,t,n){var s,a,u,p,v,m;if(!e||e.c.length==0)return null;for(u=new Uit(t,!n),a=new Q(e);a.a<a.c.c.length;)s=h(K(a),70),nbe(u,(Y7(),new ZO(s)));return p=u.i,p.a=(m=u.n,u.e.b+m.d+m.a),p.b=(v=u.n,u.e.a+v.b+v.c),u}function y0t(e){var t,n,s,a,u,p,v;for(v=hA(e.a),yfe(v,new vFe),n=null,a=v,u=0,p=a.length;u<p&&(s=a[u],s.k==(bn(),Pi));++u)t=h(U(s,(He(),co)),61),!(t!=(Qe(),un)&&t!=pn)&&(n&&h(U(n,B5),15).Fc(s),n=s)}function zTn(e,t,n){var s,a,u,p,v,m,x;m=(Kt(t,e.c.length),h(e.c[t],329)),f0(e,t),m.b/2>=n&&(s=t,x=(m.c+m.a)/2,p=x-n,m.c<=x-n&&(a=new ZY(m.c,p),aw(e,s++,a)),v=x+n,v<=m.a&&(u=new ZY(v,m.a),fw(s,e.c.length),J7(e.c,s,u)))}function Ebe(e){var t;if(!e.c&&e.g==null)e.d=e.si(e.f),qn(e,e.d),t=e.d;else{if(e.g==null)return!0;if(e.i==0)return!1;t=h(e.g[e.i-1],47)}return t==e.b&&null.km>=null.jm()?(JF(e),Ebe(e)):t.Ob()}function GTn(e,t,n){var s,a,u,p,v;if(v=n,!v&&(v=ede(new H6,0)),Bn(v,Nvt,1),$2t(e.c,t),p=rOn(e.a,t),p.gc()==1)x2t(h(p.Xb(0),37),v);else for(u=1/p.gc(),a=p.Kc();a.Ob();)s=h(a.Pb(),37),x2t(s,jo(v,u));Qfn(e.a,p,t),rMn(t),An(v)}function k0t(e){if(this.a=e,e.c.i.k==(bn(),Pi))this.c=e.c,this.d=h(U(e.c.i,(He(),co)),61);else if(e.d.i.k==Pi)this.c=e.d,this.d=h(U(e.d.i,(He(),co)),61);else throw ne(new an("Edge "+e+" is not an external edge."))}function x0t(e,t){var n,s,a;a=e.b,e.b=t,(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,3,a,e.b)),t?t!=e&&(Ko(e,t.zb),IK(e,t.d),n=(s=t.c,s==null?t.zb:s),NK(e,n==null||Nt(n,t.zb)?null:n)):(Ko(e,null),IK(e,0),NK(e,null))}function E0t(e){var t,n;if(e.f){for(;e.n<e.o;){if(t=h(e.j?e.j.pi(e.n):e.k.Xb(e.n),72),n=t.ak(),ue(n,99)&&(h(n,18).Bb&go)!=0&&(!e.e||n.Gj()!=E8||n.aj()!=0)&&t.dd()!=null)return!0;++e.n}return!1}else return e.n<e.o}function _0t(e,t){var n;this.e=(lw(),Vn(e),lw(),Ppe(e)),this.c=(Vn(t),Ppe(t)),$fe(this.e.Hd().dc()==this.c.Hd().dc()),this.d=Vht(this.e),this.b=Vht(this.c),n=Ap(yn,[Re,st],[5,1],5,[this.e.Hd().gc(),this.c.Hd().gc()],2),this.a=n,I4n(this)}function T0t(e){!Jee&&(Jee=ENn());var t=e.replace(/[\x00-\x1f\xad\u0600-\u0603\u06dd\u070f\u17b4\u17b5\u200b-\u200f\u2028-\u202e\u2060-\u2064\u206a-\u206f\ufeff\ufff9-\ufffb"\\]/g,function(n){return Dwn(n)});return'"'+t+'"'}function C0t(e){Yde();var t,n;for(this.b=N3t,this.c=B3t,this.g=(aZe(),O3t),this.d=(la(),Q1),this.a=e,qbe(this),n=new Q(e.b);n.a<n.c.c.length;)t=h(K(n),57),!t.a&&Uet(Eut(new Xle,se(ie(Gj,1),st,57,0,[t])),e),t.e=new eB(t.d)}function VTn(e){var t,n,s,a,u,p;for(a=e.e.c.length,s=pe(xl,Hw,15,a,0,1),p=new Q(e.e);p.a<p.c.c.length;)u=h(K(p),144),s[u.b]=new di;for(n=new Q(e.c);n.a<n.c.c.length;)t=h(K(n),282),s[t.c.b].Fc(t),s[t.d.b].Fc(t);return s}function UTn(e){var t,n,s,a,u,p,v;for(v=I1(e.c.length),a=new Q(e);a.a<a.c.c.length;){for(s=h(K(a),10),p=new ns,u=qi(s),n=new Sn(Dn(u.a.Kc(),new L));rr(n);)t=h(Un(n),17),t.c.i==t.d.i||Qi(p,t.d.i);v.c[v.c.length]=p}return v}function qTn(e,t){var n,s,a,u,p;if(n=h(tn(e.a,4),126),p=n==null?0:n.length,t>=p)throw ne(new sw(t,p));return a=n[t],p==1?s=null:(s=pe(nie,Lee,415,p-1,0,1),Ao(n,0,s,0,t),u=p-t-1,u>0&&Ao(n,t+1,s,t,u)),ex(e,s),qdt(e,t,a),a}function h5(){h5=re,Z5=h(he(De((_he(),bo).qb),6),34),Q5=h(he(De(bo.qb),3),34),cie=h(he(De(bo.qb),4),34),uie=h(he(De(bo.qb),5),18),VF(Z5),VF(Q5),VF(cie),VF(uie),n_t=new _u(se(ie(Ac,1),Jy,170,0,[Z5,Q5]))}function S0t(e,t){var n;this.d=new B7,this.b=t,this.e=new _a(t.qf()),n=e.u.Hc((Qc(),aL)),e.u.Hc(Ud)?e.D?this.a=n&&!t.If():this.a=!0:e.u.Hc(Zg)?n?this.a=!(t.zf().Kc().Ob()||t.Bf().Kc().Ob()):this.a=!1:this.a=!1}function A0t(e,t){var n,s,a,u;for(n=e.o.a,u=h(h(jr(e.r,t),21),84).Kc();u.Ob();)a=h(u.Pb(),111),a.e.a=(s=a.b,s.Xe((Tr(),pu))?s.Hf()==(Qe(),un)?-s.rf().a-Pe(Je(s.We(pu))):n+Pe(Je(s.We(pu))):s.Hf()==(Qe(),un)?-s.rf().a:n)}function M0t(e,t){var n,s,a,u;n=h(U(e,(tt(),Ju)),103),u=h(pt(t,u8),61),a=h(U(e,Ei),98),a!=(_s(),$f)&&a!=Qg?u==(Qe(),to)&&(u=kve(t,n),u==to&&(u=i5(n))):(s=k2t(t),s>0?u=i5(n):u=KA(i5(n))),ya(t,u8,u)}function YTn(e,t){var n,s,a,u,p;for(p=e.j,t.a!=t.b&&hs(p,new Pje),a=p.c.length/2|0,s=0;s<a;s++)u=(Kt(s,p.c.length),h(p.c[s],113)),u.c&&Ji(u.d,t.a);for(n=a;n<p.c.length;n++)u=(Kt(n,p.c.length),h(p.c[n],113)),u.c&&Ji(u.d,t.b)}function WTn(e,t,n){var s,a,u;return s=e.c[t.c.p][t.p],a=e.c[n.c.p][n.p],s.a!=null&&a.a!=null?(u=dW(s.a,a.a),u<0?EM(e,t,n):u>0&&EM(e,n,t),u):s.a!=null?(EM(e,t,n),-1):a.a!=null?(EM(e,n,t),1):0}function D0t(e,t){var n,s,a,u;e.ej()?(n=e.Vi(),u=e.fj(),++e.j,e.Hi(n,e.oi(n,t)),s=e.Zi(3,null,t,n,u),e.bj()?(a=e.cj(t,null),a?(a.Ei(s),a.Fi()):e.$i(s)):e.$i(s)):(qrt(e,t),e.bj()&&(a=e.cj(t,null),a&&a.Fi()))}function QF(e,t){var n,s,a,u,p;for(p=ac(e.e.Tg(),t),a=new UO,n=h(e.g,119),u=e.i;--u>=0;)s=n[u],p.rl(s.ak())&&qn(a,s);!mbt(e,a)&&Tu(e.e)&&$6(e,t.$j()?wg(e,6,t,(Rt(),ca),null,-1,!1):wg(e,t.Kj()?2:1,t,null,null,-1,!1))}function cx(){cx=re;var e,t;for(M5=pe(t3,Re,91,32,0,1),d_=pe(t3,Re,91,32,0,1),e=1,t=0;t<=18;t++)M5[t]=eM(e),d_[t]=eM(kd(e,t)),e=js(e,5);for(;t<d_.length;t++)M5[t]=vy(M5[t-1],M5[1]),d_[t]=vy(d_[t-1],(Dg(),ste))}function KTn(e,t){var n,s,a,u,p;return e.a==(ax(),x_)?!0:(u=t.a.c,n=t.a.c+t.a.b,!(t.j&&(s=t.A,p=s.c.c.a-s.o.a/2,a=u-(s.n.a+s.o.a),a>p)||t.q&&(s=t.C,p=s.c.c.a-s.o.a/2,a=s.n.a-n,a>p)))}function XTn(e,t){var n;Bn(t,"Partition preprocessing",1),n=h(Gu(Jr(Xa(Jr(new Vt(null,new Wt(e.a,16)),new cRe),new uRe),new lRe),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),Ti(n.Oc(),new hRe),An(t)}function L0t(e){YW();var t,n,s,a,u,p,v;for(n=new Np,a=new Q(e.e.b);a.a<a.c.c.length;)for(s=h(K(a),29),p=new Q(s.a);p.a<p.c.c.length;)u=h(K(p),10),v=e.g[u.p],t=h(mb(n,v),15),t||(t=new qe,Yp(n,v,t)),t.Fc(u);return n}function QTn(e,t){var n,s,a,u,p;for(a=t.b.b,e.a=pe(xl,Hw,15,a,0,1),e.b=pe(vu,Y1,25,a,16,1),p=pr(t.b,0);p.b!=p.d.c;)u=h(gr(p),86),e.a[u.g]=new di;for(s=pr(t.a,0);s.b!=s.d.c;)n=h(gr(s),188),e.a[n.b.g].Fc(n),e.a[n.c.g].Fc(n)}function I0t(e){var t;return(e.Db&64)!=0?_h(e):(t=new Ol(_h(e)),t.a+=" (startX: ",ny(t,e.j),t.a+=", startY: ",ny(t,e.k),t.a+=", endX: ",ny(t,e.b),t.a+=", endY: ",ny(t,e.c),t.a+=", identifier: ",va(t,e.d),t.a+=")",t.a)}function _be(e){var t;return(e.Db&64)!=0?n9(e):(t=new Ol(n9(e)),t.a+=" (ordered: ",a0(t,(e.Bb&256)!=0),t.a+=", unique: ",a0(t,(e.Bb&512)!=0),t.a+=", lowerBound: ",Gq(t,e.s),t.a+=", upperBound: ",Gq(t,e.t),t.a+=")",t.a)}function O0t(e,t,n,s,a,u,p,v){var m;return ue(e.Cb,88)&&Lw(ou(h(e.Cb,88)),4),Ko(e,n),e.f=s,Kk(e,a),Qk(e,u),Wk(e,p),Xk(e,!1),m0(e,!0),Zk(e,v),w0(e,!0),p0(e,0),e.b=0,yw(e,1),m=Df(e,t,null),m&&m.Fi(),bX(e,!1),e}function N0t(e,t){var n,s,a,u;return n=h(Fo(e.a,t),512),n||(s=new uK(t),a=(wB(),Pf?null:s.c),u=$u(a,0,d.Math.max(0,aP(a,Ec(46)))),xgn(s,N0t(e,u)),(Pf?null:s.c).length==0&&vtt(s,new me),Ta(e.a,Pf?null:s.c,s),s)}function ZTn(e,t){var n;e.b=t,e.g=new qe,n=rCn(e.b),e.e=n,e.f=n,e.c=ft(lt(U(e.b,(HF(),r4e)))),e.a=Je(U(e.b,(Tr(),U5))),e.a==null&&(e.a=1),Pe(e.a)>1?e.e*=Pe(e.a):e.f/=Pe(e.a),L6n(e),Bxn(e),wMn(e),Fe(e.b,(l9(),Wj),e.g)}function P0t(e,t,n){var s,a,u,p,v,m;for(s=0,m=n,t||(s=n*(e.c.length-1),m*=-1),u=new Q(e);u.a<u.c.c.length;){for(a=h(K(u),10),Fe(a,(tt(),m1),($1(),AH)),a.o.a=s,v=x0(a,(Qe(),pn)).Kc();v.Ob();)p=h(v.Pb(),11),p.n.a=s;s+=m}}function Tbe(e,t,n){var s,a,u;e.ej()?(u=e.fj(),YA(e,t,n),s=e.Zi(3,null,n,t,u),e.bj()?(a=e.cj(n,null),e.ij()&&(a=e.jj(n,a)),a?(a.Ei(s),a.Fi()):e.$i(s)):e.$i(s)):(YA(e,t,n),e.bj()&&(a=e.cj(n,null),a&&a.Fi()))}function f5(e,t,n){var s,a,u,p,v,m;return v=e.Gk(n),v!=n?(p=e.g[t],m=v,wE(e,t,e.oi(t,m)),u=p,e.gi(t,m,u),e.rk()&&(s=n,a=e.dj(s,null),!h(v,49).eh()&&(a=e.cj(m,a)),a&&a.Fi()),Tu(e.e)&&$6(e,e.Zi(9,n,v,t,!1)),v):n}function JTn(e,t){var n,s,a,u;for(s=new Q(e.a.a);s.a<s.c.c.length;)n=h(K(s),189),n.g=!0;for(u=new Q(e.a.b);u.a<u.c.c.length;)a=h(K(u),81),a.k=ft(lt(e.e.Kb(new As(a,t)))),a.d.g=a.d.g&ft(lt(e.e.Kb(new As(a,t))));return e}function B0t(e){var t,n,s,a,u;if(n=(t=h(Vh(ta),9),new ll(t,h(vh(t,t.length),9),0)),u=h(U(e,(He(),Zc)),10),u)for(a=new Q(u.j);a.a<a.c.c.length;)s=h(K(a),11),Me(U(s,Br))===Me(e)&&kE(new _f(s.b))&&Eh(n,s.j);return n}function F0t(e,t,n){var s,a,u,p,v;if(!e.d[n.p]){for(a=new Sn(Dn(qi(n).a.Kc(),new L));rr(a);){for(s=h(Un(a),17),v=s.d.i,p=new Sn(Dn(za(v).a.Kc(),new L));rr(p);)u=h(Un(p),17),u.c.i==t&&(e.a[u.p]=!0);F0t(e,t,v)}e.d[n.p]=!0}}function eCn(e,t){var n,s,a,u,p,v,m;if(s=Ght(e.Db&254),s==1)e.Eb=null;else if(u=Fp(e.Eb),s==2)a=WX(e,t),e.Eb=u[a==0?1:0];else{for(p=pe(yn,st,1,s-1,5,1),n=2,v=0,m=0;n<=128;n<<=1)n==t?++v:(e.Db&n)!=0&&(p[m++]=u[v++]);e.Eb=p}e.Db&=~t}function tCn(e,t){var n,s,a,u,p;for(s=(!t.s&&(t.s=new Ye(Ac,t,21,17)),t.s),u=null,a=0,p=s.i;a<p;++a)switch(n=h(he(s,a),170),fb(Sa(e,n))){case 4:case 5:case 6:{!u&&(u=new qe),u.c[u.c.length]=n;break}}return u||(Rt(),Rt(),ca)}function Cbe(e){var t;switch(t=0,e){case 105:t=2;break;case 109:t=8;break;case 115:t=4;break;case 120:t=16;break;case 117:t=32;break;case 119:t=64;break;case 70:t=256;break;case 72:t=128;break;case 88:t=512;break;case 44:t=Ch}return t}function nCn(e,t,n,s,a){var u,p,v,m;if(Me(e)===Me(t)&&s==a){Opt(e,s,n);return}for(v=0;v<s;v++){for(p=0,u=e[v],m=0;m<a;m++)p=Vs(Vs(js(Zi(u,fa),Zi(t[m],fa)),Zi(n[v+m],fa)),Zi(Gn(p),fa)),n[v+m]=Gn(p),p=ow(p,32);n[v+a]=Gn(p)}}function rCn(e){var t,n,s,a,u,p,v,m,x,_,A;for(_=0,x=0,a=e.a,v=a.a.gc(),s=a.a.ec().Kc();s.Ob();)n=h(s.Pb(),561),t=(n.b&&WQ(n),n.a),A=t.a,p=t.b,_+=A+p,x+=A*p;return m=d.Math.sqrt(400*v*x-4*x+_*_)+_,u=2*(100*v-1),u==0?m:m/u}function R0t(e,t){t.b!=0&&(isNaN(e.s)?e.s=Pe((kn(t.b!=0),Je(t.a.a.c))):e.s=d.Math.min(e.s,Pe((kn(t.b!=0),Je(t.a.a.c)))),isNaN(e.c)?e.c=Pe((kn(t.b!=0),Je(t.c.b.c))):e.c=d.Math.max(e.c,Pe((kn(t.b!=0),Je(t.c.b.c)))))}function y9(e){var t,n,s,a;for(t=null,s=gg(Cf(se(ie(Nf,1),st,20,0,[(!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c)])));rr(s);)if(n=h(Un(s),82),a=Na(n),!t)t=bi(a);else if(t!=bi(a))return!0;return!1}function kQ(e,t){var n,s,a,u;e.ej()?(n=e.i,u=e.fj(),pA(e,t),s=e.Zi(3,null,t,n,u),e.bj()?(a=e.cj(t,null),e.ij()&&(a=e.jj(t,a)),a?(a.Ei(s),a.Fi()):e.$i(s)):e.$i(s)):(pA(e,t),e.bj()&&(a=e.cj(t,null),a&&a.Fi()))}function j0t(e,t,n){var s,a,u;e.ej()?(u=e.fj(),++e.j,e.Hi(t,e.oi(t,n)),s=e.Zi(3,null,n,t,u),e.bj()?(a=e.cj(n,null),a?(a.Ei(s),a.Fi()):e.$i(s)):e.$i(s)):(++e.j,e.Hi(t,e.oi(t,n)),e.bj()&&(a=e.cj(n,null),a&&a.Fi()))}function iCn(e){var t,n,s,a;for(a=e.length,t=null,s=0;s<a;s++)n=(er(s,e.length),e.charCodeAt(s)),Zf(".*+?{[()|\\^$",Ec(n))>=0?(t||(t=new G7,s>0&&va(t,e.substr(0,s))),t.a+="\\",hk(t,n&Oi)):t&&hk(t,n&Oi);return t?t.a:e}function sCn(e){var t;if(!e.a)throw ne(new ja("IDataType class expected for layout option "+e.f));if(t=dyn(e.a),t==null)throw ne(new ja("Couldn't create new instance of property '"+e.f+"'. "+Iwt+(yd(pL),pL.k)+Cye));return h(t,414)}function xQ(e){var t,n,s,a,u;return u=e.eh(),u&&u.kh()&&(a=Tg(e,u),a!=u)?(n=e.Vg(),s=(t=e.Vg(),t>=0?e.Qg(null):e.eh().ih(e,-1-t,null,null)),e.Rg(h(a,49),n),s&&s.Fi(),e.Lg()&&e.Mg()&&n>-1&&Or(e,new fs(e,9,n,u,a)),a):u}function $0t(e){var t,n,s,a,u,p,v,m;for(p=0,u=e.f.e,s=0;s<u.c.length;++s)for(v=(Kt(s,u.c.length),h(u.c[s],144)),a=s+1;a<u.c.length;++a)m=(Kt(a,u.c.length),h(u.c[a],144)),n=yg(v.d,m.d),t=n-e.a[v.b][m.b],p+=e.i[v.b][m.b]*t*t;return p}function aCn(e,t){var n;if(!os(t,(tt(),cc))&&(n=L7n(h(U(t,B5e),360),h(U(e,cc),163)),Fe(t,B5e,n),!rr(new Sn(Dn(Id(t).a.Kc(),new L)))))switch(n.g){case 1:Fe(t,cc,(wl(),C_));break;case 2:Fe(t,cc,(wl(),S_))}}function oCn(e,t){var n;vMn(e),e.a=(n=new jq,Ti(new Vt(null,new Wt(t.d,16)),new RYe(n)),n),SAn(e,h(U(t.b,(tt(),Tne)),376)),y8n(e),fCn(e),O7n(e),k8n(e),bIn(e,t),Ti(Xa(new Vt(null,Qst(Ubn(e.b).a)),new Bje),new Fje),t.a=!1,e.a=null}function H0t(){W2e.call(this,Bx,(X2(),m9t)),this.p=null,this.a=null,this.f=null,this.n=null,this.g=null,this.c=null,this.i=null,this.j=null,this.d=null,this.b=null,this.e=null,this.k=null,this.o=null,this.s=null,this.q=!1,this.r=!1}function ux(){ux=re,eie=new I4(nwt,0),qH=new I4("INSIDE_SELF_LOOPS",1),YH=new I4("MULTI_EDGES",2),UH=new I4("EDGE_LABELS",3),Jre=new I4("PORTS",4),VH=new I4("COMPOUND",5),GH=new I4("CLUSTERS",6),Zre=new I4("DISCONNECTED",7)}function z0t(e,t){var n,s,a;if(t==0)return(e.a[0]&1)!=0;if(t<0)throw ne(new cN("Negative bit address"));if(a=t>>5,a>=e.d)return e.e<0;if(n=e.a[a],t=1<<(t&31),e.e<0){if(s=slt(e),a<s)return!1;s==a?n=-n:n=~n}return(n&t)!=0}function cCn(e,t,n,s){var a;h(n.b,65),h(n.b,65),h(s.b,65),h(s.b,65),a=ms(ro(h(n.b,65).c),h(s.b,65).c),kP(a,r0t(h(n.b,65),h(s.b,65),a)),h(s.b,65),h(s.b,65),h(s.b,65).c.a+a.a,h(s.b,65).c.b+a.b,h(s.b,65),mc(s.a,new $1e(e,t,s))}function Sbe(e,t){var n,s,a,u,p,v,m;if(u=t.e,u){for(n=xQ(u),s=h(e.g,674),p=0;p<e.i;++p)if(m=s[p],zX(m)==n&&(a=(!m.d&&(m.d=new Gi(pa,m,1)),m.d),v=h(n.ah(RQ(u,u.Cb,u.Db>>16)),15).Xc(u),v<a.i))return Sbe(e,h(he(a,v),87))}return t}function M(e,t,n){var s=Oj,a,u=s[e],p=u instanceof Array?u[0]:null;u&&!p?g=u:(g=(a=t&&t.prototype,!a&&(a=Oj[t]),Iwn(a)),g.hm=n,!t&&(g.im=de),s[e]=g);for(var v=3;v<arguments.length;++v)arguments[v].prototype=g;p&&(g.gm=p)}function rr(e){for(var t;!h(Vn(e.a),47).Ob();){if(e.d=h6n(e),!e.d)return!1;if(e.a=h(e.d.Pb(),47),ue(e.a,39)){if(t=h(e.a,39),e.a=t.a,!e.b&&(e.b=new Jm),mg(e.b,e.d),t.b)for(;!$7(t.b);)mg(e.b,h(Mbn(t.b),47));e.d=t.d}}return!0}function Abe(e,t){var n,s,a,u,p;for(u=t==null?0:e.b.se(t),s=(n=e.a.get(u),n==null?new Array:n),p=0;p<s.length;p++)if(a=s[p],e.b.re(t,a.cd()))return s.length==1?(s.length=0,J0n(e.a,u)):s.splice(p,1),--e.c,JP(e.b),a.dd();return null}function Mbe(e,t){var n,s,a,u;for(a=1,t.j=!0,u=null,s=new Q(Ty(t));s.a<s.c.c.length;)n=h(K(s),213),e.c[n.c]||(e.c[n.c]=!0,u=mF(n,t),n.f?a+=Mbe(e,u):!u.j&&n.a==n.e.e-n.d.e&&(n.f=!0,Qi(e.p,n),a+=Mbe(e,u)));return a}function uCn(e){var t,n,s;for(n=new Q(e.a.a.b);n.a<n.c.c.length;)t=h(K(n),81),s=(rn(0),0),s>0&&(!(lg(e.a.c)&&t.n.d)&&!(ek(e.a.c)&&t.n.b)&&(t.g.d+=d.Math.max(0,s/2-.5)),!(lg(e.a.c)&&t.n.a)&&!(ek(e.a.c)&&t.n.c)&&(t.g.a-=s-1))}function G0t(e){var t,n,s,a,u;if(a=new qe,u=Bpt(e,a),t=h(U(e,(He(),Zc)),10),t)for(s=new Q(t.j);s.a<s.c.c.length;)n=h(K(s),11),Me(U(n,Br))===Me(e)&&(u=d.Math.max(u,Bpt(n,a)));return a.c.length==0||Fe(e,h3,u),u!=-1?a:null}function V0t(e,t,n){var s,a,u,p,v,m;u=h(ut(t.e,0),17).c,s=u.i,a=s.k,m=h(ut(n.g,0),17).d,p=m.i,v=p.k,a==(bn(),ds)?Fe(e,(He(),rf),h(U(s,rf),11)):Fe(e,(He(),rf),u),v==ds?Fe(e,(He(),Yl),h(U(p,Yl),11)):Fe(e,(He(),Yl),m)}function U0t(e,t){var n,s,a,u;for(u=Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))),n=u&e.b.length-1,a=null,s=e.b[n];s;a=s,s=s.a)if(s.d==u&&n1(s.i,t))return a?a.a=s.a:e.b[n]=s.a,BXe(s.c,s.f),gS(s.b,s.e),--e.f,++e.e,!0;return!1}function Dbe(e,t){var n,s,a,u,p;return t&=63,n=e.h,s=(n&FM)!=0,s&&(n|=-1048576),t<22?(p=n>>t,u=e.m>>t|n<<22-t,a=e.l>>t|e.m<<22-t):t<44?(p=s?Rd:0,u=n>>t-22,a=e.m>>t-22|n<<44-t):(p=s?Rd:0,u=s?du:0,a=n>>t-44),rc(a&du,u&du,p&Rd)}function EQ(e){var t,n,s,a,u,p;for(this.c=new qe,this.d=e,s=ki,a=ki,t=Fi,n=Fi,p=pr(e,0);p.b!=p.d.c;)u=h(gr(p),8),s=d.Math.min(s,u.a),a=d.Math.min(a,u.b),t=d.Math.max(t,u.a),n=d.Math.max(n,u.b);this.a=new hl(s,a,t-s,n-a)}function q0t(e,t){var n,s,a,u,p,v;for(u=new Q(e.b);u.a<u.c.c.length;)for(a=h(K(u),29),v=new Q(a.a);v.a<v.c.c.length;)for(p=h(K(v),10),p.k==(bn(),Ou)&&d5(p,t),s=new Sn(Dn(qi(p).a.Kc(),new L));rr(s);)n=h(Un(s),17),Mlt(n,t)}function Lbe(e){var t,n,s;this.c=e,s=h(U(e,(tt(),Ju)),103),t=Pe(Je(U(e,j$))),n=Pe(Je(U(e,Pke))),s==(la(),Wl)||s==Dh||s==Q1?this.b=t*n:this.b=1/(t*n),this.j=Pe(Je(U(e,b3))),this.e=Pe(Je(U(e,uv))),this.f=e.b.c.length}function lCn(e){var t,n;for(e.e=pe(Hn,hr,25,e.p.c.length,15,1),e.k=pe(Hn,hr,25,e.p.c.length,15,1),n=new Q(e.p);n.a<n.c.c.length;)t=h(K(n),10),e.e[t.p]=Td(new Sn(Dn(za(t).a.Kc(),new L))),e.k[t.p]=Td(new Sn(Dn(qi(t).a.Kc(),new L)))}function hCn(e){var t,n,s,a,u,p;for(a=0,e.q=new qe,t=new ns,p=new Q(e.p);p.a<p.c.c.length;){for(u=h(K(p),10),u.p=a,s=new Sn(Dn(qi(u).a.Kc(),new L));rr(s);)n=h(Un(s),17),Qi(t,n.d.i);t.a.Bc(u)!=null,Ue(e.q,new CE(t)),t.a.$b(),++a}}function Fb(){Fb=re,Txe=new nb(20),Z8t=new sa((Tr(),y2),Txe),Sxe=new sa(k2,20),Y8t=new sa(U5,Mx),bH=new sa(NH,Ke(1)),e7t=new sa(Ure,(on(),!0)),Exe=ZD,K8t=m2,X8t=_3,Q8t=Wg,W8t=E3,_xe=tL,J8t=gm,t7t=(Lge(),q8t),Cxe=U8t}function Y0t(e,t){var n,s,a,u,p,v,m,x,_;if(e.a.f>0&&ue(t,42)&&(e.a.qj(),x=h(t,42),m=x.cd(),u=m==null?0:ii(m),p=p1e(e.a,u),n=e.a.d[p],n)){for(s=h(n.g,367),_=n.i,v=0;v<_;++v)if(a=s[v],a.Sh()==u&&a.Fb(x))return Y0t(e,x),!0}return!1}function fCn(e){var t,n,s,a;for(a=h(jr(e.a,(Iy(),E$)),15).Kc();a.Ob();)s=h(a.Pb(),101),n=(t=$4(s.k),t.Hc((Qe(),sn))?t.Hc(pn)?t.Hc(Fn)?t.Hc(un)?null:W5t:X5t:K5t:Y5t),lk(e,s,n[0],(Tb(),Jb),0),lk(e,s,n[1],zg,1),lk(e,s,n[2],ev,1)}function dCn(e,t){var n,s;n=HMn(t),jAn(e,t,n),Z1t(e.a,h(U(Us(t.b),(He(),R5)),230)),fDn(e),$En(e,t),s=pe(Hn,hr,25,t.b.j.c.length,15,1),sZ(e,t,(Qe(),sn),s,n),sZ(e,t,pn,s,n),sZ(e,t,Fn,s,n),sZ(e,t,un,s,n),e.a=null,e.c=null,e.b=null}function Ibe(){Ibe=re,n8e=(vF(),wre),P7t=new $t(dye,n8e),O7t=new $t(gye,(on(),!0)),Ke(-1),D7t=new $t(pye,Ke(-1)),Ke(-1),L7t=new $t(bye,Ke(-1)),N7t=new $t(vye,!1),B7t=new $t(wye,!0),I7t=new $t(ree,!1),F7t=new $t(mye,-1)}function Obe(e,t,n){switch(t){case 7:!e.e&&(e.e=new qt(us,e,7,4)),jn(e.e),!e.e&&(e.e=new qt(us,e,7,4)),mi(e.e,h(n,14));return;case 8:!e.d&&(e.d=new qt(us,e,8,5)),jn(e.d),!e.d&&(e.d=new qt(us,e,8,5)),mi(e.d,h(n,14));return}D2e(e,t,n)}function Nbe(e,t){var n,s,a,u,p;if(Me(t)===Me(e))return!0;if(!ue(t,15)||(p=h(t,15),e.gc()!=p.gc()))return!1;for(u=p.Kc(),s=e.Kc();s.Ob();)if(n=s.Pb(),a=u.Pb(),!(Me(n)===Me(a)||n!=null&&Nr(n,a)))return!1;return!0}function gCn(e,t){var n,s,a,u;for(u=h(Gu(Xa(Xa(new Vt(null,new Wt(t.b,16)),new SFe),new AFe),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),u.Jc(new MFe),n=0,a=u.Kc();a.Ob();)s=h(a.Pb(),11),s.p==-1&&Bbe(e,s,n++)}function W0t(e){switch(e.g){case 0:return new PUe;case 1:return new mUe;case 2:return new wUe;case 3:return new bJe;case 4:return new srt;default:throw ne(new an("No implementation is available for the node placer "+(e.f!=null?e.f:""+e.g)))}}function K0t(e){switch(e.g){case 0:return new dde;case 1:return new kUe;case 2:return new hUe;case 3:return new fUe;case 4:return new wJe;default:throw ne(new an("No implementation is available for the cycle breaker "+(e.f!=null?e.f:""+e.g)))}}function Pbe(){Pbe=re,b7t=new $t(oye,Ke(0)),v7t=new $t(cye,0),Rxe=(VA(),wH),g7t=new $t(eee,Rxe),Ke(0),d7t=new $t(tee,Ke(1)),$xe=(tF(),vre),w7t=new $t(uye,$xe),Hxe=(SB(),dre),m7t=new $t(lye,Hxe),jxe=(FF(),bre),p7t=new $t(hye,jxe)}function pCn(e,t,n){var s;s=null,t&&(s=t.d),b9(e,new Z6(t.n.a-s.b+n.a,t.n.b-s.d+n.b)),b9(e,new Z6(t.n.a-s.b+n.a,t.n.b+t.o.b+s.a+n.b)),b9(e,new Z6(t.n.a+t.o.a+s.c+n.a,t.n.b-s.d+n.b)),b9(e,new Z6(t.n.a+t.o.a+s.c+n.a,t.n.b+t.o.b+s.a+n.b))}function Bbe(e,t,n){var s,a,u;for(t.p=n,u=gg(Cf(se(ie(Nf,1),st,20,0,[new j6(t),new y4(t)])));rr(u);)s=h(Un(u),11),s.p==-1&&Bbe(e,s,n);if(t.i.k==(bn(),ds))for(a=new Q(t.i.j);a.a<a.c.c.length;)s=h(K(a),11),s!=t&&s.p==-1&&Bbe(e,s,n)}function X0t(e){var t,n,s,a,u;if(a=h(Gu(Hwn(Tst(e)),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),s=Ax,a.gc()>=2)for(n=a.Kc(),t=Je(n.Pb());n.Ob();)u=t,t=Je(n.Pb()),s=d.Math.min(s,(rn(t),t-(rn(u),u)));return s}function bCn(e,t){var n,s,a,u,p;s=new di,Ai(s,t,s.c.b,s.c);do for(n=(kn(s.b!=0),h(pl(s,s.a.a),86)),e.b[n.g]=1,u=pr(n.d,0);u.b!=u.d.c;)a=h(gr(u),188),p=a.c,e.b[p.g]==1?vr(e.a,a):e.b[p.g]==2?e.b[p.g]=1:Ai(s,p,s.c.b,s.c);while(s.b!=0)}function vCn(e,t){var n,s,a;if(Me(t)===Me(Vn(e)))return!0;if(!ue(t,15)||(s=h(t,15),a=e.gc(),a!=s.gc()))return!1;if(ue(s,54)){for(n=0;n<a;n++)if(!n1(e.Xb(n),s.Xb(n)))return!1;return!0}else return T8n(e.Kc(),s.Kc())}function Q0t(e,t){var n,s;if(e.c.length!=0){if(e.c.length==2)d5((Kt(0,e.c.length),h(e.c[0],10)),(Yu(),Z1)),d5((Kt(1,e.c.length),h(e.c[1],10)),Kg);else for(s=new Q(e);s.a<s.c.c.length;)n=h(K(s),10),d5(n,t);e.c=pe(yn,st,1,0,5,1)}}function wCn(e){var t,n;if(e.c.length!=2)throw ne(new ja("Order only allowed for two paths."));t=(Kt(0,e.c.length),h(e.c[0],17)),n=(Kt(1,e.c.length),h(e.c[1],17)),t.d.i!=n.c.i&&(e.c=pe(yn,st,1,0,5,1),e.c[e.c.length]=n,e.c[e.c.length]=t)}function mCn(e,t){var n,s,a,u,p,v;for(s=new Np,p=bB(new _u(e.g)),u=p.a.ec().Kc();u.Ob();){if(a=h(u.Pb(),10),!a){Sp(t,"There are no classes in a balanced layout.");break}v=e.j[a.p],n=h(mb(s,v),15),n||(n=new qe,Yp(s,v,n)),n.Fc(a)}return s}function yCn(e,t,n){var s,a,u,p,v,m,x;if(n)for(u=n.a.length,s=new Mp(u),v=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);v.Ob();)p=h(v.Pb(),19),m=wk(n,p.a),m&&(x=A4n(_d(m,kee),t),Pr(e.f,x,m),a=p1 in m.a,a&&WE(x,_d(m,p1)),qF(m,x),dbe(m,x))}function kCn(e,t){var n,s,a,u,p;for(Bn(t,"Port side processing",1),p=new Q(e.a);p.a<p.c.c.length;)a=h(K(p),10),a2t(a);for(s=new Q(e.b);s.a<s.c.c.length;)for(n=h(K(s),29),u=new Q(n.a);u.a<u.c.c.length;)a=h(K(u),10),a2t(a);An(t)}function Z0t(e,t,n){var s,a,u,p,v;if(a=e.f,!a&&(a=h(e.a.a.ec().Kc().Pb(),57)),g9(a,t,n),e.a.a.gc()!=1)for(s=t*n,p=e.a.a.ec().Kc();p.Ob();)u=h(p.Pb(),57),u!=a&&(v=W4(u),v.f.d?(u.d.d+=s+If,u.d.a-=s+If):v.f.a&&(u.d.a-=s+If))}function _Q(e,t,n,s,a){var u,p,v,m,x,_,A,I,P;return p=n-e,v=s-t,u=d.Math.atan2(p,v),m=u+cJ,x=u-cJ,_=a*d.Math.sin(m)+e,I=a*d.Math.cos(m)+t,A=a*d.Math.sin(x)+e,P=a*d.Math.cos(x)+t,Ef(se(ie(cs,1),Re,8,0,[new dt(_,I),new dt(A,P)]))}function xCn(e,t,n,s){var a,u,p,v,m,x,_,A;a=n,_=t,u=_;do u=e.a[u.p],v=(A=e.g[u.p],Pe(e.p[A.p])+Pe(e.d[u.p])-u.d.d),m=B4n(u,s),m&&(p=(x=e.g[m.p],Pe(e.p[x.p])+Pe(e.d[m.p])+m.o.b+m.d.a),a=d.Math.min(a,v-(p+uy(e.k,u,m))));while(_!=u);return a}function ECn(e,t,n,s){var a,u,p,v,m,x,_,A;a=n,_=t,u=_;do u=e.a[u.p],p=(A=e.g[u.p],Pe(e.p[A.p])+Pe(e.d[u.p])+u.o.b+u.d.a),m=Y5n(u,s),m&&(v=(x=e.g[m.p],Pe(e.p[x.p])+Pe(e.d[m.p])-m.d.d),a=d.Math.min(a,v-(p+uy(e.k,u,m))));while(_!=u);return a}function pt(e,t){var n,s;return s=(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),Xh(e.o,t)),s!=null?s:(n=t.wg(),ue(n,4)&&(n==null?(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),_F(e.o,t)):(!e.o&&(e.o=new Du((Qo(),ep),fv,e,0)),zF(e.o,t,n))),n)}function Iw(){Iw=re,Lh=new iy("H_LEFT",0),Vd=new iy("H_CENTER",1),Ih=new iy("H_RIGHT",2),Nh=new iy("V_TOP",3),J1=new iy("V_CENTER",4),Xl=new iy("V_BOTTOM",5),el=new iy("INSIDE",6),Oh=new iy("OUTSIDE",7),Kl=new iy("H_PRIORITY",8)}function _Cn(e){var t,n,s,a,u,p,v;if(t=e.Hh(Vl),t&&(v=Jn(Xh((!t.b&&(t.b=new Cu((Bt(),na),uo,t)),t.b),"settingDelegates")),v!=null)){for(n=new qe,a=Pw(v,"\\w+"),u=0,p=a.length;u<p;++u)s=a[u],n.c[n.c.length]=s;return n}return Rt(),Rt(),ca}function TCn(e,t){var n,s,a,u,p,v,m;if(!t.f)throw ne(new an("The input edge is not a tree edge."));for(u=null,a=Ir,s=new Q(e.d);s.a<s.c.c.length;)n=h(K(s),213),v=n.d,m=n.e,wQ(e,v,t)&&!wQ(e,m,t)&&(p=m.e-v.e-n.a,p<a&&(a=p,u=n));return u}function CCn(e){var t,n,s,a,u,p;if(!(e.f.e.c.length<=1)){t=0,a=$0t(e),n=ki;do{for(t>0&&(a=n),p=new Q(e.f.e);p.a<p.c.c.length;)u=h(K(p),144),!ft(lt(U(u,(sM(),$4e))))&&(s=PDn(e,u),zr(Uh(u.d),s));n=$0t(e)}while(!Xbn(e,t++,a,n))}}function SCn(e,t){var n,s,a;for(Bn(t,"Layer constraint preprocessing",1),n=new qe,a=new Ms(e.a,0);a.b<a.d.gc();)s=(kn(a.b<a.d.gc()),h(a.d.Xb(a.c=a.b++),10)),X5n(s)&&(QEn(s),n.c[n.c.length]=s,Mu(a));n.c.length==0||Fe(e,(He(),fne),n),An(t)}function ACn(e,t){var n,s,a,u,p;for(u=e.g.a,p=e.g.b,s=new Q(e.d);s.a<s.c.c.length;)n=h(K(s),70),a=n.n,e.a==(Sf(),Xx)||e.i==(Qe(),pn)?a.a=u:e.a==Qx||e.i==(Qe(),un)?a.a=u+e.j.a-n.o.a:a.a=u+(e.j.a-n.o.a)/2,a.b=p,zr(a,t),p+=n.o.b+e.e}function MCn(e,t,n){var s,a,u,p;for(Bn(n,"Processor set coordinates",1),e.a=t.b.b==0?1:t.b.b,u=null,s=pr(t.b,0);!u&&s.b!=s.d.c;)p=h(gr(s),86),ft(lt(U(p,(fo(),dm))))&&(u=p,a=p.e,a.a=h(U(p,ore),19).a,a.b=0);Edt(e,Pge(u),jo(n,1)),An(n)}function DCn(e,t,n){var s,a,u;for(Bn(n,"Processor determine the height for each level",1),e.a=t.b.b==0?1:t.b.b,a=null,s=pr(t.b,0);!a&&s.b!=s.d.c;)u=h(gr(s),86),ft(lt(U(u,(fo(),dm))))&&(a=u);a&&$gt(e,Ef(se(ie(PPn,1),Ewe,86,0,[a])),n),An(n)}function LCn(e,t){var n,s,a,u,p,v,m,x,_,A;x=e,m=HE(x,"individualSpacings"),m&&(s=Pp(t,(Tr(),Y5)),p=!s,p&&(a=new lq,ya(t,Y5,a)),v=h(pt(t,Y5),373),A=m,u=null,A&&(u=(_=HK(A,pe(it,Re,2,0,6,1)),new Uq(A,_))),u&&(n=new hJe(A,v),Os(u,n)))}function ICn(e,t){var n,s,a,u,p,v,m,x,_,A,I;return m=null,A=e,_=null,(kmt in A.a||xmt in A.a||hj in A.a)&&(x=null,I=Fge(t),p=HE(A,kmt),n=new YWe(I),ixn(n.a,p),v=HE(A,xmt),s=new rKe(I),rxn(s.a,v),u=wb(A,hj),a=new aKe(I),x=(PEn(a.a,u),u),_=x),m=_,m}function OCn(e,t){var n,s,a;if(t===e)return!0;if(ue(t,543)){if(a=h(t,835),e.a.d!=a.a.d||dy(e).gc()!=dy(a).gc())return!1;for(s=dy(a).Kc();s.Ob();)if(n=h(s.Pb(),416),_st(e,n.a.cd())!=h(n.a.dd(),14).gc())return!1;return!0}return!1}function NCn(e){var t,n,s,a;return s=h(e.a,19).a,a=h(e.b,19).a,t=s,n=a,s==0&&a==0?n-=1:s==-1&&a<=0?(t=0,n-=2):s<=0&&a>0?(t-=1,n-=1):s>=0&&a<0?(t+=1,n+=1):s>0&&a>=0?(t-=1,n+=1):(t+=1,n-=1),new As(Ke(t),Ke(n))}function PCn(e,t){return e.c<t.c?-1:e.c>t.c?1:e.b<t.b?-1:e.b>t.b?1:e.a!=t.a?ii(e.a)-ii(t.a):e.d==(GE(),$_)&&t.d==j_?-1:e.d==j_&&t.d==$_?1:0}function J0t(e,t){var n,s,a,u,p;return u=t.a,u.c.i==t.b?p=u.d:p=u.c,u.c.i==t.b?s=u.c:s=u.d,a=uxn(e.a,p,s),a>0&&a<Ax?(n=xCn(e.a,s.i,a,e.c),uut(e.a,s.i,-n),n>0):a<0&&-a<Ax?(n=ECn(e.a,s.i,-a,e.c),uut(e.a,s.i,n),n>0):!1}function BCn(e,t,n,s){var a,u,p,v,m,x,_,A;for(a=(t-e.d)/e.c.c.length,u=0,e.a+=n,e.d=t,A=new Q(e.c);A.a<A.c.c.length;)_=h(K(A),33),x=_.g,m=_.f,yc(_,_.i+u*a),kc(_,_.j+s*n),Eb(_,_.g+a),xb(_,e.a),++u,v=_.g,p=_.f,Ydt(_,new dt(v,p),new dt(x,m))}function FCn(e){var t,n,s,a,u,p,v;if(e==null)return null;for(v=e.length,a=(v+1)/2|0,p=pe(Vc,Zy,25,a,15,1),v%2!=0&&(p[--a]=rve((er(v-1,e.length),e.charCodeAt(v-1)))),n=0,s=0;n<a;++n)t=rve(Is(e,s++)),u=rve(Is(e,s++)),p[n]=(t<<4|u)<<24>>24;return p}function RCn(e){if(e.pe()){var t=e.c;t.qe()?e.o="["+t.n:t.pe()?e.o="["+t.ne():e.o="[L"+t.ne()+";",e.b=t.me()+"[]",e.k=t.oe()+"[]";return}var n=e.j,s=e.d;s=s.split("/"),e.o=SX(".",[n,SX("$",s)]),e.b=SX(".",[n,SX(".",s)]),e.k=s[s.length-1]}function jCn(e,t){var n,s,a,u,p;for(p=null,u=new Q(e.e.a);u.a<u.c.c.length;)if(a=h(K(u),121),a.b.a.c.length==a.g.a.c.length){for(s=a.e,p=x_n(a),n=a.e-h(p.a,19).a+1;n<a.e+h(p.b,19).a;n++)t[n]<t[s]&&(s=n);t[s]<t[a.e]&&(--t[a.e],++t[s],a.e=s)}}function TQ(e){var t,n,s,a,u,p,v,m;for(a=ki,s=Fi,n=new Q(e.e.b);n.a<n.c.c.length;)for(t=h(K(n),29),p=new Q(t.a);p.a<p.c.c.length;)u=h(K(p),10),m=Pe(e.p[u.p]),v=m+Pe(e.b[e.g[u.p].p]),a=d.Math.min(a,m),s=d.Math.max(s,v);return s-a}function Fbe(e,t,n,s){var a,u,p,v,m;for(a=lve(e,t),v=0,m=a.gc();v<m;++v)if(u=h(a.Xb(v),170),Nt(s,gk(Sa(e,u)))){if(p=iA(Sa(e,u)),n==null){if(p==null)return u}else if(Nt(n,p))return u}return null}function Rbe(e,t,n,s){var a,u,p,v,m;for(a=jQ(e,t),v=0,m=a.gc();v<m;++v)if(u=h(a.Xb(v),170),Nt(s,gk(Sa(e,u)))){if(p=iA(Sa(e,u)),n==null){if(p==null)return u}else if(Nt(n,p))return u}return null}function $Cn(e,t,n){var s,a,u,p,v,m;if(p=new UO,v=ac(e.e.Tg(),t),s=h(e.g,119),ia(),h(t,66).Oj())for(u=0;u<e.i;++u)a=s[u],v.rl(a.ak())&&qn(p,a);else for(u=0;u<e.i;++u)a=s[u],v.rl(a.ak())&&(m=a.dd(),qn(p,n?m9(e,t,u,p.i,m):m));return w0e(p)}function HCn(e,t){var n,s,a,u,p;for(n=new o9(e8),a=(u5(),se(ie(e8,1),ze,227,0,[Jx,m_,Zx,Jw,o3,a3])),u=0,p=a.length;u<p;++u)s=a[u],v1e(n,s,new qe);return Ti(pc(Jr(Xa(new Vt(null,new Wt(e.b,16)),new GFe),new VFe),new wYe(t)),new mYe(n)),n}function ZF(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(u=t.Kc();u.Ob();)a=h(u.Pb(),33),_=a.i+a.g/2,I=a.j+a.f/2,m=e.f,p=m.i+m.g/2,v=m.j+m.f/2,x=_-p,A=I-v,s=d.Math.sqrt(x*x+A*A),x*=e.e/s,A*=e.e/s,n?(_-=x,I-=A):(_+=x,I+=A),yc(a,_-a.g/2),kc(a,I-a.f/2)}function Ny(e){var t,n,s;if(!e.c&&e.b!=null){for(t=e.b.length-4;t>=0;t-=2)for(n=0;n<=t;n+=2)(e.b[n]>e.b[n+2]||e.b[n]===e.b[n+2]&&e.b[n+1]>e.b[n+3])&&(s=e.b[n+2],e.b[n+2]=e.b[n],e.b[n]=s,s=e.b[n+3],e.b[n+3]=e.b[n+1],e.b[n+1]=s);e.c=!0}}function egt(e,t){var n,s,a,u,p,v,m,x;for(p=t==1?Mte:Ate,u=p.a.ec().Kc();u.Ob();)for(a=h(u.Pb(),103),m=h(jr(e.f.c,a),21).Kc();m.Ob();)switch(v=h(m.Pb(),46),s=h(v.b,81),x=h(v.a,189),n=x.c,a.g){case 2:case 1:s.g.d+=n;break;case 4:case 3:s.g.c+=n}}function zCn(e,t){var n,s,a,u,p,v,m,x,_;for(x=-1,_=0,p=e,v=0,m=p.length;v<m;++v){for(u=p[v],n=new Ent(x==-1?e[0]:e[x],t,(t9(),nH)),s=0;s<u.length;s++)for(a=s+1;a<u.length;a++)os(u[s],(He(),Eo))&&os(u[a],Eo)&&kbt(n,u[s],u[a])>0&&++_;++x}return _}function _h(e){var t,n;return n=new ju(sg(e.gm)),n.a+="@",or(n,(t=ii(e)>>>0,t.toString(16))),e.kh()?(n.a+=" (eProxyURI: ",ho(n,e.qh()),e.$g()&&(n.a+=" eClass: ",ho(n,e.$g())),n.a+=")"):e.$g()&&(n.a+=" (eClass: ",ho(n,e.$g()),n.a+=")"),n.a}function k9(e){var t,n,s,a;if(e.e)throw ne(new ja((yd(cte),WZ+cte.k+KZ)));for(e.d==(la(),Q1)&&mR(e,Wl),n=new Q(e.a.a);n.a<n.c.c.length;)t=h(K(n),307),t.g=t.i;for(a=new Q(e.a.b);a.a<a.c.c.length;)s=h(K(a),57),s.i=Fi;return e.b.Le(e),e}function GCn(e,t){var n,s,a,u,p;if(t<2*e.b)throw ne(new an("The knot vector must have at least two time the dimension elements."));for(e.f=1,a=0;a<e.b;a++)Ue(e.e,0);for(p=t+1-2*e.b,n=p,u=1;u<p;u++)Ue(e.e,u/n);if(e.d)for(s=0;s<e.b;s++)Ue(e.e,1)}function tgt(e,t){var n,s,a,u,p,v,m,x,_;if(x=t,_=h(YB(CW(e.i),x),33),!_)throw a=_d(x,p1),v="Unable to find elk node for json object '"+a,m=v+"' Panic!",ne(new Xf(m));u=wb(x,"edges"),n=new rJe(e,_),TTn(n.a,n.b,u),p=wb(x,xee),s=new jWe(e),F7n(s.a,p)}function ngt(e,t,n,s){var a,u,p,v,m;if(s!=null){if(a=e.d[t],a){for(u=a.g,m=a.i,v=0;v<m;++v)if(p=h(u[v],133),p.Sh()==n&&Nr(s,p.cd()))return v}}else if(a=e.d[t],a){for(u=a.g,m=a.i,v=0;v<m;++v)if(p=h(u[v],133),Me(p.cd())===Me(s))return v}return-1}function lx(e,t){var n,s,a;return n=t==null?no(Ia(e.f,null)):Z7(e.g,t),ue(n,235)?(a=h(n,235),a.Qh()==null,a):ue(n,498)?(s=h(n,1938),a=s.a,a&&(a.yb==null||(t==null?sc(e.f,null,a):Ab(e.g,t,a))),a):null}function VCn(e){eve();var t,n,s,a,u,p,v;if(e==null||(a=e.length,a%2!=0))return null;for(t=LB(e),u=a/2|0,n=pe(Vc,Zy,25,u,15,1),s=0;s<u;s++){if(p=bT[t[s*2]],p==-1||(v=bT[t[s*2+1]],v==-1))return null;n[s]=(p<<4|v)<<24>>24}return n}function UCn(e,t,n){var s,a,u;if(a=h(wa(e.i,t),306),!a)if(a=new tut(e.d,t,n),j4(e.i,t,a),qpe(t))o0n(e.a,t.c,t.b,a);else switch(u=I_n(t),s=h(wa(e.p,u),244),u.g){case 1:case 3:a.j=!0,Oq(s,t.b,a);break;case 4:case 2:a.k=!0,Oq(s,t.c,a)}return a}function qCn(e,t,n,s){var a,u,p,v,m,x;if(v=new UO,m=ac(e.e.Tg(),t),a=h(e.g,119),ia(),h(t,66).Oj())for(p=0;p<e.i;++p)u=a[p],m.rl(u.ak())&&qn(v,u);else for(p=0;p<e.i;++p)u=a[p],m.rl(u.ak())&&(x=u.dd(),qn(v,s?m9(e,t,p,v.i,x):x));return Rpe(v,n)}function rgt(e,t){var n,s,a,u,p,v,m,x;if(a=e.b[t.p],a>=0)return a;for(u=1,v=new Q(t.j);v.a<v.c.c.length;)for(p=h(K(v),11),s=new Q(p.g);s.a<s.c.c.length;)n=h(K(s),17),x=n.d.i,t!=x&&(m=rgt(e,x),u=d.Math.max(u,m+1));return qkn(e,t,u),u}function igt(e,t,n){var s,a,u;for(s=1;s<e.c.length;s++){for(u=(Kt(s,e.c.length),h(e.c[s],10)),a=s;a>0&&t.ue((Kt(a-1,e.c.length),h(e.c[a-1],10)),u)>0;)dl(e,a,(Kt(a-1,e.c.length),h(e.c[a-1],10))),--a;Kt(a,e.c.length),e.c[a]=u}n.a=new zn,n.b=new zn}function YCn(e,t,n){var s,a,u,p,v,m,x,_;for(_=(s=h(t.e&&t.e(),9),new ll(s,h(vh(s,s.length),9),0)),m=Pw(n,"[\\[\\]\\s,]+"),u=m,p=0,v=u.length;p<v;++p)if(a=u[p],Mw(a).length!=0){if(x=v0t(e,a),x==null)return null;Eh(_,h(x,22))}return _}function WCn(e){var t,n,s;for(n=new Q(e.a.a.b);n.a<n.c.c.length;)t=h(K(n),81),s=(rn(0),0),s>0&&(!(lg(e.a.c)&&t.n.d)&&!(ek(e.a.c)&&t.n.b)&&(t.g.d-=d.Math.max(0,s/2-.5)),!(lg(e.a.c)&&t.n.a)&&!(ek(e.a.c)&&t.n.c)&&(t.g.a+=d.Math.max(0,s-1)))}function sgt(e,t,n){var s,a;if((e.c-e.b&e.a.length-1)==2)t==(Qe(),sn)||t==pn?(RB(h(e9(e),15),(Yu(),Z1)),RB(h(e9(e),15),Kg)):(RB(h(e9(e),15),(Yu(),Kg)),RB(h(e9(e),15),Z1));else for(a=new BE(e);a.a!=a.b;)s=h(fF(a),15),RB(s,n)}function KCn(e,t){var n,s,a,u,p,v,m;for(a=sk(new Ble(e)),v=new Ms(a,a.c.length),u=sk(new Ble(t)),m=new Ms(u,u.c.length),p=null;v.b>0&&m.b>0&&(n=(kn(v.b>0),h(v.a.Xb(v.c=--v.b),33)),s=(kn(m.b>0),h(m.a.Xb(m.c=--m.b),33)),n==s);)p=n;return p}function hu(e,t){var n,s,a,u,p,v;return u=e.a*VZ+e.b*1502,v=e.b*VZ+11,n=d.Math.floor(v*HM),u+=n,v-=n*rwe,u%=rwe,e.a=u,e.b=v,t<=24?d.Math.floor(e.a*V3e[t]):(a=e.a*(1<<t-24),p=d.Math.floor(e.b*U3e[t]),s=a+p,s>=2147483648&&(s-=$Z),s)}function agt(e,t,n){var s,a,u,p;Dst(e,t)>Dst(e,n)?(s=Za(n,(Qe(),pn)),e.d=s.dc()?0:rW(h(s.Xb(0),11)),p=Za(t,un),e.b=p.dc()?0:rW(h(p.Xb(0),11))):(a=Za(n,(Qe(),un)),e.d=a.dc()?0:rW(h(a.Xb(0),11)),u=Za(t,pn),e.b=u.dc()?0:rW(h(u.Xb(0),11)))}function ogt(e){var t,n,s,a,u,p,v;if(e&&(t=e.Hh(Vl),t&&(p=Jn(Xh((!t.b&&(t.b=new Cu((Bt(),na),uo,t)),t.b),"conversionDelegates")),p!=null))){for(v=new qe,s=Pw(p,"\\w+"),a=0,u=s.length;a<u;++a)n=s[a],v.c[v.c.length]=n;return v}return Rt(),Rt(),ca}function cgt(e,t){var n,s,a,u;for(n=e.o.a,u=h(h(jr(e.r,t),21),84).Kc();u.Ob();)a=h(u.Pb(),111),a.e.a=n*Pe(Je(a.b.We(Uj))),a.e.b=(s=a.b,s.Xe((Tr(),pu))?s.Hf()==(Qe(),sn)?-s.rf().b-Pe(Je(s.We(pu))):Pe(Je(s.We(pu))):s.Hf()==(Qe(),sn)?-s.rf().b:0)}function XCn(e){var t,n,s,a,u,p,v,m;t=!0,a=null,u=null;e:for(m=new Q(e.a);m.a<m.c.c.length;)for(v=h(K(m),10),s=new Sn(Dn(za(v).a.Kc(),new L));rr(s);){if(n=h(Un(s),17),!!a&&a!=v){t=!1;break e}if(a=v,p=n.c.i,!!u&&u!=p){t=!1;break e}u=p}return t}function QCn(e,t,n){var s,a,u,p,v,m;for(u=-1,v=-1,p=0;p<t.c.length&&(a=(Kt(p,t.c.length),h(t.c[p],329)),!(a.c>e.c));p++)a.a>=e.s&&(u<0&&(u=p),v=p);return m=(e.s+e.c)/2,u>=0&&(s=IMn(e,t,u,v),m=H1n((Kt(s,t.c.length),h(t.c[s],329))),zTn(t,s,n)),m}function CQ(){CQ=re,R7t=new sa((Tr(),U5),1.3),i8e=i7e,h8e=new nb(15),U7t=new sa(y2,h8e),Y7t=new sa(k2,15),j7t=DH,z7t=m2,G7t=_3,V7t=Wg,H7t=E3,c8e=tL,q7t=gm,l8e=(Ibe(),P7t),o8e=O7t,u8e=N7t,f8e=B7t,s8e=I7t,a8e=LH,$7t=a7e,VD=L7t,r8e=D7t,d8e=F7t}function Yn(e,t,n){var s,a,u,p,v,m,x;for(p=(u=new Jue,u),mge(p,(rn(t),t)),x=(!p.b&&(p.b=new Cu((Bt(),na),uo,p)),p.b),m=1;m<n.length;m+=2)zF(x,n[m-1],n[m]);for(s=(!e.Ab&&(e.Ab=new Ye(dr,e,0,3)),e.Ab),v=0;v<0;++v)a=tvn(h(he(s,s.i-1),590)),s=a;qn(s,p)}function ugt(e,t,n){var s,a,u;for(G0n.call(this,new qe),this.a=t,this.b=n,this.e=e,s=(e.b&&WQ(e),e.a),this.d=pit(s.a,this.a),this.c=pit(s.b,this.b),Ckn(this,this.d,this.c),wTn(this),u=this.e.e.a.ec().Kc();u.Ob();)a=h(u.Pb(),266),a.c.c.length>0&&TIn(this,a)}function jbe(e,t,n,s,a,u){var p,v,m;if(!a[t.b]){for(a[t.b]=!0,p=s,!p&&(p=new mB),Ue(p.e,t),m=u[t.b].Kc();m.Ob();)v=h(m.Pb(),282),!(v.d==n||v.c==n)&&(v.c!=t&&jbe(e,v.c,t,p,a,u),v.d!=t&&jbe(e,v.d,t,p,a,u),Ue(p.c,v),Vi(p.d,v.b));return p}return null}function ZCn(e){var t,n,s,a,u,p,v;for(t=0,a=new Q(e.e);a.a<a.c.c.length;)s=h(K(a),17),n=yk(new Vt(null,new Wt(s.b,16)),new oFe),n&&++t;for(p=new Q(e.g);p.a<p.c.c.length;)u=h(K(p),17),v=yk(new Vt(null,new Wt(u.b,16)),new cFe),v&&++t;return t>=2}function JCn(e,t){var n,s,a,u;for(Bn(t,"Self-Loop pre-processing",1),s=new Q(e.a);s.a<s.c.c.length;)n=h(K(s),10),Pkn(n)&&(a=(u=new Tft(n),Fe(n,(He(),s8),u),vDn(u),u),Ti(pc(Xa(new Vt(null,new Wt(a.d,16)),new LRe),new IRe),new ORe),fAn(a));An(t)}function eSn(e,t,n,s,a){var u,p,v,m,x,_;for(u=e.c.d.j,p=h(Zh(n,0),8),_=1;_<n.b;_++)x=h(Zh(n,_),8),Ai(s,p,s.c.b,s.c),v=Jf(zr(new _a(p),x),.5),m=Jf(new WW(Xge(u)),a),zr(v,m),Ai(s,v,s.c.b,s.c),p=x,u=t==0?pF(u):mpe(u);vr(s,(kn(n.b!=0),h(n.c.b.c,8)))}function tSn(e){Iw();var t,n,s;return n=ni(el,se(ie(ga,1),ze,93,0,[Oh])),!(NA(iB(n,e))>1||(t=ni(Lh,se(ie(ga,1),ze,93,0,[Vd,Ih])),NA(iB(t,e))>1)||(s=ni(Nh,se(ie(ga,1),ze,93,0,[J1,Xl])),NA(iB(s,e))>1))}function nSn(e,t){var n,s,a;return n=t.Hh(e.a),n&&(a=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),"affiliation")),a!=null)?(s=aP(a,Ec(35)),s==-1?dX(e,AE(e,Vu(t.Hj())),a):s==0?dX(e,null,a.substr(1)):dX(e,a.substr(0,s),a.substr(s+1))):null}function rSn(e){var t,n,s;try{return e==null?_c:Ga(e)}catch(a){if(a=ui(a),ue(a,102))return t=a,s=sg(uu(e))+"@"+(n=(L1(),Zpe(e)>>>0),n.toString(16)),Uxn(V5n(),(Q6(),"Exception during lenientFormat for "+s),t),"<"+s+" threw "+sg(t.gm)+">";throw ne(a)}}function lgt(e){switch(e.g){case 0:return new pUe;case 1:return new lUe;case 2:return new lZe;case 3:return new f$e;case 4:return new Ott;case 5:return new bUe;default:throw ne(new an("No implementation is available for the layerer "+(e.f!=null?e.f:""+e.g)))}}function $be(e,t,n){var s,a,u;for(u=new Q(e.t);u.a<u.c.c.length;)s=h(K(u),268),s.b.s<0&&s.c>0&&(s.b.n-=s.c,s.b.n<=0&&s.b.u>0&&vr(t,s.b));for(a=new Q(e.i);a.a<a.c.c.length;)s=h(K(a),268),s.a.s<0&&s.c>0&&(s.a.u-=s.c,s.a.u<=0&&s.a.n>0&&vr(n,s.a))}function JF(e){var t,n,s,a,u;if(e.g==null&&(e.d=e.si(e.f),qn(e,e.d),e.c))return u=e.f,u;if(t=h(e.g[e.i-1],47),a=t.Pb(),e.e=t,n=e.si(a),n.Ob())e.d=n,qn(e,n);else for(e.d=null;!t.Ob()&&(pi(e.g,--e.i,null),e.i!=0);)s=h(e.g[e.i-1],47),t=s;return a}function iSn(e,t){var n,s,a,u,p,v;if(s=t,a=s.ak(),Bd(e.e,a)){if(a.hi()&&pB(e,a,s.dd()))return!1}else for(v=ac(e.e.Tg(),a),n=h(e.g,119),u=0;u<e.i;++u)if(p=n[u],v.rl(p.ak()))return Nr(p,s)?!1:(h(Sy(e,u,t),72),!0);return qn(e,t)}function sSn(e,t,n,s){var a,u,p,v;for(a=new Nd(e),kp(a,(bn(),Ou)),Fe(a,(He(),Br),t),Fe(a,DD,s),Fe(a,(tt(),Ei),(_s(),qo)),Fe(a,rf,t.c),Fe(a,Yl,t.d),Bgt(t,a),v=d.Math.floor(n/2),p=new Q(a.j);p.a<p.c.c.length;)u=h(K(p),11),u.n.b=v;return a}function aSn(e,t){var n,s,a,u,p,v,m,x,_;for(m=I1(e.c-e.b&e.a.length-1),x=null,_=null,u=new BE(e);u.a!=u.b;)a=h(fF(u),10),n=(v=h(U(a,(He(),rf)),11),v?v.i:null),s=(p=h(U(a,Yl),11),p?p.i:null),(x!=n||_!=s)&&(Q0t(m,t),x=n,_=s),m.c[m.c.length]=a;Q0t(m,t)}function hgt(e){var t,n,s,a,u,p,v;for(t=0,s=new Q(e.a);s.a<s.c.c.length;)for(n=h(K(s),10),u=new Sn(Dn(qi(n).a.Kc(),new L));rr(u);)a=h(Un(u),17),e==a.d.i.c&&a.c.j==(Qe(),un)&&(p=kf(a.c).b,v=kf(a.d).b,t=d.Math.max(t,d.Math.abs(v-p)));return t}function oSn(e,t,n){var s,a,u;Bn(n,"Remove overlaps",1),n.n&&!!t&&mh(n,yh(t),(Lu(),rl)),s=h(pt(t,(xE(),q_)),33),e.f=s,e.a=BX(h(pt(t,(Cw(),GD)),293)),a=Je(pt(t,(Tr(),k2))),ple(e,(rn(a),a)),u=Nw(s),tbt(e,t,u,n),n.n&&!!t&&mh(n,yh(t),(Lu(),rl))}function Hbe(e,t,n){switch(n.g){case 1:return new dt(t.a,d.Math.min(e.d.b,t.b));case 2:return new dt(d.Math.max(e.c.a,t.a),t.b);case 3:return new dt(t.a,d.Math.max(e.c.b,t.b));case 4:return new dt(d.Math.min(t.a,e.d.a),t.b)}return new dt(t.a,t.b)}function cSn(e,t,n,s){var a,u,p,v,m,x,_,A,I;for(A=s?(Qe(),un):(Qe(),pn),a=!1,m=t[n],x=0,_=m.length;x<_;++x)v=m[x],!F4(h(U(v,(tt(),Ei)),98))&&(p=v.e,I=!Za(v,A).dc()&&!!p,I&&(u=R2e(p),e.b=new j2e(u,s?0:u.length-1)),a=a|aDn(e,v,A,I));return a}function wM(e){var t,n,s;for(t=I1(1+(!e.c&&(e.c=new Ye(bu,e,9,9)),e.c).i),Ue(t,(!e.d&&(e.d=new qt(us,e,8,5)),e.d)),s=new Tn((!e.c&&(e.c=new Ye(bu,e,9,9)),e.c));s.e!=s.i.gc();)n=h(On(s),118),Ue(t,(!n.d&&(n.d=new qt(us,n,8,5)),n.d));return Vn(t),new pE(t)}function Pd(e){var t,n,s;for(t=I1(1+(!e.c&&(e.c=new Ye(bu,e,9,9)),e.c).i),Ue(t,(!e.e&&(e.e=new qt(us,e,7,4)),e.e)),s=new Tn((!e.c&&(e.c=new Ye(bu,e,9,9)),e.c));s.e!=s.i.gc();)n=h(On(s),118),Ue(t,(!n.e&&(n.e=new qt(us,n,7,4)),n.e));return Vn(t),new pE(t)}function uSn(e){var t,n,s,a;if(e==null)return null;if(s=Ho(e,!0),a=cD.length,Nt(s.substr(s.length-a,a),cD)){if(n=s.length,n==4){if(t=(er(0,s.length),s.charCodeAt(0)),t==43)return FEe;if(t==45)return w_t}else if(n==3)return FEe}return Dw(s)}function lSn(e){var t,n,s,a;for(t=0,n=0,a=new Q(e.j);a.a<a.c.c.length;)if(s=h(K(a),11),t=Gn(Vs(t,_ot(Jr(new Vt(null,new Wt(s.e,16)),new O$e)))),n=Gn(Vs(n,_ot(Jr(new Vt(null,new Wt(s.g,16)),new N$e)))),t>1||n>1)return 2;return t+n==1?2:0}function fgt(e,t,n){var s,a,u,p,v;for(Bn(n,"ELK Force",1),ft(lt(pt(t,(Jh(),O4e))))||nB((s=new hS((Qv(),new Yv(t))),s)),v=zlt(t),M7n(v),c6n(e,h(U(v,I4e),424)),p=t2t(e.a,v),u=p.Kc();u.Ob();)a=h(u.Pb(),231),VMn(e.b,a,jo(n,1/p.gc()));v=gbt(p),hbt(v),An(n)}function hSn(e,t){var n,s,a,u,p;if(Bn(t,"Breaking Point Processor",1),LOn(e),ft(lt(U(e,(tt(),$ke))))){for(a=new Q(e.b);a.a<a.c.c.length;)for(s=h(K(a),29),n=0,p=new Q(s.a);p.a<p.c.c.length;)u=h(K(p),10),u.p=n++;OLn(e),Pgt(e,!0),Pgt(e,!1)}An(t)}function fSn(e,t,n){var s,a,u,p,v,m;for(v=e.c,p=(n.q?n.q:(Rt(),Rt(),K1)).vc().Kc();p.Ob();)u=h(p.Pb(),42),s=!U7(Jr(new Vt(null,new Wt(v,16)),new I7(new WZe(t,u)))).sd((Tp(),Ux)),s&&(m=u.dd(),ue(m,4)&&(a=Upe(m),a!=null&&(m=a)),t.Ye(h(u.cd(),146),m))}function eR(e,t){var n,s,a,u,p;if(t){for(u=ue(e.Cb,88)||ue(e.Cb,99),p=!u&&ue(e.Cb,322),s=new Tn((!t.a&&(t.a=new SE(t,pa,t)),t.a));s.e!=s.i.gc();)if(n=h(On(s),87),a=hR(n),u?ue(a,88):p?ue(a,148):!!a)return a;return u?(Bt(),nh):(Bt(),zf)}else return null}function dSn(e,t){var n,s,a,u,p,v;for(Bn(t,"Constraints Postprocessor",1),p=0,u=new Q(e.b);u.a<u.c.c.length;){for(a=h(K(u),29),v=0,s=new Q(a.a);s.a<s.c.c.length;)n=h(K(s),10),n.k==(bn(),Wi)&&(Fe(n,(tt(),Y$),Ke(p)),Fe(n,G$,Ke(v)),++v);++p}An(t)}function gSn(e,t,n,s){var a,u,p,v,m,x,_;for(m=new dt(n,s),ms(m,h(U(t,(fo(),G_)),8)),_=pr(t.b,0);_.b!=_.d.c;)x=h(gr(_),86),zr(x.e,m),vr(e.b,x);for(v=pr(t.a,0);v.b!=v.d.c;){for(p=h(gr(v),188),u=pr(p.a,0);u.b!=u.d.c;)a=h(gr(u),8),zr(a,m);vr(e.a,p)}}function zbe(e,t,n){var s,a,u;if(u=Hy((jc(),Ps),e.Tg(),t),u){if(ia(),!h(u,66).Oj()&&(u=Y4(Sa(Ps,u)),!u))throw ne(new an(Fg+t.ne()+J9));a=(s=e.Yg(u),h(s>=0?e._g(s,!0,!0):Bb(e,u,!0),153)),h(a,215).ml(t,n)}else throw ne(new an(Fg+t.ne()+J9))}function pSn(e,t){var n,s,a,u,p;for(n=new qe,a=Xa(new Vt(null,new Wt(e,16)),new sHe),u=Xa(new Vt(null,new Wt(e,16)),new aHe),p=z3n(r3n(qP(WSn(se(ie(xPn,1),st,833,0,[a,u])),new oHe))),s=1;s<p.length;s++)p[s]-p[s-1]>=2*t&&Ue(n,new ZY(p[s-1]+t,p[s]-t));return n}function bSn(e,t,n){Bn(n,"Eades radial",1),n.n&&!!t&&mh(n,yh(t),(Lu(),rl)),e.d=h(pt(t,(xE(),q_)),33),e.c=Pe(Je(pt(t,(Cw(),yH)))),e.e=BX(h(pt(t,GD),293)),e.a=Kkn(h(pt(t,Vxe),426)),e.b=vEn(h(pt(t,Gxe),340)),Q7n(e),n.n&&!!t&&mh(n,yh(t),(Lu(),rl))}function vSn(e,t,n){var s,a,u,p,v,m,x,_;if(n)for(u=n.a.length,s=new Mp(u),v=(s.b-s.a)*s.c<0?(cg(),sp):new fg(s);v.Ob();)p=h(v.Pb(),19),a=wk(n,p.a),a&&(m=pyn(e,(x=(X2(),_=new rhe,_),t&&Gbe(x,t),x),a),WE(m,_d(a,p1)),qF(a,m),dbe(a,m),XK(e,a,m))}function tR(e){var t,n,s,a,u,p;if(!e.j){if(p=new Vze,t=fT,u=t.a.zc(e,t),u==null){for(s=new Tn(La(e));s.e!=s.i.gc();)n=h(On(s),26),a=tR(n),mi(p,a),qn(p,n);t.a.Bc(e)!=null}kw(p),e.j=new ay((h(he(De((bg(),Xt).o),11),18),p.i),p.g),ou(e).b&=-33}return e.j}function wSn(e){var t,n,s,a;if(e==null)return null;if(s=Ho(e,!0),a=cD.length,Nt(s.substr(s.length-a,a),cD)){if(n=s.length,n==4){if(t=(er(0,s.length),s.charCodeAt(0)),t==43)return REe;if(t==45)return m_t}else if(n==3)return REe}return new qle(s)}function mSn(e){var t,n,s;return n=e.l,(n&n-1)!=0||(s=e.m,(s&s-1)!=0)||(t=e.h,(t&t-1)!=0)||t==0&&s==0&&n==0?-1:t==0&&s==0&&n!=0?fge(n):t==0&&s!=0&&n==0?fge(s)+22:t!=0&&s==0&&n==0?fge(t)+44:-1}function ySn(e,t){var n,s,a,u,p;for(Bn(t,"Edge joining",1),n=ft(lt(U(e,(tt(),Nne)))),a=new Q(e.b);a.a<a.c.c.length;)for(s=h(K(a),29),p=new Ms(s.a,0);p.b<p.d.gc();)u=(kn(p.b<p.d.gc()),h(p.d.Xb(p.c=p.b++),10)),u.k==(bn(),ds)&&(iZ(u,n),Mu(p));An(t)}function kSn(e,t,n){var s,a;if(hB(e.b),P1(e.b,(XA(),kH),(K7(),UD)),P1(e.b,xH,t.g),P1(e.b,EH,t.a),e.a=vR(e.b,t),Bn(n,"Compaction by shrinking a tree",e.a.c.length),t.i.c.length>1)for(a=new Q(e.a);a.a<a.c.c.length;)s=h(K(a),51),s.pf(t,jo(n,1));An(n)}function Py(e,t){var n,s,a,u,p;for(a=t.a&e.f,u=null,s=e.b[a];;s=s.b){if(s==t){u?u.b=t.b:e.b[a]=t.b;break}u=s}for(p=t.f&e.f,u=null,n=e.c[p];;n=n.d){if(n==t){u?u.d=t.d:e.c[p]=t.d;break}u=n}t.e?t.e.c=t.c:e.a=t.c,t.c?t.c.e=t.e:e.e=t.e,--e.i,++e.g}function xSn(e){var t,n,s,a,u,p,v,m,x,_;for(n=e.o,t=e.p,p=Ir,a=$s,v=Ir,u=$s,x=0;x<n;++x)for(_=0;_<t;++_)Ay(e,x,_)&&(p=d.Math.min(p,x),a=d.Math.max(a,x),v=d.Math.min(v,_),u=d.Math.max(u,_));return m=a-p+1,s=u-v+1,new git(Ke(p),Ke(v),Ke(m),Ke(s))}function SQ(e,t){var n,s,a,u;for(u=new Ms(e,0),n=(kn(u.b<u.d.gc()),h(u.d.Xb(u.c=u.b++),140));u.b<u.d.gc();)s=(kn(u.b<u.d.gc()),h(u.d.Xb(u.c=u.b++),140)),a=new Z1e(s.c,n.d,t),kn(u.b>0),u.a.Xb(u.c=--u.b),iw(u,a),kn(u.b<u.d.gc()),u.d.Xb(u.c=u.b++),a.a=!1,n=s}function dgt(e){var t,n,s,a,u,p;for(a=h(U(e,(He(),N$)),11),p=new Q(e.j);p.a<p.c.c.length;){for(u=h(K(p),11),s=new Q(u.g);s.a<s.c.c.length;)return t=h(K(s),17),xs(t,a),u;for(n=new Q(u.e);n.a<n.c.c.length;)return t=h(K(n),17),Gs(t,a),u}return null}function ESn(e,t,n){var s,a;s=xc(n.q.getTime()),mo(s,0)<0?(a=_0-Gn(ZA(Tk(s),_0)),a==_0&&(a=0)):a=Gn(ZA(s,_0)),t==1?(a=d.Math.min((a+50)/100|0,9),pg(e,48+a&Oi)):t==2?(a=d.Math.min((a+5)/10|0,99),j1(e,a,2)):(j1(e,a,3),t>3&&j1(e,0,t-3))}function _Sn(e){var t,n,s,a;return Me(U(e,(tt(),sm)))===Me((Ld(),B0))?!e.e&&Me(U(e,ID))!==Me((jk(),TD)):(s=h(U(e,xne),292),a=ft(lt(U(e,Ene)))||Me(U(e,M_))===Me((e5(),_D)),t=h(U(e,rke),19).a,n=e.a.c.length,!a&&s!=(jk(),TD)&&(t==0||t>n))}function TSn(e){var t,n;for(n=0;n<e.c.length&&!(Fnt((Kt(n,e.c.length),h(e.c[n],113)))>0);n++);if(n>0&&n<e.c.length-1)return n;for(t=0;t<e.c.length&&!(Fnt((Kt(t,e.c.length),h(e.c[t],113)))>0);t++);return t>0&&n<e.c.length-1?t:e.c.length/2|0}function ggt(e,t){var n,s;if(t!=e.Cb||e.Db>>16!=6&&!!t){if(nx(e,t))throw ne(new an(e_+I0t(e)));s=null,e.Cb&&(s=(n=e.Db>>16,n>=0?p2e(e,s):e.Cb.ih(e,-1-n,null,s))),t&&(s=a5(t,e,6,s)),s=d1e(e,t,s),s&&s.Fi()}else(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,6,t,t))}function Gbe(e,t){var n,s;if(t!=e.Cb||e.Db>>16!=9&&!!t){if(nx(e,t))throw ne(new an(e_+gpt(e)));s=null,e.Cb&&(s=(n=e.Db>>16,n>=0?v2e(e,s):e.Cb.ih(e,-1-n,null,s))),t&&(s=a5(t,e,9,s)),s=g1e(e,t,s),s&&s.Fi()}else(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,9,t,t))}function AQ(e,t){var n,s;if(t!=e.Cb||e.Db>>16!=3&&!!t){if(nx(e,t))throw ne(new an(e_+d2t(e)));s=null,e.Cb&&(s=(n=e.Db>>16,n>=0?m2e(e,s):e.Cb.ih(e,-1-n,null,s))),t&&(s=a5(t,e,12,s)),s=f1e(e,t,s),s&&s.Fi()}else(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,3,t,t))}function hx(e){var t,n,s,a,u;if(s=Rl(e),u=e.j,u==null&&!!s)return e.$j()?null:s.zj();if(ue(s,148)){if(n=s.Aj(),n&&(a=n.Nh(),a!=e.i)){if(t=h(s,148),t.Ej())try{e.g=a.Kh(t,u)}catch(p){if(p=ui(p),ue(p,78))e.g=null;else throw ne(p)}e.i=a}return e.g}return null}function pgt(e){var t;return t=new qe,Ue(t,new M4(new dt(e.c,e.d),new dt(e.c+e.b,e.d))),Ue(t,new M4(new dt(e.c,e.d),new dt(e.c,e.d+e.a))),Ue(t,new M4(new dt(e.c+e.b,e.d+e.a),new dt(e.c+e.b,e.d))),Ue(t,new M4(new dt(e.c+e.b,e.d+e.a),new dt(e.c,e.d+e.a))),t}function bgt(e,t,n,s){var a,u,p;if(p=S2e(t,n),s.c[s.c.length]=t,e.j[p.p]==-1||e.j[p.p]==2||e.a[t.p])return s;for(e.j[p.p]=-1,u=new Sn(Dn(Id(p).a.Kc(),new L));rr(u);)if(a=h(Un(u),17),!(!(!Xs(a)&&!(!Xs(a)&&a.c.i.c==a.d.i.c))||a==t))return bgt(e,a,p,s);return s}function CSn(e,t,n){var s,a,u;for(u=t.a.ec().Kc();u.Ob();)a=h(u.Pb(),79),s=h(_n(e.b,a),266),!s&&(bi(H1(a))==bi(Sg(a))?HAn(e,a,n):H1(a)==bi(Sg(a))?_n(e.c,a)==null&&_n(e.b,Sg(a))!=null&&K2t(e,a,n,!1):_n(e.d,a)==null&&_n(e.b,H1(a))!=null&&K2t(e,a,n,!0))}function SSn(e,t){var n,s,a,u,p,v,m;for(a=e.Kc();a.Ob();)for(s=h(a.Pb(),10),v=new So,Ka(v,s),Ji(v,(Qe(),pn)),Fe(v,(He(),R$),(on(),!0)),p=t.Kc();p.Ob();)u=h(p.Pb(),10),m=new So,Ka(m,u),Ji(m,un),Fe(m,R$,!0),n=new db,Fe(n,R$,!0),Gs(n,v),xs(n,m)}function ASn(e,t,n,s){var a,u,p,v;a=hft(e,t,n),u=hft(e,n,t),p=h(_n(e.c,t),112),v=h(_n(e.c,n),112),a<u?new Ip((qh(),fm),p,v,u-a):u<a?new Ip((qh(),fm),v,p,a-u):(a!=0||!(!t.i||!n.i)&&s[t.i.c][n.i.c])&&(new Ip((qh(),fm),p,v,0),new Ip(fm,v,p,0))}function vgt(e,t){var n,s,a,u,p,v,m;for(a=0,p=new Q(t.a);p.a<p.c.c.length;)for(u=h(K(p),10),a+=u.o.b+u.d.a+u.d.d+e.e,s=new Sn(Dn(za(u).a.Kc(),new L));rr(s);)n=h(Un(s),17),n.c.i.k==(bn(),Vo)&&(m=n.c.i,v=h(U(m,(He(),Br)),10),a+=v.o.b+v.d.a+v.d.d);return a}function wgt(e,t,n){var s,a,u,p,v,m,x;for(u=new qe,x=new di,p=new di,vLn(e,x,p,t),SOn(e,x,p,t,n),m=new Q(e);m.a<m.c.c.length;)for(v=h(K(m),112),a=new Q(v.k);a.a<a.c.c.length;)s=h(K(a),129),(!t||s.c==(qh(),qg))&&v.g>s.b.g&&(u.c[u.c.length]=s);return u}function fx(){fx=re,y3=new IS("CANDIDATE_POSITION_LAST_PLACED_RIGHT",0),G5=new IS("CANDIDATE_POSITION_LAST_PLACED_BELOW",1),W_=new IS("CANDIDATE_POSITION_WHOLE_DRAWING_RIGHT",2),Y_=new IS("CANDIDATE_POSITION_WHOLE_DRAWING_BELOW",3),K_=new IS("WHOLE_DRAWING",4)}function MSn(e,t){if(ue(t,239))return skn(e,h(t,33));if(ue(t,186))return wkn(e,h(t,118));if(ue(t,354))return Lvn(e,h(t,137));if(ue(t,352))return tLn(e,h(t,79));if(t)return null;throw ne(new an(zye+Ag(new _u(se(ie(yn,1),st,1,5,[t])))))}function DSn(e){var t,n,s,a,u,p,v;for(u=new di,a=new Q(e.d.a);a.a<a.c.c.length;)s=h(K(a),121),s.b.a.c.length==0&&Ai(u,s,u.c.b,u.c);if(u.b>1)for(t=ib((n=new _p,++e.b,n),e.d),v=pr(u,0);v.b!=v.d.c;)p=h(gr(v),121),Th(ph(gh(bh(dh(new Il,1),0),t),p))}function Vbe(e,t){var n,s;if(t!=e.Cb||e.Db>>16!=11&&!!t){if(nx(e,t))throw ne(new an(e_+cve(e)));s=null,e.Cb&&(s=(n=e.Db>>16,n>=0?y2e(e,s):e.Cb.ih(e,-1-n,null,s))),t&&(s=a5(t,e,10,s)),s=x1e(e,t,s),s&&s.Fi()}else(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,11,t,t))}function LSn(e){var t,n,s,a;for(s=new Gp(new xp(e.b).a);s.b;)n=kb(s),a=h(n.cd(),11),t=h(n.dd(),10),Fe(t,(He(),Br),a),Fe(a,Zc,t),Fe(a,AD,(on(),!0)),Ji(a,h(U(t,co),61)),U(t,co),Fe(a.i,(tt(),Ei),(_s(),k8)),h(U(Us(a.i),Uo),21).Fc((ha(),i8))}function ISn(e,t,n){var s,a,u,p,v,m;if(u=0,p=0,e.c)for(m=new Q(e.d.i.j);m.a<m.c.c.length;)v=h(K(m),11),u+=v.e.c.length;else u=1;if(e.d)for(m=new Q(e.c.i.j);m.a<m.c.c.length;)v=h(K(m),11),p+=v.g.c.length;else p=1;return a=Li(eA(p-u)),s=(n+t)/2+(n-t)*(.4*a),s}function OSn(e){Iy();var t,n;if(e.Hc((Qe(),to)))throw ne(new an("Port sides must not contain UNDEFINED"));switch(e.gc()){case 1:return x$;case 2:return t=e.Hc(pn)&&e.Hc(un),n=e.Hc(sn)&&e.Hc(Fn),t||n?T$:_$;case 3:return E$;case 4:return k$;default:return null}}function NSn(e,t,n){var s,a,u,p,v;for(Bn(n,"Breaking Point Removing",1),e.a=h(U(t,(tt(),Hd)),218),u=new Q(t.b);u.a<u.c.c.length;)for(a=h(K(u),29),v=new Q(bb(a.a));v.a<v.c.c.length;)p=h(K(v),10),Yct(p)&&(s=h(U(p,(He(),tv)),305),!s.d&&pbt(e,s));An(n)}function MQ(e,t,n){return Ry(),Pk(e,t)&&Pk(e,n)?!1:aZ(new dt(e.c,e.d),new dt(e.c+e.b,e.d),t,n)||aZ(new dt(e.c+e.b,e.d),new dt(e.c+e.b,e.d+e.a),t,n)||aZ(new dt(e.c+e.b,e.d+e.a),new dt(e.c,e.d+e.a),t,n)||aZ(new dt(e.c,e.d+e.a),new dt(e.c,e.d),t,n)}function Ube(e,t){var n,s,a,u;if(!e.dc()){for(n=0,s=e.gc();n<s;++n)if(u=Jn(e.Xb(n)),u==null?t==null:Nt(u.substr(0,3),"!##")?t!=null&&(a=t.length,!Nt(u.substr(u.length-a,a),t)||u.length!=t.length+3)&&!Nt(o2,t):Nt(u,Bee)&&!Nt(o2,t)||Nt(u,t))return!0}return!1}function PSn(e,t,n,s){var a,u,p,v,m,x;for(p=e.j.c.length,m=pe(TPn,dwe,306,p,0,1),v=0;v<p;v++)u=h(ut(e.j,v),11),u.p=v,m[v]=HTn(G0t(u),n,s);for(sAn(e,m,n,t,s),x=new zn,a=0;a<m.length;a++)m[a]&&Pr(x,h(ut(e.j,a),11),m[a]);x.f.c+x.g.c!=0&&(Fe(e,(He(),T_),x),u_n(e,m))}function BSn(e,t,n){var s,a,u;for(a=new Q(e.a.b);a.a<a.c.c.length;)if(s=h(K(a),57),u=by(s),u&&u.k==(bn(),Pi))switch(h(U(u,(He(),co)),61).g){case 4:u.n.a=t.a;break;case 2:u.n.a=n.a-(u.o.a+u.d.c);break;case 1:u.n.b=t.b;break;case 3:u.n.b=n.b-(u.o.b+u.d.a)}}function By(){By=re,iH=new D4(jd,0),BD=new D4("NIKOLOV",1),FD=new D4("NIKOLOV_PIXEL",2),Qke=new D4("NIKOLOV_IMPROVED",3),Zke=new D4("NIKOLOV_IMPROVED_PIXEL",4),Xke=new D4("DUMMYNODE_PERCENTAGE",5),Jke=new D4("NODECOUNT_PERCENTAGE",6),sH=new D4("NO_BOUNDARY",7)}function FSn(e,t,n){var s,a,u,p,v;return a=h(pt(t,(WF(),W8e)),19),!a&&(a=Ke(0)),u=h(pt(n,W8e),19),!u&&(u=Ke(0)),a.a>u.a?-1:a.a<u.a?1:e.a&&(s=Ui(t.j,n.j),s!=0||(s=Ui(t.i,n.i),s!=0))?s:(p=t.g*t.f,v=n.g*n.f,Ui(p,v))}function RSn(e,t){var n,s,a,u,p,v,m,x,_,A;if(++e.e,m=e.d==null?0:e.d.length,t>m){for(_=e.d,e.d=pe(uEe,Qye,63,2*m+4,0,1),u=0;u<m;++u)if(x=_[u],x)for(s=x.g,A=x.i,v=0;v<A;++v)a=h(s[v],133),p=p1e(e,a.Sh()),n=e.d[p],!n&&(n=e.d[p]=e.uj()),n.Fc(a);return!0}else return!1}function jSn(e,t,n){var s,a,u,p,v,m;if(a=n,u=a.ak(),Bd(e.e,u)){if(u.hi()){for(s=h(e.g,119),p=0;p<e.i;++p)if(v=s[p],Nr(v,a)&&p!=t)throw ne(new an(nD))}}else for(m=ac(e.e.Tg(),u),s=h(e.g,119),p=0;p<e.i;++p)if(v=s[p],m.rl(v.ak()))throw ne(new an(oD));s9(e,t,n)}function mgt(e,t){var n,s,a,u,p,v;for(n=h(U(t,(He(),em)),21),p=h(jr((uZ(),Wr),n),21),v=h(jr(fr,n),21),u=p.Kc();u.Ob();)if(s=h(u.Pb(),21),!h(jr(e.b,s),15).dc())return!1;for(a=v.Kc();a.Ob();)if(s=h(a.Pb(),21),!h(jr(e.b,s),15).dc())return!1;return!0}function $Sn(e,t){var n,s,a,u,p,v;for(Bn(t,"Partition postprocessing",1),s=new Q(e.b);s.a<s.c.c.length;)for(n=h(K(s),29),u=new Q(n.a);u.a<u.c.c.length;)for(a=h(K(u),10),v=new Q(a.j);v.a<v.c.c.length;)p=h(K(v),11),ft(lt(U(p,(He(),R$))))&&IE(v);An(t)}function ygt(e,t){var n,s,a,u,p,v,m,x,_;if(e.a.c.length==1)return _dt(h(ut(e.a,0),187),t);for(p=G6n(e),m=0,x=e.d,u=p,_=e.d,v=(x-u)/2+u;u+1<x;){for(m=0,s=new Q(e.a);s.a<s.c.c.length;)n=h(K(s),187),m+=(a=M9(n,v,!1),a.a);m<t?(_=v,x=v):u=v,v=(x-u)/2+u}return _}function HSn(e){var t,n,s,a,u;return isNaN(e)?(Ck(),y3e):e<-9223372036854776e3?(Ck(),f3t):e>=9223372036854776e3?(Ck(),w3e):(a=!1,e<0&&(a=!0,e=-e),s=0,e>=Jp&&(s=Li(e/Jp),e-=s*Jp),n=0,e>=v5&&(n=Li(e/v5),e-=n*v5),t=Li(e),u=rc(t,n,s),a&&KK(u),u)}function zSn(e,t){var n,s,a,u;for(n=!t||!e.u.Hc((Qc(),Ud)),u=0,a=new Q(e.e.Cf());a.a<a.c.c.length;){if(s=h(K(a),838),s.Hf()==(Qe(),to))throw ne(new an("Label and node size calculator can only be used with ports that have port sides assigned."));s.vf(u++),$6n(e,s,n)}}function GSn(e,t){var n,s,a,u,p;return a=t.Hh(e.a),a&&(s=(!a.b&&(a.b=new Cu((Bt(),na),uo,a)),a.b),n=Jn(Xh(s,Hs)),n!=null&&(u=n.lastIndexOf("#"),p=u==-1?s1e(e,t.Aj(),n):u==0?kA(e,null,n.substr(1)):kA(e,n.substr(0,u),n.substr(u+1)),ue(p,148)))?h(p,148):null}function VSn(e,t){var n,s,a,u,p;return s=t.Hh(e.a),s&&(n=(!s.b&&(s.b=new Cu((Bt(),na),uo,s)),s.b),u=Jn(Xh(n,Pee)),u!=null&&(a=u.lastIndexOf("#"),p=a==-1?s1e(e,t.Aj(),u):a==0?kA(e,null,u.substr(1)):kA(e,u.substr(0,a),u.substr(a+1)),ue(p,148)))?h(p,148):null}function qbe(e){var t,n,s,a,u;for(n=new Q(e.a.a);n.a<n.c.c.length;){for(t=h(K(n),307),t.j=null,u=t.a.a.ec().Kc();u.Ob();)s=h(u.Pb(),57),Uh(s.b),(!t.j||s.d.c<t.j.d.c)&&(t.j=s);for(a=t.a.a.ec().Kc();a.Ob();)s=h(a.Pb(),57),s.b.a=s.d.c-t.j.d.c,s.b.b=s.d.d-t.j.d.d}return e}function nR(e){var t,n,s,a,u;for(n=new Q(e.a.a);n.a<n.c.c.length;){for(t=h(K(n),189),t.f=null,u=t.a.a.ec().Kc();u.Ob();)s=h(u.Pb(),81),Uh(s.e),(!t.f||s.g.c<t.f.g.c)&&(t.f=s);for(a=t.a.a.ec().Kc();a.Ob();)s=h(a.Pb(),81),s.e.a=s.g.c-t.f.g.c,s.e.b=s.g.d-t.f.g.d}return e}function USn(e){var t,n,s;return n=h(e.a,19).a,s=h(e.b,19).a,t=d.Math.max(d.Math.abs(n),d.Math.abs(s)),n<t&&s==-t?new As(Ke(n+1),Ke(s)):n==t&&s<t?new As(Ke(n),Ke(s+1)):n>=-t&&s==t?new As(Ke(n-1),Ke(s)):new As(Ke(n),Ke(s-1))}function kgt(){return oa(),se(ie(DPn,1),ze,77,0,[f5e,u5e,v_,Pte,L5e,o$,v$,Kx,M5e,y5e,S5e,Wx,D5e,v5e,I5e,r5e,h$,Bte,s$,g$,N5e,d$,i5e,A5e,P5e,p$,O5e,a$,g5e,T5e,_5e,w$,o5e,i$,u$,a5e,Yx,x5e,w5e,C5e,w_,l5e,c5e,E5e,m5e,l$,b$,s5e,f$,k5e,c$,p5e,d5e,xD,r$,b5e,h5e])}function qSn(e,t,n){e.d=0,e.b=0,t.k==(bn(),Vo)&&n.k==Vo&&h(U(t,(He(),Br)),10)==h(U(n,Br),10)&&(xK(t).j==(Qe(),sn)?agt(e,t,n):agt(e,n,t)),t.k==Vo&&n.k==ds?xK(t).j==(Qe(),sn)?e.d=1:e.b=1:n.k==Vo&&t.k==ds&&(xK(n).j==(Qe(),sn)?e.b=1:e.d=1),O8n(e,t,n)}function YSn(e){var t,n,s,a,u,p,v,m,x,_,A;return A=G2e(e),t=e.a,m=t!=null,m&&ok(A,"category",e.a),a=wS(new P6(e.d)),p=!a,p&&(x=new yp,Wh(A,"knownOptions",x),n=new oKe(x),Os(new P6(e.d),n)),u=wS(e.g),v=!u,v&&(_=new yp,Wh(A,"supportedFeatures",_),s=new cKe(_),Os(e.g,s)),A}function WSn(e){var t,n,s,a,u,p,v,m,x;for(s=!1,t=336,n=0,u=new Ett(e.length),v=e,m=0,x=v.length;m<x;++m)p=v[m],s=s|(Up(p),!1),a=(vg(p),p.a),Ue(u.a,Vn(a)),t&=a.qd(),n=E6n(n,a.rd());return h(h(Mrt(new Vt(null,cQ(new Wt((lw(),Ppe(u.a)),16),new z,t,n)),new vqe(e)),670),833)}function KSn(e,t){var n;!!e.d&&(t.c!=e.e.c||s5n(e.e.b,t.b))&&(Ue(e.f,e.d),e.a=e.d.c+e.d.b,e.d=null,e.e=null),gdn(t.b)?e.c=t:e.b=t,(t.b==($k(),Zb)&&!t.a||t.b==Qw&&t.a||t.b==r3&&t.a||t.b==Zw&&!t.a)&&!!e.c&&!!e.b&&(n=new hl(e.a,e.c.d,t.c-e.a,e.b.d-e.c.d),e.d=n,e.e=t)}function mM(e){var t;if(mXe.call(this),this.i=new GHe,this.g=e,this.f=h(e.e&&e.e(),9).length,this.f==0)throw ne(new an("There must be at least one phase in the phase enumeration."));this.c=(t=h(Vh(this.g),9),new ll(t,h(vh(t,t.length),9),0)),this.a=new rs,this.b=new zn}function Ybe(e,t){var n,s;if(t!=e.Cb||e.Db>>16!=7&&!!t){if(nx(e,t))throw ne(new an(e_+Adt(e)));s=null,e.Cb&&(s=(n=e.Db>>16,n>=0?b2e(e,s):e.Cb.ih(e,-1-n,null,s))),t&&(s=h(t,49).gh(e,1,hL,s)),s=gde(e,t,s),s&&s.Fi()}else(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,7,t,t))}function xgt(e,t){var n,s;if(t!=e.Cb||e.Db>>16!=3&&!!t){if(nx(e,t))throw ne(new an(e_+Fht(e)));s=null,e.Cb&&(s=(n=e.Db>>16,n>=0?w2e(e,s):e.Cb.ih(e,-1-n,null,s))),t&&(s=h(t,49).gh(e,0,dL,s)),s=pde(e,t,s),s&&s.Fi()}else(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,3,t,t))}function DQ(e,t){cx();var n,s,a,u,p,v,m,x,_;return t.d>e.d&&(v=e,e=t,t=v),t.d<63?RAn(e,t):(p=(e.d&-2)<<4,x=A0e(e,p),_=A0e(t,p),s=qQ(e,K4(x,p)),a=qQ(t,K4(_,p)),m=DQ(x,_),n=DQ(s,a),u=DQ(qQ(x,s),qQ(a,_)),u=eZ(eZ(u,m),n),u=K4(u,p),m=K4(m,p<<1),eZ(eZ(m,u),n))}function XSn(e,t,n){var s,a,u,p,v;for(p=r9(e,n),v=pe(X1,T0,10,t.length,0,1),s=0,u=p.Kc();u.Ob();)a=h(u.Pb(),11),ft(lt(U(a,(He(),AD))))&&(v[s++]=h(U(a,Zc),10));if(s<t.length)throw ne(new ja("Expected "+t.length+" hierarchical ports, but found only "+s+"."));return v}function QSn(e,t){var n,s,a,u,p,v;if(!e.tb){for(u=(!e.rb&&(e.rb=new cw(e,af,e)),e.rb),v=new S4(u.i),a=new Tn(u);a.e!=a.i.gc();)s=h(On(a),138),p=s.ne(),n=h(p==null?sc(v.f,null,s):Ab(v.g,p,s),138),n&&(p==null?sc(v.f,null,n):Ab(v.g,p,n));e.tb=v}return h(Fo(e.tb,t),138)}function yM(e,t){var n,s,a,u,p;if((e.i==null&&s1(e),e.i).length,!e.p){for(p=new S4((3*e.g.i/2|0)+1),a=new P4(e.g);a.e!=a.i.gc();)s=h(NX(a),170),u=s.ne(),n=h(u==null?sc(p.f,null,s):Ab(p.g,u,s),170),n&&(u==null?sc(p.f,null,n):Ab(p.g,u,n));e.p=p}return h(Fo(e.p,t),170)}function Wbe(e,t,n,s,a){var u,p,v,m,x;for(Nxn(s+$W(n,n.$d()),a),Urt(t,axn(n)),u=n.f,u&&Wbe(e,t,u,"Caused by: ",!1),v=(n.k==null&&(n.k=pe(Zee,Re,78,0,0,1)),n.k),m=0,x=v.length;m<x;++m)p=v[m],Wbe(e,t,p,"Suppressed: ",!1);console.groupEnd!=null&&console.groupEnd.call(console)}function kM(e,t,n,s){var a,u,p,v,m;for(m=t.e,v=m.length,p=t.q._f(m,n?0:v-1,n),a=m[n?0:v-1],p=p|opt(e,a,n,s),u=n?1:v-2;n?u<v:u>=0;u+=n?1:-1)p=p|t.c.Sf(m,u,n,s&&!ft(lt(U(t.j,(He(),nv))))&&!ft(lt(U(t.j,(He(),f3))))),p=p|t.q._f(m,u,n),p=p|opt(e,m[u],n,s);return Qi(e.c,t),p}function rR(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(_=aat(e.j),A=0,I=_.length;A<I;++A){if(x=_[A],n==(ua(),Jc)||n==B_)for(m=i1(x.g),a=m,u=0,p=a.length;u<p;++u)s=a[u],c_n(t,s)&&jb(s,!0);if(n==ec||n==B_)for(v=i1(x.e),a=v,u=0,p=a.length;u<p;++u)s=a[u],o_n(t,s)&&jb(s,!0)}}function ZSn(e){var t,n;switch(t=null,n=null,kEn(e).g){case 1:t=(Qe(),pn),n=un;break;case 2:t=(Qe(),Fn),n=sn;break;case 3:t=(Qe(),un),n=pn;break;case 4:t=(Qe(),sn),n=Fn}dle(e,h(sb(BP(h(jr(e.k,t),15).Oc(),s3)),113)),fle(e,h(sb(PP(h(jr(e.k,n),15).Oc(),s3)),113))}function JSn(e){var t,n,s,a,u,p;if(a=h(ut(e.j,0),11),a.e.c.length+a.g.c.length==0)e.n.a=0;else{for(p=0,s=gg(Cf(se(ie(Nf,1),st,20,0,[new j6(a),new y4(a)])));rr(s);)n=h(Un(s),11),p+=n.i.n.a+n.n.a+n.a.a;t=h(U(e,(tt(),ov)),8),u=t?t.a:0,e.n.a=p/(a.e.c.length+a.g.c.length)-u}}function Egt(e,t){var n,s,a;for(s=new Q(t.a);s.a<s.c.c.length;)n=h(K(s),221),uW(h(n.b,65),ms(ro(h(t.b,65).c),h(t.b,65).a)),a=$pt(h(t.b,65).b,h(n.b,65).b),a>1&&(e.a=!0),fbn(h(n.b,65),zr(ro(h(t.b,65).c),Jf(ms(ro(h(n.b,65).a),h(t.b,65).a),a))),yst(e,t),Egt(e,n)}function _gt(e){var t,n,s,a,u,p,v;for(u=new Q(e.a.a);u.a<u.c.c.length;)s=h(K(u),189),s.e=0,s.d.a.$b();for(a=new Q(e.a.a);a.a<a.c.c.length;)for(s=h(K(a),189),n=s.a.a.ec().Kc();n.Ob();)for(t=h(n.Pb(),81),v=t.f.Kc();v.Ob();)p=h(v.Pb(),81),p.d!=s&&(Qi(s.d,p),++p.d.e)}function eAn(e){var t,n,s,a,u,p,v,m;for(m=e.j.c.length,n=0,t=m,a=2*m,v=new Q(e.j);v.a<v.c.c.length;)switch(p=h(K(v),11),p.j.g){case 2:case 4:p.p=-1;break;case 1:case 3:s=p.e.c.length,u=p.g.c.length,s>0&&u>0?p.p=t++:s>0?p.p=n++:u>0?p.p=a++:p.p=n++}Rt(),hs(e.j,new rRe)}function tAn(e){var t,n;n=null,t=h(ut(e.g,0),17);do{if(n=t.d.i,os(n,(He(),Yl)))return h(U(n,Yl),11).i;if(n.k!=(bn(),Wi)&&rr(new Sn(Dn(qi(n).a.Kc(),new L))))t=h(Un(new Sn(Dn(qi(n).a.Kc(),new L))),17);else if(n.k!=Wi)return null}while(!!n&&n.k!=(bn(),Wi));return n}function nAn(e,t){var n,s,a,u,p,v,m,x,_;for(v=t.j,p=t.g,m=h(ut(v,v.c.length-1),113),_=(Kt(0,v.c.length),h(v.c[0],113)),x=KX(e,p,m,_),u=1;u<v.c.length;u++)n=(Kt(u-1,v.c.length),h(v.c[u-1],113)),a=(Kt(u,v.c.length),h(v.c[u],113)),s=KX(e,p,n,a),s>x&&(m=n,_=a,x=s);t.a=_,t.c=m}function rAn(e,t){var n,s;if(s=tA(e.b,t.b),!s)throw ne(new ja("Invalid hitboxes for scanline constraint calculation."));(Llt(t.b,h(rgn(e.b,t.b),57))||Llt(t.b,h(ngn(e.b,t.b),57)))&&(L1(),t.b+""),e.a[t.b.f]=h(qY(e.b,t.b),57),n=h(UY(e.b,t.b),57),n&&(e.a[n.f]=t.b)}function Th(e){if(!e.a.d||!e.a.e)throw ne(new ja((yd($3t),$3t.k+" must have a source and target "+(yd(m4e),m4e.k)+" specified.")));if(e.a.d==e.a.e)throw ne(new ja("Network simplex does not support self-loops: "+e.a+" "+e.a.d+" "+e.a.e));return cP(e.a.d.g,e.a),cP(e.a.e.b,e.a),e.a}function iAn(e,t,n){var s,a,u,p,v,m,x;for(x=new ag(new aWe(e)),p=se(ie(S5t,1),Fvt,11,0,[t,n]),v=0,m=p.length;v<m;++v)for(u=p[v],cF(x.a,u,(on(),u2))==null,a=new _f(u.b);Wa(a.a)||Wa(a.b);)s=h(Wa(a.a)?K(a.a):K(a.b),17),s.c==s.d||tA(x,u==s.c?s.d:s.c);return Vn(x),new Bc(x)}function Tgt(e,t,n){var s,a,u,p,v,m;if(s=0,t.b!=0&&n.b!=0){u=pr(t,0),p=pr(n,0),v=Pe(Je(gr(u))),m=Pe(Je(gr(p))),a=!0;do{if(v>m-e.b&&v<m+e.b)return-1;v>m-e.a&&v<m+e.a&&++s,v<=m&&u.b!=u.d.c?v=Pe(Je(gr(u))):m<=v&&p.b!=p.d.c?m=Pe(Je(gr(p))):a=!1}while(a)}return s}function sAn(e,t,n,s,a){var u,p,v,m;for(m=(u=h(Vh(ta),9),new ll(u,h(vh(u,u.length),9),0)),v=new Q(e.j);v.a<v.c.c.length;)p=h(K(v),11),t[p.p]&&(wNn(p,t[p.p],s),Eh(m,p.j));a?(iQ(e,t,(Qe(),pn),2*n,s),iQ(e,t,un,2*n,s)):(iQ(e,t,(Qe(),sn),2*n,s),iQ(e,t,Fn,2*n,s))}function aAn(e){var t,n,s,a,u;if(u=new qe,mc(e.b,new zqe(u)),e.b.c=pe(yn,st,1,0,5,1),u.c.length!=0){for(t=(Kt(0,u.c.length),h(u.c[0],78)),n=1,s=u.c.length;n<s;++n)a=(Kt(n,u.c.length),h(u.c[n],78)),a!=t&&w9n(t,a);if(ue(t,60))throw ne(h(t,60));if(ue(t,289))throw ne(h(t,289))}}function oAn(e,t){var n,s,a,u;for(e=e==null?_c:(rn(e),e),n=new Kv,u=0,s=0;s<t.length&&(a=e.indexOf("%s",u),a!=-1);)or(n,e.substr(u,a-u)),ho(n,t[s++]),u=a+2;if(or(n,e.substr(u)),s<t.length){for(n.a+=" [",ho(n,t[s++]);s<t.length;)n.a+=Js,ho(n,t[s++]);n.a+="]"}return n.a}function cAn(e){var t,n,s,a;for(t=0,s=e.length,a=s-4,n=0;n<a;)t=(er(n+3,e.length),e.charCodeAt(n+3)+(er(n+2,e.length),31*(e.charCodeAt(n+2)+(er(n+1,e.length),31*(e.charCodeAt(n+1)+(er(n,e.length),31*(e.charCodeAt(n)+31*t))))))),t=t|0,n+=4;for(;n<s;)t=t*31+Is(e,n++);return t=t|0,t}function uAn(e){var t,n;for(n=new Sn(Dn(qi(e).a.Kc(),new L));rr(n);)if(t=h(Un(n),17),t.d.i.k!=(bn(),Ou))throw ne(new ey(bJ+iM(e)+"' has its layer constraint set to LAST, but has at least one outgoing edge that  does not go to a LAST_SEPARATE node. That must not happen."))}function lAn(e,t,n,s){var a,u,p,v,m,x,_,A,I;for(m=0,_=new Q(e.a);_.a<_.c.c.length;){for(x=h(K(_),10),v=0,u=new Sn(Dn(za(x).a.Kc(),new L));rr(u);)a=h(Un(u),17),A=kf(a.c).b,I=kf(a.d).b,v=d.Math.max(v,d.Math.abs(I-A));m=d.Math.max(m,v)}return p=s*d.Math.min(1,t/n)*m,p}function Kbe(e){var t;return t=new G7,(e&256)!=0&&(t.a+="F"),(e&128)!=0&&(t.a+="H"),(e&512)!=0&&(t.a+="X"),(e&2)!=0&&(t.a+="i"),(e&8)!=0&&(t.a+="m"),(e&4)!=0&&(t.a+="s"),(e&32)!=0&&(t.a+="u"),(e&64)!=0&&(t.a+="w"),(e&16)!=0&&(t.a+="x"),(e&Ch)!=0&&(t.a+=","),fhe(t.a)}function hAn(e,t){var n,s,a,u;for(Bn(t,"Resize child graph to fit parent.",1),s=new Q(e.b);s.a<s.c.c.length;)n=h(K(s),29),Vi(e.a,n.a),n.a.c=pe(yn,st,1,0,5,1);for(u=new Q(e.a);u.a<u.c.c.length;)a=h(K(u),10),Ca(a,null);e.b.c=pe(yn,st,1,0,5,1),NAn(e),e.e&&yDn(e.e,e),An(t)}function fAn(e){var t,n,s,a,u,p,v,m,x;if(s=e.b,u=s.e,p=F4(h(U(s,(tt(),Ei)),98)),n=!!u&&h(U(u,(He(),Uo)),21).Hc((ha(),El)),!(p||n))for(x=(v=new vd(e.e).a.vc().Kc(),new qv(v));x.a.Ob();)m=(t=h(x.a.Pb(),42),h(t.dd(),113)),m.a&&(a=m.d,Ka(a,null),m.c=!0,e.a=!0)}function dAn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P;for(I=-1,P=0,x=e,_=0,A=x.length;_<A;++_){for(m=x[_],u=m,p=0,v=u.length;p<v;++p)for(a=u[p],t=new RZe(I==-1?e[0]:e[I],C1t(a)),n=0;n<a.j.c.length;n++)for(s=n+1;s<a.j.c.length;s++)wrt(t,h(ut(a.j,n),11),h(ut(a.j,s),11))>0&&++P;++I}return P}function gAn(e,t){var n,s,a,u,p;for(p=h(U(t,(Fb(),Cxe)),425),u=pr(t.b,0);u.b!=u.d.c;)if(a=h(gr(u),86),e.b[a.g]==0){switch(p.g){case 0:Zft(e,a);break;case 1:bCn(e,a)}e.b[a.g]=2}for(s=pr(e.a,0);s.b!=s.d.c;)n=h(gr(s),188),Ew(n.b.d,n,!0),Ew(n.c.b,n,!0);Fe(t,(fo(),mxe),e.a)}function ac(e,t){ia();var n,s,a,u;return t?t==(qr(),v_t)||(t==s_t||t==_2||t==i_t)&&e!=BEe?new $ve(e,t):(s=h(t,677),n=s.pk(),n||(gk(Sa((jc(),Ps),t)),n=s.pk()),u=(!n.i&&(n.i=new zn),n.i),a=h(no(Ia(u.f,e)),1942),!a&&Pr(u,e,a=new $ve(e,t)),a):t_t}function pAn(e,t){var n,s,a,u,p,v,m,x,_;for(m=h(U(e,(He(),Br)),11),x=Qa(se(ie(cs,1),Re,8,0,[m.i.n,m.n,m.a])).a,_=e.i.n.b,n=i1(e.e),a=n,u=0,p=a.length;u<p;++u)s=a[u],xs(s,m),Cp(s.a,new dt(x,_)),t&&(v=h(U(s,(tt(),Da)),74),v||(v=new Oc,Fe(s,Da,v)),vr(v,new dt(x,_)))}function bAn(e,t){var n,s,a,u,p,v,m,x,_;for(a=h(U(e,(He(),Br)),11),x=Qa(se(ie(cs,1),Re,8,0,[a.i.n,a.n,a.a])).a,_=e.i.n.b,n=i1(e.g),p=n,v=0,m=p.length;v<m;++v)u=p[v],Gs(u,a),DJe(u.a,new dt(x,_)),t&&(s=h(U(u,(tt(),Da)),74),s||(s=new Oc,Fe(u,Da,s)),vr(s,new dt(x,_)))}function vAn(e,t){var n,s,a,u,p,v;for(e.b=new qe,e.d=h(U(t,(He(),R5)),230),e.e=Smn(e.d),u=new di,a=Ef(se(ie(_5t,1),Pvt,37,0,[t])),p=0;p<a.c.length;)s=(Kt(p,a.c.length),h(a.c[p],37)),s.p=p++,n=new R2t(s,e.a,e.b),Vi(a,n.b),Ue(e.b,n),n.s&&(v=pr(u,0),nA(v,n));return e.c=new ns,u}function wAn(e,t){var n,s,a,u,p,v;for(p=h(h(jr(e.r,t),21),84).Kc();p.Ob();)u=h(p.Pb(),111),n=u.c?nde(u.c):0,n>0?u.a?(v=u.b.rf().a,n>v&&(a=(n-v)/2,u.d.b=a,u.d.c=a)):u.d.c=e.s+n:DE(e.u)&&(s=V2e(u.b),s.c<0&&(u.d.b=-s.c),s.c+s.b>u.b.rf().a&&(u.d.c=s.c+s.b-u.b.rf().a))}function mAn(e,t){var n,s,a,u;for(Bn(t,"Semi-Interactive Crossing Minimization Processor",1),n=!1,a=new Q(e.b);a.a<a.c.c.length;)s=h(K(a),29),u=WA(FB(Jr(Jr(new Vt(null,new Wt(s.a,16)),new jRe),new $Re),new HRe),new zRe),n=n|u.a!=null;n&&Fe(e,(He(),w6e),(on(),!0)),An(t)}function yAn(e,t,n){var s,a,u,p,v;if(a=n,!a&&(a=new H6),Bn(a,"Layout",e.a.c.length),ft(lt(U(t,(Fb(),Exe)))))for(L1(),s=0;s<e.a.c.length;s++)v=(s<10?"0":"")+s++,""+v+sg(uu(h(ut(e.a,s),51)));for(p=new Q(e.a);p.a<p.c.c.length;)u=h(K(p),51),u.pf(t,jo(a,1));An(a)}function kAn(e){var t,n;if(t=h(e.a,19).a,n=h(e.b,19).a,t>=0){if(t==n)return new As(Ke(-t-1),Ke(-t-1));if(t==-n)return new As(Ke(-t),Ke(n+1))}return d.Math.abs(t)>d.Math.abs(n)?t<0?new As(Ke(-t),Ke(n)):new As(Ke(-t),Ke(n+1)):new As(Ke(t+1),Ke(n))}function xAn(e){var t,n;n=h(U(e,(tt(),cc)),163),t=h(U(e,(He(),h2)),303),n==(wl(),Gg)?(Fe(e,cc,LD),Fe(e,h2,(Ad(),l3))):n==nm?(Fe(e,cc,LD),Fe(e,h2,(Ad(),O5))):t==(Ad(),l3)?(Fe(e,cc,Gg),Fe(e,h2,CD)):t==O5&&(Fe(e,cc,nm),Fe(e,h2,CD))}function iR(){iR=re,$D=new V$e,C8t=Lr(new rs,(Zs(),oc),(oa(),s$)),M8t=Wc(Lr(new rs,oc,d$),Pa,f$),D8t=Ib(Ib(CN(Wc(Lr(new rs,w1,v$),Pa,b$),Go),p$),w$),S8t=Wc(Lr(Lr(Lr(new rs,Hg,o$),Go,u$),Go,Yx),Pa,c$),A8t=Wc(Lr(Lr(new rs,Go,Yx),Go,i$),Pa,r$)}function x9(){x9=re,O8t=Lr(Wc(new rs,(Zs(),Pa),(oa(),p5e)),oc,s$),F8t=Ib(Ib(CN(Wc(Lr(new rs,w1,v$),Pa,b$),Go),p$),w$),N8t=Wc(Lr(Lr(Lr(new rs,Hg,o$),Go,u$),Go,Yx),Pa,c$),B8t=Lr(Lr(new rs,oc,d$),Pa,f$),P8t=Wc(Lr(Lr(new rs,Go,Yx),Go,i$),Pa,r$)}function EAn(e,t,n,s,a){var u,p;(!Xs(t)&&t.c.i.c==t.d.i.c||!jut(Qa(se(ie(cs,1),Re,8,0,[a.i.n,a.n,a.a])),n))&&!Xs(t)&&(t.c==a?rk(t.a,0,new _a(n)):vr(t.a,new _a(n)),s&&!wd(e.a,n)&&(p=h(U(t,(tt(),Da)),74),p||(p=new Oc,Fe(t,Da,p)),u=new _a(n),Ai(p,u,p.c.b,p.c),Qi(e.a,u)))}function _An(e){var t,n;for(n=new Sn(Dn(za(e).a.Kc(),new L));rr(n);)if(t=h(Un(n),17),t.c.i.k!=(bn(),Ou))throw ne(new ey(bJ+iM(e)+"' has its layer constraint set to FIRST, but has at least one incoming edge that  does not come from a FIRST_SEPARATE node. That must not happen."))}function TAn(e,t,n){var s,a,u,p,v,m,x;if(a=Ght(e.Db&254),a==0)e.Eb=n;else{if(a==1)v=pe(yn,st,1,2,5,1),u=WX(e,t),u==0?(v[0]=n,v[1]=e.Eb):(v[0]=e.Eb,v[1]=n);else for(v=pe(yn,st,1,a+1,5,1),p=Fp(e.Eb),s=2,m=0,x=0;s<=128;s<<=1)s==t?v[x++]=n:(e.Db&s)!=0&&(v[x++]=p[m++]);e.Eb=v}e.Db|=t}function Cgt(e,t,n){var s,a,u,p;for(this.b=new qe,a=0,s=0,p=new Q(e);p.a<p.c.c.length;)u=h(K(p),167),n&&_Ln(u),Ue(this.b,u),a+=u.o,s+=u.p;this.b.c.length>0&&(u=h(ut(this.b,0),167),a+=u.o,s+=u.p),a*=2,s*=2,t>1?a=Li(d.Math.ceil(a*t)):s=Li(d.Math.ceil(s/t)),this.a=new Fpe(a,s)}function Sgt(e,t,n,s,a,u){var p,v,m,x,_,A,I,P,j,V,J,oe;for(_=s,t.j&&t.o?(P=h(_n(e.f,t.A),57),V=P.d.c+P.d.b,--_):V=t.a.c+t.a.b,A=a,n.q&&n.o?(P=h(_n(e.f,n.C),57),x=P.d.c,++A):x=n.a.c,J=x-V,m=d.Math.max(2,A-_),v=J/m,j=V+v,I=_;I<A;++I)p=h(u.Xb(I),128),oe=p.a.b,p.a.c=j-oe/2,j+=v}function Xbe(e,t,n,s,a,u){var p,v,m,x,_,A;for(x=n.c.length,u&&(e.c=pe(Hn,hr,25,t.length,15,1)),p=a?0:t.length-1;a?p<t.length:p>=0;p+=a?1:-1){for(v=t[p],m=s==(Qe(),pn)?a?Za(v,s):Rp(Za(v,s)):a?Rp(Za(v,s)):Za(v,s),u&&(e.c[v.p]=m.gc()),A=m.Kc();A.Ob();)_=h(A.Pb(),11),e.d[_.p]=x++;Vi(n,m)}}function Agt(e,t,n){var s,a,u,p,v,m,x,_;for(u=Pe(Je(e.b.Kc().Pb())),x=Pe(Je(N5n(t.b))),s=Jf(ro(e.a),x-n),a=Jf(ro(t.a),n-u),_=zr(s,a),Jf(_,1/(x-u)),this.a=_,this.b=new qe,v=!0,p=e.b.Kc(),p.Pb();p.Ob();)m=Pe(Je(p.Pb())),v&&m-n>XJ&&(this.b.Fc(n),v=!1),this.b.Fc(m);v&&this.b.Fc(n)}function CAn(e){var t,n,s,a;if(FMn(e,e.n),e.d.c.length>0){for(H7(e.c);Mbe(e,h(K(new Q(e.e.a)),121))<e.e.a.c.length;){for(t=v7n(e),a=t.e.e-t.d.e-t.a,t.e.j&&(a=-a),s=new Q(e.e.a);s.a<s.c.c.length;)n=h(K(s),121),n.j&&(n.e+=a);H7(e.c)}H7(e.c),wbe(e,h(K(new Q(e.e.a)),121)),P2t(e)}}function SAn(e,t){var n,s,a,u,p;for(a=h(jr(e.a,(Iy(),x$)),15).Kc();a.Ob();)switch(s=h(a.Pb(),101),n=h(ut(s.j,0),113).d.j,u=new Bc(s.j),hs(u,new $je),t.g){case 1:rQ(e,u,n,(Tb(),zg),1);break;case 0:p=TSn(u),rQ(e,new F1(u,0,p),n,(Tb(),zg),0),rQ(e,new F1(u,p,u.c.length),n,zg,1)}}function AAn(e,t){n5();var n,s;if(n=lK(Ok(),t.tg()),n){if(s=n.j,ue(e,239))return Pwn(h(e,33))?Pc(s,(Qh(),gs))||Pc(s,gn):Pc(s,(Qh(),gs));if(ue(e,352))return Pc(s,(Qh(),k1));if(ue(e,186))return Pc(s,(Qh(),w2));if(ue(e,354))return Pc(s,(Qh(),N0))}return!0}function MAn(e,t,n){var s,a,u,p,v,m;if(a=n,u=a.ak(),Bd(e.e,u)){if(u.hi()){for(s=h(e.g,119),p=0;p<e.i;++p)if(v=s[p],Nr(v,a)&&p!=t)throw ne(new an(nD))}}else for(m=ac(e.e.Tg(),u),s=h(e.g,119),p=0;p<e.i;++p)if(v=s[p],m.rl(v.ak())&&p!=t)throw ne(new an(oD));return h(Sy(e,t,n),72)}function Mgt(e,t){if(t instanceof Object)try{if(t.__java$exception=e,navigator.userAgent.toLowerCase().indexOf("msie")!=-1&&$doc.documentMode<9)return;var n=e;Object.defineProperties(t,{cause:{get:function(){var s=n.Zd();return s&&s.Xd()}},suppressed:{get:function(){return n.Yd()}}})}catch{}}function Dgt(e,t){var n,s,a,u,p;if(s=t>>5,t&=31,s>=e.d)return e.e<0?(Dg(),ite):(Dg(),Gx);if(u=e.d-s,a=pe(Hn,hr,25,u+1,15,1),m_n(a,u,e.a,s,t),e.e<0){for(n=0;n<s&&e.a[n]==0;n++);if(n<s||t>0&&e.a[n]<<32-t!=0){for(n=0;n<u&&a[n]==-1;n++)a[n]=0;n==u&&++u,++a[n]}}return p=new fy(e.e,u,a),jE(p),p}function Lgt(e){var t,n,s,a;return a=mf(e),n=new Kqe(a),s=new Xqe(a),t=new qe,Vi(t,(!e.d&&(e.d=new qt(us,e,8,5)),e.d)),Vi(t,(!e.e&&(e.e=new qt(us,e,7,4)),e.e)),h(Gu(pc(Jr(new Vt(null,new Wt(t,16)),n),s),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[(Af(),Kw),Zu]))),21)}function Igt(e,t,n,s){var a,u,p,v,m;if(v=(ia(),h(t,66).Oj()),Bd(e.e,t)){if(t.hi()&&fR(e,t,s,ue(t,99)&&(h(t,18).Bb&ea)!=0))throw ne(new an(nD))}else for(m=ac(e.e.Tg(),t),a=h(e.g,119),p=0;p<e.i;++p)if(u=a[p],m.rl(u.ak()))throw ne(new an(oD));s9(e,sbe(e,t,n),v?h(s,72):R1(t,s))}function Bd(e,t){ia();var n,s,a;return t.$j()?!0:t.Zj()==-2?t==(h5(),Z5)||t==Q5||t==cie||t==uie?!0:(a=e.Tg(),oi(a,t)>=0?!1:(n=Hy((jc(),Ps),a,t),n?(s=n.Zj(),(s>1||s==-1)&&fb(Sa(Ps,n))!=3):!0)):!1}function DAn(e,t,n,s){var a,u,p,v,m;return v=Na(h(he((!t.b&&(t.b=new qt(Pn,t,4,7)),t.b),0),82)),m=Na(h(he((!t.c&&(t.c=new qt(Pn,t,5,8)),t.c),0),82)),bi(v)==bi(m)||ww(m,v)?null:(p=uA(t),p==n?s:(u=h(_n(e.a,p),10),u&&(a=u.e,a)?a:null))}function LAn(e,t){var n;switch(n=h(U(e,(tt(),U$)),276),Bn(t,"Label side selection ("+n+")",1),n.g){case 0:q0t(e,(Yu(),Z1));break;case 1:q0t(e,(Yu(),Kg));break;case 2:o2t(e,(Yu(),Z1));break;case 3:o2t(e,(Yu(),Kg));break;case 4:zgt(e,(Yu(),Z1));break;case 5:zgt(e,(Yu(),Kg))}An(t)}function Qbe(e,t,n){var s,a,u,p,v,m;if(s=b1n(n,e.length),p=e[s],p[0].k==(bn(),Pi))for(u=VXe(n,p.length),m=t.j,a=0;a<m.c.length;a++)v=(Kt(a,m.c.length),h(m.c[a],11)),(n?v.j==(Qe(),pn):v.j==(Qe(),un))&&ft(lt(U(v,(He(),AD))))&&(dl(m,a,h(U(p[u],(He(),Br)),11)),u+=n?1:-1)}function IAn(e,t){var n,s,a,u,p;p=new qe,n=t;do u=h(_n(e.b,n),128),u.B=n.c,u.D=n.d,p.c[p.c.length]=u,n=h(_n(e.k,n),17);while(n);return s=(Kt(0,p.c.length),h(p.c[0],128)),s.j=!0,s.A=h(s.d.a.ec().Kc().Pb(),17).c.i,a=h(ut(p,p.c.length-1),128),a.q=!0,a.C=h(a.d.a.ec().Kc().Pb(),17).d.i,p}function Ogt(e){if(e.g==null)switch(e.p){case 0:e.g=Bwn(e)?(on(),Hx):(on(),u2);break;case 1:e.g=MA(byn(e));break;case 2:e.g=RA(Emn(e));break;case 3:e.g=avn(e);break;case 4:e.g=new cS(svn(e));break;case 6:e.g=qp(ovn(e));break;case 5:e.g=Ke(Twn(e));break;case 7:e.g=Yk(myn(e))}return e.g}function Zbe(e){if(e.n==null)switch(e.p){case 0:e.n=Fwn(e)?(on(),Hx):(on(),u2);break;case 1:e.n=MA(vyn(e));break;case 2:e.n=RA(_mn(e));break;case 3:e.n=uvn(e);break;case 4:e.n=new cS(lvn(e));break;case 6:e.n=qp(cvn(e));break;case 5:e.n=Ke(Cwn(e));break;case 7:e.n=Yk(wyn(e))}return e.n}function Ngt(e){var t,n,s,a,u,p,v;for(u=new Q(e.a.a);u.a<u.c.c.length;)s=h(K(u),307),s.g=0,s.i=0,s.e.a.$b();for(a=new Q(e.a.a);a.a<a.c.c.length;)for(s=h(K(a),307),n=s.a.a.ec().Kc();n.Ob();)for(t=h(n.Pb(),57),v=t.c.Kc();v.Ob();)p=h(v.Pb(),57),p.a!=s&&(Qi(s.e,p),++p.a.g,++p.a.i)}function OAn(e,t){var n,s,a,u,p,v;if(v=tA(e.a,t.b),!v)throw ne(new ja("Invalid hitboxes for scanline overlap calculation."));for(p=!1,u=(s=new EE(new yE(new uS(e.a.a).a).b),new lS(s));TS(u.a.a);)if(a=(n=hP(u.a),h(n.cd(),65)),Q5n(t.b,a))k1n(e.b.a,t.b,a),p=!0;else if(p)break}function NAn(e){var t,n,s,a,u;a=h(U(e,(tt(),g2)),21),u=h(U(e,Q$),21),n=new dt(e.f.a+e.d.b+e.d.c,e.f.b+e.d.d+e.d.a),t=new _a(n),a.Hc((Iu(),bm))&&(s=h(U(e,o8),8),u.Hc((fu(),x8))&&(s.a<=0&&(s.a=20),s.b<=0&&(s.b=20)),t.a=d.Math.max(n.a,s.a),t.b=d.Math.max(n.b,s.b)),SLn(e,n,t)}function Pgt(e,t){var n,s,a,u,p,v,m,x,_,A,I;a=t?new i$e:new s$e,u=!1;do for(u=!1,x=t?Rp(e.b):e.b,m=x.Kc();m.Ob();)for(v=h(m.Pb(),29),I=bb(v.a),t||new U2(I),A=new Q(I);A.a<A.c.c.length;)_=h(K(A),10),a.Mb(_)&&(s=_,n=h(U(_,(He(),tv)),305),p=t?n.b:n.k,u=npt(s,p,t,!1));while(u)}function PAn(e,t,n){var s,a,u,p,v;for(Bn(n,"Longest path layering",1),e.a=t,v=e.a.a,e.b=pe(Hn,hr,25,v.c.length,15,1),s=0,p=new Q(v);p.a<p.c.c.length;)a=h(K(p),10),a.p=s,e.b[s]=-1,++s;for(u=new Q(v);u.a<u.c.c.length;)a=h(K(u),10),rgt(e,a);v.c=pe(yn,st,1,0,5,1),e.a=null,e.b=null,An(n)}function BAn(e,t){var n,s,a;t.a?(tA(e.b,t.b),e.a[t.b.i]=h(qY(e.b,t.b),81),n=h(UY(e.b,t.b),81),n&&(e.a[n.i]=t.b)):(s=h(qY(e.b,t.b),81),!!s&&s==e.a[t.b.i]&&!!s.d&&s.d!=t.b.d&&s.f.Fc(t.b),a=h(UY(e.b,t.b),81),!!a&&e.a[a.i]==t.b&&!!a.d&&a.d!=t.b.d&&t.b.f.Fc(a),OY(e.b,t.b))}function Bgt(e,t){var n,s,a,u,p,v;return u=e.d,v=Pe(Je(U(e,(tt(),D0)))),v<0&&(v=0,Fe(e,D0,v)),t.o.b=v,p=d.Math.floor(v/2),s=new So,Ji(s,(Qe(),un)),Ka(s,t),s.n.b=p,a=new So,Ji(a,pn),Ka(a,t),a.n.b=p,xs(e,s),n=new db,Oa(n,e),Fe(n,Da,null),Gs(n,a),xs(n,u),zDn(t,e,n),U_n(e,n),n}function FAn(e){var t,n;return n=h(U(e,(He(),Uo)),21),t=new rs,n.Hc((ha(),E_))&&(qu(t,x8t),qu(t,vxe)),(n.Hc(i8)||ft(lt(U(e,(tt(),Sne)))))&&(qu(t,vxe),n.Hc(u3)&&qu(t,_8t)),n.Hc(El)&&qu(t,k8t),n.Hc(__)&&qu(t,T8t),n.Hc(I$)&&qu(t,E8t),n.Hc(n8)&&qu(t,m8t),n.Hc(r8)&&qu(t,y8t),t}function RAn(e,t){var n,s,a,u,p,v,m,x,_,A,I;return s=e.d,u=t.d,v=s+u,m=e.e!=t.e?-1:1,v==2?(_=js(Zi(e.a[0],fa),Zi(t.a[0],fa)),I=Gn(_),A=Gn(ow(_,32)),A==0?new d0(m,I):new fy(m,2,se(ie(Hn,1),hr,25,15,[I,A]))):(n=e.a,a=t.a,p=pe(Hn,hr,25,v,15,1),ekn(n,s,a,u,p),x=new fy(m,v,p),jE(x),x)}function Fgt(e,t,n,s){var a,u;if(t){if(a=e.a.ue(n.d,t.d),a==0)return s.d=w1e(t,n.e),s.b=!0,t;u=a<0?0:1,t.a[u]=Fgt(e,t.a[u],n,s),Wv(t.a[u])&&(Wv(t.a[1-u])?(t.b=!0,t.a[0].b=!1,t.a[1].b=!1):Wv(t.a[u].a[u])?t=KB(t,1-u):Wv(t.a[u].a[1-u])&&(t=Jst(t,1-u)))}else return n;return t}function Rgt(e,t,n){var s,a,u,p;a=e.i,s=e.n,D0e(e,(Kh(),ao),a.c+s.b,n),D0e(e,oo,a.c+a.b-s.c-n[2],n),p=a.b-s.b-s.c,n[0]>0&&(n[0]+=e.d,p-=n[0]),n[2]>0&&(n[2]+=e.d,p-=n[2]),u=d.Math.max(0,p),n[1]=d.Math.max(n[1],p),D0e(e,Jo,a.c+s.b+n[0]-(n[1]-p)/2,n),t==Jo&&(e.c.b=u,e.c.c=a.c+s.b+(u-p)/2)}function jgt(){this.c=pe(ks,ka,25,(Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])).length,15,1),this.b=pe(ks,ka,25,se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un]).length,15,1),this.a=pe(ks,ka,25,se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un]).length,15,1),$he(this.c,ki),$he(this.b,Fi),$he(this.a,Fi)}function $o(e,t,n){var s,a,u,p;if(t<=n?(a=t,u=n):(a=n,u=t),s=0,e.b==null)e.b=pe(Hn,hr,25,2,15,1),e.b[0]=a,e.b[1]=u,e.c=!0;else{if(s=e.b.length,e.b[s-1]+1==a){e.b[s-1]=u;return}p=pe(Hn,hr,25,s+2,15,1),Ao(e.b,0,p,0,s),e.b=p,e.b[s-1]>=a&&(e.c=!1,e.a=!1),e.b[s++]=a,e.b[s]=u,e.c||Ny(e)}}function jAn(e,t,n){var s,a,u,p,v,m,x;for(x=t.d,e.a=new Wo(x.c.length),e.c=new zn,v=new Q(x);v.a<v.c.c.length;)p=h(K(v),101),u=new PA(null),Ue(e.a,u),Pr(e.c,p,u);for(e.b=new zn,W_n(e,t),s=0;s<x.c.length-1;s++)for(m=h(ut(t.d,s),101),a=s+1;a<x.c.length;a++)ASn(e,m,h(ut(t.d,a),101),n)}function $gt(e,t,n){var s,a,u,p,v,m;if(!mk(t)){for(m=jo(n,(ue(t,14)?h(t,14).gc():Td(t.Kc()))/e.a|0),Bn(m,mwt,1),v=new hHe,p=0,u=t.Kc();u.Ob();)s=h(u.Pb(),86),v=Cf(se(ie(Nf,1),st,20,0,[v,new ng(s)])),p<s.f.b&&(p=s.f.b);for(a=t.Kc();a.Ob();)s=h(a.Pb(),86),Fe(s,(fo(),gH),p);An(m),$gt(e,v,n)}}function $An(e,t){var n,s,a,u,p,v,m;for(n=Fi,v=(bn(),Wi),a=new Q(t.a);a.a<a.c.c.length;)s=h(K(a),10),u=s.k,u!=Wi&&(p=Je(U(s,(He(),y6e))),p==null?(n=d.Math.max(n,0),s.n.b=n+Kfe(e.a,u,v)):s.n.b=(rn(p),p)),m=Kfe(e.a,u,v),s.n.b<n+m+s.d.d&&(s.n.b=n+m+s.d.d),n=s.n.b+s.o.b+s.d.a,v=u}function HAn(e,t,n){var s,a,u,p,v,m,x,_,A;for(u=Fy(t,!1,!1),x=hM(u),A=Pe(Je(pt(t,(l9(),wte)))),a=Abt(x,A+e.a),_=new EQ(a),Oa(_,t),Pr(e.b,t,_),n.c[n.c.length]=_,m=(!t.n&&(t.n=new Ye(Ua,t,1,7)),t.n),v=new Tn(m);v.e!=v.i.gc();)p=h(On(v),137),s=xM(e,p,!0,0,0),n.c[n.c.length]=s;return _}function Hgt(e,t,n,s,a){var u,p,v,m,x,_;if(e.d&&e.d.lg(a),u=h(a.Xb(0),33),Wft(e,n,u,!1)||(p=h(a.Xb(a.gc()-1),33),Wft(e,s,p,!0))||X2e(e,a))return!0;for(_=a.Kc();_.Ob();)for(x=h(_.Pb(),33),m=t.Kc();m.Ob();)if(v=h(m.Pb(),33),aR(e,x,v))return!0;return!1}function zAn(e,t,n){var s,a,u,p,v,m,x,_,A,I;I=t.c.length,A=(x=e.Yg(n),h(x>=0?e._g(x,!1,!0):Bb(e,n,!1),58));e:for(u=A.Kc();u.Ob();){for(a=h(u.Pb(),56),_=0;_<I;++_)if(p=(Kt(_,t.c.length),h(t.c[_],72)),m=p.dd(),v=p.ak(),s=a.bh(v,!1),m==null?s!=null:!Nr(m,s))continue e;return a}return null}function GAn(e,t,n,s){var a,u,p,v;for(a=h(x0(t,(Qe(),un)).Kc().Pb(),11),u=h(x0(t,pn).Kc().Pb(),11),v=new Q(e.j);v.a<v.c.c.length;){for(p=h(K(v),11);p.e.c.length!=0;)xs(h(ut(p.e,0),17),a);for(;p.g.c.length!=0;)Gs(h(ut(p.g,0),17),u)}n||Fe(t,(He(),rf),null),s||Fe(t,(He(),Yl),null)}function Fy(e,t,n){var s,a;if((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i==0)return Fge(e);if(s=h(he((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a),0),202),t&&(jn((!s.a&&(s.a=new Gi(Zl,s,5)),s.a)),Mk(s,0),Lk(s,0),Ak(s,0),Dk(s,0)),n)for(a=(!e.a&&(e.a=new Ye(gi,e,6,6)),e.a);a.i>1;)Ow(a,a.i-1);return s}function VAn(e,t){var n,s,a,u,p,v,m;for(Bn(t,"Comment post-processing",1),u=new Q(e.b);u.a<u.c.c.length;){for(a=h(K(u),29),s=new qe,v=new Q(a.a);v.a<v.c.c.length;)p=h(K(v),10),m=h(U(p,(He(),j5)),15),n=h(U(p,N5),15),(!!m||!!n)&&(UOn(p,m,n),m&&Vi(s,m),n&&Vi(s,n));Vi(a.a,s)}An(t)}function zgt(e,t){var n,s,a,u,p,v,m;for(n=new Jm,u=new Q(e.b);u.a<u.c.c.length;){for(a=h(K(u),29),m=!0,s=0,v=new Q(a.a);v.a<v.c.c.length;)switch(p=h(K(v),10),p.k.g){case 4:++s;case 1:b0e(n,p);break;case 0:V_n(p,t);default:n.b==n.c||Lpt(n,s,m,!1,t),m=!1,s=0}n.b==n.c||Lpt(n,s,m,!0,t)}}function UAn(e,t){var n,s,a,u,p,v,m;for(a=new qe,n=0;n<=e.i;n++)s=new Nl(t),s.p=e.i-n,a.c[a.c.length]=s;for(v=new Q(e.o);v.a<v.c.c.length;)p=h(K(v),10),Ca(p,h(ut(a,e.i-e.f[p.p]),29));for(u=new Q(a);u.a<u.c.c.length;)m=h(K(u),29),m.a.c.length==0&&IE(u);t.b.c=pe(yn,st,1,0,5,1),Vi(t.b,a)}function Jbe(e,t){var n,s,a,u,p,v;for(n=0,v=new Q(t);v.a<v.c.c.length;){for(p=h(K(v),11),npe(e.b,e.d[p.p]),a=new _f(p.b);Wa(a.a)||Wa(a.b);)s=h(Wa(a.a)?K(a.a):K(a.b),17),u=Ahe(e,p==s.c?s.d:s.c),u>e.d[p.p]&&(n+=_0e(e.b,u),mg(e.a,Ke(u)));for(;!$7(e.a);)tge(e.b,h(H4(e.a),19).a)}return n}function Ggt(e,t,n){var s,a,u,p;for(u=(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a).i,a=new Tn((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));a.e!=a.i.gc();)s=h(On(a),33),(!s.a&&(s.a=new Ye(wi,s,10,11)),s.a).i==0||(u+=Ggt(e,s,!1));if(n)for(p=bi(t);p;)u+=(!p.a&&(p.a=new Ye(wi,p,10,11)),p.a).i,p=bi(p);return u}function Ow(e,t){var n,s,a,u;return e.ej()?(s=null,a=e.fj(),e.ij()&&(s=e.kj(e.pi(t),null)),n=e.Zi(4,u=c5(e,t),null,t,a),e.bj()&&u!=null&&(s=e.dj(u,s)),s?(s.Ei(n),s.Fi()):e.$i(n),u):(u=c5(e,t),e.bj()&&u!=null&&(s=e.dj(u,null),s&&s.Fi()),u)}function qAn(e){var t,n,s,a,u,p,v,m,x,_;for(x=e.a,t=new ns,m=0,s=new Q(e.d);s.a<s.c.c.length;){for(n=h(K(s),222),_=0,my(n.b,new hc),p=pr(n.b,0);p.b!=p.d.c;)u=h(gr(p),222),t.a._b(u)&&(a=n.c,v=u.c,_<v.d+v.a+x&&_+a.a+x>v.d&&(_=v.d+v.a+x));n.c.d=_,t.a.zc(n,t),m=d.Math.max(m,n.c.d+n.c.a)}return m}function ha(){ha=re,L$=new Zv("COMMENTS",0),El=new Zv("EXTERNAL_PORTS",1),E_=new Zv("HYPEREDGES",2),I$=new Zv("HYPERNODES",3),i8=new Zv("NON_FREE_PORTS",4),u3=new Zv("NORTH_SOUTH_PORTS",5),__=new Zv(nwt,6),n8=new Zv("CENTER_LABELS",7),r8=new Zv("END_LABELS",8),O$=new Zv("PARTITIONS",9)}function Nw(e){var t,n,s,a,u;for(a=new qe,t=new CE((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a)),s=new Sn(Dn(Pd(e).a.Kc(),new L));rr(s);)n=h(Un(s),79),ue(he((!n.b&&(n.b=new qt(Pn,n,4,7)),n.b),0),186)||(u=Na(h(he((!n.c&&(n.c=new qt(Pn,n,5,8)),n.c),0),82)),t.a._b(u)||(a.c[a.c.length]=u));return a}function YAn(e){var t,n,s,a,u,p;for(u=new ns,t=new CE((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a)),a=new Sn(Dn(Pd(e).a.Kc(),new L));rr(a);)s=h(Un(a),79),ue(he((!s.b&&(s.b=new qt(Pn,s,4,7)),s.b),0),186)||(p=Na(h(he((!s.c&&(s.c=new qt(Pn,s,5,8)),s.c),0),82)),t.a._b(p)||(n=u.a.zc(p,u),n==null));return u}function WAn(e,t,n,s,a){return s<0?(s=Oy(e,a,se(ie(it,1),Re,2,6,[wZ,mZ,yZ,kZ,p5,xZ,EZ,_Z,TZ,CZ,SZ,AZ]),t),s<0&&(s=Oy(e,a,se(ie(it,1),Re,2,6,["Jan","Feb","Mar","Apr",p5,"Jun","Jul","Aug","Sep","Oct","Nov","Dec"]),t)),s<0?!1:(n.k=s,!0)):s>0?(n.k=s-1,!0):!1}function KAn(e,t,n,s,a){return s<0?(s=Oy(e,a,se(ie(it,1),Re,2,6,[wZ,mZ,yZ,kZ,p5,xZ,EZ,_Z,TZ,CZ,SZ,AZ]),t),s<0&&(s=Oy(e,a,se(ie(it,1),Re,2,6,["Jan","Feb","Mar","Apr",p5,"Jun","Jul","Aug","Sep","Oct","Nov","Dec"]),t)),s<0?!1:(n.k=s,!0)):s>0?(n.k=s-1,!0):!1}function XAn(e,t,n,s,a,u){var p,v,m,x;if(v=32,s<0){if(t[0]>=e.length||(v=Is(e,t[0]),v!=43&&v!=45)||(++t[0],s=XF(e,t),s<0))return!1;v==45&&(s=-s)}return v==32&&t[0]-n==2&&a.b==2&&(m=new mN,x=m.q.getFullYear()-Og+Og-80,p=x%100,u.a=s==p,s+=(x/100|0)*100+(s<p?100:0)),u.p=s,!0}function Vgt(e,t){var n,s,a,u,p;!bi(e)||(p=h(U(t,(tt(),g2)),174),Me(pt(e,Ei))===Me((_s(),Qg))&&ya(e,Ei,$f),s=(Qv(),new Yv(bi(e))),u=new PY(bi(e)?new Yv(bi(e)):null,e),a=ebt(s,u,!1,!0),Eh(p,(Iu(),bm)),n=h(U(t,o8),8),n.a=d.Math.max(a.a,n.a),n.b=d.Math.max(a.b,n.b))}function QAn(e,t,n){var s,a,u,p,v,m;for(p=h(U(e,(He(),fne)),15).Kc();p.Ob();){switch(u=h(p.Pb(),10),h(U(u,(tt(),cc)),163).g){case 2:Ca(u,t);break;case 4:Ca(u,n)}for(a=new Sn(Dn(Id(u).a.Kc(),new L));rr(a);)s=h(Un(a),17),!(!!s.c&&!!s.d)&&(v=!s.d,m=h(U(s,x6e),11),v?xs(s,m):Gs(s,m))}}function sR(){sR=re,zte=new G4(LR,0,(Qe(),sn),sn),Ute=new G4(eJ,1,Fn,Fn),Hte=new G4(JZ,2,pn,pn),Wte=new G4(tJ,3,un,un),Vte=new G4("NORTH_WEST_CORNER",4,un,sn),Gte=new G4("NORTH_EAST_CORNER",5,sn,pn),Yte=new G4("SOUTH_WEST_CORNER",6,Fn,un),qte=new G4("SOUTH_EAST_CORNER",7,pn,Fn)}function Ry(){Ry=re,$8e=se(ie(ap,1),RZ,25,14,[1,1,2,6,24,120,720,5040,40320,362880,3628800,39916800,479001600,6227020800,87178291200,1307674368e3,{l:3506176,m:794077,h:1},{l:884736,m:916411,h:20},{l:3342336,m:3912489,h:363},{l:589824,m:3034138,h:6914},{l:3407872,m:1962506,h:138294}]),d.Math.pow(2,-65)}function Ugt(e,t){var n,s,a,u,p;if(e.c.length==0)return new As(Ke(0),Ke(0));for(n=(Kt(0,e.c.length),h(e.c[0],11)).j,p=0,u=t.g,s=t.g+1;p<e.c.length-1&&n.g<u;)++p,n=(Kt(p,e.c.length),h(e.c[p],11)).j;for(a=p;a<e.c.length-1&&n.g<s;)++a,n=(Kt(p,e.c.length),h(e.c[p],11)).j;return new As(Ke(p),Ke(a))}function ZAn(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(u=t.c.length,p=(Kt(n,t.c.length),h(t.c[n],286)),v=p.a.o.a,A=p.c,I=0,x=p.c;x<=p.f;x++){if(v<=e.a[x])return x;for(_=e.a[x],m=null,a=n+1;a<u;a++)s=(Kt(a,t.c.length),h(t.c[a],286)),s.c<=x&&s.f>=x&&(m=s);m&&(_=d.Math.max(_,m.a.o.a)),_>I&&(A=x,I=_)}return A}function JAn(e,t,n){var s,a,u;if(e.e=n,e.d=0,e.b=0,e.f=1,e.i=t,(e.e&16)==16&&(e.i=jMn(e.i)),e.j=e.i.length,Sr(e),u=Lb(e),e.d!=e.j)throw ne(new Zn(ir((Qn(),Cmt))));if(e.g){for(s=0;s<e.g.a.c.length;s++)if(a=h(lb(e.g,s),584),e.f<=a.a)throw ne(new Zn(ir((Qn(),Smt))));e.g.a.c=pe(yn,st,1,0,5,1)}return u}function eMn(e,t){var n,s,a;if(t==null){for(s=(!e.a&&(e.a=new Ye(qd,e,9,5)),new Tn(e.a));s.e!=s.i.gc();)if(n=h(On(s),678),a=n.c,(a==null?n.zb:a)==null)return n}else for(s=(!e.a&&(e.a=new Ye(qd,e,9,5)),new Tn(e.a));s.e!=s.i.gc();)if(n=h(On(s),678),Nt(t,(a=n.c,a==null?n.zb:a)))return n;return null}function LQ(e,t){var n;switch(n=null,t.g){case 1:e.e.Xe((Tr(),jre))&&(n=h(e.e.We(jre),249));break;case 3:e.e.Xe((Tr(),$re))&&(n=h(e.e.We($re),249));break;case 2:e.e.Xe((Tr(),Rre))&&(n=h(e.e.We(Rre),249));break;case 4:e.e.Xe((Tr(),Hre))&&(n=h(e.e.We(Hre),249))}return!n&&(n=h(e.e.We((Tr(),v7e)),249)),n}function qgt(e,t,n){var s,a,u,p,v,m,x,_,A;for(t.p=1,u=t.c,A=Db(t,(ua(),ec)).Kc();A.Ob();)for(_=h(A.Pb(),11),a=new Q(_.g);a.a<a.c.c.length;)s=h(K(a),17),x=s.d.i,t!=x&&(p=x.c,p.p<=u.p&&(v=u.p+1,v==n.b.c.length?(m=new Nl(n),m.p=v,Ue(n.b,m),Ca(x,m)):(m=h(ut(n.b,v),29),Ca(x,m)),qgt(e,x,n)))}function Ygt(e,t,n){var s,a,u,p,v,m;for(a=n,u=0,v=new Q(t);v.a<v.c.c.length;)p=h(K(v),33),ya(p,(Cw(),mH),Ke(a++)),m=Nw(p),s=d.Math.atan2(p.j+p.f/2,p.i+p.g/2),s+=s<0?Ky:0,s<.7853981633974483||s>Twt?hs(m,e.b):s<=Twt&&s>Cwt?hs(m,e.d):s<=Cwt&&s>Swt?hs(m,e.c):s<=Swt&&hs(m,e.a),u=Ygt(e,m,u);return a}function Dg(){Dg=re;var e;for(Fj=new d0(1,1),ste=new d0(1,10),Gx=new d0(0,0),ite=new d0(-1,1),R3e=se(ie(t3,1),Re,91,0,[Gx,Fj,new d0(1,2),new d0(1,3),new d0(1,4),new d0(1,5),new d0(1,6),new d0(1,7),new d0(1,8),new d0(1,9),ste]),Rj=pe(t3,Re,91,32,0,1),e=0;e<Rj.length;e++)Rj[e]=eM(kd(1,e))}function tMn(e,t,n,s,a,u){var p,v,m,x;for(v=!U7(Jr(e.Oc(),new I7(new zFe))).sd((Tp(),Ux)),p=e,u==(la(),Gd)&&(p=ue(p,152)?X4(h(p,152)):ue(p,131)?h(p,131).a:ue(p,54)?new U2(p):new Xv(p)),x=p.Kc();x.Ob();)m=h(x.Pb(),70),m.n.a=t.a,v?m.n.b=t.b+(s.b-m.o.b)/2:a?m.n.b=t.b:m.n.b=t.b+s.b-m.o.b,t.a+=m.o.a+n}function Wgt(e,t,n,s){var a,u,p,v,m,x;for(a=(s.c+s.a)/2,Pl(t.j),vr(t.j,a),Pl(n.e),vr(n.e,a),x=new QXe,v=new Q(e.f);v.a<v.c.c.length;)u=h(K(v),129),m=u.a,tQ(x,t,m),tQ(x,n,m);for(p=new Q(e.k);p.a<p.c.c.length;)u=h(K(p),129),m=u.b,tQ(x,t,m),tQ(x,n,m);return x.b+=2,x.a+=Ait(t,e.q),x.a+=Ait(e.q,n),x}function Kgt(e,t,n){var s,a,u,p,v;if(!mk(t)){for(v=jo(n,(ue(t,14)?h(t,14).gc():Td(t.Kc()))/e.a|0),Bn(v,mwt,1),p=new dHe,u=null,a=t.Kc();a.Ob();)s=h(a.Pb(),86),p=Cf(se(ie(Nf,1),st,20,0,[p,new ng(s)])),u&&(Fe(u,(fo(),V8t),s),Fe(s,sre,u),mK(s)==mK(u)&&(Fe(u,are,s),Fe(s,dH,u))),u=s;An(v),Kgt(e,p,n)}}function Xgt(e){var t,n,s,a,u,p,v;for(n=e.i,t=e.n,v=n.d,e.f==(wc(),$g)?v+=(n.a-e.e.b)/2:e.f==Ah&&(v+=n.a-e.e.b),a=new Q(e.d);a.a<a.c.c.length;){switch(s=h(K(a),181),p=s.rf(),u=new Rs,u.b=v,v+=p.b+e.a,e.b.g){case 0:u.a=n.c+t.b;break;case 1:u.a=n.c+t.b+(n.b-p.a)/2;break;case 2:u.a=n.c+n.b-t.c-p.a}s.tf(u)}}function Qgt(e){var t,n,s,a,u,p,v;for(n=e.i,t=e.n,v=n.c,e.b==(Xc(),jg)?v+=(n.b-e.e.a)/2:e.b==Sh&&(v+=n.b-e.e.a),a=new Q(e.d);a.a<a.c.c.length;){switch(s=h(K(a),181),p=s.rf(),u=new Rs,u.a=v,v+=p.a+e.a,e.f.g){case 0:u.b=n.d+t.d;break;case 1:u.b=n.d+t.d+(n.a-p.b)/2;break;case 2:u.b=n.d+n.a-t.a-p.b}s.tf(u)}}function nMn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j;_=n.a.c,p=n.a.c+n.a.b,u=h(_n(n.c,t),459),P=u.f,j=u.a,m=new dt(_,P),A=new dt(p,j),a=_,n.p||(a+=e.c),a+=n.F+n.v*e.b,x=new dt(a,P),I=new dt(a,j),YE(t.a,se(ie(cs,1),Re,8,0,[m,x])),v=n.d.a.gc()>1,v&&(s=new dt(a,n.b),vr(t.a,s)),YE(t.a,se(ie(cs,1),Re,8,0,[I,A]))}function Zgt(e){J2(e,new Xp(K2(q2(W2(Y2(new mp,lj),"ELK Randomizer"),'Distributes the nodes randomly on the plane, leading to very obfuscating layouts. Can be useful to demonstrate the power of "real" layout algorithms.'),new Eze))),et(e,lj,Vb,W7e),et(e,lj,zw,15),et(e,lj,BR,Ke(0)),et(e,lj,k5,Mx)}function eve(){eve=re;var e,t,n,s,a,u;for(bT=pe(Vc,Zy,25,255,15,1),nz=pe(Cl,o1,25,16,15,1),t=0;t<255;t++)bT[t]=-1;for(n=57;n>=48;n--)bT[n]=n-48<<24>>24;for(s=70;s>=65;s--)bT[s]=s-65+10<<24>>24;for(a=102;a>=97;a--)bT[a]=a-97+10<<24>>24;for(u=0;u<10;u++)nz[u]=48+u&Oi;for(e=10;e<=15;e++)nz[e]=65+e-10&Oi}function aR(e,t,n){var s,a,u,p,v,m,x,_;return v=t.i-e.g/2,m=n.i-e.g/2,x=t.j-e.g/2,_=n.j-e.g/2,u=t.g+e.g/2,p=n.g+e.g/2,s=t.f+e.g/2,a=n.f+e.g/2,v<m+p&&m<v&&x<_+a&&_<x||m<v+u&&v<m&&_<x+s&&x<_||v<m+p&&m<v&&x<_&&_<x+s?!0:m<v+u&&v<m&&x<_+a&&_<x}function rMn(e){var t,n,s,a,u;a=h(U(e,(tt(),g2)),21),u=h(U(e,Q$),21),n=new dt(e.f.a+e.d.b+e.d.c,e.f.b+e.d.d+e.d.a),t=new _a(n),a.Hc((Iu(),bm))&&(s=h(U(e,o8),8),u.Hc((fu(),x8))&&(s.a<=0&&(s.a=20),s.b<=0&&(s.b=20)),t.a=d.Math.max(n.a,s.a),t.b=d.Math.max(n.b,s.b)),ft(lt(U(e,Dne)))||CLn(e,n,t)}function iMn(e,t){var n,s,a,u;for(u=Za(t,(Qe(),Fn)).Kc();u.Ob();)s=h(u.Pb(),11),n=h(U(s,(He(),Zc)),10),n&&Th(ph(gh(bh(dh(new Il,0),.1),e.i[t.p].d),e.i[n.p].a));for(a=Za(t,sn).Kc();a.Ob();)s=h(a.Pb(),11),n=h(U(s,(He(),Zc)),10),n&&Th(ph(gh(bh(dh(new Il,0),.1),e.i[n.p].d),e.i[t.p].a))}function IQ(e){var t,n,s,a,u,p;if(!e.c){if(p=new $ze,t=fT,u=t.a.zc(e,t),u==null){for(s=new Tn(Co(e));s.e!=s.i.gc();)n=h(On(s),87),a=hR(n),ue(a,88)&&mi(p,IQ(h(a,26))),qn(p,n);t.a.Bc(e)!=null,t.a.gc()==0}sxn(p),kw(p),e.c=new ay((h(he(De((bg(),Xt).o),15),18),p.i),p.g),ou(e).b&=-33}return e.c}function tve(e){var t;if(e.c!=10)throw ne(new Zn(ir((Qn(),pj))));switch(t=e.a,t){case 110:t=10;break;case 114:t=13;break;case 116:t=9;break;case 92:case 124:case 46:case 94:case 45:case 63:case 42:case 43:case 123:case 125:case 40:case 41:case 91:case 93:break;default:throw ne(new Zn(ir((Qn(),kl))))}return t}function Jgt(e){var t,n,s,a,u;if(e.l==0&&e.m==0&&e.h==0)return"0";if(e.h==FM&&e.m==0&&e.l==0)return"-9223372036854775808";if(e.h>>19!=0)return"-"+Jgt(Hk(e));for(n=e,s="";!(n.l==0&&n.m==0&&n.h==0);){if(a=hK(SR),n=Ive(n,a,!0),t=""+uQe(Rg),!(n.l==0&&n.m==0&&n.h==0))for(u=9-t.length;u>0;u--)t="0"+t;s=t+s}return s}function sMn(){if(!Object.create||!Object.getOwnPropertyNames)return!1;var e="__proto__",t=Object.create(null);if(t[e]!==void 0)return!1;var n=Object.getOwnPropertyNames(t);return!(n.length!=0||(t[e]=42,t[e]!==42)||Object.getOwnPropertyNames(t).length==0)}function aMn(e){var t,n,s,a,u,p,v;for(t=!1,n=0,a=new Q(e.d.b);a.a<a.c.c.length;)for(s=h(K(a),29),s.p=n++,p=new Q(s.a);p.a<p.c.c.length;)u=h(K(p),10),!t&&!mk(Id(u))&&(t=!0);v=ni((la(),Q1),se(ie(J_,1),ze,103,0,[Wl,Dh])),t||(Eh(v,Gd),Eh(v,zd)),e.a=new Jct(v),Kc(e.f),Kc(e.b),Kc(e.e),Kc(e.g)}function oMn(e,t,n){var s,a,u,p,v,m,x,_,A;for(s=n.c,a=n.d,v=kf(t.c),m=kf(t.d),s==t.c?(v=Hbe(e,v,a),m=adt(t.d)):(v=adt(t.c),m=Hbe(e,m,a)),x=new dN(t.a),Ai(x,v,x.a,x.a.a),Ai(x,m,x.c.b,x.c),p=t.c==s,A=new zKe,u=0;u<x.b-1;++u)_=new As(h(Zh(x,u),8),h(Zh(x,u+1),8)),p&&u==0||!p&&u==x.b-2?A.b=_:Ue(A.a,_);return A}function cMn(e,t){var n,s,a,u;if(u=e.j.g-t.j.g,u!=0)return u;if(n=h(U(e,(tt(),L0)),19),s=h(U(t,L0),19),!!n&&!!s&&(a=n.a-s.a,a!=0))return a;switch(e.j.g){case 1:return Ui(e.n.a,t.n.a);case 2:return Ui(e.n.b,t.n.b);case 3:return Ui(t.n.a,e.n.a);case 4:return Ui(t.n.b,e.n.b);default:throw ne(new ja(Bwe))}}function nve(e,t,n,s){var a,u,p,v,m;if(Td((XS(),new Sn(Dn(Id(t).a.Kc(),new L))))>=e.a||!H2e(t,n))return-1;if(mk(h(s.Kb(t),20)))return 1;for(a=0,p=h(s.Kb(t),20).Kc();p.Ob();)if(u=h(p.Pb(),17),m=u.c.i==t?u.d.i:u.c.i,v=nve(e,m,n,s),v==-1||(a=d.Math.max(a,v),a>e.c-1))return-1;return a+1}function ept(e,t){var n,s,a,u,p,v;if(Me(t)===Me(e))return!0;if(!ue(t,15)||(s=h(t,15),v=e.gc(),s.gc()!=v))return!1;if(p=s.Kc(),e.ni()){for(n=0;n<v;++n)if(a=e.ki(n),u=p.Pb(),a==null?u!=null:!Nr(a,u))return!1}else for(n=0;n<v;++n)if(a=e.ki(n),u=p.Pb(),Me(a)!==Me(u))return!1;return!0}function tpt(e,t){var n,s,a,u,p,v;if(e.f>0){if(e.qj(),t!=null){for(u=0;u<e.d.length;++u)if(n=e.d[u],n){for(s=h(n.g,367),v=n.i,p=0;p<v;++p)if(a=s[p],Nr(t,a.dd()))return!0}}else for(u=0;u<e.d.length;++u)if(n=e.d[u],n){for(s=h(n.g,367),v=n.i,p=0;p<v;++p)if(a=s[p],Me(t)===Me(a.dd()))return!0}}return!1}function uMn(e,t,n){var s,a,u,p;Bn(n,"Orthogonally routing hierarchical port edges",1),e.a=0,s=QLn(t),tOn(t,s),BIn(e,t,s),eNn(t),a=h(U(t,(tt(),Ei)),98),u=t.b,Z2t((Kt(0,u.c.length),h(u.c[0],29)),a,t),Z2t(h(ut(u,u.c.length-1),29),a,t),p=t.b,l2t((Kt(0,p.c.length),h(p.c[0],29))),l2t(h(ut(p,p.c.length-1),29)),An(n)}function rve(e){switch(e){case 48:case 49:case 50:case 51:case 52:case 53:case 54:case 55:case 56:case 57:return e-48<<24>>24;case 97:case 98:case 99:case 100:case 101:case 102:return e-97+10<<24>>24;case 65:case 66:case 67:case 68:case 69:case 70:return e-65+10<<24>>24;default:throw ne(new Qf("Invalid hexadecimal"))}}function lMn(e,t,n){var s,a,u,p;for(Bn(n,"Processor order nodes",2),e.a=Pe(Je(U(t,(Fb(),Sxe)))),a=new di,p=pr(t.b,0);p.b!=p.d.c;)u=h(gr(p),86),ft(lt(U(u,(fo(),dm))))&&Ai(a,u,a.c.b,a.c);s=(kn(a.b!=0),h(a.a.a.c,86)),y2t(e,s),!n.b&&BK(n,1),ove(e,s,0-Pe(Je(U(s,(fo(),gH))))/2,0),!n.b&&BK(n,1),An(n)}function oR(){oR=re,v4e=new ry("SPIRAL",0),d4e=new ry("LINE_BY_LINE",1),g4e=new ry("MANHATTAN",2),f4e=new ry("JITTER",3),hte=new ry("QUADRANTS_LINE_BY_LINE",4),b4e=new ry("QUADRANTS_MANHATTAN",5),p4e=new ry("QUADRANTS_JITTER",6),h4e=new ry("COMBINE_LINE_BY_LINE_MANHATTAN",7),l4e=new ry("COMBINE_JITTER_MANHATTAN",8)}function npt(e,t,n,s){var a,u,p,v,m,x;for(m=eQ(e,n),x=eQ(t,n),a=!1;!!m&&!!x&&(s||a7n(m,x,n));)p=eQ(m,n),v=eQ(x,n),EA(t),EA(e),u=m.c,iZ(m,!1),iZ(x,!1),n?(Sw(t,x.p,u),t.p=x.p,Sw(e,m.p+1,u),e.p=m.p):(Sw(e,m.p,u),e.p=m.p,Sw(t,x.p+1,u),t.p=x.p),Ca(m,null),Ca(x,null),m=p,x=v,a=!0;return a}function hMn(e,t,n,s){var a,u,p,v,m;for(a=!1,u=!1,v=new Q(s.j);v.a<v.c.c.length;)p=h(K(v),11),Me(U(p,(He(),Br)))===Me(n)&&(p.g.c.length==0?p.e.c.length==0||(a=!0):u=!0);return m=0,a&&a^u?m=n.j==(Qe(),sn)?-e.e[s.c.p][s.p]:t-e.e[s.c.p][s.p]:u&&a^u?m=e.e[s.c.p][s.p]+1:a&&u&&(m=n.j==(Qe(),sn)?0:t/2),m}function OQ(e,t,n,s,a,u,p,v){var m,x,_;for(m=0,t!=null&&(m^=k0(t.toLowerCase())),n!=null&&(m^=k0(n)),s!=null&&(m^=k0(s)),p!=null&&(m^=k0(p)),v!=null&&(m^=k0(v)),x=0,_=u.length;x<_;x++)m^=k0(u[x]);e?m|=256:m&=-257,a?m|=16:m&=-17,this.f=m,this.i=t==null?null:(rn(t),t),this.a=n,this.d=s,this.j=u,this.g=p,this.e=v}function ive(e,t,n){var s,a;switch(a=null,t.g){case 1:a=(Rc(),Z4e);break;case 2:a=(Rc(),e5e)}switch(s=null,n.g){case 1:s=(Rc(),J4e);break;case 2:s=(Rc(),Q4e);break;case 3:s=(Rc(),t5e);break;case 4:s=(Rc(),n5e)}return!!a&&!!s?R4(e.j,new VUe(new _u(se(ie(gPn,1),st,169,0,[h(Vn(a),169),h(Vn(s),169)])))):(Rt(),Rt(),ca)}function fMn(e){var t,n,s;switch(t=h(U(e,(tt(),o8)),8),Fe(e,o8,new dt(t.b,t.a)),h(U(e,m1),248).g){case 1:Fe(e,m1,($1(),MH));break;case 2:Fe(e,m1,($1(),SH));break;case 3:Fe(e,m1,($1(),qD));break;case 4:Fe(e,m1,($1(),YD))}(e.q?e.q:(Rt(),Rt(),K1))._b(um)&&(n=h(U(e,um),8),s=n.a,n.a=n.b,n.b=s)}function rpt(e,t,n,s,a,u){if(this.b=n,this.d=a,e>=t.length)throw ne(new Ea("Greedy SwitchDecider: Free layer not in graph."));this.c=t[e],this.e=new KS(s),$K(this.e,this.c,(Qe(),un)),this.i=new KS(s),$K(this.i,this.c,pn),this.f=new ert(this.c),this.a=!u&&a.i&&!a.s&&this.c[0].k==(bn(),Pi),this.a&&A_n(this,e,t.length)}function ipt(e,t){var n,s,a,u,p,v;u=!e.B.Hc((fu(),uL)),p=e.B.Hc(Kre),e.a=new Jht(p,u,e.c),e.n&&qde(e.a.n,e.n),Oq(e.g,(Kh(),Jo),e.a),t||(s=new u9(1,u,e.c),s.n.a=e.k,j4(e.p,(Qe(),sn),s),a=new u9(1,u,e.c),a.n.d=e.k,j4(e.p,Fn,a),v=new u9(0,u,e.c),v.n.c=e.k,j4(e.p,un,v),n=new u9(0,u,e.c),n.n.b=e.k,j4(e.p,pn,n))}function dMn(e){var t,n,s;switch(t=h(U(e.d,(tt(),Hd)),218),t.g){case 2:n=GNn(e);break;case 3:n=(s=new qe,Ti(Jr(pc(Xa(Xa(new Vt(null,new Wt(e.d.b,16)),new mje),new yje),new kje),new aje),new BYe(s)),s);break;default:throw ne(new ja("Compaction not supported for "+t+" edges."))}hIn(e,n),Os(new P6(e.g),new NYe(e))}function gMn(e,t){var n;return n=new bd,t&&Oa(n,h(_n(e.a,hL),94)),ue(t,470)&&Oa(n,h(_n(e.a,fL),94)),ue(t,354)?(Oa(n,h(_n(e.a,Ua),94)),n):(ue(t,82)&&Oa(n,h(_n(e.a,Pn),94)),ue(t,239)?(Oa(n,h(_n(e.a,wi),94)),n):ue(t,186)?(Oa(n,h(_n(e.a,bu),94)),n):(ue(t,352)&&Oa(n,h(_n(e.a,us),94)),n))}function Jh(){Jh=re,qx=new sa((Tr(),NH),Ke(1)),Zj=new sa(k2,80),X4t=new sa(T7e,5),H4t=new sa(U5,Mx),W4t=new sa(Vre,Ke(1)),K4t=new sa(Ure,(on(),!0)),N4e=new nb(50),q4t=new sa(y2,N4e),L4e=LH,P4e=Z_,z4t=new sa(Pre,!1),O4e=tL,U4t=Wg,V4t=m2,G4t=E3,Y4t=gm,I4e=(ebe(),N4t),Ete=R4t,Qj=O4t,xte=P4t,B4e=F4t}function pMn(e){var t,n,s,a,u,p,v,m;for(m=new aot,v=new Q(e.a);v.a<v.c.c.length;)if(p=h(K(v),10),p.k!=(bn(),Pi)){for(pCn(m,p,new Rs),u=new Sn(Dn(qi(p).a.Kc(),new L));rr(u);)if(a=h(Un(u),17),!(a.c.i.k==Pi||a.d.i.k==Pi))for(s=pr(a.a,0);s.b!=s.d.c;)n=h(gr(s),8),t=n,b9(m,new Z6(t.a,t.b))}return m}function NQ(){NQ=re,D8e=new Xr(iee),M8e=(K7(),UD),A8e=new $t(oee,M8e),S8e=($A(),CH),gEt=new $t(yye,S8e),C8e=(lM(),Ere),dEt=new $t(kye,C8e),lEt=new $t(see,null),T8e=(vA(),_H),fEt=new $t(aee,T8e),_8e=(EN(),yre),aEt=new $t(xye,_8e),oEt=new $t(Eye,(on(),!1)),cEt=new $t(_ye,Ke(64)),uEt=new $t(Tye,!0),hEt=xre}function spt(e){var t,n,s,a,u,p;if(e.a==null)if(e.a=pe(vu,Y1,25,e.c.b.c.length,16,1),e.a[0]=!1,os(e.c,(tt(),Bne)))for(s=h(U(e.c,Bne),15),n=s.Kc();n.Ob();)t=h(n.Pb(),19).a,t>0&&t<e.a.length&&(e.a[t]=!1);else for(p=new Q(e.c.b),p.a<p.c.c.length&&K(p),a=1;p.a<p.c.c.length;)u=h(K(p),29),e.a[a++]=XCn(u)}function apt(e,t){var n,s,a,u;switch(a=e.b,t){case 1:{e.b|=1,e.b|=4,e.b|=8;break}case 2:{e.b|=2,e.b|=4,e.b|=8;break}case 4:{e.b|=1,e.b|=2,e.b|=4,e.b|=8;break}case 3:{e.b|=16,e.b|=8;break}case 0:{e.b|=32,e.b|=16,e.b|=8,e.b|=1,e.b|=2,e.b|=4;break}}if(e.b!=a&&!!e.c)for(s=new Tn(e.c);s.e!=s.i.gc();)u=h(On(s),473),n=ou(u),Lw(n,t)}function opt(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j;for(a=!1,p=t,v=0,m=p.length;v<m;++v)u=p[v],ft((on(),!!u.e))&&!h(ut(e.b,u.e.p),214).s&&(a=a|(x=u.e,_=h(ut(e.b,x.p),214),A=_.e,I=VXe(n,A.length),P=A[I][0],P.k==(bn(),Pi)?A[I]=XSn(u,A[I],n?(Qe(),un):(Qe(),pn)):_.c.Tf(A,n),j=kM(e,_,n,s),Qbe(_.e,_.o,n),j));return a}function cpt(e,t){var n,s,a,u,p;for(u=(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a).i,a=new Tn((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));a.e!=a.i.gc();)s=h(On(a),33),Me(pt(s,(Tr(),k3)))!==Me((Ld(),tT))&&(p=h(pt(t,v8),149),n=h(pt(s,v8),149),(p==n||!!p&&C0e(p,n))&&(!s.a&&(s.a=new Ye(wi,s,10,11)),s.a).i!=0&&(u+=cpt(e,s)));return u}function bMn(e){var t,n,s,a,u,p,v;for(s=0,v=0,p=new Q(e.d);p.a<p.c.c.length;)u=h(K(p),101),a=h(Gu(Jr(new Vt(null,new Wt(u.j,16)),new zue),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),n=null,s<=v?(n=(Qe(),sn),s+=a.gc()):v<s&&(n=(Qe(),Fn),v+=a.gc()),t=n,Ti(pc(a.Oc(),new Lje),new jYe(t))}function vMn(e){var t,n,s,a,u,p,v,m;for(e.b=new _0t(new _u((Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un]))),new _u((Tb(),se(ie($te,1),ze,361,0,[ev,zg,Jb])))),p=se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un]),v=0,m=p.length;v<m;++v)for(u=p[v],n=se(ie($te,1),ze,361,0,[ev,zg,Jb]),s=0,a=n.length;s<a;++s)t=n[s],wEn(e.b,u,t,new qe)}function upt(e,t){var n,s,a,u,p,v,m,x,_,A;if(p=h(h(jr(e.r,t),21),84),v=e.u.Hc((Qc(),Zg)),n=e.u.Hc(sT),s=e.u.Hc(iT),x=e.u.Hc(K5),A=e.B.Hc((fu(),HH)),_=!n&&!s&&(x||p.gc()==2),wAn(e,t),a=null,m=null,v){for(u=p.Kc(),a=h(u.Pb(),111),m=a;u.Ob();)m=h(u.Pb(),111);a.d.b=0,m.d.c=0,_&&!a.a&&(a.d.c=0)}A&&(b9n(p),v&&(a.d.b=0,m.d.c=0))}function lpt(e,t){var n,s,a,u,p,v,m,x,_,A;if(p=h(h(jr(e.r,t),21),84),v=e.u.Hc((Qc(),Zg)),n=e.u.Hc(sT),s=e.u.Hc(iT),m=e.u.Hc(K5),A=e.B.Hc((fu(),HH)),x=!n&&!s&&(m||p.gc()==2),ODn(e,t),_=null,a=null,v){for(u=p.Kc(),_=h(u.Pb(),111),a=_;u.Ob();)a=h(u.Pb(),111);_.d.d=0,a.d.a=0,x&&!_.a&&(_.d.a=0)}A&&(v9n(p),v&&(_.d.d=0,a.d.a=0))}function hpt(e,t,n){var s,a,u,p,v,m,x,_;if(a=t.k,t.p>=0)return!1;if(t.p=n.b,Ue(n.e,t),a==(bn(),ds)||a==Vo){for(p=new Q(t.j);p.a<p.c.c.length;)for(u=h(K(p),11),_=(s=new Q(new y4(u).a.g),new Ole(s));Wa(_.a);)if(x=h(K(_.a),17).d,v=x.i,m=v.k,t.c!=v.c&&(m==ds||m==Vo)&&hpt(e,v,n))return!0}return!0}function cR(e){var t;return(e.Db&64)!=0?_be(e):(t=new Ol(_be(e)),t.a+=" (changeable: ",a0(t,(e.Bb&Ch)!=0),t.a+=", volatile: ",a0(t,(e.Bb&Ww)!=0),t.a+=", transient: ",a0(t,(e.Bb&$w)!=0),t.a+=", defaultValueLiteral: ",va(t,e.j),t.a+=", unsettable: ",a0(t,(e.Bb&zc)!=0),t.a+=", derived: ",a0(t,(e.Bb&a1)!=0),t.a+=")",t.a)}function wMn(e){var t,n,s,a,u,p,v,m,x,_,A,I;for(a=xSn(e.d),p=h(U(e.b,(l9(),S4e)),116),v=p.b+p.c,m=p.d+p.a,_=a.d.a*e.e+v,x=a.b.a*e.f+m,Nhn(e.b,new dt(_,x)),I=new Q(e.g);I.a<I.c.c.length;)A=h(K(I),562),t=A.g-a.a.a,n=A.i-a.c.a,s=zr(I0n(new dt(t,n),A.a,A.b),Jf(GS(ro(_fe(A.e)),A.d*A.a,A.c*A.b),-.5)),u=Tfe(A.e),c1n(A.e,ms(s,u))}function mMn(e,t,n,s){var a,u,p,v,m;for(m=pe(ks,Re,104,(Qe(),se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un])).length,0,2),u=se(ie(ta,1),yo,61,0,[to,sn,pn,Fn,un]),p=0,v=u.length;p<v;++p)a=u[p],m[a.g]=pe(ks,ka,25,e.c[a.g],15,1);return I1t(m,e,sn),I1t(m,e,Fn),YX(m,e,sn,t,n,s),YX(m,e,pn,t,n,s),YX(m,e,Fn,t,n,s),YX(m,e,un,t,n,s),m}function yMn(e,t,n){if(Au(e.a,t)){if(wd(h(_n(e.a,t),53),n))return 1}else Pr(e.a,t,new ns);if(Au(e.a,n)){if(wd(h(_n(e.a,n),53),t))return-1}else Pr(e.a,n,new ns);if(Au(e.b,t)){if(wd(h(_n(e.b,t),53),n))return-1}else Pr(e.b,t,new ns);if(Au(e.b,n)){if(wd(h(_n(e.b,n),53),t))return 1}else Pr(e.b,n,new ns);return 0}function sve(e,t,n,s){var a,u,p,v,m,x;if(n==null){for(a=h(e.g,119),v=0;v<e.i;++v)if(p=a[v],p.ak()==t)return qs(e,p,s)}return u=(ia(),h(t,66).Oj()?h(n,72):R1(t,n)),Tu(e.e)?(x=!QA(e,t),s=Xo(e,u,s),m=t.$j()?wg(e,3,t,null,n,px(e,t,n,ue(t,99)&&(h(t,18).Bb&ea)!=0),x):wg(e,1,t,t.zj(),n,-1,x),s?s.Ei(m):s=m):s=Xo(e,u,s),s}function kMn(e){var t,n,s,a,u,p;e.q==(_s(),ed)||e.q==qo||(a=e.f.n.d+xP(h(wa(e.b,(Qe(),sn)),124))+e.c,t=e.f.n.a+xP(h(wa(e.b,Fn),124))+e.c,s=h(wa(e.b,pn),124),p=h(wa(e.b,un),124),u=d.Math.max(0,s.n.d-a),u=d.Math.max(u,p.n.d-a),n=d.Math.max(0,s.n.a-t),n=d.Math.max(n,p.n.a-t),s.n.d=u,p.n.d=u,s.n.a=n,p.n.a=n)}function xMn(e,t){var n,s,a,u,p,v,m,x,_,A,I;for(Bn(t,"Restoring reversed edges",1),m=new Q(e.b);m.a<m.c.c.length;)for(v=h(K(m),29),_=new Q(v.a);_.a<_.c.c.length;)for(x=h(K(_),10),I=new Q(x.j);I.a<I.c.c.length;)for(A=h(K(I),11),p=i1(A.g),s=p,a=0,u=s.length;a<u;++a)n=s[a],ft(lt(U(n,(He(),Ff))))&&jb(n,!1);An(t)}function fpt(){this.b=new Np,this.d=new Np,this.e=new Np,this.c=new Np,this.a=new zn,this.f=new zn,py(cs,new YHe,new WHe),py(H8e,new sze,new aze),py(K4e,new oze,new cze),py(X4e,new lze,new hze),py(g9t,new fze,new dze),py(pPn,new KHe,new XHe),py(wPn,new QHe,new ZHe),py(bPn,new JHe,new eze),py(vPn,new tze,new nze),py(kPn,new rze,new ize)}function dpt(e){var t,n,s,a,u,p;return u=0,t=Rl(e),t.Bj()&&(u|=4),(e.Bb&zc)!=0&&(u|=2),ue(e,99)?(n=h(e,18),a=aa(n),(n.Bb&go)!=0&&(u|=32),a&&(En(hw(a)),u|=8,p=a.t,(p>1||p==-1)&&(u|=16),(a.Bb&go)!=0&&(u|=64)),(n.Bb&ea)!=0&&(u|=Ww),u|=Ch):ue(t,457)?u|=512:(s=t.Bj(),!!s&&(s.i&1)!=0&&(u|=256)),(e.Bb&512)!=0&&(u|=128),u}function E9(e,t){var n,s,a,u,p;for(e=e==null?_c:(rn(e),e),a=0;a<t.length;a++)t[a]=rSn(t[a]);for(n=new Kv,p=0,s=0;s<t.length&&(u=e.indexOf("%s",p),u!=-1);)n.a+=""+$u(e==null?_c:(rn(e),e),p,u),ho(n,t[s++]),p=u+2;if(Kst(n,e,p,e.length),s<t.length){for(n.a+=" [",ho(n,t[s++]);s<t.length;)n.a+=Js,ho(n,t[s++]);n.a+="]"}return n.a}function EMn(e){var t,n,s,a,u;for(u=new Wo(e.a.c.length),a=new Q(e.a);a.a<a.c.c.length;){switch(s=h(K(a),10),n=h(U(s,(tt(),cc)),163),t=null,n.g){case 1:case 2:t=(Hp(),c3);break;case 3:case 4:t=(Hp(),t8)}t?(Fe(s,(He(),B$),(Hp(),c3)),t==t8?rR(s,n,(ua(),Jc)):t==c3&&rR(s,n,(ua(),ec))):u.c[u.c.length]=s}return u}function ave(e,t){var n,s,a,u,p,v,m;for(n=0,m=new Q(t);m.a<m.c.c.length;){for(v=h(K(m),11),npe(e.b,e.d[v.p]),p=0,a=new _f(v.b);Wa(a.a)||Wa(a.b);)s=h(Wa(a.a)?K(a.a):K(a.b),17),Qrt(s)?(u=Ahe(e,v==s.c?s.d:s.c),u>e.d[v.p]&&(n+=_0e(e.b,u),mg(e.a,Ke(u)))):++p;for(n+=e.b.d*p;!$7(e.a);)tge(e.b,h(H4(e.a),19).a)}return n}function _Mn(e,t){var n;return e.f==oie?(n=fb(Sa((jc(),Ps),t)),e.e?n==4&&t!=(h5(),Z5)&&t!=(h5(),Q5)&&t!=(h5(),cie)&&t!=(h5(),uie):n==2):!!e.d&&(e.d.Hc(t)||e.d.Hc(Y4(Sa((jc(),Ps),t)))||e.d.Hc(Hy((jc(),Ps),e.b,t)))?!0:e.f&&Ube((jc(),e.f),iA(Sa(Ps,t)))?(n=fb(Sa(Ps,t)),e.e?n==4:n==2):!1}function TMn(e,t,n,s){var a,u,p,v,m,x,_,A;return p=h(pt(n,(Tr(),q5)),8),m=p.a,_=p.b+e,a=d.Math.atan2(_,m),a<0&&(a+=Ky),a+=t,a>Ky&&(a-=Ky),v=h(pt(s,q5),8),x=v.a,A=v.b+e,u=d.Math.atan2(A,x),u<0&&(u+=Ky),u+=t,u>Ky&&(u-=Ky),vf(),xh(1e-10),d.Math.abs(a-u)<=1e-10||a==u||isNaN(a)&&isNaN(u)?0:a<u?-1:a>u?1:tb(isNaN(a),isNaN(u))}function PQ(e){var t,n,s,a,u,p,v;for(v=new zn,s=new Q(e.a.b);s.a<s.c.c.length;)t=h(K(s),57),Pr(v,t,new qe);for(a=new Q(e.a.b);a.a<a.c.c.length;)for(t=h(K(a),57),t.i=Fi,p=t.c.Kc();p.Ob();)u=h(p.Pb(),57),h(no(Ia(v.f,u)),15).Fc(t);for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),57),t.c.$b(),t.c=h(no(Ia(v.f,t)),15);Ngt(e)}function BQ(e){var t,n,s,a,u,p,v;for(v=new zn,s=new Q(e.a.b);s.a<s.c.c.length;)t=h(K(s),81),Pr(v,t,new qe);for(a=new Q(e.a.b);a.a<a.c.c.length;)for(t=h(K(a),81),t.o=Fi,p=t.f.Kc();p.Ob();)u=h(p.Pb(),81),h(no(Ia(v.f,u)),15).Fc(t);for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),81),t.f.$b(),t.f=h(no(Ia(v.f,t)),15);_gt(e)}function CMn(e,t,n,s){var a,u;for($7n(e,t,n,s),Phn(t,e.j-t.j+n),Bhn(t,e.k-t.k+s),u=new Q(t.f);u.a<u.c.c.length;)switch(a=h(K(u),324),a.a.g){case 0:zk(e,t.g+a.b.a,0,t.g+a.c.a,t.i-1);break;case 1:zk(e,t.g+t.o,t.i+a.b.a,e.o-1,t.i+a.c.a);break;case 2:zk(e,t.g+a.b.a,t.i+t.p,t.g+a.c.a,e.p-1);break;default:zk(e,0,t.i+a.b.a,t.g-1,t.i+a.c.a)}}function uR(e,t,n,s,a){var u,p,v;try{if(t>=e.o)throw ne(new Ule);v=t>>5,p=t&31,u=kd(1,Gn(kd(p,1))),a?e.n[n][v]=xf(e.n[n][v],u):e.n[n][v]=Zi(e.n[n][v],X1e(u)),u=kd(u,1),s?e.n[n][v]=xf(e.n[n][v],u):e.n[n][v]=Zi(e.n[n][v],X1e(u))}catch(m){throw m=ui(m),ue(m,320)?ne(new Ea(nJ+e.o+"*"+e.p+rJ+t+Js+n+iJ)):ne(m)}}function ove(e,t,n,s){var a,u,p;t&&(u=Pe(Je(U(t,(fo(),O0))))+s,p=n+Pe(Je(U(t,gH)))/2,Fe(t,ore,Ke(Gn(xc(d.Math.round(u))))),Fe(t,yxe,Ke(Gn(xc(d.Math.round(p))))),t.d.b==0||ove(e,h(rP((a=pr(new ng(t).a.d,0),new k4(a))),86),n+Pe(Je(U(t,gH)))+e.a,s+Pe(Je(U(t,g8)))),U(t,are)!=null&&ove(e,h(U(t,are),86),n,s))}function SMn(e,t){var n,s,a,u,p,v,m,x,_,A,I;for(m=Us(t.a),a=Pe(Je(U(m,(tt(),b2))))*2,_=Pe(Je(U(m,b3))),x=d.Math.max(a,_),u=pe(ks,ka,25,t.f-t.c+1,15,1),s=-x,n=0,v=t.b.Kc();v.Ob();)p=h(v.Pb(),10),s+=e.a[p.c.p]+x,u[n++]=s;for(s+=e.a[t.a.c.p]+x,u[n++]=s,I=new Q(t.e);I.a<I.c.c.length;)A=h(K(I),10),s+=e.a[A.c.p]+x,u[n++]=s;return u}function AMn(e,t,n,s){var a,u,p,v,m,x,_,A,I;for(I=new ag(new sWe(e)),v=se(ie(X1,1),T0,10,0,[t,n]),m=0,x=v.length;m<x;++m)for(p=v[m],A=r9(p,s).Kc();A.Ob();)for(_=h(A.Pb(),11),u=new _f(_.b);Wa(u.a)||Wa(u.b);)a=h(Wa(u.a)?K(u.a):K(u.b),17),Xs(a)||(cF(I.a,_,(on(),u2))==null,Qrt(a)&&tA(I,_==a.c?a.d:a.c));return Vn(I),new Bc(I)}function MMn(e,t){var n,s,a,u;if(u=h(pt(e,(Tr(),T3)),61).g-h(pt(t,T3),61).g,u!=0)return u;if(n=h(pt(e,zre),19),s=h(pt(t,zre),19),!!n&&!!s&&(a=n.a-s.a,a!=0))return a;switch(h(pt(e,T3),61).g){case 1:return Ui(e.i,t.i);case 2:return Ui(e.j,t.j);case 3:return Ui(t.i,e.i);case 4:return Ui(t.j,e.j);default:throw ne(new ja(Bwe))}}function cve(e){var t,n,s;return(e.Db&64)!=0?hQ(e):(t=new ju(Iye),n=e.k,n?or(or((t.a+=' "',t),n),'"'):(!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n.i>0&&(s=(!e.n&&(e.n=new Ye(Ua,e,1,7)),h(he(e.n,0),137)).a,!s||or(or((t.a+=' "',t),s),'"'))),or(Q2(or(Q2(or(Q2(or(Q2((t.a+=" (",t),e.i),","),e.j)," | "),e.g),","),e.f),")"),t.a)}function gpt(e){var t,n,s;return(e.Db&64)!=0?hQ(e):(t=new ju(Oye),n=e.k,n?or(or((t.a+=' "',t),n),'"'):(!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n.i>0&&(s=(!e.n&&(e.n=new Ye(Ua,e,1,7)),h(he(e.n,0),137)).a,!s||or(or((t.a+=' "',t),s),'"'))),or(Q2(or(Q2(or(Q2(or(Q2((t.a+=" (",t),e.i),","),e.j)," | "),e.g),","),e.f),")"),t.a)}function FQ(e,t){var n,s,a,u,p,v,m;if(t==null||t.length==0)return null;if(a=h(Fo(e.a,t),149),!a){for(s=(v=new vd(e.b).a.vc().Kc(),new qv(v));s.a.Ob();)if(n=(u=h(s.a.Pb(),42),h(u.dd(),149)),p=n.c,m=t.length,Nt(p.substr(p.length-m,m),t)&&(t.length==p.length||Is(p,p.length-t.length-1)==46)){if(a)return null;a=n}a&&Ta(e.a,t,a)}return a}function DMn(e,t){var n,s,a,u;return n=new Si,s=h(Gu(pc(new Vt(null,new Wt(e.f,16)),n),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[(Af(),Kw),Zu]))),21),a=s.gc(),s=h(Gu(pc(new Vt(null,new Wt(t.f,16)),n),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[Kw,Zu]))),21),u=s.gc(),a<u?-1:a==u?0:1}function ppt(e){var t,n,s;!os(e,(tt(),sv))||(s=h(U(e,sv),21),!s.dc()&&(n=(t=h(Vh(ga),9),new ll(t,h(vh(t,t.length),9),0)),s.Hc((Iw(),el))?Eh(n,el):Eh(n,Oh),s.Hc(Kl)||Eh(n,Kl),s.Hc(Lh)?Eh(n,Nh):s.Hc(Vd)?Eh(n,J1):s.Hc(Ih)&&Eh(n,Xl),s.Hc(Nh)?Eh(n,Lh):s.Hc(J1)?Eh(n,Vd):s.Hc(Xl)&&Eh(n,Ih),Fe(e,sv,n)))}function LMn(e){var t,n,s,a,u,p,v;for(a=h(U(e,(He(),rv)),10),s=e.j,n=(Kt(0,s.c.length),h(s.c[0],11)),p=new Q(a.j);p.a<p.c.c.length;)if(u=h(K(p),11),Me(u)===Me(U(n,Br))){u.j==(Qe(),sn)&&e.p>a.p?(Ji(u,Fn),u.d&&(v=u.o.b,t=u.a.b,u.a.b=v-t)):u.j==Fn&&a.p>e.p&&(Ji(u,sn),u.d&&(v=u.o.b,t=u.a.b,u.a.b=-(v-t)));break}return a}function IMn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j;if(u=n,n<s)for(I=(P=new PA(e.p),j=new PA(e.p),Qs(P.e,e.e),P.q=e.q,P.r=j,HP(P),Qs(j.j,e.j),j.r=P,HP(j),new As(P,j)),A=h(I.a,112),_=h(I.b,112),a=(Kt(u,t.c.length),h(t.c[u],329)),p=Wgt(e,A,_,a),x=n+1;x<=s;x++)v=(Kt(x,t.c.length),h(t.c[x],329)),m=Wgt(e,A,_,v),f8n(v,m,a,p)&&(a=v,p=m);return u}function xM(e,t,n,s,a){var u,p,v,m,x,_,A;if(!(ue(t,239)||ue(t,354)||ue(t,186)))throw ne(new an("Method only works for ElkNode-, ElkLabel and ElkPort-objects."));return p=e.a/2,m=t.i+s-p,_=t.j+a-p,x=m+t.g+e.a,A=_+t.f+e.a,u=new Oc,vr(u,new dt(m,_)),vr(u,new dt(m,A)),vr(u,new dt(x,A)),vr(u,new dt(x,_)),v=new EQ(u),Oa(v,t),n&&Pr(e.b,t,v),v}function _9(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(u=new dt(t,n),_=new Q(e.a);_.a<_.c.c.length;)for(x=h(K(_),10),zr(x.n,u),I=new Q(x.j);I.a<I.c.c.length;)for(A=h(K(I),11),a=new Q(A.g);a.a<a.c.c.length;)for(s=h(K(a),17),mw(s.a,u),p=h(U(s,(tt(),Da)),74),p&&mw(p,u),m=new Q(s.b);m.a<m.c.c.length;)v=h(K(m),70),zr(v.n,u)}function OMn(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(u=new dt(t,n),_=new Q(e.a);_.a<_.c.c.length;)for(x=h(K(_),10),zr(x.n,u),I=new Q(x.j);I.a<I.c.c.length;)for(A=h(K(I),11),a=new Q(A.g);a.a<a.c.c.length;)for(s=h(K(a),17),mw(s.a,u),p=h(U(s,(tt(),Da)),74),p&&mw(p,u),m=new Q(s.b);m.a<m.c.c.length;)v=h(K(m),70),zr(v.n,u)}function bpt(e){if((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i==0)throw ne(new z7("Edges must have a source."));if((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c).i==0)throw ne(new z7("Edges must have a target."));if(!e.b&&(e.b=new qt(Pn,e,4,7)),!(e.b.i<=1&&(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c.i<=1)))throw ne(new z7("Hyperedges are not supported."))}function vpt(e,t){var n,s,a,u,p,v,m,x,_,A;for(A=0,u=new Jm,mg(u,t);u.b!=u.c;)for(m=h(H4(u),214),x=0,_=h(U(t.j,(tt(),Vg)),339),p=Pe(Je(U(t.j,z$))),v=Pe(Je(U(t.j,tke))),_!=(Dd(),Ug)&&(x+=p*zCn(m.e,_),x+=v*dAn(m.e)),A+=b1t(m.d,m.e)+x,a=new Q(m.b);a.a<a.c.c.length;)s=h(K(a),37),n=h(ut(e.b,s.p),214),n.s||(A+=bQ(e,n));return A}function NMn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;for(P=t.length,m=P,er(0,t.length),t.charCodeAt(0)==45?(A=-1,I=1,--P):(A=1,I=0),u=(XQ(),v3t)[10],a=P/u|0,J=P%u,J!=0&&++a,v=pe(Hn,hr,25,a,15,1),n=b3t[8],p=0,j=I+(J==0?u:J),V=I;V<m;V=j,j=V+u)s=Wu(t.substr(V,j-V),$s,Ir),x=(cx(),e2e(v,v,p,n)),x+=lxn(v,p,s),v[p++]=x;_=p,e.e=A,e.d=_,e.a=v,jE(e)}function wpt(e,t,n,s,a,u,p){if(e.c=s.qf().a,e.d=s.qf().b,a&&(e.c+=a.qf().a,e.d+=a.qf().b),e.b=t.rf().a,e.a=t.rf().b,!a)n?e.c-=p+t.rf().a:e.c+=s.rf().a+p;else switch(a.Hf().g){case 0:case 2:e.c+=a.rf().a+p+u.a+p;break;case 4:e.c-=p+u.a+p+t.rf().a;break;case 1:e.c+=a.rf().a+p,e.d-=p+u.b+p+t.rf().b;break;case 3:e.c+=a.rf().a+p,e.d+=a.rf().b+p+u.b+p}}function mpt(e,t){var n,s;for(this.b=new qe,this.e=new qe,this.a=e,this.d=t,c8n(this),vxn(this),this.b.dc()?this.c=e.c.p:this.c=h(this.b.Xb(0),10).c.p,this.e.c.length==0?this.f=e.c.p:this.f=h(ut(this.e,this.e.c.length-1),10).c.p,s=h(U(e,(He(),DD)),15).Kc();s.Ob();)if(n=h(s.Pb(),70),os(n,(tt(),V$))){this.d=h(U(n,V$),227);break}}function dx(e,t,n){var s,a,u,p,v,m,x,_;for(s=h(_n(e.a,t),53),u=h(_n(e.a,n),53),a=h(_n(e.e,t),53),p=h(_n(e.e,n),53),s.a.zc(n,s),p.a.zc(t,p),_=u.a.ec().Kc();_.Ob();)x=h(_.Pb(),10),s.a.zc(x,s),Qi(h(_n(e.e,x),53),t),Qs(h(_n(e.e,x),53),a);for(m=a.a.ec().Kc();m.Ob();)v=h(m.Pb(),10),p.a.zc(v,p),Qi(h(_n(e.a,v),53),n),Qs(h(_n(e.a,v),53),u)}function EM(e,t,n){var s,a,u,p,v,m,x,_;for(s=h(_n(e.a,t),53),u=h(_n(e.a,n),53),a=h(_n(e.b,t),53),p=h(_n(e.b,n),53),s.a.zc(n,s),p.a.zc(t,p),_=u.a.ec().Kc();_.Ob();)x=h(_.Pb(),10),s.a.zc(x,s),Qi(h(_n(e.b,x),53),t),Qs(h(_n(e.b,x),53),a);for(m=a.a.ec().Kc();m.Ob();)v=h(m.Pb(),10),p.a.zc(v,p),Qi(h(_n(e.a,v),53),n),Qs(h(_n(e.a,v),53),u)}function PMn(e,t){var n,s,a;switch(Bn(t,"Breaking Point Insertion",1),s=new Lbe(e),h(U(e,(tt(),Pne)),337).g){case 2:a=new que;case 0:a=new Gue;break;default:a=new Yue}if(n=a.Vf(e,s),ft(lt(U(e,jke)))&&(n=CIn(e,n)),!a.Wf()&&os(e,J$))switch(h(U(e,J$),338).g){case 2:n=i0t(s,n);break;case 1:n=rdt(s,n)}if(n.dc()){An(t);return}CNn(e,n),An(t)}function BMn(e,t,n){var s,a,u,p,v,m,x,_,A,I;if(_=null,I=t,A=Tat(e,kat(n),I),WE(A,_d(I,p1)),p=wb(I,Pye),s=new iJe(e,A),uTn(s.a,s.b,p),v=wb(I,Eee),a=new sJe(e,A),lTn(a.a,a.b,v),(!A.b&&(A.b=new qt(Pn,A,4,7)),A.b).i==0||(!A.c&&(A.c=new qt(Pn,A,5,8)),A.c).i==0)throw u=_d(I,p1),m=wmt+u,x=m+Fx,ne(new Xf(x));return qF(I,A),INn(e,I,A),_=XK(e,I,A),_}function FMn(e,t){var n,s,a,u,p,v,m;for(a=pe(Hn,hr,25,e.e.a.c.length,15,1),p=new Q(e.e.a);p.a<p.c.c.length;)u=h(K(p),121),a[u.d]+=u.b.a.c.length;for(v=ZS(t);v.b!=0;)for(u=h(v.b==0?null:(kn(v.b!=0),pl(v,v.a.a)),121),s=J4(new Q(u.g.a));s.Ob();)n=h(s.Pb(),213),m=n.e,m.e=d.Math.max(m.e,u.e+n.a),--a[m.d],a[m.d]==0&&Ai(v,m,v.c.b,v.c)}function ypt(e){var t,n,s,a,u,p,v,m,x,_,A;for(n=$s,a=Ir,v=new Q(e.e.a);v.a<v.c.c.length;)u=h(K(v),121),a=d.Math.min(a,u.e),n=d.Math.max(n,u.e);for(t=pe(Hn,hr,25,n-a+1,15,1),p=new Q(e.e.a);p.a<p.c.c.length;)u=h(K(p),121),u.e-=a,++t[u.e];if(s=0,e.k!=null)for(x=e.k,_=0,A=x.length;_<A&&(m=x[_],t[s++]+=m,t.length!=s);++_);return t}function kpt(e){switch(e.d){case 9:case 8:return!0;case 3:case 5:case 4:case 6:return!1;case 7:return h(Zbe(e),19).a==e.o;case 1:case 2:{if(e.o==-2)return!1;switch(e.p){case 0:case 1:case 2:case 6:case 5:case 7:return eb(e.k,e.f);case 3:case 4:return e.j==e.e;default:return e.n==null?e.g==null:Nr(e.n,e.g)}}default:return!1}}function xpt(e){J2(e,new Xp(K2(q2(W2(Y2(new mp,Q9),"ELK Fixed"),"Keeps the current layout as it is, without any automatic modification. Optional coordinates can be given for nodes and edge bend points."),new xze))),et(e,Q9,Vb,G7e),et(e,Q9,nj,at(eT)),et(e,Q9,Aye,at(j7e)),et(e,Q9,Yy,at($7e)),et(e,Q9,E5,at(z7e)),et(e,Q9,VJ,at(H7e))}function lR(e,t,n){var s,a,u,p,v;if(s=Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))),v=Gn(js(V1,B1(Gn(js(n==null?0:ii(n),U1)),15))),u=KE(e,t,s),!!u&&v==u.f&&n1(n,u.i))return n;if(p=XE(e,n,v),p)throw ne(new an("value already present: "+n));return a=new GP(t,s,n,v),u?(Py(e,u),dM(e,a,u),u.e=null,u.c=null,u.i):(dM(e,a,null),O1t(e),null)}function RMn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j;_=n.a.c,p=n.a.c+n.a.b,u=h(_n(n.c,t),459),P=u.f,j=u.a,u.b?m=new dt(p,P):m=new dt(_,P),u.c?A=new dt(_,j):A=new dt(p,j),a=_,n.p||(a+=e.c),a+=n.F+n.v*e.b,x=new dt(a,P),I=new dt(a,j),YE(t.a,se(ie(cs,1),Re,8,0,[m,x])),v=n.d.a.gc()>1,v&&(s=new dt(a,n.b),vr(t.a,s)),YE(t.a,se(ie(cs,1),Re,8,0,[I,A]))}function RQ(e,t,n){var s,a,u,p,v,m;if(t)if(n<=-1){if(s=Ht(t.Tg(),-1-n),ue(s,99))return h(s,18);for(p=h(t.ah(s),153),v=0,m=p.gc();v<m;++v)if(Me(p.jl(v))===Me(e)&&(a=p.il(v),ue(a,99)&&(u=h(a,18),(u.Bb&go)!=0)))return u;throw ne(new ja("The containment feature could not be located"))}else return aa(h(Ht(e.Tg(),n),18));else return null}function jMn(e){var t,n,s,a,u;for(s=e.length,t=new G7,u=0;u<s;)if(n=Is(e,u++),!(n==9||n==10||n==12||n==13||n==32)){if(n==35){for(;u<s&&(n=Is(e,u++),!(n==13||n==10)););continue}n==92&&u<s?(a=(er(u,e.length),e.charCodeAt(u)))==35||a==9||a==10||a==12||a==13||a==32?(hk(t,a&Oi),++u):(t.a+="\\",hk(t,a&Oi),++u):hk(t,n&Oi)}return t.a}function $Mn(e,t){var n,s,a;for(s=new Q(t);s.a<s.c.c.length;)if(n=h(K(s),33),Ot(e.a,n,n),Ot(e.b,n,n),a=Nw(n),a.c.length!=0)for(e.d&&e.d.lg(a),Ot(e.a,n,(Kt(0,a.c.length),h(a.c[0],33))),Ot(e.b,n,h(ut(a,a.c.length-1),33));qK(a).c.length!=0;)a=qK(a),e.d&&e.d.lg(a),Ot(e.a,n,(Kt(0,a.c.length),h(a.c[0],33))),Ot(e.b,n,h(ut(a,a.c.length-1),33))}function HMn(e){var t,n,s,a,u,p,v,m,x,_;for(n=0,v=new Q(e.d);v.a<v.c.c.length;)p=h(K(v),101),p.i&&(p.i.c=n++);for(t=Ap(vu,[Re,Y1],[177,25],16,[n,n],2),_=e.d,a=0;a<_.c.length;a++)if(m=(Kt(a,_.c.length),h(_.c[a],101)),m.i)for(u=a+1;u<_.c.length;u++)x=(Kt(u,_.c.length),h(_.c[u],101)),x.i&&(s=C9n(m,x),t[m.i.c][x.i.c]=s,t[x.i.c][m.i.c]=s);return t}function uve(e,t,n,s){var a,u,p;return p=new HQe(t,n),e.a?s?(a=h(_n(e.b,t),283),++a.a,p.d=s.d,p.e=s.e,p.b=s,p.c=s,s.e?s.e.c=p:h(_n(e.b,t),283).b=p,s.d?s.d.b=p:e.a=p,s.d=p,s.e=p):(e.e.b=p,p.d=e.e,e.e=p,a=h(_n(e.b,t),283),a?(++a.a,u=a.c,u.c=p,p.e=u,a.c=p):(Pr(e.b,t,a=new Xde(p)),++e.c)):(e.a=e.e=p,Pr(e.b,t,new Xde(p)),++e.c),++e.d,p}function Pw(e,t){var n,s,a,u,p,v,m,x;for(n=new RegExp(t,"g"),m=pe(it,Re,2,0,6,1),s=0,x=e,u=null;;)if(v=n.exec(x),v==null||x==""){m[s]=x;break}else p=v.index,m[s]=x.substr(0,p),x=$u(x,p+v[0].length,x.length),n.lastIndex=0,u==x&&(m[s]=x.substr(0,1),x=x.substr(1)),u=x,++s;if(e.length>0){for(a=m.length;a>0&&m[a-1]=="";)--a;a<m.length&&(m.length=a)}return m}function lve(e,t){var n,s,a,u,p,v,m,x,_,A;for(A=La(t),x=null,a=!1,v=0,_=Co(A.a).i;v<_;++v)p=h(MM(A,v,(u=h(he(Co(A.a),v),87),m=u.c,ue(m,88)?h(m,26):(Bt(),nh))),26),n=lve(e,p),n.dc()||(x?(a||(a=!0,x=new SP(x)),x.Gc(n)):x=n);return s=CTn(e,t),s.dc()?x||(Rt(),Rt(),ca):x?(a||(x=new SP(x)),x.Gc(s),x):s}function jQ(e,t){var n,s,a,u,p,v,m,x,_,A;for(A=La(t),x=null,s=!1,v=0,_=Co(A.a).i;v<_;++v)u=h(MM(A,v,(a=h(he(Co(A.a),v),87),m=a.c,ue(m,88)?h(m,26):(Bt(),nh))),26),n=jQ(e,u),n.dc()||(x?(s||(s=!0,x=new SP(x)),x.Gc(n)):x=n);return p=tCn(e,t),p.dc()?x||(Rt(),Rt(),ca):x?(s||(x=new SP(x)),x.Gc(p),x):p}function _M(e,t,n){var s,a,u,p,v,m;if(ue(t,72))return qs(e,t,n);for(v=null,u=null,s=h(e.g,119),p=0;p<e.i;++p)if(a=s[p],Nr(t,a.dd())&&(u=a.ak(),ue(u,99)&&(h(u,18).Bb&go)!=0)){v=a;break}return v&&(Tu(e.e)&&(m=u.$j()?wg(e,4,u,t,null,px(e,u,t,ue(u,99)&&(h(u,18).Bb&ea)!=0),!0):wg(e,u.Kj()?2:1,u,t,u.zj(),-1,!0),n?n.Ei(m):n=m),n=_M(e,v,n)),n}function zMn(e){var t,n,s,a;s=e.o,rw(),e.A.dc()||Nr(e.A,y4e)?a=s.a:(a=pM(e.f),e.A.Hc((Iu(),oL))&&!e.B.Hc((fu(),oT))&&(a=d.Math.max(a,pM(h(wa(e.p,(Qe(),sn)),244))),a=d.Math.max(a,pM(h(wa(e.p,Fn),244)))),t=wlt(e),t&&(a=d.Math.max(a,t.a))),ft(lt(e.e.yf().We((Tr(),IH))))?s.a=d.Math.max(s.a,a):s.a=a,n=e.f.i,n.c=0,n.b=a,VQ(e.f)}function GMn(e,t){var n,s,a,u,p,v,m,x,_;if(n=t.Hh(e.a),n&&(m=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),"memberTypes")),m!=null)){for(x=new qe,u=Pw(m,"\\w"),p=0,v=u.length;p<v;++p)a=u[p],s=a.lastIndexOf("#"),_=s==-1?s1e(e,t.Aj(),a):s==0?kA(e,null,a.substr(1)):kA(e,a.substr(0,s),a.substr(s+1)),ue(_,148)&&Ue(x,h(_,148));return x}return Rt(),Rt(),ca}function VMn(e,t,n){var s,a,u,p,v,m,x,_;for(Bn(n,Dvt,1),e.bf(t),u=0;e.df(u);){for(_=new Q(t.e);_.a<_.c.c.length;)for(m=h(K(_),144),v=gg(Cf(se(ie(Nf,1),st,20,0,[t.e,t.d,t.b])));rr(v);)p=h(Un(v),357),p!=m&&(a=e.af(p,m),a&&zr(m.a,a));for(x=new Q(t.e);x.a<x.c.c.length;)m=h(K(x),144),s=m.a,Tdt(s,-e.d,-e.d,e.d,e.d),zr(m.d,s),Uh(s);e.cf(),++u}An(n)}function UMn(e,t,n){var s,a,u,p;if(p=ac(e.e.Tg(),t),s=h(e.g,119),ia(),h(t,66).Oj()){for(u=0;u<e.i;++u)if(a=s[u],p.rl(a.ak())&&Nr(a,n))return Ow(e,u),!0}else if(n!=null){for(u=0;u<e.i;++u)if(a=s[u],p.rl(a.ak())&&Nr(n,a.dd()))return Ow(e,u),!0}else for(u=0;u<e.i;++u)if(a=s[u],p.rl(a.ak())&&a.dd()==null)return Ow(e,u),!0;return!1}function qMn(e,t){var n,s,a,u,p;for(e.c==null||e.c.length<t.c.length?e.c=pe(vu,Y1,25,t.c.length,16,1):H7(e.c),e.a=new qe,s=0,p=new Q(t);p.a<p.c.c.length;)a=h(K(p),10),a.p=s++;for(n=new di,u=new Q(t);u.a<u.c.c.length;)a=h(K(u),10),e.c[a.p]||(l0t(e,a),n.b==0||(kn(n.b!=0),h(n.a.a.c,15)).gc()<e.a.c.length?DJe(n,e.a):Cp(n,e.a),e.a=new qe);return n}function YMn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j;for(p=h(he(t,0),33),yc(p,0),kc(p,0),I=new qe,I.c[I.c.length]=p,v=p,u=new nW(e.a,p.g,p.f,(fx(),K_)),P=1;P<t.i;P++)j=h(he(t,P),33),m=KQ(e,y3,j,v,u,I,n),x=KQ(e,G5,j,v,u,I,n),_=KQ(e,W_,j,v,u,I,n),A=KQ(e,Y_,j,v,u,I,n),a=aIn(e,m,x,_,A,j,v,s),yc(j,a.d),kc(j,a.e),Zhn(a,K_),u=a,v=j,I.c[I.c.length]=j;return u}function Ept(e){J2(e,new Xp(K2(q2(W2(Y2(new mp,Ix),"ELK SPOrE Overlap Removal"),'A node overlap removal algorithm proposed by Nachmanson et al. in "Node overlap removal by growing a tree".'),new PHe))),et(e,Ix,iee,at(N8e)),et(e,Ix,Vb,O8e),et(e,Ix,zw,8),et(e,Ix,oee,at(bEt)),et(e,Ix,_ye,at(L8e)),et(e,Ix,Tye,at(I8e)),et(e,Ix,eD,(on(),!1))}function _pt(e,t,n,s){var a,u,p,v,m,x,_,A,I,P;for(p=nw(t.c,n,s),A=new Q(t.a);A.a<A.c.c.length;){for(_=h(K(A),10),zr(_.n,p),P=new Q(_.j);P.a<P.c.c.length;)for(I=h(K(P),11),u=new Q(I.g);u.a<u.c.c.length;)for(a=h(K(u),17),mw(a.a,p),v=h(U(a,(tt(),Da)),74),v&&mw(v,p),x=new Q(a.b);x.a<x.c.c.length;)m=h(K(x),70),zr(m.n,p);Ue(e.a,_),_.a=e}}function WMn(e,t){var n,s,a,u,p;if(Bn(t,"Node and Port Label Placement and Node Sizing",1),iet((Y7(),new IW(e,!0,!0,new jFe))),h(U(e,(He(),Uo)),21).Hc((ha(),El)))for(u=h(U(e,(tt(),cm)),21),a=u.Hc((Qc(),aL)),p=ft(lt(U(e,Dke))),s=new Q(e.b);s.a<s.c.c.length;)n=h(K(s),29),Ti(Jr(new Vt(null,new Wt(n.a,16)),new $Fe),new cnt(u,a,p));An(t)}function KMn(e,t){var n,s,a,u,p,v;if(n=t.Hh(e.a),n&&(v=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),fj)),v!=null))switch(a=aP(v,Ec(35)),s=t.Hj(),a==-1?(p=AE(e,Vu(s)),u=v):a==0?(p=null,u=v.substr(1)):(p=v.substr(0,a),u=v.substr(a+1)),fb(Sa(e,t))){case 2:case 3:return r6n(e,s,p,u);case 0:case 4:case 5:case 6:return i6n(e,s,p,u)}return null}function hve(e,t,n){var s,a,u,p,v;if(p=(ia(),h(t,66).Oj()),Bd(e.e,t)){if(t.hi()&&fR(e,t,n,ue(t,99)&&(h(t,18).Bb&ea)!=0))return!1}else for(v=ac(e.e.Tg(),t),s=h(e.g,119),u=0;u<e.i;++u)if(a=s[u],v.rl(a.ak()))return(p?Nr(a,n):n==null?a.dd()==null:Nr(n,a.dd()))?!1:(h(Sy(e,u,p?h(n,72):R1(t,n)),72),!0);return qn(e,p?h(n,72):R1(t,n))}function TM(e){var t,n,s,a,u,p,v,m;if(e.d)throw ne(new ja((yd(Dte),WZ+Dte.k+KZ)));for(e.c==(la(),Q1)&&zy(e,Wl),n=new Q(e.a.a);n.a<n.c.c.length;)t=h(K(n),189),t.e=0;for(p=new Q(e.a.b);p.a<p.c.c.length;)for(u=h(K(p),81),u.o=Fi,a=u.f.Kc();a.Ob();)s=h(a.Pb(),81),++s.d.e;for(zOn(e),m=new Q(e.a.b);m.a<m.c.c.length;)v=h(K(m),81),v.k=!0;return e}function XMn(e,t){var n,s,a,u,p,v,m,x;for(v=new Ndt(e),n=new di,Ai(n,t,n.c.b,n.c);n.b!=0;){for(s=h(n.b==0?null:(kn(n.b!=0),pl(n,n.a.a)),113),s.d.p=1,p=new Q(s.e);p.a<p.c.c.length;)a=h(K(p),409),tdt(v,a),x=a.d,x.d.p==0&&Ai(n,x,n.c.b,n.c);for(u=new Q(s.b);u.a<u.c.c.length;)a=h(K(u),409),tdt(v,a),m=a.c,m.d.p==0&&Ai(n,m,n.c.b,n.c)}return v}function Tpt(e){var t,n,s,a,u;if(s=Pe(Je(pt(e,(Tr(),zEt)))),s!=1)for(ZN(e,s*e.g,s*e.f),n=hdn(x2n((!e.c&&(e.c=new Ye(bu,e,9,9)),e.c),new gze)),u=gg(Cf(se(ie(Nf,1),st,20,0,[(!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n),(!e.c&&(e.c=new Ye(bu,e,9,9)),e.c),n])));rr(u);)a=h(Un(u),470),a.Gg(s*a.Dg(),s*a.Eg()),a.Fg(s*a.Cg(),s*a.Bg()),t=h(a.We(m7e),8),t&&(t.a*=s,t.b*=s)}function QMn(e,t,n,s,a){var u,p,v,m,x,_,A,I;for(p=new Q(e.b);p.a<p.c.c.length;)for(u=h(K(p),29),I=hA(u.a),x=I,_=0,A=x.length;_<A;++_)switch(m=x[_],h(U(m,(tt(),cc)),163).g){case 1:_An(m),Ca(m,t),qft(m,!0,s);break;case 3:uAn(m),Ca(m,n),qft(m,!1,a)}for(v=new Ms(e.b,0);v.b<v.d.gc();)(kn(v.b<v.d.gc()),h(v.d.Xb(v.c=v.b++),29)).a.c.length==0&&Mu(v)}function ZMn(e,t){var n,s,a,u,p,v,m;if(n=t.Hh(e.a),n&&(m=Jn(Xh((!n.b&&(n.b=new Cu((Bt(),na),uo,n)),n.b),r3e)),m!=null)){for(s=new qe,u=Pw(m,"\\w"),p=0,v=u.length;p<v;++p)a=u[p],Nt(a,"##other")?Ue(s,"!##"+AE(e,Vu(t.Hj()))):Nt(a,"##local")?s.c[s.c.length]=null:Nt(a,aD)?Ue(s,AE(e,Vu(t.Hj()))):s.c[s.c.length]=a;return s}return Rt(),Rt(),ca}function JMn(e,t){var n,s,a,u;return n=new Ss,s=h(Gu(pc(new Vt(null,new Wt(e.f,16)),n),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[(Af(),Kw),Zu]))),21),a=s.gc(),s=h(Gu(pc(new Vt(null,new Wt(t.f,16)),n),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[Kw,Zu]))),21),u=s.gc(),a=a==1?1:0,u=u==1?1:0,a<u?-1:a==u?0:1}function eDn(e){var t,n,s,a,u,p,v,m,x,_,A,I;for(v=e.i,a=ft(lt(U(v,(tt(),am)))),_=0,s=0,x=new Q(e.g);x.a<x.c.c.length;)m=h(K(x),17),p=Xs(m),u=p&&a&&ft(lt(U(m,d2))),I=m.d.i,p&&u?++s:p&&!u?++_:Us(I).e==v?++s:++_;for(n=new Q(e.e);n.a<n.c.c.length;)t=h(K(n),17),p=Xs(t),u=p&&a&&ft(lt(U(t,d2))),A=t.c.i,p&&u?++_:p&&!u?++s:Us(A).e==v?++_:++s;return _-s}function jy(e,t,n,s){this.e=e,this.k=h(U(e,(He(),d3)),304),this.g=pe(X1,T0,10,t,0,1),this.b=pe(Ts,Re,333,t,7,1),this.a=pe(X1,T0,10,t,0,1),this.d=pe(Ts,Re,333,t,7,1),this.j=pe(X1,T0,10,t,0,1),this.i=pe(Ts,Re,333,t,7,1),this.p=pe(Ts,Re,333,t,7,1),this.n=pe(es,Re,476,t,8,1),CS(this.n,(on(),!1)),this.f=pe(es,Re,476,t,8,1),CS(this.f,!0),this.o=n,this.c=s}function Cpt(e,t){var n,s,a,u,p,v;if(!t.dc())if(h(t.Xb(0),286).d==(u5(),Jw))I7n(e,t);else for(s=t.Kc();s.Ob();){switch(n=h(s.Pb(),286),n.d.g){case 5:ox(e,n,zkn(e,n));break;case 0:ox(e,n,(p=n.f-n.c+1,v=(p-1)/2|0,n.c+v));break;case 4:ox(e,n,n4n(e,n));break;case 2:Zht(n),ox(e,n,(u=P2e(n),u?n.c:n.f));break;case 1:Zht(n),ox(e,n,(a=P2e(n),a?n.f:n.c))}M9n(n.a)}}function tDn(e,t){var n,s,a,u,p,v,m;if(!t.e){for(t.e=!0,s=t.d.a.ec().Kc();s.Ob();){if(n=h(s.Pb(),17),t.o&&t.d.a.gc()<=1){p=t.a.c,v=t.a.c+t.a.b,m=new dt(p+(v-p)/2,t.b),vr(h(t.d.a.ec().Kc().Pb(),17).a,m);continue}if(a=h(_n(t.c,n),459),a.b||a.c){RMn(e,n,t);continue}u=e.d==(qk(),F_)&&(a.d||a.e)&&KTn(e,t)&&t.d.a.gc()<=1,u?kOn(n,t):nMn(e,n,t)}t.k&&Os(t.d,new gFe)}}function fve(e,t,n,s,a,u){var p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(I=u,v=(s+a)/2+I,J=n*d.Math.cos(v),oe=n*d.Math.sin(v),fe=J-t.g/2,le=oe-t.f/2,yc(t,fe),kc(t,le),A=e.a.jg(t),V=2*d.Math.acos(n/n+e.c),V<a-s?(P=V/A,p=(s+a-V)/2):(P=(a-s)/A,p=s),j=Nw(t),e.e&&(e.e.kg(e.d),e.e.lg(j)),x=new Q(j);x.a<x.c.c.length;)m=h(K(x),33),_=e.a.jg(m),fve(e,m,n+e.c,p,p+P*_,u),p+=P*_}function nDn(e,t,n){var s;switch(s=n.q.getMonth(),t){case 5:or(e,se(ie(it,1),Re,2,6,["J","F","M","A","M","J","J","A","S","O","N","D"])[s]);break;case 4:or(e,se(ie(it,1),Re,2,6,[wZ,mZ,yZ,kZ,p5,xZ,EZ,_Z,TZ,CZ,SZ,AZ])[s]);break;case 3:or(e,se(ie(it,1),Re,2,6,["Jan","Feb","Mar","Apr",p5,"Jun","Jul","Aug","Sep","Oct","Nov","Dec"])[s]);break;default:j1(e,s+1,t)}}function $Q(e,t){var n,s,a,u,p;if(Bn(t,"Network simplex",1),e.e.a.c.length<1){An(t);return}for(u=new Q(e.e.a);u.a<u.c.c.length;)a=h(K(u),121),a.e=0;for(p=e.e.a.c.length>=40,p&&GDn(e),YLn(e),CAn(e),n=jht(e),s=0;!!n&&s<e.f;)sDn(e,n,TCn(e,n)),n=jht(e),++s;p&&KEn(e),e.a?jCn(e,ypt(e)):ypt(e),e.b=null,e.d=null,e.p=null,e.c=null,e.g=null,e.i=null,e.n=null,e.o=null,An(t)}function rDn(e,t,n,s){var a,u,p,v,m,x,_,A,I;for(m=new dt(n,s),ms(m,h(U(t,(kg(),b_)),8)),I=new Q(t.e);I.a<I.c.c.length;)A=h(K(I),144),zr(A.d,m),Ue(e.e,A);for(v=new Q(t.c);v.a<v.c.c.length;){for(p=h(K(v),282),u=new Q(p.a);u.a<u.c.c.length;)a=h(K(u),559),zr(a.d,m);Ue(e.c,p)}for(_=new Q(t.d);_.a<_.c.c.length;)x=h(K(_),447),zr(x.d,m),Ue(e.d,x)}function dve(e,t){var n,s,a,u,p,v,m,x;for(m=new Q(t.j);m.a<m.c.c.length;)for(v=h(K(m),11),a=new _f(v.b);Wa(a.a)||Wa(a.b);)s=h(Wa(a.a)?K(a.a):K(a.b),17),n=s.c==v?s.d:s.c,u=n.i,t!=u&&(x=h(U(s,(tt(),l8)),19).a,x<0&&(x=0),p=u.p,e.b[p]==0&&(s.d==n?(e.a[p]-=x+1,e.a[p]<=0&&e.c[p]>0&&vr(e.f,u)):(e.c[p]-=x+1,e.c[p]<=0&&e.a[p]>0&&vr(e.e,u))))}function iDn(e){var t,n,s,a,u,p,v,m,x;for(v=new ag(h(Vn(new lh),62)),x=Fi,n=new Q(e.d);n.a<n.c.c.length;){for(t=h(K(n),222),x=t.c.c;v.a.c!=0&&(m=h(Ibn(B3n(v.a)),222),m.c.c+m.c.b<x);)FE(v.a,m)!=null;for(p=(a=new EE(new yE(new uS(v.a).a).b),new lS(a));TS(p.a.a);)u=(s=hP(p.a),h(s.cd(),222)),vr(u.b,t),vr(t.b,u);cF(v.a,t,(on(),u2))==null}}function Spt(e,t,n){var s,a,u,p,v,m,x,_,A;for(u=new Wo(t.c.length),x=new Q(t);x.a<x.c.c.length;)p=h(K(x),10),Ue(u,e.b[p.c.p][p.p]);for(dIn(e,u,n),A=null;A=gOn(u);)iLn(e,h(A.a,233),h(A.b,233),u);for(t.c=pe(yn,st,1,0,5,1),a=new Q(u);a.a<a.c.c.length;)for(s=h(K(a),233),v=s.d,m=0,_=v.length;m<_;++m)p=v[m],t.c[t.c.length]=p,e.a[p.c.p][p.p].a=wf(s.g,s.d[0]).a}function gve(e,t){var n,s,a,u;if(0<(ue(e,14)?h(e,14).gc():Td(e.Kc()))){if(a=t,1<a){for(--a,u=new J$e,s=e.Kc();s.Ob();)n=h(s.Pb(),86),u=Cf(se(ie(Nf,1),st,20,0,[u,new ng(n)]));return gve(u,a)}if(a<0){for(u=new eHe,s=e.Kc();s.Ob();)n=h(s.Pb(),86),u=Cf(se(ie(Nf,1),st,20,0,[u,new ng(n)]));if(0<(ue(u,14)?h(u,14).gc():Td(u.Kc())))return gve(u,a)}}return h(rP(e.Kc()),86)}function fu(){fu=re,x8=new sy("DEFAULT_MINIMUM_SIZE",0),lL=new sy("MINIMUM_SIZE_ACCOUNTS_FOR_PADDING",1),RH=new sy("COMPUTE_PADDING",2),oT=new sy("OUTSIDE_NODE_LABELS_OVERHANG",3),jH=new sy("PORTS_OVERHANG",4),HH=new sy("UNIFORM_PORT_SPACING",5),$H=new sy("SPACE_EFFICIENT_PORT_LABELS",6),Kre=new sy("FORCE_TABULAR_NODE_LABELS",7),uL=new sy("ASYMMETRICAL",8)}function HQ(e,t){var n,s,a,u,p,v,m,x;if(t){if(n=(u=t.Tg(),u?Vu(u).Nh().Jh(u):null),n){for(Yp(e,t,n),a=t.Tg(),m=0,x=(a.i==null&&s1(a),a.i).length;m<x;++m)v=(s=(a.i==null&&s1(a),a.i),m>=0&&m<s.length?s[m]:null),v.Ij()&&!v.Jj()&&(ue(v,322)?g8n(e,h(v,34),t,n):(p=h(v,18),(p.Bb&go)!=0&&_En(e,p,t,n)));t.kh()&&h(n,49).vh(h(t,49).qh())}return n}else return null}function sDn(e,t,n){var s,a,u;if(!t.f)throw ne(new an("Given leave edge is no tree edge."));if(n.f)throw ne(new an("Given enter edge is a tree edge already."));for(t.f=!1,Lfe(e.p,t),n.f=!0,Qi(e.p,n),s=n.e.e-n.d.e-n.a,wQ(e,n.e,t)||(s=-s),u=new Q(e.e.a);u.a<u.c.c.length;)a=h(K(u),121),wQ(e,a,t)||(a.e+=s);e.j=1,H7(e.c),wbe(e,h(K(new Q(e.e.a)),121)),P2t(e)}function Apt(e,t){var n,s,a,u,p,v;if(v=h(U(t,(tt(),Ei)),98),v==(_s(),ed)||v==qo)for(a=new dt(t.f.a+t.d.b+t.d.c,t.f.b+t.d.d+t.d.a).b,p=new Q(e.a);p.a<p.c.c.length;)u=h(K(p),10),u.k==(bn(),Pi)&&(n=h(U(u,(He(),co)),61),!(n!=(Qe(),pn)&&n!=un)&&(s=Pe(Je(U(u,iv))),v==ed&&(s*=a),u.n.b=s-h(U(u,ov),8).b,sF(u,!1,!0)))}function Mpt(e,t,n,s){var a,u,p,v,m,x,_,A,I,P;if(_4n(e,t,n),u=t[n],P=s?(Qe(),un):(Qe(),pn),s0n(t.length,n,s)){for(a=t[s?n-1:n+1],R0e(e,a,s?(ua(),ec):(ua(),Jc)),m=u,_=0,I=m.length;_<I;++_)p=m[_],r2e(e,p,P);for(R0e(e,u,s?(ua(),Jc):(ua(),ec)),v=a,x=0,A=v.length;x<A;++x)p=v[x],p.e||r2e(e,p,KA(P))}else for(v=u,x=0,A=v.length;x<A;++x)p=v[x],r2e(e,p,P);return!1}function aDn(e,t,n,s){var a,u,p,v,m,x,_;m=Za(t,n),(n==(Qe(),Fn)||n==un)&&(m=ue(m,152)?X4(h(m,152)):ue(m,131)?h(m,131).a:ue(m,54)?new U2(m):new Xv(m)),p=!1;do for(a=!1,u=0;u<m.gc()-1;u++)x=h(m.Xb(u),11),v=h(m.Xb(u+1),11),F9n(e,x,v,s)&&(p=!0,JW(e.a,h(m.Xb(u),11),h(m.Xb(u+1),11)),_=h(m.Xb(u+1),11),m._c(u+1,h(m.Xb(u),11)),m._c(u,_),a=!0);while(a);return p}function oDn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j;if(Tu(e.e)){if(t!=n&&(a=h(e.g,119),P=a[n],p=P.ak(),Bd(e.e,p))){for(j=ac(e.e.Tg(),p),m=-1,v=-1,s=0,x=0,A=t>n?t:n;x<=A;++x)x==n?v=s++:(u=a[x],_=j.rl(u.ak()),x==t&&(m=x==A&&!_?s-1:s),_&&++s);return I=h(a9(e,t,n),72),v!=m&&$6(e,new yA(e.e,7,p,Ke(v),P.dd(),m)),I}}else return h(yQ(e,t,n),72);return h(a9(e,t,n),72)}function cDn(e,t){var n,s,a,u,p,v,m;for(Bn(t,"Port order processing",1),m=h(U(e,(tt(),Lke)),421),s=new Q(e.b);s.a<s.c.c.length;)for(n=h(K(s),29),u=new Q(n.a);u.a<u.c.c.length;)a=h(K(u),10),p=h(U(a,Ei),98),v=a.j,p==(_s(),x2)||p==ed||p==qo?(Rt(),hs(v,F5e)):p!=$f&&p!=Qg&&(Rt(),hs(v,R5t),E8n(v),m==(CA(),qne)&&hs(v,F5t)),a.i=!0,pbe(a);An(t)}function uDn(e){var t,n,s,a,u,p,v,m;for(m=new zn,t=new _q,p=e.Kc();p.Ob();)a=h(p.Pb(),10),v=ib(xS(new _p,a),t),sc(m.f,a,v);for(u=e.Kc();u.Ob();)for(a=h(u.Pb(),10),s=new Sn(Dn(qi(a).a.Kc(),new L));rr(s);)n=h(Un(s),17),!Xs(n)&&Th(ph(gh(dh(bh(new Il,d.Math.max(1,h(U(n,(tt(),Ike)),19).a)),1),h(_n(m,n.c.i),121)),h(_n(m,n.d.i),121)));return t}function Dpt(){Dpt=re,x8t=Lr(new rs,(Zs(),Go),(oa(),x5e)),vxe=Lr(new rs,oc,s$),_8t=Wc(Lr(new rs,oc,d$),Pa,f$),k8t=Wc(Lr(Lr(new rs,oc,v5e),Go,w5e),Pa,m5e),T8t=Ib(Ib(CN(Wc(Lr(new rs,w1,v$),Pa,b$),Go),p$),w$),E8t=Wc(new rs,Pa,E5e),m8t=Wc(Lr(Lr(Lr(new rs,Hg,o$),Go,u$),Go,Yx),Pa,c$),y8t=Wc(Lr(Lr(new rs,Go,Yx),Go,i$),Pa,r$)}function lDn(e,t,n,s,a,u){var p,v,m,x,_,A,I;for(x=Ult(t)-Ult(e),p=ydt(t,x),m=rc(0,0,0);x>=0&&(v=h7n(e,p),!(v&&(x<22?m.l|=1<<x:x<44?m.m|=1<<x-22:m.h|=1<<x-44,e.l==0&&e.m==0&&e.h==0)));)_=p.m,A=p.h,I=p.l,p.h=A>>>1,p.m=_>>>1|(A&1)<<21,p.l=I>>>1|(_&1)<<21,--x;return n&&KK(m),u&&(s?(Rg=Hk(e),a&&(Rg=Slt(Rg,(Ck(),m3e)))):Rg=rc(e.l,e.m,e.h)),m}function hDn(e,t){var n,s,a,u,p,v,m,x,_,A;for(x=e.e[t.c.p][t.p]+1,m=t.c.a.c.length+1,v=new Q(e.a);v.a<v.c.c.length;){for(p=h(K(v),11),A=0,u=0,a=gg(Cf(se(ie(Nf,1),st,20,0,[new j6(p),new y4(p)])));rr(a);)s=h(Un(a),11),s.i.c==t.c&&(A+=Jdn(e,s.i)+1,++u);n=A/u,_=p.j,_==(Qe(),pn)?n<x?e.f[p.p]=e.c-n:e.f[p.p]=e.b+(m-n):_==un&&(n<x?e.f[p.p]=e.b+n:e.f[p.p]=e.c-(m-n))}}function Wu(e,t,n){var s,a,u,p,v;if(e==null)throw ne(new Qf(_c));for(u=e.length,p=u>0&&(er(0,e.length),e.charCodeAt(0)==45||(er(0,e.length),e.charCodeAt(0)==43))?1:0,s=p;s<u;s++)if(yft((er(s,e.length),e.charCodeAt(s)))==-1)throw ne(new Qf(Gb+e+'"'));if(v=parseInt(e,10),a=v<t,isNaN(v))throw ne(new Qf(Gb+e+'"'));if(a||v>n)throw ne(new Qf(Gb+e+'"'));return v}function fDn(e){var t,n,s,a,u,p,v;for(p=new di,u=new Q(e.a);u.a<u.c.c.length;)a=h(K(u),112),wq(a,a.f.c.length),KO(a,a.k.c.length),a.i==0&&(a.o=0,Ai(p,a,p.c.b,p.c));for(;p.b!=0;)for(a=h(p.b==0?null:(kn(p.b!=0),pl(p,p.a.a)),112),s=a.o+1,n=new Q(a.f);n.a<n.c.c.length;)t=h(K(n),129),v=t.a,gle(v,d.Math.max(v.o,s)),KO(v,v.i-1),v.i==0&&Ai(p,v,p.c.b,p.c)}function dDn(e){var t,n,s,a,u,p,v,m;for(p=new Q(e);p.a<p.c.c.length;){for(u=h(K(p),79),s=Na(h(he((!u.b&&(u.b=new qt(Pn,u,4,7)),u.b),0),82)),v=s.i,m=s.j,a=h(he((!u.a&&(u.a=new Ye(gi,u,6,6)),u.a),0),202),eP(a,a.j+v,a.k+m),JN(a,a.b+v,a.c+m),n=new Tn((!a.a&&(a.a=new Gi(Zl,a,5)),a.a));n.e!=n.i.gc();)t=h(On(n),469),kfe(t,t.a+v,t.b+m);Tge(h(pt(u,(Tr(),x3)),74),v,m)}}function gx(e){var t;switch(e){case 100:return Gy(l_,!0);case 68:return Gy(l_,!1);case 119:return Gy(jee,!0);case 87:return Gy(jee,!1);case 115:return Gy($ee,!0);case 83:return Gy($ee,!1);case 99:return Gy(Hee,!0);case 67:return Gy(Hee,!1);case 105:return Gy(zee,!0);case 73:return Gy(zee,!1);default:throw ne(new Ya((t=e,Uyt+t.toString(16))))}}function gDn(e){var t,n,s,a,u;switch(a=h(ut(e.a,0),10),t=new Nd(e),Ue(e.a,t),t.o.a=d.Math.max(1,a.o.a),t.o.b=d.Math.max(1,a.o.b),t.n.a=a.n.a,t.n.b=a.n.b,h(U(a,(He(),co)),61).g){case 4:t.n.a+=2;break;case 1:t.n.b+=2;break;case 2:t.n.a-=2;break;case 3:t.n.b-=2}return s=new So,Ka(s,t),n=new db,u=h(ut(a.j,0),11),Gs(n,u),xs(n,s),zr(Uh(s.n),u.n),zr(Uh(s.a),u.a),t}function Lpt(e,t,n,s,a){n&&(!s||(e.c-e.b&e.a.length-1)>1)&&t==1&&h(e.a[e.b],10).k==(bn(),Ou)?d5(h(e.a[e.b],10),(Yu(),Z1)):s&&(!n||(e.c-e.b&e.a.length-1)>1)&&t==1&&h(e.a[e.c-1&e.a.length-1],10).k==(bn(),Ou)?d5(h(e.a[e.c-1&e.a.length-1],10),(Yu(),Kg)):(e.c-e.b&e.a.length-1)==2?(d5(h(e9(e),10),(Yu(),Z1)),d5(h(e9(e),10),Kg)):aSn(e,a),x0e(e)}function pDn(e,t,n){var s,a,u,p,v;for(u=0,a=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));a.e!=a.i.gc();)s=h(On(a),33),p="",(!s.n&&(s.n=new Ye(Ua,s,1,7)),s.n).i==0||(p=h(he((!s.n&&(s.n=new Ye(Ua,s,1,7)),s.n),0),137).a),v=new ZK(u++,t,p),Oa(v,s),Fe(v,(fo(),U_),s),v.e.b=s.j+s.f/2,v.f.a=d.Math.max(s.g,1),v.e.a=s.i+s.g/2,v.f.b=d.Math.max(s.f,1),vr(t.b,v),sc(n.f,s,v)}function bDn(e){var t,n,s,a,u;s=h(U(e,(He(),Br)),33),u=h(pt(s,(tt(),g2)),174).Hc((Iu(),E2)),e.e||(a=h(U(e,Uo),21),t=new dt(e.f.a+e.d.b+e.d.c,e.f.b+e.d.d+e.d.a),a.Hc((ha(),El))?(ya(s,Ei,(_s(),qo)),$b(s,t.a,t.b,!1,!0)):ft(lt(pt(s,Dne)))||$b(s,t.a,t.b,!0,!0)),u?ya(s,g2,Lt(E2)):ya(s,g2,(n=h(Vh(aT),9),new ll(n,h(vh(n,n.length),9),0)))}function pve(e,t,n){var s,a,u,p;if(t[0]>=e.length)return n.o=0,!0;switch(Is(e,t[0])){case 43:a=1;break;case 45:a=-1;break;default:return n.o=0,!0}if(++t[0],u=t[0],p=XF(e,t),p==0&&t[0]==u)return!1;if(t[0]<e.length&&Is(e,t[0])==58){if(s=p*60,++t[0],u=t[0],p=XF(e,t),p==0&&t[0]==u)return!1;s+=p}else s=p,s<24&&t[0]-u<=2?s*=60:s=s%100+(s/100|0)*60;return s*=a,n.o=-s,!0}function vDn(e){var t,n,s,a,u,p,v,m,x;for(p=new qe,s=new Sn(Dn(qi(e.b).a.Kc(),new L));rr(s);)n=h(Un(s),17),Xs(n)&&Ue(p,new wot(n,rct(e,n.c),rct(e,n.d)));for(x=(u=new vd(e.e).a.vc().Kc(),new qv(u));x.a.Ob();)v=(t=h(x.a.Pb(),42),h(t.dd(),113)),v.d.p=0;for(m=(a=new vd(e.e).a.vc().Kc(),new qv(a));m.a.Ob();)v=(t=h(m.a.Pb(),42),h(t.dd(),113)),v.d.p==0&&Ue(e.d,XMn(e,v))}function wDn(e){var t,n,s,a,u,p,v;for(u=mf(e),a=new Tn((!e.e&&(e.e=new qt(us,e,7,4)),e.e));a.e!=a.i.gc();)if(s=h(On(a),79),v=Na(h(he((!s.c&&(s.c=new qt(Pn,s,5,8)),s.c),0),82)),!ww(v,u))return!0;for(n=new Tn((!e.d&&(e.d=new qt(us,e,8,5)),e.d));n.e!=n.i.gc();)if(t=h(On(n),79),p=Na(h(he((!t.b&&(t.b=new qt(Pn,t,4,7)),t.b),0),82)),!ww(p,u))return!0;return!1}function mDn(e){var t,n,s,a,u,p,v,m;for(m=new Oc,t=pr(e,0),v=null,n=h(gr(t),8),a=h(gr(t),8);t.b!=t.d.c;)v=n,n=a,a=h(gr(t),8),u=Xct(ms(new dt(v.a,v.b),n)),p=Xct(ms(new dt(a.a,a.b),n)),s=10,s=d.Math.min(s,d.Math.abs(u.a+u.b)/2),s=d.Math.min(s,d.Math.abs(p.a+p.b)/2),u.a=eA(u.a)*s,u.b=eA(u.b)*s,p.a=eA(p.a)*s,p.b=eA(p.b)*s,vr(m,zr(u,n)),vr(m,zr(p,n));return m}function Ku(e,t,n,s){var a,u,p,v,m;return p=e.eh(),m=e.Zg(),a=null,m?!!t&&(RQ(e,t,n).Bb&ea)==0?(s=qs(m.Vk(),e,s),e.uh(null),a=t.fh()):m=null:(p&&(m=p.fh()),t&&(a=t.fh())),m!=a&&!!m&&m.Zk(e),v=e.Vg(),e.Rg(t,n),m!=a&&!!a&&a.Yk(e),e.Lg()&&e.Mg()&&(!!p&&v>=0&&v!=n&&(u=new fs(e,1,v,p,null),s?s.Ei(u):s=u),n>=0&&(u=new fs(e,1,n,v==n?p:null,t),s?s.Ei(u):s=u)),s}function Ipt(e){var t,n,s;if(e.b==null){if(s=new s0,e.i!=null&&(va(s,e.i),s.a+=":"),(e.f&256)!=0){for((e.f&256)!=0&&e.a!=null&&(Ivn(e.i)||(s.a+="//"),va(s,e.a)),e.d!=null&&(s.a+="/",va(s,e.d)),(e.f&16)!=0&&(s.a+="/"),t=0,n=e.j.length;t<n;t++)t!=0&&(s.a+="/"),va(s,e.j[t]);e.g!=null&&(s.a+="?",va(s,e.g))}else va(s,e.a);e.e!=null&&(s.a+="#",va(s,e.e)),e.b=s.a}return e.b}function yDn(e,t){var n,s,a,u,p,v;for(a=new Q(t.a);a.a<a.c.c.length;)s=h(K(a),10),u=U(s,(He(),Br)),ue(u,11)&&(p=h(u,11),v=Qpt(t,s,p.o.a,p.o.b),p.n.a=v.a,p.n.b=v.b,Ji(p,h(U(s,co),61)));n=new dt(t.f.a+t.d.b+t.d.c,t.f.b+t.d.d+t.d.a),h(U(t,(He(),Uo)),21).Hc((ha(),El))?(Fe(e,(tt(),Ei),(_s(),qo)),h(U(Us(e),Uo),21).Fc(i8),N2t(e,n,!1)):N2t(e,n,!0)}function kDn(e,t,n){var s,a,u,p,v,m;if(Bn(n,"Minimize Crossings "+e.a,1),s=t.b.c.length==0||!U7(Jr(new Vt(null,new Wt(t.b,16)),new I7(new g$e))).sd((Tp(),Ux)),m=t.b.c.length==1&&h(ut(t.b,0),29).a.c.length==1,u=Me(U(t,(tt(),sm)))===Me((Ld(),B0)),s||m&&!u){An(n);return}a=vAn(e,t),p=(v=h(Zh(a,0),214),v.c.Rf()?v.c.Lf()?new eWe(e):new tWe(e):new JYe(e)),w6n(a,p),V6n(e),An(n)}function xDn(e,t,n,s){var a,u,p,v,m;if(m=Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))),a=Gn(js(V1,B1(Gn(js(n==null?0:ii(n),U1)),15))),v=XE(e,t,m),p=KE(e,n,a),!!v&&a==v.a&&n1(n,v.g))return n;if(!!p&&!s)throw ne(new an("key already present: "+n));return v&&Py(e,v),p&&Py(e,p),u=new GP(n,a,t,m),dM(e,u,p),p&&(p.e=null,p.c=null),v&&(v.e=null,v.c=null),O1t(e),v?v.g:null}function Opt(e,t,n){var s,a,u,p,v;for(u=0;u<t;u++){for(s=0,v=u+1;v<t;v++)s=Vs(Vs(js(Zi(e[u],fa),Zi(e[v],fa)),Zi(n[u+v],fa)),Zi(Gn(s),fa)),n[u+v]=Gn(s),s=ow(s,32);n[u+t]=Gn(s)}for(J4n(n,n,t<<1),s=0,a=0,p=0;a<t;++a,p++)s=Vs(Vs(js(Zi(e[a],fa),Zi(e[a],fa)),Zi(n[p],fa)),Zi(Gn(s),fa)),n[p]=Gn(s),s=ow(s,32),++p,s=Vs(s,Zi(n[p],fa)),n[p]=Gn(s),s=ow(s,32);return n}function Npt(e,t,n){var s,a,u,p,v,m,x,_;if(!mk(t)){for(m=Pe(Je(_w(n.c,(tt(),I_)))),x=h(_w(n.c,PD),142),!x&&(x=new B7),s=n.a,a=null,v=t.Kc();v.Ob();)p=h(v.Pb(),11),_=0,a?(_=m,_+=a.o.b):_=x.d,u=ib(xS(new _p,p),e.f),Pr(e.k,p,u),Th(ph(gh(dh(bh(new Il,0),Li(d.Math.ceil(_))),s),u)),a=p,s=u;Th(ph(gh(dh(bh(new Il,0),Li(d.Math.ceil(x.a+a.o.b))),s),n.d))}}function EDn(e,t,n,s,a,u,p,v){var m,x,_,A,I,P;return P=!1,I=u-n.s,_=n.t-t.f+(x=M9(n,I,!1),x.a),s.g+v>I?!1:(A=(m=M9(s,I,!1),m.a),_+v+A<=t.b&&(mA(n,u-n.s),n.c=!0,mA(s,u-n.s),tM(s,n.s,n.t+n.d+v),s.k=!0,Sge(n.q,s),P=!0,a&&(zB(t,s),s.j=t,e.c.length>p&&(aM((Kt(p,e.c.length),h(e.c[p],200)),s),(Kt(p,e.c.length),h(e.c[p],200)).a.c.length==0&&f0(e,p)))),P)}function _Dn(e,t){var n,s,a,u,p,v;if(Bn(t,"Partition midprocessing",1),a=new pb,Ti(Jr(new Vt(null,new Wt(e.a,16)),new sRe),new yYe(a)),a.d!=0){for(v=h(Gu(Tst((u=a.i,new Vt(null,(u||(a.i=new hy(a,a.c))).Nc()))),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),s=v.Kc(),n=h(s.Pb(),19);s.Ob();)p=h(s.Pb(),19),SSn(h(jr(a,n),21),h(jr(a,p),21)),n=p;An(t)}}function Ppt(e,t,n){var s,a,u,p,v,m,x,_;if(t.p==0){for(t.p=1,p=n,p||(a=new qe,u=(s=h(Vh(ta),9),new ll(s,h(vh(s,s.length),9),0)),p=new As(a,u)),h(p.a,15).Fc(t),t.k==(bn(),Pi)&&h(p.b,21).Fc(h(U(t,(He(),co)),61)),m=new Q(t.j);m.a<m.c.c.length;)for(v=h(K(m),11),_=gg(Cf(se(ie(Nf,1),st,20,0,[new j6(v),new y4(v)])));rr(_);)x=h(Un(_),11),Ppt(e,x.i,p);return p}return null}function T9(e,t){var n,s,a,u,p;if(e.Ab){if(e.Ab){if(p=e.Ab.i,p>0){if(a=h(e.Ab.g,1934),t==null){for(u=0;u<p;++u)if(n=a[u],n.d==null)return n}else for(u=0;u<p;++u)if(n=a[u],Nt(t,n.d))return n}}else if(t==null){for(s=new Tn(e.Ab);s.e!=s.i.gc();)if(n=h(On(s),590),n.d==null)return n}else for(s=new Tn(e.Ab);s.e!=s.i.gc();)if(n=h(On(s),590),Nt(t,n.d))return n}return null}function TDn(e,t){var n,s,a,u,p,v,m,x;if(x=lt(U(t,(Fb(),e7t))),x==null||(rn(x),x)){for(QTn(e,t),a=new qe,m=pr(t.b,0);m.b!=m.d.c;)p=h(gr(m),86),n=obe(e,p,null),n&&(Oa(n,t),a.c[a.c.length]=n);if(e.a=null,e.b=null,a.c.length>1)for(s=new Q(a);s.a<s.c.c.length;)for(n=h(K(s),135),u=0,v=pr(n.b,0);v.b!=v.d.c;)p=h(gr(v),86),p.g=u++;return a}return Ef(se(ie(NPn,1),Ewe,135,0,[t]))}function CDn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;P=gyn(e,Fge(t),a),kge(P,_d(a,p1)),w=null,j=a,V=HE(j,vmt),J=new RWe(P),Y9n(J.a,V),oe=HE(j,"endPoint"),fe=new zWe(P),q9n(fe.a,oe),le=wb(j,hj),we=new UWe(P),NEn(we.a,le),A=_d(a,Rye),u=new aJe(e,P),sgn(u.a,u.b,A),I=_d(a,Fye),p=new oJe(e,P),agn(p.a,p.b,I),x=wb(a,$ye),v=new cJe(n,P),w7n(v.b,v.a,x),_=wb(a,jye),m=new uJe(s,P),m7n(m.b,m.a,_)}function bve(e,t,n){var s,a,u,p,v;switch(v=null,t.g){case 1:for(a=new Q(e.j);a.a<a.c.c.length;)if(s=h(K(a),11),ft(lt(U(s,(He(),dne)))))return s;v=new So,Fe(v,(He(),dne),(on(),!0));break;case 2:for(p=new Q(e.j);p.a<p.c.c.length;)if(u=h(K(p),11),ft(lt(U(u,(He(),pne)))))return u;v=new So,Fe(v,(He(),pne),(on(),!0))}return v&&(Ka(v,e),Ji(v,n),m8n(v.n,e.o,n)),v}function Bpt(e,t){var n,s,a,u,p,v;for(v=-1,p=new di,s=new _f(e.b);Wa(s.a)||Wa(s.b);){for(n=h(Wa(s.a)?K(s.a):K(s.b),17),v=d.Math.max(v,Pe(Je(U(n,(tt(),D0))))),n.c==e?Ti(Jr(new Vt(null,new Wt(n.b,16)),new tFe),new uYe(p)):Ti(Jr(new Vt(null,new Wt(n.b,16)),new nFe),new lYe(p)),u=pr(p,0);u.b!=u.d.c;)a=h(gr(u),70),os(a,(He(),P5))||Fe(a,P5,n);Vi(t,p),Pl(p)}return v}function SDn(e,t,n,s,a){var u,p,v,m;u=new Nd(e),kp(u,(bn(),Vo)),Fe(u,(tt(),Ei),(_s(),qo)),Fe(u,(He(),Br),t.c.i),p=new So,Fe(p,Br,t.c),Ji(p,a),Ka(p,u),Fe(t.c,Zc,u),v=new Nd(e),kp(v,Vo),Fe(v,Ei,qo),Fe(v,Br,t.d.i),m=new So,Fe(m,Br,t.d),Ji(m,a),Ka(m,v),Fe(t.d,Zc,v),Gs(t,p),xs(t,m),fw(0,n.c.length),J7(n.c,0,u),s.c[s.c.length]=v,Fe(u,P$,Ke(1)),Fe(v,P$,Ke(1))}function Rb(e,t,n,s,a){var u,p,v,m,x;v=a?s.b:s.a,!wd(e.a,s)&&(x=v>n.s&&v<n.c,m=!1,n.e.b!=0&&n.j.b!=0&&(m=m|(d.Math.abs(v-Pe(Je(RY(n.e))))<l1&&d.Math.abs(v-Pe(Je(RY(n.j))))<l1),m=m|(d.Math.abs(v-Pe(Je(lP(n.e))))<l1&&d.Math.abs(v-Pe(Je(lP(n.j))))<l1)),(x||m)&&(p=h(U(t,(tt(),Da)),74),p||(p=new Oc,Fe(t,Da,p)),u=new _a(s),Ai(p,u,p.c.b,p.c),Qi(e.a,u)))}function ADn(e,t,n,s){var a,u,p,v,m,x,_;if(NTn(e,t,n,s))return!0;for(p=new Q(t.f);p.a<p.c.c.length;){switch(u=h(K(p),324),v=!1,m=e.j-t.j+n,x=m+t.o,_=e.k-t.k+s,a=_+t.p,u.a.g){case 0:v=rX(e,m+u.b.a,0,m+u.c.a,_-1);break;case 1:v=rX(e,x,_+u.b.a,e.o-1,_+u.c.a);break;case 2:v=rX(e,m+u.b.a,a,m+u.c.a,e.p-1);break;default:v=rX(e,0,_+u.b.a,m-1,_+u.c.a)}if(v)return!0}return!1}function MDn(e,t){var n,s,a,u,p,v,m,x,_;for(p=new Q(t.b);p.a<p.c.c.length;)for(u=h(K(p),29),x=new Q(u.a);x.a<x.c.c.length;){for(m=h(K(x),10),_=new qe,v=0,s=new Sn(Dn(za(m).a.Kc(),new L));rr(s);)n=h(Un(s),17),!(Xs(n)||!Xs(n)&&n.c.i.c==n.d.i.c)&&(a=h(U(n,(tt(),H5)),19).a,a>v&&(v=a,_.c=pe(yn,st,1,0,5,1)),a==v&&Ue(_,new As(n.c.i,n)));Rt(),hs(_,e.c),aw(e.b,m.p,_)}}function DDn(e,t){var n,s,a,u,p,v,m,x,_;for(p=new Q(t.b);p.a<p.c.c.length;)for(u=h(K(p),29),x=new Q(u.a);x.a<x.c.c.length;){for(m=h(K(x),10),_=new qe,v=0,s=new Sn(Dn(qi(m).a.Kc(),new L));rr(s);)n=h(Un(s),17),!(Xs(n)||!Xs(n)&&n.c.i.c==n.d.i.c)&&(a=h(U(n,(tt(),H5)),19).a,a>v&&(v=a,_.c=pe(yn,st,1,0,5,1)),a==v&&Ue(_,new As(n.d.i,n)));Rt(),hs(_,e.c),aw(e.f,m.p,_)}}function Fpt(e){J2(e,new Xp(K2(q2(W2(Y2(new mp,Yb),"ELK Box"),"Algorithm for packing of unconnected boxes, i.e. graphs without edges."),new uze))),et(e,Yb,Vb,Y8e),et(e,Yb,zw,15),et(e,Yb,UM,Ke(0)),et(e,Yb,oj,at(V8e)),et(e,Yb,Yy,at(MEt)),et(e,Yb,x5,at(DEt)),et(e,Yb,k5,Nwt),et(e,Yb,qM,at(U8e)),et(e,Yb,E5,at(q8e)),et(e,Yb,Sye,at(Lre)),et(e,Yb,tj,at(AEt))}function Rpt(e,t){var n,s,a,u,p,v,m,x,_;if(a=e.i,p=a.o.a,u=a.o.b,p<=0&&u<=0)return Qe(),to;switch(x=e.n.a,_=e.n.b,v=e.o.a,n=e.o.b,t.g){case 2:case 1:if(x<0)return Qe(),un;if(x+v>p)return Qe(),pn;break;case 4:case 3:if(_<0)return Qe(),sn;if(_+n>u)return Qe(),Fn}return m=(x+v/2)/p,s=(_+n/2)/u,m+s<=1&&m-s<=0?(Qe(),un):m+s>=1&&m-s>=0?(Qe(),pn):s<.5?(Qe(),sn):(Qe(),Fn)}function LDn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(n=!1,_=Pe(Je(U(t,(tt(),uv)))),j=Ig*_,a=new Q(t.b);a.a<a.c.c.length;)for(s=h(K(a),29),x=new Q(s.a),u=h(K(x),10),A=_de(e.a[u.p]);x.a<x.c.c.length;)v=h(K(x),10),I=_de(e.a[v.p]),A!=I&&(P=uy(e.b,u,v),p=u.n.b+u.o.b+u.d.a+A.a+P,m=v.n.b-v.d.d+I.a,p>m+j&&(V=A.g+I.g,I.a=(I.g*I.a+A.g*A.a)/V,I.g=V,A.f=I,n=!0)),u=v,A=I;return n}function jpt(e,t,n,s,a,u,p){var v,m,x,_,A,I;for(I=new O4,x=t.Kc();x.Ob();)for(v=h(x.Pb(),839),A=new Q(v.wf());A.a<A.c.c.length;)_=h(K(A),181),Me(_.We((Tr(),Bre)))===Me((Tf(),m8))&&(wpt(I,_,!1,s,a,u,p),v9(e,I));for(m=n.Kc();m.Ob();)for(v=h(m.Pb(),839),A=new Q(v.wf());A.a<A.c.c.length;)_=h(K(A),181),Me(_.We((Tr(),Bre)))===Me((Tf(),pm))&&(wpt(I,_,!0,s,a,u,p),v9(e,I))}function IDn(e,t,n){var s,a,u,p,v,m,x;for(p=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));p.e!=p.i.gc();)for(u=h(On(p),33),a=new Sn(Dn(Pd(u).a.Kc(),new L));rr(a);)s=h(Un(a),79),!y9(s)&&!y9(s)&&!Pb(s)&&(m=h(no(Ia(n.f,u)),86),x=h(_n(n,Na(h(he((!s.c&&(s.c=new qt(Pn,s,5,8)),s.c),0),82))),86),!!m&&!!x&&(v=new Wde(m,x),Fe(v,(fo(),U_),s),Oa(v,s),vr(m.d,v),vr(x.b,v),vr(t.a,v)))}function ODn(e,t){var n,s,a,u,p,v,m,x;for(m=h(h(jr(e.r,t),21),84).Kc();m.Ob();)v=h(m.Pb(),111),a=v.c?Int(v.c):0,a>0?v.a?(x=v.b.rf().b,a>x&&(e.v||v.c.d.c.length==1?(p=(a-x)/2,v.d.d=p,v.d.a=p):(n=h(ut(v.c.d,0),181).rf().b,s=(n-x)/2,v.d.d=d.Math.max(0,s),v.d.a=a-s-x))):v.d.a=e.t+a:DE(e.u)&&(u=V2e(v.b),u.d<0&&(v.d.d=-u.d),u.d+u.a>v.b.rf().b&&(v.d.a=u.d+u.a-v.b.rf().b))}function NDn(e,t){var n;switch(SA(e)){case 6:return ws(t);case 7:return ew(t);case 8:return Jv(t);case 3:return Array.isArray(t)&&(n=SA(t),!(n>=14&&n<=16));case 11:return t!=null&&typeof t===fZ;case 12:return t!=null&&(typeof t===LM||typeof t==fZ);case 0:return PX(t,e.__elementTypeId$);case 2:return mW(t)&&t.im!==de;case 1:return mW(t)&&t.im!==de||PX(t,e.__elementTypeId$);default:return!0}}function $pt(e,t){var n,s,a,u;return s=d.Math.min(d.Math.abs(e.c-(t.c+t.b)),d.Math.abs(e.c+e.b-t.c)),u=d.Math.min(d.Math.abs(e.d-(t.d+t.a)),d.Math.abs(e.d+e.a-t.d)),n=d.Math.abs(e.c+e.b/2-(t.c+t.b/2)),n>e.b/2+t.b/2||(a=d.Math.abs(e.d+e.a/2-(t.d+t.a/2)),a>e.a/2+t.a/2)?1:n==0&&a==0?0:n==0?u/a+1:a==0?s/n+1:d.Math.min(s/n,u/a)+1}function Hpt(e,t){var n,s,a,u,p,v;return a=hge(e),v=hge(t),a==v?e.e==t.e&&e.a<54&&t.a<54?e.f<t.f?-1:e.f>t.f?1:0:(s=e.e-t.e,n=(e.d>0?e.d:d.Math.floor((e.a-1)*Zbt)+1)-(t.d>0?t.d:d.Math.floor((t.a-1)*Zbt)+1),n>s+1?a:n<s-1?-a:(u=(!e.c&&(e.c=UA(e.f)),e.c),p=(!t.c&&(t.c=UA(t.f)),t.c),s<0?u=vy(u,u2t(-s)):s>0&&(p=vy(p,u2t(s))),mft(u,p))):a<v?-1:1}function PDn(e,t){var n,s,a,u,p,v,m;for(u=0,v=0,m=0,a=new Q(e.f.e);a.a<a.c.c.length;)s=h(K(a),144),t!=s&&(p=e.i[t.b][s.b],u+=p,n=yg(t.d,s.d),n>0&&e.d!=(UE(),Cte)&&(v+=p*(s.d.a+e.a[t.b][s.b]*(t.d.a-s.d.a)/n)),n>0&&e.d!=(UE(),_te)&&(m+=p*(s.d.b+e.a[t.b][s.b]*(t.d.b-s.d.b)/n)));switch(e.d.g){case 1:return new dt(v/u,t.d.b);case 2:return new dt(t.d.a,m/u);default:return new dt(v/u,m/u)}}function zpt(e,t){Uk();var n,s,a,u,p;if(p=h(U(e.i,(tt(),Ei)),98),u=e.j.g-t.j.g,u!=0||!(p==(_s(),x2)||p==ed||p==qo))return 0;if(p==(_s(),x2)&&(n=h(U(e,L0),19),s=h(U(t,L0),19),!!n&&!!s&&(a=n.a-s.a,a!=0)))return a;switch(e.j.g){case 1:return Ui(e.n.a,t.n.a);case 2:return Ui(e.n.b,t.n.b);case 3:return Ui(t.n.a,e.n.a);case 4:return Ui(t.n.b,e.n.b);default:throw ne(new ja(Bwe))}}function Gpt(e){var t,n,s,a,u,p;for(n=(!e.a&&(e.a=new Gi(Zl,e,5)),e.a).i+2,p=new Wo(n),Ue(p,new dt(e.j,e.k)),Ti(new Vt(null,(!e.a&&(e.a=new Gi(Zl,e,5)),new Wt(e.a,16))),new PWe(p)),Ue(p,new dt(e.b,e.c)),t=1;t<p.c.length-1;)s=(Kt(t-1,p.c.length),h(p.c[t-1],8)),a=(Kt(t,p.c.length),h(p.c[t],8)),u=(Kt(t+1,p.c.length),h(p.c[t+1],8)),s.a==a.a&&a.a==u.a||s.b==a.b&&a.b==u.b?f0(p,t):++t;return p}function Vpt(e,t){var n,s,a,u,p,v,m;for(n=qet(iZe(nZe(rZe(new Qle,t),new eB(t.e)),H5t),e.a),t.j.c.length==0||oct(h(ut(t.j,0),57).a,n),m=new xq,Pr(e.e,n,m),p=new ns,v=new ns,u=new Q(t.k);u.a<u.c.c.length;)a=h(K(u),17),Qi(p,a.c),Qi(v,a.d);s=p.a.gc()-v.a.gc(),s<0?(HA(m,!0,(la(),Wl)),HA(m,!1,Dh)):s>0&&(HA(m,!1,(la(),Wl)),HA(m,!0,Dh)),mc(t.g,new IZe(e,n)),Pr(e.g,t,n)}function Upt(){Upt=re;var e;for(S3e=se(ie(Hn,1),hr,25,15,[-1,-1,30,19,15,13,11,11,10,9,9,8,8,8,8,7,7,7,7,7,7,7,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5]),nte=pe(Hn,hr,25,37,15,1),g3t=se(ie(Hn,1),hr,25,15,[-1,-1,63,40,32,28,25,23,21,20,19,19,18,18,17,17,16,16,16,15,15,15,15,14,14,14,14,14,14,13,13,13,13,13,13,13,13]),A3e=pe(ap,RZ,25,37,14,1),e=2;e<=36;e++)nte[e]=Li(d.Math.pow(e,S3e[e])),A3e[e]=oM(_R,nte[e])}function BDn(e){var t;if((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i!=1)throw ne(new an(tmt+(!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i));return t=new Oc,jA(h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82))&&Qs(t,Sbt(e,jA(h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82)),!1)),jA(h(he((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c),0),82))&&Qs(t,Sbt(e,jA(h(he((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c),0),82)),!0)),t}function qpt(e,t){var n,s,a,u,p;for(t.d?a=e.a.c==(r1(),lv)?za(t.b):qi(t.b):a=e.a.c==(r1(),I0)?za(t.b):qi(t.b),u=!1,s=new Sn(Dn(a.a.Kc(),new L));rr(s);)if(n=h(Un(s),17),p=ft(e.a.f[e.a.g[t.b.p].p]),!(!p&&!Xs(n)&&n.c.i.c==n.d.i.c)&&!(ft(e.a.n[e.a.g[t.b.p].p])||ft(e.a.n[e.a.g[t.b.p].p]))&&(u=!0,wd(e.b,e.a.g[q8n(n,t.b).p])))return t.c=!0,t.a=n,t;return t.c=u,t.a=null,t}function FDn(e,t,n,s,a){var u,p,v,m,x,_,A;for(Rt(),hs(e,new yze),v=new Ms(e,0),A=new qe,u=0;v.b<v.d.gc();)p=(kn(v.b<v.d.gc()),h(v.d.Xb(v.c=v.b++),157)),A.c.length!=0&&Fc(p)*Su(p)>u*2?(_=new jB(A),x=Fc(p)/Su(p),m=cZ(_,t,new E4,n,s,a,x),zr(Uh(_.e),m),A.c=pe(yn,st,1,0,5,1),u=0,A.c[A.c.length]=_,A.c[A.c.length]=p,u=Fc(_)*Su(_)+Fc(p)*Su(p)):(A.c[A.c.length]=p,u+=Fc(p)*Su(p));return A}function vve(e,t,n){var s,a,u,p,v,m,x;if(s=n.gc(),s==0)return!1;if(e.ej())if(m=e.fj(),Ipe(e,t,n),p=s==1?e.Zi(3,null,n.Kc().Pb(),t,m):e.Zi(5,null,n,t,m),e.bj()){for(v=s<100?null:new ig(s),u=t+s,a=t;a<u;++a)x=e.Oi(a),v=e.cj(x,v),v=v;v?(v.Ei(p),v.Fi()):e.$i(p)}else e.$i(p);else if(Ipe(e,t,n),e.bj()){for(v=s<100?null:new ig(s),u=t+s,a=t;a<u;++a)v=e.cj(e.Oi(a),v);v&&v.Fi()}return!0}function Ypt(e,t,n){var s,a,u,p,v;return e.ej()?(a=null,u=e.fj(),s=e.Zi(1,v=(p=e.Ui(t,e.oi(t,n)),p),n,t,u),e.bj()&&!(e.ni()&&!!v?Nr(v,n):Me(v)===Me(n))&&(v&&(a=e.dj(v,a)),a=e.cj(n,a)),a?(a.Ei(s),a.Fi()):e.$i(s),v):(v=(p=e.Ui(t,e.oi(t,n)),p),e.bj()&&!(e.ni()&&!!v?Nr(v,n):Me(v)===Me(n))&&(a=null,v&&(a=e.dj(v,null)),a=e.cj(n,a),a&&a.Fi()),v)}function wve(e,t){var n,s,a,u,p,v,m,x,_;if(e.e=t,e.f=h(U(t,(kg(),Jj)),230),y9n(t),e.d=d.Math.max(t.e.c.length*16+t.c.c.length,256),!ft(lt(U(t,(Jh(),L4e)))))for(_=e.e.e.c.length,m=new Q(t.e);m.a<m.c.c.length;)v=h(K(m),144),x=v.d,x.a=lde(e.f)*_,x.b=lde(e.f)*_;for(n=t.b,u=new Q(t.c);u.a<u.c.c.length;)if(a=h(K(u),282),s=h(U(a,B4e),19).a,s>0){for(p=0;p<s;p++)Ue(n,new Ort(a));n0t(a)}}function d5(e,t){var n,s,a,u,p,v;if(e.k==(bn(),Ou)&&(n=U7(Jr(h(U(e,(He(),DD)),15).Oc(),new I7(new WFe))).sd((Tp(),Ux))?t:(Yu(),rL),Fe(e,F5,n),n!=(Yu(),Kg)))for(s=h(U(e,Br),17),v=Pe(Je(U(s,(tt(),D0)))),p=0,n==Z1?p=e.o.b-d.Math.ceil(v/2):n==rL&&(e.o.b-=Pe(Je(U(Us(e),z5))),p=(e.o.b-d.Math.ceil(v))/2),u=new Q(e.j);u.a<u.c.c.length;)a=h(K(u),11),a.n.b=p}function mve(){mve=re,qq(),M_t=new WVe,se(ie(X5,2),Re,368,0,[se(ie(X5,1),Mj,592,0,[new vS(Pyt)])]),se(ie(X5,2),Re,368,0,[se(ie(X5,1),Mj,592,0,[new vS(o3e)])]),se(ie(X5,2),Re,368,0,[se(ie(X5,1),Mj,592,0,[new vS(Byt)]),se(ie(X5,1),Mj,592,0,[new vS(o3e)])]),new hg("-1"),se(ie(X5,2),Re,368,0,[se(ie(X5,1),Mj,592,0,[new vS("\\c+")])]),new hg("0"),new hg("0"),new hg("1"),new hg("0"),new hg(zyt)}function hR(e){var t,n;return!!e.c&&e.c.kh()&&(n=h(e.c,49),e.c=h(Tg(e,n),138),e.c!=n&&((e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,9,2,n,e.c)),ue(e.Cb,399)?e.Db>>16==-15&&e.Cb.nh()&&_K(new yK(e.Cb,9,13,n,e.c,y0(cu(h(e.Cb,59)),e))):ue(e.Cb,88)&&e.Db>>16==-23&&e.Cb.nh()&&(t=e.c,ue(t,88)||(t=(Bt(),nh)),ue(n,88)||(n=(Bt(),nh)),_K(new yK(e.Cb,9,10,n,t,y0(Co(h(e.Cb,26)),e)))))),e.c}function RDn(e,t){var n,s,a,u,p,v,m,x,_,A;for(Bn(t,"Hypernodes processing",1),a=new Q(e.b);a.a<a.c.c.length;)for(s=h(K(a),29),v=new Q(s.a);v.a<v.c.c.length;)if(p=h(K(v),10),ft(lt(U(p,(tt(),q$))))&&p.j.c.length<=2){for(A=0,_=0,n=0,u=0,x=new Q(p.j);x.a<x.c.c.length;)switch(m=h(K(x),11),m.j.g){case 1:++A;break;case 2:++_;break;case 3:++n;break;case 4:++u}A==0&&n==0&&HNn(e,p,u<=_)}An(t)}function jDn(e,t){var n,s,a,u,p,v,m,x,_;for(Bn(t,"Layer constraint edge reversal",1),p=new Q(e.b);p.a<p.c.c.length;){for(u=h(K(p),29),_=-1,n=new qe,x=hA(u.a),a=0;a<x.length;a++)s=h(U(x[a],(He(),h2)),303),_==-1?s!=(Ad(),l3)&&(_=a):s==(Ad(),l3)&&(Ca(x[a],null),Sw(x[a],_++,u)),s==(Ad(),O5)&&Ue(n,x[a]);for(m=new Q(n);m.a<m.c.c.length;)v=h(K(m),10),Ca(v,null),Ca(v,u)}An(t)}function $Dn(e,t,n){var s,a,u,p,v,m,x,_,A;for(Bn(n,"Hyperedge merging",1),gCn(e,t),m=new Ms(t.b,0);m.b<m.d.gc();)if(v=(kn(m.b<m.d.gc()),h(m.d.Xb(m.c=m.b++),29)),_=v.a,_.c.length!=0)for(s=null,a=null,u=null,p=null,x=0;x<_.c.length;x++)s=(Kt(x,_.c.length),h(_.c[x],10)),a=s.k,a==(bn(),ds)&&p==ds&&(A=NLn(s,u),A.a&&(GAn(s,u,A.b,A.c),Kt(x,_.c.length),Mhe(_.c,x,1),--x,s=u,a=p)),u=s,p=a;An(n)}function HDn(e,t){var n,s,a;s=hu(e.d,1)!=0,!ft(lt(U(t.j,(He(),nv))))&&!ft(lt(U(t.j,f3)))||Me(U(t.j,(tt(),Vg)))===Me((Dd(),Ug))?t.c.Tf(t.e,s):s=ft(lt(U(t.j,nv))),kM(e,t,s,!0),ft(lt(U(t.j,f3)))&&Fe(t.j,f3,(on(),!1)),ft(lt(U(t.j,nv)))&&(Fe(t.j,nv,(on(),!1)),Fe(t.j,f3,!0)),n=vpt(e,t);do{if(Ege(e),n==0)return 0;s=!s,a=n,kM(e,t,s,!1),n=vpt(e,t)}while(a>n);return a}function Wpt(e,t){var n,s,a;s=hu(e.d,1)!=0,!ft(lt(U(t.j,(He(),nv))))&&!ft(lt(U(t.j,f3)))||Me(U(t.j,(tt(),Vg)))===Me((Dd(),Ug))?t.c.Tf(t.e,s):s=ft(lt(U(t.j,nv))),kM(e,t,s,!0),ft(lt(U(t.j,f3)))&&Fe(t.j,f3,(on(),!1)),ft(lt(U(t.j,nv)))&&(Fe(t.j,nv,(on(),!1)),Fe(t.j,f3,!0)),n=bQ(e,t);do{if(Ege(e),n==0)return 0;s=!s,a=n,kM(e,t,s,!1),n=bQ(e,t)}while(a>n);return a}function Kpt(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j;if(t==n)return!0;if(t=Sbe(e,t),n=Sbe(e,n),s=zX(t),s){if(_=zX(n),_!=s)return _?(m=s.Dj(),j=_.Dj(),m==j&&m!=null):!1;if(p=(!t.d&&(t.d=new Gi(pa,t,1)),t.d),u=p.i,I=(!n.d&&(n.d=new Gi(pa,n,1)),n.d),u==I.i){for(x=0;x<u;++x)if(a=h(he(p,x),87),A=h(he(I,x),87),!Kpt(e,a,A))return!1}return!0}else return v=t.e,P=n.e,v==P}function Xpt(e,t,n,s){var a,u,p,v,m,x,_,A;if(Bd(e.e,t)){for(A=ac(e.e.Tg(),t),u=h(e.g,119),_=null,m=-1,v=-1,a=0,x=0;x<e.i;++x)p=u[x],A.rl(p.ak())&&(a==n&&(m=x),a==s&&(v=x,_=p.dd()),++a);if(m==-1)throw ne(new Ea(Tee+n+s2+a));if(v==-1)throw ne(new Ea(Cee+s+s2+a));return a9(e,m,v),Tu(e.e)&&$6(e,wg(e,7,t,Ke(s),_,n,!0)),_}else throw ne(new an("The feature must be many-valued to support move"))}function Qpt(e,t,n,s){var a,u,p,v,m;switch(m=new _a(t.n),m.a+=t.o.a/2,m.b+=t.o.b/2,v=Pe(Je(U(t,(tt(),om)))),u=e.f,p=e.d,a=e.c,h(U(t,(He(),co)),61).g){case 1:m.a+=p.b+a.a-n/2,m.b=-s-v,t.n.b=-(p.d+v+a.b);break;case 2:m.a=u.a+p.b+p.c+v,m.b+=p.d+a.b-s/2,t.n.a=u.a+p.c+v-a.a;break;case 3:m.a+=p.b+a.a-n/2,m.b=u.b+p.d+p.a+v,t.n.b=u.b+p.a+v-a.b;break;case 4:m.a=-n-v,m.b+=p.d+a.b-s/2,t.n.a=-(p.b+v+a.a)}return m}function Zpt(e){var t,n,s,a,u,p;return s=new dge,Oa(s,e),Me(U(s,(tt(),Ju)))===Me((la(),Q1))&&Fe(s,Ju,bF(s)),U(s,(xB(),X_))==null&&(p=h(ddt(e),160),Fe(s,X_,KN(p.We(X_)))),Fe(s,(He(),Br),e),Fe(s,Uo,(t=h(Vh(ane),9),new ll(t,h(vh(t,t.length),9),0))),a=MOn((bi(e)&&(Qv(),new Yv(bi(e))),Qv(),new PY(bi(e)?new Yv(bi(e)):null,e)),Dh),u=h(U(s,Cke),116),n=s.d,sst(n,u),sst(n,a),s}function zDn(e,t,n){var s,a;s=t.c.i,a=n.d.i,s.k==(bn(),ds)?(Fe(e,(He(),rf),h(U(s,rf),11)),Fe(e,Yl,h(U(s,Yl),11)),Fe(e,tm,lt(U(s,tm)))):s.k==Ou?(Fe(e,(He(),rf),h(U(s,rf),11)),Fe(e,Yl,h(U(s,Yl),11)),Fe(e,tm,(on(),!0))):a.k==Ou?(Fe(e,(He(),rf),h(U(a,rf),11)),Fe(e,Yl,h(U(a,Yl),11)),Fe(e,tm,(on(),!0))):(Fe(e,(He(),rf),t.c),Fe(e,Yl,n.d))}function GDn(e){var t,n,s,a,u,p,v;for(e.o=new Jm,s=new di,p=new Q(e.e.a);p.a<p.c.c.length;)u=h(K(p),121),Ty(u).c.length==1&&Ai(s,u,s.c.b,s.c);for(;s.b!=0;)u=h(s.b==0?null:(kn(s.b!=0),pl(s,s.a.a)),121),Ty(u).c.length!=0&&(t=h(ut(Ty(u),0),213),n=u.g.a.c.length>0,v=mF(t,u),Wfe(n?v.b:v.g,t),Ty(v).c.length==1&&Ai(s,v,s.c.b,s.c),a=new As(u,t),mg(e.o,a),vc(e.e.a,u))}function Jpt(e,t){var n,s,a,u,p,v,m;return s=d.Math.abs(FP(e.b).a-FP(t.b).a),v=d.Math.abs(FP(e.b).b-FP(t.b).b),a=0,m=0,n=1,p=1,s>e.b.b/2+t.b.b/2&&(a=d.Math.min(d.Math.abs(e.b.c-(t.b.c+t.b.b)),d.Math.abs(e.b.c+e.b.b-t.b.c)),n=1-a/s),v>e.b.a/2+t.b.a/2&&(m=d.Math.min(d.Math.abs(e.b.d-(t.b.d+t.b.a)),d.Math.abs(e.b.d+e.b.a-t.b.d)),p=1-m/v),u=d.Math.min(n,p),(1-u)*d.Math.sqrt(s*s+v*v)}function VDn(e){var t,n,s,a;for(oZ(e,e.e,e.f,(gb(),Yg),!0,e.c,e.i),oZ(e,e.e,e.f,Yg,!1,e.c,e.i),oZ(e,e.e,e.f,m3,!0,e.c,e.i),oZ(e,e.e,e.f,m3,!1,e.c,e.i),qDn(e,e.c,e.e,e.f,e.i),s=new Ms(e.i,0);s.b<s.d.gc();)for(t=(kn(s.b<s.d.gc()),h(s.d.Xb(s.c=s.b++),128)),a=new Ms(e.i,s.b);a.b<a.d.gc();)n=(kn(a.b<a.d.gc()),h(a.d.Xb(a.c=a.b++),128)),_In(t,n);qNn(e.i,h(U(e.d,(He(),R5)),230)),oOn(e.i)}function zQ(e,t){var n,s;if(t!=null){if(s=Nb(e),s)if((s.i&1)!=0){if(s==vu)return Jv(t);if(s==Hn)return ue(t,19);if(s==Em)return ue(t,155);if(s==Vc)return ue(t,217);if(s==Cl)return ue(t,172);if(s==ks)return ew(t);if(s==D3)return ue(t,184);if(s==ap)return ue(t,162)}else return SN(),n=h(_n(KH,s),55),!n||n.wj(t);else if(ue(t,56))return e.uk(h(t,56))}return!1}function yve(){yve=re;var e,t,n,s,a,u,p,v,m;for(Gf=pe(Vc,Zy,25,255,15,1),H0=pe(Cl,o1,25,64,15,1),t=0;t<255;t++)Gf[t]=-1;for(n=90;n>=65;n--)Gf[n]=n-65<<24>>24;for(s=122;s>=97;s--)Gf[s]=s-97+26<<24>>24;for(a=57;a>=48;a--)Gf[a]=a-48+52<<24>>24;for(Gf[43]=62,Gf[47]=63,u=0;u<=25;u++)H0[u]=65+u&Oi;for(p=26,m=0;p<=51;++p,m++)H0[p]=97+m&Oi;for(e=52,v=0;e<=61;++e,v++)H0[e]=48+v&Oi;H0[62]=43,H0[63]=47}function UDn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P;if(e.dc())return new Rs;for(x=0,A=0,a=e.Kc();a.Ob();)s=h(a.Pb(),37),u=s.f,x=d.Math.max(x,u.a),A+=u.a*u.b;for(x=d.Math.max(x,d.Math.sqrt(A)*Pe(Je(U(h(e.Kc().Pb(),37),(tt(),j$))))),I=0,P=0,m=0,n=t,v=e.Kc();v.Ob();)p=h(v.Pb(),37),_=p.f,I+_.a>x&&(I=0,P+=m+t,m=0),_9(p,I,P),n=d.Math.max(n,I+_.a),m=d.Math.max(m,_.b),I+=_.a+t;return new dt(n+t,P+m+t)}function qDn(e,t,n,s,a){var u,p,v,m,x,_,A;for(p=new Q(t);p.a<p.c.c.length;){if(u=h(K(p),17),m=u.c,n.a._b(m))x=(gb(),Yg);else if(s.a._b(m))x=(gb(),m3);else throw ne(new an("Source port must be in one of the port sets."));if(_=u.d,n.a._b(_))A=(gb(),Yg);else if(s.a._b(_))A=(gb(),m3);else throw ne(new an("Target port must be in one of the port sets."));v=new Udt(u,x,A),Pr(e.b,u,v),a.c[a.c.length]=v}}function kve(e,t){var n,s,a,u,p,v,m;if(!mf(e))throw ne(new ja(emt));if(s=mf(e),u=s.g,a=s.f,u<=0&&a<=0)return Qe(),to;switch(v=e.i,m=e.j,t.g){case 2:case 1:if(v<0)return Qe(),un;if(v+e.g>u)return Qe(),pn;break;case 4:case 3:if(m<0)return Qe(),sn;if(m+e.f>a)return Qe(),Fn}return p=(v+e.g/2)/u,n=(m+e.f/2)/a,p+n<=1&&p-n<=0?(Qe(),un):p+n>=1&&p-n>=0?(Qe(),pn):n<.5?(Qe(),sn):(Qe(),Fn)}function YDn(e,t,n,s,a){var u,p;if(u=Vs(Zi(t[0],fa),Zi(s[0],fa)),e[0]=Gn(u),u=dg(u,32),n>=a){for(p=1;p<a;p++)u=Vs(u,Vs(Zi(t[p],fa),Zi(s[p],fa))),e[p]=Gn(u),u=dg(u,32);for(;p<n;p++)u=Vs(u,Zi(t[p],fa)),e[p]=Gn(u),u=dg(u,32)}else{for(p=1;p<n;p++)u=Vs(u,Vs(Zi(t[p],fa),Zi(s[p],fa))),e[p]=Gn(u),u=dg(u,32);for(;p<a;p++)u=Vs(u,Zi(s[p],fa)),e[p]=Gn(u),u=dg(u,32)}mo(u,0)!=0&&(e[p]=Gn(u))}function Bw(e){Mr();var t,n,s,a,u,p;if(e.e!=4&&e.e!=5)throw ne(new an("Token#complementRanges(): must be RANGE: "+e.e));for(u=e,Ny(u),D9(u),s=u.b.length+2,u.b[0]==0&&(s-=2),n=u.b[u.b.length-1],n==jx&&(s-=2),a=new zu(4),a.b=pe(Hn,hr,25,s,15,1),p=0,u.b[0]>0&&(a.b[p++]=0,a.b[p++]=u.b[0]-1),t=1;t<u.b.length-2;t+=2)a.b[p++]=u.b[t]+1,a.b[p++]=u.b[t+1]-1;return n!=jx&&(a.b[p++]=n+1,a.b[p]=jx),a.a=!0,a}function GQ(e,t,n){var s,a,u,p,v,m,x,_;if(s=n.gc(),s==0)return!1;if(e.ej())if(x=e.fj(),oQ(e,t,n),p=s==1?e.Zi(3,null,n.Kc().Pb(),t,x):e.Zi(5,null,n,t,x),e.bj()){for(v=s<100?null:new ig(s),u=t+s,a=t;a<u;++a)_=e.g[a],v=e.cj(_,v),v=e.jj(_,v);v?(v.Ei(p),v.Fi()):e.$i(p)}else e.$i(p);else if(oQ(e,t,n),e.bj()){for(v=s<100?null:new ig(s),u=t+s,a=t;a<u;++a)m=e.g[a],v=e.cj(m,v);v&&v.Fi()}return!0}function xve(e,t,n,s){var a,u,p,v,m;for(p=new Q(e.k);p.a<p.c.c.length;)a=h(K(p),129),(!s||a.c==(qh(),qg))&&(m=a.b,m.g<0&&a.d>0&&(wq(m,m.d-a.d),a.c==(qh(),qg)&&Vhn(m,m.a-a.d),m.d<=0&&m.i>0&&Ai(t,m,t.c.b,t.c)));for(u=new Q(e.f);u.a<u.c.c.length;)a=h(K(u),129),(!s||a.c==(qh(),qg))&&(v=a.a,v.g<0&&a.d>0&&(KO(v,v.i-a.d),a.c==(qh(),qg)&&Uhn(v,v.b-a.d),v.i<=0&&v.d>0&&Ai(n,v,n.c.b,n.c)))}function WDn(e,t,n){var s,a,u,p,v,m,x,_;for(Bn(n,"Processor compute fanout",1),Kc(e.b),Kc(e.a),v=null,u=pr(t.b,0);!v&&u.b!=u.d.c;)x=h(gr(u),86),ft(lt(U(x,(fo(),dm))))&&(v=x);for(m=new di,Ai(m,v,m.c.b,m.c),lbt(e,m),_=pr(t.b,0);_.b!=_.d.c;)x=h(gr(_),86),p=Jn(U(x,(fo(),V_))),a=Fo(e.b,p)!=null?h(Fo(e.b,p),19).a:0,Fe(x,fH,Ke(a)),s=1+(Fo(e.a,p)!=null?h(Fo(e.a,p),19).a:0),Fe(x,G8t,Ke(s));An(n)}function KDn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j;for(I=IEn(e,n),m=0;m<t;m++){for(iw(a,n),P=new qe,j=(kn(s.b<s.d.gc()),h(s.d.Xb(s.c=s.b++),407)),_=I+m;_<e.b;_++)v=j,j=(kn(s.b<s.d.gc()),h(s.d.Xb(s.c=s.b++),407)),Ue(P,new Agt(v,j,n));for(A=I+m;A<e.b;A++)kn(s.b>0),s.a.Xb(s.c=--s.b),A>I+m&&Mu(s);for(p=new Q(P);p.a<p.c.c.length;)u=h(K(p),407),iw(s,u);if(m<t-1)for(x=I+m;x<e.b;x++)kn(s.b>0),s.a.Xb(s.c=--s.b)}}function XDn(){Mr();var e,t,n,s,a,u;if(hie)return hie;for(e=new zu(4),Fw(e,Lg(Gee,!0)),I9(e,Lg("M",!0)),I9(e,Lg("C",!0)),u=new zu(4),s=0;s<11;s++)$o(u,s,s);return t=new zu(4),Fw(t,Lg("M",!0)),$o(t,4448,4607),$o(t,65438,65439),a=new bE(2),Qp(a,e),Qp(a,wT),n=new bE(2),n.$l(DP(u,Lg("L",!0))),n.$l(t),n=new dw(3,n),n=new Lde(a,n),hie=n,hie}function QDn(e){var t,n;if(t=Jn(pt(e,(Tr(),Q_))),!plt(t,e)&&!Pp(e,v8)&&((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a).i!=0||ft(lt(pt(e,JD)))))if(t==null||Mw(t).length==0){if(!plt(wn,e))throw n=or(or(new ju("Unable to load default layout algorithm "),wn)," for unconfigured node "),wR(e,n),ne(new ey(n.a))}else throw n=or(or(new ju("Layout algorithm '"),t),"' not found for "),wR(e,n),ne(new ey(n.a))}function VQ(e){var t,n,s,a,u,p,v,m,x,_,A,I,P;if(n=e.i,t=e.n,e.b==0)for(P=n.c+t.b,I=n.b-t.b-t.c,p=e.a,m=0,_=p.length;m<_;++m)a=p[m],IP(a,P,I);else s=Qft(e,!1),IP(e.a[0],n.c+t.b,s[0]),IP(e.a[2],n.c+n.b-t.c-s[2],s[2]),A=n.b-t.b-t.c,s[0]>0&&(A-=s[0]+e.c,s[0]+=e.c),s[2]>0&&(A-=s[2]+e.c),s[1]=d.Math.max(s[1],A),IP(e.a[1],n.c+t.b+s[0]-(s[1]-A)/2,s[1]);for(u=e.a,v=0,x=u.length;v<x;++v)a=u[v],ue(a,326)&&h(a,326).Te()}function ZDn(e){var t,n,s,a,u,p,v,m,x,_,A;for(A=new z$e,A.d=0,p=new Q(e.b);p.a<p.c.c.length;)u=h(K(p),29),A.d+=u.a.c.length;for(s=0,a=0,A.a=pe(Hn,hr,25,e.b.c.length,15,1),x=0,_=0,A.e=pe(Hn,hr,25,A.d,15,1),n=new Q(e.b);n.a<n.c.c.length;)for(t=h(K(n),29),t.p=s++,A.a[t.p]=a++,_=0,m=new Q(t.a);m.a<m.c.c.length;)v=h(K(m),10),v.p=x++,A.e[v.p]=_++;return A.c=new mWe(A),A.b=I1(A.d),MDn(A,e),A.f=I1(A.d),DDn(A,e),A}function e2t(e,t){var n,s,a,u;for(u=h(ut(e.n,e.n.c.length-1),211).d,e.p=d.Math.min(e.p,t.g),e.r=d.Math.max(e.r,u),e.g=d.Math.max(e.g,t.g+(e.b.c.length==1?0:e.i)),e.o=d.Math.min(e.o,t.f),e.e+=t.f+(e.b.c.length==1?0:e.i),e.f=d.Math.max(e.f,t.f),a=e.n.c.length>0?(e.n.c.length-1)*e.i:0,s=new Q(e.n);s.a<s.c.c.length;)n=h(K(s),211),a+=n.a;e.d=a,e.a=e.e/e.b.c.length-e.i*((e.b.c.length-1)/e.b.c.length),u2e(e.j)}function t2t(e,t){var n,s,a,u,p,v,m,x,_,A;if(_=lt(U(t,(Jh(),K4t))),_==null||(rn(_),_)){for(A=pe(vu,Y1,25,t.e.c.length,16,1),p=VTn(t),a=new di,x=new Q(t.e);x.a<x.c.c.length;)v=h(K(x),144),n=jbe(e,v,null,null,A,p),n&&(Oa(n,t),Ai(a,n,a.c.b,a.c));if(a.b>1)for(s=pr(a,0);s.b!=s.d.c;)for(n=h(gr(s),231),u=0,m=new Q(n.e);m.a<m.c.c.length;)v=h(K(m),144),v.b=u++;return a}return Ef(se(ie(SPn,1),Ewe,231,0,[t]))}function s1(e){var t,n,s,a,u,p,v;if(!e.g){if(v=new hq,t=fT,p=t.a.zc(e,t),p==null){for(s=new Tn(La(e));s.e!=s.i.gc();)n=h(On(s),26),mi(v,s1(n));t.a.Bc(e)!=null,t.a.gc()==0}for(a=v.i,u=(!e.s&&(e.s=new Ye(Ac,e,21,17)),new Tn(e.s));u.e!=u.i.gc();++a)Jhn(h(On(u),449),a);mi(v,(!e.s&&(e.s=new Ye(Ac,e,21,17)),e.s)),kw(v),e.g=new Vlt(e,v),e.i=h(v.g,247),e.i==null&&(e.i=iie),e.p=null,ou(e).b&=-5}return e.g}function UQ(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j;if(s=e.i,n=e.n,e.b==0)t=Xft(e,!1),OP(e.a[0],s.d+n.d,t[0]),OP(e.a[2],s.d+s.a-n.a-t[2],t[2]),I=s.a-n.d-n.a,A=I,t[0]>0&&(t[0]+=e.c,A-=t[0]),t[2]>0&&(A-=t[2]+e.c),t[1]=d.Math.max(t[1],A),OP(e.a[1],s.d+n.d+t[0]-(t[1]-A)/2,t[1]);else for(j=s.d+n.d,P=s.a-n.d-n.a,p=e.a,m=0,_=p.length;m<_;++m)a=p[m],OP(a,j,P);for(u=e.a,v=0,x=u.length;v<x;++v)a=u[v],ue(a,326)&&h(a,326).Ue()}function JDn(e){var t,n,s,a,u,p,v,m,x,_;for(_=pe(Hn,hr,25,e.b.c.length+1,15,1),x=new ns,s=0,u=new Q(e.b);u.a<u.c.c.length;){for(a=h(K(u),29),_[s++]=x.a.gc(),m=new Q(a.a);m.a<m.c.c.length;)for(p=h(K(m),10),n=new Sn(Dn(qi(p).a.Kc(),new L));rr(n);)t=h(Un(n),17),x.a.zc(t,x);for(v=new Q(a.a);v.a<v.c.c.length;)for(p=h(K(v),10),n=new Sn(Dn(za(p).a.Kc(),new L));rr(n);)t=h(Un(n),17),x.a.Bc(t)!=null}return _}function fR(e,t,n,s){var a,u,p,v,m;if(m=ac(e.e.Tg(),t),a=h(e.g,119),ia(),h(t,66).Oj()){for(p=0;p<e.i;++p)if(u=a[p],m.rl(u.ak())&&Nr(u,n))return!0}else if(n!=null){for(v=0;v<e.i;++v)if(u=a[v],m.rl(u.ak())&&Nr(n,u.dd()))return!0;if(s){for(p=0;p<e.i;++p)if(u=a[p],m.rl(u.ak())&&Me(n)===Me($Y(e,h(u.dd(),56))))return!0}}else for(p=0;p<e.i;++p)if(u=a[p],m.rl(u.ak())&&u.dd()==null)return!1;return!1}function n2t(e,t,n,s){var a,u,p,v,m,x;if(x=ac(e.e.Tg(),t),p=h(e.g,119),Bd(e.e,t)){if(t.hi()&&(u=px(e,t,s,ue(t,99)&&(h(t,18).Bb&ea)!=0),u>=0&&u!=n))throw ne(new an(nD));for(a=0,m=0;m<e.i;++m)if(v=p[m],x.rl(v.ak())){if(a==n)return h(Sy(e,m,(ia(),h(t,66).Oj()?h(s,72):R1(t,s))),72);++a}throw ne(new Ea(n_+n+s2+a))}else{for(m=0;m<e.i;++m)if(v=p[m],x.rl(v.ak()))return ia(),h(t,66).Oj()?v:v.dd();return null}}function r2t(e,t,n,s){var a,u,p,v;for(v=n,p=new Q(t.a);p.a<p.c.c.length;){if(u=h(K(p),221),a=h(u.b,65),Mb(e.b.c,a.b.c+a.b.b)<=0&&Mb(a.b.c,e.b.c+e.b.b)<=0&&Mb(e.b.d,a.b.d+a.b.a)<=0&&Mb(a.b.d,e.b.d+e.b.a)<=0){if(Mb(a.b.c,e.b.c+e.b.b)==0&&s.a<0||Mb(a.b.c+a.b.b,e.b.c)==0&&s.a>0||Mb(a.b.d,e.b.d+e.b.a)==0&&s.b<0||Mb(a.b.d+a.b.a,e.b.d)==0&&s.b>0){v=0;break}}else v=d.Math.min(v,r0t(e,a,s));v=d.Math.min(v,r2t(e,u,v,s))}return v}function CM(e,t){var n,s,a,u,p,v,m;if(e.b<2)throw ne(new an("The vector chain must contain at least a source and a target point."));for(a=(kn(e.b!=0),h(e.a.a.c,8)),eP(t,a.a,a.b),m=new N4((!t.a&&(t.a=new Gi(Zl,t,5)),t.a)),p=pr(e,1);p.a<e.b-1;)v=h(gr(p),8),m.e!=m.i.gc()?n=h(On(m),469):(n=(X2(),s=new zO,s),ift(m,n)),kfe(n,v.a,v.b);for(;m.e!=m.i.gc();)On(m),p9(m);u=(kn(e.b!=0),h(e.c.b.c,8)),JN(t,u.a,u.b)}function i2t(e,t){var n,s,a,u,p,v,m,x,_;for(n=0,a=new Q((Kt(0,e.c.length),h(e.c[0],101)).g.b.j);a.a<a.c.c.length;)s=h(K(a),11),s.p=n++;for(t==(Qe(),sn)?hs(e,new zje):hs(e,new Gje),v=0,_=e.c.length-1;v<_;)p=(Kt(v,e.c.length),h(e.c[v],101)),x=(Kt(_,e.c.length),h(e.c[_],101)),u=t==sn?p.c:p.a,m=t==sn?x.a:x.c,l0(p,t,(Sf(),Qx),u),l0(x,t,Xx,m),++v,--_;v==_&&l0((Kt(v,e.c.length),h(e.c[v],101)),t,(Sf(),L5),null)}function eLn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;return A=e.a.i+e.a.g/2,I=e.a.i+e.a.g/2,j=t.i+t.g/2,J=t.j+t.f/2,v=new dt(j,J),x=h(pt(t,(Tr(),q5)),8),x.a=x.a+A,x.b=x.b+I,u=(v.b-x.b)/(v.a-x.a),s=v.b-u*v.a,V=n.i+n.g/2,oe=n.j+n.f/2,m=new dt(V,oe),_=h(pt(n,q5),8),_.a=_.a+A,_.b=_.b+I,p=(m.b-_.b)/(m.a-_.a),a=m.b-p*m.a,P=(s-a)/(p-u),x.a<P&&v.a<P||P<x.a&&P<v.a?!1:!(_.a<P&&m.a<P||P<_.a&&P<m.a)}function tLn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P;if(I=h(_n(e.c,t),183),!I)throw ne(new Xf("Edge did not exist in input."));return x=Jk(I),u=wS((!t.a&&(t.a=new Ye(gi,t,6,6)),t.a)),v=!u,v&&(P=new yp,n=new lnt(e,x,P),adn((!t.a&&(t.a=new Ye(gi,t,6,6)),t.a),n),Wh(I,Bye,P)),a=Pp(t,(Tr(),x3)),a&&(_=h(pt(t,x3),74),p=!_||Krt(_),m=!p,m&&(A=new yp,s=new tKe(A),Os(_,s),Wh(I,"junctionPoints",A))),ok(I,"container",uA(t).k),null}function Eve(e,t,n){var s,a,u,p,v,m,x,_;this.a=e,this.b=t,this.c=n,this.e=Ef(se(ie(_Pn,1),st,168,0,[new M4(e,t),new M4(t,n),new M4(n,e)])),this.f=Ef(se(ie(cs,1),Re,8,0,[e,t,n])),this.d=(s=ms(ro(this.b),this.a),a=ms(ro(this.c),this.a),u=ms(ro(this.c),this.b),p=s.a*(this.a.a+this.b.a)+s.b*(this.a.b+this.b.b),v=a.a*(this.a.a+this.c.a)+a.b*(this.a.b+this.c.b),m=2*(s.a*u.b-s.b*u.a),x=(a.b*p-s.b*v)/m,_=(s.a*v-a.a*p)/m,new dt(x,_))}function s2t(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j;if(I=new uw(e.p),Wh(t,_r,I),n&&!(e.f?sA(e.f):null).a.dc())for(_=new yp,Wh(t,"logs",_),v=0,j=new F6((e.f?sA(e.f):null).b.Kc());j.b.Ob();)P=Jn(j.b.Pb()),A=new uw(P),bw(_,v),iK(_,v,A),++v;if(s&&(x=new WO(e.q),Wh(t,"executionTime",x)),!sA(e.a).a.dc())for(p=new yp,Wh(t,xee,p),v=0,u=new F6(sA(e.a).b.Kc());u.b.Ob();)a=h(u.b.Pb(),1949),m=new _4,bw(p,v),iK(p,v,m),s2t(a,m,n,s),++v}function jb(e,t){var n,s,a,u,p,v;for(u=e.c,p=e.d,Gs(e,null),xs(e,null),t&&ft(lt(U(p,(He(),dne))))?Gs(e,bve(p.i,(ua(),ec),(Qe(),pn))):Gs(e,p),t&&ft(lt(U(u,(He(),pne))))?xs(e,bve(u.i,(ua(),Jc),(Qe(),un))):xs(e,u),s=new Q(e.b);s.a<s.c.c.length;)n=h(K(s),70),a=h(U(n,(tt(),y1)),272),a==(Tf(),m8)?Fe(n,y1,pm):a==pm&&Fe(n,y1,m8);v=ft(lt(U(e,(He(),Ff)))),Fe(e,Ff,(on(),!v)),e.a=GA(e.a)}function nLn(e,t,n){var s,a,u,p,v,m;for(s=0,u=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));u.e!=u.i.gc();)a=h(On(u),33),p="",(!a.n&&(a.n=new Ye(Ua,a,1,7)),a.n).i==0||(p=h(he((!a.n&&(a.n=new Ye(Ua,a,1,7)),a.n),0),137).a),v=new Hrt(p),Oa(v,a),Fe(v,(kg(),D5),a),v.b=s++,v.d.a=a.i+a.g/2,v.d.b=a.j+a.f/2,v.e.a=d.Math.max(a.g,1),v.e.b=d.Math.max(a.f,1),Ue(t.e,v),sc(n.f,a,v),m=h(pt(a,(Jh(),P4e)),98),m==(_s(),Qg)&&(m=$f)}function rLn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;n=ib(new _p,e.f),x=e.i[t.c.i.p],P=e.i[t.d.i.p],m=t.c,I=t.d,v=m.a.b,A=I.a.b,x.b||(v+=m.n.b),P.b||(A+=I.n.b),_=Li(d.Math.max(0,v-A)),p=Li(d.Math.max(0,A-v)),j=(V=d.Math.max(1,h(U(t,(tt(),H5)),19).a),J=m0e(t.c.i.k,t.d.i.k),V*J),a=Th(ph(gh(dh(bh(new Il,j),p),n),h(_n(e.k,t.c),121))),u=Th(ph(gh(dh(bh(new Il,j),_),n),h(_n(e.k,t.d),121))),s=new HZe(a,u),e.c[t.p]=s}function iLn(e,t,n,s){var a,u,p,v,m,x;for(p=new I2t(e,t,n),m=new Ms(s,0),a=!1;m.b<m.d.gc();)v=(kn(m.b<m.d.gc()),h(m.d.Xb(m.c=m.b++),233)),v==t||v==n?Mu(m):!a&&Pe(wf(v.g,v.d[0]).a)>Pe(wf(p.g,p.d[0]).a)?(kn(m.b>0),m.a.Xb(m.c=--m.b),iw(m,p),a=!0):!!v.e&&v.e.gc()>0&&(u=(!v.e&&(v.e=new qe),v.e).Mc(t),x=(!v.e&&(v.e=new qe),v.e).Mc(n),(u||x)&&((!v.e&&(v.e=new qe),v.e).Fc(p),++p.c));a||(s.c[s.c.length]=p)}function a2t(e){var t,n,s;if(oy(h(U(e,(tt(),Ei)),98)))for(n=new Q(e.j);n.a<n.c.c.length;)t=h(K(n),11),t.j==(Qe(),to)&&(s=h(U(t,(He(),Zc)),10),s?Ji(t,h(U(s,co),61)):t.e.c.length-t.g.c.length<0?Ji(t,pn):Ji(t,un));else{for(n=new Q(e.j);n.a<n.c.c.length;)t=h(K(n),11),s=h(U(t,(He(),Zc)),10),s?Ji(t,h(U(s,co),61)):t.e.c.length-t.g.c.length<0?Ji(t,(Qe(),pn)):Ji(t,(Qe(),un));Fe(e,Ei,(_s(),k8))}}function SM(e){var t,n,s;switch(e){case 91:case 93:case 45:case 94:case 44:case 92:s="\\"+String.fromCharCode(e&Oi);break;case 12:s="\\f";break;case 10:s="\\n";break;case 13:s="\\r";break;case 9:s="\\t";break;case 27:s="\\e";break;default:e<32?(n=(t=e>>>0,"0"+t.toString(16)),s="\\x"+$u(n,n.length-2,n.length)):e>=ea?(n=(t=e>>>0,"0"+t.toString(16)),s="\\v"+$u(n,n.length-6,n.length)):s=""+String.fromCharCode(e&Oi)}return s}function qQ(e,t){var n,s,a,u,p,v,m,x,_,A;if(p=e.e,m=t.e,m==0)return e;if(p==0)return t.e==0?t:new fy(-t.e,t.d,t.a);if(u=e.d,v=t.d,u+v==2)return n=Zi(e.a[0],fa),s=Zi(t.a[0],fa),p<0&&(n=Tk(n)),m<0&&(s=Tk(s)),eM(Cg(n,s));if(a=u!=v?u>v?1:-1:epe(e.a,t.a,u),a==-1)A=-m,_=p==m?gK(t.a,v,e.a,u):bK(t.a,v,e.a,u);else if(A=p,p==m){if(a==0)return Dg(),Gx;_=gK(e.a,u,t.a,v)}else _=bK(e.a,u,t.a,v);return x=new fy(A,_.length,_),jE(x),x}function _ve(e){var t,n,s,a,u,p;for(this.e=new qe,this.a=new qe,n=e.b-1;n<3;n++)rk(e,0,h(Zh(e,0),8));if(e.b<4)throw ne(new an("At (least dimension + 1) control points are necessary!"));for(this.b=3,this.d=!0,this.c=!1,GCn(this,e.b+this.b-1),p=new qe,u=new Q(this.e),t=0;t<this.b-1;t++)Ue(p,Je(K(u)));for(a=pr(e,0);a.b!=a.d.c;)s=h(gr(a),8),Ue(p,Je(K(u))),Ue(this.a,new Yit(s,p)),Kt(0,p.c.length),p.c.splice(0,1)}function o2t(e,t){var n,s,a,u,p,v,m,x,_;for(u=new Q(e.b);u.a<u.c.c.length;)for(a=h(K(u),29),v=new Q(a.a);v.a<v.c.c.length;)for(p=h(K(v),10),p.k==(bn(),Ou)&&(m=(x=h(Un(new Sn(Dn(za(p).a.Kc(),new L))),17),_=h(Un(new Sn(Dn(qi(p).a.Kc(),new L))),17),!ft(lt(U(x,(He(),Ff))))||!ft(lt(U(_,Ff)))?t:Blt(t)),d5(p,m)),s=new Sn(Dn(qi(p).a.Kc(),new L));rr(s);)n=h(Un(s),17),m=ft(lt(U(n,(He(),Ff))))?Blt(t):t,Mlt(n,m)}function sLn(e,t,n,s,a){var u,p,v;if(n.f>=t.o&&n.f<=t.f||t.a*.5<=n.f&&t.a*1.5>=n.f){if(p=h(ut(t.n,t.n.c.length-1),211),p.e+p.d+n.g+a<=s&&(u=h(ut(t.n,t.n.c.length-1),211),u.f-e.f+n.f<=e.b||e.a.c.length==1))return Epe(t,n),!0;if(t.s+n.g<=s&&(t.t+t.d+n.f+a<=e.b||e.a.c.length==1))return Ue(t.b,n),v=h(ut(t.n,t.n.c.length-1),211),Ue(t.n,new sB(t.s,v.f+v.a+t.i,t.i)),s2e(h(ut(t.n,t.n.c.length-1),211),n),e2t(t,n),!0}return!1}function c2t(e,t,n){var s,a,u,p;return e.ej()?(a=null,u=e.fj(),s=e.Zi(1,p=CK(e,t,n),n,t,u),e.bj()&&!(e.ni()&&p!=null?Nr(p,n):Me(p)===Me(n))?(p!=null&&(a=e.dj(p,a)),a=e.cj(n,a),e.ij()&&(a=e.lj(p,n,a)),a?(a.Ei(s),a.Fi()):e.$i(s)):(e.ij()&&(a=e.lj(p,n,a)),a?(a.Ei(s),a.Fi()):e.$i(s)),p):(p=CK(e,t,n),e.bj()&&!(e.ni()&&p!=null?Nr(p,n):Me(p)===Me(n))&&(a=null,p!=null&&(a=e.dj(p,null)),a=e.cj(n,a),a&&a.Fi()),p)}function C9(e,t){var n,s,a,u,p,v,m,x;t%=24,e.q.getHours()!=t&&(s=new d.Date(e.q.getTime()),s.setDate(s.getDate()+1),v=e.q.getTimezoneOffset()-s.getTimezoneOffset(),v>0&&(m=v/60|0,x=v%60,a=e.q.getDate(),n=e.q.getHours(),n+m>=24&&++a,u=new d.Date(e.q.getFullYear(),e.q.getMonth(),a,t+m,e.q.getMinutes()+x,e.q.getSeconds(),e.q.getMilliseconds()),e.q.setTime(u.getTime()))),p=e.q.getTime(),e.q.setTime(p+36e5),e.q.getHours()!=t&&e.q.setTime(p)}function aLn(e,t){var n,s,a,u,p;if(Bn(t,"Path-Like Graph Wrapping",1),e.b.c.length==0){An(t);return}if(a=new Lbe(e),p=(a.i==null&&(a.i=_ge(a,new Vue)),Pe(a.i)*a.f),n=p/(a.i==null&&(a.i=_ge(a,new Vue)),Pe(a.i)),a.b>n){An(t);return}switch(h(U(e,(tt(),Pne)),337).g){case 2:u=new que;break;case 0:u=new Gue;break;default:u=new Yue}if(s=u.Vf(e,a),!u.Wf())switch(h(U(e,J$),338).g){case 2:s=i0t(a,s);break;case 1:s=rdt(a,s)}rIn(e,a,s),An(t)}function oLn(e,t){var n,s,a,u;if(Uwn(e.d,e.e),e.c.a.$b(),Pe(Je(U(t.j,(tt(),z$))))!=0||Pe(Je(U(t.j,z$)))!=0)for(n=Ax,Me(U(t.j,Vg))!==Me((Dd(),Ug))&&Fe(t.j,(He(),nv),(on(),!0)),u=h(U(t.j,O_),19).a,a=0;a<u&&(s=HDn(e,t),!(s<n&&(n=s,Wut(e),n==0)));a++);else for(n=Ir,Me(U(t.j,Vg))!==Me((Dd(),Ug))&&Fe(t.j,(He(),nv),(on(),!0)),u=h(U(t.j,O_),19).a,a=0;a<u&&(s=Wpt(e,t),!(s<n&&(n=s,Wut(e),n==0)));a++);}function cLn(e,t){var n,s,a,u,p,v,m,x;for(p=new qe,v=0,n=0,m=0;v<t.c.length-1&&n<e.gc();){for(s=h(e.Xb(n),19).a+m;(Kt(v+1,t.c.length),h(t.c[v+1],19)).a<s;)++v;for(x=0,u=s-(Kt(v,t.c.length),h(t.c[v],19)).a,a=(Kt(v+1,t.c.length),h(t.c[v+1],19)).a-s,u>a&&++x,Ue(p,(Kt(v+x,t.c.length),h(t.c[v+x],19))),m+=(Kt(v+x,t.c.length),h(t.c[v+x],19)).a-s,++n;n<e.gc()&&h(e.Xb(n),19).a+m<=(Kt(v+x,t.c.length),h(t.c[v+x],19)).a;)++n;v+=1+x}return p}function YQ(e){var t,n,s,a,u,p,v;if(!e.d){if(v=new zze,t=fT,u=t.a.zc(e,t),u==null){for(s=new Tn(La(e));s.e!=s.i.gc();)n=h(On(s),26),mi(v,YQ(n));t.a.Bc(e)!=null,t.a.gc()==0}for(p=v.i,a=(!e.q&&(e.q=new Ye(eh,e,11,10)),new Tn(e.q));a.e!=a.i.gc();++p)h(On(a),399);mi(v,(!e.q&&(e.q=new Ye(eh,e,11,10)),e.q)),kw(v),e.d=new ay((h(he(De((bg(),Xt).o),9),18),v.i),v.g),e.e=h(v.g,673),e.e==null&&(e.e=X9t),ou(e).b&=-17}return e.d}function px(e,t,n,s){var a,u,p,v,m,x;if(x=ac(e.e.Tg(),t),m=0,a=h(e.g,119),ia(),h(t,66).Oj()){for(p=0;p<e.i;++p)if(u=a[p],x.rl(u.ak())){if(Nr(u,n))return m;++m}}else if(n!=null){for(v=0;v<e.i;++v)if(u=a[v],x.rl(u.ak())){if(Nr(n,u.dd()))return m;++m}if(s){for(m=0,p=0;p<e.i;++p)if(u=a[p],x.rl(u.ak())){if(Me(n)===Me($Y(e,h(u.dd(),56))))return m;++m}}}else for(p=0;p<e.i;++p)if(u=a[p],x.rl(u.ak())){if(u.dd()==null)return m;++m}return-1}function uLn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P;for(Rt(),hs(e,new wze),p=ZS(e),P=new qe,I=new qe,v=null,m=0;p.b!=0;)u=h(p.b==0?null:(kn(p.b!=0),pl(p,p.a.a)),157),!v||Fc(v)*Su(v)/2<Fc(u)*Su(u)?(v=u,P.c[P.c.length]=u):(m+=Fc(u)*Su(u),I.c[I.c.length]=u,I.c.length>1&&(m>Fc(v)*Su(v)/2||p.b==0)&&(A=new jB(I),_=Fc(v)/Su(v),x=cZ(A,t,new E4,n,s,a,_),zr(Uh(A.e),x),v=A,P.c[P.c.length]=A,m=0,I.c=pe(yn,st,1,0,5,1)));return Vi(P,I),P}function lLn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V;if(n.mh(t)&&(_=(P=t,P?h(s,49).xh(P):null),_))if(V=n.bh(t,e.a),j=t.t,j>1||j==-1)if(A=h(V,69),I=h(_,69),A.dc())I.$b();else for(p=!!aa(t),u=0,v=e.a?A.Kc():A.Zh();v.Ob();)x=h(v.Pb(),56),a=h(mb(e,x),56),a?(p?(m=I.Xc(a),m==-1?I.Xh(u,a):u!=m&&I.ji(u,a)):I.Xh(u,a),++u):e.b&&!p&&(I.Xh(u,x),++u);else V==null?_.Wb(null):(a=mb(e,V),a==null?e.b&&!aa(t)&&_.Wb(V):_.Wb(a))}function hLn(e,t){var n,s,a,u,p,v,m,x;for(n=new EFe,a=new Sn(Dn(za(t).a.Kc(),new L));rr(a);)if(s=h(Un(a),17),!Xs(s)&&(v=s.c.i,H2e(v,n$))){if(x=nve(e,v,n$,t$),x==-1)continue;n.b=d.Math.max(n.b,x),!n.a&&(n.a=new qe),Ue(n.a,v)}for(p=new Sn(Dn(qi(t).a.Kc(),new L));rr(p);)if(u=h(Un(p),17),!Xs(u)&&(m=u.d.i,H2e(m,t$))){if(x=nve(e,m,t$,n$),x==-1)continue;n.d=d.Math.max(n.d,x),!n.c&&(n.c=new qe),Ue(n.c,m)}return n}function u2t(e){cx();var t,n,s,a;if(t=Li(e),e<d_.length)return d_[t];if(e<=50)return vM((Dg(),ste),t);if(e<=_0)return K4(vM(M5[1],t),t);if(e>1e6)throw ne(new cN("power of ten too big"));if(e<=Ir)return K4(vM(M5[1],t),t);for(s=vM(M5[1],Ir),a=s,n=xc(e-Ir),t=Li(e%Ir);mo(n,Ir)>0;)a=vy(a,s),n=Cg(n,Ir);for(a=vy(a,vM(M5[1],t)),a=K4(a,Ir),n=xc(e-Ir);mo(n,Ir)>0;)a=K4(a,Ir),n=Cg(n,Ir);return a=K4(a,t),a}function fLn(e,t){var n,s,a,u,p,v,m,x,_;for(Bn(t,"Hierarchical port dummy size processing",1),m=new qe,_=new qe,s=Pe(Je(U(e,(tt(),p3)))),n=s*2,u=new Q(e.b);u.a<u.c.c.length;){for(a=h(K(u),29),m.c=pe(yn,st,1,0,5,1),_.c=pe(yn,st,1,0,5,1),v=new Q(a.a);v.a<v.c.c.length;)p=h(K(v),10),p.k==(bn(),Pi)&&(x=h(U(p,(He(),co)),61),x==(Qe(),sn)?m.c[m.c.length]=p:x==Fn&&(_.c[_.c.length]=p));P0t(m,!0,n),P0t(_,!1,n)}An(t)}function dLn(e,t){var n,s,a,u,p,v,m;Bn(t,"Layer constraint postprocessing",1),m=e.b,m.c.length!=0&&(s=(Kt(0,m.c.length),h(m.c[0],29)),p=h(ut(m,m.c.length-1),29),n=new Nl(e),u=new Nl(e),QMn(e,s,p,n,u),n.a.c.length==0||(fw(0,m.c.length),J7(m.c,0,n)),u.a.c.length==0||(m.c[m.c.length]=u)),os(e,(He(),fne))&&(a=new Nl(e),v=new Nl(e),QAn(e,a,v),a.a.c.length==0||(fw(0,m.c.length),J7(m.c,0,a)),v.a.c.length==0||(m.c[m.c.length]=v)),An(t)}function l2t(e){var t,n,s,a,u,p,v,m,x,_;for(m=new Q(e.a);m.a<m.c.c.length;)if(v=h(K(m),10),v.k==(bn(),Pi)&&(a=h(U(v,(He(),co)),61),a==(Qe(),pn)||a==un))for(s=new Sn(Dn(Id(v).a.Kc(),new L));rr(s);)n=h(Un(s),17),t=n.a,t.b!=0&&(x=n.c,x.i==v&&(u=(kn(t.b!=0),h(t.a.a.c,8)),u.b=Qa(se(ie(cs,1),Re,8,0,[x.i.n,x.n,x.a])).b),_=n.d,_.i==v&&(p=(kn(t.b!=0),h(t.c.b.c,8)),p.b=Qa(se(ie(cs,1),Re,8,0,[_.i.n,_.n,_.a])).b))}function gLn(e,t){var n,s,a,u,p,v,m;for(Bn(t,"Sort By Input Model "+U(e,(tt(),Vg)),1),a=0,s=new Q(e.b);s.a<s.c.c.length;){for(n=h(K(s),29),m=a==0?0:a-1,v=h(ut(e.b,m),29),p=new Q(n.a);p.a<p.c.c.length;)u=h(K(p),10),Me(U(u,Ei))!==Me((_s(),x2))&&Me(U(u,Ei))!==Me(qo)&&(Rt(),hs(u.j,new Act(v,C1t(u))),Sp(t,"Node "+u+" ports: "+u.j));Rt(),hs(n.a,new Qut(v,h(U(e,Vg),339),h(U(e,nke),378))),Sp(t,"Layer "+a+": "+n),++a}An(t)}function pLn(e,t){var n,s,a,u;if(u=Zpt(t),Ti(new Vt(null,(!t.c&&(t.c=new Ye(bu,t,9,9)),new Wt(t.c,16))),new tYe(u)),a=h(U(u,(He(),Uo)),21),bOn(t,a),a.Hc((ha(),El)))for(s=new Tn((!t.c&&(t.c=new Ye(bu,t,9,9)),t.c));s.e!=s.i.gc();)n=h(On(s),118),YOn(e,t,u,n);return h(pt(t,(tt(),g2)),174).gc()!=0&&Vgt(t,u),ft(lt(U(u,Ake)))&&a.Fc(O$),os(u,OD)&&SXe(new Mpe(Pe(Je(U(u,OD)))),u),Me(pt(t,sm))===Me((Ld(),B0))?aPn(e,t,u):SNn(e,t,u),u}function S9(e,t,n,s){var a,u,p;if(this.j=new qe,this.k=new qe,this.b=new qe,this.c=new qe,this.e=new O4,this.i=new Oc,this.f=new xq,this.d=new qe,this.g=new qe,Ue(this.b,e),Ue(this.b,t),this.e.c=d.Math.min(e.a,t.a),this.e.d=d.Math.min(e.b,t.b),this.e.b=d.Math.abs(e.a-t.a),this.e.a=d.Math.abs(e.b-t.b),a=h(U(s,(tt(),Da)),74),a)for(p=pr(a,0);p.b!=p.d.c;)u=h(gr(p),8),ige(u.a,e.a)&&vr(this.i,u);n&&Ue(this.j,n),Ue(this.k,s)}function bLn(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(_=new RP(new Zqe(n)),v=pe(vu,Y1,25,e.f.e.c.length,16,1),Tde(v,v.length),n[t.b]=0,x=new Q(e.f.e);x.a<x.c.c.length;)m=h(K(x),144),m.b!=t.b&&(n[m.b]=Ir),xk(sx(_,m));for(;_.b.c.length!=0;)for(A=h(MK(_),144),v[A.b]=!0,u=Vet(new Vq(e.b,A),0);u.c;)a=h(n0e(u),282),I=U8n(a,A),!v[I.b]&&(os(a,(sM(),e$))?p=Pe(Je(U(a,e$))):p=e.c,s=n[A.b]+p,s<n[I.b]&&(n[I.b]=s,Hut(_,I),xk(sx(_,I))))}function h2t(e,t,n){var s,a,u,p,v,m,x,_,A;for(a=!0,p=new Q(e.b);p.a<p.c.c.length;){for(u=h(K(p),29),x=Fi,_=null,m=new Q(u.a);m.a<m.c.c.length;)if(v=h(K(m),10),A=Pe(t.p[v.p])+Pe(t.d[v.p])-v.d.d,s=Pe(t.p[v.p])+Pe(t.d[v.p])+v.o.b+v.d.a,A>x&&s>x)_=v,x=Pe(t.p[v.p])+Pe(t.d[v.p])+v.o.b+v.d.a;else{a=!1,n.n&&Sp(n,"bk node placement breaks on "+v+" which should have been after "+_);break}if(!a)break}return n.n&&Sp(n,t+" is feasible: "+a),a}function vLn(e,t,n,s){var a,u,p,v,m,x,_;for(v=-1,_=new Q(e);_.a<_.c.c.length;)x=h(K(_),112),x.g=v--,a=Gn(fB(YP(Jr(new Vt(null,new Wt(x.f,16)),new U$e),new q$e)).d),u=Gn(fB(YP(Jr(new Vt(null,new Wt(x.k,16)),new Y$e),new W$e)).d),p=a,m=u,s||(p=Gn(fB(YP(new Vt(null,new Wt(x.f,16)),new K$e)).d),m=Gn(fB(YP(new Vt(null,new Wt(x.k,16)),new X$e)).d)),x.d=p,x.a=a,x.i=m,x.b=u,m==0?Ai(n,x,n.c.b,n.c):p==0&&Ai(t,x,t.c.b,t.c)}function wLn(e,t,n,s){var a,u,p,v,m,x,_;if(n.d.i!=t.i){for(a=new Nd(e),kp(a,(bn(),ds)),Fe(a,(He(),Br),n),Fe(a,(tt(),Ei),(_s(),qo)),s.c[s.c.length]=a,p=new So,Ka(p,a),Ji(p,(Qe(),un)),v=new So,Ka(v,a),Ji(v,pn),_=n.d,xs(n,p),u=new db,Oa(u,n),Fe(u,Da,null),Gs(u,v),xs(u,_),x=new Ms(n.b,0);x.b<x.d.gc();)m=(kn(x.b<x.d.gc()),h(x.d.Xb(x.c=x.b++),70)),Me(U(m,y1))===Me((Tf(),pm))&&(Fe(m,P5,n),Mu(x),Ue(u.b,m));V0t(a,p,v)}}function mLn(e,t,n,s){var a,u,p,v,m,x,_;if(n.c.i!=t.i)for(a=new Nd(e),kp(a,(bn(),ds)),Fe(a,(He(),Br),n),Fe(a,(tt(),Ei),(_s(),qo)),s.c[s.c.length]=a,p=new So,Ka(p,a),Ji(p,(Qe(),un)),v=new So,Ka(v,a),Ji(v,pn),xs(n,p),u=new db,Oa(u,n),Fe(u,Da,null),Gs(u,v),xs(u,t),V0t(a,p,v),x=new Ms(n.b,0);x.b<x.d.gc();)m=(kn(x.b<x.d.gc()),h(x.d.Xb(x.c=x.b++),70)),_=h(U(m,y1),272),_==(Tf(),pm)&&(os(m,P5)||Fe(m,P5,n),Mu(x),Ue(u.b,m))}function yLn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(I=new qe,oe=bB(s),J=t*e.a,A=0,j=0,u=new ns,p=new ns,v=new qe,fe=0,le=0,P=0,V=0,x=0,_=0;oe.a.gc()!=0;)m=_kn(oe,a,p),m&&(oe.a.Bc(m)!=null,v.c[v.c.length]=m,u.a.zc(m,u),j=e.f[m.p],fe+=e.e[m.p]-j*e.b,A=e.c[m.p],le+=A*e.b,_+=j*e.b,V+=e.e[m.p]),(!m||oe.a.gc()==0||fe>=J&&e.e[m.p]>j*e.b||le>=n*J)&&(I.c[I.c.length]=v,v=new qe,Qs(p,u),u.a.$b(),x-=_,P=d.Math.max(P,x*e.b+V),x+=le,fe=le,le=0,_=0,V=0);return new As(P,I)}function kLn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P;for(n=(x=new vd(e.c.b).a.vc().Kc(),new qv(x));n.a.Ob();)t=(v=h(n.a.Pb(),42),h(v.dd(),149)),a=t.a,a==null&&(a=""),s=Y0n(e.c,a),!s&&a.length==0&&(s=W6n(e)),!!s&&!Ew(s.c,t,!1)&&vr(s.c,t);for(p=pr(e.a,0);p.b!=p.d.c;)u=h(gr(p),478),_=lK(e.c,u.a),P=lK(e.c,u.b),!!_&&!!P&&vr(_.c,new As(P,u.c));for(Pl(e.a),I=pr(e.b,0);I.b!=I.d.c;)A=h(gr(I),478),t=q0n(e.c,A.a),m=lK(e.c,A.b),!!t&&!!m&&B1n(t,m,A.c);Pl(e.b)}function xLn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P;u=new aS(e),p=new M1t,a=(gA(p.g),gA(p.j),Kc(p.b),gA(p.d),gA(p.i),Kc(p.k),Kc(p.c),Kc(p.e),P=f0t(p,u,null),tgt(p,u),P),t&&(x=new aS(t),v=ILn(x),J2e(a,se(ie(R8e,1),st,527,0,[v]))),I=!1,A=!1,n&&(x=new aS(n),gj in x.a&&(I=Ed(x,gj).ge().a),Tmt in x.a&&(A=Ed(x,Tmt).ge().a)),_=UXe(xut(new H6,I),A),p_n(new $He,a,_),gj in u.a&&Wh(u,gj,null),(I||A)&&(m=new _4,s2t(_,m,I,A),Wh(u,gj,m)),s=new ZWe(p),u6n(new xfe(a),s)}function ELn(e,t,n){var s,a,u,p,v,m,x,_,A;for(p=new N1t,x=se(ie(Hn,1),hr,25,15,[0]),a=-1,u=0,s=0,m=0;m<e.b.c.length;++m)if(_=h(ut(e.b,m),434),_.b>0){if(a<0&&_.a&&(a=m,u=x[0],s=0),a>=0){if(v=_.b,m==a&&(v-=s++,v==0))return 0;if(!fbt(t,x,_,v,p)){m=a-1,x[0]=u;continue}}else if(a=-1,!fbt(t,x,_,0,p))return 0}else{if(a=-1,Is(_.c,0)==32){if(A=x[0],Oct(t,x),x[0]>A)continue}else if(rwn(t,_.c,x[0])){x[0]+=_.c.length;continue}return 0}return rNn(p,n)?x[0]:0}function A9(e){var t,n,s,a,u,p,v,m;if(!e.f){if(m=new ele,v=new ele,t=fT,p=t.a.zc(e,t),p==null){for(u=new Tn(La(e));u.e!=u.i.gc();)a=h(On(u),26),mi(m,A9(a));t.a.Bc(e)!=null,t.a.gc()==0}for(s=(!e.s&&(e.s=new Ye(Ac,e,21,17)),new Tn(e.s));s.e!=s.i.gc();)n=h(On(s),170),ue(n,99)&&qn(v,h(n,18));kw(v),e.r=new Tnt(e,(h(he(De((bg(),Xt).o),6),18),v.i),v.g),mi(m,e.r),kw(m),e.f=new ay((h(he(De(Xt.o),5),18),m.i),m.g),ou(e).b&=-3}return e.f}function _Ln(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j;for(p=e.o,s=pe(Hn,hr,25,p,15,1),a=pe(Hn,hr,25,p,15,1),n=e.p,t=pe(Hn,hr,25,n,15,1),u=pe(Hn,hr,25,n,15,1),x=0;x<p;x++){for(A=0;A<n&&!Ay(e,x,A);)++A;s[x]=A}for(_=0;_<p;_++){for(A=n-1;A>=0&&!Ay(e,_,A);)--A;a[_]=A}for(P=0;P<n;P++){for(v=0;v<p&&!Ay(e,v,P);)++v;t[P]=v}for(j=0;j<n;j++){for(v=p-1;v>=0&&!Ay(e,v,j);)--v;u[j]=v}for(m=0;m<p;m++)for(I=0;I<n;I++)m<u[I]&&m>t[I]&&I<a[m]&&I>s[m]&&uR(e,m,I,!1,!0)}function Tve(e){var t,n,s,a,u,p,v,m;n=ft(lt(U(e,(Jh(),z4t)))),u=e.a.c.d,v=e.a.d.d,n?(p=Jf(ms(new dt(v.a,v.b),u),.5),m=Jf(ro(e.e),.5),t=ms(zr(new dt(u.a,u.b),p),m),t1e(e.d,t)):(a=Pe(Je(U(e.a,X4t))),s=e.d,u.a>=v.a?u.b>=v.b?(s.a=v.a+(u.a-v.a)/2+a,s.b=v.b+(u.b-v.b)/2-a-e.e.b):(s.a=v.a+(u.a-v.a)/2+a,s.b=u.b+(v.b-u.b)/2+a):u.b>=v.b?(s.a=u.a+(v.a-u.a)/2+a,s.b=v.b+(u.b-v.b)/2+a):(s.a=u.a+(v.a-u.a)/2+a,s.b=u.b+(v.b-u.b)/2-a-e.e.b))}function Ho(e,t){var n,s,a,u,p,v,m;if(e==null)return null;if(u=e.length,u==0)return"";for(m=pe(Cl,o1,25,u,15,1),H0e(0,u,e.length),H0e(0,u,m.length),Lrt(e,0,u,m,0),n=null,v=t,a=0,p=0;a<u;a++)s=m[a],Ibt(),s<=32&&(ce[s]&2)!=0?v?(!n&&(n=new Ol(e)),Qvn(n,a-p++)):(v=t,s!=32&&(!n&&(n=new Ol(e)),wvn(n,a-p,a-p+1,String.fromCharCode(32)))):v=!1;return v?n?(u=n.a.length,u>0?$u(n.a,0,u-1):""):e.substr(0,u-1):n?n.a:e}function f2t(e){J2(e,new Xp(K2(q2(W2(Y2(new mp,n2),"ELK DisCo"),"Layouter for arranging unconnected subgraphs. The subgraphs themselves are, by default, not laid out."),new au))),et(e,n2,aJ,at(A4e)),et(e,n2,oJ,at(wte)),et(e,n2,k5,at(S4t)),et(e,n2,Vb,at(S4e)),et(e,n2,lwe,at(L4t)),et(e,n2,hwe,at(D4t)),et(e,n2,uwe,at(I4t)),et(e,n2,fwe,at(M4t)),et(e,n2,mwe,at(A4t)),et(e,n2,ywe,at(vte)),et(e,n2,kwe,at(C4e)),et(e,n2,xwe,at(Wj))}function Cve(e,t,n,s){var a,u,p,v,m,x,_,A,I;if(u=new Nd(e),kp(u,(bn(),Vo)),Fe(u,(tt(),Ei),(_s(),qo)),a=0,t){for(p=new So,Fe(p,(He(),Br),t),Fe(u,Br,t.i),Ji(p,(Qe(),un)),Ka(p,u),I=i1(t.e),x=I,_=0,A=x.length;_<A;++_)m=x[_],xs(m,p);Fe(t,Zc,u),++a}if(n){for(v=new So,Fe(u,(He(),Br),n.i),Fe(v,Br,n),Ji(v,(Qe(),pn)),Ka(v,u),I=i1(n.g),x=I,_=0,A=x.length;_<A;++_)m=x[_],Gs(m,v);Fe(n,Zc,u),++a}return Fe(u,(He(),P$),Ke(a)),s.c[s.c.length]=u,u}function dR(){dR=re,oEe=se(ie(Cl,1),o1,25,15,[48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70]),C9t=new RegExp(`[ 	
\r\f]+`);try{uT=se(ie(HPn,1),st,2015,0,[new fS((Cfe(),EF("yyyy-MM-dd'T'HH:mm:ss'.'SSSZ",YS((uN(),uN(),h_))))),new fS(EF("yyyy-MM-dd'T'HH:mm:ss'.'SSS",YS(h_))),new fS(EF("yyyy-MM-dd'T'HH:mm:ss",YS(h_))),new fS(EF("yyyy-MM-dd'T'HH:mm",YS(h_))),new fS(EF("yyyy-MM-dd",YS(h_)))])}catch(e){if(e=ui(e),!ue(e,78))throw ne(e)}}function TLn(e){var t,n,s,a;if(s=lZ((!e.c&&(e.c=UA(e.f)),e.c),0),e.e==0||e.a==0&&e.f!=-1&&e.e<0)return s;if(t=hge(e)<0?1:0,n=e.e,a=(s.length+1+d.Math.abs(Li(e.e)),new Kv),t==1&&(a.a+="-"),e.e>0)if(n-=s.length-t,n>=0){for(a.a+="0.";n>l2.length;n-=l2.length)Vnt(a,l2);Ctt(a,l2,Li(n)),or(a,s.substr(t))}else n=t-n,or(a,$u(s,t,Li(n))),a.a+=".",or(a,jS(s,Li(n)));else{for(or(a,s.substr(t));n<-l2.length;n+=l2.length)Vnt(a,l2);Ctt(a,l2,Li(-n))}return a.a}function Sve(e,t,n,s){var a,u,p,v,m,x,_,A,I;return m=ms(new dt(n.a,n.b),e),x=m.a*t.b-m.b*t.a,_=t.a*s.b-t.b*s.a,A=(m.a*s.b-m.b*s.a)/_,I=x/_,_==0?x==0?(a=zr(new dt(n.a,n.b),Jf(new dt(s.a,s.b),.5)),u=yg(e,a),p=yg(zr(new dt(e.a,e.b),t),a),v=d.Math.sqrt(s.a*s.a+s.b*s.b)*.5,u<p&&u<=v?new dt(e.a,e.b):p<=v?zr(new dt(e.a,e.b),t):null):null:A>=0&&A<=1&&I>=0&&I<=1?zr(new dt(e.a,e.b),Jf(new dt(t.a,t.b),A)):null}function CLn(e,t,n){var s,a,u,p,v;if(s=h(U(e,(tt(),kne)),21),n.a>t.a&&(s.Hc((Aw(),WD))?e.c.a+=(n.a-t.a)/2:s.Hc(KD)&&(e.c.a+=n.a-t.a)),n.b>t.b&&(s.Hc((Aw(),QD))?e.c.b+=(n.b-t.b)/2:s.Hc(XD)&&(e.c.b+=n.b-t.b)),h(U(e,(He(),Uo)),21).Hc((ha(),El))&&(n.a>t.a||n.b>t.b))for(v=new Q(e.a);v.a<v.c.c.length;)p=h(K(v),10),p.k==(bn(),Pi)&&(a=h(U(p,co),61),a==(Qe(),pn)?p.n.a+=n.a-t.a:a==Fn&&(p.n.b+=n.b-t.b));u=e.d,e.f.a=n.a-u.b-u.c,e.f.b=n.b-u.d-u.a}function SLn(e,t,n){var s,a,u,p,v;if(s=h(U(e,(tt(),kne)),21),n.a>t.a&&(s.Hc((Aw(),WD))?e.c.a+=(n.a-t.a)/2:s.Hc(KD)&&(e.c.a+=n.a-t.a)),n.b>t.b&&(s.Hc((Aw(),QD))?e.c.b+=(n.b-t.b)/2:s.Hc(XD)&&(e.c.b+=n.b-t.b)),h(U(e,(He(),Uo)),21).Hc((ha(),El))&&(n.a>t.a||n.b>t.b))for(p=new Q(e.a);p.a<p.c.c.length;)u=h(K(p),10),u.k==(bn(),Pi)&&(a=h(U(u,co),61),a==(Qe(),pn)?u.n.a+=n.a-t.a:a==Fn&&(u.n.b+=n.b-t.b));v=e.d,e.f.a=n.a-v.b-v.c,e.f.b=n.b-v.d-v.a}function ALn(e){var t,n,s,a,u,p,v,m,x,_,A,I;for(t=L0t(e),_=(v=new P6(t).a.vc().Kc(),new ble(v));_.a.Ob();){for(x=(a=h(_.a.Pb(),42),h(a.cd(),10)),A=0,I=0,A=x.d.d,I=x.o.b+x.d.a,e.d[x.p]=0,n=x;(u=e.a[n.p])!=x;)s=$xn(n,u),m=0,e.c==(r1(),I0)?m=s.d.n.b+s.d.a.b-s.c.n.b-s.c.a.b:m=s.c.n.b+s.c.a.b-s.d.n.b-s.d.a.b,p=Pe(e.d[n.p])+m,e.d[u.p]=p,A=d.Math.max(A,u.d.d-p),I=d.Math.max(I,p+u.o.b+u.d.a),n=u;n=x;do e.d[n.p]=Pe(e.d[n.p])+A,n=e.a[n.p];while(n!=x);e.b[x.p]=A+I}}function WQ(e){var t,n,s,a,u,p,v,m,x,_,A,I;for(e.b=!1,A=ki,m=Fi,I=ki,x=Fi,s=e.e.a.ec().Kc();s.Ob();)for(n=h(s.Pb(),266),a=n.a,A=d.Math.min(A,a.c),m=d.Math.max(m,a.c+a.b),I=d.Math.min(I,a.d),x=d.Math.max(x,a.d+a.a),p=new Q(n.c);p.a<p.c.c.length;)u=h(K(p),395),t=u.a,t.a?(_=a.d+u.b.b,v=_+u.c,I=d.Math.min(I,_),x=d.Math.max(x,v)):(_=a.c+u.b.a,v=_+u.c,A=d.Math.min(A,_),m=d.Math.max(m,v));e.a=new dt(m-A,x-I),e.c=new dt(A+e.d.a,I+e.d.b)}function MLn(e,t,n){var s,a,u,p,v,m,x,_,A;for(A=new qe,_=new l0e(0,n),u=0,zB(_,new JK(0,0,_,n)),a=0,x=new Tn(e);x.e!=x.i.gc();)m=h(On(x),33),s=h(ut(_.a,_.a.c.length-1),187),v=a+m.g+(h(ut(_.a,0),187).b.c.length==0?0:n),v>t&&(a=0,u+=_.b+n,A.c[A.c.length]=_,_=new l0e(u,n),s=new JK(0,_.f,_,n),zB(_,s),a=0),s.b.c.length==0||m.f>=s.o&&m.f<=s.f||s.a*.5<=m.f&&s.a*1.5>=m.f?Epe(s,m):(p=new JK(s.s+s.r+n,_.f,_,n),zB(_,p),Epe(p,m)),a=m.i+m.g;return A.c[A.c.length]=_,A}function $y(e){var t,n,s,a,u,p,v,m;if(!e.a){if(e.o=null,m=new wKe(e),t=new Hze,n=fT,v=n.a.zc(e,n),v==null){for(p=new Tn(La(e));p.e!=p.i.gc();)u=h(On(p),26),mi(m,$y(u));n.a.Bc(e)!=null,n.a.gc()==0}for(a=(!e.s&&(e.s=new Ye(Ac,e,21,17)),new Tn(e.s));a.e!=a.i.gc();)s=h(On(a),170),ue(s,322)&&qn(t,h(s,34));kw(t),e.k=new _nt(e,(h(he(De((bg(),Xt).o),7),18),t.i),t.g),mi(m,e.k),kw(m),e.a=new ay((h(he(De(Xt.o),4),18),m.i),m.g),ou(e).b&=-2}return e.a}function DLn(e,t,n,s,a,u,p){var v,m,x,_,A,I;return A=!1,m=ygt(n.q,t.f+t.b-n.q.f),I=a-(n.q.e+m-p),I<s.g||(x=u==e.c.length-1&&I>=(Kt(u,e.c.length),h(e.c[u],200)).e,_=(v=M9(s,I,!1),v.a),_>t.b&&!x)?!1:((x||_<=t.b)&&(x&&_>t.b?(n.d=_,mA(n,_dt(n,_))):(G1t(n.q,m),n.c=!0),mA(s,a-(n.s+n.r)),tM(s,n.q.e+n.q.d,t.f),zB(t,s),e.c.length>u&&(aM((Kt(u,e.c.length),h(e.c[u],200)),s),(Kt(u,e.c.length),h(e.c[u],200)).a.c.length==0&&f0(e,u)),A=!0),A)}function Ave(e,t,n,s){var a,u,p,v,m,x,_;if(_=ac(e.e.Tg(),t),a=0,u=h(e.g,119),m=null,ia(),h(t,66).Oj()){for(v=0;v<e.i;++v)if(p=u[v],_.rl(p.ak())){if(Nr(p,n)){m=p;break}++a}}else if(n!=null){for(v=0;v<e.i;++v)if(p=u[v],_.rl(p.ak())){if(Nr(n,p.dd())){m=p;break}++a}}else for(v=0;v<e.i;++v)if(p=u[v],_.rl(p.ak())){if(p.dd()==null){m=p;break}++a}return m&&(Tu(e.e)&&(x=t.$j()?new aK(e.e,4,t,n,null,a,!0):wg(e,t.Kj()?2:1,t,n,t.zj(),-1,!0),s?s.Ei(x):s=x),s=_M(e,m,s)),s}function KQ(e,t,n,s,a,u,p){var v,m,x,_,A,I,P,j,V;switch(j=0,V=0,m=a.c,v=a.b,_=n.f,P=n.g,t.g){case 0:j=s.i+s.g+p,e.c?V=s9n(j,u,s,p):V=s.j,I=d.Math.max(m,j+P),x=d.Math.max(v,V+_);break;case 1:V=s.j+s.f+p,e.c?j=i9n(V,u,s,p):j=s.i,I=d.Math.max(m,j+P),x=d.Math.max(v,V+_);break;case 2:j=m+p,V=0,I=m+p+P,x=d.Math.max(v,_);break;case 3:j=0,V=v+p,I=d.Math.max(m,P),x=v+p+_;break;default:throw ne(new an("IllegalPlacementOption."))}return A=new Ape(e.a,I,x,t,j,V),A}function LLn(e){var t,n,s,a,u,p,v,m,x,_,A,I;if(v=e.d,A=h(U(e,(He(),j5)),15),t=h(U(e,N5),15),!(!A&&!t)){if(u=Pe(Je(_w(e,(tt(),Lne)))),p=Pe(Je(_w(e,Oke))),I=0,A){for(x=0,a=A.Kc();a.Ob();)s=h(a.Pb(),10),x=d.Math.max(x,s.o.b),I+=s.o.a;I+=u*(A.gc()-1),v.d+=x+p}if(n=0,t){for(x=0,a=t.Kc();a.Ob();)s=h(a.Pb(),10),x=d.Math.max(x,s.o.b),n+=s.o.a;n+=u*(t.gc()-1),v.a+=x+p}m=d.Math.max(I,n),m>e.o.a&&(_=(m-e.o.a)/2,v.b=d.Math.max(v.b,_),v.c=d.Math.max(v.c,_))}}function ILn(e){var t,n,s,a,u,p,v,m;for(u=new Ust,bdn(u,(n5(),mEt)),s=(a=HK(e,pe(it,Re,2,0,6,1)),new L7(new _u(new Uq(e,a).b)));s.b<s.d.gc();)n=(kn(s.b<s.d.gc()),Jn(s.d.Xb(s.c=s.b++))),p=Pve(tp,n),p&&(t=Ed(e,n),t.je()?v=t.je().a:t.ge()?v=""+t.ge().a:t.he()?v=""+t.he().a:v=t.Ib(),m=Ove(p,v),m!=null&&((Pc(p.j,(Qh(),gs))||Pc(p.j,gn))&&zA(DK(u,wi),p,m),Pc(p.j,k1)&&zA(DK(u,us),p,m),Pc(p.j,w2)&&zA(DK(u,bu),p,m),Pc(p.j,N0)&&zA(DK(u,Ua),p,m)));return u}function gR(e,t,n,s){var a,u,p,v,m,x;if(m=ac(e.e.Tg(),t),u=h(e.g,119),Bd(e.e,t)){for(a=0,v=0;v<e.i;++v)if(p=u[v],m.rl(p.ak())){if(a==n)return ia(),h(t,66).Oj()?p:(x=p.dd(),x!=null&&s&&ue(t,99)&&(h(t,18).Bb&ea)!=0&&(x=m9(e,t,v,a,x)),x);++a}throw ne(new Ea(n_+n+s2+a))}else{for(a=0,v=0;v<e.i;++v){if(p=u[v],m.rl(p.ak()))return ia(),h(t,66).Oj()?p:(x=p.dd(),x!=null&&s&&ue(t,99)&&(h(t,18).Bb&ea)!=0&&(x=m9(e,t,v,a,x)),x);++a}return t.zj()}}function AM(e,t,n){var s,a,u,p,v,m,x,_;if(a=h(e.g,119),Bd(e.e,t))return ia(),h(t,66).Oj()?new _P(t,e):new RS(t,e);for(x=ac(e.e.Tg(),t),s=0,v=0;v<e.i;++v){if(u=a[v],p=u.ak(),x.rl(p)){if(ia(),h(t,66).Oj())return u;if(p==(h5(),Z5)||p==Q5){for(m=new ju(Ga(u.dd()));++v<e.i;)u=a[v],p=u.ak(),(p==Z5||p==Q5)&&or(m,Ga(u.dd()));return C1e(h(t.Yj(),148),m.a)}else return _=u.dd(),_!=null&&n&&ue(t,99)&&(h(t,18).Bb&ea)!=0&&(_=m9(e,t,v,s,_)),_}++s}return t.zj()}function M9(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(u=0,p=e.t,a=0,s=0,m=0,I=0,A=0,n&&(e.n.c=pe(yn,st,1,0,5,1),Ue(e.n,new sB(e.s,e.t,e.i))),v=0,_=new Q(e.b);_.a<_.c.c.length;)x=h(K(_),33),u+x.g+(v>0?e.i:0)>t&&m>0&&(u=0,p+=m+e.i,a=d.Math.max(a,I),s+=m+e.i,m=0,I=0,n&&(++A,Ue(e.n,new sB(e.s,p,e.i))),v=0),I+=x.g+(v>0?e.i:0),m=d.Math.max(m,x.f),n&&s2e(h(ut(e.n,A),211),x),u+=x.g+(v>0?e.i:0),++v;return a=d.Math.max(a,I),s+=m,n&&(e.r=a,e.d=s,u2e(e.j)),new hl(e.s,e.t,a,s)}function Ao(e,t,n,s,a){L1();var u,p,v,m,x,_,A,I,P;if(Ade(e,"src"),Ade(n,"dest"),I=uu(e),m=uu(n),Q1e((I.i&4)!=0,"srcType is not an array"),Q1e((m.i&4)!=0,"destType is not an array"),A=I.c,p=m.c,Q1e((A.i&1)!=0?A==p:(p.i&1)==0,"Array types don't match"),P=e.length,x=n.length,t<0||s<0||a<0||t+a>P||s+a>x)throw ne(new Gle);if((A.i&1)==0&&I!=m)if(_=Fp(e),u=Fp(n),Me(e)===Me(n)&&t<s)for(t+=a,v=s+a;v-- >s;)pi(u,v,_[--t]);else for(v=s+a;s<v;)pi(u,s++,_[t++]);else a>0&&gbe(e,t,n,s,a,!0)}function XQ(){XQ=re,b3t=se(ie(Hn,1),hr,25,15,[$s,1162261467,N9,1220703125,362797056,1977326743,N9,387420489,SR,214358881,429981696,815730721,1475789056,170859375,268435456,410338673,612220032,893871739,128e7,1801088541,113379904,148035889,191102976,244140625,308915776,387420489,481890304,594823321,729e6,887503681,N9,1291467969,1544804416,1838265625,60466176]),v3t=se(ie(Hn,1),hr,25,15,[-1,-1,31,19,15,13,11,11,10,9,9,8,8,8,8,7,7,7,7,7,7,7,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5])}function OLn(e){var t,n,s,a,u,p,v,m;for(a=new Q(e.b);a.a<a.c.c.length;)for(s=h(K(a),29),p=new Q(bb(s.a));p.a<p.c.c.length;)if(u=h(K(p),10),Wct(u)&&(n=h(U(u,(He(),tv)),305),!n.g&&!!n.d))for(t=n,m=n.d;m;)npt(m.i,m.k,!1,!0),EA(t.a),EA(m.i),EA(m.k),EA(m.b),xs(m.c,t.c.d),xs(t.c,null),Ca(t.a,null),Ca(m.i,null),Ca(m.k,null),Ca(m.b,null),v=new v0e(t.i,m.a,t.e,m.j,m.f),v.k=t.k,v.n=t.n,v.b=t.b,v.c=m.c,v.g=t.g,v.d=m.d,Fe(t.i,tv,v),Fe(m.a,tv,v),m=m.d,t=v}function Fw(e,t){var n,s,a,u,p;if(p=h(t,136),Ny(e),Ny(p),p.b!=null){if(e.c=!0,e.b==null){e.b=pe(Hn,hr,25,p.b.length,15,1),Ao(p.b,0,e.b,0,p.b.length);return}for(u=pe(Hn,hr,25,e.b.length+p.b.length,15,1),n=0,s=0,a=0;n<e.b.length||s<p.b.length;)n>=e.b.length?(u[a++]=p.b[s++],u[a++]=p.b[s++]):s>=p.b.length?(u[a++]=e.b[n++],u[a++]=e.b[n++]):p.b[s]<e.b[n]||p.b[s]===e.b[n]&&p.b[s+1]<e.b[n+1]?(u[a++]=p.b[s++],u[a++]=p.b[s++]):(u[a++]=e.b[n++],u[a++]=e.b[n++]);e.b=u}}function NLn(e,t){var n,s,a,u,p,v,m,x,_,A;return n=ft(lt(U(e,(He(),tm)))),v=ft(lt(U(t,tm))),s=h(U(e,rf),11),m=h(U(t,rf),11),a=h(U(e,Yl),11),x=h(U(t,Yl),11),_=!!s&&s==m,A=!!a&&a==x,!n&&!v?new F1e(h(K(new Q(e.j)),11).p==h(K(new Q(t.j)),11).p,_,A):(u=(!ft(lt(U(e,tm)))||ft(lt(U(e,MD))))&&(!ft(lt(U(t,tm)))||ft(lt(U(t,MD)))),p=(!ft(lt(U(e,tm)))||!ft(lt(U(e,MD))))&&(!ft(lt(U(t,tm)))||!ft(lt(U(t,MD)))),new F1e(_&&u||A&&p,_,A))}function PLn(e){var t,n,s,a,u,p,v,m;for(s=0,n=0,m=new di,t=0,v=new Q(e.n);v.a<v.c.c.length;)p=h(K(v),211),p.c.c.length==0?Ai(m,p,m.c.b,m.c):(s=d.Math.max(s,p.d),n+=p.a+(t>0?e.i:0)),++t;for(Ukn(e.n,m),e.d=n,e.r=s,e.g=0,e.f=0,e.e=0,e.o=ki,e.p=ki,u=new Q(e.b);u.a<u.c.c.length;)a=h(K(u),33),e.p=d.Math.min(e.p,a.g),e.g=d.Math.max(e.g,a.g),e.f=d.Math.max(e.f,a.f),e.o=d.Math.min(e.o,a.f),e.e+=a.f+e.i;e.a=e.e/e.b.c.length-e.i*((e.b.c.length-1)/e.b.c.length),u2e(e.j)}function d2t(e){var t,n,s,a;return(e.Db&64)!=0?vX(e):(t=new ju(Dye),s=e.k,s?or(or((t.a+=' "',t),s),'"'):(!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n.i>0&&(a=(!e.n&&(e.n=new Ye(Ua,e,1,7)),h(he(e.n,0),137)).a,!a||or(or((t.a+=' "',t),a),'"'))),n=(!e.b&&(e.b=new qt(Pn,e,4,7)),!(e.b.i<=1&&(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c.i<=1))),n?t.a+=" [":t.a+=" ",or(t,Rfe(new $q,new Tn(e.b))),n&&(t.a+="]"),t.a+=dJ,n&&(t.a+="["),or(t,Rfe(new $q,new Tn(e.c))),n&&(t.a+="]"),t.a)}function QQ(e,t){var n,s,a,u,p,v,m;if(e.a){if(v=e.a.ne(),m=null,v!=null?t.a+=""+v:(p=e.a.Dj(),p!=null&&(u=Zf(p,Ec(91)),u!=-1?(m=p.substr(u),t.a+=""+$u(p==null?_c:(rn(p),p),0,u)):t.a+=""+p)),!!e.d&&e.d.i!=0){for(a=!0,t.a+="<",s=new Tn(e.d);s.e!=s.i.gc();)n=h(On(s),87),a?a=!1:t.a+=Js,QQ(n,t);t.a+=">"}m!=null&&(t.a+=""+m)}else e.e?(v=e.e.zb,v!=null&&(t.a+=""+v)):(t.a+="?",e.b?(t.a+=" super ",QQ(e.b,t)):e.f&&(t.a+=" extends ",QQ(e.f,t)))}function BLn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln;for(Ne=e.c,Ve=t.c,n=Ha(Ne.a,e,0),s=Ha(Ve.a,t,0),le=h(Db(e,(ua(),Jc)).Kc().Pb(),11),xt=h(Db(e,ec).Kc().Pb(),11),we=h(Db(t,Jc).Kc().Pb(),11),ln=h(Db(t,ec).Kc().Pb(),11),oe=i1(le.e),rt=i1(xt.g),fe=i1(we.e),wt=i1(ln.g),Sw(e,s,Ve),p=fe,_=0,j=p.length;_<j;++_)a=p[_],xs(a,le);for(v=wt,A=0,V=v.length;A<V;++A)a=v[A],Gs(a,xt);for(Sw(t,n,Ne),m=oe,I=0,J=m.length;I<J;++I)a=m[I],xs(a,we);for(u=rt,x=0,P=u.length;x<P;++x)a=u[x],Gs(a,ln)}function g2t(e,t,n,s){var a,u,p,v,m,x,_;if(u=bF(s),v=ft(lt(U(s,(tt(),xke)))),(v||ft(lt(U(e,q$))))&&!oy(h(U(e,Ei),98)))a=i5(u),m=bve(e,n,n==(ua(),ec)?a:KA(a));else switch(m=new So,Ka(m,e),t?(_=m.n,_.a=t.a-e.n.a,_.b=t.b-e.n.b,Tdt(_,0,0,e.o.a,e.o.b),Ji(m,Rpt(m,u))):(a=i5(u),Ji(m,n==(ua(),ec)?a:KA(a))),p=h(U(s,(He(),Uo)),21),x=m.j,u.g){case 2:case 1:(x==(Qe(),sn)||x==Fn)&&p.Fc((ha(),u3));break;case 4:case 3:(x==(Qe(),pn)||x==un)&&p.Fc((ha(),u3))}return m}function Mve(e,t,n){var s,a,u,p,v,m,x,_;return d.Math.abs(t.s-t.c)<l1||d.Math.abs(n.s-n.c)<l1?0:(s=Tgt(e,t.j,n.e),a=Tgt(e,n.j,t.e),u=s==-1||a==-1,p=0,u?(s==-1&&(new Ip((qh(),qg),n,t,1),++p),a==-1&&(new Ip((qh(),qg),t,n,1),++p)):(v=_y(t.j,n.s,n.c),v+=_y(n.e,t.s,t.c),m=_y(n.j,t.s,t.c),m+=_y(t.e,n.s,n.c),x=s+16*v,_=a+16*m,x<_?new Ip((qh(),fm),t,n,_-x):x>_?new Ip((qh(),fm),n,t,x-_):x>0&&_>0&&(new Ip((qh(),fm),t,n,0),new Ip(fm,n,t,0))),p)}function p2t(e,t){var n,s,a,u,p,v;for(p=new Gp(new xp(e.f.b).a);p.b;){if(u=kb(p),a=h(u.cd(),594),t==1){if(a.gf()!=(la(),Gd)&&a.gf()!=zd)continue}else if(a.gf()!=(la(),Wl)&&a.gf()!=Dh)continue;switch(s=h(h(u.dd(),46).b,81),v=h(h(u.dd(),46).a,189),n=v.c,a.gf().g){case 2:s.g.c=e.e.a,s.g.b=d.Math.max(1,s.g.b+n);break;case 1:s.g.c=s.g.c+n,s.g.b=d.Math.max(1,s.g.b-n);break;case 4:s.g.d=e.e.b,s.g.a=d.Math.max(1,s.g.a+n);break;case 3:s.g.d=s.g.d+n,s.g.a=d.Math.max(1,s.g.a-n)}}}function FLn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(v=pe(Hn,hr,25,t.b.c.length,15,1),x=pe(Ote,ze,267,t.b.c.length,0,1),m=pe(X1,T0,10,t.b.c.length,0,1),A=e.a,I=0,P=A.length;I<P;++I){for(_=A[I],V=0,p=new Q(_.e);p.a<p.c.c.length;)a=h(K(p),10),s=mfe(a.c),++v[s],j=Pe(Je(U(t,(tt(),Mh)))),v[s]>0&&!!m[s]&&(j=uy(e.b,m[s],a)),V=d.Math.max(V,a.c.c.b+j);for(u=new Q(_.e);u.a<u.c.c.length;)a=h(K(u),10),a.n.b=V+a.d.d,n=a.c,n.c.b=V+a.d.d+a.o.b+a.d.a,x[Ha(n.b.b,n,0)]=a.k,m[Ha(n.b.b,n,0)]=a}}function b2t(e,t){var n,s,a,u,p,v,m,x,_,A,I;for(s=new Sn(Dn(Pd(t).a.Kc(),new L));rr(s);)n=h(Un(s),79),ue(he((!n.b&&(n.b=new qt(Pn,n,4,7)),n.b),0),186)||(m=Na(h(he((!n.c&&(n.c=new qt(Pn,n,5,8)),n.c),0),82)),y9(n)||(p=t.i+t.g/2,v=t.j+t.f/2,_=m.i+m.g/2,A=m.j+m.f/2,I=new Rs,I.a=_-p,I.b=A-v,u=new dt(I.a,I.b),rx(u,t.g,t.f),I.a-=u.a,I.b-=u.b,p=_-I.a,v=A-I.b,x=new dt(I.a,I.b),rx(x,m.g,m.f),I.a-=x.a,I.b-=x.b,_=p+I.a,A=v+I.b,a=Fy(n,!0,!0),Mk(a,p),Lk(a,v),Ak(a,_),Dk(a,A),b2t(e,m)))}function v2t(e){J2(e,new Xp(K2(q2(W2(Y2(new mp,Ub),"ELK SPOrE Compaction"),"ShrinkTree is a compaction algorithm that maintains the topology of a layout. The relocation of diagram elements is based on contracting a spanning tree."),new NHe))),et(e,Ub,iee,at(TH)),et(e,Ub,yye,at(Sre)),et(e,Ub,kye,at(Cre)),et(e,Ub,see,at(E8e)),et(e,Ub,aee,at(Tre)),et(e,Ub,Vb,x8e),et(e,Ub,zw,8),et(e,Ub,oee,at(sEt)),et(e,Ub,xye,at(y8e)),et(e,Ub,Eye,at(k8e)),et(e,Ub,eD,(on(),!1))}function RLn(e,t){var n,s,a,u,p,v,m,x,_,A;for(Bn(t,"Simple node placement",1),A=h(U(e,(He(),d3)),304),v=0,u=new Q(e.b);u.a<u.c.c.length;){for(s=h(K(u),29),p=s.c,p.b=0,n=null,x=new Q(s.a);x.a<x.c.c.length;)m=h(K(x),10),n&&(p.b+=zpe(m,n,A.c)),p.b+=m.d.d+m.o.b+m.d.a,n=m;v=d.Math.max(v,p.b)}for(a=new Q(e.b);a.a<a.c.c.length;)for(s=h(K(a),29),p=s.c,_=(v-p.b)/2,n=null,x=new Q(s.a);x.a<x.c.c.length;)m=h(K(x),10),n&&(_+=zpe(m,n,A.c)),_+=m.d.d,m.n.b=_,_+=m.o.b+m.d.a,n=m;An(t)}function jLn(e,t,n,s){var a,u,p,v,m,x,_,A;if(s.gc()==0)return!1;if(m=(ia(),h(t,66).Oj()),p=m?s:new yb(s.gc()),Bd(e.e,t)){if(t.hi())for(_=s.Kc();_.Ob();)x=_.Pb(),fR(e,t,x,ue(t,99)&&(h(t,18).Bb&ea)!=0)||(u=R1(t,x),p.Fc(u));else if(!m)for(_=s.Kc();_.Ob();)x=_.Pb(),u=R1(t,x),p.Fc(u)}else{for(A=ac(e.e.Tg(),t),a=h(e.g,119),v=0;v<e.i;++v)if(u=a[v],A.rl(u.ak()))throw ne(new an(oD));if(s.gc()>1)throw ne(new an(oD));m||(u=R1(t,s.Kc().Pb()),p.Fc(u))}return jge(e,sbe(e,t,n),p)}function $Ln(e,t){var n,s,a,u;for(E4n(t.b.j),Ti(pc(new Vt(null,new Wt(t.d,16)),new Yje),new Wje),u=new Q(t.d);u.a<u.c.c.length;){switch(a=h(K(u),101),a.e.g){case 0:n=h(ut(a.j,0),113).d.j,dle(a,h(sb(BP(h(jr(a.k,n),15).Oc(),s3)),113)),fle(a,h(sb(PP(h(jr(a.k,n),15).Oc(),s3)),113));break;case 1:s=f2e(a),dle(a,h(sb(BP(h(jr(a.k,s[0]),15).Oc(),s3)),113)),fle(a,h(sb(PP(h(jr(a.k,s[1]),15).Oc(),s3)),113));break;case 2:B_n(e,a);break;case 3:ZSn(a);break;case 4:nAn(e,a)}x4n(a)}e.a=null}function ZQ(e,t,n){var s,a,u,p,v,m,x,_;return s=e.a.o==(yf(),Rf)?ki:Fi,v=qpt(e,new UZe(t,n)),!v.a&&v.c?(vr(e.d,v),s):v.a?(a=v.a.c,m=v.a.d,n?(x=e.a.c==(r1(),lv)?m:a,u=e.a.c==lv?a:m,p=e.a.g[u.i.p],_=Pe(e.a.p[p.p])+Pe(e.a.d[u.i.p])+u.n.b+u.a.b-Pe(e.a.d[x.i.p])-x.n.b-x.a.b):(x=e.a.c==(r1(),I0)?m:a,u=e.a.c==I0?a:m,_=Pe(e.a.p[e.a.g[u.i.p].p])+Pe(e.a.d[u.i.p])+u.n.b+u.a.b-Pe(e.a.d[x.i.p])-x.n.b-x.a.b),e.a.n[e.a.g[a.i.p].p]=(on(),!0),e.a.n[e.a.g[m.i.p].p]=!0,_):s}function pR(e,t,n){var s,a,u,p,v,m,x,_;if(Bd(e.e,t))m=(ia(),h(t,66).Oj()?new _P(t,e):new RS(t,e)),QF(m.c,m.b),vE(m,h(n,14));else{for(_=ac(e.e.Tg(),t),s=h(e.g,119),p=0;p<e.i;++p)if(a=s[p],u=a.ak(),_.rl(u)){if(u==(h5(),Z5)||u==Q5){for(x=n2e(e,t,n),v=p,x?Ow(e,p):++p;p<e.i;)a=s[p],u=a.ak(),u==Z5||u==Q5?Ow(e,p):++p;x||h(Sy(e,v,R1(t,n)),72)}else n2e(e,t,n)?Ow(e,p):h(Sy(e,p,(ia(),h(t,66).Oj()?h(n,72):R1(t,n))),72);return}n2e(e,t,n)||qn(e,(ia(),h(t,66).Oj()?h(n,72):R1(t,n)))}}function w2t(e,t,n){var s,a,u,p,v,m,x,_;return Nr(n,e.b)||(e.b=n,u=new Lc,p=h(Gu(pc(new Vt(null,new Wt(n.f,16)),u),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[(Af(),Kw),Zu]))),21),e.e=!0,e.f=!0,e.c=!0,e.d=!0,a=p.Hc((o5(),gD)),s=p.Hc(pD),a&&!s&&(e.f=!1),!a&&s&&(e.d=!1),a=p.Hc(dD),s=p.Hc(bD),a&&!s&&(e.c=!1),!a&&s&&(e.e=!1)),_=h(e.a.Ce(t,n),46),m=h(_.a,19).a,x=h(_.b,19).a,v=!1,m<0?e.c||(v=!0):e.e||(v=!0),x<0?e.d||(v=!0):e.f||(v=!0),v?w2t(e,_,n):_}function HLn(e){var t,n,s,a;a=e.o,rw(),e.A.dc()||Nr(e.A,y4e)?t=a.b:(t=gM(e.f),e.A.Hc((Iu(),oL))&&!e.B.Hc((fu(),oT))&&(t=d.Math.max(t,gM(h(wa(e.p,(Qe(),pn)),244))),t=d.Math.max(t,gM(h(wa(e.p,un),244)))),n=wlt(e),n&&(t=d.Math.max(t,n.b)),e.A.Hc(cL)&&(e.q==(_s(),ed)||e.q==qo)&&(t=d.Math.max(t,xP(h(wa(e.b,(Qe(),pn)),124))),t=d.Math.max(t,xP(h(wa(e.b,un),124))))),ft(lt(e.e.yf().We((Tr(),IH))))?a.b=d.Math.max(a.b,t):a.b=t,s=e.f.i,s.d=0,s.a=t,UQ(e.f)}function m2t(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(A=0;A<t.length;A++){for(v=e.Kc();v.Ob();)u=h(v.Pb(),225),u.Of(A,t);for(I=0;I<t[A].length;I++){for(m=e.Kc();m.Ob();)u=h(m.Pb(),225),u.Pf(A,I,t);for(V=t[A][I].j,P=0;P<V.c.length;P++){for(x=e.Kc();x.Ob();)u=h(x.Pb(),225),u.Qf(A,I,P,t);for(j=(Kt(P,V.c.length),h(V.c[P],11)),n=0,a=new _f(j.b);Wa(a.a)||Wa(a.b);)for(s=h(Wa(a.a)?K(a.a):K(a.b),17),_=e.Kc();_.Ob();)u=h(_.Pb(),225),u.Nf(A,I,P,n++,s,t)}}}for(p=e.Kc();p.Ob();)u=h(p.Pb(),225),u.Mf()}function zLn(e,t){var n,s,a,u,p,v,m;for(e.b=Pe(Je(U(t,(tt(),p3)))),e.c=Pe(Je(U(t,b2))),e.d=h(U(t,Cne),336),e.a=h(U(t,H$),275),P8n(t),v=h(Gu(Jr(Jr(Xa(Xa(new Vt(null,new Wt(t.b,16)),new lFe),new hFe),new fFe),new dFe),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),a=v.Kc();a.Ob();)n=h(a.Pb(),17),p=h(U(n,(He(),f2)),15),p.Jc(new hYe(e)),Fe(n,f2,null);for(s=v.Kc();s.Ob();)n=h(s.Pb(),17),m=h(U(n,(He(),_6e)),17),u=h(U(n,g3),15),XNn(e,u,m),Fe(n,g3,null)}function GLn(e){e.b=null,e.a=null,e.o=null,e.q=null,e.v=null,e.w=null,e.B=null,e.p=null,e.Q=null,e.R=null,e.S=null,e.T=null,e.U=null,e.V=null,e.W=null,e.bb=null,e.eb=null,e.ab=null,e.H=null,e.db=null,e.c=null,e.d=null,e.f=null,e.n=null,e.r=null,e.s=null,e.u=null,e.G=null,e.J=null,e.e=null,e.j=null,e.i=null,e.g=null,e.k=null,e.t=null,e.F=null,e.I=null,e.L=null,e.M=null,e.O=null,e.P=null,e.$=null,e.N=null,e.Z=null,e.cb=null,e.K=null,e.D=null,e.A=null,e.C=null,e._=null,e.fb=null,e.X=null,e.Y=null,e.gb=!1,e.hb=!1}function JQ(e){var t,n,s,a,u,p,v,m,x;return!(e.k!=(bn(),Wi)||e.j.c.length<=1||(u=h(U(e,(tt(),Ei)),98),u==(_s(),qo))||(a=(Tw(),(e.q?e.q:(Rt(),Rt(),K1))._b(av)?s=h(U(e,av),197):s=h(U(Us(e),L_),197),s),a==rH)||!(a==w3||a==v3)&&(p=Pe(Je(_w(e,I_))),t=h(U(e,PD),142),!t&&(t=new k1e(p,p,p,p)),x=Za(e,(Qe(),un)),m=t.d+t.a+(x.gc()-1)*p,m>e.o.b||(n=Za(e,pn),v=t.d+t.a+(n.gc()-1)*p,v>e.o.b)))}function eZ(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;if(p=e.e,m=t.e,p==0)return t;if(m==0)return e;if(u=e.d,v=t.d,u+v==2)return n=Zi(e.a[0],fa),s=Zi(t.a[0],fa),p==m?(_=Vs(n,s),j=Gn(_),P=Gn(ow(_,32)),P==0?new d0(p,j):new fy(p,2,se(ie(Hn,1),hr,25,15,[j,P]))):eM(p<0?Cg(s,n):Cg(n,s));if(p==m)I=p,A=u>=v?bK(e.a,u,t.a,v):bK(t.a,v,e.a,u);else{if(a=u!=v?u>v?1:-1:epe(e.a,t.a,u),a==0)return Dg(),Gx;a==1?(I=p,A=gK(e.a,u,t.a,v)):(I=m,A=gK(t.a,v,e.a,u))}return x=new fy(I,A.length,A),jE(x),x}function tZ(e,t,n,s,a,u,p){var v,m,x,_,A,I,P;return A=ft(lt(U(t,(tt(),Eke)))),I=null,u==(ua(),Jc)&&s.c.i==n?I=s.c:u==ec&&s.d.i==n&&(I=s.d),x=p,!x||!A||!!I?(_=(Qe(),to),I?_=I.j:oy(h(U(n,Ei),98))&&(_=u==Jc?un:pn),m=VLn(e,t,n,u,_,s),v=fK((Us(n),s)),u==Jc?(Gs(v,h(ut(m.j,0),11)),xs(v,a)):(Gs(v,a),xs(v,h(ut(m.j,0),11))),x=new Nht(s,v,m,h(U(m,(He(),Br)),11),u,!I)):(Ue(x.e,s),P=d.Math.max(Pe(Je(U(x.d,D0))),Pe(Je(U(s,D0)))),Fe(x.d,D0,P)),Ot(e.a,s,new vP(x.d,t,u)),x}function bR(e,t){var n,s,a,u,p,v,m,x,_,A;if(_=null,e.d&&(_=h(Fo(e.d,t),138)),!_){if(u=e.a.Mh(),A=u.i,!e.d||q7(e.d)!=A){for(m=new zn,e.d&&QE(m,e.d),x=m.f.c+m.g.c,v=x;v<A;++v)s=h(he(u,v),138),a=v0(e.e,s).ne(),n=h(a==null?sc(m.f,null,s):Ab(m.g,a,s),138),!!n&&n!=s&&(a==null?sc(m.f,null,n):Ab(m.g,a,n));if(m.f.c+m.g.c!=A)for(p=0;p<x;++p)s=h(he(u,p),138),a=v0(e.e,s).ne(),n=h(a==null?sc(m.f,null,s):Ab(m.g,a,s),138),!!n&&n!=s&&(a==null?sc(m.f,null,n):Ab(m.g,a,n));e.d=m}_=h(Fo(e.d,t),138)}return _}function VLn(e,t,n,s,a,u){var p,v,m,x,_,A;return p=null,x=s==(ua(),Jc)?u.c:u.d,m=bF(t),x.i==n?(p=h(_n(e.b,x),10),p||(p=DM(x,h(U(n,(tt(),Ei)),98),a,eDn(x),null,x.n,x.o,m,t),Fe(p,(He(),Br),x),Pr(e.b,x,p))):(p=DM((_=new bd,A=Pe(Je(U(t,(tt(),Mh))))/2,zA(_,om,A),_),h(U(n,Ei),98),a,s==Jc?-1:1,null,new Rs,new dt(0,0),m,t),v=REn(p,n,s),Fe(p,(He(),Br),v),Pr(e.b,v,p)),h(U(t,(He(),Uo)),21).Fc((ha(),El)),oy(h(U(t,(tt(),Ei)),98))?Fe(t,Ei,(_s(),k8)):Fe(t,Ei,(_s(),$f)),p}function ULn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;Bn(t,"Orthogonal edge routing",1),x=Pe(Je(U(e,(tt(),b3)))),n=Pe(Je(U(e,p3))),s=Pe(Je(U(e,b2))),I=new xW(0,n),J=0,p=new Ms(e.b,0),v=null,_=null,m=null,A=null;do _=p.b<p.d.gc()?(kn(p.b<p.d.gc()),h(p.d.Xb(p.c=p.b++),29)):null,A=_?_.a:null,v&&(Lve(v,J),J+=v.c.a),V=v?J+s:J,j=Bve(I,e,m,A,V),a=!v||WN(m,(iR(),$D)),u=!_||WN(A,(iR(),$D)),j>0?(P=(j-1)*n,v&&(P+=s),_&&(P+=s),P<x&&!a&&!u&&(P=x),J+=P):!a&&!u&&(J+=x),v=_,m=A;while(_);e.f.a=J,An(t)}function nZ(){nZ=re;var e;vEe=new oXe,P9t=pe(it,Re,2,0,6,1),O9t=xf(r5(33,58),r5(1,26)),N9t=xf(r5(97,122),r5(65,90)),fEe=r5(48,57),L9t=xf(O9t,0),I9t=xf(N9t,fEe),dEe=xf(xf(0,r5(1,6)),r5(33,38)),gEe=xf(xf(fEe,r5(65,70)),r5(97,102)),B9t=xf(L9t,AF("-_.!~*'()")),F9t=xf(I9t,xF("-_.!~*'()")),AF(eyt),xF(eyt),xf(B9t,AF(";:@&=+$,")),xf(F9t,xF(";:@&=+$,")),pEe=AF(":/?#"),bEe=xF(":/?#"),lT=AF("/?#"),hT=xF("/?#"),e=new ns,e.a.zc("jar",e),e.a.zc("zip",e),e.a.zc("archive",e),XH=(Rt(),new G6(e))}function y2t(e,t){var n,s,a,u,p,v,m,x,_,A;if(Fe(t,(fo(),g8),0),m=h(U(t,dH),86),t.d.b==0)m?(_=Pe(Je(U(m,O0)))+e.a+S0e(m,t),Fe(t,O0,_)):Fe(t,O0,0);else{for(s=(u=pr(new ng(t).a.d,0),new k4(u));pN(s.a);)n=h(gr(s.a),188).c,y2t(e,n);v=h(rP((p=pr(new ng(t).a.d,0),new k4(p))),86),A=h(f0n((a=pr(new ng(t).a.d,0),new k4(a))),86),x=(Pe(Je(U(A,O0)))+Pe(Je(U(v,O0))))/2,m?(_=Pe(Je(U(m,O0)))+e.a+S0e(m,t),Fe(t,O0,_),Fe(t,g8,Pe(Je(U(t,O0)))-x),JIn(e,t)):Fe(t,O0,x)}}function bx(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;v=0,j=0,m=rB(e.f,e.f.length),u=e.d,p=e.i,s=e.a,a=e.b;do{for(P=0,_=new Q(e.p);_.a<_.c.c.length;)x=h(K(_),10),I=j2t(e,x),n=!0,(e.q==(By(),BD)||e.q==FD)&&(n=ft(lt(I.b))),h(I.a,19).a<0&&n?(++P,m=rB(e.f,e.f.length),e.d=e.d+h(I.a,19).a,j+=u-e.d,u=e.d+h(I.a,19).a,p=e.i,s=bb(e.a),a=bb(e.b)):(e.f=rB(m,m.length),e.d=u,e.a=(Vn(s),s?new Bc(s):sk(new Q(s))),e.b=(Vn(a),a?new Bc(a):sk(new Q(a))),e.i=p);++v,A=P!=0&&ft(lt(t.Kb(new As(Ke(j),Ke(v)))))}while(A)}function qLn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt;return p=e.f,I=t.f,v=p==(fx(),G5)||p==Y_,P=I==G5||I==Y_,m=p==y3||p==W_,j=I==y3||I==W_,x=p==y3||p==G5,V=I==y3||I==G5,v&&P?e.f==Y_?e:t:m&&j?e.f==W_?e:t:x&&V?(p==y3?(A=e,_=t):(A=t,_=e),u=(J=n.j+n.f,oe=A.e+s.f,fe=d.Math.max(J,oe),le=fe-d.Math.min(n.j,A.e),we=A.d+s.g-n.i,we*le),a=(Ne=n.i+n.g,Ve=_.d+s.g,rt=d.Math.max(Ne,Ve),wt=rt-d.Math.min(n.i,_.d),xt=_.e+s.f-n.j,wt*xt),u<=a?e.f==y3?e:t:e.f==G5?e:t):e}function YLn(e){var t,n,s,a,u,p,v,m,x,_,A;for(_=e.e.a.c.length,p=new Q(e.e.a);p.a<p.c.c.length;)u=h(K(p),121),u.j=!1;for(e.i=pe(Hn,hr,25,_,15,1),e.g=pe(Hn,hr,25,_,15,1),e.n=new qe,a=0,A=new qe,m=new Q(e.e.a);m.a<m.c.c.length;)v=h(K(m),121),v.d=a++,v.b.a.c.length==0&&Ue(e.n,v),Vi(A,v.g);for(t=0,s=new Q(A);s.a<s.c.c.length;)n=h(K(s),213),n.c=t++,n.f=!1;x=A.c.length,e.b==null||e.b.length<x?(e.b=pe(ks,ka,25,x,15,1),e.c=pe(vu,Y1,25,x,16,1)):H7(e.c),e.d=A,e.p=new NY(vw(e.d.c.length)),e.j=1}function WLn(e,t){var n,s,a,u,p,v,m,x,_;if(!(t.e.c.length<=1)){for(e.f=t,e.d=h(U(e.f,(sM(),R4e)),379),e.g=h(U(e.f,z4e),19).a,e.e=Pe(Je(U(e.f,j4e))),e.c=Pe(Je(U(e.f,e$))),Irt(e.b),a=new Q(e.f.c);a.a<a.c.c.length;)s=h(K(a),282),uve(e.b,s.c,s,null),uve(e.b,s.d,s,null);for(v=e.f.e.c.length,e.a=Ap(ks,[Re,ka],[104,25],15,[v,v],2),x=new Q(e.f.e);x.a<x.c.c.length;)m=h(K(x),144),bLn(e,m,e.a[m.b]);for(e.i=Ap(ks,[Re,ka],[104,25],15,[v,v],2),u=0;u<v;++u)for(p=0;p<v;++p)n=e.a[u][p],_=1/(n*n),e.i[u][p]=_}}function D9(e){var t,n,s,a;if(!(e.b==null||e.b.length<=2)&&!e.a){for(t=0,a=0;a<e.b.length;){for(t!=a?(e.b[t]=e.b[a++],e.b[t+1]=e.b[a++]):a+=2,n=e.b[t+1];a<e.b.length&&!(n+1<e.b[a]);)if(n+1==e.b[a])e.b[t+1]=e.b[a+1],n=e.b[t+1],a+=2;else if(n>=e.b[a+1])a+=2;else if(n<e.b[a+1])e.b[t+1]=e.b[a+1],n=e.b[t+1],a+=2;else throw ne(new Ya("Token#compactRanges(): Internel Error: ["+e.b[t]+","+e.b[t+1]+"] ["+e.b[a]+","+e.b[a+1]+"]"));t+=2}t!=e.b.length&&(s=pe(Hn,hr,25,t,15,1),Ao(e.b,0,s,0,t),e.b=s),e.a=!0}}function KLn(e,t){var n,s,a,u,p,v,m;for(p=$4(e.a).Kc();p.Ob();){if(u=h(p.Pb(),17),u.b.c.length>0)for(s=new Bc(h(jr(e.a,u),21)),Rt(),hs(s,new Ile(t)),a=new Ms(u.b,0);a.b<a.d.gc();){switch(n=(kn(a.b<a.d.gc()),h(a.d.Xb(a.c=a.b++),70)),v=-1,h(U(n,(tt(),y1)),272).g){case 1:v=s.c.length-1;break;case 0:v=mEn(s);break;case 2:v=0}v!=-1&&(m=(Kt(v,s.c.length),h(s.c[v],243)),Ue(m.b.b,n),h(U(Us(m.b.c.i),(He(),Uo)),21).Fc((ha(),r8)),h(U(Us(m.b.c.i),Uo),21).Fc(n8),Mu(a),Fe(n,k6e,u))}Gs(u,null),xs(u,null)}}function XLn(e,t){var n,s,a,u;return n=new Ru,s=h(Gu(pc(new Vt(null,new Wt(e.f,16)),n),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[(Af(),Kw),Zu]))),21),a=s.gc(),a=a==2?1:0,a==1&&eb(ZA(h(Gu(Jr(s.Lc(),new fc),glt(qp(0),new be)),162).a,2),0)&&(a=0),s=h(Gu(pc(new Vt(null,new Wt(t.f,16)),n),pw(new Zt,new nt,new ct,new Yt,se(ie(gu,1),ze,132,0,[Kw,Zu]))),21),u=s.gc(),u=u==2?1:0,u==1&&eb(ZA(h(Gu(Jr(s.Lc(),new Ll),glt(qp(0),new be)),162).a,2),0)&&(u=0),a<u?-1:a==u?0:1}function QLn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P;if(x=new qe,!os(e,(He(),hne)))return x;for(s=h(U(e,hne),15).Kc();s.Ob();)t=h(s.Pb(),10),lIn(t,e),x.c[x.c.length]=t;for(u=new Q(e.b);u.a<u.c.c.length;)for(a=h(K(u),29),v=new Q(a.a);v.a<v.c.c.length;)p=h(K(v),10),p.k==(bn(),Pi)&&(m=h(U(p,F$),10),m&&(_=new So,Ka(_,p),A=h(U(p,co),61),Ji(_,A),I=h(ut(m.j,0),11),P=new db,Gs(P,_),xs(P,I)));for(n=new Q(x);n.a<n.c.c.length;)t=h(K(n),10),Ca(t,h(ut(e.b,e.b.c.length-1),29));return x}function k2t(e){var t,n,s,a,u,p,v,m,x,_,A,I;for(t=mf(e),u=ft(lt(pt(t,(tt(),am)))),_=0,a=0,x=new Tn((!e.e&&(e.e=new qt(us,e,7,4)),e.e));x.e!=x.i.gc();)m=h(On(x),79),v=Pb(m),p=v&&u&&ft(lt(pt(m,d2))),I=Na(h(he((!m.c&&(m.c=new qt(Pn,m,5,8)),m.c),0),82)),v&&p?++a:v&&!p?++_:bi(I)==t||I==t?++a:++_;for(s=new Tn((!e.d&&(e.d=new qt(us,e,8,5)),e.d));s.e!=s.i.gc();)n=h(On(s),79),v=Pb(n),p=v&&u&&ft(lt(pt(n,d2))),A=Na(h(he((!n.b&&(n.b=new qt(Pn,n,4,7)),n.b),0),82)),v&&p?++_:v&&!p?++a:bi(A)==t||A==t?++_:++a;return _-a}function ZLn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P;if(Bn(t,"Edge splitting",1),e.b.c.length<=2){An(t);return}for(u=new Ms(e.b,0),p=(kn(u.b<u.d.gc()),h(u.d.Xb(u.c=u.b++),29));u.b<u.d.gc();)for(a=p,p=(kn(u.b<u.d.gc()),h(u.d.Xb(u.c=u.b++),29)),m=new Q(a.a);m.a<m.c.c.length;)for(v=h(K(m),10),_=new Q(v.j);_.a<_.c.c.length;)for(x=h(K(_),11),s=new Q(x.g);s.a<s.c.c.length;)n=h(K(s),17),I=n.d,A=I.i.c,A!=a&&A!=p&&Bgt(n,(P=new Nd(e),kp(P,(bn(),ds)),Fe(P,(He(),Br),n),Fe(P,(tt(),Ei),(_s(),qo)),Ca(P,p),P));An(t)}function x2t(e,t){var n,s,a,u,p,v,m,x,_,A,I,P;if(v=t.p!=null&&!t.b,v||Bn(t,Dvt,1),n=h(U(e,(He(),bne)),15),p=1/n.gc(),t.n)for(Sp(t,"ELK Layered uses the following "+n.gc()+" modules:"),P=0,I=n.Kc();I.Ob();)_=h(I.Pb(),51),s=(P<10?"0":"")+P++,Sp(t,"   Slot "+s+": "+sg(uu(_)));for(A=n.Kc();A.Ob();)_=h(A.Pb(),51),_.pf(e,jo(t,p));for(u=new Q(e.b);u.a<u.c.c.length;)a=h(K(u),29),Vi(e.a,a.a),a.a.c=pe(yn,st,1,0,5,1);for(x=new Q(e.a);x.a<x.c.c.length;)m=h(K(x),10),Ca(m,null);e.b.c=pe(yn,st,1,0,5,1),v||An(t)}function JLn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt;s=Pe(Je(U(t,(tt(),Tke)))),Ne=h(U(t,O_),19).a,I=4,a=3,Ve=20/Ne,P=!1,m=0,p=Ir;do{for(u=m!=1,A=m!=0,rt=0,J=e.a,fe=0,we=J.length;fe<we;++fe)j=J[fe],j.f=null,$On(e,j,u,A,s),rt+=d.Math.abs(j.a);do v=LDn(e,t);while(v);for(V=e.a,oe=0,le=V.length;oe<le;++oe)if(j=V[oe],n=_de(j).a,n!=0)for(_=new Q(j.e);_.a<_.c.c.length;)x=h(K(_),10),x.n.b+=n;m==0||m==1?(--I,I<=0&&(rt<p||-I>Ne)?(m=2,p=Ir):m==0?(m=1,p=rt):(m=0,p=rt)):(P=rt>=p||p-rt<Ve,p=rt,P&&--a)}while(!(P&&a<=0))}function rZ(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j;for(j=new zn,u=e.a.ec().Kc();u.Ob();)s=h(u.Pb(),168),Pr(j,s,n.Je(s));for(p=(Vn(e),e?new Bc(e):sk(e.a.ec().Kc())),hs(p,new Vqe(j)),v=bB(p),m=new tP(t),P=new zn,sc(P.f,t,m);v.a.gc()!=0;){for(x=null,_=null,A=null,a=v.a.ec().Kc();a.Ob();)if(s=h(a.Pb(),168),Pe(Je(no(Ia(j.f,s))))<=ki){if(Au(P,s.a)&&!Au(P,s.b)){_=s.b,A=s.a,x=s;break}if(Au(P,s.b)&&!Au(P,s.a)){_=s.a,A=s.b,x=s;break}}if(!x)break;I=new tP(_),Ue(h(no(Ia(P.f,A)),221).a,I),sc(P.f,_,I),v.a.Bc(x)!=null}return m}function eIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P;for(Bn(n,"Depth-first cycle removal",1),A=t.a,_=A.c.length,e.c=new qe,e.d=pe(vu,Y1,25,_,16,1),e.a=pe(vu,Y1,25,_,16,1),e.b=new qe,p=0,x=new Q(A);x.a<x.c.c.length;)m=h(K(x),10),m.p=p,mk(za(m))&&Ue(e.c,m),++p;for(P=new Q(e.c);P.a<P.c.c.length;)I=h(K(P),10),cbe(e,I);for(u=0;u<_;u++)e.d[u]||(v=(Kt(u,A.c.length),h(A.c[u],10)),cbe(e,v));for(a=new Q(e.b);a.a<a.c.c.length;)s=h(K(a),17),jb(s,!0),Fe(t,(He(),SD),(on(),!0));e.c=null,e.d=null,e.a=null,e.b=null,An(n)}function tIn(e,t){var n,s,a,u,p,v,m;for(e.a.c=pe(yn,st,1,0,5,1),s=pr(t.b,0);s.b!=s.d.c;)n=h(gr(s),86),n.b.b==0&&(Fe(n,(fo(),dm),(on(),!0)),Ue(e.a,n));switch(e.a.c.length){case 0:a=new ZK(0,t,"DUMMY_ROOT"),Fe(a,(fo(),dm),(on(),!0)),Fe(a,ire,!0),vr(t.b,a);break;case 1:break;default:for(u=new ZK(0,t,"SUPER_ROOT"),v=new Q(e.a);v.a<v.c.c.length;)p=h(K(v),86),m=new Wde(u,p),Fe(m,(fo(),ire),(on(),!0)),vr(u.a.a,m),vr(u.d,m),vr(p.b,m),Fe(p,dm,!1);Fe(u,(fo(),dm),(on(),!0)),Fe(u,ire,!0),vr(t.b,u)}}function nIn(e,t){Ry();var n,s,a,u,p,v;return u=t.c-(e.c+e.b),a=e.c-(t.c+t.b),p=e.d-(t.d+t.a),n=t.d-(e.d+e.a),s=d.Math.max(a,u),v=d.Math.max(p,n),vf(),xh(d1),(d.Math.abs(s)<=d1||s==0||isNaN(s)&&isNaN(0)?0:s<0?-1:s>0?1:tb(isNaN(s),isNaN(0)))>=0^(xh(d1),(d.Math.abs(v)<=d1||v==0||isNaN(v)&&isNaN(0)?0:v<0?-1:v>0?1:tb(isNaN(v),isNaN(0)))>=0)?d.Math.max(v,s):(xh(d1),(d.Math.abs(s)<=d1||s==0||isNaN(s)&&isNaN(0)?0:s<0?-1:s>0?1:tb(isNaN(s),isNaN(0)))>0?d.Math.sqrt(v*v+s*s):-d.Math.sqrt(v*v+s*s))}function Qp(e,t){var n,s,a,u,p,v;if(!!t){if(!e.a&&(e.a=new sN),e.e==2){iN(e.a,t);return}if(t.e==1){for(a=0;a<t.em();a++)Qp(e,t.am(a));return}if(v=e.a.a.c.length,v==0){iN(e.a,t);return}if(p=h(lb(e.a,v-1),117),!((p.e==0||p.e==10)&&(t.e==0||t.e==10))){iN(e.a,t);return}u=t.e==0?2:t.bm().length,p.e==0?(n=new G7,s=p._l(),s>=ea?va(n,xpe(s)):hk(n,s&Oi),p=new GW(10,null,0),Lbn(e.a,p,v-1)):(n=(p.bm().length+u,new G7),va(n,p.bm())),t.e==0?(s=t._l(),s>=ea?va(n,xpe(s)):hk(n,s&Oi)):va(n,t.bm()),h(p,521).b=n.a}}function E2t(e){var t,n,s,a,u;return e.g!=null?e.g:e.a<32?(e.g=pNn(xc(e.f),Li(e.e)),e.g):(a=lZ((!e.c&&(e.c=UA(e.f)),e.c),0),e.e==0?a:(t=(!e.c&&(e.c=UA(e.f)),e.c).e<0?2:1,n=a.length,s=-e.e+n-t,u=new rg,u.a+=""+a,e.e>0&&s>=-6?s>=0?lA(u,n-Li(e.e),String.fromCharCode(46)):(u.a=$u(u.a,0,t-1)+"0."+jS(u.a,t-1),lA(u,t+1,Fl(l2,0,-Li(s)-1))):(n-t>=1&&(lA(u,t,String.fromCharCode(46)),++n),lA(u,n,String.fromCharCode(69)),s>0&&lA(u,++n,String.fromCharCode(43)),lA(u,++n,""+ME(xc(s)))),e.g=u.a,e.g))}function rIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;if(!n.dc()){for(v=0,I=0,s=n.Kc(),j=h(s.Pb(),19).a;v<t.f;){if(v==j&&(I=0,s.Ob()?j=h(s.Pb(),19).a:j=t.f+1),v!=I){for(J=h(ut(e.b,v),29),P=h(ut(e.b,I),29),V=bb(J.a),A=new Q(V);A.a<A.c.c.length;)if(_=h(K(A),10),Sw(_,P.a.c.length,P),I==0)for(p=bb(za(_)),u=new Q(p);u.a<u.c.c.length;)a=h(K(u),17),jb(a,!0),Fe(e,(He(),SD),(on(),!0)),M2t(e,a,1)}++I,++v}for(m=new Ms(e.b,0);m.b<m.d.gc();)x=(kn(m.b<m.d.gc()),h(m.d.Xb(m.c=m.b++),29)),x.a.c.length==0&&Mu(m)}}function iIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(p=t.b,_=p.o,m=p.d,s=Pe(Je(MF(p,(tt(),Mh)))),a=Pe(Je(MF(p,z5))),x=Pe(Je(MF(p,One))),v=new Tq,Y1e(v,m.d,m.c,m.a,m.b),I=mMn(t,s,a,x),oe=new Q(t.d);oe.a<oe.c.c.length;){for(J=h(K(oe),101),j=J.f.a.ec().Kc();j.Ob();)P=h(j.Pb(),409),u=P.a,A=l9n(P),n=(fe=new Oc,edt(P,P.c,I,fe),B9n(P,A,I,fe),edt(P,P.d,I,fe),fe),n=e.Uf(P,A,n),Pl(u.a),Qs(u.a,n),Ti(new Vt(null,new Wt(n,16)),new BZe(_,v));V=J.i,V&&(ZEn(J,V,I,a),le=new _a(V.g),Ype(_,v,le),zr(le,V.j),Ype(_,v,le))}Y1e(m,v.d,v.c,v.a,v.b)}function sIn(e,t,n){var s,a,u;if(a=h(U(t,(tt(),H$)),275),a!=(ax(),x_)){switch(Bn(n,"Horizontal Compaction",1),e.a=t,u=new Sct,s=new C0t((u.d=t,u.c=h(U(u.d,Hd),218),aMn(u),QIn(u),dMn(u),u.a)),i1n(s,e.b),h(U(t,eke),422).g){case 1:GXe(s,new zut(e.a));break;default:GXe(s,(Yde(),P3t))}switch(a.g){case 1:k9(s);break;case 2:k9(mR(s,(la(),Dh)));break;case 3:k9(zXe(mR(k9(s),(la(),Dh)),new sje));break;case 4:k9(zXe(mR(k9(s),(la(),Dh)),new MYe(u)));break;case 5:k9(r1n(s,j5t))}mR(s,(la(),Wl)),s.e=!0,NOn(u),An(n)}}function aIn(e,t,n,s,a,u,p,v){var m,x,_,A;switch(m=Ef(se(ie(BPn,1),st,220,0,[t,n,s,a])),A=null,e.b.g){case 1:A=Ef(se(ie(Zxe,1),st,526,0,[new uq,new oq,new cq]));break;case 0:A=Ef(se(ie(Zxe,1),st,526,0,[new cq,new oq,new uq]));break;case 2:A=Ef(se(ie(Zxe,1),st,526,0,[new oq,new uq,new cq]))}for(_=new Q(A);_.a<_.c.c.length;)x=h(K(_),526),m.c.length>1&&(m=x.mg(m,e.a,v));return m.c.length==1?h(ut(m,m.c.length-1),220):m.c.length==2?qLn((Kt(0,m.c.length),h(m.c[0],220)),(Kt(1,m.c.length),h(m.c[1],220)),p,u):null}function _2t(e){var t,n,s,a,u,p;for(mc(e.a,new hh),n=new Q(e.a);n.a<n.c.c.length;)t=h(K(n),221),s=ms(ro(h(e.b,65).c),h(t.b,65).c),y4t?(p=h(e.b,65).b,u=h(t.b,65).b,d.Math.abs(s.a)>=d.Math.abs(s.b)?(s.b=0,u.d+u.a>p.d&&u.d<p.d+p.a&&kP(s,d.Math.max(p.c-(u.c+u.b),u.c-(p.c+p.b)))):(s.a=0,u.c+u.b>p.c&&u.c<p.c+p.b&&kP(s,d.Math.max(p.d-(u.d+u.a),u.d-(p.d+p.a))))):kP(s,Jpt(h(e.b,65),h(t.b,65))),a=d.Math.sqrt(s.a*s.a+s.b*s.b),a=K1t(g_,t,a,s),kP(s,a),uW(h(t.b,65),s),mc(t.a,new Mle(s)),h(g_.b,65),$0e(g_,k4e,t)}function oIn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j;for(e.f=new _q,x=0,a=0,p=new Q(e.e.b);p.a<p.c.c.length;)for(u=h(K(p),29),m=new Q(u.a);m.a<m.c.c.length;){for(v=h(K(m),10),v.p=x++,s=new Sn(Dn(qi(v).a.Kc(),new L));rr(s);)n=h(Un(s),17),n.p=a++;for(t=JQ(v),I=new Q(v.j);I.a<I.c.c.length;)A=h(K(I),11),t&&(j=A.a.b,j!=d.Math.floor(j)&&(_=j-vb(xc(d.Math.round(j))),A.a.b-=_)),P=A.n.b+A.a.b,P!=d.Math.floor(P)&&(_=P-vb(xc(d.Math.round(P))),A.n.b-=_)}e.g=x,e.b=a,e.i=pe(OPn,st,401,x,0,1),e.c=pe(IPn,st,649,a,0,1),e.d.a.$b()}function jn(e){var t,n,s,a,u,p,v,m,x;if(e.ej())if(m=e.fj(),e.i>0){if(t=new gfe(e.i,e.g),n=e.i,u=n<100?null:new ig(n),e.ij())for(s=0;s<e.i;++s)p=e.g[s],u=e.kj(p,u);if(VE(e),a=n==1?e.Zi(4,he(t,0),null,0,m):e.Zi(6,t,null,-1,m),e.bj()){for(s=new P4(t);s.e!=s.i.gc();)u=e.dj(NX(s),u);u?(u.Ei(a),u.Fi()):e.$i(a)}else u?(u.Ei(a),u.Fi()):e.$i(a)}else VE(e),e.$i(e.Zi(6,(Rt(),ca),null,-1,m));else if(e.bj())if(e.i>0){for(v=e.g,x=e.i,VE(e),u=x<100?null:new ig(x),s=0;s<x;++s)p=v[s],u=e.dj(p,u);u&&u.Fi()}else VE(e);else VE(e)}function Dve(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(Elt(this),n==(gb(),Yg)?Qi(this.r,e):Qi(this.w,e),_=ki,x=Fi,p=t.a.ec().Kc();p.Ob();)a=h(p.Pb(),46),v=h(a.a,455),s=h(a.b,17),m=s.c,m==e&&(m=s.d),v==Yg?Qi(this.r,m):Qi(this.w,m),I=(Qe(),Cc).Hc(m.j)?Pe(Je(U(m,(He(),a8)))):Qa(se(ie(cs,1),Re,8,0,[m.i.n,m.n,m.a])).b,_=d.Math.min(_,I),x=d.Math.max(x,I);for(A=(Qe(),Cc).Hc(e.j)?Pe(Je(U(e,(He(),a8)))):Qa(se(ie(cs,1),Re,8,0,[e.i.n,e.n,e.a])).b,vdt(this,A,_,x),u=t.a.ec().Kc();u.Ob();)a=h(u.Pb(),46),Sdt(this,h(a.b,17));this.o=!1}function cIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln;return n=e.l&8191,s=e.l>>13|(e.m&15)<<9,a=e.m>>4&8191,u=e.m>>17|(e.h&255)<<5,p=(e.h&1048320)>>8,v=t.l&8191,m=t.l>>13|(t.m&15)<<9,x=t.m>>4&8191,_=t.m>>17|(t.h&255)<<5,A=(t.h&1048320)>>8,wt=n*v,xt=s*v,ln=a*v,dn=u*v,Ln=p*v,m!=0&&(xt+=n*m,ln+=s*m,dn+=a*m,Ln+=u*m),x!=0&&(ln+=n*x,dn+=s*x,Ln+=a*x),_!=0&&(dn+=n*_,Ln+=s*_),A!=0&&(Ln+=n*A),P=wt&du,j=(xt&511)<<13,I=P+j,J=wt>>22,oe=xt>>9,fe=(ln&262143)<<4,le=(dn&31)<<17,V=J+oe+fe+le,Ne=ln>>18,Ve=dn>>5,rt=(Ln&4095)<<8,we=Ne+Ve+rt,V+=I>>22,I&=du,we+=V>>22,V&=du,we&=Rd,rc(I,V,we)}function T2t(e){var t,n,s,a,u,p,v;if(v=h(ut(e.j,0),11),v.g.c.length!=0&&v.e.c.length!=0)throw ne(new ja("Interactive layout does not support NORTH/SOUTH ports with incoming _and_ outgoing edges."));if(v.g.c.length!=0){for(u=ki,n=new Q(v.g);n.a<n.c.c.length;)t=h(K(n),17),p=t.d.i,s=h(U(p,(tt(),W$)),142),u=d.Math.min(u,p.n.a-s.b);return new YO(Vn(u))}if(v.e.c.length!=0){for(a=Fi,n=new Q(v.e);n.a<n.c.c.length;)t=h(K(n),17),p=t.c.i,s=h(U(p,(tt(),W$)),142),a=d.Math.max(a,p.n.a+p.o.a+s.c);return new YO(Vn(a))}return F7(),F7(),qee}function C2t(e,t){var n,s,a,u,p,v,m;if(e.Fk()){if(e.i>4)if(e.wj(t)){if(e.rk()){if(a=h(t,49),s=a.Ug(),m=s==e.e&&(e.Dk()?a.Og(a.Vg(),e.zk())==e.Ak():-1-a.Vg()==e.aj()),e.Ek()&&!m&&!s&&!!a.Zg()){for(u=0;u<e.i;++u)if(n=e.Gk(h(e.g[u],56)),Me(n)===Me(t))return!0}return m}else if(e.Dk()&&!e.Ck()){if(p=h(t,56).ah(aa(h(e.ak(),18))),Me(p)===Me(e.e))return!0;if(p==null||!h(p,56).kh())return!1}}else return!1;if(v=ix(e,t),e.Ek()&&!v){for(u=0;u<e.i;++u)if(a=e.Gk(h(e.g[u],56)),Me(a)===Me(t))return!0}return v}else return ix(e,t)}function uIn(e,t){var n,s,a,u,p,v,m,x,_,A,I;for(_=new qe,I=new ns,p=t.b,a=0;a<p.c.length;a++){for(x=(Kt(a,p.c.length),h(p.c[a],29)).a,_.c=pe(yn,st,1,0,5,1),u=0;u<x.c.length;u++)v=e.a[a][u],v.p=u,v.k==(bn(),Vo)&&(_.c[_.c.length]=v),dl(h(ut(t.b,a),29).a,u,v),v.j.c=pe(yn,st,1,0,5,1),Vi(v.j,h(h(ut(e.b,a),15).Xb(u),14)),F4(h(U(v,(tt(),Ei)),98))||Fe(v,Ei,(_s(),x2));for(s=new Q(_);s.a<s.c.c.length;)n=h(K(s),10),A=LMn(n),I.a.zc(A,I),I.a.zc(n,I)}for(m=I.a.ec().Kc();m.Ob();)v=h(m.Pb(),10),Rt(),hs(v.j,(Uk(),F5e)),v.i=!0,pbe(v)}function lIn(e,t){var n,s,a,u,p,v,m,x,_,A;if(_=h(U(e,(He(),co)),61),s=h(ut(e.j,0),11),_==(Qe(),sn)?Ji(s,Fn):_==Fn&&Ji(s,sn),h(U(t,(tt(),g2)),174).Hc((Iu(),E2))){if(m=Pe(Je(U(e,h8))),x=Pe(Je(U(e,f8))),p=Pe(Je(U(e,lm))),v=h(U(t,cm),21),v.Hc((Qc(),Ud)))for(n=x,A=e.o.a/2-s.n.a,u=new Q(s.f);u.a<u.c.c.length;)a=h(K(u),70),a.n.b=n,a.n.a=A-a.o.a/2,n+=a.o.b+p;else if(v.Hc(Zg))for(u=new Q(s.f);u.a<u.c.c.length;)a=h(K(u),70),a.n.a=m+e.o.a-s.n.a;Imn(new XO((Y7(),new IW(t,!1,!1,new $ue))),new wP(null,e,!1))}}function hIn(e,t){var n,s,a,u,p,v,m,x,_;if(t.c.length!=0){for(Rt(),oW(t.c,t.c.length,null),a=new Q(t),s=h(K(a),145);a.a<a.c.c.length;)n=h(K(a),145),ige(s.e.c,n.e.c)&&!(Vpe(ptt(s.e).b,n.e.d)||Vpe(ptt(n.e).b,s.e.d))?s=(Vi(s.k,n.k),Vi(s.b,n.b),Vi(s.c,n.c),Qs(s.i,n.i),Vi(s.d,n.d),Vi(s.j,n.j),u=d.Math.min(s.e.c,n.e.c),p=d.Math.min(s.e.d,n.e.d),v=d.Math.max(s.e.c+s.e.b,n.e.c+n.e.b),m=v-u,x=d.Math.max(s.e.d+s.e.a,n.e.d+n.e.a),_=x-p,Ant(s.e,u,p,m,_),Cmn(s.f,n.f),!s.a&&(s.a=n.a),Vi(s.g,n.g),Ue(s.g,n),s):(Vpt(e,s),s=n);Vpt(e,s)}}function fIn(e,t,n,s){var a,u,p,v,m,x;if(v=e.j,v==(Qe(),to)&&t!=(_s(),$f)&&t!=(_s(),Qg)&&(v=Rpt(e,n),Ji(e,v),!(e.q?e.q:(Rt(),Rt(),K1))._b((tt(),om))&&v!=to&&(e.n.a!=0||e.n.b!=0)&&Fe(e,om,D7n(e,v))),t==(_s(),ed)){switch(x=0,v.g){case 1:case 3:u=e.i.o.a,u>0&&(x=e.n.a/u);break;case 2:case 4:a=e.i.o.b,a>0&&(x=e.n.b/a)}Fe(e,(He(),iv),x)}if(m=e.o,p=e.a,s)p.a=s.a,p.b=s.b,e.d=!0;else if(t!=$f&&t!=Qg&&v!=to)switch(v.g){case 1:p.a=m.a/2;break;case 2:p.a=m.a,p.b=m.b/2;break;case 3:p.a=m.a/2,p.b=m.b;break;case 4:p.b=m.b/2}else p.a=m.a/2,p.b=m.b/2}function L9(e){var t,n,s,a,u,p,v,m,x,_;if(e.ej())if(_=e.Vi(),m=e.fj(),_>0)if(t=new Cge(e.Gi()),n=_,u=n<100?null:new ig(n),US(e,n,t.g),a=n==1?e.Zi(4,he(t,0),null,0,m):e.Zi(6,t,null,-1,m),e.bj()){for(s=new Tn(t);s.e!=s.i.gc();)u=e.dj(On(s),u);u?(u.Ei(a),u.Fi()):e.$i(a)}else u?(u.Ei(a),u.Fi()):e.$i(a);else US(e,e.Vi(),e.Wi()),e.$i(e.Zi(6,(Rt(),ca),null,-1,m));else if(e.bj())if(_=e.Vi(),_>0){for(v=e.Wi(),x=_,US(e,_,v),u=x<100?null:new ig(x),s=0;s<x;++s)p=v[s],u=e.dj(p,u);u&&u.Fi()}else US(e,e.Vi(),e.Wi());else US(e,e.Vi(),e.Wi())}function dIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P;for(v=new Q(t);v.a<v.c.c.length;)u=h(K(v),233),u.e=null,u.c=0;for(m=null,p=new Q(t);p.a<p.c.c.length;)if(u=h(K(p),233),A=u.d[0],!(n&&A.k!=(bn(),Wi))){for(P=h(U(A,(He(),B5)),15).Kc();P.Ob();)I=h(P.Pb(),10),(!n||I.k==(bn(),Wi))&&((!u.e&&(u.e=new qe),u.e).Fc(e.b[I.c.p][I.p]),++e.b[I.c.p][I.p].c);if(!n&&A.k==(bn(),Wi)){if(m)for(_=h(jr(e.d,m),21).Kc();_.Ob();)for(x=h(_.Pb(),10),a=h(jr(e.d,A),21).Kc();a.Ob();)s=h(a.Pb(),10),T2n(e.b[x.c.p][x.p]).Fc(e.b[s.c.p][s.p]),++e.b[s.c.p][s.p].c;m=A}}}function gIn(e,t){var n,s,a,u,p,v,m,x,_;for(n=0,_=new qe,v=new Q(t);v.a<v.c.c.length;){switch(p=h(K(v),11),npe(e.b,e.d[p.p]),_.c=pe(yn,st,1,0,5,1),p.i.k.g){case 0:s=h(U(p,(He(),Zc)),10),mc(s.j,new oWe(_));break;case 1:Rdn(QK(Jr(new Vt(null,new Wt(p.i.j,16)),new cWe(p))),new uWe(_));break;case 3:a=h(U(p,(He(),Br)),11),Ue(_,new As(a,Ke(p.e.c.length+p.g.c.length)))}for(x=new Q(_);x.a<x.c.c.length;)m=h(K(x),46),u=Ahe(e,h(m.a,11)),u>e.d[p.p]&&(n+=_0e(e.b,u)*h(m.b,19).a,mg(e.a,Ke(u)));for(;!$7(e.a);)tge(e.b,h(H4(e.a),19).a)}return n}function pIn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J;for(A=new _a(h(pt(e,(WF(),q8e)),8)),A.a=d.Math.max(A.a-n.b-n.c,0),A.b=d.Math.max(A.b-n.d-n.a,0),a=Je(pt(e,G8e)),(a==null||(rn(a),a<=0))&&(a=1.3),v=new qe,j=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));j.e!=j.i.gc();)P=h(On(j),33),p=new ntt(P),v.c[v.c.length]=p;switch(I=h(pt(e,Lre),311),I.g){case 3:J=FDn(v,t,A.a,A.b,(x=s,rn(a),x));break;case 1:J=uLn(v,t,A.a,A.b,(_=s,rn(a),_));break;default:J=wIn(v,t,A.a,A.b,(m=s,rn(a),m))}u=new jB(J),V=cZ(u,t,n,A.a,A.b,s,(rn(a),a)),$b(e,V.a,V.b,!1,!0)}function bIn(e,t){var n,s,a,u;n=t.b,u=new Bc(n.j),a=0,s=n.j,s.c=pe(yn,st,1,0,5,1),ub(h(jp(e.b,(Qe(),sn),(Tb(),ev)),15),n),a=rM(u,a,new _je,s),ub(h(jp(e.b,sn,zg),15),n),a=rM(u,a,new Eje,s),ub(h(jp(e.b,sn,Jb),15),n),ub(h(jp(e.b,pn,ev),15),n),ub(h(jp(e.b,pn,zg),15),n),a=rM(u,a,new Tje,s),ub(h(jp(e.b,pn,Jb),15),n),ub(h(jp(e.b,Fn,ev),15),n),a=rM(u,a,new Cje,s),ub(h(jp(e.b,Fn,zg),15),n),a=rM(u,a,new Sje,s),ub(h(jp(e.b,Fn,Jb),15),n),ub(h(jp(e.b,un,ev),15),n),a=rM(u,a,new Rje,s),ub(h(jp(e.b,un,zg),15),n),ub(h(jp(e.b,un,Jb),15),n)}function vIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(Bn(t,"Layer size calculation",1),_=ki,x=Fi,a=!1,v=new Q(e.b);v.a<v.c.c.length;)if(p=h(K(v),29),m=p.c,m.a=0,m.b=0,p.a.c.length!=0){for(a=!0,I=new Q(p.a);I.a<I.c.c.length;)A=h(K(I),10),j=A.o,P=A.d,m.a=d.Math.max(m.a,j.a+P.b+P.c);s=h(ut(p.a,0),10),V=s.n.b-s.d.d,s.k==(bn(),Pi)&&(V-=h(U(e,(tt(),PD)),142).d),u=h(ut(p.a,p.a.c.length-1),10),n=u.n.b+u.o.b+u.d.a,u.k==Pi&&(n+=h(U(e,(tt(),PD)),142).a),m.b=n-V,_=d.Math.min(_,V),x=d.Math.max(x,n)}a||(_=0,x=0),e.f.b=x-_,e.c.b-=_,An(t)}function Lve(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;for(u=0,p=0,x=new Q(e.a);x.a<x.c.c.length;)v=h(K(x),10),u=d.Math.max(u,v.d.b),p=d.Math.max(p,v.d.c);for(m=new Q(e.a);m.a<m.c.c.length;){switch(v=h(K(m),10),n=h(U(v,(tt(),m1)),248),n.g){case 1:j=0;break;case 2:j=1;break;case 5:j=.5;break;default:for(s=0,A=0,P=new Q(v.j);P.a<P.c.c.length;)I=h(K(P),11),I.e.c.length==0||++s,I.g.c.length==0||++A;s+A==0?j=.5:j=A/(s+A)}J=e.c,_=v.o.a,oe=(J.a-_)*j,j>.5?oe-=p*2*(j-.5):j<.5&&(oe+=u*2*(.5-j)),a=v.d.b,oe<a&&(oe=a),V=v.d.c,oe>J.a-V-_&&(oe=J.a-V-_),v.n.a=t+oe}}function wIn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J;for(v=pe(ks,ka,25,e.c.length,15,1),I=new RP(new mze),d2e(I,e),x=0,V=new qe;I.b.c.length!=0;)if(p=h(I.b.c.length==0?null:ut(I.b,0),157),x>1&&Fc(p)*Su(p)/2>v[0]){for(u=0;u<V.c.length-1&&Fc(p)*Su(p)/2>v[u];)++u;j=new F1(V,0,u+1),A=new jB(j),_=Fc(p)/Su(p),m=cZ(A,t,new E4,n,s,a,_),zr(Uh(A.e),m),xk(sx(I,A)),P=new F1(V,u+1,V.c.length),d2e(I,P),V.c=pe(yn,st,1,0,5,1),x=0,trt(v,v.length,0)}else J=I.b.c.length==0?null:ut(I.b,0),J!=null&&jK(I,0),x>0&&(v[x]=v[x-1]),v[x]+=Fc(p)*Su(p),++x,V.c[V.c.length]=p;return V}function mIn(e){var t,n,s,a,u;if(s=h(U(e,(tt(),cc)),163),s==(wl(),Gg)){for(n=new Sn(Dn(za(e).a.Kc(),new L));rr(n);)if(t=h(Un(n),17),!fot(t))throw ne(new ey(bJ+iM(e)+"' has its layer constraint set to FIRST_SEPARATE, but has at least one incoming edge. FIRST_SEPARATE nodes must not have incoming edges."))}else if(s==nm){for(u=new Sn(Dn(qi(e).a.Kc(),new L));rr(u);)if(a=h(Un(u),17),!fot(a))throw ne(new ey(bJ+iM(e)+"' has its layer constraint set to LAST_SEPARATE, but has at least one outgoing edge. LAST_SEPARATE nodes must not have outgoing edges."))}}function yIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;for(Bn(t,"Label dummy removal",1),s=Pe(Je(U(e,(tt(),z5)))),a=Pe(Je(U(e,lm))),x=h(U(e,Ju),103),m=new Q(e.b);m.a<m.c.c.length;)for(v=h(K(m),29),A=new Ms(v.a,0);A.b<A.d.gc();)_=(kn(A.b<A.d.gc()),h(A.d.Xb(A.c=A.b++),10)),_.k==(bn(),Ou)&&(I=h(U(_,(He(),Br)),17),j=Pe(Je(U(I,D0))),p=Me(U(_,F5))===Me((Yu(),Kg)),n=new _a(_.n),p&&(n.b+=j+s),u=new dt(_.o.a,_.o.b-j-s),P=h(U(_,DD),15),x==(la(),Gd)||x==zd?tMn(P,n,a,u,p,x):B6n(P,n,a,u),Vi(I.b,P),iZ(_,Me(U(e,Hd))===Me((Od(),nL))),Mu(A));An(t)}function kIn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne;for(m=new qe,u=new Q(t.a);u.a<u.c.c.length;)for(a=h(K(u),10),v=new Q(a.j);v.a<v.c.c.length;){for(p=h(K(v),11),_=null,le=i1(p.g),we=0,Ne=le.length;we<Ne;++we)fe=le[we],Fk(fe.d.i,n)||(oe=tZ(e,t,n,fe,fe.c,(ua(),ec),_),oe!=_&&(m.c[m.c.length]=oe),oe.c&&(_=oe));for(x=null,j=i1(p.e),V=0,J=j.length;V<J;++V)P=j[V],Fk(P.c.i,n)||(oe=tZ(e,t,n,P,P.d,(ua(),Jc),x),oe!=x&&(m.c[m.c.length]=oe),oe.c&&(x=oe))}for(I=new Q(m);I.a<I.c.c.length;)A=h(K(I),441),Ha(t.a,A.a,0)!=-1||Ue(t.a,A.a),A.c&&(s.c[s.c.length]=A)}function xIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;for(Bn(n,"Interactive cycle breaking",1),A=new qe,P=new Q(t.a);P.a<P.c.c.length;)for(I=h(K(P),10),I.p=1,j=A2e(I).a,_=Db(I,(ua(),ec)).Kc();_.Ob();)for(x=h(_.Pb(),11),u=new Q(x.g);u.a<u.c.c.length;)s=h(K(u),17),V=s.d.i,V!=I&&(J=A2e(V).a,J<j&&(A.c[A.c.length]=s));for(p=new Q(A);p.a<p.c.c.length;)s=h(K(p),17),jb(s,!0);for(A.c=pe(yn,st,1,0,5,1),m=new Q(t.a);m.a<m.c.c.length;)v=h(K(m),10),v.p>0&&Idt(e,v,A);for(a=new Q(A);a.a<a.c.c.length;)s=h(K(a),17),jb(s,!0);A.c=pe(yn,st,1,0,5,1),An(n)}function S2t(e,t){var n,s,a,u,p,v,m,x,_;return x="",t.length==0?e.de(Jve,bZ,-1,-1):(_=Mw(t),Nt(_.substr(0,3),"at ")&&(_=_.substr(3)),_=_.replace(/\[.*?\]/g,""),p=_.indexOf("("),p==-1?(p=_.indexOf("@"),p==-1?(x=_,_=""):(x=Mw(_.substr(p+1)),_=Mw(_.substr(0,p)))):(n=_.indexOf(")",p),x=_.substr(p+1,n-(p+1)),_=Mw(_.substr(0,p))),p=Zf(_,Ec(46)),p!=-1&&(_=_.substr(p+1)),(_.length==0||Nt(_,"Anonymous function"))&&(_=bZ),v=aP(x,Ec(58)),a=A1e(x,Ec(58),v-1),m=-1,s=-1,u=Jve,v!=-1&&a!=-1&&(u=x.substr(0,a),m=Wet(x.substr(a+1,v-(a+1))),s=Wet(x.substr(v+1))),e.de(u,_,m,s))}function Ive(e,t,n){var s,a,u,p,v,m;if(t.l==0&&t.m==0&&t.h==0)throw ne(new cN("divide by zero"));if(e.l==0&&e.m==0&&e.h==0)return n&&(Rg=rc(0,0,0)),rc(0,0,0);if(t.h==FM&&t.m==0&&t.l==0)return Fkn(e,n);if(m=!1,t.h>>19!=0&&(t=Hk(t),m=!m),p=mSn(t),u=!1,a=!1,s=!1,e.h==FM&&e.m==0&&e.l==0)if(a=!0,u=!0,p==-1)e=UJe((Ck(),w3e)),s=!0,m=!m;else return v=Dbe(e,p),m&&KK(v),n&&(Rg=rc(0,0,0)),v;else e.h>>19!=0&&(u=!0,e=Hk(e),s=!0,m=!m);return p!=-1?a6n(e,p,m,u,n):k2e(e,t)<0?(n&&(u?Rg=Hk(e):Rg=rc(e.l,e.m,e.h)),rc(0,0,0)):lDn(s?e:rc(e.l,e.m,e.h),t,m,u,a,n)}function vR(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;if(e.e&&e.c.c<e.f)throw ne(new ja("Expected "+e.f+" phases to be configured; only found "+e.c.c));for(_=h(Vh(e.g),9),P=I1(e.f),u=_,v=0,x=u.length;v<x;++v)s=u[v],A=h(_A(e,s.g),246),A?Ue(P,h(Yht(e,A),123)):P.c[P.c.length]=null;for(j=new rs,Ti(Jr(pc(Jr(new Vt(null,new Wt(P,16)),new HHe),new MWe(t)),new zHe),new DWe(j)),qu(j,e.a),n=new qe,a=_,p=0,m=a.length;p<m;++p)s=a[p],Vi(n,Ilt(e,ZP(h(_A(j,s.g),20)))),I=h(ut(P,s.g),123),I&&(n.c[n.c.length]=I);return Vi(n,Ilt(e,ZP(h(_A(j,_[_.length-1].g+1),20)))),n}function EIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;for(Bn(n,"Model order cycle breaking",1),e.a=0,e.b=0,P=new qe,_=t.a.c.length,x=new Q(t.a);x.a<x.c.c.length;)m=h(K(x),10),os(m,(He(),Eo))&&(_=d.Math.max(_,h(U(m,Eo),19).a+1));for(V=new Q(t.a);V.a<V.c.c.length;)for(j=h(K(V),10),p=zdt(e,j,_),I=Db(j,(ua(),ec)).Kc();I.Ob();)for(A=h(I.Pb(),11),u=new Q(A.g);u.a<u.c.c.length;)s=h(K(u),17),J=s.d.i,v=zdt(e,J,_),v<p&&(P.c[P.c.length]=s);for(a=new Q(P);a.a<a.c.c.length;)s=h(K(a),17),jb(s,!0),Fe(t,(He(),SD),(on(),!0));P.c=pe(yn,st,1,0,5,1),An(n)}function _In(e,t){var n,s,a,u,p,v,m;if(!(e.g>t.f||t.g>e.f)){for(n=0,s=0,p=e.w.a.ec().Kc();p.Ob();)a=h(p.Pb(),11),oX(Qa(se(ie(cs,1),Re,8,0,[a.i.n,a.n,a.a])).b,t.g,t.f)&&++n;for(v=e.r.a.ec().Kc();v.Ob();)a=h(v.Pb(),11),oX(Qa(se(ie(cs,1),Re,8,0,[a.i.n,a.n,a.a])).b,t.g,t.f)&&--n;for(m=t.w.a.ec().Kc();m.Ob();)a=h(m.Pb(),11),oX(Qa(se(ie(cs,1),Re,8,0,[a.i.n,a.n,a.a])).b,e.g,e.f)&&++s;for(u=t.r.a.ec().Kc();u.Ob();)a=h(u.Pb(),11),oX(Qa(se(ie(cs,1),Re,8,0,[a.i.n,a.n,a.a])).b,e.g,e.f)&&--s;n<s?new yB(e,t,s-n):s<n?new yB(t,e,n-s):(new yB(t,e,0),new yB(e,t,0))}}function TIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe;for(x=t.c,a=Tfe(e.e),A=Jf(GS(ro(_fe(e.e)),e.d*e.a,e.c*e.b),-.5),n=a.a-A.a,s=a.b-A.b,p=t.a,n=p.c-n,s=p.d-s,m=new Q(x);m.a<m.c.c.length;){switch(v=h(K(m),395),I=v.b,P=n+I.a,J=s+I.b,j=Li(P/e.a),oe=Li(J/e.b),u=v.a,u.g){case 0:_=(o5(),gD);break;case 1:_=(o5(),dD);break;case 2:_=(o5(),pD);break;default:_=(o5(),bD)}u.a?(fe=Li((J+v.c)/e.b),Ue(e.f,new B1e(_,Ke(oe),Ke(fe))),u==(h9(),wD)?zk(e,0,oe,j,fe):zk(e,j,oe,e.d-1,fe)):(V=Li((P+v.c)/e.a),Ue(e.f,new B1e(_,Ke(j),Ke(V))),u==(h9(),vD)?zk(e,j,0,V,oe):zk(e,j,oe,V,e.c-1))}}function CIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;for(I=new qe,a=new qe,V=null,v=t.Kc();v.Ob();)p=h(v.Pb(),19),u=new XYe(p.a),a.c[a.c.length]=u,V&&(u.d=V,V.e=u),V=u;for(le=JDn(e),_=0;_<a.c.length;++_){for(P=null,J=F0e((Kt(0,a.c.length),h(a.c[0],652))),n=null,s=ki,A=1;A<e.b.c.length;++A)oe=J?d.Math.abs(J.b-A):d.Math.abs(A-P.b)+1,j=P?d.Math.abs(A-P.b):oe+1,j<oe?(x=P,m=j):(x=J,m=oe),fe=(we=Pe(Je(U(e,(tt(),Rke)))),le[A]+d.Math.pow(m,we)),fe<s&&(s=fe,n=x,n.c=A),!!J&&A==J.b&&(P=J,J=_bn(J));n&&(Ue(I,Ke(n.c)),n.a=!0,dkn(n))}return Rt(),oW(I.c,I.c.length,null),I}function SIn(e){var t,n,s,a,u,p,v,m,x,_;for(t=new hq,n=new hq,x=Nt(iD,(a=T9(e.b,lr),a?Jn(Xh((!a.b&&(a.b=new Cu((Bt(),na),uo,a)),a.b),Gl)):null)),m=0;m<e.i;++m)v=h(e.g[m],170),ue(v,99)?(p=h(v,18),(p.Bb&go)!=0?((p.Bb&a1)==0||!x&&(u=T9(p,lr),(u?Jn(Xh((!u.b&&(u.b=new Cu((Bt(),na),uo,u)),u.b),fj)):null)==null))&&qn(t,p):(_=aa(p),!!_&&(_.Bb&go)!=0||((p.Bb&a1)==0||!x&&(s=T9(p,lr),(s?Jn(Xh((!s.b&&(s.b=new Cu((Bt(),na),uo,s)),s.b),fj)):null)==null))&&qn(n,p))):(ia(),h(v,66).Oj()&&(v.Jj()||(qn(t,v),qn(n,v))));kw(t),kw(n),e.a=h(t.g,247),h(n.g,247)}function AIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;for(x=f_n(t),V=h(U(t,(tt(),M_)),314),V!=(e5(),I5)&&Os(x,new rS),J=h(U(t,ID),292),Os(x,new Jqe(J)),j=0,_=new qe,u=new BE(x);u.a!=u.b;)a=h(fF(u),37),$2t(e.c,a),I=h(U(a,(He(),bne)),15),j+=I.gc(),s=I.Kc(),Ue(_,new As(a,s));for(Bn(n,"Recursive hierarchical layout",j),P=h(h(ut(_,_.c.length-1),46).b,47);P.Ob();)for(m=new Q(_);m.a<m.c.c.length;)for(v=h(K(m),46),I=h(v.b,47),p=h(v.a,37);I.Ob();)if(A=h(I.Pb(),51),ue(A,507)){if(p.e)break;A.pf(p,jo(n,1));break}else A.pf(p,jo(n,1));An(n)}function A2t(e,t){var n,s,a,u,p,v,m,x,_,A;if(m=t.length-1,v=(er(m,t.length),t.charCodeAt(m)),v==93){if(p=Zf(t,Ec(91)),p>=0)return a=Mkn(e,t.substr(1,p-1)),_=t.substr(p+1,m-(p+1)),_Nn(e,_,a)}else{if(n=-1,E3e==null&&(E3e=new RegExp("\\d")),E3e.test(String.fromCharCode(v))&&(n=A1e(t,Ec(46),m-1),n>=0)){s=h(dK(e,iut(e,t.substr(1,n-1)),!1),58),x=0;try{x=Wu(t.substr(n+1),$s,Ir)}catch(I){throw I=ui(I),ue(I,127)?(u=I,ne(new MB(u))):ne(I)}if(x<s.gc())return A=s.Xb(x),ue(A,72)&&(A=h(A,72).dd()),h(A,56)}if(n<0)return h(dK(e,iut(e,t.substr(1)),!1),56)}return null}function Hy(e,t,n){var s,a,u,p,v,m,x,_,A;if(oi(t,n)>=0)return n;switch(fb(Sa(e,n))){case 2:{if(Nt("",v0(e,n.Hj()).ne())){if(m=iA(Sa(e,n)),v=gk(Sa(e,n)),_=Fbe(e,t,m,v),_)return _;for(a=lve(e,t),p=0,A=a.gc();p<A;++p)if(_=h(a.Xb(p),170),Ube(vW(Sa(e,_)),m))return _}return null}case 4:{if(Nt("",v0(e,n.Hj()).ne())){for(s=n;s;s=Tmn(Sa(e,s)))if(x=iA(Sa(e,s)),v=gk(Sa(e,s)),_=Rbe(e,t,x,v),_)return _;if(m=iA(Sa(e,n)),Nt(o2,m))return T2e(e,t);for(u=jQ(e,t),p=0,A=u.gc();p<A;++p)if(_=h(u.Xb(p),170),Ube(vW(Sa(e,_)),m))return _}return null}default:return null}}function MIn(e,t,n){var s,a,u,p,v,m,x,_;if(n.gc()==0)return!1;if(v=(ia(),h(t,66).Oj()),u=v?n:new yb(n.gc()),Bd(e.e,t)){if(t.hi())for(x=n.Kc();x.Ob();)m=x.Pb(),fR(e,t,m,ue(t,99)&&(h(t,18).Bb&ea)!=0)||(a=R1(t,m),u.Hc(a)||u.Fc(a));else if(!v)for(x=n.Kc();x.Ob();)m=x.Pb(),a=R1(t,m),u.Fc(a)}else{if(n.gc()>1)throw ne(new an(oD));for(_=ac(e.e.Tg(),t),s=h(e.g,119),p=0;p<e.i;++p)if(a=s[p],_.rl(a.ak())){if(n.Hc(v?a:a.dd()))return!1;for(x=n.Kc();x.Ob();)m=x.Pb(),h(Sy(e,p,v?h(m,72):R1(t,m)),72);return!0}v||(a=R1(t,n.Kc().Pb()),u.Fc(a))}return mi(e,u)}function DIn(e,t){var n,s,a,u,p,v,m,x,_;for(_=new di,v=(x=new vd(e.c).a.vc().Kc(),new qv(x));v.a.Ob();)u=(a=h(v.a.Pb(),42),h(a.dd(),458)),u.b==0&&Ai(_,u,_.c.b,_.c);for(;_.b!=0;)for(u=h(_.b==0?null:(kn(_.b!=0),pl(_,_.a.a)),458),u.a==null&&(u.a=0),s=new Q(u.d);s.a<s.c.c.length;)n=h(K(s),654),n.b.a==null?n.b.a=Pe(u.a)+n.a:t.o==(yf(),v2)?n.b.a=d.Math.min(Pe(n.b.a),Pe(u.a)+n.a):n.b.a=d.Math.max(Pe(n.b.a),Pe(u.a)+n.a),--n.b.b,n.b.b==0&&vr(_,n.b);for(p=(m=new vd(e.c).a.vc().Kc(),new qv(m));p.a.Ob();)u=(a=h(p.a.Pb(),42),h(a.dd(),458)),t.i[u.c.p]=u.a}function fo(){fo=re,U_=new Xr(Mwe),new Xr(Dwe),new Xi("DEPTH",Ke(0)),fH=new Xi("FAN",Ke(0)),G8t=new Xi(ywt,Ke(0)),dm=new Xi("ROOT",(on(),!1)),sre=new Xi("LEFTNEIGHBOR",null),V8t=new Xi("RIGHTNEIGHBOR",null),dH=new Xi("LEFTSIBLING",null),are=new Xi("RIGHTSIBLING",null),ire=new Xi("DUMMY",!1),new Xi("LEVEL",Ke(0)),mxe=new Xi("REMOVABLE_EDGES",new di),ore=new Xi("XCOOR",Ke(0)),yxe=new Xi("YCOOR",Ke(0)),gH=new Xi("LEVELHEIGHT",0),V_=new Xi("ID",""),pH=new Xi("POSITION",Ke(0)),O0=new Xi("PRELIM",0),g8=new Xi("MODIFIER",0),G_=new Xr(Lvt),zD=new Xr(Ivt)}function LIn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j;for(_=n+t.c.c.a,P=new Q(t.j);P.a<P.c.c.length;){if(I=h(K(P),11),a=Qa(se(ie(cs,1),Re,8,0,[I.i.n,I.n,I.a])),t.k==(bn(),Vo)&&(v=h(U(I,(He(),Br)),11),a.a=Qa(se(ie(cs,1),Re,8,0,[v.i.n,v.n,v.a])).a,t.n.a=a.a),p=new dt(0,a.b),I.j==(Qe(),pn))p.a=_;else if(I.j==un)p.a=n;else continue;if(j=d.Math.abs(a.a-p.a),!(j<=s&&!P7n(t)))for(u=I.g.c.length+I.e.c.length>1,x=new _f(I.b);Wa(x.a)||Wa(x.b);)m=h(Wa(x.a)?K(x.a):K(x.b),17),A=m.c==I?m.d:m.c,d.Math.abs(Qa(se(ie(cs,1),Re,8,0,[A.i.n,A.n,A.a])).b-p.b)>1&&EAn(e,m,p,u,I)}}function IIn(e){var t,n,s,a,u,p;if(a=new Ms(e.e,0),s=new Ms(e.a,0),e.d)for(n=0;n<e.b;n++)kn(a.b<a.d.gc()),a.d.Xb(a.c=a.b++);else for(n=0;n<e.b-1;n++)kn(a.b<a.d.gc()),a.d.Xb(a.c=a.b++),Mu(a);for(t=Pe((kn(a.b<a.d.gc()),Je(a.d.Xb(a.c=a.b++))));e.f-t>XJ;){for(u=t,p=0;d.Math.abs(t-u)<XJ;)++p,t=Pe((kn(a.b<a.d.gc()),Je(a.d.Xb(a.c=a.b++)))),kn(s.b<s.d.gc()),s.d.Xb(s.c=s.b++);p<e.b&&(kn(a.b>0),a.a.Xb(a.c=--a.b),KDn(e,e.b-p,u,s,a),kn(a.b<a.d.gc()),a.d.Xb(a.c=a.b++)),kn(s.b>0),s.a.Xb(s.c=--s.b)}if(!e.d)for(n=0;n<e.b-1;n++)kn(a.b<a.d.gc()),a.d.Xb(a.c=a.b++),Mu(a);e.d=!0,e.c=!0}function qr(){qr=re,BEe=(_he(),bo).b,s_t=h(he(De(bo.b),0),34),_2=h(he(De(bo.b),1),34),i_t=h(he(De(bo.b),2),34),S3=bo.bb,h(he(De(bo.bb),0),34),h(he(De(bo.bb),1),34),A3=bo.fb,dT=h(he(De(bo.fb),0),34),h(he(De(bo.fb),1),34),h(he(De(bo.fb),2),18),bv=bo.qb,v_t=h(he(De(bo.qb),0),34),h(he(De(bo.qb),1),18),h(he(De(bo.qb),2),18),yL=h(he(De(bo.qb),3),34),kL=h(he(De(bo.qb),4),34),pT=h(he(De(bo.qb),6),34),gT=h(he(De(bo.qb),5),18),a_t=bo.j,o_t=bo.k,c_t=bo.q,u_t=bo.w,l_t=bo.B,h_t=bo.A,f_t=bo.C,d_t=bo.D,g_t=bo._,p_t=bo.cb,b_t=bo.hb}function OIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P;e.c=0,e.b=0,s=2*t.c.a.c.length+1;e:for(A=n.Kc();A.Ob();){if(_=h(A.Pb(),11),v=_.j==(Qe(),sn)||_.j==Fn,P=0,v){if(I=h(U(_,(He(),Zc)),10),!I)continue;P+=hMn(e,s,_,I)}else{for(x=new Q(_.g);x.a<x.c.c.length;)if(m=h(K(x),17),a=m.d,a.i.c==t.c){Ue(e.a,_);continue e}else P+=e.g[a.p];for(p=new Q(_.e);p.a<p.c.c.length;)if(u=h(K(p),17),a=u.c,a.i.c==t.c){Ue(e.a,_);continue e}else P-=e.g[a.p]}_.e.c.length+_.g.c.length>0?(e.f[_.p]=P/(_.e.c.length+_.g.c.length),e.c=d.Math.min(e.c,e.f[_.p]),e.b=d.Math.max(e.b,e.f[_.p])):v&&(e.f[_.p]=P)}}function NIn(e){e.b=null,e.bb=null,e.fb=null,e.qb=null,e.a=null,e.c=null,e.d=null,e.e=null,e.f=null,e.n=null,e.M=null,e.L=null,e.Q=null,e.R=null,e.K=null,e.db=null,e.eb=null,e.g=null,e.i=null,e.j=null,e.k=null,e.gb=null,e.o=null,e.p=null,e.q=null,e.r=null,e.$=null,e.ib=null,e.S=null,e.T=null,e.t=null,e.s=null,e.u=null,e.v=null,e.w=null,e.B=null,e.A=null,e.C=null,e.D=null,e.F=null,e.G=null,e.H=null,e.I=null,e.J=null,e.P=null,e.Z=null,e.U=null,e.V=null,e.W=null,e.X=null,e.Y=null,e._=null,e.ab=null,e.cb=null,e.hb=null,e.nb=null,e.lb=null,e.mb=null,e.ob=null,e.pb=null,e.jb=null,e.kb=null,e.N=!1,e.O=!1}function PIn(e,t,n){var s,a,u,p;for(Bn(n,"Graph transformation ("+e.a+")",1),p=bb(t.a),u=new Q(t.b);u.a<u.c.c.length;)a=h(K(u),29),Vi(p,a.a);if(s=h(U(t,(tt(),ske)),419),s==(DA(),S$))switch(h(U(t,Ju),103).g){case 2:$E(t,p);break;case 3:c9(t,p);break;case 4:e.a==(Nk(),kD)?(c9(t,p),rK(t,p)):(rK(t,p),c9(t,p))}else if(e.a==(Nk(),kD))switch(h(U(t,Ju),103).g){case 2:$E(t,p),rK(t,p);break;case 3:c9(t,p),$E(t,p);break;case 4:$E(t,p),c9(t,p)}else switch(h(U(t,Ju),103).g){case 2:$E(t,p),rK(t,p);break;case 3:$E(t,p),c9(t,p);break;case 4:c9(t,p),$E(t,p)}An(n)}function BIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V;for(x=new md,_=new md,j=new md,V=new md,m=Pe(Je(U(t,(tt(),uv)))),u=Pe(Je(U(t,Mh))),v=new Q(n);v.a<v.c.c.length;)if(p=h(K(v),10),A=h(U(p,(He(),co)),61),A==(Qe(),sn))for(_.a.zc(p,_),a=new Sn(Dn(za(p).a.Kc(),new L));rr(a);)s=h(Un(a),17),Qi(x,s.c.i);else if(A==Fn)for(V.a.zc(p,V),a=new Sn(Dn(za(p).a.Kc(),new L));rr(a);)s=h(Un(a),17),Qi(j,s.c.i);x.a.gc()!=0&&(I=new xW(2,u),P=Bve(I,t,x,_,-m-t.c.b),P>0&&(e.a=m+(P-1)*u,t.c.b+=e.a,t.f.b+=e.a)),j.a.gc()!=0&&(I=new xW(1,u),P=Bve(I,t,j,V,t.f.b+m-t.c.b),P>0&&(t.f.b+=m+(P-1)*u))}function vx(e,t){var n,s,a,u;u=e.F,t==null?(e.F=null,Vk(e,null)):(e.F=(rn(t),t),s=Zf(t,Ec(60)),s!=-1?(a=t.substr(0,s),Zf(t,Ec(46))==-1&&!Nt(a,g5)&&!Nt(a,r_)&&!Nt(a,vj)&&!Nt(a,i_)&&!Nt(a,s_)&&!Nt(a,a_)&&!Nt(a,o_)&&!Nt(a,c_)&&(a=gyt),n=aP(t,Ec(62)),n!=-1&&(a+=""+t.substr(n+1)),Vk(e,a)):(a=t,Zf(t,Ec(46))==-1&&(s=Zf(t,Ec(91)),s!=-1&&(a=t.substr(0,s)),!Nt(a,g5)&&!Nt(a,r_)&&!Nt(a,vj)&&!Nt(a,i_)&&!Nt(a,s_)&&!Nt(a,a_)&&!Nt(a,o_)&&!Nt(a,c_)?(a=gyt,s!=-1&&(a+=""+t.substr(s))):a=t),Vk(e,a),a==t&&(e.F=e.D))),(e.Db&4)!=0&&(e.Db&1)==0&&Or(e,new fs(e,1,5,u,t))}function FIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;if(V=t.b.c.length,!(V<3)){for(P=pe(Hn,hr,25,V,15,1),A=0,_=new Q(t.b);_.a<_.c.c.length;)x=h(K(_),29),P[A++]=x.a.c.length;for(I=new Ms(t.b,2),s=1;s<V-1;s++)for(n=(kn(I.b<I.d.gc()),h(I.d.Xb(I.c=I.b++),29)),j=new Q(n.a),u=0,v=0,m=0;m<P[s+1];m++)if(le=h(K(j),10),m==P[s+1]-1||rbe(e,le,s+1,s)){for(p=P[s]-1,rbe(e,le,s+1,s)&&(p=e.c.e[h(h(h(ut(e.c.b,le.p),15).Xb(0),46).a,10).p]);v<=m;){if(fe=h(ut(n.a,v),10),!rbe(e,fe,s+1,s))for(oe=h(ut(e.c.b,fe.p),15).Kc();oe.Ob();)J=h(oe.Pb(),46),a=e.c.e[h(J.a,10).p],(a<u||a>p)&&Qi(e.b,h(J.b,17));++v}u=p}}}function Ove(e,t){var n;if(t==null||Nt(t,_c)||t.length==0&&e.k!=(E0(),V5))return null;switch(e.k.g){case 1:return CF(t,Nx)?(on(),Hx):CF(t,hee)?(on(),u2):null;case 2:try{return Ke(Wu(t,$s,Ir))}catch(s){if(s=ui(s),ue(s,127))return null;throw ne(s)}case 4:try{return Dw(t)}catch(s){if(s=ui(s),ue(s,127))return null;throw ne(s)}case 3:return t;case 5:return hht(e),v0t(e,t);case 6:return hht(e),YCn(e,e.a,t);case 7:try{return n=sCn(e),n.Jf(t),n}catch(s){if(s=ui(s),ue(s,32))return null;throw ne(s)}default:throw ne(new ja("Invalid type set for this layout option."))}}function RIn(e){qE();var t,n,s,a,u,p,v;for(v=new $Ke,n=new Q(e);n.a<n.c.c.length;)t=h(K(n),140),(!v.b||t.c>=v.b.c)&&(v.b=t),(!v.c||t.c<=v.c.c)&&(v.d=v.c,v.c=t),(!v.e||t.d>=v.e.d)&&(v.e=t),(!v.f||t.d<=v.f.d)&&(v.f=t);return s=new DF(($k(),Zb)),fA(e,y5t,new _u(se(ie(yD,1),st,369,0,[s]))),p=new DF(Zw),fA(e,m5t,new _u(se(ie(yD,1),st,369,0,[p]))),a=new DF(Qw),fA(e,w5t,new _u(se(ie(yD,1),st,369,0,[a]))),u=new DF(r3),fA(e,v5t,new _u(se(ie(yD,1),st,369,0,[u]))),SQ(s.c,Zb),SQ(a.c,Qw),SQ(u.c,r3),SQ(p.c,Zw),v.a.c=pe(yn,st,1,0,5,1),Vi(v.a,s.c),Vi(v.a,Rp(a.c)),Vi(v.a,u.c),Vi(v.a,Rp(p.c)),v}function Nve(e){var t;switch(e.d){case 1:{if(e.hj())return e.o!=-2;break}case 2:{if(e.hj())return e.o==-2;break}case 3:case 5:case 4:case 6:case 7:return e.o>-2;default:return!1}switch(t=e.gj(),e.p){case 0:return t!=null&&ft(lt(t))!=fE(e.k,0);case 1:return t!=null&&h(t,217).a!=Gn(e.k)<<24>>24;case 2:return t!=null&&h(t,172).a!=(Gn(e.k)&Oi);case 6:return t!=null&&fE(h(t,162).a,e.k);case 5:return t!=null&&h(t,19).a!=Gn(e.k);case 7:return t!=null&&h(t,184).a!=Gn(e.k)<<16>>16;case 3:return t!=null&&Pe(Je(t))!=e.j;case 4:return t!=null&&h(t,155).a!=e.j;default:return t==null?e.n!=null:!Nr(t,e.n)}}function MM(e,t,n){var s,a,u,p;return e.Fk()&&e.Ek()&&(p=gW(e,h(n,56)),Me(p)!==Me(n))?(e.Oi(t),e.Ui(t,sct(e,t,p)),e.rk()&&(u=(a=h(n,49),e.Dk()?e.Bk()?a.ih(e.b,aa(h(Ht(bc(e.b),e.aj()),18)).n,h(Ht(bc(e.b),e.aj()).Yj(),26).Bj(),null):a.ih(e.b,oi(a.Tg(),aa(h(Ht(bc(e.b),e.aj()),18))),null,null):a.ih(e.b,-1-e.aj(),null,null)),!h(p,49).eh()&&(u=(s=h(p,49),e.Dk()?e.Bk()?s.gh(e.b,aa(h(Ht(bc(e.b),e.aj()),18)).n,h(Ht(bc(e.b),e.aj()).Yj(),26).Bj(),u):s.gh(e.b,oi(s.Tg(),aa(h(Ht(bc(e.b),e.aj()),18))),null,u):s.gh(e.b,-1-e.aj(),null,u))),u&&u.Fi()),Tu(e.b)&&e.$i(e.Zi(9,n,p,t,!1)),p):n}function M2t(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;for(_=Pe(Je(U(e,(tt(),cv)))),s=Pe(Je(U(e,Nke))),I=new lq,Fe(I,cv,_+s),x=t,oe=x.d,V=x.c.i,fe=x.d.i,J=mfe(V.c),le=mfe(fe.c),a=new qe,A=J;A<=le;A++)v=new Nd(e),kp(v,(bn(),ds)),Fe(v,(He(),Br),x),Fe(v,Ei,(_s(),qo)),Fe(v,Z$,I),P=h(ut(e.b,A),29),A==J?Sw(v,P.a.c.length-n,P):Ca(v,P),we=Pe(Je(U(x,D0))),we<0&&(we=0,Fe(x,D0,we)),v.o.b=we,j=d.Math.floor(we/2),p=new So,Ji(p,(Qe(),un)),Ka(p,v),p.n.b=j,m=new So,Ji(m,pn),Ka(m,v),m.n.b=j,xs(x,p),u=new db,Oa(u,x),Fe(u,Da,null),Gs(u,m),xs(u,oe),H7n(v,x,u),a.c[a.c.length]=u,x=u;return a}function iZ(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(m=h(x0(e,(Qe(),un)).Kc().Pb(),11).e,P=h(x0(e,pn).Kc().Pb(),11).g,v=m.c.length,le=kf(h(ut(e.j,0),11));v-- >0;){for(V=(Kt(0,m.c.length),h(m.c[0],17)),a=(Kt(0,P.c.length),h(P.c[0],17)),fe=a.d.e,u=Ha(fe,a,0),Umn(V,a.d,u),Gs(a,null),xs(a,null),j=V.a,t&&vr(j,new _a(le)),s=pr(a.a,0);s.b!=s.d.c;)n=h(gr(s),8),vr(j,new _a(n));for(oe=V.b,I=new Q(a.b);I.a<I.c.c.length;)A=h(K(I),70),oe.c[oe.c.length]=A;if(J=h(U(V,(tt(),Da)),74),p=h(U(a,Da),74),p)for(J||(J=new Oc,Fe(V,Da,J)),_=pr(p,0);_.b!=_.d.c;)x=h(gr(_),8),vr(J,new _a(x))}}function D2t(e,t){var n,s,a,u,p,v,m,x,_,A,I,P;if(n=h(wa(e.b,t),124),m=h(h(jr(e.r,t),21),84),m.dc()){n.n.b=0,n.n.c=0;return}for(x=e.u.Hc((Qc(),Ud)),p=0,v=m.Kc(),_=null,A=0,I=0;v.Ob();)s=h(v.Pb(),111),a=Pe(Je(s.b.We((oP(),Uj)))),u=s.b.rf().a,e.A.Hc((Iu(),E2))&&upt(e,t),_?(P=I+_.d.c+e.w+s.d.b,p=d.Math.max(p,(vf(),xh(If),d.Math.abs(A-a)<=If||A==a||isNaN(A)&&isNaN(a)?0:P/(a-A)))):!!e.C&&e.C.b>0&&(p=d.Math.max(p,rlt(e.C.b+s.d.b,a))),_=s,A=a,I=u;!!e.C&&e.C.c>0&&(P=I+e.C.c,x&&(P+=_.d.c),p=d.Math.max(p,(vf(),xh(If),d.Math.abs(A-1)<=If||A==1||isNaN(A)&&isNaN(1)?0:P/(1-A)))),n.n.b=0,n.a.a=p}function L2t(e,t){var n,s,a,u,p,v,m,x,_,A,I,P;if(n=h(wa(e.b,t),124),m=h(h(jr(e.r,t),21),84),m.dc()){n.n.d=0,n.n.a=0;return}for(x=e.u.Hc((Qc(),Ud)),p=0,e.A.Hc((Iu(),E2))&&lpt(e,t),v=m.Kc(),_=null,I=0,A=0;v.Ob();)s=h(v.Pb(),111),u=Pe(Je(s.b.We((oP(),Uj)))),a=s.b.rf().b,_?(P=A+_.d.a+e.w+s.d.d,p=d.Math.max(p,(vf(),xh(If),d.Math.abs(I-u)<=If||I==u||isNaN(I)&&isNaN(u)?0:P/(u-I)))):!!e.C&&e.C.d>0&&(p=d.Math.max(p,rlt(e.C.d+s.d.d,u))),_=s,I=u,A=a;!!e.C&&e.C.a>0&&(P=A+e.C.a,x&&(P+=_.d.a),p=d.Math.max(p,(vf(),xh(If),d.Math.abs(I-1)<=If||I==1||isNaN(I)&&isNaN(1)?0:P/(1-I)))),n.n.d=0,n.a.b=p}function I2t(e,t,n){var s,a,u,p,v,m;for(this.g=e,v=t.d.length,m=n.d.length,this.d=pe(X1,T0,10,v+m,0,1),p=0;p<v;p++)this.d[p]=t.d[p];for(u=0;u<m;u++)this.d[v+u]=n.d[u];if(t.e){if(this.e=ZS(t.e),this.e.Mc(n),n.e)for(a=n.e.Kc();a.Ob();)s=h(a.Pb(),233),s!=t&&(this.e.Hc(s)?--s.c:this.e.Fc(s))}else n.e&&(this.e=ZS(n.e),this.e.Mc(t));this.f=t.f+n.f,this.a=t.a+n.a,this.a>0?LK(this,this.f/this.a):wf(t.g,t.d[0]).a!=null&&wf(n.g,n.d[0]).a!=null?LK(this,(Pe(wf(t.g,t.d[0]).a)+Pe(wf(n.g,n.d[0]).a))/2):wf(t.g,t.d[0]).a!=null?LK(this,wf(t.g,t.d[0]).a):wf(n.g,n.d[0]).a!=null&&LK(this,wf(n.g,n.d[0]).a)}function jIn(e,t){var n,s,a,u,p,v,m,x,_,A;for(e.a=new $rt(g5n(J_)),s=new Q(t.a);s.a<s.c.c.length;){for(n=h(K(s),841),v=new AX(se(ie(Ste,1),st,81,0,[])),Ue(e.a.a,v),x=new Q(n.d);x.a<x.c.c.length;)m=h(K(x),110),_=new Yfe(e,m),Fve(_,h(U(n.c,(He(),em)),21)),Au(e.g,n)||(Pr(e.g,n,new dt(m.c,m.d)),Pr(e.f,n,_)),Ue(e.a.b,_),tK(v,_);for(p=new Q(n.b);p.a<p.c.c.length;)u=h(K(p),594),_=new Yfe(e,u.kf()),Pr(e.b,u,new As(v,_)),Fve(_,h(U(n.c,(He(),em)),21)),u.hf()&&(A=new Wpe(e,u.hf(),1),Fve(A,h(U(n.c,em),21)),a=new AX(se(ie(Ste,1),st,81,0,[])),tK(a,A),Ot(e.c,u.gf(),new As(v,A)))}return e.a}function O2t(e){var t;this.a=e,t=(bn(),se(ie(Ote,1),ze,267,0,[Wi,ds,Pi,Vo,Ou,Bf])).length,this.b=Ap(Mre,[Re,eye],[593,146],0,[t,t],2),this.c=Ap(Mre,[Re,eye],[593,146],0,[t,t],2),HW(this,Wi,(tt(),uv),b3),JE(this,Wi,ds,cv,b2),oA(this,Wi,Vo,cv),oA(this,Wi,Pi,cv),JE(this,Wi,Ou,uv,b3),HW(this,ds,Mh,p3),oA(this,ds,Vo,Mh),oA(this,ds,Pi,Mh),JE(this,ds,Ou,cv,b2),set(this,Vo,Mh),oA(this,Vo,Pi,Mh),oA(this,Vo,Ou,Ine),set(this,Pi,I_),JE(this,Pi,Ou,f8,h8),HW(this,Ou,Mh,Mh),HW(this,Bf,Mh,p3),JE(this,Bf,Wi,cv,b2),JE(this,Bf,Ou,cv,b2),JE(this,Bf,ds,cv,b2)}function $In(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;if(p=n.ak(),ue(p,99)&&(h(p,18).Bb&ea)!=0&&(I=h(n.dd(),49),V=Tg(e.e,I),V!=I)){if(_=R1(p,V),wE(e,t,sQ(e,t,_)),A=null,Tu(e.e)&&(s=Hy((jc(),Ps),e.e.Tg(),p),s!=Ht(e.e.Tg(),e.c))){for(J=ac(e.e.Tg(),p),v=0,u=h(e.g,119),m=0;m<t;++m)a=u[m],J.rl(a.ak())&&++v;A=new aK(e.e,9,s,I,V,v,!1),A.Ei(new Sd(e.e,9,e.c,n,_,t,!1))}return j=h(p,18),P=aa(j),P?(A=I.ih(e.e,oi(I.Tg(),P),null,A),A=h(V,49).gh(e.e,oi(V.Tg(),P),null,A)):(j.Bb&go)!=0&&(x=-1-oi(e.e.Tg(),j),A=I.ih(e.e,x,null,null),!h(V,49).eh()&&(A=h(V,49).gh(e.e,x,null,A))),A&&A.Fi(),_}return n}function HIn(e){var t,n,s,a,u,p,v,m;for(u=new Q(e.a.b);u.a<u.c.c.length;)a=h(K(u),81),a.b.c=a.g.c,a.b.d=a.g.d;for(m=new dt(ki,ki),t=new dt(Fi,Fi),s=new Q(e.a.b);s.a<s.c.c.length;)n=h(K(s),81),m.a=d.Math.min(m.a,n.g.c),m.b=d.Math.min(m.b,n.g.d),t.a=d.Math.max(t.a,n.g.c+n.g.b),t.b=d.Math.max(t.b,n.g.d+n.g.a);for(v=zP(e.c).a.nc();v.Ob();)p=h(v.Pb(),46),n=h(p.b,81),m.a=d.Math.min(m.a,n.g.c),m.b=d.Math.min(m.b,n.g.d),t.a=d.Math.max(t.a,n.g.c+n.g.b),t.b=d.Math.max(t.b,n.g.d+n.g.a);e.d=fP(new dt(m.a,m.b)),e.e=ms(new dt(t.a,t.b),m),e.a.a.c=pe(yn,st,1,0,5,1),e.a.b.c=pe(yn,st,1,0,5,1)}function zIn(e){var t,n,s;for(Ey(tp,se(ie(n3,1),st,130,0,[new nle])),n=new ule(e),s=0;s<n.a.length;++s)t=bw(n,s).je().a,Nt(t,"layered")?Ey(tp,se(ie(n3,1),st,130,0,[new vUe])):Nt(t,"force")?Ey(tp,se(ie(n3,1),st,130,0,[new JVe])):Nt(t,"stress")?Ey(tp,se(ie(n3,1),st,130,0,[new tUe])):Nt(t,"mrtree")?Ey(tp,se(ie(n3,1),st,130,0,[new NUe])):Nt(t,"radial")?Ey(tp,se(ie(n3,1),st,130,0,[new SUe])):Nt(t,"disco")?Ey(tp,se(ie(n3,1),st,130,0,[new ZVe,new XVe])):Nt(t,"sporeOverlap")||Nt(t,"sporeCompaction")?Ey(tp,se(ie(n3,1),st,130,0,[new LUe])):Nt(t,"rectpacking")&&Ey(tp,se(ie(n3,1),st,130,0,[new FUe]))}function N2t(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;if(I=new _a(e.o),oe=t.a/I.a,v=t.b/I.b,V=t.a-I.a,u=t.b-I.b,n)for(a=Me(U(e,(tt(),Ei)))===Me((_s(),qo)),j=new Q(e.j);j.a<j.c.c.length;)switch(P=h(K(j),11),P.j.g){case 1:a||(P.n.a*=oe);break;case 2:P.n.a+=V,a||(P.n.b*=v);break;case 3:a||(P.n.a*=oe),P.n.b+=u;break;case 4:a||(P.n.b*=v)}for(x=new Q(e.b);x.a<x.c.c.length;)m=h(K(x),70),_=m.n.a+m.o.a/2,A=m.n.b+m.o.b/2,J=_/I.a,p=A/I.b,J+p>=1&&(J-p>0&&A>=0?(m.n.a+=V,m.n.b+=u*p):J-p<0&&_>=0&&(m.n.a+=V*J,m.n.b+=u));e.o.a=t.a,e.o.b=t.b,Fe(e,(tt(),g2),(Iu(),s=h(Vh(aT),9),new ll(s,h(vh(s,s.length),9),0)))}function GIn(e,t,n,s,a,u){var p;if(!(t==null||!kX(t,pEe,bEe)))throw ne(new an("invalid scheme: "+t));if(!e&&!(n!=null&&Zf(n,Ec(35))==-1&&n.length>0&&(er(0,n.length),n.charCodeAt(0)!=47)))throw ne(new an("invalid opaquePart: "+n));if(e&&!(t!=null&&kS(XH,t.toLowerCase()))&&!(n==null||!kX(n,lT,hT)))throw ne(new an(tyt+n));if(e&&t!=null&&kS(XH,t.toLowerCase())&&!BEn(n))throw ne(new an(tyt+n));if(!$kn(s))throw ne(new an("invalid device: "+s));if(!O6n(a))throw p=a==null?"invalid segments: null":"invalid segment: "+M6n(a),ne(new an(p));if(!(u==null||Zf(u,Ec(35))==-1))throw ne(new an("invalid query: "+u))}function VIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;for(Bn(t,"Calculate Graph Size",1),t.n&&!!e&&mh(t,yh(e),(Lu(),rl)),v=Ax,m=Ax,u=sye,p=sye,A=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));A.e!=A.i.gc();)x=h(On(A),33),j=x.i,V=x.j,oe=x.g,s=x.f,a=h(pt(x,(Tr(),eL)),142),v=d.Math.min(v,j-a.b),m=d.Math.min(m,V-a.d),u=d.Math.max(u,j+oe+a.c),p=d.Math.max(p,V+s+a.a);for(P=h(pt(e,(Tr(),y2)),116),I=new dt(v-P.b,m-P.d),_=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));_.e!=_.i.gc();)x=h(On(_),33),yc(x,x.i-I.a),kc(x,x.j-I.b);J=u-v+(P.b+P.c),n=p-m+(P.d+P.a),Eb(e,J),xb(e,n),t.n&&!!e&&mh(t,yh(e),(Lu(),rl))}function P2t(e){var t,n,s,a,u,p,v,m,x,_;for(s=new qe,p=new Q(e.e.a);p.a<p.c.c.length;){for(a=h(K(p),121),_=0,a.k.c=pe(yn,st,1,0,5,1),n=new Q(Ty(a));n.a<n.c.c.length;)t=h(K(n),213),t.f&&(Ue(a.k,t),++_);_==1&&(s.c[s.c.length]=a)}for(u=new Q(s);u.a<u.c.c.length;)for(a=h(K(u),121);a.k.c.length==1;){for(x=h(K(new Q(a.k)),213),e.b[x.c]=x.g,v=x.d,m=x.e,n=new Q(Ty(a));n.a<n.c.c.length;)t=h(K(n),213),Nr(t,x)||(t.f?v==t.d||m==t.e?e.b[x.c]-=e.b[t.c]-t.g:e.b[x.c]+=e.b[t.c]-t.g:a==v?t.d==a?e.b[x.c]+=t.g:e.b[x.c]-=t.g:t.d==a?e.b[x.c]-=t.g:e.b[x.c]+=t.g);vc(v.k,x),vc(m.k,x),v==a?a=x.e:a=x.d}}function Pve(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;if(t==null||t.length==0)return null;if(u=h(Fo(e.f,t),23),!u){for(a=(P=new vd(e.d).a.vc().Kc(),new qv(P));a.a.Ob();)if(n=(p=h(a.a.Pb(),42),h(p.dd(),23)),v=n.f,j=t.length,Nt(v.substr(v.length-j,j),t)&&(t.length==v.length||Is(v,v.length-t.length-1)==46)){if(u)return null;u=n}if(!u){for(s=(I=new vd(e.d).a.vc().Kc(),new qv(I));s.a.Ob();)if(n=(p=h(s.a.Pb(),42),h(p.dd(),23)),A=n.g,A!=null){for(m=A,x=0,_=m.length;x<_;++x)if(v=m[x],j=t.length,Nt(v.substr(v.length-j,j),t)&&(t.length==v.length||Is(v,v.length-t.length-1)==46)){if(u)return null;u=n}}}u&&Ta(e.f,t,u)}return u}function UIn(e,t){var n,s,a,u,p;for(n=new Kv,p=!1,u=0;u<t.length;u++){if(s=(er(u,t.length),t.charCodeAt(u)),s==32){for(LF(e,n,0),n.a+=" ",LF(e,n,0);u+1<t.length&&(er(u+1,t.length),t.charCodeAt(u+1)==32);)++u;continue}if(p){s==39?u+1<t.length&&(er(u+1,t.length),t.charCodeAt(u+1)==39)?(n.a+=String.fromCharCode(s),++u):p=!1:n.a+=String.fromCharCode(s);continue}if(Zf("GyMLdkHmsSEcDahKzZv",Ec(s))>0){LF(e,n,0),n.a+=String.fromCharCode(s),a=Mxn(t,u),LF(e,n,a),u+=a-1;continue}s==39?u+1<t.length&&(er(u+1,t.length),t.charCodeAt(u+1)==39)?(n.a+="'",++u):p=!0:n.a+=String.fromCharCode(s)}LF(e,n,0),a9n(e)}function qIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;if(Bn(n,"Network simplex layering",1),e.b=t,oe=h(U(t,(tt(),O_)),19).a*4,J=e.b.a,J.c.length<1){An(n);return}for(u=qMn(e,J),V=null,a=pr(u,0);a.b!=a.d.c;){for(s=h(gr(a),15),v=oe*Li(d.Math.sqrt(s.gc())),p=uDn(s),$Q(whe(s1n(mhe(QY(p),v),V),!0),jo(n,1)),I=e.b.b,j=new Q(p.a);j.a<j.c.c.length;){for(P=h(K(j),121);I.c.length<=P.e;)aw(I,I.c.length,new Nl(e.b));_=h(P.f,10),Ca(_,h(ut(I,P.e),29))}if(u.b>1)for(V=pe(Hn,hr,25,e.b.b.c.length,15,1),A=0,x=new Q(e.b.b);x.a<x.c.c.length;)m=h(K(x),29),V[A++]=m.a.c.length}J.c=pe(yn,st,1,0,5,1),e.a=null,e.b=null,e.c=null,An(n)}function B2t(e){var t,n,s,a,u,p,v;for(t=0,u=new Q(e.b.a);u.a<u.c.c.length;)s=h(K(u),189),s.b=0,s.c=0;for(kdt(e,0),xX(e,e.g),nR(e.c),dhe(e.c),n=(la(),Wl),TM(kY(zy(TM(kY(zy(TM(zy(e.c,n)),$ht(n)))),n))),zy(e.c,Wl),pX(e,e.g),udt(e,0),p2t(e,0),egt(e,1),kdt(e,1),xX(e,e.d),nR(e.c),p=new Q(e.b.a);p.a<p.c.c.length;)s=h(K(p),189),t+=d.Math.abs(s.c);for(v=new Q(e.b.a);v.a<v.c.c.length;)s=h(K(v),189),s.b=0,s.c=0;for(n=Gd,TM(kY(zy(TM(kY(zy(TM(dhe(zy(e.c,n))),$ht(n)))),n))),zy(e.c,Wl),pX(e,e.d),udt(e,1),p2t(e,1),egt(e,0),dhe(e.c),a=new Q(e.b.a);a.a<a.c.c.length;)s=h(K(a),189),t+=d.Math.abs(s.c);return t}function F2t(e,t){var n,s,a,u,p,v,m,x,_;if(x=t,!(x.b==null||e.b==null)){for(Ny(e),D9(e),Ny(x),D9(x),n=pe(Hn,hr,25,e.b.length+x.b.length,15,1),_=0,s=0,p=0;s<e.b.length&&p<x.b.length;)if(a=e.b[s],u=e.b[s+1],v=x.b[p],m=x.b[p+1],u<v)s+=2;else if(u>=v&&a<=m)v<=a&&u<=m?(n[_++]=a,n[_++]=u,s+=2):v<=a?(n[_++]=a,n[_++]=m,e.b[s]=m+1,p+=2):u<=m?(n[_++]=v,n[_++]=u,s+=2):(n[_++]=v,n[_++]=m,e.b[s]=m+1);else if(m<a)p+=2;else throw ne(new Ya("Token#intersectRanges(): Internal Error: ["+e.b[s]+","+e.b[s+1]+"] & ["+x.b[p]+","+x.b[p+1]+"]"));for(;s<e.b.length;)n[_++]=e.b[s++],n[_++]=e.b[s++];e.b=pe(Hn,hr,25,_,15,1),Ao(n,0,e.b,0,_)}}function YIn(e){var t,n,s,a,u,p,v;for(t=new qe,e.g=new qe,e.d=new qe,p=new Gp(new xp(e.f.b).a);p.b;)u=kb(p),Ue(t,h(h(u.dd(),46).b,81)),lg(h(u.cd(),594).gf())?Ue(e.d,h(u.dd(),46)):Ue(e.g,h(u.dd(),46));for(xX(e,e.d),xX(e,e.g),e.c=new m0t(e.b),o1n(e.c,(Ihe(),f5t)),pX(e,e.d),pX(e,e.g),Vi(t,e.c.a.b),e.e=new dt(ki,ki),e.a=new dt(Fi,Fi),s=new Q(t);s.a<s.c.c.length;)n=h(K(s),81),e.e.a=d.Math.min(e.e.a,n.g.c),e.e.b=d.Math.min(e.e.b,n.g.d),e.a.a=d.Math.max(e.a.a,n.g.c+n.g.b),e.a.b=d.Math.max(e.a.b,n.g.d+n.g.a);khe(e.c,new Zm),v=0;do a=B2t(e),++v;while((v<2||a>Ig)&&v<10);khe(e.c,new r0),B2t(e),Obn(e.c),HIn(e.f)}function WIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;if(!!ft(lt(U(n,(tt(),am)))))for(v=new Q(n.j);v.a<v.c.c.length;)for(p=h(K(v),11),I=i1(p.g),x=I,_=0,A=x.length;_<A;++_)m=x[_],u=m.d.i==n,a=u&&ft(lt(U(m,d2))),a&&(j=m.c,P=h(_n(e.b,j),10),P||(P=DM(j,(_s(),$f),j.j,-1,null,null,j.o,h(U(t,Ju),103),t),Fe(P,(He(),Br),j),Pr(e.b,j,P),Ue(t.a,P)),J=m.d,V=h(_n(e.b,J),10),V||(V=DM(J,(_s(),$f),J.j,1,null,null,J.o,h(U(t,Ju),103),t),Fe(V,(He(),Br),J),Pr(e.b,J,V),Ue(t.a,V)),s=fK(m),Gs(s,h(ut(P.j,0),11)),xs(s,h(ut(V.j,0),11)),Ot(e.a,m,new vP(s,t,(ua(),ec))),h(U(t,(He(),Uo)),21).Fc((ha(),El)))}function KIn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j;for(Bn(n,"Label dummy switching",1),s=h(U(t,(tt(),V$)),227),o5n(t),a=HCn(t,s),e.a=pe(ks,ka,25,t.b.c.length,15,1),v=(u5(),se(ie(e8,1),ze,227,0,[Jx,m_,Zx,Jw,o3,a3])),_=0,P=v.length;_<P;++_)if(u=v[_],(u==o3||u==a3||u==Jw)&&!h(Pc(a.a,u)?a.b[u.g]:null,15).dc()){w5n(e,t);break}for(m=se(ie(e8,1),ze,227,0,[Jx,m_,Zx,Jw,o3,a3]),A=0,j=m.length;A<j;++A)u=m[A],u==o3||u==a3||u==Jw||Cpt(e,h(Pc(a.a,u)?a.b[u.g]:null,15));for(p=se(ie(e8,1),ze,227,0,[Jx,m_,Zx,Jw,o3,a3]),x=0,I=p.length;x<I;++x)u=p[x],(u==o3||u==a3||u==Jw)&&Cpt(e,h(Pc(a.a,u)?a.b[u.g]:null,15));e.a=null,An(n)}function XIn(e,t){var n,s,a,u,p,v,m,x,_,A,I;switch(e.k.g){case 1:if(s=h(U(e,(He(),Br)),17),n=h(U(s,m6e),74),n?ft(lt(U(s,Ff)))&&(n=GA(n)):n=new Oc,x=h(U(e,rf),11),x){if(_=Qa(se(ie(cs,1),Re,8,0,[x.i.n,x.n,x.a])),t<=_.a)return _.b;Ai(n,_,n.a,n.a.a)}if(A=h(U(e,Yl),11),A){if(I=Qa(se(ie(cs,1),Re,8,0,[A.i.n,A.n,A.a])),I.a<=t)return I.b;Ai(n,I,n.c.b,n.c)}if(n.b>=2){for(m=pr(n,0),p=h(gr(m),8),v=h(gr(m),8);v.a<t&&m.b!=m.d.c;)p=v,v=h(gr(m),8);return p.b+(t-p.a)/(v.a-p.a)*(v.b-p.b)}break;case 3:switch(u=h(U(h(ut(e.j,0),11),(He(),Br)),11),a=u.i,u.j.g){case 1:return a.n.b;case 3:return a.n.b+a.o.b}}return A2e(e).b}function QIn(e){var t,n,s,a,u,p,v,m,x,_,A;for(p=new Q(e.d.b);p.a<p.c.c.length;)for(u=h(K(p),29),m=new Q(u.a);m.a<m.c.c.length;){if(v=h(K(m),10),ft(lt(U(v,(tt(),A_))))&&!mk(Id(v))){s=h(Xvn(Id(v)),17),_=s.c.i,_==v&&(_=s.d.i),A=new As(_,ms(ro(v.n),_.n)),Pr(e.b,v,A);continue}a=new hl(v.n.a-v.d.b,v.n.b-v.d.d,v.o.a+v.d.b+v.d.c,v.o.b+v.d.d+v.d.a),t=qet(iZe(nZe(rZe(new Qle,v),a),$5t),e.a),Uet(N1n(Eut(new Xle,se(ie(Gj,1),st,57,0,[t])),t),e.a),x=new xq,Pr(e.e,t,x),n=Td(new Sn(Dn(za(v).a.Kc(),new L)))-Td(new Sn(Dn(qi(v).a.Kc(),new L))),n<0?HA(x,!0,(la(),Wl)):n>0&&HA(x,!0,(la(),Dh)),v.k==(bn(),Pi)&&uit(x),Pr(e.f,v,t)}}function ZIn(e,t,n){var s,a,u,p,v,m,x,_,A,I;switch(Bn(n,"Node promotion heuristic",1),e.g=t,iNn(e),e.q=h(U(t,(tt(),Ane)),260),_=h(U(e.g,yke),19).a,u=new eRe,e.q.g){case 2:case 1:bx(e,u);break;case 3:for(e.q=(By(),sH),bx(e,u),m=0,v=new Q(e.a);v.a<v.c.c.length;)p=h(K(v),19),m=d.Math.max(m,p.a);m>e.j&&(e.q=BD,bx(e,u));break;case 4:for(e.q=(By(),sH),bx(e,u),x=0,a=new Q(e.b);a.a<a.c.c.length;)s=Je(K(a)),x=d.Math.max(x,(rn(s),s));x>e.k&&(e.q=FD,bx(e,u));break;case 6:I=Li(d.Math.ceil(e.f.length*_/100)),bx(e,new bYe(I));break;case 5:A=Li(d.Math.ceil(e.d*_/100)),bx(e,new vYe(A));break;default:bx(e,u)}UAn(e,t),An(n)}function R2t(e,t,n){var s,a,u,p;this.j=e,this.e=R2e(e),this.o=this.j.e,this.i=!!this.o,this.p=this.i?h(ut(n,Us(this.o).p),214):null,a=h(U(e,(He(),Uo)),21),this.g=a.Hc((ha(),El)),this.b=new qe,this.d=new aft(this.e),p=h(U(this.j,R5),230),this.q=D5n(t,p,this.e),this.k=new ost(this),u=Ef(se(ie(z5t,1),st,225,0,[this,this.d,this.k,this.q])),t==(_b(),jD)&&!ft(lt(U(e,(tt(),rm))))?(s=new $2e(this.e),u.c[u.c.length]=s,this.c=new p0e(s,p,h(this.q,402))):t==jD&&ft(lt(U(e,(tt(),rm))))?(s=new $2e(this.e),u.c[u.c.length]=s,this.c=new Gut(s,p,h(this.q,402))):this.c=new jZe(t,this),Ue(u,this.c),m2t(u,this.e),this.s=NNn(this.k)}function JIn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;for(A=h(rP((p=pr(new ng(t).a.d,0),new k4(p))),86),j=A?h(U(A,(fo(),sre)),86):null,a=1;!!A&&!!j;){for(m=0,we=0,n=A,s=j,v=0;v<a;v++)n=mK(n),s=mK(s),we+=Pe(Je(U(n,(fo(),g8)))),m+=Pe(Je(U(s,g8)));if(le=Pe(Je(U(j,(fo(),O0)))),fe=Pe(Je(U(A,O0))),I=S0e(A,j),P=le+m+e.a+I-fe-we,0<P){for(x=t,_=0;!!x&&x!=s;)++_,x=h(U(x,dH),86);if(x)for(oe=P/_,x=t;x!=s;)J=Pe(Je(U(x,O0)))+P,Fe(x,O0,J),V=Pe(Je(U(x,g8)))+P,Fe(x,g8,V),P-=oe,x=h(U(x,dH),86);else return}++a,A.d.b==0?A=gve(new ng(t),a):A=h(rP((u=pr(new ng(A).a.d,0),new k4(u))),86),j=A?h(U(A,sre),86):null}}function j2t(e,t){var n,s,a,u,p,v,m,x,_,A;for(m=!0,a=0,x=e.f[t.p],_=t.o.b+e.n,n=e.c[t.p][2],dl(e.a,x,Ke(h(ut(e.a,x),19).a-1+n)),dl(e.b,x,Pe(Je(ut(e.b,x)))-_+n*e.e),++x,x>=e.i?(++e.i,Ue(e.a,Ke(1)),Ue(e.b,_)):(s=e.c[t.p][1],dl(e.a,x,Ke(h(ut(e.a,x),19).a+1-s)),dl(e.b,x,Pe(Je(ut(e.b,x)))+_-s*e.e)),(e.q==(By(),BD)&&(h(ut(e.a,x),19).a>e.j||h(ut(e.a,x-1),19).a>e.j)||e.q==FD&&(Pe(Je(ut(e.b,x)))>e.k||Pe(Je(ut(e.b,x-1)))>e.k))&&(m=!1),p=new Sn(Dn(za(t).a.Kc(),new L));rr(p);)u=h(Un(p),17),v=u.c.i,e.f[v.p]==x&&(A=j2t(e,v),a=a+h(A.a,19).a,m=m&&ft(lt(A.b)));return e.f[t.p]=x,a=a+e.c[t.p][0],new As(Ke(a),(on(),!!m))}function Bve(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe;for(A=new zn,p=new qe,o0t(e,n,e.d.fg(),p,A),o0t(e,s,e.d.gg(),p,A),e.b=.2*(V=X0t(Xa(new Vt(null,new Wt(p,16)),new cHe)),J=X0t(Xa(new Vt(null,new Wt(p,16)),new uHe)),d.Math.min(V,J)),u=0,v=0;v<p.c.length-1;v++)for(m=(Kt(v,p.c.length),h(p.c[v],112)),j=v+1;j<p.c.length;j++)u+=Mve(e,m,(Kt(j,p.c.length),h(p.c[j],112)));for(I=h(U(t,(He(),R5)),230),u>=2&&(oe=wgt(p,!0,I),!e.e&&(e.e=new yWe(e)),Lxn(e.e,oe,p,e.b)),Z1t(p,I),pOn(p),P=-1,_=new Q(p);_.a<_.c.c.length;)x=h(K(_),112),!(d.Math.abs(x.s-x.c)<l1)&&(P=d.Math.max(P,x.o),e.d.dg(x,a,e.c));return e.d.a.a.$b(),P+1}function $2t(e,t){var n,s,a,u,p;n=Pe(Je(U(t,(tt(),Mh)))),n<2&&Fe(t,Mh,2),s=h(U(t,Ju),103),s==(la(),Q1)&&Fe(t,Ju,bF(t)),a=h(U(t,Sxt),19),a.a==0?Fe(t,(He(),R5),new mX):Fe(t,(He(),R5),new vB(a.a)),u=lt(U(t,D_)),u==null&&Fe(t,D_,(on(),Me(U(t,Hd))===Me((Od(),y8)))),Ti(new Vt(null,new Wt(t.a,16)),new Dle(e)),Ti(Xa(new Vt(null,new Wt(t.b,16)),new $O),new Lle(e)),p=new O2t(t),Fe(t,(He(),d3),p),hB(e.a),P1(e.a,(Zs(),w1),h(U(t,im),246)),P1(e.a,Hg,h(U(t,kke),246)),P1(e.a,oc,h(U(t,M_),246)),P1(e.a,Go,h(U(t,X$),246)),P1(e.a,Pa,S5n(h(U(t,Hd),218))),KJe(e.a,YNn(t)),Fe(t,bne,vR(e.a,t))}function eOn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;return I=e.c[t],P=e.c[n],j=h(U(I,(He(),B5)),15),!!j&&j.gc()!=0&&j.Hc(P)||(V=I.k!=(bn(),ds)&&P.k!=ds,J=h(U(I,rv),10),oe=h(U(P,rv),10),fe=J!=oe,le=!!J&&J!=I||!!oe&&oe!=P,we=VX(I,(Qe(),sn)),Ne=VX(P,Fn),le=le|(VX(I,Fn)||VX(P,sn)),Ve=le&&fe||we||Ne,V&&Ve)||I.k==(bn(),Vo)&&P.k==Wi||P.k==(bn(),Vo)&&I.k==Wi?!1:(_=e.c[t],u=e.c[n],a=F1t(e.e,_,u,(Qe(),un)),m=F1t(e.i,_,u,pn),qSn(e.f,_,u),x=aht(e.b,_,u)+h(a.a,19).a+h(m.a,19).a+e.f.d,v=aht(e.b,u,_)+h(a.b,19).a+h(m.b,19).a+e.f.b,e.a&&(A=h(U(_,Br),11),p=h(U(u,Br),11),s=k1t(e.g,A,p),x+=h(s.a,19).a,v+=h(s.b,19).a),x>v)}function tOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(n=h(U(e,(tt(),Ei)),98),p=e.f,u=e.d,v=p.a+u.b+u.c,m=0-u.d-e.c.b,_=p.b+u.d+u.a-e.c.b,x=new qe,A=new qe,a=new Q(t);a.a<a.c.c.length;){switch(s=h(K(a),10),n.g){case 1:case 2:case 3:JSn(s);break;case 4:I=h(U(s,ov),8),P=I?I.a:0,s.n.a=v*Pe(Je(U(s,(He(),iv))))-P,sF(s,!0,!1);break;case 5:j=h(U(s,ov),8),V=j?j.a:0,s.n.a=Pe(Je(U(s,(He(),iv))))-V,sF(s,!0,!1),p.a=d.Math.max(p.a,s.n.a+s.o.a/2)}switch(h(U(s,(He(),co)),61).g){case 1:s.n.b=m,x.c[x.c.length]=s;break;case 3:s.n.b=_,A.c[A.c.length]=s}}switch(n.g){case 1:case 2:Lht(x,e),Lht(A,e);break;case 3:Iht(x,e),Iht(A,e)}}function nOn(e,t){var n,s,a,u,p,v,m,x,_,A;for(_=new qe,A=new Jm,u=null,a=0,s=0;s<t.length;++s)switch(n=t[s],k6n(u,n)&&(a=Hpe(e,A,_,oH,a)),os(n,(He(),rv))&&(u=h(U(n,rv),10)),n.k.g){case 0:for(m=Efe(R4(Za(n,(Qe(),sn)),new Wue));FK(m);)p=h(X0e(m),11),e.d[p.p]=a++,_.c[_.c.length]=p;for(a=Hpe(e,A,_,oH,a),x=Efe(R4(Za(n,Fn),new Wue));FK(x);)p=h(X0e(x),11),e.d[p.p]=a++,_.c[_.c.length]=p;break;case 3:Za(n,bxe).dc()||(p=h(Za(n,bxe).Xb(0),11),e.d[p.p]=a++,_.c[_.c.length]=p),Za(n,oH).dc()||mg(A,n);break;case 1:for(v=Za(n,(Qe(),un)).Kc();v.Ob();)p=h(v.Pb(),11),e.d[p.p]=a++,_.c[_.c.length]=p;Za(n,pn).Jc(new $Ze(A,n))}return Hpe(e,A,_,oH,a),_}function H2t(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe;for(x=ki,_=ki,v=Fi,m=Fi,I=new Q(t.i);I.a<I.c.c.length;)A=h(K(I),65),a=h(h(_n(e.g,A.a),46).b,33),bf(a,A.b.c,A.b.d),x=d.Math.min(x,a.i),_=d.Math.min(_,a.j),v=d.Math.max(v,a.i+a.g),m=d.Math.max(m,a.j+a.f);for(P=h(pt(e.c,(uM(),rEt)),116),$b(e.c,v-x+(P.b+P.c),m-_+(P.d+P.a),!0,!0),Y2e(e.c,-x+P.b,-_+P.d),s=new Tn(bit(e.c));s.e!=s.i.gc();)n=h(On(s),79),p=Fy(n,!0,!0),j=H1(n),J=Sg(n),V=new dt(j.i+j.g/2,j.j+j.f/2),u=new dt(J.i+J.g/2,J.j+J.f/2),oe=ms(new dt(u.a,u.b),V),rx(oe,j.g,j.f),zr(V,oe),fe=ms(new dt(V.a,V.b),u),rx(fe,J.g,J.f),zr(u,fe),eP(p,V.a,V.b),JN(p,u.a,u.b)}function rOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;if(e.c=e.d,j=lt(U(t,(tt(),Axt))),P=j==null||(rn(j),j),u=h(U(t,(He(),Uo)),21).Hc((ha(),El)),a=h(U(t,Ei),98),n=!(a==(_s(),x2)||a==ed||a==qo),P&&(n||!u)){for(A=new Q(t.a);A.a<A.c.c.length;)x=h(K(A),10),x.p=0;for(I=new qe,_=new Q(t.a);_.a<_.c.c.length;)if(x=h(K(_),10),s=Ppt(e,x,null),s){for(m=new dge,Oa(m,t),Fe(m,em,h(s.b,21)),qde(m.d,t.d),Fe(m,o8,null),v=h(s.a,15).Kc();v.Ob();)p=h(v.Pb(),10),Ue(m.a,p),p.a=m;I.Fc(m)}u&&(Me(U(t,$5))===Me((Sb(),Lte))?e.c=e.b:e.c=e.a)}else I=new _u(se(ie(_5t,1),Pvt,37,0,[t]));return Me(U(t,$5))!==Me((Sb(),i3))&&(Rt(),I.ad(new CBe)),I}function z2t(e){J2(e,new Xp(xhe(fN(K2(q2(W2(Y2(new mp,S0),"ELK Mr. Tree"),"Tree-based algorithm provided by the Eclipse Layout Kernel. Computes a spanning tree of the input graph and arranges all nodes according to the resulting parent-children hierarchy. I pity the fool who doesn't use Mr. Tree Layout."),new bHe),kwt),Lt((ux(),Zre))))),et(e,S0,Vb,Txe),et(e,S0,zw,20),et(e,S0,k5,Mx),et(e,S0,UM,Ke(1)),et(e,S0,H9,(on(),!0)),et(e,S0,eD,at(Exe)),et(e,S0,Yy,at(K8t)),et(e,S0,E5,at(X8t)),et(e,S0,x5,at(Q8t)),et(e,S0,Dx,at(W8t)),et(e,S0,z9,at(_xe)),et(e,S0,Lx,at(J8t)),et(e,S0,nye,at(t7t)),et(e,S0,rye,at(Cxe))}function iOn(e){e.q||(e.q=!0,e.p=so(e,0),e.a=so(e,1),Qr(e.a,0),e.f=so(e,2),Qr(e.f,1),vi(e.f,2),e.n=so(e,3),vi(e.n,3),vi(e.n,4),vi(e.n,5),vi(e.n,6),e.g=so(e,4),Qr(e.g,7),vi(e.g,8),e.c=so(e,5),Qr(e.c,7),Qr(e.c,8),e.i=so(e,6),Qr(e.i,9),Qr(e.i,10),Qr(e.i,11),Qr(e.i,12),vi(e.i,13),e.j=so(e,7),Qr(e.j,9),e.d=so(e,8),Qr(e.d,3),Qr(e.d,4),Qr(e.d,5),Qr(e.d,6),vi(e.d,7),vi(e.d,8),vi(e.d,9),vi(e.d,10),e.b=so(e,9),vi(e.b,0),vi(e.b,1),e.e=so(e,10),vi(e.e,1),vi(e.e,2),vi(e.e,3),vi(e.e,4),Qr(e.e,5),Qr(e.e,6),Qr(e.e,7),Qr(e.e,8),Qr(e.e,9),Qr(e.e,10),vi(e.e,11),e.k=so(e,11),vi(e.k,0),vi(e.k,1),e.o=kr(e,12),e.s=kr(e,13))}function Fve(e,t){t.dc()&&O1(e.j,!0,!0,!0,!0),Nr(t,(Qe(),Ph))&&O1(e.j,!0,!0,!0,!1),Nr(t,_l)&&O1(e.j,!1,!0,!0,!0),Nr(t,tl)&&O1(e.j,!0,!0,!1,!0),Nr(t,Ql)&&O1(e.j,!0,!1,!0,!0),Nr(t,eu)&&O1(e.j,!1,!0,!0,!1),Nr(t,Tl)&&O1(e.j,!1,!0,!1,!0),Nr(t,nl)&&O1(e.j,!0,!1,!1,!0),Nr(t,Bh)&&O1(e.j,!0,!1,!0,!1),Nr(t,Cc)&&O1(e.j,!0,!0,!0,!0),Nr(t,uc)&&O1(e.j,!0,!0,!0,!0),Nr(t,Cc)&&O1(e.j,!0,!0,!0,!0),Nr(t,Tc)&&O1(e.j,!0,!0,!0,!0),Nr(t,Sc)&&O1(e.j,!0,!0,!0,!0),Nr(t,tu)&&O1(e.j,!0,!0,!0,!0),Nr(t,Yo)&&O1(e.j,!0,!0,!0,!0)}function sOn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J;for(u=new qe,x=new Q(s);x.a<x.c.c.length;)if(v=h(K(x),441),p=null,v.f==(ua(),ec))for(j=new Q(v.e);j.a<j.c.c.length;)P=h(K(j),17),J=P.d.i,Us(J)==t?rut(e,t,v,P,v.b,P.d):!n||Fk(J,n)?j9n(e,t,v,s,P):(I=tZ(e,t,n,P,v.b,ec,p),I!=p&&(u.c[u.c.length]=I),I.c&&(p=I));else for(A=new Q(v.e);A.a<A.c.c.length;)if(_=h(K(A),17),V=_.c.i,Us(V)==t)rut(e,t,v,_,_.c,v.b);else{if(!n||Fk(V,n))continue;I=tZ(e,t,n,_,v.b,Jc,p),I!=p&&(u.c[u.c.length]=I),I.c&&(p=I)}for(m=new Q(u);m.a<m.c.c.length;)v=h(K(m),441),Ha(t.a,v.a,0)!=-1||Ue(t.a,v.a),v.c&&(a.c[a.c.length]=v)}function aOn(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(x=new qe,m=new Q(t.a);m.a<m.c.c.length;)for(p=h(K(m),10),I=Za(p,(Qe(),pn)).Kc();I.Ob();)for(A=h(I.Pb(),11),a=new Q(A.g);a.a<a.c.c.length;)s=h(K(a),17),!(!Xs(s)&&s.c.i.c==s.d.i.c||Xs(s)||s.d.i.c!=n)&&(x.c[x.c.length]=s);for(v=Rp(n.a).Kc();v.Ob();)for(p=h(v.Pb(),10),I=Za(p,(Qe(),un)).Kc();I.Ob();)for(A=h(I.Pb(),11),a=new Q(A.e);a.a<a.c.c.length;)if(s=h(K(a),17),!(!Xs(s)&&s.c.i.c==s.d.i.c||Xs(s)||s.c.i.c!=t)){for(_=new Ms(x,x.c.length),u=(kn(_.b>0),h(_.a.Xb(_.c=--_.b),17));u!=s&&_.b>0;)e.a[u.p]=!0,e.a[s.p]=!0,u=(kn(_.b>0),h(_.a.Xb(_.c=--_.b),17));_.b>0&&Mu(_)}}function G2t(e,t,n){var s,a,u,p,v,m,x,_,A;if(e.a!=t.Aj())throw ne(new an(Px+t.ne()+Wb));if(s=v0((jc(),Ps),t).$k(),s)return s.Aj().Nh().Ih(s,n);if(p=v0(Ps,t).al(),p){if(n==null)return null;if(v=h(n,15),v.dc())return"";for(A=new s0,u=v.Kc();u.Ob();)a=u.Pb(),va(A,p.Aj().Nh().Ih(p,a)),A.a+=" ";return TY(A,A.a.length-1)}if(_=v0(Ps,t).bl(),!_.dc()){for(x=_.Kc();x.Ob();)if(m=h(x.Pb(),148),m.wj(n))try{if(A=m.Aj().Nh().Ih(m,n),A!=null)return A}catch(I){if(I=ui(I),!ue(I,102))throw ne(I)}throw ne(new an("Invalid value: '"+n+"' for datatype :"+t.ne()))}return h(t,834).Fj(),n==null?null:ue(n,172)?""+h(n,172).a:uu(n)==Bj?bet(uT[0],h(n,199)):Ga(n)}function oOn(e){var t,n,s,a,u,p,v,m,x,_;for(x=new di,v=new di,u=new Q(e);u.a<u.c.c.length;)s=h(K(u),128),s.v=0,s.n=s.i.c.length,s.u=s.t.c.length,s.n==0&&Ai(x,s,x.c.b,x.c),s.u==0&&s.r.a.gc()==0&&Ai(v,s,v.c.b,v.c);for(p=-1;x.b!=0;)for(s=h(ZX(x,0),128),n=new Q(s.t);n.a<n.c.c.length;)t=h(K(n),268),_=t.b,_.v=d.Math.max(_.v,s.v+1),p=d.Math.max(p,_.v),--_.n,_.n==0&&Ai(x,_,x.c.b,x.c);if(p>-1){for(a=pr(v,0);a.b!=a.d.c;)s=h(gr(a),128),s.v=p;for(;v.b!=0;)for(s=h(ZX(v,0),128),n=new Q(s.i);n.a<n.c.c.length;)t=h(K(n),268),m=t.a,m.r.a.gc()==0&&(m.v=d.Math.min(m.v,s.v-1),--m.u,m.u==0&&Ai(v,m,v.c.b,v.c))}}function V2t(e,t,n,s,a){var u,p,v,m;return m=ki,p=!1,v=Sve(e,ms(new dt(t.a,t.b),e),zr(new dt(n.a,n.b),a),ms(new dt(s.a,s.b),n)),u=!!v&&!(d.Math.abs(v.a-e.a)<=qb&&d.Math.abs(v.b-e.b)<=qb||d.Math.abs(v.a-t.a)<=qb&&d.Math.abs(v.b-t.b)<=qb),v=Sve(e,ms(new dt(t.a,t.b),e),n,a),v&&((d.Math.abs(v.a-e.a)<=qb&&d.Math.abs(v.b-e.b)<=qb)==(d.Math.abs(v.a-t.a)<=qb&&d.Math.abs(v.b-t.b)<=qb)||u?m=d.Math.min(m,NE(ms(v,n))):p=!0),v=Sve(e,ms(new dt(t.a,t.b),e),s,a),!!v&&(p||(d.Math.abs(v.a-e.a)<=qb&&d.Math.abs(v.b-e.b)<=qb)==(d.Math.abs(v.a-t.a)<=qb&&d.Math.abs(v.b-t.b)<=qb)||u)&&(m=d.Math.min(m,NE(ms(v,s)))),m}function U2t(e){J2(e,new Xp(fN(K2(q2(W2(Y2(new mp,Ng),Ovt),"Minimizes the stress within a layout using stress majorization. Stress exists if the euclidean distance between a pair of nodes doesn't match their graph theoretic distance, that is, the shortest path between the two nodes. The method allows to specify individual edge lengths."),new Qm),Xu))),et(e,Ng,qM,at(H4e)),et(e,Ng,FR,(on(),!0)),et(e,Ng,Yy,at(r5t)),et(e,Ng,E5,at(i5t)),et(e,Ng,x5,at(s5t)),et(e,Ng,Dx,at(n5t)),et(e,Ng,z9,at(G4e)),et(e,Ng,Lx,at(a5t)),et(e,Ng,Lwe,at($4e)),et(e,Ng,Owe,at(R4e)),et(e,Ng,Nwe,at(j4e)),et(e,Ng,Pwe,at(z4e)),et(e,Ng,Iwe,at(e$))}function cOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;for(Bn(t,"Interactive crossing minimization",1),p=0,u=new Q(e.b);u.a<u.c.c.length;)s=h(K(u),29),s.p=p++;for(I=R2e(e),J=new Ehe(I.length),m2t(new _u(se(ie(z5t,1),st,225,0,[J])),I),V=0,p=0,a=new Q(e.b);a.a<a.c.c.length;){for(s=h(K(a),29),n=0,A=0,_=new Q(s.a);_.a<_.c.c.length;)for(m=h(K(_),10),m.n.a>0&&(n+=m.n.a+m.o.a/2,++A),j=new Q(m.j);j.a<j.c.c.length;)P=h(K(j),11),P.p=V++;for(A>0&&(n/=A),oe=pe(ks,ka,25,s.a.c.length,15,1),v=0,x=new Q(s.a);x.a<x.c.c.length;)m=h(K(x),10),m.p=v++,oe[m.p]=XIn(m,n),m.k==(bn(),ds)&&Fe(m,(He(),y6e),oe[m.p]);Rt(),hs(s.a,new ZYe(oe)),Mpt(J,I,p,!0),++p}An(t)}function I9(e,t){var n,s,a,u,p,v,m,x,_;if(t.e==5){F2t(e,t);return}if(x=t,!(x.b==null||e.b==null)){for(Ny(e),D9(e),Ny(x),D9(x),n=pe(Hn,hr,25,e.b.length+x.b.length,15,1),_=0,s=0,p=0;s<e.b.length&&p<x.b.length;)if(a=e.b[s],u=e.b[s+1],v=x.b[p],m=x.b[p+1],u<v)n[_++]=e.b[s++],n[_++]=e.b[s++];else if(u>=v&&a<=m)v<=a&&u<=m?s+=2:v<=a?(e.b[s]=m+1,p+=2):u<=m?(n[_++]=a,n[_++]=v-1,s+=2):(n[_++]=a,n[_++]=v-1,e.b[s]=m+1,p+=2);else if(m<a)p+=2;else throw ne(new Ya("Token#subtractRanges(): Internal Error: ["+e.b[s]+","+e.b[s+1]+"] - ["+x.b[p]+","+x.b[p+1]+"]"));for(;s<e.b.length;)n[_++]=e.b[s++],n[_++]=e.b[s++];e.b=pe(Hn,hr,25,_,15,1),Ao(n,0,e.b,0,_)}}function uOn(e){var t,n,s,a,u,p,v;if(!e.A.dc()){if(e.A.Hc((Iu(),cL))&&(h(wa(e.b,(Qe(),sn)),124).k=!0,h(wa(e.b,Fn),124).k=!0,t=e.q!=(_s(),ed)&&e.q!=qo,hle(h(wa(e.b,pn),124),t),hle(h(wa(e.b,un),124),t),hle(e.g,t),e.A.Hc(E2)&&(h(wa(e.b,sn),124).j=!0,h(wa(e.b,Fn),124).j=!0,h(wa(e.b,pn),124).k=!0,h(wa(e.b,un),124).k=!0,e.g.k=!0)),e.A.Hc(oL))for(e.a.j=!0,e.a.k=!0,e.g.j=!0,e.g.k=!0,v=e.B.Hc((fu(),oT)),a=LX(),u=0,p=a.length;u<p;++u)s=a[u],n=h(wa(e.i,s),306),n&&(qpe(s)?(n.j=!0,n.k=!0):(n.j=!v,n.k=!v));e.A.Hc(bm)&&e.B.Hc((fu(),lL))&&(e.g.j=!0,e.g.j=!0,e.a.j||(e.a.j=!0,e.a.k=!0,e.a.e=!0))}}function lOn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;for(s=new Q(e.e.b);s.a<s.c.c.length;)for(n=h(K(s),29),u=new Q(n.a);u.a<u.c.c.length;)if(a=h(K(u),10),P=e.i[a.p],x=P.a.e,m=P.d.e,a.n.b=x,oe=m-x-a.o.b,t=JQ(a),I=(Tw(),(a.q?a.q:(Rt(),Rt(),K1))._b((tt(),av))?A=h(U(a,av),197):A=h(U(Us(a),L_),197),A),t&&(I==w3||I==v3)&&(a.o.b+=oe),t&&(I==$ne||I==w3||I==v3)){for(V=new Q(a.j);V.a<V.c.c.length;)j=h(K(V),11),(Qe(),uc).Hc(j.j)&&(_=h(_n(e.k,j),121),j.n.b=_.e-x);for(v=new Q(a.b);v.a<v.c.c.length;)p=h(K(v),70),J=h(U(a,sv),21),J.Hc((Iw(),Xl))?p.n.b+=oe:J.Hc(J1)&&(p.n.b+=oe/2);(I==w3||I==v3)&&Za(a,(Qe(),Fn)).Jc(new gWe(oe))}}function q2t(e,t,n){var s,a,u,p,v,m,x,_,A,I,P;if(!e.b)return!1;for(p=null,I=null,m=new wK(null,null),a=1,m.a[1]=e.b,A=m;A.a[a];)x=a,v=I,I=A,A=A.a[a],s=e.a.ue(t,A.d),a=s<0?0:1,s==0&&(!n.c||Bo(A.e,n.d))&&(p=A),!(!!A&&A.b)&&!Wv(A.a[a])&&(Wv(A.a[1-a])?I=I.a[x]=KB(A,a):Wv(A.a[1-a])||(P=I.a[1-x],P&&(!Wv(P.a[1-x])&&!Wv(P.a[x])?(I.b=!1,P.b=!0,A.b=!0):(u=v.a[1]==I?1:0,Wv(P.a[x])?v.a[u]=Jst(I,x):Wv(P.a[1-x])&&(v.a[u]=KB(I,x)),A.b=v.a[u].b=!0,v.a[u].a[0].b=!1,v.a[u].a[1].b=!1))));return p&&(n.b=!0,n.d=p.e,A!=p&&(_=new wK(A.d,A.e),D9n(e,m,p,_),I==p&&(I=_)),I.a[I.a[1]==A?1:0]=A.a[A.a[0]?0:1],--e.c),e.b=m.a[1],e.b&&(e.b.b=!1),n.b}function hOn(e){var t,n,s,a,u,p,v,m,x,_,A,I;for(a=new Q(e.a.a.b);a.a<a.c.c.length;)for(s=h(K(a),57),m=s.c.Kc();m.Ob();)v=h(m.Pb(),57),s.a!=v.a&&(lg(e.a.d)?A=e.a.g.Oe(s,v):A=e.a.g.Pe(s,v),u=s.b.a+s.d.b+A-v.b.a,u=d.Math.ceil(u),u=d.Math.max(0,u),ege(s,v)?(p=ib(new _p,e.d),x=Li(d.Math.ceil(v.b.a-s.b.a)),t=x-(v.b.a-s.b.a),_=W4(s).a,n=s,_||(_=W4(v).a,t=-t,n=v),_&&(n.b.a-=t,_.n.a-=t),Th(ph(gh(bh(dh(new Il,d.Math.max(0,x)),1),p),e.c[s.a.d])),Th(ph(gh(bh(dh(new Il,d.Math.max(0,-x)),1),p),e.c[v.a.d]))):(I=1,(ue(s.g,145)&&ue(v.g,10)||ue(v.g,145)&&ue(s.g,10))&&(I=2),Th(ph(gh(bh(dh(new Il,Li(u)),I),e.c[s.a.d]),e.c[v.a.d]))))}function Y2t(e,t,n){var s,a,u,p,v,m,x,_,A,I;if(n)for(s=-1,_=new Ms(t,0);_.b<_.d.gc();){if(v=(kn(_.b<_.d.gc()),h(_.d.Xb(_.c=_.b++),10)),A=e.c[v.c.p][v.p].a,A==null){for(p=s+1,u=new Ms(t,_.b);u.b<u.d.gc();)if(I=e0n(e,(kn(u.b<u.d.gc()),h(u.d.Xb(u.c=u.b++),10))).a,I!=null){p=(rn(I),I);break}A=(s+p)/2,e.c[v.c.p][v.p].a=A,e.c[v.c.p][v.p].d=(rn(A),A),e.c[v.c.p][v.p].b=1}s=(rn(A),A)}else{for(a=0,x=new Q(t);x.a<x.c.c.length;)v=h(K(x),10),e.c[v.c.p][v.p].a!=null&&(a=d.Math.max(a,Pe(e.c[v.c.p][v.p].a)));for(a+=2,m=new Q(t);m.a<m.c.c.length;)v=h(K(m),10),e.c[v.c.p][v.p].a==null&&(A=hu(e.i,24)*HM*a-1,e.c[v.c.p][v.p].a=A,e.c[v.c.p][v.p].d=A,e.c[v.c.p][v.p].b=1)}}function fOn(){wr(wm,new dGe),wr(dr,new _Ge),wr(Jl,new PGe),wr(af,new HGe),wr(rie,new zGe),wr(QH,new GGe),wr(qd,new VGe),wr(cT,new UGe),wr(dL,new rGe),wr(Qre,new iGe),wr(Jg,new sGe),wr(eh,new aGe),wr(sf,new oGe),wr(dv,new cGe),wr(mm,new uGe),wr(Ac,new lGe),wr(vm,new hGe),wr(uo,new fGe),wr(pa,new gGe),wr(lc,new pGe),wr(es,new bGe),wr(ie(Vc,1),new vGe),wr(A5,new wGe),wr(f_,new mGe),wr(Bj,new yGe),wr(YEe,new kGe),wr(Ts,new xGe),wr(cEe,new EGe),wr(hEe,new TGe),wr(NEe,new CGe),wr(ZH,new SGe),wr(zx,new AGe),wr(Ws,new MGe),wr(f3e,new DGe),wr(Xb,new LGe),wr(d3e,new IGe),wr(DEe,new OGe),wr(WEe,new NGe),wr(Qb,new BGe),wr(it,new FGe),wr(lEe,new RGe),wr(KEe,new jGe)}function dOn(e,t,n){var s,a,u,p,v,m,x,_,A;for(!n&&(n=K5n(t.q.getTimezoneOffset())),a=(t.q.getTimezoneOffset()-n.a)*6e4,v=new c1e(Vs(xc(t.q.getTime()),a)),m=v,v.q.getTimezoneOffset()!=t.q.getTimezoneOffset()&&(a>0?a-=864e5:a+=864e5,m=new c1e(Vs(xc(t.q.getTime()),a))),_=new Kv,x=e.a.length,u=0;u<x;)if(s=Is(e.a,u),s>=97&&s<=122||s>=65&&s<=90){for(p=u+1;p<x&&Is(e.a,p)==s;++p);oPn(_,s,p-u,v,m,n),u=p}else if(s==39){if(++u,u<x&&Is(e.a,u)==39){_.a+="'",++u;continue}for(A=!1;!A;){for(p=u;p<x&&Is(e.a,p)!=39;)++p;if(p>=x)throw ne(new an("Missing trailing '"));p+1<x&&Is(e.a,p+1)==39?++p:A=!0,or(_,$u(e.a,u,p)),u=p+1}}else _.a+=String.fromCharCode(s),++u;return _.a}function gOn(e){var t,n,s,a,u,p,v,m;for(t=null,s=new Q(e);s.a<s.c.c.length;)n=h(K(s),233),Pe(wf(n.g,n.d[0]).a),n.b=null,!!n.e&&n.e.gc()>0&&n.c==0&&(!t&&(t=new qe),t.c[t.c.length]=n);if(t)for(;t.c.length!=0;){if(n=h(f0(t,0),233),!!n.b&&n.b.c.length>0){for(u=(!n.b&&(n.b=new qe),new Q(n.b));u.a<u.c.c.length;)if(a=h(K(u),233),bS(wf(a.g,a.d[0]).a)==bS(wf(n.g,n.d[0]).a)){if(Ha(e,a,0)>Ha(e,n,0))return new As(a,n)}else if(Pe(wf(a.g,a.d[0]).a)>Pe(wf(n.g,n.d[0]).a))return new As(a,n)}for(v=(!n.e&&(n.e=new qe),n.e).Kc();v.Ob();)p=h(v.Pb(),233),m=(!p.b&&(p.b=new qe),p.b),fw(0,m.c.length),J7(m.c,0,n),p.c==m.c.length&&(t.c[t.c.length]=p)}return null}function W2t(e,t){var n,s,a,u,p,v,m,x,_;if(e==null)return _c;if(m=t.a.zc(e,t),m!=null)return"[...]";for(n=new $p(Js,"[","]"),a=e,u=0,p=a.length;u<p;++u)s=a[u],s!=null&&(uu(s).i&4)!=0?Array.isArray(s)&&(_=SA(s),!(_>=14&&_<=16))?t.a._b(s)?(n.a?or(n.a,n.b):n.a=new ju(n.d),gE(n.a,"[...]")):(v=Fp(s),x=new CE(t),Cd(n,W2t(v,x))):ue(s,177)?Cd(n,iTn(h(s,177))):ue(s,190)?Cd(n,YEn(h(s,190))):ue(s,195)?Cd(n,r_n(h(s,195))):ue(s,2012)?Cd(n,WEn(h(s,2012))):ue(s,48)?Cd(n,rTn(h(s,48))):ue(s,364)?Cd(n,vTn(h(s,364))):ue(s,832)?Cd(n,nTn(h(s,832))):ue(s,104)&&Cd(n,tTn(h(s,104))):Cd(n,s==null?_c:Ga(s));return n.a?n.e.length==0?n.a.a:n.a.a+(""+n.e):n.c}function K2t(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(v=Fy(t,!1,!1),oe=hM(v),s&&(oe=GA(oe)),le=Pe(Je(pt(t,(l9(),wte)))),J=(kn(oe.b!=0),h(oe.a.a.c,8)),A=h(Zh(oe,1),8),oe.b>2?(_=new qe,Vi(_,new F1(oe,1,oe.b)),u=Abt(_,le+e.a),fe=new EQ(u),Oa(fe,t),n.c[n.c.length]=fe):s?fe=h(_n(e.b,H1(t)),266):fe=h(_n(e.b,Sg(t)),266),m=H1(t),s&&(m=Sg(t)),p=E_n(J,m),x=le+e.a,p.a?(x+=d.Math.abs(J.b-A.b),V=new dt(A.a,(A.b+J.b)/2)):(x+=d.Math.abs(J.a-A.a),V=new dt((A.a+J.a)/2,A.b)),s?Pr(e.d,t,new o2e(fe,p,V,x)):Pr(e.c,t,new o2e(fe,p,V,x)),Pr(e.b,t,fe),j=(!t.n&&(t.n=new Ye(Ua,t,1,7)),t.n),P=new Tn(j);P.e!=P.i.gc();)I=h(On(P),137),a=xM(e,I,!0,0,0),n.c[n.c.length]=a}function pOn(e){var t,n,s,a,u,p,v,m,x,_;for(x=new qe,v=new qe,p=new Q(e);p.a<p.c.c.length;)a=h(K(p),112),wq(a,a.f.c.length),KO(a,a.k.c.length),a.d==0&&(x.c[x.c.length]=a),a.i==0&&a.e.b==0&&(v.c[v.c.length]=a);for(s=-1;x.c.length!=0;)for(a=h(f0(x,0),112),n=new Q(a.k);n.a<n.c.c.length;)t=h(K(n),129),_=t.b,gle(_,d.Math.max(_.o,a.o+1)),s=d.Math.max(s,_.o),wq(_,_.d-1),_.d==0&&(x.c[x.c.length]=_);if(s>-1){for(u=new Q(v);u.a<u.c.c.length;)a=h(K(u),112),a.o=s;for(;v.c.length!=0;)for(a=h(f0(v,0),112),n=new Q(a.f);n.a<n.c.c.length;)t=h(K(n),129),m=t.a,!(m.e.b>0)&&(gle(m,d.Math.min(m.o,a.o-1)),KO(m,m.i-1),m.i==0&&(v.c[v.c.length]=m))}}function wx(e,t,n){var s,a,u,p,v,m,x;if(x=e.c,!t&&(t=yEe),e.c=t,(e.Db&4)!=0&&(e.Db&1)==0&&(m=new fs(e,1,2,x,e.c),n?n.Ei(m):n=m),x!=t){if(ue(e.Cb,284))e.Db>>16==-10?n=h(e.Cb,284).nk(t,n):e.Db>>16==-15&&(!t&&(t=(Bt(),zf)),!x&&(x=(Bt(),zf)),e.Cb.nh()&&(m=new Sd(e.Cb,1,13,x,t,y0(cu(h(e.Cb,59)),e),!1),n?n.Ei(m):n=m));else if(ue(e.Cb,88))e.Db>>16==-23&&(ue(t,88)||(t=(Bt(),nh)),ue(x,88)||(x=(Bt(),nh)),e.Cb.nh()&&(m=new Sd(e.Cb,1,10,x,t,y0(Co(h(e.Cb,26)),e),!1),n?n.Ei(m):n=m));else if(ue(e.Cb,444))for(v=h(e.Cb,836),p=(!v.b&&(v.b=new tN(new Mq)),v.b),u=(s=new Gp(new xp(p.a).a),new nN(s));u.a.b;)a=h(kb(u.a).cd(),87),n=wx(a,eR(a,v),n)}return n}function bOn(e,t){var n,s,a,u,p,v,m,x,_,A,I;for(p=ft(lt(pt(e,(tt(),am)))),I=h(pt(e,cm),21),m=!1,x=!1,A=new Tn((!e.c&&(e.c=new Ye(bu,e,9,9)),e.c));A.e!=A.i.gc()&&(!m||!x);){for(u=h(On(A),118),v=0,a=gg(Cf(se(ie(Nf,1),st,20,0,[(!u.d&&(u.d=new qt(us,u,8,5)),u.d),(!u.e&&(u.e=new qt(us,u,7,4)),u.e)])));rr(a)&&(s=h(Un(a),79),_=p&&Pb(s)&&ft(lt(pt(s,d2))),n=C2t((!s.b&&(s.b=new qt(Pn,s,4,7)),s.b),u)?e==bi(Na(h(he((!s.c&&(s.c=new qt(Pn,s,5,8)),s.c),0),82))):e==bi(Na(h(he((!s.b&&(s.b=new qt(Pn,s,4,7)),s.b),0),82))),!((_||n)&&(++v,v>1))););(v>0||I.Hc((Qc(),Ud))&&(!u.n&&(u.n=new Ye(Ua,u,1,7)),u.n).i>0)&&(m=!0),v>1&&(x=!0)}m&&t.Fc((ha(),El)),x&&t.Fc((ha(),E_))}function X2t(e){var t,n,s,a,u,p,v,m,x,_,A,I;if(I=h(pt(e,(Tr(),m2)),21),I.dc())return null;if(v=0,p=0,I.Hc((Iu(),cL))){for(_=h(pt(e,Z_),98),s=2,n=2,a=2,u=2,t=bi(e)?h(pt(bi(e),hv),103):h(pt(e,hv),103),x=new Tn((!e.c&&(e.c=new Ye(bu,e,9,9)),e.c));x.e!=x.i.gc();)if(m=h(On(x),118),A=h(pt(m,T3),61),A==(Qe(),to)&&(A=kve(m,t),ya(m,T3,A)),_==(_s(),qo))switch(A.g){case 1:s=d.Math.max(s,m.i+m.g);break;case 2:n=d.Math.max(n,m.j+m.f);break;case 3:a=d.Math.max(a,m.i+m.g);break;case 4:u=d.Math.max(u,m.j+m.f)}else switch(A.g){case 1:s+=m.g+2;break;case 2:n+=m.f+2;break;case 3:a+=m.g+2;break;case 4:u+=m.f+2}v=d.Math.max(s,a),p=d.Math.max(n,u)}return $b(e,v,p,!0,!0)}function sZ(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;for(fe=h(Gu(FB(Jr(new Vt(null,new Wt(t.d,16)),new $Ye(n)),new HYe(n)),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)]))),15),A=Ir,_=$s,m=new Q(t.b.j);m.a<m.c.c.length;)v=h(K(m),11),v.j==n&&(A=d.Math.min(A,v.p),_=d.Math.max(_,v.p));if(A==Ir)for(p=0;p<fe.gc();p++)q0e(h(fe.Xb(p),101),n,p);else for(le=pe(Hn,hr,25,a.length,15,1),abn(le,le.length),oe=fe.Kc();oe.Ob();){for(J=h(oe.Pb(),101),u=h(_n(e.b,J),177),x=0,V=A;V<=_;V++)u[V]&&(x=d.Math.max(x,s[V]));if(J.i){for(P=J.i.c,we=new ns,I=0;I<a.length;I++)a[P][I]&&Qi(we,Ke(le[I]));for(;wd(we,Ke(x));)++x}for(q0e(J,n,x),j=A;j<=_;j++)u[j]&&(s[j]=x+1);J.i&&(le[J.i.c]=x)}}function vOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(a=null,s=new Q(t.a);s.a<s.c.c.length;)n=h(K(s),10),JQ(n)?u=(v=ib(xS(new _p,n),e.f),m=ib(xS(new _p,n),e.f),x=new Ude(n,!0,v,m),_=n.o.b,A=(Tw(),(n.q?n.q:(Rt(),Rt(),K1))._b((tt(),av))?I=h(U(n,av),197):I=h(U(Us(n),L_),197),I),P=1e4,A==v3&&(P=1),j=Th(ph(gh(dh(bh(new Il,P),Li(d.Math.ceil(_))),v),m)),A==w3&&Qi(e.d,j),Npt(e,Rp(Za(n,(Qe(),un))),x),Npt(e,Za(n,pn),x),x):u=(V=ib(xS(new _p,n),e.f),Ti(Jr(new Vt(null,new Wt(n.j,16)),new E$e),new zZe(e,V)),new Ude(n,!1,V,V)),e.i[n.p]=u,a&&(p=a.c.d.a+uy(e.n,a.c,n)+n.d.d,a.b||(p+=a.c.o.b),Th(ph(gh(bh(dh(new Il,Li(d.Math.ceil(p))),0),a.d),u.a))),a=u}function wOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(Bn(t,"Label dummy insertions",1),A=new qe,p=Pe(Je(U(e,(tt(),z5)))),x=Pe(Je(U(e,lm))),_=h(U(e,Ju),103),P=new Q(e.a);P.a<P.c.c.length;)for(I=h(K(P),10),u=new Sn(Dn(qi(I).a.Kc(),new L));rr(u);)if(a=h(Un(u),17),a.c.i!=a.d.i&&LY(a.b,O5t)){for(V=l4n(a),j=I1(a.b.c.length),n=sSn(e,a,V,j),A.c[A.c.length]=n,s=n.o,v=new Ms(a.b,0);v.b<v.d.gc();)m=(kn(v.b<v.d.gc()),h(v.d.Xb(v.c=v.b++),70)),Me(U(m,y1))===Me((Tf(),w8))&&(_==(la(),Gd)||_==zd?(s.a+=m.o.a+x,s.b=d.Math.max(s.b,m.o.b)):(s.a=d.Math.max(s.a,m.o.a),s.b+=m.o.b+x),j.c[j.c.length]=m,Mu(v));_==(la(),Gd)||_==zd?(s.a-=x,s.b+=p+V):s.b+=p-x+V}Vi(e.a,A),An(t)}function mOn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P;for(u=new k0t(t),A=oMn(e,t,u),P=d.Math.max(Pe(Je(U(t,(tt(),D0)))),1),_=new Q(A.a);_.a<_.c.c.length;)x=h(K(_),46),m=Mft(h(x.a,8),h(x.b,8),P),w=!0,w=w&ab(n,new dt(m.c,m.d)),w=w&ab(n,nw(new dt(m.c,m.d),m.b,0)),w=w&ab(n,nw(new dt(m.c,m.d),0,m.a)),w&ab(n,nw(new dt(m.c,m.d),m.b,m.a));switch(I=u.d,v=Mft(h(A.b.a,8),h(A.b.b,8),P),I==(Qe(),un)||I==pn?(s.c[I.g]=d.Math.min(s.c[I.g],v.d),s.b[I.g]=d.Math.max(s.b[I.g],v.d+v.a)):(s.c[I.g]=d.Math.min(s.c[I.g],v.c),s.b[I.g]=d.Math.max(s.b[I.g],v.c+v.b)),a=Fi,p=u.c.i.d,I.g){case 4:a=p.c;break;case 2:a=p.b;break;case 1:a=p.a;break;case 3:a=p.d}return s.a[I.g]=d.Math.max(s.a[I.g],a),u}function yOn(e){var t,n,s,a;if(n=e.D!=null?e.D:e.B,t=Zf(n,Ec(91)),t!=-1){s=n.substr(0,t),a=new s0;do a.a+="[";while((t=nk(n,91,++t))!=-1);Nt(s,g5)?a.a+="Z":Nt(s,r_)?a.a+="B":Nt(s,vj)?a.a+="C":Nt(s,i_)?a.a+="D":Nt(s,s_)?a.a+="F":Nt(s,a_)?a.a+="I":Nt(s,o_)?a.a+="J":Nt(s,c_)?a.a+="S":(a.a+="L",a.a+=""+s,a.a+=";");try{return null}catch(u){if(u=ui(u),!ue(u,60))throw ne(u)}}else if(Zf(n,Ec(46))==-1){if(Nt(n,g5))return vu;if(Nt(n,r_))return Vc;if(Nt(n,vj))return Cl;if(Nt(n,i_))return ks;if(Nt(n,s_))return Em;if(Nt(n,a_))return Hn;if(Nt(n,o_))return ap;if(Nt(n,c_))return D3}return null}function Q2t(e,t,n){var s,a,u,p,v,m,x,_;for(x=new Nd(n),Oa(x,t),Fe(x,(He(),Br),t),x.o.a=t.g,x.o.b=t.f,x.n.a=t.i,x.n.b=t.j,Ue(n.a,x),Pr(e.a,t,x),((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a).i!=0||ft(lt(pt(t,(tt(),am)))))&&Fe(x,g6e,(on(),!0)),m=h(U(n,Uo),21),_=h(U(x,(tt(),Ei)),98),_==(_s(),Qg)?Fe(x,Ei,$f):_!=$f&&m.Fc((ha(),i8)),s=h(U(n,Ju),103),v=new Tn((!t.c&&(t.c=new Ye(bu,t,9,9)),t.c));v.e!=v.i.gc();)p=h(On(v),118),ft(lt(pt(p,p2)))||KOn(e,p,x,m,s,_);for(u=new Tn((!t.n&&(t.n=new Ye(Ua,t,1,7)),t.n));u.e!=u.i.gc();)a=h(On(u),137),!ft(lt(pt(a,p2)))&&!!a.a&&Ue(x.b,hX(a));return ft(lt(U(x,A_)))&&m.Fc((ha(),L$)),ft(lt(U(x,q$)))&&(m.Fc((ha(),I$)),m.Fc(E_),Fe(x,Ei,$f)),x}function kOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln;v=h(_n(t.c,e),459),fe=t.a.c,m=t.a.c+t.a.b,xt=v.f,ln=v.a,p=xt<ln,V=new dt(fe,xt),le=new dt(m,ln),a=(fe+m)/2,J=new dt(a,xt),we=new dt(a,ln),u=ISn(e,xt,ln),Ve=kf(t.B),rt=new dt(a,u),wt=kf(t.D),n=t6n(se(ie(cs,1),Re,8,0,[Ve,rt,wt])),P=!1,oe=t.B.i,!!oe&&!!oe.c&&v.d&&(x=p&&oe.p<oe.c.a.c.length-1||!p&&oe.p>0,x?x&&(I=oe.p,p?++I:--I,A=h(ut(oe.c.a,I),10),s=Hlt(A),P=!(MQ(s,Ve,n[0])||mrt(s,Ve,n[0]))):P=!0),j=!1,Ne=t.D.i,!!Ne&&!!Ne.c&&v.e&&(_=p&&Ne.p>0||!p&&Ne.p<Ne.c.a.c.length-1,_?(I=Ne.p,p?--I:++I,A=h(ut(Ne.c.a,I),10),s=Hlt(A),j=!(MQ(s,n[0],wt)||mrt(s,n[0],wt))):j=!0),P&&j&&vr(e.a,rt),P||YE(e.a,se(ie(cs,1),Re,8,0,[V,J])),j||YE(e.a,se(ie(cs,1),Re,8,0,[we,le]))}function wR(e,t){var n,s,a,u,p,v,m,x;if(ue(e.Ug(),160)?(wR(h(e.Ug(),160),t),t.a+=" > "):t.a+="Root ",n=e.Tg().zb,Nt(n.substr(0,3),"Elk")?or(t,n.substr(3)):t.a+=""+n,a=e.zg(),a){or((t.a+=" ",t),a);return}if(ue(e,354)&&(x=h(e,137).a,x)){or((t.a+=" ",t),x);return}for(p=new Tn(e.Ag());p.e!=p.i.gc();)if(u=h(On(p),137),x=u.a,x){or((t.a+=" ",t),x);return}if(ue(e,352)&&(s=h(e,79),!s.b&&(s.b=new qt(Pn,s,4,7)),s.b.i!=0&&(!s.c&&(s.c=new qt(Pn,s,5,8)),s.c.i!=0))){for(t.a+=" (",v=new N4((!s.b&&(s.b=new qt(Pn,s,4,7)),s.b));v.e!=v.i.gc();)v.e>0&&(t.a+=Js),wR(h(On(v),160),t);for(t.a+=dJ,m=new N4((!s.c&&(s.c=new qt(Pn,s,5,8)),s.c));m.e!=m.i.gc();)m.e>0&&(t.a+=Js),wR(h(On(m),160),t);t.a+=")"}}function xOn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P;if(u=h(U(e,(He(),Br)),79),!!u){for(s=e.a,a=new _a(n),zr(a,fEn(e)),Fk(e.d.i,e.c.i)?(I=e.c,A=Qa(se(ie(cs,1),Re,8,0,[I.n,I.a])),ms(A,n)):A=kf(e.c),Ai(s,A,s.a,s.a.a),P=kf(e.d),U(e,wne)!=null&&zr(P,h(U(e,wne),8)),Ai(s,P,s.c.b,s.c),mw(s,a),p=Fy(u,!0,!0),UB(p,h(he((!u.b&&(u.b=new qt(Pn,u,4,7)),u.b),0),82)),qB(p,h(he((!u.c&&(u.c=new qt(Pn,u,5,8)),u.c),0),82)),CM(s,p),_=new Q(e.b);_.a<_.c.c.length;)x=h(K(_),70),v=h(U(x,Br),137),Eb(v,x.o.a),xb(v,x.o.b),bf(v,x.n.a+a.a,x.n.b+a.b),ya(v,(t5(),Fte),lt(U(x,Fte)));m=h(U(e,(tt(),Da)),74),m?(mw(m,a),ya(u,Da,m)):ya(u,Da,null),t==(Od(),W5)?ya(u,Hd,W5):ya(u,Hd,null)}}function EOn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe;for(P=t.c.length,I=0,A=new Q(e.b);A.a<A.c.c.length;)if(_=h(K(A),29),oe=_.a,oe.c.length!=0){for(J=new Q(oe),x=0,fe=null,a=h(K(J),10),u=null;a;){if(u=h(ut(t,a.p),257),u.c>=0){for(m=null,v=new Ms(_.a,x+1);v.b<v.d.gc()&&(p=(kn(v.b<v.d.gc()),h(v.d.Xb(v.c=v.b++),10)),m=h(ut(t,p.p),257),!(m.d==u.d&&m.c<u.c));)m=null;m&&(fe&&(dl(s,a.p,Ke(h(ut(s,a.p),19).a-1)),h(ut(n,fe.p),15).Mc(u)),u=r9n(u,a,P++),t.c[t.c.length]=u,Ue(n,new qe),fe?(h(ut(n,fe.p),15).Fc(u),Ue(s,Ke(1))):Ue(s,Ke(0)))}j=null,J.a<J.c.c.length&&(j=h(K(J),10),V=h(ut(t,j.p),257),h(ut(n,a.p),15).Fc(V),dl(s,j.p,Ke(h(ut(s,j.p),19).a+1))),u.d=I,u.c=x++,fe=a,a=j}++I}}function aZ(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;return m=e,_=ms(new dt(t.a,t.b),e),x=n,A=ms(new dt(s.a,s.b),n),I=m.a,J=m.b,j=x.a,fe=x.b,P=_.a,oe=_.b,V=A.a,le=A.b,a=V*oe-P*le,vf(),xh(d1),d.Math.abs(0-a)<=d1||a==0||isNaN(0)&&isNaN(a)?!1:(p=1/a*((I-j)*oe-(J-fe)*P),v=1/a*-(-(I-j)*le+(J-fe)*V),u=(xh(d1),(d.Math.abs(0-p)<=d1||p==0||isNaN(0)&&isNaN(p)?0:0<p?-1:0>p?1:tb(isNaN(0),isNaN(p)))<0&&(xh(d1),(d.Math.abs(p-1)<=d1||p==1||isNaN(p)&&isNaN(1)?0:p<1?-1:p>1?1:tb(isNaN(p),isNaN(1)))<0)&&(xh(d1),(d.Math.abs(0-v)<=d1||v==0||isNaN(0)&&isNaN(v)?0:0<v?-1:0>v?1:tb(isNaN(0),isNaN(v)))<0)&&(xh(d1),(d.Math.abs(v-1)<=d1||v==1||isNaN(v)&&isNaN(1)?0:v<1?-1:v>1?1:tb(isNaN(v),isNaN(1)))<0)),u)}function _On(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;for(A=new jde(new Ale(e));A.b!=A.c.a.d;)for(_=tct(A),v=h(_.d,56),t=h(_.e,56),p=v.Tg(),V=0,we=(p.i==null&&s1(p),p.i).length;V<we;++V)if(x=(u=(p.i==null&&s1(p),p.i),V>=0&&V<u.length?u[V]:null),x.Ij()&&!x.Jj()){if(ue(x,99))m=h(x,18),(m.Bb&go)==0&&(Ve=aa(m),!(!!Ve&&(Ve.Bb&go)!=0))&&lLn(e,m,v,t);else if(ia(),h(x,66).Oj()&&(n=(Ne=x,h(Ne?h(t,49).xh(Ne):null,153)),n))for(P=h(v.ah(x),153),s=n.gc(),J=0,j=P.gc();J<j;++J)if(I=P.il(J),ue(I,99)){if(le=P.jl(J),a=mb(e,le),a==null&&le!=null){if(fe=h(I,18),!e.b||(fe.Bb&go)!=0||!!aa(fe))continue;a=le}if(!n.dl(I,a)){for(oe=0;oe<s;++oe)if(n.il(oe)==I&&Me(n.jl(oe))===Me(a)){n.ii(n.gc()-1,oe),--s;break}}}else n.dl(P.il(J),P.jl(J))}}function TOn(e,t,n,s,a,u,p){var v,m,x,_,A,I,P,j,V,J,oe,fe,le;if(oe=MLn(t,n,e.g),a.n&&a.n&&!!u&&mh(a,yh(u),(Lu(),rl)),e.b)for(J=0;J<oe.c.length;J++)A=(Kt(J,oe.c.length),h(oe.c[J],200)),J!=0&&(P=(Kt(J-1,oe.c.length),h(oe.c[J-1],200)),pht(A,P.f+P.b+e.g)),jNn(J,oe,n,e.g),j7n(e,A),a.n&&!!u&&mh(a,yh(u),(Lu(),rl));else for(V=new Q(oe);V.a<V.c.c.length;)for(j=h(K(V),200),_=new Q(j.a);_.a<_.c.c.length;)x=h(K(_),187),fe=new e0e(x.s,x.t,e.g),Sge(fe,x),Ue(j.d,fe);return W8n(e,oe),a.n&&a.n&&!!u&&mh(a,yh(u),(Lu(),rl)),le=d.Math.max(e.d,s.a-(p.b+p.c)),I=d.Math.max(e.c,s.b-(p.d+p.a)),v=I-e.c,e.e&&e.f&&(m=le/I,m<e.a?le=I*e.a:v+=le/e.a-I),e.e&&yxn(oe,le,v),a.n&&a.n&&!!u&&mh(a,yh(u),(Lu(),rl)),new nW(e.a,le,e.c+v,(fx(),K_))}function COn(e){var t,n,s,a,u,p,v,m,x,_,A;for(e.j=pe(Hn,hr,25,e.g,15,1),e.o=new qe,Ti(Xa(new Vt(null,new Wt(e.e.b,16)),new D$e),new pWe(e)),e.a=pe(vu,Y1,25,e.b,16,1),WA(new Vt(null,new Wt(e.e.b,16)),new vWe(e)),s=(A=new qe,Ti(Jr(Xa(new Vt(null,new Wt(e.e.b,16)),new I$e),new bWe(e)),new GZe(e,A)),A),m=new Q(s);m.a<m.c.c.length;)if(v=h(K(m),508),!(v.c.length<=1)){if(v.c.length==2){wCn(v),JQ((Kt(0,v.c.length),h(v.c[0],17)).d.i)||Ue(e.o,v);continue}if(!(VEn(v)||O_n(v,new L$e)))for(x=new Q(v),a=null;x.a<x.c.c.length;)t=h(K(x),17),n=e.c[t.p],!a||x.a>=x.c.c.length?_=m0e((bn(),Wi),ds):_=m0e((bn(),ds),ds),_*=2,u=n.a.g,n.a.g=d.Math.max(u,u+(_-u)),p=n.b.g,n.b.g=d.Math.max(p,p+(_-p)),a=t}}function SOn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne;for(Ne=vnt(e),_=new qe,v=e.c.length,A=v-1,I=v+1;Ne.a.c!=0;){for(;n.b!=0;)le=(kn(n.b!=0),h(pl(n,n.a.a),112)),FE(Ne.a,le)!=null,le.g=A--,xve(le,t,n,s);for(;t.b!=0;)we=(kn(t.b!=0),h(pl(t,t.a.a),112)),FE(Ne.a,we)!=null,we.g=I++,xve(we,t,n,s);for(x=$s,oe=(p=new EE(new yE(new uS(Ne.a).a).b),new lS(p));TS(oe.a.a);){if(J=(u=hP(oe.a),h(u.cd(),112)),!s&&J.b>0&&J.a<=0){_.c=pe(yn,st,1,0,5,1),_.c[_.c.length]=J;break}V=J.i-J.d,V>=x&&(V>x&&(_.c=pe(yn,st,1,0,5,1),x=V),_.c[_.c.length]=J)}_.c.length!=0&&(m=h(ut(_,NF(a,_.c.length)),112),FE(Ne.a,m)!=null,m.g=I++,xve(m,t,n,s),_.c=pe(yn,st,1,0,5,1))}for(fe=e.c.length+1,j=new Q(e);j.a<j.c.c.length;)P=h(K(j),112),P.g<v&&(P.g=P.g+fe)}function mR(e,t){var n;if(e.e)throw ne(new ja((yd(cte),WZ+cte.k+KZ)));if(!K1n(e.a,t))throw ne(new Ya(uvt+t+lvt));if(t==e.d)return e;switch(n=e.d,e.d=t,n.g){case 0:switch(t.g){case 2:Ob(e);break;case 1:_g(e),Ob(e);break;case 4:Dy(e),Ob(e);break;case 3:Dy(e),_g(e),Ob(e)}break;case 2:switch(t.g){case 1:_g(e),PQ(e);break;case 4:Dy(e),Ob(e);break;case 3:Dy(e),_g(e),Ob(e)}break;case 1:switch(t.g){case 2:_g(e),PQ(e);break;case 4:_g(e),Dy(e),Ob(e);break;case 3:_g(e),Dy(e),_g(e),Ob(e)}break;case 4:switch(t.g){case 2:Dy(e),Ob(e);break;case 1:Dy(e),_g(e),Ob(e);break;case 3:_g(e),PQ(e)}break;case 3:switch(t.g){case 2:_g(e),Dy(e),Ob(e);break;case 1:_g(e),Dy(e),_g(e),Ob(e);break;case 4:_g(e),PQ(e)}}return e}function zy(e,t){var n;if(e.d)throw ne(new ja((yd(Dte),WZ+Dte.k+KZ)));if(!X1n(e.a,t))throw ne(new Ya(uvt+t+lvt));if(t==e.c)return e;switch(n=e.c,e.c=t,n.g){case 0:switch(t.g){case 2:zp(e);break;case 1:Eg(e),zp(e);break;case 4:Ly(e),zp(e);break;case 3:Ly(e),Eg(e),zp(e)}break;case 2:switch(t.g){case 1:Eg(e),BQ(e);break;case 4:Ly(e),zp(e);break;case 3:Ly(e),Eg(e),zp(e)}break;case 1:switch(t.g){case 2:Eg(e),BQ(e);break;case 4:Eg(e),Ly(e),zp(e);break;case 3:Eg(e),Ly(e),Eg(e),zp(e)}break;case 4:switch(t.g){case 2:Ly(e),zp(e);break;case 1:Ly(e),Eg(e),zp(e);break;case 3:Eg(e),BQ(e)}break;case 3:switch(t.g){case 2:Eg(e),Ly(e),zp(e);break;case 1:Eg(e),Ly(e),Eg(e),zp(e);break;case 4:Eg(e),BQ(e)}}return e}function AOn(e,t,n){var s,a,u,p,v,m,x,_;for(m=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));m.e!=m.i.gc();)for(v=h(On(m),33),a=new Sn(Dn(Pd(v).a.Kc(),new L));rr(a);){if(s=h(Un(a),79),!s.b&&(s.b=new qt(Pn,s,4,7)),!(s.b.i<=1&&(!s.c&&(s.c=new qt(Pn,s,5,8)),s.c.i<=1)))throw ne(new z7("Graph must not contain hyperedges."));if(!y9(s)&&v!=Na(h(he((!s.c&&(s.c=new qt(Pn,s,5,8)),s.c),0),82)))for(x=new Att,Oa(x,s),Fe(x,(kg(),D5),s),Fhn(x,h(no(Ia(n.f,v)),144)),Rhn(x,h(_n(n,Na(h(he((!s.c&&(s.c=new qt(Pn,s,5,8)),s.c),0),82))),144)),Ue(t.c,x),p=new Tn((!s.n&&(s.n=new Ye(Ua,s,1,7)),s.n));p.e!=p.i.gc();)u=h(On(p),137),_=new Lat(x,u.a),Oa(_,u),Fe(_,D5,u),_.e.a=d.Math.max(u.g,1),_.e.b=d.Math.max(u.f,1),Tve(_),Ue(t.d,_)}}function MOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(A=new jve(e),Swn(A,!(t==(la(),Gd)||t==zd)),_=A.a,I=new E4,a=(Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])),p=0,m=a.length;p<m;++p)n=a[p],x=IY(_,ao,n),x&&(I.d=d.Math.max(I.d,x.Re()));for(s=se(ie(Xw,1),ze,232,0,[ao,Jo,oo]),u=0,v=s.length;u<v;++u)n=s[u],x=IY(_,oo,n),x&&(I.a=d.Math.max(I.a,x.Re()));for(V=se(ie(Xw,1),ze,232,0,[ao,Jo,oo]),oe=0,le=V.length;oe<le;++oe)P=V[oe],x=IY(_,P,ao),x&&(I.b=d.Math.max(I.b,x.Se()));for(j=se(ie(Xw,1),ze,232,0,[ao,Jo,oo]),J=0,fe=j.length;J<fe;++J)P=j[J],x=IY(_,P,oo),x&&(I.c=d.Math.max(I.c,x.Se()));return I.d>0&&(I.d+=_.n.d,I.d+=_.d),I.a>0&&(I.a+=_.n.a,I.a+=_.d),I.b>0&&(I.b+=_.n.b,I.b+=_.d),I.c>0&&(I.c+=_.n.c,I.c+=_.d),I}function Z2t(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j;for(I=n.d,A=n.c,u=new dt(n.f.a+n.d.b+n.d.c,n.f.b+n.d.d+n.d.a),p=u.b,x=new Q(e.a);x.a<x.c.c.length;)if(v=h(K(x),10),v.k==(bn(),Pi)){switch(s=h(U(v,(He(),co)),61),a=h(U(v,v6e),8),_=v.n,s.g){case 2:_.a=n.f.a+I.c-A.a;break;case 4:_.a=-A.a-I.b}switch(j=0,s.g){case 2:case 4:t==(_s(),ed)?(P=Pe(Je(U(v,iv))),_.b=u.b*P-h(U(v,(tt(),ov)),8).b,j=_.b+a.b,sF(v,!1,!0)):t==qo&&(_.b=Pe(Je(U(v,iv)))-h(U(v,(tt(),ov)),8).b,j=_.b+a.b,sF(v,!1,!0))}p=d.Math.max(p,j)}for(n.f.b+=p-u.b,m=new Q(e.a);m.a<m.c.c.length;)if(v=h(K(m),10),v.k==(bn(),Pi))switch(s=h(U(v,(He(),co)),61),_=v.n,s.g){case 1:_.b=-A.b-I.d;break;case 3:_.b=n.f.b+I.a-A.b}}function DOn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt;for(a=h(U(e,(fo(),U_)),33),x=Ir,_=Ir,v=$s,m=$s,Ve=pr(e.b,0);Ve.b!=Ve.d.c;)we=h(gr(Ve),86),V=we.e,J=we.f,x=d.Math.min(x,V.a-J.a/2),_=d.Math.min(_,V.b-J.b/2),v=d.Math.max(v,V.a+J.a/2),m=d.Math.max(m,V.b+J.b/2);for(j=h(pt(a,(Fb(),Z8t)),116),P=new dt(j.b-x,j.d-_),Ne=pr(e.b,0);Ne.b!=Ne.d.c;)we=h(gr(Ne),86),I=U(we,U_),ue(I,239)&&(u=h(I,33),A=zr(we.e,P),bf(u,A.a-u.g/2,A.b-u.f/2));for(le=pr(e.a,0);le.b!=le.d.c;)fe=h(gr(le),188),s=h(U(fe,U_),79),s&&(t=fe.a,oe=new _a(fe.b.e),Ai(t,oe,t.a,t.a.a),rt=new _a(fe.c.e),Ai(t,rt,t.c.b,t.c),mdt(oe,h(Zh(t,1),8),fe.b.f),mdt(rt,h(Zh(t,t.b-2),8),fe.c.f),n=Fy(s,!0,!0),CM(t,n));wt=v-x+(j.b+j.c),p=m-_+(j.d+j.a),$b(a,wt,p,!1,!1)}function LOn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(A=e.b,_=new Ms(A,0),iw(_,new Nl(e)),fe=!1,p=1;_.b<_.d.gc();){for(x=(kn(_.b<_.d.gc()),h(_.d.Xb(_.c=_.b++),29)),V=(Kt(p,A.c.length),h(A.c[p],29)),J=bb(x.a),oe=J.c.length,j=new Q(J);j.a<j.c.c.length;)I=h(K(j),10),Ca(I,V);if(fe){for(P=a0e(new U2(J),0);P.c.Sb();)for(I=h(iot(P),10),u=new Q(bb(za(I)));u.a<u.c.c.length;)a=h(K(u),17),jb(a,!0),Fe(e,(He(),SD),(on(),!0)),s=M2t(e,a,oe),n=h(U(I,tv),305),le=h(ut(s,s.c.length-1),17),n.k=le.c.i,n.n=le,n.b=a.d.i,n.c=a;fe=!1}else J.c.length!=0&&(t=(Kt(0,J.c.length),h(J.c[0],10)),t.k==(bn(),Bf)&&(fe=!0,p=-1));++p}for(v=new Ms(e.b,0);v.b<v.d.gc();)m=(kn(v.b<v.d.gc()),h(v.d.Xb(v.c=v.b++),29)),m.a.c.length==0&&Mu(v)}function IOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;if(_=h(h(jr(e.r,t),21),84),_.gc()<=2||t==(Qe(),pn)||t==(Qe(),un)){wbt(e,t);return}for(V=e.u.Hc((Qc(),K5)),n=t==(Qe(),sn)?(Cb(),fD):(Cb(),hD),oe=t==sn?(wc(),Ah):(wc(),nf),s=yhe(R1e(n),e.s),J=t==sn?ki:Fi,x=_.Kc();x.Ob();)v=h(x.Pb(),111),!(!v.c||v.c.d.c.length<=0)&&(j=v.b.rf(),P=v.e,A=v.c,I=A.i,I.b=(u=A.n,A.e.a+u.b+u.c),I.a=(p=A.n,A.e.b+p.d+p.a),V?(I.c=P.a-(a=A.n,A.e.a+a.b+a.c)-e.s,V=!1):I.c=P.a+j.a+e.s,aA(oe,gwe),A.f=oe,Yh(A,(Xc(),Sh)),Ue(s.d,new lW(I,wpe(s,I))),J=t==sn?d.Math.min(J,P.b):d.Math.max(J,P.b+v.b.rf().b));for(J+=t==sn?-e.t:e.t,Npe((s.e=J,s)),m=_.Kc();m.Ob();)v=h(m.Pb(),111),!(!v.c||v.c.d.c.length<=0)&&(I=v.c.i,I.c-=v.e.a,I.d-=v.e.b)}function OOn(e,t,n){var s;if(Bn(n,"StretchWidth layering",1),t.a.c.length==0){An(n);return}for(e.c=t,e.t=0,e.u=0,e.i=ki,e.g=Fi,e.d=Pe(Je(U(t,(tt(),Mh)))),u8n(e),hCn(e),lCn(e),gEn(e),bxn(e),e.i=d.Math.max(1,e.i),e.g=d.Math.max(1,e.g),e.d=e.d/e.i,e.f=e.g/e.i,e.s=_8n(e),s=new Nl(e.c),Ue(e.c.b,s),e.r=bb(e.p),e.n=rB(e.k,e.k.length);e.r.c.length!=0;)e.o=R6n(e),!e.o||qlt(e)&&e.b.a.gc()!=0?(jEn(e,s),s=new Nl(e.c),Ue(e.c.b,s),Qs(e.a,e.b),e.b.a.$b(),e.t=e.u,e.u=0):qlt(e)?(e.c.b.c=pe(yn,st,1,0,5,1),s=new Nl(e.c),Ue(e.c.b,s),e.t=0,e.u=0,e.b.a.$b(),e.a.a.$b(),++e.f,e.r=bb(e.p),e.n=rB(e.k,e.k.length)):(Ca(e.o,s),vc(e.r,e.o),Qi(e.b,e.o),e.t=e.t-e.k[e.o.p]*e.d+e.j[e.o.p],e.u+=e.e[e.o.p]*e.d);t.a.c=pe(yn,st,1,0,5,1),pQ(t.b),An(n)}function NOn(e){var t,n,s,a;for(Ti(Jr(new Vt(null,new Wt(e.a.b,16)),new uje),new lje),CEn(e),Ti(Jr(new Vt(null,new Wt(e.a.b,16)),new hje),new fje),e.c==(Od(),W5)&&(Ti(Jr(Xa(new Vt(null,new Wt(new P6(e.f),1)),new dje),new gje),new OYe(e)),Ti(Jr(pc(Xa(Xa(new Vt(null,new Wt(e.d.b,16)),new pje),new bje),new vje),new wje),new PYe(e))),a=new dt(ki,ki),t=new dt(Fi,Fi),s=new Q(e.a.b);s.a<s.c.c.length;)n=h(K(s),57),a.a=d.Math.min(a.a,n.d.c),a.b=d.Math.min(a.b,n.d.d),t.a=d.Math.max(t.a,n.d.c+n.d.b),t.b=d.Math.max(t.b,n.d.d+n.d.a);zr(Uh(e.d.c),fP(new dt(a.a,a.b))),zr(Uh(e.d.f),ms(new dt(t.a,t.b),a)),BSn(e,a,t),Kc(e.f),Kc(e.b),Kc(e.g),Kc(e.e),e.a.a.c=pe(yn,st,1,0,5,1),e.a.b.c=pe(yn,st,1,0,5,1),e.a=null,e.d=null}function J2t(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(a=new qe,V=new Q(t.a);V.a<V.c.c.length;)if(j=h(K(V),10),P=j.e,P&&(s=J2t(e,P,j),Vi(a,s),WIn(e,P,j),h(U(P,(He(),Uo)),21).Hc((ha(),El))))for(fe=h(U(j,(tt(),Ei)),98),I=h(U(j,cm),174).Hc((Qc(),Ud)),oe=new Q(j.j);oe.a<oe.c.c.length;)for(J=h(K(oe),11),u=h(_n(e.b,J),10),u||(u=DM(J,fe,J.j,-(J.e.c.length-J.g.c.length),null,new Rs,J.o,h(U(P,Ju),103),P),Fe(u,Br,J),Pr(e.b,J,u),Ue(P.a,u)),p=h(ut(u.j,0),11),_=new Q(J.f);_.a<_.c.c.length;)x=h(K(_),70),v=new XXe,v.o.a=x.o.a,v.o.b=x.o.b,Ue(p.f,v),I||(le=J.j,A=0,DE(h(U(j,cm),21))&&(A=mbe(x.n,x.o,J.o,0,le)),fe==(_s(),$f)||(Qe(),uc).Hc(le)?v.o.a=A:v.o.b=A);return m=new qe,sOn(e,t,n,a,m),n&&kIn(e,t,n,m),m}function Rve(e,t,n){var s,a,u,p,v,m,x,_,A;if(!e.c[t.c.p][t.p].e){for(e.c[t.c.p][t.p].e=!0,e.c[t.c.p][t.p].b=0,e.c[t.c.p][t.p].d=0,e.c[t.c.p][t.p].a=null,_=new Q(t.j);_.a<_.c.c.length;)for(x=h(K(_),11),A=n?new j6(x):new y4(x),m=A.Kc();m.Ob();)v=h(m.Pb(),11),p=v.i,p.c==t.c?p!=t&&(Rve(e,p,n),e.c[t.c.p][t.p].b+=e.c[p.c.p][p.p].b,e.c[t.c.p][t.p].d+=e.c[p.c.p][p.p].d):(e.c[t.c.p][t.p].d+=e.g[v.p],++e.c[t.c.p][t.p].b);if(u=h(U(t,(He(),d6e)),15),u)for(a=u.Kc();a.Ob();)s=h(a.Pb(),10),t.c==s.c&&(Rve(e,s,n),e.c[t.c.p][t.p].b+=e.c[s.c.p][s.p].b,e.c[t.c.p][t.p].d+=e.c[s.c.p][s.p].d);e.c[t.c.p][t.p].b>0&&(e.c[t.c.p][t.p].d+=hu(e.i,24)*HM*.07000000029802322-.03500000014901161,e.c[t.c.p][t.p].a=e.c[t.c.p][t.p].d/e.c[t.c.p][t.p].b)}}function POn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;for(j=new Q(e);j.a<j.c.c.length;){for(P=h(K(j),10),c0(P.n),c0(P.o),j0e(P.f),ppt(P),fMn(P),J=new Q(P.j);J.a<J.c.c.length;){for(V=h(K(J),11),c0(V.n),c0(V.a),c0(V.o),Ji(V,t1t(V.j)),u=h(U(V,(tt(),L0)),19),u&&Fe(V,L0,Ke(-u.a)),a=new Q(V.g);a.a<a.c.c.length;){for(s=h(K(a),17),n=pr(s.a,0);n.b!=n.d.c;)t=h(gr(n),8),c0(t);if(m=h(U(s,Da),74),m)for(v=pr(m,0);v.b!=v.d.c;)p=h(gr(v),8),c0(p);for(A=new Q(s.b);A.a<A.c.c.length;)x=h(K(A),70),c0(x.n),c0(x.o)}for(I=new Q(V.f);I.a<I.c.c.length;)x=h(K(I),70),c0(x.n),c0(x.o)}for(P.k==(bn(),Pi)&&(Fe(P,(He(),co),t1t(h(U(P,co),61))),xAn(P)),_=new Q(P.b);_.a<_.c.c.length;)x=h(K(_),70),ppt(x),c0(x.o),c0(x.n)}}function BOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt;for(e.e=t,v=BTn(t),Ve=new qe,s=new Q(v);s.a<s.c.c.length;){for(n=h(K(s),15),rt=new qe,Ve.c[Ve.c.length]=rt,m=new ns,j=n.Kc();j.Ob();){for(P=h(j.Pb(),33),u=xM(e,P,!0,0,0),rt.c[rt.c.length]=u,V=P.i,J=P.j,new dt(V,J),I=(!P.n&&(P.n=new Ye(Ua,P,1,7)),P.n),A=new Tn(I);A.e!=A.i.gc();)x=h(On(A),137),a=xM(e,x,!1,V,J),rt.c[rt.c.length]=a;for(Ne=(!P.c&&(P.c=new Ye(bu,P,9,9)),P.c),fe=new Tn(Ne);fe.e!=fe.i.gc();)for(oe=h(On(fe),118),p=xM(e,oe,!1,V,J),rt.c[rt.c.length]=p,le=oe.i+V,we=oe.j+J,I=(!oe.n&&(oe.n=new Ye(Ua,oe,1,7)),oe.n),_=new Tn(I);_.e!=_.i.gc();)x=h(On(_),137),a=xM(e,x,!1,le,we),rt.c[rt.c.length]=a;Qs(m,ZP(Cf(se(ie(Nf,1),st,20,0,[Pd(P),wM(P)]))))}CSn(e,m,rt)}return e.f=new YXe(Ve),Oa(e.f,t),e.f}function FOn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln;ln=_n(e.e,s),ln==null&&(ln=new _4,P=h(ln,183),fe=t+"_s",le=fe+a,I=new uw(le),Wh(P,p1,I)),xt=h(ln,183),z4(n,xt),Ln=new _4,Lp(Ln,"x",s.j),Lp(Ln,"y",s.k),Wh(xt,vmt,Ln),rt=new _4,Lp(rt,"x",s.b),Lp(rt,"y",s.c),Wh(xt,"endPoint",rt),A=wS((!s.a&&(s.a=new Gi(Zl,s,5)),s.a)),j=!A,j&&(Ve=new yp,u=new JWe(Ve),Os((!s.a&&(s.a=new Gi(Zl,s,5)),s.a),u),Wh(xt,hj,Ve)),m=FX(s),we=!!m,we&&q2e(e.a,xt,Rye,gQ(e,FX(s))),oe=RX(s),Ne=!!oe,Ne&&q2e(e.a,xt,Fye,gQ(e,RX(s))),x=(!s.e&&(s.e=new qt(gi,s,10,9)),s.e).i==0,V=!x,V&&(wt=new yp,p=new mJe(e,wt),Os((!s.e&&(s.e=new qt(gi,s,10,9)),s.e),p),Wh(xt,$ye,wt)),_=(!s.g&&(s.g=new qt(gi,s,9,10)),s.g).i==0,J=!_,J&&(dn=new yp,v=new yJe(e,dn),Os((!s.g&&(s.g=new qt(gi,s,9,10)),s.g),v),Wh(xt,jye,dn))}function ROn(e){rw();var t,n,s,a,u,p,v;for(s=e.f.n,p=Ode(e.r).a.nc();p.Ob();){if(u=h(p.Pb(),111),a=0,u.b.Xe((Tr(),pu))&&(a=Pe(Je(u.b.We(pu))),a<0))switch(u.b.Hf().g){case 1:s.d=d.Math.max(s.d,-a);break;case 3:s.a=d.Math.max(s.a,-a);break;case 2:s.c=d.Math.max(s.c,-a);break;case 4:s.b=d.Math.max(s.b,-a)}if(DE(e.u))switch(t=c5n(u.b,a),v=!h(e.e.We(Wg),174).Hc((fu(),uL)),n=!1,u.b.Hf().g){case 1:n=t>s.d,s.d=d.Math.max(s.d,t),v&&n&&(s.d=d.Math.max(s.d,s.a),s.a=s.d+a);break;case 3:n=t>s.a,s.a=d.Math.max(s.a,t),v&&n&&(s.a=d.Math.max(s.a,s.d),s.d=s.a+a);break;case 2:n=t>s.c,s.c=d.Math.max(s.c,t),v&&n&&(s.c=d.Math.max(s.b,s.c),s.b=s.c+a);break;case 4:n=t>s.b,s.b=d.Math.max(s.b,t),v&&n&&(s.b=d.Math.max(s.b,s.c),s.c=s.b+a)}}}function jOn(e){var t,n,s,a,u,p,v,m,x,_,A;for(x=new Q(e);x.a<x.c.c.length;){switch(m=h(K(x),10),p=h(U(m,(tt(),cc)),163),u=null,p.g){case 1:case 2:u=(Hp(),c3);break;case 3:case 4:u=(Hp(),t8)}if(u)Fe(m,(He(),B$),(Hp(),c3)),u==t8?rR(m,p,(ua(),Jc)):u==c3&&rR(m,p,(ua(),ec));else if(oy(h(U(m,Ei),98))&&m.j.c.length!=0){for(t=!0,A=new Q(m.j);A.a<A.c.c.length;){if(_=h(K(A),11),!(_.j==(Qe(),pn)&&_.e.c.length-_.g.c.length>0||_.j==un&&_.e.c.length-_.g.c.length<0)){t=!1;break}for(a=new Q(_.g);a.a<a.c.c.length;)if(n=h(K(a),17),v=h(U(n.d.i,cc),163),v==(wl(),S_)||v==nm){t=!1;break}for(s=new Q(_.e);s.a<s.c.c.length;)if(n=h(K(s),17),v=h(U(n.c.i,cc),163),v==(wl(),C_)||v==Gg){t=!1;break}}t&&rR(m,p,(ua(),B_))}}}function $On(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;for(Ve=0,P=0,A=new Q(t.e);A.a<A.c.c.length;){for(_=h(K(A),10),I=0,v=0,m=n?h(U(_,cH),19).a:$s,oe=s?h(U(_,uH),19).a:$s,x=d.Math.max(m,oe),le=new Q(_.j);le.a<le.c.c.length;){if(fe=h(K(le),11),we=_.n.b+fe.n.b+fe.a.b,s)for(p=new Q(fe.g);p.a<p.c.c.length;)u=h(K(p),17),V=u.d,j=V.i,t!=e.a[j.p]&&(J=d.Math.max(h(U(j,cH),19).a,h(U(j,uH),19).a),Ne=h(U(u,(tt(),H5)),19).a,Ne>=x&&Ne>=J&&(I+=j.n.b+V.n.b+V.a.b-we,++v));if(n)for(p=new Q(fe.e);p.a<p.c.c.length;)u=h(K(p),17),V=u.c,j=V.i,t!=e.a[j.p]&&(J=d.Math.max(h(U(j,cH),19).a,h(U(j,uH),19).a),Ne=h(U(u,(tt(),H5)),19).a,Ne>=x&&Ne>=J&&(I+=j.n.b+V.n.b+V.a.b-we,++v))}v>0&&(Ve+=I/v,++P)}P>0?(t.a=a*Ve/P,t.g=P):(t.a=0,t.g=0)}function HOn(e,t){var n,s,a,u,p,v,m,x,_,A,I;for(a=new Q(e.a.b);a.a<a.c.c.length;)for(n=h(K(a),29),m=new Q(n.a);m.a<m.c.c.length;)v=h(K(m),10),t.j[v.p]=v,t.i[v.p]=t.o==(yf(),Rf)?Fi:ki;for(Kc(e.c),p=e.a.b,t.c==(r1(),I0)&&(p=ue(p,152)?X4(h(p,152)):ue(p,131)?h(p,131).a:ue(p,54)?new U2(p):new Xv(p)),N3n(e.e,t,e.b),CS(t.p,null),u=p.Kc();u.Ob();)for(n=h(u.Pb(),29),x=n.a,t.o==(yf(),Rf)&&(x=ue(x,152)?X4(h(x,152)):ue(x,131)?h(x,131).a:ue(x,54)?new U2(x):new Xv(x)),I=x.Kc();I.Ob();)A=h(I.Pb(),10),t.g[A.p]==A&&ybt(e,A,t);for(DIn(e,t),s=p.Kc();s.Ob();)for(n=h(s.Pb(),29),I=new Q(n.a);I.a<I.c.c.length;)A=h(K(I),10),t.p[A.p]=t.p[t.g[A.p].p],A==t.g[A.p]&&(_=Pe(t.i[t.j[A.p].p]),(t.o==(yf(),Rf)&&_>Fi||t.o==v2&&_<ki)&&(t.p[A.p]=Pe(t.p[A.p])+_));e.e.cg()}function ebt(e,t,n,s){var a,u,p,v,m;return v=new jve(t),zSn(v,s),a=!0,!!e&&e.Xe((Tr(),hv))&&(u=h(e.We((Tr(),hv)),103),a=u==(la(),Q1)||u==Wl||u==Dh),ipt(v,!1),mc(v.e.wf(),new P1e(v,!1,a)),VW(v,v.f,(Kh(),ao),(Qe(),sn)),VW(v,v.f,oo,Fn),VW(v,v.g,ao,un),VW(v,v.g,oo,pn),zft(v,sn),zft(v,Fn),oit(v,pn),oit(v,un),rw(),p=v.A.Hc((Iu(),bm))&&v.B.Hc((fu(),lL))?eft(v):null,p&&u1n(v.a,p),ROn(v),e7n(v),t7n(v),uOn(v),zMn(v),S7n(v),MX(v,sn),MX(v,Fn),kMn(v),HLn(v),n&&(jkn(v),A7n(v),MX(v,pn),MX(v,un),m=v.B.Hc((fu(),oT)),Jdt(v,m,sn),Jdt(v,m,Fn),e0t(v,m,pn),e0t(v,m,un),Ti(new Vt(null,new Wt(new vd(v.i),0)),new zi),Ti(Jr(new Vt(null,Ode(v.r).a.oc()),new as),new ai),GEn(v),v.e.uf(v.o),Ti(new Vt(null,Ode(v.r).a.oc()),new xu)),v.o}function zOn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(x=ki,s=new Q(e.a.b);s.a<s.c.c.length;)t=h(K(s),81),x=d.Math.min(x,t.d.f.g.c+t.e.a);for(P=new di,p=new Q(e.a.a);p.a<p.c.c.length;)u=h(K(p),189),u.i=x,u.e==0&&Ai(P,u,P.c.b,P.c);for(;P.b!=0;){for(u=h(P.b==0?null:(kn(P.b!=0),pl(P,P.a.a)),189),a=u.f.g.c,I=u.a.a.ec().Kc();I.Ob();)_=h(I.Pb(),81),V=u.i+_.e.a,_.d.g||_.g.c<V?_.o=V:_.o=_.g.c;for(a-=u.f.o,u.b+=a,e.c==(la(),Dh)||e.c==zd?u.c+=a:u.c-=a,A=u.a.a.ec().Kc();A.Ob();)for(_=h(A.Pb(),81),m=_.f.Kc();m.Ob();)v=h(m.Pb(),81),lg(e.c)?j=e.f.ef(_,v):j=e.f.ff(_,v),v.d.i=d.Math.max(v.d.i,_.o+_.g.b+j-v.e.a),v.k||(v.d.i=d.Math.max(v.d.i,v.g.c-v.e.a)),--v.d.e,v.d.e==0&&vr(P,v.d)}for(n=new Q(e.a.b);n.a<n.c.c.length;)t=h(K(n),81),t.g.c=t.o}function GOn(e){var t,n,s,a,u,p,v,m;switch(v=e.b,t=e.a,h(U(e,(HF(),a4e)),427).g){case 0:hs(v,new Hr(new bt));break;case 1:default:hs(v,new Hr(new $n))}switch(h(U(e,i4e),428).g){case 1:hs(v,new Ur),hs(v,new Er),hs(v,new su);break;case 0:default:hs(v,new Ur),hs(v,new ol)}switch(h(U(e,c4e),250).g){case 0:m=new Eu;break;case 1:m=new Po;break;case 2:m=new Ra;break;case 3:m=new wo;break;case 5:m=new R6(new Ra);break;case 4:m=new R6(new Po);break;case 7:m=new zhe(new R6(new Po),new R6(new Ra));break;case 8:m=new zhe(new R6(new wo),new R6(new Ra));break;case 6:default:m=new R6(new wo)}for(p=new Q(v);p.a<p.c.c.length;){for(u=h(K(p),167),s=0,a=0,n=new As(Ke(s),Ke(a));ADn(t,u,s,a);)n=h(m.Ce(n,u),46),s=h(n.a,19).a,a=h(n.b,19).a;CMn(t,u,s,a)}}function VOn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt;for(u=e.f.b,I=u.a,_=u.b,j=e.e.g,P=e.e.f,ZN(e.e,u.a,u.b),Ve=I/j,rt=_/P,x=new Tn(UP(e.e));x.e!=x.i.gc();)m=h(On(x),137),yc(m,m.i*Ve),kc(m,m.j*rt);for(fe=new Tn(DW(e.e));fe.e!=fe.i.gc();)oe=h(On(fe),118),we=oe.i,Ne=oe.j,we>0&&yc(oe,we*Ve),Ne>0&&kc(oe,Ne*rt);for(ZE(e.b,new rq),t=new qe,v=new Gp(new xp(e.c).a);v.b;)p=kb(v),s=h(p.cd(),79),n=h(p.dd(),395).a,a=Fy(s,!1,!1),A=J1t(H1(s),hM(a),n),CM(A,a),le=hdt(s),!!le&&Ha(t,le,0)==-1&&(t.c[t.c.length]=le,vit(le,(kn(A.b!=0),h(A.a.a.c,8)),n));for(J=new Gp(new xp(e.d).a);J.b;)V=kb(J),s=h(V.cd(),79),n=h(V.dd(),395).a,a=Fy(s,!1,!1),A=J1t(Sg(s),GA(hM(a)),n),A=GA(A),CM(A,a),le=fdt(s),!!le&&Ha(t,le,0)==-1&&(t.c[t.c.length]=le,vit(le,(kn(A.b!=0),h(A.c.b.c,8)),n))}function tbt(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt;if(n.c.length!=0){for(P=new qe,I=new Q(n);I.a<I.c.c.length;)A=h(K(I),33),Ue(P,new dt(A.i,A.j));for(s.n&&!!t&&mh(s,yh(t),(Lu(),rl));X2e(e,n);)ZF(e,n,!1);for(s.n&&!!t&&mh(s,yh(t),(Lu(),rl)),p=0,v=0,a=null,n.c.length!=0&&(a=(Kt(0,n.c.length),h(n.c[0],33)),p=a.i-(Kt(0,P.c.length),h(P.c[0],8)).a,v=a.j-(Kt(0,P.c.length),h(P.c[0],8)).b),u=d.Math.sqrt(p*p+v*v),_=xht(n);_.a.gc()!=0;){for(x=_.a.ec().Kc();x.Ob();)m=h(x.Pb(),33),j=e.f,V=j.i+j.g/2,J=j.j+j.f/2,oe=m.i+m.g/2,fe=m.j+m.f/2,le=oe-V,we=fe-J,Ne=d.Math.sqrt(le*le+we*we),Ve=le/Ne,rt=we/Ne,yc(m,m.i+Ve*u),kc(m,m.j+rt*u);s.n&&!!t&&mh(s,yh(t),(Lu(),rl)),_=xht(new Bc(_))}e.a&&e.a.lg(new Bc(_)),s.n&&!!t&&mh(s,yh(t),(Lu(),rl)),tbt(e,t,new Bc(_),s)}}function UOn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;if(V=e.n,J=e.o,I=e.d,A=Pe(Je(_w(e,(tt(),Lne)))),t){for(_=A*(t.gc()-1),P=0,m=t.Kc();m.Ob();)p=h(m.Pb(),10),_+=p.o.a,P=d.Math.max(P,p.o.b);for(oe=V.a-(_-J.a)/2,u=V.b-I.d+P,s=J.a/(t.gc()+1),a=s,v=t.Kc();v.Ob();)p=h(v.Pb(),10),p.n.a=oe,p.n.b=u-p.o.b,oe+=p.o.a+A,x=dgt(p),x.n.a=p.o.a/2-x.a.a,x.n.b=p.o.b,j=h(U(p,(He(),N$)),11),j.e.c.length+j.g.c.length==1&&(j.n.a=a-j.a.a,j.n.b=0,Ka(j,e)),a+=s}if(n){for(_=A*(n.gc()-1),P=0,m=n.Kc();m.Ob();)p=h(m.Pb(),10),_+=p.o.a,P=d.Math.max(P,p.o.b);for(oe=V.a-(_-J.a)/2,u=V.b+J.b+I.a-P,s=J.a/(n.gc()+1),a=s,v=n.Kc();v.Ob();)p=h(v.Pb(),10),p.n.a=oe,p.n.b=u,oe+=p.o.a+A,x=dgt(p),x.n.a=p.o.a/2-x.a.a,x.n.b=0,j=h(U(p,(He(),N$)),11),j.e.c.length+j.g.c.length==1&&(j.n.a=a-j.a.a,j.n.b=J.b,Ka(j,e)),a+=s}}function qOn(e,t){var n,s,a,u,p,v;if(!!h(U(t,(He(),Uo)),21).Hc((ha(),El))){for(v=new Q(t.a);v.a<v.c.c.length;)u=h(K(v),10),u.k==(bn(),Wi)&&(a=h(U(u,(tt(),W$)),142),e.c=d.Math.min(e.c,u.n.a-a.b),e.a=d.Math.max(e.a,u.n.a+u.o.a+a.c),e.d=d.Math.min(e.d,u.n.b-a.d),e.b=d.Math.max(e.b,u.n.b+u.o.b+a.a));for(p=new Q(t.a);p.a<p.c.c.length;)if(u=h(K(p),10),u.k!=(bn(),Wi))switch(u.k.g){case 2:if(s=h(U(u,(tt(),cc)),163),s==(wl(),Gg)){u.n.a=e.c-10,_1t(u,new NFe).Jb(new fYe(u));break}if(s==nm){u.n.a=e.a+10,_1t(u,new PFe).Jb(new dYe(u));break}if(n=h(U(u,h2),303),n==(Ad(),l3)){T2t(u).Jb(new gYe(u)),u.n.b=e.d-10;break}if(n==O5){T2t(u).Jb(new pYe(u)),u.n.b=e.b+10;break}break;default:throw ne(new an("The node type "+u.k+" is not supported by the "+MPn))}}}function YOn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J;for(m=new dt(s.i+s.g/2,s.j+s.f/2),P=k2t(s),j=h(pt(t,(tt(),Ei)),98),J=h(pt(s,u8),61),AJe(Pft(s),om)||(s.i==0&&s.j==0?V=0:V=T9n(s,J),ya(s,om,V)),x=new dt(t.g,t.f),a=DM(s,j,J,P,x,m,new dt(s.g,s.f),h(U(n,Ju),103),n),Fe(a,(He(),Br),s),u=h(ut(a.j,0),11),Hhn(u,wDn(s)),Fe(a,cm,(Qc(),Lt(Zg))),A=h(pt(t,cm),174).Hc(Ud),v=new Tn((!s.n&&(s.n=new Ye(Ua,s,1,7)),s.n));v.e!=v.i.gc();)if(p=h(On(v),137),!ft(lt(pt(p,p2)))&&!!p.a&&(I=hX(p),Ue(u.f,I),!A))switch(_=0,DE(h(pt(t,cm),21))&&(_=mbe(new dt(p.i,p.j),new dt(p.g,p.f),new dt(s.g,s.f),0,J)),J.g){case 2:case 4:I.o.a=_;break;case 1:case 3:I.o.b=_}Fe(a,h8,Je(pt(bi(t),h8))),Fe(a,f8,Je(pt(bi(t),f8))),Fe(a,lm,Je(pt(bi(t),lm))),Ue(n.a,a),Pr(e.a,s,a)}function nbt(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne;for(Bn(n,"Processor arrange level",1),_=0,Rt(),my(t,new Ple((fo(),fH))),u=t.b,v=pr(t,t.b),x=!0;x&&v.b.b!=v.d.a;)oe=h(eK(v),86),h(U(oe,fH),19).a==0?--u:x=!1;if(Ne=new F1(t,0,u),p=new WY(Ne),Ne=new F1(t,u,t.b),m=new WY(Ne),p.b==0)for(j=pr(m,0);j.b!=j.d.c;)P=h(gr(j),86),Fe(P,pH,Ke(_++));else for(A=p.b,we=pr(p,0);we.b!=we.d.c;){for(le=h(gr(we),86),Fe(le,pH,Ke(_++)),s=Pge(le),nbt(e,s,jo(n,1/A|0)),my(s,zde(new Ple(pH))),I=new di,fe=pr(s,0);fe.b!=fe.d.c;)for(oe=h(gr(fe),86),J=pr(le.d,0);J.b!=J.d.c;)V=h(gr(J),188),V.c==oe&&Ai(I,V,I.c.b,I.c);for(Pl(le.d),Qs(le.d,I),v=pr(m,m.b),a=le.d.b,x=!0;0<a&&x&&v.b.b!=v.d.a;)oe=h(eK(v),86),h(U(oe,fH),19).a==0?(Fe(oe,pH,Ke(_++)),--a,BB(v)):x=!1}An(n)}function WOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(Bn(t,"Inverted port preprocessing",1),_=e.b,x=new Ms(_,0),n=null,le=new qe;x.b<x.d.gc();){for(fe=n,n=(kn(x.b<x.d.gc()),h(x.d.Xb(x.c=x.b++),29)),P=new Q(le);P.a<P.c.c.length;)A=h(K(P),10),Ca(A,fe);for(le.c=pe(yn,st,1,0,5,1),j=new Q(n.a);j.a<j.c.c.length;)if(A=h(K(j),10),A.k==(bn(),Wi)&&!!oy(h(U(A,(tt(),Ei)),98))){for(oe=ive(A,(ua(),Jc),(Qe(),pn)).Kc();oe.Ob();)for(V=h(oe.Pb(),11),m=V.e,v=h(Mf(m,pe(Ite,gJ,17,m.c.length,0,1)),474),a=v,u=0,p=a.length;u<p;++u)s=a[u],mLn(e,V,s,le);for(J=ive(A,ec,un).Kc();J.Ob();)for(V=h(J.Pb(),11),m=V.g,v=h(Mf(m,pe(Ite,gJ,17,m.c.length,0,1)),474),a=v,u=0,p=a.length;u<p;++u)s=a[u],wLn(e,V,s,le)}}for(I=new Q(le);I.a<I.c.c.length;)A=h(K(I),10),Ca(A,n);An(t)}function KOn(e,t,n,s,a,u){var p,v,m,x,_,A;for(x=new So,Oa(x,t),Ji(x,h(pt(t,(tt(),u8)),61)),Fe(x,(He(),Br),t),Ka(x,n),A=x.o,A.a=t.g,A.b=t.f,_=x.n,_.a=t.i,_.b=t.j,Pr(e.a,t,x),p=yk(pc(Xa(new Vt(null,(!t.e&&(t.e=new qt(us,t,7,4)),new Wt(t.e,16))),new jBe),new FBe),new rYe(t)),p||(p=yk(pc(Xa(new Vt(null,(!t.d&&(t.d=new qt(us,t,8,5)),new Wt(t.d,16))),new $Be),new RBe),new iYe(t))),p||(p=yk(new Vt(null,(!t.e&&(t.e=new qt(us,t,7,4)),new Wt(t.e,16))),new HBe)),Fe(x,AD,(on(),!!p)),fIn(x,u,a,h(pt(t,ov),8)),m=new Tn((!t.n&&(t.n=new Ye(Ua,t,1,7)),t.n));m.e!=m.i.gc();)v=h(On(m),137),!ft(lt(pt(v,p2)))&&!!v.a&&Ue(x.f,hX(v));switch(a.g){case 2:case 1:(x.j==(Qe(),sn)||x.j==Fn)&&s.Fc((ha(),u3));break;case 4:case 3:(x.j==(Qe(),pn)||x.j==un)&&s.Fc((ha(),u3))}return x}function oZ(e,t,n,s,a,u,p){var v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(I=null,s==(gb(),Yg)?I=t:s==m3&&(I=n),V=I.a.ec().Kc();V.Ob();){for(j=h(V.Pb(),11),J=Qa(se(ie(cs,1),Re,8,0,[j.i.n,j.n,j.a])).b,le=new ns,v=new ns,x=new _f(j.b);Wa(x.a)||Wa(x.b);)if(m=h(Wa(x.a)?K(x.a):K(x.b),17),ft(lt(U(m,(He(),Ff))))==a&&Ha(u,m,0)!=-1){if(m.d==j?oe=m.c:oe=m.d,fe=Qa(se(ie(cs,1),Re,8,0,[oe.i.n,oe.n,oe.a])).b,d.Math.abs(fe-J)<.2)continue;fe<J?t.a._b(oe)?Qi(le,new As(Yg,m)):Qi(le,new As(m3,m)):t.a._b(oe)?Qi(v,new As(Yg,m)):Qi(v,new As(m3,m))}if(le.a.gc()>1)for(P=new Dve(j,le,s),Os(le,new qZe(e,P)),p.c[p.c.length]=P,A=le.a.ec().Kc();A.Ob();)_=h(A.Pb(),46),vc(u,_.b);if(v.a.gc()>1)for(P=new Dve(j,v,s),Os(v,new YZe(e,P)),p.c[p.c.length]=P,A=v.a.ec().Kc();A.Ob();)_=h(A.Pb(),46),vc(u,_.b)}}function rbt(e){J2(e,new Xp(fN(K2(q2(W2(Y2(new mp,g1),"ELK Radial"),'A radial layout provider which is based on the algorithm of Peter Eades published in "Drawing free trees.", published by International Institute for Advanced Study of Social Information Science, Fujitsu Limited in 1991. The radial layouter takes a tree and places the nodes in radial order around the root. The nodes of the same tree level are placed on the same radius.'),new _He),g1))),et(e,g1,nj,at(T7t)),et(e,g1,zw,at(C7t)),et(e,g1,Yy,at(k7t)),et(e,g1,E5,at(x7t)),et(e,g1,x5,at(E7t)),et(e,g1,Dx,at(y7t)),et(e,g1,z9,at(zxe)),et(e,g1,Lx,at(_7t)),et(e,g1,tee,at(gre)),et(e,g1,eee,at(pre)),et(e,g1,hye,at(Gxe)),et(e,g1,oye,at(mH)),et(e,g1,cye,at(yH)),et(e,g1,uye,at(GD)),et(e,g1,lye,at(Vxe))}function jve(e){var t;if(this.r=_vn(new vs,new ti),this.b=new o9(h(Vn(ta),290)),this.p=new o9(h(Vn(ta),290)),this.i=new o9(h(Vn(g4t),290)),this.e=e,this.o=new _a(e.rf()),this.D=e.Df()||ft(lt(e.We((Tr(),JD)))),this.A=h(e.We((Tr(),m2)),21),this.B=h(e.We(Wg),21),this.q=h(e.We(Z_),98),this.u=h(e.We(gm),21),!L9n(this.u))throw ne(new ey("Invalid port label placement: "+this.u));if(this.v=ft(lt(e.We(x7e))),this.j=h(e.We(E3),21),!tSn(this.j))throw ne(new ey("Invalid node label placement: "+this.j));this.n=h(d9(e,u7e),116),this.k=Pe(Je(d9(e,PH))),this.d=Pe(Je(d9(e,C7e))),this.w=Pe(Je(d9(e,L7e))),this.s=Pe(Je(d9(e,S7e))),this.t=Pe(Je(d9(e,A7e))),this.C=h(d9(e,M7e),142),this.c=2*this.d,t=!this.B.Hc((fu(),uL)),this.f=new u9(0,t,0),this.g=new u9(1,t,0),Oq(this.f,(Kh(),Jo),this.g)}function XOn(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln;for(le=0,j=0,P=0,I=1,fe=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));fe.e!=fe.i.gc();)J=h(On(fe),33),I+=Td(new Sn(Dn(Pd(J).a.Kc(),new L))),wt=J.g,j=d.Math.max(j,wt),A=J.f,P=d.Math.max(P,A),le+=wt*A;for(V=(!e.a&&(e.a=new Ye(wi,e,10,11)),e.a).i,p=le+2*s*s*I*V,u=d.Math.sqrt(p),m=d.Math.max(u*n,j),v=d.Math.max(u/n,P),oe=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));oe.e!=oe.i.gc();)J=h(On(oe),33),xt=a.b+(hu(t,26)*P9+hu(t,27)*B9)*(m-J.g),ln=a.b+(hu(t,26)*P9+hu(t,27)*B9)*(v-J.f),yc(J,xt),kc(J,ln);for(rt=m+(a.b+a.c),Ve=v+(a.d+a.a),Ne=new Tn((!e.a&&(e.a=new Ye(wi,e,10,11)),e.a));Ne.e!=Ne.i.gc();)for(we=h(On(Ne),33),_=new Sn(Dn(Pd(we).a.Kc(),new L));rr(_);)x=h(Un(_),79),y9(x)||QNn(x,t,rt,Ve);rt+=a.b+a.c,Ve+=a.d+a.a,$b(e,rt,Ve,!1,!0)}function yR(e){var t,n,s,a,u,p,v,m,x,_,A;if(e==null)throw ne(new Qf(_c));if(x=e,u=e.length,m=!1,u>0&&(t=(er(0,e.length),e.charCodeAt(0)),(t==45||t==43)&&(e=e.substr(1),--u,m=t==45)),u==0)throw ne(new Qf(Gb+x+'"'));for(;e.length>0&&(er(0,e.length),e.charCodeAt(0)==48);)e=e.substr(1),--u;if(u>(Upt(),g3t)[10])throw ne(new Qf(Gb+x+'"'));for(a=0;a<u;a++)if(yft((er(a,e.length),e.charCodeAt(a)))==-1)throw ne(new Qf(Gb+x+'"'));for(A=0,p=S3e[10],_=nte[10],v=Tk(A3e[10]),n=!0,s=u%p,s>0&&(A=-parseInt(e.substr(0,s),10),e=e.substr(s),u-=s,n=!1);u>=p;){if(s=parseInt(e.substr(0,p),10),e=e.substr(p),u-=p,n)n=!1;else{if(mo(A,v)<0)throw ne(new Qf(Gb+x+'"'));A=js(A,_)}A=Cg(A,s)}if(mo(A,0)>0)throw ne(new Qf(Gb+x+'"'));if(!m&&(A=Tk(A),mo(A,0)<0))throw ne(new Qf(Gb+x+'"'));return A}function $ve(e,t){Nnt();var n,s,a,u,p,v,m;if(this.a=new Dfe(this),this.b=e,this.c=t,this.f=vW(Sa((jc(),Ps),t)),this.f.dc())if((v=T2e(Ps,e))==t)for(this.e=!0,this.d=new qe,this.f=new Ize,this.f.Fc(o2),h(bR(TA(Ps,Vu(e)),""),26)==e&&this.f.Fc(AE(Ps,Vu(e))),a=jQ(Ps,e).Kc();a.Ob();)switch(s=h(a.Pb(),170),fb(Sa(Ps,s))){case 4:{this.d.Fc(s);break}case 5:{this.f.Gc(vW(Sa(Ps,s)));break}}else if(ia(),h(t,66).Oj())for(this.e=!0,this.f=null,this.d=new qe,p=0,m=(e.i==null&&s1(e),e.i).length;p<m;++p)for(s=(n=(e.i==null&&s1(e),e.i),p>=0&&p<n.length?n[p]:null),u=Y4(Sa(Ps,s));u;u=Y4(Sa(Ps,u)))u==t&&this.d.Fc(s);else fb(Sa(Ps,t))==1&&!!v?(this.f=null,this.d=(h5(),n_t)):(this.f=null,this.e=!0,this.d=(Rt(),new mq(t)));else this.e=fb(Sa(Ps,t))==5,this.f.Fb(oie)&&(this.f=oie)}function ibt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;for(n=0,s=G8n(e,t),I=e.s,P=e.t,x=h(h(jr(e.r,t),21),84).Kc();x.Ob();)if(m=h(x.Pb(),111),!(!m.c||m.c.d.c.length<=0)){switch(j=m.b.rf(),v=m.b.Xe((Tr(),pu))?Pe(Je(m.b.We(pu))):0,_=m.c,A=_.i,A.b=(p=_.n,_.e.a+p.b+p.c),A.a=(u=_.n,_.e.b+u.d+u.a),t.g){case 1:A.c=m.a?(j.a-A.b)/2:j.a+I,A.d=j.b+v+s,Yh(_,(Xc(),jg)),e1(_,(wc(),nf));break;case 3:A.c=m.a?(j.a-A.b)/2:j.a+I,A.d=-v-s-A.a,Yh(_,(Xc(),jg)),e1(_,(wc(),Ah));break;case 2:A.c=-v-s-A.b,m.a?(a=e.v?A.a:h(ut(_.d,0),181).rf().b,A.d=(j.b-a)/2):A.d=j.b+P,Yh(_,(Xc(),Sh)),e1(_,(wc(),$g));break;case 4:A.c=j.a+v+s,m.a?(a=e.v?A.a:h(ut(_.d,0),181).rf().b,A.d=(j.b-a)/2):A.d=j.b+P,Yh(_,(Xc(),v1)),e1(_,(wc(),$g))}(t==(Qe(),sn)||t==Fn)&&(n=d.Math.max(n,A.a))}n>0&&(h(wa(e.b,t),124).a.b=n)}function QOn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;for(Bn(t,"Comment pre-processing",1),n=0,m=new Q(e.a);m.a<m.c.c.length;)if(v=h(K(m),10),ft(lt(U(v,(tt(),A_))))){for(++n,a=0,s=null,x=null,j=new Q(v.j);j.a<j.c.c.length;)I=h(K(j),11),a+=I.e.c.length+I.g.c.length,I.e.c.length==1&&(s=h(ut(I.e,0),17),x=s.c),I.g.c.length==1&&(s=h(ut(I.g,0),17),x=s.d);if(a==1&&x.e.c.length+x.g.c.length==1&&!ft(lt(U(x.i,A_))))TNn(v,s,x,x.i),IE(m);else{for(oe=new qe,P=new Q(v.j);P.a<P.c.c.length;){for(I=h(K(P),11),A=new Q(I.g);A.a<A.c.c.length;)_=h(K(A),17),_.d.g.c.length==0||(oe.c[oe.c.length]=_);for(p=new Q(I.e);p.a<p.c.c.length;)u=h(K(p),17),u.c.e.c.length==0||(oe.c[oe.c.length]=u)}for(J=new Q(oe);J.a<J.c.c.length;)V=h(K(J),17),jb(V,!0)}}t.n&&Sp(t,"Found "+n+" comment boxes"),An(t)}function ZOn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V;if(I=Pe(Je(U(e,(tt(),h8)))),P=Pe(Je(U(e,f8))),A=Pe(Je(U(e,lm))),v=e.o,u=h(ut(e.j,0),11),p=u.n,V=M_n(u,A),!!V){if(t.Hc((Qc(),Ud)))switch(h(U(e,(He(),co)),61).g){case 1:V.c=(v.a-V.b)/2-p.a,V.d=P;break;case 3:V.c=(v.a-V.b)/2-p.a,V.d=-P-V.a;break;case 2:n&&u.e.c.length==0&&u.g.c.length==0?(_=s?V.a:h(ut(u.f,0),70).o.b,V.d=(v.b-_)/2-p.b):V.d=v.b+P-p.b,V.c=-I-V.b;break;case 4:n&&u.e.c.length==0&&u.g.c.length==0?(_=s?V.a:h(ut(u.f,0),70).o.b,V.d=(v.b-_)/2-p.b):V.d=v.b+P-p.b,V.c=I}else if(t.Hc(Zg))switch(h(U(e,(He(),co)),61).g){case 1:case 3:V.c=p.a+I;break;case 2:case 4:n&&!u.c?(_=s?V.a:h(ut(u.f,0),70).o.b,V.d=(v.b-_)/2-p.b):V.d=p.b+P}for(a=V.d,x=new Q(u.f);x.a<x.c.c.length;)m=h(K(x),70),j=m.n,j.a=V.c,j.b=a,a+=m.o.b+A}}function JOn(){wr(wL,new oVe),wr(tz,new wVe),wr(mL,new MVe),wr(PEe,new $Ve),wr(it,new VVe),wr(ie(Vc,1),new UVe),wr(es,new qVe),wr(A5,new YVe),wr(it,new QGe),wr(it,new ZGe),wr(it,new JGe),wr(Ts,new eVe),wr(it,new tVe),wr(xl,new nVe),wr(xl,new rVe),wr(it,new iVe),wr(zx,new sVe),wr(it,new aVe),wr(it,new cVe),wr(it,new uVe),wr(it,new lVe),wr(it,new hVe),wr(ie(Vc,1),new fVe),wr(it,new dVe),wr(it,new gVe),wr(xl,new pVe),wr(xl,new bVe),wr(it,new vVe),wr(Ws,new mVe),wr(it,new yVe),wr(Xb,new kVe),wr(it,new xVe),wr(it,new EVe),wr(it,new _Ve),wr(it,new TVe),wr(xl,new CVe),wr(xl,new SVe),wr(it,new AVe),wr(it,new DVe),wr(it,new LVe),wr(it,new IVe),wr(it,new OVe),wr(it,new NVe),wr(Qb,new PVe),wr(it,new BVe),wr(it,new FVe),wr(it,new RVe),wr(Qb,new jVe),wr(Xb,new HVe),wr(it,new zVe),wr(Ws,new GVe)}function Hve(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;if(I=t.length,I>0&&(m=(er(0,t.length),t.charCodeAt(0)),m!=64)){if(m==37&&(A=t.lastIndexOf("%"),x=!1,A!=0&&(A==I-1||(x=(er(A+1,t.length),t.charCodeAt(A+1)==46))))){if(p=t.substr(1,A-1),le=Nt("%",p)?null:zve(p),s=0,x)try{s=Wu(t.substr(A+2),$s,Ir)}catch(we){throw we=ui(we),ue(we,127)?(v=we,ne(new MB(v))):ne(we)}for(J=xge(e.Wg());J.Ob();)if(j=iF(J),ue(j,510)&&(a=h(j,590),fe=a.d,(le==null?fe==null:Nt(le,fe))&&s--==0))return a;return null}if(_=t.lastIndexOf("."),P=_==-1?t:t.substr(0,_),n=0,_!=-1)try{n=Wu(t.substr(_+1),$s,Ir)}catch(we){if(we=ui(we),ue(we,127))P=t;else throw ne(we)}for(P=Nt("%",P)?null:zve(P),V=xge(e.Wg());V.Ob();)if(j=iF(V),ue(j,191)&&(u=h(j,191),oe=u.ne(),(P==null?oe==null:Nt(P,oe))&&n--==0))return u;return null}return A2t(e,t)}function eNn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn;for(Ve=new qe,j=new Q(e.b);j.a<j.c.c.length;)for(P=h(K(j),29),oe=new Q(P.a);oe.a<oe.c.c.length;)if(V=h(K(oe),10),V.k==(bn(),Pi)&&!!os(V,(He(),F$))){for(fe=null,we=null,le=null,xt=new Q(V.j);xt.a<xt.c.c.length;)switch(wt=h(K(xt),11),wt.j.g){case 4:fe=wt;break;case 2:we=wt;break;default:le=wt}for(Ne=h(ut(le.g,0),17),_=new dN(Ne.a),x=new _a(le.n),zr(x,V.n),A=pr(_,0),nA(A,x),rt=GA(Ne.a),I=new _a(le.n),zr(I,V.n),Ai(rt,I,rt.c.b,rt.c),ln=h(U(V,F$),10),dn=h(ut(ln.j,0),11),m=h(Mf(fe.e,pe(Ite,gJ,17,0,0,1)),474),s=m,u=0,v=s.length;u<v;++u)t=s[u],xs(t,dn),aX(t.a,t.a.b,_);for(m=i1(we.g),n=m,a=0,p=n.length;a<p;++a)t=n[a],Gs(t,dn),aX(t.a,0,rt);Gs(Ne,null),xs(Ne,null),Ve.c[Ve.c.length]=V}for(J=new Q(Ve);J.a<J.c.c.length;)V=h(K(J),10),Ca(V,null)}function sbt(){sbt=re;var e,t,n;for(new AA(1,0),new AA(10,0),new AA(0,0),p3t=pe(rte,Re,240,11,0,1),l2=pe(Cl,o1,25,100,15,1),I3e=se(ie(ks,1),ka,25,15,[1,5,25,125,625,3125,15625,78125,390625,1953125,9765625,48828125,244140625,1220703125,6103515625,30517578125,152587890625,762939453125,3814697265625,19073486328125,95367431640625,476837158203125,0x878678326eac9]),O3e=pe(Hn,hr,25,I3e.length,15,1),N3e=se(ie(ks,1),ka,25,15,[1,10,100,_0,1e4,jZ,1e6,1e7,1e8,SR,1e10,1e11,1e12,1e13,1e14,1e15,1e16]),P3e=pe(Hn,hr,25,N3e.length,15,1),B3e=pe(rte,Re,240,11,0,1),e=0;e<B3e.length;e++)p3t[e]=new AA(e,0),B3e[e]=new AA(0,e),l2[e]=48;for(;e<l2.length;e++)l2[e]=48;for(n=0;n<O3e.length;n++)O3e[n]=vbe(I3e[n]);for(t=0;t<P3e.length;t++)P3e[t]=vbe(N3e[t]);cx()}function tNn(){function e(){this.obj=this.createObject()}return e.prototype.createObject=function(t){return Object.create(null)},e.prototype.get=function(t){return this.obj[t]},e.prototype.set=function(t,n){this.obj[t]=n},e.prototype[GZ]=function(t){delete this.obj[t]},e.prototype.keys=function(){return Object.getOwnPropertyNames(this.obj)},e.prototype.entries=function(){var t=this.keys(),n=this,s=0;return{next:function(){if(s>=t.length)return{done:!0};var a=t[s++];return{value:[a,n.get(a)],done:!1}}}},sMn()||(e.prototype.createObject=function(){return{}},e.prototype.get=function(t){return this.obj[":"+t]},e.prototype.set=function(t,n){this.obj[":"+t]=n},e.prototype[GZ]=function(t){delete this.obj[":"+t]},e.prototype.keys=function(){var t=[];for(var n in this.obj)n.charCodeAt(0)==58&&t.push(n.substring(1));return t}),e}function nNn(e){yve();var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;if(e==null)return null;if(A=e.length*8,A==0)return"";for(v=A%24,P=A/24|0,I=v!=0?P+1:P,u=null,u=pe(Cl,o1,25,I*4,15,1),x=0,_=0,t=0,n=0,s=0,p=0,a=0,m=0;m<P;m++)t=e[a++],n=e[a++],s=e[a++],_=(n&15)<<24>>24,x=(t&3)<<24>>24,j=(t&-128)==0?t>>2<<24>>24:(t>>2^192)<<24>>24,V=(n&-128)==0?n>>4<<24>>24:(n>>4^240)<<24>>24,J=(s&-128)==0?s>>6<<24>>24:(s>>6^252)<<24>>24,u[p++]=H0[j],u[p++]=H0[V|x<<4],u[p++]=H0[_<<2|J],u[p++]=H0[s&63];return v==8?(t=e[a],x=(t&3)<<24>>24,j=(t&-128)==0?t>>2<<24>>24:(t>>2^192)<<24>>24,u[p++]=H0[j],u[p++]=H0[x<<4],u[p++]=61,u[p++]=61):v==16&&(t=e[a],n=e[a+1],_=(n&15)<<24>>24,x=(t&3)<<24>>24,j=(t&-128)==0?t>>2<<24>>24:(t>>2^192)<<24>>24,V=(n&-128)==0?n>>4<<24>>24:(n>>4^240)<<24>>24,u[p++]=H0[j],u[p++]=H0[V|x<<4],u[p++]=H0[_<<2],u[p++]=61),Fl(u,0,u.length)}function rNn(e,t){var n,s,a,u,p,v,m;if(e.e==0&&e.p>0&&(e.p=-(e.p-1)),e.p>$s&&k0e(t,e.p-Og),p=t.q.getDate(),cA(t,1),e.k>=0&&jwn(t,e.k),e.c>=0?cA(t,e.c):e.k>=0?(m=new zge(t.q.getFullYear()-Og,t.q.getMonth(),35),s=35-m.q.getDate(),cA(t,d.Math.min(s,p))):cA(t,p),e.f<0&&(e.f=t.q.getHours()),e.b>0&&e.f<12&&(e.f+=12),Xdn(t,e.f==24&&e.g?0:e.f),e.j>=0&&Xyn(t,e.j),e.n>=0&&g3n(t,e.n),e.i>=0&&LJe(t,Vs(js(oM(xc(t.q.getTime()),_0),_0),e.i)),e.a&&(a=new mN,k0e(a,a.q.getFullYear()-Og-80),Yq(xc(t.q.getTime()),xc(a.q.getTime()))&&k0e(t,a.q.getFullYear()-Og+100)),e.d>=0){if(e.c==-1)n=(7+e.d-t.q.getDay())%7,n>3&&(n-=7),v=t.q.getMonth(),cA(t,t.q.getDate()+n),t.q.getMonth()!=v&&cA(t,t.q.getDate()+(n>0?-7:7));else if(t.q.getDay()!=e.d)return!1}return e.o>$s&&(u=t.q.getTimezoneOffset(),LJe(t,Vs(xc(t.q.getTime()),(e.o-u)*60*_0))),!0}function abt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;if(a=U(t,(He(),Br)),!!ue(a,239)){for(j=h(a,33),V=t.e,I=new _a(t.c),u=t.d,I.a+=u.b,I.b+=u.d,we=h(pt(j,(tt(),Q$)),174),Pc(we,(fu(),RH))&&(P=h(pt(j,Cke),116),Lhn(P,u.a),jhn(P,u.d),Ihn(P,u.b),$hn(P,u.c)),n=new qe,_=new Q(t.a);_.a<_.c.c.length;)for(m=h(K(_),10),ue(U(m,Br),239)?uNn(m,I):ue(U(m,Br),186)&&!V&&(s=h(U(m,Br),118),fe=Qpt(t,m,s.g,s.f),bf(s,fe.a,fe.b)),oe=new Q(m.j);oe.a<oe.c.c.length;)J=h(K(oe),11),Ti(Jr(new Vt(null,new Wt(J.g,16)),new sYe(m)),new aYe(n));if(V)for(oe=new Q(V.j);oe.a<oe.c.c.length;)J=h(K(oe),11),Ti(Jr(new Vt(null,new Wt(J.g,16)),new oYe(V)),new cYe(n));for(le=h(pt(j,Hd),218),v=new Q(n);v.a<v.c.c.length;)p=h(K(v),17),xOn(p,le,I);for(bDn(t),x=new Q(t.a);x.a<x.c.c.length;)m=h(K(x),10),A=m.e,A&&abt(e,A)}}function obt(e){J2(e,new Xp(xhe(fN(K2(q2(W2(Y2(new mp,Xu),"ELK Force"),"Force-based algorithm provided by the Eclipse Layout Kernel. Implements methods that follow physical analogies by simulating forces that move the nodes into a balanced distribution. Currently the original Eades model and the Fruchterman - Reingold model are supported."),new fh),Xu),ni((ux(),YH),se(ie(WH,1),ze,237,0,[UH]))))),et(e,Xu,UM,Ke(1)),et(e,Xu,zw,80),et(e,Xu,hJ,5),et(e,Xu,k5,Mx),et(e,Xu,BR,Ke(1)),et(e,Xu,H9,(on(),!0)),et(e,Xu,Vb,N4e),et(e,Xu,qM,at(L4e)),et(e,Xu,fJ,at(P4e)),et(e,Xu,FR,!1),et(e,Xu,z9,at(O4e)),et(e,Xu,x5,at(U4t)),et(e,Xu,Yy,at(V4t)),et(e,Xu,Dx,at(G4t)),et(e,Xu,Lx,at(Y4t)),et(e,Xu,PR,at(I4e)),et(e,Xu,uJ,at(Ete)),et(e,Xu,Swe,at(Qj)),et(e,Xu,lJ,at(xte)),et(e,Xu,Awe,at(B4e))}function cbt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P;if(!h(h(jr(e.r,t),21),84).dc()){if(p=h(wa(e.b,t),124),m=p.i,v=p.n,_=LQ(e,t),s=m.b-v.b-v.c,a=p.a.a,u=m.c+v.b,P=e.w,(_==(Cy(),Xg)||_==sL)&&h(h(jr(e.r,t),21),84).gc()==1&&(a=_==Xg?a-2*e.w:a,_=nT),s<a&&!e.B.Hc((fu(),jH)))_==Xg?(P+=(s-a)/(h(h(jr(e.r,t),21),84).gc()+1),u+=P):P+=(s-a)/(h(h(jr(e.r,t),21),84).gc()-1);else switch(s<a&&(a=_==Xg?a-2*e.w:a,_=nT),_.g){case 3:u+=(s-a)/2;break;case 4:u+=s-a;break;case 0:n=(s-a)/(h(h(jr(e.r,t),21),84).gc()+1),P+=d.Math.max(0,n),u+=P;break;case 1:n=(s-a)/(h(h(jr(e.r,t),21),84).gc()-1),P+=d.Math.max(0,n)}for(I=h(h(jr(e.r,t),21),84).Kc();I.Ob();)A=h(I.Pb(),111),A.e.a=u+A.d.b,A.e.b=(x=A.b,x.Xe((Tr(),pu))?x.Hf()==(Qe(),sn)?-x.rf().b-Pe(Je(x.We(pu))):Pe(Je(x.We(pu))):x.Hf()==(Qe(),sn)?-x.rf().b:0),u+=A.d.b+A.b.rf().a+A.d.c+P}}function ubt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;if(!h(h(jr(e.r,t),21),84).dc()){if(p=h(wa(e.b,t),124),m=p.i,v=p.n,A=LQ(e,t),s=m.a-v.d-v.a,a=p.a.b,u=m.d+v.d,j=e.w,x=e.o.a,(A==(Cy(),Xg)||A==sL)&&h(h(jr(e.r,t),21),84).gc()==1&&(a=A==Xg?a-2*e.w:a,A=nT),s<a&&!e.B.Hc((fu(),jH)))A==Xg?(j+=(s-a)/(h(h(jr(e.r,t),21),84).gc()+1),u+=j):j+=(s-a)/(h(h(jr(e.r,t),21),84).gc()-1);else switch(s<a&&(a=A==Xg?a-2*e.w:a,A=nT),A.g){case 3:u+=(s-a)/2;break;case 4:u+=s-a;break;case 0:n=(s-a)/(h(h(jr(e.r,t),21),84).gc()+1),j+=d.Math.max(0,n),u+=j;break;case 1:n=(s-a)/(h(h(jr(e.r,t),21),84).gc()-1),j+=d.Math.max(0,n)}for(P=h(h(jr(e.r,t),21),84).Kc();P.Ob();)I=h(P.Pb(),111),I.e.a=(_=I.b,_.Xe((Tr(),pu))?_.Hf()==(Qe(),un)?-_.rf().a-Pe(Je(_.We(pu))):x+Pe(Je(_.We(pu))):_.Hf()==(Qe(),un)?-_.rf().a:x),I.e.b=u+I.d.d,u+=I.d.d+I.b.rf().b+I.d.a+j}}function iNn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(e.n=Pe(Je(U(e.g,(tt(),uv)))),e.e=Pe(Je(U(e.g,b2))),e.i=e.g.b.c.length,v=e.i-1,I=0,e.j=0,e.k=0,e.a=Ef(pe(Ws,Re,19,e.i,0,1)),e.b=Ef(pe(Ts,Re,333,e.i,7,1)),p=new Q(e.g.b);p.a<p.c.c.length;){for(a=h(K(p),29),a.p=v,A=new Q(a.a);A.a<A.c.c.length;)_=h(K(A),10),_.p=I,++I;--v}for(e.f=pe(Hn,hr,25,I,15,1),e.c=Ap(Hn,[Re,hr],[48,25],15,[I,3],2),e.o=new qe,e.p=new qe,t=0,e.d=0,u=new Q(e.g.b);u.a<u.c.c.length;){for(a=h(K(u),29),v=a.p,s=0,V=0,m=a.a.c.length,x=0,A=new Q(a.a);A.a<A.c.c.length;)_=h(K(A),10),I=_.p,e.f[I]=_.c.p,x+=_.o.b+e.n,n=Td(new Sn(Dn(za(_).a.Kc(),new L))),j=Td(new Sn(Dn(qi(_).a.Kc(),new L))),e.c[I][0]=j-n,e.c[I][1]=n,e.c[I][2]=j,s+=n,V+=j,n>0&&Ue(e.p,_),Ue(e.o,_);t-=s,P=m+t,x+=t*e.e,dl(e.a,v,Ke(P)),dl(e.b,v,x),e.j=d.Math.max(e.j,P),e.k=d.Math.max(e.k,x),e.d+=t,t+=V}}function Qe(){Qe=re;var e;to=new FS(j9,0),sn=new FS(LR,1),pn=new FS(JZ,2),Fn=new FS(eJ,3),un=new FS(tJ,4),Hf=(Rt(),new G6((e=h(Vh(ta),9),new ll(e,h(vh(e,e.length),9),0)))),Ph=b0(ni(sn,se(ie(ta,1),yo,61,0,[]))),_l=b0(ni(pn,se(ie(ta,1),yo,61,0,[]))),tl=b0(ni(Fn,se(ie(ta,1),yo,61,0,[]))),Ql=b0(ni(un,se(ie(ta,1),yo,61,0,[]))),Cc=b0(ni(sn,se(ie(ta,1),yo,61,0,[Fn]))),uc=b0(ni(pn,se(ie(ta,1),yo,61,0,[un]))),Bh=b0(ni(sn,se(ie(ta,1),yo,61,0,[un]))),eu=b0(ni(sn,se(ie(ta,1),yo,61,0,[pn]))),nl=b0(ni(Fn,se(ie(ta,1),yo,61,0,[un]))),Tl=b0(ni(pn,se(ie(ta,1),yo,61,0,[Fn]))),tu=b0(ni(sn,se(ie(ta,1),yo,61,0,[pn,un]))),Tc=b0(ni(pn,se(ie(ta,1),yo,61,0,[Fn,un]))),Sc=b0(ni(sn,se(ie(ta,1),yo,61,0,[Fn,un]))),Gc=b0(ni(sn,se(ie(ta,1),yo,61,0,[pn,Fn]))),Yo=b0(ni(sn,se(ie(ta,1),yo,61,0,[pn,Fn,un])))}function lbt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;if(t.b!=0){for(P=new di,v=null,j=null,s=Li(d.Math.floor(d.Math.log(t.b)*d.Math.LOG10E)+1),m=0,le=pr(t,0);le.b!=le.d.c;)for(oe=h(gr(le),86),Me(j)!==Me(U(oe,(fo(),V_)))&&(j=Jn(U(oe,V_)),m=0),j!=null?v=j+Lst(m++,s):v=Lst(m++,s),Fe(oe,V_,v),J=(a=pr(new ng(oe).a.d,0),new k4(a));pN(J.a);)V=h(gr(J.a),188).c,Ai(P,V,P.c.b,P.c),Fe(V,V_,v);for(I=new zn,p=0;p<v.length-s;p++)for(fe=pr(t,0);fe.b!=fe.d.c;)oe=h(gr(fe),86),x=$u(Jn(U(oe,(fo(),V_))),0,p+1),n=(x==null?no(Ia(I.f,null)):Z7(I.g,x))!=null?h(x==null?no(Ia(I.f,null)):Z7(I.g,x),19).a+1:1,Ta(I,x,Ke(n));for(A=new Gp(new xp(I).a);A.b;)_=kb(A),u=Ke(_n(e.a,_.cd())!=null?h(_n(e.a,_.cd()),19).a:0),Ta(e.a,Jn(_.cd()),Ke(h(_.dd(),19).a+u.a)),u=h(_n(e.b,_.cd()),19),(!u||u.a<h(_.dd(),19).a)&&Ta(e.b,Jn(_.cd()),h(_.dd(),19));lbt(e,P)}}function sNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;for(Bn(n,"Interactive node layering",1),s=new qe,P=new Q(t.a);P.a<P.c.c.length;){for(A=h(K(P),10),x=A.n.a,m=x+A.o.a,m=d.Math.max(x+1,m),oe=new Ms(s,0),a=null;oe.b<oe.d.gc();)if(V=(kn(oe.b<oe.d.gc()),h(oe.d.Xb(oe.c=oe.b++),569)),V.c>=m){kn(oe.b>0),oe.a.Xb(oe.c=--oe.b);break}else V.a>x&&(a?(Vi(a.b,V.b),a.a=d.Math.max(a.a,V.a),Mu(oe)):(Ue(V.b,A),V.c=d.Math.min(V.c,x),V.a=d.Math.max(V.a,m),a=V));a||(a=new YKe,a.c=x,a.a=m,iw(oe,a),Ue(a.b,A))}for(v=t.b,_=0,J=new Q(s);J.a<J.c.c.length;)for(V=h(K(J),569),u=new Nl(t),u.p=_++,v.c[v.c.length]=u,j=new Q(V.b);j.a<j.c.c.length;)A=h(K(j),10),Ca(A,u),A.p=0;for(I=new Q(t.a);I.a<I.c.c.length;)A=h(K(I),10),A.p==0&&qgt(e,A,t);for(p=new Ms(v,0);p.b<p.d.gc();)(kn(p.b<p.d.gc()),h(p.d.Xb(p.c=p.b++),29)).a.c.length==0&&Mu(p);t.a.c=pe(yn,st,1,0,5,1),An(n)}function aNn(e,t,n){var s,a,u,p,v,m,x,_,A,I;if(t.e.c.length!=0&&n.e.c.length!=0){if(s=h(ut(t.e,0),17).c.i,p=h(ut(n.e,0),17).c.i,s==p)return dc(h(U(h(ut(t.e,0),17),(He(),Eo)),19).a,h(U(h(ut(n.e,0),17),Eo),19).a);for(_=e.a,A=0,I=_.length;A<I;++A){if(x=_[A],x==s)return 1;if(x==p)return-1}}return t.g.c.length!=0&&n.g.c.length!=0?(u=h(U(t,(He(),gne)),10),m=h(U(n,gne),10),a=0,v=0,os(h(ut(t.g,0),17),Eo)&&(a=h(U(h(ut(t.g,0),17),Eo),19).a),os(h(ut(n.g,0),17),Eo)&&(v=h(U(h(ut(t.g,0),17),Eo),19).a),!!u&&u==m?ft(lt(U(h(ut(t.g,0),17),Ff)))&&!ft(lt(U(h(ut(n.g,0),17),Ff)))?1:!ft(lt(U(h(ut(t.g,0),17),Ff)))&&ft(lt(U(h(ut(n.g,0),17),Ff)))||a<v?-1:a>v?1:0:(e.b&&(e.b._b(u)&&(a=h(e.b.xc(u),19).a),e.b._b(m)&&(v=h(e.b.xc(m),19).a)),a<v?-1:a>v?1:0)):t.e.c.length!=0&&n.g.c.length!=0?1:-1}function oNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt;for(Bn(t,Jvt,1),V=new qe,Ve=new qe,x=new Q(e.b);x.a<x.c.c.length;)for(m=h(K(x),29),oe=-1,j=hA(m.a),A=j,I=0,P=A.length;I<P;++I)if(_=A[I],++oe,!!(_.k==(bn(),Wi)&&oy(h(U(_,(tt(),Ei)),98)))){for(F4(h(U(_,(tt(),Ei)),98))||eAn(_),Fe(_,(He(),rv),_),V.c=pe(yn,st,1,0,5,1),Ve.c=pe(yn,st,1,0,5,1),n=new qe,we=new di,zK(we,x0(_,(Qe(),sn))),Ebt(e,we,V,Ve,n),v=oe,rt=_,u=new Q(V);u.a<u.c.c.length;)s=h(K(u),10),Sw(s,v,m),++oe,Fe(s,rv,_),p=h(ut(s.j,0),11),J=h(U(p,Br),11),ft(lt(U(J,mne)))||h(U(s,B5),15).Fc(rt);for(Pl(we),le=x0(_,Fn).Kc();le.Ob();)fe=h(le.Pb(),11),Ai(we,fe,we.a,we.a.a);for(Ebt(e,we,Ve,null,n),Ne=_,a=new Q(Ve);a.a<a.c.c.length;)s=h(K(a),10),Sw(s,++oe,m),Fe(s,rv,_),p=h(ut(s.j,0),11),J=h(U(p,Br),11),ft(lt(U(J,mne)))||h(U(Ne,B5),15).Fc(s);n.c.length==0||Fe(_,d6e,n)}An(t)}function hbt(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts;for(A=h(U(e,(kg(),D5)),33),oe=Ir,fe=Ir,V=$s,J=$s,we=new Q(e.e);we.a<we.c.c.length;)le=h(K(we),144),xt=le.d,ln=le.e,oe=d.Math.min(oe,xt.a-ln.a/2),fe=d.Math.min(fe,xt.b-ln.b/2),V=d.Math.max(V,xt.a+ln.a/2),J=d.Math.max(J,xt.b+ln.b/2);for(wt=h(pt(A,(Jh(),q4t)),116),rt=new dt(wt.b-oe,wt.d-fe),v=new Q(e.e);v.a<v.c.c.length;)p=h(K(v),144),Ve=U(p,D5),ue(Ve,239)&&(P=h(Ve,33),Ne=zr(p.d,rt),bf(P,Ne.a-P.g/2,Ne.b-P.f/2));for(s=new Q(e.c);s.a<s.c.c.length;)n=h(K(s),282),x=h(U(n,D5),79),_=Fy(x,!0,!0),dn=(Dr=ms(ro(n.d.d),n.c.d),rx(Dr,n.c.e.a,n.c.e.b),zr(Dr,n.c.d)),eP(_,dn.a,dn.b),t=(ts=ms(ro(n.c.d),n.d.d),rx(ts,n.d.e.a,n.d.e.b),zr(ts,n.d.d)),JN(_,t.a,t.b);for(u=new Q(e.d);u.a<u.c.c.length;)a=h(K(u),447),I=h(U(a,D5),137),j=zr(a.d,rt),bf(I,j.a,j.b);Ln=V-oe+(wt.b+wt.c),m=J-fe+(wt.d+wt.a),$b(A,Ln,m,!1,!0)}function cNn(e){var t,n,s,a,u,p,v,m,x,_,A,I;for(n=null,m=null,a=h(U(e.b,(tt(),Tne)),376),a==(bA(),RD)&&(n=new qe,m=new qe),v=new Q(e.d);v.a<v.c.c.length;)if(p=h(K(v),101),u=p.i,!!u)switch(p.e.g){case 0:t=h(zE(new A4(p.b)),61),a==RD&&t==(Qe(),sn)?n.c[n.c.length]=p:a==RD&&t==(Qe(),Fn)?m.c[m.c.length]=p:Y8n(p,t);break;case 1:x=p.a.d.j,_=p.c.d.j,x==(Qe(),sn)?l0(p,sn,(Sf(),Xx),p.a):_==sn?l0(p,sn,(Sf(),Qx),p.c):x==Fn?l0(p,Fn,(Sf(),Qx),p.a):_==Fn&&l0(p,Fn,(Sf(),Xx),p.c);break;case 2:case 3:s=p.b,Pc(s,(Qe(),sn))?Pc(s,Fn)?Pc(s,un)?Pc(s,pn)||l0(p,sn,(Sf(),Qx),p.c):l0(p,sn,(Sf(),Xx),p.a):l0(p,sn,(Sf(),L5),null):l0(p,Fn,(Sf(),L5),null);break;case 4:A=p.a.d.j,I=p.a.d.j,A==(Qe(),sn)||I==sn?l0(p,Fn,(Sf(),L5),null):l0(p,sn,(Sf(),L5),null)}n&&(n.c.length==0||i2t(n,(Qe(),sn)),m.c.length==0||i2t(m,(Qe(),Fn)))}function uNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V;for(s=h(U(e,(He(),Br)),33),j=h(U(e,(tt(),G$)),19).a,u=h(U(e,Y$),19).a,ya(s,G$,Ke(j)),ya(s,Y$,Ke(u)),yc(s,e.n.a+t.a),kc(s,e.n.b+t.b),(h(pt(s,g2),174).gc()!=0||!!e.e||Me(U(Us(e),X$))===Me((f9(),P_))&&QJe((Tw(),(e.q?e.q:(Rt(),Rt(),K1))._b(av)?I=h(U(e,av),197):I=h(U(Us(e),L_),197),I)))&&(Eb(s,e.o.a),xb(s,e.o.b)),A=new Q(e.j);A.a<A.c.c.length;)x=h(K(A),11),V=U(x,Br),ue(V,186)&&(a=h(V,118),bf(a,x.n.a,x.n.b),ya(a,u8,x.j));for(P=h(U(e,sv),174).gc()!=0,m=new Q(e.b);m.a<m.c.c.length;)p=h(K(m),70),(P||h(U(p,sv),174).gc()!=0)&&(n=h(U(p,Br),137),ZN(n,p.o.a,p.o.b),bf(n,p.n.a,p.n.b));if(!DE(h(U(e,cm),21)))for(_=new Q(e.j);_.a<_.c.c.length;)for(x=h(K(_),11),v=new Q(x.f);v.a<v.c.c.length;)p=h(K(v),70),n=h(U(p,Br),137),Eb(n,p.o.a),xb(n,p.o.b),bf(n,p.n.a,p.n.b)}function lNn(e){var t,n,s,a,u;switch(aA(e,_mt),(!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i+(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c).i){case 0:throw ne(new an("The edge must have at least one source or target."));case 1:return(!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i==0?bi(Na(h(he((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c),0),82))):bi(Na(h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82)))}if((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b).i==1&&(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c).i==1){if(a=Na(h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82)),u=Na(h(he((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c),0),82)),bi(a)==bi(u))return bi(a);if(a==bi(u))return a;if(u==bi(a))return u}for(s=gg(Cf(se(ie(Nf,1),st,20,0,[(!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),(!e.c&&(e.c=new qt(Pn,e,5,8)),e.c)]))),t=Na(h(Un(s),82));rr(s);)if(n=Na(h(Un(s),82)),n!=t&&!ww(n,t)){if(bi(n)==bi(t))t=bi(n);else if(t=KCn(t,n),!t)return null}return t}function hNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;for(Bn(n,"Polyline edge routing",1),J=Pe(Je(U(t,(tt(),cke)))),P=Pe(Je(U(t,b3))),a=Pe(Je(U(t,p3))),s=d.Math.min(1,a/P),le=0,m=0,t.b.c.length!=0&&(we=hgt(h(ut(t.b,0),29)),le=.4*s*we),v=new Ms(t.b,0);v.b<v.d.gc();){for(p=(kn(v.b<v.d.gc()),h(v.d.Xb(v.c=v.b++),29)),u=WN(p,$D),u&&le>0&&(le-=P),Lve(p,le),_=0,I=new Q(p.a);I.a<I.c.c.length;){for(A=h(K(I),10),x=0,V=new Sn(Dn(qi(A).a.Kc(),new L));rr(V);)j=h(Un(V),17),oe=kf(j.c).b,fe=kf(j.d).b,p==j.d.i.c&&!Xs(j)&&(PTn(j,le,.4*s*d.Math.abs(oe-fe)),j.c.j==(Qe(),un)&&(oe=0,fe=0)),x=d.Math.max(x,d.Math.abs(fe-oe));switch(A.k.g){case 0:case 4:case 1:case 3:case 5:LIn(e,A,le,J)}_=d.Math.max(_,x)}v.b<v.d.gc()&&(we=hgt((kn(v.b<v.d.gc()),h(v.d.Xb(v.c=v.b++),29))),_=d.Math.max(_,we),kn(v.b>0),v.a.Xb(v.c=--v.b)),m=.4*s*_,!u&&v.b<v.d.gc()&&(m+=P),le+=p.c.a+m}e.a.a.$b(),t.f.a=le,An(n)}function fNn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe;for(_=new zn,m=new pb,s=new Q(e.a.a.b);s.a<s.c.c.length;)if(t=h(K(s),57),x=by(t),x)sc(_.f,x,t);else if(fe=W4(t),fe)for(u=new Q(fe.k);u.a<u.c.c.length;)a=h(K(u),17),Ot(m,a,t);for(n=new Q(e.a.a.b);n.a<n.c.c.length;)if(t=h(K(n),57),x=by(t),x){for(v=new Sn(Dn(qi(x).a.Kc(),new L));rr(v);)if(p=h(Un(v),17),!Xs(p)&&(j=p.c,oe=p.d,!((Qe(),Cc).Hc(p.c.j)&&Cc.Hc(p.d.j)))){if(V=h(_n(_,p.d.i),57),Th(ph(gh(bh(dh(new Il,0),100),e.c[t.a.d]),e.c[V.a.d])),j.j==un&&vrt((Rc(),j))){for(I=h(jr(m,p),21).Kc();I.Ob();)if(A=h(I.Pb(),57),A.d.c<t.d.c){if(P=e.c[A.a.d],J=e.c[t.a.d],P==J)continue;Th(ph(gh(bh(dh(new Il,1),100),P),J))}}if(oe.j==pn&&brt((Rc(),oe))){for(I=h(jr(m,p),21).Kc();I.Ob();)if(A=h(I.Pb(),57),A.d.c>t.d.c){if(P=e.c[t.a.d],J=e.c[A.a.d],P==J)continue;Th(ph(gh(bh(dh(new Il,1),100),P),J))}}}}}function zve(e){nZ();var t,n,s,a,u,p,v,m;if(e==null)return null;if(a=Zf(e,Ec(37)),a<0)return e;for(m=new ju(e.substr(0,a)),t=pe(Vc,Zy,25,4,15,1),v=0,s=0,p=e.length;a<p;a++)if(er(a,e.length),e.charCodeAt(a)==37&&e.length>a+2&&WK((er(a+1,e.length),e.charCodeAt(a+1)),dEe,gEe)&&WK((er(a+2,e.length),e.charCodeAt(a+2)),dEe,gEe))if(n=N2n((er(a+1,e.length),e.charCodeAt(a+1)),(er(a+2,e.length),e.charCodeAt(a+2))),a+=2,s>0?(n&192)==128?t[v++]=n<<24>>24:s=0:n>=128&&((n&224)==192?(t[v++]=n<<24>>24,s=2):(n&240)==224?(t[v++]=n<<24>>24,s=3):(n&248)==240&&(t[v++]=n<<24>>24,s=4)),s>0){if(v==s){switch(v){case 2:{pg(m,((t[0]&31)<<6|t[1]&63)&Oi);break}case 3:{pg(m,((t[0]&15)<<12|(t[1]&63)<<6|t[2]&63)&Oi);break}}v=0,s=0}}else{for(u=0;u<v;++u)pg(m,t[u]&Oi);v=0,m.a+=String.fromCharCode(n)}else{for(u=0;u<v;++u)pg(m,t[u]&Oi);v=0,pg(m,(er(a,e.length),e.charCodeAt(a)))}return m.a}function fbt(e,t,n,s,a){var u,p,v;if(Oct(e,t),p=t[0],u=Is(n.c,0),v=-1,Wge(n))if(s>0){if(p+s>e.length)return!1;v=XF(e.substr(0,p+s),t)}else v=XF(e,t);switch(u){case 71:return v=Oy(e,p,se(ie(it,1),Re,2,6,[qbt,Ybt]),t),a.e=v,!0;case 77:return WAn(e,t,a,v,p);case 76:return KAn(e,t,a,v,p);case 69:return $_n(e,t,p,a);case 99:return H_n(e,t,p,a);case 97:return v=Oy(e,p,se(ie(it,1),Re,2,6,["AM","PM"]),t),a.b=v,!0;case 121:return XAn(e,t,p,v,n,a);case 100:return v<=0?!1:(a.c=v,!0);case 83:return v<0?!1:exn(v,p,t[0],a);case 104:v==12&&(v=0);case 75:case 72:return v<0?!1:(a.f=v,a.g=!1,!0);case 107:return v<0?!1:(a.f=v,a.g=!0,!0);case 109:return v<0?!1:(a.j=v,!0);case 115:return v<0?!1:(a.n=v,!0);case 90:if(p<e.length&&(er(p,e.length),e.charCodeAt(p)==90))return++t[0],a.o=0,!0;case 122:case 118:return h9n(e,p,t,a);default:return!1}}function dNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;if(I=h(h(jr(e.r,t),21),84),t==(Qe(),pn)||t==un){ibt(e,t);return}for(u=t==sn?(Cb(),hD):(Cb(),fD),we=t==sn?(wc(),nf):(wc(),Ah),n=h(wa(e.b,t),124),s=n.i,a=s.c+ky(se(ie(ks,1),ka,25,15,[n.n.b,e.C.b,e.k])),oe=s.c+s.b-ky(se(ie(ks,1),ka,25,15,[n.n.c,e.C.c,e.k])),p=yhe(R1e(u),e.t),fe=t==sn?Fi:ki,A=I.Kc();A.Ob();)x=h(A.Pb(),111),!(!x.c||x.c.d.c.length<=0)&&(J=x.b.rf(),V=x.e,P=x.c,j=P.i,j.b=(m=P.n,P.e.a+m.b+m.c),j.a=(v=P.n,P.e.b+v.d+v.a),aA(we,gwe),P.f=we,Yh(P,(Xc(),Sh)),j.c=V.a-(j.b-J.a)/2,Ne=d.Math.min(a,V.a),Ve=d.Math.max(oe,V.a+J.a),j.c<Ne?j.c=Ne:j.c+j.b>Ve&&(j.c=Ve-j.b),Ue(p.d,new lW(j,wpe(p,j))),fe=t==sn?d.Math.max(fe,V.b+x.b.rf().b):d.Math.min(fe,V.b));for(fe+=t==sn?e.t:-e.t,le=Npe((p.e=fe,p)),le>0&&(h(wa(e.b,t),124).a.b=le),_=I.Kc();_.Ob();)x=h(_.Pb(),111),!(!x.c||x.c.d.c.length<=0)&&(j=x.c.i,j.c-=x.e.a,j.d-=x.e.b)}function gNn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P;for(t=new zn,m=new Tn(e);m.e!=m.i.gc();){for(v=h(On(m),33),n=new ns,Pr(yte,v,n),P=new gt,a=h(Gu(new Vt(null,new cb(new Sn(Dn(wM(v).a.Kc(),new L)))),nrt(P,Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[(Af(),Zu)])))),83),Tut(n,h(a.xc((on(),!0)),14),new O6),s=h(Gu(Jr(h(a.xc(!1),15).Lc(),new It),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[Zu]))),15),p=s.Kc();p.Ob();)u=h(p.Pb(),79),I=hdt(u),I&&(x=h(no(Ia(t.f,I)),21),x||(x=Lgt(I),sc(t.f,I,x)),Qs(n,x));for(a=h(Gu(new Vt(null,new cb(new Sn(Dn(Pd(v).a.Kc(),new L)))),nrt(P,Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[Zu])))),83),Tut(n,h(a.xc(!0),14),new n0),s=h(Gu(Jr(h(a.xc(!1),15).Lc(),new jO),Bp(new Le,new Oe,new We,se(ie(gu,1),ze,132,0,[Zu]))),15),A=s.Kc();A.Ob();)_=h(A.Pb(),79),I=fdt(_),I&&(x=h(no(Ia(t.f,I)),21),x||(x=Lgt(I),sc(t.f,I,x)),Qs(n,x))}}function pNn(e,t){XQ();var n,s,a,u,p,v,m,x,_,A,I,P,j,V;if(m=mo(e,0)<0,m&&(e=Tk(e)),mo(e,0)==0)switch(t){case 0:return"0";case 1:return _x;case 2:return"0.00";case 3:return"0.000";case 4:return"0.0000";case 5:return"0.00000";case 6:return"0.000000";default:return P=new rg,t<0?P.a+="0E+":P.a+="0E",P.a+=t==$s?"2147483648":""+-t,P.a}_=18,A=pe(Cl,o1,25,_+1,15,1),n=_,V=e;do x=V,V=oM(V,10),A[--n]=Gn(Vs(48,Cg(x,js(V,10))))&Oi;while(mo(V,0)!=0);if(a=Cg(Cg(Cg(_,n),t),1),t==0)return m&&(A[--n]=45),Fl(A,n,_-n);if(t>0&&mo(a,-6)>=0){if(mo(a,0)>=0){for(u=n+Gn(a),v=_-1;v>=u;v--)A[v+1]=A[v];return A[++u]=46,m&&(A[--n]=45),Fl(A,n,_-n+1)}for(p=2;Yq(p,Vs(Tk(a),1));p++)A[--n]=48;return A[--n]=46,A[--n]=48,m&&(A[--n]=45),Fl(A,n,_-n)}return j=n+1,s=_,I=new Kv,m&&(I.a+="-"),s-j>=1?(pg(I,A[n]),I.a+=".",I.a+=Fl(A,n+1,_-n-1)):I.a+=Fl(A,n,_-n),I.a+="E",mo(a,0)>0&&(I.a+="+"),I.a+=""+ME(a),I.a}function bNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P;if(e.e.a.$b(),e.f.a.$b(),e.c.c=pe(yn,st,1,0,5,1),e.i.c=pe(yn,st,1,0,5,1),e.g.a.$b(),t)for(p=new Q(t.a);p.a<p.c.c.length;)for(u=h(K(p),10),A=x0(u,(Qe(),pn)).Kc();A.Ob();)for(_=h(A.Pb(),11),Qi(e.e,_),a=new Q(_.g);a.a<a.c.c.length;)s=h(K(a),17),!Xs(s)&&(Ue(e.c,s),Fft(e,s),v=s.c.i.k,(v==(bn(),Wi)||v==Vo||v==Pi||v==Bf)&&Ue(e.j,s),P=s.d,I=P.i.c,I==n?Qi(e.f,P):I==t?Qi(e.e,P):vc(e.c,s));if(n)for(p=new Q(n.a);p.a<p.c.c.length;){for(u=h(K(p),10),x=new Q(u.j);x.a<x.c.c.length;)for(m=h(K(x),11),a=new Q(m.g);a.a<a.c.c.length;)s=h(K(a),17),Xs(s)&&Qi(e.g,s);for(A=x0(u,(Qe(),un)).Kc();A.Ob();)for(_=h(A.Pb(),11),Qi(e.f,_),a=new Q(_.g);a.a<a.c.c.length;)s=h(K(a),17),!Xs(s)&&(Ue(e.c,s),Fft(e,s),v=s.c.i.k,(v==(bn(),Wi)||v==Vo||v==Pi||v==Bf)&&Ue(e.j,s),P=s.d,I=P.i.c,I==n?Qi(e.f,P):I==t?Qi(e.e,P):vc(e.c,s))}}function $b(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;if(J=new dt(e.g,e.f),V=ube(e),V.a=d.Math.max(V.a,t),V.b=d.Math.max(V.b,n),Ve=V.a/J.a,_=V.b/J.b,we=V.a-J.a,m=V.b-J.b,s)for(p=bi(e)?h(pt(bi(e),(Tr(),hv)),103):h(pt(e,(Tr(),hv)),103),v=Me(pt(e,(Tr(),Z_)))===Me((_s(),qo)),fe=new Tn((!e.c&&(e.c=new Ye(bu,e,9,9)),e.c));fe.e!=fe.i.gc();)switch(oe=h(On(fe),118),le=h(pt(oe,T3),61),le==(Qe(),to)&&(le=kve(oe,p),ya(oe,T3,le)),le.g){case 1:v||yc(oe,oe.i*Ve);break;case 2:yc(oe,oe.i+we),v||kc(oe,oe.j*_);break;case 3:v||yc(oe,oe.i*Ve),kc(oe,oe.j+m);break;case 4:v||kc(oe,oe.j*_)}if(ZN(e,V.a,V.b),a)for(I=new Tn((!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n));I.e!=I.i.gc();)A=h(On(I),137),P=A.i+A.g/2,j=A.j+A.f/2,Ne=P/J.a,x=j/J.b,Ne+x>=1&&(Ne-x>0&&j>=0?(yc(A,A.i+we),kc(A,A.j+m*x)):Ne-x<0&&P>=0&&(yc(A,A.i+we*Ne),kc(A,A.j+m)));return ya(e,(Tr(),m2),(Iu(),u=h(Vh(aT),9),new ll(u,h(vh(u,u.length),9),0))),new dt(Ve,_)}function dbt(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j;if(P=bi(Na(h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82))),j=bi(Na(h(he((!e.c&&(e.c=new qt(Pn,e,5,8)),e.c),0),82))),A=P==j,v=new Rs,t=h(pt(e,(uF(),j7e)),74),!!t&&t.b>=2){if((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i==0)n=(X2(),a=new GO,a),qn((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a),n);else if((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i>1)for(I=new N4((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a));I.e!=I.i.gc();)p9(I);CM(t,h(he((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a),0),202))}if(A)for(s=new Tn((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a));s.e!=s.i.gc();)for(n=h(On(s),202),x=new Tn((!n.a&&(n.a=new Gi(Zl,n,5)),n.a));x.e!=x.i.gc();)m=h(On(x),469),v.a=d.Math.max(v.a,m.a),v.b=d.Math.max(v.b,m.b);for(p=new Tn((!e.n&&(e.n=new Ye(Ua,e,1,7)),e.n));p.e!=p.i.gc();)u=h(On(p),137),_=h(pt(u,eT),8),_&&bf(u,_.a,_.b),A&&(v.a=d.Math.max(v.a,u.i+u.g),v.b=d.Math.max(v.b,u.j+u.f));return v}function vNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt;for(le=t.c.length,a=new jy(e.a,n,null,null),wt=pe(ks,ka,25,le,15,1),V=pe(ks,ka,25,le,15,1),j=pe(ks,ka,25,le,15,1),J=0,v=0;v<le;v++)V[v]=Ir,j[v]=$s;for(m=0;m<le;m++)for(s=(Kt(m,t.c.length),h(t.c[m],180)),wt[m]=TQ(s),wt[J]>wt[m]&&(J=m),A=new Q(e.a.b);A.a<A.c.c.length;)for(_=h(K(A),29),fe=new Q(_.a);fe.a<fe.c.c.length;)oe=h(K(fe),10),Ve=Pe(s.p[oe.p])+Pe(s.d[oe.p]),V[m]=d.Math.min(V[m],Ve),j[m]=d.Math.max(j[m],Ve+oe.o.b);for(rt=pe(ks,ka,25,le,15,1),x=0;x<le;x++)(Kt(x,t.c.length),h(t.c[x],180)).o==(yf(),v2)?rt[x]=V[J]-V[x]:rt[x]=j[J]-j[x];for(u=pe(ks,ka,25,le,15,1),P=new Q(e.a.b);P.a<P.c.c.length;)for(I=h(K(P),29),Ne=new Q(I.a);Ne.a<Ne.c.c.length;){for(we=h(K(Ne),10),p=0;p<le;p++)u[p]=Pe((Kt(p,t.c.length),h(t.c[p],180)).p[we.p])+Pe((Kt(p,t.c.length),h(t.c[p],180)).d[we.p])+rt[p];u.sort(cut(G.prototype.te,G,[])),a.p[we.p]=(u[1]+u[2])/2,a.d[we.p]=0}return a}function wNn(e,t,n){var s,a,u,p,v;switch(s=t.i,u=e.i.o,a=e.i.d,v=e.n,p=Qa(se(ie(cs,1),Re,8,0,[v,e.a])),e.j.g){case 1:e1(t,(wc(),Ah)),s.d=-a.d-n-s.a,h(h(ut(t.d,0),181).We((He(),F5)),285)==(Yu(),Z1)?(Yh(t,(Xc(),Sh)),s.c=p.a-Pe(Je(U(e,h3)))-n-s.b):(Yh(t,(Xc(),v1)),s.c=p.a+Pe(Je(U(e,h3)))+n);break;case 2:Yh(t,(Xc(),v1)),s.c=u.a+a.c+n,h(h(ut(t.d,0),181).We((He(),F5)),285)==(Yu(),Z1)?(e1(t,(wc(),Ah)),s.d=p.b-Pe(Je(U(e,h3)))-n-s.a):(e1(t,(wc(),nf)),s.d=p.b+Pe(Je(U(e,h3)))+n);break;case 3:e1(t,(wc(),nf)),s.d=u.b+a.a+n,h(h(ut(t.d,0),181).We((He(),F5)),285)==(Yu(),Z1)?(Yh(t,(Xc(),Sh)),s.c=p.a-Pe(Je(U(e,h3)))-n-s.b):(Yh(t,(Xc(),v1)),s.c=p.a+Pe(Je(U(e,h3)))+n);break;case 4:Yh(t,(Xc(),Sh)),s.c=-a.b-n-s.b,h(h(ut(t.d,0),181).We((He(),F5)),285)==(Yu(),Z1)?(e1(t,(wc(),Ah)),s.d=p.b-Pe(Je(U(e,h3)))-n-s.a):(e1(t,(wc(),nf)),s.d=p.b+Pe(Je(U(e,h3)))+n)}}function mNn(e,t,n,s,a,u,p){var v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts;for(P=0,ln=0,m=new Q(e);m.a<m.c.c.length;)v=h(K(m),33),X2t(v),P=d.Math.max(P,v.g),ln+=v.g*v.f;for(j=ln/e.c.length,xt=k7n(e,j),ln+=e.c.length*xt,P=d.Math.max(P,d.Math.sqrt(ln*p))+n.b,Dr=n.b,ts=n.d,I=0,_=n.b+n.c,wt=new di,vr(wt,Ke(0)),Ve=new di,x=new Ms(e,0);x.b<x.d.gc();)v=(kn(x.b<x.d.gc()),h(x.d.Xb(x.c=x.b++),33)),Ln=v.g,A=v.f,Dr+Ln>P&&(u&&(Cp(Ve,I),Cp(wt,Ke(x.b-1))),Dr=n.b,ts+=I+t,I=0,_=d.Math.max(_,n.b+n.c+Ln)),yc(v,Dr),kc(v,ts),_=d.Math.max(_,Dr+Ln+n.c),I=d.Math.max(I,A),Dr+=Ln+t;if(_=d.Math.max(_,s),dn=ts+I+n.a,dn<a&&(I+=a-dn,dn=a),u)for(Dr=n.b,x=new Ms(e,0),Cp(wt,Ke(e.c.length)),rt=pr(wt,0),oe=h(gr(rt),19).a,Cp(Ve,I),Ne=pr(Ve,0),we=0;x.b<x.d.gc();)x.b==oe&&(Dr=n.b,we=Pe(Je(gr(Ne))),oe=h(gr(rt),19).a),v=(kn(x.b<x.d.gc()),h(x.d.Xb(x.c=x.b++),33)),fe=v.f,xb(v,we),V=we,x.b==oe&&(J=_-Dr-n.c,le=v.g,Eb(v,J),Ydt(v,new dt(J,V),new dt(le,fe))),Dr+=v.g+t;return new dt(_,dn)}function yNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt;for(Bn(t,"Compound graph postprocessor",1),n=ft(lt(U(e,(tt(),Nne)))),v=h(U(e,(He(),b6e)),224),_=new ns,oe=v.ec().Kc();oe.Ob();){for(J=h(oe.Pb(),17),p=new Bc(v.cc(J)),Rt(),hs(p,new Ile(e)),Ne=n5n((Kt(0,p.c.length),h(p.c[0],243))),rt=nlt(h(ut(p,p.c.length-1),243)),le=Ne.i,Fk(rt.i,le)?fe=le.e:fe=Us(le),A=s8n(J,p),Pl(J.a),I=null,u=new Q(p);u.a<u.c.c.length;)a=h(K(u),243),V=new Rs,abe(V,a.a,fe),P=a.b,s=new Oc,aX(s,0,P.a),mw(s,V),we=new _a(kf(P.c)),Ve=new _a(kf(P.d)),zr(we,V),zr(Ve,V),I&&(s.b==0?j=Ve:j=(kn(s.b!=0),h(s.a.a.c,8)),wt=d.Math.abs(I.a-j.a)>l1,xt=d.Math.abs(I.b-j.b)>l1,(!n&&wt&&xt||n&&(wt||xt))&&vr(J.a,we)),Qs(J.a,s),s.b==0?I=we:I=(kn(s.b!=0),h(s.c.b.c,8)),P5n(P,A,V),nlt(a)==rt&&(Us(rt.i)!=a.a&&(V=new Rs,abe(V,Us(rt.i),fe)),Fe(J,wne,V)),s_n(P,J,fe),_.a.zc(P,_);Gs(J,Ne),xs(J,rt)}for(x=_.a.ec().Kc();x.Ob();)m=h(x.Pb(),17),Gs(m,null),xs(m,null);An(t)}function gbt(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;if(e.gc()==1)return h(e.Xb(0),231);if(e.gc()<=0)return new mB;for(a=e.Kc();a.Ob();){for(n=h(a.Pb(),231),j=0,_=Ir,A=Ir,m=$s,x=$s,P=new Q(n.e);P.a<P.c.c.length;)I=h(K(P),144),j+=h(U(I,(Jh(),qx)),19).a,_=d.Math.min(_,I.d.a-I.e.a/2),A=d.Math.min(A,I.d.b-I.e.b/2),m=d.Math.max(m,I.d.a+I.e.a/2),x=d.Math.max(x,I.d.b+I.e.b/2);Fe(n,(Jh(),qx),Ke(j)),Fe(n,(kg(),b_),new dt(_,A)),Fe(n,mD,new dt(m,x))}for(Rt(),e.ad(new iq),V=new mB,Oa(V,h(e.Xb(0),94)),v=0,fe=0,u=e.Kc();u.Ob();)n=h(u.Pb(),231),J=ms(ro(h(U(n,(kg(),mD)),8)),h(U(n,b_),8)),v=d.Math.max(v,J.a),fe+=J.a*J.b;for(v=d.Math.max(v,d.Math.sqrt(fe)*Pe(Je(U(V,(Jh(),H4t))))),oe=Pe(Je(U(V,Zj))),le=0,we=0,p=0,t=oe,s=e.Kc();s.Ob();)n=h(s.Pb(),231),J=ms(ro(h(U(n,(kg(),mD)),8)),h(U(n,b_),8)),le+J.a>v&&(le=0,we+=p+oe,p=0),rDn(V,n,le,we),t=d.Math.max(t,le+J.a),p=d.Math.max(p,J.b),le+=J.a+oe;return V}function pbt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j;switch(_=new Oc,e.a.g){case 3:I=h(U(t.e,(He(),f2)),15),P=h(U(t.j,f2),15),j=h(U(t.f,f2),15),n=h(U(t.e,g3),15),s=h(U(t.j,g3),15),a=h(U(t.f,g3),15),p=new qe,Vi(p,I),P.Jc(new o$e),Vi(p,ue(P,152)?X4(h(P,152)):ue(P,131)?h(P,131).a:ue(P,54)?new U2(P):new Xv(P)),Vi(p,j),u=new qe,Vi(u,n),Vi(u,ue(s,152)?X4(h(s,152)):ue(s,131)?h(s,131).a:ue(s,54)?new U2(s):new Xv(s)),Vi(u,a),Fe(t.f,f2,p),Fe(t.f,g3,u),Fe(t.f,_6e,t.f),Fe(t.e,f2,null),Fe(t.e,g3,null),Fe(t.j,f2,null),Fe(t.j,g3,null);break;case 1:Qs(_,t.e.a),vr(_,t.i.n),Qs(_,Rp(t.j.a)),vr(_,t.a.n),Qs(_,t.f.a);break;default:Qs(_,t.e.a),Qs(_,Rp(t.j.a)),Qs(_,t.f.a)}Pl(t.f.a),Qs(t.f.a,_),Gs(t.f,t.e.c),v=h(U(t.e,(tt(),Da)),74),x=h(U(t.j,Da),74),m=h(U(t.f,Da),74),(!!v||!!x||!!m)&&(A=new Oc,Mde(A,m),Mde(A,x),Mde(A,v),Fe(t.f,Da,A)),Gs(t.j,null),xs(t.j,null),Gs(t.e,null),xs(t.e,null),Ca(t.a,null),Ca(t.i,null),t.g&&pbt(e,t.g)}function kNn(e){yve();var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;if(e==null||(u=LB(e),j=ckn(u),j%4!=0))return null;if(V=j/4|0,V==0)return pe(Vc,Zy,25,0,15,1);for(A=null,t=0,n=0,s=0,a=0,p=0,v=0,m=0,x=0,P=0,I=0,_=0,A=pe(Vc,Zy,25,V*3,15,1);P<V-1;P++){if(!mS(p=u[_++])||!mS(v=u[_++])||!mS(m=u[_++])||!mS(x=u[_++]))return null;t=Gf[p],n=Gf[v],s=Gf[m],a=Gf[x],A[I++]=(t<<2|n>>4)<<24>>24,A[I++]=((n&15)<<4|s>>2&15)<<24>>24,A[I++]=(s<<6|a)<<24>>24}return!mS(p=u[_++])||!mS(v=u[_++])?null:(t=Gf[p],n=Gf[v],m=u[_++],x=u[_++],Gf[m]==-1||Gf[x]==-1?m==61&&x==61?(n&15)!=0?null:(J=pe(Vc,Zy,25,P*3+1,15,1),Ao(A,0,J,0,P*3),J[I]=(t<<2|n>>4)<<24>>24,J):m!=61&&x==61?(s=Gf[m],(s&3)!=0?null:(J=pe(Vc,Zy,25,P*3+2,15,1),Ao(A,0,J,0,P*3),J[I++]=(t<<2|n>>4)<<24>>24,J[I]=((n&15)<<4|s>>2&15)<<24>>24,J)):null:(s=Gf[m],a=Gf[x],A[I++]=(t<<2|n>>4)<<24>>24,A[I++]=((n&15)<<4|s>>2&15)<<24>>24,A[I++]=(s<<6|a)<<24>>24,A))}function xNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne;for(Bn(t,Jvt,1),j=h(U(e,(tt(),Hd)),218),a=new Q(e.b);a.a<a.c.c.length;)for(s=h(K(a),29),x=hA(s.a),p=x,v=0,m=p.length;v<m;++v)if(u=p[v],u.k==(bn(),Vo)){if(j==(Od(),W5))for(A=new Q(u.j);A.a<A.c.c.length;)_=h(K(A),11),_.e.c.length==0||Hxn(_),_.g.c.length==0||zxn(_);else if(ue(U(u,(He(),Br)),17))J=h(U(u,Br),17),oe=h(x0(u,(Qe(),un)).Kc().Pb(),11),fe=h(x0(u,pn).Kc().Pb(),11),le=h(U(oe,Br),11),we=h(U(fe,Br),11),Gs(J,we),xs(J,le),Ne=new _a(fe.i.n),Ne.a=Qa(se(ie(cs,1),Re,8,0,[we.i.n,we.n,we.a])).a,vr(J.a,Ne),Ne=new _a(oe.i.n),Ne.a=Qa(se(ie(cs,1),Re,8,0,[le.i.n,le.n,le.a])).a,vr(J.a,Ne);else{if(u.j.c.length>=2){for(V=!0,I=new Q(u.j),n=h(K(I),11),P=null;I.a<I.c.c.length;)if(P=n,n=h(K(I),11),!Nr(U(P,Br),U(n,Br))){V=!1;break}}else V=!1;for(A=new Q(u.j);A.a<A.c.c.length;)_=h(K(A),11),_.e.c.length==0||pAn(_,V),_.g.c.length==0||bAn(_,V)}Ca(u,null)}An(t)}function bbt(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt;return le=e.c[(Kt(0,t.c.length),h(t.c[0],17)).p],rt=e.c[(Kt(1,t.c.length),h(t.c[1],17)).p],le.a.e.e-le.a.a-(le.b.e.e-le.b.a)==0&&rt.a.e.e-rt.a.a-(rt.b.e.e-rt.b.a)==0||(oe=le.b.e.f,!ue(oe,10))?!1:(J=h(oe,10),Ne=e.i[J.p],Ve=J.c?Ha(J.c.a,J,0):-1,u=ki,Ve>0&&(a=h(ut(J.c.a,Ve-1),10),p=e.i[a.p],wt=d.Math.ceil(uy(e.n,a,J)),u=Ne.a.e-J.d.d-(p.a.e+a.o.b+a.d.a)-wt),x=ki,Ve<J.c.a.c.length-1&&(m=h(ut(J.c.a,Ve+1),10),_=e.i[m.p],wt=d.Math.ceil(uy(e.n,m,J)),x=_.a.e-m.d.d-(Ne.a.e+J.o.b+J.d.a)-wt),n&&(vf(),xh(d1),d.Math.abs(u-x)<=d1||u==x||isNaN(u)&&isNaN(x))?!0:(s=kW(le.a),v=-kW(le.b),A=-kW(rt.a),fe=kW(rt.b),V=le.a.e.e-le.a.a-(le.b.e.e-le.b.a)>0&&rt.a.e.e-rt.a.a-(rt.b.e.e-rt.b.a)<0,j=le.a.e.e-le.a.a-(le.b.e.e-le.b.a)<0&&rt.a.e.e-rt.a.a-(rt.b.e.e-rt.b.a)>0,P=le.a.e.e+le.b.a<rt.b.e.e+rt.a.a,I=le.a.e.e+le.b.a>rt.b.e.e+rt.a.a,we=0,!V&&!j&&(I?u+A>0?we=A:x-s>0&&(we=s):P&&(u+v>0?we=v:x-fe>0&&(we=fe))),Ne.a.e+=we,Ne.b&&(Ne.d.e+=we),!1))}function vbt(e,t,n){var s,a,u,p,v,m,x,_,A,I;if(s=new hl(t.qf().a,t.qf().b,t.rf().a,t.rf().b),a=new O4,e.c)for(p=new Q(t.wf());p.a<p.c.c.length;)u=h(K(p),181),a.c=u.qf().a+t.qf().a,a.d=u.qf().b+t.qf().b,a.b=u.rf().a,a.a=u.rf().b,v9(s,a);for(x=new Q(t.Cf());x.a<x.c.c.length;){if(m=h(K(x),838),_=m.qf().a+t.qf().a,A=m.qf().b+t.qf().b,e.e&&(a.c=_,a.d=A,a.b=m.rf().a,a.a=m.rf().b,v9(s,a)),e.d)for(p=new Q(m.wf());p.a<p.c.c.length;)u=h(K(p),181),a.c=u.qf().a+_,a.d=u.qf().b+A,a.b=u.rf().a,a.a=u.rf().b,v9(s,a);if(e.b){if(I=new dt(-n,-n),h(t.We((Tr(),gm)),174).Hc((Qc(),Zg)))for(p=new Q(m.wf());p.a<p.c.c.length;)u=h(K(p),181),I.a+=u.rf().a+n,I.b+=u.rf().b+n;I.a=d.Math.max(I.a,0),I.b=d.Math.max(I.b,0),jpt(s,m.Bf(),m.zf(),t,m,I,n)}}e.b&&jpt(s,t.Bf(),t.zf(),t,null,null,n),v=new XY(t.Af()),v.d=d.Math.max(0,t.qf().b-s.d),v.a=d.Math.max(0,s.d+s.a-(t.qf().b+t.rf().b)),v.b=d.Math.max(0,t.qf().a-s.c),v.c=d.Math.max(0,s.c+s.b-(t.qf().a+t.rf().a)),t.Ef(v)}function ENn(){var e=["\\u0000","\\u0001","\\u0002","\\u0003","\\u0004","\\u0005","\\u0006","\\u0007","\\b","\\t","\\n","\\u000B","\\f","\\r","\\u000E","\\u000F","\\u0010","\\u0011","\\u0012","\\u0013","\\u0014","\\u0015","\\u0016","\\u0017","\\u0018","\\u0019","\\u001A","\\u001B","\\u001C","\\u001D","\\u001E","\\u001F"];return e[34]='\\"',e[92]="\\\\",e[173]="\\u00ad",e[1536]="\\u0600",e[1537]="\\u0601",e[1538]="\\u0602",e[1539]="\\u0603",e[1757]="\\u06dd",e[1807]="\\u070f",e[6068]="\\u17b4",e[6069]="\\u17b5",e[8203]="\\u200b",e[8204]="\\u200c",e[8205]="\\u200d",e[8206]="\\u200e",e[8207]="\\u200f",e[8232]="\\u2028",e[8233]="\\u2029",e[8234]="\\u202a",e[8235]="\\u202b",e[8236]="\\u202c",e[8237]="\\u202d",e[8238]="\\u202e",e[8288]="\\u2060",e[8289]="\\u2061",e[8290]="\\u2062",e[8291]="\\u2063",e[8292]="\\u2064",e[8298]="\\u206a",e[8299]="\\u206b",e[8300]="\\u206c",e[8301]="\\u206d",e[8302]="\\u206e",e[8303]="\\u206f",e[65279]="\\ufeff",e[65529]="\\ufff9",e[65530]="\\ufffa",e[65531]="\\ufffb",e}function _Nn(e,t,n){var s,a,u,p,v,m,x,_,A,I;for(m=new qe,A=t.length,p=Vge(n),x=0;x<A;++x){switch(_=Ufe(t,Ec(61),x),s=akn(p,t.substr(x,_-x)),a=UK(s),u=a.Aj().Nh(),Is(t,++_)){case 39:{v=nk(t,39,++_),Ue(m,new qN(s,OW(t.substr(_,v-_),u,a))),x=v+1;break}case 34:{v=nk(t,34,++_),Ue(m,new qN(s,OW(t.substr(_,v-_),u,a))),x=v+1;break}case 91:{I=new qe,Ue(m,new qN(s,I));e:for(;;){switch(Is(t,++_)){case 39:{v=nk(t,39,++_),Ue(I,OW(t.substr(_,v-_),u,a)),_=v+1;break}case 34:{v=nk(t,34,++_),Ue(I,OW(t.substr(_,v-_),u,a)),_=v+1;break}case 110:{if(++_,t.indexOf("ull",_)==_)I.c[I.c.length]=null;else throw ne(new Ya(lmt));_+=3;break}}if(_<A)switch(er(_,t.length),t.charCodeAt(_)){case 44:break;case 93:break e;default:throw ne(new Ya("Expecting , or ]"))}else break}x=_+1;break}case 110:{if(++_,t.indexOf("ull",_)==_)Ue(m,new qN(s,null));else throw ne(new Ya(lmt));x=_+3;break}}if(x<A){if(er(x,t.length),t.charCodeAt(x)!=44)throw ne(new Ya("Expecting ,"))}else break}return zAn(e,m,n)}function wbt(e,t){var n,s,a,u,p,v,m,x,_,A,I;for(x=h(h(jr(e.r,t),21),84),p=_9n(e,t),n=e.u.Hc((Qc(),iT)),m=x.Kc();m.Ob();)if(v=h(m.Pb(),111),!(!v.c||v.c.d.c.length<=0)){switch(I=v.b.rf(),_=v.c,A=_.i,A.b=(u=_.n,_.e.a+u.b+u.c),A.a=(a=_.n,_.e.b+a.d+a.a),t.g){case 1:v.a?(A.c=(I.a-A.b)/2,Yh(_,(Xc(),jg))):p||n?(A.c=-A.b-e.s,Yh(_,(Xc(),Sh))):(A.c=I.a+e.s,Yh(_,(Xc(),v1))),A.d=-A.a-e.t,e1(_,(wc(),Ah));break;case 3:v.a?(A.c=(I.a-A.b)/2,Yh(_,(Xc(),jg))):p||n?(A.c=-A.b-e.s,Yh(_,(Xc(),Sh))):(A.c=I.a+e.s,Yh(_,(Xc(),v1))),A.d=I.b+e.t,e1(_,(wc(),nf));break;case 2:v.a?(s=e.v?A.a:h(ut(_.d,0),181).rf().b,A.d=(I.b-s)/2,e1(_,(wc(),$g))):p||n?(A.d=-A.a-e.t,e1(_,(wc(),Ah))):(A.d=I.b+e.t,e1(_,(wc(),nf))),A.c=I.a+e.s,Yh(_,(Xc(),v1));break;case 4:v.a?(s=e.v?A.a:h(ut(_.d,0),181).rf().b,A.d=(I.b-s)/2,e1(_,(wc(),$g))):p||n?(A.d=-A.a-e.t,e1(_,(wc(),Ah))):(A.d=I.b+e.t,e1(_,(wc(),nf))),A.c=-A.b-e.s,Yh(_,(Xc(),Sh))}p=!1}}function Lg(e,t){Mr();var n,s,a,u,p,v,m,x,_,A,I,P,j;if(q7(T8)==0){for(A=pe(zPn,Re,117,k_t.length,0,1),p=0;p<A.length;p++)A[p]=new zu(4);for(s=new G7,u=0;u<jEe.length;u++){if(_=new zu(4),u<84?(v=u*2,P=(er(v,Vee.length),Vee.charCodeAt(v)),I=(er(v+1,Vee.length),Vee.charCodeAt(v+1)),$o(_,P,I)):(v=(u-84)*2,$o(_,$Ee[v],$Ee[v+1])),m=jEe[u],Nt(m,"Specials")&&$o(_,65520,65533),Nt(m,Wyt)&&($o(_,983040,1048573),$o(_,1048576,1114109)),Ta(T8,m,_),Ta(vT,m,Bw(_)),x=s.a.length,0<x?s.a=s.a.substr(0,0):0>x&&(s.a+=ket(pe(Cl,o1,25,-x,15,1))),s.a+="Is",Zf(m,Ec(32))>=0)for(a=0;a<m.length;a++)er(a,m.length),m.charCodeAt(a)!=32&&hk(s,(er(a,m.length),m.charCodeAt(a)));else s.a+=""+m;$pe(s.a,m,!0)}$pe(Gee,"Cn",!1),$pe(l3e,"Cn",!0),n=new zu(4),$o(n,0,jx),Ta(T8,"ALL",n),Ta(vT,"ALL",Bw(n)),!xm&&(xm=new zn),Ta(xm,Gee,Gee),!xm&&(xm=new zn),Ta(xm,l3e,l3e),!xm&&(xm=new zn),Ta(xm,"ALL","ALL")}return j=h(Fo(t?T8:vT,e),136),j}function TNn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe;if(I=!1,A=!1,oy(h(U(s,(tt(),Ei)),98))){p=!1,v=!1;e:for(j=new Q(s.j);j.a<j.c.c.length;)for(P=h(K(j),11),J=gg(Cf(se(ie(Nf,1),st,20,0,[new j6(P),new y4(P)])));rr(J);)if(V=h(Un(J),11),!ft(lt(U(V.i,A_)))){if(P.j==(Qe(),sn)){p=!0;break e}if(P.j==Fn){v=!0;break e}}I=v&&!p,A=p&&!v}if(!I&&!A&&s.b.c.length!=0){for(_=0,x=new Q(s.b);x.a<x.c.c.length;)m=h(K(x),70),_+=m.n.b+m.o.b/2;_/=s.b.c.length,fe=_>=s.o.b/2}else fe=!A;fe?(oe=h(U(s,(He(),j5)),15),oe?I?u=oe:(a=h(U(s,N5),15),a?oe.gc()<=a.gc()?u=oe:u=a:(u=new qe,Fe(s,N5,u))):(u=new qe,Fe(s,j5,u))):(a=h(U(s,(He(),N5)),15),a?A?u=a:(oe=h(U(s,j5),15),oe?a.gc()<=oe.gc()?u=a:u=oe:(u=new qe,Fe(s,j5,u))):(u=new qe,Fe(s,N5,u))),u.Fc(e),Fe(e,(He(),N$),n),t.d==n?(xs(t,null),n.e.c.length+n.g.c.length==0&&Ka(n,null),b6n(n)):(Gs(t,null),n.e.c.length+n.g.c.length==0&&Ka(n,null)),Pl(t.a)}function CNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr;for(fe=new Ms(e.b,0),_=t.Kc(),j=0,x=h(_.Pb(),19).a,Ne=0,n=new ns,rt=new md;fe.b<fe.d.gc();){for(oe=(kn(fe.b<fe.d.gc()),h(fe.d.Xb(fe.c=fe.b++),29)),we=new Q(oe.a);we.a<we.c.c.length;){for(le=h(K(we),10),P=new Sn(Dn(qi(le).a.Kc(),new L));rr(P);)A=h(Un(P),17),rt.a.zc(A,rt);for(I=new Sn(Dn(za(le).a.Kc(),new L));rr(I);)A=h(Un(I),17),rt.a.Bc(A)!=null}if(j+1==x){for(a=new Nl(e),iw(fe,a),u=new Nl(e),iw(fe,u),xt=rt.a.ec().Kc();xt.Ob();)wt=h(xt.Pb(),17),n.a._b(wt)||(++Ne,n.a.zc(wt,n)),p=new Nd(e),Fe(p,(tt(),Ei),(_s(),k8)),Ca(p,a),kp(p,(bn(),Bf)),V=new So,Ka(V,p),Ji(V,(Qe(),un)),ln=new So,Ka(ln,p),Ji(ln,pn),s=new Nd(e),Fe(s,Ei,k8),Ca(s,u),kp(s,Bf),J=new So,Ka(J,s),Ji(J,un),dn=new So,Ka(dn,s),Ji(dn,pn),Ve=new db,Gs(Ve,wt.c),xs(Ve,V),Dr=new db,Gs(Dr,ln),xs(Dr,J),Gs(wt,dn),v=new v0e(p,s,Ve,Dr,wt),Fe(p,(He(),tv),v),Fe(s,tv,v),Ln=Ve.c.i,Ln.k==Bf&&(m=h(U(Ln,tv),305),m.d=v,v.g=m);if(_.Ob())x=h(_.Pb(),19).a;else break}++j}return Ke(Ne)}function SNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V;for(A=0,a=new Tn((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));a.e!=a.i.gc();)s=h(On(a),33),ft(lt(pt(s,(tt(),p2))))||((Me(pt(t,Vg))!==Me((Dd(),Ug))||Me(pt(t,im))===Me((Kp(),k_))||Me(pt(t,im))===Me((Kp(),y_))||ft(lt(pt(t,rm)))||Me(pt(t,$5))!==Me((Sb(),i3)))&&!ft(lt(pt(s,yne)))&&(ya(s,(He(),Eo),Ke(A)),++A),Q2t(e,s,n));for(A=0,x=new Tn((!t.b&&(t.b=new Ye(us,t,12,3)),t.b));x.e!=x.i.gc();)v=h(On(x),79),(Me(pt(t,(tt(),Vg)))!==Me((Dd(),Ug))||Me(pt(t,im))===Me((Kp(),k_))||Me(pt(t,im))===Me((Kp(),y_))||ft(lt(pt(t,rm)))||Me(pt(t,$5))!==Me((Sb(),i3)))&&(ya(v,(He(),Eo),Ke(A)),++A),j=H1(v),V=Sg(v),_=ft(lt(pt(j,am))),P=!ft(lt(pt(v,p2))),I=_&&Pb(v)&&ft(lt(pt(v,d2))),u=bi(j)==t&&bi(j)==bi(V),p=(bi(j)==t&&V==t)^(bi(V)==t&&j==t),P&&!I&&(p||u)&&qve(e,v,t,n);if(bi(t))for(m=new Tn(bit(bi(t)));m.e!=m.i.gc();)v=h(On(m),79),j=H1(v),j==t&&Pb(v)&&(I=ft(lt(pt(j,(tt(),am))))&&ft(lt(pt(v,d2))),I&&qve(e,v,t,n))}function ANn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts;for(Bn(n,"MinWidth layering",1),P=t.b,rt=t.a,ts=h(U(t,(tt(),wke)),19).a,v=h(U(t,mke),19).a,e.b=Pe(Je(U(t,Mh))),e.d=ki,we=new Q(rt);we.a<we.c.c.length;)fe=h(K(we),10),fe.k==(bn(),Wi)&&(ln=fe.o.b,e.d=d.Math.min(e.d,ln));for(e.d=d.Math.max(1,e.d),wt=rt.c.length,e.c=pe(Hn,hr,25,wt,15,1),e.f=pe(Hn,hr,25,wt,15,1),e.e=pe(ks,ka,25,wt,15,1),x=0,e.a=0,Ne=new Q(rt);Ne.a<Ne.c.c.length;)fe=h(K(Ne),10),fe.p=x++,e.c[fe.p]=dht(za(fe)),e.f[fe.p]=dht(qi(fe)),e.e[fe.p]=fe.o.b/e.d,e.a+=e.e[fe.p];for(e.b/=e.d,e.a/=wt,Ve=UTn(rt),hs(rt,zde(new qYe(e))),V=ki,j=Ir,p=null,Dr=ts,Ln=ts,u=v,a=v,ts<0&&(Dr=h(pxe.a.zd(),19).a,Ln=h(pxe.b.zd(),19).a),v<0&&(u=h(gxe.a.zd(),19).a,a=h(gxe.b.zd(),19).a),dn=Dr;dn<=Ln;dn++)for(s=u;s<=a;s++)xt=yLn(e,dn,s,rt,Ve),oe=Pe(Je(xt.a)),I=h(xt.b,15),J=I.gc(),(oe<V||oe==V&&J<j)&&(V=oe,j=J,p=I);for(A=p.Kc();A.Ob();){for(_=h(A.Pb(),15),m=new Nl(t),le=_.Kc();le.Ob();)fe=h(le.Pb(),10),Ca(fe,m);P.c[P.c.length]=m}pQ(P),rt.c=pe(yn,st,1,0,5,1),An(n)}function MNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln;for(e.b=t,e.a=h(U(t,(tt(),fke)),19).a,e.c=h(U(t,gke),19).a,e.c==0&&(e.c=Ir),J=new Ms(t.b,0);J.b<J.d.gc();){for(V=(kn(J.b<J.d.gc()),h(J.d.Xb(J.c=J.b++),29)),v=new qe,_=-1,we=-1,le=new Q(V.a);le.a<le.c.c.length;)fe=h(K(le),10),Td((XS(),new Sn(Dn(Id(fe).a.Kc(),new L))))>=e.a&&(s=hLn(e,fe),_=d.Math.max(_,s.b),we=d.Math.max(we,s.d),Ue(v,new As(fe,s)));for(wt=new qe,x=0;x<_;++x)aw(wt,0,(kn(J.b>0),J.a.Xb(J.c=--J.b),xt=new Nl(e.b),iw(J,xt),kn(J.b<J.d.gc()),J.d.Xb(J.c=J.b++),xt));for(p=new Q(v);p.a<p.c.c.length;)if(a=h(K(p),46),P=h(a.b,571).a,!!P)for(I=new Q(P);I.a<I.c.c.length;)A=h(K(I),10),g2e(e,A,t$,wt);for(n=new qe,m=0;m<we;++m)Ue(n,(ln=new Nl(e.b),iw(J,ln),ln));for(u=new Q(v);u.a<u.c.c.length;)if(a=h(K(u),46),rt=h(a.b,571).c,!!rt)for(Ve=new Q(rt);Ve.a<Ve.c.c.length;)Ne=h(K(Ve),10),g2e(e,Ne,n$,n)}for(oe=new Ms(t.b,0);oe.b<oe.d.gc();)j=(kn(oe.b<oe.d.gc()),h(oe.d.Xb(oe.c=oe.b++),29)),j.a.c.length==0&&Mu(oe)}function DNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln;if(Bn(n,"Spline edge routing",1),t.b.c.length==0){t.f.a=0,An(n);return}fe=Pe(Je(U(t,(tt(),b3)))),v=Pe(Je(U(t,b2))),p=Pe(Je(U(t,p3))),oe=h(U(t,Cne),336),wt=oe==(qk(),F_),rt=Pe(Je(U(t,lke))),e.d=t,e.j.c=pe(yn,st,1,0,5,1),e.a.c=pe(yn,st,1,0,5,1),Kc(e.k),m=h(ut(t.b,0),29),_=WN(m.a,(iR(),$D)),j=h(ut(t.b,t.b.c.length-1),29),A=WN(j.a,$D),V=new Q(t.b),J=null,Ln=0;do{for(le=V.a<V.c.c.length?h(K(V),29):null,bNn(e,J,le),VDn(e),xt=Zfn(t5n(YP(Jr(new Vt(null,new Wt(e.i,16)),new nHe),new tHe))),dn=0,we=Ln,I=!J||_&&J==m,P=!le||A&&le==j,xt>0?(x=0,J&&(x+=v),x+=(xt-1)*p,le&&(x+=v),wt&&!!le&&(x=d.Math.max(x,lAn(le,p,fe,rt))),x<fe&&!I&&!P&&(dn=(fe-x)/2,x=fe),we+=x):!I&&!P&&(we+=fe),le&&Lve(le,we),Ve=new Q(e.i);Ve.a<Ve.c.c.length;)Ne=h(K(Ve),128),Ne.a.c=Ln,Ne.a.b=we-Ln,Ne.F=dn,Ne.p=!J;Vi(e.a,e.i),Ln=we,le&&(Ln+=le.c.a),J=le,I=P}while(le);for(a=new Q(e.j);a.a<a.c.c.length;)s=h(K(a),17),u=C5n(e,s),Fe(s,(He(),g3),u),ln=IAn(e,s),Fe(s,f2,ln);t.f.a=Ln,e.d=null,An(n)}function mbt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;if(V=e.i!=0,le=!1,oe=null,Tu(e.e)){if(_=t.gc(),_>0){for(I=_<100?null:new ig(_),x=new Cge(t),j=x.g,oe=pe(Hn,hr,25,_,15,1),s=0,we=new yb(_),a=0;a<e.i;++a){v=e.g[a],P=v;e:for(fe=0;fe<2;++fe){for(m=_;--m>=0;)if(P!=null?Nr(P,j[m]):Me(P)===Me(j[m])){oe.length<=s&&(J=oe,oe=pe(Hn,hr,25,2*oe.length,15,1),Ao(J,0,oe,0,s)),oe[s++]=a,qn(we,j[m]);break e}if(P=P,Me(P)===Me(v))break}}if(x=we,j=we.g,_=s,s>oe.length&&(J=oe,oe=pe(Hn,hr,25,s,15,1),Ao(J,0,oe,0,s)),s>0){for(le=!0,u=0;u<s;++u)P=j[u],I=Qtt(e,h(P,72),I);for(p=s;--p>=0;)c5(e,oe[p]);if(s!=_){for(a=_;--a>=s;)c5(x,a);J=oe,oe=pe(Hn,hr,25,s,15,1),Ao(J,0,oe,0,s)}t=x}}}else for(t=c7n(e,t),a=e.i;--a>=0;)t.Hc(e.g[a])&&(c5(e,a),le=!0);if(le){if(oe!=null){for(n=t.gc(),A=n==1?RE(e,4,t.Kc().Pb(),null,oe[0],V):RE(e,6,t,oe,oe[0],V),I=n<100?null:new ig(n),a=t.Kc();a.Ob();)P=a.Pb(),I=D1e(e,h(P,72),I);I?(I.Ei(A),I.Fi()):Or(e.e,A)}else{for(I=kgn(t.gc()),a=t.Kc();a.Ob();)P=a.Pb(),I=D1e(e,h(P,72),I);I&&I.Fi()}return!0}else return!1}function LNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le;for(n=new Uft(t),n.a||gDn(t),x=pMn(t),m=new pb,J=new jgt,V=new Q(t.a);V.a<V.c.c.length;)for(j=h(K(V),10),a=new Sn(Dn(qi(j).a.Kc(),new L));rr(a);)s=h(Un(a),17),(s.c.i.k==(bn(),Pi)||s.d.i.k==Pi)&&(_=mOn(e,s,x,J),Ot(m,IX(_.d),_.a));for(p=new qe,le=h(U(n.c,(He(),em)),21).Kc();le.Ob();){switch(fe=h(le.Pb(),61),P=J.c[fe.g],I=J.b[fe.g],v=J.a[fe.g],u=null,oe=null,fe.g){case 4:u=new hl(e.d.a,P,x.b.a-e.d.a,I-P),oe=new hl(e.d.a,P,v,I-P),ab(x,new dt(u.c+u.b,u.d)),ab(x,new dt(u.c+u.b,u.d+u.a));break;case 2:u=new hl(x.a.a,P,e.c.a-x.a.a,I-P),oe=new hl(e.c.a-v,P,v,I-P),ab(x,new dt(u.c,u.d)),ab(x,new dt(u.c,u.d+u.a));break;case 1:u=new hl(P,e.d.b,I-P,x.b.b-e.d.b),oe=new hl(P,e.d.b,I-P,v),ab(x,new dt(u.c,u.d+u.a)),ab(x,new dt(u.c+u.b,u.d+u.a));break;case 3:u=new hl(P,x.a.b,I-P,e.c.b-x.a.b),oe=new hl(P,e.c.b-v,I-P,v),ab(x,new dt(u.c,u.d)),ab(x,new dt(u.c+u.b,u.d))}u&&(A=new VKe,A.d=fe,A.b=u,A.c=oe,A.a=ZP(h(jr(m,IX(fe)),21)),p.c[p.c.length]=A)}return Vi(n.b,p),n.d=P6n(RIn(x)),n}function ybt(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V;if(n.p[t.p]==null){v=!0,n.p[t.p]=0,p=t,V=n.o==(yf(),v2)?Fi:ki;do a=e.b.e[p.p],u=p.c.a.c.length,n.o==v2&&a>0||n.o==Rf&&a<u-1?(m=null,x=null,n.o==Rf?m=h(ut(p.c.a,a+1),10):m=h(ut(p.c.a,a-1),10),x=n.g[m.p],ybt(e,x,n),V=e.e.bg(V,t,p),n.j[t.p]==t&&(n.j[t.p]=n.j[x.p]),n.j[t.p]==n.j[x.p]?(j=uy(e.d,p,m),n.o==Rf?(s=Pe(n.p[t.p]),A=Pe(n.p[x.p])+Pe(n.d[m.p])-m.d.d-j-p.d.a-p.o.b-Pe(n.d[p.p]),v?(v=!1,n.p[t.p]=d.Math.min(A,V)):n.p[t.p]=d.Math.min(s,d.Math.min(A,V))):(s=Pe(n.p[t.p]),A=Pe(n.p[x.p])+Pe(n.d[m.p])+m.o.b+m.d.a+j+p.d.d-Pe(n.d[p.p]),v?(v=!1,n.p[t.p]=d.Math.max(A,V)):n.p[t.p]=d.Math.max(s,d.Math.max(A,V)))):(j=Pe(Je(U(e.a,(tt(),uv)))),P=Uut(e,n.j[t.p]),_=Uut(e,n.j[x.p]),n.o==Rf?(I=Pe(n.p[t.p])+Pe(n.d[p.p])+p.o.b+p.d.a+j-(Pe(n.p[x.p])+Pe(n.d[m.p])-m.d.d),Mst(P,_,I)):(I=Pe(n.p[t.p])+Pe(n.d[p.p])-p.d.d-Pe(n.p[x.p])-Pe(n.d[m.p])-m.o.b-m.d.a-j,Mst(P,_,I)))):V=e.e.bg(V,t,p),p=n.a[p.p];while(p!=t);Mfn(e.e,t)}}function INn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln;for(le=t,fe=new pb,we=new pb,_=wb(le,Bye),s=new mit(e,n,fe,we),b_n(s.a,s.b,s.c,s.d,_),m=(rt=fe.i,rt||(fe.i=new hy(fe,fe.c))),xt=m.Kc();xt.Ob();)for(wt=h(xt.Pb(),202),a=h(jr(fe,wt),21),V=a.Kc();V.Ob();)if(j=V.Pb(),Ne=h(wy(e.d,j),202),Ne)v=(!wt.e&&(wt.e=new qt(gi,wt,10,9)),wt.e),qn(v,Ne);else throw p=_d(le,p1),I=mmt+j+ymt+p,P=I+Fx,ne(new Xf(P));for(x=(Ve=we.i,Ve||(we.i=new hy(we,we.c))),dn=x.Kc();dn.Ob();)for(ln=h(dn.Pb(),202),u=h(jr(we,ln),21),oe=u.Kc();oe.Ob();)if(J=oe.Pb(),Ne=h(wy(e.d,J),202),Ne)A=(!ln.g&&(ln.g=new qt(gi,ln,9,10)),ln.g),qn(A,Ne);else throw p=_d(le,p1),I=mmt+J+ymt+p,P=I+Fx,ne(new Xf(P));!n.b&&(n.b=new qt(Pn,n,4,7)),n.b.i!=0&&(!n.c&&(n.c=new qt(Pn,n,5,8)),n.c.i!=0)&&(!n.b&&(n.b=new qt(Pn,n,4,7)),n.b.i<=1&&(!n.c&&(n.c=new qt(Pn,n,5,8)),n.c.i<=1))&&(!n.a&&(n.a=new Ye(gi,n,6,6)),n.a).i==1&&(Ln=h(he((!n.a&&(n.a=new Ye(gi,n,6,6)),n.a),0),202),!FX(Ln)&&!RX(Ln)&&(UB(Ln,h(he((!n.b&&(n.b=new qt(Pn,n,4,7)),n.b),0),82)),qB(Ln,h(he((!n.c&&(n.c=new qt(Pn,n,5,8)),n.c),0),82))))}function ONn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln;for(le=e.a,we=0,Ne=le.length;we<Ne;++we){for(fe=le[we],x=Ir,_=Ir,j=new Q(fe.e);j.a<j.c.c.length;)I=h(K(j),10),p=I.c?Ha(I.c.a,I,0):-1,p>0?(A=h(ut(I.c.a,p-1),10),wt=uy(e.b,I,A),J=I.n.b-I.d.d-(A.n.b+A.o.b+A.d.a+wt)):J=I.n.b-I.d.d,x=d.Math.min(J,x),p<I.c.a.c.length-1?(A=h(ut(I.c.a,p+1),10),wt=uy(e.b,I,A),oe=A.n.b-A.d.d-(I.n.b+I.o.b+I.d.a+wt)):oe=2*I.n.b,_=d.Math.min(oe,_);for(m=Ir,u=!1,a=h(ut(fe.e,0),10),ln=new Q(a.j);ln.a<ln.c.c.length;)for(xt=h(K(ln),11),V=a.n.b+xt.n.b+xt.a.b,s=new Q(xt.e);s.a<s.c.c.length;)n=h(K(s),17),Ve=n.c,t=Ve.i.n.b+Ve.n.b+Ve.a.b-V,d.Math.abs(t)<d.Math.abs(m)&&d.Math.abs(t)<(t<0?x:_)&&(m=t,u=!0);for(v=h(ut(fe.e,fe.e.c.length-1),10),rt=new Q(v.j);rt.a<rt.c.c.length;)for(Ve=h(K(rt),11),V=v.n.b+Ve.n.b+Ve.a.b,s=new Q(Ve.g);s.a<s.c.c.length;)n=h(K(s),17),xt=n.d,t=xt.i.n.b+xt.n.b+xt.a.b-V,d.Math.abs(t)<d.Math.abs(m)&&d.Math.abs(t)<(t<0?x:_)&&(m=t,u=!0);if(u&&m!=0)for(P=new Q(fe.e);P.a<P.c.c.length;)I=h(K(P),10),I.n.b+=m}}function kbt(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;if(Au(e.a,t)){if(wd(h(_n(e.a,t),53),n))return 1}else Pr(e.a,t,new ns);if(Au(e.a,n)){if(wd(h(_n(e.a,n),53),t))return-1}else Pr(e.a,n,new ns);if(Au(e.e,t)){if(wd(h(_n(e.e,t),53),n))return-1}else Pr(e.e,t,new ns);if(Au(e.e,n)){if(wd(h(_n(e.a,n),53),t))return 1}else Pr(e.e,n,new ns);if(e.c==(Dd(),Vne)||!os(t,(He(),Eo))||!os(n,(He(),Eo))){if(m=h(qfe(Nat(QK(Jr(new Vt(null,new Wt(t.j,16)),new Qje)),new Zje)),11),_=h(qfe(Nat(QK(Jr(new Vt(null,new Wt(n.j,16)),new Jje)),new e$e)),11),!!m&&!!_){if(v=m.i,x=_.i,!!v&&v==x){for(I=new Q(v.j);I.a<I.c.c.length;){if(A=h(K(I),11),A==m)return dx(e,n,t),-1;if(A==_)return dx(e,t,n),1}return dc(nQ(e,t),nQ(e,n))}for(j=e.d,V=0,J=j.length;V<J;++V){if(P=j[V],P==v)return dx(e,n,t),-1;if(P==x)return dx(e,t,n),1}}if(!os(t,(He(),Eo))||!os(n,Eo))return a=nQ(e,t),p=nQ(e,n),a>p?dx(e,t,n):dx(e,n,t),a<p?-1:a>p?1:0}return s=h(U(t,(He(),Eo)),19).a,u=h(U(n,Eo),19).a,s>u?dx(e,t,n):dx(e,n,t),s<u?-1:s>u?1:0}function Gve(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe;if(ft(lt(pt(t,(Tr(),OH)))))return Rt(),Rt(),ca;if(x=(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a).i!=0,A=mTn(t),_=!A.dc(),x||_){if(a=h(pt(t,v8),149),!a)throw ne(new ey("Resolved algorithm is not set; apply a LayoutAlgorithmResolver before computing layout."));if(fe=ffe(a,(ux(),qH)),Cft(t),!x&&_&&!fe)return Rt(),Rt(),ca;if(m=new qe,Me(pt(t,k3))===Me((Ld(),B0))&&(ffe(a,VH)||ffe(a,GH)))for(P=cpt(e,t),j=new di,Qs(j,(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));j.b!=0;)I=h(j.b==0?null:(kn(j.b!=0),pl(j,j.a.a)),33),Cft(I),oe=Me(pt(I,k3))===Me(tT),oe||Pp(I,Q_)&&!C0e(a,pt(I,v8))?(v=Gve(e,I,n,s),Vi(m,v),ya(I,k3,tT),Tpt(I)):Qs(j,(!I.a&&(I.a=new Ye(wi,I,10,11)),I.a));else for(P=(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a).i,p=new Tn((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));p.e!=p.i.gc();)u=h(On(p),33),v=Gve(e,u,n,s),Vi(m,v),Tpt(u);for(J=new Q(m);J.a<J.c.c.length;)V=h(K(J),79),ya(V,OH,(on(),!0));return Wxn(t,a,jo(s,P)),dDn(m),_&&fe?A:(Rt(),Rt(),ca)}else return Rt(),Rt(),ca}function DM(e,t,n,s,a,u,p,v,m){var x,_,A,I,P,j,V;switch(P=n,_=new Nd(m),kp(_,(bn(),Pi)),Fe(_,(He(),v6e),p),Fe(_,(tt(),Ei),(_s(),qo)),V=Pe(Je(e.We(om))),Fe(_,om,V),A=new So,Ka(A,_),t!=$f&&t!=Qg||(s>=0?P=i5(v):P=KA(i5(v)),e.Ye(u8,P)),x=new Rs,I=!1,e.Xe(ov)?(t1e(x,h(e.We(ov),8)),I=!0):a0n(x,p.a/2,p.b/2),P.g){case 4:Fe(_,cc,(wl(),Gg)),Fe(_,B$,(Hp(),c3)),_.o.b=p.b,V<0&&(_.o.a=-V),Ji(A,(Qe(),pn)),I||(x.a=p.a),x.a-=p.a;break;case 2:Fe(_,cc,(wl(),nm)),Fe(_,B$,(Hp(),t8)),_.o.b=p.b,V<0&&(_.o.a=-V),Ji(A,(Qe(),un)),I||(x.a=0);break;case 1:Fe(_,h2,(Ad(),l3)),_.o.a=p.a,V<0&&(_.o.b=-V),Ji(A,(Qe(),Fn)),I||(x.b=p.b),x.b-=p.b;break;case 3:Fe(_,h2,(Ad(),O5)),_.o.a=p.a,V<0&&(_.o.b=-V),Ji(A,(Qe(),sn)),I||(x.b=0)}if(t1e(A.n,x),Fe(_,ov,x),t==x2||t==ed||t==qo){if(j=0,t==x2&&e.Xe(L0))switch(P.g){case 1:case 2:j=h(e.We(L0),19).a;break;case 3:case 4:j=-h(e.We(L0),19).a}else switch(P.g){case 4:case 2:j=u.b,t==ed&&(j/=a.b);break;case 1:case 3:j=u.a,t==ed&&(j/=a.a)}Fe(_,iv,j)}return Fe(_,co,P),_}function NNn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt;if(n=Pe(Je(U(e.a.j,(tt(),ike)))),n<-1||!e.a.i||F4(h(U(e.a.o,Ei),98))||Za(e.a.o,(Qe(),pn)).gc()<2&&Za(e.a.o,un).gc()<2)return!0;if(e.a.c.Rf())return!1;for(Ne=0,we=0,le=new qe,m=e.a.e,x=0,_=m.length;x<_;++x){for(v=m[x],I=v,P=0,V=I.length;P<V;++P){if(A=I[P],A.k==(bn(),Vo)){le.c[le.c.length]=A;continue}for(s=e.b[A.c.p][A.p],A.k==Pi?(s.b=1,h(U(A,(He(),Br)),11).j==(Qe(),pn)&&(we+=s.a)):(xt=Za(A,(Qe(),un)),xt.dc()||!LY(xt,new v$e)?s.c=1:(a=Za(A,pn),(a.dc()||!LY(a,new b$e))&&(Ne+=s.a))),p=new Sn(Dn(qi(A).a.Kc(),new L));rr(p);)u=h(Un(p),17),Ne+=s.c,we+=s.b,wt=u.d.i,G0e(e,s,wt);for(oe=Cf(se(ie(Nf,1),st,20,0,[Za(A,(Qe(),sn)),Za(A,Fn)])),rt=new Sn(new Ffe(oe.a.length,oe.a));rr(rt);)Ve=h(Un(rt),11),fe=h(U(Ve,(He(),Zc)),10),fe&&(Ne+=s.c,we+=s.b,G0e(e,s,fe))}for(j=new Q(le);j.a<j.c.c.length;)for(A=h(K(j),10),s=e.b[A.c.p][A.p],p=new Sn(Dn(qi(A).a.Kc(),new L));rr(p);)u=h(Un(p),17),Ne+=s.c,we+=s.b,wt=u.d.i,G0e(e,s,wt);le.c=pe(yn,st,1,0,5,1)}return t=Ne+we,J=t==0?ki:(Ne-we)/t,J>=n}function PNn(){The();function e(s){var a=this;this.dispatch=function(u){var p=u.data;switch(p.cmd){case"algorithms":var v=Ope((Rt(),new B6(new vd(tp.b))));s.postMessage({id:p.id,data:v});break;case"categories":var m=Ope((Rt(),new B6(new vd(tp.c))));s.postMessage({id:p.id,data:m});break;case"options":var x=Ope((Rt(),new B6(new vd(tp.d))));s.postMessage({id:p.id,data:x});break;case"register":zIn(p.algorithms),s.postMessage({id:p.id});break;case"layout":xLn(p.graph,p.layoutOptions||{},p.options||{}),s.postMessage({id:p.id,data:p.graph});break}},this.saveDispatch=function(u){try{a.dispatch(u)}catch(p){s.postMessage({id:u.data.id,error:p})}}}function t(s){var a=this;this.dispatcher=new e({postMessage:function(u){a.onmessage({data:u})}}),this.postMessage=function(u){setTimeout(function(){a.dispatcher.saveDispatch({data:u})},0)}}if(typeof document===YZ&&typeof self!==YZ){var n=new e(self);self.onmessage=n.saveDispatch}else typeof l!==YZ&&l.exports&&(Object.defineProperty(f,"__esModule",{value:!0}),l.exports={default:t,Worker:t})}function BNn(e){e.N||(e.N=!0,e.b=so(e,0),vi(e.b,0),vi(e.b,1),vi(e.b,2),e.bb=so(e,1),vi(e.bb,0),vi(e.bb,1),e.fb=so(e,2),vi(e.fb,3),vi(e.fb,4),Qr(e.fb,5),e.qb=so(e,3),vi(e.qb,0),Qr(e.qb,1),Qr(e.qb,2),vi(e.qb,3),vi(e.qb,4),Qr(e.qb,5),vi(e.qb,6),e.a=kr(e,4),e.c=kr(e,5),e.d=kr(e,6),e.e=kr(e,7),e.f=kr(e,8),e.g=kr(e,9),e.i=kr(e,10),e.j=kr(e,11),e.k=kr(e,12),e.n=kr(e,13),e.o=kr(e,14),e.p=kr(e,15),e.q=kr(e,16),e.s=kr(e,17),e.r=kr(e,18),e.t=kr(e,19),e.u=kr(e,20),e.v=kr(e,21),e.w=kr(e,22),e.B=kr(e,23),e.A=kr(e,24),e.C=kr(e,25),e.D=kr(e,26),e.F=kr(e,27),e.G=kr(e,28),e.H=kr(e,29),e.J=kr(e,30),e.I=kr(e,31),e.K=kr(e,32),e.M=kr(e,33),e.L=kr(e,34),e.P=kr(e,35),e.Q=kr(e,36),e.R=kr(e,37),e.S=kr(e,38),e.T=kr(e,39),e.U=kr(e,40),e.V=kr(e,41),e.X=kr(e,42),e.W=kr(e,43),e.Y=kr(e,44),e.Z=kr(e,45),e.$=kr(e,46),e._=kr(e,47),e.ab=kr(e,48),e.cb=kr(e,49),e.db=kr(e,50),e.eb=kr(e,51),e.gb=kr(e,52),e.hb=kr(e,53),e.ib=kr(e,54),e.jb=kr(e,55),e.kb=kr(e,56),e.lb=kr(e,57),e.mb=kr(e,58),e.nb=kr(e,59),e.ob=kr(e,60),e.pb=kr(e,61))}function FNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;if(fe=0,t.f.a==0)for(J=new Q(e);J.a<J.c.c.length;)j=h(K(J),10),fe=d.Math.max(fe,j.n.a+j.o.a+j.d.c);else fe=t.f.a-t.c.a;for(fe-=t.c.a,V=new Q(e);V.a<V.c.c.length;){switch(j=h(K(V),10),N7(j.n,fe-j.o.a),W1e(j.f),f1t(j),(j.q?j.q:(Rt(),Rt(),K1))._b((tt(),um))&&N7(h(U(j,um),8),fe-j.o.a),h(U(j,m1),248).g){case 1:Fe(j,m1,($1(),YD));break;case 2:Fe(j,m1,($1(),qD))}for(oe=j.o,we=new Q(j.j);we.a<we.c.c.length;){for(le=h(K(we),11),N7(le.n,oe.a-le.o.a),N7(le.a,le.o.a),Ji(le,alt(le.j)),p=h(U(le,L0),19),p&&Fe(le,L0,Ke(-p.a)),u=new Q(le.g);u.a<u.c.c.length;){for(a=h(K(u),17),s=pr(a.a,0);s.b!=s.d.c;)n=h(gr(s),8),n.a=fe-n.a;if(x=h(U(a,Da),74),x)for(m=pr(x,0);m.b!=m.d.c;)v=h(gr(m),8),v.a=fe-v.a;for(I=new Q(a.b);I.a<I.c.c.length;)_=h(K(I),70),N7(_.n,fe-_.o.a)}for(P=new Q(le.f);P.a<P.c.c.length;)_=h(K(P),70),N7(_.n,le.o.a-_.o.a)}for(j.k==(bn(),Pi)&&(Fe(j,(He(),co),alt(h(U(j,co),61))),G9n(j)),A=new Q(j.b);A.a<A.c.c.length;)_=h(K(A),70),f1t(_),N7(_.n,oe.a-_.o.a)}}function RNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;if(fe=0,t.f.b==0)for(J=new Q(e);J.a<J.c.c.length;)j=h(K(J),10),fe=d.Math.max(fe,j.n.b+j.o.b+j.d.a);else fe=t.f.b-t.c.b;for(fe-=t.c.b,V=new Q(e);V.a<V.c.c.length;){switch(j=h(K(V),10),O7(j.n,fe-j.o.b),K1e(j.f),d1t(j),(j.q?j.q:(Rt(),Rt(),K1))._b((tt(),um))&&O7(h(U(j,um),8),fe-j.o.b),h(U(j,m1),248).g){case 3:Fe(j,m1,($1(),SH));break;case 4:Fe(j,m1,($1(),MH))}for(oe=j.o,we=new Q(j.j);we.a<we.c.c.length;){for(le=h(K(we),11),O7(le.n,oe.b-le.o.b),O7(le.a,le.o.b),Ji(le,olt(le.j)),p=h(U(le,L0),19),p&&Fe(le,L0,Ke(-p.a)),u=new Q(le.g);u.a<u.c.c.length;){for(a=h(K(u),17),s=pr(a.a,0);s.b!=s.d.c;)n=h(gr(s),8),n.b=fe-n.b;if(x=h(U(a,Da),74),x)for(m=pr(x,0);m.b!=m.d.c;)v=h(gr(m),8),v.b=fe-v.b;for(I=new Q(a.b);I.a<I.c.c.length;)_=h(K(I),70),O7(_.n,fe-_.o.b)}for(P=new Q(le.f);P.a<P.c.c.length;)_=h(K(P),70),O7(_.n,le.o.b-_.o.b)}for(j.k==(bn(),Pi)&&(Fe(j,(He(),co),olt(h(U(j,co),61))),hkn(j)),A=new Q(j.b);A.a<A.c.c.length;)_=h(K(A),70),d1t(_),O7(_.n,oe.b-_.o.b)}}function jNn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P;for(A=!1,x=e+1,_=(Kt(e,t.c.length),h(t.c[e],200)),p=_.a,v=null,u=0;u<_.a.c.length;u++)if(a=(Kt(u,p.c.length),h(p.c[u],187)),!a.c){if(a.b.c.length==0){L1(),aM(_,a),--u,A=!0;continue}if(a.k||(v&&RF(v),v=new e0e(v?v.e+v.d+s:0,_.f,s),tM(a,v.e+v.d,_.f),Ue(_.d,v),Sge(v,a),a.k=!0),m=null,m=(P=null,u<_.a.c.length-1?P=h(ut(_.a,u+1),187):x<t.c.length&&(Kt(x,t.c.length),h(t.c[x],200)).a.c.length!=0&&(P=h(ut((Kt(x,t.c.length),h(t.c[x],200)).a,0),187)),P),I=!1,m&&(I=!Nr(m.j,_)),m){if(m.b.c.length==0){aM(_,m);break}else mA(a,n-a.s),RF(a.q),A=A|V9n(_,a,m,n,s);if(m.b.c.length==0)for(aM((Kt(x,t.c.length),h(t.c[x],200)),m),m=null;t.c.length>x&&(Kt(x,t.c.length),h(t.c[x],200)).a.c.length==0;)vc(t,(Kt(x,t.c.length),t.c[x]));if(!m){--u;continue}if(EDn(t,_,a,m,I,n,x,s)){A=!0;continue}if(I){if(DLn(t,_,a,m,n,x,s)){A=!0;continue}else if(ape(_,a)){a.c=!0,A=!0;continue}}else if(ape(_,a)){a.c=!0,A=!0;continue}if(A)continue}if(ape(_,a)){a.c=!0,A=!0,m&&(m.k=!1);continue}else RF(a.q)}return A}function cZ(e,t,n,s,a,u,p){var v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts;for(V=0,ln=0,x=new Q(e.b);x.a<x.c.c.length;)m=h(K(x),157),m.c&&X2t(m.c),V=d.Math.max(V,Fc(m)),ln+=Fc(m)*Su(m);for(J=ln/e.b.c.length,xt=tEn(e.b,J),ln+=e.b.c.length*xt,V=d.Math.max(V,d.Math.sqrt(ln*p))+n.b,Dr=n.b,ts=n.d,P=0,A=n.b+n.c,wt=new di,vr(wt,Ke(0)),Ve=new di,_=new Ms(e.b,0),j=null,v=new qe;_.b<_.d.gc();)m=(kn(_.b<_.d.gc()),h(_.d.Xb(_.c=_.b++),157)),Ln=Fc(m),I=Su(m),Dr+Ln>V&&(u&&(Cp(Ve,P),Cp(wt,Ke(_.b-1)),Ue(e.d,j),v.c=pe(yn,st,1,0,5,1)),Dr=n.b,ts+=P+t,P=0,A=d.Math.max(A,n.b+n.c+Ln)),v.c[v.c.length]=m,Rft(m,Dr,ts),A=d.Math.max(A,Dr+Ln+n.c),P=d.Math.max(P,I),Dr+=Ln+t,j=m;if(Vi(e.a,v),Ue(e.d,h(ut(v,v.c.length-1),157)),A=d.Math.max(A,s),dn=ts+P+n.a,dn<a&&(P+=a-dn,dn=a),u)for(Dr=n.b,_=new Ms(e.b,0),Cp(wt,Ke(e.b.c.length)),rt=pr(wt,0),fe=h(gr(rt),19).a,Cp(Ve,P),Ne=pr(Ve,0),we=0;_.b<_.d.gc();)_.b==fe&&(Dr=n.b,we=Pe(Je(gr(Ne))),fe=h(gr(rt),19).a),m=(kn(_.b<_.d.gc()),h(_.d.Xb(_.c=_.b++),157)),wft(m,we),_.b==fe&&(oe=A-Dr-n.c,le=Fc(m),vft(m,oe),Hht(m,(oe-le)/2,0)),Dr+=Fc(m)+t;return new dt(A,dn)}function $Nn(e){var t,n,s,a,u;switch(t=e.c,u=null,t){case 6:return e.Vl();case 13:return e.Wl();case 23:return e.Nl();case 22:return e.Sl();case 18:return e.Pl();case 8:Sr(e),u=(Mr(),HEe);break;case 9:return e.vl(!0);case 19:return e.wl();case 10:switch(e.a){case 100:case 68:case 119:case 87:case 115:case 83:return u=e.ul(e.a),Sr(e),u;case 101:case 102:case 110:case 114:case 116:case 117:case 118:case 120:n=e.tl(),n<ea?u=(Mr(),Mr(),new N1(0,n)):u=grt(xpe(n));break;case 99:return e.Fl();case 67:return e.Al();case 105:return e.Il();case 73:return e.Bl();case 103:return e.Gl();case 88:return e.Cl();case 49:case 50:case 51:case 52:case 53:case 54:case 55:case 56:case 57:return e.xl();case 80:case 112:if(u=kbe(e,e.a),!u)throw ne(new Zn(ir((Qn(),Mee))));break;default:u=Ont(e.a)}Sr(e);break;case 0:if(e.a==93||e.a==123||e.a==125)throw ne(new Zn(ir((Qn(),Uye))));u=Ont(e.a),s=e.a,Sr(e),(s&64512)==jM&&e.c==0&&(e.a&64512)==56320&&(a=pe(Cl,o1,25,2,15,1),a[0]=s&Oi,a[1]=e.a&Oi,u=bW(grt(Fl(a,0,a.length)),0),Sr(e));break;default:throw ne(new Zn(ir((Qn(),Uye))))}return u}function HNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;if(s=new qe,a=Ir,u=Ir,p=Ir,n)for(a=e.f.a,V=new Q(t.j);V.a<V.c.c.length;)for(j=h(K(V),11),m=new Q(j.g);m.a<m.c.c.length;)v=h(K(m),17),v.a.b!=0&&(_=h(RY(v.a),8),_.a<a&&(u=a-_.a,p=Ir,s.c=pe(yn,st,1,0,5,1),a=_.a),_.a<=a&&(s.c[s.c.length]=v,v.a.b>1&&(p=d.Math.min(p,d.Math.abs(h(Zh(v.a,1),8).b-_.b)))));else for(V=new Q(t.j);V.a<V.c.c.length;)for(j=h(K(V),11),m=new Q(j.e);m.a<m.c.c.length;)v=h(K(m),17),v.a.b!=0&&(I=h(lP(v.a),8),I.a>a&&(u=I.a-a,p=Ir,s.c=pe(yn,st,1,0,5,1),a=I.a),I.a>=a&&(s.c[s.c.length]=v,v.a.b>1&&(p=d.Math.min(p,d.Math.abs(h(Zh(v.a,v.a.b-2),8).b-I.b)))));if(s.c.length!=0&&u>t.o.a/2&&p>t.o.b/2){for(P=new So,Ka(P,t),Ji(P,(Qe(),sn)),P.n.a=t.o.a/2,oe=new So,Ka(oe,t),Ji(oe,Fn),oe.n.a=t.o.a/2,oe.n.b=t.o.b,m=new Q(s);m.a<m.c.c.length;)v=h(K(m),17),n?(x=h(eW(v.a),8),J=v.a.b==0?kf(v.d):h(RY(v.a),8),J.b>=x.b?Gs(v,oe):Gs(v,P)):(x=h(O2n(v.a),8),J=v.a.b==0?kf(v.c):h(lP(v.a),8),J.b>=x.b?xs(v,oe):xs(v,P)),A=h(U(v,(tt(),Da)),74),A&&Ew(A,x,!0);t.n.a=a-t.o.a/2}}function zNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts,il,rh;if(ln=null,Ln=t,dn=Tat(e,kat(n),Ln),WE(dn,_d(Ln,p1)),Dr=h(wy(e.g,l5(Ed(Ln,vee))),33),I=Ed(Ln,"sourcePort"),s=null,I&&(s=l5(I)),ts=h(wy(e.j,s),118),!Dr)throw v=Jk(Ln),j="An edge must have a source node (edge id: '"+v,V=j+Fx,ne(new Xf(V));if(!!ts&&!n1(mf(ts),Dr))throw m=_d(Ln,p1),J="The source port of an edge must be a port of the edge's source node (edge id: '"+m,oe=J+Fx,ne(new Xf(oe));if(wt=(!dn.b&&(dn.b=new qt(Pn,dn,4,7)),dn.b),u=null,ts?u=ts:u=Dr,qn(wt,u),il=h(wy(e.g,l5(Ed(Ln,Gye))),33),P=Ed(Ln,"targetPort"),a=null,P&&(a=l5(P)),rh=h(wy(e.j,a),118),!il)throw A=Jk(Ln),fe="An edge must have a target node (edge id: '"+A,le=fe+Fx,ne(new Xf(le));if(!!rh&&!n1(mf(rh),il))throw x=_d(Ln,p1),we="The target port of an edge must be a port of the edge's target node (edge id: '"+x,Ne=we+Fx,ne(new Xf(Ne));if(xt=(!dn.c&&(dn.c=new qt(Pn,dn,5,8)),dn.c),p=null,rh?p=rh:p=il,qn(xt,p),(!dn.b&&(dn.b=new qt(Pn,dn,4,7)),dn.b).i==0||(!dn.c&&(dn.c=new qt(Pn,dn,5,8)),dn.c).i==0)throw _=_d(Ln,p1),Ve=wmt+_,rt=Ve+Fx,ne(new Xf(rt));return qF(Ln,dn),ICn(Ln,dn),ln=XK(e,Ln,dn),ln}function xbt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln;return A=UDn(gc(e,(Qe(),Hf)),t),j=My(gc(e,Ph),t),we=My(gc(e,tl),t),wt=jF(gc(e,Ql),t),I=jF(gc(e,_l),t),fe=My(gc(e,Bh),t),V=My(gc(e,eu),t),Ve=My(gc(e,nl),t),Ne=My(gc(e,Tl),t),xt=jF(gc(e,uc),t),oe=My(gc(e,Cc),t),le=My(gc(e,tu),t),rt=My(gc(e,Tc),t),ln=jF(gc(e,Sc),t),P=jF(gc(e,Gc),t),J=My(gc(e,Yo),t),n=ky(se(ie(ks,1),ka,25,15,[fe.a,wt.a,Ve.a,ln.a])),s=ky(se(ie(ks,1),ka,25,15,[j.a,A.a,we.a,J.a])),a=oe.a,u=ky(se(ie(ks,1),ka,25,15,[V.a,I.a,Ne.a,P.a])),x=ky(se(ie(ks,1),ka,25,15,[fe.b,j.b,V.b,le.b])),m=ky(se(ie(ks,1),ka,25,15,[wt.b,A.b,I.b,J.b])),_=xt.b,v=ky(se(ie(ks,1),ka,25,15,[Ve.b,we.b,Ne.b,rt.b])),g0(gc(e,Hf),n+a,x+_),g0(gc(e,Yo),n+a,x+_),g0(gc(e,Ph),n+a,0),g0(gc(e,tl),n+a,x+_+m),g0(gc(e,Ql),0,x+_),g0(gc(e,_l),n+a+s,x+_),g0(gc(e,eu),n+a+s,0),g0(gc(e,nl),0,x+_+m),g0(gc(e,Tl),n+a+s,x+_+m),g0(gc(e,uc),0,x),g0(gc(e,Cc),n,0),g0(gc(e,Tc),0,x+_+m),g0(gc(e,Gc),n+a+s,0),p=new Rs,p.a=ky(se(ie(ks,1),ka,25,15,[n+s+a+u,xt.a,le.a,rt.a])),p.b=ky(se(ie(ks,1),ka,25,15,[x+m+_+v,oe.b,ln.b,P.b])),p}function GNn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;for(V=new qe,I=new Q(e.d.b);I.a<I.c.c.length;)for(A=h(K(I),29),j=new Q(A.a);j.a<j.c.c.length;){for(P=h(K(j),10),a=h(_n(e.f,P),57),m=new Sn(Dn(qi(P).a.Kc(),new L));rr(m);)if(p=h(Un(m),17),s=pr(p.a,0),x=!0,_=null,s.b!=s.d.c){for(t=h(gr(s),8),n=null,p.c.j==(Qe(),sn)&&(J=new S9(t,new dt(t.a,a.d.d),a,p),J.f.a=!0,J.a=p.c,V.c[V.c.length]=J),p.c.j==Fn&&(J=new S9(t,new dt(t.a,a.d.d+a.d.a),a,p),J.f.d=!0,J.a=p.c,V.c[V.c.length]=J);s.b!=s.d.c;)n=h(gr(s),8),ige(t.b,n.b)||(_=new S9(t,n,null,p),V.c[V.c.length]=_,x&&(x=!1,n.b<a.d.d?_.f.a=!0:n.b>a.d.d+a.d.a?_.f.d=!0:(_.f.d=!0,_.f.a=!0))),s.b!=s.d.c&&(t=n);_&&(u=h(_n(e.f,p.d.i),57),t.b<u.d.d?_.f.a=!0:t.b>u.d.d+u.d.a?_.f.d=!0:(_.f.d=!0,_.f.a=!0))}for(v=new Sn(Dn(za(P).a.Kc(),new L));rr(v);)p=h(Un(v),17),p.a.b!=0&&(t=h(lP(p.a),8),p.d.j==(Qe(),sn)&&(J=new S9(t,new dt(t.a,a.d.d),a,p),J.f.a=!0,J.a=p.d,V.c[V.c.length]=J),p.d.j==Fn&&(J=new S9(t,new dt(t.a,a.d.d+a.d.a),a,p),J.f.d=!0,J.a=p.d,V.c[V.c.length]=J))}return V}function VNn(e,t,n){var s,a,u,p,v,m,x,_,A;if(Bn(n,"Network simplex node placement",1),e.e=t,e.n=h(U(t,(He(),d3)),304),oIn(e),HEn(e),Ti(Xa(new Vt(null,new Wt(e.e.b,16)),new _$e),new dWe(e)),Ti(Jr(Xa(Jr(Xa(new Vt(null,new Wt(e.e.b,16)),new F$e),new R$e),new j$e),new $$e),new fWe(e)),ft(lt(U(e.e,(tt(),D_))))&&(p=jo(n,1),Bn(p,"Straight Edges Pre-Processing",1),COn(e),An(p)),x8n(e.f),u=h(U(t,O_),19).a*e.f.a.c.length,$Q(whe(mhe(QY(e.f),u),!1),jo(n,1)),e.d.a.gc()!=0){for(p=jo(n,1),Bn(p,"Flexible Where Space Processing",1),v=h(sb(BP(pc(new Vt(null,new Wt(e.f.a,16)),new T$e),new w$e)),19).a,m=h(sb(PP(pc(new Vt(null,new Wt(e.f.a,16)),new C$e),new m$e)),19).a,x=m-v,_=ib(new _p,e.f),A=ib(new _p,e.f),Th(ph(gh(dh(bh(new Il,2e4),x),_),A)),Ti(Jr(Jr(wW(e.i),new S$e),new A$e),new fit(v,_,x,A)),a=e.d.a.ec().Kc();a.Ob();)s=h(a.Pb(),213),s.g=1;$Q(whe(mhe(QY(e.f),u),!1),jo(p,1)),An(p)}ft(lt(U(t,D_)))&&(p=jo(n,1),Bn(p,"Straight Edges Post-Processing",1),L_n(e),An(p)),lOn(e),e.e=null,e.f=null,e.i=null,e.c=null,Kc(e.k),e.j=null,e.a=null,e.o=null,e.d.a.$b(),An(n)}function UNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne;for(v=new Q(e.a.b);v.a<v.c.c.length;)for(u=h(K(v),29),le=new Q(u.a);le.a<le.c.c.length;)fe=h(K(le),10),t.g[fe.p]=fe,t.a[fe.p]=fe,t.d[fe.p]=0;for(m=e.a.b,t.c==(r1(),I0)&&(m=ue(m,152)?X4(h(m,152)):ue(m,131)?h(m,131).a:ue(m,54)?new U2(m):new Xv(m)),p=m.Kc();p.Ob();)for(u=h(p.Pb(),29),P=-1,I=u.a,t.o==(yf(),Rf)&&(P=Ir,I=ue(I,152)?X4(h(I,152)):ue(I,131)?h(I,131).a:ue(I,54)?new U2(I):new Xv(I)),Ne=I.Kc();Ne.Ob();)if(we=h(Ne.Pb(),10),A=null,t.c==I0?A=h(ut(e.b.f,we.p),15):A=h(ut(e.b.b,we.p),15),A.gc()>0)if(s=A.gc(),x=Li(d.Math.floor((s+1)/2))-1,a=Li(d.Math.ceil((s+1)/2))-1,t.o==Rf)for(_=a;_>=x;_--)t.a[we.p]==we&&(V=h(A.Xb(_),46),j=h(V.a,10),!wd(n,V.b)&&P>e.b.e[j.p]&&(t.a[j.p]=we,t.g[we.p]=t.g[j.p],t.a[we.p]=t.g[we.p],t.f[t.g[we.p].p]=(on(),!!(ft(t.f[t.g[we.p].p])&we.k==(bn(),ds))),P=e.b.e[j.p]));else for(_=x;_<=a;_++)t.a[we.p]==we&&(oe=h(A.Xb(_),46),J=h(oe.a,10),!wd(n,oe.b)&&P<e.b.e[J.p]&&(t.a[J.p]=we,t.g[we.p]=t.g[J.p],t.a[we.p]=t.g[we.p],t.f[t.g[we.p].p]=(on(),!!(ft(t.f[t.g[we.p].p])&we.k==(bn(),ds))),P=e.b.e[J.p]))}function Qo(){Qo=re,tQe(),_9t=ps.a,h(he(De(ps.a),0),18),x9t=ps.f,h(he(De(ps.f),0),18),h(he(De(ps.f),1),34),E9t=ps.n,h(he(De(ps.n),0),34),h(he(De(ps.n),1),34),h(he(De(ps.n),2),34),h(he(De(ps.n),3),34),iEe=ps.g,h(he(De(ps.g),0),18),h(he(De(ps.g),1),34),k9t=ps.c,h(he(De(ps.c),0),18),h(he(De(ps.c),1),18),sEe=ps.i,h(he(De(ps.i),0),18),h(he(De(ps.i),1),18),h(he(De(ps.i),2),18),h(he(De(ps.i),3),18),h(he(De(ps.i),4),34),aEe=ps.j,h(he(De(ps.j),0),18),rEe=ps.d,h(he(De(ps.d),0),18),h(he(De(ps.d),1),18),h(he(De(ps.d),2),18),h(he(De(ps.d),3),18),h(he(De(ps.d),4),34),h(he(De(ps.d),5),34),h(he(De(ps.d),6),34),h(he(De(ps.d),7),34),y9t=ps.b,h(he(De(ps.b),0),34),h(he(De(ps.b),1),34),zH=ps.e,h(he(De(ps.e),0),34),h(he(De(ps.e),1),34),h(he(De(ps.e),2),34),h(he(De(ps.e),3),34),h(he(De(ps.e),4),18),h(he(De(ps.e),5),18),h(he(De(ps.e),6),18),h(he(De(ps.e),7),18),h(he(De(ps.e),8),18),h(he(De(ps.e),9),18),h(he(De(ps.e),10),34),ep=ps.k,h(he(De(ps.k),0),34),h(he(De(ps.k),1),34)}function qNn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn;for(xt=new di,Ve=new di,J=-1,m=new Q(e);m.a<m.c.c.length;){for(p=h(K(m),128),p.s=J--,_=0,le=0,u=new Q(p.t);u.a<u.c.c.length;)s=h(K(u),268),le+=s.c;for(a=new Q(p.i);a.a<a.c.c.length;)s=h(K(a),268),_+=s.c;p.n=_,p.u=le,le==0?Ai(Ve,p,Ve.c.b,Ve.c):_==0&&Ai(xt,p,xt.c.b,xt.c)}for(dn=bB(e),A=e.c.length,V=A+1,oe=A-1,P=new qe;dn.a.gc()!=0;){for(;Ve.b!=0;)Ne=(kn(Ve.b!=0),h(pl(Ve,Ve.a.a),128)),dn.a.Bc(Ne)!=null,Ne.s=oe--,$be(Ne,xt,Ve);for(;xt.b!=0;)rt=(kn(xt.b!=0),h(pl(xt,xt.a.a),128)),dn.a.Bc(rt)!=null,rt.s=V++,$be(rt,xt,Ve);for(j=$s,x=dn.a.ec().Kc();x.Ob();)p=h(x.Pb(),128),fe=p.u-p.n,fe>=j&&(fe>j&&(P.c=pe(yn,st,1,0,5,1),j=fe),P.c[P.c.length]=p);P.c.length!=0&&(I=h(ut(P,NF(t,P.c.length)),128),dn.a.Bc(I)!=null,I.s=V++,$be(I,xt,Ve),P.c=pe(yn,st,1,0,5,1))}for(we=e.c.length+1,v=new Q(e);v.a<v.c.c.length;)p=h(K(v),128),p.s<A&&(p.s+=we);for(wt=new Q(e);wt.a<wt.c.c.length;)for(rt=h(K(wt),128),n=new Ms(rt.t,0);n.b<n.d.gc();)s=(kn(n.b<n.d.gc()),h(n.d.Xb(n.c=n.b++),268)),ln=s.b,rt.s>ln.s&&(Mu(n),vc(ln.i,s),s.c>0&&(s.a=ln,Ue(ln.t,s),s.b=rt,Ue(rt.i,s)))}function Vve(e){var t,n,s,a,u;switch(t=e.c,t){case 11:return e.Ml();case 12:return e.Ol();case 14:return e.Ql();case 15:return e.Tl();case 16:return e.Rl();case 17:return e.Ul();case 21:return Sr(e),Mr(),Mr(),wT;case 10:switch(e.a){case 65:return e.yl();case 90:return e.Dl();case 122:return e.Kl();case 98:return e.El();case 66:return e.zl();case 60:return e.Jl();case 62:return e.Hl()}}switch(u=$Nn(e),t=e.c,t){case 3:return e.Zl(u);case 4:return e.Xl(u);case 5:return e.Yl(u);case 0:if(e.a==123&&e.d<e.j){if(a=e.d,s=0,n=-1,(t=Is(e.i,a++))>=48&&t<=57){for(s=t-48;a<e.j&&(t=Is(e.i,a++))>=48&&t<=57;)if(s=s*10+t-48,s<0)throw ne(new Zn(ir((Qn(),Kye))))}else throw ne(new Zn(ir((Qn(),qmt))));if(n=s,t==44){if(a>=e.j)throw ne(new Zn(ir((Qn(),Wmt))));if((t=Is(e.i,a++))>=48&&t<=57){for(n=t-48;a<e.j&&(t=Is(e.i,a++))>=48&&t<=57;)if(n=n*10+t-48,n<0)throw ne(new Zn(ir((Qn(),Kye))));if(s>n)throw ne(new Zn(ir((Qn(),Kmt))))}else n=-1}if(t!=125)throw ne(new Zn(ir((Qn(),Ymt))));e.sl(a)?(u=(Mr(),Mr(),new dw(9,u)),e.d=a+1):(u=(Mr(),Mr(),new dw(3,u)),e.d=a),u.dm(s),u.cm(n),Sr(e)}}return u}function Ebt(e,t,n,s,a){var u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn;for(V=new Wo(t.b),we=new Wo(t.b),I=new Wo(t.b),wt=new Wo(t.b),J=new Wo(t.b),rt=pr(t,0);rt.b!=rt.d.c;)for(Ne=h(gr(rt),11),v=new Q(Ne.g);v.a<v.c.c.length;)if(u=h(K(v),17),u.c.i==u.d.i){if(Ne.j==u.d.j){wt.c[wt.c.length]=u;continue}else if(Ne.j==(Qe(),sn)&&u.d.j==Fn){J.c[J.c.length]=u;continue}}for(m=new Q(J);m.a<m.c.c.length;)u=h(K(m),17),SDn(e,u,n,s,(Qe(),pn));for(p=new Q(wt);p.a<p.c.c.length;)u=h(K(p),17),xt=new Nd(e),kp(xt,(bn(),Vo)),Fe(xt,(tt(),Ei),(_s(),qo)),Fe(xt,(He(),Br),u),ln=new So,Fe(ln,Br,u.d),Ji(ln,(Qe(),un)),Ka(ln,xt),dn=new So,Fe(dn,Br,u.c),Ji(dn,pn),Ka(dn,xt),Fe(u.c,Zc,xt),Fe(u.d,Zc,xt),Gs(u,null),xs(u,null),n.c[n.c.length]=xt,Fe(xt,P$,Ke(2));for(Ve=pr(t,0);Ve.b!=Ve.d.c;)Ne=h(gr(Ve),11),x=Ne.e.c.length>0,oe=Ne.g.c.length>0,x&&oe?I.c[I.c.length]=Ne:x?V.c[V.c.length]=Ne:oe&&(we.c[we.c.length]=Ne);for(j=new Q(V);j.a<j.c.c.length;)P=h(K(j),11),Ue(a,Cve(e,P,null,n));for(le=new Q(we);le.a<le.c.c.length;)fe=h(K(le),11),Ue(a,Cve(e,null,fe,n));for(A=new Q(I);A.a<A.c.c.length;)_=h(K(A),11),Ue(a,Cve(e,_,_,n))}function _bt(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln;for(fe=new dt(ki,ki),t=new dt(Fi,Fi),wt=new Q(e);wt.a<wt.c.c.length;)rt=h(K(wt),8),fe.a=d.Math.min(fe.a,rt.a),fe.b=d.Math.min(fe.b,rt.b),t.a=d.Math.max(t.a,rt.a),t.b=d.Math.max(t.b,rt.b);for(I=new dt(t.a-fe.a,t.b-fe.b),x=new dt(fe.a-50,fe.b-I.a-50),_=new dt(fe.a-50,t.b+I.a+50),A=new dt(t.a+I.b/2+50,fe.b+I.b/2),P=new Eve(x,_,A),Ve=new ns,u=new qe,n=new qe,Ve.a.zc(P,Ve),ln=new Q(e);ln.a<ln.c.c.length;){for(xt=h(K(ln),8),u.c=pe(yn,st,1,0,5,1),Ne=Ve.a.ec().Kc();Ne.Ob();)le=h(Ne.Pb(),308),s=le.d,yg(s,le.a),Mb(yg(le.d,xt),yg(le.d,le.a))<0&&(u.c[u.c.length]=le);for(n.c=pe(yn,st,1,0,5,1),we=new Q(u);we.a<we.c.c.length;)for(le=h(K(we),308),J=new Q(le.e);J.a<J.c.c.length;){for(j=h(K(J),168),p=!0,m=new Q(u);m.a<m.c.c.length;)v=h(K(m),308),v!=le&&(Bo(j,ut(v.e,0))||Bo(j,ut(v.e,1))||Bo(j,ut(v.e,2)))&&(p=!1);p&&(n.c[n.c.length]=j)}for(sdt(Ve,u),Os(Ve,new nr),V=new Q(n);V.a<V.c.c.length;)j=h(K(V),168),Qi(Ve,new Eve(xt,j.a,j.b))}for(oe=new ns,Os(Ve,new Gqe(oe)),a=oe.a.ec().Kc();a.Ob();)j=h(a.Pb(),168),(DB(P,j.a)||DB(P,j.b))&&a.Qb();return Os(oe,new Kn),oe}function YNn(e){var t,n,s,a,u;switch(n=h(U(e,(He(),Uo)),21),t=QN(c5t),a=h(U(e,(tt(),sm)),334),a==(Ld(),B0)&&qu(t,u5t),ft(lt(U(e,Sne)))?Lr(t,(Zs(),w1),(oa(),Bte)):Lr(t,(Zs(),oc),(oa(),Bte)),U(e,(xB(),X_))!=null&&qu(t,l5t),(ft(lt(U(e,pke)))||ft(lt(U(e,hke))))&&Wc(t,(Zs(),Pa),(oa(),l5e)),h(U(e,Ju),103).g){case 2:case 3:case 4:Wc(Lr(t,(Zs(),w1),(oa(),f5e)),Pa,h5e)}switch(n.Hc((ha(),L$))&&Wc(Lr(Lr(t,(Zs(),w1),(oa(),u5e)),Go,o5e),Pa,c5e),Me(U(e,Ane))!==Me((By(),iH))&&Lr(t,(Zs(),oc),(oa(),S5e)),n.Hc(O$)&&(Lr(t,(Zs(),w1),(oa(),L5e)),Lr(t,Hg,M5e),Lr(t,oc,D5e)),Me(U(e,H$))!==Me((ax(),x_))&&Me(U(e,Hd))!==Me((Od(),nL))&&Wc(t,(Zs(),Pa),(oa(),k5e)),ft(lt(U(e,dke)))&&Lr(t,(Zs(),oc),(oa(),y5e)),ft(lt(U(e,Ene)))&&Lr(t,(Zs(),oc),(oa(),I5e)),_Sn(e)&&(Me(U(e,sm))===Me(B0)?s=h(U(e,ID),292):s=h(U(e,xne),292),u=s==(jk(),one)?(oa(),A5e):(oa(),P5e),Lr(t,(Zs(),Go),u)),h(U(e,Hke),377).g){case 1:Lr(t,(Zs(),Go),(oa(),O5e));break;case 2:Wc(Lr(Lr(t,(Zs(),oc),(oa(),r5e)),Go,i5e),Pa,s5e)}return Me(U(e,Vg))!==Me((Dd(),Ug))&&Lr(t,(Zs(),oc),(oa(),N5e)),t}function Tbt(e){J2(e,new Xp(K2(q2(W2(Y2(new mp,Hl),"ELK Rectangle Packing"),"Algorithm for packing of unconnected boxes, i.e. graphs without edges. The given order of the boxes is always preserved and the main reading direction of the boxes is left to right. The algorithm is divided into two phases. One phase approximates the width in which the rectangles can be placed. The next phase places the rectangles in rows using the previously calculated width as bounding width and bundles rectangles with a similar height in blocks. A compaction step reduces the size of the drawing. Finally, the rectangles are expanded to fill their bounding box and eliminate empty unused spaces."),new IHe))),et(e,Hl,k5,1.3),et(e,Hl,oj,at(i8e)),et(e,Hl,Vb,h8e),et(e,Hl,zw,15),et(e,Hl,tj,at(j7t)),et(e,Hl,Yy,at(z7t)),et(e,Hl,E5,at(G7t)),et(e,Hl,x5,at(V7t)),et(e,Hl,Dx,at(H7t)),et(e,Hl,z9,at(c8e)),et(e,Hl,Lx,at(q7t)),et(e,Hl,dye,at(l8e)),et(e,Hl,gye,at(o8e)),et(e,Hl,vye,at(u8e)),et(e,Hl,wye,at(f8e)),et(e,Hl,ree,at(s8e)),et(e,Hl,qM,at(a8e)),et(e,Hl,YJ,at($7t)),et(e,Hl,bye,at(VD)),et(e,Hl,pye,at(r8e)),et(e,Hl,mye,at(d8e))}function Hb(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J;if(n==null)return null;if(e.a!=t.Aj())throw ne(new an(Px+t.ne()+Wb));if(ue(t,457)){if(J=eMn(h(t,671),n),!J)throw ne(new an(pee+n+"' is not a valid enumerator of '"+t.ne()+"'"));return J}switch(v0((jc(),Ps),t).cl()){case 2:{n=Ho(n,!1);break}case 3:{n=Ho(n,!0);break}}if(s=v0(Ps,t).$k(),s)return s.Aj().Nh().Kh(s,n);if(I=v0(Ps,t).al(),I){for(J=new qe,x=eX(n),_=0,A=x.length;_<A;++_)m=x[_],Ue(J,I.Aj().Nh().Kh(I,m));return J}if(V=v0(Ps,t).bl(),!V.dc()){for(j=V.Kc();j.Ob();){P=h(j.Pb(),148);try{if(J=P.Aj().Nh().Kh(P,n),J!=null)return J}catch(oe){if(oe=ui(oe),!ue(oe,60))throw ne(oe)}}throw ne(new an(pee+n+"' does not match any member types of the union datatype '"+t.ne()+"'"))}if(h(t,834).Fj(),a=J6n(t.Bj()),!a)return null;if(a==f_){p=0;try{p=Wu(n,$s,Ir)&Oi}catch(oe){if(oe=ui(oe),ue(oe,127))u=LB(n),p=u[0];else throw ne(oe)}return RA(p)}if(a==Bj){for(v=0;v<uT.length;++v)try{return fZe(uT[v],n)}catch(oe){if(oe=ui(oe),!ue(oe,32))throw ne(oe)}throw ne(new an(pee+n+"' is not a date formatted string of the form yyyy-MM-dd'T'HH:mm:ss'.'SSSZ or a valid subset thereof"))}throw ne(new an(pee+n+"' is invalid. "))}function WNn(e,t){var n,s,a,u,p,v,m,x;if(n=0,p=0,u=t.length,v=null,x=new Kv,p<u&&(er(p,t.length),t.charCodeAt(p)==43)&&(++p,++n,p<u&&(er(p,t.length),t.charCodeAt(p)==43||(er(p,t.length),t.charCodeAt(p)==45))))throw ne(new Qf(Gb+t+'"'));for(;p<u&&(er(p,t.length),t.charCodeAt(p)!=46)&&(er(p,t.length),t.charCodeAt(p)!=101)&&(er(p,t.length),t.charCodeAt(p)!=69);)++p;if(x.a+=""+$u(t==null?_c:(rn(t),t),n,p),p<u&&(er(p,t.length),t.charCodeAt(p)==46)){for(++p,n=p;p<u&&(er(p,t.length),t.charCodeAt(p)!=101)&&(er(p,t.length),t.charCodeAt(p)!=69);)++p;e.e=p-n,x.a+=""+$u(t==null?_c:(rn(t),t),n,p)}else e.e=0;if(p<u&&(er(p,t.length),t.charCodeAt(p)==101||(er(p,t.length),t.charCodeAt(p)==69))&&(++p,n=p,p<u&&(er(p,t.length),t.charCodeAt(p)==43)&&(++p,p<u&&(er(p,t.length),t.charCodeAt(p)!=45)&&++n),v=t.substr(n,u-n),e.e=e.e-Wu(v,$s,Ir),e.e!=Li(e.e)))throw ne(new Qf("Scale out of range."));if(m=x.a,m.length<16){if(e.f=(F3e==null&&(F3e=new RegExp("^[+-]?\\d*$","i")),F3e.test(m)?parseInt(m,10):NaN),isNaN(e.f))throw ne(new Qf(Gb+t+'"'));e.a=vbe(e.f)}else pxn(e,new hg(m));for(e.d=x.a.length,a=0;a<x.a.length&&(s=Is(x.a,a),!(s!=45&&s!=48));++a)--e.d;e.d==0&&(e.d=1)}function uZ(){uZ=re,Wr=new pb,Ot(Wr,(Qe(),Hf),Yo),Ot(Wr,Ql,Yo),Ot(Wr,Ql,Sc),Ot(Wr,_l,Gc),Ot(Wr,_l,Yo),Ot(Wr,Ph,Yo),Ot(Wr,Ph,tu),Ot(Wr,tl,Tc),Ot(Wr,tl,Yo),Ot(Wr,Cc,uc),Ot(Wr,Cc,Yo),Ot(Wr,Cc,tu),Ot(Wr,Cc,Tc),Ot(Wr,uc,Cc),Ot(Wr,uc,Sc),Ot(Wr,uc,Gc),Ot(Wr,uc,Yo),Ot(Wr,Bh,Bh),Ot(Wr,Bh,tu),Ot(Wr,Bh,Sc),Ot(Wr,eu,eu),Ot(Wr,eu,tu),Ot(Wr,eu,Gc),Ot(Wr,nl,nl),Ot(Wr,nl,Tc),Ot(Wr,nl,Sc),Ot(Wr,Tl,Tl),Ot(Wr,Tl,Tc),Ot(Wr,Tl,Gc),Ot(Wr,tu,Ph),Ot(Wr,tu,Cc),Ot(Wr,tu,Bh),Ot(Wr,tu,eu),Ot(Wr,tu,Yo),Ot(Wr,tu,tu),Ot(Wr,tu,Sc),Ot(Wr,tu,Gc),Ot(Wr,Tc,tl),Ot(Wr,Tc,Cc),Ot(Wr,Tc,nl),Ot(Wr,Tc,Tl),Ot(Wr,Tc,Tc),Ot(Wr,Tc,Sc),Ot(Wr,Tc,Gc),Ot(Wr,Tc,Yo),Ot(Wr,Sc,Ql),Ot(Wr,Sc,uc),Ot(Wr,Sc,Bh),Ot(Wr,Sc,nl),Ot(Wr,Sc,tu),Ot(Wr,Sc,Tc),Ot(Wr,Sc,Sc),Ot(Wr,Sc,Yo),Ot(Wr,Gc,_l),Ot(Wr,Gc,uc),Ot(Wr,Gc,eu),Ot(Wr,Gc,Tl),Ot(Wr,Gc,tu),Ot(Wr,Gc,Tc),Ot(Wr,Gc,Gc),Ot(Wr,Gc,Yo),Ot(Wr,Yo,Hf),Ot(Wr,Yo,Ql),Ot(Wr,Yo,_l),Ot(Wr,Yo,Ph),Ot(Wr,Yo,tl),Ot(Wr,Yo,Cc),Ot(Wr,Yo,uc),Ot(Wr,Yo,tu),Ot(Wr,Yo,Tc),Ot(Wr,Yo,Sc),Ot(Wr,Yo,Gc),Ot(Wr,Yo,Yo)}function Uve(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt;for(e.d=new dt(ki,ki),e.c=new dt(Fi,Fi),I=t.Kc();I.Ob();)for(_=h(I.Pb(),37),le=new Q(_.a);le.a<le.c.c.length;)fe=h(K(le),10),e.d.a=d.Math.min(e.d.a,fe.n.a-fe.d.b),e.d.b=d.Math.min(e.d.b,fe.n.b-fe.d.d),e.c.a=d.Math.max(e.c.a,fe.n.a+fe.o.a+fe.d.c),e.c.b=d.Math.max(e.c.b,fe.n.b+fe.o.b+fe.d.a);for(v=new HKe,A=t.Kc();A.Ob();)_=h(A.Pb(),37),s=LNn(e,_),Ue(v.a,s),s.a=s.a|!h(U(s.c,(He(),em)),21).dc();for(e.b=(uX(),wt=new D1,wt.f=new Mut(n),wt.b=jIn(wt.f,v),wt),YIn((j=e.b,new H6,j)),e.e=new Rs,e.a=e.b.f.e,p=new Q(v.a);p.a<p.c.c.length;)for(a=h(K(p),841),we=Lwn(e.b,a),OMn(a.c,we.a,we.b),J=new Q(a.c.a);J.a<J.c.c.length;)V=h(K(J),10),V.k==(bn(),Pi)&&(oe=Hbe(e,V.n,h(U(V,(He(),co)),61)),zr(Uh(V.n),oe));for(u=new Q(v.a);u.a<u.c.c.length;)for(a=h(K(u),841),x=new Q(ukn(a));x.a<x.c.c.length;)for(m=h(K(x),17),rt=new dN(m.a),rk(rt,0,kf(m.c)),vr(rt,kf(m.d)),P=null,Ve=pr(rt,0);Ve.b!=Ve.d.c;){if(Ne=h(gr(Ve),8),!P){P=Ne;continue}sge(P.a,Ne.a)?(e.e.a=d.Math.min(e.e.a,P.a),e.a.a=d.Math.max(e.a.a,P.a)):sge(P.b,Ne.b)&&(e.e.b=d.Math.min(e.e.b,P.b),e.a.b=d.Math.max(e.a.b,P.b)),P=Ne}fP(e.e),zr(e.a,e.e)}function KNn(e){Yn(e.b,Vl,se(ie(it,1),Re,2,6,[Kb,"ConsistentTransient"])),Yn(e.a,Vl,se(ie(it,1),Re,2,6,[Kb,"WellFormedSourceURI"])),Yn(e.o,Vl,se(ie(it,1),Re,2,6,[Kb,"InterfaceIsAbstract AtMostOneID UniqueFeatureNames UniqueOperationSignatures NoCircularSuperTypes WellFormedMapEntryClass ConsistentSuperTypes DisjointFeatureAndOperationSignatures"])),Yn(e.p,Vl,se(ie(it,1),Re,2,6,[Kb,"WellFormedInstanceTypeName UniqueTypeParameterNames"])),Yn(e.v,Vl,se(ie(it,1),Re,2,6,[Kb,"UniqueEnumeratorNames UniqueEnumeratorLiterals"])),Yn(e.R,Vl,se(ie(it,1),Re,2,6,[Kb,"WellFormedName"])),Yn(e.T,Vl,se(ie(it,1),Re,2,6,[Kb,"UniqueParameterNames UniqueTypeParameterNames NoRepeatingVoid"])),Yn(e.U,Vl,se(ie(it,1),Re,2,6,[Kb,"WellFormedNsURI WellFormedNsPrefix UniqueSubpackageNames UniqueClassifierNames UniqueNsURIs"])),Yn(e.W,Vl,se(ie(it,1),Re,2,6,[Kb,"ConsistentOpposite SingleContainer ConsistentKeys ConsistentUnique ConsistentContainer"])),Yn(e.bb,Vl,se(ie(it,1),Re,2,6,[Kb,"ValidDefaultValueLiteral"])),Yn(e.eb,Vl,se(ie(it,1),Re,2,6,[Kb,"ValidLowerBound ValidUpperBound ConsistentBounds ValidType"])),Yn(e.H,Vl,se(ie(it,1),Re,2,6,[Kb,"ConsistentType ConsistentBounds ConsistentArguments"]))}function XNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt;if(!t.dc()){if(a=new Oc,v=n||h(t.Xb(0),17),j=v.c,x9(),I=j.i.k,!(I==(bn(),Wi)||I==Vo||I==Pi||I==Bf))throw ne(new an("The target node of the edge must be a normal node or a northSouthPort."));for(Cp(a,Qa(se(ie(cs,1),Re,8,0,[j.i.n,j.n,j.a]))),(Qe(),Cc).Hc(j.j)&&(J=Pe(Je(U(j,(He(),a8)))),A=new dt(Qa(se(ie(cs,1),Re,8,0,[j.i.n,j.n,j.a])).a,J),Ai(a,A,a.c.b,a.c)),_=null,s=!1,m=t.Kc();m.Ob();)p=h(m.Pb(),17),u=p.a,u.b!=0&&(s?(x=Jf(zr(_,(kn(u.b!=0),h(u.a.a.c,8))),.5),Ai(a,x,a.c.b,a.c),s=!1):s=!0,_=ro((kn(u.b!=0),h(u.c.b.c,8))),Qs(a,u),Pl(u));V=v.d,Cc.Hc(V.j)&&(J=Pe(Je(U(V,(He(),a8)))),A=new dt(Qa(se(ie(cs,1),Re,8,0,[V.i.n,V.n,V.a])).a,J),Ai(a,A,a.c.b,a.c)),Cp(a,Qa(se(ie(cs,1),Re,8,0,[V.i.n,V.n,V.a]))),e.d==(qk(),Kne)&&(oe=(kn(a.b!=0),h(a.a.a.c,8)),fe=h(Zh(a,1),8),le=new WW(Xge(j.j)),le.a*=5,le.b*=5,we=ms(new dt(fe.a,fe.b),oe),Ne=new dt(UW(le.a,we.a),UW(le.b,we.b)),zr(Ne,oe),Ve=pr(a,1),nA(Ve,Ne),rt=(kn(a.b!=0),h(a.c.b.c,8)),wt=h(Zh(a,a.b-2),8),le=new WW(Xge(V.j)),le.a*=5,le.b*=5,we=ms(new dt(wt.a,wt.b),rt),xt=new dt(UW(le.a,we.a),UW(le.b,we.b)),zr(xt,rt),rk(a,a.b-1,xt)),P=new _ve(a),Qs(v.a,Kft(P))}}function QNn(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts,il,rh,Vf,iz,xL,mT,EL;if(le=h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82),Ne=le.Dg(),Ve=le.Eg(),we=le.Cg()/2,V=le.Bg()/2,ue(le,186)&&(fe=h(le,118),Ne+=mf(fe).i,Ne+=mf(fe).i),Ne+=we,Ve+=V,dn=h(he((!e.b&&(e.b=new qt(Pn,e,4,7)),e.b),0),82),Dr=dn.Dg(),ts=dn.Eg(),Ln=dn.Cg()/2,rt=dn.Bg()/2,ue(dn,186)&&(ln=h(dn,118),Dr+=mf(ln).i,Dr+=mf(ln).i),Dr+=Ln,ts+=rt,(!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i==0)v=(X2(),x=new GO,x),qn((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a),v);else if((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i>1)for(j=new N4((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a));j.e!=j.i.gc();)p9(j);for(p=h(he((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a),0),202),J=Dr,Dr>Ne+we?J=Ne+we:Dr<Ne-we&&(J=Ne-we),oe=ts,ts>Ve+V?oe=Ve+V:ts<Ve-V&&(oe=Ve-V),J>Ne-we&&J<Ne+we&&oe>Ve-V&&oe<Ve+V&&(J=Ne+we),Mk(p,J),Lk(p,oe),wt=Ne,Ne>Dr+Ln?wt=Dr+Ln:Ne<Dr-Ln&&(wt=Dr-Ln),xt=Ve,Ve>ts+rt?xt=ts+rt:Ve<ts-rt&&(xt=ts-rt),wt>Dr-Ln&&wt<Dr+Ln&&xt>ts-rt&&xt<ts+rt&&(xt=ts+rt),Ak(p,wt),Dk(p,xt),jn((!p.a&&(p.a=new Gi(Zl,p,5)),p.a)),u=NF(t,5),le==dn&&++u,Vf=wt-J,mT=xt-oe,il=d.Math.sqrt(Vf*Vf+mT*mT),A=il*.20000000298023224,iz=Vf/(u+1),EL=mT/(u+1),rh=J,xL=oe,_=0;_<u;_++)rh+=iz,xL+=EL,I=rh+hu(t,24)*HM*A-A/2,I<0?I=1:I>n&&(I=n-1),P=xL+hu(t,24)*HM*A-A/2,P<0?P=1:P>s&&(P=s-1),a=(X2(),m=new zO,m),$B(a,I),HB(a,P),qn((!p.a&&(p.a=new Gi(Zl,p,5)),p.a),a)}function tt(){tt=re,Lne=(Tr(),GEt),Oke=VEt,ND=_7e,Mh=UEt,z5=T7e,cv=qEt,lm=C7e,h8=S7e,f8=A7e,Ine=PH,uv=k2,One=YEt,I_=L7e,Z$=Y5,OD=(Yve(),$kt),p3=Hkt,b2=zkt,b3=Gkt,Cxt=new sa(NH,Ke(0)),l8=Fkt,Ike=Rkt,H5=jkt,Hke=hxt,Nke=qkt,Pke=Kkt,Pne=nxt,Bke=Zkt,Fke=ext,J$=pxt,Bne=fxt,jke=oxt,Rke=sxt,$ke=uxt,av=Lkt,L_=Ikt,Cne=K6t,lke=Q6t,Ske=new nb(12),Cke=new sa(y2,Ske),oke=(Od(),y8),Hd=new sa(e7e,oke),om=new sa(pu,0),Sxt=new sa(Vre,Ke(1)),j$=new sa(U5,Mx),p2=OH,Ei=Z_,u8=T3,mxt=ZD,m1=NEt,sm=k3,Axt=new sa(Ure,(on(),!0)),am=JD,d2=Fre,g2=m2,Q$=Wg,Dne=IH,ake=(la(),Q1),Ju=new sa(hv,ake),sv=E3,K$=u7e,cm=gm,Txt=Gre,Dke=x7e,Mke=(Cy(),sL),new sa(v7e,Mke),xxt=jre,Ext=$re,_xt=Hre,kxt=Rre,Nne=Ukt,kke=mkt,Ane=wkt,O_=Vkt,cc=hkt,im=H6t,M_=$6t,rm=S6t,rke=A6t,xne=I6t,ID=M6t,Ene=R6t,xke=ykt,Eke=kkt,bke=skt,X$=Pkt,Mne=_kt,Sne=ekt,Tke=Mkt,uke=Y6t,Tne=W6t,kne=DH,_ke=xkt,H$=y6t,eke=m6t,$$=w6t,dke=rkt,fke=nkt,gke=ikt,o8=_3,Da=x3,D0=n7e,y1=Bre,_ne=Pre,ike=N6t,L0=zre,A_=FEt,q$=REt,ov=m7e,Ake=jEt,c8=$Et,wke=dkt,mke=pkt,um=q5,mne=v6t,yke=vkt,U$=V6t,V$=G6t,W$=eL,vke=ckt,D_=Ckt,PD=M7e,ske=z6t,Lke=Bkt,cke=U6t,yxt=lkt,wxt=B6t,pke=a7e,Y$=fkt,G$=F6t,Vg=C6t,nke=_6t,z$=x6t,tke=E6t,yne=T6t,$5=k6t,hke=tkt}function lZ(e,t){XQ();var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr;if(wt=e.e,j=e.d,a=e.a,wt==0)switch(t){case 0:return"0";case 1:return _x;case 2:return"0.00";case 3:return"0.000";case 4:return"0.0000";case 5:return"0.00000";case 6:return"0.000000";default:return Ve=new rg,t<0?Ve.a+="0E+":Ve.a+="0E",Ve.a+=-t,Ve.a}if(le=j*10+1+7,we=pe(Cl,o1,25,le+1,15,1),n=le,j==1)if(v=a[0],v<0){Dr=Zi(v,fa);do V=Dr,Dr=oM(Dr,10),we[--n]=48+Gn(Cg(V,js(Dr,10)))&Oi;while(mo(Dr,0)!=0)}else{Dr=v;do V=Dr,Dr=Dr/10|0,we[--n]=48+(V-Dr*10)&Oi;while(Dr!=0)}else{ln=pe(Hn,hr,25,j,15,1),Ln=j,Ao(a,0,ln,0,Ln);e:for(;;){for(rt=0,x=Ln-1;x>=0;x--)dn=Vs(kd(rt,32),Zi(ln[x],fa)),oe=n9n(dn),ln[x]=Gn(oe),rt=Gn(dg(oe,32));fe=Gn(rt),J=n;do we[--n]=48+fe%10&Oi;while((fe=fe/10|0)!=0&&n!=0);for(s=9-J+n,m=0;m<s&&n>0;m++)we[--n]=48;for(A=Ln-1;ln[A]==0;A--)if(A==0)break e;Ln=A+1}for(;we[n]==48;)++n}if(P=wt<0,p=le-n-t-1,t==0)return P&&(we[--n]=45),Fl(we,n,le-n);if(t>0&&p>=-6){if(p>=0){for(_=n+p,I=le-1;I>=_;I--)we[I+1]=we[I];return we[++_]=46,P&&(we[--n]=45),Fl(we,n,le-n+1)}for(A=2;A<-p+1;A++)we[--n]=48;return we[--n]=46,we[--n]=48,P&&(we[--n]=45),Fl(we,n,le-n)}return xt=n+1,u=le,Ne=new Kv,P&&(Ne.a+="-"),u-xt>=1?(pg(Ne,we[n]),Ne.a+=".",Ne.a+=Fl(we,n+1,le-n-1)):Ne.a+=Fl(we,n,le-n),Ne.a+="E",p>0&&(Ne.a+="+"),Ne.a+=""+p,Ne.a}function Cbt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;switch(e.c=t,e.g=new zn,n=(Qv(),new Yv(e.c)),s=new XO(n),Spe(s),le=Jn(pt(e.c,(uM(),E8e))),m=h(pt(e.c,Cre),316),Ne=h(pt(e.c,Sre),429),p=h(pt(e.c,y8e),482),we=h(pt(e.c,Tre),430),e.j=Pe(Je(pt(e.c,iEt))),v=e.a,m.g){case 0:v=e.a;break;case 1:v=e.b;break;case 2:v=e.i;break;case 3:v=e.e;break;case 4:v=e.f;break;default:throw ne(new an(cj+(m.f!=null?m.f:""+m.g)))}if(e.d=new Xit(v,Ne,p),Fe(e.d,(Bk(),p_),lt(pt(e.c,nEt))),e.d.c=ft(lt(pt(e.c,k8e))),QP(e.c).i==0)return e.d;for(A=new Tn(QP(e.c));A.e!=A.i.gc();){for(_=h(On(A),33),P=_.g/2,I=_.f/2,Ve=new dt(_.i+P,_.j+I);Au(e.g,Ve);)nw(Ve,(d.Math.random()-.5)*l1,(d.Math.random()-.5)*l1);V=h(pt(_,(Tr(),eL)),142),J=new dst(Ve,new hl(Ve.a-P-e.j/2-V.b,Ve.b-I-e.j/2-V.d,_.g+e.j+(V.b+V.c),_.f+e.j+(V.d+V.a))),Ue(e.d.i,J),Pr(e.g,Ve,new As(J,_))}switch(we.g){case 0:if(le==null)e.d.d=h(ut(e.d.i,0),65);else for(fe=new Q(e.d.i);fe.a<fe.c.c.length;)J=h(K(fe),65),j=h(h(_n(e.g,J.a),46).b,33).zg(),j!=null&&Nt(j,le)&&(e.d.d=J);break;case 1:for(a=new dt(e.c.g,e.c.f),a.a*=.5,a.b*=.5,nw(a,e.c.i,e.c.j),u=ki,oe=new Q(e.d.i);oe.a<oe.c.c.length;)J=h(K(oe),65),x=yg(J.a,a),x<u&&(u=x,e.d.d=J);break;default:throw ne(new an(cj+(we.f!=null?we.f:""+we.g)))}return e.d}function Sbt(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;for(Ne=h(he((!e.a&&(e.a=new Ye(gi,e,6,6)),e.a),0),202),_=new Oc,we=new zn,Ve=Gpt(Ne),sc(we.f,Ne,Ve),I=new zn,s=new di,j=gg(Cf(se(ie(Nf,1),st,20,0,[(!t.d&&(t.d=new qt(us,t,8,5)),t.d),(!t.e&&(t.e=new qt(us,t,7,4)),t.e)])));rr(j);){if(P=h(Un(j),79),(!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i!=1)throw ne(new an(tmt+(!e.a&&(e.a=new Ye(gi,e,6,6)),e.a).i));P!=e&&(J=h(he((!P.a&&(P.a=new Ye(gi,P,6,6)),P.a),0),202),Ai(s,J,s.c.b,s.c),V=h(no(Ia(we.f,J)),12),V||(V=Gpt(J),sc(we.f,J,V)),A=n?ms(new _a(h(ut(Ve,Ve.c.length-1),8)),h(ut(V,V.c.length-1),8)):ms(new _a((Kt(0,Ve.c.length),h(Ve.c[0],8))),(Kt(0,V.c.length),h(V.c[0],8))),sc(I.f,J,A))}if(s.b!=0)for(oe=h(ut(Ve,n?Ve.c.length-1:0),8),x=1;x<Ve.c.length;x++){for(fe=h(ut(Ve,n?Ve.c.length-1-x:x),8),a=pr(s,0);a.b!=a.d.c;)J=h(gr(a),202),V=h(no(Ia(we.f,J)),12),V.c.length<=x?BB(a):(le=zr(new _a(h(ut(V,n?V.c.length-1-x:x),8)),h(no(Ia(I.f,J)),8)),(fe.a!=le.a||fe.b!=le.b)&&(u=fe.a-oe.a,v=fe.b-oe.b,p=le.a-oe.a,m=le.b-oe.b,p*v==m*u&&(u==0||isNaN(u)?u:u<0?-1:1)==(p==0||isNaN(p)?p:p<0?-1:1)&&(v==0||isNaN(v)?v:v<0?-1:1)==(m==0||isNaN(m)?m:m<0?-1:1)?(d.Math.abs(u)<d.Math.abs(p)||d.Math.abs(v)<d.Math.abs(m))&&Ai(_,fe,_.c.b,_.c):x>1&&Ai(_,oe,_.c.b,_.c),BB(a)));oe=fe}return _}function ZNn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts,il,rh,Vf;for(Bn(n,"Greedy cycle removal",1),le=t.a,Vf=le.c.length,e.a=pe(Hn,hr,25,Vf,15,1),e.c=pe(Hn,hr,25,Vf,15,1),e.b=pe(Hn,hr,25,Vf,15,1),x=0,oe=new Q(le);oe.a<oe.c.c.length;){for(V=h(K(oe),10),V.p=x,xt=new Q(V.j);xt.a<xt.c.c.length;){for(Ve=h(K(xt),11),v=new Q(Ve.e);v.a<v.c.c.length;)s=h(K(v),17),s.c.i!=V&&(Ln=h(U(s,(tt(),l8)),19).a,e.a[x]+=Ln>0?Ln+1:1);for(p=new Q(Ve.g);p.a<p.c.c.length;)s=h(K(p),17),s.d.i!=V&&(Ln=h(U(s,(tt(),l8)),19).a,e.c[x]+=Ln>0?Ln+1:1)}e.c[x]==0?vr(e.e,V):e.a[x]==0&&vr(e.f,V),++x}for(j=-1,P=1,A=new qe,e.d=h(U(t,(He(),R5)),230);Vf>0;){for(;e.e.b!=0;)ts=h(eW(e.e),10),e.b[ts.p]=j--,dve(e,ts),--Vf;for(;e.f.b!=0;)il=h(eW(e.f),10),e.b[il.p]=P++,dve(e,il),--Vf;if(Vf>0){for(I=$s,fe=new Q(le);fe.a<fe.c.c.length;)V=h(K(fe),10),e.b[V.p]==0&&(we=e.c[V.p]-e.a[V.p],we>=I&&(we>I&&(A.c=pe(yn,st,1,0,5,1),I=we),A.c[A.c.length]=V));_=e.Zf(A),e.b[_.p]=P++,dve(e,_),--Vf}}for(Dr=le.c.length+1,x=0;x<le.c.length;x++)e.b[x]<0&&(e.b[x]+=Dr);for(J=new Q(le);J.a<J.c.c.length;)for(V=h(K(J),10),dn=aat(V.j),rt=dn,wt=0,ln=rt.length;wt<ln;++wt)for(Ve=rt[wt],Ne=i1(Ve.g),a=Ne,u=0,m=a.length;u<m;++u)s=a[u],rh=s.d.i.p,e.b[V.p]>e.b[rh]&&(jb(s,!0),Fe(t,SD,(on(),!0)));e.a=null,e.c=null,e.b=null,Pl(e.f),Pl(e.e),An(n)}function Abt(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;for(s=new qe,v=new qe,J=t/2,P=e.gc(),a=h(e.Xb(0),8),oe=h(e.Xb(1),8),j=_Q(a.a,a.b,oe.a,oe.b,J),Ue(s,(Kt(0,j.c.length),h(j.c[0],8))),Ue(v,(Kt(1,j.c.length),h(j.c[1],8))),x=2;x<P;x++)V=a,a=oe,oe=h(e.Xb(x),8),j=_Q(a.a,a.b,V.a,V.b,J),Ue(s,(Kt(1,j.c.length),h(j.c[1],8))),Ue(v,(Kt(0,j.c.length),h(j.c[0],8))),j=_Q(a.a,a.b,oe.a,oe.b,J),Ue(s,(Kt(0,j.c.length),h(j.c[0],8))),Ue(v,(Kt(1,j.c.length),h(j.c[1],8)));for(j=_Q(oe.a,oe.b,a.a,a.b,J),Ue(s,(Kt(1,j.c.length),h(j.c[1],8))),Ue(v,(Kt(0,j.c.length),h(j.c[0],8))),n=new Oc,p=new qe,vr(n,(Kt(0,s.c.length),h(s.c[0],8))),_=1;_<s.c.length-2;_+=2)u=(Kt(_,s.c.length),h(s.c[_],8)),I=Hdt((Kt(_-1,s.c.length),h(s.c[_-1],8)),u,(Kt(_+1,s.c.length),h(s.c[_+1],8)),(Kt(_+2,s.c.length),h(s.c[_+2],8))),!isFinite(I.a)||!isFinite(I.b)?Ai(n,u,n.c.b,n.c):Ai(n,I,n.c.b,n.c);for(vr(n,h(ut(s,s.c.length-1),8)),Ue(p,(Kt(0,v.c.length),h(v.c[0],8))),A=1;A<v.c.length-2;A+=2)u=(Kt(A,v.c.length),h(v.c[A],8)),I=Hdt((Kt(A-1,v.c.length),h(v.c[A-1],8)),u,(Kt(A+1,v.c.length),h(v.c[A+1],8)),(Kt(A+2,v.c.length),h(v.c[A+2],8))),!isFinite(I.a)||!isFinite(I.b)?p.c[p.c.length]=u:p.c[p.c.length]=I;for(Ue(p,h(ut(v,v.c.length-1),8)),m=p.c.length-1;m>=0;m--)vr(n,(Kt(m,p.c.length),h(p.c[m],8)));return n}function JNn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P;if(p=!0,A=null,s=null,a=null,t=!1,P=P9t,x=null,u=null,v=0,m=DX(e,v,pEe,bEe),m<e.length&&(er(m,e.length),e.charCodeAt(m)==58)&&(A=e.substr(v,m-v),v=m+1),n=A!=null&&kS(XH,A.toLowerCase()),n){if(m=e.lastIndexOf("!/"),m==-1)throw ne(new an("no archive separator"));p=!0,s=$u(e,v,++m),v=m}else v>=0&&Nt(e.substr(v,2),"//")?(v+=2,m=DX(e,v,lT,hT),s=e.substr(v,m-v),v=m):A!=null&&(v==e.length||(er(v,e.length),e.charCodeAt(v)!=47))&&(p=!1,m=Ufe(e,Ec(35),v),m==-1&&(m=e.length),s=e.substr(v,m-v),v=m);if(!n&&v<e.length&&(er(v,e.length),e.charCodeAt(v)==47)&&(m=DX(e,v+1,lT,hT),_=e.substr(v+1,m-(v+1)),_.length>0&&Is(_,_.length-1)==58&&(a=_,v=m)),v<e.length&&(er(v,e.length),e.charCodeAt(v)==47)&&(++v,t=!0),v<e.length&&(er(v,e.length),e.charCodeAt(v)!=63)&&(er(v,e.length),e.charCodeAt(v)!=35)){for(I=new qe;v<e.length&&(er(v,e.length),e.charCodeAt(v)!=63)&&(er(v,e.length),e.charCodeAt(v)!=35);)m=DX(e,v,lT,hT),Ue(I,e.substr(v,m-v)),v=m,v<e.length&&(er(v,e.length),e.charCodeAt(v)==47)&&(H6n(e,++v)||(I.c[I.c.length]=""));P=pe(it,Re,2,I.c.length,6,1),Mf(I,P)}return v<e.length&&(er(v,e.length),e.charCodeAt(v)==63)&&(m=nk(e,35,++v),m==-1&&(m=e.length),x=e.substr(v,m-v),v=m),v<e.length&&(u=jS(e,++v)),GIn(p,A,s,a,P,x),new OQ(p,A,s,a,t,P,x,u)}function ePn(e,t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts,il,rh;for(ts=new qe,j=new Q(t.b);j.a<j.c.c.length;)for(I=h(K(j),29),Ne=new Q(I.a);Ne.a<Ne.c.c.length;){for(we=h(K(Ne),10),we.p=-1,A=$s,wt=$s,ln=new Q(we.j);ln.a<ln.c.c.length;){for(xt=h(K(ln),11),a=new Q(xt.e);a.a<a.c.c.length;)n=h(K(a),17),dn=h(U(n,(tt(),H5)),19).a,A=d.Math.max(A,dn);for(s=new Q(xt.g);s.a<s.c.c.length;)n=h(K(s),17),dn=h(U(n,(tt(),H5)),19).a,wt=d.Math.max(wt,dn)}Fe(we,cH,Ke(A)),Fe(we,uH,Ke(wt))}for(oe=0,P=new Q(t.b);P.a<P.c.c.length;)for(I=h(K(P),29),Ne=new Q(I.a);Ne.a<Ne.c.c.length;)we=h(K(Ne),10),we.p<0&&(Dr=new Jle,Dr.b=oe++,hpt(e,we,Dr),ts.c[ts.c.length]=Dr);for(rt=I1(ts.c.length),_=I1(ts.c.length),p=0;p<ts.c.length;p++)Ue(rt,new qe),Ue(_,Ke(0));for(EOn(t,ts,rt,_),il=h(Mf(ts,pe(h8t,bwt,257,ts.c.length,0,1)),840),Ve=h(Mf(rt,pe(xl,Hw,15,rt.c.length,0,1)),192),x=pe(Hn,hr,25,_.c.length,15,1),v=0;v<x.length;v++)x[v]=(Kt(v,_.c.length),h(_.c[v],19)).a;for(fe=0,le=new qe,m=0;m<il.length;m++)x[m]==0&&Ue(le,il[m]);for(J=pe(Hn,hr,25,il.length,15,1);le.c.length!=0;)for(Dr=h(f0(le,0),257),J[Dr.b]=fe++;!Ve[Dr.b].dc();)rh=h(Ve[Dr.b].$c(0),257),--x[rh.b],x[rh.b]==0&&(le.c[le.c.length]=rh);for(e.a=pe(h8t,bwt,257,il.length,0,1),u=0;u<il.length;u++)for(V=il[u],Ln=J[u],e.a[Ln]=V,V.b=Ln,Ne=new Q(V.e);Ne.a<Ne.c.c.length;)we=h(K(Ne),10),we.p=Ln;return e.a}function Sr(e){var t,n,s;if(e.d>=e.j){e.a=-1,e.c=1;return}if(t=Is(e.i,e.d++),e.a=t,e.b==1){switch(t){case 92:if(s=10,e.d>=e.j)throw ne(new Zn(ir((Qn(),pj))));e.a=Is(e.i,e.d++);break;case 45:(e.e&512)==512&&e.d<e.j&&Is(e.i,e.d)==91?(++e.d,s=24):s=0;break;case 91:if((e.e&512)!=512&&e.d<e.j&&Is(e.i,e.d)==58){++e.d,s=20;break}default:(t&64512)==jM&&e.d<e.j&&(n=Is(e.i,e.d),(n&64512)==56320&&(e.a=ea+(t-jM<<10)+n-56320,++e.d)),s=0}e.c=s;return}switch(t){case 124:s=2;break;case 42:s=3;break;case 43:s=4;break;case 63:s=5;break;case 41:s=7;break;case 46:s=8;break;case 91:s=9;break;case 94:s=11;break;case 36:s=12;break;case 40:if(s=6,e.d>=e.j||Is(e.i,e.d)!=63)break;if(++e.d>=e.j)throw ne(new Zn(ir((Qn(),Aee))));switch(t=Is(e.i,e.d++),t){case 58:s=13;break;case 61:s=14;break;case 33:s=15;break;case 91:s=19;break;case 62:s=18;break;case 60:if(e.d>=e.j)throw ne(new Zn(ir((Qn(),Aee))));if(t=Is(e.i,e.d++),t==61)s=16;else if(t==33)s=17;else throw ne(new Zn(ir((Qn(),Amt))));break;case 35:for(;e.d<e.j&&(t=Is(e.i,e.d++),t!=41););if(t!=41)throw ne(new Zn(ir((Qn(),Mmt))));s=21;break;default:if(t==45||97<=t&&t<=122||65<=t&&t<=90){--e.d,s=22;break}else if(t==40){s=23;break}throw ne(new Zn(ir((Qn(),Aee))))}break;case 92:if(s=10,e.d>=e.j)throw ne(new Zn(ir((Qn(),pj))));e.a=Is(e.i,e.d++);break;default:s=0}e.c=s}function tPn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln;if(rt=h(U(e,(tt(),Ei)),98),rt!=(_s(),$f)&&rt!=Qg){for(j=e.b,P=j.c.length,_=new Wo((Uu(P+2,pZ),PB(Vs(Vs(5,P+2),(P+2)/10|0)))),V=new Wo((Uu(P+2,pZ),PB(Vs(Vs(5,P+2),(P+2)/10|0)))),Ue(_,new zn),Ue(_,new zn),Ue(V,new qe),Ue(V,new qe),Ve=new qe,t=0;t<P;t++)for(n=(Kt(t,j.c.length),h(j.c[t],29)),wt=(Kt(t,_.c.length),h(_.c[t],83)),J=new zn,_.c[_.c.length]=J,ln=(Kt(t,V.c.length),h(V.c[t],15)),fe=new qe,V.c[V.c.length]=fe,a=new Q(n.a);a.a<a.c.c.length;){if(s=h(K(a),10),vpe(s)){Ve.c[Ve.c.length]=s;continue}for(x=new Sn(Dn(za(s).a.Kc(),new L));rr(x);)v=h(Un(x),17),dn=v.c.i,vpe(dn)&&(xt=h(wt.xc(U(dn,(He(),Br))),10),xt||(xt=c0t(e,dn),wt.zc(U(dn,Br),xt),ln.Fc(xt)),Gs(v,h(ut(xt.j,1),11)));for(m=new Sn(Dn(qi(s).a.Kc(),new L));rr(m);)v=h(Un(m),17),Ln=v.d.i,vpe(Ln)&&(oe=h(_n(J,U(Ln,(He(),Br))),10),oe||(oe=c0t(e,Ln),Pr(J,U(Ln,Br),oe),fe.c[fe.c.length]=oe),xs(v,h(ut(oe.j,0),11)))}for(A=0;A<V.c.length;A++)if(le=(Kt(A,V.c.length),h(V.c[A],15)),!le.dc())for(I=null,A==0?(I=new Nl(e),fw(0,j.c.length),J7(j.c,0,I)):A==_.c.length-1?(I=new Nl(e),j.c[j.c.length]=I):I=(Kt(A-1,j.c.length),h(j.c[A-1],29)),p=le.Kc();p.Ob();)u=h(p.Pb(),10),Ca(u,I);for(Ne=new Q(Ve);Ne.a<Ne.c.c.length;)we=h(K(Ne),10),Ca(we,null);Fe(e,(He(),hne),Ve)}}function nPn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne;if(Bn(n,"Coffman-Graham Layering",1),t.a.c.length==0){An(n);return}for(Ne=h(U(t,(tt(),vke)),19).a,m=0,p=0,I=new Q(t.a);I.a<I.c.c.length;)for(A=h(K(I),10),A.p=m++,u=new Sn(Dn(qi(A).a.Kc(),new L));rr(u);)a=h(Un(u),17),a.p=p++;for(e.d=pe(vu,Y1,25,m,16,1),e.a=pe(vu,Y1,25,p,16,1),e.b=pe(Hn,hr,25,m,15,1),e.e=pe(Hn,hr,25,m,15,1),e.f=pe(Hn,hr,25,m,15,1),VB(e.c),xEn(e,t),j=new RP(new VYe(e)),we=new Q(t.a);we.a<we.c.c.length;){for(fe=h(K(we),10),u=new Sn(Dn(za(fe).a.Kc(),new L));rr(u);)a=h(Un(u),17),e.a[a.p]||++e.b[fe.p];e.b[fe.p]==0&&xk(sx(j,fe))}for(v=0;j.b.c.length!=0;)for(fe=h(MK(j),10),e.f[fe.p]=v++,u=new Sn(Dn(qi(fe).a.Kc(),new L));rr(u);)a=h(Un(u),17),!e.a[a.p]&&(J=a.d.i,--e.b[J.p],Ot(e.c,J,Ke(e.f[fe.p])),e.b[J.p]==0&&xk(sx(j,J)));for(P=new RP(new UYe(e)),le=new Q(t.a);le.a<le.c.c.length;){for(fe=h(K(le),10),u=new Sn(Dn(qi(fe).a.Kc(),new L));rr(u);)a=h(Un(u),17),e.a[a.p]||++e.e[fe.p];e.e[fe.p]==0&&xk(sx(P,fe))}for(_=new qe,s=Est(t,_);P.b.c.length!=0;)for(oe=h(MK(P),10),(s.a.c.length>=Ne||!xxn(oe,s))&&(s=Est(t,_)),Ca(oe,s),u=new Sn(Dn(za(oe).a.Kc(),new L));rr(u);)a=h(Un(u),17),!e.a[a.p]&&(V=a.c.i,--e.e[V.p],e.e[V.p]==0&&xk(sx(P,V)));for(x=_.c.length-1;x>=0;--x)Ue(t.b,(Kt(x,_.c.length),h(_.c[x],29)));t.a.c=pe(yn,st,1,0,5,1),An(n)}function Mbt(e){var t,n,s,a,u,p,v,m,x;for(e.b=1,Sr(e),t=null,e.c==0&&e.a==94?(Sr(e),t=(Mr(),Mr(),new zu(4)),$o(t,0,jx),v=new zu(4)):v=(Mr(),Mr(),new zu(4)),a=!0;(x=e.c)!=1;){if(x==0&&e.a==93&&!a){t&&(I9(t,v),v=t);break}if(n=e.a,s=!1,x==10)switch(n){case 100:case 68:case 119:case 87:case 115:case 83:Fw(v,gx(n)),s=!0;break;case 105:case 73:case 99:case 67:n=(Fw(v,gx(n)),-1),n<0&&(s=!0);break;case 112:case 80:if(m=kbe(e,n),!m)throw ne(new Zn(ir((Qn(),Mee))));Fw(v,m),s=!0;break;default:n=tve(e)}else if(x==24&&!a){if(t&&(I9(t,v),v=t),u=Mbt(e),I9(v,u),e.c!=0||e.a!=93)throw ne(new Zn(ir((Qn(),Rmt))));break}if(Sr(e),!s){if(x==0){if(n==91)throw ne(new Zn(ir((Qn(),Yye))));if(n==93)throw ne(new Zn(ir((Qn(),Wye))));if(n==45&&!a&&e.a!=93)throw ne(new Zn(ir((Qn(),Dee))))}if(e.c!=0||e.a!=45||n==45&&a)$o(v,n,n);else{if(Sr(e),(x=e.c)==1)throw ne(new Zn(ir((Qn(),bj))));if(x==0&&e.a==93)$o(v,n,n),$o(v,45,45);else{if(x==0&&e.a==93||x==24)throw ne(new Zn(ir((Qn(),Dee))));if(p=e.a,x==0){if(p==91)throw ne(new Zn(ir((Qn(),Yye))));if(p==93)throw ne(new Zn(ir((Qn(),Wye))));if(p==45)throw ne(new Zn(ir((Qn(),Dee))))}else x==10&&(p=tve(e));if(Sr(e),n>p)throw ne(new Zn(ir((Qn(),Hmt))));$o(v,n,p)}}}a=!1}if(e.c==1)throw ne(new Zn(ir((Qn(),bj))));return Ny(v),D9(v),e.b=0,Sr(e),v}function rPn(e){Yn(e.c,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#decimal"])),Yn(e.d,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#integer"])),Yn(e.e,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#boolean"])),Yn(e.f,lr,se(ie(it,1),Re,2,6,[Hs,"EBoolean",_r,"EBoolean:Object"])),Yn(e.i,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#byte"])),Yn(e.g,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#hexBinary"])),Yn(e.j,lr,se(ie(it,1),Re,2,6,[Hs,"EByte",_r,"EByte:Object"])),Yn(e.n,lr,se(ie(it,1),Re,2,6,[Hs,"EChar",_r,"EChar:Object"])),Yn(e.t,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#double"])),Yn(e.u,lr,se(ie(it,1),Re,2,6,[Hs,"EDouble",_r,"EDouble:Object"])),Yn(e.F,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#float"])),Yn(e.G,lr,se(ie(it,1),Re,2,6,[Hs,"EFloat",_r,"EFloat:Object"])),Yn(e.I,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#int"])),Yn(e.J,lr,se(ie(it,1),Re,2,6,[Hs,"EInt",_r,"EInt:Object"])),Yn(e.N,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#long"])),Yn(e.O,lr,se(ie(it,1),Re,2,6,[Hs,"ELong",_r,"ELong:Object"])),Yn(e.Z,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#short"])),Yn(e.$,lr,se(ie(it,1),Re,2,6,[Hs,"EShort",_r,"EShort:Object"])),Yn(e._,lr,se(ie(it,1),Re,2,6,[Hs,"http://www.w3.org/2001/XMLSchema#string"]))}function iPn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln;if(e.c.length==1)return Kt(0,e.c.length),h(e.c[0],135);if(e.c.length<=0)return new kB;for(m=new Q(e);m.a<m.c.c.length;){for(p=h(K(m),135),fe=0,j=Ir,V=Ir,I=$s,P=$s,oe=pr(p.b,0);oe.b!=oe.d.c;)J=h(gr(oe),86),fe+=h(U(J,(Fb(),bH)),19).a,j=d.Math.min(j,J.e.a),V=d.Math.min(V,J.e.b),I=d.Math.max(I,J.e.a+J.f.a),P=d.Math.max(P,J.e.b+J.f.b);Fe(p,(Fb(),bH),Ke(fe)),Fe(p,(fo(),G_),new dt(j,V)),Fe(p,zD,new dt(I,P))}for(Rt(),hs(e,new Z$e),Ne=new kB,Oa(Ne,(Kt(0,e.c.length),h(e.c[0],94))),A=0,ln=0,x=new Q(e);x.a<x.c.c.length;)p=h(K(x),135),Ve=ms(ro(h(U(p,(fo(),zD)),8)),h(U(p,G_),8)),A=d.Math.max(A,Ve.a),ln+=Ve.a*Ve.b;for(A=d.Math.max(A,d.Math.sqrt(ln)*Pe(Je(U(Ne,(Fb(),Y8t))))),rt=Pe(Je(U(Ne,Sxe))),dn=0,Ln=0,_=0,t=rt,v=new Q(e);v.a<v.c.c.length;)p=h(K(v),135),Ve=ms(ro(h(U(p,(fo(),zD)),8)),h(U(p,G_),8)),dn+Ve.a>A&&(dn=0,Ln+=_+rt,_=0),gSn(Ne,p,dn,Ln),t=d.Math.max(t,dn+Ve.a),_=d.Math.max(_,Ve.b),dn+=Ve.a+rt;for(we=new zn,n=new zn,xt=new Q(e);xt.a<xt.c.c.length;)for(wt=h(K(xt),135),s=ft(lt(U(wt,(Tr(),ZD)))),le=wt.q?wt.q:K1,u=le.vc().Kc();u.Ob();)a=h(u.Pb(),42),Au(we,a.cd())?Me(h(a.cd(),146).wg())!==Me(a.dd())&&(s&&Au(n,a.cd())?(L1(),""+h(a.cd(),146).tg()):(Pr(we,h(a.cd(),146),a.dd()),Fe(Ne,h(a.cd(),146),a.dd()),s&&Pr(n,h(a.cd(),146),a.dd()))):(Pr(we,h(a.cd(),146),a.dd()),Fe(Ne,h(a.cd(),146),a.dd()));return Ne}function Dbt(){Dbt=re,uZ(),fr=new pb,Ot(fr,(Qe(),Ph),Hf),Ot(fr,Ql,Hf),Ot(fr,eu,Hf),Ot(fr,Bh,Hf),Ot(fr,Sc,Hf),Ot(fr,tu,Hf),Ot(fr,Bh,Ph),Ot(fr,Hf,_l),Ot(fr,Ph,_l),Ot(fr,Ql,_l),Ot(fr,eu,_l),Ot(fr,Cc,_l),Ot(fr,Bh,_l),Ot(fr,Sc,_l),Ot(fr,tu,_l),Ot(fr,uc,_l),Ot(fr,Hf,tl),Ot(fr,Ph,tl),Ot(fr,_l,tl),Ot(fr,Ql,tl),Ot(fr,eu,tl),Ot(fr,Cc,tl),Ot(fr,Bh,tl),Ot(fr,uc,tl),Ot(fr,nl,tl),Ot(fr,Sc,tl),Ot(fr,Gc,tl),Ot(fr,tu,tl),Ot(fr,Ph,Ql),Ot(fr,eu,Ql),Ot(fr,Bh,Ql),Ot(fr,tu,Ql),Ot(fr,Ph,eu),Ot(fr,Ql,eu),Ot(fr,Bh,eu),Ot(fr,eu,eu),Ot(fr,Sc,eu),Ot(fr,Hf,Tl),Ot(fr,Ph,Tl),Ot(fr,_l,Tl),Ot(fr,tl,Tl),Ot(fr,Ql,Tl),Ot(fr,eu,Tl),Ot(fr,Cc,Tl),Ot(fr,Bh,Tl),Ot(fr,nl,Tl),Ot(fr,uc,Tl),Ot(fr,tu,Tl),Ot(fr,Sc,Tl),Ot(fr,Yo,Tl),Ot(fr,Hf,nl),Ot(fr,Ph,nl),Ot(fr,_l,nl),Ot(fr,Ql,nl),Ot(fr,eu,nl),Ot(fr,Cc,nl),Ot(fr,Bh,nl),Ot(fr,uc,nl),Ot(fr,tu,nl),Ot(fr,Gc,nl),Ot(fr,Yo,nl),Ot(fr,Ph,uc),Ot(fr,Ql,uc),Ot(fr,eu,uc),Ot(fr,Bh,uc),Ot(fr,nl,uc),Ot(fr,tu,uc),Ot(fr,Sc,uc),Ot(fr,Hf,Tc),Ot(fr,Ph,Tc),Ot(fr,_l,Tc),Ot(fr,Ql,Tc),Ot(fr,eu,Tc),Ot(fr,Cc,Tc),Ot(fr,Bh,Tc),Ot(fr,uc,Tc),Ot(fr,tu,Tc),Ot(fr,Ph,Sc),Ot(fr,_l,Sc),Ot(fr,tl,Sc),Ot(fr,eu,Sc),Ot(fr,Hf,Gc),Ot(fr,Ph,Gc),Ot(fr,tl,Gc),Ot(fr,Ql,Gc),Ot(fr,eu,Gc),Ot(fr,Cc,Gc),Ot(fr,Bh,Gc),Ot(fr,Bh,Yo),Ot(fr,eu,Yo),Ot(fr,uc,Hf),Ot(fr,uc,Ql),Ot(fr,uc,_l),Ot(fr,Cc,Hf),Ot(fr,Cc,Ph),Ot(fr,Cc,tl)}function kR(e,t){switch(e.e){case 0:case 2:case 4:case 6:case 42:case 44:case 46:case 48:case 8:case 10:case 12:case 14:case 16:case 18:case 20:case 22:case 24:case 26:case 28:case 30:case 32:case 34:case 36:case 38:return new Lit(e.b,e.a,t,e.c);case 1:return new iP(e.a,t,oi(t.Tg(),e.c));case 43:return new Cet(e.a,t,oi(t.Tg(),e.c));case 3:return new Gi(e.a,t,oi(t.Tg(),e.c));case 45:return new Nc(e.a,t,oi(t.Tg(),e.c));case 41:return new Du(h(Rl(e.c),26),e.a,t,oi(t.Tg(),e.c));case 50:return new Dut(h(Rl(e.c),26),e.a,t,oi(t.Tg(),e.c));case 5:return new E1e(e.a,t,oi(t.Tg(),e.c),e.d.n);case 47:return new Rtt(e.a,t,oi(t.Tg(),e.c),e.d.n);case 7:return new Ye(e.a,t,oi(t.Tg(),e.c),e.d.n);case 49:return new B4(e.a,t,oi(t.Tg(),e.c),e.d.n);case 9:return new Met(e.a,t,oi(t.Tg(),e.c));case 11:return new Aet(e.a,t,oi(t.Tg(),e.c));case 13:return new Pfe(e.a,t,oi(t.Tg(),e.c));case 15:return new yP(e.a,t,oi(t.Tg(),e.c));case 17:return new Det(e.a,t,oi(t.Tg(),e.c));case 19:return new ly(e.a,t,oi(t.Tg(),e.c));case 21:return new Nfe(e.a,t,oi(t.Tg(),e.c));case 23:return new JS(e.a,t,oi(t.Tg(),e.c));case 25:return new Htt(e.a,t,oi(t.Tg(),e.c),e.d.n);case 27:return new qt(e.a,t,oi(t.Tg(),e.c),e.d.n);case 29:return new $tt(e.a,t,oi(t.Tg(),e.c),e.d.n);case 31:return new jtt(e.a,t,oi(t.Tg(),e.c),e.d.n);case 33:return new T1e(e.a,t,oi(t.Tg(),e.c),e.d.n);case 35:return new _1e(e.a,t,oi(t.Tg(),e.c),e.d.n);case 37:return new VY(e.a,t,oi(t.Tg(),e.c),e.d.n);case 39:return new jP(e.a,t,oi(t.Tg(),e.c),e.d.n);case 40:return new yi(t,oi(t.Tg(),e.c));default:throw ne(new Ya("Unknown feature style: "+e.e))}}function sPn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve;switch(Bn(n,"Brandes & Koepf node placement",1),e.a=t,e.c=ZDn(t),s=h(U(t,(tt(),Mne)),274),P=ft(lt(U(t,D_))),e.d=s==(cM(),D$)&&!P||s==sne,FIn(e,t),Ne=null,Ve=null,oe=null,fe=null,J=(Uu(4,jw),new Wo(4)),h(U(t,Mne),274).g){case 3:oe=new jy(t,e.c.d,(yf(),v2),(r1(),I0)),J.c[J.c.length]=oe;break;case 1:fe=new jy(t,e.c.d,(yf(),Rf),(r1(),I0)),J.c[J.c.length]=fe;break;case 4:Ne=new jy(t,e.c.d,(yf(),v2),(r1(),lv)),J.c[J.c.length]=Ne;break;case 2:Ve=new jy(t,e.c.d,(yf(),Rf),(r1(),lv)),J.c[J.c.length]=Ve;break;default:oe=new jy(t,e.c.d,(yf(),v2),(r1(),I0)),fe=new jy(t,e.c.d,Rf,I0),Ne=new jy(t,e.c.d,v2,lv),Ve=new jy(t,e.c.d,Rf,lv),J.c[J.c.length]=Ne,J.c[J.c.length]=Ve,J.c[J.c.length]=oe,J.c[J.c.length]=fe}for(a=new VZe(t,e.c),v=new Q(J);v.a<v.c.c.length;)u=h(K(v),180),UNn(a,u,e.b),ALn(u);for(I=new T1t(t,e.c),m=new Q(J);m.a<m.c.c.length;)u=h(K(m),180),HOn(I,u);if(n.n)for(x=new Q(J);x.a<x.c.c.length;)u=h(K(x),180),Sp(n,u+" size is "+TQ(u));if(A=null,e.d&&(_=vNn(e,J,e.c.d),h2t(t,_,n)&&(A=_)),!A)for(x=new Q(J);x.a<x.c.c.length;)u=h(K(x),180),h2t(t,u,n)&&(!A||TQ(A)>TQ(u))&&(A=u);for(!A&&(A=(Kt(0,J.c.length),h(J.c[0],180))),V=new Q(t.b);V.a<V.c.c.length;)for(j=h(K(V),29),we=new Q(j.a);we.a<we.c.c.length;)le=h(K(we),10),le.n.b=Pe(A.p[le.p])+Pe(A.d[le.p]);for(n.n&&(Sp(n,"Chosen node placement: "+A),Sp(n,"Blocks: "+L0t(A)),Sp(n,"Classes: "+mCn(A,n)),Sp(n,"Marked edges: "+e.b)),p=new Q(J);p.a<p.c.c.length;)u=h(K(p),180),u.g=null,u.b=null,u.a=null,u.d=null,u.j=null,u.i=null,u.p=null;X4n(e.c),e.b.a.$b(),An(n)}function aPn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn;for(p=new di,Ne=h(U(n,(tt(),Ju)),103),j=0,Qs(p,(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));p.b!=0;)x=h(p.b==0?null:(kn(p.b!=0),pl(p,p.a.a)),33),(Me(pt(t,Vg))!==Me((Dd(),Ug))||Me(pt(t,im))===Me((Kp(),k_))||Me(pt(t,im))===Me((Kp(),y_))||ft(lt(pt(t,rm)))||Me(pt(t,$5))!==Me((Sb(),i3)))&&!ft(lt(pt(x,yne)))&&ya(x,(He(),Eo),Ke(j++)),J=!ft(lt(pt(x,p2))),J&&(A=(!x.a&&(x.a=new Ye(wi,x,10,11)),x.a).i!=0,P=u9n(x),I=Me(pt(x,sm))===Me((Ld(),B0)),dn=!Pp(x,(Tr(),Q_))||Nt(Jn(pt(x,Q_)),wn),le=null,dn&&I&&(A||P)&&(le=Zpt(x),Fe(le,Ju,Ne),os(le,OD)&&SXe(new Mpe(Pe(Je(U(le,OD)))),le),h(pt(x,g2),174).gc()!=0&&(_=le,Ti(new Vt(null,(!x.c&&(x.c=new Ye(bu,x,9,9)),new Wt(x.c,16))),new nYe(_)),Vgt(x,le))),Ve=n,rt=h(_n(e.a,bi(x)),10),rt&&(Ve=rt.e),fe=Q2t(e,x,Ve),le&&(fe.e=le,le.e=fe,Qs(p,(!x.a&&(x.a=new Ye(wi,x,10,11)),x.a))));for(j=0,Ai(p,t,p.c.b,p.c);p.b!=0;){for(u=h(p.b==0?null:(kn(p.b!=0),pl(p,p.a.a)),33),m=new Tn((!u.b&&(u.b=new Ye(us,u,12,3)),u.b));m.e!=m.i.gc();)v=h(On(m),79),bpt(v),(Me(pt(t,Vg))!==Me((Dd(),Ug))||Me(pt(t,im))===Me((Kp(),k_))||Me(pt(t,im))===Me((Kp(),y_))||ft(lt(pt(t,rm)))||Me(pt(t,$5))!==Me((Sb(),i3)))&&ya(v,(He(),Eo),Ke(j++)),xt=Na(h(he((!v.b&&(v.b=new qt(Pn,v,4,7)),v.b),0),82)),ln=Na(h(he((!v.c&&(v.c=new qt(Pn,v,5,8)),v.c),0),82)),!(ft(lt(pt(v,p2)))||ft(lt(pt(xt,p2)))||ft(lt(pt(ln,p2))))&&(V=Pb(v)&&ft(lt(pt(xt,am)))&&ft(lt(pt(v,d2))),we=u,V||ww(ln,xt)?we=xt:ww(xt,ln)&&(we=ln),Ve=n,rt=h(_n(e.a,we),10),rt&&(Ve=rt.e),oe=qve(e,v,we,Ve),Fe(oe,(He(),p6e),DAn(e,v,t,n)));if(I=Me(pt(u,sm))===Me((Ld(),B0)),I)for(a=new Tn((!u.a&&(u.a=new Ye(wi,u,10,11)),u.a));a.e!=a.i.gc();)s=h(On(a),33),dn=!Pp(s,(Tr(),Q_))||Nt(Jn(pt(s,Q_)),wn),wt=Me(pt(s,sm))===Me(B0),dn&&wt&&Ai(p,s,p.c.b,p.c)}}function oPn(e,t,n,s,a,u){var p,v,m,x,_,A,I,P,j,V,J,oe;switch(t){case 71:v=s.q.getFullYear()-Og>=-1900?1:0,n>=4?or(e,se(ie(it,1),Re,2,6,[qbt,Ybt])[v]):or(e,se(ie(it,1),Re,2,6,["BC","AD"])[v]);break;case 121:Zxn(e,n,s);break;case 77:nDn(e,n,s);break;case 107:m=a.q.getHours(),m==0?j1(e,24,n):j1(e,m,n);break;case 83:ESn(e,n,a);break;case 69:_=s.q.getDay(),n==5?or(e,se(ie(it,1),Re,2,6,["S","M","T","W","T","F","S"])[_]):n==4?or(e,se(ie(it,1),Re,2,6,[MZ,DZ,LZ,IZ,OZ,NZ,PZ])[_]):or(e,se(ie(it,1),Re,2,6,["Sun","Mon","Tue","Wed","Thu","Fri","Sat"])[_]);break;case 97:a.q.getHours()>=12&&a.q.getHours()<24?or(e,se(ie(it,1),Re,2,6,["AM","PM"])[1]):or(e,se(ie(it,1),Re,2,6,["AM","PM"])[0]);break;case 104:A=a.q.getHours()%12,A==0?j1(e,12,n):j1(e,A,n);break;case 75:I=a.q.getHours()%12,j1(e,I,n);break;case 72:P=a.q.getHours(),j1(e,P,n);break;case 99:j=s.q.getDay(),n==5?or(e,se(ie(it,1),Re,2,6,["S","M","T","W","T","F","S"])[j]):n==4?or(e,se(ie(it,1),Re,2,6,[MZ,DZ,LZ,IZ,OZ,NZ,PZ])[j]):n==3?or(e,se(ie(it,1),Re,2,6,["Sun","Mon","Tue","Wed","Thu","Fri","Sat"])[j]):j1(e,j,1);break;case 76:V=s.q.getMonth(),n==5?or(e,se(ie(it,1),Re,2,6,["J","F","M","A","M","J","J","A","S","O","N","D"])[V]):n==4?or(e,se(ie(it,1),Re,2,6,[wZ,mZ,yZ,kZ,p5,xZ,EZ,_Z,TZ,CZ,SZ,AZ])[V]):n==3?or(e,se(ie(it,1),Re,2,6,["Jan","Feb","Mar","Apr",p5,"Jun","Jul","Aug","Sep","Oct","Nov","Dec"])[V]):j1(e,V+1,n);break;case 81:J=s.q.getMonth()/3|0,n<4?or(e,se(ie(it,1),Re,2,6,["Q1","Q2","Q3","Q4"])[J]):or(e,se(ie(it,1),Re,2,6,["1st quarter","2nd quarter","3rd quarter","4th quarter"])[J]);break;case 100:oe=s.q.getDate(),j1(e,oe,n);break;case 109:x=a.q.getMinutes(),j1(e,x,n);break;case 115:p=a.q.getSeconds(),j1(e,p,n);break;case 122:n<4?or(e,u.c[0]):or(e,u.c[1]);break;case 118:or(e,u.b);break;case 90:n<3?or(e,fTn(u)):n==3?or(e,pTn(u)):or(e,bTn(u.a));break;default:return!1}return!0}function qve(e,t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr;if(bpt(t),m=h(he((!t.b&&(t.b=new qt(Pn,t,4,7)),t.b),0),82),_=h(he((!t.c&&(t.c=new qt(Pn,t,5,8)),t.c),0),82),v=Na(m),x=Na(_),p=(!t.a&&(t.a=new Ye(gi,t,6,6)),t.a).i==0?null:h(he((!t.a&&(t.a=new Ye(gi,t,6,6)),t.a),0),202),rt=h(_n(e.a,v),10),dn=h(_n(e.a,x),10),wt=null,Ln=null,ue(m,186)&&(Ve=h(_n(e.a,m),299),ue(Ve,11)?wt=h(Ve,11):ue(Ve,10)&&(rt=h(Ve,10),wt=h(ut(rt.j,0),11))),ue(_,186)&&(ln=h(_n(e.a,_),299),ue(ln,11)?Ln=h(ln,11):ue(ln,10)&&(dn=h(ln,10),Ln=h(ut(dn.j,0),11))),!rt||!dn)throw ne(new z7("The source or the target of edge "+t+" could not be found. This usually happens when an edge connects a node laid out by ELK Layered to a node in another level of hierarchy laid out by either another instance of ELK Layered or another layout algorithm alltogether. The former can be solved by setting the hierarchyHandling option to INCLUDE_CHILDREN."));for(V=new db,Oa(V,t),Fe(V,(He(),Br),t),Fe(V,(tt(),Da),null),P=h(U(s,Uo),21),rt==dn&&P.Fc((ha(),__)),wt||(Ne=(ua(),ec),xt=null,!!p&&oy(h(U(rt,Ei),98))&&(xt=new dt(p.j,p.k),Cat(xt,uA(t)),not(xt,n),ww(x,v)&&(Ne=Jc,zr(xt,rt.n))),wt=g2t(rt,xt,Ne,s)),Ln||(Ne=(ua(),Jc),Dr=null,!!p&&oy(h(U(dn,Ei),98))&&(Dr=new dt(p.b,p.c),Cat(Dr,uA(t)),not(Dr,n)),Ln=g2t(dn,Dr,Ne,Us(dn))),Gs(V,wt),xs(V,Ln),(wt.e.c.length>1||wt.g.c.length>1||Ln.e.c.length>1||Ln.g.c.length>1)&&P.Fc((ha(),E_)),I=new Tn((!t.n&&(t.n=new Ye(Ua,t,1,7)),t.n));I.e!=I.i.gc();)if(A=h(On(I),137),!ft(lt(pt(A,p2)))&&!!A.a)switch(J=hX(A),Ue(V.b,J),h(U(J,y1),272).g){case 1:case 2:P.Fc((ha(),r8));break;case 0:P.Fc((ha(),n8)),Fe(J,y1,(Tf(),w8))}if(u=h(U(s,M_),314),oe=h(U(s,X$),315),a=u==(e5(),_D)||oe==(f9(),zne),!!p&&(!p.a&&(p.a=new Gi(Zl,p,5)),p.a).i!=0&&a){for(fe=hM(p),j=new Oc,we=pr(fe,0);we.b!=we.d.c;)le=h(gr(we),8),vr(j,new _a(le));Fe(V,m6e,j)}return V}function cPn(e){e.gb||(e.gb=!0,e.b=so(e,0),vi(e.b,18),Qr(e.b,19),e.a=so(e,1),vi(e.a,1),Qr(e.a,2),Qr(e.a,3),Qr(e.a,4),Qr(e.a,5),e.o=so(e,2),vi(e.o,8),vi(e.o,9),Qr(e.o,10),Qr(e.o,11),Qr(e.o,12),Qr(e.o,13),Qr(e.o,14),Qr(e.o,15),Qr(e.o,16),Qr(e.o,17),Qr(e.o,18),Qr(e.o,19),Qr(e.o,20),Qr(e.o,21),Qr(e.o,22),Qr(e.o,23),Aa(e.o),Aa(e.o),Aa(e.o),Aa(e.o),Aa(e.o),Aa(e.o),Aa(e.o),Aa(e.o),Aa(e.o),Aa(e.o),e.p=so(e,3),vi(e.p,2),vi(e.p,3),vi(e.p,4),vi(e.p,5),Qr(e.p,6),Qr(e.p,7),Aa(e.p),Aa(e.p),e.q=so(e,4),vi(e.q,8),e.v=so(e,5),Qr(e.v,9),Aa(e.v),Aa(e.v),Aa(e.v),e.w=so(e,6),vi(e.w,2),vi(e.w,3),vi(e.w,4),Qr(e.w,5),e.B=so(e,7),Qr(e.B,1),Aa(e.B),Aa(e.B),Aa(e.B),e.Q=so(e,8),Qr(e.Q,0),Aa(e.Q),e.R=so(e,9),vi(e.R,1),e.S=so(e,10),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),Aa(e.S),e.T=so(e,11),Qr(e.T,10),Qr(e.T,11),Qr(e.T,12),Qr(e.T,13),Qr(e.T,14),Aa(e.T),Aa(e.T),e.U=so(e,12),vi(e.U,2),vi(e.U,3),Qr(e.U,4),Qr(e.U,5),Qr(e.U,6),Qr(e.U,7),Aa(e.U),e.V=so(e,13),Qr(e.V,10),e.W=so(e,14),vi(e.W,18),vi(e.W,19),vi(e.W,20),Qr(e.W,21),Qr(e.W,22),Qr(e.W,23),e.bb=so(e,15),vi(e.bb,10),vi(e.bb,11),vi(e.bb,12),vi(e.bb,13),vi(e.bb,14),vi(e.bb,15),vi(e.bb,16),Qr(e.bb,17),Aa(e.bb),Aa(e.bb),e.eb=so(e,16),vi(e.eb,2),vi(e.eb,3),vi(e.eb,4),vi(e.eb,5),vi(e.eb,6),vi(e.eb,7),Qr(e.eb,8),Qr(e.eb,9),e.ab=so(e,17),vi(e.ab,0),vi(e.ab,1),e.H=so(e,18),Qr(e.H,0),Qr(e.H,1),Qr(e.H,2),Qr(e.H,3),Qr(e.H,4),Qr(e.H,5),Aa(e.H),e.db=so(e,19),Qr(e.db,2),e.c=kr(e,20),e.d=kr(e,21),e.e=kr(e,22),e.f=kr(e,23),e.i=kr(e,24),e.g=kr(e,25),e.j=kr(e,26),e.k=kr(e,27),e.n=kr(e,28),e.r=kr(e,29),e.s=kr(e,30),e.t=kr(e,31),e.u=kr(e,32),e.fb=kr(e,33),e.A=kr(e,34),e.C=kr(e,35),e.D=kr(e,36),e.F=kr(e,37),e.G=kr(e,38),e.I=kr(e,39),e.J=kr(e,40),e.L=kr(e,41),e.M=kr(e,42),e.N=kr(e,43),e.O=kr(e,44),e.P=kr(e,45),e.X=kr(e,46),e.Y=kr(e,47),e.Z=kr(e,48),e.$=kr(e,49),e._=kr(e,50),e.cb=kr(e,51),e.K=kr(e,52))}function Tr(){Tr=re;var e,t;Q_=new Xr(Pwt),v8=new Xr(Bwt),K8e=($1(),Dre),NEt=new $t(Nme,K8e),new dS,U5=new $t(k5,null),PEt=new Xr(Aye),Q8e=(Aw(),ni(Ore,se(ie(Nre,1),ze,291,0,[Ire]))),DH=new $t(tj,Q8e),ZD=new $t(eD,(on(),!1)),Z8e=(la(),Q1),hv=new $t(Fme,Z8e),t7e=(Od(),qre),e7e=new $t(ZM,t7e),i7e=new $t(oj,!1),s7e=(Ld(),BH),k3=new $t(ej,s7e),p7e=new nb(12),y2=new $t(Vb,p7e),LH=new $t(qM,!1),a7e=new $t(YJ,!1),tL=new $t(z9,!1),y7e=(_s(),Qg),Z_=new $t(fJ,y7e),q5=new Xr(nj),NH=new Xr(UM),Vre=new Xr(BR),Ure=new Xr(H9),o7e=new Oc,x3=new $t(qme,o7e),FEt=new $t(Kme,!1),REt=new $t(Xme,!1),new Xr(Fwt),c7e=new B7,eL=new $t(Zme,c7e),OH=new $t(Ime,!1),new dS,zEt=new $t(Rwt,1),new $t(jwt,!0),Ke(0),new $t($wt,Ke(100)),new $t(Hwt,!1),Ke(0),new $t(zwt,Ke(4e3)),Ke(0),new $t(Gwt,Ke(400)),new $t(Vwt,!1),new $t(Uwt,!1),new $t(qwt,!0),new $t(Ywt,!1),X8e=(kF(),Xre),BEt=new $t(Sye,X8e),GEt=new $t(yme,10),VEt=new $t(kme,10),_7e=new $t(aJ,20),UEt=new $t(xme,10),T7e=new $t(hJ,2),qEt=new $t(Eme,10),C7e=new $t(_me,0),PH=new $t(Sme,5),S7e=new $t(Tme,1),A7e=new $t(Cme,1),k2=new $t(zw,20),YEt=new $t(Ame,10),L7e=new $t(Mme,10),Y5=new Xr(Dme),D7e=new XJe,M7e=new $t(Jme,D7e),$Et=new Xr(qJ),b7e=!1,jEt=new $t(UJ,b7e),l7e=new nb(5),u7e=new $t(Rme,l7e),h7e=(Iw(),t=h(Vh(ga),9),new ll(t,h(vh(t,t.length),9),0)),E3=new $t(Dx,h7e),w7e=(Cy(),Xg),v7e=new $t(Hme,w7e),jre=new Xr(zme),$re=new Xr(Gme),Hre=new Xr(Vme),Rre=new Xr(Ume),f7e=(e=h(Vh(aT),9),new ll(e,h(vh(e,e.length),9),0)),m2=new $t(Yy,f7e),g7e=Lt((fu(),x8)),Wg=new $t(x5,g7e),d7e=new dt(0,0),_3=new $t(E5,d7e),IH=new $t(VJ,!1),J8e=(Tf(),w8),Bre=new $t(Yme,J8e),Pre=new $t(FR,!1),new Xr(Wwt),Ke(1),new $t(Kwt,null),m7e=new Xr(Qme),zre=new Xr(Wme),E7e=(Qe(),to),T3=new $t(Ome,E7e),pu=new Xr(Lme),k7e=(Qc(),Lt(Zg)),gm=new $t(Lx,k7e),Gre=new $t(jme,!1),x7e=new $t($me,!0),JD=new $t(Pme,!1),Fre=new $t(Bme,!1),n7e=new $t(oJ,1),r7e=(YF(),Wre),new $t(Xwt,r7e),HEt=!0}function He(){He=re;var e,t;Br=new Xr(Mwe),p6e=new Xr("coordinateOrigin"),bne=new Xr("processors"),g6e=new Xi("compoundNode",(on(),!1)),AD=new Xi("insideConnections",!1),m6e=new Xr("originalBendpoints"),y6e=new Xr("originalDummyNodePosition"),k6e=new Xr("originalLabelEdge"),DD=new Xr("representedLabels"),T_=new Xr("endLabels"),P5=new Xr("endLabel.origin"),F5=new Xi("labelSide",(Yu(),iL)),h3=new Xi("maxEdgeThickness",0),Ff=new Xi("reversed",!1),R5=new Xr(Dwe),rf=new Xi("longEdgeSource",null),Yl=new Xi("longEdgeTarget",null),tm=new Xi("longEdgeHasLabelDummies",!1),MD=new Xi("longEdgeBeforeLabelDummy",!1),B$=new Xi("edgeConstraint",(Hp(),Jte)),rv=new Xr("inLayerLayoutUnit"),h2=new Xi("inLayerConstraint",(Ad(),CD)),B5=new Xi("inLayerSuccessorConstraint",new qe),w6e=new Xi("inLayerSuccessorConstraintBetweenNonDummies",!1),Zc=new Xr("portDummy"),P$=new Xi("crossingHint",Ke(0)),Uo=new Xi("graphProperties",(t=h(Vh(ane),9),new ll(t,h(vh(t,t.length),9),0))),co=new Xi("externalPortSide",(Qe(),to)),v6e=new Xi("externalPortSize",new Rs),hne=new Xr("externalPortReplacedDummies"),F$=new Xr("externalPortReplacedDummy"),em=new Xi("externalPortConnections",(e=h(Vh(ta),9),new ll(e,h(vh(e,e.length),9),0))),iv=new Xi(Evt,0),d6e=new Xr("barycenterAssociates"),j5=new Xr("TopSideComments"),N5=new Xr("BottomSideComments"),N$=new Xr("CommentConnectionPort"),dne=new Xi("inputCollect",!1),pne=new Xi("outputCollect",!1),SD=new Xi("cyclic",!1),b6e=new Xr("crossHierarchyMap"),wne=new Xr("targetOffset"),new Xi("splineLabelSize",new Rs),d3=new Xr("spacings"),R$=new Xi("partitionConstraint",!1),tv=new Xr("breakingPoint.info"),_6e=new Xr("splines.survivingEdge"),f2=new Xr("splines.route.start"),g3=new Xr("splines.edgeChain"),E6e=new Xr("originalPortConstraints"),s8=new Xr("selfLoopHolder"),a8=new Xr("splines.nsPortY"),Eo=new Xr("modelOrder"),gne=new Xr("longEdgeTargetNode"),nv=new Xi(rwt,!1),f3=new Xi(rwt,!1),fne=new Xr("layerConstraints.hiddenNodes"),x6e=new Xr("layerConstraints.opposidePort"),vne=new Xr("targetNode.modelOrder")}function Yve(){Yve=re,P6e=(DA(),S$),z6t=new $t(Rwe,P6e),ekt=new $t(jwe,(on(),!1)),H6e=(gB(),lne),skt=new $t(HR,H6e),ykt=new $t($we,!1),kkt=new $t(Hwe,!0),v6t=new $t(zwe,!1),K6e=(CA(),Une),Bkt=new $t(Gwe,K6e),Ke(1),Vkt=new $t(Vwe,Ke(7)),Ukt=new $t(Uwe,!1),tkt=new $t(qwe,!1),N6e=(Kp(),Qte),H6t=new $t(wJ,N6e),V6e=(UF(),Rne),mkt=new $t(XM,V6e),z6e=(wl(),LD),hkt=new $t(Ywe,z6e),Ke(-1),lkt=new $t(Wwe,Ke(-1)),Ke(-1),fkt=new $t(Kwe,Ke(-1)),Ke(-1),dkt=new $t(mJ,Ke(4)),Ke(-1),pkt=new $t(yJ,Ke(2)),G6e=(By(),iH),wkt=new $t(kJ,G6e),Ke(0),vkt=new $t(xJ,Ke(0)),ckt=new $t(EJ,Ke(Ir)),O6e=(e5(),I5),$6t=new $t(q9,O6e),S6t=new $t(Xwe,!1),N6t=new $t(_J,.1),R6t=new $t(TJ,!1),Ke(-1),B6t=new $t(Qwe,Ke(-1)),Ke(-1),F6t=new $t(Zwe,Ke(-1)),Ke(0),A6t=new $t(Jwe,Ke(40)),I6e=(jk(),cne),I6t=new $t(CJ,I6e),L6e=TD,M6t=new $t(zR,L6e),W6e=(f9(),N_),Pkt=new $t(Wy,W6e),Ckt=new Xr(GR),U6e=(xA(),M$),xkt=new $t(SJ,U6e),q6e=(cM(),D$),_kt=new $t(AJ,q6e),new dS,Mkt=new $t(MJ,.3),Lkt=new Xr(DJ),Y6e=(Tw(),rH),Ikt=new $t(LJ,Y6e),R6e=(nF(),Yne),Y6t=new $t(eme,R6e),j6e=(bA(),Wne),W6t=new $t(tme,j6e),$6e=(qk(),F_),K6t=new $t(VR,$6e),Q6t=new $t(UR,.2),U6t=new $t(IJ,2),$kt=new $t(nme,null),zkt=new $t(rme,10),Hkt=new $t(ime,10),Gkt=new $t(sme,20),Ke(0),Fkt=new $t(ame,Ke(0)),Ke(0),Rkt=new $t(ome,Ke(0)),Ke(0),jkt=new $t(cme,Ke(0)),w6t=new $t(OJ,!1),S6e=(ax(),x_),y6t=new $t(ume,S6e),C6e=(TB(),Kte),m6t=new $t(lme,C6e),rkt=new $t(qR,!1),Ke(0),nkt=new $t(NJ,Ke(16)),Ke(0),ikt=new $t(PJ,Ke(5)),Z6e=(oF(),Qne),hxt=new $t($d,Z6e),qkt=new $t(YR,10),Kkt=new $t(WR,1),Q6e=(GB(),C$),nxt=new $t(Y9,Q6e),Zkt=new Xr(BJ),X6e=Ke(1),Ke(0),ext=new $t(FJ,X6e),J6e=(eF(),Xne),pxt=new $t(KR,J6e),fxt=new Xr(XR),oxt=new $t(QR,!0),sxt=new $t(ZR,2),uxt=new $t(RJ,!0),F6e=(KF(),A$),V6t=new $t(hme,F6e),B6e=(u5(),Jx),G6t=new $t(fme,B6e),D6e=(Dd(),Ug),C6t=new $t(JR,D6e),T6t=new $t(dme,!1),A6e=(Sb(),i3),k6t=new $t(jJ,A6e),M6e=(t9(),jne),_6t=new $t(gme,M6e),x6t=new $t($J,0),E6t=new $t(HJ,0),okt=Zte,akt=_D,gkt=tH,bkt=tH,ukt=Fne,P6t=(Ld(),B0),j6t=I5,O6t=I5,D6t=I5,L6t=B0,Skt=P_,Akt=N_,Ekt=N_,Tkt=N_,Dkt=Gne,Nkt=P_,Okt=P_,X6t=(Od(),W5),Z6t=W5,J6t=F_,q6t=nL,Ykt=d8,Wkt=hm,Xkt=d8,Qkt=hm,rxt=d8,ixt=hm,Jkt=Xte,txt=C$,bxt=d8,vxt=hm,dxt=d8,gxt=hm,cxt=hm,axt=hm,lxt=hm}function oa(){oa=re,f5e=new Ii("DIRECTION_PREPROCESSOR",0),u5e=new Ii("COMMENT_PREPROCESSOR",1),v_=new Ii("EDGE_AND_LAYER_CONSTRAINT_EDGE_REVERSER",2),Pte=new Ii("INTERACTIVE_EXTERNAL_PORT_POSITIONER",3),L5e=new Ii("PARTITION_PREPROCESSOR",4),o$=new Ii("LABEL_DUMMY_INSERTER",5),v$=new Ii("SELF_LOOP_PREPROCESSOR",6),Kx=new Ii("LAYER_CONSTRAINT_PREPROCESSOR",7),M5e=new Ii("PARTITION_MIDPROCESSOR",8),y5e=new Ii("HIGH_DEGREE_NODE_LAYER_PROCESSOR",9),S5e=new Ii("NODE_PROMOTION",10),Wx=new Ii("LAYER_CONSTRAINT_POSTPROCESSOR",11),D5e=new Ii("PARTITION_POSTPROCESSOR",12),v5e=new Ii("HIERARCHICAL_PORT_CONSTRAINT_PROCESSOR",13),I5e=new Ii("SEMI_INTERACTIVE_CROSSMIN_PROCESSOR",14),r5e=new Ii("BREAKING_POINT_INSERTER",15),h$=new Ii("LONG_EDGE_SPLITTER",16),Bte=new Ii("PORT_SIDE_PROCESSOR",17),s$=new Ii("INVERTED_PORT_PROCESSOR",18),g$=new Ii("PORT_LIST_SORTER",19),N5e=new Ii("SORT_BY_INPUT_ORDER_OF_MODEL",20),d$=new Ii("NORTH_SOUTH_PORT_PREPROCESSOR",21),i5e=new Ii("BREAKING_POINT_PROCESSOR",22),A5e=new Ii(Kvt,23),P5e=new Ii(Xvt,24),p$=new Ii("SELF_LOOP_PORT_RESTORER",25),O5e=new Ii("SINGLE_EDGE_GRAPH_WRAPPER",26),a$=new Ii("IN_LAYER_CONSTRAINT_PROCESSOR",27),g5e=new Ii("END_NODE_PORT_LABEL_MANAGEMENT_PROCESSOR",28),T5e=new Ii("LABEL_AND_NODE_SIZE_PROCESSOR",29),_5e=new Ii("INNERMOST_NODE_MARGIN_CALCULATOR",30),w$=new Ii("SELF_LOOP_ROUTER",31),o5e=new Ii("COMMENT_NODE_MARGIN_CALCULATOR",32),i$=new Ii("END_LABEL_PREPROCESSOR",33),u$=new Ii("LABEL_DUMMY_SWITCHER",34),a5e=new Ii("CENTER_LABEL_MANAGEMENT_PROCESSOR",35),Yx=new Ii("LABEL_SIDE_SELECTOR",36),x5e=new Ii("HYPEREDGE_DUMMY_MERGER",37),w5e=new Ii("HIERARCHICAL_PORT_DUMMY_SIZE_PROCESSOR",38),C5e=new Ii("LAYER_SIZE_AND_GRAPH_HEIGHT_CALCULATOR",39),w_=new Ii("HIERARCHICAL_PORT_POSITION_PROCESSOR",40),l5e=new Ii("CONSTRAINTS_POSTPROCESSOR",41),c5e=new Ii("COMMENT_POSTPROCESSOR",42),E5e=new Ii("HYPERNODE_PROCESSOR",43),m5e=new Ii("HIERARCHICAL_PORT_ORTHOGONAL_EDGE_ROUTER",44),l$=new Ii("LONG_EDGE_JOINER",45),b$=new Ii("SELF_LOOP_POSTPROCESSOR",46),s5e=new Ii("BREAKING_POINT_REMOVER",47),f$=new Ii("NORTH_SOUTH_PORT_POSTPROCESSOR",48),k5e=new Ii("HORIZONTAL_COMPACTOR",49),c$=new Ii("LABEL_DUMMY_REMOVER",50),p5e=new Ii("FINAL_SPLINE_BENDPOINTS_CALCULATOR",51),d5e=new Ii("END_LABEL_SORTER",52),xD=new Ii("REVERSED_EDGE_RESTORER",53),r$=new Ii("END_LABEL_POSTPROCESSOR",54),b5e=new Ii("HIERARCHICAL_NODE_RESIZER",55),h5e=new Ii("DIRECTION_POSTPROCESSOR",56)}function uPn(e,t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln,Dr,ts,il,rh,Vf,iz,xL,mT,EL,S8,fie,I_t,die,z0,vv,A8,_L,TL,e6,gie,yT,O_t,XEe,wv,kT,pie,t6,xT,_m,ET,bie,N_t;for(XEe=0,Dr=t,rh=0,xL=Dr.length;rh<xL;++rh)for(dn=Dr[rh],vv=new Q(dn.j);vv.a<vv.c.c.length;){for(z0=h(K(vv),11),_L=0,v=new Q(z0.g);v.a<v.c.c.length;)p=h(K(v),17),dn.c!=p.d.i.c&&++_L;_L>0&&(e.a[z0.p]=XEe++)}for(xT=0,ts=n,Vf=0,mT=ts.length;Vf<mT;++Vf){for(dn=ts[Vf],EL=0,vv=new Q(dn.j);vv.a<vv.c.c.length&&(z0=h(K(vv),11),z0.j==(Qe(),sn));)for(v=new Q(z0.e);v.a<v.c.c.length;)if(p=h(K(v),17),dn.c!=p.c.i.c){++EL;break}for(fie=0,TL=new Ms(dn.j,dn.j.c.length);TL.b>0;){for(z0=(kn(TL.b>0),h(TL.a.Xb(TL.c=--TL.b),11)),_L=0,v=new Q(z0.e);v.a<v.c.c.length;)p=h(K(v),17),dn.c!=p.c.i.c&&++_L;_L>0&&(z0.j==(Qe(),sn)?(e.a[z0.p]=xT,++xT):(e.a[z0.p]=xT+EL+fie,++fie))}xT+=fie}for(A8=new zn,j=new md,Ln=t,il=0,iz=Ln.length;il<iz;++il)for(dn=Ln[il],pie=new Q(dn.j);pie.a<pie.c.c.length;)for(kT=h(K(pie),11),v=new Q(kT.g);v.a<v.c.c.length;)if(p=h(K(v),17),ET=p.d,dn.c!=ET.i.c)if(wv=h(no(Ia(A8.f,kT)),467),_m=h(no(Ia(A8.f,ET)),467),!wv&&!_m)P=new Itt,j.a.zc(P,j),Ue(P.a,p),Ue(P.d,kT),sc(A8.f,kT,P),Ue(P.d,ET),sc(A8.f,ET,P);else if(!wv)Ue(_m.a,p),Ue(_m.d,kT),sc(A8.f,kT,_m);else if(!_m)Ue(wv.a,p),Ue(wv.d,ET),sc(A8.f,ET,wv);else if(wv==_m)Ue(wv.a,p);else{for(Ue(wv.a,p),die=new Q(_m.d);die.a<die.c.c.length;)I_t=h(K(die),11),sc(A8.f,I_t,wv);Vi(wv.a,_m.a),Vi(wv.d,_m.d),j.a.Bc(_m)!=null}for(V=h(nM(j,pe(LPn,{3:1,4:1,5:1,1946:1},467,j.a.gc(),0,1)),1946),ln=t[0].c,O_t=n[0].c,_=V,A=0,I=_.length;A<I;++A)for(x=_[A],x.e=XEe,x.f=xT,vv=new Q(x.d);vv.a<vv.c.c.length;)z0=h(K(vv),11),e6=e.a[z0.p],z0.i.c==ln?(e6<x.e&&(x.e=e6),e6>x.b&&(x.b=e6)):z0.i.c==O_t&&(e6<x.f&&(x.f=e6),e6>x.c&&(x.c=e6));for(_k(V,0,V.length,null),t6=pe(Hn,hr,25,V.length,15,1),s=pe(Hn,hr,25,xT+1,15,1),oe=0;oe<V.length;oe++)t6[oe]=V[oe].f,s[t6[oe]]=1;for(u=0,fe=0;fe<s.length;fe++)s[fe]==1?s[fe]=u:--u;for(gie=0,le=0;le<t6.length;le++)t6[le]+=s[t6[le]],gie=d.Math.max(gie,t6[le]+1);for(m=1;m<gie;)m*=2;for(N_t=2*m-1,m-=1,bie=pe(Hn,hr,25,N_t,15,1),a=0,wt=0;wt<t6.length;wt++)for(rt=t6[wt]+m,++bie[rt];rt>0;)rt%2>0&&(a+=bie[rt+1]),rt=(rt-1)/2|0,++bie[rt];for(xt=pe(a8t,st,362,V.length*2,0,1),we=0;we<V.length;we++)xt[2*we]=new WP(V[we],V[we].e,V[we].b,(GE(),$_)),xt[2*we+1]=new WP(V[we],V[we].b,V[we].e,j_);for(_k(xt,0,xt.length,null),S8=0,Ne=0;Ne<xt.length;Ne++)switch(xt[Ne].d.g){case 0:++S8;break;case 1:--S8,a+=S8}for(yT=pe(a8t,st,362,V.length*2,0,1),Ve=0;Ve<V.length;Ve++)yT[2*Ve]=new WP(V[Ve],V[Ve].f,V[Ve].c,(GE(),$_)),yT[2*Ve+1]=new WP(V[Ve],V[Ve].c,V[Ve].f,j_);for(_k(yT,0,yT.length,null),S8=0,J=0;J<yT.length;J++)switch(yT[J].d.g){case 0:++S8;break;case 1:--S8,a+=S8}return a}function Mr(){Mr=re,wT=new G2(7),zEe=new N1(8,94),new N1(8,64),GEe=new N1(8,36),E_t=new N1(8,65),__t=new N1(8,122),T_t=new N1(8,90),S_t=new N1(8,98),x_t=new N1(8,66),C_t=new N1(8,60),A_t=new N1(8,62),HEe=new G2(11),rz=new zu(4),$o(rz,48,57),C8=new zu(4),$o(C8,48,57),$o(C8,65,90),$o(C8,95,95),$o(C8,97,122),J5=new zu(4),$o(J5,9,9),$o(J5,10,10),$o(J5,12,12),$o(J5,13,13),$o(J5,32,32),VEe=Bw(rz),qEe=Bw(C8),UEe=Bw(J5),T8=new zn,vT=new zn,k_t=se(ie(it,1),Re,2,6,["Cn","Lu","Ll","Lt","Lm","Lo","Mn","Me","Mc","Nd","Nl","No","Zs","Zl","Zp","Cc","Cf",null,"Co","Cs","Pd","Ps","Pe","Pc","Po","Sm","Sc","Sk","So","Pi","Pf","L","M","N","Z","C","P","S"]),jEe=se(ie(it,1),Re,2,6,["Basic Latin","Latin-1 Supplement","Latin Extended-A","Latin Extended-B","IPA Extensions","Spacing Modifier Letters","Combining Diacritical Marks","Greek","Cyrillic","Armenian","Hebrew","Arabic","Syriac","Thaana","Devanagari","Bengali","Gurmukhi","Gujarati","Oriya","Tamil","Telugu","Kannada","Malayalam","Sinhala","Thai","Lao","Tibetan","Myanmar","Georgian","Hangul Jamo","Ethiopic","Cherokee","Unified Canadian Aboriginal Syllabics","Ogham","Runic","Khmer","Mongolian","Latin Extended Additional","Greek Extended","General Punctuation","Superscripts and Subscripts","Currency Symbols","Combining Marks for Symbols","Letterlike Symbols","Number Forms","Arrows","Mathematical Operators","Miscellaneous Technical","Control Pictures","Optical Character Recognition","Enclosed Alphanumerics","Box Drawing","Block Elements","Geometric Shapes","Miscellaneous Symbols","Dingbats","Braille Patterns","CJK Radicals Supplement","Kangxi Radicals","Ideographic Description Characters","CJK Symbols and Punctuation","Hiragana","Katakana","Bopomofo","Hangul Compatibility Jamo","Kanbun","Bopomofo Extended","Enclosed CJK Letters and Months","CJK Compatibility","CJK Unified Ideographs Extension A","CJK Unified Ideographs","Yi Syllables","Yi Radicals","Hangul Syllables",Wyt,"CJK Compatibility Ideographs","Alphabetic Presentation Forms","Arabic Presentation Forms-A","Combining Half Marks","CJK Compatibility Forms","Small Form Variants","Arabic Presentation Forms-B","Specials","Halfwidth and Fullwidth Forms","Old Italic","Gothic","Deseret","Byzantine Musical Symbols","Musical Symbols","Mathematical Alphanumeric Symbols","CJK Unified Ideographs Extension B","CJK Compatibility Ideographs Supplement","Tags"]),$Ee=se(ie(Hn,1),hr,25,15,[66304,66351,66352,66383,66560,66639,118784,119039,119040,119295,119808,120831,131072,173782,194560,195103,917504,917631])}function xR(){xR=re,f4t=new bl("OUT_T_L",0,(Xc(),v1),(wc(),Ah),(Kh(),ao),ao,se(ie(ql,1),st,21,0,[ni((Iw(),Oh),se(ie(ga,1),ze,93,0,[Nh,Lh]))])),h4t=new bl("OUT_T_C",1,jg,Ah,ao,Jo,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Nh,Vd])),ni(Oh,se(ie(ga,1),ze,93,0,[Nh,Vd,Kl]))])),d4t=new bl("OUT_T_R",2,Sh,Ah,ao,oo,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Nh,Ih]))])),r4t=new bl("OUT_B_L",3,v1,nf,oo,ao,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Xl,Lh]))])),n4t=new bl("OUT_B_C",4,jg,nf,oo,Jo,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Xl,Vd])),ni(Oh,se(ie(ga,1),ze,93,0,[Xl,Vd,Kl]))])),i4t=new bl("OUT_B_R",5,Sh,nf,oo,oo,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Xl,Ih]))])),o4t=new bl("OUT_L_T",6,Sh,nf,ao,ao,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Lh,Nh,Kl]))])),a4t=new bl("OUT_L_C",7,Sh,$g,Jo,ao,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Lh,J1])),ni(Oh,se(ie(ga,1),ze,93,0,[Lh,J1,Kl]))])),s4t=new bl("OUT_L_B",8,Sh,Ah,oo,ao,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Lh,Xl,Kl]))])),l4t=new bl("OUT_R_T",9,v1,nf,ao,oo,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Ih,Nh,Kl]))])),u4t=new bl("OUT_R_C",10,v1,$g,Jo,oo,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Ih,J1])),ni(Oh,se(ie(ga,1),ze,93,0,[Ih,J1,Kl]))])),c4t=new bl("OUT_R_B",11,v1,Ah,oo,oo,se(ie(ql,1),st,21,0,[ni(Oh,se(ie(ga,1),ze,93,0,[Ih,Xl,Kl]))])),e4t=new bl("IN_T_L",12,v1,nf,ao,ao,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[Nh,Lh])),ni(el,se(ie(ga,1),ze,93,0,[Nh,Lh,Kl]))])),J3t=new bl("IN_T_C",13,jg,nf,ao,Jo,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[Nh,Vd])),ni(el,se(ie(ga,1),ze,93,0,[Nh,Vd,Kl]))])),t4t=new bl("IN_T_R",14,Sh,nf,ao,oo,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[Nh,Ih])),ni(el,se(ie(ga,1),ze,93,0,[Nh,Ih,Kl]))])),Q3t=new bl("IN_C_L",15,v1,$g,Jo,ao,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[J1,Lh])),ni(el,se(ie(ga,1),ze,93,0,[J1,Lh,Kl]))])),X3t=new bl("IN_C_C",16,jg,$g,Jo,Jo,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[J1,Vd])),ni(el,se(ie(ga,1),ze,93,0,[J1,Vd,Kl]))])),Z3t=new bl("IN_C_R",17,Sh,$g,Jo,oo,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[J1,Ih])),ni(el,se(ie(ga,1),ze,93,0,[J1,Ih,Kl]))])),W3t=new bl("IN_B_L",18,v1,Ah,oo,ao,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[Xl,Lh])),ni(el,se(ie(ga,1),ze,93,0,[Xl,Lh,Kl]))])),Y3t=new bl("IN_B_C",19,jg,Ah,oo,Jo,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[Xl,Vd])),ni(el,se(ie(ga,1),ze,93,0,[Xl,Vd,Kl]))])),K3t=new bl("IN_B_R",20,Sh,Ah,oo,oo,se(ie(ql,1),st,21,0,[ni(el,se(ie(ga,1),ze,93,0,[Xl,Ih])),ni(el,se(ie(ga,1),ze,93,0,[Xl,Ih,Kl]))])),dte=new bl(j9,21,null,null,null,null,se(ie(ql,1),st,21,0,[]))}function Bt(){Bt=re,ym=(bg(),Xt).b,h(he(De(Xt.b),0),34),h(he(De(Xt.b),1),18),F0=Xt.a,h(he(De(Xt.a),0),34),h(he(De(Xt.a),1),18),h(he(De(Xt.a),2),18),h(he(De(Xt.a),3),18),h(he(De(Xt.a),4),18),np=Xt.o,h(he(De(Xt.o),0),34),h(he(De(Xt.o),1),34),H9t=h(he(De(Xt.o),2),18),h(he(De(Xt.o),3),18),h(he(De(Xt.o),4),18),h(he(De(Xt.o),5),18),h(he(De(Xt.o),6),18),h(he(De(Xt.o),7),18),h(he(De(Xt.o),8),18),h(he(De(Xt.o),9),18),h(he(De(Xt.o),10),18),h(he(De(Xt.o),11),18),h(he(De(Xt.o),12),18),h(he(De(Xt.o),13),18),h(he(De(Xt.o),14),18),h(he(De(Xt.o),15),18),h(he(Gr(Xt.o),0),59),h(he(Gr(Xt.o),1),59),h(he(Gr(Xt.o),2),59),h(he(Gr(Xt.o),3),59),h(he(Gr(Xt.o),4),59),h(he(Gr(Xt.o),5),59),h(he(Gr(Xt.o),6),59),h(he(Gr(Xt.o),7),59),h(he(Gr(Xt.o),8),59),h(he(Gr(Xt.o),9),59),$9t=Xt.p,h(he(De(Xt.p),0),34),h(he(De(Xt.p),1),34),h(he(De(Xt.p),2),34),h(he(De(Xt.p),3),34),h(he(De(Xt.p),4),18),h(he(De(Xt.p),5),18),h(he(Gr(Xt.p),0),59),h(he(Gr(Xt.p),1),59),z9t=Xt.q,h(he(De(Xt.q),0),34),rp=Xt.v,h(he(De(Xt.v),0),18),h(he(Gr(Xt.v),0),59),h(he(Gr(Xt.v),1),59),h(he(Gr(Xt.v),2),59),R0=Xt.w,h(he(De(Xt.w),0),34),h(he(De(Xt.w),1),34),h(he(De(Xt.w),2),34),h(he(De(Xt.w),3),18),ip=Xt.B,h(he(De(Xt.B),0),18),h(he(Gr(Xt.B),0),59),h(he(Gr(Xt.B),1),59),h(he(Gr(Xt.B),2),59),G9t=Xt.Q,h(he(De(Xt.Q),0),18),h(he(Gr(Xt.Q),0),59),V9t=Xt.R,h(he(De(Xt.R),0),34),nh=Xt.S,h(he(Gr(Xt.S),0),59),h(he(Gr(Xt.S),1),59),h(he(Gr(Xt.S),2),59),h(he(Gr(Xt.S),3),59),h(he(Gr(Xt.S),4),59),h(he(Gr(Xt.S),5),59),h(he(Gr(Xt.S),6),59),h(he(Gr(Xt.S),7),59),h(he(Gr(Xt.S),8),59),h(he(Gr(Xt.S),9),59),h(he(Gr(Xt.S),10),59),h(he(Gr(Xt.S),11),59),h(he(Gr(Xt.S),12),59),h(he(Gr(Xt.S),13),59),h(he(Gr(Xt.S),14),59),j0=Xt.T,h(he(De(Xt.T),0),18),h(he(De(Xt.T),2),18),U9t=h(he(De(Xt.T),3),18),h(he(De(Xt.T),4),18),h(he(Gr(Xt.T),0),59),h(he(Gr(Xt.T),1),59),h(he(De(Xt.T),1),18),$0=Xt.U,h(he(De(Xt.U),0),34),h(he(De(Xt.U),1),34),h(he(De(Xt.U),2),18),h(he(De(Xt.U),3),18),h(he(De(Xt.U),4),18),h(he(De(Xt.U),5),18),h(he(Gr(Xt.U),0),59),km=Xt.V,h(he(De(Xt.V),0),18),C3=Xt.W,h(he(De(Xt.W),0),34),h(he(De(Xt.W),1),34),h(he(De(Xt.W),2),34),h(he(De(Xt.W),3),18),h(he(De(Xt.W),4),18),h(he(De(Xt.W),5),18),q9t=Xt.bb,h(he(De(Xt.bb),0),34),h(he(De(Xt.bb),1),34),h(he(De(Xt.bb),2),34),h(he(De(Xt.bb),3),34),h(he(De(Xt.bb),4),34),h(he(De(Xt.bb),5),34),h(he(De(Xt.bb),6),34),h(he(De(Xt.bb),7),18),h(he(Gr(Xt.bb),0),59),h(he(Gr(Xt.bb),1),59),Y9t=Xt.eb,h(he(De(Xt.eb),0),34),h(he(De(Xt.eb),1),34),h(he(De(Xt.eb),2),34),h(he(De(Xt.eb),3),34),h(he(De(Xt.eb),4),34),h(he(De(Xt.eb),5),34),h(he(De(Xt.eb),6),18),h(he(De(Xt.eb),7),18),na=Xt.ab,h(he(De(Xt.ab),0),34),h(he(De(Xt.ab),1),34),gv=Xt.H,h(he(De(Xt.H),0),18),h(he(De(Xt.H),1),18),h(he(De(Xt.H),2),18),h(he(De(Xt.H),3),18),h(he(De(Xt.H),4),18),h(he(De(Xt.H),5),18),h(he(Gr(Xt.H),0),59),pv=Xt.db,h(he(De(Xt.db),0),18),zf=Xt.M}function lPn(e){var t;e.O||(e.O=!0,Ko(e,"type"),ZB(e,"ecore.xml.type"),JB(e,o2),t=h(lx((og(),th),o2),1945),qn(La(e.fb),e.b),io(e.b,wL,"AnyType",!1,!1,!0),Ci(h(he(De(e.b),0),34),e.wb.D,iD,null,0,-1,wL,!1,!1,!0,!1,!1,!1),Ci(h(he(De(e.b),1),34),e.wb.D,"any",null,0,-1,wL,!0,!0,!0,!1,!1,!0),Ci(h(he(De(e.b),2),34),e.wb.D,"anyAttribute",null,0,-1,wL,!1,!1,!0,!1,!1,!1),io(e.bb,tz,Myt,!1,!1,!0),Ci(h(he(De(e.bb),0),34),e.gb,"data",null,0,1,tz,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.bb),1),34),e.gb,Gye,null,1,1,tz,!1,!1,!0,!1,!0,!1),io(e.fb,mL,Dyt,!1,!1,!0),Ci(h(he(De(e.fb),0),34),t.gb,"rawValue",null,0,1,mL,!0,!0,!0,!1,!0,!0),Ci(h(he(De(e.fb),1),34),t.a,t_,null,0,1,mL,!0,!0,!0,!1,!0,!0),Zr(h(he(De(e.fb),2),18),e.wb.q,null,"instanceType",1,1,mL,!1,!1,!0,!1,!1,!1,!1),io(e.qb,PEe,Lyt,!1,!1,!0),Ci(h(he(De(e.qb),0),34),e.wb.D,iD,null,0,-1,null,!1,!1,!0,!1,!1,!1),Zr(h(he(De(e.qb),1),18),e.wb.ab,null,"xMLNSPrefixMap",0,-1,null,!0,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.qb),2),18),e.wb.ab,null,"xSISchemaLocation",0,-1,null,!0,!1,!0,!0,!1,!1,!1),Ci(h(he(De(e.qb),3),34),e.gb,"cDATA",null,0,-2,null,!0,!0,!0,!1,!1,!0),Ci(h(he(De(e.qb),4),34),e.gb,"comment",null,0,-2,null,!0,!0,!0,!1,!1,!0),Zr(h(he(De(e.qb),5),18),e.bb,null,Vyt,0,-2,null,!0,!0,!0,!0,!1,!1,!0),Ci(h(he(De(e.qb),6),34),e.gb,kee,null,0,-2,null,!0,!0,!0,!1,!1,!0),yr(e.a,yn,"AnySimpleType",!0),yr(e.c,it,"AnyURI",!0),yr(e.d,ie(Vc,1),"Base64Binary",!0),yr(e.e,vu,"Boolean",!0),yr(e.f,es,"BooleanObject",!0),yr(e.g,Vc,"Byte",!0),yr(e.i,A5,"ByteObject",!0),yr(e.j,it,"Date",!0),yr(e.k,it,"DateTime",!0),yr(e.n,rte,"Decimal",!0),yr(e.o,ks,"Double",!0),yr(e.p,Ts,"DoubleObject",!0),yr(e.q,it,"Duration",!0),yr(e.s,xl,"ENTITIES",!0),yr(e.r,xl,"ENTITIESBase",!0),yr(e.t,it,s3e,!0),yr(e.u,Em,"Float",!0),yr(e.v,zx,"FloatObject",!0),yr(e.w,it,"GDay",!0),yr(e.B,it,"GMonth",!0),yr(e.A,it,"GMonthDay",!0),yr(e.C,it,"GYear",!0),yr(e.D,it,"GYearMonth",!0),yr(e.F,ie(Vc,1),"HexBinary",!0),yr(e.G,it,"ID",!0),yr(e.H,it,"IDREF",!0),yr(e.J,xl,"IDREFS",!0),yr(e.I,xl,"IDREFSBase",!0),yr(e.K,Hn,"Int",!0),yr(e.M,t3,"Integer",!0),yr(e.L,Ws,"IntObject",!0),yr(e.P,it,"Language",!0),yr(e.Q,ap,"Long",!0),yr(e.R,Xb,"LongObject",!0),yr(e.S,it,"Name",!0),yr(e.T,it,Aj,!0),yr(e.U,t3,"NegativeInteger",!0),yr(e.V,it,c3e,!0),yr(e.X,xl,"NMTOKENS",!0),yr(e.W,xl,"NMTOKENSBase",!0),yr(e.Y,t3,"NonNegativeInteger",!0),yr(e.Z,t3,"NonPositiveInteger",!0),yr(e.$,it,"NormalizedString",!0),yr(e._,it,"NOTATION",!0),yr(e.ab,it,"PositiveInteger",!0),yr(e.cb,it,"QName",!0),yr(e.db,D3,"Short",!0),yr(e.eb,Qb,"ShortObject",!0),yr(e.gb,it,Qve,!0),yr(e.hb,it,"Time",!0),yr(e.ib,it,"Token",!0),yr(e.jb,D3,"UnsignedByte",!0),yr(e.kb,Qb,"UnsignedByteObject",!0),yr(e.lb,ap,"UnsignedInt",!0),yr(e.mb,Xb,"UnsignedIntObject",!0),yr(e.nb,t3,"UnsignedLong",!0),yr(e.ob,Hn,"UnsignedShort",!0),yr(e.pb,Ws,"UnsignedShortObject",!0),ope(e,o2),hPn(e))}function Lbt(e){J2(e,new Xp(xhe(fN(K2(q2(W2(Y2(new mp,wn),"ELK Layered"),"Layer-based algorithm provided by the Eclipse Layout Kernel. Arranges as many edges as possible into one direction by placing nodes into subsequent layers. This implementation supports different routing styles (straight, orthogonal, splines); if orthogonal routing is selected, arbitrary port constraints are respected, thus enabling the layout of block diagrams such as actor-oriented models or circuit schematics. Furthermore, full layout of compound graphs with cross-hierarchy edges is supported when the respective option is activated on the top level."),new h$e),wn),ni((ux(),eie),se(ie(WH,1),ze,237,0,[qH,YH,UH,Jre,VH,GH]))))),et(e,wn,yme,at(Lne)),et(e,wn,kme,at(Oke)),et(e,wn,aJ,at(ND)),et(e,wn,xme,at(Mh)),et(e,wn,hJ,at(z5)),et(e,wn,Eme,at(cv)),et(e,wn,_me,at(lm)),et(e,wn,Tme,at(h8)),et(e,wn,Cme,at(f8)),et(e,wn,Sme,at(Ine)),et(e,wn,zw,at(uv)),et(e,wn,Ame,at(One)),et(e,wn,Mme,at(I_)),et(e,wn,Dme,at(Z$)),et(e,wn,nme,at(OD)),et(e,wn,ime,at(p3)),et(e,wn,rme,at(b2)),et(e,wn,sme,at(b3)),et(e,wn,UM,Ke(0)),et(e,wn,ame,at(l8)),et(e,wn,ome,at(Ike)),et(e,wn,cme,at(H5)),et(e,wn,$d,at(Hke)),et(e,wn,YR,at(Nke)),et(e,wn,WR,at(Pke)),et(e,wn,Y9,at(Pne)),et(e,wn,BJ,at(Bke)),et(e,wn,FJ,at(Fke)),et(e,wn,KR,at(J$)),et(e,wn,XR,at(Bne)),et(e,wn,QR,at(jke)),et(e,wn,ZR,at(Rke)),et(e,wn,RJ,at($ke)),et(e,wn,DJ,at(av)),et(e,wn,LJ,at(L_)),et(e,wn,VR,at(Cne)),et(e,wn,UR,at(lke)),et(e,wn,Vb,Ske),et(e,wn,ZM,oke),et(e,wn,Lme,0),et(e,wn,BR,Ke(1)),et(e,wn,k5,Mx),et(e,wn,Ime,at(p2)),et(e,wn,fJ,at(Ei)),et(e,wn,Ome,at(u8)),et(e,wn,eD,at(mxt)),et(e,wn,Nme,at(m1)),et(e,wn,ej,at(sm)),et(e,wn,H9,(on(),!0)),et(e,wn,Pme,at(am)),et(e,wn,Bme,at(d2)),et(e,wn,Yy,at(g2)),et(e,wn,x5,at(Q$)),et(e,wn,VJ,at(Dne)),et(e,wn,Fme,ake),et(e,wn,Dx,at(sv)),et(e,wn,Rme,at(K$)),et(e,wn,Lx,at(cm)),et(e,wn,jme,at(Txt)),et(e,wn,$me,at(Dke)),et(e,wn,Hme,Mke),et(e,wn,zme,at(xxt)),et(e,wn,Gme,at(Ext)),et(e,wn,Vme,at(_xt)),et(e,wn,Ume,at(kxt)),et(e,wn,Uwe,at(Nne)),et(e,wn,XM,at(kke)),et(e,wn,kJ,at(Ane)),et(e,wn,Vwe,at(O_)),et(e,wn,Ywe,at(cc)),et(e,wn,wJ,at(im)),et(e,wn,q9,at(M_)),et(e,wn,Xwe,at(rm)),et(e,wn,Jwe,at(rke)),et(e,wn,CJ,at(xne)),et(e,wn,zR,at(ID)),et(e,wn,TJ,at(Ene)),et(e,wn,$we,at(xke)),et(e,wn,Hwe,at(Eke)),et(e,wn,HR,at(bke)),et(e,wn,Wy,at(X$)),et(e,wn,AJ,at(Mne)),et(e,wn,jwe,at(Sne)),et(e,wn,MJ,at(Tke)),et(e,wn,eme,at(uke)),et(e,wn,tme,at(Tne)),et(e,wn,tj,at(kne)),et(e,wn,SJ,at(_ke)),et(e,wn,ume,at(H$)),et(e,wn,lme,at(eke)),et(e,wn,OJ,at($$)),et(e,wn,qR,at(dke)),et(e,wn,NJ,at(fke)),et(e,wn,PJ,at(gke)),et(e,wn,E5,at(o8)),et(e,wn,qme,at(Da)),et(e,wn,oJ,at(D0)),et(e,wn,Yme,at(y1)),et(e,wn,FR,at(_ne)),et(e,wn,_J,at(ike)),et(e,wn,Wme,at(L0)),et(e,wn,Kme,at(A_)),et(e,wn,Xme,at(q$)),et(e,wn,Qme,at(ov)),et(e,wn,UJ,at(Ake)),et(e,wn,qJ,at(c8)),et(e,wn,mJ,at(wke)),et(e,wn,yJ,at(mke)),et(e,wn,nj,at(um)),et(e,wn,zwe,at(mne)),et(e,wn,xJ,at(yke)),et(e,wn,hme,at(U$)),et(e,wn,fme,at(V$)),et(e,wn,Zme,at(W$)),et(e,wn,EJ,at(vke)),et(e,wn,GR,at(D_)),et(e,wn,Jme,at(PD)),et(e,wn,Rwe,at(ske)),et(e,wn,Gwe,at(Lke)),et(e,wn,IJ,at(cke)),et(e,wn,Wwe,at(yxt)),et(e,wn,Qwe,at(wxt)),et(e,wn,YJ,at(pke)),et(e,wn,Kwe,at(Y$)),et(e,wn,Zwe,at(G$)),et(e,wn,JR,at(Vg)),et(e,wn,gme,at(nke)),et(e,wn,$J,at(z$)),et(e,wn,HJ,at(tke)),et(e,wn,dme,at(yne)),et(e,wn,jJ,at($5)),et(e,wn,qwe,at(hke))}function Gy(e,t){var n,s;return M3||(M3=new zn,_8=new zn,s=(Mr(),Mr(),new zu(4)),JA(s,`	
\r\r  `),Ta(M3,$ee,s),Ta(_8,$ee,Bw(s)),s=new zu(4),JA(s,qyt),Ta(M3,l_,s),Ta(_8,l_,Bw(s)),s=new zu(4),JA(s,qyt),Ta(M3,l_,s),Ta(_8,l_,Bw(s)),s=new zu(4),JA(s,Yyt),Fw(s,h(Fo(M3,l_),117)),Ta(M3,jee,s),Ta(_8,jee,Bw(s)),s=new zu(4),JA(s,"-.0:AZ__az\xB7\xB7\xC0\xD6\xD8\xF6\xF8\u0131\u0134\u013E\u0141\u0148\u014A\u017E\u0180\u01C3\u01CD\u01F0\u01F4\u01F5\u01FA\u0217\u0250\u02A8\u02BB\u02C1\u02D0\u02D1\u0300\u0345\u0360\u0361\u0386\u038A\u038C\u038C\u038E\u03A1\u03A3\u03CE\u03D0\u03D6\u03DA\u03DA\u03DC\u03DC\u03DE\u03DE\u03E0\u03E0\u03E2\u03F3\u0401\u040C\u040E\u044F\u0451\u045C\u045E\u0481\u0483\u0486\u0490\u04C4\u04C7\u04C8\u04CB\u04CC\u04D0\u04EB\u04EE\u04F5\u04F8\u04F9\u0531\u0556\u0559\u0559\u0561\u0586\u0591\u05A1\u05A3\u05B9\u05BB\u05BD\u05BF\u05BF\u05C1\u05C2\u05C4\u05C4\u05D0\u05EA\u05F0\u05F2\u0621\u063A\u0640\u0652\u0660\u0669\u0670\u06B7\u06BA\u06BE\u06C0\u06CE\u06D0\u06D3\u06D5\u06E8\u06EA\u06ED\u06F0\u06F9\u0901\u0903\u0905\u0939\u093C\u094D\u0951\u0954\u0958\u0963\u0966\u096F\u0981\u0983\u0985\u098C\u098F\u0990\u0993\u09A8\u09AA\u09B0\u09B2\u09B2\u09B6\u09B9\u09BC\u09BC\u09BE\u09C4\u09C7\u09C8\u09CB\u09CD\u09D7\u09D7\u09DC\u09DD\u09DF\u09E3\u09E6\u09F1\u0A02\u0A02\u0A05\u0A0A\u0A0F\u0A10\u0A13\u0A28\u0A2A\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3C\u0A3E\u0A42\u0A47\u0A48\u0A4B\u0A4D\u0A59\u0A5C\u0A5E\u0A5E\u0A66\u0A74\u0A81\u0A83\u0A85\u0A8B\u0A8D\u0A8D\u0A8F\u0A91\u0A93\u0AA8\u0AAA\u0AB0\u0AB2\u0AB3\u0AB5\u0AB9\u0ABC\u0AC5\u0AC7\u0AC9\u0ACB\u0ACD\u0AE0\u0AE0\u0AE6\u0AEF\u0B01\u0B03\u0B05\u0B0C\u0B0F\u0B10\u0B13\u0B28\u0B2A\u0B30\u0B32\u0B33\u0B36\u0B39\u0B3C\u0B43\u0B47\u0B48\u0B4B\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F\u0B61\u0B66\u0B6F\u0B82\u0B83\u0B85\u0B8A\u0B8E\u0B90\u0B92\u0B95\u0B99\u0B9A\u0B9C\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8\u0BAA\u0BAE\u0BB5\u0BB7\u0BB9\u0BBE\u0BC2\u0BC6\u0BC8\u0BCA\u0BCD\u0BD7\u0BD7\u0BE7\u0BEF\u0C01\u0C03\u0C05\u0C0C\u0C0E\u0C10\u0C12\u0C28\u0C2A\u0C33\u0C35\u0C39\u0C3E\u0C44\u0C46\u0C48\u0C4A\u0C4D\u0C55\u0C56\u0C60\u0C61\u0C66\u0C6F\u0C82\u0C83\u0C85\u0C8C\u0C8E\u0C90\u0C92\u0CA8\u0CAA\u0CB3\u0CB5\u0CB9\u0CBE\u0CC4\u0CC6\u0CC8\u0CCA\u0CCD\u0CD5\u0CD6\u0CDE\u0CDE\u0CE0\u0CE1\u0CE6\u0CEF\u0D02\u0D03\u0D05\u0D0C\u0D0E\u0D10\u0D12\u0D28\u0D2A\u0D39\u0D3E\u0D43\u0D46\u0D48\u0D4A\u0D4D\u0D57\u0D57\u0D60\u0D61\u0D66\u0D6F\u0E01\u0E2E\u0E30\u0E3A\u0E40\u0E4E\u0E50\u0E59\u0E81\u0E82\u0E84\u0E84\u0E87\u0E88\u0E8A\u0E8A\u0E8D\u0E8D\u0E94\u0E97\u0E99\u0E9F\u0EA1\u0EA3\u0EA5\u0EA5\u0EA7\u0EA7\u0EAA\u0EAB\u0EAD\u0EAE\u0EB0\u0EB9\u0EBB\u0EBD\u0EC0\u0EC4\u0EC6\u0EC6\u0EC8\u0ECD\u0ED0\u0ED9\u0F18\u0F19\u0F20\u0F29\u0F35\u0F35\u0F37\u0F37\u0F39\u0F39\u0F3E\u0F47\u0F49\u0F69\u0F71\u0F84\u0F86\u0F8B\u0F90\u0F95\u0F97\u0F97\u0F99\u0FAD\u0FB1\u0FB7\u0FB9\u0FB9\u10A0\u10C5\u10D0\u10F6\u1100\u1100\u1102\u1103\u1105\u1107\u1109\u1109\u110B\u110C\u110E\u1112\u113C\u113C\u113E\u113E\u1140\u1140\u114C\u114C\u114E\u114E\u1150\u1150\u1154\u1155\u1159\u1159\u115F\u1161\u1163\u1163\u1165\u1165\u1167\u1167\u1169\u1169\u116D\u116E\u1172\u1173\u1175\u1175\u119E\u119E\u11A8\u11A8\u11AB\u11AB\u11AE\u11AF\u11B7\u11B8\u11BA\u11BA\u11BC\u11C2\u11EB\u11EB\u11F0\u11F0\u11F9\u11F9\u1E00\u1E9B\u1EA0\u1EF9\u1F00\u1F15\u1F18\u1F1D\u1F20\u1F45\u1F48\u1F4D\u1F50\u1F57\u1F59\u1F59\u1F5B\u1F5B\u1F5D\u1F5D\u1F5F\u1F7D\u1F80\u1FB4\u1FB6\u1FBC\u1FBE\u1FBE\u1FC2\u1FC4\u1FC6\u1FCC\u1FD0\u1FD3\u1FD6\u1FDB\u1FE0\u1FEC\u1FF2\u1FF4\u1FF6\u1FFC\u20D0\u20DC\u20E1\u20E1\u2126\u2126\u212A\u212B\u212E\u212E\u2180\u2182\u3005\u3005\u3007\u3007\u3021\u302F\u3031\u3035\u3041\u3094\u3099\u309A\u309D\u309E\u30A1\u30FA\u30FC\u30FE\u3105\u312C\u4E00\u9FA5\uAC00\uD7A3"),Ta(M3,Hee,s),Ta(_8,Hee,Bw(s)),s=new zu(4),JA(s,Yyt),$o(s,95,95),$o(s,58,58),Ta(M3,zee,s),Ta(_8,zee,Bw(s))),n=h(Fo(t?M3:_8,e),136),n}function hPn(e){Yn(e.a,lr,se(ie(it,1),Re,2,6,[_r,"anySimpleType"])),Yn(e.b,lr,se(ie(it,1),Re,2,6,[_r,"anyType",Gl,iD])),Yn(h(he(De(e.b),0),34),lr,se(ie(it,1),Re,2,6,[Gl,Nee,_r,":mixed"])),Yn(h(he(De(e.b),1),34),lr,se(ie(it,1),Re,2,6,[Gl,Nee,r3e,Bee,_r,":1",Iyt,"lax"])),Yn(h(he(De(e.b),2),34),lr,se(ie(it,1),Re,2,6,[Gl,Ayt,r3e,Bee,_r,":2",Iyt,"lax"])),Yn(e.c,lr,se(ie(it,1),Re,2,6,[_r,"anyURI",Ul,tf])),Yn(e.d,lr,se(ie(it,1),Re,2,6,[_r,"base64Binary",Ul,tf])),Yn(e.e,lr,se(ie(it,1),Re,2,6,[_r,g5,Ul,tf])),Yn(e.f,lr,se(ie(it,1),Re,2,6,[_r,"boolean:Object",Hs,g5])),Yn(e.g,lr,se(ie(it,1),Re,2,6,[_r,r_])),Yn(e.i,lr,se(ie(it,1),Re,2,6,[_r,"byte:Object",Hs,r_])),Yn(e.j,lr,se(ie(it,1),Re,2,6,[_r,"date",Ul,tf])),Yn(e.k,lr,se(ie(it,1),Re,2,6,[_r,"dateTime",Ul,tf])),Yn(e.n,lr,se(ie(it,1),Re,2,6,[_r,"decimal",Ul,tf])),Yn(e.o,lr,se(ie(it,1),Re,2,6,[_r,i_,Ul,tf])),Yn(e.p,lr,se(ie(it,1),Re,2,6,[_r,"double:Object",Hs,i_])),Yn(e.q,lr,se(ie(it,1),Re,2,6,[_r,"duration",Ul,tf])),Yn(e.s,lr,se(ie(it,1),Re,2,6,[_r,"ENTITIES",Hs,Oyt,i3e,"1"])),Yn(e.r,lr,se(ie(it,1),Re,2,6,[_r,Oyt,Pee,s3e])),Yn(e.t,lr,se(ie(it,1),Re,2,6,[_r,s3e,Hs,Aj])),Yn(e.u,lr,se(ie(it,1),Re,2,6,[_r,s_,Ul,tf])),Yn(e.v,lr,se(ie(it,1),Re,2,6,[_r,"float:Object",Hs,s_])),Yn(e.w,lr,se(ie(it,1),Re,2,6,[_r,"gDay",Ul,tf])),Yn(e.B,lr,se(ie(it,1),Re,2,6,[_r,"gMonth",Ul,tf])),Yn(e.A,lr,se(ie(it,1),Re,2,6,[_r,"gMonthDay",Ul,tf])),Yn(e.C,lr,se(ie(it,1),Re,2,6,[_r,"gYear",Ul,tf])),Yn(e.D,lr,se(ie(it,1),Re,2,6,[_r,"gYearMonth",Ul,tf])),Yn(e.F,lr,se(ie(it,1),Re,2,6,[_r,"hexBinary",Ul,tf])),Yn(e.G,lr,se(ie(it,1),Re,2,6,[_r,"ID",Hs,Aj])),Yn(e.H,lr,se(ie(it,1),Re,2,6,[_r,"IDREF",Hs,Aj])),Yn(e.J,lr,se(ie(it,1),Re,2,6,[_r,"IDREFS",Hs,Nyt,i3e,"1"])),Yn(e.I,lr,se(ie(it,1),Re,2,6,[_r,Nyt,Pee,"IDREF"])),Yn(e.K,lr,se(ie(it,1),Re,2,6,[_r,a_])),Yn(e.M,lr,se(ie(it,1),Re,2,6,[_r,a3e])),Yn(e.L,lr,se(ie(it,1),Re,2,6,[_r,"int:Object",Hs,a_])),Yn(e.P,lr,se(ie(it,1),Re,2,6,[_r,"language",Hs,Fee,Ree,Pyt])),Yn(e.Q,lr,se(ie(it,1),Re,2,6,[_r,o_])),Yn(e.R,lr,se(ie(it,1),Re,2,6,[_r,"long:Object",Hs,o_])),Yn(e.S,lr,se(ie(it,1),Re,2,6,[_r,"Name",Hs,Fee,Ree,o3e])),Yn(e.T,lr,se(ie(it,1),Re,2,6,[_r,Aj,Hs,"Name",Ree,Byt])),Yn(e.U,lr,se(ie(it,1),Re,2,6,[_r,"negativeInteger",Hs,Fyt,uD,"-1"])),Yn(e.V,lr,se(ie(it,1),Re,2,6,[_r,c3e,Hs,Fee,Ree,"\\c+"])),Yn(e.X,lr,se(ie(it,1),Re,2,6,[_r,"NMTOKENS",Hs,Ryt,i3e,"1"])),Yn(e.W,lr,se(ie(it,1),Re,2,6,[_r,Ryt,Pee,c3e])),Yn(e.Y,lr,se(ie(it,1),Re,2,6,[_r,u3e,Hs,a3e,lD,"0"])),Yn(e.Z,lr,se(ie(it,1),Re,2,6,[_r,Fyt,Hs,a3e,uD,"0"])),Yn(e.$,lr,se(ie(it,1),Re,2,6,[_r,jyt,Hs,hZ,Ul,"replace"])),Yn(e._,lr,se(ie(it,1),Re,2,6,[_r,"NOTATION",Ul,tf])),Yn(e.ab,lr,se(ie(it,1),Re,2,6,[_r,"positiveInteger",Hs,u3e,lD,"1"])),Yn(e.bb,lr,se(ie(it,1),Re,2,6,[_r,"processingInstruction_._type",Gl,"empty"])),Yn(h(he(De(e.bb),0),34),lr,se(ie(it,1),Re,2,6,[Gl,_j,_r,"data"])),Yn(h(he(De(e.bb),1),34),lr,se(ie(it,1),Re,2,6,[Gl,_j,_r,Gye])),Yn(e.cb,lr,se(ie(it,1),Re,2,6,[_r,"QName",Ul,tf])),Yn(e.db,lr,se(ie(it,1),Re,2,6,[_r,c_])),Yn(e.eb,lr,se(ie(it,1),Re,2,6,[_r,"short:Object",Hs,c_])),Yn(e.fb,lr,se(ie(it,1),Re,2,6,[_r,"simpleAnyType",Gl,sD])),Yn(h(he(De(e.fb),0),34),lr,se(ie(it,1),Re,2,6,[_r,":3",Gl,sD])),Yn(h(he(De(e.fb),1),34),lr,se(ie(it,1),Re,2,6,[_r,":4",Gl,sD])),Yn(h(he(De(e.fb),2),18),lr,se(ie(it,1),Re,2,6,[_r,":5",Gl,sD])),Yn(e.gb,lr,se(ie(it,1),Re,2,6,[_r,hZ,Ul,"preserve"])),Yn(e.hb,lr,se(ie(it,1),Re,2,6,[_r,"time",Ul,tf])),Yn(e.ib,lr,se(ie(it,1),Re,2,6,[_r,Fee,Hs,jyt,Ul,tf])),Yn(e.jb,lr,se(ie(it,1),Re,2,6,[_r,$yt,uD,"255",lD,"0"])),Yn(e.kb,lr,se(ie(it,1),Re,2,6,[_r,"unsignedByte:Object",Hs,$yt])),Yn(e.lb,lr,se(ie(it,1),Re,2,6,[_r,Hyt,uD,"4294967295",lD,"0"])),Yn(e.mb,lr,se(ie(it,1),Re,2,6,[_r,"unsignedInt:Object",Hs,Hyt])),Yn(e.nb,lr,se(ie(it,1),Re,2,6,[_r,"unsignedLong",Hs,u3e,uD,zyt,lD,"0"])),Yn(e.ob,lr,se(ie(it,1),Re,2,6,[_r,Gyt,uD,"65535",lD,"0"])),Yn(e.pb,lr,se(ie(it,1),Re,2,6,[_r,"unsignedShort:Object",Hs,Gyt])),Yn(e.qb,lr,se(ie(it,1),Re,2,6,[_r,"",Gl,iD])),Yn(h(he(De(e.qb),0),34),lr,se(ie(it,1),Re,2,6,[Gl,Nee,_r,":mixed"])),Yn(h(he(De(e.qb),1),18),lr,se(ie(it,1),Re,2,6,[Gl,_j,_r,"xmlns:prefix"])),Yn(h(he(De(e.qb),2),18),lr,se(ie(it,1),Re,2,6,[Gl,_j,_r,"xsi:schemaLocation"])),Yn(h(he(De(e.qb),3),34),lr,se(ie(it,1),Re,2,6,[Gl,Tj,_r,"cDATA",Cj,aD])),Yn(h(he(De(e.qb),4),34),lr,se(ie(it,1),Re,2,6,[Gl,Tj,_r,"comment",Cj,aD])),Yn(h(he(De(e.qb),5),18),lr,se(ie(it,1),Re,2,6,[Gl,Tj,_r,Vyt,Cj,aD])),Yn(h(he(De(e.qb),6),34),lr,se(ie(it,1),Re,2,6,[Gl,Tj,_r,kee,Cj,aD]))}function ir(e){return Nt("_UI_EMFDiagnostic_marker",e)?"EMF Problem":Nt("_UI_CircularContainment_diagnostic",e)?"An object may not circularly contain itself":Nt(Cmt,e)?"Wrong character.":Nt(Smt,e)?"Invalid reference number.":Nt(pj,e)?"A character is required after \\.":Nt(Aee,e)?"'?' is not expected.  '(?:' or '(?=' or '(?!' or '(?<' or '(?#' or '(?>'?":Nt(Amt,e)?"'(?<' or '(?<!' is expected.":Nt(Mmt,e)?"A comment is not terminated.":Nt(a2,e)?"')' is expected.":Nt(Vye,e)?"Unexpected end of the pattern in a modifier group.":Nt(Dmt,e)?"':' is expected.":Nt(Lmt,e)?"Unexpected end of the pattern in a conditional group.":Nt(Imt,e)?"A back reference or an anchor or a lookahead or a look-behind is expected in a conditional pattern.":Nt(Omt,e)?"There are more than three choices in a conditional group.":Nt(Nmt,e)?"A character in U+0040-U+005f must follow \\c.":Nt(Pmt,e)?"A '{' is required before a character category.":Nt(Bmt,e)?"A property name is not closed by '}'.":Nt(Uye,e)?"Unexpected meta character.":Nt(Mee,e)?"Unknown property.":Nt(qye,e)?"A POSIX character class must be closed by ':]'.":Nt(bj,e)?"Unexpected end of the pattern in a character class.":Nt(Fmt,e)?"Unknown name for a POSIX character class.":Nt("parser.cc.4",e)?"'-' is invalid here.":Nt(Rmt,e)?"']' is expected.":Nt(Yye,e)?"'[' is invalid in a character class.  Write '\\['.":Nt(Wye,e)?"']' is invalid in a character class.  Write '\\]'.":Nt(Dee,e)?"'-' is an invalid character range. Write '\\-'.":Nt(jmt,e)?"'[' is expected.":Nt($mt,e)?"')' or '-[' or '+[' or '&[' is expected.":Nt(Hmt,e)?"The range end code point is less than the start code point.":Nt(A0,e)?"Invalid Unicode hex notation.":Nt(zmt,e)?"Overflow in a hex notation.":Nt(Gmt,e)?"'\\x{' must be closed by '}'.":Nt(Vmt,e)?"Invalid Unicode code point.":Nt(Umt,e)?"An anchor must not be here.":Nt(kl,e)?"This expression is not supported in the current option setting.":Nt(qmt,e)?"Invalid quantifier. A digit is expected.":Nt(Ymt,e)?"Invalid quantifier. Invalid quantity or a '}' is missing.":Nt(Wmt,e)?"Invalid quantifier. A digit or '}' is expected.":Nt(Kmt,e)?"Invalid quantifier. A min quantity must be <= a max quantity.":Nt(Kye,e)?"Invalid quantifier. A quantity value overflow.":Nt("_UI_PackageRegistry_extensionpoint",e)?"Ecore Package Registry for Generated Packages":Nt("_UI_DynamicPackageRegistry_extensionpoint",e)?"Ecore Package Registry for Dynamic Packages":Nt("_UI_FactoryRegistry_extensionpoint",e)?"Ecore Factory Override Registry":Nt("_UI_URIExtensionParserRegistry_extensionpoint",e)?"URI Extension Parser Registry":Nt("_UI_URIProtocolParserRegistry_extensionpoint",e)?"URI Protocol Parser Registry":Nt("_UI_URIContentParserRegistry_extensionpoint",e)?"URI Content Parser Registry":Nt("_UI_ContentHandlerRegistry_extensionpoint",e)?"Content Handler Registry":Nt("_UI_URIMappingRegistry_extensionpoint",e)?"URI Converter Mapping Registry":Nt("_UI_PackageRegistryImplementation_extensionpoint",e)?"Ecore Package Registry Implementation":Nt("_UI_ValidationDelegateRegistry_extensionpoint",e)?"Validation Delegate Registry":Nt("_UI_SettingDelegateRegistry_extensionpoint",e)?"Feature Setting Delegate Factory Registry":Nt("_UI_InvocationDelegateRegistry_extensionpoint",e)?"Operation Invocation Delegate Factory Registry":Nt("_UI_EClassInterfaceNotAbstract_diagnostic",e)?"A class that is an interface must also be abstract":Nt("_UI_EClassNoCircularSuperTypes_diagnostic",e)?"A class may not be a super type of itself":Nt("_UI_EClassNotWellFormedMapEntryNoInstanceClassName_diagnostic",e)?"A class that inherits from a map entry class must have instance class name 'java.util.Map$Entry'":Nt("_UI_EReferenceOppositeOfOppositeInconsistent_diagnostic",e)?"The opposite of the opposite may not be a reference different from this one":Nt("_UI_EReferenceOppositeNotFeatureOfType_diagnostic",e)?"The opposite must be a feature of the reference's type":Nt("_UI_EReferenceTransientOppositeNotTransient_diagnostic",e)?"The opposite of a transient reference must be transient if it is proxy resolving":Nt("_UI_EReferenceOppositeBothContainment_diagnostic",e)?"The opposite of a containment reference must not be a containment reference":Nt("_UI_EReferenceConsistentUnique_diagnostic",e)?"A containment or bidirectional reference must be unique if its upper bound is different from 1":Nt("_UI_ETypedElementNoType_diagnostic",e)?"The typed element must have a type":Nt("_UI_EAttributeNoDataType_diagnostic",e)?"The generic attribute type must not refer to a class":Nt("_UI_EReferenceNoClass_diagnostic",e)?"The generic reference type must not refer to a data type":Nt("_UI_EGenericTypeNoTypeParameterAndClassifier_diagnostic",e)?"A generic type can't refer to both a type parameter and a classifier":Nt("_UI_EGenericTypeNoClass_diagnostic",e)?"A generic super type must refer to a class":Nt("_UI_EGenericTypeNoTypeParameterOrClassifier_diagnostic",e)?"A generic type in this context must refer to a classifier or a type parameter":Nt("_UI_EGenericTypeBoundsOnlyForTypeArgument_diagnostic",e)?"A generic type may have bounds only when used as a type argument":Nt("_UI_EGenericTypeNoUpperAndLowerBound_diagnostic",e)?"A generic type must not have both a lower and an upper bound":Nt("_UI_EGenericTypeNoTypeParameterOrClassifierAndBound_diagnostic",e)?"A generic type with bounds must not also refer to a type parameter or classifier":Nt("_UI_EGenericTypeNoArguments_diagnostic",e)?"A generic type may have arguments only if it refers to a classifier":Nt("_UI_EGenericTypeOutOfScopeTypeParameter_diagnostic",e)?"A generic type may only refer to a type parameter that is in scope":e}function fPn(e){var t,n,s,a,u,p,v,m,x,_,A,I,P,j,V;e.r||(e.r=!0,Ko(e,"graph"),ZB(e,"graph"),JB(e,Bx),BA(e.o,"T"),qn(La(e.a),e.p),qn(La(e.f),e.a),qn(La(e.n),e.f),qn(La(e.g),e.n),qn(La(e.c),e.n),qn(La(e.i),e.c),qn(La(e.j),e.c),qn(La(e.d),e.f),qn(La(e.e),e.a),io(e.p,CPn,_vt,!0,!0,!1),j=xy(e.p,e.p,"setProperty"),V=Olt(j),x=xd(e.o),_=(n=(s=new V2,s),n),qn((!x.d&&(x.d=new Gi(pa,x,1)),x.d),_),A=NW(V),N2e(_,A),GF(j,x,Nye),x=NW(V),GF(j,x,t_),j=xy(e.p,null,"getProperty"),V=Olt(j),x=xd(e.o),_=NW(V),qn((!x.d&&(x.d=new Gi(pa,x,1)),x.d),_),GF(j,x,Nye),x=NW(V),P=Df(j,x,null),P&&P.Fi(),j=xy(e.p,e.wb.e,"hasProperty"),x=xd(e.o),_=(a=(u=new V2,u),a),qn((!x.d&&(x.d=new Gi(pa,x,1)),x.d),_),GF(j,x,Nye),j=xy(e.p,e.p,"copyProperties"),Ma(j,e.p,vee),j=xy(e.p,null,"getAllProperties"),x=xd(e.wb.P),_=xd(e.o),qn((!x.d&&(x.d=new Gi(pa,x,1)),x.d),_),A=(p=(v=new V2,v),p),qn((!_.d&&(_.d=new Gi(pa,_,1)),_.d),A),_=xd(e.wb.M),qn((!x.d&&(x.d=new Gi(pa,x,1)),x.d),_),I=Df(j,x,null),I&&I.Fi(),io(e.a,E8,nmt,!0,!1,!0),Zr(h(he(De(e.a),0),18),e.k,null,gmt,0,-1,E8,!1,!1,!0,!0,!1,!1,!1),io(e.f,hL,imt,!0,!1,!0),Zr(h(he(De(e.f),0),18),e.g,h(he(De(e.g),0),18),"labels",0,-1,hL,!1,!1,!0,!0,!1,!1,!1),Ci(h(he(De(e.f),1),34),e.wb._,pmt,null,0,1,hL,!1,!1,!0,!1,!0,!1),io(e.n,fL,"ElkShape",!0,!1,!0),Ci(h(he(De(e.n),0),34),e.wb.t,wee,_x,1,1,fL,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.n),1),34),e.wb.t,mee,_x,1,1,fL,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.n),2),34),e.wb.t,"x",_x,1,1,fL,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.n),3),34),e.wb.t,"y",_x,1,1,fL,!1,!1,!0,!1,!0,!1),j=xy(e.n,null,"setDimensions"),Ma(j,e.wb.t,mee),Ma(j,e.wb.t,wee),j=xy(e.n,null,"setLocation"),Ma(j,e.wb.t,"x"),Ma(j,e.wb.t,"y"),io(e.g,Ua,Lye,!1,!1,!0),Zr(h(he(De(e.g),0),18),e.f,h(he(De(e.f),0),18),yee,0,1,Ua,!1,!1,!0,!1,!1,!1,!1),Ci(h(he(De(e.g),1),34),e.wb._,kee,"",0,1,Ua,!1,!1,!0,!1,!0,!1),io(e.c,Pn,smt,!0,!1,!0),Zr(h(he(De(e.c),0),18),e.d,h(he(De(e.d),1),18),"outgoingEdges",0,-1,Pn,!1,!1,!0,!1,!0,!1,!1),Zr(h(he(De(e.c),1),18),e.d,h(he(De(e.d),2),18),"incomingEdges",0,-1,Pn,!1,!1,!0,!1,!0,!1,!1),io(e.i,wi,Iye,!1,!1,!0),Zr(h(he(De(e.i),0),18),e.j,h(he(De(e.j),0),18),"ports",0,-1,wi,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.i),1),18),e.i,h(he(De(e.i),2),18),xee,0,-1,wi,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.i),2),18),e.i,h(he(De(e.i),1),18),yee,0,1,wi,!1,!1,!0,!1,!1,!1,!1),Zr(h(he(De(e.i),3),18),e.d,h(he(De(e.d),0),18),"containedEdges",0,-1,wi,!1,!1,!0,!0,!1,!1,!1),Ci(h(he(De(e.i),4),34),e.wb.e,bmt,null,0,1,wi,!0,!0,!1,!1,!0,!0),io(e.j,bu,Oye,!1,!1,!0),Zr(h(he(De(e.j),0),18),e.i,h(he(De(e.i),0),18),yee,0,1,bu,!1,!1,!0,!1,!1,!1,!1),io(e.d,us,Dye,!1,!1,!0),Zr(h(he(De(e.d),0),18),e.i,h(he(De(e.i),3),18),"containingNode",0,1,us,!1,!1,!0,!1,!1,!1,!1),Zr(h(he(De(e.d),1),18),e.c,h(he(De(e.c),0),18),Pye,0,-1,us,!1,!1,!0,!1,!0,!1,!1),Zr(h(he(De(e.d),2),18),e.c,h(he(De(e.c),1),18),Eee,0,-1,us,!1,!1,!0,!1,!0,!1,!1),Zr(h(he(De(e.d),3),18),e.e,h(he(De(e.e),5),18),Bye,0,-1,us,!1,!1,!0,!0,!1,!1,!1),Ci(h(he(De(e.d),4),34),e.wb.e,"hyperedge",null,0,1,us,!0,!0,!1,!1,!0,!0),Ci(h(he(De(e.d),5),34),e.wb.e,bmt,null,0,1,us,!0,!0,!1,!1,!0,!0),Ci(h(he(De(e.d),6),34),e.wb.e,"selfloop",null,0,1,us,!0,!0,!1,!1,!0,!0),Ci(h(he(De(e.d),7),34),e.wb.e,"connected",null,0,1,us,!0,!0,!1,!1,!0,!0),io(e.b,Zl,rmt,!1,!1,!0),Ci(h(he(De(e.b),0),34),e.wb.t,"x",_x,1,1,Zl,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.b),1),34),e.wb.t,"y",_x,1,1,Zl,!1,!1,!0,!1,!0,!1),j=xy(e.b,null,"set"),Ma(j,e.wb.t,"x"),Ma(j,e.wb.t,"y"),io(e.e,gi,amt,!1,!1,!0),Ci(h(he(De(e.e),0),34),e.wb.t,"startX",null,0,1,gi,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.e),1),34),e.wb.t,"startY",null,0,1,gi,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.e),2),34),e.wb.t,"endX",null,0,1,gi,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.e),3),34),e.wb.t,"endY",null,0,1,gi,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.e),4),18),e.b,null,hj,0,-1,gi,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.e),5),18),e.d,h(he(De(e.d),3),18),yee,0,1,gi,!1,!1,!0,!1,!1,!1,!1),Zr(h(he(De(e.e),6),18),e.c,null,Fye,0,1,gi,!1,!1,!0,!1,!0,!1,!1),Zr(h(he(De(e.e),7),18),e.c,null,Rye,0,1,gi,!1,!1,!0,!1,!0,!1,!1),Zr(h(he(De(e.e),8),18),e.e,h(he(De(e.e),9),18),jye,0,-1,gi,!1,!1,!0,!1,!0,!1,!1),Zr(h(he(De(e.e),9),18),e.e,h(he(De(e.e),8),18),$ye,0,-1,gi,!1,!1,!0,!1,!0,!1,!1),Ci(h(he(De(e.e),10),34),e.wb._,pmt,null,0,1,gi,!1,!1,!0,!1,!0,!1),j=xy(e.e,null,"setStartLocation"),Ma(j,e.wb.t,"x"),Ma(j,e.wb.t,"y"),j=xy(e.e,null,"setEndLocation"),Ma(j,e.wb.t,"x"),Ma(j,e.wb.t,"y"),io(e.k,c2,"ElkPropertyToValueMapEntry",!1,!1,!1),x=xd(e.o),_=(m=(t=new V2,t),m),qn((!x.d&&(x.d=new Gi(pa,x,1)),x.d),_),O0t(h(he(De(e.k),0),34),x,"key",c2,!1,!1,!0,!1),Ci(h(he(De(e.k),1),34),e.s,t_,null,0,1,c2,!1,!1,!0,!1,!0,!1),yr(e.o,Mre,"IProperty",!0),yr(e.s,yn,"PropertyValue",!0),ope(e,Bx))}function Ibt(){Ibt=re,ce=pe(Vc,Zy,25,ea,15,1),ce[9]=35,ce[10]=19,ce[13]=19,ce[32]=51,ce[33]=49,ce[34]=33,ye(ce,35,38,49),ce[38]=1,ye(ce,39,45,49),ye(ce,45,47,-71),ce[47]=49,ye(ce,48,58,-71),ce[58]=61,ce[59]=49,ce[60]=1,ce[61]=49,ce[62]=33,ye(ce,63,65,49),ye(ce,65,91,-3),ye(ce,91,93,33),ce[93]=1,ce[94]=33,ce[95]=-3,ce[96]=33,ye(ce,97,123,-3),ye(ce,123,183,33),ce[183]=-87,ye(ce,184,192,33),ye(ce,192,215,-19),ce[215]=33,ye(ce,216,247,-19),ce[247]=33,ye(ce,248,306,-19),ye(ce,306,308,33),ye(ce,308,319,-19),ye(ce,319,321,33),ye(ce,321,329,-19),ce[329]=33,ye(ce,330,383,-19),ce[383]=33,ye(ce,384,452,-19),ye(ce,452,461,33),ye(ce,461,497,-19),ye(ce,497,500,33),ye(ce,500,502,-19),ye(ce,502,506,33),ye(ce,506,536,-19),ye(ce,536,592,33),ye(ce,592,681,-19),ye(ce,681,699,33),ye(ce,699,706,-19),ye(ce,706,720,33),ye(ce,720,722,-87),ye(ce,722,768,33),ye(ce,768,838,-87),ye(ce,838,864,33),ye(ce,864,866,-87),ye(ce,866,902,33),ce[902]=-19,ce[903]=-87,ye(ce,904,907,-19),ce[907]=33,ce[908]=-19,ce[909]=33,ye(ce,910,930,-19),ce[930]=33,ye(ce,931,975,-19),ce[975]=33,ye(ce,976,983,-19),ye(ce,983,986,33),ce[986]=-19,ce[987]=33,ce[988]=-19,ce[989]=33,ce[990]=-19,ce[991]=33,ce[992]=-19,ce[993]=33,ye(ce,994,1012,-19),ye(ce,1012,1025,33),ye(ce,1025,1037,-19),ce[1037]=33,ye(ce,1038,1104,-19),ce[1104]=33,ye(ce,1105,1117,-19),ce[1117]=33,ye(ce,1118,1154,-19),ce[1154]=33,ye(ce,1155,1159,-87),ye(ce,1159,1168,33),ye(ce,1168,1221,-19),ye(ce,1221,1223,33),ye(ce,1223,1225,-19),ye(ce,1225,1227,33),ye(ce,1227,1229,-19),ye(ce,1229,1232,33),ye(ce,1232,1260,-19),ye(ce,1260,1262,33),ye(ce,1262,1270,-19),ye(ce,1270,1272,33),ye(ce,1272,1274,-19),ye(ce,1274,1329,33),ye(ce,1329,1367,-19),ye(ce,1367,1369,33),ce[1369]=-19,ye(ce,1370,1377,33),ye(ce,1377,1415,-19),ye(ce,1415,1425,33),ye(ce,1425,1442,-87),ce[1442]=33,ye(ce,1443,1466,-87),ce[1466]=33,ye(ce,1467,1470,-87),ce[1470]=33,ce[1471]=-87,ce[1472]=33,ye(ce,1473,1475,-87),ce[1475]=33,ce[1476]=-87,ye(ce,1477,1488,33),ye(ce,1488,1515,-19),ye(ce,1515,1520,33),ye(ce,1520,1523,-19),ye(ce,1523,1569,33),ye(ce,1569,1595,-19),ye(ce,1595,1600,33),ce[1600]=-87,ye(ce,1601,1611,-19),ye(ce,1611,1619,-87),ye(ce,1619,1632,33),ye(ce,1632,1642,-87),ye(ce,1642,1648,33),ce[1648]=-87,ye(ce,1649,1720,-19),ye(ce,1720,1722,33),ye(ce,1722,1727,-19),ce[1727]=33,ye(ce,1728,1743,-19),ce[1743]=33,ye(ce,1744,1748,-19),ce[1748]=33,ce[1749]=-19,ye(ce,1750,1765,-87),ye(ce,1765,1767,-19),ye(ce,1767,1769,-87),ce[1769]=33,ye(ce,1770,1774,-87),ye(ce,1774,1776,33),ye(ce,1776,1786,-87),ye(ce,1786,2305,33),ye(ce,2305,2308,-87),ce[2308]=33,ye(ce,2309,2362,-19),ye(ce,2362,2364,33),ce[2364]=-87,ce[2365]=-19,ye(ce,2366,2382,-87),ye(ce,2382,2385,33),ye(ce,2385,2389,-87),ye(ce,2389,2392,33),ye(ce,2392,2402,-19),ye(ce,2402,2404,-87),ye(ce,2404,2406,33),ye(ce,2406,2416,-87),ye(ce,2416,2433,33),ye(ce,2433,2436,-87),ce[2436]=33,ye(ce,2437,2445,-19),ye(ce,2445,2447,33),ye(ce,2447,2449,-19),ye(ce,2449,2451,33),ye(ce,2451,2473,-19),ce[2473]=33,ye(ce,2474,2481,-19),ce[2481]=33,ce[2482]=-19,ye(ce,2483,2486,33),ye(ce,2486,2490,-19),ye(ce,2490,2492,33),ce[2492]=-87,ce[2493]=33,ye(ce,2494,2501,-87),ye(ce,2501,2503,33),ye(ce,2503,2505,-87),ye(ce,2505,2507,33),ye(ce,2507,2510,-87),ye(ce,2510,2519,33),ce[2519]=-87,ye(ce,2520,2524,33),ye(ce,2524,2526,-19),ce[2526]=33,ye(ce,2527,2530,-19),ye(ce,2530,2532,-87),ye(ce,2532,2534,33),ye(ce,2534,2544,-87),ye(ce,2544,2546,-19),ye(ce,2546,2562,33),ce[2562]=-87,ye(ce,2563,2565,33),ye(ce,2565,2571,-19),ye(ce,2571,2575,33),ye(ce,2575,2577,-19),ye(ce,2577,2579,33),ye(ce,2579,2601,-19),ce[2601]=33,ye(ce,2602,2609,-19),ce[2609]=33,ye(ce,2610,2612,-19),ce[2612]=33,ye(ce,2613,2615,-19),ce[2615]=33,ye(ce,2616,2618,-19),ye(ce,2618,2620,33),ce[2620]=-87,ce[2621]=33,ye(ce,2622,2627,-87),ye(ce,2627,2631,33),ye(ce,2631,2633,-87),ye(ce,2633,2635,33),ye(ce,2635,2638,-87),ye(ce,2638,2649,33),ye(ce,2649,2653,-19),ce[2653]=33,ce[2654]=-19,ye(ce,2655,2662,33),ye(ce,2662,2674,-87),ye(ce,2674,2677,-19),ye(ce,2677,2689,33),ye(ce,2689,2692,-87),ce[2692]=33,ye(ce,2693,2700,-19),ce[2700]=33,ce[2701]=-19,ce[2702]=33,ye(ce,2703,2706,-19),ce[2706]=33,ye(ce,2707,2729,-19),ce[2729]=33,ye(ce,2730,2737,-19),ce[2737]=33,ye(ce,2738,2740,-19),ce[2740]=33,ye(ce,2741,2746,-19),ye(ce,2746,2748,33),ce[2748]=-87,ce[2749]=-19,ye(ce,2750,2758,-87),ce[2758]=33,ye(ce,2759,2762,-87),ce[2762]=33,ye(ce,2763,2766,-87),ye(ce,2766,2784,33),ce[2784]=-19,ye(ce,2785,2790,33),ye(ce,2790,2800,-87),ye(ce,2800,2817,33),ye(ce,2817,2820,-87),ce[2820]=33,ye(ce,2821,2829,-19),ye(ce,2829,2831,33),ye(ce,2831,2833,-19),ye(ce,2833,2835,33),ye(ce,2835,2857,-19),ce[2857]=33,ye(ce,2858,2865,-19),ce[2865]=33,ye(ce,2866,2868,-19),ye(ce,2868,2870,33),ye(ce,2870,2874,-19),ye(ce,2874,2876,33),ce[2876]=-87,ce[2877]=-19,ye(ce,2878,2884,-87),ye(ce,2884,2887,33),ye(ce,2887,2889,-87),ye(ce,2889,2891,33),ye(ce,2891,2894,-87),ye(ce,2894,2902,33),ye(ce,2902,2904,-87),ye(ce,2904,2908,33),ye(ce,2908,2910,-19),ce[2910]=33,ye(ce,2911,2914,-19),ye(ce,2914,2918,33),ye(ce,2918,2928,-87),ye(ce,2928,2946,33),ye(ce,2946,2948,-87),ce[2948]=33,ye(ce,2949,2955,-19),ye(ce,2955,2958,33),ye(ce,2958,2961,-19),ce[2961]=33,ye(ce,2962,2966,-19),ye(ce,2966,2969,33),ye(ce,2969,2971,-19),ce[2971]=33,ce[2972]=-19,ce[2973]=33,ye(ce,2974,2976,-19),ye(ce,2976,2979,33),ye(ce,2979,2981,-19),ye(ce,2981,2984,33),ye(ce,2984,2987,-19),ye(ce,2987,2990,33),ye(ce,2990,2998,-19),ce[2998]=33,ye(ce,2999,3002,-19),ye(ce,3002,3006,33),ye(ce,3006,3011,-87),ye(ce,3011,3014,33),ye(ce,3014,3017,-87),ce[3017]=33,ye(ce,3018,3022,-87),ye(ce,3022,3031,33),ce[3031]=-87,ye(ce,3032,3047,33),ye(ce,3047,3056,-87),ye(ce,3056,3073,33),ye(ce,3073,3076,-87),ce[3076]=33,ye(ce,3077,3085,-19),ce[3085]=33,ye(ce,3086,3089,-19),ce[3089]=33,ye(ce,3090,3113,-19),ce[3113]=33,ye(ce,3114,3124,-19),ce[3124]=33,ye(ce,3125,3130,-19),ye(ce,3130,3134,33),ye(ce,3134,3141,-87),ce[3141]=33,ye(ce,3142,3145,-87),ce[3145]=33,ye(ce,3146,3150,-87),ye(ce,3150,3157,33),ye(ce,3157,3159,-87),ye(ce,3159,3168,33),ye(ce,3168,3170,-19),ye(ce,3170,3174,33),ye(ce,3174,3184,-87),ye(ce,3184,3202,33),ye(ce,3202,3204,-87),ce[3204]=33,ye(ce,3205,3213,-19),ce[3213]=33,ye(ce,3214,3217,-19),ce[3217]=33,ye(ce,3218,3241,-19),ce[3241]=33,ye(ce,3242,3252,-19),ce[3252]=33,ye(ce,3253,3258,-19),ye(ce,3258,3262,33),ye(ce,3262,3269,-87),ce[3269]=33,ye(ce,3270,3273,-87),ce[3273]=33,ye(ce,3274,3278,-87),ye(ce,3278,3285,33),ye(ce,3285,3287,-87),ye(ce,3287,3294,33),ce[3294]=-19,ce[3295]=33,ye(ce,3296,3298,-19),ye(ce,3298,3302,33),ye(ce,3302,3312,-87),ye(ce,3312,3330,33),ye(ce,3330,3332,-87),ce[3332]=33,ye(ce,3333,3341,-19),ce[3341]=33,ye(ce,3342,3345,-19),ce[3345]=33,ye(ce,3346,3369,-19),ce[3369]=33,ye(ce,3370,3386,-19),ye(ce,3386,3390,33),ye(ce,3390,3396,-87),ye(ce,3396,3398,33),ye(ce,3398,3401,-87),ce[3401]=33,ye(ce,3402,3406,-87),ye(ce,3406,3415,33),ce[3415]=-87,ye(ce,3416,3424,33),ye(ce,3424,3426,-19),ye(ce,3426,3430,33),ye(ce,3430,3440,-87),ye(ce,3440,3585,33),ye(ce,3585,3631,-19),ce[3631]=33,ce[3632]=-19,ce[3633]=-87,ye(ce,3634,3636,-19),ye(ce,3636,3643,-87),ye(ce,3643,3648,33),ye(ce,3648,3654,-19),ye(ce,3654,3663,-87),ce[3663]=33,ye(ce,3664,3674,-87),ye(ce,3674,3713,33),ye(ce,3713,3715,-19),ce[3715]=33,ce[3716]=-19,ye(ce,3717,3719,33),ye(ce,3719,3721,-19),ce[3721]=33,ce[3722]=-19,ye(ce,3723,3725,33),ce[3725]=-19,ye(ce,3726,3732,33),ye(ce,3732,3736,-19),ce[3736]=33,ye(ce,3737,3744,-19),ce[3744]=33,ye(ce,3745,3748,-19),ce[3748]=33,ce[3749]=-19,ce[3750]=33,ce[3751]=-19,ye(ce,3752,3754,33),ye(ce,3754,3756,-19),ce[3756]=33,ye(ce,3757,3759,-19),ce[3759]=33,ce[3760]=-19,ce[3761]=-87,ye(ce,3762,3764,-19),ye(ce,3764,3770,-87),ce[3770]=33,ye(ce,3771,3773,-87),ce[3773]=-19,ye(ce,3774,3776,33),ye(ce,3776,3781,-19),ce[3781]=33,ce[3782]=-87,ce[3783]=33,ye(ce,3784,3790,-87),ye(ce,3790,3792,33),ye(ce,3792,3802,-87),ye(ce,3802,3864,33),ye(ce,3864,3866,-87),ye(ce,3866,3872,33),ye(ce,3872,3882,-87),ye(ce,3882,3893,33),ce[3893]=-87,ce[3894]=33,ce[3895]=-87,ce[3896]=33,ce[3897]=-87,ye(ce,3898,3902,33),ye(ce,3902,3904,-87),ye(ce,3904,3912,-19),ce[3912]=33,ye(ce,3913,3946,-19),ye(ce,3946,3953,33),ye(ce,3953,3973,-87),ce[3973]=33,ye(ce,3974,3980,-87),ye(ce,3980,3984,33),ye(ce,3984,3990,-87),ce[3990]=33,ce[3991]=-87,ce[3992]=33,ye(ce,3993,4014,-87),ye(ce,4014,4017,33),ye(ce,4017,4024,-87),ce[4024]=33,ce[4025]=-87,ye(ce,4026,4256,33),ye(ce,4256,4294,-19),ye(ce,4294,4304,33),ye(ce,4304,4343,-19),ye(ce,4343,4352,33),ce[4352]=-19,ce[4353]=33,ye(ce,4354,4356,-19),ce[4356]=33,ye(ce,4357,4360,-19),ce[4360]=33,ce[4361]=-19,ce[4362]=33,ye(ce,4363,4365,-19),ce[4365]=33,ye(ce,4366,4371,-19),ye(ce,4371,4412,33),ce[4412]=-19,ce[4413]=33,ce[4414]=-19,ce[4415]=33,ce[4416]=-19,ye(ce,4417,4428,33),ce[4428]=-19,ce[4429]=33,ce[4430]=-19,ce[4431]=33,ce[4432]=-19,ye(ce,4433,4436,33),ye(ce,4436,4438,-19),ye(ce,4438,4441,33),ce[4441]=-19,ye(ce,4442,4447,33),ye(ce,4447,4450,-19),ce[4450]=33,ce[4451]=-19,ce[4452]=33,ce[4453]=-19,ce[4454]=33,ce[4455]=-19,ce[4456]=33,ce[4457]=-19,ye(ce,4458,4461,33),ye(ce,4461,4463,-19),ye(ce,4463,4466,33),ye(ce,4466,4468,-19),ce[4468]=33,ce[4469]=-19,ye(ce,4470,4510,33),ce[4510]=-19,ye(ce,4511,4520,33),ce[4520]=-19,ye(ce,4521,4523,33),ce[4523]=-19,ye(ce,4524,4526,33),ye(ce,4526,4528,-19),ye(ce,4528,4535,33),ye(ce,4535,4537,-19),ce[4537]=33,ce[4538]=-19,ce[4539]=33,ye(ce,4540,4547,-19),ye(ce,4547,4587,33),ce[4587]=-19,ye(ce,4588,4592,33),ce[4592]=-19,ye(ce,4593,4601,33),ce[4601]=-19,ye(ce,4602,7680,33),ye(ce,7680,7836,-19),ye(ce,7836,7840,33),ye(ce,7840,7930,-19),ye(ce,7930,7936,33),ye(ce,7936,7958,-19),ye(ce,7958,7960,33),ye(ce,7960,7966,-19),ye(ce,7966,7968,33),ye(ce,7968,8006,-19),ye(ce,8006,8008,33),ye(ce,8008,8014,-19),ye(ce,8014,8016,33),ye(ce,8016,8024,-19),ce[8024]=33,ce[8025]=-19,ce[8026]=33,ce[8027]=-19,ce[8028]=33,ce[8029]=-19,ce[8030]=33,ye(ce,8031,8062,-19),ye(ce,8062,8064,33),ye(ce,8064,8117,-19),ce[8117]=33,ye(ce,8118,8125,-19),ce[8125]=33,ce[8126]=-19,ye(ce,8127,8130,33),ye(ce,8130,8133,-19),ce[8133]=33,ye(ce,8134,8141,-19),ye(ce,8141,8144,33),ye(ce,8144,8148,-19),ye(ce,8148,8150,33),ye(ce,8150,8156,-19),ye(ce,8156,8160,33),ye(ce,8160,8173,-19),ye(ce,8173,8178,33),ye(ce,8178,8181,-19),ce[8181]=33,ye(ce,8182,8189,-19),ye(ce,8189,8400,33),ye(ce,8400,8413,-87),ye(ce,8413,8417,33),ce[8417]=-87,ye(ce,8418,8486,33),ce[8486]=-19,ye(ce,8487,8490,33),ye(ce,8490,8492,-19),ye(ce,8492,8494,33),ce[8494]=-19,ye(ce,8495,8576,33),ye(ce,8576,8579,-19),ye(ce,8579,12293,33),ce[12293]=-87,ce[12294]=33,ce[12295]=-19,ye(ce,12296,12321,33),ye(ce,12321,12330,-19),ye(ce,12330,12336,-87),ce[12336]=33,ye(ce,12337,12342,-87),ye(ce,12342,12353,33),ye(ce,12353,12437,-19),ye(ce,12437,12441,33),ye(ce,12441,12443,-87),ye(ce,12443,12445,33),ye(ce,12445,12447,-87),ye(ce,12447,12449,33),ye(ce,12449,12539,-19),ce[12539]=33,ye(ce,12540,12543,-87),ye(ce,12543,12549,33),ye(ce,12549,12589,-19),ye(ce,12589,19968,33),ye(ce,19968,40870,-19),ye(ce,40870,44032,33),ye(ce,44032,55204,-19),ye(ce,55204,jM,33),ye(ce,57344,65534,33)}function dPn(e){var t,n,s,a,u,p,v;e.hb||(e.hb=!0,Ko(e,"ecore"),ZB(e,"ecore"),JB(e,Vl),BA(e.fb,"E"),BA(e.L,"T"),BA(e.P,"K"),BA(e.P,"V"),BA(e.cb,"E"),qn(La(e.b),e.bb),qn(La(e.a),e.Q),qn(La(e.o),e.p),qn(La(e.p),e.R),qn(La(e.q),e.p),qn(La(e.v),e.q),qn(La(e.w),e.R),qn(La(e.B),e.Q),qn(La(e.R),e.Q),qn(La(e.T),e.eb),qn(La(e.U),e.R),qn(La(e.V),e.eb),qn(La(e.W),e.bb),qn(La(e.bb),e.eb),qn(La(e.eb),e.R),qn(La(e.db),e.R),io(e.b,wm,syt,!1,!1,!0),Ci(h(he(De(e.b),0),34),e.e,"iD",null,0,1,wm,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.b),1),18),e.q,null,"eAttributeType",1,1,wm,!0,!0,!1,!1,!0,!1,!0),io(e.a,dr,nyt,!1,!1,!0),Ci(h(he(De(e.a),0),34),e._,vee,null,0,1,dr,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.a),1),18),e.ab,null,"details",0,-1,dr,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.a),2),18),e.Q,h(he(De(e.Q),0),18),"eModelElement",0,1,dr,!0,!1,!0,!1,!1,!1,!1),Zr(h(he(De(e.a),3),18),e.S,null,"contents",0,-1,dr,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.a),4),18),e.S,null,"references",0,-1,dr,!1,!1,!0,!1,!0,!1,!1),io(e.o,Jl,"EClass",!1,!1,!0),Ci(h(he(De(e.o),0),34),e.e,"abstract",null,0,1,Jl,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.o),1),34),e.e,"interface",null,0,1,Jl,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.o),2),18),e.o,null,"eSuperTypes",0,-1,Jl,!1,!1,!0,!1,!0,!0,!1),Zr(h(he(De(e.o),3),18),e.T,h(he(De(e.T),0),18),"eOperations",0,-1,Jl,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.o),4),18),e.b,null,"eAllAttributes",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),5),18),e.W,null,"eAllReferences",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),6),18),e.W,null,"eReferences",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),7),18),e.b,null,"eAttributes",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),8),18),e.W,null,"eAllContainments",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),9),18),e.T,null,"eAllOperations",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),10),18),e.bb,null,"eAllStructuralFeatures",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),11),18),e.o,null,"eAllSuperTypes",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.o),12),18),e.b,null,"eIDAttribute",0,1,Jl,!0,!0,!1,!1,!1,!1,!0),Zr(h(he(De(e.o),13),18),e.bb,h(he(De(e.bb),7),18),"eStructuralFeatures",0,-1,Jl,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.o),14),18),e.H,null,"eGenericSuperTypes",0,-1,Jl,!1,!1,!0,!0,!1,!0,!1),Zr(h(he(De(e.o),15),18),e.H,null,"eAllGenericSuperTypes",0,-1,Jl,!0,!0,!1,!1,!0,!1,!0),v=ma(h(he(Gr(e.o),0),59),e.e,"isSuperTypeOf"),Ma(v,e.o,"someClass"),ma(h(he(Gr(e.o),1),59),e.I,"getFeatureCount"),v=ma(h(he(Gr(e.o),2),59),e.bb,vyt),Ma(v,e.I,"featureID"),v=ma(h(he(Gr(e.o),3),59),e.I,wyt),Ma(v,e.bb,u_),v=ma(h(he(Gr(e.o),4),59),e.bb,vyt),Ma(v,e._,"featureName"),ma(h(he(Gr(e.o),5),59),e.I,"getOperationCount"),v=ma(h(he(Gr(e.o),6),59),e.T,"getEOperation"),Ma(v,e.I,"operationID"),v=ma(h(he(Gr(e.o),7),59),e.I,myt),Ma(v,e.T,t3e),v=ma(h(he(Gr(e.o),8),59),e.T,"getOverride"),Ma(v,e.T,t3e),v=ma(h(he(Gr(e.o),9),59),e.H,"getFeatureType"),Ma(v,e.bb,u_),io(e.p,af,ayt,!0,!1,!0),Ci(h(he(De(e.p),0),34),e._,"instanceClassName",null,0,1,af,!1,!0,!0,!0,!0,!1),t=xd(e.L),n=f0e(),qn((!t.d&&(t.d=new Gi(pa,t,1)),t.d),n),O0t(h(he(De(e.p),1),34),t,"instanceClass",af,!0,!0,!1,!0),Ci(h(he(De(e.p),2),34),e.M,yyt,null,0,1,af,!0,!0,!1,!1,!0,!0),Ci(h(he(De(e.p),3),34),e._,"instanceTypeName",null,0,1,af,!1,!0,!0,!0,!0,!1),Zr(h(he(De(e.p),4),18),e.U,h(he(De(e.U),3),18),"ePackage",0,1,af,!0,!1,!1,!1,!0,!1,!1),Zr(h(he(De(e.p),5),18),e.db,null,kyt,0,-1,af,!1,!1,!0,!0,!0,!1,!1),v=ma(h(he(Gr(e.p),0),59),e.e,xyt),Ma(v,e.M,LM),ma(h(he(Gr(e.p),1),59),e.I,"getClassifierID"),io(e.q,rie,"EDataType",!1,!1,!0),Ci(h(he(De(e.q),0),34),e.e,"serializable",Nx,0,1,rie,!1,!1,!0,!1,!0,!1),io(e.v,QH,"EEnum",!1,!1,!0),Zr(h(he(De(e.v),0),18),e.w,h(he(De(e.w),3),18),"eLiterals",0,-1,QH,!1,!1,!0,!0,!1,!1,!1),v=ma(h(he(Gr(e.v),0),59),e.w,Eyt),Ma(v,e._,_r),v=ma(h(he(Gr(e.v),1),59),e.w,Eyt),Ma(v,e.I,t_),v=ma(h(he(Gr(e.v),2),59),e.w,"getEEnumLiteralByLiteral"),Ma(v,e._,"literal"),io(e.w,qd,oyt,!1,!1,!0),Ci(h(he(De(e.w),0),34),e.I,t_,null,0,1,qd,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.w),1),34),e.A,"instance",null,0,1,qd,!0,!1,!0,!1,!0,!1),Ci(h(he(De(e.w),2),34),e._,"literal",null,0,1,qd,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.w),3),18),e.v,h(he(De(e.v),0),18),"eEnum",0,1,qd,!0,!1,!1,!1,!1,!1,!1),io(e.B,cT,"EFactory",!1,!1,!0),Zr(h(he(De(e.B),0),18),e.U,h(he(De(e.U),2),18),"ePackage",1,1,cT,!0,!1,!0,!1,!1,!1,!1),v=ma(h(he(Gr(e.B),0),59),e.S,"create"),Ma(v,e.o,"eClass"),v=ma(h(he(Gr(e.B),1),59),e.M,"createFromString"),Ma(v,e.q,"eDataType"),Ma(v,e._,"literalValue"),v=ma(h(he(Gr(e.B),2),59),e._,"convertToString"),Ma(v,e.q,"eDataType"),Ma(v,e.M,"instanceValue"),io(e.Q,dL,omt,!0,!1,!0),Zr(h(he(De(e.Q),0),18),e.a,h(he(De(e.a),2),18),"eAnnotations",0,-1,dL,!1,!1,!0,!0,!1,!1,!1),v=ma(h(he(Gr(e.Q),0),59),e.a,"getEAnnotation"),Ma(v,e._,vee),io(e.R,Qre,cmt,!0,!1,!0),Ci(h(he(De(e.R),0),34),e._,_r,null,0,1,Qre,!1,!1,!0,!1,!0,!1),io(e.S,Jg,"EObject",!1,!1,!0),ma(h(he(Gr(e.S),0),59),e.o,"eClass"),ma(h(he(Gr(e.S),1),59),e.e,"eIsProxy"),ma(h(he(Gr(e.S),2),59),e.X,"eResource"),ma(h(he(Gr(e.S),3),59),e.S,"eContainer"),ma(h(he(Gr(e.S),4),59),e.bb,"eContainingFeature"),ma(h(he(Gr(e.S),5),59),e.W,"eContainmentFeature"),v=ma(h(he(Gr(e.S),6),59),null,"eContents"),t=xd(e.fb),n=xd(e.S),qn((!t.d&&(t.d=new Gi(pa,t,1)),t.d),n),a=Df(v,t,null),a&&a.Fi(),v=ma(h(he(Gr(e.S),7),59),null,"eAllContents"),t=xd(e.cb),n=xd(e.S),qn((!t.d&&(t.d=new Gi(pa,t,1)),t.d),n),u=Df(v,t,null),u&&u.Fi(),v=ma(h(he(Gr(e.S),8),59),null,"eCrossReferences"),t=xd(e.fb),n=xd(e.S),qn((!t.d&&(t.d=new Gi(pa,t,1)),t.d),n),p=Df(v,t,null),p&&p.Fi(),v=ma(h(he(Gr(e.S),9),59),e.M,"eGet"),Ma(v,e.bb,u_),v=ma(h(he(Gr(e.S),10),59),e.M,"eGet"),Ma(v,e.bb,u_),Ma(v,e.e,"resolve"),v=ma(h(he(Gr(e.S),11),59),null,"eSet"),Ma(v,e.bb,u_),Ma(v,e.M,"newValue"),v=ma(h(he(Gr(e.S),12),59),e.e,"eIsSet"),Ma(v,e.bb,u_),v=ma(h(he(Gr(e.S),13),59),null,"eUnset"),Ma(v,e.bb,u_),v=ma(h(he(Gr(e.S),14),59),e.M,"eInvoke"),Ma(v,e.T,t3e),t=xd(e.fb),n=f0e(),qn((!t.d&&(t.d=new Gi(pa,t,1)),t.d),n),GF(v,t,"arguments"),fvn(v,e.K),io(e.T,eh,uyt,!1,!1,!0),Zr(h(he(De(e.T),0),18),e.o,h(he(De(e.o),3),18),_yt,0,1,eh,!0,!1,!1,!1,!1,!1,!1),Zr(h(he(De(e.T),1),18),e.db,null,kyt,0,-1,eh,!1,!1,!0,!0,!0,!1,!1),Zr(h(he(De(e.T),2),18),e.V,h(he(De(e.V),0),18),"eParameters",0,-1,eh,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.T),3),18),e.p,null,"eExceptions",0,-1,eh,!1,!1,!0,!1,!0,!0,!1),Zr(h(he(De(e.T),4),18),e.H,null,"eGenericExceptions",0,-1,eh,!1,!1,!0,!0,!1,!0,!1),ma(h(he(Gr(e.T),0),59),e.I,myt),v=ma(h(he(Gr(e.T),1),59),e.e,"isOverrideOf"),Ma(v,e.T,"someOperation"),io(e.U,sf,"EPackage",!1,!1,!0),Ci(h(he(De(e.U),0),34),e._,"nsURI",null,0,1,sf,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.U),1),34),e._,"nsPrefix",null,0,1,sf,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.U),2),18),e.B,h(he(De(e.B),0),18),"eFactoryInstance",1,1,sf,!0,!1,!0,!1,!1,!1,!1),Zr(h(he(De(e.U),3),18),e.p,h(he(De(e.p),4),18),"eClassifiers",0,-1,sf,!1,!1,!0,!0,!0,!1,!1),Zr(h(he(De(e.U),4),18),e.U,h(he(De(e.U),5),18),"eSubpackages",0,-1,sf,!1,!1,!0,!0,!0,!1,!1),Zr(h(he(De(e.U),5),18),e.U,h(he(De(e.U),4),18),"eSuperPackage",0,1,sf,!0,!1,!1,!1,!0,!1,!1),v=ma(h(he(Gr(e.U),0),59),e.p,"getEClassifier"),Ma(v,e._,_r),io(e.V,dv,lyt,!1,!1,!0),Zr(h(he(De(e.V),0),18),e.T,h(he(De(e.T),2),18),"eOperation",0,1,dv,!0,!1,!1,!1,!1,!1,!1),io(e.W,mm,hyt,!1,!1,!0),Ci(h(he(De(e.W),0),34),e.e,"containment",null,0,1,mm,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.W),1),34),e.e,"container",null,0,1,mm,!0,!0,!1,!1,!0,!0),Ci(h(he(De(e.W),2),34),e.e,"resolveProxies",Nx,0,1,mm,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.W),3),18),e.W,null,"eOpposite",0,1,mm,!1,!1,!0,!1,!0,!1,!1),Zr(h(he(De(e.W),4),18),e.o,null,"eReferenceType",1,1,mm,!0,!0,!1,!1,!0,!1,!0),Zr(h(he(De(e.W),5),18),e.b,null,"eKeys",0,-1,mm,!1,!1,!0,!1,!0,!1,!1),io(e.bb,Ac,iyt,!0,!1,!0),Ci(h(he(De(e.bb),0),34),e.e,"changeable",Nx,0,1,Ac,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.bb),1),34),e.e,"volatile",null,0,1,Ac,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.bb),2),34),e.e,"transient",null,0,1,Ac,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.bb),3),34),e._,"defaultValueLiteral",null,0,1,Ac,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.bb),4),34),e.M,yyt,null,0,1,Ac,!0,!0,!1,!1,!0,!0),Ci(h(he(De(e.bb),5),34),e.e,"unsettable",null,0,1,Ac,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.bb),6),34),e.e,"derived",null,0,1,Ac,!1,!1,!0,!1,!0,!1),Zr(h(he(De(e.bb),7),18),e.o,h(he(De(e.o),13),18),_yt,0,1,Ac,!0,!1,!1,!1,!1,!1,!1),ma(h(he(Gr(e.bb),0),59),e.I,wyt),v=ma(h(he(Gr(e.bb),1),59),null,"getContainerClass"),t=xd(e.L),n=f0e(),qn((!t.d&&(t.d=new Gi(pa,t,1)),t.d),n),s=Df(v,t,null),s&&s.Fi(),io(e.eb,vm,ryt,!0,!1,!0),Ci(h(he(De(e.eb),0),34),e.e,"ordered",Nx,0,1,vm,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.eb),1),34),e.e,"unique",Nx,0,1,vm,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.eb),2),34),e.I,"lowerBound",null,0,1,vm,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.eb),3),34),e.I,"upperBound","1",0,1,vm,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.eb),4),34),e.e,"many",null,0,1,vm,!0,!0,!1,!1,!0,!0),Ci(h(he(De(e.eb),5),34),e.e,"required",null,0,1,vm,!0,!0,!1,!1,!0,!0),Zr(h(he(De(e.eb),6),18),e.p,null,"eType",0,1,vm,!1,!0,!0,!1,!0,!0,!1),Zr(h(he(De(e.eb),7),18),e.H,null,"eGenericType",0,1,vm,!1,!0,!0,!0,!1,!0,!1),io(e.ab,c2,"EStringToStringMapEntry",!1,!1,!1),Ci(h(he(De(e.ab),0),34),e._,"key",null,0,1,c2,!1,!1,!0,!1,!0,!1),Ci(h(he(De(e.ab),1),34),e._,t_,null,0,1,c2,!1,!1,!0,!1,!0,!1),io(e.H,pa,cyt,!1,!1,!0),Zr(h(he(De(e.H),0),18),e.H,null,"eUpperBound",0,1,pa,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.H),1),18),e.H,null,"eTypeArguments",0,-1,pa,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.H),2),18),e.p,null,"eRawType",1,1,pa,!0,!1,!1,!1,!0,!1,!0),Zr(h(he(De(e.H),3),18),e.H,null,"eLowerBound",0,1,pa,!1,!1,!0,!0,!1,!1,!1),Zr(h(he(De(e.H),4),18),e.db,null,"eTypeParameter",0,1,pa,!1,!1,!0,!1,!1,!1,!1),Zr(h(he(De(e.H),5),18),e.p,null,"eClassifier",0,1,pa,!1,!1,!0,!1,!0,!1,!1),v=ma(h(he(Gr(e.H),0),59),e.e,xyt),Ma(v,e.M,LM),io(e.db,lc,fyt,!1,!1,!0),Zr(h(he(De(e.db),0),18),e.H,null,"eBounds",0,-1,lc,!1,!1,!0,!0,!1,!1,!1),yr(e.c,rte,"EBigDecimal",!0),yr(e.d,t3,"EBigInteger",!0),yr(e.e,vu,"EBoolean",!0),yr(e.f,es,"EBooleanObject",!0),yr(e.i,Vc,"EByte",!0),yr(e.g,ie(Vc,1),"EByteArray",!0),yr(e.j,A5,"EByteObject",!0),yr(e.k,Cl,"EChar",!0),yr(e.n,f_,"ECharacterObject",!0),yr(e.r,Bj,"EDate",!0),yr(e.s,YEe,"EDiagnosticChain",!1),yr(e.t,ks,"EDouble",!0),yr(e.u,Ts,"EDoubleObject",!0),yr(e.fb,cEe,"EEList",!1),yr(e.A,hEe,"EEnumerator",!1),yr(e.C,NEe,"EFeatureMap",!1),yr(e.D,ZH,"EFeatureMapEntry",!1),yr(e.F,Em,"EFloat",!0),yr(e.G,zx,"EFloatObject",!0),yr(e.I,Hn,"EInt",!0),yr(e.J,Ws,"EIntegerObject",!0),yr(e.L,f3e,"EJavaClass",!0),yr(e.M,yn,"EJavaObject",!0),yr(e.N,ap,"ELong",!0),yr(e.O,Xb,"ELongObject",!0),yr(e.P,d3e,"EMap",!1),yr(e.X,DEe,"EResource",!1),yr(e.Y,WEe,"EResourceSet",!1),yr(e.Z,D3,"EShort",!0),yr(e.$,Qb,"EShortObject",!0),yr(e._,it,"EString",!0),yr(e.cb,lEe,"ETreeIterator",!1),yr(e.K,KEe,"EInvocationTargetException",!1),ope(e,Vl))}var LM="object",g5="boolean",Wve="number",hZ="string",fZ="function",Ir=2147483647,Ja="java.lang",IM={3:1},OM="com.google.common.base",Js=", ",Obt="%s (%s) must not be negative",st={3:1,4:1,5:1},Nbt="negative size: ",Pbt="Optional.of(",_c="null",mx={198:1,47:1},mt="com.google.common.collect",yx={198:1,47:1,125:1},Zp={224:1,3:1},ys={47:1},Nn="java.util",zb={83:1},Rw={20:1,28:1,14:1},Lf=1965,$c={20:1,28:1,14:1,21:1},Bbt={83:1,171:1,161:1},Fbt={20:1,28:1,14:1,21:1,84:1},Kve={20:1,28:1,14:1,271:1,21:1,84:1},z1={47:1,125:1},ER={345:1,42:1},Rbt="AbstractMapEntry",jbt="expectedValuesPerKey",Re={3:1,6:1,4:1,5:1},a1=16384,jl={164:1},mn={38:1},_R={l:4194303,m:4194303,h:524287},NM={196:1},dZ={245:1,3:1,35:1},$bt="range unbounded on this side",G1={20:1},Hbt={20:1,14:1},Xve={3:1,20:1,28:1,14:1},O9={152:1,3:1,20:1,28:1,14:1,15:1,54:1},TR={3:1,4:1,5:1,165:1},kx={3:1,83:1},gZ={20:1,14:1,21:1},xx={3:1,20:1,28:1,14:1,21:1},zbt={20:1,14:1,21:1,84:1},V1=461845907,U1=-862048943,PM={3:1,6:1,4:1,5:1,165:1},Gbt="expectedSize",N9=1073741824,jw="initialArraySize",ze={3:1,6:1,4:1,9:1,5:1},Ex={20:1,28:1,52:1,14:1,15:1},pZ="arraySize",Vbt={20:1,28:1,52:1,14:1,15:1,54:1},Cr={45:1},CR={365:1},Ig=1e-4,$s=-2147483648,Ubt="__noinit__",Fd={3:1,102:1,60:1,78:1},BM="com.google.gwt.core.client.impl",Qve="String",Zve="com.google.gwt.core.client",bZ="anonymous",vZ="fnStack",Jve="Unknown",o1={195:1,3:1,4:1},_0=1e3,Oi=65535,wZ="January",mZ="February",yZ="March",kZ="April",p5="May",xZ="June",EZ="July",_Z="August",TZ="September",CZ="October",SZ="November",AZ="December",Og=1900,hr={48:1,3:1,4:1},qbt="Before Christ",Ybt="Anno Domini",MZ="Sunday",DZ="Monday",LZ="Tuesday",IZ="Wednesday",OZ="Thursday",NZ="Friday",PZ="Saturday",ewe="com.google.gwt.i18n.shared",Wbt="DateTimeFormat",BZ="com.google.gwt.i18n.client",Kbt="DefaultDateTimeFormatInfo",Xbt={3:1,4:1,35:1,199:1},b5="com.google.gwt.json.client",du=4194303,Rd=1048575,FM=524288,v5=4194304,Jp=17592186044416,SR=1e9,RM=-17592186044416,twe="java.io",FZ={3:1,102:1,73:1,60:1,78:1},Qbt={3:1,289:1,78:1},Gb='For input string: "',ki=1/0,Fi=-1/0,$w=4096,RZ={3:1,4:1,364:1},ea=65536,jM=55296,ka={104:1,3:1,4:1},jZ=1e5,Zbt=.3010299956639812,fa=4294967295,$Z=4294967296,_x="0.0",HZ={42:1},Jbt={3:1,4:1,20:1,28:1,52:1,12:1,14:1,15:1,54:1},evt={3:1,20:1,28:1,52:1,14:1,15:1,54:1},tvt={20:1,14:1,15:1},zZ={3:1,62:1},$M={182:1},Vy={3:1,4:1,83:1},nwe={3:1,4:1,20:1,28:1,14:1,53:1,21:1},GZ="delete",P9=14901161193847656e-24,B9=11102230246251565e-32,VZ=15525485,HM=5960464477539063e-23,rwe=16777216,AR=16777215,iwe=", length: ",nvt={3:1,4:1,20:1,28:1,52:1,14:1,15:1,54:1},UZ={3:1,35:1,22:1,297:1},qZ="java.util.function",F9="java.util.logging",rvt={3:1,4:1,5:1,842:1},YZ="undefined",Yi="java.util.stream",swe={525:1,670:1},MR="fromIndex: ",ivt=" > toIndex: ",awe=", toIndex: ",owe="Index: ",cwe=", Size: ",Tx="org.eclipse.elk.alg.common",Yr={62:1},svt="org.eclipse.elk.alg.common.compaction",avt="Scanline/EventHandler",q1="org.eclipse.elk.alg.common.compaction.oned",ovt="CNode belongs to another CGroup.",cvt="ISpacingsHandler/1",WZ="The ",KZ=" instance has been finished already.",uvt="The direction ",lvt=" is not supported by the CGraph instance.",hvt="OneDimensionalCompactor",fvt="OneDimensionalCompactor/lambda$0$Type",dvt="Quadruplet",gvt="ScanlineConstraintCalculator",pvt="ScanlineConstraintCalculator/ConstraintsScanlineHandler",bvt="ScanlineConstraintCalculator/ConstraintsScanlineHandler/lambda$0$Type",vvt="ScanlineConstraintCalculator/Timestamp",wvt="ScanlineConstraintCalculator/lambda$0$Type",c1={169:1,45:1},XZ="org.eclipse.elk.alg.common.compaction.options",eo="org.eclipse.elk.core.data",uwe="org.eclipse.elk.polyomino.traversalStrategy",lwe="org.eclipse.elk.polyomino.lowLevelSort",hwe="org.eclipse.elk.polyomino.highLevelSort",fwe="org.eclipse.elk.polyomino.fill",$l={130:1},QZ="polyomino",R9="org.eclipse.elk.alg.common.networksimplex",Y1={177:1,3:1,4:1},mvt="org.eclipse.elk.alg.common.nodespacing",e2="org.eclipse.elk.alg.common.nodespacing.cellsystem",Cx="CENTER",yvt={212:1,326:1},dwe={3:1,4:1,5:1,595:1},w5="LEFT",m5="RIGHT",gwe="Vertical alignment cannot be null",pwe="BOTTOM",DR="org.eclipse.elk.alg.common.nodespacing.internal",j9="UNDEFINED",If=.01,zM="org.eclipse.elk.alg.common.nodespacing.internal.algorithm",kvt="LabelPlacer/lambda$0$Type",xvt="LabelPlacer/lambda$1$Type",Evt="portRatioOrPosition",Sx="org.eclipse.elk.alg.common.overlaps",ZZ="DOWN",u1="org.eclipse.elk.alg.common.polyomino",LR="NORTH",JZ="EAST",eJ="SOUTH",tJ="WEST",IR="org.eclipse.elk.alg.common.polyomino.structures",bwe="Direction",nJ="Grid is only of size ",rJ=". Requested point (",iJ=") is out of bounds.",OR=" Given center based coordinates were (",GM="org.eclipse.elk.graph.properties",_vt="IPropertyHolder",vwe={3:1,94:1,134:1},y5="org.eclipse.elk.alg.common.spore",Tvt="org.eclipse.elk.alg.common.utils",t2={209:1},Uy="org.eclipse.elk.core",Cvt="Connected Components Compaction",Svt="org.eclipse.elk.alg.disco",NR="org.eclipse.elk.alg.disco.graph",sJ="org.eclipse.elk.alg.disco.options",wwe="CompactionStrategy",mwe="org.eclipse.elk.disco.componentCompaction.strategy",ywe="org.eclipse.elk.disco.componentCompaction.componentLayoutAlgorithm",kwe="org.eclipse.elk.disco.debug.discoGraph",xwe="org.eclipse.elk.disco.debug.discoPolys",Avt="componentCompaction",n2="org.eclipse.elk.disco",aJ="org.eclipse.elk.spacing.componentComponent",oJ="org.eclipse.elk.edge.thickness",k5="org.eclipse.elk.aspectRatio",Vb="org.eclipse.elk.padding",qy="org.eclipse.elk.alg.disco.transform",cJ=1.5707963267948966,Ax=17976931348623157e292,Hw={3:1,4:1,5:1,192:1},Ewe={3:1,6:1,4:1,5:1,106:1,120:1},_we="org.eclipse.elk.alg.force",Twe="ComponentsProcessor",Mvt="ComponentsProcessor/1",VM="org.eclipse.elk.alg.force.graph",Dvt="Component Layout",Cwe="org.eclipse.elk.alg.force.model",PR="org.eclipse.elk.force.model",Swe="org.eclipse.elk.force.iterations",Awe="org.eclipse.elk.force.repulsivePower",uJ="org.eclipse.elk.force.temperature",l1=.001,lJ="org.eclipse.elk.force.repulsion",$9="org.eclipse.elk.alg.force.options",Mx=1.600000023841858,Xu="org.eclipse.elk.force",UM="org.eclipse.elk.priority",zw="org.eclipse.elk.spacing.nodeNode",hJ="org.eclipse.elk.spacing.edgeLabel",BR="org.eclipse.elk.randomSeed",H9="org.eclipse.elk.separateConnectedComponents",qM="org.eclipse.elk.interactive",fJ="org.eclipse.elk.portConstraints",FR="org.eclipse.elk.edgeLabels.inline",z9="org.eclipse.elk.omitNodeMicroLayout",x5="org.eclipse.elk.nodeSize.options",Yy="org.eclipse.elk.nodeSize.constraints",Dx="org.eclipse.elk.nodeLabels.placement",Lx="org.eclipse.elk.portLabels.placement",Mwe="origin",Dwe="random",Lvt="boundingBox.upLeft",Ivt="boundingBox.lowRight",Lwe="org.eclipse.elk.stress.fixed",Iwe="org.eclipse.elk.stress.desiredEdgeLength",Owe="org.eclipse.elk.stress.dimension",Nwe="org.eclipse.elk.stress.epsilon",Pwe="org.eclipse.elk.stress.iterationLimit",Ng="org.eclipse.elk.stress",Ovt="ELK Stress",E5="org.eclipse.elk.nodeSize.minimum",RR="org.eclipse.elk.alg.force.stress",Nvt="Layered layout",_5="org.eclipse.elk.alg.layered",YM="org.eclipse.elk.alg.layered.compaction.components",G9="org.eclipse.elk.alg.layered.compaction.oned",jR="org.eclipse.elk.alg.layered.compaction.oned.algs",r2="org.eclipse.elk.alg.layered.compaction.recthull",h1="org.eclipse.elk.alg.layered.components",jd="NONE",yo={3:1,6:1,4:1,9:1,5:1,122:1},Pvt={3:1,6:1,4:1,5:1,141:1,106:1,120:1},$R="org.eclipse.elk.alg.layered.compound",xi={51:1},Zo="org.eclipse.elk.alg.layered.graph",dJ=" -> ",Bvt="Not supported by LGraph",Bwe="Port side is undefined",gJ={3:1,6:1,4:1,5:1,474:1,141:1,106:1,120:1},T0={3:1,6:1,4:1,5:1,141:1,193:1,203:1,106:1,120:1},Fvt={3:1,6:1,4:1,5:1,141:1,1943:1,203:1,106:1,120:1},Rvt=`([{"' 	\r
`,jvt=`)]}"' 	\r
`,$vt="The given string contains parts that cannot be parsed as numbers.",WM="org.eclipse.elk.core.math",Hvt={3:1,4:1,142:1,207:1,414:1},zvt={3:1,4:1,116:1,207:1,414:1},wn="org.eclipse.elk.layered",C0="org.eclipse.elk.alg.layered.graph.transform",Gvt="ElkGraphImporter",Vvt="ElkGraphImporter/lambda$0$Type",Uvt="ElkGraphImporter/lambda$1$Type",qvt="ElkGraphImporter/lambda$2$Type",Yvt="ElkGraphImporter/lambda$4$Type",Wvt="Node margin calculation",fn="org.eclipse.elk.alg.layered.intermediate",Kvt="ONE_SIDED_GREEDY_SWITCH",Xvt="TWO_SIDED_GREEDY_SWITCH",pJ="No implementation is available for the layout processor ",Fwe="IntermediateProcessorStrategy",bJ="Node '",Qvt="FIRST_SEPARATE",Zvt="LAST_SEPARATE",Jvt="Odd port side processing",Ri="org.eclipse.elk.alg.layered.intermediate.compaction",V9="org.eclipse.elk.alg.layered.intermediate.greedyswitch",W1="org.eclipse.elk.alg.layered.p3order.counting",KM={225:1},T5="org.eclipse.elk.alg.layered.intermediate.loops",Qu="org.eclipse.elk.alg.layered.intermediate.loops.ordering",Pg="org.eclipse.elk.alg.layered.intermediate.loops.routing",U9="org.eclipse.elk.alg.layered.intermediate.preserveorder",f1="org.eclipse.elk.alg.layered.intermediate.wrapping",ko="org.eclipse.elk.alg.layered.options",vJ="INTERACTIVE",ewt="DEPTH_FIRST",twt="EDGE_LENGTH",nwt="SELF_LOOPS",rwt="firstTryWithInitialOrder",Rwe="org.eclipse.elk.layered.directionCongruency",jwe="org.eclipse.elk.layered.feedbackEdges",HR="org.eclipse.elk.layered.interactiveReferencePoint",$we="org.eclipse.elk.layered.mergeEdges",Hwe="org.eclipse.elk.layered.mergeHierarchyEdges",zwe="org.eclipse.elk.layered.allowNonFlowPortsToSwitchSides",Gwe="org.eclipse.elk.layered.portSortingStrategy",Vwe="org.eclipse.elk.layered.thoroughness",Uwe="org.eclipse.elk.layered.unnecessaryBendpoints",qwe="org.eclipse.elk.layered.generatePositionAndLayerIds",wJ="org.eclipse.elk.layered.cycleBreaking.strategy",XM="org.eclipse.elk.layered.layering.strategy",Ywe="org.eclipse.elk.layered.layering.layerConstraint",Wwe="org.eclipse.elk.layered.layering.layerChoiceConstraint",Kwe="org.eclipse.elk.layered.layering.layerId",mJ="org.eclipse.elk.layered.layering.minWidth.upperBoundOnWidth",yJ="org.eclipse.elk.layered.layering.minWidth.upperLayerEstimationScalingFactor",kJ="org.eclipse.elk.layered.layering.nodePromotion.strategy",xJ="org.eclipse.elk.layered.layering.nodePromotion.maxIterations",EJ="org.eclipse.elk.layered.layering.coffmanGraham.layerBound",q9="org.eclipse.elk.layered.crossingMinimization.strategy",Xwe="org.eclipse.elk.layered.crossingMinimization.forceNodeModelOrder",_J="org.eclipse.elk.layered.crossingMinimization.hierarchicalSweepiness",TJ="org.eclipse.elk.layered.crossingMinimization.semiInteractive",Qwe="org.eclipse.elk.layered.crossingMinimization.positionChoiceConstraint",Zwe="org.eclipse.elk.layered.crossingMinimization.positionId",Jwe="org.eclipse.elk.layered.crossingMinimization.greedySwitch.activationThreshold",CJ="org.eclipse.elk.layered.crossingMinimization.greedySwitch.type",zR="org.eclipse.elk.layered.crossingMinimization.greedySwitchHierarchical.type",Wy="org.eclipse.elk.layered.nodePlacement.strategy",GR="org.eclipse.elk.layered.nodePlacement.favorStraightEdges",SJ="org.eclipse.elk.layered.nodePlacement.bk.edgeStraightening",AJ="org.eclipse.elk.layered.nodePlacement.bk.fixedAlignment",MJ="org.eclipse.elk.layered.nodePlacement.linearSegments.deflectionDampening",DJ="org.eclipse.elk.layered.nodePlacement.networkSimplex.nodeFlexibility",LJ="org.eclipse.elk.layered.nodePlacement.networkSimplex.nodeFlexibility.default",eme="org.eclipse.elk.layered.edgeRouting.selfLoopDistribution",tme="org.eclipse.elk.layered.edgeRouting.selfLoopOrdering",VR="org.eclipse.elk.layered.edgeRouting.splines.mode",UR="org.eclipse.elk.layered.edgeRouting.splines.sloppy.layerSpacingFactor",IJ="org.eclipse.elk.layered.edgeRouting.polyline.slopedEdgeZoneWidth",nme="org.eclipse.elk.layered.spacing.baseValue",rme="org.eclipse.elk.layered.spacing.edgeNodeBetweenLayers",ime="org.eclipse.elk.layered.spacing.edgeEdgeBetweenLayers",sme="org.eclipse.elk.layered.spacing.nodeNodeBetweenLayers",ame="org.eclipse.elk.layered.priority.direction",ome="org.eclipse.elk.layered.priority.shortness",cme="org.eclipse.elk.layered.priority.straightness",OJ="org.eclipse.elk.layered.compaction.connectedComponents",ume="org.eclipse.elk.layered.compaction.postCompaction.strategy",lme="org.eclipse.elk.layered.compaction.postCompaction.constraints",qR="org.eclipse.elk.layered.highDegreeNodes.treatment",NJ="org.eclipse.elk.layered.highDegreeNodes.threshold",PJ="org.eclipse.elk.layered.highDegreeNodes.treeHeight",$d="org.eclipse.elk.layered.wrapping.strategy",YR="org.eclipse.elk.layered.wrapping.additionalEdgeSpacing",WR="org.eclipse.elk.layered.wrapping.correctionFactor",Y9="org.eclipse.elk.layered.wrapping.cutting.strategy",BJ="org.eclipse.elk.layered.wrapping.cutting.cuts",FJ="org.eclipse.elk.layered.wrapping.cutting.msd.freedom",KR="org.eclipse.elk.layered.wrapping.validify.strategy",XR="org.eclipse.elk.layered.wrapping.validify.forbiddenIndices",QR="org.eclipse.elk.layered.wrapping.multiEdge.improveCuts",ZR="org.eclipse.elk.layered.wrapping.multiEdge.distancePenalty",RJ="org.eclipse.elk.layered.wrapping.multiEdge.improveWrappedEdges",hme="org.eclipse.elk.layered.edgeLabels.sideSelection",fme="org.eclipse.elk.layered.edgeLabels.centerLabelPlacementStrategy",JR="org.eclipse.elk.layered.considerModelOrder.strategy",dme="org.eclipse.elk.layered.considerModelOrder.noModelOrder",jJ="org.eclipse.elk.layered.considerModelOrder.components",gme="org.eclipse.elk.layered.considerModelOrder.longEdgeStrategy",$J="org.eclipse.elk.layered.considerModelOrder.crossingCounterNodeInfluence",HJ="org.eclipse.elk.layered.considerModelOrder.crossingCounterPortInfluence",zJ="layering",iwt="layering.minWidth",swt="layering.nodePromotion",QM="crossingMinimization",ej="org.eclipse.elk.hierarchyHandling",awt="crossingMinimization.greedySwitch",owt="nodePlacement",cwt="nodePlacement.bk",uwt="edgeRouting",ZM="org.eclipse.elk.edgeRouting",Of="spacing",pme="priority",bme="compaction",lwt="compaction.postCompaction",hwt="Specifies whether and how post-process compaction is applied.",vme="highDegreeNodes",wme="wrapping",fwt="wrapping.cutting",dwt="wrapping.validify",mme="wrapping.multiEdge",GJ="edgeLabels",JM="considerModelOrder",yme="org.eclipse.elk.spacing.commentComment",kme="org.eclipse.elk.spacing.commentNode",xme="org.eclipse.elk.spacing.edgeEdge",Eme="org.eclipse.elk.spacing.edgeNode",_me="org.eclipse.elk.spacing.labelLabel",Tme="org.eclipse.elk.spacing.labelPortHorizontal",Cme="org.eclipse.elk.spacing.labelPortVertical",Sme="org.eclipse.elk.spacing.labelNode",Ame="org.eclipse.elk.spacing.nodeSelfLoop",Mme="org.eclipse.elk.spacing.portPort",Dme="org.eclipse.elk.spacing.individual",Lme="org.eclipse.elk.port.borderOffset",Ime="org.eclipse.elk.noLayout",Ome="org.eclipse.elk.port.side",eD="org.eclipse.elk.debugMode",Nme="org.eclipse.elk.alignment",Pme="org.eclipse.elk.insideSelfLoops.activate",Bme="org.eclipse.elk.insideSelfLoops.yo",VJ="org.eclipse.elk.nodeSize.fixedGraphSize",Fme="org.eclipse.elk.direction",Rme="org.eclipse.elk.nodeLabels.padding",jme="org.eclipse.elk.portLabels.nextToPortIfPossible",$me="org.eclipse.elk.portLabels.treatAsGroup",Hme="org.eclipse.elk.portAlignment.default",zme="org.eclipse.elk.portAlignment.north",Gme="org.eclipse.elk.portAlignment.south",Vme="org.eclipse.elk.portAlignment.west",Ume="org.eclipse.elk.portAlignment.east",tj="org.eclipse.elk.contentAlignment",qme="org.eclipse.elk.junctionPoints",Yme="org.eclipse.elk.edgeLabels.placement",Wme="org.eclipse.elk.port.index",Kme="org.eclipse.elk.commentBox",Xme="org.eclipse.elk.hypernode",Qme="org.eclipse.elk.port.anchor",UJ="org.eclipse.elk.partitioning.activate",qJ="org.eclipse.elk.partitioning.partition",nj="org.eclipse.elk.position",Zme="org.eclipse.elk.margins",Jme="org.eclipse.elk.spacing.portsSurrounding",YJ="org.eclipse.elk.interactiveLayout",xo="org.eclipse.elk.core.util",eye={3:1,4:1,5:1,593:1},gwt="NETWORK_SIMPLEX",zo={123:1,51:1},rj="org.eclipse.elk.alg.layered.p1cycles",Gw="org.eclipse.elk.alg.layered.p2layers",tye={402:1,225:1},pwt={832:1,3:1,4:1},Hc="org.eclipse.elk.alg.layered.p3order",da="org.eclipse.elk.alg.layered.p4nodes",bwt={3:1,4:1,5:1,840:1},d1=1e-5,Bg="org.eclipse.elk.alg.layered.p4nodes.bk",WJ="org.eclipse.elk.alg.layered.p5edges",ef="org.eclipse.elk.alg.layered.p5edges.orthogonal",KJ="org.eclipse.elk.alg.layered.p5edges.orthogonal.direction",XJ=1e-6,Vw="org.eclipse.elk.alg.layered.p5edges.splines",QJ=.09999999999999998,ij=1e-8,vwt=4.71238898038469,wwt=3.141592653589793,W9="org.eclipse.elk.alg.mrtree",K9="org.eclipse.elk.alg.mrtree.graph",C5="org.eclipse.elk.alg.mrtree.intermediate",mwt="Set neighbors in level",ywt="DESCENDANTS",nye="org.eclipse.elk.mrtree.weighting",rye="org.eclipse.elk.mrtree.searchOrder",sj="org.eclipse.elk.alg.mrtree.options",S0="org.eclipse.elk.mrtree",kwt="org.eclipse.elk.tree",iye="org.eclipse.elk.alg.radial",Ky=6.283185307179586,sye=5e-324,xwt="org.eclipse.elk.alg.radial.intermediate",ZJ="org.eclipse.elk.alg.radial.intermediate.compaction",Ewt={3:1,4:1,5:1,106:1},aye="org.eclipse.elk.alg.radial.intermediate.optimization",JJ="No implementation is available for the layout option ",X9="org.eclipse.elk.alg.radial.options",oye="org.eclipse.elk.radial.orderId",cye="org.eclipse.elk.radial.radius",eee="org.eclipse.elk.radial.compactor",tee="org.eclipse.elk.radial.compactionStepSize",uye="org.eclipse.elk.radial.sorter",lye="org.eclipse.elk.radial.wedgeCriteria",hye="org.eclipse.elk.radial.optimizationCriteria",g1="org.eclipse.elk.radial",_wt="org.eclipse.elk.alg.radial.p1position.wedge",fye="org.eclipse.elk.alg.radial.sorting",Twt=5.497787143782138,Cwt=3.9269908169872414,Swt=2.356194490192345,Awt="org.eclipse.elk.alg.rectpacking",aj="org.eclipse.elk.alg.rectpacking.firstiteration",nee="org.eclipse.elk.alg.rectpacking.options",dye="org.eclipse.elk.rectpacking.optimizationGoal",gye="org.eclipse.elk.rectpacking.lastPlaceShift",pye="org.eclipse.elk.rectpacking.currentPosition",bye="org.eclipse.elk.rectpacking.desiredPosition",vye="org.eclipse.elk.rectpacking.onlyFirstIteration",wye="org.eclipse.elk.rectpacking.rowCompaction",ree="org.eclipse.elk.rectpacking.expandToAspectRatio",mye="org.eclipse.elk.rectpacking.targetWidth",oj="org.eclipse.elk.expandNodes",Hl="org.eclipse.elk.rectpacking",tD="org.eclipse.elk.alg.rectpacking.util",cj="No implementation available for ",Uw="org.eclipse.elk.alg.spore",qw="org.eclipse.elk.alg.spore.options",Ub="org.eclipse.elk.sporeCompaction",iee="org.eclipse.elk.underlyingLayoutAlgorithm",yye="org.eclipse.elk.processingOrder.treeConstruction",kye="org.eclipse.elk.processingOrder.spanningTreeCostFunction",see="org.eclipse.elk.processingOrder.preferredRoot",aee="org.eclipse.elk.processingOrder.rootSelection",oee="org.eclipse.elk.structure.structureExtractionStrategy",xye="org.eclipse.elk.compaction.compactionStrategy",Eye="org.eclipse.elk.compaction.orthogonal",_ye="org.eclipse.elk.overlapRemoval.maxIterations",Tye="org.eclipse.elk.overlapRemoval.runScanline",cee="processingOrder",Mwt="overlapRemoval",Ix="org.eclipse.elk.sporeOverlap",Dwt="org.eclipse.elk.alg.spore.p1structure",uee="org.eclipse.elk.alg.spore.p2processingorder",lee="org.eclipse.elk.alg.spore.p3execution",Lwt="Invalid index: ",Ox="org.eclipse.elk.core.alg",Xy={331:1},Yw={288:1},Iwt="Make sure its type is registered with the ",Cye=" utility class.",Nx="true",hee="false",Owt="Couldn't clone property '",qb=.05,zl="org.eclipse.elk.core.options",Nwt=1.2999999523162842,Yb="org.eclipse.elk.box",Sye="org.eclipse.elk.box.packingMode",Pwt="org.eclipse.elk.algorithm",Bwt="org.eclipse.elk.resolvedAlgorithm",Aye="org.eclipse.elk.bendPoints",Fwt="org.eclipse.elk.labelManager",Rwt="org.eclipse.elk.scaleFactor",jwt="org.eclipse.elk.animate",$wt="org.eclipse.elk.animTimeFactor",Hwt="org.eclipse.elk.layoutAncestors",zwt="org.eclipse.elk.maxAnimTime",Gwt="org.eclipse.elk.minAnimTime",Vwt="org.eclipse.elk.progressBar",Uwt="org.eclipse.elk.validateGraph",qwt="org.eclipse.elk.validateOptions",Ywt="org.eclipse.elk.zoomToFit",Wwt="org.eclipse.elk.font.name",Kwt="org.eclipse.elk.font.size",Xwt="org.eclipse.elk.edge.type",Qwt="partitioning",Zwt="nodeLabels",uj="portAlignment",fee="nodeSize",dee="port",Mye="portLabels",Jwt="insideSelfLoops",Q9="org.eclipse.elk.fixed",lj="org.eclipse.elk.random",emt="port must have a parent node to calculate the port side",tmt="The edge needs to have exactly one edge section. Found: ",Z9="org.eclipse.elk.core.util.adapters",yl="org.eclipse.emf.ecore",Qy="org.eclipse.elk.graph",nmt="EMapPropertyHolder",rmt="ElkBendPoint",imt="ElkGraphElement",smt="ElkConnectableShape",Dye="ElkEdge",amt="ElkEdgeSection",omt="EModelElement",cmt="ENamedElement",Lye="ElkLabel",Iye="ElkNode",Oye="ElkPort",umt={92:1,90:1},S5="org.eclipse.emf.common.notify.impl",Fg="The feature '",J9="' is not a valid changeable feature",lmt="Expecting null",gee="' is not a valid feature",hmt="The feature ID",fmt=" is not a valid feature ID",go=32768,dmt={105:1,92:1,90:1,56:1,49:1,97:1},Qt="org.eclipse.emf.ecore.impl",i2="org.eclipse.elk.graph.impl",e_="Recursive containment not allowed for ",Px="The datatype '",Wb="' is not a valid classifier",pee="The value '",Zy={190:1,3:1,4:1},bee="The class '",Bx="http://www.eclipse.org/elk/ElkGraph",Ch=1024,Nye="property",t_="value",vee="source",gmt="properties",pmt="identifier",wee="height",mee="width",yee="parent",kee="text",xee="children",bmt="hierarchical",Pye="sources",Eee="targets",Bye="sections",hj="bendPoints",Fye="outgoingShape",Rye="incomingShape",jye="outgoingSections",$ye="incomingSections",Ys="org.eclipse.emf.common.util",Hye="Severe implementation error in the Json to ElkGraph importer.",p1="id",Ns="org.eclipse.elk.graph.json",zye="Unhandled parameter types: ",vmt="startPoint",wmt="An edge must have at least one source and one target (edge id: '",Fx="').",mmt="Referenced edge section does not exist: ",ymt=" (edge id: '",Gye="target",kmt="sourcePoint",xmt="targetPoint",fj="group",_r="name",Emt="connectableShape cannot be null",_mt="edge cannot be null",_ee="Passed edge is not 'simple'.",dj="org.eclipse.elk.graph.util",nD="The 'no duplicates' constraint is violated",Tee="targetIndex=",s2=", size=",Cee="sourceIndex=",b1={3:1,4:1,20:1,28:1,52:1,14:1,15:1,54:1,67:1,63:1,58:1},See={3:1,4:1,20:1,28:1,52:1,14:1,47:1,15:1,54:1,67:1,63:1,58:1,588:1},gj="logging",Tmt="measureExecutionTime",Cmt="parser.parse.1",Smt="parser.parse.2",pj="parser.next.1",Aee="parser.next.2",Amt="parser.next.3",Mmt="parser.next.4",a2="parser.factor.1",Vye="parser.factor.2",Dmt="parser.factor.3",Lmt="parser.factor.4",Imt="parser.factor.5",Omt="parser.factor.6",Nmt="parser.atom.1",Pmt="parser.atom.2",Bmt="parser.atom.3",Uye="parser.atom.4",Mee="parser.atom.5",qye="parser.cc.1",bj="parser.cc.2",Fmt="parser.cc.3",Rmt="parser.cc.5",Yye="parser.cc.6",Wye="parser.cc.7",Dee="parser.cc.8",jmt="parser.ope.1",$mt="parser.ope.2",Hmt="parser.ope.3",A0="parser.descape.1",zmt="parser.descape.2",Gmt="parser.descape.3",Vmt="parser.descape.4",Umt="parser.descape.5",kl="parser.process.1",qmt="parser.quantifier.1",Ymt="parser.quantifier.2",Wmt="parser.quantifier.3",Kmt="parser.quantifier.4",Kye="parser.quantifier.5",Xmt="org.eclipse.emf.common.notify",Xye={415:1,672:1},Qmt={3:1,4:1,20:1,28:1,52:1,14:1,15:1,67:1,58:1},rD={366:1,143:1},n_="index=",Lee={3:1,4:1,5:1,126:1},Zmt={3:1,4:1,20:1,28:1,52:1,14:1,15:1,54:1,67:1,58:1},Qye={3:1,6:1,4:1,5:1,192:1},Jmt={3:1,4:1,5:1,165:1,367:1},eyt=";/?:@&=+$,",tyt="invalid authority: ",nyt="EAnnotation",ryt="ETypedElement",iyt="EStructuralFeature",syt="EAttribute",ayt="EClassifier",oyt="EEnumLiteral",cyt="EGenericType",uyt="EOperation",lyt="EParameter",hyt="EReference",fyt="ETypeParameter",ri="org.eclipse.emf.ecore.util",Iee={76:1},Zye={3:1,20:1,14:1,15:1,58:1,589:1,76:1,69:1,95:1},dyt="org.eclipse.emf.ecore.util.FeatureMap$Entry",zc=8192,Ww=2048,r_="byte",vj="char",i_="double",s_="float",a_="int",o_="long",c_="short",gyt="java.lang.Object",Jy={3:1,4:1,5:1,247:1},Jye={3:1,4:1,5:1,673:1},pyt={3:1,4:1,20:1,28:1,52:1,14:1,15:1,54:1,67:1,63:1,58:1,69:1},Va={3:1,4:1,20:1,28:1,52:1,14:1,15:1,54:1,67:1,63:1,58:1,76:1,69:1,95:1},iD="mixed",lr="http:///org/eclipse/emf/ecore/util/ExtendedMetaData",Gl="kind",byt={3:1,4:1,5:1,674:1},e3e={3:1,4:1,20:1,28:1,52:1,14:1,15:1,67:1,58:1,76:1,69:1,95:1},wj={20:1,28:1,52:1,14:1,15:1,58:1,69:1},mj={47:1,125:1,279:1},yj={72:1,332:1},kj="The value of type '",xj="' must be of type '",e3=1316,Vl="http://www.eclipse.org/emf/2002/Ecore",Ej=-32768,Kb="constraints",Hs="baseType",vyt="getEStructuralFeature",wyt="getFeatureID",u_="feature",myt="getOperationID",t3e="operation",yyt="defaultValue",kyt="eTypeParameters",xyt="isInstance",Eyt="getEEnumLiteral",_yt="eContainingClass",mr={55:1},Tyt={3:1,4:1,5:1,119:1},Cyt="org.eclipse.emf.ecore.resource",Syt={92:1,90:1,591:1,1935:1},Oee="org.eclipse.emf.ecore.resource.impl",n3e="unspecified",sD="simple",_j="attribute",Ayt="attributeWildcard",Tj="element",Nee="elementWildcard",tf="collapse",Pee="itemType",Cj="namespace",aD="##targetNamespace",Ul="whiteSpace",r3e="wildcards",o2="http://www.eclipse.org/emf/2003/XMLType",Bee="##any",Rx="uninitialized",oD="The multiplicity constraint is violated",Sj="org.eclipse.emf.ecore.xml.type",Myt="ProcessingInstruction",Dyt="SimpleAnyType",Lyt="XMLTypeDocumentRoot",Ni="org.eclipse.emf.ecore.xml.type.impl",cD="INF",Iyt="processing",Oyt="ENTITIES_._base",i3e="minLength",s3e="ENTITY",Aj="NCName",Nyt="IDREFS_._base",a3e="integer",Fee="token",Ree="pattern",Pyt="[a-zA-Z]{1,8}(-[a-zA-Z0-9]{1,8})*",o3e="\\i\\c*",Byt="[\\i-[:]][\\c-[:]]*",Fyt="nonPositiveInteger",uD="maxInclusive",c3e="NMTOKEN",Ryt="NMTOKENS_._base",u3e="nonNegativeInteger",lD="minInclusive",jyt="normalizedString",$yt="unsignedByte",Hyt="unsignedInt",zyt="18446744073709551615",Gyt="unsignedShort",Vyt="processingInstruction",M0="org.eclipse.emf.ecore.xml.type.internal",jx=1114111,Uyt="Internal Error: shorthands: \\u",l_="xml:isDigit",jee="xml:isWord",$ee="xml:isSpace",Hee="xml:isNameChar",zee="xml:isInitialNameChar",qyt="09\u0660\u0669\u06F0\u06F9\u0966\u096F\u09E6\u09EF\u0A66\u0A6F\u0AE6\u0AEF\u0B66\u0B6F\u0BE7\u0BEF\u0C66\u0C6F\u0CE6\u0CEF\u0D66\u0D6F\u0E50\u0E59\u0ED0\u0ED9\u0F20\u0F29",Yyt="AZaz\xC0\xD6\xD8\xF6\xF8\u0131\u0134\u013E\u0141\u0148\u014A\u017E\u0180\u01C3\u01CD\u01F0\u01F4\u01F5\u01FA\u0217\u0250\u02A8\u02BB\u02C1\u0386\u0386\u0388\u038A\u038C\u038C\u038E\u03A1\u03A3\u03CE\u03D0\u03D6\u03DA\u03DA\u03DC\u03DC\u03DE\u03DE\u03E0\u03E0\u03E2\u03F3\u0401\u040C\u040E\u044F\u0451\u045C\u045E\u0481\u0490\u04C4\u04C7\u04C8\u04CB\u04CC\u04D0\u04EB\u04EE\u04F5\u04F8\u04F9\u0531\u0556\u0559\u0559\u0561\u0586\u05D0\u05EA\u05F0\u05F2\u0621\u063A\u0641\u064A\u0671\u06B7\u06BA\u06BE\u06C0\u06CE\u06D0\u06D3\u06D5\u06D5\u06E5\u06E6\u0905\u0939\u093D\u093D\u0958\u0961\u0985\u098C\u098F\u0990\u0993\u09A8\u09AA\u09B0\u09B2\u09B2\u09B6\u09B9\u09DC\u09DD\u09DF\u09E1\u09F0\u09F1\u0A05\u0A0A\u0A0F\u0A10\u0A13\u0A28\u0A2A\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59\u0A5C\u0A5E\u0A5E\u0A72\u0A74\u0A85\u0A8B\u0A8D\u0A8D\u0A8F\u0A91\u0A93\u0AA8\u0AAA\u0AB0\u0AB2\u0AB3\u0AB5\u0AB9\u0ABD\u0ABD\u0AE0\u0AE0\u0B05\u0B0C\u0B0F\u0B10\u0B13\u0B28\u0B2A\u0B30\u0B32\u0B33\u0B36\u0B39\u0B3D\u0B3D\u0B5C\u0B5D\u0B5F\u0B61\u0B85\u0B8A\u0B8E\u0B90\u0B92\u0B95\u0B99\u0B9A\u0B9C\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8\u0BAA\u0BAE\u0BB5\u0BB7\u0BB9\u0C05\u0C0C\u0C0E\u0C10\u0C12\u0C28\u0C2A\u0C33\u0C35\u0C39\u0C60\u0C61\u0C85\u0C8C\u0C8E\u0C90\u0C92\u0CA8\u0CAA\u0CB3\u0CB5\u0CB9\u0CDE\u0CDE\u0CE0\u0CE1\u0D05\u0D0C\u0D0E\u0D10\u0D12\u0D28\u0D2A\u0D39\u0D60\u0D61\u0E01\u0E2E\u0E30\u0E30\u0E32\u0E33\u0E40\u0E45\u0E81\u0E82\u0E84\u0E84\u0E87\u0E88\u0E8A\u0E8A\u0E8D\u0E8D\u0E94\u0E97\u0E99\u0E9F\u0EA1\u0EA3\u0EA5\u0EA5\u0EA7\u0EA7\u0EAA\u0EAB\u0EAD\u0EAE\u0EB0\u0EB0\u0EB2\u0EB3\u0EBD\u0EBD\u0EC0\u0EC4\u0F40\u0F47\u0F49\u0F69\u10A0\u10C5\u10D0\u10F6\u1100\u1100\u1102\u1103\u1105\u1107\u1109\u1109\u110B\u110C\u110E\u1112\u113C\u113C\u113E\u113E\u1140\u1140\u114C\u114C\u114E\u114E\u1150\u1150\u1154\u1155\u1159\u1159\u115F\u1161\u1163\u1163\u1165\u1165\u1167\u1167\u1169\u1169\u116D\u116E\u1172\u1173\u1175\u1175\u119E\u119E\u11A8\u11A8\u11AB\u11AB\u11AE\u11AF\u11B7\u11B8\u11BA\u11BA\u11BC\u11C2\u11EB\u11EB\u11F0\u11F0\u11F9\u11F9\u1E00\u1E9B\u1EA0\u1EF9\u1F00\u1F15\u1F18\u1F1D\u1F20\u1F45\u1F48\u1F4D\u1F50\u1F57\u1F59\u1F59\u1F5B\u1F5B\u1F5D\u1F5D\u1F5F\u1F7D\u1F80\u1FB4\u1FB6\u1FBC\u1FBE\u1FBE\u1FC2\u1FC4\u1FC6\u1FCC\u1FD0\u1FD3\u1FD6\u1FDB\u1FE0\u1FEC\u1FF2\u1FF4\u1FF6\u1FFC\u2126\u2126\u212A\u212B\u212E\u212E\u2180\u2182\u3007\u3007\u3021\u3029\u3041\u3094\u30A1\u30FA\u3105\u312C\u4E00\u9FA5\uAC00\uD7A3",Wyt="Private Use",Gee="ASSIGNED",Vee="\0\x7F\x80\xFF\u0100\u017F\u0180\u024F\u0250\u02AF\u02B0\u02FF\u0300\u036F\u0370\u03FF\u0400\u04FF\u0530\u058F\u0590\u05FF\u0600\u06FF\u0700\u074F\u0780\u07BF\u0900\u097F\u0980\u09FF\u0A00\u0A7F\u0A80\u0AFF\u0B00\u0B7F\u0B80\u0BFF\u0C00\u0C7F\u0C80\u0CFF\u0D00\u0D7F\u0D80\u0DFF\u0E00\u0E7F\u0E80\u0EFF\u0F00\u0FFF\u1000\u109F\u10A0\u10FF\u1100\u11FF\u1200\u137F\u13A0\u13FF\u1400\u167F\u1680\u169F\u16A0\u16FF\u1780\u17FF\u1800\u18AF\u1E00\u1EFF\u1F00\u1FFF\u2000\u206F\u2070\u209F\u20A0\u20CF\u20D0\u20FF\u2100\u214F\u2150\u218F\u2190\u21FF\u2200\u22FF\u2300\u23FF\u2400\u243F\u2440\u245F\u2460\u24FF\u2500\u257F\u2580\u259F\u25A0\u25FF\u2600\u26FF\u2700\u27BF\u2800\u28FF\u2E80\u2EFF\u2F00\u2FDF\u2FF0\u2FFF\u3000\u303F\u3040\u309F\u30A0\u30FF\u3100\u312F\u3130\u318F\u3190\u319F\u31A0\u31BF\u3200\u32FF\u3300\u33FF\u3400\u4DB5\u4E00\u9FFF\uA000\uA48F\uA490\uA4CF\uAC00\uD7A3\uE000\uF8FF\uF900\uFAFF\uFB00\uFB4F\uFB50\uFDFF\uFE20\uFE2F\uFE30\uFE4F\uFE50\uFE6F\uFE70\uFEFE\uFEFF\uFEFF\uFF00\uFFEF",l3e="UNASSIGNED",$x={3:1,117:1},Kyt="org.eclipse.emf.ecore.xml.type.util",Mj={3:1,4:1,5:1,368:1},h3e="org.eclipse.xtext.xbase.lib",Xyt="Cannot add elements to a Range",Qyt="Cannot set elements in a Range",Zyt="Cannot remove elements from a Range",Dj="locale",Lj="default",Ij="user.agent",g,Oj,Uee;d.goog=d.goog||{},d.goog.global=d.goog.global||d,H9n(),M(1,null,{},y),g.Fb=function(t){return VJe(this,t)},g.Gb=function(){return this.gm},g.Hb=function(){return rb(this)},g.Ib=function(){var t;return sg(uu(this))+"@"+(t=ii(this)>>>0,t.toString(16))},g.equals=function(e){return this.Fb(e)},g.hashCode=function(){return this.Hb()},g.toString=function(){return this.Ib()};var Jyt,e3t,t3t;M(290,1,{290:1,2026:1},Hge),g.le=function(t){var n;return n=new Hge,n.i=4,t>1?n.c=ust(this,t-1):n.c=this,n},g.me=function(){return yd(this),this.b},g.ne=function(){return sg(this)},g.oe=function(){return yd(this),this.k},g.pe=function(){return(this.i&4)!=0},g.qe=function(){return(this.i&1)!=0},g.Ib=function(){return nge(this)},g.i=0;var yn=D(Ja,"Object",1),f3e=D(Ja,"Class",290);M(1998,1,IM),D(OM,"Optional",1998),M(1170,1998,IM,k),g.Fb=function(t){return t===this},g.Hb=function(){return 2040732332},g.Ib=function(){return"Optional.absent()"},g.Jb=function(t){return Vn(t),F7(),qee};var qee;D(OM,"Absent",1170),M(628,1,{},$q),D(OM,"Joiner",628);var gPn=fi(OM,"Predicate");M(582,1,{169:1,582:1,3:1,45:1},VUe),g.Mb=function(t){return Klt(this,t)},g.Lb=function(t){return Klt(this,t)},g.Fb=function(t){var n;return ue(t,582)?(n=h(t,582),Nbe(this.a,n.a)):!1},g.Hb=function(){return qge(this.a)+306654252},g.Ib=function(){return S_n(this.a)},D(OM,"Predicates/AndPredicate",582),M(408,1998,{408:1,3:1},YO),g.Fb=function(t){var n;return ue(t,408)?(n=h(t,408),Nr(this.a,n.a)):!1},g.Hb=function(){return 1502476572+ii(this.a)},g.Ib=function(){return Pbt+this.a+")"},g.Jb=function(t){return new YO(XP(t.Kb(this.a),"the Function passed to Optional.transform() must not return null."))},D(OM,"Present",408),M(198,1,mx),g.Nb=function(t){Ls(this,t)},g.Qb=function(){NXe()},D(mt,"UnmodifiableIterator",198),M(1978,198,yx),g.Qb=function(){NXe()},g.Rb=function(t){throw ne(new Xn)},g.Wb=function(t){throw ne(new Xn)},D(mt,"UnmodifiableListIterator",1978),M(386,1978,yx),g.Ob=function(){return this.c<this.d},g.Sb=function(){return this.c>0},g.Pb=function(){if(this.c>=this.d)throw ne(new lo);return this.Xb(this.c++)},g.Tb=function(){return this.c},g.Ub=function(){if(this.c<=0)throw ne(new lo);return this.Xb(--this.c)},g.Vb=function(){return this.c-1},g.c=0,g.d=0,D(mt,"AbstractIndexedListIterator",386),M(699,198,mx),g.Ob=function(){return FK(this)},g.Pb=function(){return X0e(this)},g.e=1,D(mt,"AbstractIterator",699),M(1986,1,{224:1}),g.Zb=function(){var t;return t=this.f,t||(this.f=this.ac())},g.Fb=function(t){return tX(this,t)},g.Hb=function(){return ii(this.Zb())},g.dc=function(){return this.gc()==0},g.ec=function(){return $4(this)},g.Ib=function(){return Ga(this.Zb())},D(mt,"AbstractMultimap",1986),M(726,1986,Zp),g.$b=function(){VB(this)},g._b=function(t){return JXe(this,t)},g.ac=function(){return new W6(this,this.c)},g.ic=function(t){return this.hc()},g.bc=function(){return new hy(this,this.c)},g.jc=function(){return this.mc(this.hc())},g.kc=function(){return new vXe(this)},g.lc=function(){return cQ(this.c.vc().Nc(),new T,64,this.d)},g.cc=function(t){return jr(this,t)},g.fc=function(t){return qA(this,t)},g.gc=function(){return this.d},g.mc=function(t){return Rt(),new B6(t)},g.nc=function(){return new bXe(this)},g.oc=function(){return cQ(this.c.Cc().Nc(),new E,64,this.d)},g.pc=function(t,n){return new _B(this,t,n,null)},g.d=0,D(mt,"AbstractMapBasedMultimap",726),M(1631,726,Zp),g.hc=function(){return new Wo(this.a)},g.jc=function(){return Rt(),Rt(),ca},g.cc=function(t){return h(jr(this,t),15)},g.fc=function(t){return h(qA(this,t),15)},g.Zb=function(){return U4(this)},g.Fb=function(t){return tX(this,t)},g.qc=function(t){return h(jr(this,t),15)},g.rc=function(t){return h(qA(this,t),15)},g.mc=function(t){return sA(h(t,15))},g.pc=function(t,n){return gat(this,t,h(n,15),null)},D(mt,"AbstractListMultimap",1631),M(732,1,ys),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.c.Ob()||this.e.Ob()},g.Pb=function(){var t;return this.e.Ob()||(t=h(this.c.Pb(),42),this.b=t.cd(),this.a=h(t.dd(),14),this.e=this.a.Kc()),this.sc(this.b,this.e.Pb())},g.Qb=function(){this.e.Qb(),this.a.dc()&&this.c.Qb(),--this.d.d},D(mt,"AbstractMapBasedMultimap/Itr",732),M(1099,732,ys,bXe),g.sc=function(t,n){return n},D(mt,"AbstractMapBasedMultimap/1",1099),M(1100,1,{},E),g.Kb=function(t){return h(t,14).Nc()},D(mt,"AbstractMapBasedMultimap/1methodref$spliterator$Type",1100),M(1101,732,ys,vXe),g.sc=function(t,n){return new Z2(t,n)},D(mt,"AbstractMapBasedMultimap/2",1101);var d3e=fi(Nn,"Map");M(1967,1,zb),g.wc=function(t){ZE(this,t)},g.yc=function(t,n,s){return wX(this,t,n,s)},g.$b=function(){this.vc().$b()},g.tc=function(t){return UX(this,t)},g._b=function(t){return!!F2e(this,t,!1)},g.uc=function(t){var n,s,a;for(s=this.vc().Kc();s.Ob();)if(n=h(s.Pb(),42),a=n.dd(),Me(t)===Me(a)||t!=null&&Nr(t,a))return!0;return!1},g.Fb=function(t){var n,s,a;if(t===this)return!0;if(!ue(t,83)||(a=h(t,83),this.gc()!=a.gc()))return!1;for(s=a.vc().Kc();s.Ob();)if(n=h(s.Pb(),42),!this.tc(n))return!1;return!0},g.xc=function(t){return no(F2e(this,t,!1))},g.Hb=function(){return Bge(this.vc())},g.dc=function(){return this.gc()==0},g.ec=function(){return new P6(this)},g.zc=function(t,n){throw ne(new i0("Put not supported on this map"))},g.Ac=function(t){QE(this,t)},g.Bc=function(t){return no(F2e(this,t,!0))},g.gc=function(){return this.vc().gc()},g.Ib=function(){return Q1t(this)},g.Cc=function(){return new vd(this)},D(Nn,"AbstractMap",1967),M(1987,1967,zb),g.bc=function(){return new vN(this)},g.vc=function(){return drt(this)},g.ec=function(){var t;return t=this.g,t||(this.g=this.bc())},g.Cc=function(){var t;return t=this.i,t||(this.i=new UQe(this))},D(mt,"Maps/ViewCachingAbstractMap",1987),M(389,1987,zb,W6),g.xc=function(t){return u4n(this,t)},g.Bc=function(t){return T6n(this,t)},g.$b=function(){this.d==this.e.c?this.e.$b():CP(new ide(this))},g._b=function(t){return Aht(this.d,t)},g.Ec=function(){return new WUe(this)},g.Dc=function(){return this.Ec()},g.Fb=function(t){return this===t||Nr(this.d,t)},g.Hb=function(){return ii(this.d)},g.ec=function(){return this.e.ec()},g.gc=function(){return this.d.gc()},g.Ib=function(){return Ga(this.d)},D(mt,"AbstractMapBasedMultimap/AsMap",389);var Nf=fi(Ja,"Iterable");M(28,1,Rw),g.Jc=function(t){Os(this,t)},g.Lc=function(){return this.Oc()},g.Nc=function(){return new Wt(this,0)},g.Oc=function(){return new Vt(null,this.Nc())},g.Fc=function(t){throw ne(new i0("Add not supported on this collection"))},g.Gc=function(t){return Qs(this,t)},g.$b=function(){Gde(this)},g.Hc=function(t){return Ew(this,t,!1)},g.Ic=function(t){return FA(this,t)},g.dc=function(){return this.gc()==0},g.Mc=function(t){return Ew(this,t,!0)},g.Pc=function(){return vde(this)},g.Qc=function(t){return nM(this,t)},g.Ib=function(){return Ag(this)},D(Nn,"AbstractCollection",28);var ql=fi(Nn,"Set");M(Lf,28,$c),g.Nc=function(){return new Wt(this,1)},g.Fb=function(t){return kft(this,t)},g.Hb=function(){return Bge(this)},D(Nn,"AbstractSet",Lf),M(1970,Lf,$c),D(mt,"Sets/ImprovedAbstractSet",1970),M(1971,1970,$c),g.$b=function(){this.Rc().$b()},g.Hc=function(t){return nft(this,t)},g.dc=function(){return this.Rc().dc()},g.Mc=function(t){var n;return this.Hc(t)?(n=h(t,42),this.Rc().ec().Mc(n.cd())):!1},g.gc=function(){return this.Rc().gc()},D(mt,"Maps/EntrySet",1971),M(1097,1971,$c,WUe),g.Hc=function(t){return hpe(this.a.d.vc(),t)},g.Kc=function(){return new ide(this.a)},g.Rc=function(){return this.a},g.Mc=function(t){var n;return hpe(this.a.d.vc(),t)?(n=h(t,42),Kyn(this.a.e,n.cd()),!0):!1},g.Nc=function(){return QS(this.a.d.vc().Nc(),new KUe(this.a))},D(mt,"AbstractMapBasedMultimap/AsMap/AsMapEntries",1097),M(1098,1,{},KUe),g.Kb=function(t){return rot(this.a,h(t,42))},D(mt,"AbstractMapBasedMultimap/AsMap/AsMapEntries/0methodref$wrapEntry$Type",1098),M(730,1,ys,ide),g.Nb=function(t){Ls(this,t)},g.Pb=function(){var t;return t=h(this.b.Pb(),42),this.a=h(t.dd(),14),rot(this.c,t)},g.Ob=function(){return this.b.Ob()},g.Qb=function(){yy(!!this.a),this.b.Qb(),this.c.e.d-=this.a.gc(),this.a.$b(),this.a=null},D(mt,"AbstractMapBasedMultimap/AsMap/AsMapIterator",730),M(532,1970,$c,vN),g.$b=function(){this.b.$b()},g.Hc=function(t){return this.b._b(t)},g.Jc=function(t){Vn(t),this.b.wc(new fqe(t))},g.dc=function(){return this.b.dc()},g.Kc=function(){return new R7(this.b.vc().Kc())},g.Mc=function(t){return this.b._b(t)?(this.b.Bc(t),!0):!1},g.gc=function(){return this.b.gc()},D(mt,"Maps/KeySet",532),M(318,532,$c,hy),g.$b=function(){var t;CP((t=this.b.vc().Kc(),new Dhe(this,t)))},g.Ic=function(t){return this.b.ec().Ic(t)},g.Fb=function(t){return this===t||Nr(this.b.ec(),t)},g.Hb=function(){return ii(this.b.ec())},g.Kc=function(){var t;return t=this.b.vc().Kc(),new Dhe(this,t)},g.Mc=function(t){var n,s;return s=0,n=h(this.b.Bc(t),14),n&&(s=n.gc(),n.$b(),this.a.d-=s),s>0},g.Nc=function(){return this.b.ec().Nc()},D(mt,"AbstractMapBasedMultimap/KeySet",318),M(731,1,ys,Dhe),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.c.Ob()},g.Pb=function(){return this.a=h(this.c.Pb(),42),this.a.cd()},g.Qb=function(){var t;yy(!!this.a),t=h(this.a.dd(),14),this.c.Qb(),this.b.a.d-=t.gc(),t.$b(),this.a=null},D(mt,"AbstractMapBasedMultimap/KeySet/1",731),M(491,389,{83:1,161:1},VS),g.bc=function(){return this.Sc()},g.ec=function(){return this.Tc()},g.Sc=function(){return new yS(this.c,this.Uc())},g.Tc=function(){var t;return t=this.b,t||(this.b=this.Sc())},g.Uc=function(){return h(this.d,161)},D(mt,"AbstractMapBasedMultimap/SortedAsMap",491),M(542,491,Bbt,dP),g.bc=function(){return new Y6(this.a,h(h(this.d,161),171))},g.Sc=function(){return new Y6(this.a,h(h(this.d,161),171))},g.ec=function(){var t;return t=this.b,h(t||(this.b=new Y6(this.a,h(h(this.d,161),171))),271)},g.Tc=function(){var t;return t=this.b,h(t||(this.b=new Y6(this.a,h(h(this.d,161),171))),271)},g.Uc=function(){return h(h(this.d,161),171)},D(mt,"AbstractMapBasedMultimap/NavigableAsMap",542),M(490,318,Fbt,yS),g.Nc=function(){return this.b.ec().Nc()},D(mt,"AbstractMapBasedMultimap/SortedKeySet",490),M(388,490,Kve,Y6),D(mt,"AbstractMapBasedMultimap/NavigableKeySet",388),M(541,28,Rw,_B),g.Fc=function(t){var n,s;return lu(this),s=this.d.dc(),n=this.d.Fc(t),n&&(++this.f.d,s&&WS(this)),n},g.Gc=function(t){var n,s,a;return t.dc()?!1:(a=(lu(this),this.d.gc()),n=this.d.Gc(t),n&&(s=this.d.gc(),this.f.d+=s-a,a==0&&WS(this)),n)},g.$b=function(){var t;t=(lu(this),this.d.gc()),t!=0&&(this.d.$b(),this.f.d-=t,LP(this))},g.Hc=function(t){return lu(this),this.d.Hc(t)},g.Ic=function(t){return lu(this),this.d.Ic(t)},g.Fb=function(t){return t===this?!0:(lu(this),Nr(this.d,t))},g.Hb=function(){return lu(this),ii(this.d)},g.Kc=function(){return lu(this),new V1e(this)},g.Mc=function(t){var n;return lu(this),n=this.d.Mc(t),n&&(--this.f.d,LP(this)),n},g.gc=function(){return MJe(this)},g.Nc=function(){return lu(this),this.d.Nc()},g.Ib=function(){return lu(this),Ga(this.d)},D(mt,"AbstractMapBasedMultimap/WrappedCollection",541);var xl=fi(Nn,"List");M(728,541,{20:1,28:1,14:1,15:1},mde),g.ad=function(t){my(this,t)},g.Nc=function(){return lu(this),this.d.Nc()},g.Vc=function(t,n){var s;lu(this),s=this.d.dc(),h(this.d,15).Vc(t,n),++this.a.d,s&&WS(this)},g.Wc=function(t,n){var s,a,u;return n.dc()?!1:(u=(lu(this),this.d.gc()),s=h(this.d,15).Wc(t,n),s&&(a=this.d.gc(),this.a.d+=a-u,u==0&&WS(this)),s)},g.Xb=function(t){return lu(this),h(this.d,15).Xb(t)},g.Xc=function(t){return lu(this),h(this.d,15).Xc(t)},g.Yc=function(){return lu(this),new cet(this)},g.Zc=function(t){return lu(this),new Eit(this,t)},g.$c=function(t){var n;return lu(this),n=h(this.d,15).$c(t),--this.a.d,LP(this),n},g._c=function(t,n){return lu(this),h(this.d,15)._c(t,n)},g.bd=function(t,n){return lu(this),gat(this.a,this.e,h(this.d,15).bd(t,n),this.b?this.b:this)},D(mt,"AbstractMapBasedMultimap/WrappedList",728),M(1096,728,{20:1,28:1,14:1,15:1,54:1},ett),D(mt,"AbstractMapBasedMultimap/RandomAccessWrappedList",1096),M(620,1,ys,V1e),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return pk(this),this.b.Ob()},g.Pb=function(){return pk(this),this.b.Pb()},g.Qb=function(){Fet(this)},D(mt,"AbstractMapBasedMultimap/WrappedCollection/WrappedIterator",620),M(729,620,z1,cet,Eit),g.Qb=function(){Fet(this)},g.Rb=function(t){var n;n=MJe(this.a)==0,(pk(this),h(this.b,125)).Rb(t),++this.a.a.d,n&&WS(this.a)},g.Sb=function(){return(pk(this),h(this.b,125)).Sb()},g.Tb=function(){return(pk(this),h(this.b,125)).Tb()},g.Ub=function(){return(pk(this),h(this.b,125)).Ub()},g.Vb=function(){return(pk(this),h(this.b,125)).Vb()},g.Wb=function(t){(pk(this),h(this.b,125)).Wb(t)},D(mt,"AbstractMapBasedMultimap/WrappedList/WrappedListIterator",729),M(727,541,Fbt,h1e),g.Nc=function(){return lu(this),this.d.Nc()},D(mt,"AbstractMapBasedMultimap/WrappedSortedSet",727),M(1095,727,Kve,eet),D(mt,"AbstractMapBasedMultimap/WrappedNavigableSet",1095),M(1094,541,$c,gtt),g.Nc=function(){return lu(this),this.d.Nc()},D(mt,"AbstractMapBasedMultimap/WrappedSet",1094),M(1103,1,{},T),g.Kb=function(t){return t3n(h(t,42))},D(mt,"AbstractMapBasedMultimap/lambda$1$Type",1103),M(1102,1,{},XUe),g.Kb=function(t){return new Z2(this.a,t)},D(mt,"AbstractMapBasedMultimap/lambda$2$Type",1102);var c2=fi(Nn,"Map/Entry");M(345,1,ER),g.Fb=function(t){var n;return ue(t,42)?(n=h(t,42),n1(this.cd(),n.cd())&&n1(this.dd(),n.dd())):!1},g.Hb=function(){var t,n;return t=this.cd(),n=this.dd(),(t==null?0:ii(t))^(n==null?0:ii(n))},g.ed=function(t){throw ne(new Xn)},g.Ib=function(){return this.cd()+"="+this.dd()},D(mt,Rbt,345),M(1988,28,Rw),g.$b=function(){this.fd().$b()},g.Hc=function(t){var n;return ue(t,42)?(n=h(t,42),Pmn(this.fd(),n.cd(),n.dd())):!1},g.Mc=function(t){var n;return ue(t,42)?(n=h(t,42),Wst(this.fd(),n.cd(),n.dd())):!1},g.gc=function(){return this.fd().d},D(mt,"Multimaps/Entries",1988),M(733,1988,Rw,ale),g.Kc=function(){return this.a.kc()},g.fd=function(){return this.a},g.Nc=function(){return this.a.lc()},D(mt,"AbstractMultimap/Entries",733),M(734,733,$c,ahe),g.Nc=function(){return this.a.lc()},g.Fb=function(t){return tbe(this,t)},g.Hb=function(){return Kut(this)},D(mt,"AbstractMultimap/EntrySet",734),M(735,28,Rw,ole),g.$b=function(){this.a.$b()},g.Hc=function(t){return m6n(this.a,t)},g.Kc=function(){return this.a.nc()},g.gc=function(){return this.a.d},g.Nc=function(){return this.a.oc()},D(mt,"AbstractMultimap/Values",735),M(1989,28,{835:1,20:1,28:1,14:1}),g.Jc=function(t){Vn(t),dy(this).Jc(new hqe(t))},g.Nc=function(){var t;return t=dy(this).Nc(),cQ(t,new Y,64|t.qd()&1296,this.a.d)},g.Fc=function(t){return phe(),!0},g.Gc=function(t){return Vn(this),Vn(t),ue(t,543)?$mn(h(t,835)):!t.dc()&&AK(this,t.Kc())},g.Hc=function(t){var n;return n=h(xw(U4(this.a),t),14),(n?n.gc():0)>0},g.Fb=function(t){return OCn(this,t)},g.Hb=function(){return ii(dy(this))},g.dc=function(){return dy(this).dc()},g.Mc=function(t){return g0t(this,t,1)>0},g.Ib=function(){return Ga(dy(this))},D(mt,"AbstractMultiset",1989),M(1991,1970,$c),g.$b=function(){VB(this.a.a)},g.Hc=function(t){var n,s;return ue(t,492)?(s=h(t,416),h(s.a.dd(),14).gc()<=0?!1:(n=_st(this.a,s.a.cd()),n==h(s.a.dd(),14).gc())):!1},g.Mc=function(t){var n,s,a,u;return ue(t,492)&&(s=h(t,416),n=s.a.cd(),a=h(s.a.dd(),14).gc(),a!=0)?(u=this.a,kTn(u,n,a)):!1},D(mt,"Multisets/EntrySet",1991),M(1109,1991,$c,QUe),g.Kc=function(){return new TXe(drt(U4(this.a.a)).Kc())},g.gc=function(){return U4(this.a.a).gc()},D(mt,"AbstractMultiset/EntrySet",1109),M(619,726,Zp),g.hc=function(){return this.gd()},g.jc=function(){return this.hd()},g.cc=function(t){return this.jd(t)},g.fc=function(t){return this.kd(t)},g.Zb=function(){var t;return t=this.f,t||(this.f=this.ac())},g.hd=function(){return Rt(),Rt(),jj},g.Fb=function(t){return tX(this,t)},g.jd=function(t){return h(jr(this,t),21)},g.kd=function(t){return h(qA(this,t),21)},g.mc=function(t){return Rt(),new G6(h(t,21))},g.pc=function(t,n){return new gtt(this,t,h(n,21))},D(mt,"AbstractSetMultimap",619),M(1657,619,Zp),g.hc=function(){return new ag(this.b)},g.gd=function(){return new ag(this.b)},g.jc=function(){return Nde(new ag(this.b))},g.hd=function(){return Nde(new ag(this.b))},g.cc=function(t){return h(h(jr(this,t),21),84)},g.jd=function(t){return h(h(jr(this,t),21),84)},g.fc=function(t){return h(h(qA(this,t),21),84)},g.kd=function(t){return h(h(qA(this,t),21),84)},g.mc=function(t){return ue(t,271)?Nde(h(t,271)):(Rt(),new Vfe(h(t,84)))},g.Zb=function(){var t;return t=this.f,t||(this.f=ue(this.c,171)?new dP(this,h(this.c,171)):ue(this.c,161)?new VS(this,h(this.c,161)):new W6(this,this.c))},g.pc=function(t,n){return ue(n,271)?new eet(this,t,h(n,271)):new h1e(this,t,h(n,84))},D(mt,"AbstractSortedSetMultimap",1657),M(1658,1657,Zp),g.Zb=function(){var t;return t=this.f,h(h(t||(this.f=ue(this.c,171)?new dP(this,h(this.c,171)):ue(this.c,161)?new VS(this,h(this.c,161)):new W6(this,this.c)),161),171)},g.ec=function(){var t;return t=this.i,h(h(t||(this.i=ue(this.c,171)?new Y6(this,h(this.c,171)):ue(this.c,161)?new yS(this,h(this.c,161)):new hy(this,this.c)),84),271)},g.bc=function(){return ue(this.c,171)?new Y6(this,h(this.c,171)):ue(this.c,161)?new yS(this,h(this.c,161)):new hy(this,this.c)},D(mt,"AbstractSortedKeySortedSetMultimap",1658),M(2010,1,{1947:1}),g.Fb=function(t){return lEn(this,t)},g.Hb=function(){var t;return Bge((t=this.g,t||(this.g=new gq(this))))},g.Ib=function(){var t;return Q1t((t=this.f,t||(this.f=new Bfe(this))))},D(mt,"AbstractTable",2010),M(665,Lf,$c,gq),g.$b=function(){PXe()},g.Hc=function(t){var n,s;return ue(t,468)?(n=h(t,682),s=h(xw(Brt(this.a),ug(n.c.e,n.b)),83),!!s&&hpe(s.vc(),new Z2(ug(n.c.c,n.a),Z4(n.c,n.b,n.a)))):!1},g.Kc=function(){return Wvn(this.a)},g.Mc=function(t){var n,s;return ue(t,468)?(n=h(t,682),s=h(xw(Brt(this.a),ug(n.c.e,n.b)),83),!!s&&q6n(s.vc(),new Z2(ug(n.c.c,n.a),Z4(n.c,n.b,n.a)))):!1},g.gc=function(){return Wnt(this.a)},g.Nc=function(){return Gmn(this.a)},D(mt,"AbstractTable/CellSet",665),M(1928,28,Rw,ZUe),g.$b=function(){PXe()},g.Hc=function(t){return t9n(this.a,t)},g.Kc=function(){return Kvn(this.a)},g.gc=function(){return Wnt(this.a)},g.Nc=function(){return Qst(this.a)},D(mt,"AbstractTable/Values",1928),M(1632,1631,Zp),D(mt,"ArrayListMultimapGwtSerializationDependencies",1632),M(513,1632,Zp,jq,s0e),g.hc=function(){return new Wo(this.a)},g.a=0,D(mt,"ArrayListMultimap",513),M(664,2010,{664:1,1947:1,3:1},_0t),D(mt,"ArrayTable",664),M(1924,386,yx,Let),g.Xb=function(t){return new $ge(this.a,t)},D(mt,"ArrayTable/1",1924),M(1925,1,{},UUe),g.ld=function(t){return new $ge(this.a,t)},D(mt,"ArrayTable/1methodref$getCell$Type",1925),M(2011,1,{682:1}),g.Fb=function(t){var n;return t===this?!0:ue(t,468)?(n=h(t,682),n1(ug(this.c.e,this.b),ug(n.c.e,n.b))&&n1(ug(this.c.c,this.a),ug(n.c.c,n.a))&&n1(Z4(this.c,this.b,this.a),Z4(n.c,n.b,n.a))):!1},g.Hb=function(){return lF(se(ie(yn,1),st,1,5,[ug(this.c.e,this.b),ug(this.c.c,this.a),Z4(this.c,this.b,this.a)]))},g.Ib=function(){return"("+ug(this.c.e,this.b)+","+ug(this.c.c,this.a)+")="+Z4(this.c,this.b,this.a)},D(mt,"Tables/AbstractCell",2011),M(468,2011,{468:1,682:1},$ge),g.a=0,g.b=0,g.d=0,D(mt,"ArrayTable/2",468),M(1927,1,{},qUe),g.ld=function(t){return cct(this.a,t)},D(mt,"ArrayTable/2methodref$getValue$Type",1927),M(1926,386,yx,Iet),g.Xb=function(t){return cct(this.a,t)},D(mt,"ArrayTable/3",1926),M(1979,1967,zb),g.$b=function(){CP(this.kc())},g.vc=function(){return new dqe(this)},g.lc=function(){return new lit(this.kc(),this.gc())},D(mt,"Maps/IteratorBasedAbstractMap",1979),M(828,1979,zb),g.$b=function(){throw ne(new Xn)},g._b=function(t){return eQe(this.c,t)},g.kc=function(){return new Oet(this,this.c.b.c.gc())},g.lc=function(){return iW(this.c.b.c.gc(),16,new YUe(this))},g.xc=function(t){var n;return n=h(_E(this.c,t),19),n?this.nd(n.a):null},g.dc=function(){return this.c.b.c.dc()},g.ec=function(){return fW(this.c)},g.zc=function(t,n){var s;if(s=h(_E(this.c,t),19),!s)throw ne(new an(this.md()+" "+t+" not in "+fW(this.c)));return this.od(s.a,n)},g.Bc=function(t){throw ne(new Xn)},g.gc=function(){return this.c.b.c.gc()},D(mt,"ArrayTable/ArrayMap",828),M(1923,1,{},YUe),g.ld=function(t){return Rrt(this.a,t)},D(mt,"ArrayTable/ArrayMap/0methodref$getEntry$Type",1923),M(1921,345,ER,NQe),g.cd=function(){return l0n(this.a,this.b)},g.dd=function(){return this.a.nd(this.b)},g.ed=function(t){return this.a.od(this.b,t)},g.b=0,D(mt,"ArrayTable/ArrayMap/1",1921),M(1922,386,yx,Oet),g.Xb=function(t){return Rrt(this.a,t)},D(mt,"ArrayTable/ArrayMap/2",1922),M(1920,828,zb,Trt),g.md=function(){return"Column"},g.nd=function(t){return Z4(this.b,this.a,t)},g.od=function(t,n){return Nlt(this.b,this.a,t,n)},g.a=0,D(mt,"ArrayTable/Row",1920),M(829,828,zb,Bfe),g.nd=function(t){return new Trt(this.a,t)},g.zc=function(t,n){return h(n,83),Yfn()},g.od=function(t,n){return h(n,83),Wfn()},g.md=function(){return"Row"},D(mt,"ArrayTable/RowMap",829),M(1120,1,jl,PQe),g.qd=function(){return this.a.qd()&-262},g.rd=function(){return this.a.rd()},g.Nb=function(t){this.a.Nb(new DQe(t,this.b))},g.sd=function(t){return this.a.sd(new MQe(t,this.b))},D(mt,"CollectSpliterators/1",1120),M(1121,1,mn,MQe),g.td=function(t){this.a.td(this.b.Kb(t))},D(mt,"CollectSpliterators/1/lambda$0$Type",1121),M(1122,1,mn,DQe),g.td=function(t){this.a.td(this.b.Kb(t))},D(mt,"CollectSpliterators/1/lambda$1$Type",1122),M(1123,1,jl,cat),g.qd=function(){return this.a},g.rd=function(){return this.d&&(this.b=het(this.b,this.d.rd())),het(this.b,0)},g.Nb=function(t){this.d&&(this.d.Nb(t),this.d=null),this.c.Nb(new AQe(this.e,t)),this.b=0},g.sd=function(t){for(;;){if(!!this.d&&this.d.sd(t))return fE(this.b,_R)&&(this.b=Cg(this.b,1)),!0;if(this.d=null,!this.c.sd(new LQe(this,this.e)))return!1}},g.a=0,g.b=0,D(mt,"CollectSpliterators/1FlatMapSpliterator",1123),M(1124,1,mn,LQe),g.td=function(t){dgn(this.a,this.b,t)},D(mt,"CollectSpliterators/1FlatMapSpliterator/lambda$0$Type",1124),M(1125,1,mn,AQe),g.td=function(t){Qdn(this.b,this.a,t)},D(mt,"CollectSpliterators/1FlatMapSpliterator/lambda$1$Type",1125),M(1117,1,jl,Vtt),g.qd=function(){return 16464|this.b},g.rd=function(){return this.a.rd()},g.Nb=function(t){this.a.xe(new OQe(t,this.c))},g.sd=function(t){return this.a.ye(new IQe(t,this.c))},g.b=0,D(mt,"CollectSpliterators/1WithCharacteristics",1117),M(1118,1,NM,IQe),g.ud=function(t){this.a.td(this.b.ld(t))},D(mt,"CollectSpliterators/1WithCharacteristics/lambda$0$Type",1118),M(1119,1,NM,OQe),g.ud=function(t){this.a.td(this.b.ld(t))},D(mt,"CollectSpliterators/1WithCharacteristics/lambda$1$Type",1119),M(245,1,dZ),g.wd=function(t){return this.vd(h(t,245))},g.vd=function(t){var n;return t==(Dq(),Wee)?1:t==(Lq(),Yee)?-1:(n=(EP(),OA(this.a,t.a)),n!=0?n:ue(this,519)==ue(t,519)?0:ue(this,519)?1:-1)},g.zd=function(){return this.a},g.Fb=function(t){return Gpe(this,t)},D(mt,"Cut",245),M(1761,245,dZ,RXe),g.vd=function(t){return t==this?0:1},g.xd=function(t){throw ne(new zle)},g.yd=function(t){t.a+="+\u221E)"},g.zd=function(){throw ne(new ja($bt))},g.Hb=function(){return L1(),Zpe(this)},g.Ad=function(t){return!1},g.Ib=function(){return"+\u221E"};var Yee;D(mt,"Cut/AboveAll",1761),M(519,245,{245:1,519:1,3:1,35:1},Ret),g.xd=function(t){ho((t.a+="(",t),this.a)},g.yd=function(t){pg(ho(t,this.a),93)},g.Hb=function(){return~ii(this.a)},g.Ad=function(t){return EP(),OA(this.a,t)<0},g.Ib=function(){return"/"+this.a+"\\"},D(mt,"Cut/AboveValue",519),M(1760,245,dZ,jXe),g.vd=function(t){return t==this?0:-1},g.xd=function(t){t.a+="(-\u221E"},g.yd=function(t){throw ne(new zle)},g.zd=function(){throw ne(new ja($bt))},g.Hb=function(){return L1(),Zpe(this)},g.Ad=function(t){return!0},g.Ib=function(){return"-\u221E"};var Wee;D(mt,"Cut/BelowAll",1760),M(1762,245,dZ,jet),g.xd=function(t){ho((t.a+="[",t),this.a)},g.yd=function(t){pg(ho(t,this.a),41)},g.Hb=function(){return ii(this.a)},g.Ad=function(t){return EP(),OA(this.a,t)<=0},g.Ib=function(){return"\\"+this.a+"/"},D(mt,"Cut/BelowValue",1762),M(537,1,G1),g.Jc=function(t){Os(this,t)},g.Ib=function(){return xkn(h(XP(this,"use Optional.orNull() instead of Optional.or(null)"),20).Kc())},D(mt,"FluentIterable",537),M(433,537,G1,pE),g.Kc=function(){return new Sn(Dn(this.a.Kc(),new L))},D(mt,"FluentIterable/2",433),M(1046,537,G1,HJe),g.Kc=function(){return gg(this)},D(mt,"FluentIterable/3",1046),M(708,386,yx,Ffe),g.Xb=function(t){return this.a[t].Kc()},D(mt,"FluentIterable/3/1",708),M(1972,1,{}),g.Ib=function(){return Ga(this.Bd().b)},D(mt,"ForwardingObject",1972),M(1973,1972,Hbt),g.Bd=function(){return this.Cd()},g.Jc=function(t){Os(this,t)},g.Lc=function(){return this.Oc()},g.Nc=function(){return new Wt(this,0)},g.Oc=function(){return new Vt(null,this.Nc())},g.Fc=function(t){return this.Cd(),iQe()},g.Gc=function(t){return this.Cd(),sQe()},g.$b=function(){this.Cd(),aQe()},g.Hc=function(t){return this.Cd().Hc(t)},g.Ic=function(t){return this.Cd().Ic(t)},g.dc=function(){return this.Cd().b.dc()},g.Kc=function(){return this.Cd().Kc()},g.Mc=function(t){return this.Cd(),oQe()},g.gc=function(){return this.Cd().b.gc()},g.Pc=function(){return this.Cd().Pc()},g.Qc=function(t){return this.Cd().Qc(t)},D(mt,"ForwardingCollection",1973),M(1980,28,Xve),g.Kc=function(){return this.Ed()},g.Fc=function(t){throw ne(new Xn)},g.Gc=function(t){throw ne(new Xn)},g.$b=function(){throw ne(new Xn)},g.Hc=function(t){return t!=null&&Ew(this,t,!1)},g.Dd=function(){switch(this.gc()){case 0:return lw(),lw(),Kee;case 1:return lw(),new sW(Vn(this.Ed().Pb()));default:return new Crt(this,this.Pc())}},g.Mc=function(t){throw ne(new Xn)},D(mt,"ImmutableCollection",1980),M(712,1980,Xve,$le),g.Kc=function(){return J4(this.a.Kc())},g.Hc=function(t){return t!=null&&this.a.Hc(t)},g.Ic=function(t){return this.a.Ic(t)},g.dc=function(){return this.a.dc()},g.Ed=function(){return J4(this.a.Kc())},g.gc=function(){return this.a.gc()},g.Pc=function(){return this.a.Pc()},g.Qc=function(t){return this.a.Qc(t)},g.Ib=function(){return Ga(this.a)},D(mt,"ForwardingImmutableCollection",712),M(152,1980,O9),g.Kc=function(){return this.Ed()},g.Yc=function(){return this.Fd(0)},g.Zc=function(t){return this.Fd(t)},g.ad=function(t){my(this,t)},g.Nc=function(){return new Wt(this,16)},g.bd=function(t,n){return this.Gd(t,n)},g.Vc=function(t,n){throw ne(new Xn)},g.Wc=function(t,n){throw ne(new Xn)},g.Fb=function(t){return vCn(this,t)},g.Hb=function(){return A5n(this)},g.Xc=function(t){return t==null?-1:H8n(this,t)},g.Ed=function(){return this.Fd(0)},g.Fd=function(t){return Zfe(this,t)},g.$c=function(t){throw ne(new Xn)},g._c=function(t,n){throw ne(new Xn)},g.Gd=function(t,n){var s;return dF((s=new qQe(this),new F1(s,t,n)))};var Kee;D(mt,"ImmutableList",152),M(2006,152,O9),g.Kc=function(){return J4(this.Hd().Kc())},g.bd=function(t,n){return dF(this.Hd().bd(t,n))},g.Hc=function(t){return t!=null&&this.Hd().Hc(t)},g.Ic=function(t){return this.Hd().Ic(t)},g.Fb=function(t){return Nr(this.Hd(),t)},g.Xb=function(t){return ug(this,t)},g.Hb=function(){return ii(this.Hd())},g.Xc=function(t){return this.Hd().Xc(t)},g.dc=function(){return this.Hd().dc()},g.Ed=function(){return J4(this.Hd().Kc())},g.gc=function(){return this.Hd().gc()},g.Gd=function(t,n){return dF(this.Hd().bd(t,n))},g.Pc=function(){return this.Hd().Qc(pe(yn,st,1,this.Hd().gc(),5,1))},g.Qc=function(t){return this.Hd().Qc(t)},g.Ib=function(){return Ga(this.Hd())},D(mt,"ForwardingImmutableList",2006),M(714,1,kx),g.vc=function(){return ob(this)},g.wc=function(t){ZE(this,t)},g.ec=function(){return fW(this)},g.yc=function(t,n,s){return wX(this,t,n,s)},g.Cc=function(){return this.Ld()},g.$b=function(){throw ne(new Xn)},g._b=function(t){return this.xc(t)!=null},g.uc=function(t){return this.Ld().Hc(t)},g.Jd=function(){return new MKe(this)},g.Kd=function(){return new DKe(this)},g.Fb=function(t){return y6n(this,t)},g.Hb=function(){return ob(this).Hb()},g.dc=function(){return this.gc()==0},g.zc=function(t,n){return Kfn()},g.Bc=function(t){throw ne(new Xn)},g.Ib=function(){return Z9n(this)},g.Ld=function(){return this.e?this.e:this.e=this.Kd()},g.c=null,g.d=null,g.e=null;var n3t;D(mt,"ImmutableMap",714),M(715,714,kx),g._b=function(t){return eQe(this,t)},g.uc=function(t){return XQe(this.b,t)},g.Id=function(){return Sht(new JUe(this))},g.Jd=function(){return Sht(ait(this.b))},g.Kd=function(){return t1(),new $le(sit(this.b))},g.Fb=function(t){return QQe(this.b,t)},g.xc=function(t){return _E(this,t)},g.Hb=function(){return ii(this.b.c)},g.dc=function(){return this.b.c.dc()},g.gc=function(){return this.b.c.gc()},g.Ib=function(){return Ga(this.b.c)},D(mt,"ForwardingImmutableMap",715),M(1974,1973,gZ),g.Bd=function(){return this.Md()},g.Cd=function(){return this.Md()},g.Nc=function(){return new Wt(this,1)},g.Fb=function(t){return t===this||this.Md().Fb(t)},g.Hb=function(){return this.Md().Hb()},D(mt,"ForwardingSet",1974),M(1069,1974,gZ,JUe),g.Bd=function(){return dk(this.a.b)},g.Cd=function(){return dk(this.a.b)},g.Hc=function(t){if(ue(t,42)&&h(t,42).cd()==null)return!1;try{return KQe(dk(this.a.b),t)}catch(n){if(n=ui(n),ue(n,205))return!1;throw ne(n)}},g.Md=function(){return dk(this.a.b)},g.Qc=function(t){var n;return n=Hit(dk(this.a.b),t),dk(this.a.b).b.gc()<n.length&&pi(n,dk(this.a.b).b.gc(),null),n},D(mt,"ForwardingImmutableMap/1",1069),M(1981,1980,xx),g.Kc=function(){return this.Ed()},g.Nc=function(){return new Wt(this,1)},g.Fb=function(t){return tbe(this,t)},g.Hb=function(){return Kut(this)},D(mt,"ImmutableSet",1981),M(703,1981,xx),g.Kc=function(){return J4(new F6(this.a.b.Kc()))},g.Hc=function(t){return t!=null&&kS(this.a,t)},g.Ic=function(t){return YQe(this.a,t)},g.Hb=function(){return ii(this.a.b)},g.dc=function(){return this.a.b.dc()},g.Ed=function(){return J4(new F6(this.a.b.Kc()))},g.gc=function(){return this.a.b.gc()},g.Pc=function(){return this.a.b.Pc()},g.Qc=function(t){return WQe(this.a,t)},g.Ib=function(){return Ga(this.a.b)},D(mt,"ForwardingImmutableSet",703),M(1975,1974,zbt),g.Bd=function(){return this.b},g.Cd=function(){return this.b},g.Md=function(){return this.b},g.Nc=function(){return new MN(this)},D(mt,"ForwardingSortedSet",1975),M(533,1979,kx,BF),g.Ac=function(t){QE(this,t)},g.Cc=function(){var t;return t=this.d,new MY(t||(this.d=new bq(this)))},g.$b=function(){gA(this)},g._b=function(t){return!!KE(this,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))))},g.uc=function(t){return sut(this,t)},g.kc=function(){return new Net(this,this)},g.wc=function(t){Zit(this,t)},g.xc=function(t){return wy(this,t)},g.ec=function(){return new DY(this)},g.zc=function(t,n){return lR(this,t,n)},g.Bc=function(t){var n;return n=KE(this,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15)))),n?(Py(this,n),n.e=null,n.c=null,n.i):null},g.gc=function(){return this.i},g.pd=function(){var t;return t=this.d,new MY(t||(this.d=new bq(this)))},g.f=0,g.g=0,g.i=0,D(mt,"HashBiMap",533),M(534,1,ys),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return vat(this)},g.Pb=function(){var t;if(!vat(this))throw ne(new lo);return t=this.c,this.c=t.c,this.f=t,--this.d,this.Nd(t)},g.Qb=function(){if(this.e.g!=this.b)throw ne(new cl);yy(!!this.f),Py(this.e,this.f),this.b=this.e.g,this.f=null},g.b=0,g.d=0,g.f=null,D(mt,"HashBiMap/Itr",534),M(1011,534,ys,Net),g.Nd=function(t){return new RQe(this,t)},D(mt,"HashBiMap/1",1011),M(1012,345,ER,RQe),g.cd=function(){return this.a.g},g.dd=function(){return this.a.i},g.ed=function(t){var n,s,a;return s=this.a.i,a=Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))),a==this.a.f&&(Me(t)===Me(s)||t!=null&&Nr(t,s))?t:(Dlt(!XE(this.b.a,t,a),t),Py(this.b.a,this.a),n=new GP(this.a.g,this.a.a,t,a),dM(this.b.a,n,this.a),this.a.e=null,this.a.c=null,this.b.b=this.b.a.g,this.b.f==this.a&&(this.b.f=n),this.a=n,s)},D(mt,"HashBiMap/1/MapEntry",1012),M(238,345,{345:1,238:1,3:1,42:1},Z2),g.cd=function(){return this.g},g.dd=function(){return this.i},g.ed=function(t){throw ne(new Xn)},D(mt,"ImmutableEntry",238),M(317,238,{345:1,317:1,238:1,3:1,42:1},GP),g.a=0,g.f=0;var Xee=D(mt,"HashBiMap/BiEntry",317);M(610,1979,kx,bq),g.Ac=function(t){QE(this,t)},g.Cc=function(){return new DY(this.a)},g.$b=function(){gA(this.a)},g._b=function(t){return sut(this.a,t)},g.kc=function(){return new Pet(this,this.a)},g.wc=function(t){Vn(t),Zit(this.a,new eqe(t))},g.xc=function(t){return YB(this,t)},g.ec=function(){return new MY(this)},g.zc=function(t,n){return xDn(this.a,t,n,!1)},g.Bc=function(t){var n;return n=XE(this.a,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15)))),n?(Py(this.a,n),n.e=null,n.c=null,n.g):null},g.gc=function(){return this.a.i},g.pd=function(){return new DY(this.a)},D(mt,"HashBiMap/Inverse",610),M(1008,534,ys,Pet),g.Nd=function(t){return new jQe(this,t)},D(mt,"HashBiMap/Inverse/1",1008),M(1009,345,ER,jQe),g.cd=function(){return this.a.i},g.dd=function(){return this.a.g},g.ed=function(t){var n,s,a;return a=this.a.g,n=Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))),n==this.a.a&&(Me(t)===Me(a)||t!=null&&Nr(t,a))?t:(Dlt(!KE(this.b.a.a,t,n),t),Py(this.b.a.a,this.a),s=new GP(t,n,this.a.i,this.a.f),this.a=s,dM(this.b.a.a,s,null),this.b.b=this.b.a.a.g,a)},D(mt,"HashBiMap/Inverse/1/InverseEntry",1009),M(611,532,$c,MY),g.Kc=function(){return new xXe(this.a.a)},g.Mc=function(t){var n;return n=XE(this.a.a,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15)))),n?(Py(this.a.a,n),!0):!1},D(mt,"HashBiMap/Inverse/InverseKeySet",611),M(1007,534,ys,xXe),g.Nd=function(t){return t.i},D(mt,"HashBiMap/Inverse/InverseKeySet/1",1007),M(1010,1,{},eqe),g.Od=function(t,n){Sfn(this.a,t,n)},D(mt,"HashBiMap/Inverse/lambda$0$Type",1010),M(609,532,$c,DY),g.Kc=function(){return new EXe(this.a)},g.Mc=function(t){var n;return n=KE(this.a,t,Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15)))),n?(Py(this.a,n),n.e=null,n.c=null,!0):!1},D(mt,"HashBiMap/KeySet",609),M(1006,534,ys,EXe),g.Nd=function(t){return t.g},D(mt,"HashBiMap/KeySet/1",1006),M(1093,619,Zp),D(mt,"HashMultimapGwtSerializationDependencies",1093),M(265,1093,Zp,pb),g.hc=function(){return new gN(vw(this.a))},g.gd=function(){return new gN(vw(this.a))},g.a=2,D(mt,"HashMultimap",265),M(1999,152,O9),g.Hc=function(t){return this.Pd().Hc(t)},g.dc=function(){return this.Pd().dc()},g.gc=function(){return this.Pd().gc()},D(mt,"ImmutableAsList",1999),M(1931,715,kx),g.Ld=function(){return t1(),new C4(this.a)},g.Cc=function(){return t1(),new C4(this.a)},g.pd=function(){return t1(),new C4(this.a)},D(mt,"ImmutableBiMap",1931),M(1977,1,{}),D(mt,"ImmutableCollection/Builder",1977),M(1022,703,xx,_Xe),D(mt,"ImmutableEnumSet",1022),M(969,386,yx,Gtt),g.Xb=function(t){return this.a.Xb(t)},D(mt,"ImmutableList/1",969),M(968,1977,{},Ett),D(mt,"ImmutableList/Builder",968),M(614,198,mx,vq),g.Ob=function(){return this.a.Ob()},g.Pb=function(){return h(this.a.Pb(),42).cd()},D(mt,"ImmutableMap/1",614),M(1041,1,{},C),g.Kb=function(t){return h(t,42).cd()},D(mt,"ImmutableMap/2methodref$getKey$Type",1041),M(1040,1,{},_tt),D(mt,"ImmutableMap/Builder",1040),M(2e3,1981,xx),g.Kc=function(){var t;return t=ob(this.a).Ed(),new vq(t)},g.Dd=function(){return new Hle(this)},g.Jc=function(t){var n,s;for(Vn(t),s=this.gc(),n=0;n<s;n++)t.td(h(yde(ob(this.a)).Xb(n),42).cd())},g.Ed=function(){var t;return(t=this.c,t||(this.c=new Hle(this))).Ed()},g.Nc=function(){return iW(this.gc(),1296,new nqe(this))},D(mt,"IndexedImmutableSet",2e3),M(1180,2e3,xx,MKe),g.Kc=function(){var t;return t=ob(this.a).Ed(),new vq(t)},g.Hc=function(t){return this.a._b(t)},g.Jc=function(t){Vn(t),ZE(this.a,new tqe(t))},g.Ed=function(){var t;return t=ob(this.a).Ed(),new vq(t)},g.gc=function(){return this.a.gc()},g.Nc=function(){return QS(ob(this.a).Nc(),new C)},D(mt,"ImmutableMapKeySet",1180),M(1181,1,{},tqe),g.Od=function(t,n){t1(),this.a.td(t)},D(mt,"ImmutableMapKeySet/lambda$0$Type",1181),M(1178,1980,Xve,DKe),g.Kc=function(){return new tW(this)},g.Hc=function(t){return t!=null&&uEn(new tW(this),t)},g.Ed=function(){return new tW(this)},g.gc=function(){return this.a.gc()},g.Nc=function(){return QS(ob(this.a).Nc(),new S)},D(mt,"ImmutableMapValues",1178),M(1179,1,{},S),g.Kb=function(t){return h(t,42).dd()},D(mt,"ImmutableMapValues/0methodref$getValue$Type",1179),M(626,198,mx,tW),g.Ob=function(){return this.a.Ob()},g.Pb=function(){return h(this.a.Pb(),42).dd()},D(mt,"ImmutableMapValues/1",626),M(1182,1,{},nqe),g.ld=function(t){return Art(this.a,t)},D(mt,"IndexedImmutableSet/0methodref$get$Type",1182),M(752,1999,O9,Hle),g.Pd=function(){return this.a},g.Xb=function(t){return Art(this.a,t)},g.gc=function(){return this.a.a.gc()},D(mt,"IndexedImmutableSet/1",752),M(44,1,{},L),g.Kb=function(t){return h(t,20).Kc()},g.Fb=function(t){return this===t},D(mt,"Iterables/10",44),M(1042,537,G1,Mnt),g.Jc=function(t){Vn(t),this.b.Jc(new $Qe(this.a,t))},g.Kc=function(){return Efe(this)},D(mt,"Iterables/4",1042),M(1043,1,mn,$Qe),g.td=function(t){W1n(this.b,this.a,t)},D(mt,"Iterables/4/lambda$0$Type",1043),M(1044,537,G1,Dnt),g.Jc=function(t){Vn(t),Os(this.a,new BQe(t,this.b))},g.Kc=function(){return Dn(new Tn(this.a),this.b)},D(mt,"Iterables/5",1044),M(1045,1,mn,BQe),g.td=function(t){this.a.td(ZJe(t))},D(mt,"Iterables/5/lambda$0$Type",1045),M(1071,198,mx,rqe),g.Ob=function(){return this.a.Ob()},g.Pb=function(){return this.a.Pb()},D(mt,"Iterators/1",1071),M(1072,699,mx,FQe),g.Yb=function(){for(var t;this.b.Ob();)if(t=this.b.Pb(),this.a.Lb(t))return t;return this.e=2,null},D(mt,"Iterators/5",1072),M(487,1,ys),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.b.Ob()},g.Pb=function(){return this.Qd(this.b.Pb())},g.Qb=function(){this.b.Qb()},D(mt,"TransformedIterator",487),M(1073,487,ys,Bet),g.Qd=function(t){return this.a.Kb(t)},D(mt,"Iterators/6",1073),M(717,198,mx,cle),g.Ob=function(){return!this.a},g.Pb=function(){if(this.a)throw ne(new lo);return this.a=!0,this.b},g.a=!1,D(mt,"Iterators/9",717),M(1070,386,yx,Jnt),g.Xb=function(t){return this.a[this.b+t]},g.b=0;var r3t;D(mt,"Iterators/ArrayItr",1070),M(39,1,{39:1,47:1},Sn),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return rr(this)},g.Pb=function(){return Un(this)},g.Qb=function(){yy(!!this.c),this.c.Qb(),this.c=null},D(mt,"Iterators/ConcatenatedIterator",39),M(22,1,{3:1,35:1,22:1}),g.wd=function(t){return AXe(this,h(t,22))},g.Fb=function(t){return this===t},g.Hb=function(){return rb(this)},g.Ib=function(){return FY(this)},g.g=0;var sr=D(Ja,"Enum",22);M(538,22,{538:1,3:1,35:1,22:1,47:1},$et),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return!1},g.Pb=function(){throw ne(new lo)},g.Qb=function(){yy(!1)};var Qee,i3t=tr(mt,"Iterators/EmptyModifiableIterator",538,sr,zbn,E0n),s3t;M(1834,619,Zp),D(mt,"LinkedHashMultimapGwtSerializationDependencies",1834),M(1835,1834,Zp,vht),g.hc=function(){return new NY(vw(this.b))},g.$b=function(){VB(this),gS(this.a,this.a)},g.gd=function(){return new NY(vw(this.b))},g.ic=function(t){return new nht(this,t,this.b)},g.kc=function(){return new jfe(this)},g.lc=function(){var t;return new Wt((t=this.g,h(t||(this.g=new ahe(this)),21)),17)},g.ec=function(){var t;return t=this.i,t||(this.i=new hy(this,this.c))},g.nc=function(){return new hhe(new jfe(this))},g.oc=function(){var t;return QS(new Wt((t=this.g,h(t||(this.g=new ahe(this)),21)),17),new O)},g.b=2,D(mt,"LinkedHashMultimap",1835),M(1838,1,{},O),g.Kb=function(t){return h(t,42).dd()},D(mt,"LinkedHashMultimap/0methodref$getValue$Type",1838),M(824,1,ys,jfe),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return c4n(this)},g.Ob=function(){return this.a!=this.b.a},g.Qb=function(){yy(!!this.c),Wst(this.b,this.c.g,this.c.i),this.c=null},D(mt,"LinkedHashMultimap/1",824),M(330,238,{345:1,238:1,330:1,2020:1,3:1,42:1},Hde),g.Rd=function(){return this.f},g.Sd=function(t){this.c=t},g.Td=function(t){this.f=t},g.d=0;var a3t=D(mt,"LinkedHashMultimap/ValueEntry",330);M(1836,1970,{2020:1,20:1,28:1,14:1,21:1},nht),g.Fc=function(t){var n,s,a,u,p;for(p=Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))),n=p&this.b.length-1,u=this.b[n],s=u;s;s=s.a)if(s.d==p&&n1(s.i,t))return!1;return a=new Hde(this.c,t,p,u),BXe(this.d,a),a.f=this,this.d=a,gS(this.g.a.b,a),gS(a,this.g.a),this.b[n]=a,++this.f,++this.e,aEn(this),!0},g.$b=function(){var t,n;for(CS(this.b,null),this.f=0,t=this.a;t!=this;t=t.Rd())n=h(t,330),gS(n.b,n.e);this.a=this,this.d=this,++this.e},g.Hc=function(t){var n,s;for(s=Gn(js(V1,B1(Gn(js(t==null?0:ii(t),U1)),15))),n=this.b[s&this.b.length-1];n;n=n.a)if(n.d==s&&n1(n.i,t))return!0;return!1},g.Jc=function(t){var n;for(Vn(t),n=this.a;n!=this;n=n.Rd())t.td(h(n,330).i)},g.Rd=function(){return this.a},g.Kc=function(){return new Wrt(this)},g.Mc=function(t){return U0t(this,t)},g.Sd=function(t){this.d=t},g.Td=function(t){this.a=t},g.gc=function(){return this.f},g.e=0,g.f=0,D(mt,"LinkedHashMultimap/ValueSet",1836),M(1837,1,ys,Wrt),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return ude(this),this.b!=this.c},g.Pb=function(){var t,n;if(ude(this),this.b==this.c)throw ne(new lo);return t=h(this.b,330),n=t.i,this.d=t,this.b=t.f,n},g.Qb=function(){ude(this),yy(!!this.d),U0t(this.c,this.d.i),this.a=this.c.e,this.d=null},g.a=0,D(mt,"LinkedHashMultimap/ValueSet/1",1837),M(766,1986,Zp,RJe),g.Zb=function(){var t;return t=this.f,t||(this.f=new Fhe(this))},g.Fb=function(t){return tX(this,t)},g.cc=function(t){return new Vq(this,t)},g.fc=function(t){return y0e(this,t)},g.$b=function(){Irt(this)},g._b=function(t){return ZQe(this,t)},g.ac=function(){return new Fhe(this)},g.bc=function(){return new gqe(this)},g.qc=function(t){return new Vq(this,t)},g.dc=function(){return!this.a},g.rc=function(t){return y0e(this,t)},g.gc=function(){return this.d},g.c=0,g.d=0,D(mt,"LinkedListMultimap",766),M(52,28,Ex),g.ad=function(t){my(this,t)},g.Nc=function(){return new Wt(this,16)},g.Vc=function(t,n){throw ne(new i0("Add not supported on this list"))},g.Fc=function(t){return this.Vc(this.gc(),t),!0},g.Wc=function(t,n){var s,a,u;for(rn(n),s=!1,u=n.Kc();u.Ob();)a=u.Pb(),this.Vc(t++,a),s=!0;return s},g.$b=function(){this.Ud(0,this.gc())},g.Fb=function(t){return Nbe(this,t)},g.Hb=function(){return qge(this)},g.Xc=function(t){return Aut(this,t)},g.Kc=function(){return new L7(this)},g.Yc=function(){return this.Zc(0)},g.Zc=function(t){return new Ms(this,t)},g.$c=function(t){throw ne(new i0("Remove not supported on this list"))},g.Ud=function(t,n){var s,a;for(a=this.Zc(t),s=t;s<n;++s)a.Pb(),a.Qb()},g._c=function(t,n){throw ne(new i0("Set not supported on this list"))},g.bd=function(t,n){return new F1(this,t,n)},g.j=0,D(Nn,"AbstractList",52),M(1964,52,Ex),g.Vc=function(t,n){rk(this,t,n)},g.Wc=function(t,n){return iht(this,t,n)},g.Xb=function(t){return Zh(this,t)},g.Kc=function(){return this.Zc(0)},g.$c=function(t){return ZX(this,t)},g._c=function(t,n){var s,a;s=this.Zc(t);try{return a=s.Pb(),s.Wb(n),a}catch(u){throw u=ui(u),ue(u,109)?ne(new Ea("Can't set element "+t)):ne(u)}},D(Nn,"AbstractSequentialList",1964),M(636,1964,Ex,Vq),g.Zc=function(t){return Vet(this,t)},g.gc=function(){var t;return t=h(_n(this.a.b,this.b),283),t?t.a:0},D(mt,"LinkedListMultimap/1",636),M(1297,1970,$c,gqe),g.Hc=function(t){return ZQe(this.a,t)},g.Kc=function(){return new Vut(this.a)},g.Mc=function(t){return!y0e(this.a,t).a.dc()},g.gc=function(){return q7(this.a.b)},D(mt,"LinkedListMultimap/1KeySetImpl",1297),M(1296,1,ys,Vut),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return cde(this),!!this.c},g.Pb=function(){cde(this),zfe(this.c),this.a=this.c,Qi(this.d,this.a.a);do this.c=this.c.b;while(!!this.c&&!Qi(this.d,this.c.a));return this.a.a},g.Qb=function(){cde(this),yy(!!this.a),CP(new EK(this.e,this.a.a)),this.a=null,this.b=this.e.c},g.b=0,D(mt,"LinkedListMultimap/DistinctKeyIterator",1296),M(283,1,{283:1},Xde),g.a=0,D(mt,"LinkedListMultimap/KeyList",283),M(1295,345,ER,HQe),g.cd=function(){return this.a},g.dd=function(){return this.f},g.ed=function(t){var n;return n=this.f,this.f=t,n},D(mt,"LinkedListMultimap/Node",1295),M(560,1,z1,EK,Rdt),g.Nb=function(t){Ls(this,t)},g.Rb=function(t){this.e=uve(this.f,this.b,t,this.c),++this.d,this.a=null},g.Ob=function(){return!!this.c},g.Sb=function(){return!!this.e},g.Pb=function(){return n0e(this)},g.Tb=function(){return this.d},g.Ub=function(){return lst(this)},g.Vb=function(){return this.d-1},g.Qb=function(){yy(!!this.a),this.a!=this.c?(this.e=this.a.e,--this.d):this.c=this.a.c,hTn(this.f,this.a),this.a=null},g.Wb=function(t){Hfe(!!this.a),this.a.f=t},g.d=0,D(mt,"LinkedListMultimap/ValueForKeyIterator",560),M(1018,52,Ex),g.Vc=function(t,n){this.a.Vc(t,n)},g.Wc=function(t,n){return this.a.Wc(t,n)},g.Hc=function(t){return this.a.Hc(t)},g.Xb=function(t){return this.a.Xb(t)},g.$c=function(t){return this.a.$c(t)},g._c=function(t,n){return this.a._c(t,n)},g.gc=function(){return this.a.gc()},D(mt,"Lists/AbstractListWrapper",1018),M(1019,1018,Vbt),D(mt,"Lists/RandomAccessListWrapper",1019),M(1021,1019,Vbt,qQe),g.Zc=function(t){return this.a.Zc(t)},D(mt,"Lists/1",1021),M(131,52,{131:1,20:1,28:1,52:1,14:1,15:1},Xv),g.Vc=function(t,n){this.a.Vc(uk(this,t),n)},g.$b=function(){this.a.$b()},g.Xb=function(t){return this.a.Xb(Pde(this,t))},g.Kc=function(){return a0e(this,0)},g.Zc=function(t){return a0e(this,t)},g.$c=function(t){return this.a.$c(Pde(this,t))},g.Ud=function(t,n){(tat(t,n,this.a.gc()),Rp(this.a.bd(uk(this,n),uk(this,t)))).$b()},g._c=function(t,n){return this.a._c(Pde(this,t),n)},g.gc=function(){return this.a.gc()},g.bd=function(t,n){return tat(t,n,this.a.gc()),Rp(this.a.bd(uk(this,n),uk(this,t)))},D(mt,"Lists/ReverseList",131),M(280,131,{131:1,20:1,28:1,52:1,14:1,15:1,54:1},U2),D(mt,"Lists/RandomAccessReverseList",280),M(1020,1,z1,zQe),g.Nb=function(t){Ls(this,t)},g.Rb=function(t){this.c.Rb(t),this.c.Ub(),this.a=!1},g.Ob=function(){return this.c.Sb()},g.Sb=function(){return this.c.Ob()},g.Pb=function(){return iot(this)},g.Tb=function(){return uk(this.b,this.c.Tb())},g.Ub=function(){if(!this.c.Ob())throw ne(new lo);return this.a=!0,this.c.Pb()},g.Vb=function(){return uk(this.b,this.c.Tb())-1},g.Qb=function(){yy(this.a),this.c.Qb(),this.a=!1},g.Wb=function(t){Hfe(this.a),this.c.Wb(t)},g.a=!1,D(mt,"Lists/ReverseList/1",1020),M(432,487,ys,R7),g.Qd=function(t){return AN(t)},D(mt,"Maps/1",432),M(698,487,ys,hhe),g.Qd=function(t){return h(t,42).dd()},D(mt,"Maps/2",698),M(962,487,ys,Get),g.Qd=function(t){return new Z2(t,GJe(this.a,t))},D(mt,"Maps/3",962),M(959,1971,$c,dqe),g.Jc=function(t){Ifn(this.a,t)},g.Kc=function(){return this.a.kc()},g.Rc=function(){return this.a},g.Nc=function(){return this.a.lc()},D(mt,"Maps/IteratorBasedAbstractMap/1",959),M(960,1,{},fqe),g.Od=function(t,n){this.a.td(t)},D(mt,"Maps/KeySet/lambda$0$Type",960),M(958,28,Rw,UQe),g.$b=function(){this.a.$b()},g.Hc=function(t){return this.a.uc(t)},g.Jc=function(t){Vn(t),this.a.wc(new aqe(t))},g.dc=function(){return this.a.dc()},g.Kc=function(){return new hhe(this.a.vc().Kc())},g.Mc=function(t){var n,s;try{return Ew(this,t,!0)}catch(a){if(a=ui(a),ue(a,41)){for(s=this.a.vc().Kc();s.Ob();)if(n=h(s.Pb(),42),n1(t,n.dd()))return this.a.Bc(n.cd()),!0;return!1}else throw ne(a)}},g.gc=function(){return this.a.gc()},D(mt,"Maps/Values",958),M(961,1,{},aqe),g.Od=function(t,n){this.a.td(n)},D(mt,"Maps/Values/lambda$0$Type",961),M(736,1987,zb,Fhe),g.xc=function(t){return this.a._b(t)?this.a.cc(t):null},g.Bc=function(t){return this.a._b(t)?this.a.fc(t):null},g.$b=function(){this.a.$b()},g._b=function(t){return this.a._b(t)},g.Ec=function(){return new sqe(this)},g.Dc=function(){return this.Ec()},g.dc=function(){return this.a.dc()},g.ec=function(){return this.a.ec()},g.gc=function(){return this.a.ec().gc()},D(mt,"Multimaps/AsMap",736),M(1104,1971,$c,sqe),g.Kc=function(){return Kdn(this.a.a.ec(),new oqe(this))},g.Rc=function(){return this.a},g.Mc=function(t){var n;return nft(this,t)?(n=h(t,42),Xfn(this.a,n.cd()),!0):!1},D(mt,"Multimaps/AsMap/EntrySet",1104),M(1108,1,{},oqe),g.Kb=function(t){return GJe(this,t)},g.Fb=function(t){return this===t},D(mt,"Multimaps/AsMap/EntrySet/1",1108),M(543,1989,{543:1,835:1,20:1,28:1,14:1},cqe),g.$b=function(){VB(this.a)},g.Hc=function(t){return JXe(this.a,t)},g.Jc=function(t){Vn(t),Os(OE(this.a),new lqe(t))},g.Kc=function(){return new R7(OE(this.a).a.kc())},g.gc=function(){return this.a.d},g.Nc=function(){return QS(OE(this.a).Nc(),new B)},D(mt,"Multimaps/Keys",543),M(1106,1,{},B),g.Kb=function(t){return h(t,42).cd()},D(mt,"Multimaps/Keys/0methodref$getKey$Type",1106),M(1105,487,ys,TXe),g.Qd=function(t){return new uqe(h(t,42))},D(mt,"Multimaps/Keys/1",1105),M(1990,1,{416:1}),g.Fb=function(t){var n;return ue(t,492)?(n=h(t,416),h(this.a.dd(),14).gc()==h(n.a.dd(),14).gc()&&n1(this.a.cd(),n.a.cd())):!1},g.Hb=function(){var t;return t=this.a.cd(),(t==null?0:ii(t))^h(this.a.dd(),14).gc()},g.Ib=function(){var t,n;return n=mE(this.a.cd()),t=h(this.a.dd(),14).gc(),t==1?n:n+" x "+t},D(mt,"Multisets/AbstractEntry",1990),M(492,1990,{492:1,416:1},uqe),D(mt,"Multimaps/Keys/1/1",492),M(1107,1,mn,lqe),g.td=function(t){this.a.td(h(t,42).cd())},D(mt,"Multimaps/Keys/lambda$1$Type",1107),M(1110,1,mn,N),g.td=function(t){X2n(h(t,416))},D(mt,"Multiset/lambda$0$Type",1110),M(737,1,mn,hqe),g.td=function(t){d4n(this.a,h(t,416))},D(mt,"Multiset/lambda$1$Type",737),M(1111,1,{},H),D(mt,"Multisets/0methodref$add$Type",1111),M(738,1,{},Y),g.Kb=function(t){return ryn(h(t,416))},D(mt,"Multisets/lambda$3$Type",738),M(2008,1,IM),D(mt,"RangeGwtSerializationDependencies",2008),M(514,2008,{169:1,514:1,3:1,45:1},Jpe),g.Lb=function(t){return prt(this,h(t,35))},g.Mb=function(t){return prt(this,h(t,35))},g.Fb=function(t){var n;return ue(t,514)?(n=h(t,514),Gpe(this.a,n.a)&&Gpe(this.b,n.b)):!1},g.Hb=function(){return this.a.Hb()*31+this.b.Hb()},g.Ib=function(){return Eat(this.a,this.b)},D(mt,"Range",514),M(778,1999,O9,Crt),g.Zc=function(t){return Zfe(this.b,t)},g.Pd=function(){return this.a},g.Xb=function(t){return ug(this.b,t)},g.Fd=function(t){return Zfe(this.b,t)},D(mt,"RegularImmutableAsList",778),M(646,2006,O9,jW),g.Hd=function(){return this.a},D(mt,"RegularImmutableList",646),M(616,715,kx,Iq),D(mt,"RegularImmutableMap",616),M(716,703,xx,jhe);var g3e;D(mt,"RegularImmutableSet",716),M(1976,Lf,$c),g.Kc=function(){return new Vde(this.a,this.b)},g.Fc=function(t){throw ne(new Xn)},g.Gc=function(t){throw ne(new Xn)},g.$b=function(){throw ne(new Xn)},g.Mc=function(t){throw ne(new Xn)},D(mt,"Sets/SetView",1976),M(963,1976,$c,GQe),g.Kc=function(){return new Vde(this.a,this.b)},g.Hc=function(t){return cW(this.a,t)&&this.b.Hc(t)},g.Ic=function(t){return FA(this.a,t)&&this.b.Ic(t)},g.dc=function(){return Hft(this.b,this.a)},g.Lc=function(){return Jr(new Vt(null,new Wt(this.a,1)),new bqe(this.b))},g.gc=function(){return NA(this)},g.Oc=function(){return Jr(new Vt(null,new Wt(this.a,1)),new pqe(this.b))},D(mt,"Sets/2",963),M(700,699,mx,Vde),g.Yb=function(){for(var t;Sfe(this.a);)if(t=zE(this.a),this.c.Hc(t))return t;return this.e=2,null},D(mt,"Sets/2/1",700),M(964,1,Cr,pqe),g.Mb=function(t){return this.a.Hc(t)},D(mt,"Sets/2/4methodref$contains$Type",964),M(965,1,Cr,bqe),g.Mb=function(t){return this.a.Hc(t)},D(mt,"Sets/2/5methodref$contains$Type",965),M(607,1975,{607:1,3:1,20:1,14:1,271:1,21:1,84:1},qit),g.Bd=function(){return this.b},g.Cd=function(){return this.b},g.Md=function(){return this.b},g.Jc=function(t){this.a.Jc(t)},g.Lc=function(){return this.a.Lc()},g.Oc=function(){return this.a.Oc()},D(mt,"Sets/UnmodifiableNavigableSet",607),M(1932,1931,kx,eit),g.Ld=function(){return t1(),new C4(this.a)},g.Cc=function(){return t1(),new C4(this.a)},g.pd=function(){return t1(),new C4(this.a)},D(mt,"SingletonImmutableBiMap",1932),M(647,2006,O9,sW),g.Hd=function(){return this.a},D(mt,"SingletonImmutableList",647),M(350,1981,xx,C4),g.Kc=function(){return new cle(this.a)},g.Hc=function(t){return Nr(this.a,t)},g.Ed=function(){return new cle(this.a)},g.gc=function(){return 1},D(mt,"SingletonImmutableSet",350),M(1115,1,{},z),g.Kb=function(t){return h(t,164)},D(mt,"Streams/lambda$0$Type",1115),M(1116,1,CR,vqe),g.Vd=function(){Wyn(this.a)},D(mt,"Streams/lambda$1$Type",1116),M(1659,1658,Zp,Rit),g.Zb=function(){var t;return t=this.f,h(h(t||(this.f=ue(this.c,171)?new dP(this,h(this.c,171)):ue(this.c,161)?new VS(this,h(this.c,161)):new W6(this,this.c)),161),171)},g.hc=function(){return new ag(this.b)},g.gd=function(){return new ag(this.b)},g.ec=function(){var t;return t=this.i,h(h(t||(this.i=ue(this.c,171)?new Y6(this,h(this.c,171)):ue(this.c,161)?new yS(this,h(this.c,161)):new hy(this,this.c)),84),271)},g.ac=function(){return ue(this.c,171)?new dP(this,h(this.c,171)):ue(this.c,161)?new VS(this,h(this.c,161)):new W6(this,this.c)},g.ic=function(t){return t==null&&this.a.ue(t,t),new ag(this.b)},D(mt,"TreeMultimap",1659),M(78,1,{3:1,78:1}),g.Wd=function(t){return new Error(t)},g.Xd=function(){return this.e},g.Yd=function(){return oxn(pc(wW((this.k==null&&(this.k=pe(Zee,Re,78,0,0,1)),this.k)),new Be))},g.Zd=function(){return this.f},g.$d=function(){return this.g},g._d=function(){Ffn(this,Hmn(this.Wd($W(this,this.g)))),LKe(this)},g.Ib=function(){return $W(this,this.$d())},g.e=Ubt,g.i=!1,g.n=!0;var Zee=D(Ja,"Throwable",78);M(102,78,{3:1,102:1,78:1}),D(Ja,"Exception",102),M(60,102,Fd,Ep,Ya),D(Ja,"RuntimeException",60),M(598,60,Fd),D(Ja,"JsException",598),M(863,598,Fd),D(BM,"JavaScriptExceptionBase",863),M(477,863,{477:1,3:1,102:1,60:1,78:1},bht),g.$d=function(){return DEn(this),this.c},g.ae=function(){return Me(this.b)===Me(p3e)?null:this.b};var p3e;D(Zve,"JavaScriptException",477);var o3t=D(Zve,"JavaScriptObject$",0),Jee;M(1948,1,{}),D(Zve,"Scheduler",1948);var Nj=0,c3t=0,Pj=-1;M(890,1948,{},W);var b3e;D(BM,"SchedulerImpl",890);var ete;M(1960,1,{}),D(BM,"StackTraceCreator/Collector",1960),M(864,1960,{},Z),g.be=function(t){var n={},s=[];t[vZ]=s;for(var a=arguments.callee.caller;a;){var u=(bk(),a.name||(a.name=M4n(a.toString())));s.push(u);var p=":"+u,v=n[p];if(v){var m,x;for(m=0,x=v.length;m<x;m++)if(v[m]===a)return}(v||(n[p]=[])).push(a),a=a.caller}},g.ce=function(t){var n,s,a,u;for(a=(bk(),t&&t[vZ]?t[vZ]:[]),s=a.length,u=pe(D3e,Re,310,s,0,1),n=0;n<s;n++)u[n]=new LW(a[n],null,-1);return u},D(BM,"StackTraceCreator/CollectorLegacy",864),M(1961,1960,{}),g.be=function(t){},g.de=function(t,n,s,a){return new LW(n,t+"@"+a,s<0?-1:s)},g.ce=function(t){var n,s,a,u,p,v;if(u=n7n(t),p=pe(D3e,Re,310,0,0,1),n=0,a=u.length,a==0)return p;for(v=S2t(this,u[0]),Nt(v.d,bZ)||(p[n++]=v),s=1;s<a;s++)p[n++]=S2t(this,u[s]);return p},D(BM,"StackTraceCreator/CollectorModern",1961),M(865,1961,{},R),g.de=function(t,n,s,a){return new LW(n,t,-1)},D(BM,"StackTraceCreator/CollectorModernNoSourceMap",865),M(1050,1,{}),D(ewe,Wbt,1050),M(615,1050,{615:1},zrt);var v3e;D(BZ,Wbt,615),M(2001,1,{}),D(ewe,Kbt,2001),M(2002,2001,{}),D(BZ,Kbt,2002),M(1090,1,{},q);var h_;D(BZ,"LocaleInfo",1090),M(1918,1,{},X),g.a=0,D(BZ,"TimeZone",1918),M(1258,2002,{},F),D("com.google.gwt.i18n.client.impl.cldr","DateTimeFormatInfoImpl",1258),M(434,1,{434:1},wnt),g.a=!1,g.b=0,D(ewe,"DateTimeFormat/PatternPart",434),M(199,1,Xbt,mN,zge,c1e),g.wd=function(t){return Rmn(this,h(t,199))},g.Fb=function(t){return ue(t,199)&&eb(xc(this.q.getTime()),xc(h(t,199).q.getTime()))},g.Hb=function(){var t;return t=xc(this.q.getTime()),Gn(ZW(t,ow(t,32)))},g.Ib=function(){var t,n,s;return s=-this.q.getTimezoneOffset(),t=(s>=0?"+":"")+(s/60|0),n=XN(d.Math.abs(s)%60),(odt(),w3t)[this.q.getDay()]+" "+m3t[this.q.getMonth()]+" "+XN(this.q.getDate())+" "+XN(this.q.getHours())+":"+XN(this.q.getMinutes())+":"+XN(this.q.getSeconds())+" GMT"+t+n+" "+this.q.getFullYear()};var Bj=D(Nn,"Date",199);M(1915,199,Xbt,N1t),g.a=!1,g.b=0,g.c=0,g.d=0,g.e=0,g.f=0,g.g=!1,g.i=0,g.j=0,g.k=0,g.n=0,g.o=0,g.p=0,D("com.google.gwt.i18n.shared.impl","DateRecord",1915),M(1966,1,{}),g.fe=function(){return null},g.ge=function(){return null},g.he=function(){return null},g.ie=function(){return null},g.je=function(){return null},D(b5,"JSONValue",1966),M(216,1966,{216:1},yp,ule),g.Fb=function(t){return ue(t,216)?c0e(this.a,h(t,216).a):!1},g.ee=function(){return vfn},g.Hb=function(){return Kde(this.a)},g.fe=function(){return this},g.Ib=function(){var t,n,s;for(s=new ju("["),n=0,t=this.a.length;n<t;n++)n>0&&(s.a+=","),ho(s,bw(this,n));return s.a+="]",s.a},D(b5,"JSONArray",216),M(483,1966,{483:1},lle),g.ee=function(){return wfn},g.ge=function(){return this},g.Ib=function(){return on(),""+this.a},g.a=!1;var u3t,l3t;D(b5,"JSONBoolean",483),M(985,60,Fd,CXe),D(b5,"JSONException",985),M(1023,1966,{},te),g.ee=function(){return Efn},g.Ib=function(){return _c};var h3t;D(b5,"JSONNull",1023),M(258,1966,{258:1},WO),g.Fb=function(t){return ue(t,258)?this.a==h(t,258).a:!1},g.ee=function(){return mfn},g.Hb=function(){return J6(this.a)},g.he=function(){return this},g.Ib=function(){return this.a+""},g.a=0,D(b5,"JSONNumber",258),M(183,1966,{183:1},_4,aS),g.Fb=function(t){return ue(t,183)?c0e(this.a,h(t,183).a):!1},g.ee=function(){return yfn},g.Hb=function(){return Kde(this.a)},g.ie=function(){return this},g.Ib=function(){var t,n,s,a,u,p,v;for(v=new ju("{"),t=!0,p=HK(this,pe(it,Re,2,0,6,1)),s=p,a=0,u=s.length;a<u;++a)n=s[a],t?t=!1:v.a+=Js,or(v,T0t(n)),v.a+=":",ho(v,Ed(this,n));return v.a+="}",v.a},D(b5,"JSONObject",183),M(596,Lf,$c,Uq),g.Hc=function(t){return ws(t)&&n1n(this.a,Jn(t))},g.Kc=function(){return new L7(new _u(this.b))},g.gc=function(){return this.b.length},D(b5,"JSONObject/1",596);var tte;M(204,1966,{204:1},uw),g.Fb=function(t){return ue(t,204)?Nt(this.a,h(t,204).a):!1},g.ee=function(){return kfn},g.Hb=function(){return k0(this.a)},g.je=function(){return this},g.Ib=function(){return T0t(this.a)},D(b5,"JSONString",204);var Rg,w3e,f3t,m3e,y3e;M(1962,1,{525:1}),D(twe,"OutputStream",1962),M(1963,1962,{525:1}),D(twe,"FilterOutputStream",1963),M(866,1963,{525:1},ke),D(twe,"PrintStream",866),M(418,1,{475:1}),g.Ib=function(){return this.a},D(Ja,"AbstractStringBuilder",418),M(529,60,Fd,cN),D(Ja,"ArithmeticException",529),M(73,60,FZ,Gle,Ea),D(Ja,"IndexOutOfBoundsException",73),M(320,73,{3:1,320:1,102:1,73:1,60:1,78:1},Ule,bhe),D(Ja,"ArrayIndexOutOfBoundsException",320),M(528,60,Fd,Eq,MXe),D(Ja,"ArrayStoreException",528),M(289,78,Qbt,Nq),D(Ja,"Error",289),M(194,289,Qbt,zle,T0e),D(Ja,"AssertionError",194),Jyt={3:1,476:1,35:1};var u2,Hx,es=D(Ja,"Boolean",476);M(236,1,{3:1,236:1});var k3e;D(Ja,"Number",236),M(217,236,{3:1,217:1,35:1,236:1},xqe),g.wd=function(t){return Jfn(this,h(t,217))},g.ke=function(){return this.a},g.Fb=function(t){return ue(t,217)&&h(t,217).a==this.a},g.Hb=function(){return this.a},g.Ib=function(){return""+this.a},g.a=0;var A5=D(Ja,"Byte",217),x3e;M(172,1,{3:1,172:1,35:1},vle),g.wd=function(t){return e1n(this,h(t,172))},g.Fb=function(t){return ue(t,172)&&h(t,172).a==this.a},g.Hb=function(){return this.a},g.Ib=function(){return String.fromCharCode(this.a)},g.a=0;var E3e,f_=D(Ja,"Character",172),_3e;M(205,60,{3:1,205:1,102:1,60:1,78:1},IKe,z6),D(Ja,"ClassCastException",205),e3t={3:1,35:1,333:1,236:1};var Ts=D(Ja,"Double",333);M(155,236,{3:1,35:1,155:1,236:1},cS,qle),g.wd=function(t){return cdn(this,h(t,155))},g.ke=function(){return this.a},g.Fb=function(t){return ue(t,155)&&Utt(this.a,h(t,155).a)},g.Hb=function(){return Li(this.a)},g.Ib=function(){return""+this.a},g.a=0;var zx=D(Ja,"Float",155);M(32,60,{3:1,102:1,32:1,60:1,78:1},pS,an,cht),D(Ja,"IllegalArgumentException",32),M(71,60,Fd,Ic,ja),D(Ja,"IllegalStateException",71),M(19,236,{3:1,35:1,19:1,236:1},wle),g.wd=function(t){return udn(this,h(t,19))},g.ke=function(){return this.a},g.Fb=function(t){return ue(t,19)&&h(t,19).a==this.a},g.Hb=function(){return this.a},g.Ib=function(){return""+this.a},g.a=0;var Ws=D(Ja,"Integer",19),T3e,d3t;M(162,236,{3:1,35:1,162:1,236:1},mle),g.wd=function(t){return ldn(this,h(t,162))},g.ke=function(){return vb(this.a)},g.Fb=function(t){return ue(t,162)&&eb(h(t,162).a,this.a)},g.Hb=function(){return Gn(this.a)},g.Ib=function(){return""+ME(this.a)},g.a=0;var Xb=D(Ja,"Long",162),C3e;M(2039,1,{}),M(1831,60,Fd,LXe),D(Ja,"NegativeArraySizeException",1831),M(173,598,{3:1,102:1,173:1,60:1,78:1},P7,T4),g.Wd=function(t){return new TypeError(t)},D(Ja,"NullPointerException",173);var S3e,nte,g3t,A3e;M(127,32,{3:1,102:1,32:1,127:1,60:1,78:1},Qf),D(Ja,"NumberFormatException",127),M(184,236,{3:1,35:1,236:1,184:1},yle),g.wd=function(t){return t1n(this,h(t,184))},g.ke=function(){return this.a},g.Fb=function(t){return ue(t,184)&&h(t,184).a==this.a},g.Hb=function(){return this.a},g.Ib=function(){return""+this.a},g.a=0;var Qb=D(Ja,"Short",184),M3e;M(310,1,{3:1,310:1},LW),g.Fb=function(t){var n;return ue(t,310)?(n=h(t,310),this.c==n.c&&this.d==n.d&&this.a==n.a&&this.b==n.b):!1},g.Hb=function(){return lF(se(ie(yn,1),st,1,5,[Ke(this.c),this.a,this.d,this.b]))},g.Ib=function(){return this.a+"."+this.d+"("+(this.b!=null?this.b:"Unknown Source")+(this.c>=0?":"+this.c:"")+")"},g.c=0;var D3e=D(Ja,"StackTraceElement",310);t3t={3:1,475:1,35:1,2:1};var it=D(Ja,Qve,2);M(107,418,{475:1},s0,G7,Ol),D(Ja,"StringBuffer",107),M(100,418,{475:1},rg,Kv,ju),D(Ja,"StringBuilder",100),M(687,73,FZ,vhe),D(Ja,"StringIndexOutOfBoundsException",687),M(2043,1,{});var L3e;M(844,1,{},Be),g.Kb=function(t){return h(t,78).e},D(Ja,"Throwable/lambda$0$Type",844),M(41,60,{3:1,102:1,60:1,78:1,41:1},Xn,i0),D(Ja,"UnsupportedOperationException",41),M(240,236,{3:1,35:1,236:1,240:1},AA,Che),g.wd=function(t){return Hpt(this,h(t,240))},g.ke=function(){return Dw(E2t(this))},g.Fb=function(t){var n;return this===t?!0:ue(t,240)?(n=h(t,240),this.e==n.e&&Hpt(this,n)==0):!1},g.Hb=function(){var t;return this.b!=0?this.b:this.a<54?(t=xc(this.f),this.b=Gn(Zi(t,-1)),this.b=33*this.b+Gn(Zi(dg(t,32),-1)),this.b=17*this.b+Li(this.e),this.b):(this.b=17*kht(this.c)+Li(this.e),this.b)},g.Ib=function(){return E2t(this)},g.a=0,g.b=0,g.d=0,g.e=0,g.f=0;var p3t,l2,I3e,O3e,N3e,P3e,B3e,F3e,rte=D("java.math","BigDecimal",240);M(91,236,{3:1,35:1,236:1,91:1},Qpe,d0,fy,O2e,_ft,hg),g.wd=function(t){return mft(this,h(t,91))},g.ke=function(){return Dw(lZ(this,0))},g.Fb=function(t){return Dpe(this,t)},g.Hb=function(){return kht(this)},g.Ib=function(){return lZ(this,0)},g.b=-2,g.c=0,g.d=0,g.e=0;var ite,Fj,R3e,ste,Rj,Gx,t3=D("java.math","BigInteger",91),b3t,v3t,M5,d_;M(488,1967,zb),g.$b=function(){Kc(this)},g._b=function(t){return Au(this,t)},g.uc=function(t){return sht(this,t,this.g)||sht(this,t,this.f)},g.vc=function(){return new xp(this)},g.xc=function(t){return _n(this,t)},g.zc=function(t,n){return Pr(this,t,n)},g.Bc=function(t){return Q4(this,t)},g.gc=function(){return q7(this)},D(Nn,"AbstractHashMap",488),M(261,Lf,$c,xp),g.$b=function(){this.a.$b()},g.Hc=function(t){return rat(this,t)},g.Kc=function(){return new Gp(this.a)},g.Mc=function(t){var n;return rat(this,t)?(n=h(t,42).cd(),this.a.Bc(n),!0):!1},g.gc=function(){return this.a.gc()},D(Nn,"AbstractHashMap/EntrySet",261),M(262,1,ys,Gp),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return kb(this)},g.Ob=function(){return this.b},g.Qb=function(){ect(this)},g.b=!1,D(Nn,"AbstractHashMap/EntrySetIterator",262),M(417,1,ys,L7),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return TS(this)},g.Pb=function(){return Tit(this)},g.Qb=function(){Mu(this)},g.b=0,g.c=-1,D(Nn,"AbstractList/IteratorImpl",417),M(96,417,z1,Ms),g.Qb=function(){Mu(this)},g.Rb=function(t){iw(this,t)},g.Sb=function(){return this.b>0},g.Tb=function(){return this.b},g.Ub=function(){return kn(this.b>0),this.a.Xb(this.c=--this.b)},g.Vb=function(){return this.b-1},g.Wb=function(t){tw(this.c!=-1),this.a._c(this.c,t)},D(Nn,"AbstractList/ListIteratorImpl",96),M(219,52,Ex,F1),g.Vc=function(t,n){fw(t,this.b),this.c.Vc(this.a+t,n),++this.b},g.Xb=function(t){return Kt(t,this.b),this.c.Xb(this.a+t)},g.$c=function(t){var n;return Kt(t,this.b),n=this.c.$c(this.a+t),--this.b,n},g._c=function(t,n){return Kt(t,this.b),this.c._c(this.a+t,n)},g.gc=function(){return this.b},g.a=0,g.b=0,D(Nn,"AbstractList/SubList",219),M(384,Lf,$c,P6),g.$b=function(){this.a.$b()},g.Hc=function(t){return this.a._b(t)},g.Kc=function(){var t;return t=this.a.vc().Kc(),new ble(t)},g.Mc=function(t){return this.a._b(t)?(this.a.Bc(t),!0):!1},g.gc=function(){return this.a.gc()},D(Nn,"AbstractMap/1",384),M(691,1,ys,ble),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.a.Ob()},g.Pb=function(){var t;return t=h(this.a.Pb(),42),t.cd()},g.Qb=function(){this.a.Qb()},D(Nn,"AbstractMap/1/1",691),M(226,28,Rw,vd),g.$b=function(){this.a.$b()},g.Hc=function(t){return this.a.uc(t)},g.Kc=function(){var t;return t=this.a.vc().Kc(),new qv(t)},g.gc=function(){return this.a.gc()},D(Nn,"AbstractMap/2",226),M(294,1,ys,qv),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.a.Ob()},g.Pb=function(){var t;return t=h(this.a.Pb(),42),t.dd()},g.Qb=function(){this.a.Qb()},D(Nn,"AbstractMap/2/1",294),M(484,1,{484:1,42:1}),g.Fb=function(t){var n;return ue(t,42)?(n=h(t,42),Bo(this.d,n.cd())&&Bo(this.e,n.dd())):!1},g.cd=function(){return this.d},g.dd=function(){return this.e},g.Hb=function(){return cy(this.d)^cy(this.e)},g.ed=function(t){return w1e(this,t)},g.Ib=function(){return this.d+"="+this.e},D(Nn,"AbstractMap/AbstractEntry",484),M(383,484,{484:1,383:1,42:1},LN),D(Nn,"AbstractMap/SimpleEntry",383),M(1984,1,HZ),g.Fb=function(t){var n;return ue(t,42)?(n=h(t,42),Bo(this.cd(),n.cd())&&Bo(this.dd(),n.dd())):!1},g.Hb=function(){return cy(this.cd())^cy(this.dd())},g.Ib=function(){return this.cd()+"="+this.dd()},D(Nn,Rbt,1984),M(1992,1967,Bbt),g.tc=function(t){return hot(this,t)},g._b=function(t){return YY(this,t)},g.vc=function(){return new kle(this)},g.xc=function(t){var n;return n=t,no(Qge(this,n))},g.ec=function(){return new uS(this)},D(Nn,"AbstractNavigableMap",1992),M(739,Lf,$c,kle),g.Hc=function(t){return ue(t,42)&&hot(this.b,h(t,42))},g.Kc=function(){return new EE(this.b)},g.Mc=function(t){var n;return ue(t,42)?(n=h(t,42),Zot(this.b,n)):!1},g.gc=function(){return this.b.c},D(Nn,"AbstractNavigableMap/EntrySet",739),M(493,Lf,Kve,uS),g.Nc=function(){return new MN(this)},g.$b=function(){j7(this.a)},g.Hc=function(t){return YY(this.a,t)},g.Kc=function(){var t;return t=new EE(new yE(this.a).b),new lS(t)},g.Mc=function(t){return YY(this.a,t)?(FE(this.a,t),!0):!1},g.gc=function(){return this.a.c},D(Nn,"AbstractNavigableMap/NavigableKeySet",493),M(494,1,ys,lS),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return TS(this.a.a)},g.Pb=function(){var t;return t=hP(this.a),t.cd()},g.Qb=function(){tnt(this.a)},D(Nn,"AbstractNavigableMap/NavigableKeySet/1",494),M(2004,28,Rw),g.Fc=function(t){return xk(sx(this,t)),!0},g.Gc=function(t){return rn(t),mP(t!=this,"Can't add a queue to itself"),Qs(this,t)},g.$b=function(){for(;MK(this)!=null;);},D(Nn,"AbstractQueue",2004),M(302,28,{4:1,20:1,28:1,14:1},Jm,Ast),g.Fc=function(t){return b0e(this,t),!0},g.$b=function(){x0e(this)},g.Hc=function(t){return Alt(new BE(this),t)},g.dc=function(){return $7(this)},g.Kc=function(){return new BE(this)},g.Mc=function(t){return Own(new BE(this),t)},g.gc=function(){return this.c-this.b&this.a.length-1},g.Nc=function(){return new Wt(this,272)},g.Qc=function(t){var n;return n=this.c-this.b&this.a.length-1,t.length<n&&(t=hN(new Array(n),t)),$ut(this,t,n),t.length>n&&pi(t,n,null),t},g.b=0,g.c=0,D(Nn,"ArrayDeque",302),M(446,1,ys,BE),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.a!=this.b},g.Pb=function(){return fF(this)},g.Qb=function(){Cut(this)},g.a=0,g.b=0,g.c=-1,D(Nn,"ArrayDeque/IteratorImpl",446),M(12,52,Jbt,qe,Wo,Bc),g.Vc=function(t,n){aw(this,t,n)},g.Fc=function(t){return Ue(this,t)},g.Wc=function(t,n){return rpe(this,t,n)},g.Gc=function(t){return Vi(this,t)},g.$b=function(){this.c=pe(yn,st,1,0,5,1)},g.Hc=function(t){return Ha(this,t,0)!=-1},g.Jc=function(t){mc(this,t)},g.Xb=function(t){return ut(this,t)},g.Xc=function(t){return Ha(this,t,0)},g.dc=function(){return this.c.length==0},g.Kc=function(){return new Q(this)},g.$c=function(t){return f0(this,t)},g.Mc=function(t){return vc(this,t)},g.Ud=function(t,n){Sst(this,t,n)},g._c=function(t,n){return dl(this,t,n)},g.gc=function(){return this.c.length},g.ad=function(t){hs(this,t)},g.Pc=function(){return BY(this)},g.Qc=function(t){return Mf(this,t)};var pPn=D(Nn,"ArrayList",12);M(7,1,ys,Q),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return Wa(this)},g.Pb=function(){return K(this)},g.Qb=function(){IE(this)},g.a=0,g.b=-1,D(Nn,"ArrayList/1",7),M(2013,d.Function,{},G),g.te=function(t,n){return Ui(t,n)},M(154,52,evt,_u),g.Hc=function(t){return Aut(this,t)!=-1},g.Jc=function(t){var n,s,a,u;for(rn(t),s=this.a,a=0,u=s.length;a<u;++a)n=s[a],t.td(n)},g.Xb=function(t){return Unt(this,t)},g._c=function(t,n){var s;return s=(Kt(t,this.a.length),this.a[t]),pi(this.a,t,n),s},g.gc=function(){return this.a.length},g.ad=function(t){oW(this.a,this.a.length,t)},g.Pc=function(){return Ift(this,pe(yn,st,1,this.a.length,5,1))},g.Qc=function(t){return Ift(this,t)},D(Nn,"Arrays/ArrayList",154);var ca,K1,jj;M(940,52,evt,$),g.Hc=function(t){return!1},g.Xb=function(t){return vfe(t)},g.Kc=function(){return Rt(),K6(),Vx},g.Yc=function(){return Rt(),K6(),Vx},g.gc=function(){return 0},D(Nn,"Collections/EmptyList",940),M(941,1,z1,ge),g.Nb=function(t){Ls(this,t)},g.Rb=function(t){throw ne(new Xn)},g.Ob=function(){return!1},g.Sb=function(){return!1},g.Pb=function(){throw ne(new lo)},g.Tb=function(){return 0},g.Ub=function(){throw ne(new lo)},g.Vb=function(){return-1},g.Qb=function(){throw ne(new Ic)},g.Wb=function(t){throw ne(new Ic)};var Vx;D(Nn,"Collections/EmptyListIterator",941),M(943,1967,kx,ee),g._b=function(t){return!1},g.uc=function(t){return!1},g.vc=function(){return Rt(),jj},g.xc=function(t){return null},g.ec=function(){return Rt(),jj},g.gc=function(){return 0},g.Cc=function(){return Rt(),ca},D(Nn,"Collections/EmptyMap",943),M(942,Lf,xx,ae),g.Hc=function(t){return!1},g.Kc=function(){return Rt(),K6(),Vx},g.gc=function(){return 0},D(Nn,"Collections/EmptySet",942),M(599,52,{3:1,20:1,28:1,52:1,14:1,15:1},mq),g.Hc=function(t){return Bo(this.a,t)},g.Xb=function(t){return Kt(t,1),this.a},g.gc=function(){return 1},D(Nn,"Collections/SingletonList",599),M(372,1,Hbt,B6),g.Jc=function(t){Os(this,t)},g.Lc=function(){return new Vt(null,this.Nc())},g.Nc=function(){return new Wt(this,0)},g.Oc=function(){return new Vt(null,this.Nc())},g.Fc=function(t){return iQe()},g.Gc=function(t){return sQe()},g.$b=function(){aQe()},g.Hc=function(t){return kS(this,t)},g.Ic=function(t){return YQe(this,t)},g.dc=function(){return this.b.dc()},g.Kc=function(){return new F6(this.b.Kc())},g.Mc=function(t){return oQe()},g.gc=function(){return this.b.gc()},g.Pc=function(){return this.b.Pc()},g.Qc=function(t){return WQe(this,t)},g.Ib=function(){return Ga(this.b)},D(Nn,"Collections/UnmodifiableCollection",372),M(371,1,ys,F6),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.b.Ob()},g.Pb=function(){return this.b.Pb()},g.Qb=function(){cQe()},D(Nn,"Collections/UnmodifiableCollectionIterator",371),M(531,372,tvt,sP),g.Nc=function(){return new Wt(this,16)},g.Vc=function(t,n){throw ne(new Xn)},g.Wc=function(t,n){throw ne(new Xn)},g.Fb=function(t){return Nr(this.a,t)},g.Xb=function(t){return this.a.Xb(t)},g.Hb=function(){return ii(this.a)},g.Xc=function(t){return this.a.Xc(t)},g.dc=function(){return this.a.dc()},g.Yc=function(){return new Gfe(this.a.Zc(0))},g.Zc=function(t){return new Gfe(this.a.Zc(t))},g.$c=function(t){throw ne(new Xn)},g._c=function(t,n){throw ne(new Xn)},g.ad=function(t){throw ne(new Xn)},g.bd=function(t,n){return new sP(this.a.bd(t,n))},D(Nn,"Collections/UnmodifiableList",531),M(690,371,z1,Gfe),g.Qb=function(){cQe()},g.Rb=function(t){throw ne(new Xn)},g.Sb=function(){return this.a.Sb()},g.Tb=function(){return this.a.Tb()},g.Ub=function(){return this.a.Ub()},g.Vb=function(){return this.a.Vb()},g.Wb=function(t){throw ne(new Xn)},D(Nn,"Collections/UnmodifiableListIterator",690),M(600,1,zb,yq),g.wc=function(t){ZE(this,t)},g.yc=function(t,n,s){return wX(this,t,n,s)},g.$b=function(){throw ne(new Xn)},g._b=function(t){return this.c._b(t)},g.uc=function(t){return XQe(this,t)},g.vc=function(){return dk(this)},g.Fb=function(t){return QQe(this,t)},g.xc=function(t){return this.c.xc(t)},g.Hb=function(){return ii(this.c)},g.dc=function(){return this.c.dc()},g.ec=function(){return ait(this)},g.zc=function(t,n){throw ne(new Xn)},g.Bc=function(t){throw ne(new Xn)},g.gc=function(){return this.c.gc()},g.Ib=function(){return Ga(this.c)},g.Cc=function(){return sit(this)},D(Nn,"Collections/UnmodifiableMap",600),M(382,372,gZ,G6),g.Nc=function(){return new Wt(this,1)},g.Fb=function(t){return Nr(this.b,t)},g.Hb=function(){return ii(this.b)},D(Nn,"Collections/UnmodifiableSet",382),M(944,382,gZ,FXe),g.Hc=function(t){return KQe(this,t)},g.Ic=function(t){return this.b.Ic(t)},g.Kc=function(){var t;return t=this.b.Kc(),new Eqe(t)},g.Pc=function(){var t;return t=this.b.Pc(),Iat(t,t.length),t},g.Qc=function(t){return Hit(this,t)},D(Nn,"Collections/UnmodifiableMap/UnmodifiableEntrySet",944),M(945,1,ys,Eqe),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return new xle(h(this.a.Pb(),42))},g.Ob=function(){return this.a.Ob()},g.Qb=function(){throw ne(new Xn)},D(Nn,"Collections/UnmodifiableMap/UnmodifiableEntrySet/1",945),M(688,1,HZ,xle),g.Fb=function(t){return this.a.Fb(t)},g.cd=function(){return this.a.cd()},g.dd=function(){return this.a.dd()},g.Hb=function(){return this.a.Hb()},g.ed=function(t){throw ne(new Xn)},g.Ib=function(){return Ga(this.a)},D(Nn,"Collections/UnmodifiableMap/UnmodifiableEntrySet/UnmodifiableEntry",688),M(601,531,{20:1,14:1,15:1,54:1},Hq),D(Nn,"Collections/UnmodifiableRandomAccessList",601),M(689,382,zbt,Vfe),g.Nc=function(){return new MN(this)},g.Fb=function(t){return Nr(this.a,t)},g.Hb=function(){return ii(this.a)},D(Nn,"Collections/UnmodifiableSortedSet",689),M(847,1,zZ,Ge),g.ue=function(t,n){var s;return s=Oat(h(t,11),h(n,11)),s!=0?s:zpt(h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Nn,"Comparator/lambda$0$Type",847);var j3e,$3e,H3e;M(751,1,zZ,Xe),g.ue=function(t,n){return Z2n(h(t,35),h(n,35))},g.Fb=function(t){return this===t},g.ve=function(){return gy(),H3e},D(Nn,"Comparators/NaturalOrderComparator",751),M(1177,1,zZ,Ee),g.ue=function(t,n){return J2n(h(t,35),h(n,35))},g.Fb=function(t){return this===t},g.ve=function(){return gy(),$3e},D(Nn,"Comparators/ReverseNaturalOrderComparator",1177),M(64,1,zZ,Hr),g.Fb=function(t){return this===t},g.ue=function(t,n){return this.a.ue(n,t)},g.ve=function(){return this.a},D(Nn,"Comparators/ReversedComparator",64),M(166,60,Fd,cl),D(Nn,"ConcurrentModificationException",166);var w3t,m3t;M(1904,1,$M,Ze),g.we=function(t){sft(this,t)},g.Ib=function(){return"DoubleSummaryStatistics[count = "+ME(this.a)+", avg = "+(vZe(this.a,0)?bot(this)/vb(this.a):0)+", min = "+this.c+", max = "+this.b+", sum = "+bot(this)+"]"},g.a=0,g.b=Fi,g.c=ki,g.d=0,g.e=0,g.f=0,D(Nn,"DoubleSummaryStatistics",1904),M(1805,60,Fd,OKe),D(Nn,"EmptyStackException",1805),M(451,1967,zb,o9),g.zc=function(t,n){return v1e(this,t,n)},g.$b=function(){Xrt(this)},g._b=function(t){return bZe(this,t)},g.uc=function(t){var n,s;for(s=new A4(this.a);s.a<s.c.a.length;)if(n=zE(s),Bo(t,this.b[n.g]))return!0;return!1},g.vc=function(){return new Tqe(this)},g.xc=function(t){return wa(this,t)},g.Bc=function(t){return r0e(this,t)},g.gc=function(){return this.a.c},D(Nn,"EnumMap",451),M(1352,Lf,$c,Tqe),g.$b=function(){Xrt(this.a)},g.Hc=function(t){return nat(this,t)},g.Kc=function(){return new Gnt(this.a)},g.Mc=function(t){var n;return nat(this,t)?(n=h(t,42).cd(),r0e(this.a,n),!0):!1},g.gc=function(){return this.a.a.c},D(Nn,"EnumMap/EntrySet",1352),M(1353,1,ys,Gnt),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return this.b=zE(this.a),new mZe(this.c,this.b)},g.Ob=function(){return Sfe(this.a)},g.Qb=function(){tw(!!this.b),r0e(this.c,this.b),this.b=null},D(Nn,"EnumMap/EntrySetIterator",1353),M(1354,1984,HZ,mZe),g.cd=function(){return this.a},g.dd=function(){return this.b.b[this.a.g]},g.ed=function(t){return Sde(this.b,this.a.g,t)},D(Nn,"EnumMap/MapEntry",1354),M(174,Lf,{20:1,28:1,14:1,174:1,21:1});var y3t=D(Nn,"EnumSet",174);M(156,174,{20:1,28:1,14:1,174:1,156:1,21:1},ll),g.Fc=function(t){return Eh(this,h(t,22))},g.Hc=function(t){return cW(this,t)},g.Kc=function(){return new A4(this)},g.Mc=function(t){return rrt(this,t)},g.gc=function(){return this.c},g.c=0,D(Nn,"EnumSet/EnumSetImpl",156),M(343,1,ys,A4),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return zE(this)},g.Ob=function(){return Sfe(this)},g.Qb=function(){tw(this.b!=-1),pi(this.c.b,this.b,null),--this.c.c,this.b=-1},g.a=-1,g.b=-1,D(Nn,"EnumSet/EnumSetImpl/IteratorImpl",343),M(43,488,Vy,zn,S4,NJe),g.re=function(t,n){return Me(t)===Me(n)||t!=null&&Nr(t,n)},g.se=function(t){var n;return n=ii(t),n|0},D(Nn,"HashMap",43),M(53,Lf,nwe,ns,gN,CE),g.Fc=function(t){return Qi(this,t)},g.$b=function(){this.a.$b()},g.Hc=function(t){return wd(this,t)},g.dc=function(){return this.a.gc()==0},g.Kc=function(){return this.a.ec().Kc()},g.Mc=function(t){return Lfe(this,t)},g.gc=function(){return this.a.gc()};var bPn=D(Nn,"HashSet",53);M(1781,1,NM,Ie),g.ud=function(t){blt(this,t)},g.Ib=function(){return"IntSummaryStatistics[count = "+ME(this.a)+", avg = "+(vZe(this.a,0)?vb(this.d)/vb(this.a):0)+", min = "+this.c+", max = "+this.b+", sum = "+ME(this.d)+"]"},g.a=0,g.b=$s,g.c=Ir,g.d=0,D(Nn,"IntSummaryStatistics",1781),M(1049,1,G1,YJe),g.Jc=function(t){Os(this,t)},g.Kc=function(){return new d0e(this)},g.c=0,D(Nn,"InternalHashCodeMap",1049),M(711,1,ys,d0e),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return this.d=this.a[this.c++],this.d},g.Ob=function(){var t;return this.c<this.a.length?!0:(t=this.b.next(),t.done?!1:(this.a=t.value[1],this.c=0,!0))},g.Qb=function(){Abe(this.e,this.d.cd()),this.c!=0&&--this.c},g.c=0,g.d=null,D(Nn,"InternalHashCodeMap/1",711);var k3t;M(1047,1,G1,WJe),g.Jc=function(t){Os(this,t)},g.Kc=function(){return new E0e(this)},g.c=0,g.d=0,D(Nn,"InternalStringMap",1047),M(710,1,ys,E0e),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return this.c=this.a,this.a=this.b.next(),new Jtt(this.d,this.c,this.d.d)},g.Ob=function(){return!this.a.done},g.Qb=function(){Tlt(this.d,this.c.value[0])},D(Nn,"InternalStringMap/1",710),M(1048,1984,HZ,Jtt),g.cd=function(){return this.b.value[0]},g.dd=function(){return this.a.d!=this.c?Z7(this.a,this.b.value[0]):this.b.value[1]},g.ed=function(t){return Ab(this.a,this.b.value[0],t)},g.c=0,D(Nn,"InternalStringMap/2",1048),M(228,43,Vy,Np,z0e),g.$b=function(){Stt(this)},g._b=function(t){return wZe(this,t)},g.uc=function(t){var n;for(n=this.d.a;n!=this.d;){if(Bo(n.e,t))return!0;n=n.a}return!1},g.vc=function(){return new Ale(this)},g.xc=function(t){return mb(this,t)},g.zc=function(t,n){return Yp(this,t,n)},g.Bc=function(t){return Tct(this,t)},g.gc=function(){return q7(this.e)},g.c=!1,D(Nn,"LinkedHashMap",228),M(387,383,{484:1,383:1,387:1,42:1},Zet,U1e),D(Nn,"LinkedHashMap/ChainEntry",387),M(701,Lf,$c,Ale),g.$b=function(){Stt(this.a)},g.Hc=function(t){return iat(this,t)},g.Kc=function(){return new jde(this)},g.Mc=function(t){var n;return iat(this,t)?(n=h(t,42).cd(),Tct(this.a,n),!0):!1},g.gc=function(){return q7(this.a.e)},D(Nn,"LinkedHashMap/EntrySet",701),M(702,1,ys,jde),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return tct(this)},g.Ob=function(){return this.b!=this.c.a.d},g.Qb=function(){tw(!!this.a),nK(this.c.a.e,this),tde(this.a),Q4(this.c.a.e,this.a.d),a1e(this.c.a.e,this),this.a=null},D(Nn,"LinkedHashMap/EntrySet/EntryIterator",702),M(178,53,nwe,md,NY,sde);var vPn=D(Nn,"LinkedHashSet",178);M(68,1964,{3:1,4:1,20:1,28:1,52:1,14:1,68:1,15:1},di,WY),g.Fc=function(t){return vr(this,t)},g.$b=function(){Pl(this)},g.Zc=function(t){return pr(this,t)},g.gc=function(){return this.b},g.b=0;var wPn=D(Nn,"LinkedList",68);M(970,1,z1,ent),g.Nb=function(t){Ls(this,t)},g.Rb=function(t){nA(this,t)},g.Ob=function(){return pN(this)},g.Sb=function(){return this.b.b!=this.d.a},g.Pb=function(){return gr(this)},g.Tb=function(){return this.a},g.Ub=function(){return eK(this)},g.Vb=function(){return this.a-1},g.Qb=function(){BB(this)},g.Wb=function(t){tw(!!this.c),this.c.c=t},g.a=0,g.c=null,D(Nn,"LinkedList/ListIteratorImpl",970),M(608,1,{},Ce),D(Nn,"LinkedList/Node",608),M(1959,1,{});var z3e,x3t;D(Nn,"Locale",1959),M(861,1959,{},_e),g.Ib=function(){return""},D(Nn,"Locale/1",861),M(862,1959,{},Te),g.Ib=function(){return"unknown"},D(Nn,"Locale/4",862),M(109,60,{3:1,102:1,60:1,78:1,109:1},lo,cit),D(Nn,"NoSuchElementException",109),M(404,1,{404:1},oN),g.Fb=function(t){var n;return t===this?!0:ue(t,404)?(n=h(t,404),Bo(this.a,n.a)):!1},g.Hb=function(){return cy(this.a)},g.Ib=function(){return this.a!=null?Pbt+mE(this.a)+")":"Optional.empty()"};var $j;D(Nn,"Optional",404),M(463,1,{463:1},jJe,Jfe),g.Fb=function(t){var n;return t===this?!0:ue(t,463)?(n=h(t,463),this.a==n.a&&Ui(this.b,n.b)==0):!1},g.Hb=function(){return this.a?Li(this.b):0},g.Ib=function(){return this.a?"OptionalDouble.of("+(""+this.b)+")":"OptionalDouble.empty()"},g.a=!1,g.b=0;var G3e;D(Nn,"OptionalDouble",463),M(517,1,{517:1},$Je,Ket),g.Fb=function(t){var n;return t===this?!0:ue(t,517)?(n=h(t,517),this.a==n.a&&dc(this.b,n.b)==0):!1},g.Hb=function(){return this.a?this.b:0},g.Ib=function(){return this.a?"OptionalInt.of("+(""+this.b)+")":"OptionalInt.empty()"},g.a=!1,g.b=0;var E3t;D(Nn,"OptionalInt",517),M(503,2004,Rw,RP),g.Gc=function(t){return d2e(this,t)},g.$b=function(){this.b.c=pe(yn,st,1,0,5,1)},g.Hc=function(t){return(t==null?-1:Ha(this.b,t,0))!=-1},g.Kc=function(){return new _qe(this)},g.Mc=function(t){return Hut(this,t)},g.gc=function(){return this.b.c.length},g.Nc=function(){return new Wt(this,256)},g.Pc=function(){return BY(this.b)},g.Qc=function(t){return Mf(this.b,t)},D(Nn,"PriorityQueue",503),M(1277,1,ys,_qe),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.a<this.c.b.c.length},g.Pb=function(){return kn(this.a<this.c.b.c.length),this.b=this.a++,ut(this.c.b,this.b)},g.Qb=function(){tw(this.b!=-1),jK(this.c,this.a=this.b),this.b=-1},g.a=0,g.b=-1,D(Nn,"PriorityQueue/1",1277),M(230,1,{230:1},mX,vB),g.a=0,g.b=0;var V3e,U3e,mPn=0;D(Nn,"Random",230),M(27,1,jl,Wt,cb,lit),g.qd=function(){return this.a},g.rd=function(){return kde(this),this.c},g.Nb=function(t){kde(this),this.d.Nb(t)},g.sd=function(t){return Kct(this,t)},g.a=0,g.c=0,D(Nn,"Spliterators/IteratorSpliterator",27),M(485,27,jl,MN),D(Nn,"SortedSet/1",485),M(602,1,$M,Ele),g.we=function(t){this.a.td(t)},D(Nn,"Spliterator/OfDouble/0methodref$accept$Type",602),M(603,1,$M,_le),g.we=function(t){this.a.td(t)},D(Nn,"Spliterator/OfDouble/1methodref$accept$Type",603),M(604,1,NM,Tle),g.ud=function(t){this.a.td(Ke(t))},D(Nn,"Spliterator/OfInt/2methodref$accept$Type",604),M(605,1,NM,Cle),g.ud=function(t){this.a.td(Ke(t))},D(Nn,"Spliterator/OfInt/3methodref$accept$Type",605),M(617,1,jl),g.Nb=function(t){She(this,t)},g.qd=function(){return this.d},g.rd=function(){return this.e},g.d=0,g.e=0,D(Nn,"Spliterators/BaseSpliterator",617),M(721,617,jl),g.xe=function(t){ty(this,t)},g.Nb=function(t){ue(t,182)?ty(this,h(t,182)):ty(this,new _le(t))},g.sd=function(t){return ue(t,182)?this.ye(h(t,182)):this.ye(new Ele(t))},D(Nn,"Spliterators/AbstractDoubleSpliterator",721),M(720,617,jl),g.xe=function(t){ty(this,t)},g.Nb=function(t){ue(t,196)?ty(this,h(t,196)):ty(this,new Cle(t))},g.sd=function(t){return ue(t,196)?this.ye(h(t,196)):this.ye(new Tle(t))},D(Nn,"Spliterators/AbstractIntSpliterator",720),M(540,617,jl),D(Nn,"Spliterators/AbstractSpliterator",540),M(692,1,jl),g.Nb=function(t){She(this,t)},g.qd=function(){return this.b},g.rd=function(){return this.d-this.c},g.b=0,g.c=0,g.d=0,D(Nn,"Spliterators/BaseArraySpliterator",692),M(947,692,jl,Lnt),g.ze=function(t,n){l1n(this,h(t,38),n)},g.Nb=function(t){SW(this,t)},g.sd=function(t){return EB(this,t)},D(Nn,"Spliterators/ArraySpliterator",947),M(693,692,jl,Xtt),g.ze=function(t,n){h1n(this,h(t,182),n)},g.xe=function(t){SW(this,t)},g.Nb=function(t){ue(t,182)?SW(this,h(t,182)):SW(this,new _le(t))},g.ye=function(t){return EB(this,t)},g.sd=function(t){return ue(t,182)?EB(this,h(t,182)):EB(this,new Ele(t))},D(Nn,"Spliterators/DoubleArraySpliterator",693),M(1968,1,jl),g.Nb=function(t){She(this,t)},g.qd=function(){return 16448},g.rd=function(){return 0};var _3t;D(Nn,"Spliterators/EmptySpliterator",1968),M(946,1968,jl,Ae),g.xe=function(t){sle(t)},g.Nb=function(t){ue(t,196)?sle(h(t,196)):sle(new Cle(t))},g.ye=function(t){return Rhe(t)},g.sd=function(t){return ue(t,196)?Rhe(h(t,196)):Rhe(new Tle(t))},D(Nn,"Spliterators/EmptySpliterator/OfInt",946),M(580,52,nvt,sN),g.Vc=function(t,n){ck(t,this.a.c.length+1),aw(this.a,t,n)},g.Fc=function(t){return Ue(this.a,t)},g.Wc=function(t,n){return ck(t,this.a.c.length+1),rpe(this.a,t,n)},g.Gc=function(t){return Vi(this.a,t)},g.$b=function(){this.a.c=pe(yn,st,1,0,5,1)},g.Hc=function(t){return Ha(this.a,t,0)!=-1},g.Ic=function(t){return FA(this.a,t)},g.Jc=function(t){mc(this.a,t)},g.Xb=function(t){return ck(t,this.a.c.length),ut(this.a,t)},g.Xc=function(t){return Ha(this.a,t,0)},g.dc=function(){return this.a.c.length==0},g.Kc=function(){return new Q(this.a)},g.$c=function(t){return ck(t,this.a.c.length),f0(this.a,t)},g.Ud=function(t,n){Sst(this.a,t,n)},g._c=function(t,n){return ck(t,this.a.c.length),dl(this.a,t,n)},g.gc=function(){return this.a.c.length},g.ad=function(t){hs(this.a,t)},g.bd=function(t,n){return new F1(this.a,t,n)},g.Pc=function(){return BY(this.a)},g.Qc=function(t){return Mf(this.a,t)},g.Ib=function(){return Ag(this.a)},D(Nn,"Vector",580),M(809,580,nvt,ehe),D(Nn,"Stack",809),M(206,1,{206:1},$p),g.Ib=function(){return Xst(this)},D(Nn,"StringJoiner",206),M(544,1992,{3:1,83:1,171:1,161:1},eZe,pW),g.$b=function(){j7(this)},g.vc=function(){return new yE(this)},g.zc=function(t,n){return cF(this,t,n)},g.Bc=function(t){return FE(this,t)},g.gc=function(){return this.c},g.c=0,D(Nn,"TreeMap",544),M(390,1,ys,EE),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return hP(this)},g.Ob=function(){return TS(this.a)},g.Qb=function(){tnt(this)},D(Nn,"TreeMap/EntryIterator",390),M(435,739,$c,yE),g.$b=function(){j7(this.a)},D(Nn,"TreeMap/EntrySet",435),M(436,383,{484:1,383:1,42:1,436:1},wK),g.b=!1;var yPn=D(Nn,"TreeMap/Node",436);M(621,1,{},$e),g.Ib=function(){return"State: mv="+this.c+" value="+this.d+" done="+this.a+" found="+this.b},g.a=!1,g.b=!1,g.c=!1,D(Nn,"TreeMap/State",621),M(297,22,UZ,DN),g.Ae=function(){return!1},g.Be=function(){return!1};var ate,q3e,Y3e,W3e,Hj=tr(Nn,"TreeMap/SubMapType",297,sr,Kmn,Cgn);M(1112,297,UZ,tet),g.Be=function(){return!0},tr(Nn,"TreeMap/SubMapType/1",1112,Hj,null,null),M(1113,297,UZ,get),g.Ae=function(){return!0},g.Be=function(){return!0},tr(Nn,"TreeMap/SubMapType/2",1113,Hj,null,null),M(1114,297,UZ,net),g.Ae=function(){return!0},tr(Nn,"TreeMap/SubMapType/3",1114,Hj,null,null);var T3t;M(208,Lf,{3:1,20:1,28:1,14:1,271:1,21:1,84:1,208:1},Kle,ag),g.Nc=function(){return new MN(this)},g.Fc=function(t){return tA(this,t)},g.$b=function(){j7(this.a)},g.Hc=function(t){return YY(this.a,t)},g.Kc=function(){var t;return t=new EE(new yE(new uS(this.a).a).b),new lS(t)},g.Mc=function(t){return OY(this,t)},g.gc=function(){return this.a.c};var kPn=D(Nn,"TreeSet",208);M(966,1,{},Cqe),g.Ce=function(t,n){return V0n(this.a,t,n)},D(qZ,"BinaryOperator/lambda$0$Type",966),M(967,1,{},Sqe),g.Ce=function(t,n){return U0n(this.a,t,n)},D(qZ,"BinaryOperator/lambda$1$Type",967),M(846,1,{},xe),g.Kb=function(t){return t},D(qZ,"Function/lambda$0$Type",846),M(431,1,Cr,I7),g.Mb=function(t){return!this.a.Mb(t)},D(qZ,"Predicate/lambda$2$Type",431),M(572,1,{572:1});var C3t=D(F9,"Handler",572);M(2007,1,IM),g.ne=function(){return"DUMMY"},g.Ib=function(){return this.ne()};var K3e;D(F9,"Level",2007),M(1621,2007,IM,je),g.ne=function(){return"INFO"},D(F9,"Level/LevelInfo",1621),M(1640,1,{},RKe);var ote;D(F9,"LogManager",1640),M(1780,1,IM,nnt),g.b=null,D(F9,"LogRecord",1780),M(512,1,{512:1},uK),g.e=!1;var S3t=!1,A3t=!1,Pf=!1,M3t=!1,D3t=!1;D(F9,"Logger",512),M(819,572,{572:1},me),D(F9,"SimpleConsoleLogHandler",819),M(132,22,{3:1,35:1,22:1,132:1},Wq);var X3e,Zu,Kw,gu=tr(Yi,"Collector/Characteristics",132,sr,qwn,Sgn),L3t;M(744,1,{},Cde),D(Yi,"CollectorImpl",744),M(1060,1,{},vt),g.Ce=function(t,n){return Q6n(h(t,206),h(n,206))},D(Yi,"Collectors/10methodref$merge$Type",1060),M(1061,1,{},ve),g.Kb=function(t){return Xst(h(t,206))},D(Yi,"Collectors/11methodref$toString$Type",1061),M(1062,1,{},Aqe),g.Kb=function(t){return on(),!!wfe(t)},D(Yi,"Collectors/12methodref$test$Type",1062),M(251,1,{},Oe),g.Od=function(t,n){h(t,14).Fc(n)},D(Yi,"Collectors/20methodref$add$Type",251),M(253,1,{},Le),g.Ee=function(){return new qe},D(Yi,"Collectors/21methodref$ctor$Type",253),M(346,1,{},Zt),g.Ee=function(){return new ns},D(Yi,"Collectors/23methodref$ctor$Type",346),M(347,1,{},nt),g.Od=function(t,n){Qi(h(t,53),n)},D(Yi,"Collectors/24methodref$add$Type",347),M(1055,1,{},xn),g.Ce=function(t,n){return tZe(h(t,15),h(n,14))},D(Yi,"Collectors/4methodref$addAll$Type",1055),M(1059,1,{},cn),g.Od=function(t,n){Cd(h(t,206),h(n,475))},D(Yi,"Collectors/9methodref$add$Type",1059),M(1058,1,{},mnt),g.Ee=function(){return new $p(this.a,this.b,this.c)},D(Yi,"Collectors/lambda$15$Type",1058),M(1063,1,{},jt),g.Ee=function(){var t;return t=new Np,Yp(t,(on(),!1),new qe),Yp(t,!0,new qe),t},D(Yi,"Collectors/lambda$22$Type",1063),M(1064,1,{},Mqe),g.Ee=function(){return se(ie(yn,1),st,1,5,[this.a])},D(Yi,"Collectors/lambda$25$Type",1064),M(1065,1,{},Dqe),g.Od=function(t,n){ebn(this.a,Fp(t))},D(Yi,"Collectors/lambda$26$Type",1065),M(1066,1,{},Lqe),g.Ce=function(t,n){return Cbn(this.a,Fp(t),Fp(n))},D(Yi,"Collectors/lambda$27$Type",1066),M(1067,1,{},ot),g.Kb=function(t){return Fp(t)[0]},D(Yi,"Collectors/lambda$28$Type",1067),M(713,1,{},be),g.Ce=function(t,n){return ade(t,n)},D(Yi,"Collectors/lambda$4$Type",713),M(252,1,{},We),g.Ce=function(t,n){return L1n(h(t,14),h(n,14))},D(Yi,"Collectors/lambda$42$Type",252),M(348,1,{},ct),g.Ce=function(t,n){return I1n(h(t,53),h(n,53))},D(Yi,"Collectors/lambda$50$Type",348),M(349,1,{},Yt),g.Kb=function(t){return h(t,53)},D(Yi,"Collectors/lambda$51$Type",349),M(1054,1,{},Iqe),g.Od=function(t,n){g6n(this.a,h(t,83),n)},D(Yi,"Collectors/lambda$7$Type",1054),M(1056,1,{},Ut),g.Ce=function(t,n){return z5n(h(t,83),h(n,83),new xn)},D(Yi,"Collectors/lambda$8$Type",1056),M(1057,1,{},Oqe),g.Kb=function(t){return Lkn(this.a,h(t,83))},D(Yi,"Collectors/lambda$9$Type",1057),M(539,1,{}),g.He=function(){PE(this)},g.d=!1,D(Yi,"TerminatableStream",539),M(812,539,swe,u1e),g.He=function(){PE(this)},D(Yi,"DoubleStreamImpl",812),M(1784,721,jl,ynt),g.ye=function(t){return j8n(this,h(t,182))},g.a=null,D(Yi,"DoubleStreamImpl/2",1784),M(1785,1,$M,Nqe),g.we=function(t){_dn(this.a,t)},D(Yi,"DoubleStreamImpl/2/lambda$0$Type",1785),M(1782,1,$M,Pqe),g.we=function(t){Edn(this.a,t)},D(Yi,"DoubleStreamImpl/lambda$0$Type",1782),M(1783,1,$M,Bqe),g.we=function(t){sft(this.a,t)},D(Yi,"DoubleStreamImpl/lambda$2$Type",1783),M(1358,720,jl,oot),g.ye=function(t){return zmn(this,h(t,196))},g.a=0,g.b=0,g.c=0,D(Yi,"IntStream/5",1358),M(787,539,swe,l1e),g.He=function(){PE(this)},g.Ie=function(){return vg(this),this.a},D(Yi,"IntStreamImpl",787),M(788,539,swe,Lhe),g.He=function(){PE(this)},g.Ie=function(){return vg(this),Ife(),_3t},D(Yi,"IntStreamImpl/Empty",788),M(1463,1,NM,Fqe),g.ud=function(t){blt(this.a,t)},D(Yi,"IntStreamImpl/lambda$4$Type",1463);var xPn=fi(Yi,"Stream");M(30,539,{525:1,670:1,833:1},Vt),g.He=function(){PE(this)};var Ux;D(Yi,"StreamImpl",30),M(845,1,{},Wn),g.ld=function(t){return xnt(t)},D(Yi,"StreamImpl/0methodref$lambda$2$Type",845),M(1084,540,jl,Ktt),g.sd=function(t){for(;I3n(this);){if(this.a.sd(t))return!0;PE(this.b),this.b=null,this.a=null}return!1},D(Yi,"StreamImpl/1",1084),M(1085,1,mn,Rqe),g.td=function(t){A2n(this.a,h(t,833))},D(Yi,"StreamImpl/1/lambda$0$Type",1085),M(1086,1,Cr,jqe),g.Mb=function(t){return Qi(this.a,t)},D(Yi,"StreamImpl/1methodref$add$Type",1086),M(1087,540,jl,_it),g.sd=function(t){var n;return this.a||(n=new qe,this.b.a.Nb(new $qe(n)),Rt(),hs(n,this.c),this.a=new Wt(n,16)),Kct(this.a,t)},g.a=null,D(Yi,"StreamImpl/5",1087),M(1088,1,mn,$qe),g.td=function(t){Ue(this.a,t)},D(Yi,"StreamImpl/5/2methodref$add$Type",1088),M(722,540,jl,W0e),g.sd=function(t){for(this.b=!1;!this.b&&this.c.sd(new kZe(this,t)););return this.b},g.b=!1,D(Yi,"StreamImpl/FilterSpliterator",722),M(1079,1,mn,kZe),g.td=function(t){Tbn(this.a,this.b,t)},D(Yi,"StreamImpl/FilterSpliterator/lambda$0$Type",1079),M(1075,721,jl,mot),g.ye=function(t){return ggn(this,h(t,182))},D(Yi,"StreamImpl/MapToDoubleSpliterator",1075),M(1078,1,mn,xZe),g.td=function(t){q1n(this.a,this.b,t)},D(Yi,"StreamImpl/MapToDoubleSpliterator/lambda$0$Type",1078),M(1074,720,jl,yot),g.ye=function(t){return pgn(this,h(t,196))},D(Yi,"StreamImpl/MapToIntSpliterator",1074),M(1077,1,mn,EZe),g.td=function(t){U1n(this.a,this.b,t)},D(Yi,"StreamImpl/MapToIntSpliterator/lambda$0$Type",1077),M(719,540,jl,B0e),g.sd=function(t){return qtt(this,t)},D(Yi,"StreamImpl/MapToObjSpliterator",719),M(1076,1,mn,_Ze),g.td=function(t){Y1n(this.a,this.b,t)},D(Yi,"StreamImpl/MapToObjSpliterator/lambda$0$Type",1076),M(618,1,mn,Gt),g.td=function(t){wqe(this,t)},D(Yi,"StreamImpl/ValueConsumer",618),M(1080,1,mn,Rn),g.td=function(t){Tp()},D(Yi,"StreamImpl/lambda$0$Type",1080),M(1081,1,mn,si),g.td=function(t){Tp()},D(Yi,"StreamImpl/lambda$1$Type",1081),M(1082,1,{},Hqe),g.Ce=function(t,n){return Ugn(this.a,t,n)},D(Yi,"StreamImpl/lambda$4$Type",1082),M(1083,1,mn,yZe),g.td=function(t){tgn(this.b,this.a,t)},D(Yi,"StreamImpl/lambda$5$Type",1083),M(1089,1,mn,zqe),g.td=function(t){o6n(this.a,h(t,365))},D(Yi,"TerminatableStream/lambda$0$Type",1089),M(2041,1,{}),M(1914,1,{},$r),D("javaemul.internal","ConsoleLogger",1914),M(2038,1,{});var EPn=0,Q3e,Z3e=0,zj;M(1768,1,mn,nr),g.td=function(t){h(t,308)},D(Tx,"BowyerWatsonTriangulation/lambda$0$Type",1768),M(1769,1,mn,Gqe),g.td=function(t){Qs(this.a,h(t,308).e)},D(Tx,"BowyerWatsonTriangulation/lambda$1$Type",1769),M(1770,1,mn,Kn),g.td=function(t){h(t,168)},D(Tx,"BowyerWatsonTriangulation/lambda$2$Type",1770),M(1765,1,Yr,Vqe),g.ue=function(t,n){return kyn(this.a,h(t,168),h(n,168))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Tx,"NaiveMinST/lambda$0$Type",1765),M(499,1,{},hS),D(Tx,"NodeMicroLayout",499),M(168,1,{168:1},M4),g.Fb=function(t){var n;return ue(t,168)?(n=h(t,168),Bo(this.a,n.a)&&Bo(this.b,n.b)||Bo(this.a,n.b)&&Bo(this.b,n.a)):!1},g.Hb=function(){return cy(this.a)+cy(this.b)};var _Pn=D(Tx,"TEdge",168);M(308,1,{308:1},Eve),g.Fb=function(t){var n;return ue(t,308)?(n=h(t,308),DB(this,n.a)&&DB(this,n.b)&&DB(this,n.c)):!1},g.Hb=function(){return cy(this.a)+cy(this.b)+cy(this.c)},D(Tx,"TTriangle",308),M(221,1,{221:1},tP),D(Tx,"Tree",221),M(1254,1,{},hst),D(svt,"Scanline",1254);var I3t=fi(svt,avt);M(1692,1,{},Jct),D(q1,"CGraph",1692),M(307,1,{307:1},ast),g.b=0,g.c=0,g.d=0,g.g=0,g.i=0,g.k=Fi,D(q1,"CGroup",307),M(815,1,{},Xle),D(q1,"CGroup/CGroupBuilder",815),M(57,1,{57:1},Dtt),g.Ib=function(){var t;return this.j?Jn(this.j.Kb(this)):(yd(Gj),Gj.o+"@"+(t=rb(this)>>>0,t.toString(16)))},g.f=0,g.i=Fi;var Gj=D(q1,"CNode",57);M(814,1,{},Qle),D(q1,"CNode/CNodeBuilder",814);var O3t;M(1525,1,{},Jt),g.Oe=function(t,n){return 0},g.Pe=function(t,n){return 0},D(q1,cvt,1525),M(1790,1,{},en),g.Le=function(t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J;for(_=ki,a=new Q(t.a.b);a.a<a.c.c.length;)n=h(K(a),57),_=d.Math.min(_,n.a.j.d.c+n.b.a);for(j=new di,v=new Q(t.a.a);v.a<v.c.c.length;)p=h(K(v),307),p.k=_,p.g==0&&Ai(j,p,j.c.b,j.c);for(;j.b!=0;){for(p=h(j.b==0?null:(kn(j.b!=0),pl(j,j.a.a)),307),u=p.j.d.c,P=p.a.a.ec().Kc();P.Ob();)A=h(P.Pb(),57),J=p.k+A.b.a,!F6n(t,p,t.d)||A.d.c<J?A.i=J:A.i=A.d.c;for(u-=p.j.i,p.b+=u,t.d==(la(),Dh)||t.d==zd?p.c+=u:p.c-=u,I=p.a.a.ec().Kc();I.Ob();)for(A=h(I.Pb(),57),x=A.c.Kc();x.Ob();)m=h(x.Pb(),57),lg(t.d)?V=t.g.Oe(A,m):V=t.g.Pe(A,m),m.a.k=d.Math.max(m.a.k,A.i+A.d.b+V-m.b.a),Iit(t,m,t.d)&&(m.a.k=d.Math.max(m.a.k,m.d.c-m.b.a)),--m.a.g,m.a.g==0&&vr(j,m.a)}for(s=new Q(t.a.b);s.a<s.c.c.length;)n=h(K(s),57),n.d.c=n.i},D(q1,"LongestPathCompaction",1790),M(1690,1,{},C0t),g.e=!1;var N3t,P3t,B3t,cte=D(q1,hvt,1690);M(1691,1,mn,Uqe),g.td=function(t){j5n(this.a,h(t,46))},D(q1,fvt,1691),M(1791,1,{},In),g.Me=function(t){var n,s,a,u,p,v,m;for(s=new Q(t.a.b);s.a<s.c.c.length;)n=h(K(s),57),n.c.$b();for(u=new Q(t.a.b);u.a<u.c.c.length;)for(a=h(K(u),57),v=new Q(t.a.b);v.a<v.c.c.length;)p=h(K(v),57),a!=p&&(!!a.a&&a.a==p.a||(lg(t.d)?m=t.g.Pe(a,p):m=t.g.Oe(a,p),(p.d.c>a.d.c||a.d.c==p.d.c&&a.d.b<p.d.b)&&_xn(p.d.d+p.d.a+m,a.d.d)&&Vpe(p.d.d,a.d.d+a.d.a+m)&&a.c.Fc(p)))},D(q1,"QuadraticConstraintCalculation",1791),M(522,1,{522:1},xq),g.a=!1,g.b=!1,g.c=!1,g.d=!1,D(q1,dvt,522),M(803,1,{},fde),g.Me=function(t){this.c=t,fM(this,new Pt)},D(q1,gvt,803),M(1718,1,{679:1},Nit),g.Ke=function(t){N_n(this,h(t,464))},D(q1,pvt,1718),M(1719,1,Yr,hn),g.ue=function(t,n){return Pvn(h(t,57),h(n,57))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(q1,bvt,1719),M(464,1,{464:1},Hhe),g.a=!1,D(q1,vvt,464),M(1720,1,Yr,Fr),g.ue=function(t,n){return q7n(h(t,464),h(n,464))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(q1,wvt,1720),M(1721,1,c1,Pt),g.Lb=function(t){return h(t,57),!0},g.Fb=function(t){return this===t},g.Mb=function(t){return h(t,57),!0},D(q1,"ScanlineConstraintCalculator/lambda$1$Type",1721),M(428,22,{3:1,35:1,22:1,428:1},Vhe);var J3e,ute,e4e=tr(XZ,"HighLevelSortingCriterion",428,sr,kwn,Agn),F3t;M(427,22,{3:1,35:1,22:1,427:1},Uhe);var t4e,lte,n4e=tr(XZ,"LowLevelSortingCriterion",427,sr,xwn,Mgn),R3t,n3=fi(eo,"ILayoutMetaDataProvider");M(853,1,$l,ZVe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,uwe),QZ),"Polyomino Traversal Strategy"),"Traversal strategy for trying different candidate positions for polyominoes."),u4e),(E0(),_i)),w4e),Lt((Qh(),gn))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,lwe),QZ),"Polyomino Secondary Sorting Criterion"),"Possible secondary sorting criteria for the processing order of polyominoes. They are used when polyominoes are equal according to the primary sorting criterion HighLevelSortingCriterion."),o4e),_i),n4e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,hwe),QZ),"Polyomino Primary Sorting Criterion"),"Possible primary sorting criteria for the processing order of polyominoes."),s4e),_i),e4e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,fwe),QZ),"Fill Polyominoes"),"Use the Profile Fill algorithm to fill polyominoes to prevent small polyominoes from being placed inside of big polyominoes with large holes. Might increase packing area."),(on(),!0)),zs),es),Lt(gn))))};var r4e,i4e,s4e,a4e,o4e,c4e,u4e;D(XZ,"PolyominoOptions",853),M(250,22,{3:1,35:1,22:1,250:1},ry);var l4e,h4e,f4e,d4e,g4e,p4e,hte,b4e,v4e,w4e=tr(XZ,"TraversalStrategy",250,sr,P4n,Dgn),j3t;M(213,1,{213:1},ei),g.Ib=function(){return"NEdge[id="+this.b+" w="+this.g+" d="+this.a+"]"},g.a=1,g.b=0,g.c=0,g.f=!1,g.g=0;var $3t=D(R9,"NEdge",213);M(176,1,{},Il),D(R9,"NEdge/NEdgeBuilder",176),M(653,1,{},_q),D(R9,"NGraph",653),M(121,1,{121:1},vot),g.c=-1,g.d=0,g.e=0,g.i=-1,g.j=!1;var m4e=D(R9,"NNode",121);M(795,1,tvt,Yle),g.Jc=function(t){Os(this,t)},g.Lc=function(){return new Vt(null,new Wt(this,16))},g.ad=function(t){my(this,t)},g.Nc=function(){return new Wt(this,16)},g.Oc=function(){return new Vt(null,new Wt(this,16))},g.Vc=function(t,n){++this.b,aw(this.a,t,n)},g.Fc=function(t){return cP(this,t)},g.Wc=function(t,n){return++this.b,rpe(this.a,t,n)},g.Gc=function(t){return++this.b,Vi(this.a,t)},g.$b=function(){++this.b,this.a.c=pe(yn,st,1,0,5,1)},g.Hc=function(t){return Ha(this.a,t,0)!=-1},g.Ic=function(t){return FA(this.a,t)},g.Xb=function(t){return ut(this.a,t)},g.Xc=function(t){return Ha(this.a,t,0)},g.dc=function(){return this.a.c.length==0},g.Kc=function(){return J4(new Q(this.a))},g.Yc=function(){throw ne(new Xn)},g.Zc=function(t){throw ne(new Xn)},g.$c=function(t){return++this.b,f0(this.a,t)},g.Mc=function(t){return Wfe(this,t)},g._c=function(t,n){return++this.b,dl(this.a,t,n)},g.gc=function(){return this.a.c.length},g.bd=function(t,n){return new F1(this.a,t,n)},g.Pc=function(){return BY(this.a)},g.Qc=function(t){return Mf(this.a,t)},g.b=0,D(R9,"NNode/ChangeAwareArrayList",795),M(269,1,{},_p),D(R9,"NNode/NNodeBuilder",269),M(1630,1,{},nn),g.a=!1,g.f=Ir,g.j=0,D(R9,"NetworkSimplex",1630),M(1294,1,mn,qqe),g.td=function(t){ebt(this.a,h(t,680),!0,!1)},D(mvt,"NodeLabelAndSizeCalculator/lambda$0$Type",1294),M(558,1,{},XO),g.b=!0,g.c=!0,g.d=!0,g.e=!0,D(mvt,"NodeMarginCalculator",558),M(212,1,{212:1}),g.j=!1,g.k=!1;var H3t=D(e2,"Cell",212);M(124,212,{124:1,212:1},Btt),g.Re=function(){return xP(this)},g.Se=function(){var t;return t=this.n,this.a.a+t.b+t.c},D(e2,"AtomicCell",124),M(232,22,{3:1,35:1,22:1,232:1},Kq);var ao,Jo,oo,Xw=tr(e2,"ContainerArea",232,sr,Ywn,Lgn),z3t;M(326,212,yvt),D(e2,"ContainerCell",326),M(1473,326,yvt,Jht),g.Re=function(){var t;return t=0,this.e?this.b?t=this.b.b:this.a[1][1]&&(t=this.a[1][1].Re()):t=Tpe(this,h1t(this,!0)),t>0?t+this.n.d+this.n.a:0},g.Se=function(){var t,n,s,a,u;if(u=0,this.e)this.b?u=this.b.a:this.a[1][1]&&(u=this.a[1][1].Se());else if(this.g)u=Tpe(this,JX(this,null,!0));else for(n=(Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])),s=0,a=n.length;s<a;++s)t=n[s],u=d.Math.max(u,Tpe(this,JX(this,t,!0)));return u>0?u+this.n.b+this.n.c:0},g.Te=function(){var t,n,s,a,u;if(this.g)for(t=JX(this,null,!1),s=(Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])),a=0,u=s.length;a<u;++a)n=s[a],Rgt(this,n,t);else for(s=(Kh(),se(ie(Xw,1),ze,232,0,[ao,Jo,oo])),a=0,u=s.length;a<u;++a)n=s[a],t=JX(this,n,!1),Rgt(this,n,t)},g.Ue=function(){var t,n,s,a;n=this.i,t=this.n,a=h1t(this,!1),M0e(this,(Kh(),ao),n.d+t.d,a),M0e(this,oo,n.d+n.a-t.a-a[2],a),s=n.a-t.d-t.a,a[0]>0&&(a[0]+=this.d,s-=a[0]),a[2]>0&&(a[2]+=this.d,s-=a[2]),this.c.a=d.Math.max(0,s),this.c.d=n.d+t.d+(this.c.a-s)/2,a[1]=d.Math.max(a[1],s),M0e(this,Jo,n.d+t.d+a[0]-(a[1]-s)/2,a)},g.b=null,g.d=0,g.e=!1,g.f=!1,g.g=!1;var fte=0,Vj=0;D(e2,"GridContainerCell",1473),M(461,22,{3:1,35:1,22:1,461:1},Xq);var jg,v1,Sh,G3t=tr(e2,"HorizontalLabelAlignment",461,sr,Wwn,Ign),V3t;M(306,212,{212:1,306:1},Wit,tut,Uit),g.Re=function(){return Int(this)},g.Se=function(){return nde(this)},g.a=0,g.c=!1;var TPn=D(e2,"LabelCell",306);M(244,326,{212:1,326:1,244:1},u9),g.Re=function(){return gM(this)},g.Se=function(){return pM(this)},g.Te=function(){VQ(this)},g.Ue=function(){UQ(this)},g.b=0,g.c=0,g.d=!1,D(e2,"StripContainerCell",244),M(1626,1,Cr,hi),g.Mb=function(t){return Ufn(h(t,212))},D(e2,"StripContainerCell/lambda$0$Type",1626),M(1627,1,{},Hi),g.Fe=function(t){return h(t,212).Se()},D(e2,"StripContainerCell/lambda$1$Type",1627),M(1628,1,Cr,ss),g.Mb=function(t){return qfn(h(t,212))},D(e2,"StripContainerCell/lambda$2$Type",1628),M(1629,1,{},ls),g.Fe=function(t){return h(t,212).Re()},D(e2,"StripContainerCell/lambda$3$Type",1629),M(462,22,{3:1,35:1,22:1,462:1},Qq);var Ah,$g,nf,U3t=tr(e2,"VerticalLabelAlignment",462,sr,Kwn,Ogn),q3t;M(789,1,{},jve),g.c=0,g.d=0,g.k=0,g.s=0,g.t=0,g.v=!1,g.w=0,g.D=!1,D(DR,"NodeContext",789),M(1471,1,Yr,vs),g.ue=function(t,n){return qJe(h(t,61),h(n,61))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(DR,"NodeContext/0methodref$comparePortSides$Type",1471),M(1472,1,Yr,ti),g.ue=function(t,n){return m9n(h(t,111),h(n,111))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(DR,"NodeContext/1methodref$comparePortContexts$Type",1472),M(159,22,{3:1,35:1,22:1,159:1},bl);var Y3t,W3t,K3t,X3t,Q3t,Z3t,J3t,e4t,t4t,n4t,r4t,i4t,s4t,a4t,o4t,c4t,u4t,l4t,h4t,f4t,d4t,dte,g4t=tr(DR,"NodeLabelLocation",159,sr,LX,Ngn),p4t;M(111,1,{111:1},S0t),g.a=!1,D(DR,"PortContext",111),M(1476,1,mn,zi),g.td=function(t){hQe(h(t,306))},D(zM,kvt,1476),M(1477,1,Cr,as),g.Mb=function(t){return!!h(t,111).c},D(zM,xvt,1477),M(1478,1,mn,ai),g.td=function(t){hQe(h(t,111).c)},D(zM,"LabelPlacer/lambda$2$Type",1478);var y4e;M(1475,1,mn,xu),g.td=function(t){rw(),Tfn(h(t,111))},D(zM,"NodeLabelAndSizeUtilities/lambda$0$Type",1475),M(790,1,mn,P1e),g.td=function(t){F1n(this.b,this.c,this.a,h(t,181))},g.a=!1,g.c=!1,D(zM,"NodeLabelCellCreator/lambda$0$Type",790),M(1474,1,mn,Yqe),g.td=function(t){Afn(this.a,h(t,181))},D(zM,"PortContextCreator/lambda$0$Type",1474);var Uj;M(1829,1,{},No),D(Sx,"GreedyRectangleStripOverlapRemover",1829),M(1830,1,Yr,hc),g.ue=function(t,n){return p0n(h(t,222),h(n,222))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Sx,"GreedyRectangleStripOverlapRemover/0methodref$compareByYCoordinate$Type",1830),M(1786,1,{},GKe),g.a=5,g.e=0,D(Sx,"RectangleStripOverlapRemover",1786),M(1787,1,Yr,Yc),g.ue=function(t,n){return b0n(h(t,222),h(n,222))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Sx,"RectangleStripOverlapRemover/0methodref$compareLeftRectangleBorders$Type",1787),M(1789,1,Yr,lh),g.ue=function(t,n){return $bn(h(t,222),h(n,222))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Sx,"RectangleStripOverlapRemover/1methodref$compareRightRectangleBorders$Type",1789),M(406,22,{3:1,35:1,22:1,406:1},NN);var hD,gte,pte,fD,b4t=tr(Sx,"RectangleStripOverlapRemover/OverlapRemovalDirection",406,sr,Wmn,Pgn),v4t;M(222,1,{222:1},lW),D(Sx,"RectangleStripOverlapRemover/RectangleNode",222),M(1788,1,mn,Wqe),g.td=function(t){V8n(this.a,h(t,222))},D(Sx,"RectangleStripOverlapRemover/lambda$1$Type",1788),M(1304,1,Yr,su),g.ue=function(t,n){return XLn(h(t,167),h(n,167))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(u1,"PolyominoCompactor/CornerCasesGreaterThanRestComparator",1304),M(1307,1,{},Ru),g.Kb=function(t){return h(t,324).a},D(u1,"PolyominoCompactor/CornerCasesGreaterThanRestComparator/lambda$0$Type",1307),M(1308,1,Cr,fc),g.Mb=function(t){return h(t,323).a},D(u1,"PolyominoCompactor/CornerCasesGreaterThanRestComparator/lambda$1$Type",1308),M(1309,1,Cr,Ll),g.Mb=function(t){return h(t,323).a},D(u1,"PolyominoCompactor/CornerCasesGreaterThanRestComparator/lambda$2$Type",1309),M(1302,1,Yr,ol),g.ue=function(t,n){return DMn(h(t,167),h(n,167))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(u1,"PolyominoCompactor/MinNumOfExtensionDirectionsComparator",1302),M(1305,1,{},Si),g.Kb=function(t){return h(t,324).a},D(u1,"PolyominoCompactor/MinNumOfExtensionDirectionsComparator/lambda$0$Type",1305),M(767,1,Yr,Ur),g.ue=function(t,n){return I5n(h(t,167),h(n,167))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(u1,"PolyominoCompactor/MinNumOfExtensionsComparator",767),M(1300,1,Yr,bt),g.ue=function(t,n){return H4n(h(t,321),h(n,321))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(u1,"PolyominoCompactor/MinPerimeterComparator",1300),M(1301,1,Yr,$n),g.ue=function(t,n){return v8n(h(t,321),h(n,321))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(u1,"PolyominoCompactor/MinPerimeterComparatorWithShape",1301),M(1303,1,Yr,Er),g.ue=function(t,n){return JMn(h(t,167),h(n,167))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(u1,"PolyominoCompactor/SingleExtensionSideGreaterThanRestComparator",1303),M(1306,1,{},Ss),g.Kb=function(t){return h(t,324).a},D(u1,"PolyominoCompactor/SingleExtensionSideGreaterThanRestComparator/lambda$0$Type",1306),M(777,1,{},zhe),g.Ce=function(t,n){return Vmn(this,h(t,46),h(n,167))},D(u1,"SuccessorCombination",777),M(644,1,{},wo),g.Ce=function(t,n){var s;return OTn((s=h(t,46),h(n,167),s))},D(u1,"SuccessorJitter",644),M(643,1,{},Po),g.Ce=function(t,n){var s;return kAn((s=h(t,46),h(n,167),s))},D(u1,"SuccessorLineByLine",643),M(568,1,{},Ra),g.Ce=function(t,n){var s;return NCn((s=h(t,46),h(n,167),s))},D(u1,"SuccessorManhattan",568),M(1356,1,{},Eu),g.Ce=function(t,n){var s;return USn((s=h(t,46),h(n,167),s))},D(u1,"SuccessorMaxNormWindingInMathPosSense",1356),M(400,1,{},R6),g.Ce=function(t,n){return Dde(this,t,n)},g.c=!1,g.d=!1,g.e=!1,g.f=!1,D(u1,"SuccessorQuadrantsGeneric",400),M(1357,1,{},Lc),g.Kb=function(t){return h(t,324).a},D(u1,"SuccessorQuadrantsGeneric/lambda$0$Type",1357),M(323,22,{3:1,35:1,22:1,323:1},ON),g.a=!1;var dD,gD,pD,bD,w4t=tr(IR,bwe,323,sr,Qmn,Bgn),m4t;M(1298,1,{}),g.Ib=function(){var t,n,s,a,u,p;for(s=" ",t=Ke(0),u=0;u<this.o;u++)s+=""+t.a,t=Ke(Ttt(t.a));for(s+=`
`,t=Ke(0),p=0;p<this.p;p++){for(s+=""+t.a,t=Ke(Ttt(t.a)),a=0;a<this.o;a++)n=RK(this,a,p),mo(n,0)==0?s+="_":mo(n,1)==0?s+="X":s+="0";s+=`
`}return $u(s,0,s.length-1)},g.o=0,g.p=0,D(IR,"TwoBitGrid",1298),M(321,1298,{321:1},Fpe),g.j=0,g.k=0,D(IR,"PlanarGrid",321),M(167,321,{321:1,167:1}),g.g=0,g.i=0,D(IR,"Polyomino",167);var CPn=fi(GM,_vt);M(134,1,vwe,bd),g.Ye=function(t,n){return zA(this,t,n)},g.Ve=function(){return Ynt(this)},g.We=function(t){return U(this,t)},g.Xe=function(t){return os(this,t)},D(GM,"MapPropertyHolder",134),M(1299,134,vwe,Cgt),D(IR,"Polyominoes",1299);var y4t=!1,g_,k4e;M(1766,1,mn,hh),g.td=function(t){_2t(h(t,221))},D(y5,"DepthFirstCompaction/0methodref$compactTree$Type",1766),M(810,1,mn,Mle),g.td=function(t){hvn(this.a,h(t,221))},D(y5,"DepthFirstCompaction/lambda$1$Type",810),M(1767,1,mn,ant),g.td=function(t){Pxn(this.a,this.b,this.c,h(t,221))},D(y5,"DepthFirstCompaction/lambda$2$Type",1767);var p_,x4e;M(65,1,{65:1},dst),D(y5,"Node",65),M(1250,1,{},det),D(y5,"ScanlineOverlapCheck",1250),M(1251,1,{679:1},Oit),g.Ke=function(t){W0n(this,h(t,440))},D(y5,"ScanlineOverlapCheck/OverlapsScanlineHandler",1251),M(1252,1,Yr,tg),g.ue=function(t,n){return vkn(h(t,65),h(n,65))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(y5,"ScanlineOverlapCheck/OverlapsScanlineHandler/lambda$0$Type",1252),M(440,1,{440:1},Ghe),g.a=!1,D(y5,"ScanlineOverlapCheck/Timestamp",440),M(1253,1,Yr,Di),g.ue=function(t,n){return Y7n(h(t,440),h(n,440))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(y5,"ScanlineOverlapCheck/lambda$0$Type",1253),M(550,1,{},Bi),D(Tvt,"SVGImage",550),M(324,1,{324:1},B1e),g.Ib=function(){return"("+this.a+Js+this.b+Js+this.c+")"},D(Tvt,"UniqueTriple",324),M(209,1,t2),D(Uy,"AbstractLayoutProvider",209),M(1132,209,t2,ra),g.Ze=function(t,n){var s,a,u,p;switch(Bn(n,Cvt,1),this.a=Pe(Je(pt(t,(l9(),A4e)))),Pp(t,vte)&&(u=Jn(pt(t,vte)),s=FQ(Ok(),u),s&&(a=h(zW(s.f),209),a.Ze(t,jo(n,1)))),p=new Mat(this.a),this.b=BOn(p,t),h(pt(t,(sX(),_4e)),481).g){case 0:ZTn(new Rr,this.b),ya(t,Wj,U(this.b,Wj));break;default:L1()}VOn(p),ya(t,C4e,this.b),An(n)},g.a=0,D(Svt,"DisCoLayoutProvider",1132),M(1244,1,{},Rr),g.c=!1,g.e=0,g.f=0,D(Svt,"DisCoPolyominoCompactor",1244),M(561,1,{561:1},Znt),g.b=!0,D(NR,"DCComponent",561),M(394,22,{3:1,35:1,22:1,394:1},IN),g.a=!1;var qj,vD,Yj,wD,k4t=tr(NR,"DCDirection",394,sr,Xmn,Fgn),x4t;M(266,134,{3:1,266:1,94:1,134:1},EQ),D(NR,"DCElement",266),M(395,1,{395:1},o2e),g.c=0,D(NR,"DCExtension",395),M(755,134,vwe,YXe),D(NR,"DCGraph",755),M(481,22,{3:1,35:1,22:1,481:1},Jet);var bte,E4e=tr(sJ,wwe,481,sr,Cvn,Rgn),E4t;M(854,1,$l,XVe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,mwe),Avt),"Connected Components Compaction Strategy"),"Strategy for packing different connected components in order to save space and enhance readability of a graph."),T4e),(E0(),_i)),E4e),Lt((Qh(),gn))))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,ywe),Avt),"Connected Components Layout Algorithm"),"A layout algorithm that is to be applied to each connected component before the components themselves are compacted. If unspecified, the positions of the components' nodes are not altered."),b8),it),Lt(gn)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,kwe),"debug"),"DCGraph"),"Access to the DCGraph is intended for the debug view,"),jf),yn),Lt(gn)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,xwe),"debug"),"List of Polyominoes"),"Access to the polyominoes is intended for the debug view,"),jf),yn),Lt(gn)))),f2t((new QVe,t))};var _4t,_4e,T4e,T4t,C4t;D(sJ,"DisCoMetaDataProvider",854),M(998,1,$l,QVe),g.Qe=function(t){f2t(t)};var S4t,vte,A4t,C4e,Wj,wte,S4e,M4t,D4t,L4t,I4t,A4e;D(sJ,"DisCoOptions",998),M(999,1,{},au),g.$e=function(){var t;return t=new ra,t},g._e=function(t){},D(sJ,"DisCoOptions/DiscoFactory",999),M(562,167,{321:1,167:1,562:1},ugt),g.a=0,g.b=0,g.c=0,g.d=0,D("org.eclipse.elk.alg.disco.structures","DCPolyomino",562);var mte,yte,Kj;M(1268,1,Cr,gt),g.Mb=function(t){return wfe(t)},D(qy,"ElkGraphComponentsProcessor/lambda$0$Type",1268),M(1269,1,{},O6),g.Kb=function(t){return V4(),H1(h(t,79))},D(qy,"ElkGraphComponentsProcessor/lambda$1$Type",1269),M(1270,1,Cr,It),g.Mb=function(t){return F2n(h(t,79))},D(qy,"ElkGraphComponentsProcessor/lambda$2$Type",1270),M(1271,1,{},n0),g.Kb=function(t){return V4(),Sg(h(t,79))},D(qy,"ElkGraphComponentsProcessor/lambda$3$Type",1271),M(1272,1,Cr,jO),g.Mb=function(t){return R2n(h(t,79))},D(qy,"ElkGraphComponentsProcessor/lambda$4$Type",1272),M(1273,1,Cr,Kqe),g.Mb=function(t){return Awn(this.a,h(t,79))},D(qy,"ElkGraphComponentsProcessor/lambda$5$Type",1273),M(1274,1,{},Xqe),g.Kb=function(t){return qbn(this.a,h(t,79))},D(qy,"ElkGraphComponentsProcessor/lambda$6$Type",1274),M(1241,1,{},Mat),g.a=0,D(qy,"ElkGraphTransformer",1241),M(1242,1,{},rq),g.Od=function(t,n){FTn(this,h(t,160),h(n,266))},D(qy,"ElkGraphTransformer/OffsetApplier",1242),M(1243,1,mn,Qqe),g.td=function(t){c0n(this,h(t,8))},D(qy,"ElkGraphTransformer/OffsetApplier/OffSetToChainApplier",1243),M(753,1,{},tS),D(_we,Twe,753),M(1232,1,Yr,iq),g.ue=function(t,n){return ETn(h(t,231),h(n,231))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(_we,Mvt,1232),M(740,209,t2,Wle),g.Ze=function(t,n){fgt(this,t,n)},D(_we,"ForceLayoutProvider",740),M(357,134,{3:1,357:1,94:1,134:1}),D(VM,"FParticle",357),M(559,357,{3:1,559:1,357:1,94:1,134:1},Ort),g.Ib=function(){var t;return this.a?(t=Ha(this.a.a,this,0),t>=0?"b"+t+"["+cK(this.a)+"]":"b["+cK(this.a)+"]"):"b_"+rb(this)},D(VM,"FBendpoint",559),M(282,134,{3:1,282:1,94:1,134:1},Att),g.Ib=function(){return cK(this)},D(VM,"FEdge",282),M(231,134,{3:1,231:1,94:1,134:1},mB);var SPn=D(VM,"FGraph",231);M(447,357,{3:1,447:1,357:1,94:1,134:1},Lat),g.Ib=function(){return this.b==null||this.b.length==0?"l["+cK(this.a)+"]":"l_"+this.b},D(VM,"FLabel",447),M(144,357,{3:1,144:1,357:1,94:1,134:1},Hrt),g.Ib=function(){return o0e(this)},g.b=0,D(VM,"FNode",144),M(2003,1,{}),g.bf=function(t){wve(this,t)},g.cf=function(){l1t(this)},g.d=0,D(Cwe,"AbstractForceModel",2003),M(631,2003,{631:1},ylt),g.af=function(t,n){var s,a,u,p,v;return a0t(this.f,t,n),u=ms(ro(n.d),t.d),v=d.Math.sqrt(u.a*u.a+u.b*u.b),a=d.Math.max(0,v-NE(t.e)/2-NE(n.e)/2),s=Xlt(this.e,t,n),s>0?p=-Fbn(a,this.c)*s:p=M0n(a,this.b)*h(U(t,(Jh(),qx)),19).a,Jf(u,p/v),u},g.bf=function(t){wve(this,t),this.a=h(U(t,(Jh(),Qj)),19).a,this.c=Pe(Je(U(t,Zj))),this.b=Pe(Je(U(t,xte)))},g.df=function(t){return t<this.a},g.a=0,g.b=0,g.c=0,D(Cwe,"EadesModel",631),M(632,2003,{632:1},knt),g.af=function(t,n){var s,a,u,p,v;return a0t(this.f,t,n),u=ms(ro(n.d),t.d),v=d.Math.sqrt(u.a*u.a+u.b*u.b),a=d.Math.max(0,v-NE(t.e)/2-NE(n.e)/2),p=A0n(a,this.a)*h(U(t,(Jh(),qx)),19).a,s=Xlt(this.e,t,n),s>0&&(p-=Hfn(a,this.a)*s),Jf(u,p*this.b/v),u},g.bf=function(t){var n,s,a,u,p,v,m;for(wve(this,t),this.b=Pe(Je(U(t,(Jh(),Ete)))),this.c=this.b/h(U(t,Qj),19).a,a=t.e.c.length,p=0,u=0,m=new Q(t.e);m.a<m.c.c.length;)v=h(K(m),144),p+=v.e.a,u+=v.e.b;n=p*u,s=Pe(Je(U(t,Zj)))*If,this.a=d.Math.sqrt(n/(2*a))*s},g.cf=function(){l1t(this),this.b-=this.c},g.df=function(t){return this.b>0},g.a=0,g.b=0,g.c=0,D(Cwe,"FruchtermanReingoldModel",632),M(849,1,$l,JVe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,PR),""),"Force Model"),"Determines the model for force calculation."),M4e),(E0(),_i)),D4e),Lt((Qh(),gn))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Swe),""),"Iterations"),"The number of iterations on the force model."),Ke(300)),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Awe),""),"Repulsive Power"),"Determines how many bend points are added to the edge; such bend points are regarded as repelling particles in the force model"),Ke(0)),po),Ws),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,uJ),""),"FR Temperature"),"The temperature is used as a scaling factor for particle displacements."),l1),Ba),Ts),Lt(gn)))),Es(t,uJ,PR,j4t),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,lJ),""),"Eades Repulsion"),"Factor for repulsive forces in Eades' model."),5),Ba),Ts),Lt(gn)))),Es(t,lJ,PR,B4t),obt((new eUe,t))};var O4t,N4t,M4e,P4t,B4t,F4t,R4t,j4t;D($9,"ForceMetaDataProvider",849),M(424,22,{3:1,35:1,22:1,424:1},qhe);var kte,Xj,D4e=tr($9,"ForceModelStrategy",424,sr,Ewn,jgn),$4t;M(988,1,$l,eUe),g.Qe=function(t){obt(t)};var H4t,z4t,L4e,Qj,I4e,G4t,V4t,U4t,O4e,q4t,N4e,P4e,Y4t,qx,W4t,xte,B4e,K4t,X4t,Zj,Ete;D($9,"ForceOptions",988),M(989,1,{},fh),g.$e=function(){var t;return t=new Wle,t},g._e=function(t){},D($9,"ForceOptions/ForceFactory",989);var mD,b_,D5,Jj;M(850,1,$l,tUe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Lwe),""),"Fixed Position"),"Prevent that the node is moved by the layout algorithm."),(on(),!1)),(E0(),zs)),es),Lt((Qh(),gs))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Iwe),""),"Desired Edge Length"),"Either specified for parent nodes or for individual edges, where the latter takes higher precedence."),100),Ba),Ts),ni(gn,se(ie(P0,1),ze,175,0,[k1]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Owe),""),"Layout Dimension"),"Dimensions that are permitted to be altered during layout."),F4e),_i),V4e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Nwe),""),"Stress Epsilon"),"Termination criterion for the iterative process."),l1),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Pwe),""),"Iteration Limit"),"Maximum number of performed iterations. Takes higher precedence than 'epsilon'."),Ke(Ir)),po),Ws),Lt(gn)))),U2t((new nUe,t))};var Q4t,Z4t,F4e,J4t,e5t,t5t;D($9,"StressMetaDataProvider",850),M(992,1,$l,nUe),g.Qe=function(t){U2t(t)};var e$,R4e,j4e,$4e,H4e,z4e,n5t,r5t,i5t,s5t,G4e,a5t;D($9,"StressOptions",992),M(993,1,{},Qm),g.$e=function(){var t;return t=new Mtt,t},g._e=function(t){},D($9,"StressOptions/StressFactory",993),M(1128,209,t2,Mtt),g.Ze=function(t,n){var s,a,u,p,v;for(Bn(n,Ovt,1),ft(lt(pt(t,(sM(),H4e))))?ft(lt(pt(t,G4e)))||nB((s=new hS((Qv(),new Yv(t))),s)):fgt(new Wle,t,jo(n,1)),u=zlt(t),a=t2t(this.a,u),v=a.Kc();v.Ob();)p=h(v.Pb(),231),!(p.e.c.length<=1)&&(WLn(this.b,p),CCn(this.b),mc(p.d,new nS));u=gbt(a),hbt(u),An(n)},D(RR,"StressLayoutProvider",1128),M(1129,1,mn,nS),g.td=function(t){Tve(h(t,447))},D(RR,"StressLayoutProvider/lambda$0$Type",1129),M(990,1,{},BKe),g.c=0,g.e=0,g.g=0,D(RR,"StressMajorization",990),M(379,22,{3:1,35:1,22:1,379:1},Zq);var _te,Tte,Cte,V4e=tr(RR,"StressMajorization/Dimension",379,sr,Qwn,$gn),o5t;M(991,1,Yr,Zqe),g.ue=function(t,n){return bgn(this.a,h(t,144),h(n,144))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(RR,"StressMajorization/lambda$0$Type",991),M(1229,1,{},Vst),D(_5,"ElkLayered",1229),M(1230,1,mn,rS),g.td=function(t){STn(h(t,37))},D(_5,"ElkLayered/lambda$0$Type",1230),M(1231,1,mn,Jqe),g.td=function(t){vgn(this.a,h(t,37))},D(_5,"ElkLayered/lambda$1$Type",1231),M(1263,1,{},pet);var c5t,u5t,l5t;D(_5,"GraphConfigurator",1263),M(759,1,mn,Dle),g.td=function(t){wdt(this.a,h(t,10))},D(_5,"GraphConfigurator/lambda$0$Type",759),M(760,1,{},$O),g.Kb=function(t){return x2e(),new Vt(null,new Wt(h(t,29).a,16))},D(_5,"GraphConfigurator/lambda$1$Type",760),M(761,1,mn,Lle),g.td=function(t){wdt(this.a,h(t,10))},D(_5,"GraphConfigurator/lambda$2$Type",761),M(1127,209,t2,jKe),g.Ze=function(t,n){var s;s=pLn(new UKe,t),Me(pt(t,(tt(),sm)))===Me((Ld(),B0))?Ykn(this.a,s,n):GTn(this.a,s,n),abt(new iUe,s)},D(_5,"LayeredLayoutProvider",1127),M(356,22,{3:1,35:1,22:1,356:1},SS);var w1,Hg,oc,Go,Pa,U4e=tr(_5,"LayeredPhases",356,sr,Nyn,Hgn),h5t;M(1651,1,{},Mut),g.i=0;var f5t;D(YM,"ComponentsToCGraphTransformer",1651);var d5t;M(1652,1,{},sq),g.ef=function(t,n){return d.Math.min(t.a!=null?Pe(t.a):t.c.i,n.a!=null?Pe(n.a):n.c.i)},g.ff=function(t,n){return d.Math.min(t.a!=null?Pe(t.a):t.c.i,n.a!=null?Pe(n.a):n.c.i)},D(YM,"ComponentsToCGraphTransformer/1",1652),M(81,1,{81:1}),g.i=0,g.k=!0,g.o=Fi;var Ste=D(G9,"CNode",81);M(460,81,{460:1,81:1},Yfe,Wpe),g.Ib=function(){return""},D(YM,"ComponentsToCGraphTransformer/CRectNode",460),M(1623,1,{},D1);var Ate,Mte;D(YM,"OneDimensionalComponentsCompaction",1623),M(1624,1,{},Zm),g.Kb=function(t){return zwn(h(t,46))},g.Fb=function(t){return this===t},D(YM,"OneDimensionalComponentsCompaction/lambda$0$Type",1624),M(1625,1,{},r0),g.Kb=function(t){return txn(h(t,46))},g.Fb=function(t){return this===t},D(YM,"OneDimensionalComponentsCompaction/lambda$1$Type",1625),M(1654,1,{},$rt),D(G9,"CGraph",1654),M(189,1,{189:1},AX),g.b=0,g.c=0,g.e=0,g.g=!0,g.i=Fi,D(G9,"CGroup",189),M(1653,1,{},H2),g.ef=function(t,n){return d.Math.max(t.a!=null?Pe(t.a):t.c.i,n.a!=null?Pe(n.a):n.c.i)},g.ff=function(t,n){return d.Math.max(t.a!=null?Pe(t.a):t.c.i,n.a!=null?Pe(n.a):n.c.i)},D(G9,cvt,1653),M(1655,1,{},m0t),g.d=!1;var g5t,Dte=D(G9,hvt,1655);M(1656,1,{},HO),g.Kb=function(t){return Ohe(),on(),h(h(t,46).a,81).d.e!=0},g.Fb=function(t){return this===t},D(G9,fvt,1656),M(823,1,{},rde),g.a=!1,g.b=!1,g.c=!1,g.d=!1,D(G9,dvt,823),M(1825,1,{},irt),D(jR,gvt,1825);var yD=fi(r2,avt);M(1826,1,{369:1},Pit),g.Ke=function(t){BAn(this,h(t,466))},D(jR,pvt,1826),M(1827,1,Yr,D7),g.ue=function(t,n){return Bvn(h(t,81),h(n,81))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(jR,bvt,1827),M(466,1,{466:1},Whe),g.a=!1,D(jR,vvt,466),M(1828,1,Yr,sS),g.ue=function(t,n){return W7n(h(t,466),h(n,466))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(jR,wvt,1828),M(140,1,{140:1},Z6,Z1e),g.Fb=function(t){var n;return t==null||APn!=uu(t)?!1:(n=h(t,140),Bo(this.c,n.c)&&Bo(this.d,n.d))},g.Hb=function(){return lF(se(ie(yn,1),st,1,5,[this.c,this.d]))},g.Ib=function(){return"("+this.c+Js+this.d+(this.a?"cx":"")+this.b+")"},g.a=!0,g.c=0,g.d=0;var APn=D(r2,"Point",140);M(405,22,{3:1,35:1,22:1,405:1},PN);var Zb,Qw,r3,Zw,p5t=tr(r2,"Point/Quadrant",405,sr,Zmn,zgn),b5t;M(1642,1,{},$Ke),g.b=null,g.c=null,g.d=null,g.e=null,g.f=null;var v5t,w5t,m5t,y5t,k5t;D(r2,"RectilinearConvexHull",1642),M(574,1,{369:1},DF),g.Ke=function(t){P3n(this,h(t,140))},g.b=0;var q4e;D(r2,"RectilinearConvexHull/MaximalElementsEventHandler",574),M(1644,1,Yr,w4),g.ue=function(t,n){return Tvn(Je(t),Je(n))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(r2,"RectilinearConvexHull/MaximalElementsEventHandler/lambda$0$Type",1644),M(1643,1,{369:1},Zct),g.Ke=function(t){KSn(this,h(t,140))},g.a=0,g.b=null,g.c=null,g.d=null,g.e=null,D(r2,"RectilinearConvexHull/RectangleEventHandler",1643),M(1645,1,Yr,M7),g.ue=function(t,n){return Amn(h(t,140),h(n,140))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(r2,"RectilinearConvexHull/lambda$0$Type",1645),M(1646,1,Yr,iS),g.ue=function(t,n){return Mmn(h(t,140),h(n,140))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(r2,"RectilinearConvexHull/lambda$1$Type",1646),M(1647,1,Yr,z2),g.ue=function(t,n){return Lmn(h(t,140),h(n,140))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(r2,"RectilinearConvexHull/lambda$2$Type",1647),M(1648,1,Yr,_Be),g.ue=function(t,n){return Dmn(h(t,140),h(n,140))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(r2,"RectilinearConvexHull/lambda$3$Type",1648),M(1649,1,Yr,TBe),g.ue=function(t,n){return O9n(h(t,140),h(n,140))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(r2,"RectilinearConvexHull/lambda$4$Type",1649),M(1650,1,{},fst),D(r2,"Scanline",1650),M(2005,1,{}),D(h1,"AbstractGraphPlacer",2005),M(325,1,{325:1},itt),g.mf=function(t){return this.nf(t)?(Ot(this.b,h(U(t,(He(),em)),21),t),!0):!1},g.nf=function(t){var n,s,a,u;for(n=h(U(t,(He(),em)),21),u=h(jr(Wr,n),21),a=u.Kc();a.Ob();)if(s=h(a.Pb(),21),!h(jr(this.b,s),15).dc())return!1;return!0};var Wr;D(h1,"ComponentGroup",325),M(765,2005,{},Zle),g.of=function(t){var n,s;for(s=new Q(this.a);s.a<s.c.c.length;)if(n=h(K(s),325),n.mf(t))return;Ue(this.a,new itt(t))},g.lf=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V;if(this.a.c=pe(yn,st,1,0,5,1),n.a.c=pe(yn,st,1,0,5,1),t.dc()){n.f.a=0,n.f.b=0;return}for(v=h(t.Xb(0),37),Oa(n,v),u=t.Kc();u.Ob();)a=h(u.Pb(),37),this.of(a);for(V=new Rs,p=Pe(Je(U(v,(tt(),ND)))),_=new Q(this.a);_.a<_.c.c.length;)m=h(K(_),325),A=xbt(m,p),g0(zP(m.b),V.a,V.b),V.a+=A.a,V.b+=A.b;if(n.f.a=V.a-p,n.f.b=V.b-p,ft(lt(U(v,$$)))&&Me(U(v,Hd))===Me((Od(),y8))){for(j=t.Kc();j.Ob();)I=h(j.Pb(),37),_9(I,I.c.a,I.c.b);for(s=new aq,Uve(s,t,p),P=t.Kc();P.Ob();)I=h(P.Pb(),37),zr(Uh(I.c),s.e);zr(Uh(n.f),s.a)}for(x=new Q(this.a);x.a<x.c.c.length;)m=h(K(x),325),U0e(n,zP(m.b))},D(h1,"ComponentGroupGraphPlacer",765),M(1293,765,{},ZKe),g.of=function(t){Mht(this,t)},g.lf=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we;if(this.a.c=pe(yn,st,1,0,5,1),n.a.c=pe(yn,st,1,0,5,1),t.dc()){n.f.a=0,n.f.b=0;return}for(v=h(t.Xb(0),37),Oa(n,v),u=t.Kc();u.Ob();)a=h(u.Pb(),37),Mht(this,a);for(we=new Rs,le=new Rs,J=new Rs,V=new Rs,p=Pe(Je(U(v,(tt(),ND)))),_=new Q(this.a);_.a<_.c.c.length;){if(m=h(K(_),325),lg(h(U(n,(Tr(),hv)),103))){for(J.a=we.a,fe=new R7(OE(TW(m.b).a).a.kc());fe.b.Ob();)if(oe=h(AN(fe.b.Pb()),21),oe.Hc((Qe(),sn))){J.a=le.a;break}}else if(ek(h(U(n,hv),103))){for(J.b=we.b,fe=new R7(OE(TW(m.b).a).a.kc());fe.b.Ob();)if(oe=h(AN(fe.b.Pb()),21),oe.Hc((Qe(),un))){J.b=le.b;break}}if(A=xbt(h(m,570),p),g0(zP(m.b),J.a,J.b),lg(h(U(n,hv),103))){for(le.a=J.a+A.a,V.a=d.Math.max(V.a,le.a),fe=new R7(OE(TW(m.b).a).a.kc());fe.b.Ob();)if(oe=h(AN(fe.b.Pb()),21),oe.Hc((Qe(),Fn))){we.a=J.a+A.a;break}le.b=J.b+A.b,J.b=le.b,V.b=d.Math.max(V.b,J.b)}else if(ek(h(U(n,hv),103))){for(le.b=J.b+A.b,V.b=d.Math.max(V.b,le.b),fe=new R7(OE(TW(m.b).a).a.kc());fe.b.Ob();)if(oe=h(AN(fe.b.Pb()),21),oe.Hc((Qe(),pn))){we.b=J.b+A.b;break}le.a=J.a+A.a,J.a=le.a,V.a=d.Math.max(V.a,J.a)}}if(n.f.a=V.a-p,n.f.b=V.b-p,ft(lt(U(v,$$)))&&Me(U(v,Hd))===Me((Od(),y8))){for(j=t.Kc();j.Ob();)I=h(j.Pb(),37),_9(I,I.c.a,I.c.b);for(s=new aq,Uve(s,t,p),P=t.Kc();P.Ob();)I=h(P.Pb(),37),zr(Uh(I.c),s.e);zr(Uh(n.f),s.a)}for(x=new Q(this.a);x.a<x.c.c.length;)m=h(K(x),325),U0e(n,zP(m.b))},D(h1,"ComponentGroupModelOrderGraphPlacer",1293),M(423,22,{3:1,35:1,22:1,423:1},Jq);var Lte,Y4e,i3,W4e=tr(h1,"ComponentOrderingStrategy",423,sr,Xwn,Ggn),x5t;M(650,1,{},aq),D(h1,"ComponentsCompactor",650),M(1468,12,Jbt,aot),g.Fc=function(t){return b9(this,h(t,140))},D(h1,"ComponentsCompactor/Hullpoints",1468),M(1465,1,{841:1},Uft),g.a=!1,D(h1,"ComponentsCompactor/InternalComponent",1465),M(1464,1,G1,HKe),g.Jc=function(t){Os(this,t)},g.Kc=function(){return new Q(this.a)},D(h1,"ComponentsCompactor/InternalConnectedComponents",1464),M(1467,1,{594:1},k0t),g.hf=function(){return null},g.jf=function(){return this.a},g.gf=function(){return IX(this.d)},g.kf=function(){return this.b},D(h1,"ComponentsCompactor/InternalExternalExtension",1467),M(1466,1,{594:1},VKe),g.jf=function(){return this.a},g.gf=function(){return IX(this.d)},g.hf=function(){return this.c},g.kf=function(){return this.b},D(h1,"ComponentsCompactor/InternalUnionExternalExtension",1466),M(1470,1,{},jgt),D(h1,"ComponentsCompactor/OuterSegments",1470),M(1469,1,{},zKe),D(h1,"ComponentsCompactor/Segments",1469),M(1264,1,{},Bit),D(h1,Twe,1264),M(1265,1,Yr,CBe),g.ue=function(t,n){return Nmn(h(t,37),h(n,37))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(h1,"ComponentsProcessor/lambda$0$Type",1265),M(570,325,{325:1,570:1},sot),g.mf=function(t){return Zge(this,t)},g.nf=function(t){return mgt(this,t)};var fr;D(h1,"ModelOrderComponentGroup",570),M(1291,2005,{},SBe),g.lf=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt;if(t.gc()==1){we=h(t.Xb(0),37),we!=n&&(n.a.c=pe(yn,st,1,0,5,1),_pt(n,we,0,0),Oa(n,we),qde(n.d,we.d),n.f.a=we.f.a,n.f.b=we.f.b);return}else if(t.dc()){n.a.c=pe(yn,st,1,0,5,1),n.f.a=0,n.f.b=0;return}if(Me(U(n,(tt(),$5)))===Me((Sb(),i3))){for(x=t.Kc();x.Ob();){for(v=h(x.Pb(),37),fe=0,J=new Q(v.a);J.a<J.c.c.length;)V=h(K(J),10),fe+=h(U(V,Cxt),19).a;v.p=fe}Rt(),t.ad(new ABe)}for(p=h(t.Xb(0),37),n.a.c=pe(yn,st,1,0,5,1),Oa(n,p),j=0,Ne=0,_=t.Kc();_.Ob();)v=h(_.Pb(),37),le=v.f,j=d.Math.max(j,le.a),Ne+=le.a*le.b;for(j=d.Math.max(j,d.Math.sqrt(Ne)*Pe(Je(U(n,j$)))),u=Pe(Je(U(n,ND))),Ve=0,rt=0,P=0,s=u,m=t.Kc();m.Ob();)v=h(m.Pb(),37),le=v.f,Ve+le.a>j&&(Ve=0,rt+=P+u,P=0),oe=v.c,_9(v,Ve+oe.a,rt+oe.b),Uh(oe),s=d.Math.max(s,Ve+le.a),P=d.Math.max(P,le.b),Ve+=le.a+u;if(n.f.a=s,n.f.b=rt+P,ft(lt(U(p,$$)))){for(a=new aq,Uve(a,t,u),I=t.Kc();I.Ob();)A=h(I.Pb(),37),zr(Uh(A.c),a.e);zr(Uh(n.f),a.a)}U0e(n,t)},D(h1,"SimpleRowGraphPlacer",1291),M(1292,1,Yr,ABe),g.ue=function(t,n){return L5n(h(t,37),h(n,37))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(h1,"SimpleRowGraphPlacer/1",1292);var E5t;M(1262,1,c1,MBe),g.Lb=function(t){var n;return n=h(U(h(t,243).b,(tt(),Da)),74),!!n&&n.b!=0},g.Fb=function(t){return this===t},g.Mb=function(t){var n;return n=h(U(h(t,243).b,(tt(),Da)),74),!!n&&n.b!=0},D($R,"CompoundGraphPostprocessor/1",1262),M(1261,1,xi,qKe),g.pf=function(t,n){Yft(this,h(t,37),n)},D($R,"CompoundGraphPreprocessor",1261),M(441,1,{441:1},Nht),g.c=!1,D($R,"CompoundGraphPreprocessor/ExternalPort",441),M(243,1,{243:1},vP),g.Ib=function(){return FY(this.c)+":"+p0t(this.b)},D($R,"CrossHierarchyEdge",243),M(763,1,Yr,Ile),g.ue=function(t,n){return y7n(this,h(t,243),h(n,243))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D($R,"CrossHierarchyEdgeComparator",763),M(299,134,{3:1,299:1,94:1,134:1}),g.p=0,D(Zo,"LGraphElement",299),M(17,299,{3:1,17:1,299:1,94:1,134:1},db),g.Ib=function(){return p0t(this)};var Ite=D(Zo,"LEdge",17);M(37,299,{3:1,20:1,37:1,299:1,94:1,134:1},dge),g.Jc=function(t){Os(this,t)},g.Kc=function(){return new Q(this.b)},g.Ib=function(){return this.b.c.length==0?"G-unlayered"+Ag(this.a):this.a.c.length==0?"G-layered"+Ag(this.b):"G[layerless"+Ag(this.a)+", layers"+Ag(this.b)+"]"};var _5t=D(Zo,"LGraph",37),T5t;M(657,1,{}),g.qf=function(){return this.e.n},g.We=function(t){return U(this.e,t)},g.rf=function(){return this.e.o},g.sf=function(){return this.e.p},g.Xe=function(t){return os(this.e,t)},g.tf=function(t){this.e.n.a=t.a,this.e.n.b=t.b},g.uf=function(t){this.e.o.a=t.a,this.e.o.b=t.b},g.vf=function(t){this.e.p=t},D(Zo,"LGraphAdapters/AbstractLShapeAdapter",657),M(577,1,{839:1},QO),g.wf=function(){var t,n;if(!this.b)for(this.b=I1(this.a.b.c.length),n=new Q(this.a.b);n.a<n.c.c.length;)t=h(K(n),70),Ue(this.b,new ZO(t));return this.b},g.b=null,D(Zo,"LGraphAdapters/LEdgeAdapter",577),M(656,1,{},IW),g.xf=function(){var t,n,s,a,u,p;if(!this.b){for(this.b=new qe,a=new Q(this.a.b);a.a<a.c.c.length;)for(s=h(K(a),29),p=new Q(s.a);p.a<p.c.c.length;)if(u=h(K(p),10),this.c.Mb(u)&&(Ue(this.b,new wP(this,u,this.e)),this.d)){if(os(u,(He(),j5)))for(n=h(U(u,j5),15).Kc();n.Ob();)t=h(n.Pb(),10),Ue(this.b,new wP(this,t,!1));if(os(u,N5))for(n=h(U(u,N5),15).Kc();n.Ob();)t=h(n.Pb(),10),Ue(this.b,new wP(this,t,!1))}}return this.b},g.qf=function(){throw ne(new i0(Bvt))},g.We=function(t){return U(this.a,t)},g.rf=function(){return this.a.f},g.sf=function(){return this.a.p},g.Xe=function(t){return os(this.a,t)},g.tf=function(t){throw ne(new i0(Bvt))},g.uf=function(t){this.a.f.a=t.a,this.a.f.b=t.b},g.vf=function(t){this.a.p=t},g.b=null,g.d=!1,g.e=!1,D(Zo,"LGraphAdapters/LGraphAdapter",656),M(576,657,{181:1},ZO),D(Zo,"LGraphAdapters/LLabelAdapter",576),M(575,657,{680:1},wP),g.yf=function(){return this.b},g.zf=function(){return Rt(),Rt(),ca},g.wf=function(){var t,n;if(!this.a)for(this.a=I1(h(this.e,10).b.c.length),n=new Q(h(this.e,10).b);n.a<n.c.c.length;)t=h(K(n),70),Ue(this.a,new ZO(t));return this.a},g.Af=function(){var t;return t=h(this.e,10).d,new k1e(t.d,t.c,t.a,t.b)},g.Bf=function(){return Rt(),Rt(),ca},g.Cf=function(){var t,n;if(!this.c)for(this.c=I1(h(this.e,10).j.c.length),n=new Q(h(this.e,10).j);n.a<n.c.c.length;)t=h(K(n),11),Ue(this.c,new FZe(t,this.d));return this.c},g.Df=function(){return ft(lt(U(h(this.e,10),(He(),g6e))))},g.Ef=function(t){h(this.e,10).d.b=t.b,h(this.e,10).d.d=t.d,h(this.e,10).d.c=t.c,h(this.e,10).d.a=t.a},g.Ff=function(t){h(this.e,10).f.b=t.b,h(this.e,10).f.d=t.d,h(this.e,10).f.c=t.c,h(this.e,10).f.a=t.a},g.Gf=function(){p4n(this,(Y7(),T5t))},g.a=null,g.b=null,g.c=null,g.d=!1,D(Zo,"LGraphAdapters/LNodeAdapter",575),M(1722,657,{838:1},FZe),g.zf=function(){var t,n,s,a;if(this.d&&h(this.e,11).i.k==(bn(),Vo))return Rt(),Rt(),ca;if(!this.a){for(this.a=new qe,s=new Q(h(this.e,11).e);s.a<s.c.c.length;)t=h(K(s),17),Ue(this.a,new QO(t));if(this.d&&(a=h(U(h(this.e,11),(He(),Zc)),10),a))for(n=new Sn(Dn(za(a).a.Kc(),new L));rr(n);)t=h(Un(n),17),Ue(this.a,new QO(t))}return this.a},g.wf=function(){var t,n;if(!this.b)for(this.b=I1(h(this.e,11).f.c.length),n=new Q(h(this.e,11).f);n.a<n.c.c.length;)t=h(K(n),70),Ue(this.b,new ZO(t));return this.b},g.Bf=function(){var t,n,s,a;if(this.d&&h(this.e,11).i.k==(bn(),Vo))return Rt(),Rt(),ca;if(!this.c){for(this.c=new qe,s=new Q(h(this.e,11).g);s.a<s.c.c.length;)t=h(K(s),17),Ue(this.c,new QO(t));if(this.d&&(a=h(U(h(this.e,11),(He(),Zc)),10),a))for(n=new Sn(Dn(qi(a).a.Kc(),new L));rr(n);)t=h(Un(n),17),Ue(this.c,new QO(t))}return this.c},g.Hf=function(){return h(this.e,11).j},g.If=function(){return ft(lt(U(h(this.e,11),(He(),AD))))},g.a=null,g.b=null,g.c=null,g.d=!1,D(Zo,"LGraphAdapters/LPortAdapter",1722),M(1723,1,Yr,DBe),g.ue=function(t,n){return cMn(h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Zo,"LGraphAdapters/PortComparator",1723),M(804,1,Cr,$ue),g.Mb=function(t){return h(t,10),Y7(),!0},D(Zo,"LGraphAdapters/lambda$0$Type",804),M(392,299,{3:1,299:1,392:1,94:1,134:1}),D(Zo,"LShape",392),M(70,392,{3:1,299:1,70:1,392:1,94:1,134:1},XXe,Afe),g.Ib=function(){var t;return t=$2n(this),t==null?"label":"l_"+t},D(Zo,"LLabel",70),M(207,1,{3:1,4:1,207:1,414:1}),g.Fb=function(t){var n;return ue(t,207)?(n=h(t,207),this.d==n.d&&this.a==n.a&&this.b==n.b&&this.c==n.c):!1},g.Hb=function(){var t,n;return t=J6(this.b)<<16,t|=J6(this.a)&Oi,n=J6(this.c)<<16,n|=J6(this.d)&Oi,t^n},g.Jf=function(t){var n,s,a,u,p,v,m,x,_,A,I;for(p=0;p<t.length&&yht((er(p,t.length),t.charCodeAt(p)),Rvt);)++p;for(n=t.length;n>0&&yht((er(n-1,t.length),t.charCodeAt(n-1)),jvt);)--n;if(p<n){A=Pw(t.substr(p,n-p),",|;");try{for(m=A,x=0,_=m.length;x<_;++x){if(v=m[x],u=Pw(v,"="),u.length!=2)throw ne(new an("Expecting a list of key-value pairs."));a=Mw(u[0]),I=Dw(Mw(u[1])),Nt(a,"top")?this.d=I:Nt(a,"left")?this.b=I:Nt(a,"bottom")?this.a=I:Nt(a,"right")&&(this.c=I)}}catch(P){throw P=ui(P),ue(P,127)?(s=P,ne(new an($vt+s))):ne(P)}}},g.Ib=function(){return"[top="+this.d+",left="+this.b+",bottom="+this.a+",right="+this.c+"]"},g.a=0,g.b=0,g.c=0,g.d=0,D(WM,"Spacing",207),M(142,207,Hvt,B7,XJe,k1e,XY);var K4e=D(WM,"ElkMargin",142);M(651,142,Hvt,Tq),D(Zo,"LMargin",651),M(10,392,{3:1,299:1,10:1,392:1,94:1,134:1},Nd),g.Ib=function(){return $ft(this)},g.i=!1;var X1=D(Zo,"LNode",10);M(267,22,{3:1,35:1,22:1,267:1},eE);var Bf,Pi,Ou,ds,Wi,Vo,Ote=tr(Zo,"LNode/NodeType",267,sr,T3n,Vgn),C5t;M(116,207,zvt,E4,nb,N1e);var X4e=D(WM,"ElkPadding",116);M(764,116,zvt,the),D(Zo,"LPadding",764),M(11,392,{3:1,299:1,11:1,392:1,94:1,134:1},So),g.Ib=function(){var t,n,s;return t=new rg,or((t.a+="p_",t),$F(this)),this.i&&or(ho((t.a+="[",t),this.i),"]"),this.e.c.length==1&&this.g.c.length==0&&h(ut(this.e,0),17).c!=this&&(n=h(ut(this.e,0),17).c,or((t.a+=" << ",t),$F(n)),or(ho((t.a+="[",t),n.i),"]")),this.e.c.length==0&&this.g.c.length==1&&h(ut(this.g,0),17).d!=this&&(s=h(ut(this.g,0),17).d,or((t.a+=" >> ",t),$F(s)),or(ho((t.a+="[",t),s.i),"]")),t.a},g.c=!0,g.d=!1;var Q4e,Z4e,J4e,e5e,t5e,n5e,S5t=D(Zo,"LPort",11);M(397,1,G1,j6),g.Jc=function(t){Os(this,t)},g.Kc=function(){var t;return t=new Q(this.a.e),new eYe(t)},D(Zo,"LPort/1",397),M(1290,1,ys,eYe),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return h(K(this.a),17).c},g.Ob=function(){return Wa(this.a)},g.Qb=function(){IE(this.a)},D(Zo,"LPort/1/1",1290),M(359,1,G1,y4),g.Jc=function(t){Os(this,t)},g.Kc=function(){var t;return t=new Q(this.a.g),new Ole(t)},D(Zo,"LPort/2",359),M(762,1,ys,Ole),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return h(K(this.a),17).d},g.Ob=function(){return Wa(this.a)},g.Qb=function(){IE(this.a)},D(Zo,"LPort/2/1",762),M(1283,1,G1,OZe),g.Jc=function(t){Os(this,t)},g.Kc=function(){return new _f(this)},D(Zo,"LPort/CombineIter",1283),M(201,1,ys,_f),g.Nb=function(t){Ls(this,t)},g.Qb=function(){rQe()},g.Ob=function(){return kE(this)},g.Pb=function(){return Wa(this.a)?K(this.a):K(this.b)},D(Zo,"LPort/CombineIter/1",201),M(1285,1,c1,LBe),g.Lb=function(t){return brt(t)},g.Fb=function(t){return this===t},g.Mb=function(t){return Rc(),h(t,11).e.c.length!=0},D(Zo,"LPort/lambda$0$Type",1285),M(1284,1,c1,IBe),g.Lb=function(t){return vrt(t)},g.Fb=function(t){return this===t},g.Mb=function(t){return Rc(),h(t,11).g.c.length!=0},D(Zo,"LPort/lambda$1$Type",1284),M(1286,1,c1,OBe),g.Lb=function(t){return Rc(),h(t,11).j==(Qe(),sn)},g.Fb=function(t){return this===t},g.Mb=function(t){return Rc(),h(t,11).j==(Qe(),sn)},D(Zo,"LPort/lambda$2$Type",1286),M(1287,1,c1,NBe),g.Lb=function(t){return Rc(),h(t,11).j==(Qe(),pn)},g.Fb=function(t){return this===t},g.Mb=function(t){return Rc(),h(t,11).j==(Qe(),pn)},D(Zo,"LPort/lambda$3$Type",1287),M(1288,1,c1,PBe),g.Lb=function(t){return Rc(),h(t,11).j==(Qe(),Fn)},g.Fb=function(t){return this===t},g.Mb=function(t){return Rc(),h(t,11).j==(Qe(),Fn)},D(Zo,"LPort/lambda$4$Type",1288),M(1289,1,c1,BBe),g.Lb=function(t){return Rc(),h(t,11).j==(Qe(),un)},g.Fb=function(t){return this===t},g.Mb=function(t){return Rc(),h(t,11).j==(Qe(),un)},D(Zo,"LPort/lambda$5$Type",1289),M(29,299,{3:1,20:1,299:1,29:1,94:1,134:1},Nl),g.Jc=function(t){Os(this,t)},g.Kc=function(){return new Q(this.a)},g.Ib=function(){return"L_"+Ha(this.b.b,this,0)+Ag(this.a)},D(Zo,"Layer",29),M(1342,1,{},UKe),D(C0,Gvt,1342),M(1346,1,{},FBe),g.Kb=function(t){return Na(h(t,82))},D(C0,"ElkGraphImporter/0methodref$connectableShapeToNode$Type",1346),M(1349,1,{},RBe),g.Kb=function(t){return Na(h(t,82))},D(C0,"ElkGraphImporter/1methodref$connectableShapeToNode$Type",1349),M(1343,1,mn,tYe),g.td=function(t){M0t(this.a,h(t,118))},D(C0,Vvt,1343),M(1344,1,mn,nYe),g.td=function(t){M0t(this.a,h(t,118))},D(C0,Uvt,1344),M(1345,1,{},jBe),g.Kb=function(t){return new Vt(null,new Wt(yvn(h(t,79)),16))},D(C0,qvt,1345),M(1347,1,Cr,rYe),g.Mb=function(t){return kdn(this.a,h(t,33))},D(C0,Yvt,1347),M(1348,1,{},$Be),g.Kb=function(t){return new Vt(null,new Wt(kvn(h(t,79)),16))},D(C0,"ElkGraphImporter/lambda$5$Type",1348),M(1350,1,Cr,iYe),g.Mb=function(t){return xdn(this.a,h(t,33))},D(C0,"ElkGraphImporter/lambda$7$Type",1350),M(1351,1,Cr,HBe),g.Mb=function(t){return Rvn(h(t,79))},D(C0,"ElkGraphImporter/lambda$8$Type",1351),M(1278,1,{},iUe);var A5t;D(C0,"ElkGraphLayoutTransferrer",1278),M(1279,1,Cr,sYe),g.Mb=function(t){return lgn(this.a,h(t,17))},D(C0,"ElkGraphLayoutTransferrer/lambda$0$Type",1279),M(1280,1,mn,aYe),g.td=function(t){ES(),Ue(this.a,h(t,17))},D(C0,"ElkGraphLayoutTransferrer/lambda$1$Type",1280),M(1281,1,Cr,oYe),g.Mb=function(t){return K0n(this.a,h(t,17))},D(C0,"ElkGraphLayoutTransferrer/lambda$2$Type",1281),M(1282,1,mn,cYe),g.td=function(t){ES(),Ue(this.a,h(t,17))},D(C0,"ElkGraphLayoutTransferrer/lambda$3$Type",1282),M(1485,1,xi,zBe),g.pf=function(t,n){Q4n(h(t,37),n)},D(fn,"CommentNodeMarginCalculator",1485),M(1486,1,{},GBe),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"CommentNodeMarginCalculator/lambda$0$Type",1486),M(1487,1,mn,VBe),g.td=function(t){LLn(h(t,10))},D(fn,"CommentNodeMarginCalculator/lambda$1$Type",1487),M(1488,1,xi,UBe),g.pf=function(t,n){VAn(h(t,37),n)},D(fn,"CommentPostprocessor",1488),M(1489,1,xi,qBe),g.pf=function(t,n){QOn(h(t,37),n)},D(fn,"CommentPreprocessor",1489),M(1490,1,xi,YBe),g.pf=function(t,n){dSn(h(t,37),n)},D(fn,"ConstraintsPostprocessor",1490),M(1491,1,xi,WBe),g.pf=function(t,n){y5n(h(t,37),n)},D(fn,"EdgeAndLayerConstraintEdgeReverser",1491),M(1492,1,xi,KBe),g.pf=function(t,n){cxn(h(t,37),n)},D(fn,"EndLabelPostprocessor",1492),M(1493,1,{},XBe),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"EndLabelPostprocessor/lambda$0$Type",1493),M(1494,1,Cr,QBe),g.Mb=function(t){return nwn(h(t,10))},D(fn,"EndLabelPostprocessor/lambda$1$Type",1494),M(1495,1,mn,ZBe),g.td=function(t){K7n(h(t,10))},D(fn,"EndLabelPostprocessor/lambda$2$Type",1495),M(1496,1,xi,JBe),g.pf=function(t,n){R_n(h(t,37),n)},D(fn,"EndLabelPreprocessor",1496),M(1497,1,{},eFe),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"EndLabelPreprocessor/lambda$0$Type",1497),M(1498,1,mn,ont),g.td=function(t){R1n(this.a,this.b,this.c,h(t,10))},g.a=0,g.b=0,g.c=!1,D(fn,"EndLabelPreprocessor/lambda$1$Type",1498),M(1499,1,Cr,tFe),g.Mb=function(t){return Me(U(h(t,70),(tt(),y1)))===Me((Tf(),m8))},D(fn,"EndLabelPreprocessor/lambda$2$Type",1499),M(1500,1,mn,uYe),g.td=function(t){vr(this.a,h(t,70))},D(fn,"EndLabelPreprocessor/lambda$3$Type",1500),M(1501,1,Cr,nFe),g.Mb=function(t){return Me(U(h(t,70),(tt(),y1)))===Me((Tf(),pm))},D(fn,"EndLabelPreprocessor/lambda$4$Type",1501),M(1502,1,mn,lYe),g.td=function(t){vr(this.a,h(t,70))},D(fn,"EndLabelPreprocessor/lambda$5$Type",1502),M(1551,1,xi,rUe),g.pf=function(t,n){bkn(h(t,37),n)};var M5t;D(fn,"EndLabelSorter",1551),M(1552,1,Yr,rFe),g.ue=function(t,n){return qxn(h(t,456),h(n,456))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"EndLabelSorter/1",1552),M(456,1,{456:1},Sit),D(fn,"EndLabelSorter/LabelGroup",456),M(1553,1,{},iFe),g.Kb=function(t){return _S(),new Vt(null,new Wt(h(t,29).a,16))},D(fn,"EndLabelSorter/lambda$0$Type",1553),M(1554,1,Cr,sFe),g.Mb=function(t){return _S(),h(t,10).k==(bn(),Wi)},D(fn,"EndLabelSorter/lambda$1$Type",1554),M(1555,1,mn,aFe),g.td=function(t){K9n(h(t,10))},D(fn,"EndLabelSorter/lambda$2$Type",1555),M(1556,1,Cr,oFe),g.Mb=function(t){return _S(),Me(U(h(t,70),(tt(),y1)))===Me((Tf(),pm))},D(fn,"EndLabelSorter/lambda$3$Type",1556),M(1557,1,Cr,cFe),g.Mb=function(t){return _S(),Me(U(h(t,70),(tt(),y1)))===Me((Tf(),m8))},D(fn,"EndLabelSorter/lambda$4$Type",1557),M(1503,1,xi,uFe),g.pf=function(t,n){zLn(this,h(t,37))},g.b=0,g.c=0,D(fn,"FinalSplineBendpointsCalculator",1503),M(1504,1,{},lFe),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"FinalSplineBendpointsCalculator/lambda$0$Type",1504),M(1505,1,{},hFe),g.Kb=function(t){return new Vt(null,new cb(new Sn(Dn(qi(h(t,10)).a.Kc(),new L))))},D(fn,"FinalSplineBendpointsCalculator/lambda$1$Type",1505),M(1506,1,Cr,fFe),g.Mb=function(t){return!Xs(h(t,17))},D(fn,"FinalSplineBendpointsCalculator/lambda$2$Type",1506),M(1507,1,Cr,dFe),g.Mb=function(t){return os(h(t,17),(He(),f2))},D(fn,"FinalSplineBendpointsCalculator/lambda$3$Type",1507),M(1508,1,mn,hYe),g.td=function(t){tDn(this.a,h(t,128))},D(fn,"FinalSplineBendpointsCalculator/lambda$4$Type",1508),M(1509,1,mn,gFe),g.td=function(t){pQ(h(t,17).a)},D(fn,"FinalSplineBendpointsCalculator/lambda$5$Type",1509),M(792,1,xi,Nle),g.pf=function(t,n){PIn(this,h(t,37),n)},D(fn,"GraphTransformer",792),M(511,22,{3:1,35:1,22:1,511:1},Yhe);var Nte,kD,D5t=tr(fn,"GraphTransformer/Mode",511,sr,_wn,n2n),L5t;M(1510,1,xi,pFe),g.pf=function(t,n){hAn(h(t,37),n)},D(fn,"HierarchicalNodeResizingProcessor",1510),M(1511,1,xi,bFe),g.pf=function(t,n){q4n(h(t,37),n)},D(fn,"HierarchicalPortConstraintProcessor",1511),M(1512,1,Yr,vFe),g.ue=function(t,n){return r8n(h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"HierarchicalPortConstraintProcessor/NodeComparator",1512),M(1513,1,xi,wFe),g.pf=function(t,n){fLn(h(t,37),n)},D(fn,"HierarchicalPortDummySizeProcessor",1513),M(1514,1,xi,mFe),g.pf=function(t,n){uMn(this,h(t,37),n)},g.a=0,D(fn,"HierarchicalPortOrthogonalEdgeRouter",1514),M(1515,1,Yr,yFe),g.ue=function(t,n){return g0n(h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"HierarchicalPortOrthogonalEdgeRouter/1",1515),M(1516,1,Yr,kFe),g.ue=function(t,n){return A3n(h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"HierarchicalPortOrthogonalEdgeRouter/2",1516),M(1517,1,xi,xFe),g.pf=function(t,n){N9n(h(t,37),n)},D(fn,"HierarchicalPortPositionProcessor",1517),M(1518,1,xi,sUe),g.pf=function(t,n){MNn(this,h(t,37))},g.a=0,g.c=0;var t$,n$;D(fn,"HighDegreeNodeLayeringProcessor",1518),M(571,1,{571:1},EFe),g.b=-1,g.d=-1,D(fn,"HighDegreeNodeLayeringProcessor/HighDegreeNodeInformation",571),M(1519,1,{},_Fe),g.Kb=function(t){return XS(),za(h(t,10))},g.Fb=function(t){return this===t},D(fn,"HighDegreeNodeLayeringProcessor/lambda$0$Type",1519),M(1520,1,{},TFe),g.Kb=function(t){return XS(),qi(h(t,10))},g.Fb=function(t){return this===t},D(fn,"HighDegreeNodeLayeringProcessor/lambda$1$Type",1520),M(1526,1,xi,CFe),g.pf=function(t,n){$Dn(this,h(t,37),n)},D(fn,"HyperedgeDummyMerger",1526),M(793,1,{},F1e),g.a=!1,g.b=!1,g.c=!1,D(fn,"HyperedgeDummyMerger/MergeState",793),M(1527,1,{},SFe),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"HyperedgeDummyMerger/lambda$0$Type",1527),M(1528,1,{},AFe),g.Kb=function(t){return new Vt(null,new Wt(h(t,10).j,16))},D(fn,"HyperedgeDummyMerger/lambda$1$Type",1528),M(1529,1,mn,MFe),g.td=function(t){h(t,11).p=-1},D(fn,"HyperedgeDummyMerger/lambda$2$Type",1529),M(1530,1,xi,DFe),g.pf=function(t,n){RDn(h(t,37),n)},D(fn,"HypernodesProcessor",1530),M(1531,1,xi,LFe),g.pf=function(t,n){jDn(h(t,37),n)},D(fn,"InLayerConstraintProcessor",1531),M(1532,1,xi,IFe),g.pf=function(t,n){p5n(h(t,37),n)},D(fn,"InnermostNodeMarginCalculator",1532),M(1533,1,xi,OFe),g.pf=function(t,n){qOn(this,h(t,37))},g.a=Fi,g.b=Fi,g.c=ki,g.d=ki;var MPn=D(fn,"InteractiveExternalPortPositioner",1533);M(1534,1,{},NFe),g.Kb=function(t){return h(t,17).d.i},g.Fb=function(t){return this===t},D(fn,"InteractiveExternalPortPositioner/lambda$0$Type",1534),M(1535,1,{},fYe),g.Kb=function(t){return v0n(this.a,Je(t))},g.Fb=function(t){return this===t},D(fn,"InteractiveExternalPortPositioner/lambda$1$Type",1535),M(1536,1,{},PFe),g.Kb=function(t){return h(t,17).c.i},g.Fb=function(t){return this===t},D(fn,"InteractiveExternalPortPositioner/lambda$2$Type",1536),M(1537,1,{},dYe),g.Kb=function(t){return w0n(this.a,Je(t))},g.Fb=function(t){return this===t},D(fn,"InteractiveExternalPortPositioner/lambda$3$Type",1537),M(1538,1,{},gYe),g.Kb=function(t){return ogn(this.a,Je(t))},g.Fb=function(t){return this===t},D(fn,"InteractiveExternalPortPositioner/lambda$4$Type",1538),M(1539,1,{},pYe),g.Kb=function(t){return cgn(this.a,Je(t))},g.Fb=function(t){return this===t},D(fn,"InteractiveExternalPortPositioner/lambda$5$Type",1539),M(77,22,{3:1,35:1,22:1,77:1,234:1},Ii),g.Kf=function(){switch(this.g){case 15:return new n$e;case 22:return new r$e;case 47:return new a$e;case 28:case 35:return new qFe;case 32:return new zBe;case 42:return new UBe;case 1:return new qBe;case 41:return new YBe;case 56:return new Nle((Nk(),kD));case 0:return new Nle((Nk(),Nte));case 2:return new WBe;case 54:return new KBe;case 33:return new JBe;case 51:return new uFe;case 55:return new pFe;case 13:return new bFe;case 38:return new wFe;case 44:return new mFe;case 40:return new xFe;case 9:return new sUe;case 49:return new Yet;case 37:return new CFe;case 43:return new DFe;case 27:return new LFe;case 30:return new IFe;case 3:return new OFe;case 18:return new FFe;case 29:return new RFe;case 5:return new aUe;case 50:return new BFe;case 34:return new oUe;case 36:return new YFe;case 52:return new rUe;case 11:return new KFe;case 7:return new uUe;case 39:return new XFe;case 45:return new QFe;case 16:return new ZFe;case 10:return new JFe;case 48:return new tRe;case 21:return new nRe;case 23:return new Pq((_b(),R_));case 8:return new iRe;case 12:return new aRe;case 4:return new oRe;case 19:return new dUe;case 17:return new vRe;case 53:return new wRe;case 6:return new DRe;case 25:return new KKe;case 46:return new ERe;case 31:return new Ltt;case 14:return new RRe;case 26:return new u$e;case 20:return new GRe;case 24:return new Pq((_b(),aH));default:throw ne(new an(pJ+(this.f!=null?this.f:""+this.g)))}};var r5e,i5e,s5e,a5e,o5e,c5e,u5e,l5e,h5e,f5e,v_,r$,i$,d5e,g5e,p5e,b5e,v5e,w5e,m5e,w_,y5e,k5e,x5e,E5e,_5e,Pte,s$,a$,T5e,o$,c$,u$,Yx,Wx,Kx,C5e,l$,h$,S5e,f$,d$,A5e,M5e,D5e,L5e,g$,Bte,xD,p$,b$,v$,w$,I5e,O5e,N5e,P5e,DPn=tr(fn,Fwe,77,sr,kgt,t2n),I5t;M(1540,1,xi,FFe),g.pf=function(t,n){WOn(h(t,37),n)},D(fn,"InvertedPortProcessor",1540),M(1541,1,xi,RFe),g.pf=function(t,n){WMn(h(t,37),n)},D(fn,"LabelAndNodeSizeProcessor",1541),M(1542,1,Cr,jFe),g.Mb=function(t){return h(t,10).k==(bn(),Wi)},D(fn,"LabelAndNodeSizeProcessor/lambda$0$Type",1542),M(1543,1,Cr,$Fe),g.Mb=function(t){return h(t,10).k==(bn(),Pi)},D(fn,"LabelAndNodeSizeProcessor/lambda$1$Type",1543),M(1544,1,mn,cnt),g.td=function(t){j1n(this.b,this.a,this.c,h(t,10))},g.a=!1,g.c=!1,D(fn,"LabelAndNodeSizeProcessor/lambda$2$Type",1544),M(1545,1,xi,aUe),g.pf=function(t,n){wOn(h(t,37),n)};var O5t;D(fn,"LabelDummyInserter",1545),M(1546,1,c1,HFe),g.Lb=function(t){return Me(U(h(t,70),(tt(),y1)))===Me((Tf(),w8))},g.Fb=function(t){return this===t},g.Mb=function(t){return Me(U(h(t,70),(tt(),y1)))===Me((Tf(),w8))},D(fn,"LabelDummyInserter/1",1546),M(1547,1,xi,BFe),g.pf=function(t,n){yIn(h(t,37),n)},D(fn,"LabelDummyRemover",1547),M(1548,1,Cr,zFe),g.Mb=function(t){return ft(lt(U(h(t,70),(tt(),_ne))))},D(fn,"LabelDummyRemover/lambda$0$Type",1548),M(1359,1,xi,oUe),g.pf=function(t,n){KIn(this,h(t,37),n)},g.a=null;var Fte;D(fn,"LabelDummySwitcher",1359),M(286,1,{286:1},mpt),g.c=0,g.d=null,g.f=0,D(fn,"LabelDummySwitcher/LabelDummyInfo",286),M(1360,1,{},GFe),g.Kb=function(t){return t5(),new Vt(null,new Wt(h(t,29).a,16))},D(fn,"LabelDummySwitcher/lambda$0$Type",1360),M(1361,1,Cr,VFe),g.Mb=function(t){return t5(),h(t,10).k==(bn(),Ou)},D(fn,"LabelDummySwitcher/lambda$1$Type",1361),M(1362,1,{},wYe),g.Kb=function(t){return X0n(this.a,h(t,10))},D(fn,"LabelDummySwitcher/lambda$2$Type",1362),M(1363,1,mn,mYe),g.td=function(t){Kbn(this.a,h(t,286))},D(fn,"LabelDummySwitcher/lambda$3$Type",1363),M(1364,1,Yr,UFe),g.ue=function(t,n){return Ebn(h(t,286),h(n,286))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"LabelDummySwitcher/lambda$4$Type",1364),M(791,1,xi,qFe),g.pf=function(t,n){a3n(h(t,37),n)},D(fn,"LabelManagementProcessor",791),M(1549,1,xi,YFe),g.pf=function(t,n){LAn(h(t,37),n)},D(fn,"LabelSideSelector",1549),M(1550,1,Cr,WFe),g.Mb=function(t){return ft(lt(U(h(t,70),(tt(),_ne))))},D(fn,"LabelSideSelector/lambda$0$Type",1550),M(1558,1,xi,KFe),g.pf=function(t,n){dLn(h(t,37),n)},D(fn,"LayerConstraintPostprocessor",1558),M(1559,1,xi,uUe),g.pf=function(t,n){SCn(h(t,37),n)};var B5e;D(fn,"LayerConstraintPreprocessor",1559),M(360,22,{3:1,35:1,22:1,360:1},BN);var ED,m$,y$,Rte,N5t=tr(fn,"LayerConstraintPreprocessor/HiddenNodeConnections",360,sr,Jmn,qgn),P5t;M(1560,1,xi,XFe),g.pf=function(t,n){vIn(h(t,37),n)},D(fn,"LayerSizeAndGraphHeightCalculator",1560),M(1561,1,xi,QFe),g.pf=function(t,n){ySn(h(t,37),n)},D(fn,"LongEdgeJoiner",1561),M(1562,1,xi,ZFe),g.pf=function(t,n){ZLn(h(t,37),n)},D(fn,"LongEdgeSplitter",1562),M(1563,1,xi,JFe),g.pf=function(t,n){ZIn(this,h(t,37),n)},g.d=0,g.e=0,g.i=0,g.j=0,g.k=0,g.n=0,D(fn,"NodePromotion",1563),M(1564,1,{},eRe),g.Kb=function(t){return h(t,46),on(),!0},g.Fb=function(t){return this===t},D(fn,"NodePromotion/lambda$0$Type",1564),M(1565,1,{},bYe),g.Kb=function(t){return bvn(this.a,h(t,46))},g.Fb=function(t){return this===t},g.a=0,D(fn,"NodePromotion/lambda$1$Type",1565),M(1566,1,{},vYe),g.Kb=function(t){return vvn(this.a,h(t,46))},g.Fb=function(t){return this===t},g.a=0,D(fn,"NodePromotion/lambda$2$Type",1566),M(1567,1,xi,tRe),g.pf=function(t,n){xNn(h(t,37),n)},D(fn,"NorthSouthPortPostprocessor",1567),M(1568,1,xi,nRe),g.pf=function(t,n){oNn(h(t,37),n)},D(fn,"NorthSouthPortPreprocessor",1568),M(1569,1,Yr,rRe),g.ue=function(t,n){return B5n(h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"NorthSouthPortPreprocessor/lambda$0$Type",1569),M(1570,1,xi,iRe),g.pf=function(t,n){_Dn(h(t,37),n)},D(fn,"PartitionMidprocessor",1570),M(1571,1,Cr,sRe),g.Mb=function(t){return os(h(t,10),(tt(),c8))},D(fn,"PartitionMidprocessor/lambda$0$Type",1571),M(1572,1,mn,yYe),g.td=function(t){jvn(this.a,h(t,10))},D(fn,"PartitionMidprocessor/lambda$1$Type",1572),M(1573,1,xi,aRe),g.pf=function(t,n){$Sn(h(t,37),n)},D(fn,"PartitionPostprocessor",1573),M(1574,1,xi,oRe),g.pf=function(t,n){XTn(h(t,37),n)},D(fn,"PartitionPreprocessor",1574),M(1575,1,Cr,cRe),g.Mb=function(t){return os(h(t,10),(tt(),c8))},D(fn,"PartitionPreprocessor/lambda$0$Type",1575),M(1576,1,{},uRe),g.Kb=function(t){return new Vt(null,new cb(new Sn(Dn(qi(h(t,10)).a.Kc(),new L))))},D(fn,"PartitionPreprocessor/lambda$1$Type",1576),M(1577,1,Cr,lRe),g.Mb=function(t){return Gxn(h(t,17))},D(fn,"PartitionPreprocessor/lambda$2$Type",1577),M(1578,1,mn,hRe),g.td=function(t){q5n(h(t,17))},D(fn,"PartitionPreprocessor/lambda$3$Type",1578),M(1579,1,xi,dUe),g.pf=function(t,n){cDn(h(t,37),n)};var F5e,B5t,F5t,R5t,R5e,j5e;D(fn,"PortListSorter",1579),M(1580,1,{},fRe),g.Kb=function(t){return Uk(),h(t,11).e},D(fn,"PortListSorter/lambda$0$Type",1580),M(1581,1,{},dRe),g.Kb=function(t){return Uk(),h(t,11).g},D(fn,"PortListSorter/lambda$1$Type",1581),M(1582,1,Yr,gRe),g.ue=function(t,n){return Oat(h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"PortListSorter/lambda$2$Type",1582),M(1583,1,Yr,pRe),g.ue=function(t,n){return d7n(h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"PortListSorter/lambda$3$Type",1583),M(1584,1,Yr,bRe),g.ue=function(t,n){return zpt(h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"PortListSorter/lambda$4$Type",1584),M(1585,1,xi,vRe),g.pf=function(t,n){kCn(h(t,37),n)},D(fn,"PortSideProcessor",1585),M(1586,1,xi,wRe),g.pf=function(t,n){xMn(h(t,37),n)},D(fn,"ReversedEdgeRestorer",1586),M(1591,1,xi,KKe),g.pf=function(t,n){Q8n(this,h(t,37),n)},D(fn,"SelfLoopPortRestorer",1591),M(1592,1,{},mRe),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"SelfLoopPortRestorer/lambda$0$Type",1592),M(1593,1,Cr,yRe),g.Mb=function(t){return h(t,10).k==(bn(),Wi)},D(fn,"SelfLoopPortRestorer/lambda$1$Type",1593),M(1594,1,Cr,kRe),g.Mb=function(t){return os(h(t,10),(He(),s8))},D(fn,"SelfLoopPortRestorer/lambda$2$Type",1594),M(1595,1,{},xRe),g.Kb=function(t){return h(U(h(t,10),(He(),s8)),403)},D(fn,"SelfLoopPortRestorer/lambda$3$Type",1595),M(1596,1,mn,kYe),g.td=function(t){a_n(this.a,h(t,403))},D(fn,"SelfLoopPortRestorer/lambda$4$Type",1596),M(794,1,mn,Hue),g.td=function(t){k_n(h(t,101))},D(fn,"SelfLoopPortRestorer/lambda$5$Type",794),M(1597,1,xi,ERe),g.pf=function(t,n){a8n(h(t,37),n)},D(fn,"SelfLoopPostProcessor",1597),M(1598,1,{},_Re),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"SelfLoopPostProcessor/lambda$0$Type",1598),M(1599,1,Cr,TRe),g.Mb=function(t){return h(t,10).k==(bn(),Wi)},D(fn,"SelfLoopPostProcessor/lambda$1$Type",1599),M(1600,1,Cr,CRe),g.Mb=function(t){return os(h(t,10),(He(),s8))},D(fn,"SelfLoopPostProcessor/lambda$2$Type",1600),M(1601,1,mn,SRe),g.td=function(t){dEn(h(t,10))},D(fn,"SelfLoopPostProcessor/lambda$3$Type",1601),M(1602,1,{},ARe),g.Kb=function(t){return new Vt(null,new Wt(h(t,101).f,1))},D(fn,"SelfLoopPostProcessor/lambda$4$Type",1602),M(1603,1,mn,xYe),g.td=function(t){nyn(this.a,h(t,409))},D(fn,"SelfLoopPostProcessor/lambda$5$Type",1603),M(1604,1,Cr,MRe),g.Mb=function(t){return!!h(t,101).i},D(fn,"SelfLoopPostProcessor/lambda$6$Type",1604),M(1605,1,mn,EYe),g.td=function(t){$fn(this.a,h(t,101))},D(fn,"SelfLoopPostProcessor/lambda$7$Type",1605),M(1587,1,xi,DRe),g.pf=function(t,n){JCn(h(t,37),n)},D(fn,"SelfLoopPreProcessor",1587),M(1588,1,{},LRe),g.Kb=function(t){return new Vt(null,new Wt(h(t,101).f,1))},D(fn,"SelfLoopPreProcessor/lambda$0$Type",1588),M(1589,1,{},IRe),g.Kb=function(t){return h(t,409).a},D(fn,"SelfLoopPreProcessor/lambda$1$Type",1589),M(1590,1,mn,ORe),g.td=function(t){Udn(h(t,17))},D(fn,"SelfLoopPreProcessor/lambda$2$Type",1590),M(1606,1,xi,Ltt),g.pf=function(t,n){X9n(this,h(t,37),n)},D(fn,"SelfLoopRouter",1606),M(1607,1,{},NRe),g.Kb=function(t){return new Vt(null,new Wt(h(t,29).a,16))},D(fn,"SelfLoopRouter/lambda$0$Type",1607),M(1608,1,Cr,PRe),g.Mb=function(t){return h(t,10).k==(bn(),Wi)},D(fn,"SelfLoopRouter/lambda$1$Type",1608),M(1609,1,Cr,BRe),g.Mb=function(t){return os(h(t,10),(He(),s8))},D(fn,"SelfLoopRouter/lambda$2$Type",1609),M(1610,1,{},FRe),g.Kb=function(t){return h(U(h(t,10),(He(),s8)),403)},D(fn,"SelfLoopRouter/lambda$3$Type",1610),M(1611,1,mn,TZe),g.td=function(t){Dvn(this.a,this.b,h(t,403))},D(fn,"SelfLoopRouter/lambda$4$Type",1611),M(1612,1,xi,RRe),g.pf=function(t,n){mAn(h(t,37),n)},D(fn,"SemiInteractiveCrossMinProcessor",1612),M(1613,1,Cr,jRe),g.Mb=function(t){return h(t,10).k==(bn(),Wi)},D(fn,"SemiInteractiveCrossMinProcessor/lambda$0$Type",1613),M(1614,1,Cr,$Re),g.Mb=function(t){return Ynt(h(t,10))._b((tt(),um))},D(fn,"SemiInteractiveCrossMinProcessor/lambda$1$Type",1614),M(1615,1,Yr,HRe),g.ue=function(t,n){return W4n(h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fn,"SemiInteractiveCrossMinProcessor/lambda$2$Type",1615),M(1616,1,{},zRe),g.Ce=function(t,n){return Yvn(h(t,10),h(n,10))},D(fn,"SemiInteractiveCrossMinProcessor/lambda$3$Type",1616),M(1618,1,xi,GRe),g.pf=function(t,n){gLn(h(t,37),n)},D(fn,"SortByInputModelProcessor",1618),M(1619,1,Cr,VRe),g.Mb=function(t){return h(t,11).g.c.length!=0},D(fn,"SortByInputModelProcessor/lambda$0$Type",1619),M(1620,1,mn,_Ye),g.td=function(t){C_n(this.a,h(t,11))},D(fn,"SortByInputModelProcessor/lambda$1$Type",1620),M(1693,803,{},zut),g.Me=function(t){var n,s,a,u;switch(this.c=t,this.a.g){case 2:n=new qe,Ti(Jr(new Vt(null,new Wt(this.c.a.b,16)),new ije),new DZe(this,n)),fM(this,new YRe),mc(n,new WRe),n.c=pe(yn,st,1,0,5,1),Ti(Jr(new Vt(null,new Wt(this.c.a.b,16)),new KRe),new CYe(n)),fM(this,new XRe),mc(n,new QRe),n.c=pe(yn,st,1,0,5,1),s=uet(tlt(qP(new Vt(null,new Wt(this.c.a.b,16)),new SYe(this))),new ZRe),Ti(new Vt(null,new Wt(this.c.a.a,16)),new SZe(s,n)),fM(this,new eje),mc(n,new URe),n.c=pe(yn,st,1,0,5,1);break;case 3:a=new qe,fM(this,new qRe),u=uet(tlt(qP(new Vt(null,new Wt(this.c.a.b,16)),new TYe(this))),new JRe),Ti(Jr(new Vt(null,new Wt(this.c.a.b,16)),new tje),new MZe(u,a)),fM(this,new nje),mc(a,new rje),a.c=pe(yn,st,1,0,5,1);break;default:throw ne(new NKe)}},g.b=0,D(Ri,"EdgeAwareScanlineConstraintCalculation",1693),M(1694,1,c1,qRe),g.Lb=function(t){return ue(h(t,57).g,145)},g.Fb=function(t){return this===t},g.Mb=function(t){return ue(h(t,57).g,145)},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$0$Type",1694),M(1695,1,{},TYe),g.Fe=function(t){return sTn(this.a,h(t,57))},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$1$Type",1695),M(1703,1,CR,CZe),g.Vd=function(){g9(this.a,this.b,-1)},g.b=0,D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$10$Type",1703),M(1705,1,c1,YRe),g.Lb=function(t){return ue(h(t,57).g,145)},g.Fb=function(t){return this===t},g.Mb=function(t){return ue(h(t,57).g,145)},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$11$Type",1705),M(1706,1,mn,WRe),g.td=function(t){h(t,365).Vd()},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$12$Type",1706),M(1707,1,Cr,KRe),g.Mb=function(t){return ue(h(t,57).g,10)},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$13$Type",1707),M(1709,1,mn,CYe),g.td=function(t){Ikn(this.a,h(t,57))},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$14$Type",1709),M(1708,1,CR,NZe),g.Vd=function(){g9(this.b,this.a,-1)},g.a=0,D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$15$Type",1708),M(1710,1,c1,XRe),g.Lb=function(t){return ue(h(t,57).g,10)},g.Fb=function(t){return this===t},g.Mb=function(t){return ue(h(t,57).g,10)},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$16$Type",1710),M(1711,1,mn,QRe),g.td=function(t){h(t,365).Vd()},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$17$Type",1711),M(1712,1,{},SYe),g.Fe=function(t){return aTn(this.a,h(t,57))},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$18$Type",1712),M(1713,1,{},ZRe),g.De=function(){return 0},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$19$Type",1713),M(1696,1,{},JRe),g.De=function(){return 0},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$2$Type",1696),M(1715,1,mn,SZe),g.td=function(t){dbn(this.a,this.b,h(t,307))},g.a=0,D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$20$Type",1715),M(1714,1,CR,AZe),g.Vd=function(){Z0t(this.a,this.b,-1)},g.b=0,D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$21$Type",1714),M(1716,1,c1,eje),g.Lb=function(t){return h(t,57),!0},g.Fb=function(t){return this===t},g.Mb=function(t){return h(t,57),!0},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$22$Type",1716),M(1717,1,mn,URe),g.td=function(t){h(t,365).Vd()},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$23$Type",1717),M(1697,1,Cr,tje),g.Mb=function(t){return ue(h(t,57).g,10)},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$3$Type",1697),M(1699,1,mn,MZe),g.td=function(t){gbn(this.a,this.b,h(t,57))},g.a=0,D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$4$Type",1699),M(1698,1,CR,PZe),g.Vd=function(){g9(this.b,this.a,-1)},g.a=0,D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$5$Type",1698),M(1700,1,c1,nje),g.Lb=function(t){return h(t,57),!0},g.Fb=function(t){return this===t},g.Mb=function(t){return h(t,57),!0},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$6$Type",1700),M(1701,1,mn,rje),g.td=function(t){h(t,365).Vd()},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$7$Type",1701),M(1702,1,Cr,ije),g.Mb=function(t){return ue(h(t,57).g,145)},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$8$Type",1702),M(1704,1,mn,DZe),g.td=function(t){h4n(this.a,this.b,h(t,57))},D(Ri,"EdgeAwareScanlineConstraintCalculation/lambda$9$Type",1704),M(1521,1,xi,Yet),g.pf=function(t,n){sIn(this,h(t,37),n)};var j5t;D(Ri,"HorizontalGraphCompactor",1521),M(1522,1,{},AYe),g.Oe=function(t,n){var s,a,u;return ege(t,n)||(s=by(t),a=by(n),!!s&&s.k==(bn(),Pi)||!!a&&a.k==(bn(),Pi))?0:(u=h(U(this.a.a,(He(),d3)),304),m0n(u,s?s.k:(bn(),ds),a?a.k:(bn(),ds)))},g.Pe=function(t,n){var s,a,u;return ege(t,n)?1:(s=by(t),a=by(n),u=h(U(this.a.a,(He(),d3)),304),Kfe(u,s?s.k:(bn(),ds),a?a.k:(bn(),ds)))},D(Ri,"HorizontalGraphCompactor/1",1522),M(1523,1,{},sje),g.Ne=function(t,n){return W7(),t.a.i==0},D(Ri,"HorizontalGraphCompactor/lambda$0$Type",1523),M(1524,1,{},MYe),g.Ne=function(t,n){return Gvn(this.a,t,n)},D(Ri,"HorizontalGraphCompactor/lambda$1$Type",1524),M(1664,1,{},Sct);var $5t,H5t;D(Ri,"LGraphToCGraphTransformer",1664),M(1672,1,Cr,aje),g.Mb=function(t){return t!=null},D(Ri,"LGraphToCGraphTransformer/0methodref$nonNull$Type",1672),M(1665,1,{},oje),g.Kb=function(t){return wh(),Ga(U(h(h(t,57).g,10),(He(),Br)))},D(Ri,"LGraphToCGraphTransformer/lambda$0$Type",1665),M(1666,1,{},cje),g.Kb=function(t){return wh(),Dht(h(h(t,57).g,145))},D(Ri,"LGraphToCGraphTransformer/lambda$1$Type",1666),M(1675,1,Cr,uje),g.Mb=function(t){return wh(),ue(h(t,57).g,10)},D(Ri,"LGraphToCGraphTransformer/lambda$10$Type",1675),M(1676,1,mn,lje),g.td=function(t){zvn(h(t,57))},D(Ri,"LGraphToCGraphTransformer/lambda$11$Type",1676),M(1677,1,Cr,hje),g.Mb=function(t){return wh(),ue(h(t,57).g,145)},D(Ri,"LGraphToCGraphTransformer/lambda$12$Type",1677),M(1681,1,mn,fje),g.td=function(t){Y6n(h(t,57))},D(Ri,"LGraphToCGraphTransformer/lambda$13$Type",1681),M(1678,1,mn,DYe),g.td=function(t){vdn(this.a,h(t,8))},g.a=0,D(Ri,"LGraphToCGraphTransformer/lambda$14$Type",1678),M(1679,1,mn,LYe),g.td=function(t){mdn(this.a,h(t,110))},g.a=0,D(Ri,"LGraphToCGraphTransformer/lambda$15$Type",1679),M(1680,1,mn,IYe),g.td=function(t){wdn(this.a,h(t,8))},g.a=0,D(Ri,"LGraphToCGraphTransformer/lambda$16$Type",1680),M(1682,1,{},dje),g.Kb=function(t){return wh(),new Vt(null,new cb(new Sn(Dn(qi(h(t,10)).a.Kc(),new L))))},D(Ri,"LGraphToCGraphTransformer/lambda$17$Type",1682),M(1683,1,Cr,gje),g.Mb=function(t){return wh(),Xs(h(t,17))},D(Ri,"LGraphToCGraphTransformer/lambda$18$Type",1683),M(1684,1,mn,OYe),g.td=function(t){J3n(this.a,h(t,17))},D(Ri,"LGraphToCGraphTransformer/lambda$19$Type",1684),M(1668,1,mn,NYe),g.td=function(t){Omn(this.a,h(t,145))},D(Ri,"LGraphToCGraphTransformer/lambda$2$Type",1668),M(1685,1,{},pje),g.Kb=function(t){return wh(),new Vt(null,new Wt(h(t,29).a,16))},D(Ri,"LGraphToCGraphTransformer/lambda$20$Type",1685),M(1686,1,{},bje),g.Kb=function(t){return wh(),new Vt(null,new cb(new Sn(Dn(qi(h(t,10)).a.Kc(),new L))))},D(Ri,"LGraphToCGraphTransformer/lambda$21$Type",1686),M(1687,1,{},vje),g.Kb=function(t){return wh(),h(U(h(t,17),(He(),f2)),15)},D(Ri,"LGraphToCGraphTransformer/lambda$22$Type",1687),M(1688,1,Cr,wje),g.Mb=function(t){return k0n(h(t,15))},D(Ri,"LGraphToCGraphTransformer/lambda$23$Type",1688),M(1689,1,mn,PYe),g.td=function(t){X_n(this.a,h(t,15))},D(Ri,"LGraphToCGraphTransformer/lambda$24$Type",1689),M(1667,1,mn,LZe),g.td=function(t){yyn(this.a,this.b,h(t,145))},D(Ri,"LGraphToCGraphTransformer/lambda$3$Type",1667),M(1669,1,{},mje),g.Kb=function(t){return wh(),new Vt(null,new Wt(h(t,29).a,16))},D(Ri,"LGraphToCGraphTransformer/lambda$4$Type",1669),M(1670,1,{},yje),g.Kb=function(t){return wh(),new Vt(null,new cb(new Sn(Dn(qi(h(t,10)).a.Kc(),new L))))},D(Ri,"LGraphToCGraphTransformer/lambda$5$Type",1670),M(1671,1,{},kje),g.Kb=function(t){return wh(),h(U(h(t,17),(He(),f2)),15)},D(Ri,"LGraphToCGraphTransformer/lambda$6$Type",1671),M(1673,1,mn,BYe),g.td=function(t){ATn(this.a,h(t,15))},D(Ri,"LGraphToCGraphTransformer/lambda$8$Type",1673),M(1674,1,mn,IZe),g.td=function(t){Hdn(this.a,this.b,h(t,145))},D(Ri,"LGraphToCGraphTransformer/lambda$9$Type",1674),M(1663,1,{},xje),g.Le=function(t){var n,s,a,u,p;for(this.a=t,this.d=new _q,this.c=pe(m4e,st,121,this.a.a.a.c.length,0,1),this.b=0,s=new Q(this.a.a.a);s.a<s.c.c.length;)n=h(K(s),307),n.d=this.b,p=ib(xS(new _p,n),this.d),this.c[this.b]=p,++this.b;for(hOn(this),fNn(this),DSn(this),$Q(QY(this.d),new H6),u=new Q(this.a.a.b);u.a<u.c.c.length;)a=h(K(u),57),a.d.c=this.c[a.a.d].e+a.b.a},g.b=0,D(Ri,"NetworkSimplexCompaction",1663),M(145,1,{35:1,145:1},S9),g.wd=function(t){return r4n(this,h(t,145))},g.Ib=function(){return Dht(this)},D(Ri,"VerticalSegment",145),M(827,1,{},j2e),g.c=0,g.e=0,g.i=0,D(V9,"BetweenLayerEdgeTwoNodeCrossingsCounter",827),M(663,1,{663:1},ilt),g.Ib=function(){return"AdjacencyList [node="+this.d+", adjacencies= "+this.a+"]"},g.b=0,g.c=0,g.f=0,D(V9,"BetweenLayerEdgeTwoNodeCrossingsCounter/AdjacencyList",663),M(287,1,{35:1,287:1},ttt),g.wd=function(t){return V2n(this,h(t,287))},g.Ib=function(){return"Adjacency [position="+this.c+", cardinality="+this.a+", currentCardinality="+this.b+"]"},g.a=0,g.b=0,g.c=0,D(V9,"BetweenLayerEdgeTwoNodeCrossingsCounter/AdjacencyList/Adjacency",287),M(1929,1,{},Cdt),g.b=0,g.e=!1,D(V9,"CrossingMatrixFiller",1929);var z5t=fi(W1,"IInitializable");M(1804,1,KM,jZe),g.Nf=function(t,n,s,a,u,p){},g.Pf=function(t,n,s){},g.Lf=function(){return this.c!=(_b(),R_)},g.Mf=function(){this.e=pe(Hn,hr,25,this.d,15,1)},g.Of=function(t,n){n[t][0].c.p=t},g.Qf=function(t,n,s,a){++this.d},g.Rf=function(){return!0},g.Sf=function(t,n,s,a){return Eht(this,t,n,s),Iyn(this,n)},g.Tf=function(t,n){var s;return s=p1n(n,t.length),Eht(this,t,s,n),Glt(this,s)},g.d=0,D(V9,"GreedySwitchHeuristic",1804),M(1930,1,{},ert),g.b=0,g.d=0,D(V9,"NorthSouthEdgeNeighbouringNodeCrossingsCounter",1930),M(1917,1,{},rpt),g.a=!1,D(V9,"SwitchDecider",1917),M(101,1,{101:1},Ndt),g.a=null,g.c=null,g.i=null,D(T5,"SelfHyperLoop",101),M(1916,1,{},Oft),g.c=0,g.e=0,D(T5,"SelfHyperLoopLabels",1916),M(411,22,{3:1,35:1,22:1,411:1},FN);var L5,Xx,Qx,jte,G5t=tr(T5,"SelfHyperLoopLabels/Alignment",411,sr,eyn,Ygn),V5t;M(409,1,{409:1},wot),D(T5,"SelfLoopEdge",409),M(403,1,{403:1},Tft),g.a=!1,D(T5,"SelfLoopHolder",403),M(1724,1,Cr,Ije),g.Mb=function(t){return Xs(h(t,17))},D(T5,"SelfLoopHolder/lambda$0$Type",1724),M(113,1,{113:1},Nft),g.a=!1,g.c=!1,D(T5,"SelfLoopPort",113),M(1792,1,Cr,Oje),g.Mb=function(t){return Xs(h(t,17))},D(T5,"SelfLoopPort/lambda$0$Type",1792),M(363,22,{3:1,35:1,22:1,363:1},AS);var k$,x$,E$,_$,T$,U5t=tr(T5,"SelfLoopType",363,sr,Uyn,Zgn),q5t;M(1732,1,{},gUe);var Y5t,W5t,K5t,X5t;D(Qu,"PortRestorer",1732),M(361,22,{3:1,35:1,22:1,361:1},eY);var Jb,zg,ev,$te=tr(Qu,"PortRestorer/PortSideArea",361,sr,nmn,Jgn),Q5t;M(1733,1,{},Bje),g.Kb=function(t){return Mg(),h(t,15).Oc()},D(Qu,"PortRestorer/lambda$0$Type",1733),M(1734,1,mn,Fje),g.td=function(t){Mg(),h(t,113).c=!1},D(Qu,"PortRestorer/lambda$1$Type",1734),M(1743,1,Cr,Rje),g.Mb=function(t){return Mg(),h(t,11).j==(Qe(),un)},D(Qu,"PortRestorer/lambda$10$Type",1743),M(1744,1,{},jje),g.Kb=function(t){return Mg(),h(t,113).d},D(Qu,"PortRestorer/lambda$11$Type",1744),M(1745,1,mn,FYe),g.td=function(t){y1n(this.a,h(t,11))},D(Qu,"PortRestorer/lambda$12$Type",1745),M(1735,1,mn,RYe),g.td=function(t){C0n(this.a,h(t,101))},D(Qu,"PortRestorer/lambda$2$Type",1735),M(1736,1,Yr,$je),g.ue=function(t,n){return C4n(h(t,113),h(n,113))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Qu,"PortRestorer/lambda$3$Type",1736),M(1737,1,Cr,Hje),g.Mb=function(t){return Mg(),h(t,113).c},D(Qu,"PortRestorer/lambda$4$Type",1737),M(1738,1,Cr,_je),g.Mb=function(t){return d5n(h(t,11))},D(Qu,"PortRestorer/lambda$5$Type",1738),M(1739,1,Cr,Eje),g.Mb=function(t){return Mg(),h(t,11).j==(Qe(),sn)},D(Qu,"PortRestorer/lambda$6$Type",1739),M(1740,1,Cr,Tje),g.Mb=function(t){return Mg(),h(t,11).j==(Qe(),pn)},D(Qu,"PortRestorer/lambda$7$Type",1740),M(1741,1,Cr,Cje),g.Mb=function(t){return tyn(h(t,11))},D(Qu,"PortRestorer/lambda$8$Type",1741),M(1742,1,Cr,Sje),g.Mb=function(t){return Mg(),h(t,11).j==(Qe(),Fn)},D(Qu,"PortRestorer/lambda$9$Type",1742),M(270,22,{3:1,35:1,22:1,270:1},G4);var Hte,zte,Gte,Vte,Ute,qte,Yte,Wte,$5e=tr(Qu,"PortSideAssigner/Target",270,sr,w4n,Wgn),Z5t;M(1725,1,{},Aje),g.Kb=function(t){return Jr(new Vt(null,new Wt(h(t,101).j,16)),new zue)},D(Qu,"PortSideAssigner/lambda$1$Type",1725),M(1726,1,{},Mje),g.Kb=function(t){return h(t,113).d},D(Qu,"PortSideAssigner/lambda$2$Type",1726),M(1727,1,mn,Dje),g.td=function(t){Ji(h(t,11),(Qe(),sn))},D(Qu,"PortSideAssigner/lambda$3$Type",1727),M(1728,1,{},Lje),g.Kb=function(t){return h(t,113).d},D(Qu,"PortSideAssigner/lambda$4$Type",1728),M(1729,1,mn,jYe),g.td=function(t){Cfn(this.a,h(t,11))},D(Qu,"PortSideAssigner/lambda$5$Type",1729),M(1730,1,Yr,Nje),g.ue=function(t,n){return nvn(h(t,101),h(n,101))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Qu,"PortSideAssigner/lambda$6$Type",1730),M(1731,1,Yr,Pje),g.ue=function(t,n){return M2n(h(t,113),h(n,113))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Qu,"PortSideAssigner/lambda$7$Type",1731),M(805,1,Cr,zue),g.Mb=function(t){return h(t,113).c},D(Qu,"PortSideAssigner/lambda$8$Type",805),M(2009,1,{}),D(Pg,"AbstractSelfLoopRouter",2009),M(1750,1,Yr,zje),g.ue=function(t,n){return mgn(h(t,101),h(n,101))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Pg,kvt,1750),M(1751,1,Yr,Gje),g.ue=function(t,n){return wgn(h(t,101),h(n,101))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Pg,xvt,1751),M(1793,2009,{},Vje),g.Uf=function(t,n,s){return s},D(Pg,"OrthogonalSelfLoopRouter",1793),M(1795,1,mn,BZe),g.td=function(t){Ype(this.b,this.a,h(t,8))},D(Pg,"OrthogonalSelfLoopRouter/lambda$0$Type",1795),M(1794,1793,{},Uje),g.Uf=function(t,n,s){var a,u;return a=t.c.d,rk(s,0,zr(ro(a.n),a.a)),u=t.d.d,vr(s,zr(ro(u.n),u.a)),mDn(s)},D(Pg,"PolylineSelfLoopRouter",1794),M(1746,1,{},cUe),g.a=null;var s3;D(Pg,"RoutingDirector",1746),M(1747,1,Yr,qje),g.ue=function(t,n){return I2n(h(t,113),h(n,113))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Pg,"RoutingDirector/lambda$0$Type",1747),M(1748,1,{},Yje),g.Kb=function(t){return xN(),h(t,101).j},D(Pg,"RoutingDirector/lambda$1$Type",1748),M(1749,1,mn,Wje),g.td=function(t){xN(),h(t,15).ad(s3)},D(Pg,"RoutingDirector/lambda$2$Type",1749),M(1752,1,{},Kje),D(Pg,"RoutingSlotAssigner",1752),M(1753,1,Cr,$Ye),g.Mb=function(t){return Q1n(this.a,h(t,101))},D(Pg,"RoutingSlotAssigner/lambda$0$Type",1753),M(1754,1,Yr,HYe),g.ue=function(t,n){return tbn(this.a,h(t,101),h(n,101))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Pg,"RoutingSlotAssigner/lambda$1$Type",1754),M(1796,1793,{},Xje),g.Uf=function(t,n,s){var a,u,p,v;return a=Pe(Je(MF(t.b.g.b,(tt(),z5)))),v=new rtt(se(ie(cs,1),Re,8,0,[(p=t.c.d,zr(new _a(p.n),p.a))])),eSn(t,n,s,v,a),vr(v,(u=t.d.d,zr(new _a(u.n),u.a))),Kft(new _ve(v))},D(Pg,"SplineSelfLoopRouter",1796),M(578,1,Yr,Qut,Ent),g.ue=function(t,n){return kbt(this,h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(U9,"ModelOrderNodeComparator",578),M(1755,1,Cr,Qje),g.Mb=function(t){return h(t,11).e.c.length!=0},D(U9,"ModelOrderNodeComparator/lambda$0$Type",1755),M(1756,1,{},Zje),g.Kb=function(t){return h(ut(h(t,11).e,0),17).c},D(U9,"ModelOrderNodeComparator/lambda$1$Type",1756),M(1757,1,Cr,Jje),g.Mb=function(t){return h(t,11).e.c.length!=0},D(U9,"ModelOrderNodeComparator/lambda$2$Type",1757),M(1758,1,{},e$e),g.Kb=function(t){return h(ut(h(t,11).e,0),17).c},D(U9,"ModelOrderNodeComparator/lambda$3$Type",1758),M(1759,1,Cr,t$e),g.Mb=function(t){return h(t,11).e.c.length!=0},D(U9,"ModelOrderNodeComparator/lambda$4$Type",1759),M(806,1,Yr,Act,RZe),g.ue=function(t,n){return wrt(this,t,n)},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(U9,"ModelOrderPortComparator",806),M(801,1,{},Gue),g.Vf=function(t,n){var s,a,u,p;for(u=xdt(n),s=new qe,p=n.f/u,a=1;a<u;++a)Ue(s,Ke(Gn(xc(d.Math.round(a*p)))));return s},g.Wf=function(){return!1},D(f1,"ARDCutIndexHeuristic",801),M(1479,1,xi,n$e),g.pf=function(t,n){PMn(h(t,37),n)},D(f1,"BreakingPointInserter",1479),M(305,1,{305:1},v0e),g.Ib=function(){var t;return t=new rg,t.a+="BPInfo[",t.a+=`
	start=`,ho(t,this.i),t.a+=`
	end=`,ho(t,this.a),t.a+=`
	nodeStartEdge=`,ho(t,this.e),t.a+=`
	startEndEdge=`,ho(t,this.j),t.a+=`
	originalEdge=`,ho(t,this.f),t.a+=`
	startInLayerDummy=`,ho(t,this.k),t.a+=`
	startInLayerEdge=`,ho(t,this.n),t.a+=`
	endInLayerDummy=`,ho(t,this.b),t.a+=`
	endInLayerEdge=`,ho(t,this.c),t.a},D(f1,"BreakingPointInserter/BPInfo",305),M(652,1,{652:1},XYe),g.a=!1,g.b=0,g.c=0,D(f1,"BreakingPointInserter/Cut",652),M(1480,1,xi,r$e),g.pf=function(t,n){hSn(h(t,37),n)},D(f1,"BreakingPointProcessor",1480),M(1481,1,Cr,i$e),g.Mb=function(t){return Yct(h(t,10))},D(f1,"BreakingPointProcessor/0methodref$isEnd$Type",1481),M(1482,1,Cr,s$e),g.Mb=function(t){return Wct(h(t,10))},D(f1,"BreakingPointProcessor/1methodref$isStart$Type",1482),M(1483,1,xi,a$e),g.pf=function(t,n){NSn(this,h(t,37),n)},D(f1,"BreakingPointRemover",1483),M(1484,1,mn,o$e),g.td=function(t){h(t,128).k=!0},D(f1,"BreakingPointRemover/lambda$0$Type",1484),M(797,1,{},Lbe),g.b=0,g.e=0,g.f=0,g.j=0,D(f1,"GraphStats",797),M(798,1,{},Vue),g.Ce=function(t,n){return d.Math.max(Pe(Je(t)),Pe(Je(n)))},D(f1,"GraphStats/0methodref$max$Type",798),M(799,1,{},Uue),g.Ce=function(t,n){return d.Math.max(Pe(Je(t)),Pe(Je(n)))},D(f1,"GraphStats/2methodref$max$Type",799),M(1660,1,{},c$e),g.Ce=function(t,n){return m2n(Je(t),Je(n))},D(f1,"GraphStats/lambda$1$Type",1660),M(1661,1,{},zYe),g.Kb=function(t){return Bft(this.a,h(t,29))},D(f1,"GraphStats/lambda$2$Type",1661),M(1662,1,{},GYe),g.Kb=function(t){return vgt(this.a,h(t,29))},D(f1,"GraphStats/lambda$6$Type",1662),M(800,1,{},que),g.Vf=function(t,n){var s;return s=h(U(t,(tt(),Bke)),15),s||(Rt(),Rt(),ca)},g.Wf=function(){return!1},D(f1,"ICutIndexCalculator/ManualCutIndexCalculator",800),M(802,1,{},Yue),g.Vf=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne;for(Ne=(n.n==null&&a1t(n),n.n),x=(n.d==null&&a1t(n),n.d),we=pe(ks,ka,25,Ne.length,15,1),we[0]=Ne[0],fe=Ne[0],_=1;_<Ne.length;_++)we[_]=we[_-1]+Ne[_],fe+=Ne[_];for(u=xdt(n)-1,v=h(U(t,(tt(),Fke)),19).a,a=Fi,s=new qe,P=d.Math.max(0,u-v);P<=d.Math.min(n.f-1,u+v);P++){if(J=fe/(P+1),oe=0,A=1,p=new qe,le=Fi,I=0,m=0,V=x[0],P==0)le=fe,m=(n.g==null&&(n.g=Jut(n,new Uue)),Pe(n.g));else{for(;A<n.f;)we[A-1]-oe>=J&&(Ue(p,Ke(A)),le=d.Math.max(le,we[A-1]-I),m+=V,oe+=we[A-1]-oe,I=we[A-1],V=x[A]),V=d.Math.max(V,x[A]),++A;m+=V}j=d.Math.min(1/le,1/n.b/m),j>a&&(a=j,s=p)}return s},g.Wf=function(){return!1},D(f1,"MSDCutIndexHeuristic",802),M(1617,1,xi,u$e),g.pf=function(t,n){aLn(h(t,37),n)},D(f1,"SingleEdgeGraphWrapper",1617),M(227,22,{3:1,35:1,22:1,227:1},tE);var a3,Zx,Jx,Jw,m_,o3,e8=tr(ko,"CenterEdgeLabelPlacementStrategy",227,sr,v3n,Kgn),J5t;M(422,22,{3:1,35:1,22:1,422:1},Khe);var H5e,Kte,z5e=tr(ko,"ConstraintCalculationStrategy",422,sr,own,Xgn),e6t;M(314,22,{3:1,35:1,22:1,314:1,246:1,234:1},tY),g.Kf=function(){return u0t(this)},g.Xf=function(){return u0t(this)};var _D,I5,G5e,V5e=tr(ko,"CrossingMinimizationStrategy",314,sr,Jwn,Qgn),t6t;M(337,22,{3:1,35:1,22:1,337:1},nY);var U5e,Xte,C$,q5e=tr(ko,"CuttingStrategy",337,sr,emn,epn),n6t;M(335,22,{3:1,35:1,22:1,335:1,246:1,234:1},MS),g.Kf=function(){return K0t(this)},g.Xf=function(){return K0t(this)};var Y5e,Qte,y_,Zte,k_,W5e=tr(ko,"CycleBreakingStrategy",335,sr,Gyn,tpn),r6t;M(419,22,{3:1,35:1,22:1,419:1},Xhe);var S$,K5e,X5e=tr(ko,"DirectionCongruency",419,sr,awn,npn),i6t;M(450,22,{3:1,35:1,22:1,450:1},rY);var t8,Jte,c3,s6t=tr(ko,"EdgeConstraint",450,sr,tmn,rpn),a6t;M(276,22,{3:1,35:1,22:1,276:1},nE);var ene,tne,nne,rne,A$,ine,Q5e=tr(ko,"EdgeLabelSideSelection",276,sr,k3n,ipn),o6t;M(479,22,{3:1,35:1,22:1,479:1},Qhe);var M$,Z5e,J5e=tr(ko,"EdgeStraighteningStrategy",479,sr,swn,spn),c6t;M(274,22,{3:1,35:1,22:1,274:1},rE);var sne,e6e,t6e,D$,n6e,r6e,i6e=tr(ko,"FixedAlignment",274,sr,m3n,apn),u6t;M(275,22,{3:1,35:1,22:1,275:1},iE);var s6e,a6e,o6e,c6e,x_,u6e,l6e=tr(ko,"GraphCompactionStrategy",275,sr,w3n,opn),l6t;M(256,22,{3:1,35:1,22:1,256:1},Zv);var n8,L$,r8,El,E_,I$,i8,u3,O$,__,ane=tr(ko,"GraphProperties",256,sr,a5n,cpn),h6t;M(292,22,{3:1,35:1,22:1,292:1},iY);var TD,one,cne,une=tr(ko,"GreedySwitchType",292,sr,imn,upn),f6t;M(303,22,{3:1,35:1,22:1,303:1},sY);var O5,CD,l3,d6t=tr(ko,"InLayerConstraint",303,sr,rmn,lpn),g6t;M(420,22,{3:1,35:1,22:1,420:1},Zhe);var lne,h6e,f6e=tr(ko,"InteractiveReferencePoint",420,sr,cwn,hpn),p6t,d6e,N5,tv,N$,g6e,p6e,P$,b6e,SD,B$,T_,P5,em,hne,F$,co,v6e,nv,Uo,fne,dne,AD,h2,rv,B5,w6e,F5,MD,tm,rf,Yl,gne,h3,Eo,Br,m6e,y6e,k6e,x6e,E6e,pne,R$,Zc,iv,bne,R5,DD,Ff,f3,s8,d3,g3,a8,f2,_6e,vne,wne,j5;M(163,22,{3:1,35:1,22:1,163:1},LS);var C_,Gg,S_,nm,LD,T6e=tr(ko,"LayerConstraint",163,sr,qyn,fpn),b6t;M(848,1,$l,vUe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Rwe),""),"Direction Congruency"),"Specifies how drawings of the same graph with different layout directions compare to each other: either a natural reading direction is preserved or the drawings are rotated versions of each other."),P6e),(E0(),_i)),X5e),Lt((Qh(),gn))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,jwe),""),"Feedback Edges"),"Whether feedback edges should be highlighted by routing around the nodes."),(on(),!1)),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,HR),""),"Interactive Reference Point"),"Determines which point of a node is considered by interactive layout phases."),H6e),_i),f6e),Lt(gn)))),Es(t,HR,wJ,okt),Es(t,HR,q9,akt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,$we),""),"Merge Edges"),"Edges that have no ports are merged so they touch the connected nodes at the same points. When this option is disabled, one port is created for each edge directly connected to a node. When it is enabled, all such incoming edges share an input port, and all outgoing edges share an output port."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Hwe),""),"Merge Hierarchy-Crossing Edges"),"If hierarchical layout is active, hierarchy-crossing edges use as few hierarchical ports as possible. They are broken by the algorithm, with hierarchical ports inserted as required. Usually, one such port is created for each edge at each hierarchy crossing point. With this option set to true, we try to create as few hierarchical ports as possible in the process. In particular, all edges that form a hyperedge can share a port."),!0),zs),es),Lt(gn)))),Dt(t,new kt(w1n(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,zwe),""),"Allow Non-Flow Ports To Switch Sides"),"Specifies whether non-flow ports may switch sides if their node's port constraints are either FIXED_SIDE or FIXED_ORDER. A non-flow port is a port on a side that is not part of the currently configured layout flow. For instance, given a left-to-right layout direction, north and south ports would be considered non-flow ports. Further note that the underlying criterium whether to switch sides or not solely relies on the minimization of edge crossings. Hence, edge length and other aesthetics criteria are not addressed."),!1),zs),es),Lt(w2)),se(ie(it,1),Re,2,6,["org.eclipse.elk.layered.northOrSouthPort"])))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Gwe),""),"Port Sorting Strategy"),"Only relevant for nodes with FIXED_SIDE port constraints. Determines the way a node's ports are distributed on the sides of a node if their order is not prescribed. The option is set on parent nodes."),K6e),_i),rxe),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Vwe),""),"Thoroughness"),"How much effort should be spent to produce a nice layout."),Ke(7)),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Uwe),""),"Add Unnecessary Bendpoints"),"Adds bend points even if an edge does not change direction. If true, each long edge dummy will contribute a bend point to its edges and hierarchy-crossing edges will always get a bend point where they cross hierarchy boundaries. By default, bend points are only added where an edge changes direction."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,qwe),""),"Generate Position and Layer IDs"),"If enabled position id and layer id are generated, which are usually only used internally when setting the interactiveLayout option. This option should be specified on the root node."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,wJ),"cycleBreaking"),"Cycle Breaking Strategy"),"Strategy for cycle breaking. Cycle breaking looks for cycles in the graph and determines which edges to reverse to break the cycles. Reversed edges will end up pointing to the opposite direction of regular edges (that is, reversed edges will point left if edges usually point right)."),N6e),_i),W5e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,XM),zJ),"Node Layering Strategy"),"Strategy for node layering."),V6e),_i),Uke),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Ywe),zJ),"Layer Constraint"),"Determines a constraint on the placement of the node regarding the layering."),z6e),_i),T6e),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Wwe),zJ),"Layer Choice Constraint"),"Allows to set a constraint regarding the layer placement of a node. Let i be the value of teh constraint. Assumed the drawing has n layers and i < n. If set to i, it expresses that the node should be placed in i-th layer. Should i>=n be true then the node is placed in the last layer of the drawing. Note that this option is not part of any of ELK Layered's default configurations but is only evaluated as part of the `InteractiveLayeredGraphVisitor`, which must be applied manually or used via the `DiagramLayoutEngine."),Ke(-1)),po),Ws),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Kwe),zJ),"Layer ID"),"Layer identifier that was calculated by ELK Layered for a node. This is only generated if interactiveLayot or generatePositionAndLayerIds is set."),Ke(-1)),po),Ws),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,mJ),iwt),"Upper Bound On Width [MinWidth Layerer]"),"Defines a loose upper bound on the width of the MinWidth layerer. If set to '-1' multiple values are tested and the best result is selected."),Ke(4)),po),Ws),Lt(gn)))),Es(t,mJ,XM,gkt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,yJ),iwt),"Upper Layer Estimation Scaling Factor [MinWidth Layerer]"),"Multiplied with Upper Bound On Width for defining an upper bound on the width of layers which haven't been determined yet, but whose maximum width had been (roughly) estimated by the MinWidth algorithm. Compensates for too high estimations. If set to '-1' multiple values are tested and the best result is selected."),Ke(2)),po),Ws),Lt(gn)))),Es(t,yJ,XM,bkt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,kJ),swt),"Node Promotion Strategy"),"Reduces number of dummy nodes after layering phase (if possible)."),G6e),_i),exe),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,xJ),swt),"Max Node Promotion Iterations"),"Limits the number of iterations for node promotion."),Ke(0)),po),Ws),Lt(gn)))),Es(t,xJ,kJ,null),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,EJ),"layering.coffmanGraham"),"Layer Bound"),"The maximum number of nodes allowed per layer."),Ke(Ir)),po),Ws),Lt(gn)))),Es(t,EJ,XM,ukt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,q9),QM),"Crossing Minimization Strategy"),"Strategy for crossing minimization."),O6e),_i),V5e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Xwe),QM),"Force Node Model Order"),"The node order given by the model does not change to produce a better layout. E.g. if node A is before node B in the model this is not changed during crossing minimization. This assumes that the node model order is already respected before crossing minimization. This can be achieved by setting considerModelOrder.strategy to NODES_AND_EDGES."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,_J),QM),"Hierarchical Sweepiness"),"How likely it is to use cross-hierarchy (1) vs bottom-up (-1)."),.1),Ba),Ts),Lt(gn)))),Es(t,_J,ej,P6t),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,TJ),QM),"Semi-Interactive Crossing Minimization"),"Preserves the order of nodes within a layer but still minimizes crossings between edges connecting long edge dummies. Derives the desired order from positions specified by the 'org.eclipse.elk.position' layout option. Requires a crossing minimization strategy that is able to process 'in-layer' constraints."),!1),zs),es),Lt(gn)))),Es(t,TJ,q9,j6t),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Qwe),QM),"Position Choice Constraint"),"Allows to set a constraint regarding the position placement of a node in a layer. Assumed the layer in which the node placed includes n other nodes and i < n. If set to i, it expresses that the node should be placed at the i-th position. Should i>=n be true then the node is placed at the last position in the layer. Note that this option is not part of any of ELK Layered's default configurations but is only evaluated as part of the `InteractiveLayeredGraphVisitor`, which must be applied manually or used via the `DiagramLayoutEngine."),Ke(-1)),po),Ws),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Zwe),QM),"Position ID"),"Position within a layer that was determined by ELK Layered for a node. This is only generated if interactiveLayot or generatePositionAndLayerIds is set."),Ke(-1)),po),Ws),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Jwe),awt),"Greedy Switch Activation Threshold"),"By default it is decided automatically if the greedy switch is activated or not. The decision is based on whether the size of the input graph (without dummy nodes) is smaller than the value of this option. A '0' enforces the activation."),Ke(40)),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,CJ),awt),"Greedy Switch Crossing Minimization"),"Greedy Switch strategy for crossing minimization. The greedy switch heuristic is executed after the regular crossing minimization as a post-processor. Note that if 'hierarchyHandling' is set to 'INCLUDE_CHILDREN', the 'greedySwitchHierarchical.type' option must be used."),I6e),_i),une),Lt(gn)))),Es(t,CJ,q9,O6t),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,zR),"crossingMinimization.greedySwitchHierarchical"),"Greedy Switch Crossing Minimization (hierarchical)"),"Activates the greedy switch heuristic in case hierarchical layout is used. The differences to the non-hierarchical case (see 'greedySwitch.type') are: 1) greedy switch is inactive by default, 3) only the option value set on the node at which hierarchical layout starts is relevant, and 2) if it's activated by the user, it properly addresses hierarchy-crossing edges."),L6e),_i),une),Lt(gn)))),Es(t,zR,q9,D6t),Es(t,zR,ej,L6t),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Wy),owt),"Node Placement Strategy"),"Strategy for node placement."),W6e),_i),Kke),Lt(gn)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,GR),owt),"Favor Straight Edges Over Balancing"),"Favor straight edges over a balanced node placement. The default behavior is determined automatically based on the used 'edgeRouting'. For an orthogonal style it is set to true, for all other styles to false."),zs),es),Lt(gn)))),Es(t,GR,Wy,Skt),Es(t,GR,Wy,Akt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,SJ),cwt),"BK Edge Straightening"),"Specifies whether the Brandes Koepf node placer tries to increase the number of straight edges at the expense of diagram size. There is a subtle difference to the 'favorStraightEdges' option, which decides whether a balanced placement of the nodes is desired, or not. In bk terms this means combining the four alignments into a single balanced one, or not. This option on the other hand tries to straighten additional edges during the creation of each of the four alignments."),U6e),_i),J5e),Lt(gn)))),Es(t,SJ,Wy,Ekt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,AJ),cwt),"BK Fixed Alignment"),"Tells the BK node placer to use a certain alignment (out of its four) instead of the one producing the smallest height, or the combination of all four."),q6e),_i),i6e),Lt(gn)))),Es(t,AJ,Wy,Tkt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,MJ),"nodePlacement.linearSegments"),"Linear Segments Deflection Dampening"),"Dampens the movement of nodes to keep the diagram from getting too large."),.3),Ba),Ts),Lt(gn)))),Es(t,MJ,Wy,Dkt),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,DJ),"nodePlacement.networkSimplex"),"Node Flexibility"),"Aims at shorter and straighter edges. Two configurations are possible: (a) allow ports to move freely on the side they are assigned to (the order is always defined beforehand), (b) additionally allow to enlarge a node wherever it helps. If this option is not configured for a node, the 'nodeFlexibility.default' value is used, which is specified for the node's parent."),_i),Hne),Lt(gs)))),Es(t,DJ,Wy,Nkt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,LJ),"nodePlacement.networkSimplex.nodeFlexibility"),"Node Flexibility Default"),"Default value of the 'nodeFlexibility' option for the children of a hierarchical node."),Y6e),_i),Hne),Lt(gn)))),Es(t,LJ,Wy,Okt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,eme),uwt),"Self-Loop Distribution"),"Alter the distribution of the loops around the node. It only takes effect for PortConstraints.FREE."),R6e),_i),axe),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,tme),uwt),"Self-Loop Ordering"),"Alter the ordering of the loops they can either be stacked or sequenced. It only takes effect for PortConstraints.FREE."),j6e),_i),oxe),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,VR),"edgeRouting.splines"),"Spline Routing Mode"),"Specifies the way control points are assembled for each individual edge. CONSERVATIVE ensures that edges are properly routed around the nodes but feels rather orthogonal at times. SLOPPY uses fewer control points to obtain curvier edge routes but may result in edges overlapping nodes."),$6e),_i),uxe),Lt(gn)))),Es(t,VR,ZM,X6t),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,UR),"edgeRouting.splines.sloppy"),"Sloppy Spline Layer Spacing Factor"),"Spacing factor for routing area between layers when using sloppy spline routing."),.2),Ba),Ts),Lt(gn)))),Es(t,UR,ZM,Z6t),Es(t,UR,VR,J6t),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,IJ),"edgeRouting.polyline"),"Sloped Edge Zone Width"),"Width of the strip to the left and to the right of each layer where the polyline edge router is allowed to refrain from ensuring that edges are routed horizontally. This prevents awkward bend points for nodes that extent almost to the edge of their layer."),2),Ba),Ts),Lt(gn)))),Es(t,IJ,ZM,q6t),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,nme),Of),"Spacing Base Value"),"An optional base value for all other layout options of the 'spacing' group. It can be used to conveniently alter the overall 'spaciousness' of the drawing. Whenever an explicit value is set for the other layout options, this base value will have no effect. The base value is not inherited, i.e. it must be set for each hierarchical node."),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,rme),Of),"Edge Node Between Layers Spacing"),"The spacing to be preserved between nodes and edges that are routed next to the node's layer. For the spacing between nodes and edges that cross the node's layer 'spacing.edgeNode' is used."),10),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ime),Of),"Edge Edge Between Layer Spacing"),"Spacing to be preserved between pairs of edges that are routed between the same pair of layers. Note that 'spacing.edgeEdge' is used for the spacing between pairs of edges crossing the same layer."),10),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,sme),Of),"Node Node Between Layers Spacing"),"The spacing to be preserved between any pair of nodes of two adjacent layers. Note that 'spacing.nodeNode' is used for the spacing between nodes within the layer itself."),20),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ame),pme),"Direction Priority"),"Defines how important it is to have a certain edge point into the direction of the overall layout. This option is evaluated during the cycle breaking phase."),Ke(0)),po),Ws),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ome),pme),"Shortness Priority"),"Defines how important it is to keep an edge as short as possible. This option is evaluated during the layering phase."),Ke(0)),po),Ws),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,cme),pme),"Straightness Priority"),"Defines how important it is to keep an edge straight, i.e. aligned with one of the two axes. This option is evaluated during node placement."),Ke(0)),po),Ws),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,OJ),bme),Cvt),"Tries to further compact components (disconnected sub-graphs)."),!1),zs),es),Lt(gn)))),Es(t,OJ,H9,!0),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ume),lwt),"Post Compaction Strategy"),hwt),S6e),_i),l6e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,lme),lwt),"Post Compaction Constraint Calculation"),hwt),C6e),_i),z5e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,qR),vme),"High Degree Node Treatment"),"Makes room around high degree nodes to place leafs and trees."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,NJ),vme),"High Degree Node Threshold"),"Whether a node is considered to have a high degree."),Ke(16)),po),Ws),Lt(gn)))),Es(t,NJ,qR,!0),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,PJ),vme),"High Degree Node Maximum Tree Height"),"Maximum height of a subtree connected to a high degree node to be moved to separate layers."),Ke(5)),po),Ws),Lt(gn)))),Es(t,PJ,qR,!0),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,$d),wme),"Graph Wrapping Strategy"),"For certain graphs and certain prescribed drawing areas it may be desirable to split the laid out graph into chunks that are placed side by side. The edges that connect different chunks are 'wrapped' around from the end of one chunk to the start of the other chunk. The points between the chunks are referred to as 'cuts'."),Z6e),_i),dxe),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,YR),wme),"Additional Wrapped Edges Spacing"),"To visually separate edges that are wrapped from regularly routed edges an additional spacing value can be specified in form of this layout option. The spacing is added to the regular edgeNode spacing."),10),Ba),Ts),Lt(gn)))),Es(t,YR,$d,Ykt),Es(t,YR,$d,Wkt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,WR),wme),"Correction Factor for Wrapping"),"At times and for certain types of graphs the executed wrapping may produce results that are consistently biased in the same fashion: either wrapping to often or to rarely. This factor can be used to correct the bias. Internally, it is simply multiplied with the 'aspect ratio' layout option."),1),Ba),Ts),Lt(gn)))),Es(t,WR,$d,Xkt),Es(t,WR,$d,Qkt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Y9),fwt),"Cutting Strategy"),"The strategy by which the layer indexes are determined at which the layering crumbles into chunks."),Q6e),_i),q5e),Lt(gn)))),Es(t,Y9,$d,rxt),Es(t,Y9,$d,ixt),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,BJ),fwt),"Manually Specified Cuts"),"Allows the user to specify her own cuts for a certain graph."),jf),xl),Lt(gn)))),Es(t,BJ,Y9,Jkt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,FJ),"wrapping.cutting.msd"),"MSD Freedom"),"The MSD cutting strategy starts with an initial guess on the number of chunks the graph should be split into. The freedom specifies how much the strategy may deviate from this guess. E.g. if an initial number of 3 is computed, a freedom of 1 allows 2, 3, and 4 cuts."),X6e),po),Ws),Lt(gn)))),Es(t,FJ,Y9,txt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,KR),dwt),"Validification Strategy"),"When wrapping graphs, one can specify indices that are not allowed as split points. The validification strategy makes sure every computed split point is allowed."),J6e),_i),fxe),Lt(gn)))),Es(t,KR,$d,bxt),Es(t,KR,$d,vxt),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,XR),dwt),"Valid Indices for Wrapping"),null),jf),xl),Lt(gn)))),Es(t,XR,$d,dxt),Es(t,XR,$d,gxt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,QR),mme),"Improve Cuts"),"For general graphs it is important that not too many edges wrap backwards. Thus a compromise between evenly-distributed cuts and the total number of cut edges is sought."),!0),zs),es),Lt(gn)))),Es(t,QR,$d,cxt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ZR),mme),"Distance Penalty When Improving Cuts"),null),2),Ba),Ts),Lt(gn)))),Es(t,ZR,$d,axt),Es(t,ZR,QR,!0),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,RJ),mme),"Improve Wrapped Edges"),"The initial wrapping is performed in a very simple way. As a consequence, edges that wrap from one chunk to another may be unnecessarily long. Activating this option tries to shorten such edges."),!0),zs),es),Lt(gn)))),Es(t,RJ,$d,lxt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,hme),GJ),"Edge Label Side Selection"),"Method to decide on edge label sides."),F6e),_i),Q5e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,fme),GJ),"Edge Center Label Placement Strategy"),"Determines in which layer center labels of long edges should be placed."),B6e),_i),e8),ni(gn,se(ie(P0,1),ze,175,0,[N0]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,JR),JM),"Consider Model Order"),"Preserves the order of nodes and edges in the model file if this does not lead to additional edge crossings. Depending on the strategy this is not always possible since the node and edge order might be conflicting."),D6e),_i),nxe),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,dme),JM),"No Model Order"),"Set on a node to not set a model order for this node even though it is a real node."),!1),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,jJ),JM),"Consider Model Order for Components"),"If set to NONE the usual ordering strategy (by cumulative node priority and size of nodes) is used. INSIDE_PORT_SIDES orders the components with external ports only inside the groups with the same port side. FORCE_MODEL_ORDER enforces the mode order on components. This option might produce bad alignments and sub optimal drawings in terms of used area since the ordering should be respected."),A6e),_i),W4e),Lt(gn)))),Es(t,jJ,H9,null),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,gme),JM),"Long Edge Ordering Strategy"),"Indicates whether long edges are sorted under, over, or equal to nodes that have no connection to a previous layer in a left-to-right or right-to-left layout. Under and over changes to right and left in a vertical layout."),M6e),_i),Yke),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,$J),JM),"Crossing Counter Node Order Influence"),"Indicates with what percentage (1 for 100%) violations of the node model order are weighted against the crossings e.g. a value of 0.5 means two model order violations are as important as on edge crossing. This allows some edge crossings in favor of preserving the model order. It is advised to set this value to a very small positive value (e.g. 0.001) to have minimal crossing and a optimal node order. Defaults to no influence (0)."),0),Ba),Ts),Lt(gn)))),Es(t,$J,JR,null),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,HJ),JM),"Crossing Counter Port Order Influence"),"Indicates with what percentage (1 for 100%) violations of the port model order are weighted against the crossings e.g. a value of 0.5 means two model order violations are as important as on edge crossing. This allows some edge crossings in favor of preserving the model order. It is advised to set this value to a very small positive value (e.g. 0.001) to have minimal crossing and a optimal port order. Defaults to no influence (0)."),0),Ba),Ts),Lt(gn)))),Es(t,HJ,JR,null),Lbt((new yUe,t))};var v6t,w6t,m6t,C6e,y6t,S6e,k6t,A6e,x6t,E6t,_6t,M6e,T6t,C6t,D6e,S6t,A6t,M6t,L6e,D6t,L6t,I6t,I6e,O6t,N6t,P6t,B6t,F6t,R6t,j6t,$6t,O6e,H6t,N6e,z6t,P6e,G6t,B6e,V6t,F6e,U6t,q6t,Y6t,R6e,W6t,j6e,K6t,$6e,X6t,Q6t,Z6t,J6t,ekt,tkt,nkt,rkt,ikt,skt,H6e,akt,okt,ckt,ukt,lkt,hkt,z6e,fkt,dkt,gkt,pkt,bkt,vkt,wkt,G6e,mkt,V6e,ykt,kkt,xkt,U6e,Ekt,_kt,q6e,Tkt,Ckt,Skt,Akt,Mkt,Dkt,Lkt,Ikt,Y6e,Okt,Nkt,Pkt,W6e,Bkt,K6e,Fkt,Rkt,jkt,$kt,Hkt,zkt,Gkt,Vkt,Ukt,qkt,Ykt,Wkt,Kkt,Xkt,Qkt,Zkt,Jkt,ext,X6e,txt,nxt,Q6e,rxt,ixt,sxt,axt,oxt,cxt,uxt,lxt,hxt,Z6e,fxt,dxt,gxt,pxt,J6e,bxt,vxt;D(ko,"LayeredMetaDataProvider",848),M(986,1,$l,yUe),g.Qe=function(t){Lbt(t)};var m1,mne,j$,A_,$$,eke,H$,$5,z$,tke,nke,yne,Vg,kne,rm,rke,ID,xne,ike,wxt,G$,Ene,M_,im,mxt,Ju,ske,ake,V$,_ne,y1,U$,Hd,oke,cke,uke,Tne,Cne,lke,D0,Sne,hke,sm,fke,dke,gke,q$,am,d2,pke,bke,Da,vke,yxt,cc,Y$,wke,mke,yke,Ane,kke,W$,xke,Eke,K$,sv,_ke,Mne,D_,Tke,av,L_,X$,g2,Dne,o8,Q$,p2,Cke,Ske,Ake,c8,Mke,kxt,xxt,Ext,_xt,ov,om,Ei,L0,Txt,cm,Dke,u8,Lke,um,Cxt,l8,Ike,H5,Sxt,Axt,OD,Lne,Oke,ND,Mh,p3,z5,cv,b2,Z$,lm,Ine,h8,f8,uv,b3,One,PD,I_,O_,Nne,Nke,Pke,Bke,Fke,Pne,Rke,jke,$ke,Hke,Bne,J$;D(ko,"LayeredOptions",986),M(987,1,{},h$e),g.$e=function(){var t;return t=new jKe,t},g._e=function(t){},D(ko,"LayeredOptions/LayeredFactory",987),M(1372,1,{}),g.a=0;var Mxt;D(xo,"ElkSpacings/AbstractSpacingsBuilder",1372),M(779,1372,{},Mpe);var eH,Dxt;D(ko,"LayeredSpacings/LayeredSpacingsBuilder",779),M(313,22,{3:1,35:1,22:1,313:1,246:1,234:1},sE),g.Kf=function(){return lgt(this)},g.Xf=function(){return lgt(this)};var Fne,zke,Gke,tH,Rne,Vke,Uke=tr(ko,"LayeringStrategy",313,sr,y3n,dpn),Lxt;M(378,22,{3:1,35:1,22:1,378:1},aY);var jne,qke,nH,Yke=tr(ko,"LongEdgeOrderingStrategy",378,sr,Zwn,gpn),Ixt;M(197,22,{3:1,35:1,22:1,197:1},RN);var v3,w3,rH,$ne,Hne=tr(ko,"NodeFlexibility",197,sr,syn,ppn),Oxt;M(315,22,{3:1,35:1,22:1,315:1,246:1,234:1},DS),g.Kf=function(){return W0t(this)},g.Xf=function(){return W0t(this)};var N_,zne,Gne,P_,Wke,Kke=tr(ko,"NodePlacementStrategy",315,sr,zyn,kpn),Nxt;M(260,22,{3:1,35:1,22:1,260:1},D4);var Xke,BD,Qke,Zke,FD,Jke,iH,sH,exe=tr(ko,"NodePromotionStrategy",260,sr,m4n,vpn),Pxt;M(339,22,{3:1,35:1,22:1,339:1},oY);var txe,Ug,Vne,nxe=tr(ko,"OrderingStrategy",339,sr,amn,wpn),Bxt;M(421,22,{3:1,35:1,22:1,421:1},Jhe);var Une,qne,rxe=tr(ko,"PortSortingStrategy",421,sr,uwn,mpn),Fxt;M(452,22,{3:1,35:1,22:1,452:1},cY);var Jc,ec,B_,Rxt=tr(ko,"PortType",452,sr,smn,bpn),jxt;M(375,22,{3:1,35:1,22:1,375:1},uY);var ixe,Yne,sxe,axe=tr(ko,"SelfLoopDistributionStrategy",375,sr,omn,ypn),$xt;M(376,22,{3:1,35:1,22:1,376:1},efe);var RD,Wne,oxe=tr(ko,"SelfLoopOrderingStrategy",376,sr,iwn,xpn),Hxt;M(304,1,{304:1},O2t),D(ko,"Spacings",304),M(336,22,{3:1,35:1,22:1,336:1},lY);var Kne,cxe,F_,uxe=tr(ko,"SplineRoutingMode",336,sr,umn,Epn),zxt;M(338,22,{3:1,35:1,22:1,338:1},hY);var Xne,lxe,hxe,fxe=tr(ko,"ValidifyStrategy",338,sr,lmn,_pn),Gxt;M(377,22,{3:1,35:1,22:1,377:1},fY);var hm,Qne,d8,dxe=tr(ko,"WrappingStrategy",377,sr,cmn,Tpn),Vxt;M(1383,1,zo,kUe),g.Yf=function(t){return h(t,37),Uxt},g.pf=function(t,n){eIn(this,h(t,37),n)};var Uxt;D(rj,"DepthFirstCycleBreaker",1383),M(782,1,zo,dde),g.Yf=function(t){return h(t,37),qxt},g.pf=function(t,n){ZNn(this,h(t,37),n)},g.Zf=function(t){return h(ut(t,NF(this.d,t.c.length)),10)};var qxt;D(rj,"GreedyCycleBreaker",782),M(1386,782,zo,wJe),g.Zf=function(t){var n,s,a,u;for(u=null,n=Ir,a=new Q(t);a.a<a.c.c.length;)s=h(K(a),10),os(s,(He(),Eo))&&h(U(s,Eo),19).a<n&&(n=h(U(s,Eo),19).a,u=s);return u||h(ut(t,NF(this.d,t.c.length)),10)},D(rj,"GreedyModelOrderCycleBreaker",1386),M(1384,1,zo,hUe),g.Yf=function(t){return h(t,37),Yxt},g.pf=function(t,n){xIn(this,h(t,37),n)};var Yxt;D(rj,"InteractiveCycleBreaker",1384),M(1385,1,zo,fUe),g.Yf=function(t){return h(t,37),Wxt},g.pf=function(t,n){EIn(this,h(t,37),n)},g.a=0,g.b=0;var Wxt;D(rj,"ModelOrderCycleBreaker",1385),M(1389,1,zo,lZe),g.Yf=function(t){return h(t,37),Kxt},g.pf=function(t,n){nPn(this,h(t,37),n)};var Kxt;D(Gw,"CoffmanGrahamLayerer",1389),M(1390,1,Yr,VYe),g.ue=function(t,n){return DTn(this.a,h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Gw,"CoffmanGrahamLayerer/0methodref$compareNodesInTopo$Type",1390),M(1391,1,Yr,UYe),g.ue=function(t,n){return bbn(this.a,h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Gw,"CoffmanGrahamLayerer/lambda$1$Type",1391),M(1392,1,zo,f$e),g.Yf=function(t){return h(t,37),Lr(Lr(Lr(new rs,(Zs(),w1),(oa(),Pte)),Hg,Kx),oc,Wx)},g.pf=function(t,n){sNn(this,h(t,37),n)},D(Gw,"InteractiveLayerer",1392),M(569,1,{569:1},YKe),g.a=0,g.c=0,D(Gw,"InteractiveLayerer/LayerSpan",569),M(1388,1,zo,lUe),g.Yf=function(t){return h(t,37),Xxt},g.pf=function(t,n){PAn(this,h(t,37),n)};var Xxt;D(Gw,"LongestPathLayerer",1388),M(1395,1,zo,bUe),g.Yf=function(t){return h(t,37),Lr(Lr(Lr(new rs,(Zs(),w1),(oa(),v_)),Hg,Kx),oc,Wx)},g.pf=function(t,n){ANn(this,h(t,37),n)},g.a=0,g.b=0,g.d=0;var gxe,pxe;D(Gw,"MinWidthLayerer",1395),M(1396,1,Yr,qYe),g.ue=function(t,n){return E5n(this,h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Gw,"MinWidthLayerer/MinOutgoingEdgesComparator",1396),M(1387,1,zo,pUe),g.Yf=function(t){return h(t,37),Qxt},g.pf=function(t,n){qIn(this,h(t,37),n)};var Qxt;D(Gw,"NetworkSimplexLayerer",1387),M(1393,1,zo,Ott),g.Yf=function(t){return h(t,37),Lr(Lr(Lr(new rs,(Zs(),w1),(oa(),v_)),Hg,Kx),oc,Wx)},g.pf=function(t,n){OOn(this,h(t,37),n)},g.d=0,g.f=0,g.g=0,g.i=0,g.s=0,g.t=0,g.u=0,D(Gw,"StretchWidthLayerer",1393),M(1394,1,Yr,d$e),g.ue=function(t,n){return e3n(h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Gw,"StretchWidthLayerer/1",1394),M(402,1,tye),g.Nf=function(t,n,s,a,u,p){},g._f=function(t,n,s){return Mpt(this,t,n,s)},g.Mf=function(){this.g=pe(Em,pwt,25,this.d,15,1),this.f=pe(Em,pwt,25,this.d,15,1)},g.Of=function(t,n){this.e[t]=pe(Hn,hr,25,n[t].length,15,1)},g.Pf=function(t,n,s){var a;a=s[t][n],a.p=n,this.e[t][n]=n},g.Qf=function(t,n,s,a){h(ut(a[t][n].j,s),11).p=this.d++},g.b=0,g.c=0,g.d=0,D(Hc,"AbstractBarycenterPortDistributor",402),M(1633,1,Yr,YYe),g.ue=function(t,n){return Yxn(this.a,h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Hc,"AbstractBarycenterPortDistributor/lambda$0$Type",1633),M(817,1,KM,p0e),g.Nf=function(t,n,s,a,u,p){},g.Pf=function(t,n,s){},g.Qf=function(t,n,s,a){},g.Lf=function(){return!1},g.Mf=function(){this.c=this.e.a,this.g=this.f.g},g.Of=function(t,n){n[t][0].c.p=t},g.Rf=function(){return!1},g.ag=function(t,n,s,a){s?B1t(this,t):($1t(this,t,a),Y2t(this,t,n)),t.c.length>1&&(ft(lt(U(Us((Kt(0,t.c.length),h(t.c[0],10))),(tt(),rm))))?igt(t,this.d,h(this,660)):(Rt(),hs(t,this.d)),_lt(this.e,t))},g.Sf=function(t,n,s,a){var u,p,v,m,x,_,A;for(n!=Knt(s,t.length)&&(p=t[n-(s?1:-1)],R0e(this.f,p,s?(ua(),ec):(ua(),Jc))),u=t[n][0],A=!a||u.k==(bn(),Pi),_=Ef(t[n]),this.ag(_,A,!1,s),v=0,x=new Q(_);x.a<x.c.c.length;)m=h(K(x),10),t[n][v++]=m;return!1},g.Tf=function(t,n){var s,a,u,p,v;for(v=Knt(n,t.length),p=Ef(t[v]),this.ag(p,!1,!0,n),s=0,u=new Q(p);u.a<u.c.c.length;)a=h(K(u),10),t[v][s++]=a;return!1},D(Hc,"BarycenterHeuristic",817),M(658,1,{658:1},KYe),g.Ib=function(){return"BarycenterState [node="+this.c+", summedWeight="+this.d+", degree="+this.b+", barycenter="+this.a+", visited="+this.e+"]"},g.b=0,g.d=0,g.e=!1;var Zxt=D(Hc,"BarycenterHeuristic/BarycenterState",658);M(1802,1,Yr,WYe),g.ue=function(t,n){return OEn(this.a,h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Hc,"BarycenterHeuristic/lambda$0$Type",1802),M(816,1,KM,$2e),g.Mf=function(){},g.Nf=function(t,n,s,a,u,p){},g.Qf=function(t,n,s,a){},g.Of=function(t,n){this.a[t]=pe(Zxt,{3:1,4:1,5:1,2018:1},658,n[t].length,0,1),this.b[t]=pe(Jxt,{3:1,4:1,5:1,2019:1},233,n[t].length,0,1)},g.Pf=function(t,n,s){Sft(this,s[t][n],!0)},g.c=!1,D(Hc,"ForsterConstraintResolver",816),M(233,1,{233:1},Vit,I2t),g.Ib=function(){var t,n;for(n=new rg,n.a+="[",t=0;t<this.d.length;t++)or(n,$ft(this.d[t])),wf(this.g,this.d[0]).a!=null&&or(or((n.a+="<",n),rdn(wf(this.g,this.d[0]).a)),">"),t<this.d.length-1&&(n.a+=Js);return(n.a+="]",n).a},g.a=0,g.c=0,g.f=0;var Jxt=D(Hc,"ForsterConstraintResolver/ConstraintGroup",233);M(1797,1,mn,QYe),g.td=function(t){Sft(this.a,h(t,10),!1)},D(Hc,"ForsterConstraintResolver/lambda$0$Type",1797),M(214,1,{214:1,225:1},R2t),g.Nf=function(t,n,s,a,u,p){},g.Of=function(t,n){},g.Mf=function(){this.r=pe(Hn,hr,25,this.n,15,1)},g.Pf=function(t,n,s){var a,u;u=s[t][n],a=u.e,a&&Ue(this.b,a)},g.Qf=function(t,n,s,a){++this.n},g.Ib=function(){return W2t(this.e,new ns)},g.g=!1,g.i=!1,g.n=0,g.s=!1,D(Hc,"GraphInfoHolder",214),M(1832,1,KM,l$e),g.Nf=function(t,n,s,a,u,p){},g.Of=function(t,n){},g.Qf=function(t,n,s,a){},g._f=function(t,n,s){return s&&n>0?qW(this.a,t[n-1],t[n]):!s&&n<t.length-1?qW(this.a,t[n],t[n+1]):$K(this.a,t[n],s?(Qe(),un):(Qe(),pn)),cSn(this,t,n,s)},g.Mf=function(){this.d=pe(Hn,hr,25,this.c,15,1),this.a=new KS(this.d)},g.Pf=function(t,n,s){var a;a=s[t][n],this.c+=a.j.c.length},g.c=0,D(Hc,"GreedyPortDistributor",1832),M(1401,1,zo,xUe),g.Yf=function(t){return j6n(h(t,37))},g.pf=function(t,n){cOn(h(t,37),n)};var e8t;D(Hc,"InteractiveCrossingMinimizer",1401),M(1402,1,Yr,ZYe),g.ue=function(t,n){return bEn(this,h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Hc,"InteractiveCrossingMinimizer/1",1402),M(507,1,{507:1,123:1,51:1},Pq),g.Yf=function(t){var n;return h(t,37),n=QN(t8t),Lr(n,(Zs(),oc),(oa(),g$)),n},g.pf=function(t,n){kDn(this,h(t,37),n)},g.e=0;var t8t;D(Hc,"LayerSweepCrossingMinimizer",507),M(1398,1,mn,JYe),g.td=function(t){oLn(this.a,h(t,214))},D(Hc,"LayerSweepCrossingMinimizer/0methodref$compareDifferentRandomizedLayouts$Type",1398),M(1399,1,mn,eWe),g.td=function(t){I6n(this.a,h(t,214))},D(Hc,"LayerSweepCrossingMinimizer/1methodref$minimizeCrossingsNoCounter$Type",1399),M(1400,1,mn,tWe),g.td=function(t){Wpt(this.a,h(t,214))},D(Hc,"LayerSweepCrossingMinimizer/2methodref$minimizeCrossingsWithCounter$Type",1400),M(454,22,{3:1,35:1,22:1,454:1},dY);var jD,R_,aH,n8t=tr(Hc,"LayerSweepCrossingMinimizer/CrossMinType",454,sr,hmn,Cpn),r8t;M(1397,1,Cr,g$e),g.Mb=function(t){return Mge(),h(t,29).a.c.length==0},D(Hc,"LayerSweepCrossingMinimizer/lambda$0$Type",1397),M(1799,1,KM,ost),g.Mf=function(){},g.Nf=function(t,n,s,a,u,p){},g.Qf=function(t,n,s,a){},g.Of=function(t,n){n[t][0].c.p=t,this.b[t]=pe(i8t,{3:1,4:1,5:1,1944:1},659,n[t].length,0,1)},g.Pf=function(t,n,s){var a;a=s[t][n],a.p=n,pi(this.b[t],n,new p$e)},D(Hc,"LayerSweepTypeDecider",1799),M(659,1,{659:1},p$e),g.Ib=function(){return"NodeInfo [connectedEdges="+this.a+", hierarchicalInfluence="+this.b+", randomInfluence="+this.c+"]"},g.a=0,g.b=0,g.c=0;var i8t=D(Hc,"LayerSweepTypeDecider/NodeInfo",659);M(1800,1,c1,b$e),g.Lb=function(t){return kE(new _f(h(t,11).b))},g.Fb=function(t){return this===t},g.Mb=function(t){return kE(new _f(h(t,11).b))},D(Hc,"LayerSweepTypeDecider/lambda$0$Type",1800),M(1801,1,c1,v$e),g.Lb=function(t){return kE(new _f(h(t,11).b))},g.Fb=function(t){return this===t},g.Mb=function(t){return kE(new _f(h(t,11).b))},D(Hc,"LayerSweepTypeDecider/lambda$1$Type",1801),M(1833,402,tye,qXe),g.$f=function(t,n,s){var a,u,p,v,m,x,_,A,I;switch(_=this.g,s.g){case 1:{for(a=0,u=0,x=new Q(t.j);x.a<x.c.c.length;)v=h(K(x),11),v.e.c.length!=0&&(++a,v.j==(Qe(),sn)&&++u);for(p=n+u,I=n+a,m=Db(t,(ua(),Jc)).Kc();m.Ob();)v=h(m.Pb(),11),v.j==(Qe(),sn)?(_[v.p]=p,--p):(_[v.p]=I,--I);return a}case 2:{for(A=0,m=Db(t,(ua(),ec)).Kc();m.Ob();)v=h(m.Pb(),11),++A,_[v.p]=n+A;return A}default:throw ne(new pS)}},D(Hc,"LayerTotalPortDistributor",1833),M(660,817,{660:1,225:1},Gut),g.ag=function(t,n,s,a){s?B1t(this,t):($1t(this,t,a),Y2t(this,t,n)),t.c.length>1&&(ft(lt(U(Us((Kt(0,t.c.length),h(t.c[0],10))),(tt(),rm))))?igt(t,this.d,this):(Rt(),hs(t,this.d)),ft(lt(U(Us((Kt(0,t.c.length),h(t.c[0],10))),rm)))||_lt(this.e,t))},D(Hc,"ModelOrderBarycenterHeuristic",660),M(1803,1,Yr,nWe),g.ue=function(t,n){return g_n(this.a,h(t,10),h(n,10))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Hc,"ModelOrderBarycenterHeuristic/lambda$0$Type",1803),M(1403,1,zo,MUe),g.Yf=function(t){var n;return h(t,37),n=QN(s8t),Lr(n,(Zs(),oc),(oa(),g$)),n},g.pf=function(t,n){Jvn((h(t,37),n))};var s8t;D(Hc,"NoCrossingMinimizer",1403),M(796,402,tye,Ehe),g.$f=function(t,n,s){var a,u,p,v,m,x,_,A,I,P,j;switch(I=this.g,s.g){case 1:{for(u=0,p=0,A=new Q(t.j);A.a<A.c.c.length;)x=h(K(A),11),x.e.c.length!=0&&(++u,x.j==(Qe(),sn)&&++p);for(a=1/(u+1),v=n+p*a,j=n+1-a,_=Db(t,(ua(),Jc)).Kc();_.Ob();)x=h(_.Pb(),11),x.j==(Qe(),sn)?(I[x.p]=v,v-=a):(I[x.p]=j,j-=a);break}case 2:{for(m=0,A=new Q(t.j);A.a<A.c.c.length;)x=h(K(A),11),x.g.c.length==0||++m;for(a=1/(m+1),P=n+a,_=Db(t,(ua(),ec)).Kc();_.Ob();)x=h(_.Pb(),11),I[x.p]=P,P+=a;break}default:throw ne(new an("Port type is undefined"))}return 1},D(Hc,"NodeRelativePortDistributor",796),M(807,1,{},krt,pdt),D(Hc,"SweepCopy",807),M(1798,1,KM,aft),g.Of=function(t,n){},g.Mf=function(){var t;t=pe(Hn,hr,25,this.f,15,1),this.d=new lWe(t),this.a=new KS(t)},g.Nf=function(t,n,s,a,u,p){var v;v=h(ut(p[t][n].j,s),11),u.c==v&&u.c.i.c==u.d.i.c&&++this.e[t]},g.Pf=function(t,n,s){var a;a=s[t][n],this.c[t]=this.c[t]|a.k==(bn(),Vo)},g.Qf=function(t,n,s,a){var u;u=h(ut(a[t][n].j,s),11),u.p=this.f++,u.g.c.length+u.e.c.length>1&&(u.j==(Qe(),pn)?this.b[t]=!0:u.j==un&&t>0&&(this.b[t-1]=!0))},g.f=0,D(W1,"AllCrossingsCounter",1798),M(587,1,{},XB),g.b=0,g.d=0,D(W1,"BinaryIndexedTree",587),M(524,1,{},KS);var bxe,oH;D(W1,"CrossingsCounter",524),M(1906,1,Yr,rWe),g.ue=function(t,n){return nbn(this.a,h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(W1,"CrossingsCounter/lambda$0$Type",1906),M(1907,1,Yr,iWe),g.ue=function(t,n){return rbn(this.a,h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(W1,"CrossingsCounter/lambda$1$Type",1907),M(1908,1,Yr,sWe),g.ue=function(t,n){return ibn(this.a,h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(W1,"CrossingsCounter/lambda$2$Type",1908),M(1909,1,Yr,aWe),g.ue=function(t,n){return sbn(this.a,h(t,11),h(n,11))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(W1,"CrossingsCounter/lambda$3$Type",1909),M(1910,1,mn,oWe),g.td=function(t){$3n(this.a,h(t,11))},D(W1,"CrossingsCounter/lambda$4$Type",1910),M(1911,1,Cr,cWe),g.Mb=function(t){return edn(this.a,h(t,11))},D(W1,"CrossingsCounter/lambda$5$Type",1911),M(1912,1,mn,uWe),g.td=function(t){gJe(this,t)},D(W1,"CrossingsCounter/lambda$6$Type",1912),M(1913,1,mn,$Ze),g.td=function(t){var n;ak(),mg(this.b,(n=this.a,h(t,11),n))},D(W1,"CrossingsCounter/lambda$7$Type",1913),M(826,1,c1,Wue),g.Lb=function(t){return ak(),os(h(t,11),(He(),Zc))},g.Fb=function(t){return this===t},g.Mb=function(t){return ak(),os(h(t,11),(He(),Zc))},D(W1,"CrossingsCounter/lambda$8$Type",826),M(1905,1,{},lWe),D(W1,"HyperedgeCrossingsCounter",1905),M(467,1,{35:1,467:1},Itt),g.wd=function(t){return Fxn(this,h(t,467))},g.b=0,g.c=0,g.e=0,g.f=0;var LPn=D(W1,"HyperedgeCrossingsCounter/Hyperedge",467);M(362,1,{35:1,362:1},WP),g.wd=function(t){return PCn(this,h(t,362))},g.b=0,g.c=0;var a8t=D(W1,"HyperedgeCrossingsCounter/HyperedgeCorner",362);M(523,22,{3:1,35:1,22:1,523:1},tfe);var j_,$_,o8t=tr(W1,"HyperedgeCrossingsCounter/HyperedgeCorner/Type",523,sr,lwn,Spn),c8t;M(1405,1,zo,mUe),g.Yf=function(t){return h(U(h(t,37),(He(),Uo)),21).Hc((ha(),El))?u8t:null},g.pf=function(t,n){nEn(this,h(t,37),n)};var u8t;D(da,"InteractiveNodePlacer",1405),M(1406,1,zo,wUe),g.Yf=function(t){return h(U(h(t,37),(He(),Uo)),21).Hc((ha(),El))?l8t:null},g.pf=function(t,n){$8n(this,h(t,37),n)};var l8t,cH,uH;D(da,"LinearSegmentsNodePlacer",1406),M(257,1,{35:1,257:1},Jle),g.wd=function(t){return f1n(this,h(t,257))},g.Fb=function(t){var n;return ue(t,257)?(n=h(t,257),this.b==n.b):!1},g.Hb=function(){return this.b},g.Ib=function(){return"ls"+Ag(this.e)},g.a=0,g.b=0,g.c=-1,g.d=-1,g.g=0;var h8t=D(da,"LinearSegmentsNodePlacer/LinearSegment",257);M(1408,1,zo,srt),g.Yf=function(t){return h(U(h(t,37),(He(),Uo)),21).Hc((ha(),El))?f8t:null},g.pf=function(t,n){VNn(this,h(t,37),n)},g.b=0,g.g=0;var f8t;D(da,"NetworkSimplexPlacer",1408),M(1427,1,Yr,w$e),g.ue=function(t,n){return dc(h(t,19).a,h(n,19).a)},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(da,"NetworkSimplexPlacer/0methodref$compare$Type",1427),M(1429,1,Yr,m$e),g.ue=function(t,n){return dc(h(t,19).a,h(n,19).a)},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(da,"NetworkSimplexPlacer/1methodref$compare$Type",1429),M(649,1,{649:1},HZe);var IPn=D(da,"NetworkSimplexPlacer/EdgeRep",649);M(401,1,{401:1},Ude),g.b=!1;var OPn=D(da,"NetworkSimplexPlacer/NodeRep",401);M(508,12,{3:1,4:1,20:1,28:1,52:1,12:1,14:1,15:1,54:1,508:1},XKe),D(da,"NetworkSimplexPlacer/Path",508),M(1409,1,{},y$e),g.Kb=function(t){return h(t,17).d.i.k},D(da,"NetworkSimplexPlacer/Path/lambda$0$Type",1409),M(1410,1,Cr,k$e),g.Mb=function(t){return h(t,267)==(bn(),ds)},D(da,"NetworkSimplexPlacer/Path/lambda$1$Type",1410),M(1411,1,{},x$e),g.Kb=function(t){return h(t,17).d.i},D(da,"NetworkSimplexPlacer/Path/lambda$2$Type",1411),M(1412,1,Cr,hWe),g.Mb=function(t){return btt(tft(h(t,10)))},D(da,"NetworkSimplexPlacer/Path/lambda$3$Type",1412),M(1413,1,Cr,E$e),g.Mb=function(t){return U2n(h(t,11))},D(da,"NetworkSimplexPlacer/lambda$0$Type",1413),M(1414,1,mn,zZe),g.td=function(t){zdn(this.a,this.b,h(t,11))},D(da,"NetworkSimplexPlacer/lambda$1$Type",1414),M(1423,1,mn,fWe),g.td=function(t){cTn(this.a,h(t,17))},D(da,"NetworkSimplexPlacer/lambda$10$Type",1423),M(1424,1,{},_$e),g.Kb=function(t){return Hu(),new Vt(null,new Wt(h(t,29).a,16))},D(da,"NetworkSimplexPlacer/lambda$11$Type",1424),M(1425,1,mn,dWe),g.td=function(t){iMn(this.a,h(t,10))},D(da,"NetworkSimplexPlacer/lambda$12$Type",1425),M(1426,1,{},T$e),g.Kb=function(t){return Hu(),Ke(h(t,121).e)},D(da,"NetworkSimplexPlacer/lambda$13$Type",1426),M(1428,1,{},C$e),g.Kb=function(t){return Hu(),Ke(h(t,121).e)},D(da,"NetworkSimplexPlacer/lambda$15$Type",1428),M(1430,1,Cr,S$e),g.Mb=function(t){return Hu(),h(t,401).c.k==(bn(),Wi)},D(da,"NetworkSimplexPlacer/lambda$17$Type",1430),M(1431,1,Cr,A$e),g.Mb=function(t){return Hu(),h(t,401).c.j.c.length>1},D(da,"NetworkSimplexPlacer/lambda$18$Type",1431),M(1432,1,mn,fit),g.td=function(t){Wkn(this.c,this.b,this.d,this.a,h(t,401))},g.c=0,g.d=0,D(da,"NetworkSimplexPlacer/lambda$19$Type",1432),M(1415,1,{},M$e),g.Kb=function(t){return Hu(),new Vt(null,new Wt(h(t,29).a,16))},D(da,"NetworkSimplexPlacer/lambda$2$Type",1415),M(1433,1,mn,gWe),g.td=function(t){$dn(this.a,h(t,11))},g.a=0,D(da,"NetworkSimplexPlacer/lambda$20$Type",1433),M(1434,1,{},D$e),g.Kb=function(t){return Hu(),new Vt(null,new Wt(h(t,29).a,16))},D(da,"NetworkSimplexPlacer/lambda$21$Type",1434),M(1435,1,mn,pWe),g.td=function(t){t0n(this.a,h(t,10))},D(da,"NetworkSimplexPlacer/lambda$22$Type",1435),M(1436,1,Cr,L$e),g.Mb=function(t){return btt(t)},D(da,"NetworkSimplexPlacer/lambda$23$Type",1436),M(1437,1,{},I$e),g.Kb=function(t){return Hu(),new Vt(null,new Wt(h(t,29).a,16))},D(da,"NetworkSimplexPlacer/lambda$24$Type",1437),M(1438,1,Cr,bWe),g.Mb=function(t){return odn(this.a,h(t,10))},D(da,"NetworkSimplexPlacer/lambda$25$Type",1438),M(1439,1,mn,GZe),g.td=function(t){D_n(this.a,this.b,h(t,10))},D(da,"NetworkSimplexPlacer/lambda$26$Type",1439),M(1440,1,Cr,O$e),g.Mb=function(t){return Hu(),!Xs(h(t,17))},D(da,"NetworkSimplexPlacer/lambda$27$Type",1440),M(1441,1,Cr,N$e),g.Mb=function(t){return Hu(),!Xs(h(t,17))},D(da,"NetworkSimplexPlacer/lambda$28$Type",1441),M(1442,1,{},vWe),g.Ce=function(t,n){return Zdn(this.a,h(t,29),h(n,29))},D(da,"NetworkSimplexPlacer/lambda$29$Type",1442),M(1416,1,{},P$e),g.Kb=function(t){return Hu(),new Vt(null,new cb(new Sn(Dn(qi(h(t,10)).a.Kc(),new L))))},D(da,"NetworkSimplexPlacer/lambda$3$Type",1416),M(1417,1,Cr,B$e),g.Mb=function(t){return Hu(),jmn(h(t,17))},D(da,"NetworkSimplexPlacer/lambda$4$Type",1417),M(1418,1,mn,wWe),g.td=function(t){rLn(this.a,h(t,17))},D(da,"NetworkSimplexPlacer/lambda$5$Type",1418),M(1419,1,{},F$e),g.Kb=function(t){return Hu(),new Vt(null,new Wt(h(t,29).a,16))},D(da,"NetworkSimplexPlacer/lambda$6$Type",1419),M(1420,1,Cr,R$e),g.Mb=function(t){return Hu(),h(t,10).k==(bn(),Wi)},D(da,"NetworkSimplexPlacer/lambda$7$Type",1420),M(1421,1,{},j$e),g.Kb=function(t){return Hu(),new Vt(null,new cb(new Sn(Dn(Id(h(t,10)).a.Kc(),new L))))},D(da,"NetworkSimplexPlacer/lambda$8$Type",1421),M(1422,1,Cr,$$e),g.Mb=function(t){return Hu(),j2n(h(t,17))},D(da,"NetworkSimplexPlacer/lambda$9$Type",1422),M(1404,1,zo,PUe),g.Yf=function(t){return h(U(h(t,37),(He(),Uo)),21).Hc((ha(),El))?d8t:null},g.pf=function(t,n){RLn(h(t,37),n)};var d8t;D(da,"SimpleNodePlacer",1404),M(180,1,{180:1},jy),g.Ib=function(){var t;return t="",this.c==(r1(),lv)?t+=m5:this.c==I0&&(t+=w5),this.o==(yf(),v2)?t+=ZZ:this.o==Rf?t+="UP":t+="BALANCED",t},D(Bg,"BKAlignedLayout",180),M(516,22,{3:1,35:1,22:1,516:1},rfe);var I0,lv,g8t=tr(Bg,"BKAlignedLayout/HDirection",516,sr,fwn,Apn),p8t;M(515,22,{3:1,35:1,22:1,515:1},nfe);var v2,Rf,b8t=tr(Bg,"BKAlignedLayout/VDirection",515,sr,dwn,Mpn),v8t;M(1634,1,{},VZe),D(Bg,"BKAligner",1634),M(1637,1,{},T1t),D(Bg,"BKCompactor",1637),M(654,1,{654:1},H$e),g.a=0,D(Bg,"BKCompactor/ClassEdge",654),M(458,1,{458:1},WKe),g.a=null,g.b=0,D(Bg,"BKCompactor/ClassNode",458),M(1407,1,zo,bJe),g.Yf=function(t){return h(U(h(t,37),(He(),Uo)),21).Hc((ha(),El))?w8t:null},g.pf=function(t,n){sPn(this,h(t,37),n)},g.d=!1;var w8t;D(Bg,"BKNodePlacer",1407),M(1635,1,{},z$e),g.d=0,D(Bg,"NeighborhoodInformation",1635),M(1636,1,Yr,mWe),g.ue=function(t,n){return f4n(this,h(t,46),h(n,46))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Bg,"NeighborhoodInformation/NeighborComparator",1636),M(808,1,{}),D(Bg,"ThresholdStrategy",808),M(1763,808,{},JKe),g.bg=function(t,n,s){return this.a.o==(yf(),Rf)?ki:Fi},g.cg=function(){},D(Bg,"ThresholdStrategy/NullThresholdStrategy",1763),M(579,1,{579:1},UZe),g.c=!1,g.d=!1,D(Bg,"ThresholdStrategy/Postprocessable",579),M(1764,808,{},eXe),g.bg=function(t,n,s){var a,u,p;return u=n==s,a=this.a.a[s.p]==n,u||a?(p=t,this.a.c==(r1(),lv)?(u&&(p=ZQ(this,n,!0)),!isNaN(p)&&!isFinite(p)&&a&&(p=ZQ(this,s,!1))):(u&&(p=ZQ(this,n,!0)),!isNaN(p)&&!isFinite(p)&&a&&(p=ZQ(this,s,!1))),p):t},g.cg=function(){for(var t,n,s,a,u;this.d.b!=0;)u=h(Mwn(this.d),579),a=qpt(this,u),a.a&&(t=a.a,s=ft(this.a.f[this.a.g[u.b.p].p]),!(!s&&!Xs(t)&&t.c.i.c==t.d.i.c)&&(n=J0t(this,u),n||pdn(this.e,u)));for(;this.e.a.c.length!=0;)J0t(this,h(uht(this.e),579))},D(Bg,"ThresholdStrategy/SimpleThresholdStrategy",1764),M(635,1,{635:1,246:1,234:1},G$e),g.Kf=function(){return mlt(this)},g.Xf=function(){return mlt(this)};var Zne;D(WJ,"EdgeRouterFactory",635),M(1458,1,zo,BUe),g.Yf=function(t){return FAn(h(t,37))},g.pf=function(t,n){ULn(h(t,37),n)};var m8t,y8t,k8t,x8t,E8t,vxe,_8t,T8t;D(WJ,"OrthogonalEdgeRouter",1458),M(1451,1,zo,vJe),g.Yf=function(t){return oEn(h(t,37))},g.pf=function(t,n){hNn(this,h(t,37),n)};var C8t,S8t,A8t,M8t,$D,D8t;D(WJ,"PolylineEdgeRouter",1451),M(1452,1,c1,V$e),g.Lb=function(t){return Ige(h(t,10))},g.Fb=function(t){return this===t},g.Mb=function(t){return Ige(h(t,10))},D(WJ,"PolylineEdgeRouter/1",1452),M(1809,1,Cr,U$e),g.Mb=function(t){return h(t,129).c==(qh(),qg)},D(ef,"HyperEdgeCycleDetector/lambda$0$Type",1809),M(1810,1,{},q$e),g.Ge=function(t){return h(t,129).d},D(ef,"HyperEdgeCycleDetector/lambda$1$Type",1810),M(1811,1,Cr,Y$e),g.Mb=function(t){return h(t,129).c==(qh(),qg)},D(ef,"HyperEdgeCycleDetector/lambda$2$Type",1811),M(1812,1,{},W$e),g.Ge=function(t){return h(t,129).d},D(ef,"HyperEdgeCycleDetector/lambda$3$Type",1812),M(1813,1,{},K$e),g.Ge=function(t){return h(t,129).d},D(ef,"HyperEdgeCycleDetector/lambda$4$Type",1813),M(1814,1,{},X$e),g.Ge=function(t){return h(t,129).d},D(ef,"HyperEdgeCycleDetector/lambda$5$Type",1814),M(112,1,{35:1,112:1},PA),g.wd=function(t){return d1n(this,h(t,112))},g.Fb=function(t){var n;return ue(t,112)?(n=h(t,112),this.g==n.g):!1},g.Hb=function(){return this.g},g.Ib=function(){var t,n,s,a;for(t=new ju("{"),a=new Q(this.n);a.a<a.c.c.length;)s=h(K(a),11),n=iM(s.i),n==null&&(n="n"+Ytt(s.i)),t.a+=""+n,a.a<a.c.c.length&&(t.a+=",");return t.a+="}",t.a},g.a=0,g.b=0,g.c=NaN,g.d=0,g.g=0,g.i=0,g.o=0,g.s=NaN,D(ef,"HyperEdgeSegment",112),M(129,1,{129:1},Ip),g.Ib=function(){return this.a+"->"+this.b+" ("+_0n(this.c)+")"},g.d=0,D(ef,"HyperEdgeSegmentDependency",129),M(520,22,{3:1,35:1,22:1,520:1},ife);var qg,fm,L8t=tr(ef,"HyperEdgeSegmentDependency/DependencyType",520,sr,hwn,Dpn),I8t;M(1815,1,{},yWe),D(ef,"HyperEdgeSegmentSplitter",1815),M(1816,1,{},QXe),g.a=0,g.b=0,D(ef,"HyperEdgeSegmentSplitter/AreaRating",1816),M(329,1,{329:1},ZY),g.a=0,g.b=0,g.c=0,D(ef,"HyperEdgeSegmentSplitter/FreeArea",329),M(1817,1,Yr,iHe),g.ue=function(t,n){return ygn(h(t,112),h(n,112))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(ef,"HyperEdgeSegmentSplitter/lambda$0$Type",1817),M(1818,1,mn,dit),g.td=function(t){Tyn(this.a,this.d,this.c,this.b,h(t,112))},g.b=0,D(ef,"HyperEdgeSegmentSplitter/lambda$1$Type",1818),M(1819,1,{},sHe),g.Kb=function(t){return new Vt(null,new Wt(h(t,112).e,16))},D(ef,"HyperEdgeSegmentSplitter/lambda$2$Type",1819),M(1820,1,{},aHe),g.Kb=function(t){return new Vt(null,new Wt(h(t,112).j,16))},D(ef,"HyperEdgeSegmentSplitter/lambda$3$Type",1820),M(1821,1,{},oHe),g.Fe=function(t){return Pe(Je(t))},D(ef,"HyperEdgeSegmentSplitter/lambda$4$Type",1821),M(655,1,{},xW),g.a=0,g.b=0,g.c=0,D(ef,"OrthogonalRoutingGenerator",655),M(1638,1,{},cHe),g.Kb=function(t){return new Vt(null,new Wt(h(t,112).e,16))},D(ef,"OrthogonalRoutingGenerator/lambda$0$Type",1638),M(1639,1,{},uHe),g.Kb=function(t){return new Vt(null,new Wt(h(t,112).j,16))},D(ef,"OrthogonalRoutingGenerator/lambda$1$Type",1639),M(661,1,{}),D(KJ,"BaseRoutingDirectionStrategy",661),M(1807,661,{},tXe),g.dg=function(t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J;if(!(!!t.r&&!t.q))for(A=n+t.o*s,_=new Q(t.n);_.a<_.c.c.length;)for(x=h(K(_),11),I=Qa(se(ie(cs,1),Re,8,0,[x.i.n,x.n,x.a])).a,m=new Q(x.g);m.a<m.c.c.length;)v=h(K(m),17),Xs(v)||(V=v.d,J=Qa(se(ie(cs,1),Re,8,0,[V.i.n,V.n,V.a])).a,d.Math.abs(I-J)>l1&&(p=A,u=t,a=new dt(I,p),vr(v.a,a),Rb(this,v,u,a,!1),P=t.r,P&&(j=Pe(Je(Zh(P.e,0))),a=new dt(j,p),vr(v.a,a),Rb(this,v,u,a,!1),p=n+P.o*s,u=P,a=new dt(j,p),vr(v.a,a),Rb(this,v,u,a,!1)),a=new dt(J,p),vr(v.a,a),Rb(this,v,u,a,!1)))},g.eg=function(t){return t.i.n.a+t.n.a+t.a.a},g.fg=function(){return Qe(),Fn},g.gg=function(){return Qe(),sn},D(KJ,"NorthToSouthRoutingStrategy",1807),M(1808,661,{},nXe),g.dg=function(t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J;if(!(!!t.r&&!t.q))for(A=n-t.o*s,_=new Q(t.n);_.a<_.c.c.length;)for(x=h(K(_),11),I=Qa(se(ie(cs,1),Re,8,0,[x.i.n,x.n,x.a])).a,m=new Q(x.g);m.a<m.c.c.length;)v=h(K(m),17),Xs(v)||(V=v.d,J=Qa(se(ie(cs,1),Re,8,0,[V.i.n,V.n,V.a])).a,d.Math.abs(I-J)>l1&&(p=A,u=t,a=new dt(I,p),vr(v.a,a),Rb(this,v,u,a,!1),P=t.r,P&&(j=Pe(Je(Zh(P.e,0))),a=new dt(j,p),vr(v.a,a),Rb(this,v,u,a,!1),p=n-P.o*s,u=P,a=new dt(j,p),vr(v.a,a),Rb(this,v,u,a,!1)),a=new dt(J,p),vr(v.a,a),Rb(this,v,u,a,!1)))},g.eg=function(t){return t.i.n.a+t.n.a+t.a.a},g.fg=function(){return Qe(),sn},g.gg=function(){return Qe(),Fn},D(KJ,"SouthToNorthRoutingStrategy",1808),M(1806,661,{},rXe),g.dg=function(t,n,s){var a,u,p,v,m,x,_,A,I,P,j,V,J;if(!(!!t.r&&!t.q))for(A=n+t.o*s,_=new Q(t.n);_.a<_.c.c.length;)for(x=h(K(_),11),I=Qa(se(ie(cs,1),Re,8,0,[x.i.n,x.n,x.a])).b,m=new Q(x.g);m.a<m.c.c.length;)v=h(K(m),17),Xs(v)||(V=v.d,J=Qa(se(ie(cs,1),Re,8,0,[V.i.n,V.n,V.a])).b,d.Math.abs(I-J)>l1&&(p=A,u=t,a=new dt(p,I),vr(v.a,a),Rb(this,v,u,a,!0),P=t.r,P&&(j=Pe(Je(Zh(P.e,0))),a=new dt(p,j),vr(v.a,a),Rb(this,v,u,a,!0),p=n+P.o*s,u=P,a=new dt(p,j),vr(v.a,a),Rb(this,v,u,a,!0)),a=new dt(p,J),vr(v.a,a),Rb(this,v,u,a,!0)))},g.eg=function(t){return t.i.n.b+t.n.b+t.a.b},g.fg=function(){return Qe(),pn},g.gg=function(){return Qe(),un},D(KJ,"WestToEastRoutingStrategy",1806),M(813,1,{},_ve),g.Ib=function(){return Ag(this.a)},g.b=0,g.c=!1,g.d=!1,g.f=0,D(Vw,"NubSpline",813),M(407,1,{407:1},Agt,Yit),D(Vw,"NubSpline/PolarCP",407),M(1453,1,zo,v1t),g.Yf=function(t){return XEn(h(t,37))},g.pf=function(t,n){DNn(this,h(t,37),n)};var O8t,N8t,P8t,B8t,F8t;D(Vw,"SplineEdgeRouter",1453),M(268,1,{268:1},yB),g.Ib=function(){return this.a+" ->("+this.c+") "+this.b},g.c=0,D(Vw,"SplineEdgeRouter/Dependency",268),M(455,22,{3:1,35:1,22:1,455:1},sfe);var Yg,m3,R8t=tr(Vw,"SplineEdgeRouter/SideToProcess",455,sr,gwn,Lpn),j8t;M(1454,1,Cr,nHe),g.Mb=function(t){return x9(),!h(t,128).o},D(Vw,"SplineEdgeRouter/lambda$0$Type",1454),M(1455,1,{},tHe),g.Ge=function(t){return x9(),h(t,128).v+1},D(Vw,"SplineEdgeRouter/lambda$1$Type",1455),M(1456,1,mn,qZe),g.td=function(t){H2n(this.a,this.b,h(t,46))},D(Vw,"SplineEdgeRouter/lambda$2$Type",1456),M(1457,1,mn,YZe),g.td=function(t){z2n(this.a,this.b,h(t,46))},D(Vw,"SplineEdgeRouter/lambda$3$Type",1457),M(128,1,{35:1,128:1},Udt,Dve),g.wd=function(t){return g1n(this,h(t,128))},g.b=0,g.e=!1,g.f=0,g.g=0,g.j=!1,g.k=!1,g.n=0,g.o=!1,g.p=!1,g.q=!1,g.s=0,g.u=0,g.v=0,g.F=0,D(Vw,"SplineSegment",128),M(459,1,{459:1},rHe),g.a=0,g.b=!1,g.c=!1,g.d=!1,g.e=!1,g.f=0,D(Vw,"SplineSegment/EdgeInformation",459),M(1234,1,{},Q$e),D(W9,Twe,1234),M(1235,1,Yr,Z$e),g.ue=function(t,n){return _Tn(h(t,135),h(n,135))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(W9,Mvt,1235),M(1233,1,{},dQe),D(W9,"MrTree",1233),M(393,22,{3:1,35:1,22:1,393:1,246:1,234:1},jN),g.Kf=function(){return b0t(this)},g.Xf=function(){return b0t(this)};var lH,H_,HD,z_,wxe=tr(W9,"TreeLayoutPhases",393,sr,ayn,Ipn),$8t;M(1130,209,t2,Ptt),g.Ze=function(t,n){var s,a,u,p,v,m,x;for(ft(lt(pt(t,(Fb(),_xe))))||nB((s=new hS((Qv(),new Yv(t))),s)),v=(m=new kB,Oa(m,t),Fe(m,(fo(),U_),t),x=new zn,pDn(t,m,x),IDn(t,m,x),m),p=TDn(this.a,v),u=new Q(p);u.a<u.c.c.length;)a=h(K(u),135),UEn(this.b,a,jo(n,1/p.c.length));v=iPn(p),DOn(v)},D(W9,"TreeLayoutProvider",1130),M(1847,1,G1,J$e),g.Jc=function(t){Os(this,t)},g.Kc=function(){return Rt(),K6(),Vx},D(W9,"TreeUtil/1",1847),M(1848,1,G1,eHe),g.Jc=function(t){Os(this,t)},g.Kc=function(){return Rt(),K6(),Vx},D(W9,"TreeUtil/2",1848),M(502,134,{3:1,502:1,94:1,134:1}),g.g=0,D(K9,"TGraphElement",502),M(188,502,{3:1,188:1,502:1,94:1,134:1},Wde),g.Ib=function(){return!!this.b&&!!this.c?uB(this.b)+"->"+uB(this.c):"e_"+ii(this)},D(K9,"TEdge",188),M(135,134,{3:1,135:1,94:1,134:1},kB),g.Ib=function(){var t,n,s,a,u;for(u=null,a=pr(this.b,0);a.b!=a.d.c;)s=h(gr(a),86),u+=(s.c==null||s.c.length==0?"n_"+s.g:"n_"+s.c)+`
`;for(n=pr(this.a,0);n.b!=n.d.c;)t=h(gr(n),188),u+=(!!t.b&&!!t.c?uB(t.b)+"->"+uB(t.c):"e_"+ii(t))+`
`;return u};var NPn=D(K9,"TGraph",135);M(633,502,{3:1,502:1,633:1,94:1,134:1}),D(K9,"TShape",633),M(86,633,{3:1,502:1,86:1,633:1,94:1,134:1},ZK),g.Ib=function(){return uB(this)};var PPn=D(K9,"TNode",86);M(255,1,G1,ng),g.Jc=function(t){Os(this,t)},g.Kc=function(){var t;return t=pr(this.a.d,0),new k4(t)},D(K9,"TNode/2",255),M(358,1,ys,k4),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return h(gr(this.a),188).c},g.Ob=function(){return pN(this.a)},g.Qb=function(){BB(this.a)},D(K9,"TNode/2/1",358),M(1840,1,xi,Ntt),g.pf=function(t,n){WDn(this,h(t,135),n)},D(C5,"FanProcessor",1840),M(327,22,{3:1,35:1,22:1,327:1,234:1},aE),g.Kf=function(){switch(this.g){case 0:return new wXe;case 1:return new Ntt;case 2:return new fHe;case 3:return new lHe;case 4:return new gHe;case 5:return new pHe;default:throw ne(new an(pJ+(this.f!=null?this.f:""+this.g)))}};var Jne,ere,tre,nre,rre,hH,H8t=tr(C5,Fwe,327,sr,x3n,Opn),z8t;M(1843,1,xi,lHe),g.pf=function(t,n){DCn(this,h(t,135),n)},g.a=0,D(C5,"LevelHeightProcessor",1843),M(1844,1,G1,hHe),g.Jc=function(t){Os(this,t)},g.Kc=function(){return Rt(),K6(),Vx},D(C5,"LevelHeightProcessor/1",1844),M(1841,1,xi,fHe),g.pf=function(t,n){F_n(this,h(t,135),n)},g.a=0,D(C5,"NeighborsProcessor",1841),M(1842,1,G1,dHe),g.Jc=function(t){Os(this,t)},g.Kc=function(){return Rt(),K6(),Vx},D(C5,"NeighborsProcessor/1",1842),M(1845,1,xi,gHe),g.pf=function(t,n){MCn(this,h(t,135),n)},g.a=0,D(C5,"NodePositionProcessor",1845),M(1839,1,xi,wXe),g.pf=function(t,n){tIn(this,h(t,135))},D(C5,"RootProcessor",1839),M(1846,1,xi,pHe),g.pf=function(t,n){lkn(h(t,135))},D(C5,"Untreeifyer",1846);var zD,G_,G8t,ire,fH,V_,sre,dH,gH,g8,U_,pH,O0,mxe,V8t,are,dm,ore,yxe;M(851,1,$l,NUe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,nye),""),"Weighting of Nodes"),"Which weighting to use when computing a node order."),xxe),(E0(),_i)),Mxe),Lt((Qh(),gn))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,rye),""),"Search Order"),"Which search order to use when computing a spanning tree."),kxe),_i),Lxe),Lt(gn)))),z2t((new OUe,t))};var U8t,kxe,q8t,xxe;D(sj,"MrTreeMetaDataProvider",851),M(994,1,$l,OUe),g.Qe=function(t){z2t(t)};var Y8t,Exe,W8t,K8t,X8t,Q8t,_xe,Z8t,Txe,J8t,bH,Cxe,e7t,Sxe,t7t;D(sj,"MrTreeOptions",994),M(995,1,{},bHe),g.$e=function(){var t;return t=new Ptt,t},g._e=function(t){},D(sj,"MrTreeOptions/MrtreeFactory",995),M(480,22,{3:1,35:1,22:1,480:1},afe);var cre,Axe,Mxe=tr(sj,"OrderWeighting",480,sr,bwn,Npn),n7t;M(425,22,{3:1,35:1,22:1,425:1},ofe);var Dxe,ure,Lxe=tr(sj,"TreeifyingOrder",425,sr,pwn,Bpn),r7t;M(1459,1,zo,_Ue),g.Yf=function(t){return h(t,135),i7t},g.pf=function(t,n){u5n(this,h(t,135),n)};var i7t;D("org.eclipse.elk.alg.mrtree.p1treeify","DFSTreeifyer",1459),M(1460,1,zo,TUe),g.Yf=function(t){return h(t,135),s7t},g.pf=function(t,n){Y_n(this,h(t,135),n)};var s7t;D("org.eclipse.elk.alg.mrtree.p2order","NodeOrderer",1460),M(1461,1,zo,EUe),g.Yf=function(t){return h(t,135),a7t},g.pf=function(t,n){lMn(this,h(t,135),n)},g.a=0;var a7t;D("org.eclipse.elk.alg.mrtree.p3place","NodePlacer",1461),M(1462,1,zo,CUe),g.Yf=function(t){return h(t,135),o7t},g.pf=function(t,n){B7n(h(t,135),n)};var o7t;D("org.eclipse.elk.alg.mrtree.p4route","EdgeRouter",1462);var q_;M(495,22,{3:1,35:1,22:1,495:1,246:1,234:1},cfe),g.Kf=function(){return Wht(this)},g.Xf=function(){return Wht(this)};var vH,p8,Ixe=tr(iye,"RadialLayoutPhases",495,sr,vwn,Ppn),c7t;M(1131,209,t2,fQe),g.Ze=function(t,n){var s,a,u,p,v,m;if(s=Qdt(this,t),Bn(n,"Radial layout",s.c.length),ft(lt(pt(t,(Cw(),zxe))))||nB((a=new hS((Qv(),new Yv(t))),a)),m=JEn(t),ya(t,(xE(),q_),m),!m)throw ne(new an("The given graph is not a tree!"));for(u=Pe(Je(pt(t,yH))),u==0&&(u=h0t(t)),ya(t,yH,u),v=new Q(Qdt(this,t));v.a<v.c.c.length;)p=h(K(v),51),p.pf(t,jo(n,1));An(n)},D(iye,"RadialLayoutProvider",1131),M(549,1,Yr,bN),g.ue=function(t,n){return TMn(this.a,this.b,h(t,33),h(n,33))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},g.a=0,g.b=0,D(iye,"RadialUtil/lambda$0$Type",549),M(1375,1,xi,wHe),g.pf=function(t,n){VIn(h(t,33),n)},D(xwt,"CalculateGraphSize",1375),M(442,22,{3:1,35:1,22:1,442:1,234:1},gY),g.Kf=function(){switch(this.g){case 0:return new EHe;case 1:return new vHe;case 2:return new wHe;default:throw ne(new an(pJ+(this.f!=null?this.f:""+this.g)))}};var lre,hre,fre,u7t=tr(xwt,Fwe,442,sr,fmn,Fpn),l7t;M(645,1,{}),g.e=1,g.g=0,D(ZJ,"AbstractRadiusExtensionCompaction",645),M(1772,645,{},ftt),g.hg=function(t){var n,s,a,u,p,v,m,x,_;for(this.c=h(pt(t,(xE(),q_)),33),Khn(this,this.c),this.d=BX(h(pt(t,(Cw(),GD)),293)),x=h(pt(t,gre),19),x&&mqe(this,x.a),m=Je(pt(t,(Tr(),k2))),ple(this,(rn(m),m)),_=Nw(this.c),this.d&&this.d.lg(_),$Mn(this,_),v=new _u(se(ie(wi,1),Ewt,33,0,[this.c])),s=0;s<2;s++)for(n=0;n<_.c.length;n++)u=new _u(se(ie(wi,1),Ewt,33,0,[(Kt(n,_.c.length),h(_.c[n],33))])),p=n<_.c.length-1?(Kt(n+1,_.c.length),h(_.c[n+1],33)):(Kt(0,_.c.length),h(_.c[0],33)),a=n==0?h(ut(_,_.c.length-1),33):(Kt(n-1,_.c.length),h(_.c[n-1],33)),idt(this,(Kt(n,_.c.length),h(_.c[n],33),v),a,p,u)},D(ZJ,"AnnulusWedgeCompaction",1772),M(1374,1,xi,vHe),g.pf=function(t,n){rEn(h(t,33),n)},D(ZJ,"GeneralCompactor",1374),M(1771,645,{},mHe),g.hg=function(t){var n,s,a,u;s=h(pt(t,(xE(),q_)),33),this.f=s,this.b=BX(h(pt(t,(Cw(),GD)),293)),u=h(pt(t,gre),19),u&&mqe(this,u.a),a=Je(pt(t,(Tr(),k2))),ple(this,(rn(a),a)),n=Nw(s),this.b&&this.b.lg(n),Pdt(this,n)},g.a=0,D(ZJ,"RadialCompaction",1771),M(1779,1,{},yHe),g.ig=function(t){var n,s,a,u,p,v;for(this.a=t,n=0,v=Nw(t),a=0,p=new Q(v);p.a<p.c.c.length;)for(u=h(K(p),33),++a,s=a;s<v.c.length;s++)eLn(this,u,(Kt(s,v.c.length),h(v.c[s],33)))&&(n+=1);return n},D(aye,"CrossingMinimizationPosition",1779),M(1777,1,{},kHe),g.ig=function(t){var n,s,a,u,p,v,m,x,_,A,I,P,j;for(a=0,s=new Sn(Dn(Pd(t).a.Kc(),new L));rr(s);)n=h(Un(s),79),m=Na(h(he((!n.c&&(n.c=new qt(Pn,n,5,8)),n.c),0),82)),_=m.i+m.g/2,A=m.j+m.f/2,u=t.i+t.g/2,p=t.j+t.f/2,I=new Rs,I.a=_-u,I.b=A-p,v=new dt(I.a,I.b),rx(v,t.g,t.f),I.a-=v.a,I.b-=v.b,u=_-I.a,p=A-I.b,x=new dt(I.a,I.b),rx(x,m.g,m.f),I.a-=x.a,I.b-=x.b,_=u+I.a,A=p+I.b,P=_-u,j=A-p,a+=d.Math.sqrt(P*P+j*j);return a},D(aye,"EdgeLengthOptimization",1777),M(1778,1,{},xHe),g.ig=function(t){var n,s,a,u,p,v,m,x,_,A,I;for(a=0,s=new Sn(Dn(Pd(t).a.Kc(),new L));rr(s);)n=h(Un(s),79),m=Na(h(he((!n.c&&(n.c=new qt(Pn,n,5,8)),n.c),0),82)),x=m.i+m.g/2,_=m.j+m.f/2,u=h(pt(m,(Tr(),q5)),8),p=t.i+u.a+t.g/2,v=t.j+u.b+t.f,A=x-p,I=_-v,a+=d.Math.sqrt(A*A+I*I);return a},D(aye,"EdgeLengthPositionOptimization",1778),M(1373,645,xi,EHe),g.pf=function(t,n){oSn(this,h(t,33),n)},D("org.eclipse.elk.alg.radial.intermediate.overlaps","RadiusExtensionOverlapRemoval",1373),M(426,22,{3:1,35:1,22:1,426:1},ufe);var Oxe,dre,Nxe=tr(X9,"AnnulusWedgeCriteria",426,sr,wwn,Rpn),h7t;M(380,22,{3:1,35:1,22:1,380:1},pY);var wH,Pxe,Bxe,Fxe=tr(X9,wwe,380,sr,dmn,jpn),f7t;M(852,1,$l,SUe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,oye),""),"Order ID"),"The id can be used to define an order for nodes of one radius. This can be used to sort them in the layer accordingly."),Ke(0)),(E0(),po)),Ws),Lt((Qh(),gs))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,cye),""),"Radius"),"The radius option can be used to set the initial radius for the radial layouter."),0),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,eee),""),"Compaction"),"With the compacter option it can be determined how compaction on the graph is done. It can be chosen between none, the radial compaction or the compaction of wedges separately."),Rxe),_i),Fxe),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,tee),""),"Compaction Step Size"),"Determine the size of steps with which the compaction is done. Step size 1 correlates to a compaction of 1 pixel per Iteration."),Ke(1)),po),Ws),Lt(gn)))),Es(t,tee,eee,null),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,uye),""),"Sorter"),"Sort the nodes per radius according to the sorting algorithm. The strategies are none, by the given order id, or sorting them by polar coordinates."),$xe),_i),Qxe),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,lye),""),"Annulus Wedge Criteria"),"Determine how the wedge for the node placement is calculated. It can be chosen between wedge determination by the number of leaves or by the maximum sum of diagonals."),Hxe),_i),Nxe),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,hye),""),"Translation Optimization"),"Find the optimal translation of the nodes of the first radii according to this criteria. For example edge crossings can be minimized."),jxe),_i),Wxe),Lt(gn)))),rbt((new AUe,t))};var d7t,g7t,Rxe,p7t,jxe,b7t,v7t,w7t,$xe,m7t,Hxe;D(X9,"RadialMetaDataProvider",852),M(996,1,$l,AUe),g.Qe=function(t){rbt(t)};var gre,pre,y7t,k7t,x7t,E7t,zxe,Gxe,mH,_7t,T7t,yH,GD,C7t,Vxe;D(X9,"RadialOptions",996),M(997,1,{},_He),g.$e=function(){var t;return t=new fQe,t},g._e=function(t){},D(X9,"RadialOptions/RadialFactory",997),M(340,22,{3:1,35:1,22:1,340:1},$N);var Uxe,qxe,Yxe,bre,Wxe=tr(X9,"RadialTranslationStrategy",340,sr,oyn,$pn),S7t;M(293,22,{3:1,35:1,22:1,293:1},bY);var Kxe,vre,Xxe,Qxe=tr(X9,"SortingStrategy",293,sr,pmn,Hpn),A7t;M(1449,1,zo,THe),g.Yf=function(t){return h(t,33),null},g.pf=function(t,n){bSn(this,h(t,33),n)},g.c=0,D("org.eclipse.elk.alg.radial.p1position","EadesRadial",1449),M(1775,1,{},CHe),g.jg=function(t){return Gft(t)},D(_wt,"AnnulusWedgeByLeafs",1775),M(1776,1,{},SHe),g.jg=function(t){return X1t(this,t)},D(_wt,"AnnulusWedgeByNodeSpace",1776),M(1450,1,zo,AHe),g.Yf=function(t){return h(t,33),null},g.pf=function(t,n){AEn(this,h(t,33),n)},D("org.eclipse.elk.alg.radial.p2routing","StraightLineEdgeRouter",1450),M(811,1,{},lhe),g.kg=function(t){},g.lg=function(t){AKe(this,t)},D(fye,"IDSorter",811),M(1774,1,Yr,MHe),g.ue=function(t,n){return Z4n(h(t,33),h(n,33))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(fye,"IDSorter/lambda$0$Type",1774),M(1773,1,{},Yut),g.kg=function(t){eat(this,t)},g.lg=function(t){var n;t.dc()||(this.e||(n=Xnt(h(t.Xb(0),33)),eat(this,n)),AKe(this.e,t))},D(fye,"PolarCoordinateSorter",1773),M(1136,209,t2,DHe),g.Ze=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt,ln,dn,Ln;if(Bn(n,"Rectangle Packing",1),n.n&&n.n&&!!t&&mh(n,yh(t),(Lu(),rl)),s=Pe(Je(pt(t,(CQ(),R7t)))),J=h(pt(t,l8e),381),le=ft(lt(pt(t,o8e))),rt=ft(lt(pt(t,u8e))),I=ft(lt(pt(t,i8e))),wt=h(pt(t,U7t),116),Ve=Pe(Je(pt(t,Y7t))),u=ft(lt(pt(t,f8e))),P=ft(lt(pt(t,s8e))),fe=ft(lt(pt(t,a8e))),Ln=Pe(Je(pt(t,d8e))),ln=(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a),qut(ln),fe){for(V=new qe,x=new Tn(ln);x.e!=x.i.gc();)v=h(On(x),33),Pp(v,VD)&&(V.c[V.c.length]=v);for(_=new Q(V);_.a<_.c.c.length;)v=h(K(_),33),IB(ln,v);for(Rt(),hs(V,new LHe),A=new Q(V);A.a<A.c.c.length;)v=h(K(A),33),xt=h(pt(v,VD),19).a,xt=d.Math.min(xt,ln.i),s9(ln,xt,v);for(oe=0,m=new Tn(ln);m.e!=m.i.gc();)v=h(On(m),33),ya(v,r8e,Ke(oe)),++oe}Ne=ube(t),Ne.a-=wt.b+wt.c,Ne.b-=wt.d+wt.a,we=Ne.a,Ln<0||Ln<Ne.a?(j=new unt(s,J,le),p=YMn(j,ln,Ve,wt),n.n&&n.n&&!!t&&mh(n,yh(t),(Lu(),rl))):p=new nW(s,Ln,0,(fx(),K_)),Ne.a+=wt.b+wt.c,Ne.b+=wt.d+wt.a,rt||(qut(ln),dn=new yat(s,I,P,u,Ve),we=d.Math.max(Ne.a,p.c),p=TOn(dn,ln,we,Ne,n,t,wt)),_5n(ln,wt),$b(t,p.c+(wt.b+wt.c),p.b+(wt.d+wt.a),!1,!0),ft(lt(pt(t,c8e)))||nB((a=new hS((Qv(),new Yv(t))),a)),n.n&&n.n&&!!t&&mh(n,yh(t),(Lu(),rl)),An(n)},D(Awt,"RectPackingLayoutProvider",1136),M(1137,1,Yr,LHe),g.ue=function(t,n){return p6n(h(t,33),h(n,33))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Awt,"RectPackingLayoutProvider/lambda$0$Type",1137),M(1256,1,{},unt),g.a=0,g.c=!1,D(aj,"AreaApproximation",1256);var Zxe=fi(aj,"BestCandidateFilter");M(638,1,{526:1},oq),g.mg=function(t,n,s){var a,u,p,v,m,x;for(x=new qe,p=ki,m=new Q(t);m.a<m.c.c.length;)v=h(K(m),220),p=d.Math.min(p,(v.c+(s.b+s.c))*(v.b+(s.d+s.a)));for(u=new Q(t);u.a<u.c.c.length;)a=h(K(u),220),(a.c+(s.b+s.c))*(a.b+(s.d+s.a))==p&&(x.c[x.c.length]=a);return x},D(aj,"AreaFilter",638),M(639,1,{526:1},cq),g.mg=function(t,n,s){var a,u,p,v,m,x;for(m=new qe,x=ki,v=new Q(t);v.a<v.c.c.length;)p=h(K(v),220),x=d.Math.min(x,d.Math.abs((p.c+(s.b+s.c))/(p.b+(s.d+s.a))-n));for(u=new Q(t);u.a<u.c.c.length;)a=h(K(u),220),d.Math.abs((a.c+(s.b+s.c))/(a.b+(s.d+s.a))-n)==x&&(m.c[m.c.length]=a);return m},D(aj,"AspectRatioFilter",639),M(637,1,{526:1},uq),g.mg=function(t,n,s){var a,u,p,v,m,x;for(x=new qe,p=Fi,m=new Q(t);m.a<m.c.c.length;)v=h(K(m),220),p=d.Math.max(p,ode(v.c+(s.b+s.c),v.b+(s.d+s.a),v.a));for(u=new Q(t);u.a<u.c.c.length;)a=h(K(u),220),ode(a.c+(s.b+s.c),a.b+(s.d+s.a),a.a)==p&&(x.c[x.c.length]=a);return x},D(aj,"ScaleMeasureFilter",637),M(381,22,{3:1,35:1,22:1,381:1},vY);var Jxe,e8e,wre,t8e=tr(nee,"OptimizationGoal",381,sr,gmn,zpn),M7t;M(856,1,$l,FUe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,dye),""),"Optimization Goal"),"Optimization goal for approximation of the bounding box given by the first iteration. Determines whether layout is sorted by the maximum scaling, aspect ratio, or area. Depending on the strategy the aspect ratio might be nearly ignored."),n8e),(E0(),_i)),t8e),Lt((Qh(),gs))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,gye),""),"Shift Last Placed."),"When placing a rectangle behind or below the last placed rectangle in the first iteration, it is sometimes possible to shift the rectangle further to the left or right, resulting in less whitespace. True (default) enables the shift and false disables it. Disabling the shift produces a greater approximated area by the first iteration and a layout, when using ONLY the first iteration (default not the case), where it is sometimes impossible to implement a size transformation of rectangles that will fill the bounding box and eliminate empty spaces."),(on(),!0)),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,pye),""),"Current position of a node in the order of nodes"),"The rectangles are ordered. Normally according to their definition the the model. This option specifies the current position of a node."),Ke(-1)),po),Ws),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,bye),""),"Desired index of node"),"The rectangles are ordered. Normally according to their definition the the model. This option allows to specify a desired position that has preference over the original position."),Ke(-1)),po),Ws),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,vye),""),"Only Area Approximation"),"If enabled only the width approximation step is executed and the nodes are placed accordingly. The nodes are layouted according to the packingStrategy. If set to true not expansion of nodes is taking place."),!1),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,wye),""),"Compact Rows"),"Enables compaction. Compacts blocks if they do not use the full height of the row. This option allows to have a smaller drawing. If this option is disabled all nodes are placed next to each other in rows."),!0),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ree),""),"Fit Aspect Ratio"),"Expands nodes if expandNodes is true to fit the aspect ratio instead of only in their bounds. The option is only useful if the used packingStrategy is ASPECT_RATIO_DRIVEN, otherwise this may result in unreasonable ndoe expansion."),!1),zs),es),Lt(gs)))),Es(t,ree,oj,null),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,mye),""),"Target Width"),"Option to place the rectangles in the given target width instead of approximating the width using the desired aspect ratio. The padding is not included in this. Meaning a drawing will have width of targetwidth + horizontal padding."),-1),Ba),Ts),Lt(gs)))),Tbt((new RUe,t))};var D7t,L7t,I7t,O7t,N7t,P7t,n8e,B7t,F7t;D(nee,"RectPackingMetaDataProvider",856),M(1004,1,$l,RUe),g.Qe=function(t){Tbt(t)};var R7t,j7t,r8e,VD,i8e,s8e,a8e,$7t,o8e,H7t,z7t,G7t,V7t,c8e,u8e,l8e,U7t,h8e,q7t,f8e,Y7t,d8e;D(nee,"RectPackingOptions",1004),M(1005,1,{},IHe),g.$e=function(){var t;return t=new DHe,t},g._e=function(t){},D(nee,"RectPackingOptions/RectpackingFactory",1005),M(1257,1,{},yat),g.a=0,g.b=!1,g.c=0,g.d=0,g.e=!1,g.f=!1,g.g=0,D("org.eclipse.elk.alg.rectpacking.seconditeration","RowFillingAndCompaction",1257),M(187,1,{187:1},JK),g.a=0,g.c=!1,g.d=0,g.e=0,g.f=0,g.g=0,g.i=0,g.k=!1,g.o=ki,g.p=ki,g.r=0,g.s=0,g.t=0,D(tD,"Block",187),M(211,1,{211:1},sB),g.a=0,g.b=0,g.d=0,g.e=0,g.f=0,D(tD,"BlockRow",211),M(443,1,{443:1},e0e),g.b=0,g.c=0,g.d=0,g.e=0,g.f=0,D(tD,"BlockStack",443),M(220,1,{220:1},nW,Ape),g.a=0,g.b=0,g.c=0,g.d=0,g.e=0;var BPn=D(tD,"DrawingData",220);M(355,22,{3:1,35:1,22:1,355:1},IS);var G5,y3,Y_,W_,K_,W7t=tr(tD,"DrawingDataDescriptor",355,sr,Vyn,Gpn),K7t;M(200,1,{200:1},l0e),g.b=0,g.c=0,g.e=0,g.f=0,D(tD,"RectRow",200),M(756,1,{},spe),g.j=0,D(Uw,Gvt,756),M(1245,1,{},OHe),g.Je=function(t){return yg(t.a,t.b)},D(Uw,Vvt,1245),M(1246,1,{},kWe),g.Je=function(t){return Qyn(this.a,t)},D(Uw,Uvt,1246),M(1247,1,{},xWe),g.Je=function(t){return i8n(this.a,t)},D(Uw,qvt,1247),M(1248,1,{},EWe),g.Je=function(t){return e6n(this.a,t)},D(Uw,"ElkGraphImporter/lambda$3$Type",1248),M(1249,1,{},_We),g.Je=function(t){return RTn(this.a,t)},D(Uw,Yvt,1249),M(1133,209,t2,gQe),g.Ze=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j;for(Pp(t,(uM(),TH))&&(j=Jn(pt(t,(HX(),N8e))),p=FQ(Ok(),j),p&&(v=h(zW(p.f),209),v.Ze(t,jo(n,1)))),ya(t,Tre,(vA(),_H)),ya(t,Cre,(lM(),_re)),ya(t,Sre,($A(),CH)),m=h(pt(t,(HX(),L8e)),19).a,Bn(n,"Overlap removal",1),ft(lt(pt(t,pEt))),x=new ns,_=new TWe(x),a=new spe,s=Cbt(a,t),A=!0,u=0;u<m&&A;){if(ft(lt(pt(t,I8e)))){if(x.a.$b(),P_n(new det(_),s.i),x.a.gc()==0)break;s.e=x}for(hB(this.b),P1(this.b,(XA(),kH),(K7(),UD)),P1(this.b,xH,s.g),P1(this.b,EH,(_N(),kre)),this.a=vR(this.b,s),P=new Q(this.a);P.a<P.c.c.length;)I=h(K(P),51),I.pf(s,jo(n,1));Z7n(a,s),A=ft(lt(U(s,(Bk(),x4e)))),++u}H2t(a,s),An(n)},D(Uw,"OverlapRemovalLayoutProvider",1133),M(1134,1,{},TWe),D(Uw,"OverlapRemovalLayoutProvider/lambda$0$Type",1134),M(437,22,{3:1,35:1,22:1,437:1},wY);var kH,xH,EH,mre=tr(Uw,"SPOrEPhases",437,sr,bmn,Vpn),X7t;M(1255,1,{},pQe),D(Uw,"ShrinkTree",1255),M(1135,209,t2,yXe),g.Ze=function(t,n){var s,a,u,p,v;Pp(t,(uM(),TH))&&(v=Jn(pt(t,TH)),u=FQ(Ok(),v),u&&(p=h(zW(u.f),209),p.Ze(t,jo(n,1)))),a=new spe,s=Cbt(a,t),kSn(this.a,s,jo(n,1)),H2t(a,s)},D(Uw,"ShrinkTreeLayoutProvider",1135),M(300,134,{3:1,300:1,94:1,134:1},Xit),g.c=!1,D("org.eclipse.elk.alg.spore.graph","Graph",300),M(482,22,{3:1,35:1,22:1,482:1,246:1,234:1},hZe),g.Kf=function(){return Jlt(this)},g.Xf=function(){return Jlt(this)};var yre,g8e=tr(qw,wwe,482,sr,Svn,Upn),Q7t;M(551,22,{3:1,35:1,22:1,551:1,246:1,234:1},dtt),g.Kf=function(){return new Kue},g.Xf=function(){return new Kue};var kre,Z7t=tr(qw,"OverlapRemovalStrategy",551,sr,Avn,qpn),J7t;M(430,22,{3:1,35:1,22:1,430:1},lfe);var _H,xre,p8e=tr(qw,"RootSelection",430,sr,ywn,Ypn),eEt;M(316,22,{3:1,35:1,22:1,316:1},OS);var b8e,Ere,_re,v8e,w8e,m8e=tr(qw,"SpanningTreeCostFunction",316,sr,Hyn,Wpn),tEt;M(1002,1,$l,DUe),g.Qe=function(t){v2t(t)};var y8e,k8e,nEt,rEt,x8e,E8e,Tre,Cre,Sre,iEt,sEt,TH;D(qw,"SporeCompactionOptions",1002),M(1003,1,{},NHe),g.$e=function(){var t;return t=new yXe,t},g._e=function(t){},D(qw,"SporeCompactionOptions/SporeCompactionFactory",1003),M(855,1,$l,LUe),g.Qe=function(t){Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,iee),""),"Underlying Layout Algorithm"),"A layout algorithm that is applied to the graph before it is compacted. If this is null, nothing is applied before compaction."),(E0(),b8)),it),Lt((Qh(),gn))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,oee),"structure"),"Structure Extraction Strategy"),"This option defines what kind of triangulation or other partitioning of the plane is applied to the vertices."),M8e),_i),P8e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,yye),cee),"Tree Construction Strategy"),"Whether a minimum spanning tree or a maximum spanning tree should be constructed."),S8e),_i),F8e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,kye),cee),"Cost Function for Spanning Tree"),"The cost function is used in the creation of the spanning tree."),C8e),_i),m8e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,see),cee),"Root node for spanning tree construction"),"The identifier of the node that is preferred as the root of the spanning tree. If this is null, the first node is chosen."),null),b8),it),Lt(gn)))),Es(t,see,aee,hEt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,aee),cee),"Root selection for spanning tree"),"This sets the method used to select a root node for the construction of a spanning tree"),T8e),_i),p8e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,xye),bme),"Compaction Strategy"),"This option defines how the compaction is applied."),_8e),_i),g8e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Eye),bme),"Orthogonal Compaction"),"Restricts the translation of nodes to orthogonal directions in the compaction phase."),(on(),!1)),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,_ye),Mwt),"Upper limit for iterations of overlap removal"),null),Ke(64)),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Tye),Mwt),"Whether to run a supplementary scanline overlap check."),null),!0),zs),es),Lt(gn)))),Ept((new IUe,t)),v2t((new DUe,t))};var aEt,_8e,oEt,cEt,uEt,lEt,hEt,fEt,T8e,dEt,C8e,gEt,S8e,A8e,M8e,D8e;D(qw,"SporeMetaDataProvider",855),M(_0,1,$l,IUe),g.Qe=function(t){Ept(t)};var pEt,L8e,I8e,O8e,bEt,N8e;D(qw,"SporeOverlapRemovalOptions",_0),M(1001,1,{},PHe),g.$e=function(){var t;return t=new gQe,t},g._e=function(t){},D(qw,"SporeOverlapRemovalOptions/SporeOverlapFactory",1001),M(530,22,{3:1,35:1,22:1,530:1,246:1,234:1},rit),g.Kf=function(){return eht(this)},g.Xf=function(){return eht(this)};var UD,P8e=tr(qw,"StructureExtractionStrategy",530,sr,Mvn,Kpn),vEt;M(429,22,{3:1,35:1,22:1,429:1,246:1,234:1},hfe),g.Kf=function(){return Kht(this)},g.Xf=function(){return Kht(this)};var B8e,CH,F8e=tr(qw,"TreeConstructionStrategy",429,sr,mwn,Xpn),wEt;M(1443,1,zo,BHe),g.Yf=function(t){return h(t,300),new rs},g.pf=function(t,n){cEn(h(t,300),n)},D(Dwt,"DelaunayTriangulationPhase",1443),M(1444,1,mn,CWe),g.td=function(t){Ue(this.a,h(t,65).a)},D(Dwt,"DelaunayTriangulationPhase/lambda$0$Type",1444),M(783,1,zo,uhe),g.Yf=function(t){return h(t,300),new rs},g.pf=function(t,n){this.ng(h(t,300),n)},g.ng=function(t,n){var s,a,u;Bn(n,"Minimum spanning tree construction",1),t.d?a=t.d.a:a=h(ut(t.i,0),65).a,ft(lt(U(t,(Bk(),p_))))?u=rZ(t.e,a,(s=t.b,s)):u=rZ(t.e,a,t.b),Plt(this,u,t),An(n)},D(uee,"MinSTPhase",783),M(1446,783,zo,iXe),g.ng=function(t,n){var s,a,u,p;Bn(n,"Maximum spanning tree construction",1),s=new SWe(t),t.d?u=t.d.c:u=h(ut(t.i,0),65).c,ft(lt(U(t,(Bk(),p_))))?p=rZ(t.e,u,(a=s,a)):p=rZ(t.e,u,s),Plt(this,p,t),An(n)},D(uee,"MaxSTPhase",1446),M(1447,1,{},SWe),g.Je=function(t){return Z1n(this.a,t)},D(uee,"MaxSTPhase/lambda$0$Type",1447),M(1445,1,mn,AWe),g.td=function(t){Gdn(this.a,h(t,65))},D(uee,"MinSTPhase/lambda$0$Type",1445),M(785,1,zo,Kue),g.Yf=function(t){return h(t,300),new rs},g.pf=function(t,n){k9n(this,h(t,300),n)},g.a=!1,D(lee,"GrowTreePhase",785),M(786,1,mn,j1e),g.td=function(t){R5n(this.a,this.b,this.c,h(t,221))},D(lee,"GrowTreePhase/lambda$0$Type",786),M(1448,1,zo,FHe),g.Yf=function(t){return h(t,300),new rs},g.pf=function(t,n){D8n(this,h(t,300),n)},D(lee,"ShrinkTreeCompactionPhase",1448),M(784,1,mn,$1e),g.td=function(t){cCn(this.a,this.b,this.c,h(t,221))},D(lee,"ShrinkTreeCompactionPhase/lambda$0$Type",784);var R8e=fi(xo,"IGraphElementVisitor");M(860,1,{527:1},Ust),g.og=function(t){var n;n=gMn(this,t),Oa(n,h(_n(this.b,t),94)),fSn(this,t,n)};var mEt,yEt;D(Uy,"LayoutConfigurator",860);var FPn=fi(Uy,"LayoutConfigurator/IPropertyHolderOptionFilter");M(932,1,{1933:1},RHe),g.pg=function(t,n){return n5(),!t.Xe(n)},D(Uy,"LayoutConfigurator/lambda$0$Type",932),M(933,1,{1933:1},Mhn),g.pg=function(t,n){return lQe(t,n)},D(Uy,"LayoutConfigurator/lambda$1$Type",933),M(931,1,{831:1},jHe),g.qg=function(t,n){return n5(),!t.Xe(n)},D(Uy,"LayoutConfigurator/lambda$2$Type",931),M(934,1,Cr,WZe),g.Mb=function(t){return gvn(this.a,this.b,h(t,1933))},D(Uy,"LayoutConfigurator/lambda$3$Type",934),M(858,1,{},$He),D(Uy,"RecursiveGraphLayoutEngine",858),M(296,60,Fd,NKe,ey),D(Uy,"UnsupportedConfigurationException",296),M(453,60,Fd,z7),D(Uy,"UnsupportedGraphException",453),M(754,1,{}),D(xo,"AbstractRandomListAccessor",754),M(500,754,{},mM),g.rg=function(){return null},g.d=!0,g.e=!0,g.f=0,D(Ox,"AlgorithmAssembler",500),M(1236,1,Cr,HHe),g.Mb=function(t){return!!h(t,123)},D(Ox,"AlgorithmAssembler/lambda$0$Type",1236),M(1237,1,{},MWe),g.Kb=function(t){return v1n(this.a,h(t,123))},D(Ox,"AlgorithmAssembler/lambda$1$Type",1237),M(1238,1,Cr,zHe),g.Mb=function(t){return!!h(t,80)},D(Ox,"AlgorithmAssembler/lambda$2$Type",1238),M(1239,1,mn,DWe),g.td=function(t){qu(this.a,h(t,80))},D(Ox,"AlgorithmAssembler/lambda$3$Type",1239),M(1240,1,mn,KZe),g.td=function(t){D0n(this.a,this.b,h(t,234))},D(Ox,"AlgorithmAssembler/lambda$4$Type",1240),M(1355,1,Yr,GHe),g.ue=function(t,n){return Vvn(h(t,234),h(n,234))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Ox,"EnumBasedFactoryComparator",1355),M(80,754,{80:1},rs),g.rg=function(){return new ns},g.a=0,D(Ox,"LayoutProcessorConfiguration",80),M(1013,1,{527:1},jUe),g.og=function(t){ZE(xEt,new LWe(t))};var kEt,xEt,EEt;D(eo,"DeprecatedLayoutOptionReplacer",1013),M(1014,1,mn,VHe),g.td=function(t){O3n(h(t,160))},D(eo,"DeprecatedLayoutOptionReplacer/lambda$0$Type",1014),M(1015,1,mn,UHe),g.td=function(t){nxn(h(t,160))},D(eo,"DeprecatedLayoutOptionReplacer/lambda$1$Type",1015),M(1016,1,{},LWe),g.Od=function(t,n){L0n(this.a,h(t,146),h(n,38))},D(eo,"DeprecatedLayoutOptionReplacer/lambda$2$Type",1016),M(149,1,{686:1,149:1},Xp),g.Fb=function(t){return C0e(this,t)},g.sg=function(){return this.b},g.tg=function(){return this.c},g.ne=function(){return this.e},g.Hb=function(){return k0(this.c)},g.Ib=function(){return"Layout Algorithm: "+this.c};var RPn=D(eo,"LayoutAlgorithmData",149);M(263,1,{},mp),D(eo,"LayoutAlgorithmData/Builder",263),M(1017,1,{527:1},qHe),g.og=function(t){ue(t,239)&&!ft(lt(t.We((Tr(),OH))))&&QDn(h(t,33))},D(eo,"LayoutAlgorithmResolver",1017),M(229,1,{686:1,229:1},q4),g.Fb=function(t){return ue(t,229)?Nt(this.b,h(t,229).b):!1},g.sg=function(){return this.a},g.tg=function(){return this.b},g.ne=function(){return this.d},g.Hb=function(){return k0(this.b)},g.Ib=function(){return"Layout Type: "+this.b},D(eo,"LayoutCategoryData",229),M(344,1,{},m4),D(eo,"LayoutCategoryData/Builder",344),M(867,1,{},fpt);var Are;D(eo,"LayoutMetaDataService",867),M(868,1,{},Grt),D(eo,"LayoutMetaDataService/Registry",868),M(478,1,{478:1},Xue),D(eo,"LayoutMetaDataService/Registry/Triple",478),M(869,1,Xy,YHe),g.ug=function(){return new Rs},D(eo,"LayoutMetaDataService/lambda$0$Type",869),M(870,1,Yw,WHe),g.vg=function(t){return ro(h(t,8))},D(eo,"LayoutMetaDataService/lambda$1$Type",870),M(879,1,Xy,KHe),g.ug=function(){return new qe},D(eo,"LayoutMetaDataService/lambda$10$Type",879),M(880,1,Yw,XHe),g.vg=function(t){return new Bc(h(t,12))},D(eo,"LayoutMetaDataService/lambda$11$Type",880),M(881,1,Xy,QHe),g.ug=function(){return new di},D(eo,"LayoutMetaDataService/lambda$12$Type",881),M(882,1,Yw,ZHe),g.vg=function(t){return ZS(h(t,68))},D(eo,"LayoutMetaDataService/lambda$13$Type",882),M(883,1,Xy,JHe),g.ug=function(){return new ns},D(eo,"LayoutMetaDataService/lambda$14$Type",883),M(884,1,Yw,eze),g.vg=function(t){return ZP(h(t,53))},D(eo,"LayoutMetaDataService/lambda$15$Type",884),M(885,1,Xy,tze),g.ug=function(){return new md},D(eo,"LayoutMetaDataService/lambda$16$Type",885),M(886,1,Yw,nze),g.vg=function(t){return bB(h(t,53))},D(eo,"LayoutMetaDataService/lambda$17$Type",886),M(887,1,Xy,rze),g.ug=function(){return new Kle},D(eo,"LayoutMetaDataService/lambda$18$Type",887),M(888,1,Yw,ize),g.vg=function(t){return vnt(h(t,208))},D(eo,"LayoutMetaDataService/lambda$19$Type",888),M(871,1,Xy,sze),g.ug=function(){return new Oc},D(eo,"LayoutMetaDataService/lambda$2$Type",871),M(872,1,Yw,aze),g.vg=function(t){return new dN(h(t,74))},D(eo,"LayoutMetaDataService/lambda$3$Type",872),M(873,1,Xy,oze),g.ug=function(){return new B7},D(eo,"LayoutMetaDataService/lambda$4$Type",873),M(874,1,Yw,cze),g.vg=function(t){return new XY(h(t,142))},D(eo,"LayoutMetaDataService/lambda$5$Type",874),M(875,1,Xy,lze),g.ug=function(){return new E4},D(eo,"LayoutMetaDataService/lambda$6$Type",875),M(876,1,Yw,hze),g.vg=function(t){return new N1e(h(t,116))},D(eo,"LayoutMetaDataService/lambda$7$Type",876),M(877,1,Xy,fze),g.ug=function(){return new lq},D(eo,"LayoutMetaDataService/lambda$8$Type",877),M(878,1,Yw,dze),g.vg=function(t){return new Dct(h(t,373))},D(eo,"LayoutMetaDataService/lambda$9$Type",878);var Mre=fi(GM,"IProperty");M(23,1,{35:1,686:1,23:1,146:1},kt),g.wd=function(t){return n0n(this,h(t,146))},g.Fb=function(t){return ue(t,23)?Nt(this.f,h(t,23).f):ue(t,146)&&Nt(this.f,h(t,146).tg())},g.wg=function(){var t;if(ue(this.b,4)){if(t=Upe(this.b),t==null)throw ne(new ja(Owt+this.f+"'. Make sure it's type is registered with the "+(yd(pL),pL.k)+Cye));return t}else return this.b},g.sg=function(){return this.d},g.tg=function(){return this.f},g.ne=function(){return this.i},g.Hb=function(){return k0(this.f)},g.Ib=function(){return"Layout Option: "+this.f},D(eo,"LayoutOptionData",23),M(24,1,{},yt),D(eo,"LayoutOptionData/Builder",24),M(175,22,{3:1,35:1,22:1,175:1},NS);var k1,N0,gs,gn,w2,P0=tr(eo,"LayoutOptionData/Target",175,sr,$yn,Qpn),_Et;M(277,22,{3:1,35:1,22:1,277:1},L4);var zs,Ba,_i,V5,po,jf,b8,j8e,TEt=tr(eo,"LayoutOptionData/Type",277,sr,v4n,Zpn),CEt,X_,$8e;M(110,1,{110:1},O4,hl,eB),g.Fb=function(t){var n;return t==null||!ue(t,110)?!1:(n=h(t,110),Bo(this.c,n.c)&&Bo(this.d,n.d)&&Bo(this.b,n.b)&&Bo(this.a,n.a))},g.Hb=function(){return lF(se(ie(yn,1),st,1,5,[this.c,this.d,this.b,this.a]))},g.Ib=function(){return"Rect[x="+this.c+",y="+this.d+",w="+this.b+",h="+this.a+"]"},g.a=0,g.b=0,g.c=0,g.d=0,D(WM,"ElkRectangle",110),M(8,1,{3:1,4:1,8:1,414:1},Rs,WW,dt,_a),g.Fb=function(t){return jut(this,t)},g.Hb=function(){return J6(this.a)+r7n(J6(this.b))},g.Jf=function(t){var n,s,a,u;for(a=0;a<t.length&&mht((er(a,t.length),t.charCodeAt(a)),Rvt);)++a;for(n=t.length;n>0&&mht((er(n-1,t.length),t.charCodeAt(n-1)),jvt);)--n;if(a>=n)throw ne(new an("The given string does not contain any numbers."));if(u=Pw(t.substr(a,n-a),`,|;|\r|
`),u.length!=2)throw ne(new an("Exactly two numbers are expected, "+u.length+" were found."));try{this.a=Dw(Mw(u[0])),this.b=Dw(Mw(u[1]))}catch(p){throw p=ui(p),ue(p,127)?(s=p,ne(new an($vt+s))):ne(p)}},g.Ib=function(){return"("+this.a+","+this.b+")"},g.a=0,g.b=0;var cs=D(WM,"KVector",8);M(74,68,{3:1,4:1,20:1,28:1,52:1,14:1,68:1,15:1,74:1,414:1},Oc,dN,rtt),g.Pc=function(){return x6n(this)},g.Jf=function(t){var n,s,a,u,p,v;a=Pw(t,`,|;|\\(|\\)|\\[|\\]|\\{|\\}| |	|
`),Pl(this);try{for(s=0,p=0,u=0,v=0;s<a.length;)a[s]!=null&&Mw(a[s]).length>0&&(p%2==0?u=Dw(a[s]):v=Dw(a[s]),p>0&&p%2!=0&&vr(this,new dt(u,v)),++p),++s}catch(m){throw m=ui(m),ue(m,127)?(n=m,ne(new an("The given string does not match the expected format for vectors."+n))):ne(m)}},g.Ib=function(){var t,n,s;for(t=new ju("("),n=pr(this,0);n.b!=n.d.c;)s=h(gr(n),8),or(t,s.a+","+s.b),n.b!=n.d.c&&(t.a+="; ");return(t.a+=")",t).a};var H8e=D(WM,"KVectorChain",74);M(248,22,{3:1,35:1,22:1,248:1},oE);var Dre,SH,AH,qD,YD,MH,z8e=tr(zl,"Alignment",248,sr,b3n,Jpn),SEt;M(979,1,$l,$Ue),g.Qe=function(t){Fpt(t)};var G8e,Lre,AEt,V8e,U8e,MEt,q8e,DEt,LEt,Y8e,W8e,IEt;D(zl,"BoxLayouterOptions",979),M(980,1,{},uze),g.$e=function(){var t;return t=new vze,t},g._e=function(t){},D(zl,"BoxLayouterOptions/BoxFactory",980),M(291,22,{3:1,35:1,22:1,291:1},cE);var WD,Ire,KD,XD,QD,Ore,Nre=tr(zl,"ContentAlignment",291,sr,p3n,e2n),OEt;M(684,1,$l,nle),g.Qe=function(t){Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Pwt),""),"Layout Algorithm"),"Select a specific layout algorithm."),(E0(),b8)),it),Lt((Qh(),gn))))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Bwt),""),"Resolved Layout Algorithm"),"Meta data associated with the selected algorithm."),jf),RPn),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Nme),""),"Alignment"),"Alignment of the selected node relative to other nodes; the exact meaning depends on the used algorithm."),K8e),_i),z8e),Lt(gs)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,k5),""),"Aspect Ratio"),"The desired aspect ratio of the drawing, that is the quotient of width by height."),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Aye),""),"Bend Points"),"A fixed list of bend points for the edge. This is used by the 'Fixed Layout' algorithm to specify a pre-defined routing for an edge. The vector chain must include the source point, any bend points, and the target point, so it must have at least two points."),jf),H8e),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,tj),""),"Content Alignment"),"Specifies how the content of a node are aligned. Each node can individually control the alignment of its contents. I.e. if a node should be aligned top left in its parent node, the parent node should specify that option."),Q8e),V5),Nre),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,eD),""),"Debug Mode"),"Whether additional debug information shall be generated."),(on(),!1)),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Fme),""),bwe),"Overall direction of edges: horizontal (right / left) or vertical (down / up)."),Z8e),_i),J_),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ZM),""),"Edge Routing"),"What kind of edge routing style should be applied for the content of a parent node. Algorithms may also set this option to single edges in order to mark them as splines. The bend point list of edges with this option set to SPLINES must be interpreted as control points for a piecewise cubic spline."),t7e),_i),Yre),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,oj),""),"Expand Nodes"),"If active, nodes are expanded to fill the area of their parent."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,ej),""),"Hierarchy Handling"),"Determines whether separate layout runs are triggered for different compound nodes in a hierarchical graph. Setting a node's hierarchy handling to `INCLUDE_CHILDREN` will lay out that node and all of its descendants in a single layout run, until a descendant is encountered which has its hierarchy handling set to `SEPARATE_CHILDREN`. In general, `SEPARATE_CHILDREN` will ensure that a new layout run is triggered for a node with that setting. Including multiple levels of hierarchy in a single layout run may allow cross-hierarchical edges to be laid out properly. If the root node is set to `INHERIT` (or not set at all), the default behavior is `SEPARATE_CHILDREN`."),s7e),_i),V7e),ni(gn,se(ie(P0,1),ze,175,0,[gs]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Vb),""),"Padding"),"The padding to be left to a parent element's border when placing child elements. This can also serve as an output option of a layout algorithm if node size calculation is setup appropriately."),p7e),jf),X4e),ni(gn,se(ie(P0,1),ze,175,0,[gs]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,qM),""),"Interactive"),"Whether the algorithm should be run in interactive mode for the content of a parent node. What this means exactly depends on how the specific algorithm interprets this option. Usually in the interactive mode algorithms try to modify the current layout as little as possible."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,YJ),""),"interactive Layout"),"Whether the graph should be changeable interactively and by setting constraints"),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,z9),""),"Omit Node Micro Layout"),"Node micro layout comprises the computation of node dimensions (if requested), the placement of ports and their labels, and the placement of node labels. The functionality is implemented independent of any specific layout algorithm and shouldn't have any negative impact on the layout algorithm's performance itself. Yet, if any unforeseen behavior occurs, this option allows to deactivate the micro layout."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,fJ),""),"Port Constraints"),"Defines constraints of the position of the ports of a node."),y7e),_i),Y7e),Lt(gs)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,nj),""),"Position"),"The position of a node, port, or label. This is used by the 'Fixed Layout' algorithm to specify a pre-defined position."),jf),cs),ni(gs,se(ie(P0,1),ze,175,0,[w2,N0]))))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,UM),""),"Priority"),"Defines the priority of an object; its meaning depends on the specific layout algorithm and the context where it is used."),po),Ws),ni(gs,se(ie(P0,1),ze,175,0,[k1]))))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,BR),""),"Randomization Seed"),"Seed used for pseudo-random number generators to control the layout algorithm. If the value is 0, the seed shall be determined pseudo-randomly (e.g. from the system time)."),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,H9),""),"Separate Connected Components"),"Whether each connected component should be processed separately."),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,qme),""),"Junction Points"),"This option is not used as option, but as output of the layout algorithms. It is attached to edges and determines the points where junction symbols should be drawn in order to represent hyperedges with orthogonal routing. Whether such points are computed depends on the chosen layout algorithm and edge routing style. The points are put into the vector chain with no specific order."),o7e),jf),H8e),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Kme),""),"Comment Box"),"Whether the node should be regarded as a comment box instead of a regular node. In that case its placement should be similar to how labels are handled. Any edges incident to a comment box specify to which graph elements the comment is related."),!1),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Xme),""),"Hypernode"),"Whether the node should be handled as a hypernode."),!1),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Fwt),""),"Label Manager"),"Label managers can shorten labels upon a layout algorithm's request."),jf),GPn),ni(gn,se(ie(P0,1),ze,175,0,[N0]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Zme),""),"Margins"),"Margins define additional space around the actual bounds of a graph element. For instance, ports or labels being placed on the outside of a node's border might introduce such a margin. The margin is used to guarantee non-overlap of other graph elements with those ports or labels."),c7e),jf),K4e),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Ime),""),"No Layout"),"No layout is done for the associated element. This is used to mark parts of a diagram to avoid their inclusion in the layout graph, or to mark parts of the layout graph to prevent layout engines from processing them. If you wish to exclude the contents of a compound node from automatic layout, while the node itself is still considered on its own layer, use the 'Fixed Layout' algorithm for that node."),!1),zs),es),ni(gs,se(ie(P0,1),ze,175,0,[k1,w2,N0]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Rwt),""),"Scale Factor"),"The scaling factor to be applied to the corresponding node in recursive layout. It causes the corresponding node's size to be adjusted, and its ports and labels to be sized and placed accordingly after the layout of that node has been determined (and before the node itself and its siblings are arranged). The scaling is not reverted afterwards, so the resulting layout graph contains the adjusted size and position data. This option is currently not supported if 'Layout Hierarchy' is set."),1),Ba),Ts),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,jwt),""),"Animate"),"Whether the shift from the old layout to the new computed layout shall be animated."),!0),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,$wt),""),"Animation Time Factor"),"Factor for computation of animation time. The higher the value, the longer the animation time. If the value is 0, the resulting time is always equal to the minimum defined by 'Minimal Animation Time'."),Ke(100)),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Hwt),""),"Layout Ancestors"),"Whether the hierarchy levels on the path from the selected element to the root of the diagram shall be included in the layout process."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,zwt),""),"Maximal Animation Time"),"The maximal time for animations, in milliseconds."),Ke(4e3)),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Gwt),""),"Minimal Animation Time"),"The minimal time for animations, in milliseconds."),Ke(400)),po),Ws),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Vwt),""),"Progress Bar"),"Whether a progress bar shall be displayed during layout computations."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Uwt),""),"Validate Graph"),"Whether the graph shall be validated before any layout algorithm is applied. If this option is enabled and at least one error is found, the layout process is aborted and a message is shown to the user."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,qwt),""),"Validate Options"),"Whether layout options shall be validated before any layout algorithm is applied. If this option is enabled and at least one error is found, the layout process is aborted and a message is shown to the user."),!0),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Ywt),""),"Zoom to Fit"),"Whether the zoom level shall be set to view the whole diagram after layout."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Sye),"box"),"Box Layout Mode"),"Configures the packing mode used by the {@link BoxLayoutProvider}. If SIMPLE is not required (neither priorities are used nor the interactive mode), GROUP_DEC can improve the packing and decrease the area. GROUP_MIXED and GROUP_INC may, in very specific scenarios, work better."),X8e),_i),J7e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,yme),Of),"Comment Comment Spacing"),"Spacing to be preserved between a comment box and other comment boxes connected to the same node. The space left between comment boxes of different nodes is controlled by the node-node spacing."),10),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,kme),Of),"Comment Node Spacing"),"Spacing to be preserved between a node and its connected comment boxes. The space left between a node and the comments of another node is controlled by the node-node spacing."),10),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,aJ),Of),"Components Spacing"),"Spacing to be preserved between pairs of connected components. This option is only relevant if 'separateConnectedComponents' is activated."),20),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,xme),Of),"Edge Spacing"),"Spacing to be preserved between any two edges. Note that while this can somewhat easily be satisfied for the segments of orthogonally drawn edges, it is harder for general polylines or splines."),10),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,hJ),Of),"Edge Label Spacing"),"The minimal distance to be preserved between a label and the edge it is associated with. Note that the placement of a label is influenced by the 'edgelabels.placement' option."),2),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Eme),Of),"Edge Node Spacing"),"Spacing to be preserved between nodes and edges."),10),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,_me),Of),"Label Spacing"),"Determines the amount of space to be left between two labels of the same graph element."),0),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Sme),Of),"Label Node Spacing"),"Spacing to be preserved between labels and the border of node they are associated with. Note that the placement of a label is influenced by the 'nodelabels.placement' option."),5),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Tme),Of),"Horizontal spacing between Label and Port"),"Horizontal spacing to be preserved between labels and the ports they are associated with. Note that the placement of a label is influenced by the 'portlabels.placement' option."),1),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Cme),Of),"Vertical spacing between Label and Port"),"Vertical spacing to be preserved between labels and the ports they are associated with. Note that the placement of a label is influenced by the 'portlabels.placement' option."),1),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,zw),Of),"Node Spacing"),"The minimal distance to be preserved between each two nodes."),20),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Ame),Of),"Node Self Loop Spacing"),"Spacing to be preserved between a node and its self loops."),10),Ba),Ts),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Mme),Of),"Port Spacing"),"Spacing between pairs of ports of the same node."),10),Ba),Ts),ni(gn,se(ie(P0,1),ze,175,0,[gs]))))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Dme),Of),"Individual Spacing"),"Allows to specify individual spacing values for graph elements that shall be different from the value specified for the element's parent."),jf),g9t),ni(gs,se(ie(P0,1),ze,175,0,[k1,w2,N0]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Jme),Of),"Additional Port Space"),"Additional space around the sets of ports on each node side. For each side of a node, this option can reserve additional space before and after the ports on each side. For example, a top spacing of 20 makes sure that the first port on the western and eastern side is 20 units away from the northern border."),D7e),jf),K4e),Lt(gn)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,qJ),Qwt),"Layout Partition"),"Partition to which the node belongs. This requires Layout Partitioning to be active. Nodes with lower partition IDs will appear to the left of nodes with higher partition IDs (assuming a left-to-right layout direction)."),po),Ws),ni(gn,se(ie(P0,1),ze,175,0,[gs]))))),Es(t,qJ,UJ,HEt),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,UJ),Qwt),"Layout Partitioning"),"Whether to activate partitioned layout. This will allow to group nodes through the Layout Partition option. a pair of nodes with different partition indices is then placed such that the node with lower index is placed to the left of the other node (with left-to-right layout direction). Depending on the layout algorithm, this may only be guaranteed to work if all nodes have a layout partition configured, or at least if edges that cross partitions are not part of a partition-crossing cycle."),b7e),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Rme),Zwt),"Node Label Padding"),"Define padding for node labels that are placed inside of a node."),l7e),jf),X4e),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Dx),Zwt),"Node Label Placement"),"Hints for where node labels are to be placed; if empty, the node label's position is not modified."),h7e),V5),ga),ni(gs,se(ie(P0,1),ze,175,0,[N0]))))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Hme),uj),"Port Alignment"),"Defines the default port distribution for a node. May be overridden for each side individually."),w7e),_i),rT),Lt(gs)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,zme),uj),"Port Alignment (North)"),"Defines how ports on the northern side are placed, overriding the node's general port alignment."),_i),rT),Lt(gs)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Gme),uj),"Port Alignment (South)"),"Defines how ports on the southern side are placed, overriding the node's general port alignment."),_i),rT),Lt(gs)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Vme),uj),"Port Alignment (West)"),"Defines how ports on the western side are placed, overriding the node's general port alignment."),_i),rT),Lt(gs)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Ume),uj),"Port Alignment (East)"),"Defines how ports on the eastern side are placed, overriding the node's general port alignment."),_i),rT),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Yy),fee),"Node Size Constraints"),"What should be taken into account when calculating a node's size. Empty size constraints specify that a node's size is already fixed and should not be changed."),f7e),V5),aT),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,x5),fee),"Node Size Options"),"Options modifying the behavior of the size constraints set on a node. Each member of the set specifies something that should be taken into account when calculating node sizes. The empty set corresponds to no further modifications."),g7e),V5),K7e),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,E5),fee),"Node Size Minimum"),"The minimal size to which a node can be reduced."),d7e),jf),cs),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,VJ),fee),"Fixed Graph Size"),"By default, the fixed layout provider will enlarge a graph until it is large enough to contain its children. If this option is set, it won't do so."),!1),zs),es),Lt(gn)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Yme),GJ),"Edge Label Placement"),"Gives a hint on where to put edge labels."),J8e),_i),I7e),Lt(N0)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,FR),GJ),"Inline Edge Labels"),"If true, an edge label is placed directly on its edge. May only apply to center edge labels. This kind of label placement is only advisable if the label's rendering is such that it is not crossed by its edge and thus stays legible."),!1),zs),es),Lt(N0)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Wwt),"font"),"Font Name"),"Font name used for a label."),b8),it),Lt(N0)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Kwt),"font"),"Font Size"),"Font size used for a label."),po),Ws),Lt(N0)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Qme),dee),"Port Anchor Offset"),"The offset to the port position where connections shall be attached."),jf),cs),Lt(w2)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Wme),dee),"Port Index"),"The index of a port in the fixed order around a node. The order is assumed as clockwise, starting with the leftmost port on the top side. This option must be set if 'Port Constraints' is set to FIXED_ORDER and no specific positions are given for the ports. Additionally, the option 'Port Side' must be defined in this case."),po),Ws),Lt(w2)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Ome),dee),"Port Side"),"The side of a node on which a port is situated. This option must be set if 'Port Constraints' is set to FIXED_SIDE or FIXED_ORDER and no specific positions are given for the ports."),E7e),_i),ta),Lt(w2)))),Dt(t,new kt(At(St(Mt(Et(Ct(_t(Tt(new yt,Lme),dee),"Port Border Offset"),"The offset of ports on the node border. With a positive offset the port is moved outside of the node, while with a negative offset the port is moved towards the inside. An offset of 0 means that the port is placed directly on the node border, i.e. if the port side is north, the port's south border touches the nodes's north border; if the port side is east, the port's west border touches the nodes's east border; if the port side is south, the port's north border touches the node's south border; if the port side is west, the port's east border touches the node's west border."),Ba),Ts),Lt(w2)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Lx),Mye),"Port Label Placement"),"Decides on a placement method for port labels; if empty, the node label's position is not modified."),k7e),V5),FH),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,jme),Mye),"Port Labels Next to Port"),"Use 'portLabels.placement': NEXT_TO_PORT_OF_POSSIBLE."),!1),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,$me),Mye),"Treat Port Labels as Group"),"If this option is true (default), the labels of a port will be treated as a group when it comes to centering them next to their port. If this option is false, only the first label will be centered next to the port, with the others being placed below. This only applies to labels of eastern and western ports and will have no effect if labels are not placed next to their port."),!0),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Pme),Jwt),"Activate Inside Self Loops"),"Whether this node allows to route self loops inside of it instead of around it. If set to true, this will make the node a compound node if it isn't already, and will require the layout algorithm to support compound nodes with hierarchical ports."),!1),zs),es),Lt(gs)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Bme),Jwt),"Inside Self Loop"),"Whether a self loop should be routed inside a node instead of around that node."),!1),zs),es),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,oJ),"edge"),"Edge Thickness"),"The thickness of an edge. This is a hint on the line width used to draw an edge, possibly requiring more space to be reserved for it."),1),Ba),Ts),Lt(k1)))),Dt(t,new kt(At(St(Mt(zt(Et(Ct(_t(Tt(new yt,Xwt),"edge"),"Edge Type"),"The type of an edge. This is usually used for UML class diagrams, where associations must be handled differently from generalizations."),r7e),_i),R7e),Lt(k1)))),X7(t,new q4(V7(U6(V6(new m4,wn),"Layered"),'The layer-based method was introduced by Sugiyama, Tagawa and Toda in 1981. It emphasizes the direction of edges by pointing as many edges as possible into the same direction. The nodes are arranged in layers, which are sometimes called "hierarchies", and then reordered such that the number of edge crossings is minimized. Afterwards, concrete coordinates are computed for the nodes and edge bend points.'))),X7(t,new q4(V7(U6(V6(new m4,"org.eclipse.elk.orthogonal"),"Orthogonal"),`Orthogonal methods that follow the "topology-shape-metrics" approach by Batini, Nardelli and Tamassia '86. The first phase determines the topology of the drawing by applying a planarization technique, which results in a planar representation of the graph. The orthogonal shape is computed in the second phase, which aims at minimizing the number of edge bends, and is called orthogonalization. The third phase leads to concrete coordinates for nodes and edge bend points by applying a compaction method, thus defining the metrics.`))),X7(t,new q4(V7(U6(V6(new m4,Xu),"Force"),"Layout algorithms that follow physical analogies by simulating a system of attractive and repulsive forces. The first successful method of this kind was proposed by Eades in 1984."))),X7(t,new q4(V7(U6(V6(new m4,"org.eclipse.elk.circle"),"Circle"),"Circular layout algorithms emphasize cycles or biconnected components of a graph by arranging them in circles. This is useful if a drawing is desired where such components are clearly grouped, or where cycles are shown as prominent OPTIONS of the graph."))),X7(t,new q4(V7(U6(V6(new m4,kwt),"Tree"),"Specialized layout methods for trees, i.e. acyclic graphs. The regular structure of graphs that have no undirected cycles can be emphasized using an algorithm of this type."))),X7(t,new q4(V7(U6(V6(new m4,"org.eclipse.elk.planar"),"Planar"),"Algorithms that require a planar or upward planar graph. Most of these algorithms are theoretically interesting, but not practically usable."))),X7(t,new q4(V7(U6(V6(new m4,g1),"Radial"),"Radial layout algorithms usually position the nodes of the graph on concentric circles."))),xpt((new HUe,t)),Fpt((new $Ue,t)),Zgt((new zUe,t))};var Q_,NEt,K8e,U5,PEt,BEt,X8e,FEt,DH,Q8e,ZD,hv,Z8e,Pre,Bre,J8e,e7e,t7e,n7e,r7e,i7e,k3,s7e,REt,JD,Fre,LH,a7e,x3,o7e,eL,c7e,u7e,l7e,E3,h7e,m2,f7e,IH,_3,d7e,Wg,g7e,OH,tL,y2,p7e,jEt,b7e,$Et,HEt,v7e,w7e,Rre,jre,$re,Hre,m7e,pu,Z_,y7e,zre,Gre,gm,k7e,x7e,T3,E7e,q5,NH,Vre,v8,zEt,Ure,GEt,VEt,_7e,UEt,T7e,qEt,Y5,C7e,PH,S7e,A7e,k2,YEt,M7e,D7e,L7e;D(zl,"CoreOptions",684),M(103,22,{3:1,35:1,22:1,103:1},PS);var zd,Wl,Dh,Q1,Gd,J_=tr(zl,bwe,103,sr,Ryn,r2n),WEt;M(272,22,{3:1,35:1,22:1,272:1},mY);var w8,pm,m8,I7e=tr(zl,"EdgeLabelPlacement",272,sr,wmn,i2n),KEt;M(218,22,{3:1,35:1,22:1,218:1},HN);var y8,nL,W5,qre,Yre=tr(zl,"EdgeRouting",218,sr,lyn,s2n),XEt;M(312,22,{3:1,35:1,22:1,312:1},uE);var O7e,N7e,P7e,B7e,Wre,F7e,R7e=tr(zl,"EdgeType",312,sr,_3n,a2n),QEt;M(977,1,$l,HUe),g.Qe=function(t){xpt(t)};var j7e,$7e,H7e,z7e,ZEt,G7e,eT;D(zl,"FixedLayouterOptions",977),M(978,1,{},xze),g.$e=function(){var t;return t=new pze,t},g._e=function(t){},D(zl,"FixedLayouterOptions/FixedFactory",978),M(334,22,{3:1,35:1,22:1,334:1},yY);var B0,BH,tT,V7e=tr(zl,"HierarchyHandling",334,sr,vmn,o2n),JEt;M(285,22,{3:1,35:1,22:1,285:1},zN);var Z1,Kg,rL,iL,e9t=tr(zl,"LabelSide",285,sr,uyn,c2n),t9t;M(93,22,{3:1,35:1,22:1,93:1},iy);var Vd,Lh,Kl,Ih,el,Oh,Xl,J1,Nh,ga=tr(zl,"NodeLabelPlacement",93,sr,S4n,u2n),n9t;M(249,22,{3:1,35:1,22:1,249:1},BS);var U7e,nT,Xg,q7e,sL,rT=tr(zl,"PortAlignment",249,sr,jyn,l2n),r9t;M(98,22,{3:1,35:1,22:1,98:1},lE);var x2,qo,ed,k8,$f,Qg,Y7e=tr(zl,"PortConstraints",98,sr,o3n,h2n),i9t;M(273,22,{3:1,35:1,22:1,273:1},hE);var iT,sT,Ud,aL,Zg,K5,FH=tr(zl,"PortLabelPlacement",273,sr,E3n,f2n),s9t;M(61,22,{3:1,35:1,22:1,61:1},FS);var pn,sn,_l,Tl,Tc,uc,Hf,Ph,eu,Gc,Yo,tu,Cc,Sc,Bh,tl,nl,Ql,Fn,to,un,ta=tr(zl,"PortSide",61,sr,Pyn,p2n),a9t;M(981,1,$l,zUe),g.Qe=function(t){Zgt(t)};var o9t,c9t,W7e,u9t,l9t;D(zl,"RandomLayouterOptions",981),M(982,1,{},Eze),g.$e=function(){var t;return t=new Cze,t},g._e=function(t){},D(zl,"RandomLayouterOptions/RandomFactory",982),M(374,22,{3:1,35:1,22:1,374:1},GN);var bm,oL,cL,E2,aT=tr(zl,"SizeConstraint",374,sr,cyn,d2n),h9t;M(259,22,{3:1,35:1,22:1,259:1},sy);var uL,RH,x8,Kre,lL,oT,jH,$H,HH,K7e=tr(zl,"SizeOptions",259,sr,N4n,g2n),f9t;M(370,1,{1949:1},H6),g.b=!1,g.c=0,g.d=-1,g.e=null,g.f=null,g.g=-1,g.j=!1,g.k=!1,g.n=!1,g.o=0,g.q=0,g.r=0,D(xo,"BasicProgressMonitor",370),M(972,209,t2,vze),g.Ze=function(t,n){var s,a,u,p,v,m,x,_,A;switch(Bn(n,"Box layout",2),u=bS(Je(pt(t,(WF(),IEt)))),p=h(pt(t,LEt),116),s=ft(lt(pt(t,V8e))),a=ft(lt(pt(t,U8e))),h(pt(t,Lre),311).g){case 0:v=(m=new Bc((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a)),Rt(),hs(m,new IWe(a)),m),x=ube(t),_=Je(pt(t,G8e)),(_==null||(rn(_),_<=0))&&(_=1.3),A=mNn(v,u,p,x.a,x.b,s,(rn(_),_)),$b(t,A.a,A.b,!1,!0);break;default:pIn(t,u,p,s)}An(n)},D(xo,"BoxLayoutProvider",972),M(973,1,Yr,IWe),g.ue=function(t,n){return FSn(this,h(t,33),h(n,33))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},g.a=!1,D(xo,"BoxLayoutProvider/1",973),M(157,1,{157:1},jB,ntt),g.Ib=function(){return this.c?cve(this.c):Ag(this.b)},D(xo,"BoxLayoutProvider/Group",157),M(311,22,{3:1,35:1,22:1,311:1},VN);var X7e,Q7e,Z7e,Xre,J7e=tr(xo,"BoxLayoutProvider/PackingMode",311,sr,hyn,b2n),d9t;M(974,1,Yr,wze),g.ue=function(t,n){return $vn(h(t,157),h(n,157))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(xo,"BoxLayoutProvider/lambda$0$Type",974),M(975,1,Yr,mze),g.ue=function(t,n){return Ovn(h(t,157),h(n,157))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(xo,"BoxLayoutProvider/lambda$1$Type",975),M(976,1,Yr,yze),g.ue=function(t,n){return Nvn(h(t,157),h(n,157))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(xo,"BoxLayoutProvider/lambda$2$Type",976),M(1365,1,{831:1},kze),g.qg=function(t,n){return TN(),!ue(n,160)||lQe((n5(),h(t,160)),n)},D(xo,"ElkSpacings/AbstractSpacingsBuilder/lambda$0$Type",1365),M(1366,1,mn,OWe),g.td=function(t){_6n(this.a,h(t,146))},D(xo,"ElkSpacings/AbstractSpacingsBuilder/lambda$1$Type",1366),M(1367,1,mn,bze),g.td=function(t){h(t,94),TN()},D(xo,"ElkSpacings/AbstractSpacingsBuilder/lambda$2$Type",1367),M(1371,1,mn,NWe),g.td=function(t){K4n(this.a,h(t,94))},D(xo,"ElkSpacings/AbstractSpacingsBuilder/lambda$3$Type",1371),M(1369,1,Cr,XZe),g.Mb=function(t){return l6n(this.a,this.b,h(t,146))},D(xo,"ElkSpacings/AbstractSpacingsBuilder/lambda$4$Type",1369),M(1368,1,Cr,QZe),g.Mb=function(t){return S0n(this.a,this.b,h(t,831))},D(xo,"ElkSpacings/AbstractSpacingsBuilder/lambda$5$Type",1368),M(1370,1,mn,ZZe),g.td=function(t){Sbn(this.a,this.b,h(t,146))},D(xo,"ElkSpacings/AbstractSpacingsBuilder/lambda$6$Type",1370),M(935,1,{},gze),g.Kb=function(t){return ZJe(t)},g.Fb=function(t){return this===t},D(xo,"ElkUtil/lambda$0$Type",935),M(936,1,mn,JZe),g.td=function(t){jTn(this.a,this.b,h(t,79))},g.a=0,g.b=0,D(xo,"ElkUtil/lambda$1$Type",936),M(937,1,mn,eJe),g.td=function(t){jfn(this.a,this.b,h(t,202))},g.a=0,g.b=0,D(xo,"ElkUtil/lambda$2$Type",937),M(938,1,mn,tJe),g.td=function(t){Ddn(this.a,this.b,h(t,137))},g.a=0,g.b=0,D(xo,"ElkUtil/lambda$3$Type",938),M(939,1,mn,PWe),g.td=function(t){G2n(this.a,h(t,469))},D(xo,"ElkUtil/lambda$4$Type",939),M(342,1,{35:1,342:1},dS),g.wd=function(t){return i0n(this,h(t,236))},g.Fb=function(t){var n;return ue(t,342)?(n=h(t,342),this.a==n.a):!1},g.Hb=function(){return Li(this.a)},g.Ib=function(){return this.a+" (exclusive)"},g.a=0,D(xo,"ExclusiveBounds/ExclusiveLowerBound",342),M(1138,209,t2,pze),g.Ze=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe,le,we,Ne,Ve,rt,wt,xt;for(Bn(n,"Fixed Layout",1),p=h(pt(t,(Tr(),e7e)),218),I=0,P=0,le=new Tn((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));le.e!=le.i.gc();){for(oe=h(On(le),33),xt=h(pt(oe,(uF(),eT)),8),xt&&(bf(oe,xt.a,xt.b),h(pt(oe,$7e),174).Hc((Iu(),bm))&&(j=h(pt(oe,z7e),8),j.a>0&&j.b>0&&$b(oe,j.a,j.b,!0,!0))),I=d.Math.max(I,oe.i+oe.g),P=d.Math.max(P,oe.j+oe.f),_=new Tn((!oe.n&&(oe.n=new Ye(Ua,oe,1,7)),oe.n));_.e!=_.i.gc();)m=h(On(_),137),xt=h(pt(m,eT),8),xt&&bf(m,xt.a,xt.b),I=d.Math.max(I,oe.i+m.i+m.g),P=d.Math.max(P,oe.j+m.j+m.f);for(Ve=new Tn((!oe.c&&(oe.c=new Ye(bu,oe,9,9)),oe.c));Ve.e!=Ve.i.gc();)for(Ne=h(On(Ve),118),xt=h(pt(Ne,eT),8),xt&&bf(Ne,xt.a,xt.b),rt=oe.i+Ne.i,wt=oe.j+Ne.j,I=d.Math.max(I,rt+Ne.g),P=d.Math.max(P,wt+Ne.f),x=new Tn((!Ne.n&&(Ne.n=new Ye(Ua,Ne,1,7)),Ne.n));x.e!=x.i.gc();)m=h(On(x),137),xt=h(pt(m,eT),8),xt&&bf(m,xt.a,xt.b),I=d.Math.max(I,rt+m.i+m.g),P=d.Math.max(P,wt+m.j+m.f);for(u=new Sn(Dn(Pd(oe).a.Kc(),new L));rr(u);)s=h(Un(u),79),A=dbt(s),I=d.Math.max(I,A.a),P=d.Math.max(P,A.b);for(a=new Sn(Dn(wM(oe).a.Kc(),new L));rr(a);)s=h(Un(a),79),bi(H1(s))!=t&&(A=dbt(s),I=d.Math.max(I,A.a),P=d.Math.max(P,A.b))}if(p==(Od(),y8))for(fe=new Tn((!t.a&&(t.a=new Ye(wi,t,10,11)),t.a));fe.e!=fe.i.gc();)for(oe=h(On(fe),33),a=new Sn(Dn(Pd(oe).a.Kc(),new L));rr(a);)s=h(Un(a),79),v=BDn(s),v.b==0?ya(s,x3,null):ya(s,x3,v);ft(lt(pt(t,(uF(),H7e))))||(we=h(pt(t,ZEt),116),J=I+we.b+we.c,V=P+we.d+we.a,$b(t,J,V,!0,!0)),An(n)},D(xo,"FixedLayoutProvider",1138),M(373,134,{3:1,414:1,373:1,94:1,134:1},lq,Dct),g.Jf=function(t){var n,s,a,u,p,v,m,x,_;if(!!t)try{for(x=Pw(t,";,;"),p=x,v=0,m=p.length;v<m;++v){if(u=p[v],s=Pw(u,"\\:"),a=Pve(Ok(),s[0]),!a)throw ne(new an("Invalid option id: "+s[0]));if(_=Ove(a,s[1]),_==null)throw ne(new an("Invalid option value: "+s[1]));_==null?(!this.q&&(this.q=new zn),Q4(this.q,a)):(!this.q&&(this.q=new zn),Pr(this.q,a,_))}}catch(A){throw A=ui(A),ue(A,102)?(n=A,ne(new cht(n))):ne(A)}},g.Ib=function(){var t;return t=Jn(Gu(pc((this.q?this.q:(Rt(),Rt(),K1)).vc().Oc(),new _ze),pw(new mnt,new cn,new vt,new ve,se(ie(gu,1),ze,132,0,[])))),t};var g9t=D(xo,"IndividualSpacings",373);M(971,1,{},_ze),g.Kb=function(t){return Hvn(h(t,42))},D(xo,"IndividualSpacings/lambda$0$Type",971),M(709,1,{},Qnt),g.c=0,D(xo,"InstancePool",709),M(1275,1,{},Tze),D(xo,"LoggedGraph",1275),M(396,22,{3:1,35:1,22:1,396:1},UN);var eEe,rl,tEe,nEe,p9t=tr(xo,"LoggedGraph/Type",396,sr,fyn,v2n),b9t;M(46,1,{20:1,46:1},As),g.Jc=function(t){Os(this,t)},g.Fb=function(t){var n,s,a;return ue(t,46)?(s=h(t,46),n=this.a==null?s.a==null:Nr(this.a,s.a),a=this.b==null?s.b==null:Nr(this.b,s.b),n&&a):!1},g.Hb=function(){var t,n,s,a,u,p;return s=this.a==null?0:ii(this.a),t=s&Oi,n=s&-65536,p=this.b==null?0:ii(this.b),a=p&Oi,u=p&-65536,t^u>>16&Oi|n^a<<16},g.Kc=function(){return new BWe(this)},g.Ib=function(){return this.a==null&&this.b==null?"pair(null,null)":this.a==null?"pair(null,"+Ga(this.b)+")":this.b==null?"pair("+Ga(this.a)+",null)":"pair("+Ga(this.a)+","+Ga(this.b)+")"},D(xo,"Pair",46),M(983,1,ys,BWe),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return!this.c&&(!this.b&&this.a.a!=null||this.a.b!=null)},g.Pb=function(){if(!this.c&&!this.b&&this.a.a!=null)return this.b=!0,this.a.a;if(!this.c&&this.a.b!=null)return this.c=!0,this.a.b;throw ne(new lo)},g.Qb=function(){throw this.c&&this.a.b!=null?this.a.b=null:this.b&&this.a.a!=null&&(this.a.a=null),ne(new Ic)},g.b=!1,g.c=!1,D(xo,"Pair/1",983),M(448,1,{448:1},git),g.Fb=function(t){return Bo(this.a,h(t,448).a)&&Bo(this.c,h(t,448).c)&&Bo(this.d,h(t,448).d)&&Bo(this.b,h(t,448).b)},g.Hb=function(){return lF(se(ie(yn,1),st,1,5,[this.a,this.c,this.d,this.b]))},g.Ib=function(){return"("+this.a+Js+this.c+Js+this.d+Js+this.b+")"},D(xo,"Quadruple",448),M(1126,209,t2,Cze),g.Ze=function(t,n){var s,a,u,p,v;if(Bn(n,"Random Layout",1),(!t.a&&(t.a=new Ye(wi,t,10,11)),t.a).i==0){An(n);return}p=h(pt(t,(jpe(),u9t)),19),!!p&&p.a!=0?u=new vB(p.a):u=new mX,s=bS(Je(pt(t,o9t))),v=bS(Je(pt(t,l9t))),a=h(pt(t,c9t),116),XOn(t,u,s,v,a),An(n)},D(xo,"RandomLayoutProvider",1126);var v9t;M(553,1,{}),g.qf=function(){return new dt(this.f.i,this.f.j)},g.We=function(t){return $it(t,(Tr(),pu))?pt(this.f,w9t):pt(this.f,t)},g.rf=function(){return new dt(this.f.g,this.f.f)},g.sf=function(){return this.g},g.Xe=function(t){return Pp(this.f,t)},g.tf=function(t){yc(this.f,t.a),kc(this.f,t.b)},g.uf=function(t){Eb(this.f,t.a),xb(this.f,t.b)},g.vf=function(t){this.g=t},g.g=0;var w9t;D(Z9,"ElkGraphAdapters/AbstractElkGraphElementAdapter",553),M(554,1,{839:1},JO),g.wf=function(){var t,n;if(!this.b)for(this.b=dB(UP(this.a).i),n=new Tn(UP(this.a));n.e!=n.i.gc();)t=h(On(n),137),Ue(this.b,new Bq(t));return this.b},g.b=null,D(Z9,"ElkGraphAdapters/ElkEdgeAdapter",554),M(301,553,{},Yv),g.xf=function(){return g1t(this)},g.a=null,D(Z9,"ElkGraphAdapters/ElkGraphAdapter",301),M(630,553,{181:1},Bq),D(Z9,"ElkGraphAdapters/ElkLabelAdapter",630),M(629,553,{680:1},PY),g.wf=function(){return Z8n(this)},g.Af=function(){var t;return t=h(pt(this.f,(Tr(),eL)),142),!t&&(t=new B7),t},g.Cf=function(){return J8n(this)},g.Ef=function(t){var n;n=new XY(t),ya(this.f,(Tr(),eL),n)},g.Ff=function(t){ya(this.f,(Tr(),y2),new N1e(t))},g.yf=function(){return this.d},g.zf=function(){var t,n;if(!this.a)for(this.a=new qe,n=new Sn(Dn(wM(h(this.f,33)).a.Kc(),new L));rr(n);)t=h(Un(n),79),Ue(this.a,new JO(t));return this.a},g.Bf=function(){var t,n;if(!this.c)for(this.c=new qe,n=new Sn(Dn(Pd(h(this.f,33)).a.Kc(),new L));rr(n);)t=h(Un(n),79),Ue(this.c,new JO(t));return this.c},g.Df=function(){return QP(h(this.f,33)).i!=0||ft(lt(h(this.f,33).We((Tr(),JD))))},g.Gf=function(){G3n(this,(Qv(),v9t))},g.a=null,g.b=null,g.c=null,g.d=null,g.e=null,D(Z9,"ElkGraphAdapters/ElkNodeAdapter",629),M(1266,553,{838:1},hKe),g.wf=function(){return o7n(this)},g.zf=function(){var t,n;if(!this.a)for(this.a=I1(h(this.f,118).xg().i),n=new Tn(h(this.f,118).xg());n.e!=n.i.gc();)t=h(On(n),79),Ue(this.a,new JO(t));return this.a},g.Bf=function(){var t,n;if(!this.c)for(this.c=I1(h(this.f,118).yg().i),n=new Tn(h(this.f,118).yg());n.e!=n.i.gc();)t=h(On(n),79),Ue(this.c,new JO(t));return this.c},g.Hf=function(){return h(h(this.f,118).We((Tr(),T3)),61)},g.If=function(){var t,n,s,a,u,p,v,m;for(a=mf(h(this.f,118)),s=new Tn(h(this.f,118).yg());s.e!=s.i.gc();)for(t=h(On(s),79),m=new Tn((!t.c&&(t.c=new qt(Pn,t,5,8)),t.c));m.e!=m.i.gc();){if(v=h(On(m),82),ww(Na(v),a))return!0;if(Na(v)==a&&ft(lt(pt(t,(Tr(),Fre)))))return!0}for(n=new Tn(h(this.f,118).xg());n.e!=n.i.gc();)for(t=h(On(n),79),p=new Tn((!t.b&&(t.b=new qt(Pn,t,4,7)),t.b));p.e!=p.i.gc();)if(u=h(On(p),82),ww(Na(u),a))return!0;return!1},g.a=null,g.b=null,g.c=null,D(Z9,"ElkGraphAdapters/ElkPortAdapter",1266),M(1267,1,Yr,Sze),g.ue=function(t,n){return MMn(h(t,118),h(n,118))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(Z9,"ElkGraphAdapters/PortComparator",1267);var Jg=fi(yl,"EObject"),E8=fi(Qy,nmt),Zl=fi(Qy,rmt),hL=fi(Qy,imt),fL=fi(Qy,"ElkShape"),Pn=fi(Qy,smt),us=fi(Qy,Dye),gi=fi(Qy,amt),dL=fi(yl,omt),cT=fi(yl,"EFactory"),m9t,Qre=fi(yl,cmt),sf=fi(yl,"EPackage"),ps,y9t,k9t,rEe,zH,x9t,iEe,sEe,aEe,ep,E9t,_9t,Ua=fi(Qy,Lye),wi=fi(Qy,Iye),bu=fi(Qy,Oye);M(90,1,umt),g.Jg=function(){return this.Kg(),null},g.Kg=function(){return null},g.Lg=function(){return this.Kg(),!1},g.Mg=function(){return!1},g.Ng=function(t){Or(this,t)},D(S5,"BasicNotifierImpl",90),M(97,90,dmt),g.nh=function(){return Tu(this)},g.Og=function(t,n){return t},g.Pg=function(){throw ne(new Xn)},g.Qg=function(t){var n;return n=aa(h(Ht(this.Tg(),this.Vg()),18)),this.eh().ih(this,n.n,n.f,t)},g.Rg=function(t,n){throw ne(new Xn)},g.Sg=function(t,n,s){return Ku(this,t,n,s)},g.Tg=function(){var t;return this.Pg()&&(t=this.Pg().ck(),t)?t:this.zh()},g.Ug=function(){return xQ(this)},g.Vg=function(){throw ne(new Xn)},g.Wg=function(){var t,n;return n=this.ph().dk(),!n&&this.Pg().ik(n=(Q7(),t=Qde(s1(this.Tg())),t==null?sie:new $S(this,t))),n},g.Xg=function(t,n){return t},g.Yg=function(t){var n;return n=t.Gj(),n?t.aj():oi(this.Tg(),t)},g.Zg=function(){var t;return t=this.Pg(),t?t.fk():null},g.$g=function(){return this.Pg()?this.Pg().ck():null},g._g=function(t,n,s){return IF(this,t,n,s)},g.ah=function(t){return Ek(this,t)},g.bh=function(t,n){return dK(this,t,n)},g.dh=function(){var t;return t=this.Pg(),!!t&&t.gk()},g.eh=function(){throw ne(new Xn)},g.fh=function(){return TF(this)},g.gh=function(t,n,s,a){return a5(this,t,n,a)},g.hh=function(t,n,s){var a;return a=h(Ht(this.Tg(),n),66),a.Nj().Qj(this,this.yh(),n-this.Ah(),t,s)},g.ih=function(t,n,s,a){return tB(this,t,n,a)},g.jh=function(t,n,s){var a;return a=h(Ht(this.Tg(),n),66),a.Nj().Rj(this,this.yh(),n-this.Ah(),t,s)},g.kh=function(){return!!this.Pg()&&!!this.Pg().ek()},g.lh=function(t){return OX(this,t)},g.mh=function(t){return rst(this,t)},g.oh=function(t){return A2t(this,t)},g.ph=function(){throw ne(new Xn)},g.qh=function(){return this.Pg()?this.Pg().ek():null},g.rh=function(){return TF(this)},g.sh=function(t,n){vQ(this,t,n)},g.th=function(t){this.ph().hk(t)},g.uh=function(t){this.ph().kk(t)},g.vh=function(t){this.ph().jk(t)},g.wh=function(t,n){var s,a,u,p;return p=this.Zg(),!!p&&!!t&&(n=qs(p.Vk(),this,n),p.Zk(this)),a=this.eh(),a&&((RQ(this,this.eh(),this.Vg()).Bb&ea)!=0?(u=a.fh(),u&&(t?!p&&u.Zk(this):u.Yk(this))):(n=(s=this.Vg(),s>=0?this.Qg(n):this.eh().ih(this,-1-s,null,n)),n=this.Sg(null,-1,n))),this.uh(t),n},g.xh=function(t){var n,s,a,u,p,v,m,x;if(s=this.Tg(),p=oi(s,t),n=this.Ah(),p>=n)return h(t,66).Nj().Uj(this,this.yh(),p-n);if(p<=-1)if(v=Hy((jc(),Ps),s,t),v){if(ia(),h(v,66).Oj()||(v=Y4(Sa(Ps,v))),u=(a=this.Yg(v),h(a>=0?this._g(a,!0,!0):Bb(this,v,!0),153)),x=v.Zj(),x>1||x==-1)return h(h(u,215).hl(t,!1),76)}else throw ne(new an(Fg+t.ne()+gee));else if(t.$j())return a=this.Yg(t),h(a>=0?this._g(a,!1,!0):Bb(this,t,!1),76);return m=new kJe(this,t),m},g.yh=function(){return Z0e(this)},g.zh=function(){return(bg(),Xt).S},g.Ah=function(){return En(this.zh())},g.Bh=function(t){fQ(this,t)},g.Ib=function(){return _h(this)},D(Qt,"BasicEObjectImpl",97);var T9t;M(114,97,{105:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1}),g.Ch=function(t){var n;return n=J0e(this),n[t]},g.Dh=function(t,n){var s;s=J0e(this),pi(s,t,n)},g.Eh=function(t){var n;n=J0e(this),pi(n,t,null)},g.Jg=function(){return h(tn(this,4),126)},g.Kg=function(){throw ne(new Xn)},g.Lg=function(){return(this.Db&4)!=0},g.Pg=function(){throw ne(new Xn)},g.Fh=function(t){s5(this,2,t)},g.Rg=function(t,n){this.Db=n<<16|this.Db&255,this.Fh(t)},g.Tg=function(){return bc(this)},g.Vg=function(){return this.Db>>16},g.Wg=function(){var t,n;return Q7(),n=Qde(s1((t=h(tn(this,16),26),t||this.zh()))),n==null?sie:new $S(this,n)},g.Mg=function(){return(this.Db&1)==0},g.Zg=function(){return h(tn(this,128),1935)},g.$g=function(){return h(tn(this,16),26)},g.dh=function(){return(this.Db&32)!=0},g.eh=function(){return h(tn(this,2),49)},g.kh=function(){return(this.Db&64)!=0},g.ph=function(){throw ne(new Xn)},g.qh=function(){return h(tn(this,64),281)},g.th=function(t){s5(this,16,t)},g.uh=function(t){s5(this,128,t)},g.vh=function(t){s5(this,64,t)},g.yh=function(){return ic(this)},g.Db=0,D(Qt,"MinimalEObjectImpl",114),M(115,114,{105:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1}),g.Fh=function(t){this.Cb=t},g.eh=function(){return this.Cb},D(Qt,"MinimalEObjectImpl/Container",115),M(1985,115,{105:1,413:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1}),g._g=function(t,n,s){return l2e(this,t,n,s)},g.jh=function(t,n,s){return Z2e(this,t,n,s)},g.lh=function(t){return u0e(this,t)},g.sh=function(t,n){Kge(this,t,n)},g.zh=function(){return Qo(),_9t},g.Bh=function(t){Rge(this,t)},g.Ve=function(){return Pft(this)},g.We=function(t){return pt(this,t)},g.Xe=function(t){return Pp(this,t)},g.Ye=function(t,n){return ya(this,t,n)},D(i2,"EMapPropertyHolderImpl",1985),M(567,115,{105:1,469:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1},zO),g._g=function(t,n,s){switch(t){case 0:return this.a;case 1:return this.b}return IF(this,t,n,s)},g.lh=function(t){switch(t){case 0:return this.a!=0;case 1:return this.b!=0}return OX(this,t)},g.sh=function(t,n){switch(t){case 0:$B(this,Pe(Je(n)));return;case 1:HB(this,Pe(Je(n)));return}vQ(this,t,n)},g.zh=function(){return Qo(),y9t},g.Bh=function(t){switch(t){case 0:$B(this,0);return;case 1:HB(this,0);return}fQ(this,t)},g.Ib=function(){var t;return(this.Db&64)!=0?_h(this):(t=new Ol(_h(this)),t.a+=" (x: ",ny(t,this.a),t.a+=", y: ",ny(t,this.b),t.a+=")",t.a)},g.a=0,g.b=0,D(i2,"ElkBendPointImpl",567),M(723,1985,{105:1,413:1,160:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1}),g._g=function(t,n,s){return gpe(this,t,n,s)},g.hh=function(t,n,s){return aQ(this,t,n,s)},g.jh=function(t,n,s){return VK(this,t,n,s)},g.lh=function(t){return Dge(this,t)},g.sh=function(t,n){M2e(this,t,n)},g.zh=function(){return Qo(),x9t},g.Bh=function(t){lpe(this,t)},g.zg=function(){return this.k},g.Ag=function(){return UP(this)},g.Ib=function(){return vX(this)},g.k=null,D(i2,"ElkGraphElementImpl",723),M(724,723,{105:1,413:1,160:1,470:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1}),g._g=function(t,n,s){return _pe(this,t,n,s)},g.lh=function(t){return Lpe(this,t)},g.sh=function(t,n){D2e(this,t,n)},g.zh=function(){return Qo(),E9t},g.Bh=function(t){Bpe(this,t)},g.Bg=function(){return this.f},g.Cg=function(){return this.g},g.Dg=function(){return this.i},g.Eg=function(){return this.j},g.Fg=function(t,n){ZN(this,t,n)},g.Gg=function(t,n){bf(this,t,n)},g.Hg=function(t){yc(this,t)},g.Ig=function(t){kc(this,t)},g.Ib=function(){return hQ(this)},g.f=0,g.g=0,g.i=0,g.j=0,D(i2,"ElkShapeImpl",724),M(725,724,{105:1,413:1,82:1,160:1,470:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1}),g._g=function(t,n,s){return i2e(this,t,n,s)},g.hh=function(t,n,s){return E2e(this,t,n,s)},g.jh=function(t,n,s){return _2e(this,t,n,s)},g.lh=function(t){return Yge(this,t)},g.sh=function(t,n){Obe(this,t,n)},g.zh=function(){return Qo(),k9t},g.Bh=function(t){Xpe(this,t)},g.xg=function(){return!this.d&&(this.d=new qt(us,this,8,5)),this.d},g.yg=function(){return!this.e&&(this.e=new qt(us,this,7,4)),this.e},D(i2,"ElkConnectableShapeImpl",725),M(352,723,{105:1,413:1,79:1,160:1,352:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1},Que),g.Qg=function(t){return m2e(this,t)},g._g=function(t,n,s){switch(t){case 3:return uA(this);case 4:return!this.b&&(this.b=new qt(Pn,this,4,7)),this.b;case 5:return!this.c&&(this.c=new qt(Pn,this,5,8)),this.c;case 6:return!this.a&&(this.a=new Ye(gi,this,6,6)),this.a;case 7:return on(),!this.b&&(this.b=new qt(Pn,this,4,7)),!(this.b.i<=1&&(!this.c&&(this.c=new qt(Pn,this,5,8)),this.c.i<=1));case 8:return on(),!!y9(this);case 9:return on(),!!Pb(this);case 10:return on(),!this.b&&(this.b=new qt(Pn,this,4,7)),this.b.i!=0&&(!this.c&&(this.c=new qt(Pn,this,5,8)),this.c.i!=0)}return gpe(this,t,n,s)},g.hh=function(t,n,s){var a;switch(n){case 3:return this.Cb&&(s=(a=this.Db>>16,a>=0?m2e(this,s):this.Cb.ih(this,-1-a,null,s))),f1e(this,h(t,33),s);case 4:return!this.b&&(this.b=new qt(Pn,this,4,7)),Xo(this.b,t,s);case 5:return!this.c&&(this.c=new qt(Pn,this,5,8)),Xo(this.c,t,s);case 6:return!this.a&&(this.a=new Ye(gi,this,6,6)),Xo(this.a,t,s)}return aQ(this,t,n,s)},g.jh=function(t,n,s){switch(n){case 3:return f1e(this,null,s);case 4:return!this.b&&(this.b=new qt(Pn,this,4,7)),qs(this.b,t,s);case 5:return!this.c&&(this.c=new qt(Pn,this,5,8)),qs(this.c,t,s);case 6:return!this.a&&(this.a=new Ye(gi,this,6,6)),qs(this.a,t,s)}return VK(this,t,n,s)},g.lh=function(t){switch(t){case 3:return!!uA(this);case 4:return!!this.b&&this.b.i!=0;case 5:return!!this.c&&this.c.i!=0;case 6:return!!this.a&&this.a.i!=0;case 7:return!this.b&&(this.b=new qt(Pn,this,4,7)),!(this.b.i<=1&&(!this.c&&(this.c=new qt(Pn,this,5,8)),this.c.i<=1));case 8:return y9(this);case 9:return Pb(this);case 10:return!this.b&&(this.b=new qt(Pn,this,4,7)),this.b.i!=0&&(!this.c&&(this.c=new qt(Pn,this,5,8)),this.c.i!=0)}return Dge(this,t)},g.sh=function(t,n){switch(t){case 3:AQ(this,h(n,33));return;case 4:!this.b&&(this.b=new qt(Pn,this,4,7)),jn(this.b),!this.b&&(this.b=new qt(Pn,this,4,7)),mi(this.b,h(n,14));return;case 5:!this.c&&(this.c=new qt(Pn,this,5,8)),jn(this.c),!this.c&&(this.c=new qt(Pn,this,5,8)),mi(this.c,h(n,14));return;case 6:!this.a&&(this.a=new Ye(gi,this,6,6)),jn(this.a),!this.a&&(this.a=new Ye(gi,this,6,6)),mi(this.a,h(n,14));return}M2e(this,t,n)},g.zh=function(){return Qo(),rEe},g.Bh=function(t){switch(t){case 3:AQ(this,null);return;case 4:!this.b&&(this.b=new qt(Pn,this,4,7)),jn(this.b);return;case 5:!this.c&&(this.c=new qt(Pn,this,5,8)),jn(this.c);return;case 6:!this.a&&(this.a=new Ye(gi,this,6,6)),jn(this.a);return}lpe(this,t)},g.Ib=function(){return d2t(this)},D(i2,"ElkEdgeImpl",352),M(439,1985,{105:1,413:1,202:1,439:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1},GO),g.Qg=function(t){return p2e(this,t)},g._g=function(t,n,s){switch(t){case 1:return this.j;case 2:return this.k;case 3:return this.b;case 4:return this.c;case 5:return!this.a&&(this.a=new Gi(Zl,this,5)),this.a;case 6:return est(this);case 7:return n?RX(this):this.i;case 8:return n?FX(this):this.f;case 9:return!this.g&&(this.g=new qt(gi,this,9,10)),this.g;case 10:return!this.e&&(this.e=new qt(gi,this,10,9)),this.e;case 11:return this.d}return l2e(this,t,n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 6:return this.Cb&&(s=(u=this.Db>>16,u>=0?p2e(this,s):this.Cb.ih(this,-1-u,null,s))),d1e(this,h(t,79),s);case 9:return!this.g&&(this.g=new qt(gi,this,9,10)),Xo(this.g,t,s);case 10:return!this.e&&(this.e=new qt(gi,this,10,9)),Xo(this.e,t,s)}return p=h(Ht((a=h(tn(this,16),26),a||(Qo(),zH)),n),66),p.Nj().Qj(this,ic(this),n-En((Qo(),zH)),t,s)},g.jh=function(t,n,s){switch(n){case 5:return!this.a&&(this.a=new Gi(Zl,this,5)),qs(this.a,t,s);case 6:return d1e(this,null,s);case 9:return!this.g&&(this.g=new qt(gi,this,9,10)),qs(this.g,t,s);case 10:return!this.e&&(this.e=new qt(gi,this,10,9)),qs(this.e,t,s)}return Z2e(this,t,n,s)},g.lh=function(t){switch(t){case 1:return this.j!=0;case 2:return this.k!=0;case 3:return this.b!=0;case 4:return this.c!=0;case 5:return!!this.a&&this.a.i!=0;case 6:return!!est(this);case 7:return!!this.i;case 8:return!!this.f;case 9:return!!this.g&&this.g.i!=0;case 10:return!!this.e&&this.e.i!=0;case 11:return this.d!=null}return u0e(this,t)},g.sh=function(t,n){switch(t){case 1:Mk(this,Pe(Je(n)));return;case 2:Lk(this,Pe(Je(n)));return;case 3:Ak(this,Pe(Je(n)));return;case 4:Dk(this,Pe(Je(n)));return;case 5:!this.a&&(this.a=new Gi(Zl,this,5)),jn(this.a),!this.a&&(this.a=new Gi(Zl,this,5)),mi(this.a,h(n,14));return;case 6:ggt(this,h(n,79));return;case 7:qB(this,h(n,82));return;case 8:UB(this,h(n,82));return;case 9:!this.g&&(this.g=new qt(gi,this,9,10)),jn(this.g),!this.g&&(this.g=new qt(gi,this,9,10)),mi(this.g,h(n,14));return;case 10:!this.e&&(this.e=new qt(gi,this,10,9)),jn(this.e),!this.e&&(this.e=new qt(gi,this,10,9)),mi(this.e,h(n,14));return;case 11:kge(this,Jn(n));return}Kge(this,t,n)},g.zh=function(){return Qo(),zH},g.Bh=function(t){switch(t){case 1:Mk(this,0);return;case 2:Lk(this,0);return;case 3:Ak(this,0);return;case 4:Dk(this,0);return;case 5:!this.a&&(this.a=new Gi(Zl,this,5)),jn(this.a);return;case 6:ggt(this,null);return;case 7:qB(this,null);return;case 8:UB(this,null);return;case 9:!this.g&&(this.g=new qt(gi,this,9,10)),jn(this.g);return;case 10:!this.e&&(this.e=new qt(gi,this,10,9)),jn(this.e);return;case 11:kge(this,null);return}Rge(this,t)},g.Ib=function(){return I0t(this)},g.b=0,g.c=0,g.d=null,g.j=0,g.k=0,D(i2,"ElkEdgeSectionImpl",439),M(150,115,{105:1,92:1,90:1,147:1,56:1,108:1,49:1,97:1,150:1,114:1,115:1}),g._g=function(t,n,s){var a;return t==0?(!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab):gl(this,t-En(this.zh()),Ht((a=h(tn(this,16),26),a||this.zh()),t),n,s)},g.hh=function(t,n,s){var a,u;return n==0?(!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s)):(u=h(Ht((a=h(tn(this,16),26),a||this.zh()),n),66),u.Nj().Qj(this,ic(this),n-En(this.zh()),t,s))},g.jh=function(t,n,s){var a,u;return n==0?(!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s)):(u=h(Ht((a=h(tn(this,16),26),a||this.zh()),n),66),u.Nj().Rj(this,ic(this),n-En(this.zh()),t,s))},g.lh=function(t){var n;return t==0?!!this.Ab&&this.Ab.i!=0:fl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.oh=function(t){return Hve(this,t)},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return}ml(this,t-En(this.zh()),Ht((s=h(tn(this,16),26),s||this.zh()),t),n)},g.uh=function(t){s5(this,128,t)},g.zh=function(){return Bt(),G9t},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return}vl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.Gh=function(){this.Bb|=1},g.Hh=function(t){return T9(this,t)},g.Bb=0,D(Qt,"EModelElementImpl",150),M(704,150,{105:1,92:1,90:1,471:1,147:1,56:1,108:1,49:1,97:1,150:1,114:1,115:1},rle),g.Ih=function(t,n){return G2t(this,t,n)},g.Jh=function(t){var n,s,a,u,p;if(this.a!=Vu(t)||(t.Bb&256)!=0)throw ne(new an(bee+t.zb+Wb));for(a=La(t);Co(a.a).i!=0;){if(s=h(MM(a,0,(n=h(he(Co(a.a),0),87),p=n.c,ue(p,88)?h(p,26):(Bt(),nh))),26),Nb(s))return u=Vu(s).Nh().Jh(s),h(u,49).th(t),u;a=La(s)}return(t.D!=null?t.D:t.B)=="java.util.Map$Entry"?new Rnt(t):new J1e(t)},g.Kh=function(t,n){return Hb(this,t,n)},g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.a}return gl(this,t-En((Bt(),ip)),Ht((a=h(tn(this,16),26),a||ip),t),n,s)},g.hh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 1:return this.a&&(s=h(this.a,49).ih(this,4,sf,s)),cpe(this,h(t,235),s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),ip)),n),66),u.Nj().Qj(this,ic(this),n-En((Bt(),ip)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 1:return cpe(this,null,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),ip)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),ip)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return!!this.a}return fl(this,t-En((Bt(),ip)),Ht((n=h(tn(this,16),26),n||ip),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:gdt(this,h(n,235));return}ml(this,t-En((Bt(),ip)),Ht((s=h(tn(this,16),26),s||ip),t),n)},g.zh=function(){return Bt(),ip},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:gdt(this,null);return}vl(this,t-En((Bt(),ip)),Ht((n=h(tn(this,16),26),n||ip),t))};var uT,oEe,C9t;D(Qt,"EFactoryImpl",704),M(Ch,704,{105:1,2014:1,92:1,90:1,471:1,147:1,56:1,108:1,49:1,97:1,150:1,114:1,115:1},Mze),g.Ih=function(t,n){switch(t.yj()){case 12:return h(n,146).tg();case 13:return Ga(n);default:throw ne(new an(Px+t.ne()+Wb))}},g.Jh=function(t){var n,s,a,u,p,v,m,x;switch(t.G==-1&&(t.G=(n=Vu(t),n?y0(n.Mh(),t):-1)),t.G){case 4:return p=new Zue,p;case 6:return v=new nhe,v;case 7:return m=new rhe,m;case 8:return a=new Que,a;case 9:return s=new zO,s;case 10:return u=new GO,u;case 11:return x=new Dze,x;default:throw ne(new an(bee+t.zb+Wb))}},g.Kh=function(t,n){switch(t.yj()){case 13:case 12:return null;default:throw ne(new an(Px+t.ne()+Wb))}},D(i2,"ElkGraphFactoryImpl",Ch),M(438,150,{105:1,92:1,90:1,147:1,191:1,56:1,108:1,49:1,97:1,150:1,114:1,115:1}),g.Wg=function(){var t,n;return n=(t=h(tn(this,16),26),Qde(s1(t||this.zh()))),n==null?(Q7(),Q7(),sie):new stt(this,n)},g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.ne()}return gl(this,t-En(this.zh()),Ht((a=h(tn(this,16),26),a||this.zh()),t),n,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null}return fl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:this.Lh(Jn(n));return}ml(this,t-En(this.zh()),Ht((s=h(tn(this,16),26),s||this.zh()),t),n)},g.zh=function(){return Bt(),V9t},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:this.Lh(null);return}vl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.ne=function(){return this.zb},g.Lh=function(t){Ko(this,t)},g.Ib=function(){return n9(this)},g.zb=null,D(Qt,"ENamedElementImpl",438),M(179,438,{105:1,92:1,90:1,147:1,191:1,56:1,235:1,108:1,49:1,97:1,150:1,179:1,114:1,115:1,675:1},jit),g.Qg=function(t){return A1t(this,t)},g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return this.yb;case 3:return this.xb;case 4:return this.sb;case 5:return!this.rb&&(this.rb=new cw(this,af,this)),this.rb;case 6:return!this.vb&&(this.vb=new B4(sf,this,6,7)),this.vb;case 7:return n?this.Db>>16==7?h(this.Cb,235):null:tst(this)}return gl(this,t-En((Bt(),$0)),Ht((a=h(tn(this,16),26),a||$0),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 4:return this.sb&&(s=h(this.sb,49).ih(this,1,cT,s)),fpe(this,h(t,471),s);case 5:return!this.rb&&(this.rb=new cw(this,af,this)),Xo(this.rb,t,s);case 6:return!this.vb&&(this.vb=new B4(sf,this,6,7)),Xo(this.vb,t,s);case 7:return this.Cb&&(s=(u=this.Db>>16,u>=0?A1t(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,7,s)}return p=h(Ht((a=h(tn(this,16),26),a||(Bt(),$0)),n),66),p.Nj().Qj(this,ic(this),n-En((Bt(),$0)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 4:return fpe(this,null,s);case 5:return!this.rb&&(this.rb=new cw(this,af,this)),qs(this.rb,t,s);case 6:return!this.vb&&(this.vb=new B4(sf,this,6,7)),qs(this.vb,t,s);case 7:return Ku(this,null,7,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),$0)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),$0)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return this.yb!=null;case 3:return this.xb!=null;case 4:return!!this.sb;case 5:return!!this.rb&&this.rb.i!=0;case 6:return!!this.vb&&this.vb.i!=0;case 7:return!!tst(this)}return fl(this,t-En((Bt(),$0)),Ht((n=h(tn(this,16),26),n||$0),t))},g.oh=function(t){var n;return n=QSn(this,t),n||Hve(this,t)},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:Ko(this,Jn(n));return;case 2:JB(this,Jn(n));return;case 3:ZB(this,Jn(n));return;case 4:lQ(this,h(n,471));return;case 5:!this.rb&&(this.rb=new cw(this,af,this)),jn(this.rb),!this.rb&&(this.rb=new cw(this,af,this)),mi(this.rb,h(n,14));return;case 6:!this.vb&&(this.vb=new B4(sf,this,6,7)),jn(this.vb),!this.vb&&(this.vb=new B4(sf,this,6,7)),mi(this.vb,h(n,14));return}ml(this,t-En((Bt(),$0)),Ht((s=h(tn(this,16),26),s||$0),t),n)},g.vh=function(t){var n,s;if(!!t&&!!this.rb)for(s=new Tn(this.rb);s.e!=s.i.gc();)n=On(s),ue(n,351)&&(h(n,351).w=null);s5(this,64,t)},g.zh=function(){return Bt(),$0},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:Ko(this,null);return;case 2:JB(this,null);return;case 3:ZB(this,null);return;case 4:lQ(this,null);return;case 5:!this.rb&&(this.rb=new cw(this,af,this)),jn(this.rb);return;case 6:!this.vb&&(this.vb=new B4(sf,this,6,7)),jn(this.vb);return}vl(this,t-En((Bt(),$0)),Ht((n=h(tn(this,16),26),n||$0),t))},g.Gh=function(){QX(this)},g.Mh=function(){return!this.rb&&(this.rb=new cw(this,af,this)),this.rb},g.Nh=function(){return this.sb},g.Oh=function(){return this.ub},g.Ph=function(){return this.xb},g.Qh=function(){return this.yb},g.Rh=function(t){this.ub=t},g.Ib=function(){var t;return(this.Db&64)!=0?n9(this):(t=new Ol(n9(this)),t.a+=" (nsURI: ",va(t,this.yb),t.a+=", nsPrefix: ",va(t,this.xb),t.a+=")",t.a)},g.xb=null,g.yb=null,D(Qt,"EPackageImpl",179),M(555,179,{105:1,2016:1,555:1,92:1,90:1,147:1,191:1,56:1,235:1,108:1,49:1,97:1,150:1,179:1,114:1,115:1,675:1},H0t),g.q=!1,g.r=!1;var S9t=!1;D(i2,"ElkGraphPackageImpl",555),M(354,724,{105:1,413:1,160:1,137:1,470:1,354:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1},Zue),g.Qg=function(t){return b2e(this,t)},g._g=function(t,n,s){switch(t){case 7:return nst(this);case 8:return this.a}return _pe(this,t,n,s)},g.hh=function(t,n,s){var a;switch(n){case 7:return this.Cb&&(s=(a=this.Db>>16,a>=0?b2e(this,s):this.Cb.ih(this,-1-a,null,s))),gde(this,h(t,160),s)}return aQ(this,t,n,s)},g.jh=function(t,n,s){return n==7?gde(this,null,s):VK(this,t,n,s)},g.lh=function(t){switch(t){case 7:return!!nst(this);case 8:return!Nt("",this.a)}return Lpe(this,t)},g.sh=function(t,n){switch(t){case 7:Ybe(this,h(n,160));return;case 8:gge(this,Jn(n));return}D2e(this,t,n)},g.zh=function(){return Qo(),iEe},g.Bh=function(t){switch(t){case 7:Ybe(this,null);return;case 8:gge(this,"");return}Bpe(this,t)},g.Ib=function(){return Adt(this)},g.a="",D(i2,"ElkLabelImpl",354),M(239,725,{105:1,413:1,82:1,160:1,33:1,470:1,239:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1},nhe),g.Qg=function(t){return y2e(this,t)},g._g=function(t,n,s){switch(t){case 9:return!this.c&&(this.c=new Ye(bu,this,9,9)),this.c;case 10:return!this.a&&(this.a=new Ye(wi,this,10,11)),this.a;case 11:return bi(this);case 12:return!this.b&&(this.b=new Ye(us,this,12,3)),this.b;case 13:return on(),!this.a&&(this.a=new Ye(wi,this,10,11)),this.a.i>0}return i2e(this,t,n,s)},g.hh=function(t,n,s){var a;switch(n){case 9:return!this.c&&(this.c=new Ye(bu,this,9,9)),Xo(this.c,t,s);case 10:return!this.a&&(this.a=new Ye(wi,this,10,11)),Xo(this.a,t,s);case 11:return this.Cb&&(s=(a=this.Db>>16,a>=0?y2e(this,s):this.Cb.ih(this,-1-a,null,s))),x1e(this,h(t,33),s);case 12:return!this.b&&(this.b=new Ye(us,this,12,3)),Xo(this.b,t,s)}return E2e(this,t,n,s)},g.jh=function(t,n,s){switch(n){case 9:return!this.c&&(this.c=new Ye(bu,this,9,9)),qs(this.c,t,s);case 10:return!this.a&&(this.a=new Ye(wi,this,10,11)),qs(this.a,t,s);case 11:return x1e(this,null,s);case 12:return!this.b&&(this.b=new Ye(us,this,12,3)),qs(this.b,t,s)}return _2e(this,t,n,s)},g.lh=function(t){switch(t){case 9:return!!this.c&&this.c.i!=0;case 10:return!!this.a&&this.a.i!=0;case 11:return!!bi(this);case 12:return!!this.b&&this.b.i!=0;case 13:return!this.a&&(this.a=new Ye(wi,this,10,11)),this.a.i>0}return Yge(this,t)},g.sh=function(t,n){switch(t){case 9:!this.c&&(this.c=new Ye(bu,this,9,9)),jn(this.c),!this.c&&(this.c=new Ye(bu,this,9,9)),mi(this.c,h(n,14));return;case 10:!this.a&&(this.a=new Ye(wi,this,10,11)),jn(this.a),!this.a&&(this.a=new Ye(wi,this,10,11)),mi(this.a,h(n,14));return;case 11:Vbe(this,h(n,33));return;case 12:!this.b&&(this.b=new Ye(us,this,12,3)),jn(this.b),!this.b&&(this.b=new Ye(us,this,12,3)),mi(this.b,h(n,14));return}Obe(this,t,n)},g.zh=function(){return Qo(),sEe},g.Bh=function(t){switch(t){case 9:!this.c&&(this.c=new Ye(bu,this,9,9)),jn(this.c);return;case 10:!this.a&&(this.a=new Ye(wi,this,10,11)),jn(this.a);return;case 11:Vbe(this,null);return;case 12:!this.b&&(this.b=new Ye(us,this,12,3)),jn(this.b);return}Xpe(this,t)},g.Ib=function(){return cve(this)},D(i2,"ElkNodeImpl",239),M(186,725,{105:1,413:1,82:1,160:1,118:1,470:1,186:1,94:1,92:1,90:1,56:1,108:1,49:1,97:1,114:1,115:1},rhe),g.Qg=function(t){return v2e(this,t)},g._g=function(t,n,s){return t==9?mf(this):i2e(this,t,n,s)},g.hh=function(t,n,s){var a;switch(n){case 9:return this.Cb&&(s=(a=this.Db>>16,a>=0?v2e(this,s):this.Cb.ih(this,-1-a,null,s))),g1e(this,h(t,33),s)}return E2e(this,t,n,s)},g.jh=function(t,n,s){return n==9?g1e(this,null,s):_2e(this,t,n,s)},g.lh=function(t){return t==9?!!mf(this):Yge(this,t)},g.sh=function(t,n){switch(t){case 9:Gbe(this,h(n,33));return}Obe(this,t,n)},g.zh=function(){return Qo(),aEe},g.Bh=function(t){switch(t){case 9:Gbe(this,null);return}Xpe(this,t)},g.Ib=function(){return gpt(this)},D(i2,"ElkPortImpl",186);var A9t=fi(Ys,"BasicEMap/Entry");M(1092,115,{105:1,42:1,92:1,90:1,133:1,56:1,108:1,49:1,97:1,114:1,115:1},Dze),g.Fb=function(t){return this===t},g.cd=function(){return this.b},g.Hb=function(){return rb(this)},g.Uh=function(t){pge(this,h(t,146))},g._g=function(t,n,s){switch(t){case 0:return this.b;case 1:return this.c}return IF(this,t,n,s)},g.lh=function(t){switch(t){case 0:return!!this.b;case 1:return this.c!=null}return OX(this,t)},g.sh=function(t,n){switch(t){case 0:pge(this,h(n,146));return;case 1:wge(this,n);return}vQ(this,t,n)},g.zh=function(){return Qo(),ep},g.Bh=function(t){switch(t){case 0:pge(this,null);return;case 1:wge(this,null);return}fQ(this,t)},g.Sh=function(){var t;return this.a==-1&&(t=this.b,this.a=t?ii(t):0),this.a},g.dd=function(){return this.c},g.Th=function(t){this.a=t},g.ed=function(t){var n;return n=this.c,wge(this,t),n},g.Ib=function(){var t;return(this.Db&64)!=0?_h(this):(t=new rg,or(or(or(t,this.b?this.b.tg():_c),dJ),mE(this.c)),t.a)},g.a=-1,g.c=null;var fv=D(i2,"ElkPropertyToValueMapEntryImpl",1092);M(984,1,{},Oze),D(Ns,"JsonAdapter",984),M(210,60,Fd,Xf),D(Ns,"JsonImportException",210),M(857,1,{},M1t),D(Ns,"JsonImporter",857),M(891,1,{},nJe),D(Ns,"JsonImporter/lambda$0$Type",891),M(892,1,{},rJe),D(Ns,"JsonImporter/lambda$1$Type",892),M(900,1,{},FWe),D(Ns,"JsonImporter/lambda$10$Type",900),M(902,1,{},iJe),D(Ns,"JsonImporter/lambda$11$Type",902),M(903,1,{},sJe),D(Ns,"JsonImporter/lambda$12$Type",903),M(909,1,{},mit),D(Ns,"JsonImporter/lambda$13$Type",909),M(908,1,{},wit),D(Ns,"JsonImporter/lambda$14$Type",908),M(904,1,{},aJe),D(Ns,"JsonImporter/lambda$15$Type",904),M(905,1,{},oJe),D(Ns,"JsonImporter/lambda$16$Type",905),M(906,1,{},cJe),D(Ns,"JsonImporter/lambda$17$Type",906),M(907,1,{},uJe),D(Ns,"JsonImporter/lambda$18$Type",907),M(912,1,{},RWe),D(Ns,"JsonImporter/lambda$19$Type",912),M(893,1,{},jWe),D(Ns,"JsonImporter/lambda$2$Type",893),M(910,1,{},$We),D(Ns,"JsonImporter/lambda$20$Type",910),M(911,1,{},HWe),D(Ns,"JsonImporter/lambda$21$Type",911),M(915,1,{},zWe),D(Ns,"JsonImporter/lambda$22$Type",915),M(913,1,{},GWe),D(Ns,"JsonImporter/lambda$23$Type",913),M(914,1,{},VWe),D(Ns,"JsonImporter/lambda$24$Type",914),M(917,1,{},UWe),D(Ns,"JsonImporter/lambda$25$Type",917),M(916,1,{},qWe),D(Ns,"JsonImporter/lambda$26$Type",916),M(918,1,mn,lJe),g.td=function(t){C3n(this.b,this.a,Jn(t))},D(Ns,"JsonImporter/lambda$27$Type",918),M(919,1,mn,hJe),g.td=function(t){S3n(this.b,this.a,Jn(t))},D(Ns,"JsonImporter/lambda$28$Type",919),M(920,1,{},fJe),D(Ns,"JsonImporter/lambda$29$Type",920),M(896,1,{},YWe),D(Ns,"JsonImporter/lambda$3$Type",896),M(921,1,{},dJe),D(Ns,"JsonImporter/lambda$30$Type",921),M(922,1,{},WWe),D(Ns,"JsonImporter/lambda$31$Type",922),M(923,1,{},KWe),D(Ns,"JsonImporter/lambda$32$Type",923),M(924,1,{},XWe),D(Ns,"JsonImporter/lambda$33$Type",924),M(925,1,{},QWe),D(Ns,"JsonImporter/lambda$34$Type",925),M(859,1,{},ZWe),D(Ns,"JsonImporter/lambda$35$Type",859),M(929,1,{},lnt),D(Ns,"JsonImporter/lambda$36$Type",929),M(926,1,mn,JWe),g.td=function(t){Dyn(this.a,h(t,469))},D(Ns,"JsonImporter/lambda$37$Type",926),M(927,1,mn,mJe),g.td=function(t){idn(this.a,this.b,h(t,202))},D(Ns,"JsonImporter/lambda$38$Type",927),M(928,1,mn,yJe),g.td=function(t){sdn(this.a,this.b,h(t,202))},D(Ns,"JsonImporter/lambda$39$Type",928),M(894,1,{},eKe),D(Ns,"JsonImporter/lambda$4$Type",894),M(930,1,mn,tKe),g.td=function(t){Lyn(this.a,h(t,8))},D(Ns,"JsonImporter/lambda$40$Type",930),M(895,1,{},nKe),D(Ns,"JsonImporter/lambda$5$Type",895),M(899,1,{},rKe),D(Ns,"JsonImporter/lambda$6$Type",899),M(897,1,{},iKe),D(Ns,"JsonImporter/lambda$7$Type",897),M(898,1,{},sKe),D(Ns,"JsonImporter/lambda$8$Type",898),M(901,1,{},aKe),D(Ns,"JsonImporter/lambda$9$Type",901),M(948,1,mn,oKe),g.td=function(t){z4(this.a,new uw(Jn(t)))},D(Ns,"JsonMetaDataConverter/lambda$0$Type",948),M(949,1,mn,cKe),g.td=function(t){Ybn(this.a,h(t,237))},D(Ns,"JsonMetaDataConverter/lambda$1$Type",949),M(950,1,mn,uKe),g.td=function(t){Vwn(this.a,h(t,149))},D(Ns,"JsonMetaDataConverter/lambda$2$Type",950),M(951,1,mn,lKe),g.td=function(t){Wbn(this.a,h(t,175))},D(Ns,"JsonMetaDataConverter/lambda$3$Type",951),M(237,22,{3:1,35:1,22:1,237:1},I4);var GH,VH,Zre,UH,qH,YH,Jre,eie,WH=tr(GM,"GraphFeature",237,sr,b4n,w2n),M9t;M(13,1,{35:1,146:1},Xr,Xi,$t,sa),g.wd=function(t){return r0n(this,h(t,146))},g.Fb=function(t){return $it(this,t)},g.wg=function(){return at(this)},g.tg=function(){return this.b},g.Hb=function(){return k0(this.b)},g.Ib=function(){return this.b},D(GM,"Property",13),M(818,1,Yr,Ple),g.ue=function(t,n){return kkn(this,h(t,94),h(n,94))},g.Fb=function(t){return this===t},g.ve=function(){return new Hr(this)},D(GM,"PropertyHolderComparator",818),M(695,1,ys,Ble),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return L3n(this)},g.Qb=function(){rQe()},g.Ob=function(){return!!this.a},D(dj,"ElkGraphUtil/AncestorIterator",695);var cEe=fi(Ys,"EList");M(67,52,{20:1,28:1,52:1,14:1,15:1,67:1,58:1}),g.Vc=function(t,n){s9(this,t,n)},g.Fc=function(t){return qn(this,t)},g.Wc=function(t,n){return jge(this,t,n)},g.Gc=function(t){return mi(this,t)},g.Zh=function(){return new P4(this)},g.$h=function(){return new HS(this)},g._h=function(t){return IA(this,t)},g.ai=function(){return!0},g.bi=function(t,n){},g.ci=function(){},g.di=function(t,n){pK(this,t,n)},g.ei=function(t,n,s){},g.fi=function(t,n){},g.gi=function(t,n,s){},g.Fb=function(t){return ept(this,t)},g.Hb=function(){return Nge(this)},g.hi=function(){return!1},g.Kc=function(){return new Tn(this)},g.Yc=function(){return new N4(this)},g.Zc=function(t){var n;if(n=this.gc(),t<0||t>n)throw ne(new sw(t,n));return new yW(this,t)},g.ji=function(t,n){this.ii(t,this.Xc(n))},g.Mc=function(t){return IB(this,t)},g.li=function(t,n){return n},g._c=function(t,n){return Sy(this,t,n)},g.Ib=function(){return Cpe(this)},g.ni=function(){return!0},g.oi=function(t,n){return Gk(this,n)},D(Ys,"AbstractEList",67),M(63,67,b1,UO,yb,Cge),g.Vh=function(t,n){return oQ(this,t,n)},g.Wh=function(t){return n1t(this,t)},g.Xh=function(t,n){YA(this,t,n)},g.Yh=function(t){pA(this,t)},g.pi=function(t){return Y0e(this,t)},g.$b=function(){VE(this)},g.Hc=function(t){return ix(this,t)},g.Xb=function(t){return he(this,t)},g.qi=function(t){var n,s,a;++this.j,s=this.g==null?0:this.g.length,t>s&&(a=this.g,n=s+(s/2|0)+4,n<t&&(n=t),this.g=this.ri(n),a!=null&&Ao(a,0,this.g,0,this.i))},g.Xc=function(t){return E1t(this,t)},g.dc=function(){return this.i==0},g.ii=function(t,n){return yQ(this,t,n)},g.ri=function(t){return pe(yn,st,1,t,5,1)},g.ki=function(t){return this.g[t]},g.$c=function(t){return c5(this,t)},g.mi=function(t,n){return CK(this,t,n)},g.gc=function(){return this.i},g.Pc=function(){return w0e(this)},g.Qc=function(t){return Rpe(this,t)},g.i=0;var uEe=D(Ys,"BasicEList",63),lEe=fi(Ys,"TreeIterator");M(694,63,See),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.g==null&&!this.c?t0e(this):this.g==null||this.i!=0&&h(this.g[this.i-1],47).Ob()},g.Pb=function(){return JF(this)},g.Qb=function(){if(!this.e)throw ne(new ja("There is no valid object to remove."));this.e.Qb()},g.c=!1,D(Ys,"AbstractTreeIterator",694),M(685,694,See,xfe),g.si=function(t){var n;return n=h(t,56).Wg().Kc(),ue(n,279)&&h(n,279).Nk(new Nze),n},D(dj,"ElkGraphUtil/PropertiesSkippingTreeIterator",685),M(952,1,{},Nze),D(dj,"ElkGraphUtil/PropertiesSkippingTreeIterator/1",952);var gL,tie,pL=D(dj,"ElkReflect",null);M(889,1,Yw,Pze),g.vg=function(t){return oB(),n3n(h(t,174))},D(dj,"ElkReflect/lambda$0$Type",889);var tp;fi(Ys,"ResourceLocator"),M(1051,1,{}),D(Ys,"DelegatingResourceLocator",1051),M(1052,1051,{}),D("org.eclipse.emf.common","EMFPlugin",1052);var nie=fi(Xmt,"Adapter"),jPn=fi(Xmt,"Notification");M(1153,1,Xye),g.ti=function(){return this.d},g.ui=function(t){},g.vi=function(t){this.d=t},g.wi=function(t){this.d==t&&(this.d=null)},g.d=null,D(S5,"AdapterImpl",1153),M(1995,67,Qmt),g.Vh=function(t,n){return Ipe(this,t,n)},g.Wh=function(t){var n,s,a;if(++this.j,t.dc())return!1;for(n=this.Vi(),a=t.Kc();a.Ob();)s=a.Pb(),this.Ii(this.oi(n,s)),++n;return!0},g.Xh=function(t,n){wtt(this,t,n)},g.Yh=function(t){qrt(this,t)},g.Gi=function(){return this.Ji()},g.$b=function(){US(this,this.Vi(),this.Wi())},g.Hc=function(t){return this.Li(t)},g.Ic=function(t){return this.Mi(t)},g.Hi=function(t,n){this.Si().jm()},g.Ii=function(t){this.Si().jm()},g.Ji=function(){return this.Si()},g.Ki=function(){this.Si().jm()},g.Li=function(t){return this.Si().jm()},g.Mi=function(t){return this.Si().jm()},g.Ni=function(t){return this.Si().jm()},g.Oi=function(t){return this.Si().jm()},g.Pi=function(){return this.Si().jm()},g.Qi=function(t){return this.Si().jm()},g.Ri=function(){return this.Si().jm()},g.Ti=function(t){return this.Si().jm()},g.Ui=function(t,n){return this.Si().jm()},g.Vi=function(){return this.Si().jm()},g.Wi=function(){return this.Si().jm()},g.Xi=function(t){return this.Si().jm()},g.Yi=function(){return this.Si().jm()},g.Fb=function(t){return this.Ni(t)},g.Xb=function(t){return this.li(t,this.Oi(t))},g.Hb=function(){return this.Pi()},g.Xc=function(t){return this.Qi(t)},g.dc=function(){return this.Ri()},g.ii=function(t,n){return K2e(this,t,n)},g.ki=function(t){return this.Oi(t)},g.$c=function(t){return TP(this,t)},g.Mc=function(t){var n;return n=this.Xc(t),n>=0?(this.$c(n),!0):!1},g.mi=function(t,n){return this.Ui(t,this.oi(t,n))},g.gc=function(){return this.Vi()},g.Pc=function(){return this.Wi()},g.Qc=function(t){return this.Xi(t)},g.Ib=function(){return this.Yi()},D(Ys,"DelegatingEList",1995),M(1996,1995,Qmt),g.Vh=function(t,n){return vve(this,t,n)},g.Wh=function(t){return this.Vh(this.Vi(),t)},g.Xh=function(t,n){j0t(this,t,n)},g.Yh=function(t){D0t(this,t)},g.ai=function(){return!this.bj()},g.$b=function(){L9(this)},g.Zi=function(t,n,s,a,u){return new zit(this,t,n,s,a,u)},g.$i=function(t){Or(this.Ai(),t)},g._i=function(){return null},g.aj=function(){return-1},g.Ai=function(){return null},g.bj=function(){return!1},g.cj=function(t,n){return n},g.dj=function(t,n){return n},g.ej=function(){return!1},g.fj=function(){return!this.Ri()},g.ii=function(t,n){var s,a;return this.ej()?(a=this.fj(),s=K2e(this,t,n),this.$i(this.Zi(7,Ke(n),s,t,a)),s):K2e(this,t,n)},g.$c=function(t){var n,s,a,u;return this.ej()?(s=null,a=this.fj(),n=this.Zi(4,u=TP(this,t),null,t,a),this.bj()&&!!u?(s=this.dj(u,s),s?(s.Ei(n),s.Fi()):this.$i(n)):s?(s.Ei(n),s.Fi()):this.$i(n),u):(u=TP(this,t),this.bj()&&!!u&&(s=this.dj(u,null),s&&s.Fi()),u)},g.mi=function(t,n){return Ypt(this,t,n)},D(S5,"DelegatingNotifyingListImpl",1996),M(143,1,rD),g.Ei=function(t){return B2e(this,t)},g.Fi=function(){_K(this)},g.xi=function(){return this.d},g._i=function(){return null},g.gj=function(){return null},g.yi=function(t){return-1},g.zi=function(){return Ogt(this)},g.Ai=function(){return null},g.Bi=function(){return Zbe(this)},g.Ci=function(){return this.o<0?this.o<-2?-2-this.o-1:-1:this.o},g.hj=function(){return!1},g.Di=function(t){var n,s,a,u,p,v,m,x,_,A,I;switch(this.d){case 1:case 2:switch(u=t.xi(),u){case 1:case 2:if(p=t.Ai(),Me(p)===Me(this.Ai())&&this.yi(null)==t.yi(null))return this.g=t.zi(),t.xi()==1&&(this.d=1),!0}case 4:{switch(u=t.xi(),u){case 4:{if(p=t.Ai(),Me(p)===Me(this.Ai())&&this.yi(null)==t.yi(null))return _=Nve(this),x=this.o<0?this.o<-2?-2-this.o-1:-1:this.o,v=t.Ci(),this.d=6,I=new yb(2),x<=v?(qn(I,this.n),qn(I,t.Bi()),this.g=se(ie(Hn,1),hr,25,15,[this.o=x,v+1])):(qn(I,t.Bi()),qn(I,this.n),this.g=se(ie(Hn,1),hr,25,15,[this.o=v,x])),this.n=I,_||(this.o=-2-this.o-1),!0;break}}break}case 6:{switch(u=t.xi(),u){case 4:{if(p=t.Ai(),Me(p)===Me(this.Ai())&&this.yi(null)==t.yi(null)){for(_=Nve(this),v=t.Ci(),A=h(this.g,48),a=pe(Hn,hr,25,A.length+1,15,1),n=0;n<A.length&&(m=A[n],m<=v);)a[n++]=m,++v;for(s=h(this.n,15),s.Vc(n,t.Bi()),a[n]=v;++n<a.length;)a[n]=A[n-1];return this.g=a,_||(this.o=-2-a[0]),!0}break}}break}}return!1},g.Ib=function(){var t,n,s,a;switch(a=new Ol(sg(this.gm)+"@"+(n=ii(this)>>>0,n.toString(16))),a.a+=" (eventType: ",this.d){case 1:{a.a+="SET";break}case 2:{a.a+="UNSET";break}case 3:{a.a+="ADD";break}case 5:{a.a+="ADD_MANY";break}case 4:{a.a+="REMOVE";break}case 6:{a.a+="REMOVE_MANY";break}case 7:{a.a+="MOVE";break}case 8:{a.a+="REMOVING_ADAPTER";break}case 9:{a.a+="RESOLVE";break}default:{Gq(a,this.d);break}}if(kpt(this)&&(a.a+=", touch: true"),a.a+=", position: ",Gq(a,this.o<0?this.o<-2?-2-this.o-1:-1:this.o),a.a+=", notifier: ",dE(a,this.Ai()),a.a+=", feature: ",dE(a,this._i()),a.a+=", oldValue: ",dE(a,Zbe(this)),a.a+=", newValue: ",this.d==6&&ue(this.g,48)){for(s=h(this.g,48),a.a+="[",t=0;t<s.length;)a.a+=s[t],++t<s.length&&(a.a+=Js);a.a+="]"}else dE(a,Ogt(this));return a.a+=", isTouch: ",a0(a,kpt(this)),a.a+=", wasSet: ",a0(a,Nve(this)),a.a+=")",a.a},g.d=0,g.e=0,g.f=0,g.j=0,g.k=0,g.o=0,g.p=0,D(S5,"NotificationImpl",143),M(1167,143,rD,zit),g._i=function(){return this.a._i()},g.yi=function(t){return this.a.aj()},g.Ai=function(){return this.a.Ai()},D(S5,"DelegatingNotifyingListImpl/1",1167),M(242,63,b1,Bze,ig),g.Fc=function(t){return Qht(this,h(t,366))},g.Ei=function(t){return Qht(this,t)},g.Fi=function(){var t,n,s;for(t=0;t<this.i;++t)n=h(this.g[t],366),s=n.Ai(),s!=null&&n.xi()!=-1&&h(s,92).Ng(n)},g.ri=function(t){return pe(jPn,st,366,t,0,1)},D(S5,"NotificationChainImpl",242),M(1378,90,umt),g.Kg=function(){return this.e},g.Mg=function(){return(this.f&1)!=0},g.f=1,D(S5,"NotifierImpl",1378),M(1993,63,b1),g.Vh=function(t,n){return GQ(this,t,n)},g.Wh=function(t){return this.Vh(this.i,t)},g.Xh=function(t,n){Tbe(this,t,n)},g.Yh=function(t){kQ(this,t)},g.ai=function(){return!this.bj()},g.$b=function(){jn(this)},g.Zi=function(t,n,s,a,u){return new Git(this,t,n,s,a,u)},g.$i=function(t){Or(this.Ai(),t)},g._i=function(){return null},g.aj=function(){return-1},g.Ai=function(){return null},g.bj=function(){return!1},g.ij=function(){return!1},g.cj=function(t,n){return n},g.dj=function(t,n){return n},g.ej=function(){return!1},g.fj=function(){return this.i!=0},g.ii=function(t,n){return a9(this,t,n)},g.$c=function(t){return Ow(this,t)},g.mi=function(t,n){return c2t(this,t,n)},g.jj=function(t,n){return n},g.kj=function(t,n){return n},g.lj=function(t,n,s){return s},D(S5,"NotifyingListImpl",1993),M(1166,143,rD,Git),g._i=function(){return this.a._i()},g.yi=function(t){return this.a.aj()},g.Ai=function(){return this.a.Ai()},D(S5,"NotifyingListImpl/1",1166),M(953,63,b1,ktt),g.Hc=function(t){return this.i>10?((!this.b||this.c.j!=this.a)&&(this.b=new CE(this),this.a=this.j),wd(this.b,t)):ix(this,t)},g.ni=function(){return!0},g.a=0,D(Ys,"AbstractEList/1",953),M(295,73,FZ,sw),D(Ys,"AbstractEList/BasicIndexOutOfBoundsException",295),M(40,1,ys,Tn),g.Nb=function(t){Ls(this,t)},g.mj=function(){if(this.i.j!=this.f)throw ne(new cl)},g.nj=function(){return On(this)},g.Ob=function(){return this.e!=this.i.gc()},g.Pb=function(){return this.nj()},g.Qb=function(){p9(this)},g.e=0,g.f=0,g.g=-1,D(Ys,"AbstractEList/EIterator",40),M(278,40,z1,N4,yW),g.Qb=function(){p9(this)},g.Rb=function(t){ift(this,t)},g.oj=function(){var t;try{return t=this.d.Xb(--this.e),this.mj(),this.g=this.e,t}catch(n){throw n=ui(n),ue(n,73)?(this.mj(),ne(new lo)):ne(n)}},g.pj=function(t){s1t(this,t)},g.Sb=function(){return this.e!=0},g.Tb=function(){return this.e},g.Ub=function(){return this.oj()},g.Vb=function(){return this.e-1},g.Wb=function(t){this.pj(t)},D(Ys,"AbstractEList/EListIterator",278),M(341,40,ys,P4),g.nj=function(){return NX(this)},g.Qb=function(){throw ne(new Xn)},D(Ys,"AbstractEList/NonResolvingEIterator",341),M(385,278,z1,HS,M1e),g.Rb=function(t){throw ne(new Xn)},g.nj=function(){var t;try{return t=this.c.ki(this.e),this.mj(),this.g=this.e++,t}catch(n){throw n=ui(n),ue(n,73)?(this.mj(),ne(new lo)):ne(n)}},g.oj=function(){var t;try{return t=this.c.ki(--this.e),this.mj(),this.g=this.e,t}catch(n){throw n=ui(n),ue(n,73)?(this.mj(),ne(new lo)):ne(n)}},g.Qb=function(){throw ne(new Xn)},g.Wb=function(t){throw ne(new Xn)},D(Ys,"AbstractEList/NonResolvingEListIterator",385),M(1982,67,Zmt),g.Vh=function(t,n){var s,a,u,p,v,m,x,_,A,I,P;if(u=n.gc(),u!=0){for(_=h(tn(this.a,4),126),A=_==null?0:_.length,P=A+u,a=nX(this,P),I=A-t,I>0&&Ao(_,t,a,t+u,I),x=n.Kc(),v=0;v<u;++v)m=x.Pb(),s=t+v,AY(a,s,Gk(this,m));for(ex(this,a),p=0;p<u;++p)m=a[t],this.bi(t,m),++t;return!0}else return++this.j,!1},g.Wh=function(t){var n,s,a,u,p,v,m,x,_;if(a=t.gc(),a!=0){for(x=(s=h(tn(this.a,4),126),s==null?0:s.length),_=x+a,n=nX(this,_),m=t.Kc(),p=x;p<_;++p)v=m.Pb(),AY(n,p,Gk(this,v));for(ex(this,n),u=x;u<_;++u)v=n[u],this.bi(u,v);return!0}else return++this.j,!1},g.Xh=function(t,n){var s,a,u,p;a=h(tn(this.a,4),126),u=a==null?0:a.length,s=nX(this,u+1),p=Gk(this,n),t!=u&&Ao(a,t,s,t+1,u-t),pi(s,t,p),ex(this,s),this.bi(t,n)},g.Yh=function(t){var n,s,a;a=(s=h(tn(this.a,4),126),s==null?0:s.length),n=nX(this,a+1),AY(n,a,Gk(this,t)),ex(this,n),this.bi(a,t)},g.Zh=function(){return new uot(this)},g.$h=function(){return new Ert(this)},g._h=function(t){var n,s;if(s=(n=h(tn(this.a,4),126),n==null?0:n.length),t<0||t>s)throw ne(new sw(t,s));return new Jrt(this,t)},g.$b=function(){var t,n;++this.j,t=h(tn(this.a,4),126),n=t==null?0:t.length,ex(this,null),pK(this,n,t)},g.Hc=function(t){var n,s,a,u,p;if(n=h(tn(this.a,4),126),n!=null){if(t!=null){for(a=n,u=0,p=a.length;u<p;++u)if(s=a[u],Nr(t,s))return!0}else for(a=n,u=0,p=a.length;u<p;++u)if(s=a[u],Me(s)===Me(t))return!0}return!1},g.Xb=function(t){var n,s;if(n=h(tn(this.a,4),126),s=n==null?0:n.length,t>=s)throw ne(new sw(t,s));return n[t]},g.Xc=function(t){var n,s,a;if(n=h(tn(this.a,4),126),n!=null){if(t!=null){for(s=0,a=n.length;s<a;++s)if(Nr(t,n[s]))return s}else for(s=0,a=n.length;s<a;++s)if(Me(n[s])===Me(t))return s}return-1},g.dc=function(){return h(tn(this.a,4),126)==null},g.Kc=function(){return new cot(this)},g.Yc=function(){return new xrt(this)},g.Zc=function(t){var n,s;if(s=(n=h(tn(this.a,4),126),n==null?0:n.length),t<0||t>s)throw ne(new sw(t,s));return new Zrt(this,t)},g.ii=function(t,n){var s,a,u;if(s=fft(this),u=s==null?0:s.length,t>=u)throw ne(new Ea(Tee+t+s2+u));if(n>=u)throw ne(new Ea(Cee+n+s2+u));return a=s[n],t!=n&&(t<n?Ao(s,t,s,t+1,n-t):Ao(s,n+1,s,n,t-n),pi(s,t,a),ex(this,s)),a},g.ki=function(t){return h(tn(this.a,4),126)[t]},g.$c=function(t){return qTn(this,t)},g.mi=function(t,n){var s,a;return s=fft(this),a=s[t],AY(s,t,Gk(this,n)),ex(this,s),a},g.gc=function(){var t;return t=h(tn(this.a,4),126),t==null?0:t.length},g.Pc=function(){var t,n,s;return t=h(tn(this.a,4),126),s=t==null?0:t.length,n=pe(nie,Lee,415,s,0,1),s>0&&Ao(t,0,n,0,s),n},g.Qc=function(t){var n,s,a;return n=h(tn(this.a,4),126),a=n==null?0:n.length,a>0&&(t.length<a&&(s=LA(uu(t).c,a),t=s),Ao(n,0,t,0,a)),t.length>a&&pi(t,a,null),t};var D9t;D(Ys,"ArrayDelegatingEList",1982),M(1038,40,ys,cot),g.mj=function(){if(this.b.j!=this.f||Me(h(tn(this.b.a,4),126))!==Me(this.a))throw ne(new cl)},g.Qb=function(){p9(this),this.a=h(tn(this.b.a,4),126)},D(Ys,"ArrayDelegatingEList/EIterator",1038),M(706,278,z1,xrt,Zrt),g.mj=function(){if(this.b.j!=this.f||Me(h(tn(this.b.a,4),126))!==Me(this.a))throw ne(new cl)},g.pj=function(t){s1t(this,t),this.a=h(tn(this.b.a,4),126)},g.Qb=function(){p9(this),this.a=h(tn(this.b.a,4),126)},D(Ys,"ArrayDelegatingEList/EListIterator",706),M(1039,341,ys,uot),g.mj=function(){if(this.b.j!=this.f||Me(h(tn(this.b.a,4),126))!==Me(this.a))throw ne(new cl)},D(Ys,"ArrayDelegatingEList/NonResolvingEIterator",1039),M(707,385,z1,Ert,Jrt),g.mj=function(){if(this.b.j!=this.f||Me(h(tn(this.b.a,4),126))!==Me(this.a))throw ne(new cl)},D(Ys,"ArrayDelegatingEList/NonResolvingEListIterator",707),M(606,295,FZ,EY),D(Ys,"BasicEList/BasicIndexOutOfBoundsException",606),M(696,63,b1,gfe),g.Vc=function(t,n){throw ne(new Xn)},g.Fc=function(t){throw ne(new Xn)},g.Wc=function(t,n){throw ne(new Xn)},g.Gc=function(t){throw ne(new Xn)},g.$b=function(){throw ne(new Xn)},g.qi=function(t){throw ne(new Xn)},g.Kc=function(){return this.Zh()},g.Yc=function(){return this.$h()},g.Zc=function(t){return this._h(t)},g.ii=function(t,n){throw ne(new Xn)},g.ji=function(t,n){throw ne(new Xn)},g.$c=function(t){throw ne(new Xn)},g.Mc=function(t){throw ne(new Xn)},g._c=function(t,n){throw ne(new Xn)},D(Ys,"BasicEList/UnmodifiableEList",696),M(705,1,{3:1,20:1,14:1,15:1,58:1,589:1}),g.Vc=function(t,n){Ydn(this,t,h(n,42))},g.Fc=function(t){return O0n(this,h(t,42))},g.Jc=function(t){Os(this,t)},g.Xb=function(t){return h(he(this.c,t),133)},g.ii=function(t,n){return h(this.c.ii(t,n),42)},g.ji=function(t,n){Wdn(this,t,h(n,42))},g.Lc=function(){return new Vt(null,new Wt(this,16))},g.$c=function(t){return h(this.c.$c(t),42)},g._c=function(t,n){return Hbn(this,t,h(n,42))},g.ad=function(t){my(this,t)},g.Nc=function(){return new Wt(this,16)},g.Oc=function(){return new Vt(null,new Wt(this,16))},g.Wc=function(t,n){return this.c.Wc(t,n)},g.Gc=function(t){return this.c.Gc(t)},g.$b=function(){this.c.$b()},g.Hc=function(t){return this.c.Hc(t)},g.Ic=function(t){return FA(this.c,t)},g.qj=function(){var t,n,s;if(this.d==null){for(this.d=pe(uEe,Qye,63,2*this.f+1,0,1),s=this.e,this.f=0,n=this.c.Kc();n.e!=n.i.gc();)t=h(n.nj(),133),PF(this,t);this.e=s}},g.Fb=function(t){return Wtt(this,t)},g.Hb=function(){return Nge(this.c)},g.Xc=function(t){return this.c.Xc(t)},g.rj=function(){this.c=new fKe(this)},g.dc=function(){return this.f==0},g.Kc=function(){return this.c.Kc()},g.Yc=function(){return this.c.Yc()},g.Zc=function(t){return this.c.Zc(t)},g.sj=function(){return wA(this)},g.tj=function(t,n,s){return new hnt(t,n,s)},g.uj=function(){return new Fze},g.Mc=function(t){return Sut(this,t)},g.gc=function(){return this.f},g.bd=function(t,n){return new F1(this.c,t,n)},g.Pc=function(){return this.c.Pc()},g.Qc=function(t){return this.c.Qc(t)},g.Ib=function(){return Cpe(this.c)},g.e=0,g.f=0,D(Ys,"BasicEMap",705),M(1033,63,b1,fKe),g.bi=function(t,n){Dfn(this,h(n,133))},g.ei=function(t,n,s){var a;++(a=this,h(n,133),a).a.e},g.fi=function(t,n){Lfn(this,h(n,133))},g.gi=function(t,n,s){y0n(this,h(n,133),h(s,133))},g.di=function(t,n){vlt(this.a)},D(Ys,"BasicEMap/1",1033),M(1034,63,b1,Fze),g.ri=function(t){return pe($Pn,Jmt,612,t,0,1)},D(Ys,"BasicEMap/2",1034),M(1035,Lf,$c,dKe),g.$b=function(){this.a.c.$b()},g.Hc=function(t){return TX(this.a,t)},g.Kc=function(){return this.a.f==0?(ik(),bL.a):new WXe(this.a)},g.Mc=function(t){var n;return n=this.a.f,_F(this.a,t),this.a.f!=n},g.gc=function(){return this.a.f},D(Ys,"BasicEMap/3",1035),M(1036,28,Rw,gKe),g.$b=function(){this.a.c.$b()},g.Hc=function(t){return tpt(this.a,t)},g.Kc=function(){return this.a.f==0?(ik(),bL.a):new KXe(this.a)},g.gc=function(){return this.a.f},D(Ys,"BasicEMap/4",1036),M(1037,Lf,$c,pKe),g.$b=function(){this.a.c.$b()},g.Hc=function(t){var n,s,a,u,p,v,m,x,_;if(this.a.f>0&&ue(t,42)&&(this.a.qj(),x=h(t,42),m=x.cd(),u=m==null?0:ii(m),p=p1e(this.a,u),n=this.a.d[p],n)){for(s=h(n.g,367),_=n.i,v=0;v<_;++v)if(a=s[v],a.Sh()==u&&a.Fb(x))return!0}return!1},g.Kc=function(){return this.a.f==0?(ik(),bL.a):new BW(this.a)},g.Mc=function(t){return Y0t(this,t)},g.gc=function(){return this.a.f},D(Ys,"BasicEMap/5",1037),M(613,1,ys,BW),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return this.b!=-1},g.Pb=function(){var t;if(this.f.e!=this.c)throw ne(new cl);if(this.b==-1)throw ne(new lo);return this.d=this.a,this.e=this.b,P1t(this),t=h(this.f.d[this.d].g[this.e],133),this.vj(t)},g.Qb=function(){if(this.f.e!=this.c)throw ne(new cl);if(this.e==-1)throw ne(new Ic);this.f.c.Mc(he(this.f.d[this.d],this.e)),this.c=this.f.e,this.e=-1,this.a==this.d&&this.b!=-1&&--this.b},g.vj=function(t){return t},g.a=0,g.b=-1,g.c=0,g.d=0,g.e=0,D(Ys,"BasicEMap/BasicEMapIterator",613),M(1031,613,ys,WXe),g.vj=function(t){return t.cd()},D(Ys,"BasicEMap/BasicEMapKeyIterator",1031),M(1032,613,ys,KXe),g.vj=function(t){return t.dd()},D(Ys,"BasicEMap/BasicEMapValueIterator",1032),M(1030,1,zb,bKe),g.wc=function(t){ZE(this,t)},g.yc=function(t,n,s){return wX(this,t,n,s)},g.$b=function(){this.a.c.$b()},g._b=function(t){return AJe(this,t)},g.uc=function(t){return tpt(this.a,t)},g.vc=function(){return u3n(this.a)},g.Fb=function(t){return Wtt(this.a,t)},g.xc=function(t){return Xh(this.a,t)},g.Hb=function(){return Nge(this.a.c)},g.dc=function(){return this.a.f==0},g.ec=function(){return l3n(this.a)},g.zc=function(t,n){return zF(this.a,t,n)},g.Bc=function(t){return _F(this.a,t)},g.gc=function(){return this.a.f},g.Ib=function(){return Cpe(this.a.c)},g.Cc=function(){return c3n(this.a)},D(Ys,"BasicEMap/DelegatingMap",1030),M(612,1,{42:1,133:1,612:1},hnt),g.Fb=function(t){var n;return ue(t,42)?(n=h(t,42),(this.b!=null?Nr(this.b,n.cd()):Me(this.b)===Me(n.cd()))&&(this.c!=null?Nr(this.c,n.dd()):Me(this.c)===Me(n.dd()))):!1},g.Sh=function(){return this.a},g.cd=function(){return this.b},g.dd=function(){return this.c},g.Hb=function(){return this.a^(this.c==null?0:ii(this.c))},g.Th=function(t){this.a=t},g.Uh=function(t){throw ne(new Ep)},g.ed=function(t){var n;return n=this.c,this.c=t,n},g.Ib=function(){return this.b+"->"+this.c},g.a=0;var $Pn=D(Ys,"BasicEMap/EntryImpl",612);M(536,1,{},VO),D(Ys,"BasicEMap/View",536);var bL;M(768,1,{}),g.Fb=function(t){return Nbe((Rt(),ca),t)},g.Hb=function(){return qge((Rt(),ca))},g.Ib=function(){return Ag((Rt(),ca))},D(Ys,"ECollections/BasicEmptyUnmodifiableEList",768),M(1312,1,z1,Rze),g.Nb=function(t){Ls(this,t)},g.Rb=function(t){throw ne(new Xn)},g.Ob=function(){return!1},g.Sb=function(){return!1},g.Pb=function(){throw ne(new lo)},g.Tb=function(){return 0},g.Ub=function(){throw ne(new lo)},g.Vb=function(){return-1},g.Qb=function(){throw ne(new Xn)},g.Wb=function(t){throw ne(new Xn)},D(Ys,"ECollections/BasicEmptyUnmodifiableEList/1",1312),M(1310,768,{20:1,14:1,15:1,58:1},sXe),g.Vc=function(t,n){bQe()},g.Fc=function(t){return vQe()},g.Wc=function(t,n){return wQe()},g.Gc=function(t){return mQe()},g.$b=function(){yQe()},g.Hc=function(t){return!1},g.Ic=function(t){return!1},g.Jc=function(t){Os(this,t)},g.Xb=function(t){return vfe((Rt(),t)),null},g.Xc=function(t){return-1},g.dc=function(){return!0},g.Kc=function(){return this.a},g.Yc=function(){return this.a},g.Zc=function(t){return this.a},g.ii=function(t,n){return kQe()},g.ji=function(t,n){xQe()},g.Lc=function(){return new Vt(null,new Wt(this,16))},g.$c=function(t){return EQe()},g.Mc=function(t){return _Qe()},g._c=function(t,n){return TQe()},g.gc=function(){return 0},g.ad=function(t){my(this,t)},g.Nc=function(){return new Wt(this,16)},g.Oc=function(){return new Vt(null,new Wt(this,16))},g.bd=function(t,n){return Rt(),new F1(ca,t,n)},g.Pc=function(){return vde((Rt(),ca))},g.Qc=function(t){return Rt(),nM(ca,t)},D(Ys,"ECollections/EmptyUnmodifiableEList",1310),M(1311,768,{20:1,14:1,15:1,58:1,589:1},aXe),g.Vc=function(t,n){bQe()},g.Fc=function(t){return vQe()},g.Wc=function(t,n){return wQe()},g.Gc=function(t){return mQe()},g.$b=function(){yQe()},g.Hc=function(t){return!1},g.Ic=function(t){return!1},g.Jc=function(t){Os(this,t)},g.Xb=function(t){return vfe((Rt(),t)),null},g.Xc=function(t){return-1},g.dc=function(){return!0},g.Kc=function(){return this.a},g.Yc=function(){return this.a},g.Zc=function(t){return this.a},g.ii=function(t,n){return kQe()},g.ji=function(t,n){xQe()},g.Lc=function(){return new Vt(null,new Wt(this,16))},g.$c=function(t){return EQe()},g.Mc=function(t){return _Qe()},g._c=function(t,n){return TQe()},g.gc=function(){return 0},g.ad=function(t){my(this,t)},g.Nc=function(){return new Wt(this,16)},g.Oc=function(){return new Vt(null,new Wt(this,16))},g.bd=function(t,n){return Rt(),new F1(ca,t,n)},g.Pc=function(){return vde((Rt(),ca))},g.Qc=function(t){return Rt(),nM(ca,t)},g.sj=function(){return Rt(),Rt(),K1},D(Ys,"ECollections/EmptyUnmodifiableEMap",1311);var hEe=fi(Ys,"Enumerator"),KH;M(281,1,{281:1},OQ),g.Fb=function(t){var n;return this===t?!0:ue(t,281)?(n=h(t,281),this.f==n.f&&cbn(this.i,n.i)&&aW(this.a,(this.f&256)!=0?(n.f&256)!=0?n.a:null:(n.f&256)!=0?null:n.a)&&aW(this.d,n.d)&&aW(this.g,n.g)&&aW(this.e,n.e)&&n8n(this,n)):!1},g.Hb=function(){return this.f},g.Ib=function(){return Ipt(this)},g.f=0;var L9t=0,I9t=0,O9t=0,N9t=0,fEe=0,dEe=0,gEe=0,pEe=0,bEe=0,P9t,lT=0,hT=0,B9t=0,F9t=0,XH,vEe;D(Ys,"URI",281),M(1091,43,Vy,oXe),g.zc=function(t,n){return h(Ta(this,Jn(t),h(n,281)),281)},D(Ys,"URI/URICache",1091),M(497,63,b1,Ize,SP),g.hi=function(){return!0},D(Ys,"UniqueEList",497),M(581,60,Fd,MB),D(Ys,"WrappedException",581);var dr=fi(yl,nyt),vm=fi(yl,ryt),Ac=fi(yl,iyt),wm=fi(yl,syt),af=fi(yl,ayt),Jl=fi(yl,"EClass"),rie=fi(yl,"EDataType"),R9t;M(1183,43,Vy,cXe),g.xc=function(t){return ws(t)?Fo(this,t):no(Ia(this.f,t))},D(yl,"EDataType/Internal/ConversionDelegate/Factory/Registry/Impl",1183);var QH=fi(yl,"EEnum"),qd=fi(yl,oyt),pa=fi(yl,cyt),eh=fi(yl,uyt),th,dv=fi(yl,lyt),mm=fi(yl,hyt);M(1029,1,{},Lze),g.Ib=function(){return"NIL"},D(yl,"EStructuralFeature/Internal/DynamicValueHolder/1",1029);var j9t;M(1028,43,Vy,uXe),g.xc=function(t){return ws(t)?Fo(this,t):no(Ia(this.f,t))},D(yl,"EStructuralFeature/Internal/SettingDelegate/Factory/Registry/Impl",1028);var lc=fi(yl,fyt),X5=fi(yl,"EValidator/PatternMatcher"),wEe,mEe,Xt,F0,ym,np,$9t,H9t,z9t,rp,R0,ip,gv,zf,G9t,V9t,nh,j0,U9t,$0,km,C3,na,q9t,Y9t,pv,ZH=fi(ri,"FeatureMap/Entry");M(535,1,{72:1},qN),g.ak=function(){return this.a},g.dd=function(){return this.b},D(Qt,"BasicEObjectImpl/1",535),M(1027,1,Iee,kJe),g.Wj=function(t){return dK(this.a,this.b,t)},g.fj=function(){return rst(this.a,this.b)},g.Wb=function(t){Jde(this.a,this.b,t)},g.Xj=function(){ivn(this.a,this.b)},D(Qt,"BasicEObjectImpl/4",1027),M(1983,1,{108:1}),g.bk=function(t){this.e=t==0?W9t:pe(yn,st,1,t,5,1)},g.Ch=function(t){return this.e[t]},g.Dh=function(t,n){this.e[t]=n},g.Eh=function(t){this.e[t]=null},g.ck=function(){return this.c},g.dk=function(){throw ne(new Xn)},g.ek=function(){throw ne(new Xn)},g.fk=function(){return this.d},g.gk=function(){return this.e!=null},g.hk=function(t){this.c=t},g.ik=function(t){throw ne(new Xn)},g.jk=function(t){throw ne(new Xn)},g.kk=function(t){this.d=t};var W9t;D(Qt,"BasicEObjectImpl/EPropertiesHolderBaseImpl",1983),M(185,1983,{108:1},Gh),g.dk=function(){return this.a},g.ek=function(){return this.b},g.ik=function(t){this.a=t},g.jk=function(t){this.b=t},D(Qt,"BasicEObjectImpl/EPropertiesHolderImpl",185),M(506,97,dmt,qO),g.Kg=function(){return this.f},g.Pg=function(){return this.k},g.Rg=function(t,n){this.g=t,this.i=n},g.Tg=function(){return(this.j&2)==0?this.zh():this.ph().ck()},g.Vg=function(){return this.i},g.Mg=function(){return(this.j&1)!=0},g.eh=function(){return this.g},g.kh=function(){return(this.j&4)!=0},g.ph=function(){return!this.k&&(this.k=new Gh),this.k},g.th=function(t){this.ph().hk(t),t?this.j|=2:this.j&=-3},g.vh=function(t){this.ph().jk(t),t?this.j|=4:this.j&=-5},g.zh=function(){return(bg(),Xt).S},g.i=0,g.j=1,D(Qt,"EObjectImpl",506),M(780,506,{105:1,92:1,90:1,56:1,108:1,49:1,97:1},J1e),g.Ch=function(t){return this.e[t]},g.Dh=function(t,n){this.e[t]=n},g.Eh=function(t){this.e[t]=null},g.Tg=function(){return this.d},g.Yg=function(t){return oi(this.d,t)},g.$g=function(){return this.d},g.dh=function(){return this.e!=null},g.ph=function(){return!this.k&&(this.k=new jze),this.k},g.th=function(t){this.d=t},g.yh=function(){var t;return this.e==null&&(t=En(this.d),this.e=t==0?K9t:pe(yn,st,1,t,5,1)),this},g.Ah=function(){return 0};var K9t;D(Qt,"DynamicEObjectImpl",780),M(1376,780,{105:1,42:1,92:1,90:1,133:1,56:1,108:1,49:1,97:1},Rnt),g.Fb=function(t){return this===t},g.Hb=function(){return rb(this)},g.th=function(t){this.d=t,this.b=yM(t,"key"),this.c=yM(t,t_)},g.Sh=function(){var t;return this.a==-1&&(t=TK(this,this.b),this.a=t==null?0:ii(t)),this.a},g.cd=function(){return TK(this,this.b)},g.dd=function(){return TK(this,this.c)},g.Th=function(t){this.a=t},g.Uh=function(t){Jde(this,this.b,t)},g.ed=function(t){var n;return n=TK(this,this.c),Jde(this,this.c,t),n},g.a=0,D(Qt,"DynamicEObjectImpl/BasicEMapEntry",1376),M(1377,1,{108:1},jze),g.bk=function(t){throw ne(new Xn)},g.Ch=function(t){throw ne(new Xn)},g.Dh=function(t,n){throw ne(new Xn)},g.Eh=function(t){throw ne(new Xn)},g.ck=function(){throw ne(new Xn)},g.dk=function(){return this.a},g.ek=function(){return this.b},g.fk=function(){return this.c},g.gk=function(){throw ne(new Xn)},g.hk=function(t){throw ne(new Xn)},g.ik=function(t){this.a=t},g.jk=function(t){this.b=t},g.kk=function(t){this.c=t},D(Qt,"DynamicEObjectImpl/DynamicEPropertiesHolderImpl",1377),M(510,150,{105:1,92:1,90:1,590:1,147:1,56:1,108:1,49:1,97:1,510:1,150:1,114:1,115:1},Jue),g.Qg=function(t){return w2e(this,t)},g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.d;case 2:return s?(!this.b&&(this.b=new Cu((Bt(),na),uo,this)),this.b):(!this.b&&(this.b=new Cu((Bt(),na),uo,this)),wA(this.b));case 3:return cst(this);case 4:return!this.a&&(this.a=new Gi(Jg,this,4)),this.a;case 5:return!this.c&&(this.c=new ly(Jg,this,5)),this.c}return gl(this,t-En((Bt(),F0)),Ht((a=h(tn(this,16),26),a||F0),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 3:return this.Cb&&(s=(u=this.Db>>16,u>=0?w2e(this,s):this.Cb.ih(this,-1-u,null,s))),pde(this,h(t,147),s)}return p=h(Ht((a=h(tn(this,16),26),a||(Bt(),F0)),n),66),p.Nj().Qj(this,ic(this),n-En((Bt(),F0)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 2:return!this.b&&(this.b=new Cu((Bt(),na),uo,this)),pP(this.b,t,s);case 3:return pde(this,null,s);case 4:return!this.a&&(this.a=new Gi(Jg,this,4)),qs(this.a,t,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),F0)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),F0)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.d!=null;case 2:return!!this.b&&this.b.f!=0;case 3:return!!cst(this);case 4:return!!this.a&&this.a.i!=0;case 5:return!!this.c&&this.c.i!=0}return fl(this,t-En((Bt(),F0)),Ht((n=h(tn(this,16),26),n||F0),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:Y2n(this,Jn(n));return;case 2:!this.b&&(this.b=new Cu((Bt(),na),uo,this)),rF(this.b,n);return;case 3:xgt(this,h(n,147));return;case 4:!this.a&&(this.a=new Gi(Jg,this,4)),jn(this.a),!this.a&&(this.a=new Gi(Jg,this,4)),mi(this.a,h(n,14));return;case 5:!this.c&&(this.c=new ly(Jg,this,5)),jn(this.c),!this.c&&(this.c=new ly(Jg,this,5)),mi(this.c,h(n,14));return}ml(this,t-En((Bt(),F0)),Ht((s=h(tn(this,16),26),s||F0),t),n)},g.zh=function(){return Bt(),F0},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:mge(this,null);return;case 2:!this.b&&(this.b=new Cu((Bt(),na),uo,this)),this.b.c.$b();return;case 3:xgt(this,null);return;case 4:!this.a&&(this.a=new Gi(Jg,this,4)),jn(this.a);return;case 5:!this.c&&(this.c=new ly(Jg,this,5)),jn(this.c);return}vl(this,t-En((Bt(),F0)),Ht((n=h(tn(this,16),26),n||F0),t))},g.Ib=function(){return Fht(this)},g.d=null,D(Qt,"EAnnotationImpl",510),M(151,705,Zye,Du),g.Xh=function(t,n){Mdn(this,t,h(n,42))},g.lk=function(t,n){return Egn(this,h(t,42),n)},g.pi=function(t){return h(h(this.c,69).pi(t),133)},g.Zh=function(){return h(this.c,69).Zh()},g.$h=function(){return h(this.c,69).$h()},g._h=function(t){return h(this.c,69)._h(t)},g.mk=function(t,n){return pP(this,t,n)},g.Wj=function(t){return h(this.c,76).Wj(t)},g.rj=function(){},g.fj=function(){return h(this.c,76).fj()},g.tj=function(t,n,s){var a;return a=h(Vu(this.b).Nh().Jh(this.b),133),a.Th(t),a.Uh(n),a.ed(s),a},g.uj=function(){return new Rle(this)},g.Wb=function(t){rF(this,t)},g.Xj=function(){h(this.c,76).Xj()},D(ri,"EcoreEMap",151),M(158,151,Zye,Cu),g.qj=function(){var t,n,s,a,u,p;if(this.d==null){for(p=pe(uEe,Qye,63,2*this.f+1,0,1),s=this.c.Kc();s.e!=s.i.gc();)n=h(s.nj(),133),a=n.Sh(),u=(a&Ir)%p.length,t=p[u],!t&&(t=p[u]=new Rle(this)),t.Fc(n);this.d=p}},D(Qt,"EAnnotationImpl/1",158),M(284,438,{105:1,92:1,90:1,147:1,191:1,56:1,108:1,472:1,49:1,97:1,150:1,284:1,114:1,115:1}),g._g=function(t,n,s){var a,u;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return on(),(this.Bb&256)!=0;case 3:return on(),(this.Bb&512)!=0;case 4:return Ke(this.s);case 5:return Ke(this.t);case 6:return on(),!!this.$j();case 7:return on(),u=this.s,u>=1;case 8:return n?Rl(this):this.r;case 9:return this.q}return gl(this,t-En(this.zh()),Ht((a=h(tn(this,16),26),a||this.zh()),t),n,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 9:return EW(this,s)}return u=h(Ht((a=h(tn(this,16),26),a||this.zh()),n),66),u.Nj().Rj(this,ic(this),n-En(this.zh()),t,s)},g.lh=function(t){var n,s;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return(this.Bb&256)==0;case 3:return(this.Bb&512)==0;case 4:return this.s!=0;case 5:return this.t!=1;case 6:return this.$j();case 7:return s=this.s,s>=1;case 8:return!!this.r&&!this.q.e&&hb(this.q).i==0;case 9:return!!this.q&&!(!!this.r&&!this.q.e&&hb(this.q).i==0)}return fl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.sh=function(t,n){var s,a;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:this.Lh(Jn(n));return;case 2:w0(this,ft(lt(n)));return;case 3:m0(this,ft(lt(n)));return;case 4:p0(this,h(n,19).a);return;case 5:this.ok(h(n,19).a);return;case 8:Vp(this,h(n,138));return;case 9:a=Df(this,h(n,87),null),a&&a.Fi();return}ml(this,t-En(this.zh()),Ht((s=h(tn(this,16),26),s||this.zh()),t),n)},g.zh=function(){return Bt(),Y9t},g.Bh=function(t){var n,s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:this.Lh(null);return;case 2:w0(this,!0);return;case 3:m0(this,!0);return;case 4:p0(this,0);return;case 5:this.ok(1);return;case 8:Vp(this,null);return;case 9:s=Df(this,null,null),s&&s.Fi();return}vl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.Gh=function(){Rl(this),this.Bb|=1},g.Yj=function(){return Rl(this)},g.Zj=function(){return this.t},g.$j=function(){var t;return t=this.t,t>1||t==-1},g.hi=function(){return(this.Bb&512)!=0},g.nk=function(t,n){return dpe(this,t,n)},g.ok=function(t){yw(this,t)},g.Ib=function(){return _be(this)},g.s=0,g.t=1,D(Qt,"ETypedElementImpl",284),M(449,284,{105:1,92:1,90:1,147:1,191:1,56:1,170:1,66:1,108:1,472:1,49:1,97:1,150:1,449:1,284:1,114:1,115:1,677:1}),g.Qg=function(t){return m1t(this,t)},g._g=function(t,n,s){var a,u;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return on(),(this.Bb&256)!=0;case 3:return on(),(this.Bb&512)!=0;case 4:return Ke(this.s);case 5:return Ke(this.t);case 6:return on(),!!this.$j();case 7:return on(),u=this.s,u>=1;case 8:return n?Rl(this):this.r;case 9:return this.q;case 10:return on(),(this.Bb&Ch)!=0;case 11:return on(),(this.Bb&Ww)!=0;case 12:return on(),(this.Bb&$w)!=0;case 13:return this.j;case 14:return hx(this);case 15:return on(),(this.Bb&zc)!=0;case 16:return on(),(this.Bb&a1)!=0;case 17:return hw(this)}return gl(this,t-En(this.zh()),Ht((a=h(tn(this,16),26),a||this.zh()),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 17:return this.Cb&&(s=(u=this.Db>>16,u>=0?m1t(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,17,s)}return p=h(Ht((a=h(tn(this,16),26),a||this.zh()),n),66),p.Nj().Qj(this,ic(this),n-En(this.zh()),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 9:return EW(this,s);case 17:return Ku(this,null,17,s)}return u=h(Ht((a=h(tn(this,16),26),a||this.zh()),n),66),u.Nj().Rj(this,ic(this),n-En(this.zh()),t,s)},g.lh=function(t){var n,s;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return(this.Bb&256)==0;case 3:return(this.Bb&512)==0;case 4:return this.s!=0;case 5:return this.t!=1;case 6:return this.$j();case 7:return s=this.s,s>=1;case 8:return!!this.r&&!this.q.e&&hb(this.q).i==0;case 9:return!!this.q&&!(!!this.r&&!this.q.e&&hb(this.q).i==0);case 10:return(this.Bb&Ch)==0;case 11:return(this.Bb&Ww)!=0;case 12:return(this.Bb&$w)!=0;case 13:return this.j!=null;case 14:return hx(this)!=null;case 15:return(this.Bb&zc)!=0;case 16:return(this.Bb&a1)!=0;case 17:return!!hw(this)}return fl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.sh=function(t,n){var s,a;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:QW(this,Jn(n));return;case 2:w0(this,ft(lt(n)));return;case 3:m0(this,ft(lt(n)));return;case 4:p0(this,h(n,19).a);return;case 5:this.ok(h(n,19).a);return;case 8:Vp(this,h(n,138));return;case 9:a=Df(this,h(n,87),null),a&&a.Fi();return;case 10:Wk(this,ft(lt(n)));return;case 11:Qk(this,ft(lt(n)));return;case 12:Kk(this,ft(lt(n)));return;case 13:pfe(this,Jn(n));return;case 15:Xk(this,ft(lt(n)));return;case 16:Zk(this,ft(lt(n)));return}ml(this,t-En(this.zh()),Ht((s=h(tn(this,16),26),s||this.zh()),t),n)},g.zh=function(){return Bt(),q9t},g.Bh=function(t){var n,s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:ue(this.Cb,88)&&Lw(ou(h(this.Cb,88)),4),Ko(this,null);return;case 2:w0(this,!0);return;case 3:m0(this,!0);return;case 4:p0(this,0);return;case 5:this.ok(1);return;case 8:Vp(this,null);return;case 9:s=Df(this,null,null),s&&s.Fi();return;case 10:Wk(this,!0);return;case 11:Qk(this,!1);return;case 12:Kk(this,!1);return;case 13:this.i=null,WB(this,null);return;case 15:Xk(this,!1);return;case 16:Zk(this,!1);return}vl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.Gh=function(){gk(Sa((jc(),Ps),this)),Rl(this),this.Bb|=1},g.Gj=function(){return this.f},g.zj=function(){return hx(this)},g.Hj=function(){return hw(this)},g.Lj=function(){return null},g.pk=function(){return this.k},g.aj=function(){return this.n},g.Mj=function(){return VF(this)},g.Nj=function(){var t,n,s,a,u,p,v,m,x;return this.p||(s=hw(this),(s.i==null&&s1(s),s.i).length,a=this.Lj(),a&&En(hw(a)),u=Rl(this),v=u.Bj(),t=v?(v.i&1)!=0?v==vu?es:v==Hn?Ws:v==Em?zx:v==ks?Ts:v==ap?Xb:v==D3?Qb:v==Vc?A5:f_:v:null,n=hx(this),m=u.zj(),Skn(this),(this.Bb&a1)!=0&&(!!(p=T2e((jc(),Ps),s))&&p!=this||!!(p=Y4(Sa(Ps,this))))?this.p=new EJe(this,p):this.$j()?this.rk()?a?(this.Bb&zc)!=0?t?this.sk()?this.p=new Dp(47,t,this,a):this.p=new Dp(5,t,this,a):this.sk()?this.p=new Op(46,this,a):this.p=new Op(4,this,a):t?this.sk()?this.p=new Dp(49,t,this,a):this.p=new Dp(7,t,this,a):this.sk()?this.p=new Op(48,this,a):this.p=new Op(6,this,a):(this.Bb&zc)!=0?t?t==c2?this.p=new u0(50,A9t,this):this.sk()?this.p=new u0(43,t,this):this.p=new u0(1,t,this):this.sk()?this.p=new h0(42,this):this.p=new h0(0,this):t?t==c2?this.p=new u0(41,A9t,this):this.sk()?this.p=new u0(45,t,this):this.p=new u0(3,t,this):this.sk()?this.p=new h0(44,this):this.p=new h0(2,this):ue(u,148)?t==ZH?this.p=new h0(40,this):(this.Bb&512)!=0?(this.Bb&zc)!=0?t?this.p=new u0(9,t,this):this.p=new h0(8,this):t?this.p=new u0(11,t,this):this.p=new h0(10,this):(this.Bb&zc)!=0?t?this.p=new u0(13,t,this):this.p=new h0(12,this):t?this.p=new u0(15,t,this):this.p=new h0(14,this):a?(x=a.t,x>1||x==-1?this.sk()?(this.Bb&zc)!=0?t?this.p=new Dp(25,t,this,a):this.p=new Op(24,this,a):t?this.p=new Dp(27,t,this,a):this.p=new Op(26,this,a):(this.Bb&zc)!=0?t?this.p=new Dp(29,t,this,a):this.p=new Op(28,this,a):t?this.p=new Dp(31,t,this,a):this.p=new Op(30,this,a):this.sk()?(this.Bb&zc)!=0?t?this.p=new Dp(33,t,this,a):this.p=new Op(32,this,a):t?this.p=new Dp(35,t,this,a):this.p=new Op(34,this,a):(this.Bb&zc)!=0?t?this.p=new Dp(37,t,this,a):this.p=new Op(36,this,a):t?this.p=new Dp(39,t,this,a):this.p=new Op(38,this,a)):this.sk()?(this.Bb&zc)!=0?t?this.p=new u0(17,t,this):this.p=new h0(16,this):t?this.p=new u0(19,t,this):this.p=new h0(18,this):(this.Bb&zc)!=0?t?this.p=new u0(21,t,this):this.p=new h0(20,this):t?this.p=new u0(23,t,this):this.p=new h0(22,this):this.qk()?this.sk()?this.p=new fnt(h(u,26),this,a):this.p=new Zde(h(u,26),this,a):ue(u,148)?t==ZH?this.p=new h0(40,this):(this.Bb&zc)!=0?t?this.p=new lrt(n,m,this,(_X(),v==Hn?CEe:v==vu?kEe:v==ap?SEe:v==Em?TEe:v==ks?_Ee:v==D3?AEe:v==Vc?xEe:v==Cl?EEe:aie)):this.p=new xit(h(u,148),n,m,this):t?this.p=new urt(n,m,this,(_X(),v==Hn?CEe:v==vu?kEe:v==ap?SEe:v==Em?TEe:v==ks?_Ee:v==D3?AEe:v==Vc?xEe:v==Cl?EEe:aie)):this.p=new kit(h(u,148),n,m,this):this.rk()?a?(this.Bb&zc)!=0?this.sk()?this.p=new gnt(h(u,26),this,a):this.p=new G1e(h(u,26),this,a):this.sk()?this.p=new dnt(h(u,26),this,a):this.p=new JY(h(u,26),this,a):(this.Bb&zc)!=0?this.sk()?this.p=new utt(h(u,26),this):this.p=new r1e(h(u,26),this):this.sk()?this.p=new ctt(h(u,26),this):this.p=new jY(h(u,26),this):this.sk()?a?(this.Bb&zc)!=0?this.p=new pnt(h(u,26),this,a):this.p=new H1e(h(u,26),this,a):(this.Bb&zc)!=0?this.p=new ltt(h(u,26),this):this.p=new i1e(h(u,26),this):a?(this.Bb&zc)!=0?this.p=new bnt(h(u,26),this,a):this.p=new z1e(h(u,26),this,a):(this.Bb&zc)!=0?this.p=new htt(h(u,26),this):this.p=new AP(h(u,26),this)),this.p},g.Ij=function(){return(this.Bb&Ch)!=0},g.qk=function(){return!1},g.rk=function(){return!1},g.Jj=function(){return(this.Bb&a1)!=0},g.Oj=function(){return SK(this)},g.sk=function(){return!1},g.Kj=function(){return(this.Bb&zc)!=0},g.tk=function(t){this.k=t},g.Lh=function(t){QW(this,t)},g.Ib=function(){return cR(this)},g.e=!1,g.n=0,D(Qt,"EStructuralFeatureImpl",449),M(322,449,{105:1,92:1,90:1,34:1,147:1,191:1,56:1,170:1,66:1,108:1,472:1,49:1,97:1,322:1,150:1,449:1,284:1,114:1,115:1,677:1},Cq),g._g=function(t,n,s){var a,u;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return on(),(this.Bb&256)!=0;case 3:return on(),(this.Bb&512)!=0;case 4:return Ke(this.s);case 5:return Ke(this.t);case 6:return on(),!!ybe(this);case 7:return on(),u=this.s,u>=1;case 8:return n?Rl(this):this.r;case 9:return this.q;case 10:return on(),(this.Bb&Ch)!=0;case 11:return on(),(this.Bb&Ww)!=0;case 12:return on(),(this.Bb&$w)!=0;case 13:return this.j;case 14:return hx(this);case 15:return on(),(this.Bb&zc)!=0;case 16:return on(),(this.Bb&a1)!=0;case 17:return hw(this);case 18:return on(),(this.Bb&go)!=0;case 19:return n?UK(this):xot(this)}return gl(this,t-En((Bt(),ym)),Ht((a=h(tn(this,16),26),a||ym),t),n,s)},g.lh=function(t){var n,s;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return(this.Bb&256)==0;case 3:return(this.Bb&512)==0;case 4:return this.s!=0;case 5:return this.t!=1;case 6:return ybe(this);case 7:return s=this.s,s>=1;case 8:return!!this.r&&!this.q.e&&hb(this.q).i==0;case 9:return!!this.q&&!(!!this.r&&!this.q.e&&hb(this.q).i==0);case 10:return(this.Bb&Ch)==0;case 11:return(this.Bb&Ww)!=0;case 12:return(this.Bb&$w)!=0;case 13:return this.j!=null;case 14:return hx(this)!=null;case 15:return(this.Bb&zc)!=0;case 16:return(this.Bb&a1)!=0;case 17:return!!hw(this);case 18:return(this.Bb&go)!=0;case 19:return!!xot(this)}return fl(this,t-En((Bt(),ym)),Ht((n=h(tn(this,16),26),n||ym),t))},g.sh=function(t,n){var s,a;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:QW(this,Jn(n));return;case 2:w0(this,ft(lt(n)));return;case 3:m0(this,ft(lt(n)));return;case 4:p0(this,h(n,19).a);return;case 5:ZXe(this,h(n,19).a);return;case 8:Vp(this,h(n,138));return;case 9:a=Df(this,h(n,87),null),a&&a.Fi();return;case 10:Wk(this,ft(lt(n)));return;case 11:Qk(this,ft(lt(n)));return;case 12:Kk(this,ft(lt(n)));return;case 13:pfe(this,Jn(n));return;case 15:Xk(this,ft(lt(n)));return;case 16:Zk(this,ft(lt(n)));return;case 18:bX(this,ft(lt(n)));return}ml(this,t-En((Bt(),ym)),Ht((s=h(tn(this,16),26),s||ym),t),n)},g.zh=function(){return Bt(),ym},g.Bh=function(t){var n,s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:ue(this.Cb,88)&&Lw(ou(h(this.Cb,88)),4),Ko(this,null);return;case 2:w0(this,!0);return;case 3:m0(this,!0);return;case 4:p0(this,0);return;case 5:this.b=0,yw(this,1);return;case 8:Vp(this,null);return;case 9:s=Df(this,null,null),s&&s.Fi();return;case 10:Wk(this,!0);return;case 11:Qk(this,!1);return;case 12:Kk(this,!1);return;case 13:this.i=null,WB(this,null);return;case 15:Xk(this,!1);return;case 16:Zk(this,!1);return;case 18:bX(this,!1);return}vl(this,t-En((Bt(),ym)),Ht((n=h(tn(this,16),26),n||ym),t))},g.Gh=function(){UK(this),gk(Sa((jc(),Ps),this)),Rl(this),this.Bb|=1},g.$j=function(){return ybe(this)},g.nk=function(t,n){return this.b=0,this.a=null,dpe(this,t,n)},g.ok=function(t){ZXe(this,t)},g.Ib=function(){var t;return(this.Db&64)!=0?cR(this):(t=new Ol(cR(this)),t.a+=" (iD: ",a0(t,(this.Bb&go)!=0),t.a+=")",t.a)},g.b=0,D(Qt,"EAttributeImpl",322),M(351,438,{105:1,92:1,90:1,138:1,147:1,191:1,56:1,108:1,49:1,97:1,351:1,150:1,114:1,115:1,676:1}),g.uk=function(t){return t.Tg()==this},g.Qg=function(t){return XX(this,t)},g.Rg=function(t,n){this.w=null,this.Db=n<<16|this.Db&255,this.Cb=t},g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return this.D!=null?this.D:this.B;case 3:return Nb(this);case 4:return this.zj();case 5:return this.F;case 6:return n?Vu(this):vk(this);case 7:return!this.A&&(this.A=new Nc(lc,this,7)),this.A}return gl(this,t-En(this.zh()),Ht((a=h(tn(this,16),26),a||this.zh()),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 6:return this.Cb&&(s=(u=this.Db>>16,u>=0?XX(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,6,s)}return p=h(Ht((a=h(tn(this,16),26),a||this.zh()),n),66),p.Nj().Qj(this,ic(this),n-En(this.zh()),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 6:return Ku(this,null,6,s);case 7:return!this.A&&(this.A=new Nc(lc,this,7)),qs(this.A,t,s)}return u=h(Ht((a=h(tn(this,16),26),a||this.zh()),n),66),u.Nj().Rj(this,ic(this),n-En(this.zh()),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return this.D!=null&&this.D==this.F;case 3:return!!Nb(this);case 4:return this.zj()!=null;case 5:return this.F!=null&&this.F!=this.D&&this.F!=this.B;case 6:return!!vk(this);case 7:return!!this.A&&this.A.i!=0}return fl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:cB(this,Jn(n));return;case 2:CY(this,Jn(n));return;case 5:vx(this,Jn(n));return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A),!this.A&&(this.A=new Nc(lc,this,7)),mi(this.A,h(n,14));return}ml(this,t-En(this.zh()),Ht((s=h(tn(this,16),26),s||this.zh()),t),n)},g.zh=function(){return Bt(),$9t},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:ue(this.Cb,179)&&(h(this.Cb,179).tb=null),Ko(this,null);return;case 2:Vk(this,null),Ik(this,this.D);return;case 5:vx(this,null);return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A);return}vl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.yj=function(){var t;return this.G==-1&&(this.G=(t=Vu(this),t?y0(t.Mh(),this):-1)),this.G},g.zj=function(){return null},g.Aj=function(){return Vu(this)},g.vk=function(){return this.v},g.Bj=function(){return Nb(this)},g.Cj=function(){return this.D!=null?this.D:this.B},g.Dj=function(){return this.F},g.wj=function(t){return zQ(this,t)},g.wk=function(t){this.v=t},g.xk=function(t){Zut(this,t)},g.yk=function(t){this.C=t},g.Lh=function(t){cB(this,t)},g.Ib=function(){return yF(this)},g.C=null,g.D=null,g.G=-1,D(Qt,"EClassifierImpl",351),M(88,351,{105:1,92:1,90:1,26:1,138:1,147:1,191:1,56:1,108:1,49:1,97:1,88:1,351:1,150:1,473:1,114:1,115:1,676:1},ile),g.uk=function(t){return ugn(this,t.Tg())},g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return this.D!=null?this.D:this.B;case 3:return Nb(this);case 4:return null;case 5:return this.F;case 6:return n?Vu(this):vk(this);case 7:return!this.A&&(this.A=new Nc(lc,this,7)),this.A;case 8:return on(),(this.Bb&256)!=0;case 9:return on(),(this.Bb&512)!=0;case 10:return La(this);case 11:return!this.q&&(this.q=new Ye(eh,this,11,10)),this.q;case 12:return $y(this);case 13:return A9(this);case 14:return A9(this),this.r;case 15:return $y(this),this.k;case 16:return lbe(this);case 17:return YQ(this);case 18:return s1(this);case 19:return tR(this);case 20:return $y(this),this.o;case 21:return!this.s&&(this.s=new Ye(Ac,this,21,17)),this.s;case 22:return Co(this);case 23:return IQ(this)}return gl(this,t-En((Bt(),np)),Ht((a=h(tn(this,16),26),a||np),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 6:return this.Cb&&(s=(u=this.Db>>16,u>=0?XX(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,6,s);case 11:return!this.q&&(this.q=new Ye(eh,this,11,10)),Xo(this.q,t,s);case 21:return!this.s&&(this.s=new Ye(Ac,this,21,17)),Xo(this.s,t,s)}return p=h(Ht((a=h(tn(this,16),26),a||(Bt(),np)),n),66),p.Nj().Qj(this,ic(this),n-En((Bt(),np)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 6:return Ku(this,null,6,s);case 7:return!this.A&&(this.A=new Nc(lc,this,7)),qs(this.A,t,s);case 11:return!this.q&&(this.q=new Ye(eh,this,11,10)),qs(this.q,t,s);case 21:return!this.s&&(this.s=new Ye(Ac,this,21,17)),qs(this.s,t,s);case 22:return qs(Co(this),t,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),np)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),np)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return this.D!=null&&this.D==this.F;case 3:return!!Nb(this);case 4:return!1;case 5:return this.F!=null&&this.F!=this.D&&this.F!=this.B;case 6:return!!vk(this);case 7:return!!this.A&&this.A.i!=0;case 8:return(this.Bb&256)!=0;case 9:return(this.Bb&512)!=0;case 10:return!!this.u&&Co(this.u.a).i!=0&&!(!!this.n&&jX(this.n));case 11:return!!this.q&&this.q.i!=0;case 12:return $y(this).i!=0;case 13:return A9(this).i!=0;case 14:return A9(this),this.r.i!=0;case 15:return $y(this),this.k.i!=0;case 16:return lbe(this).i!=0;case 17:return YQ(this).i!=0;case 18:return s1(this).i!=0;case 19:return tR(this).i!=0;case 20:return $y(this),!!this.o;case 21:return!!this.s&&this.s.i!=0;case 22:return!!this.n&&jX(this.n);case 23:return IQ(this).i!=0}return fl(this,t-En((Bt(),np)),Ht((n=h(tn(this,16),26),n||np),t))},g.oh=function(t){var n;return n=this.i==null||!!this.q&&this.q.i!=0?null:yM(this,t),n||Hve(this,t)},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:cB(this,Jn(n));return;case 2:CY(this,Jn(n));return;case 5:vx(this,Jn(n));return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A),!this.A&&(this.A=new Nc(lc,this,7)),mi(this.A,h(n,14));return;case 8:ppe(this,ft(lt(n)));return;case 9:bpe(this,ft(lt(n)));return;case 10:L9(La(this)),mi(La(this),h(n,14));return;case 11:!this.q&&(this.q=new Ye(eh,this,11,10)),jn(this.q),!this.q&&(this.q=new Ye(eh,this,11,10)),mi(this.q,h(n,14));return;case 21:!this.s&&(this.s=new Ye(Ac,this,21,17)),jn(this.s),!this.s&&(this.s=new Ye(Ac,this,21,17)),mi(this.s,h(n,14));return;case 22:jn(Co(this)),mi(Co(this),h(n,14));return}ml(this,t-En((Bt(),np)),Ht((s=h(tn(this,16),26),s||np),t),n)},g.zh=function(){return Bt(),np},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:ue(this.Cb,179)&&(h(this.Cb,179).tb=null),Ko(this,null);return;case 2:Vk(this,null),Ik(this,this.D);return;case 5:vx(this,null);return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A);return;case 8:ppe(this,!1);return;case 9:bpe(this,!1);return;case 10:this.u&&L9(this.u);return;case 11:!this.q&&(this.q=new Ye(eh,this,11,10)),jn(this.q);return;case 21:!this.s&&(this.s=new Ye(Ac,this,21,17)),jn(this.s);return;case 22:this.n&&jn(this.n);return}vl(this,t-En((Bt(),np)),Ht((n=h(tn(this,16),26),n||np),t))},g.Gh=function(){var t,n;if($y(this),A9(this),lbe(this),YQ(this),s1(this),tR(this),IQ(this),VE(C2n(ou(this))),this.s)for(t=0,n=this.s.i;t<n;++t)zS(he(this.s,t));if(this.q)for(t=0,n=this.q.i;t<n;++t)zS(he(this.q,t));v0((jc(),Ps),this).ne(),this.Bb|=1},g.Ib=function(){return I2e(this)},g.k=null,g.r=null;var fT,X9t,iie;D(Qt,"EClassImpl",88),M(1994,1993,pyt),g.Vh=function(t,n){return GQ(this,t,n)},g.Wh=function(t){return GQ(this,this.i,t)},g.Xh=function(t,n){Tbe(this,t,n)},g.Yh=function(t){kQ(this,t)},g.lk=function(t,n){return Xo(this,t,n)},g.pi=function(t){return Y0e(this,t)},g.mk=function(t,n){return qs(this,t,n)},g.mi=function(t,n){return c2t(this,t,n)},g.Zh=function(){return new P4(this)},g.$h=function(){return new HS(this)},g._h=function(t){return IA(this,t)},D(ri,"NotifyingInternalEListImpl",1994),M(622,1994,Va),g.Hc=function(t){return C2t(this,t)},g.Zi=function(t,n,s,a,u){return RE(this,t,n,s,a,u)},g.$i=function(t){$6(this,t)},g.Wj=function(t){return this},g.ak=function(){return Ht(this.e.Tg(),this.aj())},g._i=function(){return this.ak()},g.aj=function(){return oi(this.e.Tg(),this.ak())},g.zk=function(){return h(this.ak().Yj(),26).Bj()},g.Ak=function(){return aa(h(this.ak(),18)).n},g.Ai=function(){return this.e},g.Bk=function(){return!0},g.Ck=function(){return!1},g.Dk=function(){return!1},g.Ek=function(){return!1},g.Xc=function(t){return y0(this,t)},g.cj=function(t,n){var s;return s=h(t,49),this.Dk()?this.Bk()?s.gh(this.e,this.Ak(),this.zk(),n):s.gh(this.e,oi(s.Tg(),aa(h(this.ak(),18))),null,n):s.gh(this.e,-1-this.aj(),null,n)},g.dj=function(t,n){var s;return s=h(t,49),this.Dk()?this.Bk()?s.ih(this.e,this.Ak(),this.zk(),n):s.ih(this.e,oi(s.Tg(),aa(h(this.ak(),18))),null,n):s.ih(this.e,-1-this.aj(),null,n)},g.rk=function(){return!1},g.Fk=function(){return!0},g.wj=function(t){return xat(this.d,t)},g.ej=function(){return Tu(this.e)},g.fj=function(){return this.i!=0},g.ri=function(t){return LA(this.d,t)},g.li=function(t,n){return this.Fk()&&this.Ek()?f5(this,t,h(n,56)):n},g.Gk=function(t){return t.kh()?Tg(this.e,h(t,49)):t},g.Wb=function(t){yet(this,t)},g.Pc=function(){return act(this)},g.Qc=function(t){var n;if(this.Ek())for(n=this.i-1;n>=0;--n)he(this,n);return Rpe(this,t)},g.Xj=function(){jn(this)},g.oi=function(t,n){return kut(this,t,n)},D(ri,"EcoreEList",622),M(496,622,Va,JS),g.ai=function(){return!1},g.aj=function(){return this.c},g.bj=function(){return!1},g.Fk=function(){return!0},g.hi=function(){return!0},g.li=function(t,n){return n},g.ni=function(){return!1},g.c=0,D(ri,"EObjectEList",496),M(85,496,Va,Gi),g.bj=function(){return!0},g.Dk=function(){return!1},g.rk=function(){return!0},D(ri,"EObjectContainmentEList",85),M(545,85,Va,iP),g.ci=function(){this.b=!0},g.fj=function(){return this.b},g.Xj=function(){var t;jn(this),Tu(this.e)?(t=this.b,this.b=!1,Or(this.e,new kh(this.e,2,this.c,t,!1))):this.b=!1},g.b=!1,D(ri,"EObjectContainmentEList/Unsettable",545),M(1140,545,Va,ort),g.ii=function(t,n){var s,a;return s=h(a9(this,t,n),87),Tu(this.e)&&$6(this,new yA(this.a,7,(Bt(),H9t),Ke(n),(a=s.c,ue(a,88)?h(a,26):nh),t)),s},g.jj=function(t,n){return gxn(this,h(t,87),n)},g.kj=function(t,n){return dxn(this,h(t,87),n)},g.lj=function(t,n,s){return pEn(this,h(t,87),h(n,87),s)},g.Zi=function(t,n,s,a,u){switch(t){case 3:return RE(this,t,n,s,a,this.i>1);case 5:return RE(this,t,n,s,a,this.i-h(s,15).gc()>0);default:return new Sd(this.e,t,this.c,n,s,a,!0)}},g.ij=function(){return!0},g.fj=function(){return jX(this)},g.Xj=function(){jn(this)},D(Qt,"EClassImpl/1",1140),M(1154,1153,Xye),g.ui=function(t){var n,s,a,u,p,v,m;if(s=t.xi(),s!=8){if(a=Qxn(t),a==0)switch(s){case 1:case 9:{m=t.Bi(),m!=null&&(n=ou(h(m,473)),!n.c&&(n.c=new N6),IB(n.c,t.Ai())),v=t.zi(),v!=null&&(u=h(v,473),(u.Bb&1)==0&&(n=ou(u),!n.c&&(n.c=new N6),qn(n.c,h(t.Ai(),26))));break}case 3:{v=t.zi(),v!=null&&(u=h(v,473),(u.Bb&1)==0&&(n=ou(u),!n.c&&(n.c=new N6),qn(n.c,h(t.Ai(),26))));break}case 5:{if(v=t.zi(),v!=null)for(p=h(v,14).Kc();p.Ob();)u=h(p.Pb(),473),(u.Bb&1)==0&&(n=ou(u),!n.c&&(n.c=new N6),qn(n.c,h(t.Ai(),26)));break}case 4:{m=t.Bi(),m!=null&&(u=h(m,473),(u.Bb&1)==0&&(n=ou(u),!n.c&&(n.c=new N6),IB(n.c,t.Ai())));break}case 6:{if(m=t.Bi(),m!=null)for(p=h(m,14).Kc();p.Ob();)u=h(p.Pb(),473),(u.Bb&1)==0&&(n=ou(u),!n.c&&(n.c=new N6),IB(n.c,t.Ai()));break}}this.Hk(a)}},g.Hk=function(t){apt(this,t)},g.b=63,D(Qt,"ESuperAdapter",1154),M(1155,1154,Xye,vKe),g.Hk=function(t){Lw(this,t)},D(Qt,"EClassImpl/10",1155),M(1144,696,Va),g.Vh=function(t,n){return oQ(this,t,n)},g.Wh=function(t){return n1t(this,t)},g.Xh=function(t,n){YA(this,t,n)},g.Yh=function(t){pA(this,t)},g.pi=function(t){return Y0e(this,t)},g.mi=function(t,n){return CK(this,t,n)},g.lk=function(t,n){throw ne(new Xn)},g.Zh=function(){return new P4(this)},g.$h=function(){return new HS(this)},g._h=function(t){return IA(this,t)},g.mk=function(t,n){throw ne(new Xn)},g.Wj=function(t){return this},g.fj=function(){return this.i!=0},g.Wb=function(t){throw ne(new Xn)},g.Xj=function(){throw ne(new Xn)},D(ri,"EcoreEList/UnmodifiableEList",1144),M(319,1144,Va,ay),g.ni=function(){return!1},D(ri,"EcoreEList/UnmodifiableEList/FastCompare",319),M(1147,319,Va,Vlt),g.Xc=function(t){var n,s,a;if(ue(t,170)&&(n=h(t,170),s=n.aj(),s!=-1)){for(a=this.i;s<a;++s)if(Me(this.g[s])===Me(t))return s}return-1},D(Qt,"EClassImpl/1EAllStructuralFeaturesList",1147),M(1141,497,b1,$ze),g.ri=function(t){return pe(pa,byt,87,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/1EGenericSuperTypeEList",1141),M(623,497,b1,hq),g.ri=function(t){return pe(Ac,Jy,170,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/1EStructuralFeatureUniqueEList",623),M(741,497,b1,ele),g.ri=function(t){return pe(mm,Jy,18,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/1ReferenceList",741),M(1142,497,b1,wKe),g.bi=function(t,n){vbn(this,h(n,34))},g.ri=function(t){return pe(wm,Jy,34,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/2",1142),M(1143,497,b1,Hze),g.ri=function(t){return pe(wm,Jy,34,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/3",1143),M(1145,319,Va,_nt),g.Fc=function(t){return y2n(this,h(t,34))},g.Yh=function(t){A1n(this,h(t,34))},D(Qt,"EClassImpl/4",1145),M(1146,319,Va,Tnt),g.Fc=function(t){return k2n(this,h(t,18))},g.Yh=function(t){M1n(this,h(t,18))},D(Qt,"EClassImpl/5",1146),M(1148,497,b1,zze),g.ri=function(t){return pe(eh,Jye,59,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/6",1148),M(1149,497,b1,Gze),g.ri=function(t){return pe(mm,Jy,18,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/7",1149),M(1997,1996,{3:1,4:1,20:1,28:1,52:1,14:1,15:1,67:1,58:1,69:1}),g.Vh=function(t,n){return vve(this,t,n)},g.Wh=function(t){return vve(this,this.Vi(),t)},g.Xh=function(t,n){j0t(this,t,n)},g.Yh=function(t){D0t(this,t)},g.lk=function(t,n){return F8n(this,t,n)},g.mk=function(t,n){return o8n(this,t,n)},g.mi=function(t,n){return Ypt(this,t,n)},g.pi=function(t){return this.Oi(t)},g.Zh=function(){return new P4(this)},g.Gi=function(){return this.Ji()},g.$h=function(){return new HS(this)},g._h=function(t){return IA(this,t)},D(ri,"DelegatingNotifyingInternalEListImpl",1997),M(742,1997,e3e),g.ai=function(){var t;return t=Ht(bc(this.b),this.aj()).Yj(),ue(t,148)&&!ue(t,457)&&(t.Bj().i&1)==0},g.Hc=function(t){var n,s,a,u,p,v,m,x;if(this.Fk()){if(x=this.Vi(),x>4)if(this.wj(t)){if(this.rk()){if(a=h(t,49),s=a.Ug(),m=s==this.b&&(this.Dk()?a.Og(a.Vg(),h(Ht(bc(this.b),this.aj()).Yj(),26).Bj())==aa(h(Ht(bc(this.b),this.aj()),18)).n:-1-a.Vg()==this.aj()),this.Ek()&&!m&&!s&&!!a.Zg()){for(u=0;u<x;++u)if(n=gW(this,this.Oi(u)),Me(n)===Me(t))return!0}return m}else if(this.Dk()&&!this.Ck()){if(p=h(t,56).ah(aa(h(Ht(bc(this.b),this.aj()),18))),Me(p)===Me(this.b))return!0;if(p==null||!h(p,56).kh())return!1}}else return!1;if(v=this.Li(t),this.Ek()&&!v){for(u=0;u<x;++u)if(a=gW(this,this.Oi(u)),Me(a)===Me(t))return!0}return v}else return this.Li(t)},g.Zi=function(t,n,s,a,u){return new Sd(this.b,t,this.aj(),n,s,a,u)},g.$i=function(t){Or(this.b,t)},g.Wj=function(t){return this},g._i=function(){return Ht(bc(this.b),this.aj())},g.aj=function(){return oi(bc(this.b),Ht(bc(this.b),this.aj()))},g.Ai=function(){return this.b},g.Bk=function(){return!!Ht(bc(this.b),this.aj()).Yj().Bj()},g.bj=function(){var t,n;return n=Ht(bc(this.b),this.aj()),ue(n,99)?(t=h(n,18),(t.Bb&go)!=0||!!aa(h(n,18))):!1},g.Ck=function(){var t,n,s,a;return n=Ht(bc(this.b),this.aj()),ue(n,99)?(t=h(n,18),s=aa(t),!!s&&(a=s.t,a>1||a==-1)):!1},g.Dk=function(){var t,n,s;return n=Ht(bc(this.b),this.aj()),ue(n,99)?(t=h(n,18),s=aa(t),!!s):!1},g.Ek=function(){var t,n;return n=Ht(bc(this.b),this.aj()),ue(n,99)?(t=h(n,18),(t.Bb&ea)!=0):!1},g.Xc=function(t){var n,s,a,u;if(a=this.Qi(t),a>=0)return a;if(this.Fk()){for(s=0,u=this.Vi();s<u;++s)if(n=gW(this,this.Oi(s)),Me(n)===Me(t))return s}return-1},g.cj=function(t,n){var s;return s=h(t,49),this.Dk()?this.Bk()?s.gh(this.b,aa(h(Ht(bc(this.b),this.aj()),18)).n,h(Ht(bc(this.b),this.aj()).Yj(),26).Bj(),n):s.gh(this.b,oi(s.Tg(),aa(h(Ht(bc(this.b),this.aj()),18))),null,n):s.gh(this.b,-1-this.aj(),null,n)},g.dj=function(t,n){var s;return s=h(t,49),this.Dk()?this.Bk()?s.ih(this.b,aa(h(Ht(bc(this.b),this.aj()),18)).n,h(Ht(bc(this.b),this.aj()).Yj(),26).Bj(),n):s.ih(this.b,oi(s.Tg(),aa(h(Ht(bc(this.b),this.aj()),18))),null,n):s.ih(this.b,-1-this.aj(),null,n)},g.rk=function(){var t,n;return n=Ht(bc(this.b),this.aj()),ue(n,99)?(t=h(n,18),(t.Bb&go)!=0):!1},g.Fk=function(){return ue(Ht(bc(this.b),this.aj()).Yj(),88)},g.wj=function(t){return Ht(bc(this.b),this.aj()).Yj().wj(t)},g.ej=function(){return Tu(this.b)},g.fj=function(){return!this.Ri()},g.hi=function(){return Ht(bc(this.b),this.aj()).hi()},g.li=function(t,n){return MM(this,t,n)},g.Wb=function(t){L9(this),mi(this,h(t,15))},g.Pc=function(){var t;if(this.Ek())for(t=this.Vi()-1;t>=0;--t)MM(this,t,this.Oi(t));return this.Wi()},g.Qc=function(t){var n;if(this.Ek())for(n=this.Vi()-1;n>=0;--n)MM(this,n,this.Oi(n));return this.Xi(t)},g.Xj=function(){L9(this)},g.oi=function(t,n){return sct(this,t,n)},D(ri,"DelegatingEcoreEList",742),M(1150,742,e3e,xtt),g.Hi=function(t,n){F0n(this,t,h(n,26))},g.Ii=function(t){Ldn(this,h(t,26))},g.Oi=function(t){var n,s;return n=h(he(Co(this.a),t),87),s=n.c,ue(s,88)?h(s,26):(Bt(),nh)},g.Ti=function(t){var n,s;return n=h(Ow(Co(this.a),t),87),s=n.c,ue(s,88)?h(s,26):(Bt(),nh)},g.Ui=function(t,n){return R8n(this,t,h(n,26))},g.ai=function(){return!1},g.Zi=function(t,n,s,a,u){return null},g.Ji=function(){return new mKe(this)},g.Ki=function(){jn(Co(this.a))},g.Li=function(t){return Pht(this,t)},g.Mi=function(t){var n,s;for(s=t.Kc();s.Ob();)if(n=s.Pb(),!Pht(this,n))return!1;return!0},g.Ni=function(t){var n,s,a;if(ue(t,15)&&(a=h(t,15),a.gc()==Co(this.a).i)){for(n=a.Kc(),s=new Tn(this);n.Ob();)if(Me(n.Pb())!==Me(On(s)))return!1;return!0}return!1},g.Pi=function(){var t,n,s,a,u;for(s=1,n=new Tn(Co(this.a));n.e!=n.i.gc();)t=h(On(n),87),a=(u=t.c,ue(u,88)?h(u,26):(Bt(),nh)),s=31*s+(a?rb(a):0);return s},g.Qi=function(t){var n,s,a,u;for(a=0,s=new Tn(Co(this.a));s.e!=s.i.gc();){if(n=h(On(s),87),Me(t)===Me((u=n.c,ue(u,88)?h(u,26):(Bt(),nh))))return a;++a}return-1},g.Ri=function(){return Co(this.a).i==0},g.Si=function(){return null},g.Vi=function(){return Co(this.a).i},g.Wi=function(){var t,n,s,a,u,p;for(p=Co(this.a).i,u=pe(yn,st,1,p,5,1),s=0,n=new Tn(Co(this.a));n.e!=n.i.gc();)t=h(On(n),87),u[s++]=(a=t.c,ue(a,88)?h(a,26):(Bt(),nh));return u},g.Xi=function(t){var n,s,a,u,p,v,m;for(m=Co(this.a).i,t.length<m&&(u=LA(uu(t).c,m),t=u),t.length>m&&pi(t,m,null),a=0,s=new Tn(Co(this.a));s.e!=s.i.gc();)n=h(On(s),87),p=(v=n.c,ue(v,88)?h(v,26):(Bt(),nh)),pi(t,a++,p);return t},g.Yi=function(){var t,n,s,a,u;for(u=new s0,u.a+="[",t=Co(this.a),n=0,a=Co(this.a).i;n<a;)va(u,mE((s=h(he(t,n),87).c,ue(s,88)?h(s,26):(Bt(),nh)))),++n<a&&(u.a+=Js);return u.a+="]",u.a},g.$i=function(t){},g.aj=function(){return 10},g.Bk=function(){return!0},g.bj=function(){return!1},g.Ck=function(){return!1},g.Dk=function(){return!1},g.Ek=function(){return!0},g.rk=function(){return!1},g.Fk=function(){return!0},g.wj=function(t){return ue(t,88)},g.fj=function(){return Fmn(this.a)},g.hi=function(){return!0},g.ni=function(){return!0},D(Qt,"EClassImpl/8",1150),M(1151,1964,Ex,mKe),g.Zc=function(t){return IA(this.a,t)},g.gc=function(){return Co(this.a.a).i},D(Qt,"EClassImpl/8/1",1151),M(1152,497,b1,Vze),g.ri=function(t){return pe(af,st,138,t,0,1)},g.ni=function(){return!1},D(Qt,"EClassImpl/9",1152),M(1139,53,nwe,lXe),D(Qt,"EClassImpl/MyHashSet",1139),M(566,351,{105:1,92:1,90:1,138:1,148:1,834:1,147:1,191:1,56:1,108:1,49:1,97:1,351:1,150:1,114:1,115:1,676:1},rN),g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return this.D!=null?this.D:this.B;case 3:return Nb(this);case 4:return this.zj();case 5:return this.F;case 6:return n?Vu(this):vk(this);case 7:return!this.A&&(this.A=new Nc(lc,this,7)),this.A;case 8:return on(),(this.Bb&256)!=0}return gl(this,t-En(this.zh()),Ht((a=h(tn(this,16),26),a||this.zh()),t),n,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return this.D!=null&&this.D==this.F;case 3:return!!Nb(this);case 4:return this.zj()!=null;case 5:return this.F!=null&&this.F!=this.D&&this.F!=this.B;case 6:return!!vk(this);case 7:return!!this.A&&this.A.i!=0;case 8:return(this.Bb&256)==0}return fl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:cB(this,Jn(n));return;case 2:CY(this,Jn(n));return;case 5:vx(this,Jn(n));return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A),!this.A&&(this.A=new Nc(lc,this,7)),mi(this.A,h(n,14));return;case 8:gF(this,ft(lt(n)));return}ml(this,t-En(this.zh()),Ht((s=h(tn(this,16),26),s||this.zh()),t),n)},g.zh=function(){return Bt(),z9t},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:ue(this.Cb,179)&&(h(this.Cb,179).tb=null),Ko(this,null);return;case 2:Vk(this,null),Ik(this,this.D);return;case 5:vx(this,null);return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A);return;case 8:gF(this,!0);return}vl(this,t-En(this.zh()),Ht((n=h(tn(this,16),26),n||this.zh()),t))},g.Gh=function(){v0((jc(),Ps),this).ne(),this.Bb|=1},g.Fj=function(){var t,n,s;if(!this.c&&(t=ogt(Vu(this)),!t.dc()))for(s=t.Kc();s.Ob();)n=Jn(s.Pb()),T9(this,n)&&mkn(this);return this.b},g.zj=function(){var t;if(!this.e){t=null;try{t=Nb(this)}catch(n){if(n=ui(n),!ue(n,102))throw ne(n)}this.d=null,!!t&&(t.i&1)!=0&&(t==vu?this.d=(on(),u2):t==Hn?this.d=Ke(0):t==Em?this.d=new cS(0):t==ks?this.d=0:t==ap?this.d=qp(0):t==D3?this.d=Yk(0):t==Vc?this.d=MA(0):this.d=RA(0)),this.e=!0}return this.d},g.Ej=function(){return(this.Bb&256)!=0},g.Ik=function(t){t&&(this.D="org.eclipse.emf.common.util.AbstractEnumerator")},g.xk=function(t){Zut(this,t),this.Ik(t)},g.yk=function(t){this.C=t,this.e=!1},g.Ib=function(){var t;return(this.Db&64)!=0?yF(this):(t=new Ol(yF(this)),t.a+=" (serializable: ",a0(t,(this.Bb&256)!=0),t.a+=")",t.a)},g.c=!1,g.d=null,g.e=!1,D(Qt,"EDataTypeImpl",566),M(457,566,{105:1,92:1,90:1,138:1,148:1,834:1,671:1,147:1,191:1,56:1,108:1,49:1,97:1,351:1,457:1,150:1,114:1,115:1,676:1},hXe),g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return this.D!=null?this.D:this.B;case 3:return Nb(this);case 4:return tpe(this);case 5:return this.F;case 6:return n?Vu(this):vk(this);case 7:return!this.A&&(this.A=new Nc(lc,this,7)),this.A;case 8:return on(),(this.Bb&256)!=0;case 9:return!this.a&&(this.a=new Ye(qd,this,9,5)),this.a}return gl(this,t-En((Bt(),rp)),Ht((a=h(tn(this,16),26),a||rp),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 6:return this.Cb&&(s=(u=this.Db>>16,u>=0?XX(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,6,s);case 9:return!this.a&&(this.a=new Ye(qd,this,9,5)),Xo(this.a,t,s)}return p=h(Ht((a=h(tn(this,16),26),a||(Bt(),rp)),n),66),p.Nj().Qj(this,ic(this),n-En((Bt(),rp)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 6:return Ku(this,null,6,s);case 7:return!this.A&&(this.A=new Nc(lc,this,7)),qs(this.A,t,s);case 9:return!this.a&&(this.a=new Ye(qd,this,9,5)),qs(this.a,t,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),rp)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),rp)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return this.D!=null&&this.D==this.F;case 3:return!!Nb(this);case 4:return!!tpe(this);case 5:return this.F!=null&&this.F!=this.D&&this.F!=this.B;case 6:return!!vk(this);case 7:return!!this.A&&this.A.i!=0;case 8:return(this.Bb&256)==0;case 9:return!!this.a&&this.a.i!=0}return fl(this,t-En((Bt(),rp)),Ht((n=h(tn(this,16),26),n||rp),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:cB(this,Jn(n));return;case 2:CY(this,Jn(n));return;case 5:vx(this,Jn(n));return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A),!this.A&&(this.A=new Nc(lc,this,7)),mi(this.A,h(n,14));return;case 8:gF(this,ft(lt(n)));return;case 9:!this.a&&(this.a=new Ye(qd,this,9,5)),jn(this.a),!this.a&&(this.a=new Ye(qd,this,9,5)),mi(this.a,h(n,14));return}ml(this,t-En((Bt(),rp)),Ht((s=h(tn(this,16),26),s||rp),t),n)},g.zh=function(){return Bt(),rp},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:ue(this.Cb,179)&&(h(this.Cb,179).tb=null),Ko(this,null);return;case 2:Vk(this,null),Ik(this,this.D);return;case 5:vx(this,null);return;case 7:!this.A&&(this.A=new Nc(lc,this,7)),jn(this.A);return;case 8:gF(this,!0);return;case 9:!this.a&&(this.a=new Ye(qd,this,9,5)),jn(this.a);return}vl(this,t-En((Bt(),rp)),Ht((n=h(tn(this,16),26),n||rp),t))},g.Gh=function(){var t,n;if(this.a)for(t=0,n=this.a.i;t<n;++t)zS(he(this.a,t));v0((jc(),Ps),this).ne(),this.Bb|=1},g.zj=function(){return tpe(this)},g.wj=function(t){return t!=null},g.Ik=function(t){},D(Qt,"EEnumImpl",457),M(573,438,{105:1,92:1,90:1,1940:1,678:1,147:1,191:1,56:1,108:1,49:1,97:1,573:1,150:1,114:1,115:1},PKe),g.ne=function(){return this.zb},g.Qg=function(t){return S1t(this,t)},g._g=function(t,n,s){var a,u;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return Ke(this.d);case 3:return this.b?this.b:this.a;case 4:return u=this.c,u==null?this.zb:u;case 5:return this.Db>>16==5?h(this.Cb,671):null}return gl(this,t-En((Bt(),R0)),Ht((a=h(tn(this,16),26),a||R0),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 5:return this.Cb&&(s=(u=this.Db>>16,u>=0?S1t(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,5,s)}return p=h(Ht((a=h(tn(this,16),26),a||(Bt(),R0)),n),66),p.Nj().Qj(this,ic(this),n-En((Bt(),R0)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 5:return Ku(this,null,5,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),R0)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),R0)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return this.d!=0;case 3:return!!this.b;case 4:return this.c!=null;case 5:return!!(this.Db>>16==5&&h(this.Cb,671))}return fl(this,t-En((Bt(),R0)),Ht((n=h(tn(this,16),26),n||R0),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:Ko(this,Jn(n));return;case 2:IK(this,h(n,19).a);return;case 3:x0t(this,h(n,1940));return;case 4:NK(this,Jn(n));return}ml(this,t-En((Bt(),R0)),Ht((s=h(tn(this,16),26),s||R0),t),n)},g.zh=function(){return Bt(),R0},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:Ko(this,null);return;case 2:IK(this,0);return;case 3:x0t(this,null);return;case 4:NK(this,null);return}vl(this,t-En((Bt(),R0)),Ht((n=h(tn(this,16),26),n||R0),t))},g.Ib=function(){var t;return t=this.c,t==null?this.zb:t},g.b=null,g.c=null,g.d=0,D(Qt,"EEnumLiteralImpl",573);var HPn=fi(Qt,"EFactoryImpl/InternalEDateTimeFormat");M(489,1,{2015:1},fS),D(Qt,"EFactoryImpl/1ClientInternalEDateTimeFormat",489),M(241,115,{105:1,92:1,90:1,87:1,56:1,108:1,49:1,97:1,241:1,114:1,115:1},V2),g.Sg=function(t,n,s){var a;return s=Ku(this,t,n,s),!!this.e&&ue(t,170)&&(a=eR(this,this.e),a!=this.c&&(s=wx(this,a,s))),s},g._g=function(t,n,s){var a;switch(t){case 0:return this.f;case 1:return!this.d&&(this.d=new Gi(pa,this,1)),this.d;case 2:return n?hR(this):this.c;case 3:return this.b;case 4:return this.e;case 5:return n?zX(this):this.a}return gl(this,t-En((Bt(),gv)),Ht((a=h(tn(this,16),26),a||gv),t),n,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return Tht(this,null,s);case 1:return!this.d&&(this.d=new Gi(pa,this,1)),qs(this.d,t,s);case 3:return _ht(this,null,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),gv)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),gv)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.f;case 1:return!!this.d&&this.d.i!=0;case 2:return!!this.c;case 3:return!!this.b;case 4:return!!this.e;case 5:return!!this.a}return fl(this,t-En((Bt(),gv)),Ht((n=h(tn(this,16),26),n||gv),t))},g.sh=function(t,n){var s;switch(t){case 0:U1t(this,h(n,87));return;case 1:!this.d&&(this.d=new Gi(pa,this,1)),jn(this.d),!this.d&&(this.d=new Gi(pa,this,1)),mi(this.d,h(n,14));return;case 3:N2e(this,h(n,87));return;case 4:Q2e(this,h(n,836));return;case 5:Sk(this,h(n,138));return}ml(this,t-En((Bt(),gv)),Ht((s=h(tn(this,16),26),s||gv),t),n)},g.zh=function(){return Bt(),gv},g.Bh=function(t){var n;switch(t){case 0:U1t(this,null);return;case 1:!this.d&&(this.d=new Gi(pa,this,1)),jn(this.d);return;case 3:N2e(this,null);return;case 4:Q2e(this,null);return;case 5:Sk(this,null);return}vl(this,t-En((Bt(),gv)),Ht((n=h(tn(this,16),26),n||gv),t))},g.Ib=function(){var t;return t=new ju(_h(this)),t.a+=" (expression: ",QQ(this,t),t.a+=")",t.a};var yEe;D(Qt,"EGenericTypeImpl",241),M(1969,1964,wj),g.Xh=function(t,n){mtt(this,t,n)},g.lk=function(t,n){return mtt(this,this.gc(),t),n},g.pi=function(t){return Zh(this.Gi(),t)},g.Zh=function(){return this.$h()},g.Gi=function(){return new EKe(this)},g.$h=function(){return this._h(0)},g._h=function(t){return this.Gi().Zc(t)},g.mk=function(t,n){return Ew(this,t,!0),n},g.ii=function(t,n){var s,a;return a=ZX(this,n),s=this.Zc(t),s.Rb(a),a},g.ji=function(t,n){var s;Ew(this,n,!0),s=this.Zc(t),s.Rb(n)},D(ri,"AbstractSequentialInternalEList",1969),M(486,1969,wj,$S),g.pi=function(t){return Zh(this.Gi(),t)},g.Zh=function(){return this.b==null?(o0(),o0(),vL):this.Jk()},g.Gi=function(){return new zJe(this.a,this.b)},g.$h=function(){return this.b==null?(o0(),o0(),vL):this.Jk()},g._h=function(t){var n,s;if(this.b==null){if(t<0||t>1)throw ne(new Ea(n_+t+", size=0"));return o0(),o0(),vL}for(s=this.Jk(),n=0;n<t;++n)iF(s);return s},g.dc=function(){var t,n,s,a,u,p;if(this.b!=null){for(s=0;s<this.b.length;++s)if(t=this.b[s],!this.Mk()||this.a.mh(t)){if(p=this.a.bh(t,!1),ia(),h(t,66).Oj()){for(n=h(p,153),a=0,u=n.gc();a<u;++a)if(Prt(n.il(a))&&n.jl(a)!=null)return!1}else if(t.$j()){if(!h(p,14).dc())return!1}else if(p!=null)return!1}}return!0},g.Kc=function(){return xge(this)},g.Zc=function(t){var n,s;if(this.b==null){if(t!=0)throw ne(new Ea(n_+t+", size=0"));return o0(),o0(),vL}for(s=this.Lk()?this.Kk():this.Jk(),n=0;n<t;++n)iF(s);return s},g.ii=function(t,n){throw ne(new Xn)},g.ji=function(t,n){throw ne(new Xn)},g.Jk=function(){return new uP(this.a,this.b)},g.Kk=function(){return new n1e(this.a,this.b)},g.Lk=function(){return!0},g.gc=function(){var t,n,s,a,u,p,v;if(u=0,this.b!=null){for(s=0;s<this.b.length;++s)if(t=this.b[s],!this.Mk()||this.a.mh(t))if(v=this.a.bh(t,!1),ia(),h(t,66).Oj())for(n=h(v,153),a=0,p=n.gc();a<p;++a)Prt(n.il(a))&&n.jl(a)!=null&&++u;else t.$j()?u+=h(v,14).gc():v!=null&&++u}return u},g.Mk=function(){return!0};var sie;D(ri,"EContentsEList",486),M(1156,486,wj,stt),g.Jk=function(){return new att(this.a,this.b)},g.Kk=function(){return new ott(this.a,this.b)},g.Mk=function(){return!1},D(Qt,"ENamedElementImpl/1",1156),M(279,1,mj,uP),g.Nb=function(t){Ls(this,t)},g.Rb=function(t){throw ne(new Xn)},g.Nk=function(t){if(this.g!=0||!!this.e)throw ne(new ja("Iterator already in use or already filtered"));this.e=t},g.Ob=function(){var t,n,s,a,u,p;switch(this.g){case 3:case 2:return!0;case 1:return!1;case-3:this.p?this.p.Pb():++this.n;default:if(!this.k||(this.p?!Bdt(this,this.p):!E0t(this))){for(;this.d<this.c.length;)if(n=this.c[this.d++],(!this.e||n.Gj()!=E8||n.aj()!=0)&&(!this.Mk()||this.b.mh(n))){if(p=this.b.bh(n,this.Lk()),this.f=(ia(),h(n,66).Oj()),this.f||n.$j()){if(this.Lk()?(a=h(p,15),this.k=a):(a=h(p,69),this.k=this.j=a),ue(this.k,54)?(this.p=null,this.o=this.k.gc(),this.n=0):this.p=this.j?this.j.$h():this.k.Yc(),this.p?Bdt(this,this.p):E0t(this))return u=this.p?this.p.Pb():this.j?this.j.pi(this.n++):this.k.Xb(this.n++),this.f?(t=h(u,72),t.ak(),s=t.dd(),this.i=s):(s=u,this.i=s),this.g=3,!0}else if(p!=null)return this.k=null,this.p=null,s=p,this.i=s,this.g=2,!0}return this.k=null,this.p=null,this.f=!1,this.g=1,!1}else return u=this.p?this.p.Pb():this.j?this.j.pi(this.n++):this.k.Xb(this.n++),this.f?(t=h(u,72),t.ak(),s=t.dd(),this.i=s):(s=u,this.i=s),this.g=3,!0}},g.Sb=function(){var t,n,s,a,u,p;switch(this.g){case-3:case-2:return!0;case-1:return!1;case 3:this.p?this.p.Ub():--this.n;default:if(!this.k||(this.p?!Fdt(this,this.p):!Kdt(this))){for(;this.d>0;)if(n=this.c[--this.d],(!this.e||n.Gj()!=E8||n.aj()!=0)&&(!this.Mk()||this.b.mh(n))){if(p=this.b.bh(n,this.Lk()),this.f=(ia(),h(n,66).Oj()),this.f||n.$j()){if(this.Lk()?(a=h(p,15),this.k=a):(a=h(p,69),this.k=this.j=a),ue(this.k,54)?(this.o=this.k.gc(),this.n=this.o):this.p=this.j?this.j._h(this.k.gc()):this.k.Zc(this.k.gc()),this.p?Fdt(this,this.p):Kdt(this))return u=this.p?this.p.Ub():this.j?this.j.pi(--this.n):this.k.Xb(--this.n),this.f?(t=h(u,72),t.ak(),s=t.dd(),this.i=s):(s=u,this.i=s),this.g=-3,!0}else if(p!=null)return this.k=null,this.p=null,s=p,this.i=s,this.g=-2,!0}return this.k=null,this.p=null,this.g=-1,!1}else return u=this.p?this.p.Ub():this.j?this.j.pi(--this.n):this.k.Xb(--this.n),this.f?(t=h(u,72),t.ak(),s=t.dd(),this.i=s):(s=u,this.i=s),this.g=-3,!0}},g.Pb=function(){return iF(this)},g.Tb=function(){return this.a},g.Ub=function(){var t;if(this.g<-1||this.Sb())return--this.a,this.g=0,t=this.i,this.Sb(),t;throw ne(new lo)},g.Vb=function(){return this.a-1},g.Qb=function(){throw ne(new Xn)},g.Lk=function(){return!1},g.Wb=function(t){throw ne(new Xn)},g.Mk=function(){return!0},g.a=0,g.d=0,g.f=!1,g.g=0,g.n=0,g.o=0;var vL;D(ri,"EContentsEList/FeatureIteratorImpl",279),M(697,279,mj,n1e),g.Lk=function(){return!0},D(ri,"EContentsEList/ResolvingFeatureIteratorImpl",697),M(1157,697,mj,ott),g.Mk=function(){return!1},D(Qt,"ENamedElementImpl/1/1",1157),M(1158,279,mj,att),g.Mk=function(){return!1},D(Qt,"ENamedElementImpl/1/2",1158),M(36,143,rD,gw,sK,fs,yK,Sd,kh,age,Pst,oge,Bst,L0e,Fst,lge,Rst,I0e,jst,cge,$st,LE,yA,FW,uge,Hst,O0e,zst),g._i=function(){return V0e(this)},g.gj=function(){var t;return t=V0e(this),t?t.zj():null},g.yi=function(t){return this.b==-1&&!!this.a&&(this.b=this.c.Xg(this.a.aj(),this.a.Gj())),this.c.Og(this.b,t)},g.Ai=function(){return this.c},g.hj=function(){var t;return t=V0e(this),t?t.Kj():!1},g.b=-1,D(Qt,"ENotificationImpl",36),M(399,284,{105:1,92:1,90:1,147:1,191:1,56:1,59:1,108:1,472:1,49:1,97:1,150:1,399:1,284:1,114:1,115:1},Sq),g.Qg=function(t){return D1t(this,t)},g._g=function(t,n,s){var a,u,p;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return on(),(this.Bb&256)!=0;case 3:return on(),(this.Bb&512)!=0;case 4:return Ke(this.s);case 5:return Ke(this.t);case 6:return on(),p=this.t,p>1||p==-1;case 7:return on(),u=this.s,u>=1;case 8:return n?Rl(this):this.r;case 9:return this.q;case 10:return this.Db>>16==10?h(this.Cb,26):null;case 11:return!this.d&&(this.d=new Nc(lc,this,11)),this.d;case 12:return!this.c&&(this.c=new Ye(dv,this,12,10)),this.c;case 13:return!this.a&&(this.a=new qS(this,this)),this.a;case 14:return cu(this)}return gl(this,t-En((Bt(),j0)),Ht((a=h(tn(this,16),26),a||j0),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 10:return this.Cb&&(s=(u=this.Db>>16,u>=0?D1t(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,10,s);case 12:return!this.c&&(this.c=new Ye(dv,this,12,10)),Xo(this.c,t,s)}return p=h(Ht((a=h(tn(this,16),26),a||(Bt(),j0)),n),66),p.Nj().Qj(this,ic(this),n-En((Bt(),j0)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 9:return EW(this,s);case 10:return Ku(this,null,10,s);case 11:return!this.d&&(this.d=new Nc(lc,this,11)),qs(this.d,t,s);case 12:return!this.c&&(this.c=new Ye(dv,this,12,10)),qs(this.c,t,s);case 14:return qs(cu(this),t,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),j0)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),j0)),t,s)},g.lh=function(t){var n,s,a;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return(this.Bb&256)==0;case 3:return(this.Bb&512)==0;case 4:return this.s!=0;case 5:return this.t!=1;case 6:return a=this.t,a>1||a==-1;case 7:return s=this.s,s>=1;case 8:return!!this.r&&!this.q.e&&hb(this.q).i==0;case 9:return!!this.q&&!(!!this.r&&!this.q.e&&hb(this.q).i==0);case 10:return!!(this.Db>>16==10&&h(this.Cb,26));case 11:return!!this.d&&this.d.i!=0;case 12:return!!this.c&&this.c.i!=0;case 13:return!!this.a&&cu(this.a.a).i!=0&&!(!!this.b&&$X(this.b));case 14:return!!this.b&&$X(this.b)}return fl(this,t-En((Bt(),j0)),Ht((n=h(tn(this,16),26),n||j0),t))},g.sh=function(t,n){var s,a;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:Ko(this,Jn(n));return;case 2:w0(this,ft(lt(n)));return;case 3:m0(this,ft(lt(n)));return;case 4:p0(this,h(n,19).a);return;case 5:yw(this,h(n,19).a);return;case 8:Vp(this,h(n,138));return;case 9:a=Df(this,h(n,87),null),a&&a.Fi();return;case 11:!this.d&&(this.d=new Nc(lc,this,11)),jn(this.d),!this.d&&(this.d=new Nc(lc,this,11)),mi(this.d,h(n,14));return;case 12:!this.c&&(this.c=new Ye(dv,this,12,10)),jn(this.c),!this.c&&(this.c=new Ye(dv,this,12,10)),mi(this.c,h(n,14));return;case 13:!this.a&&(this.a=new qS(this,this)),L9(this.a),!this.a&&(this.a=new qS(this,this)),mi(this.a,h(n,14));return;case 14:jn(cu(this)),mi(cu(this),h(n,14));return}ml(this,t-En((Bt(),j0)),Ht((s=h(tn(this,16),26),s||j0),t),n)},g.zh=function(){return Bt(),j0},g.Bh=function(t){var n,s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:Ko(this,null);return;case 2:w0(this,!0);return;case 3:m0(this,!0);return;case 4:p0(this,0);return;case 5:yw(this,1);return;case 8:Vp(this,null);return;case 9:s=Df(this,null,null),s&&s.Fi();return;case 11:!this.d&&(this.d=new Nc(lc,this,11)),jn(this.d);return;case 12:!this.c&&(this.c=new Ye(dv,this,12,10)),jn(this.c);return;case 13:this.a&&L9(this.a);return;case 14:this.b&&jn(this.b);return}vl(this,t-En((Bt(),j0)),Ht((n=h(tn(this,16),26),n||j0),t))},g.Gh=function(){var t,n;if(this.c)for(t=0,n=this.c.i;t<n;++t)zS(he(this.c,t));Rl(this),this.Bb|=1},D(Qt,"EOperationImpl",399),M(505,742,e3e,qS),g.Hi=function(t,n){B0n(this,t,h(n,138))},g.Ii=function(t){Idn(this,h(t,138))},g.Oi=function(t){var n,s;return n=h(he(cu(this.a),t),87),s=n.c,s||(Bt(),zf)},g.Ti=function(t){var n,s;return n=h(Ow(cu(this.a),t),87),s=n.c,s||(Bt(),zf)},g.Ui=function(t,n){return Oxn(this,t,h(n,138))},g.ai=function(){return!1},g.Zi=function(t,n,s,a,u){return null},g.Ji=function(){return new yKe(this)},g.Ki=function(){jn(cu(this.a))},g.Li=function(t){return Rht(this,t)},g.Mi=function(t){var n,s;for(s=t.Kc();s.Ob();)if(n=s.Pb(),!Rht(this,n))return!1;return!0},g.Ni=function(t){var n,s,a;if(ue(t,15)&&(a=h(t,15),a.gc()==cu(this.a).i)){for(n=a.Kc(),s=new Tn(this);n.Ob();)if(Me(n.Pb())!==Me(On(s)))return!1;return!0}return!1},g.Pi=function(){var t,n,s,a,u;for(s=1,n=new Tn(cu(this.a));n.e!=n.i.gc();)t=h(On(n),87),a=(u=t.c,u||(Bt(),zf)),s=31*s+(a?ii(a):0);return s},g.Qi=function(t){var n,s,a,u;for(a=0,s=new Tn(cu(this.a));s.e!=s.i.gc();){if(n=h(On(s),87),Me(t)===Me((u=n.c,u||(Bt(),zf))))return a;++a}return-1},g.Ri=function(){return cu(this.a).i==0},g.Si=function(){return null},g.Vi=function(){return cu(this.a).i},g.Wi=function(){var t,n,s,a,u,p;for(p=cu(this.a).i,u=pe(yn,st,1,p,5,1),s=0,n=new Tn(cu(this.a));n.e!=n.i.gc();)t=h(On(n),87),u[s++]=(a=t.c,a||(Bt(),zf));return u},g.Xi=function(t){var n,s,a,u,p,v,m;for(m=cu(this.a).i,t.length<m&&(u=LA(uu(t).c,m),t=u),t.length>m&&pi(t,m,null),a=0,s=new Tn(cu(this.a));s.e!=s.i.gc();)n=h(On(s),87),p=(v=n.c,v||(Bt(),zf)),pi(t,a++,p);return t},g.Yi=function(){var t,n,s,a,u;for(u=new s0,u.a+="[",t=cu(this.a),n=0,a=cu(this.a).i;n<a;)va(u,mE((s=h(he(t,n),87).c,s||(Bt(),zf)))),++n<a&&(u.a+=Js);return u.a+="]",u.a},g.$i=function(t){},g.aj=function(){return 13},g.Bk=function(){return!0},g.bj=function(){return!1},g.Ck=function(){return!1},g.Dk=function(){return!1},g.Ek=function(){return!0},g.rk=function(){return!1},g.Fk=function(){return!0},g.wj=function(t){return ue(t,138)},g.fj=function(){return Bmn(this.a)},g.hi=function(){return!0},g.ni=function(){return!0},D(Qt,"EOperationImpl/1",505),M(1340,1964,Ex,yKe),g.Zc=function(t){return IA(this.a,t)},g.gc=function(){return cu(this.a.a).i},D(Qt,"EOperationImpl/1/1",1340),M(1341,545,Va,crt),g.ii=function(t,n){var s,a;return s=h(a9(this,t,n),87),Tu(this.e)&&$6(this,new yA(this.a,7,(Bt(),U9t),Ke(n),(a=s.c,a||zf),t)),s},g.jj=function(t,n){return K6n(this,h(t,87),n)},g.kj=function(t,n){return X6n(this,h(t,87),n)},g.lj=function(t,n,s){return Kxn(this,h(t,87),h(n,87),s)},g.Zi=function(t,n,s,a,u){switch(t){case 3:return RE(this,t,n,s,a,this.i>1);case 5:return RE(this,t,n,s,a,this.i-h(s,15).gc()>0);default:return new Sd(this.e,t,this.c,n,s,a,!0)}},g.ij=function(){return!0},g.fj=function(){return $X(this)},g.Xj=function(){jn(this)},D(Qt,"EOperationImpl/2",1341),M(498,1,{1938:1,498:1},xJe),D(Qt,"EPackageImpl/1",498),M(16,85,Va,Ye),g.zk=function(){return this.d},g.Ak=function(){return this.b},g.Dk=function(){return!0},g.b=0,D(ri,"EObjectContainmentWithInverseEList",16),M(353,16,Va,B4),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectContainmentWithInverseEList/Resolving",353),M(298,353,Va,cw),g.ci=function(){this.a.tb=null},D(Qt,"EPackageImpl/2",298),M(1228,1,{},Dhn),D(Qt,"EPackageImpl/3",1228),M(718,43,Vy,ihe),g._b=function(t){return ws(t)?RW(this,t):!!Ia(this.f,t)},D(Qt,"EPackageRegistryImpl",718),M(509,284,{105:1,92:1,90:1,147:1,191:1,56:1,2017:1,108:1,472:1,49:1,97:1,150:1,509:1,284:1,114:1,115:1},Aq),g.Qg=function(t){return L1t(this,t)},g._g=function(t,n,s){var a,u,p;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return on(),(this.Bb&256)!=0;case 3:return on(),(this.Bb&512)!=0;case 4:return Ke(this.s);case 5:return Ke(this.t);case 6:return on(),p=this.t,p>1||p==-1;case 7:return on(),u=this.s,u>=1;case 8:return n?Rl(this):this.r;case 9:return this.q;case 10:return this.Db>>16==10?h(this.Cb,59):null}return gl(this,t-En((Bt(),km)),Ht((a=h(tn(this,16),26),a||km),t),n,s)},g.hh=function(t,n,s){var a,u,p;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),Xo(this.Ab,t,s);case 10:return this.Cb&&(s=(u=this.Db>>16,u>=0?L1t(this,s):this.Cb.ih(this,-1-u,null,s))),Ku(this,t,10,s)}return p=h(Ht((a=h(tn(this,16),26),a||(Bt(),km)),n),66),p.Nj().Qj(this,ic(this),n-En((Bt(),km)),t,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 9:return EW(this,s);case 10:return Ku(this,null,10,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),km)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),km)),t,s)},g.lh=function(t){var n,s,a;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return(this.Bb&256)==0;case 3:return(this.Bb&512)==0;case 4:return this.s!=0;case 5:return this.t!=1;case 6:return a=this.t,a>1||a==-1;case 7:return s=this.s,s>=1;case 8:return!!this.r&&!this.q.e&&hb(this.q).i==0;case 9:return!!this.q&&!(!!this.r&&!this.q.e&&hb(this.q).i==0);case 10:return!!(this.Db>>16==10&&h(this.Cb,59))}return fl(this,t-En((Bt(),km)),Ht((n=h(tn(this,16),26),n||km),t))},g.zh=function(){return Bt(),km},D(Qt,"EParameterImpl",509),M(99,449,{105:1,92:1,90:1,147:1,191:1,56:1,18:1,170:1,66:1,108:1,472:1,49:1,97:1,150:1,99:1,449:1,284:1,114:1,115:1,677:1},o1e),g._g=function(t,n,s){var a,u,p,v;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return on(),(this.Bb&256)!=0;case 3:return on(),(this.Bb&512)!=0;case 4:return Ke(this.s);case 5:return Ke(this.t);case 6:return on(),v=this.t,v>1||v==-1;case 7:return on(),u=this.s,u>=1;case 8:return n?Rl(this):this.r;case 9:return this.q;case 10:return on(),(this.Bb&Ch)!=0;case 11:return on(),(this.Bb&Ww)!=0;case 12:return on(),(this.Bb&$w)!=0;case 13:return this.j;case 14:return hx(this);case 15:return on(),(this.Bb&zc)!=0;case 16:return on(),(this.Bb&a1)!=0;case 17:return hw(this);case 18:return on(),(this.Bb&go)!=0;case 19:return on(),p=aa(this),!!p&&(p.Bb&go)!=0;case 20:return on(),(this.Bb&ea)!=0;case 21:return n?aa(this):this.b;case 22:return n?Vge(this):lot(this);case 23:return!this.a&&(this.a=new ly(wm,this,23)),this.a}return gl(this,t-En((Bt(),C3)),Ht((a=h(tn(this,16),26),a||C3),t),n,s)},g.lh=function(t){var n,s,a,u;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return(this.Bb&256)==0;case 3:return(this.Bb&512)==0;case 4:return this.s!=0;case 5:return this.t!=1;case 6:return u=this.t,u>1||u==-1;case 7:return s=this.s,s>=1;case 8:return!!this.r&&!this.q.e&&hb(this.q).i==0;case 9:return!!this.q&&!(!!this.r&&!this.q.e&&hb(this.q).i==0);case 10:return(this.Bb&Ch)==0;case 11:return(this.Bb&Ww)!=0;case 12:return(this.Bb&$w)!=0;case 13:return this.j!=null;case 14:return hx(this)!=null;case 15:return(this.Bb&zc)!=0;case 16:return(this.Bb&a1)!=0;case 17:return!!hw(this);case 18:return(this.Bb&go)!=0;case 19:return a=aa(this),!!a&&(a.Bb&go)!=0;case 20:return(this.Bb&ea)==0;case 21:return!!this.b;case 22:return!!lot(this);case 23:return!!this.a&&this.a.i!=0}return fl(this,t-En((Bt(),C3)),Ht((n=h(tn(this,16),26),n||C3),t))},g.sh=function(t,n){var s,a;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:QW(this,Jn(n));return;case 2:w0(this,ft(lt(n)));return;case 3:m0(this,ft(lt(n)));return;case 4:p0(this,h(n,19).a);return;case 5:yw(this,h(n,19).a);return;case 8:Vp(this,h(n,138));return;case 9:a=Df(this,h(n,87),null),a&&a.Fi();return;case 10:Wk(this,ft(lt(n)));return;case 11:Qk(this,ft(lt(n)));return;case 12:Kk(this,ft(lt(n)));return;case 13:pfe(this,Jn(n));return;case 15:Xk(this,ft(lt(n)));return;case 16:Zk(this,ft(lt(n)));return;case 18:Gwn(this,ft(lt(n)));return;case 20:kpe(this,ft(lt(n)));return;case 21:yge(this,h(n,18));return;case 23:!this.a&&(this.a=new ly(wm,this,23)),jn(this.a),!this.a&&(this.a=new ly(wm,this,23)),mi(this.a,h(n,14));return}ml(this,t-En((Bt(),C3)),Ht((s=h(tn(this,16),26),s||C3),t),n)},g.zh=function(){return Bt(),C3},g.Bh=function(t){var n,s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:ue(this.Cb,88)&&Lw(ou(h(this.Cb,88)),4),Ko(this,null);return;case 2:w0(this,!0);return;case 3:m0(this,!0);return;case 4:p0(this,0);return;case 5:yw(this,1);return;case 8:Vp(this,null);return;case 9:s=Df(this,null,null),s&&s.Fi();return;case 10:Wk(this,!0);return;case 11:Qk(this,!1);return;case 12:Kk(this,!1);return;case 13:this.i=null,WB(this,null);return;case 15:Xk(this,!1);return;case 16:Zk(this,!1);return;case 18:ype(this,!1),ue(this.Cb,88)&&Lw(ou(h(this.Cb,88)),2);return;case 20:kpe(this,!0);return;case 21:yge(this,null);return;case 23:!this.a&&(this.a=new ly(wm,this,23)),jn(this.a);return}vl(this,t-En((Bt(),C3)),Ht((n=h(tn(this,16),26),n||C3),t))},g.Gh=function(){Vge(this),gk(Sa((jc(),Ps),this)),Rl(this),this.Bb|=1},g.Lj=function(){return aa(this)},g.qk=function(){var t;return t=aa(this),!!t&&(t.Bb&go)!=0},g.rk=function(){return(this.Bb&go)!=0},g.sk=function(){return(this.Bb&ea)!=0},g.nk=function(t,n){return this.c=null,dpe(this,t,n)},g.Ib=function(){var t;return(this.Db&64)!=0?cR(this):(t=new Ol(cR(this)),t.a+=" (containment: ",a0(t,(this.Bb&go)!=0),t.a+=", resolveProxies: ",a0(t,(this.Bb&ea)!=0),t.a+=")",t.a)},D(Qt,"EReferenceImpl",99),M(548,115,{105:1,42:1,92:1,90:1,133:1,56:1,108:1,49:1,97:1,548:1,114:1,115:1},Uze),g.Fb=function(t){return this===t},g.cd=function(){return this.b},g.dd=function(){return this.c},g.Hb=function(){return rb(this)},g.Uh=function(t){W2n(this,Jn(t))},g.ed=function(t){return B2n(this,Jn(t))},g._g=function(t,n,s){var a;switch(t){case 0:return this.b;case 1:return this.c}return gl(this,t-En((Bt(),na)),Ht((a=h(tn(this,16),26),a||na),t),n,s)},g.lh=function(t){var n;switch(t){case 0:return this.b!=null;case 1:return this.c!=null}return fl(this,t-En((Bt(),na)),Ht((n=h(tn(this,16),26),n||na),t))},g.sh=function(t,n){var s;switch(t){case 0:K2n(this,Jn(n));return;case 1:vge(this,Jn(n));return}ml(this,t-En((Bt(),na)),Ht((s=h(tn(this,16),26),s||na),t),n)},g.zh=function(){return Bt(),na},g.Bh=function(t){var n;switch(t){case 0:bge(this,null);return;case 1:vge(this,null);return}vl(this,t-En((Bt(),na)),Ht((n=h(tn(this,16),26),n||na),t))},g.Sh=function(){var t;return this.a==-1&&(t=this.b,this.a=t==null?0:k0(t)),this.a},g.Th=function(t){this.a=t},g.Ib=function(){var t;return(this.Db&64)!=0?_h(this):(t=new Ol(_h(this)),t.a+=" (key: ",va(t,this.b),t.a+=", value: ",va(t,this.c),t.a+=")",t.a)},g.a=-1,g.b=null,g.c=null;var uo=D(Qt,"EStringToStringMapEntryImpl",548),Q9t=fi(ri,"FeatureMap/Entry/Internal");M(565,1,yj),g.Ok=function(t){return this.Pk(h(t,49))},g.Pk=function(t){return this.Ok(t)},g.Fb=function(t){var n,s;return this===t?!0:ue(t,72)?(n=h(t,72),n.ak()==this.c?(s=this.dd(),s==null?n.dd()==null:Nr(s,n.dd())):!1):!1},g.ak=function(){return this.c},g.Hb=function(){var t;return t=this.dd(),ii(this.c)^(t==null?0:ii(t))},g.Ib=function(){var t,n;return t=this.c,n=Vu(t.Hj()).Ph(),t.ne(),(n!=null&&n.length!=0?n+":"+t.ne():t.ne())+"="+this.dd()},D(Qt,"EStructuralFeatureImpl/BasicFeatureMapEntry",565),M(776,565,yj,b1e),g.Pk=function(t){return new b1e(this.c,t)},g.dd=function(){return this.a},g.Qk=function(t,n,s){return h5n(this,t,this.a,n,s)},g.Rk=function(t,n,s){return f5n(this,t,this.a,n,s)},D(Qt,"EStructuralFeatureImpl/ContainmentUpdatingFeatureMapEntry",776),M(1314,1,{},EJe),g.Pj=function(t,n,s,a,u){var p;return p=h(Ek(t,this.b),215),p.nl(this.a).Wj(a)},g.Qj=function(t,n,s,a,u){var p;return p=h(Ek(t,this.b),215),p.el(this.a,a,u)},g.Rj=function(t,n,s,a,u){var p;return p=h(Ek(t,this.b),215),p.fl(this.a,a,u)},g.Sj=function(t,n,s){var a;return a=h(Ek(t,this.b),215),a.nl(this.a).fj()},g.Tj=function(t,n,s,a){var u;u=h(Ek(t,this.b),215),u.nl(this.a).Wb(a)},g.Uj=function(t,n,s){return h(Ek(t,this.b),215).nl(this.a)},g.Vj=function(t,n,s){var a;a=h(Ek(t,this.b),215),a.nl(this.a).Xj()},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateFeatureMapDelegator",1314),M(89,1,{},u0,Dp,h0,Op),g.Pj=function(t,n,s,a,u){var p;if(p=n.Ch(s),p==null&&n.Dh(s,p=kR(this,t)),!u)switch(this.e){case 50:case 41:return h(p,589).sj();case 40:return h(p,215).kl()}return p},g.Qj=function(t,n,s,a,u){var p,v;return v=n.Ch(s),v==null&&n.Dh(s,v=kR(this,t)),p=h(v,69).lk(a,u),p},g.Rj=function(t,n,s,a,u){var p;return p=n.Ch(s),p!=null&&(u=h(p,69).mk(a,u)),u},g.Sj=function(t,n,s){var a;return a=n.Ch(s),a!=null&&h(a,76).fj()},g.Tj=function(t,n,s,a){var u;u=h(n.Ch(s),76),!u&&n.Dh(s,u=kR(this,t)),u.Wb(a)},g.Uj=function(t,n,s){var a,u;return u=n.Ch(s),u==null&&n.Dh(s,u=kR(this,t)),ue(u,76)?h(u,76):(a=h(n.Ch(s),15),new xKe(a))},g.Vj=function(t,n,s){var a;a=h(n.Ch(s),76),!a&&n.Dh(s,a=kR(this,t)),a.Xj()},g.b=0,g.e=0,D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateMany",89),M(504,1,{}),g.Qj=function(t,n,s,a,u){throw ne(new Xn)},g.Rj=function(t,n,s,a,u){throw ne(new Xn)},g.Uj=function(t,n,s){return new yit(this,t,n,s)};var td;D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingle",504),M(1331,1,Iee,yit),g.Wj=function(t){return this.a.Pj(this.c,this.d,this.b,t,!0)},g.fj=function(){return this.a.Sj(this.c,this.d,this.b)},g.Wb=function(t){this.a.Tj(this.c,this.d,this.b,t)},g.Xj=function(){this.a.Vj(this.c,this.d,this.b)},g.b=0,D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingle/1",1331),M(769,504,{},Zde),g.Pj=function(t,n,s,a,u){return RQ(t,t.eh(),t.Vg())==this.b?this.sk()&&a?xQ(t):t.eh():null},g.Qj=function(t,n,s,a,u){var p,v;return t.eh()&&(u=(p=t.Vg(),p>=0?t.Qg(u):t.eh().ih(t,-1-p,null,u))),v=oi(t.Tg(),this.e),t.Sg(a,v,u)},g.Rj=function(t,n,s,a,u){var p;return p=oi(t.Tg(),this.e),t.Sg(null,p,u)},g.Sj=function(t,n,s){var a;return a=oi(t.Tg(),this.e),!!t.eh()&&t.Vg()==a},g.Tj=function(t,n,s,a){var u,p,v,m,x;if(a!=null&&!zQ(this.a,a))throw ne(new z6(kj+(ue(a,56)?I2e(h(a,56).Tg()):nge(uu(a)))+xj+this.a+"'"));if(u=t.eh(),v=oi(t.Tg(),this.e),Me(a)!==Me(u)||t.Vg()!=v&&a!=null){if(nx(t,h(a,56)))throw ne(new an(e_+t.Ib()));x=null,u&&(x=(p=t.Vg(),p>=0?t.Qg(x):t.eh().ih(t,-1-p,null,x))),m=h(a,49),m&&(x=m.gh(t,oi(m.Tg(),this.b),null,x)),x=t.Sg(m,v,x),x&&x.Fi()}else t.Lg()&&t.Mg()&&Or(t,new fs(t,1,v,a,a))},g.Vj=function(t,n,s){var a,u,p,v;a=t.eh(),a?(v=(u=t.Vg(),u>=0?t.Qg(null):t.eh().ih(t,-1-u,null,null)),p=oi(t.Tg(),this.e),v=t.Sg(null,p,v),v&&v.Fi()):t.Lg()&&t.Mg()&&Or(t,new LE(t,1,this.e,null,null))},g.sk=function(){return!1},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleContainer",769),M(1315,769,{},fnt),g.sk=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleContainerResolving",1315),M(563,504,{}),g.Pj=function(t,n,s,a,u){var p;return p=n.Ch(s),p==null?this.b:Me(p)===Me(td)?null:p},g.Sj=function(t,n,s){var a;return a=n.Ch(s),a!=null&&(Me(a)===Me(td)||!Nr(a,this.b))},g.Tj=function(t,n,s,a){var u,p;t.Lg()&&t.Mg()?(u=(p=n.Ch(s),p==null?this.b:Me(p)===Me(td)?null:p),a==null?this.c!=null?(n.Dh(s,null),a=this.b):this.b!=null?n.Dh(s,td):n.Dh(s,null):(this.Sk(a),n.Dh(s,a)),Or(t,this.d.Tk(t,1,this.e,u,a))):a==null?this.c!=null?n.Dh(s,null):this.b!=null?n.Dh(s,td):n.Dh(s,null):(this.Sk(a),n.Dh(s,a))},g.Vj=function(t,n,s){var a,u;t.Lg()&&t.Mg()?(a=(u=n.Ch(s),u==null?this.b:Me(u)===Me(td)?null:u),n.Eh(s),Or(t,this.d.Tk(t,1,this.e,a,this.b))):n.Eh(s)},g.Sk=function(t){throw ne(new IKe)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData",563),M(e3,1,{},qze),g.Tk=function(t,n,s,a,u){return new LE(t,n,s,a,u)},g.Uk=function(t,n,s,a,u,p){return new FW(t,n,s,a,u,p)};var kEe,xEe,EEe,_Ee,TEe,CEe,SEe,aie,AEe;D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator",e3),M(1332,e3,{},Yze),g.Tk=function(t,n,s,a,u){return new O0e(t,n,s,ft(lt(a)),ft(lt(u)))},g.Uk=function(t,n,s,a,u,p){return new zst(t,n,s,ft(lt(a)),ft(lt(u)),p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/1",1332),M(1333,e3,{},Wze),g.Tk=function(t,n,s,a,u){return new age(t,n,s,h(a,217).a,h(u,217).a)},g.Uk=function(t,n,s,a,u,p){return new Pst(t,n,s,h(a,217).a,h(u,217).a,p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/2",1333),M(1334,e3,{},Kze),g.Tk=function(t,n,s,a,u){return new oge(t,n,s,h(a,172).a,h(u,172).a)},g.Uk=function(t,n,s,a,u,p){return new Bst(t,n,s,h(a,172).a,h(u,172).a,p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/3",1334),M(1335,e3,{},Xze),g.Tk=function(t,n,s,a,u){return new L0e(t,n,s,Pe(Je(a)),Pe(Je(u)))},g.Uk=function(t,n,s,a,u,p){return new Fst(t,n,s,Pe(Je(a)),Pe(Je(u)),p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/4",1335),M(1336,e3,{},Qze),g.Tk=function(t,n,s,a,u){return new lge(t,n,s,h(a,155).a,h(u,155).a)},g.Uk=function(t,n,s,a,u,p){return new Rst(t,n,s,h(a,155).a,h(u,155).a,p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/5",1336),M(1337,e3,{},Zze),g.Tk=function(t,n,s,a,u){return new I0e(t,n,s,h(a,19).a,h(u,19).a)},g.Uk=function(t,n,s,a,u,p){return new jst(t,n,s,h(a,19).a,h(u,19).a,p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/6",1337),M(1338,e3,{},Jze),g.Tk=function(t,n,s,a,u){return new cge(t,n,s,h(a,162).a,h(u,162).a)},g.Uk=function(t,n,s,a,u,p){return new $st(t,n,s,h(a,162).a,h(u,162).a,p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/7",1338),M(1339,e3,{},eGe),g.Tk=function(t,n,s,a,u){return new uge(t,n,s,h(a,184).a,h(u,184).a)},g.Uk=function(t,n,s,a,u,p){return new Hst(t,n,s,h(a,184).a,h(u,184).a,p)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleData/NotificationCreator/8",1339),M(1317,563,{},kit),g.Sk=function(t){if(!this.a.wj(t))throw ne(new z6(kj+uu(t)+xj+this.a+"'"))},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleDataDynamic",1317),M(1318,563,{},urt),g.Sk=function(t){},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleDataStatic",1318),M(770,563,{}),g.Sj=function(t,n,s){var a;return a=n.Ch(s),a!=null},g.Tj=function(t,n,s,a){var u,p;t.Lg()&&t.Mg()?(u=!0,p=n.Ch(s),p==null?(u=!1,p=this.b):Me(p)===Me(td)&&(p=null),a==null?this.c!=null?(n.Dh(s,null),a=this.b):n.Dh(s,td):(this.Sk(a),n.Dh(s,a)),Or(t,this.d.Uk(t,1,this.e,p,a,!u))):a==null?this.c!=null?n.Dh(s,null):n.Dh(s,td):(this.Sk(a),n.Dh(s,a))},g.Vj=function(t,n,s){var a,u;t.Lg()&&t.Mg()?(a=!0,u=n.Ch(s),u==null?(a=!1,u=this.b):Me(u)===Me(td)&&(u=null),n.Eh(s),Or(t,this.d.Uk(t,2,this.e,u,this.b,a))):n.Eh(s)},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleDataUnsettable",770),M(1319,770,{},xit),g.Sk=function(t){if(!this.a.wj(t))throw ne(new z6(kj+uu(t)+xj+this.a+"'"))},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleDataUnsettableDynamic",1319),M(1320,770,{},lrt),g.Sk=function(t){},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleDataUnsettableStatic",1320),M(398,504,{},AP),g.Pj=function(t,n,s,a,u){var p,v,m,x,_;if(_=n.Ch(s),this.Kj()&&Me(_)===Me(td))return null;if(this.sk()&&a&&_!=null){if(m=h(_,49),m.kh()&&(x=Tg(t,m),m!=x)){if(!zQ(this.a,x))throw ne(new z6(kj+uu(x)+xj+this.a+"'"));n.Dh(s,_=x),this.rk()&&(p=h(x,49),v=m.ih(t,this.b?oi(m.Tg(),this.b):-1-oi(t.Tg(),this.e),null,null),!p.eh()&&(v=p.gh(t,this.b?oi(p.Tg(),this.b):-1-oi(t.Tg(),this.e),null,v)),v&&v.Fi()),t.Lg()&&t.Mg()&&Or(t,new LE(t,9,this.e,m,x))}return _}else return _},g.Qj=function(t,n,s,a,u){var p,v;return v=n.Ch(s),Me(v)===Me(td)&&(v=null),n.Dh(s,a),this.bj()?Me(v)!==Me(a)&&v!=null&&(p=h(v,49),u=p.ih(t,oi(p.Tg(),this.b),null,u)):this.rk()&&v!=null&&(u=h(v,49).ih(t,-1-oi(t.Tg(),this.e),null,u)),t.Lg()&&t.Mg()&&(!u&&(u=new ig(4)),u.Ei(new LE(t,1,this.e,v,a))),u},g.Rj=function(t,n,s,a,u){var p;return p=n.Ch(s),Me(p)===Me(td)&&(p=null),n.Eh(s),t.Lg()&&t.Mg()&&(!u&&(u=new ig(4)),this.Kj()?u.Ei(new LE(t,2,this.e,p,null)):u.Ei(new LE(t,1,this.e,p,null))),u},g.Sj=function(t,n,s){var a;return a=n.Ch(s),a!=null},g.Tj=function(t,n,s,a){var u,p,v,m,x;if(a!=null&&!zQ(this.a,a))throw ne(new z6(kj+(ue(a,56)?I2e(h(a,56).Tg()):nge(uu(a)))+xj+this.a+"'"));x=n.Ch(s),m=x!=null,this.Kj()&&Me(x)===Me(td)&&(x=null),v=null,this.bj()?Me(x)!==Me(a)&&(x!=null&&(u=h(x,49),v=u.ih(t,oi(u.Tg(),this.b),null,v)),a!=null&&(u=h(a,49),v=u.gh(t,oi(u.Tg(),this.b),null,v))):this.rk()&&Me(x)!==Me(a)&&(x!=null&&(v=h(x,49).ih(t,-1-oi(t.Tg(),this.e),null,v)),a!=null&&(v=h(a,49).gh(t,-1-oi(t.Tg(),this.e),null,v))),a==null&&this.Kj()?n.Dh(s,td):n.Dh(s,a),t.Lg()&&t.Mg()?(p=new FW(t,1,this.e,x,a,this.Kj()&&!m),v?(v.Ei(p),v.Fi()):Or(t,p)):v&&v.Fi()},g.Vj=function(t,n,s){var a,u,p,v,m;m=n.Ch(s),v=m!=null,this.Kj()&&Me(m)===Me(td)&&(m=null),p=null,m!=null&&(this.bj()?(a=h(m,49),p=a.ih(t,oi(a.Tg(),this.b),null,p)):this.rk()&&(p=h(m,49).ih(t,-1-oi(t.Tg(),this.e),null,p))),n.Eh(s),t.Lg()&&t.Mg()?(u=new FW(t,this.Kj()?2:1,this.e,m,null,v),p?(p.Ei(u),p.Fi()):Or(t,u)):p&&p.Fi()},g.bj=function(){return!1},g.rk=function(){return!1},g.sk=function(){return!1},g.Kj=function(){return!1},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObject",398),M(564,398,{},jY),g.rk=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainment",564),M(1323,564,{},ctt),g.sk=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainmentResolving",1323),M(772,564,{},r1e),g.Kj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainmentUnsettable",772),M(1325,772,{},utt),g.sk=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainmentUnsettableResolving",1325),M(640,564,{},JY),g.bj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainmentWithInverse",640),M(1324,640,{},dnt),g.sk=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainmentWithInverseResolving",1324),M(773,640,{},G1e),g.Kj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainmentWithInverseUnsettable",773),M(1326,773,{},gnt),g.sk=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectContainmentWithInverseUnsettableResolving",1326),M(641,398,{},i1e),g.sk=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectResolving",641),M(1327,641,{},ltt),g.Kj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectResolvingUnsettable",1327),M(774,641,{},H1e),g.bj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectResolvingWithInverse",774),M(1328,774,{},pnt),g.Kj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectResolvingWithInverseUnsettable",1328),M(1321,398,{},htt),g.Kj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectUnsettable",1321),M(771,398,{},z1e),g.bj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectWithInverse",771),M(1322,771,{},bnt),g.Kj=function(){return!0},D(Qt,"EStructuralFeatureImpl/InternalSettingDelegateSingleEObjectWithInverseUnsettable",1322),M(775,565,yj,Rde),g.Pk=function(t){return new Rde(this.a,this.c,t)},g.dd=function(){return this.b},g.Qk=function(t,n,s){return f3n(this,t,this.b,s)},g.Rk=function(t,n,s){return d3n(this,t,this.b,s)},D(Qt,"EStructuralFeatureImpl/InverseUpdatingFeatureMapEntry",775),M(1329,1,Iee,xKe),g.Wj=function(t){return this.a},g.fj=function(){return ue(this.a,95)?h(this.a,95).fj():!this.a.dc()},g.Wb=function(t){this.a.$b(),this.a.Gc(h(t,15))},g.Xj=function(){ue(this.a,95)?h(this.a,95).Xj():this.a.$b()},D(Qt,"EStructuralFeatureImpl/SettingMany",1329),M(1330,565,yj,Dat),g.Ok=function(t){return new zY((qr(),pT),this.b.Ih(this.a,t))},g.dd=function(){return null},g.Qk=function(t,n,s){return s},g.Rk=function(t,n,s){return s},D(Qt,"EStructuralFeatureImpl/SimpleContentFeatureMapEntry",1330),M(642,565,yj,zY),g.Ok=function(t){return new zY(this.c,t)},g.dd=function(){return this.a},g.Qk=function(t,n,s){return s},g.Rk=function(t,n,s){return s},D(Qt,"EStructuralFeatureImpl/SimpleFeatureMapEntry",642),M(391,497,b1,N6),g.ri=function(t){return pe(Jl,st,26,t,0,1)},g.ni=function(){return!1},D(Qt,"ESuperAdapter/1",391),M(444,438,{105:1,92:1,90:1,147:1,191:1,56:1,108:1,836:1,49:1,97:1,150:1,444:1,114:1,115:1},fq),g._g=function(t,n,s){var a;switch(t){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),this.Ab;case 1:return this.zb;case 2:return!this.a&&(this.a=new SE(this,pa,this)),this.a}return gl(this,t-En((Bt(),pv)),Ht((a=h(tn(this,16),26),a||pv),t),n,s)},g.jh=function(t,n,s){var a,u;switch(n){case 0:return!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),qs(this.Ab,t,s);case 2:return!this.a&&(this.a=new SE(this,pa,this)),qs(this.a,t,s)}return u=h(Ht((a=h(tn(this,16),26),a||(Bt(),pv)),n),66),u.Nj().Rj(this,ic(this),n-En((Bt(),pv)),t,s)},g.lh=function(t){var n;switch(t){case 0:return!!this.Ab&&this.Ab.i!=0;case 1:return this.zb!=null;case 2:return!!this.a&&this.a.i!=0}return fl(this,t-En((Bt(),pv)),Ht((n=h(tn(this,16),26),n||pv),t))},g.sh=function(t,n){var s;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab),!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),mi(this.Ab,h(n,14));return;case 1:Ko(this,Jn(n));return;case 2:!this.a&&(this.a=new SE(this,pa,this)),jn(this.a),!this.a&&(this.a=new SE(this,pa,this)),mi(this.a,h(n,14));return}ml(this,t-En((Bt(),pv)),Ht((s=h(tn(this,16),26),s||pv),t),n)},g.zh=function(){return Bt(),pv},g.Bh=function(t){var n;switch(t){case 0:!this.Ab&&(this.Ab=new Ye(dr,this,0,3)),jn(this.Ab);return;case 1:Ko(this,null);return;case 2:!this.a&&(this.a=new SE(this,pa,this)),jn(this.a);return}vl(this,t-En((Bt(),pv)),Ht((n=h(tn(this,16),26),n||pv),t))},D(Qt,"ETypeParameterImpl",444),M(445,85,Va,SE),g.cj=function(t,n){return d9n(this,h(t,87),n)},g.dj=function(t,n){return g9n(this,h(t,87),n)},D(Qt,"ETypeParameterImpl/1",445),M(634,43,Vy,Mq),g.ec=function(){return new tN(this)},D(Qt,"ETypeParameterImpl/2",634),M(556,Lf,$c,tN),g.Fc=function(t){return Ftt(this,h(t,87))},g.Gc=function(t){var n,s,a;for(a=!1,s=t.Kc();s.Ob();)n=h(s.Pb(),87),Pr(this.a,n,"")==null&&(a=!0);return a},g.$b=function(){Kc(this.a)},g.Hc=function(t){return Au(this.a,t)},g.Kc=function(){var t;return t=new Gp(new xp(this.a).a),new nN(t)},g.Mc=function(t){return Eot(this,t)},g.gc=function(){return q7(this.a)},D(Qt,"ETypeParameterImpl/2/1",556),M(557,1,ys,nN),g.Nb=function(t){Ls(this,t)},g.Pb=function(){return h(kb(this.a).cd(),87)},g.Ob=function(){return this.a.b},g.Qb=function(){ect(this.a)},D(Qt,"ETypeParameterImpl/2/1/1",557),M(1276,43,Vy,fXe),g._b=function(t){return ws(t)?RW(this,t):!!Ia(this.f,t)},g.xc=function(t){var n,s;return n=ws(t)?Fo(this,t):no(Ia(this.f,t)),ue(n,837)?(s=h(n,837),n=s._j(),Pr(this,h(t,235),n),n):n!=null?n:t==null?(qq(),J9t):null},D(Qt,"EValidatorRegistryImpl",1276),M(1313,704,{105:1,92:1,90:1,471:1,147:1,56:1,108:1,1941:1,49:1,97:1,150:1,114:1,115:1},tGe),g.Ih=function(t,n){switch(t.yj()){case 21:case 22:case 23:case 24:case 26:case 31:case 32:case 37:case 38:case 39:case 40:case 43:case 44:case 48:case 49:case 20:return n==null?null:Ga(n);case 25:return k4n(n);case 27:return R3n(n);case 28:return j3n(n);case 29:return n==null?null:bet(uT[0],h(n,199));case 41:return n==null?"":sg(h(n,290));case 42:return Ga(n);case 50:return Jn(n);default:throw ne(new an(Px+t.ne()+Wb))}},g.Jh=function(t){var n,s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe;switch(t.G==-1&&(t.G=(P=Vu(t),P?y0(P.Mh(),t):-1)),t.G){case 0:return s=new Cq,s;case 1:return n=new Jue,n;case 2:return a=new ile,a;case 4:return u=new rN,u;case 5:return p=new hXe,p;case 6:return v=new PKe,v;case 7:return m=new rle,m;case 10:return _=new qO,_;case 11:return A=new Sq,A;case 12:return I=new jit,I;case 13:return j=new Aq,j;case 14:return V=new o1e,V;case 17:return J=new Uze,J;case 18:return x=new V2,x;case 19:return oe=new fq,oe;default:throw ne(new an(bee+t.zb+Wb))}},g.Kh=function(t,n){switch(t.yj()){case 20:return n==null?null:new Che(n);case 21:return n==null?null:new hg(n);case 23:case 22:return n==null?null:Cxn(n);case 26:case 24:return n==null?null:MA(Wu(n,-128,127)<<24>>24);case 25:return FCn(n);case 27:return u7n(n);case 28:return l7n(n);case 29:return I9n(n);case 32:case 31:return n==null?null:Dw(n);case 38:case 37:return n==null?null:new qle(n);case 40:case 39:return n==null?null:Ke(Wu(n,$s,Ir));case 41:return null;case 42:return n==null,null;case 44:case 43:return n==null?null:qp(yR(n));case 49:case 48:return n==null?null:Yk(Wu(n,Ej,32767)<<16>>16);case 50:return n;default:throw ne(new an(Px+t.ne()+Wb))}},D(Qt,"EcoreFactoryImpl",1313),M(547,179,{105:1,92:1,90:1,147:1,191:1,56:1,235:1,108:1,1939:1,49:1,97:1,150:1,179:1,547:1,114:1,115:1,675:1},nit),g.gb=!1,g.hb=!1;var MEe,Z9t=!1;D(Qt,"EcorePackageImpl",547),M(1184,1,{837:1},nGe),g._j=function(){return Het(),e_t},D(Qt,"EcorePackageImpl/1",1184),M(1193,1,mr,rGe),g.wj=function(t){return ue(t,147)},g.xj=function(t){return pe(dL,st,147,t,0,1)},D(Qt,"EcorePackageImpl/10",1193),M(1194,1,mr,iGe),g.wj=function(t){return ue(t,191)},g.xj=function(t){return pe(Qre,st,191,t,0,1)},D(Qt,"EcorePackageImpl/11",1194),M(1195,1,mr,sGe),g.wj=function(t){return ue(t,56)},g.xj=function(t){return pe(Jg,st,56,t,0,1)},D(Qt,"EcorePackageImpl/12",1195),M(1196,1,mr,aGe),g.wj=function(t){return ue(t,399)},g.xj=function(t){return pe(eh,Jye,59,t,0,1)},D(Qt,"EcorePackageImpl/13",1196),M(1197,1,mr,oGe),g.wj=function(t){return ue(t,235)},g.xj=function(t){return pe(sf,st,235,t,0,1)},D(Qt,"EcorePackageImpl/14",1197),M(1198,1,mr,cGe),g.wj=function(t){return ue(t,509)},g.xj=function(t){return pe(dv,st,2017,t,0,1)},D(Qt,"EcorePackageImpl/15",1198),M(1199,1,mr,uGe),g.wj=function(t){return ue(t,99)},g.xj=function(t){return pe(mm,Jy,18,t,0,1)},D(Qt,"EcorePackageImpl/16",1199),M(1200,1,mr,lGe),g.wj=function(t){return ue(t,170)},g.xj=function(t){return pe(Ac,Jy,170,t,0,1)},D(Qt,"EcorePackageImpl/17",1200),M(1201,1,mr,hGe),g.wj=function(t){return ue(t,472)},g.xj=function(t){return pe(vm,st,472,t,0,1)},D(Qt,"EcorePackageImpl/18",1201),M(1202,1,mr,fGe),g.wj=function(t){return ue(t,548)},g.xj=function(t){return pe(uo,Jmt,548,t,0,1)},D(Qt,"EcorePackageImpl/19",1202),M(1185,1,mr,dGe),g.wj=function(t){return ue(t,322)},g.xj=function(t){return pe(wm,Jy,34,t,0,1)},D(Qt,"EcorePackageImpl/2",1185),M(1203,1,mr,gGe),g.wj=function(t){return ue(t,241)},g.xj=function(t){return pe(pa,byt,87,t,0,1)},D(Qt,"EcorePackageImpl/20",1203),M(1204,1,mr,pGe),g.wj=function(t){return ue(t,444)},g.xj=function(t){return pe(lc,st,836,t,0,1)},D(Qt,"EcorePackageImpl/21",1204),M(1205,1,mr,bGe),g.wj=function(t){return Jv(t)},g.xj=function(t){return pe(es,Re,476,t,8,1)},D(Qt,"EcorePackageImpl/22",1205),M(1206,1,mr,vGe),g.wj=function(t){return ue(t,190)},g.xj=function(t){return pe(Vc,Re,190,t,0,2)},D(Qt,"EcorePackageImpl/23",1206),M(1207,1,mr,wGe),g.wj=function(t){return ue(t,217)},g.xj=function(t){return pe(A5,Re,217,t,0,1)},D(Qt,"EcorePackageImpl/24",1207),M(1208,1,mr,mGe),g.wj=function(t){return ue(t,172)},g.xj=function(t){return pe(f_,Re,172,t,0,1)},D(Qt,"EcorePackageImpl/25",1208),M(1209,1,mr,yGe),g.wj=function(t){return ue(t,199)},g.xj=function(t){return pe(Bj,Re,199,t,0,1)},D(Qt,"EcorePackageImpl/26",1209),M(1210,1,mr,kGe),g.wj=function(t){return!1},g.xj=function(t){return pe(YEe,st,2110,t,0,1)},D(Qt,"EcorePackageImpl/27",1210),M(1211,1,mr,xGe),g.wj=function(t){return ew(t)},g.xj=function(t){return pe(Ts,Re,333,t,7,1)},D(Qt,"EcorePackageImpl/28",1211),M(1212,1,mr,EGe),g.wj=function(t){return ue(t,58)},g.xj=function(t){return pe(cEe,Hw,58,t,0,1)},D(Qt,"EcorePackageImpl/29",1212),M(1186,1,mr,_Ge),g.wj=function(t){return ue(t,510)},g.xj=function(t){return pe(dr,{3:1,4:1,5:1,1934:1},590,t,0,1)},D(Qt,"EcorePackageImpl/3",1186),M(1213,1,mr,TGe),g.wj=function(t){return ue(t,573)},g.xj=function(t){return pe(hEe,st,1940,t,0,1)},D(Qt,"EcorePackageImpl/30",1213),M(1214,1,mr,CGe),g.wj=function(t){return ue(t,153)},g.xj=function(t){return pe(NEe,Hw,153,t,0,1)},D(Qt,"EcorePackageImpl/31",1214),M(1215,1,mr,SGe),g.wj=function(t){return ue(t,72)},g.xj=function(t){return pe(ZH,Tyt,72,t,0,1)},D(Qt,"EcorePackageImpl/32",1215),M(1216,1,mr,AGe),g.wj=function(t){return ue(t,155)},g.xj=function(t){return pe(zx,Re,155,t,0,1)},D(Qt,"EcorePackageImpl/33",1216),M(1217,1,mr,MGe),g.wj=function(t){return ue(t,19)},g.xj=function(t){return pe(Ws,Re,19,t,0,1)},D(Qt,"EcorePackageImpl/34",1217),M(1218,1,mr,DGe),g.wj=function(t){return ue(t,290)},g.xj=function(t){return pe(f3e,st,290,t,0,1)},D(Qt,"EcorePackageImpl/35",1218),M(1219,1,mr,LGe),g.wj=function(t){return ue(t,162)},g.xj=function(t){return pe(Xb,Re,162,t,0,1)},D(Qt,"EcorePackageImpl/36",1219),M(1220,1,mr,IGe),g.wj=function(t){return ue(t,83)},g.xj=function(t){return pe(d3e,st,83,t,0,1)},D(Qt,"EcorePackageImpl/37",1220),M(1221,1,mr,OGe),g.wj=function(t){return ue(t,591)},g.xj=function(t){return pe(DEe,st,591,t,0,1)},D(Qt,"EcorePackageImpl/38",1221),M(1222,1,mr,NGe),g.wj=function(t){return!1},g.xj=function(t){return pe(WEe,st,2111,t,0,1)},D(Qt,"EcorePackageImpl/39",1222),M(1187,1,mr,PGe),g.wj=function(t){return ue(t,88)},g.xj=function(t){return pe(Jl,st,26,t,0,1)},D(Qt,"EcorePackageImpl/4",1187),M(1223,1,mr,BGe),g.wj=function(t){return ue(t,184)},g.xj=function(t){return pe(Qb,Re,184,t,0,1)},D(Qt,"EcorePackageImpl/40",1223),M(1224,1,mr,FGe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Qt,"EcorePackageImpl/41",1224),M(1225,1,mr,RGe),g.wj=function(t){return ue(t,588)},g.xj=function(t){return pe(lEe,st,588,t,0,1)},D(Qt,"EcorePackageImpl/42",1225),M(1226,1,mr,jGe),g.wj=function(t){return!1},g.xj=function(t){return pe(KEe,Re,2112,t,0,1)},D(Qt,"EcorePackageImpl/43",1226),M(1227,1,mr,$Ge),g.wj=function(t){return ue(t,42)},g.xj=function(t){return pe(c2,TR,42,t,0,1)},D(Qt,"EcorePackageImpl/44",1227),M(1188,1,mr,HGe),g.wj=function(t){return ue(t,138)},g.xj=function(t){return pe(af,st,138,t,0,1)},D(Qt,"EcorePackageImpl/5",1188),M(1189,1,mr,zGe),g.wj=function(t){return ue(t,148)},g.xj=function(t){return pe(rie,st,148,t,0,1)},D(Qt,"EcorePackageImpl/6",1189),M(1190,1,mr,GGe),g.wj=function(t){return ue(t,457)},g.xj=function(t){return pe(QH,st,671,t,0,1)},D(Qt,"EcorePackageImpl/7",1190),M(1191,1,mr,VGe),g.wj=function(t){return ue(t,573)},g.xj=function(t){return pe(qd,st,678,t,0,1)},D(Qt,"EcorePackageImpl/8",1191),M(1192,1,mr,UGe),g.wj=function(t){return ue(t,471)},g.xj=function(t){return pe(cT,st,471,t,0,1)},D(Qt,"EcorePackageImpl/9",1192),M(1025,1982,Zmt,DXe),g.bi=function(t,n){Z6n(this,h(n,415))},g.fi=function(t,n){qdt(this,t,h(n,415))},D(Qt,"MinimalEObjectImpl/1ArrayDelegatingAdapterList",1025),M(1026,143,rD,Yrt),g.Ai=function(){return this.a.a},D(Qt,"MinimalEObjectImpl/1ArrayDelegatingAdapterList/1",1026),M(1053,1052,{},oet),D("org.eclipse.emf.ecore.plugin","EcorePlugin",1053);var DEe=fi(Cyt,"Resource");M(781,1378,Syt),g.Yk=function(t){},g.Zk=function(t){},g.Vk=function(){return!this.a&&(this.a=new kq(this)),this.a},g.Wk=function(t){var n,s,a,u,p;if(a=t.length,a>0)if(er(0,t.length),t.charCodeAt(0)==47){for(p=new Wo(4),u=1,n=1;n<a;++n)er(n,t.length),t.charCodeAt(n)==47&&(Ue(p,u==n?"":t.substr(u,n-u)),u=n+1);return Ue(p,t.substr(u)),eEn(this,p)}else er(a-1,t.length),t.charCodeAt(a-1)==63&&(s=A1e(t,Ec(63),a-2),s>0&&(t=t.substr(0,s)));return yTn(this,t)},g.Xk=function(){return this.c},g.Ib=function(){var t;return sg(this.gm)+"@"+(t=ii(this)>>>0,t.toString(16))+" uri='"+this.d+"'"},g.b=!1,D(Oee,"ResourceImpl",781),M(1379,781,Syt,_Ke),D(Oee,"BinaryResourceImpl",1379),M(1169,694,See),g.si=function(t){return ue(t,56)?twn(this,h(t,56)):ue(t,591)?new Tn(h(t,591).Vk()):Me(t)===Me(this.f)?h(t,14).Kc():(ik(),bL.a)},g.Ob=function(){return Ebe(this)},g.a=!1,D(ri,"EcoreUtil/ContentTreeIterator",1169),M(1380,1169,See,_rt),g.si=function(t){return Me(t)===Me(this.f)?h(t,15).Kc():new oat(h(t,56))},D(Oee,"ResourceImpl/5",1380),M(648,1994,pyt,kq),g.Hc=function(t){return this.i<=4?ix(this,t):ue(t,49)&&h(t,49).Zg()==this.a},g.bi=function(t,n){t==this.i-1&&(this.a.b||(this.a.b=!0))},g.di=function(t,n){t==0?this.a.b||(this.a.b=!0):pK(this,t,n)},g.fi=function(t,n){},g.gi=function(t,n,s){},g.aj=function(){return 2},g.Ai=function(){return this.a},g.bj=function(){return!0},g.cj=function(t,n){var s;return s=h(t,49),n=s.wh(this.a,n),n},g.dj=function(t,n){var s;return s=h(t,49),s.wh(null,n)},g.ej=function(){return!1},g.hi=function(){return!0},g.ri=function(t){return pe(Jg,st,56,t,0,1)},g.ni=function(){return!1},D(Oee,"ResourceImpl/ContentsEList",648),M(957,1964,Ex,EKe),g.Zc=function(t){return this.a._h(t)},g.gc=function(){return this.a.gc()},D(ri,"AbstractSequentialInternalEList/1",957);var LEe,IEe,Ps,OEe;M(624,1,{},Cnt);var JH,ez;D(ri,"BasicExtendedMetaData",624),M(1160,1,{},_Je),g.$k=function(){return null},g._k=function(){return this.a==-2&&efn(this,S9n(this.d,this.b)),this.a},g.al=function(){return null},g.bl=function(){return Rt(),Rt(),ca},g.ne=function(){return this.c==Rx&&nfn(this,Eft(this.d,this.b)),this.c},g.cl=function(){return 0},g.a=-2,g.c=Rx,D(ri,"BasicExtendedMetaData/EClassExtendedMetaDataImpl",1160),M(1161,1,{},Yst),g.$k=function(){return this.a==(kk(),JH)&&tfn(this,GSn(this.f,this.b)),this.a},g._k=function(){return 0},g.al=function(){return this.c==(kk(),JH)&&rfn(this,VSn(this.f,this.b)),this.c},g.bl=function(){return!this.d&&ifn(this,GMn(this.f,this.b)),this.d},g.ne=function(){return this.e==Rx&&sfn(this,Eft(this.f,this.b)),this.e},g.cl=function(){return this.g==-2&&afn(this,qEn(this.f,this.b)),this.g},g.e=Rx,g.g=-2,D(ri,"BasicExtendedMetaData/EDataTypeExtendedMetaDataImpl",1161),M(1159,1,{},CJe),g.b=!1,g.c=!1,D(ri,"BasicExtendedMetaData/EPackageExtendedMetaDataImpl",1159),M(1162,1,{},qst),g.c=-2,g.e=Rx,g.f=Rx,D(ri,"BasicExtendedMetaData/EStructuralFeatureExtendedMetaDataImpl",1162),M(585,622,Va,yP),g.aj=function(){return this.c},g.Fk=function(){return!1},g.li=function(t,n){return n},g.c=0,D(ri,"EDataTypeEList",585);var NEe=fi(ri,"FeatureMap");M(75,585,{3:1,4:1,20:1,28:1,52:1,14:1,15:1,54:1,67:1,63:1,58:1,76:1,153:1,215:1,1937:1,69:1,95:1},yi),g.Vc=function(t,n){jSn(this,t,h(n,72))},g.Fc=function(t){return iSn(this,h(t,72))},g.Yh=function(t){Vbn(this,h(t,72))},g.cj=function(t,n){return _gn(this,h(t,72),n)},g.dj=function(t,n){return D1e(this,h(t,72),n)},g.ii=function(t,n){return oDn(this,t,n)},g.li=function(t,n){return $In(this,t,h(n,72))},g._c=function(t,n){return MAn(this,t,h(n,72))},g.jj=function(t,n){return Tgn(this,h(t,72),n)},g.kj=function(t,n){return Qtt(this,h(t,72),n)},g.lj=function(t,n,s){return MEn(this,h(t,72),h(n,72),s)},g.oi=function(t,n){return sQ(this,t,h(n,72))},g.dl=function(t,n){return hve(this,t,n)},g.Wc=function(t,n){var s,a,u,p,v,m,x,_,A;for(_=new yb(n.gc()),u=n.Kc();u.Ob();)if(a=h(u.Pb(),72),p=a.ak(),Bd(this.e,p))(!p.hi()||!pB(this,p,a.dd())&&!ix(_,a))&&qn(_,a);else{for(A=ac(this.e.Tg(),p),s=h(this.g,119),v=!0,m=0;m<this.i;++m)if(x=s[m],A.rl(x.ak())){h(Sy(this,m,a),72),v=!1;break}v&&qn(_,a)}return jge(this,t,_)},g.Gc=function(t){var n,s,a,u,p,v,m,x,_;for(x=new yb(t.gc()),a=t.Kc();a.Ob();)if(s=h(a.Pb(),72),u=s.ak(),Bd(this.e,u))(!u.hi()||!pB(this,u,s.dd())&&!ix(x,s))&&qn(x,s);else{for(_=ac(this.e.Tg(),u),n=h(this.g,119),p=!0,v=0;v<this.i;++v)if(m=n[v],_.rl(m.ak())){h(Sy(this,v,s),72),p=!1;break}p&&qn(x,s)}return mi(this,x)},g.Wh=function(t){return this.j=-1,GQ(this,this.i,t)},g.el=function(t,n,s){return sve(this,t,n,s)},g.mk=function(t,n){return _M(this,t,n)},g.fl=function(t,n,s){return Ave(this,t,n,s)},g.gl=function(){return this},g.hl=function(t,n){return AM(this,t,n)},g.il=function(t){return h(he(this,t),72).ak()},g.jl=function(t){return h(he(this,t),72).dd()},g.kl=function(){return this.b},g.bj=function(){return!0},g.ij=function(){return!0},g.ll=function(t){return!QA(this,t)},g.ri=function(t){return pe(Q9t,Tyt,332,t,0,1)},g.Gk=function(t){return $Y(this,t)},g.Wb=function(t){rA(this,t)},g.ml=function(t,n){pR(this,t,n)},g.nl=function(t){return Fut(this,t)},g.ol=function(t){o1t(this,t)},D(ri,"BasicFeatureMap",75),M(1851,1,z1),g.Nb=function(t){Ls(this,t)},g.Rb=function(t){if(this.g==-1)throw ne(new Ic);NP(this);try{Igt(this.e,this.b,this.a,t),this.d=this.e.j,OF(this)}catch(n){throw n=ui(n),ue(n,73)?ne(new cl):ne(n)}},g.Ob=function(){return iX(this)},g.Sb=function(){return oht(this)},g.Pb=function(){return OF(this)},g.Tb=function(){return this.a},g.Ub=function(){var t;if(oht(this))return NP(this),this.g=--this.a,this.Lk()&&(t=m9(this.e,this.b,this.c,this.a,this.j),this.j=t),this.i=0,this.j;throw ne(new lo)},g.Vb=function(){return this.a-1},g.Qb=function(){if(this.g==-1)throw ne(new Ic);NP(this);try{Mdt(this.e,this.b,this.g),this.d=this.e.j,this.g<this.a&&(--this.a,--this.c),--this.g}catch(t){throw t=ui(t),ue(t,73)?ne(new cl):ne(t)}},g.Lk=function(){return!1},g.Wb=function(t){if(this.g==-1)throw ne(new Ic);NP(this);try{n2t(this.e,this.b,this.g,t),this.d=this.e.j}catch(n){throw n=ui(n),ue(n,73)?ne(new cl):ne(n)}},g.a=0,g.c=0,g.d=0,g.f=!1,g.g=0,g.i=0,D(ri,"FeatureMapUtil/BasicFeatureEIterator",1851),M(410,1851,z1,Rk),g.pl=function(){var t,n,s;for(s=this.e.i,t=h(this.e.g,119);this.c<s;){if(n=t[this.c],this.k.rl(n.ak()))return this.j=this.f?n:n.dd(),this.i=2,!0;++this.c}return this.i=1,this.g=-1,!1},g.ql=function(){var t,n;for(t=h(this.e.g,119);--this.c>=0;)if(n=t[this.c],this.k.rl(n.ak()))return this.j=this.f?n:n.dd(),this.i=-2,!0;return this.i=-1,this.g=-1,!1},D(ri,"BasicFeatureMap/FeatureEIterator",410),M(662,410,z1,_Y),g.Lk=function(){return!0},D(ri,"BasicFeatureMap/ResolvingFeatureEIterator",662),M(955,486,wj,wet),g.Gi=function(){return this},D(ri,"EContentsEList/1",955),M(956,486,wj,zJe),g.Lk=function(){return!1},D(ri,"EContentsEList/2",956),M(954,279,mj,met),g.Nk=function(t){},g.Ob=function(){return!1},g.Sb=function(){return!1},D(ri,"EContentsEList/FeatureIteratorImpl/1",954),M(825,585,Va,Pfe),g.ci=function(){this.a=!0},g.fj=function(){return this.a},g.Xj=function(){var t;jn(this),Tu(this.e)?(t=this.a,this.a=!1,Or(this.e,new kh(this.e,2,this.c,t,!1))):this.a=!1},g.a=!1,D(ri,"EDataTypeEList/Unsettable",825),M(1849,585,Va,Aet),g.hi=function(){return!0},D(ri,"EDataTypeUniqueEList",1849),M(1850,825,Va,Met),g.hi=function(){return!0},D(ri,"EDataTypeUniqueEList/Unsettable",1850),M(139,85,Va,Nc),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectContainmentEList/Resolving",139),M(1163,545,Va,Cet),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectContainmentEList/Unsettable/Resolving",1163),M(748,16,Va,E1e),g.ci=function(){this.a=!0},g.fj=function(){return this.a},g.Xj=function(){var t;jn(this),Tu(this.e)?(t=this.a,this.a=!1,Or(this.e,new kh(this.e,2,this.c,t,!1))):this.a=!1},g.a=!1,D(ri,"EObjectContainmentWithInverseEList/Unsettable",748),M(1173,748,Va,Rtt),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectContainmentWithInverseEList/Unsettable/Resolving",1173),M(743,496,Va,Nfe),g.ci=function(){this.a=!0},g.fj=function(){return this.a},g.Xj=function(){var t;jn(this),Tu(this.e)?(t=this.a,this.a=!1,Or(this.e,new kh(this.e,2,this.c,t,!1))):this.a=!1},g.a=!1,D(ri,"EObjectEList/Unsettable",743),M(328,496,Va,ly),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectResolvingEList",328),M(1641,743,Va,Det),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectResolvingEList/Unsettable",1641),M(1381,1,{},qGe);var J9t;D(ri,"EObjectValidator",1381),M(546,496,Va,jP),g.zk=function(){return this.d},g.Ak=function(){return this.b},g.bj=function(){return!0},g.Dk=function(){return!0},g.b=0,D(ri,"EObjectWithInverseEList",546),M(1176,546,Va,jtt),g.Ck=function(){return!0},D(ri,"EObjectWithInverseEList/ManyInverse",1176),M(625,546,Va,VY),g.ci=function(){this.a=!0},g.fj=function(){return this.a},g.Xj=function(){var t;jn(this),Tu(this.e)?(t=this.a,this.a=!1,Or(this.e,new kh(this.e,2,this.c,t,!1))):this.a=!1},g.a=!1,D(ri,"EObjectWithInverseEList/Unsettable",625),M(1175,625,Va,$tt),g.Ck=function(){return!0},D(ri,"EObjectWithInverseEList/Unsettable/ManyInverse",1175),M(749,546,Va,_1e),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectWithInverseResolvingEList",749),M(31,749,Va,qt),g.Ck=function(){return!0},D(ri,"EObjectWithInverseResolvingEList/ManyInverse",31),M(750,625,Va,T1e),g.Ek=function(){return!0},g.li=function(t,n){return f5(this,t,h(n,56))},D(ri,"EObjectWithInverseResolvingEList/Unsettable",750),M(1174,750,Va,Htt),g.Ck=function(){return!0},D(ri,"EObjectWithInverseResolvingEList/Unsettable/ManyInverse",1174),M(1164,622,Va),g.ai=function(){return(this.b&1792)==0},g.ci=function(){this.b|=1},g.Bk=function(){return(this.b&4)!=0},g.bj=function(){return(this.b&40)!=0},g.Ck=function(){return(this.b&16)!=0},g.Dk=function(){return(this.b&8)!=0},g.Ek=function(){return(this.b&Ww)!=0},g.rk=function(){return(this.b&32)!=0},g.Fk=function(){return(this.b&Ch)!=0},g.wj=function(t){return this.d?xat(this.d,t):this.ak().Yj().wj(t)},g.fj=function(){return(this.b&2)!=0?(this.b&1)!=0:this.i!=0},g.hi=function(){return(this.b&128)!=0},g.Xj=function(){var t;jn(this),(this.b&2)!=0&&(Tu(this.e)?(t=(this.b&1)!=0,this.b&=-2,$6(this,new kh(this.e,2,oi(this.e.Tg(),this.ak()),t,!1))):this.b&=-2)},g.ni=function(){return(this.b&1536)==0},g.b=0,D(ri,"EcoreEList/Generic",1164),M(1165,1164,Va,Lit),g.ak=function(){return this.a},D(ri,"EcoreEList/Dynamic",1165),M(747,63,b1,Rle),g.ri=function(t){return LA(this.a.a,t)},D(ri,"EcoreEMap/1",747),M(746,85,Va,Ede),g.bi=function(t,n){PF(this.b,h(n,133))},g.di=function(t,n){vlt(this.b)},g.ei=function(t,n,s){var a;++(a=this.b,h(n,133),a).e},g.fi=function(t,n){gX(this.b,h(n,133))},g.gi=function(t,n,s){gX(this.b,h(s,133)),Me(s)===Me(n)&&h(s,133).Th(Odn(h(n,133).cd())),PF(this.b,h(n,133))},D(ri,"EcoreEMap/DelegateEObjectContainmentEList",746),M(1171,151,Zye,Dut),D(ri,"EcoreEMap/Unsettable",1171),M(1172,746,Va,ztt),g.ci=function(){this.a=!0},g.fj=function(){return this.a},g.Xj=function(){var t;jn(this),Tu(this.e)?(t=this.a,this.a=!1,Or(this.e,new kh(this.e,2,this.c,t,!1))):this.a=!1},g.a=!1,D(ri,"EcoreEMap/Unsettable/UnsettableDelegateEObjectContainmentEList",1172),M(1168,228,Vy,Nrt),g.a=!1,g.b=!1,D(ri,"EcoreUtil/Copier",1168),M(745,1,ys,oat),g.Nb=function(t){Ls(this,t)},g.Ob=function(){return rft(this)},g.Pb=function(){var t;return rft(this),t=this.b,this.b=null,t},g.Qb=function(){this.a.Qb()},D(ri,"EcoreUtil/ProperContentIterator",745),M(1382,1381,{},GUe);var e_t;D(ri,"EcoreValidator",1382);var t_t;fi(ri,"FeatureMapUtil/Validator"),M(1260,1,{1942:1},YGe),g.rl=function(t){return!0},D(ri,"FeatureMapUtil/1",1260),M(757,1,{1942:1},$ve),g.rl=function(t){var n;return this.c==t?!0:(n=lt(_n(this.a,t)),n==null?_Mn(this,t)?(pot(this.a,t,(on(),Hx)),!0):(pot(this.a,t,(on(),u2)),!1):n==(on(),Hx))},g.e=!1;var oie;D(ri,"FeatureMapUtil/BasicValidator",757),M(758,43,Vy,Dfe),D(ri,"FeatureMapUtil/BasicValidator/Cache",758),M(501,52,{20:1,28:1,52:1,14:1,15:1,58:1,76:1,69:1,95:1},RS),g.Vc=function(t,n){Igt(this.c,this.b,t,n)},g.Fc=function(t){return hve(this.c,this.b,t)},g.Wc=function(t,n){return jLn(this.c,this.b,t,n)},g.Gc=function(t){return vE(this,t)},g.Xh=function(t,n){a4n(this.c,this.b,t,n)},g.lk=function(t,n){return sve(this.c,this.b,t,n)},g.pi=function(t){return gR(this.c,this.b,t,!1)},g.Zh=function(){return JJe(this.c,this.b)},g.$h=function(){return ydn(this.c,this.b)},g._h=function(t){return h3n(this.c,this.b,t)},g.mk=function(t,n){return ytt(this,t,n)},g.$b=function(){x4(this)},g.Hc=function(t){return pB(this.c,this.b,t)},g.Ic=function(t){return l5n(this.c,this.b,t)},g.Xb=function(t){return gR(this.c,this.b,t,!0)},g.Wj=function(t){return this},g.Xc=function(t){return xyn(this.c,this.b,t)},g.dc=function(){return YN(this)},g.fj=function(){return!QA(this.c,this.b)},g.Kc=function(){return W3n(this.c,this.b)},g.Yc=function(){return K3n(this.c,this.b)},g.Zc=function(t){return okn(this.c,this.b,t)},g.ii=function(t,n){return Xpt(this.c,this.b,t,n)},g.ji=function(t,n){s3n(this.c,this.b,t,n)},g.$c=function(t){return Mdt(this.c,this.b,t)},g.Mc=function(t){return UMn(this.c,this.b,t)},g._c=function(t,n){return n2t(this.c,this.b,t,n)},g.Wb=function(t){QF(this.c,this.b),vE(this,h(t,15))},g.gc=function(){return ykn(this.c,this.b)},g.Pc=function(){return mmn(this.c,this.b)},g.Qc=function(t){return Eyn(this.c,this.b,t)},g.Ib=function(){var t,n;for(n=new s0,n.a+="[",t=JJe(this.c,this.b);iX(t);)va(n,mE(OF(t))),iX(t)&&(n.a+=Js);return n.a+="]",n.a},g.Xj=function(){QF(this.c,this.b)},D(ri,"FeatureMapUtil/FeatureEList",501),M(627,36,rD,aK),g.yi=function(t){return i9(this,t)},g.Di=function(t){var n,s,a,u,p,v,m;switch(this.d){case 1:case 2:{if(p=t.Ai(),Me(p)===Me(this.c)&&i9(this,null)==t.yi(null))return this.g=t.zi(),t.xi()==1&&(this.d=1),!0;break}case 3:{switch(u=t.xi(),u){case 3:{if(p=t.Ai(),Me(p)===Me(this.c)&&i9(this,null)==t.yi(null))return this.d=5,n=new yb(2),qn(n,this.g),qn(n,t.zi()),this.g=n,!0;break}}break}case 5:{switch(u=t.xi(),u){case 3:{if(p=t.Ai(),Me(p)===Me(this.c)&&i9(this,null)==t.yi(null))return s=h(this.g,14),s.Fc(t.zi()),!0;break}}break}case 4:{switch(u=t.xi(),u){case 3:{if(p=t.Ai(),Me(p)===Me(this.c)&&i9(this,null)==t.yi(null))return this.d=1,this.g=t.zi(),!0;break}case 4:{if(p=t.Ai(),Me(p)===Me(this.c)&&i9(this,null)==t.yi(null))return this.d=6,m=new yb(2),qn(m,this.n),qn(m,t.Bi()),this.n=m,v=se(ie(Hn,1),hr,25,15,[this.o,t.Ci()]),this.g=v,!0;break}}break}case 6:{switch(u=t.xi(),u){case 4:{if(p=t.Ai(),Me(p)===Me(this.c)&&i9(this,null)==t.yi(null))return s=h(this.n,14),s.Fc(t.Bi()),v=h(this.g,48),a=pe(Hn,hr,25,v.length+1,15,1),Ao(v,0,a,0,v.length),a[v.length]=t.Ci(),this.g=a,!0;break}}break}}return!1},D(ri,"FeatureMapUtil/FeatureENotificationImpl",627),M(552,501,{20:1,28:1,52:1,14:1,15:1,58:1,76:1,153:1,215:1,1937:1,69:1,95:1},_P),g.dl=function(t,n){return hve(this.c,t,n)},g.el=function(t,n,s){return sve(this.c,t,n,s)},g.fl=function(t,n,s){return Ave(this.c,t,n,s)},g.gl=function(){return this},g.hl=function(t,n){return AM(this.c,t,n)},g.il=function(t){return h(gR(this.c,this.b,t,!1),72).ak()},g.jl=function(t){return h(gR(this.c,this.b,t,!1),72).dd()},g.kl=function(){return this.a},g.ll=function(t){return!QA(this.c,t)},g.ml=function(t,n){pR(this.c,t,n)},g.nl=function(t){return Fut(this.c,t)},g.ol=function(t){o1t(this.c,t)},D(ri,"FeatureMapUtil/FeatureFeatureMap",552),M(1259,1,Iee,SJe),g.Wj=function(t){return gR(this.b,this.a,-1,t)},g.fj=function(){return!QA(this.b,this.a)},g.Wb=function(t){pR(this.b,this.a,t)},g.Xj=function(){QF(this.b,this.a)},D(ri,"FeatureMapUtil/FeatureValue",1259);var Q5,cie,uie,Z5,n_t,wL=fi(Sj,"AnyType");M(666,60,Fd,Fq),D(Sj,"InvalidDatatypeValueException",666);var tz=fi(Sj,Myt),mL=fi(Sj,Dyt),PEe=fi(Sj,Lyt),r_t,bo,BEe,_2,i_t,s_t,a_t,o_t,c_t,u_t,l_t,h_t,f_t,d_t,g_t,S3,p_t,A3,dT,b_t,bv,yL,kL,v_t,gT,pT;M(830,506,{105:1,92:1,90:1,56:1,49:1,97:1,843:1},she),g._g=function(t,n,s){switch(t){case 0:return s?(!this.c&&(this.c=new yi(this,0)),this.c):(!this.c&&(this.c=new yi(this,0)),this.c.b);case 1:return s?(!this.c&&(this.c=new yi(this,0)),h(Ro(this.c,(qr(),_2)),153)):(!this.c&&(this.c=new yi(this,0)),h(h(Ro(this.c,(qr(),_2)),153),215)).kl();case 2:return s?(!this.b&&(this.b=new yi(this,2)),this.b):(!this.b&&(this.b=new yi(this,2)),this.b.b)}return gl(this,t-En(this.zh()),Ht((this.j&2)==0?this.zh():(!this.k&&(this.k=new Gh),this.k).ck(),t),n,s)},g.jh=function(t,n,s){var a;switch(n){case 0:return!this.c&&(this.c=new yi(this,0)),_M(this.c,t,s);case 1:return(!this.c&&(this.c=new yi(this,0)),h(h(Ro(this.c,(qr(),_2)),153),69)).mk(t,s);case 2:return!this.b&&(this.b=new yi(this,2)),_M(this.b,t,s)}return a=h(Ht((this.j&2)==0?this.zh():(!this.k&&(this.k=new Gh),this.k).ck(),n),66),a.Nj().Rj(this,Z0e(this),n-En(this.zh()),t,s)},g.lh=function(t){switch(t){case 0:return!!this.c&&this.c.i!=0;case 1:return!(!this.c&&(this.c=new yi(this,0)),h(Ro(this.c,(qr(),_2)),153)).dc();case 2:return!!this.b&&this.b.i!=0}return fl(this,t-En(this.zh()),Ht((this.j&2)==0?this.zh():(!this.k&&(this.k=new Gh),this.k).ck(),t))},g.sh=function(t,n){switch(t){case 0:!this.c&&(this.c=new yi(this,0)),rA(this.c,n);return;case 1:(!this.c&&(this.c=new yi(this,0)),h(h(Ro(this.c,(qr(),_2)),153),215)).Wb(n);return;case 2:!this.b&&(this.b=new yi(this,2)),rA(this.b,n);return}ml(this,t-En(this.zh()),Ht((this.j&2)==0?this.zh():(!this.k&&(this.k=new Gh),this.k).ck(),t),n)},g.zh=function(){return qr(),BEe},g.Bh=function(t){switch(t){case 0:!this.c&&(this.c=new yi(this,0)),jn(this.c);return;case 1:(!this.c&&(this.c=new yi(this,0)),h(Ro(this.c,(qr(),_2)),153)).$b();return;case 2:!this.b&&(this.b=new yi(this,2)),jn(this.b);return}vl(this,t-En(this.zh()),Ht((this.j&2)==0?this.zh():(!this.k&&(this.k=new Gh),this.k).ck(),t))},g.Ib=function(){var t;return(this.j&4)!=0?_h(this):(t=new Ol(_h(this)),t.a+=" (mixed: ",dE(t,this.c),t.a+=", anyAttribute: ",dE(t,this.b),t.a+=")",t.a)},D(Ni,"AnyTypeImpl",830),M(667,506,{105:1,92:1,90:1,56:1,49:1,97:1,2021:1,667:1},KGe),g._g=function(t,n,s){switch(t){case 0:return this.a;case 1:return this.b}return gl(this,t-En((qr(),S3)),Ht((this.j&2)==0?S3:(!this.k&&(this.k=new Gh),this.k).ck(),t),n,s)},g.lh=function(t){switch(t){case 0:return this.a!=null;case 1:return this.b!=null}return fl(this,t-En((qr(),S3)),Ht((this.j&2)==0?S3:(!this.k&&(this.k=new Gh),this.k).ck(),t))},g.sh=function(t,n){switch(t){case 0:dfn(this,Jn(n));return;case 1:gfn(this,Jn(n));return}ml(this,t-En((qr(),S3)),Ht((this.j&2)==0?S3:(!this.k&&(this.k=new Gh),this.k).ck(),t),n)},g.zh=function(){return qr(),S3},g.Bh=function(t){switch(t){case 0:this.a=null;return;case 1:this.b=null;return}vl(this,t-En((qr(),S3)),Ht((this.j&2)==0?S3:(!this.k&&(this.k=new Gh),this.k).ck(),t))},g.Ib=function(){var t;return(this.j&4)!=0?_h(this):(t=new Ol(_h(this)),t.a+=" (data: ",va(t,this.a),t.a+=", target: ",va(t,this.b),t.a+=")",t.a)},g.a=null,g.b=null,D(Ni,"ProcessingInstructionImpl",667),M(668,830,{105:1,92:1,90:1,56:1,49:1,97:1,843:1,2022:1,668:1},gXe),g._g=function(t,n,s){switch(t){case 0:return s?(!this.c&&(this.c=new yi(this,0)),this.c):(!this.c&&(this.c=new yi(this,0)),this.c.b);case 1:return s?(!this.c&&(this.c=new yi(this,0)),h(Ro(this.c,(qr(),_2)),153)):(!this.c&&(this.c=new yi(this,0)),h(h(Ro(this.c,(qr(),_2)),153),215)).kl();case 2:return s?(!this.b&&(this.b=new yi(this,2)),this.b):(!this.b&&(this.b=new yi(this,2)),this.b.b);case 3:return!this.c&&(this.c=new yi(this,0)),Jn(AM(this.c,(qr(),dT),!0));case 4:return C1e(this.a,(!this.c&&(this.c=new yi(this,0)),Jn(AM(this.c,(qr(),dT),!0))));case 5:return this.a}return gl(this,t-En((qr(),A3)),Ht((this.j&2)==0?A3:(!this.k&&(this.k=new Gh),this.k).ck(),t),n,s)},g.lh=function(t){switch(t){case 0:return!!this.c&&this.c.i!=0;case 1:return!(!this.c&&(this.c=new yi(this,0)),h(Ro(this.c,(qr(),_2)),153)).dc();case 2:return!!this.b&&this.b.i!=0;case 3:return!this.c&&(this.c=new yi(this,0)),Jn(AM(this.c,(qr(),dT),!0))!=null;case 4:return C1e(this.a,(!this.c&&(this.c=new yi(this,0)),Jn(AM(this.c,(qr(),dT),!0))))!=null;case 5:return!!this.a}return fl(this,t-En((qr(),A3)),Ht((this.j&2)==0?A3:(!this.k&&(this.k=new Gh),this.k).ck(),t))},g.sh=function(t,n){switch(t){case 0:!this.c&&(this.c=new yi(this,0)),rA(this.c,n);return;case 1:(!this.c&&(this.c=new yi(this,0)),h(h(Ro(this.c,(qr(),_2)),153),215)).Wb(n);return;case 2:!this.b&&(this.b=new yi(this,2)),rA(this.b,n);return;case 3:h0e(this,Jn(n));return;case 4:h0e(this,S1e(this.a,n));return;case 5:pfn(this,h(n,148));return}ml(this,t-En((qr(),A3)),Ht((this.j&2)==0?A3:(!this.k&&(this.k=new Gh),this.k).ck(),t),n)},g.zh=function(){return qr(),A3},g.Bh=function(t){switch(t){case 0:!this.c&&(this.c=new yi(this,0)),jn(this.c);return;case 1:(!this.c&&(this.c=new yi(this,0)),h(Ro(this.c,(qr(),_2)),153)).$b();return;case 2:!this.b&&(this.b=new yi(this,2)),jn(this.b);return;case 3:!this.c&&(this.c=new yi(this,0)),pR(this.c,(qr(),dT),null);return;case 4:h0e(this,S1e(this.a,null));return;case 5:this.a=null;return}vl(this,t-En((qr(),A3)),Ht((this.j&2)==0?A3:(!this.k&&(this.k=new Gh),this.k).ck(),t))},D(Ni,"SimpleAnyTypeImpl",668),M(669,506,{105:1,92:1,90:1,56:1,49:1,97:1,2023:1,669:1},dXe),g._g=function(t,n,s){switch(t){case 0:return s?(!this.a&&(this.a=new yi(this,0)),this.a):(!this.a&&(this.a=new yi(this,0)),this.a.b);case 1:return s?(!this.b&&(this.b=new Du((Bt(),na),uo,this,1)),this.b):(!this.b&&(this.b=new Du((Bt(),na),uo,this,1)),wA(this.b));case 2:return s?(!this.c&&(this.c=new Du((Bt(),na),uo,this,2)),this.c):(!this.c&&(this.c=new Du((Bt(),na),uo,this,2)),wA(this.c));case 3:return!this.a&&(this.a=new yi(this,0)),Ro(this.a,(qr(),yL));case 4:return!this.a&&(this.a=new yi(this,0)),Ro(this.a,(qr(),kL));case 5:return!this.a&&(this.a=new yi(this,0)),Ro(this.a,(qr(),gT));case 6:return!this.a&&(this.a=new yi(this,0)),Ro(this.a,(qr(),pT))}return gl(this,t-En((qr(),bv)),Ht((this.j&2)==0?bv:(!this.k&&(this.k=new Gh),this.k).ck(),t),n,s)},g.jh=function(t,n,s){var a;switch(n){case 0:return!this.a&&(this.a=new yi(this,0)),_M(this.a,t,s);case 1:return!this.b&&(this.b=new Du((Bt(),na),uo,this,1)),pP(this.b,t,s);case 2:return!this.c&&(this.c=new Du((Bt(),na),uo,this,2)),pP(this.c,t,s);case 5:return!this.a&&(this.a=new yi(this,0)),ytt(Ro(this.a,(qr(),gT)),t,s)}return a=h(Ht((this.j&2)==0?(qr(),bv):(!this.k&&(this.k=new Gh),this.k).ck(),n),66),a.Nj().Rj(this,Z0e(this),n-En((qr(),bv)),t,s)},g.lh=function(t){switch(t){case 0:return!!this.a&&this.a.i!=0;case 1:return!!this.b&&this.b.f!=0;case 2:return!!this.c&&this.c.f!=0;case 3:return!this.a&&(this.a=new yi(this,0)),!YN(Ro(this.a,(qr(),yL)));case 4:return!this.a&&(this.a=new yi(this,0)),!YN(Ro(this.a,(qr(),kL)));case 5:return!this.a&&(this.a=new yi(this,0)),!YN(Ro(this.a,(qr(),gT)));case 6:return!this.a&&(this.a=new yi(this,0)),!YN(Ro(this.a,(qr(),pT)))}return fl(this,t-En((qr(),bv)),Ht((this.j&2)==0?bv:(!this.k&&(this.k=new Gh),this.k).ck(),t))},g.sh=function(t,n){switch(t){case 0:!this.a&&(this.a=new yi(this,0)),rA(this.a,n);return;case 1:!this.b&&(this.b=new Du((Bt(),na),uo,this,1)),rF(this.b,n);return;case 2:!this.c&&(this.c=new Du((Bt(),na),uo,this,2)),rF(this.c,n);return;case 3:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),yL))),!this.a&&(this.a=new yi(this,0)),vE(Ro(this.a,yL),h(n,14));return;case 4:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),kL))),!this.a&&(this.a=new yi(this,0)),vE(Ro(this.a,kL),h(n,14));return;case 5:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),gT))),!this.a&&(this.a=new yi(this,0)),vE(Ro(this.a,gT),h(n,14));return;case 6:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),pT))),!this.a&&(this.a=new yi(this,0)),vE(Ro(this.a,pT),h(n,14));return}ml(this,t-En((qr(),bv)),Ht((this.j&2)==0?bv:(!this.k&&(this.k=new Gh),this.k).ck(),t),n)},g.zh=function(){return qr(),bv},g.Bh=function(t){switch(t){case 0:!this.a&&(this.a=new yi(this,0)),jn(this.a);return;case 1:!this.b&&(this.b=new Du((Bt(),na),uo,this,1)),this.b.c.$b();return;case 2:!this.c&&(this.c=new Du((Bt(),na),uo,this,2)),this.c.c.$b();return;case 3:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),yL)));return;case 4:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),kL)));return;case 5:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),gT)));return;case 6:!this.a&&(this.a=new yi(this,0)),x4(Ro(this.a,(qr(),pT)));return}vl(this,t-En((qr(),bv)),Ht((this.j&2)==0?bv:(!this.k&&(this.k=new Gh),this.k).ck(),t))},g.Ib=function(){var t;return(this.j&4)!=0?_h(this):(t=new Ol(_h(this)),t.a+=" (mixed: ",dE(t,this.a),t.a+=")",t.a)},D(Ni,"XMLTypeDocumentRootImpl",669),M(1919,704,{105:1,92:1,90:1,471:1,147:1,56:1,108:1,49:1,97:1,150:1,114:1,115:1,2024:1},WGe),g.Ih=function(t,n){switch(t.yj()){case 7:case 8:case 9:case 10:case 16:case 22:case 23:case 24:case 25:case 26:case 32:case 33:case 34:case 36:case 37:case 44:case 45:case 50:case 51:case 53:case 55:case 56:case 57:case 58:case 60:case 61:case 4:return n==null?null:Ga(n);case 19:case 28:case 29:case 35:case 38:case 39:case 41:case 46:case 52:case 54:case 5:return Jn(n);case 6:return R0n(h(n,190));case 12:case 47:case 49:case 11:return G2t(this,t,n);case 13:return n==null?null:TLn(h(n,240));case 15:case 14:return n==null?null:Pbn(Pe(Je(n)));case 17:return q1t((qr(),n));case 18:return q1t(n);case 21:case 20:return n==null?null:Bbn(h(n,155).a);case 27:return j0n(h(n,190));case 30:return c1t((qr(),h(n,15)));case 31:return c1t(h(n,15));case 40:return H0n((qr(),n));case 42:return Y1t((qr(),n));case 43:return Y1t(n);case 59:case 48:return $0n((qr(),n));default:throw ne(new an(Px+t.ne()+Wb))}},g.Jh=function(t){var n,s,a,u,p;switch(t.G==-1&&(t.G=(s=Vu(t),s?y0(s.Mh(),t):-1)),t.G){case 0:return n=new she,n;case 1:return a=new KGe,a;case 2:return u=new gXe,u;case 3:return p=new dXe,p;default:throw ne(new an(bee+t.zb+Wb))}},g.Kh=function(t,n){var s,a,u,p,v,m,x,_,A,I,P,j,V,J,oe,fe;switch(t.yj()){case 5:case 52:case 4:return n;case 6:return t8n(n);case 8:case 7:return n==null?null:zEn(n);case 9:return n==null?null:MA(Wu((a=Ho(n,!0),a.length>0&&(er(0,a.length),a.charCodeAt(0)==43)?a.substr(1):a),-128,127)<<24>>24);case 10:return n==null?null:MA(Wu((u=Ho(n,!0),u.length>0&&(er(0,u.length),u.charCodeAt(0)==43)?u.substr(1):u),-128,127)<<24>>24);case 11:return Jn(Hb(this,(qr(),a_t),n));case 12:return Jn(Hb(this,(qr(),o_t),n));case 13:return n==null?null:new Che(Ho(n,!0));case 15:case 14:return uSn(n);case 16:return Jn(Hb(this,(qr(),c_t),n));case 17:return gft((qr(),n));case 18:return gft(n);case 28:case 29:case 35:case 38:case 39:case 41:case 54:case 19:return Ho(n,!0);case 21:case 20:return wSn(n);case 22:return Jn(Hb(this,(qr(),u_t),n));case 23:return Jn(Hb(this,(qr(),l_t),n));case 24:return Jn(Hb(this,(qr(),h_t),n));case 25:return Jn(Hb(this,(qr(),f_t),n));case 26:return Jn(Hb(this,(qr(),d_t),n));case 27:return Xxn(n);case 30:return pft((qr(),n));case 31:return pft(n);case 32:return n==null?null:Ke(Wu((A=Ho(n,!0),A.length>0&&(er(0,A.length),A.charCodeAt(0)==43)?A.substr(1):A),$s,Ir));case 33:return n==null?null:new hg((I=Ho(n,!0),I.length>0&&(er(0,I.length),I.charCodeAt(0)==43)?I.substr(1):I));case 34:return n==null?null:Ke(Wu((P=Ho(n,!0),P.length>0&&(er(0,P.length),P.charCodeAt(0)==43)?P.substr(1):P),$s,Ir));case 36:return n==null?null:qp(yR((j=Ho(n,!0),j.length>0&&(er(0,j.length),j.charCodeAt(0)==43)?j.substr(1):j)));case 37:return n==null?null:qp(yR((V=Ho(n,!0),V.length>0&&(er(0,V.length),V.charCodeAt(0)==43)?V.substr(1):V)));case 40:return fxn((qr(),n));case 42:return bft((qr(),n));case 43:return bft(n);case 44:return n==null?null:new hg((J=Ho(n,!0),J.length>0&&(er(0,J.length),J.charCodeAt(0)==43)?J.substr(1):J));case 45:return n==null?null:new hg((oe=Ho(n,!0),oe.length>0&&(er(0,oe.length),oe.charCodeAt(0)==43)?oe.substr(1):oe));case 46:return Ho(n,!1);case 47:return Jn(Hb(this,(qr(),g_t),n));case 59:case 48:return hxn((qr(),n));case 49:return Jn(Hb(this,(qr(),p_t),n));case 50:return n==null?null:Yk(Wu((fe=Ho(n,!0),fe.length>0&&(er(0,fe.length),fe.charCodeAt(0)==43)?fe.substr(1):fe),Ej,32767)<<16>>16);case 51:return n==null?null:Yk(Wu((p=Ho(n,!0),p.length>0&&(er(0,p.length),p.charCodeAt(0)==43)?p.substr(1):p),Ej,32767)<<16>>16);case 53:return Jn(Hb(this,(qr(),b_t),n));case 55:return n==null?null:Yk(Wu((v=Ho(n,!0),v.length>0&&(er(0,v.length),v.charCodeAt(0)==43)?v.substr(1):v),Ej,32767)<<16>>16);case 56:return n==null?null:Yk(Wu((m=Ho(n,!0),m.length>0&&(er(0,m.length),m.charCodeAt(0)==43)?m.substr(1):m),Ej,32767)<<16>>16);case 57:return n==null?null:qp(yR((x=Ho(n,!0),x.length>0&&(er(0,x.length),x.charCodeAt(0)==43)?x.substr(1):x)));case 58:return n==null?null:qp(yR((_=Ho(n,!0),_.length>0&&(er(0,_.length),_.charCodeAt(0)==43)?_.substr(1):_)));case 60:return n==null?null:Ke(Wu((s=Ho(n,!0),s.length>0&&(er(0,s.length),s.charCodeAt(0)==43)?s.substr(1):s),$s,Ir));case 61:return n==null?null:Ke(Wu(Ho(n,!0),$s,Ir));default:throw ne(new an(Px+t.ne()+Wb))}};var w_t,FEe,m_t,REe;D(Ni,"XMLTypeFactoryImpl",1919),M(586,179,{105:1,92:1,90:1,147:1,191:1,56:1,235:1,108:1,49:1,97:1,150:1,179:1,114:1,115:1,675:1,1945:1,586:1},tit),g.N=!1,g.O=!1;var y_t=!1;D(Ni,"XMLTypePackageImpl",586),M(1852,1,{837:1},XGe),g._j=function(){return mve(),M_t},D(Ni,"XMLTypePackageImpl/1",1852),M(1861,1,mr,QGe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/10",1861),M(1862,1,mr,ZGe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/11",1862),M(1863,1,mr,JGe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/12",1863),M(1864,1,mr,eVe),g.wj=function(t){return ew(t)},g.xj=function(t){return pe(Ts,Re,333,t,7,1)},D(Ni,"XMLTypePackageImpl/13",1864),M(1865,1,mr,tVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/14",1865),M(1866,1,mr,nVe),g.wj=function(t){return ue(t,15)},g.xj=function(t){return pe(xl,Hw,15,t,0,1)},D(Ni,"XMLTypePackageImpl/15",1866),M(1867,1,mr,rVe),g.wj=function(t){return ue(t,15)},g.xj=function(t){return pe(xl,Hw,15,t,0,1)},D(Ni,"XMLTypePackageImpl/16",1867),M(1868,1,mr,iVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/17",1868),M(1869,1,mr,sVe),g.wj=function(t){return ue(t,155)},g.xj=function(t){return pe(zx,Re,155,t,0,1)},D(Ni,"XMLTypePackageImpl/18",1869),M(1870,1,mr,aVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/19",1870),M(1853,1,mr,oVe),g.wj=function(t){return ue(t,843)},g.xj=function(t){return pe(wL,st,843,t,0,1)},D(Ni,"XMLTypePackageImpl/2",1853),M(1871,1,mr,cVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/20",1871),M(1872,1,mr,uVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/21",1872),M(1873,1,mr,lVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/22",1873),M(1874,1,mr,hVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/23",1874),M(1875,1,mr,fVe),g.wj=function(t){return ue(t,190)},g.xj=function(t){return pe(Vc,Re,190,t,0,2)},D(Ni,"XMLTypePackageImpl/24",1875),M(1876,1,mr,dVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/25",1876),M(1877,1,mr,gVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/26",1877),M(1878,1,mr,pVe),g.wj=function(t){return ue(t,15)},g.xj=function(t){return pe(xl,Hw,15,t,0,1)},D(Ni,"XMLTypePackageImpl/27",1878),M(1879,1,mr,bVe),g.wj=function(t){return ue(t,15)},g.xj=function(t){return pe(xl,Hw,15,t,0,1)},D(Ni,"XMLTypePackageImpl/28",1879),M(1880,1,mr,vVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/29",1880),M(1854,1,mr,wVe),g.wj=function(t){return ue(t,667)},g.xj=function(t){return pe(tz,st,2021,t,0,1)},D(Ni,"XMLTypePackageImpl/3",1854),M(1881,1,mr,mVe),g.wj=function(t){return ue(t,19)},g.xj=function(t){return pe(Ws,Re,19,t,0,1)},D(Ni,"XMLTypePackageImpl/30",1881),M(1882,1,mr,yVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/31",1882),M(1883,1,mr,kVe),g.wj=function(t){return ue(t,162)},g.xj=function(t){return pe(Xb,Re,162,t,0,1)},D(Ni,"XMLTypePackageImpl/32",1883),M(1884,1,mr,xVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/33",1884),M(1885,1,mr,EVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/34",1885),M(1886,1,mr,_Ve),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/35",1886),M(1887,1,mr,TVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/36",1887),M(1888,1,mr,CVe),g.wj=function(t){return ue(t,15)},g.xj=function(t){return pe(xl,Hw,15,t,0,1)},D(Ni,"XMLTypePackageImpl/37",1888),M(1889,1,mr,SVe),g.wj=function(t){return ue(t,15)},g.xj=function(t){return pe(xl,Hw,15,t,0,1)},D(Ni,"XMLTypePackageImpl/38",1889),M(1890,1,mr,AVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/39",1890),M(1855,1,mr,MVe),g.wj=function(t){return ue(t,668)},g.xj=function(t){return pe(mL,st,2022,t,0,1)},D(Ni,"XMLTypePackageImpl/4",1855),M(1891,1,mr,DVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/40",1891),M(1892,1,mr,LVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/41",1892),M(1893,1,mr,IVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/42",1893),M(1894,1,mr,OVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/43",1894),M(1895,1,mr,NVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/44",1895),M(1896,1,mr,PVe),g.wj=function(t){return ue(t,184)},g.xj=function(t){return pe(Qb,Re,184,t,0,1)},D(Ni,"XMLTypePackageImpl/45",1896),M(1897,1,mr,BVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/46",1897),M(1898,1,mr,FVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/47",1898),M(1899,1,mr,RVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/48",1899),M(Og,1,mr,jVe),g.wj=function(t){return ue(t,184)},g.xj=function(t){return pe(Qb,Re,184,t,0,1)},D(Ni,"XMLTypePackageImpl/49",Og),M(1856,1,mr,$Ve),g.wj=function(t){return ue(t,669)},g.xj=function(t){return pe(PEe,st,2023,t,0,1)},D(Ni,"XMLTypePackageImpl/5",1856),M(1901,1,mr,HVe),g.wj=function(t){return ue(t,162)},g.xj=function(t){return pe(Xb,Re,162,t,0,1)},D(Ni,"XMLTypePackageImpl/50",1901),M(1902,1,mr,zVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/51",1902),M(1903,1,mr,GVe),g.wj=function(t){return ue(t,19)},g.xj=function(t){return pe(Ws,Re,19,t,0,1)},D(Ni,"XMLTypePackageImpl/52",1903),M(1857,1,mr,VVe),g.wj=function(t){return ws(t)},g.xj=function(t){return pe(it,Re,2,t,6,1)},D(Ni,"XMLTypePackageImpl/6",1857),M(1858,1,mr,UVe),g.wj=function(t){return ue(t,190)},g.xj=function(t){return pe(Vc,Re,190,t,0,2)},D(Ni,"XMLTypePackageImpl/7",1858),M(1859,1,mr,qVe),g.wj=function(t){return Jv(t)},g.xj=function(t){return pe(es,Re,476,t,8,1)},D(Ni,"XMLTypePackageImpl/8",1859),M(1860,1,mr,YVe),g.wj=function(t){return ue(t,217)},g.xj=function(t){return pe(A5,Re,217,t,0,1)},D(Ni,"XMLTypePackageImpl/9",1860);var Gf,H0,bT,nz,ce;M(50,60,Fd,Zn),D(M0,"RegEx/ParseException",50),M(820,1,{},tle),g.sl=function(t){return t<this.j&&Is(this.i,t)==63},g.tl=function(){var t,n,s,a,u;if(this.c!=10)throw ne(new Zn(ir((Qn(),pj))));switch(t=this.a,t){case 101:t=27;break;case 102:t=12;break;case 110:t=10;break;case 114:t=13;break;case 116:t=9;break;case 120:if(Sr(this),this.c!=0)throw ne(new Zn(ir((Qn(),A0))));if(this.a==123){u=0,s=0;do{if(Sr(this),this.c!=0)throw ne(new Zn(ir((Qn(),A0))));if((u=Wp(this.a))<0)break;if(s>s*16)throw ne(new Zn(ir((Qn(),zmt))));s=s*16+u}while(!0);if(this.a!=125)throw ne(new Zn(ir((Qn(),Gmt))));if(s>jx)throw ne(new Zn(ir((Qn(),Vmt))));t=s}else{if(u=0,this.c!=0||(u=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(s=u,Sr(this),this.c!=0||(u=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));s=s*16+u,t=s}break;case 117:if(a=0,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=n*16+a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=n*16+a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));n=n*16+a,t=n;break;case 118:if(Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=n*16+a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=n*16+a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=n*16+a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=n*16+a,Sr(this),this.c!=0||(a=Wp(this.a))<0)throw ne(new Zn(ir((Qn(),A0))));if(n=n*16+a,n>jx)throw ne(new Zn(ir((Qn(),"parser.descappe.4"))));t=n;break;case 65:case 90:case 122:throw ne(new Zn(ir((Qn(),Umt))))}return t},g.ul=function(t){var n,s;switch(t){case 100:s=(this.e&32)==32?Lg("Nd",!0):(Mr(),rz);break;case 68:s=(this.e&32)==32?Lg("Nd",!1):(Mr(),VEe);break;case 119:s=(this.e&32)==32?Lg("IsWord",!0):(Mr(),C8);break;case 87:s=(this.e&32)==32?Lg("IsWord",!1):(Mr(),qEe);break;case 115:s=(this.e&32)==32?Lg("IsSpace",!0):(Mr(),J5);break;case 83:s=(this.e&32)==32?Lg("IsSpace",!1):(Mr(),UEe);break;default:throw ne(new Ya((n=t,Uyt+n.toString(16))))}return s},g.vl=function(t){var n,s,a,u,p,v,m,x,_,A,I,P;for(this.b=1,Sr(this),n=null,this.c==0&&this.a==94?(Sr(this),t?A=(Mr(),Mr(),new zu(5)):(n=(Mr(),Mr(),new zu(4)),$o(n,0,jx),A=new zu(4))):A=(Mr(),Mr(),new zu(4)),u=!0;(P=this.c)!=1&&!(P==0&&this.a==93&&!u);){if(u=!1,s=this.a,a=!1,P==10)switch(s){case 100:case 68:case 119:case 87:case 115:case 83:Fw(A,this.ul(s)),a=!0;break;case 105:case 73:case 99:case 67:s=this.Ll(A,s),s<0&&(a=!0);break;case 112:case 80:if(I=kbe(this,s),!I)throw ne(new Zn(ir((Qn(),Mee))));Fw(A,I),a=!0;break;default:s=this.tl()}else if(P==20){if(v=nk(this.i,58,this.d),v<0)throw ne(new Zn(ir((Qn(),qye))));if(m=!0,Is(this.i,this.d)==94&&(++this.d,m=!1),p=$u(this.i,this.d,v),x=nct(p,m,(this.e&512)==512),!x)throw ne(new Zn(ir((Qn(),Fmt))));if(Fw(A,x),a=!0,v+1>=this.j||Is(this.i,v+1)!=93)throw ne(new Zn(ir((Qn(),qye))));this.d=v+2}if(Sr(this),!a)if(this.c!=0||this.a!=45)$o(A,s,s);else{if(Sr(this),(P=this.c)==1)throw ne(new Zn(ir((Qn(),bj))));P==0&&this.a==93?($o(A,s,s),$o(A,45,45)):(_=this.a,P==10&&(_=this.tl()),Sr(this),$o(A,s,_))}(this.e&Ch)==Ch&&this.c==0&&this.a==44&&Sr(this)}if(this.c==1)throw ne(new Zn(ir((Qn(),bj))));return n&&(I9(n,A),A=n),Ny(A),D9(A),this.b=0,Sr(this),A},g.wl=function(){var t,n,s,a;for(s=this.vl(!1);(a=this.c)!=7;)if(t=this.a,a==0&&(t==45||t==38)||a==4){if(Sr(this),this.c!=9)throw ne(new Zn(ir((Qn(),jmt))));if(n=this.vl(!1),a==4)Fw(s,n);else if(t==45)I9(s,n);else if(t==38)F2t(s,n);else throw ne(new Ya("ASSERT"))}else throw ne(new Zn(ir((Qn(),$mt))));return Sr(this),s},g.xl=function(){var t,n;return t=this.a-48,n=(Mr(),Mr(),new GW(12,null,t)),!this.g&&(this.g=new sN),iN(this.g,new jle(t)),Sr(this),n},g.yl=function(){return Sr(this),Mr(),E_t},g.zl=function(){return Sr(this),Mr(),x_t},g.Al=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Bl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Cl=function(){return Sr(this),d6n()},g.Dl=function(){return Sr(this),Mr(),T_t},g.El=function(){return Sr(this),Mr(),S_t},g.Fl=function(){var t;if(this.d>=this.j||((t=Is(this.i,this.d++))&65504)!=64)throw ne(new Zn(ir((Qn(),Nmt))));return Sr(this),Mr(),Mr(),new N1(0,t-64)},g.Gl=function(){return Sr(this),XDn()},g.Hl=function(){return Sr(this),Mr(),A_t},g.Il=function(){var t;return t=(Mr(),Mr(),new N1(0,105)),Sr(this),t},g.Jl=function(){return Sr(this),Mr(),C_t},g.Kl=function(){return Sr(this),Mr(),__t},g.Ll=function(t,n){return this.tl()},g.Ml=function(){return Sr(this),Mr(),zEe},g.Nl=function(){var t,n,s,a,u;if(this.d+1>=this.j)throw ne(new Zn(ir((Qn(),Lmt))));if(a=-1,n=null,t=Is(this.i,this.d),49<=t&&t<=57){if(a=t-48,!this.g&&(this.g=new sN),iN(this.g,new jle(a)),++this.d,Is(this.i,this.d)!=41)throw ne(new Zn(ir((Qn(),a2))));++this.d}else switch(t==63&&--this.d,Sr(this),n=Vve(this),n.e){case 20:case 21:case 22:case 23:break;case 8:if(this.c!=7)throw ne(new Zn(ir((Qn(),a2))));break;default:throw ne(new Zn(ir((Qn(),Imt))))}if(Sr(this),u=Lb(this),s=null,u.e==2){if(u.em()!=2)throw ne(new Zn(ir((Qn(),Omt))));s=u.am(1),u=u.am(0)}if(this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),Mr(),Mr(),new Nct(a,n,u,s)},g.Ol=function(){return Sr(this),Mr(),GEe},g.Pl=function(){var t;if(Sr(this),t=$P(24,Lb(this)),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),t},g.Ql=function(){var t;if(Sr(this),t=$P(20,Lb(this)),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),t},g.Rl=function(){var t;if(Sr(this),t=$P(22,Lb(this)),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),t},g.Sl=function(){var t,n,s,a,u;for(t=0,s=0,n=-1;this.d<this.j&&(n=Is(this.i,this.d),u=Cbe(n),u!=0);)t|=u,++this.d;if(this.d>=this.j)throw ne(new Zn(ir((Qn(),Vye))));if(n==45){for(++this.d;this.d<this.j&&(n=Is(this.i,this.d),u=Cbe(n),u!=0);)s|=u,++this.d;if(this.d>=this.j)throw ne(new Zn(ir((Qn(),Vye))))}if(n==58){if(++this.d,Sr(this),a=jrt(Lb(this),t,s),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));Sr(this)}else if(n==41)++this.d,Sr(this),a=jrt(Lb(this),t,s);else throw ne(new Zn(ir((Qn(),Dmt))));return a},g.Tl=function(){var t;if(Sr(this),t=$P(21,Lb(this)),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),t},g.Ul=function(){var t;if(Sr(this),t=$P(23,Lb(this)),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),t},g.Vl=function(){var t,n;if(Sr(this),t=this.f++,n=bW(Lb(this),t),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),n},g.Wl=function(){var t;if(Sr(this),t=bW(Lb(this),0),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),t},g.Xl=function(t){return Sr(this),this.c==5?(Sr(this),DP(t,(Mr(),Mr(),new dw(9,t)))):DP(t,(Mr(),Mr(),new dw(3,t)))},g.Yl=function(t){var n;return Sr(this),n=(Mr(),Mr(),new bE(2)),this.c==5?(Sr(this),Qp(n,wT),Qp(n,t)):(Qp(n,t),Qp(n,wT)),n},g.Zl=function(t){return Sr(this),this.c==5?(Sr(this),Mr(),Mr(),new dw(9,t)):(Mr(),Mr(),new dw(3,t))},g.a=0,g.b=0,g.c=0,g.d=0,g.e=0,g.f=1,g.g=null,g.j=0,D(M0,"RegEx/RegexParser",820),M(1824,820,{},pXe),g.sl=function(t){return!1},g.tl=function(){return tve(this)},g.ul=function(t){return gx(t)},g.vl=function(t){return Mbt(this)},g.wl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.xl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.yl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.zl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Al=function(){return Sr(this),gx(67)},g.Bl=function(){return Sr(this),gx(73)},g.Cl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Dl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.El=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Fl=function(){return Sr(this),gx(99)},g.Gl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Hl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Il=function(){return Sr(this),gx(105)},g.Jl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Kl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Ll=function(t,n){return Fw(t,gx(n)),-1},g.Ml=function(){return Sr(this),Mr(),Mr(),new N1(0,94)},g.Nl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Ol=function(){return Sr(this),Mr(),Mr(),new N1(0,36)},g.Pl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Ql=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Rl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Sl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Tl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Ul=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Vl=function(){var t;if(Sr(this),t=bW(Lb(this),0),this.c!=7)throw ne(new Zn(ir((Qn(),a2))));return Sr(this),t},g.Wl=function(){throw ne(new Zn(ir((Qn(),kl))))},g.Xl=function(t){return Sr(this),DP(t,(Mr(),Mr(),new dw(3,t)))},g.Yl=function(t){var n;return Sr(this),n=(Mr(),Mr(),new bE(2)),Qp(n,t),Qp(n,wT),n},g.Zl=function(t){return Sr(this),Mr(),Mr(),new dw(3,t)};var M3=null,_8=null;D(M0,"RegEx/ParserForXMLSchema",1824),M(117,1,$x,G2),g.$l=function(t){throw ne(new Ya("Not supported."))},g._l=function(){return-1},g.am=function(t){return null},g.bm=function(){return null},g.cm=function(t){},g.dm=function(t){},g.em=function(){return 0},g.Ib=function(){return this.fm(0)},g.fm=function(t){return this.e==11?".":""},g.e=0;var jEe,T8,vT,k_t,$Ee,xm=null,rz,lie=null,HEe,wT,hie=null,zEe,GEe,VEe,UEe,qEe,x_t,J5,E_t,__t,T_t,C_t,C8,S_t,A_t,zPn=D(M0,"RegEx/Token",117);M(136,117,{3:1,136:1,117:1},zu),g.fm=function(t){var n,s,a;if(this.e==4)if(this==HEe)s=".";else if(this==rz)s="\\d";else if(this==C8)s="\\w";else if(this==J5)s="\\s";else{for(a=new s0,a.a+="[",n=0;n<this.b.length;n+=2)(t&Ch)!=0&&n>0&&(a.a+=","),this.b[n]===this.b[n+1]?va(a,SM(this.b[n])):(va(a,SM(this.b[n])),a.a+="-",va(a,SM(this.b[n+1])));a.a+="]",s=a.a}else if(this==VEe)s="\\D";else if(this==qEe)s="\\W";else if(this==UEe)s="\\S";else{for(a=new s0,a.a+="[^",n=0;n<this.b.length;n+=2)(t&Ch)!=0&&n>0&&(a.a+=","),this.b[n]===this.b[n+1]?va(a,SM(this.b[n])):(va(a,SM(this.b[n])),a.a+="-",va(a,SM(this.b[n+1])));a.a+="]",s=a.a}return s},g.a=!1,g.c=!1,D(M0,"RegEx/RangeToken",136),M(584,1,{584:1},jle),g.a=0,D(M0,"RegEx/RegexParser/ReferencePosition",584),M(583,1,{3:1,583:1},CQe),g.Fb=function(t){var n;return t==null||!ue(t,583)?!1:(n=h(t,583),Nt(this.b,n.b)&&this.a==n.a)},g.Hb=function(){return k0(this.b+"/"+Kbe(this.a))},g.Ib=function(){return this.c.fm(this.a)},g.a=0,D(M0,"RegEx/RegularExpression",583),M(223,117,$x,N1),g._l=function(){return this.a},g.fm=function(t){var n,s,a;switch(this.e){case 0:switch(this.a){case 124:case 42:case 43:case 63:case 40:case 41:case 46:case 91:case 123:case 92:a="\\"+GY(this.a&Oi);break;case 12:a="\\f";break;case 10:a="\\n";break;case 13:a="\\r";break;case 9:a="\\t";break;case 27:a="\\e";break;default:this.a>=ea?(s=(n=this.a>>>0,"0"+n.toString(16)),a="\\v"+$u(s,s.length-6,s.length)):a=""+GY(this.a&Oi)}break;case 8:this==zEe||this==GEe?a=""+GY(this.a&Oi):a="\\"+GY(this.a&Oi);break;default:a=null}return a},g.a=0,D(M0,"RegEx/Token/CharToken",223),M(309,117,$x,dw),g.am=function(t){return this.a},g.cm=function(t){this.b=t},g.dm=function(t){this.c=t},g.em=function(){return 1},g.fm=function(t){var n;if(this.e==3)if(this.c<0&&this.b<0)n=this.a.fm(t)+"*";else if(this.c==this.b)n=this.a.fm(t)+"{"+this.c+"}";else if(this.c>=0&&this.b>=0)n=this.a.fm(t)+"{"+this.c+","+this.b+"}";else if(this.c>=0&&this.b<0)n=this.a.fm(t)+"{"+this.c+",}";else throw ne(new Ya("Token#toString(): CLOSURE "+this.c+Js+this.b));else if(this.c<0&&this.b<0)n=this.a.fm(t)+"*?";else if(this.c==this.b)n=this.a.fm(t)+"{"+this.c+"}?";else if(this.c>=0&&this.b>=0)n=this.a.fm(t)+"{"+this.c+","+this.b+"}?";else if(this.c>=0&&this.b<0)n=this.a.fm(t)+"{"+this.c+",}?";else throw ne(new Ya("Token#toString(): NONGREEDYCLOSURE "+this.c+Js+this.b));return n},g.b=0,g.c=0,D(M0,"RegEx/Token/ClosureToken",309),M(821,117,$x,Lde),g.am=function(t){return t==0?this.a:this.b},g.em=function(){return 2},g.fm=function(t){var n;return this.b.e==3&&this.b.am(0)==this.a?n=this.a.fm(t)+"+":this.b.e==9&&this.b.am(0)==this.a?n=this.a.fm(t)+"+?":n=this.a.fm(t)+(""+this.b.fm(t)),n},D(M0,"RegEx/Token/ConcatToken",821),M(1822,117,$x,Nct),g.am=function(t){if(t==0)return this.d;if(t==1)return this.b;throw ne(new Ya("Internal Error: "+t))},g.em=function(){return this.b?2:1},g.fm=function(t){var n;return this.c>0?n="(?("+this.c+")":this.a.e==8?n="(?("+this.a+")":n="(?"+this.a,this.b?n+=this.d+"|"+this.b+")":n+=this.d+")",n},g.c=0,D(M0,"RegEx/Token/ConditionToken",1822),M(1823,117,$x,Nst),g.am=function(t){return this.b},g.em=function(){return 1},g.fm=function(t){return"(?"+(this.a==0?"":Kbe(this.a))+(this.c==0?"":Kbe(this.c))+":"+this.b.fm(t)+")"},g.a=0,g.c=0,D(M0,"RegEx/Token/ModifierToken",1823),M(822,117,$x,Fde),g.am=function(t){return this.a},g.em=function(){return 1},g.fm=function(t){var n;switch(n=null,this.e){case 6:this.b==0?n="(?:"+this.a.fm(t)+")":n="("+this.a.fm(t)+")";break;case 20:n="(?="+this.a.fm(t)+")";break;case 21:n="(?!"+this.a.fm(t)+")";break;case 22:n="(?<="+this.a.fm(t)+")";break;case 23:n="(?<!"+this.a.fm(t)+")";break;case 24:n="(?>"+this.a.fm(t)+")"}return n},g.b=0,D(M0,"RegEx/Token/ParenToken",822),M(521,117,{3:1,117:1,521:1},GW),g.bm=function(){return this.b},g.fm=function(t){return this.e==12?"\\"+this.a:iCn(this.b)},g.a=0,D(M0,"RegEx/Token/StringToken",521),M(465,117,$x,bE),g.$l=function(t){Qp(this,t)},g.am=function(t){return h(lb(this.a,t),117)},g.em=function(){return this.a?this.a.a.c.length:0},g.fm=function(t){var n,s,a,u,p;if(this.e==1){if(this.a.a.c.length==2)n=h(lb(this.a,0),117),s=h(lb(this.a,1),117),s.e==3&&s.am(0)==n?u=n.fm(t)+"+":s.e==9&&s.am(0)==n?u=n.fm(t)+"+?":u=n.fm(t)+(""+s.fm(t));else{for(p=new s0,a=0;a<this.a.a.c.length;a++)va(p,h(lb(this.a,a),117).fm(t));u=p.a}return u}if(this.a.a.c.length==2&&h(lb(this.a,1),117).e==7)u=h(lb(this.a,0),117).fm(t)+"?";else if(this.a.a.c.length==2&&h(lb(this.a,0),117).e==7)u=h(lb(this.a,1),117).fm(t)+"??";else{for(p=new s0,va(p,h(lb(this.a,0),117).fm(t)),a=1;a<this.a.a.c.length;a++)p.a+="|",va(p,h(lb(this.a,a),117).fm(t));u=p.a}return u},D(M0,"RegEx/Token/UnionToken",465),M(518,1,{592:1},vS),g.Ib=function(){return this.a.b},D(Kyt,"XMLTypeUtil/PatternMatcherImpl",518),M(1622,1381,{},WVe);var M_t;D(Kyt,"XMLTypeValidator",1622),M(264,1,G1,Mp),g.Jc=function(t){Os(this,t)},g.Kc=function(){return(this.b-this.a)*this.c<0?sp:new fg(this)},g.a=0,g.b=0,g.c=0;var sp;D(h3e,"ExclusiveRange",264),M(1068,1,z1,KVe),g.Rb=function(t){h(t,19),Ndn()},g.Nb=function(t){Ls(this,t)},g.Pb=function(){return T1n()},g.Ub=function(){return C1n()},g.Wb=function(t){h(t,19),Bdn()},g.Ob=function(){return!1},g.Sb=function(){return!1},g.Tb=function(){return-1},g.Vb=function(){return-1},g.Qb=function(){throw ne(new i0(Zyt))},D(h3e,"ExclusiveRange/1",1068),M(254,1,z1,fg),g.Rb=function(t){h(t,19),Pdn()},g.Nb=function(t){Ls(this,t)},g.Pb=function(){return N6n(this)},g.Ub=function(){return D3n(this)},g.Wb=function(t){h(t,19),Fdn()},g.Ob=function(){return this.c.c<0?this.a>=this.c.b:this.a<=this.c.b},g.Sb=function(){return this.b>0},g.Tb=function(){return this.b},g.Vb=function(){return this.b-1},g.Qb=function(){throw ne(new i0(Zyt))},g.a=0,g.b=0,D(h3e,"ExclusiveRange/RangeIterator",254);var Cl=fk(vj,"C"),Hn=fk(a_,"I"),vu=fk(g5,"Z"),ap=fk(o_,"J"),Vc=fk(r_,"B"),ks=fk(i_,"D"),Em=fk(s_,"F"),D3=fk(c_,"S"),GPn=fi("org.eclipse.elk.core.labels","ILabelManager"),YEe=fi(Ys,"DiagnosticChain"),WEe=fi(Cyt,"ResourceSet"),KEe=D(Ys,"InvocationTargetException",null),D_t=(lN(),Byn),L_t=L_t=TEn;L4n(_fn),r5n("permProps",[[[Dj,Lj],[Ij,"gecko1_8"]],[[Dj,Lj],[Ij,"ie10"]],[[Dj,Lj],[Ij,"ie8"]],[[Dj,Lj],[Ij,"ie9"]],[[Dj,Lj],[Ij,"safari"]]]),L_t(null,"elk",null)}).call(this)}).call(this,typeof Mm<"u"?Mm:typeof self<"u"?self:typeof window<"u"?window:{})},{}],3:[function(o,l,f){function b(E,T){if(!(E instanceof T))throw new TypeError("Cannot call a class as a function")}function d(E,T){if(!E)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return T&&(typeof T=="object"||typeof T=="function")?T:E}function w(E,T){if(typeof T!="function"&&T!==null)throw new TypeError("Super expression must either be null or a function, not "+typeof T);E.prototype=Object.create(T&&T.prototype,{constructor:{value:E,enumerable:!1,writable:!0,configurable:!0}}),T&&(Object.setPrototypeOf?Object.setPrototypeOf(E,T):E.__proto__=T)}var y=o("./elk-api.js").default,k=function(E){w(T,E);function T(){var C=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};b(this,T);var S=Object.assign({},C),L=!1;try{o.resolve("web-worker"),L=!0}catch{}if(C.workerUrl)if(L){var O=o("web-worker");S.workerFactory=function(F){return new O(F)}}else console.warn(`Web worker requested but 'web-worker' package not installed. 
Consider installing the package or pass your own 'workerFactory' to ELK's constructor.
... Falling back to non-web worker version.`);if(!S.workerFactory){var B=o("./elk-worker.min.js"),N=B.Worker;S.workerFactory=function(F){return new N(F)}}return d(this,(T.__proto__||Object.getPrototypeOf(T)).call(this,S))}return T}(y);Object.defineProperty(l.exports,"__esModule",{value:!0}),l.exports=k,k.default=k},{"./elk-api.js":1,"./elk-worker.min.js":2,"web-worker":4}],4:[function(o,l,f){l.exports=Worker},{}]},{},[3])(3)})})(qIe);const ttn=q9e(qIe.exports),YIe=new ttn,zv={},ntn={};let h4={};const rtn=function(r,i,o,l,f,b,d){const w=o.select(`[id="${i}"]`),y=w.insert("g").attr("class","nodes");return Object.keys(r).forEach(function(E){const T=r[E];let C="default";T.classes.length>0&&(C=T.classes.join(" "));const S=Dv(T.styles);let L=T.text!==void 0?T.text:T.id,O;const B={width:0,height:0};if(cf(ht().flowchart.htmlLabels)){const z={label:L.replace(/fa[blrs]?:fa-[\w-]+/g,Z=>`<i class='${Z.replace(":"," ")}'></i>`)};O=$V(w,z).node();const W=O.getBBox();B.width=W.width,B.height=W.height,B.labelNode=O,O.parentNode.removeChild(O)}else{const z=l.createElementNS("http://www.w3.org/2000/svg","text");z.setAttribute("style",S.labelStyle.replace("color:","fill:"));const W=L.split(Cs.lineBreakRegex);for(const G of W){const ae=l.createElementNS("http://www.w3.org/2000/svg","tspan");ae.setAttributeNS("http://www.w3.org/XML/1998/namespace","xml:space","preserve"),ae.setAttribute("dy","1em"),ae.setAttribute("x","1"),ae.textContent=G,z.appendChild(ae)}O=z;const Z=O.getBBox();B.width=Z.width,B.height=Z.height,B.labelNode=O}const N=[{id:T.id+"-west",layoutOptions:{"port.side":"WEST"}},{id:T.id+"-east",layoutOptions:{"port.side":"EAST"}},{id:T.id+"-south",layoutOptions:{"port.side":"SOUTH"}},{id:T.id+"-north",layoutOptions:{"port.side":"NORTH"}}];let F=0,R="",q={};switch(T.type){case"round":F=5,R="rect";break;case"square":R="rect";break;case"diamond":R="question",q={portConstraints:"FIXED_SIDE"};break;case"hexagon":R="hexagon";break;case"odd":R="rect_left_inv_arrow";break;case"lean_right":R="lean_right";break;case"lean_left":R="lean_left";break;case"trapezoid":R="trapezoid";break;case"inv_trapezoid":R="inv_trapezoid";break;case"odd_right":R="rect_left_inv_arrow";break;case"circle":R="circle";break;case"ellipse":R="ellipse";break;case"stadium":R="stadium";break;case"subroutine":R="subroutine";break;case"cylinder":R="cylinder";break;case"group":R="rect";break;case"doublecircle":R="doublecircle";break;default:R="rect"}const X={labelStyle:S.labelStyle,shape:R,labelText:L,rx:F,ry:F,class:C,style:S.style,id:T.id,link:T.link,linkTarget:T.linkTarget,tooltip:f.db.getTooltip(T.id)||"",domId:f.db.lookUpDomId(T.id),haveCallback:T.haveCallback,width:T.type==="group"?500:void 0,dir:T.dir,type:T.type,props:T.props,padding:ht().flowchart.padding};let te,H;X.type!=="group"&&(H=UMe(y,X,T.dir),te=H.node().getBBox());const Y={id:T.id,ports:T.type==="diamond"?N:[],layoutOptions:q,labelText:L,labelData:B,domId:f.db.lookUpDomId(T.id),width:te==null?void 0:te.width,height:te==null?void 0:te.height,type:T.type,el:H,parent:b.parentById[T.id]};h4[X.id]=Y}),d},WIe=(r,i,o)=>{const l={TB:{in:{north:"north"},out:{south:"west",west:"east",east:"south"}},LR:{in:{west:"west"},out:{east:"south",south:"north",north:"east"}},RL:{in:{east:"east"},out:{west:"north",north:"south",south:"west"}},BT:{in:{south:"south"},out:{north:"east",east:"west",west:"north"}}};return l.TD=l.TB,Se.info("abc88",o,i,r),l[o][i][r]},KIe=(r,i,o)=>{if(Se.info("getNextPort abc88",{node:r,edgeDirection:i,graphDirection:o}),!zv[r])switch(o){case"TB":case"TD":zv[r]={inPosition:"north",outPosition:"south"};break;case"BT":zv[r]={inPosition:"south",outPosition:"north"};break;case"RL":zv[r]={inPosition:"east",outPosition:"west"};break;case"LR":zv[r]={inPosition:"west",outPosition:"east"};break}const l=i==="in"?zv[r].inPosition:zv[r].outPosition;return i==="in"?zv[r].inPosition=WIe(zv[r].inPosition,i,o):zv[r].outPosition=WIe(zv[r].outPosition,i,o),l},itn=(r,i)=>{let o=r.start,l=r.end;const f=h4[o],b=h4[l];return!f||!b?{source:o,target:l}:(f.type==="diamond"&&(o=`${o}-${KIe(o,"out",i)}`),b.type==="diamond"&&(l=`${l}-${KIe(l,"in",i)}`),{source:o,target:l})},stn=function(r,i,o,l){Se.info("abc78 edges = ",r);const f=l.insert("g").attr("class","edgeLabels");let b={},d=i.db.getDirection(),w,y;if(r.defaultStyle!==void 0){const k=Dv(r.defaultStyle);w=k.style,y=k.labelStyle}return r.forEach(function(k){var E="L-"+k.start+"-"+k.end;b[E]===void 0?(b[E]=0,Se.info("abc78 new entry",E,b[E])):(b[E]++,Se.info("abc78 new entry",E,b[E]));let T=E+"-"+b[E];Se.info("abc78 new link id to be used is",E,T,b[E]);var C="LS-"+k.start,S="LE-"+k.end;const L={style:"",labelStyle:""};switch(L.minlen=k.length||1,k.type==="arrow_open"?L.arrowhead="none":L.arrowhead="normal",L.arrowTypeStart="arrow_open",L.arrowTypeEnd="arrow_open",k.type){case"double_arrow_cross":L.arrowTypeStart="arrow_cross";case"arrow_cross":L.arrowTypeEnd="arrow_cross";break;case"double_arrow_point":L.arrowTypeStart="arrow_point";case"arrow_point":L.arrowTypeEnd="arrow_point";break;case"double_arrow_circle":L.arrowTypeStart="arrow_circle";case"arrow_circle":L.arrowTypeEnd="arrow_circle";break}let O="",B="";switch(k.stroke){case"normal":O="fill:none;",w!==void 0&&(O=w),y!==void 0&&(B=y),L.thickness="normal",L.pattern="solid";break;case"dotted":L.thickness="normal",L.pattern="dotted",L.style="fill:none;stroke-width:2px;stroke-dasharray:3;";break;case"thick":L.thickness="thick",L.pattern="solid",L.style="stroke-width: 3.5px;fill:none;";break}if(k.style!==void 0){const q=Dv(k.style);O=q.style,B=q.labelStyle}L.style=L.style+=O,L.labelStyle=L.labelStyle+=B,k.interpolate!==void 0?L.curve=q0(k.interpolate,x1):r.defaultInterpolate!==void 0?L.curve=q0(r.defaultInterpolate,x1):L.curve=q0(ntn.curve,x1),k.text===void 0?k.style!==void 0&&(L.arrowheadStyle="fill: #333"):(L.arrowheadStyle="fill: #333",L.labelpos="c"),L.labelType="text",L.label=k.text.replace(Cs.lineBreakRegex,`
`),k.style===void 0&&(L.style=L.style||"stroke: #333; stroke-width: 1.5px;fill:none;"),L.labelStyle=L.labelStyle.replace("color:","fill:"),L.id=T,L.classes="flowchart-link "+C+" "+S;const N=WMe(f,L),{source:F,target:R}=itn(k,d);Se.debug("abc78 source and target",F,R),o.edges.push({id:"e"+k.start+k.end,sources:[F],targets:[R],labelEl:N,labels:[{width:L.width,height:L.height,orgWidth:L.width,orgHeight:L.height,text:L.label,layoutOptions:{"edgeLabels.inline":"true","edgeLabels.placement":"CENTER"}}],edgeData:L})}),o},atn=function(r,i,o,l){let f="";switch(l&&(f=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,f=f.replace(/\(/g,"\\("),f=f.replace(/\)/g,"\\)")),i.arrowTypeStart){case"arrow_cross":r.attr("marker-start","url("+f+"#"+o+"-crossStart)");break;case"arrow_point":r.attr("marker-start","url("+f+"#"+o+"-pointStart)");break;case"arrow_barb":r.attr("marker-start","url("+f+"#"+o+"-barbStart)");break;case"arrow_circle":r.attr("marker-start","url("+f+"#"+o+"-circleStart)");break;case"aggregation":r.attr("marker-start","url("+f+"#"+o+"-aggregationStart)");break;case"extension":r.attr("marker-start","url("+f+"#"+o+"-extensionStart)");break;case"composition":r.attr("marker-start","url("+f+"#"+o+"-compositionStart)");break;case"dependency":r.attr("marker-start","url("+f+"#"+o+"-dependencyStart)");break;case"lollipop":r.attr("marker-start","url("+f+"#"+o+"-lollipopStart)");break}switch(i.arrowTypeEnd){case"arrow_cross":r.attr("marker-end","url("+f+"#"+o+"-crossEnd)");break;case"arrow_point":r.attr("marker-end","url("+f+"#"+o+"-pointEnd)");break;case"arrow_barb":r.attr("marker-end","url("+f+"#"+o+"-barbEnd)");break;case"arrow_circle":r.attr("marker-end","url("+f+"#"+o+"-circleEnd)");break;case"aggregation":r.attr("marker-end","url("+f+"#"+o+"-aggregationEnd)");break;case"extension":r.attr("marker-end","url("+f+"#"+o+"-extensionEnd)");break;case"composition":r.attr("marker-end","url("+f+"#"+o+"-compositionEnd)");break;case"dependency":r.attr("marker-end","url("+f+"#"+o+"-dependencyEnd)");break;case"lollipop":r.attr("marker-end","url("+f+"#"+o+"-lollipopEnd)");break}},otn=function(r,i){Se.info("Extracting classes"),i.db.clear("ver-2");try{return i.parse(r),i.db.getClasses()}catch{return{}}},ctn=function(r){const i={parentById:{},childrenById:{}},o=r.getSubGraphs();return Se.info("Subgraphs - ",o),o.forEach(function(l){l.nodes.forEach(function(f){i.parentById[f]=l.id,i.childrenById[l.id]===void 0&&(i.childrenById[l.id]=[]),i.childrenById[l.id].push(f)})}),o.forEach(function(l){l.id,i.parentById[l.id]!==void 0&&i.parentById[l.id]}),i},utn=function(r,i,o){const l=etn(r,i,o);if(l===void 0||l==="root")return{x:0,y:0};const f=h4[l].offset;return{x:f.posX,y:f.posY}},ltn=function(r,i,o,l,f){const b=utn(i.sources[0],i.targets[0],f),d=i.sections[0].startPoint,w=i.sections[0].endPoint,k=(i.sections[0].bendPoints?i.sections[0].bendPoints:[]).map(B=>[B.x+b.x,B.y+b.y]),E=[[d.x+b.x,d.y+b.y],...k,[w.x+b.x,w.y+b.y]],T=V8().curve(x1),C=r.insert("path").attr("d",T(E)).attr("class","path").attr("fill","none"),S=r.insert("g").attr("class","edgeLabel"),L=Cn(S.node().appendChild(i.labelEl)),O=L.node().firstChild.getBoundingClientRect();L.attr("width",O.width),L.attr("height",O.height),S.attr("transform",`translate(${i.labels[0].x+b.x}, ${i.labels[0].y+b.y})`),atn(C,o,l.type,l.arrowMarkerAbsolute)},XIe=(r,i)=>{r.forEach(o=>{o.children||(o.children=[]);const l=i.childrenById[o.id];l&&l.forEach(f=>{o.children.push(h4[f])}),XIe(o.children,i)})},htn=async function(r,i,o,l){var H;l.db.clear(),h4={},l.db.setGen("gen-2"),l.parser.parse(r);const f=Cn("body").append("div").attr("style","height:400px").attr("id","cy");let b={id:"root",layoutOptions:{"elk.hierarchyHandling":"INCLUDE_CHILDREN","org.eclipse.elk.padding":"[top=100, left=100, bottom=110, right=110]","elk.layered.spacing.edgeNodeBetweenLayers":"30","elk.direction":"DOWN"},children:[],edges:[]};switch(Se.info("Drawing flowchart using v3 renderer",YIe),l.db.getDirection()){case"BT":b.layoutOptions["elk.direction"]="UP";break;case"TB":b.layoutOptions["elk.direction"]="DOWN";break;case"LR":b.layoutOptions["elk.direction"]="RIGHT";break;case"RL":b.layoutOptions["elk.direction"]="LEFT";break}const{securityLevel:w,flowchart:y}=ht();let k;w==="sandbox"&&(k=Cn("#i"+i));const E=Cn(w==="sandbox"?k.nodes()[0].contentDocument.body:"body"),T=w==="sandbox"?k.nodes()[0].contentDocument:document,C=E.select(`[id="${i}"]`);OMe(C,["point","circle","cross"],l.type,l.arrowMarkerAbsolute);const L=l.db.getVertices();let O;const B=l.db.getSubGraphs();Se.info("Subgraphs - ",B);for(let Y=B.length-1;Y>=0;Y--)O=B[Y],l.db.addVertex(O.id,O.title,"group",void 0,O.classes,O.dir);const N=C.insert("g").attr("class","subgraphs"),F=ctn(l.db);b=rtn(L,i,E,T,l,F,b);const R=C.insert("g").attr("class","edges edgePath"),q=l.db.getEdges();b=stn(q,l,b,C),Object.keys(h4).forEach(Y=>{const z=h4[Y];z.parent||b.children.push(z),F.childrenById[Y]!==void 0&&(z.labels=[{text:z.labelText,layoutOptions:{"nodeLabels.placement":"[H_CENTER, V_TOP, INSIDE]"},width:z.labelData.width,height:z.labelData.height}],delete z.x,delete z.y,delete z.width,delete z.height)}),XIe(b.children,F),Se.info("after layout",JSON.stringify(b,null,2));const te=await YIe.layout(b);QIe(0,0,te.children,C,N,l,0),Se.info("after layout",te),(H=te.edges)==null||H.map(Y=>{ltn(R,Y,Y.edgeData,l,F)}),J8({},C,y.diagramPadding,y.useMaxWidth),f.remove()},QIe=(r,i,o,l,f,b,d)=>{o.forEach(function(w){if(w)if(h4[w.id].offset={posX:w.x+r,posY:w.y+i,x:r,y:i,depth:d,width:w.width,height:w.height},w.type==="group"){const y=f.insert("g").attr("class","subgraph");y.insert("rect").attr("class","subgraph subgraph-lvl-"+d%5+" node").attr("x",w.x+r).attr("y",w.y+i).attr("width",w.width).attr("height",w.height);const k=y.insert("g").attr("class","label");k.attr("transform",`translate(${w.labels[0].x+r+w.x}, ${w.labels[0].y+i+w.y})`),k.node().appendChild(w.labelData.labelNode),Se.info("Id (UGH)= ",w.type,w.labels)}else Se.info("Id (UGH)= ",w.id),w.el.attr("transform",`translate(${w.x+r+w.width/2}, ${w.y+i+w.height/2})`)}),o.forEach(function(w){w&&w.type==="group"&&QIe(r+w.x,i+w.y,w.children,l,f,b,d+1)})},ftn={getClasses:otn,draw:htn},dtn=r=>{let i="";for(let o=0;o<5;o++)i+=`
      .subgraph-lvl-${o} {
        fill: ${r[`surface${o}`]};
        stroke: ${r[`surfacePeer${o}`]};
      }
    `;return i},gtn=Object.freeze(Object.defineProperty({__proto__:null,diagram:{db:pXt,renderer:ftn,parser:Soe,styles:r=>`.label {
    font-family: ${r.fontFamily};
    color: ${r.nodeTextColor||r.textColor};
  }
  .cluster-label text {
    fill: ${r.titleColor};
  }
  .cluster-label span {
    color: ${r.titleColor};
  }

  .label text,span {
    fill: ${r.nodeTextColor||r.textColor};
    color: ${r.nodeTextColor||r.textColor};
  }

  .node rect,
  .node circle,
  .node ellipse,
  .node polygon,
  .node path {
    fill: ${r.mainBkg};
    stroke: ${r.nodeBorder};
    stroke-width: 1px;
  }

  .node .label {
    text-align: center;
  }
  .node.clickable {
    cursor: pointer;
  }

  .arrowheadPath {
    fill: ${r.arrowheadColor};
  }

  .edgePath .path {
    stroke: ${r.lineColor};
    stroke-width: 2.0px;
  }

  .flowchart-link {
    stroke: ${r.lineColor};
    fill: none;
  }

  .edgeLabel {
    background-color: ${r.edgeLabelBackground};
    rect {
      opacity: 0.5;
      background-color: ${r.edgeLabelBackground};
      fill: ${r.edgeLabelBackground};
    }
    text-align: center;
  }

  .cluster rect {
    fill: ${r.clusterBkg};
    stroke: ${r.clusterBorder};
    stroke-width: 1px;
  }

  .cluster text {
    fill: ${r.titleColor};
  }

  .cluster span {
    color: ${r.titleColor};
  }
  /* .cluster div {
    color: ${r.titleColor};
  } */

  div.mermaidTooltip {
    position: absolute;
    text-align: center;
    max-width: 200px;
    padding: 2px;
    font-family: ${r.fontFamily};
    font-size: 12px;
    background: ${r.tertiaryColor};
    border: 1px solid ${r.border2};
    border-radius: 2px;
    pointer-events: none;
    z-index: 100;
  }

  .flowchartTitleText {
    text-anchor: middle;
    font-size: 18px;
    fill: ${r.textColor};
  }
  .subgraph {
    stroke-width:2;
    rx:3;
  }
  // .subgraph-lvl-1 {
  //   fill:#ccc;
  //   // stroke:black;
  // }
  ${dtn(r)}
`}},Symbol.toStringTag,{value:"Module"}));var Fce=function(){var r=function(B,N,F,R){for(F=F||{},R=B.length;R--;F[B[R]]=N);return F},i=[1,2],o=[1,5],l=[6,9,11,17,18,20,22,23,26,27,28],f=[1,15],b=[1,16],d=[1,17],w=[1,18],y=[1,19],k=[1,23],E=[1,24],T=[1,27],C=[4,6,9,11,17,18,20,22,23,26,27,28],S={trace:function(){},yy:{},symbols_:{error:2,start:3,timeline:4,document:5,EOF:6,directive:7,line:8,SPACE:9,statement:10,NEWLINE:11,openDirective:12,typeDirective:13,closeDirective:14,":":15,argDirective:16,title:17,acc_title:18,acc_title_value:19,acc_descr:20,acc_descr_value:21,acc_descr_multiline_value:22,section:23,period_statement:24,event_statement:25,period:26,event:27,open_directive:28,type_directive:29,arg_directive:30,close_directive:31,$accept:0,$end:1},terminals_:{2:"error",4:"timeline",6:"EOF",9:"SPACE",11:"NEWLINE",15:":",17:"title",18:"acc_title",19:"acc_title_value",20:"acc_descr",21:"acc_descr_value",22:"acc_descr_multiline_value",23:"section",26:"period",27:"event",28:"open_directive",29:"type_directive",30:"arg_directive",31:"close_directive"},productions_:[0,[3,3],[3,2],[5,0],[5,2],[8,2],[8,1],[8,1],[8,1],[7,4],[7,6],[10,1],[10,2],[10,2],[10,1],[10,1],[10,1],[10,1],[10,1],[24,1],[25,1],[12,1],[13,1],[16,1],[14,1]],performAction:function(N,F,R,q,X,te,H){var Y=te.length-1;switch(X){case 1:return te[Y-1];case 3:this.$=[];break;case 4:te[Y-1].push(te[Y]),this.$=te[Y-1];break;case 5:case 6:this.$=te[Y];break;case 7:case 8:this.$=[];break;case 11:q.getCommonDb().setDiagramTitle(te[Y].substr(6)),this.$=te[Y].substr(6);break;case 12:this.$=te[Y].trim(),q.getCommonDb().setAccTitle(this.$);break;case 13:case 14:this.$=te[Y].trim(),q.getCommonDb().setAccDescription(this.$);break;case 15:q.addSection(te[Y].substr(8)),this.$=te[Y].substr(8);break;case 19:q.addTask(te[Y],0,""),this.$=te[Y];break;case 20:q.addEvent(te[Y].substr(2)),this.$=te[Y];break;case 21:q.parseDirective("%%{","open_directive");break;case 22:q.parseDirective(te[Y],"type_directive");break;case 23:te[Y]=te[Y].trim().replace(/'/g,'"'),q.parseDirective(te[Y],"arg_directive");break;case 24:q.parseDirective("}%%","close_directive","timeline");break}},table:[{3:1,4:i,7:3,12:4,28:o},{1:[3]},r(l,[2,3],{5:6}),{3:7,4:i,7:3,12:4,28:o},{13:8,29:[1,9]},{29:[2,21]},{6:[1,10],7:22,8:11,9:[1,12],10:13,11:[1,14],12:4,17:f,18:b,20:d,22:w,23:y,24:20,25:21,26:k,27:E,28:o},{1:[2,2]},{14:25,15:[1,26],31:T},r([15,31],[2,22]),r(l,[2,8],{1:[2,1]}),r(l,[2,4]),{7:22,10:28,12:4,17:f,18:b,20:d,22:w,23:y,24:20,25:21,26:k,27:E,28:o},r(l,[2,6]),r(l,[2,7]),r(l,[2,11]),{19:[1,29]},{21:[1,30]},r(l,[2,14]),r(l,[2,15]),r(l,[2,16]),r(l,[2,17]),r(l,[2,18]),r(l,[2,19]),r(l,[2,20]),{11:[1,31]},{16:32,30:[1,33]},{11:[2,24]},r(l,[2,5]),r(l,[2,12]),r(l,[2,13]),r(C,[2,9]),{14:34,31:T},{31:[2,23]},{11:[1,35]},r(C,[2,10])],defaultActions:{5:[2,21],7:[2,2],27:[2,24],33:[2,23]},parseError:function(N,F){if(F.recoverable)this.trace(N);else{var R=new Error(N);throw R.hash=F,R}},parse:function(N){var F=this,R=[0],q=[],X=[null],te=[],H=this.table,Y="",z=0,W=0,Z=2,G=1,ae=te.slice.call(arguments,1),$=Object.create(this.lexer),ge={yy:{}};for(var ee in this.yy)Object.prototype.hasOwnProperty.call(this.yy,ee)&&(ge.yy[ee]=this.yy[ee]);$.setInput(N,ge.yy),ge.yy.lexer=$,ge.yy.parser=this,typeof $.yylloc>"u"&&($.yylloc={});var de=$.yylloc;te.push(de);var re=$.options&&$.options.ranges;typeof ge.yy.parseError=="function"?this.parseError=ge.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function ke(){var Le;return Le=q.pop()||$.lex()||G,typeof Le!="number"&&(Le instanceof Array&&(q=Le,Le=q.pop()),Le=F.symbols_[Le]||Le),Le}for(var Ce,_e,Te,Be,Ge={},Xe,Ee,Ze,Ie;;){if(_e=R[R.length-1],this.defaultActions[_e]?Te=this.defaultActions[_e]:((Ce===null||typeof Ce>"u")&&(Ce=ke()),Te=H[_e]&&H[_e][Ce]),typeof Te>"u"||!Te.length||!Te[0]){var Oe="";Ie=[];for(Xe in H[_e])this.terminals_[Xe]&&Xe>Z&&Ie.push("'"+this.terminals_[Xe]+"'");$.showPosition?Oe="Parse error on line "+(z+1)+`:
`+$.showPosition()+`
Expecting `+Ie.join(", ")+", got '"+(this.terminals_[Ce]||Ce)+"'":Oe="Parse error on line "+(z+1)+": Unexpected "+(Ce==G?"end of input":"'"+(this.terminals_[Ce]||Ce)+"'"),this.parseError(Oe,{text:$.match,token:this.terminals_[Ce]||Ce,line:$.yylineno,loc:de,expected:Ie})}if(Te[0]instanceof Array&&Te.length>1)throw new Error("Parse Error: multiple actions possible at state: "+_e+", token: "+Ce);switch(Te[0]){case 1:R.push(Ce),X.push($.yytext),te.push($.yylloc),R.push(Te[1]),Ce=null,W=$.yyleng,Y=$.yytext,z=$.yylineno,de=$.yylloc;break;case 2:if(Ee=this.productions_[Te[1]][1],Ge.$=X[X.length-Ee],Ge._$={first_line:te[te.length-(Ee||1)].first_line,last_line:te[te.length-1].last_line,first_column:te[te.length-(Ee||1)].first_column,last_column:te[te.length-1].last_column},re&&(Ge._$.range=[te[te.length-(Ee||1)].range[0],te[te.length-1].range[1]]),Be=this.performAction.apply(Ge,[Y,W,z,ge.yy,Te[1],X,te].concat(ae)),typeof Be<"u")return Be;Ee&&(R=R.slice(0,-1*Ee*2),X=X.slice(0,-1*Ee),te=te.slice(0,-1*Ee)),R.push(this.productions_[Te[1]][0]),X.push(Ge.$),te.push(Ge._$),Ze=H[R[R.length-2]][R[R.length-1]],R.push(Ze);break;case 3:return!0}}return!0}},L=function(){var B={EOF:1,parseError:function(F,R){if(this.yy.parser)this.yy.parser.parseError(F,R);else throw new Error(F)},setInput:function(N,F){return this.yy=F||this.yy||{},this._input=N,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var N=this._input[0];this.yytext+=N,this.yyleng++,this.offset++,this.match+=N,this.matched+=N;var F=N.match(/(?:\r\n?|\n).*/g);return F?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),N},unput:function(N){var F=N.length,R=N.split(/(?:\r\n?|\n)/g);this._input=N+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-F),this.offset-=F;var q=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),R.length-1&&(this.yylineno-=R.length-1);var X=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:R?(R.length===q.length?this.yylloc.first_column:0)+q[q.length-R.length].length-R[0].length:this.yylloc.first_column-F},this.options.ranges&&(this.yylloc.range=[X[0],X[0]+this.yyleng-F]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(N){this.unput(this.match.slice(N))},pastInput:function(){var N=this.matched.substr(0,this.matched.length-this.match.length);return(N.length>20?"...":"")+N.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var N=this.match;return N.length<20&&(N+=this._input.substr(0,20-N.length)),(N.substr(0,20)+(N.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var N=this.pastInput(),F=new Array(N.length+1).join("-");return N+this.upcomingInput()+`
`+F+"^"},test_match:function(N,F){var R,q,X;if(this.options.backtrack_lexer&&(X={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(X.yylloc.range=this.yylloc.range.slice(0))),q=N[0].match(/(?:\r\n?|\n).*/g),q&&(this.yylineno+=q.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:q?q[q.length-1].length-q[q.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+N[0].length},this.yytext+=N[0],this.match+=N[0],this.matches=N,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(N[0].length),this.matched+=N[0],R=this.performAction.call(this,this.yy,this,F,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),R)return R;if(this._backtrack){for(var te in X)this[te]=X[te];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var N,F,R,q;this._more||(this.yytext="",this.match="");for(var X=this._currentRules(),te=0;te<X.length;te++)if(R=this._input.match(this.rules[X[te]]),R&&(!F||R[0].length>F[0].length)){if(F=R,q=te,this.options.backtrack_lexer){if(N=this.test_match(R,X[te]),N!==!1)return N;if(this._backtrack){F=!1;continue}else return!1}else if(!this.options.flex)break}return F?(N=this.test_match(F,X[q]),N!==!1?N:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var F=this.next();return F||this.lex()},begin:function(F){this.conditionStack.push(F)},popState:function(){var F=this.conditionStack.length-1;return F>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(F){return F=this.conditionStack.length-1-Math.abs(F||0),F>=0?this.conditionStack[F]:"INITIAL"},pushState:function(F){this.begin(F)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(F,R,q,X){switch(q){case 0:return this.begin("open_directive"),28;case 1:return this.begin("type_directive"),29;case 2:return this.popState(),this.begin("arg_directive"),15;case 3:return this.popState(),this.popState(),31;case 4:return 30;case 5:break;case 6:break;case 7:return 11;case 8:break;case 9:break;case 10:return 4;case 11:return 17;case 12:return this.begin("acc_title"),18;case 13:return this.popState(),"acc_title_value";case 14:return this.begin("acc_descr"),20;case 15:return this.popState(),"acc_descr_value";case 16:this.begin("acc_descr_multiline");break;case 17:this.popState();break;case 18:return"acc_descr_multiline_value";case 19:return 23;case 20:return 27;case 21:return 26;case 22:return 6;case 23:return"INVALID"}},rules:[/^(?:%%\{)/i,/^(?:((?:(?!\}%%)[^:.])*))/i,/^(?::)/i,/^(?:\}%%)/i,/^(?:((?:(?!\}%%).|\n)*))/i,/^(?:%(?!\{)[^\n]*)/i,/^(?:[^\}]%%[^\n]*)/i,/^(?:[\n]+)/i,/^(?:\s+)/i,/^(?:#[^\n]*)/i,/^(?:timeline\b)/i,/^(?:title\s[^#\n;]+)/i,/^(?:accTitle\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*:\s*)/i,/^(?:(?!\n||)*[^\n]*)/i,/^(?:accDescr\s*\{\s*)/i,/^(?:[\}])/i,/^(?:[^\}]*)/i,/^(?:section\s[^#:\n;]+)/i,/^(?::\s[^#:\n;]+)/i,/^(?:[^#:\n;]+)/i,/^(?:$)/i,/^(?:.)/i],conditions:{open_directive:{rules:[1],inclusive:!1},type_directive:{rules:[2,3],inclusive:!1},arg_directive:{rules:[3,4],inclusive:!1},acc_descr_multiline:{rules:[17,18],inclusive:!1},acc_descr:{rules:[15],inclusive:!1},acc_title:{rules:[13],inclusive:!1},INITIAL:{rules:[0,5,6,7,8,9,10,11,12,14,16,19,20,21,22,23],inclusive:!0}}};return B}();S.lexer=L;function O(){this.yy={}}return O.prototype=S,S.Parser=O,new O}();Fce.parser=Fce;const ptn=Fce;let AC="",ZIe=0;const Rce=[],uU=[],MC=[],JIe=()=>bSe,eOe=(r,i,o)=>{Mae(globalThis,r,i,o)},tOe=function(){Rce.length=0,uU.length=0,AC="",MC.length=0,W0()},nOe=function(r){AC=r,Rce.push(r)},rOe=function(){return Rce},iOe=function(){let r=cOe();const i=100;let o=0;for(;!r&&o<i;)r=cOe(),o++;return uU.push(...MC),uU},sOe=function(r,i,o){const l={id:ZIe++,section:AC,type:AC,task:r,score:i||0,events:o?[o]:[]};MC.push(l)},aOe=function(r){MC.find(o=>o.id===ZIe-1).events.push(r)},oOe=function(r){const i={section:AC,type:AC,description:r,task:r,classes:[]};uU.push(i)},cOe=function(){const r=function(o){return MC[o].processed};let i=!0;for(const[o,l]of MC.entries())r(o),i=i&&l.processed;return i},btn=Object.freeze(Object.defineProperty({__proto__:null,getCommonDb:JIe,parseDirective:eOe,clear:tOe,addSection:nOe,getSections:rOe,getTasks:iOe,addTask:sOe,addEvent:aOe,addTaskOrg:oOe,default:{clear:tOe,getCommonDb:JIe,addSection:nOe,getSections:rOe,getTasks:iOe,addTask:sOe,addTaskOrg:oOe,addEvent:aOe,parseDirective:eOe}},Symbol.toStringTag,{value:"Module"})),vtn=12,lU=function(r,i){const o=r.append("rect");return o.attr("x",i.x),o.attr("y",i.y),o.attr("fill",i.fill),o.attr("stroke",i.stroke),o.attr("width",i.width),o.attr("height",i.height),o.attr("rx",i.rx),o.attr("ry",i.ry),i.class!==void 0&&o.attr("class",i.class),o},wtn=function(r,i){const l=r.append("circle").attr("cx",i.cx).attr("cy",i.cy).attr("class","face").attr("r",15).attr("stroke-width",2).attr("overflow","visible"),f=r.append("g");f.append("circle").attr("cx",i.cx-15/3).attr("cy",i.cy-15/3).attr("r",1.5).attr("stroke-width",2).attr("fill","#666").attr("stroke","#666"),f.append("circle").attr("cx",i.cx+15/3).attr("cy",i.cy-15/3).attr("r",1.5).attr("stroke-width",2).attr("fill","#666").attr("stroke","#666");function b(y){const k=ZL().startAngle(Math.PI/2).endAngle(3*(Math.PI/2)).innerRadius(7.5).outerRadius(6.8181818181818175);y.append("path").attr("class","mouth").attr("d",k).attr("transform","translate("+i.cx+","+(i.cy+2)+")")}function d(y){const k=ZL().startAngle(3*Math.PI/2).endAngle(5*(Math.PI/2)).innerRadius(7.5).outerRadius(6.8181818181818175);y.append("path").attr("class","mouth").attr("d",k).attr("transform","translate("+i.cx+","+(i.cy+7)+")")}function w(y){y.append("line").attr("class","mouth").attr("stroke",2).attr("x1",i.cx-5).attr("y1",i.cy+7).attr("x2",i.cx+5).attr("y2",i.cy+7).attr("class","mouth").attr("stroke-width","1px").attr("stroke","#666")}return i.score>3?b(f):i.score<3?d(f):w(f),l},mtn=function(r,i){const o=r.append("circle");return o.attr("cx",i.cx),o.attr("cy",i.cy),o.attr("class","actor-"+i.pos),o.attr("fill",i.fill),o.attr("stroke",i.stroke),o.attr("r",i.r),o.class!==void 0&&o.attr("class",o.class),i.title!==void 0&&o.append("title").text(i.title),o},uOe=function(r,i){const o=i.text.replace(/<br\s*\/?>/gi," "),l=r.append("text");l.attr("x",i.x),l.attr("y",i.y),l.attr("class","legend"),l.style("text-anchor",i.anchor),i.class!==void 0&&l.attr("class",i.class);const f=l.append("tspan");return f.attr("x",i.x+i.textMargin*2),f.text(o),l},ytn=function(r,i){function o(f,b,d,w,y){return f+","+b+" "+(f+d)+","+b+" "+(f+d)+","+(b+w-y)+" "+(f+d-y*1.2)+","+(b+w)+" "+f+","+(b+w)}const l=r.append("polygon");l.attr("points",o(i.x,i.y,50,20,7)),l.attr("class","labelBox"),i.y=i.y+i.labelMargin,i.x=i.x+.5*i.labelMargin,uOe(r,i)},ktn=function(r,i,o){const l=r.append("g"),f=jce();f.x=i.x,f.y=i.y,f.fill=i.fill,f.width=o.width,f.height=o.height,f.class="journey-section section-type-"+i.num,f.rx=3,f.ry=3,lU(l,f),hOe(o)(i.text,l,f.x,f.y,f.width,f.height,{class:"journey-section section-type-"+i.num},o,i.colour)};let lOe=-1;const xtn=function(r,i,o){const l=i.x+o.width/2,f=r.append("g");lOe++;const b=300+5*30;f.append("line").attr("id","task"+lOe).attr("x1",l).attr("y1",i.y).attr("x2",l).attr("y2",b).attr("class","task-line").attr("stroke-width","1px").attr("stroke-dasharray","4 2").attr("stroke","#666"),wtn(f,{cx:l,cy:300+(5-i.score)*30,score:i.score});const d=jce();d.x=i.x,d.y=i.y,d.fill=i.fill,d.width=o.width,d.height=o.height,d.class="task task-type-"+i.num,d.rx=3,d.ry=3,lU(f,d),i.x+14,hOe(o)(i.task,f,d.x,d.y,d.width,d.height,{class:"task"},o,i.colour)},Etn=function(r,i){lU(r,{x:i.startx,y:i.starty,width:i.stopx-i.startx,height:i.stopy-i.starty,fill:i.fill,class:"rect"}).lower()},_tn=function(){return{x:0,y:0,fill:void 0,"text-anchor":"start",width:100,height:100,textMargin:0,rx:0,ry:0}},jce=function(){return{x:0,y:0,width:100,anchor:"start",height:100,rx:0,ry:0}},hOe=function(){function r(f,b,d,w,y,k,E,T){const C=b.append("text").attr("x",d+y/2).attr("y",w+k/2+5).style("font-color",T).style("text-anchor","middle").text(f);l(C,E)}function i(f,b,d,w,y,k,E,T,C){const{taskFontSize:S,taskFontFamily:L}=T,O=f.split(/<br\s*\/?>/gi);for(let B=0;B<O.length;B++){const N=B*S-S*(O.length-1)/2,F=b.append("text").attr("x",d+y/2).attr("y",w).attr("fill",C).style("text-anchor","middle").style("font-size",S).style("font-family",L);F.append("tspan").attr("x",d+y/2).attr("dy",N).text(O[B]),F.attr("y",w+k/2).attr("dominant-baseline","central").attr("alignment-baseline","central"),l(F,E)}}function o(f,b,d,w,y,k,E,T){const C=b.append("switch"),L=C.append("foreignObject").attr("x",d).attr("y",w).attr("width",y).attr("height",k).attr("position","fixed").append("xhtml:div").style("display","table").style("height","100%").style("width","100%");L.append("div").attr("class","label").style("display","table-cell").style("text-align","center").style("vertical-align","middle").text(f),i(f,C,d,w,y,k,E,T),l(L,E)}function l(f,b){for(const d in b)d in b&&f.attr(d,b[d])}return function(f){return f.textPlacement==="fo"?o:f.textPlacement==="old"?r:i}}(),Ttn=function(r){r.append("defs").append("marker").attr("id","arrowhead").attr("refX",5).attr("refY",2).attr("markerWidth",6).attr("markerHeight",4).attr("orient","auto").append("path").attr("d","M 0,0 V 4 L6,2 Z")};function fOe(r,i){r.each(function(){var o=Cn(this),l=o.text().split(/(\s+|<br>)/).reverse(),f,b=[],d=1.1,w=o.attr("y"),y=parseFloat(o.attr("dy")),k=o.text(null).append("tspan").attr("x",0).attr("y",w).attr("dy",y+"em");for(let E=0;E<l.length;E++)f=l[l.length-1-E],b.push(f),k.text(b.join(" ").trim()),(k.node().getComputedTextLength()>i||f==="<br>")&&(b.pop(),k.text(b.join(" ").trim()),f==="<br>"?b=[""]:b=[f],k=o.append("tspan").attr("x",0).attr("y",w).attr("dy",d+"em").text(f))})}const Ctn=function(r,i,o,l){const f=o%vtn-1,b=r.append("g");i.section=f,b.attr("class",(i.class?i.class+" ":"")+"timeline-node "+("section-"+f));const d=b.append("g"),w=b.append("g"),k=w.append("text").text(i.descr).attr("dy","1em").attr("alignment-baseline","middle").attr("dominant-baseline","middle").attr("text-anchor","middle").call(fOe,i.width).node().getBBox(),E=l.fontSize&&l.fontSize.replace?l.fontSize.replace("px",""):l.fontSize;return i.height=k.height+E*1.1*.5+i.padding,i.height=Math.max(i.height,i.maxHeight),i.width=i.width+2*i.padding,w.attr("transform","translate("+i.width/2+", "+i.padding/2+")"),Atn(d,i,f),i},Stn=function(r,i,o){const l=r.append("g"),b=l.append("text").text(i.descr).attr("dy","1em").attr("alignment-baseline","middle").attr("dominant-baseline","middle").attr("text-anchor","middle").call(fOe,i.width).node().getBBox(),d=o.fontSize&&o.fontSize.replace?o.fontSize.replace("px",""):o.fontSize;return l.remove(),b.height+d*1.1*.5+i.padding},Atn=function(r,i,o){r.append("path").attr("id","node-"+i.id).attr("class","node-bkg node-"+i.type).attr("d",`M0 ${i.height-5} v${-i.height+2*5} q0,-5 5,-5 h${i.width-2*5} q5,0 5,5 v${i.height-5} H0 Z`),r.append("line").attr("class","node-line-"+o).attr("x1",0).attr("y1",i.height).attr("x2",i.width).attr("y2",i.height)},b7={drawRect:lU,drawCircle:mtn,drawSection:ktn,drawText:uOe,drawLabel:ytn,drawTask:xtn,drawBackgroundRect:Etn,getTextObj:_tn,getNoteRect:jce,initGraphics:Ttn,drawNode:Ctn,getVirtualNodeHeight:Stn},Mtn=function(r){Object.keys(r).forEach(function(o){conf[o]=r[o]})},Dtn=function(r,i,o,l){const f=ht(),b=f.leftMargin?f.leftMargin:50;l.db.clear(),l.parser.parse(r+`
`),Se.debug("timeline",l.db);const d=f.securityLevel;let w;d==="sandbox"&&(w=Cn("#i"+i));const k=Cn(d==="sandbox"?w.nodes()[0].contentDocument.body:"body").select("#"+i);k.append("g");const E=l.db.getTasks(),T=l.db.getCommonDb().getDiagramTitle();Se.debug("task",E),b7.initGraphics(k);const C=l.db.getSections();Se.debug("sections",C);let S=0,L=0,O=0,B=0,N=50+b,F=50;B=50;let R=0,q=!0;C.forEach(function(z){const W={number:R,descr:z,section:R,width:150,padding:20,maxHeight:S},Z=b7.getVirtualNodeHeight(k,W,f);Se.debug("sectionHeight before draw",Z),S=Math.max(S,Z+20)});let X=0,te=0;Se.debug("tasks.length",E.length);for(const[z,W]of E.entries()){const Z={number:z,descr:W,section:W.section,width:150,padding:20,maxHeight:L},G=b7.getVirtualNodeHeight(k,Z,f);Se.debug("taskHeight before draw",G),L=Math.max(L,G+20),X=Math.max(X,W.events.length);let ae=0;for(let $=0;$<W.events.length;$++){const ee={descr:W.events[$],section:W.section,number:W.section,width:150,padding:20,maxHeight:50};ae+=b7.getVirtualNodeHeight(k,ee,f)}te=Math.max(te,ae)}Se.debug("maxSectionHeight before draw",S),Se.debug("maxTaskHeight before draw",L),C&&C.length>0?C.forEach(z=>{const W={number:R,descr:z,section:R,width:150,padding:20,maxHeight:S};Se.debug("sectionNode",W);const Z=k.append("g"),G=b7.drawNode(Z,W,R,f);Se.debug("sectionNode output",G),Z.attr("transform",`translate(${N}, ${B})`),F+=S+50;const ae=E.filter($=>$.section===z);ae.length>0&&dOe(k,ae,R,N,F,L,f,X,te,S,!1),N+=200*Math.max(ae.length,1),F=B,R++}):(q=!1,dOe(k,E,R,N,F,L,f,X,te,S,!0));const H=k.node().getBBox();Se.debug("bounds",H),T&&k.append("text").text(T).attr("x",H.width/2-b).attr("font-size","4ex").attr("font-weight","bold").attr("y",20),O=q?S+L+150:L+100,k.append("g").attr("class","lineWrapper").append("line").attr("x1",b).attr("y1",O).attr("x2",H.width+3*b).attr("y2",O).attr("stroke-width",4).attr("stroke","black").attr("marker-end","url(#arrowhead)"),J8(void 0,k,f.timeline.padding?f.timeline.padding:50,f.timeline.useMaxWidth?f.timeline.useMaxWidth:!1)},dOe=function(r,i,o,l,f,b,d,w,y,k,E){for(const T of i){const C={descr:T.task,section:o,number:o,width:150,padding:20,maxHeight:b};Se.debug("taskNode",C);const S=r.append("g").attr("class","taskWrapper"),O=b7.drawNode(S,C,o,d).height;if(Se.debug("taskHeight after draw",O),S.attr("transform",`translate(${l}, ${f})`),b=Math.max(b,O),T.events){const B=r.append("g").attr("class","lineWrapper");let N=b;f+=100,N=N+Ltn(r,T.events,o,l,f,d),f-=100,B.append("line").attr("x1",l+190/2).attr("y1",f+b).attr("x2",l+190/2).attr("y2",f+b+(E?b:k)+y+120).attr("stroke-width",2).attr("stroke","black").attr("marker-end","url(#arrowhead)").attr("stroke-dasharray","5,5")}l=l+200,E&&!ht().timeline.disableMulticolor&&o++}f=f-10},Ltn=function(r,i,o,l,f,b){let d=0;const w=f;f=f+100;for(const y of i){const k={descr:y,section:o,number:o,width:150,padding:20,maxHeight:50};Se.debug("eventNode",k);const E=r.append("g").attr("class","eventWrapper"),C=b7.drawNode(E,k,o,b).height;d=d+C,E.attr("transform",`translate(${l}, ${f})`),f=f+10+C}return f=w,d},Itn={setConf:Mtn,draw:Dtn},Otn=r=>{let i="";for(let o=0;o<r.THEME_COLOR_LIMIT;o++)r["lineColor"+o]=r["lineColor"+o]||r["cScaleInv"+o],fCe(r["lineColor"+o])?r["lineColor"+o]=is(r["lineColor"+o],20):r["lineColor"+o]=Ds(r["lineColor"+o],20);for(let o=0;o<r.THEME_COLOR_LIMIT;o++){const l=""+(17-3*o);i+=`
    .section-${o-1} rect, .section-${o-1} path, .section-${o-1} circle, .section-${o-1} path  {
      fill: ${r["cScale"+o]};
    }
    .section-${o-1} text {
     fill: ${r["cScaleLabel"+o]};
    }
    .node-icon-${o-1} {
      font-size: 40px;
      color: ${r["cScaleLabel"+o]};
    }
    .section-edge-${o-1}{
      stroke: ${r["cScale"+o]};
    }
    .edge-depth-${o-1}{
      stroke-width: ${l};
    }
    .section-${o-1} line {
      stroke: ${r["cScaleInv"+o]} ;
      stroke-width: 3;
    }

    .lineWrapper line{
      stroke: ${r["cScaleLabel"+o]} ;
    }

    .disabled, .disabled circle, .disabled text {
      fill: lightgray;
    }
    .disabled text {
      fill: #efefef;
    }
    `}return i},Ntn=Object.freeze(Object.defineProperty({__proto__:null,diagram:{db:btn,renderer:Itn,parser:ptn,styles:r=>`
  .edge {
    stroke-width: 3;
  }
  ${Otn(r)}
  .section-root rect, .section-root path, .section-root circle  {
    fill: ${r.git0};
  }
  .section-root text {
    fill: ${r.gitBranchLabel0};
  }
  .icon-container {
    height:100%;
    display: flex;
    justify-content: center;
    align-items: center;
  }
  .edge {
    fill: none;
  }
  .eventWrapper  {
   filter: brightness(120%);
  }
`}},Symbol.toStringTag,{value:"Module"}));var $ce=function(){var r=function(q,X,te,H){for(te=te||{},H=q.length;H--;te[q[H]]=X);return te},i=[1,4],o=[1,13],l=[1,12],f=[1,15],b=[1,16],d=[1,20],w=[1,19],y=[6,7,8],k=[1,26],E=[1,24],T=[1,25],C=[6,7,11],S=[1,6,13,15,16,19,22],L=[1,33],O=[1,34],B=[1,6,7,11,13,15,16,19,22],N={trace:function(){},yy:{},symbols_:{error:2,start:3,mindMap:4,spaceLines:5,SPACELINE:6,NL:7,MINDMAP:8,document:9,stop:10,EOF:11,statement:12,SPACELIST:13,node:14,ICON:15,CLASS:16,nodeWithId:17,nodeWithoutId:18,NODE_DSTART:19,NODE_DESCR:20,NODE_DEND:21,NODE_ID:22,$accept:0,$end:1},terminals_:{2:"error",6:"SPACELINE",7:"NL",8:"MINDMAP",11:"EOF",13:"SPACELIST",15:"ICON",16:"CLASS",19:"NODE_DSTART",20:"NODE_DESCR",21:"NODE_DEND",22:"NODE_ID"},productions_:[0,[3,1],[3,2],[5,1],[5,2],[5,2],[4,2],[4,3],[10,1],[10,1],[10,1],[10,2],[10,2],[9,3],[9,2],[12,2],[12,2],[12,2],[12,1],[12,1],[12,1],[12,1],[12,1],[14,1],[14,1],[18,3],[17,1],[17,4]],performAction:function(X,te,H,Y,z,W,Z){var G=W.length-1;switch(z){case 6:case 7:return Y;case 8:Y.getLogger().trace("Stop NL ");break;case 9:Y.getLogger().trace("Stop EOF ");break;case 11:Y.getLogger().trace("Stop NL2 ");break;case 12:Y.getLogger().trace("Stop EOF2 ");break;case 15:Y.getLogger().info("Node: ",W[G].id),Y.addNode(W[G-1].length,W[G].id,W[G].descr,W[G].type);break;case 16:Y.getLogger().trace("Icon: ",W[G]),Y.decorateNode({icon:W[G]});break;case 17:case 21:Y.decorateNode({class:W[G]});break;case 18:Y.getLogger().trace("SPACELIST");break;case 19:Y.getLogger().trace("Node: ",W[G].id),Y.addNode(0,W[G].id,W[G].descr,W[G].type);break;case 20:Y.decorateNode({icon:W[G]});break;case 25:Y.getLogger().trace("node found ..",W[G-2]),this.$={id:W[G-1],descr:W[G-1],type:Y.getType(W[G-2],W[G])};break;case 26:this.$={id:W[G],descr:W[G],type:Y.nodeType.DEFAULT};break;case 27:Y.getLogger().trace("node found ..",W[G-3]),this.$={id:W[G-3],descr:W[G-1],type:Y.getType(W[G-2],W[G])};break}},table:[{3:1,4:2,5:3,6:[1,5],8:i},{1:[3]},{1:[2,1]},{4:6,6:[1,7],7:[1,8],8:i},{6:o,7:[1,10],9:9,12:11,13:l,14:14,15:f,16:b,17:17,18:18,19:d,22:w},r(y,[2,3]),{1:[2,2]},r(y,[2,4]),r(y,[2,5]),{1:[2,6],6:o,12:21,13:l,14:14,15:f,16:b,17:17,18:18,19:d,22:w},{6:o,9:22,12:11,13:l,14:14,15:f,16:b,17:17,18:18,19:d,22:w},{6:k,7:E,10:23,11:T},r(C,[2,22],{17:17,18:18,14:27,15:[1,28],16:[1,29],19:d,22:w}),r(C,[2,18]),r(C,[2,19]),r(C,[2,20]),r(C,[2,21]),r(C,[2,23]),r(C,[2,24]),r(C,[2,26],{19:[1,30]}),{20:[1,31]},{6:k,7:E,10:32,11:T},{1:[2,7],6:o,12:21,13:l,14:14,15:f,16:b,17:17,18:18,19:d,22:w},r(S,[2,14],{7:L,11:O}),r(B,[2,8]),r(B,[2,9]),r(B,[2,10]),r(C,[2,15]),r(C,[2,16]),r(C,[2,17]),{20:[1,35]},{21:[1,36]},r(S,[2,13],{7:L,11:O}),r(B,[2,11]),r(B,[2,12]),{21:[1,37]},r(C,[2,25]),r(C,[2,27])],defaultActions:{2:[2,1],6:[2,2]},parseError:function(X,te){if(te.recoverable)this.trace(X);else{var H=new Error(X);throw H.hash=te,H}},parse:function(X){var te=this,H=[0],Y=[],z=[null],W=[],Z=this.table,G="",ae=0,$=0,ge=2,ee=1,de=W.slice.call(arguments,1),re=Object.create(this.lexer),ke={yy:{}};for(var Ce in this.yy)Object.prototype.hasOwnProperty.call(this.yy,Ce)&&(ke.yy[Ce]=this.yy[Ce]);re.setInput(X,ke.yy),ke.yy.lexer=re,ke.yy.parser=this,typeof re.yylloc>"u"&&(re.yylloc={});var _e=re.yylloc;W.push(_e);var Te=re.options&&re.options.ranges;typeof ke.yy.parseError=="function"?this.parseError=ke.yy.parseError:this.parseError=Object.getPrototypeOf(this).parseError;function Be(){var je;return je=Y.pop()||re.lex()||ee,typeof je!="number"&&(je instanceof Array&&(Y=je,je=Y.pop()),je=te.symbols_[je]||je),je}for(var Ge,Xe,Ee,Ze,Ie={},Oe,Le,$e,xe;;){if(Xe=H[H.length-1],this.defaultActions[Xe]?Ee=this.defaultActions[Xe]:((Ge===null||typeof Ge>"u")&&(Ge=Be()),Ee=Z[Xe]&&Z[Xe][Ge]),typeof Ee>"u"||!Ee.length||!Ee[0]){var Ae="";xe=[];for(Oe in Z[Xe])this.terminals_[Oe]&&Oe>ge&&xe.push("'"+this.terminals_[Oe]+"'");re.showPosition?Ae="Parse error on line "+(ae+1)+`:
`+re.showPosition()+`
Expecting `+xe.join(", ")+", got '"+(this.terminals_[Ge]||Ge)+"'":Ae="Parse error on line "+(ae+1)+": Unexpected "+(Ge==ee?"end of input":"'"+(this.terminals_[Ge]||Ge)+"'"),this.parseError(Ae,{text:re.match,token:this.terminals_[Ge]||Ge,line:re.yylineno,loc:_e,expected:xe})}if(Ee[0]instanceof Array&&Ee.length>1)throw new Error("Parse Error: multiple actions possible at state: "+Xe+", token: "+Ge);switch(Ee[0]){case 1:H.push(Ge),z.push(re.yytext),W.push(re.yylloc),H.push(Ee[1]),Ge=null,$=re.yyleng,G=re.yytext,ae=re.yylineno,_e=re.yylloc;break;case 2:if(Le=this.productions_[Ee[1]][1],Ie.$=z[z.length-Le],Ie._$={first_line:W[W.length-(Le||1)].first_line,last_line:W[W.length-1].last_line,first_column:W[W.length-(Le||1)].first_column,last_column:W[W.length-1].last_column},Te&&(Ie._$.range=[W[W.length-(Le||1)].range[0],W[W.length-1].range[1]]),Ze=this.performAction.apply(Ie,[G,$,ae,ke.yy,Ee[1],z,W].concat(de)),typeof Ze<"u")return Ze;Le&&(H=H.slice(0,-1*Le*2),z=z.slice(0,-1*Le),W=W.slice(0,-1*Le)),H.push(this.productions_[Ee[1]][0]),z.push(Ie.$),W.push(Ie._$),$e=Z[H[H.length-2]][H[H.length-1]],H.push($e);break;case 3:return!0}}return!0}},F=function(){var q={EOF:1,parseError:function(te,H){if(this.yy.parser)this.yy.parser.parseError(te,H);else throw new Error(te)},setInput:function(X,te){return this.yy=te||this.yy||{},this._input=X,this._more=this._backtrack=this.done=!1,this.yylineno=this.yyleng=0,this.yytext=this.matched=this.match="",this.conditionStack=["INITIAL"],this.yylloc={first_line:1,first_column:0,last_line:1,last_column:0},this.options.ranges&&(this.yylloc.range=[0,0]),this.offset=0,this},input:function(){var X=this._input[0];this.yytext+=X,this.yyleng++,this.offset++,this.match+=X,this.matched+=X;var te=X.match(/(?:\r\n?|\n).*/g);return te?(this.yylineno++,this.yylloc.last_line++):this.yylloc.last_column++,this.options.ranges&&this.yylloc.range[1]++,this._input=this._input.slice(1),X},unput:function(X){var te=X.length,H=X.split(/(?:\r\n?|\n)/g);this._input=X+this._input,this.yytext=this.yytext.substr(0,this.yytext.length-te),this.offset-=te;var Y=this.match.split(/(?:\r\n?|\n)/g);this.match=this.match.substr(0,this.match.length-1),this.matched=this.matched.substr(0,this.matched.length-1),H.length-1&&(this.yylineno-=H.length-1);var z=this.yylloc.range;return this.yylloc={first_line:this.yylloc.first_line,last_line:this.yylineno+1,first_column:this.yylloc.first_column,last_column:H?(H.length===Y.length?this.yylloc.first_column:0)+Y[Y.length-H.length].length-H[0].length:this.yylloc.first_column-te},this.options.ranges&&(this.yylloc.range=[z[0],z[0]+this.yyleng-te]),this.yyleng=this.yytext.length,this},more:function(){return this._more=!0,this},reject:function(){if(this.options.backtrack_lexer)this._backtrack=!0;else return this.parseError("Lexical error on line "+(this.yylineno+1)+`. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
`+this.showPosition(),{text:"",token:null,line:this.yylineno});return this},less:function(X){this.unput(this.match.slice(X))},pastInput:function(){var X=this.matched.substr(0,this.matched.length-this.match.length);return(X.length>20?"...":"")+X.substr(-20).replace(/\n/g,"")},upcomingInput:function(){var X=this.match;return X.length<20&&(X+=this._input.substr(0,20-X.length)),(X.substr(0,20)+(X.length>20?"...":"")).replace(/\n/g,"")},showPosition:function(){var X=this.pastInput(),te=new Array(X.length+1).join("-");return X+this.upcomingInput()+`
`+te+"^"},test_match:function(X,te){var H,Y,z;if(this.options.backtrack_lexer&&(z={yylineno:this.yylineno,yylloc:{first_line:this.yylloc.first_line,last_line:this.last_line,first_column:this.yylloc.first_column,last_column:this.yylloc.last_column},yytext:this.yytext,match:this.match,matches:this.matches,matched:this.matched,yyleng:this.yyleng,offset:this.offset,_more:this._more,_input:this._input,yy:this.yy,conditionStack:this.conditionStack.slice(0),done:this.done},this.options.ranges&&(z.yylloc.range=this.yylloc.range.slice(0))),Y=X[0].match(/(?:\r\n?|\n).*/g),Y&&(this.yylineno+=Y.length),this.yylloc={first_line:this.yylloc.last_line,last_line:this.yylineno+1,first_column:this.yylloc.last_column,last_column:Y?Y[Y.length-1].length-Y[Y.length-1].match(/\r?\n?/)[0].length:this.yylloc.last_column+X[0].length},this.yytext+=X[0],this.match+=X[0],this.matches=X,this.yyleng=this.yytext.length,this.options.ranges&&(this.yylloc.range=[this.offset,this.offset+=this.yyleng]),this._more=!1,this._backtrack=!1,this._input=this._input.slice(X[0].length),this.matched+=X[0],H=this.performAction.call(this,this.yy,this,te,this.conditionStack[this.conditionStack.length-1]),this.done&&this._input&&(this.done=!1),H)return H;if(this._backtrack){for(var W in z)this[W]=z[W];return!1}return!1},next:function(){if(this.done)return this.EOF;this._input||(this.done=!0);var X,te,H,Y;this._more||(this.yytext="",this.match="");for(var z=this._currentRules(),W=0;W<z.length;W++)if(H=this._input.match(this.rules[z[W]]),H&&(!te||H[0].length>te[0].length)){if(te=H,Y=W,this.options.backtrack_lexer){if(X=this.test_match(H,z[W]),X!==!1)return X;if(this._backtrack){te=!1;continue}else return!1}else if(!this.options.flex)break}return te?(X=this.test_match(te,z[Y]),X!==!1?X:!1):this._input===""?this.EOF:this.parseError("Lexical error on line "+(this.yylineno+1)+`. Unrecognized text.
`+this.showPosition(),{text:"",token:null,line:this.yylineno})},lex:function(){var te=this.next();return te||this.lex()},begin:function(te){this.conditionStack.push(te)},popState:function(){var te=this.conditionStack.length-1;return te>0?this.conditionStack.pop():this.conditionStack[0]},_currentRules:function(){return this.conditionStack.length&&this.conditionStack[this.conditionStack.length-1]?this.conditions[this.conditionStack[this.conditionStack.length-1]].rules:this.conditions.INITIAL.rules},topState:function(te){return te=this.conditionStack.length-1-Math.abs(te||0),te>=0?this.conditionStack[te]:"INITIAL"},pushState:function(te){this.begin(te)},stateStackSize:function(){return this.conditionStack.length},options:{"case-insensitive":!0},performAction:function(te,H,Y,z){switch(Y){case 0:te.getLogger().trace("Found comment",H.yytext);break;case 1:return 8;case 2:this.begin("CLASS");break;case 3:return this.popState(),16;case 4:this.popState();break;case 5:te.getLogger().trace("Begin icon"),this.begin("ICON");break;case 6:return te.getLogger().trace("SPACELINE"),6;case 7:return 7;case 8:return 15;case 9:te.getLogger().trace("end icon"),this.popState();break;case 10:return te.getLogger().trace("Exploding node"),this.begin("NODE"),19;case 11:return te.getLogger().trace("Cloud"),this.begin("NODE"),19;case 12:return te.getLogger().trace("Explosion Bang"),this.begin("NODE"),19;case 13:return te.getLogger().trace("Cloud Bang"),this.begin("NODE"),19;case 14:return this.begin("NODE"),19;case 15:return this.begin("NODE"),19;case 16:return this.begin("NODE"),19;case 17:return this.begin("NODE"),19;case 18:return 13;case 19:return 22;case 20:return 11;case 21:te.getLogger().trace("Starting NSTR"),this.begin("NSTR");break;case 22:return te.getLogger().trace("description:",H.yytext),"NODE_DESCR";case 23:this.popState();break;case 24:return this.popState(),te.getLogger().trace("node end ))"),"NODE_DEND";case 25:return this.popState(),te.getLogger().trace("node end )"),"NODE_DEND";case 26:return this.popState(),te.getLogger().trace("node end ...",H.yytext),"NODE_DEND";case 27:return this.popState(),te.getLogger().trace("node end (("),"NODE_DEND";case 28:return this.popState(),te.getLogger().trace("node end (-"),"NODE_DEND";case 29:return this.popState(),te.getLogger().trace("node end (-"),"NODE_DEND";case 30:return this.popState(),te.getLogger().trace("node end (("),"NODE_DEND";case 31:return this.popState(),te.getLogger().trace("node end (("),"NODE_DEND";case 32:return te.getLogger().trace("Long description:",H.yytext),20;case 33:return te.getLogger().trace("Long description:",H.yytext),20}},rules:[/^(?:\s*%%.*)/i,/^(?:mindmap\b)/i,/^(?::::)/i,/^(?:.+)/i,/^(?:\n)/i,/^(?:::icon\()/i,/^(?:[\s]+[\n])/i,/^(?:[\n]+)/i,/^(?:[^\)]+)/i,/^(?:\))/i,/^(?:-\))/i,/^(?:\(-)/i,/^(?:\)\))/i,/^(?:\))/i,/^(?:\(\()/i,/^(?:\{\{)/i,/^(?:\()/i,/^(?:\[)/i,/^(?:[\s]+)/i,/^(?:[^\(\[\n\-\)\{\}]+)/i,/^(?:$)/i,/^(?:["])/i,/^(?:[^"]+)/i,/^(?:["])/i,/^(?:[\)]\))/i,/^(?:[\)])/i,/^(?:[\]])/i,/^(?:\}\})/i,/^(?:\(-)/i,/^(?:-\))/i,/^(?:\(\()/i,/^(?:\()/i,/^(?:[^\)\]\(\}]+)/i,/^(?:.+(?!\(\())/i],conditions:{CLASS:{rules:[3,4],inclusive:!1},ICON:{rules:[8,9],inclusive:!1},NSTR:{rules:[22,23],inclusive:!1},NODE:{rules:[21,24,25,26,27,28,29,30,31,32,33],inclusive:!1},INITIAL:{rules:[0,1,2,5,6,7,10,11,12,13,14,15,16,17,18,19,20],inclusive:!0}}};return q}();N.lexer=F;function R(){this.yy={}}return R.prototype=N,N.Parser=R,new R}();$ce.parser=$ce;const Ptn=$ce,sO=r=>up(r,ht());let R2=[],gOe=0,Hce={};const Btn=()=>{R2=[],gOe=0,Hce={}},Ftn=function(r){for(let i=R2.length-1;i>=0;i--)if(R2[i].level<r)return R2[i];return null},Rtn=()=>R2.length>0?R2[0]:null,jtn=(r,i,o,l)=>{Se.info("addNode",r,i,o,l);const f=ht(),b={id:gOe++,nodeId:sO(i),level:r,descr:sO(o),type:l,children:[],width:ht().mindmap.maxNodeWidth};switch(b.type){case Uc.ROUNDED_RECT:b.padding=2*f.mindmap.padding;break;case Uc.RECT:b.padding=2*f.mindmap.padding;break;case Uc.HEXAGON:b.padding=2*f.mindmap.padding;break;default:b.padding=f.mindmap.padding}const d=Ftn(r);if(d)d.children.push(b),R2.push(b);else if(R2.length===0)R2.push(b);else{let w=new Error('There can be only one root. No parent could be found for ("'+b.descr+'")');throw w.hash={text:"branch "+name,token:"branch "+name,line:"1",loc:{first_line:1,last_line:1,first_column:1,last_column:1},expected:['"checkout '+name+'"']},w}},Uc={DEFAULT:0,NO_BORDER:0,ROUNDED_RECT:1,RECT:2,CIRCLE:3,CLOUD:4,BANG:5,HEXAGON:6},$tn=(r,i)=>{switch(Se.debug("In get type",r,i),r){case"[":return Uc.RECT;case"(":return i===")"?Uc.ROUNDED_RECT:Uc.CLOUD;case"((":return Uc.CIRCLE;case")":return Uc.CLOUD;case"))":return Uc.BANG;case"{{":return Uc.HEXAGON;default:return Uc.DEFAULT}},pOe=(r,i)=>{Hce[r]=i},Htn=r=>{const i=R2[R2.length-1];r&&r.icon&&(i.icon=sO(r.icon)),r&&r.class&&(i.class=sO(r.class))},v7=r=>{switch(r){case Uc.DEFAULT:return"no-border";case Uc.RECT:return"rect";case Uc.ROUNDED_RECT:return"rounded-rect";case Uc.CIRCLE:return"circle";case Uc.CLOUD:return"cloud";case Uc.BANG:return"bang";case Uc.HEXAGON:return"hexgon";default:return"no-border"}};let bOe;const ztn=r=>{bOe=r},Gtn=()=>Se,Vtn=r=>R2[r],zce=r=>Hce[r],Utn=Object.freeze(Object.defineProperty({__proto__:null,sanitizeText:sO,clear:Btn,getMindmap:Rtn,addNode:jtn,nodeType:Uc,getType:$tn,setElementForId:pOe,decorateNode:Htn,type2Str:v7,get parseError(){return bOe},setErrorHandler:ztn,getLogger:Gtn,getNodeById:Vtn,getElementById:zce},Symbol.toStringTag,{value:"Module"})),vOe=12;function qtn(r,i){r.each(function(){var o=Cn(this),l=o.text().split(/(\s+|<br>)/).reverse(),f,b=[],d=1.1,w=o.attr("y"),y=parseFloat(o.attr("dy")),k=o.text(null).append("tspan").attr("x",0).attr("y",w).attr("dy",y+"em");for(let E=0;E<l.length;E++)f=l[l.length-1-E],b.push(f),k.text(b.join(" ").trim()),(k.node().getComputedTextLength()>i||f==="<br>")&&(b.pop(),k.text(b.join(" ").trim()),f==="<br>"?b=[""]:b=[f],k=o.append("tspan").attr("x",0).attr("y",w).attr("dy",d+"em").text(f))})}const Ytn=function(r,i,o){r.append("path").attr("id","node-"+i.id).attr("class","node-bkg node-"+v7(i.type)).attr("d",`M0 ${i.height-5} v${-i.height+2*5} q0,-5 5,-5 h${i.width-2*5} q5,0 5,5 v${i.height-5} H0 Z`),r.append("line").attr("class","node-line-"+o).attr("x1",0).attr("y1",i.height).attr("x2",i.width).attr("y2",i.height)},Wtn=function(r,i){r.append("rect").attr("id","node-"+i.id).attr("class","node-bkg node-"+v7(i.type)).attr("height",i.height).attr("width",i.width)},Ktn=function(r,i){const o=i.width,l=i.height,f=.15*o,b=.25*o,d=.35*o,w=.2*o;r.append("path").attr("id","node-"+i.id).attr("class","node-bkg node-"+v7(i.type)).attr("d",`M0 0 a${f},${f} 0 0,1 ${o*.25},${-1*o*.1}
      a${d},${d} 1 0,1 ${o*.4},${-1*o*.1}
      a${b},${b} 1 0,1 ${o*.35},${1*o*.2}

      a${f},${f} 1 0,1 ${o*.15},${1*l*.35}
      a${w},${w} 1 0,1 ${-1*o*.15},${1*l*.65}

      a${b},${f} 1 0,1 ${-1*o*.25},${o*.15}
      a${d},${d} 1 0,1 ${-1*o*.5},${0}
      a${f},${f} 1 0,1 ${-1*o*.25},${-1*o*.15}

      a${f},${f} 1 0,1 ${-1*o*.1},${-1*l*.35}
      a${w},${w} 1 0,1 ${o*.1},${-1*l*.65}

    H0 V0 Z`)},Xtn=function(r,i){const o=i.width,l=i.height,f=.15*o;r.append("path").attr("id","node-"+i.id).attr("class","node-bkg node-"+v7(i.type)).attr("d",`M0 0 a${f},${f} 1 0,0 ${o*.25},${-1*l*.1}
      a${f},${f} 1 0,0 ${o*.25},${0}
      a${f},${f} 1 0,0 ${o*.25},${0}
      a${f},${f} 1 0,0 ${o*.25},${1*l*.1}

      a${f},${f} 1 0,0 ${o*.15},${1*l*.33}
      a${f*.8},${f*.8} 1 0,0 ${0},${1*l*.34}
      a${f},${f} 1 0,0 ${-1*o*.15},${1*l*.33}

      a${f},${f} 1 0,0 ${-1*o*.25},${l*.15}
      a${f},${f} 1 0,0 ${-1*o*.25},${0}
      a${f},${f} 1 0,0 ${-1*o*.25},${0}
      a${f},${f} 1 0,0 ${-1*o*.25},${-1*l*.15}

      a${f},${f} 1 0,0 ${-1*o*.1},${-1*l*.33}
      a${f*.8},${f*.8} 1 0,0 ${0},${-1*l*.34}
      a${f},${f} 1 0,0 ${o*.1},${-1*l*.33}

    H0 V0 Z`)},Qtn=function(r,i){r.append("circle").attr("id","node-"+i.id).attr("class","node-bkg node-"+v7(i.type)).attr("r",i.width/2)};function Ztn(r,i,o,l,f){return r.insert("polygon",":first-child").attr("points",l.map(function(b){return b.x+","+b.y}).join(" ")).attr("transform","translate("+(f.width-i)/2+", "+o+")")}const Jtn=function(r,i){const o=i.height,f=o/4,b=i.width-i.padding+2*f,d=[{x:f,y:0},{x:b-f,y:0},{x:b,y:-o/2},{x:b-f,y:-o},{x:f,y:-o},{x:0,y:-o/2}];Ztn(r,b,o,d,i)},enn=function(r,i){r.append("rect").attr("id","node-"+i.id).attr("class","node-bkg node-"+v7(i.type)).attr("height",i.height).attr("rx",i.padding).attr("ry",i.padding).attr("width",i.width)},wOe={drawNode:function(r,i,o,l){const f=o%(vOe-1),b=r.append("g");i.section=f;let d="section-"+f;f<0&&(d+=" section-root"),b.attr("class",(i.class?i.class+" ":"")+"mindmap-node "+d);const w=b.append("g"),y=b.append("g"),E=y.append("text").text(i.descr).attr("dy","1em").attr("alignment-baseline","middle").attr("dominant-baseline","middle").attr("text-anchor","middle").call(qtn,i.width).node().getBBox(),T=l.fontSize.replace?l.fontSize.replace("px",""):l.fontSize;if(i.height=E.height+T*1.1*.5+i.padding,i.width=E.width+2*i.padding,i.icon)if(i.type===Uc.CIRCLE)i.height+=50,i.width+=50,b.append("foreignObject").attr("height","50px").attr("width",i.width).attr("style","text-align: center;").append("div").attr("class","icon-container").append("i").attr("class","node-icon-"+f+" "+i.icon),y.attr("transform","translate("+i.width/2+", "+(i.height/2-1.5*i.padding)+")");else{i.width+=50;const C=i.height;i.height=Math.max(C,60);const S=Math.abs(i.height-C);b.append("foreignObject").attr("width","60px").attr("height",i.height).attr("style","text-align: center;margin-top:"+S/2+"px;").append("div").attr("class","icon-container").append("i").attr("class","node-icon-"+f+" "+i.icon),y.attr("transform","translate("+(25+i.width/2)+", "+(S/2+i.padding/2)+")")}else y.attr("transform","translate("+i.width/2+", "+i.padding/2+")");switch(i.type){case Uc.DEFAULT:Ytn(w,i,f);break;case Uc.ROUNDED_RECT:enn(w,i);break;case Uc.RECT:Wtn(w,i);break;case Uc.CIRCLE:w.attr("transform","translate("+i.width/2+", "+ +i.height/2+")"),Qtn(w,i);break;case Uc.CLOUD:Ktn(w,i);break;case Uc.BANG:Xtn(w,i);break;case Uc.HEXAGON:Jtn(w,i);break}return pOe(i.id,b),i.height},positionNode:function(r){const i=zce(r.id),o=r.x||0,l=r.y||0;i.attr("transform","translate("+o+","+l+")")},drawEdge:function(i,o,l,f,b){const d=b%(vOe-1),w=l.x+l.width/2,y=l.y+l.height/2,k=o.x+o.width/2,E=o.y+o.height/2,T=k>w?w+Math.abs(w-k)/2:w-Math.abs(w-k)/2,C=E>y?y+Math.abs(y-E)/2:y-Math.abs(y-E)/2,S=k>w?Math.abs(w-T)/2+w:-Math.abs(w-T)/2+w,L=E>y?Math.abs(y-C)/2+y:-Math.abs(y-C)/2+y;i.append("path").attr("d",l.direction==="TB"||l.direction==="BT"?`M${w},${y} Q${w},${L} ${T},${C} T${k},${E}`:`M${w},${y} Q${S},${y} ${T},${C} T${k},${E}`).attr("class","edge section-edge-"+d+" edge-depth-"+f)}};function tnn(r){var i=typeof r;return r!=null&&(i=="object"||i=="function")}var aO=tnn,nnn=typeof Mm=="object"&&Mm&&Mm.Object===Object&&Mm,rnn=nnn,inn=rnn,snn=typeof self=="object"&&self&&self.Object===Object&&self,ann=inn||snn||Function("return this")(),hU=ann,onn=hU,cnn=function(){return onn.Date.now()},unn=cnn,lnn=/\s/;function hnn(r){for(var i=r.length;i--&&lnn.test(r.charAt(i)););return i}var fnn=hnn,dnn=fnn,gnn=/^\s+/;function pnn(r){return r&&r.slice(0,dnn(r)+1).replace(gnn,"")}var bnn=pnn,vnn=hU,wnn=vnn.Symbol,Gce=wnn,mOe=Gce,yOe=Object.prototype,mnn=yOe.hasOwnProperty,ynn=yOe.toString,oO=mOe?mOe.toStringTag:void 0;function knn(r){var i=mnn.call(r,oO),o=r[oO];try{r[oO]=void 0;var l=!0}catch{}var f=ynn.call(r);return l&&(i?r[oO]=o:delete r[oO]),f}var xnn=knn,Enn=Object.prototype,_nn=Enn.toString;function Tnn(r){return _nn.call(r)}var Cnn=Tnn,kOe=Gce,Snn=xnn,Ann=Cnn,Mnn="[object Null]",Dnn="[object Undefined]",xOe=kOe?kOe.toStringTag:void 0;function Lnn(r){return r==null?r===void 0?Dnn:Mnn:xOe&&xOe in Object(r)?Snn(r):Ann(r)}var EOe=Lnn;function Inn(r){return r!=null&&typeof r=="object"}var Onn=Inn,Nnn=EOe,Pnn=Onn,Bnn="[object Symbol]";function Fnn(r){return typeof r=="symbol"||Pnn(r)&&Nnn(r)==Bnn}var cO=Fnn,Rnn=bnn,_Oe=aO,jnn=cO,TOe=0/0,$nn=/^[-+]0x[0-9a-f]+$/i,Hnn=/^0b[01]+$/i,znn=/^0o[0-7]+$/i,Gnn=parseInt;function Vnn(r){if(typeof r=="number")return r;if(jnn(r))return TOe;if(_Oe(r)){var i=typeof r.valueOf=="function"?r.valueOf():r;r=_Oe(i)?i+"":i}if(typeof r!="string")return r===0?r:+r;r=Rnn(r);var o=Hnn.test(r);return o||znn.test(r)?Gnn(r.slice(2),o?2:8):$nn.test(r)?TOe:+r}var Unn=Vnn,qnn=aO,Vce=unn,COe=Unn,Ynn="Expected a function",Wnn=Math.max,Knn=Math.min;function Xnn(r,i,o){var l,f,b,d,w,y,k=0,E=!1,T=!1,C=!0;if(typeof r!="function")throw new TypeError(Ynn);i=COe(i)||0,qnn(o)&&(E=!!o.leading,T="maxWait"in o,b=T?Wnn(COe(o.maxWait)||0,i):b,C="trailing"in o?!!o.trailing:C);function S(te){var H=l,Y=f;return l=f=void 0,k=te,d=r.apply(Y,H),d}function L(te){return k=te,w=setTimeout(N,i),E?S(te):d}function O(te){var H=te-y,Y=te-k,z=i-H;return T?Knn(z,b-Y):z}function B(te){var H=te-y,Y=te-k;return y===void 0||H>=i||H<0||T&&Y>=b}function N(){var te=Vce();if(B(te))return F(te);w=setTimeout(N,O(te))}function F(te){return w=void 0,C&&l?S(te):(l=f=void 0,d)}function R(){w!==void 0&&clearTimeout(w),k=0,l=y=f=w=void 0}function q(){return w===void 0?d:F(Vce())}function X(){var te=Vce(),H=B(te);if(l=arguments,f=this,y=te,H){if(w===void 0)return L(y);if(T)return clearTimeout(w),w=setTimeout(N,i),S(y)}return w===void 0&&(w=setTimeout(N,i)),d}return X.cancel=R,X.flush=q,X}var Qnn=Xnn,SOe={exports:{}},AOe={exports:{}};(function(r,i){(function(){var o,l,f,b,d,w,y,k,E,T,C,S,L,O,B;f=Math.floor,T=Math.min,l=function(N,F){return N<F?-1:N>F?1:0},E=function(N,F,R,q,X){var te;if(R==null&&(R=0),X==null&&(X=l),R<0)throw new Error("lo must be non-negative");for(q==null&&(q=N.length);R<q;)te=f((R+q)/2),X(F,N[te])<0?q=te:R=te+1;return[].splice.apply(N,[R,R-R].concat(F)),F},w=function(N,F,R){return R==null&&(R=l),N.push(F),O(N,0,N.length-1,R)},d=function(N,F){var R,q;return F==null&&(F=l),R=N.pop(),N.length?(q=N[0],N[0]=R,B(N,0,F)):q=R,q},k=function(N,F,R){var q;return R==null&&(R=l),q=N[0],N[0]=F,B(N,0,R),q},y=function(N,F,R){var q;return R==null&&(R=l),N.length&&R(N[0],F)<0&&(q=[N[0],F],F=q[0],N[0]=q[1],B(N,0,R)),F},b=function(N,F){var R,q,X,te,H,Y;for(F==null&&(F=l),te=function(){Y=[];for(var z=0,W=f(N.length/2);0<=W?z<W:z>W;0<=W?z++:z--)Y.push(z);return Y}.apply(this).reverse(),H=[],q=0,X=te.length;q<X;q++)R=te[q],H.push(B(N,R,F));return H},L=function(N,F,R){var q;if(R==null&&(R=l),q=N.indexOf(F),q!==-1)return O(N,0,q,R),B(N,q,R)},C=function(N,F,R){var q,X,te,H,Y;if(R==null&&(R=l),X=N.slice(0,F),!X.length)return X;for(b(X,R),Y=N.slice(F),te=0,H=Y.length;te<H;te++)q=Y[te],y(X,q,R);return X.sort(R).reverse()},S=function(N,F,R){var q,X,te,H,Y,z,W,Z,G;if(R==null&&(R=l),F*10<=N.length){if(te=N.slice(0,F).sort(R),!te.length)return te;for(X=te[te.length-1],W=N.slice(F),H=0,z=W.length;H<z;H++)q=W[H],R(q,X)<0&&(E(te,q,0,null,R),te.pop(),X=te[te.length-1]);return te}for(b(N,R),G=[],Y=0,Z=T(F,N.length);0<=Z?Y<Z:Y>Z;0<=Z?++Y:--Y)G.push(d(N,R));return G},O=function(N,F,R,q){var X,te,H;for(q==null&&(q=l),X=N[R];R>F;){if(H=R-1>>1,te=N[H],q(X,te)<0){N[R]=te,R=H;continue}break}return N[R]=X},B=function(N,F,R){var q,X,te,H,Y;for(R==null&&(R=l),X=N.length,Y=F,te=N[F],q=2*F+1;q<X;)H=q+1,H<X&&!(R(N[q],N[H])<0)&&(q=H),N[F]=N[q],F=q,q=2*F+1;return N[F]=te,O(N,Y,F,R)},o=function(){N.push=w,N.pop=d,N.replace=k,N.pushpop=y,N.heapify=b,N.updateItem=L,N.nlargest=C,N.nsmallest=S;function N(F){this.cmp=F!=null?F:l,this.nodes=[]}return N.prototype.push=function(F){return w(this.nodes,F,this.cmp)},N.prototype.pop=function(){return d(this.nodes,this.cmp)},N.prototype.peek=function(){return this.nodes[0]},N.prototype.contains=function(F){return this.nodes.indexOf(F)!==-1},N.prototype.replace=function(F){return k(this.nodes,F,this.cmp)},N.prototype.pushpop=function(F){return y(this.nodes,F,this.cmp)},N.prototype.heapify=function(){return b(this.nodes,this.cmp)},N.prototype.updateItem=function(F){return L(this.nodes,F,this.cmp)},N.prototype.clear=function(){return this.nodes=[]},N.prototype.empty=function(){return this.nodes.length===0},N.prototype.size=function(){return this.nodes.length},N.prototype.clone=function(){var F;return F=new N,F.nodes=this.nodes.slice(0),F},N.prototype.toArray=function(){return this.nodes.slice(0)},N.prototype.insert=N.prototype.push,N.prototype.top=N.prototype.peek,N.prototype.front=N.prototype.peek,N.prototype.has=N.prototype.contains,N.prototype.copy=N.prototype.clone,N}(),function(N,F){return r.exports=F()}(this,function(){return o})}).call(Mm)})(AOe),function(r){r.exports=AOe.exports}(SOe);var Znn=Array.isArray,fU=Znn,Jnn=fU,ern=cO,trn=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,nrn=/^\w*$/;function rrn(r,i){if(Jnn(r))return!1;var o=typeof r;return o=="number"||o=="symbol"||o=="boolean"||r==null||ern(r)?!0:nrn.test(r)||!trn.test(r)||i!=null&&r in Object(i)}var irn=rrn,srn=EOe,arn=aO,orn="[object AsyncFunction]",crn="[object Function]",urn="[object GeneratorFunction]",lrn="[object Proxy]";function hrn(r){if(!arn(r))return!1;var i=srn(r);return i==crn||i==urn||i==orn||i==lrn}var frn=hrn,drn=hU,grn=drn["__core-js_shared__"],prn=grn,Uce=prn,MOe=function(){var r=/[^.]+$/.exec(Uce&&Uce.keys&&Uce.keys.IE_PROTO||"");return r?"Symbol(src)_1."+r:""}();function brn(r){return!!MOe&&MOe in r}var vrn=brn,wrn=Function.prototype,mrn=wrn.toString;function yrn(r){if(r!=null){try{return mrn.call(r)}catch{}try{return r+""}catch{}}return""}var krn=yrn,xrn=frn,Ern=vrn,_rn=aO,Trn=krn,Crn=/[\\^$.*+?()[\]{}|]/g,Srn=/^\[object .+?Constructor\]$/,Arn=Function.prototype,Mrn=Object.prototype,Drn=Arn.toString,Lrn=Mrn.hasOwnProperty,Irn=RegExp("^"+Drn.call(Lrn).replace(Crn,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function Orn(r){if(!_rn(r)||Ern(r))return!1;var i=xrn(r)?Irn:Srn;return i.test(Trn(r))}var Nrn=Orn;function Prn(r,i){return r==null?void 0:r[i]}var Brn=Prn,Frn=Nrn,Rrn=Brn;function jrn(r,i){var o=Rrn(r,i);return Frn(o)?o:void 0}var qce=jrn,$rn=qce,Hrn=$rn(Object,"create"),dU=Hrn,DOe=dU;function zrn(){this.__data__=DOe?DOe(null):{},this.size=0}var Grn=zrn;function Vrn(r){var i=this.has(r)&&delete this.__data__[r];return this.size-=i?1:0,i}var Urn=Vrn,qrn=dU,Yrn="__lodash_hash_undefined__",Wrn=Object.prototype,Krn=Wrn.hasOwnProperty;function Xrn(r){var i=this.__data__;if(qrn){var o=i[r];return o===Yrn?void 0:o}return Krn.call(i,r)?i[r]:void 0}var Qrn=Xrn,Zrn=dU,Jrn=Object.prototype,ein=Jrn.hasOwnProperty;function tin(r){var i=this.__data__;return Zrn?i[r]!==void 0:ein.call(i,r)}var nin=tin,rin=dU,iin="__lodash_hash_undefined__";function sin(r,i){var o=this.__data__;return this.size+=this.has(r)?0:1,o[r]=rin&&i===void 0?iin:i,this}var ain=sin,oin=Grn,cin=Urn,uin=Qrn,lin=nin,hin=ain;function DC(r){var i=-1,o=r==null?0:r.length;for(this.clear();++i<o;){var l=r[i];this.set(l[0],l[1])}}DC.prototype.clear=oin,DC.prototype.delete=cin,DC.prototype.get=uin,DC.prototype.has=lin,DC.prototype.set=hin;var fin=DC;function din(){this.__data__=[],this.size=0}var gin=din;function pin(r,i){return r===i||r!==r&&i!==i}var LOe=pin,bin=LOe;function vin(r,i){for(var o=r.length;o--;)if(bin(r[o][0],i))return o;return-1}var gU=vin,win=gU,min=Array.prototype,yin=min.splice;function kin(r){var i=this.__data__,o=win(i,r);if(o<0)return!1;var l=i.length-1;return o==l?i.pop():yin.call(i,o,1),--this.size,!0}var xin=kin,Ein=gU;function _in(r){var i=this.__data__,o=Ein(i,r);return o<0?void 0:i[o][1]}var Tin=_in,Cin=gU;function Sin(r){return Cin(this.__data__,r)>-1}var Ain=Sin,Min=gU;function Din(r,i){var o=this.__data__,l=Min(o,r);return l<0?(++this.size,o.push([r,i])):o[l][1]=i,this}var Lin=Din,Iin=gin,Oin=xin,Nin=Tin,Pin=Ain,Bin=Lin;function LC(r){var i=-1,o=r==null?0:r.length;for(this.clear();++i<o;){var l=r[i];this.set(l[0],l[1])}}LC.prototype.clear=Iin,LC.prototype.delete=Oin,LC.prototype.get=Nin,LC.prototype.has=Pin,LC.prototype.set=Bin;var Fin=LC,Rin=qce,jin=hU,$in=Rin(jin,"Map"),Hin=$in,IOe=fin,zin=Fin,Gin=Hin;function Vin(){this.size=0,this.__data__={hash:new IOe,map:new(Gin||zin),string:new IOe}}var Uin=Vin;function qin(r){var i=typeof r;return i=="string"||i=="number"||i=="symbol"||i=="boolean"?r!=="__proto__":r===null}var Yin=qin,Win=Yin;function Kin(r,i){var o=r.__data__;return Win(i)?o[typeof i=="string"?"string":"hash"]:o.map}var pU=Kin,Xin=pU;function Qin(r){var i=Xin(this,r).delete(r);return this.size-=i?1:0,i}var Zin=Qin,Jin=pU;function esn(r){return Jin(this,r).get(r)}var tsn=esn,nsn=pU;function rsn(r){return nsn(this,r).has(r)}var isn=rsn,ssn=pU;function asn(r,i){var o=ssn(this,r),l=o.size;return o.set(r,i),this.size+=o.size==l?0:1,this}var osn=asn,csn=Uin,usn=Zin,lsn=tsn,hsn=isn,fsn=osn;function IC(r){var i=-1,o=r==null?0:r.length;for(this.clear();++i<o;){var l=r[i];this.set(l[0],l[1])}}IC.prototype.clear=csn,IC.prototype.delete=usn,IC.prototype.get=lsn,IC.prototype.has=hsn,IC.prototype.set=fsn;var dsn=IC,OOe=dsn,gsn="Expected a function";function Yce(r,i){if(typeof r!="function"||i!=null&&typeof i!="function")throw new TypeError(gsn);var o=function(){var l=arguments,f=i?i.apply(this,l):l[0],b=o.cache;if(b.has(f))return b.get(f);var d=r.apply(this,l);return o.cache=b.set(f,d)||b,d};return o.cache=new(Yce.Cache||OOe),o}Yce.Cache=OOe;var psn=Yce,bsn=psn,vsn=500;function wsn(r){var i=bsn(r,function(l){return o.size===vsn&&o.clear(),l}),o=i.cache;return i}var msn=wsn,ysn=msn,ksn=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,xsn=/\\(\\)?/g,Esn=ysn(function(r){var i=[];return r.charCodeAt(0)===46&&i.push(""),r.replace(ksn,function(o,l,f,b){i.push(f?b.replace(xsn,"$1"):l||o)}),i}),NOe=Esn;function _sn(r,i){for(var o=-1,l=r==null?0:r.length,f=Array(l);++o<l;)f[o]=i(r[o],o,r);return f}var POe=_sn,BOe=Gce,Tsn=POe,Csn=fU,Ssn=cO,Asn=1/0,FOe=BOe?BOe.prototype:void 0,ROe=FOe?FOe.toString:void 0;function jOe(r){if(typeof r=="string")return r;if(Csn(r))return Tsn(r,jOe)+"";if(Ssn(r))return ROe?ROe.call(r):"";var i=r+"";return i=="0"&&1/r==-Asn?"-0":i}var Msn=jOe,Dsn=Msn;function Lsn(r){return r==null?"":Dsn(r)}var $Oe=Lsn,Isn=fU,Osn=irn,Nsn=NOe,Psn=$Oe;function Bsn(r,i){return Isn(r)?r:Osn(r,i)?[r]:Nsn(Psn(r))}var HOe=Bsn,Fsn=cO,Rsn=1/0;function jsn(r){if(typeof r=="string"||Fsn(r))return r;var i=r+"";return i=="0"&&1/r==-Rsn?"-0":i}var Wce=jsn,$sn=HOe,Hsn=Wce;function zsn(r,i){i=$sn(i,r);for(var o=0,l=i.length;r!=null&&o<l;)r=r[Hsn(i[o++])];return o&&o==l?r:void 0}var Gsn=zsn,Vsn=Gsn;function Usn(r,i,o){var l=r==null?void 0:Vsn(r,i);return l===void 0?o:l}var qsn=Usn,Ysn=qce,Wsn=function(){try{var r=Ysn(Object,"defineProperty");return r({},"",{}),r}catch{}}(),Ksn=Wsn,zOe=Ksn;function Xsn(r,i,o){i=="__proto__"&&zOe?zOe(r,i,{configurable:!0,enumerable:!0,value:o,writable:!0}):r[i]=o}var Qsn=Xsn,Zsn=Qsn,Jsn=LOe,ean=Object.prototype,tan=ean.hasOwnProperty;function nan(r,i,o){var l=r[i];(!(tan.call(r,i)&&Jsn(l,o))||o===void 0&&!(i in r))&&Zsn(r,i,o)}var ran=nan,ian=9007199254740991,san=/^(?:0|[1-9]\d*)$/;function aan(r,i){var o=typeof r;return i=i==null?ian:i,!!i&&(o=="number"||o!="symbol"&&san.test(r))&&r>-1&&r%1==0&&r<i}var oan=aan,can=ran,uan=HOe,lan=oan,GOe=aO,han=Wce;function fan(r,i,o,l){if(!GOe(r))return r;i=uan(i,r);for(var f=-1,b=i.length,d=b-1,w=r;w!=null&&++f<b;){var y=han(i[f]),k=o;if(y==="__proto__"||y==="constructor"||y==="prototype")return r;if(f!=d){var E=w[y];k=l?l(E,y,w):void 0,k===void 0&&(k=GOe(E)?E:lan(i[f+1])?[]:{})}can(w,y,k),w=w[y]}return r}var dan=fan,gan=dan;function pan(r,i,o){return r==null?r:gan(r,i,o)}var ban=pan;function van(r,i){var o=-1,l=r.length;for(i||(i=Array(l));++o<l;)i[o]=r[o];return i}var wan=van,man=POe,yan=wan,kan=fU,xan=cO,Ean=NOe,_an=Wce,Tan=$Oe;function Can(r){return kan(r)?man(r,_an):xan(r)?[r]:yan(Ean(Tan(r)))}var San=Can,Aan=Qnn,Man=SOe.exports,Dan=qsn,Lan=ban,Ian=San;function uO(r){return r&&typeof r=="object"&&"default"in r?r:{default:r}}var bU=uO(Aan),lO=uO(Man),Oan=uO(Dan),Nan=uO(Lan),Pan=uO(Ian);function df(r){return df=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(i){return typeof i}:function(i){return i&&typeof Symbol=="function"&&i.constructor===Symbol&&i!==Symbol.prototype?"symbol":typeof i},df(r)}function Kce(r,i){if(!(r instanceof i))throw new TypeError("Cannot call a class as a function")}function VOe(r,i){for(var o=0;o<i.length;o++){var l=i[o];l.enumerable=l.enumerable||!1,l.configurable=!0,"value"in l&&(l.writable=!0),Object.defineProperty(r,l.key,l)}}function Xce(r,i,o){return i&&VOe(r.prototype,i),o&&VOe(r,o),Object.defineProperty(r,"prototype",{writable:!1}),r}function UOe(r,i,o){return i in r?Object.defineProperty(r,i,{value:o,enumerable:!0,configurable:!0,writable:!0}):r[i]=o,r}function f4(r,i){return Ban(r)||Fan(r,i)||Ran(r,i)||jan()}function Ban(r){if(Array.isArray(r))return r}function Fan(r,i){var o=r==null?null:typeof Symbol<"u"&&r[Symbol.iterator]||r["@@iterator"];if(o!=null){var l=[],f=!0,b=!1,d,w;try{for(o=o.call(r);!(f=(d=o.next()).done)&&(l.push(d.value),!(i&&l.length===i));f=!0);}catch(y){b=!0,w=y}finally{try{!f&&o.return!=null&&o.return()}finally{if(b)throw w}}return l}}function Ran(r,i){if(!!r){if(typeof r=="string")return qOe(r,i);var o=Object.prototype.toString.call(r).slice(8,-1);if(o==="Object"&&r.constructor&&(o=r.constructor.name),o==="Map"||o==="Set")return Array.from(r);if(o==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(o))return qOe(r,i)}}function qOe(r,i){(i==null||i>r.length)&&(i=r.length);for(var o=0,l=new Array(i);o<i;o++)l[o]=r[o];return l}function jan(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}var Fu=typeof window>"u"?null:window,YOe=Fu?Fu.navigator:null;Fu&&Fu.document;var $an=df(""),WOe=df({}),Han=df(function(){}),zan=typeof HTMLElement>"u"?"undefined":df(HTMLElement),hO=function(i){return i&&i.instanceString&&al(i.instanceString)?i.instanceString():null},$i=function(i){return i!=null&&df(i)==$an},al=function(i){return i!=null&&df(i)===Han},qc=function(i){return!vp(i)&&(Array.isArray?Array.isArray(i):i!=null&&i instanceof Array)},_o=function(i){return i!=null&&df(i)===WOe&&!qc(i)&&i.constructor===Object},Gan=function(i){return i!=null&&df(i)===WOe},Vr=function(i){return i!=null&&df(i)===df(1)&&!isNaN(i)},Van=function(i){return Vr(i)&&Math.floor(i)===i},vU=function(i){if(zan!=="undefined")return i!=null&&i instanceof HTMLElement},vp=function(i){return fO(i)||KOe(i)},fO=function(i){return hO(i)==="collection"&&i._private.single},KOe=function(i){return hO(i)==="collection"&&!i._private.single},Qce=function(i){return hO(i)==="core"},XOe=function(i){return hO(i)==="stylesheet"},Uan=function(i){return hO(i)==="event"},y6=function(i){return i==null?!0:!!(i===""||i.match(/^\s+$/))},qan=function(i){return typeof HTMLElement>"u"?!1:i instanceof HTMLElement},Yan=function(i){return _o(i)&&Vr(i.x1)&&Vr(i.x2)&&Vr(i.y1)&&Vr(i.y2)},Wan=function(i){return Gan(i)&&al(i.then)},Kan=function(){return YOe&&YOe.userAgent.match(/msie|trident|edge/i)},dO=function(i,o){o||(o=function(){if(arguments.length===1)return arguments[0];if(arguments.length===0)return"undefined";for(var b=[],d=0;d<arguments.length;d++)b.push(arguments[d]);return b.join("$")});var l=function f(){var b=this,d=arguments,w,y=o.apply(b,d),k=f.cache;return(w=k[y])||(w=k[y]=i.apply(b,d)),w};return l.cache={},l},Zce=dO(function(r){return r.replace(/([A-Z])/g,function(i){return"-"+i.toLowerCase()})}),wU=dO(function(r){return r.replace(/(-\w)/g,function(i){return i[1].toUpperCase()})}),QOe=dO(function(r,i){return r+i[0].toUpperCase()+i.substring(1)},function(r,i){return r+"$"+i}),ZOe=function(i){return y6(i)?i:i.charAt(0).toUpperCase()+i.substring(1)},gf="(?:[-+]?(?:(?:\\d+|\\d*\\.\\d+)(?:[Ee][+-]?\\d+)?))",Xan="rgb[a]?\\(("+gf+"[%]?)\\s*,\\s*("+gf+"[%]?)\\s*,\\s*("+gf+"[%]?)(?:\\s*,\\s*("+gf+"))?\\)",Qan="rgb[a]?\\((?:"+gf+"[%]?)\\s*,\\s*(?:"+gf+"[%]?)\\s*,\\s*(?:"+gf+"[%]?)(?:\\s*,\\s*(?:"+gf+"))?\\)",Zan="hsl[a]?\\(("+gf+")\\s*,\\s*("+gf+"[%])\\s*,\\s*("+gf+"[%])(?:\\s*,\\s*("+gf+"))?\\)",Jan="hsl[a]?\\((?:"+gf+")\\s*,\\s*(?:"+gf+"[%])\\s*,\\s*(?:"+gf+"[%])(?:\\s*,\\s*(?:"+gf+"))?\\)",eon="\\#[0-9a-fA-F]{3}",ton="\\#[0-9a-fA-F]{6}",JOe=function(i,o){return i<o?-1:i>o?1:0},non=function(i,o){return-1*JOe(i,o)},bs=Object.assign!=null?Object.assign.bind(Object):function(r){for(var i=arguments,o=1;o<i.length;o++){var l=i[o];if(l!=null)for(var f=Object.keys(l),b=0;b<f.length;b++){var d=f[b];r[d]=l[d]}}return r},ron=function(i){if(!(!(i.length===4||i.length===7)||i[0]!=="#")){var o=i.length===4,l,f,b,d=16;return o?(l=parseInt(i[1]+i[1],d),f=parseInt(i[2]+i[2],d),b=parseInt(i[3]+i[3],d)):(l=parseInt(i[1]+i[2],d),f=parseInt(i[3]+i[4],d),b=parseInt(i[5]+i[6],d)),[l,f,b]}},ion=function(i){var o,l,f,b,d,w,y,k;function E(L,O,B){return B<0&&(B+=1),B>1&&(B-=1),B<1/6?L+(O-L)*6*B:B<1/2?O:B<2/3?L+(O-L)*(2/3-B)*6:L}var T=new RegExp("^"+Zan+"$").exec(i);if(T){if(l=parseInt(T[1]),l<0?l=(360- -1*l%360)%360:l>360&&(l=l%360),l/=360,f=parseFloat(T[2]),f<0||f>100||(f=f/100,b=parseFloat(T[3]),b<0||b>100)||(b=b/100,d=T[4],d!==void 0&&(d=parseFloat(d),d<0||d>1)))return;if(f===0)w=y=k=Math.round(b*255);else{var C=b<.5?b*(1+f):b+f-b*f,S=2*b-C;w=Math.round(255*E(S,C,l+1/3)),y=Math.round(255*E(S,C,l)),k=Math.round(255*E(S,C,l-1/3))}o=[w,y,k,d]}return o},son=function(i){var o,l=new RegExp("^"+Xan+"$").exec(i);if(l){o=[];for(var f=[],b=1;b<=3;b++){var d=l[b];if(d[d.length-1]==="%"&&(f[b]=!0),d=parseFloat(d),f[b]&&(d=d/100*255),d<0||d>255)return;o.push(Math.floor(d))}var w=f[1]||f[2]||f[3],y=f[1]&&f[2]&&f[3];if(w&&!y)return;var k=l[4];if(k!==void 0){if(k=parseFloat(k),k<0||k>1)return;o.push(k)}}return o},aon=function(i){return con[i.toLowerCase()]},oon=function(i){return(qc(i)?i:null)||aon(i)||ron(i)||son(i)||ion(i)},con={transparent:[0,0,0,0],aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],grey:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]},eNe=function(i){for(var o=i.map,l=i.keys,f=l.length,b=0;b<f;b++){var d=l[b];if(_o(d))throw Error("Tried to set map with object key");b<l.length-1?(o[d]==null&&(o[d]={}),o=o[d]):o[d]=i.value}},tNe=function(i){for(var o=i.map,l=i.keys,f=l.length,b=0;b<f;b++){var d=l[b];if(_o(d))throw Error("Tried to get map with object key");if(o=o[d],o==null)return o}return o},Jce=Fu?Fu.performance:null,nNe=Jce&&Jce.now?function(){return Jce.now()}:function(){return Date.now()},uon=function(){if(Fu){if(Fu.requestAnimationFrame)return function(r){Fu.requestAnimationFrame(r)};if(Fu.mozRequestAnimationFrame)return function(r){Fu.mozRequestAnimationFrame(r)};if(Fu.webkitRequestAnimationFrame)return function(r){Fu.webkitRequestAnimationFrame(r)};if(Fu.msRequestAnimationFrame)return function(r){Fu.msRequestAnimationFrame(r)}}return function(r){r&&setTimeout(function(){r(nNe())},1e3/60)}}(),mU=function(i){return uon(i)},d4=nNe,OC=9261,rNe=65599,gO=5381,iNe=function(i){for(var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:OC,l=o,f;f=i.next(),!f.done;)l=l*rNe+f.value|0;return l},pO=function(i){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:OC;return o*rNe+i|0},bO=function(i){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:gO;return(o<<5)+o+i|0},lon=function(i,o){return i*2097152+o},k6=function(i){return i[0]*2097152+i[1]},yU=function(i,o){return[pO(i[0],o[0]),bO(i[1],o[1])]},hon=function(i,o){var l={value:0,done:!1},f=0,b=i.length,d={next:function(){return f<b?l.value=i[f++]:l.done=!0,l}};return iNe(d,o)},w7=function(i,o){var l={value:0,done:!1},f=0,b=i.length,d={next:function(){return f<b?l.value=i.charCodeAt(f++):l.done=!0,l}};return iNe(d,o)},sNe=function(){return fon(arguments)},fon=function(i){for(var o,l=0;l<i.length;l++){var f=i[l];l===0?o=w7(f):o=w7(f,o)}return o},aNe=!0,don=console.warn!=null,gon=console.trace!=null,eue=Number.MAX_SAFE_INTEGER||9007199254740991,oNe=function(){return!0},kU=function(){return!1},cNe=function(){return 0},tue=function(){},Ml=function(i){throw new Error(i)},uNe=function(i){if(i!==void 0)aNe=!!i;else return aNe},tc=function(i){!uNe()||(don?console.warn(i):(console.log(i),gon&&console.trace()))},pon=function(i){return bs({},i)},qm=function(i){return i==null?i:qc(i)?i.slice():_o(i)?pon(i):i},bon=function(i){return i.slice()},lNe=function(i,o){for(o=i="";i++<36;o+=i*51&52?(i^15?8^Math.random()*(i^20?16:4):4).toString(16):"-");return o},von={},hNe=function(){return von},T1=function(i){var o=Object.keys(i);return function(l){for(var f={},b=0;b<o.length;b++){var d=o[b],w=l==null?void 0:l[d];f[d]=w===void 0?i[d]:w}return f}},x6=function(i,o,l){for(var f=i.length-1;f>=0&&!(i[f]===o&&(i.splice(f,1),l));f--);},nue=function(i){i.splice(0,i.length)},won=function(i,o){for(var l=0;l<o.length;l++){var f=o[l];i.push(f)}},Gv=function(i,o,l){return l&&(o=QOe(l,o)),i[o]},E6=function(i,o,l,f){l&&(o=QOe(l,o)),i[o]=f},mon=function(){function r(){Kce(this,r),this._obj={}}return Xce(r,[{key:"set",value:function(o,l){return this._obj[o]=l,this}},{key:"delete",value:function(o){return this._obj[o]=void 0,this}},{key:"clear",value:function(){this._obj={}}},{key:"has",value:function(o){return this._obj[o]!==void 0}},{key:"get",value:function(o){return this._obj[o]}}]),r}(),Ym=typeof Map<"u"?Map:mon,yon="undefined",kon=function(){function r(i){if(Kce(this,r),this._obj=Object.create(null),this.size=0,i!=null){var o;i.instanceString!=null&&i.instanceString()===this.instanceString()?o=i.toArray():o=i;for(var l=0;l<o.length;l++)this.add(o[l])}}return Xce(r,[{key:"instanceString",value:function(){return"set"}},{key:"add",value:function(o){var l=this._obj;l[o]!==1&&(l[o]=1,this.size++)}},{key:"delete",value:function(o){var l=this._obj;l[o]===1&&(l[o]=0,this.size--)}},{key:"clear",value:function(){this._obj=Object.create(null)}},{key:"has",value:function(o){return this._obj[o]===1}},{key:"toArray",value:function(){var o=this;return Object.keys(this._obj).filter(function(l){return o.has(l)})}},{key:"forEach",value:function(o,l){return this.toArray().forEach(o,l)}}]),r}(),NC=(typeof Set>"u"?"undefined":df(Set))!==yon?Set:kon,xU=function(i,o){var l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0;if(i===void 0||o===void 0||!Qce(i)){Ml("An element must have a core reference and parameters set");return}var f=o.group;if(f==null&&(o.data&&o.data.source!=null&&o.data.target!=null?f="edges":f="nodes"),f!=="nodes"&&f!=="edges"){Ml("An element must be of type `nodes` or `edges`; you specified `"+f+"`");return}this.length=1,this[0]=this;var b=this._private={cy:i,single:!0,data:o.data||{},position:o.position||{x:0,y:0},autoWidth:void 0,autoHeight:void 0,autoPadding:void 0,compoundBoundsClean:!1,listeners:[],group:f,style:{},rstyle:{},styleCxts:[],styleKeys:{},removed:!0,selected:!!o.selected,selectable:o.selectable===void 0?!0:!!o.selectable,locked:!!o.locked,grabbed:!1,grabbable:o.grabbable===void 0?!0:!!o.grabbable,pannable:o.pannable===void 0?f==="edges":!!o.pannable,active:!1,classes:new NC,animation:{current:[],queue:[]},rscratch:{},scratch:o.scratch||{},edges:[],children:[],parent:o.parent&&o.parent.isNode()?o.parent:null,traversalCache:{},backgrounding:!1,bbCache:null,bbCacheShift:{x:0,y:0},bodyBounds:null,overlayBounds:null,labelBounds:{all:null,source:null,target:null,main:null},arrowBounds:{source:null,target:null,"mid-source":null,"mid-target":null}};if(b.position.x==null&&(b.position.x=0),b.position.y==null&&(b.position.y=0),o.renderedPosition){var d=o.renderedPosition,w=i.pan(),y=i.zoom();b.position={x:(d.x-w.x)/y,y:(d.y-w.y)/y}}var k=[];qc(o.classes)?k=o.classes:$i(o.classes)&&(k=o.classes.split(/\s+/));for(var E=0,T=k.length;E<T;E++){var C=k[E];!C||C===""||b.classes.add(C)}this.createEmitter();var S=o.style||o.css;S&&(tc("Setting a `style` bypass at element creation should be done only when absolutely necessary.  Try to use the stylesheet instead."),this.style(S)),(l===void 0||l)&&this.restore()},fNe=function(i){return i={bfs:i.bfs||!i.dfs,dfs:i.dfs||!i.bfs},function(l,f,b){var d;_o(l)&&!vp(l)&&(d=l,l=d.roots||d.root,f=d.visit,b=d.directed),b=arguments.length===2&&!al(f)?f:b,f=al(f)?f:function(){};for(var w=this._private.cy,y=l=$i(l)?this.filter(l):l,k=[],E=[],T={},C={},S={},L=0,O,B=this.byGroup(),N=B.nodes,F=B.edges,R=0;R<y.length;R++){var q=y[R],X=q.id();q.isNode()&&(k.unshift(q),i.bfs&&(S[X]=!0,E.push(q)),C[X]=0)}for(var te=function(){var ae=i.bfs?k.shift():k.pop(),$=ae.id();if(i.dfs){if(S[$])return"continue";S[$]=!0,E.push(ae)}var ge=C[$],ee=T[$],de=ee!=null?ee.source():null,re=ee!=null?ee.target():null,ke=ee==null?void 0:ae.same(de)?re[0]:de[0],Ce=void 0;if(Ce=f(ae,ee,ke,L++,ge),Ce===!0)return O=ae,"break";if(Ce===!1)return"break";for(var _e=ae.connectedEdges().filter(function(Ee){return(!b||Ee.source().same(ae))&&F.has(Ee)}),Te=0;Te<_e.length;Te++){var Be=_e[Te],Ge=Be.connectedNodes().filter(function(Ee){return!Ee.same(ae)&&N.has(Ee)}),Xe=Ge.id();Ge.length!==0&&!S[Xe]&&(Ge=Ge[0],k.push(Ge),i.bfs&&(S[Xe]=!0,E.push(Ge)),T[Xe]=Be,C[Xe]=C[$]+1)}};k.length!==0;){var H=te();if(H!=="continue"&&H==="break")break}for(var Y=w.collection(),z=0;z<E.length;z++){var W=E[z],Z=T[W.id()];Z!=null&&Y.push(Z),Y.push(W)}return{path:w.collection(Y),found:w.collection(O)}}},vO={breadthFirstSearch:fNe({bfs:!0}),depthFirstSearch:fNe({dfs:!0})};vO.bfs=vO.breadthFirstSearch,vO.dfs=vO.depthFirstSearch;var xon=T1({root:null,weight:function(i){return 1},directed:!1}),Eon={dijkstra:function(i){if(!_o(i)){var o=arguments;i={root:o[0],weight:o[1],directed:o[2]}}var l=xon(i),f=l.root,b=l.weight,d=l.directed,w=this,y=b,k=$i(f)?this.filter(f)[0]:f[0],E={},T={},C={},S=this.byGroup(),L=S.nodes,O=S.edges;O.unmergeBy(function(ge){return ge.isLoop()});for(var B=function(ee){return E[ee.id()]},N=function(ee,de){E[ee.id()]=de,F.updateItem(ee)},F=new lO.default(function(ge,ee){return B(ge)-B(ee)}),R=0;R<L.length;R++){var q=L[R];E[q.id()]=q.same(k)?0:1/0,F.push(q)}for(var X=function(ee,de){for(var re=(d?ee.edgesTo(de):ee.edgesWith(de)).intersect(O),ke=1/0,Ce,_e=0;_e<re.length;_e++){var Te=re[_e],Be=y(Te);(Be<ke||!Ce)&&(ke=Be,Ce=Te)}return{edge:Ce,dist:ke}};F.size()>0;){var te=F.pop(),H=B(te),Y=te.id();if(C[Y]=H,H!==1/0)for(var z=te.neighborhood().intersect(L),W=0;W<z.length;W++){var Z=z[W],G=Z.id(),ae=X(te,Z),$=H+ae.dist;$<B(Z)&&(N(Z,$),T[G]={node:te,edge:ae.edge})}}return{distanceTo:function(ee){var de=$i(ee)?L.filter(ee)[0]:ee[0];return C[de.id()]},pathTo:function(ee){var de=$i(ee)?L.filter(ee)[0]:ee[0],re=[],ke=de,Ce=ke.id();if(de.length>0)for(re.unshift(de);T[Ce];){var _e=T[Ce];re.unshift(_e.edge),re.unshift(_e.node),ke=_e.node,Ce=ke.id()}return w.spawn(re)}}}},_on={kruskal:function(i){i=i||function(R){return 1};for(var o=this.byGroup(),l=o.nodes,f=o.edges,b=l.length,d=new Array(b),w=l,y=function(q){for(var X=0;X<d.length;X++){var te=d[X];if(te.has(q))return X}},k=0;k<b;k++)d[k]=this.spawn(l[k]);for(var E=f.sort(function(R,q){return i(R)-i(q)}),T=0;T<E.length;T++){var C=E[T],S=C.source()[0],L=C.target()[0],O=y(S),B=y(L),N=d[O],F=d[B];O!==B&&(w.merge(C),N.merge(F),d.splice(B,1))}return w}},Ton=T1({root:null,goal:null,weight:function(i){return 1},heuristic:function(i){return 0},directed:!1}),Con={aStar:function(i){var o=this.cy(),l=Ton(i),f=l.root,b=l.goal,d=l.heuristic,w=l.directed,y=l.weight;f=o.collection(f)[0],b=o.collection(b)[0];var k=f.id(),E=b.id(),T={},C={},S={},L=new lO.default(function(Ce,_e){return C[Ce.id()]-C[_e.id()]}),O=new NC,B={},N={},F=function(_e,Te){L.push(_e),O.add(Te)},R,q,X=function(){R=L.pop(),q=R.id(),O.delete(q)},te=function(_e){return O.has(_e)};F(f,k),T[k]=0,C[k]=d(f);for(var H=0;L.size()>0;){if(X(),H++,q===E){for(var Y=[],z=b,W=E,Z=N[W];Y.unshift(z),Z!=null&&Y.unshift(Z),z=B[W],z!=null;)W=z.id(),Z=N[W];return{found:!0,distance:T[q],path:this.spawn(Y),steps:H}}S[q]=!0;for(var G=R._private.edges,ae=0;ae<G.length;ae++){var $=G[ae];if(!!this.hasElementWithId($.id())&&!(w&&$.data("source")!==q)){var ge=$.source(),ee=$.target(),de=ge.id()!==q?ge:ee,re=de.id();if(!!this.hasElementWithId(re)&&!S[re]){var ke=T[q]+y($);if(!te(re)){T[re]=ke,C[re]=ke+d(de),F(de,re),B[re]=R,N[re]=$;continue}ke<T[re]&&(T[re]=ke,C[re]=ke+d(de),B[re]=R,N[re]=$)}}}}return{found:!1,distance:void 0,path:void 0,steps:H}}},Son=T1({weight:function(i){return 1},directed:!1}),Aon={floydWarshall:function(i){for(var o=this.cy(),l=Son(i),f=l.weight,b=l.directed,d=f,w=this.byGroup(),y=w.nodes,k=w.edges,E=y.length,T=E*E,C=function(Be){return y.indexOf(Be)},S=function(Be){return y[Be]},L=new Array(T),O=0;O<T;O++){var B=O%E,N=(O-B)/E;N===B?L[O]=0:L[O]=1/0}for(var F=new Array(T),R=new Array(T),q=0;q<k.length;q++){var X=k[q],te=X.source()[0],H=X.target()[0];if(te!==H){var Y=C(te),z=C(H),W=Y*E+z,Z=d(X);if(L[W]>Z&&(L[W]=Z,F[W]=z,R[W]=X),!b){var G=z*E+Y;!b&&L[G]>Z&&(L[G]=Z,F[G]=Y,R[G]=X)}}}for(var ae=0;ae<E;ae++)for(var $=0;$<E;$++)for(var ge=$*E+ae,ee=0;ee<E;ee++){var de=$*E+ee,re=ae*E+ee;L[ge]+L[re]<L[de]&&(L[de]=L[ge]+L[re],F[de]=F[ge])}var ke=function(Be){return($i(Be)?o.filter(Be):Be)[0]},Ce=function(Be){return C(ke(Be))},_e={distance:function(Be,Ge){var Xe=Ce(Be),Ee=Ce(Ge);return L[Xe*E+Ee]},path:function(Be,Ge){var Xe=Ce(Be),Ee=Ce(Ge),Ze=S(Xe);if(Xe===Ee)return Ze.collection();if(F[Xe*E+Ee]==null)return o.collection();var Ie=o.collection(),Oe=Xe,Le;for(Ie.merge(Ze);Xe!==Ee;)Oe=Xe,Xe=F[Xe*E+Ee],Le=R[Oe*E+Xe],Ie.merge(Le),Ie.merge(S(Xe));return Ie}};return _e}},Mon=T1({weight:function(i){return 1},directed:!1,root:null}),Don={bellmanFord:function(i){var o=this,l=Mon(i),f=l.weight,b=l.directed,d=l.root,w=f,y=this,k=this.cy(),E=this.byGroup(),T=E.edges,C=E.nodes,S=C.length,L=new Ym,O=!1,B=[];d=k.collection(d)[0],T.unmergeBy(function(ve){return ve.isLoop()});for(var N=T.length,F=function(Zt){var nt=L.get(Zt.id());return nt||(nt={},L.set(Zt.id(),nt)),nt},R=function(Zt){return($i(Zt)?k.$(Zt):Zt)[0]},q=function(Zt){return F(R(Zt)).dist},X=function(Zt){for(var nt=arguments.length>1&&arguments[1]!==void 0?arguments[1]:d,xn=R(Zt),cn=[],jt=xn;;){if(jt==null)return o.spawn();var ot=F(jt),be=ot.edge,We=ot.pred;if(cn.unshift(jt[0]),jt.same(nt)&&cn.length>0)break;be!=null&&cn.unshift(be),jt=We}return y.spawn(cn)},te=0;te<S;te++){var H=C[te],Y=F(H);H.same(d)?Y.dist=0:Y.dist=1/0,Y.pred=null,Y.edge=null}for(var z=!1,W=function(Zt,nt,xn,cn,jt,ot){var be=cn.dist+ot;be<jt.dist&&!xn.same(cn.edge)&&(jt.dist=be,jt.pred=Zt,jt.edge=xn,z=!0)},Z=1;Z<S;Z++){z=!1;for(var G=0;G<N;G++){var ae=T[G],$=ae.source(),ge=ae.target(),ee=w(ae),de=F($),re=F(ge);W($,ge,ae,de,re,ee),b||W(ge,$,ae,re,de,ee)}if(!z)break}if(z)for(var ke=[],Ce=0;Ce<N;Ce++){var _e=T[Ce],Te=_e.source(),Be=_e.target(),Ge=w(_e),Xe=F(Te).dist,Ee=F(Be).dist;if(Xe+Ge<Ee||!b&&Ee+Ge<Xe)if(O||(tc("Graph contains a negative weight cycle for Bellman-Ford"),O=!0),i.findNegativeWeightCycles!==!1){var Ze=[];Xe+Ge<Ee&&Ze.push(Te),!b&&Ee+Ge<Xe&&Ze.push(Be);for(var Ie=Ze.length,Oe=0;Oe<Ie;Oe++){var Le=Ze[Oe],$e=[Le];$e.push(F(Le).edge);for(var xe=F(Le).pred;$e.indexOf(xe)===-1;)$e.push(xe),$e.push(F(xe).edge),xe=F(xe).pred;$e=$e.slice($e.indexOf(xe));for(var Ae=$e[0].id(),je=0,me=2;me<$e.length;me+=2)$e[me].id()<Ae&&(Ae=$e[me].id(),je=me);$e=$e.slice(je).concat($e.slice(0,je)),$e.push($e[0]);var vt=$e.map(function(ve){return ve.id()}).join(",");ke.indexOf(vt)===-1&&(B.push(y.spawn($e)),ke.push(vt))}}else break}return{distanceTo:q,pathTo:X,hasNegativeWeightCycle:O,negativeWeightCycles:B}}},Lon=Math.sqrt(2),Ion=function(i,o,l){l.length===0&&Ml("Karger-Stein must be run on a connected (sub)graph");for(var f=l[i],b=f[1],d=f[2],w=o[b],y=o[d],k=l,E=k.length-1;E>=0;E--){var T=k[E],C=T[1],S=T[2];(o[C]===w&&o[S]===y||o[C]===y&&o[S]===w)&&k.splice(E,1)}for(var L=0;L<k.length;L++){var O=k[L];O[1]===y?(k[L]=O.slice(),k[L][1]=w):O[2]===y&&(k[L]=O.slice(),k[L][2]=w)}for(var B=0;B<o.length;B++)o[B]===y&&(o[B]=w);return k},rue=function(i,o,l,f){for(;l>f;){var b=Math.floor(Math.random()*o.length);o=Ion(b,i,o),l--}return o},Oon={kargerStein:function(){var i=this,o=this.byGroup(),l=o.nodes,f=o.edges;f.unmergeBy(function(re){return re.isLoop()});var b=l.length,d=f.length,w=Math.ceil(Math.pow(Math.log(b)/Math.LN2,2)),y=Math.floor(b/Lon);if(b<2){Ml("At least 2 nodes are required for Karger-Stein algorithm");return}for(var k=[],E=0;E<d;E++){var T=f[E];k.push([E,l.indexOf(T.source()),l.indexOf(T.target())])}for(var C=1/0,S=[],L=new Array(b),O=new Array(b),B=new Array(b),N=function(ke,Ce){for(var _e=0;_e<b;_e++)Ce[_e]=ke[_e]},F=0;F<=w;F++){for(var R=0;R<b;R++)O[R]=R;var q=rue(O,k.slice(),b,y),X=q.slice();N(O,B);var te=rue(O,q,y,2),H=rue(B,X,y,2);te.length<=H.length&&te.length<C?(C=te.length,S=te,N(O,L)):H.length<=te.length&&H.length<C&&(C=H.length,S=H,N(B,L))}for(var Y=this.spawn(S.map(function(re){return f[re[0]]})),z=this.spawn(),W=this.spawn(),Z=L[0],G=0;G<L.length;G++){var ae=L[G],$=l[G];ae===Z?z.merge($):W.merge($)}var ge=function(ke){var Ce=i.spawn();return ke.forEach(function(_e){Ce.merge(_e),_e.connectedEdges().forEach(function(Te){i.contains(Te)&&!Y.contains(Te)&&Ce.merge(Te)})}),Ce},ee=[ge(z),ge(W)],de={cut:Y,components:ee,partition1:z,partition2:W};return de}},Non=function(i){return{x:i.x,y:i.y}},EU=function(i,o,l){return{x:i.x*o+l.x,y:i.y*o+l.y}},dNe=function(i,o,l){return{x:(i.x-l.x)/o,y:(i.y-l.y)/o}},PC=function(i){return{x:i[0],y:i[1]}},Pon=function(i){for(var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:i.length,f=1/0,b=o;b<l;b++){var d=i[b];isFinite(d)&&(f=Math.min(d,f))}return f},Bon=function(i){for(var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:i.length,f=-1/0,b=o;b<l;b++){var d=i[b];isFinite(d)&&(f=Math.max(d,f))}return f},Fon=function(i){for(var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:i.length,f=0,b=0,d=o;d<l;d++){var w=i[d];isFinite(w)&&(f+=w,b++)}return f/b},Ron=function(i){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:i.length,f=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!0,b=arguments.length>4&&arguments[4]!==void 0?arguments[4]:!0,d=arguments.length>5&&arguments[5]!==void 0?arguments[5]:!0;f?i=i.slice(o,l):(l<i.length&&i.splice(l,i.length-l),o>0&&i.splice(0,o));for(var w=0,y=i.length-1;y>=0;y--){var k=i[y];d?isFinite(k)||(i[y]=-1/0,w++):i.splice(y,1)}b&&i.sort(function(C,S){return C-S});var E=i.length,T=Math.floor(E/2);return E%2!==0?i[T+1+w]:(i[T-1+w]+i[T+w])/2},jon=function(i){return Math.PI*i/180},_U=function(i,o){return Math.atan2(o,i)-Math.PI/2},iue=Math.log2||function(r){return Math.log(r)/Math.log(2)},gNe=function(i){return i>0?1:i<0?-1:0},m7=function(i,o){return Math.sqrt(y7(i,o))},y7=function(i,o){var l=o.x-i.x,f=o.y-i.y;return l*l+f*f},$on=function(i){for(var o=i.length,l=0,f=0;f<o;f++)l+=i[f];for(var b=0;b<o;b++)i[b]=i[b]/l;return i},Kf=function(i,o,l,f){return(1-f)*(1-f)*i+2*(1-f)*f*o+f*f*l},BC=function(i,o,l,f){return{x:Kf(i.x,o.x,l.x,f),y:Kf(i.y,o.y,l.y,f)}},Hon=function(i,o,l,f){var b={x:o.x-i.x,y:o.y-i.y},d=m7(i,o),w={x:b.x/d,y:b.y/d};return l=l==null?0:l,f=f!=null?f:l*d,{x:i.x+w.x*f,y:i.y+w.y*f}},wO=function(i,o,l){return Math.max(i,Math.min(l,o))},J0=function(i){if(i==null)return{x1:1/0,y1:1/0,x2:-1/0,y2:-1/0,w:0,h:0};if(i.x1!=null&&i.y1!=null){if(i.x2!=null&&i.y2!=null&&i.x2>=i.x1&&i.y2>=i.y1)return{x1:i.x1,y1:i.y1,x2:i.x2,y2:i.y2,w:i.x2-i.x1,h:i.y2-i.y1};if(i.w!=null&&i.h!=null&&i.w>=0&&i.h>=0)return{x1:i.x1,y1:i.y1,x2:i.x1+i.w,y2:i.y1+i.h,w:i.w,h:i.h}}},zon=function(i){return{x1:i.x1,x2:i.x2,w:i.w,y1:i.y1,y2:i.y2,h:i.h}},Gon=function(i){i.x1=1/0,i.y1=1/0,i.x2=-1/0,i.y2=-1/0,i.w=0,i.h=0},Von=function(i,o){i.x1=Math.min(i.x1,o.x1),i.x2=Math.max(i.x2,o.x2),i.w=i.x2-i.x1,i.y1=Math.min(i.y1,o.y1),i.y2=Math.max(i.y2,o.y2),i.h=i.y2-i.y1},Uon=function(i,o,l){i.x1=Math.min(i.x1,o),i.x2=Math.max(i.x2,o),i.w=i.x2-i.x1,i.y1=Math.min(i.y1,l),i.y2=Math.max(i.y2,l),i.h=i.y2-i.y1},TU=function(i){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0;return i.x1-=o,i.x2+=o,i.y1-=o,i.y2+=o,i.w=i.x2-i.x1,i.h=i.y2-i.y1,i},sue=function(i){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[0],l,f,b,d;if(o.length===1)l=f=b=d=o[0];else if(o.length===2)l=b=o[0],d=f=o[1];else if(o.length===4){var w=f4(o,4);l=w[0],f=w[1],b=w[2],d=w[3]}return i.x1-=d,i.x2+=f,i.y1-=l,i.y2+=b,i.w=i.x2-i.x1,i.h=i.y2-i.y1,i},pNe=function(i,o){i.x1=o.x1,i.y1=o.y1,i.x2=o.x2,i.y2=o.y2,i.w=i.x2-i.x1,i.h=i.y2-i.y1},aue=function(i,o){return!(i.x1>o.x2||o.x1>i.x2||i.x2<o.x1||o.x2<i.x1||i.y2<o.y1||o.y2<i.y1||i.y1>o.y2||o.y1>i.y2)},FC=function(i,o,l){return i.x1<=o&&o<=i.x2&&i.y1<=l&&l<=i.y2},qon=function(i,o){return FC(i,o.x,o.y)},bNe=function(i,o){return FC(i,o.x1,o.y1)&&FC(i,o.x2,o.y2)},vNe=function(i,o,l,f,b,d,w){var y=kO(b,d),k=b/2,E=d/2,T;{var C=l-k+y-w,S=f-E-w,L=l+k-y+w,O=S;if(T=_6(i,o,l,f,C,S,L,O,!1),T.length>0)return T}{var B=l+k+w,N=f-E+y-w,F=B,R=f+E-y+w;if(T=_6(i,o,l,f,B,N,F,R,!1),T.length>0)return T}{var q=l-k+y-w,X=f+E+w,te=l+k-y+w,H=X;if(T=_6(i,o,l,f,q,X,te,H,!1),T.length>0)return T}{var Y=l-k-w,z=f-E+y-w,W=Y,Z=f+E-y+w;if(T=_6(i,o,l,f,Y,z,W,Z,!1),T.length>0)return T}var G;{var ae=l-k+y,$=f-E+y;if(G=mO(i,o,l,f,ae,$,y+w),G.length>0&&G[0]<=ae&&G[1]<=$)return[G[0],G[1]]}{var ge=l+k-y,ee=f-E+y;if(G=mO(i,o,l,f,ge,ee,y+w),G.length>0&&G[0]>=ge&&G[1]<=ee)return[G[0],G[1]]}{var de=l+k-y,re=f+E-y;if(G=mO(i,o,l,f,de,re,y+w),G.length>0&&G[0]>=de&&G[1]>=re)return[G[0],G[1]]}{var ke=l-k+y,Ce=f+E-y;if(G=mO(i,o,l,f,ke,Ce,y+w),G.length>0&&G[0]<=ke&&G[1]>=Ce)return[G[0],G[1]]}return[]},Yon=function(i,o,l,f,b,d,w){var y=w,k=Math.min(l,b),E=Math.max(l,b),T=Math.min(f,d),C=Math.max(f,d);return k-y<=i&&i<=E+y&&T-y<=o&&o<=C+y},Won=function(i,o,l,f,b,d,w,y,k){var E={x1:Math.min(l,w,b)-k,x2:Math.max(l,w,b)+k,y1:Math.min(f,y,d)-k,y2:Math.max(f,y,d)+k};return!(i<E.x1||i>E.x2||o<E.y1||o>E.y2)},Kon=function(i,o,l,f){l-=f;var b=o*o-4*i*l;if(b<0)return[];var d=Math.sqrt(b),w=2*i,y=(-o+d)/w,k=(-o-d)/w;return[y,k]},Xon=function(i,o,l,f,b){var d=1e-5;i===0&&(i=d),o/=i,l/=i,f/=i;var w,y,k,E,T,C,S,L;if(y=(3*l-o*o)/9,k=-(27*f)+o*(9*l-2*(o*o)),k/=54,w=y*y*y+k*k,b[1]=0,S=o/3,w>0){T=k+Math.sqrt(w),T=T<0?-Math.pow(-T,1/3):Math.pow(T,1/3),C=k-Math.sqrt(w),C=C<0?-Math.pow(-C,1/3):Math.pow(C,1/3),b[0]=-S+T+C,S+=(T+C)/2,b[4]=b[2]=-S,S=Math.sqrt(3)*(-C+T)/2,b[3]=S,b[5]=-S;return}if(b[5]=b[3]=0,w===0){L=k<0?-Math.pow(-k,1/3):Math.pow(k,1/3),b[0]=-S+2*L,b[4]=b[2]=-(L+S);return}y=-y,E=y*y*y,E=Math.acos(k/Math.sqrt(E)),L=2*Math.sqrt(y),b[0]=-S+L*Math.cos(E/3),b[2]=-S+L*Math.cos((E+2*Math.PI)/3),b[4]=-S+L*Math.cos((E+4*Math.PI)/3)},Qon=function(i,o,l,f,b,d,w,y){var k=1*l*l-4*l*b+2*l*w+4*b*b-4*b*w+w*w+f*f-4*f*d+2*f*y+4*d*d-4*d*y+y*y,E=1*9*l*b-3*l*l-3*l*w-6*b*b+3*b*w+9*f*d-3*f*f-3*f*y-6*d*d+3*d*y,T=1*3*l*l-6*l*b+l*w-l*i+2*b*b+2*b*i-w*i+3*f*f-6*f*d+f*y-f*o+2*d*d+2*d*o-y*o,C=1*l*b-l*l+l*i-b*i+f*d-f*f+f*o-d*o,S=[];Xon(k,E,T,C,S);for(var L=1e-7,O=[],B=0;B<6;B+=2)Math.abs(S[B+1])<L&&S[B]>=0&&S[B]<=1&&O.push(S[B]);O.push(1),O.push(0);for(var N=-1,F,R,q,X=0;X<O.length;X++)F=Math.pow(1-O[X],2)*l+2*(1-O[X])*O[X]*b+O[X]*O[X]*w,R=Math.pow(1-O[X],2)*f+2*(1-O[X])*O[X]*d+O[X]*O[X]*y,q=Math.pow(F-i,2)+Math.pow(R-o,2),N>=0?q<N&&(N=q):N=q;return N},Zon=function(i,o,l,f,b,d){var w=[i-l,o-f],y=[b-l,d-f],k=y[0]*y[0]+y[1]*y[1],E=w[0]*w[0]+w[1]*w[1],T=w[0]*y[0]+w[1]*y[1],C=T*T/k;return T<0?E:C>k?(i-b)*(i-b)+(o-d)*(o-d):E-C},eg=function(i,o,l){for(var f,b,d,w,y,k=0,E=0;E<l.length/2;E++)if(f=l[E*2],b=l[E*2+1],E+1<l.length/2?(d=l[(E+1)*2],w=l[(E+1)*2+1]):(d=l[(E+1-l.length/2)*2],w=l[(E+1-l.length/2)*2+1]),!(f==i&&d==i))if(f>=i&&i>=d||f<=i&&i<=d)y=(i-f)/(d-f)*(w-b)+b,y>o&&k++;else continue;return k%2!==0},g4=function(i,o,l,f,b,d,w,y,k){var E=new Array(l.length),T;y[0]!=null?(T=Math.atan(y[1]/y[0]),y[0]<0?T=T+Math.PI/2:T=-T-Math.PI/2):T=y;for(var C=Math.cos(-T),S=Math.sin(-T),L=0;L<E.length/2;L++)E[L*2]=d/2*(l[L*2]*C-l[L*2+1]*S),E[L*2+1]=w/2*(l[L*2+1]*C+l[L*2]*S),E[L*2]+=f,E[L*2+1]+=b;var O;if(k>0){var B=mNe(E,-k);O=wNe(B)}else O=E;return eg(i,o,O)},Jon=function(i,o,l,f,b,d,w){for(var y=new Array(l.length),k=d/2,E=w/2,T=uue(d,w),C=T*T,S=0;S<l.length/4;S++){var L=void 0,O=void 0;S===0?L=l.length-2:L=S*4-2,O=S*4+2;var B=f+k*l[S*4],N=b+E*l[S*4+1],F=-l[L]*l[O]-l[L+1]*l[O+1],R=T/Math.tan(Math.acos(F)/2),q=B-R*l[L],X=N-R*l[L+1],te=B+R*l[O],H=N+R*l[O+1];y[S*4]=q,y[S*4+1]=X,y[S*4+2]=te,y[S*4+3]=H;var Y=l[L+1],z=-l[L],W=Y*l[O]+z*l[O+1];W<0&&(Y*=-1,z*=-1);var Z=q+Y*T,G=X+z*T,ae=Math.pow(Z-i,2)+Math.pow(G-o,2);if(ae<=C)return!0}return eg(i,o,y)},wNe=function(i){for(var o=new Array(i.length/2),l,f,b,d,w,y,k,E,T=0;T<i.length/4;T++){l=i[T*4],f=i[T*4+1],b=i[T*4+2],d=i[T*4+3],T<i.length/4-1?(w=i[(T+1)*4],y=i[(T+1)*4+1],k=i[(T+1)*4+2],E=i[(T+1)*4+3]):(w=i[0],y=i[1],k=i[2],E=i[3]);var C=_6(l,f,b,d,w,y,k,E,!0);o[T*2]=C[0],o[T*2+1]=C[1]}return o},mNe=function(i,o){for(var l=new Array(i.length*2),f,b,d,w,y=0;y<i.length/2;y++){f=i[y*2],b=i[y*2+1],y<i.length/2-1?(d=i[(y+1)*2],w=i[(y+1)*2+1]):(d=i[0],w=i[1]);var k=w-b,E=-(d-f),T=Math.sqrt(k*k+E*E),C=k/T,S=E/T;l[y*4]=f+C*o,l[y*4+1]=b+S*o,l[y*4+2]=d+C*o,l[y*4+3]=w+S*o}return l},ecn=function(i,o,l,f,b,d){var w=l-i,y=f-o;w/=b,y/=d;var k=Math.sqrt(w*w+y*y),E=k-1;if(E<0)return[];var T=E/k;return[(l-i)*T+i,(f-o)*T+o]},k7=function(i,o,l,f,b,d,w){return i-=b,o-=d,i/=l/2+w,o/=f/2+w,i*i+o*o<=1},mO=function(i,o,l,f,b,d,w){var y=[l-i,f-o],k=[i-b,o-d],E=y[0]*y[0]+y[1]*y[1],T=2*(k[0]*y[0]+k[1]*y[1]),C=k[0]*k[0]+k[1]*k[1]-w*w,S=T*T-4*E*C;if(S<0)return[];var L=(-T+Math.sqrt(S))/(2*E),O=(-T-Math.sqrt(S))/(2*E),B=Math.min(L,O),N=Math.max(L,O),F=[];if(B>=0&&B<=1&&F.push(B),N>=0&&N<=1&&F.push(N),F.length===0)return[];var R=F[0]*y[0]+i,q=F[0]*y[1]+o;if(F.length>1){if(F[0]==F[1])return[R,q];var X=F[1]*y[0]+i,te=F[1]*y[1]+o;return[R,q,X,te]}else return[R,q]},oue=function(i,o,l){return o<=i&&i<=l||l<=i&&i<=o?i:i<=o&&o<=l||l<=o&&o<=i?o:l},_6=function(i,o,l,f,b,d,w,y,k){var E=i-b,T=l-i,C=w-b,S=o-d,L=f-o,O=y-d,B=C*S-O*E,N=T*S-L*E,F=O*T-C*L;if(F!==0){var R=B/F,q=N/F,X=.001,te=0-X,H=1+X;return te<=R&&R<=H&&te<=q&&q<=H?[i+R*T,o+R*L]:k?[i+R*T,o+R*L]:[]}else return B===0||N===0?oue(i,l,w)===w?[w,y]:oue(i,l,b)===b?[b,d]:oue(b,w,l)===l?[l,f]:[]:[]},yO=function(i,o,l,f,b,d,w,y){var k=[],E,T=new Array(l.length),C=!0;d==null&&(C=!1);var S;if(C){for(var L=0;L<T.length/2;L++)T[L*2]=l[L*2]*d+f,T[L*2+1]=l[L*2+1]*w+b;if(y>0){var O=mNe(T,-y);S=wNe(O)}else S=T}else S=l;for(var B,N,F,R,q=0;q<S.length/2;q++)B=S[q*2],N=S[q*2+1],q<S.length/2-1?(F=S[(q+1)*2],R=S[(q+1)*2+1]):(F=S[0],R=S[1]),E=_6(i,o,f,b,B,N,F,R),E.length!==0&&k.push(E[0],E[1]);return k},tcn=function(i,o,l,f,b,d,w,y){for(var k=[],E,T=new Array(l.length),C=d/2,S=w/2,L=uue(d,w),O=0;O<l.length/4;O++){var B=void 0,N=void 0;O===0?B=l.length-2:B=O*4-2,N=O*4+2;var F=f+C*l[O*4],R=b+S*l[O*4+1],q=-l[B]*l[N]-l[B+1]*l[N+1],X=L/Math.tan(Math.acos(q)/2),te=F-X*l[B],H=R-X*l[B+1],Y=F+X*l[N],z=R+X*l[N+1];O===0?(T[l.length-2]=te,T[l.length-1]=H):(T[O*4-2]=te,T[O*4-1]=H),T[O*4]=Y,T[O*4+1]=z;var W=l[B+1],Z=-l[B],G=W*l[N]+Z*l[N+1];G<0&&(W*=-1,Z*=-1);var ae=te+W*L,$=H+Z*L;E=mO(i,o,f,b,ae,$,L),E.length!==0&&k.push(E[0],E[1])}for(var ge=0;ge<T.length/4;ge++)E=_6(i,o,f,b,T[ge*4],T[ge*4+1],T[ge*4+2],T[ge*4+3],!1),E.length!==0&&k.push(E[0],E[1]);if(k.length>2){for(var ee=[k[0],k[1]],de=Math.pow(ee[0]-i,2)+Math.pow(ee[1]-o,2),re=1;re<k.length/2;re++){var ke=Math.pow(k[re*2]-i,2)+Math.pow(k[re*2+1]-o,2);ke<=de&&(ee[0]=k[re*2],ee[1]=k[re*2+1],de=ke)}return ee}return k},CU=function(i,o,l){var f=[i[0]-o[0],i[1]-o[1]],b=Math.sqrt(f[0]*f[0]+f[1]*f[1]),d=(b-l)/b;return d<0&&(d=1e-5),[o[0]+d*f[0],o[1]+d*f[1]]},e0=function(i,o){var l=cue(i,o);return l=yNe(l),l},yNe=function(i){for(var o,l,f=i.length/2,b=1/0,d=1/0,w=-1/0,y=-1/0,k=0;k<f;k++)o=i[2*k],l=i[2*k+1],b=Math.min(b,o),w=Math.max(w,o),d=Math.min(d,l),y=Math.max(y,l);for(var E=2/(w-b),T=2/(y-d),C=0;C<f;C++)o=i[2*C]=i[2*C]*E,l=i[2*C+1]=i[2*C+1]*T,b=Math.min(b,o),w=Math.max(w,o),d=Math.min(d,l),y=Math.max(y,l);if(d<-1)for(var S=0;S<f;S++)l=i[2*S+1]=i[2*S+1]+(-1-d);return i},cue=function(i,o){var l=1/i*2*Math.PI,f=i%2===0?Math.PI/2+l/2:Math.PI/2;f+=o;for(var b=new Array(i*2),d,w=0;w<i;w++)d=w*l+f,b[2*w]=Math.cos(d),b[2*w+1]=Math.sin(-d);return b},kO=function(i,o){return Math.min(i/4,o/4,8)},uue=function(i,o){return Math.min(i/10,o/10,8)},kNe=function(){return 8},ncn=function(i,o,l){return[i-2*o+l,2*(o-i),i]},lue=function(i,o){return{heightOffset:Math.min(15,.05*o),widthOffset:Math.min(100,.25*i),ctrlPtOffsetPct:.05}},rcn=T1({dampingFactor:.8,precision:1e-6,iterations:200,weight:function(i){return 1}}),icn={pageRank:function(i){for(var o=rcn(i),l=o.dampingFactor,f=o.precision,b=o.iterations,d=o.weight,w=this._private.cy,y=this.byGroup(),k=y.nodes,E=y.edges,T=k.length,C=T*T,S=E.length,L=new Array(C),O=new Array(T),B=(1-l)/T,N=0;N<T;N++){for(var F=0;F<T;F++){var R=N*T+F;L[R]=0}O[N]=0}for(var q=0;q<S;q++){var X=E[q],te=X.data("source"),H=X.data("target");if(te!==H){var Y=k.indexOfId(te),z=k.indexOfId(H),W=d(X),Z=z*T+Y;L[Z]+=W,O[Y]+=W}}for(var G=1/T+B,ae=0;ae<T;ae++)if(O[ae]===0)for(var $=0;$<T;$++){var ge=$*T+ae;L[ge]=G}else for(var ee=0;ee<T;ee++){var de=ee*T+ae;L[de]=L[de]/O[ae]+B}for(var re=new Array(T),ke=new Array(T),Ce,_e=0;_e<T;_e++)re[_e]=1;for(var Te=0;Te<b;Te++){for(var Be=0;Be<T;Be++)ke[Be]=0;for(var Ge=0;Ge<T;Ge++)for(var Xe=0;Xe<T;Xe++){var Ee=Ge*T+Xe;ke[Ge]+=L[Ee]*re[Xe]}$on(ke),Ce=re,re=ke,ke=Ce;for(var Ze=0,Ie=0;Ie<T;Ie++){var Oe=Ce[Ie]-re[Ie];Ze+=Oe*Oe}if(Ze<f)break}var Le={rank:function(xe){return xe=w.collection(xe)[0],re[k.indexOf(xe)]}};return Le}},xNe=T1({root:null,weight:function(i){return 1},directed:!1,alpha:0}),RC={degreeCentralityNormalized:function(i){i=xNe(i);var o=this.cy(),l=this.nodes(),f=l.length;if(i.directed){for(var E={},T={},C=0,S=0,L=0;L<f;L++){var O=l[L],B=O.id();i.root=O;var N=this.degreeCentrality(i);C<N.indegree&&(C=N.indegree),S<N.outdegree&&(S=N.outdegree),E[B]=N.indegree,T[B]=N.outdegree}return{indegree:function(R){return C==0?0:($i(R)&&(R=o.filter(R)),E[R.id()]/C)},outdegree:function(R){return S===0?0:($i(R)&&(R=o.filter(R)),T[R.id()]/S)}}}else{for(var b={},d=0,w=0;w<f;w++){var y=l[w];i.root=y;var k=this.degreeCentrality(i);d<k.degree&&(d=k.degree),b[y.id()]=k.degree}return{degree:function(R){return d===0?0:($i(R)&&(R=o.filter(R)),b[R.id()]/d)}}}},degreeCentrality:function(i){i=xNe(i);var o=this.cy(),l=this,f=i,b=f.root,d=f.weight,w=f.directed,y=f.alpha;if(b=o.collection(b)[0],w){for(var S=b.connectedEdges(),L=S.filter(function(te){return te.target().same(b)&&l.has(te)}),O=S.filter(function(te){return te.source().same(b)&&l.has(te)}),B=L.length,N=O.length,F=0,R=0,q=0;q<L.length;q++)F+=d(L[q]);for(var X=0;X<O.length;X++)R+=d(O[X]);return{indegree:Math.pow(B,1-y)*Math.pow(F,y),outdegree:Math.pow(N,1-y)*Math.pow(R,y)}}else{for(var k=b.connectedEdges().intersection(l),E=k.length,T=0,C=0;C<k.length;C++)T+=d(k[C]);return{degree:Math.pow(E,1-y)*Math.pow(T,y)}}}};RC.dc=RC.degreeCentrality,RC.dcn=RC.degreeCentralityNormalised=RC.degreeCentralityNormalized;var ENe=T1({harmonic:!0,weight:function(){return 1},directed:!1,root:null}),jC={closenessCentralityNormalized:function(i){for(var o=ENe(i),l=o.harmonic,f=o.weight,b=o.directed,d=this.cy(),w={},y=0,k=this.nodes(),E=this.floydWarshall({weight:f,directed:b}),T=0;T<k.length;T++){for(var C=0,S=k[T],L=0;L<k.length;L++)if(T!==L){var O=E.distance(S,k[L]);l?C+=1/O:C+=O}l||(C=1/C),y<C&&(y=C),w[S.id()]=C}return{closeness:function(N){return y==0?0:($i(N)?N=d.filter(N)[0].id():N=N.id(),w[N]/y)}}},closenessCentrality:function(i){var o=ENe(i),l=o.root,f=o.weight,b=o.directed,d=o.harmonic;l=this.filter(l)[0];for(var w=this.dijkstra({root:l,weight:f,directed:b}),y=0,k=this.nodes(),E=0;E<k.length;E++){var T=k[E];if(!T.same(l)){var C=w.distanceTo(T);d?y+=1/C:y+=C}}return d?y:1/y}};jC.cc=jC.closenessCentrality,jC.ccn=jC.closenessCentralityNormalised=jC.closenessCentralityNormalized;var scn=T1({weight:null,directed:!1}),hue={betweennessCentrality:function(i){for(var o=scn(i),l=o.directed,f=o.weight,b=f!=null,d=this.cy(),w=this.nodes(),y={},k={},E=0,T={set:function(R,q){k[R]=q,q>E&&(E=q)},get:function(R){return k[R]}},C=0;C<w.length;C++){var S=w[C],L=S.id();l?y[L]=S.outgoers().nodes():y[L]=S.openNeighborhood().nodes(),T.set(L,0)}for(var O=function(R){for(var q=w[R].id(),X=[],te={},H={},Y={},z=new lO.default(function(Xe,Ee){return Y[Xe]-Y[Ee]}),W=0;W<w.length;W++){var Z=w[W].id();te[Z]=[],H[Z]=0,Y[Z]=1/0}for(H[q]=1,Y[q]=0,z.push(q);!z.empty();){var G=z.pop();if(X.push(G),b)for(var ae=0;ae<y[G].length;ae++){var $=y[G][ae],ge=d.getElementById(G),ee=void 0;ge.edgesTo($).length>0?ee=ge.edgesTo($)[0]:ee=$.edgesTo(ge)[0];var de=f(ee);$=$.id(),Y[$]>Y[G]+de&&(Y[$]=Y[G]+de,z.nodes.indexOf($)<0?z.push($):z.updateItem($),H[$]=0,te[$]=[]),Y[$]==Y[G]+de&&(H[$]=H[$]+H[G],te[$].push(G))}else for(var re=0;re<y[G].length;re++){var ke=y[G][re].id();Y[ke]==1/0&&(z.push(ke),Y[ke]=Y[G]+1),Y[ke]==Y[G]+1&&(H[ke]=H[ke]+H[G],te[ke].push(G))}}for(var Ce={},_e=0;_e<w.length;_e++)Ce[w[_e].id()]=0;for(;X.length>0;){for(var Te=X.pop(),Be=0;Be<te[Te].length;Be++){var Ge=te[Te][Be];Ce[Ge]=Ce[Ge]+H[Ge]/H[Te]*(1+Ce[Te])}Te!=w[R].id()&&T.set(Te,T.get(Te)+Ce[Te])}},B=0;B<w.length;B++)O(B);var N={betweenness:function(R){var q=d.collection(R).id();return T.get(q)},betweennessNormalized:function(R){if(E==0)return 0;var q=d.collection(R).id();return T.get(q)/E}};return N.betweennessNormalised=N.betweennessNormalized,N}};hue.bc=hue.betweennessCentrality;var acn=T1({expandFactor:2,inflateFactor:2,multFactor:1,maxIterations:20,attributes:[function(r){return 1}]}),ocn=function(i){return acn(i)},ccn=function(i,o){for(var l=0,f=0;f<o.length;f++)l+=o[f](i);return l},ucn=function(i,o,l){for(var f=0;f<o;f++)i[f*o+f]=l},_Ne=function(i,o){for(var l,f=0;f<o;f++){l=0;for(var b=0;b<o;b++)l+=i[b*o+f];for(var d=0;d<o;d++)i[d*o+f]=i[d*o+f]/l}},lcn=function(i,o,l){for(var f=new Array(l*l),b=0;b<l;b++){for(var d=0;d<l;d++)f[b*l+d]=0;for(var w=0;w<l;w++)for(var y=0;y<l;y++)f[b*l+y]+=i[b*l+w]*o[w*l+y]}return f},hcn=function(i,o,l){for(var f=i.slice(0),b=1;b<l;b++)i=lcn(i,f,o);return i},fcn=function(i,o,l){for(var f=new Array(o*o),b=0;b<o*o;b++)f[b]=Math.pow(i[b],l);return _Ne(f,o),f},dcn=function(i,o,l,f){for(var b=0;b<l;b++){var d=Math.round(i[b]*Math.pow(10,f))/Math.pow(10,f),w=Math.round(o[b]*Math.pow(10,f))/Math.pow(10,f);if(d!==w)return!1}return!0},gcn=function(i,o,l,f){for(var b=[],d=0;d<o;d++){for(var w=[],y=0;y<o;y++)Math.round(i[d*o+y]*1e3)/1e3>0&&w.push(l[y]);w.length!==0&&b.push(f.collection(w))}return b},pcn=function(i,o){for(var l=0;l<i.length;l++)if(!o[l]||i[l].id()!==o[l].id())return!1;return!0},bcn=function(i){for(var o=0;o<i.length;o++)for(var l=0;l<i.length;l++)o!=l&&pcn(i[o],i[l])&&i.splice(l,1);return i},TNe=function(i){for(var o=this.nodes(),l=this.edges(),f=this.cy(),b=ocn(i),d={},w=0;w<o.length;w++)d[o[w].id()]=w;for(var y=o.length,k=y*y,E=new Array(k),T,C=0;C<k;C++)E[C]=0;for(var S=0;S<l.length;S++){var L=l[S],O=d[L.source().id()],B=d[L.target().id()],N=ccn(L,b.attributes);E[O*y+B]+=N,E[B*y+O]+=N}ucn(E,y,b.multFactor),_Ne(E,y);for(var F=!0,R=0;F&&R<b.maxIterations;)F=!1,T=hcn(E,y,b.expandFactor),E=fcn(T,y,b.inflateFactor),dcn(E,T,k,4)||(F=!0),R++;var q=gcn(E,y,o,f);return q=bcn(q),q},vcn={markovClustering:TNe,mcl:TNe},wcn=function(i){return i},CNe=function(i,o){return Math.abs(o-i)},SNe=function(i,o,l){return i+CNe(o,l)},ANe=function(i,o,l){return i+Math.pow(l-o,2)},mcn=function(i){return Math.sqrt(i)},ycn=function(i,o,l){return Math.max(i,CNe(o,l))},xO=function(i,o,l,f,b){for(var d=arguments.length>5&&arguments[5]!==void 0?arguments[5]:wcn,w=f,y,k,E=0;E<i;E++)y=o(E),k=l(E),w=b(w,y,k);return d(w)},$C={euclidean:function(i,o,l){return i>=2?xO(i,o,l,0,ANe,mcn):xO(i,o,l,0,SNe)},squaredEuclidean:function(i,o,l){return xO(i,o,l,0,ANe)},manhattan:function(i,o,l){return xO(i,o,l,0,SNe)},max:function(i,o,l){return xO(i,o,l,-1/0,ycn)}};$C["squared-euclidean"]=$C.squaredEuclidean,$C.squaredeuclidean=$C.squaredEuclidean;function SU(r,i,o,l,f,b){var d;return al(r)?d=r:d=$C[r]||$C.euclidean,i===0&&al(r)?d(f,b):d(i,o,l,f,b)}var kcn=T1({k:2,m:2,sensitivityThreshold:1e-4,distance:"euclidean",maxIterations:10,attributes:[],testMode:!1,testCentroids:null}),fue=function(i){return kcn(i)},AU=function(i,o,l,f,b){var d=b!=="kMedoids",w=d?function(T){return l[T]}:function(T){return f[T](l)},y=function(C){return f[C](o)},k=l,E=o;return SU(i,f.length,w,y,k,E)},due=function(i,o,l){for(var f=l.length,b=new Array(f),d=new Array(f),w=new Array(o),y=null,k=0;k<f;k++)b[k]=i.min(l[k]).value,d[k]=i.max(l[k]).value;for(var E=0;E<o;E++){y=[];for(var T=0;T<f;T++)y[T]=Math.random()*(d[T]-b[T])+b[T];w[E]=y}return w},MNe=function(i,o,l,f,b){for(var d=1/0,w=0,y=0;y<o.length;y++){var k=AU(l,i,o[y],f,b);k<d&&(d=k,w=y)}return w},DNe=function(i,o,l){for(var f=[],b=null,d=0;d<o.length;d++)b=o[d],l[b.id()]===i&&f.push(b);return f},xcn=function(i,o,l){return Math.abs(o-i)<=l},Ecn=function(i,o,l){for(var f=0;f<i.length;f++)for(var b=0;b<i[f].length;b++){var d=Math.abs(i[f][b]-o[f][b]);if(d>l)return!1}return!0},_cn=function(i,o,l){for(var f=0;f<l;f++)if(i===o[f])return!0;return!1},LNe=function(i,o){var l=new Array(o);if(i.length<50)for(var f=0;f<o;f++){for(var b=i[Math.floor(Math.random()*i.length)];_cn(b,l,f);)b=i[Math.floor(Math.random()*i.length)];l[f]=b}else for(var d=0;d<o;d++)l[d]=i[Math.floor(Math.random()*i.length)];return l},INe=function(i,o,l){for(var f=0,b=0;b<o.length;b++)f+=AU("manhattan",o[b],i,l,"kMedoids");return f},Tcn=function(i){var o=this.cy(),l=this.nodes(),f=null,b=fue(i),d=new Array(b.k),w={},y;b.testMode?typeof b.testCentroids=="number"?(b.testCentroids,y=due(l,b.k,b.attributes)):df(b.testCentroids)==="object"?y=b.testCentroids:y=due(l,b.k,b.attributes):y=due(l,b.k,b.attributes);for(var k=!0,E=0;k&&E<b.maxIterations;){for(var T=0;T<l.length;T++)f=l[T],w[f.id()]=MNe(f,y,b.distance,b.attributes,"kMeans");k=!1;for(var C=0;C<b.k;C++){var S=DNe(C,l,w);if(S.length!==0){for(var L=b.attributes.length,O=y[C],B=new Array(L),N=new Array(L),F=0;F<L;F++){N[F]=0;for(var R=0;R<S.length;R++)f=S[R],N[F]+=b.attributes[F](f);B[F]=N[F]/S.length,xcn(B[F],O[F],b.sensitivityThreshold)||(k=!0)}y[C]=B,d[C]=o.collection(S)}}E++}return d},Ccn=function(i){var o=this.cy(),l=this.nodes(),f=null,b=fue(i),d=new Array(b.k),w,y={},k,E=new Array(b.k);b.testMode?typeof b.testCentroids=="number"||(df(b.testCentroids)==="object"?w=b.testCentroids:w=LNe(l,b.k)):w=LNe(l,b.k);for(var T=!0,C=0;T&&C<b.maxIterations;){for(var S=0;S<l.length;S++)f=l[S],y[f.id()]=MNe(f,w,b.distance,b.attributes,"kMedoids");T=!1;for(var L=0;L<w.length;L++){var O=DNe(L,l,y);if(O.length!==0){E[L]=INe(w[L],O,b.attributes);for(var B=0;B<O.length;B++)k=INe(O[B],O,b.attributes),k<E[L]&&(E[L]=k,w[L]=O[B],T=!0);d[L]=o.collection(O)}}C++}return d},Scn=function(i,o,l,f,b){for(var d,w,y=0;y<o.length;y++)for(var k=0;k<i.length;k++)f[y][k]=Math.pow(l[y][k],b.m);for(var E=0;E<i.length;E++)for(var T=0;T<b.attributes.length;T++){d=0,w=0;for(var C=0;C<o.length;C++)d+=f[C][E]*b.attributes[T](o[C]),w+=f[C][E];i[E][T]=d/w}},Acn=function(i,o,l,f,b){for(var d=0;d<i.length;d++)o[d]=i[d].slice();for(var w,y,k,E=2/(b.m-1),T=0;T<l.length;T++)for(var C=0;C<f.length;C++){w=0;for(var S=0;S<l.length;S++)y=AU(b.distance,f[C],l[T],b.attributes,"cmeans"),k=AU(b.distance,f[C],l[S],b.attributes,"cmeans"),w+=Math.pow(y/k,E);i[C][T]=1/w}},Mcn=function(i,o,l,f){for(var b=new Array(l.k),d=0;d<b.length;d++)b[d]=[];for(var w,y,k=0;k<o.length;k++){w=-1/0,y=-1;for(var E=0;E<o[0].length;E++)o[k][E]>w&&(w=o[k][E],y=E);b[y].push(i[k])}for(var T=0;T<b.length;T++)b[T]=f.collection(b[T]);return b},ONe=function(i){var o=this.cy(),l=this.nodes(),f=fue(i),b,d,w,y,k;y=new Array(l.length);for(var E=0;E<l.length;E++)y[E]=new Array(f.k);w=new Array(l.length);for(var T=0;T<l.length;T++)w[T]=new Array(f.k);for(var C=0;C<l.length;C++){for(var S=0,L=0;L<f.k;L++)w[C][L]=Math.random(),S+=w[C][L];for(var O=0;O<f.k;O++)w[C][O]=w[C][O]/S}d=new Array(f.k);for(var B=0;B<f.k;B++)d[B]=new Array(f.attributes.length);k=new Array(l.length);for(var N=0;N<l.length;N++)k[N]=new Array(f.k);for(var F=!0,R=0;F&&R<f.maxIterations;)F=!1,Scn(d,l,w,k,f),Acn(w,y,d,l,f),Ecn(w,y,f.sensitivityThreshold)||(F=!0),R++;return b=Mcn(l,w,f,o),{clusters:b,degreeOfMembership:w}},Dcn={kMeans:Tcn,kMedoids:Ccn,fuzzyCMeans:ONe,fcm:ONe},Lcn=T1({distance:"euclidean",linkage:"min",mode:"threshold",threshold:1/0,addDendrogram:!1,dendrogramDepth:0,attributes:[]}),Icn={single:"min",complete:"max"},Ocn=function(i){var o=Lcn(i),l=Icn[o.linkage];return l!=null&&(o.linkage=l),o},NNe=function(i,o,l,f,b){for(var d=0,w=1/0,y,k=b.attributes,E=function(z,W){return SU(b.distance,k.length,function(Z){return k[Z](z)},function(Z){return k[Z](W)},z,W)},T=0;T<i.length;T++){var C=i[T].key,S=l[C][f[C]];S<w&&(d=C,w=S)}if(b.mode==="threshold"&&w>=b.threshold||b.mode==="dendrogram"&&i.length===1)return!1;var L=o[d],O=o[f[d]],B;b.mode==="dendrogram"?B={left:L,right:O,key:L.key}:B={value:L.value.concat(O.value),key:L.key},i[L.index]=B,i.splice(O.index,1),o[L.key]=B;for(var N=0;N<i.length;N++){var F=i[N];L.key===F.key?y=1/0:b.linkage==="min"?(y=l[L.key][F.key],l[L.key][F.key]>l[O.key][F.key]&&(y=l[O.key][F.key])):b.linkage==="max"?(y=l[L.key][F.key],l[L.key][F.key]<l[O.key][F.key]&&(y=l[O.key][F.key])):b.linkage==="mean"?y=(l[L.key][F.key]*L.size+l[O.key][F.key]*O.size)/(L.size+O.size):b.mode==="dendrogram"?y=E(F.value,L.value):y=E(F.value[0],L.value[0]),l[L.key][F.key]=l[F.key][L.key]=y}for(var R=0;R<i.length;R++){var q=i[R].key;if(f[q]===L.key||f[q]===O.key){for(var X=q,te=0;te<i.length;te++){var H=i[te].key;l[q][H]<l[q][X]&&(X=H)}f[q]=X}i[R].index=R}return L.key=O.key=L.index=O.index=null,!0},MU=function r(i,o,l){!i||(i.value?o.push(i.value):(i.left&&r(i.left,o),i.right&&r(i.right,o)))},Ncn=function r(i,o){if(!i)return"";if(i.left&&i.right){var l=r(i.left,o),f=r(i.right,o),b=o.add({group:"nodes",data:{id:l+","+f}});return o.add({group:"edges",data:{source:l,target:b.id()}}),o.add({group:"edges",data:{source:f,target:b.id()}}),b.id()}else if(i.value)return i.value.id()},Pcn=function r(i,o,l){if(!i)return[];var f=[],b=[],d=[];return o===0?(i.left&&MU(i.left,f),i.right&&MU(i.right,b),d=f.concat(b),[l.collection(d)]):o===1?i.value?[l.collection(i.value)]:(i.left&&MU(i.left,f),i.right&&MU(i.right,b),[l.collection(f),l.collection(b)]):i.value?[l.collection(i.value)]:(i.left&&(f=r(i.left,o-1,l)),i.right&&(b=r(i.right,o-1,l)),f.concat(b))},PNe=function(i){for(var o=this.cy(),l=this.nodes(),f=Ocn(i),b=f.attributes,d=function(R,q){return SU(f.distance,b.length,function(X){return b[X](R)},function(X){return b[X](q)},R,q)},w=[],y=[],k=[],E=[],T=0;T<l.length;T++){var C={value:f.mode==="dendrogram"?l[T]:[l[T]],key:T,index:T};w[T]=C,E[T]=C,y[T]=[],k[T]=0}for(var S=0;S<w.length;S++)for(var L=0;L<=S;L++){var O=void 0;f.mode==="dendrogram"?O=S===L?1/0:d(w[S].value,w[L].value):O=S===L?1/0:d(w[S].value[0],w[L].value[0]),y[S][L]=O,y[L][S]=O,O<y[S][k[S]]&&(k[S]=L)}for(var B=NNe(w,E,y,k,f);B;)B=NNe(w,E,y,k,f);var N;return f.mode==="dendrogram"?(N=Pcn(w[0],f.dendrogramDepth,o),f.addDendrogram&&Ncn(w[0],o)):(N=new Array(w.length),w.forEach(function(F,R){F.key=F.index=null,N[R]=o.collection(F.value)})),N},Bcn={hierarchicalClustering:PNe,hca:PNe},Fcn=T1({distance:"euclidean",preference:"median",damping:.8,maxIterations:1e3,minIterations:100,attributes:[]}),Rcn=function(i){var o=i.damping,l=i.preference;.5<=o&&o<1||Ml("Damping must range on [0.5, 1).  Got: ".concat(o));var f=["median","mean","min","max"];return f.some(function(b){return b===l})||Vr(l)||Ml("Preference must be one of [".concat(f.map(function(b){return"'".concat(b,"'")}).join(", "),"] or a number.  Got: ").concat(l)),Fcn(i)},jcn=function(i,o,l,f){var b=function(w,y){return f[y](w)};return-SU(i,f.length,function(d){return b(o,d)},function(d){return b(l,d)},o,l)},$cn=function(i,o){var l=null;return o==="median"?l=Ron(i):o==="mean"?l=Fon(i):o==="min"?l=Pon(i):o==="max"?l=Bon(i):l=o,l},Hcn=function(i,o,l){for(var f=[],b=0;b<i;b++)o[b*i+b]+l[b*i+b]>0&&f.push(b);return f},BNe=function(i,o,l){for(var f=[],b=0;b<i;b++){for(var d=-1,w=-1/0,y=0;y<l.length;y++){var k=l[y];o[b*i+k]>w&&(d=k,w=o[b*i+k])}d>0&&f.push(d)}for(var E=0;E<l.length;E++)f[l[E]]=l[E];return f},zcn=function(i,o,l){for(var f=BNe(i,o,l),b=0;b<l.length;b++){for(var d=[],w=0;w<f.length;w++)f[w]===l[b]&&d.push(w);for(var y=-1,k=-1/0,E=0;E<d.length;E++){for(var T=0,C=0;C<d.length;C++)T+=o[d[C]*i+d[E]];T>k&&(y=E,k=T)}l[b]=d[y]}return f=BNe(i,o,l),f},FNe=function(i){for(var o=this.cy(),l=this.nodes(),f=Rcn(i),b={},d=0;d<l.length;d++)b[l[d].id()]=d;var w,y,k,E,T,C;w=l.length,y=w*w,k=new Array(y);for(var S=0;S<y;S++)k[S]=-1/0;for(var L=0;L<w;L++)for(var O=0;O<w;O++)L!==O&&(k[L*w+O]=jcn(f.distance,l[L],l[O],f.attributes));E=$cn(k,f.preference);for(var B=0;B<w;B++)k[B*w+B]=E;T=new Array(y);for(var N=0;N<y;N++)T[N]=0;C=new Array(y);for(var F=0;F<y;F++)C[F]=0;for(var R=new Array(w),q=new Array(w),X=new Array(w),te=0;te<w;te++)R[te]=0,q[te]=0,X[te]=0;for(var H=new Array(w*f.minIterations),Y=0;Y<H.length;Y++)H[Y]=0;var z;for(z=0;z<f.maxIterations;z++){for(var W=0;W<w;W++){for(var Z=-1/0,G=-1/0,ae=-1,$=0,ge=0;ge<w;ge++)R[ge]=T[W*w+ge],$=C[W*w+ge]+k[W*w+ge],$>=Z?(G=Z,Z=$,ae=ge):$>G&&(G=$);for(var ee=0;ee<w;ee++)T[W*w+ee]=(1-f.damping)*(k[W*w+ee]-Z)+f.damping*R[ee];T[W*w+ae]=(1-f.damping)*(k[W*w+ae]-G)+f.damping*R[ae]}for(var de=0;de<w;de++){for(var re=0,ke=0;ke<w;ke++)R[ke]=C[ke*w+de],q[ke]=Math.max(0,T[ke*w+de]),re+=q[ke];re-=q[de],q[de]=T[de*w+de],re+=q[de];for(var Ce=0;Ce<w;Ce++)C[Ce*w+de]=(1-f.damping)*Math.min(0,re-q[Ce])+f.damping*R[Ce];C[de*w+de]=(1-f.damping)*(re-q[de])+f.damping*R[de]}for(var _e=0,Te=0;Te<w;Te++){var Be=C[Te*w+Te]+T[Te*w+Te]>0?1:0;H[z%f.minIterations*w+Te]=Be,_e+=Be}if(_e>0&&(z>=f.minIterations-1||z==f.maxIterations-1)){for(var Ge=0,Xe=0;Xe<w;Xe++){X[Xe]=0;for(var Ee=0;Ee<f.minIterations;Ee++)X[Xe]+=H[Ee*w+Xe];(X[Xe]===0||X[Xe]===f.minIterations)&&Ge++}if(Ge===w)break}}for(var Ze=Hcn(w,T,C),Ie=zcn(w,k,Ze),Oe={},Le=0;Le<Ze.length;Le++)Oe[Ze[Le]]=[];for(var $e=0;$e<l.length;$e++){var xe=b[l[$e].id()],Ae=Ie[xe];Ae!=null&&Oe[Ae].push(l[$e])}for(var je=new Array(Ze.length),me=0;me<Ze.length;me++)je[me]=o.collection(Oe[Ze[me]]);return je},Gcn={affinityPropagation:FNe,ap:FNe},Vcn=T1({root:void 0,directed:!1}),Ucn={hierholzer:function(i){if(!_o(i)){var o=arguments;i={root:o[0],directed:o[1]}}var l=Vcn(i),f=l.root,b=l.directed,d=this,w=!1,y,k,E;f&&(E=$i(f)?this.filter(f)[0].id():f[0].id());var T={},C={};b?d.forEach(function(F){var R=F.id();if(F.isNode()){var q=F.indegree(!0),X=F.outdegree(!0),te=q-X,H=X-q;te==1?y?w=!0:y=R:H==1?k?w=!0:k=R:(H>1||te>1)&&(w=!0),T[R]=[],F.outgoers().forEach(function(Y){Y.isEdge()&&T[R].push(Y.id())})}else C[R]=[void 0,F.target().id()]}):d.forEach(function(F){var R=F.id();if(F.isNode()){var q=F.degree(!0);q%2&&(y?k?w=!0:k=R:y=R),T[R]=[],F.connectedEdges().forEach(function(X){return T[R].push(X.id())})}else C[R]=[F.source().id(),F.target().id()]});var S={found:!1,trail:void 0};if(w)return S;if(k&&y)if(b){if(E&&k!=E)return S;E=k}else{if(E&&k!=E&&y!=E)return S;E||(E=k)}else E||(E=d[0].id());var L=function(R){for(var q=R,X=[R],te,H,Y;T[q].length;)te=T[q].shift(),H=C[te][0],Y=C[te][1],q!=Y?(T[Y]=T[Y].filter(function(z){return z!=te}),q=Y):!b&&q!=H&&(T[H]=T[H].filter(function(z){return z!=te}),q=H),X.unshift(te),X.unshift(q);return X},O=[],B=[];for(B=L(E);B.length!=1;)T[B[0]].length==0?(O.unshift(d.getElementById(B.shift())),O.unshift(d.getElementById(B.shift()))):B=L(B.shift()).concat(B);O.unshift(d.getElementById(B.shift()));for(var N in T)if(T[N].length)return S;return S.found=!0,S.trail=this.spawn(O,!0),S}},DU=function(){var i=this,o={},l=0,f=0,b=[],d=[],w={},y=function(C,S){for(var L=d.length-1,O=[],B=i.spawn();d[L].x!=C||d[L].y!=S;)O.push(d.pop().edge),L--;O.push(d.pop().edge),O.forEach(function(N){var F=N.connectedNodes().intersection(i);B.merge(N),F.forEach(function(R){var q=R.id(),X=R.connectedEdges().intersection(i);B.merge(R),o[q].cutVertex?B.merge(X.filter(function(te){return te.isLoop()})):B.merge(X)})}),b.push(B)},k=function T(C,S,L){C===L&&(f+=1),o[S]={id:l,low:l++,cutVertex:!1};var O=i.getElementById(S).connectedEdges().intersection(i);if(O.size()===0)b.push(i.spawn(i.getElementById(S)));else{var B,N,F,R;O.forEach(function(q){B=q.source().id(),N=q.target().id(),F=B===S?N:B,F!==L&&(R=q.id(),w[R]||(w[R]=!0,d.push({x:S,y:F,edge:q})),F in o?o[S].low=Math.min(o[S].low,o[F].id):(T(C,F,S),o[S].low=Math.min(o[S].low,o[F].low),o[S].id<=o[F].low&&(o[S].cutVertex=!0,y(S,F))))})}};i.forEach(function(T){if(T.isNode()){var C=T.id();C in o||(f=0,k(C,C),o[C].cutVertex=f>1)}});var E=Object.keys(o).filter(function(T){return o[T].cutVertex}).map(function(T){return i.getElementById(T)});return{cut:i.spawn(E),components:b}},qcn={hopcroftTarjanBiconnected:DU,htbc:DU,htb:DU,hopcroftTarjanBiconnectedComponents:DU},LU=function(){var i=this,o={},l=0,f=[],b=[],d=i.spawn(i),w=function y(k){b.push(k),o[k]={index:l,low:l++,explored:!1};var E=i.getElementById(k).connectedEdges().intersection(i);if(E.forEach(function(O){var B=O.target().id();B!==k&&(B in o||y(B),o[B].explored||(o[k].low=Math.min(o[k].low,o[B].low)))}),o[k].index===o[k].low){for(var T=i.spawn();;){var C=b.pop();if(T.merge(i.getElementById(C)),o[C].low=o[k].index,o[C].explored=!0,C===k)break}var S=T.edgesWith(T),L=T.merge(S);f.push(L),d=d.difference(L)}};return i.forEach(function(y){if(y.isNode()){var k=y.id();k in o||w(k)}}),{cut:d,components:f}},Ycn={tarjanStronglyConnected:LU,tsc:LU,tscc:LU,tarjanStronglyConnectedComponents:LU},RNe={};[vO,Eon,_on,Con,Aon,Don,Oon,icn,RC,jC,hue,vcn,Dcn,Bcn,Gcn,Ucn,qcn,Ycn].forEach(function(r){bs(RNe,r)});/*!
Embeddable Minimum Strictly-Compliant Promises/A+ 1.1.1 Thenable
Copyright (c) 2013-2014 Ralf S. Engelschall (http://engelschall.com)
Licensed under The MIT License (http://opensource.org/licenses/MIT)
*/var jNe=0,$Ne=1,HNe=2,p4=function r(i){if(!(this instanceof r))return new r(i);this.id="Thenable/1.0.7",this.state=jNe,this.fulfillValue=void 0,this.rejectReason=void 0,this.onFulfilled=[],this.onRejected=[],this.proxy={then:this.then.bind(this)},typeof i=="function"&&i.call(this,this.fulfill.bind(this),this.reject.bind(this))};p4.prototype={fulfill:function(i){return zNe(this,$Ne,"fulfillValue",i)},reject:function(i){return zNe(this,HNe,"rejectReason",i)},then:function(i,o){var l=this,f=new p4;return l.onFulfilled.push(UNe(i,f,"fulfill")),l.onRejected.push(UNe(o,f,"reject")),GNe(l),f.proxy}};var zNe=function(i,o,l,f){return i.state===jNe&&(i.state=o,i[l]=f,GNe(i)),i},GNe=function(i){i.state===$Ne?VNe(i,"onFulfilled",i.fulfillValue):i.state===HNe&&VNe(i,"onRejected",i.rejectReason)},VNe=function(i,o,l){if(i[o].length!==0){var f=i[o];i[o]=[];var b=function(){for(var w=0;w<f.length;w++)f[w](l)};typeof setImmediate=="function"?setImmediate(b):setTimeout(b,0)}},UNe=function(i,o,l){return function(f){if(typeof i!="function")o[l].call(o,f);else{var b;try{b=i(f)}catch(d){o.reject(d);return}Wcn(o,b)}}},Wcn=function r(i,o){if(i===o||i.proxy===o){i.reject(new TypeError("cannot resolve promise with itself"));return}var l;if(df(o)==="object"&&o!==null||typeof o=="function")try{l=o.then}catch(b){i.reject(b);return}if(typeof l=="function"){var f=!1;try{l.call(o,function(b){f||(f=!0,b===o?i.reject(new TypeError("circular thenable chain")):r(i,b))},function(b){f||(f=!0,i.reject(b))})}catch(b){f||i.reject(b)}return}i.fulfill(o)};p4.all=function(r){return new p4(function(i,o){for(var l=new Array(r.length),f=0,b=function(y,k){l[y]=k,f++,f===r.length&&i(l)},d=0;d<r.length;d++)(function(w){var y=r[w],k=y!=null&&y.then!=null;if(k)y.then(function(T){b(w,T)},function(T){o(T)});else{var E=y;b(w,E)}})(d)})},p4.resolve=function(r){return new p4(function(i,o){i(r)})},p4.reject=function(r){return new p4(function(i,o){o(r)})};var HC=typeof Promise<"u"?Promise:p4,gue=function(i,o,l){var f=Qce(i),b=!f,d=this._private=bs({duration:1e3},o,l);if(d.target=i,d.style=d.style||d.css,d.started=!1,d.playing=!1,d.hooked=!1,d.applying=!1,d.progress=0,d.completes=[],d.frames=[],d.complete&&al(d.complete)&&d.completes.push(d.complete),b){var w=i.position();d.startPosition=d.startPosition||{x:w.x,y:w.y},d.startStyle=d.startStyle||i.cy().style().getAnimationStartStyle(i,d.style)}if(f){var y=i.pan();d.startPan={x:y.x,y:y.y},d.startZoom=i.zoom()}this.length=1,this[0]=this},x7=gue.prototype;bs(x7,{instanceString:function(){return"animation"},hook:function(){var i=this._private;if(!i.hooked){var o,l=i.target._private.animation;i.queue?o=l.queue:o=l.current,o.push(this),vp(i.target)&&i.target.cy().addToAnimationPool(i.target),i.hooked=!0}return this},play:function(){var i=this._private;return i.progress===1&&(i.progress=0),i.playing=!0,i.started=!1,i.stopped=!1,this.hook(),this},playing:function(){return this._private.playing},apply:function(){var i=this._private;return i.applying=!0,i.started=!1,i.stopped=!1,this.hook(),this},applying:function(){return this._private.applying},pause:function(){var i=this._private;return i.playing=!1,i.started=!1,this},stop:function(){var i=this._private;return i.playing=!1,i.started=!1,i.stopped=!0,this},rewind:function(){return this.progress(0)},fastforward:function(){return this.progress(1)},time:function(i){var o=this._private;return i===void 0?o.progress*o.duration:this.progress(i/o.duration)},progress:function(i){var o=this._private,l=o.playing;return i===void 0?o.progress:(l&&this.pause(),o.progress=i,o.started=!1,l&&this.play(),this)},completed:function(){return this._private.progress===1},reverse:function(){var i=this._private,o=i.playing;o&&this.pause(),i.progress=1-i.progress,i.started=!1;var l=function(k,E){var T=i[k];T!=null&&(i[k]=i[E],i[E]=T)};if(l("zoom","startZoom"),l("pan","startPan"),l("position","startPosition"),i.style)for(var f=0;f<i.style.length;f++){var b=i.style[f],d=b.name,w=i.startStyle[d];i.startStyle[d]=b,i.style[f]=w}return o&&this.play(),this},promise:function(i){var o=this._private,l;switch(i){case"frame":l=o.frames;break;default:case"complete":case"completed":l=o.completes}return new HC(function(f,b){l.push(function(){f()})})}}),x7.complete=x7.completed,x7.run=x7.play,x7.running=x7.playing;var Kcn={animated:function(){return function(){var o=this,l=o.length!==void 0,f=l?o:[o],b=this._private.cy||this;if(!b.styleEnabled())return!1;var d=f[0];if(d)return d._private.animation.current.length>0}},clearQueue:function(){return function(){var o=this,l=o.length!==void 0,f=l?o:[o],b=this._private.cy||this;if(!b.styleEnabled())return this;for(var d=0;d<f.length;d++){var w=f[d];w._private.animation.queue=[]}return this}},delay:function(){return function(o,l){var f=this._private.cy||this;return f.styleEnabled()?this.animate({delay:o,duration:o,complete:l}):this}},delayAnimation:function(){return function(o,l){var f=this._private.cy||this;return f.styleEnabled()?this.animation({delay:o,duration:o,complete:l}):this}},animation:function(){return function(o,l){var f=this,b=f.length!==void 0,d=b?f:[f],w=this._private.cy||this,y=!b,k=!y;if(!w.styleEnabled())return this;var E=w.style();o=bs({},o,l);var T=Object.keys(o).length===0;if(T)return new gue(d[0],o);switch(o.duration===void 0&&(o.duration=400),o.duration){case"slow":o.duration=600;break;case"fast":o.duration=200;break}if(k&&(o.style=E.getPropsList(o.style||o.css),o.css=void 0),k&&o.renderedPosition!=null){var C=o.renderedPosition,S=w.pan(),L=w.zoom();o.position=dNe(C,L,S)}if(y&&o.panBy!=null){var O=o.panBy,B=w.pan();o.pan={x:B.x+O.x,y:B.y+O.y}}var N=o.center||o.centre;if(y&&N!=null){var F=w.getCenterPan(N.eles,o.zoom);F!=null&&(o.pan=F)}if(y&&o.fit!=null){var R=o.fit,q=w.getFitViewport(R.eles||R.boundingBox,R.padding);q!=null&&(o.pan=q.pan,o.zoom=q.zoom)}if(y&&_o(o.zoom)){var X=w.getZoomedViewport(o.zoom);X!=null?(X.zoomed&&(o.zoom=X.zoom),X.panned&&(o.pan=X.pan)):o.zoom=null}return new gue(d[0],o)}},animate:function(){return function(o,l){var f=this,b=f.length!==void 0,d=b?f:[f],w=this._private.cy||this;if(!w.styleEnabled())return this;l&&(o=bs({},o,l));for(var y=0;y<d.length;y++){var k=d[y],E=k.animated()&&(o.queue===void 0||o.queue),T=k.animation(o,E?{queue:!0}:void 0);T.play()}return this}},stop:function(){return function(o,l){var f=this,b=f.length!==void 0,d=b?f:[f],w=this._private.cy||this;if(!w.styleEnabled())return this;for(var y=0;y<d.length;y++){for(var k=d[y],E=k._private,T=E.animation.current,C=0;C<T.length;C++){var S=T[C],L=S._private;l&&(L.duration=0)}o&&(E.animation.queue=[]),l||(E.animation.current=[])}return w.notify("draw"),this}}},Xcn={data:function(i){var o={field:"data",bindingEvent:"data",allowBinding:!1,allowSetting:!1,allowGetting:!1,settingEvent:"data",settingTriggersEvent:!1,triggerFnName:"trigger",immutableKeys:{},updateStyle:!1,beforeGet:function(f){},beforeSet:function(f,b){},onSet:function(f){},canSet:function(f){return!0}};return i=bs({},o,i),function(f,b){var d=i,w=this,y=w.length!==void 0,k=y?w:[w],E=y?w[0]:w;if($i(f)){var T=f.indexOf(".")!==-1,C=T&&Pan.default(f);if(d.allowGetting&&b===void 0){var S;return E&&(d.beforeGet(E),C&&E._private[d.field][f]===void 0?S=Oan.default(E._private[d.field],C):S=E._private[d.field][f]),S}else if(d.allowSetting&&b!==void 0){var L=!d.immutableKeys[f];if(L){var O=UOe({},f,b);d.beforeSet(w,O);for(var B=0,N=k.length;B<N;B++){var F=k[B];d.canSet(F)&&(C&&E._private[d.field][f]===void 0?Nan.default(F._private[d.field],C,b):F._private[d.field][f]=b)}d.updateStyle&&w.updateStyle(),d.onSet(w),d.settingTriggersEvent&&w[d.triggerFnName](d.settingEvent)}}}else if(d.allowSetting&&_o(f)){var R=f,q,X,te=Object.keys(R);d.beforeSet(w,R);for(var H=0;H<te.length;H++){q=te[H],X=R[q];var Y=!d.immutableKeys[q];if(Y)for(var z=0;z<k.length;z++){var W=k[z];d.canSet(W)&&(W._private[d.field][q]=X)}}d.updateStyle&&w.updateStyle(),d.onSet(w),d.settingTriggersEvent&&w[d.triggerFnName](d.settingEvent)}else if(d.allowBinding&&al(f)){var Z=f;w.on(d.bindingEvent,Z)}else if(d.allowGetting&&f===void 0){var G;return E&&(d.beforeGet(E),G=E._private[d.field]),G}return w}},removeData:function(i){var o={field:"data",event:"data",triggerFnName:"trigger",triggerEvent:!1,immutableKeys:{}};return i=bs({},o,i),function(f){var b=i,d=this,w=d.length!==void 0,y=w?d:[d];if($i(f)){for(var k=f.split(/\s+/),E=k.length,T=0;T<E;T++){var C=k[T];if(!y6(C)){var S=!b.immutableKeys[C];if(S)for(var L=0,O=y.length;L<O;L++)y[L]._private[b.field][C]=void 0}}b.triggerEvent&&d[b.triggerFnName](b.event)}else if(f===void 0){for(var B=0,N=y.length;B<N;B++)for(var F=y[B]._private[b.field],R=Object.keys(F),q=0;q<R.length;q++){var X=R[q],te=!b.immutableKeys[X];te&&(F[X]=void 0)}b.triggerEvent&&d[b.triggerFnName](b.event)}return d}}},Qcn={eventAliasesOn:function(i){var o=i;o.addListener=o.listen=o.bind=o.on,o.unlisten=o.unbind=o.off=o.removeListener,o.trigger=o.emit,o.pon=o.promiseOn=function(l,f){var b=this,d=Array.prototype.slice.call(arguments,0);return new HC(function(w,y){var k=function(S){b.off.apply(b,T),w(S)},E=d.concat([k]),T=E.concat([]);b.on.apply(b,E)})}}},nc={};[Kcn,Xcn,Qcn].forEach(function(r){bs(nc,r)});var Zcn={animate:nc.animate(),animation:nc.animation(),animated:nc.animated(),clearQueue:nc.clearQueue(),delay:nc.delay(),delayAnimation:nc.delayAnimation(),stop:nc.stop()},IU={classes:function(i){var o=this;if(i===void 0){var l=[];return o[0]._private.classes.forEach(function(L){return l.push(L)}),l}else qc(i)||(i=(i||"").match(/\S+/g)||[]);for(var f=[],b=new NC(i),d=0;d<o.length;d++){for(var w=o[d],y=w._private,k=y.classes,E=!1,T=0;T<i.length;T++){var C=i[T],S=k.has(C);if(!S){E=!0;break}}E||(E=k.size!==i.length),E&&(y.classes=b,f.push(w))}return f.length>0&&this.spawn(f).updateStyle().emit("class"),o},addClass:function(i){return this.toggleClass(i,!0)},hasClass:function(i){var o=this[0];return o!=null&&o._private.classes.has(i)},toggleClass:function(i,o){qc(i)||(i=i.match(/\S+/g)||[]);for(var l=this,f=o===void 0,b=[],d=0,w=l.length;d<w;d++)for(var y=l[d],k=y._private.classes,E=!1,T=0;T<i.length;T++){var C=i[T],S=k.has(C),L=!1;o||f&&!S?(k.add(C),L=!0):(!o||f&&S)&&(k.delete(C),L=!0),!E&&L&&(b.push(y),E=!0)}return b.length>0&&this.spawn(b).updateStyle().emit("class"),l},removeClass:function(i){return this.toggleClass(i,!1)},flashClass:function(i,o){var l=this;if(o==null)o=250;else if(o===0)return l;return l.addClass(i),setTimeout(function(){l.removeClass(i)},o),l}};IU.className=IU.classNames=IU.classes;var To={metaChar:"[\\!\\\"\\#\\$\\%\\&\\'\\(\\)\\*\\+\\,\\.\\/\\:\\;\\<\\=\\>\\?\\@\\[\\]\\^\\`\\{\\|\\}\\~]",comparatorOp:"=|\\!=|>|>=|<|<=|\\$=|\\^=|\\*=",boolOp:"\\?|\\!|\\^",string:`"(?:\\\\"|[^"])*"|'(?:\\\\'|[^'])*'`,number:gf,meta:"degree|indegree|outdegree",separator:"\\s*,\\s*",descendant:"\\s+",child:"\\s+>\\s+",subject:"\\$",group:"node|edge|\\*",directedEdge:"\\s+->\\s+",undirectedEdge:"\\s+<->\\s+"};To.variable="(?:[\\w-.]|(?:\\\\"+To.metaChar+"))+",To.className="(?:[\\w-]|(?:\\\\"+To.metaChar+"))+",To.value=To.string+"|"+To.number,To.id=To.variable,function(){var r,i,o;for(r=To.comparatorOp.split("|"),o=0;o<r.length;o++)i=r[o],To.comparatorOp+="|@"+i;for(r=To.comparatorOp.split("|"),o=0;o<r.length;o++)i=r[o],!(i.indexOf("!")>=0)&&i!=="="&&(To.comparatorOp+="|\\!"+i)}();var Dc=function(){return{checks:[]}},li={GROUP:0,COLLECTION:1,FILTER:2,DATA_COMPARE:3,DATA_EXIST:4,DATA_BOOL:5,META_COMPARE:6,STATE:7,ID:8,CLASS:9,UNDIRECTED_EDGE:10,DIRECTED_EDGE:11,NODE_SOURCE:12,NODE_TARGET:13,NODE_NEIGHBOR:14,CHILD:15,DESCENDANT:16,PARENT:17,ANCESTOR:18,COMPOUND_SPLIT:19,TRUE:20},pue=[{selector:":selected",matches:function(i){return i.selected()}},{selector:":unselected",matches:function(i){return!i.selected()}},{selector:":selectable",matches:function(i){return i.selectable()}},{selector:":unselectable",matches:function(i){return!i.selectable()}},{selector:":locked",matches:function(i){return i.locked()}},{selector:":unlocked",matches:function(i){return!i.locked()}},{selector:":visible",matches:function(i){return i.visible()}},{selector:":hidden",matches:function(i){return!i.visible()}},{selector:":transparent",matches:function(i){return i.transparent()}},{selector:":grabbed",matches:function(i){return i.grabbed()}},{selector:":free",matches:function(i){return!i.grabbed()}},{selector:":removed",matches:function(i){return i.removed()}},{selector:":inside",matches:function(i){return!i.removed()}},{selector:":grabbable",matches:function(i){return i.grabbable()}},{selector:":ungrabbable",matches:function(i){return!i.grabbable()}},{selector:":animated",matches:function(i){return i.animated()}},{selector:":unanimated",matches:function(i){return!i.animated()}},{selector:":parent",matches:function(i){return i.isParent()}},{selector:":childless",matches:function(i){return i.isChildless()}},{selector:":child",matches:function(i){return i.isChild()}},{selector:":orphan",matches:function(i){return i.isOrphan()}},{selector:":nonorphan",matches:function(i){return i.isChild()}},{selector:":compound",matches:function(i){return i.isNode()?i.isParent():i.source().isParent()||i.target().isParent()}},{selector:":loop",matches:function(i){return i.isLoop()}},{selector:":simple",matches:function(i){return i.isSimple()}},{selector:":active",matches:function(i){return i.active()}},{selector:":inactive",matches:function(i){return!i.active()}},{selector:":backgrounding",matches:function(i){return i.backgrounding()}},{selector:":nonbackgrounding",matches:function(i){return!i.backgrounding()}}].sort(function(r,i){return non(r.selector,i.selector)}),Jcn=function(){for(var r={},i,o=0;o<pue.length;o++)i=pue[o],r[i.selector]=i.matches;return r}(),eun=function(i,o){return Jcn[i](o)},tun="("+pue.map(function(r){return r.selector}).join("|")+")",zC=function(i){return i.replace(new RegExp("\\\\("+To.metaChar+")","g"),function(o,l){return l})},T6=function(i,o,l){i[i.length-1]=l},bue=[{name:"group",query:!0,regex:"("+To.group+")",populate:function(i,o,l){var f=f4(l,1),b=f[0];o.checks.push({type:li.GROUP,value:b==="*"?b:b+"s"})}},{name:"state",query:!0,regex:tun,populate:function(i,o,l){var f=f4(l,1),b=f[0];o.checks.push({type:li.STATE,value:b})}},{name:"id",query:!0,regex:"\\#("+To.id+")",populate:function(i,o,l){var f=f4(l,1),b=f[0];o.checks.push({type:li.ID,value:zC(b)})}},{name:"className",query:!0,regex:"\\.("+To.className+")",populate:function(i,o,l){var f=f4(l,1),b=f[0];o.checks.push({type:li.CLASS,value:zC(b)})}},{name:"dataExists",query:!0,regex:"\\[\\s*("+To.variable+")\\s*\\]",populate:function(i,o,l){var f=f4(l,1),b=f[0];o.checks.push({type:li.DATA_EXIST,field:zC(b)})}},{name:"dataCompare",query:!0,regex:"\\[\\s*("+To.variable+")\\s*("+To.comparatorOp+")\\s*("+To.value+")\\s*\\]",populate:function(i,o,l){var f=f4(l,3),b=f[0],d=f[1],w=f[2],y=new RegExp("^"+To.string+"$").exec(w)!=null;y?w=w.substring(1,w.length-1):w=parseFloat(w),o.checks.push({type:li.DATA_COMPARE,field:zC(b),operator:d,value:w})}},{name:"dataBool",query:!0,regex:"\\[\\s*("+To.boolOp+")\\s*("+To.variable+")\\s*\\]",populate:function(i,o,l){var f=f4(l,2),b=f[0],d=f[1];o.checks.push({type:li.DATA_BOOL,field:zC(d),operator:b})}},{name:"metaCompare",query:!0,regex:"\\[\\[\\s*("+To.meta+")\\s*("+To.comparatorOp+")\\s*("+To.number+")\\s*\\]\\]",populate:function(i,o,l){var f=f4(l,3),b=f[0],d=f[1],w=f[2];o.checks.push({type:li.META_COMPARE,field:zC(b),operator:d,value:parseFloat(w)})}},{name:"nextQuery",separator:!0,regex:To.separator,populate:function(i,o){var l=i.currentSubject,f=i.edgeCount,b=i.compoundCount,d=i[i.length-1];l!=null&&(d.subject=l,i.currentSubject=null),d.edgeCount=f,d.compoundCount=b,i.edgeCount=0,i.compoundCount=0;var w=i[i.length++]=Dc();return w}},{name:"directedEdge",separator:!0,regex:To.directedEdge,populate:function(i,o){if(i.currentSubject==null){var l=Dc(),f=o,b=Dc();return l.checks.push({type:li.DIRECTED_EDGE,source:f,target:b}),T6(i,o,l),i.edgeCount++,b}else{var d=Dc(),w=o,y=Dc();return d.checks.push({type:li.NODE_SOURCE,source:w,target:y}),T6(i,o,d),i.edgeCount++,y}}},{name:"undirectedEdge",separator:!0,regex:To.undirectedEdge,populate:function(i,o){if(i.currentSubject==null){var l=Dc(),f=o,b=Dc();return l.checks.push({type:li.UNDIRECTED_EDGE,nodes:[f,b]}),T6(i,o,l),i.edgeCount++,b}else{var d=Dc(),w=o,y=Dc();return d.checks.push({type:li.NODE_NEIGHBOR,node:w,neighbor:y}),T6(i,o,d),y}}},{name:"child",separator:!0,regex:To.child,populate:function(i,o){if(i.currentSubject==null){var l=Dc(),f=Dc(),b=i[i.length-1];return l.checks.push({type:li.CHILD,parent:b,child:f}),T6(i,o,l),i.compoundCount++,f}else if(i.currentSubject===o){var d=Dc(),w=i[i.length-1],y=Dc(),k=Dc(),E=Dc(),T=Dc();return d.checks.push({type:li.COMPOUND_SPLIT,left:w,right:y,subject:k}),k.checks=o.checks,o.checks=[{type:li.TRUE}],T.checks.push({type:li.TRUE}),y.checks.push({type:li.PARENT,parent:T,child:E}),T6(i,w,d),i.currentSubject=k,i.compoundCount++,E}else{var C=Dc(),S=Dc(),L=[{type:li.PARENT,parent:C,child:S}];return C.checks=o.checks,o.checks=L,i.compoundCount++,S}}},{name:"descendant",separator:!0,regex:To.descendant,populate:function(i,o){if(i.currentSubject==null){var l=Dc(),f=Dc(),b=i[i.length-1];return l.checks.push({type:li.DESCENDANT,ancestor:b,descendant:f}),T6(i,o,l),i.compoundCount++,f}else if(i.currentSubject===o){var d=Dc(),w=i[i.length-1],y=Dc(),k=Dc(),E=Dc(),T=Dc();return d.checks.push({type:li.COMPOUND_SPLIT,left:w,right:y,subject:k}),k.checks=o.checks,o.checks=[{type:li.TRUE}],T.checks.push({type:li.TRUE}),y.checks.push({type:li.ANCESTOR,ancestor:T,descendant:E}),T6(i,w,d),i.currentSubject=k,i.compoundCount++,E}else{var C=Dc(),S=Dc(),L=[{type:li.ANCESTOR,ancestor:C,descendant:S}];return C.checks=o.checks,o.checks=L,i.compoundCount++,S}}},{name:"subject",modifier:!0,regex:To.subject,populate:function(i,o){if(i.currentSubject!=null&&i.currentSubject!==o)return tc("Redefinition of subject in selector `"+i.toString()+"`"),!1;i.currentSubject=o;var l=i[i.length-1],f=l.checks[0],b=f==null?null:f.type;b===li.DIRECTED_EDGE?f.type=li.NODE_TARGET:b===li.UNDIRECTED_EDGE&&(f.type=li.NODE_NEIGHBOR,f.node=f.nodes[1],f.neighbor=f.nodes[0],f.nodes=null)}}];bue.forEach(function(r){return r.regexObj=new RegExp("^"+r.regex)});var nun=function(i){for(var o,l,f,b=0;b<bue.length;b++){var d=bue[b],w=d.name,y=i.match(d.regexObj);if(y!=null){l=y,o=d,f=w;var k=y[0];i=i.substring(k.length);break}}return{expr:o,match:l,name:f,remaining:i}},run=function(i){var o=i.match(/^\s+/);if(o){var l=o[0];i=i.substring(l.length)}return i},iun=function(i){var o=this,l=o.inputText=i,f=o[0]=Dc();for(o.length=1,l=run(l);;){var b=nun(l);if(b.expr==null)return tc("The selector `"+i+"`is invalid"),!1;var d=b.match.slice(1),w=b.expr.populate(o,f,d);if(w===!1)return!1;if(w!=null&&(f=w),l=b.remaining,l.match(/^\s*$/))break}var y=o[o.length-1];o.currentSubject!=null&&(y.subject=o.currentSubject),y.edgeCount=o.edgeCount,y.compoundCount=o.compoundCount;for(var k=0;k<o.length;k++){var E=o[k];if(E.compoundCount>0&&E.edgeCount>0)return tc("The selector `"+i+"` is invalid because it uses both a compound selector and an edge selector"),!1;if(E.edgeCount>1)return tc("The selector `"+i+"` is invalid because it uses multiple edge selectors"),!1;E.edgeCount===1&&tc("The selector `"+i+"` is deprecated.  Edge selectors do not take effect on changes to source and target nodes after an edge is added, for performance reasons.  Use a class or data selector on edges instead, updating the class or data of an edge when your app detects a change in source or target nodes.")}return!0},sun=function(){if(this.toStringCache!=null)return this.toStringCache;for(var i=function(E){return E==null?"":E},o=function(E){return $i(E)?'"'+E+'"':i(E)},l=function(E){return" "+E+" "},f=function(E,T){var C=E.type,S=E.value;switch(C){case li.GROUP:{var L=i(S);return L.substring(0,L.length-1)}case li.DATA_COMPARE:{var O=E.field,B=E.operator;return"["+O+l(i(B))+o(S)+"]"}case li.DATA_BOOL:{var N=E.operator,F=E.field;return"["+i(N)+F+"]"}case li.DATA_EXIST:{var R=E.field;return"["+R+"]"}case li.META_COMPARE:{var q=E.operator,X=E.field;return"[["+X+l(i(q))+o(S)+"]]"}case li.STATE:return S;case li.ID:return"#"+S;case li.CLASS:return"."+S;case li.PARENT:case li.CHILD:return b(E.parent,T)+l(">")+b(E.child,T);case li.ANCESTOR:case li.DESCENDANT:return b(E.ancestor,T)+" "+b(E.descendant,T);case li.COMPOUND_SPLIT:{var te=b(E.left,T),H=b(E.subject,T),Y=b(E.right,T);return te+(te.length>0?" ":"")+H+Y}case li.TRUE:return""}},b=function(E,T){return E.checks.reduce(function(C,S,L){return C+(T===E&&L===0?"$":"")+f(S,T)},"")},d="",w=0;w<this.length;w++){var y=this[w];d+=b(y,y.subject),this.length>1&&w<this.length-1&&(d+=", ")}return this.toStringCache=d,d},aun={parse:iun,toString:sun},qNe=function(i,o,l){var f,b=$i(i),d=Vr(i),w=$i(l),y,k,E=!1,T=!1,C=!1;switch(o.indexOf("!")>=0&&(o=o.replace("!",""),T=!0),o.indexOf("@")>=0&&(o=o.replace("@",""),E=!0),(b||w||E)&&(y=!b&&!d?"":""+i,k=""+l),E&&(i=y=y.toLowerCase(),l=k=k.toLowerCase()),o){case"*=":f=y.indexOf(k)>=0;break;case"$=":f=y.indexOf(k,y.length-k.length)>=0;break;case"^=":f=y.indexOf(k)===0;break;case"=":f=i===l;break;case">":C=!0,f=i>l;break;case">=":C=!0,f=i>=l;break;case"<":C=!0,f=i<l;break;case"<=":C=!0,f=i<=l;break;default:f=!1;break}return T&&(i!=null||!C)&&(f=!f),f},oun=function(i,o){switch(o){case"?":return!!i;case"!":return!i;case"^":return i===void 0}},cun=function(i){return i!==void 0},vue=function(i,o){return i.data(o)},uun=function(i,o){return i[o]()},Dl=[],ku=function(i,o){return i.checks.every(function(l){return Dl[l.type](l,o)})};Dl[li.GROUP]=function(r,i){var o=r.value;return o==="*"||o===i.group()},Dl[li.STATE]=function(r,i){var o=r.value;return eun(o,i)},Dl[li.ID]=function(r,i){var o=r.value;return i.id()===o},Dl[li.CLASS]=function(r,i){var o=r.value;return i.hasClass(o)},Dl[li.META_COMPARE]=function(r,i){var o=r.field,l=r.operator,f=r.value;return qNe(uun(i,o),l,f)},Dl[li.DATA_COMPARE]=function(r,i){var o=r.field,l=r.operator,f=r.value;return qNe(vue(i,o),l,f)},Dl[li.DATA_BOOL]=function(r,i){var o=r.field,l=r.operator;return oun(vue(i,o),l)},Dl[li.DATA_EXIST]=function(r,i){var o=r.field;return r.operator,cun(vue(i,o))},Dl[li.UNDIRECTED_EDGE]=function(r,i){var o=r.nodes[0],l=r.nodes[1],f=i.source(),b=i.target();return ku(o,f)&&ku(l,b)||ku(l,f)&&ku(o,b)},Dl[li.NODE_NEIGHBOR]=function(r,i){return ku(r.node,i)&&i.neighborhood().some(function(o){return o.isNode()&&ku(r.neighbor,o)})},Dl[li.DIRECTED_EDGE]=function(r,i){return ku(r.source,i.source())&&ku(r.target,i.target())},Dl[li.NODE_SOURCE]=function(r,i){return ku(r.source,i)&&i.outgoers().some(function(o){return o.isNode()&&ku(r.target,o)})},Dl[li.NODE_TARGET]=function(r,i){return ku(r.target,i)&&i.incomers().some(function(o){return o.isNode()&&ku(r.source,o)})},Dl[li.CHILD]=function(r,i){return ku(r.child,i)&&ku(r.parent,i.parent())},Dl[li.PARENT]=function(r,i){return ku(r.parent,i)&&i.children().some(function(o){return ku(r.child,o)})},Dl[li.DESCENDANT]=function(r,i){return ku(r.descendant,i)&&i.ancestors().some(function(o){return ku(r.ancestor,o)})},Dl[li.ANCESTOR]=function(r,i){return ku(r.ancestor,i)&&i.descendants().some(function(o){return ku(r.descendant,o)})},Dl[li.COMPOUND_SPLIT]=function(r,i){return ku(r.subject,i)&&ku(r.left,i)&&ku(r.right,i)},Dl[li.TRUE]=function(){return!0},Dl[li.COLLECTION]=function(r,i){var o=r.value;return o.has(i)},Dl[li.FILTER]=function(r,i){var o=r.value;return o(i)};var lun=function(i){var o=this;if(o.length===1&&o[0].checks.length===1&&o[0].checks[0].type===li.ID)return i.getElementById(o[0].checks[0].value).collection();var l=function(b){for(var d=0;d<o.length;d++){var w=o[d];if(ku(w,b))return!0}return!1};return o.text()==null&&(l=function(){return!0}),i.filter(l)},hun=function(i){for(var o=this,l=0;l<o.length;l++){var f=o[l];if(ku(f,i))return!0}return!1},fun={matches:hun,filter:lun},C6=function(i){this.inputText=i,this.currentSubject=null,this.compoundCount=0,this.edgeCount=0,this.length=0,i==null||$i(i)&&i.match(/^\s*$/)||(vp(i)?this.addQuery({checks:[{type:li.COLLECTION,value:i.collection()}]}):al(i)?this.addQuery({checks:[{type:li.FILTER,value:i}]}):$i(i)?this.parse(i)||(this.invalid=!0):Ml("A selector must be created from a string; found "))},S6=C6.prototype;[aun,fun].forEach(function(r){return bs(S6,r)}),S6.text=function(){return this.inputText},S6.size=function(){return this.length},S6.eq=function(r){return this[r]},S6.sameText=function(r){return!this.invalid&&!r.invalid&&this.text()===r.text()},S6.addQuery=function(r){this[this.length++]=r},S6.selector=S6.toString;var A6={allAre:function(i){var o=new C6(i);return this.every(function(l){return o.matches(l)})},is:function(i){var o=new C6(i);return this.some(function(l){return o.matches(l)})},some:function(i,o){for(var l=0;l<this.length;l++){var f=o?i.apply(o,[this[l],l,this]):i(this[l],l,this);if(f)return!0}return!1},every:function(i,o){for(var l=0;l<this.length;l++){var f=o?i.apply(o,[this[l],l,this]):i(this[l],l,this);if(!f)return!1}return!0},same:function(i){if(this===i)return!0;i=this.cy().collection(i);var o=this.length,l=i.length;return o!==l?!1:o===1?this[0]===i[0]:this.every(function(f){return i.hasElementWithId(f.id())})},anySame:function(i){return i=this.cy().collection(i),this.some(function(o){return i.hasElementWithId(o.id())})},allAreNeighbors:function(i){i=this.cy().collection(i);var o=this.neighborhood();return i.every(function(l){return o.hasElementWithId(l.id())})},contains:function(i){i=this.cy().collection(i);var o=this;return i.every(function(l){return o.hasElementWithId(l.id())})}};A6.allAreNeighbours=A6.allAreNeighbors,A6.has=A6.contains,A6.equal=A6.equals=A6.same;var j2=function(i,o){return function(f,b,d,w){var y=f,k=this,E;if(y==null?E="":vp(y)&&y.length===1&&(E=y.id()),k.length===1&&E){var T=k[0]._private,C=T.traversalCache=T.traversalCache||{},S=C[o]=C[o]||[],L=w7(E),O=S[L];return O||(S[L]=i.call(k,f,b,d,w))}else return i.call(k,f,b,d,w)}},GC={parent:function(i){var o=[];if(this.length===1){var l=this[0]._private.parent;if(l)return l}for(var f=0;f<this.length;f++){var b=this[f],d=b._private.parent;d&&o.push(d)}return this.spawn(o,!0).filter(i)},parents:function(i){for(var o=[],l=this.parent();l.nonempty();){for(var f=0;f<l.length;f++){var b=l[f];o.push(b)}l=l.parent()}return this.spawn(o,!0).filter(i)},commonAncestors:function(i){for(var o,l=0;l<this.length;l++){var f=this[l],b=f.parents();o=o||b,o=o.intersect(b)}return o.filter(i)},orphans:function(i){return this.stdFilter(function(o){return o.isOrphan()}).filter(i)},nonorphans:function(i){return this.stdFilter(function(o){return o.isChild()}).filter(i)},children:j2(function(r){for(var i=[],o=0;o<this.length;o++)for(var l=this[o],f=l._private.children,b=0;b<f.length;b++)i.push(f[b]);return this.spawn(i,!0).filter(r)},"children"),siblings:function(i){return this.parent().children().not(this).filter(i)},isParent:function(){var i=this[0];if(i)return i.isNode()&&i._private.children.length!==0},isChildless:function(){var i=this[0];if(i)return i.isNode()&&i._private.children.length===0},isChild:function(){var i=this[0];if(i)return i.isNode()&&i._private.parent!=null},isOrphan:function(){var i=this[0];if(i)return i.isNode()&&i._private.parent==null},descendants:function(i){var o=[];function l(f){for(var b=0;b<f.length;b++){var d=f[b];o.push(d),d.children().nonempty()&&l(d.children())}}return l(this.children()),this.spawn(o,!0).filter(i)}};function wue(r,i,o,l){for(var f=[],b=new NC,d=r.cy(),w=d.hasCompoundNodes(),y=0;y<r.length;y++){var k=r[y];o?f.push(k):w&&l(f,b,k)}for(;f.length>0;){var E=f.shift();i(E),b.add(E.id()),w&&l(f,b,E)}return r}function YNe(r,i,o){if(o.isParent())for(var l=o._private.children,f=0;f<l.length;f++){var b=l[f];i.has(b.id())||r.push(b)}}GC.forEachDown=function(r){var i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;return wue(this,r,i,YNe)};function WNe(r,i,o){if(o.isChild()){var l=o._private.parent;i.has(l.id())||r.push(l)}}GC.forEachUp=function(r){var i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;return wue(this,r,i,WNe)};function dun(r,i,o){WNe(r,i,o),YNe(r,i,o)}GC.forEachUpAndDown=function(r){var i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;return wue(this,r,i,dun)},GC.ancestors=GC.parents;var EO,KNe;EO=KNe={data:nc.data({field:"data",bindingEvent:"data",allowBinding:!0,allowSetting:!0,settingEvent:"data",settingTriggersEvent:!0,triggerFnName:"trigger",allowGetting:!0,immutableKeys:{id:!0,source:!0,target:!0,parent:!0},updateStyle:!0}),removeData:nc.removeData({field:"data",event:"data",triggerFnName:"trigger",triggerEvent:!0,immutableKeys:{id:!0,source:!0,target:!0,parent:!0},updateStyle:!0}),scratch:nc.data({field:"scratch",bindingEvent:"scratch",allowBinding:!0,allowSetting:!0,settingEvent:"scratch",settingTriggersEvent:!0,triggerFnName:"trigger",allowGetting:!0,updateStyle:!0}),removeScratch:nc.removeData({field:"scratch",event:"scratch",triggerFnName:"trigger",triggerEvent:!0,updateStyle:!0}),rscratch:nc.data({field:"rscratch",allowBinding:!1,allowSetting:!0,settingTriggersEvent:!1,allowGetting:!0}),removeRscratch:nc.removeData({field:"rscratch",triggerEvent:!1}),id:function(){var i=this[0];if(i)return i._private.data.id}},EO.attr=EO.data,EO.removeAttr=EO.removeData;var gun=KNe,OU={};function mue(r){return function(i){var o=this;if(i===void 0&&(i=!0),o.length!==0)if(o.isNode()&&!o.removed()){for(var l=0,f=o[0],b=f._private.edges,d=0;d<b.length;d++){var w=b[d];!i&&w.isLoop()||(l+=r(f,w))}return l}else return}}bs(OU,{degree:mue(function(r,i){return i.source().same(i.target())?2:1}),indegree:mue(function(r,i){return i.target().same(r)?1:0}),outdegree:mue(function(r,i){return i.source().same(r)?1:0})});function VC(r,i){return function(o){for(var l,f=this.nodes(),b=0;b<f.length;b++){var d=f[b],w=d[r](o);w!==void 0&&(l===void 0||i(w,l))&&(l=w)}return l}}bs(OU,{minDegree:VC("degree",function(r,i){return r<i}),maxDegree:VC("degree",function(r,i){return r>i}),minIndegree:VC("indegree",function(r,i){return r<i}),maxIndegree:VC("indegree",function(r,i){return r>i}),minOutdegree:VC("outdegree",function(r,i){return r<i}),maxOutdegree:VC("outdegree",function(r,i){return r>i})}),bs(OU,{totalDegree:function(i){for(var o=0,l=this.nodes(),f=0;f<l.length;f++)o+=l[f].degree(i);return o}});var Vv,XNe,QNe=function(i,o,l){for(var f=0;f<i.length;f++){var b=i[f];if(!b.locked()){var d=b._private.position,w={x:o.x!=null?o.x-d.x:0,y:o.y!=null?o.y-d.y:0};b.isParent()&&!(w.x===0&&w.y===0)&&b.children().shift(w,l),b.dirtyBoundingBoxCache()}}},ZNe={field:"position",bindingEvent:"position",allowBinding:!0,allowSetting:!0,settingEvent:"position",settingTriggersEvent:!0,triggerFnName:"emitAndNotify",allowGetting:!0,validKeys:["x","y"],beforeGet:function(i){i.updateCompoundBounds()},beforeSet:function(i,o){QNe(i,o,!1)},onSet:function(i){i.dirtyCompoundBoundsCache()},canSet:function(i){return!i.locked()}};Vv=XNe={position:nc.data(ZNe),silentPosition:nc.data(bs({},ZNe,{allowBinding:!1,allowSetting:!0,settingTriggersEvent:!1,allowGetting:!1,beforeSet:function(i,o){QNe(i,o,!0)},onSet:function(i){i.dirtyCompoundBoundsCache()}})),positions:function(i,o){if(_o(i))o?this.silentPosition(i):this.position(i);else if(al(i)){var l=i,f=this.cy();f.startBatch();for(var b=0;b<this.length;b++){var d=this[b],w=void 0;(w=l(d,b))&&(o?d.silentPosition(w):d.position(w))}f.endBatch()}return this},silentPositions:function(i){return this.positions(i,!0)},shift:function(i,o,l){var f;if(_o(i)?(f={x:Vr(i.x)?i.x:0,y:Vr(i.y)?i.y:0},l=o):$i(i)&&Vr(o)&&(f={x:0,y:0},f[i]=o),f!=null){var b=this.cy();b.startBatch();for(var d=0;d<this.length;d++){var w=this[d];if(!(b.hasCompoundNodes()&&w.isChild()&&w.ancestors().anySame(this))){var y=w.position(),k={x:y.x+f.x,y:y.y+f.y};l?w.silentPosition(k):w.position(k)}}b.endBatch()}return this},silentShift:function(i,o){return _o(i)?this.shift(i,!0):$i(i)&&Vr(o)&&this.shift(i,o,!0),this},renderedPosition:function(i,o){var l=this[0],f=this.cy(),b=f.zoom(),d=f.pan(),w=_o(i)?i:void 0,y=w!==void 0||o!==void 0&&$i(i);if(l&&l.isNode())if(y)for(var k=0;k<this.length;k++){var E=this[k];o!==void 0?E.position(i,(o-d[i])/b):w!==void 0&&E.position(dNe(w,b,d))}else{var T=l.position();return w=EU(T,b,d),i===void 0?w:w[i]}else if(!y)return;return this},relativePosition:function(i,o){var l=this[0],f=this.cy(),b=_o(i)?i:void 0,d=b!==void 0||o!==void 0&&$i(i),w=f.hasCompoundNodes();if(l&&l.isNode())if(d)for(var y=0;y<this.length;y++){var k=this[y],E=w?k.parent():null,T=E&&E.length>0,C=T;T&&(E=E[0]);var S=C?E.position():{x:0,y:0};o!==void 0?k.position(i,o+S[i]):b!==void 0&&k.position({x:b.x+S.x,y:b.y+S.y})}else{var L=l.position(),O=w?l.parent():null,B=O&&O.length>0,N=B;B&&(O=O[0]);var F=N?O.position():{x:0,y:0};return b={x:L.x-F.x,y:L.y-F.y},i===void 0?b:b[i]}else if(!d)return;return this}},Vv.modelPosition=Vv.point=Vv.position,Vv.modelPositions=Vv.points=Vv.positions,Vv.renderedPoint=Vv.renderedPosition,Vv.relativePoint=Vv.relativePosition;var pun=XNe,UC,M6;UC=M6={},M6.renderedBoundingBox=function(r){var i=this.boundingBox(r),o=this.cy(),l=o.zoom(),f=o.pan(),b=i.x1*l+f.x,d=i.x2*l+f.x,w=i.y1*l+f.y,y=i.y2*l+f.y;return{x1:b,x2:d,y1:w,y2:y,w:d-b,h:y-w}},M6.dirtyCompoundBoundsCache=function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1,i=this.cy();return!i.styleEnabled()||!i.hasCompoundNodes()?this:(this.forEachUp(function(o){if(o.isParent()){var l=o._private;l.compoundBoundsClean=!1,l.bbCache=null,r||o.emitAndNotify("bounds")}}),this)},M6.updateCompoundBounds=function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1,i=this.cy();if(!i.styleEnabled()||!i.hasCompoundNodes())return this;if(!r&&i.batching())return this;function o(d){if(!d.isParent())return;var w=d._private,y=d.children(),k=d.pstyle("compound-sizing-wrt-labels").value==="include",E={width:{val:d.pstyle("min-width").pfValue,left:d.pstyle("min-width-bias-left"),right:d.pstyle("min-width-bias-right")},height:{val:d.pstyle("min-height").pfValue,top:d.pstyle("min-height-bias-top"),bottom:d.pstyle("min-height-bias-bottom")}},T=y.boundingBox({includeLabels:k,includeOverlays:!1,useCache:!1}),C=w.position;(T.w===0||T.h===0)&&(T={w:d.pstyle("width").pfValue,h:d.pstyle("height").pfValue},T.x1=C.x-T.w/2,T.x2=C.x+T.w/2,T.y1=C.y-T.h/2,T.y2=C.y+T.h/2);function S(z,W,Z){var G=0,ae=0,$=W+Z;return z>0&&$>0&&(G=W/$*z,ae=Z/$*z),{biasDiff:G,biasComplementDiff:ae}}function L(z,W,Z,G){if(Z.units==="%")switch(G){case"width":return z>0?Z.pfValue*z:0;case"height":return W>0?Z.pfValue*W:0;case"average":return z>0&&W>0?Z.pfValue*(z+W)/2:0;case"min":return z>0&&W>0?z>W?Z.pfValue*W:Z.pfValue*z:0;case"max":return z>0&&W>0?z>W?Z.pfValue*z:Z.pfValue*W:0;default:return 0}else return Z.units==="px"?Z.pfValue:0}var O=E.width.left.value;E.width.left.units==="px"&&E.width.val>0&&(O=O*100/E.width.val);var B=E.width.right.value;E.width.right.units==="px"&&E.width.val>0&&(B=B*100/E.width.val);var N=E.height.top.value;E.height.top.units==="px"&&E.height.val>0&&(N=N*100/E.height.val);var F=E.height.bottom.value;E.height.bottom.units==="px"&&E.height.val>0&&(F=F*100/E.height.val);var R=S(E.width.val-T.w,O,B),q=R.biasDiff,X=R.biasComplementDiff,te=S(E.height.val-T.h,N,F),H=te.biasDiff,Y=te.biasComplementDiff;w.autoPadding=L(T.w,T.h,d.pstyle("padding"),d.pstyle("padding-relative-to").value),w.autoWidth=Math.max(T.w,E.width.val),C.x=(-q+T.x1+T.x2+X)/2,w.autoHeight=Math.max(T.h,E.height.val),C.y=(-H+T.y1+T.y2+Y)/2}for(var l=0;l<this.length;l++){var f=this[l],b=f._private;(!b.compoundBoundsClean||r)&&(o(f),i.batching()||(b.compoundBoundsClean=!0))}return this};var $2=function(i){return i===1/0||i===-1/0?0:i},Uv=function(i,o,l,f,b){f-o===0||b-l===0||o==null||l==null||f==null||b==null||(i.x1=o<i.x1?o:i.x1,i.x2=f>i.x2?f:i.x2,i.y1=l<i.y1?l:i.y1,i.y2=b>i.y2?b:i.y2,i.w=i.x2-i.x1,i.h=i.y2-i.y1)},E7=function(i,o){return o==null?i:Uv(i,o.x1,o.y1,o.x2,o.y2)},_O=function(i,o,l){return Gv(i,o,l)},NU=function(i,o,l){if(!o.cy().headless()){var f=o._private,b=f.rstyle,d=b.arrowWidth/2,w=o.pstyle(l+"-arrow-shape").value,y,k;if(w!=="none"){l==="source"?(y=b.srcX,k=b.srcY):l==="target"?(y=b.tgtX,k=b.tgtY):(y=b.midX,k=b.midY);var E=f.arrowBounds=f.arrowBounds||{},T=E[l]=E[l]||{};T.x1=y-d,T.y1=k-d,T.x2=y+d,T.y2=k+d,T.w=T.x2-T.x1,T.h=T.y2-T.y1,TU(T,1),Uv(i,T.x1,T.y1,T.x2,T.y2)}}},yue=function(i,o,l){if(!o.cy().headless()){var f;l?f=l+"-":f="";var b=o._private,d=b.rstyle,w=o.pstyle(f+"label").strValue;if(w){var y=o.pstyle("text-halign"),k=o.pstyle("text-valign"),E=_O(d,"labelWidth",l),T=_O(d,"labelHeight",l),C=_O(d,"labelX",l),S=_O(d,"labelY",l),L=o.pstyle(f+"text-margin-x").pfValue,O=o.pstyle(f+"text-margin-y").pfValue,B=o.isEdge(),N=o.pstyle(f+"text-rotation"),F=o.pstyle("text-outline-width").pfValue,R=o.pstyle("text-border-width").pfValue,q=R/2,X=o.pstyle("text-background-padding").pfValue,te=2,H=T,Y=E,z=Y/2,W=H/2,Z,G,ae,$;if(B)Z=C-z,G=C+z,ae=S-W,$=S+W;else{switch(y.value){case"left":Z=C-Y,G=C;break;case"center":Z=C-z,G=C+z;break;case"right":Z=C,G=C+Y;break}switch(k.value){case"top":ae=S-H,$=S;break;case"center":ae=S-W,$=S+W;break;case"bottom":ae=S,$=S+H;break}}Z+=L-Math.max(F,q)-X-te,G+=L+Math.max(F,q)+X+te,ae+=O-Math.max(F,q)-X-te,$+=O+Math.max(F,q)+X+te;var ge=l||"main",ee=b.labelBounds,de=ee[ge]=ee[ge]||{};de.x1=Z,de.y1=ae,de.x2=G,de.y2=$,de.w=G-Z,de.h=$-ae;var re=B&&N.strValue==="autorotate",ke=N.pfValue!=null&&N.pfValue!==0;if(re||ke){var Ce=re?_O(b.rstyle,"labelAngle",l):N.pfValue,_e=Math.cos(Ce),Te=Math.sin(Ce),Be=(Z+G)/2,Ge=(ae+$)/2;if(!B){switch(y.value){case"left":Be=G;break;case"right":Be=Z;break}switch(k.value){case"top":Ge=$;break;case"bottom":Ge=ae;break}}var Xe=function(Ae,je){return Ae=Ae-Be,je=je-Ge,{x:Ae*_e-je*Te+Be,y:Ae*Te+je*_e+Ge}},Ee=Xe(Z,ae),Ze=Xe(Z,$),Ie=Xe(G,ae),Oe=Xe(G,$);Z=Math.min(Ee.x,Ze.x,Ie.x,Oe.x),G=Math.max(Ee.x,Ze.x,Ie.x,Oe.x),ae=Math.min(Ee.y,Ze.y,Ie.y,Oe.y),$=Math.max(Ee.y,Ze.y,Ie.y,Oe.y)}var Le=ge+"Rot",$e=ee[Le]=ee[Le]||{};$e.x1=Z,$e.y1=ae,$e.x2=G,$e.y2=$,$e.w=G-Z,$e.h=$-ae,Uv(i,Z,ae,G,$),Uv(b.labelBounds.all,Z,ae,G,$)}return i}},bun=function(i,o){var l=i._private.cy,f=l.styleEnabled(),b=l.headless(),d=J0(),w=i._private,y=i.isNode(),k=i.isEdge(),E,T,C,S,L,O,B=w.rstyle,N=y&&f?i.pstyle("bounds-expansion").pfValue:[0],F=function(vt){return vt.pstyle("display").value!=="none"},R=!f||F(i)&&(!k||F(i.source())&&F(i.target()));if(R){var q=0,X=0;f&&o.includeOverlays&&(q=i.pstyle("overlay-opacity").value,q!==0&&(X=i.pstyle("overlay-padding").value));var te=0,H=0;f&&o.includeUnderlays&&(te=i.pstyle("underlay-opacity").value,te!==0&&(H=i.pstyle("underlay-padding").value));var Y=Math.max(X,H),z=0,W=0;if(f&&(z=i.pstyle("width").pfValue,W=z/2),y&&o.includeNodes){var Z=i.position();L=Z.x,O=Z.y;var G=i.outerWidth(),ae=G/2,$=i.outerHeight(),ge=$/2;E=L-ae,T=L+ae,C=O-ge,S=O+ge,Uv(d,E,C,T,S)}else if(k&&o.includeEdges)if(f&&!b){var ee=i.pstyle("curve-style").strValue;if(E=Math.min(B.srcX,B.midX,B.tgtX),T=Math.max(B.srcX,B.midX,B.tgtX),C=Math.min(B.srcY,B.midY,B.tgtY),S=Math.max(B.srcY,B.midY,B.tgtY),E-=W,T+=W,C-=W,S+=W,Uv(d,E,C,T,S),ee==="haystack"){var de=B.haystackPts;if(de&&de.length===2){if(E=de[0].x,C=de[0].y,T=de[1].x,S=de[1].y,E>T){var re=E;E=T,T=re}if(C>S){var ke=C;C=S,S=ke}Uv(d,E-W,C-W,T+W,S+W)}}else if(ee==="bezier"||ee==="unbundled-bezier"||ee==="segments"||ee==="taxi"){var Ce;switch(ee){case"bezier":case"unbundled-bezier":Ce=B.bezierPts;break;case"segments":case"taxi":Ce=B.linePts;break}if(Ce!=null)for(var _e=0;_e<Ce.length;_e++){var Te=Ce[_e];E=Te.x-W,T=Te.x+W,C=Te.y-W,S=Te.y+W,Uv(d,E,C,T,S)}}}else{var Be=i.source(),Ge=Be.position(),Xe=i.target(),Ee=Xe.position();if(E=Ge.x,T=Ee.x,C=Ge.y,S=Ee.y,E>T){var Ze=E;E=T,T=Ze}if(C>S){var Ie=C;C=S,S=Ie}E-=W,T+=W,C-=W,S+=W,Uv(d,E,C,T,S)}if(f&&o.includeEdges&&k&&(NU(d,i,"mid-source"),NU(d,i,"mid-target"),NU(d,i,"source"),NU(d,i,"target")),f){var Oe=i.pstyle("ghost").value==="yes";if(Oe){var Le=i.pstyle("ghost-offset-x").pfValue,$e=i.pstyle("ghost-offset-y").pfValue;Uv(d,d.x1+Le,d.y1+$e,d.x2+Le,d.y2+$e)}}var xe=w.bodyBounds=w.bodyBounds||{};pNe(xe,d),sue(xe,N),TU(xe,1),f&&(E=d.x1,T=d.x2,C=d.y1,S=d.y2,Uv(d,E-Y,C-Y,T+Y,S+Y));var Ae=w.overlayBounds=w.overlayBounds||{};pNe(Ae,d),sue(Ae,N),TU(Ae,1);var je=w.labelBounds=w.labelBounds||{};je.all!=null?Gon(je.all):je.all=J0(),f&&o.includeLabels&&(o.includeMainLabels&&yue(d,i,null),k&&(o.includeSourceLabels&&yue(d,i,"source"),o.includeTargetLabels&&yue(d,i,"target")))}return d.x1=$2(d.x1),d.y1=$2(d.y1),d.x2=$2(d.x2),d.y2=$2(d.y2),d.w=$2(d.x2-d.x1),d.h=$2(d.y2-d.y1),d.w>0&&d.h>0&&R&&(sue(d,N),TU(d,1)),d},JNe=function(i){var o=0,l=function(d){return(d?1:0)<<o++},f=0;return f+=l(i.incudeNodes),f+=l(i.includeEdges),f+=l(i.includeLabels),f+=l(i.includeMainLabels),f+=l(i.includeSourceLabels),f+=l(i.includeTargetLabels),f+=l(i.includeOverlays),f},ePe=function(i){if(i.isEdge()){var o=i.source().position(),l=i.target().position(),f=function(d){return Math.round(d)};return hon([f(o.x),f(o.y),f(l.x),f(l.y)])}else return 0},tPe=function(i,o){var l=i._private,f,b=i.isEdge(),d=o==null?nPe:JNe(o),w=d===nPe,y=ePe(i),k=l.bbCachePosKey===y,E=o.useCache&&k,T=function(O){return O._private.bbCache==null||O._private.styleDirty},C=!E||T(i)||b&&T(i.source())||T(i.target());if(C?(k||i.recalculateRenderedStyle(E),f=bun(i,TO),l.bbCache=f,l.bbCachePosKey=y):f=l.bbCache,!w){var S=i.isNode();f=J0(),(o.includeNodes&&S||o.includeEdges&&!S)&&(o.includeOverlays?E7(f,l.overlayBounds):E7(f,l.bodyBounds)),o.includeLabels&&(o.includeMainLabels&&(!b||o.includeSourceLabels&&o.includeTargetLabels)?E7(f,l.labelBounds.all):(o.includeMainLabels&&E7(f,l.labelBounds.mainRot),o.includeSourceLabels&&E7(f,l.labelBounds.sourceRot),o.includeTargetLabels&&E7(f,l.labelBounds.targetRot))),f.w=f.x2-f.x1,f.h=f.y2-f.y1}return f},TO={includeNodes:!0,includeEdges:!0,includeLabels:!0,includeMainLabels:!0,includeSourceLabels:!0,includeTargetLabels:!0,includeOverlays:!0,includeUnderlays:!0,useCache:!0},nPe=JNe(TO),rPe=T1(TO);M6.boundingBox=function(r){var i;if(this.length===1&&this[0]._private.bbCache!=null&&!this[0]._private.styleDirty&&(r===void 0||r.useCache===void 0||r.useCache===!0))r===void 0?r=TO:r=rPe(r),i=tPe(this[0],r);else{i=J0(),r=r||TO;var o=rPe(r),l=this,f=l.cy(),b=f.styleEnabled();if(b)for(var d=0;d<l.length;d++){var w=l[d],y=w._private,k=ePe(w),E=y.bbCachePosKey===k,T=o.useCache&&E&&!y.styleDirty;w.recalculateRenderedStyle(T)}this.updateCompoundBounds(!r.useCache);for(var C=0;C<l.length;C++){var S=l[C];E7(i,tPe(S,o))}}return i.x1=$2(i.x1),i.y1=$2(i.y1),i.x2=$2(i.x2),i.y2=$2(i.y2),i.w=$2(i.x2-i.x1),i.h=$2(i.y2-i.y1),i},M6.dirtyBoundingBoxCache=function(){for(var r=0;r<this.length;r++){var i=this[r]._private;i.bbCache=null,i.bbCachePosKey=null,i.bodyBounds=null,i.overlayBounds=null,i.labelBounds.all=null,i.labelBounds.source=null,i.labelBounds.target=null,i.labelBounds.main=null,i.labelBounds.sourceRot=null,i.labelBounds.targetRot=null,i.labelBounds.mainRot=null,i.arrowBounds.source=null,i.arrowBounds.target=null,i.arrowBounds["mid-source"]=null,i.arrowBounds["mid-target"]=null}return this.emitAndNotify("bounds"),this},M6.boundingBoxAt=function(r){var i=this.nodes(),o=this.cy(),l=o.hasCompoundNodes(),f=o.collection();if(l&&(f=i.filter(function(k){return k.isParent()}),i=i.not(f)),_o(r)){var b=r;r=function(){return b}}var d=function(E,T){return E._private.bbAtOldPos=r(E,T)},w=function(E){return E._private.bbAtOldPos};o.startBatch(),i.forEach(d).silentPositions(r),l&&(f.dirtyCompoundBoundsCache(),f.dirtyBoundingBoxCache(),f.updateCompoundBounds(!0));var y=zon(this.boundingBox({useCache:!1}));return i.silentPositions(w),l&&(f.dirtyCompoundBoundsCache(),f.dirtyBoundingBoxCache(),f.updateCompoundBounds(!0)),o.endBatch(),y},UC.boundingbox=UC.bb=UC.boundingBox,UC.renderedBoundingbox=UC.renderedBoundingBox;var vun=M6,CO,SO;CO=SO={};var iPe=function(i){i.uppercaseName=ZOe(i.name),i.autoName="auto"+i.uppercaseName,i.labelName="label"+i.uppercaseName,i.outerName="outer"+i.uppercaseName,i.uppercaseOuterName=ZOe(i.outerName),CO[i.name]=function(){var l=this[0],f=l._private,b=f.cy,d=b._private.styleEnabled;if(l)if(d){if(l.isParent())return l.updateCompoundBounds(),f[i.autoName]||0;var w=l.pstyle(i.name);switch(w.strValue){case"label":return l.recalculateRenderedStyle(),f.rstyle[i.labelName]||0;default:return w.pfValue}}else return 1},CO["outer"+i.uppercaseName]=function(){var l=this[0],f=l._private,b=f.cy,d=b._private.styleEnabled;if(l)if(d){var w=l[i.name](),y=l.pstyle("border-width").pfValue,k=2*l.padding();return w+y+k}else return 1},CO["rendered"+i.uppercaseName]=function(){var l=this[0];if(l){var f=l[i.name]();return f*this.cy().zoom()}},CO["rendered"+i.uppercaseOuterName]=function(){var l=this[0];if(l){var f=l[i.outerName]();return f*this.cy().zoom()}}};iPe({name:"width"}),iPe({name:"height"}),SO.padding=function(){var r=this[0],i=r._private;return r.isParent()?(r.updateCompoundBounds(),i.autoPadding!==void 0?i.autoPadding:r.pstyle("padding").pfValue):r.pstyle("padding").pfValue},SO.paddedHeight=function(){var r=this[0];return r.height()+2*r.padding()},SO.paddedWidth=function(){var r=this[0];return r.width()+2*r.padding()};var wun=SO,mun=function(i,o){if(i.isEdge())return o(i)},yun=function(i,o){if(i.isEdge()){var l=i.cy();return EU(o(i),l.zoom(),l.pan())}},kun=function(i,o){if(i.isEdge()){var l=i.cy(),f=l.pan(),b=l.zoom();return o(i).map(function(d){return EU(d,b,f)})}},xun=function(i){return i.renderer().getControlPoints(i)},Eun=function(i){return i.renderer().getSegmentPoints(i)},_un=function(i){return i.renderer().getSourceEndpoint(i)},Tun=function(i){return i.renderer().getTargetEndpoint(i)},Cun=function(i){return i.renderer().getEdgeMidpoint(i)},sPe={controlPoints:{get:xun,mult:!0},segmentPoints:{get:Eun,mult:!0},sourceEndpoint:{get:_un},targetEndpoint:{get:Tun},midpoint:{get:Cun}},Sun=function(i){return"rendered"+i[0].toUpperCase()+i.substr(1)},Aun=Object.keys(sPe).reduce(function(r,i){var o=sPe[i],l=Sun(i);return r[i]=function(){return mun(this,o.get)},o.mult?r[l]=function(){return kun(this,o.get)}:r[l]=function(){return yun(this,o.get)},r},{}),Mun=bs({},pun,vun,wun,Aun);/*!
    Event object based on jQuery events, MIT license

    https://jquery.org/license/
    https://tldrlegal.com/license/mit-license
    https://github.com/jquery/jquery/blob/master/src/event.js
    */var aPe=function(i,o){this.recycle(i,o)};function AO(){return!1}function PU(){return!0}aPe.prototype={instanceString:function(){return"event"},recycle:function(i,o){if(this.isImmediatePropagationStopped=this.isPropagationStopped=this.isDefaultPrevented=AO,i!=null&&i.preventDefault?(this.type=i.type,this.isDefaultPrevented=i.defaultPrevented?PU:AO):i!=null&&i.type?o=i:this.type=i,o!=null&&(this.originalEvent=o.originalEvent,this.type=o.type!=null?o.type:this.type,this.cy=o.cy,this.target=o.target,this.position=o.position,this.renderedPosition=o.renderedPosition,this.namespace=o.namespace,this.layout=o.layout),this.cy!=null&&this.position!=null&&this.renderedPosition==null){var l=this.position,f=this.cy.zoom(),b=this.cy.pan();this.renderedPosition={x:l.x*f+b.x,y:l.y*f+b.y}}this.timeStamp=i&&i.timeStamp||Date.now()},preventDefault:function(){this.isDefaultPrevented=PU;var i=this.originalEvent;!i||i.preventDefault&&i.preventDefault()},stopPropagation:function(){this.isPropagationStopped=PU;var i=this.originalEvent;!i||i.stopPropagation&&i.stopPropagation()},stopImmediatePropagation:function(){this.isImmediatePropagationStopped=PU,this.stopPropagation()},isDefaultPrevented:AO,isPropagationStopped:AO,isImmediatePropagationStopped:AO};var oPe=/^([^.]+)(\.(?:[^.]+))?$/,Dun=".*",cPe={qualifierCompare:function(i,o){return i===o},eventMatches:function(){return!0},addEventFields:function(){},callbackContext:function(i){return i},beforeEmit:function(){},afterEmit:function(){},bubble:function(){return!1},parent:function(){return null},context:null},uPe=Object.keys(cPe),Lun={};function BU(){for(var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:Lun,i=arguments.length>1?arguments[1]:void 0,o=0;o<uPe.length;o++){var l=uPe[o];this[l]=r[l]||cPe[l]}this.context=i||this.context,this.listeners=[],this.emitting=0}var D6=BU.prototype,lPe=function(i,o,l,f,b,d,w){al(f)&&(b=f,f=null),w&&(d==null?d=w:d=bs({},d,w));for(var y=qc(l)?l:l.split(/\s+/),k=0;k<y.length;k++){var E=y[k];if(!y6(E)){var T=E.match(oPe);if(T){var C=T[1],S=T[2]?T[2]:null,L=o(i,E,C,S,f,b,d);if(L===!1)break}}}},hPe=function(i,o){return i.addEventFields(i.context,o),new aPe(o.type,o)},Iun=function(i,o,l){if(Uan(l)){o(i,l);return}else if(_o(l)){o(i,hPe(i,l));return}for(var f=qc(l)?l:l.split(/\s+/),b=0;b<f.length;b++){var d=f[b];if(!y6(d)){var w=d.match(oPe);if(w){var y=w[1],k=w[2]?w[2]:null,E=hPe(i,{type:y,namespace:k,target:i.context});o(i,E)}}}};D6.on=D6.addListener=function(r,i,o,l,f){return lPe(this,function(b,d,w,y,k,E,T){al(E)&&b.listeners.push({event:d,callback:E,type:w,namespace:y,qualifier:k,conf:T})},r,i,o,l,f),this},D6.one=function(r,i,o,l){return this.on(r,i,o,l,{one:!0})},D6.removeListener=D6.off=function(r,i,o,l){var f=this;this.emitting!==0&&(this.listeners=bon(this.listeners));for(var b=this.listeners,d=function(k){var E=b[k];lPe(f,function(T,C,S,L,O,B){if((E.type===S||r==="*")&&(!L&&E.namespace!==".*"||E.namespace===L)&&(!O||T.qualifierCompare(E.qualifier,O))&&(!B||E.callback===B))return b.splice(k,1),!1},r,i,o,l)},w=b.length-1;w>=0;w--)d(w);return this},D6.removeAllListeners=function(){return this.removeListener("*")},D6.emit=D6.trigger=function(r,i,o){var l=this.listeners,f=l.length;return this.emitting++,qc(i)||(i=[i]),Iun(this,function(b,d){o!=null&&(l=[{event:d.event,type:d.type,namespace:d.namespace,callback:o}],f=l.length);for(var w=function(E){var T=l[E];if(T.type===d.type&&(!T.namespace||T.namespace===d.namespace||T.namespace===Dun)&&b.eventMatches(b.context,T,d)){var C=[d];i!=null&&won(C,i),b.beforeEmit(b.context,T,d),T.conf&&T.conf.one&&(b.listeners=b.listeners.filter(function(O){return O!==T}));var S=b.callbackContext(b.context,T,d),L=T.callback.apply(S,C);b.afterEmit(b.context,T,d),L===!1&&(d.stopPropagation(),d.preventDefault())}},y=0;y<f;y++)w(y);b.bubble(b.context)&&!d.isPropagationStopped()&&b.parent(b.context).emit(d,i)},r),this.emitting--,this};var Oun={qualifierCompare:function(i,o){return i==null||o==null?i==null&&o==null:i.sameText(o)},eventMatches:function(i,o,l){var f=o.qualifier;return f!=null?i!==l.target&&fO(l.target)&&f.matches(l.target):!0},addEventFields:function(i,o){o.cy=i.cy(),o.target=i},callbackContext:function(i,o,l){return o.qualifier!=null?l.target:i},beforeEmit:function(i,o){o.conf&&o.conf.once&&o.conf.onceCollection.removeListener(o.event,o.qualifier,o.callback)},bubble:function(){return!0},parent:function(i){return i.isChild()?i.parent():i.cy()}},FU=function(i){return $i(i)?new C6(i):i},fPe={createEmitter:function(){for(var i=0;i<this.length;i++){var o=this[i],l=o._private;l.emitter||(l.emitter=new BU(Oun,o))}return this},emitter:function(){return this._private.emitter},on:function(i,o,l){for(var f=FU(o),b=0;b<this.length;b++){var d=this[b];d.emitter().on(i,f,l)}return this},removeListener:function(i,o,l){for(var f=FU(o),b=0;b<this.length;b++){var d=this[b];d.emitter().removeListener(i,f,l)}return this},removeAllListeners:function(){for(var i=0;i<this.length;i++){var o=this[i];o.emitter().removeAllListeners()}return this},one:function(i,o,l){for(var f=FU(o),b=0;b<this.length;b++){var d=this[b];d.emitter().one(i,f,l)}return this},once:function(i,o,l){for(var f=FU(o),b=0;b<this.length;b++){var d=this[b];d.emitter().on(i,f,l,{once:!0,onceCollection:this})}},emit:function(i,o){for(var l=0;l<this.length;l++){var f=this[l];f.emitter().emit(i,o)}return this},emitAndNotify:function(i,o){if(this.length!==0)return this.cy().notify(i,this),this.emit(i,o),this}};nc.eventAliasesOn(fPe);var dPe={nodes:function(i){return this.filter(function(o){return o.isNode()}).filter(i)},edges:function(i){return this.filter(function(o){return o.isEdge()}).filter(i)},byGroup:function(){for(var i=this.spawn(),o=this.spawn(),l=0;l<this.length;l++){var f=this[l];f.isNode()?i.push(f):o.push(f)}return{nodes:i,edges:o}},filter:function(i,o){if(i===void 0)return this;if($i(i)||vp(i))return new C6(i).filter(this);if(al(i)){for(var l=this.spawn(),f=this,b=0;b<f.length;b++){var d=f[b],w=o?i.apply(o,[d,b,f]):i(d,b,f);w&&l.push(d)}return l}return this.spawn()},not:function(i){if(i){$i(i)&&(i=this.filter(i));for(var o=this.spawn(),l=0;l<this.length;l++){var f=this[l],b=i.has(f);b||o.push(f)}return o}else return this},absoluteComplement:function(){var i=this.cy();return i.mutableElements().not(this)},intersect:function(i){if($i(i)){var o=i;return this.filter(o)}for(var l=this.spawn(),f=this,b=i,d=this.length<i.length,w=d?f:b,y=d?b:f,k=0;k<w.length;k++){var E=w[k];y.has(E)&&l.push(E)}return l},xor:function(i){var o=this._private.cy;$i(i)&&(i=o.$(i));var l=this.spawn(),f=this,b=i,d=function(y,k){for(var E=0;E<y.length;E++){var T=y[E],C=T._private.data.id,S=k.hasElementWithId(C);S||l.push(T)}};return d(f,b),d(b,f),l},diff:function(i){var o=this._private.cy;$i(i)&&(i=o.$(i));var l=this.spawn(),f=this.spawn(),b=this.spawn(),d=this,w=i,y=function(E,T,C){for(var S=0;S<E.length;S++){var L=E[S],O=L._private.data.id,B=T.hasElementWithId(O);B?b.merge(L):C.push(L)}};return y(d,w,l),y(w,d,f),{left:l,right:f,both:b}},add:function(i){var o=this._private.cy;if(!i)return this;if($i(i)){var l=i;i=o.mutableElements().filter(l)}for(var f=this.spawnSelf(),b=0;b<i.length;b++){var d=i[b],w=!this.has(d);w&&f.push(d)}return f},merge:function(i){var o=this._private,l=o.cy;if(!i)return this;if(i&&$i(i)){var f=i;i=l.mutableElements().filter(f)}for(var b=o.map,d=0;d<i.length;d++){var w=i[d],y=w._private.data.id,k=!b.has(y);if(k){var E=this.length++;this[E]=w,b.set(y,{ele:w,index:E})}}return this},unmergeAt:function(i){var o=this[i],l=o.id(),f=this._private,b=f.map;this[i]=void 0,b.delete(l);var d=i===this.length-1;if(this.length>1&&!d){var w=this.length-1,y=this[w],k=y._private.data.id;this[w]=void 0,this[i]=y,b.set(k,{ele:y,index:i})}return this.length--,this},unmergeOne:function(i){i=i[0];var o=this._private,l=i._private.data.id,f=o.map,b=f.get(l);if(!b)return this;var d=b.index;return this.unmergeAt(d),this},unmerge:function(i){var o=this._private.cy;if(!i)return this;if(i&&$i(i)){var l=i;i=o.mutableElements().filter(l)}for(var f=0;f<i.length;f++)this.unmergeOne(i[f]);return this},unmergeBy:function(i){for(var o=this.length-1;o>=0;o--){var l=this[o];i(l)&&this.unmergeAt(o)}return this},map:function(i,o){for(var l=[],f=this,b=0;b<f.length;b++){var d=f[b],w=o?i.apply(o,[d,b,f]):i(d,b,f);l.push(w)}return l},reduce:function(i,o){for(var l=o,f=this,b=0;b<f.length;b++)l=i(l,f[b],b,f);return l},max:function(i,o){for(var l=-1/0,f,b=this,d=0;d<b.length;d++){var w=b[d],y=o?i.apply(o,[w,d,b]):i(w,d,b);y>l&&(l=y,f=w)}return{value:l,ele:f}},min:function(i,o){for(var l=1/0,f,b=this,d=0;d<b.length;d++){var w=b[d],y=o?i.apply(o,[w,d,b]):i(w,d,b);y<l&&(l=y,f=w)}return{value:l,ele:f}}},Io=dPe;Io.u=Io["|"]=Io["+"]=Io.union=Io.or=Io.add,Io["\\"]=Io["!"]=Io["-"]=Io.difference=Io.relativeComplement=Io.subtract=Io.not,Io.n=Io["&"]=Io["."]=Io.and=Io.intersection=Io.intersect,Io["^"]=Io["(+)"]=Io["(-)"]=Io.symmetricDifference=Io.symdiff=Io.xor,Io.fnFilter=Io.filterFn=Io.stdFilter=Io.filter,Io.complement=Io.abscomp=Io.absoluteComplement;var Nun={isNode:function(){return this.group()==="nodes"},isEdge:function(){return this.group()==="edges"},isLoop:function(){return this.isEdge()&&this.source()[0]===this.target()[0]},isSimple:function(){return this.isEdge()&&this.source()[0]!==this.target()[0]},group:function(){var i=this[0];if(i)return i._private.group}},gPe=function(i,o){var l=i.cy(),f=l.hasCompoundNodes();function b(E){var T=E.pstyle("z-compound-depth");return T.value==="auto"?f?E.zDepth():0:T.value==="bottom"?-1:T.value==="top"?eue:0}var d=b(i)-b(o);if(d!==0)return d;function w(E){var T=E.pstyle("z-index-compare");return T.value==="auto"&&E.isNode()?1:0}var y=w(i)-w(o);if(y!==0)return y;var k=i.pstyle("z-index").value-o.pstyle("z-index").value;return k!==0?k:i.poolIndex()-o.poolIndex()},RU={forEach:function(i,o){if(al(i))for(var l=this.length,f=0;f<l;f++){var b=this[f],d=o?i.apply(o,[b,f,this]):i(b,f,this);if(d===!1)break}return this},toArray:function(){for(var i=[],o=0;o<this.length;o++)i.push(this[o]);return i},slice:function(i,o){var l=[],f=this.length;o==null&&(o=f),i==null&&(i=0),i<0&&(i=f+i),o<0&&(o=f+o);for(var b=i;b>=0&&b<o&&b<f;b++)l.push(this[b]);return this.spawn(l)},size:function(){return this.length},eq:function(i){return this[i]||this.spawn()},first:function(){return this[0]||this.spawn()},last:function(){return this[this.length-1]||this.spawn()},empty:function(){return this.length===0},nonempty:function(){return!this.empty()},sort:function(i){if(!al(i))return this;var o=this.toArray().sort(i);return this.spawn(o)},sortByZIndex:function(){return this.sort(gPe)},zDepth:function(){var i=this[0];if(!!i){var o=i._private,l=o.group;if(l==="nodes"){var f=o.data.parent?i.parents().size():0;return i.isParent()?f:eue-1}else{var b=o.source,d=o.target,w=b.zDepth(),y=d.zDepth();return Math.max(w,y,0)}}}};RU.each=RU.forEach;var Pun=function(){var i="undefined",o=(typeof Symbol>"u"?"undefined":df(Symbol))!=i&&df(Symbol.iterator)!=i;o&&(RU[Symbol.iterator]=function(){var l=this,f={value:void 0,done:!1},b=0,d=this.length;return UOe({next:function(){return b<d?f.value=l[b++]:(f.value=void 0,f.done=!0),f}},Symbol.iterator,function(){return this})})};Pun();var Bun=T1({nodeDimensionsIncludeLabels:!1}),jU={layoutDimensions:function(i){i=Bun(i);var o;if(!this.takesUpSpace())o={w:0,h:0};else if(i.nodeDimensionsIncludeLabels){var l=this.boundingBox();o={w:l.w,h:l.h}}else o={w:this.outerWidth(),h:this.outerHeight()};return(o.w===0||o.h===0)&&(o.w=o.h=1),o},layoutPositions:function(i,o,l){var f=this.nodes().filter(function(X){return!X.isParent()}),b=this.cy(),d=o.eles,w=function(te){return te.id()},y=dO(l,w);i.emit({type:"layoutstart",layout:i}),i.animations=[];var k=function(te,H,Y){var z={x:H.x1+H.w/2,y:H.y1+H.h/2},W={x:(Y.x-z.x)*te,y:(Y.y-z.y)*te};return{x:z.x+W.x,y:z.y+W.y}},E=o.spacingFactor&&o.spacingFactor!==1,T=function(){if(!E)return null;for(var te=J0(),H=0;H<f.length;H++){var Y=f[H],z=y(Y,H);Uon(te,z.x,z.y)}return te},C=T(),S=dO(function(X,te){var H=y(X,te);if(E){var Y=Math.abs(o.spacingFactor);H=k(Y,C,H)}return o.transform!=null&&(H=o.transform(X,H)),H},w);if(o.animate){for(var L=0;L<f.length;L++){var O=f[L],B=S(O,L),N=o.animateFilter==null||o.animateFilter(O,L);if(N){var F=O.animation({position:B,duration:o.animationDuration,easing:o.animationEasing});i.animations.push(F)}else O.position(B)}if(o.fit){var R=b.animation({fit:{boundingBox:d.boundingBoxAt(S),padding:o.padding},duration:o.animationDuration,easing:o.animationEasing});i.animations.push(R)}else if(o.zoom!==void 0&&o.pan!==void 0){var q=b.animation({zoom:o.zoom,pan:o.pan,duration:o.animationDuration,easing:o.animationEasing});i.animations.push(q)}i.animations.forEach(function(X){return X.play()}),i.one("layoutready",o.ready),i.emit({type:"layoutready",layout:i}),HC.all(i.animations.map(function(X){return X.promise()})).then(function(){i.one("layoutstop",o.stop),i.emit({type:"layoutstop",layout:i})})}else f.positions(S),o.fit&&b.fit(o.eles,o.padding),o.zoom!=null&&b.zoom(o.zoom),o.pan&&b.pan(o.pan),i.one("layoutready",o.ready),i.emit({type:"layoutready",layout:i}),i.one("layoutstop",o.stop),i.emit({type:"layoutstop",layout:i});return this},layout:function(i){var o=this.cy();return o.makeLayout(bs({},i,{eles:this}))}};jU.createLayout=jU.makeLayout=jU.layout;function pPe(r,i,o){var l=o._private,f=l.styleCache=l.styleCache||[],b;return(b=f[r])!=null||(b=f[r]=i(o)),b}function $U(r,i){return r=w7(r),function(l){return pPe(r,i,l)}}function HU(r,i){r=w7(r);var o=function(f){return i.call(f)};return function(){var f=this[0];if(f)return pPe(r,o,f)}}var C1={recalculateRenderedStyle:function(i){var o=this.cy(),l=o.renderer(),f=o.styleEnabled();return l&&f&&l.recalculateRenderedStyle(this,i),this},dirtyStyleCache:function(){var i=this.cy(),o=function(b){return b._private.styleCache=null};if(i.hasCompoundNodes()){var l;l=this.spawnSelf().merge(this.descendants()).merge(this.parents()),l.merge(l.connectedEdges()),l.forEach(o)}else this.forEach(function(f){o(f),f.connectedEdges().forEach(o)});return this},updateStyle:function(i){var o=this._private.cy;if(!o.styleEnabled())return this;if(o.batching()){var l=o._private.batchStyleEles;return l.merge(this),this}var f=o.hasCompoundNodes(),b=this;i=!!(i||i===void 0),f&&(b=this.spawnSelf().merge(this.descendants()).merge(this.parents()));var d=b;return i?d.emitAndNotify("style"):d.emit("style"),b.forEach(function(w){return w._private.styleDirty=!0}),this},cleanStyle:function(){var i=this.cy();if(!!i.styleEnabled())for(var o=0;o<this.length;o++){var l=this[o];l._private.styleDirty&&(l._private.styleDirty=!1,i.style().apply(l))}},parsedStyle:function(i){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0,l=this[0],f=l.cy();if(!!f.styleEnabled()&&l){this.cleanStyle();var b=l._private.style[i];return b!=null?b:o?f.style().getDefaultProperty(i):null}},numericStyle:function(i){var o=this[0];if(!!o.cy().styleEnabled()&&o){var l=o.pstyle(i);return l.pfValue!==void 0?l.pfValue:l.value}},numericStyleUnits:function(i){var o=this[0];if(!!o.cy().styleEnabled()&&o)return o.pstyle(i).units},renderedStyle:function(i){var o=this.cy();if(!o.styleEnabled())return this;var l=this[0];if(l)return o.style().getRenderedStyle(l,i)},style:function(i,o){var l=this.cy();if(!l.styleEnabled())return this;var f=!1,b=l.style();if(_o(i)){var d=i;b.applyBypass(this,d,f),this.emitAndNotify("style")}else if($i(i))if(o===void 0){var w=this[0];return w?b.getStylePropertyValue(w,i):void 0}else b.applyBypass(this,i,o,f),this.emitAndNotify("style");else if(i===void 0){var y=this[0];return y?b.getRawStyle(y):void 0}return this},removeStyle:function(i){var o=this.cy();if(!o.styleEnabled())return this;var l=!1,f=o.style(),b=this;if(i===void 0)for(var d=0;d<b.length;d++){var w=b[d];f.removeAllBypasses(w,l)}else{i=i.split(/\s+/);for(var y=0;y<b.length;y++){var k=b[y];f.removeBypasses(k,i,l)}}return this.emitAndNotify("style"),this},show:function(){return this.css("display","element"),this},hide:function(){return this.css("display","none"),this},effectiveOpacity:function(){var i=this.cy();if(!i.styleEnabled())return 1;var o=i.hasCompoundNodes(),l=this[0];if(l){var f=l._private,b=l.pstyle("opacity").value;if(!o)return b;var d=f.data.parent?l.parents():null;if(d)for(var w=0;w<d.length;w++){var y=d[w],k=y.pstyle("opacity").value;b=k*b}return b}},transparent:function(){var i=this.cy();if(!i.styleEnabled())return!1;var o=this[0],l=o.cy().hasCompoundNodes();if(o)return l?o.effectiveOpacity()===0:o.pstyle("opacity").value===0},backgrounding:function(){var i=this.cy();if(!i.styleEnabled())return!1;var o=this[0];return!!o._private.backgrounding}};function kue(r,i){var o=r._private,l=o.data.parent?r.parents():null;if(l)for(var f=0;f<l.length;f++){var b=l[f];if(!i(b))return!1}return!0}function xue(r){var i=r.ok,o=r.edgeOkViaNode||r.ok,l=r.parentOk||r.ok;return function(){var f=this.cy();if(!f.styleEnabled())return!0;var b=this[0],d=f.hasCompoundNodes();if(b){var w=b._private;if(!i(b))return!1;if(b.isNode())return!d||kue(b,l);var y=w.source,k=w.target;return o(y)&&(!d||kue(y,o))&&(y===k||o(k)&&(!d||kue(k,o)))}}}var qC=$U("eleTakesUpSpace",function(r){return r.pstyle("display").value==="element"&&r.width()!==0&&(r.isNode()?r.height()!==0:!0)});C1.takesUpSpace=HU("takesUpSpace",xue({ok:qC}));var Fun=$U("eleInteractive",function(r){return r.pstyle("events").value==="yes"&&r.pstyle("visibility").value==="visible"&&qC(r)}),Run=$U("parentInteractive",function(r){return r.pstyle("visibility").value==="visible"&&qC(r)});C1.interactive=HU("interactive",xue({ok:Fun,parentOk:Run,edgeOkViaNode:qC})),C1.noninteractive=function(){var r=this[0];if(r)return!r.interactive()};var jun=$U("eleVisible",function(r){return r.pstyle("visibility").value==="visible"&&r.pstyle("opacity").pfValue!==0&&qC(r)}),$un=qC;C1.visible=HU("visible",xue({ok:jun,edgeOkViaNode:$un})),C1.hidden=function(){var r=this[0];if(r)return!r.visible()},C1.isBundledBezier=HU("isBundledBezier",function(){return this.cy().styleEnabled()?!this.removed()&&this.pstyle("curve-style").value==="bezier"&&this.takesUpSpace():!1}),C1.bypass=C1.css=C1.style,C1.renderedCss=C1.renderedStyle,C1.removeBypass=C1.removeCss=C1.removeStyle,C1.pstyle=C1.parsedStyle;var L6={};function bPe(r){return function(){var i=arguments,o=[];if(i.length===2){var l=i[0],f=i[1];this.on(r.event,l,f)}else if(i.length===1&&al(i[0])){var b=i[0];this.on(r.event,b)}else if(i.length===0||i.length===1&&qc(i[0])){for(var d=i.length===1?i[0]:null,w=0;w<this.length;w++){var y=this[w],k=!r.ableField||y._private[r.ableField],E=y._private[r.field]!=r.value;if(r.overrideAble){var T=r.overrideAble(y);if(T!==void 0&&(k=T,!T))return this}k&&(y._private[r.field]=r.value,E&&o.push(y))}var C=this.spawn(o);C.updateStyle(),C.emit(r.event),d&&C.emit(d)}return this}}function YC(r){L6[r.field]=function(){var i=this[0];if(i){if(r.overrideField){var o=r.overrideField(i);if(o!==void 0)return o}return i._private[r.field]}},L6[r.on]=bPe({event:r.on,field:r.field,ableField:r.ableField,overrideAble:r.overrideAble,value:!0}),L6[r.off]=bPe({event:r.off,field:r.field,ableField:r.ableField,overrideAble:r.overrideAble,value:!1})}YC({field:"locked",overrideField:function(i){return i.cy().autolock()?!0:void 0},on:"lock",off:"unlock"}),YC({field:"grabbable",overrideField:function(i){return i.cy().autoungrabify()||i.pannable()?!1:void 0},on:"grabify",off:"ungrabify"}),YC({field:"selected",ableField:"selectable",overrideAble:function(i){return i.cy().autounselectify()?!1:void 0},on:"select",off:"unselect"}),YC({field:"selectable",overrideField:function(i){return i.cy().autounselectify()?!1:void 0},on:"selectify",off:"unselectify"}),L6.deselect=L6.unselect,L6.grabbed=function(){var r=this[0];if(r)return r._private.grabbed},YC({field:"active",on:"activate",off:"unactivate"}),YC({field:"pannable",on:"panify",off:"unpanify"}),L6.inactive=function(){var r=this[0];if(r)return!r._private.active};var fd={},vPe=function(i){return function(l){for(var f=this,b=[],d=0;d<f.length;d++){var w=f[d];if(!!w.isNode()){for(var y=!1,k=w.connectedEdges(),E=0;E<k.length;E++){var T=k[E],C=T.source(),S=T.target();if(i.noIncomingEdges&&S===w&&C!==w||i.noOutgoingEdges&&C===w&&S!==w){y=!0;break}}y||b.push(w)}}return this.spawn(b,!0).filter(l)}},wPe=function(i){return function(o){for(var l=this,f=[],b=0;b<l.length;b++){var d=l[b];if(!!d.isNode())for(var w=d.connectedEdges(),y=0;y<w.length;y++){var k=w[y],E=k.source(),T=k.target();i.outgoing&&E===d?(f.push(k),f.push(T)):i.incoming&&T===d&&(f.push(k),f.push(E))}}return this.spawn(f,!0).filter(o)}},mPe=function(i){return function(o){for(var l=this,f=[],b={};;){var d=i.outgoing?l.outgoers():l.incomers();if(d.length===0)break;for(var w=!1,y=0;y<d.length;y++){var k=d[y],E=k.id();b[E]||(b[E]=!0,f.push(k),w=!0)}if(!w)break;l=d}return this.spawn(f,!0).filter(o)}};fd.clearTraversalCache=function(){for(var r=0;r<this.length;r++)this[r]._private.traversalCache=null},bs(fd,{roots:vPe({noIncomingEdges:!0}),leaves:vPe({noOutgoingEdges:!0}),outgoers:j2(wPe({outgoing:!0}),"outgoers"),successors:mPe({outgoing:!0}),incomers:j2(wPe({incoming:!0}),"incomers"),predecessors:mPe({incoming:!0})}),bs(fd,{neighborhood:j2(function(r){for(var i=[],o=this.nodes(),l=0;l<o.length;l++)for(var f=o[l],b=f.connectedEdges(),d=0;d<b.length;d++){var w=b[d],y=w.source(),k=w.target(),E=f===y?k:y;E.length>0&&i.push(E[0]),i.push(w[0])}return this.spawn(i,!0).filter(r)},"neighborhood"),closedNeighborhood:function(i){return this.neighborhood().add(this).filter(i)},openNeighborhood:function(i){return this.neighborhood(i)}}),fd.neighbourhood=fd.neighborhood,fd.closedNeighbourhood=fd.closedNeighborhood,fd.openNeighbourhood=fd.openNeighborhood,bs(fd,{source:j2(function(i){var o=this[0],l;return o&&(l=o._private.source||o.cy().collection()),l&&i?l.filter(i):l},"source"),target:j2(function(i){var o=this[0],l;return o&&(l=o._private.target||o.cy().collection()),l&&i?l.filter(i):l},"target"),sources:yPe({attr:"source"}),targets:yPe({attr:"target"})});function yPe(r){return function(o){for(var l=[],f=0;f<this.length;f++){var b=this[f],d=b._private[r.attr];d&&l.push(d)}return this.spawn(l,!0).filter(o)}}bs(fd,{edgesWith:j2(kPe(),"edgesWith"),edgesTo:j2(kPe({thisIsSrc:!0}),"edgesTo")});function kPe(r){return function(o){var l=[],f=this._private.cy,b=r||{};$i(o)&&(o=f.$(o));for(var d=0;d<o.length;d++)for(var w=o[d]._private.edges,y=0;y<w.length;y++){var k=w[y],E=k._private.data,T=this.hasElementWithId(E.source)&&o.hasElementWithId(E.target),C=o.hasElementWithId(E.source)&&this.hasElementWithId(E.target),S=T||C;!S||(b.thisIsSrc||b.thisIsTgt)&&(b.thisIsSrc&&!T||b.thisIsTgt&&!C)||l.push(k)}return this.spawn(l,!0)}}bs(fd,{connectedEdges:j2(function(r){for(var i=[],o=this,l=0;l<o.length;l++){var f=o[l];if(!!f.isNode())for(var b=f._private.edges,d=0;d<b.length;d++){var w=b[d];i.push(w)}}return this.spawn(i,!0).filter(r)},"connectedEdges"),connectedNodes:j2(function(r){for(var i=[],o=this,l=0;l<o.length;l++){var f=o[l];!f.isEdge()||(i.push(f.source()[0]),i.push(f.target()[0]))}return this.spawn(i,!0).filter(r)},"connectedNodes"),parallelEdges:j2(xPe(),"parallelEdges"),codirectedEdges:j2(xPe({codirected:!0}),"codirectedEdges")});function xPe(r){var i={codirected:!1};return r=bs({},i,r),function(l){for(var f=[],b=this.edges(),d=r,w=0;w<b.length;w++)for(var y=b[w],k=y._private,E=k.source,T=E._private.data.id,C=k.data.target,S=E._private.edges,L=0;L<S.length;L++){var O=S[L],B=O._private.data,N=B.target,F=B.source,R=N===C&&F===T,q=T===N&&C===F;(d.codirected&&R||!d.codirected&&(R||q))&&f.push(O)}return this.spawn(f,!0).filter(l)}}bs(fd,{components:function(i){var o=this,l=o.cy(),f=l.collection(),b=i==null?o.nodes():i.nodes(),d=[];i!=null&&b.empty()&&(b=i.sources());var w=function(E,T){f.merge(E),b.unmerge(E),T.merge(E)};if(b.empty())return o.spawn();var y=function(){var E=l.collection();d.push(E);var T=b[0];w(T,E),o.bfs({directed:!1,roots:T,visit:function(S){return w(S,E)}}),E.forEach(function(C){C.connectedEdges().forEach(function(S){o.has(S)&&E.has(S.source())&&E.has(S.target())&&E.merge(S)})})};do y();while(b.length>0);return d},component:function(){var i=this[0];return i.cy().mutableElements().components(i)[0]}}),fd.componentsOf=fd.components;var S1=function(i,o){var l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1,f=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!1;if(i===void 0){Ml("A collection must have a reference to the core");return}var b=new Ym,d=!1;if(!o)o=[];else if(o.length>0&&_o(o[0])&&!fO(o[0])){d=!0;for(var w=[],y=new NC,k=0,E=o.length;k<E;k++){var T=o[k];T.data==null&&(T.data={});var C=T.data;if(C.id==null)C.id=lNe();else if(i.hasElementWithId(C.id)||y.has(C.id))continue;var S=new xU(i,T,!1);w.push(S),y.add(C.id)}o=w}this.length=0;for(var L=0,O=o.length;L<O;L++){var B=o[L][0];if(B!=null){var N=B._private.data.id;(!l||!b.has(N))&&(l&&b.set(N,{index:this.length,ele:B}),this[this.length]=B,this.length++)}}this._private={eles:this,cy:i,get map(){return this.lazyMap==null&&this.rebuildMap(),this.lazyMap},set map(F){this.lazyMap=F},rebuildMap:function(){for(var R=this.lazyMap=new Ym,q=this.eles,X=0;X<q.length;X++){var te=q[X];R.set(te.id(),{index:X,ele:te})}}},l&&(this._private.map=b),d&&!f&&this.restore()},ru=xU.prototype=S1.prototype=Object.create(Array.prototype);ru.instanceString=function(){return"collection"},ru.spawn=function(r,i){return new S1(this.cy(),r,i)},ru.spawnSelf=function(){return this.spawn(this)},ru.cy=function(){return this._private.cy},ru.renderer=function(){return this._private.cy.renderer()},ru.element=function(){return this[0]},ru.collection=function(){return KOe(this)?this:new S1(this._private.cy,[this])},ru.unique=function(){return new S1(this._private.cy,this,!0)},ru.hasElementWithId=function(r){return r=""+r,this._private.map.has(r)},ru.getElementById=function(r){r=""+r;var i=this._private.cy,o=this._private.map.get(r);return o?o.ele:new S1(i)},ru.$id=ru.getElementById,ru.poolIndex=function(){var r=this._private.cy,i=r._private.elements,o=this[0]._private.data.id;return i._private.map.get(o).index},ru.indexOf=function(r){var i=r[0]._private.data.id;return this._private.map.get(i).index},ru.indexOfId=function(r){return r=""+r,this._private.map.get(r).index},ru.json=function(r){var i=this.element(),o=this.cy();if(i==null&&r)return this;if(i!=null){var l=i._private;if(_o(r)){if(o.startBatch(),r.data){i.data(r.data);var f=l.data;if(i.isEdge()){var b=!1,d={},w=r.data.source,y=r.data.target;w!=null&&w!=f.source&&(d.source=""+w,b=!0),y!=null&&y!=f.target&&(d.target=""+y,b=!0),b&&(i=i.move(d))}else{var k="parent"in r.data,E=r.data.parent;k&&(E!=null||f.parent!=null)&&E!=f.parent&&(E===void 0&&(E=null),E!=null&&(E=""+E),i=i.move({parent:E}))}}r.position&&i.position(r.position);var T=function(O,B,N){var F=r[O];F!=null&&F!==l[O]&&(F?i[B]():i[N]())};return T("removed","remove","restore"),T("selected","select","unselect"),T("selectable","selectify","unselectify"),T("locked","lock","unlock"),T("grabbable","grabify","ungrabify"),T("pannable","panify","unpanify"),r.classes!=null&&i.classes(r.classes),o.endBatch(),this}else if(r===void 0){var C={data:qm(l.data),position:qm(l.position),group:l.group,removed:l.removed,selected:l.selected,selectable:l.selectable,locked:l.locked,grabbable:l.grabbable,pannable:l.pannable,classes:null};C.classes="";var S=0;return l.classes.forEach(function(L){return C.classes+=S++===0?L:" "+L}),C}}},ru.jsons=function(){for(var r=[],i=0;i<this.length;i++){var o=this[i],l=o.json();r.push(l)}return r},ru.clone=function(){for(var r=this.cy(),i=[],o=0;o<this.length;o++){var l=this[o],f=l.json(),b=new xU(r,f,!1);i.push(b)}return new S1(r,i)},ru.copy=ru.clone,ru.restore=function(){for(var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!0,i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0,o=this,l=o.cy(),f=l._private,b=[],d=[],w,y=0,k=o.length;y<k;y++){var E=o[y];i&&!E.removed()||(E.isNode()?b.push(E):d.push(E))}w=b.concat(d);var T,C=function(){w.splice(T,1),T--};for(T=0;T<w.length;T++){var S=w[T],L=S._private,O=L.data;if(S.clearTraversalCache(),!(!i&&!L.removed)){if(O.id===void 0)O.id=lNe();else if(Vr(O.id))O.id=""+O.id;else if(y6(O.id)||!$i(O.id)){Ml("Can not create element with invalid string ID `"+O.id+"`"),C();continue}else if(l.hasElementWithId(O.id)){Ml("Can not create second element with ID `"+O.id+"`"),C();continue}}var B=O.id;if(S.isNode()){var N=L.position;N.x==null&&(N.x=0),N.y==null&&(N.y=0)}if(S.isEdge()){for(var F=S,R=["source","target"],q=R.length,X=!1,te=0;te<q;te++){var H=R[te],Y=O[H];Vr(Y)&&(Y=O[H]=""+O[H]),Y==null||Y===""?(Ml("Can not create edge `"+B+"` with unspecified "+H),X=!0):l.hasElementWithId(Y)||(Ml("Can not create edge `"+B+"` with nonexistant "+H+" `"+Y+"`"),X=!0)}if(X){C();continue}var z=l.getElementById(O.source),W=l.getElementById(O.target);z.same(W)?z._private.edges.push(F):(z._private.edges.push(F),W._private.edges.push(F)),F._private.source=z,F._private.target=W}L.map=new Ym,L.map.set(B,{ele:S,index:0}),L.removed=!1,i&&l.addToPool(S)}for(var Z=0;Z<b.length;Z++){var G=b[Z],ae=G._private.data;Vr(ae.parent)&&(ae.parent=""+ae.parent);var $=ae.parent,ge=$!=null;if(ge||G._private.parent){var ee=G._private.parent?l.collection().merge(G._private.parent):l.getElementById($);if(ee.empty())ae.parent=void 0;else if(ee[0].removed())tc("Node added with missing parent, reference to parent removed"),ae.parent=void 0,G._private.parent=null;else{for(var de=!1,re=ee;!re.empty();){if(G.same(re)){de=!0,ae.parent=void 0;break}re=re.parent()}de||(ee[0]._private.children.push(G),G._private.parent=ee[0],f.hasCompoundNodes=!0)}}}if(w.length>0){for(var ke=w.length===o.length?o:new S1(l,w),Ce=0;Ce<ke.length;Ce++){var _e=ke[Ce];_e.isNode()||(_e.parallelEdges().clearTraversalCache(),_e.source().clearTraversalCache(),_e.target().clearTraversalCache())}var Te;f.hasCompoundNodes?Te=l.collection().merge(ke).merge(ke.connectedNodes()).merge(ke.parent()):Te=ke,Te.dirtyCompoundBoundsCache().dirtyBoundingBoxCache().updateStyle(r),r?ke.emitAndNotify("add"):i&&ke.emit("add")}return o},ru.removed=function(){var r=this[0];return r&&r._private.removed},ru.inside=function(){var r=this[0];return r&&!r._private.removed},ru.remove=function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!0,i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0,o=this,l=[],f={},b=o._private.cy;function d($){for(var ge=$._private.edges,ee=0;ee<ge.length;ee++)y(ge[ee])}function w($){for(var ge=$._private.children,ee=0;ee<ge.length;ee++)y(ge[ee])}function y($){var ge=f[$.id()];i&&$.removed()||ge||(f[$.id()]=!0,$.isNode()?(l.push($),d($),w($)):l.unshift($))}for(var k=0,E=o.length;k<E;k++){var T=o[k];y(T)}function C($,ge){var ee=$._private.edges;x6(ee,ge),$.clearTraversalCache()}function S($){$.clearTraversalCache()}var L=[];L.ids={};function O($,ge){ge=ge[0],$=$[0];var ee=$._private.children,de=$.id();x6(ee,ge),ge._private.parent=null,L.ids[de]||(L.ids[de]=!0,L.push($))}o.dirtyCompoundBoundsCache(),i&&b.removeFromPool(l);for(var B=0;B<l.length;B++){var N=l[B];if(N.isEdge()){var F=N.source()[0],R=N.target()[0];C(F,N),C(R,N);for(var q=N.parallelEdges(),X=0;X<q.length;X++){var te=q[X];S(te),te.isBundledBezier()&&te.dirtyBoundingBoxCache()}}else{var H=N.parent();H.length!==0&&O(H,N)}i&&(N._private.removed=!0)}var Y=b._private.elements;b._private.hasCompoundNodes=!1;for(var z=0;z<Y.length;z++){var W=Y[z];if(W.isParent()){b._private.hasCompoundNodes=!0;break}}var Z=new S1(this.cy(),l);Z.size()>0&&(r?Z.emitAndNotify("remove"):i&&Z.emit("remove"));for(var G=0;G<L.length;G++){var ae=L[G];(!i||!ae.removed())&&ae.updateStyle()}return Z},ru.move=function(r){var i=this._private.cy,o=this,l=!1,f=!1,b=function(L){return L==null?L:""+L};if(r.source!==void 0||r.target!==void 0){var d=b(r.source),w=b(r.target),y=d!=null&&i.hasElementWithId(d),k=w!=null&&i.hasElementWithId(w);(y||k)&&(i.batch(function(){o.remove(l,f),o.emitAndNotify("moveout");for(var S=0;S<o.length;S++){var L=o[S],O=L._private.data;L.isEdge()&&(y&&(O.source=d),k&&(O.target=w))}o.restore(l,f)}),o.emitAndNotify("move"))}else if(r.parent!==void 0){var E=b(r.parent),T=E===null||i.hasElementWithId(E);if(T){var C=E===null?void 0:E;i.batch(function(){var S=o.remove(l,f);S.emitAndNotify("moveout");for(var L=0;L<o.length;L++){var O=o[L],B=O._private.data;O.isNode()&&(B.parent=C)}S.restore(l,f)}),o.emitAndNotify("move")}}return this},[RNe,Zcn,IU,A6,GC,gun,OU,Mun,fPe,dPe,Nun,RU,jU,C1,L6,fd].forEach(function(r){bs(ru,r)});var Hun={add:function(i){var o,l=this;if(vp(i)){var f=i;if(f._private.cy===l)o=f.restore();else{for(var b=[],d=0;d<f.length;d++){var w=f[d];b.push(w.json())}o=new S1(l,b)}}else if(qc(i)){var y=i;o=new S1(l,y)}else if(_o(i)&&(qc(i.nodes)||qc(i.edges))){for(var k=i,E=[],T=["nodes","edges"],C=0,S=T.length;C<S;C++){var L=T[C],O=k[L];if(qc(O))for(var B=0,N=O.length;B<N;B++){var F=bs({group:L},O[B]);E.push(F)}}o=new S1(l,E)}else{var R=i;o=new xU(l,R).collection()}return o},remove:function(i){if(!vp(i)){if($i(i)){var o=i;i=this.$(o)}}return i.remove()}};/*! Bezier curve function generator. Copyright Gaetan Renaudeau. MIT License: http://en.wikipedia.org/wiki/MIT_License */function zun(r,i,o,l){var f=4,b=.001,d=1e-7,w=10,y=11,k=1/(y-1),E=typeof Float32Array<"u";if(arguments.length!==4)return!1;for(var T=0;T<4;++T)if(typeof arguments[T]!="number"||isNaN(arguments[T])||!isFinite(arguments[T]))return!1;r=Math.min(r,1),o=Math.min(o,1),r=Math.max(r,0),o=Math.max(o,0);var C=E?new Float32Array(y):new Array(y);function S(W,Z){return 1-3*Z+3*W}function L(W,Z){return 3*Z-6*W}function O(W){return 3*W}function B(W,Z,G){return((S(Z,G)*W+L(Z,G))*W+O(Z))*W}function N(W,Z,G){return 3*S(Z,G)*W*W+2*L(Z,G)*W+O(Z)}function F(W,Z){for(var G=0;G<f;++G){var ae=N(Z,r,o);if(ae===0)return Z;var $=B(Z,r,o)-W;Z-=$/ae}return Z}function R(){for(var W=0;W<y;++W)C[W]=B(W*k,r,o)}function q(W,Z,G){var ae,$,ge=0;do $=Z+(G-Z)/2,ae=B($,r,o)-W,ae>0?G=$:Z=$;while(Math.abs(ae)>d&&++ge<w);return $}function X(W){for(var Z=0,G=1,ae=y-1;G!==ae&&C[G]<=W;++G)Z+=k;--G;var $=(W-C[G])/(C[G+1]-C[G]),ge=Z+$*k,ee=N(ge,r,o);return ee>=b?F(W,ge):ee===0?ge:q(W,Z,Z+k)}var te=!1;function H(){te=!0,(r!==i||o!==l)&&R()}var Y=function(Z){return te||H(),r===i&&o===l?Z:Z===0?0:Z===1?1:B(X(Z),i,l)};Y.getControlPoints=function(){return[{x:r,y:i},{x:o,y:l}]};var z="generateBezier("+[r,i,o,l]+")";return Y.toString=function(){return z},Y}/*! Runge-Kutta spring physics function generator. Adapted from Framer.js, copyright Koen Bok. MIT License: http://en.wikipedia.org/wiki/MIT_License */var Gun=function(){function r(l){return-l.tension*l.x-l.friction*l.v}function i(l,f,b){var d={x:l.x+b.dx*f,v:l.v+b.dv*f,tension:l.tension,friction:l.friction};return{dx:d.v,dv:r(d)}}function o(l,f){var b={dx:l.v,dv:r(l)},d=i(l,f*.5,b),w=i(l,f*.5,d),y=i(l,f,w),k=1/6*(b.dx+2*(d.dx+w.dx)+y.dx),E=1/6*(b.dv+2*(d.dv+w.dv)+y.dv);return l.x=l.x+k*f,l.v=l.v+E*f,l}return function l(f,b,d){var w={x:-1,v:0,tension:null,friction:null},y=[0],k=0,E=1/1e4,T=16/1e3,C,S,L;for(f=parseFloat(f)||500,b=parseFloat(b)||20,d=d||null,w.tension=f,w.friction=b,C=d!==null,C?(k=l(f,b),S=k/d*T):S=T;L=o(L||w,S),y.push(1+L.x),k+=16,Math.abs(L.x)>E&&Math.abs(L.v)>E;);return C?function(O){return y[O*(y.length-1)|0]}:k}}(),iu=function(i,o,l,f){var b=zun(i,o,l,f);return function(d,w,y){return d+(w-d)*b(y)}},zU={linear:function(i,o,l){return i+(o-i)*l},ease:iu(.25,.1,.25,1),"ease-in":iu(.42,0,1,1),"ease-out":iu(0,0,.58,1),"ease-in-out":iu(.42,0,.58,1),"ease-in-sine":iu(.47,0,.745,.715),"ease-out-sine":iu(.39,.575,.565,1),"ease-in-out-sine":iu(.445,.05,.55,.95),"ease-in-quad":iu(.55,.085,.68,.53),"ease-out-quad":iu(.25,.46,.45,.94),"ease-in-out-quad":iu(.455,.03,.515,.955),"ease-in-cubic":iu(.55,.055,.675,.19),"ease-out-cubic":iu(.215,.61,.355,1),"ease-in-out-cubic":iu(.645,.045,.355,1),"ease-in-quart":iu(.895,.03,.685,.22),"ease-out-quart":iu(.165,.84,.44,1),"ease-in-out-quart":iu(.77,0,.175,1),"ease-in-quint":iu(.755,.05,.855,.06),"ease-out-quint":iu(.23,1,.32,1),"ease-in-out-quint":iu(.86,0,.07,1),"ease-in-expo":iu(.95,.05,.795,.035),"ease-out-expo":iu(.19,1,.22,1),"ease-in-out-expo":iu(1,0,0,1),"ease-in-circ":iu(.6,.04,.98,.335),"ease-out-circ":iu(.075,.82,.165,1),"ease-in-out-circ":iu(.785,.135,.15,.86),spring:function(i,o,l){if(l===0)return zU.linear;var f=Gun(i,o,l);return function(b,d,w){return b+(d-b)*f(w)}},"cubic-bezier":iu};function EPe(r,i,o,l,f){if(l===1||i===o)return o;var b=f(i,o,l);return r==null||((r.roundValue||r.color)&&(b=Math.round(b)),r.min!==void 0&&(b=Math.max(b,r.min)),r.max!==void 0&&(b=Math.min(b,r.max))),b}function _Pe(r,i){return r.pfValue!=null||r.value!=null?r.pfValue!=null&&(i==null||i.type.units!=="%")?r.pfValue:r.value:r}function WC(r,i,o,l,f){var b=f!=null?f.type:null;o<0?o=0:o>1&&(o=1);var d=_Pe(r,f),w=_Pe(i,f);if(Vr(d)&&Vr(w))return EPe(b,d,w,o,l);if(qc(d)&&qc(w)){for(var y=[],k=0;k<w.length;k++){var E=d[k],T=w[k];if(E!=null&&T!=null){var C=EPe(b,E,T,o,l);y.push(C)}else y.push(T)}return y}}function Vun(r,i,o,l){var f=!l,b=r._private,d=i._private,w=d.easing,y=d.startTime,k=l?r:r.cy(),E=k.style();if(!d.easingImpl)if(w==null)d.easingImpl=zU.linear;else{var T;if($i(w)){var C=E.parse("transition-timing-function",w);T=C.value}else T=w;var S,L;$i(T)?(S=T,L=[]):(S=T[1],L=T.slice(2).map(function(ke){return+ke})),L.length>0?(S==="spring"&&L.push(d.duration),d.easingImpl=zU[S].apply(null,L)):d.easingImpl=zU[S]}var O=d.easingImpl,B;if(d.duration===0?B=1:B=(o-y)/d.duration,d.applying&&(B=d.progress),B<0?B=0:B>1&&(B=1),d.delay==null){var N=d.startPosition,F=d.position;if(F&&f&&!r.locked()){var R={};MO(N.x,F.x)&&(R.x=WC(N.x,F.x,B,O)),MO(N.y,F.y)&&(R.y=WC(N.y,F.y,B,O)),r.position(R)}var q=d.startPan,X=d.pan,te=b.pan,H=X!=null&&l;H&&(MO(q.x,X.x)&&(te.x=WC(q.x,X.x,B,O)),MO(q.y,X.y)&&(te.y=WC(q.y,X.y,B,O)),r.emit("pan"));var Y=d.startZoom,z=d.zoom,W=z!=null&&l;W&&(MO(Y,z)&&(b.zoom=wO(b.minZoom,WC(Y,z,B,O),b.maxZoom)),r.emit("zoom")),(H||W)&&r.emit("viewport");var Z=d.style;if(Z&&Z.length>0&&f){for(var G=0;G<Z.length;G++){var ae=Z[G],$=ae.name,ge=ae,ee=d.startStyle[$],de=E.properties[ee.name],re=WC(ee,ge,B,O,de);E.overrideBypass(r,$,re)}r.emit("style")}}return d.progress=B,B}function MO(r,i){return r==null||i==null?!1:Vr(r)&&Vr(i)?!0:!!(r&&i)}function Uun(r,i,o,l){var f=i._private;f.started=!0,f.startTime=o-f.progress*f.duration}function TPe(r,i){var o=i._private.aniEles,l=[];function f(E,T){var C=E._private,S=C.animation.current,L=C.animation.queue,O=!1;if(S.length===0){var B=L.shift();B&&S.push(B)}for(var N=function(te){for(var H=te.length-1;H>=0;H--){var Y=te[H];Y()}te.splice(0,te.length)},F=S.length-1;F>=0;F--){var R=S[F],q=R._private;if(q.stopped){S.splice(F,1),q.hooked=!1,q.playing=!1,q.started=!1,N(q.frames);continue}!q.playing&&!q.applying||(q.playing&&q.applying&&(q.applying=!1),q.started||Uun(E,R,r),Vun(E,R,r,T),q.applying&&(q.applying=!1),N(q.frames),q.step!=null&&q.step(r),R.completed()&&(S.splice(F,1),q.hooked=!1,q.playing=!1,q.started=!1,N(q.completes)),O=!0)}return!T&&S.length===0&&L.length===0&&l.push(E),O}for(var b=!1,d=0;d<o.length;d++){var w=o[d],y=f(w);b=b||y}var k=f(i,!0);(b||k)&&(o.length>0?i.notify("draw",o):i.notify("draw")),o.unmerge(l),i.emit("step")}var qun={animate:nc.animate(),animation:nc.animation(),animated:nc.animated(),clearQueue:nc.clearQueue(),delay:nc.delay(),delayAnimation:nc.delayAnimation(),stop:nc.stop(),addToAnimationPool:function(i){var o=this;!o.styleEnabled()||o._private.aniEles.merge(i)},stopAnimationLoop:function(){this._private.animationsRunning=!1},startAnimationLoop:function(){var i=this;if(i._private.animationsRunning=!0,!i.styleEnabled())return;function o(){!i._private.animationsRunning||mU(function(b){TPe(b,i),o()})}var l=i.renderer();l&&l.beforeRender?l.beforeRender(function(b,d){TPe(d,i)},l.beforeRenderPriorities.animations):o()}},Yun={qualifierCompare:function(i,o){return i==null||o==null?i==null&&o==null:i.sameText(o)},eventMatches:function(i,o,l){var f=o.qualifier;return f!=null?i!==l.target&&fO(l.target)&&f.matches(l.target):!0},addEventFields:function(i,o){o.cy=i,o.target=i},callbackContext:function(i,o,l){return o.qualifier!=null?l.target:i}},GU=function(i){return $i(i)?new C6(i):i},CPe={createEmitter:function(){var i=this._private;return i.emitter||(i.emitter=new BU(Yun,this)),this},emitter:function(){return this._private.emitter},on:function(i,o,l){return this.emitter().on(i,GU(o),l),this},removeListener:function(i,o,l){return this.emitter().removeListener(i,GU(o),l),this},removeAllListeners:function(){return this.emitter().removeAllListeners(),this},one:function(i,o,l){return this.emitter().one(i,GU(o),l),this},once:function(i,o,l){return this.emitter().one(i,GU(o),l),this},emit:function(i,o){return this.emitter().emit(i,o),this},emitAndNotify:function(i,o){return this.emit(i),this.notify(i,o),this}};nc.eventAliasesOn(CPe);var Eue={png:function(i){var o=this._private.renderer;return i=i||{},o.png(i)},jpg:function(i){var o=this._private.renderer;return i=i||{},i.bg=i.bg||"#fff",o.jpg(i)}};Eue.jpeg=Eue.jpg;var VU={layout:function(i){var o=this;if(i==null){Ml("Layout options must be specified to make a layout");return}if(i.name==null){Ml("A `name` must be specified to make a layout");return}var l=i.name,f=o.extension("layout",l);if(f==null){Ml("No such layout `"+l+"` found.  Did you forget to import it and `cytoscape.use()` it?");return}var b;$i(i.eles)?b=o.$(i.eles):b=i.eles!=null?i.eles:o.$();var d=new f(bs({},i,{cy:o,eles:b}));return d}};VU.createLayout=VU.makeLayout=VU.layout;var Wun={notify:function(i,o){var l=this._private;if(this.batching()){l.batchNotifications=l.batchNotifications||{};var f=l.batchNotifications[i]=l.batchNotifications[i]||this.collection();o!=null&&f.merge(o);return}if(!!l.notificationsEnabled){var b=this.renderer();this.destroyed()||!b||b.notify(i,o)}},notifications:function(i){var o=this._private;return i===void 0?o.notificationsEnabled:(o.notificationsEnabled=!!i,this)},noNotifications:function(i){this.notifications(!1),i(),this.notifications(!0)},batching:function(){return this._private.batchCount>0},startBatch:function(){var i=this._private;return i.batchCount==null&&(i.batchCount=0),i.batchCount===0&&(i.batchStyleEles=this.collection(),i.batchNotifications={}),i.batchCount++,this},endBatch:function(){var i=this._private;if(i.batchCount===0)return this;if(i.batchCount--,i.batchCount===0){i.batchStyleEles.updateStyle();var o=this.renderer();Object.keys(i.batchNotifications).forEach(function(l){var f=i.batchNotifications[l];f.empty()?o.notify(l):o.notify(l,f)})}return this},batch:function(i){return this.startBatch(),i(),this.endBatch(),this},batchData:function(i){var o=this;return this.batch(function(){for(var l=Object.keys(i),f=0;f<l.length;f++){var b=l[f],d=i[b],w=o.getElementById(b);w.data(d)}})}},Kun=T1({hideEdgesOnViewport:!1,textureOnViewport:!1,motionBlur:!1,motionBlurOpacity:.05,pixelRatio:void 0,desktopTapThreshold:4,touchTapThreshold:8,wheelSensitivity:1,debug:!1,showFps:!1}),_ue={renderTo:function(i,o,l,f){var b=this._private.renderer;return b.renderTo(i,o,l,f),this},renderer:function(){return this._private.renderer},forceRender:function(){return this.notify("draw"),this},resize:function(){return this.invalidateSize(),this.emitAndNotify("resize"),this},initRenderer:function(i){var o=this,l=o.extension("renderer",i.name);if(l==null){Ml("Can not initialise: No such renderer `".concat(i.name,"` found. Did you forget to import it and `cytoscape.use()` it?"));return}i.wheelSensitivity!==void 0&&tc("You have set a custom wheel sensitivity.  This will make your app zoom unnaturally when using mainstream mice.  You should change this value from the default only if you can guarantee that all your users will use the same hardware and OS configuration as your current machine.");var f=Kun(i);f.cy=o,o._private.renderer=new l(f),this.notify("init")},destroyRenderer:function(){var i=this;i.notify("destroy");var o=i.container();if(o)for(o._cyreg=null;o.childNodes.length>0;)o.removeChild(o.childNodes[0]);i._private.renderer=null,i.mutableElements().forEach(function(l){var f=l._private;f.rscratch={},f.rstyle={},f.animation.current=[],f.animation.queue=[]})},onRender:function(i){return this.on("render",i)},offRender:function(i){return this.off("render",i)}};_ue.invalidateDimensions=_ue.resize;var UU={collection:function(i,o){return $i(i)?this.$(i):vp(i)?i.collection():qc(i)?(o||(o={}),new S1(this,i,o.unique,o.removed)):new S1(this)},nodes:function(i){var o=this.$(function(l){return l.isNode()});return i?o.filter(i):o},edges:function(i){var o=this.$(function(l){return l.isEdge()});return i?o.filter(i):o},$:function(i){var o=this._private.elements;return i?o.filter(i):o.spawnSelf()},mutableElements:function(){return this._private.elements}};UU.elements=UU.filter=UU.$;var dd={},DO="t",Xun="f";dd.apply=function(r){for(var i=this,o=i._private,l=o.cy,f=l.collection(),b=0;b<r.length;b++){var d=r[b],w=i.getContextMeta(d);if(!w.empty){var y=i.getContextStyle(w),k=i.applyContextStyle(w,y,d);d._private.appliedInitStyle?i.updateTransitions(d,k.diffProps):d._private.appliedInitStyle=!0;var E=i.updateStyleHints(d);E&&f.push(d)}}return f},dd.getPropertiesDiff=function(r,i){var o=this,l=o._private.propDiffs=o._private.propDiffs||{},f=r+"-"+i,b=l[f];if(b)return b;for(var d=[],w={},y=0;y<o.length;y++){var k=o[y],E=r[y]===DO,T=i[y]===DO,C=E!==T,S=k.mappedProperties.length>0;if(C||T&&S){var L=void 0;C&&S||C?L=k.properties:S&&(L=k.mappedProperties);for(var O=0;O<L.length;O++){for(var B=L[O],N=B.name,F=!1,R=y+1;R<o.length;R++){var q=o[R],X=i[R]===DO;if(!!X&&(F=q.properties[B.name]!=null,F))break}!w[N]&&!F&&(w[N]=!0,d.push(N))}}}return l[f]=d,d},dd.getContextMeta=function(r){for(var i=this,o="",l,f=r._private.styleCxtKey||"",b=0;b<i.length;b++){var d=i[b],w=d.selector&&d.selector.matches(r);w?o+=DO:o+=Xun}return l=i.getPropertiesDiff(f,o),r._private.styleCxtKey=o,{key:o,diffPropNames:l,empty:l.length===0}},dd.getContextStyle=function(r){var i=r.key,o=this,l=this._private.contextStyles=this._private.contextStyles||{};if(l[i])return l[i];for(var f={_private:{key:i}},b=0;b<o.length;b++){var d=o[b],w=i[b]===DO;if(!!w)for(var y=0;y<d.properties.length;y++){var k=d.properties[y];f[k.name]=k}}return l[i]=f,f},dd.applyContextStyle=function(r,i,o){for(var l=this,f=r.diffPropNames,b={},d=l.types,w=0;w<f.length;w++){var y=f[w],k=i[y],E=o.pstyle(y);if(!k)if(E)E.bypass?k={name:y,deleteBypassed:!0}:k={name:y,delete:!0};else continue;if(E!==k){if(k.mapped===d.fn&&E!=null&&E.mapping!=null&&E.mapping.value===k.value){var T=E.mapping,C=T.fnValue=k.value(o);if(C===T.prevFnValue)continue}var S=b[y]={prev:E};l.applyParsedProperty(o,k),S.next=o.pstyle(y),S.next&&S.next.bypass&&(S.next=S.next.bypassed)}}return{diffProps:b}},dd.updateStyleHints=function(r){var i=r._private,o=this,l=o.propertyGroupNames,f=o.propertyGroupKeys,b=function(Le,$e,xe){return o.getPropertiesHash(Le,$e,xe)},d=i.styleKey;if(r.removed())return!1;var w=i.group==="nodes",y=r._private.style;l=Object.keys(y);for(var k=0;k<f.length;k++){var E=f[k];i.styleKeys[E]=[OC,gO]}for(var T=function(Le,$e){return i.styleKeys[$e][0]=pO(Le,i.styleKeys[$e][0])},C=function(Le,$e){return i.styleKeys[$e][1]=bO(Le,i.styleKeys[$e][1])},S=function(Le,$e){T(Le,$e),C(Le,$e)},L=function(Le,$e){for(var xe=0;xe<Le.length;xe++){var Ae=Le.charCodeAt(xe);T(Ae,$e),C(Ae,$e)}},O=2e9,B=function(Le){return-128<Le&&Le<128&&Math.floor(Le)!==Le?O-(Le*1024|0):Le},N=0;N<l.length;N++){var F=l[N],R=y[F];if(R!=null){var q=this.properties[F],X=q.type,te=q.groupKey,H=void 0;q.hashOverride!=null?H=q.hashOverride(r,R):R.pfValue!=null&&(H=R.pfValue);var Y=q.enums==null?R.value:null,z=H!=null,W=Y!=null,Z=z||W,G=R.units;if(X.number&&Z&&!X.multiple){var ae=z?H:Y;S(B(ae),te),!z&&G!=null&&L(G,te)}else L(R.strValue,te)}}for(var $=[OC,gO],ge=0;ge<f.length;ge++){var ee=f[ge],de=i.styleKeys[ee];$[0]=pO(de[0],$[0]),$[1]=bO(de[1],$[1])}i.styleKey=lon($[0],$[1]);var re=i.styleKeys;i.labelDimsKey=k6(re.labelDimensions);var ke=b(r,["label"],re.labelDimensions);if(i.labelKey=k6(ke),i.labelStyleKey=k6(yU(re.commonLabel,ke)),!w){var Ce=b(r,["source-label"],re.labelDimensions);i.sourceLabelKey=k6(Ce),i.sourceLabelStyleKey=k6(yU(re.commonLabel,Ce));var _e=b(r,["target-label"],re.labelDimensions);i.targetLabelKey=k6(_e),i.targetLabelStyleKey=k6(yU(re.commonLabel,_e))}if(w){var Te=i.styleKeys,Be=Te.nodeBody,Ge=Te.nodeBorder,Xe=Te.backgroundImage,Ee=Te.compound,Ze=Te.pie,Ie=[Be,Ge,Xe,Ee,Ze].filter(function(Oe){return Oe!=null}).reduce(yU,[OC,gO]);i.nodeKey=k6(Ie),i.hasPie=Ze!=null&&Ze[0]!==OC&&Ze[1]!==gO}return d!==i.styleKey},dd.clearStyleHints=function(r){var i=r._private;i.styleCxtKey="",i.styleKeys={},i.styleKey=null,i.labelKey=null,i.labelStyleKey=null,i.sourceLabelKey=null,i.sourceLabelStyleKey=null,i.targetLabelKey=null,i.targetLabelStyleKey=null,i.nodeKey=null,i.hasPie=null},dd.applyParsedProperty=function(r,i){var o=this,l=i,f=r._private.style,b,d=o.types,w=o.properties[l.name].type,y=l.bypass,k=f[l.name],E=k&&k.bypass,T=r._private,C="mapping",S=function(Be){return Be==null?null:Be.pfValue!=null?Be.pfValue:Be.value},L=function(){var Be=S(k),Ge=S(l);o.checkTriggers(r,l.name,Be,Ge)};if(l&&l.name.substr(0,3)==="pie"&&tc("The pie style properties are deprecated.  Create charts using background images instead."),i.name==="curve-style"&&r.isEdge()&&(i.value!=="bezier"&&r.isLoop()||i.value==="haystack"&&(r.source().isParent()||r.target().isParent()))&&(l=i=this.parse(i.name,"bezier",y)),l.delete)return f[l.name]=void 0,L(),!0;if(l.deleteBypassed)return k?k.bypass?(k.bypassed=void 0,L(),!0):!1:(L(),!0);if(l.deleteBypass)return k?k.bypass?(f[l.name]=k.bypassed,L(),!0):!1:(L(),!0);var O=function(){tc("Do not assign mappings to elements without corresponding data (i.e. ele `"+r.id()+"` has no mapping for property `"+l.name+"` with data field `"+l.field+"`); try a `["+l.field+"]` selector to limit scope to elements with `"+l.field+"` defined")};switch(l.mapped){case d.mapData:{for(var B=l.field.split("."),N=T.data,F=0;F<B.length&&N;F++){var R=B[F];N=N[R]}if(N==null)return O(),!1;var q;if(Vr(N)){var X=l.fieldMax-l.fieldMin;X===0?q=0:q=(N-l.fieldMin)/X}else return tc("Do not use continuous mappers without specifying numeric data (i.e. `"+l.field+": "+N+"` for `"+r.id()+"` is non-numeric)"),!1;if(q<0?q=0:q>1&&(q=1),w.color){var te=l.valueMin[0],H=l.valueMax[0],Y=l.valueMin[1],z=l.valueMax[1],W=l.valueMin[2],Z=l.valueMax[2],G=l.valueMin[3]==null?1:l.valueMin[3],ae=l.valueMax[3]==null?1:l.valueMax[3],$=[Math.round(te+(H-te)*q),Math.round(Y+(z-Y)*q),Math.round(W+(Z-W)*q),Math.round(G+(ae-G)*q)];b={bypass:l.bypass,name:l.name,value:$,strValue:"rgb("+$[0]+", "+$[1]+", "+$[2]+")"}}else if(w.number){var ge=l.valueMin+(l.valueMax-l.valueMin)*q;b=this.parse(l.name,ge,l.bypass,C)}else return!1;if(!b)return O(),!1;b.mapping=l,l=b;break}case d.data:{for(var ee=l.field.split("."),de=T.data,re=0;re<ee.length&&de;re++){var ke=ee[re];de=de[ke]}if(de!=null&&(b=this.parse(l.name,de,l.bypass,C)),!b)return O(),!1;b.mapping=l,l=b;break}case d.fn:{var Ce=l.value,_e=l.fnValue!=null?l.fnValue:Ce(r);if(l.prevFnValue=_e,_e==null)return tc("Custom function mappers may not return null (i.e. `"+l.name+"` for ele `"+r.id()+"` is null)"),!1;if(b=this.parse(l.name,_e,l.bypass,C),!b)return tc("Custom function mappers may not return invalid values for the property type (i.e. `"+l.name+"` for ele `"+r.id()+"` is invalid)"),!1;b.mapping=qm(l),l=b;break}case void 0:break;default:return!1}return y?(E?l.bypassed=k.bypassed:l.bypassed=k,f[l.name]=l):E?k.bypassed=l:f[l.name]=l,L(),!0},dd.cleanElements=function(r,i){for(var o=0;o<r.length;o++){var l=r[o];if(this.clearStyleHints(l),l.dirtyCompoundBoundsCache(),l.dirtyBoundingBoxCache(),!i)l._private.style={};else for(var f=l._private.style,b=Object.keys(f),d=0;d<b.length;d++){var w=b[d],y=f[w];y!=null&&(y.bypass?y.bypassed=null:f[w]=null)}}},dd.update=function(){var r=this._private.cy,i=r.mutableElements();i.updateStyle()},dd.updateTransitions=function(r,i){var o=this,l=r._private,f=r.pstyle("transition-property").value,b=r.pstyle("transition-duration").pfValue,d=r.pstyle("transition-delay").pfValue;if(f.length>0&&b>0){for(var w={},y=!1,k=0;k<f.length;k++){var E=f[k],T=r.pstyle(E),C=i[E];if(!!C){var S=C.prev,L=S,O=C.next!=null?C.next:T,B=!1,N=void 0,F=1e-6;!L||(Vr(L.pfValue)&&Vr(O.pfValue)?(B=O.pfValue-L.pfValue,N=L.pfValue+F*B):Vr(L.value)&&Vr(O.value)?(B=O.value-L.value,N=L.value+F*B):qc(L.value)&&qc(O.value)&&(B=L.value[0]!==O.value[0]||L.value[1]!==O.value[1]||L.value[2]!==O.value[2],N=L.strValue),B&&(w[E]=O.strValue,this.applyBypass(r,E,N),y=!0))}}if(!y)return;l.transitioning=!0,new HC(function(R){d>0?r.delayAnimation(d).play().promise().then(R):R()}).then(function(){return r.animation({style:w,duration:b,easing:r.pstyle("transition-timing-function").value,queue:!1}).play().promise()}).then(function(){o.removeBypasses(r,f),r.emitAndNotify("style"),l.transitioning=!1})}else l.transitioning&&(this.removeBypasses(r,f),r.emitAndNotify("style"),l.transitioning=!1)},dd.checkTrigger=function(r,i,o,l,f,b){var d=this.properties[i],w=f(d);w!=null&&w(o,l)&&b(d)},dd.checkZOrderTrigger=function(r,i,o,l){var f=this;this.checkTrigger(r,i,o,l,function(b){return b.triggersZOrder},function(){f._private.cy.notify("zorder",r)})},dd.checkBoundsTrigger=function(r,i,o,l){this.checkTrigger(r,i,o,l,function(f){return f.triggersBounds},function(f){r.dirtyCompoundBoundsCache(),r.dirtyBoundingBoxCache(),f.triggersBoundsOfParallelBeziers&&(i==="curve-style"&&(o==="bezier"||l==="bezier")||i==="display"&&(o==="none"||l==="none"))&&r.parallelEdges().forEach(function(b){b.isBundledBezier()&&b.dirtyBoundingBoxCache()})})},dd.checkTriggers=function(r,i,o,l){r.dirtyStyleCache(),this.checkZOrderTrigger(r,i,o,l),this.checkBoundsTrigger(r,i,o,l)};var LO={};LO.applyBypass=function(r,i,o,l){var f=this,b=[],d=!0;if(i==="*"||i==="**"){if(o!==void 0)for(var w=0;w<f.properties.length;w++){var y=f.properties[w],k=y.name,E=this.parse(k,o,!0);E&&b.push(E)}}else if($i(i)){var T=this.parse(i,o,!0);T&&b.push(T)}else if(_o(i)){var C=i;l=o;for(var S=Object.keys(C),L=0;L<S.length;L++){var O=S[L],B=C[O];if(B===void 0&&(B=C[wU(O)]),B!==void 0){var N=this.parse(O,B,!0);N&&b.push(N)}}}else return!1;if(b.length===0)return!1;for(var F=!1,R=0;R<r.length;R++){for(var q=r[R],X={},te=void 0,H=0;H<b.length;H++){var Y=b[H];if(l){var z=q.pstyle(Y.name);te=X[Y.name]={prev:z}}F=this.applyParsedProperty(q,qm(Y))||F,l&&(te.next=q.pstyle(Y.name))}F&&this.updateStyleHints(q),l&&this.updateTransitions(q,X,d)}return F},LO.overrideBypass=function(r,i,o){i=Zce(i);for(var l=0;l<r.length;l++){var f=r[l],b=f._private.style[i],d=this.properties[i].type,w=d.color,y=d.mutiple,k=b?b.pfValue!=null?b.pfValue:b.value:null;!b||!b.bypass?this.applyBypass(f,i,o):(b.value=o,b.pfValue!=null&&(b.pfValue=o),w?b.strValue="rgb("+o.join(",")+")":y?b.strValue=o.join(" "):b.strValue=""+o,this.updateStyleHints(f)),this.checkTriggers(f,i,k,o)}},LO.removeAllBypasses=function(r,i){return this.removeBypasses(r,this.propertyNames,i)},LO.removeBypasses=function(r,i,o){for(var l=!0,f=0;f<r.length;f++){for(var b=r[f],d={},w=0;w<i.length;w++){var y=i[w],k=this.properties[y],E=b.pstyle(k.name);if(!(!E||!E.bypass)){var T="",C=this.parse(y,T,!0),S=d[k.name]={prev:E};this.applyParsedProperty(b,C),S.next=b.pstyle(k.name)}}this.updateStyleHints(b),o&&this.updateTransitions(b,d,l)}};var Tue={};Tue.getEmSizeInPixels=function(){var r=this.containerCss("font-size");return r!=null?parseFloat(r):1},Tue.containerCss=function(r){var i=this._private.cy,o=i.container();if(Fu&&o&&Fu.getComputedStyle)return Fu.getComputedStyle(o).getPropertyValue(r)};var Wm={};Wm.getRenderedStyle=function(r,i){return i?this.getStylePropertyValue(r,i,!0):this.getRawStyle(r,!0)},Wm.getRawStyle=function(r,i){var o=this;if(r=r[0],r){for(var l={},f=0;f<o.properties.length;f++){var b=o.properties[f],d=o.getStylePropertyValue(r,b.name,i);d!=null&&(l[b.name]=d,l[wU(b.name)]=d)}return l}},Wm.getIndexedStyle=function(r,i,o,l){var f=r.pstyle(i)[o][l];return f!=null?f:r.cy().style().getDefaultProperty(i)[o][0]},Wm.getStylePropertyValue=function(r,i,o){var l=this;if(r=r[0],r){var f=l.properties[i];f.alias&&(f=f.pointsTo);var b=f.type,d=r.pstyle(f.name);if(d){var w=d.value,y=d.units,k=d.strValue;if(o&&b.number&&w!=null&&Vr(w)){var E=r.cy().zoom(),T=function(B){return B*E},C=function(B,N){return T(B)+N},S=qc(w),L=S?y.every(function(O){return O!=null}):y!=null;return L?S?w.map(function(O,B){return C(O,y[B])}).join(" "):C(w,y):S?w.map(function(O){return $i(O)?O:""+T(O)}).join(" "):""+T(w)}else if(k!=null)return k}return null}},Wm.getAnimationStartStyle=function(r,i){for(var o={},l=0;l<i.length;l++){var f=i[l],b=f.name,d=r.pstyle(b);d!==void 0&&(_o(d)?d=this.parse(b,d.strValue):d=this.parse(b,d)),d&&(o[b]=d)}return o},Wm.getPropsList=function(r){var i=this,o=[],l=r,f=i.properties;if(l)for(var b=Object.keys(l),d=0;d<b.length;d++){var w=b[d],y=l[w],k=f[w]||f[Zce(w)],E=this.parse(k.name,y);E&&o.push(E)}return o},Wm.getNonDefaultPropertiesHash=function(r,i,o){var l=o.slice(),f,b,d,w,y,k;for(y=0;y<i.length;y++)if(f=i[y],b=r.pstyle(f,!1),b!=null)if(b.pfValue!=null)l[0]=pO(w,l[0]),l[1]=bO(w,l[1]);else for(d=b.strValue,k=0;k<d.length;k++)w=d.charCodeAt(k),l[0]=pO(w,l[0]),l[1]=bO(w,l[1]);return l},Wm.getPropertiesHash=Wm.getNonDefaultPropertiesHash;var qU={};qU.appendFromJson=function(r){for(var i=this,o=0;o<r.length;o++){var l=r[o],f=l.selector,b=l.style||l.css,d=Object.keys(b);i.selector(f);for(var w=0;w<d.length;w++){var y=d[w],k=b[y];i.css(y,k)}}return i},qU.fromJson=function(r){var i=this;return i.resetToDefault(),i.appendFromJson(r),i},qU.json=function(){for(var r=[],i=this.defaultLength;i<this.length;i++){for(var o=this[i],l=o.selector,f=o.properties,b={},d=0;d<f.length;d++){var w=f[d];b[w.name]=w.strValue}r.push({selector:l?l.toString():"core",style:b})}return r};var Cue={};Cue.appendFromString=function(r){var i=this,o=this,l=""+r,f,b,d;l=l.replace(/[/][*](\s|.)+?[*][/]/g,"");function w(){l.length>f.length?l=l.substr(f.length):l=""}function y(){b.length>d.length?b=b.substr(d.length):b=""}for(;;){var k=l.match(/^\s*$/);if(k)break;var E=l.match(/^\s*((?:.|\s)+?)\s*\{((?:.|\s)+?)\}/);if(!E){tc("Halting stylesheet parsing: String stylesheet contains more to parse but no selector and block found in: "+l);break}f=E[0];var T=E[1];if(T!=="core"){var C=new C6(T);if(C.invalid){tc("Skipping parsing of block: Invalid selector found in string stylesheet: "+T),w();continue}}var S=E[2],L=!1;b=S;for(var O=[];;){var B=b.match(/^\s*$/);if(B)break;var N=b.match(/^\s*(.+?)\s*:\s*(.+?)(?:\s*;|\s*$)/);if(!N){tc("Skipping parsing of block: Invalid formatting of style property and value definitions found in:"+S),L=!0;break}d=N[0];var F=N[1],R=N[2],q=i.properties[F];if(!q){tc("Skipping property: Invalid property name in: "+d),y();continue}var X=o.parse(F,R);if(!X){tc("Skipping property: Invalid property definition in: "+d),y();continue}O.push({name:F,val:R}),y()}if(L){w();break}o.selector(T);for(var te=0;te<O.length;te++){var H=O[te];o.css(H.name,H.val)}w()}return o},Cue.fromString=function(r){var i=this;return i.resetToDefault(),i.appendFromString(r),i};var A1={};(function(){var r=gf,i=Qan,o=Jan,l=eon,f=ton,b=function(Ie){return"^"+Ie+"\\s*\\(\\s*([\\w\\.]+)\\s*\\)$"},d=function(Ie){var Oe=r+"|\\w+|"+i+"|"+o+"|"+l+"|"+f;return"^"+Ie+"\\s*\\(([\\w\\.]+)\\s*\\,\\s*("+r+")\\s*\\,\\s*("+r+")\\s*,\\s*("+Oe+")\\s*\\,\\s*("+Oe+")\\)$"},w=[`^url\\s*\\(\\s*['"]?(.+?)['"]?\\s*\\)$`,"^(none)$","^(.+)$"];A1.types={time:{number:!0,min:0,units:"s|ms",implicitUnits:"ms"},percent:{number:!0,min:0,max:100,units:"%",implicitUnits:"%"},percentages:{number:!0,min:0,max:100,units:"%",implicitUnits:"%",multiple:!0},zeroOneNumber:{number:!0,min:0,max:1,unitless:!0},zeroOneNumbers:{number:!0,min:0,max:1,unitless:!0,multiple:!0},nOneOneNumber:{number:!0,min:-1,max:1,unitless:!0},nonNegativeInt:{number:!0,min:0,integer:!0,unitless:!0},position:{enums:["parent","origin"]},nodeSize:{number:!0,min:0,enums:["label"]},number:{number:!0,unitless:!0},numbers:{number:!0,unitless:!0,multiple:!0},positiveNumber:{number:!0,unitless:!0,min:0,strictMin:!0},size:{number:!0,min:0},bidirectionalSize:{number:!0},bidirectionalSizeMaybePercent:{number:!0,allowPercent:!0},bidirectionalSizes:{number:!0,multiple:!0},sizeMaybePercent:{number:!0,min:0,allowPercent:!0},axisDirection:{enums:["horizontal","leftward","rightward","vertical","upward","downward","auto"]},paddingRelativeTo:{enums:["width","height","average","min","max"]},bgWH:{number:!0,min:0,allowPercent:!0,enums:["auto"],multiple:!0},bgPos:{number:!0,allowPercent:!0,multiple:!0},bgRelativeTo:{enums:["inner","include-padding"],multiple:!0},bgRepeat:{enums:["repeat","repeat-x","repeat-y","no-repeat"],multiple:!0},bgFit:{enums:["none","contain","cover"],multiple:!0},bgCrossOrigin:{enums:["anonymous","use-credentials"],multiple:!0},bgClip:{enums:["none","node"],multiple:!0},bgContainment:{enums:["inside","over"],multiple:!0},color:{color:!0},colors:{color:!0,multiple:!0},fill:{enums:["solid","linear-gradient","radial-gradient"]},bool:{enums:["yes","no"]},bools:{enums:["yes","no"],multiple:!0},lineStyle:{enums:["solid","dotted","dashed"]},lineCap:{enums:["butt","round","square"]},borderStyle:{enums:["solid","dotted","dashed","double"]},curveStyle:{enums:["bezier","unbundled-bezier","haystack","segments","straight","straight-triangle","taxi"]},fontFamily:{regex:'^([\\w- \\"]+(?:\\s*,\\s*[\\w- \\"]+)*)$'},fontStyle:{enums:["italic","normal","oblique"]},fontWeight:{enums:["normal","bold","bolder","lighter","100","200","300","400","500","600","800","900",100,200,300,400,500,600,700,800,900]},textDecoration:{enums:["none","underline","overline","line-through"]},textTransform:{enums:["none","uppercase","lowercase"]},textWrap:{enums:["none","wrap","ellipsis"]},textOverflowWrap:{enums:["whitespace","anywhere"]},textBackgroundShape:{enums:["rectangle","roundrectangle","round-rectangle"]},nodeShape:{enums:["rectangle","roundrectangle","round-rectangle","cutrectangle","cut-rectangle","bottomroundrectangle","bottom-round-rectangle","barrel","ellipse","triangle","round-triangle","square","pentagon","round-pentagon","hexagon","round-hexagon","concavehexagon","concave-hexagon","heptagon","round-heptagon","octagon","round-octagon","tag","round-tag","star","diamond","round-diamond","vee","rhomboid","polygon"]},overlayShape:{enums:["roundrectangle","round-rectangle","ellipse"]},compoundIncludeLabels:{enums:["include","exclude"]},arrowShape:{enums:["tee","triangle","triangle-tee","circle-triangle","triangle-cross","triangle-backcurve","vee","square","circle","diamond","chevron","none"]},arrowFill:{enums:["filled","hollow"]},display:{enums:["element","none"]},visibility:{enums:["hidden","visible"]},zCompoundDepth:{enums:["bottom","orphan","auto","top"]},zIndexCompare:{enums:["auto","manual"]},valign:{enums:["top","center","bottom"]},halign:{enums:["left","center","right"]},justification:{enums:["left","center","right","auto"]},text:{string:!0},data:{mapping:!0,regex:b("data")},layoutData:{mapping:!0,regex:b("layoutData")},scratch:{mapping:!0,regex:b("scratch")},mapData:{mapping:!0,regex:d("mapData")},mapLayoutData:{mapping:!0,regex:d("mapLayoutData")},mapScratch:{mapping:!0,regex:d("mapScratch")},fn:{mapping:!0,fn:!0},url:{regexes:w,singleRegexMatchValue:!0},urls:{regexes:w,singleRegexMatchValue:!0,multiple:!0},propList:{propList:!0},angle:{number:!0,units:"deg|rad",implicitUnits:"rad"},textRotation:{number:!0,units:"deg|rad",implicitUnits:"rad",enums:["none","autorotate"]},polygonPointList:{number:!0,multiple:!0,evenMultiple:!0,min:-1,max:1,unitless:!0},edgeDistances:{enums:["intersection","node-position"]},edgeEndpoint:{number:!0,multiple:!0,units:"%|px|em|deg|rad",implicitUnits:"px",enums:["inside-to-node","outside-to-node","outside-to-node-or-label","outside-to-line","outside-to-line-or-label"],singleEnum:!0,validate:function(Ie,Oe){switch(Ie.length){case 2:return Oe[0]!=="deg"&&Oe[0]!=="rad"&&Oe[1]!=="deg"&&Oe[1]!=="rad";case 1:return $i(Ie[0])||Oe[0]==="deg"||Oe[0]==="rad";default:return!1}}},easing:{regexes:["^(spring)\\s*\\(\\s*("+r+")\\s*,\\s*("+r+")\\s*\\)$","^(cubic-bezier)\\s*\\(\\s*("+r+")\\s*,\\s*("+r+")\\s*,\\s*("+r+")\\s*,\\s*("+r+")\\s*\\)$"],enums:["linear","ease","ease-in","ease-out","ease-in-out","ease-in-sine","ease-out-sine","ease-in-out-sine","ease-in-quad","ease-out-quad","ease-in-out-quad","ease-in-cubic","ease-out-cubic","ease-in-out-cubic","ease-in-quart","ease-out-quart","ease-in-out-quart","ease-in-quint","ease-out-quint","ease-in-out-quint","ease-in-expo","ease-out-expo","ease-in-out-expo","ease-in-circ","ease-out-circ","ease-in-out-circ"]},gradientDirection:{enums:["to-bottom","to-top","to-left","to-right","to-bottom-right","to-bottom-left","to-top-right","to-top-left","to-right-bottom","to-left-bottom","to-right-top","to-left-top"]},boundsExpansion:{number:!0,multiple:!0,min:0,validate:function(Ie){var Oe=Ie.length;return Oe===1||Oe===2||Oe===4}}};var y={zeroNonZero:function(Ie,Oe){return(Ie==null||Oe==null)&&Ie!==Oe||Ie==0&&Oe!=0?!0:Ie!=0&&Oe==0},any:function(Ie,Oe){return Ie!=Oe},emptyNonEmpty:function(Ie,Oe){var Le=y6(Ie),$e=y6(Oe);return Le&&!$e||!Le&&$e}},k=A1.types,E=[{name:"label",type:k.text,triggersBounds:y.any,triggersZOrder:y.emptyNonEmpty},{name:"text-rotation",type:k.textRotation,triggersBounds:y.any},{name:"text-margin-x",type:k.bidirectionalSize,triggersBounds:y.any},{name:"text-margin-y",type:k.bidirectionalSize,triggersBounds:y.any}],T=[{name:"source-label",type:k.text,triggersBounds:y.any},{name:"source-text-rotation",type:k.textRotation,triggersBounds:y.any},{name:"source-text-margin-x",type:k.bidirectionalSize,triggersBounds:y.any},{name:"source-text-margin-y",type:k.bidirectionalSize,triggersBounds:y.any},{name:"source-text-offset",type:k.size,triggersBounds:y.any}],C=[{name:"target-label",type:k.text,triggersBounds:y.any},{name:"target-text-rotation",type:k.textRotation,triggersBounds:y.any},{name:"target-text-margin-x",type:k.bidirectionalSize,triggersBounds:y.any},{name:"target-text-margin-y",type:k.bidirectionalSize,triggersBounds:y.any},{name:"target-text-offset",type:k.size,triggersBounds:y.any}],S=[{name:"font-family",type:k.fontFamily,triggersBounds:y.any},{name:"font-style",type:k.fontStyle,triggersBounds:y.any},{name:"font-weight",type:k.fontWeight,triggersBounds:y.any},{name:"font-size",type:k.size,triggersBounds:y.any},{name:"text-transform",type:k.textTransform,triggersBounds:y.any},{name:"text-wrap",type:k.textWrap,triggersBounds:y.any},{name:"text-overflow-wrap",type:k.textOverflowWrap,triggersBounds:y.any},{name:"text-max-width",type:k.size,triggersBounds:y.any},{name:"text-outline-width",type:k.size,triggersBounds:y.any},{name:"line-height",type:k.positiveNumber,triggersBounds:y.any}],L=[{name:"text-valign",type:k.valign,triggersBounds:y.any},{name:"text-halign",type:k.halign,triggersBounds:y.any},{name:"color",type:k.color},{name:"text-outline-color",type:k.color},{name:"text-outline-opacity",type:k.zeroOneNumber},{name:"text-background-color",type:k.color},{name:"text-background-opacity",type:k.zeroOneNumber},{name:"text-background-padding",type:k.size,triggersBounds:y.any},{name:"text-border-opacity",type:k.zeroOneNumber},{name:"text-border-color",type:k.color},{name:"text-border-width",type:k.size,triggersBounds:y.any},{name:"text-border-style",type:k.borderStyle,triggersBounds:y.any},{name:"text-background-shape",type:k.textBackgroundShape,triggersBounds:y.any},{name:"text-justification",type:k.justification}],O=[{name:"events",type:k.bool},{name:"text-events",type:k.bool}],B=[{name:"display",type:k.display,triggersZOrder:y.any,triggersBounds:y.any,triggersBoundsOfParallelBeziers:!0},{name:"visibility",type:k.visibility,triggersZOrder:y.any},{name:"opacity",type:k.zeroOneNumber,triggersZOrder:y.zeroNonZero},{name:"text-opacity",type:k.zeroOneNumber},{name:"min-zoomed-font-size",type:k.size},{name:"z-compound-depth",type:k.zCompoundDepth,triggersZOrder:y.any},{name:"z-index-compare",type:k.zIndexCompare,triggersZOrder:y.any},{name:"z-index",type:k.nonNegativeInt,triggersZOrder:y.any}],N=[{name:"overlay-padding",type:k.size,triggersBounds:y.any},{name:"overlay-color",type:k.color},{name:"overlay-opacity",type:k.zeroOneNumber,triggersBounds:y.zeroNonZero},{name:"overlay-shape",type:k.overlayShape,triggersBounds:y.any}],F=[{name:"underlay-padding",type:k.size,triggersBounds:y.any},{name:"underlay-color",type:k.color},{name:"underlay-opacity",type:k.zeroOneNumber,triggersBounds:y.zeroNonZero},{name:"underlay-shape",type:k.overlayShape,triggersBounds:y.any}],R=[{name:"transition-property",type:k.propList},{name:"transition-duration",type:k.time},{name:"transition-delay",type:k.time},{name:"transition-timing-function",type:k.easing}],q=function(Ie,Oe){return Oe.value==="label"?-Ie.poolIndex():Oe.pfValue},X=[{name:"height",type:k.nodeSize,triggersBounds:y.any,hashOverride:q},{name:"width",type:k.nodeSize,triggersBounds:y.any,hashOverride:q},{name:"shape",type:k.nodeShape,triggersBounds:y.any},{name:"shape-polygon-points",type:k.polygonPointList,triggersBounds:y.any},{name:"background-color",type:k.color},{name:"background-fill",type:k.fill},{name:"background-opacity",type:k.zeroOneNumber},{name:"background-blacken",type:k.nOneOneNumber},{name:"background-gradient-stop-colors",type:k.colors},{name:"background-gradient-stop-positions",type:k.percentages},{name:"background-gradient-direction",type:k.gradientDirection},{name:"padding",type:k.sizeMaybePercent,triggersBounds:y.any},{name:"padding-relative-to",type:k.paddingRelativeTo,triggersBounds:y.any},{name:"bounds-expansion",type:k.boundsExpansion,triggersBounds:y.any}],te=[{name:"border-color",type:k.color},{name:"border-opacity",type:k.zeroOneNumber},{name:"border-width",type:k.size,triggersBounds:y.any},{name:"border-style",type:k.borderStyle}],H=[{name:"background-image",type:k.urls},{name:"background-image-crossorigin",type:k.bgCrossOrigin},{name:"background-image-opacity",type:k.zeroOneNumbers},{name:"background-image-containment",type:k.bgContainment},{name:"background-image-smoothing",type:k.bools},{name:"background-position-x",type:k.bgPos},{name:"background-position-y",type:k.bgPos},{name:"background-width-relative-to",type:k.bgRelativeTo},{name:"background-height-relative-to",type:k.bgRelativeTo},{name:"background-repeat",type:k.bgRepeat},{name:"background-fit",type:k.bgFit},{name:"background-clip",type:k.bgClip},{name:"background-width",type:k.bgWH},{name:"background-height",type:k.bgWH},{name:"background-offset-x",type:k.bgPos},{name:"background-offset-y",type:k.bgPos}],Y=[{name:"position",type:k.position,triggersBounds:y.any},{name:"compound-sizing-wrt-labels",type:k.compoundIncludeLabels,triggersBounds:y.any},{name:"min-width",type:k.size,triggersBounds:y.any},{name:"min-width-bias-left",type:k.sizeMaybePercent,triggersBounds:y.any},{name:"min-width-bias-right",type:k.sizeMaybePercent,triggersBounds:y.any},{name:"min-height",type:k.size,triggersBounds:y.any},{name:"min-height-bias-top",type:k.sizeMaybePercent,triggersBounds:y.any},{name:"min-height-bias-bottom",type:k.sizeMaybePercent,triggersBounds:y.any}],z=[{name:"line-style",type:k.lineStyle},{name:"line-color",type:k.color},{name:"line-fill",type:k.fill},{name:"line-cap",type:k.lineCap},{name:"line-opacity",type:k.zeroOneNumber},{name:"line-dash-pattern",type:k.numbers},{name:"line-dash-offset",type:k.number},{name:"line-gradient-stop-colors",type:k.colors},{name:"line-gradient-stop-positions",type:k.percentages},{name:"curve-style",type:k.curveStyle,triggersBounds:y.any,triggersBoundsOfParallelBeziers:!0},{name:"haystack-radius",type:k.zeroOneNumber,triggersBounds:y.any},{name:"source-endpoint",type:k.edgeEndpoint,triggersBounds:y.any},{name:"target-endpoint",type:k.edgeEndpoint,triggersBounds:y.any},{name:"control-point-step-size",type:k.size,triggersBounds:y.any},{name:"control-point-distances",type:k.bidirectionalSizes,triggersBounds:y.any},{name:"control-point-weights",type:k.numbers,triggersBounds:y.any},{name:"segment-distances",type:k.bidirectionalSizes,triggersBounds:y.any},{name:"segment-weights",type:k.numbers,triggersBounds:y.any},{name:"taxi-turn",type:k.bidirectionalSizeMaybePercent,triggersBounds:y.any},{name:"taxi-turn-min-distance",type:k.size,triggersBounds:y.any},{name:"taxi-direction",type:k.axisDirection,triggersBounds:y.any},{name:"edge-distances",type:k.edgeDistances,triggersBounds:y.any},{name:"arrow-scale",type:k.positiveNumber,triggersBounds:y.any},{name:"loop-direction",type:k.angle,triggersBounds:y.any},{name:"loop-sweep",type:k.angle,triggersBounds:y.any},{name:"source-distance-from-node",type:k.size,triggersBounds:y.any},{name:"target-distance-from-node",type:k.size,triggersBounds:y.any}],W=[{name:"ghost",type:k.bool,triggersBounds:y.any},{name:"ghost-offset-x",type:k.bidirectionalSize,triggersBounds:y.any},{name:"ghost-offset-y",type:k.bidirectionalSize,triggersBounds:y.any},{name:"ghost-opacity",type:k.zeroOneNumber}],Z=[{name:"selection-box-color",type:k.color},{name:"selection-box-opacity",type:k.zeroOneNumber},{name:"selection-box-border-color",type:k.color},{name:"selection-box-border-width",type:k.size},{name:"active-bg-color",type:k.color},{name:"active-bg-opacity",type:k.zeroOneNumber},{name:"active-bg-size",type:k.size},{name:"outside-texture-bg-color",type:k.color},{name:"outside-texture-bg-opacity",type:k.zeroOneNumber}],G=[];A1.pieBackgroundN=16,G.push({name:"pie-size",type:k.sizeMaybePercent});for(var ae=1;ae<=A1.pieBackgroundN;ae++)G.push({name:"pie-"+ae+"-background-color",type:k.color}),G.push({name:"pie-"+ae+"-background-size",type:k.percent}),G.push({name:"pie-"+ae+"-background-opacity",type:k.zeroOneNumber});var $=[],ge=A1.arrowPrefixes=["source","mid-source","target","mid-target"];[{name:"arrow-shape",type:k.arrowShape,triggersBounds:y.any},{name:"arrow-color",type:k.color},{name:"arrow-fill",type:k.arrowFill}].forEach(function(Ze){ge.forEach(function(Ie){var Oe=Ie+"-"+Ze.name,Le=Ze.type,$e=Ze.triggersBounds;$.push({name:Oe,type:Le,triggersBounds:$e})})},{});var ee=A1.properties=[].concat(O,R,B,N,F,W,L,S,E,T,C,X,te,H,G,Y,z,$,Z),de=A1.propertyGroups={behavior:O,transition:R,visibility:B,overlay:N,underlay:F,ghost:W,commonLabel:L,labelDimensions:S,mainLabel:E,sourceLabel:T,targetLabel:C,nodeBody:X,nodeBorder:te,backgroundImage:H,pie:G,compound:Y,edgeLine:z,edgeArrow:$,core:Z},re=A1.propertyGroupNames={},ke=A1.propertyGroupKeys=Object.keys(de);ke.forEach(function(Ze){re[Ze]=de[Ze].map(function(Ie){return Ie.name}),de[Ze].forEach(function(Ie){return Ie.groupKey=Ze})});var Ce=A1.aliases=[{name:"content",pointsTo:"label"},{name:"control-point-distance",pointsTo:"control-point-distances"},{name:"control-point-weight",pointsTo:"control-point-weights"},{name:"edge-text-rotation",pointsTo:"text-rotation"},{name:"padding-left",pointsTo:"padding"},{name:"padding-right",pointsTo:"padding"},{name:"padding-top",pointsTo:"padding"},{name:"padding-bottom",pointsTo:"padding"}];A1.propertyNames=ee.map(function(Ze){return Ze.name});for(var _e=0;_e<ee.length;_e++){var Te=ee[_e];ee[Te.name]=Te}for(var Be=0;Be<Ce.length;Be++){var Ge=Ce[Be],Xe=ee[Ge.pointsTo],Ee={name:Ge.name,alias:!0,pointsTo:Xe};ee.push(Ee),ee[Ge.name]=Ee}})(),A1.getDefaultProperty=function(r){return this.getDefaultProperties()[r]},A1.getDefaultProperties=function(){var r=this._private;if(r.defaultProperties!=null)return r.defaultProperties;for(var i=bs({"selection-box-color":"#ddd","selection-box-opacity":.65,"selection-box-border-color":"#aaa","selection-box-border-width":1,"active-bg-color":"black","active-bg-opacity":.15,"active-bg-size":30,"outside-texture-bg-color":"#000","outside-texture-bg-opacity":.125,events:"yes","text-events":"no","text-valign":"top","text-halign":"center","text-justification":"auto","line-height":1,color:"#000","text-outline-color":"#000","text-outline-width":0,"text-outline-opacity":1,"text-opacity":1,"text-decoration":"none","text-transform":"none","text-wrap":"none","text-overflow-wrap":"whitespace","text-max-width":9999,"text-background-color":"#000","text-background-opacity":0,"text-background-shape":"rectangle","text-background-padding":0,"text-border-opacity":0,"text-border-width":0,"text-border-style":"solid","text-border-color":"#000","font-family":"Helvetica Neue, Helvetica, sans-serif","font-style":"normal","font-weight":"normal","font-size":16,"min-zoomed-font-size":0,"text-rotation":"none","source-text-rotation":"none","target-text-rotation":"none",visibility:"visible",display:"element",opacity:1,"z-compound-depth":"auto","z-index-compare":"auto","z-index":0,label:"","text-margin-x":0,"text-margin-y":0,"source-label":"","source-text-offset":0,"source-text-margin-x":0,"source-text-margin-y":0,"target-label":"","target-text-offset":0,"target-text-margin-x":0,"target-text-margin-y":0,"overlay-opacity":0,"overlay-color":"#000","overlay-padding":10,"overlay-shape":"round-rectangle","underlay-opacity":0,"underlay-color":"#000","underlay-padding":10,"underlay-shape":"round-rectangle","transition-property":"none","transition-duration":0,"transition-delay":0,"transition-timing-function":"linear","background-blacken":0,"background-color":"#999","background-fill":"solid","background-opacity":1,"background-image":"none","background-image-crossorigin":"anonymous","background-image-opacity":1,"background-image-containment":"inside","background-image-smoothing":"yes","background-position-x":"50%","background-position-y":"50%","background-offset-x":0,"background-offset-y":0,"background-width-relative-to":"include-padding","background-height-relative-to":"include-padding","background-repeat":"no-repeat","background-fit":"none","background-clip":"node","background-width":"auto","background-height":"auto","border-color":"#000","border-opacity":1,"border-width":0,"border-style":"solid",height:30,width:30,shape:"ellipse","shape-polygon-points":"-1, -1,   1, -1,   1, 1,   -1, 1","bounds-expansion":0,"background-gradient-direction":"to-bottom","background-gradient-stop-colors":"#999","background-gradient-stop-positions":"0%",ghost:"no","ghost-offset-y":0,"ghost-offset-x":0,"ghost-opacity":0,padding:0,"padding-relative-to":"width",position:"origin","compound-sizing-wrt-labels":"include","min-width":0,"min-width-bias-left":0,"min-width-bias-right":0,"min-height":0,"min-height-bias-top":0,"min-height-bias-bottom":0},{"pie-size":"100%"},[{name:"pie-{{i}}-background-color",value:"black"},{name:"pie-{{i}}-background-size",value:"0%"},{name:"pie-{{i}}-background-opacity",value:1}].reduce(function(y,k){for(var E=1;E<=A1.pieBackgroundN;E++){var T=k.name.replace("{{i}}",E),C=k.value;y[T]=C}return y},{}),{"line-style":"solid","line-color":"#999","line-fill":"solid","line-cap":"butt","line-opacity":1,"line-gradient-stop-colors":"#999","line-gradient-stop-positions":"0%","control-point-step-size":40,"control-point-weights":.5,"segment-weights":.5,"segment-distances":20,"taxi-turn":"50%","taxi-turn-min-distance":10,"taxi-direction":"auto","edge-distances":"intersection","curve-style":"haystack","haystack-radius":0,"arrow-scale":1,"loop-direction":"-45deg","loop-sweep":"-90deg","source-distance-from-node":0,"target-distance-from-node":0,"source-endpoint":"outside-to-node","target-endpoint":"outside-to-node","line-dash-pattern":[6,3],"line-dash-offset":0},[{name:"arrow-shape",value:"none"},{name:"arrow-color",value:"#999"},{name:"arrow-fill",value:"filled"}].reduce(function(y,k){return A1.arrowPrefixes.forEach(function(E){var T=E+"-"+k.name,C=k.value;y[T]=C}),y},{})),o={},l=0;l<this.properties.length;l++){var f=this.properties[l];if(!f.pointsTo){var b=f.name,d=i[b],w=this.parse(b,d);o[b]=w}}return r.defaultProperties=o,r.defaultProperties},A1.addDefaultStylesheet=function(){this.selector(":parent").css({shape:"rectangle",padding:10,"background-color":"#eee","border-color":"#ccc","border-width":1}).selector("edge").css({width:3}).selector(":loop").css({"curve-style":"bezier"}).selector("edge:compound").css({"curve-style":"bezier","source-endpoint":"outside-to-line","target-endpoint":"outside-to-line"}).selector(":selected").css({"background-color":"#0169D9","line-color":"#0169D9","source-arrow-color":"#0169D9","target-arrow-color":"#0169D9","mid-source-arrow-color":"#0169D9","mid-target-arrow-color":"#0169D9"}).selector(":parent:selected").css({"background-color":"#CCE1F9","border-color":"#aec8e5"}).selector(":active").css({"overlay-color":"black","overlay-padding":10,"overlay-opacity":.25}),this.defaultLength=this.length};var YU={};YU.parse=function(r,i,o,l){var f=this;if(al(i))return f.parseImplWarn(r,i,o,l);var b=l==="mapping"||l===!0||l===!1||l==null?"dontcare":l,d=o?"t":"f",w=""+i,y=sNe(r,w,d,b),k=f.propCache=f.propCache||[],E;return(E=k[y])||(E=k[y]=f.parseImplWarn(r,i,o,l)),(o||l==="mapping")&&(E=qm(E),E&&(E.value=qm(E.value))),E},YU.parseImplWarn=function(r,i,o,l){var f=this.parseImpl(r,i,o,l);return!f&&i!=null&&tc("The style property `".concat(r,": ").concat(i,"` is invalid")),f&&(f.name==="width"||f.name==="height")&&i==="label"&&tc("The style value of `label` is deprecated for `"+f.name+"`"),f},YU.parseImpl=function(r,i,o,l){var f=this;r=Zce(r);var b=f.properties[r],d=i,w=f.types;if(!b||i===void 0)return null;b.alias&&(b=b.pointsTo,r=b.name);var y=$i(i);y&&(i=i.trim());var k=b.type;if(!k)return null;if(o&&(i===""||i===null))return{name:r,value:i,bypass:!0,deleteBypass:!0};if(al(i))return{name:r,value:i,strValue:"fn",mapped:w.fn,bypass:o};var E,T;if(!(!y||l||i.length<7||i[1]!=="a")){if(i.length>=7&&i[0]==="d"&&(E=new RegExp(w.data.regex).exec(i))){if(o)return!1;var C=w.data;return{name:r,value:E,strValue:""+i,mapped:C,field:E[1],bypass:o}}else if(i.length>=10&&i[0]==="m"&&(T=new RegExp(w.mapData.regex).exec(i))){if(o||k.multiple)return!1;var S=w.mapData;if(!(k.color||k.number))return!1;var L=this.parse(r,T[4]);if(!L||L.mapped)return!1;var O=this.parse(r,T[5]);if(!O||O.mapped)return!1;if(L.pfValue===O.pfValue||L.strValue===O.strValue)return tc("`"+r+": "+i+"` is not a valid mapper because the output range is zero; converting to `"+r+": "+L.strValue+"`"),this.parse(r,L.strValue);if(k.color){var B=L.value,N=O.value,F=B[0]===N[0]&&B[1]===N[1]&&B[2]===N[2]&&(B[3]===N[3]||(B[3]==null||B[3]===1)&&(N[3]==null||N[3]===1));if(F)return!1}return{name:r,value:T,strValue:""+i,mapped:S,field:T[1],fieldMin:parseFloat(T[2]),fieldMax:parseFloat(T[3]),valueMin:L.value,valueMax:O.value,bypass:o}}}if(k.multiple&&l!=="multiple"){var R;if(y?R=i.split(/\s+/):qc(i)?R=i:R=[i],k.evenMultiple&&R.length%2!==0)return null;for(var q=[],X=[],te=[],H="",Y=!1,z=0;z<R.length;z++){var W=f.parse(r,R[z],o,"multiple");Y=Y||$i(W.value),q.push(W.value),te.push(W.pfValue!=null?W.pfValue:W.value),X.push(W.units),H+=(z>0?" ":"")+W.strValue}return k.validate&&!k.validate(q,X)?null:k.singleEnum&&Y?q.length===1&&$i(q[0])?{name:r,value:q[0],strValue:q[0],bypass:o}:null:{name:r,value:q,pfValue:te,strValue:H,bypass:o,units:X}}var Z=function(){for(var Oe=0;Oe<k.enums.length;Oe++){var Le=k.enums[Oe];if(Le===i)return{name:r,value:i,strValue:""+i,bypass:o}}return null};if(k.number){var G,ae="px";if(k.units&&(G=k.units),k.implicitUnits&&(ae=k.implicitUnits),!k.unitless)if(y){var $="px|em"+(k.allowPercent?"|\\%":"");G&&($=G);var ge=i.match("^("+gf+")("+$+")?$");ge&&(i=ge[1],G=ge[2]||ae)}else(!G||k.implicitUnits)&&(G=ae);if(i=parseFloat(i),isNaN(i)&&k.enums===void 0)return null;if(isNaN(i)&&k.enums!==void 0)return i=d,Z();if(k.integer&&!Van(i)||k.min!==void 0&&(i<k.min||k.strictMin&&i===k.min)||k.max!==void 0&&(i>k.max||k.strictMax&&i===k.max))return null;var ee={name:r,value:i,strValue:""+i+(G||""),units:G,bypass:o};return k.unitless||G!=="px"&&G!=="em"?ee.pfValue=i:ee.pfValue=G==="px"||!G?i:this.getEmSizeInPixels()*i,(G==="ms"||G==="s")&&(ee.pfValue=G==="ms"?i:1e3*i),(G==="deg"||G==="rad")&&(ee.pfValue=G==="rad"?i:jon(i)),G==="%"&&(ee.pfValue=i/100),ee}else if(k.propList){var de=[],re=""+i;if(re!=="none"){for(var ke=re.split(/\s*,\s*|\s+/),Ce=0;Ce<ke.length;Ce++){var _e=ke[Ce].trim();f.properties[_e]?de.push(_e):tc("`"+_e+"` is not a valid property name")}if(de.length===0)return null}return{name:r,value:de,strValue:de.length===0?"none":de.join(" "),bypass:o}}else if(k.color){var Te=oon(i);return Te?{name:r,value:Te,pfValue:Te,strValue:"rgb("+Te[0]+","+Te[1]+","+Te[2]+")",bypass:o}:null}else if(k.regex||k.regexes){if(k.enums){var Be=Z();if(Be)return Be}for(var Ge=k.regexes?k.regexes:[k.regex],Xe=0;Xe<Ge.length;Xe++){var Ee=new RegExp(Ge[Xe]),Ze=Ee.exec(i);if(Ze)return{name:r,value:k.singleRegexMatchValue?Ze[1]:Ze,strValue:""+i,bypass:o}}return null}else return k.string?{name:r,value:""+i,strValue:""+i,bypass:o}:k.enums?Z():null};var gd=function r(i){if(!(this instanceof r))return new r(i);if(!Qce(i)){Ml("A style must have a core reference");return}this._private={cy:i,coreStyle:{}},this.length=0,this.resetToDefault()},pd=gd.prototype;pd.instanceString=function(){return"style"},pd.clear=function(){for(var r=this._private,i=r.cy,o=i.elements(),l=0;l<this.length;l++)this[l]=void 0;return this.length=0,r.contextStyles={},r.propDiffs={},this.cleanElements(o,!0),o.forEach(function(f){var b=f[0]._private;b.styleDirty=!0,b.appliedInitStyle=!1}),this},pd.resetToDefault=function(){return this.clear(),this.addDefaultStylesheet(),this},pd.core=function(r){return this._private.coreStyle[r]||this.getDefaultProperty(r)},pd.selector=function(r){var i=r==="core"?null:new C6(r),o=this.length++;return this[o]={selector:i,properties:[],mappedProperties:[],index:o},this},pd.css=function(){var r=this,i=arguments;if(i.length===1)for(var o=i[0],l=0;l<r.properties.length;l++){var f=r.properties[l],b=o[f.name];b===void 0&&(b=o[wU(f.name)]),b!==void 0&&this.cssRule(f.name,b)}else i.length===2&&this.cssRule(i[0],i[1]);return this},pd.style=pd.css,pd.cssRule=function(r,i){var o=this.parse(r,i);if(o){var l=this.length-1;this[l].properties.push(o),this[l].properties[o.name]=o,o.name.match(/pie-(\d+)-background-size/)&&o.value&&(this._private.hasPie=!0),o.mapped&&this[l].mappedProperties.push(o);var f=!this[l].selector;f&&(this._private.coreStyle[o.name]=o)}return this},pd.append=function(r){return XOe(r)?r.appendToStyle(this):qc(r)?this.appendFromJson(r):$i(r)&&this.appendFromString(r),this},gd.fromJson=function(r,i){var o=new gd(r);return o.fromJson(i),o},gd.fromString=function(r,i){return new gd(r).fromString(i)},[dd,LO,Tue,Wm,qU,Cue,A1,YU].forEach(function(r){bs(pd,r)}),gd.types=pd.types,gd.properties=pd.properties,gd.propertyGroups=pd.propertyGroups,gd.propertyGroupNames=pd.propertyGroupNames,gd.propertyGroupKeys=pd.propertyGroupKeys;var Qun={style:function(i){if(i){var o=this.setStyle(i);o.update()}return this._private.style},setStyle:function(i){var o=this._private;return XOe(i)?o.style=i.generateStyle(this):qc(i)?o.style=gd.fromJson(this,i):$i(i)?o.style=gd.fromString(this,i):o.style=gd(this),o.style},updateStyle:function(){this.mutableElements().updateStyle()}},Zun="single",_7={autolock:function(i){if(i!==void 0)this._private.autolock=!!i;else return this._private.autolock;return this},autoungrabify:function(i){if(i!==void 0)this._private.autoungrabify=!!i;else return this._private.autoungrabify;return this},autounselectify:function(i){if(i!==void 0)this._private.autounselectify=!!i;else return this._private.autounselectify;return this},selectionType:function(i){var o=this._private;if(o.selectionType==null&&(o.selectionType=Zun),i!==void 0)(i==="additive"||i==="single")&&(o.selectionType=i);else return o.selectionType;return this},panningEnabled:function(i){if(i!==void 0)this._private.panningEnabled=!!i;else return this._private.panningEnabled;return this},userPanningEnabled:function(i){if(i!==void 0)this._private.userPanningEnabled=!!i;else return this._private.userPanningEnabled;return this},zoomingEnabled:function(i){if(i!==void 0)this._private.zoomingEnabled=!!i;else return this._private.zoomingEnabled;return this},userZoomingEnabled:function(i){if(i!==void 0)this._private.userZoomingEnabled=!!i;else return this._private.userZoomingEnabled;return this},boxSelectionEnabled:function(i){if(i!==void 0)this._private.boxSelectionEnabled=!!i;else return this._private.boxSelectionEnabled;return this},pan:function(){var i=arguments,o=this._private.pan,l,f,b,d,w;switch(i.length){case 0:return o;case 1:if($i(i[0]))return l=i[0],o[l];if(_o(i[0])){if(!this._private.panningEnabled)return this;b=i[0],d=b.x,w=b.y,Vr(d)&&(o.x=d),Vr(w)&&(o.y=w),this.emit("pan viewport")}break;case 2:if(!this._private.panningEnabled)return this;l=i[0],f=i[1],(l==="x"||l==="y")&&Vr(f)&&(o[l]=f),this.emit("pan viewport");break}return this.notify("viewport"),this},panBy:function(i,o){var l=arguments,f=this._private.pan,b,d,w,y,k;if(!this._private.panningEnabled)return this;switch(l.length){case 1:_o(i)&&(w=l[0],y=w.x,k=w.y,Vr(y)&&(f.x+=y),Vr(k)&&(f.y+=k),this.emit("pan viewport"));break;case 2:b=i,d=o,(b==="x"||b==="y")&&Vr(d)&&(f[b]+=d),this.emit("pan viewport");break}return this.notify("viewport"),this},fit:function(i,o){var l=this.getFitViewport(i,o);if(l){var f=this._private;f.zoom=l.zoom,f.pan=l.pan,this.emit("pan zoom viewport"),this.notify("viewport")}return this},getFitViewport:function(i,o){if(Vr(i)&&o===void 0&&(o=i,i=void 0),!(!this._private.panningEnabled||!this._private.zoomingEnabled)){var l;if($i(i)){var f=i;i=this.$(f)}else if(Yan(i)){var b=i;l={x1:b.x1,y1:b.y1,x2:b.x2,y2:b.y2},l.w=l.x2-l.x1,l.h=l.y2-l.y1}else vp(i)||(i=this.mutableElements());if(!(vp(i)&&i.empty())){l=l||i.boundingBox();var d=this.width(),w=this.height(),y;if(o=Vr(o)?o:0,!isNaN(d)&&!isNaN(w)&&d>0&&w>0&&!isNaN(l.w)&&!isNaN(l.h)&&l.w>0&&l.h>0){y=Math.min((d-2*o)/l.w,(w-2*o)/l.h),y=y>this._private.maxZoom?this._private.maxZoom:y,y=y<this._private.minZoom?this._private.minZoom:y;var k={x:(d-y*(l.x1+l.x2))/2,y:(w-y*(l.y1+l.y2))/2};return{zoom:y,pan:k}}}}},zoomRange:function(i,o){var l=this._private;if(o==null){var f=i;i=f.min,o=f.max}return Vr(i)&&Vr(o)&&i<=o?(l.minZoom=i,l.maxZoom=o):Vr(i)&&o===void 0&&i<=l.maxZoom?l.minZoom=i:Vr(o)&&i===void 0&&o>=l.minZoom&&(l.maxZoom=o),this},minZoom:function(i){return i===void 0?this._private.minZoom:this.zoomRange({min:i})},maxZoom:function(i){return i===void 0?this._private.maxZoom:this.zoomRange({max:i})},getZoomedViewport:function(i){var o=this._private,l=o.pan,f=o.zoom,b,d,w=!1;if(o.zoomingEnabled||(w=!0),Vr(i)?d=i:_o(i)&&(d=i.level,i.position!=null?b=EU(i.position,f,l):i.renderedPosition!=null&&(b=i.renderedPosition),b!=null&&!o.panningEnabled&&(w=!0)),d=d>o.maxZoom?o.maxZoom:d,d=d<o.minZoom?o.minZoom:d,w||!Vr(d)||d===f||b!=null&&(!Vr(b.x)||!Vr(b.y)))return null;if(b!=null){var y=l,k=f,E=d,T={x:-E/k*(b.x-y.x)+b.x,y:-E/k*(b.y-y.y)+b.y};return{zoomed:!0,panned:!0,zoom:E,pan:T}}else return{zoomed:!0,panned:!1,zoom:d,pan:l}},zoom:function(i){if(i===void 0)return this._private.zoom;var o=this.getZoomedViewport(i),l=this._private;return o==null||!o.zoomed?this:(l.zoom=o.zoom,o.panned&&(l.pan.x=o.pan.x,l.pan.y=o.pan.y),this.emit("zoom"+(o.panned?" pan":"")+" viewport"),this.notify("viewport"),this)},viewport:function(i){var o=this._private,l=!0,f=!0,b=[],d=!1,w=!1;if(!i)return this;if(Vr(i.zoom)||(l=!1),_o(i.pan)||(f=!1),!l&&!f)return this;if(l){var y=i.zoom;y<o.minZoom||y>o.maxZoom||!o.zoomingEnabled?d=!0:(o.zoom=y,b.push("zoom"))}if(f&&(!d||!i.cancelOnFailedZoom)&&o.panningEnabled){var k=i.pan;Vr(k.x)&&(o.pan.x=k.x,w=!1),Vr(k.y)&&(o.pan.y=k.y,w=!1),w||b.push("pan")}return b.length>0&&(b.push("viewport"),this.emit(b.join(" ")),this.notify("viewport")),this},center:function(i){var o=this.getCenterPan(i);return o&&(this._private.pan=o,this.emit("pan viewport"),this.notify("viewport")),this},getCenterPan:function(i,o){if(!!this._private.panningEnabled){if($i(i)){var l=i;i=this.mutableElements().filter(l)}else vp(i)||(i=this.mutableElements());if(i.length!==0){var f=i.boundingBox(),b=this.width(),d=this.height();o=o===void 0?this._private.zoom:o;var w={x:(b-o*(f.x1+f.x2))/2,y:(d-o*(f.y1+f.y2))/2};return w}}},reset:function(){return!this._private.panningEnabled||!this._private.zoomingEnabled?this:(this.viewport({pan:{x:0,y:0},zoom:1}),this)},invalidateSize:function(){this._private.sizeCache=null},size:function(){var i=this._private,o=i.container;return i.sizeCache=i.sizeCache||(o?function(){var l=Fu.getComputedStyle(o),f=function(d){return parseFloat(l.getPropertyValue(d))};return{width:o.clientWidth-f("padding-left")-f("padding-right"),height:o.clientHeight-f("padding-top")-f("padding-bottom")}}():{width:1,height:1})},width:function(){return this.size().width},height:function(){return this.size().height},extent:function(){var i=this._private.pan,o=this._private.zoom,l=this.renderedExtent(),f={x1:(l.x1-i.x)/o,x2:(l.x2-i.x)/o,y1:(l.y1-i.y)/o,y2:(l.y2-i.y)/o};return f.w=f.x2-f.x1,f.h=f.y2-f.y1,f},renderedExtent:function(){var i=this.width(),o=this.height();return{x1:0,y1:0,x2:i,y2:o,w:i,h:o}},multiClickDebounceTime:function(i){if(i)this._private.multiClickDebounceTime=i;else return this._private.multiClickDebounceTime;return this}};_7.centre=_7.center,_7.autolockNodes=_7.autolock,_7.autoungrabifyNodes=_7.autoungrabify;var IO={data:nc.data({field:"data",bindingEvent:"data",allowBinding:!0,allowSetting:!0,settingEvent:"data",settingTriggersEvent:!0,triggerFnName:"trigger",allowGetting:!0,updateStyle:!0}),removeData:nc.removeData({field:"data",event:"data",triggerFnName:"trigger",triggerEvent:!0,updateStyle:!0}),scratch:nc.data({field:"scratch",bindingEvent:"scratch",allowBinding:!0,allowSetting:!0,settingEvent:"scratch",settingTriggersEvent:!0,triggerFnName:"trigger",allowGetting:!0,updateStyle:!0}),removeScratch:nc.removeData({field:"scratch",event:"scratch",triggerFnName:"trigger",triggerEvent:!0,updateStyle:!0})};IO.attr=IO.data,IO.removeAttr=IO.removeData;var OO=function(i){var o=this;i=bs({},i);var l=i.container;l&&!vU(l)&&vU(l[0])&&(l=l[0]);var f=l?l._cyreg:null;f=f||{},f&&f.cy&&(f.cy.destroy(),f={});var b=f.readies=f.readies||[];l&&(l._cyreg=f),f.cy=o;var d=Fu!==void 0&&l!==void 0&&!i.headless,w=i;w.layout=bs({name:d?"grid":"null"},w.layout),w.renderer=bs({name:d?"canvas":"null"},w.renderer);var y=function(L,O,B){return O!==void 0?O:B!==void 0?B:L},k=this._private={container:l,ready:!1,options:w,elements:new S1(this),listeners:[],aniEles:new S1(this),data:w.data||{},scratch:{},layout:null,renderer:null,destroyed:!1,notificationsEnabled:!0,minZoom:1e-50,maxZoom:1e50,zoomingEnabled:y(!0,w.zoomingEnabled),userZoomingEnabled:y(!0,w.userZoomingEnabled),panningEnabled:y(!0,w.panningEnabled),userPanningEnabled:y(!0,w.userPanningEnabled),boxSelectionEnabled:y(!0,w.boxSelectionEnabled),autolock:y(!1,w.autolock,w.autolockNodes),autoungrabify:y(!1,w.autoungrabify,w.autoungrabifyNodes),autounselectify:y(!1,w.autounselectify),styleEnabled:w.styleEnabled===void 0?d:w.styleEnabled,zoom:Vr(w.zoom)?w.zoom:1,pan:{x:_o(w.pan)&&Vr(w.pan.x)?w.pan.x:0,y:_o(w.pan)&&Vr(w.pan.y)?w.pan.y:0},animation:{current:[],queue:[]},hasCompoundNodes:!1,multiClickDebounceTime:y(250,w.multiClickDebounceTime)};this.createEmitter(),this.selectionType(w.selectionType),this.zoomRange({min:w.minZoom,max:w.maxZoom});var E=function(L,O){var B=L.some(Wan);if(B)return HC.all(L).then(O);O(L)};k.styleEnabled&&o.setStyle([]);var T=bs({},w,w.renderer);o.initRenderer(T);var C=function(L,O,B){o.notifications(!1);var N=o.mutableElements();N.length>0&&N.remove(),L!=null&&(_o(L)||qc(L))&&o.add(L),o.one("layoutready",function(R){o.notifications(!0),o.emit(R),o.one("load",O),o.emitAndNotify("load")}).one("layoutstop",function(){o.one("done",B),o.emit("done")});var F=bs({},o._private.options.layout);F.eles=o.elements(),o.layout(F).run()};E([w.style,w.elements],function(S){var L=S[0],O=S[1];k.styleEnabled&&o.style().append(L),C(O,function(){o.startAnimationLoop(),k.ready=!0,al(w.ready)&&o.on("ready",w.ready);for(var B=0;B<b.length;B++){var N=b[B];o.on("ready",N)}f&&(f.readies=[]),o.emit("ready")},w.done)})},WU=OO.prototype;bs(WU,{instanceString:function(){return"core"},isReady:function(){return this._private.ready},destroyed:function(){return this._private.destroyed},ready:function(i){return this.isReady()?this.emitter().emit("ready",[],i):this.on("ready",i),this},destroy:function(){var i=this;if(!i.destroyed())return i.stopAnimationLoop(),i.destroyRenderer(),this.emit("destroy"),i._private.destroyed=!0,i},hasElementWithId:function(i){return this._private.elements.hasElementWithId(i)},getElementById:function(i){return this._private.elements.getElementById(i)},hasCompoundNodes:function(){return this._private.hasCompoundNodes},headless:function(){return this._private.renderer.isHeadless()},styleEnabled:function(){return this._private.styleEnabled},addToPool:function(i){return this._private.elements.merge(i),this},removeFromPool:function(i){return this._private.elements.unmerge(i),this},container:function(){return this._private.container||null},mount:function(i){if(i!=null){var o=this,l=o._private,f=l.options;return!vU(i)&&vU(i[0])&&(i=i[0]),o.stopAnimationLoop(),o.destroyRenderer(),l.container=i,l.styleEnabled=!0,o.invalidateSize(),o.initRenderer(bs({},f,f.renderer,{name:f.renderer.name==="null"?"canvas":f.renderer.name})),o.startAnimationLoop(),o.style(f.style),o.emit("mount"),o}},unmount:function(){var i=this;return i.stopAnimationLoop(),i.destroyRenderer(),i.initRenderer({name:"null"}),i.emit("unmount"),i},options:function(){return qm(this._private.options)},json:function(i){var o=this,l=o._private,f=o.mutableElements(),b=function(q){return o.getElementById(q.id())};if(_o(i)){if(o.startBatch(),i.elements){var d={},w=function(q,X){for(var te=[],H=[],Y=0;Y<q.length;Y++){var z=q[Y];if(!z.data.id){tc("cy.json() cannot handle elements without an ID attribute");continue}var W=""+z.data.id,Z=o.getElementById(W);d[W]=!0,Z.length!==0?H.push({ele:Z,json:z}):(X&&(z.group=X),te.push(z))}o.add(te);for(var G=0;G<H.length;G++){var ae=H[G],$=ae.ele,ge=ae.json;$.json(ge)}};if(qc(i.elements))w(i.elements);else for(var y=["nodes","edges"],k=0;k<y.length;k++){var E=y[k],T=i.elements[E];qc(T)&&w(T,E)}var C=o.collection();f.filter(function(R){return!d[R.id()]}).forEach(function(R){R.isParent()?C.merge(R):R.remove()}),C.forEach(function(R){return R.children().move({parent:null})}),C.forEach(function(R){return b(R).remove()})}i.style&&o.style(i.style),i.zoom!=null&&i.zoom!==l.zoom&&o.zoom(i.zoom),i.pan&&(i.pan.x!==l.pan.x||i.pan.y!==l.pan.y)&&o.pan(i.pan),i.data&&o.data(i.data);for(var S=["minZoom","maxZoom","zoomingEnabled","userZoomingEnabled","panningEnabled","userPanningEnabled","boxSelectionEnabled","autolock","autoungrabify","autounselectify","multiClickDebounceTime"],L=0;L<S.length;L++){var O=S[L];i[O]!=null&&o[O](i[O])}return o.endBatch(),this}else{var B=!!i,N={};B?N.elements=this.elements().map(function(R){return R.json()}):(N.elements={},f.forEach(function(R){var q=R.group();N.elements[q]||(N.elements[q]=[]),N.elements[q].push(R.json())})),this._private.styleEnabled&&(N.style=o.style().json()),N.data=qm(o.data());var F=l.options;return N.zoomingEnabled=l.zoomingEnabled,N.userZoomingEnabled=l.userZoomingEnabled,N.zoom=l.zoom,N.minZoom=l.minZoom,N.maxZoom=l.maxZoom,N.panningEnabled=l.panningEnabled,N.userPanningEnabled=l.userPanningEnabled,N.pan=qm(l.pan),N.boxSelectionEnabled=l.boxSelectionEnabled,N.renderer=qm(F.renderer),N.hideEdgesOnViewport=F.hideEdgesOnViewport,N.textureOnViewport=F.textureOnViewport,N.wheelSensitivity=F.wheelSensitivity,N.motionBlur=F.motionBlur,N.multiClickDebounceTime=F.multiClickDebounceTime,N}}}),WU.$id=WU.getElementById,[Hun,qun,CPe,Eue,VU,Wun,_ue,UU,Qun,_7,IO].forEach(function(r){bs(WU,r)});var Jun={fit:!0,directed:!1,padding:30,circle:!1,grid:!1,spacingFactor:1.75,boundingBox:void 0,avoidOverlap:!0,nodeDimensionsIncludeLabels:!1,roots:void 0,maximal:!1,depthSort:void 0,animate:!1,animationDuration:500,animationEasing:void 0,animateFilter:function(i,o){return!0},ready:void 0,stop:void 0,transform:function(i,o){return o}},KC=function(i){return i.scratch("breadthfirst")},SPe=function(i,o){return i.scratch("breadthfirst",o)};function APe(r){this.options=bs({},Jun,r)}APe.prototype.run=function(){var r=this.options,i=r,o=r.cy,l=i.eles,f=l.nodes().filter(function(je){return!je.isParent()}),b=l,d=i.directed,w=i.maximal||i.maximalAdjustments>0,y=J0(i.boundingBox?i.boundingBox:{x1:0,y1:0,w:o.width(),h:o.height()}),k;if(vp(i.roots))k=i.roots;else if(qc(i.roots)){for(var E=[],T=0;T<i.roots.length;T++){var C=i.roots[T],S=o.getElementById(C);E.push(S)}k=o.collection(E)}else if($i(i.roots))k=o.$(i.roots);else if(d)k=f.roots();else{var L=l.components();k=o.collection();for(var O=function(me){var vt=L[me],ve=vt.maxDegree(!1),Zt=vt.filter(function(nt){return nt.degree(!1)===ve});k=k.add(Zt)},B=0;B<L.length;B++)O(B)}var N=[],F={},R=function(me,vt){N[vt]==null&&(N[vt]=[]);var ve=N[vt].length;N[vt].push(me),SPe(me,{index:ve,depth:vt})},q=function(me,vt){var ve=KC(me),Zt=ve.depth,nt=ve.index;N[Zt][nt]=null,R(me,vt)};b.bfs({roots:k,directed:i.directed,visit:function(me,vt,ve,Zt,nt){var xn=me[0],cn=xn.id();R(xn,nt),F[cn]=!0}});for(var X=[],te=0;te<f.length;te++){var H=f[te];F[H.id()]||X.push(H)}var Y=function(me){for(var vt=N[me],ve=0;ve<vt.length;ve++){var Zt=vt[ve];if(Zt==null){vt.splice(ve,1),ve--;continue}SPe(Zt,{depth:me,index:ve})}},z=function(){for(var me=0;me<N.length;me++)Y(me)},W=function(me,vt){for(var ve=KC(me),Zt=me.incomers().filter(function(be){return be.isNode()&&l.has(be)}),nt=-1,xn=me.id(),cn=0;cn<Zt.length;cn++){var jt=Zt[cn],ot=KC(jt);nt=Math.max(nt,ot.depth)}return ve.depth<=nt?vt[xn]?null:(q(me,nt+1),vt[xn]=!0,!0):!1};if(d&&w){var Z=[],G={},ae=function(me){return Z.push(me)},$=function(){return Z.shift()};for(f.forEach(function(je){return Z.push(je)});Z.length>0;){var ge=$(),ee=W(ge,G);if(ee)ge.outgoers().filter(function(je){return je.isNode()&&l.has(je)}).forEach(ae);else if(ee===null){tc("Detected double maximal shift for node `"+ge.id()+"`.  Bailing maximal adjustment due to cycle.  Use `options.maximal: true` only on DAGs.");break}}}z();var de=0;if(i.avoidOverlap)for(var re=0;re<f.length;re++){var ke=f[re],Ce=ke.layoutDimensions(i),_e=Ce.w,Te=Ce.h;de=Math.max(de,_e,Te)}var Be={},Ge=function(me){if(Be[me.id()])return Be[me.id()];for(var vt=KC(me).depth,ve=me.neighborhood(),Zt=0,nt=0,xn=0;xn<ve.length;xn++){var cn=ve[xn];if(!(cn.isEdge()||cn.isParent()||!f.has(cn))){var jt=KC(cn);if(jt!=null){var ot=jt.index,be=jt.depth;if(!(ot==null||be==null)){var We=N[be].length;be<vt&&(Zt+=ot/We,nt++)}}}}return nt=Math.max(1,nt),Zt=Zt/nt,nt===0&&(Zt=0),Be[me.id()]=Zt,Zt},Xe=function(me,vt){var ve=Ge(me),Zt=Ge(vt),nt=ve-Zt;return nt===0?JOe(me.id(),vt.id()):nt};i.depthSort!==void 0&&(Xe=i.depthSort);for(var Ee=0;Ee<N.length;Ee++)N[Ee].sort(Xe),Y(Ee);for(var Ze=[],Ie=0;Ie<X.length;Ie++)Ze.push(X[Ie]);N.unshift(Ze),z();for(var Oe=0,Le=0;Le<N.length;Le++)Oe=Math.max(N[Le].length,Oe);var $e={x:y.x1+y.w/2,y:y.x1+y.h/2},xe=N.reduce(function(je,me){return Math.max(je,me.length)},0),Ae=function(me){var vt=KC(me),ve=vt.depth,Zt=vt.index,nt=N[ve].length,xn=Math.max(y.w/((i.grid?xe:nt)+1),de),cn=Math.max(y.h/(N.length+1),de),jt=Math.min(y.w/2/N.length,y.h/2/N.length);if(jt=Math.max(jt,de),i.circle){var be=jt*ve+jt-(N.length>0&&N[0].length<=3?jt/2:0),We=2*Math.PI/N[ve].length*Zt;return ve===0&&N[0].length===1&&(be=1),{x:$e.x+be*Math.cos(We),y:$e.y+be*Math.sin(We)}}else{var ot={x:$e.x+(Zt+1-(nt+1)/2)*xn,y:(ve+1)*cn};return ot}};return l.nodes().layoutPositions(this,i,Ae),this};var eln={fit:!0,padding:30,boundingBox:void 0,avoidOverlap:!0,nodeDimensionsIncludeLabels:!1,spacingFactor:void 0,radius:void 0,startAngle:3/2*Math.PI,sweep:void 0,clockwise:!0,sort:void 0,animate:!1,animationDuration:500,animationEasing:void 0,animateFilter:function(i,o){return!0},ready:void 0,stop:void 0,transform:function(i,o){return o}};function MPe(r){this.options=bs({},eln,r)}MPe.prototype.run=function(){var r=this.options,i=r,o=r.cy,l=i.eles,f=i.counterclockwise!==void 0?!i.counterclockwise:i.clockwise,b=l.nodes().not(":parent");i.sort&&(b=b.sort(i.sort));for(var d=J0(i.boundingBox?i.boundingBox:{x1:0,y1:0,w:o.width(),h:o.height()}),w={x:d.x1+d.w/2,y:d.y1+d.h/2},y=i.sweep===void 0?2*Math.PI-2*Math.PI/b.length:i.sweep,k=y/Math.max(1,b.length-1),E,T=0,C=0;C<b.length;C++){var S=b[C],L=S.layoutDimensions(i),O=L.w,B=L.h;T=Math.max(T,O,B)}if(Vr(i.radius)?E=i.radius:b.length<=1?E=0:E=Math.min(d.h,d.w)/2-T,b.length>1&&i.avoidOverlap){T*=1.75;var N=Math.cos(k)-Math.cos(0),F=Math.sin(k)-Math.sin(0),R=Math.sqrt(T*T/(N*N+F*F));E=Math.max(R,E)}var q=function(te,H){var Y=i.startAngle+H*k*(f?1:-1),z=E*Math.cos(Y),W=E*Math.sin(Y),Z={x:w.x+z,y:w.y+W};return Z};return l.nodes().layoutPositions(this,i,q),this};var tln={fit:!0,padding:30,startAngle:3/2*Math.PI,sweep:void 0,clockwise:!0,equidistant:!1,minNodeSpacing:10,boundingBox:void 0,avoidOverlap:!0,nodeDimensionsIncludeLabels:!1,height:void 0,width:void 0,spacingFactor:void 0,concentric:function(i){return i.degree()},levelWidth:function(i){return i.maxDegree()/4},animate:!1,animationDuration:500,animationEasing:void 0,animateFilter:function(i,o){return!0},ready:void 0,stop:void 0,transform:function(i,o){return o}};function DPe(r){this.options=bs({},tln,r)}DPe.prototype.run=function(){for(var r=this.options,i=r,o=i.counterclockwise!==void 0?!i.counterclockwise:i.clockwise,l=r.cy,f=i.eles,b=f.nodes().not(":parent"),d=J0(i.boundingBox?i.boundingBox:{x1:0,y1:0,w:l.width(),h:l.height()}),w={x:d.x1+d.w/2,y:d.y1+d.h/2},y=[],k=0,E=0;E<b.length;E++){var T=b[E],C=void 0;C=i.concentric(T),y.push({value:C,node:T}),T._private.scratch.concentric=C}b.updateStyle();for(var S=0;S<b.length;S++){var L=b[S],O=L.layoutDimensions(i);k=Math.max(k,O.w,O.h)}y.sort(function(je,me){return me.value-je.value});for(var B=i.levelWidth(b),N=[[]],F=N[0],R=0;R<y.length;R++){var q=y[R];if(F.length>0){var X=Math.abs(F[0].value-q.value);X>=B&&(F=[],N.push(F))}F.push(q)}var te=k+i.minNodeSpacing;if(!i.avoidOverlap){var H=N.length>0&&N[0].length>1,Y=Math.min(d.w,d.h)/2-te,z=Y/(N.length+H?1:0);te=Math.min(te,z)}for(var W=0,Z=0;Z<N.length;Z++){var G=N[Z],ae=i.sweep===void 0?2*Math.PI-2*Math.PI/G.length:i.sweep,$=G.dTheta=ae/Math.max(1,G.length-1);if(G.length>1&&i.avoidOverlap){var ge=Math.cos($)-Math.cos(0),ee=Math.sin($)-Math.sin(0),de=Math.sqrt(te*te/(ge*ge+ee*ee));W=Math.max(de,W)}G.r=W,W+=te}if(i.equidistant){for(var re=0,ke=0,Ce=0;Ce<N.length;Ce++){var _e=N[Ce],Te=_e.r-ke;re=Math.max(re,Te)}ke=0;for(var Be=0;Be<N.length;Be++){var Ge=N[Be];Be===0&&(ke=Ge.r),Ge.r=ke,ke+=re}}for(var Xe={},Ee=0;Ee<N.length;Ee++)for(var Ze=N[Ee],Ie=Ze.dTheta,Oe=Ze.r,Le=0;Le<Ze.length;Le++){var $e=Ze[Le],xe=i.startAngle+(o?1:-1)*Ie*Le,Ae={x:w.x+Oe*Math.cos(xe),y:w.y+Oe*Math.sin(xe)};Xe[$e.node.id()]=Ae}return f.nodes().layoutPositions(this,i,function(je){var me=je.id();return Xe[me]}),this};var Sue,nln={ready:function(){},stop:function(){},animate:!0,animationEasing:void 0,animationDuration:void 0,animateFilter:function(i,o){return!0},animationThreshold:250,refresh:20,fit:!0,padding:30,boundingBox:void 0,nodeDimensionsIncludeLabels:!1,randomize:!1,componentSpacing:40,nodeRepulsion:function(i){return 2048},nodeOverlap:4,idealEdgeLength:function(i){return 32},edgeElasticity:function(i){return 32},nestingFactor:1.2,gravity:1,numIter:1e3,initialTemp:1e3,coolingFactor:.99,minTemp:1};function KU(r){this.options=bs({},nln,r),this.options.layout=this}KU.prototype.run=function(){var r=this.options,i=r.cy,o=this;o.stopped=!1,(r.animate===!0||r.animate===!1)&&o.emit({type:"layoutstart",layout:o}),r.debug===!0?Sue=!0:Sue=!1;var l=rln(i,o,r);Sue&&aln(l),r.randomize&&oln(l);var f=d4(),b=function(){cln(l,i,r),r.fit===!0&&i.fit(r.padding)},d=function(C){return!(o.stopped||C>=r.numIter||(uln(l,r),l.temperature=l.temperature*r.coolingFactor,l.temperature<r.minTemp))},w=function(){if(r.animate===!0||r.animate===!1)b(),o.one("layoutstop",r.stop),o.emit({type:"layoutstop",layout:o});else{var C=r.eles.nodes(),S=LPe(l,r,C);C.layoutPositions(o,r,S)}},y=0,k=!0;if(r.animate===!0){var E=function T(){for(var C=0;k&&C<r.refresh;)k=d(y),y++,C++;if(!k)OPe(l,r),w();else{var S=d4();S-f>=r.animationThreshold&&b(),mU(T)}};E()}else{for(;k;)k=d(y),y++;OPe(l,r),w()}return this},KU.prototype.stop=function(){return this.stopped=!0,this.thread&&this.thread.stop(),this.emit("layoutstop"),this},KU.prototype.destroy=function(){return this.thread&&this.thread.stop(),this};var rln=function(i,o,l){for(var f=l.eles.edges(),b=l.eles.nodes(),d={isCompound:i.hasCompoundNodes(),layoutNodes:[],idToIndex:{},nodeSize:b.size(),graphSet:[],indexToGraph:[],layoutEdges:[],edgeSize:f.size(),temperature:l.initialTemp,clientWidth:i.width(),clientHeight:i.width(),boundingBox:J0(l.boundingBox?l.boundingBox:{x1:0,y1:0,w:i.width(),h:i.height()})},w=l.eles.components(),y={},k=0;k<w.length;k++)for(var E=w[k],T=0;T<E.length;T++){var C=E[T];y[C.id()]=k}for(var k=0;k<d.nodeSize;k++){var S=b[k],L=S.layoutDimensions(l),O={};O.isLocked=S.locked(),O.id=S.data("id"),O.parentId=S.data("parent"),O.cmptId=y[S.id()],O.children=[],O.positionX=S.position("x"),O.positionY=S.position("y"),O.offsetX=0,O.offsetY=0,O.height=L.w,O.width=L.h,O.maxX=O.positionX+O.width/2,O.minX=O.positionX-O.width/2,O.maxY=O.positionY+O.height/2,O.minY=O.positionY-O.height/2,O.padLeft=parseFloat(S.style("padding")),O.padRight=parseFloat(S.style("padding")),O.padTop=parseFloat(S.style("padding")),O.padBottom=parseFloat(S.style("padding")),O.nodeRepulsion=al(l.nodeRepulsion)?l.nodeRepulsion(S):l.nodeRepulsion,d.layoutNodes.push(O),d.idToIndex[O.id]=k}for(var B=[],N=0,F=-1,R=[],k=0;k<d.nodeSize;k++){var S=d.layoutNodes[k],q=S.parentId;q!=null?d.layoutNodes[d.idToIndex[q]].children.push(S.id):(B[++F]=S.id,R.push(S.id))}for(d.graphSet.push(R);N<=F;){var X=B[N++],te=d.idToIndex[X],C=d.layoutNodes[te],H=C.children;if(H.length>0){d.graphSet.push(H);for(var k=0;k<H.length;k++)B[++F]=H[k]}}for(var k=0;k<d.graphSet.length;k++)for(var Y=d.graphSet[k],T=0;T<Y.length;T++){var z=d.idToIndex[Y[T]];d.indexToGraph[z]=k}for(var k=0;k<d.edgeSize;k++){var W=f[k],Z={};Z.id=W.data("id"),Z.sourceId=W.data("source"),Z.targetId=W.data("target");var G=al(l.idealEdgeLength)?l.idealEdgeLength(W):l.idealEdgeLength,ae=al(l.edgeElasticity)?l.edgeElasticity(W):l.edgeElasticity,$=d.idToIndex[Z.sourceId],ge=d.idToIndex[Z.targetId],ee=d.indexToGraph[$],de=d.indexToGraph[ge];if(ee!=de){for(var re=iln(Z.sourceId,Z.targetId,d),ke=d.graphSet[re],Ce=0,O=d.layoutNodes[$];ke.indexOf(O.id)===-1;)O=d.layoutNodes[d.idToIndex[O.parentId]],Ce++;for(O=d.layoutNodes[ge];ke.indexOf(O.id)===-1;)O=d.layoutNodes[d.idToIndex[O.parentId]],Ce++;G*=Ce*l.nestingFactor}Z.idealLength=G,Z.elasticity=ae,d.layoutEdges.push(Z)}return d},iln=function(i,o,l){var f=sln(i,o,0,l);return 2>f.count?0:f.graph},sln=function r(i,o,l,f){var b=f.graphSet[l];if(-1<b.indexOf(i)&&-1<b.indexOf(o))return{count:2,graph:l};for(var d=0,w=0;w<b.length;w++){var y=b[w],k=f.idToIndex[y],E=f.layoutNodes[k].children;if(E.length!==0){var T=f.indexToGraph[f.idToIndex[E[0]]],C=r(i,o,T,f);if(C.count!==0)if(C.count===1){if(d++,d===2)break}else return C}}return{count:d,graph:l}},aln,oln=function(i,o){for(var l=i.clientWidth,f=i.clientHeight,b=0;b<i.nodeSize;b++){var d=i.layoutNodes[b];d.children.length===0&&!d.isLocked&&(d.positionX=Math.random()*l,d.positionY=Math.random()*f)}},LPe=function(i,o,l){var f=i.boundingBox,b={x1:1/0,x2:-1/0,y1:1/0,y2:-1/0};return o.boundingBox&&(l.forEach(function(d){var w=i.layoutNodes[i.idToIndex[d.data("id")]];b.x1=Math.min(b.x1,w.positionX),b.x2=Math.max(b.x2,w.positionX),b.y1=Math.min(b.y1,w.positionY),b.y2=Math.max(b.y2,w.positionY)}),b.w=b.x2-b.x1,b.h=b.y2-b.y1),function(d,w){var y=i.layoutNodes[i.idToIndex[d.data("id")]];if(o.boundingBox){var k=(y.positionX-b.x1)/b.w,E=(y.positionY-b.y1)/b.h;return{x:f.x1+k*f.w,y:f.y1+E*f.h}}else return{x:y.positionX,y:y.positionY}}},cln=function(i,o,l){var f=l.layout,b=l.eles.nodes(),d=LPe(i,l,b);b.positions(d),i.ready!==!0&&(i.ready=!0,f.one("layoutready",l.ready),f.emit({type:"layoutready",layout:this}))},uln=function(i,o,l){lln(i,o),dln(i),gln(i,o),pln(i),bln(i)},lln=function(i,o){for(var l=0;l<i.graphSet.length;l++)for(var f=i.graphSet[l],b=f.length,d=0;d<b;d++)for(var w=i.layoutNodes[i.idToIndex[f[d]]],y=d+1;y<b;y++){var k=i.layoutNodes[i.idToIndex[f[y]]];hln(w,k,i,o)}},IPe=function(i){return-i+2*i*Math.random()},hln=function(i,o,l,f){var b=i.cmptId,d=o.cmptId;if(!(b!==d&&!l.isCompound)){var w=o.positionX-i.positionX,y=o.positionY-i.positionY,k=1;w===0&&y===0&&(w=IPe(k),y=IPe(k));var E=fln(i,o,w,y);if(E>0)var T=f.nodeOverlap*E,C=Math.sqrt(w*w+y*y),S=T*w/C,L=T*y/C;else var O=XU(i,w,y),B=XU(o,-1*w,-1*y),N=B.x-O.x,F=B.y-O.y,R=N*N+F*F,C=Math.sqrt(R),T=(i.nodeRepulsion+o.nodeRepulsion)/R,S=T*N/C,L=T*F/C;i.isLocked||(i.offsetX-=S,i.offsetY-=L),o.isLocked||(o.offsetX+=S,o.offsetY+=L)}},fln=function(i,o,l,f){if(l>0)var b=i.maxX-o.minX;else var b=o.maxX-i.minX;if(f>0)var d=i.maxY-o.minY;else var d=o.maxY-i.minY;return b>=0&&d>=0?Math.sqrt(b*b+d*d):0},XU=function(i,o,l){var f=i.positionX,b=i.positionY,d=i.height||1,w=i.width||1,y=l/o,k=d/w,E={};return o===0&&0<l||o===0&&0>l?(E.x=f,E.y=b+d/2,E):0<o&&-1*k<=y&&y<=k?(E.x=f+w/2,E.y=b+w*l/2/o,E):0>o&&-1*k<=y&&y<=k?(E.x=f-w/2,E.y=b-w*l/2/o,E):0<l&&(y<=-1*k||y>=k)?(E.x=f+d*o/2/l,E.y=b+d/2,E):(0>l&&(y<=-1*k||y>=k)&&(E.x=f-d*o/2/l,E.y=b-d/2),E)},dln=function(i,o){for(var l=0;l<i.edgeSize;l++){var f=i.layoutEdges[l],b=i.idToIndex[f.sourceId],d=i.layoutNodes[b],w=i.idToIndex[f.targetId],y=i.layoutNodes[w],k=y.positionX-d.positionX,E=y.positionY-d.positionY;if(!(k===0&&E===0)){var T=XU(d,k,E),C=XU(y,-1*k,-1*E),S=C.x-T.x,L=C.y-T.y,O=Math.sqrt(S*S+L*L),B=Math.pow(f.idealLength-O,2)/f.elasticity;if(O!==0)var N=B*S/O,F=B*L/O;else var N=0,F=0;d.isLocked||(d.offsetX+=N,d.offsetY+=F),y.isLocked||(y.offsetX-=N,y.offsetY-=F)}}},gln=function(i,o){if(o.gravity!==0)for(var l=1,f=0;f<i.graphSet.length;f++){var b=i.graphSet[f],d=b.length;if(f===0)var w=i.clientHeight/2,y=i.clientWidth/2;else var k=i.layoutNodes[i.idToIndex[b[0]]],E=i.layoutNodes[i.idToIndex[k.parentId]],w=E.positionX,y=E.positionY;for(var T=0;T<d;T++){var C=i.layoutNodes[i.idToIndex[b[T]]];if(!C.isLocked){var S=w-C.positionX,L=y-C.positionY,O=Math.sqrt(S*S+L*L);if(O>l){var B=o.gravity*S/O,N=o.gravity*L/O;C.offsetX+=B,C.offsetY+=N}}}}},pln=function(i,o){var l=[],f=0,b=-1;for(l.push.apply(l,i.graphSet[0]),b+=i.graphSet[0].length;f<=b;){var d=l[f++],w=i.idToIndex[d],y=i.layoutNodes[w],k=y.children;if(0<k.length&&!y.isLocked){for(var E=y.offsetX,T=y.offsetY,C=0;C<k.length;C++){var S=i.layoutNodes[i.idToIndex[k[C]]];S.offsetX+=E,S.offsetY+=T,l[++b]=k[C]}y.offsetX=0,y.offsetY=0}}},bln=function(i,o){for(var l=0;l<i.nodeSize;l++){var f=i.layoutNodes[l];0<f.children.length&&(f.maxX=void 0,f.minX=void 0,f.maxY=void 0,f.minY=void 0)}for(var l=0;l<i.nodeSize;l++){var f=i.layoutNodes[l];if(!(0<f.children.length||f.isLocked)){var b=vln(f.offsetX,f.offsetY,i.temperature);f.positionX+=b.x,f.positionY+=b.y,f.offsetX=0,f.offsetY=0,f.minX=f.positionX-f.width,f.maxX=f.positionX+f.width,f.minY=f.positionY-f.height,f.maxY=f.positionY+f.height,wln(f,i)}}for(var l=0;l<i.nodeSize;l++){var f=i.layoutNodes[l];0<f.children.length&&!f.isLocked&&(f.positionX=(f.maxX+f.minX)/2,f.positionY=(f.maxY+f.minY)/2,f.width=f.maxX-f.minX,f.height=f.maxY-f.minY)}},vln=function(i,o,l){var f=Math.sqrt(i*i+o*o);if(f>l)var b={x:l*i/f,y:l*o/f};else var b={x:i,y:o};return b},wln=function r(i,o){var l=i.parentId;if(l!=null){var f=o.layoutNodes[o.idToIndex[l]],b=!1;if((f.maxX==null||i.maxX+f.padRight>f.maxX)&&(f.maxX=i.maxX+f.padRight,b=!0),(f.minX==null||i.minX-f.padLeft<f.minX)&&(f.minX=i.minX-f.padLeft,b=!0),(f.maxY==null||i.maxY+f.padBottom>f.maxY)&&(f.maxY=i.maxY+f.padBottom,b=!0),(f.minY==null||i.minY-f.padTop<f.minY)&&(f.minY=i.minY-f.padTop,b=!0),b)return r(f,o)}},OPe=function(i,o){for(var l=i.layoutNodes,f=[],b=0;b<l.length;b++){var d=l[b],w=d.cmptId,y=f[w]=f[w]||[];y.push(d)}for(var k=0,b=0;b<f.length;b++){var E=f[b];if(!!E){E.x1=1/0,E.x2=-1/0,E.y1=1/0,E.y2=-1/0;for(var T=0;T<E.length;T++){var C=E[T];E.x1=Math.min(E.x1,C.positionX-C.width/2),E.x2=Math.max(E.x2,C.positionX+C.width/2),E.y1=Math.min(E.y1,C.positionY-C.height/2),E.y2=Math.max(E.y2,C.positionY+C.height/2)}E.w=E.x2-E.x1,E.h=E.y2-E.y1,k+=E.w*E.h}}f.sort(function(F,R){return R.w*R.h-F.w*F.h});for(var S=0,L=0,O=0,B=0,N=Math.sqrt(k)*i.clientWidth/i.clientHeight,b=0;b<f.length;b++){var E=f[b];if(!!E){for(var T=0;T<E.length;T++){var C=E[T];C.isLocked||(C.positionX+=S-E.x1,C.positionY+=L-E.y1)}S+=E.w+o.componentSpacing,O+=E.w+o.componentSpacing,B=Math.max(B,E.h),O>N&&(L+=B+o.componentSpacing,S=0,O=0,B=0)}}},mln={fit:!0,padding:30,boundingBox:void 0,avoidOverlap:!0,avoidOverlapPadding:10,nodeDimensionsIncludeLabels:!1,spacingFactor:void 0,condense:!1,rows:void 0,cols:void 0,position:function(i){},sort:void 0,animate:!1,animationDuration:500,animationEasing:void 0,animateFilter:function(i,o){return!0},ready:void 0,stop:void 0,transform:function(i,o){return o}};function NPe(r){this.options=bs({},mln,r)}NPe.prototype.run=function(){var r=this.options,i=r,o=r.cy,l=i.eles,f=l.nodes().not(":parent");i.sort&&(f=f.sort(i.sort));var b=J0(i.boundingBox?i.boundingBox:{x1:0,y1:0,w:o.width(),h:o.height()});if(b.h===0||b.w===0)l.nodes().layoutPositions(this,i,function(Be){return{x:b.x1,y:b.y1}});else{var d=f.size(),w=Math.sqrt(d*b.h/b.w),y=Math.round(w),k=Math.round(b.w/b.h*w),E=function(Ge){if(Ge==null)return Math.min(y,k);var Xe=Math.min(y,k);Xe==y?y=Ge:k=Ge},T=function(Ge){if(Ge==null)return Math.max(y,k);var Xe=Math.max(y,k);Xe==y?y=Ge:k=Ge},C=i.rows,S=i.cols!=null?i.cols:i.columns;if(C!=null&&S!=null)y=C,k=S;else if(C!=null&&S==null)y=C,k=Math.ceil(d/y);else if(C==null&&S!=null)k=S,y=Math.ceil(d/k);else if(k*y>d){var L=E(),O=T();(L-1)*O>=d?E(L-1):(O-1)*L>=d&&T(O-1)}else for(;k*y<d;){var B=E(),N=T();(N+1)*B>=d?T(N+1):E(B+1)}var F=b.w/k,R=b.h/y;if(i.condense&&(F=0,R=0),i.avoidOverlap)for(var q=0;q<f.length;q++){var X=f[q],te=X._private.position;(te.x==null||te.y==null)&&(te.x=0,te.y=0);var H=X.layoutDimensions(i),Y=i.avoidOverlapPadding,z=H.w+Y,W=H.h+Y;F=Math.max(F,z),R=Math.max(R,W)}for(var Z={},G=function(Ge,Xe){return!!Z["c-"+Ge+"-"+Xe]},ae=function(Ge,Xe){Z["c-"+Ge+"-"+Xe]=!0},$=0,ge=0,ee=function(){ge++,ge>=k&&(ge=0,$++)},de={},re=0;re<f.length;re++){var ke=f[re],Ce=i.position(ke);if(Ce&&(Ce.row!==void 0||Ce.col!==void 0)){var _e={row:Ce.row,col:Ce.col};if(_e.col===void 0)for(_e.col=0;G(_e.row,_e.col);)_e.col++;else if(_e.row===void 0)for(_e.row=0;G(_e.row,_e.col);)_e.row++;de[ke.id()]=_e,ae(_e.row,_e.col)}}var Te=function(Ge,Xe){var Ee,Ze;if(Ge.locked()||Ge.isParent())return!1;var Ie=de[Ge.id()];if(Ie)Ee=Ie.col*F+F/2+b.x1,Ze=Ie.row*R+R/2+b.y1;else{for(;G($,ge);)ee();Ee=ge*F+F/2+b.x1,Ze=$*R+R/2+b.y1,ae($,ge),ee()}return{x:Ee,y:Ze}};f.layoutPositions(this,i,Te)}return this};var yln={ready:function(){},stop:function(){}};function Aue(r){this.options=bs({},yln,r)}Aue.prototype.run=function(){var r=this.options,i=r.eles,o=this;return r.cy,o.emit("layoutstart"),i.nodes().positions(function(){return{x:0,y:0}}),o.one("layoutready",r.ready),o.emit("layoutready"),o.one("layoutstop",r.stop),o.emit("layoutstop"),this},Aue.prototype.stop=function(){return this};var kln={positions:void 0,zoom:void 0,pan:void 0,fit:!0,padding:30,animate:!1,animationDuration:500,animationEasing:void 0,animateFilter:function(i,o){return!0},ready:void 0,stop:void 0,transform:function(i,o){return o}};function PPe(r){this.options=bs({},kln,r)}PPe.prototype.run=function(){var r=this.options,i=r.eles,o=i.nodes(),l=al(r.positions);function f(b){if(r.positions==null)return Non(b.position());if(l)return r.positions(b);var d=r.positions[b._private.data.id];return d==null?null:d}return o.layoutPositions(this,r,function(b,d){var w=f(b);return b.locked()||w==null?!1:w}),this};var xln={fit:!0,padding:30,boundingBox:void 0,animate:!1,animationDuration:500,animationEasing:void 0,animateFilter:function(i,o){return!0},ready:void 0,stop:void 0,transform:function(i,o){return o}};function BPe(r){this.options=bs({},xln,r)}BPe.prototype.run=function(){var r=this.options,i=r.cy,o=r.eles,l=J0(r.boundingBox?r.boundingBox:{x1:0,y1:0,w:i.width(),h:i.height()}),f=function(d,w){return{x:l.x1+Math.round(Math.random()*l.w),y:l.y1+Math.round(Math.random()*l.h)}};return o.nodes().layoutPositions(this,r,f),this};var Eln=[{name:"breadthfirst",impl:APe},{name:"circle",impl:MPe},{name:"concentric",impl:DPe},{name:"cose",impl:KU},{name:"grid",impl:NPe},{name:"null",impl:Aue},{name:"preset",impl:PPe},{name:"random",impl:BPe}];function FPe(r){this.options=r,this.notifications=0}var RPe=function(){},jPe=function(){throw new Error("A headless instance can not render images")};FPe.prototype={recalculateRenderedStyle:RPe,notify:function(){this.notifications++},init:RPe,isHeadless:function(){return!0},png:jPe,jpg:jPe};var Mue={};Mue.arrowShapeWidth=.3,Mue.registerArrowShapes=function(){var r=this.arrowShapes={},i=this,o=function(k,E,T,C,S,L,O){var B=S.x-T/2-O,N=S.x+T/2+O,F=S.y-T/2-O,R=S.y+T/2+O,q=B<=k&&k<=N&&F<=E&&E<=R;return q},l=function(k,E,T,C,S){var L=k*Math.cos(C)-E*Math.sin(C),O=k*Math.sin(C)+E*Math.cos(C),B=L*T,N=O*T,F=B+S.x,R=N+S.y;return{x:F,y:R}},f=function(k,E,T,C){for(var S=[],L=0;L<k.length;L+=2){var O=k[L],B=k[L+1];S.push(l(O,B,E,T,C))}return S},b=function(k){for(var E=[],T=0;T<k.length;T++){var C=k[T];E.push(C.x,C.y)}return E},d=function(k){return k.pstyle("width").pfValue*k.pstyle("arrow-scale").pfValue*2},w=function(k,E){$i(E)&&(E=r[E]),r[k]=bs({name:k,points:[-.15,-.3,.15,-.3,.15,.3,-.15,.3],collide:function(C,S,L,O,B,N){var F=b(f(this.points,L+2*N,O,B)),R=eg(C,S,F);return R},roughCollide:o,draw:function(C,S,L,O){var B=f(this.points,S,L,O);i.arrowShapeImpl("polygon")(C,B)},spacing:function(C){return 0},gap:d},E)};w("none",{collide:kU,roughCollide:kU,draw:tue,spacing:cNe,gap:cNe}),w("triangle",{points:[-.15,-.3,0,0,.15,-.3]}),w("arrow","triangle"),w("triangle-backcurve",{points:r.triangle.points,controlPoint:[0,-.15],roughCollide:o,draw:function(k,E,T,C,S){var L=f(this.points,E,T,C),O=this.controlPoint,B=l(O[0],O[1],E,T,C);i.arrowShapeImpl(this.name)(k,L,B)},gap:function(k){return d(k)*.8}}),w("triangle-tee",{points:[0,0,.15,-.3,-.15,-.3,0,0],pointsTee:[-.15,-.4,-.15,-.5,.15,-.5,.15,-.4],collide:function(k,E,T,C,S,L,O){var B=b(f(this.points,T+2*O,C,S)),N=b(f(this.pointsTee,T+2*O,C,S)),F=eg(k,E,B)||eg(k,E,N);return F},draw:function(k,E,T,C,S){var L=f(this.points,E,T,C),O=f(this.pointsTee,E,T,C);i.arrowShapeImpl(this.name)(k,L,O)}}),w("circle-triangle",{radius:.15,pointsTr:[0,-.15,.15,-.45,-.15,-.45,0,-.15],collide:function(k,E,T,C,S,L,O){var B=S,N=Math.pow(B.x-k,2)+Math.pow(B.y-E,2)<=Math.pow((T+2*O)*this.radius,2),F=b(f(this.points,T+2*O,C,S));return eg(k,E,F)||N},draw:function(k,E,T,C,S){var L=f(this.pointsTr,E,T,C);i.arrowShapeImpl(this.name)(k,L,C.x,C.y,this.radius*E)},spacing:function(k){return i.getArrowWidth(k.pstyle("width").pfValue,k.pstyle("arrow-scale").value)*this.radius}}),w("triangle-cross",{points:[0,0,.15,-.3,-.15,-.3,0,0],baseCrossLinePts:[-.15,-.4,-.15,-.4,.15,-.4,.15,-.4],crossLinePts:function(k,E){var T=this.baseCrossLinePts.slice(),C=E/k,S=3,L=5;return T[S]=T[S]-C,T[L]=T[L]-C,T},collide:function(k,E,T,C,S,L,O){var B=b(f(this.points,T+2*O,C,S)),N=b(f(this.crossLinePts(T,L),T+2*O,C,S)),F=eg(k,E,B)||eg(k,E,N);return F},draw:function(k,E,T,C,S){var L=f(this.points,E,T,C),O=f(this.crossLinePts(E,S),E,T,C);i.arrowShapeImpl(this.name)(k,L,O)}}),w("vee",{points:[-.15,-.3,0,0,.15,-.3,0,-.15],gap:function(k){return d(k)*.525}}),w("circle",{radius:.15,collide:function(k,E,T,C,S,L,O){var B=S,N=Math.pow(B.x-k,2)+Math.pow(B.y-E,2)<=Math.pow((T+2*O)*this.radius,2);return N},draw:function(k,E,T,C,S){i.arrowShapeImpl(this.name)(k,C.x,C.y,this.radius*E)},spacing:function(k){return i.getArrowWidth(k.pstyle("width").pfValue,k.pstyle("arrow-scale").value)*this.radius}}),w("tee",{points:[-.15,0,-.15,-.1,.15,-.1,.15,0],spacing:function(k){return 1},gap:function(k){return 1}}),w("square",{points:[-.15,0,.15,0,.15,-.3,-.15,-.3]}),w("diamond",{points:[-.15,-.15,0,-.3,.15,-.15,0,0],gap:function(k){return k.pstyle("width").pfValue*k.pstyle("arrow-scale").value}}),w("chevron",{points:[0,0,-.15,-.15,-.1,-.2,0,-.1,.1,-.2,.15,-.15],gap:function(k){return .95*k.pstyle("width").pfValue*k.pstyle("arrow-scale").value}})};var T7={};T7.projectIntoViewport=function(r,i){var o=this.cy,l=this.findContainerClientCoords(),f=l[0],b=l[1],d=l[4],w=o.pan(),y=o.zoom(),k=((r-f)/d-w.x)/y,E=((i-b)/d-w.y)/y;return[k,E]},T7.findContainerClientCoords=function(){if(this.containerBB)return this.containerBB;var r=this.container,i=r.getBoundingClientRect(),o=Fu.getComputedStyle(r),l=function(N){return parseFloat(o.getPropertyValue(N))},f={left:l("padding-left"),right:l("padding-right"),top:l("padding-top"),bottom:l("padding-bottom")},b={left:l("border-left-width"),right:l("border-right-width"),top:l("border-top-width"),bottom:l("border-bottom-width")},d=r.clientWidth,w=r.clientHeight,y=f.left+f.right,k=f.top+f.bottom,E=b.left+b.right,T=i.width/(d+E),C=d-y,S=w-k,L=i.left+f.left+b.left,O=i.top+f.top+b.top;return this.containerBB=[L,O,C,S,T]},T7.invalidateContainerClientCoordsCache=function(){this.containerBB=null},T7.findNearestElement=function(r,i,o,l){return this.findNearestElements(r,i,o,l)[0]},T7.findNearestElements=function(r,i,o,l){var f=this,b=this,d=b.getCachedZSortedEles(),w=[],y=b.cy.zoom(),k=b.cy.hasCompoundNodes(),E=(l?24:8)/y,T=(l?8:2)/y,C=(l?8:2)/y,S=1/0,L,O;o&&(d=d.interactive);function B(H,Y){if(H.isNode()){if(O)return;O=H,w.push(H)}if(H.isEdge()&&(Y==null||Y<S))if(L){if(L.pstyle("z-compound-depth").value===H.pstyle("z-compound-depth").value&&L.pstyle("z-compound-depth").value===H.pstyle("z-compound-depth").value){for(var z=0;z<w.length;z++)if(w[z].isEdge()){w[z]=H,L=H,S=Y!=null?Y:S;break}}}else w.push(H),L=H,S=Y!=null?Y:S}function N(H){var Y=H.outerWidth()+2*T,z=H.outerHeight()+2*T,W=Y/2,Z=z/2,G=H.position();if(G.x-W<=r&&r<=G.x+W&&G.y-Z<=i&&i<=G.y+Z){var ae=b.nodeShapes[f.getNodeShape(H)];if(ae.checkPoint(r,i,0,Y,z,G.x,G.y))return B(H,0),!0}}function F(H){var Y=H._private,z=Y.rscratch,W=H.pstyle("width").pfValue,Z=H.pstyle("arrow-scale").value,G=W/2+E,ae=G*G,$=G*2,re=Y.source,ke=Y.target,ge;if(z.edgeType==="segments"||z.edgeType==="straight"||z.edgeType==="haystack"){for(var ee=z.allpts,de=0;de+3<ee.length;de+=2)if(Yon(r,i,ee[de],ee[de+1],ee[de+2],ee[de+3],$)&&ae>(ge=Zon(r,i,ee[de],ee[de+1],ee[de+2],ee[de+3])))return B(H,ge),!0}else if(z.edgeType==="bezier"||z.edgeType==="multibezier"||z.edgeType==="self"||z.edgeType==="compound"){for(var ee=z.allpts,de=0;de+5<z.allpts.length;de+=4)if(Won(r,i,ee[de],ee[de+1],ee[de+2],ee[de+3],ee[de+4],ee[de+5],$)&&ae>(ge=Qon(r,i,ee[de],ee[de+1],ee[de+2],ee[de+3],ee[de+4],ee[de+5])))return B(H,ge),!0}for(var re=re||Y.source,ke=ke||Y.target,Ce=f.getArrowWidth(W,Z),_e=[{name:"source",x:z.arrowStartX,y:z.arrowStartY,angle:z.srcArrowAngle},{name:"target",x:z.arrowEndX,y:z.arrowEndY,angle:z.tgtArrowAngle},{name:"mid-source",x:z.midX,y:z.midY,angle:z.midsrcArrowAngle},{name:"mid-target",x:z.midX,y:z.midY,angle:z.midtgtArrowAngle}],de=0;de<_e.length;de++){var Te=_e[de],Be=b.arrowShapes[H.pstyle(Te.name+"-arrow-shape").value],Ge=H.pstyle("width").pfValue;if(Be.roughCollide(r,i,Ce,Te.angle,{x:Te.x,y:Te.y},Ge,E)&&Be.collide(r,i,Ce,Te.angle,{x:Te.x,y:Te.y},Ge,E))return B(H),!0}k&&w.length>0&&(N(re),N(ke))}function R(H,Y,z){return Gv(H,Y,z)}function q(H,Y){var z=H._private,W=C,Z;Y?Z=Y+"-":Z="",H.boundingBox();var G=z.labelBounds[Y||"main"],ae=H.pstyle(Z+"label").value,$=H.pstyle("text-events").strValue==="yes";if(!(!$||!ae)){var ge=R(z.rscratch,"labelX",Y),ee=R(z.rscratch,"labelY",Y),de=R(z.rscratch,"labelAngle",Y),re=H.pstyle(Z+"text-margin-x").pfValue,ke=H.pstyle(Z+"text-margin-y").pfValue,Ce=G.x1-W-re,_e=G.x2+W-re,Te=G.y1-W-ke,Be=G.y2+W-ke;if(de){var Ge=Math.cos(de),Xe=Math.sin(de),Ee=function(Ae,je){return Ae=Ae-ge,je=je-ee,{x:Ae*Ge-je*Xe+ge,y:Ae*Xe+je*Ge+ee}},Ze=Ee(Ce,Te),Ie=Ee(Ce,Be),Oe=Ee(_e,Te),Le=Ee(_e,Be),$e=[Ze.x+re,Ze.y+ke,Oe.x+re,Oe.y+ke,Le.x+re,Le.y+ke,Ie.x+re,Ie.y+ke];if(eg(r,i,$e))return B(H),!0}else if(FC(G,r,i))return B(H),!0}}for(var X=d.length-1;X>=0;X--){var te=d[X];te.isNode()?N(te)||q(te):F(te)||q(te)||q(te,"source")||q(te,"target")}return w},T7.getAllInBox=function(r,i,o,l){var f=this.getCachedZSortedEles().interactive,b=[],d=Math.min(r,o),w=Math.max(r,o),y=Math.min(i,l),k=Math.max(i,l);r=d,o=w,i=y,l=k;for(var E=J0({x1:r,y1:i,x2:o,y2:l}),T=0;T<f.length;T++){var C=f[T];if(C.isNode()){var S=C,L=S.boundingBox({includeNodes:!0,includeEdges:!1,includeLabels:!1});aue(E,L)&&!bNe(L,E)&&b.push(S)}else{var O=C,B=O._private,N=B.rscratch;if(N.startX!=null&&N.startY!=null&&!FC(E,N.startX,N.startY)||N.endX!=null&&N.endY!=null&&!FC(E,N.endX,N.endY))continue;if(N.edgeType==="bezier"||N.edgeType==="multibezier"||N.edgeType==="self"||N.edgeType==="compound"||N.edgeType==="segments"||N.edgeType==="haystack"){for(var F=B.rstyle.bezierPts||B.rstyle.linePts||B.rstyle.haystackPts,R=!0,q=0;q<F.length;q++)if(!qon(E,F[q])){R=!1;break}R&&b.push(O)}else(N.edgeType==="haystack"||N.edgeType==="straight")&&b.push(O)}}return b};var QU={};QU.calculateArrowAngles=function(r){var i=r._private.rscratch,o=i.edgeType==="haystack",l=i.edgeType==="bezier",f=i.edgeType==="multibezier",b=i.edgeType==="segments",d=i.edgeType==="compound",w=i.edgeType==="self",y,k,E,T,C,S,N,F;if(o?(E=i.haystackPts[0],T=i.haystackPts[1],C=i.haystackPts[2],S=i.haystackPts[3]):(E=i.arrowStartX,T=i.arrowStartY,C=i.arrowEndX,S=i.arrowEndY),N=i.midX,F=i.midY,b)y=E-i.segpts[0],k=T-i.segpts[1];else if(f||d||w||l){var L=i.allpts,O=Kf(L[0],L[2],L[4],.1),B=Kf(L[1],L[3],L[5],.1);y=E-O,k=T-B}else y=E-N,k=T-F;i.srcArrowAngle=_U(y,k);var N=i.midX,F=i.midY;if(o&&(N=(E+C)/2,F=(T+S)/2),y=C-E,k=S-T,b){var L=i.allpts;if(L.length/2%2===0){var R=L.length/2,q=R-2;y=L[R]-L[q],k=L[R+1]-L[q+1]}else{var R=L.length/2-1,q=R-2,X=R+2;y=L[R]-L[q],k=L[R+1]-L[q+1]}}else if(f||d||w){var L=i.allpts,te=i.ctrlpts,H,Y,z,W;if(te.length/2%2===0){var Z=L.length/2-1,G=Z+2,ae=G+2;H=Kf(L[Z],L[G],L[ae],0),Y=Kf(L[Z+1],L[G+1],L[ae+1],0),z=Kf(L[Z],L[G],L[ae],1e-4),W=Kf(L[Z+1],L[G+1],L[ae+1],1e-4)}else{var G=L.length/2-1,Z=G-2,ae=G+2;H=Kf(L[Z],L[G],L[ae],.4999),Y=Kf(L[Z+1],L[G+1],L[ae+1],.4999),z=Kf(L[Z],L[G],L[ae],.5),W=Kf(L[Z+1],L[G+1],L[ae+1],.5)}y=z-H,k=W-Y}if(i.midtgtArrowAngle=_U(y,k),i.midDispX=y,i.midDispY=k,y*=-1,k*=-1,b){var L=i.allpts;if(L.length/2%2!==0){var R=L.length/2-1,X=R+2;y=-(L[X]-L[R]),k=-(L[X+1]-L[R+1])}}if(i.midsrcArrowAngle=_U(y,k),b)y=C-i.segpts[i.segpts.length-2],k=S-i.segpts[i.segpts.length-1];else if(f||d||w||l){var L=i.allpts,$=L.length,O=Kf(L[$-6],L[$-4],L[$-2],.9),B=Kf(L[$-5],L[$-3],L[$-1],.9);y=C-O,k=S-B}else y=C-N,k=S-F;i.tgtArrowAngle=_U(y,k)},QU.getArrowWidth=QU.getArrowHeight=function(r,i){var o=this.arrowWidthCache=this.arrowWidthCache||{},l=o[r+", "+i];return l||(l=Math.max(Math.pow(r*13.37,.9),29)*i,o[r+", "+i]=l,l)};var t0={};t0.findHaystackPoints=function(r){for(var i=0;i<r.length;i++){var o=r[i],l=o._private,f=l.rscratch;if(!f.haystack){var b=Math.random()*2*Math.PI;f.source={x:Math.cos(b),y:Math.sin(b)},b=Math.random()*2*Math.PI,f.target={x:Math.cos(b),y:Math.sin(b)}}var d=l.source,w=l.target,y=d.position(),k=w.position(),E=d.width(),T=w.width(),C=d.height(),S=w.height(),L=o.pstyle("haystack-radius").value,O=L/2;f.haystackPts=f.allpts=[f.source.x*E*O+y.x,f.source.y*C*O+y.y,f.target.x*T*O+k.x,f.target.y*S*O+k.y],f.midX=(f.allpts[0]+f.allpts[2])/2,f.midY=(f.allpts[1]+f.allpts[3])/2,f.edgeType="haystack",f.haystack=!0,this.storeEdgeProjections(o),this.calculateArrowAngles(o),this.recalculateEdgeLabelProjections(o),this.calculateLabelAngles(o)}},t0.findSegmentsPoints=function(r,i){var o=r._private.rscratch,l=i.posPts,f=i.intersectionPts,b=i.vectorNormInverse,d=r.pstyle("edge-distances").value,w=r.pstyle("segment-weights"),y=r.pstyle("segment-distances"),k=Math.min(w.pfValue.length,y.pfValue.length);o.edgeType="segments",o.segpts=[];for(var E=0;E<k;E++){var T=w.pfValue[E],C=y.pfValue[E],S=1-T,L=T,O=d==="node-position"?l:f,B={x:O.x1*S+O.x2*L,y:O.y1*S+O.y2*L};o.segpts.push(B.x+b.x*C,B.y+b.y*C)}},t0.findLoopPoints=function(r,i,o,l){var f=r._private.rscratch,b=i.dirCounts,d=i.srcPos,w=r.pstyle("control-point-distances"),y=w?w.pfValue[0]:void 0,k=r.pstyle("loop-direction").pfValue,E=r.pstyle("loop-sweep").pfValue,T=r.pstyle("control-point-step-size").pfValue;f.edgeType="self";var C=o,S=T;l&&(C=0,S=y);var L=k-Math.PI/2,O=L-E/2,B=L+E/2,N=String(k+"_"+E);C=b[N]===void 0?b[N]=0:++b[N],f.ctrlpts=[d.x+Math.cos(O)*1.4*S*(C/3+1),d.y+Math.sin(O)*1.4*S*(C/3+1),d.x+Math.cos(B)*1.4*S*(C/3+1),d.y+Math.sin(B)*1.4*S*(C/3+1)]},t0.findCompoundLoopPoints=function(r,i,o,l){var f=r._private.rscratch;f.edgeType="compound";var b=i.srcPos,d=i.tgtPos,w=i.srcW,y=i.srcH,k=i.tgtW,E=i.tgtH,T=r.pstyle("control-point-step-size").pfValue,C=r.pstyle("control-point-distances"),S=C?C.pfValue[0]:void 0,L=o,O=T;l&&(L=0,O=S);var B=50,N={x:b.x-w/2,y:b.y-y/2},F={x:d.x-k/2,y:d.y-E/2},R={x:Math.min(N.x,F.x),y:Math.min(N.y,F.y)},q=.5,X=Math.max(q,Math.log(w*.01)),te=Math.max(q,Math.log(k*.01));f.ctrlpts=[R.x,R.y-(1+Math.pow(B,1.12)/100)*O*(L/3+1)*X,R.x-(1+Math.pow(B,1.12)/100)*O*(L/3+1)*te,R.y]},t0.findStraightEdgePoints=function(r){r._private.rscratch.edgeType="straight"},t0.findBezierPoints=function(r,i,o,l,f){var b=r._private.rscratch,d=i.vectorNormInverse,w=i.posPts,y=i.intersectionPts,k=r.pstyle("edge-distances").value,E=r.pstyle("control-point-step-size").pfValue,T=r.pstyle("control-point-distances"),C=r.pstyle("control-point-weights"),S=T&&C?Math.min(T.value.length,C.value.length):1,L=T?T.pfValue[0]:void 0,O=C.value[0],B=l;b.edgeType=B?"multibezier":"bezier",b.ctrlpts=[];for(var N=0;N<S;N++){var F=(.5-i.eles.length/2+o)*E*(f?-1:1),R=void 0,q=gNe(F);B&&(L=T?T.pfValue[N]:E,O=C.value[N]),l?R=L:R=L!==void 0?q*L:void 0;var X=R!==void 0?R:F,te=1-O,H=O,Y=k==="node-position"?w:y,z={x:Y.x1*te+Y.x2*H,y:Y.y1*te+Y.y2*H};b.ctrlpts.push(z.x+d.x*X,z.y+d.y*X)}},t0.findTaxiPoints=function(r,i){var o=r._private.rscratch;o.edgeType="segments";var l="vertical",f="horizontal",b="leftward",d="rightward",w="downward",y="upward",k="auto",E=i.posPts,T=i.srcW,C=i.srcH,S=i.tgtW,L=i.tgtH,O=r.pstyle("edge-distances").value,B=O!=="node-position",N=r.pstyle("taxi-direction").value,F=N,R=r.pstyle("taxi-turn"),q=R.units==="%",X=R.pfValue,te=X<0,H=r.pstyle("taxi-turn-min-distance").pfValue,Y=B?(T+S)/2:0,z=B?(C+L)/2:0,W=E.x2-E.x1,Z=E.y2-E.y1,G=function(Rn,si){return Rn>0?Math.max(Rn-si,0):Math.min(Rn+si,0)},ae=G(W,Y),$=G(Z,z),ge=!1;F===k?N=Math.abs(ae)>Math.abs($)?f:l:F===y||F===w?(N=l,ge=!0):(F===b||F===d)&&(N=f,ge=!0);var ee=N===l,de=ee?$:ae,re=ee?Z:W,ke=gNe(re),Ce=!1;!(ge&&(q||te))&&(F===w&&re<0||F===y&&re>0||F===b&&re>0||F===d&&re<0)&&(ke*=-1,de=ke*Math.abs(de),Ce=!0);var _e;if(q){var Te=X<0?1+X:X;_e=Te*de}else{var Be=X<0?de:0;_e=Be+X*ke}var Ge=function(Rn){return Math.abs(Rn)<H||Math.abs(Rn)>=Math.abs(de)},Xe=Ge(_e),Ee=Ge(Math.abs(de)-Math.abs(_e)),Ze=Xe||Ee;if(Ze&&!Ce)if(ee){var Ie=Math.abs(re)<=C/2,Oe=Math.abs(W)<=S/2;if(Ie){var Le=(E.x1+E.x2)/2,$e=E.y1,xe=E.y2;o.segpts=[Le,$e,Le,xe]}else if(Oe){var Ae=(E.y1+E.y2)/2,je=E.x1,me=E.x2;o.segpts=[je,Ae,me,Ae]}else o.segpts=[E.x1,E.y2]}else{var vt=Math.abs(re)<=T/2,ve=Math.abs(Z)<=L/2;if(vt){var Zt=(E.y1+E.y2)/2,nt=E.x1,xn=E.x2;o.segpts=[nt,Zt,xn,Zt]}else if(ve){var cn=(E.x1+E.x2)/2,jt=E.y1,ot=E.y2;o.segpts=[cn,jt,cn,ot]}else o.segpts=[E.x2,E.y1]}else if(ee){var be=E.y1+_e+(B?C/2*ke:0),We=E.x1,ct=E.x2;o.segpts=[We,be,ct,be]}else{var Yt=E.x1+_e+(B?T/2*ke:0),Ut=E.y1,Wn=E.y2;o.segpts=[Yt,Ut,Yt,Wn]}},t0.tryToCorrectInvalidPoints=function(r,i){var o=r._private.rscratch;if(o.edgeType==="bezier"){var l=i.srcPos,f=i.tgtPos,b=i.srcW,d=i.srcH,w=i.tgtW,y=i.tgtH,k=i.srcShape,E=i.tgtShape,T=!Vr(o.startX)||!Vr(o.startY),C=!Vr(o.arrowStartX)||!Vr(o.arrowStartY),S=!Vr(o.endX)||!Vr(o.endY),L=!Vr(o.arrowEndX)||!Vr(o.arrowEndY),O=3,B=this.getArrowWidth(r.pstyle("width").pfValue,r.pstyle("arrow-scale").value)*this.arrowShapeWidth,N=O*B,F=m7({x:o.ctrlpts[0],y:o.ctrlpts[1]},{x:o.startX,y:o.startY}),R=F<N,q=m7({x:o.ctrlpts[0],y:o.ctrlpts[1]},{x:o.endX,y:o.endY}),X=q<N,te=!1;if(T||C||R){te=!0;var H={x:o.ctrlpts[0]-l.x,y:o.ctrlpts[1]-l.y},Y=Math.sqrt(H.x*H.x+H.y*H.y),z={x:H.x/Y,y:H.y/Y},W=Math.max(b,d),Z={x:o.ctrlpts[0]+z.x*2*W,y:o.ctrlpts[1]+z.y*2*W},G=k.intersectLine(l.x,l.y,b,d,Z.x,Z.y,0);R?(o.ctrlpts[0]=o.ctrlpts[0]+z.x*(N-F),o.ctrlpts[1]=o.ctrlpts[1]+z.y*(N-F)):(o.ctrlpts[0]=G[0]+z.x*N,o.ctrlpts[1]=G[1]+z.y*N)}if(S||L||X){te=!0;var ae={x:o.ctrlpts[0]-f.x,y:o.ctrlpts[1]-f.y},$=Math.sqrt(ae.x*ae.x+ae.y*ae.y),ge={x:ae.x/$,y:ae.y/$},ee=Math.max(b,d),de={x:o.ctrlpts[0]+ge.x*2*ee,y:o.ctrlpts[1]+ge.y*2*ee},re=E.intersectLine(f.x,f.y,w,y,de.x,de.y,0);X?(o.ctrlpts[0]=o.ctrlpts[0]+ge.x*(N-q),o.ctrlpts[1]=o.ctrlpts[1]+ge.y*(N-q)):(o.ctrlpts[0]=re[0]+ge.x*N,o.ctrlpts[1]=re[1]+ge.y*N)}te&&this.findEndpoints(r)}},t0.storeAllpts=function(r){var i=r._private.rscratch;if(i.edgeType==="multibezier"||i.edgeType==="bezier"||i.edgeType==="self"||i.edgeType==="compound"){i.allpts=[],i.allpts.push(i.startX,i.startY);for(var o=0;o+1<i.ctrlpts.length;o+=2)i.allpts.push(i.ctrlpts[o],i.ctrlpts[o+1]),o+3<i.ctrlpts.length&&i.allpts.push((i.ctrlpts[o]+i.ctrlpts[o+2])/2,(i.ctrlpts[o+1]+i.ctrlpts[o+3])/2);i.allpts.push(i.endX,i.endY);var l,f;i.ctrlpts.length/2%2===0?(l=i.allpts.length/2-1,i.midX=i.allpts[l],i.midY=i.allpts[l+1]):(l=i.allpts.length/2-3,f=.5,i.midX=Kf(i.allpts[l],i.allpts[l+2],i.allpts[l+4],f),i.midY=Kf(i.allpts[l+1],i.allpts[l+3],i.allpts[l+5],f))}else if(i.edgeType==="straight")i.allpts=[i.startX,i.startY,i.endX,i.endY],i.midX=(i.startX+i.endX+i.arrowStartX+i.arrowEndX)/4,i.midY=(i.startY+i.endY+i.arrowStartY+i.arrowEndY)/4;else if(i.edgeType==="segments")if(i.allpts=[],i.allpts.push(i.startX,i.startY),i.allpts.push.apply(i.allpts,i.segpts),i.allpts.push(i.endX,i.endY),i.segpts.length%4===0){var b=i.segpts.length/2,d=b-2;i.midX=(i.segpts[d]+i.segpts[b])/2,i.midY=(i.segpts[d+1]+i.segpts[b+1])/2}else{var w=i.segpts.length/2-1;i.midX=i.segpts[w],i.midY=i.segpts[w+1]}},t0.checkForInvalidEdgeWarning=function(r){var i=r[0]._private.rscratch;i.nodesOverlap||Vr(i.startX)&&Vr(i.startY)&&Vr(i.endX)&&Vr(i.endY)?i.loggedErr=!1:i.loggedErr||(i.loggedErr=!0,tc("Edge `"+r.id()+"` has invalid endpoints and so it is impossible to draw.  Adjust your edge style (e.g. control points) accordingly or use an alternative edge type.  This is expected behaviour when the source node and the target node overlap."))},t0.findEdgeControlPoints=function(r){var i=this;if(!(!r||r.length===0)){for(var o=this,l=o.cy,f=l.hasCompoundNodes(),b={map:new Ym,get:function(H){var Y=this.map.get(H[0]);return Y!=null?Y.get(H[1]):null},set:function(H,Y){var z=this.map.get(H[0]);z==null&&(z=new Ym,this.map.set(H[0],z)),z.set(H[1],Y)}},d=[],w=[],y=0;y<r.length;y++){var k=r[y],E=k._private,T=k.pstyle("curve-style").value;if(!(k.removed()||!k.takesUpSpace())){if(T==="haystack"){w.push(k);continue}var C=T==="unbundled-bezier"||T==="segments"||T==="straight"||T==="straight-triangle"||T==="taxi",S=T==="unbundled-bezier"||T==="bezier",L=E.source,O=E.target,B=L.poolIndex(),N=O.poolIndex(),F=[B,N].sort(),R=b.get(F);R==null&&(R={eles:[]},b.set(F,R),d.push(F)),R.eles.push(k),C&&(R.hasUnbundled=!0),S&&(R.hasBezier=!0)}}for(var q=function(H){var Y=d[H],z=b.get(Y),W=void 0;if(!z.hasUnbundled){var Z=z.eles[0].parallelEdges().filter(function(ot){return ot.isBundledBezier()});nue(z.eles),Z.forEach(function(ot){return z.eles.push(ot)}),z.eles.sort(function(ot,be){return ot.poolIndex()-be.poolIndex()})}var G=z.eles[0],ae=G.source(),$=G.target();if(ae.poolIndex()>$.poolIndex()){var ge=ae;ae=$,$=ge}var ee=z.srcPos=ae.position(),de=z.tgtPos=$.position(),re=z.srcW=ae.outerWidth(),ke=z.srcH=ae.outerHeight(),Ce=z.tgtW=$.outerWidth(),_e=z.tgtH=$.outerHeight(),Te=z.srcShape=o.nodeShapes[i.getNodeShape(ae)],Be=z.tgtShape=o.nodeShapes[i.getNodeShape($)];z.dirCounts={north:0,west:0,south:0,east:0,northwest:0,southwest:0,northeast:0,southeast:0};for(var Ge=0;Ge<z.eles.length;Ge++){var Xe=z.eles[Ge],Ee=Xe[0]._private.rscratch,Ze=Xe.pstyle("curve-style").value,Ie=Ze==="unbundled-bezier"||Ze==="segments"||Ze==="taxi",Oe=!ae.same(Xe.source());if(!z.calculatedIntersection&&ae!==$&&(z.hasBezier||z.hasUnbundled)){z.calculatedIntersection=!0;var Le=Te.intersectLine(ee.x,ee.y,re,ke,de.x,de.y,0),$e=z.srcIntn=Le,xe=Be.intersectLine(de.x,de.y,Ce,_e,ee.x,ee.y,0),Ae=z.tgtIntn=xe,je=z.intersectionPts={x1:Le[0],x2:xe[0],y1:Le[1],y2:xe[1]},me=z.posPts={x1:ee.x,x2:de.x,y1:ee.y,y2:de.y},vt=xe[1]-Le[1],ve=xe[0]-Le[0],Zt=Math.sqrt(ve*ve+vt*vt),nt=z.vector={x:ve,y:vt},xn=z.vectorNorm={x:nt.x/Zt,y:nt.y/Zt},cn={x:-xn.y,y:xn.x};z.nodesOverlap=!Vr(Zt)||Be.checkPoint(Le[0],Le[1],0,Ce,_e,de.x,de.y)||Te.checkPoint(xe[0],xe[1],0,re,ke,ee.x,ee.y),z.vectorNormInverse=cn,W={nodesOverlap:z.nodesOverlap,dirCounts:z.dirCounts,calculatedIntersection:!0,hasBezier:z.hasBezier,hasUnbundled:z.hasUnbundled,eles:z.eles,srcPos:de,tgtPos:ee,srcW:Ce,srcH:_e,tgtW:re,tgtH:ke,srcIntn:Ae,tgtIntn:$e,srcShape:Be,tgtShape:Te,posPts:{x1:me.x2,y1:me.y2,x2:me.x1,y2:me.y1},intersectionPts:{x1:je.x2,y1:je.y2,x2:je.x1,y2:je.y1},vector:{x:-nt.x,y:-nt.y},vectorNorm:{x:-xn.x,y:-xn.y},vectorNormInverse:{x:-cn.x,y:-cn.y}}}var jt=Oe?W:z;Ee.nodesOverlap=jt.nodesOverlap,Ee.srcIntn=jt.srcIntn,Ee.tgtIntn=jt.tgtIntn,f&&(ae.isParent()||ae.isChild()||$.isParent()||$.isChild())&&(ae.parents().anySame($)||$.parents().anySame(ae)||ae.same($)&&ae.isParent())?i.findCompoundLoopPoints(Xe,jt,Ge,Ie):ae===$?i.findLoopPoints(Xe,jt,Ge,Ie):Ze==="segments"?i.findSegmentsPoints(Xe,jt):Ze==="taxi"?i.findTaxiPoints(Xe,jt):Ze==="straight"||!Ie&&z.eles.length%2===1&&Ge===Math.floor(z.eles.length/2)?i.findStraightEdgePoints(Xe):i.findBezierPoints(Xe,jt,Ge,Ie,Oe),i.findEndpoints(Xe),i.tryToCorrectInvalidPoints(Xe,jt),i.checkForInvalidEdgeWarning(Xe),i.storeAllpts(Xe),i.storeEdgeProjections(Xe),i.calculateArrowAngles(Xe),i.recalculateEdgeLabelProjections(Xe),i.calculateLabelAngles(Xe)}},X=0;X<d.length;X++)q(X);this.findHaystackPoints(w)}};function $Pe(r){var i=[];if(r!=null){for(var o=0;o<r.length;o+=2){var l=r[o],f=r[o+1];i.push({x:l,y:f})}return i}}t0.getSegmentPoints=function(r){var i=r[0]._private.rscratch,o=i.edgeType;if(o==="segments")return this.recalculateRenderedStyle(r),$Pe(i.segpts)},t0.getControlPoints=function(r){var i=r[0]._private.rscratch,o=i.edgeType;if(o==="bezier"||o==="multibezier"||o==="self"||o==="compound")return this.recalculateRenderedStyle(r),$Pe(i.ctrlpts)},t0.getEdgeMidpoint=function(r){var i=r[0]._private.rscratch;return this.recalculateRenderedStyle(r),{x:i.midX,y:i.midY}};var NO={};NO.manualEndptToPx=function(r,i){var o=this,l=r.position(),f=r.outerWidth(),b=r.outerHeight();if(i.value.length===2){var d=[i.pfValue[0],i.pfValue[1]];return i.units[0]==="%"&&(d[0]=d[0]*f),i.units[1]==="%"&&(d[1]=d[1]*b),d[0]+=l.x,d[1]+=l.y,d}else{var w=i.pfValue[0];w=-Math.PI/2+w;var y=2*Math.max(f,b),k=[l.x+Math.cos(w)*y,l.y+Math.sin(w)*y];return o.nodeShapes[this.getNodeShape(r)].intersectLine(l.x,l.y,f,b,k[0],k[1],0)}},NO.findEndpoints=function(r){var i=this,o,l=r.source()[0],f=r.target()[0],b=l.position(),d=f.position(),w=r.pstyle("target-arrow-shape").value,y=r.pstyle("source-arrow-shape").value,k=r.pstyle("target-distance-from-node").pfValue,E=r.pstyle("source-distance-from-node").pfValue,T=r.pstyle("curve-style").value,C=r._private.rscratch,S=C.edgeType,L=T==="taxi",O=S==="self"||S==="compound",B=S==="bezier"||S==="multibezier"||O,N=S!=="bezier",F=S==="straight"||S==="segments",R=S==="segments",q=B||N||F,X=O||L,te=r.pstyle("source-endpoint"),H=X?"outside-to-node":te.value,Y=r.pstyle("target-endpoint"),z=X?"outside-to-node":Y.value;C.srcManEndpt=te,C.tgtManEndpt=Y;var W,Z,G,ae;if(B){var $=[C.ctrlpts[0],C.ctrlpts[1]],ge=N?[C.ctrlpts[C.ctrlpts.length-2],C.ctrlpts[C.ctrlpts.length-1]]:$;W=ge,Z=$}else if(F){var ee=R?C.segpts.slice(0,2):[d.x,d.y],de=R?C.segpts.slice(C.segpts.length-2):[b.x,b.y];W=de,Z=ee}if(z==="inside-to-node")o=[d.x,d.y];else if(Y.units)o=this.manualEndptToPx(f,Y);else if(z==="outside-to-line")o=C.tgtIntn;else if(z==="outside-to-node"||z==="outside-to-node-or-label"?G=W:(z==="outside-to-line"||z==="outside-to-line-or-label")&&(G=[b.x,b.y]),o=i.nodeShapes[this.getNodeShape(f)].intersectLine(d.x,d.y,f.outerWidth(),f.outerHeight(),G[0],G[1],0),z==="outside-to-node-or-label"||z==="outside-to-line-or-label"){var re=f._private.rscratch,ke=re.labelWidth,Ce=re.labelHeight,_e=re.labelX,Te=re.labelY,Be=ke/2,Ge=Ce/2,Xe=f.pstyle("text-valign").value;Xe==="top"?Te-=Ge:Xe==="bottom"&&(Te+=Ge);var Ee=f.pstyle("text-halign").value;Ee==="left"?_e-=Be:Ee==="right"&&(_e+=Be);var Ze=yO(G[0],G[1],[_e-Be,Te-Ge,_e+Be,Te-Ge,_e+Be,Te+Ge,_e-Be,Te+Ge],d.x,d.y);if(Ze.length>0){var Ie=b,Oe=y7(Ie,PC(o)),Le=y7(Ie,PC(Ze)),$e=Oe;if(Le<Oe&&(o=Ze,$e=Le),Ze.length>2){var xe=y7(Ie,{x:Ze[2],y:Ze[3]});xe<$e&&(o=[Ze[2],Ze[3]])}}}var Ae=CU(o,W,i.arrowShapes[w].spacing(r)+k),je=CU(o,W,i.arrowShapes[w].gap(r)+k);if(C.endX=je[0],C.endY=je[1],C.arrowEndX=Ae[0],C.arrowEndY=Ae[1],H==="inside-to-node")o=[b.x,b.y];else if(te.units)o=this.manualEndptToPx(l,te);else if(H==="outside-to-line")o=C.srcIntn;else if(H==="outside-to-node"||H==="outside-to-node-or-label"?ae=Z:(H==="outside-to-line"||H==="outside-to-line-or-label")&&(ae=[d.x,d.y]),o=i.nodeShapes[this.getNodeShape(l)].intersectLine(b.x,b.y,l.outerWidth(),l.outerHeight(),ae[0],ae[1],0),H==="outside-to-node-or-label"||H==="outside-to-line-or-label"){var me=l._private.rscratch,vt=me.labelWidth,ve=me.labelHeight,Zt=me.labelX,nt=me.labelY,xn=vt/2,cn=ve/2,jt=l.pstyle("text-valign").value;jt==="top"?nt-=cn:jt==="bottom"&&(nt+=cn);var ot=l.pstyle("text-halign").value;ot==="left"?Zt-=xn:ot==="right"&&(Zt+=xn);var be=yO(ae[0],ae[1],[Zt-xn,nt-cn,Zt+xn,nt-cn,Zt+xn,nt+cn,Zt-xn,nt+cn],b.x,b.y);if(be.length>0){var We=d,ct=y7(We,PC(o)),Yt=y7(We,PC(be)),Ut=ct;if(Yt<ct&&(o=[be[0],be[1]],Ut=Yt),be.length>2){var Wn=y7(We,{x:be[2],y:be[3]});Wn<Ut&&(o=[be[2],be[3]])}}}var Gt=CU(o,Z,i.arrowShapes[y].spacing(r)+E),Rn=CU(o,Z,i.arrowShapes[y].gap(r)+E);C.startX=Rn[0],C.startY=Rn[1],C.arrowStartX=Gt[0],C.arrowStartY=Gt[1],q&&(!Vr(C.startX)||!Vr(C.startY)||!Vr(C.endX)||!Vr(C.endY)?C.badLine=!0:C.badLine=!1)},NO.getSourceEndpoint=function(r){var i=r[0]._private.rscratch;switch(this.recalculateRenderedStyle(r),i.edgeType){case"haystack":return{x:i.haystackPts[0],y:i.haystackPts[1]};default:return{x:i.arrowStartX,y:i.arrowStartY}}},NO.getTargetEndpoint=function(r){var i=r[0]._private.rscratch;switch(this.recalculateRenderedStyle(r),i.edgeType){case"haystack":return{x:i.haystackPts[2],y:i.haystackPts[3]};default:return{x:i.arrowEndX,y:i.arrowEndY}}};var Due={};function _ln(r,i,o){for(var l=function(k,E,T,C){return Kf(k,E,T,C)},f=i._private,b=f.rstyle.bezierPts,d=0;d<r.bezierProjPcts.length;d++){var w=r.bezierProjPcts[d];b.push({x:l(o[0],o[2],o[4],w),y:l(o[1],o[3],o[5],w)})}}Due.storeEdgeProjections=function(r){var i=r._private,o=i.rscratch,l=o.edgeType;if(i.rstyle.bezierPts=null,i.rstyle.linePts=null,i.rstyle.haystackPts=null,l==="multibezier"||l==="bezier"||l==="self"||l==="compound"){i.rstyle.bezierPts=[];for(var f=0;f+5<o.allpts.length;f+=4)_ln(this,r,o.allpts.slice(f,f+6))}else if(l==="segments")for(var b=i.rstyle.linePts=[],f=0;f+1<o.allpts.length;f+=2)b.push({x:o.allpts[f],y:o.allpts[f+1]});else if(l==="haystack"){var d=o.haystackPts;i.rstyle.haystackPts=[{x:d[0],y:d[1]},{x:d[2],y:d[3]}]}i.rstyle.arrowWidth=this.getArrowWidth(r.pstyle("width").pfValue,r.pstyle("arrow-scale").value)*this.arrowShapeWidth},Due.recalculateEdgeProjections=function(r){this.findEdgeControlPoints(r)};var Km={};Km.recalculateNodeLabelProjection=function(r){var i=r.pstyle("label").strValue;if(!y6(i)){var o,l,f=r._private,b=r.width(),d=r.height(),w=r.padding(),y=r.position(),k=r.pstyle("text-halign").strValue,E=r.pstyle("text-valign").strValue,T=f.rscratch,C=f.rstyle;switch(k){case"left":o=y.x-b/2-w;break;case"right":o=y.x+b/2+w;break;default:o=y.x}switch(E){case"top":l=y.y-d/2-w;break;case"bottom":l=y.y+d/2+w;break;default:l=y.y}T.labelX=o,T.labelY=l,C.labelX=o,C.labelY=l,this.calculateLabelAngles(r),this.applyLabelDimensions(r)}};var HPe=function(i,o){var l=Math.atan(o/i);return i===0&&l<0&&(l=l*-1),l},zPe=function(i,o){var l=o.x-i.x,f=o.y-i.y;return HPe(l,f)},Tln=function(i,o,l,f){var b=wO(0,f-.001,1),d=wO(0,f+.001,1),w=BC(i,o,l,b),y=BC(i,o,l,d);return zPe(w,y)};Km.recalculateEdgeLabelProjections=function(r){var i,o=r._private,l=o.rscratch,f=this,b={mid:r.pstyle("label").strValue,source:r.pstyle("source-label").strValue,target:r.pstyle("target-label").strValue};if(!!(b.mid||b.source||b.target)){i={x:l.midX,y:l.midY};var d=function(T,C,S){E6(o.rscratch,T,C,S),E6(o.rstyle,T,C,S)};d("labelX",null,i.x),d("labelY",null,i.y);var w=HPe(l.midDispX,l.midDispY);d("labelAutoAngle",null,w);var y=function E(){if(E.cache)return E.cache;for(var T=[],C=0;C+5<l.allpts.length;C+=4){var S={x:l.allpts[C],y:l.allpts[C+1]},L={x:l.allpts[C+2],y:l.allpts[C+3]},O={x:l.allpts[C+4],y:l.allpts[C+5]};T.push({p0:S,p1:L,p2:O,startDist:0,length:0,segments:[]})}var B=o.rstyle.bezierPts,N=f.bezierProjPcts.length;function F(H,Y,z,W,Z){var G=m7(Y,z),ae=H.segments[H.segments.length-1],$={p0:Y,p1:z,t0:W,t1:Z,startDist:ae?ae.startDist+ae.length:0,length:G};H.segments.push($),H.length+=G}for(var R=0;R<T.length;R++){var q=T[R],X=T[R-1];X&&(q.startDist=X.startDist+X.length),F(q,q.p0,B[R*N],0,f.bezierProjPcts[0]);for(var te=0;te<N-1;te++)F(q,B[R*N+te],B[R*N+te+1],f.bezierProjPcts[te],f.bezierProjPcts[te+1]);F(q,B[R*N+N-1],q.p2,f.bezierProjPcts[N-1],1)}return E.cache=T},k=function(T){var C,S=T==="source";if(!!b[T]){var L=r.pstyle(T+"-text-offset").pfValue;switch(l.edgeType){case"self":case"compound":case"bezier":case"multibezier":{for(var O=y(),B,N=0,F=0,R=0;R<O.length;R++){for(var q=O[S?R:O.length-1-R],X=0;X<q.segments.length;X++){var te=q.segments[S?X:q.segments.length-1-X],H=R===O.length-1&&X===q.segments.length-1;if(N=F,F+=te.length,F>=L||H){B={cp:q,segment:te};break}}if(B)break}var Y=B.cp,z=B.segment,W=(L-N)/z.length,Z=z.t1-z.t0,G=S?z.t0+Z*W:z.t1-Z*W;G=wO(0,G,1),i=BC(Y.p0,Y.p1,Y.p2,G),C=Tln(Y.p0,Y.p1,Y.p2,G);break}case"straight":case"segments":case"haystack":{for(var ae=0,$,ge,ee,de,re=l.allpts.length,ke=0;ke+3<re&&(S?(ee={x:l.allpts[ke],y:l.allpts[ke+1]},de={x:l.allpts[ke+2],y:l.allpts[ke+3]}):(ee={x:l.allpts[re-2-ke],y:l.allpts[re-1-ke]},de={x:l.allpts[re-4-ke],y:l.allpts[re-3-ke]}),$=m7(ee,de),ge=ae,ae+=$,!(ae>=L));ke+=2);var Ce=L-ge,_e=Ce/$;_e=wO(0,_e,1),i=Hon(ee,de,_e),C=zPe(ee,de);break}}d("labelX",T,i.x),d("labelY",T,i.y),d("labelAutoAngle",T,C)}};k("source"),k("target"),this.applyLabelDimensions(r)}},Km.applyLabelDimensions=function(r){this.applyPrefixedLabelDimensions(r),r.isEdge()&&(this.applyPrefixedLabelDimensions(r,"source"),this.applyPrefixedLabelDimensions(r,"target"))},Km.applyPrefixedLabelDimensions=function(r,i){var o=r._private,l=this.getLabelText(r,i),f=this.calculateLabelDimensions(r,l),b=r.pstyle("line-height").pfValue,d=r.pstyle("text-wrap").strValue,w=Gv(o.rscratch,"labelWrapCachedLines",i)||[],y=d!=="wrap"?1:Math.max(w.length,1),k=f.height/y,E=k*b,T=f.width,C=f.height+(y-1)*(b-1)*k;E6(o.rstyle,"labelWidth",i,T),E6(o.rscratch,"labelWidth",i,T),E6(o.rstyle,"labelHeight",i,C),E6(o.rscratch,"labelHeight",i,C),E6(o.rscratch,"labelLineHeight",i,E)},Km.getLabelText=function(r,i){var o=r._private,l=i?i+"-":"",f=r.pstyle(l+"label").strValue,b=r.pstyle("text-transform").value,d=function(Ce,_e){return _e?(E6(o.rscratch,Ce,i,_e),_e):Gv(o.rscratch,Ce,i)};if(!f)return"";b=="none"||(b=="uppercase"?f=f.toUpperCase():b=="lowercase"&&(f=f.toLowerCase()));var w=r.pstyle("text-wrap").value;if(w==="wrap"){var y=d("labelKey");if(y!=null&&d("labelWrapKey")===y)return d("labelWrapCachedText");for(var k="\u200B",E=f.split(`
`),T=r.pstyle("text-max-width").pfValue,C=r.pstyle("text-overflow-wrap").value,S=C==="anywhere",L=[],O=/[\s\u200b]+/,B=S?"":" ",N=0;N<E.length;N++){var F=E[N],R=this.calculateLabelDimensions(r,F),q=R.width;if(S){var X=F.split("").join(k);F=X}if(q>T){for(var te=F.split(O),H="",Y=0;Y<te.length;Y++){var z=te[Y],W=H.length===0?z:H+B+z,Z=this.calculateLabelDimensions(r,W),G=Z.width;G<=T?H+=z+B:(H&&L.push(H),H=z+B)}H.match(/^[\s\u200b]+$/)||L.push(H)}else L.push(F)}d("labelWrapCachedLines",L),f=d("labelWrapCachedText",L.join(`
`)),d("labelWrapKey",y)}else if(w==="ellipsis"){var ae=r.pstyle("text-max-width").pfValue,$="",ge="\u2026",ee=!1;if(this.calculateLabelDimensions(r,f).width<ae)return f;for(var de=0;de<f.length;de++){var re=this.calculateLabelDimensions(r,$+f[de]+ge).width;if(re>ae)break;$+=f[de],de===f.length-1&&(ee=!0)}return ee||($+=ge),$}return f},Km.getLabelJustification=function(r){var i=r.pstyle("text-justification").strValue,o=r.pstyle("text-halign").strValue;if(i==="auto")if(r.isNode())switch(o){case"left":return"right";case"right":return"left";default:return"center"}else return"center";else return i},Km.calculateLabelDimensions=function(r,i){var o=this,l=w7(i,r._private.labelDimsKey),f=o.labelDimCache||(o.labelDimCache=[]),b=f[l];if(b!=null)return b;var d=0,w=r.pstyle("font-style").strValue,y=r.pstyle("font-size").pfValue,k=r.pstyle("font-family").strValue,E=r.pstyle("font-weight").strValue,T=this.labelCalcCanvas,C=this.labelCalcCanvasContext;if(!T){T=this.labelCalcCanvas=document.createElement("canvas"),C=this.labelCalcCanvasContext=T.getContext("2d");var S=T.style;S.position="absolute",S.left="-9999px",S.top="-9999px",S.zIndex="-1",S.visibility="hidden",S.pointerEvents="none"}C.font="".concat(w," ").concat(E," ").concat(y,"px ").concat(k);for(var L=0,O=0,B=i.split(`
`),N=0;N<B.length;N++){var F=B[N],R=C.measureText(F),q=Math.ceil(R.width),X=y;L=Math.max(q,L),O+=X}return L+=d,O+=d,f[l]={width:L,height:O}},Km.calculateLabelAngle=function(r,i){var o=r._private,l=o.rscratch,f=r.isEdge(),b=i?i+"-":"",d=r.pstyle(b+"text-rotation"),w=d.strValue;return w==="none"?0:f&&w==="autorotate"?l.labelAutoAngle:w==="autorotate"?0:d.pfValue},Km.calculateLabelAngles=function(r){var i=this,o=r.isEdge(),l=r._private,f=l.rscratch;f.labelAngle=i.calculateLabelAngle(r),o&&(f.sourceLabelAngle=i.calculateLabelAngle(r,"source"),f.targetLabelAngle=i.calculateLabelAngle(r,"target"))};var GPe={},VPe=28,UPe=!1;GPe.getNodeShape=function(r){var i=this,o=r.pstyle("shape").value;if(o==="cutrectangle"&&(r.width()<VPe||r.height()<VPe))return UPe||(tc("The `cutrectangle` node shape can not be used at small sizes so `rectangle` is used instead"),UPe=!0),"rectangle";if(r.isParent())return o==="rectangle"||o==="roundrectangle"||o==="round-rectangle"||o==="cutrectangle"||o==="cut-rectangle"||o==="barrel"?o:"rectangle";if(o==="polygon"){var l=r.pstyle("shape-polygon-points").value;return i.nodeShapes.makePolygon(l).name}return o};var ZU={};ZU.registerCalculationListeners=function(){var r=this.cy,i=r.collection(),o=this,l=function(d){var w=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;if(i.merge(d),w)for(var y=0;y<d.length;y++){var k=d[y],E=k._private,T=E.rstyle;T.clean=!1,T.cleanConnected=!1}};o.binder(r).on("bounds.* dirty.*",function(d){var w=d.target;l(w)}).on("style.* background.*",function(d){var w=d.target;l(w,!1)});var f=function(d){if(d){var w=o.onUpdateEleCalcsFns;i.cleanStyle();for(var y=0;y<i.length;y++){var k=i[y],E=k._private.rstyle;k.isNode()&&!E.cleanConnected&&(l(k.connectedEdges()),E.cleanConnected=!0)}if(w)for(var T=0;T<w.length;T++){var C=w[T];C(d,i)}o.recalculateRenderedStyle(i),i=r.collection()}};o.flushRenderedStyleQueue=function(){f(!0)},o.beforeRender(f,o.beforeRenderPriorities.eleCalcs)},ZU.onUpdateEleCalcs=function(r){var i=this.onUpdateEleCalcsFns=this.onUpdateEleCalcsFns||[];i.push(r)},ZU.recalculateRenderedStyle=function(r,i){var o=function(q){return q._private.rstyle.cleanConnected},l=[],f=[];if(!this.destroyed){i===void 0&&(i=!0);for(var b=0;b<r.length;b++){var d=r[b],w=d._private,y=w.rstyle;d.isEdge()&&(!o(d.source())||!o(d.target()))&&(y.clean=!1),!(i&&y.clean||d.removed())&&d.pstyle("display").value!=="none"&&(w.group==="nodes"?f.push(d):l.push(d),y.clean=!0)}for(var k=0;k<f.length;k++){var E=f[k],T=E._private,C=T.rstyle,S=E.position();this.recalculateNodeLabelProjection(E),C.nodeX=S.x,C.nodeY=S.y,C.nodeW=E.pstyle("width").pfValue,C.nodeH=E.pstyle("height").pfValue}this.recalculateEdgeProjections(l);for(var L=0;L<l.length;L++){var O=l[L],B=O._private,N=B.rstyle,F=B.rscratch;N.srcX=F.arrowStartX,N.srcY=F.arrowStartY,N.tgtX=F.arrowEndX,N.tgtY=F.arrowEndY,N.midX=F.midX,N.midY=F.midY,N.labelAngle=F.labelAngle,N.sourceLabelAngle=F.sourceLabelAngle,N.targetLabelAngle=F.targetLabelAngle}}};var JU={};JU.updateCachedGrabbedEles=function(){var r=this.cachedZSortedEles;if(!!r){r.drag=[],r.nondrag=[];for(var i=[],o=0;o<r.length;o++){var l=r[o],f=l._private.rscratch;l.grabbed()&&!l.isParent()?i.push(l):f.inDragLayer?r.drag.push(l):r.nondrag.push(l)}for(var o=0;o<i.length;o++){var l=i[o];r.drag.push(l)}}},JU.invalidateCachedZSortedEles=function(){this.cachedZSortedEles=null},JU.getCachedZSortedEles=function(r){if(r||!this.cachedZSortedEles){var i=this.cy.mutableElements().toArray();i.sort(gPe),i.interactive=i.filter(function(o){return o.interactive()}),this.cachedZSortedEles=i,this.updateCachedGrabbedEles()}else i=this.cachedZSortedEles;return i};var qPe={};[T7,QU,t0,NO,Due,Km,GPe,ZU,JU].forEach(function(r){bs(qPe,r)});var YPe={};YPe.getCachedImage=function(r,i,o){var l=this,f=l.imageCache=l.imageCache||{},b=f[r];if(b)return b.image.complete||b.image.addEventListener("load",o),b.image;b=f[r]=f[r]||{};var d=b.image=new Image;d.addEventListener("load",o),d.addEventListener("error",function(){d.error=!0});var w="data:",y=r.substring(0,w.length).toLowerCase()===w;return y||(d.crossOrigin=i),d.src=r,d};var XC={};XC.registerBinding=function(r,i,o,l){var f=Array.prototype.slice.apply(arguments,[1]),b=this.binder(r);return b.on.apply(b,f)},XC.binder=function(r){var i=this,o=r===window||r===document||r===document.body||qan(r);if(i.supportsPassiveEvents==null){var l=!1;try{var f=Object.defineProperty({},"passive",{get:function(){return l=!0,!0}});window.addEventListener("test",null,f)}catch{}i.supportsPassiveEvents=l}var b=function(w,y,k){var E=Array.prototype.slice.call(arguments);return o&&i.supportsPassiveEvents&&(E[2]={capture:k!=null?k:!1,passive:!1,once:!1}),i.bindings.push({target:r,args:E}),(r.addEventListener||r.on).apply(r,E),this};return{on:b,addEventListener:b,addListener:b,bind:b}},XC.nodeIsDraggable=function(r){return r&&r.isNode()&&!r.locked()&&r.grabbable()},XC.nodeIsGrabbable=function(r){return this.nodeIsDraggable(r)&&r.interactive()},XC.load=function(){var r=this,i=function(be){return be.selected()},o=function(be,We,ct,Yt){be==null&&(be=r.cy);for(var Ut=0;Ut<We.length;Ut++){var Wn=We[Ut];be.emit({originalEvent:ct,type:Wn,position:Yt})}},l=function(be){return be.shiftKey||be.metaKey||be.ctrlKey},f=function(be,We){var ct=!0;if(r.cy.hasCompoundNodes()&&be&&be.pannable())for(var Yt=0;We&&Yt<We.length;Yt++){var be=We[Yt];if(be.isNode()&&be.isParent()&&!be.pannable()){ct=!1;break}}else ct=!0;return ct},b=function(be){be[0]._private.grabbed=!0},d=function(be){be[0]._private.grabbed=!1},w=function(be){be[0]._private.rscratch.inDragLayer=!0},y=function(be){be[0]._private.rscratch.inDragLayer=!1},k=function(be){be[0]._private.rscratch.isGrabTarget=!0},E=function(be){be[0]._private.rscratch.isGrabTarget=!1},T=function(be,We){var ct=We.addToList,Yt=ct.has(be);!Yt&&be.grabbable()&&!be.locked()&&(ct.merge(be),b(be))},C=function(be,We){if(!!be.cy().hasCompoundNodes()&&!(We.inDragLayer==null&&We.addToList==null)){var ct=be.descendants();We.inDragLayer&&(ct.forEach(w),ct.connectedEdges().forEach(w)),We.addToList&&T(ct,We)}},S=function(be,We){We=We||{};var ct=be.cy().hasCompoundNodes();We.inDragLayer&&(be.forEach(w),be.neighborhood().stdFilter(function(Yt){return!ct||Yt.isEdge()}).forEach(w)),We.addToList&&be.forEach(function(Yt){T(Yt,We)}),C(be,We),B(be,{inDragLayer:We.inDragLayer}),r.updateCachedGrabbedEles()},L=S,O=function(be){!be||(r.getCachedZSortedEles().forEach(function(We){d(We),y(We),E(We)}),r.updateCachedGrabbedEles())},B=function(be,We){if(!(We.inDragLayer==null&&We.addToList==null)&&!!be.cy().hasCompoundNodes()){var ct=be.ancestors().orphans();if(!ct.same(be)){var Yt=ct.descendants().spawnSelf().merge(ct).unmerge(be).unmerge(be.descendants()),Ut=Yt.connectedEdges();We.inDragLayer&&(Ut.forEach(w),Yt.forEach(w)),We.addToList&&Yt.forEach(function(Wn){T(Wn,We)})}}},N=function(){document.activeElement!=null&&document.activeElement.blur!=null&&document.activeElement.blur()},F=typeof MutationObserver<"u",R=typeof ResizeObserver<"u";F?(r.removeObserver=new MutationObserver(function(ot){for(var be=0;be<ot.length;be++){var We=ot[be],ct=We.removedNodes;if(ct)for(var Yt=0;Yt<ct.length;Yt++){var Ut=ct[Yt];if(Ut===r.container){r.destroy();break}}}}),r.container.parentNode&&r.removeObserver.observe(r.container.parentNode,{childList:!0})):r.registerBinding(r.container,"DOMNodeRemoved",function(ot){r.destroy()});var q=bU.default(function(){r.cy.resize()},100);F&&(r.styleObserver=new MutationObserver(q),r.styleObserver.observe(r.container,{attributes:!0})),r.registerBinding(window,"resize",q),R&&(r.resizeObserver=new ResizeObserver(q),r.resizeObserver.observe(r.container));var X=function(be,We){for(;be!=null;)We(be),be=be.parentNode},te=function(){r.invalidateContainerClientCoordsCache()};X(r.container,function(ot){r.registerBinding(ot,"transitionend",te),r.registerBinding(ot,"animationend",te),r.registerBinding(ot,"scroll",te)}),r.registerBinding(r.container,"contextmenu",function(ot){ot.preventDefault()});var H=function(){return r.selection[4]!==0},Y=function(be){for(var We=r.findContainerClientCoords(),ct=We[0],Yt=We[1],Ut=We[2],Wn=We[3],Gt=be.touches?be.touches:[be],Rn=!1,si=0;si<Gt.length;si++){var $r=Gt[si];if(ct<=$r.clientX&&$r.clientX<=ct+Ut&&Yt<=$r.clientY&&$r.clientY<=Yt+Wn){Rn=!0;break}}if(!Rn)return!1;for(var nr=r.container,Kn=be.target,Jt=Kn.parentNode,en=!1;Jt;){if(Jt===nr){en=!0;break}Jt=Jt.parentNode}return!!en};r.registerBinding(r.container,"mousedown",function(be){if(!!Y(be)){be.preventDefault(),N(),r.hoverData.capture=!0,r.hoverData.which=be.which;var We=r.cy,ct=[be.clientX,be.clientY],Yt=r.projectIntoViewport(ct[0],ct[1]),Ut=r.selection,Wn=r.findNearestElements(Yt[0],Yt[1],!0,!1),Gt=Wn[0],Rn=r.dragData.possibleDragElements;r.hoverData.mdownPos=Yt,r.hoverData.mdownGPos=ct;var si=function(){r.hoverData.tapholdCancelled=!1,clearTimeout(r.hoverData.tapholdTimeout),r.hoverData.tapholdTimeout=setTimeout(function(){if(!r.hoverData.tapholdCancelled){var In=r.hoverData.down;In?In.emit({originalEvent:be,type:"taphold",position:{x:Yt[0],y:Yt[1]}}):We.emit({originalEvent:be,type:"taphold",position:{x:Yt[0],y:Yt[1]}})}},r.tapholdDuration)};if(be.which==3){r.hoverData.cxtStarted=!0;var $r={originalEvent:be,type:"cxttapstart",position:{x:Yt[0],y:Yt[1]}};Gt?(Gt.activate(),Gt.emit($r),r.hoverData.down=Gt):We.emit($r),r.hoverData.downTime=new Date().getTime(),r.hoverData.cxtDragged=!1}else if(be.which==1){Gt&&Gt.activate();{if(Gt!=null&&r.nodeIsGrabbable(Gt)){var nr=function(In){return{originalEvent:be,type:In,position:{x:Yt[0],y:Yt[1]}}},Kn=function(In){In.emit(nr("grab"))};if(k(Gt),!Gt.selected())Rn=r.dragData.possibleDragElements=We.collection(),L(Gt,{addToList:Rn}),Gt.emit(nr("grabon")).emit(nr("grab"));else{Rn=r.dragData.possibleDragElements=We.collection();var Jt=We.$(function(en){return en.isNode()&&en.selected()&&r.nodeIsGrabbable(en)});S(Jt,{addToList:Rn}),Gt.emit(nr("grabon")),Jt.forEach(Kn)}r.redrawHint("eles",!0),r.redrawHint("drag",!0)}r.hoverData.down=Gt,r.hoverData.downs=Wn,r.hoverData.downTime=new Date().getTime()}o(Gt,["mousedown","tapstart","vmousedown"],be,{x:Yt[0],y:Yt[1]}),Gt==null?(Ut[4]=1,r.data.bgActivePosistion={x:Yt[0],y:Yt[1]},r.redrawHint("select",!0),r.redraw()):Gt.pannable()&&(Ut[4]=1),si()}Ut[0]=Ut[2]=Yt[0],Ut[1]=Ut[3]=Yt[1]}},!1),r.registerBinding(window,"mousemove",function(be){var We=r.hoverData.capture;if(!(!We&&!Y(be))){var ct=!1,Yt=r.cy,Ut=Yt.zoom(),Wn=[be.clientX,be.clientY],Gt=r.projectIntoViewport(Wn[0],Wn[1]),Rn=r.hoverData.mdownPos,si=r.hoverData.mdownGPos,$r=r.selection,nr=null;!r.hoverData.draggingEles&&!r.hoverData.dragging&&!r.hoverData.selecting&&(nr=r.findNearestElement(Gt[0],Gt[1],!0,!1));var Kn=r.hoverData.last,Jt=r.hoverData.down,en=[Gt[0]-$r[2],Gt[1]-$r[3]],In=r.dragData.possibleDragElements,hn;if(si){var Fr=Wn[0]-si[0],Pt=Fr*Fr,ei=Wn[1]-si[1],nn=ei*ei,hi=Pt+nn;r.hoverData.isOverThresholdDrag=hn=hi>=r.desktopTapThreshold2}var Hi=l(be);hn&&(r.hoverData.tapholdCancelled=!0);var ss=function(){var Si=r.hoverData.dragDelta=r.hoverData.dragDelta||[];Si.length===0?(Si.push(en[0]),Si.push(en[1])):(Si[0]+=en[0],Si[1]+=en[1])};ct=!0,o(nr,["mousemove","vmousemove","tapdrag"],be,{x:Gt[0],y:Gt[1]});var ls=function(){r.data.bgActivePosistion=void 0,r.hoverData.selecting||Yt.emit({originalEvent:be,type:"boxstart",position:{x:Gt[0],y:Gt[1]}}),$r[4]=1,r.hoverData.selecting=!0,r.redrawHint("select",!0),r.redraw()};if(r.hoverData.which===3){if(hn){var vs={originalEvent:be,type:"cxtdrag",position:{x:Gt[0],y:Gt[1]}};Jt?Jt.emit(vs):Yt.emit(vs),r.hoverData.cxtDragged=!0,(!r.hoverData.cxtOver||nr!==r.hoverData.cxtOver)&&(r.hoverData.cxtOver&&r.hoverData.cxtOver.emit({originalEvent:be,type:"cxtdragout",position:{x:Gt[0],y:Gt[1]}}),r.hoverData.cxtOver=nr,nr&&nr.emit({originalEvent:be,type:"cxtdragover",position:{x:Gt[0],y:Gt[1]}}))}}else if(r.hoverData.dragging){if(ct=!0,Yt.panningEnabled()&&Yt.userPanningEnabled()){var ti;if(r.hoverData.justStartedPan){var zi=r.hoverData.mdownPos;ti={x:(Gt[0]-zi[0])*Ut,y:(Gt[1]-zi[1])*Ut},r.hoverData.justStartedPan=!1}else ti={x:en[0]*Ut,y:en[1]*Ut};Yt.panBy(ti),Yt.emit("dragpan"),r.hoverData.dragged=!0}Gt=r.projectIntoViewport(be.clientX,be.clientY)}else if($r[4]==1&&(Jt==null||Jt.pannable())){if(hn){if(!r.hoverData.dragging&&Yt.boxSelectionEnabled()&&(Hi||!Yt.panningEnabled()||!Yt.userPanningEnabled()))ls();else if(!r.hoverData.selecting&&Yt.panningEnabled()&&Yt.userPanningEnabled()){var as=f(Jt,r.hoverData.downs);as&&(r.hoverData.dragging=!0,r.hoverData.justStartedPan=!0,$r[4]=0,r.data.bgActivePosistion=PC(Rn),r.redrawHint("select",!0),r.redraw())}Jt&&Jt.pannable()&&Jt.active()&&Jt.unactivate()}}else{if(Jt&&Jt.pannable()&&Jt.active()&&Jt.unactivate(),(!Jt||!Jt.grabbed())&&nr!=Kn&&(Kn&&o(Kn,["mouseout","tapdragout"],be,{x:Gt[0],y:Gt[1]}),nr&&o(nr,["mouseover","tapdragover"],be,{x:Gt[0],y:Gt[1]}),r.hoverData.last=nr),Jt)if(hn){if(Yt.boxSelectionEnabled()&&Hi)Jt&&Jt.grabbed()&&(O(In),Jt.emit("freeon"),In.emit("free"),r.dragData.didDrag&&(Jt.emit("dragfreeon"),In.emit("dragfree"))),ls();else if(Jt&&Jt.grabbed()&&r.nodeIsDraggable(Jt)){var ai=!r.dragData.didDrag;ai&&r.redrawHint("eles",!0),r.dragData.didDrag=!0,r.hoverData.draggingEles||S(In,{inDragLayer:!0});var hc={x:0,y:0};if(Vr(en[0])&&Vr(en[1])&&(hc.x+=en[0],hc.y+=en[1],ai)){var xu=r.hoverData.dragDelta;xu&&Vr(xu[0])&&Vr(xu[1])&&(hc.x+=xu[0],hc.y+=xu[1])}r.hoverData.draggingEles=!0,In.silentShift(hc).emit("position drag"),r.redrawHint("drag",!0),r.redraw()}}else ss();ct=!0}if($r[2]=Gt[0],$r[3]=Gt[1],ct)return be.stopPropagation&&be.stopPropagation(),be.preventDefault&&be.preventDefault(),!1}},!1);var z,W,Z;r.registerBinding(window,"mouseup",function(be){var We=r.hoverData.capture;if(!!We){r.hoverData.capture=!1;var ct=r.cy,Yt=r.projectIntoViewport(be.clientX,be.clientY),Ut=r.selection,Wn=r.findNearestElement(Yt[0],Yt[1],!0,!1),Gt=r.dragData.possibleDragElements,Rn=r.hoverData.down,si=l(be);if(r.data.bgActivePosistion&&(r.redrawHint("select",!0),r.redraw()),r.hoverData.tapholdCancelled=!0,r.data.bgActivePosistion=void 0,Rn&&Rn.unactivate(),r.hoverData.which===3){var $r={originalEvent:be,type:"cxttapend",position:{x:Yt[0],y:Yt[1]}};if(Rn?Rn.emit($r):ct.emit($r),!r.hoverData.cxtDragged){var nr={originalEvent:be,type:"cxttap",position:{x:Yt[0],y:Yt[1]}};Rn?Rn.emit(nr):ct.emit(nr)}r.hoverData.cxtDragged=!1,r.hoverData.which=null}else if(r.hoverData.which===1){if(o(Wn,["mouseup","tapend","vmouseup"],be,{x:Yt[0],y:Yt[1]}),!r.dragData.didDrag&&!r.hoverData.dragged&&!r.hoverData.selecting&&!r.hoverData.isOverThresholdDrag&&(o(Rn,["click","tap","vclick"],be,{x:Yt[0],y:Yt[1]}),W=!1,be.timeStamp-Z<=ct.multiClickDebounceTime()?(z&&clearTimeout(z),W=!0,Z=null,o(Rn,["dblclick","dbltap","vdblclick"],be,{x:Yt[0],y:Yt[1]})):(z=setTimeout(function(){W||o(Rn,["oneclick","onetap","voneclick"],be,{x:Yt[0],y:Yt[1]})},ct.multiClickDebounceTime()),Z=be.timeStamp)),Rn==null&&!r.dragData.didDrag&&!r.hoverData.selecting&&!r.hoverData.dragged&&!l(be)&&(ct.$(i).unselect(["tapunselect"]),Gt.length>0&&r.redrawHint("eles",!0),r.dragData.possibleDragElements=Gt=ct.collection()),Wn==Rn&&!r.dragData.didDrag&&!r.hoverData.selecting&&Wn!=null&&Wn._private.selectable&&(r.hoverData.dragging||(ct.selectionType()==="additive"||si?Wn.selected()?Wn.unselect(["tapunselect"]):Wn.select(["tapselect"]):si||(ct.$(i).unmerge(Wn).unselect(["tapunselect"]),Wn.select(["tapselect"]))),r.redrawHint("eles",!0)),r.hoverData.selecting){var Kn=ct.collection(r.getAllInBox(Ut[0],Ut[1],Ut[2],Ut[3]));r.redrawHint("select",!0),Kn.length>0&&r.redrawHint("eles",!0),ct.emit({type:"boxend",originalEvent:be,position:{x:Yt[0],y:Yt[1]}});var Jt=function(hn){return hn.selectable()&&!hn.selected()};ct.selectionType()==="additive"||si||ct.$(i).unmerge(Kn).unselect(),Kn.emit("box").stdFilter(Jt).select().emit("boxselect"),r.redraw()}if(r.hoverData.dragging&&(r.hoverData.dragging=!1,r.redrawHint("select",!0),r.redrawHint("eles",!0),r.redraw()),!Ut[4]){r.redrawHint("drag",!0),r.redrawHint("eles",!0);var en=Rn&&Rn.grabbed();O(Gt),en&&(Rn.emit("freeon"),Gt.emit("free"),r.dragData.didDrag&&(Rn.emit("dragfreeon"),Gt.emit("dragfree")))}}Ut[4]=0,r.hoverData.down=null,r.hoverData.cxtStarted=!1,r.hoverData.draggingEles=!1,r.hoverData.selecting=!1,r.hoverData.isOverThresholdDrag=!1,r.dragData.didDrag=!1,r.hoverData.dragged=!1,r.hoverData.dragDelta=[],r.hoverData.mdownPos=null,r.hoverData.mdownGPos=null}},!1);var G=function(be){if(!r.scrollingPage){var We=r.cy,ct=We.zoom(),Yt=We.pan(),Ut=r.projectIntoViewport(be.clientX,be.clientY),Wn=[Ut[0]*ct+Yt.x,Ut[1]*ct+Yt.y];if(r.hoverData.draggingEles||r.hoverData.dragging||r.hoverData.cxtStarted||H()){be.preventDefault();return}if(We.panningEnabled()&&We.userPanningEnabled()&&We.zoomingEnabled()&&We.userZoomingEnabled()){be.preventDefault(),r.data.wheelZooming=!0,clearTimeout(r.data.wheelTimeout),r.data.wheelTimeout=setTimeout(function(){r.data.wheelZooming=!1,r.redrawHint("eles",!0),r.redraw()},150);var Gt;be.deltaY!=null?Gt=be.deltaY/-250:be.wheelDeltaY!=null?Gt=be.wheelDeltaY/1e3:Gt=be.wheelDelta/1e3,Gt=Gt*r.wheelSensitivity;var Rn=be.deltaMode===1;Rn&&(Gt*=33);var si=We.zoom()*Math.pow(10,Gt);be.type==="gesturechange"&&(si=r.gestureStartZoom*be.scale),We.zoom({level:si,renderedPosition:{x:Wn[0],y:Wn[1]}}),We.emit(be.type==="gesturechange"?"pinchzoom":"scrollzoom")}}};r.registerBinding(r.container,"wheel",G,!0),r.registerBinding(window,"scroll",function(be){r.scrollingPage=!0,clearTimeout(r.scrollingPageTimeout),r.scrollingPageTimeout=setTimeout(function(){r.scrollingPage=!1},250)},!0),r.registerBinding(r.container,"gesturestart",function(be){r.gestureStartZoom=r.cy.zoom(),r.hasTouchStarted||be.preventDefault()},!0),r.registerBinding(r.container,"gesturechange",function(ot){r.hasTouchStarted||G(ot)},!0),r.registerBinding(r.container,"mouseout",function(be){var We=r.projectIntoViewport(be.clientX,be.clientY);r.cy.emit({originalEvent:be,type:"mouseout",position:{x:We[0],y:We[1]}})},!1),r.registerBinding(r.container,"mouseover",function(be){var We=r.projectIntoViewport(be.clientX,be.clientY);r.cy.emit({originalEvent:be,type:"mouseover",position:{x:We[0],y:We[1]}})},!1);var ae,$,ge,ee,de,re,ke,Ce,_e,Te,Be,Ge,Xe,Ee=function(be,We,ct,Yt){return Math.sqrt((ct-be)*(ct-be)+(Yt-We)*(Yt-We))},Ze=function(be,We,ct,Yt){return(ct-be)*(ct-be)+(Yt-We)*(Yt-We)},Ie;r.registerBinding(r.container,"touchstart",Ie=function(be){if(r.hasTouchStarted=!0,!!Y(be)){N(),r.touchData.capture=!0,r.data.bgActivePosistion=void 0;var We=r.cy,ct=r.touchData.now,Yt=r.touchData.earlier;if(be.touches[0]){var Ut=r.projectIntoViewport(be.touches[0].clientX,be.touches[0].clientY);ct[0]=Ut[0],ct[1]=Ut[1]}if(be.touches[1]){var Ut=r.projectIntoViewport(be.touches[1].clientX,be.touches[1].clientY);ct[2]=Ut[0],ct[3]=Ut[1]}if(be.touches[2]){var Ut=r.projectIntoViewport(be.touches[2].clientX,be.touches[2].clientY);ct[4]=Ut[0],ct[5]=Ut[1]}if(be.touches[1]){r.touchData.singleTouchMoved=!0,O(r.dragData.touchDragEles);var Wn=r.findContainerClientCoords();_e=Wn[0],Te=Wn[1],Be=Wn[2],Ge=Wn[3],ae=be.touches[0].clientX-_e,$=be.touches[0].clientY-Te,ge=be.touches[1].clientX-_e,ee=be.touches[1].clientY-Te,Xe=0<=ae&&ae<=Be&&0<=ge&&ge<=Be&&0<=$&&$<=Ge&&0<=ee&&ee<=Ge;var Gt=We.pan(),Rn=We.zoom();de=Ee(ae,$,ge,ee),re=Ze(ae,$,ge,ee),ke=[(ae+ge)/2,($+ee)/2],Ce=[(ke[0]-Gt.x)/Rn,(ke[1]-Gt.y)/Rn];var si=200,$r=si*si;if(re<$r&&!be.touches[2]){var nr=r.findNearestElement(ct[0],ct[1],!0,!0),Kn=r.findNearestElement(ct[2],ct[3],!0,!0);nr&&nr.isNode()?(nr.activate().emit({originalEvent:be,type:"cxttapstart",position:{x:ct[0],y:ct[1]}}),r.touchData.start=nr):Kn&&Kn.isNode()?(Kn.activate().emit({originalEvent:be,type:"cxttapstart",position:{x:ct[0],y:ct[1]}}),r.touchData.start=Kn):We.emit({originalEvent:be,type:"cxttapstart",position:{x:ct[0],y:ct[1]}}),r.touchData.start&&(r.touchData.start._private.grabbed=!1),r.touchData.cxt=!0,r.touchData.cxtDragged=!1,r.data.bgActivePosistion=void 0,r.redraw();return}}if(be.touches[2])We.boxSelectionEnabled()&&be.preventDefault();else if(!be.touches[1]){if(be.touches[0]){var Jt=r.findNearestElements(ct[0],ct[1],!0,!0),en=Jt[0];if(en!=null&&(en.activate(),r.touchData.start=en,r.touchData.starts=Jt,r.nodeIsGrabbable(en))){var In=r.dragData.touchDragEles=We.collection(),hn=null;r.redrawHint("eles",!0),r.redrawHint("drag",!0),en.selected()?(hn=We.$(function(hi){return hi.selected()&&r.nodeIsGrabbable(hi)}),S(hn,{addToList:In})):L(en,{addToList:In}),k(en);var Fr=function(Hi){return{originalEvent:be,type:Hi,position:{x:ct[0],y:ct[1]}}};en.emit(Fr("grabon")),hn?hn.forEach(function(hi){hi.emit(Fr("grab"))}):en.emit(Fr("grab"))}o(en,["touchstart","tapstart","vmousedown"],be,{x:ct[0],y:ct[1]}),en==null&&(r.data.bgActivePosistion={x:Ut[0],y:Ut[1]},r.redrawHint("select",!0),r.redraw()),r.touchData.singleTouchMoved=!1,r.touchData.singleTouchStartTime=+new Date,clearTimeout(r.touchData.tapholdTimeout),r.touchData.tapholdTimeout=setTimeout(function(){r.touchData.singleTouchMoved===!1&&!r.pinching&&!r.touchData.selecting&&o(r.touchData.start,["taphold"],be,{x:ct[0],y:ct[1]})},r.tapholdDuration)}}if(be.touches.length>=1){for(var Pt=r.touchData.startPosition=[],ei=0;ei<ct.length;ei++)Pt[ei]=Yt[ei]=ct[ei];var nn=be.touches[0];r.touchData.startGPosition=[nn.clientX,nn.clientY]}}},!1);var Oe;r.registerBinding(window,"touchmove",Oe=function(be){var We=r.touchData.capture;if(!(!We&&!Y(be))){var ct=r.selection,Yt=r.cy,Ut=r.touchData.now,Wn=r.touchData.earlier,Gt=Yt.zoom();if(be.touches[0]){var Rn=r.projectIntoViewport(be.touches[0].clientX,be.touches[0].clientY);Ut[0]=Rn[0],Ut[1]=Rn[1]}if(be.touches[1]){var Rn=r.projectIntoViewport(be.touches[1].clientX,be.touches[1].clientY);Ut[2]=Rn[0],Ut[3]=Rn[1]}if(be.touches[2]){var Rn=r.projectIntoViewport(be.touches[2].clientX,be.touches[2].clientY);Ut[4]=Rn[0],Ut[5]=Rn[1]}var si=r.touchData.startGPosition,$r;if(We&&be.touches[0]&&si){for(var nr=[],Kn=0;Kn<Ut.length;Kn++)nr[Kn]=Ut[Kn]-Wn[Kn];var Jt=be.touches[0].clientX-si[0],en=Jt*Jt,In=be.touches[0].clientY-si[1],hn=In*In,Fr=en+hn;$r=Fr>=r.touchTapThreshold2}if(We&&r.touchData.cxt){be.preventDefault();var Pt=be.touches[0].clientX-_e,ei=be.touches[0].clientY-Te,nn=be.touches[1].clientX-_e,hi=be.touches[1].clientY-Te,Hi=Ze(Pt,ei,nn,hi),ss=Hi/re,ls=150,vs=ls*ls,ti=1.5,zi=ti*ti;if(ss>=zi||Hi>=vs){r.touchData.cxt=!1,r.data.bgActivePosistion=void 0,r.redrawHint("select",!0);var as={originalEvent:be,type:"cxttapend",position:{x:Ut[0],y:Ut[1]}};r.touchData.start?(r.touchData.start.unactivate().emit(as),r.touchData.start=null):Yt.emit(as)}}if(We&&r.touchData.cxt){var as={originalEvent:be,type:"cxtdrag",position:{x:Ut[0],y:Ut[1]}};r.data.bgActivePosistion=void 0,r.redrawHint("select",!0),r.touchData.start?r.touchData.start.emit(as):Yt.emit(as),r.touchData.start&&(r.touchData.start._private.grabbed=!1),r.touchData.cxtDragged=!0;var ai=r.findNearestElement(Ut[0],Ut[1],!0,!0);(!r.touchData.cxtOver||ai!==r.touchData.cxtOver)&&(r.touchData.cxtOver&&r.touchData.cxtOver.emit({originalEvent:be,type:"cxtdragout",position:{x:Ut[0],y:Ut[1]}}),r.touchData.cxtOver=ai,ai&&ai.emit({originalEvent:be,type:"cxtdragover",position:{x:Ut[0],y:Ut[1]}}))}else if(We&&be.touches[2]&&Yt.boxSelectionEnabled())be.preventDefault(),r.data.bgActivePosistion=void 0,this.lastThreeTouch=+new Date,r.touchData.selecting||Yt.emit({originalEvent:be,type:"boxstart",position:{x:Ut[0],y:Ut[1]}}),r.touchData.selecting=!0,r.touchData.didSelect=!0,ct[4]=1,!ct||ct.length===0||ct[0]===void 0?(ct[0]=(Ut[0]+Ut[2]+Ut[4])/3,ct[1]=(Ut[1]+Ut[3]+Ut[5])/3,ct[2]=(Ut[0]+Ut[2]+Ut[4])/3+1,ct[3]=(Ut[1]+Ut[3]+Ut[5])/3+1):(ct[2]=(Ut[0]+Ut[2]+Ut[4])/3,ct[3]=(Ut[1]+Ut[3]+Ut[5])/3),r.redrawHint("select",!0),r.redraw();else if(We&&be.touches[1]&&!r.touchData.didSelect&&Yt.zoomingEnabled()&&Yt.panningEnabled()&&Yt.userZoomingEnabled()&&Yt.userPanningEnabled()){be.preventDefault(),r.data.bgActivePosistion=void 0,r.redrawHint("select",!0);var hc=r.dragData.touchDragEles;if(hc){r.redrawHint("drag",!0);for(var xu=0;xu<hc.length;xu++){var No=hc[xu]._private;No.grabbed=!1,No.rscratch.inDragLayer=!1}}var Si=r.touchData.start,Pt=be.touches[0].clientX-_e,ei=be.touches[0].clientY-Te,nn=be.touches[1].clientX-_e,hi=be.touches[1].clientY-Te,Yc=Ee(Pt,ei,nn,hi),lh=Yc/de;if(Xe){var su=Pt-ae,Ru=ei-$,fc=nn-ge,Ll=hi-ee,ol=(su+fc)/2,Ur=(Ru+Ll)/2,bt=Yt.zoom(),$n=bt*lh,Er=Yt.pan(),Ss=Ce[0]*bt+Er.x,wo=Ce[1]*bt+Er.y,Po={x:-$n/bt*(Ss-Er.x-ol)+Ss,y:-$n/bt*(wo-Er.y-Ur)+wo};if(Si&&Si.active()){var hc=r.dragData.touchDragEles;O(hc),r.redrawHint("drag",!0),r.redrawHint("eles",!0),Si.unactivate().emit("freeon"),hc.emit("free"),r.dragData.didDrag&&(Si.emit("dragfreeon"),hc.emit("dragfree"))}Yt.viewport({zoom:$n,pan:Po,cancelOnFailedZoom:!0}),Yt.emit("pinchzoom"),de=Yc,ae=Pt,$=ei,ge=nn,ee=hi,r.pinching=!0}if(be.touches[0]){var Rn=r.projectIntoViewport(be.touches[0].clientX,be.touches[0].clientY);Ut[0]=Rn[0],Ut[1]=Rn[1]}if(be.touches[1]){var Rn=r.projectIntoViewport(be.touches[1].clientX,be.touches[1].clientY);Ut[2]=Rn[0],Ut[3]=Rn[1]}if(be.touches[2]){var Rn=r.projectIntoViewport(be.touches[2].clientX,be.touches[2].clientY);Ut[4]=Rn[0],Ut[5]=Rn[1]}}else if(be.touches[0]&&!r.touchData.didSelect){var Ra=r.touchData.start,Eu=r.touchData.last,ai;if(!r.hoverData.draggingEles&&!r.swipePanning&&(ai=r.findNearestElement(Ut[0],Ut[1],!0,!0)),We&&Ra!=null&&be.preventDefault(),We&&Ra!=null&&r.nodeIsDraggable(Ra))if($r){var hc=r.dragData.touchDragEles,Lc=!r.dragData.didDrag;Lc&&S(hc,{inDragLayer:!0}),r.dragData.didDrag=!0;var bd={x:0,y:0};if(Vr(nr[0])&&Vr(nr[1])&&(bd.x+=nr[0],bd.y+=nr[1],Lc)){r.redrawHint("eles",!0);var hh=r.touchData.dragDelta;hh&&Vr(hh[0])&&Vr(hh[1])&&(bd.x+=hh[0],bd.y+=hh[1])}r.hoverData.draggingEles=!0,hc.silentShift(bd).emit("position drag"),r.redrawHint("drag",!0),r.touchData.startPosition[0]==Wn[0]&&r.touchData.startPosition[1]==Wn[1]&&r.redrawHint("eles",!0),r.redraw()}else{var hh=r.touchData.dragDelta=r.touchData.dragDelta||[];hh.length===0?(hh.push(nr[0]),hh.push(nr[1])):(hh[0]+=nr[0],hh[1]+=nr[1])}if(o(Ra||ai,["touchmove","tapdrag","vmousemove"],be,{x:Ut[0],y:Ut[1]}),(!Ra||!Ra.grabbed())&&ai!=Eu&&(Eu&&Eu.emit({originalEvent:be,type:"tapdragout",position:{x:Ut[0],y:Ut[1]}}),ai&&ai.emit({originalEvent:be,type:"tapdragover",position:{x:Ut[0],y:Ut[1]}})),r.touchData.last=ai,We)for(var xu=0;xu<Ut.length;xu++)Ut[xu]&&r.touchData.startPosition[xu]&&$r&&(r.touchData.singleTouchMoved=!0);if(We&&(Ra==null||Ra.pannable())&&Yt.panningEnabled()&&Yt.userPanningEnabled()){var tg=f(Ra,r.touchData.starts);tg&&(be.preventDefault(),r.data.bgActivePosistion||(r.data.bgActivePosistion=PC(r.touchData.startPosition)),r.swipePanning?(Yt.panBy({x:nr[0]*Gt,y:nr[1]*Gt}),Yt.emit("dragpan")):$r&&(r.swipePanning=!0,Yt.panBy({x:Jt*Gt,y:In*Gt}),Yt.emit("dragpan"),Ra&&(Ra.unactivate(),r.redrawHint("select",!0),r.touchData.start=null)));var Rn=r.projectIntoViewport(be.touches[0].clientX,be.touches[0].clientY);Ut[0]=Rn[0],Ut[1]=Rn[1]}}for(var Kn=0;Kn<Ut.length;Kn++)Wn[Kn]=Ut[Kn];We&&be.touches.length>0&&!r.hoverData.draggingEles&&!r.swipePanning&&r.data.bgActivePosistion!=null&&(r.data.bgActivePosistion=void 0,r.redrawHint("select",!0),r.redraw())}},!1);var Le;r.registerBinding(window,"touchcancel",Le=function(be){var We=r.touchData.start;r.touchData.capture=!1,We&&We.unactivate()});var $e,xe,Ae,je;if(r.registerBinding(window,"touchend",$e=function(be){var We=r.touchData.start,ct=r.touchData.capture;if(ct)be.touches.length===0&&(r.touchData.capture=!1),be.preventDefault();else return;var Yt=r.selection;r.swipePanning=!1,r.hoverData.draggingEles=!1;var Ut=r.cy,Wn=Ut.zoom(),Gt=r.touchData.now,Rn=r.touchData.earlier;if(be.touches[0]){var si=r.projectIntoViewport(be.touches[0].clientX,be.touches[0].clientY);Gt[0]=si[0],Gt[1]=si[1]}if(be.touches[1]){var si=r.projectIntoViewport(be.touches[1].clientX,be.touches[1].clientY);Gt[2]=si[0],Gt[3]=si[1]}if(be.touches[2]){var si=r.projectIntoViewport(be.touches[2].clientX,be.touches[2].clientY);Gt[4]=si[0],Gt[5]=si[1]}We&&We.unactivate();var $r;if(r.touchData.cxt){if($r={originalEvent:be,type:"cxttapend",position:{x:Gt[0],y:Gt[1]}},We?We.emit($r):Ut.emit($r),!r.touchData.cxtDragged){var nr={originalEvent:be,type:"cxttap",position:{x:Gt[0],y:Gt[1]}};We?We.emit(nr):Ut.emit(nr)}r.touchData.start&&(r.touchData.start._private.grabbed=!1),r.touchData.cxt=!1,r.touchData.start=null,r.redraw();return}if(!be.touches[2]&&Ut.boxSelectionEnabled()&&r.touchData.selecting){r.touchData.selecting=!1;var Kn=Ut.collection(r.getAllInBox(Yt[0],Yt[1],Yt[2],Yt[3]));Yt[0]=void 0,Yt[1]=void 0,Yt[2]=void 0,Yt[3]=void 0,Yt[4]=0,r.redrawHint("select",!0),Ut.emit({type:"boxend",originalEvent:be,position:{x:Gt[0],y:Gt[1]}});var Jt=function(vs){return vs.selectable()&&!vs.selected()};Kn.emit("box").stdFilter(Jt).select().emit("boxselect"),Kn.nonempty()&&r.redrawHint("eles",!0),r.redraw()}if(We!=null&&We.unactivate(),be.touches[2])r.data.bgActivePosistion=void 0,r.redrawHint("select",!0);else if(!be.touches[1]){if(!be.touches[0]){if(!be.touches[0]){r.data.bgActivePosistion=void 0,r.redrawHint("select",!0);var en=r.dragData.touchDragEles;if(We!=null){var In=We._private.grabbed;O(en),r.redrawHint("drag",!0),r.redrawHint("eles",!0),In&&(We.emit("freeon"),en.emit("free"),r.dragData.didDrag&&(We.emit("dragfreeon"),en.emit("dragfree"))),o(We,["touchend","tapend","vmouseup","tapdragout"],be,{x:Gt[0],y:Gt[1]}),We.unactivate(),r.touchData.start=null}else{var hn=r.findNearestElement(Gt[0],Gt[1],!0,!0);o(hn,["touchend","tapend","vmouseup","tapdragout"],be,{x:Gt[0],y:Gt[1]})}var Fr=r.touchData.startPosition[0]-Gt[0],Pt=Fr*Fr,ei=r.touchData.startPosition[1]-Gt[1],nn=ei*ei,hi=Pt+nn,Hi=hi*Wn*Wn;r.touchData.singleTouchMoved||(We||Ut.$(":selected").unselect(["tapunselect"]),o(We,["tap","vclick"],be,{x:Gt[0],y:Gt[1]}),xe=!1,be.timeStamp-je<=Ut.multiClickDebounceTime()?(Ae&&clearTimeout(Ae),xe=!0,je=null,o(We,["dbltap","vdblclick"],be,{x:Gt[0],y:Gt[1]})):(Ae=setTimeout(function(){xe||o(We,["onetap","voneclick"],be,{x:Gt[0],y:Gt[1]})},Ut.multiClickDebounceTime()),je=be.timeStamp)),We!=null&&!r.dragData.didDrag&&We._private.selectable&&Hi<r.touchTapThreshold2&&!r.pinching&&(Ut.selectionType()==="single"?(Ut.$(i).unmerge(We).unselect(["tapunselect"]),We.select(["tapselect"])):We.selected()?We.unselect(["tapunselect"]):We.select(["tapselect"]),r.redrawHint("eles",!0)),r.touchData.singleTouchMoved=!0}}}for(var ss=0;ss<Gt.length;ss++)Rn[ss]=Gt[ss];r.dragData.didDrag=!1,be.touches.length===0&&(r.touchData.dragDelta=[],r.touchData.startPosition=null,r.touchData.startGPosition=null,r.touchData.didSelect=!1),be.touches.length<2&&(be.touches.length===1&&(r.touchData.startGPosition=[be.touches[0].clientX,be.touches[0].clientY]),r.pinching=!1,r.redrawHint("eles",!0),r.redraw())},!1),typeof TouchEvent>"u"){var me=[],vt=function(be){return{clientX:be.clientX,clientY:be.clientY,force:1,identifier:be.pointerId,pageX:be.pageX,pageY:be.pageY,radiusX:be.width/2,radiusY:be.height/2,screenX:be.screenX,screenY:be.screenY,target:be.target}},ve=function(be){return{event:be,touch:vt(be)}},Zt=function(be){me.push(ve(be))},nt=function(be){for(var We=0;We<me.length;We++){var ct=me[We];if(ct.event.pointerId===be.pointerId){me.splice(We,1);return}}},xn=function(be){var We=me.filter(function(ct){return ct.event.pointerId===be.pointerId})[0];We.event=be,We.touch=vt(be)},cn=function(be){be.touches=me.map(function(We){return We.touch})},jt=function(be){return be.pointerType==="mouse"||be.pointerType===4};r.registerBinding(r.container,"pointerdown",function(ot){jt(ot)||(ot.preventDefault(),Zt(ot),cn(ot),Ie(ot))}),r.registerBinding(r.container,"pointerup",function(ot){jt(ot)||(nt(ot),cn(ot),$e(ot))}),r.registerBinding(r.container,"pointercancel",function(ot){jt(ot)||(nt(ot),cn(ot),Le(ot))}),r.registerBinding(r.container,"pointermove",function(ot){jt(ot)||(ot.preventDefault(),xn(ot),cn(ot),Oe(ot))})}};var b4={};b4.generatePolygon=function(r,i){return this.nodeShapes[r]={renderer:this,name:r,points:i,draw:function(l,f,b,d,w){this.renderer.nodeShapeImpl("polygon",l,f,b,d,w,this.points)},intersectLine:function(l,f,b,d,w,y,k){return yO(w,y,this.points,l,f,b/2,d/2,k)},checkPoint:function(l,f,b,d,w,y,k){return g4(l,f,this.points,y,k,d,w,[0,-1],b)}}},b4.generateEllipse=function(){return this.nodeShapes.ellipse={renderer:this,name:"ellipse",draw:function(i,o,l,f,b){this.renderer.nodeShapeImpl(this.name,i,o,l,f,b)},intersectLine:function(i,o,l,f,b,d,w){return ecn(b,d,i,o,l/2+w,f/2+w)},checkPoint:function(i,o,l,f,b,d,w){return k7(i,o,f,b,d,w,l)}}},b4.generateRoundPolygon=function(r,i){for(var o=new Array(i.length*2),l=0;l<i.length/2;l++){var f=l*2,b=void 0;l<i.length/2-1?b=(l+1)*2:b=0,o[l*4]=i[f],o[l*4+1]=i[f+1];var d=i[b]-i[f],w=i[b+1]-i[f+1],y=Math.sqrt(d*d+w*w);o[l*4+2]=d/y,o[l*4+3]=w/y}return this.nodeShapes[r]={renderer:this,name:r,points:o,draw:function(E,T,C,S,L){this.renderer.nodeShapeImpl("round-polygon",E,T,C,S,L,this.points)},intersectLine:function(E,T,C,S,L,O,B){return tcn(L,O,this.points,E,T,C,S)},checkPoint:function(E,T,C,S,L,O,B){return Jon(E,T,this.points,O,B,S,L)}}},b4.generateRoundRectangle=function(){return this.nodeShapes["round-rectangle"]=this.nodeShapes.roundrectangle={renderer:this,name:"round-rectangle",points:e0(4,0),draw:function(i,o,l,f,b){this.renderer.nodeShapeImpl(this.name,i,o,l,f,b)},intersectLine:function(i,o,l,f,b,d,w){return vNe(b,d,i,o,l,f,w)},checkPoint:function(i,o,l,f,b,d,w){var y=kO(f,b),k=y*2;return!!(g4(i,o,this.points,d,w,f,b-k,[0,-1],l)||g4(i,o,this.points,d,w,f-k,b,[0,-1],l)||k7(i,o,k,k,d-f/2+y,w-b/2+y,l)||k7(i,o,k,k,d+f/2-y,w-b/2+y,l)||k7(i,o,k,k,d+f/2-y,w+b/2-y,l)||k7(i,o,k,k,d-f/2+y,w+b/2-y,l))}}},b4.generateCutRectangle=function(){return this.nodeShapes["cut-rectangle"]=this.nodeShapes.cutrectangle={renderer:this,name:"cut-rectangle",cornerLength:kNe(),points:e0(4,0),draw:function(i,o,l,f,b){this.renderer.nodeShapeImpl(this.name,i,o,l,f,b)},generateCutTrianglePts:function(i,o,l,f){var b=this.cornerLength,d=o/2,w=i/2,y=l-w,k=l+w,E=f-d,T=f+d;return{topLeft:[y,E+b,y+b,E,y+b,E+b],topRight:[k-b,E,k,E+b,k-b,E+b],bottomRight:[k,T-b,k-b,T,k-b,T-b],bottomLeft:[y+b,T,y,T-b,y+b,T-b]}},intersectLine:function(i,o,l,f,b,d,w){var y=this.generateCutTrianglePts(l+2*w,f+2*w,i,o),k=[].concat.apply([],[y.topLeft.splice(0,4),y.topRight.splice(0,4),y.bottomRight.splice(0,4),y.bottomLeft.splice(0,4)]);return yO(b,d,k,i,o)},checkPoint:function(i,o,l,f,b,d,w){if(g4(i,o,this.points,d,w,f,b-2*this.cornerLength,[0,-1],l)||g4(i,o,this.points,d,w,f-2*this.cornerLength,b,[0,-1],l))return!0;var y=this.generateCutTrianglePts(f,b,d,w);return eg(i,o,y.topLeft)||eg(i,o,y.topRight)||eg(i,o,y.bottomRight)||eg(i,o,y.bottomLeft)}}},b4.generateBarrel=function(){return this.nodeShapes.barrel={renderer:this,name:"barrel",points:e0(4,0),draw:function(i,o,l,f,b){this.renderer.nodeShapeImpl(this.name,i,o,l,f,b)},intersectLine:function(i,o,l,f,b,d,w){var y=.15,k=.5,E=.85,T=this.generateBarrelBezierPts(l+2*w,f+2*w,i,o),C=function(O){var B=BC({x:O[0],y:O[1]},{x:O[2],y:O[3]},{x:O[4],y:O[5]},y),N=BC({x:O[0],y:O[1]},{x:O[2],y:O[3]},{x:O[4],y:O[5]},k),F=BC({x:O[0],y:O[1]},{x:O[2],y:O[3]},{x:O[4],y:O[5]},E);return[O[0],O[1],B.x,B.y,N.x,N.y,F.x,F.y,O[4],O[5]]},S=[].concat(C(T.topLeft),C(T.topRight),C(T.bottomRight),C(T.bottomLeft));return yO(b,d,S,i,o)},generateBarrelBezierPts:function(i,o,l,f){var b=o/2,d=i/2,w=l-d,y=l+d,k=f-b,E=f+b,T=lue(i,o),C=T.heightOffset,S=T.widthOffset,L=T.ctrlPtOffsetPct*i,O={topLeft:[w,k+C,w+L,k,w+S,k],topRight:[y-S,k,y-L,k,y,k+C],bottomRight:[y,E-C,y-L,E,y-S,E],bottomLeft:[w+S,E,w+L,E,w,E-C]};return O.topLeft.isTop=!0,O.topRight.isTop=!0,O.bottomLeft.isBottom=!0,O.bottomRight.isBottom=!0,O},checkPoint:function(i,o,l,f,b,d,w){var y=lue(f,b),k=y.heightOffset,E=y.widthOffset;if(g4(i,o,this.points,d,w,f,b-2*k,[0,-1],l)||g4(i,o,this.points,d,w,f-2*E,b,[0,-1],l))return!0;for(var T=this.generateBarrelBezierPts(f,b,d,w),C=function(H,Y,z){var W=z[4],Z=z[2],G=z[0],ae=z[5],$=z[1],ge=Math.min(W,G),ee=Math.max(W,G),de=Math.min(ae,$),re=Math.max(ae,$);if(ge<=H&&H<=ee&&de<=Y&&Y<=re){var ke=ncn(W,Z,G),Ce=Kon(ke[0],ke[1],ke[2],H),_e=Ce.filter(function(Te){return 0<=Te&&Te<=1});if(_e.length>0)return _e[0]}return null},S=Object.keys(T),L=0;L<S.length;L++){var O=S[L],B=T[O],N=C(i,o,B);if(N!=null){var F=B[5],R=B[3],q=B[1],X=Kf(F,R,q,N);if(B.isTop&&X<=o||B.isBottom&&o<=X)return!0}}return!1}}},b4.generateBottomRoundrectangle=function(){return this.nodeShapes["bottom-round-rectangle"]=this.nodeShapes.bottomroundrectangle={renderer:this,name:"bottom-round-rectangle",points:e0(4,0),draw:function(i,o,l,f,b){this.renderer.nodeShapeImpl(this.name,i,o,l,f,b)},intersectLine:function(i,o,l,f,b,d,w){var y=i-(l/2+w),k=o-(f/2+w),E=k,T=i+(l/2+w),C=_6(b,d,i,o,y,k,T,E,!1);return C.length>0?C:vNe(b,d,i,o,l,f,w)},checkPoint:function(i,o,l,f,b,d,w){var y=kO(f,b),k=2*y;if(g4(i,o,this.points,d,w,f,b-k,[0,-1],l)||g4(i,o,this.points,d,w,f-k,b,[0,-1],l))return!0;var E=f/2+2*l,T=b/2+2*l,C=[d-E,w-T,d-E,w,d+E,w,d+E,w-T];return!!(eg(i,o,C)||k7(i,o,k,k,d+f/2-y,w+b/2-y,l)||k7(i,o,k,k,d-f/2+y,w+b/2-y,l))}}},b4.registerNodeShapes=function(){var r=this.nodeShapes={},i=this;this.generateEllipse(),this.generatePolygon("triangle",e0(3,0)),this.generateRoundPolygon("round-triangle",e0(3,0)),this.generatePolygon("rectangle",e0(4,0)),r.square=r.rectangle,this.generateRoundRectangle(),this.generateCutRectangle(),this.generateBarrel(),this.generateBottomRoundrectangle();{var o=[0,1,1,0,0,-1,-1,0];this.generatePolygon("diamond",o),this.generateRoundPolygon("round-diamond",o)}this.generatePolygon("pentagon",e0(5,0)),this.generateRoundPolygon("round-pentagon",e0(5,0)),this.generatePolygon("hexagon",e0(6,0)),this.generateRoundPolygon("round-hexagon",e0(6,0)),this.generatePolygon("heptagon",e0(7,0)),this.generateRoundPolygon("round-heptagon",e0(7,0)),this.generatePolygon("octagon",e0(8,0)),this.generateRoundPolygon("round-octagon",e0(8,0));var l=new Array(20);{var f=cue(5,0),b=cue(5,Math.PI/5),d=.5*(3-Math.sqrt(5));d*=1.57;for(var w=0;w<b.length/2;w++)b[w*2]*=d,b[w*2+1]*=d;for(var w=0;w<20/4;w++)l[w*4]=f[w*2],l[w*4+1]=f[w*2+1],l[w*4+2]=b[w*2],l[w*4+3]=b[w*2+1]}l=yNe(l),this.generatePolygon("star",l),this.generatePolygon("vee",[-1,-1,0,-.333,1,-1,0,1]),this.generatePolygon("rhomboid",[-1,-1,.333,-1,1,1,-.333,1]),this.nodeShapes.concavehexagon=this.generatePolygon("concave-hexagon",[-1,-.95,-.75,0,-1,.95,1,.95,.75,0,1,-.95]);{var y=[-1,-1,.25,-1,1,0,.25,1,-1,1];this.generatePolygon("tag",y),this.generateRoundPolygon("round-tag",y)}r.makePolygon=function(k){var E=k.join("$"),T="polygon-"+E,C;return(C=this[T])?C:i.generatePolygon(T,k)}};var PO={};PO.timeToRender=function(){return this.redrawTotalTime/this.redrawCount},PO.redraw=function(r){r=r||hNe();var i=this;i.averageRedrawTime===void 0&&(i.averageRedrawTime=0),i.lastRedrawTime===void 0&&(i.lastRedrawTime=0),i.lastDrawTime===void 0&&(i.lastDrawTime=0),i.requestedFrame=!0,i.renderOptions=r},PO.beforeRender=function(r,i){if(!this.destroyed){i==null&&Ml("Priority is not optional for beforeRender");var o=this.beforeRenderCallbacks;o.push({fn:r,priority:i}),o.sort(function(l,f){return f.priority-l.priority})}};var WPe=function(i,o,l){for(var f=i.beforeRenderCallbacks,b=0;b<f.length;b++)f[b].fn(o,l)};PO.startRenderLoop=function(){var r=this,i=r.cy;if(!r.renderLoopStarted){r.renderLoopStarted=!0;var o=function l(f){if(!r.destroyed){if(!i.batching())if(r.requestedFrame&&!r.skipFrame){WPe(r,!0,f);var b=d4();r.render(r.renderOptions);var d=r.lastDrawTime=d4();r.averageRedrawTime===void 0&&(r.averageRedrawTime=d-b),r.redrawCount===void 0&&(r.redrawCount=0),r.redrawCount++,r.redrawTotalTime===void 0&&(r.redrawTotalTime=0);var w=d-b;r.redrawTotalTime+=w,r.lastRedrawTime=w,r.averageRedrawTime=r.averageRedrawTime/2+w/2,r.requestedFrame=!1}else WPe(r,!1,f);r.skipFrame=!1,mU(l)}};mU(o)}};var Cln=function(i){this.init(i)},KPe=Cln,QC=KPe.prototype;QC.clientFunctions=["redrawHint","render","renderTo","matchCanvasSize","nodeShapeImpl","arrowShapeImpl"],QC.init=function(r){var i=this;i.options=r,i.cy=r.cy;var o=i.container=r.cy.container();if(Fu){var l=Fu.document,f=l.head,b="__________cytoscape_stylesheet",d="__________cytoscape_container",w=l.getElementById(b)!=null;if(o.className.indexOf(d)<0&&(o.className=(o.className||"")+" "+d),!w){var y=l.createElement("style");y.id=b,y.innerHTML="."+d+" { position: relative; }",f.insertBefore(y,f.children[0])}var k=Fu.getComputedStyle(o),E=k.getPropertyValue("position");E==="static"&&tc("A Cytoscape container has style position:static and so can not use UI extensions properly")}i.selection=[void 0,void 0,void 0,void 0,0],i.bezierProjPcts=[.05,.225,.4,.5,.6,.775,.95],i.hoverData={down:null,last:null,downTime:null,triggerMode:null,dragging:!1,initialPan:[null,null],capture:!1},i.dragData={possibleDragElements:[]},i.touchData={start:null,capture:!1,startPosition:[null,null,null,null,null,null],singleTouchStartTime:null,singleTouchMoved:!0,now:[null,null,null,null,null,null],earlier:[null,null,null,null,null,null]},i.redraws=0,i.showFps=r.showFps,i.debug=r.debug,i.hideEdgesOnViewport=r.hideEdgesOnViewport,i.textureOnViewport=r.textureOnViewport,i.wheelSensitivity=r.wheelSensitivity,i.motionBlurEnabled=r.motionBlur,i.forcedPixelRatio=Vr(r.pixelRatio)?r.pixelRatio:null,i.motionBlur=r.motionBlur,i.motionBlurOpacity=r.motionBlurOpacity,i.motionBlurTransparency=1-i.motionBlurOpacity,i.motionBlurPxRatio=1,i.mbPxRBlurry=1,i.minMbLowQualFrames=4,i.fullQualityMb=!1,i.clearedForMotionBlur=[],i.desktopTapThreshold=r.desktopTapThreshold,i.desktopTapThreshold2=r.desktopTapThreshold*r.desktopTapThreshold,i.touchTapThreshold=r.touchTapThreshold,i.touchTapThreshold2=r.touchTapThreshold*r.touchTapThreshold,i.tapholdDuration=500,i.bindings=[],i.beforeRenderCallbacks=[],i.beforeRenderPriorities={animations:400,eleCalcs:300,eleTxrDeq:200,lyrTxrDeq:150,lyrTxrSkip:100},i.registerNodeShapes(),i.registerArrowShapes(),i.registerCalculationListeners()},QC.notify=function(r,i){var o=this,l=o.cy;if(!this.destroyed){if(r==="init"){o.load();return}if(r==="destroy"){o.destroy();return}(r==="add"||r==="remove"||r==="move"&&l.hasCompoundNodes()||r==="load"||r==="zorder"||r==="mount")&&o.invalidateCachedZSortedEles(),r==="viewport"&&o.redrawHint("select",!0),(r==="load"||r==="resize"||r==="mount")&&(o.invalidateContainerClientCoordsCache(),o.matchCanvasSize(o.container)),o.redrawHint("eles",!0),o.redrawHint("drag",!0),this.startRenderLoop(),this.redraw()}},QC.destroy=function(){var r=this;r.destroyed=!0,r.cy.stopAnimationLoop();for(var i=0;i<r.bindings.length;i++){var o=r.bindings[i],l=o,f=l.target;(f.off||f.removeEventListener).apply(f,l.args)}if(r.bindings=[],r.beforeRenderCallbacks=[],r.onUpdateEleCalcsFns=[],r.removeObserver&&r.removeObserver.disconnect(),r.styleObserver&&r.styleObserver.disconnect(),r.resizeObserver&&r.resizeObserver.disconnect(),r.labelCalcDiv)try{document.body.removeChild(r.labelCalcDiv)}catch{}},QC.isHeadless=function(){return!1},[Mue,qPe,YPe,XC,b4,PO].forEach(function(r){bs(QC,r)});var Lue=1e3/60,XPe={setupDequeueing:function(i){return function(){var l=this,f=this.renderer;if(!l.dequeueingSetup){l.dequeueingSetup=!0;var b=bU.default(function(){f.redrawHint("eles",!0),f.redrawHint("drag",!0),f.redraw()},i.deqRedrawThreshold),d=function(k,E){var T=d4(),C=f.averageRedrawTime,S=f.lastRedrawTime,L=[],O=f.cy.extent(),B=f.getPixelRatio();for(k||f.flushRenderedStyleQueue();;){var N=d4(),F=N-T,R=N-E;if(S<Lue){var q=Lue-(k?C:0);if(R>=i.deqFastCost*q)break}else if(k){if(F>=i.deqCost*S||F>=i.deqAvgCost*C)break}else if(R>=i.deqNoDrawCost*Lue)break;var X=i.deq(l,B,O);if(X.length>0)for(var te=0;te<X.length;te++)L.push(X[te]);else break}L.length>0&&(i.onDeqd(l,L),!k&&i.shouldRedraw(l,L,B,O)&&b())},w=i.priority||tue;f.beforeRender(d,w(l))}}}},Sln=function(){function r(i){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:kU;Kce(this,r),this.idsByKey=new Ym,this.keyForId=new Ym,this.cachesByLvl=new Ym,this.lvls=[],this.getKey=i,this.doesEleInvalidateKey=o}return Xce(r,[{key:"getIdsFor",value:function(o){o==null&&Ml("Can not get id list for null key");var l=this.idsByKey,f=this.idsByKey.get(o);return f||(f=new NC,l.set(o,f)),f}},{key:"addIdForKey",value:function(o,l){o!=null&&this.getIdsFor(o).add(l)}},{key:"deleteIdForKey",value:function(o,l){o!=null&&this.getIdsFor(o).delete(l)}},{key:"getNumberOfIdsForKey",value:function(o){return o==null?0:this.getIdsFor(o).size}},{key:"updateKeyMappingFor",value:function(o){var l=o.id(),f=this.keyForId.get(l),b=this.getKey(o);this.deleteIdForKey(f,l),this.addIdForKey(b,l),this.keyForId.set(l,b)}},{key:"deleteKeyMappingFor",value:function(o){var l=o.id(),f=this.keyForId.get(l);this.deleteIdForKey(f,l),this.keyForId.delete(l)}},{key:"keyHasChangedFor",value:function(o){var l=o.id(),f=this.keyForId.get(l),b=this.getKey(o);return f!==b}},{key:"isInvalid",value:function(o){return this.keyHasChangedFor(o)||this.doesEleInvalidateKey(o)}},{key:"getCachesAt",value:function(o){var l=this.cachesByLvl,f=this.lvls,b=l.get(o);return b||(b=new Ym,l.set(o,b),f.push(o)),b}},{key:"getCache",value:function(o,l){return this.getCachesAt(l).get(o)}},{key:"get",value:function(o,l){var f=this.getKey(o),b=this.getCache(f,l);return b!=null&&this.updateKeyMappingFor(o),b}},{key:"getForCachedKey",value:function(o,l){var f=this.keyForId.get(o.id()),b=this.getCache(f,l);return b}},{key:"hasCache",value:function(o,l){return this.getCachesAt(l).has(o)}},{key:"has",value:function(o,l){var f=this.getKey(o);return this.hasCache(f,l)}},{key:"setCache",value:function(o,l,f){f.key=o,this.getCachesAt(l).set(o,f)}},{key:"set",value:function(o,l,f){var b=this.getKey(o);this.setCache(b,l,f),this.updateKeyMappingFor(o)}},{key:"deleteCache",value:function(o,l){this.getCachesAt(l).delete(o)}},{key:"delete",value:function(o,l){var f=this.getKey(o);this.deleteCache(f,l)}},{key:"invalidateKey",value:function(o){var l=this;this.lvls.forEach(function(f){return l.deleteCache(o,f)})}},{key:"invalidate",value:function(o){var l=o.id(),f=this.keyForId.get(l);this.deleteKeyMappingFor(o);var b=this.doesEleInvalidateKey(o);return b&&this.invalidateKey(f),b||this.getNumberOfIdsForKey(f)===0}}]),r}(),QPe=25,eq=50,tq=-4,Iue=3,Aln=7.99,Mln=8,Dln=1024,Lln=1024,Iln=1024,Oln=.2,Nln=.8,Pln=10,Bln=.15,Fln=.1,Rln=.9,jln=.9,$ln=100,Hln=1,ZC={dequeue:"dequeue",downscale:"downscale",highQuality:"highQuality"},zln=T1({getKey:null,doesEleInvalidateKey:kU,drawElement:null,getBoundingBox:null,getRotationPoint:null,getRotationOffset:null,isVisible:oNe,allowEdgeTxrCaching:!0,allowParentTxrCaching:!0}),BO=function(i,o){var l=this;l.renderer=i,l.onDequeues=[];var f=zln(o);bs(l,f),l.lookup=new Sln(f.getKey,f.doesEleInvalidateKey),l.setupDequeueing()},pf=BO.prototype;pf.reasons=ZC,pf.getTextureQueue=function(r){var i=this;return i.eleImgCaches=i.eleImgCaches||{},i.eleImgCaches[r]=i.eleImgCaches[r]||[]},pf.getRetiredTextureQueue=function(r){var i=this,o=i.eleImgCaches.retired=i.eleImgCaches.retired||{},l=o[r]=o[r]||[];return l},pf.getElementQueue=function(){var r=this,i=r.eleCacheQueue=r.eleCacheQueue||new lO.default(function(o,l){return l.reqs-o.reqs});return i},pf.getElementKeyToQueue=function(){var r=this,i=r.eleKeyToCacheQueue=r.eleKeyToCacheQueue||{};return i},pf.getElement=function(r,i,o,l,f){var b=this,d=this.renderer,w=d.cy.zoom(),y=this.lookup;if(!i||i.w===0||i.h===0||isNaN(i.w)||isNaN(i.h)||!r.visible()||r.removed()||!b.allowEdgeTxrCaching&&r.isEdge()||!b.allowParentTxrCaching&&r.isParent())return null;if(l==null&&(l=Math.ceil(iue(w*o))),l<tq)l=tq;else if(w>=Aln||l>Iue)return null;var k=Math.pow(2,l),E=i.h*k,T=i.w*k,C=d.eleTextBiggerThanMin(r,k);if(!this.isVisible(r,C))return null;var S=y.get(r,l);if(S&&S.invalidated&&(S.invalidated=!1,S.texture.invalidatedWidth-=S.width),S)return S;var L;if(E<=QPe?L=QPe:E<=eq?L=eq:L=Math.ceil(E/eq)*eq,E>Iln||T>Lln)return null;var O=b.getTextureQueue(L),B=O[O.length-2],N=function(){return b.recycleTexture(L,T)||b.addTexture(L,T)};B||(B=O[O.length-1]),B||(B=N()),B.width-B.usedWidth<T&&(B=N());for(var F=function(ee){return ee&&ee.scaledLabelShown===C},R=f&&f===ZC.dequeue,q=f&&f===ZC.highQuality,X=f&&f===ZC.downscale,te,H=l+1;H<=Iue;H++){var Y=y.get(r,H);if(Y){te=Y;break}}var z=te&&te.level===l+1?te:null,W=function(){B.context.drawImage(z.texture.canvas,z.x,0,z.width,z.height,B.usedWidth,0,T,E)};if(B.context.setTransform(1,0,0,1,0,0),B.context.clearRect(B.usedWidth,0,T,L),F(z))W();else if(F(te))if(q){for(var Z=te.level;Z>l;Z--)z=b.getElement(r,i,o,Z,ZC.downscale);W()}else return b.queueElement(r,te.level-1),te;else{var G;if(!R&&!q&&!X)for(var ae=l-1;ae>=tq;ae--){var $=y.get(r,ae);if($){G=$;break}}if(F(G))return b.queueElement(r,l),G;B.context.translate(B.usedWidth,0),B.context.scale(k,k),this.drawElement(B.context,r,i,C,!1),B.context.scale(1/k,1/k),B.context.translate(-B.usedWidth,0)}return S={x:B.usedWidth,texture:B,level:l,scale:k,width:T,height:E,scaledLabelShown:C},B.usedWidth+=Math.ceil(T+Mln),B.eleCaches.push(S),y.set(r,l,S),b.checkTextureFullness(B),S},pf.invalidateElements=function(r){for(var i=0;i<r.length;i++)this.invalidateElement(r[i])},pf.invalidateElement=function(r){var i=this,o=i.lookup,l=[],f=o.isInvalid(r);if(!!f){for(var b=tq;b<=Iue;b++){var d=o.getForCachedKey(r,b);d&&l.push(d)}var w=o.invalidate(r);if(w)for(var y=0;y<l.length;y++){var k=l[y],E=k.texture;E.invalidatedWidth+=k.width,k.invalidated=!0,i.checkTextureUtility(E)}i.removeFromQueue(r)}},pf.checkTextureUtility=function(r){r.invalidatedWidth>=Oln*r.width&&this.retireTexture(r)},pf.checkTextureFullness=function(r){var i=this,o=i.getTextureQueue(r.height);r.usedWidth/r.width>Nln&&r.fullnessChecks>=Pln?x6(o,r):r.fullnessChecks++},pf.retireTexture=function(r){var i=this,o=r.height,l=i.getTextureQueue(o),f=this.lookup;x6(l,r),r.retired=!0;for(var b=r.eleCaches,d=0;d<b.length;d++){var w=b[d];f.deleteCache(w.key,w.level)}nue(b);var y=i.getRetiredTextureQueue(o);y.push(r)},pf.addTexture=function(r,i){var o=this,l=o.getTextureQueue(r),f={};return l.push(f),f.eleCaches=[],f.height=r,f.width=Math.max(Dln,i),f.usedWidth=0,f.invalidatedWidth=0,f.fullnessChecks=0,f.canvas=o.renderer.makeOffscreenCanvas(f.width,f.height),f.context=f.canvas.getContext("2d"),f},pf.recycleTexture=function(r,i){for(var o=this,l=o.getTextureQueue(r),f=o.getRetiredTextureQueue(r),b=0;b<f.length;b++){var d=f[b];if(d.width>=i)return d.retired=!1,d.usedWidth=0,d.invalidatedWidth=0,d.fullnessChecks=0,nue(d.eleCaches),d.context.setTransform(1,0,0,1,0,0),d.context.clearRect(0,0,d.width,d.height),x6(f,d),l.push(d),d}},pf.queueElement=function(r,i){var o=this,l=o.getElementQueue(),f=o.getElementKeyToQueue(),b=this.getKey(r),d=f[b];if(d)d.level=Math.max(d.level,i),d.eles.merge(r),d.reqs++,l.updateItem(d);else{var w={eles:r.spawn().merge(r),level:i,reqs:1,key:b};l.push(w),f[b]=w}},pf.dequeue=function(r){for(var i=this,o=i.getElementQueue(),l=i.getElementKeyToQueue(),f=[],b=i.lookup,d=0;d<Hln&&o.size()>0;d++){var w=o.pop(),y=w.key,k=w.eles[0],E=b.hasCache(k,w.level);if(l[y]=null,E)continue;f.push(w);var T=i.getBoundingBox(k);i.getElement(k,T,r,w.level,ZC.dequeue)}return f},pf.removeFromQueue=function(r){var i=this,o=i.getElementQueue(),l=i.getElementKeyToQueue(),f=this.getKey(r),b=l[f];b!=null&&(b.eles.length===1?(b.reqs=eue,o.updateItem(b),o.pop(),l[f]=null):b.eles.unmerge(r))},pf.onDequeue=function(r){this.onDequeues.push(r)},pf.offDequeue=function(r){x6(this.onDequeues,r)},pf.setupDequeueing=XPe.setupDequeueing({deqRedrawThreshold:$ln,deqCost:Bln,deqAvgCost:Fln,deqNoDrawCost:Rln,deqFastCost:jln,deq:function(i,o,l){return i.dequeue(o,l)},onDeqd:function(i,o){for(var l=0;l<i.onDequeues.length;l++){var f=i.onDequeues[l];f(o)}},shouldRedraw:function(i,o,l,f){for(var b=0;b<o.length;b++)for(var d=o[b].eles,w=0;w<d.length;w++){var y=d[w].boundingBox();if(aue(y,f))return!0}return!1},priority:function(i){return i.renderer.beforeRenderPriorities.eleTxrDeq}});var Gln=1,FO=-4,nq=2,Vln=3.99,Uln=50,qln=50,Yln=.15,Wln=.1,Kln=.9,Xln=.9,Qln=1,ZPe=250,Zln=4e3*4e3,Jln=!0,JPe=function(i){var o=this,l=o.renderer=i,f=l.cy;o.layersByLevel={},o.firstGet=!0,o.lastInvalidationTime=d4()-2*ZPe,o.skipping=!1,o.eleTxrDeqs=f.collection(),o.scheduleElementRefinement=bU.default(function(){o.refineElementTextures(o.eleTxrDeqs),o.eleTxrDeqs.unmerge(o.eleTxrDeqs)},qln),l.beforeRender(function(d,w){w-o.lastInvalidationTime<=ZPe?o.skipping=!0:o.skipping=!1},l.beforeRenderPriorities.lyrTxrSkip);var b=function(w,y){return y.reqs-w.reqs};o.layersQueue=new lO.default(b),o.setupDequeueing()},M1=JPe.prototype,eBe=0,ehn=Math.pow(2,53)-1;M1.makeLayer=function(r,i){var o=Math.pow(2,i),l=Math.ceil(r.w*o),f=Math.ceil(r.h*o),b=this.renderer.makeOffscreenCanvas(l,f),d={id:eBe=++eBe%ehn,bb:r,level:i,width:l,height:f,canvas:b,context:b.getContext("2d"),eles:[],elesQueue:[],reqs:0},w=d.context,y=-d.bb.x1,k=-d.bb.y1;return w.scale(o,o),w.translate(y,k),d},M1.getLayers=function(r,i,o){var l=this,f=l.renderer,b=f.cy,d=b.zoom(),w=l.firstGet;if(l.firstGet=!1,o==null){if(o=Math.ceil(iue(d*i)),o<FO)o=FO;else if(d>=Vln||o>nq)return null}l.validateLayersElesOrdering(o,r);var y=l.layersByLevel,k=Math.pow(2,o),E=y[o]=y[o]||[],T,C=l.levelIsComplete(o,r),S,L=function(){var W=function(ge){if(l.validateLayersElesOrdering(ge,r),l.levelIsComplete(ge,r))return S=y[ge],!0},Z=function(ge){if(!S)for(var ee=o+ge;FO<=ee&&ee<=nq&&!W(ee);ee+=ge);};Z(1),Z(-1);for(var G=E.length-1;G>=0;G--){var ae=E[G];ae.invalid&&x6(E,ae)}};if(!C)L();else return E;var O=function(){if(!T){T=J0();for(var W=0;W<r.length;W++)Von(T,r[W].boundingBox())}return T},B=function(W){W=W||{};var Z=W.after;O();var G=T.w*k*(T.h*k);if(G>Zln)return null;var ae=l.makeLayer(T,o);if(Z!=null){var $=E.indexOf(Z)+1;E.splice($,0,ae)}else(W.insert===void 0||W.insert)&&E.unshift(ae);return ae};if(l.skipping&&!w)return null;for(var N=null,F=r.length/Gln,R=!w,q=0;q<r.length;q++){var X=r[q],te=X._private.rscratch,H=te.imgLayerCaches=te.imgLayerCaches||{},Y=H[o];if(Y){N=Y;continue}if((!N||N.eles.length>=F||!bNe(N.bb,X.boundingBox()))&&(N=B({insert:!0,after:N}),!N))return null;S||R?l.queueLayer(N,X):l.drawEleInLayer(N,X,o,i),N.eles.push(X),H[o]=N}return S||(R?null:E)},M1.getEleLevelForLayerLevel=function(r,i){return r},M1.drawEleInLayer=function(r,i,o,l){var f=this,b=this.renderer,d=r.context,w=i.boundingBox();w.w===0||w.h===0||!i.visible()||(o=f.getEleLevelForLayerLevel(o,l),b.setImgSmoothing(d,!1),b.drawCachedElement(d,i,null,null,o,Jln),b.setImgSmoothing(d,!0))},M1.levelIsComplete=function(r,i){var o=this,l=o.layersByLevel[r];if(!l||l.length===0)return!1;for(var f=0,b=0;b<l.length;b++){var d=l[b];if(d.reqs>0||d.invalid)return!1;f+=d.eles.length}return f===i.length},M1.validateLayersElesOrdering=function(r,i){var o=this.layersByLevel[r];if(!!o)for(var l=0;l<o.length;l++){for(var f=o[l],b=-1,d=0;d<i.length;d++)if(f.eles[0]===i[d]){b=d;break}if(b<0){this.invalidateLayer(f);continue}for(var w=b,d=0;d<f.eles.length;d++)if(f.eles[d]!==i[w+d]){this.invalidateLayer(f);break}}},M1.updateElementsInLayers=function(r,i){for(var o=this,l=fO(r[0]),f=0;f<r.length;f++)for(var b=l?null:r[f],d=l?r[f]:r[f].ele,w=d._private.rscratch,y=w.imgLayerCaches=w.imgLayerCaches||{},k=FO;k<=nq;k++){var E=y[k];!E||b&&o.getEleLevelForLayerLevel(E.level)!==b.level||i(E,d,b)}},M1.haveLayers=function(){for(var r=this,i=!1,o=FO;o<=nq;o++){var l=r.layersByLevel[o];if(l&&l.length>0){i=!0;break}}return i},M1.invalidateElements=function(r){var i=this;r.length!==0&&(i.lastInvalidationTime=d4(),!(r.length===0||!i.haveLayers())&&i.updateElementsInLayers(r,function(l,f,b){i.invalidateLayer(l)}))},M1.invalidateLayer=function(r){if(this.lastInvalidationTime=d4(),!r.invalid){var i=r.level,o=r.eles,l=this.layersByLevel[i];x6(l,r),r.elesQueue=[],r.invalid=!0,r.replacement&&(r.replacement.invalid=!0);for(var f=0;f<o.length;f++){var b=o[f]._private.rscratch.imgLayerCaches;b&&(b[i]=null)}}},M1.refineElementTextures=function(r){var i=this;i.updateElementsInLayers(r,function(l,f,b){var d=l.replacement;if(d||(d=l.replacement=i.makeLayer(l.bb,l.level),d.replaces=l,d.eles=l.eles),!d.reqs)for(var w=0;w<d.eles.length;w++)i.queueLayer(d,d.eles[w])})},M1.enqueueElementRefinement=function(r){this.eleTxrDeqs.merge(r),this.scheduleElementRefinement()},M1.queueLayer=function(r,i){var o=this,l=o.layersQueue,f=r.elesQueue,b=f.hasId=f.hasId||{};if(!r.replacement){if(i){if(b[i.id()])return;f.push(i),b[i.id()]=!0}r.reqs?(r.reqs++,l.updateItem(r)):(r.reqs=1,l.push(r))}},M1.dequeue=function(r){for(var i=this,o=i.layersQueue,l=[],f=0;f<Qln&&o.size()!==0;){var b=o.peek();if(b.replacement){o.pop();continue}if(b.replaces&&b!==b.replaces.replacement){o.pop();continue}if(b.invalid){o.pop();continue}var d=b.elesQueue.shift();d&&(i.drawEleInLayer(b,d,b.level,r),f++),l.length===0&&l.push(!0),b.elesQueue.length===0&&(o.pop(),b.reqs=0,b.replaces&&i.applyLayerReplacement(b),i.requestRedraw())}return l},M1.applyLayerReplacement=function(r){var i=this,o=i.layersByLevel[r.level],l=r.replaces,f=o.indexOf(l);if(!(f<0||l.invalid)){o[f]=r;for(var b=0;b<r.eles.length;b++){var d=r.eles[b]._private,w=d.imgLayerCaches=d.imgLayerCaches||{};w&&(w[r.level]=r)}i.requestRedraw()}},M1.requestRedraw=bU.default(function(){var r=this.renderer;r.redrawHint("eles",!0),r.redrawHint("drag",!0),r.redraw()},100),M1.setupDequeueing=XPe.setupDequeueing({deqRedrawThreshold:Uln,deqCost:Yln,deqAvgCost:Wln,deqNoDrawCost:Kln,deqFastCost:Xln,deq:function(i,o){return i.dequeue(o)},onDeqd:tue,shouldRedraw:oNe,priority:function(i){return i.renderer.beforeRenderPriorities.lyrTxrDeq}});var tBe={},nBe;function thn(r,i){for(var o=0;o<i.length;o++){var l=i[o];r.lineTo(l.x,l.y)}}function nhn(r,i,o){for(var l,f=0;f<i.length;f++){var b=i[f];f===0&&(l=b),r.lineTo(b.x,b.y)}r.quadraticCurveTo(o.x,o.y,l.x,l.y)}function rBe(r,i,o){r.beginPath&&r.beginPath();for(var l=i,f=0;f<l.length;f++){var b=l[f];r.lineTo(b.x,b.y)}var d=o,w=o[0];r.moveTo(w.x,w.y);for(var f=1;f<d.length;f++){var b=d[f];r.lineTo(b.x,b.y)}r.closePath&&r.closePath()}function rhn(r,i,o,l,f){r.beginPath&&r.beginPath(),r.arc(o,l,f,0,Math.PI*2,!1);var b=i,d=b[0];r.moveTo(d.x,d.y);for(var w=0;w<b.length;w++){var y=b[w];r.lineTo(y.x,y.y)}r.closePath&&r.closePath()}function ihn(r,i,o,l){r.arc(i,o,l,0,Math.PI*2,!1)}tBe.arrowShapeImpl=function(r){return(nBe||(nBe={polygon:thn,"triangle-backcurve":nhn,"triangle-tee":rBe,"circle-triangle":rhn,"triangle-cross":rBe,circle:ihn}))[r]};var Xm={};Xm.drawElement=function(r,i,o,l,f,b){var d=this;i.isNode()?d.drawNode(r,i,o,l,f,b):d.drawEdge(r,i,o,l,f,b)},Xm.drawElementOverlay=function(r,i){var o=this;i.isNode()?o.drawNodeOverlay(r,i):o.drawEdgeOverlay(r,i)},Xm.drawElementUnderlay=function(r,i){var o=this;i.isNode()?o.drawNodeUnderlay(r,i):o.drawEdgeUnderlay(r,i)},Xm.drawCachedElementPortion=function(r,i,o,l,f,b,d,w){var y=this,k=o.getBoundingBox(i);if(!(k.w===0||k.h===0)){var E=o.getElement(i,k,l,f,b);if(E!=null){var T=w(y,i);if(T===0)return;var C=d(y,i),S=k.x1,L=k.y1,O=k.w,B=k.h,N,F,R,q,X;if(C!==0){var te=o.getRotationPoint(i);R=te.x,q=te.y,r.translate(R,q),r.rotate(C),X=y.getImgSmoothing(r),X||y.setImgSmoothing(r,!0);var H=o.getRotationOffset(i);N=H.x,F=H.y}else N=S,F=L;var Y;T!==1&&(Y=r.globalAlpha,r.globalAlpha=Y*T),r.drawImage(E.texture.canvas,E.x,0,E.width,E.height,N,F,O,B),T!==1&&(r.globalAlpha=Y),C!==0&&(r.rotate(-C),r.translate(-R,-q),X||y.setImgSmoothing(r,!1))}else o.drawElement(r,i)}};var shn=function(){return 0},ahn=function(i,o){return i.getTextAngle(o,null)},ohn=function(i,o){return i.getTextAngle(o,"source")},chn=function(i,o){return i.getTextAngle(o,"target")},uhn=function(i,o){return o.effectiveOpacity()},Oue=function(i,o){return o.pstyle("text-opacity").pfValue*o.effectiveOpacity()};Xm.drawCachedElement=function(r,i,o,l,f,b){var d=this,w=d.data,y=w.eleTxrCache,k=w.lblTxrCache,E=w.slbTxrCache,T=w.tlbTxrCache,C=i.boundingBox(),S=b===!0?y.reasons.highQuality:null;if(!(C.w===0||C.h===0||!i.visible())&&(!l||aue(C,l))){var L=i.isEdge(),O=i.element()._private.rscratch.badLine;d.drawElementUnderlay(r,i),d.drawCachedElementPortion(r,i,y,o,f,S,shn,uhn),(!L||!O)&&d.drawCachedElementPortion(r,i,k,o,f,S,ahn,Oue),L&&!O&&(d.drawCachedElementPortion(r,i,E,o,f,S,ohn,Oue),d.drawCachedElementPortion(r,i,T,o,f,S,chn,Oue)),d.drawElementOverlay(r,i)}},Xm.drawElements=function(r,i){for(var o=this,l=0;l<i.length;l++){var f=i[l];o.drawElement(r,f)}},Xm.drawCachedElements=function(r,i,o,l){for(var f=this,b=0;b<i.length;b++){var d=i[b];f.drawCachedElement(r,d,o,l)}},Xm.drawCachedNodes=function(r,i,o,l){for(var f=this,b=0;b<i.length;b++){var d=i[b];!d.isNode()||f.drawCachedElement(r,d,o,l)}},Xm.drawLayeredElements=function(r,i,o,l){var f=this,b=f.data.lyrTxrCache.getLayers(i,o);if(b)for(var d=0;d<b.length;d++){var w=b[d],y=w.bb;y.w===0||y.h===0||r.drawImage(w.canvas,y.x1,y.y1,y.w,y.h)}else f.drawCachedElements(r,i,o,l)};var v4={};v4.drawEdge=function(r,i,o){var l=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!0,f=arguments.length>4&&arguments[4]!==void 0?arguments[4]:!0,b=arguments.length>5&&arguments[5]!==void 0?arguments[5]:!0,d=this,w=i._private.rscratch;if(!(b&&!i.visible())&&!(w.badLine||w.allpts==null||isNaN(w.allpts[0]))){var y;o&&(y=o,r.translate(-y.x1,-y.y1));var k=b?i.pstyle("opacity").value:1,E=b?i.pstyle("line-opacity").value:1,T=i.pstyle("curve-style").value,C=i.pstyle("line-style").value,S=i.pstyle("width").pfValue,L=i.pstyle("line-cap").value,O=k*E,B=k*E,N=function(){var G=arguments.length>0&&arguments[0]!==void 0?arguments[0]:O;T==="straight-triangle"?(d.eleStrokeStyle(r,i,G),d.drawEdgeTrianglePath(i,r,w.allpts)):(r.lineWidth=S,r.lineCap=L,d.eleStrokeStyle(r,i,G),d.drawEdgePath(i,r,w.allpts,C),r.lineCap="butt")},F=function(){!f||d.drawEdgeOverlay(r,i)},R=function(){!f||d.drawEdgeUnderlay(r,i)},q=function(){var G=arguments.length>0&&arguments[0]!==void 0?arguments[0]:B;d.drawArrowheads(r,i,G)},X=function(){d.drawElementText(r,i,null,l)};r.lineJoin="round";var te=i.pstyle("ghost").value==="yes";if(te){var H=i.pstyle("ghost-offset-x").pfValue,Y=i.pstyle("ghost-offset-y").pfValue,z=i.pstyle("ghost-opacity").value,W=O*z;r.translate(H,Y),N(W),q(W),r.translate(-H,-Y)}R(),N(),q(),F(),X(),o&&r.translate(y.x1,y.y1)}};var iBe=function(i){if(!["overlay","underlay"].includes(i))throw new Error("Invalid state");return function(o,l){if(!!l.visible()){var f=l.pstyle("".concat(i,"-opacity")).value;if(f!==0){var b=this,d=b.usePaths(),w=l._private.rscratch,y=l.pstyle("".concat(i,"-padding")).pfValue,k=2*y,E=l.pstyle("".concat(i,"-color")).value;o.lineWidth=k,w.edgeType==="self"&&!d?o.lineCap="butt":o.lineCap="round",b.colorStrokeStyle(o,E[0],E[1],E[2],f),b.drawEdgePath(l,o,w.allpts,"solid")}}}};v4.drawEdgeOverlay=iBe("overlay"),v4.drawEdgeUnderlay=iBe("underlay"),v4.drawEdgePath=function(r,i,o,l){var f=r._private.rscratch,b=i,d,w=!1,y=this.usePaths(),k=r.pstyle("line-dash-pattern").pfValue,E=r.pstyle("line-dash-offset").pfValue;if(y){var T=o.join("$"),C=f.pathCacheKey&&f.pathCacheKey===T;C?(d=i=f.pathCache,w=!0):(d=i=new Path2D,f.pathCacheKey=T,f.pathCache=d)}if(b.setLineDash)switch(l){case"dotted":b.setLineDash([1,1]);break;case"dashed":b.setLineDash(k),b.lineDashOffset=E;break;case"solid":b.setLineDash([]);break}if(!w&&!f.badLine)switch(i.beginPath&&i.beginPath(),i.moveTo(o[0],o[1]),f.edgeType){case"bezier":case"self":case"compound":case"multibezier":for(var S=2;S+3<o.length;S+=4)i.quadraticCurveTo(o[S],o[S+1],o[S+2],o[S+3]);break;case"straight":case"segments":case"haystack":for(var L=2;L+1<o.length;L+=2)i.lineTo(o[L],o[L+1]);break}i=b,y?i.stroke(d):i.stroke(),i.setLineDash&&i.setLineDash([])},v4.drawEdgeTrianglePath=function(r,i,o){i.fillStyle=i.strokeStyle;for(var l=r.pstyle("width").pfValue,f=0;f+1<o.length;f+=2){var b=[o[f+2]-o[f],o[f+3]-o[f+1]],d=Math.sqrt(b[0]*b[0]+b[1]*b[1]),w=[b[1]/d,-b[0]/d],y=[w[0]*l/2,w[1]*l/2];i.beginPath(),i.moveTo(o[f]-y[0],o[f+1]-y[1]),i.lineTo(o[f]+y[0],o[f+1]+y[1]),i.lineTo(o[f+2],o[f+3]),i.closePath(),i.fill()}},v4.drawArrowheads=function(r,i,o){var l=i._private.rscratch,f=l.edgeType==="haystack";f||this.drawArrowhead(r,i,"source",l.arrowStartX,l.arrowStartY,l.srcArrowAngle,o),this.drawArrowhead(r,i,"mid-target",l.midX,l.midY,l.midtgtArrowAngle,o),this.drawArrowhead(r,i,"mid-source",l.midX,l.midY,l.midsrcArrowAngle,o),f||this.drawArrowhead(r,i,"target",l.arrowEndX,l.arrowEndY,l.tgtArrowAngle,o)},v4.drawArrowhead=function(r,i,o,l,f,b,d){if(!(isNaN(l)||l==null||isNaN(f)||f==null||isNaN(b)||b==null)){var w=this,y=i.pstyle(o+"-arrow-shape").value;if(y!=="none"){var k=i.pstyle(o+"-arrow-fill").value==="hollow"?"both":"filled",E=i.pstyle(o+"-arrow-fill").value,T=i.pstyle("width").pfValue,C=i.pstyle("opacity").value;d===void 0&&(d=C);var S=r.globalCompositeOperation;(d!==1||E==="hollow")&&(r.globalCompositeOperation="destination-out",w.colorFillStyle(r,255,255,255,1),w.colorStrokeStyle(r,255,255,255,1),w.drawArrowShape(i,r,k,T,y,l,f,b),r.globalCompositeOperation=S);var L=i.pstyle(o+"-arrow-color").value;w.colorFillStyle(r,L[0],L[1],L[2],d),w.colorStrokeStyle(r,L[0],L[1],L[2],d),w.drawArrowShape(i,r,E,T,y,l,f,b)}}},v4.drawArrowShape=function(r,i,o,l,f,b,d,w){var y=this,k=this.usePaths()&&f!=="triangle-cross",E=!1,T,C=i,S={x:b,y:d},L=r.pstyle("arrow-scale").value,O=this.getArrowWidth(l,L),B=y.arrowShapes[f];if(k){var N=y.arrowPathCache=y.arrowPathCache||[],F=w7(f),R=N[F];R!=null?(T=i=R,E=!0):(T=i=new Path2D,N[F]=T)}E||(i.beginPath&&i.beginPath(),k?B.draw(i,1,0,{x:0,y:0},1):B.draw(i,O,w,S,l),i.closePath&&i.closePath()),i=C,k&&(i.translate(b,d),i.rotate(w),i.scale(O,O)),(o==="filled"||o==="both")&&(k?i.fill(T):i.fill()),(o==="hollow"||o==="both")&&(i.lineWidth=(B.matchEdgeWidth?l:1)/(k?O:1),i.lineJoin="miter",k?i.stroke(T):i.stroke()),k&&(i.scale(1/O,1/O),i.rotate(-w),i.translate(-b,-d))};var Nue={};Nue.safeDrawImage=function(r,i,o,l,f,b,d,w,y,k){if(!(f<=0||b<=0||y<=0||k<=0))try{r.drawImage(i,o,l,f,b,d,w,y,k)}catch(E){tc(E)}},Nue.drawInscribedImage=function(r,i,o,l,f){var b=this,d=o.position(),w=d.x,y=d.y,k=o.cy().style(),E=k.getIndexedStyle.bind(k),T=E(o,"background-fit","value",l),C=E(o,"background-repeat","value",l),S=o.width(),L=o.height(),O=o.padding()*2,B=S+(E(o,"background-width-relative-to","value",l)==="inner"?0:O),N=L+(E(o,"background-height-relative-to","value",l)==="inner"?0:O),F=o._private.rscratch,R=E(o,"background-clip","value",l),q=R==="node",X=E(o,"background-image-opacity","value",l)*f,te=E(o,"background-image-smoothing","value",l),H=i.width||i.cachedW,Y=i.height||i.cachedH;(H==null||Y==null)&&(document.body.appendChild(i),H=i.cachedW=i.width||i.offsetWidth,Y=i.cachedH=i.height||i.offsetHeight,document.body.removeChild(i));var z=H,W=Y;if(E(o,"background-width","value",l)!=="auto"&&(E(o,"background-width","units",l)==="%"?z=E(o,"background-width","pfValue",l)*B:z=E(o,"background-width","pfValue",l)),E(o,"background-height","value",l)!=="auto"&&(E(o,"background-height","units",l)==="%"?W=E(o,"background-height","pfValue",l)*N:W=E(o,"background-height","pfValue",l)),!(z===0||W===0)){if(T==="contain"){var Z=Math.min(B/z,N/W);z*=Z,W*=Z}else if(T==="cover"){var Z=Math.max(B/z,N/W);z*=Z,W*=Z}var G=w-B/2,ae=E(o,"background-position-x","units",l),$=E(o,"background-position-x","pfValue",l);ae==="%"?G+=(B-z)*$:G+=$;var ge=E(o,"background-offset-x","units",l),ee=E(o,"background-offset-x","pfValue",l);ge==="%"?G+=(B-z)*ee:G+=ee;var de=y-N/2,re=E(o,"background-position-y","units",l),ke=E(o,"background-position-y","pfValue",l);re==="%"?de+=(N-W)*ke:de+=ke;var Ce=E(o,"background-offset-y","units",l),_e=E(o,"background-offset-y","pfValue",l);Ce==="%"?de+=(N-W)*_e:de+=_e,F.pathCache&&(G-=w,de-=y,w=0,y=0);var Te=r.globalAlpha;r.globalAlpha=X;var Be=b.getImgSmoothing(r),Ge=!1;if(te==="no"&&Be?(b.setImgSmoothing(r,!1),Ge=!0):te==="yes"&&!Be&&(b.setImgSmoothing(r,!0),Ge=!0),C==="no-repeat")q&&(r.save(),F.pathCache?r.clip(F.pathCache):(b.nodeShapes[b.getNodeShape(o)].draw(r,w,y,B,N),r.clip())),b.safeDrawImage(r,i,0,0,H,Y,G,de,z,W),q&&r.restore();else{var Xe=r.createPattern(i,C);r.fillStyle=Xe,b.nodeShapes[b.getNodeShape(o)].draw(r,w,y,B,N),r.translate(G,de),r.fill(),r.translate(-G,-de)}r.globalAlpha=Te,Ge&&b.setImgSmoothing(r,Be)}};var C7={};C7.eleTextBiggerThanMin=function(r,i){if(!i){var o=r.cy().zoom(),l=this.getPixelRatio(),f=Math.ceil(iue(o*l));i=Math.pow(2,f)}var b=r.pstyle("font-size").pfValue*i,d=r.pstyle("min-zoomed-font-size").pfValue;return!(b<d)},C7.drawElementText=function(r,i,o,l,f){var b=arguments.length>5&&arguments[5]!==void 0?arguments[5]:!0,d=this;if(l==null){if(b&&!d.eleTextBiggerThanMin(i))return}else if(l===!1)return;if(i.isNode()){var w=i.pstyle("label");if(!w||!w.value)return;var y=d.getLabelJustification(i);r.textAlign=y,r.textBaseline="bottom"}else{var k=i.element()._private.rscratch.badLine,E=i.pstyle("label"),T=i.pstyle("source-label"),C=i.pstyle("target-label");if(k||(!E||!E.value)&&(!T||!T.value)&&(!C||!C.value))return;r.textAlign="center",r.textBaseline="bottom"}var S=!o,L;o&&(L=o,r.translate(-L.x1,-L.y1)),f==null?(d.drawText(r,i,null,S,b),i.isEdge()&&(d.drawText(r,i,"source",S,b),d.drawText(r,i,"target",S,b))):d.drawText(r,i,f,S,b),o&&r.translate(L.x1,L.y1)},C7.getFontCache=function(r){var i;this.fontCaches=this.fontCaches||[];for(var o=0;o<this.fontCaches.length;o++)if(i=this.fontCaches[o],i.context===r)return i;return i={context:r},this.fontCaches.push(i),i},C7.setupTextStyle=function(r,i){var o=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0,l=i.pstyle("font-style").strValue,f=i.pstyle("font-size").pfValue+"px",b=i.pstyle("font-family").strValue,d=i.pstyle("font-weight").strValue,w=o?i.effectiveOpacity()*i.pstyle("text-opacity").value:1,y=i.pstyle("text-outline-opacity").value*w,k=i.pstyle("color").value,E=i.pstyle("text-outline-color").value;r.font=l+" "+d+" "+f+" "+b,r.lineJoin="round",this.colorFillStyle(r,k[0],k[1],k[2],w),this.colorStrokeStyle(r,E[0],E[1],E[2],y)};function lhn(r,i,o,l,f){var b=arguments.length>5&&arguments[5]!==void 0?arguments[5]:5;r.beginPath(),r.moveTo(i+b,o),r.lineTo(i+l-b,o),r.quadraticCurveTo(i+l,o,i+l,o+b),r.lineTo(i+l,o+f-b),r.quadraticCurveTo(i+l,o+f,i+l-b,o+f),r.lineTo(i+b,o+f),r.quadraticCurveTo(i,o+f,i,o+f-b),r.lineTo(i,o+b),r.quadraticCurveTo(i,o,i+b,o),r.closePath(),r.fill()}C7.getTextAngle=function(r,i){var o,l=r._private,f=l.rscratch,b=i?i+"-":"",d=r.pstyle(b+"text-rotation"),w=Gv(f,"labelAngle",i);return d.strValue==="autorotate"?o=r.isEdge()?w:0:d.strValue==="none"?o=0:o=d.pfValue,o},C7.drawText=function(r,i,o){var l=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!0,f=arguments.length>4&&arguments[4]!==void 0?arguments[4]:!0,b=i._private,d=b.rscratch,w=f?i.effectiveOpacity():1;if(!(f&&(w===0||i.pstyle("text-opacity").value===0))){o==="main"&&(o=null);var y=Gv(d,"labelX",o),k=Gv(d,"labelY",o),E,T,C=this.getLabelText(i,o);if(C!=null&&C!==""&&!isNaN(y)&&!isNaN(k)){this.setupTextStyle(r,i,f);var S=o?o+"-":"",L=Gv(d,"labelWidth",o),O=Gv(d,"labelHeight",o),B=i.pstyle(S+"text-margin-x").pfValue,N=i.pstyle(S+"text-margin-y").pfValue,F=i.isEdge(),R=i.pstyle("text-halign").value,q=i.pstyle("text-valign").value;F&&(R="center",q="center"),y+=B,k+=N;var X;switch(l?X=this.getTextAngle(i,o):X=0,X!==0&&(E=y,T=k,r.translate(E,T),r.rotate(X),y=0,k=0),q){case"top":break;case"center":k+=O/2;break;case"bottom":k+=O;break}var te=i.pstyle("text-background-opacity").value,H=i.pstyle("text-border-opacity").value,Y=i.pstyle("text-border-width").pfValue,z=i.pstyle("text-background-padding").pfValue;if(te>0||Y>0&&H>0){var W=y-z;switch(R){case"left":W-=L;break;case"center":W-=L/2;break}var Z=k-O-z,G=L+2*z,ae=O+2*z;if(te>0){var $=r.fillStyle,ge=i.pstyle("text-background-color").value;r.fillStyle="rgba("+ge[0]+","+ge[1]+","+ge[2]+","+te*w+")";var ee=i.pstyle("text-background-shape").strValue;ee.indexOf("round")===0?lhn(r,W,Z,G,ae,2):r.fillRect(W,Z,G,ae),r.fillStyle=$}if(Y>0&&H>0){var de=r.strokeStyle,re=r.lineWidth,ke=i.pstyle("text-border-color").value,Ce=i.pstyle("text-border-style").value;if(r.strokeStyle="rgba("+ke[0]+","+ke[1]+","+ke[2]+","+H*w+")",r.lineWidth=Y,r.setLineDash)switch(Ce){case"dotted":r.setLineDash([1,1]);break;case"dashed":r.setLineDash([4,2]);break;case"double":r.lineWidth=Y/4,r.setLineDash([]);break;case"solid":r.setLineDash([]);break}if(r.strokeRect(W,Z,G,ae),Ce==="double"){var _e=Y/2;r.strokeRect(W+_e,Z+_e,G-_e*2,ae-_e*2)}r.setLineDash&&r.setLineDash([]),r.lineWidth=re,r.strokeStyle=de}}var Te=2*i.pstyle("text-outline-width").pfValue;if(Te>0&&(r.lineWidth=Te),i.pstyle("text-wrap").value==="wrap"){var Be=Gv(d,"labelWrapCachedLines",o),Ge=Gv(d,"labelLineHeight",o),Xe=L/2,Ee=this.getLabelJustification(i);switch(Ee==="auto"||(R==="left"?Ee==="left"?y+=-L:Ee==="center"&&(y+=-Xe):R==="center"?Ee==="left"?y+=-Xe:Ee==="right"&&(y+=Xe):R==="right"&&(Ee==="center"?y+=Xe:Ee==="right"&&(y+=L))),q){case"top":k-=(Be.length-1)*Ge;break;case"center":case"bottom":k-=(Be.length-1)*Ge;break}for(var Ze=0;Ze<Be.length;Ze++)Te>0&&r.strokeText(Be[Ze],y,k),r.fillText(Be[Ze],y,k),k+=Ge}else Te>0&&r.strokeText(C,y,k),r.fillText(C,y,k);X!==0&&(r.rotate(-X),r.translate(-E,-T))}}};var JC={};JC.drawNode=function(r,i,o){var l=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!0,f=arguments.length>4&&arguments[4]!==void 0?arguments[4]:!0,b=arguments.length>5&&arguments[5]!==void 0?arguments[5]:!0,d=this,w,y,k=i._private,E=k.rscratch,T=i.position();if(!(!Vr(T.x)||!Vr(T.y))&&!(b&&!i.visible())){var C=b?i.effectiveOpacity():1,S=d.usePaths(),L,O=!1,B=i.padding();w=i.width()+2*B,y=i.height()+2*B;var N;o&&(N=o,r.translate(-N.x1,-N.y1));for(var F=i.pstyle("background-image"),R=F.value,q=new Array(R.length),X=new Array(R.length),te=0,H=0;H<R.length;H++){var Y=R[H],z=q[H]=Y!=null&&Y!=="none";if(z){var W=i.cy().style().getIndexedStyle(i,"background-image-crossorigin","value",H);te++,X[H]=d.getCachedImage(Y,W,function(){k.backgroundTimestamp=Date.now(),i.emitAndNotify("background")})}}var Z=i.pstyle("background-blacken").value,G=i.pstyle("border-width").pfValue,ae=i.pstyle("background-opacity").value*C,$=i.pstyle("border-color").value,ge=i.pstyle("border-style").value,ee=i.pstyle("border-opacity").value*C;r.lineJoin="miter";var de=function(){var Zt=arguments.length>0&&arguments[0]!==void 0?arguments[0]:ae;d.eleFillStyle(r,i,Zt)},re=function(){var Zt=arguments.length>0&&arguments[0]!==void 0?arguments[0]:ee;d.colorStrokeStyle(r,$[0],$[1],$[2],Zt)},ke=i.pstyle("shape").strValue,Ce=i.pstyle("shape-polygon-points").pfValue;if(S){r.translate(T.x,T.y);var _e=d.nodePathCache=d.nodePathCache||[],Te=sNe(ke==="polygon"?ke+","+Ce.join(","):ke,""+y,""+w),Be=_e[Te];Be!=null?(L=Be,O=!0,E.pathCache=L):(L=new Path2D,_e[Te]=E.pathCache=L)}var Ge=function(){if(!O){var Zt=T;S&&(Zt={x:0,y:0}),d.nodeShapes[d.getNodeShape(i)].draw(L||r,Zt.x,Zt.y,w,y)}S?r.fill(L):r.fill()},Xe=function(){for(var Zt=arguments.length>0&&arguments[0]!==void 0?arguments[0]:C,nt=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0,xn=k.backgrounding,cn=0,jt=0;jt<X.length;jt++){var ot=i.cy().style().getIndexedStyle(i,"background-image-containment","value",jt);if(nt&&ot==="over"||!nt&&ot==="inside"){cn++;continue}q[jt]&&X[jt].complete&&!X[jt].error&&(cn++,d.drawInscribedImage(r,X[jt],i,jt,Zt))}k.backgrounding=cn!==te,xn!==k.backgrounding&&i.updateStyle(!1)},Ee=function(){var Zt=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1,nt=arguments.length>1&&arguments[1]!==void 0?arguments[1]:C;d.hasPie(i)&&(d.drawPie(r,i,nt),Zt&&(S||d.nodeShapes[d.getNodeShape(i)].draw(r,T.x,T.y,w,y)))},Ze=function(){var Zt=arguments.length>0&&arguments[0]!==void 0?arguments[0]:C,nt=(Z>0?Z:-Z)*Zt,xn=Z>0?0:255;Z!==0&&(d.colorFillStyle(r,xn,xn,xn,nt),S?r.fill(L):r.fill())},Ie=function(){if(G>0){if(r.lineWidth=G,r.lineCap="butt",r.setLineDash)switch(ge){case"dotted":r.setLineDash([1,1]);break;case"dashed":r.setLineDash([4,2]);break;case"solid":case"double":r.setLineDash([]);break}if(S?r.stroke(L):r.stroke(),ge==="double"){r.lineWidth=G/3;var Zt=r.globalCompositeOperation;r.globalCompositeOperation="destination-out",S?r.stroke(L):r.stroke(),r.globalCompositeOperation=Zt}r.setLineDash&&r.setLineDash([])}},Oe=function(){f&&d.drawNodeOverlay(r,i,T,w,y)},Le=function(){f&&d.drawNodeUnderlay(r,i,T,w,y)},$e=function(){d.drawElementText(r,i,null,l)},xe=i.pstyle("ghost").value==="yes";if(xe){var Ae=i.pstyle("ghost-offset-x").pfValue,je=i.pstyle("ghost-offset-y").pfValue,me=i.pstyle("ghost-opacity").value,vt=me*C;r.translate(Ae,je),de(me*ae),Ge(),Xe(vt,!0),re(me*ee),Ie(),Ee(Z!==0||G!==0),Xe(vt,!1),Ze(vt),r.translate(-Ae,-je)}S&&r.translate(-T.x,-T.y),Le(),S&&r.translate(T.x,T.y),de(),Ge(),Xe(C,!0),re(),Ie(),Ee(Z!==0||G!==0),Xe(C,!1),Ze(),S&&r.translate(-T.x,-T.y),$e(),Oe(),o&&r.translate(N.x1,N.y1)}};var sBe=function(i){if(!["overlay","underlay"].includes(i))throw new Error("Invalid state");return function(o,l,f,b,d){var w=this;if(!!l.visible()){var y=l.pstyle("".concat(i,"-padding")).pfValue,k=l.pstyle("".concat(i,"-opacity")).value,E=l.pstyle("".concat(i,"-color")).value,T=l.pstyle("".concat(i,"-shape")).value;if(k>0){if(f=f||l.position(),b==null||d==null){var C=l.padding();b=l.width()+2*C,d=l.height()+2*C}w.colorFillStyle(o,E[0],E[1],E[2],k),w.nodeShapes[T].draw(o,f.x,f.y,b+y*2,d+y*2),o.fill()}}}};JC.drawNodeOverlay=sBe("overlay"),JC.drawNodeUnderlay=sBe("underlay"),JC.hasPie=function(r){return r=r[0],r._private.hasPie},JC.drawPie=function(r,i,o,l){i=i[0],l=l||i.position();var f=i.cy().style(),b=i.pstyle("pie-size"),d=l.x,w=l.y,y=i.width(),k=i.height(),E=Math.min(y,k)/2,T=0,C=this.usePaths();C&&(d=0,w=0),b.units==="%"?E=E*b.pfValue:b.pfValue!==void 0&&(E=b.pfValue/2);for(var S=1;S<=f.pieBackgroundN;S++){var L=i.pstyle("pie-"+S+"-background-size").value,O=i.pstyle("pie-"+S+"-background-color").value,B=i.pstyle("pie-"+S+"-background-opacity").value*o,N=L/100;N+T>1&&(N=1-T);var F=1.5*Math.PI+2*Math.PI*T,R=2*Math.PI*N,q=F+R;L===0||T>=1||T+N>1||(r.beginPath(),r.moveTo(d,w),r.arc(d,w,E,F,q),r.closePath(),this.colorFillStyle(r,O[0],O[1],O[2],B),r.fill(),T+=N)}};var wp={},hhn=100;wp.getPixelRatio=function(){var r=this.data.contexts[0];if(this.forcedPixelRatio!=null)return this.forcedPixelRatio;var i=r.backingStorePixelRatio||r.webkitBackingStorePixelRatio||r.mozBackingStorePixelRatio||r.msBackingStorePixelRatio||r.oBackingStorePixelRatio||r.backingStorePixelRatio||1;return(window.devicePixelRatio||1)/i},wp.paintCache=function(r){for(var i=this.paintCaches=this.paintCaches||[],o=!0,l,f=0;f<i.length;f++)if(l=i[f],l.context===r){o=!1;break}return o&&(l={context:r},i.push(l)),l},wp.createGradientStyleFor=function(r,i,o,l,f){var b,d=this.usePaths(),w=o.pstyle(i+"-gradient-stop-colors").value,y=o.pstyle(i+"-gradient-stop-positions").pfValue;if(l==="radial-gradient")if(o.isEdge()){var k=o.sourceEndpoint(),E=o.targetEndpoint(),T=o.midpoint(),C=m7(k,T),S=m7(E,T);b=r.createRadialGradient(T.x,T.y,0,T.x,T.y,Math.max(C,S))}else{var L=d?{x:0,y:0}:o.position(),O=o.paddedWidth(),B=o.paddedHeight();b=r.createRadialGradient(L.x,L.y,0,L.x,L.y,Math.max(O,B))}else if(o.isEdge()){var N=o.sourceEndpoint(),F=o.targetEndpoint();b=r.createLinearGradient(N.x,N.y,F.x,F.y)}else{var R=d?{x:0,y:0}:o.position(),q=o.paddedWidth(),X=o.paddedHeight(),te=q/2,H=X/2,Y=o.pstyle("background-gradient-direction").value;switch(Y){case"to-bottom":b=r.createLinearGradient(R.x,R.y-H,R.x,R.y+H);break;case"to-top":b=r.createLinearGradient(R.x,R.y+H,R.x,R.y-H);break;case"to-left":b=r.createLinearGradient(R.x+te,R.y,R.x-te,R.y);break;case"to-right":b=r.createLinearGradient(R.x-te,R.y,R.x+te,R.y);break;case"to-bottom-right":case"to-right-bottom":b=r.createLinearGradient(R.x-te,R.y-H,R.x+te,R.y+H);break;case"to-top-right":case"to-right-top":b=r.createLinearGradient(R.x-te,R.y+H,R.x+te,R.y-H);break;case"to-bottom-left":case"to-left-bottom":b=r.createLinearGradient(R.x+te,R.y-H,R.x-te,R.y+H);break;case"to-top-left":case"to-left-top":b=r.createLinearGradient(R.x+te,R.y+H,R.x-te,R.y-H);break}}if(!b)return null;for(var z=y.length===w.length,W=w.length,Z=0;Z<W;Z++)b.addColorStop(z?y[Z]:Z/(W-1),"rgba("+w[Z][0]+","+w[Z][1]+","+w[Z][2]+","+f+")");return b},wp.gradientFillStyle=function(r,i,o,l){var f=this.createGradientStyleFor(r,"background",i,o,l);if(!f)return null;r.fillStyle=f},wp.colorFillStyle=function(r,i,o,l,f){r.fillStyle="rgba("+i+","+o+","+l+","+f+")"},wp.eleFillStyle=function(r,i,o){var l=i.pstyle("background-fill").value;if(l==="linear-gradient"||l==="radial-gradient")this.gradientFillStyle(r,i,l,o);else{var f=i.pstyle("background-color").value;this.colorFillStyle(r,f[0],f[1],f[2],o)}},wp.gradientStrokeStyle=function(r,i,o,l){var f=this.createGradientStyleFor(r,"line",i,o,l);if(!f)return null;r.strokeStyle=f},wp.colorStrokeStyle=function(r,i,o,l,f){r.strokeStyle="rgba("+i+","+o+","+l+","+f+")"},wp.eleStrokeStyle=function(r,i,o){var l=i.pstyle("line-fill").value;if(l==="linear-gradient"||l==="radial-gradient")this.gradientStrokeStyle(r,i,l,o);else{var f=i.pstyle("line-color").value;this.colorStrokeStyle(r,f[0],f[1],f[2],o)}},wp.matchCanvasSize=function(r){var i=this,o=i.data,l=i.findContainerClientCoords(),f=l[2],b=l[3],d=i.getPixelRatio(),w=i.motionBlurPxRatio;(r===i.data.bufferCanvases[i.MOTIONBLUR_BUFFER_NODE]||r===i.data.bufferCanvases[i.MOTIONBLUR_BUFFER_DRAG])&&(d=w);var y=f*d,k=b*d,E;if(!(y===i.canvasWidth&&k===i.canvasHeight)){i.fontCaches=null;var T=o.canvasContainer;T.style.width=f+"px",T.style.height=b+"px";for(var C=0;C<i.CANVAS_LAYERS;C++)E=o.canvases[C],E.width=y,E.height=k,E.style.width=f+"px",E.style.height=b+"px";for(var C=0;C<i.BUFFER_COUNT;C++)E=o.bufferCanvases[C],E.width=y,E.height=k,E.style.width=f+"px",E.style.height=b+"px";i.textureMult=1,d<=1&&(E=o.bufferCanvases[i.TEXTURE_BUFFER],i.textureMult=2,E.width=y*i.textureMult,E.height=k*i.textureMult),i.canvasWidth=y,i.canvasHeight=k}},wp.renderTo=function(r,i,o,l){this.render({forcedContext:r,forcedZoom:i,forcedPan:o,drawAllLayers:!0,forcedPxRatio:l})},wp.render=function(r){r=r||hNe();var i=r.forcedContext,o=r.drawAllLayers,l=r.drawOnlyNodeLayer,f=r.forcedZoom,b=r.forcedPan,d=this,w=r.forcedPxRatio===void 0?this.getPixelRatio():r.forcedPxRatio,y=d.cy,k=d.data,E=k.canvasNeedsRedraw,T=d.textureOnViewport&&!i&&(d.pinching||d.hoverData.dragging||d.swipePanning||d.data.wheelZooming),C=r.motionBlur!==void 0?r.motionBlur:d.motionBlur,S=d.motionBlurPxRatio,L=y.hasCompoundNodes(),O=d.hoverData.draggingEles,B=!!(d.hoverData.selecting||d.touchData.selecting);C=C&&!i&&d.motionBlurEnabled&&!B;var N=C;i||(d.prevPxRatio!==w&&(d.invalidateContainerClientCoordsCache(),d.matchCanvasSize(d.container),d.redrawHint("eles",!0),d.redrawHint("drag",!0)),d.prevPxRatio=w),!i&&d.motionBlurTimeout&&clearTimeout(d.motionBlurTimeout),C&&(d.mbFrames==null&&(d.mbFrames=0),d.mbFrames++,d.mbFrames<3&&(N=!1),d.mbFrames>d.minMbLowQualFrames&&(d.motionBlurPxRatio=d.mbPxRBlurry)),d.clearingMotionBlur&&(d.motionBlurPxRatio=1),d.textureDrawLastFrame&&!T&&(E[d.NODE]=!0,E[d.SELECT_BOX]=!0);var F=y.style(),R=y.zoom(),q=f!==void 0?f:R,X=y.pan(),te={x:X.x,y:X.y},H={zoom:R,pan:{x:X.x,y:X.y}},Y=d.prevViewport,z=Y===void 0||H.zoom!==Y.zoom||H.pan.x!==Y.pan.x||H.pan.y!==Y.pan.y;!z&&!(O&&!L)&&(d.motionBlurPxRatio=1),b&&(te=b),q*=w,te.x*=w,te.y*=w;var W=d.getCachedZSortedEles();function Z(je,me,vt,ve,Zt){var nt=je.globalCompositeOperation;je.globalCompositeOperation="destination-out",d.colorFillStyle(je,255,255,255,d.motionBlurTransparency),je.fillRect(me,vt,ve,Zt),je.globalCompositeOperation=nt}function G(je,me){var vt,ve,Zt,nt;!d.clearingMotionBlur&&(je===k.bufferContexts[d.MOTIONBLUR_BUFFER_NODE]||je===k.bufferContexts[d.MOTIONBLUR_BUFFER_DRAG])?(vt={x:X.x*S,y:X.y*S},ve=R*S,Zt=d.canvasWidth*S,nt=d.canvasHeight*S):(vt=te,ve=q,Zt=d.canvasWidth,nt=d.canvasHeight),je.setTransform(1,0,0,1,0,0),me==="motionBlur"?Z(je,0,0,Zt,nt):!i&&(me===void 0||me)&&je.clearRect(0,0,Zt,nt),o||(je.translate(vt.x,vt.y),je.scale(ve,ve)),b&&je.translate(b.x,b.y),f&&je.scale(f,f)}if(T||(d.textureDrawLastFrame=!1),T){if(d.textureDrawLastFrame=!0,!d.textureCache){d.textureCache={},d.textureCache.bb=y.mutableElements().boundingBox(),d.textureCache.texture=d.data.bufferCanvases[d.TEXTURE_BUFFER];var ae=d.data.bufferContexts[d.TEXTURE_BUFFER];ae.setTransform(1,0,0,1,0,0),ae.clearRect(0,0,d.canvasWidth*d.textureMult,d.canvasHeight*d.textureMult),d.render({forcedContext:ae,drawOnlyNodeLayer:!0,forcedPxRatio:w*d.textureMult});var H=d.textureCache.viewport={zoom:y.zoom(),pan:y.pan(),width:d.canvasWidth,height:d.canvasHeight};H.mpan={x:(0-H.pan.x)/H.zoom,y:(0-H.pan.y)/H.zoom}}E[d.DRAG]=!1,E[d.NODE]=!1;var $=k.contexts[d.NODE],ge=d.textureCache.texture,H=d.textureCache.viewport;$.setTransform(1,0,0,1,0,0),C?Z($,0,0,H.width,H.height):$.clearRect(0,0,H.width,H.height);var ee=F.core("outside-texture-bg-color").value,de=F.core("outside-texture-bg-opacity").value;d.colorFillStyle($,ee[0],ee[1],ee[2],de),$.fillRect(0,0,H.width,H.height);var R=y.zoom();G($,!1),$.clearRect(H.mpan.x,H.mpan.y,H.width/H.zoom/w,H.height/H.zoom/w),$.drawImage(ge,H.mpan.x,H.mpan.y,H.width/H.zoom/w,H.height/H.zoom/w)}else d.textureOnViewport&&!i&&(d.textureCache=null);var re=y.extent(),ke=d.pinching||d.hoverData.dragging||d.swipePanning||d.data.wheelZooming||d.hoverData.draggingEles||d.cy.animated(),Ce=d.hideEdgesOnViewport&&ke,_e=[];if(_e[d.NODE]=!E[d.NODE]&&C&&!d.clearedForMotionBlur[d.NODE]||d.clearingMotionBlur,_e[d.NODE]&&(d.clearedForMotionBlur[d.NODE]=!0),_e[d.DRAG]=!E[d.DRAG]&&C&&!d.clearedForMotionBlur[d.DRAG]||d.clearingMotionBlur,_e[d.DRAG]&&(d.clearedForMotionBlur[d.DRAG]=!0),E[d.NODE]||o||l||_e[d.NODE]){var Te=C&&!_e[d.NODE]&&S!==1,$=i||(Te?d.data.bufferContexts[d.MOTIONBLUR_BUFFER_NODE]:k.contexts[d.NODE]),Be=C&&!Te?"motionBlur":void 0;G($,Be),Ce?d.drawCachedNodes($,W.nondrag,w,re):d.drawLayeredElements($,W.nondrag,w,re),d.debug&&d.drawDebugPoints($,W.nondrag),!o&&!C&&(E[d.NODE]=!1)}if(!l&&(E[d.DRAG]||o||_e[d.DRAG])){var Te=C&&!_e[d.DRAG]&&S!==1,$=i||(Te?d.data.bufferContexts[d.MOTIONBLUR_BUFFER_DRAG]:k.contexts[d.DRAG]);G($,C&&!Te?"motionBlur":void 0),Ce?d.drawCachedNodes($,W.drag,w,re):d.drawCachedElements($,W.drag,w,re),d.debug&&d.drawDebugPoints($,W.drag),!o&&!C&&(E[d.DRAG]=!1)}if(d.showFps||!l&&E[d.SELECT_BOX]&&!o){var $=i||k.contexts[d.SELECT_BOX];if(G($),d.selection[4]==1&&(d.hoverData.selecting||d.touchData.selecting)){var R=d.cy.zoom(),Ge=F.core("selection-box-border-width").value/R;$.lineWidth=Ge,$.fillStyle="rgba("+F.core("selection-box-color").value[0]+","+F.core("selection-box-color").value[1]+","+F.core("selection-box-color").value[2]+","+F.core("selection-box-opacity").value+")",$.fillRect(d.selection[0],d.selection[1],d.selection[2]-d.selection[0],d.selection[3]-d.selection[1]),Ge>0&&($.strokeStyle="rgba("+F.core("selection-box-border-color").value[0]+","+F.core("selection-box-border-color").value[1]+","+F.core("selection-box-border-color").value[2]+","+F.core("selection-box-opacity").value+")",$.strokeRect(d.selection[0],d.selection[1],d.selection[2]-d.selection[0],d.selection[3]-d.selection[1]))}if(k.bgActivePosistion&&!d.hoverData.selecting){var R=d.cy.zoom(),Xe=k.bgActivePosistion;$.fillStyle="rgba("+F.core("active-bg-color").value[0]+","+F.core("active-bg-color").value[1]+","+F.core("active-bg-color").value[2]+","+F.core("active-bg-opacity").value+")",$.beginPath(),$.arc(Xe.x,Xe.y,F.core("active-bg-size").pfValue/R,0,2*Math.PI),$.fill()}var Ee=d.lastRedrawTime;if(d.showFps&&Ee){Ee=Math.round(Ee);var Ze=Math.round(1e3/Ee);$.setTransform(1,0,0,1,0,0),$.fillStyle="rgba(255, 0, 0, 0.75)",$.strokeStyle="rgba(255, 0, 0, 0.75)",$.lineWidth=1,$.fillText("1 frame = "+Ee+" ms = "+Ze+" fps",0,20);var Ie=60;$.strokeRect(0,30,250,20),$.fillRect(0,30,250*Math.min(Ze/Ie,1),20)}o||(E[d.SELECT_BOX]=!1)}if(C&&S!==1){var Oe=k.contexts[d.NODE],Le=d.data.bufferCanvases[d.MOTIONBLUR_BUFFER_NODE],$e=k.contexts[d.DRAG],xe=d.data.bufferCanvases[d.MOTIONBLUR_BUFFER_DRAG],Ae=function(me,vt,ve){me.setTransform(1,0,0,1,0,0),ve||!N?me.clearRect(0,0,d.canvasWidth,d.canvasHeight):Z(me,0,0,d.canvasWidth,d.canvasHeight);var Zt=S;me.drawImage(vt,0,0,d.canvasWidth*Zt,d.canvasHeight*Zt,0,0,d.canvasWidth,d.canvasHeight)};(E[d.NODE]||_e[d.NODE])&&(Ae(Oe,Le,_e[d.NODE]),E[d.NODE]=!1),(E[d.DRAG]||_e[d.DRAG])&&(Ae($e,xe,_e[d.DRAG]),E[d.DRAG]=!1)}d.prevViewport=H,d.clearingMotionBlur&&(d.clearingMotionBlur=!1,d.motionBlurCleared=!0,d.motionBlur=!0),C&&(d.motionBlurTimeout=setTimeout(function(){d.motionBlurTimeout=null,d.clearedForMotionBlur[d.NODE]=!1,d.clearedForMotionBlur[d.DRAG]=!1,d.motionBlur=!1,d.clearingMotionBlur=!T,d.mbFrames=0,E[d.NODE]=!0,E[d.DRAG]=!0,d.redraw()},hhn)),i||y.emit("render")};var I6={};I6.drawPolygonPath=function(r,i,o,l,f,b){var d=l/2,w=f/2;r.beginPath&&r.beginPath(),r.moveTo(i+d*b[0],o+w*b[1]);for(var y=1;y<b.length/2;y++)r.lineTo(i+d*b[y*2],o+w*b[y*2+1]);r.closePath()},I6.drawRoundPolygonPath=function(r,i,o,l,f,b){var d=l/2,w=f/2,y=uue(l,f);r.beginPath&&r.beginPath();for(var k=0;k<b.length/4;k++){var E=void 0,T=void 0;k===0?E=b.length-2:E=k*4-2,T=k*4+2;var C=i+d*b[k*4],S=o+w*b[k*4+1],L=-b[E]*b[T]-b[E+1]*b[T+1],O=y/Math.tan(Math.acos(L)/2),B=C-O*b[E],N=S-O*b[E+1],F=C+O*b[T],R=S+O*b[T+1];k===0?r.moveTo(B,N):r.lineTo(B,N),r.arcTo(C,S,F,R,y)}r.closePath()},I6.drawRoundRectanglePath=function(r,i,o,l,f){var b=l/2,d=f/2,w=kO(l,f);r.beginPath&&r.beginPath(),r.moveTo(i,o-d),r.arcTo(i+b,o-d,i+b,o,w),r.arcTo(i+b,o+d,i,o+d,w),r.arcTo(i-b,o+d,i-b,o,w),r.arcTo(i-b,o-d,i,o-d,w),r.lineTo(i,o-d),r.closePath()},I6.drawBottomRoundRectanglePath=function(r,i,o,l,f){var b=l/2,d=f/2,w=kO(l,f);r.beginPath&&r.beginPath(),r.moveTo(i,o-d),r.lineTo(i+b,o-d),r.lineTo(i+b,o),r.arcTo(i+b,o+d,i,o+d,w),r.arcTo(i-b,o+d,i-b,o,w),r.lineTo(i-b,o-d),r.lineTo(i,o-d),r.closePath()},I6.drawCutRectanglePath=function(r,i,o,l,f){var b=l/2,d=f/2,w=kNe();r.beginPath&&r.beginPath(),r.moveTo(i-b+w,o-d),r.lineTo(i+b-w,o-d),r.lineTo(i+b,o-d+w),r.lineTo(i+b,o+d-w),r.lineTo(i+b-w,o+d),r.lineTo(i-b+w,o+d),r.lineTo(i-b,o+d-w),r.lineTo(i-b,o-d+w),r.closePath()},I6.drawBarrelPath=function(r,i,o,l,f){var b=l/2,d=f/2,w=i-b,y=i+b,k=o-d,E=o+d,T=lue(l,f),C=T.widthOffset,S=T.heightOffset,L=T.ctrlPtOffsetPct*C;r.beginPath&&r.beginPath(),r.moveTo(w,k+S),r.lineTo(w,E-S),r.quadraticCurveTo(w+L,E,w+C,E),r.lineTo(y-C,E),r.quadraticCurveTo(y-L,E,y,E-S),r.lineTo(y,k+S),r.quadraticCurveTo(y-L,k,y-C,k),r.lineTo(w+C,k),r.quadraticCurveTo(w+L,k,w,k+S),r.closePath()};for(var aBe=Math.sin(0),oBe=Math.cos(0),Pue={},Bue={},cBe=Math.PI/40,eS=0*Math.PI;eS<2*Math.PI;eS+=cBe)Pue[eS]=Math.sin(eS),Bue[eS]=Math.cos(eS);I6.drawEllipsePath=function(r,i,o,l,f){if(r.beginPath&&r.beginPath(),r.ellipse)r.ellipse(i,o,l/2,f/2,0,0,2*Math.PI);else for(var b,d,w=l/2,y=f/2,k=0*Math.PI;k<2*Math.PI;k+=cBe)b=i-w*Pue[k]*aBe+w*Bue[k]*oBe,d=o+y*Bue[k]*aBe+y*Pue[k]*oBe,k===0?r.moveTo(b,d):r.lineTo(b,d);r.closePath()};var RO={};RO.createBuffer=function(r,i){var o=document.createElement("canvas");return o.width=r,o.height=i,[o,o.getContext("2d")]},RO.bufferCanvasImage=function(r){var i=this.cy,o=i.mutableElements(),l=o.boundingBox(),f=this.findContainerClientCoords(),b=r.full?Math.ceil(l.w):f[2],d=r.full?Math.ceil(l.h):f[3],w=Vr(r.maxWidth)||Vr(r.maxHeight),y=this.getPixelRatio(),k=1;if(r.scale!==void 0)b*=r.scale,d*=r.scale,k=r.scale;else if(w){var E=1/0,T=1/0;Vr(r.maxWidth)&&(E=k*r.maxWidth/b),Vr(r.maxHeight)&&(T=k*r.maxHeight/d),k=Math.min(E,T),b*=k,d*=k}w||(b*=y,d*=y,k*=y);var C=document.createElement("canvas");C.width=b,C.height=d,C.style.width=b+"px",C.style.height=d+"px";var S=C.getContext("2d");if(b>0&&d>0){S.clearRect(0,0,b,d),S.globalCompositeOperation="source-over";var L=this.getCachedZSortedEles();if(r.full)S.translate(-l.x1*k,-l.y1*k),S.scale(k,k),this.drawElements(S,L),S.scale(1/k,1/k),S.translate(l.x1*k,l.y1*k);else{var O=i.pan(),B={x:O.x*k,y:O.y*k};k*=i.zoom(),S.translate(B.x,B.y),S.scale(k,k),this.drawElements(S,L),S.scale(1/k,1/k),S.translate(-B.x,-B.y)}r.bg&&(S.globalCompositeOperation="destination-over",S.fillStyle=r.bg,S.rect(0,0,b,d),S.fill())}return C};function fhn(r,i){for(var o=atob(r),l=new ArrayBuffer(o.length),f=new Uint8Array(l),b=0;b<o.length;b++)f[b]=o.charCodeAt(b);return new Blob([l],{type:i})}function uBe(r){var i=r.indexOf(",");return r.substr(i+1)}function lBe(r,i,o){var l=function(){return i.toDataURL(o,r.quality)};switch(r.output){case"blob-promise":return new HC(function(f,b){try{i.toBlob(function(d){d!=null?f(d):b(new Error("`canvas.toBlob()` sent a null value in its callback"))},o,r.quality)}catch(d){b(d)}});case"blob":return fhn(uBe(l()),o);case"base64":return uBe(l());case"base64uri":default:return l()}}RO.png=function(r){return lBe(r,this.bufferCanvasImage(r),"image/png")},RO.jpg=function(r){return lBe(r,this.bufferCanvasImage(r),"image/jpeg")};var hBe={};hBe.nodeShapeImpl=function(r,i,o,l,f,b,d){switch(r){case"ellipse":return this.drawEllipsePath(i,o,l,f,b);case"polygon":return this.drawPolygonPath(i,o,l,f,b,d);case"round-polygon":return this.drawRoundPolygonPath(i,o,l,f,b,d);case"roundrectangle":case"round-rectangle":return this.drawRoundRectanglePath(i,o,l,f,b);case"cutrectangle":case"cut-rectangle":return this.drawCutRectanglePath(i,o,l,f,b);case"bottomroundrectangle":case"bottom-round-rectangle":return this.drawBottomRoundRectanglePath(i,o,l,f,b);case"barrel":return this.drawBarrelPath(i,o,l,f,b)}};var dhn=fBe,Oo=fBe.prototype;Oo.CANVAS_LAYERS=3,Oo.SELECT_BOX=0,Oo.DRAG=1,Oo.NODE=2,Oo.BUFFER_COUNT=3,Oo.TEXTURE_BUFFER=0,Oo.MOTIONBLUR_BUFFER_NODE=1,Oo.MOTIONBLUR_BUFFER_DRAG=2;function fBe(r){var i=this;i.data={canvases:new Array(Oo.CANVAS_LAYERS),contexts:new Array(Oo.CANVAS_LAYERS),canvasNeedsRedraw:new Array(Oo.CANVAS_LAYERS),bufferCanvases:new Array(Oo.BUFFER_COUNT),bufferContexts:new Array(Oo.CANVAS_LAYERS)};var o="-webkit-tap-highlight-color",l="rgba(0,0,0,0)";i.data.canvasContainer=document.createElement("div");var f=i.data.canvasContainer.style;i.data.canvasContainer.style[o]=l,f.position="relative",f.zIndex="0",f.overflow="hidden";var b=r.cy.container();b.appendChild(i.data.canvasContainer),b.style[o]=l;var d={"-webkit-user-select":"none","-moz-user-select":"-moz-none","user-select":"none","-webkit-tap-highlight-color":"rgba(0,0,0,0)","outline-style":"none"};Kan()&&(d["-ms-touch-action"]="none",d["touch-action"]="none");for(var w=0;w<Oo.CANVAS_LAYERS;w++){var y=i.data.canvases[w]=document.createElement("canvas");i.data.contexts[w]=y.getContext("2d"),Object.keys(d).forEach(function(Ee){y.style[Ee]=d[Ee]}),y.style.position="absolute",y.setAttribute("data-id","layer"+w),y.style.zIndex=String(Oo.CANVAS_LAYERS-w),i.data.canvasContainer.appendChild(y),i.data.canvasNeedsRedraw[w]=!1}i.data.topCanvas=i.data.canvases[0],i.data.canvases[Oo.NODE].setAttribute("data-id","layer"+Oo.NODE+"-node"),i.data.canvases[Oo.SELECT_BOX].setAttribute("data-id","layer"+Oo.SELECT_BOX+"-selectbox"),i.data.canvases[Oo.DRAG].setAttribute("data-id","layer"+Oo.DRAG+"-drag");for(var w=0;w<Oo.BUFFER_COUNT;w++)i.data.bufferCanvases[w]=document.createElement("canvas"),i.data.bufferContexts[w]=i.data.bufferCanvases[w].getContext("2d"),i.data.bufferCanvases[w].style.position="absolute",i.data.bufferCanvases[w].setAttribute("data-id","buffer"+w),i.data.bufferCanvases[w].style.zIndex=String(-w-1),i.data.bufferCanvases[w].style.visibility="hidden";i.pathsEnabled=!0;var k=J0(),E=function(Ze){return{x:(Ze.x1+Ze.x2)/2,y:(Ze.y1+Ze.y2)/2}},T=function(Ze){return{x:-Ze.w/2,y:-Ze.h/2}},C=function(Ze){var Ie=Ze[0]._private,Oe=Ie.oldBackgroundTimestamp===Ie.backgroundTimestamp;return!Oe},S=function(Ze){return Ze[0]._private.nodeKey},L=function(Ze){return Ze[0]._private.labelStyleKey},O=function(Ze){return Ze[0]._private.sourceLabelStyleKey},B=function(Ze){return Ze[0]._private.targetLabelStyleKey},N=function(Ze,Ie,Oe,Le,$e){return i.drawElement(Ze,Ie,Oe,!1,!1,$e)},F=function(Ze,Ie,Oe,Le,$e){return i.drawElementText(Ze,Ie,Oe,Le,"main",$e)},R=function(Ze,Ie,Oe,Le,$e){return i.drawElementText(Ze,Ie,Oe,Le,"source",$e)},q=function(Ze,Ie,Oe,Le,$e){return i.drawElementText(Ze,Ie,Oe,Le,"target",$e)},X=function(Ze){return Ze.boundingBox(),Ze[0]._private.bodyBounds},te=function(Ze){return Ze.boundingBox(),Ze[0]._private.labelBounds.main||k},H=function(Ze){return Ze.boundingBox(),Ze[0]._private.labelBounds.source||k},Y=function(Ze){return Ze.boundingBox(),Ze[0]._private.labelBounds.target||k},z=function(Ze,Ie){return Ie},W=function(Ze){return E(X(Ze))},Z=function(Ze,Ie,Oe){var Le=Ze?Ze+"-":"";return{x:Ie.x+Oe.pstyle(Le+"text-margin-x").pfValue,y:Ie.y+Oe.pstyle(Le+"text-margin-y").pfValue}},G=function(Ze,Ie,Oe){var Le=Ze[0]._private.rscratch;return{x:Le[Ie],y:Le[Oe]}},ae=function(Ze){return Z("",G(Ze,"labelX","labelY"),Ze)},$=function(Ze){return Z("source",G(Ze,"sourceLabelX","sourceLabelY"),Ze)},ge=function(Ze){return Z("target",G(Ze,"targetLabelX","targetLabelY"),Ze)},ee=function(Ze){return T(X(Ze))},de=function(Ze){return T(H(Ze))},re=function(Ze){return T(Y(Ze))},ke=function(Ze){var Ie=te(Ze),Oe=T(te(Ze));if(Ze.isNode()){switch(Ze.pstyle("text-halign").value){case"left":Oe.x=-Ie.w;break;case"right":Oe.x=0;break}switch(Ze.pstyle("text-valign").value){case"top":Oe.y=-Ie.h;break;case"bottom":Oe.y=0;break}}return Oe},Ce=i.data.eleTxrCache=new BO(i,{getKey:S,doesEleInvalidateKey:C,drawElement:N,getBoundingBox:X,getRotationPoint:W,getRotationOffset:ee,allowEdgeTxrCaching:!1,allowParentTxrCaching:!1}),_e=i.data.lblTxrCache=new BO(i,{getKey:L,drawElement:F,getBoundingBox:te,getRotationPoint:ae,getRotationOffset:ke,isVisible:z}),Te=i.data.slbTxrCache=new BO(i,{getKey:O,drawElement:R,getBoundingBox:H,getRotationPoint:$,getRotationOffset:de,isVisible:z}),Be=i.data.tlbTxrCache=new BO(i,{getKey:B,drawElement:q,getBoundingBox:Y,getRotationPoint:ge,getRotationOffset:re,isVisible:z}),Ge=i.data.lyrTxrCache=new JPe(i);i.onUpdateEleCalcs(function(Ze,Ie){Ce.invalidateElements(Ie),_e.invalidateElements(Ie),Te.invalidateElements(Ie),Be.invalidateElements(Ie),Ge.invalidateElements(Ie);for(var Oe=0;Oe<Ie.length;Oe++){var Le=Ie[Oe]._private;Le.oldBackgroundTimestamp=Le.backgroundTimestamp}});var Xe=function(Ze){for(var Ie=0;Ie<Ze.length;Ie++)Ge.enqueueElementRefinement(Ze[Ie].ele)};Ce.onDequeue(Xe),_e.onDequeue(Xe),Te.onDequeue(Xe),Be.onDequeue(Xe)}Oo.redrawHint=function(r,i){var o=this;switch(r){case"eles":o.data.canvasNeedsRedraw[Oo.NODE]=i;break;case"drag":o.data.canvasNeedsRedraw[Oo.DRAG]=i;break;case"select":o.data.canvasNeedsRedraw[Oo.SELECT_BOX]=i;break}};var ghn=typeof Path2D<"u";Oo.path2dEnabled=function(r){if(r===void 0)return this.pathsEnabled;this.pathsEnabled=!!r},Oo.usePaths=function(){return ghn&&this.pathsEnabled},Oo.setImgSmoothing=function(r,i){r.imageSmoothingEnabled!=null?r.imageSmoothingEnabled=i:(r.webkitImageSmoothingEnabled=i,r.mozImageSmoothingEnabled=i,r.msImageSmoothingEnabled=i)},Oo.getImgSmoothing=function(r){return r.imageSmoothingEnabled!=null?r.imageSmoothingEnabled:r.webkitImageSmoothingEnabled||r.mozImageSmoothingEnabled||r.msImageSmoothingEnabled},Oo.makeOffscreenCanvas=function(r,i){var o;return(typeof OffscreenCanvas>"u"?"undefined":df(OffscreenCanvas))!=="undefined"?o=new OffscreenCanvas(r,i):(o=document.createElement("canvas"),o.width=r,o.height=i),o},[tBe,Xm,v4,Nue,C7,JC,wp,I6,RO,hBe].forEach(function(r){bs(Oo,r)});var phn=[{name:"null",impl:FPe},{name:"base",impl:KPe},{name:"canvas",impl:dhn}],bhn=[{type:"layout",extensions:Eln},{type:"renderer",extensions:phn}],dBe={},gBe={};function pBe(r,i,o){var l=o,f=function(Y){tc("Can not register `"+i+"` for `"+r+"` since `"+Y+"` already exists in the prototype and can not be overridden")};if(r==="core"){if(OO.prototype[i])return f(i);OO.prototype[i]=o}else if(r==="collection"){if(S1.prototype[i])return f(i);S1.prototype[i]=o}else if(r==="layout"){for(var b=function(Y){this.options=Y,o.call(this,Y),_o(this._private)||(this._private={}),this._private.cy=Y.cy,this._private.listeners=[],this.createEmitter()},d=b.prototype=Object.create(o.prototype),w=[],y=0;y<w.length;y++){var k=w[y];d[k]=d[k]||function(){return this}}d.start&&!d.run?d.run=function(){return this.start(),this}:!d.start&&d.run&&(d.start=function(){return this.run(),this});var E=o.prototype.stop;d.stop=function(){var H=this.options;if(H&&H.animate){var Y=this.animations;if(Y)for(var z=0;z<Y.length;z++)Y[z].stop()}return E?E.call(this):this.emit("layoutstop"),this},d.destroy||(d.destroy=function(){return this}),d.cy=function(){return this._private.cy};var T=function(Y){return Y._private.cy},C={addEventFields:function(Y,z){z.layout=Y,z.cy=T(Y),z.target=Y},bubble:function(){return!0},parent:function(Y){return T(Y)}};bs(d,{createEmitter:function(){return this._private.emitter=new BU(C,this),this},emitter:function(){return this._private.emitter},on:function(Y,z){return this.emitter().on(Y,z),this},one:function(Y,z){return this.emitter().one(Y,z),this},once:function(Y,z){return this.emitter().one(Y,z),this},removeListener:function(Y,z){return this.emitter().removeListener(Y,z),this},removeAllListeners:function(){return this.emitter().removeAllListeners(),this},emit:function(Y,z){return this.emitter().emit(Y,z),this}}),nc.eventAliasesOn(d),l=b}else if(r==="renderer"&&i!=="null"&&i!=="base"){var S=bBe("renderer","base"),L=S.prototype,O=o,B=o.prototype,N=function(){S.apply(this,arguments),O.apply(this,arguments)},F=N.prototype;for(var R in L){var q=L[R],X=B[R]!=null;if(X)return f(R);F[R]=q}for(var te in B)F[te]=B[te];L.clientFunctions.forEach(function(H){F[H]=F[H]||function(){Ml("Renderer does not implement `renderer."+H+"()` on its prototype")}}),l=N}else if(r==="__proto__"||r==="constructor"||r==="prototype")return Ml(r+" is an illegal type to be registered, possibly lead to prototype pollutions");return eNe({map:dBe,keys:[r,i],value:l})}function bBe(r,i){return tNe({map:dBe,keys:[r,i]})}function vhn(r,i,o,l,f){return eNe({map:gBe,keys:[r,i,o,l],value:f})}function whn(r,i,o,l){return tNe({map:gBe,keys:[r,i,o,l]})}var Fue=function(){if(arguments.length===2)return bBe.apply(null,arguments);if(arguments.length===3)return pBe.apply(null,arguments);if(arguments.length===4)return whn.apply(null,arguments);if(arguments.length===5)return vhn.apply(null,arguments);Ml("Invalid extension access syntax")};OO.prototype.extension=Fue,bhn.forEach(function(r){r.extensions.forEach(function(i){pBe(r.type,i.name,i.impl)})});var vBe=function r(){if(!(this instanceof r))return new r;this.length=0},S7=vBe.prototype;S7.instanceString=function(){return"stylesheet"},S7.selector=function(r){var i=this.length++;return this[i]={selector:r,properties:[]},this},S7.css=function(r,i){var o=this.length-1;if($i(r))this[o].properties.push({name:r,value:i});else if(_o(r))for(var l=r,f=Object.keys(l),b=0;b<f.length;b++){var d=f[b],w=l[d];if(w!=null){var y=gd.properties[d]||gd.properties[wU(d)];if(y!=null){var k=y.name,E=w;this[o].properties.push({name:k,value:E})}}}return this},S7.style=S7.css,S7.generateStyle=function(r){var i=new gd(r);return this.appendToStyle(i)},S7.appendToStyle=function(r){for(var i=0;i<this.length;i++){var o=this[i],l=o.selector,f=o.properties;r.selector(l);for(var b=0;b<f.length;b++){var d=f[b];r.css(d.name,d.value)}}return r};var mhn="3.23.0",A7=function(i){if(i===void 0&&(i={}),_o(i))return new OO(i);if($i(i))return Fue.apply(Fue,arguments)};A7.use=function(r){var i=Array.prototype.slice.call(arguments,1);return i.unshift(A7),r.apply(null,i),this},A7.warnings=function(r){return uNe(r)},A7.version=mhn,A7.stylesheet=A7.Stylesheet=vBe;var wBe=A7,mBe={exports:{}},Rue={exports:{}},jue={exports:{}},yBe;function yhn(){return yBe||(yBe=1,function(r,i){(function(l,f){r.exports=f()})(Mm,function(){return function(o){var l={};function f(b){if(l[b])return l[b].exports;var d=l[b]={i:b,l:!1,exports:{}};return o[b].call(d.exports,d,d.exports,f),d.l=!0,d.exports}return f.m=o,f.c=l,f.i=function(b){return b},f.d=function(b,d,w){f.o(b,d)||Object.defineProperty(b,d,{configurable:!1,enumerable:!0,get:w})},f.n=function(b){var d=b&&b.__esModule?function(){return b.default}:function(){return b};return f.d(d,"a",d),d},f.o=function(b,d){return Object.prototype.hasOwnProperty.call(b,d)},f.p="",f(f.s=26)}([function(o,l,f){function b(){}b.QUALITY=1,b.DEFAULT_CREATE_BENDS_AS_NEEDED=!1,b.DEFAULT_INCREMENTAL=!1,b.DEFAULT_ANIMATION_ON_LAYOUT=!0,b.DEFAULT_ANIMATION_DURING_LAYOUT=!1,b.DEFAULT_ANIMATION_PERIOD=50,b.DEFAULT_UNIFORM_LEAF_NODE_SIZES=!1,b.DEFAULT_GRAPH_MARGIN=15,b.NODE_DIMENSIONS_INCLUDE_LABELS=!1,b.SIMPLE_NODE_SIZE=40,b.SIMPLE_NODE_HALF_SIZE=b.SIMPLE_NODE_SIZE/2,b.EMPTY_COMPOUND_NODE_SIZE=40,b.MIN_EDGE_LENGTH=1,b.WORLD_BOUNDARY=1e6,b.INITIAL_WORLD_BOUNDARY=b.WORLD_BOUNDARY/1e3,b.WORLD_CENTER_X=1200,b.WORLD_CENTER_Y=900,o.exports=b},function(o,l,f){var b=f(2),d=f(8),w=f(9);function y(E,T,C){b.call(this,C),this.isOverlapingSourceAndTarget=!1,this.vGraphObject=C,this.bendpoints=[],this.source=E,this.target=T}y.prototype=Object.create(b.prototype);for(var k in b)y[k]=b[k];y.prototype.getSource=function(){return this.source},y.prototype.getTarget=function(){return this.target},y.prototype.isInterGraph=function(){return this.isInterGraph},y.prototype.getLength=function(){return this.length},y.prototype.isOverlapingSourceAndTarget=function(){return this.isOverlapingSourceAndTarget},y.prototype.getBendpoints=function(){return this.bendpoints},y.prototype.getLca=function(){return this.lca},y.prototype.getSourceInLca=function(){return this.sourceInLca},y.prototype.getTargetInLca=function(){return this.targetInLca},y.prototype.getOtherEnd=function(E){if(this.source===E)return this.target;if(this.target===E)return this.source;throw"Node is not incident with this edge"},y.prototype.getOtherEndInGraph=function(E,T){for(var C=this.getOtherEnd(E),S=T.getGraphManager().getRoot();;){if(C.getOwner()==T)return C;if(C.getOwner()==S)break;C=C.getOwner().getParent()}return null},y.prototype.updateLength=function(){var E=new Array(4);this.isOverlapingSourceAndTarget=d.getIntersection(this.target.getRect(),this.source.getRect(),E),this.isOverlapingSourceAndTarget||(this.lengthX=E[0]-E[2],this.lengthY=E[1]-E[3],Math.abs(this.lengthX)<1&&(this.lengthX=w.sign(this.lengthX)),Math.abs(this.lengthY)<1&&(this.lengthY=w.sign(this.lengthY)),this.length=Math.sqrt(this.lengthX*this.lengthX+this.lengthY*this.lengthY))},y.prototype.updateLengthSimple=function(){this.lengthX=this.target.getCenterX()-this.source.getCenterX(),this.lengthY=this.target.getCenterY()-this.source.getCenterY(),Math.abs(this.lengthX)<1&&(this.lengthX=w.sign(this.lengthX)),Math.abs(this.lengthY)<1&&(this.lengthY=w.sign(this.lengthY)),this.length=Math.sqrt(this.lengthX*this.lengthX+this.lengthY*this.lengthY)},o.exports=y},function(o,l,f){function b(d){this.vGraphObject=d}o.exports=b},function(o,l,f){var b=f(2),d=f(10),w=f(13),y=f(0),k=f(16),E=f(4);function T(S,L,O,B){O==null&&B==null&&(B=L),b.call(this,B),S.graphManager!=null&&(S=S.graphManager),this.estimatedSize=d.MIN_VALUE,this.inclusionTreeDepth=d.MAX_VALUE,this.vGraphObject=B,this.edges=[],this.graphManager=S,O!=null&&L!=null?this.rect=new w(L.x,L.y,O.width,O.height):this.rect=new w}T.prototype=Object.create(b.prototype);for(var C in b)T[C]=b[C];T.prototype.getEdges=function(){return this.edges},T.prototype.getChild=function(){return this.child},T.prototype.getOwner=function(){return this.owner},T.prototype.getWidth=function(){return this.rect.width},T.prototype.setWidth=function(S){this.rect.width=S},T.prototype.getHeight=function(){return this.rect.height},T.prototype.setHeight=function(S){this.rect.height=S},T.prototype.getCenterX=function(){return this.rect.x+this.rect.width/2},T.prototype.getCenterY=function(){return this.rect.y+this.rect.height/2},T.prototype.getCenter=function(){return new E(this.rect.x+this.rect.width/2,this.rect.y+this.rect.height/2)},T.prototype.getLocation=function(){return new E(this.rect.x,this.rect.y)},T.prototype.getRect=function(){return this.rect},T.prototype.getDiagonal=function(){return Math.sqrt(this.rect.width*this.rect.width+this.rect.height*this.rect.height)},T.prototype.getHalfTheDiagonal=function(){return Math.sqrt(this.rect.height*this.rect.height+this.rect.width*this.rect.width)/2},T.prototype.setRect=function(S,L){this.rect.x=S.x,this.rect.y=S.y,this.rect.width=L.width,this.rect.height=L.height},T.prototype.setCenter=function(S,L){this.rect.x=S-this.rect.width/2,this.rect.y=L-this.rect.height/2},T.prototype.setLocation=function(S,L){this.rect.x=S,this.rect.y=L},T.prototype.moveBy=function(S,L){this.rect.x+=S,this.rect.y+=L},T.prototype.getEdgeListToNode=function(S){var L=[],O=this;return O.edges.forEach(function(B){if(B.target==S){if(B.source!=O)throw"Incorrect edge source!";L.push(B)}}),L},T.prototype.getEdgesBetween=function(S){var L=[],O=this;return O.edges.forEach(function(B){if(!(B.source==O||B.target==O))throw"Incorrect edge source and/or target";(B.target==S||B.source==S)&&L.push(B)}),L},T.prototype.getNeighborsList=function(){var S=new Set,L=this;return L.edges.forEach(function(O){if(O.source==L)S.add(O.target);else{if(O.target!=L)throw"Incorrect incidency!";S.add(O.source)}}),S},T.prototype.withChildren=function(){var S=new Set,L,O;if(S.add(this),this.child!=null)for(var B=this.child.getNodes(),N=0;N<B.length;N++)L=B[N],O=L.withChildren(),O.forEach(function(F){S.add(F)});return S},T.prototype.getNoOfChildren=function(){var S=0,L;if(this.child==null)S=1;else for(var O=this.child.getNodes(),B=0;B<O.length;B++)L=O[B],S+=L.getNoOfChildren();return S==0&&(S=1),S},T.prototype.getEstimatedSize=function(){if(this.estimatedSize==d.MIN_VALUE)throw"assert failed";return this.estimatedSize},T.prototype.calcEstimatedSize=function(){return this.child==null?this.estimatedSize=(this.rect.width+this.rect.height)/2:(this.estimatedSize=this.child.calcEstimatedSize(),this.rect.width=this.estimatedSize,this.rect.height=this.estimatedSize,this.estimatedSize)},T.prototype.scatter=function(){var S,L,O=-y.INITIAL_WORLD_BOUNDARY,B=y.INITIAL_WORLD_BOUNDARY;S=y.WORLD_CENTER_X+k.nextDouble()*(B-O)+O;var N=-y.INITIAL_WORLD_BOUNDARY,F=y.INITIAL_WORLD_BOUNDARY;L=y.WORLD_CENTER_Y+k.nextDouble()*(F-N)+N,this.rect.x=S,this.rect.y=L},T.prototype.updateBounds=function(){if(this.getChild()==null)throw"assert failed";if(this.getChild().getNodes().length!=0){var S=this.getChild();if(S.updateBounds(!0),this.rect.x=S.getLeft(),this.rect.y=S.getTop(),this.setWidth(S.getRight()-S.getLeft()),this.setHeight(S.getBottom()-S.getTop()),y.NODE_DIMENSIONS_INCLUDE_LABELS){var L=S.getRight()-S.getLeft(),O=S.getBottom()-S.getTop();this.labelWidth>L&&(this.rect.x-=(this.labelWidth-L)/2,this.setWidth(this.labelWidth)),this.labelHeight>O&&(this.labelPos=="center"?this.rect.y-=(this.labelHeight-O)/2:this.labelPos=="top"&&(this.rect.y-=this.labelHeight-O),this.setHeight(this.labelHeight))}}},T.prototype.getInclusionTreeDepth=function(){if(this.inclusionTreeDepth==d.MAX_VALUE)throw"assert failed";return this.inclusionTreeDepth},T.prototype.transform=function(S){var L=this.rect.x;L>y.WORLD_BOUNDARY?L=y.WORLD_BOUNDARY:L<-y.WORLD_BOUNDARY&&(L=-y.WORLD_BOUNDARY);var O=this.rect.y;O>y.WORLD_BOUNDARY?O=y.WORLD_BOUNDARY:O<-y.WORLD_BOUNDARY&&(O=-y.WORLD_BOUNDARY);var B=new E(L,O),N=S.inverseTransformPoint(B);this.setLocation(N.x,N.y)},T.prototype.getLeft=function(){return this.rect.x},T.prototype.getRight=function(){return this.rect.x+this.rect.width},T.prototype.getTop=function(){return this.rect.y},T.prototype.getBottom=function(){return this.rect.y+this.rect.height},T.prototype.getParent=function(){return this.owner==null?null:this.owner.getParent()},o.exports=T},function(o,l,f){function b(d,w){d==null&&w==null?(this.x=0,this.y=0):(this.x=d,this.y=w)}b.prototype.getX=function(){return this.x},b.prototype.getY=function(){return this.y},b.prototype.setX=function(d){this.x=d},b.prototype.setY=function(d){this.y=d},b.prototype.getDifference=function(d){return new DimensionD(this.x-d.x,this.y-d.y)},b.prototype.getCopy=function(){return new b(this.x,this.y)},b.prototype.translate=function(d){return this.x+=d.width,this.y+=d.height,this},o.exports=b},function(o,l,f){var b=f(2),d=f(10),w=f(0),y=f(6),k=f(3),E=f(1),T=f(13),C=f(12),S=f(11);function L(B,N,F){b.call(this,F),this.estimatedSize=d.MIN_VALUE,this.margin=w.DEFAULT_GRAPH_MARGIN,this.edges=[],this.nodes=[],this.isConnected=!1,this.parent=B,N!=null&&N instanceof y?this.graphManager=N:N!=null&&N instanceof Layout&&(this.graphManager=N.graphManager)}L.prototype=Object.create(b.prototype);for(var O in b)L[O]=b[O];L.prototype.getNodes=function(){return this.nodes},L.prototype.getEdges=function(){return this.edges},L.prototype.getGraphManager=function(){return this.graphManager},L.prototype.getParent=function(){return this.parent},L.prototype.getLeft=function(){return this.left},L.prototype.getRight=function(){return this.right},L.prototype.getTop=function(){return this.top},L.prototype.getBottom=function(){return this.bottom},L.prototype.isConnected=function(){return this.isConnected},L.prototype.add=function(B,N,F){if(N==null&&F==null){var R=B;if(this.graphManager==null)throw"Graph has no graph mgr!";if(this.getNodes().indexOf(R)>-1)throw"Node already in graph!";return R.owner=this,this.getNodes().push(R),R}else{var q=B;if(!(this.getNodes().indexOf(N)>-1&&this.getNodes().indexOf(F)>-1))throw"Source or target not in graph!";if(!(N.owner==F.owner&&N.owner==this))throw"Both owners must be this graph!";return N.owner!=F.owner?null:(q.source=N,q.target=F,q.isInterGraph=!1,this.getEdges().push(q),N.edges.push(q),F!=N&&F.edges.push(q),q)}},L.prototype.remove=function(B){var N=B;if(B instanceof k){if(N==null)throw"Node is null!";if(!(N.owner!=null&&N.owner==this))throw"Owner graph is invalid!";if(this.graphManager==null)throw"Owner graph manager is invalid!";for(var F=N.edges.slice(),R,q=F.length,X=0;X<q;X++)R=F[X],R.isInterGraph?this.graphManager.remove(R):R.source.owner.remove(R);var te=this.nodes.indexOf(N);if(te==-1)throw"Node not in owner node list!";this.nodes.splice(te,1)}else if(B instanceof E){var R=B;if(R==null)throw"Edge is null!";if(!(R.source!=null&&R.target!=null))throw"Source and/or target is null!";if(!(R.source.owner!=null&&R.target.owner!=null&&R.source.owner==this&&R.target.owner==this))throw"Source and/or target owner is invalid!";var H=R.source.edges.indexOf(R),Y=R.target.edges.indexOf(R);if(!(H>-1&&Y>-1))throw"Source and/or target doesn't know this edge!";R.source.edges.splice(H,1),R.target!=R.source&&R.target.edges.splice(Y,1);var te=R.source.owner.getEdges().indexOf(R);if(te==-1)throw"Not in owner's edge list!";R.source.owner.getEdges().splice(te,1)}},L.prototype.updateLeftTop=function(){for(var B=d.MAX_VALUE,N=d.MAX_VALUE,F,R,q,X=this.getNodes(),te=X.length,H=0;H<te;H++){var Y=X[H];F=Y.getTop(),R=Y.getLeft(),B>F&&(B=F),N>R&&(N=R)}return B==d.MAX_VALUE?null:(X[0].getParent().paddingLeft!=null?q=X[0].getParent().paddingLeft:q=this.margin,this.left=N-q,this.top=B-q,new C(this.left,this.top))},L.prototype.updateBounds=function(B){for(var N=d.MAX_VALUE,F=-d.MAX_VALUE,R=d.MAX_VALUE,q=-d.MAX_VALUE,X,te,H,Y,z,W=this.nodes,Z=W.length,G=0;G<Z;G++){var ae=W[G];B&&ae.child!=null&&ae.updateBounds(),X=ae.getLeft(),te=ae.getRight(),H=ae.getTop(),Y=ae.getBottom(),N>X&&(N=X),F<te&&(F=te),R>H&&(R=H),q<Y&&(q=Y)}var $=new T(N,R,F-N,q-R);N==d.MAX_VALUE&&(this.left=this.parent.getLeft(),this.right=this.parent.getRight(),this.top=this.parent.getTop(),this.bottom=this.parent.getBottom()),W[0].getParent().paddingLeft!=null?z=W[0].getParent().paddingLeft:z=this.margin,this.left=$.x-z,this.right=$.x+$.width+z,this.top=$.y-z,this.bottom=$.y+$.height+z},L.calculateBounds=function(B){for(var N=d.MAX_VALUE,F=-d.MAX_VALUE,R=d.MAX_VALUE,q=-d.MAX_VALUE,X,te,H,Y,z=B.length,W=0;W<z;W++){var Z=B[W];X=Z.getLeft(),te=Z.getRight(),H=Z.getTop(),Y=Z.getBottom(),N>X&&(N=X),F<te&&(F=te),R>H&&(R=H),q<Y&&(q=Y)}var G=new T(N,R,F-N,q-R);return G},L.prototype.getInclusionTreeDepth=function(){return this==this.graphManager.getRoot()?1:this.parent.getInclusionTreeDepth()},L.prototype.getEstimatedSize=function(){if(this.estimatedSize==d.MIN_VALUE)throw"assert failed";return this.estimatedSize},L.prototype.calcEstimatedSize=function(){for(var B=0,N=this.nodes,F=N.length,R=0;R<F;R++){var q=N[R];B+=q.calcEstimatedSize()}return B==0?this.estimatedSize=w.EMPTY_COMPOUND_NODE_SIZE:this.estimatedSize=B/Math.sqrt(this.nodes.length),this.estimatedSize},L.prototype.updateConnected=function(){var B=this;if(this.nodes.length==0){this.isConnected=!0;return}var N=new S,F=new Set,R=this.nodes[0],q,X,te=R.withChildren();for(te.forEach(function(G){N.push(G),F.add(G)});N.length!==0;){R=N.shift(),q=R.getEdges();for(var H=q.length,Y=0;Y<H;Y++){var z=q[Y];if(X=z.getOtherEndInGraph(R,this),X!=null&&!F.has(X)){var W=X.withChildren();W.forEach(function(G){N.push(G),F.add(G)})}}}if(this.isConnected=!1,F.size>=this.nodes.length){var Z=0;F.forEach(function(G){G.owner==B&&Z++}),Z==this.nodes.length&&(this.isConnected=!0)}},o.exports=L},function(o,l,f){var b,d=f(1);function w(y){b=f(5),this.layout=y,this.graphs=[],this.edges=[]}w.prototype.addRoot=function(){var y=this.layout.newGraph(),k=this.layout.newNode(null),E=this.add(y,k);return this.setRootGraph(E),this.rootGraph},w.prototype.add=function(y,k,E,T,C){if(E==null&&T==null&&C==null){if(y==null)throw"Graph is null!";if(k==null)throw"Parent node is null!";if(this.graphs.indexOf(y)>-1)throw"Graph already in this graph mgr!";if(this.graphs.push(y),y.parent!=null)throw"Already has a parent!";if(k.child!=null)throw"Already has a child!";return y.parent=k,k.child=y,y}else{C=E,T=k,E=y;var S=T.getOwner(),L=C.getOwner();if(!(S!=null&&S.getGraphManager()==this))throw"Source not in this graph mgr!";if(!(L!=null&&L.getGraphManager()==this))throw"Target not in this graph mgr!";if(S==L)return E.isInterGraph=!1,S.add(E,T,C);if(E.isInterGraph=!0,E.source=T,E.target=C,this.edges.indexOf(E)>-1)throw"Edge already in inter-graph edge list!";if(this.edges.push(E),!(E.source!=null&&E.target!=null))throw"Edge source and/or target is null!";if(!(E.source.edges.indexOf(E)==-1&&E.target.edges.indexOf(E)==-1))throw"Edge already in source and/or target incidency list!";return E.source.edges.push(E),E.target.edges.push(E),E}},w.prototype.remove=function(y){if(y instanceof b){var k=y;if(k.getGraphManager()!=this)throw"Graph not in this graph mgr";if(!(k==this.rootGraph||k.parent!=null&&k.parent.graphManager==this))throw"Invalid parent node!";var E=[];E=E.concat(k.getEdges());for(var T,C=E.length,S=0;S<C;S++)T=E[S],k.remove(T);var L=[];L=L.concat(k.getNodes());var O;C=L.length;for(var S=0;S<C;S++)O=L[S],k.remove(O);k==this.rootGraph&&this.setRootGraph(null);var B=this.graphs.indexOf(k);this.graphs.splice(B,1),k.parent=null}else if(y instanceof d){if(T=y,T==null)throw"Edge is null!";if(!T.isInterGraph)throw"Not an inter-graph edge!";if(!(T.source!=null&&T.target!=null))throw"Source and/or target is null!";if(!(T.source.edges.indexOf(T)!=-1&&T.target.edges.indexOf(T)!=-1))throw"Source and/or target doesn't know this edge!";var B=T.source.edges.indexOf(T);if(T.source.edges.splice(B,1),B=T.target.edges.indexOf(T),T.target.edges.splice(B,1),!(T.source.owner!=null&&T.source.owner.getGraphManager()!=null))throw"Edge owner graph or owner graph manager is null!";if(T.source.owner.getGraphManager().edges.indexOf(T)==-1)throw"Not in owner graph manager's edge list!";var B=T.source.owner.getGraphManager().edges.indexOf(T);T.source.owner.getGraphManager().edges.splice(B,1)}},w.prototype.updateBounds=function(){this.rootGraph.updateBounds(!0)},w.prototype.getGraphs=function(){return this.graphs},w.prototype.getAllNodes=function(){if(this.allNodes==null){for(var y=[],k=this.getGraphs(),E=k.length,T=0;T<E;T++)y=y.concat(k[T].getNodes());this.allNodes=y}return this.allNodes},w.prototype.resetAllNodes=function(){this.allNodes=null},w.prototype.resetAllEdges=function(){this.allEdges=null},w.prototype.resetAllNodesToApplyGravitation=function(){this.allNodesToApplyGravitation=null},w.prototype.getAllEdges=function(){if(this.allEdges==null){var y=[],k=this.getGraphs();k.length;for(var E=0;E<k.length;E++)y=y.concat(k[E].getEdges());y=y.concat(this.edges),this.allEdges=y}return this.allEdges},w.prototype.getAllNodesToApplyGravitation=function(){return this.allNodesToApplyGravitation},w.prototype.setAllNodesToApplyGravitation=function(y){if(this.allNodesToApplyGravitation!=null)throw"assert failed";this.allNodesToApplyGravitation=y},w.prototype.getRoot=function(){return this.rootGraph},w.prototype.setRootGraph=function(y){if(y.getGraphManager()!=this)throw"Root not in this graph mgr!";this.rootGraph=y,y.parent==null&&(y.parent=this.layout.newNode("Root node"))},w.prototype.getLayout=function(){return this.layout},w.prototype.isOneAncestorOfOther=function(y,k){if(!(y!=null&&k!=null))throw"assert failed";if(y==k)return!0;var E=y.getOwner(),T;do{if(T=E.getParent(),T==null)break;if(T==k)return!0;if(E=T.getOwner(),E==null)break}while(!0);E=k.getOwner();do{if(T=E.getParent(),T==null)break;if(T==y)return!0;if(E=T.getOwner(),E==null)break}while(!0);return!1},w.prototype.calcLowestCommonAncestors=function(){for(var y,k,E,T,C,S=this.getAllEdges(),L=S.length,O=0;O<L;O++){if(y=S[O],k=y.source,E=y.target,y.lca=null,y.sourceInLca=k,y.targetInLca=E,k==E){y.lca=k.getOwner();continue}for(T=k.getOwner();y.lca==null;){for(y.targetInLca=E,C=E.getOwner();y.lca==null;){if(C==T){y.lca=C;break}if(C==this.rootGraph)break;if(y.lca!=null)throw"assert failed";y.targetInLca=C.getParent(),C=y.targetInLca.getOwner()}if(T==this.rootGraph)break;y.lca==null&&(y.sourceInLca=T.getParent(),T=y.sourceInLca.getOwner())}if(y.lca==null)throw"assert failed"}},w.prototype.calcLowestCommonAncestor=function(y,k){if(y==k)return y.getOwner();var E=y.getOwner();do{if(E==null)break;var T=k.getOwner();do{if(T==null)break;if(T==E)return T;T=T.getParent().getOwner()}while(!0);E=E.getParent().getOwner()}while(!0);return E},w.prototype.calcInclusionTreeDepths=function(y,k){y==null&&k==null&&(y=this.rootGraph,k=1);for(var E,T=y.getNodes(),C=T.length,S=0;S<C;S++)E=T[S],E.inclusionTreeDepth=k,E.child!=null&&this.calcInclusionTreeDepths(E.child,k+1)},w.prototype.includesInvalidEdge=function(){for(var y,k=this.edges.length,E=0;E<k;E++)if(y=this.edges[E],this.isOneAncestorOfOther(y.source,y.target))return!0;return!1},o.exports=w},function(o,l,f){var b=f(0);function d(){}for(var w in b)d[w]=b[w];d.MAX_ITERATIONS=2500,d.DEFAULT_EDGE_LENGTH=50,d.DEFAULT_SPRING_STRENGTH=.45,d.DEFAULT_REPULSION_STRENGTH=4500,d.DEFAULT_GRAVITY_STRENGTH=.4,d.DEFAULT_COMPOUND_GRAVITY_STRENGTH=1,d.DEFAULT_GRAVITY_RANGE_FACTOR=3.8,d.DEFAULT_COMPOUND_GRAVITY_RANGE_FACTOR=1.5,d.DEFAULT_USE_SMART_IDEAL_EDGE_LENGTH_CALCULATION=!0,d.DEFAULT_USE_SMART_REPULSION_RANGE_CALCULATION=!0,d.DEFAULT_COOLING_FACTOR_INCREMENTAL=.3,d.COOLING_ADAPTATION_FACTOR=.33,d.ADAPTATION_LOWER_NODE_LIMIT=1e3,d.ADAPTATION_UPPER_NODE_LIMIT=5e3,d.MAX_NODE_DISPLACEMENT_INCREMENTAL=100,d.MAX_NODE_DISPLACEMENT=d.MAX_NODE_DISPLACEMENT_INCREMENTAL*3,d.MIN_REPULSION_DIST=d.DEFAULT_EDGE_LENGTH/10,d.CONVERGENCE_CHECK_PERIOD=100,d.PER_LEVEL_IDEAL_EDGE_LENGTH_FACTOR=.1,d.MIN_EDGE_LENGTH=1,d.GRID_CALCULATION_CHECK_PERIOD=10,o.exports=d},function(o,l,f){var b=f(12);function d(){}d.calcSeparationAmount=function(w,y,k,E){if(!w.intersects(y))throw"assert failed";var T=new Array(2);this.decideDirectionsForOverlappingNodes(w,y,T),k[0]=Math.min(w.getRight(),y.getRight())-Math.max(w.x,y.x),k[1]=Math.min(w.getBottom(),y.getBottom())-Math.max(w.y,y.y),w.getX()<=y.getX()&&w.getRight()>=y.getRight()?k[0]+=Math.min(y.getX()-w.getX(),w.getRight()-y.getRight()):y.getX()<=w.getX()&&y.getRight()>=w.getRight()&&(k[0]+=Math.min(w.getX()-y.getX(),y.getRight()-w.getRight())),w.getY()<=y.getY()&&w.getBottom()>=y.getBottom()?k[1]+=Math.min(y.getY()-w.getY(),w.getBottom()-y.getBottom()):y.getY()<=w.getY()&&y.getBottom()>=w.getBottom()&&(k[1]+=Math.min(w.getY()-y.getY(),y.getBottom()-w.getBottom()));var C=Math.abs((y.getCenterY()-w.getCenterY())/(y.getCenterX()-w.getCenterX()));y.getCenterY()===w.getCenterY()&&y.getCenterX()===w.getCenterX()&&(C=1);var S=C*k[0],L=k[1]/C;k[0]<L?L=k[0]:S=k[1],k[0]=-1*T[0]*(L/2+E),k[1]=-1*T[1]*(S/2+E)},d.decideDirectionsForOverlappingNodes=function(w,y,k){w.getCenterX()<y.getCenterX()?k[0]=-1:k[0]=1,w.getCenterY()<y.getCenterY()?k[1]=-1:k[1]=1},d.getIntersection2=function(w,y,k){var E=w.getCenterX(),T=w.getCenterY(),C=y.getCenterX(),S=y.getCenterY();if(w.intersects(y))return k[0]=E,k[1]=T,k[2]=C,k[3]=S,!0;var L=w.getX(),O=w.getY(),B=w.getRight(),N=w.getX(),F=w.getBottom(),R=w.getRight(),q=w.getWidthHalf(),X=w.getHeightHalf(),te=y.getX(),H=y.getY(),Y=y.getRight(),z=y.getX(),W=y.getBottom(),Z=y.getRight(),G=y.getWidthHalf(),ae=y.getHeightHalf(),$=!1,ge=!1;if(E===C){if(T>S)return k[0]=E,k[1]=O,k[2]=C,k[3]=W,!1;if(T<S)return k[0]=E,k[1]=F,k[2]=C,k[3]=H,!1}else if(T===S){if(E>C)return k[0]=L,k[1]=T,k[2]=Y,k[3]=S,!1;if(E<C)return k[0]=B,k[1]=T,k[2]=te,k[3]=S,!1}else{var ee=w.height/w.width,de=y.height/y.width,re=(S-T)/(C-E),ke=void 0,Ce=void 0,_e=void 0,Te=void 0,Be=void 0,Ge=void 0;if(-ee===re?E>C?(k[0]=N,k[1]=F,$=!0):(k[0]=B,k[1]=O,$=!0):ee===re&&(E>C?(k[0]=L,k[1]=O,$=!0):(k[0]=R,k[1]=F,$=!0)),-de===re?C>E?(k[2]=z,k[3]=W,ge=!0):(k[2]=Y,k[3]=H,ge=!0):de===re&&(C>E?(k[2]=te,k[3]=H,ge=!0):(k[2]=Z,k[3]=W,ge=!0)),$&&ge)return!1;if(E>C?T>S?(ke=this.getCardinalDirection(ee,re,4),Ce=this.getCardinalDirection(de,re,2)):(ke=this.getCardinalDirection(-ee,re,3),Ce=this.getCardinalDirection(-de,re,1)):T>S?(ke=this.getCardinalDirection(-ee,re,1),Ce=this.getCardinalDirection(-de,re,3)):(ke=this.getCardinalDirection(ee,re,2),Ce=this.getCardinalDirection(de,re,4)),!$)switch(ke){case 1:Te=O,_e=E+-X/re,k[0]=_e,k[1]=Te;break;case 2:_e=R,Te=T+q*re,k[0]=_e,k[1]=Te;break;case 3:Te=F,_e=E+X/re,k[0]=_e,k[1]=Te;break;case 4:_e=N,Te=T+-q*re,k[0]=_e,k[1]=Te;break}if(!ge)switch(Ce){case 1:Ge=H,Be=C+-ae/re,k[2]=Be,k[3]=Ge;break;case 2:Be=Z,Ge=S+G*re,k[2]=Be,k[3]=Ge;break;case 3:Ge=W,Be=C+ae/re,k[2]=Be,k[3]=Ge;break;case 4:Be=z,Ge=S+-G*re,k[2]=Be,k[3]=Ge;break}}return!1},d.getCardinalDirection=function(w,y,k){return w>y?k:1+k%4},d.getIntersection=function(w,y,k,E){if(E==null)return this.getIntersection2(w,y,k);var T=w.x,C=w.y,S=y.x,L=y.y,O=k.x,B=k.y,N=E.x,F=E.y,R=void 0,q=void 0,X=void 0,te=void 0,H=void 0,Y=void 0,z=void 0,W=void 0,Z=void 0;return X=L-C,H=T-S,z=S*C-T*L,te=F-B,Y=O-N,W=N*B-O*F,Z=X*Y-te*H,Z===0?null:(R=(H*W-Y*z)/Z,q=(te*z-X*W)/Z,new b(R,q))},d.angleOfVector=function(w,y,k,E){var T=void 0;return w!==k?(T=Math.atan((E-y)/(k-w)),k<w?T+=Math.PI:E<y&&(T+=this.TWO_PI)):E<y?T=this.ONE_AND_HALF_PI:T=this.HALF_PI,T},d.doIntersect=function(w,y,k,E){var T=w.x,C=w.y,S=y.x,L=y.y,O=k.x,B=k.y,N=E.x,F=E.y,R=(S-T)*(F-B)-(N-O)*(L-C);if(R===0)return!1;var q=((F-B)*(N-T)+(O-N)*(F-C))/R,X=((C-L)*(N-T)+(S-T)*(F-C))/R;return 0<q&&q<1&&0<X&&X<1},d.HALF_PI=.5*Math.PI,d.ONE_AND_HALF_PI=1.5*Math.PI,d.TWO_PI=2*Math.PI,d.THREE_PI=3*Math.PI,o.exports=d},function(o,l,f){function b(){}b.sign=function(d){return d>0?1:d<0?-1:0},b.floor=function(d){return d<0?Math.ceil(d):Math.floor(d)},b.ceil=function(d){return d<0?Math.floor(d):Math.ceil(d)},o.exports=b},function(o,l,f){function b(){}b.MAX_VALUE=2147483647,b.MIN_VALUE=-2147483648,o.exports=b},function(o,l,f){var b=function(){function T(C,S){for(var L=0;L<S.length;L++){var O=S[L];O.enumerable=O.enumerable||!1,O.configurable=!0,"value"in O&&(O.writable=!0),Object.defineProperty(C,O.key,O)}}return function(C,S,L){return S&&T(C.prototype,S),L&&T(C,L),C}}();function d(T,C){if(!(T instanceof C))throw new TypeError("Cannot call a class as a function")}var w=function(C){return{value:C,next:null,prev:null}},y=function(C,S,L,O){return C!==null?C.next=S:O.head=S,L!==null?L.prev=S:O.tail=S,S.prev=C,S.next=L,O.length++,S},k=function(C,S){var L=C.prev,O=C.next;return L!==null?L.next=O:S.head=O,O!==null?O.prev=L:S.tail=L,C.prev=C.next=null,S.length--,C},E=function(){function T(C){var S=this;d(this,T),this.length=0,this.head=null,this.tail=null,C!=null&&C.forEach(function(L){return S.push(L)})}return b(T,[{key:"size",value:function(){return this.length}},{key:"insertBefore",value:function(S,L){return y(L.prev,w(S),L,this)}},{key:"insertAfter",value:function(S,L){return y(L,w(S),L.next,this)}},{key:"insertNodeBefore",value:function(S,L){return y(L.prev,S,L,this)}},{key:"insertNodeAfter",value:function(S,L){return y(L,S,L.next,this)}},{key:"push",value:function(S){return y(this.tail,w(S),null,this)}},{key:"unshift",value:function(S){return y(null,w(S),this.head,this)}},{key:"remove",value:function(S){return k(S,this)}},{key:"pop",value:function(){return k(this.tail,this).value}},{key:"popNode",value:function(){return k(this.tail,this)}},{key:"shift",value:function(){return k(this.head,this).value}},{key:"shiftNode",value:function(){return k(this.head,this)}},{key:"get_object_at",value:function(S){if(S<=this.length()){for(var L=1,O=this.head;L<S;)O=O.next,L++;return O.value}}},{key:"set_object_at",value:function(S,L){if(S<=this.length()){for(var O=1,B=this.head;O<S;)B=B.next,O++;B.value=L}}}]),T}();o.exports=E},function(o,l,f){function b(d,w,y){this.x=null,this.y=null,d==null&&w==null&&y==null?(this.x=0,this.y=0):typeof d=="number"&&typeof w=="number"&&y==null?(this.x=d,this.y=w):d.constructor.name=="Point"&&w==null&&y==null&&(y=d,this.x=y.x,this.y=y.y)}b.prototype.getX=function(){return this.x},b.prototype.getY=function(){return this.y},b.prototype.getLocation=function(){return new b(this.x,this.y)},b.prototype.setLocation=function(d,w,y){d.constructor.name=="Point"&&w==null&&y==null?(y=d,this.setLocation(y.x,y.y)):typeof d=="number"&&typeof w=="number"&&y==null&&(parseInt(d)==d&&parseInt(w)==w?this.move(d,w):(this.x=Math.floor(d+.5),this.y=Math.floor(w+.5)))},b.prototype.move=function(d,w){this.x=d,this.y=w},b.prototype.translate=function(d,w){this.x+=d,this.y+=w},b.prototype.equals=function(d){if(d.constructor.name=="Point"){var w=d;return this.x==w.x&&this.y==w.y}return this==d},b.prototype.toString=function(){return new b().constructor.name+"[x="+this.x+",y="+this.y+"]"},o.exports=b},function(o,l,f){function b(d,w,y,k){this.x=0,this.y=0,this.width=0,this.height=0,d!=null&&w!=null&&y!=null&&k!=null&&(this.x=d,this.y=w,this.width=y,this.height=k)}b.prototype.getX=function(){return this.x},b.prototype.setX=function(d){this.x=d},b.prototype.getY=function(){return this.y},b.prototype.setY=function(d){this.y=d},b.prototype.getWidth=function(){return this.width},b.prototype.setWidth=function(d){this.width=d},b.prototype.getHeight=function(){return this.height},b.prototype.setHeight=function(d){this.height=d},b.prototype.getRight=function(){return this.x+this.width},b.prototype.getBottom=function(){return this.y+this.height},b.prototype.intersects=function(d){return!(this.getRight()<d.x||this.getBottom()<d.y||d.getRight()<this.x||d.getBottom()<this.y)},b.prototype.getCenterX=function(){return this.x+this.width/2},b.prototype.getMinX=function(){return this.getX()},b.prototype.getMaxX=function(){return this.getX()+this.width},b.prototype.getCenterY=function(){return this.y+this.height/2},b.prototype.getMinY=function(){return this.getY()},b.prototype.getMaxY=function(){return this.getY()+this.height},b.prototype.getWidthHalf=function(){return this.width/2},b.prototype.getHeightHalf=function(){return this.height/2},o.exports=b},function(o,l,f){var b=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(w){return typeof w}:function(w){return w&&typeof Symbol=="function"&&w.constructor===Symbol&&w!==Symbol.prototype?"symbol":typeof w};function d(){}d.lastID=0,d.createID=function(w){return d.isPrimitive(w)?w:(w.uniqueID!=null||(w.uniqueID=d.getString(),d.lastID++),w.uniqueID)},d.getString=function(w){return w==null&&(w=d.lastID),"Object#"+w},d.isPrimitive=function(w){var y=typeof w>"u"?"undefined":b(w);return w==null||y!="object"&&y!="function"},o.exports=d},function(o,l,f){function b(O){if(Array.isArray(O)){for(var B=0,N=Array(O.length);B<O.length;B++)N[B]=O[B];return N}else return Array.from(O)}var d=f(0),w=f(6),y=f(3),k=f(1),E=f(5),T=f(4),C=f(17),S=f(27);function L(O){S.call(this),this.layoutQuality=d.QUALITY,this.createBendsAsNeeded=d.DEFAULT_CREATE_BENDS_AS_NEEDED,this.incremental=d.DEFAULT_INCREMENTAL,this.animationOnLayout=d.DEFAULT_ANIMATION_ON_LAYOUT,this.animationDuringLayout=d.DEFAULT_ANIMATION_DURING_LAYOUT,this.animationPeriod=d.DEFAULT_ANIMATION_PERIOD,this.uniformLeafNodeSizes=d.DEFAULT_UNIFORM_LEAF_NODE_SIZES,this.edgeToDummyNodes=new Map,this.graphManager=new w(this),this.isLayoutFinished=!1,this.isSubLayout=!1,this.isRemoteUse=!1,O!=null&&(this.isRemoteUse=O)}L.RANDOM_SEED=1,L.prototype=Object.create(S.prototype),L.prototype.getGraphManager=function(){return this.graphManager},L.prototype.getAllNodes=function(){return this.graphManager.getAllNodes()},L.prototype.getAllEdges=function(){return this.graphManager.getAllEdges()},L.prototype.getAllNodesToApplyGravitation=function(){return this.graphManager.getAllNodesToApplyGravitation()},L.prototype.newGraphManager=function(){var O=new w(this);return this.graphManager=O,O},L.prototype.newGraph=function(O){return new E(null,this.graphManager,O)},L.prototype.newNode=function(O){return new y(this.graphManager,O)},L.prototype.newEdge=function(O){return new k(null,null,O)},L.prototype.checkLayoutSuccess=function(){return this.graphManager.getRoot()==null||this.graphManager.getRoot().getNodes().length==0||this.graphManager.includesInvalidEdge()},L.prototype.runLayout=function(){this.isLayoutFinished=!1,this.tilingPreLayout&&this.tilingPreLayout(),this.initParameters();var O;return this.checkLayoutSuccess()?O=!1:O=this.layout(),d.ANIMATE==="during"?!1:(O&&(this.isSubLayout||this.doPostLayout()),this.tilingPostLayout&&this.tilingPostLayout(),this.isLayoutFinished=!0,O)},L.prototype.doPostLayout=function(){this.incremental||this.transform(),this.update()},L.prototype.update2=function(){if(this.createBendsAsNeeded&&(this.createBendpointsFromDummyNodes(),this.graphManager.resetAllEdges()),!this.isRemoteUse){for(var O=this.graphManager.getAllEdges(),B=0;B<O.length;B++)O[B];for(var N=this.graphManager.getRoot().getNodes(),B=0;B<N.length;B++)N[B];this.update(this.graphManager.getRoot())}},L.prototype.update=function(O){if(O==null)this.update2();else if(O instanceof y){var B=O;if(B.getChild()!=null)for(var N=B.getChild().getNodes(),F=0;F<N.length;F++)update(N[F]);if(B.vGraphObject!=null){var R=B.vGraphObject;R.update(B)}}else if(O instanceof k){var q=O;if(q.vGraphObject!=null){var X=q.vGraphObject;X.update(q)}}else if(O instanceof E){var te=O;if(te.vGraphObject!=null){var H=te.vGraphObject;H.update(te)}}},L.prototype.initParameters=function(){this.isSubLayout||(this.layoutQuality=d.QUALITY,this.animationDuringLayout=d.DEFAULT_ANIMATION_DURING_LAYOUT,this.animationPeriod=d.DEFAULT_ANIMATION_PERIOD,this.animationOnLayout=d.DEFAULT_ANIMATION_ON_LAYOUT,this.incremental=d.DEFAULT_INCREMENTAL,this.createBendsAsNeeded=d.DEFAULT_CREATE_BENDS_AS_NEEDED,this.uniformLeafNodeSizes=d.DEFAULT_UNIFORM_LEAF_NODE_SIZES),this.animationDuringLayout&&(this.animationOnLayout=!1)},L.prototype.transform=function(O){if(O==null)this.transform(new T(0,0));else{var B=new C,N=this.graphManager.getRoot().updateLeftTop();if(N!=null){B.setWorldOrgX(O.x),B.setWorldOrgY(O.y),B.setDeviceOrgX(N.x),B.setDeviceOrgY(N.y);for(var F=this.getAllNodes(),R,q=0;q<F.length;q++)R=F[q],R.transform(B)}}},L.prototype.positionNodesRandomly=function(O){if(O==null)this.positionNodesRandomly(this.getGraphManager().getRoot()),this.getGraphManager().getRoot().updateBounds(!0);else for(var B,N,F=O.getNodes(),R=0;R<F.length;R++)B=F[R],N=B.getChild(),N==null||N.getNodes().length==0?B.scatter():(this.positionNodesRandomly(N),B.updateBounds())},L.prototype.getFlatForest=function(){for(var O=[],B=!0,N=this.graphManager.getRoot().getNodes(),F=!0,R=0;R<N.length;R++)N[R].getChild()!=null&&(F=!1);if(!F)return O;var q=new Set,X=[],te=new Map,H=[];for(H=H.concat(N);H.length>0&&B;){for(X.push(H[0]);X.length>0&&B;){var Y=X[0];X.splice(0,1),q.add(Y);for(var z=Y.getEdges(),R=0;R<z.length;R++){var W=z[R].getOtherEnd(Y);if(te.get(Y)!=W)if(!q.has(W))X.push(W),te.set(W,Y);else{B=!1;break}}}if(!B)O=[];else{var Z=[].concat(b(q));O.push(Z);for(var R=0;R<Z.length;R++){var G=Z[R],ae=H.indexOf(G);ae>-1&&H.splice(ae,1)}q=new Set,te=new Map}}return O},L.prototype.createDummyNodesForBendpoints=function(O){for(var B=[],N=O.source,F=this.graphManager.calcLowestCommonAncestor(O.source,O.target),R=0;R<O.bendpoints.length;R++){var q=this.newNode(null);q.setRect(new Point(0,0),new Dimension(1,1)),F.add(q);var X=this.newEdge(null);this.graphManager.add(X,N,q),B.add(q),N=q}var X=this.newEdge(null);return this.graphManager.add(X,N,O.target),this.edgeToDummyNodes.set(O,B),O.isInterGraph()?this.graphManager.remove(O):F.remove(O),B},L.prototype.createBendpointsFromDummyNodes=function(){var O=[];O=O.concat(this.graphManager.getAllEdges()),O=[].concat(b(this.edgeToDummyNodes.keys())).concat(O);for(var B=0;B<O.length;B++){var N=O[B];if(N.bendpoints.length>0){for(var F=this.edgeToDummyNodes.get(N),R=0;R<F.length;R++){var q=F[R],X=new T(q.getCenterX(),q.getCenterY()),te=N.bendpoints.get(R);te.x=X.x,te.y=X.y,q.getOwner().remove(q)}this.graphManager.add(N,N.source,N.target)}}},L.transform=function(O,B,N,F){if(N!=null&&F!=null){var R=B;if(O<=50){var q=B/N;R-=(B-q)/50*(50-O)}else{var X=B*F;R+=(X-B)/50*(O-50)}return R}else{var te,H;return O<=50?(te=9*B/500,H=B/10):(te=9*B/50,H=-8*B),te*O+H}},L.findCenterOfTree=function(O){var B=[];B=B.concat(O);var N=[],F=new Map,R=!1,q=null;(B.length==1||B.length==2)&&(R=!0,q=B[0]);for(var X=0;X<B.length;X++){var te=B[X],H=te.getNeighborsList().size;F.set(te,te.getNeighborsList().size),H==1&&N.push(te)}var Y=[];for(Y=Y.concat(N);!R;){var z=[];z=z.concat(Y),Y=[];for(var X=0;X<B.length;X++){var te=B[X],W=B.indexOf(te);W>=0&&B.splice(W,1);var Z=te.getNeighborsList();Z.forEach(function($){if(N.indexOf($)<0){var ge=F.get($),ee=ge-1;ee==1&&Y.push($),F.set($,ee)}})}N=N.concat(Y),(B.length==1||B.length==2)&&(R=!0,q=B[0])}return q},L.prototype.setGraphManager=function(O){this.graphManager=O},o.exports=L},function(o,l,f){function b(){}b.seed=1,b.x=0,b.nextDouble=function(){return b.x=Math.sin(b.seed++)*1e4,b.x-Math.floor(b.x)},o.exports=b},function(o,l,f){var b=f(4);function d(w,y){this.lworldOrgX=0,this.lworldOrgY=0,this.ldeviceOrgX=0,this.ldeviceOrgY=0,this.lworldExtX=1,this.lworldExtY=1,this.ldeviceExtX=1,this.ldeviceExtY=1}d.prototype.getWorldOrgX=function(){return this.lworldOrgX},d.prototype.setWorldOrgX=function(w){this.lworldOrgX=w},d.prototype.getWorldOrgY=function(){return this.lworldOrgY},d.prototype.setWorldOrgY=function(w){this.lworldOrgY=w},d.prototype.getWorldExtX=function(){return this.lworldExtX},d.prototype.setWorldExtX=function(w){this.lworldExtX=w},d.prototype.getWorldExtY=function(){return this.lworldExtY},d.prototype.setWorldExtY=function(w){this.lworldExtY=w},d.prototype.getDeviceOrgX=function(){return this.ldeviceOrgX},d.prototype.setDeviceOrgX=function(w){this.ldeviceOrgX=w},d.prototype.getDeviceOrgY=function(){return this.ldeviceOrgY},d.prototype.setDeviceOrgY=function(w){this.ldeviceOrgY=w},d.prototype.getDeviceExtX=function(){return this.ldeviceExtX},d.prototype.setDeviceExtX=function(w){this.ldeviceExtX=w},d.prototype.getDeviceExtY=function(){return this.ldeviceExtY},d.prototype.setDeviceExtY=function(w){this.ldeviceExtY=w},d.prototype.transformX=function(w){var y=0,k=this.lworldExtX;return k!=0&&(y=this.ldeviceOrgX+(w-this.lworldOrgX)*this.ldeviceExtX/k),y},d.prototype.transformY=function(w){var y=0,k=this.lworldExtY;return k!=0&&(y=this.ldeviceOrgY+(w-this.lworldOrgY)*this.ldeviceExtY/k),y},d.prototype.inverseTransformX=function(w){var y=0,k=this.ldeviceExtX;return k!=0&&(y=this.lworldOrgX+(w-this.ldeviceOrgX)*this.lworldExtX/k),y},d.prototype.inverseTransformY=function(w){var y=0,k=this.ldeviceExtY;return k!=0&&(y=this.lworldOrgY+(w-this.ldeviceOrgY)*this.lworldExtY/k),y},d.prototype.inverseTransformPoint=function(w){var y=new b(this.inverseTransformX(w.x),this.inverseTransformY(w.y));return y},o.exports=d},function(o,l,f){function b(S){if(Array.isArray(S)){for(var L=0,O=Array(S.length);L<S.length;L++)O[L]=S[L];return O}else return Array.from(S)}var d=f(15),w=f(7),y=f(0),k=f(8),E=f(9);function T(){d.call(this),this.useSmartIdealEdgeLengthCalculation=w.DEFAULT_USE_SMART_IDEAL_EDGE_LENGTH_CALCULATION,this.idealEdgeLength=w.DEFAULT_EDGE_LENGTH,this.springConstant=w.DEFAULT_SPRING_STRENGTH,this.repulsionConstant=w.DEFAULT_REPULSION_STRENGTH,this.gravityConstant=w.DEFAULT_GRAVITY_STRENGTH,this.compoundGravityConstant=w.DEFAULT_COMPOUND_GRAVITY_STRENGTH,this.gravityRangeFactor=w.DEFAULT_GRAVITY_RANGE_FACTOR,this.compoundGravityRangeFactor=w.DEFAULT_COMPOUND_GRAVITY_RANGE_FACTOR,this.displacementThresholdPerNode=3*w.DEFAULT_EDGE_LENGTH/100,this.coolingFactor=w.DEFAULT_COOLING_FACTOR_INCREMENTAL,this.initialCoolingFactor=w.DEFAULT_COOLING_FACTOR_INCREMENTAL,this.totalDisplacement=0,this.oldTotalDisplacement=0,this.maxIterations=w.MAX_ITERATIONS}T.prototype=Object.create(d.prototype);for(var C in d)T[C]=d[C];T.prototype.initParameters=function(){d.prototype.initParameters.call(this,arguments),this.totalIterations=0,this.notAnimatedIterations=0,this.useFRGridVariant=w.DEFAULT_USE_SMART_REPULSION_RANGE_CALCULATION,this.grid=[]},T.prototype.calcIdealEdgeLengths=function(){for(var S,L,O,B,N,F,R=this.getGraphManager().getAllEdges(),q=0;q<R.length;q++)S=R[q],S.idealLength=this.idealEdgeLength,S.isInterGraph&&(O=S.getSource(),B=S.getTarget(),N=S.getSourceInLca().getEstimatedSize(),F=S.getTargetInLca().getEstimatedSize(),this.useSmartIdealEdgeLengthCalculation&&(S.idealLength+=N+F-2*y.SIMPLE_NODE_SIZE),L=S.getLca().getInclusionTreeDepth(),S.idealLength+=w.DEFAULT_EDGE_LENGTH*w.PER_LEVEL_IDEAL_EDGE_LENGTH_FACTOR*(O.getInclusionTreeDepth()+B.getInclusionTreeDepth()-2*L))},T.prototype.initSpringEmbedder=function(){var S=this.getAllNodes().length;this.incremental?(S>w.ADAPTATION_LOWER_NODE_LIMIT&&(this.coolingFactor=Math.max(this.coolingFactor*w.COOLING_ADAPTATION_FACTOR,this.coolingFactor-(S-w.ADAPTATION_LOWER_NODE_LIMIT)/(w.ADAPTATION_UPPER_NODE_LIMIT-w.ADAPTATION_LOWER_NODE_LIMIT)*this.coolingFactor*(1-w.COOLING_ADAPTATION_FACTOR))),this.maxNodeDisplacement=w.MAX_NODE_DISPLACEMENT_INCREMENTAL):(S>w.ADAPTATION_LOWER_NODE_LIMIT?this.coolingFactor=Math.max(w.COOLING_ADAPTATION_FACTOR,1-(S-w.ADAPTATION_LOWER_NODE_LIMIT)/(w.ADAPTATION_UPPER_NODE_LIMIT-w.ADAPTATION_LOWER_NODE_LIMIT)*(1-w.COOLING_ADAPTATION_FACTOR)):this.coolingFactor=1,this.initialCoolingFactor=this.coolingFactor,this.maxNodeDisplacement=w.MAX_NODE_DISPLACEMENT),this.maxIterations=Math.max(this.getAllNodes().length*5,this.maxIterations),this.totalDisplacementThreshold=this.displacementThresholdPerNode*this.getAllNodes().length,this.repulsionRange=this.calcRepulsionRange()},T.prototype.calcSpringForces=function(){for(var S=this.getAllEdges(),L,O=0;O<S.length;O++)L=S[O],this.calcSpringForce(L,L.idealLength)},T.prototype.calcRepulsionForces=function(){var S=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!0,L=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,O,B,N,F,R=this.getAllNodes(),q;if(this.useFRGridVariant)for(this.totalIterations%w.GRID_CALCULATION_CHECK_PERIOD==1&&S&&this.updateGrid(),q=new Set,O=0;O<R.length;O++)N=R[O],this.calculateRepulsionForceOfANode(N,q,S,L),q.add(N);else for(O=0;O<R.length;O++)for(N=R[O],B=O+1;B<R.length;B++)F=R[B],N.getOwner()==F.getOwner()&&this.calcRepulsionForce(N,F)},T.prototype.calcGravitationalForces=function(){for(var S,L=this.getAllNodesToApplyGravitation(),O=0;O<L.length;O++)S=L[O],this.calcGravitationalForce(S)},T.prototype.moveNodes=function(){for(var S=this.getAllNodes(),L,O=0;O<S.length;O++)L=S[O],L.move()},T.prototype.calcSpringForce=function(S,L){var O=S.getSource(),B=S.getTarget(),N,F,R,q;if(this.uniformLeafNodeSizes&&O.getChild()==null&&B.getChild()==null)S.updateLengthSimple();else if(S.updateLength(),S.isOverlapingSourceAndTarget)return;N=S.getLength(),N!=0&&(F=this.springConstant*(N-L),R=F*(S.lengthX/N),q=F*(S.lengthY/N),O.springForceX+=R,O.springForceY+=q,B.springForceX-=R,B.springForceY-=q)},T.prototype.calcRepulsionForce=function(S,L){var O=S.getRect(),B=L.getRect(),N=new Array(2),F=new Array(4),R,q,X,te,H,Y,z;if(O.intersects(B)){k.calcSeparationAmount(O,B,N,w.DEFAULT_EDGE_LENGTH/2),Y=2*N[0],z=2*N[1];var W=S.noOfChildren*L.noOfChildren/(S.noOfChildren+L.noOfChildren);S.repulsionForceX-=W*Y,S.repulsionForceY-=W*z,L.repulsionForceX+=W*Y,L.repulsionForceY+=W*z}else this.uniformLeafNodeSizes&&S.getChild()==null&&L.getChild()==null?(R=B.getCenterX()-O.getCenterX(),q=B.getCenterY()-O.getCenterY()):(k.getIntersection(O,B,F),R=F[2]-F[0],q=F[3]-F[1]),Math.abs(R)<w.MIN_REPULSION_DIST&&(R=E.sign(R)*w.MIN_REPULSION_DIST),Math.abs(q)<w.MIN_REPULSION_DIST&&(q=E.sign(q)*w.MIN_REPULSION_DIST),X=R*R+q*q,te=Math.sqrt(X),H=this.repulsionConstant*S.noOfChildren*L.noOfChildren/X,Y=H*R/te,z=H*q/te,S.repulsionForceX-=Y,S.repulsionForceY-=z,L.repulsionForceX+=Y,L.repulsionForceY+=z},T.prototype.calcGravitationalForce=function(S){var L,O,B,N,F,R,q,X;L=S.getOwner(),O=(L.getRight()+L.getLeft())/2,B=(L.getTop()+L.getBottom())/2,N=S.getCenterX()-O,F=S.getCenterY()-B,R=Math.abs(N)+S.getWidth()/2,q=Math.abs(F)+S.getHeight()/2,S.getOwner()==this.graphManager.getRoot()?(X=L.getEstimatedSize()*this.gravityRangeFactor,(R>X||q>X)&&(S.gravitationForceX=-this.gravityConstant*N,S.gravitationForceY=-this.gravityConstant*F)):(X=L.getEstimatedSize()*this.compoundGravityRangeFactor,(R>X||q>X)&&(S.gravitationForceX=-this.gravityConstant*N*this.compoundGravityConstant,S.gravitationForceY=-this.gravityConstant*F*this.compoundGravityConstant))},T.prototype.isConverged=function(){var S,L=!1;return this.totalIterations>this.maxIterations/3&&(L=Math.abs(this.totalDisplacement-this.oldTotalDisplacement)<2),S=this.totalDisplacement<this.totalDisplacementThreshold,this.oldTotalDisplacement=this.totalDisplacement,S||L},T.prototype.animate=function(){this.animationDuringLayout&&!this.isSubLayout&&(this.notAnimatedIterations==this.animationPeriod?(this.update(),this.notAnimatedIterations=0):this.notAnimatedIterations++)},T.prototype.calcNoOfChildrenForAllNodes=function(){for(var S,L=this.graphManager.getAllNodes(),O=0;O<L.length;O++)S=L[O],S.noOfChildren=S.getNoOfChildren()},T.prototype.calcGrid=function(S){var L=0,O=0;L=parseInt(Math.ceil((S.getRight()-S.getLeft())/this.repulsionRange)),O=parseInt(Math.ceil((S.getBottom()-S.getTop())/this.repulsionRange));for(var B=new Array(L),N=0;N<L;N++)B[N]=new Array(O);for(var N=0;N<L;N++)for(var F=0;F<O;F++)B[N][F]=new Array;return B},T.prototype.addNodeToGrid=function(S,L,O){var B=0,N=0,F=0,R=0;B=parseInt(Math.floor((S.getRect().x-L)/this.repulsionRange)),N=parseInt(Math.floor((S.getRect().width+S.getRect().x-L)/this.repulsionRange)),F=parseInt(Math.floor((S.getRect().y-O)/this.repulsionRange)),R=parseInt(Math.floor((S.getRect().height+S.getRect().y-O)/this.repulsionRange));for(var q=B;q<=N;q++)for(var X=F;X<=R;X++)this.grid[q][X].push(S),S.setGridCoordinates(B,N,F,R)},T.prototype.updateGrid=function(){var S,L,O=this.getAllNodes();for(this.grid=this.calcGrid(this.graphManager.getRoot()),S=0;S<O.length;S++)L=O[S],this.addNodeToGrid(L,this.graphManager.getRoot().getLeft(),this.graphManager.getRoot().getTop())},T.prototype.calculateRepulsionForceOfANode=function(S,L,O,B){if(this.totalIterations%w.GRID_CALCULATION_CHECK_PERIOD==1&&O||B){var N=new Set;S.surrounding=new Array;for(var F,R=this.grid,q=S.startX-1;q<S.finishX+2;q++)for(var X=S.startY-1;X<S.finishY+2;X++)if(!(q<0||X<0||q>=R.length||X>=R[0].length)){for(var te=0;te<R[q][X].length;te++)if(F=R[q][X][te],!(S.getOwner()!=F.getOwner()||S==F)&&!L.has(F)&&!N.has(F)){var H=Math.abs(S.getCenterX()-F.getCenterX())-(S.getWidth()/2+F.getWidth()/2),Y=Math.abs(S.getCenterY()-F.getCenterY())-(S.getHeight()/2+F.getHeight()/2);H<=this.repulsionRange&&Y<=this.repulsionRange&&N.add(F)}}S.surrounding=[].concat(b(N))}for(q=0;q<S.surrounding.length;q++)this.calcRepulsionForce(S,S.surrounding[q])},T.prototype.calcRepulsionRange=function(){return 0},o.exports=T},function(o,l,f){var b=f(1),d=f(7);function w(k,E,T){b.call(this,k,E,T),this.idealLength=d.DEFAULT_EDGE_LENGTH}w.prototype=Object.create(b.prototype);for(var y in b)w[y]=b[y];o.exports=w},function(o,l,f){var b=f(3);function d(y,k,E,T){b.call(this,y,k,E,T),this.springForceX=0,this.springForceY=0,this.repulsionForceX=0,this.repulsionForceY=0,this.gravitationForceX=0,this.gravitationForceY=0,this.displacementX=0,this.displacementY=0,this.startX=0,this.finishX=0,this.startY=0,this.finishY=0,this.surrounding=[]}d.prototype=Object.create(b.prototype);for(var w in b)d[w]=b[w];d.prototype.setGridCoordinates=function(y,k,E,T){this.startX=y,this.finishX=k,this.startY=E,this.finishY=T},o.exports=d},function(o,l,f){function b(d,w){this.width=0,this.height=0,d!==null&&w!==null&&(this.height=w,this.width=d)}b.prototype.getWidth=function(){return this.width},b.prototype.setWidth=function(d){this.width=d},b.prototype.getHeight=function(){return this.height},b.prototype.setHeight=function(d){this.height=d},o.exports=b},function(o,l,f){var b=f(14);function d(){this.map={},this.keys=[]}d.prototype.put=function(w,y){var k=b.createID(w);this.contains(k)||(this.map[k]=y,this.keys.push(w))},d.prototype.contains=function(w){return b.createID(w),this.map[w]!=null},d.prototype.get=function(w){var y=b.createID(w);return this.map[y]},d.prototype.keySet=function(){return this.keys},o.exports=d},function(o,l,f){var b=f(14);function d(){this.set={}}d.prototype.add=function(w){var y=b.createID(w);this.contains(y)||(this.set[y]=w)},d.prototype.remove=function(w){delete this.set[b.createID(w)]},d.prototype.clear=function(){this.set={}},d.prototype.contains=function(w){return this.set[b.createID(w)]==w},d.prototype.isEmpty=function(){return this.size()===0},d.prototype.size=function(){return Object.keys(this.set).length},d.prototype.addAllTo=function(w){for(var y=Object.keys(this.set),k=y.length,E=0;E<k;E++)w.push(this.set[y[E]])},d.prototype.size=function(){return Object.keys(this.set).length},d.prototype.addAll=function(w){for(var y=w.length,k=0;k<y;k++){var E=w[k];this.add(E)}},o.exports=d},function(o,l,f){var b=function(){function k(E,T){for(var C=0;C<T.length;C++){var S=T[C];S.enumerable=S.enumerable||!1,S.configurable=!0,"value"in S&&(S.writable=!0),Object.defineProperty(E,S.key,S)}}return function(E,T,C){return T&&k(E.prototype,T),C&&k(E,C),E}}();function d(k,E){if(!(k instanceof E))throw new TypeError("Cannot call a class as a function")}var w=f(11),y=function(){function k(E,T){d(this,k),(T!==null||T!==void 0)&&(this.compareFunction=this._defaultCompareFunction);var C=void 0;E instanceof w?C=E.size():C=E.length,this._quicksort(E,0,C-1)}return b(k,[{key:"_quicksort",value:function(T,C,S){if(C<S){var L=this._partition(T,C,S);this._quicksort(T,C,L),this._quicksort(T,L+1,S)}}},{key:"_partition",value:function(T,C,S){for(var L=this._get(T,C),O=C,B=S;;){for(;this.compareFunction(L,this._get(T,B));)B--;for(;this.compareFunction(this._get(T,O),L);)O++;if(O<B)this._swap(T,O,B),O++,B--;else return B}}},{key:"_get",value:function(T,C){return T instanceof w?T.get_object_at(C):T[C]}},{key:"_set",value:function(T,C,S){T instanceof w?T.set_object_at(C,S):T[C]=S}},{key:"_swap",value:function(T,C,S){var L=this._get(T,C);this._set(T,C,this._get(T,S)),this._set(T,S,L)}},{key:"_defaultCompareFunction",value:function(T,C){return C>T}}]),k}();o.exports=y},function(o,l,f){var b=function(){function y(k,E){for(var T=0;T<E.length;T++){var C=E[T];C.enumerable=C.enumerable||!1,C.configurable=!0,"value"in C&&(C.writable=!0),Object.defineProperty(k,C.key,C)}}return function(k,E,T){return E&&y(k.prototype,E),T&&y(k,T),k}}();function d(y,k){if(!(y instanceof k))throw new TypeError("Cannot call a class as a function")}var w=function(){function y(k,E){var T=arguments.length>2&&arguments[2]!==void 0?arguments[2]:1,C=arguments.length>3&&arguments[3]!==void 0?arguments[3]:-1,S=arguments.length>4&&arguments[4]!==void 0?arguments[4]:-1;d(this,y),this.sequence1=k,this.sequence2=E,this.match_score=T,this.mismatch_penalty=C,this.gap_penalty=S,this.iMax=k.length+1,this.jMax=E.length+1,this.grid=new Array(this.iMax);for(var L=0;L<this.iMax;L++){this.grid[L]=new Array(this.jMax);for(var O=0;O<this.jMax;O++)this.grid[L][O]=0}this.tracebackGrid=new Array(this.iMax);for(var B=0;B<this.iMax;B++){this.tracebackGrid[B]=new Array(this.jMax);for(var N=0;N<this.jMax;N++)this.tracebackGrid[B][N]=[null,null,null]}this.alignments=[],this.score=-1,this.computeGrids()}return b(y,[{key:"getScore",value:function(){return this.score}},{key:"getAlignments",value:function(){return this.alignments}},{key:"computeGrids",value:function(){for(var E=1;E<this.jMax;E++)this.grid[0][E]=this.grid[0][E-1]+this.gap_penalty,this.tracebackGrid[0][E]=[!1,!1,!0];for(var T=1;T<this.iMax;T++)this.grid[T][0]=this.grid[T-1][0]+this.gap_penalty,this.tracebackGrid[T][0]=[!1,!0,!1];for(var C=1;C<this.iMax;C++)for(var S=1;S<this.jMax;S++){var L=void 0;this.sequence1[C-1]===this.sequence2[S-1]?L=this.grid[C-1][S-1]+this.match_score:L=this.grid[C-1][S-1]+this.mismatch_penalty;var O=this.grid[C-1][S]+this.gap_penalty,B=this.grid[C][S-1]+this.gap_penalty,N=[L,O,B],F=this.arrayAllMaxIndexes(N);this.grid[C][S]=N[F[0]],this.tracebackGrid[C][S]=[F.includes(0),F.includes(1),F.includes(2)]}this.score=this.grid[this.iMax-1][this.jMax-1]}},{key:"alignmentTraceback",value:function(){var E=[];for(E.push({pos:[this.sequence1.length,this.sequence2.length],seq1:"",seq2:""});E[0];){var T=E[0],C=this.tracebackGrid[T.pos[0]][T.pos[1]];C[0]&&E.push({pos:[T.pos[0]-1,T.pos[1]-1],seq1:this.sequence1[T.pos[0]-1]+T.seq1,seq2:this.sequence2[T.pos[1]-1]+T.seq2}),C[1]&&E.push({pos:[T.pos[0]-1,T.pos[1]],seq1:this.sequence1[T.pos[0]-1]+T.seq1,seq2:"-"+T.seq2}),C[2]&&E.push({pos:[T.pos[0],T.pos[1]-1],seq1:"-"+T.seq1,seq2:this.sequence2[T.pos[1]-1]+T.seq2}),T.pos[0]===0&&T.pos[1]===0&&this.alignments.push({sequence1:T.seq1,sequence2:T.seq2}),E.shift()}return this.alignments}},{key:"getAllIndexes",value:function(E,T){for(var C=[],S=-1;(S=E.indexOf(T,S+1))!==-1;)C.push(S);return C}},{key:"arrayAllMaxIndexes",value:function(E){return this.getAllIndexes(E,Math.max.apply(null,E))}}]),y}();o.exports=w},function(o,l,f){var b=function(){};b.FDLayout=f(18),b.FDLayoutConstants=f(7),b.FDLayoutEdge=f(19),b.FDLayoutNode=f(20),b.DimensionD=f(21),b.HashMap=f(22),b.HashSet=f(23),b.IGeometry=f(8),b.IMath=f(9),b.Integer=f(10),b.Point=f(12),b.PointD=f(4),b.RandomSeed=f(16),b.RectangleD=f(13),b.Transform=f(17),b.UniqueIDGeneretor=f(14),b.Quicksort=f(24),b.LinkedList=f(11),b.LGraphObject=f(2),b.LGraph=f(5),b.LEdge=f(1),b.LGraphManager=f(6),b.LNode=f(3),b.Layout=f(15),b.LayoutConstants=f(0),b.NeedlemanWunsch=f(25),o.exports=b},function(o,l,f){function b(){this.listeners=[]}var d=b.prototype;d.addListener=function(w,y){this.listeners.push({event:w,callback:y})},d.removeListener=function(w,y){for(var k=this.listeners.length;k>=0;k--){var E=this.listeners[k];E.event===w&&E.callback===y&&this.listeners.splice(k,1)}},d.emit=function(w,y){for(var k=0;k<this.listeners.length;k++){var E=this.listeners[k];w===E.event&&E.callback(y)}},o.exports=b}])})}(jue)),jue.exports}var kBe;function khn(){return kBe||(kBe=1,function(r,i){(function(l,f){r.exports=f(yhn())})(Mm,function(o){return function(l){var f={};function b(d){if(f[d])return f[d].exports;var w=f[d]={i:d,l:!1,exports:{}};return l[d].call(w.exports,w,w.exports,b),w.l=!0,w.exports}return b.m=l,b.c=f,b.i=function(d){return d},b.d=function(d,w,y){b.o(d,w)||Object.defineProperty(d,w,{configurable:!1,enumerable:!0,get:y})},b.n=function(d){var w=d&&d.__esModule?function(){return d.default}:function(){return d};return b.d(w,"a",w),w},b.o=function(d,w){return Object.prototype.hasOwnProperty.call(d,w)},b.p="",b(b.s=7)}([function(l,f){l.exports=o},function(l,f,b){var d=b(0).FDLayoutConstants;function w(){}for(var y in d)w[y]=d[y];w.DEFAULT_USE_MULTI_LEVEL_SCALING=!1,w.DEFAULT_RADIAL_SEPARATION=d.DEFAULT_EDGE_LENGTH,w.DEFAULT_COMPONENT_SEPERATION=60,w.TILE=!0,w.TILING_PADDING_VERTICAL=10,w.TILING_PADDING_HORIZONTAL=10,w.TREE_REDUCTION_ON_INCREMENTAL=!1,l.exports=w},function(l,f,b){var d=b(0).FDLayoutEdge;function w(k,E,T){d.call(this,k,E,T)}w.prototype=Object.create(d.prototype);for(var y in d)w[y]=d[y];l.exports=w},function(l,f,b){var d=b(0).LGraph;function w(k,E,T){d.call(this,k,E,T)}w.prototype=Object.create(d.prototype);for(var y in d)w[y]=d[y];l.exports=w},function(l,f,b){var d=b(0).LGraphManager;function w(k){d.call(this,k)}w.prototype=Object.create(d.prototype);for(var y in d)w[y]=d[y];l.exports=w},function(l,f,b){var d=b(0).FDLayoutNode,w=b(0).IMath;function y(E,T,C,S){d.call(this,E,T,C,S)}y.prototype=Object.create(d.prototype);for(var k in d)y[k]=d[k];y.prototype.move=function(){var E=this.graphManager.getLayout();this.displacementX=E.coolingFactor*(this.springForceX+this.repulsionForceX+this.gravitationForceX)/this.noOfChildren,this.displacementY=E.coolingFactor*(this.springForceY+this.repulsionForceY+this.gravitationForceY)/this.noOfChildren,Math.abs(this.displacementX)>E.coolingFactor*E.maxNodeDisplacement&&(this.displacementX=E.coolingFactor*E.maxNodeDisplacement*w.sign(this.displacementX)),Math.abs(this.displacementY)>E.coolingFactor*E.maxNodeDisplacement&&(this.displacementY=E.coolingFactor*E.maxNodeDisplacement*w.sign(this.displacementY)),this.child==null?this.moveBy(this.displacementX,this.displacementY):this.child.getNodes().length==0?this.moveBy(this.displacementX,this.displacementY):this.propogateDisplacementToChildren(this.displacementX,this.displacementY),E.totalDisplacement+=Math.abs(this.displacementX)+Math.abs(this.displacementY),this.springForceX=0,this.springForceY=0,this.repulsionForceX=0,this.repulsionForceY=0,this.gravitationForceX=0,this.gravitationForceY=0,this.displacementX=0,this.displacementY=0},y.prototype.propogateDisplacementToChildren=function(E,T){for(var C=this.getChild().getNodes(),S,L=0;L<C.length;L++)S=C[L],S.getChild()==null?(S.moveBy(E,T),S.displacementX+=E,S.displacementY+=T):S.propogateDisplacementToChildren(E,T)},y.prototype.setPred1=function(E){this.pred1=E},y.prototype.getPred1=function(){return pred1},y.prototype.getPred2=function(){return pred2},y.prototype.setNext=function(E){this.next=E},y.prototype.getNext=function(){return next},y.prototype.setProcessed=function(E){this.processed=E},y.prototype.isProcessed=function(){return processed},l.exports=y},function(l,f,b){var d=b(0).FDLayout,w=b(4),y=b(3),k=b(5),E=b(2),T=b(1),C=b(0).FDLayoutConstants,S=b(0).LayoutConstants,L=b(0).Point,O=b(0).PointD,B=b(0).Layout,N=b(0).Integer,F=b(0).IGeometry,R=b(0).LGraph,q=b(0).Transform;function X(){d.call(this),this.toBeTiled={}}X.prototype=Object.create(d.prototype);for(var te in d)X[te]=d[te];X.prototype.newGraphManager=function(){var H=new w(this);return this.graphManager=H,H},X.prototype.newGraph=function(H){return new y(null,this.graphManager,H)},X.prototype.newNode=function(H){return new k(this.graphManager,H)},X.prototype.newEdge=function(H){return new E(null,null,H)},X.prototype.initParameters=function(){d.prototype.initParameters.call(this,arguments),this.isSubLayout||(T.DEFAULT_EDGE_LENGTH<10?this.idealEdgeLength=10:this.idealEdgeLength=T.DEFAULT_EDGE_LENGTH,this.useSmartIdealEdgeLengthCalculation=T.DEFAULT_USE_SMART_IDEAL_EDGE_LENGTH_CALCULATION,this.springConstant=C.DEFAULT_SPRING_STRENGTH,this.repulsionConstant=C.DEFAULT_REPULSION_STRENGTH,this.gravityConstant=C.DEFAULT_GRAVITY_STRENGTH,this.compoundGravityConstant=C.DEFAULT_COMPOUND_GRAVITY_STRENGTH,this.gravityRangeFactor=C.DEFAULT_GRAVITY_RANGE_FACTOR,this.compoundGravityRangeFactor=C.DEFAULT_COMPOUND_GRAVITY_RANGE_FACTOR,this.prunedNodesAll=[],this.growTreeIterations=0,this.afterGrowthIterations=0,this.isTreeGrowing=!1,this.isGrowthFinished=!1,this.coolingCycle=0,this.maxCoolingCycle=this.maxIterations/C.CONVERGENCE_CHECK_PERIOD,this.finalTemperature=C.CONVERGENCE_CHECK_PERIOD/this.maxIterations,this.coolingAdjuster=1)},X.prototype.layout=function(){var H=S.DEFAULT_CREATE_BENDS_AS_NEEDED;return H&&(this.createBendpoints(),this.graphManager.resetAllEdges()),this.level=0,this.classicLayout()},X.prototype.classicLayout=function(){if(this.nodesWithGravity=this.calculateNodesToApplyGravitationTo(),this.graphManager.setAllNodesToApplyGravitation(this.nodesWithGravity),this.calcNoOfChildrenForAllNodes(),this.graphManager.calcLowestCommonAncestors(),this.graphManager.calcInclusionTreeDepths(),this.graphManager.getRoot().calcEstimatedSize(),this.calcIdealEdgeLengths(),this.incremental){if(T.TREE_REDUCTION_ON_INCREMENTAL){this.reduceTrees(),this.graphManager.resetAllNodesToApplyGravitation();var Y=new Set(this.getAllNodes()),z=this.nodesWithGravity.filter(function(G){return Y.has(G)});this.graphManager.setAllNodesToApplyGravitation(z)}}else{var H=this.getFlatForest();if(H.length>0)this.positionNodesRadially(H);else{this.reduceTrees(),this.graphManager.resetAllNodesToApplyGravitation();var Y=new Set(this.getAllNodes()),z=this.nodesWithGravity.filter(function(W){return Y.has(W)});this.graphManager.setAllNodesToApplyGravitation(z),this.positionNodesRandomly()}}return this.initSpringEmbedder(),this.runSpringEmbedder(),!0},X.prototype.tick=function(){if(this.totalIterations++,this.totalIterations===this.maxIterations&&!this.isTreeGrowing&&!this.isGrowthFinished)if(this.prunedNodesAll.length>0)this.isTreeGrowing=!0;else return!0;if(this.totalIterations%C.CONVERGENCE_CHECK_PERIOD==0&&!this.isTreeGrowing&&!this.isGrowthFinished){if(this.isConverged())if(this.prunedNodesAll.length>0)this.isTreeGrowing=!0;else return!0;this.coolingCycle++,this.layoutQuality==0?this.coolingAdjuster=this.coolingCycle:this.layoutQuality==1&&(this.coolingAdjuster=this.coolingCycle/3),this.coolingFactor=Math.max(this.initialCoolingFactor-Math.pow(this.coolingCycle,Math.log(100*(this.initialCoolingFactor-this.finalTemperature))/Math.log(this.maxCoolingCycle))/100*this.coolingAdjuster,this.finalTemperature),this.animationPeriod=Math.ceil(this.initialAnimationPeriod*Math.sqrt(this.coolingFactor))}if(this.isTreeGrowing){if(this.growTreeIterations%10==0)if(this.prunedNodesAll.length>0){this.graphManager.updateBounds(),this.updateGrid(),this.growTree(this.prunedNodesAll),this.graphManager.resetAllNodesToApplyGravitation();var H=new Set(this.getAllNodes()),Y=this.nodesWithGravity.filter(function(Z){return H.has(Z)});this.graphManager.setAllNodesToApplyGravitation(Y),this.graphManager.updateBounds(),this.updateGrid(),this.coolingFactor=C.DEFAULT_COOLING_FACTOR_INCREMENTAL}else this.isTreeGrowing=!1,this.isGrowthFinished=!0;this.growTreeIterations++}if(this.isGrowthFinished){if(this.isConverged())return!0;this.afterGrowthIterations%10==0&&(this.graphManager.updateBounds(),this.updateGrid()),this.coolingFactor=C.DEFAULT_COOLING_FACTOR_INCREMENTAL*((100-this.afterGrowthIterations)/100),this.afterGrowthIterations++}var z=!this.isTreeGrowing&&!this.isGrowthFinished,W=this.growTreeIterations%10==1&&this.isTreeGrowing||this.afterGrowthIterations%10==1&&this.isGrowthFinished;return this.totalDisplacement=0,this.graphManager.updateBounds(),this.calcSpringForces(),this.calcRepulsionForces(z,W),this.calcGravitationalForces(),this.moveNodes(),this.animate(),!1},X.prototype.getPositionsData=function(){for(var H=this.graphManager.getAllNodes(),Y={},z=0;z<H.length;z++){var W=H[z].rect,Z=H[z].id;Y[Z]={id:Z,x:W.getCenterX(),y:W.getCenterY(),w:W.width,h:W.height}}return Y},X.prototype.runSpringEmbedder=function(){this.initialAnimationPeriod=25,this.animationPeriod=this.initialAnimationPeriod;var H=!1;if(C.ANIMATE==="during")this.emit("layoutstarted");else{for(;!H;)H=this.tick();this.graphManager.updateBounds()}},X.prototype.calculateNodesToApplyGravitationTo=function(){var H=[],Y,z=this.graphManager.getGraphs(),W=z.length,Z;for(Z=0;Z<W;Z++)Y=z[Z],Y.updateConnected(),Y.isConnected||(H=H.concat(Y.getNodes()));return H},X.prototype.createBendpoints=function(){var H=[];H=H.concat(this.graphManager.getAllEdges());var Y=new Set,z;for(z=0;z<H.length;z++){var W=H[z];if(!Y.has(W)){var Z=W.getSource(),G=W.getTarget();if(Z==G)W.getBendpoints().push(new O),W.getBendpoints().push(new O),this.createDummyNodesForBendpoints(W),Y.add(W);else{var ae=[];if(ae=ae.concat(Z.getEdgeListToNode(G)),ae=ae.concat(G.getEdgeListToNode(Z)),!Y.has(ae[0])){if(ae.length>1){var $;for($=0;$<ae.length;$++){var ge=ae[$];ge.getBendpoints().push(new O),this.createDummyNodesForBendpoints(ge)}}ae.forEach(function(ee){Y.add(ee)})}}}if(Y.size==H.length)break}},X.prototype.positionNodesRadially=function(H){for(var Y=new L(0,0),z=Math.ceil(Math.sqrt(H.length)),W=0,Z=0,G=0,ae=new O(0,0),$=0;$<H.length;$++){$%z==0&&(G=0,Z=W,$!=0&&(Z+=T.DEFAULT_COMPONENT_SEPERATION),W=0);var ge=H[$],ee=B.findCenterOfTree(ge);Y.x=G,Y.y=Z,ae=X.radialLayout(ge,ee,Y),ae.y>W&&(W=Math.floor(ae.y)),G=Math.floor(ae.x+T.DEFAULT_COMPONENT_SEPERATION)}this.transform(new O(S.WORLD_CENTER_X-ae.x/2,S.WORLD_CENTER_Y-ae.y/2))},X.radialLayout=function(H,Y,z){var W=Math.max(this.maxDiagonalInTree(H),T.DEFAULT_RADIAL_SEPARATION);X.branchRadialLayout(Y,null,0,359,0,W);var Z=R.calculateBounds(H),G=new q;G.setDeviceOrgX(Z.getMinX()),G.setDeviceOrgY(Z.getMinY()),G.setWorldOrgX(z.x),G.setWorldOrgY(z.y);for(var ae=0;ae<H.length;ae++){var $=H[ae];$.transform(G)}var ge=new O(Z.getMaxX(),Z.getMaxY());return G.inverseTransformPoint(ge)},X.branchRadialLayout=function(H,Y,z,W,Z,G){var ae=(W-z+1)/2;ae<0&&(ae+=180);var $=(ae+z)%360,ge=$*F.TWO_PI/360,ee=Z*Math.cos(ge),de=Z*Math.sin(ge);H.setCenter(ee,de);var re=[];re=re.concat(H.getEdges());var ke=re.length;Y!=null&&ke--;for(var Ce=0,_e=re.length,Te,Be=H.getEdgesBetween(Y);Be.length>1;){var Ge=Be[0];Be.splice(0,1);var Xe=re.indexOf(Ge);Xe>=0&&re.splice(Xe,1),_e--,ke--}Y!=null?Te=(re.indexOf(Be[0])+1)%_e:Te=0;for(var Ee=Math.abs(W-z)/ke,Ze=Te;Ce!=ke;Ze=++Ze%_e){var Ie=re[Ze].getOtherEnd(H);if(Ie!=Y){var Oe=(z+Ce*Ee)%360,Le=(Oe+Ee)%360;X.branchRadialLayout(Ie,H,Oe,Le,Z+G,G),Ce++}}},X.maxDiagonalInTree=function(H){for(var Y=N.MIN_VALUE,z=0;z<H.length;z++){var W=H[z],Z=W.getDiagonal();Z>Y&&(Y=Z)}return Y},X.prototype.calcRepulsionRange=function(){return 2*(this.level+1)*this.idealEdgeLength},X.prototype.groupZeroDegreeMembers=function(){var H=this,Y={};this.memberGroups={},this.idToDummyNode={};for(var z=[],W=this.graphManager.getAllNodes(),Z=0;Z<W.length;Z++){var G=W[Z],ae=G.getParent();this.getNodeDegreeWithChildren(G)===0&&(ae.id==null||!this.getToBeTiled(ae))&&z.push(G)}for(var Z=0;Z<z.length;Z++){var G=z[Z],$=G.getParent().id;typeof Y[$]>"u"&&(Y[$]=[]),Y[$]=Y[$].concat(G)}Object.keys(Y).forEach(function(ge){if(Y[ge].length>1){var ee="DummyCompound_"+ge;H.memberGroups[ee]=Y[ge];var de=Y[ge][0].getParent(),re=new k(H.graphManager);re.id=ee,re.paddingLeft=de.paddingLeft||0,re.paddingRight=de.paddingRight||0,re.paddingBottom=de.paddingBottom||0,re.paddingTop=de.paddingTop||0,H.idToDummyNode[ee]=re;var ke=H.getGraphManager().add(H.newGraph(),re),Ce=de.getChild();Ce.add(re);for(var _e=0;_e<Y[ge].length;_e++){var Te=Y[ge][_e];Ce.remove(Te),ke.add(Te)}}})},X.prototype.clearCompounds=function(){var H={},Y={};this.performDFSOnCompounds();for(var z=0;z<this.compoundOrder.length;z++)Y[this.compoundOrder[z].id]=this.compoundOrder[z],H[this.compoundOrder[z].id]=[].concat(this.compoundOrder[z].getChild().getNodes()),this.graphManager.remove(this.compoundOrder[z].getChild()),this.compoundOrder[z].child=null;this.graphManager.resetAllNodes(),this.tileCompoundMembers(H,Y)},X.prototype.clearZeroDegreeMembers=function(){var H=this,Y=this.tiledZeroDegreePack=[];Object.keys(this.memberGroups).forEach(function(z){var W=H.idToDummyNode[z];Y[z]=H.tileNodes(H.memberGroups[z],W.paddingLeft+W.paddingRight),W.rect.width=Y[z].width,W.rect.height=Y[z].height})},X.prototype.repopulateCompounds=function(){for(var H=this.compoundOrder.length-1;H>=0;H--){var Y=this.compoundOrder[H],z=Y.id,W=Y.paddingLeft,Z=Y.paddingTop;this.adjustLocations(this.tiledMemberPack[z],Y.rect.x,Y.rect.y,W,Z)}},X.prototype.repopulateZeroDegreeMembers=function(){var H=this,Y=this.tiledZeroDegreePack;Object.keys(Y).forEach(function(z){var W=H.idToDummyNode[z],Z=W.paddingLeft,G=W.paddingTop;H.adjustLocations(Y[z],W.rect.x,W.rect.y,Z,G)})},X.prototype.getToBeTiled=function(H){var Y=H.id;if(this.toBeTiled[Y]!=null)return this.toBeTiled[Y];var z=H.getChild();if(z==null)return this.toBeTiled[Y]=!1,!1;for(var W=z.getNodes(),Z=0;Z<W.length;Z++){var G=W[Z];if(this.getNodeDegree(G)>0)return this.toBeTiled[Y]=!1,!1;if(G.getChild()==null){this.toBeTiled[G.id]=!1;continue}if(!this.getToBeTiled(G))return this.toBeTiled[Y]=!1,!1}return this.toBeTiled[Y]=!0,!0},X.prototype.getNodeDegree=function(H){H.id;for(var Y=H.getEdges(),z=0,W=0;W<Y.length;W++){var Z=Y[W];Z.getSource().id!==Z.getTarget().id&&(z=z+1)}return z},X.prototype.getNodeDegreeWithChildren=function(H){var Y=this.getNodeDegree(H);if(H.getChild()==null)return Y;for(var z=H.getChild().getNodes(),W=0;W<z.length;W++){var Z=z[W];Y+=this.getNodeDegreeWithChildren(Z)}return Y},X.prototype.performDFSOnCompounds=function(){this.compoundOrder=[],this.fillCompexOrderByDFS(this.graphManager.getRoot().getNodes())},X.prototype.fillCompexOrderByDFS=function(H){for(var Y=0;Y<H.length;Y++){var z=H[Y];z.getChild()!=null&&this.fillCompexOrderByDFS(z.getChild().getNodes()),this.getToBeTiled(z)&&this.compoundOrder.push(z)}},X.prototype.adjustLocations=function(H,Y,z,W,Z){Y+=W,z+=Z;for(var G=Y,ae=0;ae<H.rows.length;ae++){var $=H.rows[ae];Y=G;for(var ge=0,ee=0;ee<$.length;ee++){var de=$[ee];de.rect.x=Y,de.rect.y=z,Y+=de.rect.width+H.horizontalPadding,de.rect.height>ge&&(ge=de.rect.height)}z+=ge+H.verticalPadding}},X.prototype.tileCompoundMembers=function(H,Y){var z=this;this.tiledMemberPack=[],Object.keys(H).forEach(function(W){var Z=Y[W];z.tiledMemberPack[W]=z.tileNodes(H[W],Z.paddingLeft+Z.paddingRight),Z.rect.width=z.tiledMemberPack[W].width,Z.rect.height=z.tiledMemberPack[W].height})},X.prototype.tileNodes=function(H,Y){var z=T.TILING_PADDING_VERTICAL,W=T.TILING_PADDING_HORIZONTAL,Z={rows:[],rowWidth:[],rowHeight:[],width:0,height:Y,verticalPadding:z,horizontalPadding:W};H.sort(function($,ge){return $.rect.width*$.rect.height>ge.rect.width*ge.rect.height?-1:$.rect.width*$.rect.height<ge.rect.width*ge.rect.height?1:0});for(var G=0;G<H.length;G++){var ae=H[G];Z.rows.length==0?this.insertNodeToRow(Z,ae,0,Y):this.canAddHorizontal(Z,ae.rect.width,ae.rect.height)?this.insertNodeToRow(Z,ae,this.getShortestRowIndex(Z),Y):this.insertNodeToRow(Z,ae,Z.rows.length,Y),this.shiftToLastRow(Z)}return Z},X.prototype.insertNodeToRow=function(H,Y,z,W){var Z=W;if(z==H.rows.length){var G=[];H.rows.push(G),H.rowWidth.push(Z),H.rowHeight.push(0)}var ae=H.rowWidth[z]+Y.rect.width;H.rows[z].length>0&&(ae+=H.horizontalPadding),H.rowWidth[z]=ae,H.width<ae&&(H.width=ae);var $=Y.rect.height;z>0&&($+=H.verticalPadding);var ge=0;$>H.rowHeight[z]&&(ge=H.rowHeight[z],H.rowHeight[z]=$,ge=H.rowHeight[z]-ge),H.height+=ge,H.rows[z].push(Y)},X.prototype.getShortestRowIndex=function(H){for(var Y=-1,z=Number.MAX_VALUE,W=0;W<H.rows.length;W++)H.rowWidth[W]<z&&(Y=W,z=H.rowWidth[W]);return Y},X.prototype.getLongestRowIndex=function(H){for(var Y=-1,z=Number.MIN_VALUE,W=0;W<H.rows.length;W++)H.rowWidth[W]>z&&(Y=W,z=H.rowWidth[W]);return Y},X.prototype.canAddHorizontal=function(H,Y,z){var W=this.getShortestRowIndex(H);if(W<0)return!0;var Z=H.rowWidth[W];if(Z+H.horizontalPadding+Y<=H.width)return!0;var G=0;H.rowHeight[W]<z&&W>0&&(G=z+H.verticalPadding-H.rowHeight[W]);var ae;H.width-Z>=Y+H.horizontalPadding?ae=(H.height+G)/(Z+Y+H.horizontalPadding):ae=(H.height+G)/H.width,G=z+H.verticalPadding;var $;return H.width<Y?$=(H.height+G)/Y:$=(H.height+G)/H.width,$<1&&($=1/$),ae<1&&(ae=1/ae),ae<$},X.prototype.shiftToLastRow=function(H){var Y=this.getLongestRowIndex(H),z=H.rowWidth.length-1,W=H.rows[Y],Z=W[W.length-1],G=Z.width+H.horizontalPadding;if(H.width-H.rowWidth[z]>G&&Y!=z){W.splice(-1,1),H.rows[z].push(Z),H.rowWidth[Y]=H.rowWidth[Y]-G,H.rowWidth[z]=H.rowWidth[z]+G,H.width=H.rowWidth[instance.getLongestRowIndex(H)];for(var ae=Number.MIN_VALUE,$=0;$<W.length;$++)W[$].height>ae&&(ae=W[$].height);Y>0&&(ae+=H.verticalPadding);var ge=H.rowHeight[Y]+H.rowHeight[z];H.rowHeight[Y]=ae,H.rowHeight[z]<Z.height+H.verticalPadding&&(H.rowHeight[z]=Z.height+H.verticalPadding);var ee=H.rowHeight[Y]+H.rowHeight[z];H.height+=ee-ge,this.shiftToLastRow(H)}},X.prototype.tilingPreLayout=function(){T.TILE&&(this.groupZeroDegreeMembers(),this.clearCompounds(),this.clearZeroDegreeMembers())},X.prototype.tilingPostLayout=function(){T.TILE&&(this.repopulateZeroDegreeMembers(),this.repopulateCompounds())},X.prototype.reduceTrees=function(){for(var H=[],Y=!0,z;Y;){var W=this.graphManager.getAllNodes(),Z=[];Y=!1;for(var G=0;G<W.length;G++)z=W[G],z.getEdges().length==1&&!z.getEdges()[0].isInterGraph&&z.getChild()==null&&(Z.push([z,z.getEdges()[0],z.getOwner()]),Y=!0);if(Y==!0){for(var ae=[],$=0;$<Z.length;$++)Z[$][0].getEdges().length==1&&(ae.push(Z[$]),Z[$][0].getOwner().remove(Z[$][0]));H.push(ae),this.graphManager.resetAllNodes(),this.graphManager.resetAllEdges()}}this.prunedNodesAll=H},X.prototype.growTree=function(H){for(var Y=H.length,z=H[Y-1],W,Z=0;Z<z.length;Z++)W=z[Z],this.findPlaceforPrunedNode(W),W[2].add(W[0]),W[2].add(W[1],W[1].source,W[1].target);H.splice(H.length-1,1),this.graphManager.resetAllNodes(),this.graphManager.resetAllEdges()},X.prototype.findPlaceforPrunedNode=function(H){var Y,z,W=H[0];W==H[1].source?z=H[1].target:z=H[1].source;var Z=z.startX,G=z.finishX,ae=z.startY,$=z.finishY,ge=0,ee=0,de=0,re=0,ke=[ge,de,ee,re];if(ae>0)for(var Ce=Z;Ce<=G;Ce++)ke[0]+=this.grid[Ce][ae-1].length+this.grid[Ce][ae].length-1;if(G<this.grid.length-1)for(var Ce=ae;Ce<=$;Ce++)ke[1]+=this.grid[G+1][Ce].length+this.grid[G][Ce].length-1;if($<this.grid[0].length-1)for(var Ce=Z;Ce<=G;Ce++)ke[2]+=this.grid[Ce][$+1].length+this.grid[Ce][$].length-1;if(Z>0)for(var Ce=ae;Ce<=$;Ce++)ke[3]+=this.grid[Z-1][Ce].length+this.grid[Z][Ce].length-1;for(var _e=N.MAX_VALUE,Te,Be,Ge=0;Ge<ke.length;Ge++)ke[Ge]<_e?(_e=ke[Ge],Te=1,Be=Ge):ke[Ge]==_e&&Te++;if(Te==3&&_e==0)ke[0]==0&&ke[1]==0&&ke[2]==0?Y=1:ke[0]==0&&ke[1]==0&&ke[3]==0?Y=0:ke[0]==0&&ke[2]==0&&ke[3]==0?Y=3:ke[1]==0&&ke[2]==0&&ke[3]==0&&(Y=2);else if(Te==2&&_e==0){var Xe=Math.floor(Math.random()*2);ke[0]==0&&ke[1]==0?Xe==0?Y=0:Y=1:ke[0]==0&&ke[2]==0?Xe==0?Y=0:Y=2:ke[0]==0&&ke[3]==0?Xe==0?Y=0:Y=3:ke[1]==0&&ke[2]==0?Xe==0?Y=1:Y=2:ke[1]==0&&ke[3]==0?Xe==0?Y=1:Y=3:Xe==0?Y=2:Y=3}else if(Te==4&&_e==0){var Xe=Math.floor(Math.random()*4);Y=Xe}else Y=Be;Y==0?W.setCenter(z.getCenterX(),z.getCenterY()-z.getHeight()/2-C.DEFAULT_EDGE_LENGTH-W.getHeight()/2):Y==1?W.setCenter(z.getCenterX()+z.getWidth()/2+C.DEFAULT_EDGE_LENGTH+W.getWidth()/2,z.getCenterY()):Y==2?W.setCenter(z.getCenterX(),z.getCenterY()+z.getHeight()/2+C.DEFAULT_EDGE_LENGTH+W.getHeight()/2):W.setCenter(z.getCenterX()-z.getWidth()/2-C.DEFAULT_EDGE_LENGTH-W.getWidth()/2,z.getCenterY())},l.exports=X},function(l,f,b){var d={};d.layoutBase=b(0),d.CoSEConstants=b(1),d.CoSEEdge=b(2),d.CoSEGraph=b(3),d.CoSEGraphManager=b(4),d.CoSELayout=b(6),d.CoSENode=b(5),l.exports=d}])})}(Rue)),Rue.exports}(function(r,i){(function(l,f){r.exports=f(khn())})(Mm,function(o){return function(l){var f={};function b(d){if(f[d])return f[d].exports;var w=f[d]={i:d,l:!1,exports:{}};return l[d].call(w.exports,w,w.exports,b),w.l=!0,w.exports}return b.m=l,b.c=f,b.i=function(d){return d},b.d=function(d,w,y){b.o(d,w)||Object.defineProperty(d,w,{configurable:!1,enumerable:!0,get:y})},b.n=function(d){var w=d&&d.__esModule?function(){return d.default}:function(){return d};return b.d(w,"a",w),w},b.o=function(d,w){return Object.prototype.hasOwnProperty.call(d,w)},b.p="",b(b.s=1)}([function(l,f){l.exports=o},function(l,f,b){var d=b(0).layoutBase.LayoutConstants,w=b(0).layoutBase.FDLayoutConstants,y=b(0).CoSEConstants,k=b(0).CoSELayout,E=b(0).CoSENode,T=b(0).layoutBase.PointD,C=b(0).layoutBase.DimensionD,S={ready:function(){},stop:function(){},quality:"default",nodeDimensionsIncludeLabels:!1,refresh:30,fit:!0,padding:10,randomize:!0,nodeRepulsion:4500,idealEdgeLength:50,edgeElasticity:.45,nestingFactor:.1,gravity:.25,numIter:2500,tile:!0,animate:"end",animationDuration:500,tilingPaddingVertical:10,tilingPaddingHorizontal:10,gravityRangeCompound:1.5,gravityCompound:1,gravityRange:3.8,initialEnergyOnIncremental:.5};function L(F,R){var q={};for(var X in F)q[X]=F[X];for(var X in R)q[X]=R[X];return q}function O(F){this.options=L(S,F),B(this.options)}var B=function(R){R.nodeRepulsion!=null&&(y.DEFAULT_REPULSION_STRENGTH=w.DEFAULT_REPULSION_STRENGTH=R.nodeRepulsion),R.idealEdgeLength!=null&&(y.DEFAULT_EDGE_LENGTH=w.DEFAULT_EDGE_LENGTH=R.idealEdgeLength),R.edgeElasticity!=null&&(y.DEFAULT_SPRING_STRENGTH=w.DEFAULT_SPRING_STRENGTH=R.edgeElasticity),R.nestingFactor!=null&&(y.PER_LEVEL_IDEAL_EDGE_LENGTH_FACTOR=w.PER_LEVEL_IDEAL_EDGE_LENGTH_FACTOR=R.nestingFactor),R.gravity!=null&&(y.DEFAULT_GRAVITY_STRENGTH=w.DEFAULT_GRAVITY_STRENGTH=R.gravity),R.numIter!=null&&(y.MAX_ITERATIONS=w.MAX_ITERATIONS=R.numIter),R.gravityRange!=null&&(y.DEFAULT_GRAVITY_RANGE_FACTOR=w.DEFAULT_GRAVITY_RANGE_FACTOR=R.gravityRange),R.gravityCompound!=null&&(y.DEFAULT_COMPOUND_GRAVITY_STRENGTH=w.DEFAULT_COMPOUND_GRAVITY_STRENGTH=R.gravityCompound),R.gravityRangeCompound!=null&&(y.DEFAULT_COMPOUND_GRAVITY_RANGE_FACTOR=w.DEFAULT_COMPOUND_GRAVITY_RANGE_FACTOR=R.gravityRangeCompound),R.initialEnergyOnIncremental!=null&&(y.DEFAULT_COOLING_FACTOR_INCREMENTAL=w.DEFAULT_COOLING_FACTOR_INCREMENTAL=R.initialEnergyOnIncremental),R.quality=="draft"?d.QUALITY=0:R.quality=="proof"?d.QUALITY=2:d.QUALITY=1,y.NODE_DIMENSIONS_INCLUDE_LABELS=w.NODE_DIMENSIONS_INCLUDE_LABELS=d.NODE_DIMENSIONS_INCLUDE_LABELS=R.nodeDimensionsIncludeLabels,y.DEFAULT_INCREMENTAL=w.DEFAULT_INCREMENTAL=d.DEFAULT_INCREMENTAL=!R.randomize,y.ANIMATE=w.ANIMATE=d.ANIMATE=R.animate,y.TILE=R.tile,y.TILING_PADDING_VERTICAL=typeof R.tilingPaddingVertical=="function"?R.tilingPaddingVertical.call():R.tilingPaddingVertical,y.TILING_PADDING_HORIZONTAL=typeof R.tilingPaddingHorizontal=="function"?R.tilingPaddingHorizontal.call():R.tilingPaddingHorizontal};O.prototype.run=function(){var F,R,q=this.options;this.idToLNode={};var X=this.layout=new k,te=this;te.stopped=!1,this.cy=this.options.cy,this.cy.trigger({type:"layoutstart",layout:this});var H=X.newGraphManager();this.gm=H;var Y=this.options.eles.nodes(),z=this.options.eles.edges();this.root=H.addRoot(),this.processChildrenList(this.root,this.getTopMostNodes(Y),X);for(var W=0;W<z.length;W++){var Z=z[W],G=this.idToLNode[Z.data("source")],ae=this.idToLNode[Z.data("target")];if(G!==ae&&G.getEdgesBetween(ae).length==0){var $=H.add(X.newEdge(),G,ae);$.id=Z.id()}}var ge=function(re,ke){typeof re=="number"&&(re=ke);var Ce=re.data("id"),_e=te.idToLNode[Ce];return{x:_e.getRect().getCenterX(),y:_e.getRect().getCenterY()}},ee=function de(){for(var re=function(){q.fit&&q.cy.fit(q.eles,q.padding),F||(F=!0,te.cy.one("layoutready",q.ready),te.cy.trigger({type:"layoutready",layout:te}))},ke=te.options.refresh,Ce,_e=0;_e<ke&&!Ce;_e++)Ce=te.stopped||te.layout.tick();if(Ce){X.checkLayoutSuccess()&&!X.isSubLayout&&X.doPostLayout(),X.tilingPostLayout&&X.tilingPostLayout(),X.isLayoutFinished=!0,te.options.eles.nodes().positions(ge),re(),te.cy.one("layoutstop",te.options.stop),te.cy.trigger({type:"layoutstop",layout:te}),R&&cancelAnimationFrame(R),F=!1;return}var Te=te.layout.getPositionsData();q.eles.nodes().positions(function(Be,Ge){if(typeof Be=="number"&&(Be=Ge),!Be.isParent()){for(var Xe=Be.id(),Ee=Te[Xe],Ze=Be;Ee==null&&(Ee=Te[Ze.data("parent")]||Te["DummyCompound_"+Ze.data("parent")],Te[Xe]=Ee,Ze=Ze.parent()[0],Ze!=null););return Ee!=null?{x:Ee.x,y:Ee.y}:{x:Be.position("x"),y:Be.position("y")}}}),re(),R=requestAnimationFrame(de)};return X.addListener("layoutstarted",function(){te.options.animate==="during"&&(R=requestAnimationFrame(ee))}),X.runLayout(),this.options.animate!=="during"&&(te.options.eles.nodes().not(":parent").layoutPositions(te,te.options,ge),F=!1),this},O.prototype.getTopMostNodes=function(F){for(var R={},q=0;q<F.length;q++)R[F[q].id()]=!0;var X=F.filter(function(te,H){typeof te=="number"&&(te=H);for(var Y=te.parent()[0];Y!=null;){if(R[Y.id()])return!1;Y=Y.parent()[0]}return!0});return X},O.prototype.processChildrenList=function(F,R,q){for(var X=R.length,te=0;te<X;te++){var H=R[te],Y=H.children(),z,W=H.layoutDimensions({nodeDimensionsIncludeLabels:this.options.nodeDimensionsIncludeLabels});if(H.outerWidth()!=null&&H.outerHeight()!=null?z=F.add(new E(q.graphManager,new T(H.position("x")-W.w/2,H.position("y")-W.h/2),new C(parseFloat(W.w),parseFloat(W.h)))):z=F.add(new E(this.graphManager)),z.id=H.data("id"),z.paddingLeft=parseInt(H.css("padding")),z.paddingTop=parseInt(H.css("padding")),z.paddingRight=parseInt(H.css("padding")),z.paddingBottom=parseInt(H.css("padding")),this.options.nodeDimensionsIncludeLabels&&H.isParent()){var Z=H.boundingBox({includeLabels:!0,includeNodes:!1}).w,G=H.boundingBox({includeLabels:!0,includeNodes:!1}).h,ae=H.css("text-halign");z.labelWidth=Z,z.labelHeight=G,z.labelPos=ae}if(this.idToLNode[H.data("id")]=z,isNaN(z.rect.x)&&(z.rect.x=0),isNaN(z.rect.y)&&(z.rect.y=0),Y!=null&&Y.length>0){var $;$=q.getGraphManager().add(q.newGraph(),z),this.processChildrenList($,Y,q)}}},O.prototype.stop=function(){return this.stopped=!0,this};var N=function(R){R("layout","cose-bilkent",O)};typeof cytoscape<"u"&&N(cytoscape),l.exports=N}])})})(mBe);const xhn=q9e(mBe.exports);wBe.use(xhn);function xBe(r,i,o,l){wOe.drawNode(r,i,o,l),i.children&&i.children.forEach((f,b)=>{xBe(r,f,o<0?b:o,l)})}function Ehn(r,i){i.edges().map((o,l)=>{const f=o.data();if(o[0]._private.bodyBounds){const b=o[0]._private.rscratch;Se.trace("Edge: ",l,f),r.insert("path").attr("d",`M ${b.startX},${b.startY} L ${b.midX},${b.midY} L${b.endX},${b.endY} `).attr("class","edge section-edge-"+f.section+" edge-depth-"+f.depth)}})}function EBe(r,i,o,l){i.add({group:"nodes",data:{id:r.id,labelText:r.descr,height:r.height,width:r.width,level:l,nodeId:r.id,padding:r.padding,type:r.type},position:{x:r.x,y:r.y}}),r.children&&r.children.forEach(f=>{EBe(f,i,o,l+1),i.add({group:"edges",data:{id:`${r.id}_${f.id}`,source:r.id,target:f.id,depth:l,section:f.section}})})}function _hn(r,i){return new Promise(o=>{const l=Cn("body").append("div").attr("id","cy").attr("style","display:none"),f=wBe({container:document.getElementById("cy"),style:[{selector:"edge",style:{"curve-style":"bezier"}}]});l.remove(),EBe(r,f,i,0),f.nodes().forEach(function(b){b.layoutDimensions=()=>{const d=b.data();return{w:d.width,h:d.height}}}),f.layout({name:"cose-bilkent",quality:"proof",styleEnabled:!1,animate:!1}).run(),f.ready(b=>{Se.info("Ready",b),o(f)})})}function Thn(r){r.nodes().map((i,o)=>{const l=i.data();l.x=i.position().x,l.y=i.position().y,wOe.positionNode(l);const f=zce(l.nodeId);Se.info("Id:",o,"Position: (",i.position().x,", ",i.position().y,")",l),f.attr("transform",`translate(${i.position().x-l.width/2}, ${i.position().y-l.height/2})`),f.attr("attr",`apa-${o})`)})}const Chn={draw:async(r,i,o,l)=>{const f=ht();l.db.clear(),l.parser.parse(r),Se.debug(`Renering info diagram
`+r);const b=ht().securityLevel;let d;b==="sandbox"&&(d=Cn("#i"+i));const y=Cn(b==="sandbox"?d.nodes()[0].contentDocument.body:"body").select("#"+i);y.append("g");const k=l.db.getMindmap(),E=y.append("g");E.attr("class","mindmap-edges");const T=y.append("g");T.attr("class","mindmap-nodes"),xBe(T,k,-1,f);const C=await _hn(k,f);Ehn(E,C),Thn(C),J8(void 0,y,f.mindmap.padding,f.mindmap.useMaxWidth)}},Shn=r=>{let i="";for(let o=0;o<r.THEME_COLOR_LIMIT;o++)r["lineColor"+o]=r["lineColor"+o]||r["cScaleInv"+o],fCe(r["lineColor"+o])?r["lineColor"+o]=is(r["lineColor"+o],20):r["lineColor"+o]=Ds(r["lineColor"+o],20);for(let o=0;o<r.THEME_COLOR_LIMIT;o++){const l=""+(17-3*o);i+=`
    .section-${o-1} rect, .section-${o-1} path, .section-${o-1} circle, .section-${o-1} polygon, .section-${o-1} path  {
      fill: ${r["cScale"+o]};
    }
    .section-${o-1} text {
     fill: ${r["cScaleLabel"+o]};
    }
    .node-icon-${o-1} {
      font-size: 40px;
      color: ${r["cScaleLabel"+o]};
    }
    .section-edge-${o-1}{
      stroke: ${r["cScale"+o]};
    }
    .edge-depth-${o-1}{
      stroke-width: ${l};
    }
    .section-${o-1} line {
      stroke: ${r["cScaleInv"+o]} ;
      stroke-width: 3;
    }

    .disabled, .disabled circle, .disabled text {
      fill: lightgray;
    }
    .disabled text {
      fill: #efefef;
    }
    `}return i},Ahn=Object.freeze(Object.defineProperty({__proto__:null,diagram:{db:Utn,renderer:Chn,parser:Ptn,styles:r=>`
  .edge {
    stroke-width: 3;
  }
  ${Shn(r)}
  .section-root rect, .section-root path, .section-root circle, .section-root polygon  {
    fill: ${r.git0};
  }
  .section-root text {
    fill: ${r.gitBranchLabel0};
  }
  .icon-container {
    height:100%;
    display: flex;
    justify-content: center;
    align-items: center;
  }
  .edge {
    fill: none;
  }
`}},Symbol.toStringTag,{value:"Module"}));return F2});
//# sourceMappingURL=mermaid.min.js.map